#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>

#include <ToolKit/ApplicationClass.h>

Boolean g_fRun = FALSE;
static Boolean g_fApplicationClassUsed = FALSE;

/*
 * Function: AmApplicationClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmApplicationClass::initializeValues()
{
    g_fApplicationClassUsed = TRUE;
    g_fRun = TRUE;

    _wTop  = NULL;
}


/*
 * Function: AmApplicationClass::createInterface()
 *
 * Purpose:
 *   Creates the application class and top level widget used
 *   in the AmApplicationClass.
 */

void
AmApplicationClass::createInterface( int  nArgs,
                                     char **pszArgs)
{

/* Create the application context. */

    _wTop = XtVaAppInitialize( &_ac,             /* Application context. */
                               _szName,          /* Application class. */
                               NULL, 0,          /* Command line options. */
                               &nArgs, pszArgs,  /* Command line arguments. */
                               NULL,             /* If missing app-defaults. */
                               NULL);            /* Terminate varargs list. */

/* Assign the top most widget to be the base widget. */

    _wBase = _wTop;

/* Hook in a destroy callback. */

    addDestroyCallback();
}


/*
 * Function: AmApplicationClass::AmApplicationClass()
 * 
 * Purpose:
 *   Default constructor for AmApplicationClass.
 *
 * Comments:
 *   Clears out member variables.
 */

AmApplicationClass::AmApplicationClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Function: AmApplicationClass::AmApplicationClass( szClass, nArgs, pszArgs)
 * 
 * Purpose:
 *   Constructor for AmApplicationClass that takes a class name and
 *   the command line parameters.
 *
 * Parameters:
 *   const char *szClass  : The appliction class name.
 *   int        nArgs     : Number of command line arguments.
 *   char       **pszArgs : Command line arguments.
 *
 * Comments:
 *   Creates an application context using the passed in name as 
 *   the application class and the passed in command line parameters. 
 *   Also calls the BaseWidgetclass constructor.
 */

AmApplicationClass::AmApplicationClass( const char *szClass,
                                        int        nArgs,
                                        char       **pszArgs)
    : AmBaseWidgetClass(( Widget) NULL, szClass)
{
    initializeValues();
    createInterface( nArgs, pszArgs);
}

/*
 * Function: AmApplicationClass::~AmApplicationClass()
 *
 * Purpose:
 *   Destructor.
 */

AmApplicationClass::~AmApplicationClass()
{
}

/*
 * Function: AmApplicationClass::Realize() 
 *
 * Purpose:
 *   Realize the application's widgets.
 */

void
AmApplicationClass::Realize()
{
    XtRealizeWidget( _wTop);
}


/*
 * Function: AmApplicationClass::Run()
 *
 * Purpose:
 *   Run the application's main loop.
 */

void
AmApplicationClass::Run()
{
    XEvent xEvent;

    g_fRun = TRUE;

    do {
        XtAppNextEvent( _ac, &xEvent);
        XtDispatchEvent( &xEvent);
    } while ( g_fRun == TRUE);
}


/*
 * Function: AmApplicationClass::Stop()
 *
 * Purpose:
 *   Stop the application's main loop.
 */

void
AmApplicationClass::Stop()
{
    AmQuit();
}


/*
 * Function: AmApplicationClass::GetAppContext()
 *
 * Purpose:
 *   Retrieve the application's context. 
 */


XtAppContext
AmApplicationClass::GetAppContext()
{
    return( _ac);
}



/*
 * Function: AmQuit()
 *
 * Purpose:
 *   Quits the application.
 *
 * Comments:
 *   If the AmApplicationClass is used this function causes the
 *   application to quit gracefully.  If it is not used, this
 *   function executes a exit( -1) call.
 */

void
AmQuit()
{
    if ( g_fApplicationClassUsed == TRUE) {
        g_fRun = FALSE;
    } else {
        exit( -1);
    } 
}
