#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/PushB.h>
#include <ToolKit/ButtonClass.h>

/*
 * Function: AmButtonClass::AmButtonClass()
 *
 * Purpose:
 *   Default constructor for the AmButtonClass.
 */

AmButtonClass::AmButtonClass()
    : AmBaseWidgetClass()
{
    _wButton = NULL;
}


/*
 * Functions: AmButtonClass::AmButtonClass( wParent, szName)
 *            AmButtonClass::AmButtonClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmButtonClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters: 
 *   Widget          wParent    :
 *   BaseWidgetclass *bwcParent :
 *   const char      *szName    :
 *
 * Comments:
 *   Creates a button widget.  The parent's constructor is called
 *   with the widget and name arguments so that everything gets
 *   initialized correctly.
 */

AmButtonClass::AmButtonClass( Widget         wParent,
                              const char     *szName,
                              XtCallbackProc xcp,
                              XtPointer      xp)
    : AmBaseWidgetClass( wParent, szName)
{
    _wButton = XtVaCreateManagedWidget( szName,
                                        xmPushButtonWidgetClass,
                                        wParent,
                                        NULL);

    _wBase = _wButton;
    addDestroyCallback();

    if ( xcp != NULL) {
        AddCallback( XmNactivateCallback, xcp, xp);
    }
}


/* See comment above. */

AmButtonClass::AmButtonClass( AmBaseWidgetClass *bwcParent,
                              const char        *szName,
                              XtCallbackProc    xcp,
                              XtPointer         xp)
    : AmBaseWidgetClass( bwcParent, szName)
{
    _wButton = XtVaCreateManagedWidget( szName,
                                        xmPushButtonWidgetClass,
                                        _wParent,
                                        NULL);

    _wBase = _wButton;
    addDestroyCallback();

    if ( xcp != NULL) {
        AddCallback( XmNactivateCallback, xcp, xp);
    }
}


/*
 * Function: AmButtonClass::~AmButtonClass()
 *
 * Purpose:
 *   Destructor for the AmButtonClass widget class.
 */

AmButtonClass::~AmButtonClass()
{
}
