#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/CascadeB.h>
#include <Xm/MenuShell.h>
#include <Xm/RowColumn.h>

#include <ToolKit/ComboBoxClass.h>

void
AmComboBoxClass::initializeValues()
{
    _lstButtons = NULL;
    _xCallback = NULL;
    _xpClientData = NULL;
}

void
AmComboBoxClass::createInterface()
{
    _wPulldownMenu = XmCreatePulldownMenu( _wParent, "OptionMenu", NULL, 0);

    _wRowColumn = XtVaCreateManagedWidget( "RowColumn", 
                                           xmRowColumnWidgetClass, _wParent,
                                           XmNrowColumnType, XmMENU_OPTION,
                                           XmNsubMenuId, _wPulldownMenu,
                                           XmNspacing, 0,
//                                           XmNmarginHeight, 0,
                                           XmNmarginWidth, 0,
                                           NULL);
    XtVaSetValues( XmOptionButtonGadget( _wRowColumn),
                   XmNalignment, XmALIGNMENT_BEGINNING,
                   XmNmarginLeft, 5,
                   XmNmarginRight, 5,
                   NULL);

    XtUnmanageChild( XmOptionLabelGadget( _wRowColumn));

    _wBase = _wRowColumn;

    addDestroyCallback();
}

/*
 * Function: AmComboBoxClass::AmComboBoxClass()
 *
 * Description:
 *   Default constructor. 
 */

AmComboBoxClass::AmComboBoxClass()
    : AmBaseWidgetClass()
{
    _wPulldownMenu = NULL;
    _wRowColumn = NULL;
}


/*
 * Function: AmComboBoxClass::AmComboBoxClass( wParent, szName);
 *
 * Parameters:
 *   Widget     wParent : Parent of the tool bar widget.
 *   const char *szName : Name of the tool bar widget.
 * 
 * Description:
 *    
 */

AmComboBoxClass::AmComboBoxClass( Widget     wParent,
                      const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}

AmComboBoxClass::AmComboBoxClass( AmBaseWidgetClass *bwcParent,
                      const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}

AmComboBoxClass::~AmComboBoxClass()
{
    if ( _lstButtons != NULL) {
        AmJustFreeLst( _lstButtons);
    }
}

Widget
AmComboBoxClass::GetRowColumn()
{
    return( _wRowColumn);
}


void
AmComboBoxClass::AddButton( const char     *szName,
                            XtCallbackProc xCallback,
                            XtPointer      xpClientData)
{
    Widget wButton;

    wButton = XtVaCreateManagedWidget( szName,
                                       xmPushButtonWidgetClass, _wPulldownMenu,
                                       NULL);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) activateCallback,
                   ( XtPointer) this);

    if ( xCallback != NULL) {
        XtAddCallback( wButton, XmNactivateCallback,
                       ( XtCallbackProc) xCallback,
                       ( XtPointer) xpClientData);
    }

    _lstButtons = AmInsertNodeAtEndLst( wButton, _lstButtons);
}


int
AmComboBoxClass::GetSelectedPosition()
{
    Widget  wSelected, wTemp;
    AmListPtr lstTemp;
    int     iPos;
 
    XtVaGetValues( _wRowColumn, XmNmenuHistory, &wSelected, NULL);

    for ( lstTemp = _lstButtons, iPos = 0;
          lstTemp != NULL;
          lstTemp = lstTemp->next, iPos++) {

        wTemp = ( Widget) lstTemp->data;
        if ( wSelected == wTemp) {
            return( iPos);
        }
    }
   
    return( -1);
}

void
AmComboBoxClass::SetSelectedPosition( int iPos)
{
    Widget  wTemp;
    AmListPtr lstTemp;
    int     ii;

    for ( lstTemp = _lstButtons, ii = 0;
          lstTemp != NULL;
          lstTemp = lstTemp->next, ii++) {

        if ( ii == iPos) {
            wTemp = ( Widget) lstTemp->data;
            XtVaSetValues( _wRowColumn, XmNmenuHistory, wTemp, NULL);
            break;
        }
    }
}


void
AmComboBoxClass::SetCallback( XtCallbackProc xCallback,
                              XtPointer      xpClientData)
{
    _xCallback = xCallback;
    _xpClientData = xpClientData;
}


void
AmComboBoxClass::activateCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData)
{
    AmComboBoxClass *pMenuClass = ( AmComboBoxClass *) xpClientData;
    pMenuClass->activate( w, xpCallData);
}


/* ARGSUSED */
void
AmComboBoxClass::activate( Widget    w,
                           XtPointer xpCallData)
{
    Widget  wTemp;
    AmListPtr lstTemp;
    int     iPos = 0, ii;

    for ( lstTemp = _lstButtons, ii = 0;
          lstTemp != NULL;
          lstTemp = lstTemp->next, ii++) {

        wTemp = ( Widget) lstTemp->data;
        if ( w == wTemp) {
            iPos = ii;
            break;
        }
    }

    if (( lstTemp != NULL) && ( _xCallback != NULL)) {
        _xCallback( w, _xpClientData, ( XtPointer) iPos);
    }
}
