#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Xm.h>
#include <X11/Shell.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <ToolKit/DialogClass.h>

/*
 * Function: AmDialogClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmDialogClass::initializeValues()
{
}


/*
 * Function: AmDialogClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets used in the AmDialogClass.
 *
 * Comments:
 *   The dialog class consists of a popup shell with a form inside
 *   of it and an action area at the bottom of the form.
 */

void
AmDialogClass::createInterface()
{
    char sz[ TEXTLEN];

/* Create the shell. */

    _wDialogForm = XmCreateFormDialog( _wParent, _szName, NULL, 0); 
    _wShell = XtParent( _wDialogForm);

/* Add an action area at the bottom of the form. */

    sprintf( sz, "%sActionArea", _szName);
    _pwcActionArea = new AmActionAreaClass( _wDialogForm, sz);

/* Add a form for other widgets to be put into. */

    sprintf( sz, "%sForm", _szName);
    _wForm = XtVaCreateManagedWidget( sz,
                                      xmFormWidgetClass, _wDialogForm,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNrightAttachment, XmATTACH_FORM,
                                      XmNbottomAttachment, XmATTACH_WIDGET,
                                      XmNbottomWidget,
                                          _pwcActionArea->GetWidget(),
                                      NULL);

/* Make sure the top level widget is treated as the base widget. */

    _wBase = _wShell;

/* Hook in a destroy callback. */
 
    addDestroyCallback();
}


/*
 * Function: AmDialogClass::AmDialogClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmDialogClass also
 *   calls the AmBaseWidgetClass default constructor.
 */

AmDialogClass::AmDialogClass()
    : AmBaseWidgetClass()
{
    _wShell = NULL;
    _wForm  = NULL;
}


/*
 * Functions: AmDialogClass::AmDialogClass( wParent, szName)
 *            AmDialogClass::AmDialogClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmDialogClass.  Both functions work the same
 *   way except that one takes a widget and the other takes a "wrapped"
 *   widget.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 * Comments:
 *   Creates a action area widget class which consists of two forms, a
 *   separator, and as many buttons as are added later.  The constructor
 *   uses the passed in parent and passed in name as the widget's parent
 *   and widget's name respectively.  The AmBaseWidgetClass constructor is
 *   called with the widget and name arguments so that everything gets
 *   initialized correctly.
 */

AmDialogClass::AmDialogClass( Widget     wParent,
                          const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


/* See above comment. */

AmDialogClass::AmDialogClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}


/*
 * Function: AmDialogClass::~AmDialogClass()
 *
 * Purpose:
 *   Destructor for the AmDialogClass.
 *
 * Comments:
 *   Frees up all the memory used by the widget class.
 *
 */

AmDialogClass::~AmDialogClass()
{
}


/* 
 * Function: AmDialogClass::AddButton( szName, callback, xpClientData)
 *
 * Purpose:
 *   A convenience function to add buttons to the dialog's action
 *   area.
 *
 * Parameters:
 *   const char     *szName      : Name of the button.
 *   XtCallbackProc callback     : Callback function to associated with
 *                                 the button.
 *   XtPointer      xpClientData : Client data to pass to the callback
 *                                 function.
 *
 * Comments:
 *   Adds a button the action area.  The button gets added to the right 
 *   of the existing ones.  If a callback function is given then that
 *   function is associated with the button's actiate callback.
 */

void
AmDialogClass::AddButton( char           *szName,
                        XtCallbackProc callback,
                        XtPointer      xpClientData)
{
    _pwcActionArea->AddButton( szName, callback, xpClientData);
}


/*
 * Function: AmDialogClass::GetForm()
 *
 * Purpose:
 *   Accessor function for the dialog's form widget.
 */

Widget
AmDialogClass::GetForm()
{
    return( _wForm);
}


/*
 * Function: AmDialogClass::Position( x, y)
 *
 * Purpose:
 *   Position the dialog's upper left corner at the specified coordinate.
 */

void
AmDialogClass::Position( int x,
                       int y)
{

/* Make sure a shell exists before attempting to change its position. */

    if ( _wShell != NULL) { 
        XtVaSetValues( _wShell, XmNx, x, XmNy, y, NULL);
    }
}

/*
 * Function: AmDialogClass::Show()
 *
 * Purpose:
 *   Show the dialog's widgets.
 */

void
AmDialogClass::Show()
{

/* Make sure a shell exists before attempting to pop it up. */

    if ( _wShell != NULL) {
        XtManageChild( _wDialogForm);
        XtMapWidget( _wShell);
        AmForceDisplayDialog( _wDialogForm);
    }
}


/*
 * Function: AmDialogClass::Hide()
 *
 * Purpose:
 *   Hide the dialog's widgets.
 */

void
AmDialogClass::Hide()
{

/* Make sure a shell exists before attempting to pop it up. */

    if ( _wShell != NULL) {
        XtUnmanageChild( _wDialogForm);
        XtUnmapWidget( _wShell);
        AmForceRemoveDialog( _wDialogForm);
    }
}


/*
 * Function: AmDialogClass::DestroyOnDelete()
 *
 * Purpose:
 *   Hide the dialog's widgets.
 */

void
AmDialogClass::DestroyOnDelete()
{
    if ( _wShell != NULL) {
        XtVaSetValues( _wShell, XmNdeleteResponse, XmDESTROY, NULL);
    }
}


/*
 * Function: AmDialogClass::HideOnDelete()
 *
 * Purpose:
 *   Hide the dialog's widgets.
 */

void
AmDialogClass::HideOnDelete()
{
    if ( _wShell != NULL) {
        XtVaSetValues( _wShell, XmNdeleteResponse, XmUNMAP, NULL);
    }
}
