#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Form.h>
#include <ToolKit/FormClass.h>

/*
 * Function: AmFormClass::AmFormClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmFormClass.
 */

AmFormClass::AmFormClass()
    : AmBaseWidgetClass()
{
    _wForm = NULL;
}


/*
 * Function: AmFormClass::AmFormClass( wParent, szName)
 *
 * Purpose:
 *   FromClass constructor which creates the form widget
 *   using the passed in parent widget and passed in name.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 * Comments:
 *   Creates a widget class using the passed in name as the widget's name.
 */

AmFormClass::AmFormClass( Widget     wParent,
                          const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    _wForm = XtVaCreateManagedWidget( _szName,
                                      xmFormWidgetClass,
                                      _wParent,
                                      NULL);
    _wBase = _wForm;
 
    addDestroyCallback();
}


/*
 * Function: AmFormClass::AmFormClass( bwcParent, szName)
 *
 * Purpose:
 *   FromClass constructor which creates the form widget
 *   using the passed in parent widget and passed in name.
 *
 * Parameters:
 *   AmBaseWidgetClass bwcParent : Parent widget of the new widget.
 *   const char      *szName   : Name of the widget.
 */

AmFormClass::AmFormClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    _wForm = XtVaCreateManagedWidget( _szName,
                                      xmFormWidgetClass,
                                      _wParent,
                                      NULL);
    _wBase = _wForm;

    addDestroyCallback();
}

/*
 * Function: AmFormClass::~AmFormClass()
 *
 * Purpose:
 *   Deconstructor.
 */

AmFormClass::~AmFormClass()
{
}


void
AmFormClass::SetFractionBase( int iBase)
{
    XtVaSetValues( _wForm, XmNfractionBase, iBase, NULL);
}
