#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Frame.h>
#include <Xm/LabelG.h>
#include <ToolKit/FrameClass.h>

/*
 * Function: AmFrameClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmFrameClass::initializeValues()
{
}


/*
 * Function: AmFrameClass::createInterface( fTitle)
 *
 * Purpose:
 *   Creates the widgets used in the AmFrameClass.
 *
 * Parameters:
 *   Boolean fTitle : TRUE if the frame is to have a title, FALSE
 *                    otherwise.
 *
 * Comments:
 *   The frame widget class concists of at most two widgets.  The 
 *   frame widget and the optional title widget.
 */

void
AmFrameClass::createInterface( Boolean fTitle)
{
    _wFrame = XtVaCreateManagedWidget( _szName,
                                       xmFrameWidgetClass, _wParent,
                                       NULL);

    if ( fTitle == TRUE) {
        _wLabel = XtVaCreateManagedWidget( "frameTitle",
                                           xmLabelGadgetClass, _wFrame,
                                           XmNchildType, XmFRAME_TITLE_CHILD,
                                           XmNchildVerticalAlignment,
                                           XmALIGNMENT_CENTER,
                                           NULL);
    }

    _wBase = _wFrame;
 
    addDestroyCallback();
}


/*
 * Function: AmFrameClass::AmFrameClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmFrameClass also
 *   calls the AmBaseWidgetClass default constructor.
 */

AmFrameClass::AmFrameClass()
    : AmBaseWidgetClass()
{
    _wFrame = NULL;
    _wLabel = NULL;
}


/*
 * Functions: AmFrameClass::AmFrameClass( wParent, szName, fTitle)
 *            AmFrameClass::AmFrameClass( bwcParent, szName, fTitle)
 *
 * Purpose:
 *   Constructor for the AmFrameClass.  Both functions work the same
 *   way except that one takes a widget and the other takes a "wrapped"
 *   widget.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget (also of the title).
 *   Boolean    fTitle  : Indicates whether the frame is to have a title
 *                        or not.
 *
 * Comments:
 *   Creates a frame widget class which consists of a frame widget and
 *   an optional title.  The AmBaseWidgetClass constructor is called with 
 *   the widget and the name arguments so that everything gets initialized
 *   correctly.
 */

AmFrameClass::AmFrameClass( Widget     wParent,
                        const char *szName,
                        Boolean    fTitle)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface( fTitle);
}


/* See above comment. */

AmFrameClass::AmFrameClass( AmBaseWidgetClass *bwcParent,
                        const char      *szName,
                        Boolean         fTitle)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface( fTitle);
}

/*
 * Function: AmFrameClass::~AmFrameClass()
 *
 * Purpose:
 *   Destructor for the AmFrameClass.
 *
 * Comments:
 *   Frees up all the memory used by the widget.
 *
 */

AmFrameClass::~AmFrameClass()
{
}

