#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <ToolKit/InformationBoxClass.h>


/*
 * Function: AmInformationBoxClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmInformationBoxClass::initializeValues()
{
}


/*
 * Function: AmInformationBoxClass::createInterface( szMessage)
 *
 * Purpose:
 *   Creates the widgets used in the AmInformationBoxClass.
 *
 * Comments:
 *   The message box class consists of a popup shell with a message 
 *   box inside.
 */

void
AmInformationBoxClass::createInterface( const char *szMessage)
{
    setIcon( XmDIALOG_INFORMATION);

    if ( szMessage != NULL) {
        setMessageString( szMessage);
    }

    hideCancelButton();
    hideHelpButton();

    Show();
}


/*
 * Function: AmInformationBoxClass::AmInformationBoxClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmInformationBoxClass also
 *   calls the AmBaseWidgetClass default constructor.
 */

AmInformationBoxClass::AmInformationBoxClass()
    : AmBoxClass()
{
    initializeValues();
}


/*
 * Functions: AmInformationBoxClass::AmInformationBoxClass( wParent, szName)
 *            AmInformationBoxClass::AmInformationBoxClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmInformationBoxClass.  Both functions work the same
 *   way except that one takes a widget and the other takes a "wrapped"
 *   widget.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 */

AmInformationBoxClass::AmInformationBoxClass( Widget     wParent,
                                              const char *szName,
                                              const char *szMessage)
    : AmBoxClass( wParent, szName) 
{
    initializeValues();
    createInterface( szMessage);
}


/* See above comment. */

AmInformationBoxClass::AmInformationBoxClass( AmBaseWidgetClass *bwcParent,
                                              const char        *szName,
                                              const char        *szMessage)
    : AmBoxClass( bwcParent, szName) 
{
    initializeValues();
    createInterface( szMessage);
}


/*
 * Function: AmInformationBoxClass::~AmInformationBoxClass()
 *
 * Purpose:
 *   Destructor for the AmInformationBoxClass.
 *
 * Comments:
 *   Frees up all the memory used by the widget class.
 *
 */

AmInformationBoxClass::~AmInformationBoxClass()
{
}
