#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <malloc.h>

#include <Xm/Xm.h>
#include <Xm/List.h>

#include <ToolKit/ListClass.h>

void
AmListClass::initializeValues()
{
}

void
AmListClass::createInterface()
{
    Arg vArg[ 20];
    int nArg;

/* Set the default arguments. */

    nArg = AmMakeArgList( vArg,
                        XmNselectionPolicy, XmSINGLE_SELECT,
                        NULL);
  
    _wList = XmCreateScrolledList( _wParent, _szName, vArg, nArg);
    XtManageChild( _wList);

    _wBase = XtParent( _wList);
    addDestroyCallback(); 
}

/*
 * Function: AmListClass::AmListClass()
 *
 * Description:
 *   Default constructor. 
 */

AmListClass::AmListClass()
    : AmBaseWidgetClass()
{
    _wList = NULL;
}


/*
 * Function: AmListClass::AmListClass( wParent, szName);
 *
 * Parameters:
 *   Widget     wParent : Parent of the row column widget.
 *   const char *szName : Name of the row column widget.
 * 
 * Description:
 *    
 */

AmListClass::AmListClass( Widget         wParent,
                      const char     *szName,
                      XtCallbackProc xcp,
                      XtPointer      xp)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();

    if ( xcp != NULL) { 
        XtAddCallback( _wList, XmNdefaultActionCallback, xcp, xp);
    }
}

AmListClass::AmListClass( AmBaseWidgetClass *bwcParent,
                      const char      *szName,
                      XtCallbackProc  xcp,
                      XtPointer       xp)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();

    if ( xcp != NULL) { 
        XtAddCallback( _wList, XmNdefaultActionCallback, xcp, xp);
    }
}

AmListClass::~AmListClass()
{
}

/* Functions to choose what type of selections are to be
   allowed. */

void
AmListClass::SetSingleSelect()
{
    XtVaSetValues( _wList,
                   XmNselectionPolicy, XmSINGLE_SELECT,
                   NULL);
}

void
AmListClass::SetBrowseSelect()
{
    XtVaSetValues( _wList,
                   XmNselectionPolicy, XmBROWSE_SELECT,
                   NULL);
}

void
AmListClass::SetMultipleSelect()
{
    XtVaSetValues( _wList,
                   XmNselectionPolicy, XmMULTIPLE_SELECT,
                   NULL);
}

void
AmListClass::SetExtendedSelect()
{
    XtVaSetValues( _wList,
                   XmNselectionPolicy, XmEXTENDED_SELECT,
                   NULL);
}

void
AmListClass::SetNumVisibleItems( int nItems)
{
    if ( nItems > 0) {
        XtVaSetValues( _wList,
                       XmNvisibleItemCount, nItems,
                       NULL);
    }
}


void
AmListClass::AddItem( char    *szName,
                    int     iPos,
                    Boolean fShowSelected)
{
    XmString xsz;

    xsz = XmStringCreateLocalized( szName);

    if ( TRUE == fShowSelected) {
        XmListAddItem( _wList, xsz, iPos);
    } else {
        XmListAddItemUnselected( _wList, xsz, iPos);
    }

    XmStringFree( xsz);
}


void
AmListClass::AddItems( char    **vszName,
                     int     nName,
                     int     iPos,
                     Boolean fShowSelected)
{
    XmString *vxsz;
    int      ii;

    vxsz = ( XmString *) malloc( sizeof( XmString) * nName);
    if ( vxsz == NULL) {
        return;
    }

    for ( ii = 0; ii < nName; ii++) {
        vxsz[ ii] = XmStringCreateLocalized( vszName[ ii]);
    }

    if ( TRUE == fShowSelected) {
        XmListAddItems( _wList, vxsz, nName, iPos);
    } else {
        XmListAddItemsUnselected( _wList, vxsz, nName, iPos);
    }

    for ( ii = 0; ii < nName; ii++) {
        XmStringFree( vxsz[ ii]);
    }

    free( vxsz);
}


void
AmListClass::DeleteItem( char *szName)
{
    XmString xsz;

    xsz = XmStringCreateLocalized( szName);
    XmListDeleteItem( _wList, xsz);
    XmStringFree( xsz);
}


void
AmListClass::DeleteItems( char **vszName, 
                        int  nName)
{
    XmString *vxsz;
    int      ii;

    vxsz = ( XmString *) malloc( sizeof( XmString) * nName);
    if ( vxsz == NULL) {
        return;
    }

    for ( ii = 0; ii < nName; ii++) {
        vxsz[ ii] = XmStringCreateLocalized( vszName[ ii]);
    }

    XmListDeleteItems( _wList, vxsz, nName);

    for ( ii = 0; ii < nName; ii++) {
        XmStringFree( vxsz[ ii]);
    }

    free( vxsz);
}


void
AmListClass::DeleteAllItems()
{
    int nItems;

    XtVaGetValues( _wList, XmNitemCount, &nItems, NULL);
    if ( nItems > 0) {
        XmListDeleteItemsPos( _wList, nItems, 1);
    }
}


void
AmListClass::GetSelectedPositions( int **vPos,
                                 int *nPos)
{
    XmListGetSelectedPos( _wList, vPos, nPos);
}


void
AmListClass::AddActionCallback( XtCallbackProc xcp,
                              XtPointer      xp)
{
    if ( xcp != NULL) {
        XtAddCallback( _wList, XmNdefaultActionCallback, xcp, xp);
    }
}


void
AmListClass::RemoveActionCallback( XtCallbackProc xcp,
                                 XtPointer      xp)
{
    if ( xcp != NULL) {
        XtRemoveCallback( _wList, XmNdefaultActionCallback, xcp, xp);
    }
}


void
AmListClass::AddSingleCallback( XtCallbackProc xcp,
                              XtPointer      xp)
{
    if ( xcp != NULL) {
        XtAddCallback( _wList, XmNsingleSelectionCallback, xcp, xp);
    }
}


void
AmListClass::RemoveSingleCallback( XtCallbackProc xcp,
                                 XtPointer      xp)
{
    if ( xcp != NULL) {
        XtRemoveCallback( _wList, XmNsingleSelectionCallback, xcp, xp);
    }
}


void
AmListClass::AddBrowseCallback( XtCallbackProc xcp,
                              XtPointer      xp)
{
    if ( xcp != NULL) {
        XtAddCallback( _wList, XmNbrowseSelectionCallback, xcp, xp);
    }
}


void
AmListClass::RemoveBrowseCallback( XtCallbackProc xcp,
                                 XtPointer      xp)
{
    if ( xcp != NULL) {
        XtRemoveCallback( _wList, XmNbrowseSelectionCallback, xcp, xp);
    }
}


void
AmListClass::AddMultipleCallback( XtCallbackProc xcp,
                                XtPointer      xp)
{
    if ( xcp != NULL) {
        XtAddCallback( _wList, XmNmultipleSelectionCallback, xcp, xp);
    }
}


void
AmListClass::RemoveMultipleCallback( XtCallbackProc xcp,
                                   XtPointer      xp)
{
    if ( xcp != NULL) {
        XtRemoveCallback( _wList, XmNmultipleSelectionCallback, xcp, xp);
    }
}


void
AmListClass::AddExtendedCallback( XtCallbackProc xcp,
                                XtPointer      xp)
{
    if ( xcp != NULL) {
        XtAddCallback( _wList, XmNextendedSelectionCallback, xcp, xp);
    }
}


void
AmListClass::RemoveExtendedCallback( XtCallbackProc xcp,
                                   XtPointer      xp)
{
    if ( xcp != NULL) {
        XtRemoveCallback( _wList, XmNextendedSelectionCallback, xcp, xp);
    }
}
