#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>

#include <ToolKit/MenuClass.h>

/*
 * Function: AmMenuClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmMenuClass::initializeValues()
{
    _wMenu = NULL;
}


/*
 * Function: AmMenuClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets used for a AmMenuClass.
 *
 */

void
AmMenuClass::createInterface()
{
    _wMenu = XmCreateMenuBar( _wParent, _szName, NULL, 0);
    XtManageChild( _wMenu);

    _wBase = _wMenu;
    addDestroyCallback(); 
}

/*
 * Function: AmMenuClass::AmMenuClass()
 *
 * Purpose:
 *   Default constructor. 
 */

AmMenuClass::AmMenuClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Functions: AmMenuClass::AmMenuClass( wParent, szName);
 *            AmMenuClass::AmMenuClass( bwcParent, szName); 
 *
 * Purpose:
 *   Constructor for the AmMenuClass widget class.  Both
 *   constructors work the same way except that one uses
 *   a widget explicitly while the other uses a "wrapped"
 *   widget.
 *    
 * Parameters:
 *   Widget          wParent    : Parent of the tool bar widget.
 *   AmBaseWidgetClass *bwcParent : Parent of the tool bar widget.
 *   const char      *szName    : Name of the tool bar widget.
 *
 */

AmMenuClass::AmMenuClass( Widget     wParent,
                          const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


/* See comment above. */

AmMenuClass::AmMenuClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}


/*
 * Function: AmMenuClass::~AmMenuClass()
 *
 * Purpose:
 *   Desctructor. 
 */

AmMenuClass::~AmMenuClass()
{
}
