#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>

#include <ToolKit/MenuPaneClass.h>

/*
 * Function: AmMenuPaneClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmMenuPaneClass::initializeValues()
{
    _wPane      = NULL;
    _wCascade   = NULL;
    _lstButtons = NULL;
}


/*
 * Function: AmMenuPaneClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets used for a AmMenuPaneClass.
 *
 * Description:
 *   The menu pane consists of two widgets, a pulldown menu which
 *   becomes the pane, and a cascade button which pops up the
 *   pulldown menu.  The cascade button receives the name given
 *   to the constructor.  The pane uses the name with the string
 *   "Pane" appended to it.
 */

void
AmMenuPaneClass::createInterface()
{
    char sz[ TEXTLEN];

    sprintf( sz, "%sPane", _szName);

    _wPane = XmCreatePulldownMenu( _wParent, sz, NULL, 0);
    _wCascade = XtVaCreateManagedWidget( _szName, 
                                         xmCascadeButtonWidgetClass,
                                         _wParent,
                                         XmNsubMenuId, _wPane,
                                         NULL);

    _wBase = _wPane;
    addDestroyCallback(); 
}

/*
 * Function: AmMenuPaneClass::AmMenuPaneClass()
 *
 * Description:
 *   Default constructor. 
 */

AmMenuPaneClass::AmMenuPaneClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Functions: AmMenuPaneClass::AmMenuPaneClass( wParent, szName);
 *            AmMenuPaneClass::AmMenuPaneClass( pwcParent, szName); 
 * 
 * Purpose:
 *   Constructor for the AmMenuPaneClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters:
 *   Widget          wParent    : Parent of the tool bar widget.
 *   AmBaseWidgetClass *pwcParent : Pointer to "wrapped" parent widget.
 *   const char      *szName    : Name of the tool bar widget.
 * 
 */

AmMenuPaneClass::AmMenuPaneClass( Widget     wParent,
                              const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}

AmMenuPaneClass::AmMenuPaneClass( AmBaseWidgetClass *bwcParent,
                              const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}

AmMenuPaneClass::~AmMenuPaneClass()
{
    if ( _lstButtons != NULL) {
        AmJustFreeLst( _lstButtons);
        _lstButtons = NULL;
    }
}

void
AmMenuPaneClass::SetAsHelpPane()
{
    XtVaSetValues( _wParent, XmNmenuHelpWidget, _wCascade, NULL);
}


AmButtonClass *
AmMenuPaneClass::AddButton( const char     *szName,
                          XtCallbackProc xcp,
                          XtPointer      xp)
{
    AmButtonClass *pwcButton;

    if ( _wPane != NULL) {  

        pwcButton = new AmButtonClass( _wPane, szName, xcp, xp);
        _lstButtons = AmInsertNodeLst( pwcButton, _lstButtons);

        return( pwcButton);
    }

    return( NULL);
}


AmToggleButtonClass *
AmMenuPaneClass::AddToggleButton( const char     *szName,
                                XtCallbackProc xcp,
                                XtPointer      xp)
{
    AmToggleButtonClass *pwcToggleButton;

    if ( _wPane != NULL) {  

        pwcToggleButton = new AmToggleButtonClass( _wPane, szName, xcp, xp);
        _lstButtons = AmInsertNodeLst( pwcToggleButton, _lstButtons);

        return( pwcToggleButton);
    }

    return( NULL);
}


AmSeparatorClass *
AmMenuPaneClass::AddSeparator()
{
    AmSeparatorClass *pwcSeparator;
    char           sz[ TEXTLEN];

    if ( _wPane != NULL) {  

        sprintf( sz, "%sSeparator", _szName);
        pwcSeparator = new AmSeparatorClass( _wPane, sz);

        return( pwcSeparator);
    }

    return( NULL);
}


AmBaseWidgetClass *
AmMenuPaneClass::GetButton( const char *szName)
{
    AmListPtr         lstTemp;
    AmBaseWidgetClass *pwc;

    if ( szName == NULL) {
        return( NULL);
    }

    for ( lstTemp = _lstButtons; lstTemp != NULL; lstTemp = lstTemp->next) {

        pwc = ( AmBaseWidgetClass *) lstTemp->data;
        if ( strcmp( szName, pwc->GetName()) == 0) {
            return( pwc);
        }
    }

    return( NULL);
}
