#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/RowColumn.h>
#include <ToolKit/PaneClass.h>

/*
 * Function: AmPaneClass::AmPaneClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmPaneClass.
 */

AmPaneClass::AmPaneClass()
    : AmBaseWidgetClass()
{
    _wRowCol = NULL;
}


/*
 * Function: AmPaneClass::AmPaneClass( wParent, szName)
 *
 * Purpose:
 *   AmPaneClass constructor which creates the row column widget
 *   using the passed in parent widget and passed in name.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 * Comments:
 *   Creates a widget class using the passed in name as the widget's name.
 */

AmPaneClass::AmPaneClass( Widget     wParent,
                          const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    _wRowCol = XtVaCreateManagedWidget( _szName,
                                        xmRowColumnWidgetClass, _wParent,
                                        XmNrowColumnType, XmWORK_AREA,
                                        NULL);
    _wBase = _wRowCol;
 
    addDestroyCallback();
}


/*
 * Function: AmPaneClass::AmPaneClass( bwcParent, szName)
 *
 * Purpose:
 *   AmPaneClass constructor which creates the row column
 *   widget using the passed in parent widget and passed in name.
 *
 * Parameters:
 *   AmBaseWidgetClass bwcParent : Parent widget of the new widget.
 *   const char      *szName   : Name of the widget.
 */

AmPaneClass::AmPaneClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    _wRowCol = XtVaCreateManagedWidget( _szName,
                                        xmRowColumnWidgetClass, _wParent,
                                        XmNrowColumnType, XmWORK_AREA,
                                        NULL);
    _wBase = _wRowCol;
 
    addDestroyCallback();
}

/*
 * Function: AmPaneClass::~AmPaneClass()
 *
 * Purpose:
 *   Deconstructor.
 */

AmPaneClass::~AmPaneClass()
{
}
