#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>

#include <ToolKit/PopupMenuClass.h>

/*
 * Function: AmPopupMenuClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmPopupMenuClass::initializeValues()
{
    _lstButtons = NULL;
    _fAttached  = FALSE;
    _wPane      = NULL;
}


/*
 * Function: AmPopupMenuClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets used for a AmPopupMenuClass.
 *
 * Description:
 *   The popup menu consists of one widgets, a popup menu.
 */

void
AmPopupMenuClass::createInterface()
{
    _wPane = XmCreatePopupMenu( _wParent, _szName, NULL, 0);
    _wBase = _wPane;
 
    addDestroyCallback(); 
}

/*
 * Function: AmPopupMenuClass::AmPopupMenuClass()
 *
 * Description:
 *   Default constructor. 
 */

AmPopupMenuClass::AmPopupMenuClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Functions: AmPopupMenuClass::AmPopupMenuClass( wParent, szName);
 *            AmPopupMenuClass::AmPopupMenuClass( pwcParent, szName); 
 * 
 * Purpose:
 *   Constructor for the AmPopupMenuClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters:
 *   Widget          wParent    : Parent of the tool bar widget.
 *   AmBaseWidgetClass *pwcParent : Pointer to "wrapped" parent widget.
 *   const char      *szName    : Name of the tool bar widget.
 * 
 */

AmPopupMenuClass::AmPopupMenuClass( Widget     wParent,
                                const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}

AmPopupMenuClass::AmPopupMenuClass( AmBaseWidgetClass *bwcParent,
                                const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}

AmPopupMenuClass::~AmPopupMenuClass()
{
    if ( _lstButtons != NULL) {
        AmJustFreeLst( _lstButtons);
        _lstButtons = NULL;
    }

    if ( _fAttached == FALSE) {
        XtRemoveEventHandler( _wParent, ButtonPressMask, FALSE,
                              ( XtEventHandler) AmPopupEvent,
                              ( XtPointer) this);
        _fAttached = FALSE;
    }
}


AmButtonClass *
AmPopupMenuClass::AddButton( const char     *szName,
                           XtCallbackProc xcp,
                           XtPointer      xp)
{
    AmButtonClass *pwcButton;

    if ( _wPane != NULL) {  

        pwcButton = new AmButtonClass( _wPane, szName, xcp, xp);
        _lstButtons = AmInsertNodeLst( pwcButton, _lstButtons);

        return( pwcButton);
    }

    return( NULL);
}


AmToggleButtonClass *
AmPopupMenuClass::AddToggleButton( const char     *szName,
                                 XtCallbackProc xcp,
                                 XtPointer      xp)
{
    AmToggleButtonClass *pwcToggleButton;

    if ( _wPane != NULL) {  

        pwcToggleButton = new AmToggleButtonClass( _wPane, szName, xcp, xp);
        _lstButtons = AmInsertNodeLst( pwcToggleButton, _lstButtons);

        return( pwcToggleButton);
    }

    return( NULL);
}


AmSeparatorClass *
AmPopupMenuClass::AddSeparator()
{
    AmSeparatorClass *pwcSeparator;
    char           sz[ TEXTLEN];

    if ( _wPane != NULL) {  

        sprintf( sz, "%sSeparator", _szName);
        pwcSeparator = new AmSeparatorClass( _wPane, sz);

        return( pwcSeparator);
    }

    return( NULL);
}


AmBaseWidgetClass *
AmPopupMenuClass::GetButton( const char *szName)
{
    AmListPtr         lstTemp;
    AmBaseWidgetClass *pwc;

    if ( szName == NULL) {
        return( NULL);
    }

    for ( lstTemp = _lstButtons; lstTemp != NULL; lstTemp = lstTemp->next) {

        pwc = ( AmBaseWidgetClass *) lstTemp->data;
        if ( strcmp( szName, pwc->GetName()) == 0) {
            return( pwc);
        }
    }

    return( NULL);
}


void
AmPopupMenuClass::Attach()
{
    if ( _fAttached == FALSE) {
        XtAddEventHandler( _wParent, ButtonPressMask, FALSE,
                           ( XtEventHandler) AmPopupEvent,
                           ( XtPointer) this);
        _fAttached = TRUE;
    }
}

/* ARGSUSED */
void
AmPopupMenuClass::AmPopupEvent( Widget    w,
                            XtPointer xpClientData,
                            XEvent    *pxEvent)
{
    AmPopupMenuClass *pwcPopupMenu = ( AmPopupMenuClass *) xpClientData;
    pwcPopupMenu->popup( w, pxEvent);
}

/* ARGSUSED */
void
AmPopupMenuClass::popup( Widget w,
                       XEvent *pxEvent)
{
    XButtonPressedEvent *pxButtonEvent = ( XButtonPressedEvent *) pxEvent;

    XmMenuPosition( _wPane, pxButtonEvent);
    Show();
}
