#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>

#include <ToolKit/ProgressBarClass.h>

XtResource AmProgressBarClass::_xResources[] = {
    { "font",
      XmCFont,
      XmRFont,
      sizeof( Font),
      XtOffset( AmProgressBarClass *, _xFont),
      XmRString,
      (XtPointer) XtDefaultFont },
    { "barWidth",
      XmCWidth,
      XmRInt,
      sizeof( int),
      XtOffset( AmProgressBarClass *, _cxBar),
      XtRImmediate,
      ( XtPointer) 10 },
    { "labelString",
      XmCLabelString,
      XmRString,
      sizeof( char *),
      XtOffset( AmProgressBarClass *, _szLabel),
      XtRImmediate,
      ( XtPointer) 0 },
};

/*
 * Function: AmProgressBarClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmProgressBarClass::initializeValues()
{
    _pwcFrame        = NULL;
    _pwcArea         = NULL;

    _pxDisplay       = NULL;
    _xWindow         = (Window)NULL;
    _xGC             = NULL;

    _iMinValue       = 0;
    _iMaxValue       = 100;
    _iCurrValue      = 100;

    _fShowPercentage = TRUE;
}


/*
 * Function: AmProgressBarClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets used for a AmProgressBarClass.
 *
 * Description:
 *   The menu pane consists of two widgets, a pulldown menu which
 *   becomes the pane, and a cascade button which pops up the
 *   pulldown menu.  The cascade button receives the name given
 *   to the constructor.  The pane uses the name with the string
 *   "Pane" appended to it.
 */

void
AmProgressBarClass::createInterface()
{
    char      sz[ TEXTLEN];

    _pwcFrame = new AmFrameClass( _wParent, _szName, FALSE);

    _wBase = *_pwcFrame;
    addDestroyCallback(); 

    sprintf( sz, "%sDA", _szName);
    _pwcArea  = new AmDrawingAreaClass( _pwcFrame, sz);

    getResources( *_pwcFrame, _xResources, XtNumber( _xResources));

    XtAddCallback( *_pwcArea, XmNexposeCallback,
                   ( XtCallbackProc) AmExposeCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcArea, XmNresizeCallback,
                   ( XtCallbackProc) AmExposeCallback,
                   ( XtPointer) this);
}

void
AmProgressBarClass::createGC()
{
    XGCValues gcv;

    _pxDisplay = XtDisplay( *_pwcArea);
    _xWindow = XtWindow( *_pwcArea);

    XtVaGetValues( *_pwcArea,
                   XmNforeground, &_lFgPixel,
                   XmNbackground, &_lBgPixel,
                   NULL);

    gcv.foreground = _lFgPixel;
    gcv.background = _lBgPixel;
    gcv.font       = _xFont;

    _xGC = XCreateGC( _pxDisplay, _xWindow,
                      GCForeground | GCBackground | GCFont,
                      &gcv);
}

/*
 * Function: AmProgressBarClass::AmProgressBarClass()
 *
 * Description:
 *   Default constructor. 
 */

AmProgressBarClass::AmProgressBarClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Functions: AmProgressBarClass::AmProgressBarClass( wParent, szName);
 *            AmProgressBarClass::AmProgressBarClass( pwcParent, szName); 
 * 
 * Purpose:
 *   Constructor for the AmProgressBarClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters:
 *   Widget          wParent    : Parent of the tool bar widget.
 *   AmBaseWidgetClass *pwcParent : Pointer to "wrapped" parent widget.
 *   const char      *szName    : Name of the tool bar widget.
 * 
 */

AmProgressBarClass::AmProgressBarClass( Widget     wParent,
                              const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}

AmProgressBarClass::AmProgressBarClass( AmBaseWidgetClass *bwcParent,
                              const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}

AmProgressBarClass::~AmProgressBarClass()
{
}


void
AmProgressBarClass::SetMinValue( int iMinValue)
{
    _iMinValue = iMinValue;
}

void
AmProgressBarClass::SetMaxValue( int iMaxValue)
{
    _iMaxValue = iMaxValue;
}

void
AmProgressBarClass::SetCurrValue( int iCurrValue)
{
    _iCurrValue = iCurrValue;

    if ( XtIsRealized( *_pwcArea)) {

        if ( _xGC == NULL) {
            createGC();
        }

        AmSendFakeExpose( _pxDisplay, _xWindow); 
    }
}

void
AmProgressBarClass::AmExposeCallback( Widget    w,
                                  XtPointer xpClientData,
                                  XtPointer xpCallData)
{
    AmProgressBarClass *pwcProgressBar = ( AmProgressBarClass *) xpClientData;
    pwcProgressBar->expose( w, xpCallData);
}

/* ARGSUSED */
void
AmProgressBarClass::expose( Widget    w,
                          XtPointer callData)
{
    XRectangle  xClipRectangle;
    XCharStruct xcs;
    XGCValues   gcv;
    Dimension   dimWidth, dimHeight;
    int         cx, iAscent, iDescent, iDir;
    char        sz[ TEXTLEN];

    if ( XtIsRealized( *_pwcArea)) {

        if ( _xGC == NULL) {
            createGC();
        }
 
        XClearWindow( _pxDisplay, _xWindow);

        XtVaGetValues( *_pwcArea,
                       XmNwidth, &dimWidth,
                       XmNheight, &dimHeight,
                       NULL);

        if ( _fShowPercentage == TRUE) {

            cx = ( _iCurrValue - _iMinValue) * dimWidth /
                 ( _iMaxValue - _iMinValue);

            XFillRectangle( _pxDisplay, _xWindow, _xGC,
                            0, 0, cx, dimHeight); 

            gcv.foreground = _lBgPixel;

            XChangeGC( _pxDisplay, _xGC, GCForeground , &gcv);

	    sprintf( sz, "%d%%", _iCurrValue);
	    XQueryTextExtents( _pxDisplay, XGContextFromGC( _xGC),
			       sz, strlen( sz),
			       &iDir, &iAscent, &iDescent, &xcs);

            xClipRectangle.x      = 0;
            xClipRectangle.y      = 0;
            xClipRectangle.width  = cx;
            xClipRectangle.height = dimHeight;

            XSetClipRectangles( _pxDisplay, _xGC,
                                0, 0, &xClipRectangle, 1, Unsorted);

            XDrawString( _pxDisplay, _xWindow, _xGC,
                         dimWidth / 2 - xcs.width / 2, 
                         ( iAscent + dimHeight - iDescent) / 2,
                         sz, strlen( sz));

            xClipRectangle.x      = cx;
            xClipRectangle.y      = 0;
            xClipRectangle.width  = dimWidth;
            xClipRectangle.height = dimHeight;
 
            XSetClipRectangles( _pxDisplay, _xGC,
                                0, 0, &xClipRectangle, 1, Unsorted);

            gcv.foreground = _lFgPixel;

            XChangeGC( _pxDisplay, _xGC, GCForeground, &gcv);

            XDrawString( _pxDisplay, _xWindow, _xGC,
                         dimWidth / 2 - xcs.width / 2, 
                         ( iAscent + dimHeight - iDescent) / 2,
                         sz, strlen( sz));

            xClipRectangle.x      = 0; 
            xClipRectangle.y      = 0;
            xClipRectangle.width  = dimWidth;
            xClipRectangle.height = dimHeight;
 
            XSetClipRectangles( _pxDisplay, _xGC,
                                0, 0, &xClipRectangle, 1, Unsorted);
        } else {

            cx = _cxBar + ( _iCurrValue - _iMinValue) * 
                 ( dimWidth - _cxBar) / ( _iMaxValue - _iMinValue);

            XFillRectangle( _pxDisplay, _xWindow, _xGC,
                            cx - _cxBar, 0, _cxBar, dimHeight); 

            if ( _szLabel != NULL) {

                gcv.foreground = _lBgPixel;

                XChangeGC( _pxDisplay, _xGC, GCForeground , &gcv);


	        XQueryTextExtents( _pxDisplay, XGContextFromGC( _xGC),
			           _szLabel, strlen( _szLabel),
			           &iDir, &iAscent, &iDescent, &xcs);

                xClipRectangle.x      = cx - _cxBar;
                xClipRectangle.y      = 0;
                xClipRectangle.width  = _cxBar;
                xClipRectangle.height = dimHeight;

                XSetClipRectangles( _pxDisplay, _xGC,
                                    0, 0, &xClipRectangle, 1, Unsorted);

                XDrawString( _pxDisplay, _xWindow, _xGC,
                             dimWidth / 2 - xcs.width / 2, 
                             ( iAscent + dimHeight - iDescent) / 2,
                             _szLabel, strlen( _szLabel));

                xClipRectangle.x      = 0;
                xClipRectangle.y      = 0; 
                xClipRectangle.width  = cx - _cxBar;
                xClipRectangle.height = dimHeight;
 
                XSetClipRectangles( _pxDisplay, _xGC,
                                    0, 0, &xClipRectangle, 1, Unsorted);

                gcv.foreground = _lFgPixel;

                XChangeGC( _pxDisplay, _xGC, GCForeground, &gcv);

                XDrawString( _pxDisplay, _xWindow, _xGC,
                             dimWidth / 2 - xcs.width / 2, 
                             ( iAscent + dimHeight - iDescent) / 2,
                             _szLabel, strlen( _szLabel));

                xClipRectangle.x      = cx;
                xClipRectangle.y      = 0;
                xClipRectangle.width  = dimWidth - cx;
                xClipRectangle.height = dimHeight;
 
                XSetClipRectangles( _pxDisplay, _xGC,
                                    0, 0, &xClipRectangle, 1, Unsorted);

                gcv.foreground = _lFgPixel;

                XChangeGC( _pxDisplay, _xGC, GCForeground, &gcv);

                XDrawString( _pxDisplay, _xWindow, _xGC,
                             dimWidth / 2 - xcs.width / 2, 
                             ( iAscent + dimHeight - iDescent) / 2,
                             _szLabel, strlen( _szLabel));

                xClipRectangle.x      = 0; 
                xClipRectangle.y      = 0;
                xClipRectangle.width  = dimWidth;
                xClipRectangle.height = dimHeight;
 
                XSetClipRectangles( _pxDisplay, _xGC,
                                    0, 0, &xClipRectangle, 1, Unsorted);
            }
        }

        XFlush( _pxDisplay);
        XmUpdateDisplay( *_pwcArea); 
    }
}

void
AmProgressBarClass::ShowPercentage( AnvilBoolean fShowPercentage)
{
    _fShowPercentage = fShowPercentage;

    if ( XtIsRealized( *_pwcArea)) {

        if ( _xGC == NULL) {
            createGC();
        }

        AmSendFakeExpose( _pxDisplay, _xWindow); 
    }
}


void
AmProgressBarClass::ShowProgress( AnvilBoolean fShowProgress)
{
    _fShowPercentage = !fShowProgress;

    if ( XtIsRealized( *_pwcArea)) {

        if ( _xGC == NULL) {
            createGC();
        }

        AmSendFakeExpose( _pxDisplay, _xWindow); 
    }
}
