#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <ToolKit/QuestionBoxClass.h>

XtResource AmQuestionBoxClass::_xResources[] = {
    {
        "cancel",
        XmCBoolean,
        XmRBoolean,
        sizeof( Boolean),
        XtOffset( AmQuestionBoxClass *, _fShowCancel),
        XmRImmediate,
        ( XtPointer) FALSE, 
    }
};


/*
 * Function: AmQuestionBoxClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmQuestionBoxClass::initializeValues()
{
    _eAnswer = AmANSWER_NONE;
}


/*
 * Function: AmQuestionBoxClass::createInterface( szMessage)
 *
 * Purpose:
 *   Creates the widgets used in the AmQuestionBoxClass.
 *
 * Comments:
 *   The message box class consists of a popup shell with a message 
 *   box inside.
 */

void
AmQuestionBoxClass::createInterface()
{
    setIcon( XmDIALOG_QUESTION);

    setOkCallback(( XtCallbackProc) activateCallback,
                  ( XtPointer) this);
    setCancelCallback(( XtCallbackProc) activateCallback,
                      ( XtPointer) this);

    getResources( _wBox, _xResources, XtNumber( _xResources)); 

    if ( _fShowCancel) {
        setHelpCallback(( XtCallbackProc) activateCallback,
                        ( XtPointer) this);
    } else {
        hideHelpButton();
    }
}


/*
 * Function: AmQuestionBoxClass::AmQuestionBoxClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmQuestionBoxClass also
 *   calls the AmBaseWidgetClass default constructor.
 */

AmQuestionBoxClass::AmQuestionBoxClass()
    : AmBoxClass()
{
    initializeValues();
}


/*
 * Functions: AmQuestionBoxClass::AmQuestionBoxClass( wParent, szName)
 *            AmQuestionBoxClass::AmQuestionBoxClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmQuestionBoxClass.  Both functions work the same
 *   way except that one takes a widget and the other takes a "wrapped"
 *   widget.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 */

AmQuestionBoxClass::AmQuestionBoxClass( Widget     wParent,
                                        const char *szName)
    : AmBoxClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


/* See above comment. */

AmQuestionBoxClass::AmQuestionBoxClass( AmBaseWidgetClass *bwcParent,
                                        const char        *szName)
    : AmBoxClass( bwcParent, szName) 
{
    initializeValues();
    createInterface();
}


/*
 * Function: AmQuestionBoxClass::~AmQuestionBoxClass()
 *
 * Purpose:
 *   Destructor for the AmQuestionBoxClass.
 *
 * Comments:
 *   Frees up all the memory used by the widget class.
 *
 */

AmQuestionBoxClass::~AmQuestionBoxClass()
{
}




/* Response callback - gets called when the user makes some kind of
                       choice, AmBetween "OK" and "CANCEL". */

void
AmQuestionBoxClass::activateCallback( Widget    wButton,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{
    AmQuestionBoxClass *pQuestionBox = ( AmQuestionBoxClass *) xpClientData;
    pQuestionBox->activate( wButton, xpCallData);
}

/* ARGSUSED */
void
AmQuestionBoxClass::activate( Widget    wButton,
                              XtPointer xpCallData)
{
    XmAnyCallbackStruct *cbs = ( XmAnyCallbackStruct *) xpCallData;

    switch( cbs->reason) {

        case XmCR_OK:
            _eAnswer = AmANSWER_YES;
            break;

        case XmCR_CANCEL:
            _eAnswer = AmANSWER_NO;
            break;

        case XmCR_HELP:
            _eAnswer = AmANSWER_CANCEL;
            break;

        default:
            break;

    }
}


AmAnswerEnum
AmQuestionBoxClass::AskQuestion( const char *szMessage)
{
    XtAppContext ac = XtWidgetToApplicationContext( _wBox);

/* Reset the answer. */

    _eAnswer = AmANSWER_NONE;
    if ( szMessage != NULL) {
        setMessageString( szMessage);
    }

/* "Really" pop up the dialog. */

    Show();

    while ( _eAnswer == AmANSWER_NONE) {
        XtAppProcessEvent( ac, XtIMAll);
    }

/* "Really" pop down the dialog. */

    Hide();

    return( _eAnswer);
}
