#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Frame.h>

#include <ToolKit/RulerClass.h>

static int RULER_TEXT_OFF = 2;

XtResource AmRulerClass::_xResources[] = {
    {
        "rulerColor",
        XmCForeground,
        XmRPixel,
        sizeof( long),
        XtOffset( AmRulerClass *, _lRulerPixel ),
        XmRImmediate,
        ( XtPointer) 0
    }
};
    

void
AmRulerClass::initializeValues()
{
    _wArea       = NULL;
    _xPixmap    = 0;
    _xGC        = NULL;

    _iWidth      = 0;
    _iHeight     = 0;
    _iPos        = 0;
    _iDPI        = 72;

    _lFgPixel    = 0;
    _lBgPixel    = 0;
    _lRulerPixel = 0;

    _fVertical   = FALSE;
    _fExposed    = FALSE;
}

void
AmRulerClass::createInterface()
{
    char   sz[ TEXTLEN];

    sprintf( sz, "%sFrame", _szName);
    _wFrame = XtVaCreateManagedWidget( sz,                 /* Widget name. */
				       xmFrameWidgetClass, /* Widget class. */
				       _wParent,           /* Widget parent. */
				       NULL);

    _wBase = _wFrame;
    addDestroyCallback();

    _wRuler = XtVaCreateManagedWidget( _szName,
				       xmDrawingAreaWidgetClass,
				       _wFrame,
				       NULL);

    XtVaGetValues( _wRuler,
		   XmNdepth, &_iDepth,
		   XmNforeground, &_lFgPixel,
		   XmNbackground, &_lBgPixel,
		   NULL);

    _xResources[ 0].default_addr = ( XtPointer) _lBgPixel;

    getResources( _wRuler, _xResources, XtNumber( _xResources)); 

    XtAddCallback( _wRuler, XmNexposeCallback,
		   ( XtCallbackProc) AmExposeRulerCallback,
		   ( XtPointer) this);
    XtAddCallback( _wArea, XmNexposeCallback,
		   ( XtCallbackProc) AmExposeAreaCallback,
		   ( XtPointer) this);
    XtAddCallback( _wArea, XmNresizeCallback,
		   ( XtCallbackProc) AmResizeAreaCallback,
		   ( XtPointer) this);

}

/*
 * Function: AmRulerClass::AmRulerClass()
 *
 * Description:
 *   Default constructor. 
 */

AmRulerClass::AmRulerClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Function: AmRulerClass::AmRulerClass( wParent, szName, wArea, fVertical)
 *
 * Parameters:
 *   Widget     wParent   : Parent widget for the ruler widget.
 *   const char *szName   : Name of the ruler widget.
 *   Widget     wArea     : Drawing area to associate ruler with.
 *   AnvilBoolean    fVertical : Whether it is a vertical or a horizontal ruler.
 * 
 * Description:
 *    
 */

AmRulerClass::AmRulerClass( Widget     wParent,
			const char *szName,
			Widget     wArea,
			AnvilBoolean    fVertical)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();

    _wArea = wArea;
    _fVertical = fVertical;

    createInterface();
}

AmRulerClass::AmRulerClass( AmBaseWidgetClass *pwcParent,
			const char      *szName,
			Widget          wArea,
			AnvilBoolean         fVertical)
    : AmBaseWidgetClass( pwcParent, szName)
{
    initializeValues();

    _wArea = wArea;
    _fVertical = fVertical;

    createInterface();
}

AmRulerClass::~AmRulerClass()
{
    Display   *pDisplay = XtDisplay( _wRuler);
    //    Window    window    = XtWindow( _wRuler);

    if ( _xPixmap != 0) {
        XFreePixmap( pDisplay, _xPixmap);
    }

    if ( _xGC != NULL) {
        XFreeGC( XtDisplay( _wRuler), _xGC);
    }
}

/*
 * Fucntion: AmRulerClass::CreateGC()
 *
 * Description:
 *
 */

void
AmRulerClass::CreateGC()
{
    Display    *pDisplay;
    Window     window;
    const char *szFont;
    int        iFontSize;
    XGCValues  gcv;
    Font       font;
    
    pDisplay = XtDisplay( _wRuler);
    window = XtWindow( _wRuler);

    szFont = AmGetDefault( "ruler.fontName", DEFAULT_FONT);
    iFontSize = AmGetDefault( "ruler.fontSize", DEFAULT_SIZE);

    font = AmLoadFont( pDisplay, szFont, iFontSize);
    gcv.font = font;

    gcv.background = _lBgPixel;
    gcv.foreground = _lFgPixel;
    
    _xGC = XCreateGC( pDisplay, window,
                      GCFont | GCForeground | GCBackground,
                      &gcv);
}

void
AmRulerClass::CreatePixmap( int iWidth,
			  int iHeight)
{
    Display   *pDisplay = XtDisplay( _wRuler);
    Window    window    = XtWindow( _wRuler);
    XGCValues gcv;

    if (( _iWidth != iWidth + 2) || ( _iHeight != iHeight + 2)) {
    
	if ( _xPixmap != 0) {
	    XFreePixmap( pDisplay, _xPixmap);
	}
    
	_iWidth = iWidth + 2;
	_iHeight = iHeight + 2;
    
	_xPixmap = XCreatePixmap( pDisplay, window, _iWidth, _iHeight, _iDepth);
    }
    
    gcv.foreground = _lRulerPixel;
    XChangeGC( pDisplay, _xGC, GCForeground, &gcv);
	
    XFillRectangle( pDisplay, _xPixmap, _xGC,
		    0, 0, _iWidth, _iHeight);
	
    gcv.foreground = _lFgPixel; 
    XChangeGC( pDisplay, _xGC, GCForeground, &gcv);
}

void
AmRulerClass::DrawPixmapVertical()
{
    Display     *pDisplay = XtDisplay( _wRuler);
    XCharStruct xcs;
    int         iTextSpace, iDir, iAscent, iDescent;
    int         ii, y, yLittle, iInch, iTickLen, iMaxTexts;
    char        sz[ TEXTLEN];

    sprintf( sz, "%d", _iDPI);
    XQueryTextExtents( pDisplay, XGContextFromGC( _xGC), sz, strlen( sz),
		       &iDir, &iAscent, &iDescent, &xcs);

    /* Figure out how many dots there have to be inbetween each labe.
       We start with an inch worth of dots. */
      
    iTextSpace = _iDPI;

    /* Figure out how many labels we can fit at the most. */
   
    iMaxTexts = ( _iHeight - 1) / ( iAscent + iDescent);

    /* Keep increasing (doubling) the seperation until all the labels fit. */
    
    while ((( _iHeight - 1) / iTextSpace) > iMaxTexts) {
	iTextSpace *= 2;
    }

     /* Draw the inches, half inches, quarter inches, and eight inches. */

    for ( y = 0, iInch = 0;
	  y <= _iHeight;
	  y += _iDPI, iInch++) {

	/* Draw the label every iTextSpace pixels (dots). */
	
	if (( y % iTextSpace == 0) && ( y > 0)) {

	    sprintf( sz, "%d", iInch);
	    XQueryTextExtents( pDisplay, XGContextFromGC( _xGC),
			       sz, strlen( sz),
			       &iDir, &iAscent, &iDescent, &xcs);
            XDrawString( pDisplay, _xPixmap, _xGC,
                         3 * _iWidth / 8,
                         y - iDescent - RULER_TEXT_OFF, sz, strlen( sz));
	}

	/* Draw the mark for the inch. */
	
	XDrawLine( pDisplay, _xPixmap, _xGC,
		   _iWidth, y, 3 * _iWidth / 8, y);

	/* Draw the marks for the eight, quarter, and half inches. */
	
	for ( ii = 1; ii < 8; ii++) {
	    
	    yLittle = y + ii * _iDPI / 8;
	    if ( ii % 4 == 0) {
		iTickLen = _iWidth / 4;
	    } else if ( ii % 2 == 0) {
		iTickLen = 3 * _iWidth / 16;
	    } else {
		iTickLen = _iWidth / 8;
	    }

	    XDrawLine( pDisplay, _xPixmap, _xGC,
		       _iWidth, yLittle, _iWidth - iTickLen, yLittle);
	}
    }

    XDrawRectangle( pDisplay, _xPixmap, _xGC,
		    0, 0, _iWidth - 1, _iHeight - 1);
}

void
AmRulerClass::DrawPixmapHorizontal()
{
    Display     *pDisplay = XtDisplay( _wRuler);
    XCharStruct xcs;
    int         iTextSpace, iDir, iAscent, iDescent;
    int         ii, x, xLittle, iInch, iTickLen, iMaxTexts;
    char        sz[ TEXTLEN];

    sprintf( sz, "%d", _iDPI);
    XQueryTextExtents( pDisplay, XGContextFromGC( _xGC), sz, strlen( sz),
		       &iDir, &iAscent, &iDescent, &xcs);

    /* Figure out how many dots there have to be inbetween each labe.
       We start with an inch worth of dots. */
      
    iTextSpace = _iDPI;

    /* Figure out how many labels we can fit at the most. */
   
    iMaxTexts = _iWidth / xcs.width;

    /* Keep increasing (doubling) the seperation until all the labels fit. */
    
    while (( _iWidth / iTextSpace) > iMaxTexts) {
	iTextSpace *= 2;
    }

    /* Draw the inches, half inches, quarter inches, and eight inches. */

    for ( x = 0, iInch = 0;
	  x <= _iWidth;
	  x += _iDPI, iInch++) {

	/* Draw the label every iTextSpace pixels (dots). */
	
	if (( x % iTextSpace == 0) && ( x > 0)) {

	    sprintf( sz, "%d", iInch);
	    XQueryTextExtents( pDisplay, XGContextFromGC( _xGC),
			       sz, strlen( sz),
			       &iDir, &iAscent, &iDescent, &xcs);
	    XDrawString( pDisplay, _xPixmap, _xGC,
			 x - xcs.width - RULER_TEXT_OFF,
			 3 * _iHeight / 8 + iAscent,
			 sz, strlen( sz));
	}

	/* Draw the mark for the inch. */
	
	XDrawLine( pDisplay, _xPixmap, _xGC,
		   x, _iHeight, x, 3 * _iHeight / 8);


	/* Draw the marks for the eight, quarter, and half inches. */
	
	for ( ii = 1; ii < 8; ii++) {
	    
	    xLittle = x + ii * _iDPI / 8;
	    if ( ii % 4 == 0) {
		iTickLen = _iHeight / 4;
	    } else if ( ii % 2 == 0) {
		iTickLen = 3 * _iHeight / 16;
	    } else {
		iTickLen = _iHeight / 8;
	    }

	    XDrawLine( pDisplay, _xPixmap, _xGC,
		       xLittle, _iHeight, xLittle, _iHeight - iTickLen);
	}
    }

    XDrawRectangle( pDisplay, _xPixmap, _xGC,
		    0, 0, _iWidth - 1, _iHeight - 1);
}

/*
 * Function: void DrawRulerCallback( wRuler, xpClientData, xpCallData)
 *
 * Description:
 *     Static member function that is repsonsible to draw or redraw the
 *     ruler.
 */

/* ARGSUSED */
void
AmRulerClass::AmExposeRulerCallback( Widget    wRuler,
				 XtPointer xpClientData,
				 XtPointer xpCallData)
{
    AmRulerClass *rc = ( AmRulerClass *) xpClientData;
    Display    *pDisplay = XtDisplay( wRuler);
    Window     window = XtWindow( wRuler);
    Dimension  dimWidth, dimHeight;

    if ( XtIsRealized( wRuler)) {

        rc->_fExposed = TRUE;
 
        if ( rc->_xGC == NULL) {
	    rc->CreateGC();
        }

        if ( rc->_xPixmap == 0) {
    
	    if ( rc->_fVertical == TRUE) {
	        XtVaGetValues( rc->_wArea,
			       XmNheight, &dimHeight,
			       NULL);
	        XtVaGetValues( rc->_wRuler,
			       XmNwidth, &dimWidth,
			       NULL);
	    } else {
	        XtVaGetValues( rc->_wArea,
			       XmNwidth, &dimWidth,
			       NULL);
	        XtVaGetValues( rc->_wRuler,
			       XmNheight, &dimHeight,
			       NULL);
	    }
	
	    /* Create a new pixmap and clear it with the background color.  If
	       a pixmap of the correct size already exists, only clear it with
	       the background color. */
	
	    rc->CreatePixmap( dimWidth, dimHeight);
	
	    /* Draw the ruler into the pixmap. */

            if ( rc->_fVertical == TRUE) {	
	        rc->DrawPixmapVertical();
            } else {
	        rc->DrawPixmapHorizontal();
            }
        }

        /* Copy the pixmap buffer onto the ruler. */
    
        XtVaGetValues( rc->_wRuler,
		       XmNwidth, &dimWidth,
		       XmNheight, &dimHeight,
		       NULL);
  
        if ( rc->_fVertical == TRUE) {
	    XCopyArea( pDisplay, rc->_xPixmap, window, rc->_xGC,
		       0, rc->_iPos, dimWidth, dimHeight, 0, 0);
        } else {
	    XCopyArea( pDisplay, rc->_xPixmap, window, rc->_xGC,
		       rc->_iPos, 0, dimWidth, dimHeight, 0, 0);
        }
    }
}


/* ARGSUSED */
void
AmRulerClass::AmResizeAreaCallback( Widget    wArea,
				XtPointer xpClientData,
				XtPointer xpCallData)
{
    AmRulerClass *rc = ( AmRulerClass *) xpClientData;
    Position   pos;
    Dimension  dimWidth, dimHeight;

    if ( rc->_fVertical == TRUE) {
	XtVaGetValues( wArea,
		       XmNy, &pos,
		       XmNheight, &dimHeight,
		       NULL);
	XtVaGetValues( rc->_wRuler,
		       XmNwidth, &dimWidth,
		       NULL);
    } else {
	XtVaGetValues( wArea,
		       XmNx, &pos,
		       XmNwidth, &dimWidth,
		       NULL);
	XtVaGetValues( rc->_wRuler,
		       XmNheight, &dimHeight,
		       NULL);
    }

    if ( rc->_fExposed == TRUE) {

        rc->CreatePixmap( dimWidth, dimHeight);
    
        /* Draw the ruler into the pixmap. */

        rc->_iPos = - pos;
        if ( rc->_fVertical == TRUE) {	
	    rc->DrawPixmapVertical();
        } else {
	    rc->DrawPixmapHorizontal();
        }
    
        AmSendFakeExpose( XtDisplay( rc->_wRuler), XtWindow( rc->_wRuler));
    }
}

/* ARGSUSED */
void
AmRulerClass::AmExposeAreaCallback( Widget    wArea,
				XtPointer xpClientData,
				XtPointer xpCallData)
{
    AmRulerClass *rc = ( AmRulerClass *) xpClientData;
    Position   pos;
    Dimension  dimWidth, dimHeight;

    if ( rc->_fVertical == TRUE) {
	XtVaGetValues( wArea,
		       XmNy, &pos,
		       XmNheight, &dimHeight,
		       NULL);
	XtVaGetValues( rc->_wRuler,
		       XmNwidth, &dimWidth,
		       NULL);
    } else {
	XtVaGetValues( wArea,
		       XmNx, &pos,
		       XmNwidth, &dimWidth,
		       NULL);
	XtVaGetValues( rc->_wRuler,
		       XmNheight, &dimHeight,
		       NULL);
    }

    if (( rc->_iPos != pos) ||
	( rc->_iWidth != dimWidth) ||
	( rc->_iHeight != dimHeight)) {

	rc->_iPos = - pos;
	AmSendFakeExpose( XtDisplay( rc->_wRuler), XtWindow( rc->_wRuler));
    }
}

void
AmRulerClass::SetDPI( int     iDPI,
                    AnvilBoolean fUpdate)
{
    _iDPI = iDPI;

    if (( fUpdate == TRUE) && ( XtIsRealized( _wRuler))) { 
        AmSendFakeExpose( XtDisplay( _wRuler), XtWindow( _wRuler));
    }
}

Widget
AmRulerClass::GetFrameWidget()
{
    return( _wFrame);
}


Widget
AmRulerClass::GetRulerWidget()
{
    return( _wRuler);
}
