#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>

#include <ToolKit/ScrolledTextClass.h>

void
AmScrolledTextClass::initializeValues()
{
}

void
AmScrolledTextClass::createInterface()
{
    Arg vArg[ 20];
    int nArg;

/* Set the default arguments. */

    nArg = AmMakeArgList( vArg,
                        XmNeditMode, XmMULTI_LINE_EDIT,
                        XmNscrollingPolicy, XmAUTOMATIC,
                        XmNvisualPolicy,
                            ( _fVariable ) ? XmVARIABLE : XmCONSTANT, 
                        NULL);

    _wText = XmCreateScrolledText( _wParent, _szName, vArg, nArg);
    _wWindow = XtParent( _wText);

    XtManageChild( _wText);
 
    _wBase = _wWindow;

    addDestroyCallback();
}

/*
 * Function: AmScrolledTextClass::AmScrolledTextClass()
 *
 * Description:
 *   Default constructor. 
 */

AmScrolledTextClass::AmScrolledTextClass()
    : AmBaseWidgetClass()
{
    _wText = NULL;
}


/*
 * Function: AmScrolledTextClass::AmScrolledTextClass( wParent, szName);
 *
 * Parameters:
 *   Widget     wParent : Parent of the text field widget.
 *   const char *szName : Name of the text field widget.
 * 
 * Description:
 *    
 */

AmScrolledTextClass::AmScrolledTextClass( Widget     wParent,
                                      const char *szName,
                                      Boolean    fVariable)
    : AmBaseWidgetClass( wParent, szName)
{

    _fVariable = fVariable; /* Visual policy. */

    initializeValues();
    createInterface();
}

AmScrolledTextClass::AmScrolledTextClass( AmBaseWidgetClass *bwcParent,
                                      const char      *szName,
                                      Boolean         fVariable)
    : AmBaseWidgetClass( bwcParent, szName)
{

    _fVariable = fVariable; /* Visual policy. */

    initializeValues();
    createInterface();
}

AmScrolledTextClass::~AmScrolledTextClass()
{
}


void
AmScrolledTextClass::ShowCursorPosition( Boolean fShow)
{
    XtVaSetValues( _wText, XmNcursorPositionVisible, fShow, NULL);
}

void
AmScrolledTextClass::WrapWords( Boolean fWrap)
{
    XtVaSetValues( _wText, XmNwordWrap, fWrap, NULL);
}

void
AmScrolledTextClass::AllowEditing( Boolean fEditable)
{
    XtVaSetValues( _wText, XmNeditable, fEditable, NULL);
}

void
AmScrolledTextClass::SetAlwaysVisible( Boolean fVisible)
{
    if ( fVisible == TRUE) {
        XtVaSetValues( _wWindow, XmNscrollBarDisplayPolicy, XmSTATIC, NULL);
    } else {
        XtVaSetValues( _wWindow, XmNscrollBarDisplayPolicy, XmAS_NEEDED, NULL);
    }
}

void
AmScrolledTextClass::SetVisibleWhenNeeded( Boolean fVisible)
{
    if ( fVisible == TRUE) {
        XtVaSetValues( _wWindow, XmNscrollBarDisplayPolicy, XmAS_NEEDED, NULL);
    } else {
        XtVaSetValues( _wWindow, XmNscrollBarDisplayPolicy, XmSTATIC, NULL);
    }
}

void
AmScrolledTextClass::SetText( char *sz)
{
    XmTextSetString( _wText, sz);
}

void
AmScrolledTextClass::ShowHorizontalScrollBar( Boolean fShow)
{
    Widget w;

    XtVaGetValues( _wWindow, XmNhorizontalScrollBar, &w, NULL);
    if ( fShow == TRUE) {
        XtManageChild( w);
    } else {
        XtUnmanageChild( w);
    }
}


void
AmScrolledTextClass::ShowVerticalScrollBar( Boolean fShow)
{
    Widget w;

    XtVaGetValues( _wWindow, XmNverticalScrollBar, &w, NULL);
    if ( fShow == TRUE) {
        XtManageChild( w);
    } else {
        XtUnmanageChild( w);
    }
}


void
AmScrolledTextClass::AppendText( char *sz)
{
    int iPos;

    iPos = XmTextGetLastPosition( _wText);
    XmTextInsert( _wText, iPos, sz);
}
