#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/ScrolledW.h>

#include <ToolKit/ScrolledWindowClass.h>

/*
 * Function: AmScrolledWindowClass::AmScrolledWindowClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmScrolledWindowClass.
 */

AmScrolledWindowClass::AmScrolledWindowClass()
    : AmBaseWidgetClass()
{
    _wScrolledWindow = NULL;
}


/*
 * Function: AmScrolledWindowClass::AmScrolledWindowClass( wParent, szName)
 *
 * Purpose:
 *   ScrolledWindow constructor which creates the scrolled window
 *   widget using the passed in parent widget and passed in name.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 * Comments:
 *   Creates a widget class using the passed in name as the widget's name.
 */

AmScrolledWindowClass::AmScrolledWindowClass( Widget     wParent,
                                          const char *szName,
                                          Boolean    fVariable)
    : AmBaseWidgetClass( wParent, szName)
{
    _wScrolledWindow = XtVaCreateManagedWidget( _szName,
                                                xmScrolledWindowWidgetClass,
                                                _wParent,
                                                XmNscrollingPolicy,
                                                    XmAUTOMATIC,
                                                XmNvisualPolicy,
                                                ( fVariable ) ?
                                                    XmVARIABLE : 
                                                    XmCONSTANT,
                                                NULL);
    _wBase = _wScrolledWindow;
 
    addDestroyCallback();
}


/*
 * Function: AmScrolledWindowClass::AmScrolledWindowClass( bwcParent, szName)
 *
 * Purpose:
 *   ScrolledWindow constructor which creates the scrolled window
 *   widget using the passed in parent widget and passed in name.
 *
 * Parameters:
 *   AmBaseWidgetClass bwcParent : Parent widget of the new widget.
 *   const char      *szName   : Name of the widget.
 */

AmScrolledWindowClass::AmScrolledWindowClass( AmBaseWidgetClass *bwcParent,
                                          const char      *szName,
                                          Boolean         fVariable)
    : AmBaseWidgetClass( bwcParent, szName)
{
    _wScrolledWindow = XtVaCreateManagedWidget( _szName,
                                                xmScrolledWindowWidgetClass,
                                                _wParent,
                                                XmNscrollingPolicy,
                                                    XmAUTOMATIC,
                                                XmNvisualPolicy,
                                                ( fVariable ) ?
                                                    XmVARIABLE : 
                                                    XmCONSTANT,
                                                NULL);
    _wBase = _wScrolledWindow;

    addDestroyCallback();
}

/*
 * Function: AmScrolledWindowClass::~AmScrolledWindowClass()
 *
 * Purpose:
 *   Deconstructor.
 */

AmScrolledWindowClass::~AmScrolledWindowClass()
{
}


void
AmScrolledWindowClass::SetAlwaysVisible()
{
    XtVaSetValues( _wScrolledWindow,
                   XmNscrollBarDisplayPolicy, XmSTATIC,
                   NULL);
}


void
AmScrolledWindowClass::SetVisibleWhenNeeded()
{
    XtVaSetValues( _wScrolledWindow,
                   XmNscrollBarDisplayPolicy, XmAS_NEEDED,
                   NULL);
}
