#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <ToolKit/ToolBarClass.h>


void
AmToolBarClass::initializeValues()
{
    _pwcFrame   = NULL;
    _pwcPane    = NULL;
    _lstButtons = NULL;
}


void
AmToolBarClass::createInterface()
{
    char    sz[ TEXTLEN];

    sprintf( sz, "%sFrame", _szName);
    _pwcFrame = new AmFrameClass( _wParent, sz, FALSE);
    _pwcPane  = new AmPaneClass( _pwcFrame, _szName);

    _wBase      = *_pwcFrame; 

    addDestroyCallback();

    _lstButtons = NULL;
}

/*
 * Function: AmToolBarClass::AmToolBarClass()
 *
 * Description:
 *   Default constructor. 
 */

AmToolBarClass::AmToolBarClass()
    : AmBaseWidgetClass()
{
    initializeValues();
}


/*
 * Function: AmToolBarClass::AmToolBarClass( wParent, szName);
 *
 * Parameters:
 *   Widget     wParent : Parent of the tool bar widget.
 *   const char *szName : Name of the tool bar widget.
 * 
 * Description:
 *    
 */

AmToolBarClass::AmToolBarClass( Widget     wParent,
                            const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


AmToolBarClass::AmToolBarClass( AmBaseWidgetClass *bwcParent,
                            const char      *szName)
    : AmBaseWidgetClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}

AmToolBarClass::~AmToolBarClass()
{
    if ( _lstButtons != NULL) {
        AmJustFreeLst( _lstButtons);
        _lstButtons = NULL;
    }
}


AmButtonClass *
AmToolBarClass::AddButton( const char     *szName,
                         XtCallbackProc xcp,
                         XtPointer      xp)
{
    AmButtonClass *pwcButton;

    if ( _pwcPane != NULL) {  

        pwcButton = new AmButtonClass( _pwcPane, szName, xcp, xp);
        _lstButtons = AmInsertNodeLst( pwcButton, _lstButtons);

        return( pwcButton);
    }

    return( NULL);
}


AmToggleButtonClass *
AmToolBarClass::AddToggleButton( const char     *szName,
                               XtCallbackProc xcp,
                               XtPointer      xp)
{
    AmToggleButtonClass *pwcToggle;

    if ( _pwcPane != NULL) {  

        pwcToggle = new AmToggleButtonClass( _pwcPane, szName, xcp, xp);
        _lstButtons = AmInsertNodeLst( pwcToggle, _lstButtons);

        return( pwcToggle);
    }

    return( NULL);
}

void
AmToolBarClass::AddSeparator()
{
    char sz[ TEXTLEN];

    if ( _pwcPane != NULL) {  
        sprintf( sz, "%sSeparator", _szName);
        new AmSeparatorClass( _pwcPane, sz);
    }
}
