#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdarg.h>
#if !defined (__BSD44__)
#include <alloca.h>
#endif

#include <Xm/DrawingA.h>
#include <Xm/Frame.h>

#include <ToolKit/ToolKit.h>

#define RESBUFSIZE 4096

/* Make a temporary string that gets allocated on the stack. */

#define MakeTempString( szTemp, szSrc)                      \
{                                                           \
    int iLength;                                            \
                                                            \
    iLength = strlen( szSrc) + 1;                           \
    szTemp  = ( char *) alloca( sizeof( char) * iLength);   \
    if ( szTemp != NULL) {                                  \
        memcpy( szTemp, szSrc, iLength);                    \
    }                                                       \
}


/* Send a fake expose event to the passed in window. */

void
AmSendFakeExpose( Display *pDisplay,
                Window  window)
{
    XExposeEvent xEvent;
    Window       windowRoot;
    unsigned int cx, cy, iBorder, iDepth;
    int          x, y;

    xEvent.type    = Expose;
    xEvent.window  = window;
    xEvent.count   = 0;        /* Some callbacks compress exposes. */
    xEvent.display = pDisplay;

    XGetGeometry( pDisplay, window, &windowRoot, &x, &y, &cx, &cy,
                  &iBorder, &iDepth);

    xEvent.x       = 0;
    xEvent.y       = 0;
    xEvent.width   = cx;
    xEvent.height  = cy;

    XSendEvent( pDisplay, window, True, ExposureMask, ( XEvent *) &xEvent);
}


/* Given a widget in the widget tree retrieve the top level widget of the
   tree. */

Widget
AmGetTopLevelWidget( Widget w)
{
    if ( w == NULL) {
        return( NULL);
    }

    while ( XtParent( w)) {
        w = XtParent( w);
    }
  
    return( w);
}


/* Given a widget in the widget tree retrieve the closes ancestor shell 
   widget. */

Widget
AmGetShellWidget( Widget w)
{
    while ( w && !XtIsShell( w)) {
        w = XtParent( w);
    }

    return( w);
}


/* Given a widget of a dialog.  Make sure that the dialog is poped up. */ 
   
void
AmForceDisplayDialog( Widget  w)
{
    Widget            wShell, wTop;
    Display           *pxDisplay;
    Window            xTopWindow, xShellWindow;
    XtAppContext      xac;
    XWindowAttributes xwa;
    XEvent            xEvent;
 
    wShell = AmGetShellWidget( w);
    wTop   = AmGetTopLevelWidget( w);

    if ( XtIsRealized( wShell) && XtIsRealized( wTop)) {

        xac = XtWidgetToApplicationContext( w);

        pxDisplay    = XtDisplay( wTop);
        xTopWindow   = XtWindow( wTop);
        xShellWindow = XtWindow( wShell);

    /* Wait for the dialog to reach the mapped state. */

        while ( XGetWindowAttributes( pxDisplay, xShellWindow, &xwa) &&
                xwa.map_state != IsViewable) {

        /*
         * If the primary shell is (or becomes) unviewable or unmapped,
         *   it is probably iconic and nothing will happen.
         */

            if ( XGetWindowAttributes( pxDisplay, xTopWindow, &xwa) &&
                 xwa.map_state != IsViewable) {
                break;
            }

        /*
         * We are guaranteed that there will be an event of some kind -
         *   at least that's what the manual says.
         */

            XtAppNextEvent( xac, &xEvent);
            XtDispatchEvent( &xEvent);
        }

    /* The next XSync() will get an expose event. */

        XSync( pxDisplay, FALSE);
        XmUpdateDisplay( wTop);

    }
}


/* Given a widget of a dialog.  Make sure that the dialog is poped up. */ 

void
AmForceRemoveDialog( Widget w)
{
    Widget            wShell, wTop;
    Display           *pxDisplay;
    Window            xTopWindow, xShellWindow;
    XtAppContext      xac;
    XWindowAttributes xwa;
    XEvent            xEvent;
 
    wShell = AmGetShellWidget( w);
    wTop   = AmGetTopLevelWidget( w);

    if ( XtIsRealized( wShell) && XtIsRealized( wTop)) {

        xac = XtWidgetToApplicationContext( w);

        pxDisplay    = XtDisplay( wTop);
        xTopWindow   = XtWindow( wTop);
        xShellWindow = XtWindow( wShell);

    /* Wait for the dialog to reach the mapped state. */

        while ( XGetWindowAttributes( pxDisplay, xShellWindow, &xwa) &&
                xwa.map_state == IsViewable) {

        /*
         * If the primary shell is (or becomes) unviewable or unmapped,
         *   it is probably iconic and nothing will happen.
         */

            if ( XGetWindowAttributes( pxDisplay, xTopWindow, &xwa) &&
                 xwa.map_state != IsViewable) {
                break;
            }

        /*
         * We are guaranteed that there will be an event of some kind -
         *   at least that's what the manual says.
         */

            XtAppNextEvent( xac, &xEvent);
            XtDispatchEvent( &xEvent);
        }

    /* The next XSync() will get an expose event. */

        XSync( pxDisplay, FALSE);
        XmUpdateDisplay( wTop);

    }
}
