#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <ToolKit/WarningBoxClass.h>


/*
 * Function: AmWarningBoxClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmWarningBoxClass::initializeValues()
{
}


/*
 * Function: AmWarningBoxClass::createInterface( szMessage)
 *
 * Purpose:
 *   Creates the widgets used in the AmWarningBoxClass.
 *
 * Comments:
 *   The message box class consists of a popup shell with a message 
 *   box inside.
 */

void
AmWarningBoxClass::createInterface( const char *szMessage)
{
    setIcon( XmDIALOG_WARNING);

    if ( szMessage != NULL) {
        setMessageString( szMessage);
    }

    hideCancelButton();
    hideHelpButton();

    Show();
}


/*
 * Function: AmWarningBoxClass::AmWarningBoxClass()
 *
 * Purpose:
 *   Default constructor.
 *
 * Comments:
 *   Clears out all the fields specific to AmWarningBoxClass also
 *   calls the AmBaseWidgetClass default constructor.
 */

AmWarningBoxClass::AmWarningBoxClass()
    : AmBoxClass()
{
    initializeValues();
}


/*
 * Functions: AmWarningBoxClass::AmWarningBoxClass( wParent, szName)
 *            AmWarningBoxClass::AmWarningBoxClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmWarningBoxClass.  Both functions work the same
 *   way except that one takes a widget and the other takes a "wrapped"
 *   widget.
 *
 * Parameters:
 *   Widget     wParent : Parent widget of the new widget.
 *   const char *szName : Name of the widget.
 *
 */

AmWarningBoxClass::AmWarningBoxClass( Widget     wParent,
                                      const char *szName,
                                      const char *szMessage)
    : AmBoxClass( wParent, szName)
{
    initializeValues();
    createInterface( szMessage);
}


/* See above comment. */

AmWarningBoxClass::AmWarningBoxClass( AmBaseWidgetClass *bwcParent,
                                      const char        *szName,
                                      const char        *szMessage)
    : AmBoxClass( bwcParent, szName)
{
    initializeValues();
    createInterface( szMessage);
}


/*
 * Function: AmWarningBoxClass::~AmWarningBoxClass()
 *
 * Purpose:
 *   Destructor for the AmWarningBoxClass.
 *
 * Comments:
 *   Frees up all the memory used by the widget class.
 *
 */

AmWarningBoxClass::~AmWarningBoxClass()
{
}
