#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/cursorfont.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/ScrolledW.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/log.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/template.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/postscript.h>
#include <anvil/plot_cback.h>
#include <anvil/motifdisplay.h>
#include <anvil/textannot.h>
#include <anvil/imagelegend.h>
#include <anvil/linelegend.h>
#include <anvil/polylegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/user_data.h>
#include <anvil/userevents.h>
#include <anvil/envdefs.h>
#include <anvil/slick.h>
#include <anvil/editobject.h>
#include <anvil/outline.h>
#include <anvil/display.h>
#include <anvil/xapp.h>
#include <anvil/fonts.h>
#include <anvil/xpm.h>
#include <anvil/defaults.h>
#include <anvil/visual.h>
#include <anvil/xcolor.h>
#include <anvil/editobject.h>
#include <anvil/imageannot.h>
#include <anvil/rectangleannot.h>
#include <anvil/grid.h>
#include <anvil/distscale.h>
#include <anvil/dlog_utility.h>

#include <cad/CadEditorClass.h>

void
CadEditorClass::initializeValues()
{
    _pMotifDisplay = NULL;
}

void
CadEditorClass::createInterface()
{

/* Create the editor's form. */

    _pwcForm = new AmFormClass( _wParent, "editorForm");
    _wBase = *_pwcForm;
    addDestroyCallback();

/* Create the menu bar. */

    _pwcMenu = new AmMenuClass( _pwcForm, "menuBar");
    _pwcMenu->AttachLeft();
    _pwcMenu->AttachTop();
    _pwcMenu->AttachRight();

/* Message area. */

    _pwcStatusForm = new AmFormClass( _pwcForm, "statusForm");
    _pwcStatusForm->AttachLeft();
    _pwcStatusForm->AttachRight();
    _pwcStatusForm->AttachBottom();

    _pwcMessageFrame = new AmFrameClass( _pwcStatusForm, "messageFrame", FALSE); 
    _pwcMessageFrame->AttachLeft();
    _pwcMessageFrame->AttachTop();
    _pwcMessageFrame->AttachRight();
    _pwcMessageFrame->AttachBottom();
 
    _pwcTextMessage = new AmLabelClass( _pwcMessageFrame, "messageText"); 

/* Create the tool bar. */

    _pwcToolBar = new AmToolBarClass( _pwcForm, "toolBar");
    _pwcToolBar->AttachLeft();
    _pwcToolBar->AttachTop( _pwcMenu);
    _pwcToolBar->AttachRight();

/* Create a form to hold the tool bars on the right hand side. */

    _pwcRightForm = new AmFormClass( _pwcForm, "rightForm");
    _pwcRightForm->AttachTop( _pwcToolBar);
    _pwcRightForm->AttachRight();
    _pwcRightForm->AttachBottom( _pwcStatusForm);

/* Create the state related utility bar (select, add, ...). */

    _pwcStateUtilityBar = new AmToolBarClass( _pwcRightForm, "stateUtilityBar");
    _pwcStateUtilityBar->AttachLeft( );
    _pwcStateUtilityBar->AttachTop();
    _pwcStateUtilityBar->AttachRight();

/* Create the zoom related utility bar. */

    _pwcZoomUtilityBar = new AmToolBarClass( _pwcRightForm, "zoomUtilityBar");
    _pwcZoomUtilityBar->AttachLeft();
    _pwcZoomUtilityBar->AttachTop( _pwcStateUtilityBar);
    _pwcZoomUtilityBar->AttachRight();

/* Main drawing area: */

    _pwcCornerFrame = new AmFrameClass( _pwcForm, "cornerFrame", FALSE); 
    _pwcCornerFrame->AttachLeft();
    _pwcCornerFrame->AttachTop( _pwcToolBar);

    _pwcWindow = new AmScrolledWindowClass( _pwcForm, "window", TRUE);
    _pwcWindow->AttachLeft( _pwcCornerFrame);
    _pwcWindow->AttachTop( _pwcCornerFrame);
    _pwcWindow->AttachRight( _pwcRightForm);
    _pwcWindow->AttachBottom( _pwcStatusForm);

    _pwcArea = new AmDrawingAreaClass( _pwcWindow, "area");

    _pwcHorizontalRuler = new AmRulerClass( _pwcForm, "topRuler",
                                          *_pwcArea , FALSE); 
    _pwcHorizontalRuler->SetDPI( 72);
    _pwcHorizontalRuler->AttachLeft( _pwcCornerFrame);
    _pwcHorizontalRuler->AttachTop( _pwcToolBar);
    _pwcHorizontalRuler->AttachRight( _pwcZoomUtilityBar);

    XtVaSetValues( *_pwcHorizontalRuler, BottomWith( *_pwcCornerFrame), NULL);

    _pwcVerticalRuler = new AmRulerClass( _pwcForm, "leftRuler",
                                        *_pwcArea, TRUE); 
    _pwcVerticalRuler->SetDPI( 72);
    _pwcVerticalRuler->AttachLeft();
    _pwcVerticalRuler->AttachTop( _pwcCornerFrame);
    XtVaSetValues( *_pwcVerticalRuler, RightWith( *_pwcCornerFrame), NULL);
    _pwcVerticalRuler->AttachBottom( _pwcStatusForm);

/* Create the double buffered motif display. */

    _pXDisplay = new AmXDisplayClass( *_pwcArea);
    _pXWindow = new AmXWindowClass( _pXDisplay, *_pwcArea);

    _pMotifDisplay = new AmMotifDisplayClass( _pXDisplay, _pXWindow, FALSE);
    _pMotifDisplay->GetDisplay()->SetDPI( 72);

/* Set the message area. */

    _pMotifDisplay->SetMessage( *_pwcTextMessage);

/* Create the buttons for the tool bar. */

/* Create the buttons for the utility bar. */

    _pwcStateUtilityBar->AddToggleButton( "selectBtn", NULL, NULL);
    _pwcStateUtilityBar->AddToggleButton( "addPointBtn", NULL, NULL);
    _pwcStateUtilityBar->AddToggleButton( "addLineBtn", NULL, NULL);
    _pwcStateUtilityBar->AddToggleButton( "addPolyLineBtn", NULL, NULL);
    _pwcStateUtilityBar->AddToggleButton( "addPolygonBtn", NULL, NULL);

/* Create the buttons for the utility bar. */

    _pwcZoomUtilityBar->AddButton( "zoom2xBtn", NULL, NULL);
    _pwcZoomUtilityBar->AddButton( "zoomRectBtn", NULL, NULL);
    _pwcZoomUtilityBar->AddButton( "zoomTextBtn", NULL, NULL);

    _pwcZoomUtilityBar->AddButton( "unzoom2xBtn", NULL, NULL);
    _pwcZoomUtilityBar->AddButton( "unzoomBtn", NULL, NULL);
    _pwcZoomUtilityBar->AddButton( "panBtn", NULL, NULL);

/* Create the file menu. */

    _pwcFilePane = new AmMenuPaneClass( _pwcMenu, "fileCasc"); 
    _pwcFilePane->AddButton( "newMenuBtn", NULL, NULL);

    _pwcFilePane->AddSeparator();

    _pwcFilePane->AddButton( "openMenuBtn", NULL, NULL);
    _pwcFilePane->AddButton( "saveMenuBtn", NULL, NULL);
    _pwcFilePane->AddButton( "saveAsMenuBtn", NULL, NULL);

    _pwcFilePane->AddSeparator();

/*
 * Figure out whether the editor is standalone (quit button) or not (close
 *   button).
 */

    _fStandalone = ( XtParent( _wParent) == NULL);
    if ( _fStandalone) {
        _pwcFilePane->AddButton( "quitMenuBtn", NULL, NULL);
    } else {
        _pwcFilePane->AddButton( "closeMenuBtn", NULL, NULL);
    }

/* Create the edit menu. */

    _pwcEditPane = new AmMenuPaneClass( _pwcMenu, "editCasc");
    _pwcEditPane->AddButton( "cutMenuBtn", NULL, NULL);
    _pwcEditPane->AddButton( "copyMenuBtn", NULL, NULL);
    _pwcEditPane->AddButton( "pasteMenuBtn", NULL, NULL);
    _pwcEditPane->AddButton( "deleteMenuBtn", NULL, NULL);

    _pwcEditPane->AddSeparator();

    _pwcEditPane->AddButton( "selectAllMenuBtn", NULL, NULL);
    _pwcEditPane->AddButton( "selectNoneMenuBtn", NULL, NULL);

    _pwcEditPane->AddSeparator();


    _pwcPopPane = new AmMenuPaneClass( _pwcEditPane, "popCasc");
    _pwcPopPane->AddButton( "topMenuBtn", NULL, NULL);
    _pwcPopPane->AddButton( "upMenuBtn", NULL, NULL);

    _pwcPushPane = new AmMenuPaneClass( _pwcEditPane, "pushCasc");
    _pwcPushPane->AddButton( "bottomMenuBtn", NULL, NULL);
    _pwcPushPane->AddButton( "downMenuBtn", NULL, NULL);

    _pwcEditPane->AddSeparator();

    _pwcEditPane->AddButton( "prefsMenuBtn", NULL, NULL);


/* Add the handler that automatically scrolls the window when the mouse
   is being dragged out of its extents (while one of the buttons are
   pressed). */

    XtAddEventHandler( *_pwcArea, LeaveWindowMask, FALSE,
                       ( XtEventHandler) AmScrollButtonHandler,
                       NULL);
}


/* Default constructor. */

CadEditorClass::CadEditorClass()
    : AmBaseWidgetClass()
{
    _pMotifDisplay = NULL;

/* Interface widgets. */

    _pwcForm            = NULL;
    _pwcMessageFrame    = NULL;
    _pwcTextMessage     = NULL;

    _pwcCornerFrame     = NULL;
    _pwcWindow          = NULL;
    _pwcArea            = NULL;

    _pwcMenu            = NULL;
    _pwcFilePane        = NULL;
    _pwcEditPane        = NULL;

    _pwcToolBar         = NULL;
    _pwcZoomUtilityBar  = NULL;

    _pwcHorizontalRuler = NULL;
    _pwcVerticalRuler   = NULL;

/* X specific classes. */

    _pXDisplay = NULL;
    _pXWindow  = NULL;
}

CadEditorClass::CadEditorClass( Widget     wParent,
                                const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{
    initializeValues();
    createInterface();
}

CadEditorClass::CadEditorClass( AmBaseWidgetClass *pwcParent,
                                const char      *szName)
    : AmBaseWidgetClass( pwcParent, szName)
{
    initializeValues();
    createInterface();
}

CadEditorClass::~CadEditorClass()
{

    XtRemoveEventHandler( *_pwcArea, LeaveWindowMask, FALSE,
                          ( XtEventHandler) AmScrollButtonHandler,
                          NULL);

/* Note that we don't need to delete either the window, nor the display,
   nor the motif display.  These get deleted automatically when the widgets
   get destroyed. */

    _pMotifDisplay      = NULL;
    _pXWindow           = NULL;
    _pXDisplay          = NULL;

/* The same goes for all of the widgets. */

    _pwcMenu            = NULL;
    _pwcFilePane        = NULL;
    _pwcEditPane        = NULL;
    _pwcZoomUtilityBar  = NULL;
    _pwcHorizontalRuler = NULL;
    _pwcVerticalRuler   = NULL;

}
