#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#if !defined(__BSD44__)
#include <alloca.h>
#endif

#include <Xm/Xm.h>

#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/ToggleB.h>
#include <Xm/PanedW.h>
#include <Xm/Separator.h>
#include <Xm/FileSB.h>
#include <Xm/Scale.h>
#include <Xm/Frame.h>

#include <X11/cursorfont.h>
#include <X11/IntrinsicP.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/template.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/postscript.h>
#include <anvil/plot_cback.h>
#include <anvil/motifdisplay.h>
#include <anvil/cadplot.h>
#include <anvil/polygonplot.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/userevents.h>
#include <anvil/envdefs.h>
#include <anvil/slick.h>
#include <anvil/editobject.h>
#include <anvil/outline.h>
#include <anvil/display.h>
#include <anvil/xapp.h>
#include <anvil/fonts.h>
#include <anvil/xpm.h>
#include <anvil/defaults.h>
#include <anvil/question_dlog.h>

#include <anvil/cad/cad.h>
#include <anvil/cad/defs.h>
#include <anvil/cad/segment.h>
#include <anvil/cad/object.h>

#define SELECT_RADIUS		3
#define SELECT_SENSITIVITY	10
#define POINT_WIDTH		5

typedef struct cadEditArgsRec_ {
    AmMotifDisplayClass *motifDisplay;
    AmOverlayClass      *overlay;
    AmScreenObjClass    *layer;
    Widget            cadEditForm;
    XtEventHandler    handler;
    handlerArgsPtr    handlerArgs;
    int               flags;
} cadEditArgsRec, *cadEditArgsPtr;

typedef struct toolBarRec_ {
    Widget            form;
    Widget            label;
    Widget            toolForm;
    Widget            editForm;
} toolBarRec, *toolBarPtr;

const int AmADD_POINT_PRIORITY = 800;

Widget CadEditorForm( Widget parent);

static AmIndexDataRec indexData = {
    "Attribute Index Data", /* name */
    AmINDEX_LINEAR,         /* indexType ( AmINDEX_LINEAR, ... ) */
    0.0,                    /* min */
    100.0,                  /* max */
    TRUE,                   /* minFixed */
    TRUE,                   /* maxFixed */
    FALSE,                  /* minClip */
    FALSE,                  /* maxClip */
    1.0,                    /* scale */
    NULL,                   /* fn */
    NULL,                   /* clientData */
    0,                      /* length */
    NULL,                   /* binvec */
    NULL,                   /* table */
    0,                      /* offset */
    AmFLOAT,                /* dataType ( AmINT, AmCHAR, AmSHORT, ... ) */
    NULL,                   /* getIndexFn */
    0,                      /* var */
    0,                      /* units */
    NULL                    /* strings */
};

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

static void
unselectAllObjects( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->selected != NULL) {
	AmJustFreeLst( handlerArgs->selected);
	handlerArgs->selected = NULL;
    }
}

static void
selectObject( handlerArgsPtr handlerArgs, long objectID)
{
    AmListPtr temp;

    for ( temp = handlerArgs->selected; temp != NULL; temp = temp->next) {
	if ( (( void *) objectID) == temp->data) {
	    break;
	}
    }

    if ( temp == NULL) {
	handlerArgs->selected = AmInsertNodeLst( ( void *) objectID,
					       handlerArgs->selected);
    }
}

static void
unselectObject( handlerArgsPtr handlerArgs, long objectID)
{
    AmListPtr temp, prev;

    prev = NULL;
    for ( temp = handlerArgs->selected; temp != NULL; ) {
	if ( objectID == ( long) temp->data) {
	    if ( prev != NULL) {
		prev->next = temp->next;
		free( temp);
		break;
	    } else {
		handlerArgs->selected = temp->next;
		free( temp);
		break;
	    }
	} else {
	    prev = temp;
	}
    }
}

static AnvilBoolean
isSelected( handlerArgsPtr handlerArgs, long objectID)
{
    AmListPtr temp;

    for ( temp = handlerArgs->selected; temp != NULL; temp = temp->next) {
	if ( (( void *) objectID) == temp->data) {
	    break;
	}
    }

    return( temp != NULL);
}
	
    
static void
moveSelectedObjects( handlerArgsPtr handlerArgs, int x, int y)
{
    AmCadPlotClass *cadPlot = handlerArgs->cadPlot;
    AmListPtr      lstISelected, lstParents, lstIParents;
    long         iSelected, iParent;
    int          shiftX, shiftY;

    shiftX = x - handlerArgs->startX;
    shiftY = y - handlerArgs->startY;

    for ( lstISelected = handlerArgs->selected;
          lstISelected != NULL;
          lstISelected = lstISelected->next) {

	iSelected = ( long) lstISelected->data;

        lstParents = cadPlot->GetParents( iSelected);
        for ( lstIParents = lstParents;
              lstIParents != NULL;
              lstIParents = lstIParents->next) {

	    iParent = ( long) lstIParents->data;

            if ( isSelected( handlerArgs, iParent) == TRUE) {
                break;
            }
        }

        if ( lstParents == NULL || lstIParents == NULL) {
       	    cadPlot->MoveObject( iSelected, shiftX, shiftY);
        }

        AmJustFreeLst( lstParents);
    }
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/


CadOutlineClass::CadOutlineClass()
{
    _clientData = NULL;
}

CadOutlineClass::CadOutlineClass( AmMotifDisplayClass *motifDisplay,
				  handlerArgsPtr    clientData)
    :AmOutlineClass( motifDisplay)
{
    _clientData = clientData;
}

CadOutlineClass::~CadOutlineClass()
{
}

handlerArgsPtr
CadOutlineClass::GetClientData()
{
    return( _clientData);
}

void
CadOutlineClass::SetClientData( handlerArgsPtr clientData)
{
    AnvilBoolean redraw;

    if ( _active == TRUE) {
	_motifDisplay->HideOutline( this);
	redraw = TRUE;
    } else {
	redraw = FALSE;
    }

    _clientData = clientData;

    if ( redraw == TRUE) {
	_motifDisplay->ShowOutline( this);
    }
}

AmErrorCode
CadOutlineClass::Draw( AmDisplayClass *display,
		       AmWindowClass  *window,
		       AmGCClass      *gc)
{
    handlerArgsPtr args = _clientData;
    AmCadPlotClass   *plot = ( AmCadPlotClass *) args->cadPlot;
    AmListPtr        temp;
    long           objectID;
    
    AmLineStyleEnum  lineStyle;
    int            lineWidth;
    
    /* Save the GC values to be restored later. */
    
    lineStyle = gc->GetLineStyle();
    gc->SetLineStyle( AmLINE_DOUBLE_DASH);
    
    lineWidth = gc->GetLineWidth();
    gc->SetLineWidth( 1);

    for ( temp = args->parents; temp != NULL; temp = temp->next) {
	objectID = ( long) temp->data;
	
	if ( args->flag == TRUE) {
	    plot->DrawOutlineObject( window, gc, objectID,
				     args->selected,
				     args->x - args->startX,
				     args->y - args->startY); 
	} else {
	    plot->DrawOutlineObject( window, gc, objectID,
				     NULL, 0, 0);
	}
    }
    
    gc->SetLineStyle( AmLINE_SOLID);
    gc->SetLineWidth( 1);
    
    for ( temp = args->selected; temp != NULL; temp = temp->next) {
	objectID = ( long) temp->data;
	
	if ( args->flag == TRUE) {
	    plot->DrawOutlineObject( window, gc, objectID,
				     args->selected,
				     args->x - args->startX,
				     args->y - args->startY);
	} else {
	    plot->DrawOutlineObject( window, gc, objectID,
				     NULL, 0, 0);
	}
    }

/* Restore the GC values. */

    gc->SetLineWidth( lineWidth);
    gc->SetLineStyle( lineStyle);

    return( AmOK);
}

void
cadSelectPointOutlineFn( AmDisplayClass *display,
                         AmWindowClass  *window,
                         AmGCClass      *gc,
                         outlinePtr   outline)
{
    handlerArgsPtr args = ( handlerArgsPtr) outline->clientData;
    AmPointPtr       points = args->points;
    int            x0, y0, x1, y1, ii; 

    if ( args->flag == TRUE) {
        if ( args->numPoints > 0) {

            x0 = points[ 0].x;
            y0 = points[ 0].y;

            for ( ii = 1; ii < args->numPoints; ii++) {
                x1 = points[ ii].x;
                y1 = points[ ii].y;

                display->DrawLine( window, gc, x0, y0, x1, y1);
            }
        }
    } else {

        if ( args->numPoints == 1) {

            x0 = points[ 0].x;
            y0 = points[ 0].y;

            display->DrawArc( window, gc,
                         x0 - SELECT_RADIUS,
                         y0 - SELECT_RADIUS,
                         SELECT_RADIUS * 2,
                         SELECT_RADIUS * 2,
                         0, M_PI * 2);
        }

    }
}

void
cadLineOutlineFn( AmDisplayClass *display,
                  AmWindowClass  *window,
                  AmGCClass      *gc,
                  outlinePtr   outline)
{
    handlerArgsPtr args = ( handlerArgsPtr) outline->clientData;
    AmPointPtr       points = args->points;
    int            x0, y0, x1, y1, ii;

    for ( ii = 0; ii < args->numPoints; ii++) {
        x1 = points[ ii].x;
        y1 = points[ ii].y;

        if ( ii != 0) {
            if ( x0 != x1 || y0 != y1) {
	        display->DrawRectangle( window, gc,
					x1 - POINT_WIDTH / 2,
					y1 - POINT_WIDTH / 2,
					POINT_WIDTH,
					POINT_WIDTH);
                display->DrawLine( window, gc, x0, y0, x1, y1);
            }
        } else {
            display->DrawRectangle( window, gc,
				    x1 - POINT_WIDTH / 2,
				    y1 - POINT_WIDTH / 2,
				    POINT_WIDTH,
				    POINT_WIDTH);
        }
        x0 = x1;
        y0 = y1;
    }

    if ( ii > 0) {
        x1 = points[ ii].x;
        y1 = points[ ii].y;

        if ( x0 != x1 || y0 != y1) {
            display->DrawRectangle( window, gc,
				    x1 - POINT_WIDTH / 2,
				    y1 - POINT_WIDTH / 2,
				    POINT_WIDTH,
				    POINT_WIDTH);
            display->DrawLine( window, gc, x0, y0, x1, y1);
        }
    }
}

void
cadPolygonOutlineFn( AmDisplayClass *display,
                     AmWindowClass  *window,
                     AmGCClass      *gc,
                     outlinePtr   outline)
{
    handlerArgsPtr args = ( handlerArgsPtr) outline->clientData;
    AmPointPtr       points = args->points;
    int            x0, y0, x1, y1, x2, y2, ii; 

    for ( ii = 0; ii < args->numPoints; ii++) {
        x2 = points[ ii].x;
        y2 = points[ ii].y;

        if ( ii != 0) {
            if ( x1 != x2 || y1 != y2) {
                display->DrawRectangle( window, gc,
				        x2 - POINT_WIDTH / 2,
				        y2 - POINT_WIDTH / 2,
				        POINT_WIDTH,
				        POINT_WIDTH);

                display->DrawLine( window, gc, x1, y1, x2, y2);
            }
        } else {

            display->DrawRectangle( window, gc,
				    x2 - POINT_WIDTH / 2,
				    y2 - POINT_WIDTH / 2,
				    POINT_WIDTH,
				    POINT_WIDTH);

            x0 = x2;
            y0 = y2;
        }

        x1 = x2;
        y1 = y2;
    }

    if ( ii > 0) {
        x2 = points[ ii].x;
        y2 = points[ ii].y;

        if ( x1 != x2 || y1 != y2) {

            display->DrawRectangle( window, gc,
				    x2 - POINT_WIDTH / 2,
				    y2 - POINT_WIDTH / 2,
				    POINT_WIDTH,
				    POINT_WIDTH);

            display->DrawLine( window, gc, x1, y1, x2, y2);
            x1 = x2;
            y1 = y2;
            ii++;
        }

        if ( ii > 2) {
            display->DrawLine( window, gc, x1, y1, x0, y0);
        }
    }
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

AnvilBoolean
cadAddLayerWrapper( AmDisplayClass     *display,
                    AmOverlayClass     *overlay,
                    AmTagValuePtr      tagValues,
                    int              numValues,
                    AmDataGraphicClass **dataGraphic,
                    addFileDataPtr   addFileData)
{
    AnvilBoolean result;

    result = ( addFileData->addLayer)(
        display, overlay, tagValues, numValues, dataGraphic,
        addFileData->addLayerClientData);

    return( result);
}


void
quitCback( Widget            ,
           AmMotifDisplayClass *motifDisplay,
           XtPointer)
{
    Widget shell;

    shell = AmGetWidgetShell( motifDisplay->GetArea());  

    delete motifDisplay;

    XtDestroyWidget( shell);
    exit(0);
}


void
openCback( Widget w, addFileDataPtr addFileData, XtPointer) 
{
    DefaultSingleFileBrowser( AmGetWidgetShell( w), NULL, addFileData);
}


void
deleteCback( Widget w, addFileDataPtr addFileData, XtPointer callData)
{
    AmMotifDisplayClass *motifDisplay = addFileData->motifDisplay;
    AmOverlayClass      *overlay = addFileData->overlay;

    AmDeleteLayerCallback( w, motifDisplay, overlay, callData);
}


static void
layerDeleteCallback( int            ,
                     cadEditArgsPtr editArgs,
                     AmScreenObjClass *layer)
{
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    handlerArgsPtr    handlerArgs = editArgs->handlerArgs;
    XtEventHandler    handler = editArgs->handler;

/*
 * If the layer that currently is selected is being removed... make
 *   sure that there is no outstanding handler.
 */

    if ( layer == editArgs->layer) {
        if ( handlerArgs != NULL) {
            AmRemoveHandler( motifDisplay->GetArea(), 
                editArgs->flags,
                ( XtEventHandler) handler, handlerArgs);
        }

        XtSetSensitive( editArgs->cadEditForm, FALSE);
        editArgs->layer = NULL;
    }

}


static void
layerSelectCallback( int            ,
                     cadEditArgsPtr editArgs,
                     AmScreenObjClass *layer)
{
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    handlerArgsPtr    handlerArgs = editArgs->handlerArgs;
    XtEventHandler    handler = editArgs->handler;

    if (( editArgs->layer != NULL) &&
            ( editArgs->layer->IsType( AmCAD_PLOT)) &&
            ( handlerArgs != NULL)) {

        AmRemoveHandler( motifDisplay->GetArea(),
		       editArgs->flags,
		       ( XtEventHandler) handler, handlerArgs);

        editArgs->handler = ( XtEventHandler) NULL;
        editArgs->handlerArgs = NULL;

    }

    if (( layer == NULL) || ( layer->IsType( AmCAD_PLOT) == FALSE)) {

        XtSetSensitive( editArgs->cadEditForm, FALSE);
        editArgs->layer = NULL;
        
    } else {

        XtSetSensitive( editArgs->cadEditForm, TRUE);
        editArgs->layer = layer;

        if ( (( AmCadPlotClass *) layer)->AddAttributeType( "Landuse",
                                                          &indexData) == AmOK) {
            fprintf( stderr, "YES!!! I added a type.  Whooopeee!!!\n");
        } else {
            fprintf( stderr, "OH NO!!! I wasn't able to add a type.\n");
        }
    }
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

handlerArgsPtr
makeHandlerArgs( AmMotifDisplayClass *motifDisplay,
                 AmCadPlotClass      *cadPlot)
{
    handlerArgsPtr handlerArgs;

    handlerArgs = ( handlerArgsPtr) malloc( sizeof( handlerArgsRec));
    if ( handlerArgs == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->cadPlot = cadPlot;
    handlerArgs->outline = NULL;
    handlerArgs->Outline = NULL;
    handlerArgs->numPoints = 0;
    handlerArgs->flag = FALSE;

    handlerArgs->selected = NULL;
    handlerArgs->parents = NULL;

    return( handlerArgs);
}


void
freeHandlerArgs( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->selected != NULL) {
	AmJustFreeLst( handlerArgs->selected);
    }

    if ( handlerArgs->parents != NULL) {
	AmJustFreeLst( handlerArgs->parents);
    }

    free( handlerArgs);
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void
removeAddPointHandler( handlerArgsPtr handlerArgs)
{
    freeHandlerArgs( handlerArgs);
}


void
suspendAddPointHandler( handlerArgsPtr)
{
}


void
restartAddPointHandler( handlerArgsPtr)
{
}


void
addPointHandler( Widget         widget,
                 handlerArgsPtr handlerArgs,
                 XEvent         *event,
                 AnvilBoolean        *)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmCadPlotClass      *cadPlot = handlerArgs->cadPlot;
    AmWindowClass       *window;
    AmGCClass           *GC;
    AmDisplayClass      *display;
    XButtonEvent      *buttonEvent;
    Dimension         width, height;
    long              pointID;
    
    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "addPointCancelMsg");

        /* Remove the handler. */
 
           AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask,
               ( XtEventHandler) addPointHandler, handlerArgs);

        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button1) {

            XtVaGetValues( motifDisplay->GetArea(),
                XmNwidth, &width, XmNheight, &height, NULL);

            GC = motifDisplay->GetGC();
            window = motifDisplay->GetWindow();
            GC->SetClipRectangle( 0, 0, width, height);
	    
            pointID = cadPlot->AddPoint( buttonEvent->x, buttonEvent->y);
	    cadPlot->DrawPoint( window, GC, pointID);
        }
    }
}


void
AddPointCallback( Widget            button,
                  cadEditArgsPtr    editArgs,
                  XtPointer)
{
    handlerArgsPtr    handlerArgs;
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    AmScreenObjClass    *layer = editArgs->layer;
    AmCadPlotClass      *cadPlot;

    if (( layer == NULL) || ( layer->IsType( AmCAD_PLOT) == FALSE)) {
        return;
    }

    cadPlot = ( AmCadPlotClass *)layer;
    handlerArgs = makeHandlerArgs( motifDisplay, cadPlot);

    motifDisplay->WriteResourceMessage( "addPointInstructionMsg");
    AmAddHandler( button, motifDisplay->GetArea(), 
        ButtonPressMask | ButtonReleaseMask,
        ( XtEventHandler) addPointHandler,
        handlerArgs, AmADD_POINT_PRIORITY,
        ( AmStateHandlerFn) removeAddPointHandler,
        ( AmStateHandlerFn) suspendAddPointHandler,
        ( AmStateHandlerFn) restartAddPointHandler);

    editArgs->handler = ( XtEventHandler) addPointHandler;
    editArgs->flags = ButtonPressMask | ButtonReleaseMask;
    editArgs->handlerArgs = handlerArgs;
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void
removeAddLineHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->RemoveOutline( handlerArgs->outline);
    }

    freeHandlerArgs( handlerArgs);
}


void
suspendAddLineHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->HideOutline( handlerArgs->outline);
    }
}


void
restartAddLineHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->ShowOutline( handlerArgs->outline);
    }
}


void
addLineHandler( Widget         widget,
                handlerArgsPtr handlerArgs,
                XEvent         *event,
                AnvilBoolean        *)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmCadPlotClass      *cadPlot = handlerArgs->cadPlot;
    AmWindowClass       *window;
    AmGCClass           *GC;
    Dimension         width, height;
    XButtonEvent      *buttonEvent;
    XMotionEvent      *motionEvent;
    long              dest, src, line;
    int               num;

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "addLineCancelMsg");

        /* Remove the handler. */
 
           AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
               PointerMotionMask,
               ( XtEventHandler) addLineHandler, handlerArgs);

        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;

        if ( buttonEvent->button == Button1) {

            XtVaGetValues( motifDisplay->GetArea(),
                XmNwidth, &width, XmNheight, &height, NULL);

            GC = motifDisplay->GetGC();
            window = motifDisplay->GetWindow();
            GC->SetClipRectangle( 0, 0, width, height);

            if ( handlerArgs->numPoints == 0) {
                num = handlerArgs->numPoints;
                handlerArgs->points[ num].x = buttonEvent->x; 
                handlerArgs->points[ num++].y = buttonEvent->y; 

                handlerArgs->points[ num].x = buttonEvent->x; 
                handlerArgs->points[ num].y = buttonEvent->y; 

                handlerArgs->numPoints = num;

                handlerArgs->outline = makeOutline( cadLineOutlineFn,
                    handlerArgs);
                motifDisplay->AddOutline( handlerArgs->outline);
            } else {
                motifDisplay->RemoveOutline( handlerArgs->outline);
                handlerArgs->outline = NULL;

                handlerArgs->numPoints = 0;

                src = cadPlot->AddPoint( handlerArgs->points[ 0].x,
                                   handlerArgs->points[ 0].y);
                cadPlot->DrawPoint( window, GC, src);

                dest = cadPlot->AddPoint( handlerArgs->points[ 1].x,
                                    handlerArgs->points[ 1].y);
                cadPlot->DrawPoint( window, GC, dest);

                line = cadPlot->AddLine( src, dest);
                cadPlot->DrawPolyLine( window, GC, line);
            }
        }
    } else if ( event->type == MotionNotify) {

        motionEvent = ( XMotionEvent *) event;

        if ( handlerArgs->numPoints > 0) {
            motifDisplay->HideOutline( handlerArgs->outline);

            num = handlerArgs->numPoints;
            handlerArgs->points[ num].x = motionEvent->x;
            handlerArgs->points[ num].y = motionEvent->y;

            motifDisplay->ShowOutline( handlerArgs->outline);
        }
    }
}


void
AddLineCallback( Widget            button,
                 cadEditArgsPtr    editArgs,
                 XtPointer)
{
    handlerArgsPtr    handlerArgs;
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    AmScreenObjClass    *layer = editArgs->layer;
    AmCadPlotClass      *cadPlot;

    if (( layer == NULL) || ( layer->IsType( AmCAD_PLOT) == FALSE)) {
        return;
    }

    cadPlot = ( AmCadPlotClass *)layer;
    handlerArgs = makeHandlerArgs( motifDisplay, cadPlot);

    motifDisplay->WriteResourceMessage( "addLineInstructionMsg");
    AmAddHandler( button, motifDisplay->GetArea(), 
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) addLineHandler,
        handlerArgs, AmADD_POINT_PRIORITY,
        ( AmStateHandlerFn) removeAddLineHandler,
        ( AmStateHandlerFn) suspendAddLineHandler,
        ( AmStateHandlerFn) restartAddLineHandler);

    editArgs->handler = ( XtEventHandler) addLineHandler;
    editArgs->flags = ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
    editArgs->handlerArgs = handlerArgs;
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void
removeAddPolyLineHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->RemoveOutline( handlerArgs->outline);
    }

    freeHandlerArgs( handlerArgs);
}


void
suspendAddPolyLineHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->HideOutline( handlerArgs->outline);
    }
}


void
restartAddPolyLineHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->ShowOutline( handlerArgs->outline);
    }
}


void
addPolyLineHandler( Widget         widget,
                    handlerArgsPtr handlerArgs,
                    XEvent         *event,
                    AnvilBoolean        *)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmCadPlotClass      *cadPlot = handlerArgs->cadPlot;
    AmWindowClass       *window;
    AmGCClass           *GC;
    AmDisplayClass      *display;
    Dimension         width, height;
    XButtonEvent      *buttonEvent;
    XMotionEvent      *motionEvent;
    long              *points, polyline;
    int               num, ii;

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "addPolyLineCancelMsg");

        /* Remove the handler. */
 
           AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
               PointerMotionMask,
               ( XtEventHandler) addPolyLineHandler, handlerArgs);

        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;

        if ( buttonEvent->button == Button1) {

            XtVaGetValues( motifDisplay->GetArea(),
                XmNwidth, &width, XmNheight, &height, NULL);

            GC = motifDisplay->GetGC();
            window = motifDisplay->GetWindow();
            GC->SetClipRectangle( 0, 0, width, height);

            if ( handlerArgs->numPoints == 0) {
                num = handlerArgs->numPoints;
                handlerArgs->points[ num].x = buttonEvent->x; 
                handlerArgs->points[ num++].y = buttonEvent->y; 

                handlerArgs->points[ num].x = buttonEvent->x; 
                handlerArgs->points[ num].y = buttonEvent->y; 

                handlerArgs->numPoints = num;

                handlerArgs->outline = makeOutline( cadLineOutlineFn,
                    handlerArgs);
                motifDisplay->AddOutline( handlerArgs->outline);
            } else {

                num = handlerArgs->numPoints;

                if (( handlerArgs->points[ num - 1].x == buttonEvent->x) &&
                    ( handlerArgs->points[ num - 1].y == buttonEvent->y)) {

                    motifDisplay->RemoveOutline( handlerArgs->outline);
                    handlerArgs->outline = NULL;

                    points = ( long *) alloca( sizeof( long) * num);

                    for ( ii = 0; ii < num; ii++) {
                        points[ ii] = cadPlot->AddPoint(
                                                   handlerArgs->points[ ii].x,
                                                   handlerArgs->points[ ii].y);
                    }

                    polyline = cadPlot->AddPolyLine( num, points);
                    cadPlot->DrawPolyLine( window, GC, polyline);

                    handlerArgs->numPoints = 0;

                } else {

                    motifDisplay->HideOutline( handlerArgs->outline);

                    handlerArgs->points[ num].x = buttonEvent->x; 
                    handlerArgs->points[ num++].y = buttonEvent->y; 

                    handlerArgs->points[ num].x = buttonEvent->x; 
                    handlerArgs->points[ num].y = buttonEvent->y; 

                    handlerArgs->numPoints = num;

                    motifDisplay->ShowOutline( handlerArgs->outline);
                }
            }
        }
    } else if ( event->type == MotionNotify) {

        motionEvent = ( XMotionEvent *) event;

        if ( handlerArgs->numPoints > 0) {
            motifDisplay->HideOutline( handlerArgs->outline);

            num = handlerArgs->numPoints;
            handlerArgs->points[ num].x = motionEvent->x;
            handlerArgs->points[ num].y = motionEvent->y;

            motifDisplay->ShowOutline( handlerArgs->outline);
        }
    }
}


void
AddPolyLineCallback( Widget            button,
                     cadEditArgsPtr    editArgs,
                     XtPointer)
{
    handlerArgsPtr    handlerArgs;
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    AmScreenObjClass    *layer = editArgs->layer;
    AmCadPlotClass      *cadPlot;

    if (( layer == NULL) || ( layer->IsType( AmCAD_PLOT) == FALSE)) {
        return;
    }

    cadPlot = ( AmCadPlotClass *)layer;
    handlerArgs = makeHandlerArgs( motifDisplay, cadPlot);

    motifDisplay->WriteResourceMessage( "addLineInstructionMsg");
    AmAddHandler( button, motifDisplay->GetArea(), 
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) addPolyLineHandler,
        handlerArgs, AmADD_POINT_PRIORITY,
        ( AmStateHandlerFn) removeAddPolyLineHandler,
        ( AmStateHandlerFn) suspendAddPolyLineHandler,
        ( AmStateHandlerFn) restartAddPolyLineHandler);

    editArgs->handler = ( XtEventHandler) addPolyLineHandler;
    editArgs->flags = ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
    editArgs->handlerArgs = handlerArgs;
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void
removeAddPolygonHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->RemoveOutline( handlerArgs->outline);
    }

    freeHandlerArgs( handlerArgs);
}


void
suspendAddPolygonHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->HideOutline( handlerArgs->outline);
    }
}


void
restartAddPolygonHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->outline != NULL) {
        handlerArgs->motifDisplay->ShowOutline( handlerArgs->outline);
    }
}


void
addPolygonHandler( Widget         widget,
                   handlerArgsPtr handlerArgs,
                   XEvent         *event,
                   AnvilBoolean        *)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmCadPlotClass      *cadPlot = handlerArgs->cadPlot;
    AmWindowClass       *window;
    AmGCClass           *GC;
    AmDisplayClass      *display;
    Dimension         width, height;
    XButtonEvent      *buttonEvent;
    XMotionEvent      *motionEvent;
    long              *points, polyline, polygon;
    int               num, ii;

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "addPolygonCancelMsg");

        /* Remove the handler. */
 
           AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
               PointerMotionMask,
               ( XtEventHandler) addPolygonHandler, handlerArgs);

        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;

        if ( buttonEvent->button == Button1) {

            XtVaGetValues( motifDisplay->GetArea(),
                XmNwidth, &width, XmNheight, &height, NULL);

            GC = motifDisplay->GetGC();
            window = motifDisplay->GetWindow();
            GC->SetClipRectangle( 0, 0, width, height);

            if ( handlerArgs->numPoints == 0) {
                num = handlerArgs->numPoints;
                handlerArgs->points[ num].x = buttonEvent->x; 
                handlerArgs->points[ num++].y = buttonEvent->y; 

                handlerArgs->points[ num].x = buttonEvent->x; 
                handlerArgs->points[ num].y = buttonEvent->y; 

                handlerArgs->numPoints = num;

                handlerArgs->outline = makeOutline( cadPolygonOutlineFn,
                    handlerArgs);
                motifDisplay->AddOutline( handlerArgs->outline);
            } else {

                num = handlerArgs->numPoints;

                if (( handlerArgs->points[ num - 1].x == buttonEvent->x) &&
                    ( handlerArgs->points[ num - 1].y == buttonEvent->y)) {

                    motifDisplay->RemoveOutline( handlerArgs->outline);
                    handlerArgs->outline = NULL;

                    points = ( long *) alloca( sizeof( long) * num);

                    for ( ii = 0; ii < num; ii++) {
                        points[ ii] = cadPlot->AddPoint(
                                                   handlerArgs->points[ ii].x,
                                                   handlerArgs->points[ ii].y);
                    }

                    polyline = cadPlot->AddPolyLine( num, points);
                    polygon = cadPlot->AddPolygon( 1, &polyline);

                    cadPlot->DrawPolygon( window, GC, polygon);

                    handlerArgs->numPoints = 0;

                } else {

                    motifDisplay->HideOutline( handlerArgs->outline);

                    handlerArgs->points[ num].x = buttonEvent->x; 
                    handlerArgs->points[ num++].y = buttonEvent->y; 

                    handlerArgs->points[ num].x = buttonEvent->x; 
                    handlerArgs->points[ num].y = buttonEvent->y; 

                    handlerArgs->numPoints = num;

                    motifDisplay->ShowOutline( handlerArgs->outline);
                }
            }
        }
    } else if ( event->type == MotionNotify) {

        motionEvent = ( XMotionEvent *) event;

        if ( handlerArgs->numPoints > 0) {
            motifDisplay->HideOutline( handlerArgs->outline);

            num = handlerArgs->numPoints;
            handlerArgs->points[ num].x = motionEvent->x;
            handlerArgs->points[ num].y = motionEvent->y;

            motifDisplay->ShowOutline( handlerArgs->outline);
        }
    }
}


void
AddPolygonCallback( Widget            button,
                    cadEditArgsPtr    editArgs,
                    XtPointer)
{
    handlerArgsPtr    handlerArgs;
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    AmScreenObjClass    *layer = editArgs->layer;
    AmCadPlotClass      *cadPlot;

    if (( layer == NULL) || ( layer->IsType( AmCAD_PLOT) == FALSE)) {
        return;
    }

    cadPlot = ( AmCadPlotClass *)layer;
    handlerArgs = makeHandlerArgs( motifDisplay, cadPlot);

    motifDisplay->WriteResourceMessage( "addLineInstructionMsg");
    AmAddHandler( button, motifDisplay->GetArea(), 
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) addPolygonHandler,
        handlerArgs, AmADD_POINT_PRIORITY,
        ( AmStateHandlerFn) removeAddPolygonHandler,
        ( AmStateHandlerFn) suspendAddPolygonHandler,
        ( AmStateHandlerFn) restartAddPolygonHandler);

    editArgs->handler = ( XtEventHandler) addPolygonHandler;
    editArgs->flags = ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
    editArgs->handlerArgs = handlerArgs;
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

void
removeSelectPointHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->Outline != NULL) {
        handlerArgs->motifDisplay->RemoveOutline( handlerArgs->Outline);
    }

    freeHandlerArgs( handlerArgs);
}


void
suspendSelectPointHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->Outline != NULL) {
	handlerArgs->motifDisplay->HideOutline( handlerArgs->Outline);
    }
}


void
restartSelectPointHandler( handlerArgsPtr handlerArgs)
{
    if ( handlerArgs->Outline != NULL) {
        handlerArgs->motifDisplay->ShowOutline( handlerArgs->Outline);
    }
}

void
selectPointHandler( Widget         widget,
                    handlerArgsPtr handlerArgs,
                    XEvent         *event,
                    AnvilBoolean        *)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmCadPlotClass      *cadPlot = handlerArgs->cadPlot;
    AmWindowClass       *window;
    AmGCClass           *GC;
    AmDisplayClass      *display;
    Dimension         width, height;
    XButtonEvent      *buttonEvent;
    XMotionEvent      *motionEvent;
    AmCadPointPtr       point;
    AnvilBoolean           closeness;
    long              ID, *points, num, *objects;
    int               ii, x, y, deltaX, deltaY;

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

	    /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "selectPointCancelMsg");

	    /* Remove the handler. */
 
	    AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
			   PointerMotionMask,
			   ( XtEventHandler) selectPointHandler, handlerArgs);

        } else if ( buttonEvent->button == Button1 &&
                    handlerArgs->flag == FALSE) {

            ID = cadPlot->FindPoint( buttonEvent->x, buttonEvent->y);
            cadPlot->GetPointLocation( ID, &x, &y);

            deltaX = x - buttonEvent->x;
            deltaY = y - buttonEvent->y;

            closeness = (( deltaX * deltaX + deltaY * deltaY) <= 
                         ( SELECT_SENSITIVITY * SELECT_SENSITIVITY));

            if ( closeness == FALSE) {
                ID = cadPlot->FindPolygon( buttonEvent->x, buttonEvent->y);
                if ( ID != AmINVALID) {
                    fprintf( stderr, "Polygon: %d\n", ID);
                    closeness = TRUE;
                }
            }

            if ( closeness == TRUE) {

                if ( handlerArgs->Outline != NULL) {
		    motifDisplay->HideOutline( handlerArgs->Outline);
                }

		if (!( buttonEvent->state & ShiftMask)) {
		    unselectAllObjects( handlerArgs);
		}
		
		selectObject( handlerArgs, ID);

		handlerArgs->flag = TRUE;
		handlerArgs->ID = ID;

		handlerArgs->x = buttonEvent->x;
		handlerArgs->y = buttonEvent->y;

		handlerArgs->startX = buttonEvent->x;
		handlerArgs->startY = buttonEvent->y;
		
		if ( handlerArgs->parents != NULL) {
		    AmJustFreeLst( handlerArgs->parents);
		    handlerArgs->parents = NULL;
		}

		handlerArgs->parents = cadPlot->GetParents( handlerArgs->selected);

		if ( handlerArgs->Outline != NULL) {
		    motifDisplay->ShowOutline( handlerArgs->Outline);
		} else {
		    handlerArgs->Outline = new CadOutlineClass( motifDisplay,
								handlerArgs);
		    motifDisplay->AddOutline( handlerArgs->Outline);
		}
            }
        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;
	
        if ( buttonEvent->button == Button1 && handlerArgs->flag == TRUE) {
	    
            if ( handlerArgs->Outline != NULL) {
		motifDisplay->HideOutline( handlerArgs->Outline);
            }

	    handlerArgs->flag = FALSE;
	    moveSelectedObjects( handlerArgs, buttonEvent->x, buttonEvent->y);
	    
	    if ( handlerArgs->Outline != NULL) {
		motifDisplay->ShowOutline( handlerArgs->Outline);
	    } else {
		handlerArgs->Outline = new CadOutlineClass( motifDisplay,
							    handlerArgs);
		motifDisplay->AddOutline( handlerArgs->Outline);
	    }
	    
	}
    } else if ( event->type == MotionNotify) {
	
        motionEvent = ( XMotionEvent *) event;

	if ( handlerArgs->flag == TRUE) {
	    if ( handlerArgs->Outline != NULL) {
		motifDisplay->HideOutline( handlerArgs->Outline);
	    }

	    handlerArgs->x = motionEvent->x;
	    handlerArgs->y = motionEvent->y;
	    
	    if ( handlerArgs->Outline != NULL) {
		motifDisplay->ShowOutline( handlerArgs->Outline);
	    } else {
		handlerArgs->Outline = new CadOutlineClass( motifDisplay,
							    handlerArgs);
		motifDisplay->AddOutline( handlerArgs->Outline);
	    }
	}
    }
}


void
SelectPointCallback( Widget            button,
                        cadEditArgsPtr    editArgs,
                        XtPointer)
{
    handlerArgsPtr    handlerArgs;
    AmMotifDisplayClass *motifDisplay = editArgs->motifDisplay;
    AmScreenObjClass    *layer = editArgs->layer;
    AmCadPlotClass      *cadPlot;

    if (( layer == NULL) || ( layer->IsType( AmCAD_PLOT) == FALSE)) {
        return;
    }

    cadPlot = ( AmCadPlotClass *)layer;
    handlerArgs = makeHandlerArgs( motifDisplay, cadPlot);

    motifDisplay->WriteResourceMessage( "selectPointInstructionMsg");
    AmAddHandler( button, motifDisplay->GetArea(), 
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) selectPointHandler,
        handlerArgs, AmADD_POINT_PRIORITY,
        ( AmStateHandlerFn) removeSelectPointHandler,
        ( AmStateHandlerFn) suspendSelectPointHandler,
        ( AmStateHandlerFn) restartSelectPointHandler);

    editArgs->handler = ( XtEventHandler) selectPointHandler;
    editArgs->flags = ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
    editArgs->handlerArgs = handlerArgs;
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/* Shell parent for the cad editor form. */

Widget
CadEditorShell( Widget parent)
{
    Widget	shell;

    shell = XtVaCreatePopupShell( "mapEditor", applicationShellWidgetClass,
                parent, NULL);
    CadEditorForm( shell);
    XtManageChild( shell);
    return( shell);
}


toolBarPtr
ToolBarForm( Widget            parent,
             AmMotifDisplayClass *motifDisplay,
             AmOverlayClass      *overlay,
             addFileDataPtr    addFileData)
{
    Widget         label;
    Widget         form, zoom2xBtn, zoomRectBtn, unzoom2xBtn,
                   unzoomBtn, panBtn, refreshBtn, selectLayerBtn,
                   deleteLayerBtn;
    Widget         editForm, movePointBtn, addPointBtn, addLineBtn,
                   addPolyLineBtn, addPolygonBtn;
    cadEditArgsPtr editArgs;
    toolBarPtr     toolBar;

    toolBar = ( toolBarPtr) malloc( sizeof( toolBarRec)); 
    if ( toolBar == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    label = XtVaCreateManagedWidget( "toolLabel",
                xmLabelWidgetClass, parent,
                StickLeft, StickTop,
                NULL);
    toolBar->label = label;

    form = XtVaCreateManagedWidget( "toolForm", 
               xmFormWidgetClass, parent,
               RightOf( label), StickTop,
               NULL);
    toolBar->toolForm = form;

    zoom2xBtn = XtVaCreateManagedWidget( "zoom2xBtn",
                    xmPushButtonWidgetClass, form,
                    StickLeft, StickTop, StickBottom,
                    NULL);

    zoomRectBtn = XtVaCreateManagedWidget( "zoomRectBtn",
                     xmPushButtonWidgetClass, form,
                     RightOf( zoom2xBtn), StickTop, StickBottom,
                     NULL);

    unzoom2xBtn = XtVaCreateManagedWidget( "unzoom2xBtn",
                      xmPushButtonWidgetClass, form,
                      RightOf( zoomRectBtn), StickTop, StickBottom,
                      NULL);

    unzoomBtn = XtVaCreateManagedWidget( "unzoomBtn",
                    xmPushButtonWidgetClass, form,
                    RightOf( unzoom2xBtn), StickTop, StickBottom,
                    NULL);

    panBtn = XtVaCreateManagedWidget( "panBtn",
                 xmPushButtonWidgetClass, form,
                 RightOf( unzoomBtn), StickTop, StickBottom,
                 NULL);

    refreshBtn = XtVaCreateManagedWidget( "refreshBtn",
                     xmPushButtonWidgetClass, form,
                     RightOf( panBtn), StickTop, StickBottom,
                     NULL);

    selectLayerBtn = XtVaCreateManagedWidget( "selectLayerBtn",
                         xmPushButtonWidgetClass, form,
                         RightOf( refreshBtn), StickTop, StickBottom,
                         NULL);

    deleteLayerBtn = XtVaCreateManagedWidget( "deleteLayerBtn",
                         xmPushButtonWidgetClass, form,
                         RightOf( selectLayerBtn), StickTop, StickBottom,
                         NULL);

    editForm = XtVaCreateManagedWidget( "editForm",
                   xmFormWidgetClass, parent,
                   StickLeft, BelowOf( label), StickBottom, RightWith( label),
                   NULL);
    toolBar->editForm = editForm;

    movePointBtn = XtVaCreateManagedWidget( "movePointBtn",
					    xmPushButtonWidgetClass,
					    editForm,
					    StickLeft,
					    StickTop,
					    StickRight,
					    NULL);
    
    addPointBtn = XtVaCreateManagedWidget( "addPointBtn",
					   xmPushButtonWidgetClass,
					   editForm,
					   StickLeft,
					   BelowOf( movePointBtn),
					   StickRight,
					   NULL);
    
    addLineBtn = XtVaCreateManagedWidget( "addLineBtn",
					  xmPushButtonWidgetClass,
					  editForm,
					  StickLeft,
					  BelowOf( addPointBtn),
					  StickRight,
					  NULL);

    addPolyLineBtn = XtVaCreateManagedWidget( "addPolyLineBtn",
					      xmPushButtonWidgetClass,
					      editForm,
					      StickLeft,
					      BelowOf( addLineBtn),
					      StickRight,
					      NULL);

    addPolygonBtn = XtVaCreateManagedWidget( "addPolygonBtn",
					     xmPushButtonWidgetClass,
					     editForm,
					     StickLeft,
					     BelowOf( addPolyLineBtn),
					     StickRight,
					     NULL);

    AmAddVaCallback( zoom2xBtn, XmNactivateCallback,
                   ( VarargsCallback) AmZoom2xCallback,
                   motifDisplay, overlay, END_LIST);
    
    AmAddVaCallback( zoomRectBtn, XmNactivateCallback,
                   ( VarargsCallback) AmZoomRectCallback,
                   motifDisplay, overlay, END_LIST);

    AmAddVaCallback( unzoom2xBtn, XmNactivateCallback,
                   ( VarargsCallback) AmUnzoomHalfCallback,
                   motifDisplay, overlay, END_LIST);

    AmAddVaCallback( unzoomBtn, XmNactivateCallback,
                   ( VarargsCallback) AmUnzoomCallback,
                   motifDisplay, overlay, END_LIST);

    AmAddVaCallback( panBtn, XmNactivateCallback,
                   ( VarargsCallback) AmPanCallback,
                   motifDisplay, overlay, END_LIST);

    AmAddVaCallback( refreshBtn, XmNactivateCallback,
                   ( VarargsCallback) AmRefreshCallback,
                   motifDisplay, overlay, END_LIST);

    AmAddVaCallback( selectLayerBtn, XmNactivateCallback,
                   ( VarargsCallback) AmSetCurrLayerCallback,
                   motifDisplay, overlay, END_LIST);

    XtAddCallback( deleteLayerBtn, XmNactivateCallback,
                   ( XtCallbackProc) deleteCback,
                   addFileData);


    editArgs = ( cadEditArgsPtr) malloc( sizeof( cadEditArgsRec));
    if ( editArgs == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
    } else {

        editArgs->cadEditForm = editForm;
        editArgs->motifDisplay = motifDisplay;
        editArgs->overlay = overlay;
        editArgs->layer = NULL;

        editArgs->handler = ( XtEventHandler) NULL;
        editArgs->flags = 0;
	editArgs->handlerArgs = NULL;

        AmAddCallback( AM_PKG_ID, AM_SELECT_LAYER,
            ( callbackProc) layerSelectCallback,  editArgs);

        AmAddCallback( AM_PKG_ID, AM_DELETE_LAYER,
            ( callbackProc) layerDeleteCallback,  editArgs);

        AmAddVaCallback( movePointBtn, XmNactivateCallback,
            ( VarargsCallback) SelectPointCallback,
            editArgs, END_LIST);

        AmAddVaCallback( addPointBtn, XmNactivateCallback,
            ( VarargsCallback) AddPointCallback,
            editArgs, END_LIST);

        AmAddVaCallback( addLineBtn, XmNactivateCallback,
            ( VarargsCallback) AddLineCallback,
            editArgs, END_LIST);

        AmAddVaCallback( addPolyLineBtn, XmNactivateCallback,
            ( VarargsCallback) AddPolyLineCallback,
            editArgs, END_LIST);

        AmAddVaCallback( addPolygonBtn, XmNactivateCallback,
            ( VarargsCallback) AddPolygonCallback,
            editArgs, END_LIST);
    }

    XtSetSensitive( editForm, FALSE);

    return( toolBar);
}


/*********************************************************************
 *
 * Function: Widget CadEditor( Widget parent, char *name)
 *
 * Description:
 *
 *********************************************************************/

Widget
CadEditorForm( Widget parent)
{
    Widget               form, menubar,
                         fileMenu, fileCasc, newCasc, openCasc, saveCasc,
                         saveAsCasc, quitCasc,
                         separator,
                         message, mainForm, frame, area;
    AmMotifDisplayClass    *motifDisplay;
    toolBarPtr           toolBar;
    AmOverlayClass         *overlay;
    addFileDataPtr       addFileData;
    Arg                  args[ 20];
    int                  n;

    form = XtVaCreateManagedWidget( "form",
               xmFormWidgetClass, parent,
               NULL); 

    n = AmMakeArgList( args,
            StickLeft, StickTop, StickRight,
            NULL);
    menubar = XmCreateMenuBar( form, "menubar", args, n);
    XtManageChild( menubar);

/* File menu: */

    fileMenu = XmCreatePulldownMenu( menubar, "fileMenu", NULL, 0);

    fileCasc = XtVaCreateManagedWidget( "fileCasc", 
                   xmCascadeButtonWidgetClass, menubar, 
                   XmNsubMenuId, fileMenu,
                   NULL);

    newCasc = XtVaCreateManagedWidget( "newCasc", 
                   xmCascadeButtonWidgetClass, fileMenu, 
                   NULL);

    separator = XtVaCreateManagedWidget( "separator", 
                    xmSeparatorWidgetClass, fileMenu, 
                    NULL);

    openCasc = XtVaCreateManagedWidget( "openCasc", 
                   xmCascadeButtonWidgetClass, fileMenu, 
                   NULL);

    saveCasc = XtVaCreateManagedWidget( "saveCasc", 
                   xmCascadeButtonWidgetClass, fileMenu, 
                   NULL);

    saveAsCasc = XtVaCreateManagedWidget( "saveAsCasc", 
                   xmCascadeButtonWidgetClass, fileMenu, 
                   NULL);

    separator = XtVaCreateManagedWidget( "separator", 
                    xmSeparatorWidgetClass, fileMenu, 
                    NULL);

    quitCasc = XtVaCreateManagedWidget( "quitCasc", 
                   xmCascadeButtonWidgetClass, fileMenu, 
                   NULL);

/* Message area: */

    message = XtVaCreateManagedWidget( "message",
                  xmLabelWidgetClass, form, 
                  StickLeft, StickRight, StickBottom,
                  NULL);

    mainForm = XtVaCreateManagedWidget( "mainForm",
                   xmFormWidgetClass, form,
                   StickLeft, BelowOf( menubar),
                   StickRight, AboveOf( message),
                   NULL); 

    frame = XtVaCreateManagedWidget( "frame",
                xmFrameWidgetClass, mainForm,
                StickRight, StickBottom,
                NULL);

/* Drawing area: */

    area = XtVaCreateManagedWidget( "area",
               xmDrawingAreaWidgetClass, frame, 
               NULL);

/* Create the motif display: */

    motifDisplay = new AmMotifDisplayClass( area, TRUE);
    motifDisplay->SetMessage( message);

    XtRealizeWidget( parent);

    overlay = new AmOverlayClass( motifDisplay->GetDisplay(), "overlay",
                      AmNpreserveAspect, TRUE,
                      AmNx, 0,
                      AmNy, 0,
                      AmNbottomMargin, 0,
                      AmNleftMargin, 0,
                      AmNrightMargin, 0,
                      AmNtopMargin, 0,
                      NULL);

    motifDisplay->AddObject( overlay);

/* Add file data client data. */

    addFileData = ( addFileDataPtr) malloc( sizeof( addFileDataRec));

    addFileData->motifDisplay = motifDisplay;
    addFileData->overlay = overlay;
    addFileData->addLayerClientData = NULL;
    addFileData->addLayer = ( AmAddLayerFn) DefaultAddLayerFromHeader;
    addFileData->addLayerWrapper = ( AmAddLayerFn) cadAddLayerWrapper;
    addFileData->addLayerWrapperData = NULL; 

/* Create the utilities. */

    toolBar = ToolBarForm( mainForm, motifDisplay, overlay, addFileData);

    XtVaSetValues( frame,
        BelowOf( toolBar->toolForm), RightOf( toolBar->editForm),
        NULL);

/* Callbacks for the menu buttons. */

    XtAddCallback( openCasc, XmNactivateCallback,
        ( XtCallbackProc) openCback, addFileData);

    XtAddCallback( quitCasc, XmNactivateCallback, 
        ( XtCallbackProc) quitCback, motifDisplay);

/* Realize the widgets and go... */

    motifDisplay->UnmapTravelCursor();
    return( form);
}
