#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <X11/cursorfont.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/imageplot.h>
#include <anvil/imagelegend.h>
#include <anvil/lineplot.h>
#include <anvil/polygonplot.h>
#include <anvil/scatterplot.h>
#include <anvil/tick.h>
#include <anvil/plot_cback.h>
#include <anvil/plot_cbackP.h>
#include <anvil/motifdisplay.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/userevents.h>
#include <anvil/slick.h>
#include <anvil/plot_cback.h>
#include <anvil/events.h>
#include <anvil/outline.h>
#include <anvil/defaults.h>

#include <editor/handlers.h>

const int AmACTION_MASTER_PRIORITY = 6000;
const int AmACTION_RESIZE_PRIORITY = 1000;
const int AmACTION_MOVE_PRIORITY = 1000;
const int AmACTION_ADD_PRIORITY = 1000;
const int AmACTION_PICK_PRIORITY = 1000;
const int AmACTION_SECONDARY_PRIORITY = 1000;

const int MIN_SIZE = 5;

void AmPrimaryTimeout( AmSecondaryHandlerArgsPtr, XtPointer);
void AmSecondaryTimeout( AmSecondaryHandlerArgsPtr, XtPointer);

AmScreenObjClass *
findObjectFromList( AmListPtr objects, int x, int y)
{
    AmScreenObjClass     *temp, *screenObj;
    const AmRectangleRec *rect;
   
    screenObj = NULL; 
    for ( ; objects != NULL; objects = objects->next) {
        temp = ( AmScreenObjClass *)objects->data;
        rect = temp->GetRect();

    /* Check to see if the mouse is within the extends of this object. */ 

        if ( x >= rect->x && x <= rect->x + rect->width &&
             y >= rect->y && y <= rect->y + rect->height) {
            screenObj = temp;
            break;
        }
    }

    return( screenObj);
}

/*********************************************************************
 *
 * void 
 * findSurroundingRect( AmListPtr objs, AmRectangleRec *rec)
 *
 * Description:
 *     Figures out the rectangle that encompasses all the objects. 
 *
 * Note:
 *     This function sues the GetRect function in the objects and so
 *     doesn't really get the entire extent of each screen object.
 *
 *********************************************************************/

void
findExtents( AmListPtr      objs,
             AmRectanglePtr rec)
{
    AmScreenObjClass     *screenObj;
    const AmRectangleRec *rect;
    int                x1, y1, x2, y2;

    if ( objs != NULL) {
        screenObj = ( AmScreenObjClass *) objs->data;
        objs = objs->next;

        rect = screenObj->GetRect();
        x1 = rect->x;
        y1 = rect->y;
        x2 = rect->x + rect->width;
        y2 = rect->y + rect->height;
    } else {
        x1 = 0;
        y1 = 0;
        x2 = 0;
        y2 = 0;
    }

    while ( objs != NULL) {
        screenObj = ( AmScreenObjClass *) objs->data;
        objs = objs->next;

        rect = screenObj->GetRect();

        if ( (rect->x + rect->width) > x2) {
            x2 = rect->x + rect->width;
        }
        if ( (rect->y + rect->height) > y2) {
            y2 = rect->y + rect->height;
        }

        if ( rect->x < x1) {
            x1 = rect->x;
        }
        if ( rect->y < y1) {
            y1 = rect->y;
        }
    }

    rec->x = x1;
    rec->y = y1;
    rec->width = x2 - x1;
    rec->height = y2 - y1;
}


/*********************************************************************
 *
 * void 
 * findRealExtents( AmListPtr objs, AmRectangleRec *rec)
 *
 * Description:
 *     Figures out the rectangle that encompasses all the objects. 
 *
 *********************************************************************/

void
findRealExtents( AmListPtr      objs,
                 AmRectanglePtr rec)
{
    AmScreenObjClass     *screenObj;
    const AmRectangleRec *rect;
    int                x1, y1, x2, y2;

    if ( objs != NULL) {
        screenObj = ( AmScreenObjClass *) objs->data;
        objs = objs->next;

        rect = screenObj->GetExtentsRect();
        x1 = rect->x;
        y1 = rect->y;
        x2 = rect->x + rect->width;
        y2 = rect->y + rect->height;
    } else {
        x1 = 0;
        y1 = 0;
        x2 = 0;
        y2 = 0;
    }

    while ( objs != NULL) {
        screenObj = ( AmScreenObjClass *) objs->data;
        objs = objs->next;

        rect = screenObj->GetExtentsRect();

        if ( (rect->x + rect->width) > x2) {
            x2 = rect->x + rect->width;
        }
        if ( (rect->y + rect->height) > y2) {
            y2 = rect->y + rect->height;
        }

        if ( rect->x < x1) {
            x1 = rect->x;
        }
        if ( rect->y < y1) {
            y1 = rect->y;
        }
    }

    rec->x = x1;
    rec->y = y1;
    rec->width = x2 - x1;
    rec->height = y2 - y1;
}


void
updateMoveObjects( AmListPtr screenObjs,
                   int startX,
                   int startY,
                   int x,
                   int y,
                   int areaWidth,
                   int areaHeight)
{
    AmScreenObjClass     *screenObj;
    const AmRectangleRec *rect;
    AmListPtr            recs, temp;

    recs = NULL;
    for ( temp = screenObjs; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();
        screenObj->SetRectAndMargins( rect->x - startX + x,
                                      rect->y - startY + y,
                                      rect->width, rect->height,
                                      areaWidth, areaHeight);
    }
}


AmListPtr
AmGenerateMoveOutlines( AmListPtr screenObjs,
                      int startX,
                      int startY,
                      int x,
                      int y)
{
    AmScreenObjClass *screenObj;
    const AmRectangleRec *rect;
    AmRectanglePtr       newRect;
    AmListPtr            recs, temp;

    recs = NULL;
    for ( temp = screenObjs; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();

        newRect = ( AmRectanglePtr) malloc( sizeof( AmRectangleRec));
        newRect->x = rect->x - startX + x;
        newRect->y = rect->y - startY + y;
        newRect->width = rect->width;
        newRect->height = rect->height;

        recs = AmInsertNodeLst( newRect, recs); 
    }

    return( recs);
}


void
updateResizeObjects( AmListPtr    screenObjs,
                     int        startX,
                     int        startY,
                     int        startWidth,
                     int        startHeight,
                     int        x,
                     int        y,
                     int        width,
                     int        height,
                     int        areaWidth,
                     int        areaHeight)
{
    AmScreenObjClass     *screenObj;
    const AmRectangleRec *rect;
    AmListPtr            recs, temp;
    float              scaleX, scaleY;

    scaleX = ( float)width / ( float)startWidth;
    scaleY = ( float)height / ( float)startHeight;

    recs = NULL;
    for ( temp = screenObjs; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();
        screenObj->SetRectAndMargins(
        	( int) ( x + ((rect->x - startX)*scaleX + 0.5)),
		( int) ( y + ((rect->y - startY)*scaleY + 0.5)),
		( int) ( rect->width*scaleX + 0.5),
		( int) ( rect->height*scaleY + 0.5),
		 areaWidth, areaHeight);
    }
}


AmListPtr
AmGenerateResizeOutlines( AmListPtr screenObjs,
                        int startX,
                        int startY,
                        int startWidth,
                        int startHeight,
                        int x,
                        int y,
                        int width,
                        int height)
{
    AmScreenObjClass *screenObj;
    const AmRectangleRec *rect;
    AmRectanglePtr       newRect;
    AmListPtr            recs, temp;
    float              scaleX, scaleY;

    scaleX = ( float)width / ( float)startWidth;
    scaleY = ( float)height / ( float)startHeight;

    recs = NULL;
    for ( temp = screenObjs; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();

        newRect = ( AmRectanglePtr) malloc( sizeof( AmRectangleRec));
        newRect->x = ( int)( x + (rect->x - startX)*scaleX + 0.5);
        newRect->y = ( int)( y + (rect->y - startY)*scaleY + 0.5);
        newRect->width = ( int)( rect->width*scaleX + 0.5);
        newRect->height = ( int)( rect->height*scaleY + 0.5);

        recs = AmInsertNodeLst( newRect, recs); 
    }

    return( recs);
}


void
updateAddObjects( AmMotifDisplayClass *motifDisplay,
                  AmListPtr           screenObjs,
                  int               startX,
                  int               startY,
                  int               x,
                  int               y,
                  int               areaWidth,
                  int               areaHeight)
{
    AmXWindowClass       *window = motifDisplay->GetWindow();
    AmScreenObjClass     *screenObj;
    const AmRectangleRec *rect;
    AmListPtr            temp;

    for ( temp = screenObjs; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();
        screenObj->SetRectAndMargins( x + rect->x - startX,
				      y + rect->y - startY,
				      rect->width, rect->height,
				      areaWidth, areaHeight);
        motifDisplay->AddObject( screenObj);
    }
    motifDisplay->Flush();
}


AmListPtr
generateAddOutlines( AmListPtr screenObjs,
                     int     startX,
                     int     startY,
                     int     x,
                     int     y)
{
    AmScreenObjClass *screenObj;
    const AmRectangleRec *rect;
    AmRectanglePtr       newRect;
    AmListPtr            temp, recs = NULL;

    for ( temp = screenObjs; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();

        newRect = ( AmRectanglePtr) malloc( sizeof( AmRectangleRec));
        newRect->x = x + (rect->x - startX);
        newRect->y = y + (rect->y - startY);
        newRect->width = rect->width;
        newRect->height = rect->height;

        recs = AmInsertNodeLst( newRect, recs); 
    }

    return( recs);
}


void
activateObjects( AmMotifDisplayClass *motifDisplay,
                 AmListPtr           *activeObjects,
                 int               x0,
                 int               y0,
                 int               x1,
                 int               y1)
{
    AmScreenObjClass     **screenObjs;
    const AmRectangleRec *rect;
    AnvilBoolean            success;
    int                minX, minY, maxX, maxY, ii;

    if ( x0 < x1) {
        minX = x0;
        maxX = x1;
    } else {
        minX = x1;
        maxX = x0;
    }

    if ( y0 < y1) {
        minY = y0;
        maxY = y1;
    } else {
        minY = y1;
        maxY = y0;
    }

    success = FALSE;
    screenObjs = motifDisplay->GetObjects();

    for ( ii = 0; screenObjs[ ii] != NULL; ii++) {
        rect = screenObjs[ ii]->GetRect();
        if ( rect->x >= minX && rect->y >= minY &&
            rect->x + rect->width <= maxX &&
            rect->y + rect->height <= maxY) {

            if ( AmFindElLst( screenObjs[ ii], *activeObjects,
                     ( AmSearchFn) AmEquals) != NULL) {
                *activeObjects = AmDelPtrLst( screenObjs[ ii],
                                     *activeObjects,
                                     ( AmSearchFn) AmEquals);
            } else {
                *activeObjects = AmInsertNodeLst( screenObjs[ ii],
                                     *activeObjects);
            }

            success = TRUE;
        }
    }

    if ( success == TRUE) {
        motifDisplay->WriteResourceMessage( "outlineSuccessMsg");
    } else {
        motifDisplay->WriteResourceMessage( "outlineFailureMsg");
    }

    free( screenObjs);
}



AmListPtr
generateOutlines( AmListPtr objects)
{
    AmScreenObjClass     *screenObj;
    AmListPtr            recs, temp;
    const AmRectangleRec *oldRect;
    AmRectanglePtr       newRect;

    recs = NULL;
    for ( temp = objects; temp != NULL; temp = temp->next) {
        screenObj = ( AmScreenObjClass *) temp->data;

        oldRect = screenObj->GetRect();
        newRect = ( AmRectanglePtr) malloc( sizeof( AmRectangleRec));
        newRect->x = oldRect->x;
        newRect->y = oldRect->y;
        newRect->width = oldRect->width;
        newRect->height = oldRect->height;

        recs = AmInsertNodeLst( newRect, recs);
    }

    return( recs);
}


AmListPtr
generateOutlineOutlines( AmListPtr activeObjects, int x0, int y0, int x1, int y1)
{
    AmRectanglePtr rect;
    AmListPtr      recs;
    int          x, y, width, height;

    if ( x0 < x1) {
        x = x0;
        width = x1 - x0;
    } else {
        x = x1;
        width = x0 - x1;
    }

    if ( y0 < y1) {
        y = y0;
        height = y1 - y0;
    } else {
        y = y1;
        height = y0 - y1;
    }

    rect = ( AmRectanglePtr) malloc( sizeof( AmRectangleRec));
    rect->x = x;
    rect->y = y;
    rect->width = width;
    rect->height = height;
   
    recs = generateOutlines( activeObjects);
    recs = AmInsertNodeLst( rect, recs);

    return( recs);
}


/*********************************************************************
 *
 * actionToCurosr
 *
 * Description:
 *     Figures out the cursor that is to be displayed according to 
 *     the action given.
 *
 *********************************************************************/

int
actionToCursor( AmActionEnum action)
{
    int cursor;

    switch( action) {
        case AmACTION_RESIZE_UP_LEFT:
            cursor = XC_top_left_corner;
            break;
        case AmACTION_RESIZE_UP:
            cursor = XC_top_side;
            break;
        case AmACTION_RESIZE_UP_RIGHT:
            cursor = XC_top_right_corner;
            break;
        case AmACTION_RESIZE_LEFT:
            cursor = XC_left_side;
            break;
        case AmACTION_RESIZE_RIGHT:
            cursor = XC_right_side;
            break;
        case AmACTION_RESIZE_DOWN_LEFT:
            cursor = XC_bottom_left_corner;
            break;
        case AmACTION_RESIZE_DOWN:
            cursor = XC_bottom_side;
            break;
        case AmACTION_RESIZE_DOWN_RIGHT:
            cursor = XC_bottom_right_corner;
            break;
        case AmACTION_MOVE:
            cursor = XC_tcross;
            break;
        case AmACTION_DELETE:
            cursor = XC_pirate;
            break;
        case AmACTION_RAISE:
        case AmACTION_LOWER:
            cursor = XC_dot;
            break;
        case AmACTION_PICK:
            cursor = XC_fleur;
            break;
        case AmACTION_NONE:
        default:
            cursor = XC_arrow;
            break;
    }

    return( cursor);
}


/*********************************************************************
 *
 * findAction
 *
 * Description:
 *     Figures out the action that is to be performed on an object
 *     depening on the screen coordinate.
 *
 *********************************************************************/

AmActionEnum
findAction( AmListPtr objectLst,
            int     x,
            int     y)
{
    AmScreenObjClass     *screenObj;
    AmListPtr            temp;
    const AmRectangleRec *rect;
    int                relX, relY, relW, relH, relMW, relMH;
    AmActionEnum         action;
   
/* Initializations. */

    action = AmACTION_NONE;

    for ( temp = objectLst; temp != NULL; temp = temp->next) { 
        screenObj = ( AmScreenObjClass *) temp->data;
        rect = screenObj->GetRect();

        relX = x - rect->x;
        relY = y - rect->y;
        relW = relX - rect->width;
        relH = relY - rect->height;
        relMW = relX - rect->width/2;
        relMH = relY - rect->height/2;

/* Check to see if the mouse is within the extends of this object. */ 
 
        if ( relX >= 0 && relW <= 0 && relY >= 0 && relH <= 0) { 

        /* Check the resize areas in the left hand side. */

            if ( relX >= 0 && relX <= AmANCHOR_SIZE) {
                if ( relY >= 0 && relY <= AmANCHOR_SIZE) {
                    return( AmACTION_RESIZE_UP_LEFT);
                } else if ( relH >= (- AmANCHOR_SIZE) && relH <= 0) {
                    return( AmACTION_RESIZE_DOWN_LEFT);
                } else if ( relMH >= (- AmANCHOR_SIZE/2) &&
                            relMH <= AmANCHOR_SIZE/2) {
                    return( AmACTION_RESIZE_LEFT);
                }

        /* Check the resize areas in the right hand side. */

            } else if ( relW >= (- AmANCHOR_SIZE) && relW <= 0) {
                if ( relY >= 0 && relY <= AmANCHOR_SIZE) {
                    return( AmACTION_RESIZE_UP_RIGHT);
                } else if ( relH >= (- AmANCHOR_SIZE) && relH <= 0) {
                    return( AmACTION_RESIZE_DOWN_RIGHT);
                } else if ( relMH >= (- AmANCHOR_SIZE/2) &&
                    relMH <= AmANCHOR_SIZE/2) {
                    return( AmACTION_RESIZE_RIGHT);
                }

        /* Check the resize areas in the middle. */

            } else if ( relMW >= (- AmANCHOR_SIZE/2) && relMW <= AmANCHOR_SIZE/2) {
                if ( relY >= 0 && relY <= AmANCHOR_SIZE) {
                    return( AmACTION_RESIZE_UP);
                } else if ( relH >= (- AmANCHOR_SIZE) && relH <= 0) {
                    return( AmACTION_RESIZE_DOWN);
                }
            }

        /* It's not a resize so it's a move. */

            action = AmACTION_MOVE;
        }
    }

    return( action);
}


/*********************************************************************
 *
 * makeMasterHandlerArgs
 *
 * Description:
 *     Creates the arguments needed by the master handler.
 *
 *********************************************************************/

AmMasterHandlerArgsPtr
makeMasterHandlerArgs( AmMotifDisplayClass *motifDisplay,
                       AmHandlerFn         doubleClickFn)
{
    AmMasterHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmMasterHandlerArgsPtr)malloc( sizeof( AmMasterHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->activeObjects = NULL;
    handlerArgs->doubleClickFn = doubleClickFn;

    return( handlerArgs);
}


/*********************************************************************
 *
 * makeMasterHandlerArgs
 *
 * Description:
 *     Creates the arguments needed by the master handler.
 *
 *********************************************************************/

AmResizeHandlerArgsPtr
AmMakeResizeHandlerArgs( AmMotifDisplayClass *motifDisplay,
                       AmListPtr           screenObjs,
                       AmActionEnum        actionType,
                       int               x,
                       int               y,
                       int               width,
                       int               height,
                       int               xOffset,
                       int               yOffset)
{
    AmResizeHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmResizeHandlerArgsPtr)malloc( sizeof( AmResizeHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->screenObjs = screenObjs;
    handlerArgs->actionType = actionType;
    handlerArgs->startX = x;
    handlerArgs->startY = y;
    handlerArgs->startWidth = width;
    handlerArgs->startHeight = height;
    handlerArgs->x = x;
    handlerArgs->y = y;
    handlerArgs->width = width;
    handlerArgs->height = height;
    handlerArgs->xOffset = xOffset;
    handlerArgs->yOffset = yOffset;

    return( handlerArgs);
}

void
freeResizeHandlerArgs( AmResizeHandlerArgsPtr handlerArgs)
{
    free( handlerArgs);
}


void
AmRemoveResizeHandler( AmResizeHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->UnsetOutline();
    motifDisplay->UnmapTravelCursor();

    freeResizeHandlerArgs( handlerArgs);
}


void
suspendResizeHandler( AmResizeHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;  
}


void
restartResizeHandler( AmResizeHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->WriteResourceMessage( "restartResizeHandler");
}

/*********************************************************************
 *
 * AmMakeOutlineHandlerArgs
 *
 * Description:
 *     Creates the arguments needed by the outline handler.
 *
 *********************************************************************/

outlineHandlerArgsPtr
AmMakeOutlineHandlerArgs( AmMotifDisplayClass *motifDisplay,
                        AmListPtr           *activeObjects,
                        int               x,
                        int               y)
{
    outlineHandlerArgsPtr handlerArgs;

    handlerArgs = ( outlineHandlerArgsPtr)malloc( 
                                              sizeof( outlineHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->activeObjects = activeObjects;
    handlerArgs->x = x;
    handlerArgs->y = y;

    return( handlerArgs);
}

void
freeOutlineHandlerArgs( outlineHandlerArgsPtr handlerArgs)
{
    free( handlerArgs);
}


void
AmRemoveOutlineHandler( outlineHandlerArgsPtr handlerArgs)
{
    freeOutlineHandlerArgs( handlerArgs);
}


void
suspendOutlineHandler( outlineHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;  
}


void
restartOutlineHandler( AmAddHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->WriteResourceMessage( "restartOutlineHandler");
}


AmMoveHandlerArgsPtr
AmMakeMoveHandlerArgs( AmMotifDisplayClass *motifDisplay,
                     AmListPtr           screenObjs,
                     int               x,
                     int               y,
                     int               width,
                     int               height,
                     int               xOffset,
                     int               yOffset)
{
    AmMoveHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmMoveHandlerArgsPtr)malloc( sizeof( AmMoveHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->screenObjs = screenObjs;
    handlerArgs->startX = x;
    handlerArgs->startY = y;
    handlerArgs->x = x;
    handlerArgs->y = y;
    handlerArgs->width = width;
    handlerArgs->height = height;
    handlerArgs->xOffset = xOffset;
    handlerArgs->yOffset = yOffset;

    return( handlerArgs);
}

void
freeMoveHandlerArgs( AmMoveHandlerArgsPtr handlerArgs)
{
    free( handlerArgs);
}


void
AmRemoveMoveHandler( AmMoveHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->UnsetOutline();
    motifDisplay->UnmapTravelCursor();

    freeMoveHandlerArgs( handlerArgs);
}


void
AmSuspendMoveHandler( AmMoveHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;  
}


void
AmRestartMoveHandler( AmMoveHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->WriteResourceMessage( "AmRestartMoveHandler");
}

AmPickHandlerArgsPtr
makePickHandlerArgs( AmMotifDisplayClass *motifDisplay,
                     AmHandlerFn         objectFunction, 
                     void              *clientData,
                     AmObjectTypeEnum    objectType,
                     AnvilBoolean           traverseOverlay,
                     char              *successMessage,
                     char              *cancelMessage,
                     char              *failureMessage)
{
    AmPickHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmPickHandlerArgsPtr)malloc( sizeof( AmPickHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->objectFunction = objectFunction;
    handlerArgs->clientData = clientData;
    handlerArgs->objectType = objectType;
    handlerArgs->traverseOverlay = traverseOverlay;

    if ( successMessage) {
        handlerArgs->successMessage = successMessage;
    } else {
        handlerArgs->successMessage = "successMsg";
    }

    if ( failureMessage) {
        handlerArgs->failureMessage = failureMessage;
    } else {
        handlerArgs->failureMessage = "failureMsg";
    }

    if ( cancelMessage) {
        handlerArgs->cancelMessage = cancelMessage;
    } else {
        handlerArgs->cancelMessage = "cancelMsg";
    }

    return( handlerArgs);
}

void
freePickHandlerArgs( AmPickHandlerArgsPtr handlerArgs)
{
    free( handlerArgs);
}


void
removePickHandler( AmPickHandlerArgsPtr handlerArgs)
{
    freePickHandlerArgs( handlerArgs);
}


void
suspendPickHandler( AmPickHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;  
}


void
restartPickHandler( AmPickHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->WriteResourceMessage( "restartPickHandler");
}


AmAddHandlerArgsPtr
AmMakeAddHandlerArgs( AmMotifDisplayClass *motifDisplay,
                    AmScreenObjClass    *screenObj,
                    int               x,
                    int               y,
                    int               width,
                    int               height,
                    int               xOffset,
                    int               yOffset,
                    AmListPtr           *activeObjects)
{
    AmAddHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmAddHandlerArgsPtr)malloc( sizeof( AmAddHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->screenObj = screenObj;
    handlerArgs->x = x;
    handlerArgs->y = y;
    handlerArgs->width = width;
    handlerArgs->height = height;
    handlerArgs->xOffset = xOffset;
    handlerArgs->yOffset = yOffset;
    handlerArgs->activeObjects = activeObjects;
    handlerArgs->success = FALSE;

    return( handlerArgs);
}


void
freeAddHandlerArgs( AmAddHandlerArgsPtr handlerArgs)
{
    AmScreenObjClass *screenObj;

    screenObj = handlerArgs->screenObj;

    if ( handlerArgs->success == FALSE) {
        if ( screenObj != NULL) {
            delete screenObj;
        }
    }

    free( handlerArgs);
}


void
AmRemoveAddHandler( AmAddHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmListPtr           *activeObjs = handlerArgs->activeObjects;

    motifDisplay->UnsetOutline();
    motifDisplay->UnmapTravelCursor();

    if ( activeObjs != NULL) {
        if ( *activeObjs != NULL) {
            AmJustFreeLst( *activeObjs);
            *activeObjs = NULL;
        }

        if ( handlerArgs->success == TRUE && handlerArgs->screenObj != NULL) {
            *activeObjs = AmInsertNodeLst( handlerArgs->screenObj, *activeObjs);
        }
    }

    freeAddHandlerArgs( handlerArgs);
}


void
AmSuspendAddHandler( AmAddHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->UnsetOutline();
    motifDisplay->UnmapTravelCursor();
}


void
AmRestartAddHandler( AmAddHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->MapTravelCursor();
}


void
AmSetupAddHandler( AmMotifDisplayClass *motifDisplay,
                 AmScreenObjClass    *screenObj,
                 AmListPtr           *activeObjects)
{
    AmAddHandlerArgsPtr  handlerArgs;
    const AmRectangleRec *rect;

    rect = screenObj->GetRect();

    handlerArgs = AmMakeAddHandlerArgs( motifDisplay, screenObj,
                      0, 0, rect->width, rect->height,
                      rect->width/2, rect->height/2,
                      activeObjects);

    AmAddHandler( NULL, motifDisplay->GetArea(), 
        ButtonReleaseMask | ButtonPressMask | PointerMotionMask,
        ( XtEventHandler) AmAddHandler,
        handlerArgs, AmACTION_ADD_PRIORITY,
        ( AmStateHandlerFn) AmRemoveAddHandler,
        ( AmStateHandlerFn) AmSuspendAddHandler,
        ( AmStateHandlerFn) AmRestartAddHandler);

    motifDisplay->SetOutline( 0, 0, rect->width, rect->height, handlerOutline);
    motifDisplay->MapTravelCursor();
    motifDisplay->WriteResourceTravelCursor( "addingMsg", 0, 0);
    motifDisplay->SetCursor( actionToCursor( AmACTION_PICK));
}

AmAddObjectsHandlerArgsPtr
AmMakeAddObjectsHandlerArgs( AmMotifDisplayClass *motifDisplay,
                           AmListPtr           *objects,
                           int               startX,
                           int               startY,
                           int               x,
                           int               y,
                           int               width,
                           int               height,
                           int               xOffset,
                           int               yOffset)
{
    AmAddObjectsHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmAddObjectsHandlerArgsPtr)malloc(
                      sizeof( AmAddObjectsHandlerArgsRec));
    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->screenObjs = objects;
    handlerArgs->startX = startX;
    handlerArgs->startY = startY;
    handlerArgs->x = x;
    handlerArgs->y = y;
    handlerArgs->width = width;
    handlerArgs->height = height;
    handlerArgs->xOffset = xOffset;
    handlerArgs->yOffset = yOffset;
    handlerArgs->success = FALSE;

    return( handlerArgs);
}


void
freeAddObjectsHandlerArgs( AmAddObjectsHandlerArgsPtr handlerArgs)
{
    AmScreenObjClass *screenObj;
    AmListPtr        temp;

    if ( handlerArgs->success == FALSE) {

        if ( handlerArgs->screenObjs && *( handlerArgs->screenObjs)) {
            for ( temp = *( handlerArgs->screenObjs);
                  temp != NULL;
                  temp = temp->next) {
                screenObj = ( AmScreenObjClass *)temp->data;
                delete screenObj;
            }

            AmJustFreeLst( *( handlerArgs->screenObjs));
            *( handlerArgs->screenObjs) = NULL;
        }

    }

    free( handlerArgs);
}


void
AmRemoveAddObjectsHandler( AmAddObjectsHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;

    motifDisplay->UnsetOutline();
    motifDisplay->UnmapTravelCursor();

    freeAddObjectsHandlerArgs( handlerArgs);
}


/*********************************************************************
 *
 * AmMakeSecondaryHandlerArgs
 *
 * Description:
 *     Creates the arguments needed by the secondary master handler.
 *
 *********************************************************************/

AmSecondaryHandlerArgsPtr
AmMakeSecondaryHandlerArgs( AmMotifDisplayClass *motifDisplay,
                          AmScreenObjClass    *screenObj,
                          AmListPtr           screenObjs,
                          AmActionEnum        actionType,
                          int               x,
                          int               y,
                          AmHandlerFn         doubleClickFn)
{
    AmSecondaryHandlerArgsPtr handlerArgs;

    handlerArgs = ( AmSecondaryHandlerArgsPtr)
                      malloc( sizeof( AmSecondaryHandlerArgsRec));

    handlerArgs->motifDisplay = motifDisplay;
    handlerArgs->screenObj = screenObj;
    handlerArgs->screenObjs = screenObjs;
    handlerArgs->actionType = actionType;
    handlerArgs->x = x;
    handlerArgs->y = y;
    handlerArgs->doubleClickFn = doubleClickFn;

    return( handlerArgs);
}


void
freeSecondaryHandlerArgs( AmSecondaryHandlerArgsPtr handlerArgs)
{
    free( handlerArgs);
}


void
AmRemoveSecondaryHandler( AmSecondaryHandlerArgsPtr handlerArgs)
{
    freeSecondaryHandlerArgs( handlerArgs);
}



void
AmAddHandler( Widget            widget,
            AmAddHandlerArgsPtr handlerArgs,
            XEvent            *event,
            AnvilBoolean           *)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    AmScreenObjClass     *screenObj = handlerArgs->screenObj;
    XButtonEvent       *buttonEvent;
    XMotionEvent       *motionEvent;
    const AmRectangleRec *rect;
    Dimension          areaWidth, areaHeight;
    AmListPtr            *activeObjects;
    int                x = handlerArgs->x,
                       y = handlerArgs->y,
                       w = handlerArgs->width,
                       h = handlerArgs->height,
                       xOffset = handlerArgs->xOffset,
                       yOffset = handlerArgs->yOffset;

/* Figure out the extends of the drawing area. */

    XtVaGetValues( widget,
        XmNwidth, &areaWidth, XmNheight, &areaHeight, NULL); 

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "addCancelMsg");

        /* Remove the handler. */

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmAddHandler, handlerArgs);

        /* Generate a fake event to reset the outline. */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        } else if ( buttonEvent->button == Button1) {

        /* Display a busy cursor and remove the outline and the handler. */

            motifDisplay->SetCursor( XC_watch);

            handlerArgs->success = TRUE;

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmAddHandler, handlerArgs);

        /* Add the object before setting the rectangle and margins, 
	   otherwise the event list will ignore the object because it's
	   not in the motifdisplay's object list */

            motifDisplay->AddObject( screenObj);
            screenObj->SetRectAndMargins( x, y, w, h, areaWidth, areaHeight);
            rect = screenObj->GetRect();
	    motifDisplay->Flush();

        /* Give a status message. */

            motifDisplay->WriteResourceMessage( "addSuccessMsg");

        /* Generate a fake event to reset the cursor. */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);

        }
    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *)event;

    /* Figure out the extends of the rectangle. */

        x = motionEvent->x - xOffset;

        if ( x < 0) {
            x = 0;
        } else if ( x + w > areaWidth) {
            x = areaWidth - w;
        }

        y = motionEvent->y - yOffset;
        if ( y < 0) {
            y = 0;
        } else if ( y + h > areaHeight) {
            y = areaHeight - h;
        }

    /* Remove the old outline and draw the new one. */

        motifDisplay->UpdateOutline( x, y, w, h);

        handlerArgs->x = x;
        handlerArgs->y = y;
        handlerArgs->width = w;
        handlerArgs->height = h;

        motifDisplay->WriteResourceTravelCursor( "addingMsg", x, y);
    }
}


void
AmAddObjectsHandler( Widget                   widget,
                   AmAddObjectsHandlerArgsPtr handlerArgs,
                   XEvent                   *event,
                   AnvilBoolean                  *)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    AmListPtr            temp, *objects = handlerArgs->screenObjs;
    XButtonEvent       *buttonEvent;
    XMotionEvent       *motionEvent;
    Dimension          areaWidth, areaHeight;
    int                startX = handlerArgs->startX,
                       startY = handlerArgs->startY,
                       x = handlerArgs->x,
                       y = handlerArgs->y,
                       w = handlerArgs->width,
                       h = handlerArgs->height,
                       xOffset = handlerArgs->xOffset,
                       yOffset = handlerArgs->yOffset;

/* Figure out the extends of the drawing area. */

    XtVaGetValues( widget,
        XmNwidth, &areaWidth, XmNheight, &areaHeight, NULL); 

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "addCancelMsg");

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmAddObjectsHandler, handlerArgs);

        /* Generate a fake event to reset the outline. */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        } else if ( buttonEvent->button == Button1) {

        /* Display a busy cursor and remove the outline and the handler. */

            motifDisplay->SetCursor( XC_watch);

            handlerArgs->success = TRUE;
            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmAddObjectsHandler, handlerArgs);

        /* Update the relevant areas on the screen. */

            updateAddObjects( motifDisplay, *objects, startX, startY, x, y,
			      areaWidth, areaHeight); 
            
        /* Give a status message. */

            motifDisplay->WriteResourceMessage( "addSuccessMsg");

        /* Generate a fake event to reset the cursor. */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);

        }
    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *)event;

    /* Figure out the extends of the rectangle. */

        x = motionEvent->x - xOffset;
        if ( x + w > areaWidth) {
            x = areaWidth - w;
        }

        if ( x < 0) {
            x = 0;
        }

        y = motionEvent->y - yOffset;
        if ( y + h > areaHeight) {
            y = areaHeight - h;
        }

        if ( y < 0) {
            y = 0;
        }

    /* Remove the old outline and draw the new one. */

        motifDisplay->WriteResourceTravelCursor( "addingMsg", x, y);

        temp = generateAddOutlines( *objects, startX, startY, x, y);
        motifDisplay->UpdateOutlines( temp);
        AmFreeLst( temp, ( AmItemFn) free);

        handlerArgs->x = x;
        handlerArgs->y = y;
    }
}

void
pickLayerOrOverlayOkCallback( Widget             btn,
			      Widget             list,
			      AmOverlayClass       *overlay,
                              AmPickHandlerArgsPtr handlerArgs,
                              XtPointer)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmDataGraphicClass  **layerList, *layer;
    int               *posList, count, numLayers;

/* Get the index from the list and blow away the dialog. */

    if ( !XmListGetSelectedPos( list, &posList, &count)) {
        AmPrintResourceWarn( "noItemSelected");
        return;
    }

    XtDestroyWidget( AmGetWidgetShell( btn));

/* Get the list of plots. */

    if ( overlay->GetLayers( &layerList, &numLayers) != AmOK) {
        AmPrintResourceError( "failedToGetLayerList");
        motifDisplay->WriteResourceMessage( handlerArgs->failureMessage);

    /* Free allocated resources. */

        free( posList);
        free( handlerArgs);

        return;
    }

/* The AmLast item in the list is the overlayPlot */

    if ( posList[ 0] <= numLayers) {
        layer = layerList[ posList[ 0] - 1];
    } else {
        layer = overlay;
    }

    if ( handlerArgs->objectFunction( layer, handlerArgs->clientData) == TRUE) {
        motifDisplay->WriteResourceMessage( handlerArgs->successMessage);
    } else {
        motifDisplay->WriteResourceMessage( handlerArgs->failureMessage);
    }

/* Free allocated resources. */

    free( posList);
    free( layerList);
    free( handlerArgs);
}

void
pickLayerOrOverlayCancelCallback( Widget             ,
				  AmOverlayClass       *,
				  AmPickHandlerArgsPtr handlerArgs,
                                  XtPointer)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;

/* Canceled the operation. */

    motifDisplay->WriteResourceMessage( handlerArgs->cancelMessage);

/* Remove allocated resources. */

    free( handlerArgs);
}

void
pickLayerOrOverlay( Widget             ,
                    AmOverlayClass       *overlay,
                    AmPickHandlerArgsPtr handlerArgs)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    XmString           *mstrs;
    Widget             formDlog, shell, list, okBtn, cancelBtn;
    Arg                args[ 20];
    char               **titleList;
    int                n, numItems;

/* Get the list of the titles/filenames/names. */

    titleList = AmLayerStringList( overlay, TRUE);

    if ( titleList == NULL) {
        if (handlerArgs->objectFunction( overlay,
                             handlerArgs->clientData) == TRUE) {
            motifDisplay->WriteResourceMessage( handlerArgs->successMessage);
        } else {
            motifDisplay->WriteResourceMessage( handlerArgs->failureMessage);
        }

    /* Remove allocated resources. */

        free( handlerArgs);
        return;
    }

/* Create the form dialog. */

    shell = AmGetWidgetShell( motifDisplay->GetArea());
    n = AmMakeArgList( args,
            XmNdeleteResponse, XmDESTROY,
            XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
            NULL);
    formDlog = XmCreateFormDialog( shell, "deleteLayer", args, n);

/* Display a list of the plots in the overlay plot. */

    mstrs = AmMotifStringList( titleList, &numItems);
    n = AmMakeArgList( args,
            StickLeft, StickRight, StickBottom,
            XmNbottomOffset, 40,
            XmNitems, mstrs,
            XmNitemCount, numItems,
            XmNvisibleItemCount, 8,
            XmNselectionPolicy, XmSINGLE_SELECT,
            XmNwidth, 300,
            NULL);
    list = XmCreateScrolledList( formDlog, "deleteLayerList", args, n);
    AmFreeMotifStringList( mstrs, numItems);
    XtManageChild( list);
    XtManageChild( formDlog);

/* Create the ok and cancel buttons, and assign the callbacks. */

    okBtn = XtVaCreateManagedWidget( "okBtn",
                xmPushButtonWidgetClass, formDlog,
                StickLeft, StickBottom,
                NULL);
    cancelBtn = XtVaCreateManagedWidget( "cancelBtn",
                    xmPushButtonWidgetClass, formDlog,
                    StickRight, StickBottom,
                    NULL);

    XtAddCallback( cancelBtn, XmNactivateCallback,
        ( XtCallbackProc)AmPopdownCback, formDlog);

    AmAddVaCallback( cancelBtn, XmNactivateCallback,
        ( VarargsCallback)pickLayerOrOverlayCancelCallback,
        overlay, handlerArgs, END_LIST);

    AmAddVaCallback( okBtn, XmNactivateCallback,
        ( VarargsCallback)pickLayerOrOverlayOkCallback,
        list, overlay, handlerArgs, END_LIST);

    XtManageChild( formDlog);
}


void
pickHandler( Widget             widget,
             AmPickHandlerArgsPtr handlerArgs,
             XEvent             *event,
             AnvilBoolean            *)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    XButtonEvent       *buttonEvent;
    XMotionEvent       *motionEvent;
    AmScreenObjClass     *screenObj;
    AmObjectTypeEnum     objectType = handlerArgs->objectType;
    const AmRectangleRec *rect;

    motifDisplay->UnsetOutline(); 
    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {
            motifDisplay->WriteResourceMessage( handlerArgs->cancelMessage);

        /* Remove the handler */

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler)pickHandler, handlerArgs);

        /* Send a fake event to reset the cursor */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        } else if ( buttonEvent->button == Button1) {

        /* Figure out which object has been picked. */

            screenObj = motifDisplay->FindObject( buttonEvent->x, buttonEvent->y);
            if (( screenObj == NULL) || !screenObj->IsType( objectType)) {
                return;
            }

        /* Display a busy cursor. I wonder if this is what's causing the cursor
	   problems where the watch is hanging around... ? XXXXX */

            motifDisplay->SetCursor( XC_watch);

        /* Traverse the overlay if needed. */ 

            if ( (handlerArgs->traverseOverlay == TRUE) &&
                 (screenObj->GetObjectType() == AmOVERLAY)) {
                pickLayerOrOverlay( widget, ( AmOverlayClass *)screenObj,
                    handlerArgs);
            } else if ( handlerArgs->objectFunction( screenObj,
                                         handlerArgs->clientData)) {
                motifDisplay->WriteResourceMessage(
                                  handlerArgs->successMessage);
            } else {
                motifDisplay->WriteResourceMessage( 
                                  handlerArgs->failureMessage);
            }

        /* Remove the handler. */

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler)pickHandler, handlerArgs);

        /* Send a fake event to reset the cursor */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        }
    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *) event;

/* Set the outline if necessary */

        screenObj = motifDisplay->FindObject( motionEvent->x, motionEvent->y);
        if (( screenObj != NULL) && screenObj->IsType( objectType)) {
            rect = screenObj->GetRect();
            motifDisplay->SetOutline( rect->x, rect->y,
                              rect->width, rect->height,
                              handlerOutline);
        }
    }
}


void
AmMoveHandler( Widget             widget,
             AmMoveHandlerArgsPtr handlerArgs,
             XEvent             *event,
             AnvilBoolean            *)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    AmListPtr            screenObjs = handlerArgs->screenObjs;
    AmXWindowClass       *window = motifDisplay->GetWindow();
    XButtonEvent       *buttonEvent;
    XMotionEvent       *motionEvent;
    Dimension          areaWidth, areaHeight;
    AmListPtr            temp;
    int                startX = handlerArgs->startX,
                       startY = handlerArgs->startY,
                       x = handlerArgs->x,
                       y = handlerArgs->y,
                       w = handlerArgs->width,
                       h = handlerArgs->height,
                       xOffset = handlerArgs->xOffset,
                       yOffset = handlerArgs->yOffset;

/* Figure out the extends of the drawing area. */

    XtVaGetValues( widget,
        XmNwidth, &areaWidth,
        XmNheight, &areaHeight,
        NULL); 

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->WriteResourceMessage( "moveCancelMsg");
            motifDisplay->UnsetOutlines();

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler)AmMoveHandler, handlerArgs);

            motifDisplay->UnmapTravelCursor();

        /*
         * Call the master handler to reset the cursor and draw the
         *   correct outline.
         */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button1) {

        /* Remove the old outline */

            motifDisplay->UnsetOutlines();
            motifDisplay->UnmapTravelCursor();

            if ( x != handlerArgs->startX || y != handlerArgs->startY) {

            /* Display a busy cursor */

                motifDisplay->SetCursor( XC_watch);

            /* Update the relevant areas on the screen. */

                updateMoveObjects( screenObjs, startX, startY, x, y,
				   areaWidth, areaHeight);
		motifDisplay->Flush();
            }

        /* Remove the move handler. */

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler)AmMoveHandler, handlerArgs);

            motifDisplay->WriteResourceMessage( "moveSuccessMsg");

        /*
         * Generate a motion event so that the master handler will reset
         *   the cursor and correct outline.
         */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);

        }
    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *)event;

    /* Figure out the extends of the rectangle. */

        x = motionEvent->x - xOffset;

        if ( x < 0) {
            x = 0;
        } else if ( x + w > areaWidth) {
            x = areaWidth - w;
        }

        y = motionEvent->y - yOffset;
        if ( y < 0) {
            y = 0;
        } else if ( y + h > areaHeight) {
            y = areaHeight - h;
        }

    /* Remove the old rectangle and draw the new one. */

        temp = AmGenerateMoveOutlines( screenObjs, startX, startY, x, y);
        motifDisplay->UpdateOutlines( temp);
        AmFreeLst( temp, ( AmItemFn) free);

        handlerArgs->x = x;
        handlerArgs->y = y;

        motifDisplay->WriteTravelCursor( "Pos: %d, %d", x, y);
    }
}


void
AmResizeHandler( Widget               widget,
               AmResizeHandlerArgsPtr handlerArgs,
               XEvent               *event,
               AnvilBoolean              *)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    AmListPtr            temp, screenObjs = handlerArgs->screenObjs;
    AmXWindowClass       *window = motifDisplay->GetWindow();
    XButtonEvent       *buttonEvent;
    XMotionEvent       *motionEvent;
    AmRectangleRec       oldRect, newRect;
    Dimension          areaWidth, areaHeight;
    int                startX = handlerArgs->startX,
                       startY = handlerArgs->startY,
                       startW = handlerArgs->startWidth,
                       startH = handlerArgs->startHeight,
                       x = handlerArgs->x,
                       y = handlerArgs->y,
                       w = handlerArgs->width,
                       h = handlerArgs->height,
                       xOffset = handlerArgs->xOffset,
                       yOffset = handlerArgs->yOffset,
                       motionX, motionY;

/* Figure out the extends of the drawing area. */

    XtVaGetValues( widget,
        XmNwidth, &areaWidth,
        XmNheight, &areaHeight,
        NULL); 

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Remove the old rectangles. */

            motifDisplay->WriteResourceMessage( "resizeCancelMsg");
            motifDisplay->UnsetOutlines();
            motifDisplay->UnmapTravelCursor();

        /* Button 3 cancels the operation. */

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmResizeHandler,
                handlerArgs);

        /*
         * Generate a motion event so that the master handler will reset
         *   the cursor and correct outline.
         */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);

        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button1) {

        /* Display a busy cursor */

            motifDisplay->SetCursor( XC_watch);
            motifDisplay->UnsetOutlines();
            motifDisplay->UnmapTravelCursor();

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmResizeHandler,
                handlerArgs);

        /*
         * Resize the screen object and then update the relevent areas
         *   on the screen.
         */

            findRealExtents( screenObjs, &oldRect);
            updateResizeObjects( screenObjs, startX, startY,
                startW, startH, x, y, w, h, areaWidth, areaHeight);
            findRealExtents( screenObjs, &newRect);
	    motifDisplay->Flush();

            motifDisplay->WriteResourceMessage( "resizeSuccessMsg");

        /*
         * Generate a motion event so that the master handler will reset
         *   the cursor and correct outline.
         */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);

        }
    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *)event;

    /*
     * Depending on what type of resizing is being done... figure out
     *   the extends of the rectangle.
     */

        motionX = motionEvent->x - xOffset;
        motionY = motionEvent->y - yOffset;


        switch( handlerArgs->actionType) {
            case AmACTION_RESIZE_UP_LEFT:
                x = motionX;
                y = motionY;
                if ( x > (startX + startW - MIN_SIZE)) {
                    x = startX + startW - MIN_SIZE;
                } else if ( x < 0) {
                    x = 0;
                }
                if ( y > (startY + startH - MIN_SIZE)) {
                    y = startY + startH - MIN_SIZE;
                } else if ( y < 0) {
                    y = 0;
                }
                w = startX + startW - x;
                h = startY + startH - y;
                break;
            case AmACTION_RESIZE_UP:
                x = startX;
                y = motionY;
                if ( y > (startY + startH - MIN_SIZE)) {
                    y = startY + startH - MIN_SIZE;
                } else if ( y < 0) {
                    y = 0;
                }
                w = startW;
                h = startY + startH - y;
                break;
            case AmACTION_RESIZE_UP_RIGHT:
                x = startX;
                y = motionY;
                if ( y > (startY + startH - MIN_SIZE)) {
                    y = startY + startH - MIN_SIZE;
                } else if ( y < 0) {
                    y = 0;
                }
                w = motionX - x;
                if ( w < MIN_SIZE) {
                    w = MIN_SIZE;
                } else if ( x + w > areaWidth) {
                    w = areaWidth - x;
                }
                h = startY + startH - y;
                break;
            case AmACTION_RESIZE_LEFT:
                x = motionX;
                y = startY;
                if ( x > (startX + startW - MIN_SIZE)) {
                    x = startX + startW - MIN_SIZE;
                } else if ( x < 0) {
                    x = 0;
                }
                w = startX + startW - x;
                h = startH;
                break;
            case AmACTION_RESIZE_RIGHT:
                x = startX;
                y = startY;
                w = motionX - x;
                if ( w < MIN_SIZE) {
                    w = MIN_SIZE;
                } else if ( x + w > areaWidth) {
                    w = areaWidth - x;
                }
                h = startH;
                break;
            case AmACTION_RESIZE_DOWN_LEFT:
                x = motionX;
                y = startY;
                if ( x > (startX + startW - MIN_SIZE)) {
                    x = startX + startW - MIN_SIZE;
                } else if ( x < 0) {
                    x = 0;
                }
                w = startX + startW - x;
                h = motionY - y;
                if ( h < MIN_SIZE) {
                    h = MIN_SIZE;
                } else if ( y + h > areaHeight) { 
                    h = areaHeight - y;
                }
                break;
            case AmACTION_RESIZE_DOWN:
                x = startX;
                y = startY;
                w = startW;
                h = motionY - y;
                if ( h < MIN_SIZE) {
                    h = MIN_SIZE;
                } else if ( y + h > areaHeight) {
                    h = areaHeight - y;
                }
                break;
            case AmACTION_RESIZE_DOWN_RIGHT:
                x = startX;
                y = startY;
                w = motionX - x;
                if ( w < MIN_SIZE) {
                    w = MIN_SIZE;
                } else if ( x + w > areaWidth) {
                    w = areaWidth - x;
                }
                h = motionY - y;
                if ( h < MIN_SIZE) {
                    h = MIN_SIZE;
                } else if ( y + h > areaHeight) {
                    h = areaHeight - y;
                }
                break;
            default:
                break;
        }

    /* Update the outline */

        temp = AmGenerateResizeOutlines( screenObjs, startX, startY,
                   startW, startH, x, y, w, h);
        motifDisplay->UpdateOutlines( temp);
        AmFreeLst( temp, ( AmItemFn) free);

        handlerArgs->x = x;
        handlerArgs->y = y;
        handlerArgs->width = w;
        handlerArgs->height = h;

        motifDisplay->WriteTravelCursor( "Size: %d, %d", w, h);
    }
}


void
AmOutlineHandler( Widget                widget,
                outlineHandlerArgsPtr handlerArgs,
                XEvent             *event,
                AnvilBoolean            *)
{
    AmMotifDisplayClass  *motifDisplay = handlerArgs->motifDisplay;
    AmListPtr            *activeObjects = handlerArgs->activeObjects;
    AmXWindowClass       *window = motifDisplay->GetWindow();
    XButtonEvent       *buttonEvent;
    XMotionEvent       *motionEvent;
    Dimension          areaWidth, areaHeight;
    AmListPtr            temp;
    int                startX = handlerArgs->x,
                       startY = handlerArgs->y,
                       x, y;

/* Figure out the extends of the drawing area. */

    XtVaGetValues( widget,
        XmNwidth, &areaWidth,
        XmNheight, &areaHeight,
        NULL); 

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            motifDisplay->UnsetOutlines();
            motifDisplay->WriteResourceMessage( "outlineCancelMsg");

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmOutlineHandler, handlerArgs);

        /*
         * Call the master handler to reset the cursor and draw the
         *   correct outline.
         */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;
        if ( buttonEvent->button == Button1) {

            x = buttonEvent->x;
            y = buttonEvent->y;

        /* Remove the old outline and display a busy cursor.*/

            motifDisplay->UnsetOutlines();
            motifDisplay->SetCursor( XC_watch);

        /* Activate the objects within the rectangle */
 
            activateObjects( motifDisplay, activeObjects, startX, startY, x, y);

        /* Remove the outline handler. */

            AmRemoveHandler( widget, ButtonPressMask | ButtonReleaseMask |
                PointerMotionMask,
                ( XtEventHandler) AmOutlineHandler, handlerArgs);

        /*
         * Generate a motion event so that the master handler will reset
         *   the cursor and correct outline.
         */

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);

        }
    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *)event;

    /* Figure out the extends of the rectangle. */

        x = motionEvent->x;
        y = motionEvent->y;

        if ( x < 0) {
            x = 0;
        } else if ( x > areaWidth) {
            x = areaWidth;
        }

        if ( y < 0) {
            y = 0;
        } else if ( y  > areaHeight) {
            y = areaHeight;
        }

    /* Remove the old rectangle and draw the new one. */

        temp = generateOutlineOutlines( *activeObjects, startX, startY, x, y);
        motifDisplay->UpdateOutlines( temp);
        AmFreeLst( temp, ( AmItemFn) free);
    }
}

void
AmSetupSecondaryHandler( AmActionEnum        actionType,
                       AmMotifDisplayClass *motifDisplay,
                       AmScreenObjClass    *screenObj,
                       AmListPtr           screenObjs,
                       int               x,
                       int               y,
                       AmHandlerFn         doubleClickFn)
{
    Widget                  area = motifDisplay->GetArea();
    AmSecondaryHandlerArgsPtr secondaryHandlerArgs;
    XtIntervalId            timerId;

fprintf( stderr, "setting up secondary handler\n");
    secondaryHandlerArgs = AmMakeSecondaryHandlerArgs( motifDisplay,
                               screenObj, screenObjs,
                               actionType, x, y, doubleClickFn);

    timerId = XtAppAddTimeOut( XtWidgetToApplicationContext( area), AmCLICK_TIMEOUT,
                  ( XtTimerCallbackProc) AmPrimaryTimeout,
                  secondaryHandlerArgs);
    secondaryHandlerArgs->timerId = timerId;
    
    AmAddHandler( NULL, area,
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) AmSecondaryHandler,
        secondaryHandlerArgs, AmACTION_SECONDARY_PRIORITY, 
        ( AmStateHandlerFn) AmRemoveSecondaryHandler);
}

void
AmSetupResizeHandler( AmMotifDisplayClass *motifDisplay,
                    AmListPtr           screenObjs,
                    int               x,
                    int               y,
                    AmActionEnum        action)
{
    Widget               area = motifDisplay->GetArea();
    AmResizeHandlerArgsPtr resizeHandlerArgs;
    AmRectangleRec         rect;
    AmListPtr              temp;
    int                  xOffset, yOffset;

    findExtents( screenObjs, &rect);

    switch( action) {
        case AmACTION_RESIZE_UP_LEFT:
            xOffset = x - rect.x; 
            yOffset = y - rect.y; 
            break;
        case AmACTION_RESIZE_UP:
            xOffset = 0;
            yOffset = y - rect.y; 
            break;
        case AmACTION_RESIZE_UP_RIGHT:
            xOffset = - (rect.x + rect.width - x);
            yOffset = y - rect.y; 
            break;
        case AmACTION_RESIZE_RIGHT:
            xOffset = - (rect.x + rect.width - x);
            yOffset = 0;
            break;
        case AmACTION_RESIZE_LEFT:
            xOffset = x - rect.x;
            yOffset = 0;
            break;
        case AmACTION_RESIZE_DOWN_LEFT:
            xOffset = x - rect.x;
            yOffset = - (rect.y + rect.height - y);
            break;
        case AmACTION_RESIZE_DOWN:
            xOffset = 0;
            yOffset = - (rect.y + rect.height - y);
            break;
        case AmACTION_RESIZE_DOWN_RIGHT:
            xOffset = - (rect.x + rect.width - x);
            yOffset = - (rect.y + rect.height - y);
            break;
        default:
            xOffset = 0;
            yOffset = 0;
            break;
    }

    resizeHandlerArgs = AmMakeResizeHandlerArgs(
                            motifDisplay,
                            screenObjs,
                            action,
                            rect.x,
                            rect.y,
                            rect.width,
                            rect.height,
                            xOffset,
                            yOffset);

    AmAddHandler( NULL, area, ButtonPressMask |
        ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) AmResizeHandler,
        resizeHandlerArgs, AmACTION_RESIZE_PRIORITY, 
        ( AmStateHandlerFn) AmRemoveResizeHandler,
        ( AmStateHandlerFn) suspendResizeHandler,
        ( AmStateHandlerFn) restartResizeHandler);

    temp = generateOutlines( screenObjs);
    motifDisplay->SetOutlines( temp, handlerOutline);
    AmFreeLst( temp, ( AmItemFn) free);

    motifDisplay->WriteResourceMessage( "resizeMsg");

    motifDisplay->MapTravelCursor();
    motifDisplay->WriteResourceTravelCursor( "resizingMsg",
                      rect.width, rect.height);
}

void
AmSetupMoveHandler( AmMotifDisplayClass *motifDisplay,
                  AmListPtr           screenObjs,
                  int               x,
                  int               y)
{
    Widget             area = motifDisplay->GetArea();
    AmMoveHandlerArgsPtr moveHandlerArgs;
    AmRectangleRec       extentsRec;
    AmListPtr            temp;


    findExtents( screenObjs, &extentsRec);

    moveHandlerArgs = AmMakeMoveHandlerArgs(
                          motifDisplay,
                          screenObjs,
                          extentsRec.x,
                          extentsRec.y,
                          extentsRec.width,
                          extentsRec.height,
                          x - extentsRec.x,
                          y - extentsRec.y);

    AmAddHandler( NULL, area, ButtonPressMask |
        ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler)AmMoveHandler,
        moveHandlerArgs, AmACTION_MOVE_PRIORITY, 
        ( AmStateHandlerFn) AmRemoveMoveHandler,
        ( AmStateHandlerFn) AmSuspendMoveHandler,
        ( AmStateHandlerFn) AmRestartMoveHandler);

    temp = generateOutlines( screenObjs);
    motifDisplay->SetOutlines( temp, handlerOutline);
    AmFreeLst( temp, ( AmItemFn) free);

    motifDisplay->WriteResourceMessage( "moveMsg");

    motifDisplay->MapTravelCursor();
    motifDisplay->WriteResourceTravelCursor( "movingMsg",
                      extentsRec.x, extentsRec.y);
}


void
AmSetupOutlineHandler( AmMotifDisplayClass *motifDisplay,
                     AmListPtr           *screenObjs,
                     int               x,
                     int               y)
{
    Widget                area = motifDisplay->GetArea();
    outlineHandlerArgsPtr outlineHandlerArgs;

    motifDisplay->WriteResourceMessage( "outlineInstructionMsg");

    outlineHandlerArgs = AmMakeOutlineHandlerArgs( motifDisplay,
                               screenObjs, x, y);

    AmAddHandler( NULL, area,
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) AmOutlineHandler,
        outlineHandlerArgs, AmACTION_PICK_PRIORITY, 
        ( AmStateHandlerFn) AmRemoveOutlineHandler);

    motifDisplay->SetOutlineFn( handlerOutline);
    motifDisplay->SetCursor( actionToCursor( AmACTION_MOVE));
}


AmActionEnum
setupHandler( AmActionEnum        action,
              AmMotifDisplayClass *motifDisplay,
              AmListPtr           screenObjs,
              int               x,
              int               y)
{
    switch( action) {
        case AmACTION_RESIZE_UP_LEFT:
        case AmACTION_RESIZE_UP:
        case AmACTION_RESIZE_UP_RIGHT:
        case AmACTION_RESIZE_LEFT:
        case AmACTION_RESIZE_RIGHT:
        case AmACTION_RESIZE_DOWN_LEFT:
        case AmACTION_RESIZE_DOWN:
        case AmACTION_RESIZE_DOWN_RIGHT:
            AmSetupResizeHandler( motifDisplay, screenObjs, x, y, action);
            break;

        case AmACTION_MOVE:
            AmSetupMoveHandler( motifDisplay, screenObjs, x, y);
            break;

         default:
            action = AmACTION_NONE;
            break;
     }

    return( action);
}


void
AmSecondaryHandler( Widget                  widget,
                  AmSecondaryHandlerArgsPtr handlerArgs,
                  XEvent                  *event,
                  AnvilBoolean                 *)
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmScreenObjClass    *screenObj = handlerArgs->screenObj;
    AmListPtr           screenObjs = handlerArgs->screenObjs;
    AmActionEnum        action = handlerArgs->actionType;
    XButtonEvent      *buttonEvent;
    XtIntervalId      timerId = handlerArgs->timerId;
    AmHandlerFn         doubleClickFn = handlerArgs->doubleClickFn;
    int               x = handlerArgs->x,
                      y = handlerArgs->y;

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;

        if ( buttonEvent->button == Button3) {

        /* Cancel the timer. */

            XtRemoveTimeOut( timerId);

        /* Cancel whatever operation was going to happen. */

            motifDisplay->UnsetOutlines();

            AmRemoveHandler( widget,
                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                ( XtEventHandler) AmSecondaryHandler, handlerArgs);

            AmSendMotion( XtDisplay( widget), XtWindow( widget),
                buttonEvent->x, buttonEvent->y);
        } else if ( buttonEvent->button == Button1) {

        /* Cancel the timer. */

            XtRemoveTimeOut( timerId);

            AmRemoveHandler( widget,
                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                ( XtEventHandler) AmSecondaryHandler, handlerArgs);

            fprintf( stderr, "DOUBLE TROUBLE\n");
            doubleClickFn( screenObj, motifDisplay);

/* if the double click function creates a modal dialog, we want that request
   honored before any more events are processed */

	    XSync( XtDisplay( motifDisplay->GetArea()), TRUE);
        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;

        if ( buttonEvent->button == Button1) {

        /* Cancel the timer. */

            XtRemoveTimeOut( timerId);

        /* Launch the second timer. */

            timerId = XtAppAddTimeOut( XtWidgetToApplicationContext( widget),
                          AmCLICK_TIMEOUT,
                          ( XtTimerCallbackProc) AmSecondaryTimeout,
                          handlerArgs);
            handlerArgs->timerId = timerId;
            handlerArgs->actionType = AmACTION_NONE;
        }
    } else if ( event->type == MotionNotify) {

    /* Cancel the timer. */

        XtRemoveTimeOut( timerId);

    /* Remove the secondary handler and launch the correct handler. */

        AmRemoveHandler( widget,
            ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
            ( XtEventHandler) AmSecondaryHandler, handlerArgs);

        action = setupHandler( action, motifDisplay, screenObjs, x, y);
    }
}


void
AmPrimaryTimeout( AmSecondaryHandlerArgsPtr handlerArgs,
                XtPointer               )
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    AmListPtr           screenObjs = handlerArgs->screenObjs;
    AmActionEnum        action = handlerArgs->actionType;
    Widget            area = motifDisplay->GetArea();
    int               x = handlerArgs->x,
                      y = handlerArgs->y;

    fprintf( stderr, "PRIMARY TIMEOUT\n");

    AmRemoveHandler( area,
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) AmSecondaryHandler, handlerArgs);

    action = setupHandler( action, motifDisplay, screenObjs, x, y);
}


void
AmSecondaryTimeout( AmSecondaryHandlerArgsPtr handlerArgs,
                  XtPointer               )
{
    AmMotifDisplayClass *motifDisplay = handlerArgs->motifDisplay;
    Widget            area = motifDisplay->GetArea();

    fprintf( stderr, "SECONDARY TIMEOUT\n");

    AmRemoveHandler( area,
        ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
        ( XtEventHandler) AmSecondaryHandler, handlerArgs);
}

/*********************************************************************
 *
 * AmMasterHandler
 *
 * Description:
 *     This handler is responsible for objects getting put in the
 *     active object set.  And for objects getting resized and
 *     moved.
 *
 *********************************************************************/

void
AmMasterHandler( Widget               ,
               AmMasterHandlerArgsPtr handlerArgs,
               XEvent               *event,
               AnvilBoolean              *)
{
    AmMotifDisplayClass    *motifDisplay = handlerArgs->motifDisplay;
    AmScreenObjClass       *screenObj;
    XButtonEvent         *buttonEvent;
    XMotionEvent         *motionEvent;
    const AmRectangleRec   *rect;
    AmActionEnum           action;
    AnvilBoolean              outlineSet;
    AmListPtr              temp;
    int                  x, y;

    action = AmACTION_NONE;
    outlineSet = FALSE;

    if ( event->type == ButtonPress) {
        buttonEvent = ( XButtonEvent *) event;

    /* Right mouse button cancels seleciton. */

        if ( buttonEvent->button == Button3) {

            AmJustFreeLst( handlerArgs->activeObjects);
            handlerArgs->activeObjects = NULL;

        } else if ( buttonEvent->button == Button1) {
            x = buttonEvent->x;
            y = buttonEvent->y;

            screenObj = findObjectFromList( handlerArgs->activeObjects, x, y);

            if ( screenObj != NULL) {
                if ( (buttonEvent->state & ShiftMask)) {

                    handlerArgs->activeObjects = AmDelPtrLst( screenObj,
                        handlerArgs->activeObjects, ( AmSearchFn) AmEquals);

                    motifDisplay->WriteResourceMessage(
                                      "unselectSuccessMsg");
 
                } else {

                /*
                 * The user clicked somewhere in the object, setup
                 *   the secondary master handler that takes care of 
                 *   double clicks.
                 */
                      
                    action = findAction( handlerArgs->activeObjects, x, y);
                    AmSetupSecondaryHandler( action, motifDisplay,
                        screenObj,
                        handlerArgs->activeObjects,
                        x, y, handlerArgs->doubleClickFn);
                }
            } else {
                screenObj = motifDisplay->FindObject( x, y);
                if ( screenObj != NULL) {
                  
                /* A new object was selected. */

                    if ( !(buttonEvent->state & ShiftMask)) {
                        AmJustFreeLst( handlerArgs->activeObjects);
                        handlerArgs->activeObjects = NULL;
                    }

                    handlerArgs->activeObjects =
                        AmInsertNodeLst( screenObj, handlerArgs->activeObjects);

                    motifDisplay->WriteResourceMessage(
                                      "selectSuccessMsg");

                    if ( !(buttonEvent->state & ShiftMask)) {
                        action = AmACTION_MOVE;
                        AmSetupSecondaryHandler( action, motifDisplay, 
                            screenObj,
                            handlerArgs->activeObjects,
                            x, y, handlerArgs->doubleClickFn);
                    }

                } else {

                /* The user clicked somewhere where there was no object. */

                    if ( !(buttonEvent->state & ShiftMask)) {
                        AmJustFreeLst( handlerArgs->activeObjects);
                        handlerArgs->activeObjects = NULL;

                        AmSetupOutlineHandler( motifDisplay, 
                            &(handlerArgs->activeObjects),
                            x, y);
  
                        outlineSet = TRUE;
                    } else {
                        AmSetupOutlineHandler( motifDisplay, 
                            &(handlerArgs->activeObjects),
                            x, y);
  
                        outlineSet = TRUE;
                    }
                }
            }
        }
    } else if ( event->type == MotionNotify) {

        motionEvent = ( XMotionEvent *) event;
        x = motionEvent->x;
        y = motionEvent->y;

        if ( handlerArgs->activeObjects != NULL) {

        /*
         * There is an active object, so check if one of the handler
         *     shortcuts exists.
         */

            action = findAction( handlerArgs->activeObjects, x, y);
        } else {
            screenObj = motifDisplay->FindObject( x, y);

            if ( screenObj != NULL) {
                action = AmACTION_PICK;
                rect = screenObj->GetRect();
                motifDisplay->SetOutline( rect->x, rect->y,
                    rect->width, rect->height, handlerOutline);
                outlineSet = TRUE;
            }
        }
    }

    if ( outlineSet == FALSE) {
        if ( handlerArgs->activeObjects != NULL) {
            temp = generateOutlines( handlerArgs->activeObjects); 
            motifDisplay->SetOutlines( temp, showOutline);
            AmFreeLst( temp, ( AmItemFn) free);
        } else {
            motifDisplay->UnsetOutlines();
        }
    }

    motifDisplay->SetCursor( actionToCursor( action));
}
