#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if !defined( _WIN32)
#include <unistd.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <malloc.h>

#include <anvil/macros.h>
#include <anvil/mem_funcs.h>
#include <anvil/strlib.h>

#include <anvil/cad/defs.h>
#include <anvil/cad/list.h>


AmCadListPtr
AmMakeList()
{
    AmCadListPtr list;

    list = ( AmCadListPtr) malloc( sizeof( AmCadListRec));
    if ( list == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    list->head = NULL;
    list->tail = NULL;
    return( list);
}


AmErrorCode
AmFreeList( AmCadListPtr list)
{
    AmCadNodePtr next, temp;

    if ( list == NULL) {
        RETURN_ERROR( AmFREE_MEMORY);
    }

    next = list->head;
    while ( next != NULL) {
        temp = next;
        next = temp->next;
        free( temp);
    }
 
    free( list);
    return( AmOK);
}


AmErrorCode
AmFreeList( AmCadListPtr list, AmItemFn freeFn)
{
    AmCadNodePtr next, temp;

    if ( list == NULL) {
        RETURN_ERROR( AmFREE_MEMORY);
    }

    next = list->head;
    while ( next != NULL) {
        temp = next;
        next = temp->next;
        freeFn( temp->data);
        free( temp);
    }
 
    free( list);
    return( AmOK);
}


AmCadNodePtr
AmMakeNode( void *data)
{
    AmCadNodePtr node;

    node = ( AmCadNodePtr) malloc( sizeof( AmCadNodeRec));
    if ( node == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    node->data = data;
    node->next = NULL;
    return( node);
}



AmErrorCode
AmInsertHead( AmCadListPtr list, void *data)
{
    AmCadNodePtr node;

    if ( list == NULL) {
        RETURN_ERROR( AmNULL_ARGUMENT);
    }

    node = AmMakeNode( data);
    node->next = list->head;
    list->head = node;

    if ( list->tail == NULL) {
        list->tail = node;
    }

    return( AmOK);
}


AmErrorCode
AmInsertTail( AmCadListPtr list, void *data)
{
    AmCadNodePtr node;

    if ( list == NULL) {
        RETURN_ERROR( AmNULL_ARGUMENT);
    }

    node = AmMakeNode( data);
 
    if ( list->tail == NULL) {
        list->head = list->tail = node;
    } else {
        list->tail->next = node;
        list->tail = node;
    }

    return( AmOK);
}


void *
AmGetHead( AmCadListPtr list)
{
    if ( list->head == NULL) {
        return( NULL);
    }

    return( list->head->data);
}


void *
AmGetTail( AmCadListPtr list)
{
    if ( list->tail == NULL) {
        return( NULL);
    }

    return( list->tail->data);
}


AmCadNodePtr
AmGetNode( AmCadListPtr list, void *cmpData, AmSearchFn cmpFn)
{
    AmCadNodePtr temp;

    temp = list->head;
    while ( temp != NULL) {
        if ( cmpFn( temp->data, cmpData) == 0) {
            return( temp);
        }
        temp = temp->next;
    }

    return( NULL);
}


AmCadNodePtr
AmGetNode( AmCadNodePtr temp, void *cmpData, AmSearchFn cmpFn)
{
    while ( temp != NULL) {
        if ( cmpFn( temp->data, cmpData) == 0) {
            return( temp);
        }
        temp = temp->next;
    }

    return( NULL);
}
