#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if !defined(__BSD44__)
#include <alloca.h>
#endif
#include <anvil/macros.h>
#include <anvil/errcode.h>

#include <anvil/cad/defs.h>
#include <anvil/cad/mem.h>
#include <anvil/cad/segmentProto.h>
#include <anvil/cad/objectProto.h>

#define NUM_SEGS	20000
#define NUM_OBJS	2000000

#define HASH_SIZE	73
#define CACHE_SIZE	20

// #define HASH_SIZE	521
// #define CACHE_SIZE	210

#ifdef STATISTICS

long numCompacts = 0;
long numAllocs = 0;
long numAllocSeeks = 0;
long numReallocs = 0;
long numReallocSeeks = 0;

#endif

extern int mallocs;

AmErrorCode ANVIL_ERROR_CODE = AmOK;
char        *program = NULL;



void
printObjectsInSegment( AmMemorySegmentPtr segment)
{
    AmHeaderPtr header;
    pointPtr  point;
    linePtr   line;
    long ii;
 
    for ( ii = 0; ii < segment->AmLast;)  {
        header = ( AmHeaderPtr) &segment->data[ ii];
        switch( HTYPE( header)) {
            case AmTYPE_POINT:
                point = ( pointPtr) ((( uchar_t *) header) +
                            sizeof( AmHeaderRec) +
                            sizeof( objectRec));
                printf( "Point:\n    ID: %d\n    x: %f\n    y: %f\n",
                    HID( header), point->x, point->y);
                break;
            case AmTYPE_LINE:
                line = ( linePtr) ((( uchar_t *) header) +
                          sizeof( AmHeaderRec) +
                          sizeof( objectRec));
                printf(  "Line:\n    ID: %d\n    p1: %d\n    p2: %d\n",
                    HID( header), line->p1, line->p2);
                break;
            default:
                break;
        }

        if ( HSIZE( header) == 0) {
            printf( "Error in segment at location %d... aborting\n", ii);
            break;
        }

        ii += HSIZE( header);
    }
}


#ifdef STATISTICS

/*
 * Function: void printStatistics()
 *
 * Parameters:
 *   -none-
 *
 * Description:
 *   Prints information concerning access statistics.  Used for
 *   bench marking of the memory allocation scheme.
 *
 * Returns:
 *   -nothing-
 */

void
printStatistics()
{
    printf( "Num allocs: %d\n", numAllocs);
    printf( "    Seeks: %d (%d seeks per alloc)\n", numAllocSeeks,
        ( numAllocs != 0) ? numAllocSeeks / numAllocs : 0);
    printf( "Num re-allocs: %d\n", numReallocs);
    printf( "    Seeks: %d (%d seeks per alloc)\n", numReallocSeeks,
        ( numReallocs != 0) ? numReallocSeeks / numReallocs : 0);
    printf( "Num compacts: %d\n", numCompacts);
}

#endif


AmMemoryPtr
createMemory()
{
    AmMemoryPtr memory;
    long      numObj, numSegs;
    char      filename[ PATH_LEN], temp[ 256];

    printf( "What file do you want to associate with the new memory?\n");
    printf( "--> ");
    scanf( "%s", filename);
    printf( "How many objects max?\n");
    printf( "--> ");
    scanf( "%s", temp);
    numObj = atol( temp);
    printf( "How many segments max?\n");
    printf( "--> ");
    scanf( "%s", temp);
    numSegs = atol( temp);
    memory = AmMakeMemory( numSegs, numObj, CACHE_SIZE, HASH_SIZE, filename);

    return( memory); 
}


AmMemoryPtr
AmLoadMemory()
{
    AmMemoryPtr memory;
    char      filename[ PATH_LEN];

    printf( "What file do you want to load into memory?\n");
    printf( "--> ");
    scanf( "%s", filename);
    memory = AmLoadMemory( filename, CACHE_SIZE, HASH_SIZE);

    return( memory); 
}


AmErrorCode
saveMemory( AmMemoryPtr memory)
{
    return( AmDumpMemory( memory));
}

AmErrorCode
addPoint( AmMemoryPtr memory)
{
    char  temp[ 256];
    long  point;
    float x,y;

    printf( "What is the X-coordinate?\n");
    printf( "--> ");
    scanf( "%s", temp);
    x = atof( temp);
    printf( "What is the Y-coordinate?\n");
    printf( "--> ");
    scanf( "%s", temp);
    y = atof( temp);

    point = makePoint( memory, x, y);
    return( ANVIL_ERROR_CODE);
}


AmErrorCode
addLine( AmMemoryPtr memory)
{
    char      temp[ 256];
    long      line, p1, p2;

    printf( "What is the 1st end point?\n");
    printf( "--> ");
    scanf( "%s", temp);
    p1 = atol( temp);

    if ( HTYPE( ( AmHeaderPtr) AmGetData( memory, p1)) != AmTYPE_POINT) {
        RETURN_ERROR( AmOBJECT_NOT_POINT);
    }

    printf( "What is the 2nd end point?\n");
    printf( "--> ");
    scanf( "%s", temp);
    p2 = atol( temp);

    if ( HTYPE( ( AmHeaderPtr) AmGetData( memory, p2)) != AmTYPE_POINT) {
        RETURN_ERROR( AmOBJECT_NOT_POINT);
    }

    line = makeLine( memory, p1, p2);
    return( ANVIL_ERROR_CODE);
}


AmErrorCode
addPolyline( AmMemoryPtr memory)
{
    char      temp[ 256];
    long      polyline, p, p1, p2, *pointList, numSegs, ii;

    printf( "What is the 1st end point?\n");
    printf( "--> ");
    scanf( "%s", temp);
    p1 = atol( temp);

    if ( HTYPE( ( AmHeaderPtr) AmGetData( memory, p1)) != AmTYPE_POINT) {
        RETURN_ERROR( AmOBJECT_NOT_POINT);
    }

    printf( "What is the 2nd end point?\n");
    printf( "--> ");
    scanf( "%s", temp);
    p2 = atol( temp);

    if ( HTYPE( ( AmHeaderPtr) AmGetData( memory, p2)) != AmTYPE_POINT) {
        RETURN_ERROR( AmOBJECT_NOT_POINT);
    }

    do {
        printf( "How many segments does the poly-line consist of?\n"); 
        printf( "--> ");
        scanf( "%s", temp);
        printf( "\n");
        numSegs = atol( temp);
        if ( numSegs == 0) {
            printf( "There must be at least one segment.\n\n");
        }
    } while ( numSegs == 0);

    if ( numSegs > 1) {
        pointList = ( long * ) alloca( ( numSegs - 1)* sizeof( long));
    } else {
        pointList = NULL;
    }

    for ( ii = 0; ii < numSegs - 1;) {
        printf( "What is the point AmBetween segment %d and %d?\n", ii, ii + 1);
        printf( "--> ");
        scanf( "%s", temp);
        printf( "\n");
        p = atol( temp);

        if ( HTYPE( ( AmHeaderPtr) AmGetData( memory, p)) != AmTYPE_POINT) {
            printf( "Object %d is not a point.\n\n", p);
        } else {
            pointList[ ii] = p;
            ii++;
        }
    }

    polyline = makePolyLine( memory, p1, p2, pointList, numSegs - 1);
    return( ANVIL_ERROR_CODE);
}


void
printAddMenu()
{
    printf( "Add Menu:\n"); 
    printf( "    Add [P]oint.\n"); 
    printf( "    Add [L]ine.\n"); 
    printf( "    Add Pol[y]-Line.\n"); 
    printf( "\n");
    printf( "    [E]xit.\n");
    printf( "\n");
}


AmErrorCode
AmAddObject( AmMemoryPtr memory)
{
    char answer[ 256];

    do {

        printAddMenu();
        printf( "--> ");
        scanf( "%s", answer);
        printf( "\n");

        switch( answer[ 0]) {
            case 'P': case 'p':
                if ( addPoint( memory) == AmOK) {
                    printf("Point added.\n\n");
                } else {
//                    printAnvilError();
                    printf("Point NOT added.\n\n");
                }
                break;
            case 'L': case 'l':
                if ( addLine( memory) == AmOK) {
                    printf("Line added.\n\n");
                } else {
//                    printAnvilError();
                    printf("Line NOT added.\n\n");
                }
                break;
            case 'Y': case 'y':
                if ( addPolyline( memory) == AmOK) {
                    printf("Poly-line added.\n\n");
                } else {
//                    printAnvilError();
                    printf("Poly-line NOT added.\n\n");
                }
                break;
            case 'E': case 'e':
                break;
            default:
                printf("Unknown option '%c'.\n\n", answer[ 0]);
                break;
        }
    } while ( answer[ 0] != 'E' && answer[ 0] != 'e');

    return( AmOK); 
}


AmErrorCode
removeObject( AmMemoryPtr memory)
{
    AmHeaderPtr header;
    long      ID;
    char      temp[ 256];

    printf( "Which object do you want to remove?\n");
    printf( "--> ");
    scanf( "%s", temp);
    ID = atol( temp);

    header = ( AmHeaderPtr) AmGetData( memory, ID);
    if ( header == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    switch( HTYPE( header)) {
        case AmTYPE_POINT:
            return( removePoint( memory, ID));
        case AmTYPE_LINE:
            return( removeLine( memory, ID));
        default:
            RETURN_ERROR( AmOBJECT_INVALID);
    }
}


void
AmPrintObject( AmMemoryPtr memory)
{
    AmHeaderPtr header;
    pointPtr  point;
    linePtr   line;
    refPtr    ref;
    long      ID, *refList, ii;
    char      temp[ 256];

    printf( "Which object do you want to view?\n");
    printf( "--> ");
    scanf( "%s", temp);
    ID = atol( temp);

    header = ( AmHeaderPtr) AmGetData( memory, ID);
    if ( header == NULL) {
        printf( "%u is not a valid object ID.\n\n", ID);
    } else {

        switch( HTYPE( header)) {
            case AmTYPE_POINT:
                point = ( pointPtr) ((( uchar_t *) header) +
                            sizeof( AmHeaderRec) +
                            sizeof( objectRec));
                printf( "Point:\n    ID: %d\n    x: %f\n    y: %f\n",
                    HID( header), point->x, point->y);
                break;
            case AmTYPE_LINE:
                line = ( linePtr) ((( uchar_t *) header) +
                          sizeof( AmHeaderRec) +
                          sizeof( objectRec));
                printf( "Line:\n    ID: %d\n    p1: %d\n    p2: %d\n",
                    HID( header), line->p1, line->p2);
                break;
            case AmTYPE_REFERENCE_LIST:
                printf( "Reference List:\n    ID: %d\n", HID( header));
                ref = ( refPtr) ((( uchar_t *) header) +
                          sizeof( AmHeaderRec));
                printf( "    Num Refs: %d\n", ref->numRefs);
                refList = ( long *) ((( uchar_t *) header) +
                          sizeof( AmHeaderRec) + sizeof( refRec));
                for ( ii = 0; ii < ref->numRefs; ii++) {
                    printf( "    Ref #%u: %u\n", ii, refList[ ii]);
                }
                break;
            case AmTYPE_FREE:
                printf( "Freed Object:\n");
                break;
            default:
                break;
        }
        printf( "\n");
    }
}


void
printObjects( AmMemoryPtr memory)
{
    long ii;

    for ( ii = 0; ii < memory->segTable->AmLast; ii++) {
        AmActivateSegment( memory, ii);
        printObjectsInSegment( &memory->segTable->segments[ ii]);
    }
}

void
printMainMenu()
{
    printf( "Main Menu:\n"); 
    printf( "    [C]reate memory.\n"); 
    printf( "    [L]oad memory.\n"); 
    printf( "    [S]ave memory.\n");
    printf( "\n");
    printf( "    [A]dd object.\n");
    printf( "    [R]emove object.\n");
    printf( "    [V]iew object.\n");
    printf( "    [E]edit object.\n");
    printf( "\n");
    printf( "    [P]rint objects.\n");
    printf( "\n");
    printf( "    [Q]uit.\n");
    printf( "\n");
}

int
main( int argc, char **argv)
{
    AmMemoryPtr memory = NULL;
    char      answer[ 256];

    program = strdup( argv[ 0]);

    do {
        printMainMenu();
        printf( "--> ");
        scanf( "%s", answer);
        printf( "\n");

        switch( answer[ 0]) {
            case 'C': case 'c':
                if ( memory != NULL) {
                    printf("Memory already exists.\n\n");
                } else {
                    memory = createMemory();
                    if ( memory != NULL) {
                        printf("Memory created.\n\n");
                    } else {
                        printf("Memory NOT created.\n\n");
                    }
                }
                break;
            case 'L': case 'l':
                if ( memory != NULL) {
                    printf("Memory already exists.\n\n");
                } else {
                    memory = AmLoadMemory();
                    if ( memory != NULL) {
                        printf("Memory loaded.\n\n");
                    } else {
                        printf("Memory NOT loaded.\n\n");
                    }
                }
                break;
            case 'S': case 's':
                if ( memory == NULL) {
                    printf("Memory doest NOT exist.\n\n");
                } else {
                    if ( saveMemory( memory) == AmOK) {
                        printf("Memory saved.\n\n");
                    } else {
//                        printAnvilError();
                        printf("Memory NOT saved.\n\n");
                    }
                }
                break;
            case 'A': case 'a':
                if ( memory == NULL) {
                    printf("Memory doest NOT exist.\n\n");
                } else {
                    AmAddObject( memory);
                }
                break;
            case 'V': case 'v':
                if ( memory == NULL) {
                    printf("Memory doest NOT exist.\n\n");
                } else {
                    AmPrintObject( memory); 
                }
                break;
            case 'R': case 'r':
                if ( memory == NULL) {
                    printf("Memory doest NOT exist.\n\n");
                } else {
                    removeObject( memory); 
                }
                break;
            case 'P': case 'p':
                if ( memory == NULL) {
                    printf("Memory doest NOT exist.\n\n");
                } else {
                    printObjects( memory);
                }
                break;
            case 'Q': case 'q':
                break;
            default:
                printf("Unknown option '%c'.\n\n", answer[ 0]);
                break;
        }

    } while ( answer[ 0] != 'q' && answer[ 0] != 'Q');

    printf( "Thank you - goodbye.\n");
}
