#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if !defined( _WIN32) 
#include <unistd.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <malloc.h>

#if !defined( _WIN32) && !defined(__BSD44__)
#include <alloca.h>
#endif

#include <anvil/macros.h>
#include <anvil/mem_funcs.h>
#include <anvil/strlib.h>

#include <anvil/cad/defs.h>
#include <anvil/cad/mem.h>
#include <anvil/cad/segmentProto.h>
#include <anvil/cad/objectProto.h>

AmCadPointPtr
AmGetPoint( AmMemoryPtr memory, long pointID)
{
    AmCadPointPtr point, temp;
    uchar_t     *data;

    data = AmGetData( memory, pointID);
    if ( data == NULL) {
        return( NULL);
    }

    if ( HTYPE((( AmHeaderPtr) data)) != AmTYPE_POINT) {
        return( NULL);
    }

    temp = GET_POINT( data);

    point = ( AmCadPointPtr) malloc( sizeof( AmCadPointRec));
    if ( point == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    memcpy( point, temp, sizeof( AmCadPointRec));

    return( point);
}


AmCadLinePtr
AmGetLine( AmMemoryPtr memory, long lineID)
{
    AmCadLinePtr  line, temp;
    uchar_t     *data;

    data = AmGetData( memory, lineID);
    if ( data == NULL) {
        return( NULL);
    }

    if ( HTYPE((( AmHeaderPtr) data)) != AmTYPE_LINE) {
        return( NULL);
    }

    temp = GET_LINE( data);

    line = ( AmCadLinePtr) malloc( sizeof( AmCadLineRec));
    if ( line == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    memcpy( line, temp, sizeof( AmCadLineRec));

    return( line);
}


AmCadPolyLinePtr
AmGetPolyLine( AmMemoryPtr memory, long polyLineID)
{
    AmCadPolyLinePtr polyLine, temp;
    uchar_t        *data;
    int            size;

    data = AmGetData( memory, polyLineID);
    if ( data == NULL) {
        return( NULL);
    }

    if ( HTYPE((( AmHeaderPtr) data)) != AmTYPE_POLYLINE) {
        return( NULL);
    }

    temp = GET_POLYLINE( data);
    size = sizeof( AmCadPolyLineRec) + sizeof( long) * temp->numPoints;

    polyLine = ( AmCadPolyLinePtr) malloc( size);
    if ( polyLine == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    memcpy( polyLine, temp, size);

    return( polyLine);
}


AmCadPolygonPtr
AmGetPolygon( AmMemoryPtr memory, long polygonID)
{
    AmCadPolygonPtr polygon, temp;
    uchar_t       *data;
    int           size;

    data = AmGetData( memory, polygonID);
    if ( data == NULL) {
        return( NULL);
    }

    if ( HTYPE((( AmHeaderPtr) data)) != AmTYPE_POLYGON) {
        return( NULL);
    }

    temp = GET_POLYGON( data);
    size = sizeof( AmCadPolygonRec) + sizeof( long) * temp->AmNumLines;

    polygon = ( AmCadPolygonPtr) malloc( size);
    if ( polygon == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    memcpy( polygon, temp, size);

    return( polygon);
}

AmCadRefListPtr
AmGetReferences( AmMemoryPtr memory, long objectID)
{
    AmCadObjectPtr  object;
    uchar_t       *data;
    long          refID, size;
    AmCadRefListPtr refList, temp;

    data = AmGetData( memory, objectID); 
    if ( data == NULL) {
        return( NULL);
    }

    if ( HTYPE((( AmHeaderPtr) data)) != AmTYPE_POINT && 
         HTYPE((( AmHeaderPtr) data)) != AmTYPE_LINE &&
         HTYPE((( AmHeaderPtr) data)) != AmTYPE_POLYLINE &&
         HTYPE((( AmHeaderPtr) data)) != AmTYPE_POLYGON) {
        return( NULL);
    }

    object = GET_OBJECT( data);

    refID = object->references;
    if ( refID == AmINVALID) {
        return( NULL);
    }

    data = AmGetData( memory, refID);

    if ( data == NULL) {
        return( NULL);
    }

    temp = GET_REFERENCE_LIST( data);
    size = sizeof( AmCadRefListRec) + sizeof( long) * temp->numRefs;

    refList = ( AmCadRefListPtr) malloc( size);
    if ( refList == NULL) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( NULL);
    }

    memcpy( refList, temp, size);

    return( refList);
}

    
AmErrorCode
AmSetPoint( AmMemoryPtr memory, long pointID, AmCadPointPtr point)
{
    AmCadPointPtr temp;
    uchar_t     *data;

    data = AmGetData( memory, pointID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    if ( HTYPE((( AmHeaderPtr) data)) != AmTYPE_POINT) {
        return( ANVIL_ERROR_CODE);
    }

    temp = GET_POINT( data);
    memcpy( temp, point, sizeof( AmCadPointRec));

    AmChangedObject( memory, pointID);

    return( AmOK);
}


long
AmMakePoint( AmMemoryPtr memory,
           float     x,
           float     y)
{
    AmCadObjectPtr object;
    AmCadPointPtr  point;
    uchar_t      *data;
    long         ID, size;

    size = sizeof( AmCadObjectRec) + sizeof( AmCadPointRec);
    data = AmAllocObject( memory, size, AmTYPE_POINT, &ID);
    if ( data == NULL) {
        return( AmINVALID);
    }

    object = GET_OBJECT( data);
    object->references = AmINVALID;
    object->attributes = AmINVALID;

    point = GET_POINT( data);
    point->x = x;
    point->y = y;

    return( ID); 
}

long
AmMakeLine( AmMemoryPtr memory,
          long      p1,
          long      p2)
{
    AmCadObjectPtr object;
    AmCadLinePtr   line;
    uchar_t      *data;
    long         ID, size;

    size = sizeof( AmCadObjectRec) + sizeof( AmCadLineRec);
    data = AmAllocObject( memory, size, AmTYPE_LINE, &ID);
    if ( data == NULL) {
        return( AmINVALID);
    }

    object = GET_OBJECT( data);
    object->references = AmINVALID;
    object->attributes = AmINVALID;

    line = GET_LINE( data);
    line->p1 = p1;
    line->p2 = p2;

    if ( AmAddReference( memory, p1, ID) != AmOK) {
        AmFreeObject( memory, ID);
        return( AmINVALID);
    }

    if ( AmAddReference( memory, p2, ID) != AmOK) {
        AmRemoveReference( memory, p1, ID);
        AmFreeObject( memory, ID);
        return( AmINVALID);
    }

    return( ID); 
}


long
AmMakePolyLine( AmMemoryPtr memory,
              long      numPoints,
              long      *pointList)
{
    AmCadObjectPtr   object;
    AmCadPolyLinePtr polyline;
    uchar_t        *data;
    long           ID, size, *points, ii;

    if ( numPoints < 2) {
        ANVIL_ERROR_CODE = AmBAD_ARGUMENT;
        return( AmINVALID);
    }

    size = sizeof( AmCadObjectRec) + sizeof( AmCadPolyLineRec) +
           sizeof( long) * numPoints;
    data = AmAllocObject( memory, size, AmTYPE_POLYLINE, &ID);
    if ( data == NULL) {
        return( AmINVALID);
    }

    object = GET_OBJECT( data);
    object->references = AmINVALID;
    object->attributes = AmINVALID;

    polyline = GET_POLYLINE( data);
    polyline->numPoints = numPoints;

    points = ( long *) (( uchar_t *) polyline + sizeof( AmCadPolyLineRec));
    memcpy( points, pointList, sizeof( long) * numPoints);

    for ( ii = 0; ii < numPoints; ii++) {
        AmAddReference( memory, pointList[ ii], ID);
    }

    return( ID); 
}


long
AmMakePolygon( AmMemoryPtr memory,
             long      AmNumLines,
             long      *lineList)
{
    AmCadObjectPtr  object;
    AmCadPolygonPtr polygon;
    uchar_t       *data;
    long          ID, size, *lines, ii;

    if ( AmNumLines < 1) {
        ANVIL_ERROR_CODE = AmBAD_ARGUMENT;
        return( AmINVALID);
    }

    size = sizeof( AmCadObjectRec) + sizeof( AmCadPolygonRec) +
           sizeof( long) * AmNumLines;
    data = AmAllocObject( memory, size, AmTYPE_POLYGON, &ID);
    if ( data == NULL) {
        return( AmINVALID);
    }

    object = GET_OBJECT( data);
    object->references = AmINVALID;
    object->attributes = AmINVALID;

    polygon = GET_POLYGON( data);

    polygon->prev = memory->lastID;
    polygon->next = AmINVALID;

    memory->lastID = ID;

    polygon->AmNumLines = AmNumLines;

    lines = GET_POLYGON_DATA( data);

    memcpy( lines, lineList, sizeof( long) * AmNumLines);

    for ( ii = 0; ii < AmNumLines; ii++) {
        AmAddReference( memory, lineList[ ii], ID);
    }

    return( ID); 
}


AmErrorCode
AmRemovePoint( AmMemoryPtr memory,
             long      ID)
{
    AmCadObjectPtr object;
    uchar_t      *data;

    data = AmGetData( memory, ID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    object = GET_OBJECT( data);

    if ( object->references != AmINVALID) {
         RETURN_ERROR( AmOBJECT_STILL_REFERENCED);
    }

    return( AmFreeObject( memory, ID));
}

AmErrorCode
AmRemoveLine( AmMemoryPtr memory,
            long      ID)
{
    AmCadObjectPtr object;
    AmCadLinePtr   line;
    uchar_t      *data;
    long         p1, p2;

    data = AmGetData( memory, ID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    object = GET_OBJECT( data);

    if ( object->references != AmINVALID) {
         RETURN_ERROR( AmOBJECT_STILL_REFERENCED);
    }

    line = GET_LINE( data);

    p1 = line->p1;
    p2 = line->p2;
 
    if ( AmRemoveReference( memory, p1, ID) != AmOK) {
        return( ANVIL_ERROR_CODE);
    }

    if ( AmRemoveReference( memory, p2, ID) != AmOK) {
        return( ANVIL_ERROR_CODE);
    }

    return( AmFreeObject( memory, ID));
}


AmErrorCode
AmRemovePolyLine( AmMemoryPtr memory,
                long      ID)
{
    AmCadObjectPtr   object;
    AmCadPolyLinePtr polyLine;
    uchar_t        *data;
    long           *points;
    int            ii;

    data = AmGetData( memory, ID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    object = GET_OBJECT( data);

    if ( object->references != AmINVALID) {
         RETURN_ERROR( AmOBJECT_STILL_REFERENCED);
    }

    polyLine = GET_POLYLINE( data);
    points = GET_POLYLINE_DATA( data);

    for ( ii = 0; ii < polyLine->numPoints; ii++) {
        if ( AmRemoveReference( memory, points[ ii], ID) != AmOK) {
            return( ANVIL_ERROR_CODE);
        }
    }

    return( AmFreeObject( memory, ID));
}

AmErrorCode
SetPolygonNext( AmMemoryPtr memory,
                long      ID,
                long      nextID)
{
    AmCadObjectPtr   object;
    AmCadPolygonPtr  polygon;
    uchar_t        *data;

    data = AmGetData( memory, ID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    object = GET_OBJECT( data);

    if ( object->references != AmINVALID) {
         RETURN_ERROR( AmOBJECT_STILL_REFERENCED);
    }

    polygon = GET_POLYGON( data);

    polygon->next = nextID;

    return( AmOK);
}


AmErrorCode
SetPolygonPrev( AmMemoryPtr memory,
                long      ID,
                long      prevID)
{
    AmCadObjectPtr   object;
    AmCadPolygonPtr  polygon;
    uchar_t        *data;

    data = AmGetData( memory, ID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    object = GET_OBJECT( data);

    if ( object->references != AmINVALID) {
         RETURN_ERROR( AmOBJECT_STILL_REFERENCED);
    }

    polygon = GET_POLYGON( data);

    polygon->prev = prevID;

    return( AmOK);
}

AmErrorCode
AmRemovePolygon( AmMemoryPtr memory,
               long      ID)
{
    AmCadObjectPtr   object;
    AmCadPolygonPtr  polygon;
    uchar_t        *data;
    long           *lines;
    long           prev, next;
    int            ii;

    data = AmGetData( memory, ID);
    if ( data == NULL) {
        return( ANVIL_ERROR_CODE);
    }

    object = GET_OBJECT( data);

    if ( object->references != AmINVALID) {
         RETURN_ERROR( AmOBJECT_STILL_REFERENCED);
    }

    polygon = GET_POLYGON( data);

    prev = polygon->prev;
    next = polygon->next;

    lines = GET_POLYGON_DATA( data);

    for ( ii = 0; ii < polygon->AmNumLines; ii++) {
        if ( AmRemoveReference( memory, lines[ ii], ID) != AmOK) {
            return( ANVIL_ERROR_CODE);
        }
    }

    return( AmFreeObject( memory, ID));

    if ( prev != AmINVALID) {
        SetPolygonNext( memory, prev, next);
    } else {
        memory->firstID = next;
    }

    if ( next != AmINVALID) {
        SetPolygonPrev( memory, next, prev);
    } else {
        memory->lastID = prev;
    }
}
