#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <anvil/project.h>
#include <anvil/datafmt.h>
#include <util/shapefile.phil.h>

#define MAXRECORD	65536
#define MAXPARTS	2048
#define MAXPTS		16384

int main( int argc, char *argv[])
{
    shapeHeaderRec	header;
    int			fd, file, shapeType, pos, ii, jj;
    recordHeaderRec	recHeader;
    char		*record;
    shapeEnum		shapeCode;
    sPointRec		pts[ MAXPTS];
    arcRecordRec	arc;
    AmLineRec             lines[ MAXPTS];
    float               x0, y0, x1, y1;

    if ( argc < 3) {
	fprintf( stderr, "usage: %s infile outfile\n", argv[ 0]);
	exit( -1);
    }
    if (( fd = open( argv[ 1], O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open %s\n", argv[ 1]);
	exit( -1);
    }

    if (( file = open( argv[ 2], O_WRONLY | O_CREAT)) < 0) {
	fprintf( stderr, "failed to open %s\n", argv[ 2]);
	exit( -1);
    }

/* we allocate the record to ensure that it starts on a double word boundary */

    record = ( char *) malloc( MAXRECORD);

/* because of the alignment for doubles, and the file starting with 9 ints, the
   size of the header doesn't wind up being exactly 100 bytes, so we force it */

    if ( read( fd, &header, FILE_HEADER_LENGTH) < FILE_HEADER_LENGTH) {
	fprintf( stderr, "failed to read header from %s\n", argv[ 1]);
	exit( -1);
    }
    BIG_LONG( header.fileCode);
    LITTLE_LONG( header.version);
    LITTLE_LONG( header.shapeType);
    LITTLE_DOUBLE( header.xmin);
    LITTLE_DOUBLE( header.ymin);
    LITTLE_DOUBLE( header.xmax);
    LITTLE_DOUBLE( header.ymax);
    printf( "fileCode = %d, length = %d, version = %d\n", 
	    header.fileCode, header.fileLength, header.version);
    printf( "xmin = %.2f, ymin = %.2f, xmax = %.2f, ymax = %.2f\n",
	    header.xmin, header.ymin,
	    header.xmax, header.ymax);
    pos = FILE_HEADER_LENGTH;

    jj = 0;

    do {
	if ( read( fd, &recHeader, sizeof( recHeader)) < sizeof( recHeader)) {
	    break;
	}
	BIG_LONG( recHeader.recordNumber);
	BIG_LONG( recHeader.contentLength);
	if ( read( fd, record, recHeader.contentLength*2) < 
	     recHeader.contentLength*2) {
	    fprintf( stderr, "failed to read %d bytes at %d\n", 
		     recHeader.contentLength*2, pos);
	    break;
	}
	pos += recHeader.contentLength*2;
	shapeType = *( int *) record;
	LITTLE_LONG( shapeType);
	shapeCode = ( shapeEnum) shapeType;
	switch ( shapeCode) {
	case AmNULL_SHAPE:
	    printf( "null shape\n");
	    break;
	case AmPOINT_SHAPE:
	    break;
	case AmARC_SHAPE:

/* copy to the arc structure to ensure double word alignment */

	    memcpy( &arc, record, sizeof( arcRecordRec));
	    LITTLE_LONG( arc.shapeType);
	    LITTLE_LONG( arc.unused1);
	    LITTLE_DOUBLE( arc.xmin);
	    LITTLE_DOUBLE( arc.ymin);
	    LITTLE_DOUBLE( arc.xmax);
	    LITTLE_DOUBLE( arc.ymax);
	    LITTLE_LONG( arc.numPoints);
	    LITTLE_LONG( arc.numParts);

/* memcpy the points records to ensure double word alignment. */ 

	    memcpy( pts, &record[ ARC_HEADER_LENGTH], arc.numPoints*sizeof( sPointRec));
	    for ( ii = 0; ii < arc.numPoints; ii++) {
		LITTLE_DOUBLE( pts[ ii].x);
		LITTLE_DOUBLE( pts[ ii].y);
	    }

            x0 = pts[ 0].x;
            y0 = pts[ 0].y;

	    for ( ii = 1; ii < arc.numPoints; ii++) {
                x1 = pts[ ii].x;
                y1 = pts[ ii].y;
                lines[ jj].x0 = x0;
                lines[ jj].y0 = y0;
                lines[ jj].x1 = x1;
                lines[ jj].y1 = y1;

                jj++;
                if ( jj == MAXPTS) {
                    write( file, lines, sizeof( AmLineRec) * MAXPTS);
                    jj = 0;
                }

                x0 = x1;
                y0 = y1;
	    }

	    break;
	default:
	    printf( "unknown shape type %d\n", shapeCode);
	    break;
	}
    } while ( TRUE);

    if ( jj > 0) {
        write( file, lines, sizeof( AmLineRec) * jj);
    }

    close( file);
}
