#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>

#include <anvil/project.h>

#include <util/shapefile.h>

#include <anvil/cad/defs.h>
#include <anvil/cad/mem.h>
#include <anvil/cad/segmentProto.h>
#include <anvil/cad/objectProto.h>

#define MAXRECORD	65536
#define MAXPARTS	2048
#define MAXPTS		16384

#define NUM_SEGS	1000
#define NUM_OBJS	50000
#define HASH_SIZE	73
#define CACHE_SIZE	10

int main( int argc, char *argv[])
{
    AmMemoryPtr           memory;
    shapeHeaderRec	header;
    int			fd, shapeType, pos, ii, parts[ MAXPARTS], AmNumLines;
    recordHeaderRec	recHeader;
    char		*record;
    shapeEnum		shapeCode;
    shapePointRec	pts[ MAXPTS];
    pointRecordRec	point;
    multiPointRecordRec	multiPoint;
    arcRecordRec	arc;
    polygonRecordRec	polygon;
    double		xmin, ymin, xmax, ymax;
    long                poly, lines[ MAXPTS], polyLine, p[ MAXPTS], start;

    if ( argc < 3) {
	fprintf( stderr, "usage: %s infile outfile\n", argv[ 0]);
	exit( -1);
    }
    if (( fd = open( argv[ 1], O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open %s\n", argv[ 1]);
	exit( -1);
    }

/* Create the memory structure for the objects. */

    memory = AmMakeMemory( NUM_SEGS, NUM_OBJS, CACHE_SIZE, HASH_SIZE, argv[ 2]);

/* we allocate the record to ensure that it starts on a double word boundary */

    record = ( char *) malloc( MAXRECORD);

/* because of the alignment for doubles, and the file starting with 9 ints, the
   size of the header doesn't wind up being exactly 100 bytes, so we force it */

    if ( read( fd, &header, FILE_HEADER_LENGTH) < FILE_HEADER_LENGTH) {
	fprintf( stderr, "failed to read header from %s\n", argv[ 1]);
	exit( -1);
    }
    BIG_LONG( header.fileCode);
    LITTLE_LONG( header.version);
    LITTLE_LONG( header.shapeType);
    LITTLE_DOUBLE( header.xmin);
    LITTLE_DOUBLE( header.ymin);
    LITTLE_DOUBLE( header.xmax);
    LITTLE_DOUBLE( header.ymax);
    printf( "fileCode = %d, length = %d, version = %d\n", 
	    header.fileCode, header.fileLength, header.version);
    printf( "xmin = %.2f, ymin = %.2f, xmax = %.2f, ymax = %.2f\n",
	    header.xmin, header.ymin,
	    header.xmax, header.ymax);
    pos = FILE_HEADER_LENGTH;
    do {
	if ( read( fd, &recHeader, sizeof( recHeader)) < sizeof( recHeader)) {
	    break;
	}
	BIG_LONG( recHeader.recordNumber);
	BIG_LONG( recHeader.contentLength);
//	printf( "record = %d, length = %d\n",
//		recHeader.recordNumber, recHeader.contentLength);
        if ( recHeader.contentLength * 2 > MAXRECORD) {
            fprintf( stderr, "attempting too read too many bytes (%d)\n",
                recHeader.contentLength * 2);
                break;
        }
	if ( read( fd, record, recHeader.contentLength*2) < 
	     recHeader.contentLength*2) {
	    fprintf( stderr, "failed to read %d bytes at %d\n", 
		     recHeader.contentLength*2, pos);
	    break;
	}
	pos += recHeader.contentLength*2;
	shapeType = *( int *) record;
	LITTLE_LONG( shapeType);
	shapeCode = ( shapeEnum) shapeType;
	switch ( shapeCode) {
	case AmNULL_SHAPE:
	    printf( "null shape\n");
	    break;
	case AmPOINT_SHAPE:

/* copy to the point structure to ensure double word alignment */

	    memcpy( &point, record, sizeof( pointRecordRec));
	    LITTLE_DOUBLE( point.x);
	    LITTLE_DOUBLE( point.y);
	    printf( "point: %.2f, %.2f\n", point.x, point.y);
	    break;
	case AmARC_SHAPE:

/* copy to the arc structure to ensure double word alignment */

	    memcpy( &arc, record, sizeof( arcRecordRec));
	    LITTLE_LONG( arc.shapeType);
	    LITTLE_LONG( arc.unused1);
//	    printf( "type = %d\n", arc.shapeType);
	    LITTLE_DOUBLE( arc.xmin);
	    LITTLE_DOUBLE( arc.ymin);
	    LITTLE_DOUBLE( arc.xmax);
	    LITTLE_DOUBLE( arc.ymax);
	    LITTLE_LONG( arc.numPoints);
	    LITTLE_LONG( arc.numParts);

//	    printf( "arc %d parts %d points, bbox = (%.2f, %.2f, %.2f, %.2f)\n",
//		    arc.numParts, arc.numPoints, 
//		    arc.xmin, arc.ymin, arc.xmax, arc.ymax);

/* memcpy the points records to ensure double word alignment. */ 

	    memcpy( pts, &record[ ARC_HEADER_LENGTH],
                    arc.numPoints*sizeof( shapePointRec));

	    for ( ii = 0; ii < arc.numPoints; ii++) {
		LITTLE_DOUBLE( pts[ ii].x);
		LITTLE_DOUBLE( pts[ ii].y);
                p[ ii] = AmMakePoint( memory, ( float) pts[ ii].x,
                             ( float) pts[ ii].y);
                if ( p[ ii] == AmINVALID) {
                    fprintf( stderr, "Warning: could not create point.\n");
                }
	    }

            polyLine = AmMakePolyLine( memory, arc.numPoints, p);
            if ( polyLine == AmINVALID) {
                fprintf( stderr, "Warning: could not create poly line.\n");
            } else {
                fprintf( stderr, "Created polyline.\n");
            }

#ifdef UNDEFINED
  
/* memcpy the parts records to ensure word alignment */

	    memcpy( parts, 
		    &record[ ARC_HEADER_LENGTH] + sizeof( shapePointRec)*arc.numPoints,
		    arc.numParts*sizeof( int));
	    for ( ii = 0; ii < arc.numParts; ii++) {
		LITTLE_LONG( parts[ ii]);
	    }
	    for ( ii = 0; ii < arc.numParts; ii++) {
		printf( "%d ", parts[ ii]);
	    }
	    printf( "\n");

#endif

	    break;
	case AmPOLYGON_SHAPE:

/* copy to the polygon structure to ensure double word alignment */

	    memcpy( &polygon, record, sizeof( polygonRecordRec));
	    LITTLE_DOUBLE( polygon.xmin);
	    LITTLE_DOUBLE( polygon.ymin);
	    LITTLE_DOUBLE( polygon.xmax);
	    LITTLE_DOUBLE( polygon.ymax);
	    LITTLE_LONG( polygon.numParts);
	    LITTLE_LONG( polygon.numPoints);
	    printf( "polygon %d parts %d points, bbox = (%.2f, %.2f, %.2f, %.2f)\n",
		    polygon.numParts, polygon.numPoints, 
		    polygon.xmin, polygon.ymin, polygon.xmax, polygon.ymax);

/* memcpy the parts record to ensure word alignment */

	    memcpy( parts, record + sizeof( polygonRecordRec),
		    polygon.numParts*sizeof( int));
	    for ( ii = 0; ii < polygon.numParts; ii++) {
		LITTLE_LONG( parts[ ii]);
	    }
	    for ( ii = 0; ii < polygon.numParts; ii++) {
		printf( "%d ", parts[ ii]);
	    }

/* memcpy the pts record to ensure double alignment */

	    memcpy( pts, record + sizeof( polygonRecordRec) + 
			 polygon.numParts*sizeof( int) + sizeof( int),
		    polygon.numPoints*sizeof( shapePointRec));
	    for ( ii = 0; ii < polygon.numPoints; ii++) {
		LITTLE_DOUBLE( pts[ ii].x);
		LITTLE_DOUBLE( pts[ ii].y);
	    }

            p[ 0] = AmMakePoint( memory, ( float) pts[ 0].x, ( float) pts[ 0].y);
            fprintf( stderr, "Made Point: %u\n", p[ 0]);

	    for ( ii = 1; ii < polygon.numPoints; ii++) {
//		printf( "(%.2f, %.2f)", pts[ ii].x, pts[ ii].y);
                p[ 1] = AmMakePoint( memory, ( float) pts[ ii].x,
                         ( float) pts[ ii].y);
                fprintf( stderr, "Made Point: %u\n", p[ 1]);
                lines[ ii - 1] = AmMakePolyLine( memory, 2, p);
                fprintf( stderr, "Made PolyLine: %u\n", lines[ ii - 1]);
                p[ 0] = p[ 1];
	    }
	    printf( "\n");

            for ( ii = 0; ii < polygon.numParts; ii++) {
                if ( ii == polygon.numParts - 1) {
                    AmNumLines = polygon.numPoints - 1 - parts[ ii];
                } else {
                    AmNumLines = parts[ ii + 1] - parts[ ii] + 1;
                }
                poly = AmMakePolygon( memory, AmNumLines, &lines[ parts[ ii]]);
                if ( poly == AmINVALID) {
                    fprintf( stderr, "Warning: could not create polgon.\n");
                } else {
                    fprintf( stderr, "Made Polygon: %u\n", poly);
                }
            }
 
            if ( polygon.numParts == 0) {
                poly = AmMakePolygon( memory, polygon.numPoints - 1, lines);
                if ( poly == AmINVALID) {
                    fprintf( stderr, "Warning: could not create polgon.\n");
                } else {
                    fprintf( stderr, "Made Polygon: %u\n", poly);
                }
            }

	    break;
	case AmMULTIPOINT_SHAPE:

/* copy to the multipoint structure to ensure double word alignment */

	    memcpy( &multiPoint, record, sizeof( multiPointRecordRec));
	    LITTLE_DOUBLE( multiPoint.xmin);
	    LITTLE_DOUBLE( multiPoint.ymin);
	    LITTLE_DOUBLE( multiPoint.xmax);
	    LITTLE_DOUBLE( multiPoint.ymax);
	    LITTLE_LONG( multiPoint.numPoints);
	    printf( "multipoint: %d points, bbox = (%.2f, %.2f, %.2f, %.2f)\n",
		    multiPoint.numPoints,
		    multiPoint.xmin, multiPoint.ymin,
		    multiPoint.xmax, multiPoint.ymax);

/* memcpy the pts record to ensure double alignment */

	    memcpy( pts, record + sizeof( multiPointRecordRec),
		    multiPoint.numPoints*sizeof( shapePointRec));
	    for ( ii = 0; ii < multiPoint.numPoints; ii++) {
		LITTLE_DOUBLE( pts[ ii].x);
		LITTLE_DOUBLE( pts[ ii].y);
	    }
	    for ( ii = 0; ii < multiPoint.numPoints; ii++) {
		printf( "(%.2f, %.2f)", pts[ ii].x, pts[ ii].y);
	    }
	    printf( "\n");
	    break;
	default:
	    printf( "unknown shape type %d\n", shapeCode);
	    break;
	}
    } while ( TRUE);

/* Write out everything. */

    AmDumpMemory( memory);

    long total = 0;
    for ( ii = 0; ii < memory->segTable->AmLast; ii++) {
        total += AmTotalFreeSpace( &memory->segTable->segments[ ii]);
    }
    fprintf( stderr, "For %d segments there are %d bytes of free space.\n",
        memory->segTable->AmLast, total);
    fprintf( stderr, "    That's a usage of %0.2f%% (%d out of %d bytes).\n",
        (1.00 - total / (( float) memory->segTable->AmLast * SEGMENT_SIZE)) * 100,
        ( memory->segTable->AmLast * SEGMENT_SIZE) - total,
        ( memory->segTable->AmLast * SEGMENT_SIZE));
}
