#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* image color dithering routines, shamelessly plagarized from the gimp code */

#include <stdio.h>
#include <anvil/project.h>
#include <anvil/colorcell.h>
#include <anvil/macros.h>
#include <anvil/dither.h>

static uchar_t DM[ DITHER_LEVEL][ DITHER_LEVEL] = {
    { 0,  32, 8,  40, 2,  34, 10, 42 },
    { 48, 16, 56, 24, 50, 18, 58, 26 },
    { 12, 44, 4,  36, 14, 46, 6,  38 },
    { 60, 28, 52, 20, 62, 30, 54, 22 },
    { 3,  35, 11, 43, 1,  33, 9,  41 },
    { 51, 19, 59, 27, 49, 17, 57, 25 },
    { 15, 47, 7,  39, 13, 45, 5,  37 },
    { 63, 31, 55, 23, 61, 29, 53, 21 }
};

/**************************************************************************
 *
 * Description:
 *	initialize the dithering matrices. takes the number of shades 
 *	for red, green, and blue, generates a dithering decision matrix
 *	and the dither matrices for red, green, and blue.
 * Arguments:
 *	numRedShades, numGreenShades, numBlueShades - number fo shades
 *	to create the dither matrices for.
 *	descision matrix - dithering decision matrix.
 *	redMatrix, blueMatrix, greenMatrix - resulting dither matrices,
 *	preallocated.
 * Returns:
 *	AmOK
 * Modifications:
 *	The hardcoded shifts for the color dither matrices make me queasy.
 *	This routine could probably use some more comments, eh?
 *
 *************************************************************************/



AmErrorCode
AmInitDitherMatrices( ushort_t numRedShades,
		    ushort_t numGreenShades,
		    ushort_t numBlueShades,
		    uchar_t	 decisionMatrix[ NUM_DITHER_MATRICES][ DITHER_LEVEL][ DITHER_LEVEL],
		    ulong_t	 *redDitherMatrix,
		    ulong_t	 *greenDitherMatrix,
		    ulong_t	 *blueDitherMatrix)
{
    int      ii, jj, kk;
    uchar_t  lowShade, highShade; 
    ushort_t index;
    ulong_t  redMult, greenMult;
    float    redColsPerShade, blueColsPerShade, greenColsPerShade;

    redMult = numGreenShades * numBlueShades;
    greenMult = numBlueShades;

    redColsPerShade = 255 / ( numRedShades - 1);
    greenColsPerShade = 255 / ( numGreenShades - 1);
    blueColsPerShade = 255 / ( numBlueShades - 1);

    for ( ii = 0; ii < NUM_DITHER_MATRICES; ii++) {
        for ( jj = 0; jj < DITHER_LEVEL; jj++) {
            for ( kk = 0; kk < DITHER_LEVEL; kk++) {
                decisionMatrix[ ii][ jj][ kk] = ( DM[ jj][ kk] < ii) ? 1 : 0;
            }
        }
    }

    for ( ii = 0; ii < 256; ii++) {
        lowShade = ( uchar_t) ( ii / redColsPerShade);
        highShade = lowShade + 1;
        if ( highShade > numRedShades - 1) {
            highShade = numRedShades - 1; 
        }

        index = ( ushort_t) ((( ii - lowShade * redColsPerShade) /
                            redColsPerShade) * NUM_DITHER_MATRICES);

        lowShade *= redMult;
        highShade *= redMult;

        redDitherMatrix[ ii] = ( index << 16) + ( highShade << 8) + ( lowShade);
    }

    for ( ii = 0; ii < 256; ii++) {
        lowShade = ( uchar_t) ( ii / greenColsPerShade);
        highShade = lowShade + 1;
        if ( highShade > numGreenShades - 1) {
            highShade = numGreenShades - 1; 
        }

        index = ( ushort_t) ((( ii - lowShade * greenColsPerShade) /
                            greenColsPerShade) * NUM_DITHER_MATRICES);

        lowShade *= greenMult;
        highShade *= greenMult;

        greenDitherMatrix[ ii] = ( index << 16) + ( highShade << 8) +
                                 ( lowShade);
    }

    for ( ii = 0; ii < 256; ii++) {
        lowShade = ( uchar_t) ( ii / blueColsPerShade);
        highShade = lowShade + 1;
        if ( highShade > numBlueShades - 1) {
            highShade = numBlueShades - 1; 
        }

        index = ( ushort_t) ((( ii - lowShade * blueColsPerShade) /
                            blueColsPerShade) * NUM_DITHER_MATRICES);

        blueDitherMatrix[ ii] = ( index << 16) + ( highShade << 8) +
                                ( lowShade);
    }

    return( AmOK);
}

/**************************************************************************
 *
 * Description:
 *     	This function creates and fills the color cube used for 8 bit
 *     	displays in conjunction with ordered dithering.
 * Arguments:
 *     	cubeColors - resulting vector of cube colors.
 *     	numRed, numGreen, numBlue - number of red, green, and blue
 *  	values to generate the cube with.
 * Returns:
 *	AmOK
 *
 *************************************************************************/

AmErrorCode
AmCreateColorCube( AmColorCellPtr   cubeColors,
                 uint_t   	numRed,
                 uint_t   	numGreen,
                 uint_t   	numBlue,
		 ulong_t	*redOffset,
		 ulong_t	*greenOffset,
		 ulong_t	*blueOffset)
{
    uint_t   numTotal, ii, r, g, b, dr, dg, db;

    AN_DEBUG( 90, ( "Creating color cube...\n"));

/* Save some information about the color-cube. */

    numTotal = numRed * numGreen * numBlue;

/* Get the pixel values for the default colormap of the screen. */

    dr = ( numRed > 1) ? numRed - 1 : 1;
    dg = ( numGreen > 1) ? numGreen - 1 : 1;
    db = ( numBlue > 1) ? numBlue - 1 : 1;

    ii = 0;
    for ( r = 0; r < numRed; r++) {
        for ( g  = 0; g < numGreen; g++) {
            for ( b  = 0; b < numBlue; b++) {
                cubeColors[ ii].pixel = ii;
                cubeColors[ ii].red = ( uint_t) r * 255 / dr;
                cubeColors[ ii].green = ( uint_t) g * 255 / dg;
                cubeColors[ ii].blue = ( uint_t) b * 255 / db;
                ii++;
            }
        }
    }

    for ( ii = 0; ii < 256; ii++) {
        redOffset[ ii] = ( int) ( ii * dr / 255.0 + 0.5) *
                         ( numGreen * numBlue);
        greenOffset[ ii] = ( int) ( ii * dg / 255.0 + 0.5) * ( numBlue);
        blueOffset[ ii] = ( int) ( ii * db / 255.0 + 0.5);
    }
    return( AmOK);
}

