#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




/* base member class function implementations for graphics contexts, which
   are passed as arguments to draw routines to set graphics attributes
   such as foreground and background colors, line widths and raster
   operations */

#include <anvil/display.h>

/****************************************************************************
 *
 * Description:
 * 	AmGCClass default constructor: initialize the window and display 
 *	that the GC was created on to NULL
 *
 ****************************************************************************/

AmGCClass::AmGCClass()
{
    _pWindow = NULL;
    _pDisplay = NULL;
    _fillPatFile = NULL;
}

/****************************************************************************
 *
 * Description:
 * 	AmGCClass constructor: initialize the window and display 
 *	to the window and display the window was created on.
 * Arguments:
 *	pWindow - window the GC was created on.
 *
 ****************************************************************************/

AmGCClass::AmGCClass( AmWindowClass *pWindow)
{
    _pWindow = pWindow;
    _pDisplay = pWindow->GetDisplay();
    _fillPatFile = NULL;

    _pWindow->AddGC( this);
}

/****************************************************************************
 *
 * Description:
 *	AmGCClass destructor: remove the GC from the window's list of
 *	GC's
 *
 ****************************************************************************/

AmGCClass::~AmGCClass()
{
    if ( _pWindow != NULL) {
        _pWindow->RemoveGC( this);
    }
    if ( _fillPatFile != NULL) {
	FREE( _fillPatFile);
    }
}

/****************************************************************************
 *
 * Description:
 *	set the window and display to NULL.
 * Arguments:
 *	pWindow - window to remove.
 * Modifications:
 *	This function doesn't seem to be used anywhere, and should
 *	be removed.
 *
 ****************************************************************************/

void
AmGCClass::RemoveWindow( AmWindowClass *pWindow)
{
    if ( _pWindow == pWindow) {
        _pWindow = NULL;
    }
   
    _pDisplay = NULL;
}

/****************************************************************************
 *
 * Description:
 *	wrapper function Set the clip retangle for the GC, calls the
 *	virtual SetClipRectangles() function.
 * Arguments:
 *	x, y, width, height - dimensions of the clip rectangle.
 *
 ****************************************************************************/

void 
AmGCClass::SetClipRectangle( int	x,
			   int	y,
			   int	width,
			   int	height)
{
    AmRectangleRec clipRect;

    clipRect.x = x;
    clipRect.y = y;
    clipRect.width = width;
    clipRect.height = height;
    SetClipRectangles( &clipRect, 1);
}
