#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <stdlib.h>
#include <anvil/coordinates.h>
#include <anvil/outline.h>
#include <anvil/dispmgrargs.h>
#include <anvil/mapmgr.h>

/* create the motif plot args for passing to handlers */

AmDispMgrArgsPtr
AmMakeDispMgrArgs( AmMapManagerClass *displayManager,
                 AmCoordClass      *coords,
                 AmOutlineClass    *outline)
{
    AmDispMgrArgsPtr dispMgrArgs;

    dispMgrArgs = ( AmDispMgrArgsPtr) malloc( sizeof( AmDispMgrArgsRec));
    if ( dispMgrArgs == NULL) {
    	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

    dispMgrArgs->displayManager = displayManager;
    dispMgrArgs->coords = coords;
    dispMgrArgs->outline = outline;
    dispMgrArgs->flag = FALSE;

    return( dispMgrArgs);
}

/* function to free the temporary data structure passed when the cursor 
   is completed or terminated */

AmErrorCode
AmFreeDispMgrArgs( AmDispMgrArgsPtr dispMgrArgs)
{
    if ( dispMgrArgs == NULL) {
        DO_RETURN_ERROR( AmFREE_MEMORY);
    }
    free( dispMgrArgs);
    return( AmOK);
}
/* suspend, restart, and remove function pointers */

void
AmSuspendCursorHandler( AmDispMgrArgsPtr dispMgrArgs)
{
    AmMapManagerClass  *displayManager = dispMgrArgs->displayManager;
    AmOutlineClass     *outline = dispMgrArgs->outline;

    if ( outline != NULL) {
        displayManager->HideOutline( outline);
    }
}

void
AmRestartCursorHandler( AmDispMgrArgsPtr       dispMgrArgs)
{
    AmMapManagerClass  *displayManager = dispMgrArgs->displayManager;
    AmOutlineClass     *outline = dispMgrArgs->outline;

    displayManager->WriteResourceMessage( "restartCursorHandlerMsg");
    if ( outline != NULL) {
        displayManager->ShowOutline( outline);
    }
}

void
AmRemoveCursorHandler( AmDispMgrArgsPtr        dispMgrArgs)
{
    AmMapManagerClass    *displayManager = ( AmMapManagerClass *)
                                            dispMgrArgs->displayManager;
    AmOutlineClass       *outline = dispMgrArgs->outline;

    if ( outline != NULL) {

/* RemoveOutline deletes the outline as well as removing it */

        displayManager->RemoveOutline( outline);
    }
    AmFreeDispMgrArgs( dispMgrArgs);
}

