#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#if !defined(_WIN32) && !defined(__BSD44__)
#include <alloca.h>
#else 
#include <malloc.h>
#endif
#include <anvil/screenobj.h>
#include <anvil/list.h>
#include <anvil/mapmgr.h>
#include <anvil/macros.h>

#define MAXLINES 	2048
#define MAXPOINTS 	2048

AnvilBoolean
AmLineVisible( int                x0,
             int                y0,
             int                x1,
             int                y1,
             AmRectangleRec const *vClipRects,
             int                nClipRects)
{
    int ii;

    for ( ii = 0; ii < nClipRects; ii++) {
         if (( x0 < vClipRects[ ii].x) &&
             ( x1 < vClipRects[ ii].x)) {
             continue;
         }

         if (( x0 > vClipRects[ ii].x + vClipRects[ ii].width) &&
             ( x1 > vClipRects[ ii].x + vClipRects[ ii].width)) {
             continue;
         }

         if (( y0 < vClipRects[ ii].y) &&
             ( y1 < vClipRects[ ii].y)) {
             continue;
         }

         if (( y0 > vClipRects[ ii].y + vClipRects[ ii].height) &&
             ( y1 > vClipRects[ ii].y + vClipRects[ ii].height)) {
             continue;
         }

        break;
    }

    return( ii < nClipRects);
}


AnvilBoolean
AmPointVisible( int                x,
              int                y,
              AmRectangleRec const *vClipRects,
              int                nClipRects)
{
    int ii;

    for ( ii = 0; ii < nClipRects; ii++) {
        if (( x >= vClipRects[ ii].x) &&
            ( x <= vClipRects[ ii].x + vClipRects[ ii].width) &&
            ( y >= vClipRects[ ii].y) &&
            ( y <= vClipRects[ ii].y + vClipRects[ ii].height)) {
 
            return( TRUE);
        }
    }

    return( FALSE);
}

/*
 * Function: void flushDrawLines( pDisplay, pWindow, pGC, pSegment, pnSegment)
 * 
 * Purpose:
 *   Flushes the segment cache and draws the lines on the window. 
 *
 * Parameters:
 *   AmDisplayClass *pDisplay : The display.
 *   AmWindowClass  *pWindow  : The window to draw to.
 *   AmGCClass      *pGC      : The GC to use when drawing. 
 *   AmSegmentPtr   pSegment  : The array of segments to draw.
 *   int          *pnSegment: The number of segments in the array.
 *
 * Description:
 *   flushDrawLines draws all the segments in the array of segments (pSegment)
 *   and resets the segment counter (pnSegment) to zero.  The passed in
 *   display, window, and GC are used for drawing.
 *
 * Returns:
 *   -nothing-
 */

static void
flushDrawLines( AmDisplayClass *pDisplay,
                AmWindowClass  *pWindow,
                AmGCClass      *pGC,
                AmSegmentPtr   pSegment,
                int          *pnSegment)
{
    if ( *pnSegment > 0) {
        pDisplay->DrawSegments( pWindow, pGC, pSegment, *pnSegment);
        *pnSegment = 0;
    }
}


/*
 * Function: void drawLine( pDisplay,
 *                          pWindow,
 *                          pGC,
 *                          x0,
 *                          y0,
 *                          x1,
 *                          y1,
 *                          pSegment,
 *                          pnSegment)
 *
 * Purpose:
 *   Draws a segment on the window.  Remember to call flushDrawLine
 *   after all segments are drawn.
 *
 * Parameters:
 *   AmDisplayClass *pDisplay : The display.
 *   AmWindowClass  *pWindow  : The window to draw to.
 *   AmGCClass      *pGC      : The GC to use when drawing. 
 *   int          x0        : The x coordinate of the starting point. 
 *   int          y0        : The y coordinate of the starting point. 
 *   int          x1        : The x coordinate of the end point. 
 *   int          y1        : The y coordinate of the end point. 
 *   AmSegmentPtr   pSegment  : The array of segments.
 *   int          *pnSegment: The number of segments in the array.
 *
 * Description:
 *   drawLine puts the line given by the two points <x0,y0> and <x1,y1> 
 *   into the array of segment (pSegment).  When the array is full, 
 *   the function draws them to the specified window (pWindow) using the 
 *   passed in GC (pGC).
 *
 *   Note that after all lines have been drawn using drawLine the
 *   flushDrawLines function should be called to make sure that all
 *   the lines in the segment array (pSegment) get drawn.
 *
 * Returns:
 *   -nothing-
 */

static void
drawLine( AmDisplayClass *pDisplay,
          AmWindowClass  *pWindow,
          AmGCClass      *pGC,
          int          x0,
          int          y0,
          int          x1,
          int          y1,
          AmSegmentPtr   pSegment,
          int          *pnSegment)
{
    pSegment[ *pnSegment].x0 = x0;
    pSegment[ *pnSegment].y0 = y0;
    pSegment[ *pnSegment].x1 = x1;
    pSegment[ *pnSegment].y1 = y1;
    ( *pnSegment)++;

    if ( *pnSegment == MAXLINES) {
        pDisplay->DrawSegments( pWindow, pGC, pSegment, *pnSegment);
        *pnSegment = 0;
    }
}


/*
 * Function: void flushDrawPoints( pDisplay, pWindow, pGC, pPoint, pnPoint)
 *
 * Purpose:
 *   Flushes the point cache and draws the points to the window.
 *
 * Parameters:
 *   AmDisplayClass *pDisplay: The display.
 *   AmWindowClass  *pWindow : The window to draw to.
 *   AmGCClass      *pGC     : The GC to use when drawing. 
 *   AmPointPtr     pPoint   : The array of points to draw.
 *   int            *pnPoint : The number of points in the array.
 *
 * Description:
 *   flushDrawPoints draws all the points in the array of points (pPoint)
 *   and resets the point counter (pnPoint) to zero.  The passed in
 *   display, window, and GC are used for drawing.
 *
 * Returns:
 *   -nothing-
 */

static void
flushDrawPoints( AmDisplayClass *pDisplay,
                 AmWindowClass  *pWindow,
                 AmGCClass      *pGC,
                 AmPointPtr     pPoint,
                 int            *pnPoint)
{
    if ( *pnPoint > 0) {
        pDisplay->DrawPoints( pWindow, pGC, pPoint, *pnPoint);
        *pnPoint = 0;
    }
}


/*
 * Function: void drawPoint( pDisplay,
 *                           pWindow,
 *                           pGC,
 *                           x0,
 *                           y0,
 *                           pPoint,
 *                           pnPoint)
 *
 * Purpose:
 *   Draws a point on the window.  Remember to call flushDrawPoints
 *   after all points are drawn.
 *
 * Parameters:
 *   AmDisplayClass *pDisplay: The display.
 *   AmWindowClass  *pWindow : The window to draw to.
 *   AmGCClass      *pGC     : The GC to use when drawing. 
 *   int          x0       : The x coordinate of the point to draw. 
 *   int          y0       : The y coordinate of the point to draw. 
 *   Point        *pPoint  : The "cache" array of points.
 *   int          *pnPoint : The number of points in the "cache" array.
 *
 * Description:
 *   drawPoint puts the point given by <x0,y0> into the array of points
 *   (pPoint).  When the array is full, the function draws the points to
 *   the specified window (pWindow) using the passed in GC (pGC).
 *
 *   Note that after all points have been drawn using drawPoint the
 *   flushDrawPoints function should be called to make sure that all
 *   the points in the point array (pPoint) get drawn.
 *
 * Returns:
 *   -nothing-
 */

static void
drawPoint( AmDisplayClass 	*pDisplay,
           AmWindowClass  	*pWindow,
           AmGCClass      	*pGC,
           int          	x0,
           int          	y0,
           AmPointPtr        	pPoint,
           int          *	pnPoint)
{
    pPoint[ *pnPoint].x = x0;
    pPoint[ *pnPoint].y = y0;
    ( *pnPoint)++;

    if ( *pnPoint == MAXPOINTS) {
        pDisplay->DrawPoints( pWindow, pGC, pPoint, *pnPoint);
        *pnPoint = 0;
    }
}

/**************************************************************************
 *
 * Function: ClearArea( vClearRects, nClearRects)
 * 
 * Parameters:
 *   AmRectangleRec const *vClearRects : The rectangular regions to be cleared
 *                                     (can be NULL).
 *   int                nClearRects  : The number of rectangles (can be 0).
 *
 * Description:
 *   ClearArea clears an area of rectangluar regions in the window or buffer.
 *   If the grid is turned on (as indicated by the "grid" variable) then the
 *   cleared area is also filled with grid marks spaced according to the
 *   gridSpacing variable.
 *
 * Returns:
 *   -nothing-
 *
 **************************************************************************/

void
AmMapManagerClass::ClearArea( AmRectangleRec const *vClearRects,
                            int                nClearRects)
{

/* Do nothing if the window isn't valid yet... */
 
    if ( !_pWindow->IsValid()) {
        return;
    }

    AmDisplayManagerClass::ClearArea( vClearRects, nClearRects);

/* If the grid is turned on, draw it now. */

    if ( _fGrid) {
        DrawGrid( vClearRects, nClearRects);
    }
}

/**************************************************************************
 *
 * Function: DrawGrid( vClipRects, nClipRects)
 * 
 * Parameters:
 *   AmRectangleRec const *vClipRects : The region to be cleared.
 *   int                nClipRects  : The number of rectangles in the region.
 *
 * Description:
 *   Draws the grid over the designated area.  The area is also filled 
 *   with grid marks spaced according to the gridSpacing variable.
 *
 * Returns:
 *   -nothing-
 *
 **************************************************************************/

void 
AmMapManagerClass::DrawGrid( AmRectangleRec const *vClipRects,
                           int                nClipRects)
{
    AmWindowClass         	*pWindow;
    AmSegmentRec   		pSegment[ MAXLINES];
    AmPointRec        		pPoint[ MAXPOINTS];
    AmRectangleRec const 	*vTempRects;
    AmColorCellRec 		col;
    int          		ii, jj, nSegment, nPoint, x, xStart, xEnd, 
    				y, yStart, yEnd, xPoint, yPoint;

    if ( _pWindowBuffer != NULL) {
        pWindow = _pWindowBuffer;
    } else {
        pWindow = _pWindow;
    }

/*
 * GetExposeRects() makes sure that we have at least one clip
 *   rectangle.  (i.e. if vClipRects is NULL the result will be
 *   a temporary clipping rectangle that is the size of the window).
 */

    vTempRects = GetExposeRects( vClipRects, &nClipRects);
    _pGC->SetClipRectangles( vTempRects, nClipRects);

/*
 * Draw if the grid spacing is at least 2 pixels per divison.  The grid
 *   becomes pointless when the division is less than that.
 */

    if ( ScreenCoord( _iGridSpacing) >= 2 * _nGridDivision) {

        nSegment = 0;
        nPoint = 0;

    /* Set the foreground color to grey. */

        col.red = 0x8f;
        col.green = 0x8f;
        col.blue = 0x8f;

        _pGC->SetForegroundColor( &col);
        _pGC->SetLineWidth( 0);

    /* Project the rectangle extents back into the grid. */

        xStart = vTempRects[ 0].x;
        yStart = vTempRects[ 0].y;

        xEnd   = vTempRects[ 0].x + vTempRects[ 0].width;
        yEnd   = vTempRects[ 0].y + vTempRects[ 0].height;

        for ( ii = 1; ii < nClipRects; ii++) {
            if ( vTempRects[ ii].x < xStart) {           
                xStart = vTempRects[ ii].x;
            }

            if ( vTempRects[ ii].y < yStart) {           
                yStart = vTempRects[ ii].y;
            }

            if ( vTempRects[ ii].x + vTempRects[ ii].width > xEnd) {           
                xEnd = vTempRects[ ii].x + vTempRects[ ii].width;
            }

            if ( vTempRects[ ii].y + vTempRects[ ii].height > yEnd) {           
                yEnd = vTempRects[ ii].y + vTempRects[ ii].height;
            }
        }

        xStart = MAX((( xStart - 3) / _iGridSpacing), 0) * _iGridSpacing;
        xEnd = (( xEnd + 3) / _iGridSpacing + 1) * _iGridSpacing;

        yStart = MAX((( yStart - 3) / _iGridSpacing), 0) * _iGridSpacing;
        yEnd = (( yEnd + 3) / _iGridSpacing + 1) * _iGridSpacing;

    /*
     * Loop over the grid-projected rectangle and draw small X's at the
     *   grid spacing points, and points at the grid division points.  The
     *   points and lines are written into buffers which are drawn at the 
     *   end of the loop for performance (flushDraw...()).
     */

        for ( x = xStart; x <= xEnd; x += _iGridSpacing) {
            for ( y = yStart; y <= yEnd; y += _iGridSpacing) {


            /* Draw the cross. */

                if ( AmLineVisible( x - 2, y, x + 2, y,
                                  vTempRects, nClipRects)) {
                    drawLine( _pDisplay, pWindow, _pGC,
                              x - 2, y, x + 2, y, pSegment, &nSegment);
                }

                if ( AmLineVisible( x, y + 2, x, y - 2,
                                  vTempRects, nClipRects)) {
                    drawLine( _pDisplay, pWindow, _pGC,
                              x, y + 2, x, y - 2, pSegment, &nSegment);
                }

            /*
             * Only draw the divisions if there are at least 4 pixels
             *   per division.  This is to remove some clutter from the
             *   screen.
             */

                if ( ScreenCoord( _iGridSpacing) < ( 4 * _nGridDivision)) {
                    continue;
                }

                for ( ii = 0; ii < _nGridDivision; ii++) {	
                    for ( jj = 0; jj < _nGridDivision; jj++) {

                        if ( ii == 0 && jj == 0) {
                            continue;
                        }

                        xPoint = x + ( ii * _iGridSpacing) / _nGridDivision;
                        yPoint = y + ( jj * _iGridSpacing) / _nGridDivision;

//                        if ( AmPointVisible( xPoint, yPoint,
//                                           vTempRects, nClipRects)) {
                            drawPoint( _pDisplay, pWindow, _pGC,
                                       xPoint, yPoint,
                                       pPoint, &nPoint);
//                        }
                    }
                }
            }
        }

        flushDrawPoints( _pDisplay, pWindow, _pGC, pPoint, &nPoint);
        flushDrawLines( _pDisplay, pWindow, _pGC, pSegment, &nSegment);
    }

/* Reset the clip rectangle to the extents of the window. */

    _pGC->SetClipRectangle( _xOrigin, _yOrigin, _iWidth, _iHeight);
}


/**************************************************************************
 *
 * Function: Resize( cx, cy)
 * 
 * Parameters:
 *   int cx: New width in virtual coordinates.
 *   int cy: New height in virtual coordinates. 
 *
 * Description:
 *   Call the base Resize routine, then update the outlines.
 *
 * Returns:
 *   -nothing-
 *
 **************************************************************************/

void
AmMapManagerClass::Resize( int cx, 
                         int cy)
{
    AnvilBoolean            fNeedOutlines;

    /* bail if the window is invalid */
    
    if ( !_pWindow->IsValid()) {
        return;
    }

/* If there are any outlines, then unredraw them. */

    if ( _fOutlinesActive == TRUE) {
        fNeedOutlines = TRUE;
        _fOutlinesActive = FALSE;
        _pGCOutline->SetClipRectangle( _xOrigin, _yOrigin, _iWidth, _iHeight);
    } else {
        fNeedOutlines = FALSE;
    }

    AmDisplayManagerClass::Resize( cx ,cy); 
      
/* If there are any outlines, then redraw them. */

    _pGCOutline->SetClipRectangle( _xOrigin, _yOrigin, _iWidth, _iHeight);

    if ( fNeedOutlines == TRUE) {
        ShowOutlines();
    }

}

/**************************************************************************
 *
 * Function: ReDraw( vExposeRect, nExposeRect)
 * 
 * Parameters:
 *   vExposeRect:	Vector of rectangles to clear.
 *   nExposeRect:	length of rectangle vector.
 *
 * Description:
 *   	If not double buffered, then calls draw on the expose rectangles
 *	otherwise does a copyarea for each, then draws the outlines.
 *	(Draw handles the outlines otherwise).
 *
 * Returns:
 *   -nothing-
 *
 **************************************************************************/

void
AmMapManagerClass::ReDraw( AmRectangleRec const *vExposeRect,
                         int                nExposeRect)
{
    AmRectangleRec const *vTempRect;
    int                nTempRect, cx, cy;
    AnvilBoolean            fNeedOutlines=FALSE;

/* If there is no double buffer, redraw the entire window contents. */

    if ( _pWindowBuffer == NULL) {
        Draw( vExposeRect, nExposeRect);
    } else {

        if ( _fNeedResize) {
            _pWindow->GetDims( &cx, &cy);
            Resize( cx, cy);
            _fNeedResize = FALSE;
        }

        if ( _fJustResized == TRUE) {
            _fJustResized = FALSE;
            return;
        }

        nTempRect = nExposeRect;
        vTempRect = GetExposeRects( vExposeRect, &nTempRect);

        if ( _fOutlinesActive == TRUE) {
            fNeedOutlines = TRUE;
            _fOutlinesActive = FALSE;
            _pGCOutline->SetClipRectangles( vTempRect, nTempRect);
        }

    	AmDisplayManagerClass::ReDraw( vExposeRect, nExposeRect);

/* If there's an outline, we have to set outlines inactive to cause 
   ShowOutlines() to redraw them */

        if ( fNeedOutlines == TRUE) {
            ShowOutlines();
            _pGCOutline->SetClipRectangle( _xOrigin, _yOrigin,
                                           _iWidth, _iHeight);
        }
    }
}

/**************************************************************************
 *
 * Function: Draw( vExposeRect, nExposeRect)
 * 
 * Parameters:
 *   vExposeRect:	Vector of rectangles to clear.
 *   nExposeRect:	length of rectangle vector.
 *
 * Description:
 *   	Call the base draw routine, then update the outlines.
 *
 * Returns:
 *   -nothing-
 *
 **************************************************************************/

void
AmMapManagerClass::Draw( AmRectangleRec const *vExposeRect,
                       int                nExposeRect)
{
    AmRectangleRec const *vTempRect;
    int                nTempRect, cx, cy;
    AnvilBoolean            fNeedOutlines=FALSE;

    /* bail if the window is invalid */
    
    if ( !_pWindow->IsValid()) {
        return;
    }

    if ( _fNeedResize) {
        _pWindow->GetDims( &cx, &cy);
        Resize( cx, cy);
        _fNeedResize = FALSE;
    }

    if ( _fJustResized == TRUE) {
        _fJustResized = FALSE;
        return;
    }

    nTempRect = nExposeRect;
    vTempRect = GetExposeRects( vExposeRect, &nTempRect);

    if ( _fOutlinesActive == TRUE) {
        fNeedOutlines = TRUE;
        _fOutlinesActive = FALSE;
        _pGCOutline->SetClipRectangles( vTempRect, nTempRect);
    }

    AmDisplayManagerClass::Draw( vExposeRect, nExposeRect);
      
    /* If there are any outlines, then undraw them. */

    if ( fNeedOutlines == TRUE) {
        ShowOutlines();
        _pGCOutline->SetClipRectangle( _xOrigin, _yOrigin,
                                       _iWidth, _iHeight);
    }
}

/*
 * Return the grid flag for the map manager (FALSE = grid off,
 *   TRUE = grid on).
 */

AnvilBoolean
AmMapManagerClass::GetGrid()
{
    return( _fGrid);
}

/* Return the grid spacing for the map manager. */

int
AmMapManagerClass::GetGridSpacing()
{
    return( _iGridSpacing);
}

/* Return the number of grid divisions for the map manager. */

int
AmMapManagerClass::GetGridDivision()
{
    return( _nGridDivision);
}


/* Given a point, return the closest grid location. */

int
AmMapManagerClass::GetGridLocation( int iPos)
{
    int iTemp;

    if (( _fGrid == TRUE) && ( _iGridSpacing >= 2 * _nGridDivision)) {
        iTemp = iPos / _iGridSpacing * _iGridSpacing;
        iPos = iTemp + (( iPos - iTemp) * _nGridDivision / _iGridSpacing) *
               _iGridSpacing / _nGridDivision;
    }

    return( iPos);
}

/**************************************************************************
 *
 * Function: SetGrid( fGrid)
 * 
 * Parameters:
 *   fGrid:		Grid on/off flag.
 *
 * Description:
 *   	Refresh the display with the grid turned on or off.
 *
 **************************************************************************/

void
AmMapManagerClass::SetGrid( AnvilBoolean fGrid)
{

/* Change the grid mode and Update the display if it has changed. */

    if ( _fGrid != fGrid) {
        _fGrid = fGrid;

        if ( !_fGrid) {
            ClearArea();
        }

        Draw();
    }
}

/**************************************************************************
 *
 * Function: SetGridSpacing( iGridSpacing, fUpdate)
 * 
 * Parameters:
 *   iGridSpacing:	grid spacing value.
 *   fUpdate:		flag to update the display.
 *
 * Description:
 *   	flag dependent Refresh the display with a new grid spacing.
 *
 **************************************************************************/

void
AmMapManagerClass::SetGridSpacing( int     iGridSpacing,
			         AnvilBoolean fUpdate)
{

/* Change the grid spacing if it has changed, and Update the display 
   if it has changed and the flag is set. */

    if ( _iGridSpacing != iGridSpacing) {
        _iGridSpacing = iGridSpacing;
	if ( _fGrid && fUpdate) {
	    Draw();
	}
    }
}

/**************************************************************************
 *
 * Function: SetGridDivision( nGridDivision, fUpdate)
 * 
 * Parameters:
 *   nGridDivision:	grid division value.
 *   fUpdate:		flag to update the display.
 *
 * Description:
 *   	flag dependent Refresh the display with a new grid division.
 *
 **************************************************************************/

void
AmMapManagerClass::SetGridDivision( int     nGridDivision,
			          AnvilBoolean fUpdate)
{

/* Change the number of grid divisions if it has changed.  Update the 
   display if it has changed. */

    if ( _nGridDivision != nGridDivision) {
        _nGridDivision = nGridDivision;
        if ( _fGrid && fUpdate) {
            Draw();
        }
    }
}

/**************************************************************************
 *
 * Function: AmMapManagerClass()
 *
 * Description:
 *   Do nothing, default constructor.
 *
 **************************************************************************/

AmMapManagerClass::AmMapManagerClass()
{
}

/**************************************************************************
 *
 * Function: ~AmMapManagerClass()
 *
 * Description:
 *   Destructor that frees up any used resources.
 *
 **************************************************************************/

AmMapManagerClass::~AmMapManagerClass()
{
    AmListPtr      lstTemp, lstNext;
    AmOutlineClass *pOutline;

    if ( _pGCOutline != NULL) {
        delete _pGCOutline;
    }

    lstTemp = _lstOutlines;
    while ( lstTemp != NULL) {
        pOutline = ( AmOutlineClass *) lstTemp->data;
        lstNext  = lstTemp->next;

        delete pOutline;
        free( lstTemp);

        lstTemp = lstNext;
    }

    _lstOutlines = NULL;
}

/**************************************************************************
 *
 * Function: AmMapManagerClass( pDisplayClass, pWindowClass, fDoubleBuffer)
 * 
 * Parameters:
 *   Displayclass *pDisplayClass: Display class pointer
 *   AmWindowClass  *pWindowClass:  Window to manage 
 *   AnvilBoolean      fDoubleBuffer:  Double buffer flag.
 *
 * Description:
 *   Constructor: calls the base display manager class constructor, then
 *   sets up the grid and outline members.
 *
 **************************************************************************/

AmMapManagerClass::AmMapManagerClass( AmDisplayClass	*pDisplayClass,
				  AmWindowClass	*pWindowClass,
				  AnvilBoolean	fDoubleBuffer) :
	AmDisplayManagerClass( pDisplayClass, pWindowClass, fDoubleBuffer)
{
    AmColorCellRec	color;

/* create an XOR outline graphics context */

    _pGCOutline = pWindowClass->CreateGC();

    _pGCOutline->SetRasterOp( AmXOR);

    color.red   = 0x7f;
    color.green = 0x7f;
    color.blue  = 0x7f;

    _pGCOutline->SetForegroundColor( &color);
    _pGCOutline->SetClipRectangle( _xOrigin, _yOrigin, _iWidth, _iHeight);

/* NULL out the active outline list, but turn outlines on. */

    _fOutlinesActive = TRUE;
    _lstOutlines     = NULL;

/* Turn off the grid , but set it to 72-pixel spacing w/8 divisions. */

    _fGrid         = FALSE;
    _iGridSpacing  = 72;
    _nGridDivision = 8;
}


void
AmMapManagerClass::Flush()
{
    AnvilBoolean fNeedOutlines;

    if ( _fOutlinesActive == TRUE) {
        HideOutlines();
        fNeedOutlines = TRUE;
    } else {
        fNeedOutlines = FALSE;
    }

    AmDisplayManagerClass::Flush();

    if ( fNeedOutlines == TRUE) {
        ShowOutlines();
    }
}
