#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#if !defined(_WIN32) && !defined(__BSD44__)
#include <alloca.h>
#else 
#include <malloc.h>
#endif
#include <string.h>
#include <anvil/screenobj.h>
#include <anvil/list.h>
#include <anvil/mapmgr.h>
#include <anvil/macros.h>
#include <anvil/plot.h>
#include <anvil/imageplot.h>

/************************************************************************
 *
 * Description:
 *      default WriteMessage function does nothing (we just don't want
 *	to force people to write it for a pure virtual)
 * Arguments:
 *	fmt - format to use for printing the message.
 *	varargs - printf() varargs list.
 *
 ************************************************************************/

void
AmMapManagerClass::WriteMessage( char *fmt, ...)
{
}

/************************************************************************
 *
 * Description:
 *      default WriteResourceMessage function does nothing (we just don't 
 *	want *	to force people to write it for a pure virtual)
 * Arguments:
 *	fmt - string to lookup the format under in the resource database.
 *	varargs - printf() varargs list.
 *
 ************************************************************************/

void
AmMapManagerClass::WriteResourceMessage( char *fmt, ...)
{
}

/************************************************************************
 *
 * Description:
 * 	given a screen position, return the readout string by AmFirst 
 *	finding the topmost object under the cursor, and if it has
 *	a coordinate system, returning the cursor's coordinates,
 *	and if it's an image plot, showing the z value under the
 *	cursor.
 * Arguments:
 *	x,y - cursor position
 *	buffer - buffer to write into.
 * Returns:
 *	TRUE if the readout string is set, FALSE otherwise.
 * Modifications:
 *	I'm not sure the cursor position is properly in the virtual
 *	coordinate system.
 *
 ************************************************************************/

AnvilBoolean 
AmMapManagerClass::getReadoutString( int	x,
				   int	y,
				   char	*buffer)
{
    AmDataGraphicClass	*currLayer;
    AmCoordClass		*coords;
    AmImagePlotClass	*imagePlot;
    AmListPtr		ptr;
    AmObjRectPtr		objRect;
    float		dataX, dataY;
    unitType		xUnits, yUnits;
    char		buf1[ 256], buf2[ 256];

    for ( ptr = _lstObj; ptr != NULL; ptr = ptr->next) {
	objRect = ( AmObjRectPtr) ptr->data;

/* see if the pointer's in an object, and if it's a plot object */

	if ( AmPtInRect( objRect->obj->GetRect(), x, y)) {
            if ( objRect->obj->IsType( AmCOORD)) {
		coords = ( AmCoordClass *) objRect->obj;

/* get the real world value, and the units for the string */

		coords->GetDataPt( x, y, &dataX, &dataY);
		coords->GetVaFields( AmNxunits, &xUnits,
				   AmNyunits, &yUnits,
				   NULL);
		AmPrintValue( xUnits, dataX, buf1);
		AmPrintValue( yUnits, dataY, buf2);
		sprintf( buffer, "%s %s ", buf1, buf2);

/* if it's an image plot, or an overlay plot with an image plot for the 
   current layer, then we add a z readout */

		if ( coords->IsType( AmIMAGE_PLOT)) {
		    imagePlot = ( AmImagePlotClass *) coords;
		    if ( imagePlot->getZstring( dataX, dataY, buf1)) {
			strcat( buffer, buf1);
		    }
		} else if ( coords->IsType( AmOVERLAY)) {
		    coords->GetVaFields( AmNcurrLayer, &currLayer,
				         NULL);
		    if (( currLayer != NULL) &&
		        ( currLayer->IsType( AmIMAGE_PLOT))) {
			imagePlot = ( AmImagePlotClass *) currLayer;
			if ( imagePlot->getZstring( dataX, dataY, buf1)) {
			    strcat( buffer, buf1);
			}
		    }
		}

/* add some spaces to clear out what was drawn earlier */

		strcat( buffer, "                               ");
		return( TRUE);
	    }
	}
    }
    return( FALSE);
}
