#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#if !defined (__BSD44__)
#include <alloca.h>
#endif
#include <Xm/Xm.h>
#include <anvil/macros.h>
#include <anvil/overlay.h>
#include <anvil/motif.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/list.h>
#include <anvil/events.h>
#include <anvil/state_hand.h>
#include <anvil/motifdisplay.h>
#include <anvil/magic.h>

static const int    MAX_EXPOSE_STACK = 256;
static AmRectangleRec pExposeRec[ MAX_EXPOSE_STACK];
static int          nExposeRec = 0;


typedef struct magicDrag_ {
    AmMotifDisplayClass   *motifDisplay;
    AmMagicClass          *magic;
    int                 lastX;
    int                 lastY;
    AnvilBoolean             inResize;
    AnvilBoolean             AmFirst;
    AmRectangleRec        rect;
} AmMagicDragRec, *AmMagicDragPtr;


/*
 * Function: void AmUpdateMotifDisplayCallback( w,
 *                                            xpClientData,
 *                                            xpCallData)
 *
 * Parameters:
 *   Widget    w           : The widget that triggered the callback.
 *   XtPointer xpClientData: The pointer to the AmMotifDisplayClass.
 *   XtPointer xpCallData  : Data associated with the callback by X.
 *
 * Description:
 *   The updateCallback gets called when the
 *
 * Returns:
 *   -nothing-
 */


/* ARGSUSED */
void
AmUpdateMotifDisplayCallback( Widget            w,
			    AmMotifDisplayClass *pMotifDisplay,
			    XtPointer         xpCallData)
{
    pMotifDisplay->Flush();
}


/*
 * Function: MotifDisplay::MotifDisplay()
 *
 * Description:
 *   Do nothing constructor.
 */

AmMotifDisplayClass::AmMotifDisplayClass()
{
}


/*
 * Function: void AmMotifDisplayClass::AmDestroyCallback( w,
 *                                                    xpClientData,
 *                                                    xpCallData)
 *           void AmMotifDisplayClass::destroy( w,
 *                                            xpCallData)
 *
 * Parameters:
 *   Widget    w           : The widget that triggered the callback.
 *   XtPointer xpClientData: The pointer to the AmMotifDisplayClass.
 *   XtPointer xpCallData  : Data associated with the callback by X.
 *
 * Description:
 *   The AmDestroyCallback gets called when the drawing area, that the
 *   motif display is created with, is destroyed.  It simply calls
 *   the destroy function.
 *
 *   The destroy function removes the destroy callback and procedes to
 *   destroy this instantiation of the class.
 *
 * Returns:
 *   -nothing-
 */

void
AmMotifDisplayClass::AmDestroyCallback( Widget    w,
			            XtPointer xpClientData,
			            XtPointer xpCallData)
{
    AmMotifDisplayClass *pMotifDisplay = ( AmMotifDisplayClass *) xpClientData;

    pMotifDisplay->destroy( w, xpCallData);
}

/* ARGSUSED */
void
AmMotifDisplayClass::destroy( Widget    w,
                            XtPointer xpCallData)
{
    XtRemoveCallback( w, XmNdestroyCallback,
                      ( XtCallbackProc) AmDestroyCallback,
                      ( XtPointer) this);
    delete this;
}

/* The expose handler for the motif display. */

void
AmMotifDisplayClass::AmExposeHandler( Widget       w,
                                  XtPointer    xpClientData,
                                  XExposeEvent *pxEvent,
                                  AnvilBoolean      *pfContinueDispatch)
{
    AmMotifDisplayClass *pMotifDisplay = ( AmMotifDisplayClass *) xpClientData;
    pMotifDisplay->expose( w, pxEvent, pfContinueDispatch);
}

/* ARGSUSED */
void
AmMotifDisplayClass::expose( Widget       w,
                           XExposeEvent *pxEvent,
                           AnvilBoolean      *pfContinueDispatch)
{
    /*
     * If the expose event count isn't 0 yet, then just add the rectangle
     *   to the stack.
     */
    
    pExposeRec[ nExposeRec].x = VirtualCoord( pxEvent->x);
    pExposeRec[ nExposeRec].y = VirtualCoord( pxEvent->y);
    pExposeRec[ nExposeRec].width = VirtualCoord( pxEvent->width) + 1;
    pExposeRec[ nExposeRec].height = VirtualCoord( pxEvent->height) + 1;

    nExposeRec++;
    
    /* Redraw the exposed area.  Note that ReDraw just redraws what was
       already there (in the case of double buffering). */ 

    if ( pxEvent->count == 0) {
	if ( _fNeedFlush) {
	    Flush();
	}
	ReDraw( pExposeRec, nExposeRec);
	nExposeRec = 0;
    }
}

/* ARGSUSED */
void
AmMotifDisplayClass::AmResizeCallback( Widget    		w,
                                   AmMotifDisplayClass	*pMotifDisplay,
                                   XtPointer		callData)
{
    Dimension		dimWidth, dimHeight;
    Display		*pDisplay;
    Window		window;

    XtVaGetValues( pMotifDisplay->GetArea(),
		   XmNwidth, &dimWidth,
		   XmNheight, &dimHeight,
		   NULL);
    pMotifDisplay->SetNeedResize( TRUE);
    if (( dimWidth < pMotifDisplay->GetLastWidth()) || 
	( dimHeight < pMotifDisplay->GetLastHeight())) {
	pDisplay = XtDisplay( pMotifDisplay->GetArea());
	window = XtWindow( pMotifDisplay->GetArea());
	if ( window != 0) {
	    AmSendExpose( pDisplay, window);
	}
    }
    pMotifDisplay->SetLastWidth( dimWidth);
    pMotifDisplay->SetLastHeight( dimHeight);
}

/* This function is called from a user event in the object's destructor.
   it calls deleteObject on each of the children, which are deleted
   themselves, but not on the object itself. */

void
AmMotifDisplayClass::AmDeleteObjectCallback( int               iEventID,
                                         AmMotifDisplayClass *pMotifDisplay,
                                         AmScreenObjClass    *pScreenObj)
{
    pMotifDisplay->deleteObject( pScreenObj);
}

/* Redraw children when a parent is updated.  Remove the callback while
   the children are processed, then restore it again once the processing
   is completed, so the callback doesn't get called over and over again. */

void
AmMotifDisplayClass::AmUpdateObjectCallback( int               iEventID,
                                         AmMotifDisplayClass *pMotifDisplay,
                                         AmScreenObjClass    *pScreenObj)
{
    AnvilBoolean fTopLevel, fExists;
    int     iOrder;

    pMotifDisplay->inObjectList( pScreenObj, &fTopLevel, &fExists, &iOrder);
    if ( fExists) {
	pMotifDisplay->AddEvent( pScreenObj, iEventID, iOrder, fTopLevel);
    }
}

/* Free up the screen objects, window and the display gc when the display
   is destroyed. */

AmMotifDisplayClass::~AmMotifDisplayClass()
{
    fprintf( stderr, "In ~AmMotifDisplayClass()\n");
    
    XtRemoveEventHandler( _wArea, ExposureMask, FALSE,
			  ( XtEventHandler) AmExposeHandler,
                          ( XtPointer) this);
    XtRemoveCallback( _wArea, XmNresizeCallback, 
		      ( XtCallbackProc) AmResizeCallback,
                      ( XtPointer) this);

    XtRemoveEventHandler( _wArea, PointerMotionMask, FALSE,
			  ( XtEventHandler) AmCursorReadoutHandler,
                          ( XtPointer) this);
}

/* Assign the drawing area, message label and readout label for a plot */

AmMotifDisplayClass::AmMotifDisplayClass( AmXDisplayClass	*pXDisplay,
				      AmXWindowClass	*pXWindow,
				      AnvilBoolean 		fDoubleBuffer) 
    : AmMagicManagerClass( pXDisplay, pXWindow, fDoubleBuffer)
{
    
    /* Intialize the fields. NULL out all the lists */ 

    _wArea = pXWindow->GetXWidget();
    _wShell = AmGetWidgetShell( _wArea);
    _iCursor = 0;
    _fNeedFlush = FALSE;
    
    XtVaGetValues( _wArea, 
		   XmNcolormap, &_xColormap,
		   XmNwidth, &_iOldWidth,
		   XmNheight, &_iOldHeight,
		   NULL);
    
    /* Set up the draw, destroy and resize callbacks. */

    XtAddCallback( _wArea, XmNdestroyCallback,
		   ( XtCallbackProc) AmDestroyCallback,
                   ( XtPointer) this);

    /* Put the expose handler at the very start of the handler stack, so it's
       drawn before any cursors or other stuff. */

    XtInsertEventHandler( _wArea, ExposureMask, FALSE,
			  ( XtEventHandler) AmExposeHandler,
                          ( XtPointer) this, XtListHead);
    XtAddCallback( _wArea, XmNresizeCallback, 
		   ( XtCallbackProc) AmResizeCallback,
                   ( XtPointer) this);
    
    XtAddEventHandler( _wArea, PointerMotionMask, FALSE,
		       ( XtEventHandler) AmCursorReadoutHandler,
                       ( XtPointer) this);
}

void
AmMotifDisplayClass::AddUserCallback( int			packageId,
				    int			eventId,
				    callbackProc	proc,
				    void		*clientData)
{
    AmAddCallbackWidget( packageId, eventId, _wArea, proc, clientData);
}

void
AmMotifDisplayClass::RemoveUserCallback( int		packageId,
				       int		eventId,
				       callbackProc	proc,
				       void		*clientData)
{
    AmRemoveCallbackWidget( packageId, eventId, proc, clientData, _wArea);
}

/* Shortcut for doing an XDefineCursor...probably we should do some
   resource management here */

int
AmMotifDisplayClass::SetCursor( int iCursor)
{
    int    iOldCursor;
    Cursor xCursor;

    iOldCursor = _iCursor;

    if ( iCursor != _iCursor) {
        _iCursor = iCursor;
        xCursor = XCreateFontCursor( XtDisplay( _wArea), _iCursor);
        XDefineCursor( XtDisplay( _wArea), XtWindow( _wArea), xCursor);
    }

    return( iOldCursor);
}

/* for moving and resizing in non-"callOnDrag" mode, draw the rectangle with
   an X in it so we can see it clearly. */

static void 
drawRectAndX( AmDisplayClass	*display,
	      AmWindowClass	*win,
	      AmGCClass		*gc,
	      int		x, 
	      int		y,
	      int		width,
	      int		height)
{
    display->DrawRectangle( win, gc, x, y, width, height);
    display->DrawLine( win, gc, x, y, x + width, y + height);
    display->DrawLine( win, gc, x + width, y, x, y + height);
}

/* drag or resize a magic */

static void 
dragMagicHandler( Widget	area,
		  AmMagicDragPtr	magicDrag,
		  XEvent	*event,
		  AnvilBoolean	*)
{
    AmMagicClass		*magic = magicDrag->magic;
    AmMotifDisplayClass	*motifDisplay = magicDrag->motifDisplay;
    AmXDisplayClass	*display = ( AmXDisplayClass *) motifDisplay->GetDisplay();
    AmXWindowClass	*window = ( AmXWindowClass *) motifDisplay->GetWindow();
    AmRectangleRec const  *boundRect = magic->GetSourcePlot()->GetRect();
    XMotionEvent	*motionEvent;
    int			newWidth, newHeight;

/* stop the drag on the button release */

    if ( event->type == ButtonRelease) {

AN_DEBUG( 50, ("calling updateOnDrag\n"));
/* if the plot isn't call on drag, we do the resize and move on the mouse 
   release */

	if ( !magic->GetCallOnDrag()) {

/* undraw the AmLast XOR rectangle */

	    if ( !magicDrag->AmFirst) {
		drawRectAndX( display, window, motifDisplay->GetCursorGC(),
			      magicDrag->rect.x, magicDrag->rect.y,
			      magicDrag->rect.width, magicDrag->rect.height);
	    }

/* set rect calls resize */

AN_DEBUG( 40, ("setting rectangle to (%d, %d, %d, %d)\n",
		magicDrag->rect.x, magicDrag->rect.y,
		magicDrag->rect.width, magicDrag->rect.height));
	} 

/* redraw whether we drag or not */

	magic->SetRectWrapper( &magicDrag->rect);
	magic->DrawWrapper( &magicDrag->rect);
		
/* remove the drag handler and free the temporary structure */

	AmRemoveHandler( area, PointerMotionMask | ButtonReleaseMask,
		       ( XtEventHandler) dragMagicHandler, magicDrag);
	free( magicDrag);
    } else {

/* move the window so the cursor is in the center.. */

	motionEvent = ( XMotionEvent *) event;

/* if we're not calling on drag, then undraw an outline rectangle in XOR */

	if ( !magicDrag->AmFirst) {
	    if ( !magic->GetCallOnDrag()) {
		drawRectAndX( display, window, motifDisplay->GetCursorGC(),
			      magicDrag->rect.x, magicDrag->rect.y,
			      magicDrag->rect.width, magicDrag->rect.height);
	    }
	} else {
	    magicDrag->AmFirst = FALSE;
	}
	if ( magicDrag->inResize) {

/* width and height are unsigned.. */

	    newWidth = MAX( magicDrag->rect.width + motionEvent->x - 
				magicDrag->lastX, magic->GetResizeTabSize());
	    newHeight = MAX( magicDrag->rect.height + motionEvent->y - 
				magicDrag->lastY, magic->GetResizeTabSize());

/* ensure that the rectangle wasn't resized beyond the bound rectangle of
   the source plot */

	    if ( magicDrag->rect.x + newWidth > 
		    boundRect->x + boundRect->width) {
		newWidth = boundRect->x + boundRect->width - 
				magicDrag->rect.x;
	    }
	    if ( magicDrag->rect.y + newHeight > 
		    boundRect->y + boundRect->height) {
		newHeight = boundRect->y + boundRect->height - 
				magicDrag->rect.y;
	    }
	    magicDrag->rect.width = newWidth;
	    magicDrag->rect.height = newHeight;
	} else {

/* ensure that the rectangle wasn't dragged beyond the extents of the
   source plot */

	    magicDrag->rect.x += motionEvent->x - magicDrag->lastX;
	    magicDrag->rect.y += motionEvent->y - magicDrag->lastY;
	    if ( magicDrag->rect.x < boundRect->x) {
		magicDrag->rect.x = boundRect->x;
	    }
	    if ( magicDrag->rect.y < boundRect->y) {
		magicDrag->rect.y = boundRect->y;
	    }
	    if ( magicDrag->rect.x + magicDrag->rect.width >
		    boundRect->x + boundRect->width) {
		magicDrag->rect.x = boundRect->x + boundRect->width -
					magicDrag->rect.width;
	    }
	    if ( magicDrag->rect.y + magicDrag->rect.height >
		    boundRect->y + boundRect->height) {
		magicDrag->rect.y = boundRect->y + boundRect->height -
					magicDrag->rect.height;
	    }
	}

/* if we're not calling on drag, then draw an outline rectangle in XOR */

	if ( !magic->GetCallOnDrag()) {
	    drawRectAndX( display, window, motifDisplay->GetCursorGC(),
			  magicDrag->rect.x, magicDrag->rect.y, 
			  magicDrag->rect.width, magicDrag->rect.height);
	} else {

/* if he requested that the update function be called when dragging, call it. 
   figure out where the magic window is in the real world and call the
   magic resize function. SetRect calls resize */

	    magic->SetRectWrapper( &magicDrag->rect);
	    magic->DrawWrapper( &magicDrag->rect);
	}
	magicDrag->lastX = motionEvent->x;
	magicDrag->lastY = motionEvent->y;
    }
}

/* handler for dragging and resizing magics */

/* ARGSUSED */

static void 
magicHandler( Widget		area,
	      AmMotifDisplayClass	*motifDisplay,
	      XButtonEvent	*event,
	      AnvilBoolean		*continueDispatch)
{
    AmMagicDragPtr	magicDrag;
    AmMagicClass		*magic;
    AnvilBoolean		inResize;

/* find the magic window that the button press was inside */

    magic = motifDisplay->findMagic( event->x, event->y, &inResize);
    if ( magic != NULL) {

/* add a drag handler for it */

	magicDrag = ( AmMagicDragPtr) malloc( sizeof( AmMagicDragRec));
	if ( magicDrag == NULL) {
	    DO_ERROR( AmMALLOC_FAILED);
	}
	magicDrag->motifDisplay = motifDisplay;
	magicDrag->magic = magic;
	magicDrag->lastX = event->x;
	magicDrag->lastY = event->y;
	magicDrag->inResize = inResize;
	magicDrag->AmFirst = TRUE;
	magicDrag->rect = *(magic->GetRect());
	AmAddHandler( NULL, area, PointerMotionMask | ButtonReleaseMask,
		    ( XtEventHandler) dragMagicHandler, magicDrag, 
		    MAGIC_WINDOW_DRAG_PRIORITY);
    }
}

void
AmMotifDisplayClass::AddMagic( AmMagicClass	*magic)
{
    if ( _lstMagic == NULL) {
	AmAddHandler( NULL, _wArea, ButtonPressMask,
		    ( XtEventHandler) magicHandler, this,
		    MAGIC_WINDOW_PICK_PRIORITY);
    }
    AmMagicManagerClass::AddMagic( magic);
}

void
AmMotifDisplayClass::RemoveMagic( AmMagicClass	*magic)
{
    AmMagicManagerClass::RemoveMagic( magic);
    if ( _lstMagic == NULL) {
	AmRemoveHandler( _wArea, ButtonPressMask,
		       ( XtEventHandler) magicHandler, this);
    }
}


void
AmMotifDisplayClass::AmSendUserEvent( int  iPackageID,
                                  int  iEventID,
                                  void *pCallData)
{
    ::AmSendUserEventChildren( iPackageID, iEventID, pCallData, _wArea);
}

void
AmMotifDisplayClass::WriteResourceMessage( char *szResource,
                                         ...)
{
    va_list    args;
    char       sz[ MSGLEN];
    const char *szFormat;

    va_start( args, szResource);
    szFormat = AmGetDefault( szResource, szResource);
    vsprintf( sz, szFormat, args);
    va_end( args);

    ::AmSendUserEvent( AM_PKG_ID, AM_MESSAGE_STRING, sz); 
}
