#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <malloc.h>
#if !defined(_WIN32) && !defined(__BSD44__)
#include <alloca.h>
#endif
#include <anvil/macros.h>
#include <anvil/overlay.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/list.h>
#include <anvil/dispatch.h>
#include <anvil/ntdisplay.h>
#include <anvil/magicmgr.h>
#include <anvil/magic.h>
#include <anvil/nt_va_hndlr.h>
#include <anvil/nt_state_hand.h>
#include <anvil/ntdispmgr.h>
#include <anvil/transfer.h>

/* magic drag structure */
/* Modifications:  this is actually a near-duplicate of the magicDrag
   structure in motifdisplay, and they should probably be consolidated.
*/

typedef struct magicDrag_ {
    AmNTDisplayManagerClass	*dispMgr;
    AmMagicClass			*magic;
    AmWindowOutlineClass		*outline;
    int				x;
    int				y;
    AnvilBoolean			inResize;
} AmMagicDragRec, *AmMagicDragPtr;

/***********************************************************************
 *
 * Function: AmNTDisplayManagerClass::AmNTDisplayManagerClass()
 *
 * Description:
 *   Do nothing constructor.
 *
 ***********************************************************************/

AmNTDisplayManagerClass::AmNTDisplayManagerClass()
{
}

/***********************************************************************
 *
 * Function: AmNTDisplayManagerClass::~AmNTDisplayManagerClass()
 *
 * Description:
 *   Destructor 
 *
 ***********************************************************************/

AmNTDisplayManagerClass::~AmNTDisplayManagerClass()
{
}

/***********************************************************************
 *
 * Function: redrawHandler( hWnd, iMsg, wParam, lParam, pNTDisplayManager)
 *
 * Description:
 *   The WM_PAINT handler for redrawing objects controlled by the
 *   display manager.
 *
 ***********************************************************************/

static void
redrawHandler( HWND                  hWnd,
               UINT                  iMsg,
               WPARAM                wParam,
               LPARAM                lParam,
               AmNTDisplayManagerClass *pNTDisplayManager)
{
    HDC             hDC;
    PAINTSTRUCT     ps;
    AmRectangleRec    rect;
    AmNTWindowClass   *pNTWindow = ( AmNTWindowClass *)
                                 pNTDisplayManager->GetWindow();

    hDC = BeginPaint( hWnd, &ps);

    /* Copy the paint rectangle from the event to the expose rectangle. */

    rect.x = pNTDisplayManager->VirtualCoord( ps.rcPaint.left);
    rect.y = pNTDisplayManager->VirtualCoord( ps.rcPaint.top);
    rect.width = pNTDisplayManager->VirtualCoord( ps.rcPaint.right -
                                                  ps.rcPaint.left);
    rect.height = pNTDisplayManager->VirtualCoord( ps.rcPaint.bottom -
                                                  ps.rcPaint.top);
    pNTDisplayManager->ReDraw( &rect, 1);

    EndPaint( hWnd, &ps);
}

/***********************************************************************
 *
 * Function: AmResizeHandler( hWnd, iMsg, wParam, lParam, pNTDisplayManager)
 *
 * Description:
 *      the WM_SIZE handler for resizing objects controlled by the
 *	display manager.
 *
 ***********************************************************************/

static void
AmResizeHandler( HWND                   hWnd,
               UINT                   iMsg,
               WPARAM                 wParam,
               LPARAM                 lParam,
               AmNTDisplayManagerClass  *pNTDisplayManager)
{
    int cx, cy;

    /* Get the dimensions of the window (these are in "virtual coordinates"). */

    pNTDisplayManager->GetWindow()->GetDims( &cx, &cy);
    pNTDisplayManager->Resize( cx, cy);
}
    

/***********************************************************************
 *
 * Function: AmNTDisplayManagerClass()
 * 
 * Description:
 *   Constructor.
 *
 * Arguments:
 *   fDoubleBuffer - draw display contents double buffered.
 *
 ***********************************************************************/

AmNTDisplayManagerClass::AmNTDisplayManagerClass( AmNTDisplayClass *pNTDisplay,
                                              AmNTWindowClass  *pNTWindow,
                                              AnvilBoolean        fDoubleBuffer)
    : AmMagicManagerClass( pNTDisplay, pNTWindow, fDoubleBuffer)
{
    AmInsertHandler( pNTWindow->getHWND(), WM_PAINT, 0,
                     ( AmNtHandlerFn) redrawHandler, this);
    AmInsertHandler( pNTWindow->getHWND(), WM_SIZE, 0,
                     ( AmNtHandlerFn) AmResizeHandler, this);
}

/***********************************************************************
 *
 * Function: GetWidth(), GetHeight()
 * 
 * Description:
 *   Return width and height of display manager's window.
 *
 ***********************************************************************/

int 
AmNTDisplayManagerClass::GetWidth()
{
    RECT          rect;
    AmNTWindowClass *pNTWindow = ( AmNTWindowClass *) _pWindow;

    GetClientRect( pNTWindow->getHWND(), &rect);
    return( rect.right - rect.left);
}

int 
AmNTDisplayManagerClass::GetHeight()
{
    RECT          rect;
    AmNTWindowClass *pNTWindow = ( AmNTWindowClass *) _pWindow;

    GetClientRect( pNTWindow->getHWND(), &rect);
    return( rect.bottom - rect.top);
}

/***********************************************************************
 *
 * Function: Flush()
 * 
 * Description:
 *	Flush the outstanding events for screen objects, then send a 
 *	user event notifying anyone who's interested.
 *
 ***********************************************************************/

void 
AmNTDisplayManagerClass::Flush()
{

    AmNTWindowClass	*pNTWindow = ( AmNTWindowClass *) _pWindow;

/* wrapper calls flush, then sends the flush event to the window */

    AmDisplayManagerClass::Flush();
    AmSendUserEventChildren( AM_PKG_ID, AM_FLUSH, this, pNTWindow->getHWND());
}

/***********************************************************************
 *
 * Function: AddUserCallback()
 * 
 * Description:
 *	Wrapper for user event handlers which are directed towards
 *	a window, to afford a level of display independence in the
 *	display manager, and tie the callback handlers to a window.
 * Arguments:
 *	packageId - unique package ID.
 *	eventId - event identifier.
 *	proc - procedure to call.
 *	clientData - data to pass to the procedure.
 *
 ***********************************************************************/

void 
AmNTDisplayManagerClass::AddUserCallback( int		packageId,
					int		eventId,
					callbackProc	proc,
					void		*clientData)
{   
    AmNTWindowClass	*pNTWindow = ( AmNTWindowClass *) _pWindow;

    addCallbackWindow( packageId, eventId, pNTWindow->getHWND(),
		       proc, clientData);
}

/***********************************************************************
 *
 * Function: RemoveUserCallback()
 * 
 * Description:
 *	Wrapper function to remove a user callback.
 * Arguments:
 *	packageId - unique package ID.
 *	eventId - event identifier.
 *	proc - procedure to call.
 *	clientData - data to pass to the procedure.
 *
 ***********************************************************************/

void 
AmNTDisplayManagerClass::RemoveUserCallback( int		packageId,
					   int		eventId,
					   callbackProc	proc,
					   void		*clientData)
{
    AmNTWindowClass	*pNTWindow = ( AmNTWindowClass *) _pWindow;

    removeCallbackWindow( packageId, eventId, 
			  proc, clientData, pNTWindow->getHWND());
}

/***********************************************************************
 *
 * Description:
 *	Wrapper function to send a user event.
 * Arguments:
 *	packageId - unique package ID.
 *	eventId - event identifier.
 *	clientData - data to pass to the procedures which are 
 *	interested in this user event.
 *
 ***********************************************************************/

void
AmNTDisplayManagerClass::AmSendUserEvent( int       iPackageID,
                                      int       iEventID,
                                      void      *pClientData)
{
    AmSendUserEventChildren( iPackageID, iEventID, pClientData,
                           (( AmNTWindowClass *) GetWindow())->getHWND());
}


/***********************************************************************
 *
 * Description:
 *	handler for WM_MOUSEMOVE events to drag the magic window.
 *	if the GetCallOnDrag() flag is set, then actually update
 *	the magic window, otherwise move the outline.  If the 
 *	mouse position is in the resize rectangle, then resize
 *	the magic window.
 * Arguments:
 *	hWnd - parent window the magic window is being dragged on
 *	iMsg - hopefully, the WM_MOUSEMOVE message. 
 *	wParam - keyFlags (ignored)
 *	lParam - contains the x and y coordinates of the mouse.
 *	magicDrag - contains the magic window, AmLast mouse position,
 *	and display manager information.
 * Modifications:
 *	There are problems when the magic window is resized that
 *	it's position changes.  The bug is either in here, or 
 *	in the AmNTWindowClass::ResizeWindow() function.
 *
 ***********************************************************************/

static void
dragMagicHandler( HWND			hWnd,
		  UINT			iMsg,
		  WPARAM		wParam,
		  LPARAM		lParam,
		  AmMagicDragPtr		magicDrag)
{
    AmNTDisplayManagerClass	*dispMgr = magicDrag->dispMgr;
    AmMagicClass			*magic = magicDrag->magic;
    int				newX, newY, x, y, width, height;

    newX = dispMgr->VirtualCoord( LOWORD( lParam));
    newY = dispMgr->VirtualCoord( HIWORD( lParam));
    if ( magic->GetCallOnDrag()) {
	AmRectangleRec 	newRect = *( magic->GetRect());

        if ( magicDrag->inResize) {
	    newRect.width += newX - magicDrag->x;
	    newRect.height += newY - magicDrag->y;
	    magic->SetRectWrapper( &newRect);
	} else {
	    newRect.x += newX - magicDrag->x;
	    newRect.y += newY - magicDrag->y;
	    magic->SetRectWrapper( &newRect);
	}
    } else {
    	if ( magicDrag->inResize) {
	    magicDrag->outline->GetSize( &width, &height);
	    width += newX - magicDrag->x;
	    height += newY - magicDrag->y;
	    magicDrag->outline->SetSize( width, height);
	} else {
	    magicDrag->outline->GetOrigin( &x, &y);
	    x += newX - magicDrag->x;
	    y += newY - magicDrag->y;
	    magicDrag->outline->SetOrigin( x, y);
	}
    }
	
    magicDrag->x = newX;
    magicDrag->y = newY;
}

/***********************************************************************
 *
 * Description:
 *	handler for the WM_LBUTTONUP message, updates the magic 
 *	window's  position and size (if it's not being updated 
 *	during the drag with GetCallOnDrag(), and removes the 
 *	drag and release handlers.
 * Arguments:
 *	hWnd - parent window the magic window is being dragged on
 *	iMsg - hopefully, the WM_LBUTTONUP message. 
 *	wParam - keyFlags (ignored)
 *	lParam - contains the x and y coordinates of the mouse.
 *	magicDrag - contains the magic window, AmLast mouse position,
 *	and display manager information.
 *
 ***********************************************************************/
   
static void
releaseMagicHandler( HWND		hWnd,
		     UINT		iMsg,
		     WPARAM		wParam,
		     LPARAM		lParam,
		     AmMagicDragPtr	magicDrag)
{
    AmMagicClass	    *magic = magicDrag->magic;
    int		    x, y, width, height;
    AmRectangleRec    newRect;

    if ( !magic->GetCallOnDrag()) {
	magicDrag->outline->GetRectangle( &x, &y, &width, &height);
	newRect.x = x;
	newRect.y = y;
	newRect.width = width;
	newRect.height = height;
	magic->SetRectWrapper( &newRect);
    }
    if ( magicDrag->outline != NULL) {
    	magicDrag->dispMgr->RemoveOutline( magicDrag->outline);
    }
    AmDeleteHandler( hWnd, WM_MOUSEMOVE, 0, 
					 ( AmNtHandlerFn) dragMagicHandler, 
					 magicDrag);
    AmDeleteHandler( hWnd, WM_LBUTTONUP, 0,
					 ( AmNtHandlerFn) releaseMagicHandler,
					 magicDrag);
    free( magicDrag);
} 

/***********************************************************************
 *
 * Description:
 * 	WM_LBUTTONDOWN handler for picking magic windows and
 *	launching the drag and release handlers.
 * Arguments:
 *	hWnd - parent window the magic window is being picked on
 *	iMsg - hopefully, the WM_LBUTTONDOWN message. 
 *	wParam - keyFlags (ignored)
 *	lParam - contains the x and y coordinates of the mouse.
 *	dispMgr - display manager the magic window is being picked on.
 *
 ***********************************************************************/

static void
pickMagicHandler( HWND			hWnd,
		  UINT			iMsg,
		  WPARAM		wParam,
		  LPARAM		lParam,
		  AmNTDisplayManagerClass	*dispMgr)
{
    AmMagicClass		*magic;
    AnvilBoolean		inResize;
    int			x, y;
    AmMagicDragPtr	magicDrag;
    AmRectangleRec const	*magicRect;

    x = dispMgr->VirtualCoord( LOWORD( lParam));
    y = dispMgr->VirtualCoord( HIWORD( lParam));
    magic = dispMgr->findMagic( x, y, &inResize);
    if ( magic != NULL) {
    	magicDrag = ( AmMagicDragPtr) malloc( sizeof( AmMagicDragRec));
	magicDrag->x = x;
	magicDrag->y = y;
	magicDrag->dispMgr = dispMgr;
	magicDrag->magic = magic;
	magicDrag->inResize = inResize;
	if ( !magic->GetCallOnDrag()) {
	    magicRect = magic->GetRect();
	    magicDrag->outline = 
	    	new AmWindowOutlineClass( dispMgr, magicRect->x, magicRect->y,
					magicRect->width, magicRect->height,
					magic->GetResizeTabSize());
	    dispMgr->AddOutline( magicDrag->outline);
	    dispMgr->ShowOutline( magicDrag->outline);
	} else {
	    magicDrag->outline = NULL;
	}
	if ( magicDrag != NULL) {
	    AmAppendHandler( hWnd, WM_MOUSEMOVE, 0, 
						 ( AmNtHandlerFn) dragMagicHandler, 
						 magicDrag);
	    AmAppendHandler( hWnd, WM_LBUTTONUP, 0,
						 ( AmNtHandlerFn) releaseMagicHandler,
						magicDrag);
	}
    }
}

/***********************************************************************
 *
 * Description:
 * 	Add a magic window to the display manager's magic window list.
 *	If it's the AmFirst magic window being added, then set up
 *	the button press handler to pick magic windows.  If the 
 *	Magic window has its own private window, set up transfer
 *	event handlers to transfer any events it receives to the 
 *	parent window so outlines and windows can be picked and
 *	dragged in the same manner.
 * Arguments:
 *	magic - magic window to add.
 *
 ***********************************************************************/

void 
AmNTDisplayManagerClass::AddMagic( AmMagicClass	*magic)
{
    if ( _lstMagic == NULL) {
   		AmNTWindowClass	*pNTWindow = ( AmNTWindowClass *) _pWindow;

		AmInsertHandler( pNTWindow->getHWND(), WM_LBUTTONDOWN, 0,
						 ( AmNtHandlerFn) pickMagicHandler, this);
    }
    if ( magic->GetWindow() != GetWindow()) {
    	AmNTWindowClass	*pSrcWnd = ( AmNTWindowClass *) magic->GetWindow(),
			*pDstWnd = ( AmNTWindowClass *) GetWindow();

    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_LBUTTONDOWN, 0);
    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_LBUTTONUP, 0);
    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_MBUTTONDOWN, 0);
    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_MBUTTONUP, 0);
    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_RBUTTONDOWN, 0);
    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_RBUTTONUP, 0);
    	AmTransferEvent( pSrcWnd->getHWND(), pDstWnd->getHWND(), 
		       WM_MOUSEMOVE, 0);
    }
    AmMagicManagerClass::AddMagic( magic);
}

/***********************************************************************
 *
 * Description:
 * 	Delete a magic window from the display manager's magic window 
 *	list.  If it's the AmLast window being removed, then
 *	delete the mouse press magic window pick handler.
 * Arguments:
 * 	magic - magic window to remove.
 *
 ***********************************************************************/

void 
AmNTDisplayManagerClass::RemoveMagic( AmMagicClass	*magic)
{
    AmMagicManagerClass::RemoveMagic( magic);
    if ( _lstMagic == NULL) {
   		AmNTWindowClass	*pNTWindow = ( AmNTWindowClass *) _pWindow;

		AmDeleteHandler( pNTWindow->getHWND(), WM_LBUTTONDOWN, 0,
						 ( AmNtHandlerFn) pickMagicHandler, this);
    }
}
