#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


/* Motif Display Class readout functions. */

#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <anvil/strlib.h>
#include <anvil/userevents.h>
#include <anvil/units.h>
#include <anvil/plot.h>
#include <anvil/imageplot.h>
#include <anvil/tagstrs.h>
#include <anvil/motif.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/geometry.h>
#include <anvil/va_cback.h>
#include <anvil/plot_cback.h>
#include <anvil/userevents.h>
#include <anvil/list.h>
#include <anvil/fonts.h>
#include <anvil/events.h>
#include <anvil/motifdisplay.h>

/* event handler to receive the message buffer and display it in a widget */

static void messageUserEventHandler( int,	/* id */
				     Widget	wMsg,
				     char	*sz)
{
    static char szBuffer[ TEXTLEN] = { '\0' };
    XmString	xsz;
    int         ii; 

    if ( strncmp( szBuffer, sz, TEXTLEN) != 0) {

        xsz = XmStringCreateLtoR( sz, XmFONTLIST_DEFAULT_TAG);
        XtVaSetValues( wMsg, XmNlabelString, xsz, NULL);
        XmStringFree( xsz);

        for ( ii = 0; ( sz[ ii] != '\0') && ( ii < TEXTLEN - 1); ii++) {
            szBuffer[ ii] = sz[ ii];
        }

        szBuffer[ ii] = '\0';

    }
}

/* cursor readout event handler */

void AmCursorReadoutHandler( Widget		,
			   AmMotifDisplayClass	*motifDisplay,
			   XMotionEvent		*event,
			   AnvilBoolean		*)
{
    char		buffer[ MSGLEN];

    if ( motifDisplay->getReadoutString( event->x, event->y, buffer)) {
	AmSendUserEventChildren( AM_PKG_ID, AM_READOUT_STRING, buffer,
			       motifDisplay->GetShell());
    }
}

/* this function receives the readout user event and draws the string
   in the specified widget */

static void cursorReadoutUserEventHandler( int,		/* id */
					   Widget	readout,
					   char		*string)
{
    Display	*display = XtDisplay( readout);
    Window	win = XtWindow( readout);
    XGCValues	gcv;
    static GC 	readoutGC = NULL;
    XmFontList	fontList;
    Dimension	height;
    ulong_t	fg, bg;

/* create the readout GC if we haven't already */

    if ( readoutGC == NULL) {
	XtVaGetValues( readout, 
		       XmNforeground, &fg,
		       XmNbackground, &bg,
		       NULL);
	gcv.foreground = fg;
	gcv.background = bg;
	gcv.font = 0;
	readoutGC = XCreateGC( display, win, 
			       GCForeground | GCBackground, &gcv);
    }
    XtVaGetValues( readout, 
		   XmNfontList, &fontList, 
		   XmNheight, &height,
		   NULL);
    AmDrawMotifString( display, win, readoutGC, fontList, 
		     10, height - 5, string);
}

/* make a widget receive the user event for displaying a readout string */

void AmMotifDisplayClass::SetReadout( Widget readoutLbl)
{
    AmAddCallbackWidget( AM_PKG_ID, AM_READOUT_STRING, readoutLbl,
		       ( callbackProc) cursorReadoutUserEventHandler, 
		       readoutLbl);
}

/* turn off the widget from receiving the user events for displaying a
   readout string */

void AmMotifDisplayClass::RemoveReadout( Widget	readoutLbl)
{
    AmRemoveCallbackWidget( AM_PKG_ID, AM_READOUT_STRING, 
			  ( callbackProc) cursorReadoutUserEventHandler, 
			  readoutLbl, readoutLbl);
}

/* make a widget receive the user event for displaying a message string */

void AmMotifDisplayClass::SetMessage( Widget messageLbl)
{
    AmAddCallbackWidget( AM_PKG_ID, AM_MESSAGE_STRING, messageLbl,
		       ( callbackProc) messageUserEventHandler, messageLbl);
}

/* turn off the widget from receiving the user events for displaying a
   readout string */

void AmMotifDisplayClass::RemoveMessage( Widget	messageLbl)
{
    AmRemoveCallbackWidget( AM_PKG_ID, AM_MESSAGE_STRING, 
			  ( callbackProc) messageUserEventHandler, 
			  messageLbl, messageLbl);
}
