#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <Xm/Xm.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <anvil/strlib.h>
#include <anvil/userevents.h>
#include <anvil/units.h>
#include <anvil/plot.h>
#include <anvil/events.h>
#include <anvil/motif.h>
#include <anvil/resource.h>
#include <anvil/va_cback.h>
#include <anvil/plot_cback.h>
#include <anvil/userevents.h>
#include <anvil/list.h>
#include <anvil/motifdisplay.h>
#include <anvil/xfonts.h>
#include <anvil/defaults.h>


/******************************************************************/


static void drawTravelCursor( Widget widget,
                              Window win,
			      char   *string)
{
    Display	       *display = XtDisplay( widget);
    XGCValues	       gcv;
    static GC 	       readoutGC = NULL;
    static XFontStruct *fontInfo;
    const char         *fontName, *fontSize;
    char	       *q, *p;
    int                width, height, y, size;

    if ( !XtIsRealized( widget)) {
        return;
    }

/* create the readout GC if we haven't already */

    if ( readoutGC == NULL) {
	gcv.foreground = BlackPixel( display, DefaultScreen( display));
	gcv.background = WhitePixel( display, DefaultScreen( display)); 

/* see if there's a font set in the reasource database, or use the server
   default */

	fontName = AmGetDefault( "travelCursor.font", DEFAULT_FULL_FONT);
	fontSize = AmGetDefault( "travelCursor.fontSize", DEFAULT_FULL_FONT);

        size = 0;
        if ( fontSize != NULL) {
            size = atol( fontSize);
        }

        fontInfo = NULL;
        if ( fontName != NULL) {
            if ( size != 0) {
                fontInfo = AmLoadQueryFont( display, fontName, size);
            } else {
                fontInfo = AmLoadQueryFont( display, fontName);
            }
        }
        if ( fontInfo == NULL) {
            gcv.font = 0;
	    readoutGC = XCreateGC( display, win, 
				   GCForeground | GCBackground, &gcv);
            fontInfo = XQueryFont( display, XGContextFromGC( readoutGC));
        } else {
            gcv.font = fontInfo->fid;
	    readoutGC = XCreateGC( display, win, 
				   GCForeground | GCBackground | GCFont, 
				   &gcv);
        }
    }

/* draw the string on separate lines as newlines are encountered */

    XClearWindow( display, win);
    width = XTextWidth( fontInfo, string, strlen( string));
    height = fontInfo->ascent + fontInfo->descent;
    y = height;
    p = string;
    do {	
	for ( q = p; (*q != '\n') && ( *q != '\0'); q++);
	XDrawString( display, win, readoutGC, 0, y, p, q - p);
	if ( *q == '\0') {
	    break;
	} else {
	    q++;
	}
	p = q;
	y += height + fontInfo->descent;
    } while( TRUE);
}

static void cursorUserEventHandler( int,	/* id */
				    Widget	area,
				    Window	win,
				    char	*string)
{
    drawTravelCursor( area, win, string);
}


/* move the travel window with the cursor */

static void moveTravelCursor( Widget		widget,
			      Window		cursorWin,
			      XMotionEvent	*event,
			      AnvilBoolean		*)
{
    XMoveWindow( XtDisplay( widget), cursorWin,
        event->x + AmCURSOR_OFFSET,
        event->y + AmCURSOR_OFFSET);
}

/* handlers for turning the cursor on and off when the user enters and leaves
   the window */

static void mapTravelCursor( Widget		widget,
			     Window		cursorWin,
			     XCrossingEvent	*event,
			     AnvilBoolean		*)
{
    Display   	                *display = XtDisplay( widget);

    XMapWindow( display, cursorWin);
}

static void unmapTravelCursor( Widget		widget,
	  		       Window		cursorWin,
			       XCrossingEvent	*event,
			       AnvilBoolean		*)
{
    XUnmapWindow( XtDisplay( widget), cursorWin);
}

/* turn on the travel cursor, and set the event handlers which drag, map and
   unmap it */

static void mapTravelCursorAndCallbacks( int       ,
                                         Widget    widget,
                                         Window    cursorWin,
                                         void      *)
{
    Display   	                *display = XtDisplay( widget);
    Window 			root, child; 
    int				x, y, rootX, rootY;
    uint_t			mask;

    XQueryPointer( display, XtWindow( widget),
		   &root, &child, &rootX, &rootY, &x, &y, &mask);

    XtAddEventHandler( widget, PointerMotionMask, FALSE,
		       ( XtEventHandler) moveTravelCursor, 
		       ( XtPointer) cursorWin);
    XtAddEventHandler( widget, EnterWindowMask, FALSE,
		       ( XtEventHandler) mapTravelCursor, 
		       ( XtPointer) cursorWin);
    XtAddEventHandler( widget, LeaveWindowMask, FALSE,
		       ( XtEventHandler) unmapTravelCursor, 
		       ( XtPointer) cursorWin);

/* move the window and map it, then send a fake motion event to update 
   the cursor */

    XMoveWindow( display, cursorWin,
		 x + AmCURSOR_OFFSET, y + AmCURSOR_OFFSET);
    XMapWindow( display, cursorWin);
    AmSendMotion( display, XtWindow( widget), x, y);
}

/* unmap the travel cursor and turn off the handlers which move, 
   map and unmap it */

static void unmapTravelCursorAndCallbacks( int       ,
                                           Widget    widget,
                                           Window    cursorWin,
                                           void      *)
{
    XtRemoveEventHandler( widget, PointerMotionMask, FALSE,
		          ( XtEventHandler) moveTravelCursor, 
		          ( XtPointer) cursorWin);
    XtRemoveEventHandler( widget, EnterWindowMask, FALSE,
		          ( XtEventHandler) mapTravelCursor, 
		          ( XtPointer) cursorWin);
    XtRemoveEventHandler( widget, LeaveWindowMask, FALSE,
		          ( XtEventHandler) unmapTravelCursor, 
		          ( XtPointer) cursorWin);

    XUnmapWindow( XtDisplay( widget), cursorWin);
}

/* enable and disable the travel cursor when the toggle key is pressed */

void AmEnableTravelCursor( Widget				area,
			 AmMotifDisplayClass		*motifDisplay,
			 Window				cursorWin,
			 int				toggleKey,
			 XmDrawingAreaCallbackStruct	*cb)
{
    XEvent		*event = cb->event;
    XKeyEvent		*keyEvent = ( XKeyEvent *) event;
    char		buffer[ 32];
    KeySym		keysym;
    XWindowAttributes	attr;

    if ( event->type == KeyPress) {
	if ( XLookupString( keyEvent, buffer, sizeof( buffer), 
			    &keysym, NULL) > 0) {
	    XGetWindowAttributes( XtDisplay( area), cursorWin, &attr);
	    if ( toupper( buffer[ 0]) == toupper( toggleKey)) {
		if ( attr.map_state == IsUnmapped) {
AN_DEBUG( 70, ("mapping travel cursor\n"));
		    mapTravelCursorAndCallbacks( 0, area, cursorWin, NULL);
		} else {
AN_DEBUG( 70, ("unmapping travel cursor\n"));
		    unmapTravelCursorAndCallbacks( 0, area, cursorWin, NULL);
		}
	    }
	}
    }
}

/* set the travel cursor.  The arguments specify the width and height of
   the window which is dragged around with the cursor, the toggle key
   is the key used to turn it on and off. */
   
static void setTravelCursor( Widget		area,
			     AmMotifDisplayClass	*motifDisplay,
			     int		width,
			     int		height,
			     int		toggleKey,
			     AnvilBoolean		on)
{
    Display   	                *display = XtDisplay( area);
    Window 			cursorWin, root, child; 
    int				x, y, rootX, rootY;
    XSetWindowAttributes	attr;
    uint_t			mask;

/* create the cursor window with save unders turned on near the cursor */

    XQueryPointer( display, XtWindow( area),
		   &root, &child, &rootX, &rootY, &x, &y, &mask);
    attr.save_under = TRUE;
    attr.background_pixel = WhitePixel( display, DefaultScreen( display));
    attr.border_pixel = BlackPixel( display, DefaultScreen( display));
    cursorWin = XCreateWindow( display, XtWindow( area),
			       x + AmCURSOR_OFFSET,
                               y + AmCURSOR_OFFSET,
                               width, height, 1, CopyFromParent,
			       CopyFromParent, CopyFromParent,
			       CWSaveUnder | CWBackPixel | CWBorderPixel,
                               &attr);
    if ( on) {
	XMapWindow( display, cursorWin);

/* set up the handlers to turn the cursor on and off when the cursor enters
   and leaves the drawing area, and to drag it with the cursor */

	XtAddEventHandler( area, PointerMotionMask, FALSE,
			   ( XtEventHandler) moveTravelCursor, 
			   ( XtPointer) cursorWin);
	XtAddEventHandler( area, EnterWindowMask, FALSE,
			   ( XtEventHandler) mapTravelCursor, 
			   ( XtPointer) cursorWin);
	XtAddEventHandler( area, LeaveWindowMask, FALSE,
			   ( XtEventHandler) unmapTravelCursor, 
			   ( XtPointer) cursorWin);
    } 

/* set the callback to enable and disable the cursor based on the toggle key.
   There's still a focus problem hwere */

    AmAddVaCallback( area, XmNinputCallback,
		   ( VarargsCallback) AmEnableTravelCursor, 
		   motifDisplay, cursorWin, toggleKey, END_LIST);

/* user events for updating the string, and mapping and unmapping the
   cursor */

    AmAddVaCallbackWidget( AM_PKG_ID, AM_CURSOR_STRING, area,
		         ( VarargsCallback) cursorUserEventHandler,
                         area, cursorWin, END_LIST);

    AmAddVaCallbackWidget( AM_PKG_ID, AM_CURSOR_MAP, area,
		         ( VarargsCallback) mapTravelCursorAndCallbacks,
                         area, cursorWin, END_LIST);

    AmAddVaCallbackWidget( AM_PKG_ID, AM_CURSOR_UNMAP, area,
		         ( VarargsCallback) unmapTravelCursorAndCallbacks,
                         area, cursorWin, END_LIST);
}

/* callback to set the travel cursor if the window hasn't been realized */

static void setTravelCursorCallback( Widget		area,
				     AmMotifDisplayClass	*motifDisplay,
				     int		width,
				     int		height,
				     int		toggleKey,
				     AnvilBoolean		on,
				     XtPointer)
{
    AmRemoveVaCallback( area, XmNexposeCallback,
    		      ( VarargsCallback) setTravelCursorCallback,
    		      motifDisplay, width, height, toggleKey, on, END_LIST);
    setTravelCursor( area, motifDisplay, width, height, toggleKey, on);
}

/* set a cursor window that travels with the mouse */

void AmMotifDisplayClass::SetTravelCursor( int		width,
					 int		height,
					 int		toggleKey,
					 AnvilBoolean	on)
{
    if ( XtIsRealized( _wArea)) {
	setTravelCursor( _wArea, this, width, height, toggleKey, on);
    } else { 
	AmAddVaCallback( _wArea, XmNexposeCallback,
		       ( VarargsCallback) setTravelCursorCallback, 
		       this, width, height, toggleKey, on, END_LIST);
    }
}

/*
 * write a string to the travel cursor sending a user event to anyone
 *   who's listening.
 */

void AmMotifDisplayClass::WriteTravelCursor( char	*fmt,
				      ...)
{
    va_list	args;
    char	buf[ MSGLEN];

    va_start( args, fmt);
    vsprintf( buf, fmt, args);
    va_end( args);
    AmSendUserEventChildren( AM_PKG_ID, AM_CURSOR_STRING, buf, GetShell());
}

/* write a string message referenced by a name from a resource file */

void AmMotifDisplayClass::WriteResourceTravelCursor( char	*res,
					           ...)
{
    va_list	args;
    char	buf[ MSGLEN];
    const char	*fmt;

/* get the string from the resource file */

    va_start( args, res);
    fmt = AmGetDefault( res, res);
    vsprintf( buf, fmt, args);
    va_end( args);
    AmSendUserEventChildren( AM_PKG_ID, AM_CURSOR_STRING, buf, GetShell());
}


void AmMotifDisplayClass::MapTravelCursor()
{
    AmSendUserEventChildren( AM_PKG_ID, AM_CURSOR_MAP, NULL, GetShell());
}

void AmMotifDisplayClass::UnmapTravelCursor()
{
    AmSendUserEventChildren( AM_PKG_ID, AM_CURSOR_UNMAP, NULL, GetShell());
}

/* follow cursor which shows the current mouse position */

void AmCursorTravelUserEventHandler( int,         /* id */
                                   AmMotifDisplayClass    *motifDisplay,
                                   char                 *string)
{
    motifDisplay->WriteTravelCursor( string);
}

/* set the readout which follows the cursor */

void AmMotifDisplayClass::SetFollowReadout( int  		toggleKey,
					  AnvilBoolean	on)
{
    int width, height;

    width = AmGetDefault( "followReadout.width", AmFOLLOW_READOUT_WIDTH);
    height = AmGetDefault( "followReadout.height", AmFOLLOW_READOUT_HEIGHT);
    SetTravelCursor( width, height, toggleKey, on);
    AmAddCallbackWidget( AM_PKG_ID, AM_READOUT_STRING, _wArea,
                       ( callbackProc) AmCursorTravelUserEventHandler,
                       this);
}
