#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <math.h>
#include <stdarg.h>
#if !defined (__BSD44__) && !defined(LINUX)
#include <io.h>
#include <direct.h>
#endif
#include <fcntl.h>
#include <anvil/project.h>

/* wrappers for the case-insensitive string comparison functions */
#if !defined(__BSD44__) && !defined(LINUX)

int strcasecmp( const char *s1, const char *s2)
{
    return( stricmp( s1, s2));
}

int strncasecmp( const char *s1, const char *s2, size_t n)
{
    return( strnicmp( s1, s2, n));
}

/* open file.  For some reason, NT has different flags for
   binary and text files.  We use open only for binary files,
   so we assume _O_BINARY. There are also flags for _O_SEQUENTIAL
   and _O_RANDOM */

int open( const char *path, int oflag, ...)
{
    va_list	modeArg;
    int		mode;

    if ( oflag & O_CREAT) {
    	va_start( modeArg, oflag);
	mode = va_arg( modeArg, int);
	va_end( modeArg);
	return( _open( path, oflag | _O_BINARY, mode));
    } else {
	return( _open( path, oflag | _O_BINARY));
    }
}

/* close file */

int close( int fildes)
{
    return( _close( fildes));
}

/* wrappers for read and write */

int read( int fildes, void *buf, uint_t count)
{
    return( _read( fildes, buf, count));
}

int write( int fildes, void *buf, uint_t count)
{
    return( _write( fildes, buf, count));
}

/* wrapper for lseek */

long lseek( int fildes, long offset, int origin)
{
    return( _lseek( fildes, offset, origin));
}

/* wrapper for popen and pclose */

FILE *popen( const char *command, const char *type)
{
    return( _popen( command, type));
}

int pclose( FILE *fp)
{
    return( _pclose( fp));
}

/* wrapper for access() */

int access( const char *path, int mode)
{
    return( _access( path, mode));
}

int rint( float value)
{
    return( floor( value + 0.5));
}

char *mktemp( char *filetempl)
{
    return( _mktemp( filetempl));
}

int umask( int mask) 
{
    return( _umask( mask));
}

long tell( int fd)
{
    return( _tell( fd));
}

int unlink( const char *filename)
{
    return( _unlink( filename));
}

#endif

ulong_t pread( int fildes, void *buf, size_t nbyte, long offset)
{
    lseek( fildes, offset, SEEK_SET);
    return( read( fildes, buf, nbyte));
}

ulong_t pwrite( int fildes, const void *buf, size_t nbyte, long offset)
{
    lseek( fildes, offset, SEEK_SET);
    return( write( fildes, buf, nbyte));
}

#if !defined(__BSD44__) && !defined(LINUX)

char *getcwd( char *buf, int size)
{
    return( _getdcwd( 0, buf, size));
}

int isatty( int fildes)
{
    return( _isatty( fildes));
}

int ftruncate( int fd, long size) 
{
    return( _chsize( fd, size));
}
  
#endif
