#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#if defined(__BSD44__)
#include <malloc.h>
#else
#include <alloca.h>
#endif
#include <math.h>
#include <string.h>
#include <anvil/geometry.h>
#include <anvil/ratio.h>
#include <anvil/display.h>
#include <anvil/macros.h>

void 
AmRot2D( float	*x,
       float	*y,
       float	theta)
{
    float	sintheta = sin( theta),
		costheta = cos( theta),
		newX, newY;
    
    newX = *x*costheta - *y*sintheta;
    newY = *x*sintheta + *y*costheta;
    *x = newX;
    *y = newY;
}

void 
AmTrans2D( float	*x,
	 float	*y,
	 float	xtrans,
	 float	ytrans)
{
    *x += xtrans;
    *y += ytrans;
}

static AnvilBoolean
rectIsect( float	x0,
	   float	y0,
	   float	x1, 
	   float	y1,
	   float	xmin,
	   float	ymin,
	   float	xmax,
	   float	ymax,
	   float	*isectX0, 
	   float	*isectY0, 
	   float	*isectX1, 
	   float	*isectY1)
{
    float	intX[ 4], intY[ 4], tmp;
    AnvilBoolean	flags[ 4], firstSet, secondSet;
    int		ii;

/* intersect against left edge */

    AmLineIntersection( x0, y0, x1, y1, xmin, ymin, xmin, ymax,
		      &intX[ 0], &intY[ 0]);
    flags[ 0] = AmBetweenFloat( intY[ 0], ymin, ymax);

/* intersect against right edge */

    AmLineIntersection( x0, y0, x1, y1, xmax, ymin, xmax, ymax,
		      &intX[ 1], &intY[ 1]);
    flags[ 1] = AmBetweenFloat( intY[ 1], ymin, ymax);

/* intersect against bottom edge */

    AmLineIntersection( x0, y0, x1, y1, xmin, ymin, xmax, ymin,
		      &intX[ 2], &intY[ 2]);
    flags[ 2] = AmBetweenFloat( intX[ 2], xmin, xmax);

/* intersect against top edge */

    AmLineIntersection( x0, y0, x1, y1, xmin, ymax, xmax, ymax,
		      &intX[ 3], &intY[ 3]);
    flags[ 3] = AmBetweenFloat( intX[ 3], xmin, xmax);

    firstSet = FALSE;
    secondSet = FALSE;
    for ( ii = 0; ii < 4; ii++) {
	if ( flags[ ii]) {
	    if ( firstSet) {
		*isectX1 = intX[ ii];
		*isectY1 = intY[ ii];
		secondSet = TRUE;
	    } else {
		*isectX0 = intX[ ii];
		*isectY0 = intY[ ii];
		firstSet = TRUE;
	    }
	}
    }
    if ( !secondSet) {
	return( FALSE);
    }
    if ( x0 == x1) {
	if ((( y0 > y1) && ( *isectY0 < *isectY1)) ||
	    (( y0 < y1) && ( *isectY0 > *isectY1))) {
	    tmp = *isectX0;
	    *isectX0 = *isectX1;
	    *isectX1 = tmp;
	    tmp = *isectY0;
	    *isectY0 = *isectY1;
	    *isectY1 = tmp;
	}
    } else {
	if ((( x0 > x1) && ( *isectX0 < *isectX1)) ||
	    (( x0 < x1) && ( *isectX0 > *isectX1))) {
	    tmp = *isectX0;
	    *isectX0 = *isectX1;
	    *isectX1 = tmp;
	    tmp = *isectY0;
	    *isectY0 = *isectY1;
	    *isectY1 = tmp;
	}
    }
    return( TRUE);
}

static void
getMatrixSample( uchar_t	*matrix, 
		 int		rows,
		 int		cols,
		 int		size,
		 float		xmin,
		 float		ymin,
		 float		xmax,
		 float		ymax,
		 uchar_t	*matrixSample,
		 int		*matrixSampleCount,
		 float		isectX0, 
		 float		isectY0, 
		 float		isectX1, 
		 float		isectY1)
{
    int		col0, row0, col1, row1, count, rowIndex, colIndex, index;
    float	dx, dy;

/* transform the coordinates into the matrix dimensions */

    col0 = AmFloatScaleToInt( xmin, xmax, isectX0, 0, cols - 1);
    col1 = AmFloatScaleToInt( xmin, xmax, isectX1, 0, cols - 1);
    row0 = rows - AmFloatScaleToInt( ymax, ymin, isectY0, 0, rows - 1);
    row1 = rows - AmFloatScaleToInt( ymax, ymin, isectY1, 0, rows - 1);

/* sample the line depending on which axis is dominant, and the 
   orientation on that axis */

    if ( abs( row1 - row0) > abs( col1 - col0)) {
	if ( row0 > row1) {
	    dx = (( float)( col0 - col1))/(( float)( row0 - row1));
	    for ( count = 0, rowIndex = row0; 
		  rowIndex > row1; 
		  rowIndex--, count++) {
		colIndex = col0 + (int)((float)count*dx);
		index = rowIndex*cols + colIndex;
		memcpy( &matrixSample[ count*size],
			&matrix[ index*size],
			size);
	    }
	} else {
	    dx = (( float)( col1 - col0))/(( float)( row1 - row0));
	    for ( count = 0, rowIndex = row0; 
		  rowIndex < row1; 
		  rowIndex++, count++) {
		colIndex = col0 + (int)((float)count*dx);
		index = rowIndex*cols + colIndex;
		memcpy( &matrixSample[ count*size],
			&matrix[ index*size],
			size);
	    }
	}
    } else {
	if ( col0 > col1) {
	    dy = (( float) ( row1 - row0))/(( float)( col0 - col1));
	    for ( count = 0, colIndex = col0; 
		  colIndex > col1; 
		  colIndex--, count++) {
		rowIndex = row0 + (int)((float)count*dy);
		index = rowIndex*cols + colIndex;
		memcpy( &matrixSample[ count*size],
			&matrix[ index*size],
			size);
	    }
	} else {
	    dy = (( float) ( row1 - row0))/(( float)( col1 - col0));
	    for ( count = 0, colIndex = col0; 
		  colIndex < col1; 
		  colIndex++, count++) {
		rowIndex = row0 + (int)((float)count*dy);
		index = rowIndex*cols + colIndex;
		memcpy( &matrixSample[ count*size],
			&matrix[ index*size],
			size);
	    }
	}
    }
    *matrixSampleCount = count;
}

static void
getImageSample( AmImageClass	*texImage, 
		float		xmin,
		float		ymin,
		float		xmax,
		float		ymax,
		ulong_t		*imageSample, 
		int		*imageSampleCount,
		float		isectX0, 
		float		isectY0, 
		float		isectX1, 
		float		isectY1)
{
    int		col0, row0, col1, row1, count, rowIndex, colIndex, index,
		imageRows, imageCols;
    float	dx, dy;
    ulong_t	*imageData = ( ulong_t *) texImage->AmGetData();


/* transform the coordinates into the matrix dimensions */

    imageRows = texImage->GetNumVerticalPixels();
    imageCols = texImage->GetNumHorizPixels();
    col0 = AmFloatScaleToInt( xmin, xmax,
			    isectX0, 0, imageCols - 1);
    col1 = AmFloatScaleToInt( xmin, xmax,
			    isectX1, 0, imageCols - 1);
    row0 = imageRows - AmFloatScaleToInt( ymax, ymin,
					isectY0, 0, imageRows - 1);
    row1 = imageRows - AmFloatScaleToInt( ymax, ymin,
					isectY1, 0, imageRows - 1);

/* sample the line depending on which axis is dominant, and the 
   orientation on that axis */

    if ( abs( row1 - row0) > abs( col1 - col0)) {
	if ( row0 > row1) {
	    dx = (( float)( col1 - col0))/(( float)( row0 - row1));
	    for ( count = 0, rowIndex = row0; 
		  rowIndex > row1; 
		  rowIndex--, count++) {
		colIndex = col0 + (int)((float)count*dx);
		index = rowIndex*imageCols + colIndex;
		imageSample[ count] = imageData[ index];
	    }
	} else {
	    dx = (( float)( col1 - col0))/(( float)( row1 - row0));
	    for ( count = 0, rowIndex = row0; 
		  rowIndex < row1; 
		  rowIndex++, count++) {
		colIndex = col0 + (int)((float)count*dx);
		index = rowIndex*imageCols + colIndex;
		imageSample[ count] = imageData[ index];
	    }
	}
    } else {
	if ( col0 > col1) {
	    dy = (( float) ( row1 - row0))/(( float)( col0 - col1));
	    for ( count = 0, colIndex = col0; 
		  colIndex > col1; 
		  colIndex--, count++) {
		rowIndex = row0 + (int)((float)count*dy);
		index = rowIndex*imageCols + colIndex;
		imageSample[ count] = imageData[ index];
	    }
	} else {
	    dy = (( float) ( row1 - row0))/(( float)( col1 - col0));
	    for ( count = 0, colIndex = col0; 
		  colIndex < col1; 
		  colIndex++, count++) {
		rowIndex = row0 + (int)((float)count*dy);
		index = rowIndex*imageCols + colIndex;
		imageSample[ count] = imageData[ index];
	    }
	}
    }
    *imageSampleCount = count;
}

AmErrorCode
AmImage3D( AmImageClass		*displayImage,		/* display image */
	 float			ex,			/* eye position */
	 float			ey,
	 float			ez,
	 float			focal,			/* screen focal length */
	 float			theta,			/* horizontal rotation */
	 float			phi,			/* vertical rotation */
	 float			screenWidth,		/* screen dimensions */
	 float			screenHeight,
	 uchar_t		*matrix,		/* elevation matrix */
	 int			rows,			/* dimensions and */
	 int			cols,			/* element size */
	 int			size,
	 AmAccessFn		zFn,			/* accessing function */
	 void			*clientData,		/* client data */
	 float			xmin,			/* extents of image */
	 float			ymin,			/* and matrix */
	 float			xmax,
	 float			ymax,
	 AmImageClass		*texImage)		/* textured image */
{
    uchar_t	*matrixSample;
    ulong_t	*imageSample, *imageData = ( ulong_t *) displayImage->AmGetData();
    int		matrixSampleLength, imageSampleLength, 
		matrixSampleCount, imageSampleCount, imageX,
		matrixIndex, imageIndex, imageY;
    float	screenX0, screenY0, screenX1, screenY1, screenDX, screenDY,
		isectX0, isectY0, isectX1, isectY1,
		screenX, screenY, scanlineX0, scanlineY0, scanlineX1, scanlineY1, 
		d0, d1, matrixSpacing, scanlineX, scanlineY,
		scanlineDX, scanlineDY, matrixX, matrixY, imageSlope,
		matrixSlope, maxMatrixSlope, scale;
    AnvilBoolean	matrixEndFlag = FALSE;

    AN_DEBUG( 99, ("eye = (%.2f, %.2f, %.2f), focal = %.2f\n", 
		   ex, ey, ez, focal));
    AN_DEBUG( 99, ("theta = %.2f, phi = %.2f\n", theta, phi));
    AN_DEBUG( 99, ("screenWidth,Height = %.2f, %.2f\n", 
		   screenWidth, screenHeight));
    AN_DEBUG( 99, ("rows, cols = %d, %d\n", rows, cols));
    AN_DEBUG( 99, ("coords = (%.2f, %.2f, %.2f, %.2f)\n", 
		   xmin, ymin, xmax, ymax));


/* allocate the matrix and image sampling arrays. */

    matrixSampleLength = MAX( rows, cols)*2;
    imageSampleLength = MAX( texImage->GetNumHorizPixels(),
			     texImage->GetNumVerticalPixels())*2;
    matrixSample = ( uchar_t *) alloca( matrixSampleLength*size);
    if ( matrixSample == NULL) {
	DO_RETURN_ERROR( AmALLOCA_FAILED);
    }
    imageSample = ( ulong_t *) alloca( imageSampleLength*sizeof( ulong_t));
    if ( imageSample == NULL) {
	DO_RETURN_ERROR( AmALLOCA_FAILED);
    }

/* scale the screen width and focal length to the size of the matrix being
   rendered to get rid of roundoff errors */

    scale = MAX( xmax - xmin, ymax - ymin);
    focal *= scale;
    screenWidth *= scale;
    screenHeight *= scale;

/* find out what the left and right edges of the screen mean in the real world */

    screenX0 = focal;
    screenY0 = -screenWidth/2;
    screenX1 = focal;
    screenY1 = screenWidth/2;
    AmRot2D( &screenX0, &screenY0, theta);
    AmRot2D( &screenX1, &screenY1, theta);
    AmTrans2D( &screenX0, &screenY0, ex, ey);
    AmTrans2D( &screenX1, &screenY1, ex, ey);

/* find the coordinate increment for each screen pixel */

    screenDX = fabs( screenX1 - screenX0)/displayImage->GetNumHorizPixels();
    screenDY = fabs( screenY1 - screenY0)/displayImage->GetNumHorizPixels();

/* in the 2-D plane, take the screen height and vertical rotation, and 
   determine the endpoints of the vertical scanline in the coordinate
   space relative to the slice */

    scanlineX0 = focal;
    scanlineY0 = -screenHeight/2;
    scanlineX1 = focal;
    scanlineY1 = screenHeight/2;
    AmRot2D( &scanlineX0, &scanlineY0, phi);
    AmRot2D( &scanlineX1, &scanlineY1, phi);
    AmTrans2D( &scanlineX0, &scanlineY0, 0.0, ez);
    AmTrans2D( &scanlineX1, &scanlineY1, 0.0, ez);
    scanlineDX = fabs( scanlineX1 - scanlineX0)/
			displayImage->GetNumVerticalPixels();
    scanlineDY = fabs( scanlineY1 - scanlineY0)/
			displayImage->GetNumVerticalPixels();

/* walk through each vertical scanline on the screen */

    for ( screenX = screenX0, screenY = screenY0, imageX = 0;
	  imageX < displayImage->GetNumHorizPixels();
	  screenX += screenDX, screenY += screenDY, imageX++) {
	if ( !rectIsect( ex, ey, screenX, screenY, 
			 xmin, ymin, xmax, ymax, 
		         &isectX0, &isectY0, &isectX1, &isectY1)) {
	    continue;
	}
	getMatrixSample( matrix, rows, cols, size, 
			 xmin, ymin, xmax, ymax, 
			 matrixSample, &matrixSampleCount,
			 isectX0, isectY0, isectX1, isectY1);
	if ( matrixSampleCount <= 0) {
	    continue;
	}
	getImageSample( texImage, xmin, ymin, xmax, ymax,
			imageSample, &imageSampleCount,
			isectX0, isectY0, isectX1, isectY1);
	if ( imageSampleCount <= 0) {
	    continue;
	}
	d0 = AmDist3D( ex, ey, ez, isectX0, isectY0, 0.0);
	d1 = AmDist3D( ex, ey, ez, isectX1, isectY1, 0.0);
	matrixX = d0;
	matrixIndex = 0;
	matrixSpacing = ( d1 - d0)/matrixSampleCount;
	matrixEndFlag = FALSE;
	matrixY = ( zFn)( matrixSample, clientData);
	maxMatrixSlope = ( matrixY - ez)/matrixX;

/* walk up the scanline from the bottom */

	for ( imageY = displayImage->GetNumVerticalPixels() - 1, 
		    scanlineX = scanlineX0, scanlineY = scanlineY0;
	      imageY >= 0;
	      imageY--, scanlineX += scanlineDX, scanlineY += scanlineDY) {

/* calculate the slope from the viewpoint to the pixel */

	    imageSlope = ( scanlineY - ez)/scanlineX;
	    if ( imageSlope < maxMatrixSlope) {
		continue;
	    }

/* loop over the matrix until we've gotten to the desired slope */

	    do {
		matrixY = ( zFn)( &matrixSample[ matrixIndex*size], clientData);
		matrixSlope = ( matrixY - ez)/matrixX;
		if ( matrixSlope >= imageSlope) {
		    break;
		}
		matrixIndex++;
		matrixX += matrixSpacing;

/* or reached the end of the matrix sample... */

		if ( matrixIndex == matrixSampleCount) {
		    matrixEndFlag = TRUE;
		    break;
		}
	    } while ( TRUE);

/* transform the matrix sample point into the texture image, and copy the
   pixel from the texture image to the display image */

	    if ( !matrixEndFlag) {
		imageIndex = ( imageSampleCount*matrixIndex)/matrixSampleCount;
		imageData[ imageY*displayImage->GetNumHorizPixels() + imageX] =
		    imageSample[ imageIndex];
	    } else {
		break;
	    }
	}
    }
    return( AmOK);
}
