#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */






#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/pixthing.h>

/*************************************************************************
 *
 * Description:
 * 	read an X11 bitmap file into image data 
 * Arguments:
 *      filename - X11 bitmap format file.
 *      width, height - reurned width and height.
 *      data - pointer to returned raw image data.
 *	fg, bg - foreground and background of bitmap.
 * Returns:
 *      TRUE on success, FALSE otherwise.
 * Modifications:
 *	should return an AmErrorCode.  The pixmap should be an unsigned 
 *	long matrix, not a uchar.  Should also check mallocs, and alloca 
 *	the temporary buffers.
 *
 *************************************************************************/

AnvilBoolean 
AmReadBitmapFileData( char      *filename,
		      int	*width,
		      int	*height,
		      uchar_t	**data,
		      ulong_t	fg,
		      ulong_t	bg)
{
    FILE	*fp;
    char	line[ 512], define[ 128], var[ 128], *p; 
    uchar_t	*dataPtr, *dataEnd;
    int		*hexvalVec, *hexPtr, mask, ii, bytes;

    if (( fp = fopen( filename, "r")) == NULL) {
	return( FALSE);
    }

/* read the width and height from the file, look for #define starting the
   line and the words "width" & "height". stop when you hit the word "static" */

    do {
	if ( fgets( line, sizeof( line), fp) == NULL) {
	    fclose( fp);
	    return( FALSE);
	}
	if ( !strncmp( line, "#define", strlen( "#define"))) {
	    if ( strstr( (const char *)line, "width")) {
		if ( sscanf( line, "%s %s %d", define, var, width) != 3) {
		    fclose( fp);
		    return( FALSE);
		}
	    } else if ( strstr( (const char *)line, "height")) {
		if ( sscanf( line, "%s %s %d", define, var, height) != 3) {
		    fclose( fp);
		    return( FALSE);
		}
	    }
	} else if ( !strncmp( line, "static", strlen( "static"))) {
	    break;
	}
    } while ( !feof( fp));

/* step to just after the starting curly brace, then malloc the data for
   the image */

    p = strchr( ( const char *)line, '{');
    p++;
    dataPtr = *data = ( uchar_t *) malloc((*width)*(*height));

/* bitmaps are stored padded to the nearest byte */

    bytes = ((*width + 7) >> 3); 
    hexPtr = hexvalVec = ( int *) malloc( sizeof( int)*bytes*(*height)); 
    do {

/* scan to the start of the next number */

	for ( ; (( *p != '\0') && ( *p != '0')); p++);

/* reached end of line? read a new line and set the pointer */

	if ( *p == '\0') {
	    if ( fgets( line, 512, fp) == NULL) {
		break;
	    } else {
		p = line;
	    }
	} else {

/* scan the hex value, then scan to the end of the number */

	    sscanf( p, "%i", hexPtr);
	    hexPtr++;
	    for ( ; (( *p != '\0') && ( *p != ',')); p++);
	}
    } while ( !feof( fp));

/* use the mask for each of the bits to turn them to bytes */

    hexPtr = hexvalVec;
    for ( ii = 0; ii < *height; ii++) {
	dataEnd = dataPtr + *width;
	while ( dataPtr < dataEnd) {

/* walk down to the end of the row, but skip the bit padding */ 

	    for ( mask = 0x1; 
		  (mask != 0x100) && (dataPtr != dataEnd); 
		  mask <<= 1, dataPtr++) {
		*dataPtr = (( *hexPtr & mask) != 0) ? fg : bg;
	    }
	    hexPtr++;
	}
    }
    FREE( hexvalVec);
    fclose( fp);
    return( TRUE);
}

