#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <malloc.h>
#include <string.h>
#include <sys/types.h>
#include <anvil/macros.h>
#include <anvil/display.h>
#include <anvil/image.h>
#include <anvil/imageP.h>

/* read from a source file with color function and size to an
   8-bit destination image */

AmErrorCode 
AmReadSubmatrix( int			fd,
		 int			srcRows,
	         int			srcCols,
	         AmRectangleRec const	*srcRect,
	         uchar_t		*matrix,
	         int			matRows,
	         int			matCols,
	         int			dstX,
	         int			dstY,
	         int			size,
	         AmScanlineFn		readScanFn,
	         void			*readScanClientData)
{
    int		  bufWidth, yy;
    AnvilBoolean	  okReadFlag;
    uchar_t	  *buffer, *tmp;

/* check to make sure the source and destination rectangles are in bounds */

    if (( srcRect->x < 0) ||
	( srcRect->y < 0) ||
	( srcRect->x + srcRect->width > srcCols) ||
	( srcRect->y + srcRect->height > srcRows) ||
	( dstX + srcRect->width > srcCols) ||
	( dstY + srcRect->height > srcRows)) {
	DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }

    matrix += ( dstX + dstY*matCols)*size;
    bufWidth = ( int) srcRect->width + 4;
    if (( buffer = ( unsigned char *) malloc( bufWidth*size)) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    tmp = ( uchar_t *) malloc( size);

/* loop until the end of the destination subimage, setting pointers at the
   start and end of each row.  Use the vertical and horizontal forward
   differencing vectors to increment through the source image */

    okReadFlag = TRUE;

/* seek to the next source scanline, by the index vector */
   
    for ( yy = srcRect->y; yy < srcRect->y + srcRect->height; yy++) {

/* set the vertical index depending on whether the rows are flipped */

	if ( !( readScanFn)( fd, buffer, size, srcRect->width, srcCols,
			     srcRect->x, yy, readScanClientData)) {
	    okReadFlag = FALSE;
	    break;
	}
	memcpy( matrix, buffer, srcRect->width*size);
	matrix += matCols*size;
    }
    free( buffer);
    free( tmp);
    ANVIL_ERROR_CODE = (okReadFlag) ? AmOK : AmREAD_FAILED;
    return( ANVIL_ERROR_CODE);
}
