#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */





#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/gif.h>
#include <anvil/gifP.h>
#include "defines.h"

/* read a GIF file */

AnvilBoolean  ReadGIF( FILE		*fd,
	          int		imageNumber,
		  unsigned char	**image,
		  int		*width,
		  int		*height,
		  unsigned char	localColorMap[ 3][ MAXCOLORMAPSIZE],
		  int		*numColors)

{
    unsigned char	buf[ 16], c;
    int			useGlobalColormap, bitPixel, imageCount = 0;
    char		version[ 4];

    if (! ReadOK( fd, buf, 6)) {
	return( FALSE);
    }

    if ( strncmp(( char *) buf,"GIF",3) != 0) {
	return( FALSE);
    }

    strncpy(version, ( char *) buf + 3, 3);
    version[3] = '\0';

    if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0)) {
	return( FALSE);
    }

    if (! ReadOK(fd,buf,7)) {
	return( FALSE);
    }

    GifScreen.Width           = LMToUint(buf[0],buf[1]);
    GifScreen.Height          = LMToUint(buf[2],buf[3]);
    GifScreen.BitPixel        = 2<<(buf[4]&0x07);
    GifScreen.ColorResolution = (((buf[4]&0x70)>>3)+1);
    GifScreen.Background      = buf[5];
    GifScreen.AspectRatio     = buf[6];

    if (BitSet(buf[4], LOCALCOLORMAP)) {	/* Global Colormap */
	if (ReadColorMap(fd,GifScreen.BitPixel,GifScreen.ColorMap))
	    fprintf( stderr, "error reading global colormap \n" );
    }

    if (GifScreen.AspectRatio != 0 && GifScreen.AspectRatio != 49) {
	fprintf( stderr, "warning - non-square pixels \n\n");
    }

    for (;;) {
	if (! ReadOK(fd,&c,1)) {
	    return( FALSE);
	}

	if (c == ';') {		/* GIF terminator */
	    if (imageCount < imageNumber) {
		return( FALSE);
	    }
	    return( TRUE);
	}

	if (c == '!') { 	/* Extension */
	    if (! ReadOK(fd,&c,1)) {
		fprintf( stderr, "EOF / read error on extension function code \n");
	    }
	    DoExtension(fd, c);
	    continue;
	}

	if (c != ',') {		/* Not a valid start character */
	    continue;
	}
	++imageCount;
	if (! ReadOK(fd,buf,9)) {
	    return( FALSE);
	}

	useGlobalColormap = ! BitSet(buf[8], LOCALCOLORMAP);
	bitPixel = 1<<((buf[8]&0x07)+1);

/* set colors, width, and height */

	*numColors = GifScreen.BitPixel;
	*width = LMToUint(buf[4],buf[5]);
	*height = LMToUint(buf[6],buf[7]);
	if (! useGlobalColormap) {
	    if (ReadColorMap(fd, bitPixel, localColorMap)) {
		fprintf( stderr, "error reading local colormap\n" );
		return( FALSE);
	    }
	    ReadImage( fd, *width, *height, BitSet(buf[8], INTERLACE), 
		       imageCount != imageNumber, image);
	} else {
	    memcpy( localColorMap, GifScreen.ColorMap, 3*MAXCOLORMAPSIZE);
	    ReadImage( fd, *width, *height, BitSet(buf[8], INTERLACE), 
		       imageCount != imageNumber, image);
	}
    }
}

static int ReadColorMap( FILE		*fd,
			 int		number,
			 unsigned char	buffer[ 3][ MAXCOLORMAPSIZE])
{
    int			i;
    unsigned char	rgb[ 3];

    for (i = 0; i < number; ++i) {
	if (! ReadOK( fd, rgb, sizeof(rgb)))
	    fprintf( stderr, "bad colormap \n" );

	buffer[ CM_RED][ i] = rgb[ 0] ;
	buffer[ CM_GREEN][ i] = rgb[ 1] ;
	buffer[ CM_BLUE][ i] = rgb[ 2] ;
    }
    return FALSE;
}

/* step over the various extensions */

static int DoExtension( FILE	*fd,
			int	label)
{
    static char	buf[256];

    switch (label) {
    case 0x01:		/* Plain Text Extension */
	break;
    case 0xff:		/* Application Extension */
	break;
    case 0xfe:		/* Comment Extension */
	while (GetDataBlock(fd, (unsigned char*) buf) != 0);
	return FALSE;
    case 0xf9:		/* Graphic Control Extension */
	(void) GetDataBlock(fd, (unsigned char*) buf);
	Gif89.disposal    = (buf[0] >> 2) & 0x7;
	Gif89.inputFlag   = (buf[0] >> 1) & 0x1;
	Gif89.delayTime   = LMToUint(buf[1],buf[2]);
	if ((buf[0] & 0x1) != 0) {
	    Gif89.transparent = buf[3];
	}
	while (GetDataBlock(fd, (unsigned char*) buf) != 0);
	return FALSE;
    default:
	break;
    }
    while (GetDataBlock(fd, (unsigned char*) buf) != 0);
    return FALSE;
}

int	ZeroDataBlock = FALSE;

static int GetDataBlock( FILE		*fd,
			 unsigned char 	*buf)
{
    unsigned char	count;

    if (! ReadOK(fd,&count,1)) {
	fprintf( stderr, "error in getting DataBlock size " );
	return -1;
    }
    ZeroDataBlock = count == 0;
    if ((count != 0) && (! ReadOK(fd, buf, count))) {
	fprintf( stderr, "error in reading DataBlock " );
	return -1;
    }
    return count;
}

static int GetCode( FILE	*fd,
		    int		codeSize,
		    int		flag)
{
    static unsigned char	buf[280];
    static int			curbit, lastbit, done, lastByte;
    int				i, j, ret;
    unsigned char		count;

    if (flag) {
	curbit = 0;
	lastbit = 0;
	done = FALSE;
	return 0;
    }

    if ( (curbit+codeSize) >= lastbit) {
	if (done) {
	    if (curbit >= lastbit) {
		fprintf( stderr, "ran off the end of my bits \n" );
	    }
	    return -1;
	}
	buf[0] = buf[lastByte-2];
	buf[1] = buf[lastByte-1];

	if ((count = GetDataBlock(fd, &buf[2])) == 0) {
	    done = TRUE;
	}

	lastByte = 2 + count;
	curbit = (curbit - lastbit) + 16;
	lastbit = (2+count)*8 ;
    }

    ret = 0;
    for (i = curbit, j = 0; j < codeSize; ++i, ++j) {
	ret |= ((buf[ i / 8 ] & (1 << (i % 8))) != 0) << j;
    }
    curbit += codeSize;
    return ret;
}

/* read an encoded byte from the GIF image */

static int LWZReadByte( FILE	*fd,
			int	flag,
			int	inputCodeSize)
{
    static int		fresh = FALSE;
    int			code, incode;
    static int		codeSize, setCodeSize;
    static int		maxCode, maxCodeSize;
    static int		firstcode, oldcode;
    static int		clearCode, endCode;
    static int		table[2][(1<< MAX_LWZ_BITS)];
    static int		stack[(1<<(MAX_LWZ_BITS))*2], *sp;
    register int	i;

    if (flag) {
	setCodeSize = inputCodeSize;
	codeSize = setCodeSize+1;
	clearCode = 1 << setCodeSize ;
	endCode = clearCode + 1;
	maxCodeSize = 2*clearCode;
	maxCode = clearCode+2;
	GetCode(fd, 0, TRUE);
	fresh = TRUE;
	for (i = 0; i < clearCode; ++i) {
	    table[0][i] = 0;
	    table[1][i] = i;
	}
	for (; i < (1<<MAX_LWZ_BITS); ++i) {
	    table[0][i] = table[1][0] = 0;
	}
	sp = stack;
	return 0;
    } else if (fresh) {
	fresh = FALSE;
	do {
	    firstcode = oldcode = GetCode(fd, codeSize, FALSE);
	} while (firstcode == clearCode);
	return firstcode;
    }

    if (sp > stack) {
	return *--sp;
    }

    while ((code = GetCode(fd, codeSize, FALSE)) >= 0) {
	if (code == clearCode) {
	    for (i = 0; i < clearCode; ++i) {
		table[0][i] = 0;
		table[1][i] = i;
	    }
	    for (; i < (1<<MAX_LWZ_BITS); ++i) {
		table[0][i] = table[1][i] = 0;
	    }
	    codeSize = setCodeSize+1;
	    maxCodeSize = 2*clearCode;
	    maxCode = clearCode+2;
	    sp = stack;
	    firstcode = oldcode = GetCode(fd, codeSize, FALSE);
	    return firstcode;
	} else if (code == endCode) {
	    int			count;
	    unsigned char	buf[260];

	    if (ZeroDataBlock) {
		return -2;
	    }

	    while ((count = GetDataBlock(fd, buf)) > 0);
	    if (count != 0) {
		fprintf( stderr, "missing EOD in data stream (common) \n");
	    }
	    return -2;
	}
	incode = code;
	if (code >= maxCode) {
	    *sp++ = firstcode;
	    code = oldcode;
	}

	while (code >= clearCode) {
		*sp++ = table[1][code];
		if (code == table[0][code])
			fprintf( stderr, "circular table entry BIG ERROR \n");
		code = table[0][code];
	}

	*sp++ = firstcode = table[1][code];

	if ((code = maxCode) <(1<<MAX_LWZ_BITS)) {
		table[0][code] = oldcode;
		table[1][code] = firstcode;
		++maxCode;
		if ((maxCode >= maxCodeSize) &&
			(maxCodeSize < (1<<MAX_LWZ_BITS))) {
			maxCodeSize *= 2;
			++codeSize;
		}
	}

	oldcode = incode;

	if (sp > stack)
		return *--sp;
}
	return code;
}

static int ReadImage( FILE		*fd,
		      int		width, 
		      int		height,
		      int		interlace, 
		      int		ignore,
		      unsigned char 	**image)
{
    unsigned char	c;	
    int			v, xpos = 0, ypos = 0, pass = 0;

    *image = ( unsigned char *) malloc( width*height);
    if ( *image == NULL) {
	fprintf( stderr, "failed to allocate memory for image \n\n");
	return( FALSE);
    }
    /*
    **  Initialize the Compression routines
    */
    if (! ReadOK(fd,&c,1)) {
	fprintf( stderr, "EOF / read error on image data \n" );
    }
    if (LWZReadByte(fd, TRUE, c) < 0) {
	fprintf( stderr, "error reading image \n" );
    }

    /*
    **  If this is an "uninteresting picture" ignore it.
    */
    if ( ignore) {
	while (LWZReadByte(fd, FALSE, c) >= 0);
	return( FALSE);
    }

    while ((v = LWZReadByte(fd,FALSE,c)) >= 0 ) {
	(*image)[ ypos*width + xpos] = v;
	++xpos;
	if (xpos == width) {
	    xpos = 0;
	    if (interlace) {
		switch (pass) {
		case 0:
		case 1:
			ypos += 8; break;
		case 2:
			ypos += 4; break;
		case 3:
			ypos += 2; break;
		}

		if (ypos >= height) {
		    ++pass;
		    switch (pass) {
		    case 1:
			    ypos = 4; break;
		    case 2:
			    ypos = 2; break;
		    case 3:
				ypos = 1; break;
		    default:
			    goto fini;
		    }
		}
	    } else {
		++ypos;
	    }
	}
	if (ypos >= height) {
	    break;
	}
    }
    printf( "successful image read\n");
    return( TRUE);

fini:
    if ( LWZReadByte( fd,FALSE,c)>=0) {
	fprintf( stderr, "too much input data, ignoring extra... \n");
    }
    return( TRUE);
}
