#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <anvil/list.h>
#include <anvil/errcode.h>

/*********************************************************************
 *
 * Description:
 * 	copy a list and its data, assuming the data is a block of 
 *	memory of fixed size 
 * Arguments:
 *	lst - list to copy.
 *	elSize - size of each element to copy.
 * Returns:
 *	a copy of the list with all the elements copied.
 * Modifications:
 *	should return an error if any of the mallocs() fail.
 *
 *********************************************************************/

AmListPtr 
AmCopyLst( AmListPtr 	lst,
	 int 		elSize)
{
    AmListPtr 	head, walk;

/* allocate the head of the list and copy the AmFirst element of the other 
   list to it */

    if ( lst == NULL) {
	return( NULL);
    }
    head = walk = ( AmListPtr) malloc( sizeof( AmListRec));
    if ( head == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    head->data = ( char *) malloc( elSize);
    if ( head->data == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    memcpy( head->data, lst->data, elSize);
    lst = lst->next;

/* walk down the list, spurting out nodes and copying data */

    while ( lst != NULL) {
	walk->next = ( AmListPtr) malloc( sizeof( AmListRec));
	if ( walk->next == NULL) {
	    DO_ERROR( AmMALLOC_FAILED);
	    return( NULL);
	}
	walk = walk->next;
	walk->data = ( char *) malloc( elSize);
	if ( walk->data == NULL) {
	    DO_ERROR( AmMALLOC_FAILED);
	    return( NULL);
	}
	memcpy( walk->data, lst->data, elSize);
	lst = lst->next;
    }

/* terminate the list and return the head. */

    walk->next = NULL;
    return( head);
}
