#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/list.h>

/*********************************************************************
 *
 * Description:
 * 	delete an element from a list and free its data, using a 
 *	target, a comparision function, and a free function which 
 *	is applied to the delete item.  This function deletes all
 *	successful matches.
 * Arguments:
 *	target - target to search for.
 *	lst - list to search and delete elements from.
 *	cmpFn - comparison function to use for list elements.
 *	freeFn - function to call on the data pointer for each 
 *	successful match to free the data.
 * Returns:
 *	the original list with the elements removed.  Note that
 *	the AmFirst element might get deleted, so the pointer may 
 *	change and should be reset.
 *
 *********************************************************************/

AmListPtr 
AmDeleteItemLst( void 	*target,
	       AmListPtr 	lst,
	       AmSearchFn cmpFn,
	       AmItemFn	freeFn)
{
    AmListPtr 	prevNode, head;

    if ( lst == NULL) {
	return( NULL);
    }
    while (( lst != NULL) && ( cmpFn)( target, lst->data)) {
	prevNode = lst;
	lst = lst->next;	
	if ( freeFn != NULL) {
	    ( freeFn)( prevNode->data);
	}
	FREE( prevNode);
    }
    if ( lst == NULL) {
    	return( NULL);
    }

/* save the head of the list */

    head = lst;		/* result is always head of list */
    prevNode = lst;
    lst = lst->next;
    while ( lst != NULL) {

/* hook over the previous node, delete the node and return the head */

	if ( ( cmpFn)( target, lst->data)) {
	    prevNode->next = lst->next;
	    if ( freeFn != NULL) {
		( freeFn)( lst->data);
	    }
	    FREE( lst);
	    lst = prevNode->next;
	} else {
	    lst = lst->next;
	    prevNode = prevNode->next;
	}
    }
    return( head);
}
