#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/list.h>

/*********************************************************************
 *
 * Description:
 * 	delete an element from a list, and free the data segment of 
 *	the node with a single free call 
 * Arguments:
 *	target - target to search for 
 *	lst - list to delete the element from.
 *	cmpFn - comparison function to use for comparing elements.
 * Returns:
 *      the original list with the elements removed.  Note that
 *      the AmFirst element might get deleted, so the pointer may
 *      change and should be reset.
 * Modifications:
 *	the body of this function could be replaced with a call to
 *	AmDeleteItemLst()
 *
 *********************************************************************/


AmListPtr 
AmDelElLst( void 		*target,
	  AmListPtr 	lst,
	  AmSearchFn	cmpFn)
{
    AmListPtr 	prevNode, head;

    if ( lst == NULL) {
	return( NULL);
    }
    while ((cmpFn)( target, lst->data)) {
	prevNode = lst;
	lst = lst->next;
	FREE( prevNode->data);
	FREE( prevNode);
	if ( lst == NULL) {
	    return( NULL);
	}
    }

/* save the head of the list */

    head = lst;
    prevNode = lst;
    lst = lst->next;
    while ( lst != NULL) {

/* hook over the previous node, delete the node and return the head */

	if (( cmpFn)( target, lst->data)) {
	    prevNode->next = lst->next;
	    FREE( lst->data);
	    FREE( lst);
	    lst = prevNode->next;
	} else {
	    lst = lst->next;
	    prevNode = prevNode->next;
        }
    }
    return( head);
}
