#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/list.h>

/*********************************************************************
 *
 * Description:
 * 	filter a list according to a filter function.  This applies
 *	free() to each of the filtered elements..
 * Arguments:
 *	lst - list to filter.
 *	filterElFn - AnvilBoolean function to apply for filtering.
 * Returns:
 *	the original list with the elements removed.
 * Modifications:
 *	Looks like yet another duplicate of deleteItemLst()
 *
 ****************************************************************************/

AmListPtr 
AmFilterLst( AmListPtr 		lst, 
	     AmElementFn	filterElFn)
{
    AmListPtr 	prevNode, lstHead, deadNode;

/* now do the normal filtering loop. */

    prevNode = NULL;
    lstHead = NULL;
    while ( lst != NULL) {
	if ( !( filterElFn)( lst->data)) {

/* if it's the head of the list, step it down. */

	    if ( lstHead == NULL) {
		deadNode = lst;
		lst = lst->next;
		FREE( deadNode->data);
		FREE( deadNode);

/* otherwise step over the previous node. */

	    } else {
		prevNode->next = lst->next;
		FREE( lst->data);
		FREE( lst);
		lst = prevNode->next;
	    }
	} else {

/* if the list head hasn't been assigned to a live node, assign it, and
   step to the next node */

	    if ( lstHead == NULL) lstHead = lst;
	    prevNode = lst;
	    lst = lst->next;
	}
    }
    return( lstHead);
}

/***************************************************************************
 *
 * Description:
 * 	create a copy of the list by applying a filter function 
 * Arguments:
 *	lst - list to filter.
 *	filterElFn - AnvilBoolean function to apply for filtering.
 * Returns:
 *	A copy of the list with the elements filtered out.
 * Modifications:
 *	should return an error if the new list couldn't be allocated.
 *
 ****************************************************************************/

AmListPtr 
AmFilterCopyLst( AmListPtr	lst,
		 AmElementFn	filterElFn)
{
    AmListPtr	head = NULL, ptr;

    for ( ;lst != NULL; lst = lst->next) {
	if (( filterElFn)( lst->data)) {
	    if ( head == NULL) {
		head = ( AmListPtr) malloc( sizeof( AmListRec));
		if ( head == NULL) {
		    DO_ERROR( AmMALLOC_FAILED);
		    return( NULL);
		}
		ptr = head;
	    } else {
		ptr->next = ( AmListPtr) malloc( sizeof( AmListRec));
		if ( ptr->next == NULL) {
		    DO_ERROR( AmMALLOC_FAILED);
		    return( NULL);
		}
		ptr = ptr->next;
	    }
	    ptr->data = lst->data;
	}
    }
    if ( head != NULL) {
	ptr->next = NULL;
    }
    return( head);
}
