#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* utility functions for freeing generic lists and their elements. */

#include <stdio.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/list.h>

/*************************************************************************
 *
 * Description:
 * 	free an entire list, given a freeing function for each node 
 * Arguments:
 * 	lst - list to free.
 *	nodeFreeFn - free function to apply to each element
 *
 ****************************************************************************/

void 
AmFreeLst( AmListPtr 	lst,
	   AmItemFn	nodeFreeFn)

{
    AmListPtr 	deadNode;

    while ( lst) {
	if ( nodeFreeFn != NULL) {
	    ( nodeFreeFn)( lst->data);
	}
	deadNode = lst;
	lst = lst->next;
	FREE( deadNode);
    }
}

/*************************************************************************
 *
 * Description:
 * 	just free the list, don't free any of the nodes 
 * Arguments:
 * 	lst - list to free.
 *
 ****************************************************************************/

void AmJustFreeLst( AmListPtr 	lst)
{
    AmListPtr 	deadNode;

    while ( lst) {
	deadNode = lst;
	lst = lst->next;
	FREE( deadNode);
    }
}

/*************************************************************************
 *
 * Description:
 * 	chop off the head of a list 
 * Arguments:
 * 	lst - list to guillotine.
 *
 ****************************************************************************/

AmListPtr AmFreeHead( AmListPtr	lst)
{
    AmListPtr	deadNode;

    if ( lst == NULL) {
	return( NULL);
    }
    deadNode = lst;
    lst = lst->next;
    FREE( deadNode);
    return( lst);
}
