#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <anvil/project.h>
#include <anvil/list.h>

/*************************************************************************
 *
 * Description:
 * 	return a pointer to the data element at the end of the list
 * Arguments:
 *	lst - list to get the AmLast element from.
 * Returns:
 *	the data pointed to by the AmLast element in the list, or NULL
 *	if the list is NULL.
 *
 *************************************************************************/

void *
AmLast( AmListPtr	lst)
{
    if ( lst == NULL) {
	return( NULL);
    }
    while ( lst->next != NULL) {
	lst = lst->next;
    }
    return( lst->data);
}

/*************************************************************************
 *
 * Description:
 * 	return a pointer to the data element at the head of the list
 * Arguments:
 *	lst - list to get the AmFirst element from.
 * Returns:
 *	the data pointed to by the AmFirst element in the list, or NULL
 *	if the list is NULL.
 *
 *************************************************************************/

void *
AmFirst( AmListPtr	lst)
{
    if ( lst == NULL) {
	return(NULL);
    }
    return( lst->data);
}

/*************************************************************************
 *
 * Description:
 * 	return a pointer to the data element for the next to AmLast element
 *	in the list.
 * Arguments:
 *	lst - list to get the next to AmLast element from.
 * Returns:
 *	the data pointed to by the next to AmLast element in the list, 
 *	or NULL if the list is NULL, or the second element is NULL.
 *
 *************************************************************************/

void *
AmNextToLast( AmListPtr lst)
{
    if ( lst == NULL) {
	return( NULL);
    }
    if ( lst->next == NULL) {
	return( NULL);
    }
    for ( ; lst->next !=NULL; lst = lst->next);
    return( lst->data);
}

/*************************************************************************
 *
 * Description:
 * 	return the AmLast list pointer in the list.
 * Arguments:
 *	lst - list to get the AmLast pointer from.
 * Returns:
 *	the AmLast list pointer in the list, or NULL if the list is NULL.
 *
 *************************************************************************/

AmListPtr 
AmLastLst( AmListPtr	lst)
{
    if ( lst == NULL) {
	return( lst);
    } else {
	while ( lst->next != NULL) {
	    lst = lst->next;
	}
	return( lst);
    }
}
