#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/list.h>
#include <anvil/errcode.h>

/***********************************************************************
 *
 * Description:
 * 	create a vector of elements from a list of elements 
 * Arguments:
 *	list - list to make the vector from.
 * Returns:
 *	A vector of void pointers of the list elements
 * Modifications:
 * 	should return an error code if the malloc() fails.
 *
 ***********************************************************************/

void **
AmMakeVectorLst( AmListPtr 	list)
{
    int 	length;
    void 	**vector, **vecPtr;

/* allocate the vector */

    length = AmLengthLst( list) + 1;
    vecPtr = vector = ( void **) malloc( sizeof( void *) * length);
    if ( vector == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* walk down the list and set the elements */

    while ( list != NULL) {
	*vecPtr++ = list->data;
	list = list->next;
    }
    *vecPtr = NULL;

/* return the vector */

    return( vector);
}

/***********************************************************************
 *
 * Description:
 * 	copy each ( static sized, simple) element 
 * Arguments:
 * 	list - list of elements to copy.
 *	elSize - size of each of the elements.
 * Returns:
 *	a copy of the list and its elements into a vector.
 * Modifications:
 *	should return an error if the malloc() fails
 *
 ***********************************************************************/

void *
AmCopyVectorLst( AmListPtr 	list, 
	         int 	        elSize)
{
    char 	*vector, *vecptr;

    if ( list == NULL) {
	return( NULL);
    }

/* allocate the vector */

    vecptr = vector = ( char *) malloc( elSize*AmLengthLst( list));
    if ( vector == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* walk down the list and set the elements */

    for ( ; list != NULL; list = list->next) {
	memcpy( vecptr, list->data, elSize);
	vecptr += elSize;
    }

/* return the vector */

    return(( void *) vector);
}
