#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* sort a list according to a comparison function. */

#include <stdio.h>
#include <anvil/project.h>
#include <anvil/list.h>
#include <anvil/errcode.h>

/* function set for merge sorting a singly linked list */

static AmListPtr splitLst( 
    AmListPtr a
);

/***********************************************************************
 *
 * Description:
 * 	merge a pair of sorted lists.
 * Arguments:
 *	a, b - lists sorted on cmpFn.
 *	cmpFn - comparison function to sort on.
 * Returns:
 * 	the merge of the two lists (destructive)
 *
 ************************************************************************/

static AmListPtr 
listMerge( AmListPtr 	a, 
	   AmListPtr 	b, 
	   AmSearchFn	cmpFn)
{
    AmListPtr 	c, d = NULL;

    c = NULL;
    if ( a == NULL) return( b);		/* if one is null, return other */
    if ( b == NULL) return( a);
    while (( a != NULL) && ( b != NULL)) {	/* loop while both exist */
	if ( ( cmpFn)( a->data, b->data) < 0) {
	    if ( c == NULL) {			/* create list head */
		c = a;
		d = c;
	    } else {
		c->next = a;			/* or link and step */
		c = c->next;
	    }
	    a = a->next;
	} else {
	    if ( c == NULL) {			/* same for b */
		c = b;
		d = c;
	    } else {
		c->next = b;
		c = c->next;
	    }
	    b = b->next;
	}
    }
    c->next = (( a == NULL) ? b : a);		/* append non-null list */
    return( d);
}

/***********************************************************************
 *
 * Description:
 * 	split a list into two lists destructively.
 * Arguments:
 *	a - list to split.
 * Returns:
 *	the second list, the AmFirst list is kept as a.
 *
 ***********************************************************************/

static AmListPtr 
splitLst( AmListPtr a)
{
    AmListPtr 	p, b, c;

    b = p = a;
    while ( p != NULL) {		/* step b 1 for each 2 p steps. */
	p = p->next;
	if ( p != NULL) {
	    p = p->next;
	    b = b->next;
	}
    }
    c = b->next;
    b->next = NULL;
    return( c);
}

/***********************************************************************
 *
 * Description:
 * 	sort a list on a comparison function.
 * Arguments:
 *	a - list to sort.
 *	cmpFn - comparison function to use to sort.
 * Returns:
 *	the in-place sorted list.
 *
 ***********************************************************************/

AmListPtr 
AmMergesortLst( AmListPtr 	a,
	        AmSearchFn	cmpFn)
{
    AmListPtr tmp, b;

    if ( a == NULL) return( NULL);
    if ( a->next == NULL) return( a);
    if ( a->next->next == NULL) {
	if ( ( cmpFn)( a->data, a->next->data) > 0) {
	    tmp = a->next;
	    tmp->next = a;			/* swap if 2 reversed */
	    a->next = NULL;
	    return( tmp);
	} else {
	    return( a);
	}
    }
    b = splitLst( a);			/* split, sort halves, then merge */
    a = AmMergesortLst( a, cmpFn);
    b = AmMergesortLst( b, cmpFn);
    return( listMerge( a, b, cmpFn));
}

/***********************************************************************
 *
 * Description:
 * 	generate a list-pointer copied unique list from a sorted 
 *	source list 
 * Arguments:
 *	a - list to uniquify.
 *	cmpFn - comparison function for sorting and unique comparison.
 * Returns:
 *	a list of the unique elements in the source list.
 * Modifications:
 *	should return an error code if the malloc() fails.
 *
 ***********************************************************************/

AmListPtr 
AmMakeUniqueFromSortedLst( AmListPtr	a,
			   AmSearchFn	cmpFn)
{
    AmListPtr	head, newNode, ptr;

    if ( a == NULL) {
	return( NULL);
    }
    head = newNode = ( AmListPtr) malloc( sizeof( AmListRec));
    if ( head == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    head->data = a->data;
    ptr = a->next; 
    while ( ptr != NULL) {

/* next node is different */

	if (( cmpFn)( a->data, ptr->data) != 0) {
	    newNode->next = ( AmListPtr) malloc( sizeof( AmListRec));
	    if ( newNode->next == NULL) {
		DO_ERROR( AmMALLOC_FAILED);
		return( NULL);
	    }
	    newNode = newNode->next;
	    newNode->data = ptr->data;
	    a = ptr;
	}
	ptr = ptr->next;
    }
    newNode->next = NULL;
    return( head);
}
