#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <anvil/list.h>
#include <anvil/files.h>
#include <anvil/mem_funcs.h>

static int lastCompareValue;

static void 
printLst( AmListPtr	lst)
{
    for ( ; lst != NULL; lst = lst->next) {
	printf( "%d ", *( int *) lst->data);
    }
    printf( "\n");
}

static int 
compareIntPtr( int	*a,
	       int	*b)
{
    return( *a - *b);
}

static int 
revCompareIntPtr( int	*a,
	          int	*b)
{
    return( *a - *b);
}

static AnvilBoolean 
matchIntPtr( int	*a,
	     int	*b)
{
    return( *a == *b);
}

static AnvilBoolean
limitIntPtr( int	*a)
{
    return( *a < lastCompareValue);
}

static void
printIntPtr( int	*a)
{
    printf( "%d ", *a);
}


void main( int argc, char *argv[])
{
    int		*vectorA, *vectorB;
    int		numA, numB, ii, *dequeueVal, *result, index, *newNum,
		**ptrVector, *copyVector;
    AmListPtr	listA, listB, copyListA, copyListB, 
		ptr, lastPtr, lstPtr, uniqueLst, traceLstA;
 
    if ( argc < 3) {
	fprintf( stderr, "usage: %s vectorA vectorB\n", argv[ 0]);
	exit( -1);
    }

/* read the vectors from the files */

    numA = AmNumLines( argv[ 1]);
    numB = AmNumLines( argv[ 2]);
    vectorA = AmReadIntVec( argv[ 1]);
    vectorB = AmReadIntVec( argv[ 2]);
    printf( "vector A: ");
    for ( ii = 0; ii < numA; ii++) {
	printf( "%d ", vectorA[ ii]);
    }
    printf( "\n");
    printf( "vector B: ");
    for ( ii = 0; ii < numB; ii++) {
	printf( "%d ", vectorB[ ii]);
    }
    printf( "\n");

/* insert node at end tests */

    listA = NULL;
    for ( ii = 0; ii < numA; ii++) {
	listA = AmInsertNodeAtEndLst( AmCopyVec( &vectorA[ ii], sizeof( int)), 
				      listA);
    }
    printf( "list A: ");
    printLst( listA);

    listB = NULL;
    for ( ii = 0; ii < numB; ii++) {
	listB = AmInsertNodeAtEndLst( AmCopyVec( &vectorB[ ii], sizeof( int)), 
				      listB);
    }
    printf( "list B: ");
    printLst( listB);

/* copy list tests */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);

/* delete item tests */

    copyListA = AmDeleteItemLst( &vectorA[ 0], copyListA, 
				 ( AmSearchFn) matchIntPtr, ( AmItemFn) free);
    printf( "copy of list A with first item deleted: ");
    printLst( copyListA);
    if ( numA > 4) {
	copyListA = AmDeleteItemLst( &vectorA[ 3], copyListA, 
				     ( AmSearchFn) matchIntPtr, ( AmItemFn) free);
	printf( "copy of list A with third item deleted: ");
	printLst( copyListA);
    }
    copyListA = AmDeleteItemLst( &vectorA[ numA - 1], copyListA, 
				 ( AmSearchFn) matchIntPtr, ( AmItemFn) free);
    printf( "copy of list A with last item deleted: ");
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* delete pointer tests: because we're just deleting a pointer, there will 
   be a memory leak. */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    copyListA = AmDelPtrLst( &vectorA[ 0], copyListA, 
			     ( AmSearchFn) matchIntPtr);
    printf( "copy of list A with first item deleted: ");
    printLst( copyListA);
    if ( numA > 4) {
	copyListA = AmDelPtrLst( &vectorA[ 3], copyListA, 
				 ( AmSearchFn) matchIntPtr);
	printf( "copy of list A with third item deleted: ");
	printLst( copyListA);
    }
    copyListA = AmDelPtrLst( &vectorA[ numA - 1], copyListA, 
			     ( AmSearchFn) matchIntPtr);
    printf( "copy of list A with last item deleted: ");
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* delete element list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    copyListA = AmDelElLst( &vectorA[ 0], copyListA, 
			     ( AmSearchFn) matchIntPtr);
    printf( "copy of list A with first item deleted: ");
    printLst( copyListA);
    if ( numA > 4) {
	copyListA = AmDelElLst( &vectorA[ 3], copyListA, 
				 ( AmSearchFn) matchIntPtr);
	printf( "copy of list A with third item deleted: ");
	printLst( copyListA);
    }
    copyListA = AmDelElLst( &vectorA[ numA - 1], copyListA, 
			     ( AmSearchFn) matchIntPtr);
    printf( "copy of list A with last item deleted: ");
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* dequeue list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    copyListA = AmDequeueLst(( void **)  &dequeueVal, copyListA);
    printf( "dequeued value: %d\n", *dequeueVal);
    printf( "dequeued list: ");
    printLst( copyListA);
    free( dequeueVal);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* filter tests */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    lastCompareValue = 0;
    copyListA = AmFilterLst( copyListA, ( AmElementFn) limitIntPtr);
    printf( "A with values below %d filtered: ", lastCompareValue);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);
    copyListA = AmCopyLst( listA, sizeof( int));
    lastCompareValue = 10;
    copyListA = AmFilterLst( copyListA, ( AmElementFn) limitIntPtr);
    printf( "A with values below %d filtered: ", lastCompareValue);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);
    copyListA = AmCopyLst( listA, sizeof( int));
    lastCompareValue = 100;
    copyListA = AmFilterLst( copyListA, ( AmElementFn) limitIntPtr);
    printf( "A with values below %d filtered: ", lastCompareValue);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    lastCompareValue = 0;
    lstPtr = AmFilterCopyLst( copyListA, ( AmElementFn) limitIntPtr);
    printf( "A with values below %d filtered: ", lastCompareValue);
    printLst( lstPtr);
    AmJustFreeLst( lstPtr);
    lastCompareValue = 10;
    lstPtr = AmFilterCopyLst( copyListA, ( AmElementFn) limitIntPtr);
    printf( "A with values below %d filtered: ", lastCompareValue);
    printLst( lstPtr);
    AmJustFreeLst( lstPtr);
    lastCompareValue = 100;
    lstPtr = AmFilterCopyLst( copyListA, ( AmElementFn) limitIntPtr);
    printf( "A with values below %d filtered: ", lastCompareValue);
    printLst( lstPtr);
    AmJustFreeLst( lstPtr);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* find data list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    result = ( int *) AmFindDataLst( &vectorA[ 0], copyListA, 
				     ( AmSearchFn) matchIntPtr);
    printf( "first found value: %d %d\n", vectorA[ 0], *result);
    if ( numA > 4) {
	result = ( int *) AmFindDataLst( &vectorA[ 3], copyListA, 
					 ( AmSearchFn) matchIntPtr);
	printf( "third found value: %d %d\n", vectorA[ 3], *result);
    }
    result = ( int *) AmFindDataLst( &vectorA[ numA - 1], copyListA, 
				     ( AmSearchFn) matchIntPtr);
    printf( "last found value: %d %d\n", vectorA[ numA - 1], *result);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* find element list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    ptr = AmFindElLst( &vectorA[ 0], copyListA, ( AmSearchFn) matchIntPtr);
    printf( "first found value: %d %d\n", vectorA[ 0], *( int *) ptr->data);
    if ( numA > 4) {
	ptr = AmFindElLst( &vectorA[ 3], copyListA, ( AmSearchFn) matchIntPtr);
	printf( "third found value: %d %d\n", vectorA[ 3], *( int *) ptr->data);
    }
    ptr = AmFindElLst( &vectorA[ numA - 1], copyListA, ( AmSearchFn) matchIntPtr);
    printf( "last found value: %d %d\n", vectorA[ numA - 1], *( int *) ptr->data);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* find index list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    index = AmIndexDataLst( &vectorA[ 0], copyListA, 
			    ( AmSearchFn) matchIntPtr);
    printf( "first found value: %d %d\n", vectorA[ 0], index);
    if ( numA > 4) {
	index = AmIndexDataLst( &vectorA[ 3], copyListA, 
				( AmSearchFn) matchIntPtr);
	printf( "third found value: %d %d\n", vectorA[ 3], index);
    }
    index = AmIndexDataLst( &vectorA[ numA - 1], copyListA, 
			    ( AmSearchFn) matchIntPtr);
    printf( "last found value: %d %d\n", vectorA[ numA - 1], index);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* insert after node */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    newNum = ( int *) malloc( sizeof( int));
    *newNum = 99;
    copyListA = AmInsertNodeAfterLst( newNum, copyListA);
    printf( "inserted %d after first node: ", *newNum);
    printLst( copyListA);
    lastPtr = AmLastLst( copyListA);
    newNum = ( int *) malloc( sizeof( int));
    *newNum = 100;
    lastPtr = AmInsertNodeAfterLst( newNum, lastPtr);
    printf( "inserted %d after last node: ", *newNum);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* insert at end of list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    newNum = ( int *) malloc( sizeof( int));
    *newNum = 99;
    copyListA = AmInsertNodeAtEndLst( newNum, copyListA);
    printf( "inserted %d at end: ", *newNum);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

    newNum = ( int *) malloc( sizeof( int));
    *newNum = 99;
    copyListA = AmInsertNodeAtEndLst( newNum, NULL);
    printf( "inserted %d at end of NULL list: ", *newNum);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* insert at head of list */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    newNum = ( int *) malloc( sizeof( int));
    *newNum = 99;
    copyListA = AmInsertNodeLst( newNum, copyListA);
    printf( "inserted %d at head: ", *newNum);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

    newNum = ( int *) malloc( sizeof( int));
    *newNum = 99;
    copyListA = AmInsertNodeLst( newNum, NULL);
    printf( "inserted %d at head of NULL list: ", *newNum);
    printLst( copyListA);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* last, next to last, first tests. */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    result = ( int *) AmLast( copyListA);
    if ( result != NULL) {
	printf( "last element: %d\n", *result);
    }
    result = ( int *) AmLast( NULL);
    if ( result != NULL) {
	printf( "this shouldn't happen!\n");
    } else {
	printf( "last element of NULL list is NULL\n");
    }
    AmFreeLst( copyListA, ( AmItemFn) free);
    
    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    lstPtr = AmLastLst( copyListA);
    if ( lstPtr != NULL) {
	printf( "last element: %d\n", *( int *) lstPtr->data);
    }
    lstPtr = AmLastLst( copyListA);
    if ( lstPtr != NULL) {
	printf( "this shouldn't happen!\n");
    } else {
	printf( "last element of NULL list is NULL\n");
    }

    result = ( int *) AmFirst( copyListA);
    if ( result != NULL) {
	printf( "first element: %d\n", *result);
    }
    result = ( int *) AmFirst( NULL);
    if ( result != NULL) {
	printf( "this shouldn't happen!\n");
    } else {
	printf( "first element of NULL list is NULL\n");
    }
	
    result = ( int *) AmNextToLast( copyListA);
    if ( result != NULL) {
	printf( "next to last element: %d\n", *result);
    } else {
	printf( "next to last element is NULL\n");
    }
    result = ( int *) AmNextToLast( NULL);
    if ( result != NULL) {
	printf( "this shouldn't happen!\n");
    } else {
	printf( "next to last element of NULL list is NULL\n");
    }
    newNum = ( int *) malloc( sizeof( int));
    *newNum = 100;
    lstPtr = AmInsertNodeLst( newNum, NULL);
    result = ( int *) AmNextToLast( lstPtr);
    if ( result != NULL) {
	printf( "this shouldn't happen!\n");
    } else {
	printf( "next to last element of 1 element list is NULL\n");
    }
    AmFreeLst( lstPtr, ( AmItemFn) free);
    AmFreeLst( copyListA, ( AmItemFn) free);

/* last of list tests */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    lastCompareValue = 50;
    result = ( int *) AmLastOfLst(( AmElementFn) limitIntPtr, copyListA);
    if ( result != NULL) {
	printf( "last value below %d is %d\n", lastCompareValue, *result);
    } else {
	printf( "all values are above %d\n", lastCompareValue);
    }
    lastCompareValue = 0;
    result = ( int *) AmLastOfLst(( AmElementFn) limitIntPtr, copyListA);
    if ( result != NULL) {
	printf( "last value below %d is %d\n", lastCompareValue, *result);
    } else {
	printf( "all values are above %d\n", lastCompareValue);
    }
    lastCompareValue = 200;
    result = ( int *) AmLastOfLst(( AmElementFn) limitIntPtr, copyListA);
    if ( result != NULL) {
	printf( "last value below %d is %d\n", lastCompareValue, *result);
    } else {
	printf( "all values are above %d\n", lastCompareValue);
    }
    AmFreeLst( copyListA, ( AmItemFn) free);

/* length test */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    printf( "length of A is %d\n", AmLengthLst( copyListA));
    printf( "length of NULL is %d\n", AmLengthLst( NULL));

/* map test */

    printf( "mapping print to A: ");
    AmMapLst( copyListA, ( AmItemFn) printIntPtr);
    printf( "\n");
    AmFreeLst( copyListA, ( AmItemFn) free);

/* vector generation tests */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    ptrVector = ( int **) AmMakeVectorLst( copyListA);
    printf( "list A pointer vector: ");
    for ( ii = 0; ptrVector[ ii] != NULL; ii++) {
	printf( "%d ", *ptrVector[ ii]);
    }
    printf( "\n");
    free( ptrVector);
    ptrVector = ( int **) AmMakeVectorLst( NULL);
    for ( ii = 0; ptrVector[ ii] != NULL; ii++) {
	printf( "%d ", *ptrVector[ ii]);
    }
    free( ptrVector);

    copyVector = ( int *) AmCopyVectorLst( copyListA, sizeof( int));
    printf( "list A copy vector: ");
    for ( ii = 0; ii < AmLengthLst( copyListA); ii++) {
	printf( "%d ", copyVector[ ii]);
    }
    printf( "\n");
    free( copyVector);
    copyVector = ( int *) AmCopyVectorLst( NULL, sizeof( int));
    if ( copyVector != NULL) {
	printf( "copy vector of NULL list is NOT NULL\n");
    } else {
	printf( "copy vector of NULL list is NULL\n");
    }
    AmFreeLst( copyListA, ( AmItemFn) free);

/* nth tests */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);

    result = ( int *) AmNth( 0, copyListA);
    if ( result != NULL) {
	printf( "0th element = %d\n", *result); 
    } else {
	printf( "0th element is NULL\n");
    }
    result = ( int *) AmNth( 1, copyListA);
    if ( result != NULL) {
	printf( "1st element = %d\n", *result); 
    } else {
	printf( "1st element is NULL\n");
    }
    result = ( int *) AmNth( 5, copyListA);
    if ( result != NULL) {
	printf( "5th element = %d\n", *result); 
    } else {
	printf( "5th element is NULL\n");
    }
    result = ( int *) AmNth( 100, copyListA);
    if ( result != NULL) {
	printf( "100 element = %d\n", *result); 
    } else {
	printf( "100 element is NULL\n");
    }

    lstPtr =  AmNthLst( 0, copyListA);
    if ( result != NULL) {
	printf( "0th element = %d\n", *( int *) lstPtr->data);
    } else {
	printf( "0th element is NULL\n");
    }
    lstPtr = AmNthLst( 1, copyListA);
    if ( lstPtr != NULL) {
	printf( "1st element = %d\n", *( int *) lstPtr->data);
    } else {
	printf( "1st element is NULL\n");
    }
    lstPtr = AmNthLst( 5, copyListA);
    if ( lstPtr != NULL) {
	printf( "5th element = %d\n", *( int *) lstPtr->data);
    } else {
	printf( "5th element is NULL\n");
    }
    lstPtr = AmNthLst( 100, copyListA);
    if ( lstPtr != NULL) {
	printf( "100 element = %d\n", *( int *) lstPtr->data);
    } else {
	printf( "100 element is NULL\n");
    }
    AmFreeLst( copyListA, ( AmItemFn) free);


/* sorted tests */

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);

    lstPtr = AmMergesortLst( copyListA, ( AmSearchFn) compareIntPtr);
    printf( "list A sorted: ");
    printLst( lstPtr);
    AmFreeLst( lstPtr, ( AmItemFn) free);

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy of list A: ");
    printLst( copyListA);
    lstPtr = AmMergesortLst( copyListA, ( AmSearchFn) revCompareIntPtr);
    printf( "list A reverse sorted: ");
    printLst( lstPtr);
    AmFreeLst( lstPtr, ( AmItemFn) free);

    copyListA = AmCopyLst( listA, sizeof( int));
    printf( "copy list A: ");
    printLst( copyListA);

    copyListB = AmCopyLst( listB, sizeof( int));
    printf( "copy list B: ");
    printLst( listB);
    lstPtr = AmAppendLst( copyListA, copyListB);
    printf( "list A and B appended: ");
    printLst( lstPtr);
    lstPtr = AmMergesortLst( lstPtr, ( AmSearchFn) compareIntPtr);
    uniqueLst = AmMakeUniqueFromSortedLst( lstPtr, ( AmSearchFn) compareIntPtr);
    printf( "list A and B, appended, sorted, unique: ");
    printLst( uniqueLst);
    AmJustFreeLst( uniqueLst);
    AmFreeLst( lstPtr, ( AmItemFn) free);

/* NULL tests */

    lstPtr = AmMergesortLst( NULL, ( AmSearchFn) compareIntPtr);
    if ( lstPtr != NULL) {
	printf( "mergesort of NULL is NOT NULL!!!\n");
    } else {
	printf( "mergesort of NULL is NULL\n");
    }
    lstPtr = AmMakeUniqueFromSortedLst( NULL, ( AmSearchFn) compareIntPtr);
    if ( lstPtr != NULL) {
	printf( "unique of NULL is NOT NULL!!!\n");
    } else {
	printf( "unique of NULL is NULL\n");
    }

/* trace tests */

    traceLstA = AmTraceLst( listA);
    printf( "trace list A: ");
    printLst( traceLstA);
    AmJustFreeLst( traceLstA);
    AmFreeLst( listA, ( AmItemFn) free);
    AmFreeLst( listB, ( AmItemFn) free);
    free( vectorA);
    free( vectorB);
}
