#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdarg.h>
#include <anvil/project.h>
#include <anvil/list.h>

/***********************************************************************
 *
 * Description:
 * 	generate a list from a var args list of data.
 * Arguments:
 *	a, varargs - NULL terminated list of elements to create the 
 *	list with
 * Returns:
 *	the generated list.
 * Modifications:
 *	should return an error if the malloc() in the insert failed
 *
 ***********************************************************************/

AmListPtr 
AmMakeLst( void *a, ...)
{
     AmListPtr 	nameLst;
     void 		*identifier = a;
     va_list	strValst;

     va_start( strValst, a);
     nameLst = NULL;
     while ( identifier != NULL) {

/* loop through and cons the values */

		nameLst = AmInsertNodeAtEndLst( identifier, nameLst);
		identifier = va_arg( strValst, void *);
     } 
     va_end( strValst);
     return( nameLst);
}

/***********************************************************************
 *
 * Description:
 * 	given a terminated va_list, and the terminator, create a 
 *	generic list of callback/handler arguments 
 * Arguments:
 *	terminator - vararg terminator.
 *	args - vararg list of elements to generate the list with
 *
 ***********************************************************************/

AmListPtr 
AmVarargLst( void 		*terminator,
			  va_list	args)
{
    void			*clientData;
    AmListPtr		list = NULL;

/* loop through the varargs list and create a generic list */

    for ( ;;) {
        clientData = va_arg( args, void *);
        if ( clientData == terminator) {         /* better terminator */
            return( list);
        }
        list = AmInsertNodeAtEndLst( clientData, list);
    }
    return( list);
}
