#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* utility functions for maintaining a address changed track list, used 
   in the setFields functions in AmRecordClass::SetFieldsByDefintions()
   so that the setFields() functions can test whether a value has been
   changed */

#include <stdarg.h>
#if !defined(__BSD44__)
#include <search.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#include <anvil/tsearch.h>
#include <anvil/macros.h>
#include <anvil/project.h>
#include <anvil/changed.h>

static void 	*changeAddrs[ MAXTAGS];
static int	changeCount = 0;
static void	*changeRoot = NULL;

/* comparison function for tracking the change addresses in a tsearch()
   tree */

static int 
compareChangeAddr( void      *a,
		   void      *b)
{
    return(( int) a - ( int) b);
}

/****************************************************************************
 *
 * Description:
 *      initialize the AmTrackChanges() tree.  If there are elements in
 *	the tree, then delete them, set the change count to 0 and NULL
 *	out the tree.
 *
 ****************************************************************************/

void 
AmTrackChanges()
{
    int	ii;

/* delete all of the AmLast entries */

    for ( ii = 0; ii < changeCount; ii++) {
	tdelete( changeAddrs[ ii], &changeRoot, ( AmSearchFn) compareChangeAddr);
    }
    changeCount = 0;
    changeRoot = NULL;
}

/****************************************************************************
 *
 * Description:
 * 	mark an address as changed, and add it to the AmTrackChanges() tree.
 * Arguments:
 *	addr - address to mark as changed.
 *
 ****************************************************************************/

void 
AmMarkChanged( void *addr)
{
    changeAddrs[ changeCount] = addr;
    tsearch( changeAddrs[ changeCount], &changeRoot, 
	     ( AmSearchFn) compareChangeAddr);
    changeCount++;
}

/****************************************************************************
 *
 * Description:
 * 	find out if the address that was passed in has changed, by searching
 *	the AmChanged() table.
 * Arguments:
 *	addr - address to search for
 *
 ****************************************************************************/

AnvilBoolean 
AmChanged( void *addr)
{
    if ( tfind( addr, ( const void **) &changeRoot, 
		( AmSearchFn) compareChangeAddr) != NULL) {
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/****************************************************************************
 *
 * Description:
 * 	find out if anything in a list of addresses have changed.
 *  	there are two starting tags to unambiguate this from the 
 *  	scalar function.  The varargs list is NULL-terminated.
 * Arguments:
 *	addr1, addr2, varargs - address to search for
 *
 ****************************************************************************/

AnvilBoolean 
AmChanged( void *addr1, 
	 void *addr2, 
	 ...)
{
    va_list	args;

    if ( AmChanged( addr1)) {
	return( TRUE);
    }
    va_start( args, addr2);
    for ( ;addr2 != NULL; addr2 = va_arg( args, void *)) {
	if ( AmChanged( addr2)) {
	    va_end( args);
	    return( TRUE);
	}
    }
    va_end( args);
    return( FALSE);
}
