%{
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <anvil/list.h>
#include <anvil/description.h>
int yylex( void);
int yyerror( const char *msg);
%}

%union {
    AmTypePtr	type;
    char	*string;
    AmListPtr	list;
}

%token <string>	IDENT
%token <string>	STRUCT
%token <string>	TYPEDEF
%token <string> NUMBER
%token <string> SEMICOLON
%token <string> OPEN_SQ
%token <string> CLOSE_SQ
%token <string> OPEN_CURLY
%token <string> CLOSE_CURLY

%type <type> 	decl
%type <type> 	typedecl
%type <list> 	typelist
%type <list> 	fields

%start typelist

%%

typelist    : typedecl 			         { $$ = AmAddType( $1, NULL); }
	    | typelist typedecl 		 { $$ = AmAddType( $2, $1); }
	    ;

typedecl    : TYPEDEF decl 			 { $$ = AmMakeTypedef( $2); }
	    | decl
	    ;

decl	    : STRUCT IDENT '{' fields '}' ';'    { $$ = AmMakeStruct( $2, $4); }
	    | IDENT IDENT ';'		         { $$ = AmMakeScalar( $1, $2); }
	    | IDENT IDENT '[' NUMBER ']' ';'     { $$ = AmMakeStatic( $1, $2, $4); }
	    | IDENT IDENT '[' IDENT ']' ';'      { $$ = AmMakeDynamic( $1, $2, $4); }
	    | IDENT IDENT '[' ']' ';'		 { $$ = AmMakeEof( $1, $2); }
	    ;

fields	    : decl 			         { $$ = AmAddDecl( $1, NULL); }
	    | fields decl 			 { $$ = AmAddDecl( $2, $1); }
	    ;

%%
