#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* distance utility functions. */

#include <math.h>
#include <anvil/project.h>
#include <anvil/geometry.h>

/****************************************************************************
 *
 * Description:
 * 	distance AmBetween two points 
 * Arguments:
 *	x0, y0, x1, y1 - points to find the distance AmBetween.
 * Returns:
 *	distance AmBetween the points.
 *
 ****************************************************************************/

float 
AmDist( float x0, 
      float y0, 
      float x1, 
      float y1)
{
    return( sqrt(( x1 - x0) * ( x1 - x0) + ( y1 - y0) * ( y1 - y0)));
}

/****************************************************************************
 *
 * Description:
 * 	distance AmBetween two points in 3D
 * Arguments:
 *	x0, y0, z0, x1, y1 z1,- points to find the distance AmBetween.
 * Returns:
 *	distance AmBetween the points.
 *
 ****************************************************************************/

float 
AmDist3D( float x0, 
        float y0, 
	float z0,
        float x1, 
        float y1,
	float z1)
{
    return( sqrt(( x1 - x0)*( x1 - x0) + 
		 ( y1 - y0)*( y1 - y0) +
		 ( z1 - z0)*( z1 - z0)));
}

/****************************************************************************
 *
 * Description:
 * 	square of the distance AmBetween two points, used to speed up
 *	comparisons. 
 * Arguments:
 *	x0, y0, x1, y1 - points to find the distance AmBetween.
 * Returns:
 *	distance AmBetween the points.
 *
 ****************************************************************************/

float 
AmSqrDist( float x0, 
	 float y0, 
	 float x1, 
	 float y1)
{
    return(( x1 - x0) * ( x1 - x0) + ( y1 - y0) * ( y1 - y0));
}

