%{
#include <stdio.h>
#include <string.h>
#if !defined(_WIN32) && !defined(__BSD44__)
#include <alloca.h>
#else 
#include <malloc.h>
#endif
#include <anvil/list.h>
#include <anvil/units.h>
#include <anvil/unitsP.h>
int yylex( void);
int yyerror( const char *msg);
%}

%union {
    char	character;
    char	*string;
    AmExprPtr	expr;
}

%token <string> IDENT
%token <string> NUMBER
%token <string> '+'
%token <string> '-'
%token <string> '*'
%token <string> '/'
%token <string> '|'
%token <string> '&'
%token <string> '<'
%token <string> '>'
%token <string> '^'
%token <string> '='

%type <expr> logexpr
%type <expr> boolexpr
%type <expr> expr
%type <expr> term
%type <expr> factor

%%

logexpr	    : boolexpr				{ $$ = $1 }
	    | logexpr '|' boolexpr		{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | logexpr '&' boolexpr		{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    ;

boolexpr    : expr				{ $$ = $1 }
	    | boolexpr '=' expr			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | boolexpr '^' expr			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | boolexpr '>' expr			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | boolexpr '<' expr			{ $$ = AmMakeBinexpr( $2, $1, $3); }

expr	    : term				{ $$ = $1 }
	    | expr '+' term			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | expr '-' term			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    ;

term	    : factor				{ $$ = $1; }
	    | term '*' factor			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | term '/' factor			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    ; 

factor	    : IDENT						{ $$ = AmMakeIdentifier( $1); }
	    | NUMBER						{ $$ = AmMakeNumber( $1); }
	    | '(' expr ')'					{ $$ = $2; }
	    | IDENT '(' expr ')'				{ $$ = AmMakeFunexpr( $1, $3); }
	    | IDENT '(' expr ',' expr ')'			{ $$ = AmMakeFunexpr2( $1, $3, $5); }
	    | IDENT '(' expr ',' expr ',' expr ')'		{ $$ = AmMakeFunexpr3( $1, $3, $5, $7); }
	    | IDENT '(' expr ',' expr ',' expr ',' expr ')'	{ $$ = AmMakeFunexpr4( $1, $3, $5, $7, $9); }
	    ;

%%
