#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <math.h>
#include <anvil/fns.h>
#include <anvil/numerics.h>

/****************************************************************************
 *
 * Description:
 * 	fit a line, y = aa*x + bb to a set of points. sigA and sigB 
 *	return the probable uncertainties for aa and bb 
 * Arguments:
 *	  data:		vector of points
 *	  xFn:		x accessing function
 *	  xoffset:	offset of x value
 *	  yFn:		y accessing function
 *	  yoffset:	offset of y value
 *	  clientData:	client data for x and y functions
 *	  size:		size of each point data element
 *	  numPts:	length of points vector
 *	  aa:		slope of resulting line.
 *	  bb:		intercept of resulting line.
 *	  sigA:		uncertainty for slope.
 *	  sigB:		uncertainty for intercept.
 *
 ****************************************************************************/

void AmFitPoints( uchar_t		*data,
		AmAccessFn	xFn,
		int		xOffset,
		AmAccessFn	yFn,
		int		yOffset,
		void		*clientData,
		int		size,
		int		numPts,
		float		*aa,
		float		*bb,
		float		*sigA,
		float		*sigB)
{
    float	sx, sy, xAvg, st2, tt;
    int		ii;

    sx = 0.0;
    sy = 0.0;
    st2 = 0.0;
    *bb = 0.0;

/* accumulate x and y sums */

    for ( ii = 0; ii < numPts; ii++) {
	sx += ( xFn)( data + ii*size + xOffset, clientData);
	sy += ( yFn)( data + ii*size + yOffset, clientData);
    }
    xAvg = sx/numPts;
    for ( ii = 0; ii < numPts; ii++) {
	tt = ( xFn)( data + ii*size + xOffset, clientData) - xAvg;
	st2 += tt*tt;
	*bb += tt*( yFn)( data + ii*size + yOffset, clientData);
    }
    *bb /= st2;
    *aa = ( sy - sx*(*bb))/numPts;
    *sigA = sqrt(( 1.0 + sx*sx/( numPts*st2))/numPts);
    *sigB = sqrt( 1.0/st2);
}


