#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <anvil/fns.h>
#include <anvil/fndefs.h>
#include <anvil/macros.h>

/* predefined data accessing functions */

float 
AmFloatFn( void *x, void *)
{
    return( *( float *) x);
}
 
float 
AmIntFn( void *x, void *)
{
    return(( float) *( int *) x);
}
 
float 
AmUintFn( void *x, void *)
{
    return(( float) *( unsigned int *) x);
}
 
float 
AmDoubleFn( void *x, void *)
{
    return(( float) *( double *) x);
}
 
float 
AmCharFn( void *x, void *)
{
    return(( float) *( char *) x);
}
 
float 
AmUcharFn( void *x, void *)
{
    return(( float) *( unsigned char *) x);
}
 
float 
AmShortFn( void *x, void *)
{
    return(( float) *( short *) x);
}
 
float 
AmUshortFn( void *x, void *)
{
    return(( float) *( unsigned short *) x);
}

/* integer return functions.  client data is assumed to be an offset */
 
int 
AmLongIntFn( void *x, void *)
{
    return( *( long *) x);
}
 
int 
AmUlongIntFn( void *x, void *)
{
    return( *( unsigned long *) x);
}
 
int 
AmIntIntFn( void *x, void *)
{
    return( *( int *) x);
}
 
int 
AmUintIntFn( void *x, void *)
{
    return( *( unsigned int *) x);
}
 
int 
AmShortIntFn( void *x, void *)
{
    return( *( short *) x);
}
 
int 
AmUshortIntFn( void *x, void *)
{
    return( *( unsigned short *) x);
}
 
int 
AmCharIntFn( void *x, void *)
{
    return( *( char *) x);
}
 
int 
AmUcharIntFn( void *x, void *)
{
    return( *( unsigned char *) x);
}

void 
AmSetFloatFn( void *data, void *, float src)
{
    *( float *) data = src;
}

void 
AmSetIntFn( void *data, void *, float src)
{  
    *( int *) data = ( int) rint( src);
}

void 
AmSetUintFn( void *data, void *, float src)
{
    *( unsigned int *) data = ( unsigned int) rint( src);
}

void 
AmSetDoubleFn( void *data, void *, float src)
{
    *( double *) data = src;
}

void 
AmSetCharFn( void *data, void *, float src)
{
    *( char *) data = ( char) rint( src);
}

void 
AmSetUcharFn( void *data, void *, float src)
{
    *( unsigned char *) data = ( unsigned  char) rint( src);
}

void 
AmSetShortFn( void *data, void *, float src)
{
    *( short *) data = ( short) rint( src);
}

void 
AmSetUshortFn( void *data, void *, float src)
{
    *( unsigned short *) data = ( unsigned short) rint( src);
}

void 
AmSetLongIntFn( void *data, void *, int src)
{
    *( long *) data = src;
}

void 
AmSetUlongIntFn( void *data, void *, int src)
{
    *( unsigned long *) data = src;
}

void 
AmSetIntIntFn( void *data, void *, int src)
{
    *( int *) data = src;
}

void 
AmSetUintIntFn( void *data, void *, int src)
{
    *( unsigned int *) data = src;
}

void 
AmSetShortIntFn( void *data, void *, int src)
{
    *( short *) data = src;
}

void 
AmSetUshortIntFn( void *data, void *, int src)
{
    *( unsigned short *) data = src;
}

void 
AmSetCharIntFn( void *data, void *, int src)
{
    *( char *) data = src;
}

void 
AmSetUcharIntFn( void *data, void *, int src)
{
    *( unsigned char *) data = src;
}

/****************************************************************************
 *
 * Description:
 * 	given a datatype, return the corresponding accessing function 
 *	by searching on a predefined table whic maps datatypes to
 *	accessing functions.
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	accessing function, or NULL if it couldn't be found.
 *
 ****************************************************************************/

AmAccessFn 
AmDataTypeEnumToAccessFn( AmDataTypeEnum   dataType)
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( dataFns); ii++) {
        if ( dataFns[ ii].dataType == dataType) {
            return( dataFns[ ii].fn);
        }
    }
    return( NULL);
}

/****************************************************************************
 *
 * Description:
 * 	given a datatype, return the corresponding floating-point based
 *	setting function 
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	setting function, or NULL if it couldn't be found.
 *
 ****************************************************************************/

AmSetFn 
AmDataTypeEnumToSetFn( AmDataTypeEnum   dataType)
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( setDataFns); ii++) {
        if ( setDataFns[ ii].dataType == dataType) {
            return( setDataFns[ ii].fn);
        }
    }
    return( NULL);
}

/****************************************************************************
 *
 * Description:
 * 	given a datatype, return the corresponding integer-based 
 *	setting function 
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	integer setting function, or NULL if it couldn't be found.
 *
 ****************************************************************************/

AmIntSetFn 
AmDataTypeEnumToIntSetFn( AmDataTypeEnum   dataType)
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( setIntDataFns); ii++) {
        if ( setIntDataFns[ ii].dataType == dataType) {
            return( setIntDataFns[ ii].fn);
        }
    }
    return( NULL);
}

/****************************************************************************
 *
 * Description:
 * 	given a data type return the integer access function 
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	integer access function, or NULL if it couldn't be found.
 *
 ****************************************************************************/

AmIntAccessFn 
AmDataTypeEnumToIntAccessFn( AmDataTypeEnum   dataType)
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( intDataFns); ii++) {
        if ( intDataFns[ ii].dataType == dataType) {
            return( intDataFns[ ii].fn);
        }
    }
    return( NULL);
}

/****************************************************************************
 *
 * Description:
 * 	given a data type enumeration, return its size.
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	data type's size, or 0 if it wasn't found.
 *
 ****************************************************************************/

int 
AmDataTypeEnumToSize( AmDataTypeEnum    dataType)
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( dataSizes); ii++) {
        if ( dataSizes[ ii].dataType == dataType) {
            return( dataSizes[ ii].size);
        }
    }
    return( 0);
}

/****************************************************************************
 *
 * Description:
 * 	convert data type enumeration to printable string.
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	string corresponding to the type, or NULL if it wasn't found.
 *
 ****************************************************************************/

char *
AmDataTypeEnumToStr ( AmDataTypeEnum  dataType )
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( dataNames); ii++) {
        if ( dataNames[ ii].dataType == dataType) {
            return( dataNames[ ii].tag);
        }
    }
    return( NULL);
}

/****************************************************************************
 *
 * Description:
 * 	convert printable string to data type enumeration.
 * Arguments:
 *	dataType - data type to search for.
 * Returns:
 *	type corresponding to the string, or AmNO_TYPE if it wasn't found.
 *
 ****************************************************************************/

AmDataTypeEnum 
AmStrToDataTypeEnum( const char *name)
{
    unsigned int ii;

    for ( ii = 0; ii < VECLEN( dataNames); ii++) {
        if ( !strcmp( dataNames[ ii].tag, name)) {
            return( dataNames[ ii].dataType);
        }
    }
    return( AmNO_TYPE);
}
