#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* utility functions for dealing with whitespace in strings. */

#include <ctype.h>
#include <string.h>
#include <anvil/project.h>
#include <anvil/strlib.h>

/****************************************************************************
 *
 * Description:
 *      eliminate the leading whitespace in a string.
 * Arguments:
 *      str - string to eliminate the leading whitespace for
 * Returns:
 *      a pointer into the original string with the whitespace removed.
 *
 ****************************************************************************/

/* eliminate leading white space */

char *
AmElimLeadWspace( char *str)
{
    while (( *str != '\0') &&  isspace( *str)) {
	str++;
    }
    return( str);
}

/****************************************************************************
 *
 * Description:
 * 	Destructively eliminate trailing white space in a string.
 * Arguments:
 *      str - string to eliminate the trailing whitespace for
 * Returns:
 *      a pointer to the original string with the trailing whitespace removed.
 *
 ****************************************************************************/

char *
AmElimTrailWspace( char *str)
{
    char	*p;

    p = str + strlen( str) - 1;
    while ( (p>=str) && (*p==' ' || *p=='\n' || *p=='\t') ) {
	p--;
    }
    p++;
    *p = '\0';
    return( str);
}

/****************************************************************************
 *
 * Description:
 * 	given a string, substitute character b for a.
 * Arguments:
 *      str - string to substitute characters in.
 *	a - character to replace.
 *	b - character to replace it with.
 * Returns:
 *      a pointer to the original string with the characters substituted.
 *
 ****************************************************************************/

char *
AmSubstStr( char 	*str,
	  int	a,
	  int	b)
{
    char	*p;

    for ( p = str; *p != '\0'; p++) {
	if ( *p == a) {
	    *p = b;
	}
    }
    return( str);
}
