#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* functions for interpolating and stretching vectors */

#include <stdio.h>
#include <anvil/interp.h>

/****************************************************************************
 *
 * Description:
 *      linearally interpolate a set of points.
 * Arguments:
 *      y - source vector.
 *	n - number of points in the source vector.
 *	y2 - destination vector.
 *	n2 - number of points in the destination vector.
 *
 ****************************************************************************/

void 
AmInterpolateVec( float	*y,
		int	n,
		float	*y2,
		int	n2)
{
    float	frac;
    int		index, ii;

    y2[ 0] = y[ 0];
    y2[ n2 - 1] = y[ n - 1];
    for ( ii = 1; ii < n2 - 1; ii++) {
	frac = (( float) ii*( n - 1))/(( float) ( n2 - 1));
	index = ( int) frac;
	frac -= index;
	y2[ ii] = y[ index]*(1 - frac) + y[ index + 1]*frac;
    }
}

/****************************************************************************
 *
 * Description:
 * 	take a vector and destructively re-map it to new min and max values.
 * Arguments: 
 *	min, max - new endpoint values for the array.
 *	vec - vector of points to remap.
 *	length - length of the vector.
 *
 ****************************************************************************/
 
void 
AmStretchVec( float  min,
	    float  max,
	    float  *vec,
	    int    length)
{
    float       oldMin = vec[ 0], oldMax = vec[ length - 1], 
		scale = ( max - min)/( oldMax - oldMin);
    int         ii;

    vec[ 0] = min;
    vec[ length - 1] = max;
    for ( ii = 1; ii < length - 1; ii++) {
        vec[ ii] = scale*( vec[ ii] - oldMin) + min;
    }
}
