#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* functions for manipulating the error log file.  These are used by
   the error details dialogs in motif and NT. */ 

#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <anvil/project.h>
#include <anvil/errcode.h>
#include <anvil/resource.h>
#include <anvil/log.h>
#include <anvil/files.h>

static char    g_szFile[ PATHLEN];
static AnvilBoolean g_fInitialized = FALSE;

/****************************************************************************
 *
 * Description:
 *      return the intialized log file name.
 * Returns:
 *      the static name of the log file
 *
 ****************************************************************************/

const char *
AmGetLogFile()
{
    if ( g_fInitialized == FALSE) {
        AmInitLog();
    }

    return( g_szFile);
}

/****************************************************************************
 *
 * Description:
 *      blow the (static) log file away.
 * Returns:
 *	AmOK.
 *
 ****************************************************************************/

AmErrorCode
AmRemoveLogFile()
{
    if ( g_fInitialized == TRUE) {
        unlink( g_szFile);
        g_fInitialized = FALSE;
    }
    return( AmOK);
}

/****************************************************************************
 *
 * Description:
 * 	Initialize the log file. *
 * Returns:
 *	AmOK on success, AmOPEN_FAILED if the log file couldn't be
 *	opened.
 *
 ****************************************************************************/

AmErrorCode 
AmInitLog()
{
    FILE   *pFILE;
    time_t t;
    char   *szTime;

    if ( g_fInitialized == TRUE) {
        unlink( g_szFile);
    } else {
        g_fInitialized = TRUE;
    }

    t = time( NULL);  
    szTime = asctime( localtime( &t));
 
    strcpy( g_szFile, "/tmp/LogOutput.XXXXXX");
    mktemp( g_szFile);

    if (( pFILE = fopen( g_szFile, "a")) == NULL) {
	DO_RETURN_ERROR( AmOPEN_FAILED);
    }

    fprintf( pFILE, ">> ");
    fputs( szTime, pFILE);
    fclose( pFILE);

    return( AmOK);
}

/****************************************************************************
 *
 * Description:
 * 	Add a message to the log file.
 * Arguments:
 *	szLog - message to append to the log file.
 * Returns:
 *	AmOK on success, AmOPEN_FAILED if the log file couldn't be
 *	opened.
 *
 ****************************************************************************/

AmErrorCode
AmAddToLog( char *szLog)
{
    FILE   *pFILE;

    if ( g_fInitialized == FALSE) {
        AmInitLog();
    }

    if (( pFILE = fopen( g_szFile, "a")) == NULL) {
	DO_RETURN_ERROR( AmOPEN_FAILED);
    }

    fputs( szLog, pFILE);
    fprintf( pFILE, "\n");
    fclose( pFILE);
    return( AmOK);
}

/****************************************************************************
 *
 * Description:
 * 	Add a resource-based message to the logfile
 * Arguments:
 *	res - resource string which is used to search the resource data
 *	base to get the format string for the log file.
 *	varargs - arguments to pass to the sprintf for the resource 
 *	log string.
 * Returns:
 *	AmOK on success, AmOPEN_FAILED if the log file couldn't be
 *	opened.
 *
 ****************************************************************************/

AmErrorCode
AmAddResourceToLog( const char *res,
                  ...)
{
    va_list      args;
    char         buf[ MSGLEN];
    const char   *fmt;

/* get the string from the resource file */

    va_start( args, res);
    fmt = AmGetDefault( res, res);
    vsprintf( buf, fmt, args);
    va_end( args);
    return( AmAddToLog( buf));
}
