#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
      AmPolyBoolean: a polygon clip library
      Copyright (C) 1997  Alexey Nikitin, Michael Leonov
      leonov@propro.iis.nsk.su

      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
      License as published by the Free Software Foundation; either
      version 2 of the License, or (at your option) any later version.

      This library is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Library General Public License for more details.

      You should have received a copy of the GNU Library General Public
      License along with this library; if not, write to the Free
      Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

      polygon0.c
      (C) 1997 Alexey Nikitin, Michael Leonov
*/

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <anvil/vectmatr.h>
#include <anvil/polyisect.h>
#include <anvil/graphstructs.h>

/* create a polyarea from a set of points */

AmErrorCode 
AmMakePolyArea( AmPointPtr	pts,
		int		numpts,
		POLYAREA	**polyArea)
{
    int		ii;
    Vector	v;
    PLINE	*pline;
    POLYAREA	*poly;

    *polyArea = NULL;
    pline = NULL;
    for ( ii = 0; ii < numpts; ii++) {
	v[ 0] = pts[ ii].x;
	v[ 1] = pts[ ii].y;
	v[ 2] = 0.0;
	if ( pline == NULL) {
	    if (( pline = AmPolyNewContour( v)) == NULL) {
		DO_RETURN_ERROR( AmMALLOC_FAILED);
	    }
	} else {
	    AmPolyInclVertex( pline->head.prev, AmPolyCreateNode( v));
	}
    }
    AmPolyPreContour( pline, TRUE);
    poly = AmPolyCreate();
    AmPolyInclContour( poly, pline);
    AmPolyM_Incl( polyArea, poly);
    return( AmOK);
}

/* see if two poly areas are equal */

AnvilBoolean
AmEqualPolyArea( POLYAREA	*polyA,
		 POLYAREA	*polyB)
{
    POLYAREA	*polyAptr, *polyBptr;
    PLINE	*contourAptr, *contourBptr;
    VNODE	*ptAptr, *ptBptr;

    polyAptr = polyA;
    polyBptr = polyB;
    do {
	for ( contourAptr = polyAptr->contours, 
		  contourBptr = polyBptr->contours;
	      ( contourAptr != NULL) && ( contourBptr != NULL);
	      contourAptr = contourAptr->next, 
		  contourBptr = contourBptr->next) {
	    ptAptr = contourAptr->head.next;
	    ptBptr = contourBptr->head.next;
	    do {
		if (( ptAptr->point[ 0] != ptBptr->point[ 0]) ||
		    ( ptBptr->point[ 1] != ptBptr->point[ 0])) {
		    return( FALSE);
		}
		ptAptr = ptAptr->next;
		ptBptr = ptBptr->next;
	    } while (( ptAptr != &contourAptr->head) &&
		     ( ptBptr != &contourBptr->head));
	    if (( ptAptr != &contourAptr->head) ||
		( ptBptr != &contourBptr->head)) {
		return( FALSE);
	    }
	}
	if (( contourAptr != NULL) || ( contourBptr != NULL)) {
	    return( FALSE);
	}
	polyAptr = polyAptr->f;
	polyBptr = polyBptr->f;
    } while (( polyAptr != polyA) && ( polyBptr != polyB));
    if (( polyAptr != polyA) || ( polyBptr != polyB)) {
	return( FALSE);
    }
    return( TRUE);
}
	
    

#define SETBITS(n,mask,s) (((n)->Flags & ~(mask)) | ((s) & mask))
#define GETBITS(n, mask) ((n)->Flags & mask)

static void 
contourBoxAdjust(PLINE * c, Vector p)
{
    c->xmin = min(c->xmin, p[0]);
    c->xmax = max(c->xmax, p[0]);
    c->ymin = min(c->ymin, p[1]);
    c->ymax = max(c->ymax, p[1]);
}

static int 
contourBoxPointIn(PLINE * c, Vector p)
{
    return (p[0] >= c->xmin && p[1] >= c->ymin &&
            p[0] <= c->xmax && p[1] <= c->ymax);

}

static int 
contourBoxInside(PLINE * c1, PLINE * c2)
{
    assert(c1 != NULL && c2 != NULL);
    return ((c1->xmin >= c2->xmin) &&
            (c1->ymin >= c2->ymin) &&
            (c1->xmax <= c2->xmax) &&
            (c1->ymax <= c2->ymax));
}

static AnvilBoolean 
nodeNeighbors(VNODE * a, VNODE * b)
{
    return (a == b) || (a->next == b) || (b->next == a) ||
    (a->next == b->next);
}

VNODE *
AmPolyCreateNode(Vector v)
{
    VNODE *res;

    if (v == NULL)
        return NULL;
    res = (VNODE *) malloc(sizeof(VNODE));
    if (res == NULL)
        return NULL;
    res->Flags = 0;
    res->t.link = NULL;
    vect_copy(res->point, v);
    res->prev = res->next = res;
    return res;
}

void 
AmPolyInitContour(PLINE * c)
{
    if (c == NULL)
        return;
    c->next = NULL;
    c->Flags = 0;
    c->Count = 0;
    vect_copy(c->head.point, vect_zero);
    c->head.next = c->head.prev = &c->head;
    c->xmin = c->ymin = +1.0e+30f;
    c->xmax = c->ymax = -1.0e+30f;
}

PLINE *
AmPolyNewContour(Vector v)
{
    PLINE *res;

    res = (PLINE *)calloc(1, sizeof(PLINE));
    if (res == NULL)
        return NULL;

    AmPolyInitContour(res);

    if (v != NULL)
    {
        vect_copy(res->head.point, v);
        contourBoxAdjust(res, v);
    }
    
    return res;
}

void 
AmPolyClearContour(PLINE * c)
{
    VNODE *cur;

    assert(c != NULL);
    while ( ( cur = c->head.next ) != &c->head )
    {
        AmPolyExclVertex(cur);
        free(cur);
    }
    AmPolyInitContour(c);
}

void 
AmPolyDelContour(PLINE ** c)
{
    if (*c == NULL)
        return;
    AmPolyClearContour(*c);
    free(*c), *c = NULL;
}

#define IsZero(a, b) (fabs((a) - (b)) < EPSILON)

void 
AmPolyPreContour(PLINE *C, AnvilBoolean optimize)
{
    double  area = 0;
    VNODE	*p, *c;

    assert(C != NULL);

	if (optimize)
	{
		for (c = (p = &C->head)->next; c != &C->head; c = (p = c)->next)
		{
			if (IsZero(p->point[0], c->point[0]) &&
				IsZero(p->point[1], c->point[1]))
			{
				AmPolyExclVertex(c);
				free(c);
				c = p;
			}
		}
	}
	C->Count = 0;
    C->xmin = C->xmax = C->head.point[0];
    C->ymin = C->ymax = C->head.point[1];

    p = (c = &C->head)->prev;
	if (c != p)
	{
		do {
			/* calculate area for orientation */
			area += (p->point[0] - c->point[0]) * (p->point[1] + c->point[1]);
			contourBoxAdjust(C, c->point);
			C->Count++;
		} while ((c = (p = c)->next) != &C->head);
	}
    if (C->Count > 2)
        C->Flags = SETBITS(c, PLF_ORIENT, area < 0 ? PLF_INV : PLF_DIR);
} /* AmPolyPreContour */

void 
AmPolyInvContour(PLINE * c)
{
    VNODE *cur, *next;

    assert(c != NULL);
    cur = &c->head;
    do
    {
        next = cur->next;
        cur->next = cur->prev;
        cur->prev = next;
    } while ((cur = next) != &c->head);
    c->Flags ^= PLF_ORIENT;
}

void 
AmPolyInclVertex(VNODE * after, VNODE * node)
{
    assert(after != NULL);
    assert(node != NULL);

    node->prev = after;
    node->next = after->next;
    after->next = after->next->prev = node;
}

void 
AmPolyExclVertex(VNODE * node)
{
    assert (node != NULL);

    node->prev->next = node->next;
    node->next->prev = node->prev;
}

AnvilBoolean 
AmPolyCopyContour(PLINE ** dst, PLINE * src)
{
    VNODE *cur, *newnode;

    assert(src != NULL);
    *dst = NULL;
    *dst = AmPolyNewContour(src->head.point);
    if (*dst == NULL)
        return FALSE;

    (*dst)->Count = src->Count;
    (*dst)->Flags = src->Flags;
    (*dst)->xmin = src->xmin, (*dst)->xmax = src->xmax;
    (*dst)->ymin = src->ymin, (*dst)->ymax = src->ymax;

    for (cur = src->head.next; cur != &src->head; cur = cur->next)
    {
        if ( (newnode = AmPolyCreateNode(cur->point) ) == NULL)
            return FALSE;
        newnode->Flags = cur->Flags;
        AmPolyInclVertex((*dst)->head.prev, newnode);
    }
    return TRUE;
}

/**********************************************************************/
/* polygon routines */

AnvilBoolean 
AmPolyCopy0(POLYAREA ** dst, const POLYAREA * src)
{
    *dst = NULL;
    if (src != NULL)
        *dst = ( POLYAREA *) calloc(1, sizeof(POLYAREA));
    if (*dst == NULL)
        return FALSE;

    return AmPolyCopy1(*dst, src);
}

AnvilBoolean 
AmPolyCopy1(POLYAREA * dst, const POLYAREA * src)
{
    PLINE *cur, **last = &dst->contours;

    *last = NULL;
    dst->f = dst->b = dst;

    for (cur = src->contours; cur != NULL; cur = cur->next)
    {
        if (!AmPolyCopyContour(last, cur)) 
            return FALSE;
        last = &(*last)->next;
    }
    return TRUE;
}

void 
AmPolyM_Incl(POLYAREA **list, POLYAREA *a)
{
    if (*list == NULL)
        a->f = a->b = a, *list = a;
    else
    {
        a->f = *list;
        a->b = (*list)->b;
        (*list)->b = (*list)->b->f = a;
    }
}

AnvilBoolean 
AmPolyM_Copy0(POLYAREA ** dst, const POLYAREA * srcfst)
{
    const POLYAREA *src = srcfst;
    POLYAREA *di;

    *dst = NULL;
    if (src == NULL)
        return FALSE;
    do {
        if ((di = AmPolyCreate()) == NULL ||
            !AmPolyCopy1(di, src))
            return FALSE;
        AmPolyM_Incl(dst, di);
    } while ((src = src->f) != srcfst);
    return TRUE;
}

AnvilBoolean 
AmPolyInclContour(POLYAREA * p, PLINE * c)
{
    PLINE *tmp;

    if ((c == NULL) || (p == NULL))
        return FALSE;
    if (GETBITS(c, PLF_ORIENT) == PLF_DIR)
    {
        if (p->contours != NULL)
            return FALSE;
        p->contours = c;
    }
    else
    {
        if (p->contours == NULL)
            return FALSE;
        for (tmp = p->contours; tmp->next != NULL; tmp = tmp->next);
        tmp->next = c;
        c->next = NULL;
    }
    return TRUE;
}

AnvilBoolean 
AmPolyM_InclContour(POLYAREA ** p, PLINE * c)
{
    POLYAREA *a, *t;

    if (c == NULL)
        return FALSE;
    if (GETBITS(c, PLF_ORIENT) == PLF_DIR)
    {
        t = AmPolyCreate();
        if (*p != NULL)
        {
            a = (*p)->b;
            (((t->f = a->f)->b = t)->b = a)->f = t;
        }
        else
            *p = t;
        return AmPolyInclContour(t, c);
    }
    else
    {
        if (*p == NULL)
            return FALSE;
        a = *p;
        t = NULL;
        do {
            /* find the smallest container for the hole */
            if (AmPolyContourInContour(a->contours, c) &&
               (t == NULL ||
                AmPolyContourInContour(t->contours, a->contours)))
                t = a;
        } while ((a = a->f) != *p);
        if (t == NULL)
            return FALSE;
        else
            return AmPolyInclContour(t, c);
    }
}

AnvilBoolean 
AmPolyExclContour(POLYAREA * p, PLINE * c)
{
    PLINE *tmp;

    if ((c == NULL) || (p == NULL) || (p->contours == NULL))
        return FALSE;
    if (GETBITS(c, PLF_ORIENT) == PLF_DIR)
    {
        if (c != p->contours)
            return FALSE;
        assert(c->next == NULL);
        p->contours = NULL;
    }
    else
    {
        for (tmp = p->contours; (tmp->next != c) && (tmp->next != NULL);
             tmp = tmp->next) {}
        if (tmp->next == NULL)
            return FALSE;
        tmp->next = c->next;
    }
    return TRUE;
}


AnvilBoolean 
AmPolyM_ExclContour(POLYAREA * p, PLINE * c)
{
    POLYAREA *a = p;

    if (p == NULL || c == NULL)
        return FALSE;
    do {
        if (AmPolyExclContour(a, c))
            return TRUE;
    } while ((a = a->f) != p);
    return FALSE;
}

int 
AmPolyInsideContour(PLINE * c, Vector p)
{
    int f = 0;
    VNODE *cur;

    if (!contourBoxPointIn(c, p))
        return FALSE;

    cur = &c->head;
    do
    {
        if ((
             ((cur->point[1] <= p[1]) && (p[1] < cur->prev->point[1])) ||
             ((cur->prev->point[1] <= p[1]) && (p[1] < cur->point[1]))
             ) && (p[0] <
           (cur->prev->point[0] - cur->point[0]) * (p[1] - cur->point[1])
                 / (cur->prev->point[1] - cur->point[1]) + cur->point[0])
            )
            f = !f;
    } while ((cur = cur->next) != &c->head);
    return f;
}


AnvilBoolean 
AmPolyCheckInside(POLYAREA * p, Vector v0)
{
    PLINE *cur;

    if ((p == NULL) || (v0 == NULL) || (p->contours == NULL))
        return FALSE;
    cur = p->contours;
    if (AmPolyInsideContour(cur, v0))
    {
        for (cur = cur->next; cur != NULL; cur = cur->next)
            if (AmPolyInsideContour(cur, v0))
                return FALSE;
        return TRUE;
    }
    return FALSE;
}

AnvilBoolean 
AmPolyM_CheckInside(POLYAREA * p, Vector v0)
{
    POLYAREA *cur;

    if ((p == NULL) || (v0 == NULL))
        return FALSE;
    cur = p;
    do {
        if (AmPolyCheckInside(cur, v0))
            return TRUE;
    } while ((cur = cur->f) != p);
    return FALSE;
}

int 
AmPolyContourInContour(PLINE * poly, PLINE * inner)
{
    assert(poly != NULL);
    assert(inner != NULL);
    if (!contourBoxInside(inner, poly))
        return FALSE;
    return AmPolyInsideContour(poly, inner->head.point);
}

void 
AmPolyInit(POLYAREA  *p)
{
    p->f = p->b = p;
    p->contours = NULL;
}

POLYAREA *
AmPolyCreate(void)
{
    POLYAREA *res;

    if ((res = ( POLYAREA *) malloc(sizeof(POLYAREA))) != NULL) {
	AmPolyInit(res);
    }
    return res;
}

void 
AmPolyClear(POLYAREA *P)
{
    PLINE *p;

    assert(P != NULL);
    while ((p = P->contours) != NULL)
    {
        P->contours = p->next;
        AmPolyDelContour(&p);
    }
}

void 
AmPolyFree(POLYAREA ** p)
{
    POLYAREA *cur;

    if (*p == NULL)
        return;
    for (cur = (*p)->f; cur != *p; cur = (*p)->f)
    {
        AmPolyClear(cur);
        cur->f->b = cur->b;
        cur->b->f = cur->f;
        free(cur);
    }
    AmPolyClear(cur);
    free(*p), *p = NULL;
}

static AnvilBoolean 
insideSector(VNODE * pn, Vector p2)
{
    Vector cdir, ndir, pdir;
    int p_c, n_c, p_n;

    assert(pn != NULL);
    vect_sub(cdir, p2, pn->point);
    vect_sub(pdir, pn->point, pn->prev->point);
    vect_sub(ndir, pn->next->point, pn->point);

    p_c = vect_det2(pdir, cdir) >= 0;
    n_c = vect_det2(ndir, cdir) >= 0;
    p_n = vect_det2(pdir, ndir) >= 0;

    if ((p_n && p_c && n_c) || ((!p_n) && (p_c || n_c)))
        return TRUE;
    else
        return FALSE;
} /* insideSector */

/* returns TRUE if bad contour */

AnvilBoolean 
AmPolyChkContour(PLINE * a)
{
    VNODE *a1, *a2, *hit1, *hit2;
    Vector i1, i2;
    int    icnt = 0;

    assert(a != NULL);
    a1 = &a->head;
    do
    {
        a2 = a1;
        do
        {
            if (!nodeNeighbors(a1, a2) &&
                (icnt = vect_inters2(a1->point, a1->next->point,
                              a2->point, a2->next->point, i1, i2)) > 0)
            {
                if (icnt > 1)
                    return TRUE;

                if (vect_dist2(i1, a1->point) < EPSILON)
                    hit1 = a1;
                else if (vect_dist2(i1, a1->next->point) < EPSILON)
                    hit1 = a1->next;
                else
                    return TRUE;

                if (vect_dist2(i1, a2->point) < EPSILON)
                    hit2 = a2;
                else if (vect_dist2(i1, a2->next->point) < EPSILON)
                    hit2 = a2->next;
                else
                    return TRUE;

                /* now check if they are inside each other */
                if (insideSector(hit1, hit2->prev->point) ||
                    insideSector(hit1, hit2->next->point) ||
                    insideSector(hit2, hit1->prev->point) ||
                    insideSector(hit2, hit1->next->point))
                    return TRUE;
            }
        }
        while ((a2 = a2->next) != &a->head);
    }
    while ((a1 = a1->next) != &a->head);
    return FALSE;
}


AnvilBoolean 
AmPolyValid(POLYAREA * p)
{
    PLINE *c;

    if ((p == NULL) || (p->contours == NULL))
        return FALSE;

    if (GETBITS(p->contours, PLF_ORIENT) == PLF_INV ||
        AmPolyChkContour(p->contours))
        return FALSE;
    for (c = p->contours->next; c != NULL; c = c->next)
    {
        if (GETBITS(c, PLF_ORIENT) == PLF_DIR ||
            AmPolyChkContour(c) ||
            !AmPolyContourInContour(p->contours, c) )
            return FALSE;
    }
    return TRUE;
}

