#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* polygon containment utilities. */

#include <anvil/project.h>
#include <anvil/datafmt.h>
#include <anvil/geometry.h>
#include <anvil/macros.h>

/*************************************************************************
 * 
 * Description:
 *	is a point inside of a polygon?
 * Arguments:
 *	x, y - point to test.
 *	poly - polygon to test against.
 * Returns:
 *	TRUE if the point is inside the polygon, FALSE otherwise.
 * 
 ************************************************************************/

AnvilBoolean 
AmPtInPoly( float		x,
	  float		y,
	  AmPolygonPtr	poly)
{
    int		count;
    AmCoordPtr	currPt, lastPt, endPt;

/* if the minimum bounding rectangle is set, then trival test against it */

    if ( poly->flags & AM_MBR_FLAG) {
	if (( x < poly->xmin) || ( y < poly->ymin) ||
	    ( x > poly->xmax) || ( y > poly->ymax)) {
	    return( FALSE);
	}
    }
    currPt = poly->pts;
    endPt = lastPt = poly->pts + poly->numPts - 1;
    count = 0;

/* find out how many lines in the polygon straddle the point's x coordinate
   and are above it */

    while ( currPt <= endPt) {
	if ((( x >= lastPt->x) && ( x < currPt->x)) ||
	    (( x >= currPt->x) && ( x < lastPt->x))) {
	    if ( AmPtAboveLine( x, y, lastPt->x, lastPt->y, 
			      currPt->x, currPt->y)) {
		count++;
	    }
	}
	lastPt = currPt;
	currPt++;
    }
    return( ODD( count));
}

/*************************************************************************
 * 
 * Description:
 * 	is a line inside of a polygon? 
 * Arguments:
 *	line - line to test.
 *	poly - polygon to test against.
 * Returns:
 *	TRUE if the point is inside the polygon, FALSE otherwise.
 * 
 ************************************************************************/

AnvilBoolean 
AmLineInPoly( AmLinePtr	line,
	    AmPolygonPtr	poly,
	    AnvilBoolean	entireFlag)
{
    if ( entireFlag) {
	return( AmPtInPoly( line->x0, line->y0, poly) &&
		AmPtInPoly( line->x1, line->y1, poly));
    } else {
	return( AmPtInPoly( line->x0, line->y0, poly) ||
		AmPtInPoly( line->x1, line->y1, poly));
    }
}

/*************************************************************************
 * 
 * Description:
 * 	is a polygon inside of a polygon? 
 * Arguments:
 *	line - line to test.
 *	poly - polygon to test against.
 * Returns:
 *	TRUE if the point is inside the polygon, FALSE otherwise.
 * Modifications:
 *	There are concave polygons where this will not work properly.
 * 
 ************************************************************************/

AnvilBoolean 
AmPolyInPoly( AmPolygonPtr	inside,
	    AmPolygonPtr	contains,
	    AnvilBoolean	entireFlag)
{
    int	ii;

/* test the trivial reject AmFirst */

    if (( inside->flags & AM_MBR_FLAG) && ( contains->flags & AM_MBR_FLAG)) {
	if (( contains->xmax < inside->xmin) ||
  	    ( contains->ymax < inside->ymin) ||
	    ( contains->xmin > inside->xmax) ||
	    ( contains->ymin > inside->ymax)) {
	}
    }
    if ( entireFlag) {
	for ( ii = 0; ii < inside->numPts; ii++) {
	    if ( !AmPtInPoly( inside->pts[ ii].x, inside->pts[ ii].y, 
			    contains)) {
		return( FALSE);
	    }
	}
	return( TRUE);
    } else {
	for ( ii = 0; ii < inside->numPts; ii++) {
	    if ( AmPtInPoly( inside->pts[ ii].x, inside->pts[ ii].y, 
			   contains)) {
		return( TRUE);
	    }
	}
	return( FALSE);
    } 
}
