#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <anvil/project.h>
#include <anvil/geometry.h>

/****************************************************************************
 *
 * Description:
 * 	is a point (x,y) above a line (x0,y0), (x1,y1)
 * Returns:
 * 	TRUE if the point is above the line, FALSE otherwise.
 *
 ****************************************************************************/

AnvilBoolean 
AmPtAboveLine( float 	x,
	     float 	y,
	     float 	x0,
	     float 	y0,
	     float 	x1,
	     float 	y1)
{
    float m;

/* if the line is vertical or the point is below both points, then 
   trivial reject */

    if (( x1 == x0) || (( y < y0) && ( y < y1))) {
	return( FALSE);
    }

/* y - y0 = ((y1 - y0)/(x1 - x0)) * ( x - x0) line equation */

    m = ( y1 - y0) / ( x1 - x0);
    return( m*(x - x0) + y0 < y);
}

/****************************************************************************
 *
 * Description:
 * 	is a point (x,y) above a line (x0,y0), (x1,y1)
 * Returns:
 * 	TRUE if the point is above the line, FALSE otherwise.
 *
 ****************************************************************************/

AnvilBoolean 
AmIntPtAboveLine( int 	x,
		int 	y,
		int 	x0,
		int 	y0,
		int 	x1,
		int 	y1)
{
    float m;

/* if the line is vertical or the point is below both points, then 
   trivial reject */

    if (( x1 == x0) || (( y < y0) && ( y < y1))) {
	return( FALSE);
    }

/* y - y0 = ((y1 - y0)/(x1 - x0)) * ( x - x0) line equation */

    m = (( float) ( y1 - y0)) / (( float) ( x1 - x0));
    return( m*( x - x0) + y0 < y);
}
