#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* file reading utility functions. */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <anvil/project.h>
#include <anvil/strlib.h>
#include <anvil/files.h>

/****************************************************************************
 *
 * Description:
 * 	read a file into a string buffer, and return the allocated 
 *	buffer.  a '\0' is added 1 character after the end of the string.
 * Arguments:
 *	filename - filename to read from.
 * Returns:
 *	allocated buffer which data was read into, or NULL if the file
 *	couldn't be opened or statted, or if the buffer malloc() failed.
 *
 ****************************************************************************/

char *
AmReadFileString( const char *filename)
{
    int		fd;
    struct stat buf;
    char	*buffer;

    if ( stat( filename, &buf) < 0) {
	return( NULL);
    }
    if (( buffer = ( char *) malloc( buf.st_size + 1)) == NULL) {
	return( NULL);
    }
    if (( fd = open( filename, O_RDONLY)) < 0) {
	free( buffer);
	return( NULL);
    }
    if ( read( fd, buffer, buf.st_size) < 0) {
	free( buffer);
	close( fd);
	return( NULL);
    }

/* terminate the string */

    buffer[ buf.st_size] = '\0';
    close( fd);
    return( buffer);
}
