#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* utility functions for reading real values from files and strings */

#include <malloc.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <anvil/project.h>
#include <anvil/files.h>

/****************************************************************************
 *
 * Description:
 *      read a vector of integers from a file.
 * Arguments:
 *      filename - file to read values from.
 * Returns:
 *      allocated vector of integer values, or NULL if it couldn't be
 *      read or allocated.
 * Modifications:
 *      it should return the number of values, and an error code, rather
 *      than the vector.
 *
 ****************************************************************************/

int *
AmReadIntVec( const char *filename)
{
    FILE 	*fp;
    char	buffer[ 512];
    int		ii, lineno = 1, *vector;

/* read it to find the number of lines */

    if (( fp = fopen( filename, "r")) == NULL) {
	return( NULL);
    }
    do {	
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    break;
	}
	lineno++;
    } while ( TRUE);

/* rewind and allocate the vector */

    rewind( fp);
    vector = ( int *) malloc( sizeof( int)*lineno);
    if ( vector == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
 	return( NULL);
    }
    for ( ii = 0; ii < lineno; ii++) {
	fgets( buffer, sizeof( buffer), fp);
	sscanf( buffer, "%d", &vector[ ii]);
    } 

/* close and return */

    fclose( fp);
    return( vector);
}

/****************************************************************************
 *
 * Description:
 *      read a vector of integers from a string.  The string is trashed
 *      when the integers are read.
 * Arguments:
 *      str - string to scan.
 * Returns:
 *      Allocated integers vector or NULL if the vector couldn't be allocated.
 * Modifications:
 *      it should return the number of values, and an error code, rather
 *      than the vector.
 *
 ****************************************************************************/

int *
AmReadIntStrVec( char *str)
{
    int		ii, length = 0, *vec;
    char	*p, *q;

/* check for NULL */

    if ( !strcasecmp( str, "NULL")) {
	return( NULL);
    }

/* scan over each digit to find the length don't forget negatives */

    for ( p = str; *p != '\0'; ) {

	while (( *p == '-') || ( *p == '+') || isdigit( *p)) {
	    p++;
	}

	while (( *p != '\0') && ( *p != '-') &&
               ( *p != '+') && !isdigit( *p)) {
	    p++;
	}

	length++;
    }

/* atoi each digit ( writing a '\0' at the end of each #) into the array */

    vec = ( int *) malloc( length*sizeof( int));
    for ( p = str, ii = 0; *p != '\0'; ii++) {
	for ( q = p; ( *q == '-') || isdigit( *q); q++);
	*q = '\0';
	vec[ ii] = atoi( p);
	if ( ii < length - 1) {
	    q++;
	    while (( *q != '\0') && ( *q != '-') && !isdigit( *q)) {
		q++;
	    }
	}
	p = q;
    }
    return( vec);
}
