%option noyywrap
%{
#include <string.h>
#include <anvil/project.h>
#include <anvil/regparse.h>
#include "regexp.tab.h"

#undef YY_INPUT
#define YY_INPUT( buffer, result, maxSize) (  result = AmStrYYinput( buffer, maxSize))

int inCharsetFlag = 0;
%}
%option noyywrap
 
%%

 /* dot means any character outside of a charset [], where it's just another
    character to be matched */

\.			{ if ( inCharsetFlag) {
			      inCharsetFlag++;
			      regexplval.string = strdup( yytext);
			      return( CHARACTER);
			  } else {
			      regexplval.string = strdup( yytext);
			      return( DOT);
			  }
			}

 /* vertical bar means two or'ed regular expressions */

\|			{ if ( inCharsetFlag) {
			      inCharsetFlag++;
			      regexplval.string = strdup( yytext);
			      return( CHARACTER);
			  } else {
			      regexplval.string = strdup( yytext);
			      return( BAR);
			  }
			}

 /* star means 0 or more of the previous RE, unless it's in a charset [] */

\*			{ if ( inCharsetFlag) {
			      inCharsetFlag++;
			      regexplval.string = strdup( yytext);
			      return( CHARACTER);
			  } else {
			      return( STAR);
			  }
			}

 /* open bracket starts a charset, except in a charset */

\[			{ if ( inCharsetFlag) {
			      inCharsetFlag++;
			      regexplval.string = strdup( yytext);
			      return( CHARACTER);
			  } else {
			      inCharsetFlag = 1;
			      return( OPEN_SQ);
			  }
			}

 /* close bracket ends a charset, uless it's the first character in a charset 
    ( this also applies to -, so the charset count isn't incremented, because
    a '-' might follow. Once the charset is ended, turn off the charset flag */

\]			{ if ( inCharsetFlag == 1) {
			      regexplval.string = strdup( yytext);
			      return( CHARACTER);
			  } else {
			      inCharsetFlag = 0;
			      return( CLOSE_SQ);
			  }
			}

 /* - denotes a character range unless it's the first character in a charset,
    or outside of a charset */

\-			{ if ( inCharsetFlag <= 1) {
			      regexplval.string = strdup( yytext);
			      return( CHARACTER);
			  } else {
			      inCharsetFlag++;
			      return( MINUS);
			  }
			}

 /* number of character range expressions, fixed number, minimum and range */

\\\{[0-9]*\\\} 		{ regexplval.string = strdup( yytext);
			  return( FIXED_REPEAT); }
\\\{[0-9]*\,\\\} 	{ regexplval.string = strdup( yytext);
			  return( MIN_REPEAT); }
\\\{[0-9]*\,[0-9]*\\\} 	{ regexplval.string = strdup( yytext);
			  return( RANGE_REPEAT); }

 /* backslash escapes characters, unless it's just after a backslash */

\\.			{ regexplval.string = strdup( &yytext[ 1]);
			  return( CHARACTER);
			}


 /* if the character is in a charset, track number of chars in the charset */

.			{ regexplval.string = strdup( yytext);
			  if ( inCharsetFlag) {
			      inCharsetFlag++;
			  }
		  	  return( CHARACTER); 
		        }
%%
 
int yyerror( const char *msg)
{
    printf( "%s at '%s'\n", msg, yytext);
    return( -1);
}
