%{
#include <stdio.h>
#include <string.h>
#if !defined(_WIN32) && !defined(__BSD44__)
#include <alloca.h>
#else 
#include <malloc.h>
#endif
#include <anvil/regparse.h>
#undef YY_INPUT
#define YY_INPUT( buffer, result, maxSize) (  result = strYYinput( buffer, maxSize))
int yylex( void);
int yyerror( const char *msg);
%}

%union {
    AmREPtr	RE;
    char	*string;
}

%token <string> BACKSLASH
%token <string>	CHARACTER
%token <string>	DOT
%token <string>	FIXED_REPEAT
%token <string>	MIN_REPEAT
%token <string>	RANGE_REPEAT
%token <string>	STAR
%token <string> OPEN_SQ
%token <string> CLOSE_SQ
%token <string> MINUS
%token <string> HAT
%token <string> BAR

%type <RE> 	charRange
%type <RE> 	charsetRE
%type <RE> 	charset
%type <RE> 	occurRangeRE
%type <RE> 	oneCharRE
%type <RE> 	regex
%type <RE> 	starRE
%type <RE> 	singleCharRE

%start regex

%%

regex		: regex occurRangeRE		{ $$ = AmAddRegexRE( $2, $1); }
		| regex starRE			{ $$ = AmAddRegexRE( $2, $1); }
		| regex singleCharRE		{ $$ = AmAddRegexRE( $2, $1); }
		| occurRangeRE			{ $$ = AmAddRegexRE( $1, NULL); }
		| starRE			{ $$ = AmAddRegexRE( $1, NULL); }
		| singleCharRE			{ $$ = AmAddRegexRE( $1, NULL); }
		| '(' regex ')' BAR '(' regex ')'	{ $$ = AmOrRegexRE( $2, $6); }
		;

occurRangeRE	: singleCharRE FIXED_REPEAT 	{ $$ = AmMakeOccurRE( $1, $2, AmRANGE_FIXED); }
		| singleCharRE MIN_REPEAT       { $$ = AmMakeOccurRE( $1, $2, AmMIN_OCCURRENCES); }
		| singleCharRE RANGE_REPEAT	{ $$ = AmMakeOccurRE( $1, $2, AmRANGE_OCCURRENCES); }
		;

starRE		: singleCharRE STAR		{ $$ = AmMakeStarRE( $1); }

singleCharRE	: charsetRE			{ $$ = $1; }
		| oneCharRE			{ $$ = $1; }
		;

charsetRE	: OPEN_SQ charset CLOSE_SQ	{ $$ = $2; }

charset		: charRange			{ $$ = AmAddCharsetRE( $1, NULL); }
		| CHARACTER			{ $$ = AmAddCharsetRE( AmMakeOneCharRE( $1), NULL); }
		| charset charRange		{ $$ = AmAddCharsetRE( $2, $1); }
		| charset CHARACTER		{ $$ = AmAddCharsetRE( AmMakeOneCharRE( $2), $1); }
		;

charRange	: CHARACTER MINUS CHARACTER	{ $$ = AmMakeRangeRE( $1, $3); }

oneCharRE 	: CHARACTER			{ $$ = AmMakeOneCharRE( $1); }
	  	| DOT				{ $$ = AmMakeDotRE( $1); }
	  	;

%%
