#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <alloca.h>
#include <stdio.h>
#include <anvil/project.h>
#include <anvil/region.h>
#include <anvil/geometry.h>
#include <anvil/macros.h>

typedef struct mergePt_ mergePtRec, *mergePtPtr;

/* merge point for intersections, contains x,y coordinate, pointers to
   neighboring points, whether point is inside both polygons, visit
   flag for traversals */

struct mergePt_ {
    int		x;
    int		y;
    int		numpts;
    mergePtPtr	pts[ 4];
    AnvilBoolean	inside;
    AnvilBoolean	visit;
    AnvilBoolean	polyFlag;
};

/* for walking down list of merge points,  Given last point and current
   point, get the next point */

static mergePtPtr 
nextMergePt( mergePtPtr a,
	     mergePtPtr b)
{
    int	ii;

    if ( b == NULL) {
	return( a->pts[ 0]);
    }
    for ( ii = 0; ii < b->numpts; ii++) {
	if (( b->pts[ ii] != a) && ( b->pts[ ii]->polyFlag == b->polyFlag)) {
	    return( b->pts[ ii]);
	}
    }
    return( NULL);
}

/* insert point c in between points a and b */

static void
insertMergePt( mergePtPtr a,
	       mergePtPtr b,
	       mergePtPtr c)
{   
    int	ii;

    for ( ii = 0; ii < a->numpts; ii++) {
	if ( a->pts[ ii] == b) {
	    a->pts[ ii] = c;
	    break;
	}
    }
    c->pts[ c->numpts++] = a;
    for ( ii = 0; ii < b->numpts; ii++) {
	if ( b->pts[ ii] == a) {
	    b->pts[ ii] = c;
	    break;
	}
    }
    c->pts[ c->numpts++] = b;
}

static mergePtPtr 
newPoint( int		x,
	  int		y,
	  mergePtPtr	ptVec,
	  int		*numPts,
	  int		*maxPts)
{
    int	ptIndex = *numPts;

    ptVec[ ptIndex].x = x;
    ptVec[ ptIndex].y = y;
    ptVec[ ptIndex].numpts = 0;
    ptVec[ ptIndex].inside = TRUE;
    ptVec[ ptIndex].visit = FALSE;
    ptVec[ ptIndex].polyFlag = TRUE;
    *numPts++;
    return( &ptVec[ ptIndex]);
}


/* given points from polygon 1 (a1, b1), and polygon 2 (a2,b2), if the
   segments intersect, create a new point, and intersect it between the
   two segments */

static AnvilBoolean
intersectMergePts( mergePtPtr	a1,
		   mergePtPtr	b1,
		   mergePtPtr	a2,
		   mergePtPtr	b2,
		   mergePtPtr	*newPt,
		   mergePtPtr	ptVec,
		   int		*numPts,
		   int		*maxPts)
{
    int	x, y;

    if ( AmIntLineIntersection( a1->x, a1->y, b1->x, b1->y,
			        a2->x, a2->y, b2->x, b2->y, 
			        &x, &y)) {
	*newPt = newPoint( x, y, ptVec, numPts, maxPts);
	insertMergePt( a1, b1, *newPt);
	insertMergePt( a2, b2, *newPt);
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/* given a segment defined by points a,b, walk through poly, and intersect
   it with each segment. */

static mergePtPtr
isectSegmentPoly( mergePtPtr	a,
		  mergePtPtr	b,
		  mergePtPtr	poly,
		  mergePtPtr	ptVec,
		  int		*numPts,
		  int		*maxPts)
{
    mergePtPtr	othera, otherb, oldOtherb, isectPt;

    othera = poly;
    otherb = othera->pts[ 0];
    do {

printf( "isect (%d, %d) (%d, %d) x (%d, %d) (%d %d)\n",
	a->x, a->y, b->x, b->y, 
	othera->x, othera->y, otherb->x, otherb->y);

	if ( intersectMergePts( a, b, othera, otherb, &isectPt, 
				ptVec, numPts, maxPts)) {
printf( "creating intersect point (%d, %d)\n", isectPt->x, isectPt->y);
	    isectPt->polyFlag = otherb->polyFlag;
	    isectSegmentPoly( a, isectPt, poly, ptVec, numPts, maxPts);
	    isectSegmentPoly( isectPt, b, poly, ptVec, numPts, maxPts);
	    return( isectPt);
	}
	oldOtherb = otherb;
	otherb = nextMergePt( othera, otherb);
	othera = oldOtherb;
    } while ( othera != poly);
    return( a);
}

/* intersect polygon a with polygon b */

static void 
isectPolygons( mergePtPtr	polya,
	       mergePtPtr	polyb,
	       mergePtPtr	ptVec,
	       int		*numPts,
	       int		*maxPts)
{
    mergePtPtr	currPt, nextPt, oldNextPt;

    currPt = polya;
    nextPt = polya->pts[ 0];
    do {

printf( "isect segment poly (%d, %d) (%d, %d)\n",
	currPt->x, currPt->y, nextPt->x, nextPt->y);

	currPt = isectSegmentPoly( currPt, nextPt, polyb, 
				   ptVec, numPts, maxPts);
	oldNextPt = nextPt;
	nextPt = nextMergePt( currPt, nextPt);
	currPt = oldNextPt;
    } while ( currPt != polya);
}

/* determine if x,y is contained in a polygon */

static AnvilBoolean
ptInPoly( int		x,
	  int		y,
	  AmPointPtr	pts,
	  int		numPts)
{
    int         count;
    AmPointPtr    currPt, lastPt, endPt;

    currPt = pts;
    endPt = lastPt = pts + numPts - 1;
    count = 0;

/* find out how many lines in the polygon straddle the point's x coordinate
   and are above it */

    while ( currPt <= endPt) {
        if ((( x >= lastPt->x) && ( x < currPt->x)) ||
            (( x >= currPt->x) && ( x < lastPt->x))) {
            if ( AmIntPtAboveLine( x, y, lastPt->x, lastPt->y,
				   currPt->x, currPt->y)) {
                count++;
            }
        }
        lastPt = currPt;
        currPt++;
    }
    return( ODD( count));
}

/* given two vectors of points, generate the vector of merge points */

static void
createMergePolyTable( AmPointPtr	a,
		      int		aNumpts,
		      AmPointPtr	b,
		      int		bNumpts,
		      mergePtPtr	*mergePts,
		      int		*numPts,
		      int		*maxPts)
{
    int	ii, jj, kk;

    *maxPts = ( aNumpts + bNumpts)*2;
    *numPts = aNumpts + bNumpts;
    *mergePts = ( mergePtPtr) malloc( sizeof( mergePtRec)*( *maxPts));
    jj = aNumpts - 1;
    kk = 1;
    for ( ii = 0; ii < aNumpts; ii++) {
	( *mergePts)[ ii].x = a[ ii].x;
	( *mergePts)[ ii].y = a[ ii].y;
	( *mergePts)[ ii].pts[ 0] = &( *mergePts)[ jj];
	( *mergePts)[ ii].pts[ 1] = &( *mergePts)[ kk];
	( *mergePts)[ ii].numpts = 2;
	( *mergePts)[ ii].inside = ptInPoly( a[ ii].x, a[ ii].y, b, bNumpts);
	( *mergePts)[ ii].visit = FALSE;
	( *mergePts)[ ii].polyFlag = FALSE;
	jj = ii;
	if ( ii < aNumpts - 2) {
	   kk = ii + 1;
	} else {
	   kk = 0;
	}
    }
    jj = aNumpts + bNumpts - 1;
    kk = aNumpts + 1;
    for ( ii = 0; ii < bNumpts; ii++) {
	( *mergePts)[ ii + aNumpts].x = b[ ii].x;
	( *mergePts)[ ii + aNumpts].y = b[ ii].y;
	( *mergePts)[ ii + aNumpts].pts[ 0] = &( *mergePts)[ jj];
	( *mergePts)[ ii + aNumpts].pts[ 1] = &( *mergePts)[ kk];
	( *mergePts)[ ii + aNumpts].numpts = 2;
	( *mergePts)[ ii + aNumpts].inside = 
		ptInPoly( b[ ii].x, b[ ii].y, a, aNumpts);
	( *mergePts)[ ii + aNumpts].visit = FALSE;
	( *mergePts)[ ii + aNumpts].polyFlag = TRUE;
	jj =  aNumpts + ii;
	if ( ii < bNumpts - 2) {
	   kk =  aNumpts + ii + 1;
	} else {
	   kk = aNumpts;
	}
    }
}

AmErrorCode 
AmIntersectPolygons( AmPointPtr	a,
		     int	aNumpts,
		     AmPointPtr	b,
		     int	bNumpts,
		     AmPointPtr	*intPolys,
		     int	*intNumpts,
		     int	*numIntPolys)
{
    mergePtPtr	mergePts;
    int		numMergePts, maxPts, ii;

    createMergePolyTable( a, aNumpts, b, bNumpts, &mergePts, &numMergePts,
			  &maxPts);
    isectPolygons( mergePts, &mergePts[ aNumpts], mergePts, &numMergePts,
		   &maxPts);
    for ( ii = 0; ii < numMergePts; ii++) {
	printf( "%d %d\n", mergePts[ ii].x, mergePts[ ii].y);
    }
    return( AmOK);
}

static void
createIntersectPolys( mergePtPtr	mergePts,
		      int		numMergePts,
		      AmPointPtr	*intersectPolys,
		      int		*polyNumpts,
		      int		*numPts)
{
    AmPointPtr	tempVec;
    int		numTempPts, ii;

    tempVec = ( AmPointPtr) alloca( numMergePts*sizeof( AmPointRec));
    for ( ii = 0; ii < numMergePts; ii++) {
	if ( mergePts[ ii].inside && !mergePts[ ii].visit) {
	}
    }
}
