#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* base class implementation for the resource database, from with the
   X resource and NT resource classes are derived */

#include <string.h>
#include <stdlib.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/resource.h>

static AmResourceDbClass	*THE_RESOURCE_DB = NULL;

/* get and set the resource database */

AmResourceDbClass *AmGetTheResourceDb()
{
    return( THE_RESOURCE_DB);
}

void AmSetTheResourceDb( AmResourceDbClass	*db)
{
    THE_RESOURCE_DB = db;
}

/* free the resource filename */

AmResourceDbClass::~AmResourceDbClass()
{
    if ( filename != NULL) {
	free( filename);
    }
}

/* base resource class constructor */

AmResourceDbClass::AmResourceDbClass( const char *filenameArg)
{
    if ( filenameArg != NULL) {
	filename = strdup( filenameArg);
    } else {
	filename = NULL;
    }
}

/* return the filename associated with the resource database */

const char *AmResourceDbClass::GetFilename()
{
    return( filename);
}

/* get a string default value */

const char *AmGetDefault( const char *tag, 
			const char *defaultValue)
{
    char *value;

    if ( AmGetTheResourceDb()->GetResource( tag, &value)) {
	return( value);
    } else {
	return( defaultValue);
    }
}

/* get an integer default value */

int AmGetDefault( const char 	*tag, 
		int 		defaultValue)
{
    int value;

    if ( AmGetTheResourceDb()->GetResource( tag, &value)) {
	return( value);
    } else {
	return( defaultValue);
    }
}

/* get a float default value */

float AmGetDefault( const char 	*tag, 
		  float 	defaultValue)
{
    float value;

    if ( AmGetTheResourceDb()->GetResource( tag, &value)) {
	return( value);
    } else {
	return( defaultValue);
    }
}

/* get a boolean default value */

AnvilBoolean AmGetDefault( const char 	*tag, 
		    AnvilBoolean 	defaultValue)
{
    AnvilBoolean value;

    if ( AmGetTheResourceDb()->GetResource( tag, &value)) {
	return( value);
    } else {
	return( defaultValue);
    }
}

/* put a string default value */

AnvilBoolean AmPutDefault( const char 	*tag,
		    const char	*value)
{
    return( AmGetTheResourceDb()->PutResource( tag, value));
}

/* put a float default value */

AnvilBoolean AmPutDefault( const char 	*tag,
		    float	value)
{
    return( AmGetTheResourceDb()->PutResource( tag, value));
}

/* put an int default value */

AnvilBoolean AmPutDefault( const char 	*tag,
		    int		value)
{
    return( AmGetTheResourceDb()->PutResource( tag, value));
}

/* put a boolean default value */

AnvilBoolean AmPutDefault( const char 	*tag,
		    AnvilBoolean	value)
{
    return( AmGetTheResourceDb()->PutResource( tag, value));
}
