#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <anvil/project.h>
#include <anvil/strlib.h>

/* copy a vector of strings */

char **
AmCopyStrVec( char const **vec)
{ 
    char 	**newVec, **newVecPtr;
    int 	length = 1;

    if ( vec == NULL) {
	return( NULL);
    }
    length = AmVeclen( vec) + 1;
    newVec = newVecPtr = ( char **) malloc( sizeof( char *)*length);
    while ( *vec != NULL) {
	*newVecPtr++ = strdup( *vec++);
    }
    *newVecPtr = NULL;
    return( newVec);
}

/* read a NULL-terminated vector of strings from a file, elminating 
   leading and trailing white spaces */

char **
AmReadStrvec( const char *filename)
{
    FILE 	*fp;
    char	buffer[ 512];
    int		lineno = 1;
    char	**vector, **vecPtr, *ptr;

/* read it to find the number of lines */

    if (( fp = fopen( filename, "r")) == NULL) {
	return( NULL);
    }
    do {	
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    break;
	}
	lineno++;
    } while ( TRUE);

/* rewind and allocate the vector */

    rewind( fp);
    vector = vecPtr = ( char **) malloc( sizeof( char *) * ( lineno + 1));

/* read the lines and "strdup" them after getting rid of the \n */

    do {
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    break;
	}
	buffer[ strlen( buffer) - 1] = '\0';

/* get rid of leading and trailing white spaces */

	ptr = AmElimLeadWspace( AmElimTrailWspace( buffer));
	*vecPtr++ = strdup( ptr);
    } while ( TRUE);
    *vecPtr = NULL;

/* close and return */

    fclose( fp);
    return( vector);
}

/* write a vector of strings to a file */

AnvilBoolean 
AmWriteStrvec( const char *file, 
	       const char **vec)
{
    FILE 	*fp;

    if (( fp = fopen( file, "w")) == NULL) {
	return( FALSE);
    }
    for ( ; *vec != NULL; vec++) {
	fprintf( fp, "%s\n", *vec);
    }
    fclose( fp);
    return( TRUE);
}

/* is a string in a vector of strings */

AnvilBoolean 
AmInStrvec( const char	*target,
	    const char	**vec)
{
    for ( ; *vec != NULL; vec++) {
    	if ( !strcmp( target, *vec)) {
    	    return( TRUE);
	}
    }	    
    return( FALSE);
}

/* return the index of the first occurrence of a string in a vector of strings */

int 
AmIndexStrvec( const char	*target,
	       const char	**vec)
{
    int	index = 0;

    if ( target == NULL) {
	return( -1);
    }

    for ( ; *vec != NULL; vec++, index++) {
    	if ( !strcmp( target, *vec)) {
    	    return( index);
	}
    }	    
    return( -1);
}

/* compare a pair of string vectors */

int 
AmCmpStrvec( const char	**a,
	     const char	**b)
{
    int	cmpVal;

    for (; ( *a != NULL) && ( *b != NULL); a++, b++) {
	if ((cmpVal = strcmp( *a, *b)) != 0) {
	    return( cmpVal);
	}
    }
    if ( *a != NULL) {
	return( -1);
    } else if ( *b != NULL) {
	return( 1);
    } else {
	return( 0);
    }
}

/* append two string vectors.  malloc the new vector, but not the strings */ 

char **
AmAppendStrvec( const char	**a,
		const char	**b)
{
    char **c;
    int	alen = AmVeclen( a), blen = AmVeclen( b);
    
    c = ( char **) malloc(( alen + blen + 1)*sizeof( char *));
    memcpy( c, a, alen*sizeof( char *));
    memcpy( &c[ alen], b, blen*sizeof( char *));
    c[ alen + blen] = NULL;
    return( c);
}

/* take the strings in a vector of strings and write them into a single
   buffer, separated by a delimiter... */

void 
AmStrvecToString( const char 	**strvec,
		  char 		*buffer,
		  int  		delim)
{
    int	len;

    for ( ;*strvec != NULL; strvec++) {
	len = strlen( *strvec);
	strcpy( buffer, *strvec);
	buffer += len;
	if ( strvec[ 1] != NULL) {
	    *buffer++ = delim;
	}
    }
    *buffer = '\0';
}
