#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#if !defined(__BSD44__)
#include <alloca.h>
#endif

#include <anvil/strlib.h>
#include <anvil/errcode.h>

/* This function counts the number of tokens in the string "buf"
   where the token separator is given by "sep" ... */

int AmCountTokens ( char	*buf,
		  char	*sep )
{
    char	*cptr;
    int		numTokens = 0;

    cptr = strtok ( buf, sep );
    if ( cptr == NULL ) return ( numTokens );

    while ( cptr != NULL ) {
	numTokens += 1;
	cptr = strtok ( NULL, sep );
    }

    return ( numTokens );

}

/* This function returns the number of tokens, as well as the
   tokens themselves ... the tokens are AmFirst stripped of all
   leading and trailing blanks as well as trailing "\n" ...
*/

int AmGetTokens ( char	*buf,
		char	*sep,
		char	***tokens )
{
    int		ii, numTokens=0, length;
    char	buf3[512];
    char	*cptr, *qptr, *buf2;

/* take out any trailing newlines */

    *tokens = NULL;
    length = strlen( buf);
    if ( buf[ length-1] == '\n' ) {
	buf[ length-1] = '\0';
    }

/* allocate the buffer to copy the string */

    buf2 = ( char *) alloca( length);
    if ( buf2 == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return( 0);
    }

/* find the number of tokens */

    strcpy ( buf2, buf );
    numTokens = AmCountTokens ( buf2, sep );

    if ( numTokens == 0 ) {
	return ( numTokens );
    }

/* allocate the vector of tokens */

    *tokens = (char **) calloc ( numTokens, sizeof(char *) );
    if ( *tokens == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( 0);
    }

/* copy the vector again, 'cause strtok is destructive */

    strcpy ( buf2, buf );
    cptr = strtok ( buf2, sep );
    ii = 0;
    while ( cptr != NULL ) {

/* take each tolen, and eliminate the leading and trailing whitespaces */

        strcpy ( buf3, cptr );
        qptr = AmElimTrailWspace( AmElimLeadWspace ( buf3 ));
        (*tokens)[ii++] = strdup ( qptr );
	cptr = strtok ( NULL, sep );
    }
    return ( ii );
}

/* free the tokens */

void AmFreeTokens ( int	numTokens,
		  char	***tokens )
{
    int		ii;

    for ( ii=0; ii<numTokens; ii++ ) {
	free ( (*tokens)[ii] );
    }
    free ( *tokens );
    *tokens = NULL;
}
