%{
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <anvil/list.h>
#include <anvil/units.h>
#include <anvil/unitsP.h>
int yylex( void);
int yyerror( const char *msg);
%}

%union {
    char	character;
    char	*string;
    AmExprPtr	expr;
}

%token <string> IDENT
%token <string> PRECISION
%token <string> PRINT
%token <string> SCAN
%token <string> STRING
%token <string> NUMBER
%token <string> CONVERT
%token <string> COMMENT
%token <string> ANYTHING
%token <string> REGEXP
%token <string> '+'
%token <string> '-'
%token <string> '*'
%token <string> '/'
%token <character> ':'
%token <character> '.'

%type <expr> expr
%type <expr> term
%type <expr> factor

%%

decllist    : unitdecl				
	    | decllist unitdecl

unitdecl    : unitName			
	    | unitAlias
	    | unitPrint
	    | unitConvert
	    | unitRegexp
	    | comment
	    | emptyLine
	    ;

comment	    : COMMENT '\n'			{ }
	    ;
emptyLine   : '\n'
	    ;
unitName    : IDENT '\n'		 	{ AmAddUnitName( $1); }
	    ;

unitAlias   : IDENT ':' IDENT '\n'		{ AmAddUnitAlias( $1, $3); }
	    ;

unitPrint   : IDENT '.' PRECISION ':' NUMBER '\n'	{ AmAddPrecision( $1, atoi( $5)); }
	    | IDENT '.' PRINT ':' IDENT '\n'	{ AmAddPrint( $1, $5); }
	    | IDENT '.' SCAN ':' IDENT '\n' 	{ AmAddScan( $1, $5); }
	    | IDENT '.' STRING ':' '\n' 	{ AmAddString( $1); }
	    | IDENT '.' STRING ':' IDENT '\n'  	{ AmAddString( $1, $5); }
	    ;

unitConvert : IDENT CONVERT IDENT ':' expr '\n' { AmAddConvert( $1, $3, $5); }
	    ;

unitRegexp  : IDENT '.' REGEXP ':' ANYTHING '\n'	{ AmAddRegexp( $1, $5); }

expr	    : term				{ $$ = $1 }
	    | expr '+' term			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | expr '-' term			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    ;

term	    : factor				{ $$ = $1; }
	    | term '*' factor			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    | term '/' factor			{ $$ = AmMakeBinexpr( $2, $1, $3); }
	    ; 

factor	    : IDENT						{ $$ = AmMakeIdentifier( $1); }
	    | NUMBER						{ $$ = AmMakeNumber( $1); }
	    | '(' expr ')'					{ $$ = $2; }
	    | IDENT '(' expr ')'				{ $$ = AmMakeFunexpr( $1, $3); }
	    | IDENT '(' expr ',' expr ')'			{ $$ = AmMakeFunexpr2( $1, $3, $5); }
	    | IDENT '(' expr ',' expr ',' expr ')'		{ $$ = AmMakeFunexpr3( $1, $3, $5, $7); }
	    | IDENT '(' expr ',' expr ',' expr ',' expr ')'	{ $$ = AmMakeFunexpr4( $1, $3, $5, $7, $9); }
	    ;

%%
