#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/Scale.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/DrawingA.h>
#include <Xm/Frame.h>

#include <anvil/record.h>
#include <anvil/motif.h>
#include <anvil/xdisplay.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/macros.h>
#include <anvil/files.h>
#include <anvil/macros.h>
#include <anvil/fns.h>
#include <anvil/plot.h>
#include <anvil/resource.h>
#include <anvil/textcheck.h>
#include <anvil/userevents.h>

#include <anvil/choppath.h>
#include <anvil/slick.h>
#include <anvil/option.h>
#include <anvil/utility.h>
#include <anvil/xpm.h>
#include <anvil/visual.h>
#include <anvil/xcolor.h>
#include <anvil/colorcell.h>
#include <anvil/dither.h>
#include <anvil/colorsel.h>

/* Some inline functions used throughout the code. */

#define RANGE( value, min, max)                        \
    if ( value < min)                                  \
        value = min;                                   \
    else if ( value > max)                             \
        value = max

#define MAX3( r, g, b)                                 \
    ((r >= g) ? ((r >= b) ? r : b) : (g >= b) ? g : b)

#define MIN3( r, g, b)                                 \
    ((r <= g) ? ((r <= b) ? r : b) : (g <= b) ? g : b)


/* Constants used throughout the code. */

#define SHOW_NUM_COLORS	256
#define SHOW_WIDTH 	20
#define SHOW_HEIGHT 	20

#define CS_RED          0
#define CS_GREEN        1
#define CS_BLUE         2
#define CS_HUE          3
#define CS_SATURATION   4
#define CS_VALUE        5

#define CS_RGB          0
#define CS_HSV          1


/* Function to convert the chosen color (CS_RED, CS_GREEN, CS_BLUE)
   to the offset in each pixel. */

#define CS_RGB_TO_OFFSET( value)                        \
    (( value == CS_RED ) ? RED :                        \
        ( value == CS_GREEN) ? GREEN : BLUE           )


/* Define the action area on the bottom of the dialog. */

static AmActionDefRec actionAreaDef[] = {
    { "okBtn", NULL, NULL },
    { "revertBtn", NULL, NULL },
    { "cancelBtn", ( XtCallbackProc) AmPopdownCback, NULL },
    { "helpBtn", ( XtCallbackProc) AmHelpCback, NULL },
};


typedef struct colorSelectRec_ {
    AmXDisplayClass *display;
    AmXGCClass      *gc;
    AmXImageClass   *imageXY;
    XImage          *xImageXY;
    AmXWindowClass  *windowXY;
    AmXImageClass   *imageZ;
    XImage          *xImageZ;
    AmXWindowClass  *windowZ;
    AmColorCellRec  color;
    AmColorCellRec  original;
    AmColorCellPtr  cell;
    int             colorMode;
    int             rgbZValue;
    int             hsvZValue;
    int             originalValues[ 3];
    int             values[ 6];
    int             xValue;
    int             yValue;
    int             zValue;
    Widget          shell;
    Widget          form;
    Widget          colorXY;
    Widget          colorZ;
    Widget          colorArea;
    Widget          colorModel;
    Widget          rgbConstant;
    Widget          rgbFrame;
    Widget          redScale;
    Widget          greenScale;
    Widget          blueScale;
    Widget          hsvConstant;
    Widget          hsvFrame;
    Widget          hueScale;
    Widget          saturationScale;
    Widget          valueScale;
    AnvilBoolean         flag;
} colorSelectRec, *colorSelectPtr;

typedef void ( *drawFn)( colorSelectPtr, int);
typedef void ( *updateXYFn)( colorSelectPtr, int, int);
typedef void ( *updateZFn)( colorSelectPtr, int);

typedef struct updateDefRec_ {
    int        x;
    int        y;
    int        z;
    drawFn     drawXY;
    drawFn     drawZ;
    updateXYFn updateXY; 
    updateZFn  updateZ; 
} updateDefRec, *updateDefPtr;

static void drawRowRGBXY( colorSelectPtr, int); 
static void drawRowRGBZ( colorSelectPtr, int); 
static void updateRGBXY( colorSelectPtr, int, int); 
static void updateRGBZ( colorSelectPtr, int); 

static void drawRowValueSaturation( colorSelectPtr, int);
static void drawRowHue( colorSelectPtr, int);
static void drawRowValueHue( colorSelectPtr, int);
static void drawRowSaturation( colorSelectPtr, int);
static void drawRowSaturationHue( colorSelectPtr, int);
static void drawRowValue( colorSelectPtr, int);
static void updateValueSaturation( colorSelectPtr, int, int); 
static void updateHue( colorSelectPtr, int); 
static void updateValueHue( colorSelectPtr, int, int); 
static void updateHSVZ( colorSelectPtr, int); 
static void updateSaturationHue( colorSelectPtr, int, int); 

static void setHSVFromRGB( colorSelectPtr csd);
static void setRGBFromHSV( colorSelectPtr csd);
static void setRGBScales( colorSelectPtr csd);
static void setHSVScales( colorSelectPtr csd);

static updateDefRec updateDefs[] = {
    { CS_BLUE, CS_GREEN, CS_RED, drawRowRGBXY, drawRowRGBZ,
        updateRGBXY, updateRGBZ },
    { CS_BLUE, CS_RED, CS_GREEN, drawRowRGBXY, drawRowRGBZ,
        updateRGBXY, updateRGBZ },
    { CS_GREEN, CS_RED, CS_BLUE, drawRowRGBXY, drawRowRGBZ,
        updateRGBXY, updateRGBZ },
    { CS_VALUE, CS_SATURATION, CS_HUE, drawRowValueSaturation, drawRowHue,
        updateValueSaturation, updateHue },
    { CS_VALUE, CS_HUE, CS_SATURATION, drawRowValueHue, drawRowSaturation,
        updateValueHue, updateHSVZ },
    { CS_SATURATION, CS_HUE, CS_VALUE, drawRowSaturationHue, drawRowValue,
        updateSaturationHue, updateHSVZ },
};

/********************************************************************
 *
 * Code Section:
 *
 ********************************************************************/

/*
 * Function: intersectRectangles( pRect1, pRect2)
 *
 * Purpose:
 *     Calculates the resulting rectangle from the intersection
 *     of two rectangles.
 *
 * Parameters:
 *     AmRectanglePtr pRect1 : First of the two rectangles to intersect.
 *     AmRectanglePtr pRect2 : Second of the two rectangles to intersect.
 *
 * Returns:
 *     AmRectanglePtr : Pointer to a static rectangle that holds the
 *                      result of the intersection.
 *
 * Comments:
 *     The returned rectangle is a pointer to a static variable.  This
 *     means that consecutive calls to intersectRectangle will overwrite
 *     the previous value.  This might have some unwanted side effects so
 *     be careful.
 *
 * Modifications:
 *     Don't we have an intersection function somewhere else? 
 */

static AmRectanglePtr
intersectRectangles( AmRectanglePtr pRect1, AmRectanglePtr pRect2)
{
    int                 p1x1, p1y1, p1x2, p1y2, p2x1, p2y1, p2x2, p2y2;
    static AmRectangleRec result;

/* Degenerate cases (if one or both rectangles are NULL). */

    if ( pRect1 == NULL && pRect2 == NULL) {
        return( NULL);
    }

    if ( pRect1 == NULL) {
        result = *pRect2;
        return( &result);
    } else if ( pRect2 == NULL) {
        result = *pRect1;
        return( &result);
    }

/* Figure out the eight points necessary. */

    p1x1 = pRect1->x;
    p1x2 = pRect1->x + pRect1->width;
    p1y1 = pRect1->y;
    p1y2 = pRect1->y + pRect1->height;
    p2x1 = pRect2->x;
    p2x2 = pRect2->x + pRect2->width;
    p2y1 = pRect2->y;
    p2y2 = pRect2->y + pRect2->height;

/* Trivial reject (no intersection). */

    if ( p1x1 > p2x2 || p1x2 < p2x1 || p1y1 > p2y2 || p1y2 < p2y1) {
        return( NULL);
    }

/* Calculate intersection points. */

    result.x = ( p1x1 > p2x1) ? p1x1 : p2x1;
    result.width = (( p1x2 < p2x2) ? p1x2 : p2x2) - result.x;
    result.y = ( p1y1 > p2y1) ? p1y1 : p2y1;
    result.height = (( p1y2 < p2y2) ? p1y2 : p2y2) - result.y;

    return( &result);
}


/*
 * Function: createImages( csd)
 *
 * Purpose:
 *     Creates the X images used to display the color choices.
 *     
 *     +----------------------+ +---+
 *     |                      | |   |
 *     | XY Image             | | Z |
 *     |                      | |   |
 *     |                      | | i |
 *     |                      | | m |
 *     |                      | | a |
 *     |                      | | g |
 *     |                      | | e |
 *     |                      | |   |
 *     |                      | |   |
 *     |                      | |   |
 *     |                      | |   |
 *     +----------------------+ +---+
 * 
 *     The XY image lets the user select along two variables (one along
 *     the x axis and one along the y axis).  The Z image lets the user
 *     select along one variable (along the y axis).  The variable in the
 *     Z image is also refered to as the constant variable.
 *
 * Parameters:    
 *     colorSelectPtr csd : Client data (color select data) that contains 
 *                          all the widgets, and other data of the color
 *                          select dialog.
 */

static void
createImages( colorSelectPtr csd) 
{

/* Create the display and GC needed to create and draw images. */

    csd->display = new AmXDisplayClass( csd->colorXY);

/* Create the image for the colors. */

    csd->imageXY = new AmXImageClass( csd->display,
                                      SHOW_NUM_COLORS - 1, SHOW_NUM_COLORS - 1,
                                      csd->display->GetDepth(),
                                      NULL);

    csd->xImageXY = csd->imageXY->GetXImage();
    csd->windowXY = new AmXWindowClass( csd->display, csd->colorXY);

    csd->imageZ = new AmXImageClass( csd->display,
                                     SHOW_WIDTH - 1, SHOW_NUM_COLORS - 1,
                                     csd->display->GetDepth(),
                                     NULL);

    csd->xImageZ = csd->imageZ->GetXImage();
    csd->windowZ = new AmXWindowClass( csd->display, csd->colorZ);
    csd->gc = ( AmXGCClass *) csd->windowZ->CreateGC();
}


/*
 * Function: putImage( csd, image, window, exposeRect)
 *
 * Purpose:
 *     Places an X image into a window, thereby displaying it.  Only
 *     those portions intersecting with the expose rectangle are displayed. 
 *
 * Paremters:
 *     colorSelectPtr csd        : Client data contains all the widgets, and
 *                                 other data of the color select dialog.
 *     AmImageClass   *image     : Image to display.
 *     AmWindowClass  *window    : Window to display it on. 
 *     AmRectanglePtr exposeRect : Rectangle to clip the image by. 
 */

static void
putImage( colorSelectPtr csd,
          AmImageClass     *image,
          AmWindowClass    *window,
          AmRectanglePtr   exposeRect)
{
    AmRectangleRec imageRect;
    AmRectanglePtr drawRect; 

/* Check for intersection. */

    imageRect.x = 0;
    imageRect.y = 0;
    imageRect.width = image->GetWidth();
    imageRect.height = image->GetHeight();


    drawRect = intersectRectangles( &imageRect, exposeRect);

/* Display a portion of the image. */

    if ( drawRect != NULL) {
        csd->display->DrawImage( window, csd->gc, drawRect->x, drawRect->y,
                                 drawRect->x, drawRect->y, 
                                 drawRect->width, drawRect->height,
                                 image);
    }
}


/*
 * Function: AmXPosXY( csd)
 *
 * Purpose:
 *     Returns the X position of the color chooser in the XY image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

int
AmXPosXY( colorSelectPtr csd)
{

/* The HUE value is scaled up, so we need to scale it down to translate
   it into the x position (0 to 256). */

    if ( csd->colorMode == CS_HSV && csd->xValue == CS_HUE) {
        return( csd->values[ CS_HUE] / 359);
    }

    return( csd->values[ csd->xValue]);
}


/*
 * Function: AmYPosXY( csd)
 *
 * Purpose:
 *     Returns the Y position of the color chooser in the XY image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

int
AmYPosXY( colorSelectPtr csd)
{

/* The HUE value is scaled up, so we need to scale it down to translate
   it into the x position (0 to 256). */

    if ( csd->colorMode == CS_HSV && csd->yValue == CS_HUE) {
        return( csd->xImageXY->height - csd->values[ CS_HUE] / 359 - 1);
    }

    return( csd->xImageXY->height - csd->values[ csd->yValue] - 1);
}


/*
 * Function: AmYPosZ( csd)
 *
 * Purpose:
 *     Returns the Y position of the color chooser in the Z image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

int
AmYPosZ( colorSelectPtr csd)
{

/* The HUE value is scaled up, so we need to scale it down to translate
   it into the x position (0 to 256). */

    if ( csd->colorMode == CS_HSV && csd->zValue == CS_HUE) {
        return( csd->xImageZ->height - (csd->values[ CS_HUE] / 359) - 1);
    }

    return( csd->xImageZ->height - csd->values[ csd->zValue] - 1);
}


/*
 * Function: drawXYPointer( csd)
 *
 * Purpose:
 *     Draws the crosshair in the XY image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
drawXYPointer( colorSelectPtr csd)
{
    AmRasterOpEnum  rasterOp;
    AmPointRec      points[ 2];

/* Save the old raster operation. */

    rasterOp = csd->gc->GetRasterOp();

/* Switch to invert. */

    csd->gc->SetRasterOp( AmINVERT);

/* Figure out the vertical line... */

    points[ 0].x = AmXPosXY( csd);
    points[ 0].y = 0;
    points[ 1].x = AmXPosXY( csd); 
    points[ 1].y = csd->xImageXY->height;

/* ... and draw it. */ 

    csd->display->DrawLines( csd->windowXY, csd->gc, points, 2); 

/* Figure out the horizontal line... */

    points[ 0].x = 0;
    points[ 0].y = AmYPosXY( csd);
    points[ 1].x = csd->xImageXY->width;
    points[ 1].y = AmYPosXY( csd);

/* ... and draw it. */ 

    csd->display->DrawLines( csd->windowXY, csd->gc, points, 2); 

/* Restore the old raster operation. */

    csd->gc->SetRasterOp( rasterOp);
}


/*
 * Function: drawZPointer( csd)
 *
 * Purpose:
 *     Draws the line in the Z image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
drawZPointer( colorSelectPtr csd)
{
    AmRasterOpEnum  	rasterOp;
    AmPointRec 		points[ 2];

/* Save the old raster operation. */

    rasterOp = csd->gc->GetRasterOp();

/* Switch to invert. */

    csd->gc->SetRasterOp( AmINVERT);

/* Figure out the horizontal line... */

    points[ 0].x = 0;
    points[ 0].y = AmYPosZ( csd);
    points[ 1].x = csd->xImageZ->width;
    points[ 1].y = AmYPosZ( csd);

/* ... and draw it. */

    csd->display->DrawLines( csd->windowZ, csd->gc, points, 2); 

/* Restore the old raster operation. */

    csd->gc->SetRasterOp( rasterOp);
}


/*
 * Function: drawRowRGBXY( csd, y)
 *
 * Purpose:
 *     Draws a row of the XY image using the RGB representation.
 *     Note that the csd data structure contains the information
 *     as to which two values are displayed in the XY image and
 *     which value is displayed in the Z imagexValue.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowRGBXY( colorSelectPtr csd, int y)
{
    int     x, width, z, xVal, yVal, zVal;
    char    *p;

/* Get the extents of the image. */

    width = csd->xImageXY->width;

/* Figure out which RGB component is the x, y, and z value. */

    xVal = CS_RGB_TO_OFFSET( csd->xValue);
    yVal = CS_RGB_TO_OFFSET( csd->yValue);
    zVal = CS_RGB_TO_OFFSET( csd->zValue);

/* Calculate the fixed components of the row (the y and the z value). */

    y = 255 - y;
    z = csd->values[ csd->zValue];

/* Figure out the memory location of the start of the row. */

    p = csd->imageXY->AmGetData() + csd->imageXY->GetBytesPerLine() * y;

/* Fill the row. */

    for ( x = 0; x < width; x++, p += 4) {
        p[ xVal] = x;
        p[ yVal] = y;
        p[ zVal] = z;
    }
}


/*
 * Function: drawRowRGBXY( csd, y)
 *
 * Purpose:
 *     Draws a row of the Z image using the RGB representation.
 *     Note that the csd data structure contains the information
 *     as to which two values are displayed in the XY image and
 *     which value is displayed in the Z imagexValue.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowRGBZ( colorSelectPtr csd, int row)
{
    int     ii, xVal, yVal, zVal, x, y, z, width;
    char    *p;

/* Get the extents of the image. */

    width = csd->xImageZ->width;

/* Figure out which RGB component is the x, y, and z value. */

    xVal = CS_RGB_TO_OFFSET( csd->xValue);
    yVal = CS_RGB_TO_OFFSET( csd->yValue);
    zVal = CS_RGB_TO_OFFSET( csd->zValue);

/* Calculate the RGB components of the row. */

    z = 255 - row;
    y = csd->values[ csd->yValue];
    x = csd->values[ csd->xValue];

/* Figure out the memory location of the start of the row. */

    p = csd->imageZ->AmGetData() + csd->imageZ->GetBytesPerLine() * row;

/* Fill the row. */

    for ( ii = 0; ii < width; ii++, p += 4) {
        p[ xVal] = x;
        p[ yVal] = y;
        p[ zVal] = z;
    }
}


/*
 * Function: updateRGBXY( csd, x, y)
 *
 * Purpose:
 *     Updates the crosshair in the XY image and updates the
 *     RGB scales.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            x   : The x position of the crosshair. 
 *     int            y   : The y position of the corsshair. 
 */

static void
updateRGBXY( colorSelectPtr csd, int x, int y)
{

/* Make sure x and y are withing the 0 and 255 range. */

    RANGE( x, 0, 255);
    RANGE( y, 0, 255);

/* Undraw the crosshair. */

    drawXYPointer( csd);

/* Figure out the new x and y values. */

    csd->values[ csd->xValue] = x;
    csd->values[ csd->yValue] = y;

/* Redraw the crosshair. */

    drawXYPointer( csd);

/* Update the RGB scale widgets to represnt the new values. */

    setRGBScales( csd);
}


/*
 * Function: updateRGBZ( csd, y)
 *
 * Purpose:
 *     Updates the line cursor in the Z image and updates the RGB scales.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : The y position of the line. 
 */

static void
updateRGBZ( colorSelectPtr csd, int y)
{

/* Make sure y is within the 0 to 255 range. */

    RANGE( y, 0, 255);

/* Undraw the line in the Z image, update the y value, and redraw
   the line in the Z image. */

    drawZPointer( csd);
    csd->values[ csd->zValue] = y;
    drawZPointer( csd);

/* Update the scale widgets to represent the new value. */

    setRGBScales( csd);

}


/*
 * Function: drawRowValueSaturation( csd, y)
 *
 * Purpose:
 *     Draws a row of the XY image using the HSV representation
 *     with value on the x axis and saturaion on the y axis.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowValueSaturation( colorSelectPtr csd, int y)
{
    int   width;
    int   h, s, f, v;
    float p, q, t;
    char *ptr;

/* Get the extents of the image. */

    width = csd->xImageXY->width;

/*
 * Hue, saturation, and value are all scaled integers the scaling
 *   used is 255.
 */

/* Figure out what hextant is to be displayed. */

    h = csd->values[ CS_HUE] / 60;
    f = h - h / 255 * 255;
    s = 255 - y;

    p = 255 - s;
    q = 255 - ((s * f) / 255); 
    t = 255 - ((s * (255 - f)) / 255);

    p /= 255;
    q /= 255;
    t /= 255;

/* Figure out the memory location of the start of the row. */

    ptr = csd->imageXY->AmGetData() + csd->imageXY->GetBytesPerLine() * y;

    switch( h / 255) {
        case 0:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = ( char) v; 
                ptr[ GREEN] =  ( char)( v * t);
                ptr[ BLUE] = ( char)( v * p);
            }
            break;
        case 1:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = ( char)( v * q);
                ptr[ GREEN] = ( char)v;
                ptr[ BLUE] = ( char)( v * p);
            }
            break;
        case 2:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * p);
                ptr[ GREEN] = v;
                ptr[ BLUE] = (char)(v * t);
            }
            break;
        case 3:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * p);
                ptr[ GREEN] = (char)(v * q);
                ptr[ BLUE] = v; 
            }
            break;
        case 4:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * t);
                ptr[ GREEN] = (char)(v * p);
                ptr[ BLUE] = v;
            }
            break;
        case 5:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = v;
                ptr[ GREEN] = (char)(v * p);
                ptr[ BLUE] = (char)(v * q);
            }
            break;
    }
}


/*
 * Function: drawRowHue( csd, y)
 *
 * Purpose:
 *     Draws a row of the Z image using the HSV representation
 *     with value on the y axis being hue. 
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowHue( colorSelectPtr csd, int y)
{
    int  ii, width, height;
    int  h, s, f, p, q, t, v;
    char *ptr;

/* Get the extents of the image. */

    width = csd->xImageZ->width;
    height = csd->xImageZ->height;

/* Don't do anything if we are out of the image. */

    if ( y < 0 || y > height - 1) {
        return;
    }

/*
 * Hue, saturation, and value are all scaled integers the scaling
 *   used is 255.
 */

/* Figure out what hextant is to be displayed. */

    h = (255 - y) * 359 / 60;
    f = h - h / 255 * 255;

    s = 255;
    v = 255;

    p = 255 - s;
    q = 255 - ((s * f) / 255);
    t = 255 - ((s * (255 - f)) / 255);

/* Figure out the memory location of the start of the row. */

    ptr = csd->imageZ->AmGetData() + csd->imageZ->GetBytesPerLine() * y;

/* Depending on the hextant convert the HSV values to RGB values and
   fill in the row. */

    switch( h / 255) {
        case 0:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                ptr[ RED] = v; 
                ptr[ GREEN] = (v * t) / 255;
                ptr[ BLUE] = (v * p) / 255;
            }
            break;
        case 1:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                ptr[ RED] = (v * q) / 255;
                ptr[ GREEN] = v;
                ptr[ BLUE] = (v * p) / 255;
            }
            break;
        case 2:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                ptr[ RED] = (v * p) / 255;
                ptr[ GREEN] = v;
                ptr[ BLUE] = (v * t) / 255;
            }
            break;
        case 3:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                ptr[ RED] = (v * p) / 255;
                ptr[ GREEN] = (v * q) / 255;
                ptr[ BLUE] = v; 
            }
            break;
        case 4:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                ptr[ RED] = (v * t) / 255;
                ptr[ GREEN] = (v * p) / 255;
                ptr[ BLUE] = v;
            }
            break;
        case 5:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                ptr[ RED] = v;
                ptr[ GREEN] = (v * p) / 255;
                ptr[ BLUE] = (v * q) / 255;
            }
            break;
    }
}


/*
 * Function: drawRowValueHue( csd, y)
 *
 * Purpose:
 *     Draws a row of the XY image using the HSV representation
 *     with value on the x axis and hue on the y axis.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowValueHue( colorSelectPtr csd, int y)
{
    int   width;
    int   h, s, f, v;
    float p, q, t;
    char *ptr;

/* Get the extents of the image. */

    width = csd->xImageXY->width;

    h = (255 - y) * 359 / 60;
    f = h - h / 255 * 255;

    s = csd->values[ CS_SATURATION];

/* Figure out the memory location of the start of the row. */

    ptr = csd->imageXY->AmGetData() + csd->imageXY->GetBytesPerLine() * y;

    p = 255 - s;
    q = 255 - ((s * f) / 255); 
    t = 255 - ((s * (255 - f)) / 255);

    p /= 255;
    q /= 255;
    t /= 255;

/* Depending on the hextant convert the HSV values to RGB values and
   fill in the row. */

    switch( h / 255) {
        case 0:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = v; 
                ptr[ GREEN] = (char)(v * t);
                ptr[ BLUE] = (char)(v * p);
            }
            break;
        case 1:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * q);
                ptr[ GREEN] = v;
                ptr[ BLUE] = (char)(v * p);
            }
            break;
        case 2:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * p);
                ptr[ GREEN] = v;
                ptr[ BLUE] = (char)(v * t);
            }
            break;
        case 3:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * p);
                ptr[ GREEN] = (char)(v * q);
                ptr[ BLUE] = v; 
            }
            break;
        case 4:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = (char)(v * t);
                ptr[ GREEN] = (char)(v * p);
                ptr[ BLUE] = v;
            }
            break;
        case 5:
            for ( v = 0; v < width; v++, ptr += 4) {
                ptr[ RED] = v;
                ptr[ GREEN] = (char)(v * p);
                ptr[ BLUE] = (char)(v * q);
            }
            break;
    }
}
    

/*
 * Function: drawRowSaturation( csd, y)
 *
 * Purpose:
 *     Draws a row of the Z image using the HSV representation
 *     with value on the y axis being saturation. 
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowSaturation( colorSelectPtr csd, int y)
{
    int  ii, width;
    int  s;
    char *ptr;

/* Get the extents of the image. */

    width = csd->xImageZ->width;

/*
 * Hue, saturation, and value are all scaled integers the scaling
 *   used is 256 ( or 1 << 8 ).
 */


/* Figure out what hextant is to be displayed. */

    s = 255 - y;

/* Figure out the memory location of the start of the row. */

    ptr = csd->imageZ->AmGetData() + csd->imageZ->GetBytesPerLine() * y;

    for ( ii = 0; ii < width; ii++, ptr += 4) {
        ptr[ RED] = s;
        ptr[ GREEN] = s;
        ptr[ BLUE] = s;
    }
}


/*
 * Function: drawRowSaturationHue( csd, y)
 *
 * Purpose:
 *     Draws a row of the XY image using the HSV representation
 *     with saturation on the x axis and hue on the y axis.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */


static void
drawRowSaturationHue( colorSelectPtr csd, int y)
{
    int   ii, width;
    int   h, s, f, v;
    float p, q, t;
    char *ptr;

/* Get the extents of the image. */

    width = csd->xImageXY->width;

/* Figure out what hextant is to be displayed. */

    h = (255 - y) * 359 / 60;
    f = h - h / 255 * 255;

    v = csd->values[ CS_VALUE];

    p = v;
    q = (v * f) / 255;
    t = v - ((v * f) / 255);

    p /= 255;
    q /= 255;
    t /= 255;

/* Figure out the memory location of the start of the row. */

    ptr = csd->imageXY->AmGetData() + csd->imageXY->GetBytesPerLine() * y;

/* Depending on the hextant convert the HSV values to RGB values and
   fill in the row. */

    switch( h / 255) {
        case 0:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                s = ii;
                ptr[ RED] = v; 
                ptr[ GREEN] = v - (char)(s * t);
                ptr[ BLUE] = v - (char)(s * p);
            }
            break;
        case 1:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                s = ii;
                ptr[ RED] = v - (char)(s * q);
                ptr[ GREEN] = v;
                ptr[ BLUE] = v - (char)(s * p);
            }
            break;
        case 2:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                s = ii;
                ptr[ RED] = v - (char)(s * p);
                ptr[ GREEN] = v;
                ptr[ BLUE] = v - (char)(s * t);
            }
            break;
        case 3:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                s = ii;
                ptr[ RED] = v - (char)(s * p);
                ptr[ GREEN] = v - (char)(s * q);
                ptr[ BLUE] = v; 
            }
            break;
        case 4:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                s = ii;
                ptr[ RED] = v - (char)(s * t);
                ptr[ GREEN] = v - (char)(s * p);
                ptr[ BLUE] = v;
            }
            break;
        case 5:
            for ( ii = 0; ii < width; ii++, ptr += 4) {
                s = ii;
                ptr[ RED] = v;
                ptr[ GREEN] = v - (char)(s * p);
                ptr[ BLUE] = v - (char)(s * q);
            }
            break;
    }
}


/*
 * Function: drawRowValue( csd, y)
 *
 * Purpose:
 *     Draws a row of the Z image using the HSV representation
 *     with value on the y axis being value. 
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : Which row to draw.
 */

static void
drawRowValue( colorSelectPtr csd, int y)
{
    int  ii, width, height;
    int  v;
    char *ptr;

/* Get the extents of the image. */

    width = csd->xImageZ->width;
    height = csd->xImageZ->height;

/* Don't do anything if we are out of the image. */

    if ( y < 0 || y > height - 1) {
        return;
    }

/* Get the value that is used to fill the row. */

    v = 255 - y;

/* Figure out the memory location of the start of the row. */

    ptr = csd->imageZ->AmGetData() + csd->imageZ->GetBytesPerLine() * y;

/* Fill the row. */

    for ( ii = 0; ii < width; ii++, ptr += 4) {
        ptr[ RED] = v; 
        ptr[ GREEN] = v;
        ptr[ BLUE] = v;
    }
}


/*
 * Function: updateHue( csd, y)
 *
 * Purpose:
 *     Updates the hue component of the HSV color.  Also updates, the
 *     line cursor in the Z image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : The new value for the hue componenet of the HSV
 *                          color.
 */

static void
updateHue( colorSelectPtr csd, int y)
{

/* Make sure y is within 0 to 255. */

    RANGE( y, 0, 255);

/* Undraw the line in the Z image, update the hue value, and redraw the
   line cursor. */

    drawZPointer( csd);
    csd->values[ CS_HUE] = y * 359;
    drawZPointer( csd);

/* Set the HSV widget scales to represent the new values. */ 

    setHSVScales( csd);
}


/*
 * Function: updateValueSaturation( csd, x, y)
 *
 * Purpose:
 *     Updates the value and saturation components of the HSV color.
 *     Also updates, the crosshair cursor in the XY image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            x   : The new value for the value componenet of the HSV
 *                          color.
 *     int            y   : The new value for the saturation componenet of
 *                          the HSV color.
 */

static void
updateValueSaturation( colorSelectPtr csd, int x, int y)
{

/* Make sure the x and y value are within the 0 to 255 range. */

    RANGE( x, 0, 255);
    RANGE( y, 0, 255);

/* Undraw the crosshair, update the x and y values, and redraw the
   crosshair cursor. */

    drawXYPointer( csd);

    csd->values[ CS_VALUE] = x;
    csd->values[ CS_SATURATION] = y;

    drawXYPointer( csd);

/* Update the HSV widget scales to represent the new values. */

    setHSVScales( csd);
}


/*
 * Function: updateValueHue( csd, x, y)
 *
 * Purpose:
 *     Updates the value and hue components of the HSV color.
 *     Also updates, the crosshair cursor in the XY image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            x   : The new value for the value componenet of the HSV
 *                          color.
 *     int            y   : The new value for the hue componenet of the HSV
 *                          color.
 */

static void
updateValueHue( colorSelectPtr csd, int x, int y)
{

/* Make sure the x and y value are within the 0 to 255 range. */

    RANGE( x, 0, 255);
    RANGE( y, 0, 255);

/* Undraw the crosshair, update the x and y values, and redraw the
   crosshair cursor. */

    drawXYPointer( csd);

    csd->values[ CS_VALUE] = x;
    csd->values[ CS_HUE] = y * 359;

    drawXYPointer( csd);

/* Update the HSV widget scales to represent the new values. */

    setHSVScales( csd);
}


/*
 * Function: updateHSVZ( csd, y)
 *
 * Purpose:
 *     Updates either the saturation or value component of the HSV color
 *     (depending on which is currently displayed in the Z image).  Also
 *     updates, the line cursor in the Z image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            y   : The new value for the componenet of the HSV
 *                          color.
 */

static void
updateHSVZ( colorSelectPtr csd, int y)
{

/* Make sure the y value is within the 0 to 255 range. */

    RANGE( y, 0, 255);

/* Undraw the line in the Z image, update either the saturation or value
   value (depending on zValue), and redraw the line cursor. */

    drawZPointer( csd);

    csd->values[ csd->zValue] = y;

    drawZPointer( csd);

/* Set the HSV widget scales to represent the new values. */ 

    setHSVScales( csd);
}


/*
 * Function: updateSaturationHue( csd, x, y)
 *
 * Purpose:
 *     Updates the saturation and hue components of the HSV color.
 *     Also updates, the crosshair cursor in the XY image.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     int            x   : The new value for the saturation componenet of
 *                          the HSV color.
 *     int            y   : The new value for the hue componenet of the HSV
 *                          color.
 */

static void
updateSaturationHue( colorSelectPtr csd, int x, int y)
{

/* Make sure the x and y value are within the 0 to 255 range. */

    RANGE( x, 0, 255);
    RANGE( y, 0, 255);

/* Undraw the crosshair, update the x and y values, and redraw the
   crosshair cursor. */

    drawXYPointer( csd);

    csd->values[ CS_SATURATION] = x;
    csd->values[ CS_HUE] = y * 359;

    drawXYPointer( csd);

/* Update the HSV widget scales to represent the new values. */

    setHSVScales( csd);
}


/*
 * Function: updateColors( csd)
 *
 * Purpose:
 *     Updates the RGB color values of the picked color.  Also updates
 *     the color preview area.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
updateColors( colorSelectPtr csd)
{

/* If we are currently picking a color in the HSV space convert it
   to the RGB color space. */

    if ( csd->colorMode == CS_HSV) {
        setRGBFromHSV( csd);
    }

/* Get the three RGB components for the color cell. */ 

    csd->color.red = csd->values[ CS_RED];
    csd->color.green = csd->values[ CS_GREEN];
    csd->color.blue = csd->values[ CS_BLUE];

/* Convert the RGB Values to a pixel value (needed specifically for
   non true color modes. */

    AmSetPixelValue( &(csd->color));

/* Show the color in the color preview area. */

    XtVaSetValues( csd->colorArea, XmNbackground, csd->color.pixel, NULL);
}


/*
 * Function: setRGBfromHSV( csd)
 *
 * Purpose:
 *     Converts HSV values into RGB values. 
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
setRGBFromHSV( colorSelectPtr csd)
{
    float h, s, f, p, q, t, v;

/* Figure out which hextant the color is in. */

    h = csd->values[ CS_HUE] / 255.0 / 60.0;

/* Cacluate some intermediate values. */

    f = h - ( int) h;

    v = csd->values[ CS_VALUE] / 255.0;
    s = csd->values[ CS_SATURATION]/ 255.0;

    p = v;
    q = v * f;
    t = v - (v * f);

/* Depending on the hextant convert the HSV values into RGB values. */

    switch( ( int) h ) {
        case 0:
            csd->values[ CS_RED] = (int)(v * 255);
            csd->values[ CS_GREEN] = (int)( v - (int)(s * t)) * 255;
            csd->values[ CS_BLUE] = (int)( v - (int)(s * p)) * 255;
            break;
        case 1:
            csd->values[ CS_RED] = (int)( v - (int)(s * q)) * 255;
            csd->values[ CS_GREEN] = (int)(v * 255);
            csd->values[ CS_BLUE] = (int)( v - (int)(s * p)) * 255;
            break;
        case 2:
            csd->values[ CS_RED] = (int)( v - (int)(s * p)) * 255;
            csd->values[ CS_GREEN] = (int)(v * 255);
            csd->values[ CS_BLUE] = (int)( v - (int)(s * t)) * 255;
            break;
        case 3:
            csd->values[ CS_RED] = (int)( v - (int)(s * p)) * 255;
            csd->values[ CS_GREEN] = (int)( v - (int)(s * q)) * 255;
            csd->values[ CS_BLUE] = (int)(v * 255);
            break;
        case 4:
            csd->values[ CS_RED] = (int)( v - (int)(s * t)) * 255;
            csd->values[ CS_GREEN] = (int)( v - (int)(s * p)) * 255;
            csd->values[ CS_BLUE] = (int)(v * 255);
            break;
        case 5:
            csd->values[ CS_RED] = (int)(v * 255);
            csd->values[ CS_GREEN] = (int)( v - (int)(s * p)) * 255;
            csd->values[ CS_BLUE] = (int)( v - (int)(s * q)) * 255;
            break;
    }
}


/*
 * Function: setRGBScales( csd)
 *
 * Purpose:
 *     Sets the RGB widgets scales to their respective values.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
setRGBScales( colorSelectPtr csd)
{
    XtVaSetValues( csd->redScale, XmNvalue, csd->values[ CS_RED], NULL);
    XtVaSetValues( csd->greenScale, XmNvalue, csd->values[ CS_GREEN], NULL);
    XtVaSetValues( csd->blueScale, XmNvalue, csd->values[ CS_BLUE], NULL);
}


/*
 * Function: setHSVfromRGB( csd)
 *
 * Purpose:
 *     Converts RGB values into HSV values. 
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
setHSVFromRGB( colorSelectPtr csd)
{
    float r, g, b, h, s, v, min, max, delta;

/* Convert the RGB values that are in the range of 0 to 255 into 
   a range of 0.0 to 1.0. */

    r = csd->values[ CS_RED] / 255.0;
    g = csd->values[ CS_GREEN] / 255.0;
    b = csd->values[ CS_BLUE] / 255.0;

/* Get the min and max value. */

    max = MAX3( r, g, b);
    min = MIN3( r, g, b);

/* Do the conversion. */

    v = max;

    if ( max != 0) {
        s = ( max - min) / max;
    } else {
        s = 0;
    }

    if ( s == 0) {
        h = 0;
    } else {
        delta = max - min;
        if ( r == max) {
            h = ( g - b) / delta;
        } else if ( g == max) {
            h = 2 + ( b - r) / delta;
        } else {
            h = 4 + ( r - g) / delta;
        }

        h *= 60;

        if ( h < 0) {
            h += 360;
        }
    }

/* Scale the HSV values from the range 0.0 to 1.0 to the range 0 to 255. */

    csd->values[ CS_HUE] = (int)(h * 255);
    csd->values[ CS_SATURATION] = (int)(s * 255);
    csd->values[ CS_VALUE] = (int)(v * 255);
}


/*
 * Function: setHSVScales( csd)
 *
 * Purpose:
 *     Sets the HSV widgets scales to their respective values.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
setHSVScales( colorSelectPtr csd)
{
    int h, s, v;

    h = (int)(csd->values[ CS_HUE] * 10.0 / 255.0);
    s = (int)(csd->values[ CS_SATURATION] * 100.0 / 255.0);
    v = (int)(csd->values[ CS_VALUE] * 100.0 / 255.0);

    XtVaSetValues( csd->hueScale, XmNvalue, h, NULL);
    XtVaSetValues( csd->saturationScale, XmNvalue, s, NULL);
    XtVaSetValues( csd->valueScale, XmNvalue, v, NULL);
}


/*
 * Function: exposeImageXY( w, csd, cb)
 *
 * Purpose:
 *     Redraws the exposed area of the XY image.
 *
 * Paremters:
 *     Widget         w   : The drawing area widget that contains the image.
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     XmDrawingAreaCallbackStruct : Contains the expose event which gives
 *                                   the dimensions of the eposed area.
 *
 * Comments:
 *     This function gets called when the drawing area containing the
 *     XY image is exposed.
 */

/* ARGSUSED */
static void
exposeImageXY( Widget                      w,
               colorSelectPtr              csd,
               XmDrawingAreaCallbackStruct *cb)
{
    putImage( csd, csd->imageXY, csd->windowXY, NULL);
    drawXYPointer( csd);
}


/*
 * Function: exposeImageZ( w, csd, cb)
 *
 * Purpose:
 *     Redraws the exposed area of the Z image.
 *
 * Paremters:
 *     Widget         w   : The drawing area widget that contains the image.
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 *     XmDrawingAreaCallbackStruct : Contains the expose event which gives
 *                                   the dimensions of the eposed area.
 *
 * Comments:
 *     This function gets called when the drawing area containing the
 *     Z image is exposed.
 */

/* ARGSUSED */
static void
exposeImageZ( Widget                      w,
              colorSelectPtr              csd,
              XmDrawingAreaCallbackStruct *cb)
{
    putImage( csd, csd->imageZ, csd->windowZ, NULL);
    drawZPointer( csd);
}


/*
 * Function: drawColors( csd)
 *
 * Purpose:
 *     Fills the XY and Z images with the correct color gradients.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
drawColors( colorSelectPtr csd) 
{
    int  ii, constant, height;


    constant = csd->zValue;

/* Fill each row of the XY image. */

    height = csd->xImageXY->height;
    for ( ii = 0; ii < height; ii++) {
        updateDefs[ constant].drawXY( csd, ii);
    }

/* Fill each row of the Z image. */

    height = csd->xImageZ->height;
    for ( ii = 0; ii < height; ii++) {
        updateDefs[ constant].drawZ( csd, ii);
    }

/* Copy the internal data into the X images. */

    csd->imageXY->CopyData();
    csd->imageZ->CopyData();

/* Display the two imgaes. */

    putImage( csd, csd->imageXY, csd->windowXY, NULL);
    putImage( csd, csd->imageZ, csd->windowZ, NULL);

/* Draw the selection cursors. */

    drawXYPointer( csd);
    drawZPointer( csd);
}


/*
 * Function: drawColorsXY( csd)
 *
 * Purpose:
 *     Fills the XY image with the correct color gradients.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
drawColorsXY( colorSelectPtr csd) 
{
    int  ii, height;

/* Fill each row of the XY image. */

    height = csd->xImageXY->height;
    for ( ii = 0; ii < height; ii++) {
        updateDefs[ csd->zValue].drawXY( csd, ii);
    }

/* Copy the internal image data to the X image. */

    csd->imageXY->CopyData();

/* Display the image. */

    putImage( csd, csd->imageXY, csd->windowXY, NULL);

/* Draw the selection cursors. */

    drawXYPointer( csd);
}


/*
 * Function: drawColorsZ( csd)
 *
 * Purpose:
 *     Fills the Z image with the correct color gradients.
 *
 * Paremters:
 *     colorSelectPtr csd : Client data that contains all the widgets, and
 *                          other data of the color select dialog.
 */

static void
drawColorsZ( colorSelectPtr csd) 
{
    int  ii, height;

/* Fill each row of the Z image. */

    height = csd->xImageZ->height;
    for ( ii = 0; ii < height; ii++) {
        updateDefs[ csd->zValue].drawZ( csd, ii);
    }

/* Copy the internal image data to the X image. */

    csd->imageZ->CopyData();

/* Display the image. */

    putImage( csd, csd->imageZ, csd->windowZ, NULL);

/* Draw the selection cursors. */

    drawZPointer( csd);
}


/*
 * Function: colorChooseXYCallback( w, csd, event, bool)
 *
 * Purpose:
 *   The colorChooseXYCallback gets called when the user selects a color
 *   in the XY image (by moving the mouse and pressing the left button).
 *   The functions updates the Z image since its' colors are depending on
 *   the selection in the XY image.  It also updates the preview window
 *   and the XY selection cursor.
 *   
 * Paremters:
 *     Widget         w      : The drawing area (not used).
 *     colorSelectPtr csd    : Client data that contains all the widgets, and
 *                             other data of the color select dialog.
 *     XEevent        *event : The X event that caused the callback to be 
 *                             called.
 *     AnvilBoolean        bool   : Wheter or not to pass the event through (not 
 *                             used).
 */

/* ARGSUSED */
static void
colorChooseXYCallback( Widget         w,
                       colorSelectPtr csd,
                       XEvent         *event,
                       AnvilBoolean        bool1)
{
    XButtonEvent *button;
    XMotionEvent *motion;
    int          x, y, ii;

/* Get the current value. */

    ii = csd->zValue;

    if ( event->type == ButtonPress) {
        button = ( XButtonEvent *) event;

    /* If it is not the left mouse button we don't care. */

        if ( button->button != Button1) {
            return;
        }

        x = button->x;
        y = csd->xImageXY->height - button->y;

    /* Update the XY selection cursor, the Z image, and the preview area. */

        updateDefs[ ii].updateXY( csd, x, y);
        drawColorsZ( csd);
        updateColors( csd);

    /* Indicate that the left mouse butotn is pressed. */

        csd->flag = TRUE;

    } else if ( event->type == ButtonRelease) {

    /* The button is no longer being pressed. */

        csd->flag = FALSE;

    } else if ( event->type == MotionNotify) {

    /* If the left mouse button is not being pressed we don't care. */

        if ( csd->flag != TRUE) {
            return;
        }

        motion = ( XMotionEvent *) event;
        x = motion->x; 
        y = csd->xImageXY->height - motion->y;

    /* Update the XY selection cursor, the Z image, and the preview area. */

        updateDefs[ ii].updateXY( csd, x, y);
        drawColorsZ( csd);
        updateColors( csd);
    }
}


/*
 * Function: colorChooseZCallback( w, csd, event, bool)
 *
 * Purpose:
 *   The colorChooseZCallback gets called when the user selects a color
 *   in the Z image (by moving the mouse and pressing the left button).
 *   The functions updates the XY image since its' colors are depending on
 *   the selection in the Z image.  It also updates the preview window
 *   and the Z selection cursor.
 *   
 * Paremters:
 *     Widget         w      : The drawing area (not used).
 *     colorSelectPtr csd    : Client data that contains all the widgets, and
 *                             other data of the color select dialog.
 *     XEevent        *event : The X event that caused the callback to be 
 *                             called.
 *     AnvilBoolean        bool   : Wheter or not to pass the event through (not 
 *                             used).
 */

static void
colorChooseZCallback( Widget         w,
                      colorSelectPtr csd,
                      XEvent         *event,
                      AnvilBoolean        bool1)
{
    XButtonEvent *button;
    XMotionEvent *motion;
    int          y, ii;

/* Get the current value. */

    ii = csd->zValue;

    if ( event->type == ButtonPress) {

    /* If it is not the left mouse button we don't care. */

        button = ( XButtonEvent *) event;
        if ( button->button != Button1) {
            return;
        }

    /* Update the Z selection cursor, the XY image, and the preview area. */

        y = csd->xImageZ->height - button->y;
        updateDefs[ ii].updateZ( csd, y);
        drawColorsXY( csd);
        updateColors( csd);

    /* Indicate that the left mouse butotn is pressed. */

        csd->flag = TRUE;

    } else if ( event->type == ButtonRelease) {

    /* The button is no longer being pressed. */

        csd->flag = FALSE;

    } else if ( event->type == MotionNotify) {

    /* If the left mouse button is not being pressed we don't care. */

        if ( csd->flag != TRUE) {
            return;
        }

        motion = ( XMotionEvent *) event;
        y = csd->xImageZ->height - motion->y;

    /* Update the Z selection cursor, the XY image, and the preview area. */

        updateDefs[ ii].updateZ( csd, y);
        drawColorsXY( csd);
        updateColors( csd);
    }
}


/*
 * Function: colorModelChangeCallback( iID, csd, value) 
 *
 * Purpose:
 *   The colorModelChangeCallback gets called when the color model 
 *   selection option menu is used.  It selects between the RGB and
 *   the HSV color models.  The function updates the user interface
 *   to represent the correct sliders.
 *   
 * Paremters:
 *     int            iID   : The ID of the callback.
 *     colorSelectPtr csd   : Client data that contains all the widgets, and
 *                            other data of the color select dialog.
 *     int            value : The value of the selection.
 */

/* ARGSUSED */
static void
colorModelChangedCallback( int            iID,
                           colorSelectPtr csd,
                           int            value)
{

/* If the model didn't change we don't need to do anything. */

    if ( csd->colorMode == value) {
        return;
    }

/* Update the value. */

    csd->colorMode = value;
    switch( value) {
        case CS_RGB:

        /* Make sure the indecies for the update functions are correct. */

            csd->xValue = updateDefs[ csd->rgbZValue].x;
            csd->yValue = updateDefs[ csd->rgbZValue].y;
            csd->zValue = updateDefs[ csd->rgbZValue].z;

        /* Convert the current values that are in HSV into RGB values. */

            setRGBFromHSV( csd);
            setRGBScales( csd);

        /* Display the correct widgets. */
 
            XtMapWidget( csd->rgbConstant);
            XtMapWidget( csd->rgbFrame);
            XtUnmapWidget( csd->hsvFrame);
            XtUnmapWidget( csd->hsvConstant);
            break;

        case CS_HSV:

        /* Make sure the indecies for the update functions are correct. */

            csd->xValue = updateDefs[ csd->hsvZValue].x;
            csd->yValue = updateDefs[ csd->hsvZValue].y;
            csd->zValue = updateDefs[ csd->hsvZValue].z;

        /* Convert the current values that are in RGB into HSV values. */

            setHSVFromRGB( csd);
            setHSVScales( csd);

        /* Display the correct widgets. */

            XtMapWidget( csd->hsvConstant);
            XtMapWidget( csd->hsvFrame);
            XtUnmapWidget( csd->rgbFrame);
            XtUnmapWidget( csd->rgbConstant);
            break;
    }

/* Redraw the images, since they represent different color gradients now. */

    drawColors( csd);
}


/*
 * Function: rgbConstantChangedCallback( iID, csd, value) 
 *
 * Purpose:
 *   The rgbConbstantChangedCallback gets called when the value that is
 *   represented in the Z image changes.  The value can either be the
 *   red, green, or blue component of the RGB color.
 *   
 * Paremters:
 *     int            iID   : The ID of the callback.
 *     colorSelectPtr csd   : Client data that contains all the widgets, and
 *                            other data of the color select dialog.
 *     int            value : The value of the selection.
 */

/* ARGSUSED */
static void
rgbConstantChangedCallback( int            iID,
                            colorSelectPtr csd,
                            int            value)
{
    csd->rgbZValue = value;

/* Update the update indecies. */

    csd->xValue = updateDefs[ csd->rgbZValue].x;
    csd->yValue = updateDefs[ csd->rgbZValue].y;
    csd->zValue = updateDefs[ csd->rgbZValue].z;

/* Redraw the colors since the color gradients changed. */

    drawColors( csd);
}


/*
 * Function: hsvConstantChangedCallback( iID, csd, value) 
 *
 * Purpose:
 *   The hsvConbstantChangedCallback gets called when the value that is
 *   represented in the Z image changes.  The value can either be the
 *   hue, value, or saturation component of the HSV color.
 *   
 * Paremters:
 *     int            iID   : The ID of the callback.
 *     colorSelectPtr csd   : Client data that contains all the widgets, and
 *                            other data of the color select dialog.
 *     int            value : The value of the selection.
 */

/* ARGSUSED */
static void
hsvConstantChangedCallback( int            iID,
                            colorSelectPtr csd,
                            int            value)
{
    value += 3;
    csd->hsvZValue = value;

/* Update the update indecies. */

    csd->xValue = updateDefs[ value].x;
    csd->yValue = updateDefs[ value].y;
    csd->zValue = updateDefs[ value].z;

/* Redraw the colors since the color gradients changed. */

    drawColors( csd);
}


/*
 * Function: colorDestroyCallback( w, csd, xp)
 *
 * Purpose:
 *   The colorDestroyCallback gets called when the drawing area widget
 *   gets deleted.  It frees all the resources that are being used up
 *   by the color selection dialog.
 *   
 * Paremters:
 *     Widget         w     : The drawing area widget (not used). 
 *     colorSelectPtr csd   : Client data that contains all the widgets, and
 *                            other data of the color select dialog.
 *     XtPointer      xp    : The call data (not used). 
 */

static void
colorDestroyCallback( Widget         ,
                      colorSelectPtr csd,
                      XtPointer      )
{
    delete csd->imageXY;
    delete csd->windowXY;
    delete csd->imageZ;
    delete csd->windowZ;
    delete csd->gc;
    delete csd->display;
    free( csd);
}


/*
 * Function: colorOkCallback( w, csd, xp)
 *
 * Purpose:
 *   The colorOkCAllback gets called when the ok button on the color 
 *   selection dialog gets pressed.  It changes the color cell value
 *   and calls the update function if there is one.
 *
 * Paremters:
 *     Widget         w           : The OK button (not used) 
 *     Widget         colorButton : The button that launched the color
 *                                  dialog (not used). 
 *     colorSelectPtr csd         : Client data that contains all the widgets,
 *                                  and other data of the color select dialog.
 *     void           *clientData : The client data for the updater function. 
 *     XtPointer      xp          : The call data (not used). 
 */

/* ARGSUSED */
static void
colorOkCallback( Widget             w,
                 Widget             colorButton,
                 colorSelectPtr     csd,
		 AmColorSelUpdateFn updater,
		 void               *clientData,
                 XtPointer          xp)
{
    *( csd->cell) = csd->color;
    if ( updater != NULL) {
	( updater)( csd->cell, clientData);
    }
}


/*
 * Function: colorRevertCallback( w, colorButton, csd, xp)
 *
 * Purpose:
 *   Reverts all the values in the color selection dialog to
 *   the values that represent the original color passed in.
 *   That includes the preview area, the selection cursors,
 *   and the scales.
 * 
 * Parameters:
 *   Widget         w           : The revert butotn (not used).
 *   Widget         colorButotn : The button that launched the color selection
 *                                dialog (not used).
 *   colorSelectPtr csd         : Client data that contains all the widgets,
 *                                and other data of the color select dialog.
 *   XtPointer      xp          : The call data (not used).
 *   
 */

/* ARGSUSED */
static void
colorRevertCallback( Widget         w,
                     Widget         colorButton,
                     colorSelectPtr csd,
                     XtPointer      xp)
{

/* Revert to the original color. */

    csd->color = csd->original; 

/* Update the preview area. */

    XtVaSetValues( csd->colorArea, 
        XmNbackground, csd->color.pixel,
        NULL);

/* Revert to the original update functions. */

    csd->values[ CS_RED] = csd->originalValues[ CS_RED];
    csd->values[ CS_GREEN] = csd->originalValues[ CS_GREEN];
    csd->values[ CS_BLUE] = csd->originalValues[ CS_BLUE];

/* Compute the original HSV colors. */

    setHSVFromRGB( csd);

/* Update the images and both the RGB and HSV scales. */

    drawColors( csd);
    setRGBScales( csd);
    setHSVScales( csd);
}


/*
 * Function: rgbScaleCallback( scale, csd, xp)
 *
 * Purpose:
 *   The rgbScaleCallback gets called when one of the RGB scales is
 *   manipulated.  The function updates the RGB values and redraws
 *   the images that are changed.
 * 
 * Parameters:
 *   Widget         scale       : The color scale. 
 *   colorSelectPtr csd         : Client data that contains all the widgets,
 *                                and other data of the color select dialog.
 *   XtPointer      xp          : The call data (not used).
 *   
 */

/* ARGSUSED */
static void
rgbScaleCallback( Widget         scale,
                  colorSelectPtr csd,
                  XtPointer      xp)
{
    int       r, g, b, drawXY = 0;

/* Retrieve the RGB values. */

    XmScaleGetValue( csd->redScale, &r);
    XmScaleGetValue( csd->greenScale, &g);
    XmScaleGetValue( csd->blueScale, &b);

/* Figure out whether the z value changed and if so that means the
   XY image needs to be redrawn. */

    if ( scale == csd->redScale) {
        drawXY = ( csd->zValue == CS_RED);
    } else if ( scale == csd->greenScale) {
        drawXY = ( csd->zValue == CS_GREEN);
    } else if ( scale == csd->blueScale) {
        drawXY = ( csd->zValue == CS_BLUE);
    }

/* Undraw the selection cursors if needed. */

    if ( drawXY) {
        drawZPointer( csd);
    } else {
        drawXYPointer( csd);
    }

/* Update the stored values. */

    csd->values[ CS_RED] = r;
    csd->values[ CS_GREEN] = g;
    csd->values[ CS_BLUE] = b;

/* Update the imgaes and redraw the selection cursors if needed. */

    if ( drawXY) {
        drawColorsXY( csd);
        drawZPointer( csd);
    } else {
        drawColorsZ( csd);
        drawXYPointer( csd);
    }

/* Update the preview area. */

    updateColors( csd);
}


/*
 * Function: hsvScaleCallback( scale, csd, xp)
 *
 * Purpose:
 *   The hsvScaleCallback gets called when one of the HSV scales is
 *   manipulated.  The function updates the HSV values and redraws
 *   the images that are changed.
 * 
 * Parameters:
 *   Widget         scale       : The color scale. 
 *   colorSelectPtr csd         : Client data that contains all the widgets,
 *                                and other data of the color select dialog.
 *   XtPointer      xp          : The call data (not used).
 *   
 */

/* ARGSUSED */
static void
hsvScaleCallback( Widget         scale,
                  colorSelectPtr csd,
                  XtPointer      xp)
{
    int       h, s, v, drawXY = 0;

/* Retrieve the HSV values. */

    XmScaleGetValue( csd->hueScale, &h); 
    XmScaleGetValue( csd->saturationScale, &s); 
    XmScaleGetValue( csd->valueScale, &v); 

/* Figure out whether the z value changed and if so that means the
   XY image needs to be redrawn. */

    if ( scale == csd->hueScale) {
        drawXY = (csd->zValue == CS_HUE);
    } else if ( scale == csd->saturationScale) {
        drawXY = (csd->zValue == CS_SATURATION);
    } else if ( scale == csd->valueScale) {
        drawXY = (csd->zValue == CS_VALUE);
    }

/* Undraw the selection cursors if needed. */

    if ( drawXY) {
        drawZPointer( csd);
    } else {
        drawXYPointer( csd);
    }

/* Update the stored values. */

    h = (int)(h * 255.0 / 10.0);
    s = (int)(s * 255.0 / 100.0);
    v = (int)(v * 255.0 / 100.0);

    csd->values[ CS_HUE] = h;
    csd->values[ CS_SATURATION] = s;
    csd->values[ CS_VALUE] = v;

/* Update the imgaes and redraw the selection cursors if needed. */

    if ( drawXY) {
        drawColorsXY( csd);
        drawZPointer( csd);
    } else {
        drawColorsZ( csd);
        drawXYPointer( csd);
    }

/* Update the preview area. */

    updateColors( csd);
}

/* Values for the color model buttons. */

static char *colorModelStrings[] = { "RGB", "HSV", NULL };
static char *rgbConstantStrings[] = { "Red", "Green", "Blue", NULL };
static char *hsvConstantStrings[] = { "Hue", "Saturation", "Value", NULL };


/*
 * Function: createScale( parent, below, label, scale, name, value)
 *
 * Purpose:
 *   Creates a new scale widget inside a form and next to a label.
 *   The parent for the form is passed in.  Optionally another widget
 *   which the form is supposed to be below can also be passed in.
 *   Finally the name and value of the scale are also passsed in.
 *
 * Parameters:
 *   Widget     parent : The parent widget to the new form widget.
 *   Widget     below  : The widget above the form widget (NULL) if
 *                       the form isn't supposed to be attached.
 *   Widget     *label : The return variable for the label that gets
 *                       created.
 *   Widget     *scale : The return variable for the scale that gets
 *                       created.
 *   char       *name  : The name of the scale (the form is called:
 *                       nameForm, and the label is nameLabel).
 *   unsigned   value  : The value that the scale is to be initialized
 *                       with (The RGB component).
 * Returns:
 *   Widget : The form widget that gets created.
 */ 

static Widget
createScale( Widget   parent,
             Widget   below,
             Widget   *label,
             Widget   *scale,
             char     *name,
             unsigned value)
{
    Widget form;
    char   buffer[ TEXTLEN];

/* Create the form widget, optionally below another widget. */

    sprintf( buffer, "%sForm", name);

    if ( below == NULL) {
        form = XtVaCreateManagedWidget( buffer,
                   xmFormWidgetClass, parent,
                   StickLeft, StickTop, StickRight,
                   NULL);
    } else {
        form = XtVaCreateManagedWidget( buffer,
                   xmFormWidgetClass, parent,
                   StickLeft, BelowOf( below), StickRight,
                   NULL);
    }

/* Create the label widget on the left side of the form widget. */

    sprintf( buffer, "%sLabel", name);

    *label = XtVaCreateManagedWidget( buffer,
                 xmLabelWidgetClass, form,
                 StickLeft, StickTop, StickBottom,
                 XmNalignment, XmALIGNMENT_BEGINNING,
                 NULL);

    sprintf( buffer, "%sScale", name);

/* Create the scale widget on the right of the label widget. */

    *scale = XtVaCreateManagedWidget( buffer,
                 xmScaleWidgetClass, form,
                 RightOf( *label), StickTop, StickRight, StickBottom,
                 XmNwidth, 256,
                 XmNminimum, 0, XmNmaximum, 255,
                 XmNorientation, XmHORIZONTAL,
                 XmNvalue, value >> 8,
                 XmNshowValue, TRUE,
                 NULL);

/* Return the form that we created. */

    return( form);
}


/*
 * Function: AmCreateColorSelectionDialog( parent, name, cell, updater,
 *                                         clientData)
 * 
 * Purpose:
 *   Creates a color selection dialog, which allow selection of
 *   colors in either RGB or HSV mode. 
 *
 * Parameters:
 *   Widget             parent      : The parent widget to the color selection  
 *                                    dialog.
 *   const char         *name       : The resource name of the dialog.
 *   AmColorCellPtr     cell        : The color cell to change (must be
 *                                    allocated This also gives the originally
 *                                    seslected color.
 *   AmColorSelUpdateFn updater     : Update function to call on OK (defaults
 *                                    to NULL).
 *   void               *clientData : Client data passed to the update function
 *                                    (defaults to NULL).
 *   
 * Returns:
 *   Widget : The dialog widget of the color selection dialog.
 *
 * Comments:
 *   The selected color is placed in the passed in cell.  In addition, if the
 *   updater function is not NULL, it will be called with the color cell
 *   and the passed in client data.
 */

Widget
AmCreateColorSelectionDialog( Widget 	 	parent, 
			    char 	 	*name, 
			    AmColorCellPtr 	cell,
			    AmColorSelUpdateFn	updater,
			    void	 	*clientData)
{
    colorSelectPtr csd;		/* Color Selection Data. */
    AmActionRec      actionArea;
    Widget         form,
                   colorXYFrame, colorZFrame,
                   colorOptionForm,
                   colorForm,
                   modeForm,
                   newColorFrame,
                   oldColorFrame,
                   colorModelLabel, constantLabel,
                   rgbForm,
                   redForm, redLabel,
                   greenForm, greenLabel,
                   blueForm, blueLabel,
                   hsvForm,
                   hueForm, hueLabel,
                   saturationForm, saturationLabel,
                   valueForm, valueLabel,
                   actionForm;
    AmFrameRec       frame;
    short          max, temp;

    csd = ( colorSelectPtr) malloc( sizeof( colorSelectRec));
 
    csd->color = *cell;
    csd->original = *cell;
    csd->cell = cell;

    csd->colorMode = CS_HSV;
    csd->xValue = updateDefs[ CS_HUE].x;
    csd->yValue = updateDefs[ CS_HUE].y;
    csd->zValue = updateDefs[ CS_HUE].z;

    csd->rgbZValue = CS_RED;
    csd->hsvZValue = CS_HUE;

    csd->originalValues[ CS_RED] = csd->color.red;
    csd->originalValues[ CS_GREEN] = csd->color.green;
    csd->originalValues[ CS_BLUE] = csd->color.blue;

    csd->values[ CS_RED] = csd->color.red; 
    csd->values[ CS_GREEN] = csd->color.green;
    csd->values[ CS_BLUE] = csd->color.blue;
    csd->flag = FALSE;

    setHSVFromRGB( csd);

/* Create the dialog. */ 

    csd->shell = XtVaCreatePopupShell( name,
                     topLevelShellWidgetClass, parent,
                     XmNvisual, AmGetAppVisual(),
                     XmNcolormap, AmGetAppColormap(),
                     XmNdepth, AmGetAppDepth(),
                     XmNdialogStyle, XmDIALOG_APPLICATION_MODAL,
                     NULL);

    form = csd->form = XtVaCreateManagedWidget( "colorForm",
                           xmFormWidgetClass, csd->shell,
                           NULL);

    colorXYFrame = AmMakeFrame( form, "colorXYFrame", FALSE, NULL);
    XtVaSetValues( colorXYFrame, StickLeft, StickTop, NULL);

    csd->colorXY = XtVaCreateManagedWidget( "colorXY",
                       xmDrawingAreaWidgetClass, colorXYFrame,
                       XmNwidth, SHOW_NUM_COLORS,
                       XmNheight, SHOW_NUM_COLORS,
                       NULL);

    colorZFrame = AmMakeFrame( form, "colorZFrame", FALSE, NULL);
    XtVaSetValues( colorZFrame, RightOf( colorXYFrame), StickTop, NULL);

    csd->colorZ = XtVaCreateManagedWidget( "colorZ",
                      xmDrawingAreaWidgetClass, colorZFrame,
                      XmNwidth, SHOW_WIDTH,
                      XmNheight, SHOW_NUM_COLORS,
                      NULL);

    colorOptionForm = XtVaCreateManagedWidget( "colorOptionForm",
                          xmFormWidgetClass, form,
                          RightOf( colorZFrame), StickTop, StickRight,
                          NULL);

/* Display the color in a drawing area. */ 

    colorForm = XtVaCreateManagedWidget( "colorForm",
                    xmFormWidgetClass, colorOptionForm,
                    StickLeft, StickTop, StickRight,
                    XmNfractionBase, SPACING * 2 + 1,
                    NULL);

    newColorFrame = AmMakeFrame( colorForm, "newColorFrame", TRUE, &frame);
    XtVaSetValues( newColorFrame,
        XmNleftAttachment, XmATTACH_POSITION,
        XmNleftPosition, 1,
        XmNrightAttachment, XmATTACH_POSITION,
        XmNrightPosition, SPACING,
        NULL);
    XtVaSetValues( frame.label,
        XmNchildVerticalAlignment, XmALIGNMENT_WIDGET_TOP,
        NULL);

    csd->colorArea = XtVaCreateManagedWidget( "newColorArea",
                         xmDrawingAreaWidgetClass, newColorFrame,
                         XmNbackground, csd->color.pixel,
                         NULL);

    oldColorFrame = AmMakeFrame( colorForm, "oldColorFrame", TRUE, &frame);
    XtVaSetValues( oldColorFrame,
        XmNleftAttachment, XmATTACH_POSITION,
        XmNleftPosition, SPACING + 1,
        XmNrightAttachment, XmATTACH_POSITION,
        XmNrightPosition, SPACING * 2,
        NULL);
    XtVaSetValues( frame.label,
        XmNchildVerticalAlignment, XmALIGNMENT_WIDGET_TOP,
        NULL);

    XtVaCreateManagedWidget( "oldColorArea",
                             xmDrawingAreaWidgetClass, oldColorFrame,
                             XmNbackground, csd->color.pixel,
                             NULL);

    modeForm = XtVaCreateManagedWidget( "modeForm",
                   xmFormWidgetClass, colorOptionForm,
                   StickLeft, BelowOf( colorForm), StickRight,
                   NULL);

    colorModelLabel = XtVaCreateManagedWidget( "colorModelLabel",
                          xmLabelWidgetClass, modeForm,
                          StickLeft, StickTop, StickBottom, 
                          NULL);

    csd->colorModel = AmCreateOptionMenu( modeForm, "colorModel",
                          (const char **)colorModelStrings, 1);
    XtVaSetValues( csd->colorModel,
        RightOf( colorModelLabel), StickTop, StickBottom, NULL);

    AmAddCallbackWidget( AM_PKG_ID, AM_OPTION_CHANGED, csd->colorModel,
        ( callbackProc) colorModelChangedCallback,
        csd);

    constantLabel = XtVaCreateManagedWidget( "constantLabel",
                        xmLabelWidgetClass, modeForm,
                        RightOf( csd->colorModel), StickTop, StickBottom,
                        NULL);

    csd->rgbConstant = AmCreateOptionMenu( modeForm, "rgbConstant",
                             (const char **)rgbConstantStrings, 0);
    XtVaSetValues( csd->rgbConstant,
        RightOf( constantLabel), StickTop, StickBottom, NULL);
    AmAddCallbackWidget( AM_PKG_ID, AM_OPTION_CHANGED, csd->rgbConstant,
        ( callbackProc) rgbConstantChangedCallback,
        csd);

    csd->rgbFrame = AmMakeFrame( colorOptionForm, "rgbFrame", TRUE, NULL);
    XtVaSetValues( csd->rgbFrame, StickLeft, BelowOf( modeForm),
        StickRight, NULL);

    rgbForm = XtVaCreateManagedWidget( "rgbForm",
                  xmFormWidgetClass, csd->rgbFrame,
                  NULL);

    redForm = createScale( rgbForm, NULL,
                  &redLabel, &(csd->redScale),
                  "red", csd->color.red);

    greenForm = createScale( rgbForm, redForm,
                    &greenLabel, &(csd->greenScale),
                    "green", csd->color.green);

    blueForm = createScale( rgbForm, greenForm,
                   &blueLabel, &(csd->blueScale),
                   "blue", csd->color.blue);

    XtVaSetValues( blueForm, StickBottom, NULL);
   
/* The action buttons. */

    actionAreaDef[ 2].clientData = form;
    actionForm = AmMakeActionArea( form, actionAreaDef,
                                   ELEMENTS( actionAreaDef), &actionArea);
    XtVaSetValues( actionForm, BelowOf( colorXYFrame), NULL);

    csd->hsvConstant = AmCreateOptionMenu( modeForm, "hsvConstant",
                             (const char **)hsvConstantStrings, 0);
    XtVaSetValues( csd->hsvConstant,
        RightOf( constantLabel), StickTop, StickBottom, NULL);
    AmAddCallbackWidget( AM_PKG_ID, AM_OPTION_CHANGED, csd->hsvConstant,
        ( callbackProc) hsvConstantChangedCallback,
        csd);

    csd->hsvFrame = AmMakeFrame( colorOptionForm, "hsvFrame", TRUE, NULL);
    XtVaSetValues( csd->hsvFrame, StickLeft, BelowOf( modeForm),
        StickRight, NULL);

    hsvForm = XtVaCreateManagedWidget( "hsvForm",
                  xmFormWidgetClass, csd->hsvFrame,
                  NULL);

    hueForm = createScale( hsvForm, NULL,
                  &hueLabel, &(csd->hueScale),
                  "hue", csd->color.red);
    XtVaSetValues( csd->hueScale,
        XmNmaximum, 359 * 10,
        XmNdecimalPoints, 1,
        NULL);

    saturationForm = createScale( hsvForm, hueForm,
                         &saturationLabel, &(csd->saturationScale),
                         "saturation", csd->color.green);
    XtVaSetValues( csd->saturationScale,
        XmNmaximum, 100,
        XmNdecimalPoints, 2,
        NULL);

    valueForm = createScale( hsvForm, saturationForm,
                    &valueLabel, &(csd->valueScale),
                    "value", csd->color.blue);
    XtVaSetValues( csd->valueScale,
        XmNmaximum, 100,
        XmNdecimalPoints, 2,
        NULL);

    XtVaSetValues( valueForm, StickBottom, NULL);
  
/* Align the lables: */

    XtVaGetValues( redLabel, XmNwidth, &max, NULL);
    XtVaGetValues( greenLabel, XmNwidth, &temp, NULL);
    if ( temp > max) {
        max = temp;
    }
    XtVaGetValues( blueLabel, XmNwidth, &temp, NULL);
    if ( temp > max) {
        max = temp;
    }
    XtVaGetValues( hueLabel, XmNwidth, &temp, NULL);
    if ( temp > max) {
        max = temp;
    }
    XtVaGetValues( saturationLabel, XmNwidth, &temp, NULL);
    if ( temp > max) {
        max = temp;
    }
    XtVaGetValues( valueLabel, XmNwidth, &temp, NULL);
    if ( temp > max) {
        max = temp;
    }

    XtPopup( csd->shell, XtGrabNone);

    XtVaSetValues( redLabel, XmNwidth, max, NULL);
    XtVaSetValues( greenLabel, XmNwidth, max, NULL);
    XtVaSetValues( blueLabel, XmNwidth, max, NULL);
    XtVaSetValues( hueLabel, XmNwidth, max, NULL);
    XtVaSetValues( saturationLabel, XmNwidth, max, NULL);
    XtVaSetValues( valueLabel, XmNwidth, max, NULL);

    XtUnmapWidget( csd->rgbConstant);
    XtUnmapWidget( csd->rgbFrame);

/* Setup the callback functions: */

    XtAddCallback( actionArea.buttons[ 0], XmNactivateCallback,
        ( XtCallbackProc) AmPopdownCback,
        ( XtPointer) form);

    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback,
        ( VarargsCallback) colorOkCallback,
        parent, csd, updater, clientData, END_LIST);

    AmAddVaCallback( actionArea.buttons[ 1], XmNactivateCallback,
        ( VarargsCallback) colorRevertCallback,
        parent, csd, END_LIST);

/* Free the allocated memory and the allocated color. */

    XtAddCallback( form, XmNdestroyCallback,
        ( XtCallbackProc) colorDestroyCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->redScale, XmNvalueChangedCallback,
        ( XtCallbackProc) rgbScaleCallback,
        ( XtPointer) csd);
    XtAddCallback( csd->redScale, XmNdragCallback,
        ( XtCallbackProc) rgbScaleCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->greenScale, XmNvalueChangedCallback,
        ( XtCallbackProc) rgbScaleCallback,
        ( XtPointer) csd);
    XtAddCallback( csd->greenScale, XmNdragCallback,
        ( XtCallbackProc) rgbScaleCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->blueScale, XmNvalueChangedCallback,
        ( XtCallbackProc) rgbScaleCallback,
        ( XtPointer) csd);
    XtAddCallback( csd->blueScale, XmNdragCallback,
        ( XtCallbackProc) rgbScaleCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->hueScale, XmNvalueChangedCallback,
        ( XtCallbackProc) hsvScaleCallback,
        ( XtPointer) csd);
    XtAddCallback( csd->hueScale, XmNdragCallback,
        ( XtCallbackProc) hsvScaleCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->saturationScale, XmNvalueChangedCallback,
        ( XtCallbackProc) hsvScaleCallback,
        ( XtPointer) csd);
    XtAddCallback( csd->saturationScale, XmNdragCallback,
        ( XtCallbackProc) hsvScaleCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->valueScale, XmNvalueChangedCallback,
        ( XtCallbackProc) hsvScaleCallback,
        ( XtPointer) csd);
    XtAddCallback( csd->valueScale, XmNdragCallback,
        ( XtCallbackProc) hsvScaleCallback,
        ( XtPointer) csd);

    XtAddCallback( csd->colorXY, XmNexposeCallback,
        ( XtCallbackProc) exposeImageXY,
        ( XtPointer) csd);
    XtAddCallback( csd->colorZ, XmNexposeCallback,
        ( XtCallbackProc) exposeImageZ,
        ( XtPointer) csd);

    XtAddEventHandler( csd->colorXY,
        ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, False,
        ( XtEventHandler) colorChooseXYCallback,
        csd);

    XtAddEventHandler( csd->colorZ,
        ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, False,
        ( XtEventHandler) colorChooseZCallback,
        csd);

/* Create the XY and Z images. */

    createImages( csd);

/* Fill the two images with the correct color gradients. */

    drawColors( csd);

/* Update the RGB and HSV scales according to the original color. */

    setRGBScales( csd);
    setHSVScales( csd);

    return( form);
}
