#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <anvil/list.h>
#include <anvil/dialang.h>

/* for storing the static dialog declaration list */

static AmListPtr	statDeclList = NULL;
static FILE		*dialogFp;
int			dialogErrorLineNo;
AnvilBoolean			dialogErrorFlag;

/* yacckisms */

int dialogparse( void);
void dialogrestart( FILE *);

/***************************************************************************
 *
 * Description:
 *	parse a conditional expression.
 * Arguments:
 *	boolExpr - boolean expression.
 *	trueDeclList - execute if boolExpr is true.
 *	falseDeclList - execute if boolExpr is false (may be NULL)
 * Returns:
 * 	dialogEntry data structure pointer
 *
 **************************************************************************/
 
AmDialogEntryPtr 
AmMakeBooleanDecl( AmBoolExprPtr	boolExpr,
		   AmListPtr		trueDeclList,
		   AmListPtr		falseDeclList)
{
    AmDialogEntryPtr	dialogEntry;

AN_DEBUG( 20, ("making boolean declaration\n"));
    dialogEntry = ( AmDialogEntryPtr) malloc( sizeof( AmDialogEntryRec));
    if ( dialogEntry == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    dialogEntry->u.conditionalItem.boolExpr = boolExpr;
    dialogEntry->u.conditionalItem.trueDeclList = trueDeclList;
    dialogEntry->u.conditionalItem.falseDeclList = falseDeclList;
    if ( falseDeclList == NULL) {
	dialogEntry->type = AmDIALOG_IF_THEN;
    } else {
	dialogEntry->type = AmDIALOG_IF_THEN_ELSE;
    }
    return( dialogEntry);
}

/***************************************************************************
 *
 * Description:
 *	create a field list expression, which may be a box, popup
 *	or custom editor
 * Arguments:
 *	name - name for the field list: popup, box, or custom
 *	editor name.
 *	fieldList - list of fields to execute.
 * Returns:
 * 	dialogEntry data structure pointer
 *
 **************************************************************************/

AmDialogEntryPtr 
AmMakeFieldList( char		*name,
		 AmListPtr	fieldList)
{
    AmDialogEntryPtr	dialogEntry;

    dialogEntry = ( AmDialogEntryPtr) malloc( sizeof( AmDialogEntryRec));
    if ( dialogEntry == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* set the operator enumeration */

    dialogEntry->u.declItem.fields = fieldList;
    dialogEntry->u.declItem.name = name;
    if ( !strcasecmp( name, "popup")) {
AN_DEBUG( 20, ("creating popup field list\n"));
	dialogEntry->type = AmDIALOG_POPUP_FIELD_LIST;
    } else if ( !strcasecmp( name, "box")) {
AN_DEBUG( 20, ("creating box field list\n"));
	dialogEntry->type = AmDIALOG_BOX_FIELD_LIST;
    } else {
AN_DEBUG( 20, ("creating editor field list\n"));
	dialogEntry->type = AmDIALOG_EDITOR;
    }
    return( dialogEntry);
}

AmDialogEntryPtr
AmMakeIdent( char	*name)
{
    AmDialogEntryPtr	dialogEntry;

    dialogEntry = ( AmDialogEntryPtr) malloc( sizeof( AmDialogEntryRec));
    if ( dialogEntry == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* set the identifier and dialog entry type */

    dialogEntry->u.identItem = name;
    dialogEntry->type = AmDIALOG_IDENT;
    return( dialogEntry);
}

/***************************************************************************
 *
 * Description:
 *	create a declaration of the form ident = { decl-list }
 *	or custom editor
 * Arguments:
 *	ident - name of the declaration.
 *	list - list of fields to execute.
 * Returns:
 * 	declaration structure pointer.
 *
 **************************************************************************/

AmDeclarationPtr
AmMakeDecl( char	*ident,
	    AmListPtr	list)
{
    AmDeclarationPtr	declaration;

AN_DEBUG( 20, ("creating declaration %s\n", ident));
    declaration = ( AmDeclarationPtr) malloc( sizeof( AmDeclarationRec));
    if ( declaration == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    declaration->ident = ident;
    declaration->declList = list;
    return( declaration);
}

/***************************************************************************
 *
 * Description:
 *	make a boolean expression of the form ident op value.
 * Arguments:
 *	ident - name of the declaration.
 *	op - boolean operation.
 *	value - value to test against 
 * Returns:
 * 	boolean expression structure pointer.
 *
 **************************************************************************/

AmBoolExprPtr
AmMakeBoolexpr( char	*tag,
		char	*op,
		char	*value)
{   
    AmBoolExprPtr	boolExpr;

AN_DEBUG( 20, ("creating boolean expression %s %s %s\n", tag, op, value));
    boolExpr = ( AmBoolExprPtr) malloc( sizeof( AmBoolExprRec));
    if ( boolExpr == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    boolExpr->tag = tag;
    if ( !strcmp( op, "==")) {
	boolExpr->op = AmBOOLEAN_EQUAL;
    } else {
	boolExpr->op = AmBOOLEAN_NOT_EQUAL;
    }
    boolExpr->value = value;
    return( boolExpr);
}

/***************************************************************************
 *
 * Description:
 *	add a declaration to the list of declarations, and store it
 *	in the static list, because YACC can't return values.
 * Arguments:
 *	delcaration - declaration to add
 *	list - list of declarations.
 * Returns:
 *	list of declarations.
 *
 **************************************************************************/

AmListPtr	
AmAddDecl( AmDeclarationPtr	declaration,
	   AmListPtr		list)
{
AN_DEBUG( 20, ("adding declaration\n"));
    list = AmInsertNodeLst( declaration, list);
    statDeclList = list;
    return( list);
}

/***************************************************************************
 *
 * Description:
 *	wrapper for fread() to read the stream.
 * Arguments:
 *	buffer - buffer to read int.
 *	maxSize - length of the buffer.
 * Returns:
 *	number of bytes read.
 *
 **************************************************************************/
int 
AmDialogFpYYinput( char *buffer,
		   int  maxSize)
{
    return( fread( buffer, 1, maxSize, dialogFp));
}

/* take a dialog description file and parse it into a dialog parse structure */

AmErrorCode 
AmParseDialogFile( char 	*file,
		   AmListPtr	*declList,
		   char		*errorString)
{
    if (( dialogFp = fopen( file, "r")) == NULL) {
	sprintf( errorString, "failed to open %s", file);
	return( ANVIL_ERROR_CODE = AmOPEN_FAILED);
    }
    statDeclList = NULL;
    dialogrestart( dialogFp);
    dialogErrorLineNo = 1;
    dialogErrorFlag = FALSE;
    dialogparse();
    fclose( dialogFp);
    if ( dialogErrorFlag) {
	return( ANVIL_ERROR_CODE = AmBAD_ARGUMENT);
    }
    *declList = statDeclList;
    return( AmOK);
}

/* bogus function for reporting parse errors in dialog template files */

int 
dialogerror( const char *msg)
{
    return( TRUE);
}

/* free a dialog entry structure */

void 
AmFreeDialogEntry( AmDialogEntryPtr	dialogEntry)
{
    switch ( dialogEntry->type) {
    case AmDIALOG_IF_THEN:
    case AmDIALOG_IF_THEN_ELSE:
	free( dialogEntry->u.conditionalItem.boolExpr->tag);
	free( dialogEntry->u.conditionalItem.boolExpr->value);
	free( dialogEntry->u.conditionalItem.boolExpr);
	AmFreeLst( dialogEntry->u.conditionalItem.trueDeclList, 
		   ( AmItemFn) AmFreeDialogEntry);
	AmFreeLst( dialogEntry->u.conditionalItem.falseDeclList, 
		   ( AmItemFn) AmFreeDialogEntry);
	break;
    case AmDIALOG_BOX_FIELD_LIST:
    case AmDIALOG_POPUP_FIELD_LIST:
    case AmDIALOG_EDITOR:
	free( dialogEntry->u.declItem.name);
	AmFreeLst( dialogEntry->u.declItem.fields, 
		   ( AmItemFn) AmFreeDialogEntry);
	break;
    case AmDIALOG_IDENT:
	free( dialogEntry->u.identItem);
	break;
    default:
	break;
    }
    free( dialogEntry);
}
