%{
#include <stdio.h>
#include <string.h>
#include <anvil/list.h>
#include <anvil/dialang.h>
int yylex( void);
int yyerror( const char *msg);
%}

%union {
    AmDeclarationPtr	declaration;
    AmDialogEntryPtr	dialogEntry;
    AmBoolExprPtr	boolExpr;
    char		*string;
    AmListPtr		list;
}

%token <string> VALUE
%token <string> IF
%token <string> THEN
%token <string> ELSE
%token <string> IDENT
%token <string> EQUALS
%token <string> NOT_EQUALS

%type <list> 		dialogList
%type <list> 		declList
%type <list> 		declbracket
%type <list> 		fieldList
%type <declaration>	dialogdecl
%type <dialogEntry>	decl
%type <boolExpr>	boolexpr

%start dialogList

%%

dialogList  : dialogList dialogdecl 	{ $$ = AmAddDecl( $2, $1); }
	    | dialogdecl 		{ $$ = AmAddDecl( $1, NULL); }
	    ;

dialogdecl  : IDENT '=' '{' declList '}' ';'  
					{ $$ = AmMakeDecl( $1, $4); }
	    ;

declbracket : '{' declList '}'		{ $$ = $2; }
declList    : declList ',' decl		{ $$ = AmInsertNodeAtEndLst( $3, $1); }
	    | decl			{ $$ = AmInsertNodeLst( $1, NULL); }
	    ;

decl 	    : IF '(' boolexpr ')' THEN declbracket
					{ $$ = AmMakeBooleanDecl( $3, $6, NULL); }
	    | IF '(' boolexpr ')' THEN declbracket ELSE declbracket
					{ $$ = AmMakeBooleanDecl( $3, $6, $8); }
	    | IDENT '(' fieldList ')'	{ $$ = AmMakeFieldList( $1, $3); }
	    | IDENT '(' ')'		{ $$ = AmMakeFieldList( $1, NULL); }
	    | IDENT			{ $$ = AmMakeIdent( $1); }
	    ;

fieldList   : fieldList ',' IDENT	{ $$ = AmInsertNodeAtEndLst( $3, $1); }
	    | IDENT			{ $$ = AmInsertNodeLst( $1, NULL); }
	    ;

boolexpr    : IDENT EQUALS VALUE	{ $$ = AmMakeBoolexpr( $1, "==", $3); }
	    | IDENT NOT_EQUALS VALUE	{ $$ = AmMakeBoolexpr( $1, "!=", $3); }
	    | IDENT EQUALS IDENT	{ $$ = AmMakeBoolexpr( $1, "==", $3); }
	    | IDENT NOT_EQUALS IDENT	{ $$ = AmMakeBoolexpr( $1, "!=", $3); }
	    ;

%%
