#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <string.h>
#include <Xm/Xm.h>
#include <anvil/events.h>

void AmTransmitExposeCback( Widget,
			  Widget		dst,
			  XmAnyCallbackStruct	*cb)
{
    if ( cb->event->xany.send_event) {
	AmSendExpose( XtDisplay( dst), XtWindow( dst));
    }
}

void AmSendExposeCback( Widget,
		      Widget 	dst,
		      XtPointer)
{
    AmSendExpose( XtDisplay( dst), XtWindow( dst));
}

/* send a fake expose event to a screen rectangle */

void AmSendExposeRect( Display 	*display,
		       Window	win,
		       int	x,
		       int	y, 
		       int	width,
		       int	height)
{
    XExposeEvent 	event;

    event.type = Expose;
    event.window = win;
    event.count = 0;		/* some callbacks compress exposes */
    event.display = display;
    event.x = x;
    event.y = y;
    event.width = width;
    event.height = height;
    XSendEvent( display, win, True, ExposureMask, ( XEvent *) &event);
}


/* send a fake expose event */

void AmSendExpose( Display 	*display,
		  Window	win)
{
    XExposeEvent 	event;
    Window		root;
    unsigned int	width, height, border, depth;
    int			x, y;

    event.type = Expose;
    event.window = win;
    event.count = 0;		/* some callbacks compress exposes */
    event.display = display;
    XGetGeometry( display, win, &root, &x, &y, &width, &height, 
		  &border, &depth);
    event.x = 0;
    event.y = 0;
    event.width = width;
    event.height = height;
    XSendEvent( display, win, True, ExposureMask, ( XEvent *) &event);
}

/* send a fake window enter event */

void AmSendEnter( Display	*display,
		 Window		win)
{
    XCrossingEvent 	event;

    event.type = EnterNotify;
    event.window = win;
    event.display = display;
    XSendEvent( display, win, True, EnterWindowMask, ( XEvent *) &event);
}

/* send a fake window leave event */

void AmSendLeave( Display	*display,
		 Window		win)
{
    XCrossingEvent 	event;

    event.type = LeaveNotify;
    event.window = win;
    event.display = display;
    XSendEvent( display, win, True, LeaveWindowMask, ( XEvent *) &event);
}

/* send a fake button press event */

void AmSendButtonPress( Display	*display,
			Window	win,
			int	x,
			int	y)
{
    XButtonEvent	event;

    event.type = ButtonPress;
    event.window = win;
    event.display = display;
    event.x = x;
    event.y = y;
    XSendEvent( display, win, True, ButtonPressMask, ( XEvent *) &event);
}

/* send a fake button release event */

void AmSendButtonRelease( Display	*display,
			  Window	win,
			  int		x,
			  int		y)
{
    XButtonEvent	event;

    event.type = ButtonRelease;
    event.window = win;
    event.display = display;
    event.x = x;
    event.y = y;
    XSendEvent( display, win, True, ButtonReleaseMask, ( XEvent *) &event);
}

/* send a fake PointerMoved ( motion) event. Doesn't work */

void AmSendMotion( Display	*display,
		  Window	win,
		  int		x,
		  int		y)
{
    XMotionEvent	event;

    event.type = MotionNotify;
    event.window = win;
    event.subwindow = win;
    event.root = DefaultRootWindow( display);
    event.display = display;
    event.x = x;
    event.y = y;
    event.state = Button1Mask;
    XSendEvent( display, win, True, PointerMotionMask, ( XEvent *) &event);
}

/* send a client message with data */

void AmSendClientMessage( Display	*display, 
			  Window	win,
			  int		format,
			  void		*data)		/* must be 20 chars */
{
    XClientMessageEvent	event;

    event.type = ClientMessage;
    event.display = display;
    event.window = win;
    event.format = format;
    memcpy(( void *) event.data.b, data, 20);
    XSendEvent( display, win, True, NoEventMask, ( XEvent *) &event);
}

/* send an event based on the event code */

void AmSendEventByCode( Display	*display,
			 Window		win,
			 long 		eventCode,
			 int		x,
			 int		y)
{
    switch ( eventCode) {
    case Expose:
	AmSendExpose( display, win);
	break;
    case ButtonPress:
	AmSendButtonPress( display, win, x, y);
	break;
    case ButtonRelease:
	AmSendButtonRelease( display, win, x, y);
	break;
    case EnterNotify:
	AmSendEnter( display, win);
	break;
    case LeaveNotify:
	AmSendLeave( display, win);
	break;
    case MotionNotify:
	AmSendMotion( display, win, x, y);
	break;
    }
}

/* convienience: set the clipping for a gc depending on the exposed area */

void AmSetExposeClip( Display		*display, 
		      GC		gc,
		      XExposeEvent	*event)
{
    XRectangle	clipRect;

    clipRect.x = event->x;
    clipRect.y = event->y;
    clipRect.width = event->width;
    clipRect.height = event->height;
    XSetClipRectangles( display, gc, 0, 0, &clipRect, 1, YXSorted);
}
