#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <malloc.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <anvil/project.h>
#include <anvil/motif.h>

const int RESBUFSIZE = 1024;

/* return all of the widgets which are children of this widget */

AnvilBoolean AmGetWidgetChildren( Widget	w,
			   Widget	**children,
			   unsigned int	*numChildren)
{
    Window		rootWin, parentWin, *childrenWin;
    unsigned int	nchild, ii;

/* query the window tree, then convert them to widgets */

    *children    = NULL;
    *numChildren = 0;

    if ( !XQueryTree( XtDisplay( w), XtWindow( w),
		      &rootWin, &parentWin, &childrenWin, &nchild)) {
	return( FALSE);
    }
    if ( nchild == 0) {
	return( FALSE);
    }
    *children = ( Widget *) malloc( sizeof( Widget)*nchild);
    for ( ii = 0; ii < nchild; ii++) {
	(*children)[ ii] = XtWindowToWidget( XtDisplay( w), childrenWin[ ii]);
    }
    XFree( childrenWin);
    *numChildren = nchild;
    return( TRUE);
}

/* get the parent top level shell of a widget */

Widget AmGetWidgetShell( Widget	a)
{
    return( XtIsShell( a) ? a : AmGetWidgetShell( XtParent( a)));
}

/* get the parent top level shell of a widget, but avoid transient shells */ 

Widget AmGetWidgetRealShell( Widget	a)
{
    return(( XtIsShell( a) && !XtIsTransientShell( a) && 
           ( XtIsWMShell( a) || XtIsTopLevelShell( a) ||
             XtIsApplicationShell( a))) ? 
		a : AmGetWidgetRealShell( XtParent( a)));
}

/* get the top level shell of an application */

Widget AmGetTopLevelShell( Widget	wCurr)
{
    Widget wPrev = NULL;

    while ( wCurr != NULL) {
        wPrev = wCurr;
        wCurr = XtParent( wCurr);
    }

    return( wPrev);
}

/* return a parent of a widget of the specified type */

Widget AmGetParentOfType( Widget		w,
			WidgetClass	theClass)
{
    if ( w == NULL) {
	return( NULL);
    } else if ( XtIsSubclass( w, theClass)) {
	return( w);
    } else {
	return( AmGetParentOfType( XtParent( w), theClass));
    }
}

/* get the parent form of a widget, and if
   there is no form, return the parent shell */ 

Widget AmGetForm( Widget   w)
{
    Widget      parent;

    if ( XtClass( w) == xmFormWidgetClass) {
        return( w);
    } else {
        parent = XtParent( w);
        if ( XtIsShell( parent)) {
            return( w);
        } else {
            return( AmGetForm( parent));
        }
    }
}


const char *
AmGetResourceName( Widget w)
{
    static char szResource[ RESBUFSIZE];
    AnvilBoolean     fFirst, fDone, fComplete;
    Widget      wTop;
    int         iIndex, iLength, iApplication, iNeeded;
    char        *szWidget;

/* Initializations. */

    iIndex = RESBUFSIZE - 1;
    szResource[ iIndex] = '\0';
    fFirst = TRUE;

/* Get the name of the top level widget. */

    wTop = AmGetTopLevelShell( w);
    if ( wTop == NULL) {
        return( NULL);
    }

    szWidget = XtName( wTop);
    iApplication = strlen( szWidget);

/*
 * Loop through all the wigets in the branch and prepend their
 *   name to the resource name until the top widget is reached,
 *   or the buffer is out of space.  Also make sure that the
 *   top level widget is always at the beginning of the resource
 *   name.  That means if the buffer is too short then the
 *   resource string will have the format:
 *
 *       Name( 0)*Name( N - M) ... Name( N - 2).Name( N - 1).Name( N)
 *
 *   Where N is the number of widgets in the branch and M is the
 *   number of resource names that fully fit into the buffer.
 */

    fDone     = FALSE;
    fComplete = FALSE;

    while( fDone == FALSE) {

        szWidget = XtName( w);
        iLength = strlen( szWidget);

        if ( XtIsShell( w)) {
            fDone = TRUE;
        }

        w = XtParent( w);
 
        if ( w == NULL) {

        /* The top level widget was reached. */

            fDone     = TRUE;
            fComplete = TRUE;

        } else {

        /* We are not at the top, so see if we have enough room. */

            iNeeded = iApplication + 1;

            if ( fFirst == FALSE) {
                iNeeded += 1;
            }

            if ( iIndex > iNeeded) {

            /* There is enough room for the widget name, so stuff it
               in there. */

                if ( fFirst == FALSE) {
                    iIndex--;
                    szResource[ iIndex] = '.';
                }

                iIndex -= iLength;
                memcpy( &szResource[ iIndex], szWidget, iLength);

            } else {

            /* There isn't enough room for the widget name, so prepend
               the top level widget's name and a '*'. */

                fDone     = TRUE;

            }

            fFirst = FALSE;

        }
    }


    if ( fFirst == FALSE) {
        iIndex--;

        if ( fComplete == TRUE) {
            szResource[ iIndex] = '.';
        } else {
            szResource[ iIndex] = '*';
        }
    }

/* Stuff the application name in there. */

    iIndex -= iApplication;
    memcpy( &szResource[ iIndex], XtName( wTop), iApplication);

    AN_DEBUG( 2, ( "Resource String: \"%s\".\n", &szResource[ iIndex]));

    return( &szResource[ iIndex]);
}
