#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/Label.h>
#include <anvil/strlib.h>
#include <anvil/motif.h>

/*
 * Function: AmDrawMultiLineString( pxDisplay, xWindow, xGC, x, y, sz)
 *
 * Purpose:
 *     Draws a sz on multiple lines.  The '\n' character is used
 *     to seperate the szs into the lines.
 *
 * Parameters:
 *     Display *pxDisplay : The xDisplay to xDisplay on.
 *     Window  xWindow    : The xWindowdow to draw on.
 *     GC      xGC        : The graphics context used for draxWindowg.
 *     int     x          : The upper left hand x coordinate. 
 *     int     y          : The upper lef hand y coordinate.
 *     char    *sz        : The sz to xDisplay.
 *
 */

void
AmDrawMultiLineString( Display *pxDisplay,
                       Window  xWindow,
                       GC      xGC,
                       int     x,
                       int     y,
                       char    *sz)
{
    char 	**vsz, **pc;
    XGCValues	xGCV;
    int		iDir, iAscent, iDescent;
    XCharStruct	xChar;

    XGetGCValues( pxDisplay, xGC, GCFont, &xGCV);

/* Chop the strings by newlines, and draw each string. */

    vsz = AmChopString( sz, '\n');
    for ( pc = vsz; *pc != NULL; pc++) {
	XQueryTextExtents( pxDisplay, xGCV.font, *pc, strlen( *pc),
			   &iDir, &iAscent, &iDescent, &xChar);
	XDrawImageString( pxDisplay, xWindow, xGC, x, y, *pc, strlen( *pc));
	y += iAscent + iDescent;
    }

/* Free the vector of strings when we're done with it. */

    AmFreeStrVec( vsz);
}


/* draw a fast sz in a widget */

void AmDrawMotifString( Display		*pxDisplay,
		      Window		xWindow,
		      GC		xGC,
		      XmFontList	fontList,
		      int		x,
		      int		y,
		      char		*sz)
{
    XmFontContext	context;
    XmFontListEntry	entry;
    XmFontType		fontType;
    XFontStruct		**fontStructs;
    Font		fid;
    char		**names;
    void		*ptr;

/* This cryptic piece of code was lifted from Douglas Young's Book on
   Motif.  I honestly don't really know what it does. */

    XmFontListInitFontContext( &context, fontList);
    entry = XmFontListNextEntry( context);
    ptr = XmFontListEntryGetFont( entry, &fontType);
    if ( fontType == XmFONT_IS_FONT) {
	fid = (( XFontStruct *) ptr)->fid;
    } else {
	XFontsOfFontSet(( XFontSet) ptr, &fontStructs, &names);
	fid = fontStructs[ 0]->fid;
    }
    XSetFont( pxDisplay, xGC, fid);
    XDrawImageString( pxDisplay, xWindow, xGC, x, y, sz, strlen( sz));

/* free the font context when we're done with it */

    XmFontListFreeFontContext( context);
}

/* get the font for a widget */

Font AmGetWidgetFont( Widget	w)
{
    XmFontContext	context;
    XmFontListEntry	entry;
    XmFontList		fontList;
    XFontStruct		**fontStructs;
    XmFontType		fontType;
    XrmDatabase		db;
    char		**names, fontName[ 256], *resType;
    XrmValue		xrmVal;
    void		*ptr;
    XGCValues		xGCV;

/* if the widget is a subclass of bulletinBoard, then we use the 
   XmNlabelFontList.  If the widget is a subclass of label, then we
   use the XmNfontList, otherwise, we return the default font for the 
   application */

    if ( XtIsSubclass( w, xmBulletinBoardWidgetClass)) {
	XtVaGetValues( w, XmNlabelFontList, &fontList, NULL);
    } else if ( XtIsSubclass( w, xmLabelWidgetClass)) {
	XtVaGetValues( w, XmNfontList, &fontList, NULL);
    } else {
	db = XtDatabase( XtDisplay( w));
	if ( db != NULL) {
	    if ( XrmGetResource( db, "*fontList", "", &resType, &xrmVal)) {
		strcpy( fontName, xrmVal.addr);
		return( XLoadFont( XtDisplay( w), fontName));
	    }
	}

/* no font resource in widget, and no font resource in application resource 
   database, just return the default GC's font. */

	XGetGCValues( XtDisplay( w), 
		      DefaultGC( XtDisplay( w), DefaultScreen( XtDisplay( w))),
		      GCFont, &xGCV);
	return( xGCV.font);
    }

/* otherwise, do that special Motif magic, which is documented in the Motif
   programmer's manual, but I don't understand for the life of me */

    XmFontListInitFontContext( &context, fontList);
    entry = XmFontListNextEntry( context);
    ptr = XmFontListEntryGetFont( entry, &fontType);
    if ( fontType == XmFONT_IS_FONT) {
	return((( XFontStruct *) ptr)->fid);
    } else {
	XFontsOfFontSet(( XFontSet) ptr, &fontStructs, &names);
	return( fontStructs[ 0]->fid);
    }
}

/* quickie label function */

Widget AmMakeLabel( Widget        parent,
		  char 		*str,
		  char		*name)
{
    XmString    mstr;
 
    mstr = XmStringCreateLtoR( str, XmFONTLIST_DEFAULT_TAG);
    return( XtVaCreateManagedWidget( name, xmLabelWidgetClass, parent,
				     XmNlabelString, mstr,
				     NULL));
}

