#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include <stdarg.h>
#include <Xm/Xm.h>
#include <Xm/FileSB.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <anvil/project.h>
#include <anvil/errlib.h>
#include <anvil/resource.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>

/* callback to save the directory and pattern from the file selection box */

static void saveDirPatternCallback( Widget	fileSb,
			         XtPointer,
			         XtPointer)
{
    XmString 	directoryStr, patternStr;
    char	*directory, *pattern, resourceStr[ RESLEN];

/* pull the strings from the file selection box and convert to C strings */

    XtVaGetValues( fileSb,
		   XmNdirectory, &directoryStr,
		   XmNpattern, &patternStr,
		   NULL);
    XmStringGetLtoR( directoryStr, XmFONTLIST_DEFAULT_TAG, &directory);
    XmStringGetLtoR( patternStr, XmFONTLIST_DEFAULT_TAG, &pattern);

/* save the directory and pattern strings */

    if (( pattern != NULL) && ( *pattern != '\0')) {
	sprintf( resourceStr, "%s.pattern", XtName( fileSb));
	AmPutDefault( resourceStr, pattern);
	if ( directory != NULL) {
	    XtFree( pattern);
	    XmStringFree( patternStr);
	}
    }
    if (( directory != NULL) && ( *directory != '\0')) {
	sprintf( resourceStr, "%s.directory", XtName( fileSb));
	AmPutDefault( resourceStr, directory);
	if ( directory != NULL) {
	    XtFree( directory);
	    XmStringFree( directoryStr);
	}
    }
}

/* callback to add elements to the destination list */

static void addListCallback( Widget			srcList,
			     Widget			dstList,
			     Widget			prefixList,
			     XmListCallbackStruct	*callData)
{
    XmString	*itemList;
    int		itemCount, ii;
    AnvilBoolean	alreadyThere;

    XtVaGetValues( dstList, 
		   XmNitems, &itemList,
		   XmNitemCount, &itemCount,
		   NULL);

/* don't add the string if it's already in the list */

    alreadyThere = FALSE;
    for ( ii = 0; ii < itemCount; ii++) {
	if ( XmStringCompare( itemList[ ii], callData->item)) {
	    alreadyThere = TRUE;
	}
    }
    if ( !alreadyThere) {
	XmListAddItem( dstList, callData->item, 0);
    }
}

/* delete elements from the selected file list */

static void deleteListCallback( Widget		deleteBtn,
				Widget		dstList,
				XtPointer	callData)
{
    XmString	*itemList;
    int		itemCount;

    XtVaGetValues( dstList, 
		   XmNselectedItems, &itemList,
		   XmNselectedItemCount, &itemCount,
		   NULL);
    XmListDeleteItems( dstList, itemList, itemCount);
}
			
/* multiple file selection box */

void AmMultipleFileSelectionBox( Widget	parent,
			       char	*name,
			       Widget	*list,
			       Widget	*form,
			       Arg	*formArgs,
			       int	nArgs)
{
    Widget	fileSel, fileList, deleteBtn;
    char	namebuf[ RESLEN];
    int		n;
    Arg		args[ 40];

    sprintf( namebuf, "%sForm", name);
    *form = XtCreateWidget( namebuf, xmFormWidgetClass, parent, 
			    formArgs, nArgs);

/* create file selection box. Get rid of the OK, cancel and help buttons */

    fileSel = AmMakeFileSelectionBox( *form, name, NULL, 0, TRUE, FALSE, NULL);
    XtVaSetValues( fileSel, 
		   StickLeft, StickTop, StickBottom,
		   XmNbottomOffset, 40,
		   NULL);
    fileList = XmFileSelectionBoxGetChild( fileSel, XmDIALOG_LIST);
    XtVaSetValues( fileList, XmNselectionPolicy, XmMULTIPLE_SELECT, NULL);
    XtUnmanageChild( XmFileSelectionBoxGetChild( fileSel, 
						 XmDIALOG_OK_BUTTON));
    XtUnmanageChild( XmFileSelectionBoxGetChild( fileSel, 
						 XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild( XmFileSelectionBoxGetChild( fileSel, 
						 XmDIALOG_HELP_BUTTON));
    XtManageChild( fileSel);

/* create the scrolled list of selected files */

    n = AmMakeArgList( args,
		     RightOf( fileSel),
		     StickTop, StickBottom,
		     XmNbottomOffset, 40,
		     XmNwidth, 300,
    		     XmNselectionPolicy, XmMULTIPLE_SELECT, 
		     NULL);
    sprintf( namebuf, "%sList", name);
    *list = XmCreateScrolledList( *form, namebuf, args, n);

/* and the button for deleting selected files */

    sprintf( namebuf, "%sDeleteBtn", name);
    deleteBtn = XtVaCreateManagedWidget( namebuf, xmPushButtonWidgetClass,
					 *form,
					 StickBottom,
					 LeftWith( *list),
					 NULL);
    AmAddVaCallback( fileList, XmNmultipleSelectionCallback,
		   ( VarargsCallback) addListCallback, *list, 
		   XmFileSelectionBoxGetChild( fileSel, XmDIALOG_DIR_LIST),
		   END_LIST);
    XtAddCallback( deleteBtn, XmNactivateCallback,
		   ( XtCallbackProc) deleteListCallback, *list);
    XtAddCallback( fileSel, XmNdestroyCallback,
		   ( XtCallbackProc) saveDirPatternCallback, fileSel);
    XtManageChild( *list);
    XtManageChild( *form);
}

/* multiple file selection dialog */

void AmMultipleFileSelectionDialog( Widget	parent,
				  char		*name,
				  Widget	*list,
				  Widget	*okBtn)
{
    Widget	cancelBtn, form, shell;
    char	namebuf[ RESLEN];
    Arg		args[ 20];
    int		n;

    sprintf( namebuf, "%sShell", name);
    n = AmMakeArgList( args,
		     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		     NULL);
    shell = XtCreatePopupShell( namebuf, xmDialogShellWidgetClass,
				parent, args, n);

/* see if the directory and pattern strings are stored in the resourse 
   database */

/* create the embedded file selection box */

    AmMultipleFileSelectionBox( shell, name, list, &form, NULL, 0);

/* create the ok and cancel buttons */

    sprintf( namebuf, "%sOkBtn", name);
    *okBtn = XtVaCreateManagedWidget( namebuf, xmPushButtonWidgetClass,	
				      form,
				      StickBottom,
				      RightWith( *list),
				      NULL);
    sprintf( namebuf, "%sCancelBtn", name);
    cancelBtn = XtVaCreateManagedWidget( namebuf, xmPushButtonWidgetClass,
					 form,
					 StickLeft, StickBottom,
					 NULL);
    XtAddCallback( cancelBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmDestroyShellCallback, shell);
    XtAddCallback( *okBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmDestroyShellCallback, shell);
    XtManageChild( shell);
}

/* call the user's OK callback if the file exists and readFlag is TRUE */

static void callUserCallback( 
	Widget					fileSb,
        XtCallbackProc				okCallback,
        int					readFlag,
	int					dialogFlag,
        AmListPtr					callbackArgList,
        XmFileSelectionBoxCallbackStruct	*cb)
{
    char 	*filename, newFile[ PATHLEN];

    if ( okCallback == NULL) {
	return;
    }
    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &filename);

/* if it's reading the file, check to make sure we can read it before calling
   the user's callback */

    if ( readFlag) {
	if ( access( filename, R_OK) == 0) {
	    callbackArgList = AmInsertNodeLst( okCallback, callbackArgList);
	    AmVaCbackWrapper( fileSb, callbackArgList, cb);
	    AmJustFreeLst( callbackArgList);
	    if ( dialogFlag) {
		XtDestroyWidget( XtParent( fileSb));
	    }
	} else {
	    AmPrintResourceError( "fileDoesNotExist", filename);
	}
    } else {
	if ( access( filename, F_OK) == 0) {
	    sprintf( newFile, "%s.bak", filename);
	    if ( rename( filename, newFile) != 0) {
		AmPrintResourceError( "failedToRename", filename);
		XtFree( filename);
		return;
	    }
	}
	callbackArgList = AmInsertNodeLst( okCallback, callbackArgList);
	AmVaCbackWrapper( fileSb, callbackArgList, cb);
	AmJustFreeLst( callbackArgList);
	XtDestroyWidget( XtParent( fileSb));
    }

/* free the temporary filename */

    XtFree( filename);
}

/* create a file selection dialog which retains the path name used for
   selecting the file for the next instantiation.  It also automatically
   adds the popdown callback */

static Widget makeVaFileSelectionBox( Widget		parent,
				      char		*resourceName,
				      Arg		*args,
				      int		numArgs,
				      AnvilBoolean		readFlag,
				      AnvilBoolean		dialogFlag,
				      VarargsCallback	okCallback,
				      va_list		callbackArgs)
{
    Widget	fileSb;
    char	resourceStr[ RESLEN];
    const char  *directory, *pattern;
    XmString	patternStr = NULL, directoryStr = NULL;
    AmListPtr	callbackArgList;
    Arg		tempArgs[ 100];

    memcpy( tempArgs, args, numArgs*sizeof( Arg));

/* get the list of callback arguments */

    if ( okCallback != NULL) {
	callbackArgList = AmVaArgList( callbackArgs);
    } else {
	callbackArgList = NULL;
    }

/* see if the directory and pattern strings are stored in the resourse 
   database */

    sprintf( resourceStr, "%s.directory", resourceName);
    directory = AmGetDefault( resourceStr, ( const char *) NULL);
    sprintf( resourceStr, "%s.pattern", resourceName);
    pattern = AmGetDefault( resourceStr, ( const char *) NULL);

/* if the directory isn't NULL... */

    if ( directory != NULL) {
AN_DEBUG( 70, ( "directory = %s\n", directory));
	directoryStr = XmStringCreateLtoR(( char *) directory, 
					  XmFONTLIST_DEFAULT_TAG);
	XtSetArg( tempArgs[ numArgs], XmNdirectory, directoryStr); numArgs++;
    }

/* and the if the pattern isn't NULL... */

    if ( pattern != NULL) {
AN_DEBUG( 70, ( "pattern = %s\n", pattern));
	patternStr = XmStringCreateLtoR(( char *) pattern, 
					XmFONTLIST_DEFAULT_TAG);
	XtSetArg( tempArgs[ numArgs], XmNpattern, patternStr); numArgs++;
    } 

/* and if the search proc isn't NULL */

    fileSb = XmCreateFileSelectionBox( parent, resourceName, tempArgs, numArgs);
    if ( directory != NULL) {
	XmStringFree( directoryStr);
    } 
    if ( pattern != NULL) {
	XmStringFree( patternStr);
    }

/* add a callback to save the AmLast directory and pattern used, */

    XtAddCallback( fileSb, XmNokCallback,
		   ( XtCallbackProc) saveDirPatternCallback, fileSb);
    AmAddVaCallback( fileSb, XmNokCallback,
		   ( VarargsCallback) callUserCallback,
		   okCallback, ( int) readFlag, ( int) dialogFlag,
		   callbackArgList, END_LIST);
    XtManageChild( fileSb);
    return( fileSb);
}

/****************************************************************************
 *
 * Description:
 *     	make a file selection dialog which uses the resource database to
 *	remember the AmLast used directory, and automatically checks for the
 *	accessibility of a file if requested 
 * Arguments:
 *	parent: parent of the file selection box.
 *	resourceName: preferably unique string identifier to save the AmLast
 *		      directory path under.
 *	args: XtArgVals to pass as resources to the file selection box. 
 *	      (may be NULL)
 *	numArgs: number of arguments to pass.
 *	okCallback: varargs callback to pass for XmNokCallback.
 *	varargs...: arguments to pass to the varargs callback, terminated
 *		    with END_LIST.
 * Returns:
 *	The file selection box child of a Dialog Shell parent.  The cancel
 *	callback is preset to destroy down the file selection dialog.
 *
 *****************************************************************************/

Widget AmMakeFileSelectionDialog( Widget		parent,
			        char		*resourceName,
			        Arg		*args,
			        int		numArgs,
			        AnvilBoolean		readFlag,
			        VarargsCallback	okCallback,
			        ...)
{
    int		numShellArgs;
    Arg		shellArgs[ 20];
    va_list	callbackArgs;
    Widget	fileSb, shell;

AN_DEBUG( 70, ("calling AmMakeFileSelectionDialog %s\n", resourceName));

/* Create the dialog shell, and reset the parent */

    numShellArgs = 
	AmMakeArgList( shellArgs,
		     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		     NULL);
    shell = XtCreatePopupShell( resourceName, xmDialogShellWidgetClass,
				 parent, shellArgs, numShellArgs);

/* if the callback is not NULL, then parse the callback argument list */

    if ( okCallback != NULL) {
	va_start( callbackArgs, okCallback);
	fileSb = makeVaFileSelectionBox( shell, resourceName, args, numArgs,
					 readFlag, TRUE, 
					 okCallback, callbackArgs);
	va_end( callbackArgs);
    } else {
	fileSb = makeVaFileSelectionBox( shell, resourceName, args, numArgs,
					 readFlag, TRUE, 
					 okCallback, NULL);
    }

/* add the callback to pop down the dialog shell */

    XtAddCallback( fileSb, XmNcancelCallback,
		   ( XtCallbackProc) AmDestroyShellCallback, shell);
    XtManageChild( shell);
    return( fileSb);
}

/****************************************************************************
 *
 * Description:
 *     	make a file selection box which uses the resource database to
 *	remember the AmLast used directory, and automatically checks for the
 *	accessibility of a file if requested 
 * Arguments:
 *	parent: parent of the file selection box.
 *	resourceName: preferably unique string identifier to save the AmLast
 *		      directory path under.
 *	args: XtArgVals to pass as resources to the file selection box. 
 *	      (may be NULL)
 *	numArgs: number of arguments to pass.
 *	okCallback: varargs callback to pass for XmNokCallback.
 *	varargs...: arguments to pass to the varargs callback, terminated
 *		    with END_LIST.
 * Returns:
 *	The file selection box 
 *
 *****************************************************************************/

Widget AmMakeFileSelectionBox( Widget		parent,
			     char		*resourceName,
			     Arg		*args,
			     int		numArgs,
			     AnvilBoolean		readFlag,
			     VarargsCallback	okCallback,
			     ...)
{
    va_list	callbackArgs;
    Widget	fileSb;

    if ( okCallback != NULL) {
	va_start( callbackArgs, okCallback);
	fileSb = makeVaFileSelectionBox( parent, resourceName, args, numArgs,
					 readFlag, FALSE, 
					 okCallback, callbackArgs);
	va_end( callbackArgs);
    } else {
	fileSb = makeVaFileSelectionBox( parent, resourceName, args, numArgs,
					 readFlag, FALSE, 
					 okCallback, NULL);
    }
    return( fileSb);
}
