#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>

#include <anvil/record.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/macros.h>
#include <anvil/files.h>
#include <anvil/macros.h>
#include <anvil/fns.h>
#include <anvil/resource.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>

#include <anvil/choppath.h>
#include <anvil/slick.h>
#include <anvil/xpm.h>

#include <anvil/option.h>

/* Structure that holds information pertinent to option menus. */

typedef struct optionMenuRec_ {
    Widget widget;
    Widget *buttons;
    int    pos; 
} optionMenuRec, *optionMenuPtr;

/* Callback for the buttons within an option menu.  Sends a user event
   indicating that the value of the option menu changed. */

static void
optionMenuButtonCallback( Widget        button,
                          optionMenuPtr menu,
                          XtPointer)
{
    int    ii;

/* Find out which button in the option menu was pressed. */

    for ( ii = 0; menu->buttons[ ii] != NULL; ii++) {
        if ( menu->buttons[ ii] == button) {
            break;
        }
    }

    if ( menu->buttons[ ii] != NULL) { 

    /* Send a user event indicating that the option menu changed its value. */

        AmSendUserEventChildren( AM_PKG_ID, AM_OPTION_CHANGED,  
            ( void *)ii, menu->widget);

    }
}

/* Callback for when the option menu gets destroyed.  Frees any memory
   that was used up internally. */

static void
optionMenuDestroyCallback( Widget        ,
                           optionMenuPtr menu,
                           XtPointer)
{

/* Free the allocated memory. */

    free( menu->buttons);
    free( menu);
}

/*******************************************************************
 *
 * Function: AmCreateOptionMenu( parent, name, labels, pos)
 *
 * Paramters:    
 *     Widget     parent      - Parent of the option menu.
 *     const char *name       - Name of the option menu.
 *     const char **labels    - String values to be used in the option menu.
 *     int        pos         - Option menu active value. 
 *
 * Description:
 *
 *******************************************************************/

Widget
AmCreateOptionMenu( Widget     parent,
                  const char *name,
                  const char **labels,
                  int        pos)
{
    optionMenuPtr info;
    Widget        menu, optionMenu, *buttons;
    XmString      motifString;
    int           length, ii;

    if ( !(info = ( optionMenuPtr) malloc( sizeof( optionMenuRec)))) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( 0);
    }

    length = AmVeclen( labels); 
    if ( !(buttons = ( Widget *)malloc( sizeof( Widget) * ( length + 1)))) {
        free( info);
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( 0);
    }

    info->buttons = buttons;

/* Create the option menu. */

    menu = XmCreatePulldownMenu( parent, "menu", NULL, 0);

/* Add each field to the possible selections. */

    for ( ii = 0; ii < length; ii ++) {
        motifString = XmStringCreate( ( char *)labels[ ii],
                          XmFONTLIST_DEFAULT_TAG); 
        buttons[ ii] = XtVaCreateManagedWidget( "button",
                           xmPushButtonWidgetClass, menu,
                           XmNlabelString, motifString,
                           NULL);
        XmStringFree( motifString);

        XtAddCallback( buttons[ ii],  XmNactivateCallback, 
            ( XtCallbackProc) optionMenuButtonCallback, 
            info);
    }

/* Set the AmLast button to be NULL to indicate the end of the list. */

    buttons[ ii] = NULL;

    optionMenu = XtVaCreateManagedWidget( name,
        xmRowColumnWidgetClass, parent,
        XmNrowColumnType, XmMENU_OPTION,
        XmNsubMenuId, menu,
        XmNadjustLast, True,
        NULL);

    info->widget = optionMenu;

/* Set the selected option. */

    if ( pos < 0 || pos >= length) {
        pos = 0;
    }

    XtVaSetValues( optionMenu,
        XmNmenuHistory, buttons[ pos],
        NULL);

    info->pos = pos;

/* Don't want to see the label that is generated by the motif library. */

    XtUnmanageChild( XmOptionLabelGadget( optionMenu));

    XtAddCallback( optionMenu, XmNdestroyCallback,
        ( XtCallbackProc) optionMenuDestroyCallback,
        ( XtPointer) info);

    return( optionMenu);
}
