#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <malloc.h>
#include <memory.h>
#include <anvil/project.h>
#include <anvil/strlib.h>
#include <anvil/getcolor.h>

/* file format 

   colors can be specified either as names hex values or as RGB 0..255 values

    numColors
    lowest color
      .
      . 
    color
      .
    highest color
*/

AnvilBoolean readAllocShadeColors( 
	Display 	*display,	/* connection to X server */
	char 		*colorfile,	/* color file to read */
	Colormap 	cmap,		/* colormap to read colors into */
	int		numShades,	/* number of shades for each color */
	int 		numColors,	/* number of colors requested */
	int		*numAlloced,	/* total # of colors allocated */
	XColor 		**colorRet)	/* vector of XColors returned */
{
    int 	ii, jj, numReadColors;
    XColor 	*colorPtr, *shadePtr, *colorVec, *sizeColorVec;

    if ( !AmReadColors( display, colorfile, &numReadColors, &colorVec)) {
        return( FALSE);
    }

/* interpolate out the colors to something more reasonable */

    interpolateXcolors( colorVec, numReadColors, &sizeColorVec, numColors);
    *colorRet = ( XColor *) malloc( sizeof( XColor)*numColors*numShades);
    colorPtr = *colorRet + ( numShades - 1)*numColors;
    memcpy(( void *) colorPtr,
	   ( void *) sizeColorVec, numColors*sizeof( XColor));
    FREE( sizeColorVec);

/* now allocate shades for each color. walk down the original color array */

		 
    shadePtr = *colorRet; 
    for ( ii = 0; ii < numShades - 1; ii++) {
	for ( jj = 0; jj < numColors; jj++) {

/* shade each color */

	    shadePtr->red = ( colorPtr[ jj].red*( ii + 1))/numShades;
	    shadePtr->green = ( colorPtr[ jj].green*( ii + 1))/numShades;
	    shadePtr->blue = ( colorPtr[ jj].blue*( ii + 1))/numShades;

/* if he can't allocate the color, then bogue out has to be even for shading 
   algorithms to work */
  
	    shadePtr++;
	}
    }

/* now allocate the colors */

    for ( ii = 0, *numAlloced = 0, shadePtr = *colorRet; 
	  ii < numColors*numShades; 
	  ii++,( *numAlloced)++, shadePtr++) {
	if ( !XAllocColor( display, cmap, shadePtr)) {
	    return( FALSE);
	} 
    }
    return( TRUE);
}

/* given a list of colors, allocate numShades of those colors */

int allocShades( Display	*display,
		 Colormap	cmap,
		 XColor		*colors,
		 int		numColors,
		 int		numShades,
		 XColor		**colorRet,
		 int		*numAlloced)
{
    XColor		*shadePtr, *colorPtr;
    int			ii, jj, success;

/* allocate the array of shades and copy the colors to it. */

    *colorRet = ( XColor *) malloc( sizeof( XColor)*numColors*numShades);

/* now allocate shades for each color and walk down the original color array */

    memcpy(( void *) &((*colorRet)[ ( numShades - 1)*numColors]),
	   ( void *) colors, numColors*sizeof( XColor)); 
    do {
	success = TRUE;
	shadePtr = *colorRet;
	for ( ii = 1; ii < numShades; ii++) {
	    for ( jj = 0, colorPtr = colors; 
		  jj < numColors; 
		  jj++, shadePtr++, colorPtr++) {

/* shade each color */

		shadePtr->red = ( colorPtr->red*ii)/numShades;
		shadePtr->green = ( colorPtr->green*ii)/numShades;
		shadePtr->blue = ( colorPtr->blue*ii)/numShades;
	    }
	}
	for ( ii = 0, shadePtr = *colorRet, *numAlloced = 0;
	      ii < numColors*numShades; 
	      ii++, shadePtr++, (*numAlloced)++) {

/* if he can't allocate the color, put in space anyway, since the spacing
   has to be even for shading algorithms to work */
  
	    if ( !XAllocColor( display, cmap, shadePtr)) {

/* free all of the colors in this shade, and try again with less shades */ 

		success = FALSE;
		if ( numShades > 1) {
		    if ( *numAlloced > 0) {
			freeColors( display, cmap, *colorRet, *numAlloced);
		    }
		    numShades--;
		} else {
		    return( FALSE);		/* give up */
		}
	    } 
	}
    } while ( !success);
    return( TRUE);
}
