#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <Xm/Xm.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>
#include <anvil/motif.h>
#include <anvil/defaults.h>


/* private data structure to keep track of the scrolled window and the timerId
   for the scroller */

typedef struct scrollClientData_ {
    Widget		scrollWin;
    XtIntervalId	scrollTimerId;
} scrollClientDataRec, *scrollClientDataPtr;

/* timeout gets called every X milliseconds and tests how much the cursor is
   outside of the scrolled window, checks if the work window can still be scrolled
   and sets the scrollbars.  Interestingly enough, changing the X,Y of the 
   work area has no effect. */

static void scrollTimeout( scrollClientDataPtr	scrollClientData,
			   XtIntervalId		*id)
{
    Window	root, child;
    int		rootX, rootY, winX, winY, xscroll, yscroll, value, sliderSize, 
		increment, pageIncrement, scrollMin, scrollMax;
    uint_t	mask;
    Dimension	scrollWidth, scrollHeight;
    Position	scrollX, scrollY;
    Widget	workWindow, hScroll, vScroll;

/* where are we relative to the window we're scrolling, anyway? */

    XQueryPointer( XtDisplay( scrollClientData->scrollWin), 
		   XtWindow( scrollClientData->scrollWin),
		   &root, &child, &rootX, &rootY, &winX, &winY, &mask);
    XtVaGetValues( scrollClientData->scrollWin,
		   XmNwidth, &scrollWidth,
		   XmNheight, &scrollHeight,
		   XmNworkWindow, &workWindow,
		   XmNhorizontalScrollBar, &hScroll,
		   XmNverticalScrollBar, &vScroll,
		   NULL);
    XtTranslateCoords( scrollClientData->scrollWin, 0, 0, &scrollX, &scrollY);

/* find out where the current mouse position is relative to scrolled window */

    if ( rootX < scrollX) {
	xscroll = rootX - scrollX;
    } else if ( rootX > scrollX + scrollWidth) {
	xscroll = rootX - ( scrollX + scrollWidth);
    } else {
	xscroll = 0;
    }
    if ( rootY < scrollY) {
	yscroll = rootY - scrollY;
    } else if ( rootY > scrollY + scrollHeight) {
	yscroll = rootY - ( scrollY + scrollHeight);
    } else {
	yscroll = 0;
    }

/* set the values in the scrollbars and cause the notifications.  Make sure
   that the scrollbar values aren't out of range */

    XmScrollBarGetValues( hScroll, &value, &sliderSize, 
			  &increment, &pageIncrement);
    XtVaGetValues( hScroll, 
		   XmNminimum, &scrollMin,
		   XmNmaximum, &scrollMax,
		   NULL);
    if ( value + xscroll < 0) {
	value = 0;
    } else if ( value + xscroll + sliderSize > scrollMax) {
	value = scrollMax - sliderSize;
    } else {
	value += xscroll;
    }
    XmScrollBarSetValues( hScroll, value, sliderSize, 
			  increment, pageIncrement, TRUE);

/* do the same thing for the vertical scrollbar */

    XmScrollBarGetValues( vScroll, &value, &sliderSize, 
			  &increment, &pageIncrement);
    XtVaGetValues( vScroll, 
		   XmNminimum, &scrollMin,
		   XmNmaximum, &scrollMax,
		   NULL);
    if ( value + yscroll < 0) {
	value = 0;
    } else if ( value + yscroll + sliderSize > scrollMax) {
	value = scrollMax - sliderSize;
    } else {
	value += yscroll;
    }
    XmScrollBarSetValues( vScroll, value , sliderSize, 
			  increment, pageIncrement, TRUE);

/* add the timer proc again */

    scrollClientData->scrollTimerId =
	XtAppAddTimeOut( 
	    XtWidgetToApplicationContext( scrollClientData->scrollWin),
	    AmSCROLL_TIMEOUT, ( XtTimerCallbackProc) scrollTimeout,
	    scrollClientData);
}

/* remove the timeout when the window is destroyed */

static void killScrollTimeoutCallback( Widget		   area,
				       scrollClientDataPtr scrollClientData,
				       XtPointer)
{
    if ( scrollClientData->scrollTimerId != 0) {
	XtRemoveTimeOut( scrollClientData->scrollTimerId);
    }
    free( scrollClientData);
}

/* remove this event handler and the timerproc for the next scroll on a 
   button release. Also remove the function which kills the timeout on
   the widget destroy */

static void stopScrollButtonHandler( Widget		 area,
				     scrollClientDataPtr scrollClientData,
				     XButtonEvent	 *event,
				     AnvilBoolean		 *continueDispatch)
{
    XtRemoveEventHandler( area, ButtonReleaseMask | EnterWindowMask, FALSE,
			  ( XtEventHandler) stopScrollButtonHandler, 
			  scrollClientData);
    XtRemoveTimeOut( scrollClientData->scrollTimerId);
    scrollClientData->scrollTimerId = 0;
    XtRemoveCallback( area, XmNdestroyCallback,
		      ( XtCallbackProc) killScrollTimeoutCallback, 
		      scrollClientData);
    free( scrollClientData);
}
  
/* scrolled window button handler to scroll the work window when the user
   drags the mouse outside with the button depressed */

void AmScrollButtonHandler( Widget		areaWidget,
			  XtPointer,
			  XLeaveWindowEvent	*event,
			  AnvilBoolean *)
{
    if (( event->state & Button1Mask) != 0) {
	scrollClientDataPtr	scrollClientData;

/* add a timeout to scroll the drawing area, and a button release event handler
   to stop scrolling */

	scrollClientData = ( scrollClientDataPtr)
	    malloc( sizeof( scrollClientDataRec));
	scrollClientData->scrollWin = 
	    AmGetParentOfType( areaWidget, xmScrolledWindowWidgetClass);
	scrollClientData->scrollTimerId =
	    XtAppAddTimeOut( XtWidgetToApplicationContext( areaWidget),
			     AmSCROLL_TIMEOUT, 
			     ( XtTimerCallbackProc) scrollTimeout,
			     scrollClientData);

/* set up a callback to remove the timeout when the window is destroyed */

	XtAddCallback( areaWidget, XmNdestroyCallback,
		       ( XtCallbackProc) killScrollTimeoutCallback, 
		       scrollClientData);
	XtAddEventHandler( areaWidget, ButtonReleaseMask | EnterWindowMask, 
			   FALSE, ( XtEventHandler) stopScrollButtonHandler, 
			   scrollClientData);
    }
}

