#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <ctype.h>
#include <Xm/Xm.h>
#include <Xm/ToggleB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/resource.h>

static void removeSensitiveCback( Widget	btn,	
				  Widget	dlog,
				  XtPointer	callData);

/* set the sensitivity of a widget from a toggle's state */

void AmToggleSensitiveCallback( Widget	w,
			      Widget	dst,
			      XtPointer)
{
    XtSetSensitive( dst, XmToggleButtonGetState( w));
}

void AmToggleEditableCallback( Widget	tgl,
			     Widget	text,
			     XtPointer)
{
    XtVaSetValues( text, XmNeditable, XmToggleButtonGetState( tgl), NULL);
}

static void setSensitiveCback( Widget		dlog,	
			       Widget		btn,
			       XtPointer	callData)
{
    if ( dlog == NULL ) return;
    if ( btn  == NULL ) return;

    XtSetSensitive( btn, True);
    XtRemoveCallback( btn, XmNdestroyCallback,
		      ( XtCallbackProc) removeSensitiveCback, dlog);
}


static void removeSensitiveCback( Widget	btn,	
				  Widget	dlog,
				  XtPointer	callData)
{
    if ( btn  == NULL ) return;
    if ( dlog == NULL ) return;

    XtRemoveCallback( dlog, XmNdestroyCallback,
		      ( XtCallbackProc) setSensitiveCback, btn);
}

/* control a button's sensitivity when a dialog is popped up and down */

void AmSetSensitiveDestroy( Widget	btn,
			  Widget	dlog)
{
    if ( btn  == NULL ) return;
    if ( dlog == NULL ) return;

    XtSetSensitive( btn, FALSE);
    XtAddCallback( btn, XmNdestroyCallback, 
		   ( XtCallbackProc) removeSensitiveCback, dlog);
    XtAddCallback( dlog, XmNdestroyCallback,
		   ( XtCallbackProc) setSensitiveCback, btn);
}

/* turn a toggle off if this toggle is on */

void AmToggleOffCallback( Widget	tgl,
			Widget	otherTgl,
			XtPointer)
{
    if ( XmToggleButtonGetState( tgl)) {
	if ( XmToggleButtonGetState( otherTgl)) {
	    XmToggleButtonSetState( otherTgl, FALSE, TRUE);
	}
    }
}

/* take a set of toggles and enforce a radiobox behavior */

void AmRadioizeToggles( Widget    *toggles,
                      int       nToggles)
{
    int	ii, jj;

    for ( ii = 0; ii < nToggles; ii++) {
	for ( jj = 0; jj < nToggles; jj++) {
	    if ( ii != jj) {
		XtAddCallback( toggles[ ii], XmNvalueChangedCallback,
			       ( XtCallbackProc) AmToggleOffCallback,
			       toggles[ jj]);
	    }
	}
    }
}

/* raise the dialog if the button is pressed */

static void raiseCallback( Widget	btn,
			   Widget	dlog,
			   XtPointer)
{
    XRaiseWindow( XtDisplay( dlog), XtWindow( dlog));
}

/* remove the raise callback when the button is destroyed */

static void removeRaiseCallback( Widget	dlog,
				 Widget	btn,
				 XtPointer)
{
    XtRemoveCallback( btn, XmNactivateCallback,
		      ( XtCallbackProc) raiseCallback, dlog);
}

/* set the old background and free the pixel for the new one */

static void removeRaiseColorCallback( Widget		dlog,
				      Widget		btn,
				      unsigned long	oldBg,
				      unsigned long	newBg,
				      XtPointer)
{
    Colormap	cmap;

    XtVaGetValues( btn, XmNcolormap, &cmap, NULL);
    XtVaSetValues( btn, XmNbackground, oldBg, NULL);
}

/* set a raise callback which recolors the button and brings up the dialog
   when the button is pressed again */

void AmSetRaiseCallback( Widget	btn,
		       Widget	dlog)
{
    unsigned long	oldBg;
    const char		*newColor;
    XColor		screen, exact;
    Colormap		cmap;

    newColor = AmGetDefault( "raise.background", "black");
    XtVaGetValues( btn, 
		   XmNbackground, &oldBg, 
		   XmNcolormap, &cmap,
		   NULL);
    XtAddCallback( btn, XmNactivateCallback,
		   ( XtCallbackProc) raiseCallback, dlog);
    XtAddCallback( dlog, XmNdestroyCallback,
		   ( XtCallbackProc) removeRaiseCallback, btn);
    if ( XAllocNamedColor( XtDisplay( btn), cmap, newColor, &screen, &exact)) {
	XtVaSetValues( btn, XmNbackground, screen.pixel, NULL);
	AmAddVaCallback( dlog, XmNdestroyCallback,
		       ( VarargsCallback) removeRaiseColorCallback, 
		       btn, oldBg, screen.pixel, END_LIST);
    }
}

/* for semi-exclusive toggles */

void AmUnsetToggleCallback( Widget	tgl,
			  Widget	otherTgl,
			  XtPointer)
{
    if ( XmToggleButtonGetState( tgl) && XmToggleButtonGetState( otherTgl)) {
	XmToggleButtonSetState( otherTgl, FALSE, TRUE);
    }
}

/* make text entry widget convert all characters to upper case... */

static void upperCallback( Widget			w,
			   XtPointer			clientData,
			   XmTextVerifyCallbackStruct	*cb)
{
    char	*insertStr = cb->text->ptr;

    if ( insertStr == NULL) {
	return;
    }
    for ( ; *insertStr != '\0'; insertStr++) {
	*insertStr = toupper( *insertStr);
    }
    cb->doit = TRUE;
}

void AmUpperText( Widget		text)
{
    if (( !XtIsSubclass( text, xmTextWidgetClass)) &&
        ( !XtIsSubclass( text, xmTextFieldWidgetClass))) {
	AN_DEBUG( 10, ("AmUpperText: not a text widget\n"));
	return;
    }
    XtAddCallback( text, XmNmodifyVerifyCallback,
		   ( XtCallbackProc) upperCallback, NULL);
}
