#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Scale.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/FileSB.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>

#include <anvil/record.h>
#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/macros.h>
#include <anvil/files.h>
#include <anvil/macros.h>
#include <anvil/fns.h>
#include <anvil/plot.h>
#include <anvil/resource.h>
#include <anvil/textcheck.h>
#include <anvil/userevents.h>
#include <anvil/mem_funcs.h>
#include <anvil/units.h>
#include <anvil/colorsel.h>
#include <anvil/choppath.h>
#include <anvil/getset.h>
#include <anvil/xpm.h>
#include <anvil/enums.h>
#include <anvil/slick.h>
#include <anvil/slickP.h>

/**************
 * Constants: *
 **************/

/*********************************************************************
 *
 * Support functions:
 *     These functions are used in general and are not directly tied
 *     to SLICK.
 *
 *********************************************************************/

/*********************************************************************
 *
 * Description:
 *     This function allocates memory for the data structure to be
 *     passed around on an AmOK user event.
 *
 *********************************************************************/

AmOkArgsPtr
AmMakeOkArgs( AmRecordClass *record,
            AmTagValuePtr tagValues, 
            int         numValues)
{
    AmOkArgsPtr okArgs;

    if (!(okArgs = ( AmOkArgsPtr)malloc( sizeof( AmOkArgsRec)))) {
        return( NULL);
    }
    okArgs->record = record;
    okArgs->tagValues = tagValues;
    okArgs->numTags = numValues;
    return( okArgs);
}

/*********************************************************************
 *
 * Description:
 *     Given a void pointer value, a type, a string tag, and client
 *     data that contains a tag value array, the number of tags, the
 *     function fills the next slot in the tag value array with the
 *     passed in value.  It deals with headers by expanding them out 
 *     into multiple tag values.
 *
 *********************************************************************/

AnvilBoolean
AmSetTagValue( 
    void         *value,
    AmDataTypeEnum type,
    char         *tag,
    AmOkArgsPtr    okArgs)
{
    AmTagValueRec headerValues[ MAXTAGS];
    int         headerCount, ii;

    AmTagValuePtr tagValues = okArgs->tagValues;
    int         numTags = okArgs->numTags;

    if ( numTags >= MAXTAGS) {
        return( FALSE);
    }

/* if the value is a header, then expand out the tag-values inline.  
   bail out if the resulting tags are longer than we can handle */

    if ( AmMatchTag( tag, AmNheader)) {
        headerCount = AmHeaderTagValueArray( ( char *)value, headerValues);
        if ( numTags + headerCount >= MAXTAGS) {
            for ( ii = 0; ii < headerCount; ii++) {
                if ( headerValues[ ii].tagCopied) {
                    free( headerValues[ ii].tag);
                }
                if ( headerValues[ ii].copied) {
                    free( headerValues[ ii].value);
                }
            }
            return( FALSE);
        }
        memcpy(( void *) &tagValues[ numTags], ( void *)headerValues,
            headerCount*sizeof( AmTagValueRec));
        okArgs->numTags += headerCount;
        return( TRUE);
    }

    tagValues[ numTags].tag = tag;
    tagValues[ numTags].tagCopied = FALSE;
    tagValues[ numTags].copied = FALSE;

/* set the value in the tagValues array element.  I think that a lot of the
   entries can be eliminated, because they're done in void * setting anyway */

    switch( type) {
        case AmCHAR:
            tagValues[ numTags].value = ( void *)*( char *)value;
            break;
        case AmUNSIGNED_CHAR:
            tagValues[ numTags].value = ( void *)*( uchar_t *)value;
            break;
        case AmSHORT:
            tagValues[ numTags].value = ( void *)*( short *)value;
            break;
        case AmUNSIGNED_SHORT:
            tagValues[ numTags].value = ( void *)*( ushort_t *)value;
            break;
        case AmINT:
        case AmUNITS:
        case AmVARIABLE:
            tagValues[ numTags].value = ( void *)*( int *)value;
            break;
        case AmUNSIGNED_INT:
            tagValues[ numTags].value = ( void *)*( uint_t *)value;
            break;
        case AmLONG:
            tagValues[ numTags].value = ( void *)*( long *)value;
            break;
        case AmUNSIGNED_LONG:
            tagValues[ numTags].value = ( void *)*( ulong_t *)value;
            break;
        case AmFLOAT:
	    memcpy( &tagValues[ numTags].value, value, sizeof( float)); 
            break;
	case AmFLOAT_PTR:
	    tagValues[ numTags].value = ( void *) value;
	    break;
        case AmDOUBLE:
            tagValues[ numTags].value = ( void *)value;
            break;
        case AmBOOLEAN:
            tagValues[ numTags].value = ( void *)*( AnvilBoolean *)value;
            break;
        case AmSTRING:
            tagValues[ numTags].value = strdup(( char *) value);
            tagValues[ numTags].copied = TRUE;
            break;
        case AmFILE:
            tagValues[ numTags].value = 
                ( void *)strdup( AmFileEnvPath( ( char *)value));
            tagValues[ numTags].copied = TRUE;
            break;
        case AmENUM:
            tagValues[ numTags].value = ( void *)*( int *)value;
            break;
        case AmCOLOR_CELL:
            tagValues[ numTags].value = ( void *)malloc( sizeof( AmColorCellRec));
	    if ( tagValues[ numTags].value == NULL) {
		AN_DEBUG( 1, ("malloc failed! tag =%s\n", 
			       tagValues[ numTags].tag));
		DO_ERROR( AmMALLOC_FAILED);
	    }
            memcpy( tagValues[ numTags].value, value, sizeof( AmColorCellRec));
            tagValues[ numTags].copied = TRUE;
            break;
        default:
            AN_DEBUG( 30, ("data type %d not found\n", type));
            return( FALSE);
    }

    okArgs->numTags++;
    return ( TRUE);
}

/*********************************************************************
 *
 * Description:
 *     Copies the value out of the slick data structure into another
 *     value... memory is allocated for the other value if needed.
 *
 *********************************************************************/

void
AmCopyValue( AmSlickPtr clientData, void *value)
{
    AmDataTypeEnum type = clientData->type;

/*
 * Strings and files (which are just another form of string) get dealt
 *   differently, since they are a pointers in themselves.
 */

    clientData->size = AmSizeData( value, type);

/* does not apply to records */

    if ( clientData->record != NULL) {
	return;
    }

/* if the type is string, or file, then the size might change, so we 
   reallocate the buffer if we need to */
  
    if ( type == AmSTRING || type == AmFILE) {
        if ( clientData->storageType == AmDYNAMIC) { 

/* if the storage type is dynamic, free the old pointer (if it isn't NULL),
   and allocate a new one, then copy the data */

            if (( clientData->valuePtr != NULL)  && 
		( *( char **) clientData->valuePtr != NULL)) {
                free( *(char **) clientData->valuePtr);
            }
            *(char **) clientData->valuePtr = 
		( char *)malloc( clientData->size);
	    if ( *(char **) clientData->valuePtr == NULL) {
		DO_ERROR( AmMALLOC_FAILED);
		return;
	    }
	    memcpy( *(char **) clientData->valuePtr, value, 
		    clientData->size);
        } else if ( clientData->storageType == AmSTATIC) {

/* if it's static, just copy the data.  We should store the buffer length
   so it gets checked when the data is copied to ensure that we don't
   overflow */

	    memcpy((char *) clientData->valuePtr, value, 
		    clientData->size);
	} else if ( clientData->storageType == AmLOCAL) {

/* if it's local, then free our temporary buffer, then allocate a new one */

	    if ( clientData->valuePtr != NULL) {
		free( clientData->valuePtr);
	    } 
	    clientData->valuePtr = 
		( char *) malloc( clientData->size);
	    if ( *(char **) clientData->valuePtr == NULL) {
		DO_ERROR( AmMALLOC_FAILED);
		return;
	    }
	    memcpy((char *) clientData->valuePtr, value, 
		   clientData->size);
	}
    } else {
        memcpy( clientData->valuePtr, value, clientData->size);
    }
}

/*********************************************************************
 *
 * Description:
 *     Returns true if the two passed in values (which have to be
 *     of the same type) are AmEquals, otherwise it returns false. 
 *
 *********************************************************************/

static AnvilBoolean
valuesEqual( void *oldValue, void *newValue, AmDataTypeEnum type)
{
    int size;

    if ( oldValue == NULL && newValue == NULL) {
        return( TRUE);
    } else if ( oldValue == NULL || newValue == NULL) {
        return( FALSE);
    } else if ( type == AmSTRING || type == AmFILE) {
        return( !strcmp( ( char *)oldValue, ( char *)newValue));
    } else {
        size = AmDataTypeEnumToSize( type);
        return( !memcmp( oldValue, newValue, size));
    }
}

/*********************************************************************
 *
 * Description:
 *     Given a filename it tries to create the path to it (without
 *     the actual filename in it).  If the filename is a full
 *     path name, that gets returned, otherwise the environment
 *     directory that should contain the file gets returned.
 *
 *********************************************************************/

static const char *
createPath( char *filename, char *tag)
{
    const char *path;

    path = AmGetPath( filename);
    if ( path == NULL) {
        path = AmResourceToSubdir( tag);
        if ( path != NULL) {
            path = AmGetEnvFile( CONFIG_ENV, path);
        }
        if ( path == NULL) {  
            path = "/";
        }
    } 

    return( path);
}

/*********************************************************************
 *
 * General callback functions:
 *     These functions are used in general with the widgets that
 *     AmAddObject creates.  They are used to generate the user
 *     events and to free memory used.
 *
 *********************************************************************/

/*********************************************************************
 *
 * Description:
 *     This function generates an AM_OK user callback that will
 *     activate all the registered callbacks.  This means it will
 *     activate the callbacks for all the widgets that registered
 *     an event handler for the AM_OK user event.  Every widget
 *     created with the AmAddObject function registers such an
 *     event.  The event is used to get the new value from the
 *     widgets and to pass them on to the outside data structures.
 *
 *     The function also gets rid of the dialog shell.
 *
 *********************************************************************/

static void
slickOkCallback( Widget button, AmRecordClass *record, XtPointer)
{
    AmTagValueRec tagValues[ MAXTAGS];
    AmOkArgsPtr   okArgs;

    if ( record == NULL) {
	okArgs = NULL;
        AmSendUserEventChildren( AM_PKG_ID, AM_OK, NULL, 
			       AmGetWidgetShell( button)); 
    } else {
        okArgs = AmMakeOkArgs( record, tagValues, 0);
        AmSendUserEventChildren( AM_PKG_ID, AM_OK, okArgs, 
			       AmGetWidgetShell( button)); 
#if 0
        if ( okArgs->numTags > 0) {
            record->setFieldsWrapper( okArgs->tagValues, okArgs->numTags);
        }
#endif
	record->setFieldsWrapper( okArgs->tagValues, okArgs->numTags);
    }

    AmSendUserEventChildren( AM_PKG_ID, AM_FINAL, okArgs, 
			   AmGetWidgetShell( button)); 
    if ( okArgs != NULL) {
	FREE( okArgs);
    }
    XtDestroyWidget( AmGetWidgetShell( button));
}

/*********************************************************************
 
  This variant of slickOkCallback does the setting, but doesn't send the
  AM_FINAL event, or destroy the shell 
 *********************************************************************/

static void
slickAlrightCallback( Widget button, AmRecordClass *record, XtPointer)
{
    AmTagValueRec tagValues[ MAXTAGS];
    AmOkArgsPtr   okArgs;

    if ( record == NULL) {
	okArgs = NULL;
        AmSendUserEventChildren( AM_PKG_ID, AM_OK, NULL, 
			       AmGetWidgetShell( button)); 
    } else {
        okArgs = AmMakeOkArgs( record, tagValues, 0);
        AmSendUserEventChildren( AM_PKG_ID, AM_OK, okArgs, 
			       AmGetWidgetShell( button)); 
        if ( okArgs->numTags > 0) {
            record->setFieldsWrapper( okArgs->tagValues, okArgs->numTags);
        }
    }
}


/*********************************************************************
 *
 * Description:
 *     This function generates an AM_REVERT user callback that will
 *     activate all the registered callbacks.  This means it will
 *     activate the callbacks for all the widgets that registered
 *     an event handler for the AM_REVERT user event.  Every widget
 *     created with the AmAddObject function registers such an
 *     event.  The event is used to reset the value in the widget to
 *     the original value.
 *
 *********************************************************************/

static void
slickRevertCallback( Widget, Widget parent, XtPointer)
{
    AmSendUserEventChildren( AM_PKG_ID, AM_REVERT, NULL, parent); 
}

/*********************************************************************
 *
 * Description:
 *     This function gets rid of any data allocated for the widget.
 *     The clientData contains pointers to this memory and is itself
 *     memory allocated for the widgets.
 *
 *********************************************************************/

static void
slickDestroyCallback( int, AmSlickPtr clientData, XtPointer)
{
    if ( clientData->range != NULL) {
        free( clientData->range);
    }
    if ( clientData->value != NULL) {
        free( clientData->value);
    }
    if ( clientData->buttons != NULL) {
        free( clientData->buttons);
    }
    if (( clientData->storageType == AmLOCAL) &&
	( clientData->valuePtr != NULL)) {
	free( clientData->valuePtr);
    }
    free( clientData);
}

/*********************************************************************
 *
 * Widget callback functions:
 *     These functions are used in conjunction with the widgets that
 *     AmAddObject can create.
 *
 *********************************************************************/

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the text widget and sets the
 *     field, specified by the tag in the clientData, with the
 *     value.
 *
 ********************************************************************/

static void
textFieldOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    char *str;
    int  size;
    void *value;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }

/* we use getValues instead of XmTextFieldGetString, because thie can also
   be used for getting values from multiline text fields */

    XtVaGetValues( clientData->widget,
		   XmNvalue, &str,
		   NULL);
    size = AmSizeData( str, clientData->type);
    value = ( void *) malloc( size);
    if ( value == NULL) {
	AmPrintResourceError( "mallocFailed", clientData->tag);
	return;
    }
    AmScanType( str, value, clientData->type, clientData->units);

    if (( clientData->storageType != AmLOCAL) &&
	!valuesEqual( clientData->value, value, clientData->type)) {
        if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

            if (( okArgs != NULL) && ( okArgs->record == clientData->record)) {
                AmSetTagValue( value, clientData->type, clientData->tag, okArgs);
            } else {
                AmSetRecordField( clientData->record, clientData->tag, value);
            }
        } else {
            AmCopyValue( clientData, value);
        }
    }

    XtFree( str);
    free( value);
}

/********************************************************************
 *
 * Description:
 *    Sets a new value in the the multiline text and saves the value in the
 *    client data structure 
 *
 ********************************************************************/

static void
setMultiLineTextValue( Widget	text, 
		       void	*value)
{
    XmTextSetString( text, ( char *) value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
multiLineTextRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    setMultiLineTextValue( clientData->widget, clientData->value);
}


/********************************************************************
 *
 * Description:
 *    Sets a new value in the the text field 
 *
 ********************************************************************/

static void
setTextFieldValue( Widget	textField, 
		   void		*value)
{
    char 	*temp, buffer[ TEXT_BUFFER];
    AmSlickPtr	clientData;

    XtVaGetValues( textField, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( textField));
	return;
    }
    AmPrintType( buffer, value, clientData->type, clientData->units);
    temp = AmElimLeadWspace( buffer); 
    temp = AmElimTrailWspace( temp); 
    XmTextFieldSetString( clientData->widget, temp);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
textFieldRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    setTextFieldValue( clientData->widget, clientData->value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the toggle button and sets the
 *     field, specified by the tag in the clientData, with the
 *     value (TRUE or FALSE).
 *
 ********************************************************************/

static void
toggleButtonOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    AnvilBoolean value;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    value = XmToggleButtonGetState( clientData->widget);

    if (( clientData->storageType != AmLOCAL) &&
	!valuesEqual( clientData->value, &value, clientData->type)) {
        if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

            if (( okArgs != NULL) && ( okArgs->record == clientData->record)) {
                AmSetTagValue( &value, clientData->type, clientData->tag, okArgs);
            } else {
                AmSetRecordField( clientData->record, clientData->tag, &value);
            }
        } else {
            AmCopyValue( clientData, &value);
        }
    }
}

/********************************************************************
 *
 * Description:
 *    Sets a new value in the the toggle button 
 *
 ********************************************************************/

static void
setToggleButtonValue( Widget	toggleButton, 
		      void	*value)
{
    AmSlickPtr	clientData;

    XtVaGetValues( toggleButton, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( toggleButton));
	return;
    }
    XmToggleButtonSetState( toggleButton, *( AnvilBoolean *) value, FALSE);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
toggleButtonRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    setToggleButtonValue( clientData->widget, clientData->value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the scale widget and sets the
 *     field, specified by the tag in the clientData, with the
 *     value.
 *
 ********************************************************************/

static void
scaleOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    int  	intVal;
    short	decimalPoints;
    float	factor, floatVal;
    void	*scaleValue = NULL;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }

/* get the value and the decimal points, since the value is scaled by them */

    XtVaGetValues( clientData->widget, 
		   XmNvalue,  &intVal,
		   XmNdecimalPoints, &decimalPoints,
		   NULL);
    factor = pow( 10.0, -decimalPoints);
    if ( clientData->type == AmFLOAT) {
	floatVal = intVal*factor;
	scaleValue = &floatVal;
    } else if ( clientData->type == AmINT) {
	intVal = ( int) (intVal*factor);
	scaleValue = &intVal;
    }

    if (( clientData->storageType != AmLOCAL) &&
	!valuesEqual( clientData->value, scaleValue, clientData->type)) {
        if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

            if (( okArgs != NULL) && ( okArgs->record == clientData->record)) {
                AmSetTagValue( scaleValue, clientData->type, clientData->tag,
			     okArgs);
            } else {
                AmSetRecordField( clientData->record, clientData->tag, 
				scaleValue);
            }
        } else {
            AmCopyValue( clientData, scaleValue);
        }
    }
}

/********************************************************************
 *
 * Description:
 *    Lower level function to set the value in the scale.  Used from
 *    setScaleValue() and AmCreateScaleWidget().  setScaleValue() can't
 *    be called from AmCreateScaleWidget() because the clientData hasn't
 *    been stored in the widget's XmNuserData yet.
 *
 ********************************************************************/

static void
setScaleValDecimalPts( Widget	scale,
		       void	*value,
		       short	decimalPoints,
		       AmSlickPtr	clientData)
{
    float	floatDec, factor, floatValue;
    int		intValue;

/* find out the precision that's used to display the value.  It's used as
   a factor for scaling the value. Scale to the integer value and set it
   in the scale */

    floatDec = decimalPoints;
    factor = rint( pow( 10.0, floatDec));
    if ( clientData->type == AmFLOAT) {
	floatValue = *( float *) value;
	intValue = ( int) ( floatValue*factor);
    } else {
	intValue = (int)(*(( int *) value)*factor);
    }
    XmScaleSetValue( scale, intValue);
}

/********************************************************************
 *
 * Description:
 *    Sets a new value in the the scale 
 *
 ********************************************************************/

static void
setScaleValue( Widget	scale, 
	       void	*value)
{
    AmSlickPtr	clientData;
    short	decimalPoints;

/* find out the precision that's used to display the value.  It's used as
   a factor for scaling the value. Scale to the integer value and set it
   in the scale */

    XtVaGetValues( scale, 
		   XmNuserData, &clientData, 
		   XmNdecimalPoints, &decimalPoints,
		   NULL);

/* make sure the client data is there */

    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( scale));
	return;
    }
    setScaleValDecimalPts( scale, value, decimalPoints, clientData);
}

/********************************************************************
 *     
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
scaleRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    setScaleValue( clientData->widget, clientData->value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the list widget and sets the
 *     field, specified by the tag in the clientData, with the
 *     value.
 *
 ********************************************************************/

static void
listOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    void **valueList;
    int  *posList, n;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }

/*
 * The value selected by the user depends on the position of the
 *   selected item.  The client data contains a list of enumeration
 *   values that correspond to the strings in the list.
 */

    XmListGetSelectedPos( clientData->widget, &posList, &n);

    if ( n == 0) {
        AN_DEBUG( 99, ("no value selected in list\n"));
    } else {
        valueList = ( void **)clientData->range;

    /*
     * The position of the items in the list widget start at 1,
     *   but the list of values starts at 0. 
     */

        if (( clientData->storageType != AmLOCAL) && 
	    !valuesEqual( clientData->value, &valueList[ posList[ 0] - 1],
			  clientData->type)) {
            if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

		if (( okArgs != NULL) && 
		    ( okArgs->record == clientData->record)) {
                    if ( clientData->type == AmSTRING) {
                        AmSetTagValue( (( char **)valueList)[ posList[ 0] - 1],
                            clientData->type, clientData->tag, okArgs);
                    } else {
                        AmSetTagValue( &(valueList[ posList[ 0] - 1]),
                            clientData->type, clientData->tag, okArgs);
                    }
                } else {
                    if ( clientData->type == AmSTRING) {
                        AmSetRecordField( clientData->record, clientData->tag,
                            (( char **)valueList)[ posList[ 0] - 1]);
                    } else {
                        AmSetRecordField( clientData->record, clientData->tag,
                            &(valueList[ posList[ 0] - 1]));
                    }
                }
            } else {
                if ( clientData->type == AmSTRING) {
                    AmCopyValue( clientData,
                        (( char **)valueList)[ posList[ 0] - 1]);
                } else {
                    AmCopyValue( clientData, &(valueList[ posList[ 0] - 1]));
                }
            }
        }
    }
    XtFree(( char *) posList);
}

/********************************************************************
 *
 * Description:
 *    Sets a new value in the the scale 
 *
 ********************************************************************/

static void
setListValue( Widget	list, 
	      void	*value)
{
    int  	pos, ii, n;
    void 	**valueList;
    AmSlickPtr	clientData;

/* Get the list of enumeration values and Figure out which item in the list 
   corresponds to the original value. */

    XtVaGetValues( list, 
		   XmNitemCount, &n,
		   XmNuserData, &clientData,
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( list));
	return;
    }
    valueList = ( void **) clientData->range;

    pos = -1;
    for ( ii = 0; ii < n; ii++) {
        if ( clientData->type == AmSTRING) {
            if ( valuesEqual( valueList[ ii], value, clientData->type)) {
                pos = ii;    
                break;
            }
        } else {
            if ( valuesEqual( valueList + ii, value, clientData->type)) {
                pos = ii;    
                break;
            }
        }
    }

/*
 * The list widget starts its position at 1.  The value list starts
 *   at 0.
 */

    XmListSelectPos( clientData->widget, pos + 1, FALSE);
}

/********************************************************************
 *     
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
listRevertCallback( int, AmSlickPtr clientData, XtPointer)
{ 
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    setListValue( clientData->widget, clientData->value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the option menu widget and sets the
 *     field, specified by the tag in the clientData, with the
 *     value.
 *
 ********************************************************************/

static void
optionMenuOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    void    **valueList;
    int     ii;
    Widget  widget;
    AnvilBoolean equal;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }

/* Get the list of enumeration values. If there are no enumeration values, 
   then bail out */

    if (( valueList = ( void **)clientData->range) == NULL) {
	return;
    }

/*
 * The value selected by the user depends on the position of the
 *   selected item.  The client data contains the list of the
 *   buttons in the option menu.  The XmNmenuHistory value in
 *   in the option menu contains the AmLast activated widget.
 */

    XtVaGetValues( clientData->widget, XmNmenuHistory, &widget, NULL);

    for ( ii = 0; clientData->buttons[ ii] != NULL; ii++) {
        if ( clientData->buttons[ ii] == widget) {
            break;
        }
    }

    if ( clientData->storageType != AmLOCAL) {
	if ( clientData->type == AmSTRING) {
	    equal = valuesEqual( clientData->value, (( char **)valueList)[ ii],
			clientData->type);
	} else {
	    equal = valuesEqual( clientData->value, &(valueList[ ii]),
			clientData->type);
	}

	if ( !equal) {
	    if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

		if (( okArgs != NULL) && ( okArgs->record == clientData->record)) {
		    if ( clientData->type == AmSTRING) {
			AmSetTagValue( valueList[ ii], clientData->type,
			    clientData->tag, okArgs);
		    } else {
			AmSetTagValue( valueList + ii, clientData->type,
			    clientData->tag, okArgs);
		    }
		} else {
		    if ( clientData->type == AmSTRING) {
			AmSetRecordField( clientData->record, clientData->tag,
			    valueList[ ii]);
		    } else {
			AmSetRecordField( clientData->record, clientData->tag,
			    valueList + ii);
		    }
		}
	    } else {
		if ( clientData->type == AmSTRING) {
		    AmCopyValue( clientData, valueList[ ii]);
		} else {
		    AmCopyValue( clientData, valueList + ii);
		}
	    }
	}
    }
}


/********************************************************************
 *
 * Description:
 *    Sets a new value in the option menu 
 *
 ********************************************************************/

static void
setOptionMenuValue( Widget	optionMenu, 
		    void	*value)
{
    void   	**valueList;
    int    	ii;
    AmSlickPtr	clientData;

/* pull the client data from the option menu */

    XtVaGetValues( optionMenu,
		   XmNuserData, &clientData,
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( optionMenu));
	return;
    }

/* Get the list of enumeration values. */

    valueList = ( void **) clientData->range;
    for ( ii = 0; clientData->buttons[ ii] != NULL; ii++) {

/* do string compares if it's a list of strings, bit compares if it's
   anything else */

        if ( clientData->type == AmSTRING) {
            if ( valuesEqual( valueList[ ii], value, clientData->type)) {
                XtVaSetValues( clientData->widget,
			       XmNmenuHistory, clientData->buttons[ ii],
			       NULL);
                break;
            }
        } else {
            if ( valuesEqual( valueList + ii, value, clientData->type)) {
                XtVaSetValues( clientData->widget,
			       XmNmenuHistory, clientData->buttons[ ii],
			       NULL);
                break;
            }
        }
    }
}

/********************************************************************
 *     
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
optionMenuRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
    setOptionMenuValue( clientData->widget, clientData->value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the push button widget and sets the
 *     field, specified by the tag in the clientData, with the
 *     value.
 *
 ********************************************************************/

static void
pushButtonOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    XmString motifString;
    char     *string;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }

/*
 * Get the value that is stored in the push button's label.  The
 *   label represents the file name.
 */

    XtVaGetValues( clientData->widget, XmNlabelString,  &motifString, NULL);
    XmStringGetLtoR( motifString, XmFONTLIST_DEFAULT_TAG, &string);
    XmStringFree( motifString);

    if (( clientData->storageType != AmLOCAL) &&
	!valuesEqual( clientData->value, string, clientData->type)) {
        if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

	    if (( okArgs != NULL) && ( okArgs->record == clientData->record)) {
                AmSetTagValue( string, clientData->type, clientData->tag, okArgs);
            } else {
                AmSetRecordField( clientData->record, clientData->tag, string);
            }
        } else {
            AmCopyValue( clientData, string);
        }
    }

    free( string);
}

/********************************************************************
 *     
 * Description:
 *     set the value for the button 
 *
 ********************************************************************/

static void
setPushButtonValue( Widget	button, 
		    void	*valueArg)
{
    XmString 	motifString;
    AmSlickPtr	clientData;
    char	*value = ( char *) valueArg;

/* Get the orignal file name and set the button's label to it. */

    XtVaGetValues( button, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( button));
	return;
    }
    motifString = XmStringCreate(( char *)value, XmFONTLIST_DEFAULT_TAG); 
    XtVaSetValues( clientData->widget,
		   XmNlabelString, motifString, 
		   NULL);
    XmStringFree( motifString);
}

/********************************************************************
 *     
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
pushButtonRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    setPushButtonValue( clientData->widget, ( char *) clientData->value);
}

/********************************************************************
 *     
 * Description:
 *   set the value for a file selection box 
 *
 ********************************************************************/

static void 
setFileSelectionBoxValue( Widget	fileSb,
			  void		*valueArg)
{
    XmString    dirMotifString, patternMotifString, pathMotifString;
    char        extension[ TEXT_BUFFER], *tag, *value = ( char *) valueArg;
    const char	*temp;
    AmSlickPtr	clientData;

    XtVaGetValues( fileSb, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( fileSb));
	return;
    }
    tag = clientData->tag;

/* Check if the value is a full pathname */

    pathMotifString = XmStringCreate( value, XmFONTLIST_DEFAULT_TAG); 
    dirMotifString = XmStringCreate(( char *) createPath( value, tag),
				     XmFONTLIST_DEFAULT_TAG); 

    temp = AmGetExtension( value); 
    if ( temp != NULL) {
        sprintf( extension, "*.%s", temp);
    } else {
        sprintf( extension, "*");
    }

    patternMotifString = XmStringCreate( extension, XmFONTLIST_DEFAULT_TAG); 

    XtVaSetValues( fileSb,
        XmNdirectory, dirMotifString,
        XmNpattern, patternMotifString,
        XmNdirSpec, pathMotifString,
        NULL);

    XmStringFree( pathMotifString);
    XmStringFree( dirMotifString);
    XmStringFree( patternMotifString);
}


/********************************************************************
 *     
 * Description:
 *     This function gets called when a AmREVERT user event was sent.
 *     It takes the original value out of the clientData and sets
 *     the widget to that value.
 *
 ********************************************************************/

static void
fileSelectionBoxRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    setFileSelectionBoxValue( clientData->widget, clientData->value);
}

/********************************************************************
 *
 * Description:
 *     This function gets called when the user clicks on the 
 *     OK button in the file selection box.  The file that the
 *     user selects gets put into the push button's label.
 *     Which is how the push button stores the file name.
 *
 ********************************************************************/

static void
fileSbOkCallback( Widget                           fileSelectionBox,
                  Widget                           button,
                  XmFileSelectionBoxCallbackStruct *cb)
{
    XtVaSetValues( button,
        XmNlabelString, cb->value,
        NULL);
   
    XtDestroyWidget( XtParent( fileSelectionBox));
}

/********************************************************************
 *
 * Description:
 *     This function gets called when a AmOK user event was sent.
 *     It takes the value out of the file selection box widget and
 *     sets the field, specified by the tag in the clientData, with
 *     the value.
 *
 ********************************************************************/

static void
fileSelectionBoxOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    XmString motifString;
    char     *string;

    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }

/* Get the value that is stored in the widget. */

    XtVaGetValues( clientData->widget, 
            XmNdirSpec,  &motifString,
            NULL);

    XmStringGetLtoR( motifString, XmFONTLIST_DEFAULT_TAG, &string);
    XmStringFree( motifString);
    
    if (( clientData->storageType != AmLOCAL) &&
	!valuesEqual( clientData->value, string, clientData->type)) {
        if ( clientData->record != NULL) {

/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */

	   if (( okArgs != NULL) && ( okArgs->record == clientData->record)) {
                AmSetTagValue( string, clientData->type, clientData->tag, okArgs);
           } else {
                AmSetRecordField( clientData->record, clientData->tag, string);
           }
        } else {
            AmCopyValue( clientData, string);
        }
    }
    XtFree( string);
}

/********************************************************************
 *
 * Description:
 *
 ********************************************************************/

static void
colorPushButtonOkCallback( int, AmSlickPtr clientData, AmOkArgsPtr okArgs)
{
    AmColorCellPtr newCell = ( AmColorCellPtr) clientData->currValue,
                 oldCell = ( AmColorCellPtr) clientData->value;
 
    if ( !XtIsSensitive( clientData->widget)) {
        return;
    }
 
/* Only set the new value if it has changed from the original. */
 
    if (( clientData->storageType != AmLOCAL) &&
	( oldCell->pixel != newCell->pixel)) {
        if ( clientData->record != NULL) {
 
/* only set the field in the record if the record passed to the OK callback
   matches the record registered with this object */
 
            if (( okArgs != NULL) &&
                ( okArgs->record == clientData->record)) {
                AmSetTagValue( newCell, clientData->type, clientData->tag,
                    okArgs);
            } else {
                AmSetRecordField( clientData->record, clientData->tag, newCell);
            }
        } else {
            AmCopyValue( clientData, newCell);
        }
    }
}

/* set the value for the color cell controlled by the color cell editor */

static void
setColorPushButtonValue( Widget	button, 
			 void	*value)
{
    AmColorCellPtr cell = ( AmColorCellPtr) value;
    AmSlickPtr	 clientData;

    XtVaGetValues( button, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( button));
	return;
    }
    XmChangeColor( clientData->widget, cell->pixel);
    *(( AmColorCellPtr) clientData->currValue) = *cell;
    clientData->value = value;
}

static void
colorPushButtonRevertCallback( int, AmSlickPtr clientData, XtPointer)
{
    setColorPushButtonValue( clientData->widget, clientData->value);
}


#if 0
static void
colorPushButtonDestroyCallback( Widget button, AmSlickPtr clientData, XtPointer)
{
    free( clientData->currValue);
}
#endif

/********************************************************************
 *
 * Description:
 *     This function gets called when the user activates a push
 *     push button that has been added with the AmAddObject function.
 *     The push button brings up a file selection dialog which
 *     is used to select a new file.
 *
 ********************************************************************/

void
AmPushButtonCallback( Widget button, AmSlickPtr clientData, XtPointer)
{
    Widget   	fileSelectionBox;
    char     	widgetName[ TEXT_BUFFER],
             	extension[ TEXT_BUFFER],
             	*value;
    const char	*temp;
    Arg      	args[ MAX_ARGS];
    int      	n;
    XmString 	motifString, patternMotifString, dirMotifString, 
		pathMotifString;
 
/*
 * Get the current file name from the push button.  It is stored
 *   in the label of the button.
 */
  
    XtVaGetValues( button,  
        XmNlabelString, &motifString,
        NULL);
    XmStringGetLtoR( motifString, XmFONTLIST_DEFAULT_TAG, &value);

/* Check if the value is a full pathname. Create the motif strings for
   the path, directory, and pattern */

    pathMotifString = XmStringCreate( value, XmFONTLIST_DEFAULT_TAG); 
    dirMotifString = 
	XmStringCreate(( char *) createPath( value, clientData->tag),
					     XmFONTLIST_DEFAULT_TAG); 

    temp = AmGetExtension( value); 
    if ( temp) {
        sprintf( extension, "*.%s", temp);
    } else {
        sprintf( extension, "*");
    }
    patternMotifString = XmStringCreate( extension, XmFONTLIST_DEFAULT_TAG); 
    XtFree( value);

/* Create a file selection dialog. */

    sprintf(widgetName, "%sFileSB", clientData->tag);
    n = AmMakeArgList( args,
		     XmNdialogStyle, XmDIALOG_APPLICATION_MODAL,
		     XmNdirectory, dirMotifString,
		     XmNpattern, patternMotifString,
		     NULL);
    fileSelectionBox = XmCreateFileSelectionDialog( button, widgetName,
						    args, n);
    XtManageChild( fileSelectionBox);

    XtVaSetValues( fileSelectionBox,
		   XmNdirSpec, pathMotifString,
		   NULL);

    XmStringFree( motifString);
    XmStringFree( patternMotifString);
    XmStringFree( dirMotifString);
    XmStringFree( pathMotifString);

    XtAddCallback( fileSelectionBox,
        XmNokCallback, ( XtCallbackProc)fileSbOkCallback, button);

    XtAddCallback( fileSelectionBox,
        XmNcancelCallback, ( XtCallbackProc)AmPopdownCback, fileSelectionBox);
}


/********************************************************************
 *
 * Description: callback when the color was selected
 *
 ********************************************************************/

static void
AmColorUpdateFn( AmColorCellPtr pColor,
                 void           *pClientData)
{
    Widget wLaunchButton = ( Widget) pClientData;
    XmChangeColor( wLaunchButton, pColor->pixel);
}

/********************************************************************
 *
 * Description: callback to launch the color selection dialog 
 *
 ********************************************************************/

void
AmColorPushButtonCallback( Widget button, AmSlickPtr clientData, XtPointer)
{
    Widget   colorSelectionDialog;

    colorSelectionDialog = 
	AmCreateColorSelectionDialog( button, clientData->tag,
				    ( AmColorCellPtr) clientData->currValue,
                                    ( AmColorSelUpdateFn) AmColorUpdateFn,
                                    ( void *) button);
}

/********************************************************************
 *
 * Description: Create a text field widget with the slick interface.
 * Arguments:
 * form - parent form for the text widget and label 
 * label - label for the text field.
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns:
 * created XmTextField widget.
 *
 ********************************************************************/

Widget
AmCreateTextFieldWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget          textField;
    const char      *resourceBase, *minStr, *maxStr, *unitsStr = NULL;
    char            resourceName[ HUGE_BUFFER];
    void            *value = clientData->value;
    AmDataTypeEnum    type = clientData->type;
    int             minSize, maxSize;

    textField = XtVaCreateManagedWidget( clientData->tag, 
					 xmTextFieldWidgetClass, 
					 form,
					 RightOf( label), 
					 StickRight, 
					 StickTop, 
					 StickBottom,
					 XmNverifyBell, FALSE,
					 NULL);

/* get the min, max and units from the defaults database */

    resourceBase = AmGetResourceName( form);
    sprintf( resourceName, "%s.%s.min", resourceBase, clientData->tag);
    minStr = AmGetDefault( resourceName, ( const char *) NULL);
    sprintf( resourceName, "%s.%s.max", resourceBase, clientData->tag);
    maxStr = AmGetDefault( resourceName, ( const char *) NULL);

/* only grab the units from the resource database if they weren't explicitly
   passed as an argument */

    if ( clientData->units == AmUNIT_UNDEFINED) {
	sprintf( resourceName, "%s.%s.units", resourceBase, clientData->tag);
	unitsStr = AmGetDefault( resourceName, "none");
	if ( strcmp( unitsStr, "none")) {
	    if ( !AmGetUnitCode( unitsStr, &clientData->units)) {
		AmPrintError( "value for units %s not found", unitsStr);
	    }
	}
    }

/* take the minimum and maximum values and convert them to the same type as
   the value being ranged */

    if ( minStr != NULL && maxStr != NULL) {
        minSize = AmSizeData(( void *) minStr, type);
        maxSize = AmSizeData(( void *) maxStr, type);
        clientData->range = (void *)malloc( minSize + maxSize);
	if ( clientData->range == NULL) {
	    DO_ERROR( AmMALLOC_FAILED);
	    return( NULL);
	}
        AmScanType(( char *) minStr, clientData->range, type, clientData->units);
        AmScanType(( char *) maxStr, (char *)(clientData->range) + minSize, type,
		  clientData->units);
    }

/* set up the regular expression and range checking callbacks */

    AmUnitVarEntry( textField, value, type, clientData->units, clientData->range);

/* set up the OK callback to stuff the value */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, textField,
		       ( AmCallbackFn)textFieldOkCallback,
			clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, textField,
		       ( AmCallbackFn)textFieldRevertCallback,
		       clientData);
    return( textField);
}

/********************************************************************
 *
 * Description:  create a multiline text widget.  multiline texts do not
 * support mins, maxs, units or regular expressions.  They may only be
 * used for strings.
 * form - parent form for the text widget and label 
 * label - label for the text widget
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmText widget.
 *
 ********************************************************************/

Widget
AmCreateMultiLineTextWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget          text;
    void            *value = clientData->value;

    clientData->units = AmUNIT_UNDEFINED;
    text = XtVaCreateManagedWidget( clientData->tag,
				    xmTextWidgetClass, form,
				    XmNeditMode, XmMULTI_LINE_EDIT,
				    RightOf( label), 
				    StickRight, 
				    StickTop, 
				    StickBottom,
				    XmNvalue, value,
				    NULL);

/* set the callbacks for OK and revert (see AmSetOkButton and AmSetRevertButton) */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, text,
		       ( AmCallbackFn)textFieldOkCallback, clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, text,
		       ( AmCallbackFn)multiLineTextRevertCallback, clientData);
    return( text);
}

/********************************************************************
 *
 * Description:
 * Create a toggle button widget. Toggle buttons may only be used for
 * boolean values 
 * form - parent form for the toggle button widget and label 
 * label - label for the toggle
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmToggleButton widget.
 *
 ********************************************************************/

Widget
AmCreateToggleButtonWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget     toggleButton;
    XmString   motifString;
    void       *value = clientData->value;

    if ( clientData->type != AmBOOLEAN) {
	AmPrintResourceError( "toggleUnsupportedType", clientData->type);
	return( NULL);
    }
    motifString = XmStringCreate( "", XmFONTLIST_DEFAULT_TAG); 
    toggleButton = XtVaCreateManagedWidget( clientData->tag,
					    xmToggleButtonWidgetClass, 
					    form,
					    RightOf( label), 
					    StickTop, StickBottom,
					    XmNlabelString, motifString,
					    XmNset, *( AnvilBoolean *)value,
					    NULL);
    XmStringFree( motifString); 

/* set the callbacks for OK and revert (see AmSetOkButton and AmSetRevertButton) */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, toggleButton,
		       ( AmCallbackFn)toggleButtonOkCallback,
		       clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, toggleButton,
		       ( AmCallbackFn)toggleButtonRevertCallback,
		       clientData);
    return( toggleButton);
}

/********************************************************************
 *
 * Description:
 * Create a scale widget.  scales may be used for integer or float values
 * form - parent form for the scale widget and label 
 * label - label for the scale
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmToggleButton widget.
 *
 ********************************************************************/

Widget
AmCreateScaleWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget      scale;
    short	decimalPoints;

/* scales support ints and floats */

    if (( clientData->type != AmFLOAT) && ( clientData->type != AmINT)) {
	AmPrintResourceError( "scaleUnsupportedType", clientData->type);
	return( NULL);
    }
    scale = XtVaCreateManagedWidget( clientData->tag, xmScaleWidgetClass, form,
				     RightOf( label), StickRight, 
				     StickTop, StickBottom,
				     XmNorientation, XmHORIZONTAL,
				     XmNshowValue, TRUE,
				     NULL);

/* get the decimal points value from the scale (it may be set in the
   resource database */

    XtVaGetValues( scale, XmNdecimalPoints, &decimalPoints, NULL);
    setScaleValDecimalPts( scale, clientData->value, 
			   decimalPoints, clientData);

/* set up the traditional callbacks */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, scale,
		       ( AmCallbackFn) scaleOkCallback, clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, scale,
		      ( AmCallbackFn) scaleRevertCallback, clientData);
    return( scale);
}

/********************************************************************
 *
 * Description:
 *     This function is kind of misnamed.  It creates a push button
 *     widget which, in turn, creates a file selection dialog.
 * form - parent form for the push button widget and label 
 * label - label for the push button.
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmPushButton widget.
 *
 ********************************************************************/

Widget
AmCreatePushButtonWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget   pushButton;
    XmString motifString;
    char     *value = ( char *)( clientData->value);

/* take the value and display it as the button's label */

    motifString = XmStringCreate( value, XmFONTLIST_DEFAULT_TAG); 
    pushButton = XtVaCreateManagedWidget( clientData->tag,
					  xmPushButtonWidgetClass, form,
					  RightOf( label), 
					  StickRight, StickTop, StickBottom,
					  XmNlabelString, motifString,
					  NULL);
    XmStringFree( motifString);

/* set up the traditional OK and revert callbacks */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, pushButton,
		       ( AmCallbackFn) pushButtonOkCallback,
		       clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, pushButton,
		      ( AmCallbackFn) pushButtonRevertCallback,
		      clientData);

/* set up the callback to create the file selection dialog */

    XtAddCallback( pushButton, XmNactivateCallback,
		   ( XtCallbackProc) AmPushButtonCallback, clientData);
    return( pushButton);
}

/********************************************************************
 *
 * Description:
 *  	create a push button widget to launch the color selection dialog 
 *   	color cell push button widgets may only accept AmCOLOR_CELL type
 * form - parent form for the push button widget and label 
 * label - label for the push button.
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmPushButton widget.
 *
 ********************************************************************/

Widget
AmCreateColorPushButtonWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget   		pushButton;
    ulong_t  		pixel = (( AmColorCellPtr) clientData->value)->pixel;
    AmColorCellPtr	cell;

    if ( clientData->type != AmCOLOR_CELL) {
	AmPrintResourceError( "colorUnsupportedType", clientData->type);
	return( NULL);
    }
/* 
 * Allocate the color cell to pass around.
 */

    if (( clientData->currValue = malloc( sizeof( AmColorCellRec))) == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }
    cell = ( AmColorCellPtr) clientData->value;
    *(( AmColorCellPtr) ( clientData->currValue)) = *cell;

    pushButton = XtVaCreateManagedWidget( clientData->tag,
					  xmPushButtonWidgetClass, form,
					  RightOf( label), StickRight, 
					  StickTop, StickBottom,
					  XmNbackground, pixel,
					  NULL);

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, pushButton,
		       ( AmCallbackFn) colorPushButtonOkCallback, clientData);


    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, pushButton,
		       ( AmCallbackFn) colorPushButtonRevertCallback,
		       clientData);

/*
 * Need to have a special destroy callback that will free the
 * extra color that we allocated.  This is an extraordinarily dangerous
 * callback, because it dereferences data which is freed by the other
 * destroy callback WARNING WARNING WARNING.
 */
#if 0
    XtAddCallback( pushButton, XmNdestroyCallback,
		   ( XtCallbackProc) colorPushButtonDestroyCallback,
		   clientData);
#endif
    XtAddCallback( pushButton, XmNactivateCallback,
		   ( XtCallbackProc) AmColorPushButtonCallback, clientData);
    return( pushButton);
}

/********************************************************************
 *
 * Description:
 *  	Create a file selection box to select a file for the value.
 *	File selection boxes may take only AmSTRING and AmFILE types
 * form - parent form for the push button widget and label 
 * label - label for the push button.
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmPushButton widget.
 *
 ********************************************************************/

Widget
AmCreateFileSelectionBox( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget   	fileSelectionBox;
    XmString 	dirMotifString, patternMotifString, pathMotifString;
    char     	extension[ TEXT_BUFFER],
             	*value = ( char *) clientData->value;
    const char	*temp;

    if (( clientData->type != AmSTRING) && ( clientData->type != AmFILE)) {
	AmPrintResourceError( "fileSelectUnsupportedType", clientData->type);
	return( NULL);
    }

/* blow the label away... */

    XtUnmanageChild( label);

/* Check if the value is a full pathname.  If so, break it out into the
   filename and pathname, and use them for the pattern and directory
   for the file selection, otherwise create an unspecified file selection
   box */

    if ( value != NULL) {
	pathMotifString = XmStringCreate( value, XmFONTLIST_DEFAULT_TAG); 
	dirMotifString = 
		XmStringCreate(( char *) createPath( value, clientData->tag),
			       XmFONTLIST_DEFAULT_TAG); 
	temp = AmGetExtension( value); 
	if ( temp != NULL) {
	    sprintf( extension, "*.%s", temp);
	} else {
	    strcpy( extension, "*");
	}
	patternMotifString = XmStringCreate( extension, XmFONTLIST_DEFAULT_TAG);
	fileSelectionBox = 
	    XtVaCreateManagedWidget( clientData->tag,
				     xmFileSelectionBoxWidgetClass, form,
				     RightOf( label), StickRight, 
				     StickTop, StickBottom,
				     XmNdirectory, dirMotifString,
				     XmNpattern, patternMotifString,
				     XmNdirSpec, pathMotifString,
				     NULL);
	XmStringFree( pathMotifString);
	XmStringFree( dirMotifString);
	XmStringFree( patternMotifString);
    } else {
	fileSelectionBox = 
	    XtVaCreateManagedWidget( clientData->tag,
				     xmFileSelectionBoxWidgetClass, form,
				     RightOf( label), StickRight, 
				     StickTop, StickBottom,
				     NULL);
    }

/* remove the cancel, OK and Help buttons for the file selection box */

    XtUnmanageChild( XmFileSelectionBoxGetChild( fileSelectionBox,
						 XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild( XmFileSelectionBoxGetChild( fileSelectionBox,
						 XmDIALOG_OK_BUTTON));
    XtUnmanageChild( XmFileSelectionBoxGetChild( fileSelectionBox,
						 XmDIALOG_HELP_BUTTON));

/* add the OK and revert user callbacks */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, fileSelectionBox,
		       ( AmCallbackFn) fileSelectionBoxOkCallback,
		       clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, fileSelectionBox,
		       ( AmCallbackFn) fileSelectionBoxRevertCallback,
		       clientData);

    return( fileSelectionBox);
}

/* get the list of strings that are displayed for this tag */

static AnvilBoolean resourceStringList( Widget	w, 
				   char		*tag,
				   char		***stringList)
{
    char	resourceName[ HUGE_BUFFER], string[ HUGE_BUFFER], *temp;
    const char  *ptr;

    sprintf( resourceName, "%s*%s.list", AmGetResourceName( w), tag);
    ptr = AmGetDefault( resourceName, ( const char *) NULL);
    if ( ptr != NULL) {
	strcpy( string, ptr);
        temp = AmElimLeadWspace( string); 
        if ( temp != NULL) {
            temp = AmElimTrailWspace( temp); 
        }

/* the returned string list is created by chopping up the values which are
   separated by spaces */

	*stringList = AmChopString( temp, ' ');
	return( TRUE);
    } else {
	*stringList = NULL;
	return( FALSE);
    }
}

/********************************************************************
 *
 * Description: form an enumeration class, which is referenced by a
 * 		field tag, return the list of strings and values to
 *		present in a list 
 * form:		parent form for the list
 * clientData:		information about the field.
 * motifStringList:	strings to be displayed
 * valueList:		parallel list of values 
 * pos:			index of the client data value in the vector
 *			of enumerations
 * returns: AmOK on success, AmBAD_ARGUMENT or AmMALLOC_FAILED on failure
 *
 ********************************************************************/

static AmErrorCode
getEnumList( Widget   form,
             AmSlickPtr clientData,
	     char     ***stringList,
             int      **valueList,
	     int      *pos)
{
    char     	**enumStringList;
    void     	*value = clientData->value;
    int	     	ii, numel, *values;

/* if there's a list of enumerations in the resource file, then use them, 
   otherwise, get the list of enumerations specfied in the resource database 
   for the tag */

    if ( !resourceStringList( form, clientData->tag, &enumStringList)) {
	if ( !AmGetEnumStringValues( clientData->tag, &enumStringList,
				   valueList, &numel)) {
	    *valueList = NULL;
	    *stringList = NULL;
	    DO_RETURN_ERROR( AmBAD_ARGUMENT);
	} 
    } else {
	numel = AmVeclen( (const char **)enumStringList);

/* allocate the vector of values */

	if (( *valueList = ( int *) malloc( sizeof( int)*numel)) == NULL) {
	    DO_RETURN_ERROR( AmMALLOC_FAILED);
	}
	values = *valueList;

/* get the enumeration for each of the strings specified in the resource file.
   If the enumerations were retreived from the enumeration database, then
   the values are already allocated and set */

	for ( ii = 0; ii < numel; ii++) {
	    if ( !AmGetStrEnum( clientData->tag, enumStringList[ ii], 
			      &values[ ii])) {
		AN_DEBUG( 1, ( "enum code for %s not found\n",
			       enumStringList[ ii]));
	    }
	}
    }
    *stringList = enumStringList;

/* get the index of the current value in the list of values */

    *pos = AmGetIndex(( void *) *( int *) value, ( void **) *valueList, numel);
    return( AmOK);
}

/********************************************************************
 *
 * Description: for units which are referenced by a field tag, return 
 *		the list of strings and values to present in a list 
 * form:		parent form for the list
 * clientData:		information about the field.
 * stringList:		strings to be displayed
 * valueList:		parallel list of values 
 * pos:			index of the client data value in the vector
 *			of enumerations
 * returns: AmOK on success, AmBAD_ARGUMENT or AmMALLOC_FAILED on failure
 *
 ********************************************************************/

static AmErrorCode
getUnitList( Widget   form,
             AmSlickPtr clientData,
	     char     ***stringList,
             int      **valueList,
	     int      *pos)
{
    char            **unitStringList;
    void            *value = clientData->value;
    int		    numel, ii, *values;

/*
 * Check if there is a list of units specified in the resource file, 
 * otherwise, get the list of all available units
 */
    if ( !resourceStringList( form, clientData->tag, &unitStringList)) {
        AmGetUnitStringList( &unitStringList, &numel);
	if ( numel <= 0) {
	    DO_RETURN_ERROR( AmBAD_ARGUMENT);
	}
    } else {
	numel = AmVeclen( (const char **)unitStringList);
    }

/* allocate the list of values */

    if (( *valueList = ( int *) malloc( numel*sizeof( int))) == NULL) {
	*valueList = NULL;
	*stringList = NULL;
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    values = *valueList;

/* get the unit code for each for each of the unit strings */

    for ( ii = 0; ii < numel; ii++) {
	if ( !AmGetUnitCode( unitStringList[ ii], &values[ ii])) {
	    AN_DEBUG( 1, ( "unit code for %s not found\n",
			   unitStringList[ ii]));
	}
    }
    *stringList = unitStringList;
    *pos = AmGetIndex(( void *) *( int *) value, ( void **) *valueList, numel);
    return( AmOK);
}

/********************************************************************
 *
 * Description: 	get the list of integers to be displayed for
 *			a specified field tag.
 * form:		parent form for the list
 * clientData:		information about the field.
 * stringList:		strings to be displayed
 * valueList:		parallel list of values 
 * returns: AmOK on success, AmBAD_ARGUMENT or AmMALLOC_FAILED on failure
 *
 ********************************************************************/

static AmErrorCode
getIntList( Widget   form,
            AmSlickPtr clientData,
            char     ***stringList,
            int      **valueList,
	    int	     *pos)
{
    char            **resStringList;
    void            *value = clientData->value;
    int             ii, numel;

/* initialize everything to NULL in case of failure */

    *valueList = NULL;
    *stringList = NULL;
    *pos = -1;

/* get the list of values from the resource file */

    if ( resourceStringList( form, clientData->tag, &resStringList)) {
	numel = AmVeclen( (const char **)resStringList);
	if (( *valueList = ( int *) malloc( numel*sizeof( int))) == NULL) {
	    DO_RETURN_ERROR( AmMALLOC_FAILED);
	}
	for ( ii = 0; ii < numel; ii++) {
	    ( *valueList)[ ii] = atoi( resStringList[ ii]);
	}
	*stringList = resStringList;
	*pos = AmGetIndex(( void *) *( int *) value, 
			( void **) *valueList, numel);
	return( AmOK);
    } else {
	DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }
}

/********************************************************************
 *
 * Description: create the list of strings to display and to store for
 *		a list or option menu.
 *
 * form:		parent form for the list
 * clientData:		information about the field.
 * stringList:		strings to be displayed
 * valueList:		parallel list of values 
 * numElements:		number of elements to display.
 * returns: AmOK on success, AmBAD_ARGUMENT or AmMALLOC_FAILED on failure
 *
 ********************************************************************/

static AmErrorCode
getStringList( Widget   form,
               AmSlickPtr clientData,
               char	***stringList,
               char     ***valueList,
	       int	*pos)
{
    char            **resStringList, *value = ( char *) clientData->value;
    int             ii;

/* get the list of strings from the file */

    if ( resourceStringList( form, clientData->tag, &resStringList)) {
	*pos = -1;
	if (( *valueList = ( char **) 
		malloc( sizeof( char *)*AmVeclen( (const char **)resStringList)))  == NULL) {
	    DO_RETURN_ERROR( AmMALLOC_FAILED);
	}
	for ( ii = 0; resStringList[ ii] != NULL; ii++) {
	    if (( value != NULL) && !strcmp( resStringList[ ii], value)) {
		*pos = ii;
	    }
	}

/* copy the strings over to the value list and set the string list */

	if (( *valueList = AmCopyStrVec( (const char **)resStringList)) == NULL) {
	    DO_RETURN_ERROR( AmMALLOC_FAILED);
	}
	*stringList = resStringList;
	return( AmOK);
    } else {
	*valueList = NULL;
	*stringList = NULL;
	return( AmBAD_ARGUMENT);
    }
}

/********************************************************************
 *
 * Description:
 *  	Create a list widget.  List widgets may take AmENUM, AmUNITS,
 *	AmINT, AmSTRING
 * form - parent form for the list widget and label 
 * label - label for the list
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created XmList widget.
 *
 ********************************************************************/

Widget
AmCreateListWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget      list;
    char        **stringList;
    XmString    *motifStringList;
    Arg         args[ MAX_ARGS];
    void        *valueList;
    int         numElements, pos, n;
    AmErrorCode	errCode;

    switch ( clientData->type) {
        case AmENUM:
            errCode = getEnumList( form, clientData, &stringList,
				   ( int **)&valueList, &pos);
            break;
        case AmUNITS:
            errCode = getUnitList( form, clientData, &stringList,
				   ( int **)&valueList, &pos);
            break;
        case AmINT:
            errCode = getIntList( form, clientData, &stringList,
				  ( int **)&valueList, &pos);
            break;
        case AmSTRING:
            errCode = getStringList( form, clientData, &stringList,
				     ( char ***)&valueList, &pos);
        default:
	    AmPrintResourceError( "listUnsupportedType", 
				clientData->tag, clientData->type);
	    return( NULL);
    }
    if ( errCode != AmOK) {
	AmPrintResourceError( "failedToGetValues", clientData->tag);
	return( NULL);
    }
    motifStringList = AmMotifStringList( stringList, &numElements);
    if ( motifStringList == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* Save the enum values as the range in the clientData. */

    clientData->range = valueList;

/* Create the actual list widget. */

    n = AmMakeArgList( args, 
		     XmNitems, motifStringList,
		     XmNitemCount, numElements,
		     RightOf( label),
		     StickRight,
		     StickTop,
		     StickBottom,
		     NULL);
    list = XmCreateScrolledList( form, clientData->tag, args, n); 
    XtManageChild( list);

    if ( pos != -1) {
        XmListSelectPos( list, pos + 1, FALSE);
    } else {
        AN_DEBUG( 99, ( "no selected value in list\n"));
    }

/* Free the memory allocated by the motif strings. */

    AmFreeMotifStringList( motifStringList, numElements);
    AmFreeStrVec( stringList);

/* add the OK and revert callbacks */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, list,
		       ( AmCallbackFn)listOkCallback, clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, list,
		       ( AmCallbackFn)listRevertCallback, clientData);
    return( list);
}

/********************************************************************
 *
 * Description:
 *  	Create a option menu.  List widgets may take AmENUM, AmUNITS,
 *	AmINT, AmSTRING
 * form - parent form for the option menu and label 
 * label - label for the option menu
 * clientData - contains string tag, record, etc. to be filled 
 * ( see AmAddObject)
 * Returns: created Option menu
 *
 ********************************************************************/

Widget
AmCreateOptionMenuWidget( Widget form, Widget label, AmSlickPtr clientData)
{
    Widget      optionMenu, menu;
    void        *valueList;
    int         numElements, ii, pos;
    char        widgetName[ TEXT_BUFFER], **stringList;
    XmString	*motifStringList;
    AmErrorCode	errCode = AmOK;

    switch( clientData->type) {
        case AmENUM:
            errCode = getEnumList( form, clientData, &stringList,
				   ( int **)&valueList, &pos);
            break;
        case AmUNITS:
            errCode = getUnitList( form, clientData, &stringList,
				   ( int **)&valueList, &pos);
            break;
        case AmINT:
            errCode = getIntList( form, clientData, &stringList,
				  ( int **)&valueList, &pos);
            break;
        case AmSTRING:
            errCode = getStringList( form, clientData, &stringList,
				     ( char ***)&valueList, &pos);
            break;
        default:
	    AmPrintResourceError( "optionUnsupportedType", 
				clientData->tag, clientData->type);
            break;
    }
    if ( errCode != AmOK) {
	AmPrintResourceError( "failedToGetValues", clientData->tag);
	DO_ERROR( AmBAD_ARGUMENT);
	return( NULL);
    }

/* turn the list of strings into motif labels, and get the pixmaps if any */

    motifStringList = AmMotifStringList( stringList, &numElements);
    AmFreeStrVec( stringList);
    if ( motifStringList == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* Save the enum values as the range in the clientData. */

    clientData->range = valueList;
    clientData->buttons = ( Widget *)malloc( sizeof( Widget)*(numElements + 1));
    if ( clientData->buttons == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return( NULL);
    }

/* create the menu and the buttons */

    sprintf(widgetName, "%sMenu", clientData->tag);
    menu = XmCreatePulldownMenu( form, widgetName, NULL, 0);
    for ( ii = 0; ii < numElements; ii ++) {
        clientData->buttons[ ii] = 
	    XtVaCreateManagedWidget( "button",
				     xmPushButtonWidgetClass, menu,
				     XmNlabelString, motifStringList[ ii],
				     NULL);
    }
    clientData->buttons[ ii] = NULL;
    AmFreeMotifStringList( motifStringList, numElements);

/* create the option menu */

    optionMenu = XtVaCreateManagedWidget( clientData->tag,
					  xmRowColumnWidgetClass, form,
					  RightOf( label), StickRight, 
					  StickTop, StickBottom, 
					  XmNrowColumnType, XmMENU_OPTION,
					  XmNsubMenuId, menu,
					  NULL);

    if (( pos < numElements) && ( pos >= 0)) {
        XtVaSetValues( optionMenu,
		       XmNmenuHistory, clientData->buttons[ pos],
		       NULL);
    }

/* get rid of the label, which is created for us by the AmAddObject()
   interface */

    XtUnmanageChild( XmOptionLabelGadget( optionMenu));

/* Add the OK and revert callbacks */

    AmAddCallbackWidget( AM_PKG_ID, AM_OK, optionMenu,
		       ( AmCallbackFn) optionMenuOkCallback, clientData);
    AmAddCallbackWidget( AM_PKG_ID, AM_REVERT, optionMenu,
		       ( AmCallbackFn) optionMenuRevertCallback, clientData);
    return( optionMenu);
}

/********************************************************************
 *
 * Description:
 *	takes a parent of a list of forms which have been created 
 *	using AmAddObject(), and aligns the labels in the forms to 
 *	have the same width.
 *
 * Returns:
 *	TRUE if any slick-generated forms were aligned, FALSE
 *	if no slick-generated forms were aligned 
 ********************************************************************/

AnvilBoolean
AmAlignObjects( Widget parent)
{
    WidgetList   children, grandchildren;
    Dimension    size;
    int          ii, numChildren, numGrandchildren, max;
  

/* get the rowcolumn's children */

    XtVaGetValues( parent,
		   XmNnumChildren, &numChildren,
		   XmNchildren, &children,
		   NULL);

/* find the widest child */

    max = 0;
    for ( ii = 0; ii < numChildren; ii ++) {

/* only test child forms which have the slick label (look for the 
   union label..) */

        if ( !strcmp( XtName( children[ii]), UNIQUE_FORM_LABEL)) {
            XtVaGetValues( children[ ii],
			   XmNnumChildren, &numGrandchildren,
			   XmNchildren, &grandchildren,
			   NULL);

/* rather than using the 0th widget, it should loop and check for the label */

            XtVaGetValues( grandchildren[ 0],
			   XmNwidth, &size,
			   NULL);
            if ( size > max) {
                max = size;
            }
        }
    }

/* bail if the width is still zero */

    if ( max == 0) {
	return( FALSE);
    }

/* set the widths of each of the labels to the largest width */

    for ( ii = 0; ii < numChildren; ii++) {
        if ( !strcmp( XtName( children[ii]), UNIQUE_FORM_LABEL)) {
            XtVaGetValues( children[ ii],
			   XmNnumChildren, &numGrandchildren,
			   XmNchildren, &grandchildren,
			   NULL);
            XtVaSetValues( grandchildren[ 0], XmNwidth, max, NULL);
        } 
    }
    return( TRUE);
}

/********************************************************************
 *
 * Function: AmCoerceWidgetType( type, widgetType)
 *
 * Parameters:
 *     AmDataTypeEnum type
 *         - The type of the data to generate a widget for. 
 *     AmWidgetEnum widgetType
 *         - The type that will override the default widget type
 *           for the data.
 *
 * Description:
 *     This function returns the widget type to use for data 
 *     with the passed in type.  If a widget type is supplied
 *     then the function AmFirst checks to see if it is a legal
 *     widget type for the data and if it is returns that
 *     instead of the default.
 *
 ********************************************************************/

typedef struct slickData_ {
    AmDataTypeEnum dataType;
    AmWidgetEnum   widgetType;
} slickDataRec, *slickDataPtr;

slickDataRec dataToWidgetTable[] = {
    { AmSHORT, AmWIDGET_SCALE },
    { AmSHORT, AmWIDGET_TEXT },
    { AmUNSIGNED_SHORT, AmWIDGET_SCALE },
    { AmUNSIGNED_SHORT, AmWIDGET_TEXT },
    { AmINT, AmWIDGET_SCALE },
    { AmINT, AmWIDGET_TEXT },
    { AmINT, AmWIDGET_LIST },
    { AmINT, AmWIDGET_OPTION },
    { AmUNSIGNED_INT, AmWIDGET_SCALE },
    { AmUNSIGNED_INT, AmWIDGET_TEXT },
    { AmLONG, AmWIDGET_TEXT },
    { AmLONG, AmWIDGET_SCALE },
    { AmUNSIGNED_LONG, AmWIDGET_TEXT },
    { AmUNSIGNED_LONG, AmWIDGET_SCALE },
    { AmFLOAT, AmWIDGET_TEXT },
    { AmFLOAT, AmWIDGET_SCALE },
    { AmDOUBLE, AmWIDGET_TEXT },
    { AmBOOLEAN, AmWIDGET_TOGGLE },
    { AmBOOLEAN, AmWIDGET_TEXT },
    { AmSTRING, AmWIDGET_TEXT },
    { AmSTRING, AmWIDGET_OPTION },
    { AmSTRING, AmWIDGET_MULTILINE_TEXT },
    { AmFILE, AmWIDGET_FILESB },
    { AmFILE, AmWIDGET_BUTTON },
    { AmENUM, AmWIDGET_LIST },
    { AmENUM, AmWIDGET_OPTION },
    { AmCOLOR_CELL, AmWIDGET_COLOR_BUTTON },
    { AmCOLOR_CELL, AmWIDGET_TEXT },
    { AmUNITS, AmWIDGET_OPTION },
    { AmUNITS, AmWIDGET_LIST }
};

AmWidgetEnum
AmCoerceWidgetType( AmDataTypeEnum type, AmWidgetEnum widgetType)
{
    unsigned ii;

/*
 * Check if the widget type provided is a legal widget for the
 *   type of data.
 */

    if ( widgetType != AmWIDGET_DEFAULT) {
        for ( ii = 0; ii < VECLEN( dataToWidgetTable); ii++) { 
            if (( dataToWidgetTable[ ii].dataType == type) && 
                ( dataToWidgetTable[ ii].widgetType == widgetType)) {
                return( widgetType);
            }
        }
    }

/* Get the AmFirst legal widget type for type of data.  */

    for ( ii = 0; ii < VECLEN( dataToWidgetTable); ii++) {
        if ( dataToWidgetTable[ ii].dataType == type) {
            return( dataToWidgetTable[ ii].widgetType);
        }
    }

/* If all else fails, return the text widget type. */ 

    return( AmWIDGET_TEXT);
}
           

/**********************************************************************
 *
 * Description:
 *	set the value for a slick-generated object given a void pointer
 *	to the value.  The type of the value is determined by looking
 * 	at the user data in the widget 
 * Arguments:
 *	w - widget to set value in 
 *	value - pointer to value to set 
 * Returns:
 *	TRUE if data was set successfully, FALSE if slick client data
 *	wasn't stored in the widget's user data 
 *
 ***********************************************************************/

AnvilBoolean AmSetObjectValue( Widget	w,
		        void	*value)
{
    AmSlickPtr	clientData;

    XtVaGetValues( w, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( w));
	return( FALSE);
    }
    switch ( clientData->widgetType) {
        case AmWIDGET_TEXT:
	    setTextFieldValue( w, value);
            break;
	case AmWIDGET_MULTILINE_TEXT:
	    setMultiLineTextValue( w, value);
	    break;
        case AmWIDGET_SCALE:
	    setScaleValue( w, value);
            break;
        case AmWIDGET_TOGGLE:
	    setToggleButtonValue( w, value);
            break;
        case AmWIDGET_BUTTON:
	    setPushButtonValue( w, value);
            break;
        case AmWIDGET_FILESB:
	    setFileSelectionBoxValue( w, value);
            break;
        case AmWIDGET_LIST:
	    setListValue( w, value);
            break;
        case AmWIDGET_OPTION:
	    setOptionMenuValue( w, value);
            break;
        case AmWIDGET_COLOR_BUTTON:
	    setColorPushButtonValue( w, value);
            break;
        default:
            AmPrintResourceError( "unknownWidgetType", 
				XtName( w), clientData->widgetType);
            return( FALSE);
    }
    return( TRUE);
}

/* wrapper for set scalar */

static AnvilBoolean slickSetScalar( AmSlickPtr		clientData,
			       void		*valuePtr,
			       void		*value)
{

/* if the storage type is dynamic, set the value pointer to NULL, because
   AmSetScalar tries to free it otherwise... */

    if ( clientData->storageType == AmDYNAMIC) {
	*( void **) valuePtr = NULL;
    }
    return( AmSetScalar(( uchar_t *) valuePtr, clientData->type, 
		      clientData->storageType, value));
}

/* retrieve the value from a text field into a preallocated pointer */ 

static AnvilBoolean getTextFieldValue( AmSlickPtr	clientData,
				  void		*valuePtr)
{
    char	*str;
    AnvilBoolean	flag;

    XtVaGetValues( clientData->widget,
		   XmNvalue, &str,
		   NULL);
    if (( clientData->storageType == AmDYNAMIC) && 
	(( clientData->type == AmFILE) || ( clientData->type == AmSTRING))) {
	*( char **) valuePtr = strdup( str);
	flag = TRUE;
    } else {
	flag = AmScanType( str, valuePtr, clientData->type, clientData->units);
    }
    XtFree( str);
    return( flag);
}

/* retreive the value from a scale into a preallocated pointer */

static AnvilBoolean getScaleValue( AmSlickPtr	clientData,
			      void	*valuePtr)
{
    int		intVal;
    float	factor, floatVal;
    short	decimalPoints;

/* get the value and the decimal points, since the value is scaled by them */

    XtVaGetValues( clientData->widget, 
		   XmNvalue,  &intVal,
		   XmNdecimalPoints, &decimalPoints,
		   NULL);
    factor = pow( 10.0, -decimalPoints);
    if ( clientData->type == AmFLOAT) {
	floatVal = intVal*factor;
	memcpy( valuePtr, &floatVal, sizeof( float));
	return( TRUE);
    } else if ( clientData->type == AmINT) {
	intVal = ( int) (intVal*factor);
	memcpy( valuePtr, &intVal, sizeof( int));
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/* return the value from a toggle button */

static AnvilBoolean getToggleButtonValue( AmSlickPtr	clientData,
				     void	*valuePtr)
{
    AnvilBoolean	flag;

    flag = XmToggleButtonGetState( clientData->widget);
    if ( clientData->type == AmBOOLEAN) {
	*( AnvilBoolean *) valuePtr = flag;
	return( TRUE);
    } else { 
	return( FALSE);
    }
}

/* return the value from a push button */

static AnvilBoolean getPushButtonValue( AmSlickPtr	clientData,
				   void		*valuePtr)
{
    XmString	mstr;
    char	*str;
    AnvilBoolean	flag;

    XtVaGetValues( clientData->widget, XmNlabelString, &mstr, NULL);
    XmStringGetLtoR( mstr, XmFONTLIST_DEFAULT_TAG, &str);
    flag = slickSetScalar( clientData, valuePtr, str);
    XmStringFree( mstr);
    XtFree( str);
    return( flag);
}

/* return the value from a file selection box */

static AnvilBoolean getFileSelectionBoxValue( AmSlickPtr	clientData,
				         void		*valuePtr)
{
    XmString	mstr;
    char	*str;
    AnvilBoolean	flag;

/* the filename was stored in the buttons label */

    XtVaGetValues( clientData->widget, 
		   XmNdirSpec, &mstr,
		   NULL);
    XmStringGetLtoR( mstr, XmFONTLIST_DEFAULT_TAG, &str);
    flag = slickSetScalar( clientData, valuePtr, str);
    XmStringFree( mstr);
    XtFree( str);
    return( flag);
}

/* return the value from a list */

static AnvilBoolean getListValue( AmSlickPtr	clientData,
			     void	*valuePtr)
{
    void	**valueList;
    int		*posList, n, selIndex;

/*
 * The value selected by the user depends on the position of the
 *   selected item.  The client data contains a list of enumeration
 *   values that correspond to the strings in the list.
 */

    XmListGetSelectedPos( clientData->widget, &posList, &n);
    if ( n == 0) {
        return( FALSE);
    } else {
        valueList = ( void **)clientData->range;
	selIndex = posList[ 0] - 1;
	XtFree(( char *) posList);
    }

    /*
     * The position of the items in the list widget start at 1,
     *   but the list of values starts at 0. 
     */

    return( slickSetScalar( clientData, valuePtr, valueList[ selIndex]));
}

/* return the value from an option menu */

static AnvilBoolean getOptionMenuValue( AmSlickPtr	clientData,
				   void		*valuePtr)
{
    void	**valueList;
    int		ii;
    Widget	lastButton;

/* Get the list of enumeration values. If there are no enumeration values, 
   then bail out */

    if (( valueList = ( void **) clientData->range) == NULL) {
	return( FALSE);
    }

/*
 * The value selected by the user depends on the position of the
 *   selected item.  The client data contains the list of the
 *   buttons in the option menu.  The XmNmenuHistory value in
 *   in the option menu contains the AmLast activated widget.
 */

    XtVaGetValues( clientData->widget, XmNmenuHistory, &lastButton, NULL);
    for ( ii = 0; clientData->buttons[ ii] != NULL; ii++) {
        if ( clientData->buttons[ ii] == lastButton) {
            break;
        }
    }
    if ( clientData->buttons[ ii] == NULL) {
	return( FALSE);
    } else {
	return( slickSetScalar( clientData, valuePtr, valueList[ ii]));
    }
}

/* return the value from a color push button */

static AnvilBoolean getColorPushButtonValue( AmSlickPtr	clientData,
				        void		*valuePtr)
{
    if ( clientData->type == AmCOLOR_CELL) {
	memcpy( valuePtr, clientData->currValue, sizeof( AmColorCellRec));
	return( TRUE);
    } else {	
	return( FALSE);
    }
}

/* return the value of an object */

AnvilBoolean AmGetObjectValue( Widget	editable,
			void	*valuePtr)
{
    AmSlickPtr        clientData;

    XtVaGetValues( editable, 
		   XmNuserData, &clientData, 
		   NULL);
    if ( clientData == NULL) {
	AmPrintResourceError( "noSlickClientData", XtName( editable));
	return( FALSE);
    }
    switch( clientData->widgetType) {
    case AmWIDGET_TEXT:
	return( getTextFieldValue( clientData, valuePtr));
    case AmWIDGET_MULTILINE_TEXT:
	return( getTextFieldValue( clientData, valuePtr));
    case AmWIDGET_SCALE:
	return( getScaleValue( clientData, valuePtr));
    case AmWIDGET_TOGGLE:
	return( getToggleButtonValue( clientData, valuePtr));
    case AmWIDGET_BUTTON:
	return( getPushButtonValue( clientData, valuePtr));
    case AmWIDGET_FILESB:
	return( getFileSelectionBoxValue( clientData, valuePtr));
    case AmWIDGET_LIST:
	return( getListValue( clientData, valuePtr));
    case AmWIDGET_OPTION:
	return( getOptionMenuValue( clientData, valuePtr));
    case AmWIDGET_COLOR_BUTTON:
	return( getColorPushButtonValue( clientData, valuePtr));
    default:
	AmPrintResourceError( "unknownWidgetType",
			      XtName( editable), clientData->widgetType);
	return( FALSE);
    }
}

/********************************************************************
 *
 * Function: AmAddObject( record, parent, tag)
 *
 * Parameters:
 *     AmRecordClass record
 *         - A pointer to a class based on AmRecordClass which contains the
 *           data to be edited.
 *     Widget parent
 *         - The parent widget to which the new widget is to be added.
 *     char *tag
 *         - The tag string that indicates which element of the data is to
 *           be edited.
 *     type - 
 *	   - data type of the value to edit 
 *     units -
 *	   - the units to display the value with.
 *     storageType -
 *	   - used if there is no record present. AmDYNAMIC for malloc/free
 *           pointer storage, AmSTATIC for copy to static array 
 *     lengthPtr -
 * 	   - length of the object being edited.  Used if there is no record
 *           present.
 *     record -
 * 	   - record to edit....may be NULL
 *     
 * Description:
 *     This function creates a label and an editable widget depending on
 *     what type of variable is associated with the tag string and, if
 *     supplied in the resource file, the widget type.
 *
 *     The label and the editable widget are contained in a form which
 *     gets added to the passed in parent.
 *
 *     Specifics about the widgets are set in the resources.  If
 *     min and max values are specified in the resources, a range
 *     checking function gets associated with the editable widget.
 *
 *     A user event also gets associated with the editable widget
 *     so that when an ok or revert event gets sent the widgets
 *     automatically fill in the data structure or automatically
 *     revert to the values in the data structure.
 *
 ********************************************************************/

/* add object core function */

static Widget
AmAddObject( Widget          parent,
           void            *valuePtr,
           char            *name,
	   AmDataTypeEnum	   type,
	   unitType	   units,
           AmStorageTypeEnum storageType,
           int             *lengthPtr,
	   AmRecordClass	   *record)
{
    Widget          form, label, editable;
    AmWidgetEnum      widgetType = AmWIDGET_DEFAULT;
    const char      *resourceBase, *alignmentStr, *widgetStr;
    char            widgetName[ TEXT_BUFFER],
                    resourceName[ HUGE_BUFFER];
    int             enumVal, size;
    void            *value;
    AmSlickPtr        clientData;

/* create the form with the unique form label that identifies that it's
   children are slick widgets */

    form = XtVaCreateWidget( UNIQUE_FORM_LABEL,
			     xmFormWidgetClass, parent,
			     NULL);

/* create the label on the left hand side */

    sprintf( widgetName, "%sLabel", name);
    label = XtVaCreateManagedWidget( widgetName,
				     xmLabelWidgetClass, form,
				     StickLeft, StickTop, StickBottom,
				     NULL);

/* uh, are we supposed to be aligned?  If not, just go for alignment begining */ 
    resourceBase = AmGetResourceName( form);
    sprintf( resourceName, "%s.%s.alignment", resourceBase, name);
    alignmentStr = AmGetDefault( resourceName, ( const char *) NULL);
    if ( alignmentStr == NULL) {
        XtVaSetValues( label,
		       XmNalignment, XmALIGNMENT_BEGINNING,
		       NULL);
    }

/* if the record isn't NULL, the value's already malloced by the 
   GetVaFields() call.  If the record is NULL, then we allocate the 
   value here */

    if ( record != NULL) {
	if (( type == AmSTRING || type == AmFILE) && 
	    ( storageType == AmDYNAMIC)) {
	    value = *(char **)valuePtr;
	} else {
	    value = valuePtr;
	}
    } else {
	if ( type == AmSTRING || type == AmFILE) {

/* if the storage type is dynamic, then the argument is a char **.  If the
   storage type is static, then it's a char * */

	    if ( storageType == AmDYNAMIC) {
		if ( *( char **) valuePtr != NULL) {
		    size = AmSizeData( *(char **)valuePtr, type);
		    value = ( void *)malloc( size);
		    if ( value == NULL) {
			AN_DEBUG( 1, ("malloc failed! %s", name));
			DO_ERROR( AmMALLOC_FAILED);
			return( NULL);
		    }
		    memcpy( value, *(char **)valuePtr, size);
		} else {	
		    value = NULL;
		}
	    } else {
		size = AmSizeData(( char *) valuePtr, type);
		value = ( void *) malloc( size);
		if ( value == NULL) {
		    AN_DEBUG( 1, ("malloc failed! %s", name));
		    DO_ERROR( AmMALLOC_FAILED);
		    return( NULL);
		}
		memcpy( value, valuePtr, size);
	    }
	} else {
	    size = AmDataTypeEnumToSize( type);
	    value = ( void *)malloc( size);
	    if ( value == NULL) {
		AN_DEBUG( 1, ("malloc failed! %s", name));
		DO_ERROR( AmMALLOC_FAILED);
		return( NULL);
	    }
	    memcpy( value, valuePtr, size);
	}
    }

/* see if there's a widget type specified in the resource file */

    sprintf( resourceName, "%s.%s.widget", resourceBase, name);
    widgetStr = AmGetDefault( resourceName, ( const char *) NULL);

/* map the widget string to an enumeration */

    if ( widgetStr != NULL) {
	if ( AmGetStrEnum( "widget", ( char *) widgetStr, &enumVal)) {
            widgetType = ( AmWidgetEnum)enumVal;
        } else {
            AmPrintError( "value for enumeration %s not found", widgetStr);
        }
    }

/* stuff the client data structure */

    clientData = ( AmSlickPtr)malloc( sizeof( AmSlickRec));
    clientData->record = record;
    clientData->widget = NULL;
    clientData->tag = name;
    clientData->value = value;
    clientData->type = type;
    clientData->range = NULL;
    clientData->buttons = NULL;
    clientData->size = AmSizeData( valuePtr, type);
    clientData->units = units;

/* if there's no record, then we use a value pointer, otherwise the record
   contains the data */

    if ( record == NULL) {

/* if the storage type is static or dynamic, then we set the pointer
   directly, otherwise, we allocate a local buffer to contain the
   information */

	if (( storageType == AmSTATIC) || ( storageType == AmDYNAMIC)) {
	    clientData->valuePtr = valuePtr;
	} else {
	    clientData->valuePtr = malloc( clientData->size);
	    if ( clientData->valuePtr == NULL) {
		AN_DEBUG( 1, ("malloc failed! %s", name));
		DO_ERROR( AmMALLOC_FAILED);
		return( NULL);
	    }
	    memcpy( clientData->valuePtr, valuePtr, clientData->size);
	}
    } else {
	clientData->valuePtr = NULL;
    }
    clientData->storageType = storageType;
    clientData->lengthPtr = lengthPtr;

    widgetType = AmCoerceWidgetType( type, widgetType);
    clientData->widgetType = widgetType;
    switch( widgetType) {
        case AmWIDGET_TEXT:
            editable = AmCreateTextFieldWidget( form, label, clientData);
            break;
	case AmWIDGET_MULTILINE_TEXT:
	    editable = AmCreateMultiLineTextWidget( form, label, clientData);
	    break;
        case AmWIDGET_SCALE:
            editable = AmCreateScaleWidget( form, label, clientData);
            break;
        case AmWIDGET_TOGGLE:
            editable = AmCreateToggleButtonWidget( form, label, clientData);
            break;
        case AmWIDGET_BUTTON:
            editable = AmCreatePushButtonWidget( form, label, clientData);
            break;
        case AmWIDGET_FILESB:
            editable = AmCreateFileSelectionBox( form, label, clientData);
            break;
        case AmWIDGET_LIST:
            editable = AmCreateListWidget( form, label, clientData);
            break;
        case AmWIDGET_OPTION:
            editable = AmCreateOptionMenuWidget( form, label, clientData);
            break;
        case AmWIDGET_COLOR_BUTTON:
            editable = AmCreateColorPushButtonWidget( form, label, clientData);
            break;
        default:
            AmPrintResourceError( "unknownWidgetType",
			          name, clientData->widgetType);
            free( clientData);
            free( value);
            free( name);
            return( NULL);
    }

/* set the client data in the editable object so it can be recovered by
   the AmSetObjectValue() function.  Add the callback to free the data
   when the editable object is destroyed */

    if ( editable != NULL) {
	XtVaSetValues( editable, 
		       XmNuserData, clientData, 
		       NULL);
	XtAddCallback( editable, XmNdestroyCallback,
		       ( XtCallbackProc) slickDestroyCallback,
		       clientData);
    } else {
	AmPrintResourceError( "failedToCreateWidget", name);
    }
    clientData->widget = editable;
    XtManageChild( form);
    return( editable);
}

/********************************************************************** 
 *  top level AmAddObject function which uses the AmRecordClass interface:
 *  record - AmRecordClass object to retreive and edit fields from
 *  parent - parent widget
 *  tag - string tag used to look up dataType, storageType, length and units 
 **********************************************************************/ 

Widget 
AmAddObject( AmRecordClass *record,
           Widget      parent,
           char        *tag)
{
    AmDataTypeEnum	type;
    void		*value;

/* find the type for the tag. */

    type = AmGetTagType( tag);
    if ( type == AmUNKNOWN) {
	AmPrintResourceError( "unknownTypeForTag", tag);
	return( NULL);
    }

/* if it's a string or file, the value is allocated by GetVaFields() */

    if ( type == AmSTRING || type == AmFILE) {
        if ( record->GetVaFields( tag, &value, NULL) == 1) {
            AmPrintError( "tag %s not found", tag);
            return( NULL);
        }
	return( AmAddObject( parent, &value, tag, type, AmUNIT_UNDEFINED,
			   AmDYNAMIC, NULL, record));
    } else {

/* otherwise, we allocate it by hand depending on the type, and pass it in */

        if (( value = ( void *) malloc( AmDataTypeEnumToSize( type))) == NULL) {
	    AmPrintResourceError( "mallocFailed", tag);
	    return( NULL);
	}
        if ( record->GetVaFields( tag, value, NULL) == 1) {
            AmPrintResourceError( "tagNotFound", tag);
            free( value);
            return( NULL);
        }
	return( AmAddObject( parent, value, tag, type, AmUNIT_UNDEFINED,
			   AmDYNAMIC, NULL, record));
    }
}

/********************************************************************** 
 *  Overloaded variant that gets the data type from the tag in strdefs.h 
 *  parent - parent widget
 *  valuePtr - pointer to value to edit
 *  tag - refers to tag from strdefs.h for data type.
 *  storageType - AmDYNAMIC, AmSTATIC, or AmLOCAL 
 *  lengthPtr - length of array value.  Unused for scalar values 
 **********************************************************************/ 

Widget
AmAddObject( Widget          parent,
           void            *valuePtr,
           char            *tag,
           AmStorageTypeEnum storageType,
           int             *lengthPtr)
{
   return( AmAddObject( parent, valuePtr, tag, AmGetTagType( tag), 
		      AmUNIT_UNDEFINED, storageType, lengthPtr, NULL));
}

/********************************************************************** 
 *  lowest level AmAddObject() interface:
 *  parent - parent widget
 *  valuePtr - pointer to value to edit
 *  name - string name to look up resources in defaults file
 *  type - type of value to edit ( AmFLOAT, AmINT, ...)
 *  storageType - AmDYNAMIC, AmSTATIC, or AmLOCAL 
 *  lengthPtr - length of array value.  Unused for scalar values 
 **********************************************************************/ 

Widget
AmAddObject( Widget          parent,
           void            *valuePtr,
           char            *name,
	   AmDataTypeEnum	   type,
           AmStorageTypeEnum storageType,
           int             *lengthPtr)
{
    return( AmAddObject( parent, valuePtr, name, type, 
		       AmUNIT_UNDEFINED, storageType, lengthPtr, NULL));
}

/********************************************************************** 
 *   Even lower than the lowest level AmAddObject() interface:
 *   parent - parent widget
 *   valuePtr - pointer to value to edit
 *   name - string name to look up resources in defaults file
 *   type - type of value to edit ( AmFLOAT, AmINT, ...)
 *   units - units to display and read the value with 
 *   storageType - AmDYNAMIC, AmSTATIC, or AmLOCAL 
 *   lengthPtr - length of array value.  Unused for scalar values 
 **********************************************************************/ 

Widget
AmAddObject( Widget          parent,
           void            *valuePtr,
           char            *name,
	   AmDataTypeEnum	   type,
	   unitType	   units,
           AmStorageTypeEnum storageType,
           int             *lengthPtr)
{
    return( AmAddObject( parent, valuePtr, name, type, units,
		       storageType, lengthPtr, NULL));
}

/* set up OK button callback, which stuffs values via user events and pops
   down the dialog */

AnvilBoolean
AmSetOkButton( Widget button, AmRecordClass *record)
{
    XtAddCallback( button, XmNactivateCallback,
        ( XtCallbackProc) slickOkCallback,
        ( XtPointer)record);
 
    return( TRUE);
}

/* set up an Alright button callback, which stuffs values, but doesn't 
   pop down the dialog.  If the values are record values, they're not
   stuffed in the record, but held onto temporarily.  If they're pointer
   values, they're stuffed immediately */

AnvilBoolean
AmSetAlrightButton( Widget button, AmRecordClass *record)
{
    XtAddCallback( button, XmNactivateCallback,
        ( XtCallbackProc) slickAlrightCallback,
        ( XtPointer)record);
 
    return( TRUE);
}

/* set up the revert button to set widgets to their original values */

AnvilBoolean
AmSetRevertButton( Widget button)
{
    XtAddCallback( button, XmNactivateCallback,
        ( XtCallbackProc) slickRevertCallback,
        ( XtPointer)AmGetWidgetShell( button));
 
    return( TRUE);
}

/* pop down the dialog callback */

AnvilBoolean
AmSetCancelButton( Widget button, 
		 Widget formDlog)
{
    XtAddCallback( button, XmNactivateCallback,
        ( XtCallbackProc) AmPopdownCback,
        ( XtPointer)formDlog);
    return( TRUE);
}

/* send the OK event only to this widget...for non-record slick calls only */

static void singleOkCallback( Widget  w,
                              Widget    receiver,
			      XtPointer)
{
    AmSendUserEventChildren( AM_PKG_ID, AM_OK, NULL, receiver);
}

/* set callbacks so the widget stuffs the value immediately when the user
   hits enter, toggles, changes the slider, loses focus, etc */

AnvilBoolean AmImmediateAccept( Widget       w)
{
    if ( XtIsSubclass( w, xmTextFieldWidgetClass) ||
       XtIsSubclass( w, xmTextWidgetClass)) {
      XtAddCallback( w, XmNlosingFocusCallback,
                     ( XtCallbackProc) singleOkCallback, w);
      XtAddCallback( w, XmNactivateCallback,
                     ( XtCallbackProc) singleOkCallback, w);
    } else if ( XtIsSubclass( w, xmScaleWidgetClass)) {
      XtAddCallback( w, XmNvalueChangedCallback,
                     ( XtCallbackProc) singleOkCallback, w);
    } else if ( XtIsSubclass( w, xmToggleButtonWidgetClass)) {
      XtAddCallback( w, XmNvalueChangedCallback,
                     ( XtCallbackProc) singleOkCallback, w);
    } else if ( XtIsSubclass( w, xmPushButtonWidgetClass)) {
      XtAddCallback( w, XmNactivateCallback,
                     ( XtCallbackProc) singleOkCallback, w);
    } else if ( XtIsSubclass( w, xmPushButtonWidgetClass)) {
      XtAddCallback( XmFileSelectionBoxGetChild( w, XmDIALOG_TEXT),
                     XmNlosingFocusCallback, 
                     ( XtCallbackProc) singleOkCallback, w);
      XtAddCallback( XmFileSelectionBoxGetChild( w, XmDIALOG_TEXT),
                     XmNlosingFocusCallback, 
                     ( XtCallbackProc) singleOkCallback, w);
    } else if ( XtIsSubclass( w, xmListWidgetClass)) {
      XtAddCallback( w, XmNsingleSelectionCallback,
                     ( XtCallbackProc) singleOkCallback, w);

/* have to test if the widget is a rowcolumn of type XmMenuOption, 
   and add the callback to each pushbutton */

    } else if ( XtIsSubclass( w, xmRowColumnWidgetClass)) {
	uchar_t rowcolType;
	Widget  *children;
	int     numChildren, ii;

	XtVaGetValues( w, XmNrowColumnType, &rowcolType, NULL);
	if ( rowcolType == XmMENU_OPTION) {
	    XtVaGetValues( w, 
			   XmNchildren, &children,
			   XmNnumChildren, &numChildren,
			   NULL);
	    for ( ii = 0; ii < numChildren; ii++) {
		XtAddCallback( children[ ii], XmNactivateCallback,
			       ( XtCallbackProc) singleOkCallback, w);
	    }
	    free( children);
	} else {
	    return( FALSE);
	}
    } else {
	return( FALSE);
    }
    return( TRUE);
}

/* remove the label widget from a slick-added object */

Widget AmGetLabel( Widget	w)
{
    Widget	label = NULL, *children;
    int		numChildren, ii;

    XtVaGetValues( XtParent( w),
		   XmNchildren, &children,
		   XmNnumChildren, &numChildren,
		   NULL);
    for ( ii = 0; ii < numChildren; ii++) {
	if ( XtIsSubclass( children[ ii], xmLabelWidgetClass)) {
	    label = children[ ii];
	} 
    }
    XtFree(( char *) children);
    return( label);
}

