#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <Xm/Xm.h>
#include <Xm/Scale.h>
#include <Xm/TextF.h>
#include <Xm/Form.h>
#include <anvil/motif.h>
#include <anvil/textcheck.h>

/* update the slider when the text changes */

static void textUpdateScaleCallback( Widget	text,
				     Widget	scale,
				     XtPointer)
{
    char	*str;
    short	decimalPoints;
    int		scaleValue;
    float	textValue;

/* scan the value from the text field and set it in the scale */

    str = XmTextFieldGetString( text);
    XtVaGetValues( scale,
		   XmNdecimalPoints, &decimalPoints,
		   NULL);
    textValue = atof( str);
    scaleValue = (int)(textValue*pow( 10.0, decimalPoints));
    XmScaleSetValue( scale, scaleValue);
    free( str);
}

/* update the text when the slider changes */

static void scaleUpdateTextCallback( Widget	scale,
				     Widget	text,
				     XtPointer)
{
    int		scaleValue;
    short	decimalPoints;
    float	textValue;
    char	fmt[ 32], buffer[ 64];


    XtVaGetValues( scale,
		   XmNvalue, &scaleValue,
		   XmNdecimalPoints, &decimalPoints,
		   NULL);

/* use the decimal points from the scale to set the precision in the text */

    textValue = scaleValue/pow( 10.0, decimalPoints);
    sprintf( fmt, "%%.%df", decimalPoints);
    sprintf( buffer, fmt, textValue);
    XmTextFieldSetString( text, buffer);
}
    
/* combination slider and text, where the slider and text update each other */

void AmSliderText( Widget	parent,
		 char	*resName,
		 Widget	*form,
		 Widget	*text,
		 Widget	*scale)
{
    char	nameBuf[ 128];

    sprintf( nameBuf, "%sForm", resName);
    *form = XtVaCreateManagedWidget( nameBuf, xmFormWidgetClass, parent, NULL);
    sprintf( nameBuf, "%sText", resName);
    *text = XtVaCreateManagedWidget( nameBuf, xmTextFieldWidgetClass, *form, 
				     StickLeft, StickTop, StickBottom,
				     NULL);
    sprintf( nameBuf, "%sScale", resName);
    *scale = XtVaCreateManagedWidget( nameBuf, xmScaleWidgetClass, *form, 
				      RightOf( *text), StickTop, StickBottom,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
    XtAddCallback( *text, XmNactivateCallback,
		   ( XtCallbackProc) textUpdateScaleCallback, *scale);
    XtAddCallback( *text, XmNlosingFocusCallback,
		   ( XtCallbackProc) textUpdateScaleCallback, *scale);
    XtAddCallback( *scale, XmNdragCallback,
		   ( XtCallbackProc) scaleUpdateTextCallback, *text);
    XtAddCallback( *scale, XmNvalueChangedCallback,
		   ( XtCallbackProc) scaleUpdateTextCallback, *text);
}

/* update the text when the slider changes, and print the value with units */

static void scaleUpdateTextUnitsCallback( Widget	scale,
					  Widget	text,
					  unitType	units,
					  XtPointer)
{
    int		scaleValue;
    short	decimalPoints;
    float	textValue;
    char	buffer[ FIELDLEN];


    XtVaGetValues( scale,
		   XmNvalue, &scaleValue,
		   XmNdecimalPoints, &decimalPoints,
		   NULL);

/* use the decimal points from the scale to set the precision in the text */

    textValue = scaleValue/pow( 10.0, decimalPoints);
    AmPrintType( buffer, &textValue, AmFLOAT, units);
    XmTextFieldSetString( text, buffer);
}

/* combination slider and text, where the slider and text update each other */

void AmSliderTextUnits( Widget	parent,
		      char	*resName,
		      float	value,
		      unitType	units,
		      Widget	*form,
		      Widget	*text,
		      Widget	*scale)
{
    char	nameBuf[ RESLEN];

    sprintf( nameBuf, "%sForm", resName);
    *form = XtVaCreateManagedWidget( nameBuf, xmFormWidgetClass, parent, NULL);
    sprintf( nameBuf, "%sText", resName);
    *text = XtVaCreateManagedWidget( nameBuf, xmTextFieldWidgetClass, *form, 
				     StickLeft, StickTop, StickBottom,
				     NULL);
    AmUnitVarEntry( *text, &value, AmFLOAT, units, NULL);
    sprintf( nameBuf, "%sScale", resName);
    *scale = XtVaCreateManagedWidget( nameBuf, xmScaleWidgetClass, *form, 
				      RightOf( *text), StickTop, StickBottom,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
    XtAddCallback( *text, XmNactivateCallback,
		   ( XtCallbackProc) textUpdateScaleCallback, *scale);
    XtAddCallback( *text, XmNlosingFocusCallback,
		   ( XtCallbackProc) textUpdateScaleCallback, *scale);
    AmAddVaCallback( *scale, XmNdragCallback,
		   ( VarargsCallback) scaleUpdateTextUnitsCallback, 
		   *text, units, 
		   END_LIST);
    AmAddVaCallback( *scale, XmNvalueChangedCallback,
		   ( VarargsCallback) scaleUpdateTextUnitsCallback, 
		   *text, units,
		   END_LIST);
}
