#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <anvil/project.h>
#include <anvil/strlib.h>
#include <anvil/errlib.h>
#include <anvil/motif.h>
#include <anvil/regexp.h>
#include <anvil/macros.h>
#include <anvil/record.h>
#include <anvil/fns.h>
#include <anvil/textcheck.h>

/* check function takes a pointer to a value, clientData, and a string
   buffer which is used for printing errors
   scan function takes a string and a void pointer to the value to be scanned
   print function takes the buffer, value, variable and units. */

typedef int 	( *checkFn)( void *, void *);
typedef void 	( *AmPrintFn)( unitType, void *, char *);

typedef struct unitEntryData_ {
    unitType		units;		/* units for the value */
    AmDataTypeEnum	dataType;
    char		*regexp;	/* regular expression to check */
    void		*checkData;	/* losing focus */ 
} unitEntryDataRec, *unitEntryDataPtr;

static int rangeCheckFloat( float 	*value,
			    float	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckInt( int 	*value,
			  int	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckDouble( double *value,
			     double *rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckChar( char *value,
			   char	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckUnsignedChar( uchar_t 	*value,
				   uchar_t	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckShort( short 	*value,
			    short	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckUnsignedShort( ushort_t 	*value,
				    ushort_t 	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckLong( long *value,
			   long	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckUnsignedLong( ulong_t 		*value,
				   ulong_t		*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

static int rangeCheckUnsignedInt( uint_t 	*value,
				  uint_t	*rangePair)
{
    if ( *value < rangePair[ 0]) {
	return( -1);
    }
    if ( *value > rangePair[ 1]) {
	return( 1);
    }
    return( 0);
}

/* range check depending on type */

static int rangeCheckType( void		*value,
			   void		*range,
			   AmDataTypeEnum	dataType)
{

/* if there's no range to check, then return OK */

    if ( range == NULL) {
	return( 0);
    }
    switch ( dataType) {
    case AmFLOAT:
	return( rangeCheckFloat(( float *) value, ( float *) range));
    case AmDOUBLE:
	return( rangeCheckDouble(( double *) value, ( double *) range));
    case AmSHORT:
	return( rangeCheckShort(( short *) value, ( short *) range));
    case AmINT:
	return( rangeCheckInt(( int *) value, ( int *) range));
    case AmLONG:
	return( rangeCheckLong(( long *) value, ( long *) range));
    case AmCHAR:
	return( rangeCheckChar(( char *) value, ( char *) range));
    case AmUNSIGNED_CHAR:
	return( rangeCheckUnsignedChar(( uchar_t *) value, 
				       ( uchar_t *) range));
    case AmUNSIGNED_INT:
	return( rangeCheckUnsignedInt(( uint_t *) value, ( uint_t *) range));
    case AmUNSIGNED_LONG:
	return( rangeCheckUnsignedLong(( ulong_t *) value, 
				       ( ulong_t *) range));
    case AmUNSIGNED_SHORT:
	return( rangeCheckUnsignedShort(( ushort_t *)  value, 
					( ushort_t *) range));
    default:
	return( 0);
    }
}

/* Scan function for any type. */

AnvilBoolean AmScanType( char 		*buffer,
		  void 		*value,
		  AmDataTypeEnum  dataType,
		  unitType      units)
{
    float	floatVal;

    if ( dataType == AmSTRING) {
	if ( buffer != NULL) {
	    strcpy( ( char *) value, buffer);
	} else {
	    (( char *) value)[ 0] = '\0';
	}
	return( TRUE);
    }

/* scan the value into a float if we can... */

    if ( !AmScanValue( units, buffer, &floatVal)) {
	return( FALSE);
    }

/* use conversions for all numeric values */

    switch ( dataType) {
    case AmFLOAT:
	*( float *) value = floatVal;
	break;
    case AmDOUBLE:
	*( double *) value = floatVal;
	break;
    case AmSHORT:
	*( short *) value = (short)floatVal;
	break;
    case AmINT:
	*( int *) value = (int)floatVal;
	break;
    case AmLONG:
	*( long *) value = (long)floatVal;
	break;
    case AmUNSIGNED_INT:
	*( uint_t *) value = (uint_t)floatVal;
	break;
    case AmUNSIGNED_LONG:
	*( ulong_t *) value = (ulong_t)floatVal;
	break;
    case AmUNSIGNED_SHORT:
	*( ushort_t *) value = (ushort_t)floatVal;
	break;
    default:
	return( FALSE);
    }
    return( TRUE);
}

/* Print function for any type. */

void AmPrintType( char         *buffer,
	        void         *value,
	        AmDataTypeEnum dataType,
	        unitType     units)
{
    float	floatVal;

    if ( dataType == AmSTRING) {
	if ( value != NULL) {
	    strcpy( buffer, ( char *) value);
	} else {
	    buffer[ 0] = '\0';
	}
	return;
    }
    switch ( dataType) {
    case AmFLOAT:
	floatVal = *( float *) value;
	break;
    case AmDOUBLE:
	floatVal = *( double *) value;
	break;
    case AmSHORT:
	floatVal = *( short *) value;
	break;
    case AmINT:
	floatVal = *( int *) value;
	break;
    case AmLONG:
	floatVal = *( long *) value;
	break;
    case AmUNSIGNED_INT:
	floatVal = *( uint_t *) value;
	break;
    case AmUNSIGNED_LONG:
	floatVal = *( ulong_t *) value;
	break;
    case AmUNSIGNED_SHORT:
	floatVal = *( ushort_t *) value;
	break;
    case AmBOOLEAN:
	floatVal = *( AnvilBoolean *) value;
	break;
    default:
	floatVal = 0.0;
    }
    AmPrintValue( units, floatVal, buffer);
}

/* when gaining focus, select the text, so that it can be replaced by
   typing a single character. Note the different functions for Text and
   Text Fields */

static void focusHighlightCallback( Widget    w, 
				    XtPointer,
				    XtPointer)
{
    if ( XmIsTextField( w)) {
	XmTextFieldSetSelection( w, 0, XmTextFieldGetLastPosition( w),
				 XtLastTimestampProcessed( XtDisplay( w)));
    } else if ( XmIsText( w)) {
	XmTextSetSelection( w, 0, XmTextGetLastPosition( w),
			    XtLastTimestampProcessed( XtDisplay( w)));
    }
}

/* given the original string, string to inser, and the start pos, end pos,
   and length from an text verify callback struct, insert the string as it
   will appear if everything is OK */

static char *insertString( char 	*origStr,
			   char 	*insertStr,
			   int  	startpos,
			   int		endpos,
			   int		newlength)
{
    static char	newStr[ 512];
    int		oldlength;

/* if the old string isn't NULL, copy up to the starting insert position,
   insert the new string, and append the old string, after the ending insert 
   position to its end, then return the static string buffer */ 

    if ( origStr != NULL) {
	oldlength = strlen( origStr);
	strncpy( newStr, origStr, startpos);
    }
    strncpy( &newStr[ startpos], insertStr, newlength);
    if ( origStr != NULL) {
	strcpy( &newStr[ startpos + newlength], &origStr[ endpos]);
    }
    return( newStr);
}

/* when the user enters a character, check it against a regular expression */

static void regexpCheckCallback( Widget				w,
				 char				*regexpStr,
				 XmTextVerifyCallbackStruct	*cb)
{
    char	*insertStr = cb->text->ptr, *origStr, *newStr;

/* grab the original text from the TextField Widget */

    origStr = XmTextFieldGetString( w);

/* if nothing got inserted, then we don't care */

    if ( insertStr == NULL) {
        XtFree( origStr);
	return;
    }

/* insert the string and check it against the regular expression */

    newStr = insertString( origStr, insertStr, 
			   cb->startPos, cb->endPos, cb->text->length);
    cb->doit = AmRegexpMatch( regexpStr, newStr, TRUE);
    XtFree( origStr);
}

/* preprovided check function checks against a pair of floats */

AnvilBoolean AmCheckRangeFn( float	*value,
		      void	*clientData,
		      char	*errmsg)
{
    unitEntryDataPtr	unitEntryData = ( unitEntryDataPtr) clientData;
    float		*rangePair = ( float *) unitEntryData->checkData;
    char		valbuf[ 128];

    if ( *value < rangePair[ 0]) {
	AmPrintValue( unitEntryData->units, *value, valbuf);
	sprintf( errmsg, "value must be more than %s", valbuf);
	*value = rangePair[ 0];
	return( FALSE);
    }
    if ( *value > rangePair[ 1]) {
	AmPrintValue( unitEntryData->units, *value, valbuf);
	sprintf( errmsg, "value must be more than %s", valbuf);
	*value = rangePair[ 1];
        return( FALSE);
    }
    return( TRUE);
}

/* return the appropriate size for a data type */

int AmSizeData( void		*dataPtr,
	      AmDataTypeEnum	dataType)
{
    char *data;

    if ( dataType == AmSTRING || dataType == AmFILE) {
	data = ( char *) dataPtr;
	return( strlen( data) + 1);
    } else {
	return( AmDataTypeEnumToSize( dataType));
    }
}

/* call the check function.  If it's our preset function for range checking,
   then pass the whole unitEntryData struct to it, so it can print the
   error message with variable and units */

static void callCheckFnCallback( Widget			w,
				 unitEntryDataPtr	unitEntryData,
				 XtPointer)
{
    char	*str, buffer[ 512];
    int		rangeFlag, size;
    void	*value, *ptr;

/* read the string from the text buffer, and allocate a value large enough
   to hold it */

    XtVaGetValues( w, XmNvalue, &str, NULL);
    size = AmSizeData( str, unitEntryData->dataType);
    value = ( void *) malloc( size);
    if ( value == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return;
    }
    if ( !AmScanType( str, value, unitEntryData->dataType, 
		    unitEntryData->units)) {
	free( value);
	XtFree( str);
	return;
    }
    XtFree( str);

/* apply the check function, and if there's an error, use the data type to
   pull the values from the check data and create an error message */

    rangeFlag = rangeCheckType( value, unitEntryData->checkData,
				unitEntryData->dataType);
    if ( rangeFlag == -1) {
	AmPrintType( buffer, unitEntryData->checkData,
		   unitEntryData->dataType, unitEntryData->units);
	AmSetErrorColor( w, TRUE);
#if 0
	AmPrintResourceError( "valueMustGreater", buffer);
#endif
    } else if ( rangeFlag == 1) {
	ptr = &(( char *) unitEntryData->checkData)[ size];
	AmPrintType( buffer, ptr,
		   unitEntryData->dataType, unitEntryData->units);
	AmSetErrorColor( w, TRUE);
#if 0
	AmPrintResourceError( "valueMustLess", buffer);
#endif
    } else {

/* reprint the value if it was valid */

	AmSetErrorColor( w, FALSE);
	AmPrintType( buffer, value,
		   unitEntryData->dataType, unitEntryData->units);
    }
    free( value);
    AN_DEBUG( 10, ("buffer = %s\n", buffer));
    XmTextFieldSetString( w, buffer);
}

/* create the string that the bubble help shows for the range */

static AnvilBoolean createRangeHelpString( AmDataTypeEnum	dataType,
				      unitType		units,
				      void 		*checkData,
				      char		*helpString)
{
    int		size;
    char	minbuf[ 512], maxbuf[ 512];
    uchar_t	*checkPtr = ( uchar_t *) checkData;

    if ( checkData == NULL) {
	return( FALSE);
    }
    size = AmDataTypeEnumToSize( dataType);
    if ( size <= 0) {
	return( FALSE);
    }
    AmPrintType( minbuf, checkPtr, dataType, units);
    AmPrintType( maxbuf, &checkPtr[ size], dataType, units);
    sprintf( helpString, "%s < X < %s", minbuf, maxbuf);
    return( TRUE);
}

/* set up all the information for checking values as they are entered.
   var & units define how the string is to be displayed, and what 
   regular expression to use (always float format w/units).
   checkFn and checkData are called on focus out to check the value.  
   typically, it's a range function */

void AmUnitVarEntry( Widget	w,
		   void		*val,
		   AmDataTypeEnum	dataType,
		   unitType	units,
		   void		*checkData)
{
    unitEntryDataPtr	unitEntryData;
    char		buffer[ FIELDLEN], *regexp, helpString[ MSGLEN],
			resourceName[ HUGE_BUFFER];

/* create the struct to hold all the information passed about the value */

    unitEntryData = ( unitEntryDataPtr) malloc( sizeof( unitEntryDataRec));
    unitEntryData->dataType = dataType;
    unitEntryData->units = units;
    unitEntryData->checkData = checkData;

/* get the regular expression for the units */

    if ( dataType != AmSTRING) {
	AmGetRegexp( units, &regexp);
	unitEntryData->regexp = regexp;
    } else { 
	unitEntryData->regexp = NULL;
    }

/* print the value in the text widget */

    AmPrintType( buffer, val, dataType, units);
    XmTextFieldSetString( w, buffer);

/* on activate or losing focus, call the check function, on text entry,
   call the regular expression checking function, and on destroy, free the
   unitEntryData struct */

    XtAddCallback( w, XmNactivateCallback,
                   ( XtCallbackProc) callCheckFnCallback, unitEntryData);
    XtAddCallback( w, XmNlosingFocusCallback,
                   ( XtCallbackProc) callCheckFnCallback, unitEntryData);
    XtAddCallback( w, XmNfocusCallback,
                   ( XtCallbackProc) focusHighlightCallback, NULL);
    XtAddCallback( w, XmNdestroyCallback,
		   ( XtCallbackProc) AmFreeStructPtrCallback, unitEntryData);

/* put in the bubble help to show the range */

    if ( checkData != NULL) {
	createRangeHelpString( dataType, units, checkData, helpString);
	sprintf( resourceName, "%s.quickHelp", AmGetResourceName( w));
	AmPutDefault( resourceName, helpString);
    }
    if ( unitEntryData->regexp != NULL) {
        AmRegexpCheck( w, unitEntryData->regexp);
    }
}

/* just set up the regular expression checking */

void AmRegexpCheck( Widget	w,
		  char		*regexp)
{
    XtAddCallback( w, XmNmodifyVerifyCallback,
		   ( XtCallbackProc) regexpCheckCallback, regexp);
}

/* convienience functions for pulling values from widgets */

int AmGetTextFieldInt( Widget	w)
{
    int		val;
    char	*str = XmTextFieldGetString( w);

    val = atoi( str);
    FREE( str);
    return( val);
}

float AmGetTextFieldFloat( Widget	w)
{
    float		val;
    char	*str = XmTextFieldGetString( w);

    val = atof( str);
    FREE( str);
    return( val);
}
