#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <anvil/tiffio.h>
#include <anvil/project.h>
#include <anvil/tif_ximage.h>

/* take an XImage and write it to a TIFF file */

AnvilBoolean AmWriteXImageToTIFF( Display	*display,
			   Colormap	cmap,
			   XImage	*image,
			   char		*tiffFile)
{
    ushort_t	*redCmap, *blueCmap, *greenCmap;
    uchar_t	*outbuf;
    TIFF 	*tif;
    int		numcolors, ii, jj;
    XColor	cell;
    ulong_t	pixel;

    if (( tif = TIFFOpen( tiffFile, "w")) == NULL) {
	return( FALSE);
    }
    if ( !TIFFSetField( tif, TIFFTAG_BITSPERSAMPLE, image->depth)) {
	return( FALSE);
    }
    if ( image->depth == 8) {
	if ( !TIFFSetField( tif, TIFFTAG_SAMPLESPERPIXEL, 1)) {
	    return( FALSE);
	}
	if ( !TIFFSetField( tif, TIFFTAG_BITSPERSAMPLE, 8)) {
	    return( FALSE);
	}
    } else if ( image->depth == 24) {
	if ( !TIFFSetField( tif, TIFFTAG_SAMPLESPERPIXEL, 3)) {
	    return( FALSE);
	}
	if ( !TIFFSetField( tif, TIFFTAG_BITSPERSAMPLE, 8, 8, 8)) {
	    return( FALSE);
	}
    }

/* set the format to "chunky" format */

    if ( !TIFFSetField( tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG)) {
	return( FALSE);
    }

/* 0,0 is upper left hand corner */

    if ( !TIFFSetField( tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT)) {
    	return( FALSE);
    }
    if ( image->depth == 24) {
	if ( !TIFFSetField( tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB)) {
	    return( FALSE);
	}
	outbuf = ( uchar_t *) malloc( 3*image->width);
    } else {
	if ( !TIFFSetField( tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_PALETTE)) {
	    return( FALSE);
	}

/* if it's a palette image, then write out the colormap */

	numcolors = 1 << image->depth;
	redCmap = ( ushort_t *) malloc( sizeof( ushort_t)*numcolors);
	blueCmap = ( ushort_t *) malloc( sizeof( ushort_t)*numcolors);
	greenCmap = ( ushort_t *) malloc( sizeof( ushort_t)*numcolors);
	for ( ii = 0; ii < numcolors; ii++) {
	    cell.pixel = ii;
	    XQueryColor( display, cmap, &cell);
	    redCmap[ ii] = cell.red;
	    blueCmap[ ii] = cell.blue;
	    greenCmap[ ii] = cell.green;
	}
	if ( !TIFFSetField( tif, TIFFTAG_COLORMAP,
			    redCmap, greenCmap, blueCmap)) {
	    return( FALSE);
	}
	free( redCmap);
	free( blueCmap);
	free( greenCmap);
	outbuf = ( uchar_t *) malloc( image->width);
    }
    if ( !TIFFSetField( tif, TIFFTAG_IMAGEWIDTH, image->width)) {
	return( FALSE);
    }
    if ( !TIFFSetField( tif, TIFFTAG_IMAGELENGTH, image->height)) {
	return( FALSE);
    }
    for ( ii = 0; ii < image->height; ii++) {

/* if it's an 8-bit image, then copy it straight, if it's 24-bit, then get
   the RGB values from the pixel and write to the output buffer before writing
   it */

	if ( image->depth == 8) {
	    memcpy( outbuf, &image->data[ ii*image->bytes_per_line], 
		    image->width);
	} else if ( image->depth == 24) {
	    for ( jj = 0; jj < image->width; jj++) {
		pixel = *( ulong_t *) &image->data[ ii*image->bytes_per_line +
						    jj*sizeof( ulong_t)];
		outbuf[ jj*3] = pixel & 0xff;
		outbuf[ jj*3 + 1] = ( pixel & 0xff00) >> 8;
		outbuf[ jj*3 + 2] = ( pixel & 0xff0000) >> 16;
	    }
	}
	if ( !TIFFWriteScanline( tif, outbuf, ii, 0) < 0) {
	    return( FALSE);
	}
    }
    free( outbuf);
    TIFFClose( tif);
    return( TRUE);
}
