#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <Xm/Xm.h>
#include <anvil/errlib.h>
#include <anvil/motif.h>
#include <anvil/list.h>
#include <anvil/user_data.h>


static void
freeUserDataItem( AmUserDataItemPtr pUserDataItem)
{
    if ( pUserDataItem != NULL) {

        FREE( pUserDataItem->szKey);
        if ( pUserDataItem->fnFreeData != NULL) {
            pUserDataItem->fnFreeData( pUserDataItem->pData);
        }
        FREE( pUserDataItem);

    }
}


static void
destroyCallback( Widget    w,
                 XtPointer xpClientData,
                 XtPointer xpCallData)
{
   AmUserDataPtr pUserData;

   XtVaGetValues( w, XmNuserData, &pUserData, NULL);
   if ( pUserData != NULL) {
       if ( pUserData->iMagicNo == USER_DATA_MAGIC) { 
           AmFreeLst( pUserData->lstUserData, ( AmItemFn) freeUserDataItem);
           FREE( pUserData);
       }
   }
}


AmErrorCode
AmAddUserData( Widget     w,
               const char *szKey,
               void       *pData,
               AmItemFn   fnFreeData)
{
    AmUserDataPtr     pUserData;
    AmUserDataItemPtr pUserDataItem;
    AmListPtr         lstTemp;

    XtVaGetValues( w, XmNuserData, &pUserData, NULL);
    if ( pUserData == NULL) {
        pUserData = ( AmUserDataPtr) malloc( sizeof( AmUserDataRec)); 
        if ( pUserData == NULL) {
            DO_RETURN_ERROR( AmMALLOC_FAILED);
        }
        pUserData->iMagicNo = USER_DATA_MAGIC;
        pUserData->lstUserData = NULL;

        XtVaSetValues( w, XmNuserData, pUserData, NULL);
        XtAddCallback( w, XmNdestroyCallback, 
                       ( XtCallbackProc) destroyCallback,
                       NULL);

    } else if ( pUserData->iMagicNo != USER_DATA_MAGIC) {
        DO_RETURN_ERROR( AmUNKNOWN_USER_DATA);
    }

    for ( lstTemp = pUserData->lstUserData;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pUserDataItem = ( AmUserDataItemPtr) lstTemp->data;
        if ( strcmp( pUserDataItem->szKey, szKey) == 0) {
            DO_RETURN_ERROR( AmUSER_DATA_KEY_EXISTS);
        }
    }

    pUserDataItem = ( AmUserDataItemPtr) malloc( sizeof( AmUserDataItemRec));
    if ( pUserDataItem == NULL) {
        DO_RETURN_ERROR( AmMALLOC_FAILED);
    }

    pUserDataItem->szKey = strdup( szKey);
    pUserDataItem->pData = pData;
    pUserDataItem->fnFreeData = fnFreeData;

    pUserData->lstUserData = AmInsertNodeLst( pUserDataItem,
                                              pUserData->lstUserData);

    return( AmOK);
}


void *
AmGetUserData( Widget w,
               const char *szKey)
{
    AmUserDataPtr     pUserData;
    AmUserDataItemPtr pUserDataItem;
    AmListPtr         lstTemp;

    if ( w == NULL) {
        DO_ERROR( AmNULL_WIDGET);
        return( NULL);
    }

    XtVaGetValues( w, XmNuserData, &pUserData, NULL);
    if ( pUserData == NULL) {
        DO_ERROR( AmMISSING_USER_DATA);
        return( NULL);
    } else if ( pUserData->iMagicNo != USER_DATA_MAGIC) {
        DO_ERROR( AmUNKNOWN_USER_DATA);
        return( NULL);
    }

    for ( lstTemp = pUserData->lstUserData;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pUserDataItem = ( AmUserDataItemPtr) lstTemp->data;
        if ( strcmp( pUserDataItem->szKey, szKey) == 0) {
            break;
        }
    }

    if ( lstTemp == NULL) {
        DO_ERROR( AmMISSING_USER_DATA_KEY);
        return( NULL);
    }

    return( pUserDataItem->pData);
}


void *
AmRemoveUserData( Widget w,
                  const char *szKey)
{
    AmUserDataPtr     pUserData;
    AmUserDataItemPtr pUserDataItem;
    AmListPtr         lstTemp, lstPrev;
    void              *pData;

    if ( w == NULL) {
        DO_ERROR( AmNULL_WIDGET);
        return( NULL);
    }

    XtVaGetValues( w, XmNuserData, &pUserData, NULL);
    if ( pUserData == NULL) {
        DO_ERROR( AmMISSING_USER_DATA);
        return( NULL);
    } else if ( pUserData->iMagicNo != USER_DATA_MAGIC) {
        DO_ERROR( AmUNKNOWN_USER_DATA);
        return( NULL);
    }

    lstPrev = NULL;
    for ( lstTemp = pUserData->lstUserData;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pUserDataItem = ( AmUserDataItemPtr) lstTemp->data;
        if ( strcmp( pUserDataItem->szKey, szKey) == 0) {
            break;
        }

        lstPrev = lstTemp;
    }

    if ( lstTemp == NULL) {
        DO_ERROR( AmMISSING_USER_DATA_KEY);
        return( NULL);
    }

    if ( lstPrev != NULL) {
        lstPrev->next = lstTemp->next;
    } else {
        pUserData->lstUserData = lstTemp->next; 
    }

    pData = pUserDataItem->pData; 

    FREE( pUserDataItem->szKey);
    FREE( pUserDataItem);
    FREE( lstTemp);

/*
 * If the user data list is empty remove the user data item from
 *   the widget.  Also get rid of the destroy callback which attempts
 *   to remove all the user data items when the widget is destroyed.
 */

    if ( pUserData->lstUserData == NULL) {
        XtRemoveCallback( w, XmNdestroyCallback, 
                          ( XtCallbackProc) destroyCallback,
                          NULL);
        XtVaSetValues( w, XmNuserData, NULL, NULL);
        FREE( pUserData);
    }

    return( pData);
}
