#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <search.h>
#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>
#include <Xm/Xm.h>
#include <anvil/motif.h>
#include <anvil/list.h>
#include <anvil/errlib.h>
#include <anvil/va_cback.h>
#include <anvil/va_cbackP.h>
#include <anvil/userevents.h>
#include <anvil/usereventsP.h>

static void 	*handlerRoot = NULL;
static void 	*vaHandlerRoot = NULL;
extern char	*END_LIST;

/* is child a descendent of parent? */

static int descendentOf( Widget	parent,
			 Widget	child)
{

/* everyone is a child of nothing, nobody is a child of everyone */ 

    if (( parent == NULL) || ( child == NULL)) {
	return( TRUE);
    }
    while ( child != NULL) {
	if ( child == parent) {
	    return( TRUE);
	} 
	child = XtParent( child);
    }
    return( FALSE);
}

/* find the handler list for a particular event and package id */

static int cmpEvent( AmUserHandlerListPtr	ah,
		     AmUserHandlerListPtr	bh)
{
    if ( ah->packageId == bh->packageId) {
    	return( ah->eventId - bh->eventId);
    } else {
    	return( ah->packageId - bh->packageId);
    }
}

/* comparison function for user event handlers */

static int cmpHandlerWidget( AmUserHandlerPtr	a,
			     AmUserHandlerPtr	b)
{
    return(( a->proc == b->proc) &&
	   ( a->clientData == b->clientData) &&
	   ( a->widget == b->widget));
}

/* check the handler-called-once flag, and see if the widget is the 
   descendent of the targeted parent */

static int deleteThisHandler( Widget		a,
			      AmUserHandlerPtr	b)
{
    if ( a == NULL) {
	return( b->deleted || b->callOnce);
    } else {
	return( b->deleted || ( b->callOnce && descendentOf( a, b->widget)));
    }
}

/* register an interest in a user event */

AnvilBoolean AmAddCallback( int		packageId,
		     int		eventId,
		     callbackProc	proc,
		     void		*clientData)
{
    return( addCallbackHandler( packageId, eventId, proc, clientData,
		FALSE, NULL));
}

/* tie a callback to an event for a one-time call, but set it up so it's 
   removed if the widget is destroyed */

AnvilBoolean AmAddCallbackWidgetOnce( int		packageId,
			       int		eventId,
			       Widget		w,
			       callbackProc	proc,
			       void		*clientData)
{
    AmUserDestroyPtr	userDestroy;

    userDestroy = ( AmUserDestroyPtr) malloc( sizeof( AmUserDestroyRec));
    userDestroy->packageId = packageId;
    userDestroy->eventId = eventId;
    userDestroy->proc = proc;
    userDestroy->clientData = clientData;
    XtAddCallback( w, XmNdestroyCallback, 
		   ( XtCallbackProc) removeCallbackDestroy, userDestroy);
    return( addCallbackHandler( packageId, eventId, proc, clientData, TRUE, w));
}

/* tie a callback to an event, but set it up so it's removed if the 
   widget is destroyed */

AnvilBoolean AmAddCallbackWidget( int		packageId,
			   int		eventId,
			   Widget	w,
			   callbackProc	proc,
			   void		*clientData)
{
    AmUserDestroyPtr	userDestroy;

    userDestroy = ( AmUserDestroyPtr) malloc( sizeof( AmUserDestroyRec));
    userDestroy->packageId = packageId;
    userDestroy->eventId = eventId;
    userDestroy->proc = proc;
    userDestroy->clientData = clientData;
    XtAddCallback( w, XmNdestroyCallback, 
		   ( XtCallbackProc) removeCallbackDestroy, userDestroy);
    return( addCallbackHandler( packageId, eventId, proc, clientData,
			FALSE, w));
}

/* when the widget is destroyed, remove the callback, and free the 
   call structure. */

static void removeCallbackDestroy( Widget		w,
				   AmUserDestroyPtr	userDestroy,
				   XtPointer)
{
    AmRemoveCallbackWidget( userDestroy->packageId, userDestroy->eventId,
			  userDestroy->proc, userDestroy->clientData, w);
    free( userDestroy);
}

/* remove an interest in a user event */

AnvilBoolean AmRemoveCallback( int		packageId,
			int		eventId,
			callbackProc	proc,
			void		*clientData) 
{
    return( AmRemoveCallbackWidget( packageId, eventId, proc, clientData, NULL));
}

/* remove a callback which is associated with a widget */

AnvilBoolean AmRemoveCallbackWidget( int		packageId,
			      int		eventId,
			      callbackProc	proc,
			      void		*clientData,
			      Widget		w)
{
    AmUserHandlerListRec	target;
    AmUserHandlerListPtr	*pUserHandlerList;
    AmUserHandlerRec	userHandler;
    AmUserHandlerPtr	found;

/* try to find the event in the tree... */

    target.packageId = packageId;
    target.eventId = eventId;
    pUserHandlerList = ( AmUserHandlerListPtr *) 
    	tfind( &target, ( const void **) &handlerRoot, ( AmSearchFn) cmpEvent);
    if ( pUserHandlerList == NULL) {
	return( FALSE);
    }

/* find the handler in the list and set its delete flag */

    userHandler.clientData = clientData;
    userHandler.proc = proc;
    userHandler.widget = w;
    found = ( AmUserHandlerPtr) AmFindDataLst( &userHandler, 
    					   ( *pUserHandlerList)->handlerList, 
					   ( AmSearchFn) cmpHandlerWidget);
    if ( found != NULL) {
    	found->deleted = TRUE;
	return( TRUE);
    } else {
AN_DEBUG( 40,( "failed to delete %p( %p), eventId = %d\n",
	       proc, clientData, eventId));
    	return( FALSE);
    }
}

/* register an interest in an event to be called just once */

AnvilBoolean AmAddCallbackOnce( int		packageId,
			 int		eventId,
			 callbackProc	proc,
			 void		*clientData)
{
    return( addCallbackHandler( packageId, eventId, proc, clientData,
		TRUE, NULL));
}

static AnvilBoolean addCallbackHandler( int		packageId,
				   int		eventId,
				   callbackProc	proc,
				   void		*clientData,
				   AnvilBoolean	callOnce,
				   Widget	w)
{
    AmUserHandlerListRec	target;
    AmUserHandlerListPtr	*pUserHandlerList, userHandlerList;
    AmUserHandlerPtr	userHandler;

/* create a structure to hold the user event. Set the deleted flag to false.
   We use deleted flags, because a user event can be deleted from the list
   of handlers for that event while the list is being traversed in 
   AmSendUserEventChildren.  The deletions are applied after the list is
   traversed, and any handlers  */

    userHandler = ( AmUserHandlerPtr) malloc( sizeof( AmUserHandlerRec));
    userHandler->clientData = clientData;
    userHandler->proc = proc;
    userHandler->callOnce = callOnce;
    userHandler->widget = w;
    userHandler->deleted = FALSE;

/* try to find the event in the tree... */

    target.packageId = packageId;
    target.eventId = eventId;
    pUserHandlerList = ( AmUserHandlerListPtr *) 
    	tfind( &target, ( const void **) &handlerRoot, ( AmSearchFn) cmpEvent);

/* if not, then insert it */

    if ( pUserHandlerList == NULL) {
	userHandlerList = ( AmUserHandlerListPtr) 
	    malloc( sizeof( AmUserHandlerListRec));
	userHandlerList->handlerList = NULL;
	userHandlerList->packageId = packageId;
	userHandlerList->eventId = eventId;
	tsearch(( void *) userHandlerList, &handlerRoot, ( AmSearchFn) cmpEvent);
    } else { 
	userHandlerList = *pUserHandlerList;
    }

/* and add the user event to the list of handlers */

    userHandlerList->handlerList = 
	AmInsertNodeLst( userHandler, userHandlerList->handlerList);
    return( TRUE);
}


/* send a user event to be immediately processed by all functions which
   have registered an interest, meaning that the call data doesn't have
   to be allocated */

AnvilBoolean AmSendUserEvent( int	packageId,
		       int 	eventId,
		       void	*callData)
{
    return( AmSendUserEventChildren( packageId, eventId, callData, NULL));
}

/* send a user event to be immediately processed by all functions which
   have registered an interest which are children of another widget, so
   that the event is not sent to multiple instantiations of the same dialog */

AnvilBoolean AmSendUserEventChildren( int	packageId,
			       int 	eventId,
			       void	*callData,
			       Widget	w)
{
    AmUserHandlerListRec	target;
    AmUserHandlerListPtr	userHandlerList, *pUserHandlerList;
    AmUserHandlerPtr	userHandler;
    AmListPtr		ptr;
    static int		callCount = 0;

// AN_DEBUG( 99, ("AmSendUserEvent: %d, %x %x\n", eventId, callData, w));
/* try to find the event in the tree... */

    target.packageId = packageId;
    target.eventId = eventId;
    pUserHandlerList = ( AmUserHandlerListPtr *) 
	tfind( &target, ( const void **) &handlerRoot, ( AmSearchFn) cmpEvent);
    if ( pUserHandlerList != NULL) {
	userHandlerList = *pUserHandlerList;
	callCount++;
	for ( ptr = userHandlerList->handlerList; 
	      ptr != NULL; 
	      ptr = ptr->next) {
	    userHandler = ( AmUserHandlerPtr) ptr->data;

/* if the event is not tagged to a parent widget, then send it to all 
   listeners, otherwise only send it to descendents of the specified parent 
   (see descendentOf) */

	    if ( !userHandler->deleted && 
	         descendentOf( w, userHandler->widget)) {

// AN_DEBUG( 99, ( "calling proc %x(%x)\n", 
// 		userHandler->proc, userHandler->clientData));

		( userHandler->proc)
		    ( eventId, userHandler->clientData, callData);
	    } else {
// AN_DEBUG( 90, ( "target not descendent for %d\n", eventId));
	    }
	}
	callCount--;

/* delete all of the handlers which are supposed to be called once, or 
   any handlers which have been deleted. Only call this if the event is
   at the top level.  If we call it from lower in the list in another 
   sendEvent call, then it will trash the list */

	if ( callCount == 0) {
	    userHandlerList->handlerList = 
		AmDelElLst( w, userHandlerList->handlerList, 
			  ( AmSearchFn) deleteThisHandler);
	}

/* remove the node from the tree if there are no more associated handlers */

	if ( userHandlerList->handlerList == NULL) {
	    tdelete( userHandlerList, &handlerRoot, ( AmSearchFn) cmpEvent);
	}
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/* when the widget is destroyed, then we remove the callback from the
   tree of handlers we use for tracking */

static void destroyUserVaCallback( Widget,
				   AmListPtr	fnDataLst,
				   XtPointer)
{
    tdelete( fnDataLst, &vaHandlerRoot, ( AmSearchFn) AmCompareListAddrs);
}

/* wrapper function to call the varargs user callback with the correct #
   of arguments */

void AmVaUserCallbackWrapper( int 	event,
			    AmListPtr	clientDataAndEventIdLst,
			    XtPointer	callData)
{
    AmListPtr		clientDataLst;
    VarargsCallback	proc = ( VarargsCallback) clientDataAndEventIdLst->data;

/* the format of the list is proc, pkg id, event id, arguments */

    clientDataLst = clientDataAndEventIdLst->next->next->next;

/* since varargs doesn't convert straight to a stack, we have to do a
   switch statement kludge for the user's function. */

    switch( AmLengthLst( clientDataLst)) {
    case 0:
        ( proc)( event, callData);
        break;
    case 1:
        ( proc)( event,
                       clientDataLst->data,
                       callData);
        break;
    case 2:
        ( proc)( event,
                       clientDataLst->data,
                       clientDataLst->next->data,
                       callData);
        break;
    case 3:
        ( proc)( event,
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       callData);
        break;
    case 4:
        ( proc)( event,
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       callData);
        break;
    case 5:
        ( proc)( event,
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       clientDataLst->next->next->next->next->data,
                       callData);
        break;
    case 6:
        ( proc)( event,
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       clientDataLst->next->next->next->next->data,
                       clientDataLst->next->next->next->next->next->data,
                       callData);
        break;
    case 7:
        ( proc) ( event,
                   clientDataLst->data,
                   clientDataLst->next->data,
                   clientDataLst->next->next->data,
                   clientDataLst->next->next->next->data,
                   clientDataLst->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->next->data,
                   callData);
        break;
    case 8:
        ( proc) ( event,
		  clientDataLst->data,
		  clientDataLst->next->data,
		  clientDataLst->next->next->data,
		  clientDataLst->next->next->next->data,
		  clientDataLst->next->next->next->next->data,
		  clientDataLst->next->next->next->next->next->data,
		  clientDataLst->next->next->next->next->next->next->data,
		  clientDataLst->next->next->next->next->next->next->next->data,
		  callData);
        break;
    default:
        AmPrintError( "Too many arguments, callback not called");
        break;
    }
}

/* add a varargs user callback to a widget */

AnvilBoolean AmAddVaCallbackWidget( int		packageId,
			     int		eventId,
			     Widget		w,
			     VarargsCallback	proc, 
			     ...)
{
    va_list	clientDataArgs;
    XtPointer	clientData;
    AmListPtr	clientDataLst = NULL, fnDataLst;
    AnvilBoolean	flag;

/* prepend the package id and event id as discriminators */

    clientDataLst = AmInsertNodeLst(( void *) eventId, clientDataLst);
    clientDataLst = AmInsertNodeLst(( void *) packageId, clientDataLst);

/* loop through the varargs list and create a generic list */

    va_start( clientDataArgs, proc);
    for ( ;;) {
        clientData = va_arg( clientDataArgs, XtPointer);
        if ( clientData == END_LIST) {         /* better terminator */
            break;
        }
        clientDataLst = AmInsertNodeAtEndLst( clientData, clientDataLst);
    }
    va_end( clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) proc, clientDataLst);

/* add the callback and the data list to the binary tree of all varargs
   callbacks */

    tsearch( fnDataLst, &vaHandlerRoot, ( AmSearchFn) AmCompareListAddrs);
    flag = AmAddCallbackWidget( packageId, eventId, w, 
			      ( callbackProc) AmVaUserCallbackWrapper, fnDataLst);
    XtAddCallback( w, XmNdestroyCallback,
		   ( XtCallbackProc) destroyUserVaCallback, fnDataLst);
    return( flag);
}

/* remove a varargs user callback */

AnvilBoolean AmRemoveVaCallbackWidget( int		packageId,
			        int		eventId,
			        Widget		w,
			        VarargsCallback	proc, 
				...)
{
    va_list     clientDataArgs;
    XtPointer   clientData;
    AmListPtr     clientDataLst = NULL, fnDataLst, *target, ptr;
    AnvilBoolean	flag = FALSE;

/* prepend the package id and event id as discriminators */

    clientDataLst = AmInsertNodeLst(( void *) eventId, clientDataLst);
    clientDataLst = AmInsertNodeLst(( void *) packageId, clientDataLst);

/* loop through the varargs list and create a generic list */

    va_start( clientDataArgs, proc);
    for ( ;;) {
        clientData = va_arg( clientDataArgs, XtPointer);
        if ( clientData == END_LIST) {         /* better terminator */
            break;
        }
        clientDataLst = AmInsertNodeAtEndLst( clientData, clientDataLst);
    }
    va_end( clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) proc, clientDataLst);

/* remove the callback from the tree of all callbacks */

    target = ( AmListPtr *) tfind( fnDataLst, ( const void **) &vaHandlerRoot,
                                 ( AmSearchFn) AmCompareListAddrs);
    if ( target != NULL) {
	flag = AmRemoveCallbackWidget( packageId, eventId, 
				     ( callbackProc) AmVaUserCallbackWrapper, 
				     *target, w);

/* remove the destroy callback with the same pointer it was created with */

	XtRemoveCallback( w, XmNdestroyCallback,
		          ( XtCallbackProc) destroyUserVaCallback, *target);
	ptr = *target;
	tdelete( *target, &vaHandlerRoot, ( AmSearchFn) AmCompareListAddrs);
	AmJustFreeLst( ptr);
    } else {
AN_DEBUG( 40, ( "failed to delete va proc %p event = %d\n", proc, eventId));
    }
    AmJustFreeLst( fnDataLst);
    return( flag);
}

/* a couple of widely used callbacks. make buttons sensitive, set labels,
   etc */

void AmSensitizeCback( int,	/* event */
		     Widget	w,
		     int	sensFlag)
{
    XtSetSensitive( w, sensFlag);
}


void AmActiveCback( int,		/* event */
		  Widget	w,
		  XtPointer)
{
    XtSetSensitive( w, TRUE);
}

void AmInactiveCback( int,	/* event */
		    Widget	w,
		    XtPointer)
{
    XtSetSensitive( w, FALSE);
}

void AmSetLabelCback( int,	/* event */
		    Widget	w,
		    char	*label)
{
    XmString	mstr;

    mstr = XmStringCreateLtoR( label, XmFONTLIST_DEFAULT_TAG);
    XtVaSetValues( w, XmNlabelString, mstr, NULL);
    XmStringFree( mstr);
}

/* callback used for auto-destroying dialogs */

void AmDestroyDlogCback( int,		/* event */
		       Widget		dlog,
		       XtPointer)
{
    Widget	shell = AmGetWidgetShell( dlog);

    XtDestroyWidget( shell);
}

