#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <stdarg.h>
#include <search.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <anvil/errlib.h>
#include <anvil/list.h>
#include <anvil/va_cback.h>
#include <anvil/va_cbackP.h>

/* end list marker instead of NULL, so varargs termination isn't mistakenly
   done by 0 */
 
char *END_LIST = "end_list";

/* internal callback list and tag value */

static void *cbackRoot = NULL;

static void destroyVaCallback( Widget		w,
			       AmListPtr		fnDataLst,
			       XtPointer	callData);

/* compare two lists to see if they point to the same data */

int AmCompareListAddrs( AmListPtr	aPtr,
		      AmListPtr	bPtr)
{
    for (;( aPtr != NULL) && ( bPtr != NULL); 
	  aPtr = aPtr->next, bPtr = bPtr->next) {
	if ( aPtr->data != bPtr->data) {
	    return(( int) aPtr->data - ( int) bPtr->data);
	}
    }
    if ((aPtr == NULL) && ( bPtr == NULL)) {
	return( 0);
    }
    if ( aPtr == NULL) {
	return( -1);
    } else {
	return( 1);
    }
}

/* multi-argument callback wrapper function */

void AmVaCbackWrapper( Widget      	w,
		     AmListPtr		clientDataLst,
		     XtPointer   	callData)
{
    VarargsCallback	cbackProc;

/* pull the user's function out of the linked list */

    cbackProc = ( VarargsCallback) clientDataLst->data;
    clientDataLst = clientDataLst->next;

/* since varargs doesn't convert straight to a stack, we have to do a 
   switch statement kludge for the user's function. */

    switch( AmLengthLst( clientDataLst)) {
    case 0:
        ( cbackProc)( w, callData);
        break;
    case 1:
        ( cbackProc)( w, 
                       clientDataLst->data, 
                       callData);
        break;
    case 2:
        ( cbackProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data, 
                       callData);
        break;
    case 3:
        ( cbackProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       callData);
        break;
    case 4:
        ( cbackProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       callData);
        break;
    case 5:
        ( cbackProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       clientDataLst->next->next->next->next->data,
                       callData);
        break;
    case 6:
        ( cbackProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       clientDataLst->next->next->next->next->data,
                       clientDataLst->next->next->next->next->next->data,
                       callData);
        break;
    case 7:
        ( cbackProc)
                 ( w, 
                   clientDataLst->data,
                   clientDataLst->next->data,
                   clientDataLst->next->next->data,
                   clientDataLst->next->next->next->data,
                   clientDataLst->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->next->data,
                   callData);
        break;
    case 8:
        ( cbackProc)
             ( w, 
               clientDataLst->data,
               clientDataLst->next->data,
               clientDataLst->next->next->data,
               clientDataLst->next->next->next->data,
               clientDataLst->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->next->next->data,
               callData);
        break;
    default:
        AmPrintError( "Too many arguments, callback not called");
        break;
    }
}

/* varargs version of XtAddCallback */

void AmAddVaCallback( Widget		w, 
		    String		callbackName, 
		    VarargsCallback	cbackProc,
		    ...)
{
    va_list	clientDataArgs;

    va_start( clientDataArgs, cbackProc);
    AmAddVaCallbackVararg( w, callbackName, cbackProc, clientDataArgs);
    va_end( clientDataArgs);
}

/* given a va_list, terminated with END_LIST, create a generic list of
   callback/handler arguments */

AmListPtr AmVaArgList( va_list	args)
{
    XtPointer   clientData;
    AmListPtr     list = NULL;

/* loop through the varargs list and create a generic list */

    for ( ;;) {
        clientData = va_arg( args, XtPointer);
        if ( clientData == END_LIST) {         /* better terminator */
            return( list);
        }
        list = AmInsertNodeAtEndLst( clientData, list);
    }
    return( list);
}

/* varargs version of XtAddCallback */

void AmAddVaCallbackVararg( Widget		w, 
			  String		callbackName, 
			  VarargsCallback	cbackProc,
			  va_list		clientDataArgs)
{
    AmListPtr     clientDataLst, fnDataLst;

/* loop through the varargs list and create a generic list */

    clientDataLst = AmVaArgList( clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) cbackProc, clientDataLst);

/* add the callback and the data list to the binary tree of all varargs
   callbacks */

    tsearch( fnDataLst, &cbackRoot, ( AmSearchFn) AmCompareListAddrs);
    
/* put the pointer to the user's function at the head of the list and add
   our private function as a REAL callback */

    XtAddCallback( w, callbackName, 
		   ( XtCallbackProc) AmVaCbackWrapper, fnDataLst);
    XtAddCallback( w, XmNdestroyCallback, 
		   ( XtCallbackProc) destroyVaCallback, fnDataLst);
}

/* destroy a varargs callback's data structs along with the widget. Don't 
   call XtRemoveCallback, because it's called when the widget's destroyed */

static void destroyVaCallback( Widget,
			       AmListPtr		fnDataLst,
			       XtPointer)
{
    tdelete( fnDataLst, &cbackRoot, ( AmSearchFn) AmCompareListAddrs);
    AmJustFreeLst( fnDataLst);
}

/* remove a varargs callback, given the widget, callback name and varargs
   callback id */

void AmRemoveVaCallback( Widget      	w,
		       String      	callbackName,
		       VarargsCallback	cbackProc,
		       ...)
				
{
    va_list     clientDataArgs;
    AmListPtr     clientDataLst = NULL, fnDataLst, *target, ptr;

/* loop through the varargs list and create a generic list */

    va_start( clientDataArgs, cbackProc);
    clientDataLst = AmVaArgList( clientDataArgs);
    va_end( clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) cbackProc, clientDataLst);

/* remove the callback from the tree of all callbacks */

    target = ( AmListPtr *) tfind( fnDataLst, (const void **)&cbackRoot, 
				 ( AmSearchFn) AmCompareListAddrs);
    if ( target != NULL) {

/* remove the callback, remove the data from the list, then free the 
   list of pointers ( but not the data). Also remove the destroy callback */

	XtRemoveCallback( w, callbackName, 
			  ( XtCallbackProc) AmVaCbackWrapper, *target);
	XtRemoveCallback( w, XmNdestroyCallback, 
			  ( XtCallbackProc) destroyVaCallback, *target);
	ptr = *target;
	tdelete( *target, &cbackRoot, ( AmSearchFn) AmCompareListAddrs);
	AmJustFreeLst( ptr);
    } 
    AmJustFreeLst( fnDataLst);
}
