#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <stdarg.h>
#include <search.h>
#include <Xm/Xm.h>
#include <X11/Intrinsic.h>
#include <anvil/list.h>
#include <anvil/errlib.h>
#include <anvil/va_cback.h>
#include <anvil/va_cbackP.h>

extern char *END_LIST;

static void *handlerRoot = NULL;

static void destroyVaHandler( Widget          w,
			      AmListPtr         fnDataLst,
			      XtPointer       callData);

/* wrapper XtEventHandler function for multi-arg event handlers */

void AmVaHndlrWrapper( Widget       w,
		     AmListPtr      clientDataLst,
		     XEvent       *event,
		     AnvilBoolean	  *continueDispatch)
{
    VarargsHandler	handlerProc;

/* pull the user's function out of the linked list */

    handlerProc = ( VarargsHandler) clientDataLst->data;
    clientDataLst = clientDataLst->next;

/* since varargs doesn't convert straight to a stack, we have to do a 
   switch statement kludge for the user's function. */

    switch( AmLengthLst( clientDataLst)) {
    case 0:
        ( handlerProc)( w, event);
        break;
    case 1:
        ( handlerProc)( w, 
                       clientDataLst->data, 
                       event);
        break;
    case 2:
        ( handlerProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data, 
                       event);
        break;
    case 3:
        ( handlerProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       event);
        break;
    case 4:
        ( handlerProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       event);
        break;
    case 5:
        ( handlerProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       clientDataLst->next->next->next->next->data,
                       event);
        break;
    case 6:
        ( handlerProc)( w, 
                       clientDataLst->data,
                       clientDataLst->next->data,
                       clientDataLst->next->next->data,
                       clientDataLst->next->next->next->data,
                       clientDataLst->next->next->next->next->data,
                       clientDataLst->next->next->next->next->next->data,
                       event);
        break;
    case 7:
        ( handlerProc)
                 ( w, 
                   clientDataLst->data,
                   clientDataLst->next->data,
                   clientDataLst->next->next->data,
                   clientDataLst->next->next->next->data,
                   clientDataLst->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->next->data,
                   event);
        break;
    case 8:
        ( handlerProc)
             ( w, 
               clientDataLst->data,
               clientDataLst->next->data,
               clientDataLst->next->next->data,
               clientDataLst->next->next->next->data,
               clientDataLst->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->next->next->data,
               event);
        break;
    default:
        AmPrintError( "too many arguments, callback not called (335)\n");
        break;
    }
}

/* varargs version of XtAddCallback */

void AmAddVaHandler( Widget              w,                      
		   unsigned int        eventMask,
		   int                 propFlag,
		   VarargsHandler      handlerProc,	
		   ...)
{
    va_list	clientDataArgs;

    va_start( clientDataArgs, handlerProc);
    AmAddVaHandlerVararg( w, eventMask, propFlag, handlerProc, clientDataArgs);
    va_end( clientDataArgs);
}

/* varargs version of XtAddCallback */

void AmAddVaHandlerVararg( Widget              w,                      
			 unsigned int        eventMask,
			 int                 propFlag,
			 VarargsHandler      handlerProc,	
			 va_list	     clientDataArgs)
{
    AmListPtr      clientDataLst, fnDataLst;

    clientDataLst = AmVaArgList( clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) handlerProc, clientDataLst);

/* take the data for the handler and assign it a tag number to put into
   an interned association list.  The tag number is returned so that the
   actual handler client data pointer can be recovered so the event handler
   can be removed */

    tsearch( fnDataLst, &handlerRoot,( AmSearchFn) AmCompareListAddrs);
          
/* put the pointer to the user's function at the head of the list and add
   our private function as a REAL callback */
   
    XtAddEventHandler( w, eventMask, propFlag, 
		       ( XtEventHandler) AmVaHndlrWrapper, fnDataLst);
    XtAddCallback( w, XmNdestroyCallback,
		   ( XtCallbackProc) destroyVaHandler, fnDataLst);
}

/* destroy a varargs handler's data structs */

static void destroyVaHandler( Widget          w,
			      AmListPtr         fnDataLst,
			      XtPointer       callData)
{
    if ( tdelete( fnDataLst, &handlerRoot, 
		  ( AmSearchFn) AmCompareListAddrs) != NULL) {
	AmJustFreeLst( fnDataLst);
    }
}

/* remove a varargs event handler, given the widget, event mask, propogation
   flag, and varargs event handler id */
    
void AmRemoveVaHandler( Widget              w,
		      unsigned int        eventMask,
		      int                 propFlag,
		      VarargsHandler	  handlerProc,
		      ...)
{
    va_list     clientDataArgs;
    AmListPtr     clientDataLst, fnDataLst, ptr, *target;
 
/* loop through the varargs list and create a generic list */
 
    va_start( clientDataArgs, handlerProc);
    clientDataLst = AmVaArgList( clientDataArgs);
    va_end( clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) handlerProc, clientDataLst);
 
/* remove the handler from the tree of all handlers */
 
    target = ( AmListPtr *) tfind( fnDataLst, (const void **)&handlerRoot, 
				 ( AmSearchFn) AmCompareListAddrs);
    if ( target != NULL) {

/* remove the event handler and the destroy cleanup function */

	XtRemoveEventHandler( w, eventMask, propFlag, 
			      ( XtEventHandler) AmVaHndlrWrapper, *target);
	XtRemoveEventHandler( w, eventMask, propFlag, 
			      ( XtEventHandler) destroyVaHandler, *target);
	ptr = *target;
        tdelete( ptr, &handlerRoot, ( AmSearchFn) AmCompareListAddrs);
	AmJustFreeLst( ptr);
    }
    AmJustFreeLst( fnDataLst);
}
