#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <X11/cursorfont.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/MessageB.h>

#include <anvil/project.h>
#include <anvil/motif.h>
#include <anvil/waitdlog.h>
#include <anvil/resource.h>

#include <ToolKit/ToolKit.h>

/********************************************************************
 *
 * Private functions:
 *
 ********************************************************************/

static void
stopFn( Widget    ,
        AmWaitPtr   wait,
        XtPointer )
{
    wait->fStopped = TRUE;
}


static void
destroyFn( Widget    ,
           AmWaitPtr   wait,
           XtPointer )
{
    if ( wait != NULL) {
        free( wait);
    }
}


/********************************************************************
 *
 * Public functions:
 *
 ********************************************************************/

AmWaitPtr
AmMakeWaitDialog( Widget     wParent,
                const char *szMessage,
                AnvilBoolean    fInterruptable)
{
    XSetWindowAttributes xwa;
    Display              *display = XtDisplay( wParent);
    Window               window = XtWindow( wParent);
    XmString             xsz;
    Arg                  args[ 20];
    int                  n;
    AmWaitPtr              wait;
    Widget               wDialog; 

    if ( !AmGetDefault( "showWait", TRUE)) {
	return( NULL);
    }

    wait = ( AmWaitPtr) malloc( sizeof( AmWaitRec));
    if ( wait == NULL) {
        DO_ERROR( AmMALLOC_FAILED);
        return( NULL);
    }

    xsz = XmStringCreateLtoR( ( char *) szMessage, XmFONTLIST_DEFAULT_TAG);

    n = AmMakeArgList( args, XmNmessageString, xsz, NULL);
    wDialog = XmCreateWorkingDialog( wParent, "busy", args, n); 
    XmStringFree( xsz);

    XtUnmanageChild( XmMessageBoxGetChild( wDialog, XmDIALOG_OK_BUTTON));
    XtUnmanageChild( XmMessageBoxGetChild( wDialog, XmDIALOG_HELP_BUTTON));

    if ( fInterruptable == TRUE) {
        XtAddCallback( wDialog, XmNcancelCallback,
            ( XtCallbackProc) stopFn, wait);
    } else {
        XtUnmanageChild(
            XmMessageBoxGetChild( wDialog, XmDIALOG_SEPARATOR));
        XtUnmanageChild(
            XmMessageBoxGetChild( wDialog, XmDIALOG_CANCEL_BUTTON));
    }

    XtManageChild( wDialog);
    AmForceDisplayDialog( wDialog);

    wait->wParent = wParent;
    wait->wDialog = wDialog;
    wait->fStopped = FALSE;

/* Set the function to be called when the dialog gets destroyed. */

    XtAddCallback( wDialog, XmNdestroyCallback,
        ( XtCallbackProc) destroyFn, wait);
          
/* Set the wait cursor. */

    xwa.cursor = XCreateFontCursor( display, XC_watch);
    XChangeWindowAttributes( display, window, CWCursor, &xwa);
    XFlush( display);

    return( wait); 
}

void
AmRemoveWaitDialog( AmWaitPtr wait)
{
    XSetWindowAttributes xwa;
    Display              *display = XtDisplay( wait->wParent);
    Window               window = XtWindow( wait->wParent);
    XEvent               event;

    if ( wait == NULL) {
	return;
    }

/* Reset the cursor back to normal. */

    xwa.cursor = None;
    XChangeWindowAttributes( display, window, CWCursor, &xwa);
    XFlush( display);

    XtUnmapWidget( XtParent( wait->wDialog));
    AmForceRemoveDialog( wait->wDialog);

    XtDestroyWidget( XtParent( wait->wDialog));

/*
 * Get rid of all button and keyboard events that occured during
 *   the time out.  The user shouldn't have done anything during this
 *   time.
 */

    while( XCheckMaskEvent( display,
               ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
               PointerMotionMask | KeyPressMask, &event));
}


AnvilBoolean
AmCheckForInterrupt( AmWaitPtr wait)
{
    Display *display = XtDisplay( wait->wParent);
    Window  window = XtWindow( wait->wDialog);
    XEvent  event;

    XFlush( display);

    XmUpdateDisplay( wait->wParent);

    while( XCheckMaskEvent( display,
               ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
               PointerMotionMask | KeyPressMask, &event)) {
        if ( event.xany.window == window) {
            XtDispatchEvent( &event);
        }
    }

    return( wait->fStopped);
}
