#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <X11/xpm.h>

#include <anvil/va_cback.h>
#include <anvil/errcode.h>
#include <anvil/errlib.h>

/**************
 * Constants: *
 **************/

/**************
 * Functions: *
 **************/

void
AmXpmWidgetDestroyCallback( Widget        w,
                          Pixmap        pixmap,
                          XpmAttributes *attributes,
                          XtPointer)
{
    Display  *display = XtDisplay( w);
#if 0
    Colormap colormap = DefaultColormap( display, DefaultScreen( display));

#if defined(xpm2)
    Pixel    *pixels = attributes->alloc_pixels;
    int      n = attributes->nalloc_pixels;
#else
    Pixel    *pixels = attributes->pixels;
    int      n = attributes->npixels;
#endif
#endif

    XFreePixmap( display, pixmap);
#if 0
    XFreeColors( XtDisplay( w), colormap, pixels, n, 0);
#endif
    XpmFreeAttributes( attributes);
    free( attributes);
}

AmErrorCode
AmSetTransparentPixmap( Widget w,
           char   *name)
{
    Display        *display = XtDisplay( w);
    Window         win = XRootWindowOfScreen( XtScreen( w));
    XpmAttributes  *attributes;
    XpmColorSymbol symbols[ 2]; 
    Pixmap         pixmap;
    Pixel          color;
    int            status;

    XtVaGetValues( w, XmNbackground, &color, NULL);

    symbols[ 0].name = NULL;
    symbols[ 0].value = "none";
    symbols[ 0].pixel = color;

    if ( !(attributes = ( XpmAttributes *) malloc( sizeof( XpmAttributes)))) {
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
        return( ANVIL_ERROR_CODE);
    }

    attributes->colorsymbols = symbols;
    attributes->numsymbols = 1;

#if defined(xpm2)
    attributes->valuemask = XpmColorSymbols | XpmReturnAllocPixels;
#else
    attributes->valuemask = XpmColorSymbols | XpmReturnPixels;
#endif

    status = XpmReadFileToPixmap( display, win,
                  name, &pixmap, NULL, attributes);

    if ( status != XpmSuccess) {
        AmPrintResourceError( "failedCreatePixmap", XpmGetErrorString( status));
        free( attributes);

        switch ( status) {
            case XpmOpenFailed:
                ANVIL_ERROR_CODE = AmOPEN_FAILED;
                break;
            case XpmFileInvalid:
                ANVIL_ERROR_CODE = AmFILE_INVALID;
                break;
            case XpmNoMemory:
                ANVIL_ERROR_CODE = AmMALLOC_FAILED;
                break;
            case XpmColorError:
            case XpmColorFailed:
                ANVIL_ERROR_CODE = AmCOLOR_ALLOCATION_FAILED;
                break;
            default:
                ANVIL_ERROR_CODE = AmUNKNOWN_ERROR;
                break;
        } 

        return( ANVIL_ERROR_CODE);
    }

    XtVaSetValues( w,
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, pixmap,
        NULL);

    AmAddVaCallback( w, XmNdestroyCallback,
        ( VarargsCallback) AmXpmWidgetDestroyCallback,
        pixmap, attributes, END_LIST);

    return( AmOK);
}
