#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <malloc.h>
#include <anvil/display.h>
#include <anvil/xpm_ximage.h>
#include <anvil/errcode.h>

#define MAXCOLORS 256
#define MAXPATH	256

static char pixelChars[] = " #+.acbdefghijklmnopqurstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789[]{}';:`~!@#$%^&*()-_+=|<>?/";
static char hexChars[] = "0123456789ABCDEF";

static void writeXPMPixel( int	pixel, char *buffer)
{
    buffer[ 0] = hexChars[ (pixel & 0xf0) >> 4];
    buffer[ 1] = hexChars[ pixel & 0xf];
    buffer[ 2] = '0';
    buffer[ 3] = '0';
    buffer[ 4] = '\0';
}


AmErrorCode AmImageToXPM( char		*filename,
			AmImageClass	*image,
			AmColorCellPtr	colors,
			int		numColors)
{
    FILE	*fp;
    char	redStr[ 16], greenStr[ 16], blueStr[ 16], *data, *writeBuf;
    int		ii, jj, height, width, bytesPerLine, indexVec[ MAXCOLORS],
		pixelCount;

    if (( fp = fopen( filename, "w")) == NULL) {
	DO_RETURN_ERROR( AmOPEN_FAILED);
    }

/* scan through the image to find which pixels have been set */

    width = image->GetWidth();
    height = image->GetHeight();
    bytesPerLine = image->GetBytesPerLine();
    data = image->AmGetData();
    for ( ii = 0; ii < MAXCOLORS; ii++) {
    	indexVec[ ii] = -1;			/* unassigned marker */
    }
    pixelCount = 0;
    for ( ii = 0; ii < height; ii++) {
    	for ( jj = 0; jj < width; jj++) {
    	    if ( indexVec[ data[ ii*bytesPerLine + jj]] == -1) {
    	    	indexVec[ data[ ii*bytesPerLine + jj]] = pixelCount++;
	    }
	}
    }

/* write out the colormap in a header */

    fprintf( fp, "static char *%s_xpm[] = {\n", filename);
    fprintf( fp, "\"%d %d %d 1\",\n", width, height, numColors);
    for ( ii = 0; ii < numColors; ii++) {
    	writeXPMPixel( colors[ ii].red, redStr);
    	writeXPMPixel( colors[ ii].green, greenStr);
    	writeXPMPixel( colors[ ii].blue, blueStr);
	fprintf( fp, "\"%c\tc #%s%s%s\",\n", 
		 pixelChars[ indexVec[ colors[ ii].pixel]], 
		 redStr, greenStr, blueStr);
    }

/* write the data out in buffers */

    if (( writeBuf = ( char *) malloc( width + 1)) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    for ( ii = 0; ii < height; ii++) {
    	for ( jj = 0; jj < width; jj++) {
    	    writeBuf[ jj] = 
    	    	pixelChars[ indexVec[ data[ ii*bytesPerLine + jj]]];
    	}
    	writeBuf[ width] = '\0';
    	fprintf( fp, "\"%s\",\n", writeBuf);
    }
    fprintf( fp, "};\n");
    free( writeBuf);
    fclose( fp); 
    return( AmOK);
}
