#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <anvil/xresource.h>

/* open the resource file and read in the resource database */

AmXResourceDbClass::AmXResourceDbClass( const char *filenameArg) :
			AmResourceDbClass( filenameArg)
{
    if ( filenameArg != NULL) {
	db = XrmGetFileDatabase( filename);
    } else {
	filename = NULL;
	db = NULL;
    }
}

/* Set the resource database to the passed in resource database */

AmXResourceDbClass::AmXResourceDbClass( XrmDatabase dbArg) :
			AmResourceDbClass( NULL)
{
    db = dbArg;
}

/* destroy the resource database and free the filename */

AmXResourceDbClass::~AmXResourceDbClass()
{
    XrmDestroyDatabase( db);
}

/* save a resource database to a file */

void AmXResourceDbClass::SaveResourceDb()
{
    XrmPutFileDatabase( db, filename);
}

/* return whether the resource database is valid */

AnvilBoolean AmXResourceDbClass::IsValid()
{
    return( db != NULL);
}

/* get a resource from a direct resource database */

char *AmGetDbResource( XrmDatabase	db,
		     const char		*string)
{
    XrmValue	xrmVal;
    char	*resType;

    if ( db == NULL) {
	return( NULL);
    }
    if ( XrmGetResource( db, string, string, &resType, &xrmVal)) {
	return( xrmVal.addr);
    } else {
	return( NULL);
    }
}

/* perform type conversions for the different resources.  Return FALSE if
   the resource isn't found */

AnvilBoolean AmXResourceDbClass::GetResource( const char *tag,
				       char	 **value)
{
    return(( *value = AmGetDbResource( db, tag)) != NULL);
}

/* integer. */

AnvilBoolean AmXResourceDbClass::GetResource( const char   *tag,
				       int	    *value)
{   
    char *resStr;

    resStr = AmGetDbResource( db, tag);
    if ( resStr != NULL) {
	*value = atoi( resStr);
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/* float */

AnvilBoolean AmXResourceDbClass::GetResource( const char *tag,
				       float	  *value)
{   
    char *resStr;

    resStr = AmGetDbResource( db, tag);
    if ( resStr != NULL) {
	*value = atof( resStr);
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/* boolean */

AnvilBoolean AmXResourceDbClass::GetResource( const char *tag,
				       AnvilBoolean	  *value)
{   
    char *resStr;

    resStr = AmGetDbResource( db, tag);
    if ( resStr == NULL) {
	return( FALSE);
    } else {
        *value = (!strcasecmp( resStr, "true"));
	return( TRUE);
    }
}

/* save information in the resource defaults database (string) */

AnvilBoolean AmXResourceDbClass::PutResource( const char  *tag,
				       const char  *value)
{
    if ( IsValid()) {
        XrmPutStringResource( &db, tag, value);
        return( TRUE);
    } else {
        return( FALSE);
    }
}

/* save information in the resource defaults database (boolean) */

AnvilBoolean AmXResourceDbClass::PutResource( const char  *tag,
				       AnvilBoolean     value)
{
    if ( IsValid()) {
        XrmPutStringResource( &db, tag, ( value) ? "true" : "false");
        return( TRUE);
    } else {
        return( FALSE);
    }
}

/* save information in the resource defaults database (integer) */

AnvilBoolean AmXResourceDbClass::PutResource( const char  *tag,
				       int         value)
{
    char	buffer[ 64];

    if ( IsValid()) {
	sprintf( buffer, "%d", value);
        XrmPutStringResource( &db, tag, buffer);
        return( TRUE);
    } else {
        return( FALSE);
    }
}

/* save information in the resource defaults database (float) */

AnvilBoolean AmXResourceDbClass::PutResource( const char  *tag,
				       float         value)
{
    char	buffer[ 64];

    if ( IsValid()) {
	sprintf( buffer, "%f", value);
        XrmPutStringResource( &db, tag, buffer);
        return( TRUE);
    } else {
        return( FALSE);
    }
}

