#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <windows.h>
#include <winreg.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <anvil/project.h>
#include <anvil/registry.h>
#include "dialogs.h"

static char szAppName[] = "TestDriver";
static HINSTANCE 	hInstance = 0;
static HWND       	hWnd;

/* #defines for event id's */

#define IMPORT_ID	1001
#define QUIT_ID		1002
#define POPUP_ID	1003
#define FILE_ID		1004
#define ABOUT_ID	1005


LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM);
	

int WINAPI WinMain( HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    PSTR      szCmdLine,
                    int       iCmdShow)
{
    MSG        msg;
    WNDCLASSEX wc;
    HMENU      hMenuBar, hMenuPopup;

    if ( !hPrevInstance) {

        wc.cbSize        = sizeof( wc);
        wc.style         = CS_HREDRAW | CS_VREDRAW;
        wc.lpfnWndProc   = ( WNDPROC) WndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = hInstance;
        wc.hIcon         = LoadIcon( hInstance, szAppName);
        wc.hCursor       = LoadCursor( NULL, IDC_ARROW);
        wc.hbrBackground = ( HBRUSH)( COLOR_WINDOW + 1);
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = szAppName;
        wc.hIconSm       = LoadIcon( hInstance, szAppName);

        if ( RegisterClassEx( &wc) == NULL) {
            return( FALSE);
        }
    }

    hWnd = CreateWindow( szAppName, "Test Driver",
               WS_OVERLAPPEDWINDOW,
               CW_USEDEFAULT, CW_USEDEFAULT,
               CW_USEDEFAULT, CW_USEDEFAULT,
               NULL, NULL,
               hInstance, NULL);

    if ( hWnd == NULL) {
        return( FALSE);
    }    

/* create a menu for importing the file and quitting our test app */

    hMenuBar = CreateMenu();
    SetMenu( hWnd, hMenuBar);
#if 0
    AppendMenu( hMenuBar, MFT_STRING, FILE_ID, "file");
#endif

    hMenuPopup = CreatePopupMenu();
    AppendMenu( hMenuPopup, MFT_STRING, IMPORT_ID, "import");
    AppendMenu( hMenuPopup, MFT_STRING, QUIT_ID, "quit");
    AppendMenu( hMenuBar, MF_POPUP, ( int) hMenuPopup, "file");
    AppendMenu( hMenuBar, MFT_STRING, ABOUT_ID, "about");


    ShowWindow( hWnd, iCmdShow);
    UpdateWindow( hWnd);
    DrawMenuBar( hWnd);

    while ( GetMessage( &msg, NULL, 0, 0)) {
        TranslateMessage( &msg);
        DispatchMessage( &msg);
    }

    return( msg.wParam);
}

/* dialog interaction callback */

BOOL CALLBACK AboutDlgProc( HWND hDlg, 
			    UINT iMsg, 
			    WPARAM wParam,
			    LPARAM lParam)
{
    switch( iMsg) {
    case WM_INITDIALOG:
    	return( TRUE);
    case WM_COMMAND: 
    	switch ( LOWORD( wParam)) {
	case IDOK:
	case IDCANCEL:
	    EndDialog( hDlg, 0);
	    return( TRUE);
	}
	break;
    }
    return( FALSE);
}

/* dialog interaction callback */

BOOL CALLBACK ImportDlgProc( HWND hDlg, 
			     UINT iMsg, 
			     WPARAM wParam,
			     LPARAM lParam)
{
    char	fileBuffer[ 512], errorBuffer[ 1024];

    switch( iMsg) {
    case WM_INITDIALOG:
    	return( TRUE);
    case WM_COMMAND: 
    	switch ( LOWORD( wParam)) {
	case IDOK:
	    GetDlgItemText( hDlg, IDC_EDIT, fileBuffer,
	    		    sizeof( fileBuffer) - 1);
	    if ( !importANVILregistry( "ANVIL", fileBuffer)) {
		sprintf( errorBuffer, "error importing %s", fileBuffer);
		MessageBox( hWnd, errorBuffer, "Error", MB_OK);
	    } else {
		MessageBox( hWnd, "import successful", "Success", MB_OK);
	    }
	case IDCANCEL:
	    EndDialog( hDlg, 0);
	    return( TRUE);
	}
	break;
    }
    return( FALSE);
}

LRESULT CALLBACK WndProc( HWND   hWnd,
                          UINT   iMsg,
                          WPARAM wParam,
                          LPARAM lParam)
{

    switch( iMsg) {
    case WM_CREATE:
	hInstance = ((LPCREATESTRUCT) lParam)->hInstance;
	break;

    case WM_COMMAND:
	switch ( LOWORD( wParam)) {
	case QUIT_ID:
	    PostQuitMessage( 0);
	    return( 0);
	case ABOUT_ID:

/* create the dialog specified in the resource file */

	    if ( DialogBox( hInstance, "AboutBox", 
			    hWnd, ( DLGPROC) AboutDlgProc) != -1) {
		return( 0);
	    } else {
	    	break;
	    }
	case IMPORT_ID:

/* create the dialog specified in the resource file */

	    if ( DialogBox( hInstance, "ImportBox", 
			    hWnd, ( DLGPROC) ImportDlgProc) != -1) {
		return( 0);
	    } else {
	    	break;
	    }
	default:
	    break;
	}
	break;
    case WM_DESTROY:
	PostQuitMessage( 0);
	return( 0);

    default:
	return( DefWindowProc( hWnd, iMsg, wParam, lParam));
    }
    return( 0);
}
