#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <string.h>
#include <windows.h>
#include <anvil/dispatch.h>
#include <anvil/nt_va_hndlr.h>
#include <anvil/transfer.h>

/*************************************************************************
 *
 * Description:
 *	handler which transfers events from source window to destination.
 * Arguments:
 *	hSrcWnd - window events are sent from.
 *	iMsg, wParam, lParam - event to transfer.
 *	hDstWnd - window events are sent to.
 *
 *************************************************************************/

static void 
transferHandlerFn( HWND		hSrcWnd,
		   UINT		iMsg,
		   WPARAM	wParam,
		   LPARAM	lParam,
		   HWND		hDstWnd)
{
 
    if (( iMsg == WM_LBUTTONDOWN) || ( iMsg == WM_LBUTTONUP) ||
	( iMsg == WM_RBUTTONDOWN) || ( iMsg == WM_RBUTTONUP) ||
	( iMsg == WM_MBUTTONDOWN) || ( iMsg == WM_MBUTTONDOWN) ||
	( iMsg == WM_MOUSEMOVE)) {
	POINT	point;

	point.x = LOWORD( lParam);
	point.y = HIWORD( lParam);
	MapWindowPoints( hSrcWnd, hDstWnd, &point, 1);
	lParam = MAKELONG( point.x, point.y);
    }
    SendMessage( hDstWnd, iMsg, wParam, lParam);

}

/*************************************************************************
 *
 * Description:
 *	function to transfer events from one window to another.  Sets 
 *	up a handlert to do SendMessages() when events are received by
 *	the source window.  When the destination window is destroyed,
 *	the event handler is removed.
 * Arguments:
 *	hSrcWnd - window events are transferred from.
 *	hDstWnd - window events are sent to.
 *	iMsg - message to transfer.
 *	wParam - wParam to transfer.
 *
 *************************************************************************/

void 
AmTransferEvent( HWND	hSrcWnd,
		 HWND	hDstWnd,
	         UINT	iMsg,
		 WPARAM	wParam)
{
    AmInsertHandler( hSrcWnd, iMsg, wParam, 
    				 ( AmNtHandlerFn) transferHandlerFn, hDstWnd);
}
