#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <windows.h>
#include <anvil/errcode.h>
#include <anvil/nt_user_data.h>

/***********************************************************************
 *
 * Description:
 *	store user data in a window under a named key.  This is simply
 *	a wrapper for the MS-Windows SetProp() function.
 * Arguments:
 *	hWnd:	window to set the user data for
 *	szKey:	key to set the user data under.
 *	pData:  void pointer to set for the user data 
 * Returns:
 *	AmBAD_ARGUMENT if SetProp() failed.
 * 	AmOK otherwise 
 *
 ***********************************************************************/

AmErrorCode
AmAddUserData( HWND	hWnd,
             const char *szKey,
             void       *pData)
{
    if ( !SetProp( hWnd, szKey, pData)) {
	DO_RETURN_ERROR( AmBAD_ARGUMENT);
    } else {
    	return( AmOK);
    }
}

/***********************************************************************
 *
 * Description:
 *	retreive the user data from a window under a defined key.
 *	this is a wrapper for the MS-Windows GetProp() function.
 * Arguments:
 *	hWnd:	window to get the user data from
 *	szKey:	key to get the user data with.
 * Returns:
 *	AmMISSING_USER_DATA_KEY if GetProp() failed.
 * 	AmOK otherwise 
 *
 ***********************************************************************/

void *
AmGetUserData( HWND	hWnd,
             const char *szKey)
{
    HANDLE	userData;

    if (( userData = GetProp( hWnd, szKey)) == NULL) {
        DO_ERROR( AmMISSING_USER_DATA_KEY);
        return( NULL);
    } else {
    	return( userData);
    }
}

/***********************************************************************
 *
 * Description:
 *	remove the user data for a specified key.
 *	this is a wrapper for the MS-Windows RemoveProp() function.
 * Arguments:
 *	hWnd:	window to remove the user data from.
 *	szKey:	key to remove the user data for.
 * Returns:
 *	AmMISSING_USER_DATA_KEY if RemoveProp() failed.
 * 	AmOK otherwise 
 *
 ***********************************************************************/

void *
AmRemoveUserData( HWND		hWnd,
                const char 	*szKey)
{
    HANDLE	userData;

    if (( userData = RemoveProp( hWnd, szKey)) == NULL) {
        DO_ERROR( AmMISSING_USER_DATA_KEY);
        return( NULL);
    } else {
    	return( userData);
    }
}
