#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdarg.h>
#include <search.h>
#include <windows.h>
#include <anvil/list.h>
#include <anvil/dispatch.h>
#include <anvil/tsearch.h>
#include <anvil/nt_va_hndlr.h>

const char *END_LIST = "END_LIST";

static void *handlerRoot = NULL;

/*************************************************************************
 *
 * Description:
 * 	wrapper event handler function for multi-arg event handlers.
 *	basically a big switch statement based on the length of the
 *	generic list passed in as the handler client data.
 *
 * Arguments of interest:
 *	clientDataLst: actual callback function, followed by arguments
 *	passed by varargs in AmAddVaCallback()
 *
 *************************************************************************/

static void 
vaHandlerWrapper( HWND          hWnd,
		  UINT		iMsg,
		  WPARAM	wParam,
		  LPARAM	lParam,
		  AmListPtr	clientDataLst)
{
    VarargsHandler	handlerProc;

/* pull the user's function out of the linked list */

    handlerProc = ( VarargsHandler) clientDataLst->data;
    clientDataLst = clientDataLst->next;

/* since varargs doesn't convert straight to a stack, we have to do a 
   switch statement kludge for the user's function. */

    switch( AmLengthLst( clientDataLst)) {
    case 0:
        ( handlerProc)( hWnd, iMsg, wParam, lParam);
        break;
    case 1:
        ( handlerProc)( hWnd, iMsg, wParam, lParam,
                        clientDataLst->data);
        break;
    case 2:
        ( handlerProc)( hWnd, iMsg, wParam, lParam, 
                        clientDataLst->data,
                        clientDataLst->next->data);
        break;
    case 3:
        ( handlerProc)( hWnd, iMsg, wParam, lParam, 
                        clientDataLst->data,
                        clientDataLst->next->data,
                        clientDataLst->next->next->data);
        break;
    case 4:
        ( handlerProc)( hWnd, iMsg, wParam, lParam, 
                        clientDataLst->data,
                        clientDataLst->next->data,
                        clientDataLst->next->next->data,
                        clientDataLst->next->next->next->data);
        break;
    case 5:
        ( handlerProc)( hWnd, iMsg, wParam, lParam, 
                        clientDataLst->data,
                        clientDataLst->next->data,
                        clientDataLst->next->next->data,
                        clientDataLst->next->next->next->data,
                        clientDataLst->next->next->next->next->data);
        break;
    case 6:
        ( handlerProc)( hWnd, iMsg, wParam, lParam, 
                        clientDataLst->data,
                        clientDataLst->next->data,
                        clientDataLst->next->next->data,
                        clientDataLst->next->next->next->data,
                        clientDataLst->next->next->next->next->data,
                        clientDataLst->next->next->next->next->next->data);
        break;
    case 7:
        ( handlerProc)
                 ( hWnd, iMsg, wParam, lParam, 
                   clientDataLst->data,
                   clientDataLst->next->data,
                   clientDataLst->next->next->data,
                   clientDataLst->next->next->next->data,
                   clientDataLst->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->data,
                   clientDataLst->next->next->next->next->next->next->data);
        break;
    case 8:
        ( handlerProc)
             ( hWnd, iMsg, wParam, lParam, 
               clientDataLst->data,
               clientDataLst->next->data,
               clientDataLst->next->next->data,
               clientDataLst->next->next->next->data,
               clientDataLst->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->next->data,
               clientDataLst->next->next->next->next->next->next->next->data);
        break;
    default:
        break;
    }
}

/* destroy a varargs handler's data structs */

static void 
destroyVaHandler( HWND             	hWnd,                      
		  UINT	       		iEvent,
		  WPARAM		wParam,
		  LPARAM		lParam,
		  AmListPtr		fnDataLst)
{
    if ( tdelete( fnDataLst, &handlerRoot, 
		  ( AmSearchFn) AmCompareListAddrs) != NULL) {
		AmJustFreeLst( fnDataLst);
    }
}

/* this is the internal varargs function called by AmAddVaHandler()
   which tags the varargs list and turns it into a va_list */

static void 
AmAddVaHandlerVararg( HWND          hWnd,                      
					  UINT	        *eventCodeVec,
		    WPARAM	        *wParamVec,
		    int	       		numEvents,
		    VarargsHandler   	handlerProc,	
		    va_list	        clientDataArgs)
{
    AmListPtr      clientDataLst, fnDataLst;

    clientDataLst = AmVarargLst(( void *) END_LIST, clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) handlerProc, clientDataLst);

/* take the data for the handler and assign it a tag number to put into
   an interned association list.  The tag number is returned so that the
   actual handler client data pointer can be recovered so the event handler
   can be removed */

    tsearch( fnDataLst, &handlerRoot,( AmSearchFn) AmCompareListAddrs);
          
/* put the pointer to the user's function at the head of the list and add
   our private function as a REAL callback */
   
    AmInsertHandler( hWnd, eventCodeVec, wParamVec, numEvents,
				     ( AmNtHandlerFn) vaHandlerWrapper, fnDataLst);
    AmInsertHandler( hWnd, WM_DESTROY, 0,
				     ( AmNtHandlerFn) destroyVaHandler, fnDataLst);
}

/************************************************************************
 * 
 * Description:
 * 	varargs version of InsertHandler().  Takes a varargs list of
 *	arguments terminated with the keyword END_LIST, generates a
 *	generic list with the varargs event handler at the head, then
 *	sets up InsertHandler() with the correct event codes and
 *	vaHandlerWrapper() as the actual handler.  the wrapper routine
 *	breaks out the arguments from the list and calls the handler
 *	with the correct number of arguments.
 *
 * Arguments:
 *	hWnd: window event handler is being added to.
 * 	eventCodeVec: event codes to listen for.
 *	wParamVec: wParams to compare if the event is WM_COMMAND.
 *	numEvents: number of events and params.
 *	handlerProc: event handler with up to 8 client data parameters,
 *	which are the AmLast N arguments of the function.
 *	varargs_list - list of void pointers terminated by the 
 *	key address constant END_LIST
 *
 ************************************************************************/

void 
AmAddVaHandler( HWND                hWnd,                      
	      UINT		  *eventCodeVec,
	      WPARAM	          *wParamVec,
	      int		  numEvents,
	      VarargsHandler      handlerProc,	
	      ...)
{
    va_list	clientDataArgs;

    va_start( clientDataArgs, handlerProc);
    AmAddVaHandlerVararg( hWnd, eventCodeVec, wParamVec, numEvents, 
						  handlerProc, clientDataArgs);
    va_end( clientDataArgs);
}

/************************************************************************
 * 
 * Description:
 * 	same as above, but takes a single event and wParam 
 *
 ************************************************************************/

void 
AmAddVaHandler( HWND                hWnd,                      
	      UINT		  iEvent,
	      WPARAM	          wParam,
	      VarargsHandler      handlerProc,	
	      ...)
{
    va_list	clientDataArgs;

    va_start( clientDataArgs, handlerProc);
    AmAddVaHandlerVararg( hWnd, &iEvent, &wParam, 1, handlerProc, clientDataArgs);
    va_end( clientDataArgs);
}

/* internal function for removing varargs handlers */

static void
removeVaHandlerVararg( HWND             hWnd,                      
		       UINT	        *eventCodeVec,
		       WPARAM	        *wParamVec,
		       int	       	numEvents,
		       VarargsHandler   handlerProc,	
		       va_list	        clientDataArgs)
{
    AmListPtr     clientDataLst, fnDataLst, ptr, *target;

    clientDataLst = AmVarargLst(( void *) END_LIST, clientDataArgs);
    fnDataLst = AmInsertNodeLst(( void *) handlerProc, clientDataLst);
 
/* remove the handler from the tree of all handlers */
 
    target = ( AmListPtr *) tfind( fnDataLst, &handlerRoot, 
				 ( AmSearchFn) AmCompareListAddrs);
    if ( target != NULL) {

/* remove the event handler and the destroy cleanup function. 
   The handler wrapper is set for the original event set, the 
   destroyVaHandler() is set for WM_DESTROY */ 

	AmDeleteHandler( hWnd, eventCodeVec, wParamVec, numEvents,
					 ( AmNtHandlerFn) vaHandlerWrapper, *target);
	AmDeleteHandler( hWnd, WM_DESTROY, 0, 
					 ( AmNtHandlerFn) destroyVaHandler, *target);
	ptr = *target;
        tdelete( ptr, &handlerRoot, ( AmSearchFn) AmCompareListAddrs);
	AmJustFreeLst( ptr);
    }
    AmJustFreeLst( fnDataLst);
}

/************************************************************************
 * 
 * Description:
 * 	remove a varargs event handler, given the widget, event code vector, 
 *	wParam vector, and client data list.  The handler argument list
 *	is matched in the handler master tree, then the handler wrapper
 *	(and destroy handler) are deleted using DeleteHandler()
 *
 * Arguments:
 *	hWnd: window event handler is being removed ffrom.
 * 	eventCodeVec: event codes to listen for.
 *	wParamVec: wParams to compare if the event is WM_COMMAND.
 *	numEvents: number of events and params.
 *	handlerProc: event handler with up to 8 client data parameters,
 *	which are the AmLast N arguments of the function.
 *	varargs_list - list of void pointers terminated by the 
 *	key address constant END_LIST
 *
 ************************************************************************/
    
void 
AmRemoveVaHandler( HWND		hWnd,
		 UINT		*eventCodeVec,
		 WPARAM		*wParamVec,
		 int		numEvents,
		 VarargsHandler	handlerProc,
		 ...)
{
    va_list     clientDataArgs;

    va_start( clientDataArgs, handlerProc);
    removeVaHandlerVararg( hWnd, eventCodeVec, wParamVec, numEvents, 
		           handlerProc, clientDataArgs);
    va_end( clientDataArgs);
}

/************************************************************************
 * 
 * Description:
 * 	same as above, except that a single event code and wParam are 
 *	passed 
 *
 ************************************************************************/

void 
AmRemoveVaHandler( HWND		hWnd,
		 UINT		iEvent,
		 WPARAM		wParam,
		 VarargsHandler	handlerProc,
		 ...)
{
    va_list     clientDataArgs;

    va_start( clientDataArgs, handlerProc);
    removeVaHandlerVararg( hWnd, &iEvent, &wParam, 1,
		           handlerProc, clientDataArgs);
    va_end( clientDataArgs);
}
