#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <anvil/geometry.h>
#include <anvil/projects.h>
#include <anvil/units.h>

/* given two points and units, return the distance in meters if it can
   be converted or projected */

AnvilBoolean UTMdist( float		x0, 
	         float		y0,
	         float		x1,
	         float		y1,
	         unitType	units,
		 float		*utmdist)
{
    float	distance;
    char	*proj[ 4], zoneBuf[ FIELDLEN];
    static PJ	*pj = NULL;
    static int	lastZone = -1;
    int		zone;

/* if we can convert to meters, then go for it direct */

    if ( AmConverterExists( units, AmUNIT_METER)) {
	distance = AmDist( x0, y0, x1, y1);
	AmConvertValue( units, AmUNIT_METER, distance, utmdist);
	return( TRUE);
    } else if ( AmConverterExists( units, AmUNIT_ARCDEG)) {

/* we have to do a projection */
	
	zone = ( int) ( 1 + ( 180 + (( x0 + x1)/2)))/6;
	if (( pj == NULL) || ( zone != lastZone)) {
	    if ( pj != NULL) {
		pj_free( pj);
	    }
	    proj[ 0] = "proj=utm";
	    proj[ 1] = "ellps=WGS84";
	    sprintf( zoneBuf, "zone=%d", zone);
	    proj[ 2] = zoneBuf;
	    proj[ 3] = NULL;
	    if (( pj = pj_init( 3, ( const char **) proj)) == NULL) {
		return( FALSE);
	    }
	    lastZone = zone;
	}
	projectPt( pj, FALSE, &x0, &y0);
	projectPt( pj, FALSE, &x1, &y1);
	*utmdist = AmDist( x0, y0, x1, y1);
	return( TRUE);
    } else {	
	return( FALSE);
    }
}
