#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



/* definition of standard cartesian units */
#ifndef lint
static const char SCCSID[]="@(#)pj_units.c	4.6	93/06/12	GIE	REL";
#endif
#define PJ_UNITS__
#include <stdio.h>
#include <string.h>
#include <anvil/projects.h>
#include <anvil/projectsP.h>
#include <anvil/errcode.h>
#include <anvil/files.h>
#include <anvil/units.h>

/* Field 2 that contains the multiplier to convert named units to meters
** may be expressed by either a simple floating point constant or a
** numerator/denomenator values (e.g. 1/1000) */

#if 0
	struct PJ_UNITS
pj_units[] = {
	"km",	"1000.",	"Kilometer",
	"m",	"1.",		"Meter",
	"dm",	"1/10",		"Decimeter",
	"cm",	"1/100",	"Centimeter",
	"mm",	"1/1000",	"Millimeter",
	"kmi",	"1852.0",	"International Nautical Mile",
	"in",	"0.0254",	"International Inch",
	"ft",	"0.3048",	"International Foot",
	"yd",	"0.9144",	"International Yard",
	"mi",	"1609.344",	"International Statute Mile",
	"fath",	"1.8288",	"International Fathom",
	"ch",	"20.1168",	"International Chain",
	"link",	"0.201168",	"International Link",
	"us-in",	"1./39.37",	"U.S. Surveyor's Inch",
	"us-ft",	"0.304800609601219",	"U.S. Surveyor's Foot",
	"us-yd",	"0.914401828803658",	"U.S. Surveyor's Yard",
	"us-ch",	"20.11684023368047",	"U.S. Surveyor's Chain",
	"us-mi",	"1609.347218694437",	"U.S. Surveyor's Statute Mile",
	"ind-yd",	"0.91439523",	"Indian Yard",
	"ind-ft",	"0.30479841",	"Indian Foot",
	"ind-ch",	"20.11669506",	"Indian Chain",
(char *)0, (char *)0, (char *)0
};
#endif

extern "C" struct PJ_UNITS *pj_units;

/* read the projection unit constants, conversion values and notes from
   the lookup file */

AmErrorCode ReadProjUnits( char	*filename)
{
    int		length, ii;
    FILE	*fp;
    char	buffer[ FIELDLEN], unitbuf[ FIELDLEN], 
		convbuf[ FIELDLEN], notebuf[ FIELDLEN];
    unitType	code;

/* find out how big the file is, open it, and allocate a vector of structures */

    if (( length = AmNumLines( filename)) < 0) {
	return( ANVIL_ERROR_CODE = AmOPEN_FAILED);
    }
    if (( fp = fopen( filename, "r")) == NULL) {
	return( ANVIL_ERROR_CODE = AmOPEN_FAILED);
    }
    if (( pj_units = ( struct PJ_UNITS *) 
	  malloc( sizeof( struct PJ_UNITS)*( length + 1))) == NULL) {
	fclose( fp);
	return( ANVIL_ERROR_CODE = AmMALLOC_FAILED);
    }

/* loop over and read the id, conversion and name fields */

    for ( ii = 0; ii < length; ii++) {
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    break;
	}
	if ( sscanf( buffer, "%s %s %[^\n]", 
		     unitbuf, convbuf, notebuf) != 3) {
	    break;
	}
	pj_units[ ii].id = strdup( unitbuf);
	pj_units[ ii].to_meter = strdup( convbuf);

/* chop the '\n' from the end of the note buffer */

	notebuf[ strlen( notebuf) - 1] = '\0';
	pj_units[ ii].name = strdup( notebuf);
	AN_DEBUG( 90, ("proj[ %d] = %s, %s, %s\n", ii,
			pj_units[ ii].id,
			pj_units[ ii].to_meter,
			pj_units[ ii].name));
	if ( !AmGetUnitCode( pj_units[ ii].id, &code)) {
	    AN_DEBUG( 10, ( "failedToFindProjUnitCode %s %s",
			    pj_units[ ii].id, filename));
	}
    }
    fclose( fp);

/* terminate the vector and return */

    pj_units[ ii].id = NULL;
    pj_units[ ii].to_meter = NULL;
    pj_units[ ii].name = NULL;
    return( AmOK);
}
