#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <math.h>
#include <stdio.h>
#include "strlib.h"
#include <anvil/projectsP.h>
#include "genericfile.h"

int projectImage( int		fd,
		   fileDataPtr	fileData,
		   char		**args)
{
    PJ		*proj;
    UV		p;
    int		i, j;
    coordsRec	coords;
    double	x, y;

    if (( proj = pjInit( veclen( args), args)) == NULL) {
	return( FALSE);
    }

/* project the map corners */

    p.u = fileData->coords.xmin;
    p.v = fileData->coords.ymin;
    p.u *= DEG_TO_RAD;
    p.v *= DEG_TO_RAD;
    p = pjFwd( p, proj);
    coords.xmin = coords.currXmin = p.u;
    coords.ymin = coords.currYmin = p.v;
    p.u = fileData->coords.xmax;
    p.v = fileData->coords.ymax;
    p.u *= DEG_TO_RAD;
    p.v *= DEG_TO_RAD;
    p = pjFwd( p, proj);
    coords.xmax = coords.currXmax = p.u;
    coords.ymax = coords.currYmax = p.v;
    for ( i = 0; i < fileData.dimensions.dims[ 0]; i++) {
	for ( j = 0; j < fileData.dimensions.dims[ 1]; j++) {
	}
    }
    return( TRUE);
}




void main( int argc, char *argv[])
{
    PJ		*proj;
    fileDataRec	fileData;
    FILE	*fpIn, *fpOut;
    char	**args;

    if (( fpIn = fopen( argv[ 1], "r")) == NULL) {
	perror( "opening input file");
	exit( -1);
    }
    if (( fpOut = fopen( argv[ 1], "w")) == NULL) {
	perror( "opening output file");
	exit( -1);
    }
    args = readStrvec( argv[ 3]);
    if ( args == NULL) {
	perror( "opening arguments file");
	exit( -1);
    }
    if ( !readGeneric( fileno( fpIn), &fileData)) {
	perror( "generic read failed");
	exit( -1);
    }
    switch ( fileData.fileType) {
    case imageFile:
	projectImage( fileno( fpOut), &fileData, args);
	break;
    case scatterFile:
	break;
    case networkFile:
	break;
    case polygonFile:
	break;
    default:
	break;
    }
}
