#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <windows.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <anvil/ntresource.h>
#include <anvil/project.h>
#include <anvil/envdefs.h>
#include <anvil/units.h>
#include <anvil/projects.h>
#include <anvil/strlib.h>
#include <anvil/display.h>
#include <anvil/ntapp.h>
#include <anvil/enums.h>
#include <anvil/template.h>
#include <anvil/ntcolor.h>
#include <anvil/log.h>


/* Perform initializations needed for any application. */

AmErrorCode
AmInitApplication( const char *configFile)
{
    AmErrorCode	errCode;
    struct stat	buf;
    char        sz[ TEXTLEN];

  /* Do some sanity checking. */

  /* 
   * NOTE:
   *   Check to make sure CONFIG_ENV is set, and that it points to a valid
   *   directory .
   */

    if ( getenv( CONFIG_ENV) == NULL) {
        sprintf( sz, "Error: The environment variable \"%s\" is not set.\n",
                 CONFIG_ENV);
        fprintf( stderr, sz);
	DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }

    if ( stat( getenv( CONFIG_ENV), &buf) < 0) {
	sprintf( sz, "Error: Configuration directory \"%s does not exist.\n",
		 getenv( CONFIG_ENV));
        fprintf( stderr, sz);
	DO_RETURN_ERROR( AmFILE_NO_EXIST);
    }

    /* Initialize the error log */

    AmInitLog();

    /* Initialize the NT database and the units and variables database. */

    AmSetTheResourceDb( new AmNTResourceDbClass( configFile));
    if (( errCode = AmInitUnitStuff()) != AmOK) {
	sprintf( sz, "Error: Failed to initialize units.\n");
        fprintf( stderr, sz);
	return( errCode);
    }

    /* Initialize the table of object initializers. */

    AmAddDefaultObjectInits();

   /* Initialize the standard lookup table. */

    AmCreateLookupTable();

    /* Initialize the colormaps. */

    InitColormaps();

    return( AmOK);
}
