#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <windows.h>
#include <anvil/project.h>
#include <anvil/dither.h>
#include <anvil/colorcell.h>

#define NUM_RED   8
#define NUM_GREEN 8
#define NUM_BLUE  4

/* Colormap in device independent format. */

AmColorCellRec pcolColorCube[ 256];

/* Number of shades used for each of the colors in the color cube. */

static ushort_t nRedShade;
static ushort_t nGreenShade;
static ushort_t nBlueShade;

/* Offset tables into the color cube for specific shades of a color. */

static ulong_t  plRedOffset[ 256];
static ulong_t  plGreenOffset[ 256];
static ulong_t  plBlueOffset[ 256];

/* Dithering tables. */

uchar_t pcDecisionMatrix[ NUM_DITHER_MATRICES][ DITHER_LEVEL][ DITHER_LEVEL];

static ulong_t plRedDitherMatrix[ 256]; 
static ulong_t plGreenDitherMatrix[ 256]; 
static ulong_t plBlueDitherMatrix[ 256]; 

/* Specific NT color information for the application. */

static HPALETTE     hPalette;
static int          nPaletteSize;
static ulong_t      *plPaletteOffset;

/* Specific NT display information for the application. */

static int          nColors;
static int          nDepth;

const HPALETTE
GetAppPalette()
{
    return( hPalette);
}


const ulong_t *
GetAppPaletteOffsets()
{
    return( plPaletteOffset);
}

ushort_t
GetNTNumRedShades()
{
    return( nRedShade);
}

ushort_t
GetNTNumGreenShades()
{
    return( nGreenShade);
}

ushort_t
GetNTNumBlueShades()
{
    return( nBlueShade);
}

const ulong_t *
GetNTRedOffsetLookup()
{
    return( plRedOffset);
}

const ulong_t *
GetNTGreenOffsetLookup()
{
    return( plGreenOffset);
}

const ulong_t *
GetNTBlueOffsetLookup()
{
    return( plBlueOffset);
}

const ulong_t *
GetNTRedDitherMatrix()
{
    return( plRedDitherMatrix);
}

const ulong_t *
GetNTGreenDitherMatrix()
{
    return( plGreenDitherMatrix);
}

const ulong_t *
GetNTBlueDitherMatrix()
{
    return( plBlueDitherMatrix);
}

int
GetAppNumColors()
{
    return( nColors);
}

int
AmGetAppDepth()
{
    return( nDepth);
}

const AmColorCellRec *
GetAppColorCube()
{
	return( pcolColorCube);
}

AmErrorCode
InitColormaps()
{
    HDC          hDCInfo, hDC;
    HPALETTE     hPaletteOld;
	LPLOGPALETTE pLogPalette;
    PALETTEENTRY *ppe;
    int          nBitsPerPixel, nPlanes, nColor, nPalette, ii;
    ulong_t      rgb;

    /* Initializations. */

    pLogPalette     = NULL;
    hPalette        = NULL;
    nPaletteSize    = 0;
    plPaletteOffset = 0;

    /* Get information about the color capabilities of the device. */

    hDC = CreateDC( "DISPLAY", NULL, NULL, NULL);

    nBitsPerPixel = GetDeviceCaps( hDC, BITSPIXEL);
    nPlanes       = GetDeviceCaps( hDC, PLANES);
    nColor        = GetDeviceCaps( hDC, NUMCOLORS);

    /* If the device doesn't support true color, create a color cube
       for use in dithering. */

    if ( nBitsPerPixel < 16) {

        nPaletteSize = NUM_RED * NUM_GREEN * NUM_BLUE;
        pLogPalette = ( LOGPALETTE *) malloc( sizeof( LOGPALETTE) +
                                              sizeof( PALETTEENTRY) *
                                              nPaletteSize);
        if ( pLogPalette == NULL) {
            DO_RETURN_ERROR( AmMALLOC_FAILED);
        }

        pLogPalette->palVersion    = 0x300;
        pLogPalette->palNumEntries = nPaletteSize;

        AmCreateColorCube( pcolColorCube, NUM_RED, NUM_GREEN, NUM_BLUE,
                         plRedOffset, plGreenOffset, plBlueOffset);

        for ( ii = 0; ii < nPaletteSize; ii++) {
            ppe = &( pLogPalette->palPalEntry[ ii]);
            ppe->peRed = pcolColorCube[ nPaletteSize - ii - 1].red;
            ppe->peGreen = pcolColorCube[ nPaletteSize - ii - 1].green;
            ppe->peBlue = pcolColorCube[ nPaletteSize - ii - 1].blue;
            ppe->peFlags = 0; 
        }

        hPalette = CreatePalette( pLogPalette);
        free( pLogPalette);

        nColor   = GetDeviceCaps( hDC, NUMCOLORS);
        nPalette = GetDeviceCaps( hDC, SIZEPALETTE);

        /* Combine our palette with the system palette. */

        SetSystemPaletteUse( hDC, SYSPAL_NOSTATIC);
        SetSystemPaletteUse( hDC, SYSPAL_STATIC);

        /* Realize "optimal" palette. */

        hPaletteOld = SelectPalette( hDC, hPalette, FALSE);
        RealizePalette( hDC);
        SelectPalette( hDC, hPaletteOld, FALSE);

        /*
         * Read the system palette which is now a combination of
         *   its'  20 reserved colors and our palette.
         */

        ppe = ( PALETTEENTRY *) malloc( sizeof( PALETTEENTRY) * nPalette);
        if ( ppe == NULL) {
            DO_RETURN_ERROR( AmMALLOC_FAILED);
        }
        GetSystemPaletteEntries( hDC, 0, nPalette, ppe);

        for ( ii = 0; ii < nColor / 2; ii++) {
            ppe[ ii].peFlags = 0; 
        }

        for ( ii = nColor / 2; ii < nPalette - nColor / 2; ii++) {
            ppe[ ii].peFlags = PC_NOCOLLAPSE; 
        }

        for ( ii = nPalette - nColor / 2; ii < nPalette / 2; ii++) {
            ppe[ ii].peFlags = 0; 
        }

        /* Create the "identity palette". */
 
        ResizePalette( hPalette, nPalette);
        ii = SetPaletteEntries( hPalette, 0, nPalette, ppe);

        plPaletteOffset = ( ulong_t *) malloc( sizeof( ulong_t) * nPaletteSize);
        for ( ii = 0; ii < nPaletteSize; ii++) {
            rgb = RGB( pcolColorCube[ ii].red, pcolColorCube[ ii].green,
                       pcolColorCube[ ii].blue);
            plPaletteOffset[ ii] = GetNearestPaletteIndex( hPalette,
                                                           rgb);
        }

        free( ppe);

        AmInitDitherMatrices( NUM_RED, NUM_GREEN, NUM_BLUE, 
                            pcDecisionMatrix, plRedDitherMatrix,
                            plGreenDitherMatrix, plBlueDitherMatrix);

        nColors = NUM_RED * NUM_GREEN * NUM_BLUE;
    } else {
        nColors = 0;
    }

    nDepth = nBitsPerPixel;

    DeleteDC( hDC);

    return( AmOK);
}


/* 
 * Comments:
 *   Use this to set the pixel value of a color cell.  DO NOT USE to set a pixel value in
 *   an image, use the MACRO CALCULATE_PIXEL( r, g, b) instead.
 */

void
AmSetPixelValue( AmColorCellPtr pcol)
{
    if ( hPalette != NULL) {
    } else {
        pcol->pixel = RGB( pcol->red, pcol->green, pcol->blue);
    }
}


void
SetPalette( HDC hDC)
{
    if ( hPalette != NULL) {
        SelectPalette( hDC, hPalette, FALSE);
        RealizePalette( hDC);
    }
}
