#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

#include <anvil/mem_funcs.h>
#include <anvil/ntdisplay.h>
#include <anvil/ntcolor.h>

#define SCREEN_COORD( iVirtual)                                          \
    ( iVirtual * _pDisplay->GetDPI() / _pDisplay->GetDPIBase())
#define VIRTUAL_COORD( iScreen)                                          \
    ( iScreen * _pDisplay->GetDPIBase() / _pDisplay->GetDPI())

#define SCREEN_COORD_FLOAT( dVirtual)                                    \
    ( dVirtual * _pDisplay->GetDPI() / ( float) _pDisplay->GetDPIBase())

HFONT
LoadFont( char           *szFontName, 
          int            iFontSize,
          AmFontWeightEnum eFontWeight,
          AmFontSlantEnum  eFontSlant,
          float          dxScale,
          float          dyScale,
          int            iFontDPI,
          float          dAngle)
{
    LOGFONT lf;
    HFONT   hFont;

    strcpy( lf.lfFaceName, szFontName);
    lf.lfHeight         = - ( iFontSize * iFontDPI / 72);
    lf.lfWidth          = 0;
    lf.lfEscapement     = ( int)( dAngle * 10);
    lf.lfOrientation    = ( int)( dAngle * 10);
    lf.lfWeight         = eFontWeight == AmWEIGHT_BOLD ? 700 : 0;
    lf.lfItalic         = eFontSlant == AmSLANT_ITALIC ? TRUE : FALSE;
    lf.lfUnderline      = 0;
    lf.lfStrikeOut      = 0;
    lf.lfCharSet        = 0;
    lf.lfOutPrecision   = 0;
    lf.lfClipPrecision  = 0;
    lf.lfQuality        = 0;
    lf.lfPitchAndFamily = 0;

    hFont = CreateFontIndirect( &lf);
    return( hFont);
}


/* Create a Microsoft GC class. */

AmNTGCClass::AmNTGCClass()
    : AmGCClass()
{
    _dLineWidth        = ( float) 0.0;
    _eLineJoin         = AmLINEJOIN_ERROR;
    _eLineStyle        = AmLINESTYLE_ERROR;
    _eLineCap          = AmLINECAP_ERROR;
    _fLineDirty        = FALSE;

    _eRasterOp         = AmRASTEROP_ERROR;
    _fRasterOpDirty    = FALSE;
    
    _pcDashPattern     = NULL;
    _nDashPattern      = 0;
    _iDashOffset       = 0;
    _fDashPatternDirty = FALSE;

    _pRec              = NULL;
    _nRec              = 0;
    _fRecDirty         = FALSE;
 
    _fDirty            = FALSE; 

    _hPen              = NULL;
    _hBrush            = NULL;
    _hRgn              = NULL;

    _hFont             = NULL;
    _clipRegion	       = NULL;
    _clipRegionDirty   = FALSE;
}

AmNTGCClass::AmNTGCClass( AmWindowClass *pWindow)
    : AmGCClass( pWindow)
{
    AmNTWindowClass *pNTWindow = ( AmNTWindowClass *) pWindow;

    _dLineWidth          = ( float) 0.0;    
    _eLineJoin           = AmJOIN_MITER; 
    _eLineStyle          = AmLINE_SOLID;
    _eLineCap            = AmCAP_NOT_LAST;
    _fLineDirty          = TRUE;

    _eRasterOp           = AmCOPY;
    _fRasterOpDirty      = TRUE;
    
    _pcDashPattern       = NULL;
    _nDashPattern        = 0;
    _iDashOffset         = 0;
    _fDashPatternDirty   = TRUE;

    _font.szFont   = strdup( "Times New Roman");
    _font.szFamily = strdup( "");
    _font.iSize    = 10;
    _font.eWeight  = AmWEIGHT_MEDIUM;
    _font.eSlant   = AmSLANT_ROMAN;
    _font.dxScale  = 1.0; 
    _font.dyScale  = 1.0;
    _font.iDPI     = _pDisplay->GetDPI();
    _fFontDirty    = TRUE;

    _colForeground.red   = 255;
    _colForeground.green = 255;
    _colForeground.blue  = 255;
    _pDisplay->AllocColor( &_colForeground);
    _fColForegroundDirty = TRUE;

    _colBackground.red   = 255;
    _colBackground.green = 255;
    _colBackground.blue  = 255;
    _pDisplay->AllocColor( &_colBackground);
    _fColBackgroundDirty = TRUE;

    _pRec                = NULL;
    _nRec                = 0;
    _fRecDirty           = TRUE;

    _fDirty              = TRUE;

    _hPen                = NULL;
    _hBrush              = NULL;
    _hRgn                = NULL;
  
    _hFont               = NULL;
}

/* Destroy a Microsoft GC class. */

AmNTGCClass::~AmNTGCClass()
{
    AmNTWindowClass *pNTWindow = ( AmNTWindowClass *) _pWindow;

    /* Make sure that the currently active DC doesn't have any pens
       or brushes selected into it. */

    if ( pNTWindow != NULL) {
        pNTWindow->releaseHDC( this);
    }

    if ( _hPen != NULL) {
        DeleteObject( _hPen);
    }

    if ( _hBrush != NULL) {
        DeleteObject( _hBrush);
    }

    if ( _hRgn != NULL) {
        DeleteObject( _hRgn);
    }

    if ( _hFont != NULL) {
        DeleteObject( _hFont);
    }

    if ( _font.szFont != NULL) {
        FREE( _font.szFont);
    }

    if ( _font.szFamily != NULL) {
        FREE( _font.szFamily);
    }
}

/* Set the line width. */

void
AmNTGCClass::SetLineWidth( float dLineWidth)
{
    _dLineWidth = dLineWidth;
    _fDirty = _fLineDirty = TRUE;
}


/* Retrieve the line width */

float AmNTGCClass::GetLineWidth()
{
    return( _dLineWidth);
}


/* Set the line join. */

void
AmNTGCClass::SetLineJoin( AmLineJoinEnum eLineJoin)
{
    _eLineJoin = eLineJoin;
    _fDirty = _fLineDirty = TRUE;
}

/* Retrieve the line join */

AmLineJoinEnum
AmNTGCClass::GetLineJoin() 
{
    return( _eLineJoin);
}


/* Set the line style. */

void
AmNTGCClass::SetLineStyle( AmLineStyleEnum eLineStyle)
{
    _eLineStyle = eLineStyle;
    _fDirty = _fLineDirty = TRUE;
}

/* Retrieve the line style. */

AmLineStyleEnum
AmNTGCClass::GetLineStyle() 
{
    return( _eLineStyle);
}

/* Retrieve the NT text metrics structure from the GC. */

TEXTMETRIC *
AmNTGCClass::GetNTFont()
{
    AmNTWindowClass *pNTWindow = ( AmNTWindowClass *) _pWindow;

    if ( _fFontDirty == TRUE) {
        pNTWindow->getHDC( this);
    }

    return( &_tmFont); 
}

/* Set the font information. */

AnvilBoolean
AmNTGCClass::SetFont( AmFontRec *pFont)
{
    if ( pFont != NULL) {
        return( SetFont( pFont->szFont, pFont->iSize, pFont->eWeight,
                         pFont->eSlant, pFont->dxScale, pFont->dyScale)); 
    }

    return( FALSE);
}

/* Retrieve the current font information. */

void
AmNTGCClass::GetFont( AmFontRec *pFont)
{
    if ( pFont != NULL) {
        pFont->szFont = strdup( _font.szFont);
        pFont->iSize = _font.iSize; 
        pFont->eWeight = _font.eWeight; 
        pFont->eSlant = _font.eSlant; 
        pFont->dxScale = _font.dxScale; 
        pFont->dyScale = _font.dyScale; 
    }
}

/* Set the font, and font sizing. */

AnvilBoolean
AmNTGCClass::SetFont( char           *szFontName, 
                    int            iFontSize,
                    AmFontWeightEnum eFontWeight,
                    AmFontSlantEnum  eFontSlant,
                    float          dxFontScale,
                    float          dyFontScale)
{
    /* See if anything has changed before we do anything. */

    if (( strcmp( _font.szFont, szFontName) != 0) ||
        ( _font.iSize != iFontSize) ||
        ( _font.eWeight != eFontWeight) ||
        ( _font.eSlant != eFontSlant) ||
        ( _font.dxScale != dxFontScale) ||
        ( _font.dyScale != dyFontScale) ||
        ( _font.iDPI != _pDisplay->GetDPI())) {

        if ( strcmp( _font.szFont, szFontName) != 0) {
            FREE( _font.szFont);
            _font.szFont = strdup( szFontName);
        }

        _font.iSize = iFontSize;
        _font.eWeight = eFontWeight;
        _font.eSlant = eFontSlant;
        _font.dxScale = dxFontScale;
        _font.dyScale = dyFontScale;
        _font.iDPI = _pDisplay->GetDPI();

        _fDirty = _fFontDirty = TRUE;
    }

    return( TRUE);
}


/* Retrieve the current font information. */

void
AmNTGCClass::GetFont( char           **pszFontName,
                    int            *piFontSize,
                    AmFontWeightEnum *peFontWeight,
                    AmFontSlantEnum  *peFontSlant,
                    float          *pdxFontScale,
                    float          *pdyFontScale)
{
    *pszFontName = strdup( _font.szFont);

    if ( piFontSize != NULL) {
        *piFontSize = _font.iSize;
    }

    if ( peFontWeight != NULL) {
        *peFontWeight = _font.eWeight;
    }

    if ( peFontSlant != NULL) {
        *peFontSlant = _font.eSlant;
    }

    if ( pdxFontScale != NULL) {
        *pdxFontScale = _font.dxScale;
    }

    if ( pdyFontScale != NULL) {
        *pdyFontScale = _font.dyScale;
    }
}


/* Get the foreground pixel. */

void
AmNTGCClass::SetForegroundColor( AmColorCellPtr pcol)
{
    _colForeground.red = pcol->red;
    _colForeground.green = pcol->green;
    _colForeground.blue = pcol->blue;
    _pDisplay->AllocColor( &_colForeground);

//    AmSetPixelValue( &_colForeground);

    _fDirty = _fColForegroundDirty = TRUE;
}

/* Retrieve the foreground pixel. */

void
AmNTGCClass::GetForegroundColor( AmColorCellPtr pcol)
{
    *pcol = _colForeground;
}

/* Set the background pixel. */

void
AmNTGCClass::SetBackgroundColor( AmColorCellPtr pcol)
{
    _colBackground.red = pcol->red;
    _colBackground.green = pcol->green;
    _colBackground.blue = pcol->blue;
    _pDisplay->AllocColor( &_colBackground);
//    AmSetPixelValue( &_colBackground);

    _fDirty = _fColBackgroundDirty = TRUE;
}

/* Retrieve the background pixel. */

void
AmNTGCClass::GetBackgroundColor( AmColorCellPtr pcol)
{
    *pcol = _colBackground;
}


/* Set the vector of clipping rectangles */

void
AmNTGCClass::SetClipRectangles( AmRectangleRec const *pRec,
                              int                nRec)
{
    if ( _pRec != NULL) {
        FREE( _pRec);
    }

    _nRec = nRec;
    _pRec = ( AmRectanglePtr) AmCopyVec( pRec, nRec * sizeof( AmRectangleRec)); 

    _fDirty = _fRecDirty = TRUE;
}

void 
AmNTGCClass::SetClipRegion( AmRegionClass	*region)
{
    if ( _clipRegion != NULL) {
	delete _clipRegion;
    }
    _clipRegion = region->CopyRegion();
    _clipRegionDirty = TRUE;
}


/* Retrieve the clipping rectangle vector */

void
AmNTGCClass::GetClipRectangles( AmRectanglePtr *ppRec,
                              int          *nRec)
{
    *nRec = 0;
    *ppRec = ( AmRectanglePtr) AmCopyVec( _pRec, _nRec * sizeof( AmRectangleRec));
}

/* set the raster operation */

void
AmNTGCClass::SetRasterOp( AmRasterOpEnum eRasterOp)
{
    _eRasterOp = eRasterOp;

    _fDirty = _fRasterOpDirty = TRUE;
}


/* Retrieve the raster operation */

AmRasterOpEnum
AmNTGCClass::GetRasterOp()
{
    return( _eRasterOp);
}

/* Set the dash pattern. */

void
AmNTGCClass::SetDashes( char *pcDashPattern,
                      int  nDashPattern,
                      int  iDashOffset)
{
    _fDirty = _fLineDirty = TRUE;
}

/* Retrieve the dash pattern. */

void
AmNTGCClass::GetDashes( char **ppcDashPattern,
                      int  *pnDashPattern,
                      int  *piDashOffset)
{
    *ppcDashPattern = NULL;
    *pnDashPattern = 0;
    *piDashOffset = 0;
}

/* Set the line cap style from the enumeration */

void
AmNTGCClass::SetLineCap( AmLineCapEnum eLineCap)
{
    _eLineCap = eLineCap;

    _fDirty = _fLineDirty = TRUE;
}


/* Retrieve the current line cap. */

AmLineCapEnum
AmNTGCClass::GetLineCap()
{ 
    return( _eLineCap);
}


void
AmNTGCClass::SetFillStyle( AmFillStyleEnum fillStyle)
{ 
	_fillStyle = fillStyle;
}

AnvilBoolean
AmNTGCClass::SetFillPattern( const char *fillPatFile)
{
	if ( _fillPatFile != NULL) {
		free( _fillPatFile);
	}
	_fillPatFile = strdup( fillPatFile);
	return( TRUE);
}

HDC
AmNTGCClass::getHDC()
{
    AmNTWindowClass *pNTWindow = ( AmNTWindowClass *) _pWindow;
    HDC           hDC;

    hDC = pNTWindow->getHDC( this);
    return( hDC);
}


void
AmNTGCClass::setPen( HDC hDC)
{
    HPEN hPen;
    int  iPenStyle;

    switch( _eLineStyle) {
    case AmLINE_SOLID:
        iPenStyle = PS_SOLID;
        break;
    case AmLINE_ON_OFF_DASH:
        iPenStyle = PS_DASH;
        break;
    case AmLINE_DOUBLE_DASH:
        iPenStyle = PS_DASHDOTDOT;
        break;
    default:
        iPenStyle = PS_SOLID;
    }    

    hPen = CreatePen( iPenStyle,
                      ( int) SCREEN_COORD_FLOAT( _dLineWidth),
                      _colForeground.pixel);

    SelectObject( hDC, hPen);
    if ( _hPen != NULL) {
        DeleteObject( _hPen);
    }
    _hPen = hPen; 

    _fLineDirty = FALSE;
}

void
AmNTGCClass::setRasterOp( HDC hDC)
{
    int iDrawMode;

    switch( _eRasterOp) {
    case AmSET:
        iDrawMode = R2_WHITE;
        break;
    case AmCLEAR:
        iDrawMode = R2_BLACK;
        break;
    case AmAND:
        iDrawMode = R2_MASKPEN;
        break;
    case AmAND_REVERSE:
        iDrawMode = R2_MASKPENNOT;
        break;
    case AmAND_INVERTED:
        iDrawMode = R2_MASKNOTPEN;
        break;
    case AmNAND:
        iDrawMode = R2_NOTMASKPEN;
        break;
    case AmOR:
        iDrawMode = R2_MERGEPEN;
        break;
    case AmOR_REVERSE:
        iDrawMode = R2_MERGEPENNOT;
        break;
    case AmOR_INVERTED:
        iDrawMode = R2_MERGENOTPEN;
        break;
    case AmNOR:
        iDrawMode = R2_NOTMERGEPEN;
        break;
    case AmXOR:
        iDrawMode = R2_XORPEN;
        break;
    case AmEQUIV:
        iDrawMode = R2_NOTXORPEN;
        break;
    case AmCOPY:
        iDrawMode = R2_COPYPEN;
        break;
    case AmCOPY_INVERTED:
        iDrawMode = R2_NOTCOPYPEN;
        break;
    case AmNOOP:
        iDrawMode = R2_NOP;
        break;
    case AmINVERT:
        iDrawMode = R2_NOT;
        break;
    default:
        iDrawMode = R2_COPYPEN;
    }

    SetROP2( hDC, iDrawMode);
}

void
AmNTGCClass::setClipRegion( HDC hDC)
{
    HRGN  hRgn, hRgnTemp;
    int   ii, x, y, cx, cy;
    float dx, dy;

    hRgn = NULL;
    for ( ii = 0; ii < _nRec; ii++) {

        x = SCREEN_COORD( _pRec[ ii].x);
//        dx = SCREEN_COORD_FLOAT( _pRec[ ii].x) - x;

        y = SCREEN_COORD( _pRec[ ii].y);
//        dy = SCREEN_COORD_FLOAT( _pRec[ ii].y) - y;


        cx = SCREEN_COORD( _pRec[ ii].width);
        cy = SCREEN_COORD( _pRec[ ii].height);

//        cx = ceil( SCREEN_COORD_FLOAT( _pRec[ ii].width) + dx);
//        cy = ceil( SCREEN_COORD_FLOAT( _pRec[ ii].height) + dy);

        hRgnTemp = CreateRectRgn( x, y, x + cx + 1, y + cy + 1);
  
        if ( ii == 0) {
            hRgn = hRgnTemp;
        } else {
            CombineRgn( hRgn, hRgn, hRgnTemp, RGN_OR);
            DeleteObject( hRgnTemp);
        }
    }

    SelectClipRgn( hDC, hRgn);
    if ( _hRgn != NULL) {
        DeleteObject( _hRgn);
    }
    _hRgn = hRgn;

    _fRecDirty = FALSE;
}

void
AmNTGCClass::setFont( HDC hDC)
{
    HFONT hFont;

    hFont = LoadFont( _font.szFont, _font.iSize,
                      _font.eWeight, _font.eSlant,
                      _font.dxScale, _font.dyScale,
                      _pDisplay->GetDPI(), 0.0);
    SelectObject( hDC, hFont);
    if ( _hFont != NULL) {
        DeleteObject( _hFont);
    }
    _hFont = hFont;

    GetTextMetrics( hDC, &_tmFont);

    _fFontDirty = FALSE;
}

/* Set the DC and create a pen and brush according to the GC values. */

void
AmNTGCClass::setHDC( HDC hDC)
{
    HBRUSH   hBrush;

    /* If the application has a colormap, realize it. */

    SetPalette( hDC);

    /* If the pen has changed since the AmLast time the DC was set or
       cleaned, create a new one. */
 
    if (( _hPen == NULL) || ( _fLineDirty == TRUE) ||
        ( _fColForegroundDirty == TRUE)) {
        setPen( hDC);
    } else {
        SelectObject( hDC, _hPen);
    }

    /* Set the foreground and background colors, they always need
       to be set, since they don't stay around. */

    SetTextColor( hDC, _colForeground.pixel);
    _fColForegroundDirty = FALSE;

    SetBkColor( hDC, _colBackground.pixel);
    _fColBackgroundDirty = FALSE;

    /* Set the raster operation. */

    setRasterOp( hDC);

    /* Set the clipping regions. */

    if ( _fRecDirty == TRUE) {
        setClipRegion( hDC);
    } else {
        SelectClipRgn( hDC, _hRgn);
    }

    /* If the font has changed since the AmLast time the DC was set or
       cleaned, create a new one. */

    if (( _hFont == NULL) || ( _fFontDirty == TRUE)) {
        setFont( hDC);
    } else {
        SelectObject( hDC, _hFont);
    }

    _fDirty = FALSE;
}

/* Set the fields in the DC that have changed. */

void
AmNTGCClass::cleanHDC( HDC hDC)
{
    HBRUSH hBrush;
  
    if ( _fDirty == FALSE) {
        return;
    }

    /* Update the pen if the line style, color, or width has changed. */

    if (( _fLineDirty == TRUE) || ( _fColForegroundDirty == TRUE)) {
        setPen( hDC);
    }

    /* Update the foreground and background color if they have 
       changed. */

    if ( _fColForegroundDirty == TRUE) {
        SetTextColor( hDC, _colForeground.pixel);
        _fColForegroundDirty = FALSE;
    }

    if ( _fColBackgroundDirty == TRUE) {
        SetBkColor( hDC, _colBackground.pixel);
        _fColBackgroundDirty = FALSE;
    }

    /* Update the raster operation if it has changed. */

    if ( _fRasterOpDirty == TRUE) {
        setRasterOp( hDC);
    }

    /* Update the clipping region. */

    if ( _fRecDirty == TRUE) {
        setClipRegion( hDC);
    }

    /* Update the font. */

    if ( _fFontDirty == TRUE) {
        setFont( hDC);
    }

    _fDirty = FALSE;
}

/*
 * Function: void Update()
 *
 * Description:
 *   Update gets called from the GC's parent window.  Which in turn gets
 *   called from its' display.  This happens when the display was changed.
 *   Specifically, the update function is used to reload DPI dependent
 *   parts of the GC when the DPI of the display has changed.
 *
 * Returns:
 *   -nothing-
 */

void
AmNTGCClass::Update()
{
    _fLineDirty = TRUE;
    _fFontDirty = TRUE;
    _fDirty = TRUE;
}


HFONT
AmNTGCClass::GetRotatedFont( float dAngle)
{
    HFONT hFont;

    hFont = LoadFont( _font.szFont, _font.iSize,
                      _font.eWeight, _font.eSlant,
                      _font.dxScale, _font.dyScale,
                      _pDisplay->GetDPI(), dAngle);

    return( hFont);
}
