#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <windows.h>
#include <anvil/resource.h>
#include <anvil/plot_handlers.h>
#include <anvil/template.h>
#include <anvil/nt.h>
#include <anvil/log.h>
#include <anvil/files.h>
#include <anvil/legend.h>
#include <anvil/imagelegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/linelegend.h>
#include <anvil/polylegend.h>
#include <anvil/nt_user_data.h>
#include <resources/resource.h>

static AmNTDisplayManagerClass	*staticDisplayManager = NULL;
static AmPlotClass		*staticPlot = NULL;

static BOOL CALLBACK
colorsDialogProc( HWND		hDlg,
		  UINT		iMsg,
		  WPARAM	wParam,
		  LPARAM	lParam)
{
    char			**cmapListStrings, relPath[ PATHLEN], 
    				*cmapListFile, **ptr, **strings;
    HWND			hWndList, hWndLegend;
    AmNTDisplayClass		*pNTDisplay;
    AmNTWindowClass		*pNTWindow;
    AmNTDisplayManagerClass	*legendDisplayManager;
    AmPlotClass			*plot;
    AmLegendClass			*legend;
    float			min, max, *binvec;
    int				table, width, height, numColors;
    RECT			legendRect;
    AmColorCellPtr		colors;

    switch ( iMsg) {
    case WM_INITDIALOG:

/* read the list of colormaps into the selection list */

	sprintf( relPath, "%s/%s", AM_COLORMAP_DIR, AM_CMAP_LIST_FILE);
	cmapListFile = AmGetEnvFile( CONFIG_ENV, relPath);
	if ( cmapListFile == NULL) {
	    AmPrintResourceError( hDlg, "colormapListOpenFailed");
	    return( FALSE);
	}
	cmapListStrings = AmReadLines( cmapListFile);
	if ( cmapListStrings == NULL) {
	    AmPrintResourceError( hDlg, "colormapListReadFailed");
	    return( FALSE);
	}
	hWndList = GetDlgItem( hDlg, CMAP_LIST);
	for ( ptr = cmapListStrings; *ptr != NULL; ptr++) {
	    SendMessage( hWndList, LB_ADDSTRING, 0, ( LPARAM) *ptr);
	}

/* create the display manager to display a legend of the colormap */

	hWndLegend = GetDlgItem( hDlg, LEGEND_AREA);
	pNTDisplay = new AmNTDisplayClass();
	pNTWindow = new AmNTWindowClass( pNTDisplay, hWndLegend);
	legendDisplayManager = 
	    new AmNTDisplayManagerClass( pNTDisplay, pNTWindow, FALSE);
	plot = staticPlot;
	GetWindowRect( hWndLegend, &legendRect);
	height = legendRect.bottom - legendRect.top;
	width = legendRect.right - legendRect.left;

/* Get the fields that are important for any kind of legend. */

	plot->GetVaFields( AmNcolors, &colors,
			   AmNnumColors, &numColors,
			   AmNcolorsBinvec, &binvec,
			   AmNcolorsTable, &table,
			   AmNcolorsStrings, &strings,
			   AmNmin, &min,
			   AmNmax, &max,
			   NULL);

	if ( plot->IsType( AmIMAGE_PLOT)) {
	    legend = new AmImageLegendClass( pNTWindow, NULL,
					   "imagePlotClass",
					   AmNlegendType, AmLEGEND_CONTINUOUS,
					   AmNnumColors, numColors,
					   AmNcolors, colors,
					   AmNmin, min,
					   AmNmax, max,
					   AmNcolorsBinvec, binvec,
					   AmNcolorsTable, table,
					   AmNcolorsStrings, strings,
					   AmNitemSpacing, 5,
					   AmNfontSize, 10,
					   AmNshowTitle, TRUE,
					   AmNtitle, "Colors",
					   AmNtitleFontSize, 16,
					   AmNtitleFontWeight, AmWEIGHT_BOLD,
					   AmNtitleOffset, 5,
					   AmNeditable, TRUE,
					   AmNshowColors, TRUE,
					   AmNoutline, TRUE,
					   AmNitemWidth, 20,
					   AmNitemHeight, 20,
					   AmNmainItemHeight, height - 90,
					   AmNx, 20,
					   AmNy, 20,
					   NULL);
	} else if ( plot->IsType( AmLINE_PLOT)) {
	    legend = new AmLineLegendClass( pNTWindow, NULL,
					  "linePlotClass",
					  AmNnumColors, numColors,
					  AmNcolors, colors,
					  AmNmin, min,
					  AmNmax, max,
					  AmNcolorsBinvec, binvec,
					  AmNcolorsTable, table,
					  AmNcolorsStrings, strings,
					  AmNitemSpacing, 5,
					  AmNfontSize, 10,
					  AmNshowTitle, TRUE,
					  AmNtitle, "Colors",
					  AmNtitleFontSize, 16,
					  AmNtitleFontWeight, AmWEIGHT_BOLD,
					  AmNtitleOffset, 5,
					  AmNeditable, TRUE,
					  AmNshowColors, TRUE,
					  AmNoutline, TRUE,
					  AmNitemWidth, 20,
					  AmNitemHeight, 20,
					  AmNnumRows, ( height + 5) / 25,
					  AmNx, 20,
					  AmNy, 20,
					  NULL);
	} else if ( plot->IsType( AmPOLYGON_PLOT)) {
	    legend = new AmPolyLegendClass( pNTWindow, NULL,
					  "polygonPlotClass",
					  AmNnumColors, numColors,
					  AmNcolors, colors,
					  AmNmin, min,
					  AmNmax, max,
					  AmNcolorsBinvec, binvec,
					  AmNcolorsTable, table,
					  AmNcolorsStrings, strings,
					  AmNitemSpacing, 5,
					  AmNfontSize, 10,
					  AmNshowTitle, TRUE,
					  AmNtitle, "Colors",
					  AmNtitleFontSize, 16,
					  AmNtitleFontWeight, AmWEIGHT_BOLD,
					  AmNtitleOffset, 5,
					  AmNeditable, TRUE,
					  AmNshowColors, TRUE,
					  AmNoutline, TRUE,
					  AmNitemWidth, 20,
					  AmNitemHeight, 20,
					  AmNnumRows, ( height + 5) / 25,
					  AmNx, 20,
					  AmNy, 20,
					  NULL);
	} else if ( plot->IsType( AmSCATTER_PLOT)) {
	    legend = new AmScatterLegendClass( pNTWindow, NULL,
					     "scatterPlotClass",
					     AmNnumColors, numColors,
					     AmNcolors, colors,
					     AmNmin, min,
					     AmNmax, max,
					     AmNcolorsBinvec, binvec,
					     AmNcolorsTable, table,
					     AmNcolorsStrings, strings,
					     AmNitemSpacing, 5,
					     AmNfontSize, 10,
					     AmNshowTitle, TRUE,
					     AmNtitle, "Colors",
					     AmNtitleFontSize, 16,
					     AmNtitleFontWeight, AmWEIGHT_BOLD,
					     AmNtitleOffset, 5,
					     AmNeditable, TRUE,
					     AmNshowColors, TRUE,
					     AmNoutline, TRUE,
					     AmNitemWidth, 20,
					     AmNitemHeight, 20,
					     AmNnumRows, ( height + 5) / 25,
					     AmNx, 20,
					     AmNy, 20,
					     NULL);
	} else {
	    AmPrintResourceError( hDlg, "screenObjectNotSupported", 
				plot->GetObjectType());
	    return( FALSE);
	}
	legendDisplayManager->AddObject( legend);
	legendDisplayManager->Flush();
    	return( FALSE);
    case WM_COMMAND:
	switch ( LOWORD( wParam)) {
	case IDOK:
	case IDCANCEL:
	    EndDialog( hDlg, 0);
	    return( TRUE);
	}
	break;
    case WM_DESTROY:
    	EndDialog( hDlg, 0);
	return( TRUE);
    }
    return( FALSE);
}

void AmSetColorsCallback( HWND				hWnd,
			  UINT				iMsg,
			  WPARAM			wParam,
			  LPARAM			lParam,
			  AmNTDisplayManagerClass	*displayManager,
			  AmPlotClass			*plot)
{
    HINSTANCE	hInstance;
    int	errCode;

    staticDisplayManager = displayManager;
    staticPlot = plot;
    hInstance = ( HINSTANCE) GetWindowLong( hWnd, GWL_HINSTANCE);
    errCode = DialogBox( hInstance, MAKEINTRESOURCE(COLOR_DIALOG),
    			 hWnd, ( DLGPROC) colorsDialogProc);
    if ( errCode == -1) {
    	AmPrintResourceError( hWnd, "ColorsDialogFailed");
    }
}
			

