#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <stdlib.h>
#include <windows.h>
#include <anvil/ntdispmgr.h>
#include <anvil/coordinates.h>
#include <anvil/dispatch.h>
#include <anvil/userevents.h>
#include <anvil/nt_state_hand.h>
#include <anvil/outline.h>
#include <anvil/macros.h>
#include <anvil/dispmgrargs.h>

/* list of events listened for by the zoom2xHandler */

static UINT panEvents[] = { WM_LBUTTONDOWN, WM_RBUTTONDOWN };


/* cursor suspend function, simply hides the outline */

static void
suspendPanHandler( AmDispMgrArgsPtr dispMgrArgs)
{
    dispMgrArgs->displayManager->HideOutline( dispMgrArgs->outline);
}

/* cursor restore function, draws the outline and displays a restart message */

static void
restartPanHandler( AmDispMgrArgsPtr dispMgrArgs)
{
    dispMgrArgs->displayManager->ShowOutline( dispMgrArgs->outline);
    dispMgrArgs->displayManager->WriteResourceMessage( "restartZoom2xHandler");
}

/* remove function removes the outline from the outline list, and frees
   the plot arguments */

static void
removePanHandler( AmDispMgrArgsPtr dispMgrArgs)
{
    dispMgrArgs->displayManager->RemoveOutline( dispMgrArgs->outline);
    AmFreeDispMgrArgs( dispMgrArgs);
}

/***************************************************************************
 *
 * Description:
 *	pan drag handler...update the outline's current X,Y.
 * Arguments:
 *	iMsg - WM_MOUSEMOVE.
 *
 ***************************************************************************/

static void 
panDragHandler( HWND		hWnd,
		UINT		iMsg,
		WPARAM		wParam,
		LPARAM		lParam,
		AmDispMgrArgsPtr	dispMgrArgs)
{
    AmDragLineOutlineClass        *outline = ( AmDragLineOutlineClass *)
						dispMgrArgs->outline;
    int				x, y;

    x = dispMgrArgs->displayManager->VirtualCoord( LOWORD( lParam)); 
    y = dispMgrArgs->displayManager->VirtualCoord( HIWORD( lParam));
    outline->SetCurrPos( x, y);
}

/***************************************************************************
 *
 * Description:
 *	pan release handler...update the coordinates, refresh object.
 * Arguments:
 *	iMsg - WM_LBUTTONUP.
 *
 ***************************************************************************/

static void 
panReleaseHandler( HWND			hWnd,
		   UINT			iMsg,
		   WPARAM		wParam,
		   LPARAM		lParam,
		   AmDispMgrArgsPtr	dispMgrArgs)
{
    AmMapManagerClass		  *displayManager = dispMgrArgs->displayManager;
    AmDragLineOutlineClass        *outline = ( AmDragLineOutlineClass *)
						dispMgrArgs->outline;
    AmCoordClass			*coords = dispMgrArgs->coords;
    float			startXpos, startYpos, currXpos, currYpos,
    				currXmin, currYmin, currXmax, currYmax,
				panX, panY, xmin, ymin, xmax, ymax;
    int                 	x, y, startX, startY, currX, currY;

/* project the coordinates to virtual space, get the start and end pan
   coordinates */

    x = displayManager->VirtualCoord( LOWORD( lParam)); 
    y = displayManager->VirtualCoord( HIWORD( lParam));
    outline->GetStartPos( &startX, &startY);
    outline->GetCurrPos( &currX, &currY);

/* transform to the object's coordinate system and figure out how much
   we've panned */

    coords->GetDataPt( startX, startY, &startXpos, &startYpos);
    coords->GetDataPt( currX, currY, &currXpos, &currYpos);
    coords->GetCoords( &currXmin, &currYmin, &currXmax, &currYmax);
    panX = currXpos - startXpos;
    panY = currYpos - startYpos;

/* execute the pan, but make sure it's within the bounds of the plot, 
   and flush the display */

    coords->GetRange( &xmin, &ymin, &xmax, &ymax);
    if ( currXmin - panX < xmin) {
    	panX = currXmin - xmin;
    }
    if ( currYmin - panY < ymin) {
    	panY = currYmin - ymin;
    }
    if ( currXmax - panX > xmax) {
    	panX = currXmax - xmax;
    }
    if ( currYmax - panY > ymax) {
    	panY = currYmax - ymax;
    }
    coords->SetCoords( currXmin - panX, currYmin - panY,
		       currXmax - panX, currYmax - panY);
    displayManager->HideOutline( outline);
    displayManager->Flush();

/* and remove the handlers */

    AmDeleteHandler( hWnd, WM_MOUSEMOVE, 0,
		     ( AmNtHandlerFn) panDragHandler, dispMgrArgs);
    AmDeleteHandler( hWnd, WM_LBUTTONUP, 0,
		     ( AmNtHandlerFn) panReleaseHandler, dispMgrArgs);
}

/***************************************************************************
 *
 * Description:
 * Arguments:
 *	hWnd: window the object is displayed on, and the handler is 
 *	activated for.
 *	iMsg: either WM_RBUTTONDOWN, WM_LBUTTONDOWN, or WM_MOUSEMOVE
 *	wParam: contains the mouse x,y position.
 *	dispMgrArgs: contstructed by the callback, contains the display
 *	manager, the screen object and the outline to be dragged.
 *
 ***************************************************************************/

static void 
panButtonHandler( HWND			hWnd,
		  UINT			iMsg,
		  WPARAM		wParam,
		  LPARAM		lParam,
		  AmDispMgrArgsPtr	dispMgrArgs)
{
    AmCoordClass		  *coords = dispMgrArgs->coords;
    AmDragLineOutlineClass        *outline = ( AmDragLineOutlineClass *)
						dispMgrArgs->outline;
    int                 	  x, y;

/* remove the event handler on right button press */

    if ( iMsg == WM_RBUTTONDOWN) {
	AmRemoveHandler( hWnd, panEvents, VECLEN( panEvents),
		       ( AmNtHandlerFn) panButtonHandler, dispMgrArgs);
	dispMgrArgs->displayManager->WriteResourceMessage( "panCancelMsg");
	return;
    } else {
	x = dispMgrArgs->displayManager->VirtualCoord( LOWORD( lParam)); 
	y = dispMgrArgs->displayManager->VirtualCoord( HIWORD( lParam));

/* if it's the left button down, then set the starting pan coordinates */

    	outline->SetLine( x, y, x, y);
	dispMgrArgs->displayManager->ShowOutline( outline);
	AmInsertHandler( hWnd, WM_MOUSEMOVE, 0,
		         ( AmNtHandlerFn) panDragHandler, dispMgrArgs);
	AmInsertHandler( hWnd, WM_LBUTTONUP, 0,
		         ( AmNtHandlerFn) panReleaseHandler, dispMgrArgs);
    } 
}

/***************************************************************************
 *
 * Description:
 *	Pan callback to allow the user to pan the passed coordinate
 *	system. left button starts the pan, mouse move drags the outline
 *	cursor, button release currs the pan, and third button turns
 *	off the pan handler.
 * Arguments:
 *	hWnd: The control that launches the callback (there are problems
 *	with this...)
 *	iMsg, wParam, lParam: the message id that launched the zoom callback.
 *	displayManager: display manager the object is displayed on.
 *	coords: The object to pan.
 *
 ***************************************************************************/

void AmPanCallback( HWND			hWnd,
		  UINT			iMsg,
		  WPARAM		wParam,
		  LPARAM		lParam,
		  AmNTDisplayManagerClass	*displayManager,
		  AmCoordClass		*coords)
{
    AmDragLineOutlineClass        *outline;
    AmNTWindowClass		*window = ( AmNTWindowClass *) 
					displayManager->GetWindow();
    UINT			msgCode;
    
    displayManager->WriteResourceMessage( "panInstructionMsg");
    outline = new AmDragLineOutlineClass( displayManager, 0, 0);
    displayManager->AddOutline( outline);
    displayManager->HideOutline( outline);

/* if the pan is launched by a button or menu, then we pass the wParam
   parameter from the WM_COMMAND message */

    msgCode = ( iMsg == WM_COMMAND) ? wParam : iMsg;
    AmAddHandler( hWnd, msgCode, window->getHWND(),
    		panEvents, VECLEN( panEvents),
		( AmNtHandlerFn) panButtonHandler,
	        AmMakeDispMgrArgs( displayManager, coords, outline), 
		PAN_PRIORITY,
		( AmStateHandlerFn) removePanHandler,
		( AmStateHandlerFn) suspendPanHandler,
		( AmStateHandlerFn) restartPanHandler);
}

