#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <windows.h>
#include <commdlg.h>
#include <anvil/resource.h>
#include <anvil/plot_handlers.h>
#include <anvil/template.h>
#include <anvil/nt.h>
#include <anvil/layerstrs.h>
#include <anvil/log.h>
#include <resources/resource.h>

void 
setupOpenFileName( HWND		hWnd,
		   OPENFILENAME	*ofn,
		   char		*fileBuffer,
		   int		bufferLen)
{
    const char		*szFilter;

    szFilter = AmGetDefault( "addLayer.filter", 
		   "ANVIL Headers (*.hdr)\0*.hdr\0All Files (*.*)\0*.*\0\0");
    ofn->lStructSize = sizeof( OPENFILENAME);
    ofn->hwndOwner = hWnd;
    ofn->hInstance = NULL;
    ofn->lpstrFilter = szFilter;
    ofn->lpstrCustomFilter = NULL;
    ofn->nMaxCustFilter = 0;
    ofn->nFilterIndex = 0;
    ofn->nMaxFile = bufferLen;
    ofn->lpstrFile = fileBuffer;

/* Pointer to a buffer that contains a filename used to initialize the 
   File Name edit control.   The AmFirst character of this buffer must be 
   NULL if initialization is not necessary. */

    ofn->lpstrFile[ 0] = '\0';
    ofn->lpstrFileTitle = NULL;
    ofn->nMaxFileTitle = 0;
    ofn->lpstrInitialDir = NULL;
    ofn->lpstrTitle = AmGetDefault( "addLayer.title", "Add Layer");
    ofn->Flags = 0;
    ofn->nFileOffset = 0;
    ofn->nFileExtension = 0;
    ofn->lpstrDefExt = "hdr";
    ofn->lCustData = NULL;
    ofn->lpfnHook = NULL;
    ofn->lpTemplateName = NULL;
}
  

/*******************************************************************
 *
 * Description:
 *	callback to add a layer to an overlay plot. Flushes the
 *	display when the overlay has been added.
 * Arguments:
 *	hWnd - window receiving callback message.
 *	iMsg, wParam, lParam - windows message.
 *	displayManager - display manager containing the overlay to
 *	which the object is to be added.
 *	overlay - overlay to add layers to.
 *	screenObjCallback - function to call when the layer is added 
 *	clientData - additional parameter to pass to the function.
 *
 ********************************************************************/

void 
AddLayerAndCallbackCallback( HWND			hWnd,
			     UINT			iMsg,
			     WPARAM			wParam,
			     LPARAM			lParam,
			     AmNTDisplayManagerClass	*displayManager,
			     AmOverlayClass		*overlay,
			     AmScreenObjCallbackFn	screenObjCallback,
			     void			*clientData)
{
    OPENFILENAME	ofn;
    AmScreenObjClass	*screenObj;
    char		fileBuffer[ PATHLEN];

    setupOpenFileName( hWnd, &ofn, fileBuffer, sizeof( fileBuffer));

/* if the user hit "OK" in the dialog, try to load the header */

    if ( GetOpenFileName( &ofn)) {
	AmInitLog();
	if ( AmAddLayerFromHeader( displayManager->GetWindow(),
				 ofn.lpstrFile, overlay, AM_PREFS_DIR,
				 &screenObj) != AmOK) {
	     AmPrintLogError( hWnd, "addLayerFailed", ofn.lpstrFile);
	} else {
	    displayManager->Flush();
	    ( screenObjCallback)( displayManager, screenObj, clientData);
	}
    }
}

/*******************************************************************
 * Description:
 *	callback to add a layer to an overlay plot. Flushes the
 *	display when the overlay has been added.
 * Arguments:
 *	hWnd - window receiving callback message.
 *	iMsg, wParam, lParam - windows message.
 *	displayManager - display manager containing the overlay to
 *	which the object is to be added.
 *	overlay - overlay to add layers to.
 ********************************************************************/

void AmAddLayerCallback( HWND			hWnd,
		       UINT			iMsg,
		       WPARAM			wParam,
		       LPARAM			lParam,
		       AmNTDisplayManagerClass	*displayManager,
		       AmOverlayClass		*overlay)
{
    OPENFILENAME	ofn;
    AmScreenObjClass	*screenObj;
    char		fileBuffer[ PATHLEN];

    setupOpenFileName( hWnd, &ofn, fileBuffer, sizeof( fileBuffer));

/* if the user hit "OK" in the dialog, try to load the header */

    if ( GetOpenFileName( &ofn)) {
	AmInitLog();
	if ( AmAddLayerFromHeader( displayManager->GetWindow(),
				 ofn.lpstrFile, overlay, AM_PREFS_DIR,
				 &screenObj) != AmOK) {
	     AmPrintLogError( hWnd, "addLayerFailed", ofn.lpstrFile);
	} else {
	    displayManager->Flush();
	}
    }
}

/*******************************************************************
 * Description:
 *	callback to delete a layer from an overlay plot. Flushes the
 *	display when the overlay has been removed.
 * Arguments:
 *	hWnd - window receiving callback message.
 *	iMsg, wParam, lParam - windows message.
 *	displayManager - display manager containing the overlay to
 *	which the object is to be deleted.
 *	overlay - overlay to remove layers from.
 ********************************************************************/

static AmOverlayClass		*_deleteLayerOverlay;
static AmNTDisplayManagerClass	*_deleteLayerDisplayManager;

static BOOL CALLBACK deleteLayerDialogProc( HWND	hDlg,
					    UINT	iMsg,
					    WPARAM	wParam,
					    LPARAM	lParam)
{
    char		**layerStrings, **ptr;
    HWND		hWndList;
    INT			*selItems;
    int	    		numLayers, ii, cItems;
    AmDataGraphicClass	**layerList;

    switch ( iMsg) {
    case WM_INITDIALOG:

/* insert the layer strings into the list box from the overlay's layer list */

	layerStrings = AmLayerStringList( _deleteLayerOverlay, FALSE);
	if ( layerStrings == NULL) {
	    AmPrintResourceError( hDlg, "noLayersToDelete");
	    EndDialog( hDlg, 0);
	    return( FALSE);
	}
	hWndList = GetDlgItem( hDlg, DELETE_LISTBOX);
	for ( ptr = layerStrings, ii = 0; *ptr != NULL; ptr++, ii++) {
	    SendMessage( hWndList, LB_INSERTSTRING, 
			 ( WPARAM) ii, ( LPARAM) *ptr);
	}
	return( FALSE);
    case WM_COMMAND:
    	switch ( LOWORD( wParam)) {
	case IDOK:
	    hWndList = GetDlgItem( hDlg, DELETE_LISTBOX);

/* get the selected indexes from the list and the layers from the overlay */

	    cItems = SendMessage( hWndList, LB_GETSELCOUNT, 
				  ( WPARAM) 0, ( LPARAM) 0);
	    if ( cItems > 0) {
		selItems = ( LPINT) malloc( sizeof( INT)*cItems);
		SendMessage( hWndList, LB_GETSELITEMS, 
			     ( WPARAM) cItems, ( LPARAM) selItems);
		_deleteLayerOverlay->GetLayers( &layerList, &numLayers);

/* check to make sure the index is reasonable, then remove the layers 
   from the overlay and flush the display */

		for ( ii = 0; ii < cItems; ii++) {
		    if ( selItems[ ii] < numLayers) {
			_deleteLayerOverlay->RemoveLayer( 
						layerList[ selItems[ ii]]);
		    } else {
			AmPrintResourceError( hDlg, "badListIndex", 
					    selItems[ ii], numLayers);
		    }
		}
		 _deleteLayerDisplayManager->Flush();
		free( layerList);
		free( selItems);
	    }
	case IDCANCEL:
	    EndDialog( hDlg, 0);
	    return( TRUE);
	}
	break;
    }
    return( FALSE);
}

void AmDeleteLayerCallback( HWND			hWnd,
			  UINT			iMsg,
			  WPARAM		wParam,
			  LPARAM		lParam,
			  AmNTDisplayManagerClass	*displayManager,
			  AmOverlayClass		*overlay)
{
    int		errCode;
    HINSTANCE	hInstance;

    _deleteLayerDisplayManager = displayManager;
    _deleteLayerOverlay = overlay;
    hInstance = ( HINSTANCE) GetWindowLong( hWnd, GWL_HINSTANCE);
    errCode = DialogBox( hInstance,  MAKEINTRESOURCE(DELETE_LAYER_DLOG),
			 hWnd, ( DLGPROC) deleteLayerDialogProc);
    if ( errCode == -1) {
	AmPrintResourceError( hWnd, "DeleteLayerDialogFailed");
    }
}

