#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <windows.h>
#include <anvil/resource.h>
#include <anvil/plot_handlers.h>
#include <anvil/template.h>
#include <anvil/nt.h>
#include <anvil/log.h>
#include <anvil/files.h>
#include <anvil/nt_user_data.h>
#include <anvil/nt_textcheck.h>
#include <anvil/macros.h>
#include <anvil/vector.h>
#include <anvil/tick.h>
#include <anvil/linegraph.h>
#include <anvil/dispatch.h>
#include <resources/resource.h>

static AmNTDisplayManagerClass	*staticDisplayManager = NULL;
static AmPlotClass		*staticPlot = NULL;

static BOOL CALLBACK
rangeDialogProc( HWND	hDlg,
		 UINT	iMsg,
		 WPARAM	wParam,
		 LPARAM	lParam)
{
    char			relPath[ PATHLEN], *mapListFile, **mapNameList,
				**ptr;
    HWND			hWndList, hWndRange, hMinText, hMaxText,
				hNonLinearTgl;
    AmNTDisplayClass		*pNTDisplay;
    AmNTWindowClass		*pNTWindow;
    AmNTDisplayManagerClass	*rangeDisplayManager;
    RECT			rangeRect;
    AmPlotClass			*plot = staticPlot;
    int				width, height, numColors, fNonLinear;
    float			*minmaxMin, *minmaxMax;
    float			min, max, *binvec;
    unitType			units, xunits;
    AmLineGraphClass		*scaleGraph;
    AmTickClass			*hTicks, *vTicks;

    switch ( iMsg) {
    case WM_INITDIALOG:

/* read the list of colormaps into the selection list */

	sprintf( relPath, "%s/%s", AM_BINVEC_DIR, AM_MAP_LIST_FILE);
	mapListFile = AmGetEnvFile( CONFIG_ENV, relPath);
	if ( mapListFile == NULL) {
	    AmPrintResourceError( hDlg, "cantOpenMapFile");
	    return( FALSE);
	}
	mapNameList = AmReadLines( mapListFile);
	if ( mapNameList == NULL) {
	    AmPrintResourceError( hDlg, "cantReadMapFile");
	    return( FALSE);
	}
	hWndList = GetDlgItem( hDlg, BINVEC_LIST);
	for ( ptr = mapNameList; *ptr != NULL; ptr++) {
	    SendMessage( hWndList, LB_ADDSTRING, 0, ( LPARAM) *ptr);
	}

/* extract the values from the plot */

	plot->GetVaFields( AmNcolorsMin, &min,
			   AmNcolorsMax, &max,
			   AmNcolorsUnits, &units,
			   AmNcolorsBinvec, &binvec,
			   AmNnumColors, &numColors,
			   NULL);

/* set up the text entry fields for the minimum and maximum */

	minmaxMin = ( float *) malloc( sizeof( float)*2);
	minmaxMax = ( float *) malloc( sizeof( float)*2);
	minmaxMin[ 0] = -MY_HUGE;
	minmaxMin[ 1] = max;
	minmaxMax[ 0] = min;
	minmaxMax[ 1] = MY_HUGE;
	hMinText = GetDlgItem( hDlg, MIN_EDIT);
	hMaxText = GetDlgItem( hDlg, MAX_EDIT);
	AmUnitVarEntry( hMinText, &min, AmFLOAT, units, minmaxMin);
	AmUnitVarEntry( hMaxText, &max, AmFLOAT, units, minmaxMax);
	hNonLinearTgl = GetDlgItem( hDlg, TGL_NONLINEAR);
	SendMessage( hNonLinearTgl, BM_SETCHECK, 
		     ( WPARAM) ( binvec != NULL), 0);

/* create a graph to show the values for scaling */

	if ( binvec == NULL) {
	    binvec = AmMakeRangeVector( min, max, numColors - 1);
	}
	AmGetUnitCode( "colors", &xunits);

/* create the display manager to display a range graph */

	hWndRange = GetDlgItem( hDlg, RANGE_AREA);
	SetWindowLong( hWndRange, GWL_WNDPROC, ( long) AmDispatchEvent);
	pNTDisplay = new AmNTDisplayClass();
	pNTWindow = new AmNTWindowClass( pNTDisplay, hWndRange);
	rangeDisplayManager = 
	    new AmNTDisplayManagerClass( pNTDisplay, pNTWindow, FALSE);
	plot = staticPlot;
	GetWindowRect( hWndRange, &rangeRect);
	height = rangeRect.bottom - rangeRect.top;
	width = rangeRect.right - rangeRect.left;

/* create a graph and tickmarks to show the current range */

	scaleGraph = new AmLineGraphClass( pNTWindow, "scaleGraph",
					 AmNleftMargin, 80,
					 AmNbottomMargin, 40,
					 AmNtopMargin, 20,
					 AmNrightMargin, 20,
					 AmNdata, binvec,
					 AmNrows, numColors - 1,
					 AmNheightFn, AmFloatFn,
					 AmNheightUnits, units,
					 AmNheightType, AmFLOAT,
					 AmNsize, sizeof( float),
					 AmNyunits, units,
					 AmNxunits, xunits,
					 AmNxmin, 0.0,
					 AmNxmax, ( float) numColors,
					 NULL);
	hTicks = new AmTickClass( pNTWindow, scaleGraph, "scaleHticks",
				AmNtickPosition, AmBOTTOM,
				AmNtickStyle, AmBASE_TEN,
				AmNtextVertical, FALSE,
				AmNtitle, "colors",
				AmNtitleOffset, 40,
				AmNtitleVertical, FALSE,
				AmNleftMargin, 80,
				AmNrightMargin, 20,
				AmNbottomMargin, 0,
				AmNheight, 40,
				AmNtitleOffset, 30,
				AmNresizeFont, FALSE,
				NULL);
	vTicks = new AmTickClass( pNTWindow, scaleGraph, "scaleVticks",
				AmNtickPosition, AmLEFT,
				AmNtickStyle, AmBASE_TEN,
				AmNtextVertical, FALSE,
				AmNtitleOffset, 10,
				AmNleftMargin, 0,
				AmNwidth, 80,
				AmNtopMargin, 20,
				AmNbottomMargin, 40,
				AmNtitleOffset, 30,
				AmNresizeFont, FALSE,
				NULL);
	rangeDisplayManager->AddObject( scaleGraph);
	rangeDisplayManager->AddObject( hTicks);
	rangeDisplayManager->AddObject( vTicks);
	rangeDisplayManager->Flush();
    	return( FALSE);
    case WM_COMMAND:
	switch ( LOWORD( wParam)) {
	case IDOK:
	case IDCANCEL:
	    EndDialog( hDlg, 0);
	    return( TRUE);
	case TGL_NONLINEAR:
	    hWndList = GetDlgItem( hDlg, BINVEC_LIST);
	    hNonLinearTgl = GetDlgItem( hDlg, TGL_NONLINEAR);
	    fNonLinear = SendMessage( hNonLinearTgl, BM_GETSTATE,
	    			      ( WPARAM) 0, ( LPARAM) 0);
	    SendMessage( hWndList, WM_ENABLE, 
	    		 ( WPARAM) ( fNonLinear == BST_CHECKED), 0);
	    break;
	}
	break;
    case WM_DESTROY:
    	EndDialog( hDlg, 0);
	return( TRUE);
    }
    return( FALSE);
}

void AmSetRangeCallback( HWND				hWnd,
		         UINT				iMsg,
		         WPARAM				wParam,
		         LPARAM				lParam,
		         AmNTDisplayManagerClass	*displayManager,
		         AmPlotClass			*plot)

{
    HINSTANCE	hInstance;
    int		errCode;
    HWND	hDlg;

    staticDisplayManager = displayManager;
    staticPlot = plot;
    hInstance = ( HINSTANCE) GetWindowLong( hWnd, GWL_HINSTANCE);
    hDlg = CreateDialog( hInstance, MAKEINTRESOURCE(RANGE_DIALOG),
			 hWnd, ( DLGPROC) rangeDialogProc);
    if ( hDlg == NULL) {
    	AmPrintResourceError( hWnd, "RangeDialogFailed");
    }
    AmRegisterDialog( hDlg);
    ShowWindow( hDlg, SW_SHOW);
}
