#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/fns.h>
#include <anvil/list.h>
#include <anvil/screenobj.h>
#include <anvil/overlay.h>
#include <anvil/copyobj.h>
#include <anvil/template.h>

/* functions for copying screen objects. */

/****************************************************************************
 *
 * Description:
 *	copy an object.  The parent is used for recursive calls, and the
 *	window is needed for object initialization/construction.
 * Arguments:
 *	window - window to create the new object on.
 *	obj - object to copy.
 *	parent - parent for the new object (may be NULL).  
 *	If the copied object is an overlay, then the created overlay
 *	is used as the new object's parent.
 * Returns:
 *	The newly created object, or NULL if it couldn't be created.
 *
 ****************************************************************************/

AmScreenObjClass *AmCopyObject( AmWindowClass		*window,
			    AmScreenObjClass	*obj,
			    AmScreenObjClass	*parent)
{
    AmTagValueRec		tagValues[ MAXTAGS];
    int			numValues, numPlots, ii;
    char		name[ TEXTLEN];
    AmDataGraphicClass	**layerList, *layer;
    AmOverlayClass	*overlay;
    
    strcpy( name, obj->GetName());
    obj->RetreiveTagValues( tagValues, &numValues);
    AmChangeGetToSet( tagValues, numValues);
    if ( obj->GetObjectType() == AmOVERLAY) {
	(( AmOverlayClass *) obj)->GetLayers( &layerList, &numPlots);
	overlay = ( AmOverlayClass *) 
	    AmCreateObjectFromType( window, parent, name, obj->GetObjectType(), 
				  tagValues, numValues);
	for ( ii = 0; ii < numPlots; ii++) {
	    layer = ( AmDataGraphicClass *) AmCopyObject( window, layerList[ ii],
						      overlay);
	    overlay->AddLayer( layer);
	}
	return( overlay);
    } else {
	return( AmCreateObjectFromType( window, parent, name, 
				      obj->GetObjectType(), 
				      tagValues, numValues));
    }
}

/****************************************************************************
 *
 * Description:
 * 	copy from one object to another, but keep a pointer to the data if the
 *  	sourceType is AmFILE_SOURCE.  
 * Arguments:
 *	window - window to create the new object on.
 *	obj - object to copy.
 *	parent - parent for the new object (may be NULL).  
 *	If the copied object is an overlay, then the created overlay
 *	is used as the new object's parent.
 * Returns:
 *	The newly created object, or NULL if it couldn't be created.
 * Modifications:
 *	This function is obsolete, since the filenames and data are cached.
 *	Eventually, it will be removed.
 *
 ****************************************************************************/

AmScreenObjClass *AmTraceObject( AmWindowClass	*window,
			     AmScreenObjClass	*obj,
			     AmScreenObjClass	*parent)
{
    AmTagValueRec		tagValues[ MAXTAGS];
    int			numValues, numLayers, ii;
    char		name[ TEXTLEN];
    AmDataGraphicClass	**layerList, *layer, *newObj;
    AmOverlayClass	*overlay;
    
    strcpy( name, obj->GetName());

/* get the tag values from the object, and change the values from set to get,
   then make specific changes for tracing objects (shared = true) */

    obj->RetreiveTagValues( tagValues, &numValues);
    AmChangeGetToSet( tagValues, numValues);

/* if the plot is an overlay plot, then create the overlay plot, and trace
   each of the individual plots */

    if ( obj->IsType( AmOVERLAY)) {
	(( AmOverlayClass *) obj)->GetLayers( &layerList, &numLayers);
	overlay = ( AmOverlayClass *) 
	    AmCreateObjectFromType( window, parent, 
				  ( char *) obj->getObjectName(),
				  obj->GetObjectType(),
				  tagValues, numValues);
	if ( overlay != NULL) {
	    for ( ii = 0; ii < numLayers; ii++) {
		layer = ( AmDataGraphicClass *) 
		    AmTraceObject( window, layerList[ ii], overlay);
		layer->SetShared( TRUE);
	    }
	    free( layerList);
	}
	return( overlay);
    } else {
	newObj = ( AmDataGraphicClass *)
	    AmCreateObjectFromType( window, parent, 
				  ( char *) obj->getObjectName(),
				  obj->GetObjectType(), tagValues, numValues);
	if ( newObj != NULL) {
	    newObj->SetShared( TRUE);
	}
	return( newObj);
    }
}
