#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/macros.h>
#include <anvil/screenobj.h>
#include <anvil/plot.h>
#include <anvil/cadplot.h>
#include <anvil/imageplot.h>
#include <anvil/lineplot.h>
#include <anvil/plineplot.h>
#include <anvil/scatterplot.h>
#include <anvil/scattergraph.h>
#include <anvil/bargraph.h>
#include <anvil/linegraph.h>
#include <anvil/multitextannot.h>
#include <anvil/polygonplot.h>
#include <anvil/objectplot.h>
#include <anvil/graph.h>
#include <anvil/overlay.h>
#include <anvil/imagelegend.h>
#include <anvil/linelegend.h>
#include <anvil/polylegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/grid.h>
#include <anvil/annotation.h>
#include <anvil/circleannot.h>
#include <anvil/rectangleannot.h>
#include <anvil/textannot.h>
#include <anvil/lineannot.h>
#include <anvil/imageannot.h>
#include <anvil/distscale.h>
#include <anvil/createobject.h>

/* wrapper function to create an overlay plot/graph */

AmScreenObjClass *AmMakeOverlay( AmWindowClass	*window,
			     AmScreenObjClass     *,
			     char		*name,
			     AmTagValuePtr	tagValues,
			     int		numValues)
{
    return( new AmOverlayClass( window, name, 
			      AmNnumTagValues, numValues,
			      AmNtagValues, tagValues, NULL));
}

/* wrapper function to create an image plot */
		    
AmScreenObjClass *AmMakeImagePlot( AmWindowClass	*window,
			       AmScreenObjClass	*parent,
			       char		*name,
			       AmTagValuePtr	tagValues,
			       int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmImagePlotClass(( AmOverlayClass *) parent, name,
				       AmNnumTagValues, numValues,
				       AmNtagValues, tagValues, 
				       NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmImagePlotClass( window, name,
				    AmNnumTagValues, numValues,
				    AmNtagValues, tagValues, 
				    NULL));
    }
}

/* wrapper function to create a CAD plot */

AmScreenObjClass *AmMakeCadPlot( AmWindowClass	*window,
			     AmScreenObjClass	*parent,
			     char		*name,
			     AmTagValuePtr	tagValues,
			     int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmCadPlotClass(( AmOverlayClass *) parent, name,
				     AmNnumTagValues, numValues,
				     AmNtagValues, tagValues, 
				     NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmCadPlotClass( window, name,
				  AmNnumTagValues, numValues,
				  AmNtagValues, tagValues, 
				  NULL));
    }
}

/* wrapper function to create a line plot */
		    
AmScreenObjClass *AmMakeLinePlot( AmWindowClass	*window,
			      AmScreenObjClass	*parent,
			      char		*name,
			      AmTagValuePtr	tagValues,
			      int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmLinePlotClass(( AmOverlayClass *) parent, name,
				      AmNnumTagValues, numValues,
				      AmNtagValues, tagValues, 
				      NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmLinePlotClass( window, name,
				   AmNnumTagValues, numValues,
				   AmNtagValues, tagValues, 
				   NULL));
    }
}

/* wrapper function to create a polyline plot */
		    
AmScreenObjClass *AmMakePolyLinePlot( AmWindowClass	*window,
				      AmScreenObjClass	*parent,
				      char		*name,
				      AmTagValuePtr	tagValues,
				      int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmPolyLinePlotClass(( AmOverlayClass *) parent, name,
					    AmNnumTagValues, numValues,
					    AmNtagValues, tagValues, 
					    NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmPolyLinePlotClass( window, name,
				         AmNnumTagValues, numValues,
				         AmNtagValues, tagValues, 
				         NULL));
    }
}

/* wrapper function to create a polygon plot */
		    
AmScreenObjClass *AmMakePolygonPlot( AmWindowClass	*window,
				 AmScreenObjClass	*parent,
				 char		*name,
				 AmTagValuePtr	tagValues,
				 int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmPolygonPlotClass(( AmOverlayClass *) parent, name,
					 AmNnumTagValues, numValues,
					 AmNtagValues, tagValues, 
					 NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmPolygonPlotClass( window, name,
				      AmNnumTagValues, numValues,
				      AmNtagValues, tagValues, 
				      NULL));
    }
}

/* wrapper function to create a scatter plot */
		    
AmScreenObjClass *AmMakeScatterPlot( AmWindowClass	*window,
				 AmScreenObjClass	*parent,
				 char		*name,
				 AmTagValuePtr	tagValues,
				 int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmScatterPlotClass(( AmOverlayClass *) parent, name,
					 AmNnumTagValues, numValues,
					 AmNtagValues, tagValues, 
					 NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmScatterPlotClass( window, name,
				      AmNnumTagValues, numValues,
				      AmNtagValues, tagValues, 
				      NULL));
    }
}

/* wrapper function to create a scatter graph */
		    
AmScreenObjClass *AmMakeScatterGraph( AmWindowClass	*window,
				  AmScreenObjClass	*parent,
				  char			*name,
				  AmTagValuePtr	tagValues,
				  int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmScatterGraphClass(( AmOverlayClass *) parent, name,
					  AmNnumTagValues, numValues,
					  AmNtagValues, tagValues, 
					  NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmScatterGraphClass( window, name,
				       AmNnumTagValues, numValues,
				       AmNtagValues, tagValues, 
				       NULL));
    }
}

/* wrapper function to create a bar graph */
		    
AmScreenObjClass *AmMakeBarGraph( AmWindowClass	*window,
			      AmScreenObjClass	*parent,
			      char		*name,
			      AmTagValuePtr	tagValues,
			      int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmBarGraphClass(( AmOverlayClass *) parent, name,
				      AmNnumTagValues, numValues,
				      AmNtagValues, tagValues, 
				      NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmBarGraphClass( window, name,
				   AmNnumTagValues, numValues,
				   AmNtagValues, tagValues, 
				   NULL));
    }
}

/* wrapper function to create a line graph */

AmScreenObjClass *AmMakeLineGraph( AmWindowClass	*window,
			       AmScreenObjClass	*parent,
			       char		*name,
			       AmTagValuePtr	tagValues,
			       int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmLineGraphClass(( AmOverlayClass *) parent, name,
				       AmNnumTagValues, numValues,
				       AmNtagValues, tagValues, 
				       NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmLineGraphClass( window, name,
				    AmNnumTagValues, numValues,
				    AmNtagValues, tagValues, 
				    NULL));
    }
}

/* wrapper function to create an object plot */

AmScreenObjClass *AmMakeObjectPlot( AmWindowClass	*window,
			        AmScreenObjClass	*parent,
			        char		*name,
			        AmTagValuePtr	tagValues,
				int		numValues)
{
    if ( parent != NULL) {
	if ( parent->IsType( AmOVERLAY)) {
	    return( new AmObjectPlotClass(( AmOverlayClass *) parent, name,
				        AmNnumTagValues, numValues,
				        AmNtagValues, tagValues, 
					NULL));
	} else {
	    return( NULL);
	}
    } else {
	return( new AmObjectPlotClass( window, name,
				     AmNnumTagValues, numValues,
				     AmNtagValues, tagValues, 
				     NULL));
    }
}

/* wrapper function to construct a circle annotation */
		    
AmScreenObjClass *AmMakeCircleAnnot( AmWindowClass	*window,
				 AmScreenObjClass *,
				 char		*name,
				 AmTagValuePtr	tagValues,
				 int		numValues)
{
    return( new AmCircleAnnotClass( window, name,
				  AmNnumTagValues, numValues,
				  AmNtagValues, tagValues, 
				  NULL));
}

/* wrapper function to construct a rectangle annotation */
		    
AmScreenObjClass *AmMakeRectangleAnnot( AmWindowClass	*window,
				    AmScreenObjClass 	*,
				    char		*name,
				    AmTagValuePtr	tagValues,
				    int		numValues)
{
    return( new AmRectangleAnnotClass( window, name,
				     AmNnumTagValues, numValues,
				     AmNtagValues, tagValues, 
				     NULL));
}

/* wrapper function to construct a text annotation */
		    
AmScreenObjClass *AmMakeTextAnnot( AmWindowClass	*window,
			       AmScreenObjClass   *,
			       char		*name,
			       AmTagValuePtr	tagValues,
			       int		numValues)
{
    return( new AmTextAnnotClass( window, name,
				AmNnumTagValues, numValues,
				AmNtagValues, tagValues, 
				NULL));
}

/* wrapper function to construct a multiline text annotation */
		    
AmScreenObjClass *AmMakeMultiLineTextAnnot( AmWindowClass	*window,
				        AmScreenObjClass  *,
				        char		*name,
				        AmTagValuePtr	tagValues,
					int		numValues)
{
    return( new AmMultiTextAnnotClass( window, name,
				     AmNnumTagValues, numValues,
				     AmNtagValues, tagValues, 
				     NULL));
}

/* wrapper function to construct an image annotation */
		    
AmScreenObjClass *AmMakeImageAnnot( AmWindowClass	*window,
			        AmScreenObjClass  *,
			        char		*name,
			        AmTagValuePtr	tagValues,
				int		numValues)
{
    return( new AmImageAnnotClass( window, name,
			         AmNnumTagValues, numValues,
			         AmNtagValues, tagValues, 
				 NULL));
}

/* wrapper function to construct a line annotation */

AmScreenObjClass *AmMakeLineAnnot( AmWindowClass	*window,
			       AmScreenObjClass  *,
			       char		*name,
			       AmTagValuePtr	tagValues,
			       int		numValues)
{
    return( new AmLineAnnotClass( window, name,
			        AmNnumTagValues, numValues,
			        AmNtagValues, tagValues, 
				NULL));
}
		    
/* wrapper function to construct an image legend */

AmScreenObjClass *AmMakeImageLegend( AmWindowClass	*window,
			         AmScreenObjClass	*parent,
			         char		*name,
			         AmTagValuePtr	tagValues,
				 int		numValues)
{
    if ( parent->IsType( AmIMAGE_PLOT)) {
	return( new AmImageLegendClass( window, ( AmImagePlotClass *) parent,
				      name,
				      AmNnumTagValues, numValues,
				      AmNtagValues, tagValues, 
				      NULL));
    } else {
	return( NULL);
    }
}

/* wrapper function to construct a line legend */
		    
AmScreenObjClass *AmMakeLineLegend( AmWindowClass	*window,
			        AmScreenObjClass	*parent,
			        char		*name,
			        AmTagValuePtr	tagValues,
				int		numValues)
{
    if ( parent->IsType( AmLINE_PLOT)) {
	return( new AmLineLegendClass( window, ( AmLinePlotClass *) parent, 
				     name,
				     AmNnumTagValues, numValues,
				     AmNtagValues, tagValues, 
				     NULL));
    } else {
	return( NULL);
    }
}

/* wrapper function to construct a scatter legend */
		    
AmScreenObjClass *AmMakeScatterLegend( AmWindowClass		*window,
			           AmScreenObjClass	*parent,
				   char			*name,
				   AmTagValuePtr		tagValues,
				   int			numValues)
{
    if ( parent->IsType( AmSCATTER_PLOT)) {
	return( new AmScatterLegendClass( window, ( AmScatterPlotClass *) parent, 
					name,
					AmNnumTagValues, numValues,
					AmNtagValues, tagValues, 
					NULL));
    } else {
	return( NULL);
    }
}

/* wrapper function to construct a polygon legend */
		    
AmScreenObjClass *AmMakePolyLegend( AmWindowClass	*window,
			        AmScreenObjClass	*parent,
			        char		*name,
			        AmTagValuePtr	tagValues,
				int		numValues)
{
    if ( parent->IsType( AmPOLYGON_PLOT)) {
	return( new AmPolyLegendClass( window, ( AmPolygonPlotClass *) parent,
				     name,
				     AmNnumTagValues, numValues,
				     AmNtagValues, tagValues, 
				     NULL));
    } else {
	return( NULL);
    }
}

/* wrapper function to construct a grid */
		    
AmScreenObjClass *AmMakeGrid( AmWindowClass		*window,
			  AmScreenObjClass	*parent,
			  char			*name,
			  AmTagValuePtr		tagValues,
			  int			numValues)
{
    if ( parent->IsType( AmCOORD)) {
	return( new AmGridClass( window, ( AmDataGraphicClass *) parent, name,
			       AmNnumTagValues, numValues,
			       AmNtagValues, tagValues, 
			       NULL));
    } else {
	return( NULL);
    }
}

/* wrapper function to construct tickmarks */
		    
AmScreenObjClass *AmMakeTicks( AmWindowClass		*window,
			   AmScreenObjClass	*parent,
			   char			*name,
			   AmTagValuePtr		tagValues,
			   int			numValues)
{
    if ( parent->IsType( AmCOORD)) {
	return( new AmTickClass( window, ( AmCoordClass *) parent, name,
			       AmNnumTagValues, numValues,
			       AmNtagValues, tagValues, 
			       NULL));
    } else {
	return( NULL);
    }
}
		    
/* wrapper function to construct a distance scale */

AmScreenObjClass *AmMakeDistanceScale( AmWindowClass		*window,
				   AmScreenObjClass	*parent,
				   char			*name,
				   AmTagValuePtr		tagValues,
				   int			numValues)
{
    if ( parent->IsType( AmCOORD)) {
	return( new AmDistanceScaleClass( window, ( AmCoordClass *) parent, name,
				        AmNnumTagValues, numValues,
				        AmNtagValues, tagValues, 
					NULL));
    } else {
	return( NULL);
    }
}
