#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <anvil/datagraphic.h>
#include <anvil/overlay.h>

/* functions to get the strings for layers, to display in selection lists */

/****************************************************************************
 *
 * Description:
 *	Return the name of the layer.  If the layer has a tile, return it,
 *	or if the layer has a filename, return that, otherwise, return
 *	the layer type name.
 * Arguments:
 *	layer to get the name for.
 * Returns:
 *	The layer's name (allocated)
 *
 ****************************************************************************/

char *
AmGetLayerName( AmDataGraphicClass	*layer)
{
    char	*name, buffer[ PATHLEN];

    layer->GetVaFields( AmNtitle, &name, NULL);
    if ( name == NULL) {
	layer->GetVaFields( AmNfilename, &name, NULL);
	if ( name == NULL) {
	    sprintf( buffer, "Layer Type: %s", 
			     layer->getObjectName());
	    return( strdup( buffer));
	}
    }
    return( name);
}

/****************************************************************************
 *
 * Description:
 * 	given an overlay plot, return a vector of strings which show the title
 *  	of the plot (if there is one), or the filename, if there is one, or
 *	a string for the type of plot 
 * Arguments:
 *	overlay - overlay plot containing the layers to get the strings for
 *	showOverlay - control whether to show the overlay as the AmLast string
 *	in the vector of strings.
 * Returns:
 *	An allocated NULL-terminated vector of strings containing the layer
 *	names.  If the overlay is shown, it's the AmLast string in the list.
 *
 ****************************************************************************/

char **
AmLayerStringList( AmOverlayClass	*overlay, 
		 AnvilBoolean	showOverlay)
{
    AmDataGraphicClass	**layerList;
    int			numLayers, ii, length;
    char		**vector;

    if ( overlay->GetLayers( &layerList, &numLayers) != AmOK) {
	return( NULL);
    }
    if ( numLayers == 0) {
	return( NULL);
    }
    if ( showOverlay) {
    	length = numLayers + 2;
    } else {
    	length = numLayers + 1;
    }
    if (( vector = ( char **) malloc( length*sizeof( char **))) == NULL) {
	return( NULL);
    }

/* try the title, and if it's NULL, get the filename */

    for ( ii = 0 ; ii < numLayers; ii++) {
	vector[ ii] = AmGetLayerName( layerList[ ii]);
    }

/* show the overlay if requested */

    if ( showOverlay) {
	vector[ ii++] = "Overlay";
    }
    free( layerList);
    vector[ ii] = NULL;
    return( vector);
}
