%{
#include <stdio.h>
#include <string.h>
#if !defined(_WIN32) && !defined(__BSD44__)
#include <alloca.h>
#endif
#include <malloc.h>
#include <anvil/list.h>
#include <anvil/templateP.h>
#include <anvil/macros.h>

extern AmListPtr templObjectList;

/***************
 * Prototypes: *
 ***************/

int yylex( void);
int yyerror( char *fmt, ...);

/*********************
 * Type Definitions: *
 *********************/

%}

%union {
    char            *string;
    int             integer;
    AmListPtr       list;
    AmTemplDeclPtr  decl;
    AmTemplFieldPtr field;
}

%token <string>	IDENT
%token <string>	VALUE EMPTY_VALUE 

%type <decl>    Decl
%type <list>    FieldList
%type <field>   Field

%start DeclList

%%

DeclList	: Decl
		{
		    if ( !AmTemplAddDecl( $1)) {
		       yyerror( "Attempted redecleration of \"%s\".",
                           $1->name);
		    }
		}
		| DeclList Decl
		{
		    if ( !AmTemplAddDecl( $2)) {
		       yyerror( "Attempted redecleration of \"%s\".",
                           $2->name);
		    }
		}
		;

Decl		: IDENT IDENT '{' FieldList '}'
		{
		    AmObjectTypeEnum type;

		    type = AmFindKeyword( $1);
		    free( $1);
		    $$ = AmTemplCreateDecl( $2, type, NULL, $4);
		}
		| IDENT IDENT '(' IDENT ')' '{' FieldList '}'
		{
		    AmObjectTypeEnum type;

		    type = AmFindKeyword( $1);
		    free( $1);
		    if ( type == AmOVERLAY) {
                        yyerror( "Overlay object \"%s\" can not have a parent.",
		            $2);
		        $$ = AmTemplCreateDecl( $2, type, NULL, $7);
		    } else {
		        $$ = AmTemplCreateDecl( $2, type, $4, $7);
		    }
                }
		;

FieldList	: Field
		{
		    if ( $1 != NULL) {
		        $$ = AmInsertNodeAtEndLst( $1, NULL);
                    } else {
		        $$ = NULL;
		    }
		}
		| FieldList Field
		{
		    if ( $2 != NULL) {
		        $$ = AmInsertNodeAtEndLst( $2, $1);
		    } else {
		        $$ = $1;
		    }
		}
		;

Field		: IDENT ':' '{' Decl '}'
		{
		    $$ = AmTemplCreateChild( $1, $4);
		}
		| IDENT ':' VALUE 
		{
		    $$ = AmTemplCreateField( $1, $3);
		}
		| IDENT ':' 
		{
		    yyerror( "Ignoring empty field: \"%s\".", $1);
		    $$ = NULL;
		}
		;
%%

