#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#if !defined(__BSD44__)
#include <search.h>
#endif
#include <anvil/project.h>
#include <anvil/macros.h>
#include <anvil/resource.h>
#include <anvil/strlib.h>
#include <anvil/tsearch.h>
#include <anvil/tagstrs.h>
#include <anvil/fns.h>
#include <anvil/list.h>
#include <anvil/screenobj.h>
#include <anvil/datagraphic.h>
#include <anvil/overlay.h>
#include <anvil/enums.h>
#include <anvil/log.h>
#include <anvil/template.h>
#include <anvil/templateP.h>

/* yacc */

int templparse( void);
void templrestart( FILE *);

/* Functions to support the template parser, which saves and restores
   object descriptions.  This will likely be modified, because there's
   a separation AmBetween data-based objects (plots and graphs) and non-
   data-based objects, which has been kept around for historical 
   purposes. */

const int      NUMSYMBOLS = 200; 

extern char    *templtext;

int            templLineNo;
int            templNumSymbols;
AmTemplSymbolPtr templSymbolTable[ NUMSYMBOLS];

static AmListPtr templDeclList;
static FILE    *templFp;


/*************************************************************************
 *
 * Description:
 * 	create the initializations for the default set of objects.  This
 *	function should be called at initialization, since it registers
 *	the initialization function to the string object name
 * 
 *************************************************************************/

void 
AmAddDefaultObjectInits()
{
    AmAddObjectInit( "barGraph", AmBAR_GRAPH, AmMakeBarGraph);
    AmAddObjectInit( "cadPlot", AmCAD_PLOT, AmMakeCadPlot);
    AmAddObjectInit( "circleAnnotation", AmCIRCLE_ANNOTATION, AmMakeCircleAnnot);
    AmAddObjectInit( "distanceScale", AmDISTANCE_SCALE, AmMakeDistanceScale);
    AmAddObjectInit( "grid", AmGRID, AmMakeGrid);
    AmAddObjectInit( "image", AmIMAGE_PLOT, AmMakeImagePlot);
    AmAddObjectInit( "imageAnnotation", AmIMAGE_ANNOTATION, AmMakeImageAnnot);
    AmAddObjectInit( "imageLegend", AmIMAGE_LEGEND, AmMakeImageLegend);
    AmAddObjectInit( "imagePlot", AmIMAGE_PLOT, AmMakeImagePlot);
    AmAddObjectInit( "lineAnnotation", AmLINE_ANNOTATION, AmMakeLineAnnot);
    AmAddObjectInit( "lineGraph", AmLINE_GRAPH, AmMakeLineGraph);
    AmAddObjectInit( "lineLegend", AmLINE_LEGEND, AmMakeLineLegend);
    AmAddObjectInit( "linePlot", AmLINE_PLOT, AmMakeLinePlot);
    AmAddObjectInit( "multilineTextAnnotation", AmMULTILINE_TEXT_ANNOTATION, 
		   AmMakeMultiLineTextAnnot);
    AmAddObjectInit( "objectPlot", AmOBJECT_PLOT, AmMakeObjectPlot);
    AmAddObjectInit( "overlay", AmOVERLAY,  AmMakeOverlay);
    AmAddObjectInit( "point ", AmSCATTER_PLOT, AmMakeScatterPlot);
    AmAddObjectInit( "polygon", AmPOLYGON_PLOT, AmMakePolygonPlot);
    AmAddObjectInit( "polygonLegend", AmPOLYGON_LEGEND, AmMakePolyLegend);
    AmAddObjectInit( "polygonPlot", AmPOLYGON_PLOT, AmMakePolygonPlot);
    AmAddObjectInit( "polylinePlot", AmPOLYLINE_PLOT, AmMakePolyLinePlot);
    AmAddObjectInit( "rectangleAnnotation", AmRECTANGLE_ANNOTATION,  
		   AmMakeRectangleAnnot);
    AmAddObjectInit( "scatterGraph", AmSCATTER_GRAPH, AmMakeScatterGraph);
    AmAddObjectInit( "scatterLegend", AmSCATTER_LEGEND, AmMakeScatterLegend);
    AmAddObjectInit( "scatterPlot", AmSCATTER_PLOT, AmMakeScatterPlot);
    AmAddObjectInit( "textAnnotation", AmTEXT_ANNOTATION, AmMakeTextAnnot);
    AmAddObjectInit( "ticks", AmTICKS, AmMakeTicks);
    AmAddObjectInit( "vector", AmLINE_PLOT, AmMakeLinePlot);
};

static void *objectInitNameRoot = NULL;
static void *objectInitEnumRoot = NULL;

AmObjectTypeEnum nonDataCodes[] = {
    AmOVERLAY, AmIMAGE_LEGEND, AmLINE_LEGEND, AmSCATTER_LEGEND, 
    AmPOLYGON_LEGEND, AmTICKS, AmGRID, AmCIRCLE_ANNOTATION, 
    AmRECTANGLE_ANNOTATION, AmTEXT_ANNOTATION, AmIMAGE_ANNOTATION, 
    AmLINE_ANNOTATION, AmMULTILINE_TEXT_ANNOTATION, AmDISTANCE_SCALE
 };


/*************************************************************************
 *
 * Description:
 * 	predicate function to test if a declaration is a data graphic? 
 *	(not a non-data graphic?) 
 * Arguments:
 *	objType - object type to test.
 * Returns:
 *	TRUE if the object is not a non-data graphic screen object.
 * Modifications:
 *  	This method of testing will have to be changed eventually, so
 *	that non-datagraphic objects may be subclassed.
 * 
 *************************************************************************/

static AnvilBoolean 
isDataGraphic( AmObjectTypeEnum	objType)
{
    uint_t	ii;

    for ( ii = 0; ii < VECLEN( nonDataCodes); ii++) {
	if ( objType == nonDataCodes[ ii]) {
	    return( FALSE);
	}
    }

    return( TRUE);
}

/*************************************************************************
 *
 * Description:
 * 	predicate function to search for an object initialization structure 
 *	by the object string name.  Used for adding object initializers to
 *	the object initialization table.
 * Arguments:
 *	a,b - object initialization structure pointers.
 * Returns:
 *	strcmp() of their type names.
 * 
 *************************************************************************/

static int 
byTypeName( AmObjectInitPtr	a,
	    AmObjectInitPtr	b)
{
    return( strcmp( a->typeName, b->typeName));
}

/*************************************************************************
 *
 * Description:
 * 	predicate function to search for an object initialization structure 
 *	by the object type enumeration. 
 * Arguments:
 *	a,b - object initialization structure pointers.
 * Returns:
 *	difference of their type enumerations.
 * 
 *************************************************************************/

static int 
byTypeEnum( AmObjectInitPtr a,
	    AmObjectInitPtr b)
{
    return( a->objectType - b->objectType);
}


/*************************************************************************
 *
 * Description:
 * 	add an object initialization to the list of initializers.  This
 *	registers the object initializer under the object's string and
 *	enumeration names.
 * Arguments:
 *	typeName - unique string name of the object to register.
 *	type - unique object type enumeration.
 *	createFn - object initialization function.
 * Returns:
 *	AmOK on success, AmMALLOC_FAILED if the node couldn't be allocated.
 * 
 *************************************************************************/

AmErrorCode 
AmAddObjectInit( const char	*typeName,
	       AmObjectTypeEnum	type,
	       AmMakeObjectFn	createFn)
{
    AmObjectInitPtr	objectInit;

    if (( objectInit = ( AmObjectInitPtr) 
		malloc( sizeof( AmObjectInitRec))) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    objectInit->typeName = strdup( typeName);
    objectInit->objectType = type;
    objectInit->createFn = createFn;
    tsearch( objectInit, &objectInitNameRoot, ( AmSearchFn) byTypeName);
    tsearch( objectInit, &objectInitEnumRoot, ( AmSearchFn) byTypeEnum);
    return( AmOK);
}

/*************************************************************************
 *
 * Description:
 * 	given the string name of the object, return the object type. 
 * Arguments:
 *	name - name of the object to search for the initalizer.
 * Returns:
 *	the object's enumeration value, from its objectInit structure.
 * 
 *************************************************************************/

objectType 
AmFindKeyword( char	*name)
{
    AmObjectInitRec	target;
    AmObjectInitPtr	*result;

    target.typeName = name;
    result = ( AmObjectInitPtr *)
	tfind( &target, ( const void **) &objectInitNameRoot, 
	       ( AmSearchFn) byTypeName);
    if ( result != NULL){
	return(( *result)->objectType);
    } else {
	return( AmERROR_OBJECT);
    }
}

/*************************************************************************
 *
 * Description:
 * 	return the initialization function for the passed object type 
 * Arguments:
 *	objectType - the object's enumerated type.
 * Returns:
 *	The object initialization structure pointer.
 * 
 *************************************************************************/

static AmObjectInitPtr 
findInit( AmObjectTypeEnum	objectType)
{
    AmObjectInitRec	target;
    AmObjectInitPtr	*result;

    target.objectType = objectType;
    result = ( AmObjectInitPtr *)
	tfind( &target, ( const void **) &objectInitEnumRoot, 
	       ( AmSearchFn) byTypeEnum);
    if ( result != NULL){
	return( *result);
    } else {
	return( NULL);
    }
}
/**********************************************************************
 * 
 * Description:
 *	Given an a set of tag-values and length, and an overlay,
 *	find the appropriate object initializer for the fileType, 
 *	and parse the header strings into an object to add to the overlay.
 * Arguments:
 *	tagValues - tag value pairs to initialize the object with
 *	nTagValues - the number of tag values.
 *	overlay - overlay to add object to.
 *	headerFile - header or string used for the object name
 *	and for error messages.
 *	prefsDir - preferences directory containing files of the form
 *	XXXX.prefs, where XXX is the file source used in the header.
 * Returns:
 *	AmOPEN_FAILED if the file couldn't be read.
 *	AmBAD_ARGUMENT if the AmNtype tag couldn't be found, or if
 *	it was an invalid type.
 *	AmNOT_VALID if the object was not constructed properly.
 *
 **********************************************************************/

AmErrorCode
AmAddLayerFromTagValues( AmWindowClass    *window,
                       AmOverlayClass   *overlay,
                       AmTagValuePtr    tagValues,
                       int            nTagValues,
                       const char     *headerFile,
                       const char     *prefsDir,
                       AmScreenObjClass **ppScreenObj)
{
    char		prefsPath[ PATHLEN], *envPath, *fileType,
			prefsFileType[ PATHLEN];
    const char		*fileSource;
    int			objectCode, ii;
    AmObjectInitPtr	objectInit;
    AnvilBoolean		prefsFlag;


/* See if there's a file source, which we can use to get the preferences
   file. */

    if ( AmGetTagValueFromTags( tagValues, nTagValues, 
			      AmNfileSource, ( void **) &fileSource)) {
	envPath = getenv( CONFIG_ENV);
	sprintf( prefsPath, "%s/%s/%s.prefs", 
		 envPath, prefsDir, fileSource);
	prefsFlag = AmIsHeader( prefsPath, prefsFileType);
    } else {
    	prefsFlag = FALSE;
    }

    if ( prefsFlag) {

    /* Put the default values at the beginning so that they
       can be over-written by the user.  This means we need
       to shift the old tag values to make room for the two
       new (default) tag value pairs. */

        for ( ii = nTagValues + 1; ii >= 2; ii--) {
            tagValues[ ii] = tagValues[ ii - 2];
        }
	nTagValues += 2;

    	tagValues[ 0].tag = AmNheader;
	tagValues[ 0].value = strdup( prefsPath);
	tagValues[ 0].tagCopied = FALSE;
	tagValues[ 0].copied = TRUE;

    	tagValues[ 1].tag = AmNappId;
	tagValues[ 1].value = strdup( fileSource);
	tagValues[ 1].tagCopied = FALSE;
	tagValues[ 1].copied = TRUE;
    }

/* see if there's a file type, which is used to get the object initializer,
   get the object code, and call the object creation function */

    if ( !AmGetTagValueFromTags( tagValues, nTagValues, 
                               AmNtype, ( void **) &fileType)) {
	AmAddResourceToLog( "noFileType", headerFile);
    	DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }

    if ( !AmGetStrEnum( AmNobjectType, ( char *) fileType, &objectCode)) {
        AmAddResourceToLog( "badFileType", headerFile);
        DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }

    if (( objectInit = findInit(( AmObjectTypeEnum) objectCode)) == NULL) {
        AmAddResourceToLog( "noPlotInitialize", fileType);
        DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }

    ( *ppScreenObj) = ( objectInit->createFn)( window, overlay, 
	 				       ( char *) headerFile,
	 				       tagValues, nTagValues);
    if ( !( *ppScreenObj)->IsValid()) {
	AmAddResourceToLog( "plotInitializeFailed", headerFile);
	return( AmNOT_VALID);
    } else {
	return( AmOK);
    }
}

/**********************************************************************
 * 
 * Description:
 *	Given an header file of the form "tag: value", and an overlay,
 *	load the header file, find the appropriate object initializer
 *	for the fileType, and parse the header strings into an object
 *	to add to the overlay.
 * Arguments:
 *	headerFile - filename containing header strings. 
 *	overlay - overlay to add object to.
 *	prefsDir - preferences directory containing files of the form
 *	XXXX.prefs, where XXX is the file source used in the header.
 * Returns:
 *	AmOPEN_FAILED if the file couldn't be read.
 *	AmBAD_ARGUMENT if the AmNtype tag couldn't be found, or if
 *	it was an invalid type.
 *	AmNOT_VALID if the object was not constructed properly.
 *
 **********************************************************************/

AmErrorCode
AmAddLayerFromHeader( AmWindowClass    *window,
                    const char     *headerFile,
                    AmOverlayClass   *overlay,
                    const char     *prefsDir,
                    AmScreenObjClass **pScreenObj)
{
    char		**headerStrings;
    int			numValues;
    AmTagValueRec		tagValues[ MAXTAGS];
    AmErrorCode		errCode;

/* read the header, get the file type, get the enumeration for the file
   type string, and get the intializer for the enumeration type. */

    if (( headerStrings = AmReadHeader( headerFile)) == NULL) {
	AmAddResourceToLog( "readHeaderFailed", headerFile);
    	DO_RETURN_ERROR( AmOPEN_FAILED);
    }

    /* convert from header strings to tag values, and create the object */

    numValues = AmHeaderStringsTagValueArray( headerStrings, tagValues);
    errCode = AmAddLayerFromTagValues( window, overlay, tagValues, numValues, 
				       headerFile, prefsDir, pScreenObj);
    AmFreeStrVec( headerStrings);
    if ( errCode != AmOK) {
	AmAddResourceToLog( "plotInitializeFailed", headerFile);
	return( errCode);
    } else {
	return( AmOK);
    }
}

static AmListPtr templAddObject( AmWindowClass	*pWindow, 
			       AmTemplDeclPtr	decl,
			       AmListPtr		table,
			       AmScreenObjClass	**obj);

static void templFreeField( AmTemplFieldPtr field);

/**********************************************************************
 * 
 * Description:
 * 	given a type enumeration, tagValues array and a parent, 
 *	create an object.
 * Arguments:
 *	pWindow - window to create the object on.
 *	parent - object's parent (may be NULL)
 *	name - string name to assign to the object.
 *	objectType - object type enumeration, which maps to the object's
 *	constructor.
 *	tagValues, numValues - resources to assign to the object.
 * Returns:
 *	The created object if the initializer was found, NULL otherwise.
 *
 **********************************************************************/

AmScreenObjClass *
AmCreateObjectFromType( AmWindowClass	*pWindow,
		      AmScreenObjClass	*parent,
		      char		*name,
		      AmObjectTypeEnum	objectType,
		      AmTagValuePtr	tagValues,
		      int		numValues)
{
    AmObjectInitPtr	init;

    init = findInit( objectType);
    if ( init != NULL) {
	return(( init->createFn)( pWindow, parent, name, 
				  tagValues, numValues));
    } else {
	return( NULL);
    }
}
/**********************************************************************
 * 
 * Description:
 * 	given an object string name, tagValues array and a parent, 
 *	create an object.
 * Arguments:
 *	pWindow - window to create the object on.
 *	parent - object's parent (may be NULL)
 *	name - string name to assign to the object.
 *	type Name - object type string name, which maps to the object's
 *	constructor.
 *	tagValues, numValues - resources to assign to the object.
 * Returns:
 *	The created object if the initializer was found, NULL otherwise.
 *
 **********************************************************************/

AmScreenObjClass *
AmCreateNamedObject( AmWindowClass		*pWindow,
		   AmScreenObjClass	*parent,
		   char			*name,
		   char			*typeName,
		   AmTagValuePtr		tagValues,
		   int			numValues)
{
    AmObjectTypeEnum	objectType;

    objectType = AmFindKeyword( typeName);
    return( AmCreateObjectFromType( pWindow, parent, name, objectType, 
				  tagValues, numValues));
}

/**********************************************************************
 * 
 * Description:
 * 	given a type enumeration, tagValues array and a parent, 
 *	create an object.
 * Arguments:
 *	pWindow - window to create the object on.
 *	parent - object's parent (may be NULL)
 *	name - string name to assign to the object.
 *	tagValues, numValues - resources to assign to the object.
 *	objectType - object type enumeration, which maps to the object's
 *	constructor.
 * Returns:
 *	The created object if the initializer was found, NULL otherwise.
 *
 **********************************************************************/

static AmScreenObjClass *
AmCreateObject( AmWindowClass	*pWindow,
	      AmScreenObjClass	*parent,
	      char		*name,
	      AmTagValuePtr	tagValues,
	      int		numValues,
	      AmObjectTypeEnum	objectType)
{
    AmObjectInitPtr	init;

    init = findInit( objectType);
    if ( init != NULL) {
	return(( init->createFn)( pWindow, parent, name,
                                  tagValues, numValues));
    } else {
	return( NULL);
    }
}

/* the following functions are called by the parser, templ.y, which 
   constructs the temporary data structure used to create objects. */

/**********************************************************************
 * 
 * Description:
 * 	Create a field which is will be turned into an element in the
 *	tagValues array to create an object.
 * Arguments:
 *	tag - tag string name.
 *	value - tag value.
 * Returns:
 *	tempField structure containing the tag and value.
 *
 **********************************************************************/

AmTemplFieldPtr
AmTemplCreateField( char *tag, char *value)
{
    AmTemplFieldPtr field = ( AmTemplFieldPtr) malloc( sizeof( AmTemplFieldRec));

    field->type = AmFIELD_ATOMIC;
    field->tag = tag; 
    field->u.atomicValue = value;	/* allocated by lexer */
    return( field);
}

/**********************************************************************
 * 
 * Description:
 * 	create a field whose value is a child object.  This was 
 *	designed to be used for grids, which are made of 4 tickmark
 *	child objects.
 * Arguments:
 *	tag - tag string name.
 *	value - object declaration structure.
 * Returns:
 *	tempField structure containing the tag and object declaration.
 *
 **********************************************************************/

AmTemplFieldPtr
AmTemplCreateChild( char *tag, AmTemplDeclPtr	value)
{
    AmTemplFieldPtr field = ( AmTemplFieldPtr)malloc( sizeof( AmTemplFieldRec));

    field->type = AmFIELD_OBJECT;
    field->tag = tag; 			/* allocated by lexer */
    field->u.objectValue = value;
    return( field);
}

/**********************************************************************
 * 
 * Description:
 * 	Create an object declaration.
 * Arguments:
 *	name - object identifier name
 *	type - object type enumeration.
 *	parent - parent object name,
 *	fields - list of fields to set in the object.
 * Returns:
 *	tempDeclPtr structure pointer which defines the object.
 *
 **********************************************************************/

AmTemplDeclPtr
AmTemplCreateDecl( char 		*name, 
		 AmObjectTypeEnum type, 
		 char 		*parent, 
		 AmListPtr 	fields)
{
    AmTemplDeclPtr decl = ( AmTemplDeclPtr) malloc( sizeof( AmTemplDeclRec));

    decl->name = name; 			/* allocated by lexer */
    decl->parent = parent;		/* parent may be NULL */
    decl->type = type;
    decl->fields = fields;
    return( decl);
}

/**********************************************************************
 * 
 * Description:
 * 	Delete a decleration. Free the name, parent name, and the list
 *	of fields.
 * Arguments:
 *	decl - declaration to free.
 *
 **********************************************************************/

static void
templFreeDecl( AmTemplDeclPtr decl)
{
    if ( decl != NULL) {
        if ( decl->name != NULL) {
            free( decl->name);
        }
        if ( decl->parent != NULL) {
            free( decl->parent);
        }
        if ( decl->fields != NULL) {
            AmFreeLst( decl->fields, ( AmItemFn)templFreeField);
        }
        free( decl);
    }
}

/**********************************************************************
 * 
 * Description:
 * 	Delete a field.  If the field is an object, call the object
 *	free function.
 * Arguments:
 *	decl - declaration to free.
 *
 **********************************************************************/

static void
templFreeField( AmTemplFieldPtr field)
{
    if ( field != NULL) {
        if ( field->tag != NULL) {
            free( field->tag);
        }
	if ( field->type == AmFIELD_ATOMIC) {
	    if ( field->u.atomicValue != NULL) {
		free( field->u.atomicValue);
	    }
	} else {
	    if ( field->u.objectValue != NULL) {
		templFreeDecl( field->u.objectValue);
	    }
	}
        free( field);
    }
}

/**********************************************************************
 * 
 * Description:
 * 	Add an object declaration to the list of object declarations. 
 * Arguments:
 * 	newDecl - declaration to add.
 * Returns:
 *	true if the declaration was added, false if it's already in
 *	the list of declarations.
 *
 **********************************************************************/

AnvilBoolean
AmTemplAddDecl( AmTemplDeclPtr newDecl)
{
    AmListPtr      list;
    AmTemplDeclPtr decl;

    for ( list = templDeclList; list != NULL; list = list->next) {
        decl = ( AmTemplDeclPtr) list->data;
        if ( !strcmp(decl->name, newDecl->name)) {
            return( FALSE);
        }
    }
    templDeclList = AmInsertNodeAtEndLst( newDecl, templDeclList);
    return( TRUE);
}

/**********************************************************************
 * 
 * Description:
 *	given a symbol name, find the associated symbol in the symbol
 *	table.
 * Arguments:
 *	table - symbol table list.
 *	name - name of the symbol to search for.
 * Returns:
 *	The symbol if found, NULL otherwise.
 *
 **********************************************************************/

static AmTemplSymbolPtr
templGetSymbol( AmListPtr table, char *name)
{
    AmTemplSymbolPtr symbol;
    AmListPtr        list;

/* Take care of the degenerate case. */

    if ( name == NULL) {
        return( NULL);
    }

/* Look through the symbol table. */

    for (list = table; list != NULL; list = list->next) {
        symbol = ( AmTemplSymbolPtr) list->data;
        if ( !strcmp( symbol->name, name)) {
            return( symbol);
        }
    }

/* No symbol with that name found. */

    return( NULL);
}

/**********************************************************************
 * 
 * Description:
 * 	Add a symbol to the symbol table.
 * Arguments:
 *	table - the symbol table.
 *	name - symbol name
 *	object - symbol to add to the table.
 * Returns:
 *	the new symbol table with the object added, or the original
 *	symbol table if a new node couldn't be allocated.
 *
 **********************************************************************/

static AmListPtr
templAddSymbol( AmListPtr table, char *name, AmScreenObjClass *object)
{
    AmTemplSymbolPtr symbol;
   
    if ((symbol = ( AmTemplSymbolPtr) malloc( sizeof( AmTemplSymbolRec))) == NULL) {
        return( table);
    }
    symbol->name = strdup( name);
    symbol->object = object; 
    table = AmInsertNodeAtEndLst( symbol, table);
    return( table);
}

/**********************************************************************
 * 
 * Description:
 * 	Free a symbol.
 * Arguments:
 *	symbol - symbol to free.
 *
 **********************************************************************/

static void
templFreeSymbol( AmTemplSymbolPtr symbol)
{
    if ( symbol != NULL) {
        if ( symbol->name != NULL) {
            free( symbol->name);
        }
        free( symbol);
    }
}

/**********************************************************************
 * 
 * Description:
 *	Given a list of fieldPtr structures, generaete a list of 
 *	header strings pairs, where the AmFirst string is the tag name,
 *	and the second is the value.
 * Arguments:
 *	lstFields - generic list of field declarations.
 * Returns:
 *	NULL-terminated, allocated vector of header strings.
 *
 **********************************************************************/

static char **
templCreateAtomicHeaderStrings( AmListPtr lstFields)
{
    AmListPtr       lstTemp;
    AmTemplFieldPtr pField;
    char          **pszHeaderStrings;
    int           nStrings, ii;

    nStrings = 0;

    for ( lstTemp = lstFields;
          lstTemp != NULL;
          lstTemp = lstTemp->next)
    {
        pField = ( AmTemplFieldPtr) lstFields->data;
	if ( pField->type == AmFIELD_ATOMIC) {
            nStrings += 2;
        }
    }

    pszHeaderStrings = ( char **) malloc( sizeof( char *) * ( nStrings + 1));
    if ( pszHeaderStrings == NULL) {
        return( NULL);
    }
    pszHeaderStrings[ nStrings] = NULL;

    ii = 0;

    for ( lstTemp = lstFields;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pField = ( AmTemplFieldPtr) lstTemp->data;
	if ( pField->type == AmFIELD_ATOMIC) {

        /*
         * Any fields with '\n's are written out with the newlines
         *   literalized, so we unliteralize them when reading them
         *   in.
         */

	    AmReplaceNewlines( pField->u.atomicValue);
            pszHeaderStrings[ ii++] = strdup( pField->tag);
            pszHeaderStrings[ ii++] = strdup( pField->u.atomicValue);

	} 
    }

    return( pszHeaderStrings);
}

/**********************************************************************
 * 
 * Description:
 *	given a list of fields, generate a list of tag values.
 * Arguments:
 *	fields - list of object field definitions.
 *	tagValues - resulting array of tag values.
 *	numValues - number of values in the array of tag values.
 *
 **********************************************************************/

static void 
templCreateAtomicTagValues( AmListPtr 	fields,
			    AmTagValuePtr	tagValues,
			    int		*numValues)
{
    AmTemplFieldPtr 	field;

    *numValues = 0;
    for ( ; fields != NULL; fields = fields->next) {
        field = ( AmTemplFieldPtr) fields->data;
	if ( field->type == AmFIELD_ATOMIC) {

/* any fields with '\n's are written out with the newlines literalized, so
   we unliteralize them when reading them in */

	    AmReplaceNewlines( field->u.atomicValue);
	    if ( AmScanValue( field->tag, field->u.atomicValue,
			    &tagValues[ *numValues].value, 
			    &tagValues[ *numValues].copied) == AmOK) {
		tagValues[ *numValues].tag = strdup( field->tag);
		tagValues[ *numValues].tagCopied = TRUE;
		( *numValues)++;
	    } else {

/* if the type isn't specified, assume it's a string */

		tagValues[ *numValues].tag = strdup( field->tag);
		tagValues[ *numValues].tagCopied = TRUE;
		tagValues[ *numValues].value = strdup( field->u.atomicValue);
		tagValues[ *numValues].copied = TRUE;
		( *numValues)++;
	    }
	} 
    }
}

/**********************************************************************
 * 
 * Description:
 * 	this function creates the objects from the specified fields, and 
 *	puts them in a tagValue array suitable for setting using 
 *	setFields() 
 * Arguments:
 *	pWindow - window to create the objects on.
 *	fields - fields to set for the object.
 *	table - the symbol table.
 *	tagValues, numValues - the tagValues to set for the object.
 *
 **********************************************************************/

static AmListPtr 
templCreateObjectTagValues( AmWindowClass		*pWindow,
			    AmListPtr		fields,
			    AmListPtr		table,
			    AmTagValuePtr		tagValues,
			    int			*numValues)
{
    AmScreenObjClass	*obj;
    AmTemplFieldPtr	field;

    *numValues = 0;
    for ( ; fields != NULL; fields = fields->next) {
        field = ( AmTemplFieldPtr) fields->data;

/* for each of the objects, create the object, and add it to the tag values
   vector */

	if ( field->type == AmFIELD_OBJECT) {
	    table = templAddObject( pWindow, field->u.objectValue, table, &obj);
	    tagValues[ *numValues].tag = field->tag;
	    tagValues[ *numValues].value = obj;
	    tagValues[ *numValues].tagCopied = FALSE;
	    tagValues[ *numValues].copied = FALSE;
	    ( *numValues)++;
	}
    }
    return( table);
}

/**********************************************************************
 * 
 * Description:
 * 	the caller-provided add layer function works only with plots 
 *	and graphs, not overlays, ticks, grids, annotations or legends. 
 *	this function creates a plot object (derived from data graphic)
 * Arguments:
 *	pDecl - declaration to parse.
 *	pWindow - window to create the object on.
 *	lstTable - symbol table.
 *	fnCreateObject - object creation function pointer.
 *	pClientData - pointer to client data to pass to the object
 *	creation function.
 * Returns:
 *	the symbol table.
 *
 **********************************************************************/

static AmListPtr 
templAddPlotObject( AmTemplDeclPtr	pDecl,
		    AmWindowClass         *pWindow,
		    AmListPtr		lstTable,
		    AmCreateObjectFn      fnCreateObject,
		    void                *pClientData)
{
    AmDataGraphicClass    *pDataGraphic;
    AmTemplSymbolPtr   	pSymbol;
    char                **pszHeaderStrings;

    pszHeaderStrings = templCreateAtomicHeaderStrings( pDecl->fields);

    pSymbol = templGetSymbol( lstTable, pDecl->parent);    
    if ( pSymbol != NULL) {
        pDataGraphic = ( AmDataGraphicClass *) fnCreateObject( pWindow,
                                                             pSymbol->object,
                                                             pszHeaderStrings,
                                                             pClientData);
    } else {
        pDataGraphic = ( AmDataGraphicClass *) fnCreateObject( pWindow,
                                                             NULL,
                                                             pszHeaderStrings,
                                                             pClientData);
    }
    AmFreeStrVec( pszHeaderStrings);

    if ( pDataGraphic != NULL) {
        lstTable = templAddSymbol( lstTable, pDecl->name, pDataGraphic);
    } else {
        AmAddResourceToLog( "createFailed", pDecl->name);
    }

    return( lstTable);
}   

/**********************************************************************
 * 
 * Description:
 * 	given an integer code referencing an object, the symbol table, 
 *	window, and declaration, create the object 
 * Arguments:
 *	pDecl - declaration to parse.
 *	pWindow - window to create the object on.
 *	lstTable - symbol table.
 *	obj - returned object
 * Returns:
 *	the symbol table.
 *
 **********************************************************************/

static AmListPtr 
templAddObject( AmWindowClass	*pWindow, 
		AmTemplDeclPtr	decl,
		AmListPtr		table,
		AmScreenObjClass	**obj)
{
    AmTemplSymbolPtr   	symbol;
    AmTagValueRec		tagValues[ MAXTAGS];
    int			numValues;

    *obj = NULL;
    templCreateAtomicTagValues( decl->fields, tagValues, &numValues);

/* if there's a parent reference, then find the (hopefully) already 
    created parent, and create the child object */

    if ( decl->parent != NULL) {
	symbol = templGetSymbol( table, decl->parent);    
	if ( symbol != NULL) {
	    *obj = AmCreateObject( pWindow, symbol->object, decl->name, 
				 tagValues, numValues, decl->type);
	} else {
            AmAddResourceToLog( "invalidParent", decl->parent);
	    *obj = NULL;
	}
    } else {
	*obj = AmCreateObject( pWindow, NULL, decl->name, tagValues, numValues,
			     decl->type);
    }

/* only add the object to the table if it isn't NULL */

    if ( *obj != NULL) {

/* create any of child objects and set them in the tagValues array, then
   set them in the object */

	table = templCreateObjectTagValues( pWindow, decl->fields, table,
					    tagValues, &numValues);
	if ( numValues != 0) {
	    ( *obj)->SetVaFields( AmNnumTagValues, numValues,
				  AmNtagValues, tagValues,
				  NULL);
	}

        table = templAddSymbol( table, decl->name, *obj);
    }
    return( table);
}
    
/**********************************************************************
 * 
 * Description:
 * 	is the object a non-manipulatable child? 
 * Arguments:
 *	object - object to test.
 *	parent - parent to test against. 
 * Returns:
 *	TRUE if the object is a plot or graph, and the parent is not an
 *	overlay, or if the object is ticks with a non-grid parent.
 * Modifications:
 *	This is likely to go away when the data-based object exception
 *	case is removed.
 *
 **********************************************************************/

static AnvilBoolean 
isChildObject( AmScreenObjClass	*object,
	       AmScreenObjClass	*parent)
{
    if ( parent == NULL) {
    	return( FALSE);
    } else {
    	return((( object->IsType( AmPLOT) || object->IsType( AmGRAPH)) &&
	       ( parent->GetObjectType() != AmOVERLAY)) ||
	       (( object->GetObjectType() == AmTICKS) &&
	        ( parent->GetObjectType() != AmGRID)));
    }
}

/**********************************************************************
 * 
 * Description:
 * 	take the list of declarations and parse them into objects.
 * Arguments:
 *	objectList - list of object declarations.
 *	pWindow - window to create the objects on.
 *	fnCreateObject - function to call when creating objects.
 *	pClientData - client data to pass to the function.
 * Returns:
 * 	NULL-terminated allocated vector of screen objects.
 *
 **********************************************************************/

static AmScreenObjClass **
templParseObjectList( AmListPtr 		objectList, 
		      AmWindowClass 	*pWindow,
                      AmCreateObjectFn    fnCreateObject,
                      void              *pClientData)
{
    AmScreenObjClass    	**vector, *obj;
    AmListPtr           ptr, table, remainList, findPtr, declList;
    AmTemplDeclPtr      decl; 
    AmTemplSymbolPtr    symbol, parent;
    int               ii, size;
    AnvilBoolean	      found;

    table = NULL;
    declList = AmTraceLst( objectList);
    do {

/* in each pass, take all the objects which were parsed, and remove them from the
   list */

	remainList = NULL;
	for ( ptr = declList; ptr != NULL; ptr = ptr->next) { 
	    decl = ( AmTemplDeclPtr) ptr->data;

/* if the object has no parent, then it's OK to create */

	    found = FALSE;
	    if ( decl->parent == NULL) {
		if ( isDataGraphic( decl->type)) {
		    table = templAddPlotObject( decl, pWindow, table,
                                                fnCreateObject, pClientData);
		} else {
		    table = templAddObject( pWindow, decl, table, &obj);
		}
		found = TRUE;
	    } else {

/* see if the object's parent has been created yet... */

		for ( findPtr = table; 
		      findPtr != NULL; 
		      findPtr = findPtr->next) {
		    symbol = ( AmTemplSymbolPtr) findPtr->data;
		    if ( !strcmp( symbol->name, decl->parent)) {
			if ( isDataGraphic( decl->type)) {
			    table = templAddPlotObject( decl,
                                                        pWindow,
                                                        table, 
							fnCreateObject,
                                                        pClientData);
			} else {
			    table = templAddObject( pWindow, decl,
                                                    table, &obj);
			}
			found = TRUE;
			break;
		    }
		}
	    }

/* if the object hasn't been parsed yet, add it to the remainder list */

	    if ( !found) {
		remainList = AmInsertNodeAtEndLst( decl, remainList);
	    }
	}

/* if the remain list is the same size as the object list, then we're not 
   reducing and we need to avoid an infinite loop... */

	if ( AmLengthLst( declList) == AmLengthLst( remainList)) {
	    for ( ptr = declList; ptr != NULL; ptr = ptr->next) {
		decl = ( AmTemplDeclPtr) ptr->data;
                AmAddResourceToLog( "parseDeclFailed", decl->name);
	    }
	    return( NULL);
	}

/* set the object list to the objects that weren't parsed */

	AmJustFreeLst( declList);
	declList = remainList;
    } while ( remainList != NULL); 

/* Resort the list so that it is in the same order as before, and track the
   size of the vector we're going to allocate to hold all of the top-level
   display objects... This should be cleaned up... */

    size = 0;
    for ( ptr = objectList; ptr != NULL; ptr = ptr->next) { 
        decl = ( AmTemplDeclPtr) ptr->data;
        symbol = templGetSymbol( table, decl->name);
        if ( symbol != NULL) {
            parent = templGetSymbol( table, decl->parent);

/* if the object is a plot, and its parent isn't an overlay, a graph, and
   is parent isn't an overlay, or a tick, and it's parent isn't a grid, then
   add it to the list */

            if (( parent == NULL) ||
            	!isChildObject( symbol->object, parent->object)) {
                size++;
            } 
        }
    }

/* create a vector which contains all of the created objects which are
   not datagraphics which are children of overlays (layers) */

    vector = ( AmScreenObjClass **)malloc( sizeof( AmScreenObjClass *)*( size + 1));
    for ( ptr = objectList, ii = 0; ptr != NULL; ptr = ptr->next) { 
        decl = ( AmTemplDeclPtr) ptr->data;
        symbol = templGetSymbol( table, decl->name);
        if ( symbol != NULL) {
            parent = templGetSymbol( table, decl->parent);
	    if (( parent == NULL) || !symbol->object->IsType( AmDATA_GRAPHIC)) {
	    	vector[ ii++] = symbol->object;
	    }
	}
    }

/* terminate the vector and blow away the symbol table */

    vector[ ii] = NULL;
    AmFreeLst( table, ( AmItemFn) templFreeSymbol);
    return( vector);
}


/**********************************************************************
 * 
 * Description:
 * 	take a description file and parse it into a type structure 
 * Arguments:
 *	file - file containing the list of object descriptions.
 *	pWindow - window to create the objects on.
 *	fnCreateObject - function to call when creating objects.
 *	pClientData - client data to pass to the function.
 *	contains the appropriate error string.
 * Returns:
 * 	NULL-terminated allocated vector of screen objects.
 *
 **********************************************************************/

AmScreenObjClass **
AmParseTemplateFile( char         	*szFile,
		   AmWindowClass		*pWindow,
                   AmCreateObjectFn       fnCreateObject,
                   void                 *pClientData)
{
    FILE	   *fp;
    AmScreenObjClass **vpScreenObj;

/* Open the template file. */

    if (( fp = fopen( szFile, "r")) == NULL) {
        AmAddResourceToLog( "openFailed", szFile);
	return( NULL);
    }

/* Parse the template file. */

    vpScreenObj = AmParseTemplateFp( fp, pWindow, fnCreateObject, pClientData);

/* Close the template file. */

    fclose( fp);

/* Return the result. */

    return( vpScreenObj);
}


/**********************************************************************
 * 
 * Description:
 * 	take a file pointer to description file and parse it into a 
 *	type structure 
 * Arguments:
 *	fp - file pointer containing the list of object descriptions.
 *	pWindow - window to create the objects on.
 *	fnCreateObject - function to call when creating objects.
 *	pClientData - client data to pass to the function.
 *	contains the appropriate error string.
 * Returns:
 * 	NULL-terminated allocated vector of screen objects.
 *
 **********************************************************************/

AmScreenObjClass **
AmParseTemplateFp( FILE		*fp,
	         AmWindowClass	*pWindow,
                 AmCreateObjectFn fnCreateObject,
                 void           *pClientData)
{
    AmScreenObjClass **vpScreenObj;

/* Initializations: */

    templLineNo   = 1;
    templDeclList = NULL;
    vpScreenObj   = NULL;

/* Set the template file descriptor to the newly opened file.  We need a global
   descriptor for the lexer line reader */

    templFp = fp;

/* Parse that file */

    templrestart( templFp);
    templparse();

/*
 * The result of the parse is in the static variable templObjectList.
 *   Parse that result and create the resulting plots and images.
 */

    vpScreenObj = templParseObjectList( templDeclList, pWindow,
                                        fnCreateObject, pClientData);
    AmFreeLst( templDeclList, ( AmItemFn) templFreeDecl);

    return( vpScreenObj);
}

/* Wrapper error function for the lexer and parser. */

int
templerror( char *fmt, ...)
{
    va_list args;
    char    buf[ 256];

    va_start (args, fmt);
    vsprintf (buf, fmt, args);
    va_end (args);

    AmAddResourceToLog( "Error (line %d): %s", templLineNo, buf);

    return( 0);
}

int
AmTemplFpYYinput( char *buffer, int  maxSize)
{
    return( fread( buffer, 1, maxSize, templFp));
}
