#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */






#include <stdio.h>
#include <malloc.h>
#include <anvil/dither.h>
#include <anvil/postscriptcolor.h>
#include <anvil/postscript.h>
#include <anvil/postscriptP.h>

/* Number of shades used for each primary color in the color cube. */

static ushort_t numRedShades;
static ushort_t numGreenShades;
static ushort_t numBlueShades;

/* Offset tables into the color cube for specific shades of a color. */ 

static ulong_t redOffset[ 256];
static ulong_t greenOffset[ 256];
static ulong_t blueOffset[ 256];

uchar_t PostScriptDecisionMatrix[ NUM_DITHER_MATRICES][ DITHER_LEVEL][ DITHER_LEVEL];

static ulong_t redDitherMatrix[ 256];
static ulong_t greenDitherMatrix[ 256];
static ulong_t blueDitherMatrix[ 256];

ulong_t *
AmGetPostScriptRedDitherMatrix()
{
    return( redDitherMatrix);
}

ulong_t *
AmGetPostScriptGreenDitherMatrix()
{
    return( greenDitherMatrix);
}

ulong_t *
AmGetPostScriptBlueDitherMatrix()
{
    return( blueDitherMatrix);
}


/*
 * Function: AmGetPostScriptRedOffsetLookup(), AmGetPostScriptGreenOffsetLookup(),
 *           AmGetPostScriptBlueOffsetLookup()
 *
 * Description:
 *     These functions return a lookup table that is used in translation
 *     from a 32-bit image to an n-bit image.  The lookup tables are only
 *     used for pseudo color visuals.  The pixel value to be used is generated
 *     by adding the values gotten by indexing the red, green, and blue
 *     component of a color into the offset lookup tables.  The resulting
 *     value (the sum) is then used as an index into the static color cube.
 *
 * For example:
 *
 *     redOffset = AmGetPostScriptRedOffsetLookup();
 *     greenOffset = AmGetPostScriptGreenOffsetLookup();
 *     blueOffset = AmGetPostScriptBlueOffsetLookup();
 *     color = AmGetAppColors() + redOffset[ red] + greenOffset[ green] +
 *             blueOffset[ blue];
 *     pixel = color->pixel; 
 */

const ulong_t *
AmGetPostScriptRedOffsetLookup()
{
    return( redOffset);
}

const ulong_t *
AmGetPostScriptGreenOffsetLookup()
{
    return( greenOffset);
}

const ulong_t *
AmGetPostScriptBlueOffsetLookup()
{
    return( blueOffset);
}

/*
 * Function: AmGetPostScriptNumRedShades(), AmGetPostScriptNumBlueShades(), 
 * AmGetPostScriptNumGreenShades()
 *
 * Description:
 *     These functions return the number of shades allocated per color in
 *     the color cube.  The total number of colors is the product of the
 *     three values.
 */

ushort_t
AmGetPostScriptNumRedShades()
{
    return( numRedShades);
}

ushort_t
AmGetPostScriptNumGreenShades()
{
    return( numGreenShades);
}

ushort_t
AmGetPostScriptNumBlueShades()
{
    return( numBlueShades);
}

/* create a color cube for dithering postscript output */

AmErrorCode 
AmCreatePostScriptColorCube( AmColorCellPtr		*cubeColors,
			   uint_t		numRed,
			   uint_t		numGreen,
			   uint_t		numBlue)
{

    *cubeColors = ( AmColorCellPtr) 
	    malloc( sizeof( AmColorCellRec)*numRed*numGreen*numBlue); 
    if ( *cubeColors == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    AmCreateColorCube( *cubeColors, numRed, numGreen, numBlue, 
		     redOffset, greenOffset, blueOffset);
    numRedShades = numRed;
    numGreenShades = numGreen;
    numBlueShades = numBlue;
    AmInitDitherMatrices( numRedShades, numGreenShades, numBlueShades,
			PostScriptDecisionMatrix,
			redDitherMatrix, greenDitherMatrix, blueDitherMatrix);
    return( AmOK);
}
