#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#define SCREEN_COORD( iVirtual)                                            \
     ( iVirtual * _pDisplay->GetDPI() / _pDisplay->GetDPIBase())
#define SCREEN_COORD_FLOAT( dVirtual)                                      \
     ( dVirtual * _pDisplay->GetDPI() / ( float) _pDisplay->GetDPIBase())

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#if !defined(__BSD44__)
#include <search.h>
#endif
#include <anvil/errlib.h>
#include <anvil/envdefs.h>
#include <anvil/mem_funcs.h>
#include <anvil/shmimg.h>
#include <anvil/postscript.h>
#include <anvil/parseAFM.h>
#include <anvil/fonts.h>
#include <anvil/tsearch.h>
#include <anvil/choppath.h>

/* Create an X GC class */

AmPostScriptGCClass::AmPostScriptGCClass( 
			AmPostScriptOutputClass *postScriptOutput)
    : AmGCClass( postScriptOutput)
{
/* foreground is black */

    fg.red = 0;
    fg.green = 0;
    fg.blue = 0;
    _pDisplay->AllocColor( &fg);
    fgDirty = FALSE;

/* background is white */

    bg.red = 255;
    bg.green = 255;
    bg.blue = 255;
    _pDisplay->AllocColor( &bg);
    bgDirty = FALSE;

/* thinnest line width */

    _dLineWidth = 0.0;
    _fLineWidthDirty = FALSE;

/* standard cap and join */

    lineStyle = AmLINE_SOLID;
    lineStyleDirty = FALSE;
    lineJoin = AmJOIN_MITER;
    lineJoinDirty = FALSE;

/* load a basic font */
 
    strcpy( fontName, DEFAULT_FONT);
    strcpy( fontPostName, DEFAULT_POST_FONT);
    fontSize = DEFAULT_SIZE; 
    fontWeight = DEFAULT_WEIGHT;
    fontSlant = DEFAULT_SLANT;

    fontInfo = NULL;		/* AFM info is NULL */
    fontDirty = FALSE;

    SetFont( fontName, fontSize, fontWeight, fontSlant);

/* no clipping unless specified explicitly */

    clipRectDirty = FALSE;
    clipRectVec = NULL;
    nrects = 0;

/* NULL out the clipping region */

    clipRegion = NULL;
    clipRegionDirty = FALSE;
    clipRectDirty = FALSE;

/* turn off dashes */

    dashPattern = NULL;
    numDashes = 0;
    dashOffset = 0;
    dashDirty = FALSE;

/* raster operation is copy */

    rasterOp = AmCOPY;
    rasterOpDirty = FALSE;

/* initialize fill pattern stuff */

    fillPatternRoot = NULL;
    fillPatDirty = FALSE;
}

static void 
freePatternAction( const void *pattern, VISIT order, int)
{
    if (( order == preorder) || (order == leaf)) {
	free(*( char **) pattern);
    }
}

/* destroy an X GC class */

AmPostScriptGCClass::~AmPostScriptGCClass()
{
    AN_DEBUG( 100, ( "In ~AmPostScriptGCClass()\n"));

    if ( fontInfo != NULL) {
	AFMFreeInfo( fontInfo);
    }
    if ( clipRectVec != NULL) {
	FREE( clipRectVec);
    }
    twalk( fillPatternRoot, ( AmWalkFn) freePatternAction);
}

/* set the line width */

void 
AmPostScriptGCClass::SetLineWidth( float dLineWidth)
{
    _dLineWidth = dLineWidth;
    _fLineWidthDirty = TRUE;
}

/* return the line width */

float
AmPostScriptGCClass::GetLineWidth()
{
    return( _dLineWidth);
}

/* set the line join */

void 
AmPostScriptGCClass::SetLineJoin( AmLineJoinEnum	lineJoinArg)
{
    lineJoin = lineJoinArg;
    lineJoinDirty = TRUE;
}

/* retreive the line join */

AmLineJoinEnum 
AmPostScriptGCClass::GetLineJoin() 
{
    return( lineJoin);
}

/* set the line style */

void 
AmPostScriptGCClass::SetLineStyle( AmLineStyleEnum	lineStyleArg)
{
    lineStyle = lineStyleArg;
    lineStyleDirty = TRUE;
}

/* retreive the line style */

AmLineStyleEnum 
AmPostScriptGCClass::GetLineStyle() 
{
    return( lineStyle);
}

/* match a font with a font template */

static AnvilBoolean 
matchFontTemplate( char	*fontTemplate,
		   char	*font)
{
    char 	*start, *end, field[ PATHLEN];

/* the string is of the form -*-blah blah blah-*, and we want blah blah blah */

    for ( start = fontTemplate; !isalpha( *start); start++);
    for ( end = start; ( *end != '\0') && ( *end != '*'); end++);
    while ( !isalpha( *end)) {
	end--;
    }
    strncpy( field, start, end - start + 1);
    field[ end - start + 1] = '\0';

/* if the resulting subfield is the same or than the requested font, then 
   see if they match */

    if ( strlen( field) >= strlen( font)) {
	return( !strncmp( field, font, sizeof( font)));
    } else {

/* if the subfield is shorter, then scan for it in the font */

	return( strstr( font, field) != NULL);
    }
}

/* Return the text's extents, using AFM files if possible...
   note that the extents are in virtual coordinates at this
   point. */ 

AnvilBoolean
AmPostScriptGCClass::GetTextExtents( char *szText,
                                     int  *piWidth,
                                     int  *piAscent,
                                     int  *piDescent)
{
    AnvilBoolean fSuccess;

    if ( fontInfo == NULL) {

    /* Some default values. */

	*piWidth = (int)(strlen( szText) * fontSize * fontXScale *
                   _pDisplay->GetDPIBase() / 72);
	*piAscent = (int)(fontSize * fontYScale / 5.0 *
                    _pDisplay->GetDPIBase() / 72);
	*piDescent = (int)(fontSize * fontYScale * 3.0 / 5.0 *
                     _pDisplay->GetDPIBase() / 72);

        fSuccess = FALSE;

    } else {

    /* Get the extents from the display postscript crap. */

	fSuccess = AFMTextExtents( szText, fontInfo, piWidth, piAscent,
                                   piDescent);

	*piWidth *= (int)(fontSize * fontXScale * _pDisplay->GetDPIBase()
                    / 72000); 
	*piAscent *= (int)(fontSize * fontYScale * _pDisplay->GetDPIBase()
                     / 72000); 
	*piDescent *= (int)(fontSize * fontYScale * _pDisplay->GetDPIBase()
                      / 72000); 
    }

    return( fSuccess);
}
	

/* set the font, and font sizing */

AnvilBoolean 
AmPostScriptGCClass::SetFont( AmFontRec *pFont)
{
    return( SetFont( pFont->szFont, pFont->iSize, pFont->eWeight,
                     pFont->eSlant, pFont->dxScale, pFont->dyScale));
}

AnvilBoolean 
AmPostScriptGCClass::SetFont( char 	       *name, 
			      int 	       size,
			      AmFontWeightEnum weight,
			      AmFontSlantEnum  slant,
			      float            xScale,
			      float            yScale)
{
    FILE	*fp;
    char	postScriptName[ PATHLEN], xFontTemplate[ PATHLEN], 
		fontpath[ PATHLEN], *env;
    const char  *xFontName;
    AnvilBoolean	found; 

    xFontName = AmGetFontFromCache( name, weight, slant);

    found = FALSE;
    env = getenv( CONFIG_ENV);
    if ( env != NULL && xFontName != NULL) {

/* we have a file which maps the X fonts to postscript fonts, of the form
   "postscript"	"x-template", so we try to match the x-template */

	sprintf( fontpath, "%s/fonts/XtoPostScriptFonts", env);
	if (( fp = fopen( fontpath, "r")) != NULL) {
	    do {
		if ( fscanf( fp, "%s %[^\n]", 
			     postScriptName, xFontTemplate) != 2) {
		    break;
		}

		if ( matchFontTemplate( xFontTemplate, 
					( char *)xFontName)) {
		    found = TRUE;
		    break;
		}
	    } while ( TRUE);
	    fclose( fp);

/* now, let's parse the AFM font information file for getting font extents */

	    if ( found) {
		sprintf( fontpath, "%s/%s", AFM_DIR, postScriptName);
		if ( fontInfo != NULL) {
		    AFMFreeInfo( fontInfo);
		}
		if (( fp = fopen( fontpath, "r")) != NULL) {
		    if ( AFMParseFile( fp, &fontInfo, AFM_ALL) != afm_ok) {
			fontInfo = NULL;
		    }
		} else {
                    AN_DEBUG( 10, ("failed to open AFM file %s\n", fontpath));
		}
	    }
	    fclose( fp);
	} else {
            AN_DEBUG( 10, ("failed to open font mapping file %s\n", fontpath));
	}
    }

    if ( found) {
        strcpy( fontPostName, postScriptName);
    } else {
        strcpy( fontPostName, DEFAULT_POST_FONT);
    }

    strcpy( fontName, name);
    fontDirty = TRUE;
    fontSize = size;
    fontWeight = weight;
    fontSlant = slant;
    fontXScale = xScale;
    fontYScale = yScale;

    return( TRUE);
}

/* get the current font name */

void 
AmPostScriptGCClass::GetFont( char 	        **fontNameArg,
			      int	        *size,
			      AmFontWeightEnum  *weight,
			      AmFontSlantEnum   *slant,
			      float             *xScale,
			      float             *yScale)
{
    *fontNameArg = strdup( fontName);
    *size = fontSize;

    if ( weight != NULL) {
        *weight = fontWeight;
    }

    if ( slant != NULL) {
        *slant = fontSlant;
    }

    if ( xScale != NULL) {
        *xScale = fontXScale;
    }

    if ( yScale != NULL) {
        *yScale = fontYScale;
    }
}

void 
AmPostScriptGCClass::GetFont( AmFontRec *pFont)
{
    pFont->szFont = strdup( fontName);
    pFont->iSize = fontSize;
    pFont->eWeight = fontWeight;
    pFont->eSlant = fontSlant;
    pFont->dxScale = fontXScale;
    pFont->dyScale = fontYScale;
}

/* get and set the foreground pixel */

void 
AmPostScriptGCClass::SetForegroundColor( AmColorCellPtr cell)
{
    fg = *cell;
    fgDirty = TRUE;
}

void 
AmPostScriptGCClass::GetForegroundColor( AmColorCellPtr cell)
{
    *cell = fg;
}

/* set and get the background pixel */

void 
AmPostScriptGCClass::SetBackgroundColor( AmColorCellPtr cell)
{
    bg = *cell;
    bgDirty = TRUE;
}

void 
AmPostScriptGCClass::GetBackgroundColor( AmColorCellPtr cell) 
{
    *cell = bg;
}

/* set the clipping rectangle */

void 
AmPostScriptGCClass::SetClipRectangles( AmRectangleRec const *clipRectVecArg,
					int		      nrectsArg)
{
    if ( clipRectVec != NULL) {
	FREE( clipRectVec);
    }
    nrects = nrectsArg;
    clipRectVec = ( AmRectanglePtr) 
	AmCopyVec( clipRectVecArg, nrects*sizeof( AmRectangleRec));
    clipRectDirty = TRUE;
}

/* get the clipping rectangle */

void 
AmPostScriptGCClass::GetClipRectangles( AmRectanglePtr *clipRectVecArg,
					int		*nrectsArg)
{
    *clipRectVecArg = ( AmRectanglePtr) 
	AmCopyVec( clipRectVec, nrects*sizeof( AmRectangleRec));
    *nrectsArg = nrects;
}

void 
AmPostScriptGCClass::SetClipRegion( AmRegionClass	*region)
{
    if ( clipRegion != NULL) {
	delete clipRegion;
    }
    clipRegion = (( AmPostScriptRegionClass *) region)->CopyRegion();
    clipRegionDirty = TRUE;
}

/* set the raster operation */

void 
AmPostScriptGCClass::SetRasterOp( AmRasterOpEnum	rasterOpArg)
{
    rasterOp = rasterOpArg;
    rasterOpDirty = TRUE;
}


/* return the raster operation */

AmRasterOpEnum 
AmPostScriptGCClass::GetRasterOp()
{
    return( rasterOp);
}

/* set the dash pattern */

void 
AmPostScriptGCClass::SetDashes( char	*dashPatArg,
				int	nDashArg,
				int	dashOffsetArg)
{
    if ( dashPattern != NULL) {
	FREE( dashPattern);
    } 
    numDashes = nDashArg;
    dashPattern = ( char *) AmCopyVec( dashPatArg, numDashes);
    dashOffset = dashOffsetArg;
    dashDirty = TRUE;
}

/* get the dash pattern */

void 
AmPostScriptGCClass::GetDashes( char	**dashPatArg,
			        int	*nDashArg,
			        int	*dashOffsetArg)
{
    *dashPatArg = ( char *) AmCopyVec( dashPattern, numDashes);
    *nDashArg = numDashes;
    if ( dashOffsetArg != NULL) {
	*dashOffsetArg = dashOffset;
    }
}

/* output all of the GC fields which are dirty. */

void 
AmPostScriptGCClass::OutputDirtyGCchanges( AmPostScriptOutputClass *pw)
{
    int	  	ii, x, y, iWidth, iHeight;
    float 	dx, dy;
    char	fillPatBuf[ PATHLEN];

/* set the foreground color if it's dirty */

    if ( fgDirty) {
	fprintf( pw->getFp(), "%.3f %.3f %.3f setrgbcolor\n", 
		 fg.red/255.0, fg.green/255.0, fg.blue/255.0);
	fgDirty = FALSE;
    }

/* NOTE: we can't really do anything with the background yet... */

    if ( bgDirty) {
	bgDirty = FALSE;
    }

/* set the line width */

    if ( _fLineWidthDirty) {
	fprintf( pw->getFp(), "%f setlinewidth\n",
                 SCREEN_COORD_FLOAT( _dLineWidth));
	_fLineWidthDirty = FALSE;
    }

/* set the line style */

    if ( lineStyleDirty || dashDirty) {
	if (( lineStyle == AmLINE_ON_OFF_DASH) || 
	    ( lineStyle == AmLINE_DOUBLE_DASH)) {
	    fprintf( pw->getFp(), "[ ");
	    for ( ii = 0; ii < numDashes; ii++) {
		fprintf( pw->getFp(), "%d ", dashPattern[ ii]);
	    }
	    fprintf( pw->getFp(), " ] %d setdash\n",
                     SCREEN_COORD( dashOffset));
	} else if ( lineStyle == AmLINE_SOLID) {
	    fprintf( pw->getFp(), "[] 0 setdash\n");
	} 
	lineStyleDirty = FALSE;
    }

    if ( lineJoinDirty) {
	if ( lineJoin == AmJOIN_MITER) {
	    fprintf( pw->getFp(), "0 setlinejoin\n");
	} else if ( lineJoin == AmJOIN_ROUND) {
	    fprintf( pw->getFp(), "1 setlinejoin\n");
	} else if ( lineJoin == AmJOIN_BEVEL) {
	    fprintf( pw->getFp(), "2 setlinejoin\n");
	}
	lineJoinDirty = FALSE;
    }

    /*
     * Load a basic font.  Note the translation from the fontSize into
     *   the DPI coordinates.  Font size is given in 1/72 inches.  This
     *   is different from everything else where sizes are given in
     *   "virtual" coordinates.
     */

    if ( fontDirty) {
	fprintf( pw->getFp(), "/%s findfont %d scalefont setfont\n",
		 fontPostName, fontSize * _pDisplay->GetDPI() / 72);
	fontDirty = FALSE;
    }

/* if there's a clip region, then use that */

    if ( clipRegionDirty && ( clipRegion != NULL)) {
	POLYAREA	*clipPolyArea, *polyPtr;
	PLINE		*contourPtr;
	VNODE		*ptPtr;

	clipPolyArea = (( AmPostScriptRegionClass *) 
				clipRegion)->getPolyRegion();
	if ( clipPolyArea != NULL) {
	    fprintf( pw->getFp(), "initclip\n");

/* loop over each polygon set */

	    polyPtr = clipPolyArea;
	    do {

/* loop over each contour */

		for ( contourPtr = polyPtr->contours; 
		      contourPtr != NULL;
		      contourPtr = contourPtr->next) {

/* loop over each contour's points */

		    x = (int)SCREEN_COORD( contourPtr->head.point[ 0]);
		    y = (int)SCREEN_COORD( contourPtr->head.point[ 1]);
		    fprintf( pw->getFp(), "newpath\n");
		    fprintf( pw->getFp(), "%d %d moveto\n", x, y);
		    ptPtr = contourPtr->head.next;
		    do {
			x = (int)SCREEN_COORD( ptPtr->point[ 0]);
			y = (int)SCREEN_COORD( ptPtr->point[ 1]);
			fprintf( pw->getFp(), "%d %d lineto\n", x, y);
			ptPtr = ptPtr->next;
		    } while ( ptPtr != &contourPtr->head);
		} 
		polyPtr = polyPtr->f;
	    } while ( polyPtr != clipPolyArea);
	    fprintf( pw->getFp(), "closepath clip\n");
	}
	clipRegionDirty = FALSE;
    } else if ( clipRectDirty) {

/* deal with the clip rectangle */


        fprintf( pw->getFp(), "initclip\n");

	for ( ii = 0; ii < nrects; ii++) {

            x = SCREEN_COORD( clipRectVec[ ii].x);
            dx = SCREEN_COORD_FLOAT( clipRectVec[ ii].x) - x;

            y = SCREEN_COORD( clipRectVec[ ii].y);
            dy = SCREEN_COORD_FLOAT( clipRectVec[ ii].y) - y;

            iWidth = (int)ceil( SCREEN_COORD_FLOAT( clipRectVec[ ii].width) + dx);
            iHeight = (int)ceil( SCREEN_COORD_FLOAT( clipRectVec[ ii].height) + dy);

            fprintf( pw->getFp(), "%d %d %d %d rectclip\n",
                     x, y, iWidth, iHeight);
	}

	clipRectDirty = FALSE;
    }

/* if the fill pattern is dirty, then set it */

    if ( fillPatDirty && ( _fillPatFile != NULL)) {
	fprintf( pw->getFp(), "[/Pattern [/DeviceRGB]] setcolorspace\n");
	strcpy( fillPatBuf, AmGetRootName( _fillPatFile));
	AmChopExtension( fillPatBuf);
	fprintf( pw->getFp(), "%.3f %.3f %.3f pattern%s setcolor\n", 
		 fg.red/255.0, fg.green/255.0, fg.blue/255.0,
		 fillPatBuf);
	fillPatDirty = FALSE;
    }

/* NOTE: Postscript doesn't support different raster operations like X does */

    if ( rasterOpDirty) {
	rasterOpDirty = FALSE;
    }
}

/* set and get the linecap */

void 
AmPostScriptGCClass::SetLineCap( AmLineCapEnum	lineCap)
{
}

AmLineCapEnum 
AmPostScriptGCClass::GetLineCap()
{
    return( AmCAP_ROUND);
}

void
AmPostScriptGCClass::Update()
{
}

static char hexchars[] = "0123456789abcdef";

AnvilBoolean
AmPostScriptGCClass::SetFillPattern( const char *fillPatFile)
{
    uint_t			width, height; 
    int				x_hot, y_hot, bytesPerLine, ii, jj;
    uchar_t			*data;
    AmPostScriptOutputClass	*pw = ( AmPostScriptOutputClass *) _pWindow;
    char			fillPatBuf[ PATHLEN], **fillPatRes;

/* free the last pattern file and stipple pixmap */

    if ( _fillPatFile != NULL) {
        FREE( _fillPatFile);
    }

/* search for the fill pattern in the cache */

    _fillPatFile = strdup( fillPatFile);
    fillPatRes = ( char **) tfind( fillPatFile,
				   ( const void **) &fillPatternRoot,
				   ( AmSearchFn) strcmp);
    if ( fillPatRes != NULL) {
	return( TRUE);
    }

/* insert the fill pattern into the cache */

    tsearch( strdup( fillPatFile), &fillPatternRoot, ( AmSearchFn) strcmp);

#ifdef sparc
    return( FALSE);
#else
    if ( XReadBitmapFileData( _fillPatFile, &width, &height, &data,
			      &x_hot, &y_hot) != BitmapSuccess) {
	return( FALSE);
    }
#endif
    strcpy( fillPatBuf, AmGetRootName( _fillPatFile));
    AmChopExtension( fillPatBuf);
    bytesPerLine = ( width + 7) >> 3;

/* set up the image mask procedure. */

    fprintf( pw->getFp(), "/%s\n", fillPatBuf);
    fprintf( pw->getFp(), "{\n");
    fprintf( pw->getFp(), "<<\n");
    fprintf( pw->getFp(), "/ImageType 1\n");
    fprintf( pw->getFp(), "/Width %d /Height %d\n", width, height);
    fprintf( pw->getFp(), "/BitsPerComponent 1\n");
    fprintf( pw->getFp(), "/ImageMatrix [1 0 0 1 0 0]\n");
    fprintf( pw->getFp(), "/Decode [0 1]\n");
    fprintf( pw->getFp(), "/DataSource\n");
    fprintf( pw->getFp(), "<");
    for ( ii = 0; (unsigned)ii < height; ii++) {
	for ( jj = 0; jj < bytesPerLine; jj++) {
	    fprintf( pw->getFp(), "%c%c", 
		     hexchars[( data[ ii*bytesPerLine + jj] & 0xf0) >> 4],
		    hexchars[( data[ ii*bytesPerLine + jj] & 0xf)]);
	}
    }
    fprintf( pw->getFp(), ">\n");
    fprintf( pw->getFp(), ">>\n");
    fprintf( pw->getFp(), "imagemask\n");
    fprintf( pw->getFp(), "} def\n");

/* set up the pattern */

    fprintf( pw->getFp(), "/pattern%s\n", fillPatBuf);
    fprintf( pw->getFp(), "{\n");
    fprintf( pw->getFp(), "<<\n");
    fprintf( pw->getFp(), "/PaintType 2\n");
    fprintf( pw->getFp(), "/PatternType 1 /TilingType 1\n");
    fprintf( pw->getFp(), "/BBox [0 0 %d %d]\n", width, height);
    fprintf( pw->getFp(), "/XStep %d /YStep %d\n", width, height);
    fprintf( pw->getFp(), "/PaintProc { %s }\n", fillPatBuf);
    fprintf( pw->getFp(), ">>\n");
    fprintf( pw->getFp(), "matrix\n");
    fprintf( pw->getFp(), "makepattern\n");
    fprintf( pw->getFp(), "} def\n");
    setFillPatternDirty();
    return( FALSE);
}

char *
AmPostScriptGCClass::GetFillPattern()
{
    return( _fillPatFile);
}

void
AmPostScriptGCClass::SetFillStyle( AmFillStyleEnum fillStyle)
{
    _fillStyle = fillStyle;
}

AmFillStyleEnum
AmPostScriptGCClass::GetFillStyle()
{
    return( _fillStyle);
}

