#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <anvil/macros.h>
#include <anvil/log.h>
#include <anvil/postscript.h>
#include <anvil/postscriptP.h>

/* create a postscript image */

AmPostScriptImageClass::AmPostScriptImageClass( AmDisplayClass  *pDisplay,
					    int           cxVirtual,
					    int           cyVirtual,
					    int           iDepth,
					    char          *pcData)
    : AmImageClass( pDisplay, cxVirtual, cyVirtual)
{
    int iPixelsPerByte, iRemainder;

/* Currently the only image type supported under postscript is
   a 32 bit image.  */

    _iDepth     = 32;
    _pcInternal = NULL;

/* Describe the internal image. */

    if ( iDepth == 1)  {

        _iDepth         = 1;
        _iBytesPerPixel = 1;
        _iBitsPerPixel  = _iBytesPerPixel * 8;
        _iBytesPerLine  = _iBytesPerPixel * _cxPixels;
        _iSize          = _iBytesPerLine * _cyPixels;

    } else {

        _iDepth           = 32;
        _iBytesPerPixel   = 4;
        _iBitsPerPixel    = _iBytesPerPixel * 8;
        _iBytesPerLine    = _iBytesPerPixel * _cxPixels;
        _iSize            = _iBytesPerLine * _cyPixels;

    }

/* Describe the device dependent image. */

    if ( _iDepth >= 8) {
 
        _iDDDepth         = _iDepth; 
        _iDDBytesPerPixel = ( _iDepth + 7) >> 3;
        _iDDBitsPerPixel  = _iDepth; 
        _iDDBytesPerLine  = ( _cxPixels * _iDepth) >> 3;
        _iDDSize          = _iDDBytesPerLine * _cyPixels;
    
    } else {

        _iDDDepth         = _iDepth;
        _iDDBytesPerPixel = ( _iDepth + 7) >> 3;
        _iDDBitsPerPixel  = _iDepth;

        iPixelsPerByte = 8 / _iDepth;
        iRemainder     = _cxPixels % iPixelsPerByte;
        if ( iRemainder == 0) {
            _iDDBytesPerLine = _cxPixels / iPixelsPerByte;
        } else {
            _iDDBytesPerLine = _cxPixels / iPixelsPerByte + 1;
        }

        _iDDSize          = _iDDBytesPerLine * _cyPixels;

    }
 
    _pcInternal = ( char *) malloc( _iSize);
    if ( _pcInternal == NULL) {
        AmAddResourceToLog( "mallocFailed", _iDDSize);
        ANVIL_ERROR_CODE = AmMALLOC_FAILED;
    }

    SetData( pcData);
}

/* Remove the temporary file for storing the image. */

AmPostScriptImageClass::~AmPostScriptImageClass()
{
    if ( _pcInternal != NULL) {
	free( _pcInternal);
    }
}

/* Write the image out to the store file in binary form. */

AnvilBoolean
AmPostScriptImageClass::SetData( char *pcData)
{
    if ( pcData != NULL) {
	memcpy( _pcInternal, pcData, _iSize);
    }

    return( TRUE);
}

/* Read the image data from the file and return it */

char *
AmPostScriptImageClass::AmGetData()
{
    return( _pcInternal);
}
