#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




/* PostScript region functions */

#include <malloc.h>
#if defined(__BSD44__)
#include <malloc.h>
#else
#include <alloca.h>
#endif
#include <anvil/postscript.h>

/* PostScript region class creation function */

AmPostScriptRegionClass::AmPostScriptRegionClass()
{
    polyRegion = NULL;
}

/* PostScript region class creation function */

AmPostScriptRegionClass::AmPostScriptRegionClass( POLYAREA *newPolyRegion)
{
    polyRegion = newPolyRegion;
}

/* PostScript region class virtual destructor */

AmPostScriptRegionClass::~AmPostScriptRegionClass()
{
    if ( polyRegion != NULL) {
	AmPolyFree( &polyRegion);
    }
}

AnvilBoolean
AmPostScriptRegionClass::PointInRegion( int	x,
				        int	y)
{
    return( FALSE);
}

AnvilBoolean
AmPostScriptRegionClass::EqualRegion( AmRegionClass	*otherRegion)
{
    POLYAREA *otherPolyRegion = 
		(( AmPostScriptRegionClass *) otherRegion)->getPolyRegion();
    return( AmEqualPolyArea( otherPolyRegion, polyRegion));
}

void 
AmPostScriptRegionClass::AddPolygon( AmPointPtr		pts,
				     int		n,
				     AmFillRuleEnum	fillRule)
{
    POLYAREA	*newPolyArea, *newPolyRegion;

    AmMakePolyArea( pts, n, &newPolyArea);
    if ( polyRegion != NULL) {
	AmPolyBoolean( polyRegion, newPolyArea, &newPolyRegion, PBO_UNITE);
	AmPolyFree( &polyRegion);
	polyRegion = newPolyRegion;
    } else {
	polyRegion = newPolyArea;
    }
}

void 
AmPostScriptRegionClass::AddRectangle( AmRectangleRec const	*rect)
{
    AmPointRec	pts[ 4];

    pts[ 0].x = rect->x;
    pts[ 0].y = rect->y;
    pts[ 1].x = rect->x + rect->width;
    pts[ 1].y = rect->y;
    pts[ 2].x = rect->x + rect->width;
    pts[ 2].y = rect->y + rect->height;
    pts[ 3].x = rect->x;
    pts[ 3].y = rect->y + rect->height;
    AddPolygon( pts, 4, AmFILL_EVEN_ODD);
}
    
void 
AmPostScriptRegionClass::UnionRegion( AmRegionClass	*otherRegion)
{
    POLYAREA	*otherPolyRegion = 
		    (( AmPostScriptRegionClass *) otherRegion)->getPolyRegion(),
		*newPolyRegion;

    if ( polyRegion != NULL) {
	AmPolyBoolean( polyRegion, otherPolyRegion, &newPolyRegion, PBO_UNITE);
	AmPolyFree( &polyRegion);
	polyRegion = newPolyRegion;
    }
}
    
void 
AmPostScriptRegionClass::IntersectRegion( AmRegionClass	*otherRegion)
{
    POLYAREA	*otherPolyRegion = 
		    (( AmPostScriptRegionClass *) otherRegion)->getPolyRegion(),
		*newPolyRegion;

    if ( polyRegion != NULL) {
	AmPolyBoolean( polyRegion, otherPolyRegion, &newPolyRegion, PBO_ISECT);
	AmPolyFree( &polyRegion);
	polyRegion = newPolyRegion;
    }
}
    
void 
AmPostScriptRegionClass::XorRegion( AmRegionClass	*otherRegion)
{
    POLYAREA	*otherPolyRegion = 
		    (( AmPostScriptRegionClass *) otherRegion)->getPolyRegion(),
		*newPolyRegion;

    if ( polyRegion != NULL) {
	AmPolyBoolean( polyRegion, otherPolyRegion, &newPolyRegion, PBO_XOR);
	AmPolyFree( &polyRegion);
	polyRegion = newPolyRegion;
    }
}
    
void 
AmPostScriptRegionClass::SubtractRegion( AmRegionClass	*otherRegion)
{
    POLYAREA	*otherPolyRegion = 
		    (( AmPostScriptRegionClass *) otherRegion)->getPolyRegion(),
		*newPolyRegion;

    if ( polyRegion != NULL) {
	AmPolyBoolean( polyRegion, otherPolyRegion, &newPolyRegion, PBO_SUB);
	AmPolyFree( &polyRegion);
	polyRegion = newPolyRegion;
    }
}

AmRegionClass *
AmPostScriptRegionClass::CopyRegion()
{
    POLYAREA	*dstPolyRegion;

    AmPolyM_Copy0( &dstPolyRegion, polyRegion);
    return(( AmRegionClass *) new AmPostScriptRegionClass( dstPolyRegion));
}
