#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#if defined(__BSD44__)
#include <malloc.h>
#else
#include <alloca.h>
#endif
#include <stdlib.h>
#include <sys/types.h>
#include <anvil/errcode.h>
#include <anvil/dither.h>
#include <anvil/postscript.h>
#include <anvil/postscriptcolor.h>
#include <anvil/postscriptP.h>

static char hexchars[] = "0123456789abcdef";

/* print an image to a file in hex format according to postscript format by
   changing the colormap to greyscale */

AnvilBoolean AmPrintPostScriptPseudoColorImage( ulong_t    		*srcImage,
					   int 	    		srcX,
					   int 	    		srcY,
					   int 	    		dstX,
					   int 	    		dstY,
					   int 	    		srcWidth,
					   int 	    		srcHeight,
					   int	    		imgWidth,
					   int	    		imgHeight,
					   int 	    	  	depth,
					   AmColorCellPtr  	colors,
					   int 	    		numColors,
					   FILE 	    	*fp)
{
    char	  	*printrow, *printptr;
    int           	ii, jj;
    AmColorCellPtr     	col;
    ulong_t	  	r, g, b, ra, ga, ba;
    ushort_t	  	xmod, ymod;
    uchar_t		*srcPtr;
    const ulong_t 	*redDitherMatrix = AmGetPostScriptRedDitherMatrix(),
			*greenDitherMatrix = AmGetPostScriptGreenDitherMatrix(),
			*blueDitherMatrix = AmGetPostScriptBlueDitherMatrix();

/* create an indexed color space for displaying the image */

    fprintf( fp, "[/Indexed /DeviceRGB %d\n", numColors - 1);
    fprintf( fp, "<\n");
    for( ii = 0; ii < numColors; ii++) {
	fprintf( fp, "%x",( colors[ ii].red & 0xf0) >> 4);
	fprintf( fp, "%x", colors[ ii].red & 0xf);
	fprintf( fp, "%x",( colors[ ii].green & 0xf0) >> 4);
	fprintf( fp, "%x", colors[ ii].green & 0xf);
	fprintf( fp, "%x",( colors[ ii].blue & 0xf0) >> 4);
	fprintf( fp, "%x", colors[ ii].blue & 0xf);
	fprintf( fp, "\n");
    }
    fprintf( fp, "> ] setcolorspace\n");

/* print the image dictionary */

    fprintf( fp, "/picstr %d string def\n", srcWidth);
    fprintf( fp, "<<\n");
    fprintf( fp, "/ImageType 1\n");
    fprintf( fp, "/Width %d /Height %d\n", srcWidth, srcHeight);
    fprintf( fp, "/BitsPerComponent 8\n");
    fprintf( fp, "/Decode [0 255]\n");
    fprintf( fp, "/ImageMatrix [ 1 0 0 1 %d %d]\n", -dstX, -dstY);
    fprintf( fp, "/DataSource { currentfile picstr readhexstring pop }\n");
    fprintf( fp, ">>\n");
    fprintf( fp, "image\n");

/* set to the start of the subimage that we're going to render */

    srcImage += srcY*imgWidth;
    
/* read and write the image a row at a time */

    if (( printrow = ( char *) alloca( srcWidth * 2)) == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return( FALSE);
    }
    ymod = 0;
    for ( ii = 0; ii < srcHeight; ii++, srcImage += imgWidth) {
	printptr = printrow;

/* start in the row at the start of the subimage */

	xmod = 0;
	for ( jj = 0, srcPtr = ( uchar_t *) &srcImage[ srcX]; 
	      jj < srcWidth; 
	      jj++, srcPtr += sizeof( ulong_t)) {

            r = redDitherMatrix[ srcPtr[ RED]];
            g = greenDitherMatrix[ srcPtr[ GREEN]];
            b = blueDitherMatrix[ srcPtr[ BLUE]];
            ra = (PostScriptDecisionMatrix[ SECONDWORD( r)][ ymod][ xmod]) ?
                     SECONDBYTE( r) : FIRSTBYTE( r);
            ga = (PostScriptDecisionMatrix[ SECONDWORD( g)][ ymod][ xmod]) ?
                     SECONDBYTE( g) : FIRSTBYTE( g);
            ba = (PostScriptDecisionMatrix[ SECONDWORD( b)][ ymod][ xmod]) ?
                     SECONDBYTE( b) : FIRSTBYTE( b);
            col = colors + ra + ga + ba; 

/* translate the image pixels to hex characters and print the row in a 
   single shot */

	    *printptr++ = hexchars[( col->pixel & 0xf0) >> 4];
	    *printptr++ = hexchars[ col->pixel & 0xf];
	    xmod++;
	    xmod &= 0x7;
	}
	fwrite( printrow, 1, srcWidth * 2, fp);
	fprintf( fp, "\n");
	ymod++;
	ymod &= 0x7;
    }
    return( TRUE);
}

/* print an image in 24-bit TrueColor */

AnvilBoolean AmPrintPostScriptTrueColorImage( ulong_t     *image,
				       int 	   srcX,
				       int 	   srcY,
				       int 	   dstX,
				       int 	   dstY,
				       int         srcWidth,
				       int         srcHeight,
				       int         imgWidth,
				       int         imgHeight,
				       int         depth,
				       FILE        *fp)
{
    ulong_t  	*srcPtr;
    uchar_t	*pixel;
    char     	*printrow, *printptr;
    int	  	ii, jj;

/* image header..presumes we have defined imagedraw function. y translation
   negative as inverse of moveto */
   
    fprintf( fp, "/DeviceRGB setcolorspace\n");
    fprintf( fp, "/picstr %d string def\n", srcWidth*3);
    fprintf( fp, "<<\n");
    fprintf( fp, "/ImageType 1\n");
    fprintf( fp, "/Width %d /Height %d\n", srcWidth, srcHeight);
    fprintf( fp, "/BitsPerComponent 8\n");
    fprintf( fp, "/Decode [0 1 0 1 0 1]\n");
    fprintf( fp, "/ImageMatrix [ 1 0 0 1 %d %d]\n", -dstX, -dstY);
    fprintf( fp, "/DataSource { currentfile picstr readhexstring pop }\n");
    fprintf( fp, ">>\n");
    fprintf( fp, "image\n");

/* 6 bytes are needed to print PostScript 24-bit in RRGGBB form. 
   the source image is stored as unsigned longs */

    if (( printrow = ( char *) alloca( srcWidth * 6)) == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return( FALSE);
    }
    image += srcY*imgWidth;
    for ( ii = 0; ii < srcHeight; ii++, image += imgWidth) {
	printptr = printrow;
	for ( jj = 0, srcPtr = &image[ srcX]; jj < srcWidth; jj++, srcPtr++) {

/* in the long, the values are stored as 0BGR */
	
	    pixel = ( uchar_t *) srcPtr;
	    *printptr++ = hexchars[( pixel[ RED] & 0xf0) >> 4];
	    *printptr++ = hexchars[ pixel[ RED] & 0xf];
	    *printptr++ = hexchars[( pixel[ GREEN] & 0xf0) >> 4];
	    *printptr++ = hexchars[ pixel[ GREEN] & 0xf];
	    *printptr++ = hexchars[( pixel[ BLUE] & 0xf0) >> 4];
	    *printptr++ = hexchars[ pixel[ BLUE] & 0xf];
	}
	fwrite( printrow, 1, srcWidth*6, fp);
	fprintf( fp, "\n");
    }
    return( TRUE);
}

static int roundit( int value, 
		    int denom)
{
    return(( value % denom == 0) ? value/denom : value/denom + 1);
}

static unsigned char mkmask( int nbits)
{
    return(( 0x1 << nbits) - 1);
}

/* print a mask image in 1-bit.  We're assuming that the image is padded to
   byte boundaries. */

AnvilBoolean AmPrintPostScriptMaskImage( uchar_t	*srcImage,
				  int 	       	srcX,
				  int 	       	srcY,
				  int 	       	dstX,
				  int 	       	dstY,
				  int         	srcWidth,
				  int         	srcHeight,
				  int         	imgWidth,
				  int         	imgHeight,
				  FILE        	*fp)
{
    unsigned char  *srcPtr, byte;
    char           *printrow, *printptr;
    int		   ii, jj, srclen, printlen, imgRowBytes;

/* printrow needs 1 byte per nibble, plus pad, source row is number of 
   bytes plus pad */

    printlen = roundit( srcWidth, 4);
    srclen = roundit( srcWidth, 8);
    imgRowBytes = roundit( imgWidth, 8);
    if (( printrow = ( char *) alloca( printlen)) == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return( FALSE);
    }

/* image header..presumes we have defined imagedraw function. y translation
   negative as inverse of moveto */
    
    fprintf( fp, "/picstr %d string def\n", printlen);
    fprintf( fp, "<<\n");
    fprintf( fp, "/ImageType 1\n");
    fprintf( fp, "/Width %d /Height %d\n", srcWidth, srcHeight);
    fprintf( fp, "/BitsPerComponent 1\n");
    fprintf( fp, "/Decode [1 0]\n");
    fprintf( fp, "/ImageMatrix [ 1 0 0 1 %d %d]\n", -dstX, -dstY);
    fprintf( fp, "/DataSource { currentfile picstr readhexstring pop }\n");
    fprintf( fp, ">>\n");
    fprintf( fp, "imagemask\n");

/* seek to the start of the source image */

    srcImage += srcY*imgWidth;
    for ( ii = 0; ii < srcHeight; ii++) {
	printptr = printrow;

/* get the byte that srcX is in */

	for ( jj = 0, srcPtr = &srcImage[ srcX/8 + ii*imgRowBytes];
	      jj < srclen;
	      jj++, srcPtr++) {

/* if we're in the AmFirst column, then mask the bits AmBetween srcX and the 
   end of the byte.  If we're in the AmLast column, mask the bits AmBetween
   srcX + width and the start of the byte, otherwise, take the entire byte */

	    if ( jj == 0) {
		byte = *srcPtr & mkmask( 8 - ( srcX % 8));
	    } else if ( jj == srclen - 1) {
		byte = *srcPtr & mkmask(( srcX + srcWidth) % 8);
	    } else {
		byte = *srcPtr;
	    }

/* in the long, the values are stored as 0BGR */

	    *printptr++ = hexchars[( byte & 0xf0) >> 4];
	    *printptr++ = hexchars[ byte & 0xf];
	}
	fwrite( printrow, 1, printlen, fp);
	fprintf( fp, "\n");
    }
    return( TRUE);
}
