#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <unistd.h>
#include <anvil/postscript.h>


void drawStuff( AmDisplayClass    *disp, 
		AmWindowClass  	*win,
		AmGCClass		*gc,
		AmImageClass	*image,
		AmImageClass	*mask)
{
    int			ii, jj, width, ascent,descent;
    char		*matrix, *maskmat;
    Point		pts[ 20];
    unsigned long	pixel, pixelVec[ 16];
    unsigned char	byte;

    matrix = image->AmGetData();
    maskmat = mask->AmGetData();
    for ( ii = 0; ii < 16; ii++) {
	disp->AllocatePixel( ii*16, ii*16, ii*16, &pixelVec[ ii]);
    }

/* generate the image and the mask */

    for ( ii = 0; ii < 128; ii++) {
	for ( jj = 0; jj < 128; jj++) {
	    if (( ii > 32) && ( ii < 96) && ( jj > 32) && ( jj < 96)) {
		matrix[ 128*ii + jj] = pixelVec[ (ii + jj)/16];
		byte = maskmat[ (128*ii)/8 + jj/8];
		byte |= ( 1 << (jj % 8));
	    } else {
		matrix[ 128*ii + jj] = 0;
		byte &= ~( 1 << ( jj % 8));
	    }
	    maskmat[ (128*ii)/8 + jj/8] = byte;
	}
    }
    image->SetData( matrix);
    mask->SetData( maskmat);
    disp->DrawImage( win, gc, 0, 0, 0, 0, 128, 128, image);
    disp->AllocatePixel( 0, 255, 0, &pixel);
    gc->SetForegroundPixel( pixel);
    disp->DrawImage( win, gc, 0, 0, 128, 0, 128, 128, mask);
    for ( ii = 0; ii < 8; ii++) {
	if ( ii == 0) {
	    disp->AllocatePixel( 255, 0, 0, &pixel);
	    gc->SetForegroundPixel( pixel);
	} else if ( ii == 1) {
	    gc->SetLineWidth( 4);
	} else if ( ii == 2) {
	    gc->SetLineJoin( JoinRoundE);
	} else if ( ii == 3) {
	    gc->SetLineStyle( LineOnOffDashE);
	    gc->SetDashes( "\2\2", 2, 0);
	} else if ( ii == 4) {
	    disp->AllocatePixel( 0, 0, 255, &pixel);
	    gc->SetBackgroundPixel( pixel);
	} else if ( ii == 5) {
	    gc->SetFont( "courier", 14);
	} else if ( ii == 6) {
	    gc->SetClipRectangle( 50, 50, 150, 150);
	} else if ( ii == 7) {
	    gc->SetRasterOp( xorE);
	}
		
	pts[ 0].x = 0;
	pts[ 0].y = 0;
	pts[ 1].x = 200;
	pts[ 1].y = 200;
	disp->DrawLines( win, gc, pts, 2);
	pts[ 0].x += 15;
	pts[ 0].y += 5;
	pts[ 1].y -= 5;
	pts[ 1].x -= 15;
	disp->DrawPoints( win, gc, pts, 2);
	disp->DrawArc( win, gc, 50, 50, 200, 200, 0.0, 4);
	disp->DrawText( win, gc, 120, 50, "this is a test");
	disp->DrawImageText( win, gc, 120, 70, "this is a test");
	disp->DrawRotatedText( win, gc, 50, 100, 1.00, "this is a test");
	disp->ExtentsText( gc, "Sproing!", &width, &ascent, &descent);
	printf( "width = %d, ascent = %d, descent = %d\n",
		width, ascent, descent);
	pts[ 0].x = 100;
	pts[ 0].y = 0;
	pts[ 1].x = 120;
	pts[ 1].y = 50;
	pts[ 2].x = 100;
	pts[ 2].y = 100;
	pts[ 3].x = 80;
	pts[ 3].y = 50;
	disp->FillPolygon( win, gc, pts, 4);
	disp->FillRectangle( win, gc, 110, 20, 100, 40);
	disp->DrawRectangle( win, gc, 150, 40, 100, 40);
    }
}

int main( int argc, char *argv[])
{
    AmPostScriptClass     	*disp;
    AmPostScriptOutputClass       *output;
    AmPostScriptGCClass           *gc;
    AmPostScriptImageClass        *image, *mask;
    FILE			*fp = fopen( argv[ 1], "w");

    disp = new AmPostScriptClass( 8, PseudoColorE);
    output = new AmPostScriptOutputClass( fp, 612, 792, 72, FALSE, NULL);
    gc = new AmPostScriptGCClass( disp);
    image = new AmPostScriptImageClass( disp, 128, 128, 8, NULL);
    mask = new AmPostScriptImageClass( disp, 128, 128, 1, NULL);
    drawStuff( disp, output, gc, image, mask);
    delete output;
}
