#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <anvil/screenobj.h>
#include <anvil/datagraphic.h>
#include <anvil/overlay.h>
#include <anvil/layerstrs.h>
#include <anvil/motif.h>
#include <anvil/log.h>

#include <anvil/DeleteLayerDialogClass.h>

/*
 * Function: AmDeleteLayerDialogClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmDeleteLayerDialogClass::initializeValues()
{
    _eObjectType    = AmANY_OBJECT;
    _wArea          = _pMotifDisplay->GetArea();
    _fDeleteOverlay = TRUE;
}


/*
 * Function: AmDeleteLayerDialogClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets for the layer selection dialog.
 */

void
AmDeleteLayerDialogClass::createInterface()
{
    _pwcLayerList->SetExtendedSelect();
}


/*
 * Function: AmDeleteLayerDialogClass::AmDeleteLayerDialogClass()
 *
 * Purpose:
 *   Default constructor.
 */

AmDeleteLayerDialogClass::AmDeleteLayerDialogClass()
    : AmPickLayerDialogClass()
{
    initializeValues();
}


/*
 * Functions: AmDeleteLayerDialogClass::AmDeleteLayerDialogClass( wParent,
 *                                                                szName)
 *            AmDeleteLayerDialogClass::AmDeleteLayerDialogClass( bwcParent,
 *                                                                szName)
 *
 * Purpose:
 *   Constructor for the AmDeleteLayerDialogClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters: 
 *   Widget          wParent    : The parent of the dialog.
 *   BaseWidgetclass *bwcParent : The "wrapped" parent of the dialog.
 *   const char      *szName    : The name of the dialog.
 */

AmDeleteLayerDialogClass::AmDeleteLayerDialogClass(
                              Widget              wParent,
                              const char          *szName,
                              AmOverlayClass      *pOverlay,
                              AmMotifDisplayClass *pMotifDisplay)
    : AmPickLayerDialogClass( wParent, szName, pOverlay)
{
    _pMotifDisplay = pMotifDisplay;

    initializeValues();
    createInterface();
}


/* See above comment. */

AmDeleteLayerDialogClass::AmDeleteLayerDialogClass(
                              AmBaseWidgetClass   *bwcParent,
                              const char          *szName,
                              AmOverlayClass      *pOverlay,
                              AmMotifDisplayClass *pMotifDisplay)
    : AmPickLayerDialogClass( bwcParent, szName, pOverlay)
{
    _pMotifDisplay = pMotifDisplay;
 
    initializeValues();
    createInterface();
}


/*
 * Function: AmDeleteLayerDialogClass::~AmDeleteLayerDialogClass()
 *
 * Purpose:
 *   Destructor for the AmDeleteLayerDialogClass.
 */

AmDeleteLayerDialogClass::~AmDeleteLayerDialogClass()
{
}


/*
 * Function: AmDeleteLayerDialogClass::DeleteOverlay( fDeleteOverlay)
 *
 * Purpose:
 *   To set the delete overlay flag.  This flag indicates
 *   whether or not the overlay is supposed to deleted when
 *   all the layers in the overlay have been deleted.
 *   It does not override the user selecting the overlay
 *   to be delted (only possible if fShowOverlay is TRUE).
 *
 * Paramters:
 *   AnvilBoolean fDeleteOverlay : Indicates whether or not the overlay is
 *                            supposed to be deleted when all layers in
 *                            the overlay have been removed.
 */

void
AmDeleteLayerDialogClass::DeleteOverlay( AnvilBoolean fDeleteOverlay)
{
    _fDeleteOverlay = fDeleteOverlay;
}


/* ARGSUSED */
void
AmDeleteLayerDialogClass::action( Widget    w,
                                  XtPointer xpCallData)
{
    int     *viPos, nPos, ii, iPos;
    AnvilBoolean fDeleteOverlay;

    AmRemoveCallback( AM_PKG_ID, AM_DELETE_OBJECT, 
                      ( callbackProc) AmObjectDeletedCallback,
                      ( XtPointer) this);

    AmRemoveCallback( AM_PKG_ID, AM_DELETE_LAYER, 
                      ( callbackProc) AmLayerDeletedCallback,
                      ( XtPointer) this);

    Hide();

/* Get the selected layers from the list. */

    _pwcLayerList->GetSelectedPositions( &viPos, &nPos);

    if ( nPos > 0) {

        fDeleteOverlay = FALSE;

    /* Check if one of the layers selected was the overlay. */ 

        if ( _fShowOverlay == TRUE) {
            for ( ii = 0; ii < nPos; ii++) {
                iPos = viPos[ ii] - 1;
                if ( iPos == _nLayerPos) {
                    fDeleteOverlay = TRUE;  
                }
            }
        }

    /* Check if all layers were removed, if so and if the delete overlay 
       flag is set, remove the overlay. */

        if (( _fDeleteOverlay == TRUE) && ( nPos == _nDataGraphic)) {
            fDeleteOverlay = TRUE;  
        }

        if ( fDeleteOverlay == FALSE) {
            for ( ii = 0; ii < nPos; ii++) {
                iPos = viPos[ ii] - 1;
                _pOverlay->RemoveLayer( _vpDataGraphic[ _viLayerPos[ iPos]]);
                delete _vpDataGraphic[ _viLayerPos[ iPos]];
            }
        } else {
            _pMotifDisplay->RemoveObject( _pOverlay);
        }

        _pMotifDisplay->Flush();
    }

    XtFree(( char *) viPos);

    delete this;
}
