#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <anvil/screenobj.h>
#include <anvil/datagraphic.h>
#include <anvil/overlay.h>
#include <anvil/layerstrs.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/log.h>
#include <anvil/editobject.h>

#include <anvil/EditLayerDialogClass.h>

/*
 * Function: AmEditLayerDialogClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmEditLayerDialogClass::initializeValues()
{
    _eObjectType   = AmANY_OBJECT;
    _fShowOverlay  = TRUE;

    _wArea         = _pMotifDisplay->GetArea();
}


/*
 * Function: AmEditLayerDialogClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets for the layer selection dialog.
 */

void
AmEditLayerDialogClass::createInterface()
{
    _pwcLayerList->SetBrowseSelect();

    updateLayerList();
}


/*
 * Function: AmEditLayerDialogClass::AmEditLayerDialogClass()
 *
 * Purpose:
 *   Default constructor.
 */

AmEditLayerDialogClass::AmEditLayerDialogClass()
    : AmPickLayerDialogClass()
{
}


/*
 * Functions: AmEditLayerDialogClass::AmEditLayerDialogClass( wParent, szName,
 *                                                            pOverlay,
 *                                                            pMotifDisplay)
 *            AmEditLayerDialogClass::AmEditLayerDialogClass( bwcParent, szName,
 *                                                            pOverlay,
 *                                                            pMotifDisplay)
 *
 * Purpose:
 *   Constructor for the AmEditLayerDialogClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters: 
 *   Widget            wParent        : The parent of the dialog.
 *   BaseWidgetclass   *bwcParent     : The "wrapped" parent of the dialog.
 *   const char        *szName        : The name of the dialog.
 *   AmOverlayClass      *pOverlay      : The overlay class containing the layer
 *                                      to edit.
 *   AmMotifDisplayClass *pMotifDisplay :
 */

AmEditLayerDialogClass::AmEditLayerDialogClass(
                            Widget            wParent,
                            const char          *szName,
                            AmOverlayClass      *pOverlay,
                            AmMotifDisplayClass *pMotifDisplay)
    : AmPickLayerDialogClass( wParent, szName, pOverlay)
{
    _pMotifDisplay = pMotifDisplay;

    initializeValues();
    createInterface();
}


/* See above comment. */

AmEditLayerDialogClass::AmEditLayerDialogClass(
                            AmBaseWidgetClass   *bwcParent,
                            const char          *szName,
                            AmOverlayClass      *pOverlay,
                            AmMotifDisplayClass *pMotifDisplay)
    : AmPickLayerDialogClass( bwcParent, szName, pOverlay)
{
    _pMotifDisplay = pMotifDisplay;

    initializeValues();
    createInterface();
}


/*
 * Function: AmEditLayerDialogClass::~AmEditLayerDialogClass()
 *
 * Purpose:
 *   Destructor for the AmEditLayerDialogClass.
 */

AmEditLayerDialogClass::~AmEditLayerDialogClass()
{
}


/* ARGSUSED */
void
AmEditLayerDialogClass::action( Widget    w,
                                XtPointer xpCallData)
{
    int            *viPos, nPos;
    AmScreenObjClass *pScreenObj;

    Hide();

/* Get the selected layers from the list. */

    _pwcLayerList->GetSelectedPositions( &viPos, &nPos);

    if ( nPos > 0) {
        if (( viPos[ 0] - 1) == _nLayerPos) {
            pScreenObj = _pOverlay;
        } else {
            pScreenObj = _vpDataGraphic[ viPos[ 0] - 1];
        }

        if ( pScreenObj != NULL) {

            AmInitLog();
            if ( AmEditScreenObject( _wArea, _pMotifDisplay,
                                     pScreenObj, NULL) != AmOK) {
                AmPrintLogError( AmGetWidgetRealShell( _wArea),
                                 "editObjectFailed");
            }

        }
    }

    XtFree(( char *) viPos);

    delete this;
}
