#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdarg.h>
#include <malloc.h>
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Scale.h>
#include <Xm/DrawingA.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/errlib.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/va_cback.h>
#include <anvil/envdefs.h>
#include <anvil/plot_cback.h>
#include <anvil/legend.h>
#include <anvil/linelegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/polylegend.h>
#include <anvil/imagelegend.h>
#include <anvil/plinelegend.h>
#include <anvil/resource.h>
#include <anvil/interp.h>
#include <anvil/userevents.h>
#include <anvil/slick.h>
#include <anvil/slickP.h>
#include <anvil/utility.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>
#include <anvil/colorlegendP.h>

/* action area definition for the colormap editor dialog */

static AmActionDefRec actionAreaDef[] = {
    { "okBtn", NULL, NULL },
    { "closeBtn", ( XtCallbackProc) AmPopdownCback, NULL },
    { "helpBtn", NULL, NULL }
};

/*************************************************************************
 *
 * Description:
 * 	create a standalone legend based on a plot
 * Arguments:
 *	legendDisplay - display manager showing the legend.
 *	plot - plot to create the legend from
 * Returns:
 *	the legend, or NULL if the plot type is not supported.
 *
 **************************************************************************/

AmLegendClass *
AmMakeStandaloneLegend( AmMotifDisplayClass	*legendDisplay,
			AmPlotClass		*plot)
{
    AmColorCellPtr	colors;
    int			numColors, *table;
    float		*binvec, min, max;
    char		**strings;
    Widget		cmapArea;
    AmXWindowClass	*xwin;
    Dimension		width, height;
    AmLegendClass	*legend;

/* pull the window and widget from the legend display to get the dimensions */

    xwin = ( AmXWindowClass *) legendDisplay->GetWindow();
    cmapArea = legendDisplay->GetArea();
    XtVaGetValues( cmapArea,
		   XmNwidth, &width,
		   XmNheight, &height,
		   NULL);

/* Get the fields that are important for any kind of legend. */

    plot->GetVaFields( AmNcolors, &colors,
                       AmNnumColors, &numColors,
                       AmNcolorsBinvec, &binvec,
                       AmNcolorsTable, &table,
                       AmNcolorsStrings, &strings,
                       AmNmin, &min,
                       AmNmax, &max,
                       NULL);

/* create the legend depending on the plot type */

    if ( plot->IsType( AmIMAGE_PLOT)) {
	legend = new AmImageLegendClass( xwin, NULL,
					 "imagePlotClass",
					 AmNlegendType, AmLEGEND_CONTINUOUS,
					 AmNnumColors, numColors,
					 AmNcolors, colors,
					 AmNmin, min,
					 AmNmax, max,
					 AmNcolorsBinvec, binvec,
					 AmNcolorsTable, table,
					 AmNcolorsStrings, strings,
					 AmNitemSpacing, 5,
					 AmNfontSize, 10,
					 AmNshowTitle, TRUE,
					 AmNtitle, "Colors",
					 AmNtitleFontSize, 16,
					 AmNtitleFontWeight, AmWEIGHT_BOLD,
					 AmNtitleOffset, 5,
					 AmNeditable, TRUE,
					 AmNshowColors, TRUE,
					 AmNoutline, TRUE,
					 AmNitemWidth, 20,
					 AmNitemHeight, 20,
					 AmNmainItemHeight, height - 90,
					 AmNx, 20,
					 AmNy, 20,
					 AmNleftMargin, 20,
					 AmNrightMargin, 20,
					 AmNtopMargin, 20,
					 AmNbottomMargin, 20,
					 NULL);
    } else if ( plot->IsType( AmLINE_PLOT)) {
	legend = new AmLineLegendClass( xwin, NULL,
					"linePlotClass",
					AmNnumColors, numColors,
					AmNcolors, colors,
					AmNmin, min,
					AmNmax, max,
					AmNcolorsBinvec, binvec,
					AmNcolorsTable, table,
					AmNcolorsStrings, strings,
					AmNitemSpacing, 5,
					AmNfontSize, 10,
					AmNshowTitle, TRUE,
					AmNtitle, "Colors",
					AmNtitleFontSize, 16,
					AmNtitleFontWeight, AmWEIGHT_BOLD,
					AmNtitleOffset, 5,
					AmNeditable, TRUE,
					AmNshowColors, TRUE,
					AmNoutline, TRUE,
					AmNitemWidth, 20,
					AmNitemHeight, 20,
					AmNnumRows, ( height + 5) / 25,
					AmNx, 20,
					AmNy, 20,
					AmNleftMargin, 20,
					AmNrightMargin, 20,
					AmNtopMargin, 20,
					AmNbottomMargin, 20,
					NULL);
    } else if ( plot->IsType( AmPOLYLINE_PLOT)) {
	legend = new AmPolyLineLegendClass( xwin, NULL,
					    "plinePlotClass",
					    AmNnumColors, numColors,
					    AmNcolors, colors,
					    AmNmin, min,
					    AmNmax, max,
					    AmNcolorsBinvec, binvec,
					    AmNcolorsTable, table,
					    AmNcolorsStrings, strings,
					    AmNitemSpacing, 5,
					    AmNfontSize, 10,
					    AmNshowTitle, TRUE,
					    AmNtitle, "Colors",
					    AmNtitleFontSize, 16,
					    AmNtitleFontWeight, AmWEIGHT_BOLD,
					    AmNtitleOffset, 5,
					    AmNeditable, TRUE,
					    AmNshowColors, TRUE,
					    AmNoutline, TRUE,
					    AmNitemWidth, 20,
					    AmNitemHeight, 20,
					    AmNnumRows, ( height + 5) / 25,
					    AmNx, 20,
					    AmNy, 20,
					    AmNleftMargin, 20,
					    AmNrightMargin, 20,
					    AmNtopMargin, 20,
					    AmNbottomMargin, 20,
					    NULL);
    } else if ( plot->IsType( AmPOLYGON_PLOT)) {
	legend = new AmPolyLegendClass( xwin, NULL,
					"polygonPlotClass",
					AmNnumColors, numColors,
					AmNcolors, colors,
					AmNmin, min,
					AmNmax, max,
					AmNcolorsBinvec, binvec,
					AmNcolorsTable, table,
					AmNcolorsStrings, strings,
					AmNitemSpacing, 5,
					AmNfontSize, 10,
					AmNshowTitle, TRUE,
					AmNtitle, "Colors",
					AmNtitleFontSize, 16,
					AmNtitleFontWeight, AmWEIGHT_BOLD,
					AmNtitleOffset, 5,
					AmNeditable, TRUE,
					AmNshowColors, TRUE,
					AmNoutline, TRUE,
					AmNitemWidth, 20,
					AmNitemHeight, 20,
					AmNnumRows, ( height + 5) / 25,
					AmNx, 20,
					AmNy, 20,
					AmNleftMargin, 20,
					AmNrightMargin, 20,
					AmNtopMargin, 20,
					AmNbottomMargin, 20,
					NULL);
    } else if ( plot->IsType( AmSCATTER_PLOT)) {
	legend = new AmScatterLegendClass( xwin, NULL,
					   "scatterPlotClass",
					   AmNnumColors, numColors,
					   AmNcolors, colors,
					   AmNmin, min,
					   AmNmax, max,
					   AmNcolorsBinvec, binvec,
					   AmNcolorsTable, table,
					   AmNcolorsStrings, strings,
					   AmNitemSpacing, 5,
					   AmNfontSize, 10,
					   AmNshowTitle, TRUE,
					   AmNtitle, "Colors",
					   AmNtitleFontSize, 16,
					   AmNtitleFontWeight, AmWEIGHT_BOLD,
					   AmNtitleOffset, 5,
					   AmNeditable, TRUE,
					   AmNshowColors, TRUE,
					   AmNoutline, TRUE,
					   AmNitemWidth, 20,
					   AmNitemHeight, 20,
					   AmNnumRows, ( height + 5) / 25,
					   AmNx, 20,
					   AmNy, 20,
					   AmNleftMargin, 20,
					   AmNrightMargin, 20,
					   AmNtopMargin, 20,
					   AmNbottomMargin, 20,
					   NULL);
    } else {
	return( NULL);
    }

/* free the vectors retrieved from the plot */

    if ( binvec != NULL) { 
        free( binvec);
    }
    if ( table != NULL) {
        free( table);
    }
    if ( strings != NULL) {
        AmFreeStrVec( strings);
    }
    if ( colors != NULL) {
	free( colors);
    }
    return( legend);
}


/*************************************************************************
 *
 * Description:
 * 	callback to change the number of colors in the colormap.  This 
 *	is called when the scale value is changed.
 * Arguments:
 *	legendDisplay - display manager containing the legend in the
 *	colormap editor.
 *	legend - legend displayed in the colormap editor.
 *	plot - plot which is having its colormap changed.
 *	colormapFile - currently selected colormap file
 *	callData - unused 
 *
 **************************************************************************/

/* ARGSUSED */

static void 
changeNumColorsCallback( Widget			scl,
			 AmMotifDisplayClass	*legendDisplay,
			 AmLegendClass		*legend,
			 AmPlotClass		*plot,
			 char			**colormapFile,
			 XtPointer		callData)
{
    int			numColors, newNumColors, oldNumColors, *table; 
    float		*binvec, *binvec2;
    AmColorCellPtr	currColors, newColors;
    char		*cmapPath, relPath[ PATHLEN];

/* find out the new number of colors, the current number colors, current 
   colors, bin vector and table */

    XmScaleGetValue( scl, &newNumColors);
    legend->GetVaFields( AmNtable, &table,
			 AmNbinvec, &binvec,
			 AmNnumColors, &oldNumColors,
		         NULL);
AN_DEBUG( 70, ("legend fields returned = %p %p %d\n", 
		table, binvec, oldNumColors));

/* read the colors from the color file, then interpolate to them, if there's
   a color file.  If not, get the colors from the plot (because we haven't
   selected a color file), and interpolate them */

    if ( *colormapFile != NULL) { 
	sprintf( relPath, "%s/%s", AM_COLORMAP_DIR, *colormapFile);
	cmapPath = AmGetEnvFile( CONFIG_ENV, relPath);
	if ( cmapPath == NULL) {
	    AmPrintResourceError( "cantFindColormapFile", *colormapFile);
	    return;
	}
	if ( !AmReadColors( cmapPath, &numColors, &currColors)) {
	    AmPrintResourceError( "cantReadColormapFile", cmapPath);
	    return;
	}
    } else {
	plot->GetVaFields( AmNnumColors, &numColors,
			   AmNcolors, &currColors,
			   NULL);
    }

/* interpolate the old colors to the new colors */

    AmInterpolateColorCells( currColors, numColors, &newColors, newNumColors);
    free( currColors);

/* if its a table, we're not sure what to do */

    if ( table != NULL) {
    } else if ( binvec != NULL) {	

AN_DEBUG( 80, ("interpolating bin vector\n"));
/* if binvec, then interpolate it to the new number of colors.  Note that a
   binvector has one less element than the colors vector */

	binvec2 = ( float *) malloc( sizeof( float)*newNumColors);
	AmInterpolateVec( binvec, oldNumColors - 1, binvec2, newNumColors - 1);
AN_DEBUG( 80, ("setting bin vector\n"));
	legend->SetVaFields( AmNnumColors, newNumColors,
			     AmNcolors, newColors,
			     AmNbinvec, binvec2,
			     NULL);
AN_DEBUG( 80, ("done setting bin vector\n"));
	free( binvec2);
	free( binvec);
AN_DEBUG( 80, ("freeing bin vector\n"));
    } else {

/* min, max, just reset the colors */

	legend->SetVaFields( AmNnumColors, newNumColors,
			     AmNcolors, newColors,
			     NULL);
    }
    free( newColors);

/* clear out the window to redraw the legend */

    legendDisplay->Flush();
}

/*************************************************************************
 *
 * Description:
 * 	change the colormap in the legend when the user picks a new colormap.
 * Arguments:
 *	legendDisplay - display manager showing the legend.
 *	legend - legend to display.
 *	colormapFile - currently selected file (retrieved from list)
 *	cb - XmList callback struct
 *
 **************************************************************************/

/* ARGSUSED */

static void 
changeColormapCallback( Widget			w,
			AmMotifDisplayClass	*legendDisplay,
			AmLegendClass		*legend,
			char			**colormapFile,
			XmListCallbackStruct	*cb)
{
    char 		*filename, *filepath, relPath[ PATHLEN];
    AmColorCellPtr	fileColors, newColors;
    int			numFileColors, numColors;

    XmStringGetLtoR( cb->item, XmFONTLIST_DEFAULT_TAG, &filename);
    sprintf( relPath, "%s/%s", AM_COLORMAP_DIR, filename);
    filepath = AmGetEnvFile( CONFIG_ENV, relPath);
    if ( filepath == NULL) {
	AmPrintResourceError( "cantFindColormapFile", relPath);
	return;
    }

/* save the colormap file name */

    if ( *colormapFile != NULL) {
	free( *colormapFile);
    }

/* to get rid of a FMM error from purify... */

    *colormapFile = strdup( filename);
    XtFree( filename);
    legend->GetVaFields( AmNnumColors, &numColors, NULL);

/* read the colors from the file and interpolate them to the number of 
   colors currently in the legend */

    if ( !AmReadColors( filepath, &numFileColors, &fileColors)) {
	AmPrintResourceError( "cantReadColormapFile", filepath);
	return;
    }
    AmInterpolateColorCells( fileColors, numFileColors, &newColors, numColors);
    legend->SetVaFields( AmNcolors, newColors, NULL);
    legendDisplay->Flush();
    free( newColors);
    free( fileColors);
}

/*************************************************************************
 *
 * Description:
 *	User event handler for handling the OK event from the colormap 
 *	selection/# colors dialog.  It extracts the colors from the 
 *	legend displayed in the dialog and sets them in the plot.
 * Arguments:
 *	eventId - OK event ID.
 *	formDlog - form Dialog from the colormap editor.
 *	legend - legend display in the colormap editor.
 *	colormapFile - pointer to the file resulting from the colormap
 *	file selection list 
 *	okArgs - slick okArgs used to set the values in the plot.
 *
 *************************************************************************/

/* ARGSUSED */

static void 
setColormapDialogOkCallback( int		eventId,
			     Widget		formDlog,
			     AmLegendClass	*legend,
			     char		**colormapFile,
			     AmOkArgsPtr	okArgs)
{
    int			*table, numLegendColors;
    float		*binvec;
    AmColorCellPtr	colors;

/* blow away the form dialog, and get the fields from the legend.  
   NOTE: The table, binvec, and colors retrieval causes a memory leak. */

    legend->GetVaFields( AmNnumColors, &numLegendColors,
			 AmNcolorsTable, &table,
			 AmNcolorsBinvec, &binvec,
			 AmNcolors, &colors,
			 NULL);
    XtDestroyWidget( XtParent( formDlog));

/* if its a table, we just set the colormap, because the number of colors
   has to remain exactly the same */

    if ( table != NULL) {
	free( table);
	if ( *colormapFile != NULL) {
	    AmSetTagValue( *colormapFile, AmSTRING, AmNcolormap, okArgs);
	} else {
	    AmSetTagValue( colors, AmCOLOR_CELL_PTR, AmNcolors, okArgs);
	}
    } else if ( binvec != NULL) {	

/* if the colormap filename has been set, then set that in the plot, 
   otherwise, just copy the colors */

	if ( *colormapFile != NULL) {
	    AmSetTagValue( *colormapFile, AmSTRING, AmNcolormap, okArgs);
	} else {
	    AmSetTagValue( colors, AmCOLOR_CELL_PTR, AmNcolors, okArgs);
	}
	AmSetTagValue( binvec, AmFLOAT_PTR, AmNbinvec, okArgs);
	AmSetTagValue(( void *) &numLegendColors, AmINT, 
		    AmNnumColors, okArgs);
    } else {

/* just set either the colormap file or the color vector */

	if ( *colormapFile != NULL) {
	    AmSetTagValue( *colormapFile, AmSTRING, AmNcolormap, okArgs);
	    AmSetTagValue(( void *) &numLegendColors, AmINT, 
			AmNnumColors, okArgs);
	} else {
	    AmSetTagValue( colors, AmCOLOR_CELL_PTR, AmNcolors, okArgs);
	    AmSetTagValue(( void *) &numLegendColors, AmINT, 
			AmNnumColors, okArgs);
	}
    }

/* set up destroy callbacks to free the data pulled from the legend. */

    if ( table != NULL) {
	XtAddCallback( formDlog, XmNdestroyCallback,
		       ( XtCallbackProc) AmFreeStructPtrCallback, table);
    }
    if ( binvec != NULL) {
	XtAddCallback( formDlog, XmNdestroyCallback,
		       ( XtCallbackProc) AmFreeStructPtrCallback, binvec);
    }
#if 0
    if ( colors != NULL) {
	XtAddCallback( formDlog, XmNdestroyCallback,
		       ( XtCallbackProc) AmFreeStructPtrCallback, colors);
    }
#endif

/* and the colormap file */

    if ( *colormapFile != NULL) {
	XtAddCallback( formDlog, XmNdestroyCallback,
		       ( XtCallbackProc) AmFreeStructPtrCallback, *colormapFile);
    }
}

/*************************************************************************
 *
 * Description:
 * 	create a colormap editor dialog.  This creates a modal dialog 
 *	which allows the user to select a colormap and the number of 
 *	colors.  The colormap can be selected from a list, on the left
 *	hand side of the dialog.  The number of colors is set using a 
 *	scale on the left side.  The colormap is displayed for the plot
 *	using a legend.  This supports image, polygon, line and scatter
 *	plots.
 * Arguments:
 *  	parent -  is the parent shell.
 * 	plot - is the plot whose colormap to edit
 * 	dialog - returns the created dialog 
 *	okBtn - returns the OK button to tie callbacks to 
 * Modifications:
 *	the colormapFile allocation causes a memory leak.
 *
 **************************************************************************/

void 
AmCreateColormapDialog( Widget		parent,
		        AmPlotClass	*plot,
		        Widget		*dialog,
		        Widget		*okBtn)
{
    AmActionRec         actionArea;
    Widget		formDlog, numColorsScl, cmapArea, cmapList, actionForm;
    Arg			args[ 20];
    char		**cmapNameList, *cmapListFile, **colormapFile,
			relPath[ PATHLEN];
    XmString		*motifStrings;
    int			numStrings, n, numColors, *table, cmapIndex=0;
    Dimension		width, height;
    AmMotifDisplayClass	*legendDisplay;
    AmLegendClass	*legend;
    AmXDisplayClass	*xdisp;
    AmXWindowClass	*xwin;
    float               scale;
    short               legendWidth, legendHeight;

/* also a failure flag */

    *dialog = NULL;

/* get the list of non-linear numeric mappings */

    sprintf( relPath, "%s/%s", AM_COLORMAP_DIR, AM_CMAP_LIST_FILE);
    cmapListFile = AmGetEnvFile( CONFIG_ENV, relPath);
    if ( cmapListFile == NULL) {
	AmPrintResourceError( "colormapListOpenFailed", relPath);
	return;
    }
    cmapNameList = AmReadLines( cmapListFile); 
    if ( cmapNameList == NULL) {
	AmPrintResourceError( "colormapListReadFailed", cmapListFile);
	return;
    }

/* get the pertinent fields from the plot we want to set */

    colormapFile = ( char **) malloc( sizeof( char *));
    plot->GetVaFields( AmNcolormap, colormapFile,
		       AmNtable, &table,
		       AmNnumColors, &numColors,
		       NULL);
    if ( numColors == 0) {
	AmPrintResourceError( "noColormap");
	return;
    }

/* create a form dialog to contain everything */

    n = AmMakeArgList( args, 
		       XmNdeleteResponse, XmDESTROY,
		       XmNvisual, AmGetAppVisual(),
		       XmNdepth, AmGetAppDepth(),
		       XmNcolormap, AmGetAppColormap(),
		       XmNautoUnmanage, FALSE,
		       XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		       NULL);
    formDlog = XmCreateFormDialog( parent, "colorSel", args, n);
    *dialog = formDlog;

/* Create the action area thingy. */

    actionAreaDef[ 1].clientData = formDlog;
    actionForm = AmMakeActionArea( formDlog,
				   actionAreaDef, ELEMENTS( actionAreaDef),
				   &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);
    XtSetSensitive( actionArea.buttons[ 2], False);



/* put the scale, area, and list in the form.  If the colormap is table 
   based, then we can't change the number of colors */

    numColorsScl = XtVaCreateManagedWidget( "numColorsScl", xmScaleWidgetClass, 
					    formDlog,
					    StickLeft, StickTop, 
					    AboveOf( actionForm),
					    XmNshowValue, TRUE,
					    XmNminimum, 1,
					    XmNmaximum, 150,
					    XmNvalue, numColors,
					    XmNsensitive, ( table == NULL),
					    NULL);
    if ( table != NULL) {
	free( table);
    }
    cmapArea = XtVaCreateManagedWidget( "cmapArea", 
					xmDrawingAreaWidgetClass, 
					formDlog,
					RightOf( numColorsScl),
					StickTop, AboveOf( actionForm),
					XmNwidth, 300,
					XmNheight, 400,
					NULL);

/* the list contains the list of available colormaps */

    motifStrings = AmMotifStringList( cmapNameList, &numStrings);
    n = AmMakeArgList( args,	
		     XmNitemCount, numStrings,
		     XmNitems, motifStrings,
		     RightOf( cmapArea),
		     StickTop,
		     StickRight,
		     AboveOf( actionForm),
		     XmNselectionPolicy, XmSINGLE_SELECT,
#if 0
		     XmNvisibleItemCount, 12,
#endif
		     XmNscrollBarDisplayPolicy, XmSTATIC,
		     NULL);
    cmapList = XmCreateScrolledList( formDlog, "cmapList", args, n);
    XtManageChild( cmapList);

/* set the index of the colormap in the colormap list */

    cmapIndex = AmIndexStrvec( (const char *)(*colormapFile), (const char **)cmapNameList);
    if ( cmapIndex >= 0) {
	XmListSelectPos( cmapList, cmapIndex + 1, FALSE);
    }
    AmFreeMotifStringList( motifStrings, numStrings);
    AmFreeStrVec( cmapNameList);

/* create a legend based on the plot type. Make it editable so we can
   change the colors. */

    XtVaGetValues( cmapArea,
		   XmNwidth, &width,
		   XmNheight, &height,
		   NULL);

    xdisp = new AmXDisplayClass( *dialog);
    xwin = new AmXWindowClass( xdisp, cmapArea);
    legendDisplay = new AmMotifDisplayClass( xdisp, xwin);
    legend = AmMakeStandaloneLegend( legendDisplay, plot);
    if ( legend == NULL) {
	AmPrintResourceError( parent, "screenObjectNotSupported", 
			    plot->GetObjectType());
	return;
    }
    legend->GetVaFields( AmNwidth, &legendWidth,
			 AmNheight, &legendHeight,
			 NULL);

    scale = 1.0;
    if ( legendWidth > width) {
        scale = ( float) width / ( float) legendWidth;
    }
    if ( legendHeight * scale > height) {
        scale = ( float) height / ( float) legendHeight;
    }
    if ( scale != 1.0) {
        legend->SetRect( 20, 20, legendWidth * scale, legendHeight * scale);
    }

/* add the legend to the display, and flush the display */

    legendDisplay->AddObject( legend);
    legendDisplay->Flush();

/* OK and cancel buttons */

    *okBtn = actionArea.buttons[ 0];

/* add the callbacks for the different widgets */

    AmAddVaCallback( numColorsScl, XmNvalueChangedCallback,
		   ( VarargsCallback) changeNumColorsCallback, 
		   legendDisplay, legend, plot, colormapFile,
		   END_LIST);
    AmAddVaCallback( cmapList, XmNsingleSelectionCallback,
		   ( VarargsCallback) changeColormapCallback,
		   legendDisplay, legend, colormapFile,
		   END_LIST);
    AmAddVaCallbackWidget( AM_PKG_ID, AM_OK, *dialog, 
			 ( VarargsCallback) setColormapDialogOkCallback,
			 *dialog, legend, colormapFile,
			 END_LIST);
    XtManageChild( formDlog);
}

/*************************************************************************
 *
 * Description:
 * 	reassign the colormap dialog creation callback if the dialog is
 * 	destroyed.  This is used in the map editor slick interface 
 * Arguments:
 *	w - launching widget.
 *	button - button colormap dialog callback is atteached to.
 *	plot - plot whose colormap is being edited 
 *	callData - unused 
 *
 **************************************************************************/

/* ARGSUSED */

static void 
setColormapDialogCallback( Widget		w,
			   Widget		button,
			   AmPlotClass		*plot,
			   char			**args,
			   XtPointer		callData)
{
    AmAddVaCallback( button, XmNactivateCallback,
		     ( VarargsCallback) AmColormapDialogCallback, 
		     plot, args,
		     END_LIST);
}


/***************************************************************************
 *
 * Description:
 * 	callback for embedding in a slick subform.  It unmanages the dialog
 * 	on the OK, and sets it up to be remanaged again when the button is 
 *	hit.
 * Arguments:
 *	w - launching widget.
 *	plot - plot whose colormap is being edited 
 *	callData - unused 
 *
 ***************************************************************************/

/* ARGSUSED */

AnvilBoolean
AmColormapDialogCallback( Widget	w,
			  AmPlotClass	*plot,
			  char		**args,
			  XtPointer	callData)
{
    Widget	dialog, okBtn;

    AmCreateColormapDialog( AmGetWidgetShell( w), plot, &dialog, &okBtn);
    if ( dialog != NULL) {
	AmAddVaCallback( okBtn, XmNactivateCallback,
		         ( VarargsCallback) AmUnmanageSubDialogCallback,
		         w, dialog, 
			 END_LIST);

/* we remove and replace the callback to launch the dialog, because we
   keep the dialog around on OK for our parent slick */

	AmRemoveVaCallback( w, XmNactivateCallback,
			    ( VarargsCallback) AmColormapDialogCallback, 
			    plot, args, 
			    END_LIST);
	AmAddVaCallback( dialog, XmNdestroyCallback,
		         ( VarargsCallback) setColormapDialogCallback,
		         w, plot, args,
		         END_LIST);
        return( TRUE);
    } else {
        AmAddResourceToLog( "cantCreateColormapDialog");
        return( FALSE);
    }
}

/***************************************************************************
 *
 * Description:
 * 	standard colormap editing callback function for direct use.  Allows
 *	the user to edit the colormap of a plot 
 * Arguments:
 *	w - launching widget.
 *	motifDisplay - display to update when the plot is edited.
 *	plot - plot whose colormap is being edited 
 *	callData - unused 
 *
 ***************************************************************************/

/* ARGSUSED */

AnvilBoolean
AmSetColormapDialogCallback( Widget			w,
			     AmMotifDisplayClass	*motifDisplay,
			     AmPlotClass		*plot,
			     XtPointer			callData)
{
    Widget	dialog, okBtn;

    AmCreateColormapDialog( AmGetWidgetShell( w), plot, &dialog, &okBtn);

    if ( dialog != NULL) {
	AmSetOkButton( okBtn, plot);
	XtAddCallback( okBtn, XmNactivateCallback,
		       ( XtCallbackProc) AmUpdateMotifDisplayCallback,
		       motifDisplay);
        return( TRUE);
    } else {
        AmAddResourceToLog( "cantCreateColormapDialog");
        return( FALSE);
    }
}
