#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#if !defined(__BSD44__)
#include <alloca.h>
#endif
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Scale.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <anvil/display.h>
#include <anvil/plot.h>
#include <anvil/motifdisplay.h>
#include <anvil/magic.h>
#include <anvil/rect.h>
#include <anvil/macros.h>
#include <anvil/image.h>
#include <anvil/colorcell.h>
#include <anvil/va_cback.h>
#include <anvil/slick.h>
#include <anvil/textcheck.h>
#include <anvil/utility.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/contour.h>
#include <anvil/utility.h>

class ContourMagicClass : public AmMagicWindowClass {
private: 
    Widget		dialog;
    float		min;
    float		max;
    int			steps;
public:
    ContourMagicClass( AmMotifDisplayClass	*motifDisplay,
		       AmImagePlotClass           *sourcePlot,
                       AnvilBoolean                  callOnDrag);
    Widget              GetDialog() { return( dialog); };
    void                SetDialog( Widget dialogArg) { dialog = dialogArg; };
    void		SetMinmax( float min, float max);
    void		SetSteps( int stepArg) { steps = stepArg; };
    virtual void        Draw( AmRectangleRec const *exposeRect);
    virtual             ~ContourMagicClass();
}; 

static AmActionDefRec actionAreaDef[] = {
    { "applyBtn", NULL, NULL },
    { "closeBtn", NULL, NULL }
};

void 
ContourMagicClass::SetMinmax( float	minArg,
			      float	maxArg)
{
    min = minArg;
    max = maxArg;
}

/* take a submatrix from an image plot and apply a contour to it, then
   draw it to the window */

void 
ContourMagicClass::Draw( AmRectangleRec const *exposeRect)
{
    float		*colorsBinvec, step, xmin, ymin, xmax, ymax, 
			xscale, yscale, winXmin, winYmin, winXmax, winYmax,
			subXmin, subYmin, subXmax, subYmax;
    int			numColors, maxLength, size, 
			subRows, subCols, srcRows, srcCols, ii;
    AmListPtr		edgeListList, inListList, levelPtr, polyPtr;
    AmAccessFn		colorsFn;
    void		*colorsClientData;
    uchar_t		*data, *subData;
    AnvilBoolean		flipRows, flipCols;
    AmRectangleRec	sourceIntRect, screenRect;
    AmPointPtr		screenpts;
    AmPolygonPtr	polygon;
    AmRealRectRec	sourceRect; 
    AmErrorCode		errCode;

/* pull the information from the source plot */

    sourcePlot->GetVaFields( AmNdata, &data,
			     AmNsubXmin, &subXmin,
			     AmNsubYmin, &subYmin,
			     AmNsubXmax, &subXmax,
			     AmNsubYmax, &subYmax,
			     AmNsubRows, &subRows,
			     AmNsubCols, &subCols,
			     AmNcolorsFn, &colorsFn,
			     AmNcolorsClientData, &colorsClientData,
			     AmNsize, &size,
			     AmNflipRows, &flipRows,
			     AmNflipCols, &flipCols,
			     AmNcolorsBinvec, &colorsBinvec,
			     AmNnumColors, &numColors,
			     NULL);

/* figure out what coordinates the magic window refers to, and where
   that is in the source matrix.  Since the matrix might be clipped,
   we have to change the source coordinates as well */

    GetSourceCoords( &winXmin, &winYmin, &winXmax, &winYmax);
    xmin = winXmin;
    ymin = winYmin;
    xmax = winXmax;
    ymax = winYmax;
    screenRect = rect;

/* don't contour if we're not in the source matrix at all.... */

    if ( !sourcePlot->MatrixRealExtents( subXmin, subYmin, subXmax, subYmax,
					 winXmin, winYmin, winXmax, winYmax,
					 subRows, subCols, flipRows, flipCols,
					 &screenRect, &sourceRect)) {
	if ( colorsBinvec != NULL) {
	    free( colorsBinvec);
	}
	return;
    }
			
    sourcePlot->GetDataPt( screenRect.x, screenRect.y + screenRect.height,
			   &winXmin, &winYmin);
    sourcePlot->GetDataPt( screenRect.x + screenRect.width, screenRect.y,
			   &winXmax, &winYmax);
    AmMakeContainedIntRect( &sourceRect, &sourceIntRect);

/* figure the dimensions and location of the sub matrix */

    srcRows = sourceIntRect.height;
    srcCols = sourceIntRect.width;
    if (( subData = ( uchar_t *) alloca( srcRows*srcCols*size)) == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return;
    }
    errCode = AmCopySubmatrix( data, subRows, subCols, &sourceIntRect, 
			       subData, srcRows, srcCols, 0, 0, size, 
			       flipRows, flipCols);
    if ( errCode != AmOK) {
	if ( colorsBinvec != NULL) {
	    free( colorsBinvec);
	}
	DO_ERROR( errCode);
	return;
    }

/* if the bin vector is NULL, then do a linearally scaled contour. */

    if ( colorsBinvec == NULL) {
	step = ( max - min)/steps;
	AmMakeContourList( subData, colorsFn, colorsClientData, size, 
			 srcRows, srcCols, min, max, step, 
			 &edgeListList, &inListList, &maxLength, 
			 winXmin, winYmin, winXmax, winYmax);
    } else {
	AmBinnedContourList( subData, colorsFn, colorsClientData, size,
			   srcRows, srcCols, colorsBinvec, numColors - 1,
			   &edgeListList, &inListList, &maxLength, 
			   winXmin, winYmin, winXmax, winYmax);
    }
    if (( screenpts = ( AmPointPtr) 
		alloca(( maxLength + 1)*sizeof( AmPointRec))) == NULL) {
	AmFreeContourList( edgeListList);
	AmFreeContourList( inListList);
	if ( colorsBinvec != NULL) {
	    free( colorsBinvec);
	}
	DO_ERROR( AmALLOCA_FAILED);
	return;
    }
 
    xscale = rect.width/( xmax - xmin);
    yscale = rect.height/( ymax - ymin);

/* draw the edge intersecting polygons, then the interior polygons.
   walk through each contour level, them each contour polygon... */

    for ( levelPtr = edgeListList; 
	  levelPtr != NULL; 
	  levelPtr = levelPtr->next) {
	for ( polyPtr = ( AmListPtr) levelPtr->data;
	      polyPtr != NULL; 
	      polyPtr = polyPtr->next) {
	    polygon = ( AmPolygonPtr) polyPtr->data;

/* and scale the points to the drag window */

	    for ( ii = 0; ii < polygon->numPts; ii++) {
		screenpts[ ii].x = (short int)
		    (( polygon->pts[ ii].x - xmin)*xscale);
		screenpts[ ii].y = (short int)
		    (rect.height - ( polygon->pts[ ii].y - ymin)*yscale);
	    }
	    display->DrawLines( window, gc, screenpts, polygon->numPts);
	}
    }

/* the interior polygons are drawn closed... */

    for ( levelPtr = inListList; 
	  levelPtr != NULL; 
	  levelPtr = levelPtr->next) {
	for ( polyPtr = ( AmListPtr) levelPtr->data;
	      polyPtr != NULL; 
	      polyPtr = polyPtr->next) {
	    polygon = ( AmPolygonPtr) polyPtr->data;

/* and scale the points to the drag window */

	    for ( ii = 0; ii < polygon->numPts; ii++) {
		screenpts[ ii].x = (short int)
		    (( polygon->pts[ ii].x - xmin)*xscale);
		screenpts[ ii].y = (short int)
		    (rect.height - ( polygon->pts[ ii].y - ymin)*yscale);
	    }
	    screenpts[ polygon->numPts] = screenpts[ 0];
	    display->DrawLines( window, gc, screenpts, polygon->numPts + 1);
	}
    }

/* free the contour lists and the colors bin vector */

    if ( colorsBinvec != NULL) {
	free( colorsBinvec);
    }
    AmFreeContourList( edgeListList);
    AmFreeContourList( inListList);
}

/* destroy the magic window, and free the client data */

static void 
destroyContourCallback( Widget			dialog,
			ContourMagicClass	*contourMagic,
			XtPointer)
{
    contourMagic->SetDialog( NULL);
    delete contourMagic;
}

/* destructor: destroy the dialog if it isn't NULL */

ContourMagicClass::~ContourMagicClass()
{
    if ( dialog != NULL) {
	
    
/* remove the callback to delete the object when the dialog is destroyed,
   otherwise, we get deleted again when the dialog is blown away... */

        XtRemoveCallback( dialog, XmNdestroyCallback,
                          ( XtCallbackProc) destroyContourCallback, this);
	XtDestroyWidget( dialog);
    }
}

/* grab the min and max and step values, and update the contour
   magic window */

static void
contourValuesCallback( Widget			button,
		       ContourMagicClass	*contourMagic,
		       Widget			minText,
		       Widget			maxText,
		       Widget			stepScale,
		       XtPointer)
{
    float	min, max;
    char	*minStr, *maxStr;
    int		steps;

    minStr = XmTextFieldGetString( minText);
    maxStr = XmTextFieldGetString( maxText);
    max = atof( maxStr);
    min = atof( minStr);
    XtFree( minStr);
    XtFree( maxStr);
    XmScaleGetValue( stepScale, &steps);
    contourMagic->SetSteps( steps);
    if ( min <= max) {
        AmSetErrorColor( minText, FALSE);
        AmSetErrorColor( maxText, FALSE);
	contourMagic->SetMinmax( min, max);
        contourMagic->Update();
	contourMagic->DrawWrapper( contourMagic->GetRect());
    } else {
        AmSetErrorColor( minText, TRUE);
        AmSetErrorColor( maxText, TRUE);
    }
}

/* magic window class for contouring images */

ContourMagicClass::ContourMagicClass( AmMotifDisplayClass	*motifDisplay,
				      AmImagePlotClass	*imagePlot,
				      AnvilBoolean		callOnDrag) :
			AmMagicWindowClass( motifDisplay, 
					  ( AmPlotClass *) imagePlot,
					  callOnDrag, TRUE)
{
    Arg			args[ 20];
    int			n;
    Widget		actionForm, minText, maxText, stepScale,
			minLbl, minForm, maxLbl, maxForm;
    AmActionRec		actionArea;
    AmDataTypeEnum	colorsUnits;

    imagePlot->GetVaFields( AmNcolorsMin, &min,
			    AmNcolorsMax, &max,
			    AmNnumColors, &steps,
			    AmNcolorsUnits, &colorsUnits,
			    NULL);

/* create the popup dialog and the action area with the apply and close
   buttons */

    n = AmMakeArgList( args,
		     XmNautoUnmanage, FALSE,
		     NULL);
    dialog = 
	XmCreateFormDialog( motifDisplay->GetShell(), "contour", args, n);
    actionForm = AmMakeActionArea( dialog,
				 actionAreaDef, ELEMENTS( actionAreaDef),
				 &actionArea);

/* create the text widgets for the min and max text, and a slider for
   the number of steps */

    maxForm = XtVaCreateManagedWidget( "maxForm", xmFormWidgetClass, dialog,
				       StickLeft, StickRight, StickTop,
				       NULL);
    maxLbl = XtVaCreateManagedWidget( "maxLbl", xmLabelWidgetClass, 
				       maxForm, 
				       StickLeft, StickTop, StickBottom,
				       NULL);
    maxText = XtVaCreateManagedWidget( "maxText", xmTextFieldWidgetClass,
				       maxForm, 
				       RightOf( maxLbl), StickTop, StickBottom,
				       NULL);
    AmUnitVarEntry( maxText, &max, AmFLOAT, colorsUnits, NULL);
    minForm = XtVaCreateManagedWidget( "minForm", xmFormWidgetClass, 
				       dialog,
				       StickLeft, StickRight, 
				       BelowOf( maxForm),
				       NULL);
    minLbl = XtVaCreateManagedWidget( "minLbl", xmLabelWidgetClass,
				       minForm, 
				       StickLeft, StickTop, StickBottom,
				       NULL);
    minText = XtVaCreateManagedWidget( "minText", xmTextFieldWidgetClass,
				       minForm, 
				       RightOf( minLbl), StickTop, StickBottom,
				       NULL);
    AmUnitVarEntry( minText, &min, AmFLOAT, colorsUnits, NULL);
    stepScale = AmAddObject( dialog, &steps, "steps", AmINT, AmLOCAL);
    XtVaSetValues( XtParent( stepScale), 
		   StickLeft, StickRight, BelowOf( minForm), 
		   AboveOf( actionForm),
		   NULL);

/* text callbacks to accept the new ranges */

    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, dialog);
    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback,
		     ( VarargsCallback) contourValuesCallback,
		     this, minText, maxText, stepScale, 
		     END_LIST);
    XtAddCallback( dialog, XmNdestroyCallback,
		   ( XtCallbackProc) destroyContourCallback, this);

/* manage the dialog ... */

    XtManageChild( dialog);
}

/* callback for a contour cursor */

AnvilBoolean
AmContourCursorCallback( Widget			btn,
		         AmMotifDisplayClass	*motifDisplay,
         	         AmImagePlotClass	*imagePlot,
		         XtPointer)
{
    if ( !imagePlot->IsType( AmIMAGE_PLOT)) {
	AmAddResourceToLog( "ContourCursorNotImagePlot");
	return( FALSE);
    }

    new ContourMagicClass( motifDisplay, imagePlot, FALSE);
    return( TRUE);
}
