#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <Xm/Xm.h>
#include <anvil/userevents.h>
#include <anvil/motifdisplay.h>
#include <anvil/screenobj.h>
#include <anvil/eventcodes.h>

/* temporary structure to pass arguments to the callbacks */

typedef struct dialogObj_ {
    AmMagicManagerClass	*dispMgr;
    Widget		dialog;
    AmScreenObjClass	*obj;
} dialogObjRec, *dialogObjPtr;

/* destroy the dialog when the object is deleted */

static void 
destroyDialogOnDelete( int		eventId,
		       dialogObjPtr	dialogObj,
		       AmScreenObjClass	*deleteObj)
{
    if ( deleteObj == dialogObj->obj) {
	XtDestroyWidget( dialogObj->dialog);
    }
}

/* remove the user event handler when the dialog is deleted */

static void 
removeDestroyDialogOnDelete( Widget		dialog,
			     dialogObjPtr	dialogObj,
			     XtPointer		callData)
{
    dialogObj->dispMgr->RemoveUserCallback( 
	AM_PKG_ID, AM_DELETE_OBJECT, 
	( callbackProc) destroyDialogOnDelete, dialogObj);
    free( dialogObj);
}

/************************************************************************
 *
 * Description:
 *	set up the Xt Callbacks and event handlers to destroy a 
 *	dialog whenever the screen object it depends on is deleted
 * Arguments:
 *	dispMgr - display manager object is displayed on.
 *	dialog - dialog shell to destroy.
 *	obj - object which dialog depends on
 * Returns:
 *	AmOK on success, AmMALLOC_FAILED if the argpassing 
 *	structure couldn't be allocated.
 *
 ************************************************************************/

AmErrorCode 
AmDestroyDialogOnDeleteObject( AmMagicManagerClass	*dispMgr,
			     Widget		dialog,
			     AmScreenObjClass	*obj)
{
    dialogObjPtr	dialogObj;

    dialogObj = ( dialogObjPtr) malloc( sizeof( dialogObjRec));
    if ( dialogObj == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    dialogObj->dialog = dialog;
    dialogObj->dispMgr = dispMgr;
    dialogObj->obj = obj;

    XtAddCallback( dialog, XmNdestroyCallback,
		   ( XtCallbackProc) removeDestroyDialogOnDelete,
		   dialogObj);
    dispMgr->AddUserCallback( AM_PKG_ID, AM_DELETE_OBJECT,
			      ( callbackProc) destroyDialogOnDelete,
			      dialogObj);
    return( AmOK);
}
