#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdarg.h>
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/DrawingA.h>
#include <anvil/strlib.h>
#include <anvil/list.h>
#include <anvil/tagstrs.h>
#include <anvil/motif.h>
#include <anvil/geometry.h>
#include <anvil/va_cback.h>
#include <anvil/plot_cback.h>
#include <anvil/macros.h>
#include <anvil/copyobj.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/errlib.h>
#include <anvil/utility.h>
#include <anvil/textcheck.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>
#include <anvil/imageP.h>
#include <anvil/imageplot.h>
#include <anvil/scatterplot.h>
#include <anvil/plot.h>
#include <anvil/imagelegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/motifdisplay.h>
#include <anvil/magic.h>

typedef struct xyzRec_ {
    float	x;
    float	y;
    float	z;
} xyzRec, *xyzPtr;

/* client data for the difference routines */

class DifferenceMagicClass : public AmMagicPlotClass {
private:
    AmPlotClass		*plot1;
    AmPlotClass		*plot2;
    Widget		dialog;
    float		diffMin;
    float		diffMax;
    AmLegendClass		*legend;
    AmMotifDisplayClass	*legendDisplay;
public:
    DifferenceMagicClass( AmMotifDisplayClass	*motifDisplay,
			  AmPlotClass		*plot1Arg,
			  AmPlotClass		*plot2Arg,
			  AnvilBoolean		callOnDrag = FALSE);
    virtual 			~DifferenceMagicClass();
    virtual AmDataGraphicClass	*Init( AmDataGraphicClass	*sourcePlot);
    virtual void		SetRect( AmRectangleRec const	*rectArg);
    AmPlotClass			*GetPlot1() { return( plot1); };
    AmPlotClass			*GetPlot2() { return( plot2); };
    void			SetDialog( Widget dialogArg) 
					{ dialog = dialogArg; };
    void			SetMinmax( float	minArg,
					   float	maxArg);
};

static AmActionDefRec actionAreaDef[] = {
    { "applyBtn", NULL, NULL },
    { "closeBtn", NULL, NULL }
};

AmDataGraphicClass *DifferenceMagicClass::Init( AmDataGraphicClass	*sourcePlot)
{
    if ( plot2->IsType( AmSCATTER_PLOT)) {
	AmScatterPlotClass	*scatterPlot;
	AmImagePlotClass		*imagePlot;
	AmAccessFn		scatterColorsFn, imageColorsFn, xFn, yFn;
	void			*scatterColorsClientData, *clientData, 
				*imageColorsClientData;
	AmColorCellPtr		colors;
	int			scatterRows, imageRows, imageCols, 
				xOffset, yOffset, scatterSize, imageSize, 
				scatterColorsOffset, imageColorsOffset,
				ii, index, rowIndex, colIndex, numColors;
	xyzPtr			newData;
	uchar_t			*scatterData, *imageData, *ptr, *imagePtr;
	AnvilBoolean			flipRows, flipCols;
	float			scatterZ, imageZ, colorsMin, colorsMax,
				subXmin, subYmin, subXmax, subYmax;
	char			*symbol;

/* extract the pertinent values from the source plots */

	scatterPlot = ( AmScatterPlotClass *) plot2;
	imagePlot = ( AmImagePlotClass *) plot1;
	scatterPlot->GetVaFields( AmNcolorsFn, &scatterColorsFn,
			          AmNcolorsClientData, &scatterColorsClientData,
			          AmNcolors, &colors,
			          AmNcolorsOffset, &scatterColorsOffset,
			          AmNnumColors, &numColors,
			          AmNrows, &scatterRows,
			          AmNsize, &scatterSize,
			          AmNdata, &scatterData,
			          AmNxFn, &xFn,
			          AmNyFn, &yFn,
			          AmNxOffset, &xOffset,
			          AmNyOffset, &yOffset,
			          AmNclientData, &clientData,
				  AmNsymbol, &symbol,
			          NULL);
	imagePlot->GetVaFields( AmNsubRows, &imageRows,
				AmNsubCols, &imageCols,
				AmNsize, &imageSize,
				AmNdata, &imageData,
				AmNflipRows, &flipRows,
				AmNflipCols, &flipCols,
				AmNcolorsFn, &imageColorsFn,
				AmNcolorsOffset, &imageColorsOffset,
				AmNcolorsClientData, &imageColorsClientData,
				AmNsubXmin, &subXmin,
				AmNsubYmin, &subYmin,
				AmNsubXmax, &subXmax,
				AmNsubYmax, &subYmax,
				NULL);

/* get the x,y value for each element in the diffed scatter plot, 
   and find the corresponding element in the matrix if there is one.
   Also track the range of values */

	newData = ( xyzPtr) malloc( scatterRows*sizeof( xyzRec));
	colorsMin = HUGE;
	colorsMax = -HUGE;
	for ( ii = 0; ii < scatterRows; ii++) {
	    ptr = scatterData + ii*scatterSize;
	    newData[ ii].x = ( xFn)( ptr + xOffset, clientData);
	    newData[ ii].y = ( yFn)( ptr + yOffset, clientData);
	    scatterZ = ( scatterColorsFn) ( ptr + scatterColorsOffset, 
					    scatterColorsClientData);
	    if ( imagePlot->GetRowcol( newData[ ii].x, newData[ ii].y,
				       &rowIndex, &colIndex)) {
		index = rowIndex*imageCols + colIndex;
		imagePtr = &imageData[ index*imageSize];
		imageZ = ( imageColorsFn)( imagePtr + imageColorsOffset,
					   imageColorsClientData);
		newData[ ii].z = scatterZ - imageZ;
	    } else {
		newData[ ii].z = scatterZ;
	    }
	    if ( newData[ ii].z < colorsMin) {
		colorsMin = newData[ ii].z;
	    }
	    if ( newData[ ii].z > colorsMax) {
		colorsMax = newData[ ii].z;
	    }
	}

/* create a scatter plot to reflect the difference */

	return( new AmScatterPlotClass( imagePlot->GetWindow(), "diffPlot",
				      AmNcolors, colors,
				      AmNnumColors, numColors,
				      AmNdata, newData,
				      AmNrows, scatterRows,
				      AmNsize, sizeof( xyzRec),
				      AmNxFn, AmFloatFn,
				      AmNxOffset, offsetof( xyzRec, x),
				      AmNyFn, AmFloatFn,
				      AmNyOffset, offsetof( xyzRec, y),
				      AmNcolorsFn, AmFloatFn,
				      AmNcolorsOffset, offsetof( xyzRec, z),
				      AmNcolorsMin, diffMin,
				      AmNcolorsMax, diffMax,
				      AmNsymbol, symbol,
				      NULL));
    } else if ( plot2->IsType( AmIMAGE_PLOT)) {
	AmImagePlotClass	*imagePlot1;
	AmImagePlotClass	*imagePlot2;
	int		imageRows, imageCols, imageRows2, imageCols2,
			imageSize, imageSize2, imageColorsOffset,
			imageColorsOffset2, numColors;
	AnvilBoolean		flipRows, flipCols, flipRows2, flipCols2;
	void		*imageColorsClientData, *imageColorsClientData2;
	AmAccessFn	imageColorsFn, imageColorsFn2;
	uchar_t		*imageData, *imageData2;
	AmColorCellPtr	colors;
	float		colorsMin, colorsMax, colorsMin2, colorsMax2,
			subXmin, subYmin, subXmax, subYmax, *diffData;

	imagePlot1 = ( AmImagePlotClass *) plot1;
	imagePlot2 = ( AmImagePlotClass *) plot2;

	imagePlot1->GetVaFields( AmNsubRows, &imageRows,
				 AmNsubCols, &imageCols,
				 AmNsize, &imageSize,
				 AmNdata, &imageData,
				 AmNflipRows, &flipRows,
				 AmNflipCols, &flipCols,
				 AmNcolorsFn, &imageColorsFn,
				 AmNcolorsOffset, &imageColorsOffset,
				 AmNcolorsClientData, &imageColorsClientData,
				 AmNcolorsMin, &colorsMin,
				 AmNcolorsMax, &colorsMax,
				 AmNcolors, &colors,
				 AmNnumColors, &numColors,
				 AmNsubXmin, &subXmin,
				 AmNsubYmin, &subYmin,
				 AmNsubXmax, &subXmax,
				 AmNsubYmax, &subYmax,
				 NULL);
	imagePlot2->GetVaFields( AmNsubRows, &imageRows2,
				 AmNsubCols, &imageCols2,
				 AmNsize, &imageSize2,
				 AmNdata, &imageData2,
				 AmNflipRows, &flipRows2,
				 AmNflipCols, &flipCols2,
				 AmNcolorsFn, &imageColorsFn2,
				 AmNcolorsOffset, &imageColorsOffset2,
				 AmNcolorsClientData, &imageColorsClientData2,
				 AmNcolorsMin, &colorsMin2,
				 AmNcolorsMax, &colorsMax2,
				 NULL);

/* create a scatter plot to reflect the difference */

	diffData = ( float *) malloc( 100*100*sizeof( float));
	return( new AmImagePlotClass( imagePlot1->GetWindow(), "diffPlot",
				    AmNcolors, colors,
				    AmNnumColors, numColors,
				    AmNdata, diffData,
				    AmNrows, 100,
				    AmNcols, 100,
				    AmNdataType, AmFLOAT,
				    AmNcolorsFn, AmFloatFn,
				    AmNcolorsMin, diffMin,
				    AmNcolorsMax, diffMax,
				    AmNxmin, subXmin,
				    AmNymin, subYmin,
				    AmNxmax, subXmax,
				    AmNymax, subYmax,
				    NULL));

    } else {
	return( NULL);
    }
}

/* function to apply the difference AmBetween the two matrices */

AmErrorCode AmApplyDifference( AmRealRectPtr 	dataRect, 
			     AmRealRectPtr 	dataRect2, 
			     float	 	*diffData, 
			     int	 	matRows, 
			     int	 	matCols,
			     AmRectangleRec const	*drawRect,
			     uchar_t	 	*data,
			     int	 	size,
			     int	 	rows,
			     int	 	cols,
			     AnvilBoolean	 	flipRows,
			     AnvilBoolean	 	flipCols,
			     AmAccessFn	 	colorsFn,
			     int	 	colorsOffset,
			     void	 	*colorsClientData,
			     uchar_t	 	*data2,
			     int	 	size2,
			     int	 	rows2,
			     int	 	cols2,
			     AnvilBoolean	 	flipRows2,
			     AnvilBoolean	 	flipCols2,
			     AmAccessFn	 	colorsFn2,
			     int	 	colorsOffset2,
			     void	 	*colorsClientData2)
{
    int		*hVec, *hVecPtr, *vVec, *vVecPtr, *hVec2, *hVecPtr2, 
		*vVec2, *vVecPtr2;
    uchar_t	*srcPtr, *srcPtr2, *srcRowPtr, *srcRowPtr2;
    float	*rowPtr, *rowEnd, *dstPtr, *dstEnd, a, b;

/* apply the flips to the source rectangles */

    AmApplyFlips( dataRect, flipRows, flipCols);
    AmApplyFlips( dataRect2, flipRows, flipCols);

/* create the mapping vectors from the 2 source matrices */

    hVec = AmMakeRealFwdDiffVec( dataRect->x, dataRect->x + dataRect->width, 
			       drawRect->width);
    if ( hVec == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    vVec = AmMakeRealFwdDiffVec( dataRect->y, dataRect->y + dataRect->height, 
			       drawRect->height);
    if ( vVec == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    hVec2 = AmMakeRealFwdDiffVec( dataRect2->x, dataRect2->x + dataRect2->width, 
				drawRect->width);
    if ( hVec2 == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    vVec2 = AmMakeRealFwdDiffVec( dataRect2->y, dataRect2->y + dataRect2->height,
				drawRect->height);
    if ( vVec2 == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    srcPtr = data + (( int) dataRect->y*cols + ( int) dataRect->x)*size;
    srcPtr2 = data2 + (( int) dataRect2->y*cols2 + ( int) dataRect2->x)*size2;
    vVecPtr = vVec;
    vVecPtr2 = vVec2;
    dstPtr = diffData + drawRect->x + drawRect->y*matCols;
    dstEnd = dstPtr + drawRect->height*matCols;

/* loop over each row, setting pointers to the start of the source and
destination matrix, and set the end row marker */

    while ( dstPtr < dstEnd) {
	hVecPtr = hVec;
	hVecPtr2 = hVec2;
	rowPtr = dstPtr;
	rowEnd = dstPtr + drawRect->width;
	srcRowPtr = srcPtr;
	srcRowPtr2 = srcPtr2;

/* loop to the end of the row of the destination matrix */

	while ( rowPtr < rowEnd) {
	    a = ( colorsFn)( srcRowPtr + colorsOffset, colorsClientData);
	    b = ( colorsFn)( srcRowPtr2 + colorsOffset, colorsClientData2);
	    *rowPtr++ = a - b;
	    srcRowPtr += ( *hVecPtr++)*size;
	    srcRowPtr2 += ( *hVecPtr2++)*size2;
	}
	dstPtr += matCols;
	srcPtr += ( *vVecPtr++)*cols*size;
	srcPtr2 += ( *vVecPtr2++)*cols2*size2;
    }
    free( hVec);
    free( vVec);
    free( hVec2);
    free( vVec2);
    return( AmOK);
}


/* called when the window is moved...update the difference plot */


void DifferenceMagicClass::SetRect( AmRectangleRec const *rectArg)
{
    uchar_t	*data, *data2; 
    float	*diffData, res1, res2, subXmin, subYmin, subXmax, subYmax,
		subXmin2, subYmin2, subXmax2, subYmax2, xmin, ymin, xmax, ymax,
		isectXmin, isectYmin, isectXmax, isectYmax, x1, y1;
    int		rows, cols, rows2, cols2, colorsOffset, colorsOffset2, 
		matRows, matCols, size, size2, ii;
    AmAccessFn	colorsFn, colorsFn2;
    void	*colorsClientData, *colorsClientData2;
    AmRealRectRec	dataRect, dataRect2;
    AnvilBoolean	flipRows, flipCols, flipRows2, flipCols2;
    AmRectangleRec drawRect, drawRect2, diffRect;

/* reset the rectangle AmFirst... */

    rect = *rectArg;
    drawRect = *rectArg;
    drawRect2 = drawRect;
    GetSourceCoords( &xmin, &ymin, &xmax, &ymax);

/* the world is complicated if we're an image plot */

    if ( plot2->IsType( AmIMAGE_PLOT)) {
	AmImagePlotClass *imagePlot1 = ( AmImagePlotClass *) plot1;
	AmImagePlotClass *imagePlot2 = ( AmImagePlotClass *) plot2;


/* get the dimensions for the AmFirst image */

	imagePlot1->GetVaFields( AmNsubRows, &rows,
				 AmNsubCols, &cols,
				 AmNsize, &size,
				 AmNflipRows, &flipRows,
				 AmNflipCols, &flipCols,
				 AmNdata, &data,
				 AmNcolorsFn, &colorsFn,
				 AmNcolorsClientData, &colorsClientData,
				 AmNcolorsOffset, &colorsOffset,
				 AmNsubXmin, &subXmin,
				 AmNsubYmin, &subYmin,
				 AmNsubXmax, &subXmax,
				 AmNsubYmax, &subYmax,
				 NULL);

/* get the dimensions for the second image */

	imagePlot2->GetVaFields( AmNsubRows, &rows2,
				 AmNsubCols, &cols2,
				 AmNsize, &size2,
				 AmNflipRows, &flipRows2,
				 AmNflipCols, &flipCols2,
				 AmNdata, &data2,
				 AmNcolorsFn, &colorsFn2,
				 AmNcolorsClientData, &colorsClientData2,
				 AmNcolorsOffset, &colorsOffset2,
				 AmNsubXmin, &subXmin2,
				 AmNsubYmin, &subYmin2,
				 AmNsubXmax, &subXmax2,
				 AmNsubYmax, &subYmax2,
				 NULL);

/* figure out the intersection of the two image plots and the magic window
   in the coordinate system. bail if there is no intersection */

	isectXmin = MAX( MAX( subXmin, subXmin2), xmin);
	isectYmin = MAX( MAX( subYmin, subYmin2), ymin);
	isectXmax = MIN( MIN( subXmax, subXmax2), xmax);
	isectYmax = MIN( MIN( subYmax, subYmax2), ymax);
	if (( isectXmin > isectXmax) || ( isectYmin > isectYmax)) {
	    return;
	}

/* determine the resolutions of the matrices, and find out what the
   intersection's rectangle projects to in each of the source matrices 
   dimensions */

	res1 = MIN(( subXmax - subXmin)/cols, ( subYmax - subYmin)/rows);
	res2 = MIN(( subXmax2 - subXmin2)/cols2, ( subYmax2 - subYmin2)/rows2);
	imagePlot1->MatrixRealExtents( subXmin, subYmin, subXmax, subYmax,
				       isectXmin, isectYmin, 
				       isectXmax, isectYmax,
				       rows, cols, flipRows, flipCols, 
				       &drawRect, &dataRect);
	imagePlot2->MatrixRealExtents( subXmin2, subYmin2, subXmax2, subYmax2,
				       isectXmin, isectYmin, 
				       isectXmax, isectYmax,
				       rows2, cols2, flipRows2, flipCols2, 
				       &drawRect2, &dataRect2);

/* figure out the dimensions of the magic window in the source matrix with 
   the higher resolution */

	if ( res1 > res2) {
	    matRows = (int)rint(( ymax - ymin)/( subYmax - subYmin)*rows);
	    matCols = (int)rint(( xmax - xmin)/( subXmax - subXmin)*cols);
	} else {
	    matRows = (int)rint(( ymax - ymin)/( subYmax2 - subYmin2)*rows2);
	    matCols = (int)rint(( xmax - xmin)/( subXmax2 - subXmin2)*cols2);
	}

/* take the intersection and calculate the offsets within the dimensions
   of the new matrix */

	diffRect.x = (int)(matCols*( isectXmin - xmin)/( xmax - xmin));
	diffRect.y = (int)(matRows - matRows*( isectYmax - ymin)/( ymax - ymin));
	x1 = matCols*( isectXmax - xmin)/( xmax - xmin);
	y1 = matRows - matRows*( isectYmin - ymin)/( ymax - ymin);
	diffRect.width = (int)(x1 - diffRect.x);
	diffRect.height = (int)(y1 - diffRect.y);

AN_DEBUG( 90, ("dataRect= %.2f, %.2f, %.2f, %.2f\n", 
		dataRect.x, dataRect.y, dataRect.width, dataRect.height));
AN_DEBUG( 90, ("dataRect2= %.2f, %.2f, %.2f, %.2f\n", 
		dataRect2.x, dataRect2.y, dataRect2.width, dataRect2.height));
AN_DEBUG( 90, ("diffRect= %d, %d, %d, %d\n", 
		diffRect.x, diffRect.y, diffRect.width, diffRect.height));
AN_DEBUG( 90, ("mat rows = %d, cols = %d\n", matRows, matCols));

	plot->GetVaFields( AmNdata, &diffData, NULL);
	if ( diffData == NULL) {
	    FREE( diffData);
	}
	if (( diffData = ( float *) 
		malloc( matRows*matCols*sizeof( float))) == NULL) {
	    DO_ERROR( AmMALLOC_FAILED);
	}
	for ( ii = 0; ii < matRows*matCols; ii++) {
	    diffData[ ii] = diffMin;
	}
	AmApplyDifference( &dataRect, &dataRect2, diffData, matRows, matCols,
			 &diffRect, data, size, rows, cols, flipRows, flipCols,
			 colorsFn, colorsOffset, colorsClientData,
			 data2, size2, rows2, cols2, flipRows2, flipCols2,
			 colorsFn2, colorsOffset2, colorsClientData2);

/* replace the data and dimensions in the difference plot */

	plot->SetVaFields( AmNdata, diffData,
			   AmNrows, matRows,
			   AmNcols, matCols,
			   AmNcurrXmin, xmin,
			   AmNcurrYmin, ymin,
			   AmNcurrXmax, xmax,
			   AmNcurrYmax, ymax,
			   AmNxmin, xmin,
			   AmNymin, ymin,
			   AmNxmax, xmax,
			   AmNymax, ymax,
			   NULL);
	plot->Update();
    } else {
	AmMagicPlotClass::SetRect( rectArg);
    }
}

/* magic window to show the difference AmBetween two plots */

AnvilBoolean
AmDifferenceCursorCallback( Widget			btn,
			  AmMotifDisplayClass	*motifDisplay,
			  AmPlotClass		*plot1,
			  AmPlotClass		*plot2,
			  XtPointer)
{
    if ( !plot1->IsType( AmIMAGE_PLOT)) {
	AmAddResourceToLog( "diffCursorPlot1NotImage");
	return( FALSE);
    }
    if (( !plot1->IsType( AmIMAGE_PLOT)) && !plot2->IsType( AmSCATTER_PLOT)) {
	AmAddResourceToLog( "diffCursorPlot2NotImageOrScatter");
	return( FALSE);
    }
    new DifferenceMagicClass( motifDisplay, plot1, plot2);
    return( TRUE);
}

/* set the range for the difference cursor */

void DifferenceMagicClass::SetMinmax( float	min,
				      float	max)
{
    diffMin = min;
    diffMax = max;
    plot->SetVaFields( AmNcolorsMin, min,
		       AmNcolorsMax, max,
		       NULL);
    Update();
    legend->SetVaFields( AmNmin, min,
			 AmNmax, max,
			 NULL);
    legendDisplay->Flush();
}

/* set the gradient range and redraw the image in the window */

static void minmaxValueCallback( Widget,
                                 Widget                 minText,
                                 Widget                 maxText,
                                 DifferenceMagicClass   *differenceMagic,
                                 XtPointer)
{
    float       min, max;
    char	*minStr, *maxStr;

    minStr = XmTextFieldGetString( minText);
    maxStr = XmTextFieldGetString( maxText);
    min = atof( minStr);
    max = atof( maxStr);
    XtFree( minStr);
    XtFree( maxStr);
    if ( min < max) {
        AmSetErrorColor( minText, FALSE);
        AmSetErrorColor( maxText, FALSE);
        differenceMagic->SetMinmax( min, max);
    } else {
        AmSetErrorColor( minText, TRUE);
        AmSetErrorColor( maxText, TRUE);
    }
}

/* destroy the magic window if either of the source are deleted */

static void invalidDifferenceCallback( Widget			dialog,
				       DifferenceMagicClass	*diffMagic,
				       AmPlotClass		*source)
{
    if (( source == diffMagic->GetPlot1()) ||
        ( source == diffMagic->GetPlot2())) {
	delete diffMagic;
    }
}

static void destroyDifferenceCallback( Widget			dialog,
				       DifferenceMagicClass	*diffMagic,
				       XtPointer)
{
    diffMagic->SetDialog( NULL);
    delete diffMagic;
}

/* destructor for difference magic window */

DifferenceMagicClass::~DifferenceMagicClass()
{
    AmMotifDisplayClass	*motifDisplay = ( AmMotifDisplayClass *) magicManager;

    if ( dialog != NULL) {
    
/* remove the callback to delete the object when the dialog is destroyed,
   otherwise, we get deleted again when the dialog is blown away... */

        XtRemoveCallback( dialog, XmNdestroyCallback,
                          ( XtCallbackProc) destroyDifferenceCallback, this);
	XtDestroyWidget( dialog);
    }

/* remove the callback which pays attention to whether either of the plots
   are deleted */

    AmRemoveCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT,
			  ( callbackProc) invalidDifferenceCallback, this,
			  motifDisplay->GetArea());
}

/* difference magic window plot */

DifferenceMagicClass::DifferenceMagicClass( AmMotifDisplayClass	*motifDisplay,
					    AmPlotClass		*plot1Arg,
					    AmPlotClass		*plot2Arg,
					    AnvilBoolean		callOnDrag) :
			    AmMagicPlotClass( motifDisplay, plot1Arg, callOnDrag)
{
    Widget		parent = motifDisplay->GetShell(), minText, maxText, 
			minLbl, minForm, maxLbl, maxForm, actionForm, area;
    AmActionRec		actionArea;
    Arg			args[ 20];
    int			n, numColors;
    float		min1, max1, min2, max2;
    unitType		colorsUnits;
    AmColorCellPtr	colors;
    AmXDisplayClass	*pXDisplay;
    AmXWindowClass	*pXWindow;

/* assign the plots and get their ranges */

    plot1 = plot1Arg;
    plot2 = plot2Arg;
    plot1->GetVaFields( AmNcolorsMin, &min1,
			AmNcolorsMax, &max1,
			AmNcolorsUnits, &colorsUnits,
			NULL);
    plot2->GetVaFields( AmNcolorsMin, &min2,
			AmNcolorsMax, &max2,
			AmNcolors, &colors,
			AmNnumColors, &numColors,
			NULL);
    diffMin = min1 - max2;
    diffMax = max1 - min2;

/* create a form to control the min and max for the difference */

    n = AmMakeArgList( args,
                     XmNautoUnmanage, XmDESTROY,
                     XmNcolormap, AmGetAppColormap(),
                     XmNvisual, AmGetAppVisual(),
                     XmNdepth, AmGetAppDepth(),
                     NULL);
    dialog = XmCreateFormDialog( parent, "differenceCursor", args, n);

    actionForm = AmMakeActionArea( dialog,
                                 actionAreaDef, ELEMENTS( actionAreaDef),
                                 &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);

    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, dialog,
                                    StickLeft, StickTop,
                                    AboveOf( actionForm),
                                    XmNwidth, 120,
                                    XmNheight, 300,
                                    NULL);

/* create a display for the legend for the gradient cursor, and a legend
   without a parent, because we'll control the min/max ourselves.  Create
   the legend depending on the type of the second plot */


    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    legendDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);
    if ( plot2Arg->IsType( AmSCATTER_PLOT)) {
	legend = new AmScatterLegendClass( legendDisplay->GetWindow(),
					 NULL, "gradientLegend",
					 AmNmin, diffMin,
					 AmNmax, diffMax,
					 AmNcolors, colors,
					 AmNnumColors, numColors,
					 AmNwidth, 100,
					 AmNheight, 250,
					 AmNx, 20,
					 AmNy, 20,
					 AmNleftMargin, 20,
					 AmNtopMargin, 20,
					 AmNbottomMargin, 20,
					 AmNrightMargin, 20,
					 NULL);
    } else {
	legend = new AmImageLegendClass( legendDisplay->GetWindow(),
				       NULL, "gradientLegend",
				       AmNmin, diffMin,
				       AmNmax, diffMax,
				       AmNcolors, colors,
				       AmNnumColors, numColors,
				       AmNwidth, 100,
				       AmNheight, 250,
				       AmNx, 20,
				       AmNy, 20,
				       AmNleftMargin, 20,
				       AmNtopMargin, 20,
				       AmNbottomMargin, 20,
				       AmNrightMargin, 20,
				       NULL);
    }
    free( colors);
    legendDisplay->AddObject( legend);
    legend->SetRect( 20, 20, 100, 250);

/* create the text widgets for the min and max text, and a slider for
   the number of steps */

    maxForm = XtVaCreateManagedWidget( "maxForm", xmFormWidgetClass, dialog,
                                       RightOf( area), StickRight, StickTop,
                                       NULL);
    maxLbl = XtVaCreateManagedWidget( "maxLbl", xmLabelWidgetClass,
                                       maxForm,
                                       StickLeft, StickTop, StickBottom,
                                       NULL);
    maxText = XtVaCreateManagedWidget( "maxText", xmTextFieldWidgetClass,
                                       maxForm,
                                       RightOf( maxLbl), StickTop, StickBottom,
                                       NULL);
    AmUnitVarEntry( maxText, &diffMax, AmFLOAT, colorsUnits, NULL);
    minForm = XtVaCreateManagedWidget( "minForm", xmFormWidgetClass,
                                       dialog,
                                       RightOf( area), StickRight,
                                       BelowOf( maxForm),
                                       NULL);
    minLbl = XtVaCreateManagedWidget( "minLbl", xmLabelWidgetClass,
                                       minForm,
                                       StickLeft, StickTop, StickBottom,
                                       NULL);
    minText = XtVaCreateManagedWidget( "minText", xmTextFieldWidgetClass,
                                       minForm,
                                       RightOf( minLbl), StickTop, StickBottom,
                                       NULL);
    AmUnitVarEntry( minText, &diffMin, AmFLOAT, colorsUnits, NULL);

/* text callbacks to accept the new ranges */

    AmAddVaCallback( maxText, XmNactivateCallback,
                   ( VarargsCallback) minmaxValueCallback,
                   minText, maxText, this,
                   END_LIST);
    AmAddVaCallback( minText, XmNactivateCallback,
                   ( VarargsCallback) minmaxValueCallback,
                   minText, maxText, this,
                   END_LIST);
    AmAddVaCallback( maxText, XmNlosingFocusCallback,
                   ( VarargsCallback) minmaxValueCallback,
                   minText, maxText, this,
                   END_LIST);
    AmAddVaCallback( minText, XmNlosingFocusCallback,
                   ( VarargsCallback) minmaxValueCallback,
                   minText, maxText, this,
                   END_LIST);


/* add a user callback which destroys the difference magic window if either
   of the source layers are deleted */

    AmAddCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, motifDisplay->GetArea(),
                       ( callbackProc) invalidDifferenceCallback, this);

/* assign the destroy callback */

    XtAddCallback( dialog, XmNdestroyCallback,
                   ( XtCallbackProc) destroyDifferenceCallback, this);

/* add callbacks to pop down the dialog, and to re-apply the minmaxs */

    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
                   ( XtCallbackProc) AmPopdownCback, dialog);
    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback,
                   ( VarargsCallback) minmaxValueCallback,
                   minText, maxText, this,
                   END_LIST);
    Update();

/* manage the dialog */

    XtManageChild( dialog);
}

static void enableOkButton( Widget			list,
			    Widget			okBtn,
			    XmListCallbackStruct	*cb)
{
    XtSetSensitive( okBtn, cb->selected_item_count == 2);
}

/* launch the cursor on OK */

static void differenceLayerCursorOkCallback( Widget		btn,
					     Widget		parentBtn,
					     AmMotifDisplayClass	*motifDisplay,
					     AmPlotClass		**layers,
					     Widget		list,
					     XtPointer		callData)
{
    int	*posList, count;

/* grab the selection list, pop down the dialog, set up the cursor and
   free stuff */

    XmListGetSelectedPos( list, &posList, &count);
    if ( count == 2) {
	XtDestroyWidget( AmGetWidgetShell( btn));
	if ( layers[ posList[ 0] - 1]->IsType( AmSCATTER_PLOT)) {
	    AmDifferenceCursorCallback( parentBtn, motifDisplay, 
				      layers[ posList[ 1] - 1], 
				      layers[ posList[ 0] - 1], callData);
	} else {
	    AmDifferenceCursorCallback( parentBtn, motifDisplay, 
				      layers[ posList[ 0] - 1], 
				      layers[ posList[ 1] - 1], callData);
	}
	free( layers);
    } else {
	AmPrintResourceError( AmGetWidgetRealShell( btn), 
			    "differenceMustSelect2Layers");
	free( posList);
    }
}

/* wrapper for the magic window function which allows the user to select
   the layers for which the difference cursor will be performed. */

AnvilBoolean
AmDifferenceLayerCursorCallback( Widget		btn,
			       AmMotifDisplayClass	*motifDisplay,
			       AmOverlayClass	*overlay,
			       XtPointer)
{
    AmPlotClass	**layers;
    int		numLayers, ii, jj, n;
    XmString	*AmLayerStringList;
    Widget	formDlog, list, actionForm;
    AmActionRec	actionArea;
    Arg		args[ 20];

/* get the layers and create the strings to select the layers */

    overlay->GetLayers(( AmDataGraphicClass ***) &layers, &numLayers);
    jj = 0;
    for ( ii = 0; ii < numLayers; ii++) {
	if ( layers[ ii]->IsType( AmSCATTER_PLOT) ||
	     layers[ ii]->IsType( AmIMAGE_PLOT)) {
	    layers[ jj++] = layers[ ii];
	}
    }
    numLayers = jj;
    if ( numLayers < 2) {
	AmAddResourceToLog( "differenceMust2Layers");
	return( FALSE);
    }

    AmLayerStringList = ( XmString *) malloc( sizeof( XmString)*numLayers);
    for ( ii = 0; ii < numLayers; ii++) {
	AmLayerStringList[ ii] = XmStringCreateLtoR( AmGetLayerName( layers[ ii]),
						   XmFONTLIST_DEFAULT_TAG);
    }

/* create the form dialog and the action form across the bottom */

    n = AmMakeArgList( args,
		     XmNautoUnmanage, FALSE,
		     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		     NULL);
    formDlog = XmCreateFormDialog( AmGetWidgetRealShell( btn), "differenceLayer",
				   args, n);
    actionForm = AmMakeActionArea( formDlog, actionAreaDef, 
				 ELEMENTS( actionAreaDef), &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);

/* create the list of available layers */

    n = AmMakeArgList( args,
		     StickLeft, StickTop, StickRight,
		     XmNitems, AmLayerStringList,
		     XmNitemCount, numLayers,
		     XmNselectionPolicy, XmMULTIPLE_SELECT,
		     XmNvisibleItemCount, 6,
		     NULL);
    list = XmCreateScrolledList( formDlog, "list", args, n);
    XtManageChild( list);
    AmFreeMotifStringList( AmLayerStringList, numLayers);

/* turn off the OK button until two layers are selected */

    XtSetSensitive( actionArea.buttons[ 0], FALSE);
    XtAddCallback( list, XmNmultipleSelectionCallback,
		   ( XtCallbackProc) enableOkButton, actionArea.buttons[ 0]);
    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback,
		   ( VarargsCallback) differenceLayerCursorOkCallback,
		   btn, motifDisplay, layers, list,
		   END_LIST);
    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
    XtManageChild( formDlog);

    return( TRUE);
}
