#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <X11/cursorfont.h>
#include <Xm/Xm.h> 
#include <Xm/Form.h> 
#include <Xm/PushB.h> 
#include <Xm/Label.h> 
#include <Xm/ToggleB.h> 
#include <Xm/RowColumn.h> 
#include <Xm/Separator.h> 
#include <Xm/Frame.h> 
#include <anvil/record.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/state_hand.h>
#include <anvil/envdefs.h>
#include <anvil/slick.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/va_cback.h>
#include <anvil/plot_cback.h>
#include <anvil/userevents.h>
#include <anvil/macros.h>
#include <anvil/dialang.h>
#include <anvil/screenobj.h>
#include <anvil/editobject.h>
#include <anvil/editobjectP.h>
#include <anvil/utility.h>
#include <anvil/visual.h>
#include <anvil/xcolor.h>

/* action area for the main dialog */

static AmActionDefRec actionDefs[] = {
    { "okBtn", NULL, NULL },
    { "revertBtn", NULL, NULL },
    { "cancelBtn",  NULL, NULL }
};

/* action area for the popup sub dialogs */

static AmActionDefRec dialogDefs[] = {
    { "okBtn", NULL, NULL },
    { "cancelBtn", NULL, NULL }
};

static void subDialogCallback( Widget		button,
			       AmRecordClass	*record,
			       AmDeclItemPtr	declItem,
			       AmListPtr	declList,
			       Widget		okBtn,
			       XtPointer	callData);

static Widget makePanel( Widget         	parent,
			 AmRecordClass    	*record,
			 AmDeclarationPtr	pDecl,
			 AmListPtr        	declList,
			 Widget         	okBtn);

static void makeDecl( Widget		rowcol,
		      AmRecordClass	*record,
		      AmDialogEntryPtr	pDialogEntry,
		      AmListPtr		declList,
		      Widget		okBtn);

/**************************************************************************
 * 
 * Description:
 * 	given a name for a declaration, search the list of declarations.
 * Arguments:
 * 	name - name to search for.
 *	declList - list of AmDeclarationPtr declarations.
 * Returns:
 *	the first found matching declaration, or NULL if it wasn't found
 *
 *************************************************************************/

static AmDeclarationPtr 
findDecl( const char	*name,
	  AmListPtr	declList)
{
    AmDeclarationPtr	pDecl;

    for ( ; declList != NULL; declList = declList->next) {
	pDecl = ( AmDeclarationPtr) declList->data;
	if ( !strcmp( name, pDecl->ident)) {
	    return( pDecl);
	}
    }
    return( NULL);
}

static AmErrorCode
addField( Widget	rowcol,
	  AmRecordClass	*record,
	  char		*fieldName,
	  AmListPtr	declList,
	  Widget	okBtn)
{
    AmRecordClass	*child;
    AmDeclarationPtr	ref;
    AmErrorCode		errCode;

    errCode = AmOK;

/* if it's an object, then we get the object's definition from the file by
   using the object name.  We also add the object to the OK button, so its
   fields are set separately */

    if ( AmGetTagType( fieldName) == AmOBJECT) {
	record->GetVaFields( fieldName, &child, NULL);
	ref = findDecl( fieldName, declList);
	if ( ref != NULL) {
	    makePanel( rowcol, child, ref, declList, okBtn);
	} else {
	    if ( ref == NULL) {
		AmAddResourceToLog( "cantFindDeclaration", fieldName);
		errCode = AmBAD_ARGUMENT;
	    }
	}
    } else {
	ref = findDecl( fieldName, declList);
	if ( ref != NULL) {
	    makePanel( rowcol, record, ref, declList, okBtn);
	} else {
	    AmAddObject( record, rowcol, fieldName);
	}
    }
    return( errCode);
}

/**************************************************************************
 * 
 * Description:
 * 	fill a rowcolumn with the list of fields in field list, using
 *	the record to retreive the current values.
 * Arguments:
 * 	rowcol - rowcolumn widget to stuff the widgets into.
 *	record - record to retreive the values from.
 *	fieldList - list of fields string tags.
 *	declList - list of declarations, possibly to be used for name
 *	referencing.
 *	okBtn - the OK button for the dialog.
 * Returns:
 *	AmOK on success, error if there were any problems.
 *
 *************************************************************************/

static AmErrorCode
fillRowcol( Widget		rowcol,
	    AmRecordClass	*record,
	    AmListPtr		fieldList,
	    AmListPtr		declList,
	    Widget		okBtn)
{
    AmListPtr		fieldPtr;
    char		*fieldName;
    AmErrorCode		errCode;

    errCode = AmOK;
    for ( fieldPtr = fieldList; fieldPtr != NULL; fieldPtr = fieldPtr->next) {
	fieldName = ( char *) fieldPtr->data;
	errCode = addField( rowcol, record, fieldName, declList, okBtn);
    }
    return( errCode);
}

/**************************************************************************
 * 
 * Description:
 * 	if the dialog has been destroyed, restore the callback which 
 *	recreates it 
 * Arguments:
 *  	launchBtn:	button which launches the sub dialog
 *  	record:		record to edit
 *  	decl:		declaration of the dialog.
 *  	declList:	list of declarations to edit in dialog.
 *  	parentOkBtn:	ok button of parent dialog 
 *
 *************************************************************************/

void AmRestoreCreateSubDialogCallback( Widget		,
				     Widget		launchBtn,
				     AmRecordClass	*record,
				     AmDialogEntryPtr	decl,
				     AmListPtr		declList,
				     Widget		parentOkBtn,
				     XtPointer		)
{
    AmAddVaCallback( launchBtn, XmNactivateCallback,
		   ( VarargsCallback) subDialogCallback,
		   record, decl, declList, parentOkBtn, END_LIST);
    XtSetSensitive( launchBtn, TRUE);
}

/**************************************************************************
 * 
 * Description:
 * 	callback to launch a sub dialog to edit a list of field
 *	declarations.
 * Arguments:
 *  	button:		button which launches the sub dialog
 *  	record:		record to edit
 *  	decl:		declaration of the dialog.
 *  	declList:	list of declarations to edit in dialog.
 *  	parentOkBtn:	ok button of parent dialog 
 *	callData:	unused
 *
 *************************************************************************/

/* ARGSUSED */

static void 
subDialogCallback( Widget		button,
		   AmRecordClass	*record,
		   AmDeclItemPtr	declItem,
		   AmListPtr		declList,
		   Widget		parentOkBtn,
		   XtPointer		callData)
{
    Widget		parent = AmGetWidgetShell( button), formDlog,
			fieldRowcol, actionForm;
    Arg			args[ 20];
    int			n;
    AmActionRec         action;
    char		*popupName;

/* remove the callback that launched this dialog */

    AmRemoveVaCallback( button, XmNactivateCallback,
		        ( VarargsCallback) subDialogCallback,
		        record, declItem, declList, parentOkBtn, END_LIST);
    XtSetSensitive( button, FALSE);

/* create a dialog shell and a form inside of it */

    n = AmMakeArgList( args, 
		       XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		       XmNdeleteResponse, XmDESTROY,
		       NULL);
    popupName = ( char *) declItem->fields->data;
    formDlog = XmCreateFormDialog( parent, popupName, args, n);
    fieldRowcol = XtVaCreateManagedWidget( "fieldRowcol", 
					   xmRowColumnWidgetClass,
					   formDlog, NULL);
    fillRowcol( fieldRowcol, record, declItem->fields->next, 
		declList, parentOkBtn);

/* make the panel the contains the fields and OK and cancel buttons */

    XtVaSetValues( fieldRowcol, 
		   StickLeft, StickRight, StickTop, StickBottom,
		   XmNbottomOffset, 60,
		   NULL);
    actionForm = AmMakeActionArea( formDlog, dialogDefs, 
			           ELEMENTS( dialogDefs),
                                   &action);
    XtVaSetValues( actionForm,
		   StickLeft, StickRight, StickBottom,
		   NULL);

/* if the user hits OK, we unmanage the dialog, and install a callback to
   remanage it.  If he hits cancel, then we destroy the dialog */

    AmAddVaCallback( action.buttons[ 0], XmNactivateCallback,
		   ( VarargsCallback) AmUnmanageSubDialogCallback,
		   button, formDlog, 
		   END_LIST);
    XtAddCallback( action.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
    AmAddVaCallback( formDlog, XmNdestroyCallback,
		   ( VarargsCallback) AmRestoreCreateSubDialogCallback,
		   button, record, declItem, declList, parentOkBtn,
		   END_LIST);
    XtManageChild( formDlog);
}

/**************************************************************************
 * 
 * Description:
 * 	create the button and set up the callback to launch a subdialog
 * Arguments:
 *  	rowcol:		put the button in here.
 *  	record:		record to edit
 *  	decl:		declaration of the dialog.
 *  	declList:	list of declarations to edit in dialog.
 *  	okBtn:		ok button of parent dialog 
 *
 *************************************************************************/

static AmErrorCode
popupDialog( Widget		rowcol,
	     AmRecordClass	*record,
	     AmDeclItemPtr	declItem,
	     AmListPtr		declList,
	     Widget		okBtn) 
{
    Widget	button;
    char	*popupName;

/* the first field is the name of the popup */

    popupName = ( char *) declItem->fields->data;
    button = XtVaCreateManagedWidget( popupName, xmPushButtonWidgetClass,
				      rowcol, NULL);
    AmAddVaCallback( button, XmNactivateCallback,
		    ( VarargsCallback) subDialogCallback,
		    record, declItem, declList, okBtn, END_LIST);
    return( AmOK);
}

/**************************************************************************
 * 
 * Description:
 * 	create a box and fill it with widgets to edit the fields
 *	in a record.
 * Arguments:
 *  	rowcol:		put the widgets in here.
 *  	record:		record to edit
 *  	decl:		declaration of the dialog.
 *  	declList:	list of declarations to edit in dialog.
 *  	okBtn:		ok button of parent dialog 
 *
 *************************************************************************/

static AmErrorCode
boxDialog( Widget		rowcol,
	   AmRecordClass	*record,
	   AmDeclItemPtr	declItem,
	   AmListPtr		declList,
	   Widget		okBtn)
{
    Widget	frame, subRowcol, frameLabel;
    char	*frameName;

/* the first field is the name of the frame */

    frameName = ( char *) declItem->fields->data;
    frame = XtVaCreateManagedWidget( "frame", xmFrameWidgetClass, rowcol,
				     NULL);
    frameLabel = XtVaCreateManagedWidget( frameName, xmLabelWidgetClass,
					  frame,
					  XmNchildType, XmFRAME_TITLE_CHILD,
					  NULL);
    subRowcol = XtVaCreateManagedWidget( "subRowcol", xmRowColumnWidgetClass,
					 frame, NULL);
    return( fillRowcol( subRowcol, record, declItem->fields->next, 
			declList, okBtn));
}


/**************************************************************************
 * 
 * Description:
 * 	conditionally create a dialog, depending on a simplified
 *	boolean expression, which tests for a NULL value.
 * Arguments:
 *  	rowcol:			put the widgets in here.
 *  	record:			record to edit
 *  	conditionalItem:	declaration of the dialog.
 *  	declList:		list of declarations to edit in dialog.
 *  	okBtn:			ok button of parent dialog 
 * Modifications:
 *	memory leak associated with pulling the value from the record.
 *
 *************************************************************************/

static AmErrorCode
conditionalDialog( Widget		rowcol,
		   AmRecordClass	*record,
		   AmConditionalItemPtr	conditionalItem,
		   AmListPtr		declList,
		   Widget		okBtn)
{
    char		*testTagName;
    void		*value;
    AmDialogEntryPtr	fieldDecl;
    AmListPtr		fieldPtr;

    testTagName = conditionalItem->boolExpr->tag;
    record->GetVaFields( testTagName, &value, NULL);
    if ((( conditionalItem->boolExpr->op == AmBOOLEAN_EQUAL) &&
	 ( value == NULL)) ||
	(( conditionalItem->boolExpr->op == AmBOOLEAN_NOT_EQUAL) &&
	 ( value != NULL))) {
	for ( fieldPtr = conditionalItem->trueDeclList;
	      fieldPtr != NULL;
	      fieldPtr = fieldPtr->next) {
	    fieldDecl = ( AmDialogEntryPtr) fieldPtr->data;
	    makeDecl( rowcol, record, fieldDecl, declList, okBtn);
	}
    } else if ( conditionalItem->falseDeclList != NULL) {
	for ( fieldPtr = conditionalItem->falseDeclList;
	      fieldPtr != NULL;
	      fieldPtr = fieldPtr->next) {
	    fieldDecl = ( AmDialogEntryPtr) fieldPtr->data;
	    makeDecl( rowcol, record, fieldDecl, declList, okBtn);
	}
    }

/* free the test value if it's a vector */

    if ( !AmIsAtomic( AmGetTagType( testTagName))) {
	free( value);
    }
    return( AmOK);
}
/**************************************************************************
 * 
 * Description:
 * 	create a custom dialog.
 * Arguments:
 *  	rowcol:		put the widgets in here.
 *  	record:		record to edit
 *  	declItem:	declaration of the dialog.
 *  	declList:	list of declarations to edit in dialog.
 *  	okBtn:		ok button of parent dialog 
 * Modifications:
 *	memory leak associated with pulling the value from the record.
 *
 *************************************************************************/

static AmErrorCode
customEditorDialog( Widget		rowcol,
		    AmRecordClass	*record,
		    AmDeclItemPtr	declItem,
		    AmListPtr		declList,
		    Widget		okBtn)
{
    int		ii, index;
    Widget	button;
    char	**argvec, *buttonName;

    index = -1;
    for ( ii = 0; ii < (signed)VECLEN( customEditors); ii++) {
	if ( !strcmp( customEditors[ ii].name, declItem->name)) {
            index = ii;
            break;
        }
    }
    if ( index == -1) {
        AmPrintResourceError( "cantFindEditor", declItem->name);
        return( AmBAD_ARGUMENT);
    }
    buttonName = ( char *) declItem->fields->data;
    button = XtVaCreateManagedWidget( buttonName, xmPushButtonWidgetClass,
				      rowcol, NULL);
    if ( declItem->fields == NULL) {
	XtAddCallback( button, XmNactivateCallback,
		       ( XtCallbackProc) customEditors[ index].editor, record);
    } else {
	argvec = ( char **) AmMakeVectorLst( declItem->fields->next);


/* register a callback for the custom editor, and a callback to free its
   arguments when it's popped down */

        AmAddVaCallback( button, XmNactivateCallback,
                         ( VarargsCallback) customEditors[ index].editor,
                         record, argvec, END_LIST);
        XtAddCallback( button, XmNdestroyCallback,
                       ( XtCallbackProc) AmFreeStructPtrCallback, argvec);
    }
    return( AmOK);
}

static void
makeDecl( Widget		rowcol,
	  AmRecordClass		*record,
	  AmDialogEntryPtr	pDialogEntry,
	  AmListPtr		declList,
	  Widget		okBtn)
{
    if (( pDialogEntry->type == AmDIALOG_IF_THEN) ||
	( pDialogEntry->type == AmDIALOG_IF_THEN_ELSE)) {
	conditionalDialog( rowcol, record,
			   &pDialogEntry->u.conditionalItem, declList, okBtn);
    } else if ( pDialogEntry->type == AmDIALOG_BOX_FIELD_LIST) {
	boxDialog( rowcol, record,
		     &pDialogEntry->u.declItem, declList, okBtn);
    } else if ( pDialogEntry->type == AmDIALOG_POPUP_FIELD_LIST) {
	popupDialog( rowcol, record,
		     &pDialogEntry->u.declItem, declList, okBtn);
    } else if ( pDialogEntry->type == AmDIALOG_EDITOR) {
	customEditorDialog( rowcol, record,
			    &pDialogEntry->u.declItem, declList, okBtn);
    } else if ( pDialogEntry->type == AmDIALOG_IDENT) {
	addField( rowcol, record, pDialogEntry->u.identItem, declList, okBtn);
    }
}

/* given a parent widget, a record to edit, and a dialog declaration,
   create a dialog panel */

static Widget
makePanel( Widget         	parent,
           AmRecordClass    	*record,
           AmDeclarationPtr	pDecl,
           AmListPtr        	declList,
           Widget         	okBtn)
{
    Widget		rowcol;
    AmListPtr		pDeclList;
    AmDialogEntryPtr	pDialogEntry;

    rowcol = XtVaCreateManagedWidget( "rowcol", xmRowColumnWidgetClass, 
				      parent, NULL);
    for ( pDeclList = pDecl->declList; 
	  pDeclList != NULL;
	  pDeclList = pDeclList->next) {
	pDialogEntry = ( AmDialogEntryPtr) pDeclList->data;
	makeDecl( rowcol, record, pDialogEntry, declList, okBtn);
    }
    return( rowcol);
}

/* free the dialog entry structure */

/* ARGSUSED */

void
AmDestroyDialogEntryCallback( Widget    	w,
                              AmListPtr   	lstDecl,
                              XtPointer 	xpCallData)
{
    AmListPtr		ptr;
    AmDeclarationPtr	decl;

    for ( ptr = lstDecl; ptr != NULL; ptr = ptr->next) {
	decl = ( AmDeclarationPtr) ptr->data;
	AmFreeLst( decl->declList, ( AmItemFn) AmFreeDialogEntry);
	free( decl->ident);
	free( decl);
    }
    AmJustFreeLst( lstDecl);
}

/* Update the motif display when the values have been updated, and the 
   AM_FINAL event is sent by slick */

/* ARGSUSED */

static void
flushDisplayCallback( int               	iEventID,
                      AmMotifDisplayClass	*pMotifDisplay,
		      AmScreenObjClass		*pScreenObj,
                      XtPointer         	xpCallData)
{
    if ( !pScreenObj->IsValid()) {
	AmPrintLogError( pMotifDisplay->GetShell(),
		         "editObjectError", pScreenObj->GetName());
    }
    pMotifDisplay->Flush();
}


/* Edit a selected object. */

/* ARGSUSED */

AmErrorCode
AmEditScreenObject( Widget              w,
                    AmMotifDisplayClass *pMotifDisplay,
                    AmScreenObjClass    *pScreenObj,
                    XtPointer           xpCallData)
{
    Widget              	wRowCol, wForm, wDialog;
    AmDeclarationPtr		pDecl;
    const char			*szName;
    char	        	*szPath, szError[ MSGLEN], filebuf[ PATHLEN];
    AmListPtr			lstDecl;
    AmEditDataPtr		pEditData;
    AmActionRec           	action;
    Arg				args[ 20];
    int				n;

/* initialize the log to record any errors from the edit */
 
    AmInitLog();

/* take the object type, and turn it into a string */

    szName = pScreenObj->getObjectTypeName(); 
    if ( szName == NULL) {
        AmAddResourceToLog( "cantGetName");
        DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }
    sprintf( filebuf, "interp/%s", szName);

/* read the interpreter file into a list of declarations */

    szPath = AmGetEnvFile( CONFIG_ENV, filebuf);
    if ( szPath == NULL) {
	AmAddResourceToLog( "cantFindFile", filebuf);
	DO_RETURN_ERROR( AmFILE_NO_EXIST);
    }

    if ( AmParseDialogFile( szPath, &lstDecl, szError) != AmOK) {
	AmAddResourceToLog( "parseFileFailed", filebuf);
	DO_RETURN_ERROR( AmPARSE_FAILED);
    }


    pDecl = findDecl( szName, lstDecl);
    if ( pDecl == NULL) {
	AmAddResourceToLog( "cantFindDeclaration", szName);
        DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }

/* Before creating the panel, we add a user callback which causes the plot
   to update when the user hits the OK button. */

    pEditData = ( AmEditDataPtr) malloc( sizeof( AmEditDataRec));
    if ( pEditData == NULL) {
        AmAddResourceToLog( "mallocFailed", sizeof( AmEditDataRec));
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    n = AmMakeArgList( args, 
		       XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		       XmNdeleteResponse, XmDESTROY,
                       XmNvisual, AmGetAppVisual(),
                       XmNcolormap, AmGetAppColormap(),
                       XmNdepth, AmGetAppDepth(),
		       NULL);
    wDialog = XmCreateFormDialog( AmGetWidgetShell( w), "editObject", args, n);

/* Setup the action area. */

    wForm = AmMakeActionArea( wDialog, actionDefs, ELEMENTS( actionDefs),
                              &action);
    pEditData->object = pScreenObj;
    pEditData->motifDisplay = pMotifDisplay;
    pEditData->extents = *( pScreenObj->GetExtentsRect()); 

    wRowCol = makePanel( wDialog, pScreenObj, pDecl, lstDecl, 
			 action.buttons[ 0]);

/* Do the alignments for the field rowcolumn and the action area */

    AmAlignObjects( wRowCol);
    XtVaSetValues( wRowCol, 
		   StickLeft, StickRight, StickTop,
		   NULL);
    XtVaSetValues( wForm, BelowOf( wRowCol), NULL);

/* set up the OK and revert buttons, and the user event handler for
   the "final" OK */

    AmSetOkButton( action.buttons[ 0], pScreenObj);
    AmSetRevertButton( action.buttons[ 1]);
    AmAddVaCallbackWidget( AM_PKG_ID, AM_FINAL, wDialog, 
		           ( VarargsCallback) flushDisplayCallback,
		           pMotifDisplay, pScreenObj, 
			   END_LIST);

/* Add a callback to destroy the dialog entry structure when the dialog is
   popped down */

    XtAddCallback( action.buttons[ 2], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, wDialog);
    XtAddCallback( wDialog, XmNdestroyCallback,
		   ( XtCallbackProc) AmDestroyDialogEntryCallback, lstDecl);
    XtAddCallback( wDialog, XmNdestroyCallback,
		   ( XtCallbackProc) AmFreeStructPtrCallback, pEditData);
    XtManageChild( wDialog);
    return( AmOK);
}

/* edit object handler called on button press */

static void editObjectHandler( Widget            w,
			       AmMotifDisplayClass *pMotifDisplay,
			       XButtonEvent      *xEvent,
			       AnvilBoolean           *)
{
    AmScreenObjClass *pScreenObj;

/* Find the object that the mouse press is in */

    pScreenObj = pMotifDisplay->FindObject( xEvent->x, xEvent->y);
    if ( pScreenObj != NULL) {
	AmEditScreenObject( w, pMotifDisplay, pScreenObj, NULL);
    }
}

/* reset to a normal cursor when the edit object handler is removed */

static void
editObjectRemoveFn( AmMotifDisplayClass *pMotifDisplay)
{
    pMotifDisplay->SetCursor( XC_arrow);
}

/* callback to edit an object */

/* ARGSUSED */
void
AmEditObjectCallback( Widget            w,
                    AmMotifDisplayClass *pMotifDisplay,
                    XtPointer         xpCallData)
{
    AmAddHandler( w, pMotifDisplay->GetArea(),
                ButtonPressMask,
		( XtEventHandler) editObjectHandler,
                pMotifDisplay, EDIT_OBJECT_PRIORITY,
                ( AmStateHandlerFn) editObjectRemoveFn);

    pMotifDisplay->SetCursor( XC_pencil);
}
