#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/ScrolledW.h>
#include <anvil/motif.h>
#include <anvil/utility.h>

static AmActionDefRec actionAreaDef[] = {
    { "cancelBtn", ( XtCallbackProc) AmPopdownCback, NULL },
    { "okBtn", NULL, NULL }
};

/* create a scrolled rowcolumn of buttons containing pixmaps read from files
   specified by a vector of strings */

Widget createSymbolList( Widget	parent,
			 char   **symbolVec,
			 Widget	**symbolBtns)
{
    Widget	scrollWin, rowcol, rowForm, symbolBtn;
    Display	*display = XtDisplay( parent);
    Window	window = DefaultRootWindow( display);
    int		xHot, yHot;
    uint_t	width, height;
    Pixmap	bitmap;


    scrollWin = XtVaCreateManagedWidget( "scrollWin",
					 xmScrolledWindowWidgetClass, parent,
					 NULL);
    rowcol = XtVaCreateManagedWidget( "rowcol",
				      xmRowColumnWidgetClass,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* allocate a vector of widgets for the buttons, so we can add callbacks */

    *symbolBtns = ( Widget *) malloc( sizeof( Widget)*AmVeclen( symbolVec));
    for ( ii = 0; symbolVec[ ii] != NULL; ii++) {

/* if we can read the bitmap, put it in the button, otherwise, just put up
   an empty button */

	if ( XReadBitmapFile( display, window, symbolVec[ ii],
			      &width, &height, &bitmap, &xHot, &yHot)) {
	    ( *symbolBtns)[ ii] = 
		XtVaCreateManagedWidget( "symbolBtn", 
					 xmPushButtonWidgetClass,
					 symbolRowcol,
					 XmNlabelType,
					     XmPIXMAP,
					 XmNlabelPixmap, bitmap,
					 NULL):
	} else {
	    ( *symbolBtns)[ ii] = 
		XtVaCreateManagedWidget( "symbolBtn", 
					 xmPushButtonWidgetClass,
					 symbolRowcol,
					 NULL):
	}
    }
}

	


    


void EditSymbolListCallback( Widget		btn,
			     AmDataGraphicClass	*graphic,
			     XtPointer)
{
    Display	*display = XtDisplay( btn);
    Window	win = DefaultRootWindow( display);
    Widget	formDlog, symbolRowcol, symbolBtn, symbolScroll,
		legendRowcol, legendBtn, legendScroll, okBtn, cancelBtn,
		actionForm, symbolRowForm, symbolLabel, *symbolMinTexts,
		*symbolMaxTexts, *symbolTexts, symbolCmpText;
    Arg		args[ 20];
    int		n, *table, numSymbols;
    float	*binvec;
    AmActionRec	actionArea;
    char	**plotSymbols, **availSymbols, symbolListPath[ PATHLEN],
		**symbolStrings;

/* make sure that it's a scatter graph or scatter plot that we're passed */

    if (( graphic->GetObjectType() != AmSCATTER_PLOT) ||
	( graphic->GetObjectType() != AmSCATTER_GRAPH)) {
	AmPrintResourceError( "symbolListEditNotScatter");
	return;
    }

/* read the list of available symbols */

    if (( configDir = getenv( CONFIG_ENV)) == NULL) {
	AmPrintResourceError( "configDirNotDefined", CONFIG_ENV);
	return;
    }
    sprintf( symbolListPath, "%s/%s/%s", 
	     CONFIG_ENV, AM_SYMBOL_DIR, AM_SYMBOL_LIST_FILE);
    availSymbols = AmReadStrvec( symbolListPath); 
    if ( availSymbols == NULL) {
	AmPrintResourceError( "failedToReadSymbolList", symbolListPath);
	return;
    }

/* get the list of symbols from the plot */

    graphic->GetVaFields( AmNsymbolList, &plotSymbols,
			  AmNsymbolTable, &symbolTable,
			  AmNsymbolBinvec, &symbolBinvec,
			  AmNsymbolStrings, &symbolStrings,
			  NULL);
    if ( plotSymbols == NULL) {
	AmPrintResourceError( "sourcePlotNoSymbolList");
	return;
    }

/* create the dialog and action form */

    n = AmMakeArgList( args,
		     XmNdeleteResponse, XmDESTROY,
		     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		     XmNautoUnmanage, FALSE,
		     NULL);
    formDlog = XmCreateFormDialog( AmGetWidgetRealShell( btn), "editSymbol", 
				   args, n);
    actionForm = makeActinoArea( formDlog,
				 actionAreaDef, VECLEN( actionAreaDef),
				 &actionArea);

/* create a scrolled rowcolumn to display the contents of the legend */

    legendScroll = XtVaCreateManagedWidget( "legendScoll",
					    xmScrolledWindowWidgetClass,
					    formDlog,
					    StickLeft, StickTop,
					    AboveOf( actionForm),
					    NULL);
    legendRowcol = XtVaCreateManagedWidget( "legendRowcol",
					    xmRowColumnWidgetClass,
					    XmNorientation, XmVERTICAL,
					    NULL);

/* allocate the text vectors so we can save them */

    numSymbols = AmVeclen( plotSymbols);
    if ( symbolTable != NULL) {
	symbolTexts = ( Widget *) malloc( numSymbols);
    } else {
	symbolMinTexts = ( Widget *) malloc( numSymbols);
	symbolMaxTexts = ( Widget *) malloc( numSymbols);
    }
    for ( ii = 0; ii < numSymbols; ii++) {
	symbolRowForm = XtVaCreateManagedWidget( "symbolRowForm",
						 xmFormWidgetClass,
						 legendRowcol,
						 NULL);
	

/* if we can read the bitmap, put it in the button, otherwise, just put up
   an empty button */

	if ( XReadBitmapFile( display, window, plotSymbols[ ii],
			      &width, &height, &bitmap, &xHot, &yHot)) {
	    symbolBtn = XtVaCreateManagedWidget( "symbolBtn", 
						 xmPushButtonWidgetClass,
						 symbolRowForm,
						 XmNlabelType,
						     XmPIXMAP,
						 XmNlabelPixmap, bitmap,
						 StickLeft, StickTop,
						 StickBottom,
						 NULL):
	} else {
	    symbolBtn = XtVaCreateManagedWidget( "symbolBtn", 
						 xmPushButtonWidgetClass,
						 symbolRowForm,
						 StickLeft, StickTop,
						 StickBottom,
						 NULL):
	}

/* if the symbol plot is table based, then display the strings for 
   each category next to the symbol */

	if ( symbolTable != NULL) {
	    if ( symbolStrings != NULL) {
		strcpy( buffer, symbolStrings[ ii]);
	    } else {
		sprintf( buffer, "%d", symbolTable[ ii]);
	    }
	    symbolTexts[ ii] = 
		XtVaCreateManagedWidget( "symbolText",
					 xmTextFieldWidgetClass,
					 XmNcolumns, 20,
					 RightOf( symbolBtn),
					 StickTop, StickBottom,
					 XmNvalue, buffer,
					 NULL);
	} else if ( symbolBinvec != NULL) {
	    if ( ii != 0) {
		symbolMinTexts[ ii] = 
		    XtVaCreateManagedWidget( "symbolMinText",
					     xmTextFieldWidgetClass,
					     XmNcolumns, 20,
					     RightOf( symbolBtn),
					     StickTop, StickBottom,
					     XmNvalue, buffer,
					     NULL);
	   } else if 
		

		

	


