#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/ScrolledW.h>
#include <Xm/List.h>
#include <Xm/TextF.h>
#include <Xm/DrawingA.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/vector.h>
#include <anvil/errlib.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/va_cback.h>
#include <anvil/envdefs.h>
#include <anvil/plot_cback.h>
#include <anvil/linegraph.h>
#include <anvil/legend.h>
#include <anvil/linelegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/polylegend.h>
#include <anvil/imagelegend.h>
#include <anvil/tick.h>
#include <anvil/textcheck.h>
#include <anvil/interp.h>
#include <anvil/userevents.h>
#include <anvil/slick.h>
#include <anvil/slickP.h>
#include <anvil/xfonts.h>
#include <anvil/option.h>
#include <anvil/textcheck.h>
#include <anvil/utility.h>

static AmActionDefRec actionDefs[] = {
    { "okBtn", NULL, NULL },
    { "revertBtn", NULL, NULL },
    { "cancelBtn", ( XtCallbackProc) AmPopdownCback, NULL }
};

typedef struct infoRec_ {
    Widget         gfx;
    Widget         familyOption;
    Widget         sizeOption;
    Widget         sizeText;
    Widget         weightToggle;
    Widget         slantToggle;
    Widget         xScaleText;
    Widget         yScaleText;
    int            numFamilies;
    WidgetList     familyButtons;
    int            numSizes;
    WidgetList     sizeButtons;
    const char     **families;
    GC             gc;
    XFontStruct    *fontInfo; 
    AmPlotClass      *plot;
    char           *name;
    int            size;
    AmFontWeightEnum weight;
    AmFontSlantEnum  slant;
    float          xScale;
    float          yScale;
    int            dpi;
} infoRec, *infoPtr;

AnvilBoolean AmFontDialogCallback( Widget    w,
                            AmPlotClass *plot,
                            char      **args,
                            XtPointer);

const char *fontSizes[] = {
    "6",
    "8",
    "10",
    "12",
    "14",
    "18",
    "24",
    "32",
    "36",
    "48",
    "64",
    "96",
    "Custom",
    NULL,
};


void
AmDrawFont( Widget    gfxArea,
          infoPtr   info,
          XtPointer)
{
    XFontStruct  *fontInfo;
    GC           gc;
    XGCValues    gcv;
    XCharStruct  overall;
    ulong_t      fg, bg;
    int          dir, ascent, descent, width;

/* Make sure a font is loaded. */

    if ( info->fontInfo == NULL) {
        info->fontInfo = AmLoadQueryFont( XtDisplay( gfxArea),
                                            info->name, info->size,
                                            info->weight, info->slant,
                                            info->xScale, info->yScale,
                                            info->dpi);
    }


    fontInfo = info->fontInfo;

/* Make sure a GC exists. */

    if ( info->gc == NULL) {

        XtVaGetValues( gfxArea,
            XmNforeground, &fg,
            XmNbackground, &bg,
            NULL);

        gcv.foreground = fg;
        gcv.background = bg;
        gcv.font = fontInfo->fid;
        info->gc = XCreateGC( XtDisplay( gfxArea), XtWindow( gfxArea),
                    GCForeground | GCBackground | GCFont, &gcv);
    } 


    gc = info->gc;

    XTextExtents( fontInfo, "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 26,
        &dir, &ascent, &descent, &overall);

    width = overall.width;

    XTextExtents( fontInfo, "abcdefghijklmnopqrstuvwxyz", 26,
        &dir, &ascent, &descent, &overall);

    if ( overall.width > width) {
        width = overall.width;
    }

    XtVaSetValues( info->gfx,
        XmNheight, (fontInfo->ascent + fontInfo->descent) * 2,
        XmNwidth, width,
        NULL);

    XClearWindow( XtDisplay( gfxArea), XtWindow( gfxArea));
    XDrawString( XtDisplay( gfxArea), XtWindow( gfxArea), gc,
        0, fontInfo->ascent, "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 26);

    XDrawString( XtDisplay( gfxArea), XtWindow( gfxArea), gc,
        0, fontInfo->ascent + fontInfo->ascent + fontInfo->descent,
        "abcdefghijklmnopqrstuvwxyz", 26);
}


void
AmChangeFont( infoPtr info) {
    XGCValues    gcv;
    ulong_t      fg, bg;

/* Make sure a font is loaded. */

    if ( info->fontInfo != NULL) {
        XUnloadFont( XtDisplay( info->gfx), info->fontInfo->fid);
        XFreeFontInfo( NULL, info->fontInfo, 1);
    }

    info->fontInfo = AmLoadQueryFont( XtDisplay( info->gfx),
                         info->name, info->size,
                         info->weight, info->slant,
                         info->xScale, info->yScale,
                         info->dpi);

/* Make sure a GC exists. */

    if ( info->gc == NULL) {

        XtVaGetValues( info->gfx,
            XmNforeground, &fg,
            XmNbackground, &bg,
            NULL);

        gcv.foreground = fg;
        gcv.background = bg;
        gcv.font = info->fontInfo->fid;
        info->gc = XCreateGC( XtDisplay( info->gfx), XtWindow( info->gfx),
                    GCForeground | GCBackground | GCFont, &gcv);

    }  else {

        gcv.font = info->fontInfo->fid;
        XChangeGC( XtDisplay( info->gfx), info->gc, GCFont, &gcv); 
    
    }

    AmDrawFont( info->gfx, info, NULL);
}


static void
familyChangedCallback( int       ,
                       infoPtr   info,
                       int       pos)
{
    if ( info->name != NULL) {
        free( info->name);
    }
    info->name = strdup( info->families[ pos]);
    AmChangeFont( info);
}


static void
sizeChangedCallback( int       ,
                     infoPtr   info,
                     int       pos)
{
    char buffer[ TEXTLEN];
    int  size;

    if ( !strcmp( fontSizes[ pos], "Custom")) {
        return;
    }

    size = atoi( fontSizes[ pos]);

    if ( size < 2) {
        size = 2;
    }

    info->size = size;

    sprintf( buffer, "%2d", size);
    XtVaSetValues( info->sizeText, XmNvalue, buffer, NULL);

    AmChangeFont( info);
}


static void
sizeTextChangedCallback( Widget    text,
                         infoPtr   info,
                         XtPointer)
{
    char       *value;
    int        size;

    XtVaGetValues( text, XmNvalue, &value, NULL);
    size = atoi( value);

    if ( size < 2) {
        size = 2;
    }

/*
 * We know the AmLast widget in the option menu has the "custom" entry in
 *   it... so set the value of the option widget to that.
 */
  
    XtVaSetValues( info->sizeOption,
        XmNmenuHistory, info->sizeButtons[ info->numSizes - 1],
        NULL);

    info->size = size;
    AmChangeFont( info);
}


static void
xScaleTextChangedCallback( Widget    text,
                           infoPtr   info,
                           XtPointer)
{
    char       *value, buffer[ TEXTLEN];
    float      scale;

    XtVaGetValues( text, XmNvalue, &value, NULL);
    scale = atof( value);

    if ( scale <= 0) {
        scale = 1.0;
        sprintf( buffer, "%4.2f", scale);
        XtVaSetValues( text, XmNvalue, buffer, NULL);
    }

    info->xScale = scale;
    AmChangeFont( info);
}


static void
yScaleTextChangedCallback( Widget    text,
                           infoPtr   info,
                           XtPointer)
{
    char       *value, buffer[ TEXTLEN];
    float      scale;

    XtVaGetValues( text, XmNvalue, &value, NULL);
    scale = atof( value);

    if ( scale <= 0) {
        scale = 1.0;
        sprintf( buffer, "%4.2f", scale);
        XtVaSetValues( text, XmNvalue, buffer, NULL);
    }

    info->yScale = scale;
    AmChangeFont( info);
}


static void
weightToggleCallback( Widget    toggle,
                      infoPtr   info,
                      XtPointer)
{
    AnvilBoolean value; 

    XtVaGetValues( toggle, XmNset, &value, NULL);

    if ( value == TRUE) {
        info->weight = AmWEIGHT_BOLD;
    } else {
        info->weight = AmWEIGHT_MEDIUM;
    }

    AmChangeFont( info);
}


static void
slantToggleCallback( Widget    toggle,
                     infoPtr   info,
                     XtPointer)
{
    AnvilBoolean value; 

    XtVaGetValues( toggle, XmNset, &value, NULL);

    if ( value == TRUE) {
        info->slant = AmSLANT_ITALIC;
    } else {
        info->slant = AmSLANT_ROMAN;
    }

    AmChangeFont( info);
}

static void
fontEditOkCallback( int       , 
                    Widget    formDlog,
                    char      *nameTag,
                    char      *sizeTag,
                    char      *weightTag,
                    char      *slantTag,
                    char      *xScaleTag,
                    char      *yScaleTag,
                    infoPtr   info,
                    AmOkArgsPtr okArgs)
{
    AmSetTagValue( ( void *)(info->name), AmSTRING, nameTag, okArgs);
    AmSetTagValue( &(info->size), AmINT, sizeTag, okArgs);
    AmSetTagValue( &(info->weight), AmENUM, weightTag, okArgs);
    AmSetTagValue( &(info->slant), AmENUM, slantTag, okArgs);
    AmSetTagValue( &(info->xScale), AmFLOAT, xScaleTag, okArgs);
    AmSetTagValue( &(info->yScale), AmFLOAT, yScaleTag, okArgs);

    XtDestroyWidget( XtParent( formDlog));
}


static void
fontEditRevertCallback( Widget    button,
                        AmPlotClass *plot,
                        char      *nameTag,
                        char      *sizeTag,
                        char      *weightTag,
                        char      *slantTag,
                        char      *xScaleTag,
                        char      *yScaleTag,
                        infoPtr   info,
                        XtPointer)
{
    char    buffer[ TEXTLEN];
    int     ii;
    AnvilBoolean state;

    if ( info->name != NULL) {
        free( info->name);
    }

    plot->GetVaFields( nameTag, &(info->name),
              sizeTag, &(info->size),
              weightTag, &(info->weight),
              slantTag, &(info->slant),
              xScaleTag, &(info->xScale),
              yScaleTag, &(info->yScale),
              NULL);

    for ( ii = 0; info->families[ ii] != NULL; ii++) {
        if ( !strcmp( info->families[ ii], info->name)) {
            break;
        }
    }

    if ( info->families[ ii] == NULL) {
        ii = 0;
    }

    XtVaSetValues( info->familyOption,
        XmNmenuHistory, info->familyButtons[ ii], NULL);

    sprintf( buffer, "%2d", info->size);
    XtVaSetValues( info->sizeText, XmNvalue, buffer, NULL);

    for ( ii = 0; fontSizes[ ii] != NULL; ii++) {
        if ( atoi( fontSizes[ ii]) == info->size) {
            break;
        }
    }

    if ( fontSizes[ ii] == NULL) {
        ii--;
    } 

    XtVaSetValues( info->sizeOption,
        XmNmenuHistory, info->sizeButtons[ ii], NULL);

    sprintf( buffer, "%4.2f", info->xScale);
    XtVaSetValues( info->xScaleText, XmNvalue, buffer, NULL);

    sprintf( buffer, "%4.2f", info->yScale);
    XtVaSetValues( info->yScaleText, XmNvalue, buffer, NULL);

    if ( info->weight == AmWEIGHT_BOLD) {
        state = TRUE;
    } else {
        state = FALSE;
    }

    XtVaSetValues( info->weightToggle, XmNset, state, NULL);

    if ( info->slant == AmSLANT_ITALIC) {
        state = TRUE;
    } else {
        state = FALSE;
    }

    XtVaSetValues( info->slantToggle, XmNset, state, NULL);

    AmChangeFont( info);
}

static void
fontEditDestroyCallback( Widget    formDlog,
                         infoPtr   info,
                         XtPointer)
{
    if ( info->name != NULL) {
        free( info->name);
    }

    if ( info->fontInfo != NULL) {
        XUnloadFont( XtDisplay( info->gfx), info->fontInfo->fid);
        XFreeFontInfo( NULL, info->fontInfo, 1);
    }

    if ( info->gc != NULL) {
        XFreeGC( XtDisplay( info->gfx), info->gc);
    }

    free( info);
}

/*******************************************************************
*
 * Function: AmCreateFontEditDialog( parent, plot, nameTag, sizeTag, weightTag,
 *               slantTag, xScaleTag, yScaleTag, dialog, okBtn)
 *
 * Description:
 *     Creates a dialog used to edit a font specified by the fields
 *     as indicated by the six passed in tags.  The result is returned
 *     through the passed in dialog and okBtn variables.
 *
 *******************************************************************/

void AmCreateFontEditDialog( Widget    parent,
                           AmPlotClass *plot,
                           char      *nameTag,
                           char      *sizeTag,
                           char      *weightTag,
                           char      *slantTag,
                           char      *xScaleTag,
                           char      *yScaleTag,
                           Widget    *dialog,
                           Widget    *okBtn)
{
    AmActionRec           actionArea;
    Widget		formDlog,
                        familyForm, familyLabel,
                        sizeForm, sizeLabel,
                        toggleForm,
                        scaleForm, xScaleLabel, yScaleLabel,
                        gfxWin, gfxForm, menu, actionForm;
    Dimension           width, maxWidth;
    AnvilBoolean             state;
    infoPtr             info;
    Arg			args[ 20];
    char                buffer[ TEXTLEN];
    const char          **family;
    int                 ii, n;

/* Also serves as failure flag. */

    *dialog = NULL;
    info = ( infoPtr) malloc( sizeof( infoRec));

/* Get the pertinent fields from the plot we want to set. */

    info->name = NULL;
    if (plot->GetVaFields( nameTag, &(info->name),
                           sizeTag, &(info->size),
                           weightTag, &(info->weight),
		           slantTag, &(info->slant),
                           xScaleTag, &(info->xScale),
		           yScaleTag, &(info->yScale),
                           NULL) != 0) {

        if ( info->name != NULL) {
            free( info->name);
        }

	AmPrintResourceError( "wrongTagsForFontEdit");
        return;
    }

/* Create a form dialog to contain everything. */

    n = AmMakeArgList( args,
            XmNdeleteResponse, XmDESTROY,
            XmNautoUnmanage, FALSE,
            XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
            NULL);

    formDlog = XmCreateFormDialog( AmGetWidgetShell( parent), "editFont", 
                   args, n);
    *dialog = formDlog;



    familyForm = XtVaCreateManagedWidget( "familyForm",
                      xmFormWidgetClass, formDlog,
                      StickLeft, StickTop, StickRight,
                      NULL);

    familyLabel = XtVaCreateManagedWidget( "familyLabel",
                      xmLabelWidgetClass, familyForm,
                      StickLeft, StickTop, StickBottom,
                      NULL);

    XtVaGetValues( familyLabel, XmNwidth, &maxWidth, NULL);

    family = AmGetFontFamilies();
    for ( ii = 0; family[ ii] != NULL; ii++) {
        if ( !strcmp( family[ ii], info->name)) {
            break;
        }
    }

    if ( family[ ii] == NULL) {
        ii = 0;
    }

    info->familyOption = AmCreateOptionMenu( familyForm, "familyOption",
                             family, ii);
    XtVaSetValues( info->familyOption, StickTop, StickBottom,
         RightOf( familyLabel), StickRight, NULL);

    XtVaGetValues( info->familyOption, XmNsubMenuId, &menu, NULL);
    XtVaGetValues( menu,
        XmNnumChildren, &(info->numFamilies),
        XmNchildren, &(info->familyButtons),
        NULL);

    sizeForm = XtVaCreateManagedWidget( "sizeForm",
                     xmFormWidgetClass, formDlog,
                     StickLeft, BelowOf( familyForm), StickRight,
                     NULL);

    sizeLabel = XtVaCreateManagedWidget( "sizeLabel",
                    xmLabelWidgetClass, sizeForm,
                    StickLeft, StickTop, StickBottom,
                    NULL);

    XtVaGetValues( sizeLabel, XmNwidth, &width, NULL);

    if ( width > maxWidth) {
        maxWidth = width;
    }

    info->sizeText = XtVaCreateManagedWidget( "sizeText",
                         xmTextFieldWidgetClass, sizeForm,
                         StickTop, StickBottom, RightOf( sizeLabel),
                         XmNcolumns, 5,
                         NULL);
    AmUnitVarEntry( info->sizeText, &(info->size), AmINT, 
		  AmUNIT_UNDEFINED, NULL);

    for ( ii = 0; fontSizes[ ii] != NULL; ii++) {
        if ( atoi( fontSizes[ ii]) == info->size) {
            break;
        }
    }

    if ( fontSizes[ ii] == NULL) {
        ii--;
    } 

    info->sizeOption = AmCreateOptionMenu( sizeForm, "sizeOption", fontSizes,
                           ii);
    XtVaSetValues( info->sizeOption,
        RightOf( info->sizeText), StickTop, StickBottom, NULL);

    XtVaGetValues( info->sizeOption, XmNsubMenuId, &menu, NULL);
    XtVaGetValues( menu,
        XmNnumChildren, &(info->numSizes),
        XmNchildren, &(info->sizeButtons),
        NULL);

    scaleForm = XtVaCreateManagedWidget( "scaleForm",
                     xmFormWidgetClass, formDlog,
                     StickLeft, BelowOf( sizeForm), StickRight,
                     NULL);

    xScaleLabel = XtVaCreateManagedWidget( "xScaleLabel",
                      xmLabelWidgetClass, scaleForm,
                      StickLeft, StickTop, StickBottom,
                      NULL);

    XtVaGetValues( xScaleLabel, XmNwidth, &width, NULL);

    if ( width > maxWidth) {
        maxWidth = width;
    }

    sprintf( buffer, "%4.2f", ( float) info->xScale);
    info->xScaleText = XtVaCreateManagedWidget( "xScaleLabel",
                           xmTextFieldWidgetClass, scaleForm,
                           RightOf( xScaleLabel), StickTop, StickBottom,
                           XmNcolumns, 5,
                           XmNvalue, buffer,
                           NULL);

    AmUnitVarEntry( info->xScaleText, &(info->xScale), AmFLOAT, 
        AmUNIT_UNDEFINED, NULL);

    yScaleLabel = XtVaCreateManagedWidget( "yScaleLabel",
                      xmLabelWidgetClass, scaleForm,
                      RightOf( info->xScaleText), StickTop, StickBottom,
                      NULL);

    XtVaGetValues( yScaleLabel, XmNwidth, &width, NULL);

    if ( width > maxWidth) {
        maxWidth = width;
    }

    sprintf( buffer, "%4.2f", ( float) info->yScale);
    info->yScaleText = XtVaCreateManagedWidget( "yScaleLabel",
                           xmTextFieldWidgetClass, scaleForm,
                           RightOf( yScaleLabel), StickTop, StickBottom,
                           XmNcolumns, 5,
                           XmNvalue, buffer,
                           NULL);

    AmUnitVarEntry( info->yScaleText, &(info->yScale), AmFLOAT, 
        AmUNIT_UNDEFINED, NULL);

    toggleForm = XtVaCreateManagedWidget( "toggleForm",
                     xmFormWidgetClass, formDlog,
                     StickLeft, BelowOf( scaleForm), StickRight,
                     NULL);

    if ( info->weight == AmWEIGHT_BOLD) {
        state = TRUE;
    } else {
        state = FALSE;
    }

    info->weightToggle = XtVaCreateManagedWidget( "weightToggle",
                             xmToggleButtonWidgetClass, toggleForm,
                             StickLeft, StickTop, StickBottom,
                             XmNset, state,
                             NULL);

    if ( info->slant == AmSLANT_ITALIC) {
        state = TRUE;
    } else {
        state = FALSE;
    }

    info->slantToggle = XtVaCreateManagedWidget( "slantToggle",
                            xmToggleButtonWidgetClass, toggleForm,
                            RightOf( info->weightToggle),
                            StickTop, StickBottom,
                            XmNset, state,
                            NULL);

    gfxWin = XtVaCreateManagedWidget( "gfxWin",
                     xmScrolledWindowWidgetClass, formDlog,
                     XmNscrollingPolicy, XmAUTOMATIC,
                     StickLeft, BelowOf( toggleForm), StickRight,
                     StickBottom,
                     NULL);

    gfxForm = XtVaCreateManagedWidget( "gfxForm",
                  xmFormWidgetClass, gfxWin,
                  NULL);

    info->gfx  = XtVaCreateManagedWidget( "gfxArea",
                     xmDrawingAreaWidgetClass, gfxForm,
                     StickLeft, StickRight, StickTop, StickBottom,
                     NULL);

    XtVaSetValues( familyLabel, XmNwidth, maxWidth, NULL);
    XtVaSetValues( sizeLabel, XmNwidth, maxWidth, NULL);
    XtVaSetValues( xScaleLabel, XmNwidth, maxWidth, NULL);
    XtVaSetValues( yScaleLabel, XmNwidth, maxWidth, NULL);

    actionDefs[ 2].clientData = formDlog;
    actionForm = AmMakeActionArea( formDlog, actionDefs, ELEMENTS( actionDefs),
                     &actionArea);

    *okBtn = actionArea.buttons[ 0];

    info->plot = plot;
    info->families = family;
    info->fontInfo = NULL;
    info->gc = NULL;
    info->dpi = plot->GetDisplay()->GetDPI();

    XtAddCallback( info->sizeText, XmNactivateCallback, 
        ( XtCallbackProc) sizeTextChangedCallback,
        info);
    XtAddCallback( info->sizeText, XmNlosingFocusCallback,
        ( XtCallbackProc) sizeTextChangedCallback,
        info);
    XtAddCallback( info->xScaleText, XmNactivateCallback, 
        ( XtCallbackProc) xScaleTextChangedCallback,
        info);
    XtAddCallback( info->xScaleText, XmNlosingFocusCallback,
        ( XtCallbackProc) xScaleTextChangedCallback,
        info);
    XtAddCallback( info->yScaleText, XmNactivateCallback, 
        ( XtCallbackProc) yScaleTextChangedCallback,
        info);
    XtAddCallback( info->yScaleText, XmNlosingFocusCallback,
        ( XtCallbackProc) yScaleTextChangedCallback,
        info);
    XtAddCallback( info->gfx, XmNexposeCallback, 
        ( XtCallbackProc) AmDrawFont,
        info);
    XtAddCallback( info->weightToggle, XmNvalueChangedCallback, 
        ( XtCallbackProc) weightToggleCallback,
        info);
    XtAddCallback( info->slantToggle, XmNvalueChangedCallback, 
        ( XtCallbackProc) slantToggleCallback,
        info);
    AmAddCallbackWidget( AM_PKG_ID, AM_OPTION_CHANGED, info->familyOption,
        ( AmCallbackFn) familyChangedCallback,
        info);
    AmAddCallbackWidget( AM_PKG_ID, AM_OPTION_CHANGED, info->sizeOption,
        ( AmCallbackFn) sizeChangedCallback,
        info);
    AmAddVaCallbackWidget( AM_PKG_ID, AM_OK, formDlog,
        ( VarargsCallback) fontEditOkCallback, formDlog,
        nameTag, sizeTag, weightTag, slantTag, xScaleTag, yScaleTag,
        info, END_LIST);
    AmAddVaCallback( actionArea.buttons[ 1], XmNactivateCallback, 
        ( VarargsCallback) fontEditRevertCallback,
        plot, nameTag, sizeTag, weightTag, slantTag, xScaleTag, yScaleTag,
        info, END_LIST);
    XtAddCallback( formDlog, XmNdestroyCallback,
        ( XtCallbackProc) fontEditDestroyCallback, info);

    XtManageChild( formDlog);
}

/* slick dialog to assign a range dialog callback */

void AmSetFontEditDialogCallback( Widget    dialog,
                                Widget    button,
                                AmPlotClass *plot,
                                char      *args,
                                XtPointer)
{
    AmAddVaCallback( button, XmNactivateCallback,
        ( VarargsCallback) AmFontDialogCallback, 
        plot, args, END_LIST);
}
 
/*
 * Callback for embedding in a slick subform.  It unmanages the dialog
 *   on the OK, and sets it up to be remanaged again when the button
 *   is hit.
 */
 
AnvilBoolean
AmFontDialogCallback( Widget    w,
                    AmPlotClass *plot,
                    char      **args,
                    XtPointer)
{
    Widget dialog, okBtn, parent = AmGetWidgetShell( w);

/*
 * The arguments to AmCreateFontEditDialog are the following:
 *
 *   parent - the parent of the dialog shell.
 *   plot - the plot to edit the font for.
 *
 *   arg[ 0] - the tag that retrieves the font name.
 *   arg[ 1] - the tag that retrieves the font size.
 *   arg[ 2] - the tag that retrieves the font weight enum.
 *   arg[ 3] - the tag that retrieves the font slant enum.
 *   arg[ 4] - the tag that retrieves the font x scaling factor.
 *   arg[ 5] - the tag that retrieves the font y scaling factor.
 *
 *   &dialog - returns the dialog shell widget.
 *   &okBtn - returns the ok button widget.
 */
 
    AmCreateFontEditDialog( parent, plot, args[ 0], args[ 1], args[ 2],
		          args[ 3], args[ 4], args[ 5],
		          &dialog, &okBtn);

    if ( dialog != NULL) {
        AmAddVaCallback( okBtn, XmNactivateCallback,
            ( VarargsCallback) AmUnmanageSubDialogCallback,
            w, dialog, END_LIST);

    /* We remove and replace the callback to launch the dialog, because we 
       keep the dialog around on OK for our parent slick. */

        AmRemoveVaCallback( w, XmNactivateCallback,
            ( VarargsCallback) AmFontDialogCallback, 
            plot, args, END_LIST);

	AmAddVaCallback( dialog, XmNdestroyCallback,
		       ( VarargsCallback) AmSetFontEditDialogCallback,
		       w, plot, args, END_LIST);
        return( TRUE);
    } else {
        AmAddResourceToLog( "cantCreateFontEditDialog");
        return( FALSE);
    }
}

 
/* Manually setting the font editor - for testing purposes. */ 
 
void AmSetFontEditDialogCallback( Widget                  w,
                                AmMotifDisplayClass *motifDisplay,
                                AmPlotClass         *plot,
                                XtPointer)
{
    Widget      dialog, okBtn;
 
    AmCreateFontEditDialog( AmGetWidgetShell( w), plot, 
		          AmNfont, AmNfontSize, AmNfontWeight, 
		          AmNfontSlant, AmNfontXScale, AmNfontYScale,
		          &dialog, &okBtn);
    AmSetOkButton( okBtn, plot);
    XtAddCallback( okBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmUpdateMotifDisplayCallback,
		   motifDisplay);
}
