#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#if !defined(__BSD44__)
#include <alloca.h>
#endif
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/TextF.h>
#include <Xm/DrawingA.h>
#include <anvil/display.h>
#include <anvil/plot.h>
#include <anvil/imagelegend.h>
#include <anvil/motifdisplay.h>
#include <anvil/magic.h>
#include <anvil/macros.h>
#include <anvil/image.h>
#include <anvil/colorcell.h>
#include <anvil/va_cback.h>
#include <anvil/slick.h>
#include <anvil/utility.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>

/* client data for the dialog */

class GradientMagicClass : public AmMagicImageClass {
private:
    Widget		dialog;
    float		min;
    float		max;
    int			numColors;
    AmColorCellPtr	colors;
    AmMotifDisplayClass	*legendDisplay;
    AmImageLegendClass	*legend;
public:
    GradientMagicClass( AmMotifDisplayClass        *motifDisplay,
                        AmImagePlotClass           *sourcePlot,
                        AnvilBoolean                  callOnDrag);
    Widget              GetDialog() { return( dialog); };
    void                SetDialog( Widget dialogArg) { dialog = dialogArg; };
    void                SetMinmax( float min, float max);
    float		GetMin() { return( min); };
    float		GetMax() { return( max); };
    int			GetNumColors() { return( numColors); };
    AmColorCellPtr	GetColors() { return( colors); };
    virtual void	SetRect( AmRectangleRec const *rectArg);
    virtual		~GradientMagicClass();

};

/* client data for the pixel function */

typedef struct gradientCursorData_ {
    AmMagicImageClass	*magicImage;
    float		min;
    float		max;
} gradientCursorDataRec, *gradientCursorDataPtr;

static AmActionDefRec actionAreaDef[] = {
    { "applyBtn", NULL, NULL },
    { "closeBtn", NULL, NULL }
};

/* gradient pixel function */

static ulong_t gradientPixelFn( float			*val,
				GradientMagicClass	*gradientMagic)
{
    int			index, numColors;
    float		min, max;
    AmColorCellPtr	colors;
    uchar_t		*p;
    ulong_t		returnVal = 0;

    p = ( uchar_t *) &returnVal;
    min = gradientMagic->GetMin();
    max = gradientMagic->GetMax();
    numColors = gradientMagic->GetNumColors();
    colors = gradientMagic->GetColors();
    index = (int)(( numColors - 1)*( *val - min)/( max - min));
    if ( index <= 0) {
	ASSIGN_PIXEL( colors[ 0], p);
    } else if ( index >= numColors - 1) {
	ASSIGN_PIXEL( colors[ numColors - 1], p);
    } else {
	ASSIGN_PIXEL( colors[ index], p);
    }
    return( returnVal);
}

/* set the gradient range and redraw the image in the window */

static void minmaxValueCallback( Widget,
				 Widget			minText,
			         Widget			maxText,
				 GradientMagicClass	*gradientMagic,
				 XtPointer)
{
    float	min, max;

    AmGetObjectValue( minText, &min);
    AmGetObjectValue( maxText, &max);
    if ( min < max) {
	AmSetErrorColor( minText, FALSE);
	AmSetErrorColor( maxText, FALSE);
	gradientMagic->SetMinmax( min, max);
	gradientMagic->Update();
    } else {
	AmSetErrorColor( minText, TRUE);
	AmSetErrorColor( maxText, TRUE);
    }
}

/* set the min and max for the gradient, then update the legend and display */

void GradientMagicClass::SetMinmax( float	minArg,
				    float	maxArg)
{

AN_DEBUG( 40, ("calling SetMinMax %.2f, %.2f\n", minArg, maxArg));
    min = minArg;
    max = maxArg;
    legend->SetVaFields( AmNmin, min,
			 AmNmax, max,
			 NULL);
    legendDisplay->Flush();
}

/* apply the gradient when the rectangle is set */

void GradientMagicClass::SetRect( AmRectangleRec const *rectArg)
{
    float		winXmin, winYmin, winXmax, winYmax, *matrix, 
			subXmin, subYmin, subXmax, subYmax, z0, z1, z2;
    int			size, numColors, matRows, matCols, 
			ii, jj, subRows, subCols;
    AnvilBoolean		flipRows, flipCols;
    AmAccessFn		zFn;
    void		*zClientData;
    uchar_t		*data, *sourceMatrix, *ptr;
    AmRectangleRec	sourceIntRect, screenRect, offsetRect;
    AmRealRectRec		sourceRect;
    AmColorCellRec	bgColor;
    AmErrorCode		errCode;

AN_DEBUG( 99, ("getting fields\n"));
    rect = *rectArg;

/* if the colors have been allocated, then free them before retrieve */

    if ( colors != NULL) {
	free( colors);
    }
    sourcePlot->GetVaFields( AmNflipRows, &flipRows,
			     AmNflipCols, &flipCols,
			     AmNsubXmin, &subXmin,
			     AmNsubYmin, &subYmin,
			     AmNsubXmax, &subXmax,
			     AmNsubYmax, &subYmax,
			     AmNsubRows, &subRows,
			     AmNsubCols, &subCols,
			     AmNcolorsFn, &zFn,
			     AmNcolorsClientData, &zClientData,
			     AmNdata, &data,
			     AmNsize, &size,
			     AmNcolors, &colors,
			     AmNnumColors, &numColors,
			     NULL);

/* where is the window on the source plot? */

    GetSourceCoords( &winXmin, &winYmin, &winXmax, &winYmax);
 
/* don't gradient if we're not in the source matrix at all.... */

    screenRect = rect;
    if ( !sourcePlot->MatrixRealExtents( subXmin, subYmin, subXmax, subYmax,
                                         winXmin, winYmin, winXmax, winYmax,
                                         subRows, subCols, flipRows, flipCols,
                                         &screenRect, &sourceRect)) {
        return;
    }
 
    sourcePlot->GetDataPt( screenRect.x, screenRect.y + screenRect.height,
                           &winXmin, &winYmin);
    sourcePlot->GetDataPt( screenRect.x + screenRect.width, screenRect.y,
                           &winXmax, &winYmax);
    AmMakeContainedIntRect( &sourceRect, &sourceIntRect);
    matCols = sourceIntRect.width;
    matRows = sourceIntRect.height;

/* allocate source and gradient matrices */

    if (( sourceMatrix = ( uchar_t *) alloca( matRows*matCols*size)) == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return;
    }
    if (( matrix = ( float *) 
	    alloca( matRows*matCols*sizeof( float))) == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return;
    }
    errCode = AmCopySubmatrix( data, subRows, subCols, &sourceIntRect, 
			       sourceMatrix, matRows, matCols, 0, 0,
			       size, flipRows, flipCols);
    if ( errCode != AmOK) {
	DO_ERROR( errCode);
	return;
    }
AN_DEBUG( 99, ("applying gradient\n"));
/* generate the gradient */

    for ( ii = 0; ii < matRows - 1; ii++) {
	for ( jj = 0; jj < matCols - 1; jj++) {
	    ptr = &sourceMatrix[ ( ii*matCols + jj)*size];
	    z0 = ( zFn)( ptr, zClientData);
	    z1 = ( zFn)( ptr + size, zClientData);
	    z2 = ( zFn)( ptr + matCols*size, zClientData);
	    matrix[ ii*matCols + jj] = fabs((( z0 - z1) + ( z0 - z2)))/2;
	}
	matrix[ ii*matCols + ( matCols - 1)] = 
	    matrix[ ii*matCols + ( matCols - 2)];
    }

/* the AmLast line is the same as the previous */

    for ( ii = 0; ii < matCols; ii++) {
	matrix[ ( matRows - 1)*matCols + ii] = 
	    matrix[ ( matRows - 2)*matCols + ii]; 
    }

/* clear out the image */

    offsetRect = rect;
    offsetRect.x = 0;
    offsetRect.y = 0;
    bgColor.red = 0x0;
    bgColor.blue = 0x0;
    bgColor.green = 0x0;
    image->ClearImageRect( &bgColor, &offsetRect);

/* translate the source and destination rectangles to the magic window's
   coordinate system */

    sourceRect.x = 0.0;
    sourceRect.y = 0.0;
    screenRect.x -= rect.x;
    screenRect.y -= rect.y;

/* project the gradient matrix to the image */

AN_DEBUG( 99, ("resizing image\n"));
    image->resizeMatrixImage(
	       ( uchar_t *) matrix, sizeof( float), matRows, matCols,
	       FALSE, FALSE, &sourceRect, &screenRect, 
	       ( AmColorFn) gradientPixelFn, this, 1.0);
}

/* delete the magic image and free the callback argument structure */

static void destroyGradientCallback( Widget		   dialog,
				     GradientMagicClass	   *gradientMagic,
				     XtPointer)
{
    delete gradientMagic;
}

/* destructor... wipe out the dialog if it isn't NULL */

GradientMagicClass::~GradientMagicClass()
{
    if ( colors != NULL) {
	FREE( colors);
    }
    if ( dialog != NULL) {

/* remove the callback to delete the object when the dialog is destroyed,
   otherwise, we get deleted again when the dialog is blown away... */

	XtRemoveCallback( dialog, XmNdestroyCallback,
			  ( XtCallbackProc) destroyGradientCallback, this);
	XtDestroyWidget( dialog);
    }
}

/* callback to display the gradient of an image */

AnvilBoolean
AmGradientCursorCallback( Widget		btn,
			     AmMotifDisplayClass	*motifDisplay,
			     AmImagePlotClass	*imagePlot,
			     XtPointer)
{
    if ( !imagePlot->IsType( AmIMAGE_PLOT)) {
	AmAddResourceToLog( "gradientCursorNotImagePlot");
	return( FALSE);
    }

    new GradientMagicClass( motifDisplay, imagePlot, FALSE);
    return( TRUE);
}

/* magic window for a gradient cursor */

GradientMagicClass::GradientMagicClass( AmMotifDisplayClass *motifDisplay, 
					AmImagePlotClass    *imagePlot,
					AnvilBoolean		    callOnDrag) :
				AmMagicImageClass( motifDisplay,
						   ( AmPlotClass *) imagePlot,
						   callOnDrag)
{
    Widget			parent = motifDisplay->GetShell(),
				minText, maxText, actionForm, area;
    AmActionRec			actionArea;
    Arg				args[ 20];
    int				n;
    AmXDisplayClass		*pXDisplay;
    AmXWindowClass		*pXWindow;

    imagePlot->GetVaFields( AmNcolorsMin, &min,
			    AmNcolorsMax, &max,
			    AmNnumColors, &numColors,
			    AmNcolors, &colors,
			    NULL);

/* create a form to control the min and max for the gradient */

    n = AmMakeArgList( args,
		     XmNautoUnmanage, FALSE,
		     XmNcolormap, AmGetAppColormap(),
		     XmNvisual, AmGetAppVisual(),
		     XmNdepth, AmGetAppDepth(),
		     NULL);
    dialog = XmCreateFormDialog( parent, "gradientCursor", args, n);

    actionForm = AmMakeActionArea( dialog,
				 actionAreaDef, ELEMENTS( actionAreaDef),
				 &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);
    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, dialog,
				    StickLeft, StickTop, 
				    AboveOf( actionForm),
				    XmNwidth, 120,
				    XmNheight, 300,
				    NULL);

/* create a display for the legend for the gradient cursor, and a legend
   without a parent, because we'll control the min/max ourselves. */

    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    legendDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);
    legend = new AmImageLegendClass( legendDisplay->GetWindow(),
				   NULL, "gradientLegend",
				   AmNmin, min,
				   AmNmax, max,
				   AmNcolors, colors,
				   AmNnumColors, numColors,
				   AmNwidth, 100,
				   AmNheight, 250,
				   AmNx, 20,
				   AmNy, 20,
				   AmNleftMargin, 20,
				   AmNtopMargin, 20,
				   AmNbottomMargin, 20,
				   AmNrightMargin, 20,
				   NULL);
    legendDisplay->AddObject( legend);
    legend->SetRect( 20, 20, 100, 250);

/* text fields to enter the min and max values */

    maxText = AmAddObject( dialog, &max, "max", AmLOCAL);
    XtVaSetValues( XtParent( maxText), 
		   RightOf( area), StickRight, StickTop, NULL);
    minText = AmAddObject( dialog, &min, "min", AmLOCAL);
    XtVaSetValues( XtParent( minText), 
		   RightOf( area), StickRight, 
		   BelowOf( XtParent( maxText)), 
		   NULL);

/* text callbacks to accept the new ranges */

    AmAddVaCallback( maxText, XmNactivateCallback,
		   ( VarargsCallback) minmaxValueCallback, 
		   minText, maxText, this,
		   END_LIST);
    AmAddVaCallback( minText, XmNactivateCallback,
		   ( VarargsCallback) minmaxValueCallback, 
		   minText, maxText, this,
		   END_LIST);
    AmAddVaCallback( maxText, XmNlosingFocusCallback,
		   ( VarargsCallback) minmaxValueCallback, 
		   minText, maxText, this,
		   END_LIST);
    AmAddVaCallback( minText, XmNlosingFocusCallback,
		   ( VarargsCallback) minmaxValueCallback, 
		   minText, maxText, this,
		   END_LIST);

/* assign the destroy callback */

    XtAddCallback( dialog, XmNdestroyCallback,
		   ( XtCallbackProc) destroyGradientCallback, this);

/* add callbacks to pop down the dialog, and to re-apply the minmaxs */

    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, dialog);
    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback, 
		   ( VarargsCallback) minmaxValueCallback, 
		   minText, maxText, this,
		   END_LIST);
    Update();

/* manage the dialog */

    XtManageChild( dialog);
}
