#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Scale.h>
#include <Xm/PushB.h>
#include <Xm/DrawingA.h>
#include <stdarg.h>
#include <stdio.h>
#include <anvil/strlib.h>
#include <anvil/plot.h>
#include <anvil/motif.h>
#include <anvil/resource.h>
#include <anvil/plot_cback.h>
#include <anvil/copyobj.h>
#include <anvil/motifdisplay.h>
#include <anvil/magic.h>

class MagnifyDialogClass : public AmMagicOutlineClass {
private:
    float		factor;
    Widget		dialog;
    AmPlotClass		*magnifyPlot;
    AmMotifDisplayClass	*privateDisplay;
public:
    MagnifyDialogClass( AmMotifDisplayClass     *motifDisplay,
		       AmPlotClass             *sourcePlot,
		       AnvilBoolean               callOnDrag);
    void 		SetFactor( float value) { factor = value; };
    float		GetFactor() { return( factor); };
    Widget		GetDialog() { return( dialog); };
    void		SetDialog( Widget dialogArg) { dialog = dialogArg; };
    AmMotifDisplayClass	*GetPrivateDisplay() { return( privateDisplay); };
    virtual void	SetRect( AmRectangleRec const *rect);
    virtual AmErrorCode	Update();
    virtual 		~MagnifyDialogClass();
};

/* set the magnify scale */

static void setMagnifyScale( Widget		scale,
			     MagnifyDialogClass	*magnify,
			     XtPointer)
{
    int	value;

    XmScaleGetValue( scale, &value);
    magnify->SetFactor( value);
    magnify->Update();
}

/* default destroy frees the structure */

MagnifyDialogClass::~MagnifyDialogClass()
{
    if ( dialog != NULL) {
	XtDestroyWidget( dialog);
    }
}

/* turn off the magnifying glass window when the dialog is destroyed */

static void destroyMagnifyCallback( Widget,	
				    MagnifyDialogClass	*magnify,
				    XtPointer)
{
    magnify->SetDialog( NULL);
    delete magnify;
}

void MagnifyDialogClass::SetRect( AmRectangleRec const	*rectArg)
{
    float	xmin, ymin, xmax, ymax, cx, cy, wx, wy;

    rect = *rectArg;
    GetSourceCoords( &xmin, &ymin, &xmax, &ymax);
    cx = ( xmax + xmin)/2;
    cy = ( ymax + ymin)/2;
    wx = ( xmax - xmin)/( factor*2);
    wy = ( ymax - ymin)/( factor*2);

AN_DEBUG(80, ("resizing to %.2f, %.2f, %.2f, %.2f\n",
		cx - wx, cy - wy, cx + wx, cy + wy));

    magnifyPlot->SetCoords( cx - wx, cy - wy, cx + wx, cy + wy);
    privateDisplay->Flush();
}

/* update function for the magnify dialog */

AmErrorCode MagnifyDialogClass::Update()
{
    delete magnifyPlot;
    magnifyPlot = ( AmPlotClass *) AmTraceObject( sourcePlot->GetWindow(),
					      sourcePlot, NULL);
    privateDisplay->AddObject( magnifyPlot);
    magnifyPlot->Update();
    SetRect( GetRect());
    return( AmOK);
}

/* constructor to set up a magnifying glass for an object in an external
   window */


MagnifyDialogClass::MagnifyDialogClass( AmMotifDisplayClass	*motifDisplay,
					AmPlotClass		*sourcePlot,
					AnvilBoolean			callOnDrag) :
			    AmMagicOutlineClass( motifDisplay,
					       sourcePlot,
					       callOnDrag)
{
    Widget		scale, closeBtn, area;
    Arg			args[ 20];
    int			n;
    AmXDisplayClass	*pXDisplay;
    AmXWindowClass	*pXWindow;

    factor = 2.0;

/* create a dialog for setting the zoom factor */

    n = AmMakeArgList( args,
                     XmNdeleteResponse, XmDESTROY,
                     XmNautoUnmanage, FALSE,
                     NULL);
    dialog = XmCreateFormDialog( motifDisplay->GetShell(), "magnify", args, n);
    area = XtVaCreateManagedWidget( "magnifyArea", xmDrawingAreaWidgetClass,
				    dialog,
				    StickTop, StickLeft, StickRight,
				    StickBottom,
				    XmNwidth, 200,
				    XmNheight, 200,
				    XmNbottomOffset, 40,
				    NULL);
    scale = XtVaCreateManagedWidget( "magnifyScale", xmScaleWidgetClass,
				     dialog,
				     XmNorientation, XmHORIZONTAL,
				     XmNminimum, 1,
				     XmNmaximum, 20,
				     XmNvalue, 2,
				     StickLeft, StickBottom,
				     XmNshowValue, TRUE,
				     NULL);
    closeBtn = XtVaCreateManagedWidget( "closeBtn", xmPushButtonWidgetClass,
					dialog,
					RightOf( scale),
					StickBottom,
					NULL);
    XtManageChild( dialog);

/* create a display to show the magnified plot, create and add the plot 
   to the display, and update it */

    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    privateDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);
    magnifyPlot = ( AmPlotClass *) 
	AmTraceObject( sourcePlot->GetWindow(), sourcePlot, NULL);
    privateDisplay->AddObject( magnifyPlot);
    SetRect( GetRect());

/* add callbacks to destroy the dialog and set the scale */

    XtAddCallback( closeBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, dialog);
    XtAddCallback( scale, XmNvalueChangedCallback,
		   ( XtCallbackProc) setMagnifyScale, this);
    XtAddCallback( dialog, XmNdestroyCallback,
		   ( XtCallbackProc) destroyMagnifyCallback, this);
}

/* callback to set up the magic dialog. */

AnvilBoolean
AmMagnifyGlassDialogCallback( Widget                 btn,
                            AmMotifDisplayClass      *motifDisplay,
                            AmPlotClass              *plot,
                            XtPointer              callData)
{
    new MagnifyDialogClass( motifDisplay, plot,
			    AmGetDefault( "magnify.drag", ( AnvilBoolean) FALSE));
    return( TRUE);
}
