#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <stdlib.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <anvil/errlib.h>
#include <anvil/list.h>
#include <anvil/resource.h>
#include <anvil/plot.h>
#include <anvil/objectplot.h>
#include <anvil/motifdisplay.h>
#include <anvil/va_cback.h>
#include <anvil/plot_cback.h>
#include <anvil/userevents.h>
#include <anvil/selectutil.h>
#include <anvil/units.h>
#include <anvil/motif.h>
#include <anvil/macros.h>
#include <anvil/state_hand.h>

typedef struct moveData_ {
     AmMotifDisplayClass	*motifDisplay;
     AmObjectPlotClass	*plot;
     uchar_t		*objPtr;	/* pointer to object to drag */
     AmRectangleRec	invalRect;	/* old rectangle to clear */
     addFunc		addFn;		/* for copying added objects */
     void		*addFnClientData;
} moveDataRec, *moveDataPtr;

typedef struct addData_ {
     AmMotifDisplayClass	*motifDisplay;
     AmObjectPlotClass	*plot;
     addFunc		addFn;
     void		*addFnClientData;
} addDataRec, *addDataPtr;

static AnvilBoolean firstMove = TRUE;

/* drag an object across the screen */

static void dragObjectHandler( Widget		,
			       moveDataPtr	moveData,
			       XMotionEvent	*event,
			       AnvilBoolean		*)
{
    AmObjectPlotClass	*plot = moveData->plot;
    AmMotifDisplayClass	*motifDisplay = moveData->motifDisplay;
    AmDisplayClass	*display = motifDisplay->GetDisplay();
    AmWindowClass         *window = motifDisplay->GetWindow();
    int			size;
    AmAccessFn		xFn, yFn;
    AmSetFn		setXFn, setYFn;
    AmSetABFn		setXYFn;
    void		*clientData;
    uchar_t		*data;
    float		x, y;

    plot->GetVaFields( AmNxFn, &xFn,
		       AmNyFn, &yFn,
		       AmNsetXFn, &setXFn,
		       AmNsetYFn, &setYFn,
		       AmNsetXYFn, &setXYFn,
		       AmNclientData, &clientData,
		       AmNsize, &size,
		       AmNdata, &data,
		       NULL);

/* undraw the object in its current location */

    if ( !firstMove) {
	plot->DrawObject( display, window, motifDisplay->GetCursorGC(), 
			  moveData->objPtr);
    } else {
	firstMove = FALSE;
    }

/* set the x and y coordinates and draw it in the new location */

    plot->GetDataPt( event->x, event->y, &x, &y);
    if (( setXFn != NULL) && ( setYFn != NULL)) {
	( setXFn)( moveData->objPtr, clientData, x);
	( setYFn)( moveData->objPtr, clientData, y);
    } else if ( setXYFn != NULL) {
	( setXYFn)( moveData->objPtr, clientData, x, y);
    }
    plot->DrawObject( display, window, motifDisplay->GetCursorGC(), 
		      moveData->objPtr);
}

/* draw the object in its new location once the button is released */

static void objectMovedHandler( Widget		area,
			        moveDataPtr	moveData,
			        XButtonEvent	*,
			        AnvilBoolean		*)
{
    AmObjectPlotClass	*plot = moveData->plot;
    AmMotifDisplayClass	*motifDisplay = moveData->motifDisplay;
    AmWindowClass         *window = motifDisplay->GetWindow();
    AmRectangleRec	invalRect;
    unitType		xunits, yunits;
    AmAccessFn		xFn, yFn;
    char		resbuf[ RESLEN], buf1[ 128], buf2[ 128];
    void		*clientData;

    XtRemoveEventHandler( area, PointerMotionMask, FALSE,
			  ( XtEventHandler) dragObjectHandler, moveData);
    XtRemoveEventHandler( area, ButtonReleaseMask, FALSE,
			  ( XtEventHandler) objectMovedHandler, moveData);
    plot->SetMaxObjectExtents( window, motifDisplay->GetGC());
    motifDisplay->Draw( &moveData->invalRect, 1);
    plot->GetVaFields( AmNxunits, &xunits,
		       AmNyunits, &yunits,
		       AmNxFn, &xFn,
		       AmNyFn, &yFn,
		       AmNclientData, &clientData,
		       NULL);

/* undraw the object in its current location */

    plot->GetObjectExtents( moveData->objPtr, window, motifDisplay->GetGC(),
			    &invalRect);
    motifDisplay->Draw( &invalRect, 1);

/* write a string showing where the object was dragged to */

    sprintf( resbuf, "%s.dragString", moveData->plot->getObjectName());
    AmPrintValue( xunits, ( xFn)( moveData->objPtr, clientData), buf1);
    AmPrintValue( yunits, ( yFn)( moveData->objPtr, clientData), buf2);
    motifDisplay->WriteResourceMessage( resbuf, buf1, buf2);
}

/* draw the object in its new location once the button is released */

static void objectCopiedHandler( Widget		area,
			         moveDataPtr	moveData,
			         XButtonEvent	*,
			         AnvilBoolean	*)
{
    AmObjectPlotClass	*plot = moveData->plot;
    AmMotifDisplayClass	*motifDisplay = moveData->motifDisplay;
    AmWindowClass         *window = motifDisplay->GetWindow();
    AmRectangleRec	invalRect;
    unitType		xunits, yunits;
    AmAccessFn		xFn, yFn;
    char		resbuf[ RESLEN], buf1[ 128], buf2[ 128];
    void		*clientData;
    int			rows, size;
    uchar_t		*data;

    XtRemoveEventHandler( area, PointerMotionMask, FALSE,
			  ( XtEventHandler) dragObjectHandler, moveData);
    XtRemoveEventHandler( area, ButtonReleaseMask, FALSE,
			  ( XtEventHandler) objectCopiedHandler, moveData);
    plot->SetMaxObjectExtents( window, motifDisplay->GetGC());
    motifDisplay->Draw( &moveData->invalRect, 1);
//    window->InvalidateRect( &moveData->invalRect);
    plot->GetVaFields( AmNxunits, &xunits,
		       AmNyunits, &yunits,
		       AmNxFn, &xFn,
		       AmNyFn, &yFn,
		       AmNclientData, &clientData,
		       AmNdata, &data,
		       AmNrows, &rows,
		       AmNsize, &size,
		       NULL);
    if (( data = ( uchar_t *) realloc( data, ( rows + 1)*size)) == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return;
    }
    memcpy( &data[ rows*size], moveData->objPtr, size);
    plot->SetVaFields( AmNdata, data,
		       AmNrows, rows + 1,
		       NULL);

/* undraw the object in its current location */

    plot->GetObjectExtents( moveData->objPtr, window, motifDisplay->GetGC(),
			    &invalRect);
    motifDisplay->Draw( &invalRect, 1);
//    window->InvalidateRect( &invalRect);

/* write a string showing where the object was dragged to */

    sprintf( resbuf, "%s.dragString", moveData->plot->getObjectName());
    AmPrintValue( xunits, ( xFn)( moveData->objPtr, clientData), buf1);
    AmPrintValue( yunits, ( yFn)( moveData->objPtr, clientData), buf2);
    motifDisplay->WriteResourceMessage( resbuf, buf1, buf2);
}
/* pick the object we're going to move */

static void pickMoveObjectHandler( Widget		area,
				   moveDataPtr		moveData,
				   XButtonEvent		*event,
				   AnvilBoolean		*)
{
    AmMotifDisplayClass	  *motifDisplay = moveData->motifDisplay;
    AmWindowClass         *window = motifDisplay->GetWindow();
    AmObjectPlotClass	  *plot = moveData->plot;
    int			  size, index, fontSize;
    uchar_t		  *data;
    char		  resbuf[ RESLEN], *fontName;
    AmFontWeightEnum	  fontWeight;
    AmFontSlantEnum	  fontSlant;
    float		  fontXScale, fontYScale;

    AmRemoveHandler( area, ButtonPressMask, 
		   ( XtEventHandler) pickMoveObjectHandler, moveData);
    if ( event->button == Button3) {
	sprintf( resbuf, "%s.cancelString", moveData->plot->getObjectName());
	motifDisplay->WriteResourceMessage( resbuf);
    } else {

/* get the index of the object closest to the mouse press */

	index = AmNearestIndex( moveData->plot, event->x, event->y);

/* set the font to match the font for the object plot's GC */

	plot->GetVaFields( AmNsize, &size,
			   AmNdata, &data,
			   AmNfont, &fontName,
			   AmNfontSize, &fontSize,
			   AmNfontWeight, &fontWeight,
			   AmNfontSlant, &fontSlant,
			   AmNfontXScale, &fontXScale,
			   AmNfontYScale, &fontYScale,
			   NULL);
	motifDisplay->GetCursorGC()->SetFont( fontName, fontSize,
					      fontWeight, fontSlant,
					      fontXScale, fontYScale);
	moveData->objPtr = &data[ size*index];

/* get its extents to get the invalidate rectangle */

	plot->GetObjectExtents( moveData->objPtr, window, motifDisplay->GetGC(),
			        &moveData->invalRect);
	sprintf( resbuf, "%s.dragString", moveData->plot->getObjectName());
	motifDisplay->WriteResourceMessage( resbuf);
	firstMove = TRUE;
	XtAddEventHandler( area, PointerMotionMask, FALSE,
			   ( XtEventHandler) dragObjectHandler, moveData);
	XtAddEventHandler( area, ButtonReleaseMask, FALSE,
			   ( XtEventHandler) objectMovedHandler, moveData);
    }
}

/* click and drag an object */

AnvilBoolean
AmMoveObjectCallback( Widget			btn,
  		      AmMotifDisplayClass	*motifDisplay,
		      AmPlotClass		*plot,
		      XtPointer)
{
    moveDataPtr		  moveData;
    AmObjectPlotClass	  *objectPlot = ( AmObjectPlotClass *) plot;
    AmSetFn		  setXFn, setYFn;
    AmSetABFn		  setXYFn;
    char		  resbuf[ RESLEN]; 

/* make sure it's an object plot... */

    if ( !objectPlot->IsType( AmOBJECT_PLOT)) {
	AmAddResourceToLog( "MoveObjectCallbackWrongPlotType",
			  objectPlot->GetObjectType());
	return( FALSE);
    }

/* create a passing structure with the motif display, plot, and a vector of
   flags which show items that have been selected, then add a button
   press handler which does most of the work */

    objectPlot->GetVaFields( AmNsetXFn, &setXFn,
			     AmNsetYFn, &setYFn,
			     AmNsetXYFn, &setXYFn,
			     NULL);
    if ((( setXFn == NULL) || ( setYFn == NULL)) && ( setXYFn == NULL)) {
	AmPrintResourceError( AmGetWidgetRealShell( motifDisplay->GetArea()),
			    "layerMustHaveSetXYFns");
	return( FALSE);
    }
    moveData = ( moveDataPtr) malloc( sizeof( moveDataRec));
    moveData->motifDisplay = motifDisplay;
    moveData->plot = objectPlot;
    moveData->objPtr = NULL;
    moveData->addFn = NULL;
    moveData->addFnClientData = NULL;
    sprintf( resbuf, "%s.pickString", objectPlot->getObjectName());
    motifDisplay->WriteResourceMessage( resbuf);
    AmAddHandler( btn, motifDisplay->GetArea(), ButtonPressMask, 
		( XtEventHandler) pickMoveObjectHandler, moveData,
		SELECT_PRIORITY);

    return( TRUE);
}
    
/* pick the object we're going to copy and drag */

static void pickCopyObjectHandler( Widget		area,
				   moveDataPtr		moveData,
				   XButtonEvent		*event,
				   AnvilBoolean		*)
{
    AmMotifDisplayClass	*motifDisplay = moveData->motifDisplay;
    AmWindowClass         *window = motifDisplay->GetWindow();
    AmObjectPlotClass	*plot = moveData->plot;
    int			size, index, rows, fontSize;
    uchar_t		*data;
    char		resbuf[ RESLEN], *fontName;
    AmFontWeightEnum	fontWeight;
    AmFontSlantEnum	fontSlant;
    float		fontXScale, fontYScale;

    AmRemoveHandler( area, ButtonPressMask, 
		   ( XtEventHandler) pickCopyObjectHandler, moveData);
    if ( event->button == Button3) {
	sprintf( resbuf, "%s.cancelString", moveData->plot->getObjectName());
	motifDisplay->WriteResourceMessage( resbuf);
    } else {

/* get the index of the object closest to the mouse press, to add the object, 
   copy the source object, call the passed add function, then reset the data 
   in the plot, and increment the rows */

	plot->GetVaFields( AmNsize, &size,
			   AmNdata, &data,
			   AmNrows, &rows,
			   AmNfont, &fontName,
			   AmNfontSize, &fontSize,
			   AmNfontWeight, &fontWeight,
			   AmNfontSlant, &fontSlant,
			   AmNfontXScale, &fontXScale,
			   AmNfontYScale, &fontYScale,
			   NULL);
	motifDisplay->GetCursorGC()->SetFont( fontName, fontSize,
					      fontWeight, fontSlant,
					      fontXScale, fontYScale);
	index = AmNearestIndex( moveData->plot, event->x, event->y);
	moveData->objPtr = ( uchar_t *) malloc( size);
	if ( moveData->objPtr == NULL) {
	    DO_ERROR( AmMALLOC_FAILED);
	    return;
	}
	memcpy( moveData->objPtr, &data[ index*size], size);
	( moveData->addFn)( moveData->objPtr, moveData->addFnClientData);

/* get its extents to get the invalidate rectangle */

	plot->GetObjectExtents( moveData->objPtr, window, motifDisplay->GetGC(),
			        &moveData->invalRect);
	sprintf( resbuf, "%s.dragString", moveData->plot->getObjectName());
	motifDisplay->WriteResourceMessage( resbuf);

/* add handlers to drag the copied object, and complete dragging */

	firstMove = TRUE;
	XtAddEventHandler( area, PointerMotionMask, FALSE,
			   ( XtEventHandler) dragObjectHandler, moveData);
	XtAddEventHandler( area, ButtonReleaseMask, FALSE,
			   ( XtEventHandler) objectCopiedHandler, moveData);
    }
}

/* Copy an object, then drag it to a new location */

AnvilBoolean
AmCopyObjectCallback( Widget			btn,
		    AmMotifDisplayClass	*motifDisplay,
		    AmPlotClass		*plot,
		    AnvilBoolean		( *addFn)( void *, void *),
		    void			*addFnClientData,
		    XtPointer)
{
    moveDataPtr		moveData;
    AmObjectPlotClass	*objectPlot = ( AmObjectPlotClass *) plot;
    AmSetFn		setXFn, setYFn;
    AmSetABFn		setXYFn;
    char		resbuf[ RESLEN];

/* make sure it's an object plot... */

    if ( !objectPlot->IsType( AmOBJECT_PLOT)) {
	AmAddResourceToLog( "MoveObjectCallbackWrongPlotType",
			  objectPlot->GetObjectType());
	return( FALSE);
    }

/* create a passing structure with the motif display, plot, and a vector of
   flags which show items that have been selected, then add a button
   press handler which does most of the work */

    objectPlot->GetVaFields( AmNsetXFn, &setXFn,
			     AmNsetYFn, &setYFn,
			     AmNsetXYFn, &setXYFn,
			     NULL);
    if ((( setXFn == NULL) || ( setYFn == NULL)) && ( setXYFn == NULL)) {
	AmAddResourceToLog( "layerMustHaveSetXYFns");
	return( FALSE);
    }

/* create the callback arguments structure */

    moveData = ( moveDataPtr) malloc( sizeof( moveDataRec));
    moveData->motifDisplay = motifDisplay;
    moveData->plot = objectPlot;
    moveData->objPtr = NULL;
    moveData->addFn = addFn;
    moveData->addFnClientData = addFnClientData;
    sprintf( resbuf, "%s.pickString", objectPlot->getObjectName());
    motifDisplay->WriteResourceMessage( resbuf);
    AmAddHandler( btn, motifDisplay->GetArea(), ButtonPressMask, 
		( XtEventHandler) pickCopyObjectHandler, moveData,
		SELECT_PRIORITY);

    return( TRUE);
}

/* add object handler */

void AmAddObjectHandler( Widget		,
		       addDataPtr	addData,
		       XButtonEvent	*event,
		       AnvilBoolean		*)
{
    AmPlotClass		*plot = addData->plot;
    float		x0, y0;
    AmSetFn		setXFn, setYFn;
    AmAccessFn		xFn, yFn;
    void		*clientData;
    uchar_t		*data;
    int			size, rows;

    plot->GetDataPt( event->x, event->y, &x0, &y0);
    plot->GetVaFields( AmNxFn, &xFn,
		       AmNyFn, &yFn,
		       AmNsetXFn, &setXFn,
		       AmNsetYFn, &setYFn,
		       AmNclientData, &clientData,
		       AmNsize, &size,
		       AmNrows, &rows,
		       AmNdata, &data,
		       NULL);

/* add, allocate, set the xy for, and draw the newly added object.  This
   assumes that the original data pointer is malloced */

    if (( data = ( uchar_t *) realloc( data, ( rows + 1)*size)) == NULL) {
	DO_ERROR( AmREALLOC_FAILED);
	return;
    }
    ( addData->addFn)( &data[ rows*size], addData->addFnClientData);
    ( setXFn)( &data[ rows*size], clientData, x0);
    ( setYFn)( &data[ rows*size], clientData, y0);
    plot->SetVaFields( AmNdata, data,
		       AmNrows, rows + 1,
		       NULL);
}

/* function to add objects, given an add function and add client data */

AnvilBoolean
AmAddObjectsCallback( Widget			btn,
		    AmMotifDisplayClass	*motifDisplay,
		    AmPlotClass		*plot,
		    AnvilBoolean		( *addFn)( void *, void *),
		    void			*addFnClientData,
		    XtPointer)
{
    addDataPtr		addData = ( addDataPtr) malloc( sizeof( addDataRec));
    AmObjectPlotClass	*objectPlot = ( AmObjectPlotClass *) plot;

    if ( !plot->IsType( AmOBJECT_PLOT)) {
	AmAddResourceToLog( "AddObjectsCallbackWrongPlotType",
			  objectPlot->GetObjectType());
	return( FALSE);
    }

    addData->motifDisplay = motifDisplay;
    addData->plot = objectPlot;
    addData->addFn = addFn;
    addData->addFnClientData = addFnClientData;
    AmAddHandler( btn, motifDisplay->GetArea(), ButtonPressMask, 
		( XtEventHandler) AmAddObjectHandler, addData,
		SELECT_PRIORITY);

    return( TRUE);
}

/* take the currently selected objects and delete them */

AnvilBoolean
AmDeleteSelectedObjectsCallback( 
	Widget			,
	AmMotifDisplayClass	*motifDisplay,
	AmPlotClass		*plot,
	AnvilBoolean			( *deleteFn)( void *, void *),
	void			*deleteFnClientData,
	XtPointer)
{
    AmObjectPlotClass	*objectPlot = ( AmObjectPlotClass *) plot;
    AmWindowClass         *window = motifDisplay->GetWindow();
    uchar_t		*selection, *data;
    int			nSel, ii, jj, kk, rows, size;
    char		resbuf[ RESLEN];

    if ( !plot->IsType( AmOBJECT_PLOT)) {
	AmAddResourceToLog( "DeleteObjectsCallbackWrongPlotType",
			  objectPlot->GetObjectType());
	return( FALSE);;
    }

/* close the selection set handlers which may still be active... */

    AmSendUserEventChildren( AM_PKG_ID, AM_CLOSE_SELECTION, NULL,
			     motifDisplay->GetShell());
    objectPlot->GetSelection(( void **) &selection, &nSel);
    if ( nSel == 0) {
	AmAddResourceToLog( "SelectionSetEmpty");
	return( FALSE);
    }
    objectPlot->GetVaFields( AmNsize, &size,
			     AmNrows, &rows,
			     AmNdata, &data,
			     NULL);
    objectPlot->SetMaxObjectExtents( window, motifDisplay->GetGC());

/* scrunch any of the elements in the selection, and call the delete function
   on any deleted elements.  Terminate when the end of the data or the
   end of the selection set is reached. Note that this depends on the
   selection set being in the same ordering as the data segment */

    jj = 0;
    kk = 0;
    for ( ii = 0; ( ii < rows) && ( jj < nSel); ii++) {
	if ( memcmp( &selection[ jj*size], &data[ ii*size], size) != 0) {
	    memcpy( &data[ kk*size], &data[ ii*size], size);
	    kk++;
	} else {
	    if ( deleteFn != NULL) {
		( deleteFn) ( &data[ ii*size], deleteFnClientData);
	    }
	    jj++;
	}
    }

/* scrunch the rest of the data array */

    for ( ; ii < rows; ii++, kk++) {
	memcpy( &data[ kk*size], &data[ ii*size], size);
    }

/* clear the plot's current selection */

    objectPlot->ClearSelection();
    objectPlot->SetVaFields( AmNrows, rows - nSel, NULL);
    sprintf( resbuf, "%s.deleteObjectString", objectPlot->getObjectName());
    motifDisplay->WriteResourceMessage( resbuf);
    motifDisplay->Flush();

    return( TRUE);
}
