#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <malloc.h>
#include <stdlib.h>
#include <Xm/Xm.h>
#include <X11/cursorfont.h>
#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/userevents.h>
#include <anvil/macros.h>
#include <anvil/plot_cback.h>
#include <anvil/state_hand.h>
#include <anvil/events.h>
#include <anvil/outline.h>
#include <anvil/dispmgrargs.h>

/***************************************************************************
 * 
 * Description:
 *	Used in the state handler when the pan handler is removed.  
 *	Remove the cursor handler, and restore the cursor pointer to normal
 * Arguments:
 *	dispMgrArgs - contains the object, display manager, outline, and flags
 *
 ****************************************************************************/

static void
removePanHandler( AmDispMgrArgsPtr	dispMgrArgs)
{
    AmMotifDisplayClass  *motifDisplay = ( AmMotifDisplayClass *) 
					    dispMgrArgs->displayManager;

    motifDisplay->SetCursor( XC_arrow);
    AmRemoveCursorHandler( dispMgrArgs);
}

/***************************************************************************
 * 
 * Description:
 * 	execute the pan drag by updating the drag outline to the current
 *	position 
 * Arguments:
 *	dispMgrArgs - contains the object, display manager, outline, and flags
 *	x, y - current position.
 *
 ****************************************************************************/

static void 
panDrag( AmDispMgrArgsPtr	   dispMgrArgs,
         int               x,
         int               y)
{
    AmMapManagerClass		*displayManager = dispMgrArgs->displayManager;
    AmDragLineOutlineClass	*outline;

    if ( dispMgrArgs->outline != NULL) {
	outline = ( AmDragLineOutlineClass *) dispMgrArgs->outline;
	outline->SetCurrPos( x, y);
	if ( dispMgrArgs->flag == TRUE) {
	    displayManager->ShowOutline( dispMgrArgs->outline);
	}
    }
}

/***************************************************************************
 * 
 * Description:
 *	execute the pan on button release.  Turn off the outline, get
 *	the start and end pan positions from the outline, execute the pan
 *	and flush the display 
 * Arguments:
 *	dispMgrArgs - contains the object, display manager, outline, and flags
 *
 ****************************************************************************/


static void
panRelease( AmDispMgrArgsPtr	dispMgrArgs)
{
    AmMapManagerClass		*displayManager = dispMgrArgs->displayManager;
    AmDragLineOutlineClass	*outline = ( AmDragLineOutlineClass *)
						dispMgrArgs->outline;
    int				x0, y0, x1, y1;

/* Pan the coordinate system of the plot, and restrict it to the extents of
   the data. */

    if ( outline != NULL) {
        displayManager->HideOutline( dispMgrArgs->outline);
    }
    outline->GetStartPos( &x0, &y0);
    outline->GetCurrPos( &x1, &y1);
    dispMgrArgs->coords->Pan( x0, y0, x1, y1, TRUE);
    displayManager->Flush();
}
 
/***************************************************************************
 * 
 * Description:
 *	Event handler for pan.  Active on MotionNotify, ButtonPress, 
 *      ButtonRelease.  If it's the AmFirst button press, then create the
 *	outline, otherwise set the starting coordinates.  The flag 
 *	field in dispMgrArgs is used to control updating the outline
 *	when the mouse is moved.  When the button is released, execute
 *	the pan.  If the third button is pressed, then return from pan
 *	mode.
 * Arguments:
 *	w - widget from the motif display that the pan is attached to
 *	dispMgrArgs - contains the object, display manager, outline, and flags
 *	event - Button press, release or motion event 
 *	continueDispatch - unused. 
 *
 ****************************************************************************/

/* ARGSUSED */

static void
panHandler( Widget		w,
	    AmDispMgrArgsPtr	dispMgrArgs,
            XEvent		*event,
            AnvilBoolean		*continueDispatch)
{
    AmMapManagerClass  *displayManager = dispMgrArgs->displayManager;
    XButtonEvent     *buttonEvent=NULL;
    XMotionEvent     *motionEvent;
    int              x=0, y=0;

    if ( event->type == ButtonPress) { 
        buttonEvent = ( XButtonEvent *)event;

        x = displayManager->VirtualCoord( buttonEvent->x);
        y = displayManager->VirtualCoord( buttonEvent->y);

    /* Right mouse button cancels the handler. */

        if ( buttonEvent->button == Button3) {
            displayManager->WriteMessage( "Pan canceled.");

        /* Remove the handler. */

            AmRemoveHandler( w, ButtonPressMask | ButtonReleaseMask |
			   PointerMotionMask,
			   ( XtEventHandler) panHandler, dispMgrArgs);
            return;
        }
    } else if ( event->type == ButtonRelease) {
        buttonEvent = ( XButtonEvent *) event;

        x = displayManager->VirtualCoord( buttonEvent->x);
        y = displayManager->VirtualCoord( buttonEvent->y);

    } else if ( event->type == MotionNotify) {
        motionEvent = ( XMotionEvent *) event;

        x = displayManager->VirtualCoord( motionEvent->x);
        y = displayManager->VirtualCoord( motionEvent->y);
    }
 
 /* on button press, set the selected object and the start position. */

    if ( event->type == ButtonPress) {
        if ( buttonEvent->button == Button1) {

/* create the outline on the AmFirst button press */

	    if ( dispMgrArgs->outline == NULL) {
		dispMgrArgs->outline = 
		    new AmDragLineOutlineClass( displayManager, x, y);
		displayManager->AddOutline( dispMgrArgs->outline);
	    } else {
		AmDragLineOutlineClass *dragOutline = 
		    (( AmDragLineOutlineClass *) dispMgrArgs->outline);

		dragOutline->SetStartPos( x, y);
		dragOutline->SetCurrPos( x, y);
		displayManager->ShowOutline( dispMgrArgs->outline);
	    }
	    dispMgrArgs->flag = TRUE;
        }
    } else if ( event->type == ButtonRelease) {
        if ( buttonEvent->button == Button1) {
	    if ( dispMgrArgs->flag == TRUE) {
		panRelease( dispMgrArgs);
	    }
        }
	dispMgrArgs->flag = FALSE;
    } else if ( event->type == MotionNotify) { 
        if ( dispMgrArgs->coords != NULL) {
            panDrag( dispMgrArgs, x, y);
        }
    }
}

/***************************************************************************
 *
 * Description:
 *	Set up the pan handlers for a plot.  The pan is set up as a 
 *	"constant" pan, where it stays in pan mode until button 3 is
 *	pressed.  Button 1 starts the pan, motion drags the outline
 *	and Button release executes the pan.  While the pan mode is
 *	active, the cursor is shown as a hand.  Pan uses an outline
 *	handler which draws a line from the start to the ending pan
 *	position.
 * Arguments:
 *	MotifDisplay - display manager containing the object to pan
 *	coords - object with coordinate system to pan (either a plot
 *	or an overlay 
 *
 ***************************************************************************/

/* ARGSUSED */

AnvilBoolean
AmPanCallback( Widget		w,
	     AmMotifDisplayClass	*motifDisplay,
	     AmCoordClass		*coords,
	     XtPointer)
{

    motifDisplay->WriteResourceMessage( "panInstructionMsg");
    motifDisplay->SetCursor( XC_hand1);

    AmAddHandler( w, motifDisplay->GetArea(), ButtonPressMask |
        ButtonReleaseMask | PointerMotionMask, 
        ( XtEventHandler) panHandler, 
        AmMakeDispMgrArgs( motifDisplay, coords, NULL),
	PAN_PRIORITY,
        ( AmStateHandlerFn) removePanHandler,
        ( AmStateHandlerFn) AmSuspendCursorHandler,
        ( AmStateHandlerFn) AmRestartCursorHandler);

    return( TRUE);
}
