#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <anvil/motif.h>
#include <anvil/mem_funcs.h>
#include <anvil/errlib.h>
#include <anvil/envdefs.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/va_cback.h>
#include <anvil/overlay.h>
#include <anvil/plot_cback.h>
#include <anvil/motifdisplay.h>
#include <anvil/createobject.h>
#include <anvil/template.h>
#include <anvil/utility.h>

#include <anvil/PickLayerDialogClass.h>
#include <anvil/EditLayerDialogClass.h>
#include <anvil/DeleteLayerDialogClass.h>

static AmActionDefRec actionAreaDef[] = {
    { "okBtn", NULL, NULL },
    { "cancelBtn", ( XtCallbackProc) AmPopdownCback, NULL },
};

/* OK callback to set the current layer */

static void SetCurrLayerOkCallback( Widget		btn,
				    Widget		list,
				    AmMotifDisplayClass	*motifDisplay,
				    AmOverlayClass	*overlay,
				    XtPointer)
{
    int			*posList, count, index, numLayers;
    AmDataGraphicClass	**layerList;

/* get the index from the list and blow away the dialog */

    if ( !XmListGetSelectedPos( list, &posList, &count)) {
	return;
    }
    XtDestroyWidget( AmGetWidgetShell( btn));

/* get the list of plots, and index it to get the current layer */

    if ( overlay->GetLayers( &layerList, &numLayers) != AmOK) {
	AmPrintResourceError( "failedToGetLayerList");
	return;
    }
    index = posList[ 0] - 1;
    if ( index < numLayers) {

/* if valid, send a user event that the current layer has changed */

	overlay->SetVaFields( AmNcurrLayer, layerList[ index], NULL);
	motifDisplay->Flush();
	AmSendUserEvent( AM_PKG_ID, AM_SELECT_LAYER, layerList[ index]);
    } else {
	AmPrintResourceError( "indexOutOfRange", index, numLayers);
    }
    free( posList);
    free( layerList);
}

/* callback function to set the current layer */

AnvilBoolean
AmSetCurrLayerCallback( Widget,	
                      AmMotifDisplayClass	*motifDisplay,
                      AmOverlayClass      *overlay,
                      XtPointer)
{
    AmActionRec           actionArea;
    char		**titleList;
    XmString		*mstrs;
    Widget		formDlog, shell, list, okBtn, actionForm;
    Arg			args[ 20];
    int			n, index, numLayers, numItems;
    AmDataGraphicClass	*currLayer, **layerList;

/* get the list of titles */

    titleList = AmLayerStringList( overlay, FALSE);
    if ( titleList == NULL) {
	AmAddResourceToLog( "failedToGetLayerList");
	return( FALSE);
    }
    if ( overlay->GetLayers( &layerList, &numLayers) != AmOK) {
	AmAddResourceToLog( "failedToGetLayerList");
	return( FALSE);
    }
    overlay->GetVaFields( AmNcurrLayer, &currLayer, NULL);
    index = AmGetIndex( currLayer, ( void **) layerList, numLayers);
    free( layerList);

/* create the form dialog */

    shell = motifDisplay->GetShell();
    n = AmMakeArgList( args,
                     XmNdeleteResponse, XmDESTROY,
                     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
                     NULL);
    formDlog = XmCreateFormDialog( shell, "currLayer", args, n);

    actionAreaDef[ 1].clientData = formDlog,
    actionForm = AmMakeActionArea( formDlog,
                     actionAreaDef, ELEMENTS( actionAreaDef),
                     &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);

/* display a list of the plots in the  overlay plot, with the current 
   layer selected */

    mstrs = AmMotifStringList( titleList, &numItems);
    if ( index == -1) {
	n = AmMakeArgList( args, 
			 StickLeft, StickRight,
			 AboveOf( actionForm),
			 XmNitems, mstrs,
			 XmNitemCount, numItems,
			 XmNvisibleItemCount, 8,
			 XmNselectionPolicy, XmSINGLE_SELECT,
			 NULL);
    } else {
	n = AmMakeArgList( args, 
			 StickLeft, StickRight,
			 AboveOf( actionForm),
			 XmNitems, mstrs,
			 XmNitemCount, numItems,
			 XmNvisibleItemCount, 8,
			 XmNselectionPolicy, XmSINGLE_SELECT,
			 XmNselectedItems, &mstrs[ index],
			 XmNselectedItemCount, 1,
			 NULL);
    }
    list = XmCreateScrolledList( formDlog, "currLayerList", args, n);
    XtManageChild( list);
    AmFreeMotifStringList( mstrs, numItems);

/* create the ok and cancel buttons, and assign the callbacks */

    okBtn = actionArea.buttons[ 0];
    AmAddVaCallback( okBtn, XmNactivateCallback,
		   ( VarargsCallback) SetCurrLayerOkCallback, 
		   list, motifDisplay, overlay, 
		   END_LIST);
    XtManageChild( formDlog);

    return( TRUE);
}

/* delete the selected layers */

/* ARGSUSED */
AnvilBoolean
AmDeleteLayerCallback( Widget              w,
                       AmMotifDisplayClass *pMotifDisplay,
                       AmOverlayClass      *pOverlay,
                       XtPointer           xpCallData)
{
    AmDeleteLayerDialogClass *pwcDialog;
   
    pwcDialog = new AmDeleteLayerDialogClass( w, "deleteLayerDialog",
                                              pOverlay, pMotifDisplay);
    pwcDialog->DeleteOverlay( FALSE);
    pwcDialog->Show();

    return( TRUE);
}



static void 
addLayerOkCallback( Widget				fileSB,
		    AmMotifDisplayClass			*motifDisplay,
		    AmOverlayClass			*overlay,
		    XmFileSelectionBoxCallbackStruct	*cb)
{
    char	 	*filename;
    AmScreenObjClass	*screenObj;
    Widget		shell = AmGetWidgetRealShell( fileSB);

/* grab the string from the file selection dialog.  The
   AmMakeFileSelectionDialog() function ensures that the file exists */

    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &filename);

/* add the layer to the overlay and flush the display, if successful, otherwise
   print an error and allow the user to access the log file */

    if ( AmInitLog() != AmOK) {
	AmPrintResourceWarn( shell, "failedToInitLog");
    }
    if ( AmAddLayerFromHeader( motifDisplay->GetWindow(), filename, overlay, 
			     AM_PREFS_DIR, &screenObj) == AmOK) {
	motifDisplay->Flush();
	XtDestroyWidget( XtParent( fileSB));
    } else {
	AmPrintLogError( shell, "AddLayerFailed", filename);
    }
    XtFree( filename);
}

/* the add layer callback creates a file selection dialog, then adds the
   file on the OK callback from the file selection */

/* ARGSUSED */
AnvilBoolean 
AmAddLayerCallback( Widget		btn,
		  AmMotifDisplayClass	*motifDisplay,
		  AmOverlayClass		*overlay,
		  XtPointer		callData)
{
    Widget	fileSB;
    Arg		args[ 20];
    int		n;

/* the "AmGetWidgetShell" recursively walks up the button's tree to find
   a shell widget (dialogs have to have shell parents) */
  
    n = AmMakeArgList( args,
		     XmNautoUnmanage, FALSE,
		     NULL);
    fileSB = AmMakeFileSelectionDialog( AmGetWidgetRealShell( btn), "AddLayer",
				      args, n, TRUE, 
				      ( VarargsCallback) addLayerOkCallback, 
				      motifDisplay, overlay, END_LIST);

/* add a callback to pop down the dialog on cancel */

    XtAddCallback( fileSB, XmNcancelCallback,
    		   ( XtCallbackProc) AmPopdownCback, fileSB);

    return( TRUE);
}
