#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <malloc.h>
#include <math.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <anvil/motif.h>
#include <anvil/plot.h>
#include <anvil/imageplot.h>
#include <anvil/scatterplot.h>
#include <anvil/overlay.h>
#include <anvil/linegraph.h>
#include <anvil/scattergraph.h>
#include <anvil/scale.h>
#include <anvil/tick.h>
#include <anvil/plot_cback.h>
#include <anvil/va_cback.h>
#include <anvil/state_hand.h>
#include <anvil/macros.h>
#include <anvil/geometry.h>
#include <anvil/userevents.h>
#include <anvil/units.h>
#include <anvil/sample.h>
#include <anvil/profile.h>
#include <anvil/defaults.h>
#include <anvil/utility.h>

static const int LEFT_MARGIN = 10, RECT_SIZE = 15;

/* thie client data for scatter graphs converts the x function for 
   distance, and the scatter plot's z function to a height function, 
   and keeps the profile start line position */

typedef struct scatterDistData_ {
    float	*startx;		/* needs to be pointer because */
    float	*starty;		/* values change */
    unitType	units;
    void	*xyClientData;
    void	*zClientData;
    AmAccessFn	xFn;
    int		xOffset;
    AmAccessFn	yFn;
    int		yOffset;
    AmAccessFn	zFn;
    int		zOffset;
} scatterDistDataRec, *scatterDistDataPtr;

static AmActionDefRec actionAreaDef[] = {
    { "closeBtn", ( XtCallbackProc) AmPopdownCback, NULL },
    { "helpBtn", NULL, NULL }
};

/* static variables for resources for graph-display manager offsets */

static int	sideOffset, bottomOffset;

/* update the legend when the profiler is refreshed */

void AmProfileClass::updateLegend()
{
    AmGraphClass		**graphList;
    int			numGraphs, numColors, ii, yy, elementHeight;
    Dimension		width, height;
    AmColorCellRec	foreground, textFg;
    AmColorCellPtr	colors;
    AmRectangleRec	clearRect;
    char		*title;

/* create the display, window, and gc for drawing with */

    legendDisplay->AllocNamedColor( AmGetDefault( "profile.textForeground", 
						"black"), &textFg);

/* get the size of the area we're drawing to, and the graphs we're profiling
   with */

    XtVaGetValues( legendArea,
		   XmNwidth, &width,
		   XmNheight, &height,
		   NULL);
    clearRect.x = 0;
    clearRect.y = 0;
    clearRect.width = width;
    clearRect.height = height;
    legendWindow->ClearRect( &clearRect);
    graph->GetLayers(( AmDataGraphicClass ***) &graphList, &numGraphs);

/* calculate the height for each element */

    elementHeight = (int)(height/( numGraphs*1.5));
    for ( ii = 0, yy = elementHeight/2; 
	  ii < numGraphs; 
	  ii++, yy += elementHeight) {

/* use the foreground color for line graphs, and the middle color for
   scatter graphs */

	if ( graphList[ ii]->GetObjectType() == AmLINE_GRAPH) {
	    graphList[ ii]->GetVaFields( AmNforeground, &foreground, 
					 AmNtitle, &title,
					 NULL);
	    legendGC->SetForegroundColor( &foreground);
	    legendDisplay->FillRectangle( legendWindow, legendGC, 
					  LEFT_MARGIN, yy,
					  RECT_SIZE, RECT_SIZE);

/* draw the title if there is one */

	    if ( title != NULL) {
		legendGC->SetForegroundColor( &textFg);
		legendDisplay->DrawText( legendWindow, legendGC, 
					 LEFT_MARGIN*2 + RECT_SIZE, 
					 yy + RECT_SIZE, title);
		FREE( title);
	    }
	} else if ( graphList[ ii]->GetObjectType() == AmSCATTER_GRAPH) {
	    graphList[ ii]->GetVaFields( AmNcolors, &colors,
					 AmNnumColors, &numColors,
					 AmNforeground, &foreground,
					 AmNtitle, &title,
					 NULL);
	    if (( numColors != 0) && ( colors != NULL)) {
		legendGC->SetForegroundColor( &colors[ numColors/2]);
		FREE( colors);
	    } else {
		legendGC->SetForegroundColor( &foreground);
	    }
	    legendDisplay->FillRectangle( legendWindow, legendGC, 
					  LEFT_MARGIN, yy,
				          RECT_SIZE, RECT_SIZE);

/* draw the title if there is one */

	    if ( title != NULL) {
		legendGC->SetForegroundColor( &textFg);
		legendDisplay->DrawText( legendWindow, legendGC, 
				         LEFT_MARGIN*2 + RECT_SIZE, 
				         yy + RECT_SIZE, title);
		FREE( title);
	    }
	}
    }
    if ( graphList != NULL) {
	FREE( graphList);
    }
}

/* update the profile graph with the data sampled under the line and 
   draw the text for the endpoints */

void AmProfileClass::updateProfileGraph()
{
    int			ii, width, height, size, x0, y0, x1, y1, 
    			numGraphs, rows, numpts, xOffset, yOffset,
			outlineStartX, outlineStartY, outlineLastX,
			outlineLastY;
    AnvilBoolean		flipRows, flipCols;
    AmAccessFn		xFn, yFn, zFn;
    void		*clientData;
    uchar_t		*data, *buffer;
    AmGraphClass		**graphList;
    unitType		xunit, yunit, measureUnit;
    float		x0pos, y0pos, x1pos, y1pos, xmin, ymin, 
			xmax, ymax, mindist, maxdist, min, max,
			subXmin, subYmin, subXmax, subYmax,
			startX, startY, lastX, lastY;
    AmSourceTypeEnum	sourceType;
    AmPlotClass		*plot;

    if ( outline == NULL) {
	return;
    }

/* get the start and end coordinates from the outline */

    outline->GetStartPos( &outlineStartX, &outlineStartY);
    outline->GetCurrPos( &outlineLastX, &outlineLastY);
    sourcePlot->GetDataPt( outlineStartX, outlineStartY, &startX, &startY);
    sourcePlot->GetDataPt( outlineLastX, outlineLastY, &lastX, &lastY);

/* get the overall distance for x scaling. Convert it to the units used in the
   profiler */
   
    sourcePlot->GetVaFields( AmNxunits, &measureUnit, NULL);
    UTMdist( startX, startY, lastX, lastY, measureUnit, &distance);
AN_DEBUG( 90, ( "distance = %.2f\n", distance));

/* if there are no plots to profile, then bail */

    if ( numProfilePlots == 0) {
	return;
    }

/* for each of the plots we're profiling, get the pertinent data */

    AN_DEBUG( 99, ("calling update profile handler\n"));
    graph->GetLayers(( AmDataGraphicClass ***) &graphList, &numGraphs);
    for ( ii = 0; ii < numProfilePlots; ii++) {
	plot = profilePlots[ ii];
	plot->GetRange( &xmin, &ymin, &xmax, &ymax);

/* get the points, clip them to the extents of the data, and if the line
   crosses the matrix, then sample the data. */

	x0pos = startX; 
	y0pos = startY;
	x1pos = lastX; 
	y1pos = lastY;
	if ( AmClip2d( &x0pos, &y0pos, &x1pos, &y1pos, xmin, ymin, xmax, ymax)) {
AN_DEBUG( 99, ("(%.2f, %.2f) (%.2f %.2f) clipped to (%.2f, %.2f) (%.2f %.2f)\n",
	    startX, startY, lastX, lastY, x0pos, y0pos, x1pos, y1pos));

/* profiling an image plot */

	    if ( plot->IsType( AmIMAGE_PLOT)) {
		plot->GetVaFields( AmNsourceType, &sourceType,
		    		   AmNdata, &data,
				   AmNsubRows, &height,
				   AmNsubCols, &width,
				   AmNsize, &size,
				   AmNcolorsFn,  &zFn,
				   AmNxunits, &xunit,
				   AmNyunits, &yunit,
				   AmNflipRows, &flipRows,
				   AmNflipCols, &flipCols,
				   AmNcolorsMin, &min,
				   AmNcolorsMax, &max,
				   NULL);
		buffer = graphList[ ii]->GetDataPointer();

/* is easy if the data is read in directly to a matrix... */

		if (( sourceType == AmDIRECT_SOURCE) ||
		    ( sourceType == AmMMAP_SOURCE) ||
		    ( sourceType == AmFILE_SOURCE)) {
		    plot->GetRowcol( x0pos, y0pos, height, width, 
				     flipRows, flipCols, &x0, &y0);
		    plot->GetRowcol( x1pos, y1pos, height, width, 
				     flipRows, flipCols, &x1, &y1);
		    AmSampleMatrix( data, size, width, height, 
				  x0, y0, x1, y1, buffer);
		} else if (( sourceType == AmPYRAMID_SOURCE) ||
			   ( sourceType == AmREAD_SOURCE)) {

/* and is a little harder to do if it's a pyramid, or read into a submatrix. */

		    plot->GetVaFields( AmNsubXmin, &subXmin,
				       AmNsubYmin, &subYmin,
				       AmNsubXmax, &subXmax,
				       AmNsubYmax, &subYmax,
				       NULL);
		    AmGetRowcolumn( x0pos, y0pos, height, width, 
				  flipRows, flipCols, 
				  subXmin, subYmin, subXmax, subYmax, &x0, &y0);
		    AmGetRowcolumn( x1pos, y1pos, height, width, 
				  flipRows, flipCols, 
				  subXmin, subYmin, subXmax, subYmax, &x1, &y1);
		    AmSampleMatrix( data, size, width, height, 
				  x0, y0, x1, y1, buffer);
		}

/* put the values in the graph */

		rows = MAX( abs(x1 - x0), abs( y1 - y0));
		graphList[ ii]->SetVaFields( AmNrows, rows,
					     AmNheightMin, min,
					     AmNheightMax, max,
					     AmNcurrYmin, min,
					     AmNcurrYmax, max,
					     NULL);
	    } else if ( plot->IsType( AmSCATTER_PLOT)) {

/* sample the scatter plot data and set in in the scatter graph. */

		plot->GetVaFields( AmNdata, &data,
				   AmNrows, &rows,
				   AmNsize, &size,
				   AmNxFn,  &xFn,
				   AmNxOffset, &xOffset,
				   AmNyFn,  &yFn,
				   AmNyOffset, &yOffset,
				   AmNclientData, &clientData,
				   AmNcolorsMin, &min, 
				   AmNcolorsMax, &max,
				   NULL);
		buffer = graphList[ ii]->GetDataPointer();
		AmSampleScatter( data, rows, xFn, xOffset, yFn, yOffset, 
			       clientData, size,
			       MIN( x0pos, x1pos), MIN( y0pos, y1pos),
			       MAX( x0pos, x1pos), MAX( y0pos, y1pos),
			       AmDist( x0pos, y0pos, x1pos, y1pos)*sampleWidth,
			       buffer, &numpts);
AN_DEBUG( 99, ("sampled %d points\n", numpts));
		graphList[ ii]->SetVaFields( AmNrows, numpts,
					     AmNheightMin, min,
					     AmNheightMax, max,
					     AmNcurrYmin, min,
					     AmNcurrYmax, max,
					     NULL);
	    }
	} else {

/* totally clipped out, show nothing */

	    graphList[ ii]->SetVaFields( AmNrows, 0,
					 NULL);
	}

/* take the distance of the line's start and end, and update the plot's
   xmin and xmax */

	if ( UTMdist( startX, startY, x0pos, y0pos, measureUnit, &mindist) &&
	     UTMdist( startX, startY, x1pos, y1pos, measureUnit, &maxdist)) {
AN_DEBUG( 99, ("xmin = %.2f, xmax = %.2f, currXmax = %.2f\n", 
		mindist, maxdist, distance));
	    graphList[ ii]->SetVaFields( AmNxmin, mindist,
					 AmNxmax, maxdist,
					 AmNcurrXmax, distance,
					 NULL);
	}
    }

/* set the x range of the plot to reflect the distance of the sampled line,
   and normalize the y range */

    graph->SetVaFields( AmNxmin, 0.0,
			AmNcurrXmin, 0.0,
			AmNxmax, distance,
			AmNcurrXmax, distance,
			AmNymin, 0.0,
			AmNcurrYmin, 0.0,
			AmNymax, 1.0,
			AmNcurrYmax, 1.0,
			NULL);

/* update the display and the readout */

    graphDisplay->Flush();
    free( graphList);
}

/* update the readout for the profiler */

void AmProfileClass::updateReadout()
{
    char		buffer[ MSGLEN], bufX0[ MSGLEN], bufY0[ MSGLEN],
			bufX1[ MSGLEN], bufY1[ MSGLEN], bufDist[ MSGLEN];
    unitType		xunit, yunit;
    float		bearing, startX, startY, lastX, lastY;
    AmRectangleRec	clearRect;
    Dimension		width, height;
    AmColorCellRec	textFg;
    int			outlineStartX, outlineStartY, outlineLastX,
			outlineLastY;

    if ( outline == NULL) {
	return;
    }

/* get the start and end coordinates from the outline, and convert them to
   the coordinate system of the plot being profiled */

    outline->GetStartPos( &outlineStartX, &outlineStartY);
    outline->GetCurrPos( &outlineLastX, &outlineLastY);
    sourcePlot->GetDataPt( outlineStartX, outlineStartY, &startX, &startY);
    sourcePlot->GetDataPt( outlineLastX, outlineLastY, &lastX, &lastY);

/* clear the readout widget */

    XtVaGetValues( readoutArea,
		   XmNwidth, &width,
		   XmNheight, &height,
		   NULL);
    clearRect.x = 0;
    clearRect.y = 0;
    clearRect.width = width;
    clearRect.height = height;
    readoutWindow->ClearRect( &clearRect);

/* get the x and y units from the source plot */

    sourcePlot->GetVaFields( AmNxunits, &xunit,
			     AmNyunits, &yunit,
			     NULL);

/* calculate the bearing, and change it to navigational bearing */

    bearing = 90.0 - RADTODEG( atan2( lastY - startY, lastX - startX));
    if ( bearing < 0.0) {
	bearing += 360.0;
    }

/* draw the readout text in the readout label. Use the units and variable from
   the source plot */

    AmPrintValue( xunit, startX, bufX0);
    AmPrintValue( yunit, startY, bufY0);
    AmPrintValue( xunit, lastX, bufX1);
    AmPrintValue( yunit, lastY, bufY1);
    AmPrintValue( units, distance, bufDist);
    sprintf( buffer, 
    "X: %s Y: %s          \nX: %s Y: %s          \ndistance: %s		\nbearing: %.2f\260       ", 
	     bufX0, bufY0, bufX1, bufY1, bufDist, bearing);

/* update the display and the readout */

    readoutDisplay->AllocNamedColor( AmGetDefault( "profile.readoutForeground", 
						 "black"), &textFg);
    readoutGC->SetForegroundColor( &textFg);
    readoutDisplay->DrawMultilineText( readoutWindow, readoutGC, 
				       20, 20, buffer, AmLEFT_HORIZONTAL);
}

/* update the profile display */

static void updateProfileHandler( Widget,
				  AmProfileClass		*profileClass,
				  XEvent		*,
				  AnvilBoolean		*)
{
    profileClass->updateProfileGraph();
    profileClass->updateReadout();
    profileClass->updateLegend();
}

/* redraw the profile readout when an expose event occurs */

static void refreshProfileReadoutHandler( Widget	w,
					  AmProfileClass	*profileClass,
					  XExposeEvent	*event,
					  AnvilBoolean 	*)
{
    profileClass->updateReadout();
}

/* redraw the profile graph when an expose event occurs */

static void refreshProfileGraphHandler( Widget		w,
					AmProfileClass	*profileClass,
					XExposeEvent	*event,
					AnvilBoolean 	*)
{
    profileClass->updateProfileGraph();
}

/* refresh the legend when it is exposed */

static void refreshLegendHandler( Widget	area,
				  AmProfileClass	*profile,
				  XExposeEvent	*event,
				  AnvilBoolean 	*)
{
    profile->updateLegend();
}

/* draw the line AmBetween the the button press point and the current 
   mouse drag point */

/* ARGSUSED */

static void drawProfileLineHandler( Widget		w,
				    AmProfileClass	*profileClass,
				    XMotionEvent	*event,
				    AnvilBoolean		*)
{
    AmPlotClass		*plot = profileClass->GetPlot();
    int			currX, currY;
    float		currXpos, currYpos;

    if ( plot->InRect( event->x, event->y)) {
	profileClass->GetOutline()->SetCurrPos( event->x, event->y);
	profileClass->GetOutline()->GetCurrPos( &currX, &currY);
	plot->GetDataPt( currX, currY, &currXpos, &currYpos);
	profileClass->SetEndPos( currXpos, currYpos);
    }
}

/* on button release, remove the draw profile line handler */

/* ARGSUSED */

static void removeClickProfileHandler( Widget	  		w,
				       AmProfileClass  		*profileClass,
				       XButtonEvent		*,
				       AnvilBoolean	  		*)
{
    XtRemoveEventHandler( w, PointerMotionMask, FALSE,
			 ( XtEventHandler) drawProfileLineHandler, 
			 profileClass);
    XtRemoveEventHandler( w, ButtonReleaseMask, FALSE,
			 ( XtEventHandler) removeClickProfileHandler,
			 profileClass);
    XtRemoveEventHandler( w, ButtonReleaseMask, FALSE,
			 ( XtEventHandler) updateProfileHandler, 
			 profileClass);
}

/* click profile handler only updates on button release */

static void clickProfileHandler( Widget			w,
			         AmProfileClass  		*profileClass,
				 XButtonEvent		*event,
				 AnvilBoolean		*)
{
    AmPlotClass			*plot = profileClass->GetPlot();
    AmMotifDisplayClass		*mainDisplay = profileClass->GetMainDisplay();
    AmDragLineOutlineClass	*outline;
    float			startX, startY;

/* erase the AmLast dragged line if there was one, otherwise, create a new outline */ 

    if ( profileClass->GetOutline() != NULL) {
	mainDisplay->HideOutline( profileClass->GetOutline());
    } else {
	outline = new AmDragLineOutlineClass( mainDisplay, event->x, event->y);
	mainDisplay->AddOutline( outline);
	profileClass->SetOutline( outline);
    }
    if ( plot->InRect( event->x, event->y)) {
	profileClass->GetOutline()->SetStartPos( event->x, event->y);
	profileClass->GetOutline()->SetCurrPos( event->x, event->y);
	plot->GetDataPt( event->x, event->y, &startX, &startY);
	profileClass->SetStartPos( startX, startY);
	profileClass->SetEndPos( startX, startY);
	mainDisplay->ShowOutline( profileClass->GetOutline());

/* insert the drawProfileLineHandler before the updateProfileHandler, because
   it updates the current position of the outline, which needs to be done before
   the update */

	XtAddEventHandler( w, ButtonReleaseMask, FALSE,
			   ( XtEventHandler) updateProfileHandler, 
			   profileClass);
	XtInsertEventHandler( w, PointerMotionMask, FALSE,
			     ( XtEventHandler) drawProfileLineHandler, 
			     profileClass, XtListHead);
	XtAddEventHandler( w, ButtonReleaseMask, FALSE,
			   ( XtEventHandler) removeClickProfileHandler, 
			   profileClass);
    }
}

/* on button release, remove the draw profile line handler */

static void removeClickDragProfileHandler( Widget	  	w,
					   AmProfileClass  	*profileClass,
					   XButtonEvent		*,
					   AnvilBoolean	  	*)
{
    XtRemoveEventHandler( w, PointerMotionMask, FALSE,
			 ( XtEventHandler) updateProfileHandler, 
			 profileClass);
    XtRemoveEventHandler( w, PointerMotionMask, FALSE,
			  ( XtEventHandler) drawProfileLineHandler, 
			  profileClass);
    XtRemoveEventHandler( w, ButtonReleaseMask, FALSE,
			  ( XtEventHandler) removeClickProfileHandler,
			  profileClass);
}

/* profile handler updates as mouse is dragged */

static void clickDragProfileHandler( Widget		w,
				     AmProfileClass  	*profileClass,
				     XButtonEvent	*event,
				     AnvilBoolean		*)
{
    AmPlotClass			*plot = profileClass->GetPlot();
    AmDragLineOutlineClass	*outline;
    AmMotifDisplayClass		*mainDisplay = profileClass->GetMainDisplay();
    float			startX, startY;

/* erase the AmLast dragged line if there was one, otherwise, create a new outline */ 

    if ( profileClass->GetOutline() != NULL) {
	mainDisplay->HideOutline( profileClass->GetOutline());
    } else {
	outline = new AmDragLineOutlineClass( mainDisplay, event->x, event->y);
	mainDisplay->AddOutline( outline);
	profileClass->SetOutline( outline);
    }
    if ( plot->InRect( event->x, event->y)) {
	profileClass->GetOutline()->SetStartPos( event->x, event->y);
	plot->GetDataPt( event->x, event->y, &startX, &startY);
	profileClass->SetStartPos( startX, startY);
	profileClass->SetEndPos( startX, startY);
	mainDisplay->ShowOutline( profileClass->GetOutline());
	XtAddEventHandler( w, PointerMotionMask, FALSE,
		           ( XtEventHandler) updateProfileHandler, 
			   profileClass);

/* insert the drawProfileLineHandler before the updateProfileHandler, because
   it updates the current position of the outline, which needs to be done before
   the update */

	XtInsertEventHandler( w, PointerMotionMask, FALSE,
			     ( XtEventHandler) drawProfileLineHandler, 
			     profileClass, XtListHead);
	XtAddEventHandler( w, ButtonReleaseMask, FALSE,
			   ( XtEventHandler) removeClickDragProfileHandler, 
			   profileClass);
    }
}

/* normalize graph ranges with the same app ID */

static void normalizeGraphRanges( AmGraphClass	**graphList,
				  int		numGraphs)
{
    char	*appId, *appIdVec[ MAXPLOTS];
    float	min, max, mins[ MAXPLOTS], maxs[ MAXPLOTS];
    int		numAppIds, ii, jj;
    AnvilBoolean	found;

/* find the broadest range by appId */

    numAppIds = 0;
    for ( ii = 0; ii < numGraphs; ii++) {
        if ( graphList[ ii] == NULL) {
            AN_DEBUG( 10, ( "normalizeGraphRanges: null graph.\n"));
            continue;
        }

	graphList[ ii]->GetVaFields( AmNappId, &appId, 
				     AmNheightMin, &min,
				     AmNheightMax, &max,
				     NULL);
	found = FALSE;
	for ( jj = 0; jj < numAppIds; jj++) {
	    if ( !strcmp( appIdVec[ jj], appId)) {
		if ( min < mins[ jj]) {
		    mins[ jj] = min;
		}
		if ( max > maxs[ jj]) {
		    maxs[ jj] = max;
		}
		found = TRUE;
		break;
	    }
	}
	if ( !found) {
	    appIdVec[ numAppIds] = strdup( appId);
	    mins[ numAppIds] = min;
	    maxs[ numAppIds] = max;
	    numAppIds++;
	}
	free( appId);
    }

/* reset the plot ranges by the new min and max values */

    for ( ii = 0; ii < numGraphs; ii++) {
	graphList[ ii]->GetVaFields( AmNappId, &appId, NULL);
	for ( jj = 0; jj < numAppIds; jj++) {
	    if ( !strcmp( appIdVec[ jj], appId)) {

AN_DEBUG( 10, ("setting range to %.2f, %.2f\n", mins[ jj], maxs[ jj]));

		graphList[ ii]->SetVaFields( AmNheightMin, mins[ jj],
					     AmNheightMax, maxs[ jj],
					     AmNcurrYmin, mins[ jj],
					     AmNcurrYmax, maxs[ jj],
					     NULL);
		break;
	    }
	}
	free( appId);
    }

/* free the temporary vector of app Ids */

    for ( ii = 0; ii < numAppIds; ii++) {
	free( appIdVec[ ii]);
    }
}

/* create the vertical tickmarks for the profile graphs, and adjust the margins
   for the overlay graph and bottom tickmarks if there's only one tick */

static void makeProfileTicks( AmMotifDisplayClass	*graphDisplay,
			      AmGraphClass	**graphList,
			      int		numGraphs,
			      AmOverlayClass	*overlay,
			      AmTickClass		*bottomTicks,
			      AmTickClass		**vertTicks,
			      int		*numTicks)
{
    AmWindowClass	*window = graphDisplay->GetWindow();
    int			numAppIds, ii, jj, leftOffset, rightOffset;
    char		*appId, *appIdVec[ MAXPLOTS];
    AnvilBoolean		found;
    AmGraphClass		*graph;
    AmColorCellRec	foregrounds[ MAXPLOTS], fgColor;
    Dimension		width, height;

/* find the appIds for the profile graphs */

    numAppIds = 0;
    for ( ii = 0; ii < numGraphs; ii++) {
	graphList[ ii]->GetVaFields( AmNappId, &appId,
				     AmNforeground, &fgColor,
				     NULL);
	found = FALSE;
	for ( jj = 0; jj < numAppIds; jj++) {
	    if ( !strcmp( appId, appIdVec[ jj])) {
		found = TRUE;
		break;
	    }
	}
	if ( !found) {
	    appIdVec[ numAppIds] = strdup( appId);
	    foregrounds[ numAppIds] = fgColor;
	    numAppIds++;
	}
	free( appId);
    }

/* draw the tickmarks for each unique appId */

    leftOffset = 0;
    rightOffset = 0;
    for ( ii = 0; ii < numAppIds; ii++) {
	graph = NULL;
	for ( jj = 0; jj < numGraphs; jj++) {
	    graphList[ jj]->GetVaFields( AmNappId, &appId,
					 NULL);
	    if ( !strcmp( appId, appIdVec[ ii])) {
		graph = graphList[ jj];
		free( appId);
		break;
	    }
	    free( appId);
	}
	if ( graph == NULL) {
	    AN_DEBUG( 10, ("graph is NULL!!!\n"));
	    continue;
	}

/* draw the tickmarks on the left and right in the foreground color 
   of their associated appId.  Save the tickmarks so we can blow them away
   when the new order comes along ... */

	if ( EVEN( ii)) {
	    vertTicks[ ii] = 
		new AmTickClass( window, graph, "profileTicks",
			       AmNleftMargin, leftOffset,
			       AmNbottomMargin, bottomOffset,
			       AmNwidth, sideOffset,
			       AmNtopMargin, 0,
			       AmNtickPosition, AmLEFT,
			       AmNresizeFont, FALSE,
			       AmNforeground, &foregrounds[ ii],
			       NULL);
	    leftOffset += sideOffset;
	} else {
	    vertTicks[ ii] = 
	    	new AmTickClass( window, graph, "profileTicks",
			       AmNrightMargin, rightOffset,
			       AmNtopMargin, 0,
			       AmNwidth, sideOffset,
			       AmNbottomMargin, bottomOffset,
			       AmNtickPosition, AmRIGHT,
			       AmNresizeFont, FALSE,
			       AmNforeground, &foregrounds[ ii],
			       NULL);
	    rightOffset += sideOffset;
	}
	graphDisplay->AddObject( vertTicks[ ii]);
    }

/* free the temporary vector of appIds */

    for ( ii = 0; ii < numAppIds; ii++) {
	free( appIdVec[ ii]);
    }
    *numTicks = numAppIds;


/* set the left and right margins for the overlay and distance tickmarks */

    overlay->SetVaFields( AmNleftMargin, leftOffset,
			  AmNrightMargin, rightOffset,
			  NULL);
    bottomTicks->SetVaFields( AmNleftMargin, leftOffset,
			      AmNrightMargin, rightOffset,
			      NULL);

/* set the window size to update the overlay and bottom ticks according
   to their margins */

    XtVaGetValues( graphDisplay->GetArea(),
		   XmNwidth, &width,
		   XmNheight, &height,
		   NULL);
    overlay->SetWindowSize( width, height);
    bottomTicks->SetWindowSize( width, height);
}

/* make the profile graph for an image */

AmErrorCode AmProfileClass::makeImageProfile( AmImagePlotClass	*imagePlot)
{
    unitType		zunits;
    int			size, numColors, rows, cols;
    float		min, max;
    AmAccessFn		zFn;
    void		*clientData;
    AmLineGraphClass	*lineGraph;
    unitType		zUnits;
    AmColorCellPtr	colors;
    const char		*title, *filename;
    uchar_t		*buffer;

/* get all the color information for the image plot.  The color index
   gets transposed into the line graph's heightIndex. */

    imagePlot->GetVaFields( AmNzunits, &zunits, 
			    AmNcolorsMin, &min,
			    AmNcolorsMax, &max,
			    AmNcolorsFn, &zFn,
			    AmNcolorsUnits, &zUnits,
			    AmNcolors, &colors,
			    AmNnumColors, &numColors,
			    AmNclientData, &clientData,
			    NULL);

/* get a decent title string to display in the legend */

    title = imagePlot->GetTitle();
    filename = imagePlot->GetFilename();
    if ( title == NULL) {
	if ( filename == NULL) {
	    title = AmGetDefault( "profile.unknown", "Unknown scatter graph");
	} else {
	    title = filename;
	}
    } 

/* allocate a large enough buffer to hold the data */

    rows = imagePlot->GetRows();
    cols = imagePlot->GetCols();
    size = imagePlot->GetSize();
    if (( buffer = ( uchar_t *) malloc( size*MAX( rows, cols))) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }

/* pick a color halfway through the image's colormap for the line graph's
   color so they're more distinctive */

    lineGraph = new AmLineGraphClass( graph, "lineGraph",
				    AmNxunits, units,
				    AmNheightUnits, zunits, 
				    AmNyunits, zUnits,
				    AmNheightMin, min,
				    AmNheightMax, max,
				    AmNcurrYmin, min,
				    AmNcurrYmax, max,
				    AmNxmin, 0.0,
				    AmNheightFn, zFn,
				    AmNclientData, clientData,
				    AmNsize, size,
				    AmNrows, 0,
				    AmNdata, buffer,
				    AmNsourceType, AmDIRECT_SOURCE,
				    AmNappId, imagePlot->GetAppId(),
				    AmNforeground, &colors[ numColors/2],
				    AmNtitle, title,
				    NULL);
    if ( colors != NULL) {
	FREE( colors);
    }
    return( AmOK);
}

/* function which takes the x,y value for a scatter point and the
   starting position of the profiler, and figures the distance for the
   x value in the graph */

static float scatterDistFn( uchar_t		*scatterValue,
			    scatterDistDataPtr	scatterDistData)
{
    float	x, y, distance;

    x = ( scatterDistData->xFn)( scatterValue + scatterDistData->xOffset, 
				 scatterDistData->xyClientData);
    y = ( scatterDistData->yFn)( scatterValue + scatterDistData->yOffset, 
				 scatterDistData->xyClientData);
    UTMdist( *scatterDistData->startx, *scatterDistData->starty, x, y, 
	     scatterDistData->units, &distance);
    return( distance);
}

/* calls the source plot's z function for the profile graph's y value */

static float scatterHeightFn( uchar_t			*scatterValue,
			      scatterDistDataPtr	scatterDistData)
{
    return(( scatterDistData->zFn)( scatterValue + scatterDistData->zOffset, 
				    scatterDistData->zClientData));
}
    
/* make the scatter graph profile for a scatter plot */

AmErrorCode AmProfileClass::makeScatterProfile( AmScatterPlotClass	*scatterPlot)
{
    unitType		xunits, zunits;
    int			size, zOffset, numColors, colorsOffset,
			xOffset, yOffset, rows; 
    float		min, max, colorsMin, colorsMax, *colorsBinvec;
    AmAccessFn		xFn, yFn, zFn, colorsFn;
    void		*xyClientData, *zClientData, *colorsClientData;
    AmColorCellPtr	colors;
    AmScatterGraphClass	*scatterGraph;
    scatterDistDataPtr	scatterDistData;
    const char		*filename, *title;
    uchar_t		*buffer;

    scatterPlot->GetVaFields( AmNcolorsUnits, &zunits, 
			      AmNxunits, &xunits,
			      AmNcolorsMin, &min,
			      AmNcolorsMax, &max,
			      AmNxFn, &xFn,
			      AmNxOffset, &xOffset,
			      AmNyFn, &yFn,
			      AmNyOffset, &yOffset,
			      AmNcolorsFn, &zFn,
			      AmNclientData, &xyClientData,
			      AmNcolorsClientData, &zClientData,
			      AmNcolorsOffset, &zOffset,
			      AmNcolorsFn, &colorsFn,
			      AmNcolorsClientData, &colorsClientData,
			      AmNnumColors, &numColors,
			      AmNcolors, &colors,
			      AmNcolorsMin, &colorsMin,
			      AmNcolorsMax, &colorsMax,
			      AmNcolorsBinvec, &colorsBinvec,
			      AmNcolorsOffset, &colorsOffset,
			      NULL);

/* get a decent title string to display in the legend */
 
    title = scatterPlot->GetTitle();
    filename = scatterPlot->GetFilename();
    if ( title == NULL) {
	if ( filename == NULL) {
	    title = AmGetDefault( "profile.unknown", "Unknown scatter graph");
	} else {
	    title = filename;
	}
    } 

/* create wrappers for the x and y functions, because X now represents
   distance from the start of the line.  The client data is a wrapper for
   the original client data */

    if (( scatterDistData = ( scatterDistDataPtr) 
		malloc( sizeof( scatterDistDataRec))) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    scatterDistData->xFn = xFn;
    scatterDistData->xOffset = xOffset;
    scatterDistData->yFn = yFn;
    scatterDistData->yOffset = yOffset;
    scatterDistData->zFn = zFn;
    scatterDistData->zOffset = zOffset;
    scatterDistData->units = xunits;
    scatterDistData->startx = &startXpos;
    scatterDistData->starty = &startYpos;
    scatterDistData->xyClientData = xyClientData;
    scatterDistData->zClientData = zClientData;

/* allocate a buffer large enough to hold the scatter data */

    size = scatterPlot->GetSize();
    rows = scatterPlot->GetRows();
    if (( buffer = ( uchar_t *) malloc( rows*size)) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }

/* the x and y pointer offsets are embedded in scatterDistData in their
   accessing functions, so the x and y offsets are set to 0 */

    scatterGraph = new AmScatterGraphClass( graph, 
					  "profileGraph",
					  AmNxunits, units,
					  AmNheightUnits, zunits, 
					  AmNheightMin, min,
					  AmNheightMax, max,
					  AmNyunits, zunits,
					  AmNxmin, 0.0,
					  AmNxFn, scatterDistFn,
					  AmNheightFn, scatterHeightFn,
					  AmNxOffset, 0,
					  AmNheightOffset, 0,
				    	  AmNclientData, scatterDistData,
					  AmNsize, size,
					  AmNrows, 0,
					  AmNdata, buffer,
					  AmNsourceType, AmDIRECT_SOURCE,
					  AmNappId, scatterPlot->GetAppId(),
					  AmNcolorsFn, colorsFn,
				          AmNcolorsClientData, colorsClientData,
				          AmNnumColors, numColors,
				          AmNcolors, colors,
					  AmNcolorsMin, colorsMin,
					  AmNcolorsMax, colorsMax,
					  AmNcolorsBinvec, colorsBinvec,
					  AmNcolorsOffset, colorsOffset,
					  AmNtitle, title,
					  NULL);
    if ( colors != NULL) {
	FREE( colors);
    }
    return( AmOK);
}

/* if the plot is updated, then free the old profile graphs, make
   the new ones, and update the display */

static void updateProfileGraphCallback( int		eventId,
					AmProfileClass	*clientData,
					AmScreenObjClass	*callData)
{
    if ( clientData->GetPlot() == ( AmPlotClass *) callData) {
AN_DEBUG( 40, ("calling updateProfileGraphCallback\n"));
	clientData->freeProfileGraph();
	clientData->makeProfileGraph();
	clientData->updateProfileGraph();
    }
}

/* free the graphs and tickmarks make in the AmLast profile graph so we
   can refresh the profiler to reflect the display */

void AmProfileClass::freeProfileGraph()
{

/* get the layers from the overlay and delete them */

    AmGraphClass	**graphList;
    int		numGraphs, ii;

    graph->GetLayers(( AmDataGraphicClass ***) &graphList, &numGraphs);
    for ( ii = 0; ii < numGraphs; ii++) {

/* remove the layer and delete the object */

	graph->RemoveLayer( graphList[ ii]);
	delete graphList[ ii];
    }
    free( graphList);

/* the tickmarks are automatically removed by deleting their parent graphs */

}

/* make the graph for the profile */

void AmProfileClass::makeProfileGraph()
{
    int			ii, jj, numGraphs;
    AmPlotClass		**plotPtr;
    AmOverlayClass	*overlay;
    char		resbuf[ RESLEN];
    const char		*appId;
    AmGraphClass		**graphList;
    AnvilBoolean		keepData;

    if ( profilePlots != NULL) {
	FREE( profilePlots);
    }
    if ( sourcePlot->IsType( AmOVERLAY)) {
	overlay = ( AmOverlayClass *) sourcePlot;
	overlay->GetLayers(( AmDataGraphicClass ***) &profilePlots, 
			   &numProfilePlots);
	plotPtr = profilePlots;
	jj = 0;

/* for each of the plots, use its appId and the name of the profile dialog to
   find out whether it's supposed to be profiled, then create a graph to
   draw the data profile */

	for ( ii = 0; ii < numProfilePlots; ii++) {
	    appId = plotPtr[ ii]->GetAppId();
	    plotPtr[ ii]->GetVaFields( AmNkeepData, &keepData,
				       NULL);
	    sprintf( resbuf, "%s.showProfile", appId);
	    if ( AmGetDefault( resbuf, ( AnvilBoolean) FALSE)) {
		plotPtr[ jj] = plotPtr[ ii];
		if ( plotPtr[ ii]->IsType( AmIMAGE_PLOT)) {
		    plotPtr[ ii]->GetVaFields( AmNkeepData, &keepData,
					       NULL);
		    if ( keepData) {
			makeImageProfile(( AmImagePlotClass *) plotPtr[ jj]);
		    }

/* track the ranges by appId */

		} else if ( plotPtr[ ii]->IsType( AmSCATTER_PLOT)) {
		    makeScatterProfile(( AmScatterPlotClass *) plotPtr[ jj]);
		}
		jj++;
	    }
	}
	numProfilePlots = jj;
	if ( numProfilePlots > 0) {
	    graph->GetLayers(( AmDataGraphicClass ***) &graphList, &numGraphs);

/* normalize the graphs and create the vertical tickmarks */

	    normalizeGraphRanges( graphList, numGraphs);
	    makeProfileTicks( graphDisplay, graphList, 
			      numGraphs, graph, bottomTicks,
			      vertTicks, &numTicks);
	    free( graphList);
	}
    }
}

/* turn off the drag handler when the dialog is destroyed */

static void removeProfileHandlerCallback( Widget,
					  Widget		area,
					  XtEventHandler	profileHandler,
					  AmProfileClass		*profileClass,
					  XtPointer)
{
    profileClass->GetMainDisplay()->RemoveOutline( profileClass->GetOutline());
    AmRemoveHandler( area, ButtonPressMask, profileHandler, profileClass);
}

/* turn off the remove handler when the area is destroyed */

static void removeRemoveProfileHandlerCallback( Widget		area,
					        Widget		formDlog,
					        XtEventHandler	profileHandler,
					        AmProfileClass	*profileClass,
					        XtPointer)
{
    AmRemoveVaCallback( formDlog, XmNdestroyCallback, 
		      ( VarargsCallback) removeProfileHandlerCallback,
		      area, profileHandler, profileClass, 
		      END_LIST);
}

/* free the profile class when the dialog is destroyed */

static void destroyDataProfileCallback( Widget,
					AmProfileClass	*profileClass,
					XtPointer)
{
    delete profileClass;
}

/* callback for data profiling */
			      
AnvilBoolean
AmDataProfileCallback( Widget		btn,
                     AmMotifDisplayClass	*mainDisplay,
                     AmPlotClass		*plot,
                     char		*name,
                     XtPointer)	
{
    AmProfileClass *profileClass = new AmProfileClass( mainDisplay, plot, name);
    AmSetRaiseCallback( btn, profileClass->GetDialog());

    return( TRUE);
}

/* destroy the profiler dialog when the profiler's event handler is
   superseded by a higher priority handler */

static void removeProfiler( AmProfileClass	*profiler)
{
    XtDestroyWidget( profiler->GetDialog());
}

/* create the profile class */

AmProfileClass::AmProfileClass( AmMotifDisplayClass	*mainDisplayArg,
			    AmPlotClass		*plotArg,
			    char		*nameArg)
{
    AmActionRec           actionArea;
    Widget		area;
    Arg			args[ 20];
    int			n;
    char		resbuf[ RESLEN];
    const char 		*mode, *unitStr;
    AmXDisplayClass	*pXDisplay;
    AmXWindowClass	*pXWindow;

    profilePlots = NULL;

/* create a form dialog, label for readout, close button and profile drawing
   area */

    n = AmMakeArgList( args,
		     XmNdeleteResponse, XmDESTROY,
		     XmNautoUnmanage, FALSE,
		     NULL);
    formDlog = XmCreateFormDialog( mainDisplayArg->GetShell(), nameArg, 
				   args, n);

    actionAreaDef[ 0].clientData = formDlog;
    actionForm = AmMakeActionArea( formDlog,
                     actionAreaDef, ELEMENTS( actionAreaDef),
                     &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);
    XtSetSensitive( actionArea.buttons[ 1], False);

    readoutArea = XtVaCreateManagedWidget( "readoutArea", xmLabelWidgetClass,
					  formDlog,
					  StickLeft, StickTop, 
					  XmNwidth, 200,
					  XmNheight, 100,
					  NULL);
    legendArea = XtVaCreateManagedWidget( "legendArea", 
					  xmDrawingAreaWidgetClass,
					  formDlog,
					  StickLeft, BelowOf( readoutArea),
					  RightWith( readoutArea),
					  AboveOf( actionForm),
					  XmNwidth, 200,
					  NULL);
    profArea = XtVaCreateManagedWidget( "profArea", xmDrawingAreaWidgetClass,
					formDlog,
					RightOf( readoutArea),
					StickTop, StickBottom, StickRight,
					XmNbottomOffset, 40,
					NULL);
    XtManageChild( formDlog);

/* set the start and end point to something... */

    distance = 0.0;

/* create the data structure to pass to the event handlers */

    mainDisplay = mainDisplayArg;
    sourcePlot = plotArg;
    name = nameArg;
    pXDisplay = new AmXDisplayClass( profArea);
    pXWindow = new AmXWindowClass( pXDisplay, profArea);
    graphDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);
    outline = NULL;

/* find out what units we're using for distance, and get the side and bottom
   offsets for the tickmarks and graph.  Get the sampling width for scatter 
   plots */ 

    sprintf( resbuf, "%s.graph.bottomOffset", name);
    unitStr = AmGetDefault( resbuf, AmPROFILE_DEFAULT_UNITS);
    AmGetUnitCode( unitStr, &units);
    sprintf( resbuf, "%s.profile.bottomOffset", name);
    bottomOffset = AmGetDefault( resbuf, AmPROFILE_BOTTOM_OFFSET);
    sprintf( resbuf, "%s.profile.sideOffset", name);
    sideOffset = AmGetDefault( resbuf, AmPROFILE_SIDE_OFFSET);
    sprintf( resbuf, "%s.profile.sampleWidth", name);
    sampleWidth = AmGetDefault( resbuf, AmPROFILE_SAMPLE_WIDTH);

/* create the overlay graph and the bottom tickmarks */

    graph = new AmOverlayClass( graphDisplay->GetWindow(), "profileGraph", 
			      AmNleftMargin, sideOffset,
			      AmNrightMargin, sideOffset,
			      AmNtopMargin, 0,
			      AmNbottomMargin, bottomOffset,
			      AmNxmin, 0.0,
			      AmNmaintainRange, FALSE,
			      AmNxunits, units,
			      NULL);
/* create the profile graphs. */

    bottomTicks = new AmTickClass( graphDisplay->GetWindow(), 	
				 graph, "profileBottomTicks",
				 AmNleftMargin, sideOffset,
				 AmNrightMargin, sideOffset,
				 AmNbottomMargin, 0,
				 AmNtickPosition, AmBOTTOM,
				 AmNheight, bottomOffset,
			         AmNresizeFont, FALSE,
				 NULL);
    graphDisplay->AddObject( graph);
    graphDisplay->AddObject( bottomTicks);
    makeProfileGraph();

/* create graphics context for the readout and legend */

    area = mainDisplay->GetArea();
    readoutDisplay = new AmXDisplayClass( readoutArea);
    readoutWindow = new AmXWindowClass( readoutDisplay, readoutArea);
    readoutGC = ( AmXGCClass *) readoutWindow->CreateGC();
    legendDisplay = new AmXDisplayClass( legendArea);
    legendWindow = new AmXWindowClass( legendDisplay, legendArea);
    legendGC = ( AmXGCClass *) legendWindow->CreateGC();

/* flag to not undraw the AmLast dragged line. add the drag or click handler 
   depending on the cursor mode add a callback to remove the profile handler 
   when the dialog is destroyed, and a callback to turn that handler off when 
   the area is destroyed */

    sprintf( resbuf, "%s.cursorMode", name);
    mode = AmGetDefault( resbuf, "click");
    if ( !strcasecmp( mode, "click")) {
	AmAddHandler( NULL, area, ButtonPressMask, 
		    ( XtEventHandler) clickProfileHandler, this, 
		    PROFILE_PRIORITY, ( AmStateHandlerFn) removeProfiler);
	AmAddVaCallback( formDlog, XmNdestroyCallback,
		       ( VarargsCallback) removeProfileHandlerCallback,
		       area, clickProfileHandler, this, 
		       END_LIST);
	AmAddVaCallback( area, XmNdestroyCallback,
		       ( VarargsCallback) removeRemoveProfileHandlerCallback,
		       formDlog, clickProfileHandler, this, 
		       END_LIST);

    } else {
	AmAddHandler( NULL, area, ButtonPressMask, 
		    ( XtEventHandler) clickDragProfileHandler, this,
		    PROFILE_PRIORITY, ( AmStateHandlerFn) removeProfiler);
	AmAddVaCallback( formDlog, XmNdestroyCallback,
		       ( VarargsCallback) removeProfileHandlerCallback,
		       area, clickDragProfileHandler, this, 
		       END_LIST);
	AmAddVaCallback( area, XmNdestroyCallback,
		       ( VarargsCallback) removeRemoveProfileHandlerCallback,
		       formDlog, clickDragProfileHandler, this, 
		       END_LIST);
    }

/* add the destroy callback to free everything up when the dialog's destroyed */

    XtAddCallback( formDlog, XmNdestroyCallback,
		   ( XtCallbackProc) destroyDataProfileCallback, this);

/* add the handlers to refresh the readout, graph, and legend when they're
   exposed */

    XtAddEventHandler( readoutArea, ExposureMask, FALSE,
		       ( XtEventHandler) refreshProfileReadoutHandler, this);
    XtAddEventHandler( profArea, ExposureMask, FALSE,
		       ( XtEventHandler) refreshProfileGraphHandler, this);
    XtAddEventHandler( legendArea, ExposureMask, FALSE,
		       ( XtEventHandler) refreshLegendHandler, this);

/* add the user event handler to regenerate the profile handler when the
   source object has been changed */

    AmAddCallbackWidget( AM_PKG_ID, AM_SET_FIELD_OBJECT, mainDisplay->GetArea(),
		       ( callbackProc) updateProfileGraphCallback, this);
}

/* destroy the profile class */

AmProfileClass::~AmProfileClass()
{
    AmRemoveCallbackWidget( AM_PKG_ID, AM_SET_FIELD_OBJECT, 
		          ( callbackProc) updateProfileGraphCallback, this,
			  mainDisplay->GetArea());
    if ( profilePlots != NULL) {
	FREE( profilePlots);
    }
    delete readoutGC;
    delete readoutWindow;
    delete readoutDisplay;
    delete legendGC;
    delete legendWindow;
    delete legendDisplay;
}
