#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <ctype.h>
#include <stddef.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/ScrolledW.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>

#include <anvil/errlib.h>
#include <anvil/tagstrs.h>
#include <anvil/strlib.h>
#include <anvil/xresource.h>
#include <anvil/macros.h>
#include <anvil/motif.h>
#include <anvil/envdefs.h>
#include <anvil/projects.h>
#include <anvil/projectsP.h>
#include <anvil/mem_funcs.h>
#include <anvil/events.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/files.h>
#include <anvil/units.h>
#include <anvil/slick.h>
#include <anvil/proj_setup.h>
#include <anvil/proj_setupP.h>

extern struct PJ_ELLPS pj_ellps[];
extern struct PJ_LIST pj_list[];
static GC quadGC = NULL;
static char *ellipseFields[] = { AmNaEllipseArg, 
			         AmNbEllipseArg,
			         AmNeEllipseArg,
			         AmNesEllipseArg,
			         AmNfEllipseArg,
			         AmNrfEllipseArg };

/* find an argument in a list of strings in the form "arg=value" */

static char *findArgValue( char *argName,
			   char **argList,
			   int  nArgs)
{
    int             ii, length = strlen( argName);
    static char     retArg[ 128];
    char           *val;

/* copy everything after the '=' sign if it's a value.
 If it's a flag, then just copy the flag */

    for( ii = 0; ii < nArgs; ii++) {
	if( !strncmp( argName, argList[ ii], length)) {
	    val = strchr( argList[ ii], '=');
AN_DEBUG( 90, ("found arg %s\n", argList[ ii]));
	    if( val != NULL) {
		strcpy( retArg, val + 1);
		return( retArg);
	    } else {
		strcpy( retArg, argList[ ii]);
	    }
	}
    }
    return( NULL);
}

/* convert NEWS format latitude longitude values to +/- signed values */

char *AmSignLatLonStr( char *str)
{
    char 	*p = &str[ strlen( str) - 1];
    static char	buf[ 64];
    float	val;

    if (( toupper( *p) == 'W') || ( toupper( *p) == 'S')) {
	val = atof( str);
	sprintf( buf, "%f", -val);
	return( buf);
    } else {
	val = atof( str);
	sprintf( buf, "%f", val);
	return( buf);
    }
}

/* create the form and widgets for the projection arguments */

static int projArgForm( char            *projName,
			AmProjFormDataPtr	projFormData)
{ 
    AmProjArgDataPtr	projArgData = projFormData->projArgData;
    int             	ii, count;
    Widget		form, field;
    char		*argName;

AN_DEBUG( 50, ("calling projArgForm\n"));
/* search for all arguments with the matching projection code */

    count = 0;
    for ( ii = 0; ii < projArgData->numProjArg; ii++) {
	if ( !strcmp( projArgData->projArg[ ii].projName, projName)) {
	    argName = projArgData->projArg[ ii].argName;
	    if ( AmGetTagType( argName) == AmBOOLEAN) {

/* set the toggle as the object itself */

		projFormData->argToggles[ count++] = 
		    AmAddObject( projFormData->projInfo, 
			       projFormData->argRowcol, argName);
	    } else {

/* create a form, toggle and text field */

		form = XtVaCreateManagedWidget( "form", xmFormWidgetClass,
						projFormData->argRowcol,
						NULL);
		field = AmAddObject( projFormData->projInfo, form, argName);
		XtVaSetValues( XtParent( field), 
			       StickLeft, StickTop, StickBottom,
			       XmNeditable, FALSE,
			       NULL);
		projFormData->argToggles[ count] = 
		    XtVaCreateManagedWidget( "tgl", xmToggleButtonWidgetClass,
					     form,
					     RightOf( XtParent( field)),
					     StickTop, 
					     StickBottom,
					     NULL);

/* make the toggles control the editability of the fields */

		XtAddCallback( projFormData->argToggles[ count],
			       XmNvalueChangedCallback,
			       ( XtCallbackProc) AmToggleEditableCallback, field);
		count++;
	    }
	}
    }
    AmAlignObjects( projFormData->argRowcol);
    return( TRUE);
}

/* callback for bringing up the panel of arguments when the user picks a
   projection */

static void projArgsCback( Widget 		list,
			   AmProjFormDataPtr	projFormData,
			   XtPointer 		)
{
    int                 count, *posList;

    XmListGetSelectedPos( list, &posList, &count);
    if ( count == 0) {
	return;
    }
    XtDestroyWidget( projFormData->argRowcol);
    projFormData->argRowcol =
	XtVaCreateWidget( "argRowcol", xmRowColumnWidgetClass, 
			  projFormData->scrollWin,
			  XmNorientation, XmVERTICAL,
			  XmNnumColumns, 1,
			  XmNpacking, XmPACK_COLUMN,
			  NULL);

/* get the name of the projection from the USGS list of projections,
   realise the widgets, because motif is broken */

    projArgForm( pj_list[ posList[ 0] - 1].id, projFormData);
    XtManageChild( projFormData->argRowcol);
    FREE( posList);
}

/* callback to show the ellipse values in the ellipse text fields */

static void showEllipseCallback( Widget			,
				 AmProjFormDataPtr	projFormData,
				 XmListCallbackStruct	*callData)
{
    char	buffer[ 256], unused[ 128], *ellparam;
    uint_t	ii;

/* the major axis is always the AmFirst field */

    sscanf( pj_ellps[ callData->item_position - 1].major,
	    "%[^=]=%s", unused, buffer);
    XmTextFieldSetString( projFormData->ellipseFields[ 0], buffer);
    XmToggleButtonSetState( projFormData->ellipseToggles[ 0], 
			    TRUE, FALSE);

/* but the other field might be inverse flattening, eccentricity sq, etc */

    ellparam = pj_ellps[ callData->item_position - 1].ell;
    for ( ii = 1; ii < VECLEN( ellipseFields); ii++) {
	if ( !strncmp( ellipseFields[ ii], ellparam, 
		       strlen( ellipseFields[ ii]))) {
	    sscanf( pj_ellps[ callData->item_position - 1].ell,
		    "%[^=]=%s", unused, buffer);
	    XmTextFieldSetString( projFormData->ellipseFields[ ii], buffer);
	    XmToggleButtonSetState( projFormData->ellipseToggles[ ii], 
				    TRUE, FALSE);
	} else {
	    XmTextFieldSetString( projFormData->ellipseFields[ ii], "0.0");
	    XmToggleButtonSetState( projFormData->ellipseToggles[ ii], 
				    FALSE, FALSE);
	}
    }
}

/* read the projection arguments from the dialog */

static void readProjArgs( AmProjFormDataPtr	projFormData,	
			  char			***projArgs,
			  int			*nArgs)
{
    int                 *posList, count, intVal, stringCount, ii;
    float		floatVal;
    AnvilBoolean		booleanVal;
    AmProjArgClass	*projInfo = projFormData->projInfo;
    AmProjArgDataPtr	projArgData = projFormData->projArgData;
    char		*usgsStrings[ 64], *argName, buffer[ 128],
			*projName, *ellipseName=NULL;
    varType		varCode;
    unitType		unitCode;

AN_DEBUG( 50, ("calling projArgForm\n"));

/* set the projection and ellipse values in the object */

    XmListGetSelectedPos( projFormData->projList, &posList, &count);
    projName = pj_list[ posList[ 0] - 1].id;
    XmListGetSelectedPos( projFormData->ellipseList, &posList, &count);

/* if the ellipse is set, then use it */

    if ( count > 0)  {
	ellipseName = pj_ellps[ posList[ 0] - 1].id;
	projFormData->projInfo->SetVaFields( AmNproj, projName,
					     AmNellipse, ellipseName,
					     NULL);
    } else {
	projFormData->projInfo->SetVaFields( AmNproj, projName,
					     NULL);
    }

/* now that the projection, ellipse name and projection fields have been
   set, we now set this projection information into the string list
   in the USGS projection format. Search for all arguments with the matching 
   projection code.  There's a parallel toggle button for each of the fields 
   ( which actually may be the field itself), and determines whether the 
   value should be transferred from the projection object to the USGS 
   projection strings */

    count = 0;
    sprintf( buffer, "proj=%s", projName);
    usgsStrings[ 0] = strdup( buffer);

/* if the user has selected a predefined ellipse, then use that string, 
   otherwise, find out what toggles were set and use those to set the 
   hand-coded ellipse parameters */

    if ( !XmToggleButtonGetState( projFormData->userEllipseTgl)) {
	sprintf( buffer, "ellps=%s", ellipseName);
	usgsStrings[ 1] = strdup( buffer);
	stringCount = 2;
    } else {
	stringCount = 1;
	for ( ii = 0; ii < (signed)VECLEN( ellipseFields); ii++) {
	    if ( XmToggleButtonGetState( projFormData->ellipseToggles[ ii])) {
		projInfo->GetVaFields( ellipseFields[ ii], &floatVal, NULL);
		sprintf( buffer, "%s=%f", ellipseFields[ ii], floatVal);
		usgsStrings[ stringCount++] = strdup( buffer);
	    }
	}
    }

/* args and toggles are misaligned and need separate counters */

    for ( ii = 0; ( int)ii < projArgData->numProjArg; ii++) {
	if ( !strcmp( projArgData->projArg[ ii].projName, projName)) {
	    argName = projArgData->projArg[ ii].argName;
	    if ( XmToggleButtonGetState( projFormData->argToggles[ count++])) {
		if ( AmGetTagType( argName) == AmBOOLEAN) {
		    projInfo->GetVaFields( argName, &booleanVal, NULL);
		    if ( booleanVal) {
			usgsStrings[ stringCount++] = strdup( argName);
		    }
		} else if ( AmGetTagType( argName) == AmINT) {
		    projInfo->GetVaFields( argName, &intVal, NULL);
		    sprintf( buffer, "%s=%d", argName, intVal);
		    usgsStrings[ stringCount++] = strdup( buffer);
		} else if ( AmGetTagType( argName) == AmFLOAT) {
		    projInfo->GetVaFields( argName, &floatVal, NULL);

/* get the units for the tag, and find out if it's latitude or longitude.
   If so, then convert the value from a perfectly reasonable signed value 
   the NEWS format used by the usgs */

		    if ( AmGetVariableCode( argName, &varCode)) {
			if ( AmGetVariableUnits( varCode, &unitCode)) {
			    if ( AmConverterExists( unitCode, AmUNIT_ARCDEG)) {
				if ( floatVal < 0.0) {
				    sprintf( buffer, "%s=%fs", 
					     argName, fabs( floatVal));
				} else {
				    sprintf( buffer, "%s=%fn", 
					     argName, fabs( floatVal));
				}
			    } else if ( unitCode == AmUNIT_LONGITUDE_ARCDEG) {
				if ( floatVal < 0.0) {
				    sprintf( buffer, "%s=%fw", 
					     argName, fabs( floatVal));
				} else {
				    sprintf( buffer, "%s=%fe", 
					     argName, fabs( floatVal));
				}
			    } else if ( unitCode == AmUNIT_LATITUDE_ARCDEG) {
				if ( floatVal < 0.0) {
				    sprintf( buffer, "%s=%fs", 
					     argName, fabs( floatVal));
				} else {
				    sprintf( buffer, "%s=%fn", 
					     argName, fabs( floatVal));
				}
			    } else {
				sprintf( buffer, "%s=%f", argName, floatVal);
			    }
			} else {
			    sprintf( buffer, "%s=%f", argName, floatVal);
			}
		    }
		    usgsStrings[ stringCount++] = strdup( buffer);
		}
	    }
	}
    }
    usgsStrings[ stringCount] = NULL;
    for ( ii = 0; ( int)ii < stringCount; ii++) {
	AN_DEBUG( 50, ("args[ %d] = %s\n", ii, usgsStrings[ ii]));
    }

/* copy over the vector */

    *nArgs = stringCount;
    *projArgs = ( char **) malloc( sizeof( char *)*( stringCount + 1));
    memcpy( *projArgs, usgsStrings, sizeof( char *)*( stringCount + 1));
}

/* read the arguments from the text widgets, and the ellipse and projection
   from the dialog. */

static void setProjArgsCback( Widget 		btn,
			      AmProjFormDataPtr	projFormData,
			      XtPointer)
{
    AnvilBoolean	inverseFlag;
    char	buffer[ 256];

    AmSendUserEventChildren( AM_PKG_ID, AM_OK, NULL, AmGetWidgetRealShell( btn));
    readProjArgs( projFormData, projFormData->projArgs, 
		  projFormData->nArgs);

/* if the projection inverse flag is set, then we change the "proj"
   directive to "inv-proj" */
   
    projFormData->projInfo->GetVaFields( AmNinverseProjection, &inverseFlag,
					 NULL);
    if ( inverseFlag) {
	sprintf( buffer, "inv-%s", ( *projFormData->projArgs)[ 0]);
	free( (*projFormData->projArgs)[ 0]);
	( *projFormData->projArgs)[ 0] = strdup( buffer);
    }
    XtDestroyWidget( AmGetWidgetShell( btn));
}


/* construct the list of the names of projections */

static int projectionNames( struct PJ_LIST  *pjList,
			    char            ***names)
{
    int           	ii;
    struct PJ_LIST	*ptr;

    for ( ii = 0, ptr = pjList; ptr->id != NULL; ii++, ptr++);
    *names = ( char **) malloc( sizeof( char *)*( ii + 1));
    for ( ptr = pjList, ii = 0; ptr->id != NULL; ptr++, ii++) {
	( *names)[ ii] = ptr->name;
    }
    ( *names)[ ii] = NULL;
    return( ii);
}

/* construct the list of the names of the ellipses */

static int ellipseNames( struct PJ_ELLPS 	*ellpsList,
			 char 			***names)
{
    int             	ii;
    struct PJ_ELLPS	*ptr;

    for ( ii = 0, ptr = ellpsList; ptr->id != NULL; ii++, ptr++);
    *names = ( char **) malloc( sizeof( char *)*( ii + 1));
    for ( ptr = ellpsList, ii = 0; ptr->id != NULL; ptr++, ii++) {
	( *names)[ ii] = ptr->name;
    }
    ( *names)[ ii] = NULL;
    return( ii);
}

/* find out what our current projection is */

static int projectionIdIndex( struct PJ_LIST *pjList,
			      char 	     *pjName)
{
    int index;

    for( index = 0; pjList->id != NULL; pjList++, index++) {
	if( !strcmp( pjList->id, pjName)) {
	    return( index);
	}
    }
    return( 0);
}

/* find out what our current ellipse is */

static int ellipseIdIndex( struct PJ_ELLPS *ellpsList,
			   char 	   *ellpsName)
{
    int  index;

    for ( index = 0; ellpsList->id != NULL; ellpsList++, index++) {
	if ( !strcmp( ellpsList->id, ellpsName)) {
	    return( index);
	}
    }
    return( 0);
}

/* read the data for the projection arguments. */

static AnvilBoolean readProjArgData( char 		*filename,
				AmProjArgDataPtr 	*projArgData)
{
    char    **strs, valType[32];
    int     ii;

AN_DEBUG( 50, ("proj filename = %s\n", filename));
    *projArgData =( AmProjArgDataPtr) calloc( 1, sizeof( AmProjArgDataRec));
    if (( strs = AmReadStrvec( filename)) == NULL) {
	return( FALSE);
    }
    ( *projArgData)->numProjArg = AmVeclen( ( const char **)strs);
    ( *projArgData)->projArg =
        ( AmProjArgPtr) calloc( (*projArgData)->numProjArg, sizeof( AmProjArgRec));
    for ( ii = 0; ii <( *projArgData)->numProjArg; ii++) {
	if ( sscanf( strs[ii], "%s %s %s %[0-9A-Za-z ]",
		     ( *projArgData)->projArg[ ii].projName,
		     ( *projArgData)->projArg[ ii].argName,
		     valType,
		     ( *projArgData)->projArg[ ii].argDesc) != 4) {
	    AmPrintResourceError( "failedReadProjArgs", filename, ii + 1);
	    return( FALSE);
        }

/* "val" is a value, "flag" is a toggle flag */
	
	if( !strcmp( valType, "val")) {
	   ( *projArgData)->projArg[ ii].argType = VALUE;
	} else {
	   ( *projArgData)->projArg[ ii].argType = FLAG;
	}
    }

/* free the string vector when we're done with it */

    AmFreeStrVec( strs);
    return( TRUE);
}

/* draw the quadrangles */

static void drawQuadCallback( Widget		area,
			      AmProjFormDataPtr	projFormData,
			      XtPointer		)
{
    Display	*display = XtDisplay( area);
    Window	win = XtWindow( area);
    AmProjQuadPtr	quads = projFormData->projQuads;
    float	xmin, ymin, xmax, ymax, pxmin, pymin, pxmax, pymax, 
		xmargin, ymargin;
    Dimension	width, height;
    int		ii, jj, dir, ascent, descent, size;
    XPoint	points[ 5];
    char	buffer[ MSGLEN];
    XCharStruct	overall;

    XtVaGetValues( area, 
		   XmNwidth, &width, 
		   XmNheight, &height,
		   NULL);
    XClearWindow( display, win);
    if ( quadGC == NULL) {
	quadGC = XCreateGC( display, win, 0, NULL);
    }
    xmin = HUGE;
    ymin = HUGE;
    xmax = -HUGE;
    ymax = -HUGE;
    for ( ii = 0; ii < projFormData->nQuads; ii++) {
	for ( jj = 0; jj < 4; jj++) {
	    if ( quads[ ii].xs[ jj] < xmin) {
		xmin = quads[ ii].xs[ jj];
	    }
	    if ( quads[ ii].ys[ jj] < ymin) {
		ymin = quads[ ii].ys[ jj];
	    }
	    if ( quads[ ii].xs[ jj] > xmax) {
		xmax = quads[ ii].xs[ jj];
	    }
	    if ( quads[ ii].ys[ jj] > ymax) {
		ymax = quads[ ii].ys[ jj];
	    }
	}
    }

/* inset everything with a nice margin */

    xmargin = ( xmax - xmin)/15.0;
    ymargin = ( ymax - ymin)/15.0;
    pxmin = xmin - xmargin;
    pymin = ymin - ymargin;
    pxmax = xmax + xmargin;
    pymax = ymax + ymargin;
    size = MIN( width, height);
    for ( ii = 0; ii < projFormData->nQuads; ii++) {
	for ( jj = 0; jj < 4; jj++) {
	    points[ jj].x = ( short)(size*( quads[ ii].xs[ jj] - pxmin)/
					  ( pxmax - pxmin));
	    points[ jj].y = ( short)(size*( pymax - quads[ ii].ys[ jj])/
				   	  ( pymax - pymin));
	}
	points[ 4] = points[ 0];
	XDrawLines( display, win, quadGC, points, 5, CoordModeOrigin);
    }

/* draw the lower left and upper right coordinates. */

    sprintf( buffer, "%.0f, %.0f", xmin, ymin);
    XQueryTextExtents( display, XGContextFromGC( quadGC),
		       buffer, strlen( buffer), 
		       &dir, &ascent, &descent, &overall);
    XDrawString( display, win, quadGC, 
		 ( int)(size*( xmin - pxmin)/( pxmax - pxmin)),
		 ( int)(size*( pymax - ymin)/( pymax - pymin)
                 + ascent + descent),
		 buffer, strlen( buffer));
    sprintf( buffer, "%.0f, %.0f", xmax, ymax);
    XQueryTextExtents( display, XGContextFromGC( quadGC),
		       buffer, strlen( buffer), 
		       &dir, &ascent, &descent, &overall);
    XDrawString( display, win, quadGC, 
		 ( int)(size*( xmax - pxmin)/( pxmax - pxmin) - overall.width),
		 ( int)(size*( pymax - ymax)/( pymax - pymin) - descent),
		 buffer, strlen( buffer));
}

/* test the projection, and send an expose to the drawProjectedQuads
   callback */

static void testProjectionCallback( Widget		btn,
				    AmProjFormDataPtr	projFormData,
				    XtPointer		)
{

/* send the user event that causes all the values to be read */

    AmSendUserEventChildren( AM_PKG_ID, AM_OK, NULL, AmGetWidgetRealShell( btn));

/* free the AmLast projection if there was one */

    if ( projFormData->testPJ != NULL){
	pj_free( projFormData->testPJ);
    }
/* read the projection arguments and initialize the projection */

    readProjArgs( projFormData, &projFormData->testProjArgs,
		  &projFormData->nTestProjArgs);
    projFormData->testPJ = pj_init( projFormData->nTestProjArgs,
				    ( const char **)projFormData->testProjArgs);
    if ( projFormData->testPJ == NULL) {
	AmPrintResourceError( "projFailed");
	return;
    }

/* make the projected quads area redraw */

    AmSendExpose( XtDisplay( projFormData->projQuadArea),
		XtWindow( projFormData->projQuadArea));
}

/* draw the projected quadrangles */

static void drawProjectedQuadCallback( Widget		area,
				       AmProjFormDataPtr	projFormData,
				       XtPointer	)
{
    Display	*display = XtDisplay( area);
    Window	win = XtWindow( area);
    AmProjQuadPtr	quads = projFormData->projQuads;
    float	xmin, ymin, xmax, ymax, pxmin, pymin, pxmax, pymax,
		xmargin, ymargin, x, y;
    Dimension	width, height;
    int		ii, jj, dir, ascent, descent, size;
    XPoint	points[ 5];
    char	buffer[ MSGLEN];
    XCharStruct	overall;
    AnvilBoolean	inverseFlag;

/* figure out the size of the window and close it */

    XtVaGetValues( area, 
		   XmNwidth, &width, 
		   XmNheight, &height,
		   NULL);
    XClearWindow( display, win);

/* get the inverse flag from the projection */

    projFormData->projInfo->GetVaFields( AmNinverseProjection, &inverseFlag,
					 NULL);

/* if the projection failed, then don't redraw */

    if ( projFormData->testPJ == NULL) {
	return;
    }
    if ( quadGC == NULL) {
	quadGC = XCreateGC( display, win, 0, NULL);
    }

/* find the extents of what we want to draw */

    xmin = HUGE;
    ymin = HUGE;
    xmax = -HUGE;
    ymax = -HUGE;
    for ( ii = 0; ii < projFormData->nQuads; ii++) {
	for ( jj = 0; jj < 4; jj++) {
	    x = quads[ ii].xs[ jj];
	    y = quads[ ii].ys[ jj];
	    projectPt( projFormData->testPJ, inverseFlag, &x, &y);
	    if ( x < xmin) {
		xmin = x;
	    }
	    if ( y < ymin) {
		ymin = y;
	    }
	    if ( x > xmax) {
		xmax = x;
	    }
	    if ( y > ymax) {
		ymax = y;
	    }
	}
    }

/* inset everything with a nice margin */

    xmargin = ( xmax - xmin)/15.0;
    ymargin = ( ymax - ymin)/15.0;
    pxmin = xmin - xmargin;
    pymin = ymin - ymargin;
    pxmax = xmax + xmargin;
    pymax = ymax + ymargin;
    size = MIN( width, height);
    for ( ii = 0; ii < projFormData->nQuads; ii++) {
	for ( jj = 0; jj < 4; jj++) {
	    x = ( short)quads[ ii].xs[ jj];
	    y = ( short)quads[ ii].ys[ jj];
	    projectPt( projFormData->testPJ, inverseFlag, &x, &y);
	    points[ jj].x = ( short)(size*( x - pxmin)/( pxmax - pxmin));
	    points[ jj].y = ( short)(size*( pymax - y)/( pymax - pymin));
	}
	points[ 4] = points[ 0];
	XDrawLines( display, win, quadGC, points, 5, CoordModeOrigin);
    }

/* draw the lower left and upper right coordinates. */

    sprintf( buffer, "%.0f, %.0f", xmin, ymin);
    XQueryTextExtents( display, XGContextFromGC( quadGC),
		       buffer, strlen( buffer), 
		       &dir, &ascent, &descent, &overall);
    XDrawString( display, win, quadGC, 
		 ( int)(size*( xmin - pxmin)/( pxmax - pxmin)),
		 ( int)(size*( pymax - ymin)/( pymax - pymin)
                 + ascent + descent),
		 buffer, strlen( buffer));
    sprintf( buffer, "%.0f, %.0f", xmax, ymax);
    XQueryTextExtents( display, XGContextFromGC( quadGC),
		       buffer, strlen( buffer), 
		       &dir, &ascent, &descent, &overall);
    XDrawString( display, win, quadGC, 
		 ( int)(size*( xmax - pxmin)/( pxmax - pxmin) - overall.width),
		 ( int)(size*( pymax - ymax)/( pymax - pymin) - descent),
		 buffer, strlen( buffer));

/* free the projection strings and the projection */

}

/* destroying projection information and dialog */

static void destroyProjDialogCback( Widget,
				    AmProjFormDataPtr 	projFormData,
				    XtPointer)
{
    FREE( projFormData->projArgData->projArg);
    FREE( projFormData->projArgData);
    FREE( projFormData);
}

/* form for dealing with map projections, ellipses and parameters */

void AmProjectionFormDialog( Widget 	parent,
			   char 	***projArgs,
			   int 		*nArgs,
			   Widget	*okBtn,
			   AmProjQuadPtr	projQuads,
			   int		nQuads,
			   AnvilBoolean	inverseFlag) 
{
    Widget	     projLbl, cancelBtn, ellipseLbl, formDlog, 
		     ellipseRowcol, projTestBtn, form;
    uint_t           ii;
    int              n, numProj, projIndex, ellipseIndex, numEllipse, 
		     numStrings;
    Arg              args[ 20];
    XmString         *mtStrings;
    AmProjFormDataPtr  projFormData;
    char             *projName, *ellipseName, **projNames, **ellipseNameList,
		     buf[ 256];

/* create the top level form for handling the map projection parameters */

    projFormData = ( AmProjFormDataPtr) calloc( 1, sizeof( AmProjFormDataRec));
    projFormData->projArgs = projArgs;
    projFormData->nArgs = nArgs;
    projFormData->projQuads = projQuads;
    projFormData->nQuads = nQuads;
    projFormData->projInfo = new AmProjArgClass( *projArgs, *nArgs, inverseFlag);
    projFormData->testProjArgs = NULL;
    projFormData->nTestProjArgs = 0;
    projFormData->testPJ = NULL;

/* open the file containing arguments for the projections and read it */

    sprintf( buf, "%s/interp/%s", getenv( CONFIG_ENV), PROJ_ARG_FILE);
    if( !readProjArgData( buf, &projFormData->projArgData)) {
	AmPrintResourceError( "failedReadProjArg", PROJ_ARG_FILE);
	return;
    }

/* create a modal dialog for selecting map projection parameters */

    n = AmMakeArgList( args, 
		     XmNautoUnmanage, FALSE,
		     XmNdeleteResponse, XmDESTROY,
		     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		     NULL);
    formDlog = XmCreateFormDialog( parent, "projForm", args, n);

/* create a list for the available projection ellipses.  put the
   ellipse list on top, because we want the projection list to  resize
   when the form is stretched */

    ellipseLbl = XtVaCreateManagedWidget( "ellipseLbl", xmLabelWidgetClass,
					  formDlog,
					  StickTop, StickLeft, 
					  XmNtopOffset, 10,
					  NULL);
    numEllipse = ellipseNames( pj_ellps, &ellipseNameList);
    mtStrings = AmMotifStringList( ellipseNameList, &numStrings);
    n = AmMakeArgList( args, 
		     XmNitems, mtStrings,
		     XmNitemCount, numStrings,
		     BelowOf( ellipseLbl),
		     StickLeft, 
		     XmNvisibleItemCount, 12,
		     XmNselectionPolicy, XmSINGLE_SELECT,
		     NULL);
    projFormData->ellipseList =
	XmCreateScrolledList( formDlog, "ellpseList", args, n);
    XtManageChild( projFormData->ellipseList);
    AmFreeMotifStringList( mtStrings, numStrings);
    FREE( ellipseNameList);

/* create a rowcolumn for displaying the ellipse values */

    ellipseRowcol = 
	XtVaCreateManagedWidget( "ellipseRowcol", xmRowColumnWidgetClass, 
				 formDlog,
				 RightOf( projFormData->ellipseList), StickTop,
				 XmNwidth, 300,
				 NULL);
    projFormData->userEllipseTgl = 
	AmAddObject( projFormData->projInfo, ellipseRowcol, AmNcustomEllipse);
    projFormData->inverseTgl = 
	AmAddObject( projFormData->projInfo, ellipseRowcol, AmNinverseProjection);
    for ( ii = 0; ii < VECLEN( ellipseFields); ii++) {
	form = XtVaCreateManagedWidget( "form", xmFormWidgetClass,
					ellipseRowcol, NULL);
	projFormData->ellipseFields[ ii] = 
	    AmAddObject( projFormData->projInfo, form, ellipseFields[ ii]);
	XtVaSetValues( XtParent( projFormData->ellipseFields[ ii]),
		       StickLeft, StickTop, StickBottom,
		       NULL);
	projFormData->ellipseToggles[ ii] = 
	    XtVaCreateManagedWidget( 
		"tgl", xmToggleButtonWidgetClass, form,
		 RightOf( XtParent( projFormData->ellipseFields[ ii])), 
		 StickTop, StickBottom,
		 NULL);
    }
    AmAlignObjects( ellipseRowcol);

/* create the projection label and projection list, from the USGS list
   of projection names */

    projLbl = XtVaCreateManagedWidget( "projLbl", xmLabelWidgetClass, 
				       formDlog,
				       BelowOf( projFormData->ellipseList),
				       StickLeft, 
				       XmNtopOffset, 10,
				       NULL);
    numProj = projectionNames( pj_list, &projNames);
    mtStrings = AmMotifStringList( projNames, &numStrings);

/* select the current projection name as the default if there is one */

    n = AmMakeArgList( args, 
		     XmNitems, mtStrings,
		     XmNitemCount, numStrings,
		     BelowOf( projLbl),
		     StickLeft, 
		     StickBottom,
		     XmNbottomOffset, 40,
		     XmNvisibleItemCount, 6,
		     XmNselectionPolicy, XmSINGLE_SELECT,
		     XmNbottomOffset, 40,
		     StickBottom,
		     NULL);
    projFormData->projList = 
	XmCreateScrolledList( formDlog, "projList", args, n);
    XtManageChild( projFormData->projList);
    AmFreeMotifStringList( mtStrings, numStrings);
    FREE( projNames);

/* scrolled window to display projection arguments */

    projFormData->scrollWin = 
	XtVaCreateManagedWidget( "argForm", 
				 xmScrolledWindowWidgetClass, formDlog, 
				 RightOf( projFormData->projList),
				 TopWith( projFormData->projList), 
				 StickBottom, 
				 XmNbottomOffset, 40,
				 XmNwidth, 300,
				 XmNscrollBarDisplayPolicy, XmSTATIC,
				 XmNscrollingPolicy, XmAUTOMATIC,
				 NULL);
    projFormData->argRowcol =
	XtVaCreateManagedWidget( "argRowcol", xmRowColumnWidgetClass, 
				 projFormData->scrollWin,
				 XmNorientation, XmVERTICAL,
				 XmNnumColumns, 1,
				 XmNpacking, XmPACK_COLUMN,
				 NULL);

/* if there are projection quads to display, then create a pair of
   drawing areas to display them, and a button to test the projection */

    if ( nQuads > 0) {
	projTestBtn = 
	    XtVaCreateManagedWidget( "projTestBtn", xmPushButtonWidgetClass,
				     formDlog, 
				     StickTop, 
				     RightOf( projFormData->scrollWin), 
				     StickRight,
				     NULL);
	projFormData->quadArea =
	    XtVaCreateManagedWidget( "quadArea", xmDrawingAreaWidgetClass,
				     formDlog, 
				     BelowOf( projTestBtn), 
				     RightOf( projFormData->scrollWin),
				     StickRight,
				     XmNwidth, 300,
				     XmNheight, 300,
				     XmNborderWidth, 2,
				     NULL);
	projFormData->projQuadArea =
	    XtVaCreateManagedWidget( "projQuadArea", xmDrawingAreaWidgetClass,
				     formDlog, 
				     BelowOf( projFormData->quadArea),
				     RightOf( projFormData->scrollWin),
				     StickRight,
				     StickBottom,
				     XmNbottomOffset, 40,
				     XmNwidth, 300,
				     XmNheight, 300,
				     XmNborderWidth, 2,
				     NULL);
	XtAddCallback( projFormData->quadArea, XmNexposeCallback,
		       ( XtCallbackProc) drawQuadCallback, projFormData);
	XtAddCallback( projFormData->projQuadArea, XmNexposeCallback,
		       ( XtCallbackProc) drawProjectedQuadCallback, 
		       projFormData);
	XtAddCallback( projTestBtn, XmNactivateCallback,
		       ( XtCallbackProc) testProjectionCallback, 
		       projFormData);
    } else {
	projFormData->quadArea = NULL;
	projFormData->projQuadArea = NULL;
    }

/* create OK and cancel buttons */

    *okBtn = XtVaCreateManagedWidget( "okBtn", xmPushButtonWidgetClass, 
				      formDlog, 
				      StickLeft, StickBottom, 
				      NULL);
    cancelBtn = XtVaCreateManagedWidget( "cancelBtn", xmPushButtonWidgetClass,
					 formDlog,
					 StickRight, StickBottom, 
					 NULL);

/* add callback to reset the form when a new projection is chosen */

    XtAddCallback( projFormData->projList, XmNsingleSelectionCallback,
		   ( XtCallbackProc) projArgsCback, projFormData);
    XtAddCallback( cancelBtn, XmNactivateCallback,
		  ( XtCallbackProc) AmPopdownCback, formDlog);
    XtAddCallback( formDlog, XmNdestroyCallback,
		  ( XtCallbackProc) destroyProjDialogCback, projFormData);
    XtAddCallback( *okBtn, XmNactivateCallback, 
		   ( XtCallbackProc) setProjArgsCback, projFormData);

/* add callback to show currently selected ellipse values */

    XtAddCallback( projFormData->ellipseList, XmNsingleSelectionCallback,
		   ( XtCallbackProc) showEllipseCallback,
		   projFormData);

/* select the map projection in the list if there is one.  We have to
   call this after the callbacks have been added, or they don't get
   called */

    projName = findArgValue( "proj", *projArgs, *nArgs);
    if ( projName != NULL) {
	projIndex = projectionIdIndex( pj_list, projName);
	XmListSelectPos( projFormData->projList, projIndex + 1, TRUE);
	XmListSetPos( projFormData->projList, projIndex + 1);
    }

/* set the ellispse in the  list if there is one */

    ellipseName = findArgValue( "ellps", *projArgs, *nArgs);
    if ( ellipseName != NULL) {
	ellipseIndex = ellipseIdIndex( pj_ellps, ellipseName);
	XmListSelectPos( projFormData->ellipseList, ellipseIndex + 1, TRUE);
	XmListSetPos( projFormData->ellipseList, ellipseIndex + 1);
    }
    XtManageChild( formDlog);
}

/* take a set of header files and create projection quads from them */


AnvilBoolean AmMakeProjQuads( char             **headers,
                       int              nHeaders,
                       AmProjQuadPtr      *quads,
                       int              *nQuads)
{
    float	xmin, ymin, xmax, ymax;
    int		ii, jj;
    AnvilBoolean	okFlag;

    *quads = ( AmProjQuadPtr) malloc( nHeaders*sizeof( AmProjQuadRec));
    jj = 0;
    for ( ii = 0; ii < nHeaders; ii++) {
	AmXResourceDbClass db( headers[ ii]);
	okFlag = TRUE;
	if ( !db.GetResource( AmNxmin, &xmin)) {
	    okFlag = FALSE;
	}
	if ( !db.GetResource( AmNymin, &ymin)) {
	    okFlag = FALSE;
	}
	if ( !db.GetResource( AmNxmax, &xmax)) {
	    okFlag = FALSE;
	}
	if ( !db.GetResource( AmNymax, &ymax)) {
	    okFlag = FALSE;
	}
	if ( okFlag) {
	    ( *quads)[ jj].xs[ 0] = xmin;
	    ( *quads)[ jj].ys[ 0] = ymin;
	    ( *quads)[ jj].xs[ 1] = xmax;
	    ( *quads)[ jj].ys[ 1] = ymin;
	    ( *quads)[ jj].xs[ 2] = xmax;
	    ( *quads)[ jj].ys[ 2] = ymax;
	    ( *quads)[ jj].xs[ 3] = xmin;
	    ( *quads)[ jj].ys[ 3] = ymax;
	    jj++;
	}
    }
    *nQuads = jj;
    return( TRUE);
}

/* constructor for projection arguments class */

void AmProjArgClass::initFields() 
{
    strcpy( projName, "utm");
    strcpy( ellipseName, "WGS84");
    customEllipse = FALSE;
    W = 0.0;
    alpha = 0.0;
    azi = 0.0;
    h = 0.0;
    k = 0.0;
    lat_0 = 0.0;
    lat_1 = 0.0;
    lat_2 = 0.0;
    lat_b = 0.0;
    lat_ts = 0.0;
    lon_0 = 0.0;
    lon_1 = 0.0;
    lon_2 = 0.0;
    lonc = 0.0;
    lsat = 1;
    no_cut = FALSE;
    no_rot = FALSE;
    ns = FALSE;
    path = 1;
    south = FALSE;
    tilt = 0.0;
    x_0 = 0.0;
    y_0 = 0.0;
    zone = 10;
    ellipse_a = 0.0;
    ellipse_b = 0.0;
    ellipse_es = 0.0;
    ellipse_e = 0.0;
    ellipse_rf = 0.0;
    ellipse_f = 0.0;
    inverseFlag = FALSE;
}

AmProjArgClass::AmProjArgClass() 
{
    initFields();
}
    
	
/* constructor which takes USGS format arguments */

AmProjArgClass::AmProjArgClass( char 	**args,
			    int		nArgs,
			    AnvilBoolean	inverseFlagArg)
{
    AmTagValueRec	tagValues[ MAXTAGS];
    char	tag[ FIELDLEN], value[ FIELDLEN];
    int		ii;

/* do the basic field initialization */

    initFields();
    inverseFlag = inverseFlagArg;

/* convert from the '=' format to tag-value pairs */

    for ( ii = 0; ii < nArgs; ii++) {
	sscanf( args[ ii], "%[^=]=%s", tag, value);
	tagValues[ ii].tag = strdup( tag);
	tagValues[ ii].tagCopied = TRUE;
	AmScanValue( tag, AmSignLatLonStr( value), 
		   &tagValues[ ii].value, &tagValues[ ii].copied);
    }
    setFields( tagValues, nArgs);
}

/* destructor for projection arguments */

AmProjArgClass::~AmProjArgClass()
{
}

/* set the fields for projection arguments */

int AmProjArgClass::setFields( AmTagValuePtr       tagValues,
			     int               numValues)
{
    int         numFieldDefs;
    AmFieldDefPtr fieldDefs;

    AmProjArgClass::GetFieldDefinitions( &fieldDefs, &numFieldDefs);
    return( SetFieldByDefinitions( fieldDefs, numFieldDefs,
                                   tagValues, numValues));
}

/* get the fields for projection arguments */

int AmProjArgClass::getFields( AmTagValuePtr      tagValues,
			     int              numValues)
{
    int         numFieldDefs;
    AmFieldDefPtr fieldDefs;

    AmProjArgClass::GetFieldDefinitions( &fieldDefs, &numFieldDefs);
    return( GetFieldByDefinitions( fieldDefs, numFieldDefs,
                                   tagValues, numValues));
}

/* dump the fields for a projection class */

int AmProjArgClass::dumpFields( AmFieldLengthPtr	fieldLengths,
			      AnvilBoolean		allFields)
{
    int         numFieldDefs;
    AmFieldDefPtr fieldDefs;

    AmProjArgClass::GetFieldDefinitions( &fieldDefs, &numFieldDefs);
    return( DumpFieldsByDefinitions( fieldLengths, 0, fieldDefs, numFieldDefs));
}

/* return the field definitions for a projection arguments class */

void AmProjArgClass::GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
					int		*numFieldDefs)
{
    static AmFieldDefRec ProjArgFieldDefinitions[] = {
	{ AmNWProjArg, offsetof( AmProjArgClass, W) },
	{ AmNaEllipseArg, offsetof( AmProjArgClass, ellipse_a) },
	{ AmNalphaProjArg, offsetof( AmProjArgClass, alpha) },
	{ AmNaziProjArg, offsetof( AmProjArgClass, azi) },
	{ AmNbEllipseArg, offsetof( AmProjArgClass, ellipse_b) },
	{ AmNcustomEllipse, offsetof( AmProjArgClass, customEllipse) },
	{ AmNeEllipseArg, offsetof( AmProjArgClass, ellipse_e) },
	{ AmNellipse, offsetof( AmProjArgClass, ellipseName), 0, AmSTATIC },
	{ AmNesEllipseArg, offsetof( AmProjArgClass, ellipse_es) },
	{ AmNfEllipseArg, offsetof( AmProjArgClass, ellipse_f) },
	{ AmNhProjArg, offsetof( AmProjArgClass, h) },
	{ AmNinvProj, offsetof( AmProjArgClass, projName), 0, AmSTATIC },
	{ AmNinverseProjection, offsetof( AmProjArgClass, inverseFlag) },
	{ AmNkProjArg, offsetof( AmProjArgClass, k) },
	{ AmNlat_0ProjArg, offsetof( AmProjArgClass, lat_0) },
	{ AmNlat_1ProjArg, offsetof( AmProjArgClass, lat_1) },
	{ AmNlat_2ProjArg, offsetof( AmProjArgClass, lat_2) },
	{ AmNlat_bProjArg, offsetof( AmProjArgClass, lat_b) },
	{ AmNlat_tsProjArg, offsetof( AmProjArgClass, lat_ts) },
	{ AmNlon_0ProjArg, offsetof( AmProjArgClass, lon_0) },
	{ AmNlon_1ProjArg, offsetof( AmProjArgClass, lon_1) },
	{ AmNlon_2ProjArg, offsetof( AmProjArgClass, lon_2) },
	{ AmNloncProjArg, offsetof( AmProjArgClass, lonc) },
	{ AmNlsatProjArg, offsetof( AmProjArgClass, lsat) },
	{ AmNno_cutProjArg, offsetof( AmProjArgClass, no_cut) },
	{ AmNno_rotProjArg, offsetof( AmProjArgClass, no_rot) },
	{ AmNnsProjArg, offsetof( AmProjArgClass, ns) },
	{ AmNpathProjArg, offsetof( AmProjArgClass, path) },
	{ AmNproj, offsetof( AmProjArgClass, projName), 0, AmSTATIC },
	{ AmNrfEllipseArg, offsetof( AmProjArgClass, ellipse_rf) },
	{ AmNsouthProjArg, offsetof( AmProjArgClass, south) },
	{ AmNtiltProjArg, offsetof( AmProjArgClass, tilt) },
	{ AmNx_0ProjArg, offsetof( AmProjArgClass, x_0) },
	{ AmNy_0ProjArg, offsetof( AmProjArgClass, y_0) },
	{ AmNzoneProjArg, offsetof( AmProjArgClass, zone) }
    };
    *fieldDefs = ProjArgFieldDefinitions;
    *numFieldDefs = VECLEN( ProjArgFieldDefinitions);
}
