#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <stdarg.h>
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <anvil/utility.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/userevents.h>
#include <anvil/plot.h>
#include <anvil/reference.h>
#include <anvil/plot_cback.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>

static AmActionDefRec actionAreaDef[] = {
    { "closeBtn", NULL, NULL }
};

static void flushChildCallback( int			eventId,
				AmMotifDisplayClass	*motifDisplay,
				AmMotifDisplayClass	*sourceDisplay,
				AmMotifDisplayClass	*flushDisplay)
{
    if ( flushDisplay == sourceDisplay) {
	motifDisplay->Flush();
    }
}

static void FlushCallback( Widget		area,
			   AmMotifDisplayClass	*refDisplay,
			   XtPointer)
{
    XtRemoveCallback( area, XmNexposeCallback,
		      ( XtCallbackProc) FlushCallback, refDisplay);
    refDisplay->Flush();
}

/* callback to create a popup dialog for a reference window */

AnvilBoolean AmReferenceWindowCallback( Widget    		btn,
                                 AmMotifDisplayClass 	*motifDisplay,
                                 AmCoordClass  		*source,
                                 XtPointer)
{
    Widget      		formDlog, area, actionForm, 
    				parent = motifDisplay->GetShell();
    AmXDisplayClass		*pXDisplay;
    AmXWindowClass		*pXWindow;
    AmActionRec   		actionArea;
    AmMotifDisplayClass		*refDisplay;
    AmReferenceWindowClass	*refWin;
    Arg				args[ 20];
    int				n;

/* create a form to control the min and max for the gradient */

    n = AmMakeArgList( args,
                     XmNautoUnmanage, FALSE,
		     XmNdeleteResponse, XmDESTROY,
                     XmNcolormap, AmGetAppColormap(),
                     XmNvisual, AmGetAppVisual(),
                     XmNdepth, AmGetAppDepth(),
                     NULL);
    formDlog = XmCreateFormDialog( parent, "referenceWindow", args, n);

/* make it so the user can only pop up one reference window */

    AmSetSensitiveDestroy( btn, formDlog);
    actionForm = AmMakeActionArea( formDlog,
                                 actionAreaDef, ELEMENTS( actionAreaDef),
                                 &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);
    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, formDlog,
				    StickTop, StickRight, StickLeft,
				    StickBottom,
				    XmNbottomOffset, 50,
				    XmNwidth, 300,
				    XmNheight, 300,
				    NULL);
    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    refDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);
    refWin = new AmReferenceWindowClass( motifDisplay->GetWindow(),
				       source, "reference",
				       AmNleftMargin, 0,
				       AmNrightMargin, 0,
				       AmNtopMargin, 0,
				       AmNbottomMargin, 0,
				       NULL);
    refDisplay->AddObject( refWin);

/* set up handlers to receive events from the source display */

    AmSetReferenceHandler( refDisplay, motifDisplay, refWin);
    XtAddCallback( actionArea.buttons[ 0], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
    XtAddCallback( area, XmNexposeCallback,
		   ( XtCallbackProc) FlushCallback, refDisplay);
    XtManageChild( formDlog);

    return( TRUE);
}

/* drag the reference window rectangle as the user moves the mouse */

static int 	lastX, lastY;

static void dragReferenceRectHandler( Widget		   area,
				      AmMotifDisplayClass	   *motifDisplay,
				      AmReferenceWindowClass *referenceWindow,
				      XMotionEvent	   *event,
				      AnvilBoolean		   *)
{
    AmCoordClass	*parent = ( AmCoordClass *) referenceWindow->getParent();
    float		x0, y0, x1, y1, xmin, ymin, xmax, ymax,
			currXmin, currYmin, currXmax, currYmax;
    
AN_DEBUG( 90, ("dragging pan handler\n"));

/* restrict the bounds of the drag rectangle to the extents of the parent */

    parent->GetRange( &xmin, &ymin, &xmax, &ymax);
    parent->GetCoords( &currXmin, &currYmin, &currXmax, &currYmax);
    referenceWindow->GetPlot()->GetDataPt( lastX, lastY, &x0, &y0);
    referenceWindow->GetPlot()->GetDataPt( event->x, event->y, &x1, &y1);

/* change the parent's coordinate system */

    currXmin += x1 - x0;
    currYmin += y1 - y0;
    currXmax += x1 - x0;
    currYmax += y1 - y0;
    if (( currXmin > xmin) && ( currYmin > ymin) &&
        ( currXmax < xmax) && ( currYmax < ymax)) {
        parent->SetCoords( currXmin, currYmin, currXmax, currYmax);
    }
    lastX = event->x;
    lastY = event->y;
}

/* remove the handlers and update the parent plot on button release */

static void donePanReferenceHandler( Widget		   area,
				     AmMotifDisplayClass	   *motifDisplay,
				     AmMotifDisplayClass	   *sourceDisplay,     
				     AmReferenceWindowClass  *referenceWindow,
				     XMotionEvent	   *event,
				     AnvilBoolean		   *)
{
    AmCoordClass	*parent = ( AmCoordClass *) referenceWindow->getParent();
    float		x0, y0, x1, y1, xmin, ymin, xmax, ymax,
			currXmin, currYmin, currXmax, currYmax;

AN_DEBUG( 90, ("done pan handler\n"));
    AmRemoveVaHandler( motifDisplay->GetArea(), PointerMotionMask, FALSE, 
		  ( VarargsHandler) dragReferenceRectHandler, 
		  motifDisplay, referenceWindow, END_LIST);
    AmRemoveVaHandler( motifDisplay->GetArea(), ButtonReleaseMask, FALSE, 
		  ( VarargsHandler) donePanReferenceHandler,
		  motifDisplay, sourceDisplay, referenceWindow, END_LIST);

/* restrict the bounds of the drag rectangle to the extents of the parent */

    parent->GetRange( &xmin, &ymin, &xmax, &ymax);
    parent->GetCoords( &currXmin, &currYmin, &currXmax, &currYmax);
    referenceWindow->GetPlot()->GetDataPt( lastX, lastY, &x0, &y0);
    referenceWindow->GetPlot()->GetDataPt( event->x, event->y, &x1, &y1);

/* change the parent's coordinate system */

    currXmin += x1 - x0;
    currYmin += y1 - y0;
    currXmax += x1 - x0;
    currYmax += y1 - y0;
    if (( currXmin > xmin) && ( currYmin > ymin) &&
        ( currXmax < xmax) && ( currYmax < ymax)) {
        parent->SetCoords( currXmin, currYmin, currXmax, currYmax);
    }
    sourceDisplay->Flush();
}

/* start the pan handlers for the reference window */

static void referencePanHandler( Widget			area,
				 AmMotifDisplayClass	*motifDisplay,
				 AmMotifDisplayClass	*sourceDisplay,     
				 AmReferenceWindowClass	*referenceWindow,
				 XButtonEvent		*event,
				 AnvilBoolean		*)
{
    AmRectangleRec const 	*refRect = referenceWindow->GetRect();
    AmCoordClass		*parent = ( AmCoordClass *) referenceWindow->getParent(),
			*refPlot = referenceWindow->GetPlot();
    float		xpos, ypos;

AN_DEBUG( 90, ("referencePanHandler\n"));
/* find out if it's in the screen object rectangle, then if it's in the 
   zoom rectangle */

    if ( AmPtInRect( refRect, event->x, event->y)) {

/* remember that the reference plot has 0,0, for x,y */

	refPlot->GetDataPt( event->x - refRect->x, event->y - refRect->y, 
			    &xpos, &ypos);
	lastX = event->x;
	lastY = event->y;
	if ( parent->InCoords( xpos, ypos)) {
AN_DEBUG( 90, ("starting pan handler\n"));
	    AmAddVaHandler( motifDisplay->GetArea(), PointerMotionMask, FALSE, 
			  ( VarargsHandler) dragReferenceRectHandler, 
			  motifDisplay, referenceWindow, 
			  END_LIST);
	    AmAddVaHandler( motifDisplay->GetArea(), ButtonReleaseMask, FALSE, 
			  ( VarargsHandler) donePanReferenceHandler,
			  motifDisplay, sourceDisplay, referenceWindow, 
			  END_LIST);
	}
    }
}

/* update the reference window when the source plot changes */

static void updateReferenceHandler( int				eventId,
				    AmReferenceWindowClass	*refWin,
				    AmMotifDisplayClass		*motifDisplay,
				    AmScreenObjClass		*source)
{
    if ( source == refWin->getParent()) {

/* "transfer" the event to the reference window on this motif display */

AN_DEBUG( 40, ("sending reference window event %d\n", eventId));
	AmSendUserEvent( AM_PKG_ID, eventId, refWin);
	motifDisplay->Flush();
    }
}

/* remove the pan handler when the reference window is destroyed */

static void destroyReferenceHandler( int		   eventId,
				     AmReferenceWindowClass  *refWin,
				     AmMotifDisplayClass	   *sourceDisplay,     
				     AmMotifDisplayClass	   *motifDisplay,     
				     AmScreenObjClass	   *source)

{
    if (( source == refWin->getParent()) || ( source == refWin)) {
AN_DEBUG( 40, ( "calling destroyReferenceHandler\n"));
	AmRemoveVaHandler( motifDisplay->GetArea(), ButtonPressMask, FALSE,
			( VarargsHandler) referencePanHandler, 
			motifDisplay, sourceDisplay, refWin, 
			END_LIST);

/* if the parent display is flushed, then flush the child display as well */

	AmRemoveVaCallbackWidget( AM_PKG_ID, AM_FLUSH, sourceDisplay->GetArea(),
			        ( VarargsCallback) flushChildCallback, 
			        motifDisplay, sourceDisplay,
			        END_LIST);
	AmRemoveVaCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, 
				 sourceDisplay->GetArea(),
			        ( VarargsCallback) destroyReferenceHandler, 
			         refWin, sourceDisplay, motifDisplay,
				 END_LIST);
	if ( motifDisplay != sourceDisplay) {
	    AmRemoveVaCallbackWidget( AM_PKG_ID, AM_SET_FIELD_OBJECT, 
				    sourceDisplay->GetArea(),
				    ( VarargsCallback) updateReferenceHandler, 
				    refWin, motifDisplay,
				    END_LIST);
	    AmRemoveVaCallbackWidget( AM_PKG_ID, AM_COORDS_CHANGE, 
				    sourceDisplay->GetArea(),
				    ( VarargsCallback) updateReferenceHandler, 
				    refWin, motifDisplay,
				    END_LIST);
	}
    }
}
				     
/* set a permanent event handler which tests for panning the 
   reference window */

AnvilBoolean
AmSetReferenceHandler( AmMotifDisplayClass	*motifDisplay,
		     AmMotifDisplayClass	*sourceDisplay,
		     AmReferenceWindowClass	*refWin)
{

/* add callbacks to remove all the reference handlers if the drawing areas
   for the parent or reference display manager are deleted */


/* add an event handler to support panning in the reference window */

    AmAddVaHandler( motifDisplay->GetArea(), ButtonPressMask, FALSE,
		  ( VarargsHandler) referencePanHandler, 
		  motifDisplay, sourceDisplay, refWin, END_LIST);

/* if the parent display is flushed, then flush the child display as well */

    AmAddVaCallbackWidget( AM_PKG_ID, AM_FLUSH, sourceDisplay->GetArea(),
		         ( VarargsCallback) flushChildCallback, 
			 motifDisplay, sourceDisplay,
			 END_LIST);

/* if the parent is deleted, then blow away the reference handler */

    AmAddVaCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, sourceDisplay->GetArea(),
		         ( VarargsCallback) destroyReferenceHandler, 
			 refWin, sourceDisplay, motifDisplay, 
			 END_LIST);

/* if the source and reference display managers aren't the same, then
   add callbacks to update the reference window whenever the parent is
   changed or zoomed */

    if ( sourceDisplay != motifDisplay) {
	AmAddVaCallbackWidget( AM_PKG_ID, AM_SET_FIELD_OBJECT, 
			     sourceDisplay->GetArea(),
			     ( VarargsCallback) updateReferenceHandler, 
			     refWin, motifDisplay, 
			     END_LIST);
	AmAddVaCallbackWidget( AM_PKG_ID, AM_COORDS_CHANGE, 
			     sourceDisplay->GetArea(),
			     ( VarargsCallback) updateReferenceHandler, 
			     refWin, motifDisplay, 
			     END_LIST);
    }

    return( TRUE);
}
