#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <anvil/scatterplot.h>
#include <anvil/motifdisplay.h>
#include <anvil/plot_cback.h>
#include <anvil/state_hand.h>
#include <anvil/eventcodes.h>
#include <anvil/userevents.h>

static int READOUT_CURSOR_SIZE = 10;

typedef struct scatterReadoutData_ {
    AmMotifDisplayClass	*motifDisplay;
    AmScatterPlotClass	*scatterPlot;
    AmObjectOutlineClass	*outline;
} scatterReadoutDataRec, *scatterReadoutDataPtr;

/* drag handler for the scatter plot readout */

static void scatterReadoutDragHandler( 
	Widget			area,
        scatterReadoutDataPtr	scatterReadoutData,
        XMotionEvent		*event,
        AnvilBoolean			*continueDispatch)
{
    AmScatterPlotClass	*scatterPlot = scatterReadoutData->scatterPlot;
    float		ptX, ptY;
    int			index, xOffset, yOffset, size, xpos, ypos;
    void		*clientData;
    AmAccessFn		xFn, yFn;
    uchar_t		*scatterData;

/* find the closest point */

    scatterPlot->GetDataPt( event->x, event->y, &ptX, &ptY);
    index = AmClosestScatterPointIndex( ptX, ptY, scatterPlot);

/* get the coordinates of the point and draw the new cursor */

    scatterPlot->GetVaFields( AmNdata, &scatterData,
			      AmNxFn, &xFn,
			      AmNxOffset, &xOffset,
			      AmNyFn, &yFn,
			      AmNyOffset, &yOffset,
			      AmNclientData, &clientData,
			      NULL);
    size = scatterPlot->GetSize();
    ptX = ( xFn)( &scatterData[ size*index + xOffset], clientData);
    ptY = ( yFn)( &scatterData[ size*index + yOffset], clientData);
    scatterPlot->GetScreenPt( ptX, ptY, &xpos, &ypos);
    scatterReadoutData->outline->SetRectangle( xpos - READOUT_CURSOR_SIZE/2,
					       ypos - READOUT_CURSOR_SIZE/2,
					       READOUT_CURSOR_SIZE, 
					       READOUT_CURSOR_SIZE);
}

/* press and release button handler to launch and remove the drag handler */

static void scatterReadoutButtonHandler( 
	Widget			area,
	scatterReadoutDataPtr	scatterReadoutData,
	XButtonEvent		*event,
	AnvilBoolean			*continueDispatch)
{
    AmMotifDisplayClass  	*motifDisplay = scatterReadoutData->motifDisplay;

    if ( event->type == ButtonPress) {

/* if the button was cancel, then cancel the event handler */

	if ( event->button == Button3) {
	    AmRemoveHandler( area, ButtonPressMask | ButtonReleaseMask,
			   ( XtEventHandler) scatterReadoutButtonHandler, 
			   scatterReadoutData);
	    return;
	}
	scatterReadoutData->outline = 
		new AmObjectOutlineClass( motifDisplay, 
					event->x - READOUT_CURSOR_SIZE/2,
				        event->y - READOUT_CURSOR_SIZE/2,
					READOUT_CURSOR_SIZE, 
					READOUT_CURSOR_SIZE);
	motifDisplay->AddOutline( scatterReadoutData->outline);
	XtAddEventHandler( area, PointerMotionMask, FALSE,
			   ( XtEventHandler) scatterReadoutDragHandler,
			   scatterReadoutData);
    } else {

	if ( scatterReadoutData->outline != NULL) {
	    motifDisplay->RemoveOutline( scatterReadoutData->outline);
	    delete scatterReadoutData->outline;
	    scatterReadoutData->outline = NULL;
	}
	XtRemoveEventHandler( area, PointerMotionMask, FALSE,
			      ( XtEventHandler) scatterReadoutDragHandler,
			      scatterReadoutData);
    
    }
}

/* if the layer or object is deleted, then remove the event handler */

static void removeScatterReadoutButtonHandler( 
	int		     	eventId,
        scatterReadoutDataPtr 	scatterReadoutData,
        AmScatterPlotClass 	*object)
{
    if ( object == scatterReadoutData->scatterPlot) {
	AmMotifDisplayClass   *motifDisplay = scatterReadoutData->motifDisplay;

	AmRemoveCallbackWidget( AM_PKG_ID, AM_DELETE_LAYER, 
			      ( callbackProc) removeScatterReadoutButtonHandler,
			      scatterReadoutData, motifDisplay->GetArea());
	AmRemoveCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, 
			      ( callbackProc) removeScatterReadoutButtonHandler,
			      scatterReadoutData, motifDisplay->GetArea());
	AmRemoveHandler( motifDisplay->GetArea(), 
		       ButtonPressMask | ButtonReleaseMask,
		       ( XtEventHandler) scatterReadoutButtonHandler, 
		       scatterReadoutData);
    }
}

/* launch a scatter plot readout handler which follows the mouse */

AnvilBoolean
AmScatterReadoutHandlerCallback( Widget		 btn,
			       AmMotifDisplayClass *motifDisplay,
			       AmScatterPlotClass	 *scatterPlot,
			       XtPointer)
{
    scatterReadoutDataPtr scatterReadoutData;

    scatterReadoutData = ( scatterReadoutDataPtr) 
				malloc( sizeof( scatterReadoutDataRec));
    scatterReadoutData->motifDisplay = motifDisplay;
    scatterReadoutData->scatterPlot = scatterPlot;
    scatterReadoutData->outline = NULL;

/* add the button press handler, which launches the drag handler */

    AmAddHandler( btn, motifDisplay->GetArea(), 
		ButtonPressMask | ButtonReleaseMask,
		( XtEventHandler) scatterReadoutButtonHandler, 
		scatterReadoutData, SCATTER_READOUT_PRIORITY);

/* add callbacks to kill the handler if the object is deleted, or removed as a
   layer from the overlay plot */

    AmAddCallbackWidget( AM_PKG_ID, AM_DELETE_LAYER, motifDisplay->GetArea(),
		       ( callbackProc) removeScatterReadoutButtonHandler,
		       scatterReadoutData);
    AmAddCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, motifDisplay->GetArea(),
		       ( callbackProc) removeScatterReadoutButtonHandler,
		       scatterReadoutData);

    return( TRUE);
}
