#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Separator.h>
#include <Xm/Frame.h>
#include <X11/IntrinsicP.h>
#include <anvil/overlay.h>
#include <anvil/errlib.h>
#include <anvil/motifdisplay.h>
#include <anvil/motif.h>
#include <anvil/plot_cback.h>
#include <anvil/va_cback.h>
#include <anvil/utility.h>
#include <anvil/macros.h>

static int		lastY;
static Widget		dragWidget = NULL;
static const int	AmSORT_OFFSET = 20;

/* compare the y position of two widgets */

static int byWidgetY( Widget	*a,
		      Widget	*b)
{
    Position	ay, by;

    XtVaGetValues( *a, XmNy, &ay, NULL);
    XtVaGetValues( *b, XmNy, &by, NULL);
    return( ay - by);
}

/* place the buttons in the appropriate positions */

static void placeButtonsNicely( Widget	bboard)
{
    Widget	*children;
    Cardinal	numChildren;
    uint_t	ii;
    Dimension	height;
    Position	y, oldY;

/* get the children from the bulletin board and the height */

    XtVaGetValues( bboard, 
		   XmNchildren, &children,
		   XmNnumChildren, &numChildren,
		   NULL);
    XtVaGetValues( children[ 0],
		   XmNheight, &height,
		   NULL);
    qsort( children, numChildren, sizeof( Widget), ( AmSearchFn) byWidgetY);
    for ( ii = 0; ii < numChildren; ii++) {
	XtVaGetValues( children[ ii],
		       XmNy, &oldY,
		       NULL);
    }

/* line them up nicely. */
  
    y = AmSORT_OFFSET;
    for ( ii = 0; ii < numChildren; ii++) {
	XtVaSetValues( children[ ii],
		       XmNy, y,
		       NULL);
	y += height;
    }
}

/* replace the buttons when the user releases the button drag */

static void buttonReleaseAction( Widget		btn,
				 XButtonEvent	*event,
				 String		*,
				 Cardinal	*)
{
    dragWidget = NULL;
    placeButtonsNicely( XtParent( btn));
}

/* drag the button */

static void buttonDragAction( Widget		button,
			      XMotionEvent	*event,
			      String		*,
			      Cardinal		*)
{
    Position 	currY, newY;
    Dimension 	parentHeight, buttonHeight;

    if ( button == dragWidget) {
	XtVaGetValues( button, 
		       XmNy, &currY,
		       XmNheight, &buttonHeight,
		       NULL);
	XtVaGetValues( XtParent( button),
		       XmNheight, &parentHeight,
		       NULL);

/* make sure the button isn't dragged outside of the parent */
	
	newY = currY + event->y - lastY;
	if (( newY >= 0) && ( newY + buttonHeight <= parentHeight)) {
	    XtVaSetValues( button,
			   XmNy, newY,
			   NULL);
	    currY = newY;
	}
    }
}

/* set the widget, drag start position and raise the widget when the user
   selects it */

static void buttonPressAction( Widget		btn,
			       XButtonEvent	*event,
			       String		*,
			       Cardinal		*)
{
    lastY = event->y;
    dragWidget = btn;
    XRaiseWindow( XtDisplay( btn), XtWindow( btn));
}


static void sortLayersOkCallback( Widget		btn,
				  Widget		vectorBboard,
				  Widget		rasterBboard,
				  AmMotifDisplayClass	*motifDisplay,
				  AmOverlayClass		*overlay,
				  XtPointer)
{
    Widget		*vectorBtns, *rasterBtns;
    int			numVectorBtns, numRasterBtns, numLayers, 
			*priorities, ii;
    AmDataGraphicClass	**newLayers, **layerPtr, **oldLayers;

AN_DEBUG( 10, ("calling sort layers ok callback\n"));
    XtVaGetValues( vectorBboard, 
		   XmNchildren, &vectorBtns,
		   XmNnumChildren, &numVectorBtns,
		   NULL);
    XtVaGetValues( rasterBboard, 
		   XmNchildren, &rasterBtns,
		   XmNnumChildren, &numRasterBtns,
		   NULL);

/* sort the objects by their vertical position */

    qsort( vectorBtns, numVectorBtns, sizeof( Widget), ( AmSearchFn) byWidgetY);
    qsort( rasterBtns, numRasterBtns, sizeof( Widget), ( AmSearchFn) byWidgetY);

/* copy them into the layer list.  Remember that the list is drawn back to
   front, so the AmFirst element is the AmFirst to be drawn */

    newLayers = ( AmDataGraphicClass **) 
	malloc( sizeof( AmDataGraphicClass *)*( numVectorBtns + numRasterBtns));
    if ( newLayers == NULL) {
	AmPrintResourceError( AmGetWidgetShell( btn), "mallocFailed");
	return;
    }
    layerPtr = newLayers;
    for ( ii = numRasterBtns - 1; ii >= 0; ii--) {
	XtVaGetValues( rasterBtns[ ii],
		       XmNuserData, layerPtr,
		       NULL);
	layerPtr++;
    }
    for ( ii = numVectorBtns - 1; ii >= 0; ii--) {
	XtVaGetValues( vectorBtns[ ii],
		       XmNuserData, layerPtr,
		       NULL);
	layerPtr++;
    }

/* get the old layers to find out what the priorities are */

    overlay->GetLayers( &oldLayers, &numLayers);
    priorities = ( int *) malloc( sizeof( int)*numLayers);
    if ( priorities == NULL) {
	AmPrintResourceError( AmGetWidgetShell( btn), "mallocFailed");
	return;
    }
    for ( ii = 0; ii < numLayers; ii++) {
	priorities[ ii] = oldLayers[ ii]->GetPriority();
    }
    for ( ii = 0; ii < numLayers; ii++) {
	newLayers[ ii]->SetVaFields( AmNpriority, &priorities[ ii], NULL);
    }
    overlay->SetLayers( newLayers, numVectorBtns + numRasterBtns);
    motifDisplay->Flush();
    free( oldLayers);
    free( priorities);

/* free the temporary storage and destroy the dialog */

    XtDestroyWidget( AmGetWidgetShell( btn));
    free( newLayers);
}

/* action area definitions for the sortlayers dialog */

static AmActionDefRec actionAreaDef[] = {
    { "okBtn", NULL, NULL },
    { "cancelBtn", ( XtCallbackProc) AmPopdownCback, NULL }
};

/* actions and translation table for dragging buttons */

static XtActionsRec actions[] = {
    { "StartDrag", ( XtActionProc) buttonPressAction },
    { "Drag", ( XtActionProc) buttonDragAction },
    { "EndDrag", ( XtActionProc) buttonReleaseAction }
};

static char translations[] = 
    "<Btn1Down>:	StartDrag()\n\
    <Motion>:		Drag()\n\
    <Btn1Up>:		EndDrag()\n";
	 
/* callback to sort layers from a list */

AnvilBoolean
AmSortLayersCallback( Widget              button,
                    AmMotifDisplayClass	*motifDisplay,
                    AmOverlayClass        *overlay,
                    XtPointer)
{
    AmActionRec		actionArea;
    Widget		formDlog, rasterBboard, vectorBboard, layerBtn,
			actionForm, sep, rasterFrame, vectorFrame, 
			rasterLbl, vectorLbl;
    Arg			args[ 20];
    char		*str;
    XmString		mstr;
    int			n, numLayers, ii, numRasterLayers, numVectorLayers;
    Position		rasterY, vectorY;
    Dimension		btnHeight, btnWidth, maxBtnWidth;
    XtTranslations	transTable;
    AmDataGraphicClass	**layers;

    if ( overlay->GetObjectType() != AmOVERLAY) {
    	AmAddResourceToLog( "sortLayersNotOverlay");
    	return( FALSE);
    }

    if ( overlay->GetLayers( &layers, &numLayers) != AmOK) {
    	AmAddResourceToLog( "failedToGetLayers");
	return( FALSE);
    }

/* add the dragging actions and parse the translation table */

    XtAppAddActions( XtWidgetToApplicationContext( button), 
		     actions, XtNumber( actions));
    transTable = XtParseTranslationTable( translations);
	    
    n = AmMakeArgList( args,
    	             XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
    	             XmNdeleteResponse, XmDESTROY, 
		     XmNautoUnmanage, FALSE,
    	             NULL);
    formDlog = XmCreateFormDialog( AmGetWidgetRealShell( button), "layerSort", 
    				   args, n);

/* create an action area for the OK and cancel buttons */

    actionAreaDef[ 1].clientData = formDlog;
    actionForm = AmMakeActionArea( formDlog,
				 actionAreaDef, VECLEN( actionAreaDef),
				 &actionArea);
    XtVaSetValues( actionForm, 
		   StickLeft, StickRight, StickBottom, 
		   NULL);

/* find out how many raster and vector plots are being displayed */

    numRasterLayers = 0;
    for ( ii = 0; ii < numLayers; ii++)  {
	if ( AmImageTypeLayer( layers[ ii])) {
	    numRasterLayers++;
	}
    }
    numVectorLayers = numLayers - numRasterLayers;

/* vector data on top */

    vectorFrame = XtVaCreateManagedWidget( "vectorFrame", 
					    xmFrameWidgetClass,
					    formDlog,
					    StickLeft, StickRight,
					    StickTop, 
					    NULL);
    vectorLbl = XtVaCreateManagedWidget( "vectorLbl", 
					 xmLabelWidgetClass,
					 vectorFrame,
					 XmNchildType, XmFRAME_TITLE_CHILD,
					 NULL);
    vectorBboard = XtVaCreateManagedWidget( "vectorBboard", 
					    xmDrawingAreaWidgetClass,
					    vectorFrame,
					    XmNwidth, 300,
					    XmNresizePolicy, XmRESIZE_NONE,
					    XmNchildType, 
						XmFRAME_WORKAREA_CHILD,
					    NULL);
    sep = XtVaCreateManagedWidget( "sep",
				   xmSeparatorWidgetClass,
				   formDlog,
				   StickLeft, StickRight,
				   BelowOf( vectorFrame),
				   NULL);

/* raster data below */

    rasterFrame = XtVaCreateManagedWidget( "rasterFrame", 
					    xmFrameWidgetClass,
					    formDlog,
					    StickLeft, StickRight,
					    BelowOf( sep),
					    XmNwidth, 300,
					    NULL);
    rasterLbl = XtVaCreateManagedWidget( "rasterLbl", 
					 xmLabelWidgetClass,
					 rasterFrame,
					 XmNchildType, XmFRAME_TITLE_CHILD,
					 NULL);
    rasterBboard = XtVaCreateManagedWidget( "rasterBboard", 
					    xmDrawingAreaWidgetClass,
					    rasterFrame,
					    XmNwidth, 300,
					    XmNresizePolicy, XmRESIZE_NONE,
					    NULL);
    vectorY = AmSORT_OFFSET;
    rasterY = AmSORT_OFFSET;
    maxBtnWidth = 0;

/* reverse order, because Y increases in screen coordinate system */

    for ( ii = numLayers - 1; ii >= 0; ii--) {
	str = AmGetLayerName( layers[ ii]);
    	mstr = XmStringCreateLtoR( str, XmFONTLIST_DEFAULT_TAG);

/* divide the layers AmBetween the raster and vector boards */

	if ( AmImageTypeLayer( layers[ ii])) {
	    layerBtn = XtVaCreateManagedWidget( "layerBtn", 
						xmPushButtonWidgetClass,
						rasterBboard, 
						XmNlabelString, mstr,
						XmNy, rasterY,
						XmNtraversalOn, FALSE,
						XmNuserData, layers[ ii],
						NULL);
	    XtVaGetValues( layerBtn, 
			   XmNheight, &btnHeight, 
			   XmNwidth, &btnWidth,
			   NULL);
	    rasterY += btnHeight;
	} else {
	    layerBtn = XtVaCreateManagedWidget( "layerBtn", 
						xmPushButtonWidgetClass,
						vectorBboard, 
						XmNlabelString, mstr,
						XmNy, vectorY,
						XmNtraversalOn, FALSE,
						XmNuserData, layers[ ii],
						NULL);
	    XtVaGetValues( layerBtn, 
			   XmNheight, &btnHeight, 
			   XmNwidth, &btnWidth,
			   NULL);
	    vectorY += btnHeight;
	}

/* track the maximum button width */

	if ( btnWidth > maxBtnWidth) {
	    maxBtnWidth = btnWidth;
	}

/* put in our translations to drag the widgets, and override the standard
   pushbutton translations */

	XtOverrideTranslations( layerBtn, transTable);
	XmStringFree( mstr);
	free( str);
    }
    free( layers);

/* resize the boards to contain their children, and reset the action area */

    XtVaSetValues( vectorBboard, 
		   AmNheight, vectorY + AmSORT_OFFSET, 
		   AmNwidth, maxBtnWidth + AmSORT_OFFSET*2,
		   NULL);
    XtVaSetValues( rasterBboard, 
		   AmNheight, rasterY + AmSORT_OFFSET, 
		   AmNwidth, maxBtnWidth + AmSORT_OFFSET*2,
		   NULL);
    XtVaSetValues( actionArea.form,
		   BelowOf( rasterBboard),
		   NULL);

/* add the callback to actually sort the layers, and bring up the dialog */

    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback,
		   ( VarargsCallback) sortLayersOkCallback,
		   vectorBboard, rasterBboard, motifDisplay, overlay,
		   END_LIST);
    XtManageChild( formDlog);

    return( TRUE);
}
