#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <math.h>
#include <Xm/Xm.h>
#include <anvil/tagstrs.h>
#include <anvil/va_cback.h>
#include <anvil/sphereplot.h>
#include <anvil/plot_cback.h>
#include <anvil/errlib.h>

static int startX, startY;

static void panSphereDragHandler( Widget		area,
				  AmMotifDisplayClass	*motifDisplay,
				  AmSpherePlotClass	*spherePlot,
				  XMotionEvent		*event,
				  AnvilBoolean		*continueDispatch)
{
    int		relX, relY;
    Dimension	width, height;
    float	theta, phi;

    XSync( XtDisplay( area), FALSE);
    relX = event->x - startX;
    relY = event->y - startY;
    startX = event->x;
    startY = event->y;
    XtVaGetValues( area,
		   XmNwidth, &width,
		   XmNheight, &height,
		   NULL);
    spherePlot->GetVaFields( AmNphi, &phi,
			     AmNtheta, &theta,
			     NULL);
    theta += ( M_PI*relX)/width;
    while ( theta < -M_PI) {
	theta += 2*M_PI;
    }
    while ( theta > M_PI) {
	theta -= 2*M_PI;
    }
    phi += ( M_PI*relY)/width;
    while ( phi < 0.0) {
	phi += 2*M_PI;
    }
    while ( phi > 2*M_PI) {
	phi -= 2*M_PI;
    }
    spherePlot->SetVaFields( AmNphi, phi,
			     AmNtheta, theta,
			     NULL);
AN_DEBUG( 99, ("dragging %.2f %.2f\n", theta, phi));
    motifDisplay->Flush();
}

/* stop dragging on release */

static void panSphereReleaseHandler( Widget		area,
				     AmMotifDisplayClass	*motifDisplay,
				     AmSpherePlotClass	*spherePlot,
				     XButtonEvent	*event,
				     AnvilBoolean		*continueDispatch)
{
    AmRemoveVaHandler( area, PointerMotionMask, FALSE,
		     ( VarargsHandler) panSphereDragHandler,
		     motifDisplay, spherePlot,
		     END_LIST);
    AmRemoveVaHandler( area, ButtonReleaseMask, FALSE,
		     ( VarargsHandler) panSphereReleaseHandler,
		     motifDisplay, spherePlot,
		     END_LIST);
}

/* start dragging on press */

static void panSpherePressHandler( Widget		area,
				   AmMotifDisplayClass	*motifDisplay,
				   AmSpherePlotClass	*spherePlot,
				   XButtonEvent		*event,
				   AnvilBoolean		*continueDispatch)
{
    startX = event->x;
    startY = event->y;
    AmAddVaHandler( area, PointerMotionMask, FALSE,
		  ( VarargsHandler) panSphereDragHandler,
		  motifDisplay, spherePlot,
		  END_LIST);
    AmAddVaHandler( area, ButtonReleaseMask, FALSE,
		  ( VarargsHandler) panSphereReleaseHandler,
		  motifDisplay, spherePlot,
		  END_LIST);
}
		     
/* add drag handlers to rotate the sphere */

AnvilBoolean
AmAddSphereRotationHandlers( Widget			area,
			   AmMotifDisplayClass	*motifDisplay,
			   AmSpherePlotClass		*spherePlot)
{
    if ( !spherePlot->IsType( AmSPHERE_PLOT)) {
	AmAddResourceToLog( "rotationHandlerNotSpherePlot");
	return( FALSE);
    }

    AmAddVaHandler( area, ButtonPressMask, FALSE,
		  ( VarargsHandler) panSpherePressHandler, 
		  motifDisplay, spherePlot,
		  END_LIST);

    return( TRUE);
}
