%{
#include <stdio.h>
#include <string.h>
#include <anvil/list.h>
#include <anvil/templateP.h>
#include <anvil/macros.h>

extern listPtr templObjectList;

/***************
 * Prototypes: *
 ***************/

int yylex( void);
int yyerror( char *fmt, ...);

/*********************
 * Type Definitions: *
 *********************/

%}

%union {
    char          *string;
    int           integer;
    listPtr       list;
    templDeclPtr  decl;
    templFieldPtr field;
}

%token <string>	IDENT
%token <string>	VALUE EMPTY_VALUE 

%type <decl>    Decl
%type <list>    FieldList
%type <field>   Field

%start DeclList

%%

DeclList	: Decl
		{
		    if ( !templAddDecl( $1)) {
		       yyerror( "Attempted redecleration of \"%s\".",
                           $1->name);
		    }
		}
		| DeclList Decl
		{
		    if ( !templAddDecl( $2)) {
		       yyerror( "Attempted redecleration of \"%s\".",
                           $2->name);
		    }
		}
		;

Decl		: IDENT IDENT '{' FieldList '}'
		{
		    objectTypeEnum type;

		    type = findKeyword( $1);
		    $$ = templCreateDecl( $2, type, NULL, $4);
		}
		| IDENT IDENT '(' IDENT ')' '{' FieldList '}'
		{
		    objectTypeEnum type;

		    type = findKeyword( $1);
		    if ( type == AmOVERLAY) {
                        yyerror( "Overlay object \"%s\" can not have a parent.",
		            $2);
		        $$ = templCreateDecl( $2, type, NULL, $7);
		    } else {
		        $$ = templCreateDecl( $2, type, $4, $7);
		    }
                }
		;

FieldList	: Field
		{
		    if ( $1 != NULL) {
		        $$ = insertNodeAtEndLst( $1, NULL);
                    } else {
		        $$ = NULL;
		    }
		}
		| FieldList Field
		{
		    if ( $2 != NULL) {
		        $$ = insertNodeAtEndLst( $2, $1);
		    } else {
		        $$ = $1;
		    }
		}
		;

Field		: IDENT ':' '{' Decl '}'
		{
		    $$ = templCreateChild( $1, $4);
		}
		| IDENT ':' VALUE 
		{
		    $$ = templCreateField( $1, $3);
		}
		| IDENT ':' 
		{
		    yyerror( "Ignoring empty field: \"%s\".", $1);
		    $$ = NULL;
		}
		;
%%

