#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdarg.h>
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <anvil/utility.h>
#include <anvil/geometry.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/userevents.h>
#include <anvil/plot.h>
#include <anvil/render3d.h>
#include <anvil/plot_cback.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>
#include <anvil/macros.h>

const int PANE_WIDTH = 250, PANE_HEIGHT = 250;

static AmActionDefRec actionAreaDef[] = {
    { "viewportBtn", NULL, NULL },
    { "closeBtn", NULL, NULL }
};

static AmActionDefRec controlAreaDef[] = {
    { "zoomInBtn", NULL, NULL },
    { "zoomOutBtn", NULL, NULL },
    { "closeBtn", NULL, NULL }
};

typedef struct renderControl_ {
    XColor	bgColor; 
    XColor	outlineColor; 
    XColor	viewportColor; 
    XColor	screenColor;
    XColor	pickColor;
    GC		gc;
    Widget	hArea;
    Pixmap	hPixmap;
    int		hPixmapWidth;
    int		hPixmapHeight;
    Widget	vArea;
    Pixmap	vPixmap;
    int		vPixmapWidth;
    int		vPixmapHeight;
    int		zoomLevel;
    float	ex;
    float	ey;
    float	ez;
    float	screenTopX;
    float	screenTopZ;
    float	screenBottomX;
    float	screenBottomZ;
    float	screenLeftX;
    float	screenLeftY;
    float	screenRightX;
    float	screenRightY;
    float	horizCenterX;
    float	horizCenterY;
    float	vertCenterX;
    float	vertCenterZ;
} renderControlRec, *renderControlPtr;

static void flushChildCallback( int			eventId,
				AmMotifDisplayClass	*motifDisplay,
				AmMotifDisplayClass	*sourceDisplay,
				AmMotifDisplayClass	*flushDisplay)
{
    if ( flushDisplay == sourceDisplay) {
	motifDisplay->Flush();
    }
}

static void FlushCallback( Widget		area,
			   AmMotifDisplayClass	*refDisplay,
			   XtPointer)
{
    XtRemoveCallback( area, XmNexposeCallback,
		      ( XtCallbackProc) FlushCallback, refDisplay);
    refDisplay->Flush();
}

void AmGetHorizRenderValues( Render3DClass	*render3D,
			   renderControlPtr	renderControl,
			   float		*toolXmin,
			   float		*toolYmin,
			   float		*toolXmax,
			   float		*toolYmax,
			   float		*hScale,
			   float		*vScale)
{
    float		ex, ey, screenWidth, theta, focal, zoomFactor, cx, cy,
			extents, xmin, ymin, xmax, ymax;
    AmOverlayClass	*overlay;

/* get the parameters from the 3D renderer */

    render3D->GetVaFields( AmNex, &ex,
			   AmNey, &ey,
			   AmNscreenWidth, &screenWidth,
			   AmNtheta, &theta,
			   AmNfocal, &focal,
			   NULL);

/* use the overlay's coordinates and the zoom factor to figure out the
   overall scale */

    zoomFactor = pow( 2.0, ( float) renderControl->zoomLevel);
    overlay = ( AmOverlayClass *) render3D->getParent();
    overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
    cx = ( xmax + xmin)/2.0;
    cy = ( ymax + ymin)/2.0;
    extents = MAX( xmax - xmin, ymax - ymin);
    *toolXmin = cx - zoomFactor*extents/2.0;
    *toolYmin = cy - zoomFactor*extents/2.0;
    *toolXmax = cx + zoomFactor*extents/2.0;
    *toolYmax = cy + zoomFactor*extents/2.0;
    *hScale = renderControl->hPixmapWidth/MAX(( *toolXmax - *toolXmin), 
					      ( *toolYmax - *toolYmin));
    *vScale = renderControl->hPixmapHeight/MAX(( *toolXmax - *toolXmin), 
					       ( *toolYmax - *toolYmin));
}

void AmGetVertRenderValues( Render3DClass		*render3D,
			  renderControlPtr	renderControl,
			  float			*toolXmin,
			  float			*toolZmin,
			  float			*toolXmax,
			  float			*toolZmax,
			  float			*hScale,
			  float			*vScale)
{
    float		ex, ez, screenHeight, phi, focal, zoomFactor,
			cx, cz, vz, xmin, ymin, xmax, ymax, extents,
			zmax, zmin;
    AmOverlayClass	*overlay;
    AmImagePlotClass	*elevationPlot;

/* get the parameters from the 3D renderer */

    render3D->GetVaFields( AmNex, &ex,
			   AmNez, &ez,
			   AmNscreenHeight, &screenHeight,
			   AmNphi, &phi,
			   AmNfocal, &focal,
			   AmNelevationPlot, &elevationPlot,
			   NULL);

/* use the overlay's coordinates and the zoom factor to figure out the
   overall scale */

    zoomFactor = pow( 2.0, ( float) renderControl->zoomLevel);
    overlay = ( AmOverlayClass *) render3D->getParent();
    overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
    elevationPlot->GetVaFields( AmNcolorsMin, &zmin,
				AmNcolorsMax, &zmax,
				NULL);
    vz = MAX( ez, zmax);
    extents = MAX( xmax - xmin, vz - zmin);
    cx = ( xmax + xmin)/2.0;
    cz = ( vz + zmin)/2.0;
    *toolXmin = cx - zoomFactor*extents/2.0;
    *toolZmin = cz - zoomFactor*extents/2.0;
    *toolXmax = cx + zoomFactor*extents/2.0;
    *toolZmax = cz + zoomFactor*extents/2.0;
    *hScale = renderControl->vPixmapWidth/MAX(( *toolXmax - *toolXmin), 
					      ( *toolZmax - *toolZmin));
    *vScale = renderControl->vPixmapHeight/MAX(( *toolXmax - *toolXmin), 
					       ( *toolZmax - *toolZmin));
}

static void 
drawHorizRender3D( Display		*display,
		   Drawable		drawable,
		   Render3DClass	*render3D,
		   renderControlPtr	renderControl)
{
    float		ex, ey, xmin, ymin, xmax, ymax, 
			toolXmin, toolYmin, toolXmax, toolYmax, screenWidth,
			theta, focal, hScale, vScale, 
			viewLeftX, viewLeftY, viewRightX, viewRightY,
			screenLeftX, screenLeftY, screenRightX, screenRightY,
			centerX, centerY;
    AmOverlayClass	*overlay;
    int			screenEx, screenEy, screenXmin, screenYmin, 
			screenXmax, screenYmax, screenViewLeftX, 
			screenViewLeftY, screenViewRightX, screenViewRightY,
			screenScreenLeftX, screenScreenLeftY, 
			screenScreenRightX, screenScreenRightY,
			screenCenterX, screenCenterY, textX, textY, x, y;
    char		buffer[ 256];

/* get the parameters from the 3D renderer */

    render3D->GetVaFields( AmNex, &ex,
			   AmNey, &ey,
			   AmNscreenWidth, &screenWidth,
			   AmNtheta, &theta,
			   AmNfocal, &focal,
			   NULL);
    AmGetHorizRenderValues( render3D, renderControl,
			  &toolXmin, &toolYmin, &toolXmax, &toolYmax,
			  &hScale, &vScale);

/* figure out the overlay's coordinates in the display */

    overlay = ( AmOverlayClass *) render3D->getParent();
    overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
    screenXmin = (int)(hScale*( xmin - toolXmin));
    screenYmin = (int)(renderControl->hPixmapHeight - vScale*( ymax - toolYmin));
    screenXmax = (int)(hScale*( xmax - toolXmin));
    screenYmax = (int)(renderControl->hPixmapHeight - vScale*( ymin - toolYmin));

/* figure out the eye's coordinates in the display */

    screenEx = (int)(hScale*( ex - toolXmin));
    screenEy = (int)(renderControl->hPixmapHeight - vScale*( ey - toolYmin));

/* figure the left and right viewport edges in the display */

    viewLeftX = (xmax - xmin)*2;
    viewLeftY = ( screenWidth/( 2*focal))*viewLeftX;
    AmRot2D( &viewLeftX, &viewLeftY, theta);
    AmTrans2D( &viewLeftX, &viewLeftY, ex, ey);
    screenViewLeftX = (int)(hScale*( viewLeftX - toolXmin));
    screenViewLeftY = (int)(renderControl->hPixmapHeight - 
			vScale*( viewLeftY - toolYmin));
    viewRightX = (xmax - xmin)*2;
    viewRightY = -( screenWidth/( 2*focal))*viewRightX;
    AmRot2D( &viewRightX, &viewRightY, theta);
    AmTrans2D( &viewRightX, &viewRightY, ex, ey);
    screenViewRightX = (int)(hScale*( viewRightX - toolXmin));
    screenViewRightY = (int)(renderControl->hPixmapHeight - 
			vScale*( viewRightY - toolYmin));

    screenLeftX = ( xmax - xmin)/2.0;
    screenLeftY = screenLeftX*screenWidth/( 2*focal);
    AmRot2D( &screenLeftX, &screenLeftY, theta);
    AmTrans2D( &screenLeftX, &screenLeftY, ex, ey);
    screenScreenLeftX = (int)(hScale*( screenLeftX - toolXmin));
    screenScreenLeftY = (int)(renderControl->hPixmapHeight - 
				vScale*( screenLeftY - toolYmin));

    screenRightX = ( xmax - xmin)/2.0;
    screenRightY = -screenRightX*screenWidth/( 2*focal);
    AmRot2D( &screenRightX, &screenRightY, theta);
    AmTrans2D( &screenRightX, &screenRightY, ex, ey);
    screenScreenRightX = (int)(hScale*( screenRightX - toolXmin));
    screenScreenRightY = (int)(renderControl->hPixmapHeight - 
				vScale*( screenRightY - toolYmin));
    XSetLineAttributes( display, renderControl->gc, 0, LineSolid, 
			CapRound, JoinBevel);

/* fill the background */

    XSetForeground( display, renderControl->gc, renderControl->bgColor.pixel);
    XFillRectangle( display, drawable, renderControl->gc, 
		    0, 0, renderControl->vPixmapWidth, 
		    renderControl->hPixmapHeight);

/* draw the overlay's coordinate outline */

    XSetForeground( display, renderControl->gc, 
		    renderControl->outlineColor.pixel);
    XDrawRectangle( display, drawable, renderControl->gc,
		    screenXmin, screenYmin,
		    screenXmax - screenXmin, screenYmax - screenYmin);

/* draw the viewport */

    XSetForeground( display, renderControl->gc, 
		    renderControl->viewportColor.pixel);
    XDrawLine( display, drawable, renderControl->gc,
	       screenEx, screenEy, screenViewLeftX, screenViewLeftY);
    XDrawLine( display, drawable, renderControl->gc,
	       screenEx, screenEy, screenViewRightX, screenViewRightY);

/* draw the pick ponts for the screen x,y */

    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    XDrawArc( display, drawable, renderControl->gc, 
	      screenEx - 3, screenEy - 3, 6, 6, 0, 360*64);

/* draw the screen */

    XSetForeground( display, renderControl->gc, renderControl->screenColor.pixel);
    XDrawLine( display, drawable, renderControl->gc,
	       screenScreenLeftX, screenScreenLeftY, 
	       screenScreenRightX, screenScreenRightY);
    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    x =(int)(( screenScreenLeftX + screenScreenRightX)/2.0);
    y = (int)(( screenScreenLeftY + screenScreenRightY)/2.0);

/* draw the screen pick point */

    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    XDrawArc( display, drawable, renderControl->gc,
	      x - 3, y - 3, 6, 6, 0, 360*64);

/* draw the centerline */

    XSetForeground( display, renderControl->gc, renderControl->screenColor.pixel);
    XSetLineAttributes( display, renderControl->gc, 0, LineOnOffDash, 
			CapRound, JoinBevel);
    centerX = ( xmax - xmin)*2;
    centerY = 0.0;
    AmRot2D( &centerX, &centerY, theta);
    AmTrans2D( &centerX, &centerY, ex, ey);
    screenCenterX = (int)(hScale*( centerX - toolXmin));
    screenCenterY = (int)(renderControl->vPixmapHeight - vScale*( centerY - toolYmin));
    XDrawLine( display, drawable, renderControl->gc,
	       screenEx, screenEy, screenCenterX, screenCenterY);

/* draw the centerline angle */

    textX = ( screenEx + screenCenterX)/2;
    textY = ( screenEy + screenCenterY)/2;
    sprintf( buffer, "%.0f\260", RADTODEG( theta));
    XDrawString( display, drawable, renderControl->gc, 
		 textX, textY, buffer, strlen( buffer));

/* draw the centerline pickPoint */

    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    XSetLineAttributes( display, renderControl->gc, 0, LineSolid, 
			CapRound, JoinBevel);
    XDrawArc( display, drawable, renderControl->gc,
	      screenCenterX - 3, screenCenterY - 3, 6, 6, 0, 360*64);
    renderControl->ex = ex;
    renderControl->ey = ey;
    renderControl->screenLeftX = screenLeftX;
    renderControl->screenLeftY = screenLeftY;
    renderControl->screenRightX = screenRightX;
    renderControl->screenRightY = screenRightY;
    renderControl->horizCenterX = centerX;
    renderControl->horizCenterY = centerY;
}

void 
AmDrawVertRender3D( Display		*display,
		  Drawable		drawable,
		  Render3DClass		*render3D,
		  renderControlPtr	renderControl)
{
    float		ex, ez, xmin, ymin, xmax, ymax, 
			toolXmin, toolZmin, toolXmax, toolZmax, screenHeight,
			phi, focal, hScale, vScale, 
			viewTopX, viewTopZ, viewBottomX, viewBottomZ,
			screenTopX, screenTopZ, screenBottomX, screenBottomZ,
			centerX, centerZ, zmin, zmax; 
    AmOverlayClass	*overlay;
    AmImagePlotClass	*elevationPlot;
    int			screenEx, screenEz, screenXmin, screenZmin, 
			screenXmax, screenZmax, screenViewTopX, 
			screenViewTopZ, screenViewBottomX, screenViewBottomZ,
			screenScreenTopX, screenScreenTopZ, 
			screenScreenBottomX, screenScreenBottomZ,
			screenCenterX, screenCenterZ, textX, textZ, x, y;
    char		buffer[ 256];

/* get the parameters from the 3D renderer */

    render3D->GetVaFields( AmNex, &ex,
			   AmNez, &ez,
			   AmNscreenHeight, &screenHeight,
			   AmNphi, &phi,
			   AmNfocal, &focal,
			   AmNelevationPlot, &elevationPlot,
			   NULL);

/* use the overlay's coordinates and the zoom factor to figure out the
   overall scale */

    AmGetVertRenderValues( render3D, renderControl,
			 &toolXmin, &toolZmin, &toolXmax, &toolZmax,
			 &hScale, &vScale);
    overlay = ( AmOverlayClass *) render3D->getParent();
    overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
    elevationPlot->GetVaFields( AmNcolorsMin, &zmin,
				AmNcolorsMax, &zmax,
				NULL);

/* figure out the overlay's coordinates in the display */

    screenXmin = (int)(hScale*( xmin - toolXmin));
    screenZmin = (int)(renderControl->vPixmapHeight - vScale*( zmax - toolZmin));
    screenXmax = (int)(hScale*( xmax - toolXmin));
    screenZmax = (int)(renderControl->vPixmapHeight - vScale*( zmin - toolZmin));

/* figure out the eye's coordinates in the display */

    screenEx = (int)(hScale*( ex - toolXmin));
    screenEz = (int)(renderControl->vPixmapHeight - vScale*( ez - toolZmin));

/* figure the left and right viewport edges in the display */

    viewTopX = ( xmax - xmin)*2;
    viewTopZ = ( screenHeight/( 2*focal))*viewTopX;
    AmRot2D( &viewTopX, &viewTopZ, phi);
    AmTrans2D( &viewTopX, &viewTopZ, ex, ez);
    screenViewTopX = (int)(hScale*( viewTopX - toolXmin));
    screenViewTopZ = (int)(renderControl->vPixmapHeight - 
			vScale*( viewTopZ - toolZmin));
    viewBottomX = (xmax - xmin)*2;
    viewBottomZ = -( screenHeight/( 2*focal))*viewBottomX;
    AmRot2D( &viewBottomX, &viewBottomZ, phi);
    AmTrans2D( &viewBottomX, &viewBottomZ, ex, ez);
    screenViewBottomX = (int)(hScale*( viewBottomX - toolXmin));
    screenViewBottomZ = (int)(renderControl->vPixmapHeight - 
				vScale*( viewBottomZ - toolZmin));

    screenTopX = ( xmax - xmin)/2.0;
    screenTopZ = screenTopX*screenHeight/( 2*focal);
    AmRot2D( &screenTopX, &screenTopZ, phi);
    AmTrans2D( &screenTopX, &screenTopZ, ex, ez);
    screenScreenTopX = (int)(hScale*( screenTopX - toolXmin));
    screenScreenTopZ = (int)(renderControl->vPixmapHeight - 
				vScale*( screenTopZ - toolZmin));

    screenBottomX = ( xmax - xmin)/2.0;
    screenBottomZ = -screenBottomX*screenHeight/( 2*focal);
    AmRot2D( &screenBottomX, &screenBottomZ, phi);
    AmTrans2D( &screenBottomX, &screenBottomZ, ex, ez);
    screenScreenBottomX = (int)(hScale*( screenBottomX - toolXmin));
    screenScreenBottomZ = (int)(renderControl->vPixmapHeight - 
				vScale*( screenBottomZ - toolZmin));

    XSetLineAttributes( display, renderControl->gc, 0, LineSolid, 
			CapRound, JoinBevel);

/* fill the background */

    XSetForeground( display, renderControl->gc, renderControl->bgColor.pixel);
    XFillRectangle( display, drawable, renderControl->gc, 
		    0, 0, renderControl->vPixmapWidth, 
		    renderControl->vPixmapHeight);

/* draw the overlay's coordinate outline */

    XSetForeground( display, renderControl->gc, 
		    renderControl->outlineColor.pixel);
    XDrawRectangle( display, drawable, renderControl->gc,
		    screenXmin, screenZmin,
		    screenXmax - screenXmin, screenZmax - screenZmin);

/* draw the viewport */

    XSetForeground( display, renderControl->gc, 
		    renderControl->viewportColor.pixel);
    XDrawLine( display, drawable, renderControl->gc,
	       screenEx, screenEz, screenViewTopX, screenViewTopZ);
    XDrawLine( display, drawable, renderControl->gc,
	       screenEx, screenEz, screenViewBottomX, screenViewBottomZ);

/* draw the pick ponts for the screen x,y */

    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    XDrawArc( display, drawable, renderControl->gc,
	      screenEx - 3, screenEz - 3, 6, 6, 0, 360*64);

/* draw the screen */

    XSetForeground( display, renderControl->gc, renderControl->screenColor.pixel);
    XDrawLine( display, drawable, renderControl->gc,
	       screenScreenTopX, screenScreenTopZ, 
	       screenScreenBottomX, screenScreenBottomZ);

/* draw the screen pick point */

    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    x = (int)(( screenScreenTopX + screenScreenBottomX)/2.0);
    y = (int)(( screenScreenTopZ + screenScreenBottomZ)/2.0);
    XDrawArc( display, drawable, renderControl->gc,
	      x - 3, y - 3, 6, 6, 0, 360*64);

/* draw the centerline */

    XSetForeground( display, renderControl->gc, renderControl->screenColor.pixel);
    XSetLineAttributes( display, renderControl->gc, 0, LineOnOffDash, 
			CapRound, JoinBevel);
    centerX = ( xmax - xmin)*2;
    centerZ = 0.0;
    AmRot2D( &centerX, &centerZ, phi);
    AmTrans2D( &centerX, &centerZ, ex, ez);
    screenCenterX = (int)(hScale*( centerX - toolXmin));
    screenCenterZ = (int)(renderControl->vPixmapHeight - vScale*( centerZ - toolZmin));
    XDrawLine( display, drawable, renderControl->gc,
	       screenEx, screenEz, screenCenterX, screenCenterZ);
    textX = ( screenEx + screenCenterX)/2;
    textZ = ( screenEz + screenCenterZ)/2;
    sprintf( buffer, "%.0f\260", RADTODEG( phi));
    XDrawString( display, drawable, renderControl->gc, 
		 textX, textZ, buffer, strlen( buffer));

/* draw the centerline pickPoint */

    XSetForeground( display, renderControl->gc, renderControl->pickColor.pixel);
    XSetLineAttributes( display, renderControl->gc, 0, LineSolid, 
			CapRound, JoinBevel);
    XDrawArc( display, drawable, renderControl->gc,
	      screenCenterX - 3, screenCenterZ - 3, 6, 6, 0, 360*64);
    renderControl->ex = ex;
    renderControl->ez = ez;
    renderControl->screenTopX = screenTopX;
    renderControl->screenTopZ = screenTopZ;
    renderControl->screenBottomX = screenBottomX;
    renderControl->screenBottomZ = screenBottomZ;
    renderControl->vertCenterX = centerX;
    renderControl->vertCenterZ = centerZ;
}

static void
horizDragHandler( Widget		area,
		  renderControlPtr	renderControl,
		  AmMotifDisplayClass	*motifDisplay,
		  Render3DClass		*render3D,
		  int			pointIndex,
		  XEvent		*event,
		  AnvilBoolean		*continueDispatch)
{
    float		toolXmin, toolYmin,
			toolXmax, toolYmax, hScale, vScale, dragX, dragY,
			xmin, ymin, xmax, ymax, focal, theta;
    AmOverlayClass	*overlay;
    Display		*display = XtDisplay( area);

    if ( event->type == ButtonRelease) {
	AmRemoveVaHandler( area, PointerMotionMask | ButtonReleaseMask, FALSE,
			 ( VarargsHandler) horizDragHandler,
			 renderControl, motifDisplay, render3D, pointIndex,
			 END_LIST);
	render3D->updateViewpoint();
	motifDisplay->Draw( NULL, 0);
    } else if ( event->type == MotionNotify) {
	XMotionEvent	*motionEvent = ( XMotionEvent *) event;

	AmGetHorizRenderValues( render3D, renderControl,
			      &toolXmin, &toolYmin, &toolXmax, &toolYmax,
			      &hScale, &vScale);
	overlay = ( AmOverlayClass *) render3D->getParent();
	overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
	dragX = toolXmin + motionEvent->x/hScale;
	dragY = toolYmin + ( renderControl->hPixmapHeight - 
			     motionEvent->y)/vScale;
	if ( pointIndex == 0) {

/* dragging eye point */

	    AN_DEBUG( 99, ( "dragX,Y = (%.2f, %.2f)\n", dragX, dragY));
	    renderControl->ex = dragX;
	    renderControl->ey = dragY;
	    if ( renderControl->ex < toolXmin) {
		renderControl->ex = toolXmin;
	    } 
	    if ( renderControl->ex > toolXmax) {
		renderControl->ex = toolXmax;
	    } 
	    if ( renderControl->ey < toolYmin) {
		renderControl->ey = toolYmin;
	    } 
	    if ( renderControl->ey > toolYmax) {
		renderControl->ey = toolYmax;
	    } 
	    render3D->SetVaFields( AmNex, renderControl->ex,
				   AmNey, renderControl->ey,
				   NULL);
	} else if ( pointIndex == 1) {

/* dragging focal point */
	 
	    focal = AmDist( dragX, dragY, renderControl->ex, renderControl->ey)/
		    (( xmax - xmin)/2.0);
	    if ( focal < 0.5) {
		focal = 0.5;
	    }
	    AN_DEBUG( 99, ( "dragX,Y = (%.2f, %.2f) focal = %.2f\n", 
			    dragX, dragY, focal));
	    render3D->SetVaFields( AmNfocal, focal,
				   NULL);
	} else if ( pointIndex == 2) {

/* rotating view angle */

	    theta = atan2( dragY - renderControl->ey,
			   dragX - renderControl->ex);
	    render3D->SetVaFields( AmNtheta, theta,
				   NULL);
	    AN_DEBUG( 99, ( "dragX,Y = (%.2f, %.2f) theta = %.2f\n", 
			    dragX, dragY, theta));
	}
	drawHorizRender3D( XtDisplay( area), renderControl->hPixmap,
			   render3D, renderControl);
	AmDrawVertRender3D( XtDisplay( area), renderControl->vPixmap,
			   render3D, renderControl);
	XCopyArea( display, renderControl->hPixmap, XtWindow( renderControl->hArea),
		   DefaultGC( display, DefaultScreen( display)),
		   0, 0, renderControl->hPixmapWidth, 
		   renderControl->hPixmapHeight, 
		   0, 0);
	XCopyArea( display, renderControl->vPixmap, XtWindow( renderControl->vArea),
		   DefaultGC( display, DefaultScreen( display)),
		   0, 0, renderControl->vPixmapWidth, 
		   renderControl->vPixmapHeight, 
		   0, 0);
    }
}

static void
horizPickHandler( Widget		area,
		  renderControlPtr	renderControl,
		  AmMotifDisplayClass	*motifDisplay,
		  Render3DClass		*render3D,
		  XButtonEvent		*event,
		  AnvilBoolean		*continueDispatch)
{
    float		zoomFactor, cx, cy, extents, toolXmin, toolYmin,
			toolXmax, toolYmax, hScale, vScale, pickX, pickY,
			xmin, ymin, xmax, ymax, dists[ 10],
			screenCenterX, screenCenterY, minDist;
    int			minIndex, ii;
    AmOverlayClass	*overlay;

    zoomFactor = pow( 2.0, ( float) renderControl->zoomLevel);
    overlay = ( AmOverlayClass *) render3D->getParent();
    overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
    cx = ( xmax + xmin)/2.0;
    cy = ( ymax + ymin)/2.0;
    extents = MAX( xmax - xmin, ymax - ymin);
    toolXmin = cx - zoomFactor*extents/2.0;
    toolYmin = cy - zoomFactor*extents/2.0;
    toolXmax = cx + zoomFactor*extents/2.0;
    toolYmax = cy + zoomFactor*extents/2.0;
    hScale = renderControl->hPixmapWidth/MAX(( toolXmax - toolXmin), 
					     ( toolYmax - toolYmin));
    vScale = renderControl->hPixmapHeight/MAX(( toolXmax - toolXmin), 
					      ( toolYmax - toolYmin));
    pickX = toolXmin + event->x/hScale;
    pickY = toolYmin + ( renderControl->hPixmapHeight - event->y)/vScale;
    AN_DEBUG( 99, ( "pickX,Y = (%.2f, %.2f)\n", pickX, pickY));
    dists[ 0] = AmDist( pickX, pickY, renderControl->ex, renderControl->ey);
    screenCenterX = ( renderControl->screenLeftX + 
		      renderControl->screenRightX)/2.0;
    screenCenterY = ( renderControl->screenLeftY + 
		      renderControl->screenRightY)/2.0;
    dists[ 1] = AmDist( pickX, pickY, screenCenterX, screenCenterY);
    dists[ 2] = AmDist( pickX, pickY, renderControl->horizCenterX, 
		      renderControl->horizCenterY);
    minDist = dists[ 0];
    minIndex = 0;
    for ( ii = 1; ii < 3; ii++) {
	if ( dists[ ii] < minDist) {
	    minDist = dists[ ii];
	    minIndex = ii;
	}
    }
    if ( minIndex == 0) {
	AN_DEBUG( 99, ( "eyepoint\n"));
    } else if ( minIndex == 1) {
	AN_DEBUG( 99, ( "screen center\n"));
    } else if ( minIndex == 2) {
	AN_DEBUG( 99, ( "horiz center\n"));
    }
    AmAddVaHandler( area, PointerMotionMask | ButtonReleaseMask, FALSE,
		  ( VarargsHandler) horizDragHandler,
		  renderControl, motifDisplay, render3D, minIndex,
		  END_LIST);
}

static void
vertDragHandler( Widget			area,
		 renderControlPtr	renderControl,
		 AmMotifDisplayClass	*motifDisplay,
		 Render3DClass		*render3D,
		 int			pointIndex,
		 XEvent			*event,
		 AnvilBoolean		*continueDispatch)
{
    if ( event->type == ButtonRelease) {
	AmRemoveVaHandler( area, PointerMotionMask | ButtonReleaseMask, FALSE,
			 ( VarargsHandler) vertDragHandler,
			 renderControl, motifDisplay, render3D, pointIndex,
			 END_LIST);
	render3D->updateViewpoint();
	motifDisplay->Draw( NULL, 0);
    } else if ( event->type == MotionNotify) {
	XMotionEvent	*motionEvent = ( XMotionEvent *) event;
	AmOverlayClass	*overlay;
	Display		*display = XtDisplay( area);
	float		dragX, dragZ, toolXmin, toolZmin, toolXmax, toolZmax,
			hScale, vScale, phi, xmin, ymin, xmax, ymax, focal;

	AmGetVertRenderValues( render3D, renderControl,
			      &toolXmin, &toolZmin, &toolXmax, &toolZmax,
			      &hScale, &vScale);
	overlay = ( AmOverlayClass *) render3D->getParent();
	overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
	dragX = toolXmin + motionEvent->x/hScale;
	dragZ = toolZmin + ( renderControl->vPixmapHeight - 
			     motionEvent->y)/vScale;
	if ( pointIndex == 0) {

/* dragging eye point */

	    AN_DEBUG( 99, ( "dragX,Z = (%.2f, %.2f)\n", dragX, dragZ));
	    renderControl->ex = dragX;
	    renderControl->ez = dragZ;
	    if ( renderControl->ex < toolXmin) {
		renderControl->ex = toolXmin;
	    } 
	    if ( renderControl->ex > toolXmax) {
		renderControl->ex = toolXmax;
	    } 
	    if ( renderControl->ez < toolZmin) {
		renderControl->ey = toolZmin;
	    } 
	    if ( renderControl->ez > toolZmax) {
		renderControl->ez = toolZmax;
	    } 
	    render3D->SetVaFields( AmNex, renderControl->ex,
				   AmNez, renderControl->ez,
				   NULL);
	} else if ( pointIndex == 1) {

/* dragging focal point */
	 
	    focal = AmDist( dragX, dragZ, renderControl->ex, renderControl->ez)/
		    (( xmax - xmin)/2.0);
	    if ( focal < 0.5) {
		focal = 0.5;
	    }
	    AN_DEBUG( 99, ( "dragX,Z = (%.2f, %.2f) focal = %.2f\n", 
			    dragX, dragZ, focal));
	    render3D->SetVaFields( AmNfocal, focal,
				   NULL);
	} else if ( pointIndex == 2) {

/* rotating view angle */

	    phi = atan2( dragZ - renderControl->ez,
			 dragX - renderControl->ex);
	    render3D->SetVaFields( AmNphi, phi,
				   NULL);
	    AN_DEBUG( 99, ( "dragX,Z = (%.2f, %.2f) theta = %.2f\n", 
			    dragX, dragZ, phi));
	}
	AmDrawVertRender3D( XtDisplay( area), renderControl->vPixmap,
			  render3D, renderControl);
	drawHorizRender3D( XtDisplay( area), renderControl->hPixmap,
			   render3D, renderControl);
	XCopyArea( display, renderControl->hPixmap, XtWindow( renderControl->hArea),
		   DefaultGC( display, DefaultScreen( display)),
		   0, 0, renderControl->hPixmapWidth, 
		   renderControl->hPixmapHeight, 
		   0, 0);
	XCopyArea( display, renderControl->vPixmap, XtWindow( renderControl->vArea),
		   DefaultGC( display, DefaultScreen( display)),
		   0, 0, renderControl->vPixmapWidth, 
		   renderControl->vPixmapHeight, 
		   0, 0);
    }
}

static void
vertPickHandler( Widget			area,
		 renderControlPtr	renderControl,
		 AmMotifDisplayClass	*motifDisplay,
		 Render3DClass		*render3D,
		 XButtonEvent		*event,
		 AnvilBoolean		*continueDispatch)
{
    float		toolXmin, toolZmin,
			toolXmax, toolZmax, hScale, vScale, pickX, pickZ,
			xmin, ymin, xmax, ymax, zmin, zmax, dists[ 10],
			screenCenterX, screenCenterZ, minDist;
    int			minIndex, ii;
    AmOverlayClass	*overlay;
    AmImagePlotClass	*elevationPlot;

    overlay = ( AmOverlayClass *) render3D->getParent();
    overlay->GetCoords( &xmin, &ymin, &xmax, &ymax);
    render3D->GetVaFields( AmNelevationPlot, &elevationPlot,
			   NULL);
    elevationPlot->GetVaFields( AmNcolorsMin, &zmin, 
				AmNcolorsMax, &zmax,
				NULL);
    AmGetVertRenderValues( render3D, renderControl,
			 &toolXmin, &toolZmin, &toolXmax, &toolZmax,
			 &hScale, &vScale);
    pickX = toolXmin + event->x/hScale;
    pickZ = toolZmin + ( renderControl->vPixmapHeight - event->y)/vScale;
    AN_DEBUG( 99, ( "pickX,Z = (%.2f, %.2f)\n", pickX, pickZ));
    dists[ 0] = AmDist( pickX, pickZ, renderControl->ex, renderControl->ez);
    screenCenterX = ( renderControl->screenTopX + 
		      renderControl->screenBottomX)/2.0;
    screenCenterZ = ( renderControl->screenTopZ + 
		      renderControl->screenBottomZ)/2.0;
    dists[ 1] = AmDist( pickX, pickZ, screenCenterX, screenCenterZ);
    dists[ 2] = AmDist( pickX, pickZ, renderControl->vertCenterX, 
		      renderControl->vertCenterZ);
    minDist = dists[ 0];
    minIndex = 0;
    for ( ii = 1; ii < 3; ii++) {
	if ( dists[ ii] < minDist) {
	    minDist = dists[ ii];
	    minIndex = ii;
	}
    }
    if ( minIndex == 0) {
	AN_DEBUG( 99, ( "eyepoint\n"));
    } else if ( minIndex == 1) {
	AN_DEBUG( 99, ( "screen center\n"));
    } else if ( minIndex == 2) {
	AN_DEBUG( 99, ( "vert center\n"));
    }
    AmAddVaHandler( area, PointerMotionMask | ButtonReleaseMask, FALSE,
		  ( VarargsHandler) vertDragHandler,
		  renderControl, motifDisplay, render3D, minIndex,
		  END_LIST);
}

static void
drawPixmapCallback( Widget	area,
		    Pixmap	pixmap,
		    int		*pixmapWidth,
		    int		*pixmapHeight,
		    XtPointer)
{
    Display	*display = XtDisplay( area);

    XCopyArea( XtDisplay( area), pixmap, XtWindow( area),
	       DefaultGC( display, DefaultScreen( display)),
	       0, 0, *pixmapWidth, *pixmapHeight, 0, 0);
}

static void 
zoomIn3DCallback( Widget		scale,
		  Render3DClass		*render3D,
		  renderControlPtr	renderControl,
		  Widget		hArea,
		  Widget		vArea,
		  XtPointer)
{
    Display 	*display = XtDisplay( hArea);

    renderControl->zoomLevel++;
    AmDrawVertRender3D( display, renderControl->vPixmap,
		      render3D, renderControl);
    XCopyArea( display, renderControl->hPixmap, XtWindow( hArea),
	       DefaultGC( display, DefaultScreen( display)),
	       0, 0, renderControl->hPixmapWidth, 
	       renderControl->hPixmapHeight, 0, 0);
    drawHorizRender3D( display, renderControl->hPixmap,
		       render3D, renderControl);
    XCopyArea( display, renderControl->vPixmap, XtWindow( vArea),
	       DefaultGC( display, DefaultScreen( display)),
	       0, 0, renderControl->vPixmapWidth, 
	       renderControl->vPixmapHeight, 0, 0);
}

static void 
zoomOut3DCallback( Widget		scale,
		   Render3DClass	*render3D,
		   renderControlPtr	renderControl,
		   Widget		hArea,
		   Widget		vArea,
		   XtPointer)
{
    Display 	*display = XtDisplay( hArea);

    renderControl->zoomLevel++;
    AmDrawVertRender3D( display, renderControl->vPixmap,
		      render3D, renderControl);
    XCopyArea( display, renderControl->hPixmap, XtWindow( hArea),
	       DefaultGC( display, DefaultScreen( display)),
	       0, 0, renderControl->hPixmapWidth, 
	       renderControl->hPixmapHeight, 
	       0, 0);
    drawHorizRender3D( display, renderControl->hPixmap,
		       render3D, renderControl);
    XCopyArea( display, renderControl->vPixmap, XtWindow( vArea),
	       DefaultGC( display, DefaultScreen( display)),
	       0, 0, renderControl->vPixmapWidth, 
	       renderControl->vPixmapHeight, 
	       0, 0);
}

void AmSetDetailLevelCallback( Widget		detailScale,
			     renderControlPtr	renderControl,
			     AmMotifDisplayClass	*motifDisplay,
			     Render3DClass	*render3D,
			     XtPointer)
{
    int detailLevel;

    XmScaleGetValue( detailScale, &detailLevel);
    render3D->SetVaFields( AmNtexWidth, detailLevel,
			   AmNtexHeight, detailLevel,
			   NULL);
}

void AmSetVerticalExaggerationCallback( Widget		verticalScale,
				      renderControlPtr	renderControl,
				      AmMotifDisplayClass	*motifDisplay,
				      Render3DClass	*render3D,
				      XtPointer)
{
    int verticalExaggeration;

    XmScaleGetValue( verticalScale, &verticalExaggeration);
    render3D->SetVaFields( AmNverticalExaggeration, 
				( float) verticalExaggeration,
			   NULL);
}

static void 
control3DDialogCallback( Widget			btn,
			 AmMotifDisplayClass	*motifDisplay,
			 Render3DClass		*render3D,
			 XtPointer)
{
    Widget		formDlog, rowcol, horizArea, vertArea, controlForm, 
			parent, verticalScale, detailScale;
    renderControlPtr	renderControl;
    Display		*display;
    Colormap		cmap;
    XColor		exactDef;
    AmActionRec   	controlArea;

    renderControl = ( renderControlPtr) malloc( sizeof( renderControlRec));
    renderControl->zoomLevel = 2;
    parent = AmGetWidgetShell( btn);
    display = XtDisplay( parent);
    XtVaGetValues( parent,
		   XmNcolormap, &cmap,
		   NULL);
    XAllocNamedColor( display, cmap, 
		      AmGetDefault( "render3Dcontrol.bg", "black"),
		      &renderControl->bgColor, &exactDef);
    XAllocNamedColor( display, cmap, 
		      AmGetDefault( "render3Dcontrol.outline", "Green"),
		      &renderControl->outlineColor, &exactDef);
    XAllocNamedColor( display, cmap, 
		      AmGetDefault( "render3Dcontrol.viewport", "Red"),
		      &renderControl->viewportColor, &exactDef);
    XAllocNamedColor( display, cmap, 
		      AmGetDefault( "render3Dcontrol.screen", "Yellow"),
		      &renderControl->screenColor, &exactDef);
    XAllocNamedColor( display, cmap, 
		      AmGetDefault( "render3Dcontrol.pick", "Blue"),
		      &renderControl->pickColor, &exactDef);
    renderControl->gc = XCreateGC( display, DefaultRootWindow( display), 0, NULL);

    renderControl->hPixmap = 
	XCreatePixmap( display, XtWindow( parent),
		       PANE_WIDTH, PANE_HEIGHT,
		       DefaultDepth( display, DefaultScreen( display)));

    renderControl->hPixmapWidth = PANE_WIDTH;
    renderControl->hPixmapHeight = PANE_HEIGHT;

    renderControl->vPixmap = 
	XCreatePixmap( display, XtWindow( parent),
		       PANE_WIDTH, PANE_HEIGHT,
		       DefaultDepth( display, DefaultScreen( display)));
    renderControl->vPixmapWidth = PANE_WIDTH;
    renderControl->vPixmapHeight = PANE_HEIGHT;

    formDlog = XmCreateFormDialog( parent, "control3D", NULL, 0);
    verticalScale = XtVaCreateManagedWidget( "vScale", xmScaleWidgetClass, 
					     formDlog,
					     StickTop, StickLeft, 
					     XmNorientation, XmVERTICAL,
					     XmNshowValue, TRUE,
					     XmNminimum, 1,
					     XmNmaximum, 10,
					     NULL);
    detailScale = XtVaCreateManagedWidget( "detailScale", xmScaleWidgetClass, 
					   formDlog,
					   StickTop, StickLeft, 
					   XmNorientation, XmVERTICAL,
					   XmNshowValue, TRUE,
					   XmNminimum, 256,
					   XmNmaximum, 1024,
					   BelowOf( verticalScale),
					   NULL);
    rowcol = XtVaCreateManagedWidget( "rowcol", xmRowColumnWidgetClass, formDlog, 
				      XmNorientation, XmVERTICAL,
				      RightOf( verticalScale),
				      StickBottom,
				      XmNbottomOffset, 60,
				      NULL);
    horizArea = XtVaCreateManagedWidget( "horizArea", xmDrawingAreaWidgetClass, 
					 rowcol,
					 XmNwidth, PANE_WIDTH,
					 XmNheight, PANE_HEIGHT,
					 NULL);
    vertArea = XtVaCreateManagedWidget( "vertArea", xmDrawingAreaWidgetClass, 
					rowcol,
					XmNwidth, PANE_WIDTH,
					XmNheight, PANE_HEIGHT,
					NULL);
    controlForm = AmMakeActionArea( formDlog,
                                  controlAreaDef, ELEMENTS( controlAreaDef),
                                  &controlArea);
    renderControl->hArea = horizArea;
    renderControl->vArea = vertArea;

/* draw the current situation into the pixmaps */

    drawHorizRender3D( XtDisplay( horizArea), renderControl->hPixmap,
		       render3D, renderControl);
    AmDrawVertRender3D( XtDisplay( vertArea), renderControl->vPixmap,
		       render3D, renderControl);
    AmAddVaHandler( horizArea, ButtonPressMask, FALSE, 
		  ( VarargsHandler) horizPickHandler, 
		  renderControl, motifDisplay, render3D,
		  END_LIST);
    AmAddVaHandler( vertArea, ButtonPressMask, FALSE, 
		  ( VarargsHandler) vertPickHandler, 
		  renderControl, motifDisplay, render3D,
		  END_LIST);
    AmAddVaCallback( detailScale, XmNvalueChangedCallback,
		  ( VarargsCallback) AmSetDetailLevelCallback,
		  renderControl, motifDisplay, render3D,
		  END_LIST);
    AmAddVaCallback( verticalScale, XmNvalueChangedCallback,
		  ( VarargsCallback) AmSetVerticalExaggerationCallback,
		  renderControl, motifDisplay, render3D,
		  END_LIST);

    XtVaSetValues( controlForm, 
		   StickLeft, StickRight, StickBottom, NULL);
    AmAddVaCallback( horizArea, XmNexposeCallback,
		   ( VarargsCallback) drawPixmapCallback, 
		   renderControl->hPixmap, 
		   &renderControl->hPixmapWidth,
		   &renderControl->hPixmapHeight,
		   END_LIST);
    AmAddVaCallback( vertArea, XmNexposeCallback,
		   ( VarargsCallback) drawPixmapCallback, 
		   renderControl->vPixmap, 
		   &renderControl->vPixmapWidth,
		   &renderControl->vPixmapHeight,
		   END_LIST);
    AmAddVaCallback( controlArea.buttons[ 0], XmNactivateCallback,
		   ( VarargsCallback) zoomIn3DCallback,
		   render3D, renderControl, horizArea, vertArea,
		   END_LIST);
    AmAddVaCallback( controlArea.buttons[ 1], XmNactivateCallback,
		   ( VarargsCallback) zoomOut3DCallback,
		   render3D, renderControl, horizArea, vertArea,
		   END_LIST);
    XtAddCallback( controlArea.buttons[ 2], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
    XtManageChild( formDlog);
}


/* callback to create a popup dialog for a reference window */

AnvilBoolean AmRender3DCallback( Widget    		btn,
			  AmMotifDisplayClass 	*motifDisplay,
			  AmOverlayClass		*overlay,
			  AmImagePlotClass	*elevationPlot,
			  XtPointer)
{
    Widget      		formDlog, area, actionForm, 
    				parent = motifDisplay->GetShell();
    AmXDisplayClass		*pXDisplay;
    AmXWindowClass		*pXWindow;
    AmActionRec   		actionArea;
    AmMotifDisplayClass		*refDisplay;
    Render3DClass		*render3D;
    Arg				args[ 20];
    int				n;
    float			currXmin, currYmin, currXmax, currYmax,
				zmin, zmax, ex, ey, ez, focal, phi, theta,
				screenWidth, screenHeight;

/* create a form to control the min and max for the gradient */

    n = AmMakeArgList( args,
                     XmNautoUnmanage, FALSE,
		     XmNdeleteResponse, XmDESTROY,
                     XmNcolormap, AmGetAppColormap(),
                     XmNvisual, AmGetAppVisual(),
                     XmNdepth, AmGetAppDepth(),
                     NULL);
    formDlog = XmCreateFormDialog( parent, "render3DWindow", args, n);

/* make it so the user can only pop up one reference window */

    AmSetSensitiveDestroy( btn, formDlog);
    actionForm = AmMakeActionArea( formDlog,
                                 actionAreaDef, ELEMENTS( actionAreaDef),
                                 &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);
    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, formDlog,
				    StickTop, StickRight, StickLeft,
				    StickBottom,
				    XmNbottomOffset, 50,
				    XmNwidth, 300,
				    XmNheight, 300,
				    NULL);
    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    refDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);

    overlay->GetCoords( &currXmin, &currYmin, &currXmax, &currYmax);
    overlay->GetVaFields( AmNcolorsMin, &zmin,
			  AmNcolorsMax, &zmax,
			  NULL);
    ex = currXmin - ( currXmax - currXmin)/2.0;
    ey = ( currYmax + currYmin)/2.0;
#if 0
    ez = zmax * 2;
#endif
    ez = 10000.0;
    focal = 1.0;
    phi = -0.1;
    theta = 0.0;
    screenWidth = 1.0;
    screenHeight = 1.0;

    render3D = new Render3DClass( motifDisplay->GetWindow(),
			          overlay, "render3d",
				  AmNelevationPlot, elevationPlot,
				  AmNex, ex,
				  AmNey, ey,
				  AmNez, ez,
				  AmNfocal, focal,
				  AmNphi, phi,
				  AmNtheta, theta,
				  AmNscreenWidth, screenWidth,
				  AmNscreenHeight, screenHeight,
				  AmNverticalExaggeration, 5.0,
			          AmNleftMargin, 0,
			          AmNrightMargin, 0,
			          AmNtopMargin, 0,
			          AmNbottomMargin, 0,
			          NULL);
    refDisplay->AddObject( render3D);

/* set up handlers to receive events from the source display */

    AmAddVaCallback( actionArea.buttons[ 0], XmNactivateCallback,
		   ( VarargsCallback) control3DDialogCallback,
		   refDisplay, render3D,
		   END_LIST);
    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
    XtAddCallback( area, XmNexposeCallback,
		   ( XtCallbackProc) FlushCallback, refDisplay);
    XtManageChild( formDlog);

    return( TRUE);
}

/* update the reference window when the source plot changes */

static void updateRender3DHandler( int			eventId,
				   Render3DClass	*render3D,
				   AmMotifDisplayClass	*motifDisplay,
				   AmScreenObjClass	*source)
{
    if ( source == render3D->getParent()) {

/* "transfer" the event to the reference window on this motif display */

AN_DEBUG( 40, ("sending reference window event %d\n", eventId));
	AmSendUserEvent( AM_PKG_ID, eventId, render3D);
	motifDisplay->Flush();
    }
}

/* remove the pan handler when the reference window is destroyed */

static void destroyRender3DHandler( int		   	eventId,
				    Render3DClass  	*render3D,
				    AmMotifDisplayClass	*sourceDisplay,     
				    AmMotifDisplayClass	*motifDisplay,     
				    AmScreenObjClass	*source)

{
    if (( source == render3D->getParent()) || ( source == render3D)) {
AN_DEBUG( 40, ( "calling destroyRender3DHandler\n"));

/* if the parent display is flushed, then flush the child display as well */

	AmRemoveVaCallbackWidget( AM_PKG_ID, AM_FLUSH, sourceDisplay->GetArea(),
			        ( VarargsCallback) flushChildCallback, 
			        motifDisplay, sourceDisplay,
			        END_LIST);
	AmRemoveVaCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, 
				 sourceDisplay->GetArea(),
			        ( VarargsCallback) destroyRender3DHandler, 
			         render3D, sourceDisplay, motifDisplay,
				 END_LIST);
	if ( motifDisplay != sourceDisplay) {
	    AmRemoveVaCallbackWidget( AM_PKG_ID, AM_SET_FIELD_OBJECT, 
				    sourceDisplay->GetArea(),
				    ( VarargsCallback) updateRender3DHandler, 
				    render3D, motifDisplay,
				    END_LIST);
	    AmRemoveVaCallbackWidget( AM_PKG_ID, AM_COORDS_CHANGE, 
				    sourceDisplay->GetArea(),
				    ( VarargsCallback) updateRender3DHandler, 
				    render3D, motifDisplay,
				    END_LIST);
	}
    }
}
				     

AnvilBoolean
AmSetRender3DHandler( AmMotifDisplayClass	*motifDisplay,
		    AmMotifDisplayClass	*sourceDisplay,
		    Render3DClass	*render3D)
{

/* if the parent display is flushed, then flush the child display as well */

    AmAddVaCallbackWidget( AM_PKG_ID, AM_FLUSH, sourceDisplay->GetArea(),
		         ( VarargsCallback) flushChildCallback, 
			 motifDisplay, sourceDisplay,
			 END_LIST);

/* if the parent is deleted, then blow away the reference handler */

    AmAddVaCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, sourceDisplay->GetArea(),
		         ( VarargsCallback) destroyRender3DHandler, 
			 render3D, sourceDisplay, motifDisplay, 
			 END_LIST);

/* if the source and reference display managers aren't the same, then
   add callbacks to update the reference window whenever the parent is
   changed or zoomed */

    if ( sourceDisplay != motifDisplay) {
	AmAddVaCallbackWidget( AM_PKG_ID, AM_SET_FIELD_OBJECT, 
			     sourceDisplay->GetArea(),
			     ( VarargsCallback) updateRender3DHandler, 
			     render3D, motifDisplay, 
			     END_LIST);
	AmAddVaCallbackWidget( AM_PKG_ID, AM_COORDS_CHANGE, 
			     sourceDisplay->GetArea(),
			     ( VarargsCallback) updateRender3DHandler, 
			     render3D, motifDisplay, 
			     END_LIST);
    }
    return( TRUE);
}
