#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Scale.h>
#include <Xm/PushB.h>
#include <stdarg.h>
#include <stdio.h>
#include <anvil/plot.h>
#include <anvil/tagstrs.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/plot_cback.h>
#include <anvil/copyobj.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/motifdisplay.h>
#include <anvil/magic.h>

class TransparentMagicClass : public AmMagicPlotClass {
private:
    float		factor;		/* transparency factor */
    Widget		dialog;		/* control dialog */
    AmImagePlotClass	*srcImagePlot;	/* source image to transparent */
    AmImagePlotClass	*newImagePlot;	/* source image to transparent */
public: 
    TransparentMagicClass( AmMotifDisplayClass     *motifDisplay,
			   AmOverlayClass		 *srcOverlay,
			   AmImagePlotClass	 *imagePlot,
			   AnvilBoolean               callOnDrag);
    void                       SetTransparent( float value);
    Widget                     GetDialog() { return( dialog); };
    void                       SetDialog( Widget dialogArg) 
				    { dialog = dialogArg; };
    AmImagePlotClass	       *GetSourceImagePlot() 
				    { return( srcImagePlot); };
    virtual AmDataGraphicClass   *Init( AmDataGraphicClass	*sourcePlot);
    virtual		       ~TransparentMagicClass();
};

void TransparentMagicClass::SetTransparent( float	factor)
{
    AN_DEBUG( 30, ( "setting transparency to %.2f\n", factor));
    newImagePlot->SetVaFields( AmNtransparency, factor, NULL);
}

/* set the transparency scale */

static void setTransparencyScale( Widget		scale,
				  TransparentMagicClass	*transparentMagic,
				  XtPointer		callData)
{
    int	value;

    XmScaleGetValue( scale, &value);
    transparentMagic->SetTransparent( value/100.0);
    transparentMagic->GetPlot()->Update();
    transparentMagic->DrawWrapper( transparentMagic->GetRect());
}

/* turn off the transparencying glass window when the dialog is destroyed */

static void destroyTransparencyCallback( 
			     Widget,	
			     TransparentMagicClass	*transparentMagic,
			     XtPointer)
{
    transparentMagic->SetDialog( NULL);
    delete transparentMagic;
}

/* the standard magic window destroy callbacks get called when the source plot
   is destroyed.  We need to have a second one which is called when the
   transparency layer is destroyed */

static void destroyTransparencyMagicCallback( int,
					      TransparentMagicClass	*magic,
					      AmScreenObjClass		*obj)
{
    if ( obj == ( AmScreenObjClass *) magic->GetSourceImagePlot()) {
	delete magic;
    }
}

/* transparent cursor destructor */

TransparentMagicClass::~TransparentMagicClass()
{
    AmMotifDisplayClass	*motifDisplay = ( AmMotifDisplayClass *) magicManager;

    AmRemoveCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, 
		          ( callbackProc) destroyTransparencyMagicCallback, 
			  this, motifDisplay->GetArea());
    if ( dialog != NULL) {

/* remove the dialog's destroy callback before destroying the widget, or
   else we'll get deleted twice */

	XtRemoveCallback( dialog, XmNdestroyCallback,
		          ( XtCallbackProc) destroyTransparencyCallback, this);
	XtDestroyWidget( dialog);
    }
}

/* initialization function for transparency magic. */

AmDataGraphicClass *TransparentMagicClass::Init( AmDataGraphicClass	*sourcePlot)
{
     AmOverlayClass	*newOverlay, *srcOverlay = ( AmOverlayClass *) sourcePlot;
     AmDataGraphicClass	**srcLayers, **newLayers;
     int		numSrcLayers, numNewLayers, ii;

/* trace the object, and get the source and new layers */

     newOverlay = ( AmOverlayClass *) AmTraceObject( srcOverlay->GetWindow(),
					         srcOverlay, NULL);
     srcOverlay->GetLayers( &srcLayers, &numSrcLayers);
     newOverlay->GetLayers( &newLayers, &numNewLayers);

/* find the layer the user wanted to apply transparency to, save it 
   and set the transparency. */

     for ( ii = 0; ii < numSrcLayers; ii++) {
	if ( srcLayers[ ii] == ( AmDataGraphicClass *) srcImagePlot) {
	     newImagePlot = ( AmImagePlotClass *) newLayers[ ii];
	     newImagePlot->SetVaFields( AmNtransparency, factor, NULL);
	}
    }
    free( srcLayers);
    free( newLayers);
    return(( AmPlotClass *) newOverlay);
}

/* callback to set up a transparencying glass for an object */

AnvilBoolean
AmTransparencyGlassCallback( Widget			btn,
			   AmMotifDisplayClass	*motifDisplay,
			   AmOverlayClass		*srcOverlay,
			   AmImagePlotClass		*imagePlot,
			   XtPointer		callData)
{

/* make sure that the source overlay and image plot are legitimate */

    if ( !srcOverlay->IsType( AmOVERLAY)) {
	AmAddResourceToLog( "transparencySourceNotOverlay");
	return( FALSE);
    }
    if ( !imagePlot->IsType( AmIMAGE_PLOT)) {
	AmAddResourceToLog( "transparencyLayerNotImagePlot");
	return( FALSE);
    }

    new TransparentMagicClass( motifDisplay, srcOverlay, imagePlot, 
			   AmGetDefault( "transparency.drag", ( AnvilBoolean) FALSE)); 

    return( TRUE);
}

/* subclass for magic windows which allows transparency control for the
   layer */

TransparentMagicClass::TransparentMagicClass( AmMotifDisplayClass	*motifDisplay,
					      AmOverlayClass	*srcOverlay,
					      AmImagePlotClass	*imagePlot,
					      AnvilBoolean		callOnDrag) :
				AmMagicPlotClass( motifDisplay, 
						( AmPlotClass *) srcOverlay,
						callOnDrag),
				factor( 0.5), 
				srcImagePlot( imagePlot)
{
    Widget	scale, closeBtn;
    Arg		args[ 20];
    int		n;

/* initialize the magic plot */

    Update();

/* create a dialog for setting the zoom factor */

    n = AmMakeArgList( args,
                     XmNdeleteResponse, XmDESTROY,
                     XmNautoUnmanage, FALSE,
                     NULL);
    dialog = XmCreateFormDialog( motifDisplay->GetShell(), "transparency", 
				 args, n);
    scale = XtVaCreateManagedWidget( "transparencyScale", xmScaleWidgetClass,
				     dialog,
				     XmNorientation, XmHORIZONTAL,
				     XmNminimum, 1,
				     XmNmaximum, 100,
				     XmNvalue, 50,
				     XmNdecimalPoints, 2,
				     StickLeft, StickTop, StickBottom,
				     XmNrightOffset, 80,
				     XmNshowValue, TRUE,
				     NULL);
    closeBtn = XtVaCreateManagedWidget( "closeBtn", xmPushButtonWidgetClass,
					dialog,
					RightOf( scale),
					StickTop, StickBottom, StickRight,
					NULL);
    XtManageChild( dialog);

/* add callbacks to destroy the dialog and set the scale */

    XtAddCallback( closeBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, dialog);
    XtAddCallback( scale, XmNvalueChangedCallback,
		   ( XtCallbackProc) setTransparencyScale, this);
    XtAddCallback( dialog, XmNdestroyCallback,
		   ( XtCallbackProc) destroyTransparencyCallback, 
		   this);

/* add the callback to destroy the magic window when the layer is deleted */

    AmAddCallbackWidget( AM_PKG_ID, AM_DELETE_OBJECT, motifDisplay->GetArea(),
		       ( callbackProc) destroyTransparencyMagicCallback, this);
}
