#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <malloc.h>
#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>
#include <X11/cursorfont.h>
#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/userevents.h>
#include <anvil/macros.h>
#include <anvil/plot_cback.h>
#include <anvil/events.h>
#include <anvil/outline.h>
#include <anvil/utility.h>
#include <anvil/textcheck.h>
#include <anvil/state_hand.h>

static AmActionDefRec actionAreaDef[] = {
    { "okBtn", NULL, NULL },
    { "cancelBtn", NULL, NULL },
};

typedef struct zoomTextData_ {
    AmPlotClass			*plot;
    float			xminRange[ 2];		/* range values for zooming */
    float			yminRange[ 2];
    float			xmaxRange[ 2];
    float			ymaxRange[ 2];
    float			currXmin;
    float			currYmin;
    float			currXmax;
    float			currYmax;
    Widget			xminText;		/* zoom rect text widgets */
    Widget			yminText;
    Widget			xmaxText;
    Widget			ymaxText;
    Widget			formDlog;
    unitType			xunits;
    unitType			yunits;
    AmMotifDisplayClass		*motifDisplay;
    int				startX;
    int				startY;
    AmObjectOutlineClass        	*outline;
} zoomTextDataRec, *zoomTextDataPtr;

static void zoomTextXminYminPressHandler( Widget		area,
					  zoomTextDataPtr	zoomTextData,
					  XButtonEvent		*event,
					  AnvilBoolean		*);

/* remove the handler when the dialog is destroyed */

static void 
textDragDestroyCallback( Widget			formDlog,
			 zoomTextDataPtr	zoomTextData,
			 XtPointer)
{
    AmRemoveHandler( zoomTextData->motifDisplay->GetArea(), ButtonPressMask,
		   ( XtEventHandler) zoomTextXminYminPressHandler,
		   zoomTextData);
    zoomTextData->motifDisplay->SetCursor( XC_arrow);
    if ( zoomTextData->outline != NULL) {
        zoomTextData->motifDisplay->RemoveOutline( zoomTextData->outline);
    }

    free( zoomTextData);
}

/* drag handler to update xmax, ymax fields */

static void 
zoomTextXmaxYmaxHandler( Widget			area,
			 zoomTextDataPtr	zoomTextData,
			 XEvent			*event,
			 AnvilBoolean		*continueDispatch)
{
    AmMotifDisplayClass	*motifDisplay = zoomTextData->motifDisplay;

    if ( event->type == MotionNotify) {
	XMotionEvent		*motionEvent = ( XMotionEvent *) event;
	float			dx, dy;
        int             	x, y;
	char			buffer[ 64];

        x = zoomTextData->motifDisplay->VirtualCoord( motionEvent->x);
        y = zoomTextData->motifDisplay->VirtualCoord( motionEvent->y);

	if ( zoomTextData->plot->InRect( x, y)) {

/* fill in the text widgets with the values obtained from the mouse */

	    zoomTextData->plot->GetDataPt( x, y, &dx, &dy);
	    if (( dx > zoomTextData->currXmin) &&
		( dy > zoomTextData->currYmin)) {
		zoomTextData->currXmax = dx;
		zoomTextData->currYmax = dy;
		zoomTextData->xminRange[ 1] = zoomTextData->currXmax;
		zoomTextData->yminRange[ 1] = zoomTextData->currYmax;
		AmPrintValue( zoomTextData->xunits, dx, buffer);
		XmTextFieldSetString( zoomTextData->xmaxText, buffer);
		AmPrintValue( zoomTextData->yunits, dy, buffer);
		XmTextFieldSetString( zoomTextData->ymaxText, buffer);

		if ( zoomTextData->outline == NULL) {
		    zoomTextData->outline = 
			new AmObjectOutlineClass( motifDisplay,
						MIN( zoomTextData->startX, x),
						MIN( zoomTextData->startY, y),
						abs( x - zoomTextData->startX), 
						abs( y - zoomTextData->startY));
		    motifDisplay->AddOutline( zoomTextData->outline);
		} else {
		    zoomTextData->outline->SetRectangle(
						 MIN( zoomTextData->startX, x),
						 MIN( zoomTextData->startY, y),
						 abs( x - zoomTextData->startX), 
						 abs( y - zoomTextData->startY));
		    motifDisplay->ShowOutline( zoomTextData->outline);
		}
	    }
	}
    } else if ( event->type == ButtonRelease) {

/* remove this handler and restore the initial button press  handler */

	AmRemoveHandler( area, PointerMotionMask | ButtonReleaseMask,
		       ( XtEventHandler) zoomTextXmaxYmaxHandler, 
		       zoomTextData);
	AmAddHandler( NULL, area, ButtonPressMask, 
		    ( XtEventHandler) zoomTextXminYminPressHandler,
		    zoomTextData, ZOOM_RECTANGLE_PRIORITY);
	motifDisplay->SetCursor( XC_ll_angle);

        if ( zoomTextData->outline != NULL) {
            motifDisplay->HideOutline( zoomTextData->outline);
        }
    }
}

/* update the xmin, ymin widgets on button motion, then remove this 
   handler on button release, and start the xmax, ymax handler */

static void 
zoomTextXminYminMotionHandler( Widget		area,
			       zoomTextDataPtr	zoomTextData,
			       XEvent		*event,
			       AnvilBoolean		*)
{
    AmMotifDisplayClass	*motifDisplay = zoomTextData->motifDisplay;
    char  		buffer[ 64];
    float 		dx, dy;
    int   		x, y;

/* fill in the text widgets with the values obtained from the mouse */

    if ( event->type == MotionNotify) {
	XMotionEvent	*motionEvent = ( XMotionEvent *) event;

        x = motifDisplay->VirtualCoord( motionEvent->x);
        y = motifDisplay->VirtualCoord( motionEvent->y);

	if ( zoomTextData->plot->InRect( x, y)) {
	    zoomTextData->plot->GetDataPt( x, y, &dx, &dy);
	    zoomTextData->currXmin = dx;
	    zoomTextData->currYmin = dy;
	    zoomTextData->xmaxRange[ 0] = zoomTextData->currXmin;
	    zoomTextData->ymaxRange[ 0] = zoomTextData->currYmin;
	    AmPrintValue( zoomTextData->xunits, dx, buffer);
	    XmTextFieldSetString( zoomTextData->xminText, buffer);
	    AmPrintValue( zoomTextData->yunits, dy, buffer);
	    XmTextFieldSetString( zoomTextData->yminText, buffer);
	}
    } else {
	XButtonEvent	*buttonEvent = ( XButtonEvent *) event;

        x = motifDisplay->VirtualCoord( buttonEvent->x);
        y = motifDisplay->VirtualCoord( buttonEvent->y);

/* on button release, remove this handler, and start the xmax, ymax
   motion handler */

	AmRemoveHandler( area, PointerMotionMask | ButtonReleaseMask,
		       ( XtEventHandler) zoomTextXminYminMotionHandler,
		       zoomTextData);
	AmAddHandler( NULL, area, PointerMotionMask | ButtonReleaseMask, 
		    ( XtEventHandler) zoomTextXmaxYmaxHandler,
		    zoomTextData, ZOOM_RECTANGLE_PRIORITY);
	motifDisplay->SetCursor( XC_ur_angle);
	zoomTextData->startX = x;
	zoomTextData->startY = y;
    }
}

/* handler to kick off the motion handler to update the xmin, ymin
   corners of the zoom rectangle */

static void 
zoomTextXminYminPressHandler( Widget		area,
			      zoomTextDataPtr	zoomTextData,
			      XButtonEvent	*event,
			      AnvilBoolean		*)
{

/* remove this handler */

    AmRemoveHandler( area, ButtonPressMask, 
		   ( XtEventHandler) zoomTextXminYminPressHandler,
		   zoomTextData);

/* add the event handler to update the xmax, ymax on drag */

    AmAddHandler( NULL, area, PointerMotionMask | ButtonReleaseMask,
	        ( XtEventHandler) zoomTextXminYminMotionHandler,
		zoomTextData, ZOOM_RECTANGLE_PRIORITY);
}

/* zoom the plot on OK */

static void 
zoomTextOkCallback( Widget		btn,
		    zoomTextDataPtr	zoomTextData,
		    XtPointer)
{
    char	*str;
    float	xmin, ymin, xmax, ymax, txmin, tymin, txmax, tymax;
    unitType	xunits, yunits;

    zoomTextData->plot->GetVaFields( AmNxunits, &xunits,
				     AmNyunits, &yunits,
				     NULL);
    str = XmTextFieldGetString( zoomTextData->xminText);
    AmScanType( str, &txmin, AmFLOAT, xunits);
    XtFree( str);
    str = XmTextFieldGetString( zoomTextData->yminText);
    AmScanType( str, &tymin, AmFLOAT, yunits);
    XtFree( str);
    str = XmTextFieldGetString( zoomTextData->xmaxText);
    AmScanType( str, &txmax, AmFLOAT, xunits);
    XtFree( str);
    str = XmTextFieldGetString( zoomTextData->ymaxText);
    AmScanType( str, &tymax, AmFLOAT, yunits);
    XtFree( str);
    xmin = MIN( txmin, txmax);
    ymin = MIN( tymin, tymax);
    xmax = MAX( txmin, txmax);
    ymax = MAX( tymin, tymax);
    AN_DEBUG( 40, ( "calling zoomText ok %.2f, %.2f, %.2f, %.2f\n",
		    xmin, ymin, xmax, ymax));
    zoomTextData->plot->SetCoords( xmin, ymin, xmax, ymax);
    zoomTextData->motifDisplay->Flush();
    XtDestroyWidget( AmGetWidgetShell( btn));
}
	

/***************************************************************************
 *
 * Description:
 * 	zoom rectangle dragged by cursor with a dialog for text entry for the
 *  	zoom coordinates 
 * Arguments:
 *      w - widget that launched the callback
 *      motifDisplay - display manager for the plot being zoomed. 
 *	plot - plot or overlay to zoom.
 *
 **************************************************************************/


AnvilBoolean
AmZoomTextCallback( Widget		w,
		    AmMotifDisplayClass	*motifDisplay,
		    AmCoordClass	*plot,
		    XtPointer)
{
    Widget		xminLbl, xminForm, yminLbl, yminForm, xmaxLbl, 
			xmaxForm, ymaxLbl, ymaxForm, rowcol, actionForm;
    AmActionRec		actionArea;
    unitType		xunits, yunits;
    float		xmin, ymin, xmax, ymax, 
			currXmin, currYmin, currXmax, currYmax;
    zoomTextDataPtr	zoomTextData;

/* get the units and range for the plot for range and regular expression checking
   in the dialog */

    plot->GetVaFields( AmNxunits, &xunits,
		       AmNyunits, &yunits,
		       NULL);
    plot->GetRange( &xmin, &ymin, &xmax, &ymax);
    plot->GetCoords( &currXmin, &currYmin, &currXmax, &currYmax);
    zoomTextData = ( zoomTextDataPtr) malloc( sizeof( zoomTextDataRec));
    zoomTextData->plot = ( AmPlotClass *) plot;
    zoomTextData->xminRange[ 0] = xmin;
    zoomTextData->xminRange[ 1] = currXmax;
    zoomTextData->yminRange[ 0] = xmin;
    zoomTextData->yminRange[ 1] = currYmax;
    zoomTextData->xmaxRange[ 0] = currXmin;
    zoomTextData->xmaxRange[ 1] = xmax;
    zoomTextData->ymaxRange[ 0] = currYmin;
    zoomTextData->ymaxRange[ 1] = ymax;
    zoomTextData->currXmin = xmin;
    zoomTextData->currYmin = ymin;
    zoomTextData->currXmax = xmax;
    zoomTextData->currYmax = ymax;
    zoomTextData->xunits = xunits;
    zoomTextData->yunits = yunits;
    zoomTextData->motifDisplay = motifDisplay;
    zoomTextData->outline = NULL;

/* create the form dialog and the apply and close buttons */

    zoomTextData->formDlog = XmCreateFormDialog( motifDisplay->GetShell(), 
					         "zoomRectForm", NULL, 0);
    actionForm = AmMakeActionArea( zoomTextData->formDlog,
				 actionAreaDef, ELEMENTS( actionAreaDef),
				 &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);
    rowcol = XtVaCreateManagedWidget( "rowcol", xmRowColumnWidgetClass,
				      zoomTextData->formDlog,
				      StickTop, StickLeft, StickRight,
				      AboveOf( actionForm),
				      NULL);

/* create the text fields to enter the coordinate values */

    xminForm = XtVaCreateManagedWidget( "xminForm", xmFormWidgetClass,
					rowcol, NULL);
    xminLbl = XtVaCreateManagedWidget( "xminLbl", xmLabelWidgetClass,
				       xminForm, StickLeft, StickTop,
				       StickBottom,
				       NULL);
    zoomTextData->xminText = 
	XtVaCreateManagedWidget( "xminText", xmTextFieldWidgetClass,
				 xminForm, RightOf( xminLbl), StickTop,
			         StickBottom,
			         NULL);

    yminForm = XtVaCreateManagedWidget( "yminForm", xmFormWidgetClass,
					rowcol, NULL);
    yminLbl = XtVaCreateManagedWidget( "yminLbl", xmLabelWidgetClass,
				       yminForm, StickLeft, StickTop,
				       StickBottom,
				       NULL);
    zoomTextData->yminText = 
	XtVaCreateManagedWidget( "yminText", xmTextFieldWidgetClass,
			         yminForm, RightOf( yminLbl), StickTop,
			         StickBottom,
			         NULL);

    xmaxForm = XtVaCreateManagedWidget( "xmaxForm", xmFormWidgetClass,
					rowcol, NULL);
    xmaxLbl = XtVaCreateManagedWidget( "xmaxLbl", xmLabelWidgetClass,
				       xmaxForm, StickLeft, StickTop,
				       StickBottom,
				       NULL);
    zoomTextData->xmaxText = 
	XtVaCreateManagedWidget( "xmaxText", xmTextFieldWidgetClass,
			         xmaxForm, RightOf( xmaxLbl), StickTop,
			         StickBottom,
			         NULL);

    ymaxForm = XtVaCreateManagedWidget( "ymaxForm", xmFormWidgetClass,
					rowcol, NULL);
    ymaxLbl = XtVaCreateManagedWidget( "ymaxLbl", xmLabelWidgetClass,
				       ymaxForm, StickLeft, StickTop,
				       StickBottom,
				       NULL);
    zoomTextData->ymaxText = 
	XtVaCreateManagedWidget( "ymaxText", xmTextFieldWidgetClass,
			         ymaxForm, RightOf( ymaxLbl), StickTop,
			         StickBottom,
			         NULL);

/* set up the range checking for the text values */

    AmUnitVarEntry( zoomTextData->xminText, &currXmin, AmFLOAT, 
		    xunits, zoomTextData->xminRange);
    AmUnitVarEntry( zoomTextData->yminText, &currYmin, AmFLOAT, 
		    yunits, zoomTextData->yminRange);
    AmUnitVarEntry( zoomTextData->xmaxText, &currXmax, AmFLOAT, 
		    xunits, zoomTextData->xmaxRange);
    AmUnitVarEntry( zoomTextData->ymaxText, &currYmax, AmFLOAT, 
		    yunits, zoomTextData->ymaxRange);
    AmAddHandler( NULL, motifDisplay->GetArea(), ButtonPressMask, 
		  ( XtEventHandler) zoomTextXminYminPressHandler,
		  zoomTextData, ZOOM_RECTANGLE_PRIORITY);
    motifDisplay->SetCursor( XC_ll_angle);
    XtManageChild( zoomTextData->formDlog);

/* set up the callbacks for destroy, OK, and close */

    XtAddCallback( zoomTextData->formDlog, XmNdestroyCallback,
		   ( XtCallbackProc) textDragDestroyCallback, 
		   zoomTextData);
    XtAddCallback( actionArea.buttons[ 0], XmNactivateCallback,
		   ( XtCallbackProc) zoomTextOkCallback, zoomTextData);
    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, zoomTextData->formDlog);
    return( TRUE);
}
