#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <anvil/project.h>
#include <anvil/geometry.h>
#include <anvil/projects.h>
#include <anvil/macros.h>


int main( int argc, char *argv[])
{
    float	lat0, lon0, lat1, lon1, x0, y0, x1, y1, distance, bearing;
    PJ		*pj;
    char	*proj[ 4], zonebuf[ 256];
    int		zone, ii;

    if ( argc < 5) {
	fprintf( stderr, "usage %s lon0 lat0 lon1 lat1\n", argv[ 0]);
	exit( -1);
    }
    lat0 = atof( argv[ 1]);
    lon0 = atof( argv[ 2]);
    lat1 = atof( argv[ 3]);
    lon1 = atof( argv[ 4]);
    zone = ( int) ( 1 + ( 180 + (( lon0 + lon1)/2)))/6;
    proj[ 0] = "proj=utm";
    proj[ 1] = "ellps=WGS84";
    sprintf( zonebuf, "zone=%d", zone);
    proj[ 2] = zonebuf;
    proj[ 3] = NULL;
    if (( pj = pj_init( 3, (const char **)proj)) == NULL) {
	fprintf( stderr, "failed to initialize projection ");
	for ( ii = 0; ii < 3; ii++) {
	    fprintf( stderr, "%s ", proj[ ii]);
	}
	printf( "\n");
    }
    x0 = lon0;
    y0 = lat0;
    x1 = lon1;
    y1 = lat1;
    projectPt( pj, FALSE, &x0, &y0);
    projectPt( pj, FALSE, &x1, &y1);
    distance = AmDist( x0, y0, x1, y1);
    bearing = atan2( y1 - y0, x1 - x0);
    printf( "%.2f %.4f\n", distance, RADTODEG( bearing));
    return 0;
}
