#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <malloc.h>
#include <stdio.h>
#include <anvil/datatype.h>
#include <anvil/fns.h>

#define READBUFSIZE 10000

/* program for converting raw binary matrices from one format to another */

int main( int argc, char *argv[])
{
    AmDataTypeEnum	inType, outType;
    AmAccessFn		inFn; 
    AmSetFn		outFn;
    int			inSize, outSize, inFd, outFd, numbytes, numel, ii;
    uchar_t		*inBuf, *inPtr, *outBuf, *outPtr;
    float		val;

    if ( argc < 5) {
	fprintf( stderr, "%s infile outfile intype outttype\n", argv[ 0]);
	exit( -1);
    }
    inType = AmStrToDataTypeEnum( argv[ 3]);
    if ( inType == AmNO_TYPE) {
	fprintf( stderr, "unrecognized type %s\n", argv[ 3]);
	exit( -1);
    }
    outType = AmStrToDataTypeEnum( argv[ 4]);
    if ( outType == AmNO_TYPE) {
	fprintf( stderr, "unrecognized type %s\n", argv[ 4]);
	exit( -1);
    }
    if (( inFd = open( argv[ 1], O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }
    if (( outFd = open( argv[ 2], O_CREAT | O_WRONLY, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", argv[ 2]);
	exit( -1);
    }
    inSize = AmDataTypeEnumToSize( inType);
    outSize = AmDataTypeEnumToSize( outType);
    inFn = AmDataTypeEnumToAccessFn( inType);
    outFn = AmDataTypeEnumToSetFn( outType);
    inBuf = ( uchar_t *) malloc( READBUFSIZE*inSize);
    outBuf = ( uchar_t *) malloc( READBUFSIZE*outSize);
    do {
	numbytes = read( inFd, inBuf, READBUFSIZE*inSize);
	numel = numbytes/inSize;
	for ( ii = 0, inPtr = inBuf, outPtr = outBuf; 
	      ii < numel;
	      ii++, inPtr += inSize, outPtr += outSize) {
	    val = ( inFn)( inPtr, NULL);
	    ( outFn)( outPtr, NULL, val);
	}
	write( outFd, outBuf, numel*outSize);
    } while ( numbytes == READBUFSIZE*inSize);
    close( inFd);
    close( outFd);
}
