#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>
#if !defined( _WIN32) && !defined(__BSD44__)
#include <alloca.h>
#endif
#include <string.h>
#include <anvil/tagstrs.h>
#include <anvil/macros.h>
#include <anvil/geometry.h>
#include <anvil/datafmt.h>
#if defined(_WIN32)
#include <anvil/ntresource.h>
#else
#include <anvil/xresource.h>
#endif
#include <anvil/files.h>
#include <util/utils.h>
#include <anvil/choppath.h>

/* program for cropping files to newly specified extents */

const int LINEBUFCNT = 65536, 
	 PTBUFCNT = 65536,
	 POLYBUFSIZE = 262144,
	 MAXPOLYPTS = 16384;

int main( int argc, char **argv)
{
    int			fd, rows, cols, size, fdOut, numread, ii, jj, kk, 
			total, x0, y0, x1, y1, leftOver, numBytes, readBytes,
			writeBytes, polySize, numPts, plineSize;
    float		xmin, ymin, xmax, ymax, cropXmin, cropYmin, cropXmax, 
			cropYmax;
    FILE		*hdrFp;
    char		*fileType, *buffer, *ptr, outpath[ 1024], 
			hdrpath[ 1024], polybuf[ POLYBUFSIZE], 
			plinebuf[ POLYBUFSIZE],
			writeBuf[ POLYBUFSIZE], *polyend, *plineend;
    char		*fields[] = { AmNfilename, AmNrows, AmNcols,
				      AmNxmin, AmNymin, AmNxmax, AmNymax, 
				      NULL };
    AmCoordRec		tmpPts[ MAXPOLYPTS], clipPts[ MAXPOLYPTS];
    AmCoordPtr		coord;
    AmLinePtr		line;
    AmPolygonPtr	poly;
    AmPolylinePtr	pline;
    AnvilBoolean		pipeFlag;

    if ( argc < 8) {
	fprintf( stderr, 
		 "usage: %s inhdr infile xmin ymin xmax ymax outfile\n",
		 argv[ 0]);
	exit( -1);
    }

/* get the extents of the original file */

#if defined(_WIN32)
    AmNTResourceDbClass db( argv[ 1]);
#else
    AmXResourceDbClass db( argv[ 1]);
#endif
    if ( !db.GetResource( AmNxmin, &xmin)) {
	fprintf( stderr, "failed to get xmin from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNymin, &ymin)) {
	fprintf( stderr, "failed to get ymin from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNxmax, &xmax)) {
	fprintf( stderr, "failed to get xmax from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNymax, &ymax)) {
	fprintf( stderr, "failed to get ymax from %s\n", argv[ 1]);
	exit( -1);
    }

/* find the extents that we're going to crop to */

    cropXmin = atof( argv[ 3]);
    cropYmin = atof( argv[ 4]);
    cropXmax = atof( argv[ 5]);
    cropYmax = atof( argv[ 6]);

/* check to make sure the extents are reasonable */

    if ( cropXmin >= cropXmax) {
	fprintf( stderr, "crop Xmin %.2f, is greater than crop Xmax %.2f\n",
		 cropXmin, cropXmax);
    }
    if ( cropXmin < xmin) {
	fprintf( stderr, "crop Xmin %.2f truncated to xmin %.2f\n", 
		 cropXmin, xmin);
	cropXmin = xmin;
    }
    if ( cropXmax > xmax) {
	fprintf( stderr, "crop Xmax %.2f truncated to xmax %.2f\n", 
		 cropXmax, xmax);
	cropXmax = xmax;
    }
    if ( cropYmin >= cropYmax) {
	fprintf( stderr, "crop Ymin %.2f, is greater than crop Ymax %.2f\n",
		 cropYmin, cropYmax);
    }
    if ( cropYmin < ymin) {
	fprintf( stderr, "crop Ymin %.2f truncated to ymin %.2f\n", 
		 cropYmin, ymin);
	cropYmin = ymin;
    }
    if ( cropYmax > ymax) {
	fprintf( stderr, "crop Ymax %.2f truncated to ymax %.2f\n", 
		 cropYmax, ymax);
	cropYmax = ymax;
    }

/* find out what kind of file we're dealing with and open it */

    if ( !db.GetResource( AmNtype, &fileType)) {
	fprintf( stderr, "failed to get fileType from %s\n", argv[ 1]);
	exit( -1);
    }
    if (( fd = AmOpenReadFile( argv[ 2], &pipeFlag)) < 0) {
	fprintf( stderr, "failed to open file %s\n", argv[ 2]);
	exit(-1);
    }

/* if it's a relative path for the output file, then prepend the current
   working directory to it */

    sprintf( outpath, "%s.dat", argv[ 7]);
    sprintf( hdrpath, "%s.hdr", argv[ 7]);
    if (( fdOut = open( outpath, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open file %s\n", argv[ 7]);
	exit(-1);
    }
    if (( hdrFp = fopen( hdrpath, "w")) == NULL) {
	 fprintf( stderr, "failed to open header file %s\n", hdrpath);
	 exit( -1);
    }
    if ( !strcmp( fileType, AmNimagePlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to get rows from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNcols, &cols)) {
	    fprintf( stderr, "failed to get cols from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to get size from %s\n", argv[ 1]);
	    exit( -1);
	}

/* find out where the cropped coordinates are in the image coords */

	x0 = ( int) ( cols*(cropXmin - xmin)/( xmax - xmin));
	y0 = ( int) ( rows - rows*(cropYmax - ymin)/( ymax - ymin));
	x1 = ( int) ( cols*(cropXmax - xmin)/( xmax - xmin));
	y1 = ( int) ( rows - rows*(cropYmin - ymin)/( ymax - ymin));
	if (( x0 < 0) || ( y0 < 0) || ( x1 > cols) || ( y1 > rows)) {
	    fprintf( stderr, "the coordinates (%.2f, %.2f), (%.2f, %.2f)\n",
		     cropXmin, cropYmin, cropXmax, cropYmax);
	    fprintf( stderr, "map to: (%d, %d), (%d, %d)\n", x0, y0, x1, y1);
	    fprintf( stderr, "outside the matrix dimensions rows=%d, cols=%d\n",
		     rows, cols);
	    exit( -1);
	}

/* allocate a row buffer, seek to the start of the subimage and read by rows */

	if (( buffer = ( char *) malloc( cols*size)) == NULL) {
	    fprintf( stderr, "failed to malloc %d bytes\n", cols*size);
	    exit( -1);
	}
	if ( !pipeFlag) {
	    if ( lseek( fd, cols*y0*size, SEEK_SET) < 0) {
		fprintf( stderr, "failed to seek %d bytes\n", cols*y0*size);
		exit( -1);
	    }
	} else {
	    for ( ii = 0; ii < y0; ii++) {
		if ( AmReadLoopFile( fd, buffer, cols*size) < 
				   cols*size) {
		    fprintf( stderr, "failed to read %d bytes row %d\n", 
			     ii, cols*size);
		    exit( -1);
		}
	    }
	}
	for ( ii = 0; ii < y1 - y0; ii++) {
	    AmPrintHatch( ii, y1 - y0, NHATCH);
	    if ( AmReadLoopFile( fd, buffer, cols*size) < cols*size) {
		fprintf( stderr, "failed to read %d bytes row %d\n", 
			 ii, cols*size);
		exit( -1);
	    }
	    if ( write( fdOut, &buffer[ x0*size], (x1 - x0)*size) < 
		 ( x1 - x0)*size) {
		fprintf( stderr, "failed to write %d bytes row %d\n", 
			 ii, ( x1 - x0)*size);
		exit( -1);
	    }
	}

/* create the header file which describes the cropped output */
 
        fprintf( hdrFp, "rows:          %d\n", y1 - y0);
        fprintf( hdrFp, "cols:          %d\n", x1 - x0);
    } else if ( !strcmp( fileType, AmNlinePlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to get rows from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to get size from %s\n", argv[ 1]);
	    exit( -1);
	}
	if (( buffer = ( char *) malloc( LINEBUFCNT*size)) == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", LINEBUFCNT*size);
	    exit( -1);
	}

/* read the lines block by block, clip them to the rectangle, and write out
   the resulting block */
	
	total = 0;
	for ( ii = 0; ii < rows; ii += LINEBUFCNT) {
	    AmPrintHatch( ii, rows, NHATCH);
	    numread = MIN( LINEBUFCNT, rows - ii);
	    if ( read( fd, buffer, numread*size) < numread*size) {
		fprintf( stderr, "failed to read %d bytes\n", numread*size);
		exit( -1);
	    }
	    kk = 0;
AN_DEBUG( 70, ("read %d vectors\n", numread));
	    for ( jj = 0, ptr = buffer; jj < numread; jj++, ptr += size) {
		line = ( AmLinePtr) ptr;
		if ( AmClip2d( &line->x0, &line->y0, &line->x1, &line->y1,
			     cropXmin, cropYmin, cropXmax, cropYmax)) {
		    memcpy( &buffer[ kk*size], ptr, size);
		    kk++;
		}
	    }
	    numread = kk;
AN_DEBUG( 70, ("cropped to %d vectors\n", numread));
	    if ( write( fdOut, buffer, numread*size) < numread*size) {
		fprintf( stderr, "failed to write %d bytes\n", numread*size);
		exit( -1);
	    }
	    total += numread;
	}

/* write the total # of elements in the crop segment, and write rest later */
 
        fprintf( hdrFp, "rows:          %d\n", total);
    } else if ( !strcmp( fileType, AmNscatterPlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to get rows from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to get size from %s\n", argv[ 1]);
	    exit( -1);
	}
	if (( buffer = ( char *) malloc( PTBUFCNT*size)) == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", PTBUFCNT*size);
	    exit( -1);
	}

/* read the points block by block, clip them to the rectangle, and write out
   the resulting block */
	
	total = 0;
	for ( ii = 0; ii < rows; ii += PTBUFCNT) {
	    AmPrintHatch( ii, rows, NHATCH);
	    numread = MIN( PTBUFCNT, rows - ii);
	    if ( read( fd, buffer, numread*size) < numread*size) {
		fprintf( stderr, "failed to read %d bytes\n", numread*size);
		exit( -1);
	    }
	    kk = 0;
	    for ( jj = 0, ptr = buffer; jj < numread; jj++, ptr += size) {
		coord = ( AmCoordPtr) ptr;
		if ( AmBetweenFloat( coord->x, cropXmin, cropXmax) &&
		     AmBetweenFloat( coord->y, cropYmin, cropYmax)) {
		    memcpy( &buffer[ kk*size], ptr, size);
		    kk++;
		}
	    }
	    numread = kk;
	    if ( write( fdOut, buffer, numread*size) < numread*size) {
		fprintf( stderr, "failed to write %d bytes\n", numread*size);
		exit( -1);
	    }
	    total += numread;
	}

/* create the header file which describes the cropped output */
 
        fprintf( hdrFp, "rows:          %d\n", total);
    } else if ( !strcmp( fileType, AmNpolygonPlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to get rows from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to get size from %s\n", argv[ 1]);
	    exit( -1);
	}
	if (( buffer = ( char *) alloca( size)) == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", size);
	    exit( -1);
	}

/* read the polygons individually */

	leftOver = 0;
	ii = 0;
	total = 0;
	do {
	    readBytes = sizeof( polybuf) - leftOver;
	    numBytes = read( fd, &polybuf[ leftOver], readBytes);
	    polySize = 0;
	    polyend = polybuf + numBytes + leftOver;
 
/* loop through the buffer as long as we can keep reading polygons */

	    writeBytes = 0;
	    for ( ptr = polybuf; ptr < polyend; ptr += polySize, ii++) {
		AmPrintHatch( ii, rows, NHATCH);
		poly = ( AmPolygonPtr) ptr;
		polySize = size + poly->numPts*sizeof( AmCoordRec);
 
/* if the polygon extends past the end of the buffer then copy that stuff to
   the head of the buffer */
 
		if (( ptr + polySize > polyend) || ( ptr + size > polyend)) {
		    leftOver = polyend - ptr;
		    memcpy( polybuf, ptr, leftOver);
		    break;
		} else {
		    poly->pts = ( AmCoordPtr) &ptr[ size];

/* clip the polygon to te extents, and copy the clipped points to the 
   write buffer */

		    numPts = poly->numPts;
		    if ( AmClipPoly( poly->pts, tmpPts, clipPts,
				   cropXmin, cropYmin, cropXmax, cropYmax, 
				   &numPts)) {
			poly->numPts = numPts;
			total++;
			memcpy( &writeBuf[ writeBytes], poly, size);
			writeBytes += size;
			memcpy( &writeBuf[ writeBytes], clipPts, 
				sizeof( AmCoordRec)*poly->numPts);
			writeBytes += sizeof( AmCoordRec)*poly->numPts;
		    }
		}
	    }

/* write the buffer */

AN_DEBUG( 70, ( "ii = %d, total = %d, writeBytes = %d\n", 
	        ii, total, writeBytes));
	    if ( write( fdOut, writeBuf, writeBytes) < writeBytes) {
		fprintf( stderr, "failed to write buffer of %d bytes\n", writeBytes);
	    }

/* bail if we're at the end of the file */

	    if ( numBytes < readBytes) {
		break;
	    }
	} while ( TRUE);

/* create the header file which describes the cropped output */
 
        fprintf( hdrFp, "rows:          %d\n", total);
    } else if ( !strcmp( fileType, AmNpolylinePlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to get rows from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to get size from %s\n", argv[ 1]);
	    exit( -1);
	}
	if (( buffer = ( char *) alloca( size)) == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", size);
	    exit( -1);
	}

/* read the polygons individually */

	leftOver = 0;
	ii = 0;
	total = 0;
	do {
	    readBytes = sizeof( plinebuf) - leftOver;
	    numBytes = read( fd, &plinebuf[ leftOver], readBytes);
	    plineSize = 0;
	    plineend = plinebuf + numBytes + leftOver;
 
/* loop through the buffer as long as we can keep reading polygons */

	    writeBytes = 0;
	    for ( ptr = plinebuf; ptr < plineend; ptr += plineSize, ii++) {
		AmPrintHatch( ii, rows, NHATCH);
		pline = ( AmPolylinePtr) ptr;
		plineSize = size + pline->numPts*sizeof( AmCoordRec);
 
/* if the polyline extends past the end of the buffer then copy that stuff to
   the head of the buffer */
 
		if (( ptr + plineSize > plineend) || ( ptr + size > plineend)) {
		    leftOver = plineend - ptr;
		    memcpy( plinebuf, ptr, leftOver);
		    break;
		} else {
		    pline->pts = ( AmCoordPtr) &ptr[ size];

/* copy the polyline if it falls within the clip boundary */

		    numPts = pline->numPts;
		    if (( pline->xmin <= cropXmax) &&
			( pline->ymin <= cropYmax) &&
			( pline->xmax >= cropXmin) &&
			( pline->ymax >= cropYmin)) {
			pline->numPts = numPts;
			memcpy( &writeBuf[ writeBytes], pline, size);
			writeBytes += size;
			memcpy( &writeBuf[ writeBytes], pline->pts,
				sizeof( AmCoordRec)*pline->numPts);
			writeBytes += sizeof( AmCoordRec)*pline->numPts;
			total++;
		    }
		}
	    }

/* write the buffer */

AN_DEBUG( 70, ( "ii = %d, total = %d, writeBytes = %d\n", 
	        ii, total, writeBytes));
	    if ( write( fdOut, writeBuf, writeBytes) < writeBytes) {
		fprintf( stderr, "failed to write buffer of %d bytes\n", writeBytes);
	    }

/* bail if we're at the end of the file */

	    if (( numBytes < readBytes) || ( numBytes == 0)) {
		break;
	    }
	} while (( numBytes == readBytes) && ( numBytes != 0));

/* create the header file which describes the cropped output */
 
        fprintf( hdrFp, "rows:          %d\n", total);
    }
    AmCloseReadFile( fd, pipeFlag);
    close( fdOut);
    fprintf( hdrFp, "filename:      %s\n", AmGetRootName( outpath));
    fprintf( hdrFp, "xmin:          %f\n", cropXmin);
    fprintf( hdrFp, "ymin:          %f\n", cropYmin);
    fprintf( hdrFp, "xmax:          %f\n", cropXmax);
    fprintf( hdrFp, "ymax:          %f\n", cropYmax);
    fprintf( hdrFp, "sourceFile:    %s\n", argv[ 1]);
    AmDumpHeaderExcept( argv[ 1], fields, hdrFp);
    fclose( hdrFp);
}
