#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <anvil/tagstrs.h>
#include <anvil/macros.h>
#include <anvil/geometry.h>
#include <anvil/datafmt.h>
#if defined(_WIN32)
#include <anvil/ntresource.h>
#else 
#include <anvil/xresource.h>
#endif
#include <anvil/files.h>
#include <util/utils.h>

#define LINEBUFCNT	65536
#define PTBUFCNT	65536
#define POLYBUFSIZE	262144
#define MAXPOLYPTS	16384

int main( int argc, char **argv)
{
    int		fd, rows, cols, size, fdOut, ii, x0, y0, x1, y1;
    float	xmin, ymin, xmax, ymax, 
		cropXmin, cropYmin, cropXmax, cropYmax;
    FILE	*hdrFp;
    char	*fileType, *buffer, outpath[ 1024], hdrpath[ 1024], 
		*outfile;
    char	*fields[] = { AmNfilename, AmNrows, AmNcols,
			      AmNxmin, AmNymin, AmNxmax, AmNymax, NULL };
    AnvilBoolean	pipeFlag;

    if ( argc < 8) {
	fprintf( stderr, 
		 "usage: %s inhdr infile x0 y0 x1 y1 outfile\n",
		 argv[ 0]);
	exit( -1);
    }

/* get the extents of the original file */

#if defined(_WIN32)
    AmNTResourceDbClass db( argv[ 1]);
#else
    AmXResourceDbClass db( argv[ 1]);
#endif
    if ( !db.GetResource( AmNxmin, &xmin)) {
	fprintf( stderr, "failed to get xmin from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNymin, &ymin)) {
	fprintf( stderr, "failed to get ymin from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNxmax, &xmax)) {
	fprintf( stderr, "failed to get xmax from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNymax, &ymax)) {
	fprintf( stderr, "failed to get ymax from %s\n", argv[ 1]);
	exit( -1);
    }

/* find the extents that we're going to crop to */

    x0 = atoi( argv[ 3]);
    y0 = atoi( argv[ 4]);
    x1 = atoi( argv[ 5]);
    y1 = atoi( argv[ 6]);

/* find out what kind of file we're dealing with and open it */

    if ( !db.GetResource( AmNtype, &fileType)) {
	fprintf( stderr, "failed to get fileType from %s\n", argv[ 1]);
	exit( -1);
    }
    if (( fd = AmOpenReadFile( argv[ 2], &pipeFlag)) < 0) {
	fprintf( stderr, "failed to open file %s\n", argv[ 2]);
	exit(-1);
    }

/* if it's a relative path for the output file, then prepend the current
   working directory to it */

    outfile = argv[ 7];
    sprintf( outpath, "%s.dat", outfile);
    sprintf( hdrpath, "%s.hdr", outfile);
    if (( fdOut = open( outpath, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open file %s\n", argv[ 7]);
	exit(-1);
    }
    if (( hdrFp = fopen( hdrpath, "w")) == NULL) {
	 fprintf( stderr, "failed to open header file %s\n", hdrpath);
	 exit( -1);
    }
    if ( !strcmp( fileType, AmNimagePlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to get rows from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNcols, &cols)) {
	    fprintf( stderr, "failed to get cols from %s\n", argv[ 1]);
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to get size from %s\n", argv[ 1]);
	    exit( -1);
	}

/* allocate a row buffer, seek to the start of the subimage and read by rows */

	if (( buffer = ( char *) malloc( cols*size)) == NULL) {
	    fprintf( stderr, "failed to malloc %d bytes\n", cols*size);
	    exit( -1);
	}
	if ( !pipeFlag) {
	    if ( lseek( fd, cols*y0*size, SEEK_SET) < 0) {
		fprintf( stderr, "failed to seek %d bytes\n", cols*y0*size);
		exit( -1);
	    }
	} else {
	    for ( ii = 0; ii < y0; ii++) {
		if ( AmReadLoopFile( fd, buffer, cols*size) < 
				   cols*size) {
		    fprintf( stderr, "failed to read %d bytes row %d\n", 
			     ii, cols*size);
		    exit( -1);
		}
	    }
	}
	for ( ii = 0; ii < y1 - y0; ii++) {
	    AmPrintHatch( ii, y1 - y0, NHATCH);
	    if ( AmReadLoopFile( fd, buffer, cols*size) < cols*size) {
		fprintf( stderr, "failed to read %d bytes row %d\n", 
			 cols*size, ii + y0);
		exit( -1);
	    }
	    if ( write( fdOut, &buffer[ x0*size], (x1 - x0)*size) < 
		 ( x1 - x0)*size) {
		fprintf( stderr, "failed to write %d bytes row %d\n", 
			 ii, ( x1 - x0)*size);
		exit( -1);
	    }
	}
	cropXmin = xmin + (( xmax - xmin)*x0)/cols;
	cropYmin = ymax - (( ymax - ymin)*y1)/rows;
	cropXmax = xmin + (( xmax - xmin)*x1)/cols;
	cropYmax = ymax - (( ymax - ymin)*y0)/rows;

/* create the header file which describes the cropped output */
 
        fprintf( hdrFp, "rows:          %d\n", y1 - y0);
        fprintf( hdrFp, "cols:          %d\n", x1 - x0);
	AmCloseReadFile( fd, pipeFlag);
	close( fdOut);
	fprintf( hdrFp, "filename:      %s\n", outpath);
	fprintf( hdrFp, "xmin:          %f\n", cropXmin);
	fprintf( hdrFp, "ymin:          %f\n", cropYmin);
	fprintf( hdrFp, "xmax:          %f\n", cropXmax);
	fprintf( hdrFp, "ymax:          %f\n", cropYmax);
	fprintf( hdrFp, "sourceFile:    %s\n", argv[ 1]);
	AmDumpHeaderExcept( argv[ 1], fields, hdrFp);
	fclose( hdrFp);
    } else {
	fprintf( stderr, "%s only supports image files\n", argv[ 0]);
    }
}
