#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */







#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <malloc.h>
#include <string.h>
#include <sys/types.h>
#if !defined(__BSD44__)
#include <anvil/endian.h>
#endif
#include <util/dlg.h>
#include <anvil/project.h>
#include <anvil/geocodes.h>
#include <anvil/macros.h>
#include <anvil/files.h>
#include <anvil/envdefs.h>
#include <anvil/projects.h>
#include <anvil/strlib.h>
#include <anvil/errcode.h>
#include <anvil/choppath.h>
#include <util/utils.h>

#define RECLEN	256

const char *descriptionFile = "$ANVIL_CONFIG/descriptions/landuse_line_format";

/* speedy atoi() function */

/* read a record with carriage return */

void readRecord( FILE		*fp,
	         void 		*record,
	         int		size,
		 AnvilBoolean	CRFlag)
{
    char	buffer[ 1024];

    if ( CRFlag) {
	fgets( buffer, sizeof( buffer), fp);
	memcpy( record, buffer, size);
    } else {
	fread( buffer, 1, 80, fp);
	memcpy( record, buffer, size);
    }
}

/* read a sequence of values from a set of lines (defined by the DLG format) */

int fscanfLinesValues( FILE	*fp,
		       char	*format,
		       int	spacing,
		       int	perLine,
		       void	*values,
		       int	valSize,
		       int	nval,
		       AnvilBoolean	CRFlag)
{
    int		lineCount, jj, kk, mm;
    char	*p, buffer[ RECLEN];
    uchar_t	*valptr = (uchar_t *)values;

    lineCount = (nval % perLine == 0) ? nval/perLine : nval/perLine + 1;
    kk = 0;
    for ( jj = 0; jj < lineCount; jj++) {
	readRecord( fp, buffer, sizeof( buffer), CRFlag);
	for ( mm = 0, p = buffer; 
	      ( mm < perLine) && ( kk < nval);
	      mm++, kk++, p += spacing, valptr += valSize) {
	    if ( sscanf( p, format, valptr) != 1) {
		fprintf( stderr, "failed to read value #%d\n", kk); 
		return( FALSE);
	    }
	}
    }
    return( TRUE);
}

void skipLines( FILE	*fp,
		int	AmNumLines,
		AnvilBoolean	CRFlag)
{
    int		ii;
    char	buffer[ 1024];

    for ( ii = 0; ii < AmNumLines; ii++) {
	readRecord( fp, buffer, sizeof( buffer), CRFlag);
    }
}

int byLineId( tempDLGLinePtr	a,
	      tempDLGLinePtr	b)
{
    return( a->idNo - b->idNo);
}

/* read the file which maps the DLG major and minor attrbiute codes to
   the unified code */

int readDlgAttrs( char		*file,
		  dlgAttrPtr	*attrs,
		  int		*numAttrs)
{
    int		ii, jj;
    FILE	*fp;
    char	buffer[ FIELDLEN];

    if (( fp = fopen( file, "r")) == NULL) {
	return( FALSE);
    }
    if ( fscanf( fp, "%d\n", numAttrs) != 1) {
	fclose( fp);
	return( FALSE);
    }
    *attrs = ( dlgAttrPtr) malloc( sizeof( dlgAttrRec)*(*numAttrs));
    for ( ii = 0; ii < *numAttrs; ii++) {
	fgets( buffer, sizeof( buffer), fp);

/* get rid of the comment in the attribute file */

	for ( jj = 0; buffer[ jj] != '\0'; jj++) {
	    if ( buffer[ jj] == '#') {
		buffer[ jj] = '\0';
		break;
	    }
	}
	if ( sscanf( buffer, "%d %d %d\n", 
		     &(( *attrs)[ ii].major),
		     &(( *attrs)[ ii].minor),
		     &(( *attrs)[ ii].attr)) != 3) {
	    fprintf( stderr, "failed to read attributes at line %d\n", ii);
	    return( FALSE);
	}
    }
    return( TRUE);
}

/* map the major and minor DLG attribute code to the unified attribute code */

int getDlgAttr( int		major,
	        int		minor,
	        dlgAttrPtr	dlgAttrs,
	        int		numAttrs)
{
    int	jj;

    for ( jj = 0; jj < numAttrs; jj++) {
	if (( major == dlgAttrs[ jj].major) && 
	    (( minor == dlgAttrs[ jj].minor) || ( dlgAttrs[ jj].minor == -1))) {
	    return( dlgAttrs[ jj].attr);
	}
    }
    return( -1);
}

/**************************************************************************
 * 
 * Description:
 *	read an optional format DLG file and write it out to separate 
 *	vector and polygon files
 * Arguments:
 *	infile:	input Optional format DLG file.
 *	dlgAttrFile: attribute file which maps DLG major and minor codes
 *		     to a single integer code.
 *	file: output file.
 *	projFile: file containing USGS projection information.
 *
 **************************************************************************/

AnvilBoolean readOptionalDlgData( char		*infile,
			     char		*dlgAttrFile,
			     char		*file,
			     char		*projFile)
{
    FILE		*fp, *fpOut;
    int			ii, jj, kk, mm, numNodes, numAreas, AmNumLines, lineCount,
			numAttCodes, numCoords, totalLines, length,
			totalCoords, attr, numAttrAreas, numAttrLines, 
			fdOut, zone, numAttrs, numAreaLines, numNodeLines;
    char		vecfile[ PATHLEN], vechdr[ PATHLEN], 
			polyfile[ PATHLEN], polyhdr[ PATHLEN], buffer[ RECLEN],
			*title;
    dlgHead1Rec		header1;
    dlgHead2Rec		header2;
    dlgHead3Rec		header3;
    dlgHead4Rec		header4;
    dlgHead5thru9Rec	header5thru9[ 5];
    dlgHead10Rec	header10;
    ctlPtRec		ctlPt[ 4];
    dataCategoryRec	dataCategory;
    nodeAreaRec		nodeArea;
    lineRec		line;
    tempDLGPolyPtr	tempPolys;
    tempDLGLinePtr	tempLines, foundLine;
    tempDLGLineRec	targetLine;
    char		**projArgs;
    PJ			*pj;
    UV			uv;
    dlgLinePtr		lineList;
    dlgPolyPtr		polyList;
    dlgAttrPtr		dlgAttrs;
    float		xmin, ymin, xmax, ymax;
    AnvilBoolean		CRFlag = TRUE, pipeFlag;

    /* AmOpenReadFilePointer checks the extension of the file and reads it 
       using either gunzip or uncompress if the extension is .Z or .gz */

    if (( fp = AmOpenReadFilePointer( infile, &pipeFlag)) == NULL) {
	fprintf( stderr, "failed to open DLG file %s\n", infile);
	return( FALSE);
    }

    /* read the DLG atribute file into the DLG attribute vector */

    if ( !readDlgAttrs( dlgAttrFile, &dlgAttrs, &numAttrs)) {
	fprintf( stderr, "can't read DLG attribute code file %s\n", 
		 dlgAttrFile);
	return( FALSE);
    }

/* find out whether the file has carriage returns at the end of each record, 
   and start over */

    if ( fread( buffer, 1, sizeof( buffer), fp) != sizeof( buffer)) {
	fprintf( stderr, "failed to read headerBuffer from %s\n", infile);
	exit( -1);
    }
    AmCloseReadFilePointer( fp, pipeFlag);
    if (( fp = AmOpenReadFilePointer( infile, &pipeFlag)) == NULL) {
	fprintf( stderr, "failed to open DLG file %s\n", infile);
	return( FALSE);
    }
    buffer[ 255] = '\0';
    CRFlag = ( strchr( buffer, '\n') != NULL);

/* read the header records up to #4, which has the utm zone for projection.
   see the USGS dlg 1:100,000K user's guides for information about the format
   of the DLG header records */

    readRecord( fp, &header1, sizeof( header1), CRFlag);
    readRecord( fp, &header2, sizeof( header2), CRFlag);
    readRecord( fp, &header3, sizeof( header3), CRFlag);
    readRecord( fp, &header4, sizeof( header4), CRFlag);
    zone = AmAtoi2( header4.refZone, sizeof( header4.refZone));
    if ( projFile == NULL) {
	projArgs = ( char **) malloc( sizeof( char *)*4);
	sprintf( buffer, "zone=%d", zone);
	projArgs[ 0] = "proj=utm";
	projArgs[ 1] = "ellps=clrk66";
	projArgs[ 2] = buffer;
	projArgs[ 3] = NULL;
    } else {
	projArgs = AmReadStrvec( projFile);
	if ( projArgs == NULL) {
	    fprintf( stderr, "failed to read projection from %s\n", projFile);
	    return( FALSE);
	}
    }
    if ( !(pj = pj_init( AmVeclen(( const char **) projArgs), 
			 ( const char **) projArgs)))  {
	fprintf( stderr, "projection initialization failed\n");
	exit( -1);
    }
    for ( ii = 0; ii < 5; ii++) {
	readRecord( fp, &header5thru9[ ii], sizeof( header5thru9[ ii]), CRFlag);
    }
    readRecord( fp, &header10, sizeof( header10), CRFlag);
    for ( ii = 0; ii < 4; ii++) {
	readRecord( fp, &ctlPt[ ii], sizeof( ctlPtRec), CRFlag);
    }
    readRecord( fp, &dataCategory, sizeof( dataCategory), CRFlag);
    numNodes = AmAtoi2( dataCategory.numRealNodes, 
		      sizeof( dataCategory.numRealNodes));
    numAreas = AmAtoi2( dataCategory.numRealAreas,
		      sizeof( dataCategory.numRealAreas));
    AmNumLines = AmAtoi2( dataCategory.numRealLines, 
		      sizeof( dataCategory.numRealLines));

/* read the nodes */

    for ( ii = 0; ii < numNodes; ii++) {
	AmPrintHatch( ii, (numNodes + numAreas + AmNumLines)*2, NHATCH);
	readRecord( fp, &nodeArea, sizeof( nodeArea), CRFlag);
	numNodeLines = AmAtoi2( nodeArea.AmNumLines, sizeof( nodeArea.AmNumLines));

/* read the lines which intersect this node */

	if ( numNodeLines > 0) {
	    lineCount = ( numNodeLines % 12 == 0) ? 
				numNodeLines/12 : numNodeLines/12 + 1;
	    for ( jj = 0; jj < lineCount; jj++) {
		readRecord( fp, buffer, sizeof( buffer), CRFlag);
	    }
	}
	numAttCodes = AmAtoi2( nodeArea.numAttCodes, 
			     sizeof( nodeArea.numAttCodes));
	if ( numAttCodes > 0) {
	    lineCount = ( numAttCodes % 6 == 0) ? 
			    numAttCodes/6 : numAttCodes/6 + 1;
	    skipLines( fp, lineCount, CRFlag);
	}
    }

/* allocate the DLG polygons and read the areas. Only accumulate areas with
   attributes */

    tempPolys = ( tempDLGPolyPtr) malloc( sizeof( tempDLGPolyRec)*numAreas);
    numAttrAreas = 0;
    for ( ii = 0; ii < numAreas; ii++) {
	AmPrintHatch( numAreas + ii, ( numNodes + numAreas + AmNumLines)*2, NHATCH);
	readRecord( fp, &nodeArea, sizeof( nodeArea), CRFlag);

/* allocate and read the lines which border this area, and put them in 
   the temporary polygon */

	numAreaLines = AmAtoi2( nodeArea.AmNumLines, sizeof( nodeArea.AmNumLines));
	numAttCodes = AmAtoi2( nodeArea.numAttCodes, 
			     sizeof( nodeArea.numAttCodes));
	tempPolys[ ii].AmNumLines = numAreaLines;
	if (( numAreaLines > 0)  && ( numAttCodes > 0)) {
	    tempPolys[ numAttrAreas].lines = 
		( int *) malloc( sizeof( int)*numAreaLines);
	    fscanfLinesValues( fp, "%d", 6, 12, tempPolys[ numAttrAreas].lines,
			       sizeof( int), numAreaLines, CRFlag);

/* read the polygon's attributes */

	    if ( numAttCodes > 0) {
		tempPolys[ numAttrAreas].numAttrs = numAttCodes;
		tempPolys[ numAttrAreas].attrs =
		    ( int *) malloc( sizeof( int)*numAttCodes);
	    } else {
		tempPolys[ numAttrAreas].numAttrs = 0;
		tempPolys[ numAttrAreas].attrs = NULL;
	    }
	    fscanfLinesValues( fp, "%d", 6, 12, tempPolys[ numAttrAreas].attrs,
			       sizeof( int), numAttCodes, CRFlag);
	    numAttrAreas++;
 
/* if no attributes, just skip over the polygon */
 
        } else if ( numAreaLines > 0) {
            lineCount = ( numAreaLines % 12 == 0) ? 
			    numAreaLines/12 : numAreaLines/12 + 1;
	    skipLines( fp, lineCount, CRFlag);
        }
    }
    tempLines = ( tempDLGLinePtr) malloc( sizeof( tempDLGLineRec)*AmNumLines);
    totalCoords = 0;
    totalLines = 0;
    for ( ii = 0; ii < AmNumLines; ii++) {
	AmPrintHatch( AmNumLines + numAreas + ii, 
		    ( numNodes + numAreas + AmNumLines)*2, NHATCH);
	readRecord( fp, buffer, sizeof( buffer), CRFlag);
	memcpy(( void *) &line, ( void *) buffer, sizeof( lineRec));

/* allocate and read the coordinates for this line */

	numCoords = AmAtoi2( line.numCoords, sizeof( line.numCoords));
	tempLines[ ii].numCoords = numCoords;
        totalCoords += numCoords;
        totalLines += numCoords - 1;
	tempLines[ ii].coords = ( AmCoordPtr) 
		malloc( sizeof( AmCoordRec)*numCoords);
	tempLines[ ii].idNo = AmAtoi2( line.id, sizeof( line.id));
        tempLines[ ii].startNode =
            AmAtoi2( line.startNode, sizeof( line.startNode));
        tempLines[ ii].endNode =
            AmAtoi2( line.endNode, sizeof( line.endNode));
	if ( numCoords > 0) {
	    fscanfLinesValues( fp, "%f", 12, 6, tempLines[ ii].coords,
			       sizeof( float), numCoords*2, CRFlag);

/* apply the inverse projection to degrees */

	    for ( jj = 0; jj < numCoords; jj++) {
		uv.u = tempLines[ ii].coords[ jj].x;
		uv.v = tempLines[ ii].coords[ jj].y;
		uv = pj_inv( uv, pj);
		tempLines[ ii].coords[ jj].x = RADTODEG( uv.u);
		tempLines[ ii].coords[ jj].y = RADTODEG( uv.v);
	    }
	}
	numAttCodes = AmAtoi2( line.numAttCodes, sizeof( line.numAttCodes));
	if ( numAttCodes > 0) {
	    tempLines[ ii].attrs = ( int *) malloc( sizeof( int)*numAttCodes*2);
	    tempLines[ ii].numAttrs = numAttCodes;
	    fscanfLinesValues( fp, "%d", 6, 12, tempLines[ ii].attrs,
			       sizeof( int), numAttCodes*2, CRFlag);
	} else {
	    tempLines[ ii].attrs = NULL;
	    tempLines[ ii].numAttrs = 0;
	}
    }

/* sort the lines by id */

    qsort( tempLines, AmNumLines, sizeof( tempDLGLineRec), ( AmSearchFn) byLineId);

/* now, we generate a list of real polygons */

    polyList = ( dlgPolyPtr) malloc( sizeof( dlgPolyRec)*numAreas);
    for ( ii = 0; ii < numAttrAreas; ii++) {
	AmPrintHatch( AmNumLines + numAreas + AmNumLines + ii, 
		    ( numNodes + numAreas + AmNumLines)*2, NHATCH);
	length = 0;

/* find all of the lines that make up the polygon, and get their lengths to
   allocate the points for the polygon */

	for ( jj = 0;
	      ( jj < tempPolys[ ii].AmNumLines) && 
	      ( tempPolys[ ii].lines[ jj] != 0); 
	      jj++) {
	    targetLine.idNo = abs( tempPolys[ ii].lines[ jj]);
	    foundLine = ( tempDLGLinePtr) 
		bsearch( &targetLine, tempLines, AmNumLines, 
			 sizeof( tempDLGLineRec), ( AmSearchFn) byLineId);
	    if ( foundLine == NULL) {
		fprintf( stderr, "failed to find line %d in area %d (241)\n", 
			    abs( tempPolys[ ii].lines[ jj]), ii);
	    } else {
		length += foundLine->numCoords;
	    }
	}

/* copy each of the points for the polygon */

	polyList[ ii].geoPoly.polygon.numPts = length;
	polyList[ ii].geoPoly.polygon.pts = 
	    ( AmCoordPtr) malloc( sizeof( AmCoordRec)*length);
	if ( tempPolys[ ii].numAttrs > 0) {
	    polyList[ ii].geoPoly.attributeCode = 
		getDlgAttr( tempPolys[ ii].attrs[ 0], tempPolys[ ii].attrs[ 1],
			    dlgAttrs, numAttrs);
	    if ( polyList[ ii].geoPoly.attributeCode == -1) {
		fprintf( stderr, 
			 "failed to read attributes %d %d polygon %d\n",
			 tempPolys[ ii].attrs[ 0], 
			 tempPolys[ ii].attrs[ 1], ii);
	    }
	    polyList[ ii].majorCode = tempPolys[ ii].attrs[ 0];
	    polyList[ ii].minorCode = tempPolys[ ii].attrs[ 1];
	} else {
	    polyList[ ii].geoPoly.attributeCode = -1;
	    polyList[ ii].majorCode = 0;
	    polyList[ ii].minorCode = 0;
	}
 
/* accumulate the points and lines for the polygon */
 
	mm = 0;
	for ( jj = 0;
	      ( jj < tempPolys[ ii].AmNumLines) && 
	      ( tempPolys[ ii].lines[ jj] != 0); 
	      jj++) {
	    targetLine.idNo = abs( tempPolys[ ii].lines[ jj]);
	    foundLine = ( tempDLGLinePtr) 
		bsearch( &targetLine, tempLines, AmNumLines, 
			 sizeof( tempDLGLineRec), ( AmSearchFn) byLineId);
	    if ( foundLine == NULL) {
		fprintf( stderr, "failed to find line %d in area %d (242)\n", 
			 abs( tempPolys[ ii].lines[ jj]), ii);
	    } else {
		if ( tempPolys[ ii].lines[ jj] >= 0) {
		    for ( kk = 0; kk < foundLine->numCoords; kk++, mm++) {
			polyList[ ii].geoPoly.polygon.pts[ mm] = 
			    foundLine->coords[ kk];
		    }
		} else {
		    for ( kk = foundLine->numCoords - 1; kk >= 0; kk--, mm++) {
			polyList[ ii].geoPoly.polygon.pts[ mm] = 
			    foundLine->coords[ kk];
		    }
		} 
	    }
	}
    }

/* allocate a vector of nodes large enough to hold all the  "real" DLG nodes
   and the nodes which are simply coordinate points in the line */
 
    lineList = ( dlgLinePtr) malloc( sizeof( dlgLineRec)*totalLines);
    numAttrLines = 0;
    for ( ii = 0; ii < AmNumLines; ii++) {
	AmPrintHatch( AmNumLines + numAreas + AmNumLines*2 + ii, 
		    ( numNodes + numAreas + AmNumLines)*2, NHATCH);
 
/* only convert lines which have attributes */

	if ( tempLines[ ii].numAttrs > 0) {
	    attr = getDlgAttr( tempLines[ ii].attrs[ 0], 
			       tempLines[ ii].attrs[ 1],
			       dlgAttrs, numAttrs);
	    if ( attr == -1) {
		fprintf( stderr, "failed to find attribute code %d %d\n",
			 tempLines[ ii].attrs[ 0], tempLines[ ii].attrs[ 1]);
	    }
 
/* create the nodes, copy the coordinates, and allocate the lists for lines */ 
 
	    for ( jj = 0; jj < tempLines[ ii].numCoords - 1; jj++) {
		lineList[ numAttrLines].geoLine.line.x0 = 
		    tempLines[ ii].coords[ jj].x;
		lineList[ numAttrLines].geoLine.line.y0 = 
		    tempLines[ ii].coords[ jj].y;
		lineList[ numAttrLines].geoLine.line.x1 = 
		    tempLines[ ii].coords[ jj + 1].x;
		lineList[ numAttrLines].geoLine.line.y1 = 
		    tempLines[ ii].coords[ jj + 1].y;
		lineList[ numAttrLines].geoLine.attributeCode = attr;
		lineList[ numAttrLines].majorCode = tempLines[ ii].attrs[ 0];
		lineList[ numAttrLines].minorCode = tempLines[ ii].attrs[ 1];
		numAttrLines++;
	    }
	}
    }
AN_DEBUG( 99, ( "numAreas = %d, AmNumLines = %d\n", numAreas, AmNumLines));
AN_DEBUG( 99, ( "numAttrAreas = %d, numAttrLines = %d\n", 
		numAttrAreas, numAttrLines));

    xmin = MY_HUGE;
    ymin = MY_HUGE;
    xmax = -MY_HUGE;
    ymax = -MY_HUGE;
    for ( ii = 0; ii < numAttrAreas; ii++) {
	for ( jj = 0; jj < polyList[ ii].geoPoly.polygon.numPts; jj++) {
	    if ( polyList[ ii].geoPoly.polygon.pts[ jj].x < xmin) {
		xmin = polyList[ ii].geoPoly.polygon.pts[ jj].x;
	    }
	    if ( polyList[ ii].geoPoly.polygon.pts[ jj].y > ymin) {
		ymin = polyList[ ii].geoPoly.polygon.pts[ jj].y;
	    }
	    if ( polyList[ ii].geoPoly.polygon.pts[ jj].x < xmax) {
		xmax = polyList[ ii].geoPoly.polygon.pts[ jj].x;
	    }
	    if ( polyList[ ii].geoPoly.polygon.pts[ jj].y > ymax) {
		ymax = polyList[ ii].geoPoly.polygon.pts[ jj].y;
	    }
	}
    }

/* take the title from the DLG file */

    strncpy( buffer, header2.mapName, sizeof( header2.mapName));
    buffer[ sizeof( header2.mapName)] = '\0';
    title = AmElimLeadWspace( buffer);
    title = AmElimTrailWspace( title);

/* create the data file and header for the polygon files */

    sprintf( polyfile, "%s.poly.dat", file);
    sprintf( polyhdr, "%s.poly.hdr", file);
    if (( fdOut = open( polyfile, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", polyfile);
	return( FALSE);
    }
    if (( fpOut = fopen( polyhdr, "w")) == NULL) {
	fprintf( stderr, "failed to open header file %s\n", polyhdr);
	return( FALSE);
    }

    /* although the actual data file is written out with the fully qualified path
       name, the filename is put in without the path, because the AmReadHeader()
       function takes the path for the header file and prefixes it on the data 
       file */

    fprintf( fpOut, "filename:	%s\n", AmGetRootName( polyfile));
    fprintf( fpOut, "title:		%s polygon data\n", title);
    fprintf( fpOut, "rows:		%d\n", numAttrAreas);
    fprintf( fpOut, "xvariable:	longitude\n");
    fprintf( fpOut, "xunits:		arcdeg\n");
    fprintf( fpOut, "yvariable:	latitude\n");
    fprintf( fpOut, "yunits:		arcdeg\n");
    fprintf( fpOut, "type:		polygonPlot\n");
    fprintf( fpOut, "size:		%d\n", sizeof( dlgPolyRec));
    fprintf( fpOut, "xmin:		%f\n", xmin);
    fprintf( fpOut, "ymin:		%f\n", ymin);
    fprintf( fpOut, "xmax:		%f\n", xmax);
    fprintf( fpOut, "ymax:		%f\n", ymax);
    fprintf( fpOut, "attributeMap:	%s\n", dlgAttrFile);
#if BYTE_ORDER == LITTLE_ENDIAN
    fprintf( fpOut, "byteOrder:	littleEndian\n");
#else 
    fprintf( fpOut, "byteOrder:	bigEndian\n");
#endif
    fprintf( fpOut, "descriptionFile:	$ANVIL_CONFIG/descriptions/dlg_format\n");
    AmCloseReadFilePointer( fp, pipeFlag);
    for ( ii = 0; ii < numAttrAreas; ii++) {
	write( fdOut, &polyList[ ii], sizeof( polyList[ ii]));
	write( fdOut, polyList[ ii].geoPoly.polygon.pts, 
		      polyList[ ii].geoPoly.polygon.numPts*sizeof( AmCoordRec));
    }
    close( fdOut);

/* find the extents of the vectors to write the extents for the header file */

    xmin = MY_HUGE;
    ymin = MY_HUGE;
    xmax = -MY_HUGE;
    ymax = -MY_HUGE;
    for ( ii = 0; ii < numAttrLines; ii++) {
	if ( lineList[ ii].geoLine.line.x0 < xmin) {
	    xmin = lineList[ ii].geoLine.line.x0;
	}
	if ( lineList[ ii].geoLine.line.x1 < xmin) {
	    xmin = lineList[ ii].geoLine.line.x1;
	}
	if ( lineList[ ii].geoLine.line.y0 < ymin) {
	    ymin = lineList[ ii].geoLine.line.y0;
	}
	if ( lineList[ ii].geoLine.line.y1 < ymin) {
	    ymin = lineList[ ii].geoLine.line.y1;
	}
	if ( lineList[ ii].geoLine.line.x0 > xmax) {
	    xmax = lineList[ ii].geoLine.line.x0;
	}
	if ( lineList[ ii].geoLine.line.x1 > xmax) {
	    xmax = lineList[ ii].geoLine.line.x1;
	}
	if ( lineList[ ii].geoLine.line.y0 > ymax) {
	    ymax = lineList[ ii].geoLine.line.y0;
	}
	if ( lineList[ ii].geoLine.line.y1 > ymax) {
	    ymax = lineList[ ii].geoLine.line.y1;
	}
    }

/* write out the vectors */

    sprintf( vecfile, "%s.vec.dat", file);
    sprintf( vechdr, "%s.vec.hdr", file);
    if (( fdOut = open( vecfile, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", vecfile);
	return( FALSE);
    }
    if (( fpOut = fopen( vechdr, "w")) == NULL) {
	fprintf( stderr, "failed to open header file %s\n", vechdr);
	return( FALSE);
    }
    fprintf( fpOut, "filename:	%s\n", AmGetRootName( vecfile));
    fprintf( fpOut, "title:		%s vector data\n", title);
    fprintf( fpOut, "rows:		%d\n", numAttrLines);
    fprintf( fpOut, "xunits:		arcdeg\n");
    fprintf( fpOut, "xvariable:	longitude\n");
    fprintf( fpOut, "yunits:		arcdeg\n");
    fprintf( fpOut, "yvariable:	latitude\n");
    fprintf( fpOut, "size:		%d\n", sizeof( dlgLineRec));
    fprintf( fpOut, "xmin:		%f\n", xmin);
    fprintf( fpOut, "ymin:		%f\n", ymin);
    fprintf( fpOut, "xmax:		%f\n", xmax);
    fprintf( fpOut, "ymax:		%f\n", ymax);
    fprintf( fpOut, "type:		linePlot\n");
    fprintf( fpOut, "attributeMap:	%s\n", dlgAttrFile);
    fprintf( fpOut, "fileSource:	DLG\n");
    fprintf( fpOut, "colorsType:	int\n");
    fprintf( fpOut, "colorsOffset:	%d\n", sizeof( AmLineRec));
    fprintf( fpOut, "sourceFile:	%s\n", infile);
    fprintf( fpOut, "description:	%s\n", descriptionFile);
#if BYTE_ORDER == LITTLE_ENDIAN
    fprintf( fpOut, "byteOrder:	littleEndian\n");
#else 
    fprintf( fpOut, "byteOrder:	bigEndian\n");
#endif
    fprintf( fpOut, "descriptionFile:	$ANVIL_CONFIG/descriptions/dlg_format\n");
    fclose( fpOut);
    write( fdOut, lineList, sizeof( dlgLineRec)*numAttrLines);
    close( fdOut);
    return( TRUE);
}


int main( int argc, char **argv)
{
    char	attrpath[ PATHLEN], *env, *projFile;

    if ( argc < 3) {
	fprintf( stderr, 
		 "usage: %s infile outfile [attr-file] [projection]\n", 
		 argv[ 0]);
	exit( -1);
    }

/* check for a mapping file on the command line.  If there isn't one, then
   use the default from the environment variable */

    if ( argc == 3) {
	env = getenv( CONFIG_ENV);
	if ( env == NULL) {
	    fprintf( stderr, "%s environment variable not set\n", CONFIG_ENV);
	    exit( -1);
	}
	sprintf( attrpath, "%s/mappings/dlg_trans_mapping", env);
    } else {
	strcpy( attrpath, argv[ 3]);
    }

/* see if there's a map projection file on the command line */

    if ( argc == 5) {
	projFile = argv[ 4];
    } else {
	projFile = NULL;
    }
    if ( !readOptionalDlgData( argv[ 1], attrpath, argv[ 2], projFile)) {
	fprintf( stderr, "failed to translate optional DLG data\n");
	exit( -1);
    }
}
