#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <anvil/project.h>
#include <anvil/strlib.h>
#include <anvil/datatype.h>
#include <anvil/tagstrs.h>
#include <anvil/macros.h>
#include <anvil/choppath.h>
#include <anvil/fns.h>

#define MAXCOLS	256

int main( int argc, char *argv[])
{
    char		buffer[ 16384], *titles[ MAXCOLS], colbuf[ 1024], 
			*valstr, *c, hdrpath[ PATHLEN],
			path[ PATHLEN];
    uchar_t		outbuf[ 4096], *outptr;
    int			numCols, rows, size, ii, lengthList[ MAXCOLS], cols=0,
			fdOut, xindex = 0, yindex = 1, offset;
    FILE		*fp, *hdrFp;
    AmDataTypeEnum	typeList[ MAXCOLS];
    float		xmin, ymin, xmax, ymax, x, y;

    if ( argc < 5) {
	fprintf( stderr, "usage: %s infile xcol# ycol# outfile\n", argv[ 0]);
	exit( -1);
    }
    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }

/* get the indexes for the x, y values, which are reordered to be written
   out AmFirst */

    xindex = atoi( argv[ 2]) - 1;
    yindex = atoi( argv[ 3]) - 1;

/* get the path for the data file and the header, and open them */

    sprintf( path, "%s.dat", argv[ 4]);
    sprintf( hdrpath, "%s.hdr", argv[ 4]);
    if (( fdOut = open( path, O_WRONLY | O_CREAT, PERMS)) < 0) {
        fprintf( stderr, "failed to open output file %s\n", path);
        exit( -1);
    }
    if (( hdrFp = fopen( hdrpath, "w")) == NULL) {
        fprintf( stderr, "failed to open output header %s\n", hdrpath);
        exit( -1);
    }

/* get the buffers containing the titles. */

    numCols = 0;
    if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	fprintf( stderr, "failed to read title\n");
	exit( -1);
    }
    buffer[ strlen( buffer) - 1] = '\0';
    numCols = 0;
    for ( valstr = strtok( buffer, "\t"); 
	  valstr != NULL; 
	  valstr = strtok( NULL, "\t")) {
	if ( sscanf( valstr, "%s", colbuf) > 0) {
	    titles[ numCols++] = strdup( colbuf);
	}
    }

/* now we scan the entire file, and since we don't know the data type, we
   try to figure it out for each column. We start with integer, and if the
   column is [-0-9], then it stays that way.  If the column is of the form
   [-0-9]*.[0-9]*, then the assumed type is a float.  If any other character
   comes in, then the assumed type is string */

    for ( ii = 0; ii < numCols; ii++) {
	typeList[ ii] = AmINT;
	lengthList[ ii] = 4;
    }
    rows = 0;
    do {
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    break;
	}
	buffer[ strlen( buffer) - 1] = '\0';
	for ( valstr = strtok( buffer, "\t"), cols = 0; 
	      valstr != NULL; 
	      valstr = strtok( NULL, "\t"), cols++) {
	    sscanf( valstr, "%s", colbuf);

/* if it's already a string, we don't try harder */

	    if ( typeList[ cols] != AmSTRING) {
		for ( c = colbuf; *c != '\0'; c++) {
		    if ( !isdigit( *c) && ( *c != '-')) {
			if ( *c != '.') {
			    typeList[ cols] = AmSTRING;
			    lengthList[ cols] = strlen( valstr);
			    break;
			} else {
			    typeList[ cols] = AmFLOAT;
			}
		    }
		}
	    }
	}

/* if the designated x,y coordianates aren't floats or ints, then flag an
   error */

	if (( typeList[ xindex] != AmINT) && ( typeList[ xindex] != AmFLOAT)) {
	    fprintf( stderr, 
		     "non-numeric values in xindex (%d) at row %d\n", 
		     xindex + 1, rows);
	    break;
	}
	if (( typeList[ yindex] != AmINT) && ( typeList[ yindex] != AmFLOAT)) {
	    fprintf( stderr, 
		     "non-numeric values in yindex (%d) at row %d\n", 
		     yindex + 1, rows);
	    break;
	}
	rows++;
    }  while ( TRUE);
    for ( ii = 0; ii < numCols; ii++) {
	printf( "%s %d %d\n", titles[ ii], typeList[ ii], lengthList[ ii]);
    }
    fclose( fp);
    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }

/* AmPointRec requires that the AmFirst two values be floats */

    typeList[ xindex] = AmFLOAT;
    lengthList[ xindex] = sizeof( float);
    typeList[ yindex] = AmFLOAT;
    lengthList[ yindex] = sizeof( float);

/* skip over the titles */

    fgets( buffer, sizeof( buffer), fp);

/* scan and conver the values to binary in a row and write the row, and track
   the extents of the x,y data */

    xmin = MY_HUGE;
    ymin = MY_HUGE;
    xmax = -MY_HUGE;
    ymax = -MY_HUGE;
    rows = 0;
    size = 0;
    for ( ii = 0; ii < cols; ii++) {
	size += lengthList[ ii];
    }
    do {
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    break;
	}
	buffer[ strlen( buffer) - 1] = '\0';
	for ( valstr = strtok( buffer, "\t"), 
		  cols = 0, outptr = &outbuf[ sizeof( float)*2]; 
	      valstr != NULL; 
	      valstr = strtok( NULL, "\t"), 
		  outptr += lengthList[ cols++]) {
	    sscanf( valstr, "%s", colbuf);

/* check for the xindex or yindex AmFirst, and hold onto the x and y */ 

	    if ( cols == xindex) {
		sscanf( valstr, "%f", &x);
		memcpy( outbuf, &x, sizeof( float));
	    } else if ( cols == yindex) {
		sscanf( valstr, "%f", &y);
		memcpy( &outbuf[ sizeof( float)], &y, sizeof( float));
	    } else if ( typeList[ cols] == AmFLOAT) {
		sscanf( valstr, "%f", ( float *) outptr);
	    } else if ( typeList[ cols] == AmINT) {
		sscanf( valstr, "%d", ( int *) outptr);
	    } else if ( typeList[ cols] == AmSTRING) {
		strncpy(( char *) outptr, valstr, lengthList[ cols]);
	    } else {
		fprintf( stderr, "bizarro type value col = %d, row = %d\n",
			 cols, rows);
		exit( -1);
	    }
	}

/* track the extents */

	if ( x < xmin) {
	    xmin = x;
	}
	if ( y < ymin) {
	    ymin = y;
	}
	if ( x > xmax) {
	    xmax = x;
	}
	if ( y > ymax) {
	    ymax = y;
	}

/* write the output buffer */

	if ( write( fdOut, outbuf, size) < size) {
	    fprintf( stderr, "failed to write output line %d\n", rows);
	    exit( -1);
	}
	rows++;
    } while ( TRUE);

/* now print out all the data */

    fprintf( hdrFp, "%s:		%s\n", AmNfilename, 
					       AmGetRootName( path));
    fprintf( hdrFp, "%s:		%d\n", AmNrows, rows);
    fprintf( hdrFp, "%s:		%d\n", AmNsize, size);
    fprintf( hdrFp, "%s:		point\n", AmNtype);
    fprintf( hdrFp, "%s:		%f\n", AmNxmin, xmin);
    fprintf( hdrFp, "%s:		%f\n", AmNymin, ymin);
    fprintf( hdrFp, "%s:		%f\n", AmNxmax, xmax);
    fprintf( hdrFp, "%s:		%f\n", AmNymax, ymax);

/* a hack for latitude longitude */

    if (( xmin >= -180.0) && ( xmax < 180.0) && 
	( ymin > -90.0) && ( ymax < 90.0)) {
	fprintf( hdrFp, "xunits:	longitude_arcdeg\n");
	fprintf( hdrFp, "yunits:	latitude_arcdeg\n");
    } else {
	fprintf( hdrFp, "xunits:	meter\n");
	fprintf( hdrFp, "yunits:	meter\n");
    }

/* print out the attribute names, offsets, sizes and data types for each of 
   the non-xy fields */

    fprintf( hdrFp, "%s:	", AmNattributes);
    for ( ii = 0; ii < cols; ii++) {
	if (( ii != xindex) && ( ii != yindex)) {
	    fprintf( hdrFp, "%s ", titles[ ii]);
	}
    }
    fprintf( hdrFp, "\n");
    offset = sizeof( float)*2;
    for ( ii = 0; ii < cols; ii++) {
	if (( ii != xindex) && ( ii != yindex)) {
	    fprintf( hdrFp, "%s.%s:	%s\n", 
		     titles[ ii], AmNdataType, 
		     AmDataTypeEnumToStr( typeList[ ii]));
	    fprintf( hdrFp, "%s.%s:	%d\n", 
		     titles[ ii], AmNoffset, offset);
	    fprintf( hdrFp, "%s.%s:	%d\n", 
		     titles[ ii], AmNsize, lengthList[ ii]);
	    offset += lengthList[ ii];
	}
    }
    return 0;
}
