#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */






#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <anvil/project.h>
#if defined(_WIN32)
#include <anvil/ntresource.h>
#else
#include <anvil/xresource.h>
#endif
#include <anvil/datafmt.h>
#include <anvil/macros.h>
#include <anvil/datatype.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/tagstrs.h>
#include <anvil/fns.h>
#include <anvil/choppath.h>
#include <util/utils.h>
#include <util/misc.h>

#define BUFSIZE	1024


int cmpInt( int *a, int *b)
{
    return( *a - *b);
}

int main( int argc, char *argv[])
{
    int			fdOut, ii, jj, fdIn, rows, size, 
   			skipBytes, *filterIds, numIds,
			lookupOffset, ival, count, newrows=0, buflen;
    char 		*fields[] = { "filename", "rows", NULL },
			dataPath[ PATHLEN], hdrPath[ PATHLEN], 
			*kind, *lookupTypeStr, *sourceTypeStr;
    uchar_t		*ptr, *buffer;
    FILE		*hdrFp;
    AmDataTypeEnum	lookupType;

    if ( argc < 5) {
	fprintf( stderr, "usage %s inheader infile outfile filterids\n", 
		 argv[ 0]);
	exit( -1);
    }
    if ( AmEndswith( argv[ 2], ".gz") || AmEndswith( argv[ 2], ".Z") ||
         AmEndswith( argv[ 2], ".z")) {
        fprintf( stderr, "filter may not be run with compressed files\n");
        exit( -1);
    }
    AmInitUnits();
 
/* construct the paths for the output files. */
 
    sprintf( dataPath, "%s.dat", argv[ 3]);
    sprintf( hdrPath, "%s.hdr", argv[ 3]);

/* read the file of filter ids and get the number of ids, sort them to make
   sure they're in ascending order */

    if (( filterIds = AmReadIntVec( argv[ 4])) == NULL) {
	fprintf( stderr, "failed to read filter ID file %s\n", argv[ 4]);
	exit( -1);
    }
    numIds = AmNumLines( argv[ 4]);
    qsort( filterIds, numIds, sizeof( int), ( AmSearchFn) cmpInt);

/* open the header and get the essentials */

#if defined(_WIN32)
    AmNTResourceDbClass db( argv[ 1]);
#else
    AmXResourceDbClass db( argv[ 1]);
#endif
    if ( !db.GetResource( "type", &kind)) {
	fprintf( stderr, "failed to read kind from file\n");
	exit( -1);
    }
 
/* open the input header, and output files */
 
    if (( fdIn = open( argv[ 2], O_RDONLY)) < 0) {
        fprintf( stderr, "failed to open input file %s\n", argv[ 2]);
        exit( -1);
    }
    if (( fdOut = open( dataPath, O_WRONLY | O_CREAT, PERMS)) < 0) {        fprintf( stderr, "failed to open output file %s\n", dataPath);
        exit( -1);
    }
    if (( hdrFp = fopen( hdrPath, "w")) == NULL) {
        fprintf( stderr, "failed to open output header %s\n", hdrPath);
        exit( -1);
    }

/* get the lookup type, lookup offset, rows, and size from the header */

    if ( !db.GetResource( AmNcolorsType, &lookupTypeStr)) {
	fprintf( stderr, "failed to read lookup type from file\n");
	exit( -1);
    }
    lookupType = AmStrToDataTypeEnum( lookupTypeStr);
    if ( !db.GetResource( AmNcolorsOffset, &lookupOffset)) {
	fprintf( stderr, "failed to read lookup offset from file\n");
	exit( -1);
    }
    if ( !db.GetResource( AmNrows, &rows)) {
	fprintf( stderr, "failed to read rows from file\n");
	exit( -1);
    }
    if ( !db.GetResource( AmNsize, &size)) {
	fprintf( stderr, "failed to read size from file\n");
	exit( -1);
    }
    if ( !db.GetResource( AmNskipBytes, &skipBytes)) {
	skipBytes = 0;
    }
    if ( db.GetResource( AmNsourceType, &sourceTypeStr) &&
	 !strcmp( sourceTypeStr, "pyramidSource")) {
	fprintf( stderr, "%s cannot be run on tiled data\n", argv[ 0]);
	exit( -1);
    } 

    if ( !strcmp( kind, AmNlinePlot)) {
	buffer = ( uchar_t *) malloc( size*BUFSIZE);
	if ( buffer == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", size*BUFSIZE);
	    exit( -1);
	}

/* skip any header bytes if there are any */

	if ( skipBytes != 0) {
	    if ( lseek( fdIn, skipBytes, SEEK_SET) < 0) {
		fprintf( stderr, "failed to seek %d bytes\n", skipBytes);
		exit( -1);
	    }
	}

/* read the data in BUFISZE*size chunks for efficiency */

	newrows = 0;
	for ( ii = 0; ii < rows; ii += BUFSIZE) {	
	    buflen = MIN( rows - ii, BUFSIZE);
	    if ( read( fdIn, buffer, buflen*size) < buflen*size) {
		fprintf( stderr, "failed to read line buffer at %d\n", 
			 ii*BUFSIZE);
		exit( -1);
	    }
	    count = 0;
	    for ( jj = 0, ptr = buffer; jj < buflen; jj++, ptr += size) {

/* copy the filtered lines in place, and write out the block */

		ival = AmGetLookupVal(( char *) ptr, lookupType, lookupOffset);
		printf ("ival: %d\n", ival);
		if ( AmInTable( ival, filterIds, numIds)) {
		    memcpy( &buffer[ size*count], ptr, size);
		    count++;
		}
	    }
	    if ( write( fdOut, buffer, count*size) < count*size) {
		fprintf( stderr, "failed to write line buffer at %d\n", 
			 ii*BUFSIZE);
		exit( -1);
	    }
	    newrows += count;
	}
    }

/* write out the new header information and close the files */

    fprintf( hdrFp, "filename:      %s\n", AmGetRootName( dataPath));
    fprintf( hdrFp, "rows:          %d\n", newrows);
    fprintf( hdrFp, "sourceFile:    %s\n", argv[ 1]);
    fprintf( hdrFp, "note:    	filter %s\n", argv[ 4]);
    AmDumpHeaderExcept( argv[ 1], fields, hdrFp);
    fclose( hdrFp);
    close( fdIn);
    close( fdOut);
}
