#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */





#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <anvil/project.h>
#include <anvil/gif.h>
#include <anvil/choppath.h>
#include <util/utils.h>


const char *descriptionFile = "$ANVIL_CONFIG/descriptions/unsignedChar_image";

int main( int argc, char *argv[])
{
    FILE		*fp, *hdrFp, *paletteFp;
    int			fd, ii, jj, rows, cols, numColors; 
    AnvilBoolean		flags[ MAXCOLORMAPSIZE];
    char		dataFile[ PATHLEN], headerFile[ PATHLEN], 
			paletteFile[ PATHLEN];
    unsigned char	*image, localColorMap[ 3][ MAXCOLORMAPSIZE];

    if ( argc < 3) {
	fprintf( stderr, "usage: %s infile outfile\n", argv[ 0]);
	exit( -1);
    }
    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open GIF file %s\n", argv[ 1]);
    }

/* generate the data, header, and palette output paths */

    sprintf( dataFile, "%s.dat", argv[ 2]);
    sprintf( headerFile, "%s.hdr", argv[ 2]);
    sprintf( paletteFile, "%s.palette", argv[ 2]);
    if (( hdrFp = fopen( headerFile, "w")) == NULL) {
        fprintf( stderr, "failed to open header file %s", headerFile);
        exit( -1);
    }
    if (( paletteFp = fopen( paletteFile, "w")) == NULL) {
        fprintf( stderr, "failed to open palette file %s", paletteFile);
        exit( -1);
    }
    if (( fd = open( dataFile, O_WRONLY | O_CREAT, PERMS)) < 0) {
        fprintf( stderr, "failed to open data file %s", dataFile);
        exit( -1);
    }

/* read the GIF file */

    if ( !ReadGIF( fp, 1, &image, &cols, &rows, localColorMap, &numColors)) {
        fprintf( stderr, "failed to read GIF file %s", argv[ 1]);
        exit( -1);
    }

/* write out the image */

    write( fd, image, rows*cols);
    close( fd);

/* write out the header file */

    fprintf( hdrFp, "filename:     %s\n", AmGetRootName( dataFile));
    fprintf( hdrFp, "rows:         %d\n", rows);
    fprintf( hdrFp, "cols:         %d\n", cols);
    fprintf( hdrFp, "palette:      %s\n", paletteFile);
    fprintf( hdrFp, "dataType:     char\n");
    fprintf( hdrFp, "size:     	   1\n");
    fprintf( hdrFp, "type:     	   imagePlot\n");
    fprintf( hdrFp, "fileSource:   GIF\n");
    fprintf( hdrFp, "description:  %s\n", descriptionFile);
    fclose( hdrFp);

/* write out the colormap for all the pixels that are actually used in 
   the image */

    for ( ii = 0; ii < MAXCOLORMAPSIZE; ii++) {
	flags[ ii] = FALSE;
    }
    for ( ii = 0; ii < rows; ii++) {
	for ( jj = 0; jj < cols; jj++) {
	    flags[ image[ ii*cols + jj]] = TRUE;
	}
    }
    for ( ii = 0; ii < MAXCOLORMAPSIZE; ii++) {
	if ( flags[ ii]) {
	    fprintf( paletteFp, "%d %d %d %d\n",
		     ii,
		     localColorMap[ 0][ ii],
		     localColorMap[ 1][ ii],
		     localColorMap[ 2][ ii]);
	} 
    }
    fclose( paletteFp);
}
