#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <anvil/project.h>
#include <anvil/choppath.h>

#define MAXBANDS		10
#define PROJECTION_NAME		"MAP_PROJECTION_NAME"
#define PROJECTION_ZONE 	"USGS_MAP_ZONE"
#define HORIZONTAL_DATUM 	"HORIZONTAL_DATUM"
#define BITS_PER_PIXEL		"BITS_PER_PIXEL"
#define PIXELS_PER_LINE		"PIXELS_PER_LINE"
#define LINES_PER_DATA_FILE	"LINES_PER_DATA_FILE"
#define UPPER_LEFT_CORNER	"UPPER_LEFT_CORNER"
#define UPPER_RIGHT_CORNER	"UPPER_RIGHT_CORNER"
#define LOWER_RIGHT_CORNER	"LOWER_RIGHT_CORNER"
#define LOWER_LEFT_CORNER	"LOWER_LEFT_CORNER"
#define PIXEL_SPACING_UNITS	"PIXEL_SPACING_UNITS"


int main( int argc, char *argv[])
{
    FILE	*fpIn, *hdrFp;
    int		cols=0, rows=0, ii, jj, zone=0, fdIn1, fdIn2, fdIn3, fdOut, bitsPerPixel=0;
    float	ulX, ulY, llX, llY, urX, urY, lrX, lrY, 
		ulLongitude, ulLatitude, urLongitude, urLatitude, 
		llLongitude, llLatitude, lrLongitude, lrLatitude;
    char	units[ 128], buffer[ 512], *p,
		outpath[ PATHLEN], 
		outhdr[ PATHLEN], horizontalDatum[ 128], tag[ 128], value[ 256],
		ulLongDir, ulLatDir, urLongDir, urLatDir,
		llLongDir, llLatDir, lrLongDir, lrLatDir;
    uchar_t	*band1, *band2, *band3;
    ulong_t	*outband;

    if ( argc < 4) {
	fprintf( stderr, "usage: %s inhdr infile [infile2][infile3] outfile\n",
		 argv[ 0]);
	exit( -1);
    }
    if (( fpIn = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open input header %s\n", argv[ 1]);
	exit( -1);
    }

/* scan for the bands that we're interested in from the LANDSAT data */

    do {
	if ( fgets( buffer, sizeof( buffer), fpIn) == NULL) {
	    break;
	}
	sscanf( buffer, "%[A-Za-z_]=%s", tag, value);
	if ( !strcmp( tag, PROJECTION_NAME)) {
	} else if ( !strcmp( tag, PROJECTION_ZONE)) {
	    zone = atoi( value);
	} else if ( !strcmp( tag, HORIZONTAL_DATUM)) {	
	    strcpy( horizontalDatum, value);
	} else if ( !strcmp( tag, BITS_PER_PIXEL)) {
	    bitsPerPixel = atoi( value);
	} else if ( !strcmp( tag, PIXELS_PER_LINE)) {
	    cols = atoi( value);
	} else if ( !strcmp( tag, LINES_PER_DATA_FILE)) {
	    rows = atoi( value);
	} else if ( !strcmp( tag, UPPER_LEFT_CORNER)) {
	    sscanf( value, "%f%c,%f%c,%f,%f", 
		    &ulLongitude, &ulLongDir, &ulLatitude, &ulLatDir,
		    &ulX, &ulY);
	} else if ( !strcmp( tag, UPPER_RIGHT_CORNER)) {
	    sscanf( value, "%f%c,%f%c,%f,%f", 
		    &urLongitude, &urLongDir, &urLatitude, &urLatDir,
		    &urX, &urY);
	} else if ( !strcmp( tag, LOWER_RIGHT_CORNER)) {
	    sscanf( value, "%f%c,%f%c,%f,%f", 
		    &lrLongitude, &lrLongDir, &lrLatitude, &lrLatDir,
		    &lrX, &lrY);
	} else if ( !strcmp( tag, LOWER_LEFT_CORNER)) {
	    sscanf( value, "%f%c,%f%c,%f,%f", 
		    &llLongitude, &llLongDir, &llLatitude, &llLatDir,
		    &llX, &llY);
	}  else if ( !strcmp( tag, PIXEL_SPACING_UNITS)) {
	    strcpy( units, value);
	    for ( p = units; isalpha( *p); *p = tolower( *p), p++);
	    *p = '\0';
	}
    } while ( TRUE);
AN_DEBUG( 60, ( "zone = %d, horizontalDatum = %s\n", zone, horizontalDatum));
AN_DEBUG( 60, ( "bitsPerPixel = %d, rows = %d, cols = %d\n", 
		bitsPerPixel, rows, cols));

/* if we're just creating an image from one band, then we just write a header
   for that file */

    if ( argc == 4) {
	sprintf( outhdr, "%s.hdr", argv[ 3]);
	if (( hdrFp = fopen( outhdr, "w")) == NULL) {
	    fprintf( stderr, "failed to open output header %s\n", outhdr);
	    exit( -1);
	}
	sprintf( outpath, "%s.dat", argv[ 3]);
	fprintf( hdrFp, "filename:	%s\n", AmGetRootName( outpath));
	fprintf( hdrFp, "rows:		%d\n", rows);
	fprintf( hdrFp, "cols:		%d\n", cols);
	fprintf( hdrFp, "xmin:		%.2f\n", llX);
	fprintf( hdrFp, "ymin:		%.2f\n", llY);
	fprintf( hdrFp, "xmax:		%.2f\n", urX);
	fprintf( hdrFp, "ymax:		%.2f\n", urY);
	fprintf( hdrFp, "size:		1\n");
	fprintf( hdrFp, "dataType:	char\n");
	fprintf( hdrFp, "type:		imagePlot\n");
	fprintf( hdrFp, "xvariable:	distance\n");
	fprintf( hdrFp, "xunits:	%s\n", units);
	fprintf( hdrFp, "yvariable:	distance\n");
	fprintf( hdrFp, "yunits:	%s\n", units);
	fprintf( hdrFp, "fileSource:	landsat\n");
    } else {

/* we're compositing an image from 3 images ( selected by band) */

	if (( fdIn1 = open( argv[ 2], O_RDONLY)) < 0) {
	    fprintf( stderr, "failed to open input file %s\n", argv[ 2]);
	    exit( -1);
	}
	if (( fdIn2 = open( argv[ 3], O_RDONLY)) < 0) {
	    fprintf( stderr, "failed to open input file %s\n", argv[ 3]);
	    exit( -1);
	}
	if (( fdIn3 = open( argv[ 4], O_RDONLY)) < 0) {
	    fprintf( stderr, "failed to open input file %s\n", argv[ 4]);
	    exit( -1);
	}
	sprintf( outpath, "%s.dat", argv[ 5]);
	if (( fdOut = open( outpath, O_WRONLY | O_CREAT, PERMS)) < 0) {
	    fprintf( stderr, "failed to open output file %s\n", outpath);
	    exit( -1);
	}
	band1 = ( uchar_t *) malloc( cols);
	if ( band1 == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", cols);
	    exit( -1);
	}
	band2 = ( uchar_t *) malloc( cols);
	if ( band2 == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", cols);
	    exit( -1);
	}
	band3 = ( uchar_t *) malloc( cols);
	if ( band3 == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", cols);
	    exit( -1);
	}
	outband = ( ulong_t *) malloc( cols*sizeof( ulong_t));
	if ( outband == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", 
		     cols*sizeof( ulong_t));
	    exit( -1);
	}
	for ( ii = 0; ii < rows; ii++) {
	    if ( read( fdIn1, band1, cols) < cols) {
		fprintf( stderr, 
			 "failed to read %d bytes from file %s, row %d\n",
			 cols, argv[ 2], ii);
		exit( -1);
	    }
	    if ( read( fdIn2, band2, cols) < cols) {
		fprintf( stderr, 
			 "failed to read %d bytes from file %s, row %d\n",
			 cols, argv[ 3], ii);
		exit( -1);
	    }
	    if ( read( fdIn3, band3, cols) < cols) {
		fprintf( stderr, 
			 "failed to read %d bytes from file %s, row %d\n",
			 cols, argv[ 4], ii);
		exit( -1);
	    }
	    for ( jj = 0; jj < cols; jj++) {
		outband[ jj] = (( uint_t) band1[ jj]) | 
			       ((( uint_t) band2[ jj]) << 8) || 
			       ((( uint_t) band3[ jj]) << 16);
	    }
	    if ( write( fdOut, outband, cols*sizeof( ulong_t)) < 
			cols*(signed)sizeof( ulong_t)) {
		fprintf( stderr, 
			 "failed to write %d bytes to file %s, row %d\n",
			 cols*sizeof( ulong_t), outpath, ii);
		exit( -1);
	    }
	}
	sprintf( outhdr, "%s.hdr", argv[ 5]);
	if (( hdrFp = fopen( outhdr, "w")) == NULL) {
	    fprintf( stderr, "failed to open output header %s\n", outhdr);
	    exit( -1);
	}
	fprintf( hdrFp, "filename:	%s\n", AmGetRootName( argv[ 2]));
	fprintf( hdrFp, "rows:		%d\n", rows);
	fprintf( hdrFp, "cols:		%d\n", cols);
	fprintf( hdrFp, "xmin:		%.2f\n", llX);
	fprintf( hdrFp, "ymin:		%.2f\n", llY);
	fprintf( hdrFp, "xmax:		%.2f\n", urX);
	fprintf( hdrFp, "ymax:		%.2f\n", urY);
	fprintf( hdrFp, "size:		4\n");
	fprintf( hdrFp, "dataType:	unsignedLong\n");
	fprintf( hdrFp, "type:		imagePlot\n");
	fprintf( hdrFp, "xvariable:	distance\n");
	fprintf( hdrFp, "xunits:		%s\n", units);
	fprintf( hdrFp, "yvariable:	distance\n");
	fprintf( hdrFp, "yunits:		%s\n", units);
	fprintf( hdrFp, "fileSource:	landsat\n");
	close( fdIn1);
	close( fdIn2);
	close( fdIn3);
	close( fdOut);
    }
    fclose( hdrFp);
}
