#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <anvil/project.h>
#include <anvil/files.h>
#include <anvil/macros.h>
#include <anvil/choppath.h>

/* simple program to convert a set of ASCII vectors to binaries and make
   a header */

int main( int argc, char *argv[])
{
    int		fd, rows;
    FILE	*fp, *hdrFp;
    char	file[ PATHLEN], header[ PATHLEN], path[ PATHLEN];
    float	writeBuf[ 4], xmin, ymin, xmax, ymax;
    AnvilBoolean	pipeFlag;

    if ( argc < 3) {
	fprintf( stderr, "usage: %s infile outfile\n", argv[ 0]);
	exit( -1);
    }
    if (( fp = AmOpenReadFilePointer( argv[ 1], &pipeFlag)) == NULL) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }

        strcpy( path, argv[ 2]);
    
    sprintf( file, "%s.dat", argv[ 2]);
    sprintf( header, "%s.hdr", argv[ 2]);
    if (( fd = open( file, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", file);
	exit( -1);
    }
    if (( hdrFp = fopen( header, "w")) == NULL) {
	fprintf( stderr, "failed to open header file %s\n", header);
	exit( -1);
    }
    rows = 0;
    xmin = MY_HUGE;
    ymin = MY_HUGE;
    xmax = -MY_HUGE;
    ymax = -MY_HUGE;
    do {
	if ( fscanf( fp, "%f %f %f %f", 
		     &writeBuf[ 0], &writeBuf[ 1], 
		     &writeBuf[ 2], &writeBuf[ 3]) != 4) {
			
	    break;
	}
	if ( writeBuf[ 0] < xmin) {
	    xmin = writeBuf[ 0];
	}
	if ( writeBuf[ 1] < ymin) {
	    ymin = writeBuf[ 1];
	}
	if ( writeBuf[ 0] > xmax) {
	    xmax = writeBuf[ 0];
	}
	if ( writeBuf[ 1] > ymax) {
	    ymax = writeBuf[ 1];
	}
	if ( writeBuf[ 2] < xmin) {
	    xmin = writeBuf[ 2];
	}
	if ( writeBuf[ 3] < ymin) {
	    ymin = writeBuf[ 3];
	}
	if ( writeBuf[ 2] > xmax) {
	    xmax = writeBuf[ 2];
	}
	if ( writeBuf[ 3] > ymax) {
	    ymax = writeBuf[ 3];
	}
	write( fd, writeBuf, sizeof( writeBuf));
	rows++;
    } while ( TRUE);
    fprintf( hdrFp, "filename:	%s\n", AmGetRootName( file));
    fprintf( hdrFp, "rows:		%d\n", rows);
    fprintf( hdrFp, "size:		%d\n", sizeof( writeBuf));
    fprintf( hdrFp, "type:		linePlot\n");
    fprintf( hdrFp, "xmin:          %f\n", xmin);
    fprintf( hdrFp, "ymin:          %f\n", ymin);
    fprintf( hdrFp, "xmax:          %f\n", xmax);
    fprintf( hdrFp, "ymax:          %f\n", ymax);
    fprintf( hdrFp, "fileSource:    vector\n");
    AmCloseReadFilePointer( fp, pipeFlag);
    fclose( hdrFp);
    close( fd);
    return 0;
}
