#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>
#include <anvil/project.h>
#include <anvil/datatype.h>
#include <util/misc.h>

/* given a table of string pairs, return the destination string for the
   matching source string */

char *AmMatchingStr( char 	*srcStr,
		   strPairPtr	strPairVec,
		   int		numPair)
{
    int	ii;

    for ( ii = 0; ii < numPair; ii++) {
    	if ( !strcmp( srcStr, strPairVec[ ii].srcStr)) {
	    return( strPairVec[ ii].dstStr);
	}
    }
    return( NULL);
}
			

/* lookup an integer value in a sorted table */
 
AnvilBoolean AmInTable( int    ival,
                 int    *table,
                 int    length)
{
    int          lo = 0, hi = length - 1, mid;
 
    while ( lo <= hi) {
        mid = ( lo + hi) >> 1;
        if ( ival < table[ mid]) {
            hi = mid - 1;
        } else if ( ival > table[ mid]) {
            lo = mid + 1;
        } else {
            return( TRUE);
        }
    }
    return( table[ lo] == ival);
}

/* look up an integer value in a sorted table and return the index */

int AmTableIndex( int     ival,
                int     *table,
                int     length)
{
    int          lo = 0, hi = length - 1, mid;

    while ( lo <= hi) {
        mid = ( lo + hi) >> 1;
        if ( ival < table[ mid]) {
            hi = mid - 1;
        } else if ( ival > table[ mid]) {
            lo = mid + 1;
        } else {
            return( mid);
        }
    }
    if ( table[ lo] == ival) {
        return( lo);
    } else {
        return( -1);
    }
}

/* depending on the lookup type and lookup offset, return the lookup value */
 
int AmGetLookupVal( char          *ptr,
                  AmDataTypeEnum  lookupType,
                  int           lookupOffset)
{
    switch ( lookupType) {
    case AmINT:
        return( *( int *)( ptr + lookupOffset));
    case AmUNSIGNED_INT:
        return( *( unsigned int *)( ptr + lookupOffset));
    case AmCHAR:
        return( *( ptr + lookupOffset));
    case AmUNSIGNED_CHAR:
        return( *( unsigned char *)( ptr + lookupOffset));
    case AmSHORT:
        return( *( short *)( ptr + lookupOffset));
    case AmUNSIGNED_SHORT:
        return( *( unsigned short *)( ptr + lookupOffset));
    default:
        return( *( int *)( ptr + lookupOffset));
    }
}

/* print hatch marks  */

void AmPrintHatch( int	index,
		 int	length,
		 int	nhatch)
{
    if ( length/nhatch > 0) {
	if ( index % ( length/nhatch) == 0) {
	    printf( "#");
	    fflush( stdout);
	}
    }
}
#if 0
/* stubs for AmPrintError and PrintResource error, so utilities don't have
   to link with X11 */

void AmPrintError( const char *fmt,
                 ...)
{
    va_list      args;
    char         buf[ MSGLEN];

    va_start( args, fmt);
    vsprintf( buf, fmt, args);
    va_end( args);
    fprintf( stderr, "%s\n", buf);
}

void AmPrintResourceError( const char *res, ...)
{
    fprintf( stderr, "%s\n", res);
}
#endif

/* atoi conversion on a length-limited field (rather than 0 terminated) */
 
int
AmAtoi2( char *buf, int len)
{
    char        tmpbuf[ 256];

    memcpy( tmpbuf, buf, len);
    tmpbuf[ len] = '\0';
    return( atoi( tmpbuf));
}

/*
    This ascii-to-float function is slightly different from the "atof" 
    function in that it will allow for certain FORTRAN conventions in 
    writing out floating point numbers.  This function was written    
    specifically to handle the format of double precision values in   
    the ASCII USGS data files which are written like this :          
    0.xxxxxxxxxxD+06                                    
*/

float AmAtof2 ( char *pc, int nmax )
{
    char        buffer[ 128];
    float       decimalPart;
    int         exponent;

    strncpy( buffer, pc, nmax);
    buffer[ nmax] = '\0';
    if ( strchr( buffer, 'D') != NULL) {
        sscanf( buffer, "%fD%d", &decimalPart, &exponent);
        return( decimalPart*pow( 10.0, ( double) exponent));
    } else {
        return( atof( buffer));
    } 
}




