#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#if defined(_WIN32)
#include <anvil/ntresource.h>
#else
#include <anvil/xresource.h>
#endif
#include <anvil/fns.h>
#include <anvil/strlib.h>
#include <anvil/tagstrs.h>
#include <anvil/datatype.h>
#include <anvil/datafmt.h>
#include <anvil/macros.h>
#include <anvil/raspoly.h>
#include <anvil/envdefs.h>
#include <anvil/choppath.h>
#include <util/utils.h>

const int MAXPTS = 16384;

int main( int argc, char *argv[])
{
    char		hdrpath[ PATHLEN], datapath[ PATHLEN], 
			enumDbPath[ PATHLEN], *filename, *kind,
			*outTypeStr, *inTypeStr, *sourceType, buffer[ MSGLEN],
			*fileSourceStr, fileSourceBuf[ PATHLEN];
    uchar_t		*data, *ptr, *ptr2, *matrix;
    float		ymin, ymax, xmin, xmax, resolution, 
			srcXmin, srcYmin, srcXmax, srcYmax, xscale, yscale;
    int			rows=0, cols=0, ii, jj, inSize, length, fdIn, fdOut, offset,
			numpolys, mostpts, matrows, matcols, val, outSize,
			xpos=0, ypos=0;
    AmDataTypeEnum	intype, outtype;
    AnvilBoolean		fileExists;
    struct stat		statbuf;
    AmPolygonPtr		polygon;
    AmIntAccessFn		AmIntFn;
    AmPtRec		points[ MAXPTS];
    FILE		*hdrFp=NULL;
    char                *fields[] = { AmNfilename, AmNtype, AmNnumdims,
				      AmNrows, AmNcols, AmNsize, AmNdataType, 
				      AmNxmin, AmNymin, AmNxmax, AmNymax, 
				      AmNcolorsType, AmNcolorsOffset,
				      AmNfileSource,
				      NULL };

    if ( argc < 4) {
	fprintf( stderr, "usage %s inhdr infile outfile [outype] [res]\n",
		 argv[ 0]);
	exit( -1);
    }
    if ( AmEndswith( argv[ 2], ".gz") || AmEndswith( argv[ 2], ".Z") ||
         AmEndswith( argv[ 2], ".z")) {
        fprintf( stderr, "rasterize may not be run with compressed files\n");
        exit( -1);
    }
    if ( getenv( CONFIG_ENV) == NULL) {
	fprintf( stderr, "environment variable %s not defined", CONFIG_ENV);
        exit( -1);
    }
    sprintf( enumDbPath, "%s/lookups/AnvilEnumLookup", getenv( CONFIG_ENV));
#if defined(_WIN32)
    AmNTResourceDbClass db( argv[ 1]);
#else
    AmXResourceDbClass db( argv[ 1]);
#endif

/* find out if the output file exists, and so, then use its resolution */

    sprintf( hdrpath, "%s.hdr", argv[ 3]);

/* get the extents of the polygon file */

    if ( !db.GetResource( AmNxmin, &srcXmin)) {
	fprintf( stderr, "failed to read xmin from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNymin, &srcYmin)) {
	fprintf( stderr, "failed to read ymin from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNxmax, &srcXmax)) {
	fprintf( stderr, "failed to read xmax from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNymax, &srcYmax)) {
	fprintf( stderr, "failed to read ymax from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNcolorsOffset, &offset)) {
	fprintf( stderr, "failed to read ymax from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNcolorsType, &inTypeStr)) {
	fprintf( stderr, "failed to read input type from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNfileSource, &fileSourceStr)) {
	fprintf( stderr, "failed to read file source from %s\n", argv[ 1]);
	exit( -1);
    }
    intype = AmStrToDataTypeEnum( inTypeStr);
    fileExists = ( access( hdrpath, R_OK) == 0);

/* get all the params from the file that we need */

    if ( fileExists) {
#if defined(_WIN32)
	AmNTResourceDbClass hdrdb( hdrpath);
#else
	AmXResourceDbClass hdrdb( hdrpath);
#endif

	AN_DEBUG( 10, ("rasterizing into existing file %s\n", hdrpath));
	if ( !hdrdb.GetResource( AmNtype, &kind)) {
	    fprintf( stderr, "failed to read kind from file\n");
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNfilename, &filename)) {
	    fprintf( stderr, "failed to get filename from output header %s\n",
		     hdrpath);
	    exit( -1);
	}
	if ( strcasecmp( kind, AmNimagePlot)) {
	    fprintf( stderr, "existing output file (%s) must be an image\n",
		     filename);
	    exit( -1);
	}
	strcpy( datapath, filename);
	if ( !hdrdb.GetResource( AmNxmin, &xmin)) {
	    fprintf( stderr, "failed to read xmin from %s\n", hdrpath);
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNymin, &ymin)) {
	    fprintf( stderr, "failed to read ymin from %s\n", hdrpath);
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNxmax, &xmax)) {
	    fprintf( stderr, "failed to read xmax from %s\n", hdrpath);
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNymax, &ymax)) {
	    fprintf( stderr, "failed to read ymax from %s\n", hdrpath);
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNrows, &matrows)) {
	    fprintf( stderr, "failed to read rows from %s\n", hdrpath);
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNcols, &matcols)) {
	    fprintf( stderr, "failed to read rows from %s\n", hdrpath);
	    exit( -1);
	}
	if ( !hdrdb.GetResource( AmNdataType, &outTypeStr)) {
	    fprintf( stderr, "failed to output type from %s\n", hdrpath);
	    exit( -1);
	}
	outtype = AmStrToDataTypeEnum( outTypeStr);
	resolution = ( ymax - ymin)/matcols;
    } else {
	sprintf( datapath, "%s.dat", argv[ 3]);

/* get the extents from the source file and use the rasterization resolution
   to generate the rows and cols */

	outtype = AmStrToDataTypeEnum( argv[ 4]);
	outTypeStr = argv[ 4];
	resolution = atof( argv[ 5]);
	rows = ( int) (( srcYmax - srcYmin)/resolution);
	cols = ( int) (( srcXmax - srcXmin)/resolution);
	if (( rows > 10000) || ( cols > 10000)) {
	    fprintf( stderr, "the resolution that you specified will create\n");
	    fprintf( stderr, "a matrix of %d bytes.  ARE YOU SURE? (y or n)\n",
		     rows*cols*AmDataTypeEnumToSize( outtype));
	    scanf("%c", buffer);
	    if (( buffer[ 0] == 'N') || ( buffer[ 0] == 'n')) {
		fprintf( stderr, "exiting...\n");
		exit( -1);
	    }
	}
	AN_DEBUG( 10, ("output matrix dims = (%.2f, %.2f), (%.2f, %.2f)\n",
		       srcXmin, srcYmin, srcXmax, srcYmax));
	AN_DEBUG( 10, ("output resolution = %.6f\n", resolution));
	AN_DEBUG( 10, ("output matrix size = %d, %d\n", rows, cols));
	matrows = rows;
	matcols = cols;
    }

/* get the offset to get values from, the input type, output type, and data
   size for each element in  the source file */

    outSize = AmDataTypeEnumToSize( outtype);
    AmIntFn = AmDataTypeEnumToIntAccessFn( intype);
    if ( !db.GetResource( AmNsize, &inSize)) {
	fprintf( stderr, "failed to read size from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNtype, &sourceType)) {
	fprintf( stderr, "failed to read type from %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNrows, &numpolys)) {
	fprintf( stderr, "failed to read numpolys from %s\n", argv[ 1]);
	exit( -1);
    }
    if (( matrix = ( uchar_t *) calloc( rows*cols, outSize)) == NULL) {
	fprintf( stderr, "failed to allocate %d bytes", rows*cols*outSize);
	exit( -1);
    }
    if (( fdIn = open( argv[ 2], O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 2]);
	exit( -1);
    }
    if ( fileExists) {

/* if the file exists, read the segment of the matrix which bounds the
   polygons which are going to be scan converted */

	if (( fdOut = open( datapath, O_RDWR)) < 0) {
	    fprintf( stderr, "failed to open %s for writing\n", datapath);
	    exit( -1);
	}
	xpos = ( int)(( srcXmin - xmin)*matcols);
	ypos = ( int)(( ymax - srcYmax)*matrows);
	for ( ii = 0; ii < rows; ii++) {
	    if ( lseek( fdOut, (( ypos + ii)*matcols + xpos)*outSize, 
			SEEK_SET) < 0) {
		fprintf( stderr, "failed to seek %d bytes in %s\n", 
			(( ypos + ii)*matcols + xpos)*outSize, datapath);
		exit( -1);
	    }
	    if ( read( fdOut, &matrix[ ii*cols], cols*outSize) < cols*outSize) {
		fprintf( stderr, "failed to read row %d\n", ii);
		exit( -1);
	    }
	}
    } else {

/* otherwise, just open it for writing */

	if (( fdOut = open( datapath, O_WRONLY | O_CREAT, PERMS)) < 0) {
	    fprintf( stderr, "failed to open %s for writing\n", datapath);
	    exit( -1);
	}
	if (( hdrFp = fopen( hdrpath, "w")) == NULL) {
	    fprintf( stderr, "failed to open output header file %s\n", hdrpath);
	    exit( -1);
	}
    }
    if ( !strcmp( sourceType, AmNpolygonPlot)) {
	if ( fstat( fdIn, &statbuf) < 0) {
	    exit( -1);
	}
	length = statbuf.st_size;
     
/* polygons AmFirst, points second */
     
	if (( data = ( uchar_t *) malloc( length)) == NULL) {
	    fprintf( stderr, "failed to malloc %d bytes\n", length);
	    exit( -1);
	}
	ptr2 = data + inSize*numpolys;
     
/* step the data pointer by the size of each polygon */
    
	mostpts = 0;
	AN_DEBUG( 10, ("rasterizing %d polygons\n", numpolys));
	for ( ii = 0, ptr = data; ii < numpolys; ii++, ptr += inSize) {
	    polygon = ( AmPolygonPtr) ptr;
	    if ( read( fdIn, polygon, inSize) < inSize) {
		fprintf( stderr, "failed to read polygon %d\n", ii);
		exit( -1);
	    }
     
/* set the polygon's points to the databuffer read just after it */
     
	    polygon->pts = ( AmCoordPtr) ptr2;
	    if ( read( fdIn, ptr2, sizeof( AmCoordRec)*polygon->numPts) <
		 (signed)sizeof( AmCoordRec)*polygon->numPts) {
		fprintf( stderr, "failed to read polygon %d\n", ii);
		exit( -1);
	    }
	    if ( polygon->numPts > mostpts) {
		mostpts = polygon->numPts;
	    }
	    ptr2 += sizeof( AmCoordRec)*polygon->numPts;
	}
	close( fdIn);
	xscale = cols/( srcXmax - srcXmin);
	yscale = rows/( srcYmax - srcYmin);

/* scale each polygon into integer coordinates and scan convert it */

	for ( ii = 0, ptr = data; ii < numpolys; ii++, ptr += inSize) {
	    AmPrintHatch( ii, numpolys, NHATCH);
	    polygon = ( AmPolygonPtr) ptr;
	    for ( jj = 0; jj < polygon->numPts; jj++) {
		points[ jj].x = ( int) 
		    ((polygon->pts[ jj].x - srcXmin)*xscale);
		points[ jj].y = ( int) 
		    ((srcYmax - polygon->pts[ jj].y)*yscale);
	    }
	    val = ( AmIntFn)((( uchar_t *) polygon) + offset, NULL);
	    AmRasterPoly( points, polygon->numPts, val, 
			matrix, outtype, rows, cols);
	}

/* if the file exists, write the part we rasterized back in */

	if ( fileExists) {
	    for ( ii = 0; ii < rows; ii++) {
		if ( lseek( fdOut, (( ypos + ii)*matcols + xpos)*outSize, 
			    SEEK_SET) < 0) {
		    fprintf( stderr, "failed to seek %d bytes in %s\n", 
			    (( ypos + ii)*matcols + xpos)*outSize, datapath);
		    exit( -1);
		}
		if ( write( fdOut, &matrix[ ii*cols], cols*outSize) < 
			    cols*outSize) {
		    fprintf( stderr, "failed to write row %d\n", ii);
		    exit( -1);
		}
	    }
	} else {
	    if ( write( fdOut, matrix, rows*cols*outSize) < 
		 rows*cols*outSize) {
		fprintf( stderr, "failed to write %d bytes\n",
			 rows*cols*outSize);
		exit( -1);
	    }
	    fprintf( hdrFp, "filename:     %s\n", AmGetRootName( datapath));
	    fprintf( hdrFp, "xmin:         %f\n", srcXmin);
	    fprintf( hdrFp, "ymin:         %f\n", srcYmin);
	    fprintf( hdrFp, "xmax:         %f\n", srcXmax);
	    fprintf( hdrFp, "ymax:         %f\n", srcYmax);
	    fprintf( hdrFp, "rows:         %d\n", rows);
	    fprintf( hdrFp, "cols:         %d\n", cols);
	    fprintf( hdrFp, "numdims:      2\n");
	    fprintf( hdrFp, "dataType:     %s\n", outTypeStr);
	    fprintf( hdrFp, "colorsType:   %s\n", outTypeStr);
	    fprintf( hdrFp, "size:         %d\n", outSize);
	    fprintf( hdrFp, "type:         %s\n", AmNimagePlot);
	    fprintf( hdrFp, "sourceFile:   %s\n", argv[ 1]);
	    sprintf( fileSourceBuf, "%s-IMAGE", fileSourceStr);
	    fprintf( hdrFp, "fileSource:   %s\n", fileSourceBuf);
	    fprintf( hdrFp, "note:   	rasterized\n");
	    AmDumpHeaderExcept( argv[ 1], fields, hdrFp);
	}
    }
}

