#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <fcntl.h>
#include <string.h>
#if defined (_WIN32)
#include <anvil/ntresource.h>
#else
#include <anvil/xresource.h>
#endif
#include <anvil/files.h>
#include <anvil/tagstrs.h>
#include <anvil/choppath.h>
#include <anvil/project.h>
#include <anvil/imageP.h>
#include <util/utils.h>


int main( int argc, char *argv[])
{
    char 		path[ PATHLEN], hdrPath[ PATHLEN], *kind, *dataType;
    uchar_t		*readRow, *writeRow; 
    int			fdIn, fdOut, rows, cols, size, *hIndex,
			newCols, newRows, ii, jj, val;
    FILE		*hdrFp;
    float		factor, xmin, ymin, xmax, ymax;
    AnvilBoolean		pipeFlag;
    char *fields[] = { AmNfilename, AmNrows, AmNcols, NULL };

    if ( argc < 5) {
	fprintf( stderr, "usage %s inheader infile outfile factor\n",
		 argv[ 0]);
	exit( -1);
    }
    sprintf( path, "%s.dat", argv[ 3]);
    sprintf( hdrPath, "%s.hdr", argv[ 3]);
    factor = atof( argv[ 4]);
    if ( factor > 1.0) {
	fprintf( stderr, "reduce factors should be less than 1.0\n");
	exit( -1);
    }

/* open the input header, and output files */
 
    if (( fdIn = AmOpenReadFile( argv[ 2], &pipeFlag)) < 0) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 2]);
	exit( -1);
    }
    if (( fdOut = open( path, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", path);
	exit( -1);
    }
    if (( hdrFp = fopen( hdrPath, "w")) == NULL) {
	fprintf( stderr, "failed to open output header %s\n", hdrPath);
	exit( -1);
    }
 
#if defined(_WIN32)
    AmNTResourceDbClass db( argv[ 1]);
#else
    AmXResourceDbClass db( argv[ 1]);
#endif
    if ( !db.IsValid()) {
	fprintf( stderr, "failed to read header file %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db.GetResource( AmNtype, &kind)) {
	fprintf( stderr, "failed to get file type from header %s\n", 
		 argv[ 2]);
	exit( -1);
    }
    if ( !strcmp( kind, AmNimagePlot)) {
	if ( !db.GetResource( AmNrows, &rows)) {
	    fprintf( stderr, "failed to read rows from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNcols, &cols)) {
	    fprintf( stderr, "failed to read cols from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNsize, &size)) {
	    fprintf( stderr, "failed to read size from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNdataType, &dataType)) {
	    fprintf( stderr, "failed to read dataType from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNxmin, &xmin)) {
	    fprintf( stderr, "failed to read xmin from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNymin, &ymin)) {
	    fprintf( stderr, "failed to read ymin from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNxmax, &xmax)) {
	    fprintf( stderr, "failed to read xmax from image file\n");
	    exit( -1);
	}
	if ( !db.GetResource( AmNymax, &ymax)) {
	    fprintf( stderr, "failed to read ymax from image file\n");
	    exit( -1);
	}
	newCols = ( int) ( factor*cols);
	newRows = ( int) ( factor*rows);

/* reduce the image */

	readRow = ( uchar_t *) malloc( size*cols);
	writeRow = ( uchar_t *) malloc( newCols*size);
	hIndex = AmMakeIndexVec( cols, newCols);
	val = newRows;
	for ( ii = 0; ii < rows; ii++) {
	    AmPrintHatch( ii, rows, NHATCH);
	    if ( AmReadLoopFile( fdIn, readRow, size*cols) < size*cols) {
		fprintf( stderr, "failed to read %d bytes at row %d\n",
			 ii, size*cols);
		exit( -1);
	    }
	    if ( val/rows > 0) {
		for ( jj = 0; jj < newCols; jj++) {
		    memcpy( &writeRow[ jj*size], 
			    &readRow[ hIndex[ jj]*size], size);
		}
		write( fdOut, writeRow, newCols*size);
		val -= rows;
	    }
	    val += newRows;
	}
	close( fdOut);
	AmCloseReadFile( fdIn, pipeFlag);

/* write the header */

	fprintf( hdrFp, "filename:	%s\n", AmGetRootName( path));
	fprintf( hdrFp, "rows:		%d\n", newRows);
	fprintf( hdrFp, "cols:		%d\n", newCols);
	fprintf( hdrFp, "sourceFile:    %s\n", argv[ 1]);
	fprintf( hdrFp, "note:    	reduce\n");
	AmDumpHeaderExcept( argv[ 1], fields, hdrFp);
	fclose( hdrFp);
    } else {
	fprintf( stderr, "only images are supported by reduce\n");
    }
}
