#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>
#include <malloc.h>
#include <fcntl.h>
#include <anvil/project.h>
#include <anvil/xresource.h>
#include <anvil/files.h>
#include <anvil/tagstrs.h>
#include <anvil/datafmt.h>
#include <anvil/quadnet.h>
#include <anvil/newtile.h>
#include <anvil/choppath.h>
#include <anvil/macros.h>
#include <util/utils.h>
#include <util/tile.h>

int main( int argc, char *argv[])
{
    char 		path[ PATHLEN], hdrPath[ PATHLEN], *kind, *sourceType;
    AmXResourceDbClass	*db;
    int			fdIn, fdOut, rows, size;
    FILE		*hdrFp;
    float		nodeTolerance, angleTolerance, vectorTolerance, 
			xmin, ymin, xmax, ymax;
    AnvilBoolean		pipeFlag;
    AmLinePtr		vectors;
    char		*exceptFields[] = { AmNrows, AmNfilename, NULL };

    if ( argc < 7) {
	fprintf( stderr, 
"usage %s inheader infile outfile vecthold anglethold nodethold\n",
		 argv[ 0]);
	exit( -1);
    }

/* generate the output header and data pathnames */

    sprintf( path, "%s.dat", argv[ 3]);
    sprintf( hdrPath, "%s.hdr", argv[ 3]);
    vectorTolerance = atof( argv[ 4]);
    angleTolerance = DEGTORAD( atof( argv[ 5]));
    nodeTolerance = atof( argv[ 6]);

/* open the input header, and output files */
 
    if (( fdIn = AmOpenReadFile( argv[ 2], &pipeFlag)) < 0) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 2]);
	exit( -1);
    }
    if (( fdOut = open( path, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", path);
	exit( -1);
    }
    if (( hdrFp = fopen( hdrPath, "w")) == NULL) {
	fprintf( stderr, "failed to open output header %s\n", hdrPath);
	exit( -1);
    }

    db = new AmXResourceDbClass( argv[ 1]);
    if ( !db->IsValid()) {
	fprintf( stderr, "failed to read header file %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNtype, &kind)) {
	fprintf( stderr, "failed to get file type from header %s\n", 
		 argv[ 2]);
	exit( -1);
    }
    if ( strcmp( kind, AmNlinePlot)) {
	fprintf( stderr, "source file %s is not a vector file\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNsize, &size)) {
	fprintf( stderr, "failed to get size type from header %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNrows, &rows)) {
	fprintf( stderr, "failed to get # of rows from header %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNxmin, &xmin)) {
	fprintf( stderr, "failed to get xmin from header %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNymin, &ymin)) {
	fprintf( stderr, "failed to get ymin from header %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNxmax, &xmax)) {
	fprintf( stderr, "failed to get xmax from header %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNymax, &ymax)) {
	fprintf( stderr, "failed to get ymax from header %s\n", argv[ 1]);
	exit( -1);
    }
    if ( !db->GetResource( AmNsourceType, &sourceType)) {
    	sourceType = "definitelyNotPyramidSource";
    }
    vectors = ( AmLinePtr) malloc( rows*size);
    if ( vectors == NULL) {
	fprintf( stderr, "failed to allocate %d bytes\n", rows*size);
	exit( -1);
    }
    if ( read( fdIn, vectors, rows*size) < rows*size) {
	fprintf( stderr, "failed to read %d bytes from %s\n", 
			 rows*size, argv[ 2]);
	exit( -1);
    }
    if ( AmReduceLines( vectors, &rows, size, xmin, ymin, xmax, ymax, 
		      vectorTolerance, angleTolerance, 
		      nodeTolerance, FALSE) != AmOK) {
	fprintf( stderr, "error reducing lines\n");
	exit( -1);
    }
    if ( write( fdOut, vectors, rows*size) < rows*size) {
	fprintf( stderr, "failed to write %d bytes to %s\n", 
		 rows*size, path);
	exit( -1);
    }
    close( fdOut);

/* write out the new header information */

    fprintf( hdrFp, "filename:		%s\n", AmGetRootName( path));
    fprintf( hdrFp, "rows:		%d\n", rows);
    fprintf( hdrFp, "note:		reduced by line tolerance %f\n",
	     vectorTolerance);
    fprintf( hdrFp, "note:		reduced by angle tolerance %f\n",
	     angleTolerance);
    AmDumpHeaderExcept( argv[ 1], exceptFields, hdrFp);
    fclose( hdrFp);
}
