#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <anvil/project.h>
#include <anvil/files.h>
#include <anvil/macros.h>
#include <anvil/choppath.h>

/***************************************************************************
 *
 * Description:
 *	program for converting ASCII files with x,y,z values into 
 *	binary scatter plot files, with whitespace AmBetween the x,y,z values
 *
 ***************************************************************************/

typedef struct xyz_ {
    float	x, y, z;
} xyzRec, *xyzPtr;

int main( int argc, char *argv[])
{
    FILE	*fp, *hdrFp;
    int		fd, rows;
    xyzRec	xyz;
    char	file[ 512], header[ 512];
    float	xmin, ymin, xmax, ymax, tmp, min, max;
    AnvilBoolean	pipeFlag, reverseFlag;

    if ( argc < 3) {
	fprintf( stderr, "usage: %s infile outfile [-yx]\n", argv[0]);
	exit( -1);
    }
    if (( fp = AmOpenReadFilePointer( argv[ 1], &pipeFlag)) == NULL) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }
    reverseFlag = (( argc == 4) && !strcasecmp( argv[ 3], "-yx"));
    sprintf( file, "%s.dat", argv[ 2]);
    sprintf( header, "%s.hdr", argv[ 2]);
    if (( fd = open( file, O_WRONLY | O_CREAT, PERMS)) < 0) {
	fprintf( stderr, "failed to open output file %s\n", file);
	exit( -1);
    }
    if (( hdrFp = fopen( header, "w")) == NULL) {
	fprintf( stderr, "failed to open header file %s\n", header);
	exit( -1);
    }
    rows = 0;
    xmin = MY_HUGE;
    ymin = MY_HUGE;
    xmax = -MY_HUGE;
    ymax = -MY_HUGE;
    min = MY_HUGE;
    max = -MY_HUGE;
    do {
	if ( fscanf( fp, "%f %f %f", &xyz.x, &xyz.y, &xyz.z) != 3) {
	    break;
	}
	if ( reverseFlag) {
	    tmp = xyz.x;
	    xyz.x = xyz.y;
	    xyz.y = tmp;
	}
	if ( xyz.x < xmin) {
	    xmin = xyz.x;
	}
	if ( xyz.y < ymin) {
	    ymin = xyz.y;
	}
	if ( xyz.x > xmax) {
	    xmax = xyz.x;
	}
	if ( xyz.y > ymax) {
	    ymax = xyz.y;
	}
	if ( xyz.z < min) {
	    min = xyz.z;
	}
	if ( xyz.z > max) {
	    max = xyz.z;
	}
	write( fd, &xyz, sizeof( xyzRec));
	rows++;
    } while ( TRUE);
    fprintf( hdrFp, "filename:	%s\n", AmGetRootName( file));
    fprintf( hdrFp, "rows:		%d\n", rows);
    fprintf( hdrFp, "size:		%d\n", sizeof( xyzRec));
    fprintf( hdrFp, "type:		scatterPlot\n");
    fprintf( hdrFp, "xmin:          %f\n", xmin);
    fprintf( hdrFp, "ymin:          %f\n", ymin);
    fprintf( hdrFp, "xmax:          %f\n", xmax);
    fprintf( hdrFp, "ymax:          %f\n", ymax);
    fprintf( hdrFp, "colorsOffset:  %d\n", sizeof( float)*2);
    fprintf( hdrFp, "colorsType:    float\n");
    fprintf( hdrFp, "colorsMin:	    %f\n", min);
    fprintf( hdrFp, "colorsMax:	    %f\n", max);
    fprintf( hdrFp, "fileSource:    measurement\n");
    fprintf( hdrFp, "sourceFile:    %s\n", argv[ 1]);

/* if the range is within -180, 180, -90, 90, it's probably arcdeg, otherwise
   guess for meters */

    if (( xmin > -180.0) && ( xmax < 180.0) && 
	( ymin > -90.0) && ( ymax < 90.0)) {
	fprintf( hdrFp, "xunits:	arcdeg\n");
	fprintf( hdrFp, "yunits:	arcdeg\n");
    } else {
	fprintf( hdrFp, "xunits:	meters\n");
	fprintf( hdrFp, "yunits:	meters\n");
    }
    AmCloseReadFilePointer( fp, pipeFlag);
    fclose( hdrFp);
    close( fd);
}
