#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <anvil/project.h>
#include <anvil/choppath.h>

#define XDIM		"XDIM"
#define YDIM		"YDIM"
#define NCOLS		"NCOLS"
#define NROWS		"NROWS"
#define ULXMAP		"ULXMAP"
#define ULYMAP		"ULYMAP"
#define NBITS		"NBITS"
#define NBANDS		"NBANDS"
#define MAPUNITS	"MAPUNITS"
#define BAND_RGB	"BAND_RGB"

int main( int argc, char *argv[])
{
    FILE	*fpIn, *fpOut, *cmapFpIn, *cmapFpOut;
    int		ncols, nrows, nbits, red, green, blue, pixel, nbands,
		rgbindex[ 10];
    float	xdim, ydim, ulxmap, ulymap;
    char	buffer[ PATHLEN], name[ 128], mapunits[ 128], 
		outhdr[ PATHLEN], outfile[ PATHLEN], bandrgb[ 128], *p;

    if ( argc < 4) {
	fprintf( stderr, "usage: %s inhdr infile outfile [colorin colorout]\n", 
			 argv[ 0]);
	exit( -1);
    }
    if (( fpIn = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open input header %s\n", argv[ 1]);
	exit( -1);
    }
    sprintf( outhdr, "%s.hdr", argv[ 3]);
    if (( fpOut = fopen( outhdr, "w")) == NULL) {
	fprintf( stderr, "failed to open output header %s\n", outhdr);
	exit( -1);
    }

/* scan for the critical information from the SPOT header */

    do {
	if ( fgets( buffer, sizeof( buffer), fpIn) == NULL) {
	    break;
	}
	if ( !strncasecmp( buffer, XDIM, strlen( XDIM))) {
	    if ( sscanf( buffer, "%s %f", name, &xdim) < 2) {
		fprintf( stderr, "failed to scan %s\n", XDIM);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, YDIM, strlen( YDIM))) {
	    if ( sscanf( buffer, "%s %f", name, &ydim) < 2) {
		fprintf( stderr, "failed to scan %s\n", YDIM);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, ULXMAP, strlen( ULXMAP))) {
	    if ( sscanf( buffer, "%s %f", name, &ulxmap) < 2) {
		fprintf( stderr, "failed to scan %s\n", ULXMAP);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, ULYMAP, strlen( ULYMAP))) {
	    if ( sscanf( buffer, "%s %f", name, &ulymap) < 2) {
		fprintf( stderr, "failed to scan %s\n", ULYMAP);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, NCOLS, strlen( NCOLS))) {
	    sscanf( buffer, "%s	%d", name, &ncols);
	    if ( sscanf( buffer, "%s %d", name, &ncols) < 2) {
		fprintf( stderr, "failed to scan %s\n", NCOLS);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, NROWS, strlen( NROWS))) {
	    if ( sscanf( buffer, "%s %d", name, &nrows) < 2) {
		fprintf( stderr, "failed to scan %s\n", NROWS);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, MAPUNITS, strlen( MAPUNITS))) {
	    if ( sscanf( buffer, "%s %s", name, mapunits) < 2) {
		fprintf( stderr, "failed to scan %s\n", MAPUNITS);
		exit( -1);
	    }
	    for ( p = mapunits; *p != '\0'; p++) {
		*p = tolower( *p);
	    }
	} else if ( !strncasecmp( buffer, NBITS, strlen( NBITS))) {
	    if ( sscanf( buffer, "%s %d", name, &nbits) < 2) {
		fprintf( stderr, "failed to scan %s\n", NBITS);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, NBANDS, strlen( NBANDS))) {
	    if ( sscanf( buffer, "%s %d", name, &nbands) < 2) {
		fprintf( stderr, "failed to scan %s\n", NBANDS);
		exit( -1);
	    }
	} else if ( !strncasecmp( buffer, BAND_RGB, strlen( BAND_RGB))) {
	    if ( sscanf( buffer, "%s %[0-9 \t]", name, bandrgb) < 2) {
		fprintf( stderr, "failed to scan %s\n", BAND_RGB);
		exit( -1);
	    }
	}
    } while ( TRUE);

/* if there's only 1 band, then we can just take the file as is, otherwise
   we have to write a new file out in pixel format */

    if ( nbands == 1) {
	strcpy( outfile, argv[ 2]);
    } else {
	sprintf( outfile, "%s.dat", argv[ 3]);
    }
    fprintf( fpOut, "filename:		%s\n", AmGetRootName( outfile));
    fprintf( fpOut, "sourceFile:	%s\n", outfile);
    fprintf( fpOut, "rows:	%d\n", nrows);
    fprintf( fpOut, "cols:	%d\n", ncols);
    fprintf( fpOut, "xmin:	%.2f\n", ulxmap);
    fprintf( fpOut, "ymin:	%.2f\n", ulymap - nrows*ydim);
    fprintf( fpOut, "xmax:	%.2f\n", ulxmap + ncols*xdim);
    fprintf( fpOut, "ymax:	%.2f\n", ulymap);
    fprintf( fpOut, "xunits:	%s\n", mapunits);
    fprintf( fpOut, "yunits:	%s\n", mapunits);

/* if 1 band, then take it as is, otherwise, write out as true color 
   long pixels */

    if ( nbands == 1) {
	fprintf( fpOut, "size:	%d\n", nbits/8);
	if ( nbits == 8) {
	    fprintf( fpOut, "dataType:	char\n");
	} else if ( nbits == 16) {
	    fprintf( fpOut, "dataType:	short\n");
	}
    } else if ( nbands == 3) {
	fprintf( fpOut, "size:		4\n");
	fprintf( fpOut, "dataType:	long\n");
	fprintf( fpOut, "trueColor:	true\n");
    }
    fprintf( fpOut, "type:	imagePlot\n");
    fprintf( fpOut, "fileSource:SPOT\n");
    if ( argc == 6) {
	if (( cmapFpIn = fopen( argv[ 4], "r")) == NULL) {
	    fprintf( stderr, "failed to open input color file %s\n", argv[ 4]);
	    exit( -1);
	}
	if (( cmapFpOut = fopen( argv[ 5], "w")) == NULL) {
	    fprintf( stderr, "failed to open output color file %s\n", argv[ 5]);
	    exit( -1);
	}
	do {
	    if ( fgets( buffer, sizeof( buffer), cmapFpIn) == NULL) {
		break;
	    }

/* skip over comments */

	    if ( buffer[ 0] == '!') {
		continue;
	    }
	    sscanf( buffer, "%d %d %d %d", &pixel, &red, &green, &blue);
	    fprintf( cmapFpOut, "%d %d %d %d\n", pixel, red, green, blue);
	} while( TRUE);
	fclose( cmapFpIn);
	fclose( cmapFpOut);

/* add the palette reference to the header file */

	fprintf( fpOut, "palette:	%s\n", argv[ 5]);
    }
    fclose( fpIn);
    fclose( fpOut);

/* if there are 3 bands, then write them as pixels */

    if ( nbands == 3) {
	ulong_t	*row;
	uchar_t	*bands[ 3];
	int	fdIn, fdOut, ii, jj;
	
	if (( fdOut = open( outfile, O_CREAT | O_WRONLY, PERMS)) < 0) {
	    fprintf( stderr, "failed to open output file %s\n", outfile);
	    exit( -1);
	}
	if (( fdIn = open( argv[ 2], O_RDONLY)) < 0) {
	    fprintf( stderr, "failed to open input file %s\n", argv[ 2]);
	    exit( -1);
	}

/* dissect the rgb string into an index vector */
	
	ii = 0;
	p = bandrgb;
	while ( *p != '\0') {
	    if ( isdigit( *p)) {
		rgbindex[ ii++] = atoi( p) - 1;
		while ( isdigit( *p++));
	    } else {
		p++;
	    }
	}

/* allocate the buffers for reading and writing the vectors */

	row = ( ulong_t *) malloc( sizeof( ulong_t)*ncols);
	if ( row == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n",
		     sizeof( ulong_t)*ncols);
	    exit( -1);
	}
	for ( ii = 0; ii < 3; ii++) {
	    bands[ ii] = ( uchar_t *) malloc( sizeof( uchar_t)*ncols);
	    if ( bands[ ii] == NULL) {
		fprintf( stderr, "failed to allocate %d bytes\n",
			 sizeof( uchar_t)*ncols);
		exit( -1);
	    }
	}
	for ( ii = 0; ii < nrows; ii++) {
	    for ( jj = 0; jj < 3; jj++) {
		if ( read( fdIn, bands[ rgbindex[ jj]], ncols) < ncols) {
		    fprintf( stderr, "failed to read row %d\n", ii);
		    exit( -1);
		}
	    }
	    for ( jj = 0; jj < ncols; jj++) {
		row[ jj] = bands[ 0][ jj] | 
			   bands[ 1][ jj] << 8 | 
			   bands[ 2][ jj] << 16;
	    }
	    if ( write( fdOut, row, ncols*sizeof( ulong_t)) <
		 (signed)(ncols*sizeof( ulong_t))) {
		fprintf( stderr, "failed to write row %d\n", ii);
		exit( -1);
	    }
	}
	close( fdIn);
	close( fdOut);
    }
}
