#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <anvil/envdefs.h>
#include <anvil/description.h>
#include <anvil/strlib.h>

#define BUFLEN	1024*1024

int main( int argc, char *argv[])
{
    AmListPtr	typeList;
    char	errorString[ MSGLEN];
    uchar_t	*buffer;
    int		length = BUFLEN, fdIn, fdOut, readbytes, backup, numToRead;
    AmReadTypeRec	readType;

    if ( argc < 3) {
	fprintf( stderr, "usage: %s infile description outfile [bufsize]\n", 
		 argv[ 0]);
	exit( -1);
    }
    if ( AmEndswith( argv[ 1], ".gz") || AmEndswith( argv[ 1], ".Z") ||
         AmEndswith( argv[ 1], ".z")) {
        fprintf( stderr, "swapper may not be run with compressed files\n");
        exit( -1);
    }
    if ( argc == 5) {
	length = atoi( argv[ 4]);
    }
    if (( fdIn = open( argv[ 1], O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }
    if (( fdOut = open( argv[ 3], O_CREAT | O_WRONLY, PERMS)) < 0) {
	fprintf( stderr, "failed to open input file %s\n", argv[ 1]);
	exit( -1);
    }
    if ( AmParseDescriptionFile( argv[ 2], &typeList, errorString) != AmOK) {
	fprintf( stderr, "failed to parse description %s\n", errorString);
	exit( -1);
    }
    typeList = AmGetDeclarations( typeList);
    AmPrintDescription( typeList);
    buffer = ( uchar_t *) malloc( length);
    readType.currTypePtr = typeList;
    readType.typeList = typeList;
    readType.currIndex = 0;
    numToRead = 0;
    do {
	if ((readbytes = read( fdIn, buffer, length)) < 0) {
	    fprintf( stderr, "failed to read %d bytes from source file\n", length);
	    exit( -1);
	}
	if ( readbytes == 0) {
	    break;
	}
AN_DEBUG( 50, ("parsing %d bytes\n", readbytes));
	backup = AmReadParseTypeList( &readType, buffer, readbytes, 
				    &numToRead, TRUE);
	if ( readbytes == backup) {
	    fprintf( stderr, "WARNING: file has truncated prematurely\n");
	    exit( -1);
	}
AN_DEBUG( 50, ("backing up %d bytes\n", backup));
	if ( lseek( fdIn, -backup, SEEK_CUR) < 0) {
	    fprintf( stderr, "failed to seek %d bytes in source file\n",
		     -backup);
	    exit( -1);
	}
AN_DEBUG( 50, ("writing %d bytes\n", readbytes - backup));
	if ( write( fdOut, buffer, readbytes - backup) < 0) {
	    fprintf( stderr, "failed to write %d bytes to dest file\n", 
		      readbytes - backup);
	    exit( -1);
	}
    } while ( TRUE);
    close( fdIn);
    close( fdOut);
}
