#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include <anvil/project.h>
#include <anvil/xdisplay.h>
#include <anvil/shmimg.h>
#include <anvil/visual.h>

void
AmDrawImage1( AmXDisplayClass *xdisplay,
            AmXWindowClass  *xw, 
            AmXGCClass      *xgc,
            int           srcX,
            int           srcY,
            int           dstX,
            int           dstY,
            int           srcWidth,
            int           srcHeight,
            AmXImageClass   *ximage)
{
    AmColorCellRec	cell;
    Display   *display = xdisplay->GetXDisplay();
    GC        maskGC;
    XGCValues gcv;
    int       displayDepth;

/* using a 0xff.. mask, draw the image in OR, which causes all the places
   where the image gets drawn to be set to 0xf... */

    displayDepth = AmGetAppDepth();

    gcv.foreground = ( 0x1 << displayDepth) - 1;
    gcv.function = GXor;
    gcv.background = 0;
    maskGC = XCreateGC( display, DefaultRootWindow( display),
                 GCForeground | GCBackground | GCFunction, &gcv);

    AmShmShowImage( display, xw->GetXWindow(), maskGC, ximage->GetXImage(), 
                  srcX, srcY, dstX, dstY, srcWidth, srcHeight,
                  ximage->getShmInfo());

/* Draw the image using AND, where all the 0xf... gets replaced by the 
   actual foreground color */

    xgc->GetForegroundColor( &cell);
    XSetForeground( display, maskGC, cell.pixel);
    XSetBackground( display, maskGC, ( 0x1 << displayDepth) - 1);
    XSetFunction( display, maskGC, GXand);
    AmShmShowImage( display, xw->GetXWindow(), maskGC, ximage->GetXImage(), 
        srcX, srcY, dstX, dstY, srcWidth, srcHeight,
        ximage->getShmInfo());

/* Free up our temporary GC */

    XFreeGC( display, maskGC);
}


void
AmDrawImage8( AmXDisplayClass *xdisplay,
            AmXWindowClass  *xw, 
            AmXGCClass      *xgc,
            int           srcX,
            int           srcY,
            int           dstX,
            int           dstY,
            int           srcWidth,
            int           srcHeight,
            AmXImageClass   *ximage)
{
    Display *display = xdisplay->GetXDisplay();

    AmShmShowImage( display, xw->GetXWindow(), xgc->GetXGC(), 
        ximage->GetXImage(), 
        srcX, srcY, dstX, dstY, srcWidth, srcHeight,
        ximage->getShmInfo());
}


void
AmDrawImage16( AmXDisplayClass *xdisplay,
             AmXWindowClass  *xw, 
             AmXGCClass      *xgc,
             int           srcX,
             int           srcY,
             int           dstX,
             int           dstY,
             int           srcWidth,
             int           srcHeight,
             AmXImageClass   *ximage)
{
    Display *display = xdisplay->GetXDisplay();

    AmShmShowImage( display, xw->GetXWindow(), xgc->GetXGC(), 
        ximage->GetXImage(), 
        srcX, srcY, dstX, dstY, srcWidth, srcHeight,
        ximage->getShmInfo());
}


void
AmDrawImage24( AmXDisplayClass *xdisplay,
             AmXWindowClass  *xw, 
             AmXGCClass      *xgc,
             int           srcX,
             int           srcY,
             int           dstX,
             int           dstY,
             int           srcWidth,
             int           srcHeight,
             AmXImageClass   *ximage)
{
    Display *display = xdisplay->GetXDisplay();

    AmShmShowImage( display, xw->GetXWindow(), xgc->GetXGC(), 
        ximage->GetXImage(), 
        srcX, srcY, dstX, dstY, srcWidth, srcHeight,
        ximage->getShmInfo());
}
