#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#if !defined (__BSD44__)
#include <alloca.h>
#endif
#include <malloc.h>
#include <X11/Xlib.h>
#include <anvil/pixthing.h>
#include <anvil/xdisplay.h>
#include <anvil/imageP.h>

#define MASK24 0xffffffff
#define MASK8 0xff

static GC glyphGC = NULL, glyphMaskGC = NULL;

/* read the symbol file into a pixmap and mask */

static AmErrorCode 
readPixmapAndMask( Display	*display,
		   Window	w,
		   uint_t	depth,
		   char		*symfile,
		   ulong_t	color,
		   Pixmap	*pixmap,
		   Pixmap	*mask,
		   int		*width,
		   int		*height)
{
    uchar_t	*data, *dataPtr, *dataEnd;
    XImage	*image, *maskImage;
    int		screen = DefaultScreen( display);

    if ( !AmReadBitmapFileData( symfile, width, height, &data, color, 0)) {
	DO_RETURN_ERROR( AmREAD_FAILED);
    }

/* create the pixmap and associated mask */

    *pixmap = XCreatePixmap( display, w, *width, *height, depth);
    *mask = XCreatePixmap( display, w, *width, *height, depth);
    image = XCreateImage( display, DefaultVisual( display, screen),
			  depth, ZPixmap, 0, NULL, *width, *height, 8, 0);

/* create images for the pixmap and mask */

    if ( image == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    image->data = ( char *) malloc( *height*image->bytes_per_line);
    if ( image->data == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    maskImage = XCreateImage( display, DefaultVisual( display, screen),
			      depth, ZPixmap, 0, NULL, *width, *height, 8, 0);
    if ( maskImage == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    maskImage->data = ( char *) malloc( *height*maskImage->bytes_per_line);
    if ( maskImage->data == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    dataEnd = data + ( *width)*( *height);

/* map the symbol data to the display image depending on the size */

    if ( image->depth == 8) {
	uchar_t	*imagePtr, *maskPtr;

	for ( imagePtr = ( uchar_t *) image->data,
	      maskPtr = ( uchar_t *) maskImage->data,
	      dataPtr = ( uchar_t *) data;
	      dataPtr < dataEnd;
	      imagePtr++, maskPtr++, dataPtr++)  {
	    if ( *dataPtr) {
		*imagePtr = color;
		*maskPtr = MASK8;
	    } else {
		*imagePtr = MASK8;
		*maskPtr = 0;
	    }
	}
    } else {
	ulong_t	*imagePtr, *maskPtr;

	for ( imagePtr = ( ulong_t *) image->data,
	      maskPtr = ( ulong_t *) maskImage->data,
	      dataPtr = ( uchar_t *) data;
	      dataPtr < dataEnd;
	      imagePtr++, maskPtr++, dataPtr++)  {
	    if ( *dataPtr) {
		*imagePtr = color;
		*maskPtr = MASK24;
	    } else {
		*imagePtr = MASK24;
		*maskPtr = 0;
	    }
	}
    } 

/* put the images into the pixmaps */

    XPutImage( display, *pixmap, DefaultGC( display, screen), image,
	       0, 0, 0, 0, *width, *height);
    XPutImage( display, *mask, DefaultGC( display, screen), maskImage,
	       0, 0, 0, 0, *width, *height);

/* free the symbol data and the image */
    
    free( image->data);
    free( image);
    free( maskImage->data);
    free( maskImage);
    return( AmOK);
}

/* x glyph class creation function */

AmXGlyphClass::AmXGlyphClass( AmDisplayClass	*display,
			      AmWindowClass	*window,
			      char		*symfile,
			      ulong_t		colorArg)
{
    Window	xwin;

    xdisp = (( AmXDisplayClass *) display)->GetXDisplay();
    xwin = (( AmXWindowClass *) window)->GetXWindow();
    depth = display->GetDepth();
    color = colorArg;
    valid = ( readPixmapAndMask( xdisp, xwin, depth, symfile, color, &pixmap,
				 &mask, &width, &height) != AmOK);

/* create the static GC's for drawing the pixmap and the mask */

    if ( glyphGC == NULL) {
	XGCValues	gcv;

	gcv.function = GXor;
	glyphMaskGC = XCreateGC( xdisp, xwin, GCFunction, &gcv);
	gcv.function = GXand;
	glyphGC = XCreateGC( xdisp, xwin, GCFunction, &gcv);
    }
}

/* x glyph class cration with pixmap and mask pixmaps */

AmXGlyphClass::AmXGlyphClass( Display        *displayArg,
			      Pixmap         pixmapArg,
			      Pixmap         maskArg,
			      int            widthArg,
			      int            heightArg,
			      int            depthArg,
			      ulong_t	     colorArg)
{
    xdisp = displayArg;
    pixmap = pixmapArg;
    mask = maskArg;
    width = widthArg;
    height = heightArg;
    depth = depthArg;
    color = colorArg;
    valid = TRUE;
}

/* paint a glyph to the screen. Draw the mask, then the pixmap */ 

void AmXGlyphClass::DrawGlyph( AmDisplayClass	*display,
			     AmWindowClass	*window,
			     int		x,
			     int		y)
{
    XCopyArea((( AmXDisplayClass *) display)->GetXDisplay(), mask, 
	      (( AmXWindowClass *) window)->GetXWindow(), glyphMaskGC, 
	      0, 0, width, height, x, y);
    XCopyArea((( AmXDisplayClass *) display)->GetXDisplay(), pixmap, 
	      (( AmXWindowClass *) window)->GetXWindow(), glyphGC, 
	      0, 0, width, height, x, y);
}

/* create a new glyph from a source glyph */

AmGlyphClass *AmXGlyphClass::ResizeGlyph( int	newWidth,
				      int	newHeight)
{
    Pixmap	newPixmap, newMask;
    XImage	*image, *maskImage, *srcImage;
    int		*hIndex, *vIndex, *pHindex, *pVindex,
		screen = DefaultScreen( xdisp);
    AmXGlyphClass	*newGlyph;

    newPixmap = XCreatePixmap( xdisp, pixmap, newWidth, newHeight, depth);
    newMask = XCreatePixmap( xdisp, pixmap, newWidth, newHeight, depth);

/* create images for the pixmap and mask */

    image = XCreateImage( xdisp, DefaultVisual( xdisp, screen),
			  depth, ZPixmap, 0, NULL, newWidth, newHeight, 8, 0);
    if ( image == NULL) {
	return( NULL);
    }
    image->data = ( char *) malloc( newHeight*image->bytes_per_line);
    if ( image->data == NULL) {
	return( NULL);
    }
    maskImage = XCreateImage( xdisp, DefaultVisual( xdisp, screen),
			      depth, ZPixmap, 0, NULL, newWidth, newHeight, 8, 0);
    if ( maskImage == NULL) {
	return( NULL);
    }
    maskImage->data = ( char *) malloc( newWidth*maskImage->bytes_per_line);
    if ( maskImage->data == NULL) {
	return( NULL);
    }

/* grab the image from the source glyph */
 
    srcImage = XGetImage( xdisp, pixmap, 0, 0, width, height, MASK24, ZPixmap);

/* create the resize vectors */

    hIndex = ( int *) alloca( newWidth*sizeof( int));
    vIndex = ( int *) alloca( newHeight*sizeof( int));
    AmFwdDiffVec( width, newWidth, hIndex);
    AmFwdDiffVec( height, newHeight, vIndex);

/* stuff the images depending on the pixel size */

    if ( depth == 32) {
	ulong_t	*srcDataLong, *dstDataLong, *dstEndLong, *dstMaskLong,
		*rowStartLong, *rowEndLong, *rowMaskStartLong, *rowSrcStartLong;

/* assign the initial pointers */

	pVindex = vIndex;
	dstDataLong = ( ulong_t *) image->data;
	dstMaskLong = ( ulong_t *) maskImage->data;
	srcDataLong = ( ulong_t *) srcImage->data;
	dstEndLong = dstDataLong + newWidth*newHeight;

/* step through the image */

	while ( dstDataLong < dstEndLong) {
	    pHindex = hIndex;
	    rowStartLong = dstDataLong;
	    rowEndLong = dstDataLong + newWidth;
	    rowMaskStartLong = dstMaskLong;
	    rowSrcStartLong = srcDataLong;

/* assign the new image pixel and the mask while stepping through the row */ 

	    while ( rowStartLong < rowEndLong) {
		*rowStartLong = *rowSrcStartLong;
		if ( *rowStartLong) {
		    *rowMaskStartLong = MASK24;
		} else {
		    *rowMaskStartLong = 0;
		}
		rowStartLong++;
		rowSrcStartLong += *pHindex++;
	    }
	    dstDataLong += newWidth;
	    dstMaskLong += newWidth;
	    srcDataLong += ( *pVindex++)*width;
	}
    } else {
	uchar_t	*srcDataChar, *dstDataChar, *dstEndChar, *dstMaskChar,
		*rowStartChar, *rowEndChar, *rowMaskStartChar, *rowSrcStartChar;

/* assign the initial pointers */

	pVindex = vIndex;
	dstDataChar = ( uchar_t *) image->data;
	dstMaskChar = ( uchar_t *) maskImage->data;
	srcDataChar = ( uchar_t *) srcImage->data;
	dstEndChar = dstDataChar + newWidth*newHeight;

/* step through the image */

	while ( dstDataChar < dstEndChar) {
	    pHindex = hIndex;
	    rowStartChar = dstDataChar;
	    rowEndChar = dstDataChar + newWidth;
	    rowMaskStartChar = dstMaskChar;
	    rowSrcStartChar = srcDataChar;

/* assign the new image pixel and the mask while stepping through the row */ 

	    while ( rowStartChar < rowEndChar) {
		*rowStartChar = *rowSrcStartChar;
		if ( *rowStartChar) {
		    *rowMaskStartChar = MASK8;
		} else {
		    *rowMaskStartChar = 0;
		}
		rowStartChar++;
		rowSrcStartChar += *pHindex++;
	    }
	    dstDataChar += newWidth;
	    dstMaskChar += newWidth;
	    srcDataChar += ( *pVindex++)*width;
	}
    }

/* put the images into the pixmaps and free them */

    XPutImage( xdisp, newPixmap, DefaultGC( xdisp, screen), image,
	       0, 0, 0, 0, newWidth, newHeight);
    XPutImage( xdisp, newMask, DefaultGC( xdisp, screen), maskImage,
	       0, 0, 0, 0, newWidth, newHeight);
    free( srcImage->data);
    free( srcImage);
    free( image->data);
    free( image);
    free( maskImage->data);
    free( maskImage);
    newGlyph = new AmXGlyphClass( xdisp, newPixmap, newMask, newWidth, newHeight,
				depth, color);
    return( newGlyph);
}
    

/* glyph class destructor */

AmXGlyphClass::~AmXGlyphClass()
{
    XFreePixmap( xdisp, pixmap);
    XFreePixmap( xdisp, mask);
}

/* function to retreive the glyph pixmap */

Pixmap AmXGlyphClass::getPixmap()
{
    return( pixmap);
}

/* function to retreive the glyph mask */

Pixmap AmXGlyphClass::getMask()
{
    return( mask);
}
