#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <malloc.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include <anvil/project.h>
#include <anvil/shmimg.h>
#include <anvil/shmimgP.h>
#include <anvil/errcode.h>
#include <anvil/samehost.h>

extern "C" {
Bool XShmQueryExtension( Display *dpy);
};

/* make a shared memory image, just like a XCreateImage call. */

XImage *AmShmMakeImage( 
    Display             *display,       /* the usual server connection */
    Visual              *visual,        /* DefaultVisual( display, screen) */
    int                 depth,          /* DefaultDepth( display, screen) */
    int                 format,         /* always ZPixmap */
    unsigned int        width,          /* dimensions of the image */
    unsigned int        height,
    XShmSegmentInfo     **shminfoPtr)  /* stick around */
{
    XImage      	*image;
    int         	length;

/* create the shared memory image if the extension exists.  sparcs choke
   on XYBitmap formats */

#ifdef sparc
    if ( XShmQueryExtension( display) && ( format != XYBitmap) && 
	 AmSamehost( ConnectionNumber( display)) && ( shminfoPtr != NULL)) {
#else
    if ( XShmQueryExtension( display) && 
	 AmSamehost( ConnectionNumber( display)) && ( shminfoPtr != NULL)) {
#endif
        *shminfoPtr = ( XShmSegmentInfo *) calloc( 1, sizeof( XShmSegmentInfo));
        image = XShmCreateImage( display, visual, depth, format, NULL,
                                 *shminfoPtr, width, height);

/* get a shared memory identifier of the appropriate size ( allocates it).
   the bytes_per_line is kinda unneccessary on 8-bit displays, but nice */

        length = height*image->bytes_per_line;

/* shmget is the shared memory version of malloc */

        (*shminfoPtr)->shmid = shmget( IPC_PRIVATE, length, IPC_CREAT | 0644);
        if ( (*shminfoPtr)->shmid != -1) {

/* attach the shared memory segment to this process */

            (*shminfoPtr)->shmaddr = image->data =
                ( char *) shmat( (*shminfoPtr)->shmid, 0, 0);
            if ( image->data != ( char *) -1) {

/* let the server write the segment, and attach it to the server's process */

                (*shminfoPtr)->readOnly = True;
                if ( XShmAttach( display, *shminfoPtr)) {
		    AmInitImageDestroyer( display);
		    AmAddImageDestroy( image, *shminfoPtr);
                    return( image);
                } else {
		    DO_ERROR( AmSHM_ATTACH_ERROR);
		    shmdt( image->data);
		    shmctl(( *shminfoPtr)->shmid, IPC_RMID, NULL);
                }
            } else {
		DO_ERROR( AmSHM_ATTACH_ERROR);
		shmdt( image->data);
		shmctl(( *shminfoPtr)->shmid, IPC_RMID, NULL);
            }
        } else {
	    DO_ERROR( AmSHM_ATTACH_ERROR);
            FREE( image);
            FREE( *shminfoPtr);
        }
    }

/* flag for bimodal free image call */

    if ( shminfoPtr != NULL) {
	*shminfoPtr = NULL;        
    }

/* create a normal XImage if we can't create a shared memory XImage */

    image = XCreateImage( display, visual, depth, format, 0, NULL,
                          width, height, 8, 0);
    length = height*image->bytes_per_line;
    image->data = ( char *) malloc( length);

/* considering the size of some images, we should check this */

    if ( image->data == NULL) {
        DO_ERROR( AmMALLOC_FAILED);
    }
    return( image);
}
