#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



/* xvertext 3.0, Copyright (c) 1993 Alan Richardson (mppa3@uk.ac.sussex.syma)
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  All work developed as a consequence of the use of
 * this program should duly acknowledge such use. No representations are
 * made about the suitability of this software for any purpose.  It is
 * provided "as is" without express or implied warranty.
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>   /* maybe your system doesn't want this */

#include <anvil/rotated.h>

#define ERROR 1

/* ---------------------------------------------------------------------- */


/**************************************************************************/
/*  Create an XImage structure and allocate memory for it                 */
/**************************************************************************/

XImage *AmMakeXImage( Display *dpy,
		    int w, 
		    int h)
{
    XImage 		*I;
    unsigned char 	*data;
    
/* reserve memory for image */

    data=(unsigned char *)calloc((unsigned)(((w-1)/8+1)*h), 1);
    if(data==NULL)
	return NULL;
    
/* create the XImage */
    I=XCreateImage(dpy, DefaultVisual(dpy, DefaultScreen(dpy)), 1, XYBitmap,
                   0, ( char *) data, w, h, 8, 0);
    if(I==NULL)
	return NULL;
    
    I->byte_order=I->bitmap_bit_order=MSBFirst;
    return I;
}


/* ---------------------------------------------------------------------- */


/**************************************************************************/
/*  A front end to AmXRotPaintAlignedString:                                */
/*      -no alignment, no background                                      */
/**************************************************************************/

int AmXRotDrawString( Display 	*dpy,
		    XFontStruct *font,
		    double 	angle,
		    Drawable 	drawable,
		    GC 		gc,
		    int 	x, 
		    int		y,
		    char 	*str)
{
    return (AmXRotPaintAlignedString(dpy, font, angle, drawable, gc,
				   x, y, str, NONE, 0));
}


/* ---------------------------------------------------------------------- */


/**************************************************************************/
/*  A front end to AmXRotPaintAlignedString:                                */
/*      -no alignment, paints background                                  */
/**************************************************************************/

int AmXRotDrawImageString( Display 	*dpy,
			 XFontStruct 	*font,
			 double 	angle,
			 Drawable 	drawable,
			 GC 		gc,
			 int 		x, 
			 int		y,
			 char 		*str)
{
    return(AmXRotPaintAlignedString(dpy, font, angle, drawable, gc,
				  x, y, str, NONE, 1));
}


/* ---------------------------------------------------------------------- */


/**************************************************************************/
/*  A front end to AmXRotPaintAlignedString:                                */
/*      -does alignment, no background                                    */
/**************************************************************************/

int AmXRotDrawAlignedString( Display 	*dpy,
			   XFontStruct 	*font,
			   double 	angle,
			   Drawable 	drawable,
			   GC 		gc,
			   int 		x, 
			   int		y,
			   char 	*text,
			   int 		align)
{
    return(AmXRotPaintAlignedString(dpy, font, angle, drawable, gc,
				  x, y, text, align, 0));
}


/* ---------------------------------------------------------------------- */


/**************************************************************************/
/*  A front end to AmXRotPaintAlignedString:                                */
/*      -does alignment, paints background                                */
/**************************************************************************/

int AmXRotDrawAlignedImageString( Display 	*dpy,
				XFontStruct 	*font,
				float 		angle,
				Drawable 	drawable,
				GC 		gc,
				int 		x, 
				int		y,
				char 		*text,
				int 		align)
{
    return(AmXRotPaintAlignedString(dpy, font, angle, drawable, gc,
				  x, y, text, align, 1));
}


/* ---------------------------------------------------------------------- */


/**************************************************************************/
/*  The routine which does the rotating and painting                      */
/**************************************************************************/

int AmXRotPaintAlignedString( Display 	*dpy,
			    XFontStruct *font,
			    float 	angle,
			    Drawable 	drawable,
			    GC 		gc,
			    int 	x, 
			    int		y,
			    char 	*text,
			    int 	align,
			    int 	bg)
{
    Pixmap canvas=0;
    GC fontGc=0, myGc;
    XImage *IIn;
    XImage *IOut;
    XPoint *xpoints=NULL;
    int nl=1;
    int max_width;
    int i, j;
    char *str1, str2[ 32], *str3;
    int height, ascent, descent;
    int colsIn, rowsIn;
    int colsOut, rowsOut;
    int byteWIn, byteWOut;
    int xp, yp;
    float hotX, hotY;
    float hotXp, hotYp;
    float sinAngle, cosAngle;
    float *cornersX=NULL, *cornersY=NULL;
    int it, jt;
    float di, dj;
    int ic=0;
    unsigned int bestw, besth;
    Pixmap emptyStipple;
    float xl=0., xr=0., xinc=0.;
    int byteOut;
    
    if((text==NULL) || ( *text == '\0')) {
        return 0;
    }
    
    /* manipulate angle */
    while(angle<-360)
        angle+=360;
    
    while(angle>360)
        angle-=360;
    
    angle*=M_PI/180;
    
    /* this gc has similar properties to the user's gc */
    myGc=XCreateGC(dpy, drawable, 0, NULL);
    XCopyGC(dpy, gc, GCForeground|GCBackground|GCFunction, myGc);
    
    /* count number of sections in string */
    if(align!=NONE)
	for(i=0; i<( int) strlen(text)-1; i++)
	    if(text[i]=='\n')
		nl++;
    
    /* ignore newline characters if not doing alignment */
    if(align==NONE)
	str2[ 0] = '\0';
    else
	strcpy( str2, "\n");
    
    if(str2==NULL)
	return ERROR;
    
    /* find width of longest section */
    str1=strdup(text);
    if(str1==NULL)
        return ERROR;
    
    str3=strtok(str1, str2);
    max_width=XTextWidth(font, str3, strlen(str3));
    
    /* loop through each section */
    do {
        str3=strtok((char *)NULL, str2);
        if(str3!=NULL)
            if(XTextWidth(font, str3, strlen(str3))>max_width)
                max_width=XTextWidth(font, str3, strlen(str3));
    }
    while(str3!=NULL);
    
    /* overall font info */
    ascent=font->ascent;
    descent=font->descent;
    height=ascent+descent;
    
    /* dimensions horizontal text will have */
    colsIn=max_width;
    rowsIn=nl*height;
    
    colsIn+=2;
    
    if(colsIn%2==0)
	colsIn++;
    
    if(rowsIn%2==0)
	rowsIn++;
    
    /* is text horizontal ... */
    if(angle==0)
	XSetFont(dpy, myGc, font->fid);
    
    /* or vertical ? */
    else {
	/* bitmap for drawing on */
	canvas=XCreatePixmap(dpy, DefaultRootWindow(dpy),
			     colsIn, rowsIn, 1);

	/* create a GC for the bitmap */
	fontGc=XCreateGC(dpy, canvas, 0, NULL);
	XSetBackground(dpy, fontGc, 0);
	XSetFont(dpy, fontGc, font->fid);
	
	/* make sure the bitmap is blank */
	XSetForeground(dpy, fontGc, 0);
	XFillRectangle(dpy, canvas, fontGc, 0, 0, colsIn+1, rowsIn+1);
    }
    
    /* alignment : which point (hotX, hotY) relative to bitmap centre
       coincides with user's specified point? */
    
    /* y position */
    if(align==TLEFT || align==TCENTRE || align==TRIGHT)
        hotY=(float)rowsIn/2;
    else if(align==MLEFT || align==MCENTRE || align==MRIGHT)
	hotY=0;
    else if(align==BLEFT || align==BCENTRE || align==BRIGHT)
	hotY=-(float)rowsIn/2;
    else
	hotY=-(float)rowsIn/2+(float)descent;
    
    /* x position */
    if(align==TLEFT || align==MLEFT || align==BLEFT || align==NONE)
	hotX=-(float)max_width/2;
    else if(align==TCENTRE || align==MCENTRE || align==BCENTRE)
	hotX=0;
    else
        hotX=(float)max_width/2;
    
    /* pre-calculate sin and cos */
    sinAngle=sin(angle);
    cosAngle=cos(angle);
    
    /* rotate hotX and hotY around bitmap centre */
    hotXp= hotX*cosAngle - hotY*sinAngle;
    hotYp= hotX*sinAngle + hotY*cosAngle;
    
    /* text background will be drawn using XFillPolygon */
    if(angle!=0) {
	if(bg) {
	    xpoints=(XPoint *)malloc((unsigned)(4*nl*sizeof(XPoint)));
	    if(!xpoints)
		return ERROR;
	    cornersX=(float *)malloc((unsigned)(4*nl*sizeof(float)));
	    if(!cornersX)
		return ERROR;
	    cornersY=(float *)malloc((unsigned)(4*nl*sizeof(float)));
	    if(!cornersY)
		return ERROR;
	    
	    ic=0;
	}
	XSetForeground(dpy, fontGc, 1);
    }
    
    /* draw text horizontally */
    
    /* start at top of bitmap */
    yp=ascent;
    free( str1); 
    str1=strdup(text);
    if(str1==NULL)
	return ERROR;
    
    str3=strtok(str1, str2);
    
    /* loop through each section in the string */
    do {
	/* where to draw section in x ? */
	if(align==TLEFT || align==MLEFT || align==BLEFT || align==NONE)
	    xp=0;
	else if(align==TCENTRE || align==MCENTRE || align==BCENTRE)
	    xp=(max_width-XTextWidth(font, str3, strlen(str3)))/2;
	else
	    xp=max_width-XTextWidth(font, str3, strlen(str3));
	
	/* if no rotation is required, draw straight to target drawable */
	if(angle==0) {
	    if(bg)
		XDrawImageString(dpy, drawable, myGc, 
				 (int)((float)x+
				       (float)xp-((float)colsIn/2+hotX)),
				 (int)((float)y-
				       ((float)rowsIn/2-hotY-(float)yp)),
				 str3, strlen(str3));
	    else
		XDrawString(dpy, drawable, myGc, 
			    (int)((float)x+
				  (float)xp-((float)colsIn/2+hotX)),
			    (int)((float)y-
				  ((float)rowsIn/2-hotY-(float)yp)),
			    str3, strlen(str3));
	    
	}
	/* rotation is required */
	else {
	    /* draw string onto bitmap */
	    XDrawString(dpy, canvas, fontGc, xp, yp, str3, strlen(str3));
	    
	    /* keep a note of corner positions of this string */
	    if(bg) {
		cornersX[ic]=(float)xp-(float)colsIn/2-hotX;
		cornersY[ic]=(float)(yp-ascent)-(float)rowsIn/2+hotY;
		cornersX[ic+1]=cornersX[ic];
		cornersY[ic+1]=cornersY[ic]+(float)height;
		cornersX[nl*4-1-ic]=cornersX[ic]+
		    (float)XTextWidth(font, str3, strlen(str3));
		cornersY[nl*4-1-ic]=cornersY[ic];
		cornersX[nl*4-2-ic]=cornersX[nl*4-1-ic];
		cornersY[nl*4-2-ic]=cornersY[ic+1];
		ic+=2;
	    }
	}
	/* move to next line */
	yp+=height;
	
        str3=strtok((char *)NULL, str2);
    }
    while(str3!=NULL);
    
    /* we can leave here if no rotation is required */
    if(angle==0)
	return 0;
    
    /* create image to hold horizontal text */
    IIn=AmMakeXImage(dpy, colsIn, rowsIn);
    if(IIn==NULL)
	return ERROR;
    
    /* extract horizontal text */
    XGetSubImage(dpy, canvas, 0, 0, colsIn, rowsIn,
		 1, XYPixmap, IIn, 0, 0);
    IIn->format=XYBitmap;
    
    /* how big will rotated text be ? */
    colsOut=( int) (fabs((float)rowsIn*sinAngle) +
		    fabs((float)colsIn*cosAngle) +0.99999);
    rowsOut=( int) (fabs((float)rowsIn*cosAngle) +
		    fabs((float)colsIn*sinAngle) +0.99999);
    
    if(colsOut%2==0)
	colsOut++;
    
    if(rowsOut%2==0)
	rowsOut++;
    
    /* create image to hold rotated text */
    IOut=AmMakeXImage(dpy, colsOut, rowsOut);
    if(IOut==NULL)
	return ERROR;
    
    byteWIn=(colsIn-1)/8+1;
    byteWOut=(colsOut-1)/8+1;
    
    /* loop through all bits in rotated image */

    /* we try to make this as fast as possible - which is why it looks
       a bit over-the-top */

    /* loop through all rows in the image */
    for(j=0; j<rowsOut; j++) {

	/* distance from centre */
	dj=(float)j+0.5-(float)rowsOut/2;

	/* some bits in final image don't map to bits in original -
	   try to pre-calculate these */
	if(j==0) {
	    if(angle==M_PI/2 || angle==M_PI || angle==3*M_PI/2) {
		xl=0;
                xr=(float)colsOut;
                xinc=0;
            }
	    else if(angle<M_PI) {
		xl=(float)colsOut/2+
		    (dj-(float)rowsIn/(2*cosAngle))/tan(angle)-2;
		xr=(float)colsOut/2+
		    (dj+(float)rowsIn/(2*cosAngle))/tan(angle)+2;
		xinc=1./tan(angle);
	    }
	    else {
		xl=(float)colsOut/2+
		    (dj+(float)rowsIn/(2*cosAngle))/tan(angle)-2;
		xr=(float)colsOut/2+
		    (dj-(float)rowsIn/(2*cosAngle))/tan(angle)+2;

		xinc=1./tan(angle);
	    }
	}
	else {
	    xl+=xinc;
	    xr+=xinc;
	}

	/* no point re-calculating these every pass */
	di=(float)((xl<0)?0:(int)xl)+0.5-(float)colsOut/2;
#if 0
	byteOut=(rowsOut-j)*byteWOut;
#endif
	byteOut=(rowsOut-j-1)*byteWOut;

	/* loop through meaningful columns */
	for(i=((xl<0)?0:(int)xl); i<((xr>=colsOut)?colsOut:(int)xr); i++) {
	    /* rotate coordinates */
	    it=( int) ((float)colsIn/2 + ( di*cosAngle + dj*sinAngle));
	    jt=( int) ((float)rowsIn/2 - (-di*sinAngle + dj*cosAngle));
	    
	    /* set pixel if required */
	    if(it>=0 && it<colsIn && jt>=0 && jt<rowsIn) 
		if((IIn->data[jt*byteWIn+it/8] & 128>>(it%8))>0)
		    IOut->data[byteOut+i/8] |= 128>>i%8;

	    di+=1.;
	}
    }

    XFreePixmap(dpy, canvas);
    
    /* create a new bitmap to hold rotated text */
    canvas=XCreatePixmap(dpy, DefaultRootWindow(dpy), colsOut, rowsOut, 1);
    XPutImage(dpy, canvas, fontGc, IOut, 0, 0, 0, 0, colsOut, rowsOut);
    
    /* fill in the background to the text, if required */
    if(bg) {
	/* rotate corner positions */
	for(i=0; i<4*nl; i++) {
	    xpoints[i].x= ( short) ((float)x + ( cornersX[i]*cosAngle + 
						 cornersY[i]*sinAngle));
	    xpoints[i].y= ( short) ((float)y + (-cornersX[i]*sinAngle + 
						 cornersY[i]*cosAngle));
	}
	
	/* we would like to swap foreground and background colors here,
	   using XGetGCValues, but X11R3 doesn't have it */

	XQueryBestStipple(dpy, drawable, 1, 1, &bestw, &besth);
	emptyStipple=XCreatePixmap(dpy, drawable, bestw, besth, 1);

	XSetForeground(dpy, fontGc, 0);
	XFillRectangle(dpy, emptyStipple, fontGc, 0, 0, bestw+1, besth+1);
	    
	XSetStipple(dpy, myGc, emptyStipple);
	XSetFillStyle(dpy, myGc, FillOpaqueStippled);
	    
	XFillPolygon(dpy, drawable, myGc, xpoints, 4*nl, 
		     Nonconvex, CoordModeOrigin);
	
	/* free our resources */
	free(xpoints);
	free(cornersX);
	free(cornersY);
    }
    
    /* where should top left corner of bitmap go ? */
    xp=( int) ((float)x-((float)colsOut/2 +hotXp));
    yp=( int) ((float)y-((float)rowsOut/2 -hotYp));
    
    /* paint text using stipple technique */
    XSetFillStyle(dpy, myGc, FillStippled);
    XSetStipple(dpy, myGc, canvas);
    XSetTSOrigin(dpy, myGc, xp, yp);
    XFillRectangle(dpy, drawable, myGc, xp, yp, colsOut, rowsOut);
    
    /* free our resources */
    XDestroyImage(IIn);
    XDestroyImage(IOut);
    XFreePixmap(dpy, canvas);
    XFreeGC(dpy, myGc);
    XFreeGC(dpy, fontGc);
    free( str1);
    
    /* we got to the end OK! */
    return 0;
}


