#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include <anvil/project.h>
#include <anvil/shmimg.h>
#include <anvil/shmimgP.h>
#include <anvil/errcode.h>
#include <anvil/list.h>

typedef struct shmImageData_ {
    XImage		*image;
    XShmSegmentInfo	*shminfo;
} shmImageDataRec, *shmImageDataPtr;

static AmListPtr	SHM_IMAGE_DATA_LIST = NULL;
static Display	*THE_DISPLAY = NULL;

static int sameShmImageData( shmImageDataPtr	a,
			     shmImageDataPtr	b)
{
    return( a->image == b->image);
}

/* atexit() handler to destroy any outstanding shared memory images */

static void imageDestroyer()
{
    shmImageDataPtr	shmImageData;

/* walk through the list of outstanding images, and destroy them.  
   AmShmDestroyImage() calls removeImageDestroy(), which removes the
   element from the list, so we keep calling it until the list is NULL. */

    while ( SHM_IMAGE_DATA_LIST != NULL) {
	shmImageData = ( shmImageDataPtr) SHM_IMAGE_DATA_LIST->data;
	AmShmDestroyImage( THE_DISPLAY, 
			 shmImageData->image, shmImageData->shminfo);
    }
}

/* initialize the destroyer's private display, set up the atexit() handler */

AmErrorCode AmInitImageDestroyer( Display	*display)
{
    if ( THE_DISPLAY == NULL) {
	THE_DISPLAY = display;
	if ( atexit( imageDestroyer) != 0) {
	    DO_RETURN_ERROR( AmATEXIT_FAILED);
	}
    }
    return( AmOK);
}

/* add a shared memory image to the list of outstanding images to destroy */

AmErrorCode AmAddImageDestroy( XImage		*image,
			     XShmSegmentInfo	*shminfo)
{
    shmImageDataPtr	shmImageData;

/* don't add it to the destroyer if it's not a shared memory image */

    if ( shminfo == NULL) {
	return( AmOK);
    }
    shmImageData = ( shmImageDataPtr) malloc( sizeof( shmImageDataRec));
    if ( shmImageData == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    shmImageData->image = image;
    shmImageData->shminfo = shminfo;
    SHM_IMAGE_DATA_LIST = AmInsertNodeLst( shmImageData, SHM_IMAGE_DATA_LIST);
    return( AmOK);
}

/* remove an image from the outstanding destroy list */

static void removeImageDestroy( XImage		*image,
			        XShmSegmentInfo	*shminfo)
{
    shmImageDataRec	target;

    target.image = image;
    target.shminfo = shminfo;

    SHM_IMAGE_DATA_LIST = AmDeleteItemLst( &target, SHM_IMAGE_DATA_LIST, 
					 ( AmSearchFn) sameShmImageData, 
					 ( AmItemFn) free);
}
	
/* destroy a shared memory image and its associate shared memory id. */

AmErrorCode AmShmDestroyImage( Display 		*display,
			     XImage 		*image,
			     XShmSegmentInfo 	*shminfo)
{

/* no shminfo pointer impilies that this is a normal XImage */

    if ( shminfo == NULL) {
	FREE( image->data);
    } else {
	removeImageDestroy( image, shminfo);

/* detach it from the server and process, then remove it from memory. */

	XSynchronize( display, TRUE);
	XShmDetach( display, shminfo);
	XSynchronize( display, FALSE);

/* detach and remove the shared memory segment */

	if ( shmdt( shminfo->shmaddr) == -1) {
	    DO_RETURN_ERROR( AmSHM_DETACH_ERROR);
	}
	if ( shmctl( shminfo->shmid, IPC_RMID, 0) == -1) {
	    DO_RETURN_ERROR( AmSHM_DETACH_ERROR);
	}

/* free the structure holding the shared memory information */
	
	FREE( shminfo);
    }

/* free the image structure in both cases */

    FREE( image);
    return( AmOK);
}
