#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* Includes: */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <X11/Xlib.h>
#include <X11/Xresource.h>

#include <Xm/Xm.h>

#include <anvil/project.h>
#include <anvil/envdefs.h>
#include <anvil/resource.h>
#include <anvil/units.h>
#include <anvil/projects.h>
#include <anvil/textcheck.h>
#include <anvil/strlib.h>
#include <anvil/display.h>
#include <anvil/xapp.h>
#include <anvil/visual.h>
#include <anvil/xfonts.h>
#include <anvil/xresource.h>
#include <anvil/errlib.h>
#include <anvil/xcolor.h>
#include <anvil/template.h>
#include <anvil/xpm.h>
#include <anvil/enums.h>
#include <anvil/textcheck.h>

/* Evil global variables. */

static Display *appDisplay;

AnvilBoolean
AmCvtStringToPixel( Display     *display,
                    XrmValue    *,
                    Cardinal    *num_args,
                    XrmValue    *fromVal,
                    XrmValue    *toVal,
                    XtPointer   *)
{
    XColor              xColor;
    static AmColorCellRec color;

    if ( *num_args != 0) {
        XtWarningMsg( "wrongParameters", "cvtStringToPixel",
            "XtToolkitError", "String to Pixel conversion needs\
            no extra arguments",
            ( String *) NULL, ( Cardinal *) NULL);
    }
 
    AN_DEBUG( 2, ( "In string to pixel translator routine.\n"));

    XParseColor( display, AmGetAppColormap(), ( char *) fromVal->addr,
        &xColor);
 
    color.red = xColor.red >> 8;
    color.green = xColor.green >> 8;
    color.blue = xColor.blue >> 8;
    AmSetPixelValue( &color);
    if ( toVal->addr != NULL) {
	if ( toVal->size < sizeof( ulong_t)) {
	    toVal->size = sizeof( ulong_t);
	    return( FALSE);
	}
	*( ulong_t *)( toVal->addr) = color.pixel;
    } else {
	toVal->size = sizeof( ulong_t);
	toVal->addr = ( XPointer) &color.pixel;
    }
    toVal->size = sizeof( ulong_t);

    AN_DEBUG( 2, ( "Converted: \"%s\" to %ld.\n", ( char *) fromVal->addr,
        color.pixel));

    return( TRUE);
}

AnvilBoolean
AmCvtStringToPixmap( Display     *display,
                     XrmValue    *,
                     Cardinal    *num_args,
                     XrmValue    *fromVal,
                     XrmValue    *toVal,
                     XtPointer   *)
{
  /*
    Pixmap pixmap;
  */

    if ( *num_args != 0) {
        XtWarningMsg( "wrongParameters", "cvtStringToPixmap",
            "XtToolkitError", "String to Pixmap conversion needs\
            no extra arguments",
            ( String *) NULL, ( Cardinal *) NULL);
    }
 
    AN_DEBUG( 2, ( "In string to pixmap translator routine.\n"));

/*
    pixmap = loadPixmap( ( char *) fromVal->addr);

    if ( toVal->addr != NULL) {
	if ( toVal->size < sizeof( Pixmap)) {
	    toVal->size = sizeof( Pixmap);
	    return( FALSE);
	}
	*( Pixmap *)( toVal->addr) = pixmap;
    } else {
	toVal->size = sizeof( Pixmap);
	toVal->addr = ( XPointer) &pixmap;
    }
    toVal->size = sizeof( Pixmap);
*/

    fprintf( stderr,"Converted: \"%s\".\n", ( char *) fromVal->addr);
    return( TRUE);
}

/* Perform initializations needed for any application. */

AmErrorCode
AmInitApplication( Widget top)
{
    Display 	*display = XtDisplay( top);
    Window  	window = DefaultRootWindow( display);
    AmErrorCode	errCode;
    struct stat	buf;
    struct tm	*timebuf;
    time_t	timer;

  /* Do some sanity checking. */

  /* 
   * NOTE:
   * check to make sure CONFIG_ENV is set, and that it points to a valid
   * directory 
   */

    if ( getenv( CONFIG_ENV) == NULL) {
        fprintf( stderr,
            "Error: The environment variable \"%s\" must be set.\n",
            CONFIG_ENV);
	DO_RETURN_ERROR( AmBAD_ARGUMENT);
    }
    if ( stat( getenv( CONFIG_ENV), &buf) < 0) {
	perror( "stat failed");
	fprintf( stderr, "configuration directory %s does not exist\n",
		 getenv( CONFIG_ENV));
	DO_RETURN_ERROR( AmFILE_NO_EXIST);
    }
    time( &timer);
    timebuf = localtime( &timer);
#if 0
    if ( timebuf->tm_mon > 7) {
	fprintf( stderr, "evaluation period has expired\n");
	exit( -1);
    }
#endif

  /* Set some global parameters that can be reused throughout the application */

    appDisplay = display;
    AmInitVisual( display);
    AmInitColormaps( display, window, AmGetAppVisual());

/* Initialize the X database and the units and variables database */

    AmSetTheResourceDb( new AmXResourceDbClass( XtDatabase( display)));
    if (( errCode = AmInitUnitStuff()) != AmOK) {
	return( errCode);
    }

/* Setup the font name chaching mechanisms. */

    AmInitFontCache( display);

/* initialize the table of object initializers */

    AmAddDefaultObjectInits();

/* initialize the standard lookup table */

    AmCreateLookupTable();

/* set up the resource converters. */ 
#if 0

/* Disabled for now - Phil. */

    XtSetTypeConverter( XmRString, XmRPixel, AmCvtStringToPixel,
			( XtConvertArgList) NULL, 0, XtCacheAll, NULL);

    XtSetTypeConverter( XmRString, XmRPixmap, AmCvtStringToPixmap,
			( XtConvertArgList) NULL, 0, XtCacheAll, NULL);
#endif

/* set up the shell for bringing up error dialogs */

    AmInitializeError( top);

/* Install the colormaps. */

    XtVaSetValues( top,
                   XmNcolormap, AmGetAppColormap(), 
                   XmNvisual, AmGetAppVisual(), 
                   XmNdepth, AmGetAppDepth(), 
                   NULL);

    return( AmOK);
}

Display *
AmGetAppDisplay()
{
    return( appDisplay);
}
