#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */



#include <stdio.h>
#include <string.h>
#include <malloc.h>
#if !defined (__BSD44__)
#include <alloca.h>
#endif
#include <Xm/Xm.h>
#include <anvil/xdisplay.h>
#include <anvil/colorcell.h>
#include <anvil/visual.h>
#include <anvil/xcolor.h>
#include <anvil/rotated.h>
#include <anvil/macros.h>
#include <anvil/events.h>


#define SCREEN_COORD( iVirtual) ( iVirtual * _iDPI / _iDPIBase)
#define VIRTUAL_COORD( iScreen) ( iScreen * _iDPIBase / _iDPI)

/* Do nothing constructor. */

AmXDisplayClass::AmXDisplayClass()
    : AmDisplayClass()
{
}

/* Construct a display from a widget. */

AmXDisplayClass::AmXDisplayClass( Widget wShell)
    : AmDisplayClass()
{
    _wShell = wShell;
    _pxDisplay = XtDisplay( wShell);
    _eDisplayType = AmX_DISPLAY;

    XtVaGetValues( wShell, XmNcolormap, &_xColormap, NULL);
}

/* Destructor. */

AmXDisplayClass::~AmXDisplayClass()
{
}

/* Return the colormap for the display */

Colormap
AmXDisplayClass::GetXColormap()
{
    return( _xColormap);
}
/* Return the default visual for the display */

Visual *
AmXDisplayClass::GetXVisual()
{
    return( AmGetAppVisual());
}

Display *
AmXDisplayClass::GetXDisplay()
{
    return( _pxDisplay);
}

/* Return the bit-depth of the display */

int
AmXDisplayClass::GetDepth()
{
    return( AmGetAppDepth());
}

/* return the validity of the window */

AnvilBoolean
AmXWindowClass::IsValid()
{
    if ( _xWidget != NULL) {
	return( XtWindow( _xWidget) != 0);
    } else if ( _xWindow != 0) {
	return( TRUE);
    } else {
	return( FALSE);
    }
}

/* Call the XDrawLines function from the wrapper */

void
AmXDisplayClass::DrawLines( AmWindowClass *pWindow,
			    AmGCClass     *pGC,
			    AmPointPtr    pPoint,
			    int           nPoint)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    XPoint       *pxPoint;
    int          ii;

/* Allocate storage locally for the X points. */

    pxPoint = ( XPoint *) alloca( sizeof( XPoint) * nPoint);
    if ( pxPoint == NULL) {
        ANVIL_ERROR_CODE = AmALLOCA_FAILED;
        return;
    }

/* Translate from "virtual" coordinates to "screen" coordinates. */

    for ( ii = 0; ii < nPoint; ii++) {
        pxPoint[ ii].x = SCREEN_COORD( pPoint[ ii].x);
        pxPoint[ ii].y = SCREEN_COORD( pPoint[ ii].y);
    }

    XDrawLines( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
		pxPoint, nPoint, CoordModeOrigin);
}

/* Call the XDrawSegments function from the wrapper */

void
AmXDisplayClass::DrawSegments( AmWindowClass *pWindow,
                             AmGCClass     *pGC,
                             AmSegmentRec  *pSegment,
                             int         nSegment) 
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    XSegment     *pxSegment;
    int          ii;

/* Allocate storage locally for the X segments. */

    pxSegment = ( XSegment *) alloca( sizeof( XSegment) * nSegment);
    if ( pxSegment == NULL) {
        ANVIL_ERROR_CODE = AmALLOCA_FAILED;
        return;
    }
  
/* Translate from "virtual" coordinates to "screen" coordinates. */

    for ( ii = 0; ii < nSegment; ii++) {
        pxSegment[ ii].x1 = SCREEN_COORD( pSegment[ ii].x0);
        pxSegment[ ii].y1 = SCREEN_COORD( pSegment[ ii].y0);
        pxSegment[ ii].x2 = SCREEN_COORD( pSegment[ ii].x1);
        pxSegment[ ii].y2 = SCREEN_COORD( pSegment[ ii].y1);
    }

    XDrawSegments( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
		   pxSegment, nSegment);
}

/* Call the XDrawPoints function from the wrapper */

void
AmXDisplayClass::DrawPoints( AmWindowClass *pWindow,
                             AmGCClass     *pGC,
                             AmPointPtr    pPoint,
                             int           nPoint)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    XPoint       *pxPoint;
    int          ii;

/* Allocate storage locally for the X points. */

    pxPoint = ( XPoint *) alloca( sizeof( XPoint) * nPoint);
    if ( pxPoint == NULL) {
        ANVIL_ERROR_CODE = AmALLOCA_FAILED;
        return;
    }

/* Translate from "virtual" coordinates to "screen" coordinates. */

    for ( ii = 0; ii < nPoint; ii++) {
        pxPoint[ ii].x = SCREEN_COORD( pPoint[ ii].x);
        pxPoint[ ii].y = SCREEN_COORD( pPoint[ ii].y);
    }

    XDrawPoints( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
		 pxPoint, nPoint, CoordModeOrigin);
}

/* Draw an arc starting at angle1, of angle2 width */

void
AmXDisplayClass::DrawArc( AmWindowClass *pWindow,
                        AmGCClass     *pGC,
                        int         x,
                        int         y,
                        int         cx,
                        int         cy,
                        float       dAngleStart,
                        float       dAngleEnd)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = x;
    rctVirtual.y      = y;
    rctVirtual.width  = cx;
    rctVirtual.height = cy;

    ScreenRect( &rctVirtual, &rctScreen); 

    XDrawArc( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
              rctScreen.x, rctScreen.y,
              rctScreen.width, rctScreen.height,
	      (( int) RADTODEG( dAngleStart)) * 64,
              (( int) RADTODEG( dAngleEnd)) * 64);
}

/* Draw a filled arc starting at angle1, of angle2 width */

void
AmXDisplayClass::FillArc( AmWindowClass *pWindow,
			AmGCClass     *pGC,
			int         x,
			int         y,
			int         cx,
			int         cy,
			float       dAngleStart,
			float       dAngleEnd)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = x;
    rctVirtual.y      = y;
    rctVirtual.width  = cx;
    rctVirtual.height = cy;

    ScreenRect( &rctVirtual, &rctScreen); 

    XFillArc( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
              rctScreen.x, rctScreen.y,
              rctScreen.width + 1, rctScreen.height + 1,
	      (( int) RADTODEG( dAngleStart)) * 64,
              (( int) RADTODEG( dAngleEnd)) * 64);
}

/* Copy a rectangle in a window */

void
AmXDisplayClass::CopyArea( AmWindowClass *pSrcWindow,
			 AmWindowClass *pDstWindow,
			 AmGCClass     *pGC,
			 int	     xSrc,
			 int	     ySrc,
			 int	     xDst,
			 int	     yDst,
			 int         cxSrc,
			 int         cySrc)
{
    AmXWindowClass *pXSrcWindow = ( AmXWindowClass *) pSrcWindow,
		 *pXDstWindow = ( AmXWindowClass *) pDstWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = xSrc;
    rctVirtual.y      = ySrc;
    rctVirtual.width  = cxSrc;
    rctVirtual.height = cySrc;

    ScreenRect( &rctVirtual, &rctScreen); 

    XCopyArea( _pxDisplay, pXSrcWindow->GetXWindow(),
               pXDstWindow->GetXWindow(), pXGC->GetXGC(), 
               rctScreen.x, rctScreen.y,
               rctScreen.width + 1, rctScreen.height + 1,
               SCREEN_COORD( xDst),
               SCREEN_COORD( yDst));
}

/* Copy from the screen to an image */

void
AmXDisplayClass::GetImage( AmWindowClass *pWindow,
			 AmImageClass  *pImage,
                         int         x,
                         int         y)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXImageClass	 *pXImage = ( AmXImageClass *) pImage;
    AmRectangleRec rctVirtual, rctScreen;

#if 0
    if ( pXImage->getShmInfo() != NULL) {
	XShmGetImage( _pxDisplay, pXWindow->GetXWindow(), pXImage->GetXImage(), 
		      SCREEN_COORD( x),
                      SCREEN_COORD( y),
                      AllPlanes);
    } else {

        rctVirtual.x      = x;
        rctVirtual.y      = y;
        rctVirtual.width  = pXImage->GetWidth();
        rctVirtual.height = pXImage->GetHeight();

	ScreenRect( &rctVirtual, &rctScreen); 
	XGetSubImage( _pxDisplay, pXWindow->GetXWindow(), 
                      rctScreen.x, rctScreen.y,
                      rctScreen.width, rctScreen.height,
                      AllPlanes, XYPixmap, 
		      pXImage->GetXImage(), 0, 0);
    }
#endif
    rctVirtual.x      = x;
    rctVirtual.y      = y;
    rctVirtual.width  = pXImage->GetWidth();
    rctVirtual.height = pXImage->GetHeight();

    ScreenRect( &rctVirtual, &rctScreen); 
    XGetSubImage( _pxDisplay, pXWindow->GetXWindow(), 
		  rctScreen.x, rctScreen.y,
		  rctScreen.width, rctScreen.height,
		  AllPlanes, XYPixmap, 
		  pXImage->GetXImage(), 0, 0);

/* Copy data from the display image back to the source image. */

    pXImage->RevCopyData();
}

/* Draw a glyph to the screen. */

void
AmXDisplayClass::DrawGlyph( AmWindowClass *pWindow,
			  AmGlyphClass  *pGlyph,
			  int         x,
			  int         y)
{
    pGlyph->DrawGlyph( this, pWindow, x, y);
}

/* Put an image on the screen.  Get the X image and shared memory information
   from the image class. */

void
AmXDisplayClass::DrawImage( AmWindowClass *pWindow,
			  AmGCClass     *pGC,
			  int         xSrc,
			  int         ySrc,
			  int	      xDst,
			  int	      yDst,
			  int         cxSrc,
			  int         cySrc,
			  AmImageClass  *pImage)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    AmXImageClass	 *pXImage = ( AmXImageClass *) pImage;
    GC		 xGCMask;
    AmColorCellRec col;
    XGCValues	 xGCV;
    int		 iDepth;
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = xSrc;
    rctVirtual.y      = ySrc;
    rctVirtual.width  = cxSrc;
    rctVirtual.height = cySrc;

    ScreenRect( &rctVirtual, &rctScreen); 

/* If the image depth is 1, then the image is assumed to be a glyph, where
   0 values aren't drawn, and 1 values are drawn in the foreground color. */

    if ( pXImage->GetDepth() == 1) {

/* Using a 0xff.. mask, draw the image in OR, which causes all the places
   where the image gets drawn to be set to 0xf... */

	iDepth = AmGetAppDepth();
	xGCV.foreground = ( 0x1 << iDepth) - 1;
	xGCV.function = GXor;
	xGCV.background = 0;
	xGCMask = XCreateGC( _pxDisplay, DefaultRootWindow( _pxDisplay),
			     GCForeground | GCBackground | GCFunction, &xGCV);
	AmShmShowImage( _pxDisplay, pXWindow->GetXWindow(), xGCMask,
		      pXImage->GetXImage(), 
                      rctScreen.x, rctScreen.y,
                      SCREEN_COORD( xDst),
                      SCREEN_COORD( yDst),
                      DimInPixels( rctScreen.width),
                      DimInPixels( rctScreen.height),
		      pXImage->getShmInfo());

/* Draw the image using AND, where all the 0xf... gets replaced by the 
   actual foreground color. */

        pXGC->GetForegroundColor( &col);
	XSetForeground( _pxDisplay, xGCMask, col.pixel);
	XSetBackground( _pxDisplay, xGCMask, ( 0x1 << iDepth) - 1);
	XSetFunction( _pxDisplay, xGCMask, GXand);
	AmShmShowImage( _pxDisplay, pXWindow->GetXWindow(), xGCMask,
		      pXImage->GetXImage(), 
                      rctScreen.x, rctScreen.y,
                      SCREEN_COORD( xDst),
                      SCREEN_COORD( yDst),
                      DimInPixels( rctScreen.width),
                      DimInPixels( rctScreen.height),
		      pXImage->getShmInfo());

/* Free up our temporary GC */

	XFreeGC( _pxDisplay, xGCMask);
    } else {
        pXImage->CopyData();
	AmShmShowImage( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(), 
		      pXImage->GetXImage(), 
                      rctScreen.x, rctScreen.y,
                      SCREEN_COORD( xDst),
                      SCREEN_COORD( yDst),
                      DimInPixels( rctScreen.width),
                      DimInPixels( rctScreen.height),
		      pXImage->getShmInfo());
    }
}

/* Draw text on the screen.  This is only for 8-bit text */

void
AmXDisplayClass::DrawText( AmWindowClass *pWindow,
			 AmGCClass     *pGC,
			 int         x,
			 int         y,
			 char        *szText)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;

    XDrawString( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
		 SCREEN_COORD( x),
                 SCREEN_COORD( y),
                 szText, strlen( szText));
}

/* Draw text on the screen.  This is only for 8-bit text */

void
AmXDisplayClass::DrawImageText( AmWindowClass *pWindow,
			      AmGCClass     *pGC,
			      int         x,
			      int         y,
			      char        *szText)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;

    XDrawImageString( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
                      SCREEN_COORD( x),
                      SCREEN_COORD( y),
                      szText, strlen( szText));
}

/* Draw rotated text on the screen, using the draw rotated text functions */

void
AmXDisplayClass::DrawRotatedText( AmWindowClass *pWindow,
				AmGCClass     *pGC,
				int         x,
				int         y,
				float       dAngle,
				char        *szText)
{ 
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;

    AmXRotDrawString( _pxDisplay, pXGC->GetXFont(),
                    RADTODEG( dAngle),
                    pXWindow->GetXWindow(), pXGC->GetXGC(),
                    SCREEN_COORD( x),
                    SCREEN_COORD( y),
                    szText);
}

/* Return the extents of a text string if it were to be drawn to the screen */

void
AmXDisplayClass::ExtentsText( AmGCClass *pGC,
			    char    *szText,
			    int     *piWidth,
			    int     *piAscent,
			    int     *piDescent)
{  
    int		iDir, iAscent, iDescent;
    XCharStruct	xChar;
    AmXGCClass	*pXGC = ( AmXGCClass *) pGC;

    XTextExtents( pXGC->GetXFont(), szText, strlen( szText),
		  &iDir, &iAscent, &iDescent, &xChar);

    *piWidth = VIRTUAL_COORD( xChar.width);
    *piAscent = VIRTUAL_COORD( iAscent);
    *piDescent = VIRTUAL_COORD( iDescent);
}


int
AmXDisplayClass::TextWidth( AmGCClass *pGC,
                          char    *szText,
                          int     iLength)
{  
    AmXGCClass *pXGC = ( AmXGCClass *) pGC;
    int      iWidth;

    if ( iLength != 0) {
        iWidth = XTextWidth( pXGC->GetXFont(), szText, iLength);
    } else {
        iWidth = XTextWidth( pXGC->GetXFont(), szText, strlen( szText));
    }

    return( VIRTUAL_COORD( iWidth));
}

int
AmXDisplayClass::FontHeight( AmGCClass *pGC,
                           int     *piAscent,
                           int     *piDescent)
{
    AmXGCClass	*pXGC = ( AmXGCClass *) pGC;
    XFontStruct *pxFont = pXGC->GetXFont();

    *piAscent = VIRTUAL_COORD( pxFont->ascent);
    *piDescent = VIRTUAL_COORD( pxFont->descent);

    return( *piAscent + *piDescent);
}

/* Fill a polygon */

void
AmXDisplayClass::FillPolygon( AmWindowClass *pWindow,
                              AmGCClass     *pGC,
                              AmPointPtr    pPoint,
                              int           nPoint)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    XPoint       *pxPoint;
    int          ii;

/* Allocate storage locally for the X points. */

    pxPoint = ( XPoint *) alloca( sizeof( XPoint) * nPoint);
    if ( pxPoint == NULL) {
        ANVIL_ERROR_CODE = AmALLOCA_FAILED;
        return;
    }

/* Translate from "virtual" coordinates to "real" screen coordinates. */

    for ( ii = 0; ii < nPoint; ii++) {
        pxPoint[ ii].x = SCREEN_COORD( pPoint[ ii].x);
        pxPoint[ ii].y = SCREEN_COORD( pPoint[ ii].y);
    }

    XFillPolygon( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
		  pxPoint, nPoint, Nonconvex, CoordModeOrigin);
}

/* Draw a rectangle */

void
AmXDisplayClass::DrawRectangle( AmWindowClass  *pWindow,
				AmGCClass      *pGC,
				int            x,
				int            y,
				int            cx,
				int            cy)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = x;
    rctVirtual.y      = y;
    rctVirtual.width  = cx;
    rctVirtual.height = cy;

    ScreenRect( &rctVirtual, &rctScreen); 

    XDrawRectangle( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
                    rctScreen.x, rctScreen.y,
                    rctScreen.width, rctScreen.height);
}

/* Fill a rectangle */

void
AmXDisplayClass::FillRectangle( AmWindowClass  *pWindow,
                              AmGCClass      *pGC,
                              int          x,
                              int          y,
                              int          cx,
                              int          cy)
{
    AmXWindowClass *pXWindow = ( AmXWindowClass *) pWindow;
    AmXGCClass	 *pXGC = ( AmXGCClass *) pGC;
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = x;
    rctVirtual.y      = y;
    rctVirtual.width  = cx;
    rctVirtual.height = cy;

    ScreenRect( &rctVirtual, &rctScreen); 

    XFillRectangle( _pxDisplay, pXWindow->GetXWindow(), pXGC->GetXGC(),
                    rctScreen.x, rctScreen.y,
                    rctScreen.width + 1, rctScreen.height + 1);
}

/* Given 0-255 values for Red, Green and Blue, return the allocated
   pixel value */

AnvilBoolean
AmXDisplayClass::AllocColor( AmColorCellPtr pCol)
{
    AmSetPixelValue( pCol);
    return( TRUE);
}

/* Given a string name for a color, return the RGB values */

AnvilBoolean
AmXDisplayClass::AllocNamedColor( const char   *szName,
                                AmColorCellPtr pCol)
{
    XColor  xColor;
    AnvilBoolean fParsed;

    fParsed = XParseColor( _pxDisplay, _xColormap, szName, &xColor);

    if ( fParsed) {
        pCol->red = xColor.red >> 8;
        pCol->green = xColor.green >> 8;
        pCol->blue = xColor.blue >> 8;
        AmSetPixelValue( pCol);
    }

    return( fParsed);
}

/* Create an image (virtual function semi-constructor */

AmImageClass *
AmXDisplayClass::CreateImage( int	    cx,
			    int	    cy,
			    int	    iDepth,
			    char    *pData)
{
    return(( AmImageClass *) 
		new AmXImageClass( this, cx, cy, iDepth, pData));
}


/* Virtual function semi-contstructor to create glyphs. */

AmGlyphClass *
AmXDisplayClass::CreateGlyph( AmWindowClass	*pWindow,
			    char        *szFile,
			    ulong_t     lPixel)
{
    return(( AmGlyphClass *) new AmXGlyphClass( this, pWindow, szFile, lPixel));
}

AmRegionClass *
AmXDisplayClass::CreateRegion()
{
    return(( AmRegionClass *) new AmXRegionClass( this));
}

/* temporary functions for creating buffers and windows */

AmWindowClass *
AmXDisplayClass::CreateBuffer( AmWindowClass	*pWindow,
			     int		cx,
			     int		cy)
{
    AmXWindowClass	*pXWindow  = ( AmXWindowClass *) pWindow;
    AmXDisplayClass	*pXDisplay = ( AmXDisplayClass *) pXWindow->GetDisplay();
    Display		*pxDisplay = pXDisplay->GetXDisplay();
    Pixmap		xPixmap;
    AmColorCellRec	col;

    col.red   = 0xff;
    col.green = 0xff;
    col.blue  = 0xff;

    xPixmap = XCreatePixmap( pxDisplay, DefaultRootWindow( pxDisplay),
                             cx, cy, 
			     DefaultDepth( pxDisplay,
                                           DefaultScreen( pxDisplay)));

    return( new AmXWindowClass( pXDisplay, xPixmap, &col));
}

AmWindowClass *
AmXDisplayClass::CreateWin( AmWindowClass	*pWindow,
			  int		x,
			  int		y,
			  int		cx,
			  int		cy)
{
    XSetWindowAttributes	attr;
    Display			*display = GetXDisplay();
    Window			newWin, parentWin;

    attr.save_under = TRUE;
    attr.background_pixel = WhitePixel( display, DefaultScreen( display));
    attr.border_pixel = BlackPixel( display, DefaultScreen( display));
    attr.backing_store = Always;
    parentWin = (( AmXWindowClass *) pWindow)->GetXWindow();
    newWin = XCreateWindow( display, parentWin,
			    x, y, cx, cy, 1,
			    CopyFromParent, CopyFromParent, CopyFromParent,
			    CWSaveUnder | CWBackPixel |
			  	CWBorderPixel | CWBackingStore, &attr);
    XMapWindow( display, newWin);
    return( new AmXWindowClass( this, newWin));
}

void
AmXWindowClass::ResizeWindow( int	cx,
			    int cy)
{
    Display      *pxDisplay = _pXDisplay->GetXDisplay();
    AmRectangleRec rctVirtual, rctScreen;

    rctVirtual.x      = 0;
    rctVirtual.y      = 0;
    rctVirtual.width  = cx;
    rctVirtual.height = cy;

    _pXDisplay->ScreenRect( &rctVirtual, &rctScreen); 

    if ( _eWindowType == AmWINDOW_WINDOW) {
	if ( _xWindow != 0) {
	    XResizeWindow( pxDisplay, _xWindow,
                           rctScreen.width, rctScreen.height);
	}
    } else if ( _eWindowType == AmWINDOW_PIXMAP) {
	XFreePixmap( pxDisplay, _xWindow);
	_xWindow = XCreatePixmap( pxDisplay, DefaultRootWindow( pxDisplay), 
                                  rctScreen.width, rctScreen.height,
				  DefaultDepth( pxDisplay, 
						DefaultScreen( pxDisplay)));
    }
}

void 
AmXWindowClass::MoveWindow( int x,
			  int y)
{
    Display *pxDisplay = _pXDisplay->GetXDisplay();

    if ( _xWindow != 0) {
	XMoveWindow( pxDisplay, _xWindow,
                     _pXDisplay->ScreenCoord( x), _pXDisplay->ScreenCoord( y));
    } 
}

/**************************
 *
 * AmXWindowClass
 *
 **************************/

/* Do nothing constructor. */

AmXWindowClass::AmXWindowClass()
    : AmWindowClass()
{
    _eWindowType = AmWINDOW_ERROR;

    _xWindow = 0;
    _xWidget = NULL;
    _pXDisplay = NULL;
}


/* Create an X window class from a widget. */

AmXWindowClass::AmXWindowClass( AmXDisplayClass *pXDisplay,
                            Widget        w)
    : AmWindowClass( pXDisplay)
{
    _eWindowType = AmWINDOW_WIDGET;

    _xWidget = w;
    _pXDisplay = pXDisplay;
    _xWindow = XtWindow( w);

    _colBackground.red = 0;
    _colBackground.green = 0;
    _colBackground.blue = 0;
    _colBackground.pixel = 0;
}

/* Create an X window class from a window. */

AmXWindowClass::AmXWindowClass( AmXDisplayClass *pXDisplay,
                            Window        xWindow)
    : AmWindowClass( pXDisplay)
{
    _eWindowType = AmWINDOW_WINDOW;

    _pXDisplay = pXDisplay;
    _xWindow = xWindow;
    _xWidget = NULL;

    _colBackground.red = 0;
    _colBackground.green = 0;
    _colBackground.blue = 0;
    _colBackground.pixel = 0;
}

/* Create an X window class from a pixmap and a display */

AmXWindowClass::AmXWindowClass( AmXDisplayClass *pXDisplay,
			    Pixmap	  xPixmap,
                            AmColorCellPtr  pcolBackground)
    : AmWindowClass( pXDisplay)
{
    _eWindowType = AmWINDOW_PIXMAP;

    _pXDisplay = pXDisplay;
    _xWindow = xPixmap;
    _xWidget = NULL;
    _colBackground = *pcolBackground;

    AmSetPixelValue( &_colBackground);
}

/*
 * Function: Destructor.
 *
 * Description:
 *   If the window is a real window or a pixmap, then destroy it.  If it's
 *   a widget, then the we're getting called because the widget is getting
 *   destroyed.
 */
 
AmXWindowClass::~AmXWindowClass()
{
    if ( _eWindowType == AmWINDOW_WINDOW) {
	if ( GetXWindow() != 0) {
	    XDestroyWindow( _pXDisplay->GetXDisplay(), GetXWindow());
	}
    } else if ( _eWindowType == AmWINDOW_PIXMAP) {
	if ( GetXWindow() != 0) {
	    XFreePixmap( _pXDisplay->GetXDisplay(), GetXWindow());
	}
    }
}

/* Return the window's X window XID */

Window AmXWindowClass::GetXWindow()
{ 
    if ( _xWidget != NULL) {
	return( XtWindow( _xWidget));
    } else {
	return( _xWindow);
    }
}

/* return the window's widget if there is one */


Widget AmXWindowClass::GetXWidget()
{ 
    if ( _xWidget != NULL) {
	return( _xWidget);
    } else {
	return( NULL);
    }
}

/* return the AmXDisplayClass that the window was created with */

AmDisplayClass *
AmXWindowClass::GetDisplay()
{
    return( _pXDisplay);
}

/* Clear a section of a window.  If it's a pixmap, then we fill the
   rectangle with the background color. */

void
AmXWindowClass::ClearRect( AmRectangleRec const *pRect)
{
    XGCValues    xGCV;
    GC           xGC;
    uint_t       cx, cy, iBorder, iDepth;
    int          x, y;
    Window       xWindowRoot;
    AmRectangleRec rctScreen;

    if ( _eWindowType == AmWINDOW_PIXMAP) {

    /* If the window is a pixmap, fill it with the background color. */

	xGCV.foreground = _colBackground.pixel;
	xGC = XCreateGC( _pXDisplay->GetXDisplay(), 
			 DefaultRootWindow( _pXDisplay->GetXDisplay()),
			 GCForeground, &xGCV);

	if ( pRect != NULL) {

            _pXDisplay->ScreenRect( pRect, &rctScreen); 

	    XFillRectangle( _pXDisplay->GetXDisplay(), GetXWindow(), xGC, 
                            rctScreen.x, rctScreen.y,
                            rctScreen.width + 1, rctScreen.height + 1);
	} else {
	    XGetGeometry( _pXDisplay->GetXDisplay(), GetXWindow(),
                          &xWindowRoot, &x, &y, &cx, &cy, &iBorder, &iDepth);
	    XFillRectangle( _pXDisplay->GetXDisplay(), GetXWindow(), xGC, 
			    0, 0, cx + 1, cy + 1);
	}

	XFreeGC( _pXDisplay->GetXDisplay(), xGC);

    } else {

    /* Otherwise clear the x window. */ 

	if ( pRect != NULL) {

            _pXDisplay->ScreenRect( pRect, &rctScreen); 

	    XClearArea( _pXDisplay->GetXDisplay(), GetXWindow(),
                        rctScreen.x, rctScreen.y,
                        rctScreen.width + 1, rctScreen.height + 1,
                        FALSE);
	} else {
	    XClearWindow( _pXDisplay->GetXDisplay(), GetXWindow());
	}
    }
}

/* Return the geometry of a window */

void
AmXWindowClass::GetWindowGeometry( int *px,
                                 int *py,
                                 int *piWidth,
                                 int *piHeight)
{

    if ( _xWidget == NULL) {
	unsigned int iWidth, iHeight, iBorder, iDepth;
	int          x, y;
	Window       xWindowRoot;

	XGetGeometry( _pXDisplay->GetXDisplay(), GetXWindow(), &xWindowRoot,
		      &x, &y, &iWidth, &iHeight, &iBorder, &iDepth);

	*px = _pXDisplay->VirtualCoord( x);
	*py = _pXDisplay->VirtualCoord( y);
	*piWidth = _pXDisplay->VirtualCoord( iWidth);
	*piHeight = _pXDisplay->VirtualCoord( iHeight);
    } else {
	Position	x, y;
	Dimension	width, height;

	XtVaGetValues( _xWidget, 
		       XmNx, &x,
		       XmNy, &y,
		       XmNwidth, &width,
		       XmNheight, &height,
		       NULL);
	*px = x;
	*py = y;
	*piWidth = width;
	*piHeight = height;
    }
}

/* Return the extents of a window */

void
AmXWindowClass::GetDims( int *piWidth,
                       int *piHeight) 
{
    unsigned int iWidth, iHeight, iBorder, iDepth;
    int          x, y;
    Window       xWindowRoot;

    XGetGeometry( _pXDisplay->GetXDisplay(), GetXWindow(), &xWindowRoot, 
		 &x, &y, &iWidth, &iHeight, &iBorder, &iDepth);

    *piWidth = _pXDisplay->VirtualCoord( iWidth);
    *piHeight = _pXDisplay->VirtualCoord( iHeight);
}

/* Create a new GC */

AmGCClass *
AmXWindowClass::CreateGC()
{
    return( new AmXGCClass( this));
}

void
AmXWindowClass::Update()
{
    AmWindowClass::Update();
}
