#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */






#include <stdio.h>
#include <string.h>
#include <math.h>
#include <search.h>
#include <malloc.h>
#if !defined (__BSD44__)
#include <alloca.h>
#endif
#include <X11/Xlib.h>
#include <anvil/errlib.h>
#include <anvil/shmimg.h>
#include <anvil/mem_funcs.h>
#include <anvil/xdisplay.h>
#include <anvil/xfonts.h>
#include <anvil/xcolor.h>

#define SCREEN_COORD( iVirtual)                                            \
     ( iVirtual * _pDisplay->GetDPI() / _pDisplay->GetDPIBase()) 
#define VIRTUAL_COORD( iScreen)                                            \
     ( iScreen * _pDisplay->GetDPIBase() / _pDisplay->GetDPI()) 

#define SCREEN_COORD_FLOAT( dVirtual)                                      \
     ( dVirtual * _pDisplay->GetDPI() / ( float) _pDisplay->GetDPIBase()) 

/* X GC constructor. */

AmXGCClass::AmXGCClass( AmXWindowClass *pXWindow)
    : AmGCClass( pXWindow)
{
    _pXDisplay = ( AmXDisplayClass *) pXWindow->GetDisplay();

    _pcDashPattern = NULL;
    _nDashPattern = 0;
    _iDashOffset = 0;

    _pClipRectangle = NULL;
    _nClipRectangle = 0;

    _xGC = XCreateGC( _pXDisplay->GetXDisplay(), 
		      DefaultRootWindow( _pXDisplay->GetXDisplay()), 0, NULL);

    _pxFont = NULL;

    _dLineWidth = 0.0;

    _stipplePixmap = 0;
    _fillPatFile = NULL;

    _font.szFont = strdup( ""); 
    _font.szFamily = strdup( ""); 
    _font.eWeight = DEFAULT_WEIGHT;
    _font.eSlant = DEFAULT_SLANT;
    _font.iSize = DEFAULT_SIZE; 
    _font.dxScale = 1.0;
    _font.dyScale = 1.0;
    _font.iDPI = _pDisplay->GetDPI();
  
    SetFont( DEFAULT_FONT, DEFAULT_SIZE, DEFAULT_WEIGHT, DEFAULT_SLANT);
}

/* X GC Destructor. */

AmXGCClass::~AmXGCClass()
{

/* Free the font information and the dash pattern. */

    if ( _pxFont != NULL) {
        XUnloadFont( _pXDisplay->GetXDisplay(), _pxFont->fid);
        XFreeFontInfo( NULL, _pxFont, 1);
    }

    if ( _font.szFont != NULL) {
        FREE( _font.szFont);
    }

    if ( _font.szFamily != NULL) {
        FREE( _font.szFamily);
    }

/* Free the dash patterns. */

    if ( _pcDashPattern != NULL) {
	FREE( _pcDashPattern);
    } 

/* Get rid of the GC. */

    if ( _xGC != NULL) {
        XFreeGC( _pXDisplay->GetXDisplay(), _xGC);
    }

/* Free the vector of clip rectangles if there are any. */

    if ( _pClipRectangle != NULL) {
	FREE( _pClipRectangle);
    }

/* free the stipple pixmap if it was used. */
    if ( _stipplePixmap != 0) {
	XFreePixmap( _pXDisplay->GetXDisplay(), _stipplePixmap);
	_stipplePixmap = 0;
    }
}

/* Return the X windows GC for a GC class */

GC
AmXGCClass::GetXGC()
{
    return( _xGC);
}

/* Set the line width */

void
AmXGCClass::SetLineWidth( float dLineWidth)
{
    XGCValues xGCV;
 
    _dLineWidth = dLineWidth; 

    xGCV.line_width = ( int)( SCREEN_COORD_FLOAT( dLineWidth) + 0.5);
    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCLineWidth, &xGCV);
}

/* return the line width */

float
AmXGCClass::GetLineWidth()
{
    return( _dLineWidth);
}

/* Set the line join */

void
AmXGCClass::SetLineJoin( AmLineJoinEnum eLineJoin)
{
    XGCValues xGCV;

    switch ( eLineJoin) {
    case AmJOIN_MITER:
	xGCV.join_style = JoinMiter;
	break;

    case AmJOIN_ROUND:
	xGCV.join_style = JoinRound;
	break;

    case AmJOIN_BEVEL:
	xGCV.join_style = JoinBevel;
	break;

    default:
	xGCV.join_style = JoinMiter;
	break;
    }

    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCJoinStyle, &xGCV);
}

/* Retreive the line join */

AmLineJoinEnum
AmXGCClass::GetLineJoin() 
{
    XGCValues xGCV;

    XGetGCValues( _pXDisplay->GetXDisplay(), _xGC, GCJoinStyle, &xGCV);

    switch ( xGCV.join_style) {
    case JoinMiter:
	return( AmJOIN_MITER);
    case JoinRound:
	return( AmJOIN_ROUND);
    case JoinBevel:
	return( AmJOIN_BEVEL);
    default:
	return( AmJOIN_MITER);
    }
}

/* Set the line style */

void
AmXGCClass::SetLineStyle( AmLineStyleEnum eLineStyle)
{
    XGCValues xGCV;

    switch ( eLineStyle) {
    case AmLINE_SOLID:
	xGCV.line_style = LineSolid;
	break;
    case AmLINE_ON_OFF_DASH:
	xGCV.line_style = LineOnOffDash;
	break;
    case AmLINE_DOUBLE_DASH:
	xGCV.line_style = LineDoubleDash;
	break;
    default:
	xGCV.line_style = LineSolid;
	break;
    }

    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCLineStyle, &xGCV);
}

/* Retreive the line style */

AmLineStyleEnum
AmXGCClass::GetLineStyle() 
{
    XGCValues xGCV;

    XGetGCValues( _pXDisplay->GetXDisplay(), _xGC, GCLineStyle, &xGCV);
    switch ( xGCV.line_style) {
    case LineSolid:
	return( AmLINE_SOLID);
    case LineOnOffDash:
	return( AmLINE_ON_OFF_DASH);
    case LineDoubleDash:
	return( AmLINE_DOUBLE_DASH);
    default:
	return( AmLINE_SOLID);
    }
}

/* Retrieve the X font structure from the GC. */

XFontStruct *
AmXGCClass::GetXFont()
{
    XGCValues	xGCV;

    if ( _pxFont == NULL) {
        _pxFont = AmLoadQueryFont( _pXDisplay->GetXDisplay(), 
                                 _font.szFont, _font.iSize,
                                 _font.eWeight, _font.eSlant,
                                 _font.dxScale, _font.dyScale,
                                 _pDisplay->GetDPI());

        if ( _pxFont != NULL) {
            xGCV.font = _pxFont->fid;
            XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCFont, &xGCV);
        }
    }

    return( _pxFont);
}

/* Set the font, and font sizing... */

AnvilBoolean
AmXGCClass::SetFont( char 	  *szFont, 
                   int 		  iSize,
                   AmFontWeightEnum eWeight,
                   AmFontSlantEnum  eSlant,
                   float          dxScale,
                   float          dyScale)
{
    XFontStruct *pxFont = NULL;
    XGCValues	xGCV;

  /* See if anything has changed before we do anything. */

    if (( strcmp( _font.szFont, szFont) != 0) ||
        ( _font.iSize != iSize) ||
        ( _font.eWeight != eWeight) ||
        ( _font.eSlant != eSlant) ||
        ( _font.dxScale != dxScale) ||
        ( _font.dyScale != dyScale) ||
        ( _font.iDPI != _pDisplay->GetDPI())) {

      /* Attempt to load the new font. */

        pxFont = AmLoadQueryFont( _pXDisplay->GetXDisplay(),
                                szFont, iSize, eWeight, eSlant,
                                dxScale, dyScale, _pDisplay->GetDPI());

        if ( pxFont != NULL) {

          /* Free the old font and font info. */

            XGetGCValues( _pXDisplay->GetXDisplay(), _xGC, GCFont, &xGCV);
            if ( _pxFont != NULL) {
                XUnloadFont( _pXDisplay->GetXDisplay(), _pxFont->fid);
                XFreeFontInfo( NULL, _pxFont, 1);
            }

          /* Set the new font. */

            _pxFont = pxFont;


            if ( strcmp( _font.szFont, szFont) != 0) { 
                FREE( _font.szFont);
                _font.szFont = strdup( szFont);
            }

            _font.eWeight = eWeight;
            _font.eSlant = eSlant;
            _font.iSize = iSize;
            _font.dxScale = dxScale;
            _font.dyScale = dyScale;
            _font.iDPI = _pDisplay->GetDPI();

	    xGCV.font = _pxFont->fid;
	    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCFont, &xGCV);

        } else {

	    AN_DEBUG( 10, ("Failed to load font %s\n", szFont));
	    return( FALSE);

        }
    }

    return( TRUE);
}


AnvilBoolean
AmXGCClass::SetFont( AmFontRec *pFont)
{
    if ( pFont != NULL) {
        return( SetFont( pFont->szFont, pFont->iSize, pFont->eWeight,
                         pFont->eSlant, pFont->dxScale, pFont->dyScale));
    }

    return( FALSE);
}

/* get the current font name */

void
AmXGCClass::GetFont( char           **pszFont,
		   int	          *piSize,
                   AmFontWeightEnum *peWeight,
                   AmFontSlantEnum  *peSlant,
                   float          *pdxScale,
                   float          *pdyScale)
{

    if ( pszFont != NULL) {
        *pszFont = strdup( _font.szFont);
    }

    if ( piSize != NULL) {
        *piSize = _font.iSize;
    }

    if ( peWeight != NULL) {
        *peWeight = _font.eWeight;
    }

    if ( peSlant != NULL) {
        *peSlant = _font.eSlant;
    }

    if ( pdxScale != NULL) {
        *pdxScale = _font.dxScale;
    }

    if ( pdyScale != NULL) {
        *pdyScale = _font.dyScale;
    }
}


void
AmXGCClass::GetFont( AmFontRec *pFont)
{
    if ( pFont != NULL) {
        pFont->szFont = strdup( _font.szFont);
        pFont->iSize = _font.iSize;
        pFont->eWeight = _font.eWeight;
        pFont->eSlant = _font.eSlant;
        pFont->dxScale = _font.dxScale;
        pFont->dyScale = _font.dyScale;
    }
}


/* get and set the foreground pixel */

void
AmXGCClass::SetForegroundColor( AmColorCellPtr pCol)
{
    XGCValues xGCV;

    _colForeground.red = pCol->red;
    _colForeground.green = pCol->green;
    _colForeground.blue = pCol->blue;
    AmSetPixelValue( &_colForeground);

    xGCV.foreground = _colForeground.pixel;
    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCForeground, &xGCV);
}

void
AmXGCClass::GetForegroundColor( AmColorCellPtr pCol)
{
    *pCol = _colForeground;
}


/* set and get the background pixel */

void
AmXGCClass::SetBackgroundColor( AmColorCellPtr pCol)
{
    XGCValues xGCV;

    _colBackground.red = pCol->red;
    _colBackground.green = pCol->green;
    _colBackground.blue = pCol->blue;
    AmSetPixelValue( &_colBackground);

    xGCV.background = _colBackground.pixel;
    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCBackground, &xGCV);
}

void
AmXGCClass::GetBackgroundColor( AmColorCellPtr pCol)
{
    *pCol = _colBackground;
}


/* Set the vector of clipping rectangles */

void
AmXGCClass::SetClipRectangles( AmRectangleRec const	*pClipRectangle,
                               int                	nClipRectangle)
{
    XRectangle	   *pxClipRectangle;
    AmRectangleRec rctScreen;
    Region         xRegion;
    int		   ii;

    if ( _pClipRectangle != NULL) {
	FREE( _pClipRectangle);
    }
	
    pxClipRectangle = 
	( XRectangle *) alloca( nClipRectangle*sizeof( XRectangle));
    if ( pxClipRectangle == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return;
    }
    _nClipRectangle = nClipRectangle;
    _pClipRectangle = ( AmRectanglePtr) 
	AmCopyVec( pClipRectangle, nClipRectangle*sizeof( AmRectangleRec));
    if ( _pClipRectangle == NULL) {
	DO_ERROR( AmMALLOC_FAILED);
	return;
    }

    /* Copy to a vector of X clip rectangles and translate from "virtual"
       coordinates into "real" screen coordinates. */

    xRegion = XCreateRegion();

    for ( ii = 0; ii < nClipRectangle; ii++) {

        _pDisplay->ScreenRect( &pClipRectangle[ ii], &rctScreen);

	pxClipRectangle[ ii].x      = rctScreen.x; 
	pxClipRectangle[ ii].y      = rctScreen.y; 
	pxClipRectangle[ ii].width  = rctScreen.width + 1;
	pxClipRectangle[ ii].height = rctScreen.height + 1;

        XUnionRectWithRegion( &pxClipRectangle[ ii], xRegion, xRegion);
    }

    XSetRegion( _pXDisplay->GetXDisplay(), _xGC, xRegion);
    XDestroyRegion( xRegion);
}

/* get the clipping rectangle vector */

void
AmXGCClass::GetClipRectangles( AmRectanglePtr *pClipRectangle,
			     int          *pnClipRectangle)
{
    *pnClipRectangle = _nClipRectangle;
    *pClipRectangle = ( AmRectanglePtr) AmCopyVec( _pClipRectangle,
                                               _nClipRectangle *
                                               sizeof( AmRectangleRec));
}

/* set the raster operation */

void
AmXGCClass::SetRasterOp( AmRasterOpEnum eRasterOp)
{
    XGCValues xGCV;

    switch ( eRasterOp) {
    case AmCLEAR:
	xGCV.function = GXclear;
	break;
    case AmAND:
	xGCV.function = GXand;
	break;
    case AmAND_REVERSE:
	xGCV.function = GXandReverse;
	break;
    case AmCOPY:
	xGCV.function = GXcopy;
	break;
    case AmAND_INVERTED:
	xGCV.function = GXandInverted;
	break;
    case AmNOOP:
	xGCV.function = GXnoop;
	break;
    case AmXOR:
	xGCV.function = GXxor;
	break;
    case AmOR:
	xGCV.function = GXor;
	break;
    case AmNOR:
	xGCV.function = GXnor;
	break;
    case AmEQUIV:
	xGCV.function = GXequiv;
	break;
    case AmINVERT:
	xGCV.function = GXinvert;
	break;
    case AmOR_REVERSE:
	xGCV.function = GXorReverse;
	break;
    case AmCOPY_INVERTED:
	xGCV.function = GXcopyInverted;
	break;
    case AmOR_INVERTED:
	xGCV.function = GXorInverted;
	break;
    case AmNAND:
	xGCV.function = GXnand;
	break;
    case AmSET:
	xGCV.function = GXset;
	break;
    default:
   	xGCV.function = GXcopy;
	break;
    }
    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCFunction, &xGCV);
}


/* return the raster operation */

AmRasterOpEnum
AmXGCClass::GetRasterOp()
{
    XGCValues xGCV;

    XGetGCValues( _pXDisplay->GetXDisplay(), _xGC, GCFunction, &xGCV);
    switch ( xGCV.function) {
    case GXclear:
	return( AmCLEAR);
    case GXand:
	return( AmAND);
    case GXandReverse:
	return( AmAND_REVERSE);
    case GXcopy:
	return( AmCOPY);
    case GXandInverted:
	return( AmAND_INVERTED);
    case GXnoop:
	return( AmNOOP);
    case GXxor:
	return( AmXOR);
    case GXor:
	return( AmOR);
    case GXnor:
	return( AmNOR);
    case GXequiv:
	return( AmEQUIV);
    case GXinvert:
	return( AmINVERT);
    case GXorReverse:
	return( AmOR_REVERSE);
    case GXcopyInverted:
	return( AmCOPY_INVERTED);
    case GXorInverted:
	return( AmOR_INVERTED);
    case GXnand:
	return( AmNAND);
    case GXset:
	return( AmSET);
    default:
   	return( AmCOPY);
    }
}

/* set the dash pattern */

void
AmXGCClass::SetDashes( char *pcDashPattern,
		     int  nDashPattern,
		     int  iDashOffset)
{
    XGCValues xGCV;

    if ( _pcDashPattern != NULL) {
	FREE( _pcDashPattern);
    } 

    if ( nDashPattern != 0) {
	_pcDashPattern = ( char *) AmCopyVec( pcDashPattern, nDashPattern);
	_nDashPattern = nDashPattern;
	_iDashOffset = iDashOffset;
	XSetDashes( _pXDisplay->GetXDisplay(), _xGC, 
		    SCREEN_COORD( _iDashOffset),
                    _pcDashPattern, _nDashPattern);
	xGCV.line_style = LineOnOffDash;
    } else {
	_pcDashPattern = NULL;
	_nDashPattern = 0;
	_iDashOffset = 0;
	xGCV.line_style = LineSolid;
    }

    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCLineStyle, &xGCV);
}

/* Get the dash pattern. */

void
AmXGCClass::GetDashes( char **ppcDashPattern,
		     int  *pnDashPattern,
		     int  *piDashOffset)
{

    *ppcDashPattern = ( char *) AmCopyVec( _pcDashPattern, _nDashPattern);
    *pnDashPattern = _nDashPattern;
    if ( piDashOffset != NULL) {
	*piDashOffset = _iDashOffset;
    }
}

/* Set the line cap style from the enumeration */

void
AmXGCClass::SetLineCap( AmLineCapEnum eLineCap)
{
    XGCValues xGCV;

    switch ( eLineCap) {
    case AmCAP_NOT_LAST:
    	xGCV.cap_style = CapNotLast;
    	break;
    case AmCAP_BUTT:
    	xGCV.cap_style = CapButt;
    	break;
    case AmCAP_ROUND:
    	xGCV.cap_style = CapRound;
    	break;
    case AmCAP_PROJECTING:
    	xGCV.cap_style = CapProjecting;
    	break;
    default:
    	xGCV.cap_style = CapRound;
    	break;
    }

    XChangeGC( _pXDisplay->GetXDisplay(), _xGC, GCCapStyle, &xGCV);
}

/* return the current line cap */

AmLineCapEnum
AmXGCClass::GetLineCap()
{ 
    XGCValues xGCV;

    XGetGCValues( _pXDisplay->GetXDisplay(), _xGC, GCCapStyle, &xGCV);
    switch ( xGCV.cap_style) {
    case CapNotLast:
	return( AmCAP_NOT_LAST);
    case CapButt:
	return( AmCAP_BUTT);
    case CapRound:
	return( AmCAP_ROUND);
    case CapProjecting:
	return( AmCAP_PROJECTING);
    default:
	return( AmCAP_ROUND);
    }
}

void
AmXGCClass::Update()
{
    SetFont( _font.szFont, _font.iSize, _font.eWeight,
             _font.eSlant, _font.dxScale, _font.dyScale);
    SetLineWidth( _dLineWidth);
}

void
AmXGCClass::SetClipRegion( AmRegionClass	*region)
{
    XSetRegion( _pXDisplay->GetXDisplay(), _xGC, 
		(( AmXRegionClass *) region)->getXRegion());
    clipRegion = region;
}

AnvilBoolean
AmXGCClass::SetFillPattern( const char *fillPatFile)
{
    Pixmap	pixmap;
    uint_t	width, height;
    int		xhot, yhot;

/* free the last pattern file and stipple pixmap */

    if ( _fillPatFile != NULL) {
	FREE( _fillPatFile);
    }
    if ( _stipplePixmap != 0) {
	XFreePixmap( _pXDisplay->GetXDisplay(), _stipplePixmap);
	_stipplePixmap = 0;
    }
    _fillPatFile = strdup( fillPatFile);
    if ( XReadBitmapFile( _pXDisplay->GetXDisplay(), 
		          (( AmXWindowClass *) _pWindow)->GetXWindow(),
		          ( char *) fillPatFile, &width, &height, 
			  &pixmap, &xhot, &yhot) != BitmapSuccess) {
	return( FALSE);
    }
    _stipplePixmap = pixmap;
    XSetStipple( _pXDisplay->GetXDisplay(), _xGC,  pixmap);
    return( TRUE);
}

char *
AmXGCClass::GetFillPattern()
{
    return( _fillPatFile);
}

void
AmXGCClass::SetFillStyle( AmFillStyleEnum fillStyle)
{
    _fillStyle = fillStyle;
    switch ( fillStyle) {
    case AmFILLSTYLE_SOLID:
	XSetFillStyle( _pXDisplay->GetXDisplay(), _xGC, FillSolid);
	break;
    case AmFILLSTYLE_TILED:
	XSetFillStyle( _pXDisplay->GetXDisplay(), _xGC, FillTiled);
	break;
    case AmFILLSTYLE_STIPPLED:
	XSetFillStyle( _pXDisplay->GetXDisplay(), _xGC, FillStippled);
	break;
    case AmFILLSTYLE_OPAQUE_STIPPLED:
	XSetFillStyle( _pXDisplay->GetXDisplay(), _xGC, FillOpaqueStippled);
	break;
    default:
	break;
    }
}

AmFillStyleEnum
AmXGCClass::GetFillStyle()
{
    return( _fillStyle);
}
