#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */




/* X region functions */

#include <malloc.h>
#if !defined (__BSD44__)
#include <alloca.h>
#endif
#include <anvil/xdisplay.h>

/* X region class creation function */

AmXRegionClass::AmXRegionClass( AmDisplayClass *display) :
		AmRegionClass( display)
{
    xRegion = XCreateRegion();
}

AmXRegionClass::AmXRegionClass( Region 	region)
{
    xRegion = region;
}

/* X region class virtual destructor */

AmXRegionClass::~AmXRegionClass()
{
    XDestroyRegion( xRegion);
}

Region 
AmXRegionClass::getXRegion()
{
    return( xRegion);
}

AnvilBoolean
AmXRegionClass::PointInRegion( int	x,
			       int	y)
{
    return( XPointInRegion( xRegion, x, y));
}

AnvilBoolean
AmXRegionClass::EqualRegion( AmRegionClass	*otherRegion)
{
    return( XEqualRegion( xRegion, 
			  (( AmXRegionClass *) otherRegion)->getXRegion()));
}

void 
AmXRegionClass::AddPolygon( AmPointPtr		pts,
			    int			n,
			    AmFillRuleEnum	fillRule)
{
    Region	polyRegion, newRegion;
    XPoint	*xPoints;
    int		ii;

    if ( n == 0) {
	return;
    }
    xPoints = ( XPoint *) alloca( sizeof( XPoint)*n);
    if ( xPoints == NULL) {
	DO_ERROR( AmALLOCA_FAILED);
	return;
    }
    for ( ii = 0; ii < n; ii++) {
	xPoints[ ii].x = _pDisplay->ScreenCoord( pts[ ii].x);
	xPoints[ ii].y = _pDisplay->ScreenCoord( pts[ ii].y);
    }
    if ( fillRule == AmFILL_EVEN_ODD) {
	polyRegion = XPolygonRegion( xPoints, n, EvenOddRule);
    } else {
	polyRegion = XPolygonRegion( xPoints, n, WindingRule);
    }
    newRegion = XCreateRegion();
    XUnionRegion( xRegion, polyRegion, newRegion);

/* destroy the old region, and the temporary polygon region */

    XDestroyRegion( xRegion);
    XDestroyRegion( polyRegion);
    xRegion = newRegion;
}

void 
AmXRegionClass::AddRectangle( AmRectangleRec const	*rect)
{
    Region		newRegion;
    AmRectangleRec	screenRect;
    XRectangle		xRect;

    _pDisplay->ScreenRect( rect, &screenRect);
    xRect.x = screenRect.x;
    xRect.y = screenRect.y;
    xRect.width = screenRect.width;
    xRect.height = screenRect.height;
    newRegion = XCreateRegion();
    XUnionRectWithRegion( &xRect, xRegion, newRegion);
    XDestroyRegion( xRegion);
    xRegion = newRegion;
}
    
void 
AmXRegionClass::UnionRegion( AmRegionClass	*otherRegion)
{
    Region	newRegion;

    newRegion = XCreateRegion();
    XUnionRegion( xRegion, (( AmXRegionClass *) otherRegion)->getXRegion(), 
		  newRegion);
    XDestroyRegion( xRegion);
    xRegion = newRegion;
}
    
void 
AmXRegionClass::IntersectRegion( AmRegionClass	*otherRegion)
{
    Region	newRegion;

    newRegion = XCreateRegion();
    XIntersectRegion( xRegion, (( AmXRegionClass *) otherRegion)->getXRegion(), 
		      newRegion);
    XDestroyRegion( xRegion);
    xRegion = newRegion;
}
    
void 
AmXRegionClass::XorRegion( AmRegionClass	*otherRegion)
{
    Region	newRegion;

    newRegion = XCreateRegion();
    XXorRegion( xRegion, (( AmXRegionClass *) otherRegion)->getXRegion(), 
		newRegion);
    XDestroyRegion( xRegion);
    xRegion = newRegion;
}
    
void 
AmXRegionClass::SubtractRegion( AmRegionClass	*otherRegion)
{
    Region	newRegion;

    newRegion = XCreateRegion();
    XSubtractRegion( xRegion, (( AmXRegionClass *) otherRegion)->getXRegion(), 
		     newRegion);
    XDestroyRegion( xRegion);
    xRegion = newRegion;
}

AmRegionClass *
AmXRegionClass::CopyRegion()
{
    Region 	newRegion;

    newRegion = XCreateRegion();
    XUnionRegion( getXRegion(), getXRegion(), newRegion);
    return(( AmRegionClass *) new AmXRegionClass( newRegion));
}
