#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <X11/cursorfont.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/imageplot.h>
#include <anvil/imagelegend.h>
#include <anvil/lineplot.h>
#include <anvil/polygonplot.h>
#include <anvil/scatterplot.h>
#include <anvil/tick.h>
#include <anvil/plot_cback.h>
#include <anvil/motifdisplay.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/userevents.h>
#include <anvil/slick.h>
#include <anvil/plot_cback.h>
#include <anvil/events.h>
#include <anvil/outline.h>
#include <anvil/defaults.h>

#include <editor/EditorClass.h>

const int AmACTION_ADD_PRIORITY = 1000;

typedef struct addHandlerArgs_ {
    AmEditorClass     *pEditor;
    AmScreenObjClass  *pScreenObj;
    AmAddOutlineClass *pOutline;
    int             x;
    int             y;
    int             iWidth;
    int             iHeight;
    int             xOffset;
    int             yOffset;
    AnvilBoolean         fSuccess;
} AmAddHandlerArgsRec, *AmAddHandlerArgsPtr;

void
addHandler( Widget            widget,
            AmAddHandlerArgsPtr pHandlerArgs,
            XEvent            *pxEvent,
            AnvilBoolean           *);

/************************************************************
 *
 * Add Handler stuff:
 *
 ************************************************************/


AmAddHandlerArgsPtr
AmMakeAddHandlerArgs( AmEditorClass     *pEditor,
                    AmScreenObjClass  *pScreenObj,
                    int             x,
                    int             y,
                    int             iWidth,
                    int             iHeight,
                    int             xOffset,
                    int             yOffset,
                    AmAddOutlineClass *pOutline)
{
    AmAddHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = ( AmAddHandlerArgsPtr) malloc( sizeof( AmAddHandlerArgsRec));

    pHandlerArgs->pEditor = pEditor; /* AmEditorClass */
    pHandlerArgs->pScreenObj = pScreenObj;
    pHandlerArgs->x = x;
    pHandlerArgs->y = y;
    pHandlerArgs->iWidth = iWidth;
    pHandlerArgs->iHeight = iHeight;
    pHandlerArgs->xOffset = xOffset;
    pHandlerArgs->yOffset = yOffset;
    pHandlerArgs->pOutline = pOutline;
    pHandlerArgs->fSuccess = FALSE;

    return( pHandlerArgs);
}


void
AmRemoveAddHandler( AmAddHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay(); 

    pMotifDisplay->RemoveOutline( pHandlerArgs->pOutline);
    pMotifDisplay->UnmapTravelCursor();

    if ( pHandlerArgs->fSuccess == FALSE) {
        delete pHandlerArgs->pScreenObj;
    }

    free( pHandlerArgs);
}


void
AmSuspendAddHandler( AmAddHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay();

    pMotifDisplay->UnmapTravelCursor();
    pMotifDisplay->HideOutline( pHandlerArgs->pOutline);
}


void
AmRestartAddHandler( AmAddHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay();

    pMotifDisplay->WriteResourceMessage( "AmRestartAddHandler");
    pMotifDisplay->MapTravelCursor();
    pMotifDisplay->ShowOutline( pHandlerArgs->pOutline);
}


void
AmSetupAddHandler( AmEditorClass    *pEditor,
                   AmScreenObjClass *pScreenObj)
{
    AmMotifDisplayClass  *pMotifDisplay = pEditor->GetMotifDisplay();
    AmAddHandlerArgsPtr  pHandlerArgs;
    AmAddOutlineClass    *pOutline;
    const AmRectangleRec *pRect;

    pEditor->UnselectAll();

    pRect = pScreenObj->GetRect();

    pOutline = new AmAddOutlineClass ( pMotifDisplay,
                                     pRect->x, pRect->y,
                                     pRect->width, pRect->height);
    pMotifDisplay->AddOutline( pOutline);

    pHandlerArgs = AmMakeAddHandlerArgs( pEditor, pScreenObj,
                                       0, 0,
                                       pRect->width, pRect->height,
                                       pRect->width/2, pRect->height/2,
                                       pOutline);

    AmAddHandler( NULL, pMotifDisplay->GetArea(), 
                ButtonReleaseMask | ButtonPressMask | PointerMotionMask,
                ( XtEventHandler) addHandler,
                pHandlerArgs, AmACTION_ADD_PRIORITY,
                ( AmStateHandlerFn) AmRemoveAddHandler,
                ( AmStateHandlerFn) AmSuspendAddHandler,
                ( AmStateHandlerFn) AmRestartAddHandler);

    pMotifDisplay->MapTravelCursor();
    pMotifDisplay->WriteResourceTravelCursor( "addingMsg", 0, 0);
//     pMotifDisplay->SetCursor( pEditor->ActionToCursor( AmACTION_PICK));
}

/************************************************************
 *
 * Add Handler:
 *
 ************************************************************/

void
addHandler( Widget            widget,
            AmAddHandlerArgsPtr pHandlerArgs,
            XEvent            *pxEvent,
            AnvilBoolean           *)
{
    AmEditorClass          *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass    *pMotifDisplay = pEditor->GetMotifDisplay(); 
    AmScreenObjClass       *pScreenObj = pHandlerArgs->pScreenObj;
    AmAddOutlineClass      *pOutline = pHandlerArgs->pOutline;
    XButtonEvent         *pxButtonEvent;
    XMotionEvent         *pxMotionEvent;
    int                  x = pHandlerArgs->x,
                         y = pHandlerArgs->y,
                         w = pHandlerArgs->iWidth,
                         h = pHandlerArgs->iHeight,
                         xOffset = pHandlerArgs->xOffset,
                         yOffset = pHandlerArgs->yOffset,
                         iWidthArea = pMotifDisplay->GetWidth(),
                         iHeightArea = pMotifDisplay->GetHeight();

    if ( pxEvent->type == ButtonPress) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            pMotifDisplay->WriteResourceMessage( "addCancelMsg");

        /* Remove the handler. */

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) addHandler,
                           pHandlerArgs);

        } else if ( pxButtonEvent->button == Button1) {

        /* Display a busy cursor. */

            pMotifDisplay->WriteResourceMessage( "addSuccessMsg");
            pMotifDisplay->SetCursor( XC_watch);

        /* Add the object before setting the rectangle and margins, 
	   otherwise the event list will ignore the object because it's
	   not in the motifdisplay's object list. */

            pMotifDisplay->AddObject( pScreenObj);
            pScreenObj->SetRectAndMargins( x, y, w, h, iWidthArea, iHeightArea);
            pEditor->SelectObject( pScreenObj);
	    pMotifDisplay->Flush();
 
            pEditor->SetMapChanged( TRUE);

        /* Give a status message. */

            pHandlerArgs->fSuccess = TRUE;

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) addHandler,
                           pHandlerArgs);
        }
    } else if ( pxEvent->type == MotionNotify) {
        pxMotionEvent = ( XMotionEvent *) pxEvent;

    /* Figure out the extends of the rectangle. */

        x = pxMotionEvent->x;
        y = pxMotionEvent->y;

        x = pMotifDisplay->GetDisplay()->VirtualCoord( x);
        y = pMotifDisplay->GetDisplay()->VirtualCoord( y);

        x = x - xOffset;
        y = y - yOffset;

        if ( pEditor->GetSnapToGrid() == TRUE) {
            x = pMotifDisplay->GetGridLocation( x);
            y = pMotifDisplay->GetGridLocation( y);
        }

        if ( x < 0) {
            x = 0;
        } else if ( x + w > iWidthArea) {
            x = iWidthArea - w;
        }

        if ( y < 0) {
            y = 0;
        } else if ( y + h > iHeightArea) {
            y = iHeightArea - h;
        }

    /* Remove the old outline and draw the new one. */

        pOutline->SetOrigin( x, y);

        pHandlerArgs->x = x;
        pHandlerArgs->y = y;

        pMotifDisplay->WriteResourceTravelCursor( "addingMsg", x, y);
    }
}
