#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <X11/cursorfont.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/imageplot.h>
#include <anvil/imagelegend.h>
#include <anvil/lineplot.h>
#include <anvil/polygonplot.h>
#include <anvil/scatterplot.h>
#include <anvil/tick.h>
#include <anvil/plot_cback.h>
#include <anvil/motifdisplay.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/userevents.h>
#include <anvil/slick.h>
#include <anvil/plot_cback.h>
#include <anvil/events.h>
#include <anvil/outline.h>
#include <anvil/defaults.h>

#include <editor/EditorClass.h>

const int AmACTION_ADD_PRIORITY = 1000;

typedef struct addObjectsHandlerArgs_ {
    AmEditorClass     *pEditor;
    AmListPtr         lstObjects;
    int             xStart;
    int             yStart;
    int             x;
    int             y;
    int             iWidth;
    int             iHeight;
    int             xOffset;
    int             yOffset;
    AnvilBoolean         fSuccess;
    AmAddOutlineClass *pOutline;
} AmAddObjectsHandlerArgsRec, *AmAddObjectsHandlerArgsPtr;

void
AmAddObjectsHandler( Widget                   widget,
                   AmAddObjectsHandlerArgsPtr pHandlerArgs,
                   XEvent                   *pxEvent,
                   AnvilBoolean                  *);

void
AmGetObjectsExtents( AmListPtr      lstObjects,
                   AmRectangleRec *pRec)
{
    AmScreenObjClass     *pScreenObj;
    AmListPtr            lstTemp;
    int                x1, y1, x2, y2;
    AnvilBoolean            fFirst; 
    const AmRectangleRec *pRecTemp;

    fFirst = TRUE;
    x1 = y1 = x2 = y2 = 0;

    for ( lstTemp = lstObjects;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pScreenObj = ( AmScreenObjClass *) lstTemp->data;
        pRecTemp = pScreenObj->GetRect();

        if ( fFirst || ( pRecTemp->x < x1)) {
            x1 = pRecTemp->x;
        }

        if ( fFirst || ( pRecTemp->y < y1)) {
            y1 = pRecTemp->y;
        }

        if ( fFirst || (( pRecTemp->x + pRecTemp->width) > x2)) {
            x2 = pRecTemp->x + pRecTemp->width;
        }

        if ( fFirst || (( pRecTemp->y + pRecTemp->height) > y2)) {
            y2 = pRecTemp->y + pRecTemp->height;
        }

        fFirst = FALSE;
    }

    pRec->x = x1;
    pRec->y = y1;
    pRec->width = x2 - x1;
    pRec->height = y2 - y1;
}

void
AmAddObjects( AmAddObjectsHandlerArgsPtr pHandlerArgs,
            int                      iWidthArea,
            int                      iHeightArea)
{
    AmEditorClass        *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass  *pMotifDisplay = pEditor->GetMotifDisplay(); 
    AmScreenObjClass     *pScreenObj;
    const AmRectangleRec *pRec;
    AmListPtr            lstTemp;
    int                x, y, xStart, yStart;

    x = pHandlerArgs->x;
    y = pHandlerArgs->y;

    xStart = pHandlerArgs->xStart;
    yStart = pHandlerArgs->yStart;

    for ( lstTemp = pHandlerArgs->lstObjects;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pScreenObj = ( AmScreenObjClass *) lstTemp->data;
        pRec = pScreenObj->GetRect();

        pScreenObj->SetRectAndMargins( pRec->x + x - xStart,
                                       pRec->y + y - yStart,
                                       pRec->width, pRec->height,
                                       iWidthArea, iHeightArea);
        pMotifDisplay->AddObject( pScreenObj);
        pEditor->SelectObject( pScreenObj);
    }

    pMotifDisplay->Flush();
}

/************************************************************
 *
 * Add Objects Handler stuff:
 *
 ************************************************************/


AmAddObjectsHandlerArgsPtr
AmMakeAddObjectsHandlerArgs( AmEditorClass     *pEditor,
                           AmListPtr         lstObjects,
                           int             x,
                           int             y,
                           int             xStart,
                           int             yStart,
                           int             iWidth,
                           int             iHeight,
                           int             xOffset,
                           int             yOffset,
                           AmAddOutlineClass *pOutline)
{
    AmAddObjectsHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = ( AmAddObjectsHandlerArgsPtr)malloc(
                       sizeof( AmAddObjectsHandlerArgsRec));

    pHandlerArgs->pEditor = pEditor;
    pHandlerArgs->lstObjects = lstObjects;
    pHandlerArgs->x = x;
    pHandlerArgs->y = y;
    pHandlerArgs->xStart = xStart;
    pHandlerArgs->yStart = yStart;
    pHandlerArgs->iWidth = iWidth;
    pHandlerArgs->iHeight = iHeight;
    pHandlerArgs->xOffset = xOffset;
    pHandlerArgs->yOffset = yOffset;
    pHandlerArgs->fSuccess = FALSE;
    pHandlerArgs->pOutline = pOutline;

    return( pHandlerArgs);
}


void
AmRemoveAddObjectsHandler( AmAddObjectsHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay();
    AmOutlineClass      *pOutline = pHandlerArgs->pOutline;
    AmScreenObjClass    *pScreenObj;
    AmListPtr           lstTemp;

    if ( pOutline != NULL) {
        pMotifDisplay->RemoveOutline( pOutline);
    }

    pMotifDisplay->UnmapTravelCursor();

    if ( pHandlerArgs->fSuccess == FALSE) {
 
        for ( lstTemp = pHandlerArgs->lstObjects; 
              lstTemp != NULL;
              lstTemp = lstTemp->next) {

            pScreenObj = ( AmScreenObjClass *) lstTemp->data; 
            delete pScreenObj; 
        }
    }

    AmJustFreeLst( pHandlerArgs->lstObjects);

    free( pHandlerArgs);
}


void
AmSuspendAddObjectsHandler( AmAddObjectsHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay();
    AmOutlineClass      *pOutline = pHandlerArgs->pOutline;

    if ( pOutline != NULL) {
        pMotifDisplay->HideOutline( pOutline);
    }

    pMotifDisplay->UnmapTravelCursor();
}


void
AmRestartAddObjectsHandler( AmAddObjectsHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay();
    AmOutlineClass      *pOutline = pHandlerArgs->pOutline;

    pMotifDisplay->WriteResourceMessage( "restartObjectsAddHandler");

    if ( pOutline != NULL) {
        pMotifDisplay->ShowOutline( pOutline);
    }

    pMotifDisplay->MapTravelCursor();
}

void
AmSetupAddObjectsHandler( AmEditorClass    *pEditor,
                        AmListPtr        lstObjects) 
{
    AmMotifDisplayClass        *pMotifDisplay = pEditor->GetMotifDisplay();
    AmAddObjectsHandlerArgsPtr pHandlerArgs;
    AmAddOutlineClass          *pOutline;
    AmRectangleRec             rec;

    pEditor->UnselectAll();

    AmGetObjectsExtents( lstObjects, &rec);

    pOutline = new AmAddOutlineClass( pMotifDisplay,
                                    rec.x, rec.y,
                                    rec.width, rec.height);
    pMotifDisplay->AddOutline( pOutline);

    pHandlerArgs = AmMakeAddObjectsHandlerArgs( pEditor,
                                              lstObjects,
                                              0, 0,
                                              rec.x, rec.y,
                                              rec.width, rec.height,
                                              rec.width / 2, rec.height / 2, 
                                              pOutline);

    AmAddHandler( NULL, pMotifDisplay->GetArea(),
                ButtonReleaseMask | ButtonPressMask | PointerMotionMask,
                ( XtEventHandler) AmAddObjectsHandler,
                pHandlerArgs, AmACTION_ADD_PRIORITY,
                ( AmStateHandlerFn) AmRemoveAddObjectsHandler,
                ( AmStateHandlerFn) AmSuspendAddObjectsHandler,
                ( AmStateHandlerFn) AmRestartAddObjectsHandler);

    pMotifDisplay->MapTravelCursor();
    pMotifDisplay->WriteResourceTravelCursor( "addingMsg", 0, 0); 
}

/************************************************************
 *
 * Add Objects Handler:
 *
 ************************************************************/

void
AmAddObjectsHandler( Widget                   widget,
                   AmAddObjectsHandlerArgsPtr pHandlerArgs,
                   XEvent                   *pxEvent,
                   AnvilBoolean                  *)
{
    AmEditorClass       *pEditor = pHandlerArgs->pEditor;
    AmMotifDisplayClass *pMotifDisplay = pEditor->GetMotifDisplay();
    XButtonEvent      *pxButtonEvent;
    XMotionEvent      *pxMotionEvent;
    int               x, y, iWidth, iHeight,
                      iWidthArea = pMotifDisplay->GetWidth(),
                      iHeightArea = pMotifDisplay->GetHeight();

    if ( pxEvent->type == ButtonPress) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            pMotifDisplay->WriteResourceMessage( "addCancelMsg");

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmAddObjectsHandler, pHandlerArgs);

        } else if ( pxButtonEvent->button == Button1) {

        /* Display a busy cursor and remove the outline and the handler. */

            pMotifDisplay->SetCursor( XC_watch);
            pMotifDisplay->WriteResourceMessage( "addSuccessMsg");

            AmAddObjects( pHandlerArgs, iWidthArea, iHeightArea);
            pHandlerArgs->fSuccess = TRUE;
            pEditor->SetMapChanged( TRUE);

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmAddObjectsHandler, pHandlerArgs);
        }
    } else if ( pxEvent->type == MotionNotify) {
        pxMotionEvent = ( XMotionEvent *) pxEvent;

        x = pxMotionEvent->x;
        y = pxMotionEvent->y;

        x = pMotifDisplay->GetDisplay()->VirtualCoord( x);
        y = pMotifDisplay->GetDisplay()->VirtualCoord( y);

        x = x - pHandlerArgs->xOffset;
        y = y - pHandlerArgs->yOffset;

        iWidth = pHandlerArgs->iWidth;
        iHeight = pHandlerArgs->iHeight;

    /* Figure out the extends of the rectangle. */

        if ( pEditor->GetSnapToGrid() == TRUE) {

            x = pMotifDisplay->GetGridLocation( x);
            y = pMotifDisplay->GetGridLocation( y);
        }

        if ( x + iWidth > iWidthArea) {
            x = iWidthArea - iWidth;
        }

        if ( x < 0) {
            x = 0;
        }

        if ( y + iHeight > iHeightArea) {
            y = iHeightArea - iHeight;
        }

        if ( y < 0) {
            y = 0;
        }

        pHandlerArgs->pOutline->SetOrigin( x, y);

        pHandlerArgs->x = x;
        pHandlerArgs->y = y;

        pMotifDisplay->WriteResourceTravelCursor( "addingMsg", x, y);
    }
}
