#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/DrawingA.h>

#include <anvil/macros.h>
#include <anvil/strlib.h>
#include <anvil/errlib.h>
#include <anvil/events.h>
#include <anvil/va_cback.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/editobject.h>
#include <anvil/motif.h>
#include <anvil/log.h>

#include <editor/DeleteLayerDialogClass.h>

/*
 * Function: AmDeleteLayerDialogClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmDeleteLayerDialogClass::initializeValues()
{
    _eObjectType   = AmANY_OBJECT;
    _wArea         = _pMotifDisplay->GetArea();
}


/*
 * Function: AmDeleteLayerDialogClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets for the layer selection dialog.
 */

void
AmDeleteLayerDialogClass::createInterface()
{
    _pwcLayerList->SetExtendedSelect();
}


/*
 * Function: AmDeleteLayerDialogClass::AmDeleteLayerDialogClass()
 *
 * Purpose:
 *   Default constructor.
 */

AmDeleteLayerDialogClass::AmDeleteLayerDialogClass()
    : AmPickLayerDialogClass()
{
}


/*
 * Functions: AmDeleteLayerDialogClass::AmDeleteLayerDialogClass( wParent, szName)
 *            AmDeleteLayerDialogClass::AmDeleteLayerDialogClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmDeleteLayerDialogClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters: 
 *   Widget          wParent    : The parent of the dialog.
 *   BaseWidgetclass *bwcParent : The "wrapped" parent of the dialog.
 *   const char      *szName    : The name of the dialog.
 */

AmDeleteLayerDialogClass::AmDeleteLayerDialogClass(
                            Widget            wParent,
                            const char        *szName,
                            AmOverlayClass      *pOverlay,
                            AmMotifDisplayClass *pMotifDisplay)
    : AmPickLayerDialogClass( wParent, szName, pOverlay)
{
    _pMotifDisplay = pMotifDisplay;

    initializeValues();
    createInterface();
}


/* See above comment. */

AmDeleteLayerDialogClass::AmDeleteLayerDialogClass(
                            AmBaseWidgetClass *bwcParent,
                            const char        *szName,
                            AmOverlayClass      *pOverlay,
                            AmMotifDisplayClass *pMotifDisplay)
    : AmPickLayerDialogClass( bwcParent, szName, pOverlay)
{
    _pMotifDisplay = pMotifDisplay;
 
    initializeValues();
    createInterface();
}


/*
 * Function: AmDeleteLayerDialogClass::~AmDeleteLayerDialogClass()
 *
 * Purpose:
 *   Destructor for the AmDeleteLayerDialogClass.
 */

AmDeleteLayerDialogClass::~AmDeleteLayerDialogClass()
{
}


/* ARGSUSED */
void
AmDeleteLayerDialogClass::action( Widget    w,
                                XtPointer xpCallData)
{
    int            *viPos, nPos, ii, iPos;
    AnvilBoolean        fDeleteOverlay;

    AmRemoveCallback( AM_PKG_ID, AM_DELETE_OBJECT, 
                    ( callbackProc) AmObjectDeletedCallback,
                    ( XtPointer) this);

    AmRemoveCallback( AM_PKG_ID, AM_DELETE_LAYER, 
                    ( callbackProc) AmLayerDeletedCallback,
                    ( XtPointer) this);

    Hide();

/* Get the selected layers from the list. */

    _pwcLayerList->GetSelectedPositions( &viPos, &nPos);

    if ( nPos > 0) {

        fDeleteOverlay = FALSE;
        for ( ii = 0; ii < nPos; ii++) {
            iPos = viPos[ ii] - 1;
            if ( iPos == _nLayerPos) {
                fDeleteOverlay = TRUE;  
            }
        }

        if ( nPos == _nDataGraphic) {
            fDeleteOverlay = TRUE;  
        }

        if ( fDeleteOverlay == FALSE) {
            for ( ii = 0; ii < nPos; ii++) {
                iPos = viPos[ ii] - 1;
                _pOverlay->RemoveLayer( _vpDataGraphic[ _viLayerPos[ iPos]]);
                delete _vpDataGraphic[ _viLayerPos[ iPos]];
            }
        } else {
            _pMotifDisplay->RemoveObject( _pOverlay);
        }

        _pMotifDisplay->Flush();
    }

    XtFree(( char *) viPos);

    delete this;
}
