#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <signal.h>
#include <anvil/xapp.h>
#include <anvil/envdefs.h>
#include <anvil/errlib.h>
#include <anvil/motif.h>
#include <anvil/user_data.h>

#include <editor/EditorApplicationClass.h>

#ifdef PSCOPE
#include <sys/time.h>
extern "C" {
#include <linux/pscope.h>
}
#endif

#ifdef LISTEN_SIGNAL
int listening_signal = 0, map_width = 0, map_height = 0;
char *signal_file = NULL;
AmEditorApplicationClass* signal_edobj = 0;
#endif

#include "odyapi.h"

#ifdef PSCOPE
char* pscope_ifile = 0;
int pscope_ilisten = 0;
int pscope_inotify = 0;
int pscope_notify_pid = 0;
int pscope_ih = 0, pscope_iw = 0;
int pscope_loading = 0;
int pscope_cnt = 0;
AmEditorApplicationClass* pscope_edobj = 0;

#define FILE_NUM 6
#define PSCOPE_REP_NUM 5
#define MAP_DIR "/ody/map/"
char ifiles[FILE_NUM][255] = 
{
  MAP_DIR "boston.vec.hdr",
  MAP_DIR "allentown.vec.hdr",
  MAP_DIR "washington.vec.hdr",
  MAP_DIR "chicago.vec.hdr",
  MAP_DIR "sanjose.vec.hdr",
  MAP_DIR "pittsburgh.vec.hdr"
};

char cfiles[FILE_NUM][255] = 
{
  MAP_DIR "boston.crop.hdr",
  MAP_DIR "allentown.crop.hdr",
  MAP_DIR "washington.crop.hdr",
  MAP_DIR "chicago.crop.hdr",
  MAP_DIR "sanjose.crop.hdr",
  MAP_DIR "pittsburgh.crop.hdr"
};

#ifdef PSCOPE_DEBUG
struct timeval tv1, tv2, tv3;
#endif

int pscope_reps = 0;
int pscope_crop = 0;
int pscope_trace = 0;
int pscope_standby = 0;
int pscope_cuttimeout = 5000; /* Default time to display map */
int pscope_ttime = 0; /* Measured user "think-time" */
XtAppContext pscope_ac;  
#endif

AmEditorApplicationClass::AmEditorApplicationClass()
    : AmApplicationClass()
{
    _pwcEditor = NULL;
}

AmEditorApplicationClass::AmEditorApplicationClass( const char *szClass,
                                                int        nArgs,
                                                char       **pszArgs)
    : AmApplicationClass( szClass, nArgs, pszArgs)
{
    if ( AmInitApplication( _wTop) != AmOK) {
	fprintf( stderr, "failed to initialize\n");
	exit( -1);
    }

    if ( getenv( CONFIG_ENV) == NULL) {
        AmPrintError( "Missing environment variable: \"%s\".", CONFIG_ENV);
    } else {
        _pwcEditor = new AmEditorClass( _wTop, szClass);
    }

    {
      int i;
      for (i = 0; i < nArgs; i++) {
#ifdef LISTEN_SIGNAL
	if (pszArgs[i] && !strcmp(pszArgs[i], "-listen_signal")) {
	  if (i + 3 >= nArgs) {
	    printf ("ERROR: must specify file name, width, and height\n");
	    exit (1);
	  }
	  listening_signal = 1;
	  signal_file = pszArgs[i+1];
	  map_width = atoi (pszArgs[i+2]);
	  map_height = atoi (pszArgs[i+3]);
	  fprintf(stderr,"listen_signal: %s, %d, %d\n", signal_file,
		  map_width, map_height);
	} 
#endif
#ifdef PSCOPE
	if (pszArgs[i] && !strcmp(pszArgs[i], "-ifile")) {
	  if (i + 3 >= nArgs) {
	    printf ("ERROR: must specify file name, height, and width\n");
	    exit (1);
	  }
	  pscope_ifile = pszArgs[i+1];
	  pscope_iw = atoi (pszArgs[i+2]);
	  pscope_ih = atoi (pszArgs[i+3]);
	} else if (pszArgs[i] && !strcmp(pszArgs[i], "-ilisten")) {
	  if (i + 2 >= nArgs) {
	    printf ("ERROR: must specify width and height\n");
	    exit (1);
	  }
	  pscope_ilisten = 1;
	  pscope_iw = atoi (pszArgs[i+1]);
	  pscope_ih = atoi (pszArgs[i+2]);
	} else if (pszArgs[i] && !strcmp(pszArgs[i], "-ttime")) {
	  if (i + 1 >= nArgs) {
	    printf ("ERROR: must specify think time (in ms)\n");
	    exit (1);
	  }
	  pscope_ttime = atoi (pszArgs[i+1]);
	  if (pscope_ttime > 0) {
	    pscope_cuttimeout = pscope_ttime;
	  }
	} else if (pszArgs[i] && !strcmp(pszArgs[i], "-inotify")) {
	    pscope_inotify = 1;
	} else if (pszArgs[i] && !strcmp(pszArgs[i], "-trace")) {
	  pscope_trace = 1;
	} else if (pszArgs[i] && !strcmp(pszArgs[i], "-crop")) {
	  pscope_crop = 1;
	} else if (pszArgs[i] && !strcmp(pszArgs[i], "-standby")) {
	  pscope_standby = 1;
	}
#endif
      }
    }
}

AmEditorApplicationClass::~AmEditorApplicationClass()
{
    AN_DEBUG( 99, ( "AmEditorApplicationClass::~AmEditorApplicationClass()\n"));
}

#ifdef LISTEN_SIGNAL
static void
catch_sigload(int sig)
{
  fprintf(stderr,"listen_signal: sigload\n");
  signal_edobj->UnloadInstr();
  signal_edobj->LoadInstr(signal_file);
}
void 
AmEditorApplicationClass::LoadInstr (char* file)
{
  _pwcEditor->loadOk (NULL, NULL, file);
  fprintf(stderr,"%s loaded\n", signal_file);
}

void 
AmEditorApplicationClass::UnloadInstr ()
{
  if (_pwcEditor->_lstSelectedObjects) {
    _pwcEditor->cut (NULL, NULL);
    fprintf(stderr,"Old map removed, if any\n");
  }
}

#endif

#ifdef PSCOPE 
void timeout2 (XtPointer p, XtIntervalId* id);

static void
catch_sigload(int sig)
{
  if (pscope_cnt == FILE_NUM) {
    pscope_cnt = 0;
  }
  if (pscope_crop) {
    pscope_edobj->LoadInstr(cfiles[pscope_cnt++]);
  } else {
    pscope_edobj->LoadInstr(ifiles[pscope_cnt++]);
  }
}

static void
catch_sigalarm(int sig)
{
  timeout2 (pscope_edobj, 0);
}

void 
AmEditorApplicationClass::LoadInstr (char* file)
{
  pscope_loading = 1;
  if (pscope_ifile) {
    if (!pscope_trace) {
      pscope_start (PSCOPE_INTR_LPT, 1);
    }
  }
  _pwcEditor->loadOk (NULL, NULL, file);
}

void 
AmEditorApplicationClass::UnloadInstr ()
{
  _pwcEditor->cut (NULL, NULL);
}

void timeout (XtPointer p, XtIntervalId* id)
{
  AmEditorApplicationClass* papp = (AmEditorApplicationClass *) p;

  if (pscope_standby) {
    system ("/usr/odyssey/bin/disk standby");
    sleep (3);
  } 
  
  if (pscope_cnt == FILE_NUM) {
    pscope_cnt = 0;
  }
  if (pscope_trace && pscope_cnt == 0) {
    pscope_start (PSCOPE_INTR_LPT, 1);
  }
  if (pscope_crop) {
    papp->LoadInstr(cfiles[pscope_cnt++]);
  } else {
    papp->LoadInstr(ifiles[pscope_cnt++]);
  }
}

void timeout2 (XtPointer p, XtIntervalId* id)
{
  AmEditorApplicationClass* papp = (AmEditorApplicationClass *) p;

  /* If we are measuring user think-time, stop profile here since
     think time has expired. */
  if (pscope_ttime > 0) {
#ifdef PSCOPE_DEBUG
    gettimeofday (&tv2, 0);
    timersub (&tv2, &tv1, &tv3);
#endif
    pscope_stop ();
#ifdef PSCOPE_DEBUG
    printf ("Measured think time: %ld.%ld seconds\n", tv3.tv_sec, tv3.tv_usec);
#endif
    fflush (stdout);
  }

  papp->UnloadInstr();
  if (pscope_cnt == FILE_NUM) {
    pscope_cnt = 0;
    pscope_reps++;
    if (pscope_reps == PSCOPE_REP_NUM && !pscope_ilisten) {
      printf ("All done\n");
      exit (0);
    }
  }

  if (pscope_inotify) {
      if (pscope_notify_pid <= 0) {
	  FILE* file = fopen ("/tmp/notify", "r");
	  if (file) {
	      fscanf (file, "%d\n", &pscope_notify_pid);
	      fclose (file);
	  } else {
	      printf ("Unable to read notify file\n");
	      exit (0);
	  }
      }
      kill (pscope_notify_pid, SIGUSR1);
  }
	  
  if (pscope_ifile) {
    /* Wait 5 s. then load next map */
    XtAppAddTimeOut (pscope_ac, 5000, timeout, papp);
  }
}
#endif

void
AmEditorApplicationClass::Run()
{
    XEvent xEvent;

    g_fRun = TRUE;

#ifndef NO_ADAPT
    /* Initialize applications */
    init_adaptation ();
#endif

#ifdef PSCOPE
    if (pscope_ttime) {
      /* Set up signal handler for SIGLARM */
      struct sigaction sa;

      pscope_edobj = this;

      /* Set signal handler */
      sa.sa_handler = catch_sigalarm;
      if (sigemptyset(&sa.sa_mask)) {
	perror ("sigemptyset");
	exit (1);
      }

      /* I think we always want to restart on this signal... */
      sa.sa_flags = 0;
      sa.sa_flags |= SA_RESTART;
      if (sigaction(SIGALRM, &sa, NULL) == -1) {
	perror ("sigaction");
	exit (1);
      }
      
    }

    if (pscope_ilisten) {
      struct sigaction sa;

      pscope_edobj = this;

      /* Set signal handler */
      sa.sa_handler = catch_sigload;
      if (sigemptyset(&sa.sa_mask)) {
	perror ("sigemptyset");
	exit (1);
      }

      /* I think we always want to restart on this signal... */
      sa.sa_flags = 0;
      sa.sa_flags |= SA_RESTART;
      if (sigaction(SIGUSR1, &sa, NULL) == -1) {
	perror ("sigaction");
	exit (1);
      }
      
    } else if (pscope_ifile) {
      /* Load 1st map after 5 s. */
      XtAppAddTimeOut (_ac, 5000, timeout, this);
    }

    pscope_ac = _ac;
#endif

#ifdef LISTEN_SIGNAL
    if (listening_signal) {
      struct sigaction sa;

      signal_edobj = this;

      /* Set signal handler */
      sa.sa_handler = catch_sigload;
      if (sigemptyset(&sa.sa_mask)) {
	perror ("sigemptyset");
	exit (1);
      }

      /* I think we always want to restart on this signal... */
      sa.sa_flags = 0;
      sa.sa_flags |= SA_RESTART;
      if (sigaction(SIGUSR1, &sa, NULL) == -1) {
	perror ("sigaction");
	exit (1);
      }

      fprintf(stderr,"listen_signal: set signal handler\n");
      
    }
#endif

    do {
        XtAppNextEvent( _ac, &xEvent);
#ifdef PSCOPE	
	if (pscope_loading) {
	  if (xEvent.xany.type == NoExpose) {
	    if (!pscope_trace || pscope_cnt == FILE_NUM) {
	      /* If we're incorporating think-time stop profile later */
	      if (pscope_ttime <= 0) {
		pscope_stop ();
	      } else {
#ifdef PSCOPE_DEBUG
		gettimeofday (&tv1, 0);
#endif
	      }
	    }
	    pscope_loading = 0;

	    /* Unload after 5 s. */
	    if (pscope_ttime) {
	      printf ("Setting itimer\n");
	      struct itimerval it;
	      it.it_value.tv_sec = pscope_ttime / 1000;
	      it.it_value.tv_usec = pscope_ttime % 1000;
	      it.it_interval.tv_sec = 0;
	      it.it_interval.tv_usec = 0;
	      setitimer (ITIMER_REAL, &it, 0);
	    } else {
	      XtAppAddTimeOut (_ac, pscope_cuttimeout, timeout2, this);
	    }
	  }
	}
#endif
        AmHelpHandler( XtDisplay( _wTop), _ac, &xEvent);
        XtDispatchEvent( &xEvent);
    } while ( g_fRun == TRUE);
}
