#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/cursorfont.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/ScrolledW.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/log.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/files.h>
#include <anvil/template.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/postscript.h>
#include <anvil/plot_cback.h>
#include <anvil/motifdisplay.h>
#include <anvil/textannot.h>
#include <anvil/imagelegend.h>
#include <anvil/linelegend.h>
#include <anvil/polylegend.h>
#include <anvil/scatterlegend.h>
#include <anvil/resource.h>
#include <anvil/state_hand.h>
#include <anvil/user_data.h>
#include <anvil/userevents.h>
#include <anvil/envdefs.h>
#include <anvil/slick.h>
#include <anvil/editobject.h>
#include <anvil/outline.h>
#include <anvil/display.h>
#include <anvil/xapp.h>
#include <anvil/fonts.h>
#include <anvil/xpm.h>
#include <anvil/defaults.h>
#include <anvil/visual.h>
#include <anvil/xcolor.h>
#include <anvil/editobject.h>
#include <anvil/imageannot.h>
#include <anvil/rectangleannot.h>
#include <anvil/grid.h>
#include <anvil/distscale.h>
#include <anvil/dlog_utility.h>

#include <editor/EditorClass.h>
#include <editor/EditorApplicationClass.h>
#include <editor/handlers.h>

const int AmACTION_MASTER_PRIORITY = 6000;
const int AM_SELECTION_SET_CHANGED = 8001;


//////////////////////////////////////////////////////////////////////
//
// Function: initializeValues
//
// Description:
//   Initializes the member variables to their default values. 
//
//   By default the map is an A sized page 8.5" by 11.0" in portrait
//   mode, it is neither zoomed in or zoomed out which means a resolution
//   of 72 DPI on the the screen.  The print resolution is also set to 72
//   DPI (currently that is a fixed value).
//
//   Finally a temporary clip file is created which is used for cuting,
//   copying, and pasting objects.
//   
//

void
AmEditorClass::initializeValues()
{
    _pMotifDisplay = NULL;
    _pOutline = NULL;

// There is no file that is being edited at startup.

    _fMapChanged = FALSE;
    _szCurrentFile = NULL;
    _szTitle = NULL;

// There are no objects that are selected.

    _lstSelectedObjects = NULL;

// There is no grid at startup.

    _fGrid = FALSE;
    _dGridSpacing = 1; 
    _fSnapToGrid = FALSE;

// The page starts as A size. 

    _dWidthInInches = 8.5;
    _dHeightInInches = 11;

// The DPI for printing and on the screen are the same.

    _iScreenDPI = 72;
    _iPrintDPI = 72;

// The display is in portrait mode and uses true color capabilities.

    _fLandscape = FALSE;
    _fTrueColor = TRUE;

    _szPrologFile = strdup( AmGetDefault( "print.prologFile", "HP750C_prolog"));

    _fClipFileValid = FALSE;
    _szClipFile = strdup( "/tmp/AnClipboardOutput.XXXXXX");
    mktemp( _szClipFile);

    _fStandalone = TRUE;
}

void
AmEditorClass::createInterface()
{
    AmToggleButtonClass *pwcToggle;
    char              *sz;
    int               iScreenWidth, iScreenHeight;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::createInterface()\n"));

// Create the editor's form.

    _wForm = XtVaCreateManagedWidget( "editorForm",
                                      xmFormWidgetClass,
                                      _wParent,
                                      NULL); 
    _wBase = _wForm;
    addDestroyCallback();

// Create the menu bar.

    _pwcMenu = new AmMenuClass( _wForm, "menuBar");
    XtVaSetValues( *_pwcMenu, StickLeft, StickTop, StickRight, NULL);

// Message area.

    _pwcStatusForm = new AmFormClass( _wForm, "statusForm");
    _pwcStatusForm->AttachLeft();
    _pwcStatusForm->AttachRight();
    _pwcStatusForm->AttachBottom();

// There is no progress bar currently.

//    _pwcProgressBar = new AmProgressBarClass( _pwcStatusForm, "progressBar");
//    _pwcProgressBar->AttachTop();
//    _pwcProgressBar->AttachRight();
//    _pwcProgressBar->AttachBottom();

//    _pwcProgressBar->SetCurrValue( 75);
//    _pwcProgressBar->ShowProgress( FALSE);

    _pwcMessageFrame = new AmFrameClass( _pwcStatusForm, "messageFrame", FALSE); 
    _pwcMessageFrame->AttachLeft();
    _pwcMessageFrame->AttachTop();
//    _pwcMessageFrame->AttachRight( _pwcProgressBar);
    _pwcMessageFrame->AttachRight();
    _pwcMessageFrame->AttachBottom();
 
    _pwcTextMessage = new AmLabelClass( _pwcMessageFrame, "messageText"); 

// Create the tool bar.

    _pwcToolBar = new AmEditorToolBarClass( _wForm, "toolBar");
    XtVaSetValues( *_pwcToolBar,
                   StickLeft, BelowOf( *_pwcMenu), StickRight,
                   NULL);

// Create the zoom related utility bar.

    _pwcZoomUtilityBar = new AmEditorToolBarClass( _wForm, "zoomUtilityBar");
    _pwcZoomUtilityBar->AttachTop( _pwcToolBar);
    _pwcZoomUtilityBar->AttachRight();

// Create file related utility bar.

    _pwcFileUtilityBar = new AmEditorToolBarClass( _wForm, "fileUtilityBar");
    _pwcFileUtilityBar->AttachTop( _pwcZoomUtilityBar);
    _pwcFileUtilityBar->AttachRight();

// Create the legend related utility bar.

    _pwcLegendUtilityBar = new AmEditorToolBarClass( _wForm,
                                                     "legendUtilityBar");
    _pwcLegendUtilityBar->AttachTop( _pwcFileUtilityBar);
    _pwcLegendUtilityBar->AttachRight();

// Create the annotation related utility bar.

    _pwcAnnotUtilityBar = new AmEditorToolBarClass( _wForm, "annotUtilityBar");
    _pwcAnnotUtilityBar->AttachTop( _pwcLegendUtilityBar);
    _pwcAnnotUtilityBar->AttachRight();
    _pwcAnnotUtilityBar->AttachBottom( *_pwcStatusForm);

// Main drawing area.

    _wCornerFrame = XtVaCreateManagedWidget( "cornerFrame",
                                             xmFrameWidgetClass,
                                             _wForm,
                                             StickLeft,
                                             BelowOf( *_pwcToolBar),
                                             NULL);

    _wWindow = XtVaCreateManagedWidget( "window", 
                                        xmScrolledWindowWidgetClass,
			  		_wForm,
                                        RightOf( _wCornerFrame),
				        BelowOf( _wCornerFrame),
				        LeftOf( *_pwcZoomUtilityBar), 
                                        AboveOf( *_pwcStatusForm),
					XmNscrollingPolicy, XmAUTOMATIC,
					XmNvisualPolicy, XmCONSTANT,
					NULL);

    iScreenWidth = (int)(_iScreenDPI * _dWidthInInches);
    iScreenHeight = (int)(_iScreenDPI * _dHeightInInches);

    _wArea = XtVaCreateManagedWidget( "area",
				      xmDrawingAreaWidgetClass,
                                      _wWindow, 
				      XmNwidth, iScreenWidth,
				      XmNheight, iScreenHeight,
				      NULL);

    _pwcHorizontalRuler = new AmRulerClass( _wForm, "topRuler", _wArea, FALSE); 
    _pwcHorizontalRuler->SetDPI( _iScreenDPI);
    XtVaSetValues( *_pwcHorizontalRuler,
                   RightOf( _wCornerFrame),
                   BelowOf( *_pwcToolBar),
                   LeftOf( *_pwcZoomUtilityBar),
                   BottomWith( _wCornerFrame),
                   NULL);

    _pwcVerticalRuler = new AmRulerClass( _wForm, "leftRuler", _wArea, TRUE); 
    _pwcVerticalRuler->SetDPI( _iScreenDPI);
    XtVaSetValues( *_pwcVerticalRuler,
                   StickLeft,
                   BelowOf( _wCornerFrame),
                   RightWith( _wCornerFrame),
                   AboveOf( *_pwcStatusForm),
                   NULL);

// Create the double buffered motif display.

    _pXDisplay = new AmXDisplayClass( _wArea);
    _pXWindow = new AmXWindowClass( _pXDisplay, _wArea);

    _pMotifDisplay = new AmMotifDisplayClass( _pXDisplay, _pXWindow, TRUE);
    _pMotifDisplay->GetDisplay()->SetDPI( _iScreenDPI);

// Set the message area.

    _pMotifDisplay->SetMessage( *_pwcTextMessage);

// Create the buttons for the tool bar.

    _pwcToolBar->AddButton( "newMapBtn",
                            ( XtCallbackProc) AmNewMapCallback,
                            ( XtPointer) this);
    _pwcToolBar->AddButton( "openMapBtn",
                            ( XtCallbackProc) AmLoadMapCallback,
                            ( XtPointer) this);
    _pwcToolBar->AddButton( "saveMapBtn",
                            ( XtCallbackProc) AmSaveMapCallback,
                            ( XtPointer) this);
    _pwcToolBar->AddSeparator();
    _pwcToolBar->AddButton( "printMapBtn",
                            ( XtCallbackProc) AmPrintMapCallback,
                            ( XtPointer) this);
    _pwcToolBar->AddButton( "previewMapBtn", 
                            ( XtCallbackProc) AmPrintMapPreviewCallback,
                            ( XtPointer) this);
    _pwcToolBar->AddSeparator();
    _pwcToolBar->AddButton( "cutBtn",
                            ( XtCallbackProc) AmCutCallback,
                            ( XtPointer) this,
                             AmSELECT_ONE_OR_MORE, AmANY_OBJECT);
    _pwcToolBar->AddButton( "copyBtn",
                            ( XtCallbackProc) AmCopyCallback,
                            ( XtPointer) this,
                             AmSELECT_ONE_OR_MORE, AmANY_OBJECT);
    _pwcToolBar->AddButton( "pasteBtn",
                            ( XtCallbackProc) AmPasteCallback,
                            ( XtPointer) this,
                             AmSELECT_CUSTOM, AmANY_OBJECT);
    _pwcToolBar->AddSeparator();
    _pwcToolBar->AddButton( "editBtn",
                            ( XtCallbackProc) AmEditCallback,
                            ( XtPointer) this,
                             AmSELECT_ONE_OR_MORE, AmANY_OBJECT);
    _pwcToolBar->AddSeparator();
    _pwcToolBar->AddButton( "refreshBtn",
                            ( XtCallbackProc) AmRefreshCallback,
                            ( XtPointer) _pMotifDisplay);
    _pwcToolBar->AddSeparator();

// Create the buttons for the utility bar.

    _pwcZoomUtilityBar->AddButton( "zoom2xBtn", 
                                   ( XtCallbackProc) AmZoom2xCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);
    _pwcZoomUtilityBar->AddButton( "zoomRectBtn",
                                   ( XtCallbackProc) AmZoomRectangleCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);
    _pwcZoomUtilityBar->AddButton( "zoomTextBtn",
                                   ( XtCallbackProc) AmZoomTextCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);

    _pwcZoomUtilityBar->AddButton( "unzoom2xBtn",
                                   ( XtCallbackProc) AmUnzoomHalfCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);
    _pwcZoomUtilityBar->AddButton( "unzoomBtn",
                                   ( XtCallbackProc) AmUnzoomCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);
    _pwcZoomUtilityBar->AddButton( "panBtn",
                                   ( XtCallbackProc) AmPanCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);

    _pwcFileUtilityBar->AddButton( "addObjectBtn",
                                   ( XtCallbackProc) AmAddObjectCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ANY, AmANY_OBJECT);
    _pwcFileUtilityBar->AddButton( "sortLayerBtn",
                                   ( XtCallbackProc) AmSortLayerCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);
    _pwcFileUtilityBar->AddButton( "addLayerBtn",
                                   ( XtCallbackProc) AmAddLayerCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);
    _pwcFileUtilityBar->AddButton( "deleteLayerBtn",
                                   ( XtCallbackProc) AmDeleteLayerCallback,
                                   ( XtPointer) this,
                                   AmSELECT_ONE, AmOVERLAY);

    _pwcLegendUtilityBar->AddButton( "addLegendBtn",
                                     ( XtCallbackProc) AmAddLegendCallback,
                                     ( XtPointer) this,
                                     AmSELECT_ONE, AmOVERLAY);
    _pwcLegendUtilityBar->AddButton( "addTickBtn",
                                     ( XtCallbackProc) AmTickmarksCallback,
                                     ( XtPointer) this,
                                     AmSELECT_ONE, AmOVERLAY);
    _pwcLegendUtilityBar->AddButton( "mapScale",
                                     ( XtCallbackProc) AmDistanceScaleCallback,
                                     ( XtPointer) this,
                                     AmSELECT_ONE, AmOVERLAY);

    _pwcAnnotUtilityBar->AddButton( "textBtn",
                                    ( XtCallbackProc) AmTextCallback,
                                    ( XtPointer) this);
    _pwcAnnotUtilityBar->AddButton( "rectangleBtn",
                                    ( XtCallbackProc) AmRectangleCallback,
                                    ( XtPointer) this);

// Create the file menu.

    _pwcFilePane = new AmMenuPaneClass( _pwcMenu, "fileCasc"); 
    _pwcFilePane->AddButton( "newMenuBtn",
                            ( XtCallbackProc) AmNewMapCallback,
                            ( XtPointer) this);
    _pwcFilePane->AddSeparator();
    _pwcFilePane->AddButton( "openMenuBtn",
                            ( XtCallbackProc) AmLoadMapCallback,
                            ( XtPointer) this);
    _pwcFilePane->AddButton( "saveMenuBtn",
                            ( XtCallbackProc) AmSaveMapCallback,
                            ( XtPointer) this);

    _pwcSaveAsPane = new AmMenuPaneClass( _pwcFilePane, "saveAsCasc");
    _pwcSaveAsPane->AddButton( "saveMapMenuBtn",
                               ( XtCallbackProc) AmSaveMapAsCallback,
                               ( XtPointer) this);
    _pwcSaveAsPane->AddButton( "saveImageMenuBtn",
                               ( XtCallbackProc) AmSaveMapAsTiffCallback,
                               ( XtPointer) this);
    _pwcSaveAsPane->AddButton( "savePostscriptMenuBtn",
                               ( XtCallbackProc) AmSaveMapAsPostscriptCallback,
                               ( XtPointer) this);

    _pwcFilePane->AddSeparator();
    _pwcFilePane->AddButton( "pageSetupMenuBtn",
                             ( XtCallbackProc) AmPageSetupCallback,
                             ( XtPointer) this);
    _pwcFilePane->AddButton( "previewMenuBtn",
                             ( XtCallbackProc) AmPrintMapPreviewCallback,
                             ( XtPointer) this);
    _pwcFilePane->AddButton( "printMenuBtn", NULL, ( XtPointer) this);
    _pwcFilePane->AddSeparator();

//
// Figure out whether the editor is standalone (quit button) or not (close
//   button).
//

    _fStandalone = ( XtParent( _wParent) == NULL);
    if ( _fStandalone) {
        _pwcFilePane->AddButton( "quitMenuBtn",
                                 ( XtCallbackProc) AmQuitCallback,
                                 ( XtPointer) this);
    } else {
        _pwcFilePane->AddButton( "closeMenuBtn", NULL, ( XtPointer) this);
    }

// Create the edit menu.

    _pwcEditPane = new AmMenuPaneClass( _pwcMenu, "editCasc");
    _pwcEditPane->AddButton( "editObjectMenuBtn", 
                             ( XtCallbackProc) AmEditCallback,
                             ( XtPointer) this);
    _pwcEditPane->AddSeparator();
    _pwcEditPane->AddButton( "cutMenuBtn", 
                             ( XtCallbackProc) AmCutCallback,
                             ( XtPointer) this);
    _pwcEditPane->AddButton( "copyMenuBtn", 
                             ( XtCallbackProc) AmCopyCallback,
                             ( XtPointer) this);
    _pwcEditPane->AddButton( "pasteMenuBtn", 
                             ( XtCallbackProc) AmPasteCallback,
                             ( XtPointer) this);
    _pwcEditPane->AddButton( "deleteMenuBtn", 
                             ( XtCallbackProc) AmDelCallback,
                             ( XtPointer) this);

    _pwcEditPane->AddSeparator();

    _pwcEditPane->AddButton( "selectAllMenuBtn", 
                             ( XtCallbackProc) AmSelectAllCallback,
                             ( XtPointer) this);
    _pwcEditPane->AddButton( "selectNoneMenuBtn", 
                             ( XtCallbackProc) AmSelectNoneCallback,
                             ( XtPointer) this);

    _pwcEditPane->AddSeparator();


    _pwcPopPane = new AmMenuPaneClass( _pwcEditPane, "popCasc");
    _pwcPopPane->AddButton( "topMenuBtn", 
                            ( XtCallbackProc) AmPopTopCallback,
                            ( XtPointer) this);
    _pwcPopPane->AddButton( "upMenuBtn", 
                            ( XtCallbackProc) AmPopCallback,
                            ( XtPointer) this);

    _pwcPushPane = new AmMenuPaneClass( _pwcEditPane, "pushCasc");
    _pwcPushPane->AddButton( "bottomMenuBtn", 
                             ( XtCallbackProc) AmPushBottomCallback,
                             ( XtPointer) this);
    _pwcPushPane->AddButton( "downMenuBtn", 
                             ( XtCallbackProc) AmPushCallback,
                             ( XtPointer) this);

// Create the insert menu.

    _pwcInsertPane = new AmMenuPaneClass( _pwcMenu, "insertCasc");
    _pwcInsertPane->AddButton( "objectMenuBtn",
                               ( XtCallbackProc) AmAddObjectCallback,
                               ( XtPointer) this);
    _pwcInsertPane->AddButton( "mapMenuBtn", 
                               ( XtCallbackProc) AmImportMapCallback,
                               ( XtPointer) this);
    _pwcInsertPane->AddButton( "imageMenuBtn",
                               ( XtCallbackProc) AmImageCallback,
                               ( XtPointer) this);

    _pwcAnnotationPane = new AmMenuPaneClass( _pwcInsertPane, "annotationCasc");
    _pwcAnnotationPane->AddButton( "rectangleMenuBtn",
                                   ( XtCallbackProc) AmRectangleCallback,
                                   ( XtPointer) this);
    _pwcAnnotationPane->AddButton( "textMenuBtn",
                                   ( XtCallbackProc) AmTextCallback,
                                   ( XtPointer) this);

    _pwcLegendPane = new AmMenuPaneClass( _pwcInsertPane, "legendCasc");
    _pwcLegendPane->AddButton( "legendMenuBtn",
                               ( XtCallbackProc) AmAddLegendCallback,
                               ( XtPointer) this);
    _pwcLegendPane->AddButton( "tickmarksMenuBtn",
                               ( XtCallbackProc) AmTickmarksCallback,
                               ( XtPointer) this);
    _pwcLegendPane->AddButton( "distanceMenuBtn",
                               ( XtCallbackProc) AmDistanceScaleCallback,
                               ( XtPointer) this);
// Create the view menu.

    _pwcViewPane = new AmMenuPaneClass( _pwcMenu, "viewCasc");
    _pwcViewPane->AddToggleButton( "gridMenuBtn",
                                   ( XtCallbackProc) AmGridCallback,
                                   ( XtPointer) this);
    pwcToggle = _pwcViewPane->AddToggleButton( "rulerMenuBtn",
                                  ( XtCallbackProc) AmRulerCallback,
                                  ( XtPointer) this);
    pwcToggle->SetState( TRUE);

    _pwcViewPane->AddSeparator();

    _pwcZoomPane = new AmMenuPaneClass( _pwcViewPane, "zoomCasc");
    _pwcZoomPane->AddButton( "map25MenuBtn",
                             ( XtCallbackProc) AmMap25Callback,
                             ( XtPointer) this);
    _pwcZoomPane->AddButton( "map33MenuBtn",
                             ( XtCallbackProc) AmMap33Callback,
                             ( XtPointer) this);
    _pwcZoomPane->AddButton( "map50MenuBtn",
                             ( XtCallbackProc) AmMap50Callback,
                             ( XtPointer) this);
    _pwcZoomPane->AddButton( "map100MenuBtn",
                             ( XtCallbackProc) AmMap100Callback,
                             ( XtPointer) this);
    _pwcZoomPane->AddButton( "map200MenuBtn",
                             ( XtCallbackProc) AmMap200Callback,
                             ( XtPointer) this);

    _pwcViewPane->AddSeparator();
    _pwcViewPane->AddButton( "refreshMenuBtn",
                             ( XtCallbackProc) AmRefreshCallback,
                             ( XtPointer) _pMotifDisplay);

    _pwcHelpPane = new AmMenuPaneClass( _pwcMenu, "helpCasc");
    _pwcHelpPane->SetAsHelpPane();

    _pwcHelpPane->AddButton( "aboutMenuBtn",
                             ( XtCallbackProc) AmAboutCallback,
                             ( XtPointer) this);

// Create the option menu.

    _pwcToolPane = new AmMenuPaneClass( _pwcMenu, "toolCasc");
    _pwcToolPane->AddButton( "alignMenuBtn", AmAlignCallback,
                             ( XtPointer) this);

    _pwcPageSetupDialog = new AmPageSetupDialogClass( _wForm,
                                                      "pageSetupDialog");
    _pwcPageSetupDialog->HideOnDelete();

    _pwcPageSetupDialog->AddButton( "okBtn",
                                    ( XtCallbackProc) AmPageSetupOkCallback,
                                    ( XtPointer) this);
    _pwcPageSetupDialog->AddButton( "cancelBtn",
                                    ( XtCallbackProc) AmPageSetupCancelCallback,
                                    ( XtPointer) this);
    _pwcPageSetupDialog->AddButton( "helpBtn", NULL, NULL);

    _pwcTextDialog = new AmTextDialogClass( _wForm, "textDialog");
    _pwcTextDialog->HideOnDelete();

    _pwcTextDialog->AddButton( "okBtn",
                               ( XtCallbackProc) AmTextOkCallback,
                               ( XtPointer) this);
    _pwcTextDialog->AddButton( "cancelBtn",
                               ( XtCallbackProc) AmTextCancelCallback,
                               ( XtPointer) this);
    _pwcTextDialog->AddButton( "helpBtn", NULL, NULL);

    
    _pwcAlignDialog = new AmAlignDialogClass( _wForm, "alignDialog", this);
    _pwcAlignDialog->HideOnDelete();

    _pwcAlignDialog->AddButton( "cancelBtn",
                                ( XtCallbackProc) AmAlignCancelCallback,
                                ( XtPointer) this);
    _pwcAlignDialog->AddButton( "helpBtn", NULL, NULL);
    

//
// Add the handler that automatically scrolls the window when the mouse
//   is being dragged out of its extents (while one of the buttons are
//   pressed).
//

    XtAddEventHandler( _wArea, LeaveWindowMask, FALSE,
                       ( XtEventHandler) AmScrollButtonHandler,
                       NULL);

//
// Add the master handler that takes care of our objects (i.e. moving
//   resizing, selecting, etc).
//
  
    AmAddHandler( NULL, _wArea,
                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                ( XtEventHandler) & AmEditorClass::AmMasterHandler,
                ( XtPointer) this, AmACTION_MASTER_PRIORITY,
                ( AmStateHandlerFn) & AmEditorClass::AmRemoveMasterCallback,
                ( AmStateHandlerFn) & AmEditorClass::AmSuspendMasterCallback,
                ( AmStateHandlerFn) & AmEditorClass::AmRestartMasterCallback);

    AmSendUserEvent( AM_PKG_ID, AM_SELECTION_SET_CHANGED, _lstSelectedObjects);

    XtVaGetValues( AmGetWidgetShell( _wParent), XmNtitle, &sz, NULL);
    _szTitle = strdup( sz);

    SetTitle( "Untitled");

    AmAddCallback( AM_PKG_ID, AM_DELETE_OBJECT,
                 ( callbackProc) AmObjectDeletedCallback,
                 ( XtPointer) this);

    AmAddCallback( AM_PKG_ID, AM_RECT_CHANGE,
                 ( callbackProc) AmRectChangedCallback,
                 ( XtPointer) this);

    AmAddCallback( AM_PKG_ID, AM_DISPLAY_CHANGE,
                 ( callbackProc) AmDisplayChangedCallback,
                 ( XtPointer) this);

}


// Default constructor.

AmEditorClass::AmEditorClass()
    : AmBaseWidgetClass()
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmEditorClass()\n"));

// Initialize all the member variables to sane, but easily identifiable
//   values.

    _pMotifDisplay = NULL;
    _pOutline      = NULL;

    _fMapChanged   = FALSE;
    _szCurrentFile = NULL;
    _szTitle       = NULL;

    _lstSelectedObjects = NULL;

    _fGrid        = FALSE;
    _dGridSpacing = 0;
    _fSnapToGrid  = FALSE;

    _dWidthInInches  = 0;
    _dHeightInInches = 0;

    _iScreenDPI = 0;
    _iPrintDPI  = 0;

    _fLandscape = FALSE;
    _fTrueColor = FALSE;

    _szPrologFile = NULL;

    _fClipFileValid = FALSE;
    _szClipFile     = NULL;

    _fStandalone              = FALSE;

// Interface widgets.

    _wForm              = NULL;
    _pwcMessageFrame    = NULL;
    _pwcTextMessage     = NULL;

    _wCornerFrame       = NULL;
    _wWindow            = NULL;
    _wArea              = NULL;

    _pwcMenu            = NULL;
    _pwcFilePane        = NULL;
    _pwcSaveAsPane      = NULL;
    _pwcEditPane        = NULL;
    _pwcInsertPane      = NULL;
    _pwcAnnotationPane  = NULL;
    _pwcToolPane        = NULL;
    _pwcViewPane        = NULL;
    _pwcZoomPane        = NULL;

    _pwcToolBar         = NULL;
    _pwcZoomUtilityBar  = NULL;
    _pwcFileUtilityBar  = NULL;

    _pwcHorizontalRuler = NULL;
    _pwcVerticalRuler   = NULL;

/* X specific classes. */

    _pXDisplay = NULL;
    _pXWindow  = NULL;
}

AmEditorClass::AmEditorClass( Widget     wParent,
                              const char *szName)
    : AmBaseWidgetClass( wParent, szName)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmEditorClass()\n"));

// Do the rest of the initialization.

    initializeValues();
    createInterface();
}

AmEditorClass::AmEditorClass( AmBaseWidgetClass *pwcParent,
                              const char        *szName)
    : AmBaseWidgetClass( pwcParent, szName)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmEditorClass()\n"));

// Do the rest of the initialization.

    initializeValues();
    createInterface();
}

AmEditorClass::~AmEditorClass()
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::~AmEditorClass()\n"));

// Remove any installed callbacks.

    AmRemoveCallback( AM_PKG_ID, AM_DISPLAY_CHANGE,
                    ( callbackProc) AmDisplayChangedCallback,
                    ( XtPointer) this);

    AmRemoveCallback( AM_PKG_ID, AM_DELETE_OBJECT,
                    ( callbackProc) AmObjectDeletedCallback,
                    ( XtPointer) this);

    AmRemoveCallback( AM_PKG_ID, AM_RECT_CHANGE,
                    ( callbackProc) AmRectChangedCallback,
                    ( XtPointer) this);

//
// Remove the master handler, since we are about to remove its
//   client data (the this pointer).
//

#if 0

    AmRemoveHandler( _wArea,
                   ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                   ( XtEventHandler) & AmEditorClass::AmMasterHandler,
                   ( XtPointer) this);

#endif

    XtRemoveEventHandler( _wArea, LeaveWindowMask, FALSE,
                          ( XtEventHandler) AmScrollButtonHandler,
                          NULL);

    if ( _lstSelectedObjects != NULL) {
        AmJustFreeLst( _lstSelectedObjects);
        _lstSelectedObjects = NULL;
    }

/*
 * Remove all the possible strings that could be allocated
 *   (They are mostly temporary file names). Also remove any
 *   temporary files that could still be hanging around.
 */

    if ( _szCurrentFile != NULL) {
        free( _szCurrentFile);
        _szCurrentFile = NULL;
    }

    if ( _szPrologFile != NULL) {
        free( _szPrologFile);
        _szPrologFile = NULL;
    }

    if ( _szClipFile != NULL) {
        if ( _fClipFileValid == TRUE) {
            unlink( _szClipFile);
        }
        free( _szClipFile);
        _szClipFile = NULL;
    }

/* Motif display removes any outlines that are added to it. */

    _pOutline           = NULL;

/* Note that we don't need to delete either the window, nor the display,
   nor the motif display.  These get deleted automatically when the widgets
   get destroyed. */

    _pMotifDisplay      = NULL;
    _pXWindow           = NULL;
    _pXDisplay          = NULL;

/* The same goes for all of the widgets. */

    _pwcMenu            = NULL;
    _pwcFilePane        = NULL;
    _pwcToolPane        = NULL;
    _pwcInsertPane      = NULL;
    _pwcAnnotationPane  = NULL;
    _pwcToolBar         = NULL;
    _pwcZoomUtilityBar  = NULL;
    _pwcFileUtilityBar  = NULL;
    _pwcHorizontalRuler = NULL;
    _pwcVerticalRuler   = NULL;

/* Clean up after the error reporting routines. */

    AmRemoveLogFile();

}


void
AmEditorClass::AmSelectAllCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSelectAllCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->selectAll( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::selectAll( Widget    w,
                        XtPointer xpCallData)
{
    AmScreenObjClass **vpScreenObj;
    int            ii;
    
    UnselectAll();

    vpScreenObj = _pMotifDisplay->GetObjects();

    if ( vpScreenObj != NULL) {
        for ( ii = 0; vpScreenObj[ ii] != NULL; ii++) {
            SelectObject( vpScreenObj[ ii]);
        }

        FREE( vpScreenObj);
    }
}


void
AmEditorClass::AmSelectNoneCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSelectNoneCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->selectNone( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::selectNone( Widget    w,
                         XtPointer xpCallData)
{
    UnselectAll();
}


AmAnswerEnum
AmEditorClass::AskForSaveBeforeChange( char           *szQuestion, 
                                       Widget         w,
                                       XtCallbackProc xcp,
                                       XtPointer      xpCallData)
{
    AmScreenObjClass **vpScreenObj;
    AmAnswerEnum     eAnswer;

    eAnswer     = AmANSWER_NO;
    vpScreenObj = _pMotifDisplay->GetObjects();

//
// If changes were made to the map, and the map is not empty ask the 
//   user for permission to quit (annoying isn't it?)
//

    if ( vpScreenObj && vpScreenObj[ 0] && _fMapChanged) {
        eAnswer = AmQuestionResourceDialog( _wForm, szQuestion);
        if ( eAnswer == AmANSWER_YES) {
            saveMap( w, xcp, xpCallData);
        }
    }

// Get rid of the temporary vector. 

    if ( vpScreenObj) {
        FREE( vpScreenObj);
    }

// Return either the result of the query, or AmANSWER_NO if the map 
//   wasn't changed, or the map was empty.

    return( eAnswer);

}

/* ARGSUSED */
void
AmEditorClass::AmQuitCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmQuitCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->quit( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::quit( Widget    w,
                     XtPointer xpCallData)
{
    AmAnswerEnum     eAnswer;

    eAnswer = AskForSaveBeforeChange( "quitQuestion", w,
                                      ( XtCallbackProc) AmQuitCallback,
                                      xpCallData);

    if ( eAnswer == AmANSWER_NO) {
        if ( _fStandalone == TRUE) {
            AmQuit();
        } else {
            XtPopdown( _wParent);
        }
    }
}


void
AmEditorClass::AmPageSetupCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPageSetupCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->pageSetup( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::pageSetup( Widget    w,
                          XtPointer xpCallData)
{

    _pwcPageSetupDialog->SetWidth( _dWidthInInches); 
    _pwcPageSetupDialog->SetHeight( _dHeightInInches);
    _pwcPageSetupDialog->SetLandscape( _fLandscape);
    _pwcPageSetupDialog->SetTrueColor( _fTrueColor);

    _pwcPageSetupDialog->Show();
}

void
AmEditorClass::AmPageSetupOkCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPageSetupOkCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->pageSetupOk( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::pageSetupOk( Widget    w,
                            XtPointer xpCallData)
{
    int iScreenWidth, iScreenHeight;

    XtUnmapWidget( *_pwcPageSetupDialog);

    _dWidthInInches = _pwcPageSetupDialog->GetPageWidth();
    _dHeightInInches = _pwcPageSetupDialog->GetPageHeight();
    _fLandscape = _pwcPageSetupDialog->GetLandscape();
    _fTrueColor = _pwcPageSetupDialog->GetTrueColor();

    iScreenWidth = (int)(_dWidthInInches * _iScreenDPI);
    iScreenHeight = (int)(_dHeightInInches * _iScreenDPI);

    XtVaSetValues( _wArea,
                   XmNwidth, iScreenWidth,
                   XmNheight, iScreenHeight,
                   NULL);
}


void
AmEditorClass::AmPageSetupCancelCallback( Widget    w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPageSetupCancelCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->pageSetupCancel( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::pageSetupCancel( Widget    w,
                                XtPointer xpCallData)
{
    XtUnmapWidget( *_pwcPageSetupDialog);
}

void
AmEditorClass::AmCutCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmCutCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->cut( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::cut( Widget    w,
                    XtPointer xpCallData)
{
    AmSelectObjClass   *pSelectObj;
    AmScreenObjClass   *pScreenObj, **vpScreenObj, *pTempObj, *pParent;
    AmDataGraphicClass **vpDataGraphic;
    AmOverlayClass     *pOverlay;
    AmListPtr          lstCheck, lstTemp, lstDelete;
    AnvilBoolean            fFound, fAppend, fError;
    int                ii, nDataGraphic;


    if ( _lstSelectedObjects == NULL) {

        AmPrintResourceError( AmGetWidgetShell( _wParent),
                              "noSelectedObjects");

    } else {

        _fClipFileValid = FALSE;

    // Initializations.

        lstDelete = NULL;
        fAppend   = FALSE;
        fError    = FALSE;

        AmInitLog();

        for ( lstTemp = _lstSelectedObjects;
              lstTemp != NULL;
              lstTemp = lstTemp->next) {

            pSelectObj = ( AmSelectObjClass *) lstTemp->data;
            pScreenObj = pSelectObj->GetScreenObj();
            pParent = pScreenObj->getParent();

        // If the object being cut has a parent, make sure we only 
        //   save it if it's parent is also being cut.

            if ( pParent != NULL) {

                for ( lstCheck = _lstSelectedObjects, fFound = FALSE;
                      ( lstCheck != NULL) && ( fFound == FALSE);
                      lstCheck = lstCheck->next) { 

                    pSelectObj = ( AmSelectObjClass *) lstCheck->data;
                    pTempObj   = pSelectObj->GetScreenObj();

                    if ( pTempObj == pParent) { 
                        fFound = TRUE;
                    } else if ( pTempObj->IsType( AmOVERLAY)) {

                        pOverlay = ( AmOverlayClass *) pTempObj; 
                        pOverlay->GetLayers( &vpDataGraphic, &nDataGraphic);

                        for ( ii = 0; ii < nDataGraphic; ii++) {
                            if ( vpDataGraphic[ ii] == pParent) {
                                fFound = TRUE;
                            }
                        }

                        if ( vpDataGraphic != NULL) {
                            FREE( vpDataGraphic);
                        }

                    }

                }

            }

            if (( pParent == NULL) || ( fFound == TRUE)) {

                if ( SaveObjectToFile( _szClipFile, pScreenObj,
                                       fAppend) != AmOK) {
                    fError = TRUE;
                } else {
                    fAppend = TRUE;

                    if ( pParent == NULL) {
                        _fClipFileValid = TRUE;
                    }

                }

            }

            lstDelete = AmInsertNodeLst( pScreenObj, lstDelete);

        }

        vpScreenObj = ( AmScreenObjClass **) AmMakeVectorLst( lstDelete);
        _pMotifDisplay->RemoveObjectSet( vpScreenObj, AmLengthLst( lstDelete));
        AmJustFreeLst( lstDelete);

        _pMotifDisplay->Flush();
        _pMotifDisplay->WriteResourceMessage( "cutMsg");

        if ( fError == TRUE) {
            AmPrintLogError( AmGetWidgetShell( _wParent), "saveFailed");
        }

    }
}


void
AmEditorClass::AmCopyCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmCopyCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->copy( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::copy( Widget    w,
                     XtPointer xpCallData)
{
    AmSelectObjClass   *pSelectObj;
    AmScreenObjClass   *pScreenObj, *pTempObj, *pParent;
    AmDataGraphicClass **vpDataGraphic;
    AmOverlayClass     *pOverlay;
    AmListPtr          lstTemp, lstCheck;
    AnvilBoolean            fFound, fAppend, fError;
    int                ii, nDataGraphic;

    if ( _lstSelectedObjects == NULL) {

        AmPrintResourceError( AmGetWidgetShell( _wParent),
                              "noSelectedObjects");

    } else {

        _fClipFileValid = FALSE;

        fAppend = FALSE;
        fError  = FALSE;

        for ( lstTemp = _lstSelectedObjects;
              lstTemp != NULL;
              lstTemp = lstTemp->next) {

            pSelectObj = ( AmSelectObjClass *) lstTemp->data;
            pScreenObj = pSelectObj->GetScreenObj();
            pParent = pScreenObj->getParent();

            if ( pParent != NULL) {

                for ( lstCheck = _lstSelectedObjects, fFound = FALSE;
                      ( lstCheck != NULL) && ( fFound == FALSE);
                      lstCheck = lstCheck->next) { 

                    pSelectObj = ( AmSelectObjClass *) lstCheck->data;
                    pTempObj   = pSelectObj->GetScreenObj();

                    if ( pTempObj == pParent) { 
                        fFound = TRUE;
                    } else if ( pTempObj->IsType( AmOVERLAY)) {

                        pOverlay = ( AmOverlayClass *) pTempObj; 
                        pOverlay->GetLayers( &vpDataGraphic, &nDataGraphic);

                        for ( ii = 0; ii < nDataGraphic; ii++) {
                            if ( vpDataGraphic[ ii] == pParent) {
                                fFound = TRUE;
                            }
                        }

                        if ( vpDataGraphic != NULL) {
                            FREE( vpDataGraphic);
                        }

                    }

                }

            }

            if (( pParent == NULL) || ( fFound == TRUE)) {

                if ( SaveObjectToFile( _szClipFile, pScreenObj,
                                       fAppend) != AmOK) {
                    fError = TRUE;
                } else {
                    fAppend = TRUE;

                    if ( pParent == NULL) {
                        _fClipFileValid = TRUE;
                    }

                }

            }

        }

        if ( fError == TRUE) {
            AmPrintLogError( AmGetWidgetShell( _wParent), "saveFailed");
        } else {
            _pMotifDisplay->WriteResourceMessage( "copyMsg");
        }
    }
}


void
AmEditorClass::AmPasteCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPasteCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->paste( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::paste( Widget    w,
                      XtPointer xpCallData)
{
    AmListPtr lstObjects, lstTemp, lstPrev, lstNext;

    if ( _fClipFileValid == FALSE) {

        _pMotifDisplay->WriteResourceMessage( "cantPasteMsg");

    } else {

        lstObjects = LoadObjectsFromFile( _szClipFile);

        if ( lstObjects == NULL) {

            AmPrintLogError( AmGetWidgetShell( _wParent),
                             "pasteFailed");
            return;
        }

    /* Reverse the list of objects, so that they get added to the
       motifdisplay in correct order. */

       lstTemp = lstObjects;
       lstPrev = NULL;

       while ( lstTemp != NULL) {
           lstNext = lstTemp->next;
           lstTemp->next = lstPrev;
           lstPrev = lstTemp;
           lstTemp = lstNext;
       }

       lstObjects = lstPrev;

        _pMotifDisplay->WriteResourceMessage( "addInstructionMsg");
        AmSetupAddObjectsHandler( this, lstObjects);

    }
}


void
AmEditorClass::AmDelCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmDelCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->del( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::del( Widget    w,
                  XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj, **vpScreenObj;
    AmListPtr        lstTemp, lstDelete;

    if ( _lstSelectedObjects == NULL) {

        AmPrintResourceError( AmGetWidgetShell( _wParent), "noSelectedObjects");

    } else {

        lstDelete = NULL;

        for ( lstTemp = _lstSelectedObjects;
              lstTemp != NULL;
              lstTemp = lstTemp->next) {

            pSelectObj = ( AmSelectObjClass *) lstTemp->data;
            pScreenObj = pSelectObj->GetScreenObj();

            lstDelete = AmInsertNodeLst( pScreenObj, lstDelete);
        }

        vpScreenObj = ( AmScreenObjClass **) AmMakeVectorLst( lstDelete);
        _pMotifDisplay->RemoveObjectSet( vpScreenObj, AmLengthLst( lstDelete));
        AmJustFreeLst( lstDelete);

        _pMotifDisplay->Flush();
        _pMotifDisplay->WriteResourceMessage( "deleteMsg");

    }
}


void
AmEditorClass::AmPopTopCallback( Widget    w,
                             XtPointer xpClientData,
                             XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPopTopCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->popTop( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::popTop( Widget    w,
                       XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;

    if ( ObjectSelected( AmANY_OBJECT) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        _pMotifDisplay->PopToTop( pScreenObj);
        _pMotifDisplay->Flush();

    }
}


void
AmEditorClass::AmPopCallback( Widget    w,
                          XtPointer xpClientData,
                          XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPopCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->pop( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::pop( Widget    w,
                    XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;

    if ( ObjectSelected( AmANY_OBJECT) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        _pMotifDisplay->Pop( pScreenObj);
        _pMotifDisplay->Flush();

    }
}


void
AmEditorClass::AmPushBottomCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPushButtonCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->pushBottom( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::pushBottom( Widget    w,
                         XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;

    if ( ObjectSelected( AmANY_OBJECT) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        _pMotifDisplay->PushToBottom( pScreenObj);
        _pMotifDisplay->Flush();

    }
}


void
AmEditorClass::AmPushCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPushCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->push( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::push( Widget    w,
                     XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;

    if ( ObjectSelected( AmANY_OBJECT) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        _pMotifDisplay->Push( pScreenObj);
        _pMotifDisplay->Flush();

    }
}


void
AmEditorClass::AmAddLayerCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmAddLayerCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->addLayer( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::addLayer( Widget    w,
                       XtPointer xpCallData)
{
    AmOverlayClass        *pOverlay;
    AmRectangleRec const  *pRec;
    AmScreenObjClass      *pScreenObj;
    AmSelectObjClass      *pSelectObj;

    pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
    pScreenObj = pSelectObj->GetScreenObj();
    pRec = pScreenObj->GetRect();

/* If the object is already an overlay then use it as is, otherwise, create
   an overlay to contain the selected object and the newly added object,
   remove the non-overlay selected object from the display list, add it as
   a layer in the overlay, and add the overlay to the display list. */

    if ( pScreenObj->IsType( AmOVERLAY)) {

    /* Selected object is already an overlay. */

        pOverlay = ( AmOverlayClass *) pScreenObj;
    } else {

    /* Create the new overlay. */

        pOverlay = new AmOverlayClass( _pXWindow, "Overlay",
                                     AmNx, pRec->x,
                                     AmNy, pRec->y,
                                     AmNwidth, pRec->width,
                                     AmNheight, pRec->height,
                                     AmNpreserveAspect, TRUE,
                                     NULL);

    /* Add the existing object into the overlay and remove it from the
       display. */

        pOverlay->AddLayer(( AmDataGraphicClass *) pScreenObj);
        UnselectObject( pScreenObj);
        _pMotifDisplay->RemoveObjectFromList( pScreenObj);

    /* Add the overlay to the display and select it. */

        _pMotifDisplay->AddObject( pOverlay);
        SelectObject( pOverlay);
    }

    pOverlay->Update();
    _pMotifDisplay->Flush();

/* Bring up a file browser to add the new layer.  Note that there is a
   slight side effect in that if no layer is loaded the old object has 
   been converted into an overlay. */

    LoadObject( pOverlay);
}

void
AmEditorClass::AmEditCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmEditCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->edit( w, xpCallData); 
}


/* ARGSUSED */
void
AmEditorClass::edit( Widget    w,
                     XtPointer xpCallData)
{
    AmEditLayerDialogClass *pwcDialog;
    AmSelectObjClass       *pSelectObj;
    AmScreenObjClass       *pScreenObj;

// Make sure that there is a selected object.

    if ( ObjectSelected( AmANY_OBJECT) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        if ( pScreenObj->IsType( AmOVERLAY) == TRUE) {

        // The selected object was an overlay, so let the user choose which
        //   layer of the overlay to edit.

            pwcDialog = new AmEditLayerDialogClass( _wForm, "pickLayerDialog",
                                ( AmOverlayClass *) pScreenObj, _pMotifDisplay);
            pwcDialog->Show();

        } else {

        // The object is ready to be edited. 

            finalEdit( _wForm, pScreenObj);

        }

    }

}

void
AmEditorClass::AmFinalEditCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmFinalEditCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->finalEdit( w, xpCallData); 
}


/* ARGSUSED */
void
AmEditorClass::finalEdit( Widget    w,
                          XtPointer xpCallData)
{
    AmScreenObjClass *pScreenObj = ( AmScreenObjClass *) xpCallData;

    AmInitLog();

    if ( AmEditScreenObject( w, _pMotifDisplay, pScreenObj, NULL) != AmOK) {
        AmPrintLogError( AmGetWidgetShell( _wParent), "editObjectFailed");
    }
}


void
AmEditorClass::AmSortLayerCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSortLayerCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmSortLayersCallback,
                           xpCallData, AmOVERLAY);
}


void
AmEditorClass::AmDeleteLayerCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmDeleteLayerCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->deleteLayer( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::deleteLayer( Widget    w,
                            XtPointer xpCallData)
{
    AmDeleteLayerDialogClass *pwcDialog;
    AmSelectObjClass         *pSelectObj;
    AmScreenObjClass         *pScreenObj;

    if ( ObjectSelected( AmOVERLAY) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        pwcDialog = new AmDeleteLayerDialogClass( _wForm, "deleteLayerDialog",
                            ( AmOverlayClass *) pScreenObj,
                            _pMotifDisplay);
        pwcDialog->Show();

    }
}


void
AmEditorClass::AmZoom2xCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmZoom2xCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmZoom2xCallback, xpCallData);
}

void
AmEditorClass::AmZoomRectangleCallback( Widget    w,
                                        XtPointer xpClientData,
                                        XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmZoomRectangleCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmZoomRectCallback, xpCallData);
}

void
AmEditorClass::AmZoomTextCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmZoomTextCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmZoomTextCallback, xpCallData);
}

void
AmEditorClass::AmUnzoomHalfCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmUnzoomHalfCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmUnzoomHalfCallback, xpCallData);
}

void
AmEditorClass::AmUnzoomCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmUnzoomCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmUnzoomCallback, xpCallData);
}

void
AmEditorClass::AmPanCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPanCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->callCallback( w, ( AmCallbackFn) ::AmPanCallback, xpCallData);
}

AnvilBoolean
AmEditorClass::callCallback( Widget           w,
                             AmCallbackFn     fnCallback,
                             XtPointer        xpCallData,
                             AmObjectTypeEnum eObjectType,
                             char             *szResource)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;

    if ( ObjectSelected( eObjectType) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        if ( !fnCallback( w, _pMotifDisplay, pScreenObj, xpCallData)) {
            AmPrintResourceError( AmGetWidgetShell( _wParent), szResource);
            return( FALSE);
        }

        return( TRUE);

    } else {

        return( FALSE);

    }
}


AnvilBoolean
AmEditorClass::ObjectSelected( AmObjectTypeEnum eObjectType)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;
    int            nElements;

    if ( _lstSelectedObjects == NULL) {

        AmPrintResourceError( "noSelectedObjects");
        return( FALSE);

    }

    nElements = AmLengthLst( _lstSelectedObjects);

    if ( nElements > 1) {
        AmPrintResourceError( "moreThanOneObjectSelected", nElements);
        return( FALSE);
    }

    pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
    pScreenObj = pSelectObj->GetScreenObj();

    if ( eObjectType != AmANY_OBJECT) {

        if ( pScreenObj->IsType( eObjectType) == FALSE) {
            AmPrintResourceError( "selectedObjectNotType");
            return( FALSE);
        }

    }

    return( TRUE);
}

void
AmEditorClass::AmRectangleCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmRectangleCallbackCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->rectangle( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::rectangle( Widget    w,
                          XtPointer xpCallData)
{
    AmRectangleAnnotClass *prac;

    prac = new AmRectangleAnnotClass( _pXWindow,
                                    "Rectangle",
                                    NULL);

    _pMotifDisplay->WriteResourceMessage( "addRectangleInstructionMsg");
    AmSetupAddHandler( this, ( AmScreenObjClass *) prac); 
}

/* ARGSUSED */
void
AmEditorClass::AmNewMapCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmNewMapCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->newMap( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::newMap( Widget    w,
                       XtPointer xpCallData)
{
    AmScreenObjClass **vpScreenObj;
    int              nScreenObj;
    AmAnswerEnum     eAnswer;

    AmInitLog();

    eAnswer = AskForSaveBeforeChange( "mapClearQuestion", w,
                                      ( XtCallbackProc) AmNewMapCallback,
                                      xpCallData);

    if ( eAnswer == AmANSWER_NO) {

        UnselectAll();
      
        vpScreenObj = _pMotifDisplay->GetObjects(); 
        nScreenObj = AmVeclen(( char const **) vpScreenObj);

        _pMotifDisplay->RemoveObjectSet( vpScreenObj, nScreenObj);


        _pMotifDisplay->Flush();

        _fMapChanged = FALSE;

        _pMotifDisplay->WriteResourceMessage( "newMapMsg");

        if ( _szCurrentFile != NULL) {
            free( _szCurrentFile);
            _szCurrentFile = NULL;
        }

        SetTitle( "Untitled");
    }
}

/* ARGSUSED */
void
AmEditorClass::AmLoadMapCallback( Widget    w,
                                  XtPointer xpClientData,
                                  XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmLoadMapCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->loadMap( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::loadMap( Widget    w,
                        XtPointer xpCallData)
{
    AmAnswerEnum eAnswer;

    eAnswer = AskForSaveBeforeChange( "mapLoadQuestion", w,
                                      ( XtCallbackProc) AmLoadMapCallback,
                                      xpCallData);

    if ( eAnswer == AmANSWER_NO) {
        AmMakeFileSelectionDialog( AmGetWidgetShell( _wParent), "LoadMap",
                                 NULL, 0, TRUE,
                                 ( VarargsCallback) AmLoadMapOkCallback,
                                 ( XtPointer) this,
                                 END_LIST);
    }
}

void
AmEditorClass::AmLoadMapOkCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    AmEditorClass                      *pEditor;
    char                             *szFileName;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmLoadMapOkCallback()\n"));

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;
    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    pEditor = ( AmEditorClass *) xpClientData;

    pEditor->loadMapOk( w, szFileName); 
    XtFree( szFileName);
}

/* ARGSUSED */
void
AmEditorClass::loadMapOk( Widget w,
                          char   *szFileName)
{
    AmInitLog();

    UnselectAll();

    if ( LoadMapFromFile( szFileName) == TRUE) { 

        _pMotifDisplay->WriteResourceMessage( "loadDoneMsg", szFileName);

        if ( _szCurrentFile != NULL) {
            free( _szCurrentFile);
        }

        _fMapChanged = FALSE;
        _szCurrentFile = strdup( szFileName);
        SetTitle( _szCurrentFile);

    } else {

        _pMotifDisplay->WriteResourceMessage( "loadFailedMsg", szFileName);
        AmPrintLogError( AmGetWidgetShell( _wParent), "loadFailedMsg",
                         szFileName);

    }
}

/* ARGSUSED */
void
AmEditorClass::AmSaveMapAsTiffCallback( Widget    w,
                                        XtPointer xpClientData,
                                        XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapAsTiffCallback()\n"));

    AmInitLog();

    AmMakeFileSelectionDialog( AmGetWidgetShell( pEditor->_wParent),
                             "SaveMapAsTiff",
                             NULL, 0, FALSE,
                             ( VarargsCallback) AmSaveMapAsTiffOkCallback,
                             ( XtPointer) pEditor,
                             END_LIST);
}

void
AmEditorClass::AmSaveMapAsTiffOkCallback( Widget    w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapAsTiffOkCallback()\n"));

    pEditor->saveMapAsTiffOk( w, xpCallData); 
}

/* ARGSUSED */
void
AmEditorClass::saveMapAsTiffOk( Widget    w,
                              XtPointer xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    char                             *szFileName;

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;

    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    if ( PrintMapToFile( szFileName, AmOUTPUT_TIFF) == AmOK) {
        _pMotifDisplay->WriteResourceMessage( "saveDoneMsg", szFileName);
    } else {
        _pMotifDisplay->WriteResourceMessage( "saveFailedMsg", szFileName);
    }
}

/* ARGSUSED */
void
AmEditorClass::AmSaveMapAsPostscriptCallback( Widget    w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapAsPostscriptCallback()\n"));

    AmInitLog();

    AmMakeFileSelectionDialog( AmGetWidgetShell( pEditor->_wParent),
                             "SaveMapAsPostscript",
                             NULL, 0, FALSE,
                             ( VarargsCallback) AmSaveMapAsPostscriptOkCallback,
                             ( XtPointer) pEditor,
                             END_LIST);
}

void
AmEditorClass::AmSaveMapAsPostscriptOkCallback( Widget    w,
                                            XtPointer xpClientData,
                                            XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapAsPostscriptOkCallback()\n"));

    pEditor->saveMapAsPostscriptOk( w, xpCallData); 
}

/* ARGSUSED */
void
AmEditorClass::saveMapAsPostscriptOk( Widget    w,
                                    XtPointer xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    char                             *szFileName;

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;

    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    if ( PrintMapToFile( szFileName, AmOUTPUT_POSTSCRIPT) == AmOK) {
        _pMotifDisplay->WriteResourceMessage( "saveDoneMsg", szFileName);
    } else {
        _pMotifDisplay->WriteResourceMessage( "saveFailedMsg", szFileName);
    }
    XtFree( szFileName);
}

/* ARGSUSED */
void
AmEditorClass::AmSaveMapAsCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapAsCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->saveMapAs( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::saveMapAs( Widget    w,
                          XtPointer xpCallData)
{
    AmInitLog();

    AmMakeFileSelectionDialog( AmGetWidgetShell( _wParent), "SaveMapAs",
                             NULL, 0, FALSE,
                             ( VarargsCallback) AmSaveMapOkCallback,
                             ( XtPointer) this,
                             ( XtCallbackProc) NULL,
                             END_LIST);
}

/* ARGSUSED */
void
AmEditorClass::AmSaveMapCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->saveMap( w, ( XtCallbackProc) NULL, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::saveMap( Widget         w,
                        XtCallbackProc callback,
                        XtPointer      xpCallData)
{
    if ( _szCurrentFile == NULL) {
        AmMakeFileSelectionDialog( AmGetWidgetShell( _wParent), "SaveMap",
                                 NULL, 0, FALSE,
                                 ( VarargsCallback) AmSaveMapOkCallback,
                                 ( XtPointer) this,
                                 ( XtCallbackProc) callback,
                                 END_LIST);
    } else {
        saveMapOk( w, _szCurrentFile, callback);
    }
}

void
AmEditorClass::AmSaveMapOkCallback( Widget         w,
                                XtPointer      xpClientData,
                                XtCallbackProc callback,
                                XtPointer      xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    AmEditorClass                    *pEditor;
    char                             *szFileName;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmSaveMapOkCallback()\n"));

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;
    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    pEditor = ( AmEditorClass *) xpClientData;

    pEditor->saveMapOk( w, szFileName, callback); 
    XtFree( szFileName);
}

/* ARGSUSED */
void
AmEditorClass::saveMapOk( Widget         w,
                          char           *szFileName,
                          XtCallbackProc callback)
{
    if ( SaveMapToFile( szFileName) == AmOK) {
        _pMotifDisplay->WriteResourceMessage( "saveDoneMsg", szFileName);
        if ( _szCurrentFile != szFileName) {
            if ( _szCurrentFile != NULL) {
                free( _szCurrentFile);
            }
            _szCurrentFile = strdup( szFileName);
            SetTitle( _szCurrentFile);
        }
        _fMapChanged = FALSE;

        if ( callback != NULL) {
            callback( w, ( XtPointer) this, NULL);
        }

    } else {
        _pMotifDisplay->WriteResourceMessage( "saveFailedMsg", szFileName);
    }
}

/* ARGSUSED */
void
AmEditorClass::AmPrintMapCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPrintMapCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->printMap( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::printMap( Widget    w,
                         XtPointer xpCallData)
{
    char       sz[ TEXTLEN];
    char       szTemp[] = "/tmp/AmPostscriptOutput.XXXXXX";
    const char *szPath;

    AmInitLog();

    mktemp( szTemp);

    if ( PrintMapToFile( szTemp, AmOUTPUT_POSTSCRIPT) == AmOK) {

        szPath = AmGetEnvFile( CONFIG_ENV, "scripts/printpostscript" );
        if ( szPath == NULL) {
            unlink( szTemp);
            DO_ERROR( AmFILE_NO_EXIST);
            return;
        }
  
        sprintf( sz, "%s %s &", szPath, szTemp);
        system( sz);

    } else {

        unlink( szTemp);

    }
}


/* ARGSUSED */
void
AmEditorClass::AmPrintMapPreviewCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmPrintMapPreviewCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->printMapPreview( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::printMapPreview( Widget    w,
                                XtPointer xpCallData)
{
    char       sz[ TEXTLEN], szTemp[] = "/tmp/AmPostscriptOutput.XXXXXX";
    const char *szPath;

    AmInitLog();

    mktemp( szTemp);

    if ( PrintMapToFile( szTemp, AmOUTPUT_POSTSCRIPT) == AmOK) {

    /* Figure out the shell script to run to display the postscript
       file. */

        szPath = AmGetEnvFile( CONFIG_ENV, "scripts/showpostscript" );
        if ( szPath == NULL) {
            unlink( szTemp);
            DO_ERROR( AmFILE_NO_EXIST);
            return;
        }
  
        if ( _fLandscape == TRUE) {
            sprintf( sz, "%s %.1f %.1f %s %s &",
                     szPath,
                     ( double) _dHeightInInches,
                     ( double) _dWidthInInches,
                     szTemp,
                     "landscape");
        } else {
            sprintf( sz, "%s %.1f %.1f %s %s &",
                     szPath,
                     ( double) _dWidthInInches, 
                     ( double) _dHeightInInches,
                     szTemp,
                     "portrait");
        }

    /* Execute the command in the background. */

        system( sz);

    } else {

    /* Try to remove the file in case the function call left
       something unsightly behind. */

        unlink( szTemp);

    }
}


void
AmEditorClass::AmGridCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmGridCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->grid( w, xpCallData); 
}

/* ARGSUSED */
void
AmEditorClass::grid( Widget    w,
                     XtPointer xpCallData)
{
    XmToggleButtonCallbackStruct *tbc;

    tbc = ( XmToggleButtonCallbackStruct *) xpCallData;

    _pMotifDisplay->SetGrid( tbc->set);
    _fGrid = tbc->set;
    _fSnapToGrid = tbc->set;
}


void
AmEditorClass::AmRulerCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmRulerCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->ruler( w, xpCallData); 
}

/* ARGSUSED */
void
AmEditorClass::ruler( Widget    w,
                      XtPointer xpCallData)
{
    XmToggleButtonCallbackStruct *tbc;
    int                          iOffsetTop, iOffsetLeft;

    tbc = ( XmToggleButtonCallbackStruct *) xpCallData;

    if ( tbc->set == TRUE) {
        XtManageChild( *_pwcHorizontalRuler);
        XtManageChild( *_pwcVerticalRuler);
        XtManageChild( _wCornerFrame);

        XtVaGetValues( *_pwcVerticalRuler,
                       XmNtopOffset, &iOffsetTop,
                       NULL);

        XtVaGetValues( *_pwcHorizontalRuler,
                       XmNleftOffset, &iOffsetLeft,
                       NULL);

        XtVaSetValues( _wWindow,
                       RightOf( _wCornerFrame),
                       BelowOf( _wCornerFrame),
                       XmNtopOffset, iOffsetTop,
                       XmNleftOffset, iOffsetLeft,
                       NULL);
    } else {

        XtVaGetValues( _wCornerFrame,
                       XmNtopOffset, &iOffsetTop,
                       XmNleftOffset, &iOffsetLeft,
                       NULL);

        XtUnmanageChild( *_pwcHorizontalRuler);
        XtUnmanageChild( *_pwcVerticalRuler);
        XtUnmanageChild( _wCornerFrame);

        XtVaSetValues( _wWindow,
                       StickLeft,
                       BelowOf( *_pwcToolBar), 
                       XmNtopOffset, iOffsetTop,
                       XmNleftOffset, iOffsetLeft,
                       NULL);
    }
}


void
AmEditorClass::AmAboutCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->about( w, xpCallData);
}


void
AmEditorClass::about( Widget    w,
                      XtPointer xpCallData)
{
    AmPrintResourceInfo( AmGetWidgetShell( _wParent),
                         "aboutMsg");
}


void
AmEditorClass::AmMap25Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap25Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 25); 
}

void
AmEditorClass::AmMap33Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap33Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 33); 
}

void
AmEditorClass::AmMap50Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap50Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 50); 
}

void
AmEditorClass::AmMap100Callback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap100Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 100); 
}

void
AmEditorClass::AmMap125Callback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap125Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 125); 
}

void
AmEditorClass::AmMap150Callback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap150Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 150); 
}

void
AmEditorClass::AmMap175Callback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap175Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 175); 
}

void
AmEditorClass::AmMap200Callback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmMap200Callback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->mapSize( w, xpCallData, 200); 
}

/* ARGSUSED */
void
AmEditorClass::mapSize( Widget    w,
                        XtPointer xpCallData,
                        int       iSize)
{
    ScaleMap( 72 * iSize / 100);
}

/* ARGSUSED */
void
AmEditorClass::AmImageCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmImageCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->image( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::image( Widget    w,
                      XtPointer xpCallData)
{
    AmInitLog();

    AmMakeFileSelectionDialog( AmGetWidgetShell( _wParent), "LoadImage",
                             NULL, 0, TRUE,
                             ( VarargsCallback) AmLoadImageOkCallback,
                             ( XtPointer) this,
                             END_LIST);
}

void
AmEditorClass::AmLoadImageOkCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    AmEditorClass                      *pEditor;
    char                             *szFileName;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmLoadImageOkCallback()\n"));

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;
    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    pEditor = ( AmEditorClass *) xpClientData;

    pEditor->loadImageOk( w, szFileName); 
    XtFree( szFileName);
}

/* ARGSUSED */
void
AmEditorClass::loadImageOk( Widget w,
                            char   *szFileName)
{
    AmImageAnnotClass *pImageAnnot;
    pImageAnnot = new AmImageAnnotClass( _pXWindow, "image",
                                       AmNfilename, szFileName,
                                       NULL);

    if ( pImageAnnot == NULL || pImageAnnot->IsValid() == FALSE) {

        AmPrintResourceError( AmGetWidgetShell( _wParent),
                              "imageAnnotationFailed", szFileName);

    } else {

        _pMotifDisplay->WriteResourceMessage( "addImageInstructionMsg");
        AmSetupAddHandler( this, ( AmScreenObjClass *) pImageAnnot); 

    }
}

void
AmEditorClass::AmAddObjectCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->AmAddObject( w, xpCallData);
}


/* ARGSUSED */
void
AmEditorClass::AmAddObject( Widget    w,
                            XtPointer xpCallData)
{
    LoadObject( NULL);
}


/*
 * Function: LoadObject( pScreenObjParent)
 *
 * Description:
 *   LoadObject is responsible for launching the browser to 
 *   select the object (i.e. a file browser to select the
 *   object from a file, or a database browser to select the
 *   object from a database).
 *
 *   Once the object is selected, CreateObject needs to be
 *   called to create it.
 * 
 *   Once the object is created, AddObject needs to be called
 *   to add the object to the editor.
 *
 *   These AmLast two things are usually done in the OK callback
 *   of the browser.
 */

void
AmEditorClass::LoadObject( AmScreenObjClass *pScreenObjParent)
{
    AmMakeFileSelectionDialog( AmGetWidgetShell( _wParent), "LoadObject",
                             NULL, 0, TRUE,
                             ( VarargsCallback) AmLoadOkCallback,
                             ( XtPointer) this, pScreenObjParent, END_LIST);

    /*
     * Note: The AmLoadOkCallback is responsible for calling CreateObject
     *       and then is also responsible for calling AddObject.
     */
}

void
AmEditorClass::AmLoadOkCallback( Widget           w,
                                 AmEditorClass    *pEditor,
                                 AmScreenObjClass *pScreenObjParent,
                                 XtPointer        xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    char                             *szFileName, szTypeName[ TEXTLEN];

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmLoadOkCallback()\n"));

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;
    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    if ( AmIsMap( szFileName)) {
        pEditor->importMapOk( w, szFileName); 
    } else if ( AmIsHeader( szFileName, szTypeName)) {
        pEditor->loadOk( w, pScreenObjParent, szFileName); 
    }

    XtFree( szFileName);
}

/* ARGSUSED */
void
AmEditorClass::loadOk( Widget           w,
                       AmScreenObjClass *pScreenObjParent,
                       char             *szFileName)
{
    AmScreenObjClass *pScreenObjNew;
    char             **pszHeaderStrings;

    AmInitLog();

    pszHeaderStrings = AmReadHeader( szFileName);
    if ( pszHeaderStrings != NULL) {

        pScreenObjNew = CreateObject( _pXWindow,
                                      pScreenObjParent,
                                      pszHeaderStrings);
	AmFreeStrVec( pszHeaderStrings);

        if ( pScreenObjNew == NULL) {
            AmPrintLogError( AmGetWidgetShell( _wParent),
                           "loadFileFailed", szFileName);
        } else {
            AddObject( pScreenObjNew);
        }

    } else {

        AmPrintLogError( AmGetWidgetShell( _wParent),
                         "loadFileFailed", szFileName);

    }
}

AmScreenObjClass *
AmEditorClass::AmCreateObject( AmWindowClass	  *pWindow,
                               AmScreenObjClass *pScreenObjParent,
                               char           **pszHeaderStrings,
                               XtPointer      xpClientData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;

    return( pEditor->CreateObject( pWindow,
                                   pScreenObjParent,
                                   pszHeaderStrings));
}

AmScreenObjClass *
AmEditorClass::CreateObject( AmWindowClass	  *pWindow,
                           AmScreenObjClass *pScreenObjParent,
                           char           **pszHeaderStrings)
{
    AmScreenObjClass   *pScreenObj;
    AmOverlayClass     *pOverlay;
    AmTagValueRec      pTagValue[ MAXTAGS];
    int              nTagValue;

    nTagValue = AmHeaderStringsTagValueArray( pszHeaderStrings,
                                            pTagValue);

    if ( pScreenObjParent && pScreenObjParent->IsType( AmOVERLAY)) {
        pOverlay = ( AmOverlayClass *) pScreenObjParent;
    } else {
        pOverlay = NULL;
    }

    if ( AmAddLayerFromTagValues( pWindow, pOverlay, pTagValue, nTagValue,
			        "object", AM_PREFS_DIR, &pScreenObj) == AmOK) {
	return( pScreenObj);
    } else {
	return( NULL);
    }
}

void
AmEditorClass::AddObject( AmScreenObjClass *pScreenObjNew)
{
    AmScreenObjClass     *pScreenObjParent;
    AmOverlayClass       *pOverlay;
    AmDataGraphicClass   **vpDataGraphic;
    const AmRectangleRec *pRect;
    int                nDataGraphic, ii;
    AnvilBoolean            fNeedInteractivePlacement;

    if ( pScreenObjNew != NULL) {
        pScreenObjParent = pScreenObjNew->getParent();

        fNeedInteractivePlacement = TRUE;
        if (( pScreenObjParent != NULL) && 
            ( pScreenObjParent->IsType( AmOVERLAY))) {

            pOverlay = ( AmOverlayClass *) pScreenObjParent; 
            pOverlay->GetLayers( &vpDataGraphic, &nDataGraphic);
            for ( ii = 0; ii < nDataGraphic; ii++) {
                if ( vpDataGraphic[ ii] == pScreenObjNew) {
                    fNeedInteractivePlacement = FALSE;
                }
            }

            if ( vpDataGraphic != NULL) {
                FREE( vpDataGraphic);
            }
        } else if (( pScreenObjNew->IsType( AmCOORD)) &&
                   ( !pScreenObjNew->IsType( AmOVERLAY))) {

            pRect = pScreenObjNew->GetRect();

        /* Create the new overlay. */

            pOverlay = new AmOverlayClass( _pXWindow, "Overlay",
                                         AmNx, pRect->x,
                                         AmNy, pRect->y,
                                         AmNwidth, pRect->width,
                                         AmNheight, pRect->height,
                                         AmNpreserveAspect, TRUE,
                                         NULL);

        /* Add the existing object into the overlay and remove it from the
           display. */

            pOverlay->AddLayer(( AmDataGraphicClass *) pScreenObjNew);
            pOverlay->Update();

            pScreenObjNew = pOverlay;
        }

        if ( fNeedInteractivePlacement == TRUE) {
#ifdef PSCOPE
	  extern char* pscope_ifile;
	  extern int pscope_ilisten;
	  extern int pscope_iw, pscope_ih;
	  extern int pscope_crop;

	  if (pscope_ifile || pscope_ilisten) {

	    const AmRectangleRec *pRect;
	    
	    UnselectAll();
	    pRect = pScreenObjNew->GetRect();
	    
	    _pMotifDisplay->AddObject( pScreenObjNew);
	    if (pscope_crop) {
	      pScreenObjNew->SetRectAndMargins( 0, 0, pscope_iw/2, 
						pscope_ih/2, 
						_pMotifDisplay->GetWidth(), 
						_pMotifDisplay->GetHeight());
	    } else {	
	      pScreenObjNew->SetRectAndMargins( 0, 0, pscope_iw, 
						pscope_ih, 
						_pMotifDisplay->GetWidth(), 
						_pMotifDisplay->GetHeight());
	    }
	    SelectObject( pScreenObjNew);
	    _pMotifDisplay->Flush();
	  
	    SetMapChanged( TRUE);

	  } else
#elif defined(LISTEN_SIGNAL)
	  extern char* signal_file;
	  extern int listening_signal;
	  extern int map_width, map_height;

	  if (listening_signal) {

	    const AmRectangleRec *pRect;

	    fprintf(stderr,"listen_signal: resized\n");
	    
	    UnselectAll();
	    pRect = pScreenObjNew->GetRect();
	    
	    _pMotifDisplay->AddObject( pScreenObjNew);
	    pScreenObjNew->SetRectAndMargins( 0, 0, map_width, map_height,
					      _pMotifDisplay->GetWidth(), 
					      _pMotifDisplay->GetHeight());
	    SelectObject( pScreenObjNew);
	    _pMotifDisplay->Flush();
	  
	    SetMapChanged( TRUE);

	  } else
#endif
	      {
            _pMotifDisplay->WriteResourceMessage( "addInstructionMsg");
            AmSetupAddHandler( this, pScreenObjNew);
	  }
        } else {
            _pMotifDisplay->Flush();
        }
    }
}


void
AmEditorClass::AmAddLegendCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmAddLegendCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->addLegend( w, xpCallData);
}


/* ARGSUSED */
void
AmEditorClass::addLegend( Widget    w,
                          XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObjParent;

    if ( ObjectSelected( AmOVERLAY) == TRUE) {

        pSelectObj       = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObjParent = pSelectObj->GetScreenObj();

        createLegend( w, ( XtPointer) pScreenObjParent);
    }
}

void
AmEditorClass::AmCreateLegendCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmCreateLegendCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->createLegend( w, xpCallData);
}


void
AmEditorClass::createLegend( Widget    w,
                             XtPointer xpCallData)
{
    AmScreenObjClass       *pScreenObjParent = ( AmScreenObjClass *) xpCallData;
    AmScreenObjClass       *pScreenObj;
    AmPickLayerDialogClass *pwcDialog;
    char                   sz[ TEXTLEN];

    if ( pScreenObjParent->IsType( AmOVERLAY)) {

        pwcDialog = new AmPickLayerDialogClass( _wForm, "pickLayerDialog",
                                              ( AmOverlayClass *) pScreenObjParent,
                                              ( XtCallbackProc)
                                                  AmCreateLegendCallback,
                                              ( XtPointer) this); 
        pwcDialog->Show();

    } else {

        sprintf( sz, "%s Legend", pScreenObjParent->GetName());

        if ( pScreenObjParent->IsType( AmIMAGE_PLOT) == TRUE) {

            pScreenObj = new AmImageLegendClass( _pXWindow,
                                               ( AmImagePlotClass *)
                                                   pScreenObjParent,
                                               sz,
                                               AmNshowColors, TRUE,
                                               NULL);

        } else if ( pScreenObjParent->IsType( AmLINE_PLOT) == TRUE) {

            pScreenObj = new AmLineLegendClass( _pXWindow,
                                              ( AmLinePlotClass *)
                                                   pScreenObjParent,
                                              sz,
                                              AmNshowColors, TRUE,
                                              AmNshowLineWidths, TRUE,
                                              NULL);

        } else if ( pScreenObjParent->IsType( AmPOLYGON_PLOT) == TRUE) {

            pScreenObj = new AmPolyLegendClass( _pXWindow,
                                              ( AmPolygonPlotClass *)
                                                   pScreenObjParent,
                                              sz,
                                              AmNshowColors, TRUE,
                                              NULL);

        } else if ( pScreenObjParent->IsType( AmSCATTER_PLOT) == TRUE) {

            pScreenObj = new AmScatterLegendClass( _pXWindow,
                                                 ( AmScatterPlotClass *)
                                                      pScreenObjParent,
                                                 sz,
                                                 AmNshowColors, TRUE,
                                                 NULL);

        } else {

            pScreenObj = NULL;
            AmPrintResourceError( AmGetWidgetShell( _wParent),
                                "illegalObjectForLegend");

        }
  
        if ( pScreenObj != NULL) { 
            AddObject( pScreenObj);
        }
    }
}


void
AmEditorClass::AmTickmarksCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmTickmarksCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->tickmarks( w, xpCallData);
}


/* ARGSUSED */
void
AmEditorClass::tickmarks( Widget    w,
                        XtPointer xpCallData)
{
    AmSelectObjClass     *pSelectObj;
    AmScreenObjClass     *pScreenObjParent;
    AmGridClass          *pGrid;
    const AmRectangleRec *pRect;

    if ( ObjectSelected( AmOVERLAY) == TRUE) {

        pSelectObj       = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObjParent = pSelectObj->GetScreenObj();

        pRect = pScreenObjParent->GetRect();

        pGrid = new AmGridClass( _pXWindow,
                               ( AmCoordClass *) pScreenObjParent,
                               "Tickmarks",
                               AmNx, pRect->x,
                               AmNy, pRect->y,
                               AmNwidth, pRect->width,
                               AmNheight, pRect->height,
                               NULL);

        UnselectAll();
        _pMotifDisplay->AddObject( pGrid);
        SelectObject( pGrid);
  
        _pMotifDisplay->Flush();
    }
}


void
AmEditorClass::AmDistanceScaleCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{
    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->distanceScale( w, xpCallData);
}


/* ARGSUSED */
void
AmEditorClass::distanceScale( Widget    w,
                            XtPointer xpCallData)
{
    AmSelectObjClass     *pSelectObj;
    AmScreenObjClass     *pScreenObj;
    AmDistanceScaleClass *pDistanceScale;

    if ( ObjectSelected( AmOVERLAY) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        pDistanceScale = new AmDistanceScaleClass( _pXWindow,
                                                 ( AmCoordClass *) pScreenObj,
                                                 "Distance Scale",
                                                 NULL);

        AddObject( pDistanceScale);
    }
}


void
AmEditorClass::AmAddGridCallback( Widget    w,
                                  XtPointer xpClientData,
                                  XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmGridCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->addGrid( w, xpCallData);
}


/* ARGSUSED */
void
AmEditorClass::addGrid( Widget    w,
                        XtPointer xpCallData)
{
    AmSelectObjClass *pSelectObj;
    AmScreenObjClass *pScreenObj;

    if ( ObjectSelected( AmANY_OBJECT) == TRUE) {

        pSelectObj = ( AmSelectObjClass *) _lstSelectedObjects->data;
        pScreenObj = pSelectObj->GetScreenObj();

        createLegend( w, ( XtPointer) pScreenObj);
    }
}

void
AmEditorClass::SetTitle( const char *szTitle)
{
    char sz[ TEXTLEN];

    if ( _wParent != NULL) {
        sprintf( sz, "%s: %s", _szTitle, szTitle);
        XtVaSetValues( AmGetWidgetShell( _wParent), XmNtitle, sz, NULL);
    }
}


void
AmEditorClass::DoubleClick( AmScreenObjClass *pScreenObj)
{
    AmEditLayerDialogClass *pwcDialog;

    if ( pScreenObj->IsType( AmOVERLAY) == TRUE) {
        pwcDialog = new AmEditLayerDialogClass( _wForm, "pickLayerDialog",
                                              ( AmOverlayClass *) pScreenObj,
                                              _pMotifDisplay);
        pwcDialog->Show();
    } else {
        finalEdit( _wForm, pScreenObj);
    }
}


void
AmEditorClass::AmTextCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmTextCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->text( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::text( Widget    w,
                     XtPointer xpCallData)
{

/* If the font is not set, the previously selected font will be used. */

//    _pwcTextDialog->SetFont( DEFAULT_FONT, DEFAULT_SIZE, DEFAULT_WEIGHT,
//                             DEFAULT_SLANT, 1.0, 1.0);

    _pwcTextDialog->SetText( "");
    _pwcTextDialog->Show();
}

void
AmEditorClass::AmTextOkCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmTextOkCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;

    pEditor->textOk( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::textOk( Widget    w,
                       XtPointer xpCallData)
{
    AmTextAnnotClass *pTextAnnot;
    const char     *szText;
    char           *szName;
    int            iSize;
    AmFontWeightEnum eWeight;
    AmFontSlantEnum  eSlant;
    float          dScaleX;
    float          dScaleY;

    _pwcTextDialog->GetFont( &szName, &iSize, &eWeight, &eSlant,
                             &dScaleX, &dScaleY);
    szText = _pwcTextDialog->GetText();

    XtUnmapWidget( *_pwcTextDialog);

    if ( szText != NULL) {
        pTextAnnot = new AmTextAnnotClass( _pXWindow, 
                                         "Text Annotation",
                                         AmNfont, szName,
                                         AmNfontSize, iSize,
                                         AmNfontWeight, eWeight,
                                         AmNfontSlant, eSlant,
                                         AmNfontXScale, dScaleX,
                                         AmNfontYScale, dScaleY,
                                         AmNlabel, szText,
                                         NULL);

        _pMotifDisplay->WriteResourceMessage( "addInstructionMsg");
        AmSetupAddHandler( this, ( AmScreenObjClass *) pTextAnnot); 
    }

    free( szName);
}


void
AmEditorClass::AmTextCancelCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmTextCancelCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->textCancel( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::textCancel( Widget    w,
                           XtPointer xpCallData)
{
    XtUnmapWidget( *_pwcTextDialog);
}


void
AmEditorClass::AmAlignCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmAlignCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->align( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::align( Widget    w,
                      XtPointer xpCallData)
{

/* If the font is not set, the previously selected font will be used. */

    _pwcAlignDialog->Show();
}

void
AmEditorClass::AmAlignCancelCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmAlignCancelCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->alignCancel( w, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::alignCancel( Widget    w,
                            XtPointer xpCallData)
{
    XtUnmapWidget( *_pwcAlignDialog);
}


void
AmEditorClass::AmRectChangedCallback( int       iID,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmRectChangedCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->rectChanged( iID, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::rectChanged( int       iID,
                            XtPointer xpCallData)
{
    AmScreenObjClass     *pScreenObj = ( AmScreenObjClass *) xpCallData;
    AmSelectObjClass     *pSelectObj;
    AmObjectOutlineClass *pOutline;
    const AmRectangleRec *pRect;
    AmListPtr            lstTemp;

    pRect = pScreenObj->GetRect(); 

    for ( lstTemp = _lstSelectedObjects;
          lstTemp != NULL;
          lstTemp = lstTemp->next) { 

        pSelectObj = ( AmSelectObjClass *) lstTemp->data;
        if ( pSelectObj->GetScreenObj() == pScreenObj) {
            pOutline = ( AmObjectOutlineClass *) pSelectObj->GetOutline();
            pOutline->SetRectangle( pRect->x, pRect->y,
                                    pRect->width, pRect->height);
            break;
        }
    }
}


void
AmEditorClass::AmObjectDeletedCallback( int       iID,
                                        XtPointer xpClientData,
                                        XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmObjectDeletedCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->objectDeleted( iID, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::objectDeleted( int       iID,
                              XtPointer xpCallData)
{
    AmScreenObjClass     *pScreenObj = ( AmScreenObjClass *) xpCallData;
 
// If the object isn't selected the UnselectObject function does
//   nothing.
 
    UnselectObject( pScreenObj);

}



void
AmEditorClass::AmImportMapCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmImportMapCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->importMap( w, xpCallData);
}


/* ARGSUSED */
void
AmEditorClass::importMap( Widget    w,
                          XtPointer xpCallData)
{

    AmMakeFileSelectionDialog( AmGetWidgetShell( _wParent), "LoadMap",
                             NULL, 0, TRUE,
                             ( VarargsCallback) AmImportMapOkCallback,
                             ( XtPointer) this, END_LIST);
}

void
AmEditorClass::AmImportMapOkCallback( Widget         w,
                                      AmEditorClass    *pEditor,
                                      XtPointer      xpCallData)
{
    XmFileSelectionBoxCallbackStruct *cb;
    char                             *szFileName;

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmImportOkMapCallback()\n"));

    cb = ( XmFileSelectionBoxCallbackStruct *) xpCallData;
    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &szFileName); 

    pEditor->importMapOk( w, szFileName); 
}

/* ARGSUSED */
void
AmEditorClass::importMapOk( Widget         w,
                            char           *szFileName)
{
    AmListPtr lstObjects;

    AmInitLog();

    lstObjects = LoadObjectsFromFile( szFileName);

    if ( lstObjects != NULL) {

        _pMotifDisplay->WriteResourceMessage( "addInstructionMsg");
        AmSetupAddObjectsHandler( this, lstObjects);

    } else {

        _pMotifDisplay->WriteResourceMessage( "cantLoadMapMsg", szFileName);
        AmPrintLogError( AmGetWidgetShell( _wParent), "errorReadHeader",
                         szFileName);

    }

}


void
AmEditorClass::AmDisplayChangedCallback( int       iID,
                                         XtPointer xpClientData,
                                         XtPointer xpCallData)
{

// Print a debug string, used for regression testing.

    AN_DEBUG( 99, ( "AmEditorClass::AmDisplayChangedCallback()\n"));

    AmEditorClass *pEditor = ( AmEditorClass *) xpClientData;
    pEditor->displayChanged( iID, xpCallData);
}

/* ARGSUSED */
void
AmEditorClass::displayChanged( int       iID,
                               XtPointer xpCallData)
{
    AmMotifDisplayClass *pMotifDisplay = ( AmMotifDisplayClass *) xpCallData;

    if ( pMotifDisplay == _pMotifDisplay) {
        _fMapChanged = TRUE;

    }
}
