#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <Xm/Xm.h>

#include <X11/cursorfont.h>

#include <anvil/xdisplay.h>
#include <anvil/motif.h>
#include <anvil/va_cback.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/userevents.h>
#include <anvil/events.h>
#include <anvil/envdefs.h>
#include <anvil/fonts.h>
#include <anvil/resource.h>
#include <anvil/list.h>

#include <editor/EditorClass.h>
#include <editor/EditorToolBarClassP.h>

const int AM_SELECTION_SET_CHANGED = 8001;

/* ARGSUSED */
void
AmEditorToolBarClass::AmSelectionChangedCallback( int  iEventID,
                                              void *pCallData,
                                              void *pClientData)
{
    AmEditorToolBarClass *etbc = ( AmEditorToolBarClass *) pCallData;
    AmListPtr            lstSelected, lstIButtons, lstISelected;
    AmSelectEnum         eSelect;
    AmObjectTypeEnum     eType;
    AmSelectObjClass     *pSelectObj;
    AnvilBoolean            fSensitize, fIsSensitive;
    int                iLength;
    AmButtonPtr          pButton;

    lstSelected = ( AmListPtr) pClientData;
    iLength = AmLengthLst( lstSelected);

    for ( lstIButtons = etbc->_lstButtons; 
          lstIButtons != NULL;
          lstIButtons = lstIButtons->next) {

        pButton = ( AmButtonPtr) lstIButtons->data;

        eSelect = pButton->eSelect;
        eType = pButton->eType;

        fSensitize = FALSE;

        if ( eSelect == AmSELECT_CUSTOM) {
            continue;
        }

        if (( eSelect == AmSELECT_ANY) ||
            ( eSelect == AmSELECT_NONE && iLength == 0) ||
            ( eSelect == AmSELECT_ONE && iLength == 1) ||
            ( eSelect == AmSELECT_ONE_OR_MORE && iLength >= 1)) {

	    fSensitize = TRUE;

	    if ( eType != AmANY_OBJECT) {
		for ( lstISelected = lstSelected;
                      lstISelected != NULL; 
                      lstISelected = lstISelected->next) {
		    
                    pSelectObj = ( AmSelectObjClass *) lstISelected->data;
		    
                    if ( pSelectObj->GetScreenObj()->IsType( eType)) {
                        continue;
                    } else {
                        fSensitize = FALSE;
                        break;
                    }
                }
            }
        }
	
        XtVaGetValues( pButton->wButton, XmNsensitive, &fIsSensitive, NULL);

        if ( fIsSensitive != fSensitize) {
            XtSetSensitive( pButton->wButton, fSensitize);
        }
    }
}

/*
 * Function: AmEditorToolBarClass::AmEditorToolBarClass()
 *
 * Description:
 *   Default constructor. 
 */

AmEditorToolBarClass::AmEditorToolBarClass()
    : AmToolBarClass()
{
}


/*
 * Function: AmEditorToolBarClass::
 *               AmEditorToolBarClass( wParent, szName, wArea, fVertical)
 *
 * Parameters:
 *   Widget     wParent   : Parent widget for the tool bar widget.
 *   const char *szName   : Name of the tool bar widget.
 * 
 * Description:
 *    
 */

AmEditorToolBarClass::AmEditorToolBarClass( Widget     wParent,
			                const char *szName)
    : AmToolBarClass( wParent, szName)
{
    AmAddCallbackWidget( AM_PKG_ID, AM_SELECTION_SET_CHANGED, _wBase,
                       ( callbackProc)
		       & AmEditorToolBarClass::AmSelectionChangedCallback,
                       ( XtPointer) this);
}

/*
 * Function: AmEditorToolBarClass::
 *               AmEditorToolBarClass( pwcParent, szName, wArea, fVertical)
 *
 * Parameters:
 *   Widget     pwcParent : Parent widget for the tool bar widget.
 *   const char *szName   : Name of the tool bar widget.
 * 
 * Description:
 *    
 */

AmEditorToolBarClass::AmEditorToolBarClass( AmBaseWidgetClass *pwcParent,
			                const char      *szName)
    : AmToolBarClass( pwcParent, szName)
{
    AmAddCallbackWidget( AM_PKG_ID, AM_SELECTION_SET_CHANGED, _wBase,
                       ( callbackProc)
		       & AmEditorToolBarClass::AmSelectionChangedCallback,
                       ( XtPointer) this);
}

AmEditorToolBarClass::~AmEditorToolBarClass()
{

    AN_DEBUG( 99, ( "AmEditorToolBarClass::~AmEditorToolBarClass()\n"));

    if ( _lstButtons != NULL) {
        AmFreeLst( _lstButtons, ( AmItemFn) free);
        _lstButtons = NULL;
    }
}

AmButtonClass *
AmEditorToolBarClass::AddButton( const char     *szName,
                               XtCallbackProc xcp,
                               XtPointer      xp)
{
    return( AddButton( szName, xcp, xp, AmSELECT_ANY, AmANY_OBJECT));
}


AmButtonClass *
AmEditorToolBarClass::AddButton( const char     *szName,
                               XtCallbackProc xcp,
                               XtPointer      xp, 
                               AmSelectEnum     eSelect,
                               AmObjectTypeEnum eType)
{
    AmButtonClass *pwcButton;
    AmButtonPtr   pButton;

    if ( _pwcPane != NULL) {  

        pwcButton = new AmButtonClass( _pwcPane, szName, xcp, xp); 
        pButton = ( AmButtonPtr) malloc( sizeof( AmButtonRec));
        if ( pButton == NULL) {

            AN_DEBUG( 0, ( "AmEditorToolBarClass::AddButton() : memory allocation failed.\n"));

        } else {

            pButton->wButton = *pwcButton;
            pButton->eSelect = eSelect;
            pButton->eType = eType;

            _lstButtons = AmInsertNodeLst( pButton, _lstButtons);

        }

        return( pwcButton);
    }

    return( NULL);
}
