#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <editor/handlers.h>

AmListPtr
AmGenerateMoveOutlines( AmEditorClass *pEditorClass)
{
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    AmListPtr            lstTemp, lstOutlines;
    AmSelectObjClass     *pSelectObj;
    AmMoveOutlineClass   *pOutline;
    const AmRectangleRec *pRec;

/* Generate the outlines and add them to the display. */

    lstOutlines = NULL;

    for ( lstTemp = pEditorClass->GetSelectedObjects();
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pSelectObj = ( AmSelectObjClass *) lstTemp->data;
        pRec = pSelectObj->GetScreenObj()->GetRect();
        pOutline = new AmMoveOutlineClass( pMotifDisplay,
                                         pRec->x, pRec->y,
                                         pRec->width, pRec->height);
        pMotifDisplay->AddOutline( pOutline);
        lstOutlines = AmInsertNodeAtEndLst( pOutline, lstOutlines);
    }

    return( lstOutlines);
}


AmListPtr
AmGenerateResizeOutlines( AmEditorClass *pEditorClass)
{
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    AmListPtr            lstTemp, lstOutlines;
    AmSelectObjClass     *pSelectObj;
    AmResizeOutlineClass *pOutline;
    const AmRectangleRec *pRec;

/* Generate the outlines and add them to the display. */

    lstOutlines = NULL;

    for ( lstTemp = pEditorClass->GetSelectedObjects();
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pSelectObj = ( AmSelectObjClass *) lstTemp->data;
        pRec = pSelectObj->GetScreenObj()->GetRect();
        pOutline = new AmResizeOutlineClass( pMotifDisplay,
                                           pRec->x, pRec->y,
                                           pRec->width, pRec->height);
        pMotifDisplay->AddOutline( pOutline);
        lstOutlines = AmInsertNodeAtEndLst( pOutline, lstOutlines);
    }

    return( lstOutlines);
}


void
AmRemoveOutlines( AmMotifDisplayClass *pMotifDisplay,
                AmListPtr           lstOutlines)
{
    AmListPtr      lstTemp, lstNext;
    AmOutlineClass *pOutline;

    /* Generate the outlines and add them to the display. */

    lstTemp = lstOutlines;
    while ( lstTemp != NULL) {
        lstNext = lstTemp->next;
        pOutline = ( AmOutlineClass *) lstTemp->data;
        pMotifDisplay->RemoveOutline( pOutline);
        free( lstTemp);
        lstTemp = lstNext;
    }
}


void
AmHideOutlines( AmMotifDisplayClass *pMotifDisplay,
              AmListPtr           lstOutlines)
{
    AmListPtr      lstTemp;
    AmOutlineClass *pOutline;

    /* Show the outlines. */

    for ( lstTemp = lstOutlines;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pOutline = ( AmOutlineClass *) lstTemp->data;
        pMotifDisplay->HideOutline( pOutline);
    }
}


void
AmShowOutlines( AmMotifDisplayClass *pMotifDisplay,
              AmListPtr           lstOutlines)
{
    AmListPtr      lstTemp;
    AmOutlineClass *pOutline;

    /* Hide the outlines. */ 

    for ( lstTemp = lstOutlines;
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pOutline = ( AmOutlineClass *) lstTemp->data;
        pMotifDisplay->ShowOutline( pOutline);
    }
}
