#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/DrawingA.h>

#include <anvil/macros.h>
#include <anvil/strlib.h>
#include <anvil/events.h>
#include <anvil/va_cback.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/textcheck.h>
#include <anvil/motif.h>

#include <editor/LegendDialogClass.h>

/*
 * Function: AmLegendDialogClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmLegendDialogClass::initializeValues()
{
    _szTitle = strdup( "Title");
    _szTitleFont = strdup( "adobe-helvetica");
    _iTitleSize = 16;
    _eTitleWeight = AmWEIGHT_BOLD;
    _eTitleSlant = AmSLANT_ROMAN;
    _dTitleXScale = 1.0;
    _dTitleYScale = 1.0;

    _szLabelFont = strdup( "adobe-helvetica");
    _iLabelSize = 10;
    _eLabelWeight = AmWEIGHT_MEDIUM;
    _eLabelSlant = AmSLANT_ROMAN;
    _dLabelXScale = 1.0;
    _dLabelYScale = 1.0;
}


/*
 * Function: AmLegendDialogClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets for the legend editing dialog.
 */

void
AmLegendDialogClass::createInterface()
{
    AmFrameClass     *pwcFrame;
    AmFormClass      *pwcForm, *pwcLabelForm, *pwcFontForm, *pwcOffsetForm,
                   *pwcWidthForm, *pwcHeightForm;
    AmLabelClass     *pwcLabel;
    AmRadioPaneClass *pwcRadioPane;
    AmXDisplayClass  *pXDisplay;
    AmXWindowClass   *pXWindow;

/* Create the poor man's version of a folder.  That is create buttons
   that manage and unmanage forms below them. */

    _pwcTitleButton = new AmButtonClass( _wForm, "titleButton",
                                       ( XtCallbackProc) AmActivateCallback,
                                       this);
    _pwcTitleButton->AttachLeft();
    _pwcTitleButton->AttachTop();

    _pwcLabelsButton = new AmButtonClass( _wForm, "labelsButton",
                                        ( XtCallbackProc) AmActivateCallback,
                                        this);
    _pwcLabelsButton->AttachLeft( _pwcTitleButton);
    _pwcLabelsButton->AttachTop();

    _pwcItemsButton = new AmButtonClass( _wForm, "itemsButton",
                                       ( XtCallbackProc) AmActivateCallback,
                                       this);
    _pwcItemsButton->AttachLeft( _pwcLabelsButton);
    _pwcItemsButton->AttachTop();
    
    _pwcTitleFrame = new AmFrameClass( _wForm, "titleFrame", TRUE);
    _pwcTitleFrame->AttachLeft();
    _pwcTitleFrame->AttachTop( _pwcTitleButton);

/* Create the title form (fake folder). */

    pwcForm = new AmFormClass( _pwcTitleFrame, "titleForm");

    pwcLabelForm = new AmFormClass( pwcForm, "labelForm");
    pwcLabelForm->AttachLeft();
    pwcLabelForm->AttachTop();
    pwcLabelForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcLabelForm, "titleLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcTitleText = new AmTextFieldClass( pwcLabelForm, "titleText");
    _pwcTitleText->AttachLeft( pwcLabel);
    _pwcTitleText->AttachTop();
    _pwcTitleText->AttachRight();
    _pwcTitleText->AttachBottom();

    XtAddCallback( *_pwcTitleText, XmNactivateCallback,
                   ( XtCallbackProc) AmTextCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcTitleText, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmTextCallback,
                   ( XtPointer) this);

    pwcFontForm = new AmFormClass( pwcForm, "fontForm");
    pwcFontForm->AttachLeft();
    pwcFontForm->AttachTop( pwcLabelForm);
    pwcFontForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcFontForm, "fontLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcTitleFont = new AmButtonClass( pwcFontForm, "titleFont",
                                     ( XtCallbackProc) AmFontCallback,
                                     ( XtPointer) this);
    _pwcTitleFont->AttachLeft( pwcLabel);
    _pwcTitleFont->AttachTop();
    _pwcTitleFont->AttachRight();
    _pwcTitleFont->AttachBottom();

    pwcOffsetForm = new AmFormClass( pwcForm, "offsetForm");
    pwcOffsetForm->AttachLeft();
    pwcOffsetForm->AttachTop( pwcFontForm);
    pwcOffsetForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcOffsetForm, "offsetLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcTitleOffset = new AmTextFieldClass( pwcOffsetForm, "titleOffset");
    _pwcTitleOffset->AttachTop();
    _pwcTitleOffset->AttachRight();
    _pwcTitleOffset->AttachBottom();

    XtAddCallback( *_pwcTitleOffset, XmNactivateCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcTitleOffset, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);

    pwcFrame = new AmFrameClass( pwcForm, "orientationFrame", TRUE);
    pwcFrame->AttachLeft();
    pwcFrame->AttachTop( pwcOffsetForm);
    pwcFrame->AttachRight();
    pwcFrame->AttachBottom();

    pwcRadioPane = new AmRadioPaneClass( pwcFrame, "orientationPane");
    XtVaSetValues( *pwcRadioPane, XmNorientation, XmHORIZONTAL, NULL);

    _pwcTitleVert = pwcRadioPane->AddButton( "verticalBtn",
                                             ( XtCallbackProc) AmToggleCallback,
                                             ( XtPointer) this);
    _pwcTitleHoriz = pwcRadioPane->AddButton( "horizontalBtn",
                                              ( XtCallbackProc) NULL,
                                              ( XtPointer) this);

/* Create the labels form (fake folder). */

    _pwcLabelsFrame = new AmFrameClass( _wForm, "labelsFrame", TRUE);
    _pwcLabelsFrame->AttachLeft();
    _pwcLabelsFrame->AttachTop( _pwcTitleButton);

    XtVaSetValues( *_pwcLabelsFrame, XmNmappedWhenManaged, FALSE, NULL);

    pwcForm = new AmFormClass( _pwcLabelsFrame, "labelsForm");

    pwcFontForm = new AmFormClass( pwcForm, "fontForm");
    pwcFontForm->AttachLeft();
    pwcFontForm->AttachTop();
    pwcFontForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcFontForm, "fontLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcLabelFont = new AmButtonClass( pwcFontForm, "labelFont",
                                     ( XtCallbackProc) AmFontCallback,
                                     ( XtPointer) this);
    _pwcLabelFont->AttachLeft( pwcLabel);
    _pwcLabelFont->AttachTop();
    _pwcLabelFont->AttachRight();
    _pwcLabelFont->AttachBottom();

    pwcOffsetForm = new AmFormClass( pwcForm, "offsetForm");
    pwcOffsetForm->AttachLeft();
    pwcOffsetForm->AttachTop( pwcFontForm);
    pwcOffsetForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcOffsetForm, "offsetLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcLabelOffset = new AmTextFieldClass( pwcOffsetForm, "labelOffset");
    _pwcLabelOffset->AttachTop();
    _pwcLabelOffset->AttachRight();
    _pwcLabelOffset->AttachBottom();

    XtAddCallback( *_pwcLabelOffset, XmNactivateCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcLabelOffset, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);

/* Create the items form (fake folder). */

    _pwcItemsFrame = new AmFrameClass( _wForm, "itemsFrame", TRUE);
    _pwcItemsFrame->AttachLeft();
    _pwcItemsFrame->AttachTop( _pwcTitleButton);

    XtVaSetValues( *_pwcItemsFrame, XmNmappedWhenManaged, FALSE, NULL);

    pwcForm = new AmFormClass( _pwcItemsFrame, "itemsForm");

    pwcWidthForm = new AmFormClass( pwcForm, "widthForm");
    pwcWidthForm->AttachLeft();
    pwcWidthForm->AttachTop();
    pwcWidthForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcWidthForm, "widthLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcItemWidth = new AmTextFieldClass( pwcWidthForm, "itemWidth");
    _pwcItemWidth->AttachTop();
    _pwcItemWidth->AttachRight();
    _pwcItemWidth->AttachBottom();

    XtAddCallback( *_pwcItemWidth, XmNactivateCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcItemWidth, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);

    pwcHeightForm = new AmFormClass( pwcForm, "heightForm");
    pwcHeightForm->AttachLeft();
    pwcHeightForm->AttachTop( pwcWidthForm);
    pwcHeightForm->AttachRight();

    pwcLabel = new AmLabelClass( pwcHeightForm, "heightLabel");
    pwcLabel->AttachLeft();
    pwcLabel->AttachTop();
    pwcLabel->AttachBottom();

    _pwcItemHeight = new AmTextFieldClass( pwcHeightForm, "itemHeight");
    _pwcItemHeight->AttachTop();
    _pwcItemHeight->AttachRight();
    _pwcItemHeight->AttachBottom();

    XtAddCallback( *_pwcItemHeight, XmNactivateCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcItemHeight, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmOffsetCallback,
                   ( XtPointer) this);

/* Create the main item form (fake folder). */

    _pwcMainItemFrame = new AmFrameClass( _wForm, "mainItemFrame", TRUE);
    _pwcMainItemFrame->AttachLeft();
    _pwcMainItemFrame->AttachTop();

    XtVaSetValues( *_pwcMainItemFrame, XmNmappedWhenManaged, FALSE, NULL);

    _pwcLegendFrame = new AmFrameClass( _wForm, "legendFrame", FALSE);
    _pwcLegendFrame->AttachTop();
    _pwcLegendFrame->AttachLeft( _pwcTitleFrame);
    _pwcLegendFrame->AttachRight();
    _pwcLegendFrame->AttachBottom();

    _pwcArea = new AmDrawingAreaClass( _pwcLegendFrame, "legendArea");


    pXDisplay = new AmXDisplayClass( *_pwcArea);
    pXWindow = new AmXWindowClass( pXDisplay, ( Widget)( *_pwcArea));
    _pMotifDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow, FALSE);

    _pImageLegend = new AmImageLegendClass( pXWindow, NULL, "Image Legend",
                                          AmNlegendType, AmLEGEND_CONTINUOUS,
                                          AmNnumColors, 200,
                                          AmNcolormap, "Prism",
                                          AmNmin, 0.0,
                                          AmNmax, 100.0,
                                          AmNcolorsBinvec, NULL,
                                          AmNcolorsTable, NULL,
                                          AmNcolorsStrings, NULL,
                                          AmNitemSpacing, 5,
                                          AmNfontSize, 10,
                                          AmNshowTitle, TRUE,
                                          AmNtitle, "Title",
                                          AmNtitleFontSize, 16,
                                          AmNtitleFontWeight, AmWEIGHT_BOLD,
                                          AmNtitleOffset, 5,
                                          AmNeditable, TRUE,
                                          AmNitemWidth, 20,
                                          AmNitemHeight, 20,
                                          AmNmainItemHeight, 200,
                                          AmNx, 20,
                                          AmNy, 20,
                                          AmNopaque, FALSE,
                                          NULL);

    _pMotifDisplay->AddObject( _pImageLegend);
    _pMotifDisplay->Flush();

    AddButton( "okBtn", NULL, NULL);
    AddButton( "cancelBtn", 
               ( XtCallbackProc) AmPopdownCback,
               _wForm);
    AddButton( "helpBtn", NULL, NULL);

    _pwcTitleFontDialog = new AmFontDialogClass( _wForm, "fontDialog");
    XtVaSetValues( *_pwcTitleFontDialog, XmNdeleteResponse, XmUNMAP, NULL);

    _pwcTitleFontDialog->AddButton( "applyBtn", 
                                    ( XtCallbackProc) AmFontApplyCallback,
                                    ( XtPointer) this);
    _pwcTitleFontDialog->AddButton( "cancelBtn",
                                    ( XtCallbackProc) AmFontCancelCallback,
                                    ( XtPointer) this);
    _pwcTitleFontDialog->AddButton( "helpBtn", NULL, NULL);

    _pwcLabelFontDialog = new AmFontDialogClass( _wForm, "fontDialog");
    XtVaSetValues( *_pwcLabelFontDialog, XmNdeleteResponse, XmUNMAP, NULL);

    _pwcLabelFontDialog->AddButton( "applyBtn", 
                                    ( XtCallbackProc) AmLabelFontApplyCallback,
                                    ( XtPointer) this);
    _pwcLabelFontDialog->AddButton( "cancelBtn",
                                    ( XtCallbackProc) AmFontCancelCallback,
                                    ( XtPointer) this);
    _pwcLabelFontDialog->AddButton( "helpBtn", NULL, NULL);
}


/*
 * Function: AmLegendDialogClass::AmLegendDialogClass()
 *
 * Purpose:
 *   Default constructor.
 */

AmLegendDialogClass::AmLegendDialogClass()
    : AmDialogClass()
{
}


/*
 * Functions: AmLegendDialogClass::AmLegendDialogClass( wParent, szName)
 *            AmLegendDialogClass::AmLegendDialogClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmLegendDialogClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters: 
 *   Widget          wParent    : The parent of the dialog.
 *   BaseWidgetclass *bwcParent : The "wrapped" parent of the dialog.
 *   const char      *szName    : The name of the dialog.
 */

AmLegendDialogClass::AmLegendDialogClass( Widget      wParent,
                                      const char  *szName)
    : AmDialogClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


/* See above comment. */

AmLegendDialogClass::AmLegendDialogClass( AmBaseWidgetClass *bwcParent,
                                      const char      *szName)
    : AmDialogClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}


/*
 * Function: AmLegendDialogClass::~AmLegendDialogClass()
 *
 * Purpose:
 *   Destructor for the AmLegendDialogClass.
 */

AmLegendDialogClass::~AmLegendDialogClass()
{
    if ( _szTitle != NULL) {
        free( _szTitle);
    }

    if ( _szTitleFont != NULL) {
        free( _szTitleFont);
    }

    if ( _szLabelFont != NULL) {
        free( _szLabelFont);
    }
}


void
AmLegendDialogClass::AmTextCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->text( w, xpCallData);
}

void
AmLegendDialogClass::text( Widget    w,
                         XtPointer xpCallData)
{
    char *sz;

    XtVaGetValues( w, XmNvalue, &sz, NULL);

    _pwcTitleFontDialog->SetText( sz);

    _pImageLegend->SetVaFields( AmNtitle, sz, NULL);
    _pMotifDisplay->Flush();

    free( sz);
}


void
AmLegendDialogClass::AmToggleCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->toggle( w, xpCallData);
}

void
AmLegendDialogClass::toggle( Widget    w,
                           XtPointer xpCallData)
{
    if ( w == (Widget) *_pwcTitleVert) {
        _pImageLegend->SetVaFields( AmNtitleVertical,
                                    _pwcTitleVert->GetState(),
                                    NULL);
        _pMotifDisplay->Flush();
    }
}


void
AmLegendDialogClass::AmActivateCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->activate( w, xpCallData);
}

void
AmLegendDialogClass::activate( Widget    w,
                            XtPointer xpCallData)
{
    if ( w == (Widget) *_pwcTitleButton) {
        XtMapWidget( *_pwcTitleFrame);
        XtUnmapWidget( *_pwcLabelsFrame);
        XtUnmapWidget( *_pwcItemsFrame);
    } else if ( w == (Widget) *_pwcLabelsButton) {
        XtUnmapWidget( *_pwcTitleFrame);
        XtMapWidget( *_pwcLabelsFrame);
        XtUnmapWidget( *_pwcItemsFrame);
    } else if ( w == (Widget) *_pwcItemsButton) {
        XtUnmapWidget( *_pwcTitleFrame);
        XtUnmapWidget( *_pwcLabelsFrame);
        XtMapWidget( *_pwcItemsFrame);
    }
}


void
AmLegendDialogClass::AmFontCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->font( w, xpCallData);
}


void
AmLegendDialogClass::font( Widget    w,
                         XtPointer xpCallData)
{

    if ( w == (Widget) *_pwcTitleFont) {

        _pwcTitleFontDialog->SetFont( _szTitleFont, _iTitleSize,
                                      _eTitleWeight, _eTitleSlant,
                                      _dTitleXScale, _dTitleYScale);

        _pwcTitleFontDialog->Show();

    } else if ( w == (Widget) *_pwcLabelFont) {

        _pwcLabelFontDialog->SetFont( _szLabelFont, _iLabelSize,
                                      _eLabelWeight, _eLabelSlant,
                                      _dLabelXScale, _dLabelYScale);

        _pwcLabelFontDialog->Show();
    }
}


void
AmLegendDialogClass::AmFontApplyCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->fontApply( w, xpCallData);
}


void
AmLegendDialogClass::fontApply( Widget    w,
                              XtPointer xpCallData)
{
    XmString xsz;

    if ( _szTitleFont != NULL) {
        free( _szTitleFont);
        _szTitleFont = NULL;
    }

    _pwcTitleFontDialog->GetFont( &_szTitleFont, &_iTitleSize,
                                  &_eTitleWeight, &_eTitleSlant,
                                  &_dTitleXScale, &_dTitleYScale);

    xsz = XmStringCreateLocalized( _szTitleFont);
    XtVaSetValues( *_pwcTitleFont, XmNlabelString, xsz, NULL);
    XmStringFree( xsz);

    _pImageLegend->SetVaFields( AmNtitleFont, _szTitleFont,
                                AmNtitleFontSize, _iTitleSize,
                                AmNtitleFontWeight, _eTitleWeight,
                                AmNtitleFontSlant, _eTitleSlant,
                                AmNtitleFontXScale, _dTitleXScale,
                                AmNtitleFontYScale, _dTitleYScale,
                                NULL);
    _pMotifDisplay->Flush();
}


void
AmLegendDialogClass::AmFontCancelCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->fontCancel( w, xpCallData);
}


void
AmLegendDialogClass::fontCancel( Widget    w,
                               XtPointer xpCallData)
{
    XtUnmapWidget( AmGetWidgetShell ( w));
}

void
AmLegendDialogClass::AmLabelFontApplyCallback( Widget    w,
                                           XtPointer xpClientData,
                                           XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->labelFontApply( w, xpCallData);
}


void
AmLegendDialogClass::labelFontApply( Widget    w,
                                   XtPointer xpCallData)
{
    XmString xsz;

    if ( _szLabelFont != NULL) {
        free( _szLabelFont);
        _szLabelFont = NULL;
    }

    _pwcLabelFontDialog->GetFont( &_szLabelFont, &_iLabelSize,
                                  &_eLabelWeight, &_eLabelSlant,
                                  &_dLabelXScale, &_dLabelYScale);

    xsz = XmStringCreateLocalized( _szLabelFont);
    XtVaSetValues( *_pwcLabelFont, XmNlabelString, xsz, NULL);
    XmStringFree( xsz);

    _pImageLegend->SetVaFields( AmNfont, _szLabelFont,
                                AmNfontSize, _iLabelSize,
                                AmNfontWeight, _eLabelWeight,
                                AmNfontSlant, _eLabelSlant,
                                AmNfontXScale, _dLabelXScale,
                                AmNfontYScale, _dLabelYScale,
                                NULL);
    _pMotifDisplay->Flush();
}


void
AmLegendDialogClass::AmOffsetCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData)
{
    AmLegendDialogClass *pLegendDialog = ( AmLegendDialogClass *) xpClientData;
    pLegendDialog->offset( w, xpCallData);
}

void
AmLegendDialogClass::offset( Widget    w,
                           XtPointer xpCallData)
{
    char *sz;
    int  iValue;

    XtVaGetValues( w, XmNvalue, &sz, NULL);
    iValue = atol( sz);
    free( sz);

    if ( w == (Widget) *_pwcTitleOffset) {
        _pImageLegend->SetVaFields( AmNtitleOffset, iValue, NULL);
    }

    _pMotifDisplay->Flush();
}
