#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/cursorfont.h>
#include <anvil/state_hand.h>
#include <editor/handlers.h>

const int AmACTION_MOVE_PRIORITY = 1000;

typedef struct moveHandlerArgs_ {
    AmEditorClass      *pEditorClass;
    AmMoveOutlineClass *pOutline;
    int              xStart;
    int              yStart;
    int              x;
    int              y;
    int              iWidth;
    int              iHeight;
    int              xOffset;
    int              yOffset;
} AmMoveHandlerArgsRec, *AmMoveHandlerArgsPtr;

void
AmMoveHandler( Widget             widget,
             AmMoveHandlerArgsPtr pHandlerArgs,
             XEvent             *pxEvent,
             AnvilBoolean            *);

/************************************************************
 *
 * Move Handler stuff:
 *
 ************************************************************/


AmMoveHandlerArgsPtr
AmMakeMoveHandlerArgs( AmEditorClass *pEditorClass,
                     int         x,
                     int         y,
                     int         iWidth,
                     int         iHeight,
                     int         xOffset,
                     int         yOffset)
{
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay();
    AmMoveHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = ( AmMoveHandlerArgsPtr) malloc( sizeof( AmMoveHandlerArgsRec));

    pHandlerArgs->pEditorClass = pEditorClass;
    pHandlerArgs->xStart = x;
    pHandlerArgs->yStart = y;
    pHandlerArgs->x = x;
    pHandlerArgs->y = y;
    pHandlerArgs->iWidth = iWidth;
    pHandlerArgs->iHeight = iHeight;
    pHandlerArgs->xOffset = xOffset;
    pHandlerArgs->yOffset = yOffset;

    pHandlerArgs->pOutline = new AmMoveOutlineClass( pMotifDisplay,
                                                   x, y, iWidth, iHeight);
    pMotifDisplay->AddOutline( pHandlerArgs->pOutline);

    return( pHandlerArgs);
}


void
AmRemoveMoveHandler( AmMoveHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay();

    pMotifDisplay->RemoveOutline( pHandlerArgs->pOutline);
    pMotifDisplay->UnmapTravelCursor();

    free( pHandlerArgs);
}


void
AmSuspendMoveHandler( AmMoveHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay();

    pMotifDisplay->HideOutline( pHandlerArgs->pOutline);
    pMotifDisplay->UnmapTravelCursor();
}


void
AmRestartMoveHandler( AmMoveHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay();

    pMotifDisplay->ShowOutline( pHandlerArgs->pOutline);
    pMotifDisplay->MapTravelCursor();
    pMotifDisplay->WriteResourceMessage( "AmRestartMoveHandler");
}


void
AmSetupMoveHandler( AmEditorClass *pEditorClass,
                  int         x,
                  int         y)
{
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    Widget             wArea = pMotifDisplay->GetArea();
    AmMoveHandlerArgsPtr pHandlerArgs;
    AmRectangleRec       rect;

    pEditorClass->GetSelectedObjectsExtents( &rect);
    pHandlerArgs = AmMakeMoveHandlerArgs( pEditorClass, rect.x, rect.y,
                                        rect.width, rect.height,
                                        x - rect.x, y - rect.y);

/* Add the actual handler. */

    AmAddHandler( NULL, wArea,
                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                ( XtEventHandler) AmMoveHandler,
                pHandlerArgs, AmACTION_MOVE_PRIORITY, 
                ( AmStateHandlerFn) AmRemoveMoveHandler,
                ( AmStateHandlerFn) AmSuspendMoveHandler,
                ( AmStateHandlerFn) AmRestartMoveHandler);

    pMotifDisplay->WriteResourceMessage( "moveMsg");
    pMotifDisplay->MapTravelCursor();
    pMotifDisplay->WriteResourceTravelCursor( "movingMsg", rect.x, rect.y);
}

/************************************************************
 *
 * Move Handler:
 *
 ************************************************************/

void
AmMoveHandler( Widget             widget,
             AmMoveHandlerArgsPtr pHandlerArgs,
             XEvent             *pxEvent,
             AnvilBoolean            *)
{
    AmEditorClass          *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass    *pMotifDisplay = pEditorClass->GetMotifDisplay();
    XButtonEvent         *pxButtonEvent;
    XMotionEvent         *pxMotionEvent;
    int                  x = pHandlerArgs->x,
                         y = pHandlerArgs->y,
                         w = pHandlerArgs->iWidth,
                         h = pHandlerArgs->iHeight,
                         xStart = pHandlerArgs->xStart,
                         yStart = pHandlerArgs->yStart,
                         iHeightArea, iWidthArea;
    AnvilBoolean              fSnapToGrid = pEditorClass->GetSnapToGrid();

    iWidthArea = pMotifDisplay->GetWidth();
    iHeightArea = pMotifDisplay->GetHeight();

/* Figure out the extends of the drawing area. */

    if ( pxEvent->type == ButtonPress) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            pMotifDisplay->WriteResourceMessage( "moveCancelMsg");

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmMoveHandler,
                           pHandlerArgs);
        }
    } else if ( pxEvent->type == ButtonRelease) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button1) {

            pMotifDisplay->SetCursor( XC_watch);
            pMotifDisplay->WriteResourceMessage( "moveSuccessMsg");

            if ( x != xStart || y != yStart) {
                pEditorClass->MoveSelectedObjects( x - xStart, y - yStart);
                pEditorClass->SetMapChanged( TRUE);
            }

        /* Remove the move handler. */

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmMoveHandler,
                           pHandlerArgs);
        }
    } else if ( pxEvent->type == MotionNotify) {
        pxMotionEvent = ( XMotionEvent *) pxEvent;

    /* Figure out the extends of the rectangle. */

        x = pxMotionEvent->x;
        y = pxMotionEvent->y;

        x = pMotifDisplay->GetDisplay()->VirtualCoord( x);
        y = pMotifDisplay->GetDisplay()->VirtualCoord( y);

        x = x - pHandlerArgs->xOffset;
        y = y - pHandlerArgs->yOffset;

        if ( fSnapToGrid == TRUE) {
            x = pMotifDisplay->GetGridLocation( x);
            y = pMotifDisplay->GetGridLocation( y);
        }

        if ( x < 0) {
            x = 0;
        } else if ( x + w > iWidthArea) {
            x = iWidthArea - w;
        }

        if ( y < 0) {
            y = 0;
        } else if ( y + h > iHeightArea) {
            y = iHeightArea - h;
        }

    /* Remove the old rectangle and draw the new one. */

        pHandlerArgs->x = x;
        pHandlerArgs->y = y;

        pHandlerArgs->pOutline->SetOrigin( x, y);
        pMotifDisplay->WriteTravelCursor( "Pos: %d, %d", x, y);
    }
}
