#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <X11/cursorfont.h>

#include <anvil/defaults.h>
#include <editor/OutlineClasses.h>

#define AmINDICATOR_MARGIN	8
#define MIN_SIZE		5

/*
 * Draws a rectangle as an outline.  This function is used if no
 *   outline function is given for drawing the XOR outlines.
 */

AmErrorCode
AmResizeOutlineClass::Draw( AmDisplayClass *pDisplay,
                          AmWindowClass  *pWindow,
                          AmGCClass      *pGC)
{
    char sz[ TEXTLEN];
    int  iMargin, iIndicatorLength,
         iTextHeight, iTextWidth,
         iAscent, iDescent,
         xText, yText;

    iMargin = pDisplay->VirtualCoord( AmINDICATOR_MARGIN);
    iIndicatorLength = iMargin / 2;

    sprintf( sz, "%d,%d", _x, _y);

    iTextHeight = pDisplay->FontHeight( pGC, &iAscent, &iDescent);
    iTextWidth = pDisplay->TextWidth( pGC, sz, strlen( sz));

    xText = _x + iIndicatorLength;
    yText = _y + iAscent + iIndicatorLength;

    pDisplay->DrawText( pWindow, pGC, xText, yText, sz);

    pDisplay->DrawRectangle( pWindow, pGC, _x, _y, _iWidth, _iHeight);

    pDisplay->DrawLine( pWindow, pGC,
                        _x - iMargin - iIndicatorLength - 1, _y,
                        _x - iMargin + iIndicatorLength, _y);

    pDisplay->DrawLine( pWindow, pGC,
                        _x - iMargin - 1, _y + 1,
                        _x - iMargin - 1, _y + _iHeight - 2);

    pDisplay->DrawLine( pWindow, pGC,
                        _x - iMargin - iIndicatorLength - 1,
                        _y + _iHeight - 1,
                        _x - iMargin + iIndicatorLength,
                        _y + _iHeight - 1);

    sprintf( sz, "%d", _iHeight);
 
    iTextWidth = pDisplay->TextWidth( pGC, sz, strlen( sz));

    xText = _x - iMargin - iTextWidth - iIndicatorLength;
    yText = _y + _iHeight / 2 + iDescent;

    pDisplay->DrawText( pWindow, pGC, xText, yText, sz);

    pDisplay->DrawLine( pWindow, pGC,
                        _x, _y - iMargin - iIndicatorLength - 1,
                        _x, _y - iMargin + iIndicatorLength);
                           
    pDisplay->DrawLine( pWindow, pGC,
                        _x + 1, _y - iMargin - 1,
                        _x + _iWidth - 2, _y - iMargin - 1);

    pDisplay->DrawLine( pWindow, pGC,
                        _x + _iWidth - 1,
                        _y - iMargin - iIndicatorLength - 1,
                        _x + _iWidth - 1,
                        _y - iMargin + iIndicatorLength);

    sprintf( sz, "%d", _iWidth);
 
    iTextWidth = pDisplay->TextWidth( pGC, sz, strlen( sz));

    xText = _x + _iWidth / 2 - iTextWidth / 2;
    yText = _y - iMargin - iIndicatorLength;

    pDisplay->DrawText( pWindow, pGC, xText, yText, sz);

    return( AmOK);
}


AmErrorCode
AmMoveOutlineClass::Draw( AmDisplayClass *pDisplay,
                        AmWindowClass  *pWindow,
                        AmGCClass      *pGC)
{
    char sz[ TEXTLEN];
    int  iMargin, iIndicatorLength,
         iTextHeight, iTextWidth,
         iAscent, iDescent,
         xText, yText,
         iSize;

    iMargin = pDisplay->VirtualCoord( AmINDICATOR_MARGIN);
    iIndicatorLength = iMargin / 2;

    sprintf( sz, "%d,%d", _x, _y);

    iTextHeight = pDisplay->FontHeight( pGC, &iAscent, &iDescent);
    iTextWidth = pDisplay->TextWidth( pGC, sz, strlen( sz));

    xText = _x + iIndicatorLength;
    yText = _y + iAscent + iIndicatorLength;

    pDisplay->DrawText( pWindow, pGC, xText, yText, sz);

    pDisplay->DrawRectangle( pWindow, pGC, _x, _y, _iWidth, _iHeight);

    iSize = pDisplay->VirtualCoord( 3);

    pDisplay->DrawLine( pWindow, pGC,
                        _x + _iWidth / 2 - iSize, _y + _iHeight / 2,
                        _x + _iWidth / 2 + iSize, _y + _iHeight / 2);

    pDisplay->DrawLine( pWindow, pGC,
                        _x + _iWidth / 2, _y + _iHeight / 2 - iSize,
                        _x + _iWidth / 2, _y + _iHeight / 2 + iSize);

    return( AmOK);
}

AmErrorCode
AmHighlightOutlineClass::Draw( AmDisplayClass *pDisplay,
                             AmWindowClass  *pWindow,
                             AmGCClass      *pGC)
{
    AmLineStyleEnum eLineStyle; 
    int           iLineWidth, iSize;

    /* Safe the GC values that we are going to step on. */

    eLineStyle = pGC->GetLineStyle();
    iLineWidth = pGC->GetLineWidth();

    /* Change to a fat double dashed line. */

    pGC->SetLineStyle( AmLINE_DOUBLE_DASH);
    pGC->SetLineWidth( pDisplay->VirtualCoord( 3));

    /* Draw a rectangle. */

    pDisplay->DrawRectangle( pWindow, pGC,
                             _x, _y, _iWidth, _iHeight);

    /* Change to the thinnest possible solid line. */

    pGC->SetLineStyle( AmLINE_SOLID);
    pGC->SetLineWidth( 0);

    /* Draw a cross hair. */

    iSize = pDisplay->VirtualCoord( 3);

    pDisplay->DrawLine( pWindow, pGC,
                        _x + _iWidth / 2 - iSize, _y + _iHeight / 2,
                        _x + _iWidth / 2 + iSize, _y + _iHeight / 2);

    pDisplay->DrawLine( pWindow, pGC,
                        _x + _iWidth / 2, _y + _iHeight / 2 - iSize,
                        _x + _iWidth / 2, _y + _iHeight / 2 + iSize);

    /* Restore the GC values. */

    pGC->SetLineStyle( eLineStyle);
    pGC->SetLineWidth( iLineWidth);

    return( AmOK);
}

/* Outline which draws the rectangle, the eight tab points and the X 
   through the rectangle */

void
AmSelectOutlineClass::initializeValues()
{
    _colBorder.red = 0xf0;
    _colBorder.green = 0xf0;
    _colBorder.blue = 0xf0;

    _colFill.red = 0xff;
    _colFill.green = 0x88;
    _colFill.blue = 0x88;
}

AmErrorCode
AmSelectOutlineClass::Draw( AmDisplayClass *pDisplay,
                          AmWindowClass  *pWindow,
                          AmGCClass      *pGC)
{
    AmColorCellRec colSave;
    int          iMidWidth = _iWidth / 2,
                 iMidHeight= _iHeight / 2,
                 iSize,
                 iAnchorSize, iMidAnchorSize;


    if (( _iWidth <= pDisplay->VirtualCoord( MIN_SIZE)) ||
        ( _iHeight <= pDisplay->VirtualCoord( MIN_SIZE))) {
        return( AmUNKNOWN_ERROR);
    }

    iAnchorSize = pDisplay->VirtualCoord( AmANCHOR_SIZE);
    iMidAnchorSize = iAnchorSize / 2;

    iSize = pDisplay->VirtualCoord( 1);

    pGC->GetForegroundColor( &colSave);
    pGC->SetForegroundColor( &_colBorder);

#if 0

    /* Upper, left anchor. */ 

    pDisplay->FillRectangle( pWindow, pGC,
                             _x - iMidAnchorSize - iSize,
                             _y - iMidAnchorSize - iSize,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Middle, left anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x - iMidAnchorSize - iSize, 
                             _y + iMidHeight - iMidAnchorSize - iSize,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Lower, left anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x - iMidAnchorSize - iSize, 
                             _y + _iHeight - iMidAnchorSize - iSize * 2,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Upper, middle anchor. */ 

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + iMidWidth - iMidAnchorSize - iSize,
                             _y - iMidAnchorSize - iSize,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Lower, middle anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + iMidWidth - iMidAnchorSize - iSize,
                             _y + _iHeight - iMidAnchorSize - iSize * 2,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Upper, right anchor. */ 

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + _iWidth - iMidAnchorSize - iSize * 2,
                             _y - iMidAnchorSize - iSize,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Middle, right anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + _iWidth - iMidAnchorSize - iSize * 2,
                             _y + iMidHeight - iMidAnchorSize - iSize * 2,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

    /* Lower, right anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + _iWidth - iMidAnchorSize - iSize * 2,
                             _y + _iHeight - iMidAnchorSize - iSize * 2,
                             iAnchorSize + iSize * 2,
                             iAnchorSize + iSize * 2);

#endif

    pGC->SetForegroundColor( &_colFill);

    /* Upper, left anchor. */ 

    pDisplay->FillRectangle( pWindow, pGC,
                             _x - iMidAnchorSize,
                             _y - iMidAnchorSize,
                             iAnchorSize, iAnchorSize);

    /* Middle, left anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x - iMidAnchorSize, 
                             _y + iMidHeight - iMidAnchorSize - iSize,
                             iAnchorSize, iAnchorSize);

    /* Lower, left anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x - iMidAnchorSize, 
                             _y + _iHeight - iMidAnchorSize - iSize,
                             iAnchorSize, iAnchorSize);

    /* Upper, middle anchor. */ 

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + iMidWidth - iMidAnchorSize - iSize,
                             _y - iMidAnchorSize,
                             iAnchorSize, iAnchorSize);

    /* Lower, middle anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + iMidWidth - iMidAnchorSize - iSize,
                             _y + _iHeight - iMidAnchorSize - iSize,
                             iAnchorSize, iAnchorSize);

    /* Upper, right anchor. */ 

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + _iWidth - iMidAnchorSize - iSize,
                             _y - iMidAnchorSize,
                             iAnchorSize, iAnchorSize);

    /* Middle, right anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + _iWidth - iMidAnchorSize - iSize,
                             _y + iMidHeight - iMidAnchorSize - iSize,
                             iAnchorSize, iAnchorSize);

    /* Lower, right anchor. */

    pDisplay->FillRectangle( pWindow, pGC,
                             _x + _iWidth - iMidAnchorSize - iSize,
                             _y + _iHeight - iMidAnchorSize - iSize,
                             iAnchorSize, iAnchorSize);

    pGC->SetForegroundColor( &colSave);

    return( AmOK);
}

void
AmFirstSelectOutlineClass::initializeValues()
{
    _colBorder.red = 0xf0;
    _colBorder.green = 0xf0;
    _colBorder.blue = 0xf0;

    _colFill.red = 0x00;
    _colFill.green = 0xff;
    _colFill.blue = 0xff;
}

AmErrorCode
AmFirstSelectOutlineClass::Draw( AmDisplayClass *pDisplay,
                               AmWindowClass  *pWindow,
                               AmGCClass      *pGC)
{
    return( AmSelectOutlineClass::Draw( pDisplay, pWindow, pGC));
}

AmErrorCode
AmOutlineOutlineClass::Draw( AmDisplayClass *pDisplay,
                           AmWindowClass  *pWindow,
                           AmGCClass      *pGC)
{
    AmLineStyleEnum eLineStyle;

    if ( _iWidth > 0 && _iHeight > 0) {

        eLineStyle = pGC->GetLineStyle();

        pGC->SetLineStyle( AmLINE_ON_OFF_DASH);
 
        pDisplay->DrawRectangle( pWindow, pGC,
                                 _x, _y, _iWidth, _iHeight);

        pGC->SetLineStyle( eLineStyle);
    }

    return( AmOK);
}
