#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/cursorfont.h>

#include <anvil/state_hand.h>
#include <anvil/userevents.h>

#include <editor/handlers.h>

const int AmACTION_OUTLINE_PRIORITY = 1000;

typedef struct outlineHandlerArgs {
    AmEditorClass         *pEditorClass;
    AmOutlineOutlineClass *pOutline;
    int                 xStart;
    int                 yStart;
} outlineHandlerArgsRec, *outlineHandlerArgsPtr;

void
AmOutlineHandler( Widget                widget,
                outlineHandlerArgsPtr pHandlerArgs,
                XEvent                *xpEvent,
                AnvilBoolean               *);

AmErrorCode
AmToggleObjects( AmEditorClass *pEditorClass,
               int         x0,
               int         y0,
               int         x1,
               int         y1)
{
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay();
    AmScreenObjClass     **ppScreenObjects;
    const AmRectangleRec *pRec;
    AnvilBoolean            fSuccess;
    int                xMin, yMin, xMax, yMax, ii;

/* Sort the endpoints. */

    if ( x0 < x1) {
        xMin = x0;
        xMax = x1;
    } else {
        xMin = x1;
        xMax = x0;
    }

    if ( y0 < y1) {
        yMin = y0;
        yMax = y1;
    } else {
        yMin = y1;
        yMax = y0;
    }

    fSuccess = FALSE;
    ppScreenObjects = pMotifDisplay->GetObjects();

    for ( ii = 0; ppScreenObjects[ ii] != NULL; ii++) {

        pRec = ppScreenObjects[ ii]->GetRect();
        if (( pRec->x >= xMin) && ( pRec->y >= yMin) &&
            ( pRec->x + pRec->width <= xMax) &&
            ( pRec->y + pRec->height <= yMax)) {

            if ( pEditorClass->IsObjectSelected( ppScreenObjects[ ii])) {
                pEditorClass->UnselectObject( ppScreenObjects[ ii]);
            } else {
                pEditorClass->SelectObject( ppScreenObjects[ ii]);
            }

            fSuccess = TRUE;
        }
    }

    if ( fSuccess == TRUE) {
        pMotifDisplay->WriteResourceMessage( "outlineSuccessMsg");
    } else {
        pMotifDisplay->WriteResourceMessage( "outlineFailureMsg");
    }

    free( ppScreenObjects);
    return( AmOK);
}


void
AmOutlineOutline( outlineHandlerArgsPtr pHandlerArgs,
                int                   x1,
                int                   y1)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    int               x0, y0, x, y, width, height;

    x0 = pHandlerArgs->xStart;
    y0 = pHandlerArgs->yStart;

    if ( x0 < x1) {
        x = x0;
        width = x1 - x0;
    } else {
        x = x1;
        width = x0 - x1;
    }

    if ( y0 < y1) {
        y = y0;
        height = y1 - y0;
    } else {
        y = y1;
        height = y0 - y1;
    }

    if ( pHandlerArgs->pOutline != NULL) {
        pHandlerArgs->pOutline->SetRectangle( x, y, width, height);
    } else {
        pHandlerArgs->pOutline = new AmOutlineOutlineClass( pMotifDisplay,
                                                          x, y,
                                                          width, height); 
        pMotifDisplay->AddOutline( pHandlerArgs->pOutline);
    }
}

/*********************************************************************
 *
 * Outline Handler Stuff:
 *
 *********************************************************************/

outlineHandlerArgsPtr
AmMakeOutlineHandlerArgs( AmEditorClass *pEditorClass,
                        int         x,
                        int         y)
{
    outlineHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = ( outlineHandlerArgsPtr)
        malloc( sizeof( outlineHandlerArgsRec));

    pHandlerArgs->pEditorClass = pEditorClass;
    pHandlerArgs->xStart = x;
    pHandlerArgs->yStart = y;
    pHandlerArgs->pOutline = NULL;

    return( pHandlerArgs);
}


void
AmRemoveOutlineHandler( outlineHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay();

    if ( pHandlerArgs->pOutline != NULL) {
        pMotifDisplay->RemoveOutline( pHandlerArgs->pOutline);
    }

    free( pHandlerArgs);
}


void
AmSetupOutlineHandler( AmEditorClass *pEditorClass,
                     int         x,
                     int         y)
{
    AmMotifDisplayClass     *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    Widget                wArea = pMotifDisplay->GetArea();
    outlineHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = AmMakeOutlineHandlerArgs( pEditorClass, x, y);

    AmAddHandler( NULL, wArea,
                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                ( XtEventHandler) AmOutlineHandler,
                pHandlerArgs, AmACTION_OUTLINE_PRIORITY, 
                ( AmStateHandlerFn) AmRemoveOutlineHandler);

    pMotifDisplay->WriteResourceMessage( "outlineInstructionMsg");
}

/************************************************************
 *
 * Outline Handler:
 *
 ************************************************************/

/* ARGSUSED */
void
AmOutlineHandler( Widget                widget,
                outlineHandlerArgsPtr pHandlerArgs,
                XEvent                *pxEvent,
                AnvilBoolean               *)
{
    AmEditorClass        *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    XButtonEvent       *pxButtonEvent;
    XMotionEvent       *pxMotionEvent;
    int                xStart = pHandlerArgs->xStart,
                       yStart = pHandlerArgs->yStart,
                       x, y,
                       iWidthArea = pMotifDisplay->GetWidth(),
                       iHeightArea = pMotifDisplay->GetHeight();

    if ( pxEvent->type == ButtonPress) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            pMotifDisplay->WriteResourceMessage( "outlineCancelMsg");
            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmOutlineHandler, pHandlerArgs);
        }
    } else if ( pxEvent->type == ButtonRelease) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button1) {

            x = pxButtonEvent->x;
            y = pxButtonEvent->y;

            x = pMotifDisplay->GetDisplay()->VirtualCoord( x);
            y = pMotifDisplay->GetDisplay()->VirtualCoord( y);

        /* Display the busy cursor.*/

            pMotifDisplay->SetCursor( XC_watch);

        /* Activate the objects within the rectangle */
 
            AmToggleObjects( pEditorClass, xStart, yStart, x, y);

        /* Remove the outline handler. */

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmOutlineHandler, pHandlerArgs);
        }
    } else if ( pxEvent->type == MotionNotify) {
        pxMotionEvent = ( XMotionEvent *) pxEvent;

    /* Figure out the extends of the rectangle. */

        x = pxMotionEvent->x;
        y = pxMotionEvent->y;

        x = pMotifDisplay->GetDisplay()->VirtualCoord( x);
        y = pMotifDisplay->GetDisplay()->VirtualCoord( y);

        if ( x < 0) {
            x = 0;
        } else if ( x > iWidthArea) {
            x = iWidthArea;
        }

        if ( y < 0) {
            y = 0;
        } else if ( y  > iHeightArea) {
            y = iHeightArea;
        }

    /* Remove the old rectangle and draw the new one. */

        AmOutlineOutline( pHandlerArgs, x, y);
    }
}
