#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Form.h>

#include <anvil/macros.h>
#include <anvil/events.h>
#include <anvil/motif.h>

#include <editor/PageSetupDialogClass.h>

const int AmMinSize = 10;

typedef struct PageSizeRec_ {
    char  *szName;
    float dWidth; 
    float dHeight; 
} PageSizeRec, *PageSizePtr;

static PageSizeRec pPageSize[] = {
    { "A Size: 8.5\" x 11\"", 8.5, 11 },
    { "B Size: 11\" x 17\"", 11, 17 },
    { "C Size: 17\" x 24\"", 17, 24 },
    { "D Size: 24\" x 36\"", 24, 36 },
    { "E Size: 36\" x 48\"", 36, 48 }
};

void
AmPageSetupDialogClass::initializeValues()
{
    _dWidth       = pPageSize[ 0].dWidth;
    _dHeight      = pPageSize[ 0].dHeight;
    _fLandscape   = FALSE;
    _fTrueColor   = TRUE;

    _pixPortrait  = 0;
    _pixLandscape = 0;
    _pXDisplay = NULL;
    _pXWindow = NULL;
    _pXGC = NULL;
}


void
AmPageSetupDialogClass::createInterface()
{
    char     sz[ TEXTLEN];
    int      iForeground, iBackground, ii;

    _pwcFrame = new AmFrameClass( _wForm, "frame", FALSE);
    _pwcFrame->AttachLeft();
    _pwcFrame->AttachTop();
    _pwcFrame->AttachRight();
    _pwcFrame->AttachBottom();

    _pwcOptionForm = new AmFormClass( _pwcFrame, "optionForm"); 
    _pwcOptionForm->SetFractionBase( 10);

    _pwcFrameForm = new AmFormClass( _pwcOptionForm, "frameForm"); 
    _pwcFrameForm->AttachLeft();
    _pwcFrameForm->AttachTop();
    _pwcFrameForm->AttachBottom();
    _pwcFrameForm->AttachRight( 5);

    _pwcSizeForm = new AmFormClass( _pwcFrameForm, "sizeForm");
    _pwcSizeForm->AttachLeft();
    _pwcSizeForm->AttachTop();
    _pwcSizeForm->AttachRight();

    _pwcSizeLabel = new AmLabelClass( _pwcSizeForm, "sizeLbl");
    _pwcSizeLabel->AttachLeft();
    _pwcSizeLabel->AttachTop();
    _pwcSizeLabel->AttachBottom();

    _pwcOptionMenu = new AmComboBoxClass( _pwcSizeForm, "optionMenu");
    _pwcOptionMenu->AttachLeft( _pwcSizeLabel);
    _pwcOptionMenu->AttachTop();
    _pwcOptionMenu->AttachRight();
    _pwcOptionMenu->AttachBottom();

    _pwcOptionMenu->SetCallback(( XtCallbackProc) AmUpdateSizeCallback, 
                                ( XtPointer) this);

 
    for ( ii = 0; ii < (signed)VECLEN( pPageSize); ii++) {
        _pwcOptionMenu->AddButton( pPageSize[ ii].szName, NULL, NULL);
    }
    _pwcOptionMenu->AddButton( "Custom", NULL, NULL);

    _pwcWidthForm = new AmFormClass( _pwcFrameForm, "widthForm");
    _pwcWidthForm->AttachLeft();
    _pwcWidthForm->AttachTop( _pwcSizeForm);
    _pwcWidthForm->AttachRight();

    _pwcWidthLabel = new AmLabelClass( _pwcWidthForm, "widthLbl");
    _pwcWidthLabel->AttachLeft();
    _pwcWidthLabel->AttachTop();
    _pwcWidthLabel->AttachBottom();

    _pwcWidthText = new AmTextFieldClass( _pwcWidthForm, "widthText");
    _pwcWidthText->AttachTop();
    _pwcWidthText->AttachRight();
    _pwcWidthText->AttachBottom();

    sprintf( sz, "%.2f", _dWidth);
    _pwcWidthText->SetValue( sz);

    _pwcHeightForm = new AmFormClass( _pwcFrameForm, "heightForm");
    _pwcHeightForm->AttachLeft();
    _pwcHeightForm->AttachTop( _pwcWidthForm);
    _pwcHeightForm->AttachRight();

    _pwcHeightLabel = new AmLabelClass( _pwcHeightForm, "heightLbl");
    _pwcHeightLabel->AttachLeft();
    _pwcHeightLabel->AttachTop();
    _pwcHeightLabel->AttachBottom();

    _pwcHeightText = new AmTextFieldClass( _pwcHeightForm, "heightText");
    _pwcHeightText->AttachTop();
    _pwcHeightText->AttachRight();
    _pwcHeightText->AttachBottom();

    sprintf( sz, "%.2f", _dHeight);
    _pwcHeightText->SetValue( sz);

    _pwcModeFrame = new AmFrameClass( _pwcFrameForm, "modeFrame");
    _pwcModeFrame->AttachLeft();
    _pwcModeFrame->AttachTop( _pwcHeightForm);
    _pwcModeFrame->AttachRight();

    _pwcModeForm = new AmFormClass( _pwcModeFrame, "modeForm");
    _pwcModeForm->SetFractionBase( 10);

    _pwcModeLabel = new AmLabelClass( _pwcModeForm, "modeLabel");
    _pwcModeLabel->AttachLeft();
    _pwcModeLabel->AttachTop();
    _pwcModeLabel->AttachBottom();
    _pwcModeLabel->AttachRight( 3);

    _pwcModePane = new AmRadioPaneClass( _pwcModeForm, "modeRowColumn");
    _pwcModePane->AttachTop();
    _pwcModePane->AttachRight();
    _pwcModePane->AttachBottom();
    _pwcModePane->AttachLeft( 3);

    _pwcPortraitToggle = _pwcModePane->AddButton( "portraitToggle",
                                           ( XtCallbackProc) AmToggleCallback,
                                           ( XtPointer) this);
    _pwcPortraitToggle->SetState( !_fLandscape, FALSE);

    _pwcLandscapeToggle = _pwcModePane->AddButton( "landscapeToggle",
                                            ( XtCallbackProc) AmToggleCallback,
                                            ( XtPointer) this);
    _pwcLandscapeToggle->SetState( _fLandscape, FALSE);

    _pwcPrintFrame = new AmFrameClass( _pwcFrameForm, "printFrame");
    _pwcPrintFrame->AttachLeft();
    _pwcPrintFrame->AttachTop( _pwcModeFrame);
    _pwcPrintFrame->AttachRight();
    _pwcPrintFrame->AttachBottom();

    _pwcPrintForm = new AmFormClass( _pwcPrintFrame, "printForm");
    _pwcPrintForm->SetFractionBase( 10);

    _pwcColorLabel = new AmLabelClass( _pwcPrintForm, "colorLabel");
    _pwcColorLabel->AttachLeft();
    _pwcColorLabel->AttachTop();
    _pwcColorLabel->AttachBottom();
    _pwcColorLabel->AttachRight( 3);

    _pwcColorPane = new AmRadioPaneClass( _pwcPrintForm, "colorPane");
    _pwcColorPane->AttachTop();
    _pwcColorPane->AttachRight();
    _pwcColorPane->AttachBottom();
    _pwcColorPane->AttachLeft( 3);

    _pwcTrueColorToggle = _pwcColorPane->AddButton( "trueColorToggle",
                                             ( XtCallbackProc) AmToggleCallback,
                                             ( XtPointer) this);
    _pwcTrueColorToggle->SetState( _fTrueColor, FALSE);

    _pwcPseudoColorToggle = _pwcColorPane->AddButton( "pseudoColorToggle",
                                               ( XtCallbackProc) AmToggleCallback,
                                               ( XtPointer) this);
    _pwcPseudoColorToggle->SetState( !_fTrueColor, FALSE);

    _pwcPreviewFrame = new AmFrameClass( _pwcOptionForm, "previewFrame", FALSE);
    _pwcPreviewFrame->AttachLeft( 5);
    _pwcPreviewFrame->AttachTop();
    _pwcPreviewFrame->AttachRight();
    _pwcPreviewFrame->AttachBottom();

    _pwcArea = new AmDrawingAreaClass( _pwcPreviewFrame, "previewArea");

    _pXDisplay = new AmXDisplayClass( *_pwcArea);
    _pXWindow = NULL;

    _display = XtDisplay( *_pwcArea);
    _screen = XtScreen( *_pwcArea);
 
    XtVaGetValues( *_pwcModeLabel,
                   XmNforeground, &iForeground,
                   XmNbackground, &iBackground,
                   NULL);

    _pixPortrait = XmGetPixmap( _screen, 
                                "/home/philt/icons/portrait.xbm",
                                iForeground,
                                iBackground);

    _pixLandscape = XmGetPixmap( _screen, 
                                 "/home/philt/icons/landscape.xbm",
                                 iForeground,
                                 iBackground);

   
    XtAddCallback( *_pwcArea, XmNexposeCallback,
                   ( XtCallbackProc) AmExposeCallback,
                   ( XtPointer) this);

    XtAddCallback( *_pwcArea, XmNresizeCallback,
                   ( XtCallbackProc) AmExposeCallback,
                   ( XtPointer) this);

    XtAddCallback( *_pwcWidthText, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmUpdateWidthCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcWidthText, XmNactivateCallback,
                   ( XtCallbackProc) AmUpdateWidthCallback,
                   ( XtPointer) this);

    XtAddCallback( *_pwcHeightText, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmUpdateHeightCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcHeightText, XmNactivateCallback,
                   ( XtCallbackProc) AmUpdateHeightCallback,
                   ( XtPointer) this);
}


AmPageSetupDialogClass::AmPageSetupDialogClass()
    : AmDialogClass()
{
    initializeValues();
}


AmPageSetupDialogClass::AmPageSetupDialogClass( Widget     wParent,
                                            const char *szName)
    : AmDialogClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


AmPageSetupDialogClass::AmPageSetupDialogClass( AmBaseWidgetClass *bwcParent,
                                            const char      *szName)
    : AmDialogClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}


AmPageSetupDialogClass::~AmPageSetupDialogClass()
{
    if ( _pXGC != NULL) {
	delete _pXGC;
        _pXGC = NULL;
    }

    if ( _pXWindow != NULL) {
        delete _pXWindow;
        _pXWindow = NULL;
    }

    if ( _pXDisplay != NULL) {
        delete _pXDisplay;
        _pXDisplay = NULL;
    }

    if ( _pixPortrait != 0) {
        XmDestroyPixmap( _screen, _pixPortrait);
    }

    if ( _pixLandscape != 0) {
        XmDestroyPixmap( _screen, _pixLandscape);
    }
}


float
AmPageSetupDialogClass::GetPageWidth()
{
    return( _dWidth);
}


float
AmPageSetupDialogClass::GetPageHeight()
{
    return( _dHeight);
}


AnvilBoolean
AmPageSetupDialogClass::GetLandscape()
{
    return( _fLandscape);
}


AnvilBoolean
AmPageSetupDialogClass::GetTrueColor()
{
    return( _fTrueColor);
}


void
AmPageSetupDialogClass::SetWidth( float dWidth)
{
    _dWidth = dWidth;

    valuesChanged();
}


void
AmPageSetupDialogClass::SetHeight( float dHeight)
{
    _dHeight = dHeight;

    valuesChanged();
}


void
AmPageSetupDialogClass::SetLandscape( AnvilBoolean fLandscape)
{
    _fLandscape = fLandscape;
}


void
AmPageSetupDialogClass::SetTrueColor( AnvilBoolean fTrueColor)
{
    _fTrueColor = fTrueColor;

    _pwcTrueColorToggle->SetState( _fTrueColor, FALSE);
    _pwcPseudoColorToggle->SetState( !_fTrueColor, FALSE);
}


void
AmPageSetupDialogClass::valuesChanged()
{
    int     ii, iPos=0;
    AnvilBoolean fFound, fLandscape=FALSE;
 
    for ( ii = 0, fFound = FALSE;
          ( fFound == FALSE) && ( ii < (signed)VECLEN( pPageSize));
          ii++) {

        if (( _dWidth == pPageSize[ ii].dWidth) &&
            ( _dHeight == pPageSize[ ii].dHeight)) {

            fFound = TRUE;
            fLandscape = FALSE;
            iPos = ii;

        } else if (( _dHeight == pPageSize[ ii].dWidth) &&
                   ( _dWidth == pPageSize[ ii].dHeight)) {

            fFound = TRUE;
            fLandscape = TRUE;
            iPos = ii;

        }
    }

    if ( fFound == TRUE) {
        _pwcOptionMenu->SetSelectedPosition( iPos);

        if ( fLandscape != _fLandscape) {
            _fLandscape = fLandscape;
            _pwcPortraitToggle->SetState( !_fLandscape, FALSE);
            _pwcLandscapeToggle->SetState( _fLandscape, FALSE);
        }

    } else {
        _pwcOptionMenu->SetSelectedPosition( VECLEN( pPageSize));
    }

    if ( _fLandscape == TRUE) {
        if ( _pixLandscape != 0) {
            XtVaSetValues( *_pwcModeLabel,
                           XmNlabelPixmap, _pixLandscape,
                           NULL);
        }
    } else {
        if ( _pixPortrait != 0) {
            XtVaSetValues( *_pwcModeLabel,
                           XmNlabelPixmap, _pixPortrait,
                           NULL);
        }
    }

    if ( XtIsRealized( *_pwcArea)) {
        AmSendExpose( _display, XtWindow( *_pwcArea));
    }
}


void
AmPageSetupDialogClass::AmExposeCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{
    AmPageSetupDialogClass *pPageSetupDialog;
    pPageSetupDialog = ( AmPageSetupDialogClass *) xpClientData;

    pPageSetupDialog->expose( w, xpCallData);
}

void
AmPageSetupDialogClass::expose( Widget    w,
                              XtPointer xpCallData)
{
    Dimension    dimWidth, dimHeight;
    int          iWidth, iHeight, x, y, xOffset, yOffset, iTextWidth, 
                 iInch, iDPI, iWidthTemp, iHeightTemp;
    float        dScale;
    AmColorCellRec colWhite, colBlack;
    char         sz[ TEXTLEN];

    if ( XtIsRealized( *_pwcArea)) {

        if ( _pXWindow == NULL) {
	    XColor	xColor;
	    Pixel	iBackground;
	    Colormap	colormap;

            _pXWindow = new AmXWindowClass( _pXDisplay, *_pwcArea);
	    _pXGC = new AmXGCClass( _pXWindow);
  
	    XtVaGetValues( *_pwcArea,
			   XmNcolormap, &colormap,
			   XmNbackground, &iBackground,
			   NULL);

	    xColor.pixel = iBackground;
	    XQueryColor( _display, colormap, &xColor);

	    _colBackground.red   = xColor.red >> 8;
	    _colBackground.green = xColor.green >> 8;
	    _colBackground.blue  = xColor.blue >> 8;

	    _pXDisplay->AllocColor( &_colBackground);

	    _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
	    _iMargin     = (int)(_iTextHeight * 1.5);
        }

        XtVaGetValues( *_pwcArea,
                       XmNwidth, &dimWidth,
                       XmNheight, &dimHeight,
                       NULL);

        iWidthTemp  = _pXDisplay->VirtualCoord( dimWidth);
        iHeightTemp = _pXDisplay->VirtualCoord( dimHeight);

        _pXGC->SetForegroundColor( &_colBackground);
        _pXDisplay->FillRectangle( _pXWindow, _pXGC, 0, 0,
                                   iWidthTemp, iHeightTemp);

        if ( _dHeight > _dWidth) { 
            iHeight = iHeightTemp - 2 * _iMargin;
            iWidth = (int)(iHeight * _dWidth / _dHeight);
        } else {
            iWidth = iWidthTemp - 2 * _iMargin;
            iHeight = (int)(iWidth * _dHeight / _dWidth);
        }

        if ( iHeight > ( iHeightTemp - 2 * _iMargin)) {

            dScale = ( float)( iHeightTemp - 2 * _iMargin) / ( float) iHeight;
            iHeight = iHeightTemp - 2 * _iMargin; 
            iWidth = (int)(iWidth * dScale);

        } else if ( iWidth > ( iWidthTemp - 2 * _iMargin)) {

            dScale = ( float)( iWidthTemp - 2 * _iMargin) / ( float) iWidth;
            iHeight = (int)(iHeight * dScale);
            iWidth = iWidthTemp - 2 * _iMargin;

        }

        if ( iWidth > AmMinSize && iHeight > AmMinSize) {

            x = ( int)( ( iWidthTemp - iWidth) / 2.0 + 0.5),
            y = ( int)( ( iHeightTemp - iHeight) / 2.0 + 0.5),

            colWhite.red = 0xff;
            colWhite.green = 0xff;
            colWhite.blue = 0xff;
  
            _pXDisplay->AllocColor( &colWhite);
            _pXGC->SetForegroundColor( &colWhite);

            colBlack.red = 0x0;
            colBlack.green = 0x0;
            colBlack.blue = 0x0;
  
            _pXDisplay->AllocColor( &colBlack);

            _pXDisplay->FillRectangle( _pXWindow, _pXGC,
                                       x, y, iWidth, iHeight);

            _pXGC->SetForegroundColor( &colBlack);

            _pXDisplay->DrawRectangle( _pXWindow, _pXGC,
                                       x, y, iWidth, iHeight);

            xOffset = ( int)(( _iMargin - _iTextHeight) / 2.0 + 0.5);
            yOffset = ( int)(( _iMargin - _iTextHeight) / 2.0 + 0.5);

            _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                  x, y - yOffset, x + iWidth - 1, y - yOffset); 

            _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                  x, y - yOffset * 3 / 2,
                                  x, y - yOffset * 1 / 2); 

            _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                  x + iWidth - 1, y - yOffset * 3 / 2,
                                  x + iWidth - 1, y - yOffset * 1 / 2); 

            sprintf( sz, "%.1f\"", _dWidth); 
            iTextWidth = _pXDisplay->TextWidth( _pXGC, sz, strlen( sz));
            _pXDisplay->DrawText( _pXWindow, _pXGC,
                                  x + iWidth / 2 - iTextWidth / 2,
                                  y - yOffset * 3 / 2,
                                  sz);

            _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                  x - xOffset, y, x - xOffset, y + iHeight - 1);

            _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                  x - xOffset * 3 / 2, y,
                                  x - xOffset * 1 / 2, y);

            _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                  x - xOffset * 3 / 2, y + iHeight - 1,
                                  x - xOffset * 1 / 2, y + iHeight - 1);

            sprintf( sz, "%.1f\"", _dHeight); 
            iTextWidth = _pXDisplay->TextWidth( _pXGC, sz, strlen( sz));
            _pXDisplay->DrawRotatedText( _pXWindow, _pXGC,
                                         x - xOffset * 3 / 2,
                                         y + iHeight / 2 + iTextWidth / 2,
                                         DEGTORAD( 90),
                                         sz);

            iDPI = ( int)( iHeight / _dHeight + 0.5);

            if ( iDPI > _pXDisplay->VirtualCoord( 1)) {

                for ( iInch = 1; iInch < _dHeight; iInch++) {
          
                    if ( iInch * iDPI >= iHeight) { 
                        break;
                    }

                    _pXDisplay->DrawLine( _pXWindow, _pXGC,
                                          x + xOffset, 
                                          y + iInch * iDPI, 
                                          x + iWidth - 1 - xOffset,
                                          y + iInch * iDPI); 
 
                }
            }
        }
    }
}


void
AmPageSetupDialogClass::AmUpdateWidthCallback( Widget    w,
                                           XtPointer xpClientData,
                                           XtPointer xpCallData)
{
    AmPageSetupDialogClass *pPageSetupDialog;
    pPageSetupDialog = ( AmPageSetupDialogClass *) xpClientData;

    pPageSetupDialog->updateWidth( w, xpCallData);
}

void
AmPageSetupDialogClass::updateWidth( Widget    w,
                                   XtPointer xpCallData)
{
    char  *sz;
    float dTemp;

    XtVaGetValues( w, XmNvalue, &sz, NULL);
    dTemp = atof( sz);
    free( sz); 

    if ( dTemp != _dWidth) {
        _dWidth = dTemp;
        valuesChanged();
    }
}


void
AmPageSetupDialogClass::AmUpdateHeightCallback( Widget    w,
                                            XtPointer xpClientData,
                                            XtPointer xpCallData)
{
    AmPageSetupDialogClass *pPageSetupDialog;
    pPageSetupDialog = ( AmPageSetupDialogClass *) xpClientData;

    pPageSetupDialog->updateHeight( w, xpCallData);
}

void
AmPageSetupDialogClass::updateHeight( Widget    w,
                                    XtPointer xpCallData)
{
    char  *sz;
    float dTemp;

    XtVaGetValues( w, XmNvalue, &sz, NULL);
    dTemp = atof( sz);
    free( sz); 

    if ( dTemp != _dHeight) {
        _dHeight = dTemp;
        valuesChanged();
    }
}

void
AmPageSetupDialogClass::AmUpdateSizeCallback( Widget    w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{
    AmPageSetupDialogClass *pPageSetupDialog;
    pPageSetupDialog = ( AmPageSetupDialogClass *) xpClientData;

    pPageSetupDialog->updateSize( w, xpCallData);
}

void
AmPageSetupDialogClass::updateSize( Widget    w,
                                  XtPointer xpCallData)
{
    int  iPos = ( int) xpCallData; 
    char sz[ TEXTLEN];

    if ( iPos >= (signed)VECLEN( pPageSize)) {
        return;
    }

    if ( _fLandscape == FALSE) {
        _dWidth = pPageSize[ iPos].dWidth;
        _dHeight = pPageSize[ iPos].dHeight;
    } else {
        _dWidth = pPageSize[ iPos].dHeight;
        _dHeight = pPageSize[ iPos].dWidth;
    }

    sprintf( sz, "%.2f", _dWidth);
    XtVaSetValues( *_pwcWidthText, XmNvalue, sz, NULL);

    sprintf( sz, "%.2f", _dHeight);
    XtVaSetValues( *_pwcHeightText, XmNvalue, sz, NULL);

    AmSendExpose( _display, XtWindow( *_pwcArea));
}


void
AmPageSetupDialogClass::AmToggleCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData)
{
    AmPageSetupDialogClass *pPageSetupDialog;
    pPageSetupDialog = ( AmPageSetupDialogClass *) xpClientData;

    pPageSetupDialog->toggle( w, xpCallData);
}


void
AmPageSetupDialogClass::toggle( Widget    w,
                              XtPointer xpCallData)
{
    XmToggleButtonCallbackStruct *tbc;
    char                         sz[ TEXTLEN];
    AnvilBoolean                      fLandscape;
    float                        dTemp;

    tbc = ( XmToggleButtonCallbackStruct *) xpCallData;

    /* JNF */
    if (w == (Widget) *_pwcPortraitToggle) {
        if ( tbc->set == TRUE) {
            fLandscape = FALSE;
        } else {
            fLandscape = TRUE;
        }

        if ( fLandscape != _fLandscape) {

            dTemp = _dWidth;
            _dWidth = _dHeight;
            _dHeight = dTemp;

            _fLandscape = fLandscape;

            sprintf( sz, "%.2f", _dWidth);
            XtVaSetValues( *_pwcWidthText, XmNvalue, sz, NULL);

            sprintf( sz, "%.2f", _dHeight);
            XtVaSetValues( *_pwcHeightText, XmNvalue, sz, NULL);

            valuesChanged();
        }
    } else if ( w == /* JNF?? */ (Widget) *_pwcTrueColorToggle) { 
        _fTrueColor = tbc->set;
    }
}
