#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#if !defined (__BSD44__)
#include <alloca.h>
#endif
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/DrawingA.h>

#include <anvil/macros.h>
#include <anvil/strlib.h>
#include <anvil/events.h>
#include <anvil/va_cback.h>
#include <anvil/userevents.h>
#include <anvil/eventcodes.h>
#include <anvil/log.h>
#include <anvil/resource.h>
#include <anvil/motif.h>

#include <editor/PickLayerDialogClass.h>

#include <anvil/layerstrs.h>

/* Utility function. */

static char *
GetLayerName( AmScreenObjClass *pScreenObj)
{
    char       *szName;
    const char *szTemp;

    if ( pScreenObj != NULL) {

        if ( pScreenObj->IsType( AmDATA_GRAPHIC)) {
            pScreenObj->GetVaFields( AmNtitle, &szName, NULL); 
            if ( szName != NULL) {
                return( szName);
            }

            pScreenObj->GetVaFields( AmNfilename, &szName, NULL); 
            if ( szName != NULL) {
                return( szName);
            }
        }

        szTemp = pScreenObj->getObjectName();
        if ( szTemp != NULL) {
            return( strdup( szTemp));
        }
    }

    return( strdup( "Unknown Layer"));
}


/*
 * Function: AmPickLayerDialogClass::initializeValues()
 *
 * Purpose:
 *   Initialize the member variables to sane values.
 */

void
AmPickLayerDialogClass::initializeValues()
{
    _vpDataGraphic     = NULL;
    _nDataGraphic      = 0;

    _viLayerPos        = NULL;
    _nLayerPos         = 0;

    _viSelectedLayers  = 0;
    _nSelectedLayers   = 0;

    _eObjectType       = AmANY_OBJECT;
    _fShowOverlay      = FALSE;

    _pXDisplay         = NULL;
    _pXWindow          = NULL;
    _pRefMotifDisplay  = NULL;
    _pRefOverlay       = NULL;

    _vpScreenObj       = NULL;
}


/*
 * Function: AmPickLayerDialogClass::createInterface()
 *
 * Purpose:
 *   Creates the widgets for the layer selection dialog.
 */

void
AmPickLayerDialogClass::createInterface()
{
    AmLabelClass *pwcLayerLabel;
    int          iWidth, iHeight;

    pwcLayerLabel = new AmLabelClass( _wForm, "layerLbl");
    pwcLayerLabel->AttachTop();
    pwcLayerLabel->AttachLeft();

    _pwcInfoFrame = new AmFrameClass( _wForm, "infoFrame", FALSE);
    _pwcInfoFrame->AttachTop( pwcLayerLabel); 
    _pwcInfoFrame->AttachRight();

    _pwcLayerList = new AmListClass( _wForm, "list");
    _pwcLayerList->AttachLeft();
    _pwcLayerList->AttachTop( pwcLayerLabel);
    _pwcLayerList->AttachBottom();
    _pwcLayerList->AttachRight( _pwcInfoFrame);

    _pwcToggle = new AmToggleButtonClass( _wForm, "infoToggle",
                                        ( XtCallbackProc) AmToggleCallback,
                                        ( XtPointer) this);
    _pwcToggle->AttachLeft( _pwcLayerList);
    _pwcToggle->AttachTop( _pwcInfoFrame);
    _pwcToggle->AttachRight();

    _pwcToggle->SetState( FALSE);

    _pwcArea = new AmDrawingAreaClass( _pwcInfoFrame, "infoArea");

    _pXDisplay        = new AmXDisplayClass( *_pwcArea);
    _pXWindow         = new AmXWindowClass( _pXDisplay, *_pwcArea);
    _pRefMotifDisplay = new AmMotifDisplayClass( _pXDisplay, _pXWindow, FALSE);

    iWidth  = _pwcArea->GetWidth();
    iHeight = _pwcArea->GetHeight();

    _pRefOverlay   = new AmOverlayClass( _pXWindow, "Overlay Plot",
                                         AmNx, 0,
                                         AmNy, 0,
                                         AmNwidth, iWidth,
                                         AmNheight, iHeight,
                                         AmNpreserveAspect, TRUE,
                                         NULL);

    _pRefMotifDisplay->AddObject( _pRefOverlay);
    _pRefMotifDisplay->Flush();

    AddButton( "okBtn", 
               ( XtCallbackProc) AmActionCallback,
               ( XtPointer) this);
    AddButton( "cancelBtn",
               ( XtCallbackProc) AmPopdownCback,
               ( XtPointer) _wForm);
    AddButton( "helpBtn", NULL, NULL);

/* Regardless of what kind of selection is to be done, we want to
   update the information box. */

    _pwcLayerList->AddSingleCallback(( XtCallbackProc) AmSelectCallback,
                                     ( XtPointer) this);
    _pwcLayerList->AddBrowseCallback(( XtCallbackProc) AmSelectCallback,
                                     ( XtPointer) this);
    _pwcLayerList->AddMultipleCallback(( XtCallbackProc) AmSelectCallback,
                                       ( XtPointer) this);
    _pwcLayerList->AddExtendedCallback(( XtCallbackProc) AmSelectCallback,
                                       ( XtPointer) this);

/* Always tie in the action callback which gets activated when the
   user double clicks on a list item. */

    _pwcLayerList->AddActionCallback(( XtCallbackProc) AmActionCallback,
                                     ( XtPointer) this);

    AmAddCallback( AM_PKG_ID, AM_DELETE_LAYER,
                 ( callbackProc) AmLayerDeletedCallback,
                 ( XtPointer) this);

    AmAddCallback( AM_PKG_ID, AM_DELETE_OBJECT,
                 ( callbackProc) AmObjectDeletedCallback,
                 ( XtPointer) this);

    updateLayerList();
}


/*
 * Function: AmPickLayerDialogClass::AmPickLayerDialogClass()
 *
 * Purpose:
 *   Default constructor.
 */

AmPickLayerDialogClass::AmPickLayerDialogClass()
    : AmDialogClass()
{
    _pOverlay          = NULL;
    _xcpActionCallback = NULL;
    _xpClientData      = NULL;

    initializeValues();
}


/*
 * Functions: AmPickLayerDialogClass::AmPickLayerDialogClass( wParent, szName)
 *            AmPickLayerDialogClass::AmPickLayerDialogClass( bwcParent, szName)
 *
 * Purpose:
 *   Constructor for the AmPickLayerDialogClass.  Both functions work
 *   the same way except that one takes a widget and the other
 *   takes a "wrapped" widget.
 *
 * Parameters: 
 *   Widget          wParent    : The parent of the dialog.
 *   BaseWidgetclass *bwcParent : The "wrapped" parent of the dialog.
 *   const char      *szName    : The name of the dialog.
 */

AmPickLayerDialogClass::AmPickLayerDialogClass( Widget         wParent,
                                                const char     *szName,
                                                AmOverlayClass *pOverlay,
                                                XtCallbackProc xcpCallback,
                                                XtPointer      xpClientData)
    : AmDialogClass( wParent, szName)
{
    _pOverlay          = pOverlay;
    _xcpActionCallback = xcpCallback;
    _xpClientData      = xpClientData;

    initializeValues();
    createInterface();
}


/* See above comment. */

AmPickLayerDialogClass::AmPickLayerDialogClass( AmBaseWidgetClass *bwcParent,
                                                const char     *szName, 
                                                AmOverlayClass *pOverlay,
                                                XtCallbackProc xcpCallback,
                                                XtPointer      xpClientData)
    : AmDialogClass( bwcParent, szName)
{
    _pOverlay          = pOverlay;
    _xcpActionCallback = xcpCallback;
    _xpClientData      = xpClientData;

    initializeValues();
    createInterface();
}


/*
 * Function: AmPickLayerDialogClass::~AmPickLayerDialogClass()
 *
 * Purpose:
 *   Destructor for the AmPickLayerDialogClass.
 */

AmPickLayerDialogClass::~AmPickLayerDialogClass()
{
    AmRemoveCallback( AM_PKG_ID, AM_DELETE_OBJECT,
                    ( callbackProc) AmObjectDeletedCallback,
                    ( XtPointer) this);

    AmRemoveCallback( AM_PKG_ID, AM_DELETE_LAYER,
                    ( callbackProc) AmLayerDeletedCallback,
                    ( XtPointer) this);

/* Free any allocated memory. */

    if ( _vpScreenObj != NULL) {
        FREE( _vpScreenObj);
        _vpScreenObj = NULL;
    }

    if ( _vpDataGraphic != NULL) {
        FREE( _vpDataGraphic);
        _vpDataGraphic = NULL;
        _nDataGraphic  = 0;
    }

    if ( _viLayerPos != NULL) {
        FREE( _viLayerPos);
        _viLayerPos = NULL;
        _nLayerPos  = 0;
    }

    if ( _viSelectedLayers != NULL) {
        FREE( _viSelectedLayers);
        _viSelectedLayers = NULL;
        _nSelectedLayers  = 0;
    }

    _pRefMotifDisplay = NULL;
    _pXWindow = NULL;
    _pXDisplay = NULL;
}


void
AmPickLayerDialogClass::SetShowOverlay( AnvilBoolean fShowOverlay)
{
    _fShowOverlay = fShowOverlay;
    updateLayerList();     
}


void
AmPickLayerDialogClass::SetFilter( AmObjectTypeEnum eObjectType)
{
    _eObjectType = eObjectType;
    updateLayerList();     
}

void
AmPickLayerDialogClass::SetSingleSelect()
{
    _pwcLayerList->SetSingleSelect();
}


void
AmPickLayerDialogClass::SetBrowseSelect()
{
    _pwcLayerList->SetBrowseSelect();
}


void
AmPickLayerDialogClass::SetMultipleSelect()
{
    _pwcLayerList->SetMultipleSelect();
}


void
AmPickLayerDialogClass::SetExtendedSelect()
{
    _pwcLayerList->SetExtendedSelect();
}

void
AmPickLayerDialogClass::updateLayerList()
{
    char *szLayerName;
    int  ii;

/* First release the old vector of layers and generate a new
   one. */

    if ( _vpDataGraphic != NULL) {
        FREE( _vpDataGraphic);
        _vpDataGraphic = NULL;
        _nDataGraphic  = 0;
    }

    if ( _viLayerPos != NULL) {
        FREE( _viLayerPos);
        _viLayerPos = NULL;
        _nLayerPos  = 0;
    }

/* Then remove the items from the layer list... */

    _pwcLayerList->DeleteAllItems();

/* ..and generate the new ones. */

    if ( _pOverlay != NULL) {

        _pOverlay->GetLayers( &_vpDataGraphic, &_nDataGraphic);

        if ( _nDataGraphic > 0) {

            _viLayerPos = ( int *) malloc( sizeof( int) * _nDataGraphic);
            if ( _viLayerPos == NULL) {
                ANVIL_ERROR_CODE = AmMALLOC_FAILED;
                AmAddResourceToLog( "mallocFailed",
                                  ( sizeof( int) * _nDataGraphic));
                return;
            }

        /* Go through the list of layers and filter them according to
           the object type given by _eObjectType.  Each layer that 
           passes the filter gets added to the list and the layer
           number gets recorded. */ 
   
            for ( ii = 0; ii < _nDataGraphic; ii++) { 
                if (( _eObjectType == AmANY_OBJECT) ||
                    ( _vpDataGraphic[ ii]->IsType( _eObjectType))) {

                /* Get the name and add it to the list .*/

                    szLayerName = GetLayerName( _vpDataGraphic[ ii]);
                    _pwcLayerList->AddItem( szLayerName);
                    FREE( szLayerName);

                /* Record the layer number for later use. */

                    _viLayerPos[ _nLayerPos++] = ii;
                }
            }
        }

        if ( _fShowOverlay == TRUE) {
            szLayerName = GetLayerName( _pOverlay);
            _pwcLayerList->AddItem( szLayerName); 
            FREE( szLayerName);
        }

    }

    select( *_pwcLayerList, NULL);
}


void
AmPickLayerDialogClass::updateInfoBox()
{
    AmScreenObjClass   *pScreenObj;
    AnvilBoolean          fNeedFlush;
    int              ii, iLayerNum;

    fNeedFlush = FALSE;

/* Get rid of the previously selected screen objects. */

    if ( _vpScreenObj != NULL) {

        for ( ii = 0; _vpScreenObj[ ii] != NULL; ii++) {
            _pRefOverlay->RemoveLayer( _vpScreenObj[ ii]);
            fNeedFlush = TRUE;
        }
 
        FREE( _vpScreenObj);
        _vpScreenObj = NULL;

    }

/* Collect and copy the newly selected screen objects. */

    if ( _pwcToggle->GetState() && ( _nSelectedLayers > 0)) {
 
        _vpScreenObj = ( AmScreenObjClass **) malloc(
                                                  sizeof( AmScreenObjClass *) *
                                                  ( _nSelectedLayers + 1));
        if ( _vpScreenObj != NULL) {

            for ( ii = 0; ii < _nSelectedLayers; ii++) {
               
                iLayerNum = _viSelectedLayers[ ii];

                if ( iLayerNum == _nDataGraphic) { 
                    pScreenObj = _pOverlay;
                } else {
                    pScreenObj = _vpDataGraphic[ iLayerNum];
                }

                if ( pScreenObj != NULL) {
                    _vpScreenObj[ ii] = AmTraceObject( _pXWindow, pScreenObj,
                                                       _pRefOverlay);
                    _pRefOverlay->Update();
                    fNeedFlush = TRUE;
                }
            }

            _vpScreenObj[ ii] = NULL;
        }
    }

    if ( fNeedFlush == TRUE) {
        _pRefMotifDisplay->Flush();
    }
}


void
AmPickLayerDialogClass::AmSelectCallback( Widget    w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{
    AmPickLayerDialogClass *pPickLayerDialog;
    pPickLayerDialog = ( AmPickLayerDialogClass *) xpClientData;
    pPickLayerDialog->select( w, xpCallData);
}

/* ARGSUSED */
void
AmPickLayerDialogClass::select( Widget    w,
                                XtPointer xpCallData)
{
    int            *viPos, nPos, ii, iPos;

/* Remove the list of previously selected layers. */

    if ( _viSelectedLayers != NULL) {
        FREE( _viSelectedLayers);
        _viSelectedLayers = NULL;
        _nSelectedLayers  = 0;
    }

/* Call the action function on all selected objects. */

    _pwcLayerList->GetSelectedPositions( &viPos, &nPos);
 
    if ( nPos > 0) {

        _viSelectedLayers = ( int *) malloc( sizeof( int) * nPos);
        if ( _viSelectedLayers != NULL) {

            for ( ii = 0; ii < nPos; ii++) {
  
                iPos = viPos[ ii] - 1;
                
                if ( iPos == _nLayerPos) {
                    _viSelectedLayers[ ii] = _nDataGraphic;
                } else {
                    _viSelectedLayers[ ii] = _viLayerPos[ iPos];
                }
            }

            _nSelectedLayers = nPos;
        }
    }

    updateInfoBox();

    XtFree(( char *) viPos);
}


void
AmPickLayerDialogClass::AmActionCallback( Widget    w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{
    AmPickLayerDialogClass *pPickLayerDialog;
    pPickLayerDialog = ( AmPickLayerDialogClass *) xpClientData;
    pPickLayerDialog->action( w, xpCallData);
}

/* ARGSUSED */
void
AmPickLayerDialogClass::action( Widget    w,
                                XtPointer xpCallData)
{
    AmScreenObjClass *pScreenObj;
    int            *viPos, nPos, ii, iPos;

    if ( _xcpActionCallback != NULL) {

    /* Get the selected layers from the list. */

        _pwcLayerList->GetSelectedPositions( &viPos, &nPos);

    /* Call the callback function for each layer. */

        for ( ii = 0; ii < nPos; ii++) {
            iPos = viPos[ ii] - 1;          
            if ( iPos == _nLayerPos) {
                pScreenObj = _pOverlay;
            } else {
                pScreenObj = _vpDataGraphic[ _viLayerPos[ iPos]];
            }

            _xcpActionCallback( w, _xpClientData, pScreenObj);
        }

        XtFree(( char *) viPos);

    }

    delete this;
}


/* ARGSUSED */
void
AmPickLayerDialogClass::AmToggleCallback( Widget   w,
                                          XtPointer xpClientData,
                                          XtPointer xpCallData)
{
    AmPickLayerDialogClass *pPickLayerDialog;
    pPickLayerDialog = ( AmPickLayerDialogClass *) xpClientData;
    pPickLayerDialog->updateInfoBox();
}


void
AmPickLayerDialogClass::AmLayerDeletedCallback( int       iID,
                                                XtPointer xpClientData,
                                                XtPointer xpCallData)
{
    AmPickLayerDialogClass *pPickLayerDialog; 
    pPickLayerDialog = ( AmPickLayerDialogClass *) xpClientData;

    pPickLayerDialog->layerDeleted( iID, xpCallData);
}


/* ARGSUSED */
void
AmPickLayerDialogClass::layerDeleted( int       iID,
                                      XtPointer xpCallData)
{
    AmDataGraphicClass *pDataGraphic = ( AmDataGraphicClass *) xpCallData;
    int              ii;

    for ( ii = 0; ii < _nDataGraphic; ii++) {
        if ( pDataGraphic == _vpDataGraphic[ ii]) {
            updateLayerList();
            break;
        }
    }
}


void
AmPickLayerDialogClass::AmObjectDeletedCallback( int       iID,
                                                 XtPointer xpClientData,
                                                 XtPointer xpCallData)
{
    AmPickLayerDialogClass *pPickLayerDialog; 
    pPickLayerDialog = ( AmPickLayerDialogClass *) xpClientData;

    pPickLayerDialog->objectDeleted( iID, xpCallData);
}


/* ARGSUSED */
void
AmPickLayerDialogClass::objectDeleted( int       iID,
                                       XtPointer xpCallData)
{
    AmScreenObjClass *pScreenObj = ( AmScreenObjClass *) xpCallData;
 
    if ( pScreenObj == _pOverlay) {
        delete this;
    }
}
