#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/CascadeB.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <anvil/motif.h>
#include <anvil/slick.h>
#include <anvil/tagstrs.h>
#include <anvil/macros.h>
#include <anvil/userevents.h>
#include <anvil/va_cback.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/errlib.h>
#include <anvil/utility.h>
#include <editor/editorP.h>

#define SCREEN_SIZE 2200

typedef struct printDialogData_ {
     Widget		area;
     printInfoPtr	printInfo;
     float		widthInches;
     float		heightInches;
     Widget		horizRuler;
     Widget		vertRuler;
     Widget 		outputTypeMenu;
     Widget		printerMenu;
     Widget		scaleMenu;
     Widget		landscapeTgl;
     AnvilBoolean		lastLandscape;
} printDialogDataRec, *printDialogDataPtr;

/* for the page size option menu */

typedef struct pageSize_ {
    char	*name;
    int		pageWidth;
    int		pageHeight;
} pageSizeRec, *pageSizePtr;

pageSizeRec pageSizes[] = {
    { "A: 8.5x11", 612, 792 },
    { "B: 11x17", 792, 1224 },
    { "C: 17x24", 1224, 1728 },
    { "D: 24x36", 1728, 2592 },
    { "E: 36x48", 2592, 3456 }
};

/* for the screen scale option menu */

typedef struct screenScale_ {
    char	*name;
    int		denominator;
    int		numerator;
} screenScaleRec, *screenScalePtr;

screenScaleRec screenScales[] = {
    { "3:1", 3, 1 },
    { "2:1", 2, 1 },
    { "3:2", 3, 2 },
    { "1:1", 1, 1 },
    { "2:3", 2, 3 },
    { "1:2", 1, 2 },
    { "1:3", 1, 3 },
    { "1:4", 1, 4 },
};

char *outputNames[] = { "PostScript", "TIFF" };
AmOutputTypeEnum outputTypes[] = { AmPOSTSCRIPT, AmTIFF };

static AmActionDefRec printActionAreaDef[] = {
    { "okBtn", NULL, NULL },
    { "cancelBtn",  NULL, NULL }
};


/* set the two page width and height text fields on the option menu choice */

static void setPageTextCallback( Widget		pageSizeBtn,
				 Widget		widthText,
				 Widget		heightText,
				 XtPointer)
{
    pageSizePtr		pageSizeData;
    char		buffer[ 256];

    XtVaGetValues( pageSizeBtn, XmNuserData, &pageSizeData, NULL);
    sprintf( buffer, "%f", pageSizeData->pageWidth/72.0);
    XmTextFieldSetString( widthText, buffer);
    sprintf( buffer, "%f", pageSizeData->pageHeight/72.0);
    XmTextFieldSetString( heightText, buffer);
}

/* convert the inches for the setPage to points */

static void setPageCallback( int		eventId,
			     printDialogDataPtr	printDialogData,
			     XtPointer)
{
    printInfoPtr	printInfo = printDialogData->printInfo;
    Dimension		oldWidth, oldHeight, areaWidth, areaHeight;
    Widget		outputBtn, printerBtn, scaleBtn;
    XmString		mstr;
    char		*prologFile;
    AnvilBoolean		newLandscape;
    int			pixels;
    screenScalePtr	screenScale;

/* get the width of the drawing area, selected output type, the selected 
   printer, and landscape toggle.  The prolog file for the printer is stored 
   in the client data in the printer buttons ( in the option menu) */

    XtVaGetValues( printDialogData->area, 
		   XmNwidth, &oldWidth, 
		   XmNheight, &oldHeight, 
		   NULL);
    XtVaGetValues( printDialogData->outputTypeMenu, 
		   XmNmenuHistory, &outputBtn, 
		   NULL);
    XtVaGetValues( outputBtn, 
		   XmNuserData, &printInfo->outputType, 
		   NULL);
    XtVaGetValues( printDialogData->printerMenu, 
		   XmNmenuHistory, &printerBtn, 
		   NULL);
    XtVaGetValues( printDialogData->scaleMenu, 
		   XmNmenuHistory, &scaleBtn, 
		   NULL);
    if ( printInfo->printerName != NULL) {
	free( printInfo->printerName);
    }
    if ( printInfo->prologFile != NULL) {
	free( printInfo->prologFile);
    }
    XtVaGetValues( printerBtn, 
		   XmNlabelString, &mstr, 
		   XmNuserData, &prologFile,
		   NULL);
    XtVaGetValues( scaleBtn,
		   XmNuserData, &screenScale,
		   NULL);
    XmStringGetLtoR( mstr, XmFONTLIST_DEFAULT_TAG, &printInfo->printerName);
    printInfo->prologFile = strdup( prologFile);
    printInfo->landscape = 
	XmToggleButtonGetState( printDialogData->landscapeTgl);

/* swap the width and height if the landscape toggle is changed */

    if ( printInfo->landscape != printDialogData->lastLandscape) {
	printInfo->pageWidth = printDialogData->heightInches*BASE_DPI;
	printInfo->pageHeight = printDialogData->widthInches*BASE_DPI;
    } else {
	printInfo->pageWidth = printDialogData->widthInches*BASE_DPI;
	printInfo->pageHeight = printDialogData->heightInches*BASE_DPI;
    }

/* set the screen size and save it in the print info structure. We do stuff
   in integer for exact scaling */

    areaWidth = ( printInfo->pageWidth*screenScale->numerator)/
			screenScale->denominator;
    areaHeight = ( printInfo->pageHeight*screenScale->numerator)/
			screenScale->denominator;
    AN_DEBUG( 20, ("area width = %d, height = %d\n", areaWidth, areaHeight));
    printInfo->screenWidth = areaWidth;
    printInfo->screenHeight = areaHeight;
    XtVaSetValues( printDialogData->area, 
		   XmNwidth, areaWidth,
		   XmNheight, areaHeight,
		   NULL);

/* cause the rulers to be redrawn */

    XClearArea( XtDisplay( printDialogData->horizRuler), 
		XtWindow( printDialogData->horizRuler),
		0, 0, 0, 0, TRUE);
    XClearArea( XtDisplay( printDialogData->vertRuler), 
		XtWindow( printDialogData->vertRuler),
		0, 0, 0, 0, TRUE);
}

/* read the file containing the different printer types and printer
   prolog files */

static AmErrorCode readPrinterFile( char	*filename,
				    char	***nameVec,
				    char	***prologVec)
{
    char	buffer[ PATHLEN], printerName[ TEXTLEN], prologName[ PATHLEN];
    int		length, ii;
    FILE	*fp;

    if (( length = AmNumLines( filename)) < 0) {
	DO_RETURN_ERROR( AmOPEN_FAILED);
    }
    if (( *nameVec = ( char **) 
		malloc( sizeof( char *)*( length + 1))) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    if (( *prologVec = ( char **) 
		malloc( sizeof( char *)*( length + 1))) == NULL) {
	DO_RETURN_ERROR( AmMALLOC_FAILED);
    }
    if (( fp = fopen( filename, "r")) == NULL) {
	free( *nameVec);
	free( *prologVec);
	DO_RETURN_ERROR( AmOPEN_FAILED);
    }

/* printer file format is "printerName" prologFile.  The prolog files are
   presumed to be in the CONFIG/printers subdirectory */

    for ( ii = 0; ii < length; ii++) {
	if ( fgets( buffer, sizeof( buffer), fp) == NULL) {
	    fclose( fp);
	    DO_RETURN_ERROR( AmREAD_FAILED);
	}
	sscanf( buffer, "\"%[^\"]\" %s", printerName, prologName);
	AN_DEBUG( 99, ( "printer = %s, prolog = %s\n", 
			printerName, prologName));
	if ((( *nameVec)[ ii] = strdup( printerName)) == NULL) {
	    DO_RETURN_ERROR( AmMALLOC_FAILED);
	}
	if ((( *prologVec)[ ii] = strdup( prologName)) == NULL) {
	    DO_RETURN_ERROR( AmMALLOC_FAILED);
	}
    }
    ( *nameVec)[ length] = NULL;
    ( *prologVec)[ length] = NULL;
    fclose( fp);
    return( AmOK);
}
	

/* dialog to set the pageWidth, pageHeight, dpi and landscape parameters
   for printing */

void printSetupDialogCallback( Widget		btn,
			       Widget		area,
			       printInfoPtr	printInfo,
			       Widget		horizRuler,
			       Widget		vertRuler,
			       XtPointer)
{ 
    Widget		formDlog, pageSizeMenu, pageSizeBtn, pageSizeCasc,
			rowcol1, rowcol2, landscapeTgl, widthText, 
			heightText, outputTypeMenu, 
			outputTypeCasc, outputTypeBtn, actionForm,
			parent = AmGetWidgetRealShell( btn), printerMenu, 
			printerCasc, dpiScl, trueColorTgl, printerBtn,
			scaleMenu, scaleCasc, scaleBtn, bestButton;
    int			n, ii, bestWidth;
    Arg			args[ 20];
    XmString		mstr;
    char		**printerNames, **prologFiles, pathname[ PATHLEN];
    printDialogDataPtr  printDialogData;
    AmActionRec		actionArea;
    float		factor, bestFactor, candScale;

/* read the printer file */

    sprintf( pathname, "%s/%s/%s", getenv( CONFIG_ENV), AM_PRINTER_DIR,
				   AM_PRINTER_LIST_FILE);
    if ( readPrinterFile( pathname, &printerNames, &prologFiles) != AmOK) {
	AmPrintResourceError( parent, "failedToReadPrinterFile", pathname);
	return;
    }

/* create the callback structure */

    if (( printDialogData = ( printDialogDataPtr) 
		malloc( sizeof( printDialogDataRec))) == NULL) {
	AmPrintResourceError( parent, "mallocFailed");
	return;
    }
    printDialogData->area = area;
    printDialogData->printInfo = printInfo;
    printDialogData->widthInches = printInfo->pageWidth/( float) BASE_DPI;
    printDialogData->heightInches = printInfo->pageHeight/( float) BASE_DPI;
    printDialogData->horizRuler = horizRuler;
    printDialogData->vertRuler = vertRuler;
    printDialogData->lastLandscape = printInfo->landscape;

/* create a form to contain everything */

    n = AmMakeArgList( args,
		     XmNdeleteResponse, XmDESTROY,
		     XmNautoUnmanage, FALSE,
		     XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		     NULL);
    formDlog = XmCreateFormDialog( parent, "printSetup", args, n);

/* create the action area */

    actionForm = AmMakeActionArea( formDlog,
				 printActionAreaDef, 
				 ELEMENTS( printActionAreaDef),
				 &actionArea);
    XtVaSetValues( actionForm, StickLeft, StickRight, StickBottom, NULL);

/* create a rowcolumn to hold the objects */

    rowcol1 = XtVaCreateManagedWidget( "rowcol1", xmRowColumnWidgetClass,
				       formDlog, 
				       StickLeft, StickTop, 
				       XmNbottomOffset, 40,
				       XmNorientation, XmHORIZONTAL,
				       NULL);
    rowcol2 = XtVaCreateManagedWidget( "rowcol2", xmRowColumnWidgetClass,
				       formDlog, 
				       StickLeft, BelowOf( rowcol1), 
				       AboveOf( actionForm),
				       XmNorientation, XmHORIZONTAL,
				       NULL);


/* create an option menu for setting the different page sizes, but put the
   buttons in later, because they need the text widgets */

    pageSizeMenu = XmCreatePulldownMenu( rowcol1, "pageSizeMenu", NULL, 0);
    n = AmMakeArgList( args,
		     XmNsubMenuId, pageSizeMenu,
		     NULL);
    pageSizeCasc = XmCreateOptionMenu( rowcol1, "pageSizeCasc", args, n);
    XtManageChild( pageSizeCasc);

/* create widgets for the page width, height, dpi, and landscape */

    widthText = AmAddObject( rowcol1, &printDialogData->widthInches, 
			   AmNpageWidth);
    heightText = AmAddObject( rowcol1, &printDialogData->heightInches, 
			    AmNpageHeight);
    dpiScl = AmAddObject( rowcol2, &printInfo->dpi, AmNdpi);
    landscapeTgl = AmAddObject( rowcol2, &printInfo->landscape, AmNlandscape);

/* now create the buttons in the option menu */

    bestButton = NULL;
    bestWidth = HUGE;
    for ( ii = 0; ii < VECLEN( pageSizes); ii++) {
	mstr = XmStringCreateLtoR( pageSizes[ ii].name, XmFONTLIST_DEFAULT_TAG);
	pageSizeBtn = XtVaCreateManagedWidget( "pageSizeBtn",
					       xmPushButtonWidgetClass,
					       pageSizeMenu,
					       XmNlabelString, mstr,
					       XmNuserData, &pageSizes[ ii],
					       NULL);

/* figure out which button best approximates the selected page size */

	if ( printInfo->landscape) {
	    if ( abs( pageSizes[ ii].pageHeight - printInfo->pageWidth) <
		 abs( bestWidth - printInfo->pageWidth)) {
		bestWidth = pageSizes[ ii].pageHeight;
		bestButton = pageSizeBtn;
	    }
	} else {
	    if ( abs( pageSizes[ ii].pageWidth - printInfo->pageWidth) <
		 abs( bestWidth - printInfo->pageWidth)) {
		bestWidth = pageSizes[ ii].pageWidth;
		bestButton = pageSizeBtn;
	    }
	}
	AmAddVaCallback( pageSizeBtn, XmNactivateCallback,
		       ( VarargsCallback) setPageTextCallback, 
		       widthText, heightText, 
		       END_LIST);
	XmStringFree( mstr);
    }
    if ( bestButton != NULL) {
	XtVaSetValues( pageSizeCasc, XmNmenuHistory, bestButton, NULL);
    }

/* create an option menu for the screen scales */

    scaleMenu = XmCreatePulldownMenu( rowcol1, "scaleMenu", NULL, 0);
    n = AmMakeArgList( args, 
		     XmNsubMenuId, scaleMenu,
		     NULL);
    scaleCasc = XmCreateOptionMenu( rowcol1, "scaleCasc", args, n);
    XtManageChild( scaleCasc);
    bestButton = NULL;
    bestFactor = HUGE;
    factor = (( float) printInfo->screenWidth)/
		    (( float) printInfo->pageWidth);
AN_DEBUG( 40, ("scale factor = %.2f\n", factor));
    for ( ii = 0; ii < VECLEN( screenScales); ii++) {
	mstr = XmStringCreateLtoR( screenScales[ ii].name, 
				   XmFONTLIST_DEFAULT_TAG);
	scaleBtn = XtVaCreateManagedWidget( "scaleBtn",
					    xmPushButtonWidgetClass,
					    scaleMenu,
					    XmNlabelString, mstr,
					    XmNuserData, &screenScales[ ii],
					    NULL);
	candScale = (( float) screenScales[ ii].numerator)/
			(( float) screenScales[ ii].denominator);

/* figure out which scaling factor best approximates the screen to page
   size ratio */

AN_DEBUG( 40, ("scale factor = %.2f\n", candScale));
	if ( fabs( candScale - factor) < fabs( bestFactor - factor)) {
	    bestFactor = candScale;
	    bestButton = scaleBtn;
	}
    }
    if ( bestButton != NULL) {
	XtVaSetValues( scaleCasc, XmNmenuHistory, bestButton, NULL);
    }

/* create an option menu for the different output types. */

    outputTypeMenu = XmCreatePulldownMenu( rowcol2, "outputTypeMenu", NULL, 0);
    n = AmMakeArgList( args, 
		     XmNsubMenuId, outputTypeMenu,
		     NULL);
    outputTypeCasc = XmCreateOptionMenu( rowcol2, "outputTypeCasc", args, n);
    XtManageChild( outputTypeCasc);
    for ( ii = 0; ii < VECLEN( outputNames); ii++) {
	mstr = XmStringCreateLtoR( outputNames[ ii], XmFONTLIST_DEFAULT_TAG);
	outputTypeBtn = XtVaCreateManagedWidget( "outputTypeBtn",
						 xmPushButtonWidgetClass,
						 outputTypeMenu,
						 XmNlabelString, mstr,
						 XmNuserData, outputTypes[ ii],
						 NULL);
	XmStringFree( mstr);
	if ( outputTypes[ ii] == printInfo->outputType) {
	    XtVaSetValues( outputTypeMenu, XmNmenuHistory, outputTypeBtn, NULL);
	}
    }

/* true color toggle in the next row */

    trueColorTgl = AmAddObject( rowcol2, &printInfo->trueColor, AmNtrueColor);

/* create the menu of printers and prolog files */

    printerMenu = XmCreatePulldownMenu( rowcol2, "printerMenu", NULL, 0);
    n = AmMakeArgList( args, 
		     XmNsubMenuId, printerMenu,
		     NULL);
    printerCasc = XmCreateOptionMenu( rowcol2, "printerCasc", args, n);
    XtManageChild( printerCasc);
    for ( ii = 0; ii < AmVeclen( printerNames); ii++) {
	mstr = XmStringCreateLtoR( printerNames[ ii], XmFONTLIST_DEFAULT_TAG);
	printerBtn = XtVaCreateManagedWidget( "printerBtn",
					      xmPushButtonWidgetClass,
					      printerMenu,
					      XmNlabelString, mstr,
					      XmNuserData, prologFiles[ ii],
					      NULL);
	if ( !strcmp( printerNames[ ii], printInfo->printerName)) {
	    XtVaSetValues( printerMenu, XmNmenuHistory, printerBtn, NULL);
	}
	XmStringFree( mstr);
    }

/* fill the rest of the callback structure with the widgets created in the
   dialog */

    printDialogData->outputTypeMenu = outputTypeMenu;
    printDialogData->printerMenu = printerMenu;
    printDialogData->scaleMenu = scaleMenu;
    printDialogData->landscapeTgl = landscapeTgl;

/* create the OK and cancel buttons */

    XtAddCallback( actionArea.buttons[ 1], XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
    AmAddCallbackWidget( AM_PKG_ID, AM_FINAL, formDlog, 
		       ( callbackProc) setPageCallback, printDialogData);

/* set up the callbacks to free the temporary string vectors when the dialog
   is popped down */

    XtAddCallback( formDlog, XmNdestroyCallback,
		   ( XtCallbackProc) AmFreeStrvecCallback, printerNames);
    XtAddCallback( formDlog, XmNdestroyCallback,
		   ( XtCallbackProc) AmFreeStrvecCallback, prologFiles);
    XtAddCallback( formDlog, XmNdestroyCallback,
		   ( XtCallbackProc) AmFreeStructPtrCallback, printDialogData);
    AmSetOkButton( actionArea.buttons[ 0]);
    XtManageChild( formDlog);
}
