#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/cursorfont.h>

#include <anvil/motifdisplay.h>
#include <anvil/state_hand.h>
#include <anvil/userevents.h>

#include <editor/handlers.h>

const int AmACTION_RESIZE_PRIORITY = 1000;
const int AmMIN_SIZE = 5;

typedef struct resizeHandlerArgs_ {
    AmEditorClass        *pEditorClass;
    AmResizeOutlineClass *pOutline;
    AmActionEnum         eAction;
    int                xStart;
    int                yStart;
    int                iWidthStart;
    int                iHeightStart;
    int                x;
    int                y;
    int                iWidth;
    int                iHeight;
    int                xOffset;
    int                yOffset;
} AmResizeHandlerArgsRec, *AmResizeHandlerArgsPtr;

void
AmResizeHandler( Widget               widget,
               AmResizeHandlerArgsPtr pHandlerArgs,
               XEvent               *pxEvent,
               AnvilBoolean              *);

void
AmResizeObjects( AmResizeHandlerArgsPtr pHandlerArgs,
               int                  iWidth,
               int                  iHeight)
{
    AmEditorClass        *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass  *pMotifDisplay = pEditorClass->GetMotifDisplay();
    AmScreenObjClass     *pScreenObj;
    AmSelectObjClass     *pSelectObj;
    AmObjectOutlineClass *pOutline;
    const AmRectangleRec *pRec;
    AmListPtr            lstTemp;
    float              dWidthScale, dHeightScale;
    int                xStart, yStart, x, y, xNew, yNew, wNew, hNew;

    xStart = pHandlerArgs->xStart;
    yStart = pHandlerArgs->yStart;
    x = pHandlerArgs->x;
    y = pHandlerArgs->y;

    dWidthScale = ( float) pHandlerArgs->iWidth /
                  ( float) pHandlerArgs->iWidthStart;
    dHeightScale = ( float) pHandlerArgs->iHeight /
                   ( float) pHandlerArgs->iHeightStart;

    for ( lstTemp = pEditorClass->GetSelectedObjects();
          lstTemp != NULL;
          lstTemp = lstTemp->next) {

        pSelectObj = ( AmSelectObjClass *) lstTemp->data;
        pScreenObj = pSelectObj->GetScreenObj();
        pOutline = pSelectObj->GetOutline();

        pRec = pScreenObj->GetRect();

        xNew = ( int) ( x + (pRec->x - xStart) * dWidthScale + 0.5);
        yNew = ( int) ( y + (pRec->y - yStart) * dHeightScale + 0.5);
        wNew = ( int) ( pRec->width * dWidthScale + 0.5);
        hNew = ( int) ( pRec->height * dHeightScale + 0.5);

        pScreenObj->SetRectAndMargins( xNew, yNew, wNew, hNew,
                                       iWidth, iHeight);
        pOutline->SetRectangle( xNew, yNew, wNew, hNew);
    }

    pMotifDisplay->Flush();
}

/*********************************************************************
 *
 * Resize Handler Stuff:
 *
 *********************************************************************/

AmResizeHandlerArgsPtr
AmMakeResizeHandlerArgs( AmEditorClass *pEditorClass,
                       AmActionEnum  eAction,
                       int         x,
                       int         y,
                       int         iWidth,
                       int         iHeight,
                       int         xOffset,
                       int         yOffset)
{
    AmResizeHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = ( AmResizeHandlerArgsPtr) malloc(
                       sizeof( AmResizeHandlerArgsRec));

    pHandlerArgs->pEditorClass = pEditorClass;
    pHandlerArgs->eAction = eAction;
    pHandlerArgs->xStart = x;
    pHandlerArgs->yStart = y;
    pHandlerArgs->iWidthStart = iWidth;
    pHandlerArgs->iHeightStart = iHeight;
    pHandlerArgs->x = x;
    pHandlerArgs->y = y;
    pHandlerArgs->iWidth = iWidth;
    pHandlerArgs->iHeight = iHeight;
    pHandlerArgs->xOffset = xOffset;
    pHandlerArgs->yOffset = yOffset;

    return( pHandlerArgs);
}


void
AmRemoveResizeHandler( AmResizeHandlerArgsPtr pHandlerArgs)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay();

    pMotifDisplay->UnmapTravelCursor();
    pMotifDisplay->RemoveOutline( pHandlerArgs->pOutline);

    free( pHandlerArgs);
}


void
AmSetupResizeHandler( AmEditorClass *pEditorClass,
                    int         x,
                    int         y,
                    AmActionEnum  eAction)
{
    AmMotifDisplayClass    *pMotifDisplay = pEditorClass->GetMotifDisplay();
    Window               window = 
			    (( AmXWindowClass *) pMotifDisplay->GetWindow())->GetXWindow();
    Display              *pDisplay = 
			(( AmXDisplayClass *) pMotifDisplay->GetDisplay())->GetXDisplay();
    Widget               wArea = pMotifDisplay->GetArea();
    AmResizeHandlerArgsPtr pHandlerArgs;
    AmRectangleRec         rec;
    Dimension            dimWidth, dimHeight;
    int                  xOffset, yOffset;

    XtVaGetValues( wArea,
                   XmNwidth, &dimWidth,
                   XmNheight, &dimHeight,
                   NULL);

    pEditorClass->GetSelectedObjectsExtents( &rec);

    switch( eAction) {
        case AmACTION_RESIZE_UP_LEFT:
            x = rec.x;
            y = rec.y;
            break;

        case AmACTION_RESIZE_UP:
            x = rec.x + rec.width / 2;
            y = rec.y;
            break;

        case AmACTION_RESIZE_UP_RIGHT:
            x = rec.x + rec.width;
            y = rec.y;
            break;

        case AmACTION_RESIZE_RIGHT:
            x = rec.x + rec.width;
            y = rec.y + rec.height / 2;
            break;

        case AmACTION_RESIZE_LEFT:
            x = rec.x;
            y = rec.y + rec.height / 2;
            break;

        case AmACTION_RESIZE_DOWN_LEFT:
            x = rec.x;
            y = rec.y + rec.height;
            break;

        case AmACTION_RESIZE_DOWN:
            x = rec.x + rec.width / 2;
            y = rec.y + rec.height;
            break;

        case AmACTION_RESIZE_DOWN_RIGHT:
            x = rec.x + rec.width;
            y = rec.y + rec.height;
            break;

        default:
            break;
    }

    XWarpPointer( pDisplay, window, window,
                  0, 0, dimWidth, dimHeight,
                  pMotifDisplay->GetDisplay()->ScreenCoord( x),
                  pMotifDisplay->GetDisplay()->ScreenCoord( y));

    xOffset = 0;
    yOffset = 0;

    pHandlerArgs = AmMakeResizeHandlerArgs( pEditorClass, eAction,
                                          rec.x, rec.y,
                                          rec.width, rec.height,
                                          xOffset, yOffset);

    pHandlerArgs->pOutline = new AmResizeOutlineClass( pMotifDisplay,
                                                     rec.x, rec.y,
                                                     rec.width, rec.height);
    pMotifDisplay->AddOutline( pHandlerArgs->pOutline);

    AmAddHandler( NULL, wArea, 
                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                ( XtEventHandler) AmResizeHandler,
                pHandlerArgs, AmACTION_RESIZE_PRIORITY, 
                ( AmStateHandlerFn) AmRemoveResizeHandler);

    pMotifDisplay->WriteResourceMessage( "resizeMsg");
    pMotifDisplay->MapTravelCursor();
    pMotifDisplay->WriteResourceTravelCursor( "resizingMsg",
                                              rec.width, rec.height);
}

/************************************************************
 *
 * Resize Handler:
 *
 ************************************************************/

void
AmResizeHandler( Widget               widget,
               AmResizeHandlerArgsPtr pHandlerArgs,
               XEvent               *pxEvent,
               AnvilBoolean              *)
{
    AmEditorClass          *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass    *pMotifDisplay = pEditorClass->GetMotifDisplay();
    XButtonEvent         *pxButtonEvent;
    XMotionEvent         *pxMotionEvent;
    int                  xStart, yStart, wStart, hStart,
                         x, y, w, h, xMotion, yMotion,
                         iWidthArea, iHeightArea;
    AnvilBoolean              fSnapToGrid = pEditorClass->GetSnapToGrid();

/* Figure out the extends of the drawing area. */

    iWidthArea = pMotifDisplay->GetWidth();
    iHeightArea = pMotifDisplay->GetHeight();

    if ( pxEvent->type == ButtonPress) {

        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button3) {

        /* Button 3 cancels the operation. */

            pMotifDisplay->WriteResourceMessage( "resizeCancelMsg");

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmResizeHandler, pHandlerArgs);
        }
    } else if ( pxEvent->type == ButtonRelease) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;
        if ( pxButtonEvent->button == Button1) {

        /* Display a busy cursor */

            pMotifDisplay->SetCursor( XC_watch);
            pMotifDisplay->WriteResourceMessage( "resizeSuccessMsg");

        /*
         * Resize the screen object and then update the relevent areas
         *   on the screen.
         */

            AmResizeObjects( pHandlerArgs, iWidthArea, iHeightArea);

            pEditorClass->SetMapChanged( TRUE);

            AmRemoveHandler( widget,
                           ButtonPressMask | ButtonReleaseMask |
                           PointerMotionMask,
                           ( XtEventHandler) AmResizeHandler, pHandlerArgs);
        }
    } else if ( pxEvent->type == MotionNotify) {
        pxMotionEvent = ( XMotionEvent *) pxEvent;

    /*
     * Depending on what type of resizing is being done... figure out
     *   the extends of the rectangle.
     */

        xMotion = pxMotionEvent->x;
        yMotion = pxMotionEvent->y;

        xMotion = pMotifDisplay->GetDisplay()->VirtualCoord( xMotion);
        yMotion = pMotifDisplay->GetDisplay()->VirtualCoord( yMotion);

        xMotion = xMotion - pHandlerArgs->xOffset;
        yMotion = yMotion - pHandlerArgs->yOffset;

        if ( fSnapToGrid == TRUE) {
            xMotion = pMotifDisplay->GetGridLocation( xMotion);
            yMotion = pMotifDisplay->GetGridLocation( yMotion);
        }

        xStart = pHandlerArgs->xStart;
        yStart = pHandlerArgs->yStart;
        wStart = pHandlerArgs->iWidthStart;
        hStart = pHandlerArgs->iHeightStart,

        x = pHandlerArgs->x;
        y = pHandlerArgs->y;
        w = pHandlerArgs->iWidth;
        h = pHandlerArgs->iHeight;

        switch( pHandlerArgs->eAction) {
            case AmACTION_RESIZE_UP_LEFT:
                x = xMotion;
                y = yMotion;
                if ( x > ( xStart + wStart - AmMIN_SIZE)) {
                    x = xStart + wStart - AmMIN_SIZE;
                } else if ( x < 0) {
                    x = 0;
                }
                if ( y > ( yStart + hStart - AmMIN_SIZE)) {
                    y = yStart + hStart - AmMIN_SIZE;
                } else if ( y < 0) {
                    y = 0;
                }
                w = xStart + wStart - x;
                h = yStart + hStart - y;
                break;
            case AmACTION_RESIZE_UP:
                x = xStart;
                y = yMotion;
                if ( y > ( yStart + hStart - AmMIN_SIZE)) {
                    y = yStart + hStart - AmMIN_SIZE;
                } else if ( y < 0) {
                    y = 0;
                }
                w = wStart;
                h = yStart + hStart - y;
                break;
            case AmACTION_RESIZE_UP_RIGHT:
                x = xStart;
                y = yMotion;
                if ( y > ( yStart + hStart - AmMIN_SIZE)) {
                    y = yStart + hStart - AmMIN_SIZE;
                } else if ( y < 0) {
                    y = 0;
                }
                w = xMotion - x;
                if ( w < AmMIN_SIZE) {
                    w = AmMIN_SIZE;
                } else if ( x + w > iWidthArea) {
                    w = iWidthArea - x;
                }
                h = yStart + hStart - y;
                break;
            case AmACTION_RESIZE_LEFT:
                x = xMotion;
                y = yStart;
                if ( x > ( xStart + wStart - AmMIN_SIZE)) {
                    x = xStart + wStart - AmMIN_SIZE;
                } else if ( x < 0) {
                    x = 0;
                }
                w = xStart + wStart - x;
                h = hStart;
                break;
            case AmACTION_RESIZE_RIGHT:
                x = xStart;
                y = yStart;
                w = xMotion - x;
                if ( w < AmMIN_SIZE) {
                    w = AmMIN_SIZE;
                } else if ( x + w > iWidthArea) {
                    w = iWidthArea - x;
                }
                h = hStart;
                break;
            case AmACTION_RESIZE_DOWN_LEFT:
                x = xMotion;
                y = yStart;
                if ( x > ( xStart + wStart - AmMIN_SIZE)) {
                    x = xStart + wStart - AmMIN_SIZE;
                } else if ( x < 0) {
                    x = 0;
                }
                w = xStart + wStart - x;
                h = yMotion - y;
                if ( h < AmMIN_SIZE) {
                    h = AmMIN_SIZE;
                } else if ( y + h > iHeightArea) { 
                    h = iHeightArea - y;
                }
                break;
            case AmACTION_RESIZE_DOWN:
                x = xStart;
                y = yStart;
                w = wStart;
                h = yMotion - y;
                if ( h < AmMIN_SIZE) {
                    h = AmMIN_SIZE;
                } else if ( y + h > iHeightArea) {
                    h = iHeightArea - y;
                }
                break;
            case AmACTION_RESIZE_DOWN_RIGHT:
                x = xStart;
                y = yStart;
                w = xMotion - x;
                if ( w < AmMIN_SIZE) {
                    w = AmMIN_SIZE;
                } else if ( x + w > iWidthArea) {
                    w = iWidthArea - x;
                }
                h = yMotion - y;
                if ( h < AmMIN_SIZE) {
                    h = AmMIN_SIZE;
                } else if ( y + h > iHeightArea) {
                    h = iHeightArea - y;
                }
                break;
            default:
                break;
        }

    /* Update the outline */

        pHandlerArgs->x = x;
        pHandlerArgs->y = y;
        pHandlerArgs->iWidth = w;
        pHandlerArgs->iHeight = h;

        pHandlerArgs->pOutline->SetRectangle( x, y, w, h);
        pMotifDisplay->WriteTravelCursor( "Size: %d, %d", w, h);

    }
}
