#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <X11/cursorfont.h>

#include <anvil/defaults.h>
#include <anvil/motifdisplay.h>
#include <anvil/state_hand.h>
#include <anvil/userevents.h>

#include <editor/handlers.h>

const int AmACTION_SECONDARY_PRIORITY = 1000;
const int AmMAX_MOTION = 5;

typedef struct secondaryHandlerArgs_ {
    AmEditorClass    *pEditorClass;
    AmScreenObjClass *pScreenObj;
    int              x;
    int              y;
    AmActionEnum     eAction;
    XtIntervalId     xTimerId;
    AnvilBoolean          fTimerActive;
} AmSecondaryHandlerArgsRec, *AmSecondaryHandlerArgsPtr;

void
AmSecondaryHandler( Widget                    widget,
                    AmSecondaryHandlerArgsPtr pHandlerArgs,
                    XEvent                    *pxEvent,
                    AnvilBoolean                   *);

AmActionEnum
AmLaunchHandler( AmEditorClass *pEditorClass,
                 int         x,
                 int         y,
                 AmActionEnum  eAction)
{

    switch( eAction) {
        case AmACTION_RESIZE_UP_LEFT:
        case AmACTION_RESIZE_UP:
        case AmACTION_RESIZE_UP_RIGHT:
        case AmACTION_RESIZE_LEFT:
        case AmACTION_RESIZE_RIGHT:
        case AmACTION_RESIZE_DOWN_LEFT:
        case AmACTION_RESIZE_DOWN:
        case AmACTION_RESIZE_DOWN_RIGHT:
            AmSetupResizeHandler( pEditorClass, x, y, eAction);
            break;

        case AmACTION_MOVE:
            AmSetupMoveHandler( pEditorClass, x, y);
            break;

         default:
            eAction = AmACTION_NONE;
            break;
     }

    return( eAction);
}


/* ARGSUSED */
void
AmPrimaryTimeout( AmSecondaryHandlerArgsPtr pHandlerArgs,
                  XtPointer               xpCallData)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    AmActionEnum        eAction = pHandlerArgs->eAction;
    Widget              wArea = pMotifDisplay->GetArea();
    int                 x = pHandlerArgs->x,
                        y = pHandlerArgs->y;

    AmLaunchHandler( pEditorClass, x, y, eAction);

    AmRemoveHandler( wArea,
                     ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                     ( XtEventHandler) AmSecondaryHandler, pHandlerArgs);
}


void
AmSecondaryTimeout( AmSecondaryHandlerArgsPtr pHandlerArgs,
                    XtPointer               xpCallData)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    Widget              wArea = pMotifDisplay->GetArea();

    AmRemoveHandler( wArea,
                     ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                     ( XtEventHandler) AmSecondaryHandler, pHandlerArgs);
}

AmSecondaryHandlerArgsPtr
AmMakeSecondaryHandlerArgs( AmEditorClass    *pEditorClass,
                            int            x,
                            int            y,
                            AmActionEnum     eAction,
                            AmScreenObjClass *pScreenObj)
{
    AmSecondaryHandlerArgsPtr pHandlerArgs;

    pHandlerArgs = ( AmSecondaryHandlerArgsPtr)
                       malloc( sizeof( AmSecondaryHandlerArgsRec));

    pHandlerArgs->pEditorClass = pEditorClass;
    pHandlerArgs->pScreenObj = pScreenObj;
    pHandlerArgs->eAction = eAction;
    pHandlerArgs->x = x;
    pHandlerArgs->y = y;

    return( pHandlerArgs);
}


void
AmRemoveSecondaryHandler( AmSecondaryHandlerArgsPtr pHandlerArgs)
{
    if ( pHandlerArgs->fTimerActive == TRUE) { 
        XtRemoveTimeOut( pHandlerArgs->xTimerId);
        pHandlerArgs->fTimerActive = FALSE;
    }

    free( pHandlerArgs);
}

void
AmSetupSecondaryHandler( AmEditorClass    *pEditorClass,
                         AmScreenObjClass *pScreenObj,
                         int            x,
                         int            y,
                         AmActionEnum     eAction)
{
    AmMotifDisplayClass       *pMotifDisplay = pEditorClass->GetMotifDisplay();
    Widget                    wArea = pMotifDisplay->GetArea();
    AmSecondaryHandlerArgsPtr pHandlerArgs;
    XtIntervalId              xTimerId;

    pHandlerArgs = AmMakeSecondaryHandlerArgs( pEditorClass, x, y,
                                               eAction, pScreenObj);

    xTimerId = XtAppAddTimeOut( XtWidgetToApplicationContext( wArea),
                                AmCLICK_TIMEOUT,
                                ( XtTimerCallbackProc) AmPrimaryTimeout,
                                pHandlerArgs);
    pHandlerArgs->xTimerId = xTimerId;
    pHandlerArgs->fTimerActive = TRUE;

    AmAddHandler( NULL, wArea,
                  ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                  ( XtEventHandler) AmSecondaryHandler,
                  pHandlerArgs, AmACTION_SECONDARY_PRIORITY,
                  ( AmStateHandlerFn) AmRemoveSecondaryHandler);
}

/************************************************************
 *
 * Secondary Handler:
 *
 ************************************************************/

void
AmSecondaryHandler( Widget                  widget,
                    AmSecondaryHandlerArgsPtr pHandlerArgs,
                    XEvent                  *pxEvent,
                    AnvilBoolean                 *)
{
    AmEditorClass       *pEditorClass = pHandlerArgs->pEditorClass;
    AmMotifDisplayClass *pMotifDisplay = pEditorClass->GetMotifDisplay(); 
    AmScreenObjClass    *pScreenObj = pHandlerArgs->pScreenObj;
    AmActionEnum        eAction = pHandlerArgs->eAction;
    XButtonEvent        *pxButtonEvent;
    XMotionEvent        *pxMotionEvent;
    XtIntervalId        xTimerId = pHandlerArgs->xTimerId;
    int                 x = pHandlerArgs->x,
                        y = pHandlerArgs->y,
                        xDelta, yDelta;

    if ( pxEvent->type == ButtonPress) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;

        if ( pxButtonEvent->button == Button3) {

        /* Cancel the timer. */

            if ( pHandlerArgs->fTimerActive == TRUE) { 
                XtRemoveTimeOut( xTimerId);
                pHandlerArgs->fTimerActive = FALSE;
            }

        /* Cancel whatever operation was going to happen. */

            pEditorClass->UnselectAll();

            AmRemoveHandler( widget,
                             ButtonPressMask | ButtonReleaseMask |
                             PointerMotionMask,
                             ( XtEventHandler) AmSecondaryHandler,
                             pHandlerArgs);

        } else if ( pxButtonEvent->button == Button1) {

        /* Cancel the timer. */

            if ( pHandlerArgs->fTimerActive == TRUE) { 
                XtRemoveTimeOut( xTimerId);
                pHandlerArgs->fTimerActive = FALSE;
            }

            AmRemoveHandler( widget,
                             ButtonPressMask | ButtonReleaseMask |
                             PointerMotionMask,
                             ( XtEventHandler) AmSecondaryHandler,
                             pHandlerArgs);

            pEditorClass->DoubleClick( pScreenObj);

        /*
         * If the double click function creates a modal dialog, we want
         *   that request honored before any more events are processed.
         */

	    XSync( XtDisplay( pMotifDisplay->GetArea()), TRUE);
        }
    } else if ( pxEvent->type == ButtonRelease) {
        pxButtonEvent = ( XButtonEvent *) pxEvent;

        if ( pxButtonEvent->button == Button1) {

        /* Cancel the timer. */

            if ( pHandlerArgs->fTimerActive == TRUE) { 
                XtRemoveTimeOut( xTimerId);
                pHandlerArgs->fTimerActive = FALSE;
            }

        /* Launch the second timer. */

            xTimerId = XtAppAddTimeOut(
                           XtWidgetToApplicationContext( widget),
                           AmCLICK_TIMEOUT,
                           ( XtTimerCallbackProc) AmSecondaryTimeout,
                           pHandlerArgs);

            pHandlerArgs->xTimerId = xTimerId;
            pHandlerArgs->fTimerActive = TRUE;
            pHandlerArgs->eAction = AmACTION_NONE;
        }
    } else if ( pxEvent->type == MotionNotify) {
        pxMotionEvent = ( XMotionEvent *) pxEvent;

        xDelta = pxMotionEvent->x - x;
        yDelta = pxMotionEvent->y - y;

        if ( xDelta * xDelta + yDelta * yDelta > AmMAX_MOTION * AmMAX_MOTION) {

        /* Cancel the timer. */

            if ( pHandlerArgs->fTimerActive == TRUE) { 
                XtRemoveTimeOut( xTimerId);
                pHandlerArgs->fTimerActive = FALSE;
            }

        /* Launch the correct handler and lemove the secondary handler. */

            AmLaunchHandler( pEditorClass, x, y, eAction);
            AmRemoveHandler( widget,
                             ButtonPressMask | ButtonReleaseMask |
                             PointerMotionMask,
                             ( XtEventHandler) AmSecondaryHandler,
                             pHandlerArgs);
        }
    }
}

