#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/ToggleB.h>

#include <anvil/macros.h>
#include <anvil/events.h>
#include <anvil/motif.h>

#include <editor/TextDialogClass.h>

typedef struct FontSizeRec_ {
    char *szName;
    int  iSize;
} FontSizeRec, *FontSizePtr;

static FontSizeRec pFontSize[] = {
    { "6", 6 },
    { "8", 8 },
    { "10", 10 },
    { "12", 12 },
    { "14", 14 },
    { "18", 18 },
    { "24", 24 },
    { "32", 32 },
    { "36", 36 },
    { "48", 48 },
    { "64", 64 },
    { "96", 96 }
};

void
AmTextDialogClass::initializeValues()
{
    _iMargin = 5;
    _szText = NULL;
    _szName = NULL; 
    _pszFamily = NULL;
}


void
AmTextDialogClass::createInterface()
{
    Colormap   colormap;
    XColor     xColor;
    Dimension  dimWidth;
    int        iForeground, iBackground, ii;

    _pwcFrame = new AmFrameClass( _wForm, "frame", FALSE);
    _pwcFrame->AttachLeft();
    _pwcFrame->AttachTop();
    _pwcFrame->AttachRight();

    _pwcFrameForm = new AmFormClass( _pwcFrame, "frameForm");

    _pwcTextForm = new AmFormClass( _pwcFrameForm, "textForm");
    _pwcTextForm->AttachLeft();
    _pwcTextForm->AttachTop();
    _pwcTextForm->AttachRight();

    _pwcTextLabel = new AmLabelClass( _pwcTextForm, "textLbl");
    _pwcTextLabel->AttachLeft();
    _pwcTextLabel->AttachTop();
    _pwcTextLabel->AttachBottom();

    _pwcTextTextField = new AmTextFieldClass( _pwcTextForm, "textText");
    _pwcTextTextField->AttachLeft( _pwcTextLabel);
    _pwcTextTextField->AttachTop();
    _pwcTextTextField->AttachRight();
    _pwcTextTextField->AttachBottom();

    _pwcFamilyForm = new AmFormClass( _pwcFrameForm, "familyForm");
    _pwcFamilyForm->AttachLeft();
    _pwcFamilyForm->AttachTop( _pwcTextForm);
    _pwcFamilyForm->AttachRight();

    _pwcFamilyLabel = new AmLabelClass( _pwcFamilyForm, "familyLbl");
    _pwcFamilyLabel->AttachLeft();
    _pwcFamilyLabel->AttachTop();
    _pwcFamilyLabel->AttachBottom();

    _pwcFamilyOptionMenu = new AmComboBoxClass( _pwcFamilyForm, "familyOption");
    XtVaSetValues( *_pwcFamilyOptionMenu,
                   RightOf( *_pwcFamilyLabel),
                   StickTop, StickRight, StickBottom,
                   NULL);

    _pszFamily = AmGetFontFamilies();
    for ( ii = 0; _pszFamily[ ii] != NULL; ii++) {
        _pwcFamilyOptionMenu->AddButton( _pszFamily[ ii], NULL, NULL);
    }

    _pwcFamilyOptionMenu->SetCallback(( XtCallbackProc) AmUpdateFamilyCallback, 
                                      ( XtPointer) this);

    _pwcSizeForm = new AmFormClass( _pwcFrameForm, "sizeForm");
    _pwcSizeForm->AttachLeft();
    _pwcSizeForm->AttachTop( _pwcFamilyForm);
    _pwcSizeForm->AttachRight();

    _pwcSizeLabel = new AmLabelClass( _pwcSizeForm, "sizeLbl");
    _pwcSizeLabel->AttachLeft(); 
    _pwcSizeLabel->AttachTop(); 
    _pwcSizeLabel->AttachBottom(); 

    _pwcSizeTextField = new AmTextFieldClass( _pwcSizeForm, "sizeText");
    _pwcSizeTextField->AttachLeft( _pwcSizeLabel); 
    _pwcSizeTextField->AttachTop(); 
    _pwcSizeTextField->AttachBottom(); 

    _pwcSizeOptionMenu = new AmComboBoxClass( _pwcSizeForm, "sizeOption");
    XtVaSetValues( *_pwcSizeOptionMenu,
                   RightOf( *_pwcSizeTextField), StickTop,
                   StickRight, StickBottom,
                   NULL);

    for ( ii = 0; ii < (signed)VECLEN( pFontSize); ii++) {
        _pwcSizeOptionMenu->AddButton( pFontSize[ ii].szName, NULL, NULL);
    }
    _pwcSizeOptionMenu->AddButton( "Custom", NULL, NULL);

    _pwcSizeOptionMenu->SetCallback(( XtCallbackProc) AmUpdateSizeCallback, 
                                    ( XtPointer) this);

    _pwcStyleForm = new AmFormClass( _pwcFrameForm, "styleForm");
    _pwcStyleForm->AttachTop( _pwcSizeForm);
    _pwcStyleForm->AttachLeft();
    _pwcStyleForm->AttachBottom();
    _pwcStyleForm->AttachRight();

    XtVaSetValues( *_pwcStyleForm, XmNfractionBase, 4, NULL);

    _pwcBoldToggle = new AmToggleButtonClass( _pwcStyleForm, "boldTgl",
                                            ( XtCallbackProc)
                                                AmToggleBoldCallback,
                                            ( XtPointer) this);
    _pwcBoldToggle->AttachTop();
    _pwcBoldToggle->AttachBottom();

    XtVaGetValues( *_pwcBoldToggle,
                   XmNwidth, &dimWidth,
                   NULL);

    XtVaSetValues( *_pwcBoldToggle,
                   XmNleftAttachment, XmATTACH_POSITION,
                   XmNleftPosition, 1,
                   XmNleftOffset, - dimWidth / 2,
                   NULL); 

    _pwcItalicToggle= new AmToggleButtonClass( _pwcStyleForm, "italicTgl",
                                            ( XtCallbackProc)
                                                AmToggleItalicCallback,
                                            ( XtPointer) this);
    _pwcItalicToggle->AttachTop();
    _pwcItalicToggle->AttachBottom();

    XtVaGetValues( *_pwcItalicToggle,
                   XmNwidth, &dimWidth,
                   NULL);

    XtVaSetValues( *_pwcItalicToggle,
                   XmNleftAttachment, XmATTACH_POSITION,
                   XmNleftPosition, 2,
                   XmNleftOffset, - dimWidth / 2,
                   NULL); 

    _pwcUnderlineToggle = new AmToggleButtonClass( _pwcStyleForm, "underlineTgl",
                                                 NULL, NULL);
    _pwcUnderlineToggle->AttachTop();
    _pwcUnderlineToggle->AttachBottom();

    XtVaGetValues( *_pwcUnderlineToggle,
                   XmNwidth, &dimWidth,
                   NULL);

    XtVaSetValues( *_pwcUnderlineToggle,
                   XmNleftAttachment, XmATTACH_POSITION,
                   XmNleftPosition, 3,
                   XmNleftOffset, - dimWidth / 2,
                   NULL); 

    _wWindow = XtVaCreateManagedWidget( "window",
                                        xmScrolledWindowWidgetClass, _wForm,
                                        StickLeft, BelowOf( *_pwcFrame),
                                        StickRight, StickBottom,
                                        XmNscrollingPolicy, XmAUTOMATIC,
                                        XmNvisualPolicy, XmCONSTANT,
                                        NULL);

    _wDrawingArea = XtVaCreateManagedWidget( "previewArea",
                                             xmDrawingAreaWidgetClass,
                                             _wWindow, 
                                             NULL);
   
    _pXDisplay = new AmXDisplayClass( _wDrawingArea);
    _pXWindow = new AmXWindowClass( _pXDisplay, _wDrawingArea);
    _pXGC = new AmXGCClass( _pXWindow);

    _display = XtDisplay( _wDrawingArea);
    _screen = XtScreen( _wDrawingArea);

    XtVaGetValues( _wDrawingArea,
                   XmNcolormap, &colormap,
                   XmNforeground, &iForeground,
                   XmNbackground, &iBackground,
                   NULL);

    xColor.pixel = iForeground;
    XQueryColor( _display, colormap, &xColor);

    _colForeground.red = xColor.red >> 8;
    _colForeground.green = xColor.green >> 8;
    _colForeground.blue = xColor.blue >> 8;

    _pXDisplay->AllocColor( &_colForeground);

    xColor.pixel = iBackground;
    XQueryColor( _display, colormap, &xColor);

    _colBackground.red = xColor.red >> 8;
    _colBackground.green = xColor.green >> 8;
    _colBackground.blue = xColor.blue >> 8;

    _pXDisplay->AllocColor( &_colBackground);

    _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);

    XtAddCallback( _wDrawingArea, XmNexposeCallback,
                   ( XtCallbackProc) AmExposeCallback,
                   ( XtPointer) this);
    XtAddCallback( _wDrawingArea, XmNresizeCallback,
                   ( XtCallbackProc) AmExposeCallback,
                   ( XtPointer) this);

    XtAddCallback( *_pwcTextTextField, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmUpdateTextCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcTextTextField, XmNactivateCallback,
                   ( XtCallbackProc) AmUpdateTextCallback,
                   ( XtPointer) this);

    XtAddCallback( *_pwcSizeTextField, XmNlosingFocusCallback,
                   ( XtCallbackProc) AmUpdateSizeTextCallback,
                   ( XtPointer) this);
    XtAddCallback( *_pwcSizeTextField, XmNactivateCallback,
                   ( XtCallbackProc) AmUpdateSizeTextCallback,
                   ( XtPointer) this);

    _pXGC->GetFont( &_szName, &_iSize, &_eWeight, &_eSlant,
                    &_dScaleX, &_dScaleY);

    valuesChanged();
}


AmTextDialogClass::AmTextDialogClass()
    : AmDialogClass()
{
}


AmTextDialogClass::AmTextDialogClass( Widget     wParent,
                                  const char *szName)
    : AmDialogClass( wParent, szName)
{
    initializeValues();
    createInterface();
}


AmTextDialogClass::AmTextDialogClass( AmBaseWidgetClass *bwcParent,
                                  const char      *szName)
    : AmDialogClass( bwcParent, szName)
{
    initializeValues();
    createInterface();
}


AmTextDialogClass::~AmTextDialogClass()
{
    if ( _szName != NULL) {
        free( _szName);
    }

    if ( _szText != NULL) {
        free( _szText);
    }

    if ( _pXWindow != NULL) {
        delete _pXWindow;
    }

    if ( _pXGC != NULL) { 
        delete _pXGC;
    }

    if ( _pXDisplay != NULL) {
        delete _pXDisplay;
    }
}


void
AmTextDialogClass::AmExposeCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->expose( w, xpCallData);
}

void
AmTextDialogClass::expose( Widget    w,
                         XtPointer xpCallData)
{
    Dimension    dimWidth, dimHeight;

    if ( XtIsRealized( _wDrawingArea)) {

        XtVaGetValues( _wDrawingArea,
                       XmNwidth, &dimWidth,
                       XmNheight, &dimHeight,
                       NULL);

        _pXGC->SetForegroundColor( &_colBackground);
        _pXDisplay->FillRectangle( _pXWindow, _pXGC,
                                   0, 0, dimWidth, dimHeight);

        if ( _szText != NULL) {
            _pXGC->SetForegroundColor( &_colForeground);
            _pXDisplay->DrawText( _pXWindow, _pXGC,
                                  _iMargin, _iMargin + _iAscent, _szText);
        }
    }
}


void
AmTextDialogClass::AmUpdateTextCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->updateText( w, xpCallData);
}

void
AmTextDialogClass::updateText( Widget    w,
                             XtPointer xpCallData)
{
    int   iTextWidth;
    char  *sz;

    XtVaGetValues( w, XmNvalue, &sz, NULL);

    if (( _szText == NULL) || ( strcmp( _szText, sz) != 0)) {
        if ( _szText != NULL) {
            free( _szText);
        }
        _szText = strdup( sz); 

        _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
        if ( _szText != NULL) {
            iTextWidth = _pXDisplay->TextWidth( _pXGC,
                                                _szText, strlen( _szText));
        } else {
            iTextWidth = 0;
        }

        if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
            XtVaSetValues( _wDrawingArea,
                           XmNwidth, iTextWidth + _iMargin * 2,
                           XmNheight, _iTextHeight + _iMargin * 2,
                           NULL);
        }

        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }

    XtFree( sz);
}


void
AmTextDialogClass::AmUpdateSizeTextCallback( Widget    w,
                                         XtPointer xpClientData,
                                         XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->updateSizeText( w, xpCallData);
}

void
AmTextDialogClass::updateSizeText( Widget    w,
                                 XtPointer xpCallData)
{
    AnvilBoolean fFound;
    int     iTemp, iTextWidth, iPos=0, ii;
    char    *sz;

    XtVaGetValues( w, XmNvalue, &sz, NULL);
    iTemp = atoi( sz);
    free( sz);

    if ( _iSize != iTemp) {

        _iSize = iTemp;

        for ( ii = 0, fFound = FALSE;
              ( fFound == FALSE) && ( ii < (signed)VECLEN( pFontSize));
              ii++) {
            if ( _iSize == pFontSize[ ii].iSize) {
                fFound = TRUE;
                iPos = ii;
            }
        }

        if ( fFound == TRUE) {
            _pwcSizeOptionMenu->SetSelectedPosition( iPos);
        } else {
            _pwcSizeOptionMenu->SetSelectedPosition( VECLEN( pFontSize));
        }

        _pXGC->SetFont( _szName, _iSize, _eWeight, _eSlant, _dScaleX, _dScaleY);

        _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
        if ( _szText != NULL) {
            iTextWidth = _pXDisplay->TextWidth( _pXGC,
                                                _szText, strlen( _szText));
        } else {
            iTextWidth = 0;
        }

        if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
            XtVaSetValues( _wDrawingArea,
                           XmNwidth, iTextWidth + _iMargin * 2,
                           XmNheight, _iTextHeight + _iMargin * 2,
                           NULL);
        }

        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }
}


void
AmTextDialogClass::AmUpdateSizeCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->updateSize( w, xpCallData);
}

void
AmTextDialogClass::updateSize( Widget    w,
                             XtPointer xpCallData)
{
    int   iPos = ( int) xpCallData;
    int   iTextWidth;
    char  sz[ TEXTLEN];

    if ( iPos >= (signed)VECLEN( pFontSize)) {
        return;
    }

    if ( _iSize != pFontSize[ iPos].iSize) {

        _iSize = pFontSize[ iPos].iSize;
        _pXGC->SetFont( _szName, _iSize, _eWeight, _eSlant, _dScaleX, _dScaleY);

        _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
        if ( _szText != NULL) {
            iTextWidth = _pXDisplay->TextWidth( _pXGC,
                                                _szText, strlen( _szText));
        } else {
            iTextWidth = 0;
        }
 
        sprintf( sz, "%d", _iSize);
        XtVaSetValues( *_pwcSizeTextField, XmNvalue, sz, NULL);

        if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
            XtVaSetValues( _wDrawingArea,
                           XmNwidth, iTextWidth + _iMargin * 2,
                           XmNheight, _iTextHeight + _iMargin * 2,
                           NULL);
        }

        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }
}


void
AmTextDialogClass::AmUpdateFamilyCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->updateFamily( w, xpCallData);
}

void
AmTextDialogClass::updateFamily( Widget    w,
                               XtPointer xpCallData)
{
    int   iPos = ( int) xpCallData;
    int   iTextWidth;

    if (( _szName == NULL) ||
        ( strcmp( _szName, _pszFamily[ iPos]) != 0)) {

        if ( _szName != NULL) {
            free( _szName);
        }

        _szName = strdup( _pszFamily[ iPos]);
        _pXGC->SetFont( _szName, _iSize, _eWeight, _eSlant, _dScaleX, _dScaleY);

        _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
        if ( _szText != NULL) {
            iTextWidth = _pXDisplay->TextWidth( _pXGC,
                                                _szText, strlen( _szText));
        } else {
            iTextWidth = 0;
        }

        if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
            XtVaSetValues( _wDrawingArea,
                           XmNwidth, iTextWidth + _iMargin * 2,
                           XmNheight, _iTextHeight + _iMargin * 2,
                           NULL);
        }

        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }
}


void
AmTextDialogClass::AmToggleBoldCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->toggleBold( w, xpCallData);
}

void
AmTextDialogClass::toggleBold( Widget    w,
                             XtPointer xpCallData)
{
    XmToggleButtonCallbackStruct *tbc;
    int                          iTextWidth;

    tbc = ( XmToggleButtonCallbackStruct *) xpCallData;

    if ( tbc->set != ( _eWeight == AmWEIGHT_BOLD)) {

        _eWeight = ( tbc->set == TRUE) ? AmWEIGHT_BOLD : AmWEIGHT_MEDIUM;

        _pXGC->SetFont( _szName, _iSize, _eWeight, _eSlant, _dScaleX, _dScaleY);

        _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
        if ( _szText != NULL) {
            iTextWidth = _pXDisplay->TextWidth( _pXGC,
                                                _szText, strlen( _szText));
        } else {
            iTextWidth = 0;
        }

        if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
            XtVaSetValues( _wDrawingArea,
                           XmNwidth, iTextWidth + _iMargin * 2,
                           XmNheight, _iTextHeight + _iMargin * 2,
                           NULL);
        }

        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }
}


void
AmTextDialogClass::AmToggleItalicCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData)
{
    AmTextDialogClass *pTextDialog;
    pTextDialog = ( AmTextDialogClass *) xpClientData;

    pTextDialog->toggleItalic( w, xpCallData);
}

void
AmTextDialogClass::toggleItalic( Widget    w,
                               XtPointer xpCallData)
{
    XmToggleButtonCallbackStruct *tbc;
    int                          iTextWidth;

    tbc = ( XmToggleButtonCallbackStruct *) xpCallData;

    if ( tbc->set != ( _eSlant == AmSLANT_ITALIC)) {

        _eSlant = ( tbc->set == TRUE) ? AmSLANT_ITALIC : AmSLANT_ROMAN;

        _pXGC->SetFont( _szName, _iSize, _eWeight, _eSlant, _dScaleX, _dScaleY);

        _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
        if ( _szText != NULL) {
            iTextWidth = _pXDisplay->TextWidth( _pXGC,
                                                _szText, strlen( _szText));
        } else {
            iTextWidth = 0;
        }

        if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
            XtVaSetValues( _wDrawingArea,
                           XmNwidth, iTextWidth + _iMargin * 2,
                           XmNheight, _iTextHeight + _iMargin * 2,
                           NULL);
        }

        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }
}


void
AmTextDialogClass::SetFont( char           *szName,
                          int            iSize,
                          AmFontWeightEnum eWeight,
                          AmFontSlantEnum  eSlant,
                          float          dScaleX,
                          float          dScaleY)
{
    if ( _szName != NULL) {
        free( _szName); 
    }
    _szName = strdup( szName);

    _iSize = iSize;
    _eWeight = eWeight;
    _eSlant = eSlant;

    _dScaleX = dScaleX;
    _dScaleY = dScaleY;

    valuesChanged();
}

void
AmTextDialogClass::SetText( const char *szText)
{
    char    sz[ TEXTLEN];

    if ( _szText != NULL) {
        free( _szText);
    }

    if ( szText != NULL) {
        _szText = strdup( szText);
    }

    sprintf( sz, "%s", _szText);
    XtVaSetValues( *_pwcTextTextField, XmNvalue, sz, NULL);

    valuesChanged();
}

void
AmTextDialogClass::GetFont( char           **pszName,
                          int            *piSize,
                          AmFontWeightEnum *peWeight,
                          AmFontSlantEnum  *peSlant,
                          float          *pdScaleX,
                          float          *pdScaleY)
{
    *pszName = strdup( _szName);

    *piSize = _iSize;
    *peWeight = _eWeight;
    *peSlant = _eSlant;

    *pdScaleX = _dScaleX;
    *pdScaleY = _dScaleY;
}

const char *
AmTextDialogClass::GetText()
{
    return( _szText);
}


void
AmTextDialogClass::valuesChanged()
{
    AnvilBoolean fFound;
    int     iTextWidth, iPos=0, ii;
    char    sz[ TEXTLEN];

/* Set the family option menu. */

    for ( ii = 0, fFound = FALSE;
        ( fFound == FALSE) && ( _pszFamily[ ii] != NULL);
        ii++) {

        if (( _szName != NULL) && ( strcmp( _szName, _pszFamily[ ii]) == 0)) {
            fFound = TRUE;
            iPos = ii;
        }
    }

    if ( fFound == TRUE) {
        _pwcFamilyOptionMenu->SetSelectedPosition( iPos);
    } else {
        _pwcFamilyOptionMenu->SetSelectedPosition( 0);
    }

/* Set the size option menu. */

    for ( ii = 0, fFound = FALSE;
        ( fFound == FALSE) && ( ii < (signed)VECLEN( pFontSize));
        ii++) {

        if ( _iSize == pFontSize[ ii].iSize) {
            fFound = TRUE;
            iPos = ii;
        }
    }

    if ( fFound == TRUE) {
        _pwcSizeOptionMenu->SetSelectedPosition( iPos);
    } else {
        _pwcSizeOptionMenu->SetSelectedPosition( VECLEN( pFontSize));
    }

/* Set the text field. */
 
    sprintf( sz, "%d", _iSize);
    XtVaSetValues( *_pwcSizeTextField, XmNvalue, sz, NULL);

/* Set the toggle buttons. */

    XmToggleButtonSetState( *_pwcBoldToggle,
                            _eWeight == AmWEIGHT_BOLD,
                            FALSE);
    XmToggleButtonSetState( *_pwcItalicToggle, 
                            _eSlant == AmSLANT_ITALIC,
                            FALSE);

/* Set the font, and update the display. */

    _pXGC->SetFont( _szName, _iSize, _eWeight, _eSlant, _dScaleX, _dScaleY);

    _iTextHeight = _pXDisplay->FontHeight( _pXGC, &_iAscent, &_iDescent);
    if ( _szText != NULL) {
        iTextWidth = _pXDisplay->TextWidth( _pXGC, _szText, strlen( _szText));
    } else {
        iTextWidth = 0;
    }

    if (( iTextWidth != 0) && ( _iTextHeight != 0)) {
        XtVaSetValues( _wDrawingArea,
                       XmNwidth, iTextWidth + _iMargin * 2,
                       XmNheight, _iTextHeight + _iMargin * 2,
                       NULL);
    }

    if ( XtIsRealized( _wDrawingArea)) {
        AmSendExpose( _display, XtWindow( _wDrawingArea));
    }
}
