#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* This is intended to contain all code necessary to interface anvil
   with Odyssey, including adaptation routines. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

#include <include/odyssey.h>
#include <include/odytypes.h>
#include <odyssey/wardens/map/mapwarden.h>
#include <odyssey/wardens/map/libodymap_tsops.h>

/* Anvil currently supports 4 levels of fidelity:
   0) minor+secondary road filter / cropped
   1) minor+secondary road filter
   2) minor road filter
   3) full fidelity - no degradation 
*/
#define FIDELITY_CROPPED          0
#define FIDELITY_MINOR_FILTER     1
#define FIDELITY_SECONDARY_FILTER 2
#define FIDELITY_FULL             3

int cur_fidelity = FIDELITY_FULL;
int filterIds[2] = { 5004, 5003 };

#ifdef PSCOPE
extern int pscope_crop;
extern int ody_fd;
#endif

#define MAPFILE "/ody/map"

static void
switch_fidelity (int new_fidelity) 
{

  if (new_fidelity == FIDELITY_FULL) {
    map_set_filter (MAPFILE, NULL, 0);
  } else if (new_fidelity == FIDELITY_MINOR_FILTER) {
    map_set_filter (MAPFILE, filterIds, 1);
  } else {
    map_set_filter (MAPFILE, filterIds, 2);
  }

#ifdef PSCOPE
  if (new_fidelity == FIDELITY_CROPPED) {
    pscope_crop = 1;
  } else {
    pscope_crop = 0;
  }
#endif

}

void 
adapt_new_en(int reqid, unsigned int rsrc, int val)
{
  struct ody_req_des or_des;
  struct timeval tv;

  ASSERT(rsrc == ODY_GR_POWER);

  while (1) {

      ASSERT(val >= 0 && val <= FIDELITY_FULL);
      
      /* Switch to new fidelity level */
      cur_fidelity = val;
      switch_fidelity (cur_fidelity);
      
      gettimeofday (&tv, NULL);
      printf ("%ld.%ld: Anvil adapted to fidelity %d\n", tv.tv_sec, 
	      tv.tv_usec, val);
      fflush (stdout);

      /* Register new fidelity level */
      or_des.ord_resource = ODY_GR_POWER;
      or_des.ord_version.ov_gs = ODY_GENERIC_INTERFACE;
      or_des.ord_version.ov_type = MAP_TTYPE;
      or_des.ord_version.ov_tsvnum = MAPWARDEN_VERSION;
      or_des.ord_low = val;
      or_des.ord_high = val;
      or_des.ord_fp = adapt_new_en;
      
      if (ody_request ("/ody/map", &or_des, &reqid) < 0) {
	  if (errno != ODY_ENOTINWINDOW) {
	      perror("ody_frequest");
	      exit(1);
	  } 
	  val = reqid;
      } else {
	  break;
      }
  }

}

void 
init_adaptation ()
{
  adapt_new_en (0, ODY_GR_POWER, FIDELITY_FULL);
}
