#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <anvil/project.h>		/* all-purpose include */
#include <anvil/motifdisplay.h>		/* display manager include file */
#include <anvil/overlay.h>		/* overlay manager header file */
#include <anvil/motif.h>		/* motif convienience macros */
#include <anvil/errlib.h>		/* error messages */
#include <anvil/header.h>		/* for tag-value header file parsing */
#include <anvil/plot_cback.h>		/* plot callback header file */
#include <anvil/xapp.h>                 /* initialization functions */


/* fake parameter-passing structure for the readout event handler */

typedef struct readoutData_ {
    Widget		readoutLbl;
    Pixmap		buffer;
    AmOverlayClass	*source;
    int			lastWidth;
    int			lastHeight;
    GC			gc;
} readoutDataRec, *readoutDataPtr;

/* readout event handler:
   Area:	drawing area for the display manager, where we're listening to
		the mouse events.
   readoutData:	structure containing arguments to use for the readout
   event:	Pointer Motion event received from the server.
   continueDispatch: ignore this.
*/

void readoutHandler( Widget		area,
		     readoutDataPtr	readoutData,
		     XMotionEvent	*event,
		     AnvilBoolean		*continueDispatch)
{
    Dimension	width, height;
    Pixel	background, foreground;
    int		depth;
    float	x, y;
    char	readoutStr[ 512];

/* extract the dimensions, foreground, background, and pixel depth from the 
   readout widget */

    XtVaGetValues( readoutData->readoutLbl,
		   XmNwidth, &width,
		   XmNheight, &height,
		   XmNbackground, &background,
		   XmNforeground, &foreground,
		   XmNdepth, &depth,
		   NULL);

/* if the double buffering pixmap doesn't exist or the widget has been resized,
   then we create a new one */

    if (( readoutData->buffer == NULL) || 
        ( readoutData->lastWidth != width) ||
        ( readoutData->lastHeight != height)) {
        if ( readoutData->buffer != NULL) {
	    XFreePixmap( XtDisplay( area), readoutData->buffer);
	}
	readoutData->buffer = XCreatePixmap( XtDisplay( area), XtWindow( area),
					     width, height, depth);
	readoutData->lastWidth = width;
	readoutData->lastHeight = height;
    }

/* clear out the pixmap */

    XSetForeground( XtDisplay( area), readoutData->gc, background);
    XFillRectangle( XtDisplay( area), readoutData->buffer, readoutData->gc,
		    0, 0, width, height);

/* reset the foreground and background for the graphics context */

    XSetForeground( XtDisplay( area), readoutData->gc, foreground);
    XSetBackground( XtDisplay( area), readoutData->gc, background);

/* find out where the cursor is in the source plot's coordinate system, and
   write it to a string */

    readoutData->source->GetDataPt( event->x, event->y, &x, &y);
    sprintf( readoutStr, "X: %.2f, Y: %.2f", x, y);

/* draw the string to the pixmap, and copy the pixmap to the readout widget */

    XDrawString( XtDisplay( readoutData->readoutLbl), readoutData->buffer,
		 readoutData->gc, 20, 20, readoutStr, strlen( readoutStr));
    XCopyArea( XtDisplay( readoutData->readoutLbl), readoutData->buffer,
	       XtWindow( readoutData->readoutLbl), readoutData->gc, 
	       0, 0, width, height, 0, 0);
}

/* when the readout dialog has been destroyed, remove the pointer motion event
   handler and free the data structure we used for parameter passing */

void destroyReadoutCallback( Widget		dialog,
			     readoutDataPtr	readoutData,
			     Widget		area,
			     XtPointer)
{
    XtRemoveEventHandler( area, PointerMotionMask, FALSE,
		          ( XtEventHandler) readoutHandler, readoutData);
    if ( readoutData->buffer != NULL) {
	XFreePixmap( XtDisplay( dialog), readoutData->buffer);
    }
    free( readoutData);
}

/* callback to add a readout dialog to an overlay plot */

void addReadoutCallback( Widget			btn,
			 AmMotifDisplayClass	*motifDisplay,
			 AmOverlayClass		*overlay,
			 XtPointer)
{
    Widget		formDlog, readoutLbl, closeBtn;
    readoutDataPtr	readoutData;
    XGCValues		gcv;

    formDlog = XmCreateFormDialog( AmGetWidgetShell( btn), "readout", NULL, 0);
    readoutLbl = XtVaCreateManagedWidget( "readout", xmLabelWidgetClass, formDlog,
				     StickLeft, StickTop, StickRight,
				     XmNheight, 80, XmNwidth, 200,
				     NULL);
    closeBtn = XtVaCreateManagedWidget( "closeBtn", xmPushButtonWidgetClass, 
					formDlog,
					StickLeft, StickRight, StickBottom,
					BelowOf( readoutLbl),
					NULL);
    XtManageChild( formDlog);

/* create a fake parameter-passing structure to pass the arguments to the
   event handler */

    readoutData = ( readoutDataPtr) malloc( sizeof( readoutDataRec));
    readoutData->readoutLbl = readoutLbl;
    readoutData->buffer = 0;
    readoutData->source = overlay;
    readoutData->lastWidth = 0;
    readoutData->lastHeight = 0;

/* get the font from the widget using the magic font function */

    gcv.font = AmGetWidgetFont( readoutLbl);

/* create a graphics context for drawing with */

    readoutData->gc = XCreateGC( XtDisplay( btn), XtWindow( btn), GCFont, &gcv);

/* attach an event handler to the display manager's drawing area, which will
   get called whenever the mouse is moved */

    XtAddEventHandler( motifDisplay->GetArea(), PointerMotionMask, FALSE,
		       ( XtEventHandler) readoutHandler, readoutData);

/* add a varargs callback to remove the event handler and free the data when the
   dialog has been destroyed */

    AmAddVaCallback( formDlog, XmNdestroyCallback,
		   ( VarargsCallback) destroyReadoutCallback, 
		   readoutData, motifDisplay->GetArea(), 
		   END_LIST);
    XtAddCallback( closeBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmPopdownCback, formDlog);
}


int main( int argc, char *argv[])
{
    Widget		top, form, area, addBtn, delBtn, zoomBtn, unzoomBtn,
			readoutBtn;
    AmMotifDisplayClass	*motifDisplay;
    AmOverlayClass	*overlay;
    AmXDisplayClass	*pXDisplay;
    AmXWindowClass	*pXWindow;

/* initialize the connection to the server, read the resource database
   and parse the command line arguments.  Create a form and a drawing
   area to draw the overlay in.  add buttons to add and remove layers */

    top = XtInitialize ( "editor", "editor-db", NULL, 0, &argc, argv);

/* call the init function */

    AmInitApplication( top);
    form = XtVaCreateManagedWidget( "form", xmFormWidgetClass, top, NULL);
    addBtn = XtVaCreateManagedWidget( "addBtn", xmPushButtonWidgetClass,
    				      form, 
    				      StickLeft, StickTop,
    				      NULL);
    delBtn = XtVaCreateManagedWidget( "delBtn", xmPushButtonWidgetClass,
    				      form, 
    				      RightOf( addBtn), StickTop,
    				      NULL);
    zoomBtn = XtVaCreateManagedWidget( "zoomBtn", xmPushButtonWidgetClass,
    				       form, 
    				       RightOf( delBtn), StickTop,
    				       NULL);
    unzoomBtn = XtVaCreateManagedWidget( "unzoomBtn", xmPushButtonWidgetClass,
    				         form, 
    				         RightOf( zoomBtn), StickTop,
    				         NULL);
    readoutBtn = XtVaCreateManagedWidget( "readoutBtn", xmPushButtonWidgetClass,
    				          form, 
    				          RightOf( unzoomBtn), StickTop,
					  NULL);
    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, form,
    				    StickLeft, StickRight, 
    				    StickBottom, 
    				    BelowOf( addBtn),
    				    XmNwidth, 300,
    				    XmNheight, 300,
    				    NULL);

/* initialize a display manager for the drawing area */

    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    motifDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);

/* create an overlay from the command line argument */

    overlay = new AmOverlayClass( motifDisplay->GetWindow(), "overlay", 
                                AmNleftMargin, 0,
				AmNrightMargin, 0,
				AmNtopMargin, 0,
				AmNbottomMargin, 0, 
				NULL);


/* add the object to the display manager, so it gets drawn and refreshed */

    motifDisplay->AddObject( overlay);

/* add a callback to add layers.  use the varargs callback interface. 
   Note the END_LIST terminator.  We can't use NULL, because it's a common
   callback argument */ 

    AmAddVaCallback( addBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmAddLayerCallback,
    		   motifDisplay, overlay, END_LIST);

/* add a callback to delete layers */

    AmAddVaCallback( delBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmDeleteLayerCallback,
    		   motifDisplay, overlay, END_LIST);

/* add callbacks to zoom and unzoom */

    AmAddVaCallback( zoomBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmZoomRectCallback,
    		   motifDisplay, overlay, END_LIST);
    AmAddVaCallback( unzoomBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmUnzoomCallback,
    		   motifDisplay, overlay, END_LIST);

/* add the readout callback */

    AmAddVaCallback( readoutBtn, XmNactivateCallback,
		   ( VarargsCallback) addReadoutCallback,
		   motifDisplay, overlay, END_LIST);

/* realize the top level widget and start processing events */

    XtRealizeWidget( top);
    XtMainLoop();
}
