#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <anvil/slick.h>		/* display manager include file */
#include <anvil/motif.h>		/* motif convienience macros */
#include <anvil/xapp.h>		
#include <anvil/colorcell.h>		
#include <anvil/xdisplay.h>		

/* NOTE:  THIS TEST DRIVER DEPENDS ON THE RESOURCE FILE slick-db to set the
   widget types and default values. */

Widget localTextFloat, dynamicTextFloat, localTextInt, 
	dynamicTextInt, localTextStr, dynamicTextStr, localScaleInt, 
	dynamicScaleInt, localScaleFloat, dynamicScaleFloat, 
	localOptInt, dynamicOptInt, localListInt, dynamicListInt,
	localTgl, dynamicTgl, localTextDbl, dynamicTextDbl, 
	localFile, dynamicFile, localFileBtn, dynamicFileBtn, colorBtn;

void getValuesCallback( Widget		btn,
			XtPointer,
			XtPointer)
{
    char 		localFileName[ 256], *dynamicFileName, localStr[ 256],
			*dynamicStr;
    float		x1, x2;
    int			y1, y2, opt1, opt2, list1, list2;
    AnvilBoolean		flag1, flag2;
    AmColorCellRec	color;

/* text fields */

    if ( AmGetObjectValue( localTextFloat, &x1)) {
	printf( "x1 = %f\n", x1);
    } else {
	printf( "failed to get value for x1\n");
    }
    if ( AmGetObjectValue( dynamicTextFloat, &x2)) {
	printf( "x2 = %f\n", x2);
    } else {
	printf( "failed to get value for x2\n");
    }
    if ( AmGetObjectValue( localTextInt, &y1)) {
	printf( "x1 = %d\n", y1);
    } else {
	printf( "failed to get value for y1\n");
    }
    if ( AmGetObjectValue( dynamicTextInt, &y2)) {
	printf( "x2 = %d\n", y2);
    } else {
	printf( "failed to get value for y2\n");
    }
    if ( AmGetObjectValue( localTextStr, localStr)) {
	printf( "localStr = %s\n", localStr);
    } else {
	printf( "failed to get value for localStr\n");
    }
    if ( AmGetObjectValue( dynamicTextStr, &dynamicStr)) {
	printf( "dynamicStr = %s\n", dynamicStr);
	if ( dynamicStr != NULL) {
	    free( dynamicStr);
	}
    } else {
	printf( "failed to get value for dynamicStr\n");
    }

/* toggle fields */

    if ( AmGetObjectValue( localTgl, &flag1)) {
	printf( "flag1 = %d\n", flag1);
    } else {
	printf( "failed to get value for flag1\n");
    }
    if ( AmGetObjectValue( dynamicTgl, &flag2)) {
	printf( "flag2 = %d\n", flag2);
    } else {
	printf( "failed to get value for flag2\n");
    }

/* option menus */

    if ( AmGetObjectValue( localOptInt, &opt1)) {
	printf( "opt1 = %d\n", opt1);
    } else {
	printf( "failed to get value for opt1\n");
    }
    if ( AmGetObjectValue( dynamicOptInt, &opt2)) {
	printf( "opt2 = %d\n", opt2);
    } else {
	printf( "failed to get value for opt2\n");
    }

/* lists */

    if ( AmGetObjectValue( localListInt, &list1)) {
	printf( "list1 = %d\n", list1);
    } else {
	printf( "failed to get value for list1\n");
    }
    if ( AmGetObjectValue( dynamicListInt, &list2)) {
	printf( "list2 = %d\n", list2);
    } else {
	printf( "failed to get value for list2\n");
    }

/* color button */

    if ( AmGetObjectValue( colorBtn, &color)) {
	printf( "color = %d %d %d\n", color.red, color.green, color.blue);
    } else {
	printf( "failed to get value for colorBtn\n");
    }

/* filenames */

    if ( AmGetObjectValue( localFile, localFileName)) {
	printf( "localFile = %s\n", localFileName);
    } else {
	printf( "failed to get value for localFileName\n");
    }
    if ( AmGetObjectValue( dynamicFile, &dynamicFileName)) {
	printf( "dynamicFile = %s\n", dynamicFileName);
	if ( dynamicFileName != NULL) {
	    free( dynamicFileName);
	}
    } else {
	printf( "failed to get value for dynamicFileName\n");
    }
    if ( AmGetObjectValue( localFileBtn, localFileName)) {
	printf( "localFileBtn = %s\n", localFileName);
    } else {
	printf( "failed to get value for localFileName\n");
    }
    if ( AmGetObjectValue( dynamicFileBtn, &dynamicFileName)) {
	printf( "dynamicFileBtn = %s\n", dynamicFileName);
	if ( dynamicFileName != NULL) {
	    free( dynamicFileName);
	}
    } else {
	printf( "failed to get value for dynamicFileName\n");
    }
}

int main( int argc, char *argv[])
{
    Widget		top, rowcol, okBtn, revertBtn; 
    float		xText = 0.5, xScale = 0.5;
    double		dblText = 0.25;
    char		*strText = "this is a test", bufText[ 256],
    			*filename, localFileName[ 1024];
    int			yText = 10, yScale = 11, yOpt = 2, yList = 1;
    AnvilBoolean		flagTgl = TRUE;
    AmColorCellRec	color;
    AmXDisplayClass	*display;
    XtAppContext	app;
    XEvent		event;

    strcpy( localFileName, "/etc/passwd");

/* initialize the connection to the server, read the resource database
   and parse the command line arguments.  */

    top = XtInitialize ( "slick", "slick-db", NULL, 0, &argc, argv);

/* THIS IS THE SECRET INITIALIZATION CALL!!!! */

    if ( AmInitApplication( top) != AmOK) {
	fprintf( stderr, "failed to init , errCode = %d file = %s line = %d\n",
			 ANVIL_ERROR_CODE, ANVIL_ERROR_FILE, ANVIL_ERROR_LINE);
	exit( -1);
    }
    rowcol = XtVaCreateManagedWidget( "rowcol", xmRowColumnWidgetClass, 
				      top, 
				      XmNpacking, XmPACK_TIGHT,
				      XmNcolumns, 2,
				      NULL);
    strcpy( bufText, "this is a test");

/* text tests... */

    localTextFloat = AmAddObject( rowcol, &xText, "xText", 
				AmFLOAT, AmLOCAL);
    dynamicTextFloat = AmAddObject( rowcol, &xText, "xText", 
				  AmFLOAT, AmDYNAMIC);
    localTextInt = AmAddObject( rowcol, &yText, "yText", 
			      AmINT, AmLOCAL);
    dynamicTextInt = AmAddObject( rowcol, &yText, "yText", 
				AmINT, AmDYNAMIC);
    localTextStr = AmAddObject( rowcol, &bufText, "bufText", 
			      AmSTRING, AmLOCAL);
    dynamicTextStr = AmAddObject( rowcol, &strText, "strText", 
			        AmSTRING, AmDYNAMIC);
    localTextDbl = AmAddObject( rowcol, &dblText, "dblText", 
			      AmDOUBLE, AmLOCAL);
    dynamicTextDbl = AmAddObject( rowcol, &dblText, "dblText", 
			        AmDOUBLE, AmDYNAMIC);

/* scale tests... */

    localScaleFloat = AmAddObject( rowcol, &xScale, "xScale", 
				 AmFLOAT, AmLOCAL);
    dynamicScaleFloat = AmAddObject( rowcol, &xScale, "xScale", 
				   AmFLOAT, AmDYNAMIC);
    localScaleInt = AmAddObject( rowcol, &yScale, "yScale", 
			       AmINT, AmLOCAL);
    dynamicScaleInt = AmAddObject( rowcol, &yScale, "yScale", 
				 AmINT, AmDYNAMIC);

/* toggle tests */

    localTgl = AmAddObject( rowcol, &flagTgl, "flagTgl", 
			  AmBOOLEAN, AmLOCAL);
    dynamicTgl = AmAddObject( rowcol, &flagTgl, "flagTgl", 
			    AmBOOLEAN, AmDYNAMIC);

/* option menu tests */

    localOptInt = AmAddObject( rowcol, &yOpt, "yOpt", AmINT, AmLOCAL);
    dynamicOptInt = AmAddObject( rowcol, &yOpt, "yOpt", AmINT, AmDYNAMIC);

/* list tests */

    localListInt = AmAddObject( rowcol, &yList, "yList", AmINT, AmLOCAL);
    dynamicListInt = AmAddObject( rowcol, &yList, "yList", AmINT, AmDYNAMIC);

/* color cell tests */

    color.red = 255;
    color.green = 0;
    color.blue = 0;
    display = new AmXDisplayClass( top);
    display->AllocColor( &color);
    colorBtn = AmAddObject( rowcol, &color, "color", AmCOLOR_CELL, AmLOCAL);

/* file tests */
  
    filename = NULL;
    dynamicFile = AmAddObject( rowcol, &filename, "filename", AmFILE, AmDYNAMIC);
    localFile = AmAddObject( rowcol, localFileName, "filename", AmFILE, AmLOCAL);
    dynamicFileBtn = AmAddObject( rowcol, &filename, "filenameBtn", 
				AmFILE, AmDYNAMIC);
    localFileBtn = AmAddObject( rowcol, localFileName, "filenameBtn", 
			      AmFILE, AmLOCAL);

/* OK button */

    okBtn = XtVaCreateManagedWidget( "okBtn", xmPushButtonWidgetClass, 
				     rowcol, NULL);
    XtAddCallback( okBtn, XmNactivateCallback,
		   ( XtCallbackProc) getValuesCallback, NULL);

/* revert button */

    revertBtn = XtVaCreateManagedWidget( "revertBtn", xmPushButtonWidgetClass, 
					 rowcol, NULL);
    AmSetRevertButton( revertBtn);

/* filenames */

    filename = NULL;

/* realize the top level widget and start processing events */

    XtRealizeWidget( top);
    app = XtWidgetToApplicationContext( top);
    do {
	XtNextEvent( &event);
	AmHelpHandler( XtDisplay( top), app, &event);
	XtDispatchEvent( &event);
    } while ( TRUE);
}
