#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <anvil/motifdisplay.h>		/* display manager include file */
#include <anvil/sphereplot.h>		/* image plot header file */
#include <anvil/motif.h>		/* motif convienience macros */
#include <anvil/xapp.h>			/* initialization functions */
#include <anvil/plot_cback.h>		

int main( int argc, char *argv[])
{
    Widget		top, form, area;
    AmMotifDisplayClass	*motifDisplay;
    AmSpherePlotClass	*spherePlot;
    AmXDisplayClass	*pXDisplay;
    AmXWindowClass	*pXWindow;

    if ( argc < 2) {
    	fprintf( stderr, "usage %s: matrix-hdr\n", argv[ 0]);
    	exit( -1);
    }

/* initialize the connection to the server, read the resource database
   and parse the command line arguments.  Create a form and a drawing
   area to draw the image plot in.  Note the StickLeft... macros in
   the drawing area definition.  They've convienience macros from
   <anvil/motif.h> */

    top = XtInitialize ( "sphere", "sphere-db", NULL, 0, &argc, argv);

/* THIS IS THE SECRET INITIALIZATION CALL!!!! */

    AmInitApplication( top);
    form = XtVaCreateManagedWidget( "form", xmFormWidgetClass, top, NULL);
    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, form,
    				    StickLeft, StickRight, 
    				    StickBottom, StickTop,
    				    XmNwidth, 300,
    				    XmNheight, 300,
    				    NULL);

/* initialize a display manager for the drawing area */

    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    motifDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow);

/* create an image plot from the AmFirst command line argument  Get the
   preferences file from the second command line argument.  The
   xxxMargin arguments set up attachments so the plot is resized with the
   drawing area */

    spherePlot = new AmSpherePlotClass( motifDisplay->GetWindow(), "spherePlot", 
    				     AmNheader, argv[ 1],
				     AmNleftMargin, 0,
				     AmNrightMargin, 0,
				     AmNtopMargin, 0,
				     AmNbottomMargin, 0,
    				     NULL);

/* add the object to the display manager, so it gets drawn and refreshed */

    motifDisplay->AddObject( spherePlot);
    motifDisplay->Flush();
    AmAddSphereRotationHandlers( motifDisplay->GetArea(),
			       motifDisplay, spherePlot);

/* realize the top level widget and start processing events */

    XtRealizeWidget( top);
    XtMainLoop();
}
