#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#include <anvil/colorcell.h>
#include <anvil/xapp.h>
#include <anvil/xcolor.h>
#include <anvil/motif.h>
#include <anvil/colorsel.h>


/* Globals: */

static AmColorCellRec g_color = { 0, 0, 0, 0 };


/* ARGSUSED */
static void
ColorOk( AmColorCellPtr pColor,
         void           *pClientData)
{
    Widget wLaunchButton = ( Widget) pClientData;  
  
    fprintf( stdout, "Selected:\n");
    fprintf( stdout, "    Color: ( %02X %02X %02X ) Pixel: %08X\n",
             pColor->red, pColor->green, pColor->blue, pColor->pixel);

    XmChangeColor( wLaunchButton, pColor->pixel);
}


/*
 * Function: CreateDialog( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The CreateDialog function launches a new color selection 
 *     dialog.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : Unused.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 *
 * Comments:
 *     The callback can launch multiple color selection dialogs at the
 *     same time.  Each color selection dialog eidts the same color which
 *     is stored statically.
 */

/* ARGSUSED */
void
CreateDialog( Widget    wBtn,
              XtPointer xClientData,
              XtPointer xCallData)
{
    Widget                wDialog;

    fprintf( stdout, "Launching Color Selection Dialog ... ");

    wDialog = AmCreateColorSelectionDialog( wBtn, "ColorDialog", &g_color,
                                            ( AmColorSelUpdateFn) ColorOk,
                                            ( void *) wBtn);

    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: Quit( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The Quit function forces the event loop to terminate on the 
 *     next itteration.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : Unused.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 *
 * Comments:
 *     The function simple sets the Run flag to FALSE which in turn forces
 *     the event loop to terminate.
 */

/* ARGSUSED */
void
Quit( Widget    wBtn,
      XtPointer xClientData,
      XtPointer xCallData)
{

    fprintf( stdout, "Quitting the application.\n");

    AnvilBoolean *pfRun = ( AnvilBoolean *) xClientData; 
    *pfRun = FALSE;
}


/*
 * Function: main( nArgs, vszArgs)
 *
 * Purpose:
 *     The main function sets up the application and creates a 
 *     dialog which provides a button to launch the color selection
 *     dialog and another button to quit the test program.
 *
 * Parameters:
 *     int  nARgs    : The number of command line arguments passed in.
 *     char *vszArgs : The vector of command line arguments.
 *
 * Returns:
 *     int - whether the program executed without an error or not.
 */


int
main( int nArgs, char *vszArgs[])
{
    Widget       wTop, wForm, wButton;
    AnvilBoolean      fRun;
    XtAppContext ac;
    XEvent       xEvent;

    fprintf( stdout, "Starting the application.\n");

    fRun = TRUE;

    wTop  = XtVaAppInitialize( &ac, "TestApp",
                               NULL, 0, 
                               &nArgs, vszArgs,
                               NULL, NULL);

    AmInitApplication( wTop);

/* Set the pixel value for the color. */

    AmSetPixelValue( &g_color);

    wForm = XtVaCreateManagedWidget( "ColorForm",
                                     xmFormWidgetClass, wTop,
                                     XmNfractionBase, 13,
                                     NULL);

    wButton = XtVaCreateManagedWidget( "Color",
                                       xmPushButtonWidgetClass, wForm,
                                       XmNleftAttachment, XmATTACH_POSITION,
                                       XmNleftPosition, 1, 
                                       XmNtopAttachment, XmATTACH_POSITION,
                                       XmNtopPosition, 1, 
                                       XmNrightAttachment, XmATTACH_POSITION,
                                       XmNrightPosition, 12, 
                                       XmNbottomAttachment, XmATTACH_POSITION,
                                       XmNbottomPosition, 6, 
                                       NULL);
    XmChangeColor( wButton, g_color.pixel);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) CreateDialog, NULL);

    wButton = XtVaCreateManagedWidget( "Quit",
                                       xmPushButtonWidgetClass, wForm,
                                       XmNleftAttachment, XmATTACH_POSITION,
                                       XmNleftPosition, 1, 
                                       XmNtopAttachment, XmATTACH_POSITION,
                                       XmNtopPosition, 7, 
                                       XmNrightAttachment, XmATTACH_POSITION,
                                       XmNrightPosition, 12, 
                                       XmNbottomAttachment, XmATTACH_POSITION,
                                       XmNbottomPosition, 12, 
                                       NULL);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) Quit,
                   ( XtPointer) &fRun);
  

    XtRealizeWidget( wTop);

    do {
        XtAppNextEvent( ac, &xEvent);
        XtDispatchEvent( &xEvent);
    } while ( fRun == TRUE);

    fprintf( stdout, "Done.\n");

    return( 1);
}
