#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#include <anvil/colorcell.h>
#include <anvil/xapp.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/log.h>

static void MakeDialog( Widget    wBtn,
                        XtPointer xClientData,
                        XtPointer xCallData);

/* Globals: */

static int g_nPrintErrorWShell   = 0;
static int g_nPrintErrorWOShell  = 0;
static int g_nPrintResourceError = 0;
static int g_nPrintWarnWShell    = 0;
static int g_nPrintWarnWOShell   = 0;
static int g_nPrintResourceWarn  = 0;
static int g_nPrintInfoWShell    = 0;
static int g_nPrintInfoWOShell   = 0;
static int g_nPrintResourceInfo  = 0;
static int g_nPrintLogError      = 0;
static int g_nLog                = 0;

static AnvilBoolean g_fRun            = TRUE;

/*
 * Function: PrintError( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintError function prints an error with or without a shell.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the error
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintError( Widget    wBtn,
            XtPointer xClientData,
            XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Error Dialog ");

    if ( xClientData != NULL) { 

        fprintf( stdout, "With Shell (%d) ... ", g_nPrintErrorWShell);

        wShell = ( Widget) xClientData;
        wDialog = AmPrintError( wShell, "Print Error (with shell) %d",
                                g_nPrintErrorWShell++);

    } else {

        fprintf( stdout, "Without Shell (%d) ... ", g_nPrintErrorWOShell);

        wDialog = AmPrintError( "Print Error (without shell) %d",
                                g_nPrintErrorWOShell++);

    }


    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: PrintResourceError( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintResourceError function prints an error using a resource 
 *     string.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the error
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintResourceError( Widget    wBtn,
                    XtPointer xClientData,
                    XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Resource Error Dialog (%d) ... ", 
             g_nPrintResourceError);

    wShell = ( Widget) xClientData;
    wDialog = AmPrintResourceError( wShell, "ResourceError",
                                    g_nPrintResourceError++);

    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: PrintWarn( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintWarn function prints an warning with or without a shell.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the warning
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintWarn( Widget    wBtn,
           XtPointer xClientData,
           XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Warning Dialog ");

    if ( xClientData != NULL) { 

        fprintf( stdout, "With Shell (%d) ... ", g_nPrintWarnWShell);

        wShell = ( Widget) xClientData;
        wDialog = AmPrintWarn( wShell, "Print Warning (with shell) %d",
                               g_nPrintWarnWShell++);

    } else {

        fprintf( stdout, "Without Shell (%d) ... ", g_nPrintWarnWOShell);
        wDialog = AmPrintWarn( "Print Warning (without shell) %d",
                               g_nPrintWarnWOShell++);

    }


    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: PrintResourceWarn( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintResourceWarn function prints an warning using a resource 
 *     string.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the warning
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintResourceWarn( Widget    wBtn,
                   XtPointer xClientData,
                   XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Resource Warning Dialog (%d) ... ",
             g_nPrintResourceWarn);

    wShell = ( Widget) xClientData;
    wDialog = AmPrintResourceWarn( wShell, "ResourceWarn",
                                   g_nPrintResourceWarn++);

    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: PrintInfo( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintInfo function prints information with or without a shell.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the information
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintInfo( Widget    wBtn,
           XtPointer xClientData,
           XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Information Dialog ");

    if ( xClientData != NULL) { 

        fprintf( stdout, "With Shell (%d) ... ", g_nPrintInfoWShell);

        wShell = ( Widget) xClientData;
        wDialog = AmPrintInfo( wShell, "Print Information (with shell) %d",
                               g_nPrintInfoWShell++);

    } else {

        fprintf( stdout, "Without Shell (%d) ... ", g_nPrintInfoWOShell);
        wDialog = AmPrintInfo( "Print Information (without shell) %d",
                               g_nPrintInfoWOShell++);

    }


    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: PrintResourceInfo( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintResourceInfo function prints information using a resource 
 *     string.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the information
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintResourceInfo( Widget    wBtn,
                   XtPointer xClientData,
                   XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Resource Information Dialog (%d) ... ",
             g_nPrintResourceInfo);

    wShell = ( Widget) xClientData;
    wDialog = AmPrintResourceInfo( wShell, "ResourceInfo",
                                   g_nPrintResourceInfo++);

    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: AddLog( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The AddLog function adds a new resource to the log file.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to add the log. 
 *     XtPointer xClientData : Unused. 
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
AddLog( Widget    wBtn,
        XtPointer xClientData,
        XtPointer xCallData)
{
    fprintf( stdout, "Adding Message to Log %d.\n", g_nLog);

    AmAddResourceToLog( "ResourceLog", g_nLog++);
}


/*
 * Function: PrintLog( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintLog function prints an error with a log file.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the error 
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintLog( Widget    wBtn,
          XtPointer xClientData,
          XtPointer xCallData)
{
    Widget wShell;
    Widget wDialog;

    fprintf( stdout, "Launching Error Log Dialog (%d) ... ",
             g_nPrintLogError);

    wShell = ( Widget) xClientData;
    wDialog = AmPrintLogError( wShell, "Print Log Error %d",
                               g_nPrintLogError++);

    if ( wDialog != NULL) {
        fprintf( stdout, "success.\n");
    } else {
        fprintf( stdout, "failure.\n");
    }
}


/*
 * Function: ClearLog( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The ClearLog function clears out the log detail file. 
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to add the log. 
 *     XtPointer xClientData : Unused. 
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
ClearLog( Widget    wBtn,
          XtPointer xClientData,
          XtPointer xCallData)
{
    fprintf( stdout, "Reseting Log.\n");

    AmInitLog();
}


/*
 * Function: Quit( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The Quit function forces the event loop to terminate on the 
 *     next itteration.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to quit the
 *                             application.
 *     XtPointer xClientData : Unused.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 *
 * Comments:
 *     The function simple sets the Run flag to FALSE which in turn forces
 *     the event loop to terminate.
 */

/* ARGSUSED */
static void
Quit( Widget    wBtn,
      XtPointer xClientData,
      XtPointer xCallData)
{
    fprintf( stdout, "Quitting the application.\n");
    g_fRun = FALSE;
}


/*
 * Function: Popdown( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The Popdown function brings down the dialog. 
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to popdwon the 
 *                             dialog.
 *     XtPointer xClientData : The dialog's top level widget. 
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
Popdown( Widget    wBtn,
         XtPointer xClientData,
         XtPointer xCallData)
{
    Widget wDialog = ( Widget) xClientData; 

    fprintf( stdout, "Popping down the dialog.\n");
    XtDestroyWidget( wDialog);
}


static Widget
MakeButton( Widget wParent,
            char   *szName,
            int    iLeft,
            int    iTop,
            int    iRight,
            int    iBottom)
{
    Widget wButton;

    wButton = XtVaCreateManagedWidget( szName, 
                                       xmPushButtonWidgetClass, wParent,
                                       XmNleftAttachment, XmATTACH_POSITION,
                                       XmNleftPosition, iLeft, 
                                       XmNtopAttachment, XmATTACH_POSITION,
                                       XmNtopPosition, iTop, 
                                       XmNrightAttachment, XmATTACH_POSITION,
                                       XmNrightPosition, iRight, 
                                       XmNbottomAttachment, XmATTACH_POSITION,
                                       XmNbottomPosition, iBottom, 
                                       NULL);

    return( wButton);
}


static Widget
CreateLaunchButtons( Widget  wTop,
                     AnvilBoolean fDialog)
{
    Widget wForm, wButton;
    int    iRight, iBottom, iFractionBase;

    if ( fDialog == TRUE) {
        iRight        = 72;
        iBottom       = 72;
        iFractionBase = 73;
    } else {
        iRight        = 67;
        iBottom       = 67;
        iFractionBase = 68;
    }

    wForm = XtVaCreateManagedWidget( "ErrorForm",
                                     xmFormWidgetClass, wTop,
                                     XmNfractionBase, iFractionBase,
                                     NULL);

    wButton = MakeButton( wForm, "Print Error (without shell)",
                          1, 1, iRight, 6);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintError,
                   NULL);

    wButton = MakeButton( wForm, "Print Error (with shell)",
                          1, 6, iRight, 11);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintError,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Print Resource Error",
                          1, 11, iRight, 16);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintResourceError,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Print Warning (without shell)",
                          1, 16, iRight, 21);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintWarn,
                   NULL);

    wButton = MakeButton( wForm, "Print Warning (with shell)",
                          1, 21, iRight, 26);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintWarn,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Print Resource Warning",
                          1, 26, iRight, 31);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintResourceWarn,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Print Info (without shell)",
                          1, 31, iRight, 36);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintInfo,
                   NULL);

    wButton = MakeButton( wForm, "Print Info (with shell)",
                          1, 36, iRight, 41);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintInfo,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Print Resource Info",
                          1, 41, iRight, 46);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintResourceInfo,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Add Log",
                          1, 46, iRight, 51);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) AddLog,
                   NULL);

    wButton = MakeButton( wForm, "Print Log Error",
                          1, 51, iRight, 56);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintLog,
                   ( XtPointer) wTop);

    wButton = MakeButton( wForm, "Clear Log",
                          1, 56, iRight, 61);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) ClearLog,
                   NULL);

    if ( fDialog == TRUE) {
        wButton = MakeButton( wForm, "Launch Dialog",
                              1, 61, iRight, 66);

        XtAddCallback( wButton, XmNactivateCallback,
                       ( XtCallbackProc) MakeDialog,
                       ( XtPointer) wTop);

        wButton = MakeButton( wForm, "Quit",
                              1, 67, iRight, iBottom);
        XtAddCallback( wButton, XmNactivateCallback,
                       ( XtCallbackProc) Quit,
                       NULL);
    } else {
        wButton = MakeButton( wForm, "Close",
                              1, 62, iRight, iBottom);
        XtAddCallback( wButton, XmNactivateCallback,
                       ( XtCallbackProc) Popdown,
                       ( XtPointer) wTop);
    }

    return( wForm);
}


/* ARGSUSED */
static void
MakeDialog( Widget    wBtn,
            XtPointer xClientData,
            XtPointer xCallData)
{
    Widget wShell, wParent = ( Widget) xClientData;

    wShell = XtVaCreatePopupShell( "Error Shell", 
                                   topLevelShellWidgetClass, wParent,
                                   XmNvisual, AmGetAppVisual(),
                                   XmNdepth, AmGetAppDepth(),
                                   XmNcolormap, AmGetAppColormap(),
                                   NULL);

    CreateLaunchButtons( wShell, FALSE);

    XtPopup( wShell, XtGrabNone);
}


/*
 * Function: main( nArgs, vszArgs)
 *
 * Purpose:
 *     The main function sets up the application and creates a 
 *     dialog which provides several buttons to test the various 
 *     error printing functions, it also provides a button to
 *     launch another dialog that also has buttons to print
 *     errors, finally there is a button to quit the test program.
 *
 * Parameters:
 *     int  nARgs    : The number of command line arguments passed in.
 *     char *vszArgs : The vector of command line arguments.
 *
 * Returns:
 *     int - whether the program executed without an error or not.
 */


int
main( int nArgs, char *vszArgs[])
{
    Widget       wTop;
    XtAppContext ac;
    XEvent       xEvent;

    fprintf( stdout, "Starting the application.\n");

    AmInitLog();

    wTop  = XtVaAppInitialize( &ac, "TestApp",
                               NULL, 0, 
                               &nArgs, vszArgs,
                               NULL, NULL);

    AmInitApplication( wTop);

    CreateLaunchButtons( wTop, TRUE);
  
    XtRealizeWidget( wTop);

    do {
        XtAppNextEvent( ac, &xEvent);
        XtDispatchEvent( &xEvent);
    } while ( g_fRun == TRUE);

    AmRemoveLogFile();

    fprintf( stdout, "Done.\n");

    return( 1);
}
