#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#include <anvil/colorcell.h>
#include <anvil/xapp.h>
#include <anvil/xcolor.h>
#include <anvil/visual.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/log.h>

/* Globals: */

static AnvilBoolean g_fRun            = TRUE;


/*
 * Function: PrintTree( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The PrintError function prints an error with or without a shell.
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to launch the
 *                             dialog.
 *     XtPointer xClientData : The shell to use as a parent to the error
 *                             dialog.
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
PrintTree( Widget    wTest,
           XtPointer xClientData,
           XtPointer xCallData)
{
    Widget   *vChildren, w;
    unsigned nChildren, ii; 

    fprintf( stdout, "Widget: %s\n", XtName( wTest));

    fprintf( stdout, "Resource Name: %s\n", AmGetResourceName( wTest));

    fprintf( stdout, "Getting children:\n");

    AmGetWidgetChildren( wTest, &vChildren, &nChildren); 

    if ( nChildren == 0) {
        fprintf( stdout, "    No children.\n");
    } else {
        for ( ii = 0; ii < nChildren; ii++) {
            fprintf( stdout, "    %s.\n", XtName( vChildren[ ii]));
        }
    }

    w = AmGetWidgetShell( wTest);
    fprintf( stdout, "Widget Shell: %s\n",
             w == NULL ? "None" : XtName( w));

    w = AmGetWidgetRealShell( wTest);
    fprintf( stdout, "Widget Real Shell: %s\n",
             w == NULL ? "None" : XtName( w));

    w = AmGetTopLevelShell( wTest);
    fprintf( stdout, "Top Level Shell: %s\n",
             w == NULL ? "None" : XtName( w));

    w = AmGetParentOfType( wTest, xmPushButtonWidgetClass);
    fprintf( stdout, "Closest widget of type Button: %s\n",
             w == NULL ? "None" : XtName( w));

    w = AmGetParentOfType( wTest, xmFormWidgetClass);
    fprintf( stdout, "Closest widget of type Form: %s\n",
             w == NULL ? "None" : XtName( w));

    fprintf( stdout, "\n");

    for ( ii = 0; ii < nChildren; ii++) {
        PrintTree( vChildren[ ii], NULL, NULL);
    }

    free( vChildren);
}

/* ARGSUSED */
static void
Quit( Widget    wBtn,
      XtPointer xClientData,
      XtPointer xCallData)
{
    AnvilBoolean *pfRun = ( AnvilBoolean *) xClientData; 

    fprintf( stdout, "Quitting the application.\n");
    *pfRun = FALSE;
}


/*
 * Function: Popdown( wBtn, xClientData, xCallData) 
 *
 * Purpose:
 *     The Popdown function brings down the dialog. 
 *
 * Parameters:
 *     Widget    wBtn        : The button that was pressed to popdwon the 
 *                             dialog.
 *     XtPointer xClientData : The dialog's top level widget. 
 *     XtPointer xCallData   : The callback data associated with an activate
 *                             callback.
 */

/* ARGSUSED */
static void
Popdown( Widget    wBtn,
         XtPointer xClientData,
         XtPointer xCallData)
{
    Widget wDialog = ( Widget) xClientData; 

    fprintf( stdout, "Popping down the dialog.\n");
    XtDestroyWidget( wDialog);
}


static Widget
MakeForm( Widget wParent,
          char   *szName,
          int    iLeft,
          int    iTop,
          int    iRight,
          int    iBottom)
{
    Widget wForm;

    wForm = XtVaCreateManagedWidget( szName, 
                                     xmFormWidgetClass, wParent,
                                     XmNleftAttachment, XmATTACH_POSITION,
                                     XmNleftPosition, iLeft, 
                                     XmNtopAttachment, XmATTACH_POSITION,
                                     XmNtopPosition, iTop, 
                                     XmNrightAttachment, XmATTACH_POSITION,
                                     XmNrightPosition, iRight, 
                                     XmNbottomAttachment, XmATTACH_POSITION,
                                     XmNbottomPosition, iBottom, 
                                     XmNfractionBase, 21,
                                     NULL);

    return( wForm);
}


static Widget
MakeButton( Widget wParent,
            char   *szName)
{
    Widget wButton;

    wButton = XtVaCreateManagedWidget( szName, 
                                       xmPushButtonWidgetClass, wParent,
                                       XmNleftAttachment, XmATTACH_FORM,
                                       XmNtopAttachment, XmATTACH_FORM,
                                       XmNrightAttachment, XmATTACH_FORM,
                                       XmNbottomAttachment, XmATTACH_FORM,
                                       NULL);

    XtAddCallback( wButton, XmNactivateCallback,
                   ( XtCallbackProc) PrintTree,
                   NULL);

    return( wButton);
}


static void
MakeButtons( Widget wForm, int nDepth)
{
    Widget wFormTopLeft, wFormBottomLeft,
           wFormTopRight, wFormBottomRight;
    Widget wButton;

    wFormTopLeft     = MakeForm( wForm, "TopLeftForm", 1, 1, 10, 10);
    wFormBottomLeft  = MakeForm( wForm, "BottomLeftForm", 1, 11, 10, 20);
    wFormTopRight    = MakeForm( wForm, "TopRightForm", 11, 1, 20, 10);

    wFormBottomRight = MakeForm( wForm, "BottomRightForm", 11, 11, 20, 20);

    wFormBottomRight = MakeForm( wFormBottomRight, "Form", 0, 0, 21, 21); 
    wFormBottomRight = MakeForm( wFormBottomRight, "Form", 0, 0, 21, 21); 
    wFormBottomRight = MakeForm( wFormBottomRight, "Form", 0, 0, 21, 21); 
    wFormBottomRight = MakeForm( wFormBottomRight, "Form", 0, 0, 21, 21); 

    if ( nDepth <= 0) {

        wButton = MakeButton( wFormTopLeft, "1");
        wButton = MakeButton( wFormBottomLeft, "3");
        wButton = MakeButton( wFormTopRight, "2");
        wButton = MakeButton( wFormBottomRight, "4");
 
    } else {

        MakeButtons( wFormTopLeft, nDepth - 1);
        MakeButtons( wFormBottomRight, nDepth - 1);
        MakeButtons( wFormBottomLeft, nDepth - 1);
        MakeButtons( wFormTopRight, nDepth - 1);

    }
}


/* ARGSUSED */
static void
MakeDialog( Widget    wBtn,
            XtPointer xClientData,
            XtPointer xCallData)
{
    Widget wShell, wParent = ( Widget) xClientData;

    wShell = XtVaCreatePopupShell( "Error Shell", 
                                   topLevelShellWidgetClass, wParent,
                                   XmNvisual, AmGetAppVisual(),
                                   XmNdepth, AmGetAppDepth(),
                                   XmNcolormap, AmGetAppColormap(),
                                   NULL);

    XtPopup( wShell, XtGrabNone);
}


/*
 * Function: main( nArgs, vszArgs)
 *
 * Purpose:
 *     The main function sets up the application and creates a 
 *     dialog which provides several buttons to test the various 
 *     error printing functions, it also provides a button to
 *     launch another dialog that also has buttons to print
 *     errors, finally there is a button to quit the test program.
 *
 * Parameters:
 *     int  nARgs    : The number of command line arguments passed in.
 *     char *vszArgs : The vector of command line arguments.
 *
 * Returns:
 *     int - whether the program executed without an error or not.
 */


int
main( int nArgs, char *vszArgs[])
{
    Widget       wTop, wForm, wFormTopLeft, wFormTopRight, wFormBottomLeft,
                 wFormBottomRight;
    XtAppContext ac;
    XEvent       xEvent;

    fprintf( stdout, "Starting the application.\n");

    wTop  = XtVaAppInitialize( &ac, "TestApp",
                               NULL, 0, 
                               &nArgs, vszArgs,
                               NULL, NULL);

    wForm = XtVaCreateManagedWidget( "GetTreeForm",
                                     xmFormWidgetClass, wTop,
                                     XmNfractionBase, 21,
                                     NULL);

    MakeButtons( wForm, 2);

    AmInitApplication( wTop);

    XtRealizeWidget( wTop);

    PrintTree( wTop, NULL, NULL);

//   do {
//       XtAppNextEvent( ac, &xEvent);
//       XtDispatchEvent( &xEvent);
//   } while ( g_fRun == TRUE);

    fprintf( stdout, "Done.\n");

    return( 1);
}
