#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <anvil/geometry.h>


int main( int argc, char *argv[])
{
    FILE		*fp;
    float		x0, y0, z0, x1, y1, z1, x2, y2, x3, y3, x, y;
    AnvilBoolean		isectFlag;

    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open %s\n", argv[ 1]);
	exit( -1);
    }
    if ( fscanf( fp, "%f %f %f %f %f %f", 
		 &x0, &y0, &z0, &x1, &y1, &z1) != 6) {
	fprintf( stderr, "failed to scan xyz values\n");
	exit( -1);
    }
    if ( fscanf( fp, "%f %f %f %f", &x2, &y2, &x3, &y3) != 4) {
	fprintf( stderr, "failed to scan xy values\n");
	exit( -1);
    }
    fclose( fp);

/* distance tests */

    printf( "dist( %.2f, %.2f) -> ( %.2f, %.2f) = %.2f\n", 
	    x0, y0, x1, y1, AmDist( x0, y0, x1, y1));
    printf( "dist( %.2f, %.2f, %.2f) -> ( %.2f, %.2f, %.2f) = %.2f\n", 
	    x0, y0, z0, x1, y1, z1, AmDist3D( x0, y0, z0, x1, y1, z1));
    printf( "sqrdist( %.2f, %.2f) -> ( %.2f, %.2f) = %.2f\n", 
	    x0, y0, x1, y1, AmSqrDist( x0, y0, x1, y1));
    
/* line distance/angle tests */

    printf( "linedist (%.2f, %.2f), ( %.2f, %.2f) => ( %.2f, %.2f) = %.2f\n", 
	    x2, y2, x0, y0, x1, y1, AmLineDist( x0, y0, x1, y1, x2, y2));
    printf( "lineangle (%.2f, %.2f), ( %.2f, %.2f) => ( %.2f, %.2f) = %.2f\n", 
	    x2, y2, x0, y0, x1, y1, AmLineAngle( x0, y0, x1, y1, x2, y2));

/* line intersection test */

    isectFlag = AmLineIntersection( x0, y0, x1, y1, x2, y2, x3, y3, &x, &y);
    printf( "intersect 	(%.2f, %.2f), ( %.2f, %.2f)\n", x0, y0, x1, y1);
    printf( "		(%.2f, %.2f), ( %.2f, %.2f)\n", x2, y2, x3, y3);
    if ( isectFlag) {
	printf( "intersects at %.2f, %.2f\n", x, y);
    } else {
	printf( "does not intersect at %.2f, %.2f\n", x, y);
    }
}
