#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <anvil/xdisplay.h>
#include <anvil/image.h>

int main( int argc, char *argv[])
{
    int		srcRows, srcCols, srcX, srcY, srcWidth, srcHeight,
		dstRows, dstCols, dstX, dstY, ii, jj, fd, 
		dstRectX, dstRectY, dstWidth, dstHeight;
    AnvilBoolean 	flipRows, flipCols;
    FILE	*fp;
    char	flipRowsStr[ 256], flipColsStr[ 256], filename[ 512];
    float	*srcMatrix, *dstMatrix;
    AmRectangleRec 	srcRect, dstRect;
    AmErrorCode		errCode;

/* read the srcMatrix dimensions and source and destiation rectangles
   from the source file */

    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open %s\n", argv[ 1]);
	exit( -1);
    }
    if ( fscanf( fp, "%d %d %d %d %d %d %d %d %d %d",
		 &srcRows, &srcCols, &srcX, &srcY, &srcWidth,
		 &srcHeight, &dstRows, &dstCols, &dstX, &dstY) != 10) {
	fprintf( stderr, "failed to scan srcMatrix dims from file\n");
	exit( -1);
    }
    srcRect.x = srcX;
    srcRect.y = srcY;
    srcRect.width = srcWidth;
    srcRect.height = srcHeight;

/* read the destination rectangle from the source file */

    if ( fscanf( fp, "%d %d %d %d",
		 &dstRectX, &dstRectY, &dstWidth, &dstHeight) != 4) {
	fprintf( stderr, "failed to scan destination rect from file\n");
	exit( -1);
    }
    dstRect.x = dstRectX;
    dstRect.y = dstRectY;
    dstRect.width = dstWidth;
    dstRect.height = dstHeight;

/* read the srcMatrix flip flags from the file */

    if ( fscanf( fp, "%s %s", flipRowsStr, flipColsStr) != 2) {
	fprintf( stderr, "failed to scan flips from file\n");
	exit( -1);
    }
    fclose( fp);
    flipRows = !strcasecmp( flipRowsStr, "true");
    flipCols = !strcasecmp( flipColsStr, "true");

/* allocate the srcMatrix */

    srcMatrix = ( float *) malloc( srcRows*srcCols*sizeof( float));
    if ( srcMatrix == NULL) {
	fprintf( stderr, "failed to allocate %d bytes\n", 
		 srcRows*srcCols*sizeof( float));
	exit( -1);
    }
    dstMatrix = ( float *) malloc( dstRows*dstCols*sizeof( float));
    if ( dstMatrix == NULL) {
	fprintf( stderr, "failed to allocate %d bytes\n", 
		 dstRows*dstCols*sizeof( float));
	exit( -1);
    }

/* fill the srcMatrix with sine-waves */

    for ( ii = 0; ii < srcRows; ii++) {
	for ( jj = 0; jj < srcCols; jj++) {
	    srcMatrix[ ii*srcCols + jj] = 
	        sin( ii*M_PI*4/( float) srcRows) +
	        cos( jj*M_PI*4/( float) srcCols);
	}
    }

/* fill the destination matrix to prevent UMR's */

    for ( ii = 0; ii < dstRows; ii++) {
	for ( jj = 0; jj < dstCols; jj++) {
	    dstMatrix[ ii*dstCols + jj] = 0;
	}
    }


/* copy submatrix tests */

    printf( "resize submatrix args\n");
    printf( "srcRows = %d, srcCols = %d\n", srcRows, srcCols);
    printf( "srcRect = (%d, %d, %d, %d)\n", 
	    srcRect.x, srcRect.y, srcRect.width, srcRect.height);
    printf( "dstRows = %d, dstCols = %d\n", dstRows, dstCols);
    printf( "dstX = %d, dstY = %d\n", dstX, dstY);
    printf( "flipRows = %d, flipCols = %d\n", flipRows, flipCols);
	    
    errCode = AmCopySubmatrix(( uchar_t *) srcMatrix, 
			      srcRows, srcCols, &srcRect,
			      ( uchar_t *) dstMatrix, 
			      dstRows, dstCols, dstX, dstY,
			      sizeof( float), flipRows, flipCols);
    if ( errCode != AmOK) {
	fprintf( stderr, "AmCopySubmatrix failed\n");
	exit( -1);
    }
    sprintf( filename, "%s1.hdr", argv[ 1]);
    if (( fp = fopen( filename, "w")) == NULL) {
	fprintf( stderr, "failed to open output header file %s\n", filename);
	exit( -1);
    }
    fprintf( fp, "filename:	%s\n", filename);
    fprintf( fp, "rows:		%d\n", dstRows);
    fprintf( fp, "cols:		%d\n", dstCols);
    fprintf( fp, "size:		%d\n", sizeof( float));
    fprintf( fp, "dataType:	float\n");
    fclose( fp);
    sprintf( filename, "%s1.dat", argv[ 1]);
    if (( fd = open( filename, O_CREAT | O_WRONLY, 0666)) == -1) {
	fprintf( stderr, "failed to open output data file %s\n", filename);
	exit( -1);
    }
    write( fd, dstMatrix, dstRows*dstCols*sizeof( float));
    close( fd);

/* resize submatrix tests */

    printf( "copy submatrix args\n");
    printf( "srcRows = %d, srcCols = %d\n", srcRows, srcCols);
    printf( "srcRect = (%d, %d, %d, %d)\n", 
	    srcRect.x, srcRect.y, srcRect.width, srcRect.height);
    printf( "dstRows = %d, dstCols = %d\n", dstRows, dstCols);
    printf( "dstRect = (%d, %d, %d, %d)\n", 
	    dstRect.x, dstRect.y, dstRect.width, dstRect.height);
	    
    errCode = AmResizeMatrix(( uchar_t *) srcMatrix,
			     srcRows, srcCols, 
			     ( uchar_t *) dstMatrix,
			     dstRows, dstCols,
			     sizeof( float),
			     &srcRect, &dstRect);
    if ( errCode != AmOK) {
	fprintf( stderr, "AmResizeMatrix failed\n");
	exit( -1);
    }
    sprintf( filename, "%s2.hdr", argv[ 1]);
    if (( fp = fopen( filename, "w")) == NULL) {
	fprintf( stderr, "failed to open output header file %s\n", filename);
	exit( -1);
    }
    fprintf( fp, "filename:	%s\n", filename);
    fprintf( fp, "rows:		%d\n", dstRows);
    fprintf( fp, "cols:		%d\n", dstCols);
    fprintf( fp, "size:		%d\n", sizeof( float));
    fprintf( fp, "dataType:	float\n");
    fclose( fp);
    sprintf( filename, "%s2.dat", argv[ 1]);
    if (( fd = open( filename, O_CREAT | O_WRONLY, 0666)) == -1) {
	fprintf( stderr, "failed to open output data file %s\n", filename);
	exit( -1);
    }
    write( fd, dstMatrix, dstRows*dstCols*sizeof( float));
    close( fd);
    free( srcMatrix);
    free( dstMatrix);
}
