#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <anvil/template.h>
#include <anvil/motif.h>
#include <anvil/xdisplay.h>
#include <anvil/log.h>
#include <anvil/copyobj.h>
#include <anvil/xapp.h>

AmScreenObjClass	*screenObj, *copyScreenObj, *traceScreenObj;
AmXDisplayClass		*xDisplay;
AmXWindowClass		*xWindow;
AmXGCClass		*xGC;
FILE			*logFp, *dumpFp;
const char		*logFile; 
char			errorBuf[ 4096], filename[ 256], *objectName;
Widget			top, form, area;
AnvilBoolean			firstFlag = TRUE;

static void exposeCallback( Widget	area,
			    XtPointer,
			    XtPointer)
{
printf( "in expose\n");
    if ( firstFlag) {

printf( "initializing objects\n");

	xDisplay = new AmXDisplayClass( area);
	xWindow = new AmXWindowClass( xDisplay, area);
	xGC = ( AmXGCClass *) xWindow->CreateGC();
	AmInitLog();

/* read the object from the header */

printf( "creating layer from header\n");
	if ( AmAddLayerFromHeader( xWindow, objectName, NULL, "NetMentor",
				   &screenObj) != AmOK) {
	    printf( "failed to create object from %s\n", objectName);
	    logFile = AmGetLogFile();
	    logFp = fopen( logFile, "r");
	    if ( logFp != NULL) {
		fread( errorBuf, 1, sizeof( errorBuf), logFp);
		fclose( logFp);
		printf( "error log:\n%s\n", errorBuf);
	    }
	    exit( -1);
	}
	if ( screenObj->IsValid()) {
	    printf( "%s is valid\n", screenObj->getObjectName());
	} else {
	    printf( "%s is not valid\n", screenObj->getObjectName());
	}

/* copy the object */

printf( "copying object\n");
	copyScreenObj = AmCopyObject( xWindow, screenObj, NULL);
	sprintf( filename, "%s.copyObject", objectName);
printf( "dumping copied object\n");
	dumpFp = fopen( filename, "w");
	if ( dumpFp == NULL) {
	    fprintf( stderr, "failed to open output file %s\n", filename);
	    exit( -1);
	}
	copyScreenObj->DumpFields( dumpFp);
	fclose( dumpFp);
printf( "dumped copied object\n");

    /* trace the object */
printf( "tracing object\n");
	traceScreenObj = AmTraceObject( xWindow, screenObj, NULL);
	sprintf( filename, "%s.traceObject", objectName);
	dumpFp = fopen( filename, "w");
	traceScreenObj->DumpFields( dumpFp);
	if ( dumpFp == NULL) {
	    fprintf( stderr, "failed to open output file %s\n", filename);
	    exit( -1);
	}
	fclose( dumpFp);
printf( "dumping traced object\n");
	firstFlag = FALSE;

/* update and draw the object */

	screenObj->Update();
	screenObj->Draw( xWindow, xGC, screenObj->GetRect(), 1);
printf( "destroying shell\n");
	XtDestroyWidget( AmGetWidgetShell( area));
    }
}

static void 
destroyCallback( Widget	area,
		 XtPointer,
		 XtPointer)
{
    printf( "in destroy\n");
    delete screenObj;
    delete copyScreenObj;
#if 0
    delete traceScreenObj;
#endif
    delete xGC;
    delete xWindow;
    delete xDisplay;
    exit( 0);
}

int main( int argc, char *argv[])
{
    top = XtInitialize ( "testplot", "testplot-db", NULL, 0, &argc, argv);
    objectName = argv[ 1];
    AmInitApplication( top);
    form = XtVaCreateManagedWidget( "form", xmFormWidgetClass, top,
				    NULL);
    area =  XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, form,
				     XmNwidth, 200,
				     XmNheight, 200,
				     NULL);
    XtRealizeWidget( top);
    XtAddCallback( area, XmNexposeCallback,
		   ( XtCallbackProc) exposeCallback, NULL);
    XtAddCallback( area, XmNdestroyCallback,
		   ( XtCallbackProc) destroyCallback, NULL);
    XtMainLoop();
}
