#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <anvil/xdisplay.h>
#include <anvil/image.h>

static AnvilBoolean readScanline( int	fd,
			     uchar_t	*buffer,
			     int	size,
			     int	length,
			     int	cols,
			     int	x,
			     int	y,
			     void	*clientData)
{
    ulong_t	offset;

    offset = (y*cols + x)*size;

    if ( lseek( fd, offset, SEEK_SET) < 0) {
	return( FALSE);
    }
    if ( read( fd, buffer, length*size) < length*size) {
	return( FALSE);
    }
    return( TRUE);
}


int main( int argc, char *argv[])
{
    int			srcRows, srcCols, srcX, srcY, srcWidth, srcHeight,
			dstRows, dstCols, dstX, dstY, ii, jj, fd, 
			dstRectX, dstRectY, dstWidth, dstHeight;
    AnvilBoolean 		flipRows, flipCols;
    FILE		*fp;
    char		flipRowsStr[ 256], flipColsStr[ 256], filename[ 512];
    float		*srcMatrix, *dstMatrix;
    AmRectangleRec 	srcRect, dstRect;
    AmErrorCode		errCode;
    AmRealRectRec	srcRealRect;

/* read the srcMatrix dimensions and source and destiation rectangles
   from the source file */

    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open %s\n", argv[ 1]);
	exit( -1);
    }
    if ( fscanf( fp, "%d %d %d %d %d %d %d %d %d %d",
		 &srcRows, &srcCols, &srcX, &srcY, &srcWidth,
		 &srcHeight, &dstRows, &dstCols, &dstX, &dstY) != 10) {
	fprintf( stderr, "failed to scan source dims from file\n");
	exit( -1);
    }
    srcRect.x = srcX;
    srcRect.y = srcY;
    srcRect.width = srcWidth;
    srcRect.height = srcHeight;

/* read the destination rectangle from the source file */

    if ( fscanf( fp, "%d %d %d %d",
		 &dstRectX, &dstRectY, &dstWidth, &dstHeight) != 4) {
	fprintf( stderr, "failed to scan destination rect from file\n");
	exit( -1);
    }
    dstRect.x = dstRectX;
    dstRect.y = dstRectY;
    dstRect.width = dstWidth;
    dstRect.height = dstHeight;

/* read the matrix read rows source rectangle from the source file */

    if ( fscanf( fp, "%f %f %f %f",
		 &srcRealRect.x, &srcRealRect.y, 
		 &srcRealRect.width, &srcRealRect.height) != 4) {
	fprintf( stderr, "failed to scan source rect from file\n");
	exit( -1);
    }

/* read the srcMatrix flip flags from the file */

    if ( fscanf( fp, "%s %s", flipRowsStr, flipColsStr) != 2) {
	fprintf( stderr, "failed to scan flips from file\n");
	exit( -1);
    }
    fclose( fp);
    flipRows = !strcasecmp( flipRowsStr, "true");
    flipCols = !strcasecmp( flipColsStr, "true");

    dstMatrix = ( float *) malloc( dstRows*dstCols*sizeof( float));
    if ( dstMatrix == NULL) {
	fprintf( stderr, "failed to allocate %d bytes\n", 
		 dstRows*dstCols*sizeof( float));
	exit( -1);
    }
    srcMatrix = ( float *) malloc( srcRows*srcCols*sizeof( float));
    if ( srcMatrix == NULL) {
	fprintf( stderr, "failed to allocate %d bytes\n", 
		 srcRows*srcCols*sizeof( float));
	exit( -1);
    }

/* fill the srcMatrix with sine-waves */

    for ( ii = 0; ii < srcRows; ii++) {
	for ( jj = 0; jj < srcCols; jj++) {
	    srcMatrix[ ii*srcCols + jj] = 
	        sin( ii*M_PI*4/( float) srcRows) +
	        cos( jj*M_PI*4/( float) srcCols);
	}
    }
    if (( fd = open( "testmatfile", O_WRONLY | O_CREAT, 0666)) < 0) {
	fprintf( stderr, "failed to open testmatfile for writing\n");
	exit( -1);
    }
    write( fd, srcMatrix, srcRows*srcCols*sizeof( float));
    close( fd);
    free( srcMatrix);

/* fill the destination matrix to prevent UMR's */

    for ( ii = 0; ii < dstRows; ii++) {
	for ( jj = 0; jj < dstCols; jj++) {
	    dstMatrix[ ii*dstCols + jj] = 0;
	}
    }

/* copy submatrix tests */

    printf( "AmReadSubmatrix args\n");
    printf( "srcRows = %d, srcCols = %d\n", srcRows, srcCols);
    printf( "srcRect = (%d, %d, %d, %d)\n", 
	    srcRect.x, srcRect.y, srcRect.width, srcRect.height);
    printf( "dstRows = %d, dstCols = %d\n", dstRows, dstCols);
    printf( "dstX = %d, dstY = %d\n", dstX, dstY);
	
    if (( fd = open( "testmatfile", O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open testmatfile for reading\n");
	exit( -1);
    }
    errCode = AmReadSubmatrix( fd, srcRows, srcCols, &srcRect,
			      ( uchar_t *) dstMatrix, 
			      dstRows, dstCols, dstX, dstY, sizeof( float), 
			      ( AmScanlineFn) readScanline, NULL);
    if ( errCode != AmOK) {
	fprintf( stderr, "AmReadSubmatrix failed\n");
	exit( -1);
    }
    close( fd);
    sprintf( filename, "%s1.hdr", argv[ 1]);
    if (( fp = fopen( filename, "w")) == NULL) {
	fprintf( stderr, "failed to open output header file %s\n", filename);
	exit( -1);
    }
    fprintf( fp, "filename:	%s\n", filename);
    fprintf( fp, "rows:		%d\n", dstRows);
    fprintf( fp, "cols:		%d\n", dstCols);
    fprintf( fp, "size:		%d\n", sizeof( float));
    fprintf( fp, "dataType:	float\n");
    fclose( fp);
    sprintf( filename, "%s1.dat", argv[ 1]);
    if (( fd = open( filename, O_CREAT | O_WRONLY, 0666)) == -1) {
	fprintf( stderr, "failed to open output data file %s\n", filename);
	exit( -1);
    }
    write( fd, dstMatrix, dstRows*dstCols*sizeof( float));
    close( fd);

/* fill the destination matrix to prevent UMR's */

    for ( ii = 0; ii < dstRows; ii++) {
	for ( jj = 0; jj < dstCols; jj++) {
	    dstMatrix[ ii*dstCols + jj] = 0;
	}
    }

/* resize submatrix tests */

    printf( "AmMatrixReadRows args\n");
    printf( "srcRows = %d, srcCols = %d\n", srcRows, srcCols);
    printf( "srcRealRect = (%.2f, %.2f, %.2f, %.2f)\n", 
	    srcRealRect.x, srcRealRect.y, 
	    srcRealRect.width, srcRealRect.height);
    printf( "dstRows = %d, dstCols = %d\n", dstRows, dstCols);
    printf( "dstRect = (%d, %d, %d, %d)\n", 
	    dstRect.x, dstRect.y, dstRect.width, dstRect.height);
	    
    if (( fd = open( "testmatfile", O_RDONLY)) < 0) {
	fprintf( stderr, "failed to open testmatfile for reading\n");
	exit( -1);
    }
    errCode = AmMatrixReadRows( fd, srcRows, srcCols, &srcRealRect,
				( uchar_t *) dstMatrix, dstRows, dstCols,
				&dstRect, sizeof( float), flipRows, flipCols,
			        ( AmScanlineFn) readScanline, NULL);
			     
    if ( errCode != AmOK) {
	fprintf( stderr, "AmMatrixReadRows failed\n");
	exit( -1);
    }
    close( fd);
    sprintf( filename, "%s2.hdr", argv[ 1]);
    if (( fp = fopen( filename, "w")) == NULL) {
	fprintf( stderr, "failed to open output header file %s\n", filename);
	exit( -1);
    }
    fprintf( fp, "filename:	%s\n", filename);
    fprintf( fp, "rows:		%d\n", dstRows);
    fprintf( fp, "cols:		%d\n", dstCols);
    fprintf( fp, "size:		%d\n", sizeof( float));
    fprintf( fp, "dataType:	float\n");
    fclose( fp);
    sprintf( filename, "%s2.dat", argv[ 1]);
    if (( fd = open( filename, O_CREAT | O_WRONLY, 0666)) == -1) {
	fprintf( stderr, "failed to open output data file %s\n", filename);
	exit( -1);
    }
    write( fd, dstMatrix, dstRows*dstCols*sizeof( float));
    close( fd);
    free( dstMatrix);
}
