#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <anvil/rect.h>


int main( int argc, char *argv[])
{
    FILE		*fp;
    float		rectX, rectY, rectHeight, rectWidth, ptX, ptY;
    int			rectCount, ii;
    AmRealRectRec	rects[ 100], rectA, rectB, intersectRect,
			flipRect;
    AnvilBoolean		intersectFlag;
    AmRectangleRec	intRect;

    if ( argc < 4) {
	fprintf( stderr, "usage %s rectfile X Y\n", argv[ 0]);
	exit( -1);
    }
    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open file %s\n", argv[ 1]);
	exit( -1);
    }
    rectCount = 0;
    do {
	if ( fscanf( fp, "%f %f %f %f", 
		     &rectX, &rectY, &rectWidth, &rectHeight) != 4) {
	    break;
	}
	rects[ rectCount].x = rectX;
	rects[ rectCount].y = rectY;
	rects[ rectCount].width = rectWidth;
	rects[ rectCount].height = rectHeight;
	rectCount++;
    } while ( TRUE);
    fclose( fp);
    printf( "rectCount = %d\n", rectCount);
    ptX = atof( argv[ 2]);
    ptY = atof( argv[ 3]);
    rectA = rects[ 0];
    rectB = rects[ 1];

/* point in real rect tests */

    printf( "point in realRect tests\n");
    for ( ii = 0; ii < rectCount; ii++) {
	if ( AmInRealRect( &rects[ ii], ptX, ptY)) {
	    printf( "(%.2f, %.2f) is in (%.2f, %.2f, %.2f, %.2f)\n",
		    ptX, ptY, rects[ ii].x, rects[ ii].y, 
		    rects[ ii].width, rects[ ii].height);
	} else {
	    printf( "(%.2f, %.2f) is not in (%.2f, %.2f, %.2f, %.2f)\n",
		    ptX, ptY, rects[ ii].x, rects[ ii].y, 
		    rects[ ii].width, rects[ ii].height);
	}
    }

/* intersection tests */

    printf( "intersection tests\n");
    rectA = rects[ 0];
    rectB = rects[ 1];
    intersectFlag = AmIntersectRect( &rectA, &rectB, &intersectRect);
    printf( "intersection (%.2f, %.2f, %.2f, %.2f), (%.2f, %.2f, %.2f %.2f) =",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    rectB.x, rectB.y, rectB.width, rectB.height);
    if ( intersectFlag) {
	printf( " (%.2f, %.2f, %.2f, %.2f)\n", 
		intersectRect.x, intersectRect.y, 
		intersectRect.width, intersectRect.height);
    } else {
	printf( "does not intersect\n");
    }

/* make int tests */

    printf( "make int tests\n");
    rectA = rects[ 0];
    AmMakeIntRect( &rectA, &intRect);
    printf( "intRect (%.2f, %.2f, %.2f, %.2f) = (%d, %d, %d %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    intRect.x, intRect.y, intRect.width, intRect.height);
/* flip tests */

    printf( "flip tests\n");
    flipRect = rectA;
    AmApplyFlips( &flipRect, TRUE, TRUE);
    printf( "flip (1,1) (%.2f, %.2f, %.2f, %.2f) = (%.2f, %.2f, %.2f, %.2f)\n",
            rectA.x, rectA.y, rectA.width, rectA.height,
            flipRect.x, flipRect.y, flipRect.width, flipRect.height);
    flipRect = rectA;
    AmApplyFlips( &flipRect, TRUE, FALSE);
    printf( "flip (1,0) (%.2f, %.2f, %.2f, %.2f) = (%.2f, %.2f, %.2f, %.2f)\n",
            rectA.x, rectA.y, rectA.width, rectA.height,
            flipRect.x, flipRect.y, flipRect.width, flipRect.height);
    flipRect = rectA;
    AmApplyFlips( &flipRect, FALSE, TRUE);
    printf( "flip (0,1) (%.2f, %.2f, %.2f, %.2f) = (%.2f, %.2f, %.2f, %.2f)\n",
            rectA.x, rectA.y, rectA.width, rectA.height,
            flipRect.x, flipRect.y, flipRect.width, flipRect.height);
    flipRect = rectA;
    AmApplyFlips( &flipRect, FALSE, FALSE);
    printf( "flip (0,0) (%.2f, %.2f, %.2f, %.2f) = (%.2f, %.2f, %.2f, %.2f)\n",
            rectA.x, rectA.y, rectA.width, rectA.height,
            flipRect.x, flipRect.y, flipRect.width, flipRect.height);
}
