#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <anvil/files.h>
#include <anvil/tagstrs.h>
#include <anvil/strlib.h>
#include <anvil/xdisplay.h>
#include <anvil/screenobj.h>


int main( int argc, char *argv[])
{
    AmScreenObjClass	*screenObj;
    AmXDisplayClass	xDisplay;
    AmXWindowClass	xWindow( &xDisplay, ( Window) NULL);
    char		typeName[ 128], buffer[ 1024];
    FILE		*fp;
    AmTagValueRec	tagValues[ 1000];
    int			numValues, ii;
    AmDataTypeEnum	dataType;

/* create the default lookup table */

    if ( AmCreateLookupTable() != AmOK) {
	printf( "AmCreateLookupTable failed\n");
    }

    if ( AmInitUnitStuff() != AmOK) {
	printf( "AmInitUnitStuff failed\n");
    }

/* create a screen object from the header (just as a derived class 
   from record class) */


    if ( AmIsHeader( argv[ 1], typeName)) {
	printf( "%s is a header of type %s\n", argv[ 1], typeName);
    } else {
	printf( "%s is not a header\n", argv[ 1]);
	exit( -1);
    }
    screenObj = new AmScreenObjClass( &xWindow, "screenObj",
				      AmNheader, argv[ 1],
				      NULL);

/* write the screen objects contents out as a header file */

    if (( fp = fopen( "headertestfile", "w")) == NULL) {
	fprintf( stderr, "failed to open header test file\n");
	exit( -1);
    }
    if ( !screenObj->WriteHeader( fp)) {
	printf( "write header failed\n");
    } else {
	printf( "write header succeeded\n");
    }
    fclose( fp);

/* dump the screen object contents out as a dump file */

    if (( fp = fopen( "dumptestfile", "w")) == NULL) {
	fprintf( stderr, "failed to open header test file\n");
	exit( -1);
    }
    if ( !screenObj->DumpFields( fp)) {
	printf( "dump header failed\n");
    } else {
	printf( "dump header succeeded\n");
    }
    fclose( fp);

/* retrieve the tag values into a tag-value array and change from get
   to set, then reset them and dump the screen object again */

    screenObj->RetreiveTagValues( tagValues, &numValues);
    AmChangeGetToSet( tagValues, numValues);
    numValues = screenObj->setFieldsWrapper( tagValues, numValues);

    if (( fp = fopen( "dumptestfile2", "w")) == NULL) {
	fprintf( stderr, "failed to open header test file\n");
	exit( -1);
    }
    if ( !screenObj->DumpFields( fp)) {
	printf( "dump header failed\n");
    } else {
	printf( "dump header succeeded\n");
    }
    fclose( fp);
    delete screenObj;
}
