#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <anvil/rect.h>


int main( int argc, char *argv[])
{
    FILE		*fp;
    int			rectX, rectY, rectHeight, rectWidth, rectCount,
			numRects, ii, numDiffRects, ptX, ptY;
    AmRectangleRec	rects[ 100], rectA, rectB, clipRect,
			clipRectVec[ 100], unionRect, intersectRect,
			leftRect, topRect, rightRect, bottomRect,
			flipRect, unifyRect, unifyRectVec[ 100],
			diffRectVec[ 100];
    AnvilBoolean		clipFlag, unionFlag, intersectFlag, unifyFlag;
    AmRectangleRec const	*relativeRect;
    ulong_t		exposeFlags;

    if ( argc < 4) {
	fprintf( stderr, "usage %s rectfile X Y\n", argv[ 0]);
	exit( -1);
    }
    if (( fp = fopen( argv[ 1], "r")) == NULL) {
	fprintf( stderr, "failed to open file %s\n", argv[ 1]);
	exit( -1);
    }
    rectCount = 0;

    do {
	if ( fscanf( fp, "%d %d %d %d", 
		     &rectX, &rectY, &rectWidth, &rectHeight) != 4) {
	    break;
	}
	rects[ rectCount].x = rectX;
	rects[ rectCount].y = rectY;
	rects[ rectCount].width = rectWidth;
	rects[ rectCount].height = rectHeight;
	rectCount++;
    } while ( TRUE);
    fclose( fp);
    printf( "rectCount = %d\n", rectCount);
    ptX = atoi( argv[ 2]);
    ptY = atoi( argv[ 3]);
    rectA = rects[ 0];
    rectB = rects[ 1];

/* clip rectangle test */

    clipRect = rectB;
    clipFlag = AmClipRect( &rectA, &clipRect);
    printf( "clip (%d, %d, %d, %d), (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height, 
	    clipRect.x, clipRect.y, clipRect.width, clipRect.height);

/* clip rectangle vector test */

    memcpy( clipRectVec, rects, rectCount*sizeof( AmRectangleRec));
    numRects = rectCount;
    printf( "clipping with (%d, %d, %d, %d)\n", 
	    rectA.x, rectA.y, rectA.width, rectA.height);
    for ( ii = 0; ii < rectCount; ii++) {
	printf( "(%d, %d, %d, %d)\n", 
		rects[ ii].x, rects[ ii].y, 
		rects[ ii].width, rects[ ii].height);
    }
    clipFlag = AmClipRectVec( &rectA, clipRectVec, &numRects);
    printf( "result\n");
    for ( ii = 0; ii < numRects; ii++) {
	printf( "(%d, %d, %d, %d)\n", 
		clipRectVec[ ii].x, clipRectVec[ ii].y, 
		clipRectVec[ ii].width, clipRectVec[ ii].height);
    }

/* degenerate rectangle test */
    
    printf( "degenerate tests\n");
    for ( ii = 0; ii < rectCount; ii++) {
	if ( AmDegenRect( &rects[ ii])) {
	    printf( "(%d, %d, %d %d) is degenerate\n", 
		    rects[ ii].x, rects[ ii].y, 
		    rects[ ii].width, rects[ ii].height);
	} else {
	    printf( "(%d, %d, %d %d) is not degenerate\n", 
		    rects[ ii].x, rects[ ii].y, 
		    rects[ ii].width, rects[ ii].height);
	}
    }

/* union rectangle test */

    printf( "union tests\n");
    unionFlag = AmUnionRect( &rectA, &rectB, &unionRect);
    printf( "union (%d, %d, %d, %d), (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    rectB.x, rectB.y, rectB.width, rectB.height,
	    unionRect.x, unionRect.y, unionRect.width, unionRect.height);

/* union rectangle vector test */

    AmUnionRectVec( rects, rectCount, &unionRect);
    printf( "union of all rectangles = (%d, %d, %d, %d)\n",
	    unionRect.x, unionRect.y, unionRect.width, unionRect.height);

/* intersect rectangle test */

    printf( "intersect tests\n");
    intersectFlag = AmIntersectRect( &rectA, &rectB, &intersectRect);
    printf( "(%d, %d, %d, %d), (%d, %d, %d, %d) ",
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height);
    if ( intersectFlag) {
	printf( " intersect at (%d, %d, %d, %d)\n", 
		intersectRect.x, intersectRect.y,
		intersectRect.width, intersectRect.height);
    } else {
	printf( "do not intersect\n");
    }

/* relative rectangle test */
	
    printf( "relative tests\n");
    relativeRect = AmRelativeRect( &rectA, &rectB);
    printf( "(%d, %d, %d, %d) relative (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height,
	    relativeRect->x, relativeRect->y, 
	    relativeRect->width, relativeRect->height);

/* exposure rectangles test */

    printf( "exposure tests\n");
    AmCreateExposureRects( &rectA, &rectB,
			   &leftRect, &topRect, &rightRect, &bottomRect,
			   &exposeFlags);
    printf( "exposures (%d, %d, %d, %d), (%d, %d, %d, %d)\n", 
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height);
    if ( exposeFlags & AmRECT_LEFT) {
	printf( "left = (%d, %d, %d, %d)\n", 
		leftRect.x, leftRect.y, leftRect.width, leftRect.height);
    }
    if ( exposeFlags & AmRECT_RIGHT) {
	printf( "right = (%d, %d, %d, %d)\n", 
		rightRect.x, rightRect.y, rightRect.width, rightRect.height);
    }
    if ( exposeFlags & AmRECT_ABOVE) {
	printf( "top = (%d, %d, %d, %d)\n", 
		topRect.x, topRect.y, topRect.width, topRect.height);
    }
    if ( exposeFlags & AmRECT_BELOW) {
	printf( "bottom = (%d, %d, %d, %d)\n", 
		bottomRect.x, bottomRect.y, 
		bottomRect.width, bottomRect.height);
    }

/* difference rects */

    printf( "difference tests\n");
    AmDifferenceRects( &rectA, &rectB, diffRectVec, &numDiffRects);
    printf( "difference (%d, %d, %d, %d), (%d, %d, %d, %d)\n", 
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height);
    for ( ii = 0; ii < numDiffRects; ii++) {
	printf( "(%d, %d, %d, %d)\n", 
		diffRectVec[ ii].x, diffRectVec[ ii].y,
		diffRectVec[ ii].width, diffRectVec[ ii].height);
    }
    AmDifferenceIncludeRects( &rectA, &rectB, diffRectVec, &numDiffRects);
    printf( "difference include (%d, %d, %d, %d), (%d, %d, %d, %d)\n", 
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height);
    for ( ii = 0; ii < numDiffRects; ii++) {
	printf( "(%d, %d, %d, %d)\n", 
		diffRectVec[ ii].x, diffRectVec[ ii].y,
		diffRectVec[ ii].width, diffRectVec[ ii].height);
    }

/* equal test */

    printf( "equal tests\n");
    printf( "(%d, %d, %d, %d), (%d, %d, %d, %d) ",
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height);
    if ( AmEqualRect( &rectA, &rectB)) {
	printf( "are equal\n");
    } else {
	printf( "are not equal\n");
    }

/* flip tests */

    printf( "flip tests\n");
    flipRect = rectA;
    AmApplyFlips( &flipRect, TRUE, TRUE);
    printf( "flip (1,1) (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    flipRect.x, flipRect.y, flipRect.width, flipRect.height);
    flipRect = rectA;
    AmApplyFlips( &flipRect, TRUE, FALSE);
    printf( "flip (1,0) (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    flipRect.x, flipRect.y, flipRect.width, flipRect.height);
    flipRect = rectA;
    AmApplyFlips( &flipRect, FALSE, TRUE);
    printf( "flip (0,1) (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    flipRect.x, flipRect.y, flipRect.width, flipRect.height);
    flipRect = rectA;
    AmApplyFlips( &flipRect, FALSE, FALSE);
    printf( "flip (0,0) (%d, %d, %d, %d) = (%d, %d, %d, %d)\n",
	    rectA.x, rectA.y, rectA.width, rectA.height,
	    flipRect.x, flipRect.y, flipRect.width, flipRect.height);

/* unify tests */

    printf( "unify tests\n");
    unifyFlag = AmUnifyRects( &rectA, &rectB, &unifyRect);
    printf( "unify (%d, %d, %d, %d), (%d, %d, %d, %d) = ", 
	    rectA.x, rectA.y, rectA.width, rectA.height, 
	    rectB.x, rectB.y, rectB.width, rectB.height);
    if ( unifyFlag) {
	printf( "(%d, %d, %d, %d)\n", 
		unifyRect.x, unifyRect.y, unifyRect.width, unifyRect.height);
    } else {
	printf( "false\n");
    }
    memcpy( unifyRectVec, rects, rectCount*sizeof( AmRectangleRec));
    numRects = rectCount;
    AmUnifyRectVec( unifyRectVec, &numRects);
    printf( "unify rects\n");
    for ( ii = 0; ii < numRects; ii++) {
	printf( "(%d, %d, %d, %d)\n", 
		unifyRectVec[ ii].x, unifyRectVec[ ii].y, 
		unifyRectVec[ ii].width, unifyRectVec[ ii].height);
    }

/* point in rect tests */

    for ( ii = 0; ii < rectCount; ii++) {
	if ( AmPtInRect( &rects[ ii], ptX, ptY)) {
	   printf( "(%d, %d) is in (%d, %d, %d, %d)\n",
		   ptX, ptY, rects[ ii].x, rects[ ii].y,
		   rects[ ii].width, rects[ ii].height);
	} else {
	   printf( "(%d, %d) is not in (%d, %d, %d, %d)\n",
		   ptX, ptY, rects[ ii].x, rects[ ii].y,
		   rects[ ii].width, rects[ ii].height);
	}
    }
}
