#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <anvil/strlib.h>
#include <anvil/files.h>

static void
printStrvec( char **strvec)
{
    if ( strvec == NULL) {
	printf( "string vector is NULL\n");
	return;
    }
    for ( ; *strvec != NULL; strvec++) {
	printf( "%s\n", *strvec);
    }
}

int main( int argc, char *argv[])
{
    char	*string, **strvec, **testStrvec, buffer[ 2048], **strvec2,
		**strvec3;

    testStrvec = AmReadLines( argv[ 1]);

/* copy test */

    strvec2 = AmCopyStrVec( testStrvec);
    printf( "copy of \n");
    printStrvec( strvec2);
    printf( "is:\n");
    printStrvec( testStrvec);

/* length test */
  
    printf( "length = %d\n", AmVeclen( testStrvec));
    printf( "length of NULL = %d\n", AmVeclen( NULL));

/* chopping strings */

    string = testStrvec[ 0];
    strvec = AmChopString( string, ' ');
    printf( "chopping %s by '%c'\n", string, ' ');
    printStrvec( strvec);
    AmFreeStrVec( strvec);

/* NULL test */

    strvec = AmChopString( NULL, ' ');
    printf( "chopping NULL by '%c'\n", ' ');
    printStrvec( strvec);
    AmFreeStrVec( strvec);

/* chopping strings by whitespace and quotes */

    string = testStrvec[ 1];
    strvec = AmVectorQuotedStrings( string);
    printf( "vector quoted strings of %s\n", string);
    printStrvec( strvec);
    AmFreeStrVec( strvec);

/* NULL test */

    strvec = AmVectorQuotedStrings( NULL);
    printf( "vector quoted strings of NULL\n");
    printStrvec( strvec);
    AmFreeStrVec( strvec);

/* copy newline string test */

    string = testStrvec[ 2];
    strcpy( buffer, string);
    AmReplaceNewlines( string);
    printf( "newline conversion: from %s to %s\n", buffer, string);
    AmCopyNewlineString( buffer, string);
    printf( "back to '\n's %s\n", buffer);

/* endswith test */

    if ( AmEndswith( "endswith", "with")) {
	printf( "endswith test OK\n");
    } else {
	printf( "endswith test not OK\n");
    }
    if ( AmEndswith( "endswith", "t")) {
	printf( "endswith test not OK\n");
    } else {
	printf( "endswith test OK\n");
    }

/* leading and trailing white space */

    string = AmElimLeadWspace( testStrvec[ 3]);
    printf( "elimLeadWspace:\n%s\n%s\n", testStrvec[ 3], string);
    string = testStrvec[ 3];
    printf( "elimTrailWspace:\n%s\n", string);

/* write read strvec test */

    if ( !AmWriteStrvec( "newtestfile", strvec2)) {
	printf( "writeStrvec failed\n");
    } else {
	printf( "writeStrvec succeeded\n");
    }
    strvec3 = AmReadStrvec( "newtestfile");
    printf( "read strvec of %s is:\n", argv[ 1]);
    printStrvec( strvec3);

    AmFreeStrVec( testStrvec);
    AmFreeStrVec( strvec3);
}
