#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <anvil/files.h>
#include <anvil/tagstrs.h>
#include <anvil/fns.h>
#include <anvil/strlib.h>
#include <anvil/tagvalues.h>
#include <anvil/envdefs.h>
#include <anvil/record.h>

static void 
printTagValue( AmTagValuePtr	tagValue)
{
    char		buffer[ 512];
    AmDataTypeEnum	dataType;

    printf( "%s: ", tagValue->tag);
    dataType = AmGetTagType( tagValue->tag);
    if ( dataType == AmUNKNOWN) {
	printf( "(unknown): ");
    } else {
	printf( "(%s): ", AmDataTypeEnumToStr( dataType));
    }
    if ( AmIsAtomic( dataType)) {
	AmPrintValue( buffer, tagValue->tag, 0, &tagValue->value, dataType);
	printf( "%s\n", buffer);
    } else {
	printf( "vector\n");
    }
}

int main( int argc, char *argv[])
{
    char		typeName[ 512], **headerStrings, prefixTag[ 512],
			*filename;
    int			length, numValues, ii;
    AmTagValueRec	tagValues[ 1024];
    AmDataTypeEnum	dataType;
    const char		*stringVal;
    AnvilBoolean		found;

/* create the default lookup table */

    if ( AmCreateLookupTable() != AmOK) {
	printf( "AmCreateLookupTable failed\n");
    }

    if ( AmInitUnitStuff() != AmOK) {
	printf( "AmInitUnitStuff failed\n");
    }

/* read the header into a string vector and print it out */

    if ( AmIsHeader( argv[ 1], typeName)) {
	printf( "%s is a header of type %s\n", argv[ 1], typeName);
    } else {
	printf( "%s is not a header\n");
    }
    headerStrings = AmReadHeader( argv[ 1]);
    length = AmVeclen( headerStrings);
    printf( "header strings from %s\n", argv[ 1]);
    for ( ii = 0; ii < length; ii += 2) {
	printf( "%s: %s\n", headerStrings[ ii], headerStrings[ ii + 1]);
    }

/* convert the strings into a tag-value array */

    numValues = AmHeaderStringsTagValueArray( headerStrings, tagValues);
    printf( "converted to tag-values\n");
    for ( ii = 0; ii < numValues; ii++) {
	printTagValue( &tagValues[ ii]);
    }
    printf( "\n");
    AmFreeNonAtomicValues( tagValues, numValues);

/* get a tag-value from the header */

    if ( argc >= 3) {
	stringVal = AmGetHeaderStringTagValue( headerStrings, argv[ 2]);
	if ( stringVal != NULL) {
	    printf( "tag = %s, value = %s\n", argv[ 2], stringVal);
	} else {
	    printf( "tag not found\n");
	}
    }

/* read the header into a tag-value array and print it out */

    numValues = AmHeaderTagValueArray( argv[ 1], tagValues);
    if ( numValues == 0) {
	printf( "failed to read %s into tag values\n", argv[ 1]);
    } else {
	for ( ii = 0; ii < numValues; ii++) {
	    printTagValue( &tagValues[ ii]);
	}
    }

/* try the tag match stuff */

    if ( argc >= 3) {
	found = FALSE;
	for ( ii = 0; ii < numValues; ii++) {
	    if ( AmMatchTag( argv[ 2], tagValues[ ii].tag)) {
		found = TRUE;
		break;
	    }
	}
	if ( found) {
	    printf( "found tag %s\n", argv[ 2], tagValues[ ii].tag);
	} else {
	    printf( "didn't find tag %s\n", argv[ 2]);
	}

/* try it with a prefix */

	sprintf( prefixTag, "prefix.%s", argv[ 2]);
	found = FALSE;
	for ( ii = 0; ii < numValues; ii++) {
	    if ( AmMatchTag( argv[ 2], tagValues[ ii].tag)) {
		found = TRUE;
		break;
	    }
	}
	if ( found) {
	    printf( "found tag %s\n", prefixTag, tagValues[ ii].tag);
	} else {
	    printf( "didn't find tag %s\n", prefixTag);
	}
	if ( AmGetTagValueFromTags( tagValues, numValues, AmNfilename,
				    ( void **) &filename)) {
	    printf( "found tag %s = %s\n", AmNfilename, filename);
	} else {
	    printf( "failed to find tag %s\n", AmNfilename);
	}

/* remove the tag */

	if ( AmRemoveTagValue( prefixTag, tagValues, &numValues)) {
	    printf( "successfully removed tag %s\n", prefixTag);
	    for ( ii = 0; ii < numValues; ii++) {
		printTagValue( &tagValues[ ii]);
	    }
	}
    }

/* add a couple of tag/type pairs and look them up again */

    AmAddTagType( "tagMe", AmFLOAT);
    AmAddTagType( "tagYou", AmSTRING);
    AmAddTagType( "tagOther", AmINT);
    AmAddTagType( "tagHeuer", AmFILE);
    AmAddTagType( "taggant", AmSTRING_PTR);
    printf( "type %s = %s\n", 
	    "tagMe", AmDataTypeEnumToStr( AmGetTagType( "tagMe")));
    printf( "type %s = %s\n", 
	    "tagYou", AmDataTypeEnumToStr( AmGetTagType( "tagYou")));
    printf( "type %s = %s\n", 
	    "tagOther", AmDataTypeEnumToStr( AmGetTagType( "tagOther")));
    printf( "type %s = %s\n", 
	    "tagHeuer", AmDataTypeEnumToStr( AmGetTagType( "tagHeuer")));
    printf( "type %s = %s\n", 
	    "taggant", AmDataTypeEnumToStr( AmGetTagType( "taggant")));

/* free everything to verify memory leaks */

    AmFreeNonAtomicValues( tagValues, numValues);
    AmFreeStrVec( headerStrings);

}
