#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <anvil/template.h>
#include <anvil/motif.h>
#include <anvil/motifdisplay.h>
#include <anvil/log.h>
#include <anvil/copyobj.h>
#include <anvil/xapp.h>

Widget			top, form, area;
AmXDisplayClass		*xDisplay;
AmXWindowClass		*xWindow;
AmMotifDisplayClass	*motifDisplay;
AmScreenObjClass	**vpScreenObj;
AnvilBoolean			destroyFlag;

static void
exposeCallback( Widget  area,
                XtPointer,
                XtPointer)
{
//    XtDestroyWidget( AmGetWidgetShell( area));
}

static void
destroyCallback( Widget  area,
                 XtPointer,
                 XtPointer)
{
    destroyFlag = TRUE;
}

static AmScreenObjClass *
createObject( AmWindowClass		*pWindow,
	      AmScreenObjClass		*pParent,
	      char			**headerStrings,
	      void			*clientData)
{
    AmErrorCode 	errCode;
    AmTagValueRec	vTagValues[ MAXTAGS];
    int			nTagValues;
    AmScreenObjClass	*pScreenObj;

    nTagValues = AmHeaderStringsTagValueArray( headerStrings, vTagValues);
    if ( AmAddLayerFromTagValues( pWindow, ( AmOverlayClass *) pParent,
				  vTagValues, nTagValues,
				  "object", AM_PREFS_DIR, 
				  &pScreenObj) == AmOK) {
	return( pScreenObj);
    } else {
	return( NULL);
    }
}

int main( int argc, char *argv[])
{
    int		ii;
    XEvent	event;

    destroyFlag = FALSE;
    top = XtInitialize ( "testplot", "testplot-db", NULL, 0, &argc, argv);
    AmInitApplication( top);
    form = XtVaCreateManagedWidget( "form", xmFormWidgetClass, top,
				    NULL);
    area =  XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, form,
				     XmNwidth, 1000,
				     XmNheight, 600,
				     NULL);
    xDisplay = new AmXDisplayClass( area);
    xWindow = new AmXWindowClass( xDisplay, area);
    motifDisplay = new AmMotifDisplayClass( xDisplay, xWindow, FALSE);
    vpScreenObj = AmParseTemplateFile( argv[ 1],
				       motifDisplay->GetWindow(),
				       ( AmCreateObjectFn) createObject,
				       NULL);
    for ( ii = 0; vpScreenObj[ ii] != NULL; ii++) {
	if ( vpScreenObj[ ii]->IsValid()) {
	    motifDisplay->AddObject( vpScreenObj[ ii]);
	}
    }
    free( vpScreenObj);
    motifDisplay->Flush();
    XtRealizeWidget( top);
    XtAddCallback( area, XmNexposeCallback,
                   ( XtCallbackProc) exposeCallback, NULL);
    XtAddCallback( area, XmNdestroyCallback,
                   ( XtCallbackProc) destroyCallback, NULL);
    do {
	XtNextEvent( &event);
	XtDispatchEvent( &event);
	if ( destroyFlag) {
	    exit( 0);
	}
    } while ( TRUE);
}
