#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <anvil/units.h>
#include <anvil/files.h>

int main( int argc, char *argv[])
{
    char	errorString[ 512], buffer1[ 1024], buffer2[ 1024];
    int		*codes, numCodes, numStrings, ii, jj, numValues;
    char	**strings, *unitString;
    unitType	unitCode;
    float	*values, value, convertValue;

    if ( argc < 3) {
	fprintf( stderr, "usage: %s units-file values\n", argv[ 0]);
	exit( -1);
    }

/* parse the units file */

    if ( AmParseUnitsFile( argv[ 1], errorString)) {
	fprintf( stderr, "failed to parse units file: %s\n", errorString);
	exit( -1);
    }
    AmRegisterUnitFns();

/* read the values */

    numValues = AmNumLines( argv[ 2]);
    values = AmReadFloatVec( argv[ 2]);
    AmGetUnitCodeList( &codes, &numCodes);
    AmGetUnitStringList( &strings, &numStrings);
    if ( numCodes != numStrings) {
	printf( "#codes != #strings!! %d != %d\n", numCodes, numStrings);
	exit( -1);
    }
    printf( "strings and codes\n");
    for ( ii = 0; ii < numCodes; ii++) {
	printf( "%s: %d\n", strings[ ii], codes[ ii]);
    }
    printf( "code lookup\n");
    for ( ii = 0; ii < numCodes; ii++) {
	if ( !AmGetUnitCode( strings[ ii], &unitCode)) {
	    fprintf( stderr, "failed to get code for %s\n", strings[ ii]);
	    exit( -1);
	}
	printf( "%s: %d\n", strings[ ii], unitCode);
    }
    printf( "string lookup\n");
    for ( ii = 0; ii < numStrings; ii++) {
	if ( !AmGetUnitString( codes[ ii], &unitString)) {
	    fprintf( stderr, "failed to get code for %s\n", strings[ ii]);
	    exit( -1);
	}
	printf( "%s: %d\n", unitString, codes[ ii]);
    }

/* print and scan tests */

    printf( "print and scan tests\n");
    for ( ii = 0; ii < numCodes; ii++) {
	if ( !AmGetUnitString( codes[ ii], &unitString)) {
	    fprintf( stderr, "failed to get code for %s\n", strings[ ii]);
	    exit( -1);
	}
	for ( jj = 0; jj < numValues; jj++) {
	    printf( "code = %d, string = %s ", codes[ ii], strings[ ii]);
	    AmPrintValue( codes[ ii], values[ jj], buffer1);
	    AmPrintValueNoUnits( codes[ ii], values[ jj], buffer2);
	    printf( "%s, %s\n", buffer1, buffer2);
	    AmScanValue( codes[ ii], buffer1, &value);
	    AmPrintValue( codes[ ii], value, buffer1);
	    printf( "rescanned = %s\n", buffer1);
	}
    }
    printf( "conversion tests\n");
    value = values[ 0];
    for ( ii = 0; ii < numCodes; ii++) {
	for ( jj = 0; jj < numCodes; jj++) {
	    if ( ii != jj) {
		if ( AmConverterExists( codes[ ii], codes[ jj])) {
		    AmConvertValue( codes[ ii], codes[ jj], value,
				    &convertValue);
		    AmPrintValue( codes[ ii], value, buffer1);
		    AmPrintValue( codes[ jj], convertValue, buffer2);
		    printf( "%s -> %s\n", buffer1, buffer2);
		}
	    }
	}
    }
}
