#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Xm/Label.h>
#include <Xm/Scale.h>
#include <Xm/Frame.h>
#include <anvil/plot.h>
#include <anvil/files.h>
#include <anvil/overlay.h>
#include <anvil/grid.h>
#include <anvil/plot_cback.h>
#include <anvil/profile.h>
#include <anvil/xapp.h>
#include <anvil/motif.h>
#include <anvil/errlib.h>
#include <anvil/strlib.h>
#include <anvil/userevents.h>
#include <anvil/macros.h>

typedef struct nameCallback_ {
    char		*name;
    VarargsCallback	callback;
    WidgetClass		widgetClass;
    char		*callbackName;
} nameCallbackRec, *nameCallbackPtr;

nameCallbackRec nameCallbacks[] = {
    { "range", 
      ( VarargsCallback) AmSetRangeDialogCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "hatchAngleRange", 
      ( VarargsCallback) AmSetHatchAngleRangeDialogCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "lineWidthRange", 
      ( VarargsCallback) AmSetLineWidthRangeDialogCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "symbolRange", 
      ( VarargsCallback) AmSetSymbolRangeDialogCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "symbolSizeRange", 
      ( VarargsCallback) AmSetSymbolSizeRangeDialogCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "colormap", 
      ( VarargsCallback) AmSetColormapDialogCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "transparency", 
      ( VarargsCallback) AmScaleTransparencyCallback,
      xmScaleWidgetClass,
      XmNvalueChangedCallback
    },
    { "moveObject", 
      ( VarargsCallback) AmMoveObjectCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "lightShade", 
      ( VarargsCallback) AmLightShadeCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "selectObject", 
      ( VarargsCallback) AmSelectObjectCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "selectScatter", 
      ( VarargsCallback) AmSelectScatterCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "selectPolygon", 
      ( VarargsCallback) AmSelectPolygonCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "selectLine", 
      ( VarargsCallback) AmSelectLineCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "showHistogram", 
      ( VarargsCallback) AmShowHistogramCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    },
    { "contourCursor", 
      ( VarargsCallback) AmContourCursorCallback,
      xmPushButtonWidgetClass,
      XmNactivateCallback
    }
};

static void 
delLayerProc( int			eventId,
	      Widget			shell,
	      AmDataGraphicClass	*layer,
	      Widget			button,
	      Widget			dialog,
	      AmDataGraphicClass	*delLayer)
{
    if ( delLayer == layer) {
	XtDestroyWidget( dialog);
	XtDestroyWidget( button);
	AmRemoveVaCallbackWidget( AM_PKG_ID, AM_DELETE_LAYER, shell,
				  ( VarargsCallback) delLayerProc,
				  shell, layer, button, dialog,
				  END_LIST);
    }
}

static void
addLayerProc( int			eventId,
	      Widget			layerRowcol,
	      AmMotifDisplayClass	*motifDisplay,
	      AmOverlayClass		*overlay,
	      AmDataGraphicClass	*layer)
{
    AmDataGraphicClass	**layerList;
    int			numLayers, ii;
    AnvilBoolean		layerFound;
    const char		*appId;
    char		**fnNames, **fnNamePtr, filename[ 512], *fnNameFile;
    Widget		button, dialog, rowcol, w, closeBtn;

    overlay->GetLayers( &layerList, &numLayers);
    layerFound = FALSE;
    for ( ii = 0; ii < numLayers; ii++) {
	if ( layerList[ ii] == layer) {
	    layerFound = TRUE;
	    break;
	}
    }
    free( layerList);
    if ( !layerFound) {
	return;
    }
    if ( layerFound) {
	AN_DEBUG( 20, ( "found layer in overlay\n"));
    } 
    appId = layer->getAppId();
    sprintf( filename, "examples/%s.funcs", appId);
    fnNameFile = AmGetEnvFile( CONFIG_ENV, filename);
    if ( fnNameFile == NULL) {
	return;
    }
    fnNames = AmReadLines( fnNameFile);

/* add callbacks for each of the names listed in the file referred to by
   the layer's appId. */

    if ( fnNames == NULL) {
	return;
    }

/* create the button, dialog, and rowcolumn for the widgets. */

    button = XtVaCreateManagedWidget( appId, xmPushButtonWidgetClass,
				      layerRowcol, NULL);
    dialog = XmCreateFormDialog( layerRowcol, ( char *) appId, NULL, 0);
    rowcol = XtVaCreateManagedWidget( "rowcol", xmRowColumnWidgetClass,
				      dialog,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* create the callbacks for each of the names in the callbacks file for
   this layer's appId */

    for ( fnNamePtr = fnNames; *fnNamePtr != NULL; fnNamePtr++) {
	for ( ii = 0; ii < VECLEN( nameCallbacks); ii++) {
	    if ( !strcmp( *fnNamePtr, nameCallbacks[ ii].name)) {
		w = XtVaCreateManagedWidget( *fnNamePtr,
					     nameCallbacks[ ii].widgetClass,
					     rowcol,
					     NULL);
		AmAddVaCallback( w, nameCallbacks[ ii].callbackName,
				 nameCallbacks[ ii].callback,
				 motifDisplay, layer, dialog,
				 END_LIST);
	    }
	}
    }
    AmFreeStrVec( fnNames);

/* add a close callback, and manage and unmanage callbacks */

    closeBtn = XtVaCreateManagedWidget( "closeBtn", xmPushButtonWidgetClass,
					rowcol, NULL);
    XtAddCallback( button, XmNactivateCallback,
		  ( XtCallbackProc) AmManageSubDialogCallback,
		  dialog);
    AmAddVaCallback( closeBtn, XmNactivateCallback,
		  ( VarargsCallback) AmUnmanageSubDialogCallback,
		  button, dialog,
		  END_LIST);

    AmAddVaCallbackWidget( AM_PKG_ID, AM_DELETE_LAYER, 
			   motifDisplay->GetShell(),
			   ( VarargsCallback) delLayerProc,
			   motifDisplay->GetShell(),
			   layer, button, dialog,
			   END_LIST);
}
    
int main( int argc, char *argv[])
{
    Widget		form, areaFrame, area, readoutForm, 
			readout, messageSep, messageForm, message, layerFrame,
			unzoom2xBtn, refreshBtn, delLayerBtn, zoomRectBtn, 
			zoom2xBtn, unzoomBtn, panBtn, toolbar, bottomWin, 
			sortLayerBtn, magnifyBtn, toolForm, addLayerBtn,
			scrollWin, magnifyDlogBtn, refWinBtn, 
			parent, profileBtn, zoomTextBtn, layerRowcol;
    AmOverlayClass	*overlay;
    AmGridClass		**grid;
    AmMotifDisplayClass	*motifDisplay;
    AmTickClass		*ticks;
    AmXDisplayClass	*pXDisplay;
    AmXWindowClass	*pXWindow;

    parent = XtInitialize( "tooldriver", "tooldriver-db", NULL, 0, &argc, argv);
    AmInitializeError( parent);
    AmInitApplication( parent);
    form = XtVaCreateManagedWidget( "form",
				    xmFormWidgetClass, parent,
				    NULL);
 
/* Toolbar contains icons with basic operations. */

    toolForm = XtVaCreateManagedWidget( "toolForm",
                   xmFormWidgetClass, form,
                   StickLeft, StickTop, StickRight,
                   NULL);

    toolbar = XtVaCreateManagedWidget( "toolbar",
                  xmRowColumnWidgetClass, toolForm,
                  StickLeft, StickTop, StickRight, StickBottom,
                  XmNorientation, XmHORIZONTAL,
                  NULL);

    zoom2xBtn = XtVaCreateManagedWidget( "zoom2xBtn",
                    xmPushButtonWidgetClass, toolbar,
                    NULL);
    zoomRectBtn = XtVaCreateManagedWidget( "zoomRectBtn", 
                      xmPushButtonWidgetClass, toolbar,
                      NULL);
    zoomTextBtn = XtVaCreateManagedWidget( "zoomTextBtn", 
                      xmPushButtonWidgetClass, toolbar,
                      NULL);
    unzoom2xBtn = XtVaCreateManagedWidget( "unzoom2xBtn", 
                      xmPushButtonWidgetClass, toolbar,
                      NULL);
    unzoomBtn = XtVaCreateManagedWidget( "unzoomBtn", 
                    xmPushButtonWidgetClass, toolbar,
                    NULL);

    (void) XtVaCreateManagedWidget( "toolSep",
               xmSeparatorWidgetClass, toolbar,
               XmNorientation, XmVERTICAL,
               XmNseparatorType, XmNO_LINE,
               NULL);

    magnifyBtn = XtVaCreateManagedWidget( "magnifyBtn",
                     xmPushButtonWidgetClass, toolbar,
                     NULL);

    magnifyDlogBtn = XtVaCreateManagedWidget( "magnifyDlogBtn", 
                         xmPushButtonWidgetClass, toolbar,
                         NULL);

    (void) XtVaCreateManagedWidget( "toolSep",
               xmSeparatorWidgetClass, toolbar,
               XmNorientation, XmVERTICAL,
               XmNseparatorType, XmNO_LINE,
               NULL);

    panBtn = XtVaCreateManagedWidget( "panBtn",
                 xmPushButtonWidgetClass, toolbar,
                 NULL);

    refreshBtn = XtVaCreateManagedWidget( "refreshBtn",
                     xmPushButtonWidgetClass, toolbar,
                     NULL);

    (void) XtVaCreateManagedWidget( "toolSep",
               xmSeparatorWidgetClass, toolbar,
               XmNorientation, XmVERTICAL,
               XmNseparatorType, XmNO_LINE,
               NULL);


    (void) XtVaCreateManagedWidget( "toolSep",
               xmSeparatorWidgetClass, toolbar,
               XmNorientation, XmVERTICAL,
               XmNseparatorType, XmNO_LINE,
               NULL);

    (void) XtVaCreateManagedWidget( "toolSep",
               xmSeparatorWidgetClass, toolbar,
               XmNorientation, XmVERTICAL,
               XmNseparatorType, XmNO_LINE,
               NULL);

    addLayerBtn = XtVaCreateManagedWidget( "addLayerBtn",
                      xmPushButtonWidgetClass, toolbar,
                      NULL);
    delLayerBtn = XtVaCreateManagedWidget( "delLayerBtn",
                      xmPushButtonWidgetClass, toolbar,
                      NULL);
    sortLayerBtn = XtVaCreateManagedWidget( "sortLayerBtn",
                      xmPushButtonWidgetClass, toolbar,
                      NULL);
    refWinBtn = XtVaCreateManagedWidget( "refWinBtn",
		      xmPushButtonWidgetClass, toolbar,
		      NULL);
    profileBtn = XtVaCreateManagedWidget( "profileBtn",
		      xmPushButtonWidgetClass, toolbar,
		      NULL);

/* Plot drawing area */

    messageForm = XtVaCreateManagedWidget( "messageForm",
                      xmFormWidgetClass, form,
                      StickLeft, StickBottom, StickRight,
                      NULL);

    message = XtVaCreateManagedWidget( "message",
                  xmLabelWidgetClass, messageForm,
                  StickLeft, StickTop, StickBottom,
                  NULL);

    messageSep = XtVaCreateManagedWidget( "messageSep",
                     xmSeparatorWidgetClass, form,
                     StickLeft, StickRight, AboveOf( messageForm),
                     NULL);


    readoutForm = XtVaCreateManagedWidget( "readoutForm",
                      xmFormWidgetClass, form,
                      StickLeft, StickRight, AboveOf( messageSep),
                      NULL);

    readout = XtVaCreateManagedWidget( "readout",
                  xmLabelWidgetClass, readoutForm,
                  StickLeft, StickTop, StickRight, StickBottom, 
                  NULL);

    bottomWin = XtVaCreateManagedWidget( "bottomWin",
                    xmFormWidgetClass, form,
                    StickLeft, StickRight, AboveOf( readout),
                    NULL);

    layerFrame = XtVaCreateManagedWidget( "layerFrame",
                   xmFrameWidgetClass, bottomWin,
                   StickLeft, StickTop, StickRight, StickBottom, 
                   NULL);

    scrollWin = XtVaCreateManagedWidget( "layerWin",
                    xmScrolledWindowWidgetClass, layerFrame,
                    NULL);

    layerRowcol = XtVaCreateManagedWidget( "layerRowcol",
                      xmRowColumnWidgetClass, scrollWin,
                      XmNorientation, XmHORIZONTAL,
                      NULL);

    areaFrame = XtVaCreateManagedWidget( "areaFrame",
                    xmFrameWidgetClass, form,
                    StickLeft, BelowOf( toolForm), StickRight,
                    AboveOf( bottomWin),
                    NULL);

    area = XtVaCreateManagedWidget( "area",
               xmDrawingAreaWidgetClass, areaFrame, 
               XmNwidth, 800, XmNheight, 600,
               NULL);

    pXDisplay = new AmXDisplayClass( area);
    pXWindow = new AmXWindowClass( pXDisplay, area);
    motifDisplay = new AmMotifDisplayClass( pXDisplay, pXWindow, TRUE);
    overlay = new AmOverlayClass( motifDisplay->GetWindow(), "overlay",
                      AmNpreserveAspect, TRUE,
                      AmNleftMargin, 30,
                      AmNrightMargin, 30,
                      AmNtopMargin, 30,
                      AmNbottomMargin, 30,
                      NULL);
    AmAddVaCallbackWidget( AM_PKG_ID, AM_ADD_LAYER, 
			   motifDisplay->GetShell(),
			   ( VarargsCallback) addLayerProc,
			   layerRowcol, motifDisplay, overlay,
			   END_LIST);

/* add a grid for showing the current extents */

    grid = ( AmGridClass **) malloc( sizeof( AmGridClass *));
    *grid = new AmGridClass( motifDisplay->GetWindow(), overlay, "grid", 
                    AmNleftMargin, 30,
                    AmNrightMargin, 30,
                    AmNtopMargin, 30,
                    AmNbottomMargin, 30,
                    AmNgridStyle, AmGRID_GRID,
		    AmNresizeFont, FALSE,
                    NULL);

/* re-orient the tickmarks so they take less space */

    (*grid)->GetVaFields( AmNleftTicks, &ticks, NULL);
    ticks->SetVaFields( AmNtextVertical, TRUE, NULL);
    (*grid)->GetVaFields( AmNrightTicks, &ticks, NULL);
    ticks->SetVaFields( AmNtextVertical, TRUE, NULL);
    (*grid)->GetVaFields( AmNtopTicks, &ticks, NULL);
    ticks->SetVaFields( AmNtextVertical, FALSE, NULL);
    (*grid)->GetVaFields( AmNbottomTicks, &ticks, NULL);
    ticks->SetVaFields( AmNtextVertical, FALSE, NULL);

/* Set the readout area and the message area. */

    motifDisplay->SetMessage( message);
    motifDisplay->SetReadout( readout);

/* add the objects to the motif display */

    motifDisplay->AddObject( overlay);
    motifDisplay->AddObject( *grid);
    motifDisplay->Flush();

/* turn on the following cursor */

    motifDisplay->SetFollowReadout( 'K', FALSE);

/* toolbar callback section */

/* zooms, unzooms, magnifying glasses, refreshes and pans */

    AmAddVaCallback( zoomRectBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmZoomRectCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( zoomTextBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmZoomTextCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( magnifyBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmMagnifyGlassCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( magnifyDlogBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmMagnifyGlassDialogCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( zoom2xBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmZoom2xCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( unzoom2xBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmUnzoomHalfCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( unzoomBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmUnzoomCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    AmAddVaCallback( panBtn, XmNactivateCallback, 
		   ( VarargsCallback) AmPanCallback, 
		   motifDisplay, overlay, 
		   END_LIST);
    XtAddCallback( refreshBtn, XmNactivateCallback,
		   ( XtCallbackProc) AmRefreshCallback, motifDisplay);

/* profiler */

    AmAddVaCallback( profileBtn, XmNactivateCallback,
		     ( VarargsCallback) AmDataProfileCallback,
		     motifDisplay, overlay, "profiler", 
		     END_LIST);

/* add, delete and sort layers */

    AmAddVaCallback( addLayerBtn, XmNactivateCallback,
		     ( VarargsCallback) AmAddLayerCallback,
		     motifDisplay, overlay, 
		     END_LIST);
    AmAddVaCallback( delLayerBtn, XmNactivateCallback, 
		     ( VarargsCallback) AmDeleteLayerCallback, 
		     motifDisplay, overlay, 
		     END_LIST);
    AmAddVaCallback( sortLayerBtn, XmNactivateCallback, 
		     ( VarargsCallback) AmSortLayersCallback, 
		     motifDisplay, overlay, 
		     END_LIST);

/* reference windows and 3D renderers */

    AmAddVaCallback( refWinBtn, XmNactivateCallback,
		     ( VarargsCallback) AmReferenceWindowCallback,
		     motifDisplay, overlay, 
		     END_LIST);
    XtRealizeWidget( parent);
    XtMainLoop();
}
