#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/PushB.h>
#include <anvil/project.h>		/* all-purpose include */
#include <anvil/motifdisplay.h>		/* display manager include file */
#include <anvil/overlay.h>		/* overlay manager header file */
#include <anvil/motif.h>		/* motif convienience macros */
#include <anvil/errlib.h>		/* error messages */
#include <anvil/header.h>		/* for tag-value header file parsing */
#include <anvil/plot_cback.h>		/* plot callback header file */
#include <anvil/xapp.h>                 /* initialization functions */


void addLayerOkCallback( Widget				fileSB,
		         AmMotifDisplayClass		*motifDisplay,
		         AmOverlayClass			*overlay,
		         XmFileSelectionBoxCallbackStruct	*cb)
{
    char	 	*filename, **headerStrings;
    AmDataGraphicClass	*dataGraphic;

/* grab the string from the file selection dialog and
   make sure that it's a valid file */

    XmStringGetLtoR( cb->value, XmFONTLIST_DEFAULT_TAG, &filename);
    if ( access( filename, R_OK) < 0) {

/* the error message looks in the resource file for the printf format string.
   If it doesn't exist, then it prints the resource tag */

    	AmPrintResourceError( AmGetWidgetShell( fileSB), 
			    "fileDoesNotExist", filename);
	return;
    }

/* blow away the dialog and read the strings from the header file */

    XtDestroyWidget( XtParent( fileSB));
    headerStrings = AmReadHeader( filename);

/* add the layer to the overlay and flush the display */

    AmDefaultAddLayerFromHeader( motifDisplay->GetDisplay(),
    			       overlay, headerStrings,
    			       &dataGraphic, "NetMentor");
    motifDisplay->Flush();
}

/* the add layer callback creates a file selection dialog, then adds the
   file on the OK callback from the file selection */

void AmAddLayerCallback( Widget			btn,
		       AmMotifDisplayClass	*motifDisplay,
		       AmOverlayClass		*overlay,
		       XtPointer)
{
    Widget	fileSB;

/* the "AmGetWidgetShell" recursively walks up the button's tree to find
   a shell widget (dialogs have to have shell parents) */

    fileSB = XmCreateFileSelectionDialog( AmGetWidgetShell( btn), "fileSB",
    					  NULL, 0);
    XtManageChild( fileSB);

/* add a callback to pop down the dialog on cancel */

    XtAddCallback( fileSB, XmNcancelCallback,
    		   ( XtCallbackProc) AmPopdownCback, fileSB);

/* add a callback to add the file to the overlay on OK */

    AmAddVaCallback( fileSB, XmNokCallback,
    		   ( VarargsCallback) addLayerOkCallback, 
    		   motifDisplay, overlay, END_LIST);
}


int main( int argc, char *argv[])
{
    Widget		top, form, area, addBtn, delBtn, zoomBtn, unzoomBtn;
    AmMotifDisplayClass	*motifDisplay;
    AmOverlayClass	*overlay;

/* initialize the connection to the server, read the resource database
   and parse the command line arguments.  Create a form and a drawing
   area to draw the overlay in.  add buttons to add and remove layers */

    top = XtInitialize ( "editor", "editor-db", NULL, 0, &argc, argv);

/* call the init function */

    AmInitApplication( top);
    form = XtVaCreateManagedWidget( "form", xmFormWidgetClass, top, NULL);
    addBtn = XtVaCreateManagedWidget( "addBtn", xmPushButtonWidgetClass,
    				      form, 
    				      StickLeft, StickTop,
    				      NULL);
    delBtn = XtVaCreateManagedWidget( "delBtn", xmPushButtonWidgetClass,
    				      form, 
    				      RightOf( addBtn), StickTop,
    				      NULL);
    zoomBtn = XtVaCreateManagedWidget( "zoomBtn", xmPushButtonWidgetClass,
    				       form, 
    				       RightOf( delBtn), StickTop,
    				       NULL);
    unzoomBtn = XtVaCreateManagedWidget( "unzoomBtn", xmPushButtonWidgetClass,
    				         form, 
    				         RightOf( zoomBtn), StickTop,
    				         NULL);
    area = XtVaCreateManagedWidget( "area", xmDrawingAreaWidgetClass, form,
    				    StickLeft, StickRight, 
    				    StickBottom, 
    				    BelowOf( addBtn),
    				    XmNwidth, 300,
    				    XmNheight, 300,
    				    NULL);

/* initialize a display manager for the drawing area */

    motifDisplay = new AmMotifDisplayClass( area);

/* create an overlay from the command line argument */

    overlay = new AmOverlayClass( motifDisplay->GetDisplay(), "overlay", 
                                AmNleftMargin, 0,
				AmNrightMargin, 0,
				AmNtopMargin, 0,
				AmNbottomMargin, 0, 
				NULL);


/* add the object to the display manager, so it gets drawn and refreshed */

    motifDisplay->AddObject( overlay);

/* add a callback to add layers.  use the varargs callback interface. 
   Note the END_LIST terminator.  We can't use NULL, because it's a common
   callback argument */ 

    AmAddVaCallback( addBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmAddLayerCallback,
    		   motifDisplay, overlay, END_LIST);

/* add a callback to delete layers */

    AmAddVaCallback( delBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmDeleteLayerCallback,
    		   motifDisplay, overlay, END_LIST);

/* add callbacks to zoom and unzoom */

    AmAddVaCallback( zoomBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmZoomRectCallback,
    		   motifDisplay, overlay, END_LIST);
    AmAddVaCallback( unzoomBtn, XmNactivateCallback,
    		   ( VarargsCallback) AmUnzoomCallback,
    		   motifDisplay, overlay, END_LIST);

/* realize the top level widget and start processing events */

    XtRealizeWidget( top);
    XtMainLoop();
}
