#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _tool_kit_base_widget_class_h_
#define _tool_kit_base_widget_class_h_

#include <ToolKit/ToolKit.h>

class AmBaseWidgetClass {

private:

    void initializeValues();

protected:

    /* Member variables. */

    Widget      _wParent;
    Widget      _wBase;

    char        *_szName;

    Boolean     _fDestroyCallback;

    /* Member functions. */


    void setDefaultResources( const char **szResources);
    void getResources( Widget               w,
                       const XtResourceList xResources,
                       const int            nResources);

    virtual void show();
    virtual void hide();

    const char *getResource( const char *szResourceName,
                             const char *szResourceClass,
                             const char *szDefaultValue);

    void addDestroyCallback();
    void removeDestroyCallback();

    /* Callbacks. */

    static void AmDestroyCallback( Widget w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void destroy( Widget    w,
                  XtPointer xpCallData);

public:

    /* Constructors/Desctructors. */

    AmBaseWidgetClass();
    AmBaseWidgetClass( Widget          wParent,
                       const char      *szName);
    AmBaseWidgetClass( AmBaseWidgetClass *bwcParent,
                       const char      *szName);
    virtual ~AmBaseWidgetClass();

    /* Display functions. */

    virtual void Show();
    virtual void Hide();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "AmBaseWidgetClass"); };
    virtual const char *GetName();
    virtual Widget     GetParentWidget();
    virtual Widget     GetWidget();
    virtual int        GetWidth();
    virtual int        GetHeight();

    /* Mutator functions. */

    void AttachLeft(); 
    void AttachLeft( Widget w); 
    void AttachLeft( AmBaseWidgetClass *pwc); 
    void AttachLeft( int iPos); 
    void AttachTop(); 
    void AttachTop( Widget w); 
    void AttachTop( AmBaseWidgetClass *pwc); 
    void AttachTop( int iPos); 
    void AttachRight(); 
    void AttachRight( Widget w); 
    void AttachRight( AmBaseWidgetClass *pwc); 
    void AttachRight( int iPos); 
    void AttachBottom(); 
    void AttachBottom( Widget w); 
    void AttachBottom( AmBaseWidgetClass *pwc); 
    void AttachBottom( int iPos); 

    void LeftOffset( int iOffset);
    void TopOffset( int iOffset);
    void RightOffset( int iOffset);
    void BottomOffset( int iOffset);

    void AddCallback( const char     *szReason,
                      XtCallbackProc xcpCallback,
                      XtPointer      xpData);
    void RemoveCallback( const char     *szReason,
                         XtCallbackProc xcpCallback,
                         XtPointer      xpData);

    /* Casting functions. */

    virtual operator const char *();
    virtual operator Widget();
};

#endif /* _tool_kit_base_widget_class_h_ */
