#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef OBJECT_H
#define OBJECT_H

/* Constant definitions: */

#define AmOBJECT_ERROR          0x00	/* Segment status. */
#define AmOBJECT_DIRTY          0x01
#define AmOBJECT_INITIALIZED	0x40
#define AmOBJECT_UNINITIALIZED  0x80

#define AmOBJECT_MASK_DIRTY	0xfe

/* Type definitions: */

typedef struct CadPointRec_ {
    float x;
    float y;
} AmCadPointRec, *AmCadPointPtr;

typedef struct CadLineRec_ {
    long  p1;          /* Pointer to end point #1. */
    long  p2;          /* Pointer to end point #2. */
} AmCadLineRec, *AmCadLinePtr;

typedef struct AmCadPolyLineRec {
    float xmin;
    float ymin;
    float xmax;
    float ymax;
    long  numPoints;   /* Number of pointers in point object list. */
//  long  points[];    /* Point object list. */
} AmCadPolyLineRec, *AmCadPolyLinePtr;

typedef struct CadPolygonRec_ {
    float xmin;
    float ymin;
    float xmax;
    float ymax;
    long  prev;
    long  next;
    long  AmNumLines;    /* Number of pointers in polyline object list. */
//  long  lines[];     /* Polyline object list. */
} AmCadPolygonRec, *AmCadPolygonPtr;

typedef struct CadAttributeRec_ {
    long    type;
    long    size;
//  uchar_t data[];
} AmCadAttributeRec, *AmCadAttributePtr;

typedef struct CadRefListRec_ {
    long  numRefs;     /* Number of pointers in reference list. */
//  long  refs[];      /* Reference list. */
} AmCadRefListRec, *AmCadRefListPtr;

typedef struct CadAttribListRec_ {
    long  numAttribs;  /* Number of pointers in attribute list. */
//  long  attribs[];   /* Attribute list. */
} AmCadAttribListRec, *AmCadAttribListPtr;

typedef struct CadObjectRec_ {
    long  references;
    long  attributes;
} AmCadObjectRec, *AmCadObjectPtr;

typedef struct objectRefRec_ {
    long         segmentID;
    long         offset;
} AmCadObjectRefRec, *AmCadObjectRefPtr;

typedef struct objectTableRec_ {
    long            segSize;
    long            loadedSeg;
    long            flag;
    long            AmLast;
    long            free;
    AmCadObjectRefPtr objects;
} AmObjectTableRec, *AmObjectTablePtr;

/* Inline functions: */

#define INVALIDOBJECT( object)                                      \
    (( object == NULL) || ( object->segmentID == AmINVALID))

#define GET_OBJECT( header)                                         \
    (( AmCadObjectPtr) (( uchar_t *) header + sizeof( AmHeaderRec)))

#define GET_OBJECT_DATA( header)                                    \
    ((( uchar_t *) header) + sizeof( AmHeaderRec) +                   \
        sizeof( AmCadObjectRec))

#define GET_POINT( header)                                          \
    (( AmCadPointPtr) (( uchar_t *) header + sizeof( AmHeaderRec) +     \
        sizeof( AmCadObjectRec)))

#define GET_LINE( header)                                           \
    (( AmCadLinePtr) (( uchar_t *) header + sizeof( AmHeaderRec) +      \
        sizeof( AmCadObjectRec)))

#define GET_POLYLINE( header)                                       \
    (( AmCadPolyLinePtr) (( uchar_t *) header + sizeof( AmHeaderRec) +  \
        sizeof( AmCadObjectRec)))

#define GET_POLYLINE_DATA( header)                                  \
    (( long *) (( uchar_t *) header + sizeof( AmHeaderRec) +          \
        sizeof( AmCadObjectRec) + sizeof( AmCadPolyLineRec)))

#define GET_POLYGON( header)                                        \
    (( AmCadPolygonPtr) (( uchar_t *) header + sizeof( AmHeaderRec) +   \
        sizeof( AmCadObjectRec)))

#define GET_POLYGON_DATA( header)                                   \
    (( long *) (( uchar_t *) header + sizeof( AmHeaderRec) +          \
        sizeof( AmCadObjectRec) + sizeof( AmCadPolygonRec)))

#define GET_REFERENCE_LIST( header)                                 \
    (( AmCadRefListPtr) (( uchar_t *) header + sizeof( AmHeaderRec)))

#define GET_REFERENCE_LIST_DATA( header)                            \
    (( long *) (( uchar_t *) header + sizeof( AmHeaderRec) +          \
        sizeof( AmCadRefListRec)))

#define GET_ATTRIBUTE_LIST( header)                                 \
    (( AmCadAttribListPtr) (( uchar_t *) header +                     \
        sizeof( AmHeaderRec)))

#define GET_ATTRIBUTE_LIST_DATA( header)                            \
    (( long *) (( uchar_t *) header + sizeof( AmHeaderRec) +          \
        sizeof( AmCadAttribListRec)))

#define GetPointLOCAL( memory, pointID, point)                          \
{                                                                       \
    AmCadPointPtr temp;                                                   \
    uchar_t     *data;                                                  \
                                                                        \
    data = AmGetData( memory, pointID);                                   \
    if ( data != NULL) {                                                \
        if ( HTYPE((( AmHeaderPtr) data)) == AmTYPE_POINT) {              \
            temp = GET_POINT( data);                                    \
            point = ( AmCadPointPtr) alloca( sizeof( AmCadPointRec));       \
            if ( point != NULL) {                                       \
                memcpy( point, temp, sizeof( AmCadPointRec));             \
            }                                                           \
        }                                                               \
    }                                                                   \
}

#define GetPolyLineLOCAL( memory, polyLineID, polyLine)                 \
{                                                                       \
    AmCadPolyLinePtr temp;                                                \
    uchar_t        *data;                                               \
    int            size;                                                \
									\
    data = AmGetData( memory, polyLineID);                                \
    if ( data != NULL) {                                                \
        if ( HTYPE((( AmHeaderPtr) data)) == AmTYPE_POLYLINE) {           \
            temp = GET_POLYLINE( data);                                 \
            size = sizeof( AmCadPolyLineRec) + sizeof( long) *            \
                   temp->numPoints;                                     \
									\
            polyLine = ( AmCadPolyLinePtr) alloca( size);                 \
            if ( polyLine != NULL) {                                    \
                memcpy( polyLine, temp, size);                          \
            }                                                           \
        }                                                               \
    }                                                                   \
}

#define GetPolygonLOCAL( memory, polygonID, polygon)                    \
{                                                                       \
    AmCadPolygonPtr temp;                                                 \
    uchar_t       *data;                                                \
    int           size;                                                 \
									\
    polygon = NULL;                                                     \
    data = AmGetData( memory, polygonID);                                 \
    if ( data != NULL) {                                                \
        if ( HTYPE((( AmHeaderPtr) data)) == AmTYPE_POLYGON) {            \
            temp = GET_POLYGON( data);                                  \
            size = sizeof( AmCadPolygonRec) + sizeof( long) *             \
                   temp->AmNumLines;                                      \
									\
            polygon = ( AmCadPolygonPtr) alloca( size);                   \
            if ( polygon != NULL) {                                     \
                memcpy( polygon, temp, size);                           \
            }                                                           \
        }                                                               \
    }                                                                   \
}

#define GetReferenceListLOCAL( memory, objectID, refList)               \
{                                                                       \
    AmCadObjectPtr   object;                                              \
    uchar_t        *data;                                               \
    int            refID, size;                                         \
    AmCadRefListPtr  temp;                                                \
									\
    refList = NULL;                                                     \
    data = AmGetData( memory, objectID);                                  \
    if ( data != NULL) {                                                \
                                                                        \
        if ( HTYPE((( AmHeaderPtr) data)) == AmTYPE_POINT ||              \
             HTYPE((( AmHeaderPtr) data)) == AmTYPE_LINE ||               \
             HTYPE((( AmHeaderPtr) data)) == AmTYPE_POLYLINE ||           \
             HTYPE((( AmHeaderPtr) data)) == AmTYPE_POLYGON) {            \
                                                                        \
            object = GET_OBJECT( data);                                 \
                                                                        \
            refID = object->references;                                 \
            if ( refID != AmINVALID) {                                  \
                                                                        \
                data = AmGetData( memory, refID);                         \
                if ( data != NULL) {                                    \
                    temp = GET_REFERENCE_LIST( data);                   \
                    size = sizeof( AmCadRefListRec) + sizeof( long) *     \
                           temp->numRefs;                               \
                                                                        \
                    refList = ( AmCadRefListPtr) alloca( size);           \
                    if ( refList != NULL) {                             \
                        memcpy( refList, temp, size);                   \
                    }                                                   \
                }                                                       \
            }                                                           \
        }                                                               \
    }                                                                   \
}
    
#endif
