#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef SEGMENT_H
#define SEGMENT_H

#include <anvil/cad/list.h>

/* Constant definitions: */

#define AmINVALID	(signed)0xffffffff	/* The "NULL" pointer used. */

#define AmSEGMENT_ERROR         0x00	/* Segment status. */
#define AmSEGMENT_DIRTY         0x01
#define AmSEGMENT_SWAPPED       0x02
#define AmSEGMENT_INITIALIZED   0x40
#define AmSEGMENT_UNINITIALIZED 0x80

#define AmSEGMENT_MASK_DIRTY	0xfe
#define AmSEGMENT_MASK_SWAPPED	0xfd

#define AmTYPE_ERROR		0	/* Object types. */
#define AmTYPE_POINT		1
#define AmTYPE_LINE		2
#define AmTYPE_POLYLINE		3
#define AmTYPE_POLYGON		4
#define AmTYPE_LIST		5
#define AmTYPE_REFERENCE_LIST	6
#define AmTYPE_ATTRIBUTE_LIST	7
#define AmTYPE_ATTRIBUTE        8
#define AmTYPE_OBJECT		9
#define AmTYPE_FREE		10

#define NUM_TYPES		9

/* Some inline functions: */

/*
 * TYPE( header)
 *     Gets the type of an object.
 */

#define TYPE( s) ( s >> 24)

/*
 * SIZE( header)
 *     Gets the size of an object.
 */

#define SIZE( s) ( s & 0x0fff)

/*
 * HTYPE( header)
 *     Gets the type of an object from a header.
 */

#define HTYPE( h) ( h->size >> 24)	/* Get the type from a header. */

/*
 * HSIZE( header)
 *     Gets the size of an object from a header.
 */

#define HSIZE( h) ( h->size & 0x0fff)

/*
 * HID( header)
 *     Gets the ID of an object from a header.
 */

#define HID( h) ( h->ID)

/*
 * SET_HSIZE( header, size)
 *     Sets the type of an object in a header.
 */

#define SET_HTYPE( h, t)		/* Set the type in a header */	\
    h->size &= 0x0fff;							\
    h->size += ( t << 24)

/*
 * SET_HSIZE( header, size)
 *     Sets the size of an object in a header.
 */

#define SET_HSIZE( h, s)						\
    h->size &= 0xf000;							\
    h->size += s

/*
 * SET_H( header, type, size)
 *     Sets the type and the size of an object in a header.
 */

#define SET_H( h, t, s)						\
    h->size = ( t << 24) + s

/*
 * SET_HID( header, ID)
 *     Sets the ID in a header.
 */

#define SET_HID( h, i)						\
    h->ID = i

/*
 * PARITYSIZE( size) 
 *     Rounds the size up to the nearest even memory location.
 */

#define PARITYSIZE( size)					\
     if ( ( size % iSize) != 0) {				\
         size = size + iSize - ( size % iSize);			\
     }

/*
 * VALID( segment) 
 *     TRUE when the segment is initialized, FALSE otherwise.
 */

#define VALID( seg)						\
    ( seg->status & AmSEGMENT_INITIALIZED)

/*
 * INVALID( segment) 
 *     TRUE when the segment is not initialized, FALSE otherwise.
 */

#define INVALID( seg)						\
    ( seg->status & AmSEGMENT_UNINITIALIZED)

/*
 * SWAPPED( segment) 
 *     TRUE when the segment is swapped, FALSE otherwise.
 */

#define SWAPPED( seg)						\
    ( seg->status & AmSEGMENT_SWAPPED)

/*
 * SWAPPED( segment) 
 *     TRUE when the segment is dirty, FALSE otherwise.
 */

#define DIRTY( seg)						\
    ( seg->status & AmSEGMENT_DIRTY)

/* Type definitions: */

typedef struct headerRec_ {
    long size;                 /* Contains the size and the type. */
    long ID;                   /* The object's ID. */
} AmHeaderRec, *AmHeaderPtr;

typedef struct memorySegmentRec_ {
    uchar_t      status;       /* Contains the dirty and swap flags. */
    long         size;
    long         AmLast;         /* Last used segment. */
    long         freed;        /* How much memory has been freed. */ 
    uchar_t      *data;        /* Dynamically allocated, depends on the size. */
    char         *swap;
} AmMemorySegmentRec, *AmMemorySegmentPtr;

typedef struct segmentTableRec_ {
    long         	size;
    long         	AmLast;
    AmMemorySegmentRec  *segments;
    AmCadListPtr   	map[ NUM_TYPES];
    AmCadNodePtr   	free[ NUM_TYPES];
} AmSegmentTableRec, *AmSegmentTablePtr;

#endif
