#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_cadplot_h_
#define _anvil_cadplot_h_

#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/cad/segment.h>
#include <anvil/cad/object.h>
#include <anvil/cad/mem.h>

#define MAXLINES   2048	

typedef struct AmAttribRec_ {
    char         *szType;
    AmIndexDataPtr indexData;
    AmListPtr      lstDpyParams;
} AmAttribRec, *AmAttribPtr;

typedef struct AmDisplayParameterRec_ {
    char        *szType;
    void        *pData;
    long        iElementSize;
    long        iLength;
} AmDisplayParameterRec, *AmDisplayParameterPtr;


class AmCadPlotClass : public AmPlotClass {
private:

    /* Private member variables. */

    AmMemoryPtr _memory;        /* The data. */
    char      *_szCmdHistory; /* Filename of command history . */
    AnvilBoolean   _fChanged;      /* Flag indicating change in the data. */ 

    /* Object plot initialization function. */

    void        initSpecificFields();
    AmErrorCode initMemory();

    /* Low level drawing functions. */

    AmErrorCode drawPoint( AmWindowClass *window,
			   AmGCClass     *gc,
			   AmCadPointPtr point);
    AmErrorCode drawPolyLine( AmWindowClass        *window,
			      AmGCClass            *gc,
			      AmCadPolyLinePtr     polyLine);
    AmErrorCode drawPolygon( AmWindowClass        *window,
			     AmGCClass            *gc,
			     AmCadPolygonPtr      polygon);
    AmErrorCode drawOutlinePolygon( AmWindowClass   *window,
			            AmGCClass       *gc,
				    long          polygonID,
				    AmCadPolygonPtr polygon,
				    AmListPtr       selected,
				    int           shiftX,
				    int           shiftY);

protected:
    void    GetFieldDefinitions( AmFieldDefPtr *fieldDefs,
                                 int	     *numDefs);
    void    initializeFields( AmWindowClass *windowArg,
                              const char  *nameArg);
    virtual AnvilBoolean	checkValues();

public:

    AmCadPlotClass( AmWindowClass *windowArg,
                  const char   *nameArg,
                  ...);
    AmCadPlotClass( AmOverlayClass *overlayArg,
                  const char   *nameArg,
                  ...);
    virtual ~AmCadPlotClass();

    virtual int setFields( AmTagValuePtr  tagValues,
			   int	        numValues);
    virtual int getFields( AmTagValuePtr  tagValues,
			   int	        numValues);
    virtual void updateFields();
    virtual int dumpFields( AmFieldLengthPtr	fieldLengths,
			    AnvilBoolean		allFields = FALSE);
    virtual AnvilBoolean IsType( AmObjectTypeEnum type);
    
    virtual AmErrorCode draw( AmWindowClass        *window,
			      AmGCClass		 *gc,
			      AmRectangleRec const *exposeRects,
			      int                nRects);
    
    virtual AmErrorCode DrawPoint( AmWindowClass      *window,
				   AmGCClass		*gc,
				   long             pointID);
    
    AmErrorCode DrawOutlinePoint( AmWindowClass *window,
				  AmGCClass     *gc,
				  long        pointID,
				  AmListPtr     selected,
				  int         shiftX,
				  int         shiftY);
    
    virtual AmErrorCode DrawPolyLine( AmWindowClass *window,
				      AmGCClass     *gc,
				      long        polyLineID);
    
    AmErrorCode DrawOutlinePolyLine( AmWindowClass *window,
				     AmGCClass     *gc,
				     long        polyLineID,
				     AmListPtr     selected,
				     int         shiftX,
				     int         shiftY);
    
    virtual AmErrorCode DrawPolygon( AmWindowClass *window,
				     AmGCClass     *gc,
				     long        polygonID);
    
    AmErrorCode DrawOutlinePolygon( AmWindowClass *window,
				    AmGCClass     *gc,
				    long        polygonID,
				    AmListPtr     selected,
				    int         shiftX,
				    int         shiftY);
    
    virtual AmErrorCode DrawObject( AmWindowClass *window,
				    AmGCClass     *gc,
				    long        objectID);
    
    AmErrorCode DrawOutlineObject( AmWindowClass *window,
				   AmGCClass     *gc,
				   long        objectID,
				   AmListPtr     selected,
				   int         shiftX,
				   int         shiftY);
    
    long                AddPoint( float x, float y);
    long                AddPoint( int x, int y);
    
    AmCadPointPtr         AmGetPoint( long pointID);
    AmErrorCode         GetPointLocation( long pointID, int *x, int *y);
    AmErrorCode         AmSetPoint( long pointID, int x, int y);
    AmErrorCode         MovePoint( long pointID, int shiftX, int shiftY);
    
    long                AddLine( long p1, long p2);
    long                AddPolyLine( long numPts, long *pts);
    long                AddPolygon( long AmNumLines, long *lines);

    AmErrorCode         MovePolygon( long polygonID, int shiftX, int shiftY);
    
    virtual AmErrorCode AmRemovePoint( long pointID);
    virtual AmErrorCode AmRemoveLine( long lineID);
    virtual AmErrorCode AmRemovePolyLine( long polyLineID);
    virtual AmErrorCode AmRemovePolygon( long polygonID);

    AmErrorCode         MoveObject( long objectID, int shiftX, int shiftY);
    
    long                FindPoint( int x, int y);
    long                FindPolygon( int x, int y);
    
    AmListPtr             GetParents( long objectID);
    AmListPtr             GetParents( AmListPtr objects);

    /*
     * Some stuff that I am putting in here... they are used as a 
     *   quick attempt to introduce attributes to the cad objects.
     */

    AmListPtr             lstAttTypes;

    AmErrorCode         AddAttributeType( char         *szAttribute,
                                          AmIndexDataPtr dataInfo);

    AmErrorCode         AddAttributeDisplayParameter( char *szAttribute,
                                                      char *szDpyParam,
                                                      void *pData,
                                                      long iElementSize,
                                                      long iLength);

    int                 GetAttributeIndex( char *szAttType,
                                           char *szDpyParam,
                                           void *pValue);
};

#endif /* _anvil_cadplot_h_ */

