#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_colorcell_h_
#define _anvil_colorcell_h_

#include <sys/types.h>	/* for uchar_t and ulong_t */
#include <anvil/project.h>

/* red/green/blue, pixel, and number of allocators */

#if defined( _WIN32)

/* Character offsets. */

#define RED               2
#define GREEN             1
#define BLUE              0

/* Unsigned long shifts and masks. */

#define RED_SHIFT         16
#define GREEN_SHIFT       8
#define BLUE_SHIFT        0
#define RED_MASK	  0x00ff0000
#define GREEN_MASK	  0x0000ff00
#define BLUE_MASK	  0x000000ff

#else

/* Character offsets */

#define RED               3
#define GREEN             2
#define BLUE              1

/* Unsigned long shifts and masks. */

#define RED_SHIFT         0
#define GREEN_SHIFT       8
#define BLUE_SHIFT        16
#define RED_MASK	  0x000000ff
#define GREEN_MASK	  0x0000ff00
#define BLUE_MASK	  0x00ff0000

#endif

 
typedef struct AmColorCell {
    uchar_t     red;
    uchar_t     green;
    uchar_t     blue;
    ulong_t	pixel;
} AmColorCellRec, *AmColorCellPtr;

#define CALCULATE_PIXEL( r, g, b)                       \
    ((( ulong_t) r << RED_SHIFT) |                      \
     (( ulong_t) g << GREEN_SHIFT) |                    \
     (( ulong_t) b << BLUE_SHIFT))


#define ASSIGN_PIXEL( c, p)                             \
{                                                       \
    ( * ( ulong_t *) p) =                               \
        CALCULATE_PIXEL( (c).red, (c).green, (c).blue); \
}

#define CALCULATE_RED( p)                               \
    ((( ulong_t p) >> RED_SHIFT) & 0xff)

#define CALCULATE_GREEN( p)                             \
    ((( ulong_t p) >> GREEN_SHIFT) & 0xff)

#define CALCULATE_BLUE( p)                              \
    ((( ulong_t p) >> BLUE_SHIFT) & 0xff)

#define ASIGN_RGB( p, c)                                \
{                                                       \
    (c).red   = CALCULATE_RED( p);                      \
    (c).green = CALCULATE_GREEN( p);                    \
    (c).blue  = CALCULATE_BLUE( p);                     \
}


AnvilBoolean AmReadColors( char                *colorfile,
                    int                 *numColors,
                    AmColorCellPtr        *colorRet);

void AmBetweenColorcell( double           frac,
                       AmColorCellPtr     color1,
                       AmColorCellPtr     color2,
                       AmColorCellPtr     retColor);

void AmInterpolateColorCells( AmColorCellPtr   colors,
                            int            numColors,
                            AmColorCellPtr   *newColors,
                            int            numNewColors);

void AmShadeColors( AmColorCellPtr  colors,
                  AmColorCellPtr  shades,
                  int           numColors,
                  int           numShades);

AnvilBoolean AmCompareColor( AmColorCellPtr      a,
                      AmColorCellPtr      b);

AmColorCellPtr AmReadColorCellStr( char *str);

#endif /* _anvil_colorcell_h_ */
