#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_coordinates_h_
#define _anvil_coordinates_h_
#include <sys/types.h>
#include <anvil/project.h>
#include <anvil/units.h>
#include <anvil/rect.h>
#include <anvil/screenobj.h>
#include <anvil/projclass.h>

typedef struct AmCoords_ {
    float	currXmin;		/* current view extents */
    float	currYmin;
    float	currXmax;
    float	currYmax;
    float	xmin;			/* data extents */
    float	ymin;
    float	xmax;
    float	ymax;
} AmCoordsRec, *AmCoordsPtr;

/* the SetRect and SetCoords functions are virtual, because the overlay
   plot class also has virtual functions which set the rectangle and 
   coordinates for each of the plots */

class AmCoordClass : public AmScreenObjClass {
protected:
    AmProjectionClass	fwdProject;		/* forward projection */
    AmProjectionClass	rvsProject;
    AmCoordsRec		projCoords;		/* projected coord system */
    AnvilBoolean		coordsSet;		/* have coords been set? */
    AnvilBoolean		rangeSet;		/* has range been set? */
    AnvilBoolean		rectSet;		/* has rectangle been set */
    AnvilBoolean		preserveAspect;		/* preserve aspect ratio */
    float		aspectFactor;		/* aspect ratio factor */
    unitType		xUnits;
    unitType		yUnits;
    AmCoordsRec		coords;			/* coordinate system */
    char		**inProjStrs;		/* what data is projected in */
    int			numInProjStrs;		/* AmRecordClass vector req'd */
    char		**projStrs;
    int			numProjStrs;
    void setCoordClass();
    void GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
			      int		*numDefs);
    virtual AnvilBoolean	checkValues();
public:
    virtual int setFields( AmTagValuePtr  tagValues,
                           int          numValues);
    virtual int getFields( AmTagValuePtr      tagValues,
			   int              numValues);
    virtual int dumpFields( AmFieldLengthPtr	fieldLengths,
			    AnvilBoolean		allFields = FALSE);
    virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
    AmCoordClass();
    virtual 		~AmCoordClass();
    AnvilBoolean		IsProjected() { return( fwdProject.IsProjected()); };
    AmProjectionClass 	*GetForwardProjection();
    AmProjectionClass 	*GetReverseProjection();
    void 		GetRange( float *xmin,
				  float *ymin,
				  float *xmax,
				  float *ymax);
    void 		GetProjectRange( float *projXmin,
					 float *projYmin,
					 float *projXmax,
					 float *projYmax);
    void 		GetCoords( float *currXmin,
				   float *currYmin,
				   float *currXmax,
				   float *currYmax);
    void 		GetProjectCoords( float *projCurrXmin,
					  float *projCurrYmin,
					  float *projCurrXmax,
					  float *projCurrYmax);
   void        		SetProjection( const char **args);

    virtual void	SetRect( int	x,
				 int	y, 
				 int	width,
				 int	height);
    virtual void	SetCoords( float currXmin,
				   float currYmin,
				   float currXmax,
				   float currYmax);
    void	SetRange( float	currXmin,
			  float	currYmin,
			  float	currXmax,
			  float	currYmax);
    void 	ZoomRect( int	x,
			  int	y,
			  int	width,
			  int	height);
    void	Unzoom();
    void	GetDataPt( int		scrX,
			   int		scrY,
			   float	*dataX,
			   float	*dataY);
    void	GetScreenPt( float	dataX,
			     float	dataY,
			     int	*scrX,
			     int	*scrY);
    AnvilBoolean	InRect( int	scrX,
			int	scrY);
    AnvilBoolean	InCoords( float	dataX,
			  float dataY);
    AnvilBoolean	ClipRange( float	*dataX,
			   float	*dataY);
    AnvilBoolean	ClipCoords( float	*dataX,
			    float	*dataY);
    AnvilBoolean	OverlapRectCoords( float	xmin,
				   float	ymin,
				   float	xmax,
				   float	ymax);
    AnvilBoolean	BoundPt( int	*scrX,
			 int	*scrY);
    AnvilBoolean 	ScreenRealExtents( int              rows,
				   int              cols,
				   AnvilBoolean	    flipRows,
				   AnvilBoolean	    flipCols,
				   AmRectanglePtr     drawRect,
				   AmRealRectPtr      screenDataRect);
    AnvilBoolean MatrixRealExtents( float            matXmin,
			       float            matYmin,
			       float            matXmax,
			       float            matYmax,
			       float		dstXmin,
			       float		dstYmin,
			       float		dstXmax,
			       float		dstYmax,
			       int              rows,
			       int              cols,
			       AnvilBoolean		flipRows,
			       AnvilBoolean		flipCols,
			       AmRectanglePtr     drawRect,
			       AmRealRectPtr      screenDataRect);
    float       ScreenLineWidth( float dLineWidth);
    float	PixelSize();
    AnvilBoolean	CoordsSet();
    AnvilBoolean	RangeSet();
    AnvilBoolean	RectSet();
    void 	Pan( int       startX,
                     int       startY,
                     int       lastX,
                     int       lastY,
                     AnvilBoolean   restrictFlag = TRUE);
    void	PanInvalRects( int		x,
			       int		y,
			       AmRectanglePtr	vertRect,
			       AmRectanglePtr	horizRect);
    AnvilBoolean	GetRowcol( float	x,
			   float	y,
			   int		rows,
			   int		cols,
			   AnvilBoolean	rowFlip,
			   AnvilBoolean	colFlip,
			   int		*xpos,
			   int		*ypos);
    AmImageClass          *ResizeDisplayImage( AmImageClass         *image);
}; 

AnvilBoolean AmOverlapCoords( float  xminA,
			 float  yminA,
			 float  xmaxA,
			 float  ymaxA,
			 float  xminB,
			 float  yminB,
			 float  xmaxB,
			 float  ymaxB);

AnvilBoolean AmGetRowcolumn( float     x,
                      float     y,
                      int       rows,
                      int       cols,
                      AnvilBoolean   flipRows,
                      AnvilBoolean   flipCols,
                      float     xmin,
                      float     ymin,
                      float     xmax,
                      float     ymax,
                      int       *xpos,
                      int       *ypos);

#endif /* _anvil_coordinates_h_ */
