#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_description_h_
#define _anvil_description_h_

#include <anvil/datatype.h>
#include <anvil/list.h>
#include <anvil/record.h>

typedef enum {
    AmTYPE_ERROR = 0,
    AmTYPE_SIMPLE = 1,
    AmTYPE_REFERENCE = 2,
    AmTYPE_SCALAR = 3,
    AmTYPE_STATIC_ARRAY = 4,
    AmTYPE_DYNAMIC_ARRAY = 5,
    AmTYPE_EOF_ARRAY = 6,
    AmTYPE_STRUCT = 7
} AmTypeEnum;

typedef struct AmType_ AmTypeRec, *AmTypePtr;

typedef struct AmSymbol_ {			/* symbol table entry */
    char	*name;				/* symbol name */
    AmTypePtr	thing;				/* what it represents */
} AmSymbolRec, *AmSymbolPtr;

typedef struct AmSimpleType_ {
    char		*typeName;		/* type string name */
    AmDataTypeEnum	primitiveType;		/* its data type */
    int			refValue;		/* reference value for arrays */
} AmSimpleTypeRec, *AmSimpleTypePtr;

typedef struct AmReferenceType_ {
    char		*typeName;		/* name for later reference */
    AmTypePtr		type;			/* created pointer */
} AmReferenceTypeRec, *AmReferenceTypePtr;

typedef struct AmScalarType_ {
    AmTypePtr		type;			/* scalar type */
} AmScalarTypeRec, *AmScalarTypePtr;

typedef struct AmStaticArrayType_ {
    AmTypePtr		type;			/* field type */
    int			length;			/* field length */
} AmStaticArrayTypeRec, *AmStaticArrayTypePtr;

typedef struct AmDynamicArrayType_ {
    AmTypePtr		type;
    char		*lengthname;
    AmTypePtr		lengthType;
} AmDynamicArrayTypeRec, *AmDyanmicArrayTypePtr;

typedef struct AmEofArrayType_ {
    AmTypePtr	type;
} AmEofArrayTypeRec, *AmEofArrayTypePtr;

typedef struct AmStructType_ {
    int		numFields;
    AmTypePtr	*fields;
} AmStructTypeRec, *AmStructTypePtr;
  

struct AmType_ {
    AmTypeEnum	tag;				/* what kind of type */
    char	*name;				/* string token */
    AnvilBoolean	typedefFlag;			/* typdef or declaration */
    int		lineNo;				/* line # parsed at */
    union {
	AmSimpleTypeRec		simpleType;		/* primitive */
	AmReferenceTypeRec	referenceType;		/* typedef reference */
	AmScalarTypeRec		scalarType;		/* scalar variable */
	AmStaticArrayTypeRec	staticArrayType;	/* e.g. int x[ 10] */
	AmDynamicArrayTypeRec	dynamicArrayType;	/* e.g. int x[ n] */
	AmStructTypeRec		structType;		/* struct fields */
	AmEofArrayTypeRec		eofArrayType;		/* this type to EOF */
    } u;
};

/* structure for reading types into buffers on successive reads, with
   the list of types, a pointer to the next (or current) type to be read,
   and if the current type is an incomplete static or dynamic array,
   the index into that array */

typedef struct AmReadType_ {
    AmListPtr	typeList;
    AmListPtr	currTypePtr;
    int		currIndex;
} AmReadTypeRec, *AmReadTypePtr;

AnvilBoolean AmResolveType( AmTypePtr    type,
                     char       *errorString);

AnvilBoolean AmResolveTypeList( AmListPtr        typeList,
                         char           *errorString);

AmTypePtr AmMakeScalar( char	*typeName,
                    char        *name);

AmTypePtr AmMakeStatic( char	*typeName,
                    char        *name,
                    char        *length);

AmTypePtr AmMakeDynamic( char       *typeName,
                     char       *name,
                     char       *lengthname);

AmTypePtr AmMakeEof( char       *typeName,
		 char       *name);

AmListPtr AmAddDecl( AmTypePtr        decl,
                 AmListPtr        declList);

AmListPtr AmAddType( AmTypePtr        type,
                 AmListPtr        typeList);

AmTypePtr AmMakeStruct( char        *name,
                    AmListPtr     fieldList);

AmTypePtr AmMakeTypedef( AmTypePtr	decl);

AmErrorCode AmParseDescriptionFile( char          *file,
                                  AmListPtr       *typeList,
                                  char          *errorString);

int AmFpYYinput( char    *buffer,
	       int     maxSize);

void AmPrintDescription( AmListPtr  typeList);

void AmPrintType( AmTypePtr	type);

AmListPtr AmGetDeclarations( AmListPtr typeList);

int AmReadParseTypeList( AmReadTypePtr      readType,
                       uchar_t          *buffer,
                       int              length,
		       int		*numelParsed,
                       AnvilBoolean          swap);

#endif /* _anvil_description_h_ */
