#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_dialang_h_
#define _anvil_dialang_h_

#include <anvil/list.h>
#include <anvil/errcode.h>

typedef enum dialogType_ {
    AmDIALOG_ERROR = 0,
    AmDIALOG_IF_THEN = 1,
    AmDIALOG_IF_THEN_ELSE = 2,
    AmDIALOG_BOX_FIELD_LIST = 3,
    AmDIALOG_POPUP_FIELD_LIST = 4,
    AmDIALOG_EDITOR = 5,
    AmDIALOG_IDENT = 6
} AmDialogTypeEnum;

typedef enum boolOpType_ {
    AmBOOLEAN_ERROR = 0,
    AmBOOLEAN_EQUAL = 1,
    AmBOOLEAN_NOT_EQUAL = 2
} AmBoolOpType;

typedef struct AmBoolExpr_ {
    AmBoolOpType	op;
    char		*tag;
    char		*value;
} AmBoolExprRec, *AmBoolExprPtr;

typedef struct AmConditionalItem_ {
    AmBoolExprPtr	boolExpr;
    AmListPtr		trueDeclList;
    AmListPtr		falseDeclList;
} AmConditionalItemRec, *AmConditionalItemPtr;

typedef struct AmDeclItem_ {
    char		*name;
    AmListPtr		fields;
} AmDeclItemRec, *AmDeclItemPtr;

typedef struct AmDialogEntry_ {
    AmDialogTypeEnum	type;
    union {
	AmConditionalItemRec	conditionalItem;
	AmDeclItemRec		declItem;
	char			*identItem;
    } u;
} AmDialogEntryRec, *AmDialogEntryPtr;

typedef struct AmDeclaration_ {
    char	*ident;
    AmListPtr	declList;
} AmDeclarationRec, *AmDeclarationPtr;


AmDialogEntryPtr AmMakeBooleanDecl( AmBoolExprPtr	boolExpr,
				    AmListPtr		trueDeclList,
				    AmListPtr		falseDeclList);

AmDialogEntryPtr AmMakeFieldList( char		*ident,
				  AmListPtr	list);

AmDialogEntryPtr AmMakeIdent( char	*ident);

AmDeclarationPtr AmMakeDecl( char	*ident,
			     AmListPtr	list);

AmBoolExprPtr AmMakeBoolexpr( char	*ident,
			      char	*op,
			      char	*value);

AmListPtr AmAddDecl( AmDeclarationPtr	decl,
		     AmListPtr		declList);

int AmDialogFpYYinput( char 	*buffer,
		       int	maxSize);

AmErrorCode AmParseDialogFile( char       *file,
                               AmListPtr    *declList,
                               char       *errorString);

void
AmFreeDialogEntry( AmDialogEntryPtr     dialogEntry);

#endif /* _anvil_dialang_h_ */
