#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_errcode_h_
#define _anvil_errcode_h_

#include <anvil/project.h>

typedef enum {
	AmOK = 0,
	AmREAD_FAILED = 1,
	AmMALLOC_FAILED = 2,
	AmOPEN_FAILED = 3,
	AmNULL_ARG_PTR = 4,
	AmSEEK_FAILED = 5,
	AmREAD_PYRAMID_HEADER_FAILED = 6,
	AmCOLOR_ALLOCATION_FAILED = 7,
	AmNULL_ARGUMENT = 8,
	AmBAD_TYPE = 9,
	AmSTAT_FAILED = 10,
	AmFILE_NO_EXIST = 11,
	AmBAD_ARGUMENT = 12,
	AmBAD_LOOKUP_FUNCTION = 13,
	AmBAD_ACCESS_FUNCTION = 14,
	AmNO_SOURCE_RANGE = 15,
	AmNOT_VALID = 16,
	AmENV_VAR_UNDEF = 17,
	AmFILE_INVALID = 18,
	AmMISSING_CONFIG = 19,
	AmUNINITIALIZED = 20,
	AmNO_FONTS = 21,
	AmWRITE_FAILED = 22,
	AmNO_APPLICABLE_VISUAL = 23,
        AmILLEGAL_STRING = 24,
        AmSEGMENT_INVALID = 25,
        AmFREE_MEMORY = 26,
        AmSEGMENT_FULL = 27,
        AmOBJECT_INVALID = 28,
        AmOBJECT_TABLE_FULL = 29,
        AmSEGMENT_TABLE_FULL = 30,
        AmOBJECT_STILL_REFERENCED = 31,
        AmID_NOT_IN_CACHE = 32,
        AmOBJECT_NOT_POINT = 33,
	AmALLOCA_FAILED = 34,
	AmPARSE_FAILED = 35,
	AmMMAP_FAILED = 36,
	AmATEXIT_FAILED = 37,
	AmSHM_DETACH_ERROR = 38,
	AmSHM_ATTACH_ERROR = 39,
	AmBUFFER_OVERFLOW = 40,
	AmDUP_FAILED = 41,
	AmMAGIC_FAILED = 42,
	AmREALLOC_FAILED = 43,
	AmZERO_DIMENSION = 44,
	AmMISSING_FILENAME = 45,
	AmDB_UNINITIALIZED = 46,
	AmUNKNOWN_USER_DATA = 47,
	AmUSER_DATA_KEY_EXISTS = 48,
	AmMISSING_USER_DATA = 49,
	AmMISSING_USER_DATA_KEY = 50,
	AmMISSING_EVENT_HANDLER = 51,
	AmSTRDUP_FAILED = 53,
	AmNULL_WIDGET = 54,
	AmUNKNOWN_ERROR = 99
} AmErrorCode;

extern AmErrorCode	ANVIL_ERROR_CODE;
extern char		*ANVIL_ERROR_FILE;
extern int		ANVIL_ERROR_LINE;
extern char		ANVIL_ERROR_DETAIL[ 512];

#define RETURN_NOT_OK( expr) 			\
{						\
	AmErrorCode errcode;			\
	if (( errcode = ( expr)) != AmOK) 	\
	    return( errcode);			\
}				

#define RETURN_ERROR( code)                     \
    ANVIL_ERROR_CODE = code;                    \
    return( code)

#define DO_ERROR( code) 			\
    AmSetError( code, __FILE__, __LINE__);

#define DO_RETURN_ERROR( code) 			\
{						\
    AmSetError( code, __FILE__, __LINE__);	\
    return( code);				\
}

#define DO_RETURN_ERROR_DETAIL( code, msg) 	\
{						\
    AmSetError( code, __FILE__, __LINE__);	\
    strcpy( ANVIL_ERROR_DETAIL, msg);		\
    return( code);				\
}

void AmSetError( AmErrorCode      code,
	       char             *file,
	       int              line);

#endif /* _anvil_errcode_h_ */
