#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_expr_h_
#define _anvil_expr_h_

#include <anvil/project.h>

typedef enum {
    AmEXPR_ERROR = 0,
    AmEXPR_NUMBER = 1,
    AmEXPR_IDENT = 2,
    AmEXPR_FUNCTION = 3,
    AmEXPR_BINEXPR = 4,
    AmEXPR_FUNCTION2 = 5,
    AmEXPR_FUNCTION3 = 6,
    AmEXPR_FUNCTION4 = 7,
} AmExprTypeEnum; 

typedef struct AmExpr_ AmExprRec, *AmExprPtr;

typedef float ( *AmFunFn)( float);
typedef float ( *AmFunFn2)( float, float);
typedef float ( *AmFunFn3)( float, float, float);
typedef float ( *AmFunFn4)( float, float, float, float);

typedef struct AmFunexpr_ {
    AmFunFn	fn;
    AmExprPtr	arg;
} AmFunexprRec, *AmFunexprPtr;

typedef struct AmFunexpr2_ {
    AmFunFn2	fn;
    AmExprPtr	arg1;
    AmExprPtr	arg2;
} AmFunexpr2Rec, *AmFunexpr2Ptr;

typedef struct AmFunexpr3_ {
    AmFunFn3	fn;
    AmExprPtr	arg1;
    AmExprPtr	arg2;
    AmExprPtr	arg3;
} AmFunexpr3Rec, *AmFunexpr3Ptr;

typedef struct AmFunexpr4_ {
    AmFunFn4	fn;
    AmExprPtr	arg1;
    AmExprPtr	arg2;
    AmExprPtr	arg3;
    AmExprPtr	arg4;
} AmFunexpr4Rec, *AmFunexpr4Ptr;

typedef struct AmBinexpr_ {
    char	op;
    AmExprPtr	left;
    AmExprPtr	right;
} AmBinexprRec, *AmBinexprPtr;

typedef struct AmIdent_ {
    char	*name;
    float	value;
} AmIdentRec, *AmIdentPtr;

struct AmExpr_ {
    AmExprTypeEnum	exprType;
    union {
    	AmIdentPtr	ident;
    	float		number;
    	AmFunexprRec	funexpr;
    	AmFunexpr2Rec	funexpr2;
    	AmFunexpr3Rec	funexpr3;
    	AmFunexpr4Rec	funexpr4;
    	AmBinexprRec	binexpr;
    } u;    
};

AmExprPtr AmMakeBinexpr( char 	*op,
		     AmExprPtr	leftExpr,
		     AmExprPtr	rightExpr);

AmExprPtr AmMakeIdentifier( char	*ident);

AmExprPtr AmMakeNumber( char *numberStr);

AmExprPtr AmMakeFunexpr( char 	*fn,
		     AmExprPtr	arg);

AmExprPtr AmMakeFunexpr2( char 	*fn,
		      AmExprPtr	arg1,
		      AmExprPtr	arg2);

AmExprPtr AmMakeFunexpr3( char 	*fn,
		      AmExprPtr	arg1,
		      AmExprPtr	arg2,
		      AmExprPtr	arg3);

AmExprPtr AmMakeFunexpr4( char 	*fn,
		      AmExprPtr	arg1,
		      AmExprPtr	arg2,
		      AmExprPtr	arg3,
		      AmExprPtr	arg4);

float AmEvalSimpleExpr( AmExprPtr	expr,
		      float	identVal);

void AmAssignIdents( AmIdentPtr     identVec,
                   float        *values,
                   int          numIdents);

float AmEvalAssignedExpr( AmExprPtr expr);

AnvilBoolean AmSubstituteExpr( AmExprPtr         expr,
                        AmIdentPtr        identVec,
                        int             numIdents,
                        char            *errorMsg);

AnvilBoolean AmCopyExprBuffer( char    *buffer);

int AmExprYYinput( char   *buffer,
                 int    maxSize);

AmExprPtr AmParseExpr( char	*exprStr);

#endif /* _anvil_expr_h_ */
