#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_extents_h_
#define _anvil_extents_h_

#include <anvil/display.h>

/* this offshoot from an X display class is for tracking the extents from
   a sequence of drawing commands. */

class AmExtentsClass : public AmDisplayClass {
private:
    AmDisplayClass	*sourceDisplay;
    int			xmin;
    int			ymin;
    int			xmax;
    int			ymax;
    void addExtents( int	xmin,
		     int	ymin,
		     int	xmax,
		     int	ymax);
    void addExtents( int	x,
		     int	y);
public:
    AmExtentsClass( AmDisplayClass	*disp);
    virtual ~AmExtentsClass();
    void StartExtents();
    AmRectangleRec const *GetExtents();
    virtual int GetDepth() { return( sourceDisplay->GetDepth()); }
    virtual void GetImage( AmWindowClass 	*pWindow, 
			   AmImageClass 	*pImage, 
			   int	      	x,
			   int		y);
    virtual void DrawLines( AmWindowClass 	*w, 
			    AmGCClass 		*gc, 
			    AmPointPtr 		pts, 
			    int 		numpts);
    virtual void DrawPoints( AmWindowClass *w, 
			     AmGCClass     *gc, 
			     AmPointPtr    pts, 
			     int	 numpts);
    virtual void DrawSegments( AmWindowClass 	*w, 
			       AmGCClass 		*gc, 
			       AmSegmentRec	*segments,
			       int 		numpts);
    virtual void DrawArc( AmWindowClass 	*win, 
			  AmGCClass 	*gc, 
			  int		x, 
			  int		y, 
			  int		w, 
			  int		h, 
			  float		angle1,
			  float		angle2);
    virtual void FillArc( AmWindowClass 	*win, 
			  AmGCClass 	*gc, 
			  int		x, 
			  int		y, 
			  int		w, 
			  int		h, 
			  float		angle1,
			  float		angle2);
    virtual void DrawGlyph( AmWindowClass    *w,
			    AmGlyphClass    *glyph,
			    int        x,
			    int        y);
    virtual void DrawImage( AmWindowClass	*w,
			    AmGCClass	*gc,
			    int		srcX, 
			    int		srcY, 
			    int		dstX, 
			    int		dstY, 
			    int		srcWidth,
			    int		srcHeight,
			    AmImageClass	*image);
    virtual void CopyArea( AmWindowClass  *src,
			   AmWindowClass	*dst,
                           AmGCClass      *gc,
                           int          srcX,
                           int          srcY,
                           int          dstX,
                           int          dstY,
                           int          srcWidth,
                           int          srcHeight);
    virtual void DrawText( AmWindowClass	*w,
			   AmGCClass	*gc,
			   int		x, 
			   int		y,
			   char		*text);
    virtual void ExtentsText( AmGCClass    *gc,
			      char       *text,
			      int        *width,
			      int        *ascentReturn,
			      int        *descentReturn);
    virtual int     TextWidth( AmGCClass              *gc,
                               char                 *text,
                               int                  length = 0);
    virtual int     FontHeight( AmGCClass             *gc,
                                int                 *ascentReturn,
                                int                 *descentReturn);
    virtual void DrawImageText( AmWindowClass	*w,
			        AmGCClass	*gc,
			        int		x, 
			        int		y,
			        char		*text);
    virtual void DrawRotatedText( AmWindowClass	*w,
				  AmGCClass	*gc,
				  int		x, 
				  int		y,
				  float		radianAngle,
				  char		*text);
    virtual void FillPolygon( AmWindowClass	*w,
			      AmGCClass		*gc,
			      AmPointPtr	pts,
			      int		numpts);
    virtual void DrawRectangle( AmWindowClass	*win,
				AmGCClass		*gc,
				int		x, 
				int		y, 
				int		w, 
				int		h);
    virtual void FillRectangle( AmWindowClass	*win,
				AmGCClass		*gc,
				int		x, 
				int		y, 
				int		w, 
				int		h);
    virtual AnvilBoolean AllocColor( AmColorCellPtr pCol);
    virtual AnvilBoolean AllocNamedColor( const char   *szName,
                                     AmColorCellPtr pCol);
    virtual AmWindowClass *CreateBuffer( AmWindowClass *pWindow,
                                       int         cx,
                                       int         cy);
    virtual AmWindowClass *CreateWin( AmWindowClass *pWindow,
                                    int         x,
                                    int         y,
                                    int         cx,
                                    int         cy);
    virtual AmImageClass  *CreateImage( int     iWidth,
                                      int     iHeight,
                                      int     iDepth,
                                      char    *pData);
    virtual AmGlyphClass  *CreateGlyph( AmWindowClass *pWindow,
                                      char    *szFile,
                                      ulong_t lPixel);
    virtual AmRegionClass *CreateRegion();
};

#endif /* _anvil_extents_h_ */
