#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_fndefs_h_
#define _anvil_fndefs_h_

#include <anvil/datatype.h>
#include <anvil/tagstrs.h>
#include <anvil/envdefs.h>
#include <anvil/colorcell.h>

/* lookup from data type to access function */

typedef struct AmEnumFnDef_ {
    AmDataTypeEnum	dataType;
    AmAccessFn		fn;
} AmEnumFnDefRec, *AmEnumFnDefPtr;

typedef struct AmSetEnumFnDef_ {
    AmDataTypeEnum	dataType;
    AmSetFn		fn;
} AmSetEnumFnDefRec, *AmSetEnumFnDefPtr;

typedef struct AmSetIntEnumFnDef {
    AmDataTypeEnum	dataType;
    AmIntSetFn		fn;
} AmSetIntEnumFnDefRec, *AmSetIntEnumFnDefPtr;

/* lookup from data type to integer access function */

typedef struct AmEnumIntFnDef_ {
    AmDataTypeEnum	dataType;
    AmIntAccessFn		fn;
} AmEnumIntFnDefRec, *AmEnumIntFnDefPtr;

/* lookup from data type to size */

typedef struct AmEnumSizeDef_ {
    AmDataTypeEnum	dataType;
    int			size;
} AmEnumSizeDefRec, *AmEnumSizeDefPtr;

/* lookup from string to source enumeration */

typedef struct AmEnumSourceDef_ {
    char		*tag;
    AmSourceTypeEnum	sourceType;
} AmEnumSourceDefRec, *AmEnumSourceDefPtr;

/* lookup from a resource name to a subdirectory from config to read 
   files from */

static AmEnumSizeDefRec dataSizes[] = {
    { AmFLOAT, sizeof( float) },
    { AmINT, sizeof( int) },
    { AmUNSIGNED_INT, sizeof( unsigned int) },
    { AmENUM, sizeof( int) },
    { AmDOUBLE, sizeof( double) },
    { AmCHAR, sizeof( char) },
    { AmUNSIGNED_CHAR, sizeof( unsigned char) },
    { AmSHORT, sizeof( short) },
    { AmUNSIGNED_SHORT, sizeof( unsigned short) },
    { AmLONG, sizeof( long) },
    { AmUNSIGNED_LONG, sizeof( unsigned long) },
    { AmBOOLEAN, sizeof( AnvilBoolean) },
    { AmCOLOR_CELL, sizeof( AmColorCellRec) },
    { AmUNITS, sizeof( int) },
    { AmFUNC_PTR, sizeof( void *) },
    { AmVOID_PTR, sizeof( void *) }
};
 
static AmEnumFnDefRec dataFns[] = {
    { AmFLOAT, AmFloatFn },
    { AmINT, AmIntFn },
    { AmUNSIGNED_INT, AmUintFn },
    { AmENUM, AmIntFn },
    { AmDOUBLE, AmDoubleFn },
    { AmCHAR, AmCharFn },
    { AmUNSIGNED_CHAR, AmUcharFn },
    { AmSHORT, AmShortFn },
    { AmUNSIGNED_SHORT, AmUshortFn },
    { AmLONG, AmIntFn },
    { AmUNSIGNED_LONG, AmIntFn },
    { AmBOOLEAN, AmIntFn }
};
 
static AmSetEnumFnDefRec setDataFns[] = {
    { AmFLOAT, AmSetFloatFn },
    { AmINT, AmSetIntFn },
    { AmUNSIGNED_INT, AmSetUintFn },
    { AmENUM, AmSetIntFn },
    { AmDOUBLE, AmSetDoubleFn },
    { AmCHAR, AmSetCharFn },
    { AmUNSIGNED_CHAR, AmSetUcharFn },
    { AmSHORT, AmSetShortFn },
    { AmUNSIGNED_SHORT, AmSetUshortFn },
    { AmLONG, AmSetIntFn },
    { AmUNSIGNED_LONG, AmSetIntFn },
    { AmBOOLEAN, AmSetIntFn }
};

static AmEnumIntFnDefRec intDataFns[] = {
    { AmINT, AmIntIntFn },
    { AmUNSIGNED_INT, AmIntIntFn },
    { AmCHAR, AmCharIntFn },
    { AmUNSIGNED_CHAR, AmUcharIntFn },
    { AmSHORT, AmShortIntFn },
    { AmUNSIGNED_SHORT, AmUshortIntFn },
    { AmLONG, AmLongIntFn },
    { AmUNSIGNED_LONG, AmUlongIntFn }
};


static AmSetIntEnumFnDefRec setIntDataFns[] = {
    { AmINT, AmSetIntIntFn },
    { AmUNSIGNED_INT, AmSetIntIntFn },
    { AmCHAR, AmSetCharIntFn },
    { AmUNSIGNED_CHAR, AmSetUcharIntFn },
    { AmSHORT, AmSetShortIntFn },
    { AmUNSIGNED_SHORT, AmSetUshortIntFn },
    { AmLONG, AmSetLongIntFn },
    { AmUNSIGNED_LONG, AmSetUlongIntFn }
};

static AmStrdefRec dataNames[] = {
    { "float", AmFLOAT },
    { "int", AmINT },
    { "enum", AmENUM },
    { "double", AmDOUBLE },
    { "char", AmCHAR },
    { "byte", AmUNSIGNED_CHAR },
    { "uchar", AmUNSIGNED_CHAR },
    { "short", AmSHORT },
    { "ushort", AmUNSIGNED_SHORT },
    { "long", AmLONG },
    { "ulong", AmUNSIGNED_LONG },
    { "uint", AmUNSIGNED_INT },
    { "string", AmSTRING },
    { "stringPtr", AmSTRING_PTR },
    { "intPtr", AmINT_PTR },
    { "floatPtr", AmFLOAT_PTR },
    { "voidPtr", AmVOID_PTR },
    { "boolean", AmBOOLEAN },
    { "funcPtr", AmFUNC_PTR },
    { "colorCell", AmCOLOR_CELL },
    { "colorCellPtr", AmCOLOR_CELL_PTR },
    { "file", AmFILE },
    { "object", AmOBJECT },
    { "objectPtr", AmOBJECT_PTR },
    { "units", AmUNITS },
    { "variable", AmVARIABLE },
};

#endif /* _anvil_fndefs_h_ */
