#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_fns_h_
#define _anvil_fns_h_
#include <anvil/project.h>
#include <anvil/datatype.h>
#include <anvil/units.h>

typedef float ( *AmAccessFn)( void *, void *);
typedef void ( *AmSetFn)( void *, void *, float);
typedef void ( *AmIntSetFn)( void *, void *, int);
typedef int (*AmIntAccessFn)( void *, void *);
typedef void ( *AmBivarFn)( void *, void *, float *, float *);
typedef int ( *AmIndexFn)( void *, void *);
typedef unsigned long ( *AmColorFn)( void *, void *);
typedef char *( *AmStringFn)( void *, void *);
typedef void ( *AmSetABFn)( void *, void *, float, float);

/* for reading scanlines from files */

typedef AnvilBoolean (*AmScanlineFn)( int      fd,
                               uchar_t  *buffer,
                               int      size,
                               int      length,
                               int      cols,
                               int      x,
                               int      y,
                               void     *clientData);


/* integer accessing functions */

int AmLongIntFn( void *x, void *);
int AmUlongIntFn( void *x, void *);
int AmIntIntFn( void *x, void *);
int AmUintIntFn( void *x, void *);
int AmShortIntFn( void *x, void *);
int AmUshortIntFn( void *x, void *);
int AmCharIntFn( void *x, void *);
int AmUcharIntFn( void *x, void *);

/* integer setting functions */

void AmSetLongIntFn( void *data, void *, int src);
void AmSetUlongIntFn( void *data, void *, int src);
void AmSetIntIntFn( void *data, void *, int src);
void AmSetUintIntFn( void *data, void *, int src);
void AmSetShortIntFn( void *data, void *, int src);
void AmSetUshortIntFn( void *data, void *, int src);
void AmSetCharIntFn( void *data, void *, int src);
void AmSetUcharIntFn( void *data, void *, int src);

/* float accessing functions */

float AmFloatFn( void *x, void *);
float AmIntFn( void *x, void *);
float AmUintFn( void *x, void *);
float AmDoubleFn( void *x, void *);
float AmCharFn( void *x, void *);
float AmUcharFn( void *x, void *);
float AmShortFn( void *x, void *);
float AmUshortFn( void *x, void *);

/* float setting functions */

void AmSetFloatFn( void *, void *, float);
void AmSetIntFn( void *, void *, float);
void AmSetUintFn( void *, void *, float);
void AmSetDoubleFn( void *, void *, float);
void AmSetCharFn( void *, void *, float);
void AmSetUcharFn( void *, void *, float);
void AmSetShortFn( void *, void *, float);
void AmSetUshortFn( void *, void *, float);

int AmLinearIndexFn( void *, void *);
int AmNonlinearIndexFn( void *, void *);
int AmTableLookupFn( void *, void *);

/* integer table lookup functions */

int AmLookupCharFn( void *, void *);
int AmLookupShortFn( void *, void *);
int lookupIntFn( void *, void *);
int AmLookupVal( int    ival,
	       int    *table,
	       int    length);

/* convert data type to access function, set function, string to data type,
   data type to size, and data type to table lookup function */

AmAccessFn AmDataTypeEnumToAccessFn( AmDataTypeEnum   dataType);
AmIntAccessFn AmDataTypeEnumToIntAccessFn( AmDataTypeEnum	dataType);
AmSetFn AmDataTypeEnumToSetFn( AmDataTypeEnum	dataType);
AmIntSetFn AmDataTypeEnumToIntSetFn( AmDataTypeEnum dataType);
AmDataTypeEnum AmStrToDataTypeEnum( const char *str);
char *AmDataTypeEnumToStr( AmDataTypeEnum	dataType);
int AmDataTypeEnumToSize( AmDataTypeEnum    dataType);
AmIndexFn dataTypeEnumToLookupFn( AmDataTypeEnum	dataType);

/* defines how values are translated into indices for color, string, linewidth,
   and other tables.  */
 
typedef enum indexType_ {
        AmINDEX_ERROR = 0,
        AmINDEX_LOOKUP = 1,           /* table lookup into an integer vector */
        AmINDEX_NONLINEAR = 2,        /* non-linear lookup to a float vector */
        AmINDEX_LINEAR = 3            /* linear min-max scaling */
} AmIndexTypeEnum;

/* this has to be a structure instead of a union, because the fields have to
   have different addresses so they can be tracked by AmChanged().
   The functions for getIndexFn() are AmLinearIndexFn(), nonLinearIndexFn(),
   lookupCharIndexFn(), lookupShortIndexFn(), lookupIntIndexFn() */
 
typedef struct AmIndexData_ {
    char		*name;		      /* name of the index data */
    AmIndexTypeEnum     indexType;            /* what type of indexing? */
    float               min;                  /* min, max for AmINDEX_LINEAR */
    float               max;
    AnvilBoolean		minFixed;	      /* min value is fixed */
    AnvilBoolean		maxFixed;	      /* max value is fixed. */
    AnvilBoolean		minClip;	      /* don't display < min value */
    AnvilBoolean		maxClip;	      /* don't display > max value */
    float               scale;                /* scale from min-max -> length */
    AmIntAccessFn	AmIntFn;	      /* fn for int value */
    AmAccessFn          fn;                   /* fn for float value */
    void                *clientData;          /* client data for fn */
    AmItemFn		clientDataFreeFn;     /* free client data on dstrctr */
    int                 length;               /* length of binvec or table */
    float               *binvec;              /* AmINDEX_NONLINEAR bin vector */
    int                 *table;               /* AmINDEX_LOOKUP table */
    int                 offset;               /* fn data pointer offset */
    AmDataTypeEnum      dataType;             /* AmINT, AmCHAR, AmSHORT */
    AmIndexFn           getIndexFn;           /* AmGetIndex function */
    int			var;
    int			units;
    char		**strings;	      /* strings for index */
} AmIndexDataRec, *AmIndexDataPtr;

#endif /* _anvil_fns_h_ */
