#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_handlers_h_
#define _anvil_handlers_h_

extern const int MIN_SIZE;

typedef enum {
    AmACTION_NONE = 100,
    AmACTION_RESIZE_UP_LEFT = 1,
    AmACTION_RESIZE_UP = 2,
    AmACTION_RESIZE_UP_RIGHT = 3,
    AmACTION_RESIZE_LEFT = 4,
    AmACTION_RESIZE_RIGHT = 5,
    AmACTION_RESIZE_DOWN_LEFT = 6,
    AmACTION_RESIZE_DOWN = 7,
    AmACTION_RESIZE_DOWN_RIGHT = 8,
    AmACTION_MOVE = 9,
    AmACTION_DELETE = 10,
    AmACTION_RAISE = 11,
    AmACTION_LOWER = 12,
    AmACTION_PICK = 13,
    AmACTION_MOVE_BELOW = 14,
    AmACTION_MOVE_ABOVE = 15,
} AmActionEnum;

extern const int AmACTION_MASTER_PRIORITY;
extern const int AmACTION_SECONDARY_PRIORITY;
extern const int AmACTION_RESIZE_PRIORITY;
extern const int AmACTION_MOVE_PRIORITY;
extern const int AmACTION_ADD_PRIORITY;
extern const int AmACTION_PICK_PRIORITY;

typedef AnvilBoolean ( *AmHandlerFn)( AmScreenObjClass *screenObj,
                               void           *clientData);

typedef struct AmMasterHandlerArgs_ { 
    AmMotifDisplayClass *motifDisplay;
    AmListPtr           activeObjects;
    AmHandlerFn         doubleClickFn;
} AmMasterHandlerArgsRec, *AmMasterHandlerArgsPtr;

typedef struct AmResizeHandlerArgs_ {
    AmMotifDisplayClass *motifDisplay;
    AmListPtr           screenObjs;
    AmActionEnum        actionType;
    int               startX;
    int               startY;
    int               startWidth;
    int               startHeight;
    int               x;
    int               y;
    int               width;
    int               height;
    int               xOffset;
    int               yOffset;
} AmResizeHandlerArgsRec, *AmResizeHandlerArgsPtr;

typedef struct AmMoveHandlerArgs_ {
    AmMotifDisplayClass *motifDisplay;
    AmListPtr           screenObjs;
    int               startX;
    int               startY;
    int               x;
    int               y;
    int               width;
    int               height;
    int               xOffset;
    int               yOffset;
} AmMoveHandlerArgsRec, *AmMoveHandlerArgsPtr;

typedef struct AmPickHandlerArgs_ {
    AmMotifDisplayClass *motifDisplay;
    int               x;
    int               y;
    int               xOffset;
    int               yOffset;
    AmHandlerFn         objectFunction;
    void              *clientData;
    AmObjectTypeEnum    objectType;
    AnvilBoolean           traverseOverlay;
    char              *successMessage;
    char              *cancelMessage;
    char              *failureMessage;
} AmPickHandlerArgsRec, *AmPickHandlerArgsPtr;

typedef struct AmAddHandlerArgs_ {
    AmMotifDisplayClass *motifDisplay;
    AmScreenObjClass    *screenObj;
    int               x;
    int               y;
    int               width;
    int               height;
    int               xOffset;
    int               yOffset;
    AmListPtr           *activeObjects;
} AmAddHandlerArgsRec, *AmAddHandlerArgsPtr;


typedef struct AmAddObjectsHandlerArgs_ {
    AmMotifDisplayClass *motifDisplay;
    AmListPtr           *screenObjs;
    int               startX;
    int               startY;
    int               x;
    int               y;
    int               width;
    int               height;
    int               xOffset;
    int               yOffset;
} AmAddObjectsHandlerArgsRec, *AmAddObjectsHandlerArgsPtr;


typedef struct AmSecondaryHandlerArgs_ {
    AmMotifDisplayClass *motifDisplay;
    AmScreenObjClass    *screenObj;
    AmListPtr           screenObjs;
    AmActionEnum        actionType;
    int               x;
    int               y;
    XtIntervalId      timerId;
    AmHandlerFn         doubleClickFn;
} AmSecondaryHandlerArgsRec, *AmSecondaryHandlerArgsPtr;

int actionToCursor( AmActionEnum actionType);

void findAction( AmMotifDisplayClass *motifDisplay,
                 int               x,
                 int               y,
                 AmActionEnum        *actionType,
                 AmScreenObjClass    **screenObj);

AmMasterHandlerArgsPtr makeMasterHandlerArgs( AmMotifDisplayClass *motifDisplay,
                                            AmHandlerFn         doubleClickFn =
                                                                  NULL);

AmResizeHandlerArgsPtr AmMakeResizeHandlerArgs( AmMotifDisplayClass *motifDisplay,
                                            AmListPtr           screenObjs,
                                            AmActionEnum        actionType,
                                            int               x,
                                            int               y,
                                            int               width,
                                            int               height,
                                            int               xOffset,
                                            int               yOffset);

AmMoveHandlerArgsPtr AmMakeMoveHandlerArgs( AmMotifDisplayClass *motifDisplay,
                                        AmListPtr           screenObjs,
                                        int               x,
                                        int               y,
                                        int               width,
                                        int               height,
                                        int               xOffset,
                                        int               yOffset);

AmPickHandlerArgsPtr makePickHandlerArgs( AmMotifDisplayClass *motifDisplay,
                                        AmHandlerFn         objectFunction,
                                        void              *clientData,
                                        AmObjectTypeEnum    objectType,
                                        AnvilBoolean           traverseOverlay =
                                                              FALSE,
                                        char              *successMessage =
                                                              NULL,
                                        char              *cancelMessage =
                                                              NULL,
                                        char              *failureMessage =
                                                              NULL);

AmAddHandlerArgsPtr AmMakeAddHandlerArgs( AmMotifDisplayClass *motifDisplay,
                                      AmScreenObjClass    *screenObj,
                                      int               x,
                                      int               y,
                                      int               width,
                                      int               height,
                                      int               xOffset,
                                      int               yOffset,
                                      AmListPtr           *activeObjects = NULL);

AmAddObjectsHandlerArgsPtr
AmMakeAddObjectsHandlerArgs( AmMotifDisplayClass *motifDisplay,
                           AmListPtr           *objects,
                           int               startX,
                           int               startY,
                           int               x,
                           int               y,
                           int               width,
                           int               height,
                           int               xOffset,
                           int               yOffset);

AmSecondaryHandlerArgsPtr AmMakeSecondaryHandlerArgs( AmMotifDisplayClass *motifDisplay,
                                                  AmScreenObjClass    *screenObj,
                                                  AmListPtr           screenObjs,
                                                  AmActionEnum        action,
                                                  int               x,
                                                  int               y,
                                                  AmHandlerFn         dClickFn =
                                                                        NULL);

void AmAddHandler( Widget            widget,
                 AmAddHandlerArgsPtr handlerArgs,
                 XEvent            *event,
                 AnvilBoolean           *);

void AmAddObjectsHandler( Widget                   widget,
                        AmAddObjectsHandlerArgsPtr handlerArgs,
                        XEvent                   *event,
                        AnvilBoolean                  *);

void pickHandler( Widget             widget,
                  AmPickHandlerArgsPtr handlerArgs,
                  XEvent             *event,
                  AnvilBoolean            *);

void AmMoveHandler( Widget             widget,
                  AmMoveHandlerArgsPtr handlerArgs,
                  XEvent             *event,
                  AnvilBoolean            *);

void AmResizeHandler( Widget               widget,
                    AmResizeHandlerArgsPtr handlerArgs,
                    XEvent               *event,
                    AnvilBoolean              *);

void AmSecondaryHandler( Widget                  widget,
                       AmSecondaryHandlerArgsPtr handlerArgs,
                       XEvent                  *event,
                       AnvilBoolean                 *);

void AmMasterHandler( Widget               widget,
                    AmMasterHandlerArgsPtr handlerArgs,
                    XEvent               *event,
                    AnvilBoolean              *);


void findExtents( AmListPtr      screenObjs,
                  AmRectanglePtr extentsRect);

AmListPtr generateAddOutlines( AmListPtr screenObjs,
                             int     startX,
                             int     startY,
                             int     x,
                             int     y);

#endif /* _anvil_handerls_h_ */
