#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_image_h_
#define _anvil_image_h_

#include <sys/types.h>
#include <anvil/display.h>
#include <anvil/rect.h>
#include <anvil/errcode.h>
#include <anvil/fns.h>

AnvilBoolean AmReadScanline( int       fd,
                      uchar_t   *buffer,
                      int       size,
                      int       length,
                      int       cols,
                      int       x,
                      int       y,
                      void      *);

AmErrorCode AmMatrixReadRows( int                   fd,
                              int                   srcRows,
                              int                   srcCols,
                              AmRealRectRec const   *srcRect,
                              uchar_t               *matrix,
			      int		    dstRows,
                              int                   dstCols,
                              AmRectangleRec const  *dstRect,
                              int                   size,
                              AnvilBoolean		    flipRows,
                              AnvilBoolean		    flipCols,
                              AmScanlineFn          readScanFn,
                              void                  *readScanClientData);

AmErrorCode AmReadSubmatrix( int                  fd,
			     int		  srcRows,
                             int                  srcCols,
                             AmRectangleRec const *srcRect,
                             uchar_t              *matrix,
                             int		  matRows,
			     int		  matCols,
			     int		  dstX,
			     int		  dstY,
                             int                  size,
                             AmScanlineFn         readScanFn,
                             void                 *readScanClientData);

AmErrorCode AmCopySubmatrix( uchar_t               *srcMatrix,
			     int		   srcRows,
			     int                   srcCols,
			     AmRectangleRec const  *srcRect,
			     uchar_t               *dstMatrix,
			     int		   dstRows,
			     int                   dstCols,
			     int                   dstX,
			     int                   dstY,
			     int                   size,
			     AnvilBoolean		   flipRows = FALSE,
			     AnvilBoolean		   flipCols = FALSE);

AmErrorCode AmResizeMatrix( uchar_t               *srcMatrix,
                          int                   srcRows,
                          int                   srcCols,
                          uchar_t               *dstMatrix,
                          int                   dstRows,
                          int                   dstCols,
                          int                   size,
                          AmRectangleRec const    *srcRect,
                          AmRectangleRec const    *dstRect);

AmErrorCode
AmImage3D( AmImageClass             *image,                 /* display image */
         float                  ex,                     /* eye position */
         float                  ey,
         float                  ez,
         float                  focal,                  /* screen focal length */
         float                  theta,                  /* horizontal rotation */
         float                  phi,                    /* vertical rotation */
         float                  screenWidth,            /* screen dimensions */
         float                  screenHeight,
         uchar_t                *matrix,                /* elevation matrix */
         int                    rows,                   /* dimensions and */
         int                    cols,                   /* element size */
         int                    size,
         AmAccessFn               zFn,                    /* accessing function and */
         void                   *clientData,            /* client data */
         float                  xmin,                   /* extents of image */
         float                  ymin,                   /* and matrix */
         float                  xmax,
         float                  ymax,
         AmImageClass             *texImage);              /* textured image */

void AmRot2D( float    *x,
	    float    *y,
	    float    theta);

void AmTrans2D( float  *x,
	      float  *y,
	      float  xtrans,
	      float  ytrans);

#endif /* _anvil_image_h_ */
