#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_imageplot_h_
#define _anvil_imageplot_h_
#include <anvil/plot.h>
#include <anvil/overlay.h>
#include <anvil/image.h>

class AmImagePlotClass : public AmPlotClass {
protected:

    AmImageClass	*_pImage;		/* Displayed image.             */
    float	_dTransparency;		/* Transparency level.          */
    AnvilBoolean	_fKeepData;		/* Keep data around.            */

    AnvilBoolean	_fTrueColor;		/* True color 24 bit flag.      */

    AnvilBoolean	_fLightShaded;		/* Light shaded flag.           */
    float	_dLightHeight;		/* Height and angle for light   */
    float	_dLightAngle;		/*   shaded images.             */

    AnvilBoolean	_fTileDetail;		/* Use higher detail for tiles. */

    AmColorFn	_fnPixel;
    void	*_pPixelClientData;

    AnvilBoolean	_fInterpolate;	        /* Bilinear interpolate images. */

    AmScanlineFn	_fnScanline;		/* For image read rows custom   */
    void	*_pScanlineClientData;	/*   scanline functions.        */
    AmItemFn	_scanlineDataFreeFn;	/* free scanline data on delete */

    int		_iSubRows;		/* rows&cols for data when */
    int		_iSubCols;		/* submatrix is loaded */

    AnvilBoolean	_fFlipRows;		/* 0 is on bottom */
    AnvilBoolean	_fFlipCols;		/* 0 on right */

    float	_dSubXmin;		/* coordinates for submatrix */
    float	_dSubXmax;		/* which slightly overlap */
    float	_dSubYmin;		/* coords.curr... */
    float	_dSubYmax;

    AnvilBoolean	_fScreenRes;		/* submatrix at screen res */

    AnvilBoolean	getZptr( float	dataX,
			 float	dataY,
			 void	**ptr);
    void 	setPixelFnClientData( uchar_t	*data,
				      int	rows,
				      int	cols,
				      int	size);
    AmErrorCode 	UpdateMemorySource();
    AmErrorCode		UpdateReadSource();
    AmErrorCode		UpdateTileSource();
    AmErrorCode		UpdateTileSourceKeepData();
    AmErrorCode		UpdateTileSourceNoKeepData();
    void    		GetFieldDefinitions( AmFieldDefPtr       *fieldDefs,
					     int               *numDefs);
    virtual AnvilBoolean	checkValues();
public:
    virtual int 	setFields( AmTagValuePtr	tagValues,
				   int		numValues);
    virtual int     	getFields( AmTagValuePtr	tagValues,
				   int		numValues);
    virtual void 	updateFields();
    virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
    virtual int     	dumpFields( AmFieldLengthPtr      fieldLengths,
				    AnvilBoolean		allFields = FALSE);
			AmImagePlotClass();
    void		initializeFields( AmWindowClass	*windowArg,
					  const char	*nameArg);
			AmImagePlotClass( AmWindowClass  *windowArg,
					const char   *nameArg,
					...);
		    	AmImagePlotClass( AmOverlayClass *overlayArg, 
					const char   *nameArg,
					...);
    virtual 		~AmImagePlotClass();
    virtual AmErrorCode draw( AmWindowClass  	 *window,
			      AmGCClass	     	 *gc,
			      AmRectangleRec const *exposeRects,
			      int		 nRects);
    virtual AmErrorCode Update( int	*eventCodes = NULL,
				int	numCodes = 0);
    AnvilBoolean		getZvalue( float	dataX,
				   float	dataY,
				   float	*dataZ);
    AnvilBoolean		getZstring( float	dataX,
				    float	dataY,
				    char	*buffer);
    AmImageClass		*GetImage();
    void		SetImage( AmImageClass *);
    AnvilBoolean		GetRowcol( float	x,
				   float	y,
				   int		*row,
				   int		*col);
};

#endif /* _anvil_imageplot_h_ */
