#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_legend_h_
#define _anvil_legend_h_
#include <anvil/scale.h>

typedef enum {
    AmLEGEND_ERROR = 0,
    AmLEGEND_CONTINUOUS = 1,
    AmLEGEND_DISTINCT = 2
} AmLegendTypeEnum;

typedef enum {
    AmALIGN_ERROR = 0,
    AmALIGN_TOP = 1,
    AmALIGN_BOTTOM = 2,
    AmALIGN_LEFT = 3,
    AmALIGN_RIGHT = 4,
    AmALIGN_CENTER = 5
} AmAlignEnum;

extern const int            AmLEGEND_WIDTH;
extern const int            AmLEGEND_HEIGHT;
extern const int            AmLEGEND_NUM_ITEMS;
extern const int            AmLEGEND_ITEM_WIDTH;
extern const int            AmLEGEND_ITEM_HEIGHT;
extern const int            AmLEGEND_ITEM_SPACING;
extern const int            AmLEGEND_MAIN_ITEM_HEIGHT;
extern const AmLegendTypeEnum AmLEGEND_TYPE;

class AmLegendClass : public AmScaleClass {
protected:
    AmLegendTypeEnum    legendType;     /* Continuous or distinct. */
    AmIndexDataRec      colorIndex;
    int                 numItems;       /* Number of items in legend.
                                           In the continuous legend type
                                           this indicates how many labels.
                                           In the spaced legend type this
                                           indicates how many boxes. */
    AnvilBoolean             minClip;
    AnvilBoolean             maxClip;

    float               _dXScale;
    float               _dYScale;

    int                 _iOriginalWidth;
    int                 _iOriginalHeight;

    int		        itemWidth;	/* Item width */
    int		        itemHeight;	/* Item height */
    int		        itemSpacing;	/* Item spacing */
    int                 mainItemHeight;
    int                 labelsPerItem;  /* Number of labels for the main
                                           item. */

    AnvilBoolean             increaseItems;  /* Items increase from bottom to
                                           to top and left to right (or
                                           vice versa). */
    int                 numLabels;
    char                **labels;

    int                 numRows;        /* How many rows there should be.
                                           0, means one column. */
    AmRectangleRec      titleRect;
    AmRectanglePtr      itemRects;
    AmRectanglePtr      labelRects;
    AnvilBoolean             vertical;	/* Whether or not the legend should be
                                           vertical or horizontal. */
    AnvilBoolean             editable;	/* Use the plot for initial params,
                                           then edit. */
    AnvilBoolean             outline;

    void                initializeFields( AmWindowClass      *window,
					  AmDataGraphicClass *graphicArg, 
					  const char	   *nameArg);
    void                GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
					     int		*numDefs);
    virtual AmErrorCode layoutRects();

public:
    AmLegendClass();
    virtual ~AmLegendClass();

    virtual int         getFields( AmTagValuePtr	tagValues,
				   int		numValues);
    virtual int		setFields( AmTagValuePtr	tagValues,
				   int		numValues);
    virtual int 	dumpFields( AmFieldLengthPtr	fieldLengths,
				    AnvilBoolean		allFields = FALSE);
    virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
    virtual AmErrorCode Update( int	*eventCodes = NULL,
				int	numCodes = 0);
    virtual AmErrorCode	DrawItem( AmWindowClass		*w,
			           AmGCClass 	        *gc,
                                   int                  ,
			           AmRectangleRec const	*itemRect);
    virtual AmErrorCode	DrawItems( AmWindowClass		*w,
			           AmGCClass 	        *gc,
			           AmRectangleRec const	*itemRects);
    virtual AmErrorCode	draw( AmWindowClass		*w,
			      AmGCClass			*gc,
			      AmRectangleRec const	*exposeRects,
			      int			nRects);
    virtual void	SetRect( int	x,
				 int	y,
				 int	width,
				 int	height);

    AmErrorCode         CenterAreas( int x, int y, int cx, int cy);
};

char **AmIndexesToStrings( AmIndexDataPtr   index,
                         int            numLabels,
                         AmLegendTypeEnum legendType = AmLEGEND_DISTINCT);

char **compactStrings( char     **strings,
                       int      length,
                       int      newLength);

char **getStringsFromBinvec( AmIndexDataPtr index,
                             int          numLabels);

char **getStringsFromLinear( AmIndexDataPtr   index,
                             int            numLabels,
                             AmLegendTypeEnum legendType);

#endif /* anvil_legend_h_ */
