#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_list_h_
#define _anvil_list_h_
#include <stdio.h>
#include <stdarg.h>
#include <anvil/project.h>

typedef struct AmList {
    void 	*data;                     /* data can be anything */
    struct AmList *next;             	/* next linked list element */
} AmListRec, *AmListPtr;

AmListPtr AmDequeueLst(
    void   **data,
    AmListPtr lst            
);

AmListPtr AmAppendLst(
    AmListPtr a,                           /* append two lists */
    AmListPtr b
);

AmListPtr AmCopyLst(
    AmListPtr lst,                         /* copy elements of a list */
    int elSize
);


AmListPtr AmFindElLst(
    void 	*target,                /* find the pointer to a list el. */
    AmListPtr 	lst,
    AmSearchFn 	cmpFn
);

void AmFreeLst(
    AmListPtr lst,
    AmItemFn  freeFn
);

void AmJustFreeLst(                     /* just free the list */
    AmListPtr lst
);

AmListPtr AmInsertNodeAfterLst(           /* insert node after head of list */
    void *newdata,
    AmListPtr after
);

AmListPtr AmInsertNodeAtEndLst(          /* insert node at end of list */
    void *newdata,
    AmListPtr list
);

AmListPtr AmInsertNodeLst(                 /* insert node at head of list */
    void *newdata,
    AmListPtr lst
);

int AmLengthLst(                         /* list length */
    AmListPtr lst
);

void AmMapLst(                           /* apply a function to a list */
    AmListPtr 	lstPtr,
    AmItemFn	elFn
);

AmListPtr AmMergesortLst(                   /* sort a list */
    AmListPtr 	a,
    AmSearchFn 	cmpFn
);

void **AmMakeVectorLst(
    AmListPtr      list
);

void *AmCopyVectorLst(
    AmListPtr	list,
    int		elSize
);

void **AmExtVectorLst( 
    AmListPtr      list,
    void        *( *extFn)( const void *)      /* NOTE:returns void pointer */
);

void *AmNth(
    int         index,
    AmListPtr      lst
);

AmListPtr AmNthLst(
    int         index,
    AmListPtr      lst
);

void *AmLast(
    AmListPtr      lst
);

AmListPtr AmLastLst( AmListPtr lst);

void *AmNextToLast( AmListPtr	lst);


void *AmFirst(
    AmListPtr      lst
);

void *AmFindDataLst( 
    void        *target,
    AmListPtr     lst,
    AmSearchFn	cmpFn
);

int AmIndexDataLst(
    void        *target,
    AmListPtr     lst,
    AmSearchFn	cmpFn
);

int AmEquals( 
    void        *a,
    void        *b
);

AmListPtr AmDelPtrLst( 
    void         *target,
    AmListPtr      lst,
    AmSearchFn	 cmpFn
);

AmListPtr  AmTraceLst(
    AmListPtr      a
);

AmListPtr AmDelElLst(
    void        *target,
    AmListPtr     lst,
    AmSearchFn	cmpFn
);

void *AmLastOfLst(
    AmElementFn	cmpFn,
    AmListPtr	list
);

AmListPtr AmInsertSortLst( void    		*el,
                       AmListPtr  	lst,
		       AmSearchFn	cmpFn);

AmListPtr AmInsertUniqLst( void    		*el,
                       AmListPtr  	lst,
		       AmSearchFn	cmpFn);

AmListPtr AmFreeHead( AmListPtr       lst);

AmListPtr AmDeleteItemLst( void 	*target,
		       AmListPtr 	lst,
		       AmSearchFn cmpFn,
		       AmItemFn	freeFn);

AmListPtr AmMakeUniqueFromSortedLst( AmListPtr	a,
				 AmSearchFn	cmpFn);

int AmCompareListAddrs( AmListPtr	aPtr,
		      AmListPtr	bPtr);

AmListPtr
AmFilterCopyLst( AmListPtr        lst,
                 AmElementFn      filterElFn);
AmListPtr
AmFilterLst( AmListPtr    lst,
             AmElementFn  filterElFn);

AmListPtr
AmVarargLst( void 		*terminator,
			 va_list	args);

AmListPtr 
AmMakeLst( void *a, ...);

#endif /* _anvil_list_h_ */
