#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_magic_h_
#define _anvil_magic_h_

#include <anvil/list.h>
#include <anvil/display.h>
#include <anvil/plot.h>
#include <anvil/imageplot.h>
#include <anvil/plot.h>
#include <anvil/magicmgr.h>
#include <anvil/outline.h>

/* base magic window class defines handlers, source plot, source display, and
   window state information */

class AmMagicClass {
protected:
    AmDisplayClass	*display;
    AmGCClass		*gc;
    AmWindowClass		*window;
    AmMagicManagerClass	*magicManager;		/* handlers source display */
    AmDataGraphicClass	*sourcePlot;		/* to get source info from */
    AmRectangleRec	rect;			/* draw rect in source coords */
    AnvilBoolean		callOnDrag;		/* or call on button release */
    int			resizeTabSize;		/* size of LR resize rect */
public:
    AmMagicClass( AmMagicManagerClass *magicDisplayArg,
		AmDataGraphicClass  *sourcePlotArg,
		AnvilBoolean	       	  callOnDrag = FALSE);
    virtual 		~AmMagicClass();
    			AmMagicClass();
    AmRectangleRec const	*GetRect() { return(( AmRectangleRec const *) &rect); };
    AnvilBoolean		GetCallOnDrag() { return( callOnDrag); };
    AmMagicManagerClass	*GetMagicManager() { return( magicManager); };
    AmDisplayClass	*GetDisplay() { return( display); };	
    AmWindowClass		*GetWindow() { return( window); };
    AmGCClass		*GetGC() { return( gc); };
    AmDataGraphicClass	*GetSourcePlot() { return( sourcePlot); };
    int			GetResizeTabSize() { return( resizeTabSize); };
    void		GetSourceCoords( float	*xmin,
					 float	*ymin,
					 float	*xmax,
					 float	*ymax);
    AnvilBoolean		inResize( int x, int y); 
    virtual AmErrorCode	Update() = 0; 
    virtual void	Draw( AmRectangleRec const *exposeRect) = 0; 
    virtual void	DrawWrapper( AmRectangleRec const *exposeRect) = 0;
    virtual void	SetRect( AmRectangleRec const *rectArg) 
						{ rect = *rectArg; };
    virtual void	SetRectWrapper( AmRectangleRec const *rectArg) = 0;
};

/* the simplest type of magic window, which simply drags a window around that
   the caller has to fill in with data. There is a flag which controls 
   whether the source is drawn in the window before the caller's draw
   routine is called */

class AmMagicWindowClass : public AmMagicClass {
protected:
    AnvilBoolean		copySource;
public:
    AmMagicWindowClass( AmMagicManagerClass  *magicDisplayArg,
		      AmDataGraphicClass   *sourcePlotArg,
		      AnvilBoolean	         callOnDrag = FALSE,
		      AnvilBoolean		 drawSourceFlag = FALSE);
    virtual void	SetRectWrapper( AmRectangleRec const *rect);
    virtual 		~AmMagicWindowClass();
    virtual AmErrorCode	Update();
    virtual void	Draw( AmRectangleRec const *exposeRect);
    virtual void	DrawWrapper( AmRectangleRec const *exposeRect);
};

/* magic plot creates a plot which is generated from the source plot.  
   Once the initialization is completed, the AmDataGraphicClass is supposed to
   take care of itself for dragging and resizing */

class AmMagicPlotClass : public AmMagicWindowClass {
protected:
    AmDataGraphicClass		*plot;		/* plot to render */
public:
    AmMagicPlotClass( AmMagicManagerClass *magicDisplayArg,
		    AmDataGraphicClass  *sourcePlot,
		    AnvilBoolean	      callOnDrag = FALSE);
    AmDataGraphicClass		*GetPlot() { return( plot); };
    virtual void		SetRect( AmRectangleRec const *rect);
    virtual void		SetRectWrapper( AmRectangleRec const *rect);
    virtual 			~AmMagicPlotClass();
    virtual AmErrorCode		Update();
    virtual void		Draw( AmRectangleRec const *exposeRect);
    virtual AmDataGraphicClass	*Init( AmDataGraphicClass *sourcePlot);
};

class AmMagicImageClass : public AmMagicWindowClass {
protected:
    AmImageClass		*image;		/* image to render */
public:
    AmMagicImageClass( AmMagicManagerClass *motifDisplay,
		     AmDataGraphicClass  *sourcePlot,
		     AnvilBoolean	       callOnDrag = FALSE);
    virtual void	SetRectWrapper( AmRectangleRec const *rect);
    virtual 		~AmMagicImageClass();
    virtual AmErrorCode	Update();
    virtual void	Draw( AmRectangleRec const *exposeRect);
    virtual void	DrawWrapper( AmRectangleRec const *exposeRect);
};

/* magic outline is the simplest implementation, where the rectangle being
   dragged around simply defines the source area to work with, which is
   typically displayed in another window */

class AmMagicOutlineClass : public AmMagicClass {
protected:
    AmRectangleRec		lastRect;
    AnvilBoolean			lastDrawn;
    AmWindowOutlineClass          *outline;
public:
    AmMagicOutlineClass( AmMagicManagerClass 	*motifDisplay,
		       AmDataGraphicClass         *sourcePlot,
		       AnvilBoolean	         	callOnDrag = FALSE);
    virtual 		~AmMagicOutlineClass();
    virtual AmErrorCode	Update();
    virtual void	Undraw( AmRectangleRec const *exposeRect);
    virtual void	SetRect( AmRectangleRec const *rect);
    virtual void	SetRectWrapper( AmRectangleRec const *rect);
    virtual void	Draw( AmRectangleRec const *exposeRect);
    virtual void	DrawWrapper( AmRectangleRec const *exposeRect);
    void		drawMagicOutline();
};

#endif /* _anvil_magic_h_ */
