#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_mapmgr_h_
#define _anvil_mapmgr_h_

#include <anvil/dispmgr.h>
#include <anvil/outline.h>

class AmMapManagerClass : public AmDisplayManagerClass {
protected:
    AmGCClass       *_pGCOutline;     /* GC for outlines of objects.         */
    AnvilBoolean       _fOutlinesActive; /* For tracking the outline.           */

    AmListPtr       _lstOutlines;     /* The list of active outline classes. */

    AnvilBoolean       _fGrid;           /* draw grid flag			   */
    int           _iGridSpacing;    /* spacing in "pixels" AmBetween gridpts */
    int           _nGridDivision;   /* number of grid cell divisions 	   */

    void	  drawActiveOutlines( AmDisplayClass	*pDisplayClass,
				      AmWindowClass	*pWindowClass,
				      AmGCClass		*pGC);
public:
/* constructors and destructors */

    AmMapManagerClass();
    ~AmMapManagerClass();
    AmMapManagerClass( AmDisplayClass	*pDisplayClass,
		     AmWindowClass	*pWindowClass,
		     AnvilBoolean		fDoubleBuffer);

/* drawing functions */
 
    virtual void    Draw( AmRectangleRec const *exposeRects = NULL,
                          int                nRects = 0);
    virtual void    ReDraw( AmRectangleRec const *exposeRects = NULL,
                            int                nRects = 0);
    virtual void    ClearArea( AmRectangleRec const *pClearRects = NULL,
                               int                nClearRects = 0);
    void 	    DrawGrid( AmRectangleRec const *pClipRects = NULL,
                              int                nClipRects = 0);
    virtual void    Resize( int cx, 
                            int cy);

/* grid support functions */

    AnvilBoolean         GetGrid();
    int             GetGridSpacing();
    int             GetGridDivision();
    int             GetGridLocation( int iPos);
    void            SetGrid( AnvilBoolean fGrid);
    void            SetGridSpacing( int     iGridSpacing,
                                    AnvilBoolean fUpdate = TRUE);
    void            SetGridDivision( int     nGridDivision,
                                     AnvilBoolean fUpdate = TRUE);

/* outline functions are in outline.C */

    AmErrorCode        AddOutline( AmOutlineClass *outline);
    AmErrorCode        RemoveOutline( AmOutlineClass *outline);

    AmErrorCode        RemoveAllOutlines();

    void               ShowOutlines();
    void               HideOutlines();

    AmErrorCode        ShowOutline( AmOutlineClass *outline);
    AmErrorCode        HideOutline( AmOutlineClass *outline);

/* this function wraps the dispmgr flush function, it is needed   
   due to outlines. */

    virtual void       Flush();

/* message/readout sending functions */

    void               WriteMessage( char *fmt, ...);
    void               WriteResourceMessage( char *res, ...);
    AnvilBoolean            getReadoutString( int  x,
                                         int  y,
                                         char *buffer);

};

#endif /* _anvil_mapmgr_h_ */
