#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_motif_h_
#define _anvil_motif_h_

#include <stdarg.h>
#include <Xm/Xm.h>
#include <Xm/FileSB.h>
#include <anvil/va_cback.h>

/* convenience  macros   */

#define MtString(str)   XmStringCreate(str, XmSTRING_DEFAULT_CHARSET)
#define LeftOf(w)       XmNrightAttachment, XmATTACH_WIDGET, \
                        XmNrightWidget, (( Widget) w)
#define RightOf(w)      XmNleftAttachment, XmATTACH_WIDGET, \
                        XmNleftWidget, (( Widget) w)
#define BelowOf(w)      XmNtopAttachment, XmATTACH_WIDGET, \
                        XmNtopWidget, (( Widget) w)
#define AboveOf(w)      XmNbottomAttachment, XmATTACH_WIDGET, \
                        XmNbottomWidget, (( Widget) w)
#define LeftWith(w)     XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET, \
                        XmNleftWidget, (( Widget) w)
#define RightWith(w)    XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET, \
                        XmNrightWidget, (( Widget) w)
#define TopWith(w)      XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET, \
                        XmNtopWidget, (( Widget) w)
#define AboveWith(w)    TopWith( w)
#define BottomWith(w)   XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET, \
                        XmNbottomWidget, (( Widget) w)
#define BelowWith(w)    BottomWith(w)
#define StickRight      XmNrightAttachment, XmATTACH_FORM
#define StickLeft       XmNleftAttachment, XmATTACH_FORM
#define StickTop        XmNtopAttachment, XmATTACH_FORM
#define StickBottom     XmNbottomAttachment, XmATTACH_FORM
#define NoRight         XmNrightAttachment, XmATTACH_NONE
#define NoLeft          XmNleftAttachment, XmATTACH_NONE
#define NoTop           XmNtopAttachment, XmATTACH_NONE
#define NoBottom        XmNbottomAttachment, XmATTACH_NONE
#define QuickLabel(s)   XmNlabelString, MtString(s)
#define FieldWidth(w)   XmNmaxLength, (( Widget) w), XmNcolumns, (( Widget) w)

int AmMakeArgList( ArgList        args,
                 ...);

int makeVarargList( ArgList     args,
                    va_list     argList);

AnvilBoolean AmGetWidgetChildren( Widget         w,
			   Widget         **children,
			   unsigned int   *numChildren);

Widget AmGetWidgetShell( Widget   a);

Widget AmGetWidgetRealShell( Widget   a);

Widget AmGetTopLevelShell( Widget	w);

Widget AmGetParentOfType( Widget		w,
			WidgetClass	theClass);

Font AmGetWidgetFont( Widget   a);

XmString *AmMotifStringList( char         **strings,
                           int          *numStr);

void AmFreeMotifStringList( XmString      *strings,
                          int           numStr);

void AmUnmapCback( Widget, Widget, XtPointer);
void AmPopdownCback( Widget, Widget, XtPointer);

void AmDestroyShellCallback( Widget, Widget, XtPointer);

void AmFreeStructPtrCallback( Widget,     
                            XtPointer   clientData,
                            XtPointer);

void AmFreeStrvecCallback( Widget,     
		 char	**clientData,
		 XtPointer);

void AmToggleSensitiveCallback( Widget	w,
			      Widget	dst,
			      XtPointer);

void AmToggleEditableCallback( Widget	w,
			     Widget	dst,
			     XtPointer);

void AmToggleOffCallback( Widget  tgl,
                        Widget  otherTgl,
                        XtPointer);

void AmRadioizeToggles( Widget	*toggles,
		      int	nToggles);

void AmSetSensitiveDestroy( Widget        btn,
                          Widget        dlog);

void AmDrawMotifString( Display           *display,
                      Window            win,
                      GC                gc,
                      XmFontList        fontList,
                      int               x,
                      int               y,
                      char              *string);

void AmDrawMultiLineString( Display       *display,
                          Window        win,
                          GC            gc,
                          int           x,
                          int           y,
                          char          *string);

const char *AmGetResourceName( Widget 	w);

void AmRegexpCheck( Widget        w,
		  char          *regexp);

void AmMultipleFileSelectionDialog( Widget        parent,
                                  char          *name,
                                  Widget        *list,
                                  Widget        *okBtn);

void AmMultipleFileSelectionBox( Widget   parent,
                               char     *name,
                               Widget   *list,
                               Widget   *form,
                               Arg      *formArgs,
                               int      nArgs);

void AmManageSubDialogCallback( Widget    button,
                              Widget    dlogShell,
                              XtPointer);

void AmUnmanageSubDialogCallback( Widget  okBtn,
                                Widget  launchBtn,
                                Widget  dlogShell,
                                XtPointer);

void AmPopupSubDialogCallback( Widget    button,
                             Widget    dlogShell,
                             XtPointer);

void AmPopdownSubDialogCallback( Widget  okBtn,
                               Widget  launchBtn,
                               Widget  dlogShell,
                               XtPointer);

void AmSetCmap( Widget    w,
              Colormap  cmap);

void AmSetRaiseCallback( Widget	btn,
		       Widget	dlog);

Widget AmMakeLabel( Widget	parent,
		  char		*str,
		  char		*name = "label");

void AmHelpHandler( Display       *display,
                  XtAppContext  appContext,
                  XEvent        *event);

void AmHelpWindow( int	x,
		 int	y,
		 Window	window,
		 const char	*string);

void AmHelpCback( Widget wButton, XtPointer, XtPointer);

void AmUnsetToggleCallback( Widget	tgl,
			  Widget	otherTgl,
			  XtPointer);

Widget AmMakeFileSelectionBox( Widget           parent,
			     char             *resourceName,
			     Arg	      *args,
			     int	      numArgs,
			     AnvilBoolean	      readFlag = TRUE,
			     VarargsCallback  okCallback = NULL,
			     ...);

Widget AmMakeFileSelectionDialog( Widget           parent,
			        char             *resourceName,
			        Arg	         *args,
			        int	         numArgs,
			        AnvilBoolean	         readFlag = TRUE,
			        VarargsCallback  okCallback = NULL,
			        ...);
void AmSliderText( Widget parent,
                 char   *resName,
                 Widget *form,
                 Widget *text,
                 Widget *scale);

void AmScrollButtonHandler( Widget                areaWidget,
                          XtPointer,
                          XLeaveWindowEvent     *event,
                          AnvilBoolean *);

void AmUpperText( Widget	text);

int AmGetTextFieldInt( Widget     w);

float AmGetTextFieldFloat( Widget w);

void AmSetLabel( Widget		labelWidget,
	       const char	*labelString);

#endif /* _anvil_motif_h_ */
