#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_motifdisplay_h_
#define _anvil_motifdisplay_h_

#include <Xm/Xm.h>
#include <anvil/list.h>
#include <anvil/screenobj.h>
#include <anvil/xdisplay.h>
#include <anvil/magicmgr.h>

#define MAXLINES 2048
#define MAXPOINTS 2048

class AmMagicClass;

class AmMotifDisplayClass : public AmMagicManagerClass {
private:
    static void AmDestroyCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void destroy( Widget    w,
                  XtPointer xpCallData);

    static void AmExposeHandler( Widget       w,
                               XtPointer    xpClientData,
                               XExposeEvent *pxEvent,
                               AnvilBoolean      *pfContinueDispatch);
    void expose( Widget       w,
                 XExposeEvent *pxEvent,
                 AnvilBoolean      *pfContinueDispatch);

    static void AmResizeCallback( Widget    		w,
                                AmMotifDisplayClass	*pMotifDisplay,
                                XtPointer xpCallData);

    static void AmDeleteObjectCallback( int               iEventID,
                                      AmMotifDisplayClass *pMotifDisplay,
                                      AmScreenObjClass    *pScreenObj);

    static void AmUpdateObjectCallback( int               iEventID,
                                      AmMotifDisplayClass *pMotifDisplay,
                                      AmScreenObjClass    *pScreenObj);

protected:
    Widget        _wArea;           /* Drawing area to draw in.            */
    Widget        _wShell;          /* AmGetWidgetShell( _wArea).            */
    Colormap      _xColormap;       /* Colormap (starts from _wArea).      */
    int           _iCursor;
    Dimension	  _iOldWidth;	    /* dimensions before resize */
    Dimension	  _iOldHeight;


public:
    AnvilBoolean        IsValid();

    AmMotifDisplayClass();
    AmMotifDisplayClass( AmXDisplayClass	*pXDisplay,
		       AmXWindowClass	*pXWindow,
		       AnvilBoolean		fDoubleBuffer = FALSE);
    virtual 		~AmMotifDisplayClass();
    Widget          	GetArea() { return( _wArea); };
    Widget          	GetShell() { return( _wShell); };

    virtual void       AmSendUserEvent( int  iPackageID,
                                      int  iEventID,
                                      void *pClientData);

/* Set the mouse pointer. */

    int	               SetCursor( int	iCursor);

/* travel cursor functions are in travel.C */

    void               SetTravelCursor( int     width,
                                        int     height,
                                        int     toggleKey,
                                        AnvilBoolean on = FALSE);
    void               MapTravelCursor();
    void               UnmapTravelCursor();
    void               WriteTravelCursor( char *fmt, ...);
    void               WriteResourceTravelCursor( char *res, ...);

    void               AddMagic( AmMagicClass *magic);
    void               RemoveMagic( AmMagicClass *magic);
    void               SetFollowReadout( int     toggleKey = 'k',
                                         AnvilBoolean on = FALSE);
    virtual void    AddUserCallback( int                packageId,
                                     int                eventId,
                                     callbackProc       proc,
                                     void               *clientData);
    virtual void    RemoveUserCallback( int             packageId,
                                        int             eventId,
                                        callbackProc    proc,
                                        void            *clientData);

/* cursor readout functions are in readout.C */

    void               SetReadout( Widget);
    void               RemoveReadout( Widget);
    void               SetMessage( Widget);
    void               RemoveMessage( Widget);
    void               WriteResourceMessage( char *szFormat, ...);

/* functions for setting and retreiving AmLast width and height for resizing */

    int		GetLastWidth() { return( _iOldWidth); };
    int		GetLastHeight() { return( _iOldHeight); };
    void	SetLastWidth( int oldWidth) { _iOldWidth = oldWidth; };
    void	SetLastHeight( int oldHeight) { _iOldHeight = oldHeight; };
};

void AmUpdateMotifDisplayCallback( Widget            w,
                                 AmMotifDisplayClass *motifDisplay,
                                 XtPointer);

void AmCursorReadoutHandler( Widget               ,
                           AmMotifDisplayClass    *motifDisplay,
                           XMotionEvent         *event,
                           AnvilBoolean              *);

#endif /* _anvil_motifdisplay_h_ */
