#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_motifdisplayP_h_
#define _anvil_motifdisplayP_h_

#include <anvil/list.h>
#include <anvil/screenobj.h>
#include <anvil/xdisplay.h>
#include <anvil/outline.h>

class AmMotifDisplayClass;

/* used within the Flush() routine for tracking the list of objects and 
   clip rectangles which are generated by an update.  These are consolidated
   into draw requests for each affected object */ 

typedef struct AmDrawClip_ {
    AmScreenObjClass	*obj;
    AmRectangleRec	clipRect;
    int			order;
} AmDrawClipRec, *AmDrawClipPtr;
 
/* elements of objList.  Contains object and AmLast rectangle.  When the
   events are flushed, the old rectangle is needed if the object is resized
   to calculate whatever objects need to be cleared */
 
typedef struct AmObjRect_ {
    AmRectangleRec        oldRect;
    AmScreenObjClass      *obj;
} AmObjRectRec, *AmObjRectPtr;


/* elements of the delete list.  Contains the rectangle and order of the
   deleted object */

typedef struct AmDeleteRect_ {
    AmRectangleRec	rect;
    int			order;
} AmDeleteRectRec, *AmDeleteRectPtr;

/* when events are sent from objects to the motif display,
   AmUpdateObjectCallback stuffs one of these in the eventList, which is
   then processed and interpreted by Flush().  The order is the object's
   stacking order, the eventId is the event that was sent. topLevel is
   a flag for whether it's a top level object, or something in an overlay,
   and obj is the object itself. */
 
typedef struct AmEventRect_ {
    int             order;
    int             eventId;
    AnvilBoolean         topLevel;
    AmScreenObjClass  *obj;
} AmEventRectRec, *AmEventRectPtr;


void AmCursorReadoutHandler( Widget               ,
			   AmMotifDisplayClass    *motifDisplay,
			   XMotionEvent         *event,
			   AnvilBoolean              *);

void AmClipByObjectList( AmRectangleRec const	*rects,
		       int			nRects,
                       AmListPtr           	objList,
		       AmScreenObjClass		*lastObject,
                       AmRectanglePtr      	clipRectVec,
                       int               	*nClip,
		       int			maxClip,
		       AnvilBoolean			ignoreTransparent);

AnvilBoolean AmDifferenceRects( const AmRectangleRec     *oldRect,
                         const AmRectangleRec     *newRect,
                         AmRectanglePtr           rectVec,
                         int                    *nRects);

AnvilBoolean AmUnionRects( const AmRectangleRec     *oldRect,
		    const AmRectangleRec     *newRect,
		    AmRectanglePtr           rectVec,
		    int                    *nRects);

#endif /* _anvil_motifdisplayP_h_ */
