#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_newtile_h_
#define _anvil_newtile_h_

#include <anvil/errcode.h>

#define AmTILE_64BIT_SEEK_FLAG	0x1
#define AmTILE_COMPRESSED_FLAG	0x2
#define AmTILE_INTERP_FLAG	0x4
#define AmTILE_MAGIC		0xcafecafe

#if (_MIPS_SZLONG == 64) 
#define SEEK64
#endif

#if defined(SEEK64)
typedef off64_t	seekAddress;
#define LSEEK 	lseek64
#define FSTAT	fstat64
#define STATBUF	struct stat64
#else
typedef ulong_t	seekAddress;
#define LSEEK 	lseek
#define FSTAT	fstat
#define STATBUF	struct stat
#endif

typedef struct AmTile_ {
    float	xmin;		/* tile's coordinate extents */
    float	ymin;
    float	xmax;
    float	ymax;
    ulong_t	ulSeekAddr;		/* upper left tile seek address */
    ulong_t	urSeekAddr;
    ulong_t	llSeekAddr;		/* lower left tile seek address */
    ulong_t	lrSeekAddr;		/* lower right tile seek address */
    ushort_t	rows;			/* tile dimensions */
    ushort_t	cols;
    ulong_t	numBytes;		/* tile data size (compressed) */
    ulong_t	numBytesUncompressed;	/* tile data size (uncompressed) */
#if defined(SEEK64)
    off64_t	seekAddr;	/* tile seek address */
#else
    ulong_t	seekAddrSpace;
    ulong_t	seekAddr;
#endif
    ushort_t	fileIndex;	/* index into file table */
    uchar_t	flags;			/* flags */
} AmTileRec, *AmTilePtr;

typedef struct AmTileHeader_ {
    int		numFiles;
    char	**files;
    int		*fds;
    AmTileRec	rootTile;
} AmTileHeaderRec, *AmTileHeaderPtr;

typedef struct memTile_ memTileRec, *memTilePtr;

struct memTile_ {
    AmTileRec		fileTile;	     /* actual tree tile */
    struct memTile_	*llChild;
    struct memTile_	*ulChild;	     /* child tiles */
    struct memTile_	*lrChild;
    struct memTile_	*urChild;
    seekAddress		currSeekAddr;
    int			tileIndex;
    AnvilBoolean		filled;
};

AnvilBoolean AmLeafTile( AmTilePtr	tile);

AmErrorCode AmReadTileHeader( const char          *tileHeaderFile,
                            AmTileHeaderPtr       *tileHeader,
			    int			*fd);

AmErrorCode AmFindImageTiles( float        xmin,
			    float        ymin,
			    float        xmax,
			    float        ymax,
			    int          rows,
			    int          cols,
			    int          tileFd,
			    AmTilePtr      tile,
			    AmTilePtr      tileVec,
			    int          *nTiles,
			    int          tileLength,
			    int		 detailLevel = 1);

AmErrorCode AmFindLayerTiles( float       xmin,
                            float       ymin,
                            float       xmax,
                            float       ymax,
			    float	detailLevel,
                            int         tileFd,
                            AmTilePtr     tile,
                            AmTilePtr     tileVec,
                            int         *nTiles,
                            int         tileLength);

void AmTileVectorExtents( AmTilePtr  tiles,
		        int      numTiles,
		        int      *rows,
		        int      *cols,
		        float    *xmin,
		        float    *ymin,
		        float    *xmax,
		        float    *ymax);

int AmCompareTileSeekAddr( AmTilePtr	a,
			 AmTilePtr	b);

AmErrorCode AmReadTile( AmTileHeaderPtr tileHeader,
		      AmTilePtr       tile,
		      uchar_t       *buffer);

#endif /* _anvil_newtile_h_ */
