#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_ntdisplay_h_
#define _anvil_ntdisplay_h_

#include <windows.h>
#include <sys/types.h>

#include <anvil/display.h>

#define FONT_NAME_SIZE        128
#define MAX_CLIP_RECTS        1024

/* Enumeration for window types. */

typedef enum AmWindowType_ {
    AmWINDOW_ERROR = 0,
    AmWINDOW_WINDOW = 1,
    AmWINDOW_BUFFER = 2
} AmWindowTypeEnum;

/* Forward declerations. */

class AmNTGCClass;

/* Class declerations. */

class AmNTDisplayClass : public AmDisplayClass {
private:

public:
    AmNTDisplayClass();
    virtual ~AmNTDisplayClass();

    virtual int  GetDepth( void);

    /* Drawing functions. */

    virtual void DrawLines( AmWindowClass *w, 
                            AmGCClass     *gc, 
                            AmPointPtr  pts, 
                            int         numpts);
    virtual void DrawPoints( AmWindowClass *w, 
                             AmGCClass     *gc, 
                             AmPointPtr  pts, 
                             int         numpts);
    virtual void DrawSegments( AmWindowClass *w, 
                               AmGCClass     *gc, 
                               AmSegmentPtr  segments,
                               int         numpts);
    virtual void FillArc( AmWindowClass *win, 
                          AmGCClass     *gc, 
                          int         x, 
                          int         y, 
                          int         w, 
                          int         h, 
                          float       angle1,
                          float       angle2);
    virtual void DrawArc( AmWindowClass *win, 
                          AmGCClass     *gc, 
                          int         x, 
                          int         y, 
                          int         w, 
                          int         h, 
                          float       angle1,
                          float       angle2);
    virtual void DrawImage( AmWindowClass *w,
                            AmGCClass     *gc,
                            int         srcX, 
                            int         srcY, 
                            int         dstX, 
                            int         dstY, 
                            int         srcWidth,
                            int         srcHeight,
                            AmImageClass  *image);
    virtual void DrawGlyph( AmWindowClass *w,
                            AmGlyphClass  *glyph,
                            int         x,
                            int         y);
    virtual void GetImage( AmWindowClass *w,
                           AmImageClass  *image,
                           int         x,
                           int         y);
    virtual void CopyArea( AmWindowClass *src,
                           AmWindowClass *dst,
                           AmGCClass     *gc,
                           int         srcX,
                           int         srcY,
                           int         dstX,
                           int         dstY,
                           int         srcWidth,
                           int         srcHeight);
    virtual void DrawText( AmWindowClass *pWindow,
                           AmGCClass     *pGC,
                           int         x, 
                           int         y,
                           char        *szText);
    virtual void DrawImageText( AmWindowClass *pWindow,
                                AmGCClass     *pGC,
                                int         x, 
                                int         y,
                                char        *szText);
    virtual void DrawRotatedText( AmWindowClass *pWindow,
                                  AmGCClass     *pGC,
                                  int         x, 
                                  int         y,
                                  float       dAngle,
                                  char        *szText);
    virtual void ExtentsText( AmGCClass *pGC,
                              char    *szText,
                              int     *pcx,
                              int     *piAscent,
                              int     *piDescent);
    virtual void FillPolygon( AmWindowClass *pWindow,
                              AmGCClass     *pGC,
                              AmPointPtr    pPoint,
                              int           nPoint);
    virtual void DrawRectangle( AmWindowClass *pWindow,
                                AmGCClass     *pGC,
                                int         x, 
                                int         y, 
                                int         cx, 
                                int         cy);
    virtual void FillRectangle( AmWindowClass *pWindow,
                                AmGCClass     *pGC,
                                int         x, 
                                int         y, 
                                int         cx, 
                                int         cy);
    virtual AnvilBoolean AllocColor( AmColorCellPtr pcol);
    virtual AnvilBoolean AllocNamedColor( const char   *szName,
                                     AmColorCellPtr pcol);

    /* Creation functions. */

    virtual AmWindowClass *CreateBuffer( AmWindowClass *pWindow,
                                       int         cx,
                                       int         cy);
    virtual AmWindowClass *CreateWin( AmWindowClass *pWindow,
                                    int         x,
                                    int         y,
                                    int         cx,
                                    int         cy);

    virtual AmImageClass  *CreateImage( int     cx,
                                      int     cy,
                                      int     iDepth,
                                      char    *pcData);
    virtual AmGlyphClass  *CreateGlyph( AmWindowClass *window,
                                      char        *symfile,
                                      ulong_t     colorArg);
    virtual AmRegionClass *CreateRegion();
    int TextWidth( AmGCClass *pGC,
                   char    *szText,
                   int     iLength);
    int FontHeight( AmGCClass *pGC,
                    int     *piAscent,
                    int     *piDescent);
};


class AmNTWindowClass : public AmWindowClass {

private:
    HWND           _hWindow;
    HBITMAP        _hBitmap;
    HDC            _hDC;
    AmNTGCClass      *_pNTGC;
    AnvilBoolean	   _fPrivate;

    int            _cx;
    int            _cy;

    AmWindowTypeEnum _eWindowType;
    AmColorCellRec   _colBackground;

public:

    /* Constructors/Destructors. */
  
    AmNTWindowClass();
    AmNTWindowClass( AmDisplayClass *pDisplay,
                   HWND         hWnd);

    /* validity test function */

    virtual AnvilBoolean	IsValid(); 
 
    /* Create a buffer. */

    AmNTWindowClass( AmDisplayClass *pDisplay,
                   AmWindowClass  *pWindow,
                   int          cx,
                   int          cy);
    AmNTWindowClass( AmDisplayClass *pDisplay,
                   AmWindowClass  *pWindow,
                   int          cx,
                   int          cy,
		   int		width,
		   int		height);
    virtual ~AmNTWindowClass();

    /* Accessor functions. */

    virtual void    GetDims( int *pcx,
                             int *pcy);
    virtual void    GetWindowGeometry( int *x,
                                       int *y,
                                       int *pcx,
                                       int *pcy);

    /* Mutator functions. */

    virtual void    InvalidateRect( AmRectangleRec const *prec);
    virtual void    ClearRect( AmRectangleRec const *prec);
    virtual void    ResizeWindow( int cx,
                                  int cy);
    virtual void    MoveWindow( int	cx,
    				int	cy);

    /* Creationg functions */

    virtual AmGCClass *CreateGC();

    /* Microsoft Windows NT specific functions. */

    HWND            getHWND();
    HDC             getHDC( AmGCClass *pGC);
    void            releaseHDC( AmGCClass *pGC);
};


class AmNTGCClass : public AmGCClass {

private:
    HPEN           _hPen;
    HBRUSH         _hBrush;
    HRGN           _hRgn;

    TEXTMETRIC     _tmFont;
    HFONT          _hFont;

    AnvilBoolean        _fDirty;

    float          _dLineWidth;
    AmLineJoinEnum   _eLineJoin;
    AmLineStyleEnum  _eLineStyle;
    AmLineCapEnum    _eLineCap;
    AnvilBoolean        _fLineDirty;

    AmColorCellRec   _colForeground;
    AnvilBoolean        _fColForegroundDirty;

    AmColorCellRec   _colBackground;
    AnvilBoolean        _fColBackgroundDirty;

    AmRasterOpEnum   _eRasterOp;
    AnvilBoolean        _fRasterOpDirty;

    char           *_pcDashPattern;
    int            _nDashPattern;
    int            _iDashOffset;
    AnvilBoolean        _fDashPatternDirty;
	AmFillStyleEnum	_eFillStyle;
	char			*_fillPatFile;

    AmRectangleRec   *_pRec;
    int            _nRec;
    AnvilBoolean        _fRecDirty;

    AmFontRec        _font;
    AnvilBoolean        _fFontDirty;
    AmRegionClass  *_clipRegion;
    AnvilBoolean	   _clipRegionDirty;

    void           setPen( HDC hDC);
    void           setRasterOp( HDC hDC);
    void           setClipRegion( HDC hDC);
    void           setFont( HDC hDC);

public:

    /* Constructor/Destructor. */

    AmNTGCClass();
    AmNTGCClass( AmWindowClass *pWindow);
    virtual ~AmNTGCClass();

    /* Accessor functions. */

    virtual void GetForegroundColor( AmColorCellPtr pCol);
    virtual void GetBackgroundColor( AmColorCellPtr pCol);
    virtual void GetFont( AmFontRec *pFont);
    virtual void GetFont( char           **fontname, 
                          int            *size,
                          AmFontWeightEnum *weight = NULL,
                          AmFontSlantEnum  *slant = NULL,
                          float          *xScale = NULL,
                          float          *yScale = NULL);
    virtual float           GetLineWidth();
    virtual AmLineJoinEnum  GetLineJoin();
    virtual AmLineStyleEnum GetLineStyle();
    virtual AmLineCapEnum   GetLineCap();
    virtual void            GetClipRectangles( AmRectanglePtr *pRec,
                                               int          *pnRec);
    virtual AmRasterOpEnum      GetRasterOp();
    virtual AnvilBoolean	        SetFillPattern( const char *fillPatFile);
    virtual void	        SetFillStyle( AmFillStyleEnum fillStyle);
    virtual void          GetDashes( char **ppcDashPattern, 
                                     int  *pnDashPattern,
                                     int  *piDashOffset = NULL);

    /* Mutator functions. */

    virtual void    SetForegroundColor( AmColorCellPtr pCol);
    virtual void    SetBackgroundColor( AmColorCellPtr pCol);
    virtual AnvilBoolean SetFont( AmFontRec *pFont);
    virtual AnvilBoolean SetFont( char           *szFontName, 
                             int            iFontSize,
                             AmFontWeightEnum eFontWeight = AmWEIGHT_MEDIUM,
                             AmFontSlantEnum  eFontSlant = AmSLANT_ROMAN,
                             float          dxFontScale = 1.0,
                             float          dyFontScale = 1.0);
    virtual void    SetLineWidth( float dLineWidth);

    virtual void    SetLineJoin( AmLineJoinEnum eLineJoin);
    virtual void    SetLineStyle( AmLineStyleEnum eLineStyle);
    virtual void    SetLineCap( AmLineCapEnum eLineCap);
    virtual void    SetClipRectangles( AmRectangleRec const *pRec,
                                       int                nRec);
    virtual void    SetClipRegion( AmRegionClass	*region);
    virtual AmRegionClass *GetClipRegion() { return( _clipRegion); };
    virtual void    SetRasterOp( AmRasterOpEnum eRasterOp);
    virtual void    SetDashes( char *pcDashPattern, 
                               int  nDashPattern,
                               int  iDashOffset = 0);

    virtual void    Update();

    /* Microsoft Windows NT specific functions. */

    TEXTMETRIC   *GetNTFont(); 
    HFONT        GetRotatedFont( float dAngle);

    void         setHDC( HDC hDC);    /* Create a device context.            */
    void         cleanHDC( HDC hDC);  /* Clean a device context.             */
    HDC          getHDC();            /* Retrieve the device context.        */
};

class AmNTImageClass : public AmImageClass {

private:
    HBITMAP          _hBitmap;

    int              _nDDColors;       /* Device dependent values.            */
    uchar_t          *_pcDDData;       /* Device dependent data. */

    uchar_t          *_pcData;         /* Internal device independent
                                          representation of the image.        */

    BITMAPINFOHEADER _bmih;
    BITMAPINFO       *_pbmi;

    AnvilBoolean          CopyData8Bit();
    AnvilBoolean          CopyData32Bit();

    HBITMAP          CreateBitmap8Bit( HDC hDC);
    HBITMAP          CreateBitmap32Bit( HDC hDC);

public: 
    AmNTImageClass( AmDisplayClass *pDisplay,
                  int          cx,
                  int          cy,
                  int          iDepth,
                  char         *pcData);
    virtual ~AmNTImageClass();

    virtual AnvilBoolean CopyData();
    virtual AnvilBoolean SetData( char *pcData);
    virtual char    *AmGetData();

    HBITMAP         CreateBitmap( HDC hDC);
    HBITMAP         GetBitmap( HDC hDC);
};

class AmNTGlyphClass : public AmGlyphClass {
private:
public:
    NTGlyphClass( AmDisplayClass *display,
                  AmWindowClass  *window,
                  char         *symfile,
                  ulong_t       color);
    virtual ~AmNTGlyphClass();
    virtual void       DrawGlyph( AmDisplayClass *display,
                                  AmWindowClass  *window,
                                  int          x,
                                  int          y);
    virtual AmGlyphClass *ResizeGlyph( int newWidth,
                                     int newHeight);
};


class AmNTRegionClass : public AmRegionClass {
private:
	HRGN	_hRgn;
protected:
    AmDisplayClass	*_pDisplay;
public:
    			AmNTRegionClass( AmDisplayClass *display);
    			AmNTRegionClass( HRGN hRgn);
    			AmNTRegionClass();
    virtual 	~AmNTRegionClass();
	HRGN		getHRGN() { return( _hRgn); };
    virtual AnvilBoolean 	PointInRegion( int	x,
									   int	y);
    virtual AnvilBoolean 	EqualRegion( AmRegionClass	*otherRegion);
    virtual void	AddPolygon( AmPointPtr		pt,
								int				n,
								AmFillRuleEnum	fillRule);
    virtual void	AddRectangle( AmRectangleRec const *rect);
    virtual void	UnionRegion( AmRegionClass	*otherRegion);
    virtual void	IntersectRegion( AmRegionClass	*otherRegion);
    virtual void	XorRegion( AmRegionClass	*otherRegion);
    virtual void 	SubtractRegion( AmRegionClass	*otherRegion);
    virtual AmRegionClass 	*CopyRegion();
}; 

void DrawBitmap( HDC     hDC,
                 int     xSrc,
                 int     ySrc,
                 int     xDst,
                 int     yDst,
                 int     cx,
                 int     cy,
                 HBITMAP hBitmap);

#endif /* _anvil_ntdisplay_h_ */
