#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_objectplot_h_
#define _anvil_objectplot_h_

#include <anvil/datafmt.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>

/* maximum # of objects in display buffer */

#define MAXLINES	2048	
#define OBJBUFSIZE	16384

typedef void (*AmDrawObjectFn)( AmDisplayClass *,
			      AmWindowClass *,
			      AmGCClass *,
			      int,
			      int,
			      float,
			      void *, 
			      void *);

class AmObjectPlotClass : public AmPlotClass {
protected:
	AmDrawObjectFn	drawFn;		       /* function to draw objs */
	void		*drawFnClientData;     /* draw function client data */
	AmItemFn	drawFnClientDataFreeFn;	/* free on delete */
	AmSetFn		setXFn;		       /* X,Y set functions for */
	AmSetFn		setYFn;		       /* for dragging objects */
	AmSetABFn	setXYFn;	       /* convienience x,y set fn */
	int		maxWidth;
	int		maxHeight;
	float		_dLineWidth;	       /* object line width */
	char		*font;		       /* font information */
	int		fontSize;
	AmFontWeightEnum	fontWeight;
	AmFontSlantEnum	fontSlant;
	float		fontXScale;
	float		fontYScale;
	void		GetFieldDefinitions( AmFieldDefPtr *fieldDefs,
					     int	 *numDefs);
	void		initializeFields( AmWindowClass	*windowArg,
					  const char	*nameArg);
	virtual AnvilBoolean	checkValues();
public:
	virtual int     setFields( AmTagValuePtr  tagValues,
			           int	        numValues);
	virtual int 	getFields( AmTagValuePtr  tagValues,
				   int	        numValues);
    	virtual void    updateFields();
	virtual int	dumpFields( AmFieldLengthPtr	fieldLengths,
				    AnvilBoolean		allFields = FALSE);
	virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
	AmObjectPlotClass();
	AmObjectPlotClass( AmWindowClass  *windowArg,
			 const char   *nameArg,
			 ...);
	AmObjectPlotClass( AmOverlayClass *overlayArg,
			 const char   *nameArg,
			 ...);
	virtual         ~AmObjectPlotClass();
	virtual void 	DrawObject( AmDisplayClass	*display,
				    AmWindowClass		*window,
				    AmGCClass		*gc,
				    uchar_t		*obj,
				    AmRectangleRec const	*boundRect = NULL);
	virtual AmErrorCode draw( AmWindowClass  		*window,
			          AmGCClass		*gc,
			          AmRectangleRec const    *exposeRects,
				  int			nRects);
	virtual AmErrorCode DrawSelection( AmWindowClass  	*window,
					   AmGCClass		*gc,
					   AmRectangleRec const   *exposeRects,
					   int			nRects,
					   AnvilBoolean		fHighlight);
	void GetObjectExtents( void               *object,
			       AmWindowClass        *win,
			       AmGCClass	          *gc,
			       AmRectanglePtr       rect);
	void SetMaxObjectExtents( AmWindowClass	*win,
				  AmGCClass	*gc);
	void GetMaxObjectExtents( int		*maxWidthArg,
				  int		*maxHeightArg);
};

#endif /* _anvil_objectplot_h_ */
