#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_outline_h
#define _anvil_outline_h

#include <anvil/display.h>

extern const int ANCHOR_SIZE;

class AmMapManagerClass;

/*************************************************************************
 * 
 * Base Outline class:
 * 	All cursors displayed in an overlay should be subclassed from the
 * 	outline class, which is managed by the map manager for exposures
 *	and preventing duplicates.  Each Derived class must provide an
 *	== operator to prevent overlaid XOR duplicates, and a draw function
 *	for exposures.
 *
 *************************************************************************/

class AmOutlineClass {
protected:
    AmMapManagerClass   *_pMapManager;
    AnvilBoolean           _fActive;
public:

    /* Constructors. */

    AmOutlineClass();
    AmOutlineClass( AmMapManagerClass *mapManager);

    /* Destructor. */

    virtual ~AmOutlineClass();

    /* Comparison functions. */

    virtual int operator == ( AmOutlineClass &outline); 

    /* Display functions. */

    virtual AmErrorCode Draw( AmDisplayClass *display,
			      AmWindowClass  *window,
			      AmGCClass      *gc);

    /* Accessor functions. */

    AmMapManagerClass   *GetMapManager();
    AnvilBoolean           GetActive();

    /* Setting functions. */

    void SetActive( AnvilBoolean activeFlag); 
};

/* Outline class for rectangular outlines, either for selecting objects
   or dragging outline rectangles */

class AmObjectOutlineClass: public AmOutlineClass {
protected:
    int _x;
    int _y;
    int _iWidth;
    int _iHeight;
public:

    /* Constructors. */

    AmObjectOutlineClass();
    AmObjectOutlineClass( AmMapManagerClass *mapManager);
    AmObjectOutlineClass( AmMapManagerClass *mapManager,
			int             x,
			int             y,
			int             width,
			int             height);

    /* Destructor. */

    virtual ~AmObjectOutlineClass();

    /* Comparison operator. */

    virtual int operator == ( AmObjectOutlineClass &outline);

    /* Display functions. */

    virtual AmErrorCode Draw( AmDisplayClass *display,
			      AmWindowClass  *window,
			      AmGCClass      *gc);

    /* Accessor functions. */

    void GetRectangle( int *x,
		       int *y,
		       int *width,
		       int *height);
    void GetOrigin( int *x,
		    int *y);
    void GetSize( int *width,
		  int *height);

    /* Setting functions. The setRectangle function does the graphics
       update, and SetOrigin() and SetSize() call it */ 

    void SetRectangle( int x,
		       int y,
		       int width,
		       int height);
    void SetOrigin( int x,
		    int y);
    void SetSize( int width,
		  int height);
};

/* outline for dragging a single line, for profilers and pan dragging. */

class AmDragLineOutlineClass : public AmOutlineClass {
protected:
    int _startX;
    int _startY;
    int _currX;
    int _currY;
public:
    AmDragLineOutlineClass( AmMapManagerClass *pMapManager,
			  int             startX,
			  int             startY);
    virtual ~AmDragLineOutlineClass();

    /* Comparison operator. */

    virtual int operator == ( AmDragLineOutlineClass &outline);

    /* Display functions. */

    virtual AmErrorCode Draw( AmDisplayClass *display,
			      AmWindowClass  *window,
			      AmGCClass      *gc);
    void SetStartPos( int x, int y);
    void SetCurrPos( int x, int	y);
    void GetStartPos( int *x, int *y);
    void GetCurrPos( int *x, int *y);
    void SetLine( int	startY,
    		  int	startX,
		  int	currX,
		  int	currY);
};

/* outline for dragging a rectangle with a small resize rectangle in the
   lower right hand corner.  The comparison operator is borrowed from the
   object outline */

class AmWindowOutlineClass : public AmObjectOutlineClass {
protected:
    int _resizeTabSize;
public:
    AmWindowOutlineClass();
    AmWindowOutlineClass( AmMapManagerClass *mapManager);
    AmWindowOutlineClass( AmMapManagerClass *mapManager,
		        int             x,
		        int             y,
		        int             width,
		        int             height,
		        int	        resizeTabSize);

    /* Destructor. */

    virtual ~AmWindowOutlineClass();

    /* Display functions. */

    virtual AmErrorCode Draw( AmDisplayClass *display,
			      AmWindowClass  *window,
			      AmGCClass      *gc);
    AnvilBoolean inResize( int x, int y);
};


#endif /* _anvil_outline_h_ */
