#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_plineplot_h_
#define _anvil_plineplot_h_

#include <anvil/datafmt.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>

/* maximum # of lines in display buffer */

#define MAXLINES     2048
#define PLINEBUFSIZE 1048576

class AmPolyLinePlotClass : public AmPlotClass {

private:
    AmIndexDataRec 	_indexLineWidth;	/* line width fn index data */
    float        	*_pdLineWidth;		/* line width list */
    float        	*_pdLineWidthScale;	/* scaleable line width list */
    float        	_dLineWidth;		/* line width */
    float	 	_dLineWidthLast;
    int          	plineBufSize;		/* buf size (AmREAD_SOURCE) */
    AmColorCellRec	_col;
    AmColorCellRec	_colLast;
    int			_textGridDensity;	/* density to draw text */

    AmErrorCode         drawPline( AmWindowClass  	*window,
				   AmGCClass      	*gc,
				   AmPolylinePtr        pline,
				   AnvilBoolean   		fHighlight);
    AmErrorCode		drawPolylineVector( AmWindowClass	*window, 
					    AmGCClass		*gc,
					    uchar_t		*plineBuffer,
					    int			numPlines,
					    AnvilBoolean		fHighlight,
					    float		xmin,
					    float		ymin,
					    float		xmax,
					    float		ymax);
    AmErrorCode 	drawPolylineVectorText( AmWindowClass      *window,
                                                AmGCClass          *gc,
                                                uchar_t            *plineBuffer,
                                                int                numPlines,
                                                float              xmin,
                                                float              ymin,
                                                float              xmax,
                                                float              ymax,
                                                AnvilBoolean            *densityGrid,
                                                int                densityRows,
                                                int                densityCols);
    virtual AmErrorCode	readData();
protected:
    void         	initializeFields( AmWindowClass	*windowArg,
					  const char	*nameArg);	
    void		GetFieldDefinitions( AmFieldDefPtr *fieldDefs,
				 	     int	 *numDefs);
    virtual AnvilBoolean 	checkValues();
    virtual void        GetDataExtents( float   *xmin,
					float   *ymin,
					float   *xmax,
					float   *ymax);
public:
    virtual int		setFields( AmTagValuePtr  tagValues,
				   int	        numValues);
    virtual int		getFields( AmTagValuePtr  tagValues,
			   	   int	        numValues);
    virtual void 	updateFields();
    virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
    virtual int		dumpFields( AmFieldLengthPtr	fieldLength,
				    AnvilBoolean		allFields = FALSE);
			AmPolyLinePlotClass();
		    	AmPolyLinePlotClass( AmWindowClass 	*windowArg,
					     const char		*nameArg,
					     ...);
		    	AmPolyLinePlotClass( AmOverlayClass  *overlayArg, 
					     const char      *nameArg,
					     ...);
    virtual	    	~AmPolyLinePlotClass();
    virtual AmErrorCode draw( AmWindowClass  	        *window,
			      AmGCClass	       		*gc,
			      AmRectangleRec const   	*exposeRects,
			      int		        nRects);
    virtual void 	ObjectExtents( void      *ptr,
				       float     *xmin,
				       float     *ymin,
				       float     *xmax,
				       float     *ymax);
    virtual AmErrorCode DrawSelection( AmWindowClass		*window,
				       AmGCClass		*gc,
				       AmRectangleRec const	*exposeRects,
				       int			nRects,
				       AnvilBoolean			fHighlight);
};

#endif /* _anvil_plineplot_h_ */
