#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_polygonplot_h_
#define _anvil_polygonplot_h_
#include <anvil/datafmt.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>

#define POLYBUFSIZE	1048576
#define MOSTPOLYPTS	1024

class AmPolygonPlotClass : public AmPlotClass {
private:
    AmIndexDataRec	hatchWidthIndex;
    float		*hatchWidthList;	/* list of hatch widths */
    AmIndexDataRec	hatchAngleIndex;
    float		*hatchAngleList;	/* list of hatch angles */
    AmIndexDataRec	hatchPatternIndex;	
    char		**hatchPatternList;	/* list of fill pattern names */
    float		hatchAngle;		/* angle in degrees */
    float		hatchWidth;		/* width in coord space */
    char		*hatchPattern;		/* hatch pattern name */
    AnvilBoolean		hatched;		/* draw hatches? */
    AnvilBoolean		solid;
    int			mostPolyPts;		/* polygon with most pts */
    int			polyBufSize;		/* buf size (AmREAD_SOURCE) */
    AmGCClass		*hatchGC;		/* for hatch clipping */
    void drawPolygon( AmWindowClass	*window,
		      AmGCClass		*gc,
		      AmPolygonPtr	poly,
		      AmPointPtr		screenPts);
    void scalePoints( AmCoordPtr      points,
		      int           *numpts,
		      AmPointPtr      screenpts);
    AmErrorCode drawPolygonVector( AmDisplayClass   *display,
				   AmWindowClass    *window,
				   AmGCClass        *gc,
				   uchar_t        *polyBuffer,
				   uchar_t	  **endPtr,
				   int            numPolys,
				   AnvilBoolean	  fHighlight);
    virtual AmErrorCode readData();
protected:
    void GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
			      int		*numDefs);
    virtual void GetDataExtents( float   *xmin,
				 float   *ymin,
				 float   *xmax,
				 float   *ymax);
    void 	initializeFields( AmWindowClass	*windowArg,
				  const char 	*nameArg);
    virtual AnvilBoolean	checkValues();
public:
    virtual int setFields( AmTagValuePtr  tagValues,
			   int	        numValues);
    virtual int getFields( AmTagValuePtr  tagValues,
			   int	        numValues);
    virtual void updateFields();
    virtual int dumpFields( AmFieldLengthPtr      fieldLength,
			    AnvilBoolean	    	allFields = FALSE);
    virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
    AmPolygonPlotClass();
    AmPolygonPlotClass( AmWindowClass   *windowArg,
		      const char    *nameArg,
		      ...);
    AmPolygonPlotClass( AmOverlayClass *overlayArg, 
		      const char   *nameArg,
		      ...);
    virtual ~AmPolygonPlotClass();
    virtual AmErrorCode draw( AmWindowClass  		*window,
			      AmGCClass			*gc,
			      AmRectangleRec const    	*exposeRects,
			      int			nRects);
    virtual AmErrorCode DrawSelection( AmWindowClass  	     *window,
				       AmGCClass		     *gc,
				       AmRectangleRec const    *exposeRects,
				       int		     nRects,
				       AnvilBoolean		     fHighlight);
    virtual void ObjectExtents( void      *ptr,
				float     *xmin,
				float     *ymin,
				float     *xmax,
				float     *ymax);
};

#endif /* _anvil_polygonplot_h_ */
