#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
      AmPolyBoolean: a polygon clip library
      Copyright (C) 1997  Alexey Nikitin, Michael Leonov
      leonov@propro.iis.nsk.su

      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
      License as published by the Free Software Foundation; either
      version 2 of the License, or (at your option) any later version.

      This library is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Library General Public License for more details.

      You should have received a copy of the GNU Library General Public
      License along with this library; if not, write to the Free
      Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

      polyarea.h
      (C) 1997 Alexey Nikitin, Michael Leonov
      (C) 1997 Klamer Schutte (minor patches)
*/

#ifndef	_anvil_polyisect_h_
#define	_anvil_polyisect_h_

#include <anvil/project.h>
#include <anvil/vectmatr.h>
#include <anvil/graphstructs.h>


#define PLF_STATUS      0x0003
#define PLF_CYCLINK     0x0004
#define PLF_MARK	0x0008
#define PLF_RESERVED	0x00FF

#define PLF_ORIENT      0x0100
#define PLF_DIR         0x0100
#define PLF_INV         0x0000

typedef struct VNODE VNODE;
struct VNODE
{
    VNODE *next, *prev;
    unsigned int Flags;
    Vector point;
    union
    {
        VNODE *link;
        void *v;
        int i;
    } t;
};

typedef struct PLINE PLINE;
struct PLINE
{
    PLINE *next;
    VNODE head;
    unsigned int Count;
	unsigned int Flags;
    float xmin, ymin, xmax, ymax;
};

PLINE *AmPolyNewContour(Vector v);

void AmPolyInitContour(PLINE *  c);
void AmPolyClearContour(PLINE *  c);  /* clears list of vertices */
void AmPolyDelContour(PLINE ** c);

AnvilBoolean AmPolyCopyContour(PLINE ** dst, PLINE * src);

void AmPolyPreContour(PLINE * c, AnvilBoolean optimize); /* prepare contour */
void AmPolyInvContour(PLINE * c);  /* invert contour */

VNODE *AmPolyCreateNode(Vector v);

void AmPolyInclVertex(VNODE * after, VNODE * node);
void AmPolyExclVertex(VNODE * node);

/**********************************************************************/

typedef struct POLYAREA POLYAREA;
struct POLYAREA
{
    POLYAREA *f, *b;
    PLINE *contours;
};

AnvilBoolean AmPolyM_Copy0(POLYAREA ** dst, const POLYAREA * srcfst);
void AmPolyM_Incl(POLYAREA **list, POLYAREA *a);

AnvilBoolean AmPolyCopy0(POLYAREA **dst, const POLYAREA *src);
AnvilBoolean AmPolyCopy1(POLYAREA  *dst, const POLYAREA *src);

AnvilBoolean AmPolyInclContour(POLYAREA * p, PLINE * c);
AnvilBoolean AmPolyExclContour(POLYAREA * p, PLINE * c);


AnvilBoolean AmPolyChkContour(PLINE * a);

AnvilBoolean AmPolyCheckInside(POLYAREA * c, Vector v0);

/**********************************************************************/

/* tools for clipping */

/* checks whether point lies within contour
independently of its orientation */

int AmPolyInsideContour(PLINE *c, Vector v);
int AmPolyContourInContour(PLINE * poly, PLINE * inner);
POLYAREA *AmPolyCreate(void);

void AmPolyFree(POLYAREA **p);
void AmPolyInit(POLYAREA  *p);
void AmPolyClear(POLYAREA *p);
AnvilBoolean AmPolyValid(POLYAREA *p);

enum PolygonBooleanOperation {
	PBO_UNITE,
	PBO_ISECT,
	PBO_SUB,
	PBO_XOR
};

AmErrorCode AmPolyBoolean( const POLYAREA *a, 
			   const POLYAREA *b, 
			   POLYAREA 	  **res, 
			   int 		  action);

AmErrorCode AmMakePolyArea( AmPointPtr  pts,
                            int         numpts,
                            POLYAREA    **polyArea);

AnvilBoolean AmEqualPolyArea( POLYAREA *polyA,
			 POLYAREA *polyB);

#endif /* _anvil_polyisect_h_ */
