#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_postscript_h_
#define _anvil_postscript_h_

#include <anvil/fonts.h>
#include <anvil/display.h>
#include <anvil/parseAFM.h>
#include <anvil/polyisect.h>

#define FONT_NAME_SIZE 128

/* forward declaration for the GC, which is saved in the window so 
   that when the gc is changed, the drawing attributes are changed
   accordingly */

class AmPostScriptGCClass;
class AmPostScriptClass;

class AmPostScriptOutputClass : public AmWindowClass {

private:
    void PrintHeader();
    void PrintDefaults();
    void PrintProlog();
    void PrintSetup();
    void PrintPageSetup( int iPage);
    void PrintPageTrailer();
    void PrintTrailer();

protected:
    FILE              *_pFile;
    float             _dWidthInInches;
    float             _dHeightInInches;
    AnvilBoolean           _fLandscape;
    char              *_szPrinter;
    int               _iWidthPage;
    int               _iHeightPage;
    char              *_szTitle;
    AmPostScriptGCClass *_pPostScriptGC; /* Save AmLast graphics context. */

public:
    AmPostScriptOutputClass( FILE            *pFile,
                           float           dWidthInInches,
                           float           dHeightInInches,
                           AmPostScriptClass *pPostScript,
                           AnvilBoolean         fLandscape = FALSE,
                           char            *szProlog = NULL);
    virtual ~AmPostScriptOutputClass();

    void ApplyGCAttributes( AmPostScriptGCClass *pPostScriptGC);

    FILE         *getFp();
    virtual AnvilBoolean IsValid() { return( TRUE); };
    virtual void GetDims( int *piWidth, int *piHeight);
    virtual void InvalidateRect( AmRectangleRec const *pRec);
    virtual void ClearRect( AmRectangleRec const *pRec);
    virtual void GetWindowGeometry( int *px,
                                    int *py,
                                    int *piWidth,
                                    int *piHeight);
    virtual void ResizeWindow( int	cx,
			       int	cy);
    virtual void MoveWindow( int	x,
			     int	y);

/* create GC needs a window parameter to support windows NT */

    virtual AmGCClass *CreateGC();
    virtual void Update();
};

class AmPostScriptClass : public AmDisplayClass {

protected:
    int              _iDepth;        /* Depth of output images */
    AmColormapTypeEnum _eColormapType; /* TrueColor or PseudoColor */
    AmColorCellPtr     _pCol ;         /* Colormap if PseudoColor */

public:
    AmPostScriptClass( int              iDepth,
                     AmColormapTypeEnum eColormapEnum,
                     int              iDPI);
    virtual ~AmPostScriptClass();

    virtual int  DimInPixels( int iDimension);

    virtual void DrawLines( AmWindowClass *w, 
                            AmGCClass     *gc, 
                            AmPointPtr    pts, 
                            int           numpts);
    virtual void DrawPoints( AmWindowClass *w, 
                             AmGCClass     *gc, 
                             AmPointPtr    pts, 
                             int           numpts);
    virtual void DrawSegments( AmWindowClass *w,
                               AmGCClass     *gc,
                               AmSegmentRec  *segments,
                               int         nseg);
    virtual void FillArc( AmWindowClass *win, 
                          AmGCClass     *gc, 
                          int         x, 
                          int         y, 
                          int         w, 
                          int         h, 
                          float       angle1,
                          float       angle2);
    virtual void DrawArc( AmWindowClass *win, 
                          AmGCClass     *gc, 
                          int         x, 
                          int         y, 
                          int         w, 
                          int         h, 
                          float       angle1,
                          float       angle2);
    virtual void DrawImage( AmWindowClass *w,
                            AmGCClass     *gc,
                            int         srcX, 
                            int         srcY, 
                            int         dstX, 
                            int         dstY, 
                            int         srcWidth,
                            int         srcHeight,
                            AmImageClass  *image);
    virtual void DrawGlyph( AmWindowClass *w,
                            AmGlyphClass  *glyph,
                            int         x,
                            int         y);
    virtual void GetImage( AmWindowClass *w,
                           AmImageClass  *image,
                           int         x,
                           int         y);
    virtual void CopyArea( AmWindowClass *src,
                           AmWindowClass *dst,
                           AmGCClass     *gc,
                           int         srcX,
                           int         srcY,
                           int         dstX,
                           int         dstY,
                           int         srcWidth,
                           int         srcHeight);
    virtual void DrawText( AmWindowClass *w,
                           AmGCClass     *gc,
                           int         x, 
                           int         y,
                           char        *text);
    virtual void DrawImageText( AmWindowClass *w,
                                AmGCClass     *gc,
                                int         x, 
                                int         y,
                                char        *text);
    virtual void DrawRotatedText( AmWindowClass *w,
                                  AmGCClass     *gc,
                                  int         x, 
                                  int         y,
                                  float       radianAngle,
                                  char        *text);
    virtual void ExtentsText( AmGCClass *gc,
                              char    *text,
                              int     *width,
                              int     *ascentReturn,
                              int     *descentReturn);
    virtual void FillPolygon( AmWindowClass *w,
                              AmGCClass     *gc,
                              AmPointPtr    pts,
                              int           numpts);
    virtual void DrawRectangle( AmWindowClass *win,
                                AmGCClass     *gc,
                                int         x, 
                                int         y, 
                                int         w, 
                                int         h);
    virtual void FillRectangle( AmWindowClass *win,
                                AmGCClass     *gc,
                                int         x, 
                                int         y, 
                                int         w, 
                                int         h);
    virtual AnvilBoolean AllocColor( AmColorCellPtr cell);
    virtual AnvilBoolean AllocNamedColor( const char   *name,
                                     AmColorCellPtr cell);
    virtual AmImageClass *CreateImage( int     width,
                                     int     height,
                                     int     depth,
                                     char    *data);
    virtual AmGlyphClass *CreateGlyph( AmWindowClass *window,
                                     char        *symfile,
                                     ulong_t     color);
    virtual AmWindowClass *CreateBuffer( AmWindowClass *pWindow,
                                       int         cx,
                                       int         cy);
    virtual AmWindowClass *CreateWin( AmWindowClass *pWindow,
                                    int         x,
                                    int         y,
                                    int         cx,
                                    int         cy);
    virtual AmRegionClass *CreateRegion();
    int TextWidth( AmGCClass   *gc,
                   char      *text,
                   int       length);
    int FontHeight( AmGCClass  *gc,
                    int      *ascentReturn,
                    int     *descentReturn);
    virtual int GetDepth();
};

/* each of the fields have dirty flags so when the GC is changed, only the
   corresponding attributes are set, rather than resetting all of them.  */

class AmPostScriptGCClass : public AmGCClass {

private:
    AmColorCellRec     fg;
    AnvilBoolean            fgDirty;
    AmColorCellRec     bg;
    AnvilBoolean            bgDirty;
    float              _dLineWidth;
    AnvilBoolean            _fLineWidthDirty;
    AmLineStyleEnum    lineStyle;
    AnvilBoolean            lineStyleDirty;
    AmLineJoinEnum     lineJoin;
    AnvilBoolean            lineJoinDirty;
    char               fontName[ FONT_NAME_SIZE];    /* save font name */
    char               fontPostName[ FONT_NAME_SIZE];    /* save font name */
    AmFontWeightEnum   fontWeight;
    AmFontSlantEnum    fontSlant;
    int                fontSize;
    float              fontXScale;
    float              fontYScale;
    AnvilBoolean            fontDirty;
    AmRectanglePtr     clipRectVec;    
    int                nrects;
    AnvilBoolean            clipRectDirty;
    AmRegionClass      *clipRegion;
    AnvilBoolean	       clipRegionDirty;
    char               *dashPattern;
    int                numDashes;
    int                dashOffset;
    AnvilBoolean            dashDirty;
    AmRasterOpEnum     rasterOp;
    AnvilBoolean            rasterOpDirty;
    AFMFontInfo        *fontInfo;
    AnvilBoolean	       fillPatDirty;
    void	       *fillPatternRoot;

public:

    AmPostScriptGCClass( AmPostScriptOutputClass	*postScriptOutput);
    virtual ~AmPostScriptGCClass();

    void setFgDirty() { fgDirty = TRUE; };
    void setBgDirty() { bgDirty = TRUE; };
    void setLineWidthDirty() { _fLineWidthDirty = TRUE; };
    void setLineStyleDirty() { lineStyleDirty = TRUE; };
    void setLineJoinDirty() { lineJoinDirty = TRUE; };
    void setFontDirty() { fontDirty = TRUE; }; 
    void setClipRectDirty() { clipRectDirty = TRUE; };
    void setDashDirty() { dashDirty = TRUE; };
    void setRasterOpDirty() { rasterOpDirty = TRUE; };
    void setClipRegionDirty() { clipRegionDirty = TRUE; };
    void setFillPatternDirty() { fillPatDirty = TRUE; };
    virtual AnvilBoolean SetFillPattern( const char *fillPatFile);
    virtual char *GetFillPattern();
    virtual void SetFillStyle( AmFillStyleEnum fillStyle);
    virtual AmFillStyleEnum	GetFillStyle();
    virtual void SetForegroundColor( AmColorCellPtr cell);
    virtual void GetForegroundColor( AmColorCellPtr cell);
    virtual void SetBackgroundColor( AmColorCellPtr cell);
    virtual void GetBackgroundColor( AmColorCellPtr cell);
    virtual AnvilBoolean SetFont( char           *name, 
                             int            size,
                             AmFontWeightEnum weight = AmWEIGHT_MEDIUM,
                             AmFontSlantEnum  slant = AmSLANT_ROMAN,
                             float          xScale = 1.0,
                             float          yScale = 1.0);
    virtual AnvilBoolean SetFont( AmFontRec *pFont);
    virtual void GetFont( char           **fontname, 
                          int            *size,
                          AmFontWeightEnum *weight = NULL,
                          AmFontSlantEnum  *slant = NULL,
                          float          *xScale = NULL,
                          float          *yScale = NULL);
    virtual void GetFont( AmFontRec *pFont);
    virtual void SetLineWidth( float dLineWidth);
    virtual float GetLineWidth();
    virtual void SetLineJoin( AmLineJoinEnum    lineJoin);
    virtual AmLineJoinEnum GetLineJoin();
    virtual void SetLineStyle( AmLineStyleEnum    lineStyle);
    virtual AmLineStyleEnum GetLineStyle();
    virtual void SetLineCap( AmLineCapEnum    lineCap);
    virtual AmLineCapEnum GetLineCap();
    virtual void SetClipRectangles( AmRectangleRec const *clipRectVec,
                                    int                nrects);
    virtual void GetClipRectangles( AmRectanglePtr *clipRectVec,
                                    int          *nrects);
    virtual void SetClipRegion( AmRegionClass *region);
    virtual AmRegionClass *GetClipRegion() { return( clipRegion); };
    virtual void SetRasterOp( AmRasterOpEnum rasterOp);
    virtual AmRasterOpEnum GetRasterOp();
    virtual void SetDashes( char *dashPat, 
                            int  ndash,
                            int  dashOffset = 0);
    virtual void GetDashes( char **dashPat, 
                            int  *ndash,
                            int  *dashOffset = NULL);
    AnvilBoolean GetTextExtents( char *text,
                            int  *width,
                            int  *ascent,
                            int  *descent);
    void OutputDirtyGCchanges( AmPostScriptOutputClass *pw);
    virtual void Update();
};

class AmPostScriptImageClass : public AmImageClass {

private:
    char *_pcInternal;

public: 
    AmPostScriptImageClass( AmDisplayClass *pDisplay,
                          int          cx,
                          int          cy,
                          int          iDepth,
                          char         *pcData);        /*  may be NULL */

    virtual ~AmPostScriptImageClass();
    virtual AnvilBoolean SetData( char *data);
    virtual char *AmGetData();
};

class AmPostScriptGlyphClass : public AmGlyphClass {

public:
    AmPostScriptGlyphClass( AmDisplayClass  *display,
                          AmWindowClass   *window,
                          char          *symfile,
                          ulong_t       color);
    virtual ~AmPostScriptGlyphClass();

    virtual void DrawGlyph( AmDisplayClass *display,
                            AmWindowClass  *window,
                            int          x,
                            int          y);
    virtual AmGlyphClass  *ResizeGlyph( int newWidth,
                                      int newHeight);
};

class AmPostScriptRegionClass {
private:
    POLYAREA		*polyRegion;
public:
    POLYAREA		*getPolyRegion() { return( polyRegion); };
                        AmPostScriptRegionClass();
                        AmPostScriptRegionClass( POLYAREA *newPolyRegion);
    virtual             ~AmPostScriptRegionClass();
    virtual AnvilBoolean     PointInRegion( int  x,
                                       int  y);
    virtual AnvilBoolean     EqualRegion( AmRegionClass    *otherRegion);
    virtual void        AddPolygon( AmPointPtr        pt,
                                    int             n,
                                    AmFillRuleEnum    fillRule);
    virtual void        AddRectangle( AmRectangleRec const *rect);
    virtual void        UnionRegion( AmRegionClass    *otherRegion);
    virtual void        IntersectRegion( AmRegionClass        *otherRegion);
    virtual void        XorRegion( AmRegionClass      *otherRegion);
    virtual void        SubtractRegion( AmRegionClass *otherRegion);
    virtual AmRegionClass	*CopyRegion();
};


#endif /* _anvil_postscript_h_ */
