#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_profile_h_
#define _anvil_profile_h_
#include <anvil/motifdisplay.h>
#include <anvil/plot.h>
#include <anvil/graph.h>
#include <anvil/imageplot.h>
#include <anvil/scatterplot.h>
#include <anvil/tick.h>
#include <anvil/overlay.h>
#include <anvil/outline.h>

#define MAXPLOTS	128

class AmProfileClass {
protected:
    AmDragLineOutlineClass	*outline;	/* drag outline cursor */
    AmMotifDisplayClass   *mainDisplay;           /* display for handler */
    AmPlotClass           *sourcePlot;            /* plot we're profiling */
    AmPlotClass           **profilePlots;         /* plot layers to profile */
    int                 numProfilePlots;        /* # of layers to profile */
    AmMotifDisplayClass   *graphDisplay;          /* display to update */
    AmOverlayClass        *graph;                 /* graphs to draw */
    AmXDisplayClass	*legendDisplay;		/* profile legend display */
    AmXWindowClass	*legendWindow;		/* profile legend window */
    AmXGCClass		*legendGC;		/* profile legend GC */
    AmXDisplayClass	*readoutDisplay;	/* profile readout display */
    AmXWindowClass	*readoutWindow;		/* profile readout window */
    AmXGCClass		*readoutGC;		/* profile readout GC */
    AmTickClass           *bottomTicks;
    AmTickClass		*vertTicks[ MAXPLOTS];	/* vertical tickmarks */
    int			numTicks;		/* number of tickmarks */
    Widget              readoutArea;            /* readout label */
    Widget              legendArea;             /* legend label */
    Widget              formDlog;               /* profile form dialog */
    Widget		actionForm;		/* bottom of dialog */
    Widget		profArea;		/* where profile graph drawn */
    unitType            units;                  /* distance units */
    float               distance;               /* distance of profile line */
    float		sampleWidth;		/* scatter plot sample width */
    float		startXpos;		/* drag starting point in */
    float		startYpos;		/* plot coordinates */
    float		endXpos;		/* drag ending point in */
    float		endYpos;		/* plot coordinates */
    char		*name;			/* string name for resources */
    AmErrorCode		makeImageProfile( AmImagePlotClass	*imagePlot);
    AmErrorCode		makeScatterProfile( AmScatterPlotClass	*scatterPlot);
public:
    AmProfileClass( AmMotifDisplayClass     *mainDisplay,
                  AmPlotClass             *plot,
                  char                  *name);
    virtual 		~AmProfileClass();
    virtual void        updateProfileGraph();
    virtual void        updateReadout();
    void		updateLegend();
    void                makeProfileGraph();
    void		freeProfileGraph();
    void		SetStartPos( float	x,
				     float	y) { startXpos = x; startYpos = y; };
    void		SetEndPos( float	x,
				   float	y) { endXpos = x; endYpos = y; };
    AmPlotClass           *GetPlot() { return( sourcePlot); };
    AmMotifDisplayClass   *GetMainDisplay() { return( mainDisplay); };
    Widget              GetDialog() { return( formDlog); };
    AmDragLineOutlineClass	*GetOutline() { return( outline); };
    void			SetOutline( AmDragLineOutlineClass *outlineArg) 
					{ outline = outlineArg; };
};

AnvilBoolean AmDataProfileCallback( Widget                btn,
                             AmMotifDisplayClass     *mainDisplay,
                             AmPlotClass             *plot,
                             char                  *name,
                             XtPointer);

#endif /* _anvil_profile_h_ */
