#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_proj_setupP_H_ 
#define _anvil_proj_setupP_H_ 

#include <anvil/record.h>

#define PROJ_ARG_FILE   "projargs"
#define NAMELEN		16
#define NPROJFIELDS	40
#define NELLFIELDS	20

typedef enum {
    FLAG,
    VALUE
} EprojArgType;

typedef struct AmProjArg_ {
    char                projName[ NAMELEN];	/* name of projection */
    char                argName[ NAMELEN];	/* name of argument */
    EprojArgType        argType;		/* value/flag argument */
    char                argDesc[ 64];		/* argument description */
} AmProjArgRec, *AmProjArgPtr;

typedef struct AmProjField_ {
    char	fieldName[ NAMELEN];		/* same as argName */
    void	*data;				/* to store arg data */
} AmProjFieldRec, *AmProjFieldPtr;

/* arguments for all map projections */

typedef struct AmProjArgData_ {
    int         numProjArg;			
    AmProjArgPtr  projArg;
} AmProjArgDataRec, *AmProjArgDataPtr;

/* class for storing projection information */

class AmProjArgClass : public AmRecordClass {
protected:
    char	projName[ NAMELEN];
    char	ellipseName[ NAMELEN];
    AnvilBoolean     customEllipse;
    float	W;
    float	alpha;
    float	azi;
    float	h;
    float	k;
    float	lat_0;
    float	lat_1;
    float	lat_2;
    float	lat_b;
    float	lat_ts;
    float	lon_0;
    float	lon_1;
    float	lon_2;
    float	lonc;
    int		lsat;
    AnvilBoolean	no_cut;
    AnvilBoolean	no_rot;
    AnvilBoolean	ns;
    int		path;
    AnvilBoolean	south;
    float	tilt;
    float	x_0;
    float	y_0;
    int		zone;
    float	ellipse_a;
    float	ellipse_es;
    float	ellipse_b;
    float	ellipse_e;
    float	ellipse_rf;
    float	ellipse_f;
    AnvilBoolean	inverseFlag;
public:
		AmProjArgClass();
    		AmProjArgClass( char 	**args,
			      int	nArgs,
			      AnvilBoolean	inverseFlag);
    void 	initFields();
		~AmProjArgClass();
    virtual int getFields( AmTagValuePtr	tagValues,
			   int		numValues);
    virtual int setFields( AmTagValuePtr	tagValues,
			   int		numValues);
    virtual int dumpFields( AmFieldLengthPtr	fieldLengths,
			    AnvilBoolean		allFields = FALSE);
    void GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
			      int		*numDefs);
};

typedef struct AmProjFormData_ {
    AmProjArgClass	*projInfo;		/* projection arguments */
    Widget		projList;		/* list of projections */
    Widget		ellipseList;		/* list of world ellipses */
    Widget		userEllipseTgl;		/* user defined ellipse parms */
    Widget		argRowcol;		/* projection args rowcolumn */
    Widget		quadArea;		/* display original quads */
    Widget		projQuadArea;		/* display projected quads */
    Widget		scrollWin;		/* rowcol parent scrollwin */
    Widget		inverseTgl;		/* inverse projection toggle */
    AmProjArgDataPtr	projArgData;		/* all projection arguments */
    char		***projArgs;		/* result projection args */
    int			*nArgs;			/* result # args */
    char		**testProjArgs;		/* test projection args */
    int			nTestProjArgs;		/* # test projection args */
    PJ			*testPJ;		/* test projection */
    AmProjQuadPtr		projQuads;		/* quads to display */
    int			nQuads;			/* #of display quadrangles */
    Widget		argToggles[ NPROJFIELDS];   /* toggles for arguments */
    Widget		ellipseToggles[ NELLFIELDS];
    Widget		ellipseFields[ NELLFIELDS];	/* userdef ellipse */
} AmProjFormDataRec, *AmProjFormDataPtr;

#endif /* _anvil_proj_setupP_H_  */
