#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_projclass_h_ 
#define _anvil_projclass_h_ 
#include <anvil/projects.h>
#include <anvil/record.h>
#include <anvil/units.h>

#define INV_PROJ_TAG "inv-proj="

class AmProjectionClass { 
protected: 
     char       	**projStrs;	/* strings for the projection */
     int		numProjStrs;	/* for AmRecordClass vector consistency */
     unitType		srcUnits;	/* projection expected units */
     unitType		dstUnits;	/* projection result units */
     PJ         	*pj;		/* internal projection data structure */
     AnvilBoolean		inverseFlag;	/* projection inverse flag */
     AnvilBoolean		valid;
public:

		AmProjectionClass();
		AmProjectionClass( const char *projName,
				 AnvilBoolean	inverseFlag,
				 const char  **args);
		AmProjectionClass( const char	**args);
		AmProjectionClass( const char **args,
				 AnvilBoolean	inverseFlag);

    virtual  	~AmProjectionClass();
    AnvilBoolean	IsValid() { return( valid); };
    void	SetValid( AnvilBoolean	flag) { valid = flag; };

    unitType	expectsUnits();
    unitType	resultUnits();

    AmErrorCode	SetProjection( const char **args);

    const char **DumpProjection();
    AnvilBoolean	IsProjected();

    AnvilBoolean     ProjectPt( float	*dstX,
			   float	*dstY);

    AnvilBoolean	InvProjectPt( float	*dstX,
			      float	*dstY);

    AnvilBoolean 	ProjectPtUnits( float     *dstX,
				float     *dstY,
				unitType  xUnits,
				unitType  yUnits);

    AnvilBoolean 	InvProjectPtUnits( float          *dstX,
				   float          *dstY,
				   unitType       xUnits,
				   unitType       yUnits);

    virtual int setFields( AmTagValuePtr  tagValues,
                           int          numValues);
    virtual int getFields( AmTagValuePtr      tagValues,
                           int              numValues);
};

#endif /* _anvil_projclass_h_ */

