#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_project_h_
#define _anvil_project_h_
#include <stdlib.h>
#include <assert.h>
#include <anvil/errcode.h>

#if defined(linux) || defined(_WIN32) || (__BSD44__)

typedef unsigned char uchar_t;
typedef unsigned short ushort_t;
typedef unsigned int uint_t;
typedef unsigned long ulong_t;

#else
#include <sys/types.h>
#endif

#if defined(_WIN32)
#include <io.h>
#include <stdio.h>


#ifdef __cplusplus 
extern "C" {
#endif
int access(const char *, int);
int chmod(const char *, int);
int chsize(int, long);
int close(int);
int creat(const char *, int);
int dup(int);
int dup2(int, int);
int eof(int);
long filelength(int);
int isatty(int);
int locking(int, int, long);
long lseek(int, long, int);
char * mktemp(char *);
int open(const char *, int, ...);
int read(int, void *, unsigned int);
int setmode(int, int);
int sopen(const char *, int, int, ...);
long tell(int);
int umask(int);
int unlink(const char *);
int write(int, const void *, unsigned int);
FILE *popen( const char *command, const char *mode);
int pclose( FILE *fp);
ulong_t pread( int fildes, void *buf, size_t nbyte, long offset);
ulong_t pwrite( int fildes, const void *buf, size_t nbyte, long offset);
long lseek( int fildes, long offset, int whence);
char *getcwd( char *buf, int size);
int isattty( int fildes);
int strcasecmp( const char *s1, const char *s2);
int strncasecmp( const char *s1, const char *s2, size_t length);
int rint( float value);
int ftruncate( int fd, long size);
#ifdef __cplusplus 
};
#endif

/* #defines for MS-Win */

#define F_OK		00
#define W_OK		02
#define R_OK		04
#define O_CREAT		_O_CREAT
#define O_APPEND	_O_APPEND
#define O_RDONLY	_O_RDONLY
#define O_TRUNC		_O_TRUNC


/* there's a read and write permission flag in MS, but there's no
   version of it in NT... */

#else
#include <unistd.h>
#endif

#ifndef AnvilBoolean
typedef char  AnvilBoolean;
#endif
#define TRUE	1
#define FALSE	0
typedef int 	      ( *AmSearchFn)( const void *, const void *);
typedef int 	      ( *AmElementFn)( const void *);
typedef void 	      ( *AmItemFn)( const void *);
#define FREE( x)	{ free( x); x = NULL; }
#define PATHLEN		512
#define RESLEN		512
#define FIELDLEN	512
#define MSGLEN		512
#define BUFFERLEN	1024
#define HUGE_BUFFER	5096
#define ENVLEN		128
#define TEXTLEN		256
#define MAXARGS		20
#define PERMS		0666

#define ASSERT(expr) assert(expr)

#define ENV_DEBUG_LEVEL	"AN_DEBUG_LEVEL"

#define _DEBUG_ON
#ifdef _DEBUG_ON
#define AN_DEBUG(x,y) \
    if ( getenv(ENV_DEBUG_LEVEL) ) { \
        if ( x <= atoi(getenv(ENV_DEBUG_LEVEL) ) ) printf y; \
    }
#else
#define AN_DEBUG(x,y)   {}
#endif

/* this is not defined in Solaris ... */

#ifdef sparc
#include <values.h>
#endif

/* this is not defined in Solaris ... */
#ifndef M_PI
#define M_PI	3.14159265358979323846
#endif

#endif /* _anvil_project_h_ */
