#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* General projections header file */
#ifndef _anvil_projectsP_h
#define _anvil_projectsP_h

    /* standard inclusions */
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <anvil/project.h>

/* external header has needed structure declarations */

#include <anvil/projects.h>
	/* prototype hypot for systems where absent */
extern double hypot(double, double);
	/* some useful constants */
#ifndef HALFPI
#define HALFPI		1.5707963267948966
#endif

#ifndef FORTPI
#define FORTPI		0.78539816339744833
#endif

#ifndef PI
#define PI		3.14159265358979323846
#endif

#ifndef TWOPI
#define TWOPI		6.2831853071795864769
#endif

#ifndef RAD_TO_DEG
#define RAD_TO_DEG	57.29577951308232
#endif 

#ifndef DEG_TO_RAD
#define DEG_TO_RAD	.0174532925199432958
#endif

/* environment parameter name */
#ifndef PROJ_LIB
#define PROJ_LIB "PROJ_LIB"
#endif
/* maximum tag id length for +init and default files */
#ifndef ID_TAG_MAX
#define ID_TAG_MAX 50
#endif

/* directory delimiter for DOS support */
#ifdef DOS
#define DIR_CHAR '\\'
#else
#define DIR_CHAR '/'
#endif


struct PJ_LIST {
	char	*id;		/* projection keyword */
	PJ	*( *proj)( PJ *);	/* projection entry point */
	char	*name;		/* basic projection full name */
};
struct PJ_ELLPS {
	char	*id;	/* ellipse keyword name */
	char	*major;	/* a= value */
	char	*ell;	/* elliptical parameter */
	char	*name;	/* comments */
};
struct PJ_UNITS {
	char	*id;	/* units keyword */
	char	*to_meter;	/* multiply by value to get meters */
	char	*name;	/* comments */
};

#define IS_ANAL_XL_YL 01	/* derivatives of lon analytic */
#define IS_ANAL_XP_YP 02	/* derivatives of lat analytic */
#define IS_ANAL_HK	04		/* h and k analytic */
#define IS_ANAL_CONV 010	/* convergence analytic */
    /* parameter list struct */

#ifndef PJ_LIST__
extern struct PJ_LIST pj_list[];
#endif

#ifndef PJ_ELLPS__
extern struct PJ_ELLPS pj_ellps[];
#endif

#if 0
#ifndef PJ_UNITS__
extern struct PJ_UNITS pj_units[];
#endif
#endif

#ifdef PJ_LIB__
    /* repeatative projection code */
#define ENTRYA(name) PJ *name(PJ *P) { if (!(P)) { \
	if ((P = pj_malloc(sizeof(PJ))) != NULL) { \
	P->pfree = freeup; P->fwd = 0; P->inv = 0; P->spc = 0;
#define ENTRYX } return P; } else {
#define ENTRY0(name) ENTRYA(name) ENTRYX
#define ENTRY1(name, a) ENTRYA(name) P->a = 0; ENTRYX
#define ENTRY2(name, a, b) ENTRYA(name) P->a = 0; P->b = 0; ENTRYX
#define ENDENTRY(p) } return (p); }
#define E_ERROR(err) { pj_errno = err; freeup(P); return(0); }
#define E_ERROR_0 { freeup(P); return(0); }
#define F_ERROR { pj_errno = -20; return(xy); }
#define I_ERROR { pj_errno = -20; return(lp); }
#define FORWARD(name) static XY name(LP lp, PJ *P) { XY xy
#define INVERSE(name) static LP name(XY xy, PJ *P) { LP lp
#define FREEUP static void freeup(PJ *P) {
#define SPECIAL(name) static void name(LP lp, PJ *P, struct FACTORS *fac)
#endif

void pr_series( Tseries *T, FILE *file, char *fmt);

#endif /* end of basic projections header */
