#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_record_h_
#define _anvil_record_h_
#include <stdarg.h>
#include <stdio.h>
#include <anvil/project.h>
#include <anvil/datatype.h>
#include <anvil/units.h>
#include <anvil/resource.h>
#include <anvil/register.h>
#include <anvil/changed.h>
#include <anvil/getset.h>
#include <anvil/header.h>
#include <anvil/tagvalues.h>

#define MAXASCIIBUF	4096

typedef struct AmFieldDef_ {
    char	     *tag;		/* string tag */
    int		     offset;		/* structure offset */
    int		     countOffset;	/* AmVeclen offset,-1 if NULL ended */
    AmStorageTypeEnum  storage;		/* static array or alloced pointer */
} AmFieldDefRec, *AmFieldDefPtr;

/* used by dump fields, this contains the tag, and the length of the vector
   (if the value is a vector) */

typedef struct AmFieldLength_ {
    char	*tag;			/* string tag */
    int		length;			/* vector length */
} AmFieldLengthRec, *AmFieldLengthPtr;

/* extra fields is sorted by the tag, for a fast zipper merge */

class AmRecordClass {
private:
    AnvilBoolean 		valid;		/* valid object or not */
    AmTagValuePtr	extraFields;	/* fields not eaten by setFields() */
    int			numExtraFields;	/* #of extra fields */
protected:
public:
    void		setExtraFields( AmTagValuePtr	tagValues,
					int		numValues);
    void		getExtraFields( AmTagValuePtr	tagValues,
					int		numValues);
    void		AmSetTagValue( AmTagValuePtr	dst,
				     AmTagValuePtr	src);
    virtual int		setFieldsWrapper( AmTagValuePtr	tagValues,
				          int		numValues);
    int			getFieldsWrapper( AmTagValuePtr	tagValues,
				          int		numValues);
    virtual int		setFields( AmTagValuePtr	tagValues,
				   int		numValues) = 0;
    virtual int 	getFields( AmTagValuePtr	tagValues,
				   int		numValues) = 0;
    virtual int 	dumpFields( AmFieldLengthPtr fieldLengths,
				    AnvilBoolean	   allFields = FALSE);
    int 		GetFieldByDefinitions( AmFieldDefPtr  fieldDefs,
					       int          numfieldDefs,
					       AmTagValuePtr  tagValues,
					       int          numValues);
    int 		SetFieldByDefinitions( AmFieldDefPtr  fieldDefs,
					       int          numfieldDefs,
					       AmTagValuePtr  tagValues,
					       int          numValues);
    int 	DumpFieldsByDefinitions( AmFieldLengthPtr	fieldLength,
					 int            numStrings,
					 AmFieldDefPtr    fieldDefs,
					 int            numFieldDefs,
					 char		**exceptFields = NULL);
    void		freeTagValue( AmTagValuePtr	tagValue);
    			AmRecordClass();
    virtual 		~AmRecordClass();
    AnvilBoolean		SetVaFields( const char *tag, ...);
    int    		GetVaFields( const char *tag, ...);
    AnvilBoolean		IsValid();
    void		SetValid( AnvilBoolean	flag);
    AnvilBoolean		WriteHeader( FILE *fp);
    virtual AnvilBoolean	DumpFields( FILE *fp);
    AnvilBoolean		RetreiveTagValues( AmTagValuePtr	tagValues,
					   int		*numValues);
};

void 
AmFreeNonAtomicValues( AmTagValuePtr	tagValues,
		       int		numValues);

AnvilBoolean
AmSetRecordField( AmRecordClass  *record,
                const char   *tag,
                void         *value);

int AmCompareFieldTag( char               *tag,
                     AmFieldDefPtr        fieldDef);

#endif /* _anvil_record_h_ */
