#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_rect_h_
#define _anvil_rect_h_
#include <sys/types.h>
#include <anvil/project.h>
#include <anvil/graphstructs.h>

extern const ulong_t AmRECT_ABOVE;
extern const ulong_t AmRECT_BELOW;
extern const ulong_t AmRECT_LEFT;
extern const ulong_t AmRECT_RIGHT;

typedef enum AmExpose_ {
    AmNO_OVERLAP = 1,
    AmINTERSECT = 2,
    AmCOVER = 3,
} AmExposeEnum;


AnvilBoolean AmClipRect( AmRectangleRec const *clipRect,
                  AmRectanglePtr       rect);

AnvilBoolean AmClipRectVec( AmRectangleRec const *clipRect,
                     AmRectanglePtr       rectVec,
                     int                *nRect);

AnvilBoolean AmDegenRect( AmRectangleRec const *rect);

AnvilBoolean AmPtInRect( AmRectangleRec const *rect,
		  int		     x,
		  int		     y);

AnvilBoolean AmUnionRect( AmRectangleRec const   *rect1,
                   AmRectangleRec const   *rect2,
                   AmRectanglePtr         rect3);

AnvilBoolean AmIntersectRect( AmRectangleRec const       *rect1,
                       AmRectangleRec const       *rect2,
                       AmRectanglePtr             rect3);

AnvilBoolean AmIntersectRect( AmRealRectRec const       *rect1,
                       AmRealRectRec const       *rect2,
                       AmRealRectPtr             rect3);

AmRectangleRec const *AmRelativeRect( AmRectangleRec const	*fromRect,
				  AmRectangleRec const	*toRect);

void AmUnionRectVec( AmRectangleRec const   *rectVec,
		   int                  nRects,
		   AmRectanglePtr         rect);

AnvilBoolean AmInRealRect( AmRealRectRec const 	*rect,
		    float		x,
		    float		y);

AnvilBoolean	AmEqualRect( AmRectangleRec const	*a,
		   AmRectangleRec const	*b);

AnvilBoolean AmSrcAndDestRects( int            srcRows,
                         int            srcCols,
                         float          srcXmin,
                         float          srcYmin,
                         float          srcXmax,
                         float          srcYmax,
                         int            dstRows,
                         int            dstCols,
                         float          dstXmin,
                         float          dstYmin,
                         float          dstXmax,
                         float          dstYmax,
                         AmRectanglePtr   srcRect,
                         AmRectanglePtr   dstRect);

void AmMakeIntRect( AmRealRectRec const      *srcRect,
                    AmRectanglePtr           dstRect);

void AmMakeContainedIntRect( AmRealRectRec const      *srcRect,
			     AmRectanglePtr           dstRect);

void AmApplyFlips( AmRealRectPtr	rect,
		 AnvilBoolean	flipRows,
		 AnvilBoolean	flipCols);

void AmApplyFlips( AmRectanglePtr	rect,
		 AnvilBoolean	flipRows,
		 AnvilBoolean	flipCols);

AnvilBoolean AmEqRect( AmRectangleRec const	*a,
		AmRectangleRec const	*b);

AmExposeEnum AmCreateExposureRects( AmRectangleRec const    *frontRect,
				    AmRectangleRec const    *backRect,
				    AmRectanglePtr          leftRect,
				    AmRectanglePtr          topRect,
				    AmRectanglePtr          rightRect,
				    AmRectanglePtr          bottomRect,
				    ulong_t                 *flags);

AnvilBoolean AmDifferenceRects( const AmRectangleRec *oldRect,
			   const AmRectangleRec *newRect,
			   AmRectanglePtr       rectVec,
			   int                  *nRects);

AnvilBoolean AmDifferenceIncludeRects( const AmRectangleRec  *oldRect,
				  const AmRectangleRec  *newRect,
				  AmRectanglePtr        rectVec,
				  int                   *nRects);

AnvilBoolean AmUnifyRects( AmRectangleRec const   *a,
                      AmRectangleRec const   *b,
                      AmRectanglePtr         c);

void AmUnifyRectVec( AmRectanglePtr rects,
                     int            *nRects);

int AmWidthToPixels( int cx); 
int AmHeightToPixels( int cy);
int AmPixelsToWidth( int nHorizPixels); 
int AmPixelsToHeight( int nVertPixels); 

#endif /* _anvil_rect_h_ */
