#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef __anvil_regparse_h_
#define __anvil_regparse_h_

typedef enum {
    AmRANGE_FIXED= 1,
    AmMIN_OCCURRENCES = 2,
    AmRANGE_OCCURRENCES = 3
} AmRangeTypeEnum;

typedef enum {
    AmOR_REGEXP = 1,
    AmCHAR_REGEXP = 2,
    AmDOT_REGEXP = 3,
    AmRANGE_REGEXP = 4,
    AmCHARSET_REGEXP = 5,
    AmSTAR_REGEXP = 6,
    AmOCCURRENCE_REGEXP= 7,
    AmREGEXP = 8 
} AmRegexpTagEnum;

typedef struct AmCharRE_ {
    char		character;	/* one-character: match the character */
} AmCharRERec, *AmCharREPtr;

typedef struct AmRangeRE_ {
    char		minChar;	/* range: match [A-Z] */
    char		maxChar;
} AmRangeRERec, *AmRangeREPtr;

typedef struct AmCharsetRE_ {
    int			numSet;		/* charse: match [eruASHIU] */
    struct AmRE_  	**charset;
} AmCharsetRERec, *AmCharsetREPtr;

typedef struct AmStarRE_ {
    struct AmRE_  	*RE;		/* match 0-? occurences */
} AmStarRERec, *AmStarREPtr;

typedef struct AmOccurRE_ {
    struct AmRE_  	*RE;		/* match min-max occurrences */
    int			min;
    int			max;
} AmOccurRERec, *AmOccurREPtr;

typedef struct AmFullRE_ {
    int			numSet;		/* match a series of regular expressions */
    struct AmRE_	**REset;
} AmFullRERec, *AmFullREPtr;

typedef struct AmOrRE_ {		/* expression 'a' or expression 'b' */
    struct AmRE_	*REleft;
    struct AmRE_	*REright;
} AmOrRERec, *AmOrREPtr;

/* regular expression union */

typedef struct AmRE_ {
    AmRegexpTagEnum	tag;
    union {
	AmCharRERec	charRE;
	AmRangeRERec	rangeRE;
	AmCharsetRERec	charsetRE;
	AmOccurRERec	occurRE;
	AmStarRERec	starRE;
	AmFullRERec	fullRE;
	AmOrRERec		orRE;
    } REVal;
} AmRERec, *AmREPtr;

/* for fast lookup of regular expression strings */

typedef struct REString_ {
    char	*regexpString;
    AmREPtr	regexp;
} AmREStringRec, *AmREStringPtr;

AmREPtr AmAddRegexRE( AmREPtr		a, 
		  AmREPtr 	set);

AmREPtr AmOrRegexRE( AmREPtr	a, 
		 AmREPtr 	b);

AmREPtr AmMakeStarRE( AmREPtr a);

AmREPtr AmMakeRangeRE( char 	*a,
		   char		*b);

AmREPtr AmAddCharsetRE( AmREPtr 	a, 
		    AmREPtr 	set);

AmREPtr AmMakeOneCharRE( char *str);
AmREPtr AmMakeDotRE( char *str);

AmREPtr AmMakeOccurRE( AmREPtr 		re,
		   char 		*str, 
		   AmRangeTypeEnum	type);

int AmStrYYinput( char 	*buffer,
		int	maxSize);

#endif /* __anvil_regparse_h_ */
