#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_render3d_h_
#define _anvil_render3d_h_

#include <anvil/overlay.h>
#include <anvil/imageplot.h>

typedef struct AmVerticalClientData_ {
    AmAccessFn		zFn;
    void		*clientData;
    float		verticalExaggeration;
} AmVerticalClientDataRec, *AmVerticalClientDataPtr;

class Render3DClass : public AmScreenObjClass {
protected:
    AmImageClass          	*displayImage;
    AmImageClass			*texImage;
    AmImagePlotClass      	*elevationPlot;
    AmImagePlotClass		*sourceElevationPlot;
    float               	ex;
    float               	ey;
    float               	ez;
    float               	focal;
    float               	phi;
    float               	theta;
    float               	screenWidth;
    float               	screenHeight;
    int				texWidth;
    int				texHeight;
    AmVerticalClientDataRec	verticalClientData;
    uchar_t			*elevationMatrix;
    int				matrixRows;
    int				matrixCols;
public:
    void                initializeFields( AmWindowClass   *windowArg,
                                          const char     *nameArg);
                        Render3DClass();
                        Render3DClass( AmWindowClass	*windowArg,
				       AmOverlayClass	*overlayArg,
                                       const char       *nameArg,
                                       ...);
    virtual             ~Render3DClass();
    virtual int         setFields( AmTagValuePtr  tagValues,
                                   int          numValues);
    virtual int         getFields( AmTagValuePtr      tagValues,
                                   int              numValues = FALSE);
    virtual int         dumpFields( AmFieldLengthPtr      fieldLengths,
                                    AnvilBoolean             allFields);
    virtual AmErrorCode	Update( int	*eventCodes = NULL,
				int	numCodes = 0);
    void		updateViewpoint();
    virtual void	updateFields();
    virtual AnvilBoolean     IsType( AmObjectTypeEnum type);
    virtual AnvilBoolean     checkValues();
    void                GetFieldDefinitions( AmFieldDefPtr        *fieldDefs,
                                             int                *numDefs);
    virtual AmErrorCode draw( AmWindowClass        *w,
                              AmGCClass            *gc,
                              AmRectangleRec const *vrctExposeRects,
                              int                nExposeRects);

};


#endif /* _anvil_render3d_h_ */
