#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_scatter_h_
#define _anvil_scatter_h_

#include <sys/types.h>
#include <anvil/display.h>
#include <anvil/coordinates.h>
#include <anvil/fns.h>

#define MAXSYM	256

/* if there's a symbol function defined, it returns a symbol name, which
   is the symbol filename.  This caching mechanism makes it so the program
   doesn't load a new symbol every time the symfile changes */

typedef struct AmSymCache_ {
    char	*symfile;	
    int		symWidth;
    int		symHeight;
    uchar_t	*symbol;
} AmSymCacheRec, *AmSymCachePtr;

typedef struct AmGlyphCache_ {
    char	*symfile;
    float	scale;
    ulong_t	color;
    AmGlyphClass	*glyph;
} AmGlyphCacheRec, *AmGlyphCachePtr;

AmSymCachePtr AmGetSymbol( char     *symfile, 		/* symbol file */
                       void     **symbolCache);		/* tsearch root */

AmGlyphCachePtr AmGetGlyph( AmDisplayClass    *display,
                        AmWindowClass     *window,
                        char            *symfile,
                        float           scale,
                        ulong_t         color,
                        void            **glyphCache);

void AmFreeSymbolCache( void	**symbolCache);

ulong_t AmColormapPixelFn( void            *val,
                         AmIndexDataPtr    indexData,
                         AmColorCellPtr    colors);

void AmScalePix( ulong_t      *image,
               int          imgWidth,
               int          imgHeight,
               uchar_t      *pix,
               int          pixWidth,
               int          pixHeight,
	       ulong_t	    pixel,
               int          x,
               int          y,
               float        scale);

AmErrorCode 
AmSymbolDraw( AmDisplayClass	*display, 
		 AmCoordClass     *coords,         /* coordinate system */
                 AmRectangleRec const *drawRect,   /* update this rectangle */
                 uchar_t        *srcData,        /* scatter points */
                 int            srcSize,         /* point data size */
                 int            length,          /* # of points */
                 AmAccessFn       xFn,             /* scatter x,y functions */
		 int		xOffset,
                 AmAccessFn       yFn,
		 int		yOffset,
                 AmIndexDataPtr   colorIndexData,  /* color index data */
                 AmColorCellPtr   colorList,      /* color list to index */
                 AmIndexDataPtr   symbolIndexData, /* symbol index data */
                 char           **symbolList,    /* symbol list to index */
                 AmIndexDataPtr   symbolSizeIndexData,     /* size index data */
                 float          *symbolSizeList, /* symbol size list */
                 void           *clientData,     /* client data for x,y fns */
                 AmImageClass     *image,          /* image to write to */
                 AmSymCachePtr    sym,             /* current symbol */
                 void           **symbolCache,   /* for loading new symbols */
		 AmColorCellPtr	fgColor);	 /* 2-tuple foreground color */

AmErrorCode
AmBilinSymbolDraw( AmDisplayClass	 *display, 
		      AmCoordClass     	 *coords,
		      AmRectangleRec const *drawRect,
		      uchar_t            *srcData,
		      int                srcSize, 
		      int                length, 
		      AmAccessFn           xFn,   
		      int		 xOffset,
		      AmAccessFn           yFn,
		      int		 yOffset,
		      AmIndexDataPtr       colorIndexData, 
		      AmColorCellPtr       colorList,  
		      AmIndexDataPtr       symbolIndexData,
		      char               **symbolList,  
		      AmIndexDataPtr       symbolSizeIndexData,
		      float              *symbolSizeList,
		      void               *clientData,
		      AmImageClass         *image,
		      AmSymCachePtr        sym,
		      void               **symbolCache, 
		      AmColorCellPtr	 fgColor);


AmErrorCode
AmDrawSymbolImage( AmDisplayClass	 *display, 
		      AmCoordClass     	 *coords,
		      AmRectangleRec const *drawRect,
		      uchar_t            *srcData,
		      int                srcSize, 
		      int                length, 
		      AmAccessFn           xFn,   
		      int		 xOffset,
		      AmAccessFn           yFn,
		      int		 yOffset,
		      AmIndexDataPtr       colorIndexData, 
		      AmColorCellPtr       colorList,  
		      AmIndexDataPtr       symbolIndexData,
		      char               **symbolList,  
		      AmIndexDataPtr       symbolSizeIndexData,
		      float              *symbolSizeList,
		      void               *clientData,
		      AmImageClass         *image,
		      AmSymCachePtr        sym,
		      void               **symbolCache, 
		      AmColorCellPtr	 fgColor,
		      AnvilBoolean		 interpolate);

AmErrorCode
AmGlyphDraw( AmDisplayClass            *display,
                AmWindowClass             *window,
                AmCoordClass              *coords,
                AmRectangleRec const      *drawRect,
                uchar_t                 *srcData,
                int                     srcSize,
                int                     length,
                AmAccessFn                xFn,
	        int		 	xOffset,
                AmAccessFn                yFn,
	        int		 	yOffset,
                AmIndexDataPtr            colorIndexData,
                AmColorCellPtr            colorList,
                AmIndexDataPtr            symbolIndexData,
                char                    **symbolList,
                AmIndexDataPtr            symbolSizeIndexData,
                float                   *symbolSizeList,
                void                    *clientData,
                AmGlyphCachePtr           glyph,
                void                    **glyphCache,
                AmColorCellPtr            fgColor);

#endif _anvil_scatter_h_
