#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_scattergraph_h_
#define _anvil_scattergraph_h_

#include <anvil/scatter.h>
#include <anvil/graph.h>

extern const int SCATTERBUFSIZE;

/* scatter graph class:
   The display image is written into, just like a standard image.
   the symbol is the symbol to use for plotting.  It's read from a
   standard X11 bitmap file. symWidth and symHeight define the
   dimensions of the symbol.  symfile is the symbol's source file 
   initSpecificFilelds initializes the AmScatterGraphClass specific
   fields (relative to AmGraphClass)
   The symbol function takes the object and a client data, and returns
   the name of the symbol.
   symbolCache is the root of the symbol cache tree.
   symbolSizeFn take the object and client data, and returns 0.0 to 1.0,
   which scales the symbol as it is drawn
*/

class AmScatterGraphClass : public AmGraphClass {
protected:
	AmImageClass	*displayImage;		/* image buffer */
	char		*symfile;		/* single symbol file */
	AmSymCachePtr	sym;			/* symbol cache */
	AmGlyphCachePtr	glyph;
	void		*symbolCache;		/* to speed symbol access */
	void		*glyphCache;
	AmIndexDataRec	symbolIndex;		/* symbol list index table */
	char		**symbolList;		/* symbol name list */
	AmIndexDataRec	symbolSizeIndex;	/* symbol size index table */
	float		*symbolSizeList;	/* symbol size list */
	int		scatterBufSize;		/* buf size (AmREAD_SOURCE) */
	AmIndexDataRec	colorIndex;		/* color index */
	AnvilBoolean		interpolate;		/* bilinear interpolate flag */
	void 		GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
					     int		*numValues);
	void		initializeFields( AmWindowClass	*windowArg,
					  const char	*nameArg);
	virtual AnvilBoolean	checkValues();
public:
	virtual int 		setFields( AmTagValuePtr  tagValues,
			       		   int	    numValues);
	virtual int 		getFields( AmTagValuePtr  tagValues,
					   int	    	numValues);
    	virtual void 		updateFields();
	virtual int		dumpFields( AmFieldLengthPtr fieldLengths,
					    AnvilBoolean	   allFields = FALSE);
	virtual AnvilBoolean		IsType( AmObjectTypeEnum type);
	AmScatterGraphClass();
	AmScatterGraphClass( AmWindowClass   *windowArg,
			   const char	 *nameArg,
			   ...);
	AmScatterGraphClass( AmOverlayClass *overlayArg,
			   const char	*nameArg, 
			   ...);
	virtual ~AmScatterGraphClass();
	virtual AmErrorCode DrawSelection( AmWindowClass  	*window,
					   AmGCClass		*gc,
					   AmRectangleRec const	*exposeRects,
					   int			nRects,
					   AnvilBoolean		fHighlight);

	virtual AmErrorCode draw( AmWindowClass  		*window,
				  AmGCClass		*gc,
				  AmRectangleRec const	*exposeRects,
				  int			nRects);
	virtual AmErrorCode Update( int	*eventCodes = NULL,
				    int	numCodes = 0);
	virtual void 		ObjectExtents( void      *ptr,
						float     *xmin,
						float     *ymin,
						float     *xmax,
						float     *ymax);
};

#endif /* _anvil_scattergraph_h_ */
