#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_scatterplot_h_
#define _anvil_scatterplot_h_

#include <anvil/scatter.h>
#include <anvil/plot.h>
#include <anvil/overlay.h>

extern const int SCATTERBUFSIZE;

/* scatter plot class:
   The display image is written into, just like a standard image.
   the symbol is the symbol to use for plotting.  It's read from a
   standard X11 bitmap file. symWidth and symHeight define the
   dimensions of the symbol.  symfile is the symbol's source file 
   initSpecificFilelds initializes the AmScatterPlotClass specific
   fields (relative to AmPlotClass)
   The symbol function takes the object and a client data, and returns
   the name of the symbol.
   symbolCache is the root of the symbol cache tree.
   symbolSizeFn take the object and client data, and returns 0.0 to 1.0,
   which scales the symbol as it is drawn
*/

class AmScatterPlotClass : public AmPlotClass {
protected:
    AmImageClass		*displayImage;		/* image buffer */
    char		*symfile;		/* single symbol file */
    AmSymCachePtr		sym;			/* single symbol (cacheptr) */
    AmGlyphCachePtr	glyph;			/* glyph of symbol */
    void		*symbolCache;		/* to speed symbol access */
    void		*glyphCache;		/* to speed glyph access */
    AmIndexDataRec	symbolIndex;		/* symbol list index table */
    char		**symbolList;		/* symbol name list */
    AmIndexDataRec	symbolSizeIndex;	/* symbol size index table */
    float		*symbolSizeList;	/* symbol size list */
    int			scatterBufSize;		/* buf size (AmREAD_SOURCE) */
    AnvilBoolean		interpolate;
    void GetFieldDefinitions( AmFieldDefPtr	*fieldDefs,
			      int		*numValues);
    void drawSymbolStrings( AmWindowClass		*window,
			    AmGCClass		*gc,
			    uchar_t		*ptr,
			    int			numPts,
			    AmRectangleRec const	*boundRect);
    void	initializeFields( AmWindowClass	*windowArg,
				  const char	*nameArg);
    virtual AnvilBoolean	checkValues();
public:
    virtual int 	setFields( AmTagValuePtr  tagValues,
				   int	    	numValues);
    virtual int 	getFields( AmTagValuePtr  tagValues,
				   int	    	numValues);
    virtual void 	updateFields();
    virtual int		dumpFields( AmFieldLengthPtr fieldLengths,
				    AnvilBoolean	   allFields = FALSE);
    virtual AnvilBoolean 	IsType( AmObjectTypeEnum type);
    AmScatterPlotClass();
    AmScatterPlotClass( AmWindowClass   *windowArg,
		      const char	*nameArg,
		      ...);
    AmScatterPlotClass( AmOverlayClass *overlayArg,
		      const char   *nameArg, 
		      ...);
    virtual 		~AmScatterPlotClass();
    virtual AmErrorCode draw( AmWindowClass  		*window,
			      AmGCClass			*gc,
			      AmRectangleRec const	*exposeRects,
			      int			nRects);
    virtual AmErrorCode DrawSelection( AmWindowClass  		*window,
				       AmGCClass			*gc,
				       AmRectangleRec const	*exposeRects,
				       int			nRects,
				       AnvilBoolean			fHighlight);
    virtual AmErrorCode Update( int	*eventCodes = NULL,
				int	numCodes = 0);
    virtual void 	ObjectExtents( void      *ptr,
				       float     *xmin,
				       float     *ymin,
				       float     *xmax,
				       float     *ymax);
};

int AmClosestScatterPointIndex( float             ptX,
			      float             ptY,
			      AmScatterPlotClass  *scatterPlot);

#endif /* _anvil_scatterplot_h_ */
