#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_screenobj_h_
#define _anvil_screenobj_h_
#include <anvil/record.h>
#include <anvil/graphstructs.h>
#include <anvil/display.h>
#include <anvil/list.h>
#include <anvil/eventcodes.h>

typedef enum objectType { 
    AmERROR_OBJECT = 0,
    AmIMAGE_PLOT = 1,
    AmSCATTER_PLOT = 3,
    AmLINE_PLOT = 4,
    AmPOLYGON_PLOT = 5,
    AmOBJECT_PLOT = 7,
    AmOVERLAY = 8,
    AmCAD_PLOT = 9,
    AmPOLYLINE_PLOT = 10,
    AmLINE_GRAPH = 20,
    AmBAR_GRAPH = 21,
    AmSCATTER_GRAPH = 23,
    AmPOLAR_GRAPH = 24,
    AmIMAGE_LEGEND = 31,
    AmLINE_LEGEND = 32,
    AmSCATTER_LEGEND = 33,
    AmPOLYGON_LEGEND = 34,
    AmPOLYLINE_LEGEND = 35,
    AmTICKS = 35,
    AmGRID = 36,
    AmCIRCLE_ANNOTATION = 41,
    AmRECTANGLE_ANNOTATION = 42,
    AmTEXT_ANNOTATION = 43,
    AmIMAGE_ANNOTATION = 44,
    AmLINE_ANNOTATION = 45,
    AmMULTILINE_TEXT_ANNOTATION = 46,
    AmDISTANCE_SCALE = 47,

    AmSCREEN_OBJECT = 50,
    AmCOORD = 51,
    AmDATA_GRAPHIC = 52,
    AmPLOT = 53,
    AmGRAPH = 54,
    AmSCALE = 55,
    AmLEGEND = 56,
    AmANNOTATION = 57,
    AmREFERENCE_WINDOW = 58,
    AmSPHERE_PLOT = 59,
    AmRENDER_3D = 60,
    AmANY_OBJECT = 99
} AmObjectTypeEnum;


/* the mother of all screen objects */

class AmScreenObjClass : public AmRecordClass {
protected:
    AmObjectTypeEnum	objectType;
    AmScreenObjClass	*parent;
    AmDisplayClass	*display;
    AmWindowClass		*window;
    AmRectangleRec	rect;		/* position rectangle */
    AmRectangleRec	extentsRect;
    int			leftMargin;	/* offsets to window edges */
    int			rightMargin;
    int			topMargin;
    int			bottomMargin;
    char		*name;		/* object name for resources */
    AnvilBoolean	 	opaque;
    AmColorCellRec	fgColor;	/* foreground and background */
    AmColorCellRec	bgColor;	/* colors */
    AmColorCellPtr	colors;		/* color vector */
    int			numColors;	/* color vector length */
    char		*cmapName;	/* colormap name */

    AnvilBoolean	 	_fFrame;
public:
    void		initializeFields( AmWindowClass	*windowArg,
					  const char	*nameArg);
    			AmScreenObjClass();
			AmScreenObjClass( AmWindowClass	*windowArg,
					const char	*nameArg,
					...);
    virtual 		~AmScreenObjClass();
    virtual int 	setFieldsWrapper( AmTagValuePtr  tagValues,
				          int          numValues);
    virtual int 	setFields( AmTagValuePtr  tagValues,
				   int          numValues);
    virtual int 	getFields( AmTagValuePtr      tagValues,
				   int              numValues = FALSE);
    virtual int 	dumpFields( AmFieldLengthPtr	fieldLengths,
				    AnvilBoolean		allFields);
    AnvilBoolean		WriteHeader( FILE	*fp);
    virtual AnvilBoolean	IsType( AmObjectTypeEnum type);
    virtual AnvilBoolean	checkValues();
    void 		GetFieldDefinitions( AmFieldDefPtr 	*fieldDefs,
					     int		*numDefs);
    AmRectangleRec const	*GetRect();
    AmRectangleRec const  *GetExtentsRect();
    AmDisplayClass	*GetDisplay();
    AmWindowClass		*GetWindow();
    virtual void	SetRect( int	x,
				 int	y,
				 int	width,
				 int	height);
    virtual void	SetWindowSize( int	winWidth,
				       int	winHeight);

    AmErrorCode	Draw( AmWindowClass        *w,
                      AmGCClass            *gc,
                      AmRectangleRec const *vrctExposeRects,
                      int                nExposeRects);
    virtual AmErrorCode draw( AmWindowClass        *w,
                              AmGCClass            *gc,
                              AmRectangleRec const *vrctExposeRects,
                              int                nExposeRects);

    virtual void	DrawOutline( AmWindowClass		*w,
				     AmGCClass			*gc,
				     AmRectangleRec const	*exposeRect);
    void 		SetRectAndMargins( int	x,
					   int	y,
					   int	width,
					   int	height,
					   int	winWidth,
					   int	winHeight);
    virtual AmErrorCode	Update( int	*eventCodes = NULL,
				int	numCodes = 0);
    AmObjectTypeEnum	GetObjectType();
    const char		*GetName();
    int 		ParseColorCellArgs( AmFieldDefPtr     fieldDefs,
					    int             numFieldDefs,
					    AmTagValuePtr     tagValues,
					    int             numValues);
    const char		*getObjectName();
    const char		*getObjectTypeName();
    virtual AnvilBoolean	DumpFields( FILE *fp);
    AmScreenObjClass	*getParent();
    AnvilBoolean		getOpaque();
    void		setParent( AmScreenObjClass	*parentArg);
};

void deleteChildObjects( int,                    /* eventId */
                         AmScreenObjClass          *obj,
                         AmScreenObjClass          *parentCandidate);

#endif /* _anvil_screenobj_h_ */
