#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_tagvalues_h_
#define _anvil_tagvalues_h_
#include <stdarg.h>
#include <stdio.h>
#include <anvil/project.h>
#include <anvil/datatype.h>
#include <anvil/units.h>
#include <anvil/resource.h>
#include <anvil/register.h>
#include <anvil/changed.h>
#include <anvil/getset.h>
#include <anvil/header.h>

#define FILE_PREFIX	"file."

/* argument list set up in a tag value pair */
 
typedef struct AmTagValue_ {
    char        *tag;			/* string tag */
    void        *value;			/* value to set */
    AnvilBoolean	copied;			/* value is copied */
    AnvilBoolean	tagCopied;		/* whether tag is copied */
} AmTagValueRec, *AmTagValuePtr;

void AmChangeGetToSet( AmTagValuePtr	tagValues,
		     int		numValues);

int AmMakeTagValueArrayVararg( AmTagValuePtr	tagValues,
			     va_list		args);

int AmMakeTagValueArrayVararg( AmTagValuePtr	tagValues,
			     const char		*tag,
			     va_list		args);

int AmHeaderStringsTagValueArray( char		**table,
				AmTagValuePtr	tagValues);

int AmHeaderTagValueArray( char		*file,
			 AmTagValuePtr	tagValues);

AmDataTypeEnum AmGetTagType( const char  *tag);

AnvilBoolean AmGetTagValueFromTags( AmTagValuePtr	tagValues,
			     int		numValues,
			     const char		*tag,
			     void		**value);

AmStrdefPtr AmGetStrdef( const char *tag);

AmStrdefPtr AmGetLiteralStrdef( const char *tag);

void freeCopiedValues( AmTagValuePtr	tagValues,
		       int		numValues);

void copyTagValues( AmTagValuePtr	src,
		    AmTagValuePtr	dst,
		    int		numValues);

void AmSetBinvec( float           **binvec,
                int             length,
                AmTagValuePtr     tagValue);

AnvilBoolean AmMatchTag( const char 	*tag1,
		  const char	*tag2);

AnvilBoolean AmPrefixTag( const char	*prefix,
		   const char	*tag);

AnvilBoolean AmRemoveTagValue( char		*tag,	
			AmTagValuePtr	tagValues,
			int		*numValues);

AnvilBoolean AmIsAtomic( AmDataTypeEnum dataType);


AmErrorCode AmAddTagType( const char      *tag,
                        AmDataTypeEnum    dataType);

AmErrorCode AmCreateLookupTable();


#endif /* _anvil_tagvalues_h_ */
