#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_unitsP_h_
#define _anvil_unitsP_h_

#include <anvil/project.h>
#include <anvil/expr.h>

typedef enum {
    AmUNIT_ERROR = 0,
    AmUNIT_NAME = 1,
    AmUNIT_ALIAS = 2
} AmUnitTypeEnum;

typedef struct unitName_ unitNameRec, *unitNamePtr;

typedef void ( *AmPrintFn)( float, char *);
typedef AnvilBoolean ( *AmScanFn)( char *, float *);

/* definition for a unit, including enumeration code, precision and print 
   function */

typedef struct unitDef_ {
    unitType	code;			/* code # for the unit */
    int		precision;		/* relative precision to print with */
    char	*string;		/* string to print for units */
    AmPrintFn	printFunc;		/* custom print function */
    AmScanFn	scanFunc;		/* custom scan function */
    char	*regexp;		/* non-default regexp template */
} AmUnitDefRec, *AmUnitDefPtr;

struct unitName_ {
    AmUnitTypeEnum	kind;		/* direct name or alias */
    char		*name;		/* string name */
    union {	
	AmUnitDefRec	unitDef;	/* unit definition */
	unitNamePtr	alias;		/* pointer to aliased unit */
    } u;
};

typedef struct unitConvert_ {
    unitType	sourceUnits;		/* source and destination unit codes */
    unitType	destUnits;	
    AmExprPtr	conversion;		/* conversion expression */
} AmUnitConvertRec, *AmUnitConvertPtr;

typedef struct varDef_ {
    varType	variableCode;		/* variable code */
    char	*variableName;		/* string name */
    unitType	unitCode;		/* units to display variable in */
} AmVarDefRec, *AmVarDefPtr;

void AmAddUnitName( char	*name);

void AmAddUnitAlias( char	*alias, 
		   char *name);

void AmAddPrecision( char *name, 
		   int precision);

void AmAddPrint( char *name, 
	       char *printFnName);

void AmAddScan( char *name, 
	      char *scanFnName);

void AmAddString( char *name, 
	        char *str);

void AmAddString( char *name);

void AmAddConvert( char 	*sourceName,
		   char 	*destName,
		   AmExprPtr	expr);

AnvilBoolean convertValue( unitType	sourceCode,
		      unitType	destCode,
		      float	value,
		      float	*convertValue);

int AmUnitsFpYYinput( char	*buffer,
		    int		maxSize);

AnvilBoolean AmAddVariableUnits( char	*varStr,
			  char	*unitStr);

void AmAddRegexp( char	*varStr,
		char	*unitStr);

#endif /* _anvil_unitsP_h_ */
