#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_userevents_h_
#define _anvil_userevents_h_

#if defined( _WIN32)
#include <windows.h>

/* this is normally defined in va_cback.h, but those functions (for the
   moment) aren't defined for NT */

typedef void (*VarargsCallback)(...);
#else
#include <X11/Intrinsic.h>
#include <anvil/va_cback.h>
#endif

typedef void (*callbackProc)( int , void *, void *);

/* register an interest in a user event */

AnvilBoolean AmAddCallback( int		packageId,
		     int		eventId,
		     callbackProc	proc,
		     void		*clientData);

/* remove an interest in a user event */

AnvilBoolean AmRemoveCallback( int		packageId,
			int		eventId,
			callbackProc	proc,
			void		*clientData);

/* send a user event to be immediately processed by all functions which
   have registered an interest, meaning that the call data doesn't have
   to be allocated */

AnvilBoolean AmSendUserEvent( int	packageId,
		       int 	eventId,
		       void	*callData);

#if defined( _WIN32)

AnvilBoolean AmSendUserEventChildren( int	packageId,
			       int 	eventId,
			       void	*callData,
			       HWND	hWnd);

AnvilBoolean addCallbackWindow( int		packageId,
			   int		eventId,
			   HWND		hWnd,
			   callbackProc	proc,
			   void		*clientData);

AnvilBoolean removeCallbackWindow( int		packageId,
			      int		eventId,
			      callbackProc	proc,
			      void		*clientData,
			      HWND		hWnd);

AnvilBoolean addVaCallbackWindow( int		packageId,
			     int		eventId,
			     HWND		hWnd,
			     VarargsCallback	proc, 
			     ...);


AnvilBoolean removeVaCallbackWindow( int		packageId,
				int		eventId,
				HWND		hWnd,
				VarargsCallback	proc, 
				...);

#else		/* the wonderful world of UNIX/X!!! */

/* register an interest in an event to be called just once */

AnvilBoolean AmAddCallbackOnce( int		packageId,
			 int		eventId,
			 callbackProc	proc,
			 void		*clientData);

/* send a user event to handlers which are associated with descendents of
   another widget */

AnvilBoolean AmSendUserEventChildren( int	    packageId,
			       int          eventId,
			       void         *callData,
			       Widget       w);

/* tie a callback to an event for a one-time call, but set it up so it's 
   removed if the widget is destroyed */
 
AnvilBoolean AmAddCallbackWidgetOnce( int		packageId,
			       int          	eventId,
			       Widget       	w,
			       callbackProc	proc,
			       void         *clientData);

AnvilBoolean AmAddCallbackWidget( int	    	packageId,
			   int      	eventId,
			   Widget   	w,
			   callbackProc	proc,
			   void         *clientData);

AnvilBoolean AmRemoveCallbackWidget( int	    packageId,
			      int           eventId,
			      callbackProc  proc,
			      void          *clientData,
			      Widget        w);

/* varargs variants of the user event callback functions */

AnvilBoolean AmAddVaCallbackWidget( int                packageId,
                             int                eventId,
                             Widget             w,
                             VarargsCallback    proc,
                             ...);


AnvilBoolean AmRemoveVaCallbackWidget( int             packageId,
                                int             eventId,
                                Widget          w,
                                VarargsCallback proc,
                                ...);

/* (de) sensitize buttons on the callback */

void AmSensitizeCback( int        event,
                     Widget     w,
                     int        sensFlag);

void AmActiveCback( int           event,
		  Widget     	w,
		  XtPointer	callData);

void AmSetLabelCback( int         event,
		    Widget      w,
		    char        *label);

void AmDestroyDlogCback( int              event,
                       Widget           dlog,
                       XtPointer        callData);

#endif


#endif /* _anvil_userevents_h_ */
