#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_xdisplay_h_
#define _anvil_xdisplay_h_

#include <sys/types.h>
#include <X11/Intrinsic.h>
#include <anvil/fonts.h>
#include <anvil/display.h>
#include <anvil/shmimg.h>

#define FONT_NAME_SIZE	128

typedef enum windowType {
    AmWINDOW_ERROR = 0,
    AmWINDOW_WIDGET = 1,
    AmWINDOW_WINDOW = 2,
    AmWINDOW_PIXMAP = 3
} AmWindowTypeEnum;

class AmXDisplayClass;

class AmXWindowClass : public AmWindowClass {

protected:
    Window	   _xWindow;
    Widget	   _xWidget;
    AmXDisplayClass  *_pXDisplay;

    AmWindowTypeEnum _eWindowType;
    AmColorCellRec   _colBackground;    /* For clearing pixmap. */

public:

    /* Constructors/Destructors. */

    AmXWindowClass();
    AmXWindowClass( AmXDisplayClass *pXDisplay,
                  Widget        w);
    AmXWindowClass( AmXDisplayClass *pXDisplay,
		  Pixmap        xPixmap,
		  AmColorCellPtr  pcolBackground);
    AmXWindowClass( AmXDisplayClass *pXDisplay,
		  Window        xWindow);
    virtual ~AmXWindowClass();
    virtual AnvilBoolean	IsValid();

/* GC's have to be created with a window under windows NT, hence the 
   window-restriction... */

    virtual AmGCClass *CreateGC();

    /* Accessor functions. */
    
    Window       GetXWindow();
    Widget	 GetXWidget();
    virtual void GetDims( int *width, int *height);
    virtual void GetWindowGeometry( int *x,
                                    int *y,
                                    int *width,
                                    int *height);

/* for retrieving the display the window was created with */

    virtual AmDisplayClass	*GetDisplay();

    /* Mutator functions. */

    virtual void ClearRect( AmRectangleRec const *rect);    
    virtual void ResizeWindow( int	cx,
			       int	cy);
    virtual void MoveWindow( int	x,
			     int	y);
    virtual void Update();
};

class AmXDisplayClass : public AmDisplayClass {

protected:
    Display  *_pxDisplay; /* Server connection */
    Colormap _xColormap;
    Widget   _wShell;    /* Widget for the display. */

public:

    /* Constructors/Destructors. */
    
    AmXDisplayClass();
    AmXDisplayClass( Display *displayArg);
    AmXDisplayClass( Widget  shellArg);
    virtual ~AmXDisplayClass();

    /* Accessor functions */
    
    Colormap    GetXColormap();
    Display     *GetXDisplay();
    Visual      *GetXVisual();
    virtual int GetDepth();

    /* Drawing functions. */

    virtual void DrawLines( AmWindowClass *pWindow, 
			    AmGCClass 	*pGC, 
			    AmPointPtr 	pPoint, 
			    int 	nPoint);
    virtual void DrawPoints( AmWindowClass *pWindow, 
			     AmGCClass     *pGC, 
			     AmPointPtr    pPoint, 
			     int	 nPoint);
    virtual void DrawSegments( AmWindowClass *pWindow, 
			       AmGCClass 	   *pGC, 
			       AmSegmentRec  *pSegment,
			       int 	   nSegment);
    virtual void FillArc( AmWindowClass *pWindow, 
			  AmGCClass     *pGC, 
			  int	      x, 
			  int	      y, 
			  int	      w, 
			  int	      h, 
			  float	      dAngleStart,
			  float	      dAngleEnd);
    virtual void DrawArc( AmWindowClass *pWindow,
			  AmGCClass     *pGC, 
			  int	      x, 
			  int	      y, 
			  int	      w, 
			  int	      h, 
			  float	      dAngleStart,
			  float	      dAngleEnd);
    virtual void DrawImage( AmWindowClass	*pWindow,
			    AmGCClass	*pGC,
			    int		xSrc,
			    int		ySrc,
			    int		xDest,
			    int		yDest, 
			    int		iWidthSrc,
			    int		iHeightSrc,
			    AmImageClass	*pImage);
    virtual void DrawGlyph( AmWindowClass	*pWindow,
			    AmGlyphClass	*pGlyph,
			    int		x,
			    int		y);
    virtual void GetImage( AmWindowClass  *w,
                           AmImageClass   *image,
                           int          x,
                           int          y);
    virtual void CopyArea( AmWindowClass  *pWindowSrc,
			   AmWindowClass	*pWindowDest,
                           AmGCClass      *pGC,
                           int          xSrc,
                           int          ySrc,
                           int          xDest,
                           int          yDest,
                           int          iWidthSrc,
                           int          iHeightSrc);
    virtual void DrawText( AmWindowClass	*pWindow,
			   AmGCClass	*pGC,
			   int		x, 
			   int		y,
			   char		*szText);
    virtual void DrawImageText( AmWindowClass *pWindow,
			        AmGCClass	    *pGC,
			        int	    x,
			        int	    y,
			        char	    *szText);
    virtual void DrawRotatedText( AmWindowClass *pWindow,
				  AmGCClass     *pGC,
				  int	      x, 
				  int	      y,
				  float	      dAngleRadians,
				  char	      *szText);
    virtual void ExtentsText( AmGCClass  *pGC,
			      char     *szText,
			      int      *piWidth,
			      int      *piAscent,
			      int      *piDescent);
    virtual void FillPolygon( AmWindowClass *pWindow,
			      AmGCClass	  *pGC,
			      AmPointPtr  pPoint,
			      int	  nPoint);
    virtual void DrawRectangle( AmWindowClass *pWindow,
				AmGCClass	    *pGC,
				int	    x, 
				int	    y, 
				int	    w, 
				int	    h);
    virtual void FillRectangle( AmWindowClass *pWindow,
				AmGCClass	    *pGC,
				int	    x, 
				int	    y, 
				int	    w, 
				int	    h);

    /* Misc functions. */
    
    virtual AnvilBoolean AllocColor( AmColorCellPtr cell);
    virtual AnvilBoolean AllocNamedColor( const char   *szName,
				     AmColorCellPtr pCol);
  
    int TextWidth( AmGCClass *pGC,
		   char    *szText,
		   int     iLength);
    int FontHeight( AmGCClass *pGC,
		    int     *iAscent,
		    int     *iDescent);

    /* Creation functions. */
    
    virtual AmImageClass *CreateImage( int     iWidth,
				     int     iHeight,
				     int     iDepth,
				     char    *pcData);
    virtual AmGlyphClass *CreateGlyph( AmWindowClass *pWindow,
				     char	 *szSymfile,
				     ulong_t	 pixel);
    virtual AmWindowClass *CreateBuffer( AmWindowClass *pWindow,
                                       int         cx,
                                       int         cy);
    virtual AmWindowClass *CreateWin( AmWindowClass *pWindow,
                                    int         x,
                                    int         y,
                                    int         cx,
                                    int         cy);
    virtual AmRegionClass *CreateRegion();
};


class AmXGCClass : public AmGCClass {

protected:
    GC		   	_xGC;
    XFontStruct    	*_pxFont;
    AmXDisplayClass  	*_pXDisplay;
    Pixmap		_stipplePixmap;

    float          	_dLineWidth;
        
    AmRectanglePtr   	_pClipRectangle; /* X gc doesn't save clip rectangle. */
    int		   	_nClipRectangle;
    char	   	*_pcDashPattern; /* X gc doesn't save dash pattern. */
    int		   	_nDashPattern;
    int		   	_iDashOffset;

    AmFontRec        	_font;

    AmColorCellRec   	_colForeground;
    AmColorCellRec   	_colBackground;
    AmRegionClass	*clipRegion;
    
public:

    /* Constructors/Destructors. */
    
    AmXGCClass( AmXWindowClass	*pXWindow);
    virtual ~AmXGCClass();

    /* Accessor functions. */

    GC                    GetXGC();
    XFontStruct           *GetXFont();

    virtual void          GetForegroundColor( AmColorCellPtr pCol);
    virtual void          GetBackgroundColor( AmColorCellPtr pCol);
    virtual void          GetFont( char           **pszName, 
                                   int            *piSize,
                                   AmFontWeightEnum *peWeight = NULL,
                                   AmFontSlantEnum  *peSlant = NULL,
                                   float          *pfxScale = NULL,
                                   float          *pfyScale = NULL);
    virtual void          GetFont( AmFontRec *pFont);
    virtual float         GetLineWidth();
    virtual AmLineJoinEnum  GetLineJoin();    
    virtual AmLineStyleEnum GetLineStyle();
    virtual AmLineCapEnum   GetLineCap();
    virtual AmRasterOpEnum  GetRasterOp();
    virtual AnvilBoolean         SetFillPattern( const char *fillPatFile);
    virtual char	    *GetFillPattern();
    virtual void	    SetFillStyle( AmFillStyleEnum fillStyle);
    virtual AmFillStyleEnum GetFillStyle();
    virtual void          GetClipRectangles( AmRectanglePtr *pClipRectangle,
				             int	  *nRectangle);
    virtual void          GetDashes( char **ppcDashPattern, 
			             int  *pnDashPattern,
			             int  *piDashOffset = NULL);
    /* Mutator functions. */
    
    virtual void SetForegroundColor( AmColorCellPtr pCol);
    virtual void SetBackgroundColor( AmColorCellPtr pCol);
    virtual AnvilBoolean SetFont( char           *szName, 
		  	     int            iSize,
			     AmFontWeightEnum eWeight = AmWEIGHT_MEDIUM,
			     AmFontSlantEnum  eSlant = AmSLANT_ROMAN,
			     float          dxScale = 1.0,
			     float          dyScale = 1.0);
    virtual AnvilBoolean SetFont( AmFontRec *pFont);
    virtual void SetLineWidth( float dLineWidth);
    virtual void SetLineJoin( AmLineJoinEnum eLineJoin);
    virtual void SetLineStyle( AmLineStyleEnum eLineStyle);
    virtual void SetLineCap( AmLineCapEnum eLineCap);
    virtual void SetClipRectangles( AmRectangleRec const  *pClipRectangle,
				    int			nClipRectangle);
    virtual void SetClipRegion( AmRegionClass	*region);
    virtual AmRegionClass *GetClipRegion() { return( clipRegion); };
    virtual void SetRasterOp( AmRasterOpEnum eRasterOp);
    virtual void SetDashes( char *pcDashPattern, 
			    int	 nDashPattern,
			    int	 iDashOffset = 0);

    virtual void Update();
};

class AmXImageClass : public AmImageClass {

private:

    XImage		*_pxImage;
    XShmSegmentInfo	*_pShmSegment;
    char		*_pcInternal;

    AnvilBoolean copyDataPseudoColor();
    AnvilBoolean copyDataTrueColor();
    AnvilBoolean copyData8Bit();
    AnvilBoolean revCopyDataPseudoColor();
    AnvilBoolean revCopyData8Bit();
    AnvilBoolean revCopyDataTrueColor();

public: 
    XImage		*GetXImage();
    XShmSegmentInfo	*getShmInfo();
    AmXImageClass( AmDisplayClass	*pDisplay,
		 int		cxVirtual,
		 int		cyVirtual,
		 int		iDepth,
		 char		*pcData);
    virtual ~AmXImageClass();

/* Accessor functions. */

    virtual char *AmGetData();

/* Mutator functions. */

    AnvilBoolean CopyData();
    AnvilBoolean RevCopyData();
    virtual AnvilBoolean SetData( char *data);
};

class AmXGlyphClass : public AmGlyphClass {

private:
    Display	*xdisp;
    Pixmap	pixmap;
    Pixmap	mask;

public:
    AmXGlyphClass( AmDisplayClass  *display,
		 AmWindowClass   *window,
		 char          *symfile,
		 ulong_t        color);
    AmXGlyphClass( Display	*display,
		 Pixmap		pixmap,
		 Pixmap		mask,
		 int		width,
		 int		height,
		 int		depth,
		 ulong_t	color);
    virtual void DrawGlyph( AmDisplayClass	*display,
			    AmWindowClass		*window,
			    int			x,
			    int			y);
    virtual AmGlyphClass	*ResizeGlyph( int	newWidth,
				      int	newHeight);
    virtual	~AmXGlyphClass();
    Pixmap	getPixmap();
    Pixmap	getMask();
};

class AmXRegionClass : public AmRegionClass {
private:
    Region	xRegion;
public:
    Region		getXRegion();
    			AmXRegionClass( AmDisplayClass *display);
			AmXRegionClass( Region region);
    virtual 		~AmXRegionClass();
    virtual AnvilBoolean 	PointInRegion( int  x,
				       int  y);
    virtual AnvilBoolean 	EqualRegion( AmRegionClass    *otherRegion);
    virtual void	AddPolygon( AmPointPtr        pt,
				    int             n,
				    AmFillRuleEnum    fillRule);
    virtual void	AddRectangle( AmRectangleRec const *rect);
    virtual void	UnionRegion( AmRegionClass    *otherRegion);
    virtual void	IntersectRegion( AmRegionClass        *otherRegion);
    virtual void	XorRegion( AmRegionClass      *otherRegion);
    virtual void	SubtractRegion( AmRegionClass *otherRegion);
    virtual AmRegionClass	*CopyRegion();
};


#endif /* _anvil_xdisplay_h_ */
