#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _editor_align_dialog_class_h_
#define _editor_align_dialog_class_h_

#include <anvil/xdisplay.h>
#include <anvil/screenobj.h>

#include <ToolKit/DialogClass.h>
#include <ToolKit/FormClass.h>
#include <ToolKit/PaneClass.h>
#include <ToolKit/RadioPaneClass.h>
#include <ToolKit/FrameClass.h>
#include <ToolKit/DrawingAreaClass.h>
#include <ToolKit/SeparatorClass.h>
#include <ToolKit/LabelClass.h>
#include <ToolKit/ToggleButtonClass.h>
#include <ToolKit/TextFieldClass.h>

/* Forward reference. */

class AmEditorClass;

typedef enum {
    AmALIGN_OBJECTS_ERROR = 0,
    AmALIGN_OBJECTS_EACH_OTHER = 1,
    AmALIGN_OBJECTS_SEQUENTIAL = 2,
    AmALIGN_OBJECTS_PAGE = 3,
} AmAlignObjectsEnum;

class AmAlignDialogClass : public AmDialogClass {

private:

    void         initializeValues();
    void         createInterface();

    static void AmToggleCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    void toggle( Widget    w,
                 XtPointer xpCallData);

    static void AmTextCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData);
    void text( Widget    w,
               XtPointer xpCallData);

    static void AmExposeCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    void expose( Widget    w,
                 XtPointer xpCallData);

    static void AmRedrawCallback( int       iID,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    void redraw( int       iID,
                 XtPointer xpCallData);

    static void AmApplyCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    void apply( Widget    w,
                XtPointer xpCallData);

    void         alignRect( int                left,
                            int                top,
                            int                right,
                            int                bottom,
                            const AmRectangleRec *pRectSrc,
                            AmRectangleRec       *pRectDst);
    AmRectangleRec *createAlignRects( AmScreenObjClass **vpScreenObj,
                                    int            nScreenObj);

protected:

    AmLabelClass        *_pwcLabel;
    AmSeparatorClass    *_pwcTopSeparator;
    AmSeparatorClass    *_pwcSeparator;
    AmFormClass         *_pwcVertForm;
    AmRadioPaneClass    *_pwcRadioPane;

    AmToggleButtonClass *_pwcObjects;
    AmToggleButtonClass *_pwcSequential;
    AmToggleButtonClass *_pwcPage;

    AmFormClass         *_pwcHorizForm;
    AmFrameClass        *_pwcFrame;
    AmDrawingAreaClass  *_pwcArea;

    AmPaneClass         *_pwcVertPane;

    AmToggleButtonClass *_pwcTop;
    AmToggleButtonClass *_pwcVertCenter;
    AmToggleButtonClass *_pwcBottom;

    AmPaneClass         *_pwcHorizPane;

    AmToggleButtonClass *_pwcLeft;
    AmToggleButtonClass *_pwcHorizCenter;
    AmToggleButtonClass *_pwcRight;

    AmXDisplayClass     *_pXDisplay;
    AmXGCClass          *_pXGC;
    AmXWindowClass      *_pXWindow;

    AmFormClass         *_pwcVMarginForm;
    AmTextFieldClass    *_pwcVMarginText;

    AmFormClass         *_pwcHMarginForm;
    AmTextFieldClass    *_pwcHMarginText;

    AmPaneClass         *_pwcVertSeqPane;
    AmToggleButtonClass *_pwcVert;

    AmFormClass         *_pwcVertSeqForm;
    AmTextFieldClass    *_pwcVertSeqText;

    AmPaneClass         *_pwcHorizSeqPane;
    AmToggleButtonClass *_pwcHoriz;

    AmFormClass         *_pwcHorizSeqForm;
    AmTextFieldClass    *_pwcHorizSeqText;

    AmAlignObjectsEnum  _eAlignType;

    float             _dHorizSpacing;
    float             _dVertSpacing;

    float             _dHorizMargin;
    float             _dVertMargin;
 
    AmColorCellRec      _colBackground;

    float             _dWidth;
    float             _dHeight;
    int               _iMargin;

    Display           *_pDisplay;
    AmEditorClass       *_pEditor;

    void              draw();

public:

    /* Constructors/Desctructors. */

    AmAlignDialogClass();
    AmAlignDialogClass( Widget      wParent,
                      const char  *szName,
                      AmEditorClass *pEditor);
    AmAlignDialogClass( AmBaseWidgetClass *bwcParent,
                      const char      *szName,
                      AmEditorClass     *pEditor);
    virtual ~AmAlignDialogClass();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "AmAlignDialogClass"); };

    /* Mutator functions. */

};

#endif /* _editor_align_dialog_class_h_ */
