#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _editor_editor_class_h_
#define _editor_editor_class_h_

#include <anvil/motifdisplay.h>
#include <anvil/coordinates.h>
#include <anvil/question_dlog.h>

#include <anvil/PickLayerDialogClass.h>
#include <anvil/EditLayerDialogClass.h>
#include <anvil/DeleteLayerDialogClass.h>

#include <ToolKit/BaseWidgetClass.h>
#include <ToolKit/MenuClass.h>
#include <ToolKit/MenuPaneClass.h>
#include <ToolKit/PopupMenuClass.h>
#include <ToolKit/ToolBarClass.h>
#include <ToolKit/RulerClass.h>
#include <ToolKit/ScrolledTextClass.h>
#include <ToolKit/ProgressBarClass.h>

#include <editor/OutlineClasses.h>
#include <editor/SelectObjClass.h>
#include <editor/EditorToolBarClass.h>
#include <editor/PageSetupDialogClass.h>
#include <editor/TextDialogClass.h>
#include <editor/AlignDialogClass.h>
#include <editor/LegendDialogClass.h>

#define AM_HEADER_STRING        "HEADER"
#define AM_END_HEADER_STRING    "END_HEADER"
#define AM_PAGE_WIDTH_STRING    "PAGE_WIDTH"
#define AM_PAGE_HEIGHT_STRING   "PAGE_HEIGHT"
#define AM_SCREEN_DPI_STRING    "SCREEN_DPI"
#define AM_PRINT_DPI_STRING     "PRINT_DPI"
#define AM_LANDSCAPE_STRING     "LANDSCAPE"
#define AM_TRUECOLOR_STRING     "TRUECOLOR"
#define AM_PROLOG_FILE_STRING   "PROLOG_FILE"

// For now the page dimensions can be anywhere from 0 to 100 inches.

#define AM_MIN_PAGE_WIDTH   0.0
#define AM_MAX_PAGE_WIDTH   100.0

#define AM_MIN_PAGE_HEIGHT  0.0
#define AM_MAX_PAGE_HEIGHT  100.0

// The screen resolution (what zoom level the map is at from 25% to 400%) 

#define AM_MIN_SCREEN_DPI   18
#define AM_MAX_SCREEN_DPI   288

// The print resolution is currently fixed at 72

#define AM_MIN_PRINT_DPI    72
#define AM_MAX_PRINT_DPI    72

// Booleans can be either false (0) or true (1)

#define AM_MIN_BOOLEAN      0
#define AM_MAX_BOOLEAN      1


#define EDITOR_STRING "EditorUserData"

typedef AnvilBoolean ( * AmCallbackFn)( Widget            w,
                                 AmMotifDisplayClass *pMotifDisplay,
                                 AmScreenObjClass    *pScreenObj,
                                 XtPointer         xpCallData);

typedef enum outputType_ {
    AmOUTPUT_ERROR = 0,
    AmOUTPUT_POSTSCRIPT = 1,
    AmOUTPUT_TIFF = 2
} AmOutputTypeEnum;

class AmEditorClass : public AmBaseWidgetClass {

#ifdef PSCOPE
friend class AmEditorApplicationClass;
#endif

#ifdef LISTEN_SIGNAL
friend class AmEditorApplicationClass;
#endif

private:

/* Initialization and creation functions. */

    void initializeValues(); 
    void createInterface();

/* Helper functions. */

    AmAnswerEnum AskForSaveBeforeChange( char           *szQuestion,
                                       Widget         w,
                                       XtCallbackProc xcp,
                                       XtPointer      xpCallData);

    AnvilBoolean ObjectSelected( AmObjectTypeEnum eObjectType);

/* Master handler functions. */

    static void AmRemoveMasterCallback( XtPointer xpClientData);
    void removeMaster();
    static void AmSuspendMasterCallback( XtPointer xpClientData);
    void suspendMaster();
    static void AmRestartMasterCallback( XtPointer xpClientData);
    void restartMaster();
 
    static void AmMasterHandler( Widget    w,
                               XtPointer xpClientData,
                               XEvent    *pxEvent,
                               AnvilBoolean   *pfContinueDispatch);
    void master( Widget  w, 
                 XEvent  *pxEvent,
                 AnvilBoolean *pfContinueDispatch);

    /* File menu callbacks. */

    static void AmPageSetupCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData);
    void pageSetup( Widget    w,
                    XtPointer xpCallData);

    static void AmPageSetupOkCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void pageSetupOk( Widget    w,
                      XtPointer xpCallData);

    static void AmPageSetupCancelCallback( Widget    w,
                                         XtPointer xpClientData,
                                         XtPointer xpCallData);
    void pageSetupCancel( Widget    w,
                          XtPointer xpCallData);

    static void AmQuitCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData);
    void quit( Widget    w,
               XtPointer xpCallData);

    /* Edit menu callbacks. */

    static void AmSelectAllCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void selectAll( Widget    w,
                    XtPointer xpClientData);

    static void AmSelectNoneCallback( Widget    w,
                                    XtPointer xpClientData, 
                                    XtPointer xpCallData);
    void selectNone( Widget    w,
                     XtPointer xpClientData);

    /* Insert menu callbacks. */

    /* View menu callbacks. */

    static void AmGridCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData);
    void grid( Widget    w,
               XtPointer xpCallData);

    static void AmRulerCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    void ruler( Widget    w,
                XtPointer xpCallData);

    /* Options menu callbacks. */

    static void AmMap25Callback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    static void AmMap33Callback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    static void AmMap50Callback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    static void AmMap100Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    static void AmMap125Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    static void AmMap150Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    static void AmMap175Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    static void AmMap200Callback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    void mapSize( Widget    w,
                  XtPointer xpCallData,
                  int       iSize);

    static void AmTextCallback( Widget    w,
                              XtPointer xpClientData,
                              XtPointer xpCallData);
    void text( Widget    w,
               XtPointer xpCallData);

    static void AmTextOkCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    void textOk( Widget    w,
                 XtPointer xpCallData);

    static void AmTextCancelCallback( Widget    w,
                                    XtPointer xpClientData,
                                    XtPointer xpCallData);
    void textCancel( Widget    w,
                     XtPointer xpCallData);

    static void AmAlignCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    void align( Widget    w,
                XtPointer xpCallData);

    static void AmAlignCancelCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void alignCancel( Widget    w,
                      XtPointer xpCallData);

    static void AmAboutCallback( Widget    w,
                                 XtPointer xpClientData, 
                                 XtPointer xpCallData);
    void about( Widget    w,
                XtPointer xpClientData);

    static void AmCutCallback( Widget    w,
                             XtPointer xpClientData, 
                             XtPointer xpCallData);
    void cut( Widget    w,
              XtPointer xpClientData);

    static void AmCopyCallback( Widget    w,
                              XtPointer xpClientData, 
                              XtPointer xpCallData);
    void copy( Widget    w,
               XtPointer xpClientData);

    static void AmPasteCallback( Widget    w,
                               XtPointer xpClientData, 
                               XtPointer xpCallData);
    void paste( Widget    w,
                XtPointer xpClientData);

    static void AmDelCallback( Widget    w,
                             XtPointer xpClientData, 
                             XtPointer xpCallData);
    void del( Widget    w,
              XtPointer xpClientData);

    static void AmPopTopCallback( Widget    w,
                                XtPointer xpClientData, 
                                XtPointer xpCallData);
    void popTop( Widget    w,
                 XtPointer xpClientData);

    static void AmPopCallback( Widget    w,
                             XtPointer xpClientData, 
                             XtPointer xpCallData);
    void pop( Widget    w,
              XtPointer xpClientData);

    static void AmPushBottomCallback( Widget    w,
                                    XtPointer xpClientData, 
                                    XtPointer xpCallData);
    void pushBottom( Widget    w,
                     XtPointer xpClientData);

    static void AmPushCallback( Widget    w,
                              XtPointer xpClientData, 
                              XtPointer xpCallData);
    void push( Widget    w,
               XtPointer xpClientData);

    static void AmEditCallback( Widget    w,
                              XtPointer xpClientData, 
                              XtPointer xpCallData);
    void edit( Widget    w,
               XtPointer xpCallData);

    static void AmFinalEditCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void finalEdit( Widget    w,
                    XtPointer xpCallData);

    static void AmZoom2xCallback( Widget    w,
                                XtPointer xpClientData, 
                                XtPointer xpCallData);
    static void AmZoomRectangleCallback( Widget    w,
                                       XtPointer xpClientData, 
                                       XtPointer xpCallData);
    static void AmZoomTextCallback( Widget    w,
                                  XtPointer xpClientData, 
                                  XtPointer xpCallData);
    static void AmUnzoomHalfCallback( Widget    w,
                                    XtPointer xpClientData, 
                                    XtPointer xpCallData);
    static void AmUnzoomCallback( Widget    w,
                                XtPointer xpClientData, 
                                XtPointer xpCallData);
    static void AmPanCallback( Widget    w,
                             XtPointer xpClientData, 
                             XtPointer xpCallData);
    static void refreshCallback( Widget    w,
                                 XtPointer xpClientData, 
                                 XtPointer xpCallData);

    AnvilBoolean callCallback( Widget         w,
                          AmCallbackFn     fnCallback,
                          XtPointer      xpCallData,
                          AmObjectTypeEnum eObjectType = AmCOORD,
                          char           *szResource = "operationFailed");

    static void AmRectangleCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void rectangle( Widget    w,
                    XtPointer xpCallData);

    static void circleCallback( Widget    w,
                                XtPointer xpClientData, 
                                XtPointer xpCallData);
    void circle( Widget    w,
                 XtPointer xpCallData);

    static void linesCallback( Widget    w,
                               XtPointer xpClientData, 
                                XtPointer xpCallData);
    void lines( Widget    w,
                XtPointer xpCallData);

    static void polygonCallback( Widget    w,
                                 XtPointer xpClientData, 
                                XtPointer xpCallData);
    void polygon( Widget    w,
                  XtPointer xpCallData);

    static void AmTickmarksCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void tickmarks( Widget    w,
                    XtPointer xpCallData);

    static void AmDistanceScaleCallback( Widget    w,
                                       XtPointer xpClientData, 
                                       XtPointer xpCallData);

    void distanceScale( Widget    w,
                        XtPointer xpCallData);

    static void AmAddGridCallback( Widget    w,
                                 XtPointer xpClientData, 
                                 XtPointer xpCallData);

    void addGrid( Widget    w,
                  XtPointer xpCallData);

    static void AmNewMapCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
    void newMap( Widget    w,
                 XtPointer xpCallData);
    static void AmLoadMapCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void loadMap( Widget    w,
		  XtPointer xpCallData);
    static void AmLoadMapOkCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData);
    void loadMapOk( Widget    w,
                    char      *szFileName);

    static void AmSaveMapAsTiffCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData);
    static void AmSaveMapAsTiffOkCallback( Widget    w,
                                         XtPointer xpClientData,
                                         XtPointer xpCallData);
    void saveMapAsTiffOk( Widget    w,
                          XtPointer xpCallData);

    static void AmSaveMapAsPostscriptCallback( Widget    w,
                                             XtPointer xpClientData,
                                             XtPointer xpCallData);
    static void AmSaveMapAsPostscriptOkCallback( Widget    w,
                                               XtPointer xpClientData,
                                               XtPointer xpCallData);
    void saveMapAsPostscriptOk( Widget    w,
                                XtPointer xpCallData);

    static void AmSaveMapAsCallback( Widget    w,
                                   XtPointer xpClientData,
                                   XtPointer xpCallData);
    void saveMapAs( Widget    w,
                    XtPointer xpCallData);

    static void AmSaveMapCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);

    void saveMap( Widget         w,
                  XtCallbackProc callback,
                  XtPointer      xpCallData);
    static void AmSaveMapOkCallback( Widget         w,
                                   XtPointer      xpClientData,
                                   XtCallbackProc callback,
                                   XtPointer      xpCallData);
    void saveMapOk( Widget         w,
                    char           *szFileName,
                    XtCallbackProc callback);


    static void AmPrintMapCallback( Widget    w,
                                  XtPointer xpClientData,
                                  XtPointer xpCallData);
    void printMap( Widget    w,
                   XtPointer xpCallData);


    static void AmPrintMapPreviewCallback( Widget    w,
                                         XtPointer xpClientData,
                                         XtPointer xpCallData);
    void printMapPreview( Widget    w,
                          XtPointer xpCallData);


    static void AmImageCallback( Widget    w,
                               XtPointer xpClientData,
                               XtPointer xpCallData);
    void image( Widget    w,
                XtPointer xpCallData);
    static void AmLoadImageOkCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void loadImageOk( Widget    w,
                      char      *szFileName);

    static void AmAddObjectCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void AmAddObject( Widget    w,
                    XtPointer xpCallData);

    static void AmLoadOkCallback( Widget         w,
                                AmEditorClass    *pEditorClass,
                                AmScreenObjClass *pScreenObjParent,
                                XtPointer      xpCallData);
    void loadOk( Widget         w,
                 AmScreenObjClass *pScreenObjParent,
                 char           *szFileName);

    static AmScreenObjClass *AmCreateObject( AmWindowClass	*pWindow,
                                         AmScreenObjClass *pScreenObjClass,
                                         char           **pszHeaderStrings,
                                         XtPointer      xpClientData);

    static void AmAddLayerCallback( Widget    w,
                                  XtPointer xpClientData, 
                                  XtPointer xpCallData);
    void addLayer( Widget    w,
                   XtPointer xpCallData);

    static void AmSortLayerCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    static void AmDeleteLayerCallback( Widget    w,
                                     XtPointer xpClientData, 
                                     XtPointer xpCallData);
   void deleteLayer( Widget    w,
                     XtPointer xpCallData);

    static void AmAddLegendCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void addLegend( Widget    w,
                    XtPointer xpCallData);

    static void AmCreateLegendCallback( Widget    w,
                                      XtPointer xpClientData, 
                                      XtPointer xpCallData);
    void createLegend( Widget    w,
                       XtPointer xpCallData);

    static void AmRectChangedCallback( int       iID,
                                     XtPointer xpClientData, 
                                     XtPointer xpCallData);
    void rectChanged( int       iID,
                      XtPointer xpCallData);

    static void AmObjectDeletedCallback( int       iID,
                                       XtPointer xpClientData, 
                                       XtPointer xpCallData);
    void objectDeleted( int       iID,
                        XtPointer xpCallData);

    static void AmDisplayChangedCallback( int       iID,
                                          XtPointer xpClientData, 
                                          XtPointer xpCallData);
    void displayChanged( int       iID,
                         XtPointer xpCallData);

    /* File related functions. */

    AmErrorCode SaveObjectToFile( char           *szFileName,
                                  AmScreenObjClass *pScreenObj,
                                  AnvilBoolean        fAppend);
    AmListPtr   LoadObjectsFromFile( char *szFileName);

    void        WriteHeaderInfo( FILE *pFile);
    AnvilBoolean     ReadHeaderInfo( FILE *pFile);

    AmErrorCode SaveMapToFile( char *szFileName);
    AnvilBoolean     LoadMapFromFile( char *szFileName);

    AmErrorCode PrintMapToFile( char           *szFileName,
                                AmOutputTypeEnum eOutputType);

    AmErrorCode ScaleMap( int iDPI);


    static void AmImportMapCallback( Widget    w,
                                   XtPointer xpClientData, 
                                   XtPointer xpCallData);
    void importMap( Widget    w,
                    XtPointer xpCallData);

    static void AmImportMapOkCallback( Widget         w,
                                     AmEditorClass    *pEditorClass,
                                     XtPointer      xpCallData);
    void importMapOk( Widget         w,
                      char           *szFileName);
    
protected:

    AmMotifDisplayClass     *_pMotifDisplay;
    AmHighlightOutlineClass *_pOutline;

    AnvilBoolean               _fMapChanged;          /* TRUE if the map has been   
                                                    changed, since the AmLast    
                                                    new, open, or save.       */
    char                  *_szCurrentFile;       /* Name of the map file.     */
    char                  *_szTitle;             /* Title of the editor.      */

    AmListPtr               _lstSelectedObjects;   /* List of selected objects. */ 

    AnvilBoolean               _fGrid;                /* Show/Hide grid. */
    AnvilBoolean               _fSnapToGrid;
    int                   _dGridSpacing;

    float                 _dWidthInInches;
    float                 _dHeightInInches;

    int                   _iScreenDPI;
    int                   _iPrintDPI;

    AnvilBoolean               _fLandscape;
    AnvilBoolean               _fTrueColor;
 
    char                  *_szPrologFile;

    AnvilBoolean               _fClipFileValid;   /* Clipping. */
    char                  *_szClipFile;

    AnvilBoolean               _fStandalone;      /* If it's a standalone map editor,
                                                or if it is integrated into the
                                                application. */

    /* Interface widgets. */

    Widget                 _wForm;
    Widget                 _wMessageFrame;
    Widget                 _wMessage;

    Widget                 _wCornerFrame;
    Widget                 _wWindow;
    Widget                 _wArea;

    AmMenuClass              *_pwcMenu;
    AmMenuPaneClass          *_pwcFilePane;
    AmMenuPaneClass          *_pwcSaveAsPane;
    AmMenuPaneClass          *_pwcEditPane;
    AmMenuPaneClass          *_pwcPopPane;
    AmMenuPaneClass          *_pwcPushPane;
    AmMenuPaneClass          *_pwcInsertPane;
    AmMenuPaneClass          *_pwcAnnotationPane;
    AmMenuPaneClass          *_pwcLegendPane;
    AmMenuPaneClass          *_pwcToolPane;
    AmMenuPaneClass          *_pwcViewPane;
    AmMenuPaneClass          *_pwcZoomPane;
    AmMenuPaneClass          *_pwcHelpPane;

    AmPopupMenuClass         *_pwcPopupMenu;
    AmMenuPaneClass          *_pwcPopupEditPane;
    AmMenuPaneClass          *_pwcPopupSelectPane;

    AmEditorToolBarClass     *_pwcToolBar;
    AmEditorToolBarClass     *_pwcZoomUtilityBar;
    AmEditorToolBarClass     *_pwcFileUtilityBar;
    AmEditorToolBarClass     *_pwcLegendUtilityBar;
    AmEditorToolBarClass     *_pwcAnnotUtilityBar;

    AmRulerClass             *_pwcHorizontalRuler;
    AmRulerClass             *_pwcVerticalRuler;

    AmFormClass              *_pwcStatusForm;
    AmFrameClass             *_pwcMessageFrame;
    AmLabelClass             *_pwcTextMessage;
    AmProgressBarClass       *_pwcProgressBar;

    AmPageSetupDialogClass   *_pwcPageSetupDialog;
    AmTextDialogClass        *_pwcTextDialog;
    AmAlignDialogClass       *_pwcAlignDialog;

    AmXDisplayClass          *_pXDisplay;
    AmXWindowClass           *_pXWindow;

public:

    /* Constructors/Desctructors. */

    AmEditorClass();
    AmEditorClass( Widget          wParent,
                 const char      *szName);
    AmEditorClass( AmBaseWidgetClass *pwcParent,
                 const char      *szName);
    virtual ~AmEditorClass();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "EditorCLass"); };
    AmMotifDisplayClass  *GetMotifDisplay() { return( _pMotifDisplay); };
    AmListPtr            GetSelectedObjects() { return( _lstSelectedObjects); };
    AnvilBoolean            GetSnapToGrid() { return( _fSnapToGrid); };

    /* Mutator functions. */

    void SetMapChanged( AnvilBoolean fMapChanged) { _fMapChanged = fMapChanged; };

    /* Misc functions. */

    void SetTitle( const char *szTitle);

    /* Object selection functions. */

    AmErrorCode    SelectObject( AmScreenObjClass *pScreenObj);
    AmErrorCode    UnselectObject( AmScreenObjClass *pScreenObj);
    AmErrorCode    UnselectAll();
    AmScreenObjClass *FindSelectedObject( int x, int y);
    void           GetSelectedObjectsExtents( AmRectangleRec *pRect);
    void           GetSelectedObjectsRealExtents( AmRectangleRec *pRect);
    AnvilBoolean        IsObjectSelected( AmScreenObjClass *pScreenObj);
    void           MoveSelectedObjects( int xShift, int yShift);
    void           ResizeSelectedObjects( int x,
                                          int y,
                                          int iWidth,
                                          int iHeight);
    AmScreenObjClass **GetSelectedObjectsV();

    /* Object action functions. */

    AmActionEnum     FindAction( int x, int y, AmScreenObjClass **ppScreenObj);
    AmActionEnum     PositionToAction( int x, int y);
    int            ActionToCursor( AmActionEnum eAction);

    /* Misc functions. */

    void DoubleClick( AmScreenObjClass *pScreenObj);

    /* Functions needed by derived classes of editor. */

    virtual void   LoadObject( AmScreenObjClass *pScreenObjParent);
    AmScreenObjClass *CreateObject( AmWindowClass	 *pWindow,
                                  AmScreenObjClass *pScreenObjParent,
                                  char           **pszHeaderStrings);
    void           AddObject( AmScreenObjClass *pScreenObjNew);
};


AnvilBoolean AmIsMap( const char *szFilename);

#endif /* _editor_editor_class_h_ */
