#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _editor_grid_setup_dialog_class_h_
#define _editor_grid_setup_dialog_class_h_

#include <anvil/xdisplay.h>

#include <ToolKit/DialogClass.h>
#include <ToolKit/FormClass.h>
#include <ToolKit/FrameClass.h>
#include <ToolKit/OptionMenuClass.h>
#include <ToolKit/TextFieldClass.h>

class AmGridSetupDialogClass : public AmDialogClass {

private:

    void         initializeValues();
    void         createInterface();

    static void  AmExposeCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void  expose( Widget    w,
                  XtPointer xpCallData);

    static void  updateInchesTextCallback( Widget    w,
                                           XtPointer xpClientData,
                                           XtPointer xpCallData);
    void  updateInchesText( Widget    w,
                            XtPointer xpCallData);

    static void  updateInchesOptionCallback( Widget    w,
                                             XtPointer xpClientData,
                                             XtPointer xpCallData);
    void  updateInchesOption( Widget    w,
                              XtPointer xpCallData);

    static void AmToggleCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
 
    void toggle( Widget    w,
                 XtPointer xpCallData);

    void valuesChanged();

protected:

    AmFrameClass      *_pwcFrame;
    AmFormClass       *_pwcFrameForm;

    AmFormClass       *_pwcOptionForm;

    AmFormClass       *_pwcInchesOptionForm;
    Widget          _wInchesOptionLabel;
    AmComboBoxClass     *_pwcOptionMenu;

    AmFormClass       *_pwcInchesTextForm; 
    Widget          _wInchesTextLabel;
    AmTextFieldClass  *_pwcInchesText;

    AmFrameClass      *_pwcModeFrame;
    AmFormClass       *_pwcModeForm;
    Widget          _wModeLabel;
    Widget          _wModeRowColumn;
    Widget          _wGridOnToggle;
    Widget          _wGridOffToggle;

    AmFrameClass      *_pwcPreviewFrame;
    Widget          _wDrawingArea;

    AmDisplayClass    *_pDisplay;
    AmWindowClass     *_pWindow;
    AmGCClass         *_pGC;
 
    AmColorCellRec    _colBackground;
    AmColorCellRec    _colForeground;

    Display         *_display;
    Screen          *_screen;

    int             _iTextHeight;
    int             _iAscent;
    int             _iDescent;
    int             _iMargin;

    float           _dInches;    /* Grid spacing in inches. */ 

    float           _dWidth;     /* Width in inches. */
    float           _dHeight;    /* Height in inches. */

    AnvilBoolean         _fGrid;      /* Grid on/off. */


public:

    /* Constructors/Desctructors. */

    AmGridSetupDialogClass();
    AmGridSetupDialogClass( Widget     wParent,
                          const char *szName);
    AmGridSetupDialogClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName);
    virtual ~AmGridSetupDialogClass();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "AmGridSetupDialogClass"); };
    float              GetInches(); 
    float              GetWidth(); 
    float              GetHeight();
    AnvilBoolean            GetGrid();

    /* Mutator functions. */

    void               SetInches( float dInches);
    void               SetWidth( float dWidth);
    void               SetHeight( float dHeight);
    void               SetGrid( AnvilBoolean fGrid);
};

#endif /* _editor_grid_setup_dialog_class_h_ */
