#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _editor_page_setup_dialog_class_h_
#define _editor_page_setup_dialog_class_h_

#include <anvil/xdisplay.h>

#include <ToolKit/DialogClass.h>
#include <ToolKit/FormClass.h>
#include <ToolKit/FrameClass.h>
#include <ToolKit/LabelClass.h>
#include <ToolKit/ComboBoxClass.h>
#include <ToolKit/TextFieldClass.h>
#include <ToolKit/RadioPaneClass.h>
#include <ToolKit/ToggleButtonClass.h>
#include <ToolKit/DrawingAreaClass.h>

class AmPageSetupDialogClass : public AmDialogClass {

private:

    void         initializeValues();
    void         createInterface();

    static void  AmExposeCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void  expose( Widget    w,
                  XtPointer xpCallData);

    static void  AmUpdateWidthCallback( Widget    w,
                                      XtPointer xpClientData,
                                      XtPointer xpCallData);
    void  updateWidth( Widget    w,
                       XtPointer xpCallData);

    static void  AmUpdateHeightCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData);
    void  updateHeight( Widget    w,
                        XtPointer xpCallData);

    static void  AmUpdateSizeCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void  updateSize( Widget    w,
                      XtPointer xpCallData);

    static void AmToggleCallback( Widget    w,
                                XtPointer xpClientData,
                                XtPointer xpCallData);
 
    void toggle( Widget    w,
                 XtPointer xpCallData);

    void valuesChanged();

protected:

    AmFrameClass        *_pwcFrame;
    AmFormClass         *_pwcFrameForm;

    AmFormClass         *_pwcOptionForm;

    AmFormClass         *_pwcSizeForm;
    AmLabelClass        *_pwcSizeLabel;
    AmComboBoxClass     *_pwcOptionMenu;

    AmFormClass         *_pwcWidthForm; 
    AmLabelClass        *_pwcWidthLabel;
    AmTextFieldClass    *_pwcWidthText;

    AmFormClass         *_pwcHeightForm; 
    AmLabelClass        *_pwcHeightLabel;
    AmTextFieldClass    *_pwcHeightText;

    AmFrameClass        *_pwcModeFrame;
    AmFormClass         *_pwcModeForm;

    AmLabelClass        *_pwcModeLabel;
    AmRadioPaneClass    *_pwcModePane;

    AmToggleButtonClass *_pwcPortraitToggle;
    AmToggleButtonClass *_pwcLandscapeToggle;

    AmFrameClass        *_pwcPreviewFrame;
    AmDrawingAreaClass  *_pwcArea;

    AmFrameClass        *_pwcPrintFrame;
    AmFormClass         *_pwcPrintForm;

    AmLabelClass        *_pwcColorLabel;
    AmRadioPaneClass    *_pwcColorPane;

    AmToggleButtonClass *_pwcTrueColorToggle;
    AmToggleButtonClass *_pwcPseudoColorToggle;

    AmXDisplayClass     *_pXDisplay;
    AmXWindowClass      *_pXWindow;
    AmXGCClass          *_pXGC;
 
    AmColorCellRec      _colBackground;

    Display           *_display;
    Screen            *_screen;

    Pixmap            _pixPortrait;
    Pixmap            _pixLandscape;

    int               _iTextHeight;
    int               _iAscent;
    int               _iDescent;
    int               _iMargin;

    float             _dWidth;          /* Width in inches.                  */
    float             _dHeight;         /* Height in inches.                 */

    AnvilBoolean           _fLandscape;      /* Portrait vs. Landscape mode.      */
    AnvilBoolean           _fTrueColor;      /* True color vs. pseudo color mode. */

public:

    /* Constructors/Desctructors. */

    AmPageSetupDialogClass();
    AmPageSetupDialogClass( Widget     wParent,
                          const char *szName);
    AmPageSetupDialogClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName);
    virtual ~AmPageSetupDialogClass();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "AmPageSetupDialogClass"); };
    float              GetPageWidth(); 
    float              GetPageHeight();
    AnvilBoolean            GetLandscape();
    AnvilBoolean            GetTrueColor();

    /* Mutator functions. */

    void               SetWidth( float dWidth);
    void               SetHeight( float dHeight);
    void               SetLandscape( AnvilBoolean fLandscape);
    void               SetTrueColor( AnvilBoolean fTrueColor);
};

#endif /* _editor_page_setup_dialog_class_h_ */
