#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _editor_pick_layer_dialog_class_h_
#define _editor_pick_layer_dialog_class_h_

#include <anvil/xdisplay.h>
#include <anvil/screenobj.h>
#include <anvil/motifdisplay.h>
#include <anvil/overlay.h>
#include <anvil/copyobj.h>

#include <ToolKit/DialogClass.h>
#include <ToolKit/FormClass.h>
#include <ToolKit/PaneClass.h>
#include <ToolKit/RadioPaneClass.h>
#include <ToolKit/FrameClass.h>
#include <ToolKit/DrawingAreaClass.h>
#include <ToolKit/SeparatorClass.h>
#include <ToolKit/LabelClass.h>
#include <ToolKit/ButtonClass.h>
#include <ToolKit/ToggleButtonClass.h>
#include <ToolKit/TextFieldClass.h>
#include <ToolKit/ListClass.h>
#include <ToolKit/ScrolledWindowClass.h>

class AmPickLayerDialogClass : public AmDialogClass {

protected:

    void         initializeValues();
    void         createInterface();

    static void  AmActionCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    virtual void action( Widget    w,
                         XtPointer xpCallData);

    static void  AmSelectCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void         select( Widget    w,
                         XtPointer xpCallData);

    static void  AmToggleCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);

    static void  AmLayerDeletedCallback( int       iID,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData);

    void         layerDeleted( int       iID,
                               XtPointer xpCallData);

    static void  AmObjectDeletedCallback( int       iID,
                                        XtPointer xpClientData,
                                        XtPointer xpCallData);

    void         objectDeleted( int       iID,
                                XtPointer xpCallData);

protected:
    AmListClass         *_pwcLayerList;
    AmFrameClass        *_pwcInfoFrame;
    AmToggleButtonClass *_pwcToggle;
    AmDrawingAreaClass  *_pwcArea;

    AmOverlayClass      *_pOverlay;
    AmDataGraphicClass  **_vpDataGraphic;
    int               _nDataGraphic;

    int               *_viLayerPos;
    int               _nLayerPos;
    
    int               *_viSelectedLayers;
    int               _nSelectedLayers;

    AmObjectTypeEnum    _eObjectType;
    AnvilBoolean           _fShowOverlay;

    AmXDisplayClass     *_pXDisplay;
    AmXWindowClass      *_pXWindow;
    AmMotifDisplayClass *_pRefMotifDisplay;
    AmOverlayClass      *_pRefOverlay;

    AmScreenObjClass    **_vpScreenObj;

    XtCallbackProc    _xcpActionCallback;
    XtPointer         _xpClientData;

public:

    /* Constructors/Desctructors. */

    AmPickLayerDialogClass();
    AmPickLayerDialogClass( Widget         wParent,
                          const char     *szName,
                          AmOverlayClass   *pOverlay,
                          XtCallbackProc xcpCallback = NULL,
                          XtPointer      xpClientData = NULL);
    AmPickLayerDialogClass( AmBaseWidgetClass *bwcParent,
                          const char      *szName,
                          AmOverlayClass    *pOverlay,
                          XtCallbackProc  xcpCallback = NULL,
                          XtPointer       xpClientData = NULL);
    virtual ~AmPickLayerDialogClass();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "AmPickLayerDialogClass"); };

    /* Mutator functions. */

    void SetOverlay( AmOverlayClass *pOverlay);
    void SetShowOverlay( AnvilBoolean fShowOverlay);
    void SetFilter( AmObjectTypeEnum eObjectType);
    void SetSingleSelect();
    void SetBrowseSelect();
    void SetMultipleSelect();
    void SetExtendedSelect();

    /* Semi-private functions. */

    void updateLayerList();
    void updateInfoBox();

};

#endif /* _editor_pick_layer_dialog_class_h_ */
