#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _editor_text_dialog_class_
#define _editor_text_dialog_class_

#include <anvil/xdisplay.h>
#include <anvil/fonts.h>

#include <ToolKit/DialogClass.h>
#include <ToolKit/FormClass.h>
#include <ToolKit/FrameClass.h>
#include <ToolKit/ComboBoxClass.h>
#include <ToolKit/LabelClass.h>
#include <ToolKit/ToggleButtonClass.h>
#include <ToolKit/TextFieldClass.h>

class AmTextDialogClass : public AmDialogClass {

private:

    void         initializeValues();
    void         createInterface();

    static void  AmExposeCallback( Widget    w,
                                 XtPointer xpClientData,
                                 XtPointer xpCallData);
    void expose( Widget    w,
                 XtPointer xpCallData);

    static void  AmUpdateTextCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void updateText( Widget    w,
                     XtPointer xpCallData);

    static void  AmUpdateSizeTextCallback( Widget    w,
                                         XtPointer xpClientData,
                                         XtPointer xpCallData);
    void updateSizeText( Widget    w,
                         XtPointer xpCallData);

    static void  AmUpdateSizeCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void updateSize( Widget    w,
                     XtPointer xpCallData);

    static void  AmUpdateFamilyCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData);
    void updateFamily( Widget    w,
                       XtPointer xpCallData);

    static void  AmToggleBoldCallback( Widget    w,
                                     XtPointer xpClientData,
                                     XtPointer xpCallData);
    void toggleBold( Widget    w,
                     XtPointer xpCallData);

    static void  AmToggleItalicCallback( Widget    w,
                                       XtPointer xpClientData,
                                       XtPointer xpCallData);
    void toggleItalic( Widget    w,
                       XtPointer xpCallData);

    void valuesChanged();

protected:

    AmFrameClass        *_pwcFrame;
    AmFormClass         *_pwcFrameForm;

    AmFormClass         *_pwcTextForm;
    AmLabelClass        *_pwcTextLabel;
    AmTextFieldClass    *_pwcTextTextField;

    AmFormClass         *_pwcFamilyForm;
    AmLabelClass        *_pwcFamilyLabel;
    AmComboBoxClass     *_pwcFamilyOptionMenu;

    AmFormClass         *_pwcSizeForm;
    AmLabelClass        *_pwcSizeLabel;
    AmTextFieldClass    *_pwcSizeTextField;
    AmComboBoxClass     *_pwcSizeOptionMenu;

    AmFormClass         *_pwcStyleForm;
    AmToggleButtonClass *_pwcBoldToggle;
    AmToggleButtonClass *_pwcItalicToggle;
    AmToggleButtonClass *_pwcUnderlineToggle;

    AmXDisplayClass     *_pXDisplay;
    AmXWindowClass      *_pXWindow;
    AmXGCClass          *_pXGC;

    Widget            _wWindow;
    Widget            _wDrawingArea;
 
    AmColorCellRec      _colBackground;
    AmColorCellRec      _colForeground;

    Display           *_display;
    Screen            *_screen;

    int               _iMargin;
    const char        **_pszFamily;

    int               _iTextHeight;
    int               _iAscent;
    int               _iDescent;

    char              *_szText;

    char              *_szName;
    int               _iSize;
    AmFontWeightEnum    _eWeight;
    AmFontSlantEnum     _eSlant;
    float             _dScaleX;
    float             _dScaleY;

public:

    /* Constructors/Desctructors. */

    AmTextDialogClass();
    AmTextDialogClass( Widget     wParent,
                     const char *szName);
    AmTextDialogClass( AmBaseWidgetClass *bwcParent,
                     const char      *szName);
    virtual ~AmTextDialogClass();

    /* Accessor functions. */

    virtual const char *GetClassName() { return( "AmTextDialogClass"); };
    void               GetFont( char           **pszName,
                                int            *piSize,
                                AmFontWeightEnum *peWeight,
                                AmFontSlantEnum  *peSlant,
                                float          *pdScaleX,
                                float          *pdScaleY);
    const char         *GetText();

    /* Mutator functions. */

    void               SetFont( char           *szName,
                                int            iSize,
                                AmFontWeightEnum eWeight,
                                AmFontSlantEnum  eSlant,
                                float          dScaleX,
                                float          dScaleY);
    void               SetText( const char *szText);
};

#endif /* _editor_text_dialog_class_ */
