#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef anvil_dem_h_
#define anvil_dem_h_

#include <anvil/units.h>

/*------------------------------------------*/
/* Struct for a USGS DEM type A record.     */
/* ( Contains all items from the original A */
/*   record, plus two additional items to   */
/*   store the smallest # of points in a    */
/*   profile as well as the largest # )     */

typedef struct oldDemHeader {
    char	fileName[40];
    char	text[40];
    char	filler1[55];
    char	processCode;
    char	filler2;
    char	section[3];
    char	mcOrigin[4];
    char	demLevel[6];
    char	pattern[6];
    char	reference[6];
    char	zone[6];
    char	parameters[15][24];
    char	groundUnit[6];
    char	elevUnit[6];
    char	nsides[6];
    char	corners[4][2][24];
    char	min[24], max[24];
    char	angle[24];
    char	accuracy[6];
    char	resolution[3][12];
    char	nrows[6], ncols[6];
} oldDemHeaderRec, *oldDemHeaderPtr;

typedef struct newDemHeader {
    oldDemHeaderRec	oldDemHeader;
    char	maxContour[5];
    char	units1;
    char	minContour[5];
    char	units2;
    char	sourceDate[4];
    char	revDate[4];
    char	revFlag;
    char	dataFlag;
    char	areaFlag[2];
    char	vertDatum[2];
    char	horzDatum[2];
    char	edition[4];
} newDemHeaderRec, *newDemHeaderPtr;

/*------------------------------------------*/
/* Struct for a USGS DEM type B record.     */
/* ( enough space will be allocated for the */
/*   # of points in this profile, and the   */
/*   (short *) elevs will point at it ... ) */

typedef struct demColHead {
    char	rowId[ 6];
    char	colId[ 6];
    char	m[ 6];
    char	n[ 6];
    char	xGo[ 24];
    char	yGo[ 24];
    char	localDatum[ 24];
    char	minElev[ 24];
    char	maxElev[ 24];
} demColHeadRec, *demColHeadPtr;

void printHeader( 
    oldDemHeaderPtr oldHead
); 

int readDemFileSet(char 	**filenames,
		   int	        *width,
		   int	        *height,
		   short	**data,
		   double	*xmin,
		   double	*xmax,
		   double	*ymin,
		   double	*ymax,
		   double	*xres,
		   double	*yres,
		   unitType     *xyUnits);

#endif /* anvil_dem_h_ */
