#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_dlg_h_
#define _anvil_dlg_h_

#include <anvil/datafmt.h>
#include <anvil/geocodes.h>

typedef struct dlgHead1_ {
    char	banner[ 72];
} dlgHead1Rec, *dlgHead1Ptr;

typedef struct dlgHead2_ {
    char	mapName[ 40];
    char	filler1[ 1];
    char	dateAndRev[ 10];
    char	filler2[ 1];
    char	scale[ 8];
} dlgHead2Rec, *dlgHead2Ptr;

typedef struct dlgHead3_ {
    char 	filler1[ 68];
} dlgHead3Rec, *dlgHead3Ptr;

typedef struct dlgHead4_ { 
    char	dlgCode[ 6];
    char	refSystem[ 6];
    char	refZone[ 6];
    char	refUnits[ 6];
    char	resolution[ 18];
    char	transformParams[ 6];
    char	accuracyRecs[ 6];
    char	controlPts[ 6];
    char	dlgCats[ 6];
} dlgHead4Rec, *dlgHead4Ptr;

typedef struct dlgHead5thru9_ {
    char	projectParams[ 3][ 24];
} dlgHead5thru9Rec, *dlgHead5thru9Ptr; 

typedef struct dlgHead10_ {
    char	projectTransParams[ 4][ 18];
} dlgHead10Rec, *dlgHead10Ptr;

typedef struct ctlPt_ {
    char	label[ 2];
    char	filler1[ 4];
    char	latitude[ 12];
    char	longitude[ 12];
    char	filler2[ 6];
    char	filex[ 12];
    char	filey[ 12];
} ctlPtRec, *ctlPtPtr;

typedef struct dataCategory_ {
    char	name[ 20];
    char	formatCodes[ 4];
    char	numRefNodes[ 6];
    char	numRealNodes[ 6];
    char	filler1[ 1];
    char	nodeAreaFlags[ 1];
    char	nodeLineFlags[ 1];
    char	filler2[ 1];
    char	numRefAreas[ 6];
    char	numRealAreas[ 6];
    char	filler3[ 1];
    char	areaNodeFlag[ 1];
    char	areaLineFlag[ 1];
    char	areaCoordFlag[ 1];
    char	numRefLines[ 6];
    char	numRealLines[ 6];
    char	filler4[ 3];
    char	lineCoordListFlag[ 1];
} dataCategoryRec, *dataCategoryPtr;

typedef struct nodeArea_ {
    char	kind[ 1];
    char	id[ 5];
    char	coords[ 2][ 12];
    char	numElements[ 6];
    char	AmNumLines[ 6];
    char	numXyPts[ 6];
    char	numAttCodes[ 6];
    char	numTextChars[ 6];
    char	numIslands[ 6];
} nodeAreaRec, *nodeAreaPtr;

typedef struct line_ {
    char	kind[ 1];
    char	id[ 5];
    char	startNode[ 6];
    char	endNode[ 6];
    char	leftArea[ 6];
    char	rightArea[ 6];
    char	filler1[ 12];
    char	numCoords[ 6];
    char	numAttCodes[ 6];
    char	numTextChars[ 6];
} lineRec, *linePtr;

typedef struct tempDLGPoly_ {
    int		AmNumLines;
    int		*lines;
    int		numAttrs;
    int		*attrs;
} tempDLGPolyRec, *tempDLGPolyPtr;

typedef struct tempDLGLine_ {
    int		idNo;
    int		numCoords;
    AmCoordPtr	coords;
    int		numAttrs;
    int		*attrs;
    int		startNode;
    int		endNode;
} tempDLGLineRec, *tempDLGLinePtr;

typedef struct dlgAttrs_ {
    int		major;
    int		minor;
    int		attr;
} dlgAttrRec, *dlgAttrPtr;

/* structure for a DLG line which retains the original 
   major and minor attribute codes */

typedef struct dlgLine_ {
    AmGeoLineRec	geoLine;
    short	majorCode;
    short	minorCode;
} dlgLineRec, *dlgLinePtr;


/* same thing for a DLG polygon */

typedef struct dlgPoly_ {
    AmGeoPolygonRec	geoPoly;
    short		majorCode;
    short		minorCode;
} dlgPolyRec, *dlgPolyPtr;

#endif /* __anvil_dlg_h_ */
