#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef util_landuse_h_
#define util_landuse_h_
#include <anvil/project.h>

#define LANDRECLEN	1025


typedef struct lanudseAttrCode_ {
    int		attr;
    int		code;
} landuseAttrCodeRec, *landuseAttrCodePtr;

typedef struct fpoint {
    float	x;
    float	y;
} fpointRec, *fpointPtr;

typedef struct larc {
    short	arcNo;
    int		npts;
    int		plc;
} larcRec, *larcPtr;

typedef struct lpoly_ {
    int			polyNo;
    int			attr;		    /* converted LULC attribute code */
    int			origAttr;	    /* original LULC attribute code */
    larcPtr		*arcs;		    /* arc vector */
    int			narcs;
    int			pla;
    int			npts;
    fpointPtr		pts;
    int			surroundPolyNo;
    int			numIslands;
    struct lpoly_	**islands;
} lpolyRec, *lpolyPtr;

typedef struct ltext {
    int		attr;
    int		polyNo;
    int		polyCode;
    char	desc[ 58];
} ltextRec, *ltextPtr;

/* returned landuse data structure */

typedef struct landuseSection {
    int		numCoords;
    fpointPtr	coords;				/* coordinates */
    int		numArcs;
    larcPtr	arcs;				/* arcs */
    int		numPolys;
    lpolyPtr	lpolys;				/* polygons */
    int		numFaps;
    short	*faps;				/* arcs by polygon */
} landuseSectionRec, *landuseSectionPtr;

typedef struct landuseData {
    int			numSections;
    landuseSectionPtr	sections;
    int			numTexts;
    ltextPtr		texts;
} landuseDataRec, *landuseDataPtr;

/* map header record */

typedef struct mapHeader {
    char 	numArcRecs[ 10];
    char 	numArcCoords[ 10];
    char	numPolyRecs[ 10];
    char	pointTolerance[ 5];
    char	arclenTolerance[ 5];
    char	numMapSections[ 5];
    char	mapCode[ 5];
    char	textSubfileLen[ 5];
    char	mapProjectionCode[ 5];
    char	mapScaleDenom[ 10];
    char	sourceMapDate[ 10];
    char	rangeCoords[ 4][ 5];
    char	controlPts[ 12][ 5];
    char	latlonControlPts[ 12][ 10];
    char	numAssocData[ 5];
    char	titleLen[ 5];
    char	numFapSubfiles[ 5];
    char	pad1[ 5];
    char	julianDate[ 5];
    char	createTime[ 5];
    char	title[ 64];
    char	pad2[ 26];
} mapHeaderRec, *mapHeaderPtr;

/* section  header */

typedef struct sectionHeader {
    char	sectionNumber[ 5];
    char	numArcs[ 5];
    char	numArcCoords[ 5];
    char	numPolys[ 5];
    char	numFaps[ 5];
    char	processHistory[ 5];
    char	minCoords[ 2][ 5];
    char	maxCoords[ 2][ 5];
    char	numNodes[ 5];
    char	pad1[ 25];
} sectionHeaderRec, *sectionHeaderPtr;

/* arc record */

typedef struct arcRecord {
    char 	arcNo[ 5];
    char	lastCoordPos[ 5];
    char	leftPolyNo[ 5];
    char	rightPolyNo[ 5];
    char	leftPolyAttr[ 10];
    char	rightPolyAttr[ 10];
    char	minCoords[ 2][ 5];
    char	maxCoords[ 2][ 5];
    char	arcLength[ 10];
    char	startNodeNum[ 5];
    char	endNodeNum[ 5];
} arcRecordRec, *arcRecordPtr;

/* coordinate record */

typedef struct coordRecord {
    char	x[ 5];
    char	y[ 5];
} coordRecordRec, *coordRecordPtr;

/* polygon record */

typedef struct polyRecord {
    char	polyNo[ 5];
    char	lastArcNo[ 5];
    char	interiorCoords[ 2][ 5];
    char	attr[ 10];
    char	area[ 10];
    char	minCoords[ 2][ 5];
    char	maxCoords[ 2][ 5];
    char	perimeterLength[ 10];
    char	numIslands[ 5];	/* number of islands in poly */
    char	surroundPoly[ 5];	/* if island, number of contain poly */
} polyRecordRec, *polyRecordPtr;

/* file of arcs by polygon subfile record */

typedef struct fapRecord {
    char	arcNo[ 5];
} fapRecordRec, *fapRecordPtr;

typedef struct fapLineRecord {
    fapRecordRec	fapRecords[ 16];
} fapLineRecordRec, *fapLineRecordPtr;

/* text subfile record */

typedef struct textRecord {
    char	attr[ 10];
    char	polyCodeType[ 5];
    char	description[ 58];
    char	pad[ 1];
} textRecordRec, *textRecordPtr;

typedef struct lulcPoly_ {
    AmGeoPolygonRec	geoPoly;
    short		origCode;	/* original LULC code */
} lulcPolyRec, *lulcPolyPtr;

#endif /* util_landuse_h_ */
