#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _anvil_shapefile_h_
#define _anvil_shapefile_h_
#include <sys/types.h>
#include <util/endian.h>

#define SHP_FILE_HEADER_LENGTH	100
#define ARC_HEADER_LENGTH	52

typedef struct shapeFileHeader_ {
    ulong_t	fileCode;	/* 9994, bigEndian */
    int		unused1;	/* apparently reserved */
    int		unused2;	/* apparently reserved */
    int		unused3;	/* apparently reserved */
    int		unused4;	/* apparently reserved */
    int		unused5;	/* apparently reserved */
    int		fileLength;	/* bigEndian */
    int		version;	/* little endian */
    int		shapeType;	/* little endian */
    double	xmin;		/* little endian bounding box */
    double	ymin;		
    double	xmax;		
    double	ymax;		
    int		unused6;	/* second unused block */
    int		unused7;	/* second unused block */
    int		unused8;	/* second unused block */
    int		unused9;	/* second unused block */
    int		unused10;	/* second unused block */
    int		unused11;	/* second unused block */
    int		unused12;	/* second unused block */
} shapeFileHeaderRec, *shapeFileHeaderPtr;

typedef struct shapeRecordHeader_ {
    ulong_t	recordNumber;	/* big endian */
    ulong_t	contentLength;	/* big endian */
} shapeRecordHeaderRec, *shapeRecordHeaderPtr;

typedef struct pointRecord_ {
    int		shapeType;
    double	x;		/* little endian */
    double	y;		/* little endian */
} pointRecordRec, *pointRecordPtr;

typedef struct point_ {
    double	x;
    double	y;
} shapePointRec, *shapePointPtr;

typedef struct multiPointRecord_ {
    int		shapeType;
    double	xmin;		/* little endian bounding box */
    double	ymin;		
    double	xmax;		
    double	ymax;		
    int		numPoints;	/* little endian */
} multiPointRecordRec, *multiPointRecordPtr;

/* in the file, the multiPoint structure is followed by a points array */

typedef struct arcRecord_ {
    int		shapeType;
    int		unused1;	/* alignment is wrong in documentation */
    double	xmin;		/* little endian bounding box */
    double	ymin;		
    double	xmax;		
    double	ymax;		
    int		numPoints;	/* little endian */
    int		numParts;	/* little endian */
} arcRecordRec, *arcRecordPtr;

/* in the file, the arcRecord structure is followed by an integer parts array
   and a points array */

typedef struct polygonRecord_ {
    int		shapeType;
    int		unused1;
    double	xmin;		/* little endian bounding box */
    double	ymin;		
    double	xmax;		
    double	ymax;		
    int		numPoints;		/* little endian */
    int		numParts;		/* little endian */
} polygonRecordRec, *polygonRecordPtr;

/* in the file, the polygonRecord structure is followed by an integer parts array
   and a points array */

typedef struct indexRecord {
    int		offset;			/* big endian */
    int		length;			/* big endian */
} indexRecordRec, *indexRecordPtr;

typedef enum shapeEnum_ {
    AmNULL_SHAPE = 0,
    AmPOINT_SHAPE = 1,
    AmARC_SHAPE = 3,
    AmPOLYGON_SHAPE = 5,
    AmMULTIPOINT_SHAPE = 8
} shapeEnum;

#endif /* _anvil_shapefile_h_ */
