#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _util_tiger_h_
#define _util_tiger_h_

#include <anvil/geocodes.h>

#define TPOLYNUMPTS	10

typedef struct attrCode_ {
    char	code[ 32];
    int		attr;
} attrCodeRec, *attrCodePtr;

typedef struct lonlat {
    char	lon[ 10];
    char	lat[ 9];
} lonlatRec, *lonlatPtr;

/* this is a record type 1 ( also called a basic data record) */

typedef struct basicData {
    char	type[ 1];		/* type and version */
    char	version[ 4];
    char	recnum[ 10];		/* record number ( index) */
    char	singleSide[ 1];	/* single-size segment flag "1" */ 
    char	source[ 1];
    char	featureDirPrefix[ 2];	/* AmFirst name of the feature. */
    char	featureName[ 30];	
    char	featureType[ 4];
    char	featureDirSuffix[ 2];
    char	classCode[ 3];
    char	fromAddrLeft[ 11];
    char	toAddrLeft[ 11];
    char	fromAddrRight[ 11];
    char	toAddrRight[ 11];
    char 	fromImputLeft[ 1];
    char 	toImputLeft[ 1];
    char 	fromImputRight[ 1];
    char 	toImputRight[ 1];
    char	zipLeft[ 5];
    char	zipRight[ 5];
    char	fipsLeft[ 5];
    char	fipsRight[ 5];
    char	anrcl[ 2];
    char	anrcr[ 2];
    char	stateLeft[ 2];
    char	stateRight[ 2];
    char	countyLeft[ 3];
    char	countyRight[ 3];
    char	mcdCcdLeft[ 5];		/* some kind of census data */
    char	mcdCcdRight[ 5];
    char	subMcdLeft[ 5];
    char	subMcdRight[ 5];
    char	placeLeft[ 5];
    char	placeRight[ 5];
    char	basicNumLeft[ 4];
    char	suffixLeft[ 2];
    char	basicNumRight[ 4];
    char	suffixRight[ 2];
    char	collBlockLeft[ 3];
    char	tabSuffixLeft[ 1];
    char	collBlockRight[ 3];
    char	tabSuffixRight[ 1];
    lonlatRec	from;
    lonlatRec	to;
    char	newline[ 1];			/* record followed by newline */
} basicDataRec, *basicDataPtr;

/* shape coordinate points (record type 2) */

typedef struct shapeCoord { 
    char	type[ 1];		/* type and version */
    char	version[ 4];
    char	recnum[ 10];		/* record number ( index) */
    char	recseqnum[ 3];		/* 1st set of pts, 2nd set... */
    lonlatRec	coords[ TPOLYNUMPTS];
    char	newline[ 1];			/* record followed by newline */
} shapeCoordRec, *shapeCoordPtr;

/* additional decennial census geographics area codes ( record type 3) */

typedef struct censusGeoCodes {
    char	type[ 1];
    char	version[ 4];
    char	recnum[ 10];
    char	stateLeft[ 2];
    char	stateRight[ 2];
    char	countyLeft[ 2];
    char	countyRight[ 2];
    char	mcdCcdLeft80[ 5];		/* some kind of census data */
    char	mcdCcdRight80[ 5];
    char	subMcdLeft80[ 5];
    char	subMcdRight80[ 5];
    char	basicNumLeft[ 4];
    char	suffixLeft[ 2];
    char	basicNumRight[ 4];
    char	suffixRight[ 2];
    char	blkNumLeft80[ 3];
    char	blkNumRight80[ 3];
    char	divCodeLeft80[ 3];
    char	divCodeRight80[ 3];
    char	placeLeft80[ 4];
    char	placeRight80[ 4];
    char	airCodeLeft[ 4];
    char	airCodeRight[ 4];
    char	mcdCcdLeft[ 5];		/* some kind of census data */
    char	mcdCcdRight[ 5];
    char	subMcdLeft[ 5];
    char	subMcdRight[ 5];
    char	placeLeft[ 4];
    char	placeRight[ 4];
    char	votingLeft[ 4];
    char	votingRight[ 4];
    char	newline[ 1];			/* record followed by newline */
} censusGeoCodesRec, *censusGeoCodesPtr;

/* index to alternate feature names ( record type 4) */

typedef struct altFeature {
    char	type[ 1];
    char	version[ 4];
    char	recnum[ 10];
    char	recseqnum[ 3];
    char	altCodes[ 5][ 8];
    char	newline[ 1];			/* record followed by newline */
} altFeatureRec, *altFeaturePtr;

/* feature name list ( record type 5) */

typedef struct featureName {
    char	type[ 1];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	altCode[ 8];
    char	dirPrefix[ 2];
    char	featureName[ 30];
    char	featureType[ 4];
    char	dirSuffix[ 2];
    char	newline[ 1];			/* record followed by newline */
} featureNameRec, *featureNamePtr;

/* additional address range & zip code data ( record type 6) */

typedef struct addressZip {
    char	type[ 1];
    char	version[ 4];
    char	recnum[ 10];
    char	recseqnum[ 3];
    char	fromAddrLeft[ 11];
    char	toAddrLeft[ 11];
    char	fromAddrRight[ 11];
    char	toAddrRight[ 11];
    char	fromImputFlagLeft[ 1];
    char	toImputFlagLeft[ 1];
    char	fromImputFlagRight[ 1];
    char	toImputFlagRight[ 1];
    char	zipLeft[ 5];
    char	zipRight[ 5];
    char	newline[ 1];			/* record followed by newline */
} addressZipRec, *addressZipPtr;

/* landmark features ( record type 7) churches, schools, etc */

typedef struct landmarkFeature {
    char	type[ 1];
    char	version[ 4];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	landmarkId[ 10];
    char	source[ 1];
    char	classCode[ 3];
    char	featureName[ 30];
    lonlatRec	position;
    char	filler[ 1];
    char	newline[ 1];			/* record followed by newline */
} landmarkFeatureRec, *landmarkFeaturePtr;

/* area landmarks ( record type 8) (polygons) */

typedef struct areaLandmark {
    char	type[ 1];
    char	version[ 4];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	censusId[ 5];
    char	polyId[ 10];
    char	landmarkId[ 10];
    char	filler[ 2];
    char	newline[ 1];			/* record followed by newline */
} areaLandmarkRec, *areaLandmarkPtr;

/* addtional polygon geographic area codes ( record type A) */

typedef struct polygonGeo {
    char	type[ 1];
    char	version[ 4];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	censusId[ 5];
    char	polyId[ 10];
    char	fairCode[ 5];
    char	mcdCcd[ 5];		/* some kind of census data */
    char	place[ 5];
    char	tractBna[ 6];
    char	tabBlk[ 6];
    char	cd101[ 2];		/* congressional district codes */
    char	cd103[ 2];
    char	elemdist[ 5];		/* future */
    char	middist[ 5];
    char	secdist[ 5];
    char	unifdist[ 5];
    char	trafzone[ 6];
    char	urbanarea[ 4];
    char	urbanflag[ 1];
    char	unused[ 13];
    char	newline[ 1];			/* record followed by newline */
} polygonGeoRec, *polygonGeoPtr;

/* area boundaries ( record type I) */

typedef struct areaBound {
    char	type[ 1];
    char	version[ 4];
    char	recnum[ 10];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	areaType[ 1];		/* P = polygon identification code */
    char	cenidLeft[ 5];
    char	polyLeft[ 10];
    char	cenidRight[ 5];
    char	polyRight[ 10];
    char	filler[ 1];
    char	newline[ 1];			/* record followed by newline */
} areaBoundRec, *areaBoundPtr;

/* polygon location ( record type P) */

typedef struct polyLocation {
    char	type[ 1];
    char	version[ 4];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	cenid[ 5];
    char	polyId[ 10];
    lonlatRec	position;
    char	newline[ 1];			/* record followed by newline */
} polyLocationRec, *polyLocationPtr;

/* record number range ( record type R) */

typedef struct numberRange {
    char	type[ 1];
    char	version[ 4];
    char	stateCode[ 2];
    char	countyCode[ 3];
    char	cenid[ 5];
    char	cenidMin[ 10];
    char	cenidMax[ 10];
    char	cenidHigh[ 10];
    char	filler[ 1];
    char	newline[ 1];			/* record followed by newline */
} numberRangeRec, *numberRangePtr;

int atoilen( char *buffer, int len);

float readLatCoord( char *str);

float readLonCoord( char *str);

#endif /* _util_tiger_h_ */


