#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * bitsrv.c
 *
 * Kip Walker, Jan 1997
 *
 * basic server for the bitstream warden.  exports "bits_put" and "bits_get"
 */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <sys/types.h>
#include <netinet/in.h>

#include "include/odytypes.h"

#include <lwp/lwp.h>
#include <lwp/timer.h>
#include <rpc2/rpc2.h>
#include <rpc2/se.h>
#include <rpc2/sftp.h>

#include "odyssey/wardens/bit/bitwarden.h"

#include "ody_bits.h"

void Init_RPC();
void error_report(char *);

static char *big_buf = NULL;

/*
 * start the server and handle our RPCs
 */
int main (int argc, char **argv)
{
    RPC2_Handle the_connection;
    RPC2_RequestFilter reqfilter;
    RPC2_PacketBuffer *reqbuffer;
    int return_code;
    
    BALLOC(big_buf,BITW_MAX_CHUNK);
    Init_RPC();
   
    reqfilter.FromWhom = ANY;
    reqfilter.OldOrNew = OLDORNEW;
    reqfilter.ConnOrSubsys.SubsysId = BITS_SUBSYSID;
    
    while (1) {
	return_code = RPC2_GetRequest(&reqfilter, &the_connection, &reqbuffer,
				      NULL, NULL, 0, NULL);
	if (return_code != RPC2_SUCCESS)
	    fprintf(stderr, RPC2_ErrorMsg(return_code));
	return_code = ody_bits_ExecuteRequest(the_connection, reqbuffer, NULL);
	if (return_code != RPC2_SUCCESS)
	    fprintf(stderr, RPC2_ErrorMsg(return_code));
    }
}

void error_report(char *message)
{
    fprintf(stderr, message);
    fprintf(stderr, "\n");
    exit(1);
}

void Init_RPC()
{
    PROCESS dummy;
    RPC2_PortIdent pid;
    RPC2_SubsysIdent sid;
    SFTP_Initializer sftpi;
    int return_code;
    char error_msg[100];
    
    if (LWP_Init(LWP_VERSION, LWP_NORMAL_PRIORITY, &dummy) != LWP_SUCCESS)
	error_report("Can't initialize LWP");
    
    SFTP_SetDefaults(&sftpi);
    sftpi.EnforceQuota = 0;
    SFTP_Activate(&sftpi);
    
    pid.Tag = RPC2_PORTBYINETNUMBER;
    pid.Value.InetPortNumber = htons(BITS_PORTAL);
    return_code = RPC2_Init(RPC2_VERSION, NULL, &pid, 1, NULL);
    
    if (return_code != RPC2_SUCCESS) {
	sprintf(error_msg, "%s\nCan't initialize RPC2", 
		RPC2_ErrorMsg(return_code));
	error_report(error_msg);
    }
    
    sid.Tag = RPC2_SUBSYSBYID;
    sid.Value.SubsysId = BITS_SUBSYSID;
    return_code = RPC2_Export(&sid);
    if (return_code != RPC2_SUCCESS) {
	sprintf(error_msg, "%s\nCan't export the bitstream subsystem", 
		RPC2_ErrorMsg(return_code));
	error_report(error_msg);
    }
}

long bits_put(RPC2_Handle _cid, RPC2_Integer length, SE_Descriptor *bp)
{
    SE_Descriptor sed;
    
    assert(length < BITW_MAX_CHUNK);
    
    bzero(&sed, sizeof(sed));
    sed.Tag = SMARTFTP;
    sed.Value.SmartFTPD.Tag = FILEINVM;
    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.MaxSeqLen = length;
    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqBody = big_buf;
    sed.Value.SmartFTPD.TransmissionDirection = CLIENTTOSERVER;
    
    if (RPC2_InitSideEffect(_cid, &sed) != RPC2_SUCCESS)
	return BITS_FAILED;
    if (RPC2_CheckSideEffect(_cid, &sed, SE_AWAITLOCALSTATUS) != RPC2_SUCCESS)
	return BITS_FAILED;
    
    return RPC2_SUCCESS;
}

long bits_get(RPC2_Handle _cid, RPC2_Integer* length, SE_Descriptor *bp)
{
    SE_Descriptor sed;
    
    assert(*length < BITW_MAX_CHUNK);
    
    bzero(&sed, sizeof(sed));
    sed.Tag = SMARTFTP;
    sed.Value.SmartFTPD.Tag = FILEINVM;
    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqLen = *length;
    sed.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqBody = big_buf;
    sed.Value.SmartFTPD.TransmissionDirection = SERVERTOCLIENT;
    
    if (RPC2_InitSideEffect(_cid, &sed) != RPC2_SUCCESS)
	return BITS_FAILED;
    if (RPC2_CheckSideEffect(_cid, &sed, SE_AWAITLOCALSTATUS) != RPC2_SUCCESS)
	return BITS_FAILED;
    
    return RPC2_SUCCESS;
}

void iopen(void) {}
