#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <lwp/lwp.h>
#include <rpc2/rpc2.h>
#include "../../odyssey/wardens/speech/ody_speech.h"

RPC2_Handle connectToServer (char* hostname, int port)
{
  RPC2_HostIdent hid;
  RPC2_PortIdent pid;
  RPC2_SubsysIdent sid;
  RPC2_Handle cid;
  int rc;
  RPC2_BindParms bp;

  hid.Tag = RPC2_HOSTBYNAME;
  strcpy (hid.Value.Name, hostname);
  pid.Tag = RPC2_PORTBYINETNUMBER;
  pid.Value.InetPortNumber = htons(port);
  sid.Tag = RPC2_SUBSYSBYID;
  sid.Value.SubsysId = SPEECH_SUBSYSID;
  
  bp.SecurityLevel = RPC2_OPENKIMONO;
  bp.EncryptionType = 0;
  bp.SideEffectType = 0;
  bp.ClientIdent = NULL;
  bp.SharedSecret = NULL;
  rc = RPC2_NewBinding (&hid, &pid, &sid, &bp, &cid);
  if (rc != RPC2_SUCCESS) {
    printf ("Can't connect to host %s port %d\n", hostname, port);
  } /* endif */
  
  return (cid);
}

int Init_RPC ()
{
  PROCESS mylpid;
  int rc;
  
  if (LWP_Init (LWP_VERSION, LWP_NORMAL_PRIORITY, &mylpid) != LWP_SUCCESS) {
    printf ("Can't initialize LWP");
    return (-1);
  } /* endif */

  rc = RPC2_Init (RPC2_VERSION, NULL, NULL, -1, NULL);
  if (rc != RPC2_SUCCESS) {
    printf ("Can't initialize RPC2, rc = %d\n", rc);
  } /* endif */

  return (rc);
}

int main (int argc, char* argv[])
{
  RPC2_Handle cid;
  char* hostname;
  int port;
  int rc;

  if (argc != 3) {
    printf ("Usage haltServer <hostname> <port>\n");
    exit (0);
  } /* endif */

  hostname = argv[1];
  port = atoi (argv[2]);

  Init_RPC();

  cid = connectToServer (hostname, port);
  rc = speech_haltServer (cid);
  if (rc != RPC2_SUCCESS) {
    printf ("Unable to halt server, rc = %d\n", rc);
  } /* endif */

  rc = RPC2_Unbind (cid);
  if (rc != RPC2_SUCCESS) {
    printf ("Unable to unbind, rc = %d\n", rc);
  } /* endif */

  return (rc);
}
