#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 *  loadgen.c
 *
 *  Kip Walker, Feb 1997
 *
 *  Load generating program for Odyssey
 *  Thie first incarnation solely deals with loading the network, using
 *  the bitstream warden.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <kernel/upcall/includes/upcall.h>
#ifdef NETTR_UPCALL
#include <nettr/replay/ntrp_upcall.h>
#endif
#include <unistd.h>
#include <errno.h>

#include "kernel/upcall/includes/upcall.h"
#include "include/odytypes.h"
#include "odyssey/wardens/bit/bitwarden.h"

char buf[BITW_MAX_CHUNK];
uint block_size = 32768;
uint chunk_size = 65536;
u_long total;
u_long seconds;		/* initial bias to keep numbers smaller */
struct timeval run_begin, run_end;
pid_t mypid;
#ifdef NETTR_UPCALL
int go_to_it = 1;
int which_upcall = 0;
#endif

#define tv2ms(tv,bias) (tv.tv_usec/1000.0+(tv.tv_sec-bias)*1000)

void usage(void)
{
    printf("usage: loadgen [-tome <tome #>] [-blocksz <read size>]\n"
	   "               [-bw <desired bw>] [-total <total bytes>]\n"
#ifdef NETTR_UPCALL
	   "               [-u <upcall #>]\n"
#endif
	   "               [-chunksz <chunk size>] [-v]\n");
    exit(-1);
}

void clean_up()
{
    gettimeofday(&run_end, NULL);
    
    printf("%f msec, %f KB/s\n", 
	   tv2ms(run_end,seconds)-tv2ms(run_begin,seconds),
	   total*125/(128*(tv2ms(run_end,seconds)-tv2ms(run_begin,seconds))));

    exit(0);
}

#ifdef NETTR_UPCALL
int lg_end(upclass_t c, upcall_t uc, size_t argsz, void *argp,
	   size_t *retsz, void *retp, int flags)
{
    clean_up();
    return 0;
}
 
int lg_start(upclass_t c, upcall_t uc, size_t argsz, void *argp,
	   size_t *retsz, void *retp, int flags)
{
    if (--which_upcall == 0)
	go_to_it = 1;
    *retsz = 0;
    return 0;
}
#endif
 
/*
 * remaining questions:
 *    1) currently specify "read" size, using default sftp chunk size.  OK?
 *    2) allow initial prefetch before timing?
 */
int main (int argc, char **argv)
{
    int length = 0;		/* default is forever! */
    int verbose = 0;
    int retval;
    float desired_bw = 0.0;	/* KB/s */
    struct timeval interval = {0,0};
    struct timeval so_far = {0,0};
    struct timeval elapsed;
    struct timeval epsilon = {0, 10000};
    struct timeval sleep_time;
    struct timeval start, end, end2;
    u_long left, len;
    int fd;
    int tome = 0;
    char filename[256];
    
    argv++; argc--;
    while (argc > 0) {
	if (*argv[0] != '-')
	    usage();
	if (strcmp(*argv, "-blocksz") == 0) {
	    argv++; argc--;
	    sscanf(*argv, "%d", &block_size);
	} else if (strcmp(*argv, "-chunksz") == 0) {
	    argv++; argc--;
	    sscanf(*argv, "%d", &chunk_size);
	} else if (strcmp(*argv, "-bw") == 0) {
	    argv++; argc--;
	    sscanf(*argv, "%f", &desired_bw);
	} else if (strcmp(*argv, "-total") == 0) {
	    argv++; argc--;
	    sscanf(*argv, "%d", &length);
        } else if (strcmp(*argv, "-v") == 0)
            verbose = 1;
        else if (strcmp(*argv, "-tome") == 0) {
	    argv++; argc--;
	    sscanf(*argv, "%d", &tome);
#ifdef NETTR_UPCALL
        } else if (strcmp(*argv, "-u") == 0) {
	    if (upcall_reg(UC_NTRP, NTRP_NOTIFY_START, lg_start)) {
		perror("upcall_reg");
		exit(-1);
	    }
	    if (upcall_reg(UC_NTRP, NTRP_NOTIFY_TSEV, lg_start)) {
		perror("upcall_reg");
		exit(-1);
	    }
	    if (upcall_reg(UC_NTRP, NTRP_NOTIFY_END, lg_end)) {
		perror("upcall_reg");
		exit(-1);
	    }
	    argv++; argc--;
	    sscanf(*argv, "%d", &which_upcall);
	    go_to_it = 0;
#endif
	} else
	    usage();
	argv++; argc--;
    }
    
    mypid = getpid();
    sprintf(filename, "/ody/bits%d/%d", tome, mypid);

    signal(SIGINT, clean_up);

    if (desired_bw != 0.0) {
	interval.tv_sec = (int)((block_size>>10)/desired_bw);
	interval.tv_usec = (int)((((block_size>>10)/desired_bw) - 
	    (float)interval.tv_sec)*1e6);
    }

    gettimeofday(&start, NULL);
    seconds = start.tv_sec;		/* make the timeval crunching 
					 * numerically sane */
    
    left = length ? length : block_size + 1;

#ifdef NETTR_UPCALL
    while (!go_to_it)
	;
#endif
    
    gettimeofday(&run_begin, NULL);
    fd = open(filename, O_RDONLY, 0);
    
    if (chunk_size != BITW_DEFAULT_CHUNK) {
	bitw_quality_t qual;
	bitw_quality_t *oldq;

	qual.prefetch_depth = BITW_DEFAULT_DEPTH;
	qual.chunk_size = chunk_size;
	bits_set_params(fd, &qual, &oldq);

        free(oldq);
    }

    do {
	timeradd(&interval, &so_far, &so_far);
	len = (left > block_size) ? block_size : left;
	len = read(fd, buf, len);
	if (len < 0) {
	    perror("read");
	    exit(-1);
	}
	if (length > 0)
	    left -= len;
	total += len;
	gettimeofday(&end, NULL);
	timersub(&end, &run_begin, &elapsed);
	timersub(&so_far, &elapsed, &sleep_time);
	if (timercmp(&sleep_time, &epsilon, >)) {
            while(1) {
                retval = select(0,0,0,0,&sleep_time);
                if (retval < 0)
                    if (errno != EINTR)
                        perror("select");
                    else ;
                else break;
            }
	}
	gettimeofday(&end2, NULL);
	if (verbose)
	    ;			/* print something fascinating */
	gettimeofday(&end2, NULL);
    } while (left > 0);
    gettimeofday(&run_end, NULL);
    
    printf("%f msec, %f KB/s\n", 
	   tv2ms(run_end,seconds)-tv2ms(run_begin,seconds),
	   total*125/(128*(tv2ms(run_end,seconds)-tv2ms(run_begin,seconds))));
    
    close(fd);
    
    return 0;
}
