/* Modified by JNF from filter.cc in ANVIL pakcage */

/*****************************************************************************/
/*                                                                           */ 
/*          Copyright (C) 1996 ANVIL Software Incorporated		     */
/*                          All rights reserved.                             */ 
/*                                                                           */ 
/*  All data and information contained in or disclosed by this document is   */ 
/*  confidential and proprietary information of ANVIL Software 		     */
/*  Incorporated and all rights therein are expressly reserved.  By 	     */
/*  accepting this material the recipient agrees that this material and      */
/*  the information contained therein is held in confidence and in trust     */
/*  and will not be used, copied, reproduced in whole or in part, nor its    */
/*  contents revealed in any  manner to others without the express written   */
/*  permission of ANVIL Software Incorporated.                       	     */
/*                                                                           */ 
/*****************************************************************************/



#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <anvil/project.h>
#include <anvil/xresource.h>
#include <anvil/datafmt.h>
#include <anvil/macros.h>
#include <anvil/datatype.h>
#include <anvil/files.h>
#include <anvil/strlib.h>
#include <anvil/tagstrs.h>
#include <anvil/fns.h>
#include <anvil/choppath.h>
#include <util/utils.h>
#include <util/misc.h>

#define BUFSIZE	1024

int cmpInt( int *a, int *b)
{
    return( *a - *b);
}

int distill_me(char* filename, int* filterIds, int numIds)
{
    int			fdOut, ii, jj, fdIn, rows, size, 
   			skipBytes, newrows = 0,
			lookupOffset, ival, count, buflen;
    char 		*fields[] = { "filename", "rows", NULL },
			dataPath[PATHLEN], hdrPath[PATHLEN], hdrIn[PATHLEN],
			dataIn[PATHLEN],
			*kind, *lookupTypeStr, *sourceTypeStr;
    uchar_t		*ptr, *buffer;
    FILE		*hdrFp;
    AmDataTypeEnum	lookupType;

    AmInitUnits();
 
/* construct the paths for the output files. */
 
    sprintf (dataPath, "%s.dat.dist", filename);
    sprintf (hdrPath, "%s.hdr.dist", filename);

/* open the header and get the essentials */
    sprintf (hdrIn, "%s.hdr", filename);
    AmXResourceDbClass db (hdrIn);
    if (!db.GetResource( "type", &kind)) {
	fprintf (stderr, "failed to read kind from file\n");
	return (-1);
    }
 
/* open the input header, and output files */
    sprintf (dataIn, "%s.dat", filename);
    if (( fdIn = open (dataIn, O_RDONLY)) < 0) {
        fprintf( stderr, "failed to open input file %s\n", dataIn);
	return (-1);
    }
    if (( fdOut = open (dataPath, O_WRONLY | O_CREAT | O_TRUNC, PERMS)) < 0) {
        fprintf( stderr, "failed to open output file %s\n", dataPath);
	return (-1);
    }
    if (( hdrFp = fopen (hdrPath, "w")) == NULL) {
        fprintf( stderr, "failed to open output header %s\n", hdrPath);
	return (-1);
    }

/* get the lookup type, lookup offset, rows, and size from the header */

    if ( !db.GetResource (AmNcolorsType, &lookupTypeStr)) {
	fprintf( stderr, "failed to read lookup type from file\n");
	return (-1);
    }
    lookupType = AmStrToDataTypeEnum (lookupTypeStr);
    if ( !db.GetResource (AmNcolorsOffset, &lookupOffset)) {
	fprintf( stderr, "failed to read lookup offset from file\n");
	return( -1);
    }
    if ( !db.GetResource (AmNrows, &rows)) {
	fprintf( stderr, "failed to read rows from file\n");
	return (-1);
    }
    if ( !db.GetResource (AmNsize, &size)) {
	fprintf( stderr, "failed to read size from file\n");
	return (-1);
    }
    if ( !db.GetResource (AmNskipBytes, &skipBytes)) {
	skipBytes = 0;
    }
    if ( db.GetResource (AmNsourceType, &sourceTypeStr) &&
	 !strcmp( sourceTypeStr, "pyramidSource")) {
        fprintf( stderr, "distillation cannot be run on tiled data\n");
        return( -1);
    } 

    if (!strcmp (kind, AmNlinePlot)) {
	buffer = ( uchar_t *) malloc( size*BUFSIZE);
	if ( buffer == NULL) {
	    fprintf( stderr, "failed to allocate %d bytes\n", size*BUFSIZE);
	    return (-1);
	}

/* skip any header bytes if there are any */

	if (skipBytes != 0) {
	    if (lseek (fdIn, skipBytes, SEEK_SET) < 0) {
		fprintf (stderr, "failed to seek %d bytes\n", skipBytes);
		return (-1);
	    }
	}

/* read the data in BUFISZE*size chunks for efficiency */

	newrows = 0;
	for ( ii = 0; ii < rows; ii += BUFSIZE) {	
	    buflen = MIN( rows - ii, BUFSIZE);
	    if ( read( fdIn, buffer, buflen*size) < buflen*size) {
		fprintf( stderr, "failed to read line buffer at %d\n", 
			 ii*BUFSIZE);
		return (-1);
	    }
	    count = 0;
	    for ( jj = 0, ptr = buffer; jj < buflen; jj++, ptr += size) {

/* copy the filtered lines in place, and write out the block */

		ival = AmGetLookupVal(( char *) ptr, lookupType, lookupOffset);
		if (!AmInTable( ival, filterIds, numIds)) {
		    memcpy( &buffer[ size*count], ptr, size);
		    count++;
		}
	    }
	    if (write (fdOut, buffer, count*size) < count*size) {
		fprintf (stderr, "failed to write line buffer at %d\n", 
			 ii*BUFSIZE);
		return (-1);
	    }
	    newrows += count;
	}
    }

/* write out the new header information and close the files */

    fprintf( hdrFp, "filename:      %s\n", AmGetRootName( dataIn));
    fprintf( hdrFp, "rows:          %d\n", newrows);
    fprintf( hdrFp, "sourceFile:    %s\n", hdrIn);
    fprintf( hdrFp, "note:    	filter Odyssey\n");
    AmDumpHeaderExcept (hdrIn, fields, hdrFp);
    fclose( hdrFp);
    close( fdIn);
    close( fdOut);

    return (0);
}

extern "C" {
int distill (char* filename, int* filterIds, int numIDs)
{
  return (distill_me (filename, filterIds, numIDs));
}
	   }

