#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <Xm/Xm.h>
#include "odyssey/wardens/speech/libodyspeech_tsops.h"
#include "apps/xspeech/speech_interface.h"
#include "apps/xspeech/speech_awedio.h"
#include "paris.yyak.h"

#define VOICE_DISCONNECT "/usr/odyssey/speech/disconn.au"
#define VOICE_RECONNECT "/usr/odyssey/speech/reconn.au"

static int switchToLocal = 0;
static int switchToRemote = 0;

void strategyCallback (enum strategyType newStrategy, void* arg)
{
  static enum strategyType currentStrategy = SPEECHW_STRATEGY_REMOTE;    

  /* Set a flag so that local/remote switch will be announced.  Look
     for cases where a double-switch actually has no effect. */
  if (currentStrategy == SPEECHW_STRATEGY_LOCAL && 
      newStrategy != SPEECHW_STRATEGY_LOCAL) {
    if (switchToLocal == 0) {
      switchToRemote = 1;
    } else {
      switchToLocal = 0;
    } /* endif */
  } else if (currentStrategy != SPEECHW_STRATEGY_LOCAL &&
	     newStrategy == SPEECHW_STRATEGY_LOCAL) {
    if (switchToRemote == 0) {
      switchToLocal = 1;
    } else { 
      switchToRemote = 0;
    } /* endif */
  } /* endif */

  currentStrategy = newStrategy;
}

int checkForRelease (void* arg)
{
  XEvent event;
  int done = 0;
  Display* display = (Display *) arg;
  
  while (XPending (display) && !done) {
    XNextEvent (display, &event);
    if (event.xany.type == ButtonRelease) {
      done = 1;
    } /* endif */
  } /* endwhile */

  return (!done);
}

void doAction (char* action)
{
  printf ("%s\n", action);
  fflush(stdout);
}

int statusWindow ()
{
  int pid;

  pid = fork ();
  if (pid == 0) {
    execl ("/usr/odyssey/bin/spstatus", NULL);
    fprintf (stderr, "Can't execute status window\n");
    exit (0);
  } /* endif */ 
  return (pid);
}

int main (int argc, char* argv[]) 
{
  Display* display;
  Window root;
  XEvent event;
  char utterance[1024*1024];
  char translation[1024];
  int len;
  int fd;
  int pid;
  int rc;

  if (argc < 2) {
    fprintf (stderr, "format: spinput <model>\n");
    exit (1);
  } /* endif */
  
  speech_set_callback (strategyCallback, 0);
  /* Initialize the model */
  fd = speech_init(argv[1]);
  if (fd < 0) {
    fprintf (stderr, "Unable to initialize speech interface.  Perhaps the viceroy\n");
    fprintf (stderr, "is not started.\n");
    exit (1);
  } /* endif */

  pid = statusWindow();

  /* Grab the right mouse button */
  display = XOpenDisplay ("localhost:0");
  if (!display) {
    fprintf (stderr, "Can't open display\n");
    exit (1);
  } /* endif */

  root = RootWindowOfScreen (DefaultScreenOfDisplay (display));

  XGrabButton (display, Button3, AnyModifier, root, True, 
	       ButtonPressMask | ButtonReleaseMask,
	       GrabModeAsync, GrabModeAsync, None, None);

  /* Wait for mouse events */
  do {
    XNextEvent (display, &event);
    if (event.xany.type == ButtonPress) {
      fprintf (stderr, "Recording\n");
      kill (pid, SIGUSR1);
      len = record_audio (utterance, sizeof(utterance), checkForRelease, 
			  display); 
      fprintf (stderr, "Recognizing\n");
      kill (pid, SIGUSR1);
      rc = speechRecognize (fd, argv[1], utterance, len, translation, 
			    sizeof (translation));
      if (rc == 0) {
	fprintf (stderr, "Utterance: %s\n", translation);
	parseUtt (translation);
      } else {
	fprintf (stderr, "Recognition failed, rc = %d\n", rc);
      } /* endif */
      kill (pid, SIGUSR2);
    } /* endif */

    /* Check to see if strategy has changed */
    if (switchToLocal) {
      play_audio (VOICE_DISCONNECT);
      switchToLocal = 0;
    } /* endif */
    if (switchToRemote) {
      play_audio (VOICE_RECONNECT);
      switchToRemote = 0;
    } /* endif */

  } while (1);
    
  close (fd);
}
