#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * cellophane_main.c -- main() setup for
 *                      Odysseyized-Web browser cellophane
 *
 * Written by Eric Tilton <tilt+@cs.cmu.edu>
 *
 * XXX cmu/odyssey blurb here.
 *     
 */

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/time.h>
#include <fcntl.h>

#include <include/odyssey.h>

#include "base/ds/ds_safeq.h"
#include "base/simplewww/swww.h"
#include "odyssey/wardens/www/webwarden.h"

#include <lwp/lwp.h>
#include "cellophane_main.h"

#ifdef BW_TOOL
#include <sys/socket.h>
#include <netinet/in.h>
#include "include/bw_tool.h"
#ifdef __FreeBSD__
#include <rpc/types.h>  /* for INADDR_LOOPBACK */
#endif
const char *q_names[] = {"lowest", "low", "mid", "high"};
int mon_fd;
#endif

int lowest_quality = 10;
int low_quality    = 25;
int mid_quality    = 50;
int high_quality   = 100;

int   debugging          = 0;	 /* debugging messages off by default */
int   html_distillation  = 0;	 /* html distillation off by default */
int   image_distillation = 1;	 /* image distillation ON by default */
int   generate_log       = 0;	 /* log generation off by default */
char *remote_control     = NULL; /* remote control mode off by default */
int   adapt_power        = 0;    /* if on, adapt to power, not bw */

int speculative_fetch = 0;

FILE *logfile;

int main (int argc, char ** argv)
{    
    int     response_code;
    PROCESS pid;
    int     current;
    ds_safeq_t *request_q;

    /* mal_debug(3); */

    SWWW_debugging(0);

    current = 1;
    while(current < argc) {
	current += parse_args(current, argc, argv);
    }

    if (remote_control != NULL) {
	/* remotely control already running cellophane, then
	   exit */
	exit(do_remote());
    }

    /* initialize LWP and IOMGR */
    /* XXX note: I'm having the main thread (which is the one which
                 accepts server connections) jumped up to max priority;
		 perhaps this will fix my yielding problems. */
    if(LWP_Init(LWP_VERSION, LWP_MAX_PRIORITY, &pid) != LWP_SUCCESS) {
	fprintf(stderr, "main: can't initialize LWP!\n");
	exit(-1);
    }

    if(IOMGR_Initialize() != LWP_SUCCESS) {
	fprintf(stderr, "main: can't initialize IOMGR!\n");
	exit(-1);
    }
    
    if(SWWW_version(SWWW_VERSION)) {
	/* if library version doesn't match header version, die */
	exit(1);
    }
    
    /* XXX For now, leave this here. I'll be moving it into
       SimpleWWW at some later point, once I figure out the
       proper way to do it. */
    
    signal(SIGPIPE, SIG_IGN);

#ifdef BW_TOOL
    mon_fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (mon_fd < 0) {
	perror("socket");
    }
#endif

    /* Adapt to the current bandwidth. Begin by assuming life is pretty good,
       at ~150 KB/s . */
    if (adapt_power) {
	adapt_to_en (0, ODY_GR_POWER, 4);
    } else {
	adapt_to_bw(0, ODY_GR_NETBW, 150000);
    }

    if (generate_log)
	/* open a log file for generating traces. */
	logfile = fopen("CELLOPHANE_LOG", "w");

    SWWW_server_init(8, &request_q);
    /* Initiate SWWW_accept_requests event loop */
    response_code = SWWW_accept_requests(8001,
					 process_requests,
					 LWP,
					 request_q);
    
    /* Only gets here if event loop fails */
    return 0;
}

int parse_args(int current, int argc, char **argv)
{
    if (strcmp(argv[current], "-debug") == 0) {
	/* turn on debugging messages */
	debugging = 1;
	SWWW_debugging(1);
	return 1;
    } else if (strcmp(argv[current], "-html") == 0) {
	/* turn on HTML distillation (default is for it to be off) */
	html_distillation = 1;
	return 1;
    } else if (strcmp(argv[current], "-noimage") == 0) {
	/* turn off image distillation (default is for it to be on) */
	image_distillation = 0;
	return 1;
    } else if (strcmp(argv[current], "-generatelog") == 0) {
	/* turn on log generation */
	generate_log = 1;
	return 1;
    } else if (strcmp(argv[current], "-adaptpower") == 0) {
	adapt_power = 1;
	return 1;
    } else if (argc - current > 1) {
	/* set the various quality levels */
	if (strcmp(argv[current], "-lowest") == 0) {
	    lowest_quality = atoi(argv[current + 1]);
	    if ((lowest_quality < 1) || (lowest_quality > 100))
		USAGE;
	} else if (strcmp(argv[current], "-low") == 0) {
	    low_quality = atoi(argv[current + 1]);
	    if ((low_quality < 1) || (low_quality > 100))
		USAGE;
	} else if (strcmp(argv[current], "-mid") == 0) {
	    mid_quality = atoi(argv[current + 1]);
	    if ((mid_quality < 1) || (mid_quality > 100))
		USAGE;
	} else if (strcmp(argv[current], "-high") == 0) {
	    high_quality = atoi(argv[current + 1]);
	    if ((high_quality < 1) || (high_quality > 100))
		USAGE;
	} else if (strcmp(argv[current], "-remote") == 0) {
	    /* instead of starting cellophane, use this to remotely
	       control an already running cellophane */
	    remote_control = argv[current + 1];
	}  else {
	    USAGE;
	}

	return 2;
    } else {
	USAGE;
    }
}

int do_remote(void)
{
    struct SWWW_fetch_in  in;
    struct SWWW_fetch_out out;
    int                   status_code;
    char                  buffer[1024];


    SWWW_SFI_INIT(&in);
    in.sfi_method        = GET;
    in.sfi_url           = buffer;
    in.sfi_acceptdata    = bogus_accept;
    strcpy(buffer, "http://127.0.0.1:8001/REMOTE/"); /* the local cellophane */
    strncat(buffer, remote_control, 100);

    status_code = SWWW_fetch_url(&in, &out);

    if (status_code == 200) {
	printf("Successfully changed cellophane settings.\n");
	return(0);
    } else {
	printf("Error: %s\n", out.sfo_status);
	return(-1);
    }
}

int bogus_accept(int        reqid,
		 const void *data,
		 int        size)
{
    /* cheerfully disregard results of fetch */
    return(size);
}

int handle_remote(struct SWWW_fetch_in  *inp,
		  struct SWWW_fetch_out *outp)
{
    int success       = 1;
    int response_code = 500;

    /* compare the string against the remote control commands we're
       willing to accept */
    if (strncmp(inp->sfi_url, "/REMOTE/50", 10) == 0) {
	/* set all levels to 50 */
	/* N.B. 50 needs to go before 5, for tolerably obvious reasons
	   (c.f. the strNcmp) */
	lowest_quality = 50;
	low_quality    = 50;
	mid_quality    = 50;
	high_quality   = 50;
    } else if (strncmp(inp->sfi_url, "/REMOTE/75", 10) == 0) {
	/* set all levels to 75 */
	lowest_quality = 75;
	low_quality    = 75;
	mid_quality    = 75;
	high_quality   = 75;
    } else if (strncmp(inp->sfi_url, "/REMOTE/25", 10) == 0) {
	/* set all levels to 25 */
	lowest_quality = 25;
	low_quality    = 25;
	mid_quality    = 25;
	high_quality   = 25;
    } else if (strncmp(inp->sfi_url, "/REMOTE/5", 9) == 0) {
	/* set all levels to 5 */
	lowest_quality = 5;
	low_quality    = 5;
	mid_quality    = 5;
	high_quality   = 5;
    } else if (strncmp(inp->sfi_url, "/REMOTE/100", 11) == 0) {
	/* set all levels to 100 */
	lowest_quality = 100;
	low_quality    = 100;
	mid_quality    = 100;
	high_quality   = 100;
    } else if (strncmp(inp->sfi_url, "/REMOTE/odyssey", 15) == 0) {
	/* set levels to 5, 25, 50, and 100, respectively */
	lowest_quality = 5;
	low_quality    = 25;
	mid_quality    = 50;
	high_quality   = 100;
    } else {
	success = 0;
	outp->sfo_status      = malloc(
				 strlen("HTTP/1.0 501 "
					"Incorrect Remote Command") + 1);
	strcpy(outp->sfo_status, "HTTP/1.0 501 "
	                         "Incorrect Remote Command");
	response_code         = 501;
    }
    
    if (success) {
	/* generate a successful response */
	outp->sfo_status      = malloc(
				 strlen("HTTP/1.0 200 Remote Control OK") + 1);
	strcpy(outp->sfo_status, "HTTP/1.0 200 Remote Control OK");
	response_code         = 200;

	/* force an adaption. since we'll fail if we're outside the
	   window (and find out the right bandwidth), the number I
	   pick here is pretty arbitrary. */
	if (adapt_power) {
	    adapt_to_en(0, ODY_GR_POWER, 4);
	} else {
	    adapt_to_bw(0, ODY_GR_NETBW, 150000);
	}

	DEBUG(fprintf(stderr, "Changed quality levels to "
		              "(%d, %d, %d, %d)\n",
		      lowest_quality, low_quality,
		      mid_quality, high_quality));
    }

    outp->sfo_header_size = 0;
    outp->sfo_headers     = NULL;
    
    (*(inp->sfi_acceptheaders))(inp->sfi_reqid,
				response_code,
				outp->sfo_status,
				outp->sfo_header_size,
				outp->sfo_headers);
    
    return(response_code);
}

int refetchable(struct SWWW_headers *headers)
{
    char *refetch = NULL;
    int   result = 0;

    refetch = SWWW_gethvalue(headers, "Pragma");
    if (refetch != NULL) {
	if (strcasecmp("no-cache", refetch) == 0) {
	    result = 1;
	}

	free(refetch);
    }

    /*
     * this is what HTTP/1.1 uses; we're not supporting that yet, though.

    refetch = SWWW_gethvalue(headers, "Cache-Control");

    if (refetch != NULL) {
	free(refetch);
    }

    */

    return result;
}

int process_requests(struct SWWW_fetch_in  *inp,
		     struct SWWW_fetch_out *outp)
{
    int             fd, size, result, response_code;
    char           *ody_path;
    char            buffer[32768];
    char           *urltemp;
    struct timeval  time;

    DEBUG(fprintf(stderr, "request start: %s\n", inp->sfi_url));

    /* is this a cellophane remote control? */
    if (strncmp(inp->sfi_url, "/REMOTE/", 8) == 0) {
	return(handle_remote(inp, outp));
    }

#ifdef HEADER_DEBUGGING
    fprintf(stderr, "req hdrs (%s):\n", inp->sfi_url);
    SWWW_print_headerlist(inp->sfi_headers);
#endif

    /* XXX we are ignoring the Pragma: no-cache header here.  I
       believe that the viceroy cache will do the right thing anyway,
       but we may want to make it possible for important headers to
       get to the warden. -kwalker */

    /* workaround to deal with using Netscape's remote control features */
    if (*(inp->sfi_url) == '/')
	urltemp = (inp->sfi_url) + 1;
    else
	urltemp = inp->sfi_url;

    if (generate_log) {
	gettimeofday(&time, NULL);
	fprintf(logfile,
		"%ld.%06ld\t%s\n", time.tv_sec, time.tv_usec, urltemp);
	fflush(logfile);
    }

    /* XXX should be some locking around this if statement */
    if (speculative_fetch) {
	/* speculatively fetch something at GIF if we just moved
	   up from a lower bandwidth range */
	DEBUG(fprintf(stderr, "Speculatively fetching!\n"));
	speculative_fetch = 0;
	if (image_distillation)
	    Web_setMinQuality("image/gif", high_quality);
	if (html_distillation)
	    Web_setMinQuality("text/html", high_quality);
    }

    if (refetchable(inp->sfi_headers)) {
	DEBUG(fprintf(stderr, "Invalidating page (user hit reload?)\n"));
	Web_invalidatePage(urltemp);
    }

    fd = Web_openByURL(urltemp, &ody_path);

    if (fd == -1) {
	outp->sfo_status      = malloc(
				 strlen("HTTP/1.0 400 Bad URL") + 1);
	strcpy(outp->sfo_status, "HTTP/1.0 400 Bad URL");
	outp->sfo_header_size = 0;
	outp->sfo_headers     = NULL;
	response_code         = 400;

	(*(inp->sfi_acceptheaders))(inp->sfi_reqid,
				    response_code,
				    outp->sfo_status,
				    outp->sfo_header_size,
				    outp->sfo_headers);

	return(response_code);
    }

#if defined(__linux__) && defined(MODULE)
    Web_stat(ody_path,
	     &response_code,
	     &(outp->sfo_status),
	     &(outp->sfo_header_size),
	     &(outp->sfo_headers));
#else
    Web_fstat(fd,
	      &response_code,
	      &(outp->sfo_status),
	      &(outp->sfo_header_size),
	      &(outp->sfo_headers));
#endif
    
    (*(inp->sfi_acceptheaders))(inp->sfi_reqid,
				response_code,
				outp->sfo_status,
				outp->sfo_header_size,
				outp->sfo_headers);
    
    if (fcntl(fd, F_SETFL, O_NONBLOCK) == -1)
	fprintf(stderr, "fcntl failed: %d\n", errno);
    
    /* read the data, and pass it to the acceptdata_fn */
    size = read(fd, buffer, 32768);
    while (size == -1 && errno == EAGAIN) {
	int    rc;
	fd_set readset;
	
	FD_ZERO(&readset);
	FD_SET(fd, &readset);
	
	rc = IOMGR_Select(fd+1, (int *) &readset, NULL, NULL, NULL);
	
	if (rc != 1) {
	    fprintf(stderr,
		    "process_requests: IOMGR_Select returned %d\n", rc);
	}
	size = read(fd, buffer, 32768);
    }
    
    DEBUG(fprintf(stderr, "Got %d bytes on %s (%d)\n", size, inp->sfi_url,
		  errno));
    
    while(size > 0) {
	result = (*(inp->sfi_acceptdata))(inp->sfi_reqid,
					  buffer,
					  size);
	if(result == -1) {
	    fprintf(stderr,
		    "process_requests: "
		    "acceptdata refusing data (transfer aborted?)\n");
	    break;			/* acceptdata_fn is no longer
					   accepting data, so stop
					   sending to it */
	}
	
	size = read(fd, buffer, 32768);
	while (size == -1 && errno == EAGAIN) {
	    int    rc;
	    fd_set readset;
	    
	    FD_ZERO(&readset);
	    FD_SET(fd, &readset);
	    
	    rc = IOMGR_Select(fd+1, (int *) &readset, NULL, NULL, NULL);
	    
	    if (rc != 1) {
		fprintf(stderr,
			"process_requests: IOMGR_Select returned %d\n", rc);
	    }
	    size = read(fd, buffer, 32768);
	}
	
	DEBUG(fprintf(stderr, "Got %d bytes on %s\n", size, inp->sfi_url));
    }

    close(fd);

    DEBUG(fprintf(stderr, "request end: %s\n", inp->sfi_url));

    return(response_code);
}

void adapt_to_bw(int reqid, unsigned int rsrc, int val)
{
    struct ody_req_des new_request;
    int                flag = 1;
    int                result, value;
    webw_quality_t     quality;
    static int         last_level = 4; /* last level we were in */

    if (rsrc != ODY_GR_NETBW) {
	fprintf(stderr, "Resource (%d) is not ODY_GR_NETBW (%d)!\n",
	       rsrc, ODY_GR_NETBW);
	return;
    }

    if (val == 0) {
	/* XXX This is Jason's way of telling us the connection went
	 * down.  Is there something intelligent to do here? */
	fprintf(stderr, "Connection Down notification.\n");
	return;
    }

    new_request.ord_resource          = ODY_GR_NETBW;
    new_request.ord_version.ov_gs     = ODY_GENERIC_INTERFACE;
    new_request.ord_version.ov_type   = ODY_NO_TYPE;
    new_request.ord_version.ov_tsvnum = ODY_GENERIC_INTERFACE;
    new_request.ord_fp                = adapt_to_bw;

    /* adapt, based on bandwidth.
       XXX these numbers are totally arbitrary, and don't have
           much hysteresis built-in */
    while (flag) {
	if (val < 6000) {
	    new_request.ord_low  = 0; /* XXX fix for 0 BW notification */
	    new_request.ord_high = 6000;
	} else if (val < 20000) { /* a slow modem (less then 14.4KB/s */
	    new_request.ord_low  = 6000;
	    new_request.ord_high = 20000;
	} else if (val < 100000) {
	    new_request.ord_low  = 20000;
	    new_request.ord_high = 100000;
	} else {   /* wavelan or better */
	    new_request.ord_low  = 100000;
	    new_request.ord_high = (10000000 * 8);
	}

	
	result = ody_request("/ody/www", &new_request, &value);
	if (result) {
	    if (errno != ODY_ENOTINWINDOW) {
		/* sanity check against API problems */
		perror("ody_request");
		exit(-1);
	    }
	    DEBUG(fprintf(stderr,
			  "ody_request: thought we were at %d, really at %d\n",
		   val, value));
	    val = value;	/* try again with new value */
	} else {
	    DEBUG(fprintf(stderr,
			  "ody_request: succeeded at %d "
			  "(thought we were at %d)\n",
		   value, val));
	    flag = 0;		/* success */
	}
    }
    
    /* at this point, we've registered the proper new upcall, and
       feel pretty confident that "val" is representative of the
       current state of things. based on "val", then, set a new
       quality level */
    if (val < 6000) {
	quality = lowest_quality;
	speculative_fetch = 0;
	last_level = 1;
    } else if (val < 20000) {	        /* a modem (less then 20KB/s */
	quality = low_quality;
	speculative_fetch = 0;
	last_level = 2;
    } else if (val < 100000) {
	quality = mid_quality;
	if (last_level < 3)
	    speculative_fetch = 1;
	else
	    speculative_fetch = 0;
	last_level = 3;
    } else {			/* wavelan or better */
	quality = high_quality;
	last_level = 4;
    }

#ifdef BW_TOOL
    poke_monitor(last_level);
#endif
    if (image_distillation)
	Web_setMinQuality("image/gif", quality);
    if (html_distillation)
	Web_setMinQuality("text/html", quality);
}

void adapt_to_en(int reqid, unsigned int rsrc, int val)
{
    struct ody_req_des new_request;
    int                value;
    webw_quality_t     quality=5;
    struct timeval     tv;
    
    if (rsrc != ODY_GR_POWER) {
	fprintf(stderr, "Resource (%d) is not ODY_GR_POWER (%d)!\n",
		rsrc, ODY_GR_POWER);
	return;
    }
    
    while (1) {

	/* Set a new quality level based upon the value */
	if (val == 0) {
	    quality = 5;
	} else if (val == 1) {
	    quality = 25;
	} else if (val == 2) {
	    quality = 50;
	} else if (val == 3) {
	    quality = 75;
	} else if (val == 4) {
	    quality = 100;
	}
	
	if (image_distillation)
	    Web_setMinQuality("image/gif", quality);
	if (html_distillation)
	    Web_setMinQuality("text/html", quality);
	
	gettimeofday (&tv, NULL);
	printf ("%ld.%06ld www: adapted to fidelity level %d\n", tv.tv_sec,
		tv.tv_usec, val);
	fflush (stdout);

	new_request.ord_resource          = ODY_GR_POWER;
	new_request.ord_version.ov_gs     = ODY_GENERIC_INTERFACE;
	new_request.ord_version.ov_type   = ODY_NO_TYPE;
	new_request.ord_version.ov_tsvnum = ODY_GENERIC_INTERFACE;
	new_request.ord_fp                = adapt_to_en;
	
	/* adapt, based upon energy */
	new_request.ord_low = val;
	new_request.ord_high = val;
	
	if (ody_request("/ody/www", &new_request, &value) < 0) {
	    if (errno != ODY_ENOTINWINDOW) {
		perror("ody_request");
		exit(-1);
	    } 
	    val = value;
	} else {
	    break;
	}
    }
	
}

#ifdef BW_TOOL
void poke_monitor(int level)
{
    struct sockaddr_in mon_addr;
    int rc;

    mon_addr.sin_family = AF_INET;
    /* right way to do localhost? */
    mon_addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);	
    mon_addr.sin_port = htons(BW_WEB_MON_PORT);
    rc = sendto(mon_fd, q_names[level-1], strlen(q_names[level-1]), 0,
		(struct sockaddr *)&mon_addr, sizeof(mon_addr));
    if (rc < strlen(q_names[level-1])) {
	perror("sendto");
    }
}
#endif
