#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * ody_webserver_cjpeg.c -- this is a modified version of cjpeg.c from
 *                          the libjpeg library.
 */

#include <setjmp.h>
#include <base/ds/ds_safeq.h>
#include <lwp/lwp.h>

/*
 * cjpeg.c
 *
 * Copyright (C) 1991-1996, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains a command-line user interface for the JPEG compressor.
 * It should work on any system with Unix- or MS-DOS-style command lines.
 *
 * Two different command line styles are permitted, depending on the
 * compile-time switch TWO_FILE_COMMANDLINE:
 *	cjpeg [options]  inputfile outputfile
 *	cjpeg [options]  [inputfile]
 * In the second style, output is always to standard output, which you'd
 * normally redirect to a file or pipe to some other program.  Input is
 * either from a named file or from standard input (typically redirected).
 * The second style is convenient on Unix but is unhelpful on systems that
 * don't support pipes.  Also, you MUST use the first style if your system
 * doesn't do binary I/O to stdin/stdout.
 * To simplify script writing, the "-outfile" switch is provided.  The syntax
 *	cjpeg [options]  -outfile outputfile  inputfile
 * works regardless of which command line style is used.
 */

#include "distiller_cdjpeg.h"  /* Common decls for cjpeg/djpeg
				  applications */
#include "external/jpeg-6a/jversion.h" /* for version message */

/* Create the add-on message string table. */

#define JMESSAGE(code,string)	string ,

static const char * const cdjpeg_message_table[] = {
#include "external/jpeg-6a/cderror.h"
  NULL
};

/*
 * This routine determines what format the input file is,
 * and selects the appropriate input-reading module.
 *
 * To determine which family of input formats the file belongs to,
 * we may look only at the first byte of the file, since C does not
 * guarantee that more than one character can be pushed back with ungetc.
 * Looking at additional bytes would require one of these approaches:
 *     1) assume we can fseek() the input file (fails for piped input);
 *     2) assume we can push back more than one character (works in
 *        some C implementations, but unportable);
 *     3) provide our own buffering (breaks input readers that want to use
 *        stdio directly, such as the RLE library);
 * or  4) don't put back the data, and modify the input_init methods to assume
 *        they start reading after the start of file (also breaks RLE library).
 * #1 is attractive for MS-DOS but is untenable on Unix.
 *
 * The most portable solution for file types that can't be identified by their
 * first byte is to make the user tell us what they are.  This is also the
 * only approach for "raw" file types that contain only arbitrary values.
 * We presently apply this method for Targa files.  Most of the time Targa
 * files start with 0x00, so we recognize that case.  Potentially, however,
 * a Targa file could start with any byte value (byte 0 is the length of the
 * seldom-used ID field), so we provide a switch to force Targa input mode.
 */

LOCAL(cjpeg_source_ptr)
select_file_type (j_compress_ptr cinfo, char *content_type)
{

    /* N.B. I added "UNDEF_" to all of the symbols below that I don't
            want to care about today, because I'm trying to use the
            jpeg library in a quick way to see if this all works.
	    I'll atone for my sins later. */

#ifdef UNDEF_BMP_SUPPORTED
    if (strcasecmp(content_type, "image/bmp") == 0)
	return jinit_read_bmp(cinfo);
#endif

#ifdef GIF_SUPPORTED
    if (strcasecmp(content_type, "image/gif") == 0)
	return jinit_read_gif(cinfo);
#endif

#ifdef UNDEF_PPM_SUPPORTED
    if (strcasecmp(content_type, "image/ppm") == 0)
	return jinit_read_ppm(cinfo);
#endif

#ifdef UNDEF_RLE_SUPPORTED
    if (strcasecmp(content_type, "image/rle") == 0)
	return jinit_read_rle(cinfo);
#endif

#ifdef UNDEF_TARGA_SUPPORTED
    if (strcasecmp(content_type, "image/targa") == 0)
	return jinit_read_targa(cinfo);
#endif

    ERREXIT(cinfo, JERR_UNKNOWN_FORMAT);
    
    return NULL;			/* suppress compiler warnings */
}

/* error handling, stolen from example.c of the jpeg library code */

METHODDEF(void)
my_error_exit (j_common_ptr cinfo)
{
  jmp_buf *return_target;

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
  (*cinfo->err->output_message) (cinfo);

  fprintf(stderr, "my_error_exit attempting to return from fatal error\n");

  LWP_GetRock(90210, (char **) &return_target);

  /* Return control to the setjmp point */
  longjmp(*return_target, 1);
}

/*
 * The main program.
 */

int
cjpeg_main (ds_safeq_t                  *readq,
	    char                        *content_type,
	    int                          quality,
	    struct jpeg_destination_mgr *dest)
{
    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr       jerr;
    cjpeg_source_ptr            src_mgr;
    JDIMENSION                  num_scanlines;
    jmp_buf                     return_target;
    
    /* Initialize the JPEG compression object with default error handling,
       but then override error_exit */
    cinfo.err = jpeg_std_error(&jerr);
    jerr.error_exit = my_error_exit;
    if (setjmp(return_target)) {
	/* if we get here, the JPEG code has had a fatal error.
	 * clean up and get out.
	 */
	fprintf(stderr, "my_error_exit did return from fatal error\n");
	jpeg_destroy_compress(&cinfo);
	return 0;
    } else {
	LWP_NewRock(90210, (char *) &return_target);
    }

    jpeg_create_compress(&cinfo);
    /* Add some application-specific error messages (from cderror.h) */
    jerr.addon_message_table = cdjpeg_message_table;
    jerr.first_addon_message = JMSG_FIRSTADDONCODE;
    jerr.last_addon_message = JMSG_LASTADDONCODE;
    
    /* Initialize JPEG parameters.
     * Much of this may be overridden later.
     * In particular, we don't yet know the input file's color space,
     * but we need to provide some value for jpeg_set_defaults() to work.
     */
    
    cinfo.in_color_space = JCS_RGB; /* arbitrary guess */
    jpeg_set_defaults(&cinfo);

    /* Figure out the input file format, and set up to read it. */
    src_mgr = select_file_type(&cinfo, content_type);
    INIT_CJPEG_SAFEQ(&(src_mgr->input_queue), readq);
    
    /* Read the input file header to obtain file size & colorspace. */
    (*src_mgr->start_input) (&cinfo, src_mgr);
    
    /* Now that we know input colorspace, fix colorspace-dependent defaults */
    jpeg_default_colorspace(&cinfo);

    /* set the quality as requested */
    jpeg_set_quality(&cinfo, quality, FALSE);
    
    /* Specify data destination for compression */
    cinfo.dest = dest;
    
    /* Start compressor */
    jpeg_start_compress(&cinfo, TRUE);
    
    /* Process data */
    while (cinfo.next_scanline < cinfo.image_height) {
	num_scanlines = (*src_mgr->get_pixel_rows) (&cinfo, src_mgr);
	(void) jpeg_write_scanlines(&cinfo, src_mgr->buffer, num_scanlines);
    }
    
    /* Finish compression and release memory */
    (*src_mgr->finish_input) (&cinfo, src_mgr);
    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);
    
    /* All done. */
    /* exit(jerr.num_warnings ? EXIT_WARNING : EXIT_SUCCESS); D'OH! */
    return 0;			/* suppress no-return-value warnings */
}
