#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <sys/uio.h>
#include "include/odyssey.h"
#include "include/odytypes.h"
#include "DigestDialog.h"
#include "WWWSets.h"
#include "SetsMenu.h"
#include "sets.h"
#include "xutils.h"
#include <Xm/Xm.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>

DigestDialogLL* pDigestDialogList = 0;

/* findDigestDialog() finds the Digest Dialog assocaited with the
   specified window.  A window is associated with a digest dialog if it
   is the dialog window or one of its subchildren.  The function
   returns a pointer to the dialog data structure if an associated
   dialog is found, and NULL otherwise. */
Digest_ClientData* findDigestDialog (Display* display, Window w)
{
  DigestDialogLL* pNode;

  for (pNode = pDigestDialogList; pNode; pNode = pNode->pNext) {
    if (isDescendant (display, w, XtWindow(pNode->pClientData->dialog))) {
      return pNode->pClientData;
    } /* endif */
  } /* endfor */

  return (NULL);
}
    
/* firstDigestDialog() returns the data structure associated with the first
   digest dialog in the list.  If no set dialog is in the list, the function
   reuturns NULL.  Since the list is maintained in order of usage, the
   first dialog in the list is the most recent dialog touched by the user. */
Digest_ClientData* firstDigestDialog ()
{
  if (pDigestDialogList) {
    return (pDigestDialogList->pClientData);
  } else {
    return (NULL);
  } /* endif */
}

/* RemoveFromDialogList() is called to remove a digest dialog from the linked
   list when a set is closed. */
static void RemoveFromDialogList (Digest_ClientData* pClientData)
{
  DigestDialogLL* pNode;
  DigestDialogLL* pLast = 0;
  
  for (pNode = pDigestDialogList; pNode; pNode = pNode->pNext) {
    if (pNode->pClientData == pClientData) {

      if (pLast) {
	pLast->pNext = pNode->pNext;
      } else {
	pDigestDialogList = pNode->pNext;
      } /* endif */
      FREE (pNode);
      return;

    } else {
      pLast = pNode;
    } /* endif */
  } /* endfor */

  /* There should always be a dialog to delete */
  ASSERT (0);
}

/* AdjustDialogList() is called whenever a dialog is touched or created
   by the user.  If the specified set dialog data structure already 
   exists in the list, the dialog is moved to the front of the list.  If
   the data structure is not in the list, it is inserted at the front of
   the list. */
static void AdjustDialogList (Digest_ClientData* pClientData) 
{
  DigestDialogLL* pNode;
  DigestDialogLL* pLast = 0;
  
  for (pNode = pDigestDialogList; pNode; pNode = pNode->pNext) {
    if (pNode->pClientData == pClientData) {
      if (pLast) {
	pLast->pNext = pNode->pNext;
	pNode->pNext = pDigestDialogList;
	pDigestDialogList = pNode;
      } /* endif */
      return;
    } /* endif */
  } /* endfor */

  ALLOC (pNode, DigestDialogLL);
  pNode->pClientData = pClientData;
  pNode->pNext = pDigestDialogList;
  pDigestDialogList = pNode;
}

/* Digest_Open_Member () displays the specified menu-item (first item
   is identified as item 0. */
void Digest_Open_Member (Digest_ClientData* pClientData, int mem)
{
  Arg args[2];
  XmStringTable sel;
  int cnt;
  char* url = 0;

  /* Determine which set member was selected */
  AdjustDialogList (pClientData);
  XtSetArg(args[0], XmNitemCount, &cnt);
  XtSetArg(args[1], XmNitems, &sel);
  XtGetValues(pClientData->list, args, 2);

  if (mem < cnt) {	

    /* Open that item and display it. */
    XmStringGetLtoR (sel[mem], "fetch", &url);
    if (!url) {
      XmStringGetLtoR (sel[mem], "nofetch", &url);
    } /* endif */
    if (url) {
      netscapeRemoteOpenURL (url, pClientData->display, 
			     pClientData->pNetscapeWindow, 0);  
    } else {
      printf ("Can't get specified list item\n");
    } /* endif */

  } else {
     /* Tell user to select an actual member */
     printf ("The specified memeber does not exist.");
  } /* endif */

  /* Dealolocate data allocated by X */
  FREE (url);
}

/* This function displays the set member selected by the user from 
   the digest list by sending a command to Netscape. */
void Digest_Open_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  Digest_ClientData* pClientData;
  Arg args[2];
  XmStringTable sel;
  int cnt;
  char* url = 0;

  /* Determine which set member was selected */
  pClientData = (Digest_ClientData *) client_data;
  AdjustDialogList (pClientData);
  XtSetArg(args[0], XmNselectedItemCount, &cnt);
  XtSetArg(args[1], XmNselectedItems, &sel);
  XtGetValues(pClientData->list, args, 2);

  /* Should only be 1 selected item. */
  if (cnt == 1) {	

    /* Open that item and display it. */
    XmStringGetLtoR (sel[0], "fetch", &url);
    if (!url) {
      XmStringGetLtoR (sel[0], "nofetch", &url);
    } /* endif */
    if (url) {
      netscapeRemoteOpenURL (url, pClientData->display, 
			     pClientData->pNetscapeWindow, 0);  
    } else {
      printf ("Can't get specified list item\n");
    } /* endif */

  } else {
     /* Tell user to select a member */
    ErrorMessage (w, "No Member Selected", 
		  "You must select one member to perform this operation.");
  } /* endif */

  /* Dealolocate data allocated by X */
  FREE (url);
}

/* This function is called when the user selects the Rewind push
   button on the dialog.  It rewinds the digest and deletes all 
   elements of the dialog list. */
void Digest_Rewind_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  Digest_ClientData* pClientData;

  pClientData = (Digest_ClientData *) client_data;  
  AdjustDialogList (pClientData);

  RewindSetDigest (pClientData->set);
  XmListDeleteAllItems (pClientData->list);
  netscapeString ("The digest has been reinitialized", pClientData->display,
		  pClientData->pNetscapeWindow);
}

/* This function is called when the user selects the Re-Digest push
   button on the dialog.  It gets another digest of the set, and
   repopulates the dialog with the new digest data */
void Digest_Redigest_Callback (Widget w, XtPointer client_data, 
			       XtPointer call_data)
{
  Digest_ClientData* pClientData;
  Digest digest;
  DigestElem* pElem;

  /* Get a digest of the set */
  pClientData = (Digest_ClientData *) client_data;
  AdjustDialogList (pClientData);

  GetDigestOfSet (pClientData->set, &digest);

  /* Repopulate dialog */
  Populate_Digest_List (pClientData->list, pClientData->label, &digest);

  /* Deallocate the digest members*/
  while (digest.pList) {
    pElem = digest.pList;
    digest.pList = pElem->pNext;
    free (pElem);
  } /* end while */
}

/* This function is called when the user selects the Cancel push
   button on the dialog.  It deallocates the client data structure and
   destroys the dialog. */
void Digest_Cancel_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  Digest_ClientData* pClientData;

  /* Close the dialog */
  pClientData = (Digest_ClientData *) client_data;
  RemoveFromDialogList (pClientData);

  XtDestroyWidget (pClientData->dialog);
  free (pClientData);
}

/* This function populates the digest window with the contents of a set
   digest.  It modifies the label to reflect whether the digest is 
   partial or full, and adds setmembers to the digest list. */
void Populate_Digest_List (Widget list, Widget label, Digest* pDigest)
{				  
  DigestElem* pElem;
  Arg args[1];
  XmString xmstr;

  /* First adjust the label to reflect state of digest */
  if (pDigest->isFullDigest) {
    xmstr = XmStringCreateLtoR("Full digest of", XmSTRING_DEFAULT_CHARSET);
  } else {
    xmstr = XmStringCreateLtoR("Partial digest of", XmSTRING_DEFAULT_CHARSET);
  } /* endif */
  XtSetArg(args[0], XmNlabelString, xmstr);
  XtSetValues(label, args, 1);
  XmStringFree(xmstr);  

  /* Add members from digest to the list */
  for (pElem = pDigest->pList; pElem; pElem = pElem->pNext) {
    if (pElem->isFetched) {
      xmstr = XmStringCreateLtoR(pElem->name, "fetch");
    } else {
      xmstr = XmStringCreateLtoR(pElem->name, "nofetch");
    } /* endif */
    if (!XmListItemExists(list, xmstr)) {
      XmListAddItem(list, xmstr, 1);
    } /* endif */
    fflush (stdout);
    XmStringFree(xmstr);
  } /* endfor */
}

/* This function creates a dialog for the digest of an open set.  The
   set id, name, and digest should be passed into the function, along
   with the widget for the root application window. */
void Create_Digest_Dialog (Widget parent, int set, char* name, Digest* pDigest,
			   Display* display, Window* pNetscapeWindow)
{
  Widget dialog, dialog_frame, list_form, buttons_form;
  Widget label, text, list;
  Widget openButton, rewindButton, redigestButton, cancelButton;
  Arg args[11];
  XmString xmstr;
  Digest_ClientData* pClientData;  
    
  /* Create a new client data structure for this dialog */
  pClientData = (Digest_ClientData *) malloc (sizeof(Digest_ClientData));

  /* Create dialog box */
  /* XmxSetArg (XmNwidth, 475); */

  xmstr = XmStringCreateLtoR ("Digest Window", XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0], XmNautoUnmanage, (XtArgVal) False);
  XtSetArg(args[1], XmNdialogTitle, (XtArgVal) xmstr);
  dialog = XmCreateFormDialog (parent, "dialog", args, 2);
  XmStringFree (xmstr);  

  /* Create a frame for the dialog window */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  dialog_frame = XtCreateManagedWidget ("frame", xmFrameWidgetClass,
					dialog, args, 5);   
      
  /* Create the main form for the dialog */
  list_form = XtCreateManagedWidget ("form", xmFormWidgetClass, dialog_frame,
				     NULL, 0);

  /* Create a label for the entry field */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  label = XtCreateManagedWidget ("Partial digest of: ", xmLabelWidgetClass,
                                 list_form, args, 4); 

  /* Create a read-only text field to show the set's name. */
  XtSetArg(args[0], XmNwidth, (XtArgVal) 310);
  XtSetArg(args[1], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 5);
  XtSetArg(args[3], XmNrightOffset, (XtArgVal) 10);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNleftWidget, (XtArgVal) label);
  XtSetArg(args[8], XmNeditable, 0); /* Don't allow user to edit this field */
  XtSetArg(args[9], XmNvalue, name);
  text = XmCreateTextField (list_form, "textfield", args, 10);
  XtManageChild (text);

  /* Create the digest list */
  XtSetArg(args[0], XmNresizable, False);
  XtSetArg(args[1], XmNscrollBarDisplayPolicy, XmAS_NEEDED);
  XtSetArg(args[2], XmNlistSizePolicy, XmCONSTANT);
  XtSetArg(args[3], XmNvisibleItemCount, 10);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[5], XmNtopWidget, (XtArgVal) text);
  XtSetArg(args[6], XmNtopOffset, 8);
  XtSetArg(args[7], XmNleftAttachment, XmATTACH_FORM);	
  XtSetArg(args[8], XmNleftOffset, 8);
  XtSetArg(args[9], XmNrightAttachment, XmATTACH_FORM); 
  XtSetArg(args[10], XmNrightOffset, 8);
  list = XmCreateScrolledList(list_form, "scrolled_list", args, 11);
  XtManageChild(list);

  /* defaultAction gets triggered on double click and sends item with it...*/
  XtAddCallback(list, XmNdefaultActionCallback, Digest_Open_Callback,
		pClientData);

  /* Create form for push buttons */
  XtSetArg(args[0], XmNverticalSpacing, (XtArgVal) 8);
  XtSetArg(args[1], XmNfractionBase, (XtArgVal) 4);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNtopWidget, (XtArgVal) list);
  buttons_form = XtCreateManagedWidget ("form", xmFormWidgetClass, list_form,
					args, 7);

  /* Create the Open push button */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNleftPosition, 0);
  XtSetArg(args[5], XmNrightPosition, 1);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 8);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 4);
  openButton = XtCreateManagedWidget ("Open member", xmPushButtonWidgetClass,
				      buttons_form, args, 8);
  XtAddCallback (openButton, XmNactivateCallback, Digest_Open_Callback, 
		 (XtPointer) pClientData);

  /* Create the Rewind push button */
  XtSetArg(args[4], XmNleftPosition, 1);
  XtSetArg(args[5], XmNrightPosition, 2);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 4);
  rewindButton = XtCreateManagedWidget ("Rewind", xmPushButtonWidgetClass,
				      buttons_form, args, 8);
  XtAddCallback (rewindButton, XmNactivateCallback, Digest_Rewind_Callback, 
		 (XtPointer) pClientData);

  /* Create the Re-Digest push button */
  XtSetArg(args[4], XmNleftPosition, 2);
  XtSetArg(args[5], XmNrightPosition, 3);
  redigestButton = XtCreateManagedWidget ("Re-Digest", xmPushButtonWidgetClass,
					  buttons_form, args, 8);
  XtAddCallback (redigestButton, XmNactivateCallback, 
		 Digest_Redigest_Callback, (XtPointer) pClientData);

  /* Create the Cancel push button */
  XtSetArg(args[4], XmNleftPosition, 3);
  XtSetArg(args[5], XmNrightPosition, 4);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 8);
  cancelButton = XtCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
				        buttons_form, args, 8);
  XtAddCallback (cancelButton, XmNactivateCallback, Digest_Cancel_Callback, 
		 (XtPointer) pClientData);

  /* Populate Client Data structure */
  pClientData->parent = parent;
  pClientData->dialog = dialog;
  pClientData->set = set;
  pClientData->display = display;
  pClientData->list = list;
  pClientData->label = label;
  pClientData->pNetscapeWindow = pNetscapeWindow;

  /* Populate the list with data */
  Populate_Digest_List (list, label, pDigest);

  XtManageChild (dialog);

  AdjustDialogList (pClientData);
}
