#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "NewDialog.h"
#include "SetDialog.h"
#include "SetsMenu.h"
#include "WWWSets.h"
#include "sets.h"
#include <Xm/Xm.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/SeparatoG.h>
#include <Xm/MessageB.h>
#include <stdio.h>
#include <stdlib.h>

typedef struct {
  Widget parent;
  Widget menu;
  Widget dialog; 
  Widget text;
  Widget unionWidget;
  Widget intersectWidget;
  Display* display;
  Window* pNetscapeWindow;
} OpenSet_ClientData;

/* This function is called when the user selects the Open push button on the
   dialog.  It opens the set specified in the dialog's text field.  This 
   dialog is destroyed and a set dialog is created for the newly opened
   set. */
void OpenSet_Open_Callback (Widget w, XtPointer client_data, 
			    XtPointer call_data)
{
  char* url;
  int set;
  OpenSet_ClientData* pClientData;
  Widget parent, menu, unionWidget, intersectWidget;
  Display* display;
  Window* pNetscapeWindow;
  
  /* Save some client data for later use */
  pClientData = (OpenSet_ClientData *) client_data;
  parent = pClientData->parent;
  menu = pClientData->menu;
  unionWidget = pClientData->unionWidget;
  intersectWidget = pClientData->intersectWidget;
  display = pClientData->display;
  pNetscapeWindow = pClientData->pNetscapeWindow;
  
  /* Open the specified set */
  url = XmTextGetString (pClientData->text);
  if (strcmp (url, "")) {

    set = OpenSetForURL (url);

    /* If the open was successful, have Netscape fetch the specified URL */
    netscapeRemoteOpenURL (url, pClientData->display, 
			   pClientData->pNetscapeWindow, 0);

    /* Close the New dialog */
    OpenSet_Cancel_Callback (w, client_data, call_data);
    
    /* Create a dialog for the opened set */
    Create_Set_Dialog (parent, menu, unionWidget, intersectWidget, set, url, 
		       display, pNetscapeWindow);
  } else {
    /* Display an error that no text was entered */
    ErrorMessage (w, "No Set Name Entered", 
		  "You must enter the name of the set to be opened");
  } /* endif */
}

/* This function is called when the user selects the Clear push button on the
   dialog.  It deletes any text in the dialog's text field. */
void OpenSet_Clear_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  OpenSet_ClientData* pClientData;

  pClientData = (OpenSet_ClientData *) client_data;
  XmTextSetString (pClientData->text, "");
  XmTextShowPosition (pClientData->text, 0);
}

/* This function is called when the user selects the Cancel push button on the
   dialog.  It deallocates the call-data structure and closes the dialog. */
void OpenSet_Cancel_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  OpenSet_ClientData* pClientData;

  pClientData = (OpenSet_ClientData *) client_data;
  XtDestroyWidget (pClientData->dialog);
  free (pClientData);
}

/* This function is called when the user selects the New menu-item on the
   Sets Menu.  It displays a dialog that allows the user to specify a new 
   set to be opened */
void Create_OpenSet_Dialog (Widget w, Widget menu, Widget unionWidget,
			    Widget intersectWidget, Display* display, 
			    Window* pNetscapeWindow)

{
  Widget dialog, dialog_frame, sets_form, label, text, dialog_sep;
  Widget buttons_form, openButton, clearButton, cancelButton;
  XmString xmstr;
  OpenSet_ClientData* pClientData;
  Arg args[8];

  /* Create a new client data structure for this dialog */
  pClientData = (OpenSet_ClientData *) malloc (sizeof(OpenSet_ClientData));

  /* Create dialog box */
  xmstr = XmStringCreateLtoR ("Open Dynamic Set", XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0], XmNautoUnmanage, (XtArgVal) False);
  XtSetArg(args[1], XmNdialogTitle, (XtArgVal) xmstr);
  dialog = XmCreateFormDialog (w, "dialog", args, 2);
  XmStringFree (xmstr);

  /* Create a frame for the dialog window */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  dialog_frame = XtCreateManagedWidget ("frame", xmFrameWidgetClass,
					dialog, args, 5);

  /* Create the main form for the dialog */
  sets_form = XtCreateManagedWidget ("form", xmFormWidgetClass, dialog_frame,
				     NULL, 0);
       
  /* Create a label for the entry field */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  label = XtCreateManagedWidget ("Set Name: ", xmLabelWidgetClass,
                                 sets_form, args, 4);  

  /* Create an entry field */
  XtSetArg(args[0], XmNwidth, (XtArgVal) 310);
  XtSetArg(args[1], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 5);
  XtSetArg(args[3], XmNrightOffset, (XtArgVal) 10);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNleftWidget, (XtArgVal) label);
  text = XmCreateTextField (sets_form, "textfield", args, 8);
  XtManageChild (text);
  XtAddCallback (text, XmNactivateCallback, OpenSet_Open_Callback, 
		 (XtPointer) pClientData);
  
  /* Create form for push buttons */
  XtSetArg(args[0], XmNverticalSpacing, (XtArgVal) 8);
  XtSetArg(args[1], XmNfractionBase, (XtArgVal) 4);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  buttons_form = XtCreateManagedWidget ("form", xmFormWidgetClass, sets_form,
					args, 5);

  /* Create a separator between entry field and push buttons */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNtopWidget, (XtArgVal) text);
  XtSetArg(args[6], XmNbottomWidget, (XtArgVal) buttons_form);
  dialog_sep = XmCreateSeparatorGadget (sets_form, "separator", args, 7);
  XtManageChild (dialog_sep);
      
  /* Create the Open push button */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNleftPosition, 0);
  XtSetArg(args[5], XmNrightPosition, 1);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 8);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 4);
  openButton = XtCreateManagedWidget ("Open", xmPushButtonWidgetClass,
				      buttons_form, args, 8);
  XtAddCallback (openButton, XmNactivateCallback, OpenSet_Open_Callback, 
		 (XtPointer) pClientData);

  /* Create the Clear push button */
  XtSetArg(args[4], XmNleftPosition, 1);
  XtSetArg(args[5], XmNrightPosition, 2);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 4);
  clearButton = XtCreateManagedWidget ("Clear", xmPushButtonWidgetClass,
				      buttons_form, args, 8);
  XtAddCallback (clearButton, XmNactivateCallback, OpenSet_Clear_Callback, 
		 (XtPointer) pClientData);

  /* Create the Cancel push button */
  XtSetArg(args[4], XmNleftPosition, 2);
  XtSetArg(args[5], XmNrightPosition, 3);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 8);
  cancelButton = XtCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
				        buttons_form, args, 8);
  XtAddCallback (cancelButton, XmNactivateCallback, OpenSet_Cancel_Callback, 
		 (XtPointer) pClientData);

  /* Populate Client Data structure */
  pClientData->parent = w;
  pClientData->menu = menu;
  pClientData->dialog = dialog;
  pClientData->text = text;
  pClientData->unionWidget = unionWidget;
  pClientData->intersectWidget = intersectWidget;
  pClientData->display = display;
  pClientData->pNetscapeWindow = pNetscapeWindow;

  XtManageChild (dialog);
}
