#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "SetsMenu.h"
#include "NewDialog.h"
#include "UnionDialog.h"
#include "SpeechDialog.h"
#include "sets.h"
#include "SetDialog.h"
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeB.h>
#include <Xm/MessageB.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

extern int interpret_pages;
extern int proxy_interface;

/* Generic routine to display an informational message dialog */
void InfoMessage (Widget parent, char* title, char* message)
{
  Widget dialog;
  XmString str_title, str_message;
  Arg args[2];

  str_title = XmStringCreateLtoR (title, XmSTRING_DEFAULT_CHARSET);
  str_message = XmStringCreateLtoR (message, XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0], XmNdialogTitle, (XtArgVal) str_title);
  XtSetArg(args[1], XmNmessageString, (XtArgVal) str_message);
  dialog = XmCreateInformationDialog (parent, "dialog", args, 2);
  XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));  
  XmStringFree (str_title);
  XmStringFree (str_message);

  XtManageChild (dialog);
}

/* Generic routine to display an error message dialog */
void ErrorMessage (Widget parent, char* title, char* message)
{
  Widget dialog;
  XmString str_title, str_message;
  Arg args[2];

  str_title = XmStringCreateLtoR (title, XmSTRING_DEFAULT_CHARSET);
  str_message = XmStringCreateLtoR (message, XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0], XmNdialogTitle, (XtArgVal) str_title);
  XtSetArg(args[1], XmNmessageString, (XtArgVal) str_message);
  dialog = XmCreateErrorDialog (parent, "dialog", args, 2);
  XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));  
  XmStringFree (str_title);
  XmStringFree (str_message);

  XtManageChild (dialog);
}

/* This function is called when the user selects the Quit menu-item
   from the sets menu.  The reparenting of all Netscape windows is
   undone, so that they remain on the desktop.  Then, the application
   is exited, causing all windows to be destroyed. */
void SetsMenu_Quit_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  WindowData* pWindowData;
  WindowItem* pItem;

  /* Undo the reparenting of Netscape windows */
  pWindowData = (WindowData *) client_data;
  for (pItem = pWindowData->pSetsData->pWindowList; pItem; 
       pItem = pItem->pNext) {
    unparentNetscapeWindow (pWindowData->pSetsData->display, 
			    pItem->pData->netscapeWindow,
			    pItem->pData->setWindow);
  } /* endfor */

  /* We're done - exit the application */
  exit (0);
}

/* This function is called when the user selects the Open
   new... menu-item on the Sets Menu.  It displays a dialog that
   allows the user to specify a new set to be opened */
void SetsMenu_New_Callback (Widget w, XtPointer client_data, 
			    XtPointer call_data)
{
  WindowData* pWindowData;

  /* Display the New dialog */
  pWindowData = (WindowData *) client_data;
  Create_OpenSet_Dialog (pWindowData->mainWidget, 
			 pWindowData->setsMenuWidget, 
			 pWindowData->unionWidget,
			 pWindowData->intersectWidget, 
			 pWindowData->pSetsData->display,
			 &pWindowData->netscapeWindow);
}

/* This function is called when the user selects the Open as set
   menu-item on the Sets Menu.  It opens a set corresponding to the
   URL currently being viewed. */
void SetsMenu_This_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  char* url;
  int set;
  WindowData* pWindowData;
  char msg[100];

  pWindowData = (WindowData *) client_data;

  /* Open the set corresponding to the current URL */
  url = getNetscapeURL (pWindowData->pSetsData->display, 
			pWindowData->netscapeWindow);
  if (url) {

    set = OpenSetForURL (url);
    if (set >= 0) {
      /* Create a dialog for the opened set */
      Create_Set_Dialog (pWindowData->mainWidget, pWindowData->setsMenuWidget, 
			 pWindowData->unionWidget, 
			 pWindowData->intersectWidget, set,
			 url, pWindowData->pSetsData->display, 
			 &pWindowData->netscapeWindow);
    } else {
      sprintf (msg, "An error occurred opening the set.  The associated "
	            "return code is %d\n", set);
      ErrorMessage (pWindowData->mainWidget, "Set Open Error", msg);
    } /* endif */

    free (url);

  } else {
    ErrorMessage (pWindowData->mainWidget, "Set Open Error", 
		  "Apparently no URL is being displayed");
  } /* endif */
}

/* This function is called when the user selects the Intersect
   menu-item on the Actions Menu.  It displays a dialog that allows
   the user to create an intersection of two open sets. */
void SetsMenu_Intersect_Callback (Widget w, XtPointer client_data, 
				  XtPointer call_data)
{
  WindowData* pWindowData;

  /* Display the New Set dialog */
  pWindowData = (WindowData *) client_data;
  
  Create_Union_Dialog (w, pWindowData->setsMenuWidget, 
		       pWindowData->unionWidget,
		       pWindowData->intersectWidget, 
		       pWindowData->pSetsData->display,
		       &pWindowData->netscapeWindow, False);
}

/* This function is called when the user selects the Union
   menu-item on the Actions Menu.  It displays a dialog that allows
   the user to create a union of two open sets. */
void SetsMenu_Union_Callback (Widget w, XtPointer client_data, 
			      XtPointer call_data)
{
  WindowData* pWindowData;

  /* Display the Union dialog */
  pWindowData = (WindowData *) client_data;
  Create_Union_Dialog (w, pWindowData->setsMenuWidget, 
		       pWindowData->unionWidget,
		       pWindowData->intersectWidget, 
		       pWindowData->pSetsData->display,
		       &pWindowData->netscapeWindow, True);
}

/* This function is called when the user selects the Interpret search
   results menu-item on the Options menu.  It causes SETS to interpret
   the results of certain search engines intelligently. */
void SetsMenu_Search_Callback (Widget w, XtPointer client_data, 
			       XtPointer call_data)
{
  WindowData* pWindowData;

  pWindowData = (WindowData *) client_data;

  interpret_pages = interpret_pages ? 0 : 1;
}

/* This function is called when the user selects the Show speech details
   menu-item on the Options menu. It pops up a dialog showing information
   about the current state of speech-to-text processing. */
void SetsMenu_Speech_Callback (Widget w, XtPointer client_data, 
			       XtPointer call_data)
{
  WindowData* pWindowData;

  pWindowData = (WindowData *) client_data;

  Create_Speech_Dialog (w);
}


/* This function creates the main window for the extended application.
   The main window consists of a menu-bar which allows the user to
   invoke various sets extensions, and a frame window on which the
   extended application will be placed.  The function returns a structure
   containing info about the created window on success. */
WindowData* Create_Application_Window (Widget topLevel, SetsData* pSetsData)
{
  WindowData* pWindowData;
  WindowItem* pWindowItem;
  Widget menuBar = 0, setsMenu = 0, actionsMenu, optionsMenu;
  Widget setsButton, actionsButton, optionsButton;
  Widget newButton, thisButton, closeButton;
  Widget intersectButton = 0, unionButton = 0;
  Widget searchButton, speechButton;
  Widget mainWindow, frame;

  /* Allocate a new window data structure for this window */
  pWindowData = (WindowData *) malloc (sizeof(WindowData));
  pWindowItem = (WindowItem *) malloc (sizeof(WindowItem));
  if (!pWindowData || !pWindowItem) {
    printf ("Memory allocation error\n");
    exit (1);
  } /* endif */

  /* Create main window to be the body of the window */
  mainWindow = XtVaCreateManagedWidget ("mainWindow", xmMainWindowWidgetClass,
					topLevel, XmNmappedWhenManaged, False,
					NULL);

  /* Create frame window (which will be hidden by extended application */
  frame = XtVaCreateManagedWidget ("frame", xmFrameWidgetClass, mainWindow,
				   NULL);

  /* Create menu bar as child of top-level window */
  if (proxy_interface) {
    menuBar = XmCreateMenuBar (mainWindow, "menuBar", NULL, 0);
    XtManageChild (menuBar);
    
    /* Create sets menu */
    setsButton = XtVaCreateManagedWidget ("Set", xmCascadeButtonWidgetClass, 
					  menuBar, NULL);
    setsMenu = XmCreatePulldownMenu (menuBar, "setsMenu", NULL, 0);
    thisButton = XtVaCreateManagedWidget ("Open as set", 
					  xmPushButtonWidgetClass, 
					  setsMenu, NULL);
    XtAddCallback (thisButton, XmNactivateCallback, SetsMenu_This_Callback, 
		   pWindowData);
    newButton = XtVaCreateManagedWidget ("Open new...", 
					 xmPushButtonWidgetClass, 
					 setsMenu, NULL);
    XtAddCallback (newButton, XmNactivateCallback, SetsMenu_New_Callback, 
		   pWindowData);
    
    actionsButton = XtVaCreateManagedWidget ("Actions", 
					     xmCascadeButtonWidgetClass, 
					     menuBar, NULL);
    actionsMenu = XmCreatePulldownMenu (menuBar, "actionsMenu", NULL, 0);
    intersectButton = XtVaCreateManagedWidget ("Intersect", 
					       xmPushButtonWidgetClass, 
					       actionsMenu, NULL);
    XtSetSensitive (intersectButton, False);
    XtAddCallback (intersectButton, XmNactivateCallback, 
		   SetsMenu_Intersect_Callback, pWindowData);
    unionButton = XtVaCreateManagedWidget ("Union", 
					   xmPushButtonWidgetClass, 
					 actionsMenu, NULL);
    XtSetSensitive (unionButton, False);
    XtAddCallback (unionButton, XmNactivateCallback, 
		   SetsMenu_Union_Callback, pWindowData);
    
    optionsButton = XtVaCreateManagedWidget ("Options", 
					     xmCascadeButtonWidgetClass, 
					     menuBar, NULL);
    optionsMenu = XmCreatePulldownMenu (menuBar, "optionsMenu", NULL, 0);
    searchButton = XtVaCreateManagedWidget ("Interpret search results", 
					    xmToggleButtonWidgetClass, 
					    optionsMenu, NULL);
    XmToggleButtonSetState(searchButton, 1, 0);
    XtAddCallback (searchButton, XmNvalueChangedCallback, 
		   SetsMenu_Search_Callback, pWindowData);
    speechButton = XtVaCreateManagedWidget ("Show speech details",
					    xmPushButtonWidgetClass, 
					    optionsMenu, NULL);
    XtAddCallback (speechButton, XmNactivateCallback, SetsMenu_Speech_Callback,
		   pWindowData);
    
    closeButton = XtVaCreateManagedWidget ("Quit", xmCascadeButtonWidgetClass, 
					   menuBar, NULL);
    XtAddCallback (closeButton, XmNactivateCallback, SetsMenu_Quit_Callback, 
		   pWindowData);
    
    /* Attach menus to buttons on menu-bar */
    XtVaSetValues (setsButton, XmNsubMenuId, setsMenu, NULL);
    XtVaSetValues (actionsButton, XmNsubMenuId, actionsMenu, NULL);
    XtVaSetValues (optionsButton, XmNsubMenuId, optionsMenu, NULL);

    XmMainWindowSetAreas (mainWindow, menuBar, NULL, NULL, NULL, frame);

  } else {
    XmMainWindowSetAreas (mainWindow, NULL, NULL, NULL, NULL, frame);
  } /* endif */

  /* Set members of WindowData structure */
  pWindowData->mainWidget = mainWindow;
  pWindowData->setsMenuBarWidget = menuBar;
  pWindowData->setsMenuWidget = setsMenu;
  pWindowData->frameWidget = frame;
  pWindowData->unionWidget = unionButton;
  pWindowData->intersectWidget = intersectButton;
  pWindowData->pSetsData = pSetsData;
  pWindowData->controlPress = 0;

  /* Add window to the list of sets windows */
  pWindowItem->pNext = pSetsData->pWindowList;
  pWindowItem->pData = pWindowData;
  pSetsData->pWindowList = pWindowItem;

  return (pWindowData);
}
