#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <sys/uio.h>
#include <sys/types.h>
#include "include/odyssey.h"
#include "include/odytypes.h"
#include <Xm/Xm.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/SeparatoG.h>
#include <Xm/MessageB.h>
#include "odyssey/wardens/speech/libodyspeech_tsops.h"
#include "SpeechDialog.h"
#ifdef BW_TOOL
#include "include/bw_tool.h"
#include <sys/socket.h>
#include <netinet/in.h>
#ifdef __FreeBSD__
#include <rpc/types.h>
#endif
#endif

typedef struct {
  Widget dialog;
  Widget strategyField;
  Widget utteranceField;
} Speech_ClientData;

static Speech_ClientData* pClientData = 0;
int mon_fd;
extern enum strategyType currentStrategy;

void Speech_Update_Strategy ()
{
  char* new_label = 0;
  Arg args[1];
#ifdef BW_TOOL
  struct sockaddr_in mon_addr;
  int rc;
#endif

  if (currentStrategy == SPEECHW_STRATEGY_LOCAL) {
    new_label = "Local";
  } else if (currentStrategy == SPEECHW_STRATEGY_HYBRID) {
    new_label = "Hybrid";
  } else if (currentStrategy == SPEECHW_STRATEGY_REMOTE) {
    new_label = "Remote";
  } /* endif */

#ifdef BW_TOOL
  /* Send the new string to the bwmon 
   * XXX no check if there IS a bwmon -- does it matter? XXX */
  mon_addr.sin_family = AF_INET;
  /* write way to do localhost? */
  mon_addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);	
  mon_addr.sin_port = htons(BW_SP_MON_PORT);
  rc = sendto(mon_fd, new_label, strlen(new_label), 0,
	      (struct sockaddr *)&mon_addr, sizeof(mon_addr));
  if (rc < strlen(new_label)) {
      perror("sendto");
  }
#endif

  /* Make sure the dialog exists
   * kwalker -- doing it down here so the bwmon window update
   *            can always happen */
  if (!pClientData) {
    return;
  } /* endif */

  /* Update the label field with the new label */
  XtSetArg(args[0], XmNvalue, new_label);
  XtSetValues (pClientData->strategyField, args, 1);
}

void Speech_Update_Utterance (char* utterance)
{
  Arg args[1];

  printf ("%s\n", utterance);

  /* Make sure the dialog exists */
  if (!pClientData) {
    return;
  } /* endif */

  /* Update the label field with the new label */
  XtSetArg(args[0], XmNvalue, utterance);
  XtSetValues (pClientData->utteranceField, args, 1);
}

/* This function is called when the user selects the Cancel push button on the
   dialog.  It deallocates the call-data structure and closes the dialog. */
void Speech_Cancel_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  XtDestroyWidget (pClientData->dialog);
  free (pClientData);
  pClientData = 0;
}

/* This function is called when the user selects the Speech details
   menu-item on the Opitins Menu.  It displays a dialog that displays
   information about the current status of the speech interface */
void Create_Speech_Dialog (Widget w)

{
  Widget dialog, dialog_frame, sets_form, dialog_sep;
  Widget utterance_label, utterance_text, strategy_label, strategy_text;
  Widget buttons_form, cancelButton;
  XmString xmstr;
  Arg args[10];

  /* Make sure the dialog doesn't exist */
  if (pClientData) {
    /* Map the specified window */
    XtUnmanageChild (pClientData->dialog);
    XtManageChild (pClientData->dialog);
    return;
  } /* endif */

  /* Create a new client data structure for this dialog */
  pClientData = (Speech_ClientData *) malloc (sizeof(Speech_ClientData));

  /* Create dialog box */
  xmstr = XmStringCreateLtoR ("Speech Interface Details", 
			      XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0], XmNautoUnmanage, (XtArgVal) False);
  XtSetArg(args[1], XmNdialogTitle, (XtArgVal) xmstr);
  dialog = XmCreateFormDialog (w, "dialog", args, 2);
  XmStringFree (xmstr);

  /* Create a frame for the dialog window */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  dialog_frame = XtCreateManagedWidget ("frame", xmFrameWidgetClass,
					dialog, args, 5);

  /* Create the main form for the dialog */
  sets_form = XtCreateManagedWidget ("form", xmFormWidgetClass, dialog_frame,
				     NULL, 0);
       
  /* Create a label for the strategy */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  strategy_label = XtCreateManagedWidget ("Current strategy:", 
					  xmLabelWidgetClass,
					  sets_form, args, 4);  

  /* Create the strategy entry field */
  XtSetArg(args[0], XmNwidth, (XtArgVal) 310);
  XtSetArg(args[1], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 5);
  XtSetArg(args[3], XmNrightOffset, (XtArgVal) 10);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNleftWidget, (XtArgVal) strategy_label);
  XtSetArg(args[8], XmNeditable, 0); /* Don't allow user to edit this field */
  strategy_text = XmCreateTextField (sets_form, "strategy", args, 9);
  XtManageChild (strategy_text);

  /* Create a label for the utterance */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNtopWidget, (XtArgVal) strategy_text);
  XtSetArg(args[4], XmNleftAttachment, XmATTACH_FORM);
  utterance_label = XtCreateManagedWidget ("Last utterance: ", 
					   xmLabelWidgetClass,
					   sets_form, args, 5);  
  /* Create the utterance entry field */
  XtSetArg(args[0], XmNwidth, (XtArgVal) 310);
  XtSetArg(args[1], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 5);
  XtSetArg(args[3], XmNrightOffset, (XtArgVal) 10);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[5], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNtopWidget, (XtArgVal) strategy_text);
  XtSetArg(args[8], XmNleftWidget, (XtArgVal) utterance_label);
  XtSetArg(args[9], XmNeditable, 0); /* Don't allow user to edit this field */
  utterance_text = XmCreateTextField (sets_form, "utterance", args, 10);
  XtManageChild (utterance_text);

  /* Create form for push buttons */
  XtSetArg(args[0], XmNverticalSpacing, (XtArgVal) 8);
  XtSetArg(args[1], XmNfractionBase, (XtArgVal) 4);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  buttons_form = XtCreateManagedWidget ("form", xmFormWidgetClass, sets_form,
					args, 5);

  /* Create a separator between entry field and push button */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNtopWidget, (XtArgVal) utterance_text);
  XtSetArg(args[6], XmNbottomWidget, (XtArgVal) buttons_form);
  dialog_sep = XmCreateSeparatorGadget (sets_form, "separator", args, 7);
  XtManageChild (dialog_sep);
      
  /* Create the Cancel push button */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNleftPosition, 0);
  XtSetArg(args[5], XmNrightPosition, 1);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 8);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 8);
  cancelButton = XtCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
					buttons_form, args, 8);
  XtAddCallback (cancelButton, XmNactivateCallback, Speech_Cancel_Callback, 
		 (XtPointer) pClientData);

  /* Populate Client Data structure */
  pClientData->dialog = dialog;
  pClientData->strategyField = strategy_text;
  pClientData->utteranceField = utterance_text;

  XtManageChild (dialog);

  Speech_Update_Strategy();
  Speech_Update_Utterance ("<NONE>");

}
