#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "UnionDialog.h"
#include "SetDialog.h"
#include "sets.h"
#include <Xm/Xm.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <stdio.h>
#include <stdlib.h>

typedef struct {
  Widget parent;
  Widget dialog; 
  Widget list;
  Widget menu;
  Widget unionWidget;
  Widget intersectWidget;
  int isUnion;
  Display* display;
  Window* pNetscapeWindow;
} Union_ClientData;

/* This function is called when the user selects the Union/Intersect 
   push button.  It checks to make sure whether two sets are selected,
   then creates the union/intersection of the selected sets. */
void Union_Select_Callback (Widget w, XtPointer client_data, 
			    XtPointer call_data)
{
  Union_ClientData* pClientData;
  Widget dialog;
  XmString title, message;
  Arg args[2];
  XmStringTable sel;
  int cnt;
  char* set1 = 0;
  char* set2 = 0;
  char* setName = 0;
  int set;

  /* Determine which sets were selected */
  pClientData = (Union_ClientData *) client_data;
  XtSetArg(args[0], XmNselectedItemCount, &cnt);
  XtSetArg(args[1], XmNselectedItems, &sel);
  XtGetValues(pClientData->list, args, 2);

  /* Should only be 2 selected items. */
  if (cnt == 2) {	

    /* Perform the specified action on the sets */
    XmStringGetLtoR (sel[0], XmSTRING_DEFAULT_CHARSET, &set1);
    XmStringGetLtoR (sel[1], XmSTRING_DEFAULT_CHARSET, &set2);
    if (pClientData->isUnion) {
      setName = malloc (strlen(set1) + strlen(set2) + 9);
      sprintf (setName, "union(%s,%s)", set1, set2);
      set = CreateSetUnion (set1, set2, setName);
    } else {
      setName = malloc (strlen(set1) + strlen(set2) + 16);
      sprintf (setName, "intersection(%s,%s)", set1, set2);
      set = CreateSetIntersection (set1, set2, setName);
    } /* endif */

    /* Create a dialog for the opened set */
    Create_Set_Dialog (pClientData->parent, pClientData->menu, 
		       pClientData->unionWidget, pClientData->intersectWidget, 
		       set, setName, pClientData->display, 
		       pClientData->pNetscapeWindow);    

    /* Close the New dialog */
    Union_Cancel_Callback (w, client_data, call_data);

  } else {

    /* Tell the user to select 2 sets */
    title = XmStringCreateLtoR ("Two Sets Must Be Selected", 
				XmSTRING_DEFAULT_CHARSET);
    message = XmStringCreateLtoR (
	       "You must select exactly two sets to perform this operation.", 
	       XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNdialogTitle, (XtArgVal) title);
    XtSetArg(args[1], XmNmessageString, (XtArgVal) message);
    dialog = XmCreateErrorDialog (w, "dialog", args, 2);
    XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));  
    XmStringFree (title);
    XmStringFree (message);
    XtManageChild (dialog);

  } /* endif */

  /* Deallocate temporary data */
  if (set1) {
    free (set1);
  } /* endif */
  if (set2) {
    free (set2);
  } /* endif */
  if (setName) {
    free (setName);
  } /* endif */
  /* Don't know how to deallocate sel */
}

/* This function is called when the user selects the Cancel push
   button on the dialog.  It deallocates the client data structure and
   destroys the dialog. */
void Union_Cancel_Callback (Widget w, XtPointer client_data, 
			    XtPointer call_data)
{
  Union_ClientData* pClientData;

  /* Close the dialog */
  pClientData = (Union_ClientData *) client_data;
  XtDestroyWidget (pClientData->dialog);
  free (pClientData);
}

void Populate_Union_List (Widget list)
{
  char** setNames;
  int numberOfSets;
  XmString xmstr;
  int i;

  GetOpenSetNames (&setNames, &numberOfSets);

  /* Iterate through all open sets, and add each one to the list */
  for (i = 0; i < numberOfSets; i++) {
    xmstr = XmStringCreateLtoR(setNames[i], XmSTRING_DEFAULT_CHARSET);
    XmListAddItem(list, xmstr, 0);
    XmStringFree(xmstr);
  } /* endfor */

  /* Deallocate the list of set names */
  if (setNames) {
    for (i = 0; i < numberOfSets; i++) {
      if (setNames[i]) {
	free (setNames[i]);
      } /* endif */
    } /* endfor */
  } /* endif */
}

/* This function creates a dialog allowing the user to create the union or
   intersection of two sets.  If the isUnion flag is set, then a Union
   dialog is created, otherwise an Intersection dialog is created. */
void Create_Union_Dialog (Widget parent, Widget menu, Widget unionWidget, 
			  Widget intersectWidget, Display* display, 
			  Window* pNetscapeWindow, int isUnion)
{
  Widget dialog, dialog_frame, list_form, buttons_form;
  Widget list, label, unionButton, cancelButton;
  Union_ClientData* pClientData;
  XmString xmstr;
  Arg args[12];
    
  /* Create a new client data structure for this dialog */
  pClientData = (Union_ClientData *) malloc (sizeof(Union_ClientData));

  /* Create dialog box */
  /* XmxSetArg (XmNwidth, 475); */
  if (isUnion) {
    xmstr = XmStringCreateLtoR ("Union Window", XmSTRING_DEFAULT_CHARSET);
  } else {
    xmstr = XmStringCreateLtoR ("Intersection Window", 
				XmSTRING_DEFAULT_CHARSET);
  } /* endif */
  XtSetArg(args[0], XmNautoUnmanage, (XtArgVal) False);
  XtSetArg(args[1], XmNdialogTitle, (XtArgVal) xmstr);
  dialog = XmCreateFormDialog (parent, "dialog", args, 2);
  XmStringFree (xmstr);  

  /* Create a frame for the dialog window */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  dialog_frame = XtCreateManagedWidget ("frame", xmFrameWidgetClass,
					dialog, args, 5);   
      
  /* Create the main form for the dialog */
  list_form = XtCreateManagedWidget ("form", xmFormWidgetClass, dialog_frame,
				     NULL, 0);

  /* Create label containing user instructions */
  if (isUnion) {
    xmstr = XmStringCreateLtoR (
	     "Select two open sets then click on the Union button:", 
	     XmSTRING_DEFAULT_CHARSET);
  } else {
    xmstr = XmStringCreateLtoR (
	     "Select two open sets then click on the Intersection button:", 
	     XmSTRING_DEFAULT_CHARSET);
  } /* endif */
  XtSetArg(args[0], XmNlabelString, xmstr);
  XtSetArg(args[1], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[3], XmNrightOffset, (XtArgVal) 4);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[6], XmNrightAttachment, XmATTACH_FORM);
  label = XtCreateManagedWidget ("label", xmLabelWidgetClass,
                                 list_form, args, 7); 
  XmStringFree (xmstr);  

  /* Create a list that will contain all open sets */
  XtSetArg(args[0], XmNresizable, False);
  XtSetArg(args[1], XmNscrollBarDisplayPolicy, XmAS_NEEDED);
  XtSetArg(args[2], XmNlistSizePolicy, XmRESIZE_IF_POSSIBLE);
  XtSetArg(args[3], XmNvisibleItemCount, 10);
  XtSetArg(args[4], XmNselectionPolicy, XmMULTIPLE_SELECT);
  XtSetArg(args[5], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNtopWidget, (XtArgVal) label);
  XtSetArg(args[7], XmNtopOffset, 8);
  XtSetArg(args[8], XmNleftAttachment, XmATTACH_FORM);	
  XtSetArg(args[9], XmNleftOffset, 8);
  XtSetArg(args[10], XmNrightAttachment, XmATTACH_FORM); 
  XtSetArg(args[11], XmNrightOffset, 8);
  list = XmCreateScrolledList(list_form, "scrolled_list", args, 12);
  XtManageChild(list);
  XtAddCallback(list, XmNdefaultActionCallback, Union_Select_Callback,
		pClientData);

  /* Create form for push buttons */
  XtSetArg(args[0], XmNverticalSpacing, (XtArgVal) 8);
  XtSetArg(args[1], XmNfractionBase, (XtArgVal) 4);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNtopWidget, (XtArgVal) list);
  buttons_form = XtCreateManagedWidget ("form", xmFormWidgetClass, list_form,
					args, 7);

  /* Create the Union/Intersection push button */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNleftPosition, 0);
  XtSetArg(args[5], XmNrightPosition, 1);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 8);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 4);
  if (isUnion) {
    xmstr = XmStringCreateLtoR ("Union", XmSTRING_DEFAULT_CHARSET);
  } else {
    xmstr = XmStringCreateLtoR ("Intersection", XmSTRING_DEFAULT_CHARSET);
  } /* endif */
  XtSetArg(args[8], XmNlabelString, xmstr);
  unionButton = XtCreateManagedWidget ("button", xmPushButtonWidgetClass,
				      buttons_form, args, 9);
  XmStringFree (xmstr);  
  XtAddCallback (unionButton, XmNactivateCallback, Union_Select_Callback, 
		 (XtPointer) pClientData);

  /* Create the Cancel push button */
  XtSetArg(args[4], XmNleftPosition, 1);
  XtSetArg(args[5], XmNrightPosition, 2);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 8);
  cancelButton = XtCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
				        buttons_form, args, 8);
  XtAddCallback (cancelButton, XmNactivateCallback, Union_Cancel_Callback, 
		 (XtPointer) pClientData);

  /* Populate Client Data structure */
  pClientData->parent = parent;
  pClientData->dialog = dialog;
  pClientData->list = list;
  pClientData->menu = menu;
  pClientData->unionWidget = unionWidget;
  pClientData->intersectWidget = intersectWidget;
  pClientData->isUnion = isUnion;
  pClientData->display = display;
  pClientData->pNetscapeWindow = pNetscapeWindow;

  /* Populate the list with data */
  Populate_Union_List (list);

  XtManageChild (dialog);
}
