#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <sys/uio.h>
#include <unistd.h>
#include <assert.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/signal.h>
#include "include/odyssey.h"
#include "include/odytypes.h"
#include "odyssey/wardens/speech/libodyspeech_tsops.h"
#include "WWWSets.h"
#include "SetsMenu.h"
#include "SetDialog.h"
#include "DigestDialog.h"
#include "SpeechDialog.h"
#include "sets.h"
#include "xutils.h"
#include "speech_awedio.h"
#include "speech_interface.h"
#include "nettext.h"
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/MessageB.h>
#include <X11/Xmu/Editres.h>
#include <X11/Xmu/WinUtil.h>
#include <X11/keysym.h>

/* Function Prototypes */
int findWindow (Display* display, Window root, int res_id, Window* ret,
		Window* ret_parent);

/* The following functions are part of the Netscape remote-control interface */
extern void mozilla_remote_init_atoms (Display *dpy);
extern Window mozilla_remote_find_window (Display *dpy);
extern int mozilla_remote_command (Display *dpy, Window window, 
				   const char *command, Bool raise_p,
				   Bool waitForCompletion);
extern void mozilla_remote_obtain_lock (Display *dpy, Window window);
extern void mozilla_remote_free_lock (Display *dpy, Window window);

extern Atom XA_MOZILLA_URL; /* Intern'd when netscape is initialized */
extern Atom XA_MOZILLA_VERSION;

extern int switchToLocal; /* Used to trigger "friendly voice" */
extern int switchToRemote;

extern int mon_fd;

/* Global variables which indicate whether the proxy and/or speech interface
   is enabled. */
int proxy_interface = 1;
int speech_interface = 1;
SetsData setsData;

/* Signal handler called when control-C is pressed */
void cntl_c ()
{
  SetsMenu_Quit_Callback (0, setsData.pWindowList->pData, 0);
}

/* findNetscapeWindows searches the X window hierarchy for Netscape Navigator
   main windows.  It returns a linked list of all windows found. */
WindowLL* findNetscapeWindows (Display* display)
{
  int i;
  Window w, root, root2, parent, *kids;
  unsigned int nkids;
  Atom type;
  int format;
  unsigned long nitems, bytesafter;
  unsigned char *version = 0;
  int status;
  WindowLL* pItem;
  WindowLL* pList = 0;
  
  root = XDefaultRootWindow(display);

  if (!XQueryTree (display, root, &root2, &parent, &kids, &nkids)) {
    printf ("ERROR: Unable to query tree\n");
    return (0);
  } /* endif */

  for (i = 0; i < nkids; i++) {
      w = XmuClientWindow (display, kids[i]);
      status = XGetWindowProperty (display, w, XA_MOZILLA_VERSION,
				   0, (65536 / sizeof (long)),
				   False, XA_STRING,
				   &type, &format, &nitems, &bytesafter,
				   &version);
      if (status == 0 && version) {

	/* Prepend to linked list of Netscape windows */
	pItem = (WindowLL *) malloc (sizeof (WindowLL));
	if (!pItem) {
	  printf ("Memory allocation error\n");
	  exit (1);
	} /* endif */
	pItem->window = w;
	pItem->pNext = pList;
	pList = pItem;
	
	free (version);
	version = 0;

      } /* endif */

  } /* endfor */

  /* Deallocate list of child windows */
  XFree (kids);

  return (pList);
}

/* initNetscapeRemote() does the X-stuff and Netscape-stuff necessary
   to start sending Netscape commands - excluding only Netscape
   locking.  Returns the first netscape window found. */
WindowLL* initNetscapeRemote(Display* display) 
{
  WindowLL* pList = 0;
  WindowLL* pItem;

  /* Initialize Netscape-specific atoms */
  mozilla_remote_init_atoms (display);

  /* Locate the main Netscape windows */
  pList = findNetscapeWindows (display);

  /* Say that we are interested in the following X events:
     Property change - for interclient communication
     Structure notify - to detect when Netscape is closed */
  for (pItem = pList; pItem; pItem = pItem->pNext) {
    XMapWindow (display, pItem->window);
    XSelectInput (display, pItem->window, 
		  (PropertyChangeMask | StructureNotifyMask));
  } /* endfor */
    
  return (pList);
}

/* reparentNetscapeWindow() locates the Netscape window, queries its
   geometry, moves and resizes the dynamic sets window to match this
   geometry, then shrinks the Netscape window, and reparents it onto
   the dynamic sets window. */
void reparentNetscapeWindow(Display* display, Window window, Window dsetWindow,
			    Window frameWindow) 
{
  Window parent;
  Window gRoot; /* Following info returned by XGetGeometry */
  int gX, gY;
  unsigned int gH, gW, gBW, gD;

  /* Query the geometry of the netscape window */
  if (XGetGeometry (display, window, &gRoot, &gX, &gY, &gW, &gH, &gBW, &gD)) {
    
    /* Resize dynamic-sets window to be the same size as the Netscape
       window */
    XResizeWindow (display, dsetWindow, gW, gH);
    
    /* Query the geometry of Netscape's parent window */
    parent = getParent (display, window);
    if (XGetGeometry (display, parent, &gRoot, &gX, &gY, &gW, &gH, &gBW, 
		      &gD)) {
      
      /* Move dynamic-sets window to same location as the Netscape 
	 window */
      XMoveWindow (display, dsetWindow, gX, gY);
      
      /* Reparent Netscape window onto dynamic-sets window */
      XReparentWindow (display, window, frameWindow, 0, 0); 
      
    } else {
      printf ("Unable to querry Netscape parent window geometry\n");
      exit (1);
    } /* endif */

  } else {
    printf ("Unable to query Netscape window geometry\n");
    exit (1);
  } /* endif */
}

/* findWindow() recursively scans the resource tree to locate a window
   with the specified id.  If the window is found, findWindow()
   returns 1 and copies the window to the location specified by ret.
   If the window is not found, findWIndow() returns 0. */
int findWindow (Display* display, Window root, int res_id, Window* ret,
		Window* ret_parent)
{
  int i;
  Window root2, parent, *kids;
  unsigned int nkids;
  int found = 0; /* Set if window is found */
  
  if (!XQueryTree (display, root, &root2, &parent, &kids, &nkids)) {
    printf ("ERROR: Unable to query tree\n");
    return (0);
  } /* endif */

  for (i = nkids -1; i >= 0 && !found; i--) {
    if (kids[i] == res_id) {
      /* Return the window and its parent */
      *ret = kids[i];
      *ret_parent = root;
      found = 1;
    } else {
      /* Look at the children of the window */
      found = findWindow (display, kids[i], res_id, ret, ret_parent);
    } /* endif */
  } /* endfor */

  /* Deallocate list of child windows */
  XFree (kids);

  return (found);
}

/* unparentNetscapeWindow() undoes the effect of the reparenting when
   the dynamic sets window is closed. */
void unparentNetscapeWindow(Display* display, Window window, 
			    Window dsetWindow) 
{
  Window parent;
  Window gRoot; /* The following info is returned by XGetGeometry */
  int gX, gY;
  unsigned int gH, gW, gBW, gD;
  XEvent event; /* Used to loop through termination events */
  int done = 0;

  /* Query the geometry of the dynamic sets window */
  if (XGetGeometry (display, dsetWindow, &gRoot, &gX, &gY, &gW, &gH, &gBW, 
		    &gD)) {

    /* Resize Netscape window to be the same size as the dynamic-sets */
    /* window */
    XResizeWindow (display, window, gW, gH); 

    /* Query the geometry of dynamic sets parent window */
    parent = getParent (display, dsetWindow);
    if (XGetGeometry (display, parent, &gRoot, &gX, &gY, &gW, &gH, &gBW, 
    		      &gD)) {

      /* Reparent Netscape window onto desktop */
      XReparentWindow (display, window, gRoot, gX, gY);

      /* Wait for Focus-Out event - This is needed for reparenting to complete
	 successfully. */
      do {
	XNextEvent (display, &event);
	if (event.xany.type == ReparentNotify && 
	    event.xany.window == window) {
	  done = 1;
	} /* endif */
      } while (!done);

    } else {
      printf ("Unable to querry Netscape parent window geometry\n");
      exit (1);
    } /* endif */

  } else {
    printf ("Unable to query Netscape window geometry\n");
    exit (1);
  } /* endif */

  /* No longer interested in Netscape events */
  XSelectInput (display, window, 0);
}

/* handleResize() is called whenever the dynamic-sets window is 
   reconfigured.  It adjusts the size of the Netscape window to match
   the size of the reserved frame in the dynamic-sets application */
void handleResize (Widget w, XtPointer client_data, XEvent* pEvent, 
		   Boolean* continueToDispatch)
{
  WindowData* pWindowData;

  if (pEvent->xany.type == ConfigureNotify) {
    
    /* Resize the Netscape window */
    pWindowData = (WindowData *) client_data;
    XResizeWindow (pWindowData->pSetsData->display, 
		   pWindowData->netscapeWindow, 
		   pEvent->xconfigure.width,
		   pEvent->xconfigure.height);

  } /* endif */
}

/* netscapeRemoteOpenURL() issues a remote-control netscape command to
   open the specified URL.  It returns the status returned by
   Netscape. */
int netscapeRemoteOpenURL (char* url, Display* display, 
			   Window* pNetscapeWindow, int waitForCompletion)
{
  char* command;
  int status;

  command = (char *) malloc (strlen(url) + 10);
  sprintf (command, "openURL(%s)", url);
  status = netscapeRemote (command, display, pNetscapeWindow, 
			   waitForCompletion);
  free (command);

  return (status);
}

/* netscapeRemote() issues a remote-control netscape command.  It
   returns the status returned by Netscape. */
int netscapeRemote (char* command, Display* display, Window* pWindow, 
		    int waitForCompletion)
{
  int status;  /* Status returned by Netscape */
  Window bigNWindow = 0;

  /* If we are waiting for completion of the URL, register interest in
     "big N" expose events */
  if (waitForCompletion) {
    bigNWindow = getChildByGeometry (display, *pWindow, DIR_NORTH, 1);
    bigNWindow = getChildByGeometry (display, bigNWindow, DIR_NORTH, 2);
    bigNWindow = getChildByGeometry (display, bigNWindow, DIR_WEST, 1);
    
    XSelectInput (display, bigNWindow, ExposureMask);
  } /* endif */

  /* Obtain a Netscape lock (so that we're the only one sending commands) */
  mozilla_remote_obtain_lock (display, *pWindow);

  /* Send the command to Netscape, and get the response status */
  status = mozilla_remote_command (display, *pWindow, command, False, 
				   waitForCompletion);

  /* Release Netscape lock (so that others can send commands) */
  mozilla_remote_free_lock (display, *pWindow);

  /* If we were waiting for completion of the URL, deregister interest
     in "big N" expose events */
  if (waitForCompletion) {
    XSelectInput (display, bigNWindow, 0);
  } /* endif */

  return (status);
}

/* getNetscapeURL () returns the URL that Netscape is currenly
   displaying.  This information is obtained by querying the
   _MOZILLA_URL property on the main Netscape window */
char* getNetscapeURL (Display* display, Window netscapeWindow)
{
  int status;
  Atom type;
  int format;
  unsigned long nitems;
  unsigned long bytesafter;
  unsigned char* url = 0;

  status = XGetWindowProperty (display, netscapeWindow, XA_MOZILLA_URL, 0, 
			       (65536 / sizeof (long)), False, XA_STRING, &type,
			       &format, &nitems, &bytesafter, &url); 
  if (status != Success) {
    printf ("Error getting URL name, rc = %d\n", status);
  } /* endif */

  return ((char *) url);
}

/* netscapeString() displays a string in the Netscape text output window */
void netscapeString (char* str, Display* display, Window* pNetscapeWindow)
{
  Window strWindow, strParent;
  XRectangle rect;
  GC gc;
  Window gRoot; /* Following info returned by XGetGeometry */
  int gX, gY;
  unsigned int gH, gW, gBW, gD;
  Window root, parent, *kids; /* Following info returned by XQueryTree */
  unsigned int nkids;

  /* Locate Netscapes text output window */
  strParent = getChildByGeometry (display, *pNetscapeWindow, DIR_NORTH, 1);
  strWindow = getChildByGeometry (display, strParent, DIR_SOUTH, 1);

  /* Figure out how big the window is */
  if (XGetGeometry (display, strWindow, &gRoot, &gX, &gY, &gW, 
		    &gH, &gBW, &gD)) {
    
    rect.width = gW;
    rect.height = gH;
    
    /* Should be one child - the Netscape status bar */
    if (XQueryTree (display, strWindow, &root, &parent, &kids, &nkids)) {
      
      if (nkids == 1) {
	
	/* Query the geometry of the child window */
	if (XGetGeometry (display, kids[0], &gRoot, &gX, &gY, &gW, 
			  &gH, &gBW, &gD)) {
	  
	  /* Clear the text area */
	  rect.width = gX - 48 - 8;
	  rect.x = 48;
	  rect.y = 0;
	  XClearArea (display, strWindow, rect.x, rect.y, rect.width, 
		      rect.height, False);
	  
	  /* Draw the text in the window */
	  gc = XCreateGC (display, strWindow, 0, NULL);
	  XSetForeground (display, gc, BlackPixel(display, 0));
	  XSetClipRectangles (display, gc, 0, 0, &rect, 1, Unsorted);
	  XDrawString (display, strWindow, gc, rect.x, 12, str, strlen(str));
	  XFreeGC (display, gc);
	  
	} else {
	  printf ("ERROR: Unable to query geometry of Netscape status bar\n");
	} /* endif */
	
      } else {
	printf ("ERROR: Unexpected number of child windows\n");
      } /* endif */
      
      /* Deallocate list of child windows */
      XFree (kids);
      
    } else {
      printf ("ERROR: Unable to query tree\n");
    } /* endif */
    
  } else {
    printf ("ERROR: Unable to query geometry of Netscape text output window\n");
  } /* endif */
  

}

/* extendNetscapeWindow () extends a Netscape main window with a Sets 
   extension.  The Netscape and Sets window must alread exst and be
   specified in the window data structure. */
void extendNetscapeWindow (SetsData* pSetsData, WindowData* pWindowData)
{
  XEvent event;
  Window urlParent, buttonParent;

  /* Map the top-level widget and wait for notification */
  XtMapWidget (pWindowData->topLevel);
  do {
    XNextEvent (pSetsData->display, &event);
    XtDispatchEvent (&event);
  } while (event.xany.type != MapNotify ||
	   event.xany.window != pWindowData->setWindow);

  /* Set a handler to notify application when window is reconfigured */
  XtAddEventHandler (pWindowData->frameWidget, StructureNotifyMask, False, 
		     handleResize, pWindowData);

  /* Reparent Netscape window inside this application's window */
  reparentNetscapeWindow (pSetsData->display, pWindowData->netscapeWindow, 
			  pWindowData->setWindow, 
			  XtWindow(pWindowData->frameWidget));
  
  /* Grab the left button on the Netscape URL window */
  urlParent = getChildByGeometry (pSetsData->display, 
				  pWindowData->netscapeWindow, DIR_NORTH, 1);
  buttonParent = urlParent;
  urlParent = getChildByGeometry (pSetsData->display, urlParent, 
				 DIR_SOUTH, 2);
  urlParent = getChildByGeometry (pSetsData->display, urlParent, 
				 DIR_NORTH, 1);
  pWindowData->urlWindow = getChildByGeometry (pSetsData->display, 
					       urlParent, DIR_NORTHEAST, 1);
  if (pWindowData->urlWindow > 0) {
    XGrabButton (pSetsData->display, Button1, AnyModifier, 
		 pWindowData->urlWindow, True, 
		 ButtonPressMask | ButtonReleaseMask | Button1MotionMask, 
		 GrabModeAsync, GrabModeAsync, None, None);
    XGrabKey (pSetsData->display, XKeysymToKeycode(pSetsData->display, XK_F11),
	      AnyModifier, pWindowData->urlWindow, True, GrabModeAsync, 
	      GrabModeAsync);
  } else {
    printf ("Unable to locate URL window");
  } /* endif */
  
  /* Grab the left button on the Netscape button window */
  buttonParent = getChildByGeometry (pSetsData->display, buttonParent, 
				     DIR_NORTH, 2);
  buttonParent = getChildByGeometry (pSetsData->display, buttonParent, 
				     DIR_NORTH, 1);
  pWindowData->buttonWindow = getChildByGeometry (pSetsData->display, 
						  buttonParent, DIR_NORTH, 1);
  if (pWindowData->buttonWindow >= 0) {
    XGrabButton (pSetsData->display, Button1, AnyModifier, 
		 pWindowData->buttonWindow, True, 
		 ButtonPressMask | ButtonReleaseMask | Button1MotionMask, 
		 GrabModeAsync, GrabModeAsync, None, None);
  } else {
    printf ("Unable to locate button window\n");
  } /* endif */

  XtMapWidget (pWindowData->mainWidget);
}

/* handleNewNetscapeWindow () is called when a window creation event is 
   received.  If the window being created, is a new Netscape Navigator main
   window, it is extended with the Sets interface. */
void handleNewNetscapeWindow (SetsData* pSetsData, XEvent* pEvent)
{
  XTextProperty property;
  WindowData* pWindowData;
  Widget topLevel;
  XEvent event;

  XGetWMName (pSetsData->display, pEvent->xcreatewindow.window, &property);
  if (property.value && !strncmp (property.value, NAV_TITLE,  
				  strlen (NAV_TITLE))) {

    /* Create shell for new window */
    topLevel = XtVaAppCreateShell ("XDSet", "XDSet", 
				   applicationShellWidgetClass, 
				   pSetsData->display, XmNmappedWhenManaged, 
				   False, XtNtitle, "Setscape", NULL);

    /* Create main window */
    pWindowData = Create_Application_Window (topLevel, pSetsData);
    XtRealizeWidget (topLevel); 

    pWindowData->topLevel = topLevel;
    pWindowData->netscapeWindow = pEvent->xcreatewindow.window;
    pWindowData->setWindow = XtWindow (topLevel);

    /* Say that we are interested in the following X events:
       Property change - for interclient communication
       Structure notify - to detect when Netscape is mapped or closed */
    XSelectInput (pSetsData->display, pEvent->xcreatewindow.window, 
		  (PropertyChangeMask | StructureNotifyMask));

    if (!isWindowMapped (pSetsData->display, event.xany.window)) {
      do {
	XNextEvent (pSetsData->display, &event);
	XtDispatchEvent (&event);
      } while ((event.xany.type != MapNotify ||
		event.xany.window != pWindowData->netscapeWindow) && 
	       !isWindowMapped (pSetsData->display, event.xany.window));
    } /* emdif */

    extendNetscapeWindow (pSetsData, pWindowData);

  } /* endif */
}

/* resendButtonEvent () is called when the user presses or releases 
   the left mouse button on the netscape URL window.  It passes the 
   button event to Netscape but subtracts the Control Mask from the 
   modifier flags.  This makes a control-left click appear to Netscape 
   as a normal left-click.  It also sends through unmodified left-clicks. 
   For URLs that contain frames, it is necessary to determine which
   sub-window contains the button event and translate the event 
   coordinates to the frame of reference of that sub-window. */
void resendButtonEvent (WindowData* pWindowData, XEvent* pEvent)
{
  Window child;
  int x, y;

  /* Resend left-click events to netscape - turn off control bit */
  pEvent->xbutton.state &= ~ControlMask;
  if (pEvent->xbutton.subwindow) {
    
    if (getChildCoord (pWindowData->pSetsData->display,
		       pEvent->xbutton.window, pEvent->xbutton.x,
		       pEvent->xbutton.y, &child, &x, &y) == 0) {
      
      activeGrab.x_offset = x - pEvent->xbutton.x;
      activeGrab.y_offset = y - pEvent->xbutton.y;
      activeGrab.window = child;
      
      pEvent->xbutton.window = child;
      pEvent->xbutton.subwindow = 0;
      pEvent->xbutton.x = x;
      pEvent->xbutton.y = y;
    } /* endif */
  } /* endif */
  
  if (pEvent->xany.type == ButtonPress) {
    XSendEvent (pWindowData->pSetsData->display, pEvent->xbutton.window, 
		True, ButtonPressMask, pEvent);
  } else {
    XSendEvent (pWindowData->pSetsData->display, pEvent->xbutton.window, 
		True, ButtonReleaseMask, pEvent);
  } /* endif */
}

/* resendMotionEvent () is called when the user moves the mouse with
   the left-mouse button held down on the netscape URL window.  For
   URLs that contain frames, it is necessary to determine which
   sub-window contains the button event and translate the event
   coordinates to the frame of reference of that sub-window. For URLs
   that do not contain frames, the event is sent through
   unmodified. */
void resendMotionEvent (WindowData* pWindowData, XEvent* pEvent)
{
  if (pEvent->xmotion.subwindow) {

    pEvent->xmotion.window = activeGrab.window;
    pEvent->xmotion.subwindow = 0;
    pEvent->xmotion.x += activeGrab.x_offset;
    pEvent->xmotion.y += activeGrab.y_offset;
    
  } /* endif */
  
  XSendEvent (pWindowData->pSetsData->display, pEvent->xmotion.window, 
	      True, Button1MotionMask, pEvent);
}

/* handleDestroyedWindow() is called whenever a DestroyNotify event is 
   received.  It checks to see if the destroyed window is a Netscape window.  
   If this is the case, it destroys the associated sets window.  If there are
   no more Netscape windows open, the application exits. */
void handleDestroyedWindow (SetsData* pSetsData, XEvent* pEvent)
{
  WindowItem* pItem;
  WindowItem* pLast;

  pLast = 0;
  for (pItem = pSetsData->pWindowList; pItem; pItem = pItem->pNext) {

    if (pItem->pData->netscapeWindow == pEvent->xany.window) {
      if (pLast) {
	pLast->pNext = pItem->pNext;
      } else {
	pSetsData->pWindowList = pItem->pNext;
      } /* endif */
      
      /* Destroy the sets window */
      XtDestroyWidget (pItem->pData->topLevel);

      free (pItem->pData);
      free (pItem);
      
      if (!pSetsData->pWindowList) {
	/* No more Netscape windows */
	exit (0);
      } else {
	return;
      } /* endif */

    } /* endif */

    pLast = pItem;

  } /* endfor */
}

/* findURLWindow finds a Netscape URL window with the specified id.  If 
   one is found, it returns the window data structure associated with
   that Netscape instance.  If one is not found, it returns zero. */
WindowData* findURLWindow (SetsData* pSetsData, Window window) 
{
  WindowItem* pItem;

  for (pItem = pSetsData->pWindowList; pItem; pItem = pItem->pNext) {
    if (pItem->pData->urlWindow == window) {
      return (pItem->pData);
    } /* endif */
  } /* endfor */

  return (0);
}

/* findButtonWindow finds a Netscape button window with the specified id.  If 
   one is found, it returns the window data structure associated with
   that Netscape instance.  If one is not found, it returns zero. */
WindowData* findButtonWindow (SetsData* pSetsData, Window window) 
{
  WindowItem* pItem;

  for (pItem = pSetsData->pWindowList; pItem; pItem = pItem->pNext) {
    if (pItem->pData->buttonWindow == window) {
      return (pItem->pData);
    } /* endif */
  } /* endfor */

  return (0);
}

/* findNetscapeWindow finds a Netscape URL window with the specified id.  If 
   one is found, it returns the window data structure associated with
   that Netscape instance.  If one is not found, it returns zero. */
WindowData* findNetscapeWindow (SetsData* pSetsData, Window window) 
{
  WindowItem* pItem;

  for (pItem = pSetsData->pWindowList; pItem; pItem = pItem->pNext) {
    if (pItem->pData->netscapeWindow == window) {
      return (pItem->pData);
    } /* endif */
  } /* endfor */

  return (0);
}

void XtMyErrorHandler (String msg)
{
  fprintf (stderr, "Xt Error Message:\n%s\n", msg);
  assert (0);
}

int XMyErrorHandler (Display* display, XErrorEvent* err)
{
  char msg[80];

  if (err->error_code != BadWindow) {
    XGetErrorText(display, err->error_code, msg, 80);
    fprintf (stderr, "X Error Message:\n%s\n", msg); 
    assert (0); 
  } else {
#ifdef SUPERDEBUG
    fprintf (stderr, "Bad window\n");
#endif
  } /* endif */
  return (-1);
}

/* findFrameWindow finds a Sets top-level window with the specified id.  If 
   one is found, it returns the window data structure associated with
   that instance.  If one is not found, it returns zero. */
WindowData* findTopLevel (SetsData* pSetsData, Window window) 
{
  WindowItem* pItem;

  for (pItem = pSetsData->pWindowList; pItem; pItem = pItem->pNext) {
    if (XtWindow(pItem->pData->topLevel) == window) {
      return (pItem->pData);
    } /* endif */
  } /* endfor */

  return (0);
}

void cmdline ()
{
  printf ("WWWSets [-wait_for_netscape] [-nospeech] [-noproxy]\n");
  exit (0);
}

int main (int argc, char* argv[])
{
  XtAppContext app_context;
  Widget topLevel;
  WindowData* pWindowData;
  Set_ClientData* pSetDialogData = 0;
  Digest_ClientData* pDigestDialogData = 0;
  WindowLL* pList;
  WindowLL* pItem;
  char* url = 0;
  int set = 0;
  int wait_for_netscape = 0;
  int ody_fd = -1;
  char utterance[1024*1024];
  char translation[1024];
  int len;
  int rc;
  int i;

  setsData.pWindowList = 0;

  /* Quick check for "-wait_for_netscape" option */
  for (i = 0; i < argc; i++) {
    if (!strcmp (argv[i], "-wait_for_netscape")) {
      wait_for_netscape = 1;
    } else if (!strcmp (argv[i], "-nospeech")) {
      speech_interface = 0;
    } else if (!strcmp (argv[i], "-noproxy")) {
      proxy_interface = 0;
    } else if (!strcmp (argv[i], "-h")) {
      cmdline();
    } /* endif */
  } /* endfor */
      
#ifdef BW_TOOL
  mon_fd = socket(AF_INET, SOCK_DGRAM, 0);
  printf("made socket %d\n", mon_fd);
  if (mon_fd < 0) {
    perror("socket");
  }
#endif

  if (speech_interface) {
    ody_fd = speech_init();
    if (ody_fd < 0) {
      printf ("Unable to initialize speech interface.  Perhaps the viceroy\n");
      printf ("is not started.\n");
      exit (1);
    } /* endif */
  } /* endif */

  /* Initialize Dynamic Sets application */ 
  XtSetLanguageProc (NULL, (XtLanguageProc)NULL, NULL);
  topLevel = XtVaAppInitialize (&app_context, "XDSet", NULL, 0, &argc,
				argv, NULL, XtNtitle, "Setscape", 
				XmNmappedWhenManaged, False, NULL);

  /* For debugging purposes, specify an error handler */
  XtAppSetErrorHandler (app_context, XtMyErrorHandler);
  XSetErrorHandler (XMyErrorHandler);

  XtRealizeWidget (topLevel);
  setsData.display = XtDisplay (topLevel);
  
  /* Create main window */
  pWindowData = Create_Application_Window (topLevel, &setsData);

  /* Get XLib display and window values for the main application window */
  pWindowData->topLevel = topLevel;
  pWindowData->setWindow = XtWindow (topLevel); 

  /* Initialize netscape remote control */
  do {
      pList = initNetscapeRemote (setsData.display);
      if (!pList) {
	sleep(1);
      } /* endif */
  }
  while (wait_for_netscape && !pList);

  if (!pList) {
    printf ("Netscape does not appear to be running\n");
    exit (0);
  } /* endif */

  /* Extend the first Netscape window found */
  pWindowData->netscapeWindow = pList->window;
  extendNetscapeWindow (&setsData, pWindowData);

  /* Also create extensions for any additional windows found */
  for (pItem = pList->pNext; pItem; pItem = pItem->pNext) {

    /* Create shell for new window */
    topLevel = XtVaAppCreateShell ("XDSet", "XDSet", 
				   applicationShellWidgetClass, 
				   setsData.display, XmNmappedWhenManaged, 
				   False, XtNtitle, "Setscape", NULL);

    /* Create main window */
    pWindowData = Create_Application_Window (topLevel, &setsData);
    XtRealizeWidget (topLevel); 

    pWindowData->topLevel = topLevel;
    pWindowData->netscapeWindow = pItem->window;
    pWindowData->setWindow = XtWindow (topLevel);

    extendNetscapeWindow (&setsData, pWindowData);

  } /* endfor */

  /* Deallocate window list */
  while (pList) {
    pItem = pList;
    pList = pList->pNext;
    free (pItem);
  } /* endif */

  /* Ask for notification whenever a new root window is created */
  XSelectInput (setsData.display, XDefaultRootWindow(setsData.display), 
		SubstructureNotifyMask);

  /* Catch cntl-c */
  signal (SIGINT, cntl_c);

  /* Handle X Events */
  while (1) {
    XEvent event;
    XtAppNextEvent (app_context, &event);
    XtDispatchEvent (&event);

    if (switchToLocal) {
      play_audio ("/usr/odyssey/speech/disconn.au");
      switchToLocal = 0;
    } /* endif */
    if (switchToRemote) {
      play_audio ("/usr/odyssey/speech/reconn.au");
      switchToRemote = 0;
    } /* endif */

    if (event.xany.type == ConfigureNotify) {

      /* If the top-level sets window is reconfigured, it is necessary
	 to "kick" the Netscape window so that it will adjust the 
	 positions of its menus. */
      pWindowData = findTopLevel (&setsData, event.xany.window);
      if (pWindowData) {
	XMoveWindow (setsData.display, pWindowData->netscapeWindow, 1, 1); 
	XMoveWindow (setsData.display, pWindowData->netscapeWindow, 0, 0); 
      } /* endif */

    } /* endif */

    if (event.xany.type == CreateNotify && 
	((event.xcreatewindow.window&0xff000000) == 
	 (setsData.pWindowList->pData->netscapeWindow&0xff000000))) {
      handleNewNetscapeWindow (&setsData, &event);
    } /* endif */

    if (event.xany.type == KeyRelease && 
	event.xkey.keycode == XKeysymToKeycode (setsData.display, XK_F11) &&
	speech_interface) {

      /* Check if speech hot key was pressed in a Netscape window */
      pWindowData = findURLWindow (&setsData, event.xany.window);
      if (pWindowData) {
	pSetDialogData = firstSetDialog();
	pDigestDialogData = 0;
      } else {
	/* Otherwise check if speech hot key was pressed in a set window */
	pSetDialogData = findSetDialog (setsData.display, event.xany.window);
	if (pSetDialogData) {
	  /* Find the Netscape window with which the set window is assoc. */
	  pWindowData = findNetscapeWindow (&setsData, 
					    *pSetDialogData->pNetscapeWindow);
	  if (pWindowData) {
	    pDigestDialogData = 0;
	  } else {
	    printf ("No Netscape window associated with a URL window.");
	    ASSERT (0);
	  } /* endif */
	} else {
	  /* Check if speech hot key was pressed in a digest window */
	  pDigestDialogData = findDigestDialog (setsData.display, 
						event.xany.window);
	  if (pDigestDialogData) {
	    pWindowData = findNetscapeWindow (&setsData, 
					 *pDigestDialogData->pNetscapeWindow);
	    if (!pWindowData) {
	      printf ("No Netscape window associated with a URL window.");
	      ASSERT (0);
	    } /* endif */
	    pSetDialogData = 0;
	  } /* endif */
	} /* endif */
      } /* endif */

      if (pWindowData) {

	Speech_Update_Utterance ("<RECORDING>");

	/* Recording audio input */
	len = record_audio (utterance, sizeof(utterance), &app_context, 
			    &setsData);
	if (len) {

	  /* Recognize the input */
	  rc = speechRecognize (ody_fd, SPEECH_MODEL, utterance, len,
				translation, sizeof (translation));
	  if (rc == 0) {
	    Speech_Update_Utterance(translation);
	    
	    /* Perform the requested action */
	    translateUtterance (translation, pWindowData, pSetDialogData, 
				pDigestDialogData);
	  } else {
	    printf ("Translation failed, rc = %d\n", rc);
	  } /* endif */
	} else {
	  printf ("Error recording audio input\n");
	} /* endif */
	  
      } /* endif */

    } /* endif */
      
    if (event.xany.type == ButtonPress) {

      pWindowData = findURLWindow (&setsData, event.xany.window);
      if (pWindowData && event.xbutton.state != ControlMask) {
	/* If this is not a special "control" click, then just pass the
	   event to Netscape.  Otherwise, the proxy must monitor continue
	   to convert the events to "regular" netscape clicks. */
	XUngrabPointer (setsData.display, CurrentTime);
	pWindowData->controlPress = 0;
#ifdef DEBUG_PRESS
	printf ("Ungrabbed pointer - press is off\n");
#endif
      } /* endif */
      if (!pWindowData) {
	pWindowData = findButtonWindow (&setsData, event.xany.window);
      } /* endif */

      if (pWindowData) {
	/* Replay the button press to Netscape as a normal left-click */
	resendButtonEvent (pWindowData, &event);
      } /* endif */

    } /* endif */

    if (event.xany.type == ButtonRelease) {

      pWindowData = findURLWindow (&setsData, event.xany.window);
      if (pWindowData) {

	/* If this is a control-left click, the next URL should be loaded
	   as a set.  If it is not, the next URL should not be loaded as
	   a set. */
	if (event.xbutton.state == (Button1Mask | ControlMask)) {
	  pWindowData->controlPress = 1;
#ifdef DEBUG_PRESS
	  printf ("Press is on\n");
#endif
	} else {
	  pWindowData->controlPress = 0;
#ifdef DEBUG_PRESS
	  printf ("Press is off\n");
#endif
	} /* endif */

	/* Replay the button press to Netscape as a normal left-click */
	resendButtonEvent (pWindowData, &event);

      } else {
	pWindowData = findButtonWindow (&setsData, event.xany.window);
	if (pWindowData) {
	  resendButtonEvent (pWindowData, &event);
	  pWindowData->controlPress = 0;
#ifdef DEBUG_PRESS
	  printf ("Press is off\n");
#endif
	} /* endif */
      } /* endif */

    } /* endif */
    
    if (event.xany.type == MotionNotify) {

      pWindowData = findURLWindow (&setsData, event.xany.window);
      if (!pWindowData) {
	pWindowData = findButtonWindow (&setsData, event.xany.window);
      } /* endif */

      if (pWindowData) {
	/* Replay the motion event to Netscape */
	resendMotionEvent (pWindowData, &event);
      } /* endif */    

    } /* endif */

    /* Handle case where Netscape is closed before extension.  In this case,
       the extension should simply be destroyed. */
    if (event.xany.type == DestroyNotify) {
      handleDestroyedWindow (&setsData, &event);
    } /* endif */

    if (event.xany.type == PropertyNotify &&
	event.xproperty.state == PropertyNewValue && 
	event.xproperty.atom == XA_MOZILLA_URL) {

      url = getNetscapeURL (setsData.display, event.xany.window);
      if (url) {
	
	pWindowData = findNetscapeWindow (&setsData, event.xany.window);
	if (pWindowData) {

	  if (pWindowData->controlPress) {

	    /* Open a set */
	    set = OpenSetForURL (url); 

	    /* Create a dialog for the opened set */
	    Create_Set_Dialog (pWindowData->mainWidget, 
			       pWindowData->setsMenuWidget, 
			       pWindowData->unionWidget, 
			       pWindowData->intersectWidget, 
			       set, url, setsData.display, 
			       &pWindowData->netscapeWindow);

	    pWindowData->controlPress = 0;

	  } /* endif */

	} /* endif */

	free (url);
	url = 0;

      } else {
	printf ("Unable to get URL name\n");
      } /* endif */

    } /* endif */

  } /* endwhile */
}

