#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <sys/uio.h>
#include "include/odyssey.h"
#include "include/odytypes.h"
#include "WWWSets.h"
#include "SetsMenu.h"
#include "SetDialog.h"
#include "DigestDialog.h"
#include "xutils.h"
#include "nettext.h"

extern void parseUtt (char*);

/* Structure to save values */
static struct {
  WindowData* pWindowData;
  Set_ClientData* pSetDialogData;
  Digest_ClientData* pDigestDialogData;
} clientdata;

void callNetscapeRemote (char* cmd)
{
  netscapeRemote (cmd, clientdata.pWindowData->pSetsData->display, 
		  &clientdata.pWindowData->netscapeWindow, 0);
}

void sendEvent (char* keyEvent)
{
  unsigned int modifier;
  KeySym keysym;
  char* key;

  if (!strncmp (keyEvent, "ALT-", 4)) {
    modifier = Mod1Mask;
    key = keyEvent + 4;
  } else {
    modifier = 0;
    key = keyEvent;
  } /* endif */
  keysym = XStringToKeysym (key);
  if (keysym != NoSymbol) {
    pressKey (clientdata.pWindowData->pSetsData->display, 
	      clientdata.pWindowData->netscapeWindow, modifier,
	      XKeysymToKeycode (clientdata.pWindowData->pSetsData->display, 
				keysym));
  } else {
    printf ("Unable to lookup key: %s\n", key);
  } /* endif */
}

void openAsSet ()
{
  SetsMenu_This_Callback (clientdata.pWindowData->setsMenuBarWidget, 
			  clientdata.pWindowData, 0);
}

void openNewSet ()
{
  SetsMenu_New_Callback (clientdata.pWindowData->setsMenuBarWidget, 
			 clientdata.pWindowData, 0);
}  

void iterateSet ()
{
  if (clientdata.pSetDialogData) {
    Set_Iterate_Callback (clientdata.pSetDialogData->dialog, 
			  clientdata.pSetDialogData, 0);
  } else {
    printf ("The action requires an open set\n");
  } /* endif */
}

void digestSet ()
{
  if (clientdata.pSetDialogData) {
    Set_Digest_Callback (clientdata.pSetDialogData->dialog, 
			 clientdata.pSetDialogData, 0);
  } else {
    printf ("The action requires an open set\n");
  } /* endif */
}

void rewindSetIterator ()
{
  if (clientdata.pSetDialogData) {
    Set_Rewind_Callback (clientdata.pSetDialogData->dialog, 
			 clientdata.pSetDialogData, 0);
  } else {
    printf ("The action requires an open set\n");
  } /* endif */
}

void setSize ()
{
  if (clientdata.pSetDialogData) {
    Set_Size_Callback (clientdata.pSetDialogData->dialog, 
			 clientdata.pSetDialogData, 0);
  } else {
    printf ("The action requires an open set\n");
  } /* endif */
}

void closeSet ()
{
  if (clientdata.pSetDialogData) {
    Set_Close_Callback (clientdata.pSetDialogData->dialog, 
			 clientdata.pSetDialogData, 0);
  } else {
    printf ("The action requires an open set\n");
  } /* endif */
}

void cancel ()
{
  if (clientdata.pSetDialogData) {
    Set_Cancel_Callback (clientdata.pSetDialogData->dialog, 
			 clientdata.pSetDialogData, 0);
  } else if (clientdata.pDigestDialogData) {
    Digest_Cancel_Callback (clientdata.pDigestDialogData->dialog, 
			    clientdata.pDigestDialogData, 0);
  } else {
    printf ("The action requires an open set\n");
  } /* endif */
}

void openDigestMember (char* member)
{
  if (clientdata.pDigestDialogData) {
    Digest_Open_Member (clientdata.pDigestDialogData, atoi (member));
  } else {
    printf ("The action requires an open digest\n");
  } /* endif */
}

void rewindDigest ()
{
  if (clientdata.pDigestDialogData) {
    Digest_Rewind_Callback (clientdata.pDigestDialogData->dialog,
			    clientdata.pDigestDialogData, 0);
  } else {
    printf ("The action requires an open digest\n");
  } /* endif */
}

void redigest ()
{
  if (clientdata.pDigestDialogData) {
    Digest_Redigest_Callback (clientdata.pDigestDialogData->dialog,
			    clientdata.pDigestDialogData, 0);
  } else {
    printf ("The action requires an open digest\n");
  } /* endif */
}

/* translateUtterance() translates and executes a recognized utterance,
   which is passed into the fucntion as an ASCII string.  */
void translateUtterance (char* utt, WindowData* pWindowData,
			 Set_ClientData* pSetDialogData, 
			 Digest_ClientData* pDigestDialogData) 
{
  clientdata.pWindowData = pWindowData;
  clientdata.pSetDialogData = pSetDialogData;
  clientdata.pDigestDialogData = pDigestDialogData;

  parseUtt (utt);
}
