#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <errno.h>

#include "sets.h"
#include "kernel/sets/common/sets.h"

typedef struct SetNode {
  int set;
  char* name;
  struct SetNode* next;
} SetNode;

/* Globally define a linked-list of open sets */
SetNode* pOpenSetList = 0;
int interpret_pages = 1;

#define ODY_ROOT "/ody/www/"

int OdyToWWW (char* ody_name, char* www_name, int len)
{
  int rc = 0; /* Assume success */

  if (!strncmp (ody_name, ODY_ROOT, strlen (ODY_ROOT))) {
    ody_name += strlen (ODY_ROOT);
    if (strlen (ody_name)+1 <= len) {
      while (*ody_name) {
	if (*ody_name == '\\') {
	  *www_name = '/';
	} else {
	  *www_name = *ody_name;
	} /* endif */
	www_name++;
	ody_name++;
      } /* end while */
      *www_name = '\0';
    } else {
      rc = 2;
    }  /* endif */
  } else {
    rc = 1;
  } /* endif */

  return (rc);
}
    
/* AddOpenSet() adds the information for a newly-opened set to the global
   list. */
void AddOpenSet (int set, char* name)
{
  SetNode* pNode;

  pNode = (SetNode *) malloc (sizeof (SetNode));
  pNode->set = set;
  pNode->name = strdup (name);
  pNode->next = pOpenSetList;
  pOpenSetList = pNode;
}

/* RemoveOpenSet() removes an open set from the list of newly-opened sets if
   the set exists in the list. */
void RemoveOpenSet (int set)
{
  SetNode* pNode = pOpenSetList;
  SetNode* pLast = 0;

  while (pNode) {

    if (pNode->set == set) {
      if (pLast) {
	pLast->next = pNode->next;
      } else {
	pOpenSetList = pNode->next;
      } /* endif */
      if (pNode->name) {
	free (pNode->name);
      } /* endif */
      free (pNode);
      pNode = NULL;
    } else {
      pLast = pNode;
      pNode = pNode->next;
    } /* endif */

  } /* endwhile */
}

/* FindOpenSetWithName() finds the open set with the specified name.  If
   such a set is not found, the function returns -1. */
int FindOpenSetWithName (char* name)
{
  SetNode* pNode = pOpenSetList;
  int set = -1;

  while (pNode && set == -1) {
    if (!strcmp (pNode->name, name)) {
      set = pNode->set;
    } /* endif */
    pNode = pNode->next;
  } /* endwhile */
  
  return (set);
}

/* GetNumberOfOpenSets() returns the number of sets that are currently open */
int GetNumberOfOpenSets ()
{
  int count = 0;
  SetNode* pNode = pOpenSetList;
  
  while (pNode) {
    count++;
    pNode = pNode->next;
  } /* endwhile */

  return (count);
}

/* GetOpenSetNames() returns a list containing the name of every open set */
void GetOpenSetNames (char*** names, int* number)
{
  SetNode* pNode = pOpenSetList;
  int count = 0;

  *number = GetNumberOfOpenSets();
  *names = (char **) malloc (*number * sizeof(char *));

  while (pNode) {
    (*names)[count++] = strdup (pNode->name);
    pNode = pNode->next;
  } /* endwhile */
}

/* OpenSetForURL opens a set specified by the given URL.  At
   present, it does not convert the URL name into the Odyssey
   name space, but this may need to be done in the future. */
int OpenSetForURL (char* url)
{
  int set;
  char* setName = 0;
  char* pch;

  /* Have to convert URL to set name here */
  setName = malloc (strlen(ODY_ROOT) + strlen(url) + 3);
  if (setName) {
    strcpy (setName, ODY_ROOT);
    strcat (setName, "!");
    strcat (setName, url);
    strcat (setName, "!");
    for (pch = setName + strlen(ODY_ROOT); *pch; pch++) {
      if (*pch == '/') {
	*pch = '\\';
      } /* endif */
    } /* endfor */

    /* Open the set */
    if (interpret_pages)
      set = set_open(setName, SETS_ANTICIPATE_DIGEST | SETS_ANTICIPATE_ITERATE
		     | SETS_PREFETCH | SETS_INTERPRET_PAGE);
    else
      set = set_open(setName, SETS_ANTICIPATE_DIGEST | SETS_ANTICIPATE_ITERATE
		     | SETS_PREFETCH);
    AddOpenSet (set, url);

    free (setName);
  } else {
    printf ("Memory error\n");
    exit (1);
  } /* endif */

  return (set);
}

int CreateSetUnion (char* set1, char* set2, char* setName)
{
  int sp1, sp2;
  int set = -1;

  /* Find the specified sets in the list of open sets */
  sp1 = FindOpenSetWithName (set1);
  sp2 = FindOpenSetWithName (set2);
  if (sp1 >= 0 && sp2 >= 0) {
    set = set_union (sp1, sp2, 0);
    AddOpenSet (set, setName);
  } /* endif */

  return (set);
}

int CreateSetIntersection (char* set1, char* set2, char* setName)
{
  int sp1, sp2;
  int set = -1;

  /* Find the specified sets in the list of open sets */
  sp1 = FindOpenSetWithName (set1);
  sp2 = FindOpenSetWithName (set2);
  if (sp1 >= 0 && sp2 >= 0) {
    set = set_intersect (sp1, sp2, 0);
    AddOpenSet (set, setName);
  } /* endif */

  return (set);
}

int IterateOnSet (int set, char* url, int len) 
{
  char name[MAX_DIGEST_NAME_LEN];
  int rc = 0; /* Return code from this function - assume success */
  int fd;  /* Handle to iterated set member */
  int error; /* ? */

  /* Iterate on the set */
  fd = set_iterate(set, (SETS_GETNAME | SETS_GETERR), name, MAX_DIGEST_NAME_LEN, &error);
  if (fd >= 0) {
    close(fd);
    rc = OdyToWWW (name, url, len);
    if (rc) {
	printf("copy error: %s -> %s (%d)\n", url, name, len);
	rc = 2; /* Error occurred during copy */
    } /* endif */
  } else if (errno == ENOENT) {
    /* Set is yeilded */
    rc = 1; /* Special yeilded rc */
  } else {
    if (errno == ENOENT) {
      /* Set is yeilded */
      rc = 1; /* Special yeilded rc */
    } else {
      rc = 3; /* Sets error occurred */
    }
  }

  return (rc);
}

int GetSetSize (int set) 
{
  return (set_size (set));
}

void RewindSetDigest (int set)
{
  int rc;

  rc = set_rewind_digest (set);
}

void RewindSetIterator (int set)
{
  int rc;

  rc = set_rewind_iterator (set);
}

void GetDigestOfSet (int set, Digest* pDigest)
{
  int nbytes; 
  struct DigestT *buffer, *elem;
  DigestElem* pDigestElement;
  int rc;
  int digest_again = 1;

  pDigest->pList = 0;
  pDigest->isFullDigest = 0; /* Assume partial digest */

  buffer = (struct DigestT *)malloc(4096);
  while (digest_again) {
      elem = NULL;
      nbytes = set_digest (set, buffer, 4096);
      if (nbytes) {
	  for (elem = buffer; elem; elem = elem->next) {
	      if (strcmp(&elem->name, "...") == 0) {
		  break;
	      } else {
		  pDigestElement = (DigestElem *) malloc (sizeof (DigestElem));
		  rc = OdyToWWW (&elem->name, pDigestElement->name, 
				 MAX_DIGEST_NAME_LEN);
		  if (rc) {
		      printf ("Unable to copy buffer, rc = %d\n(%s)\n", rc,
			      &elem->name);
		  } else {
		      pDigest->numberOfMembers++;
		      pDigestElement->isFetched = elem->fetched;
		      pDigestElement->pNext = pDigest->pList;
		      pDigest->pList = pDigestElement;
		  }
	      }
	  }
      }
      /* nbytes == 0 treated as complete digest */
      if (!elem) {
	  digest_again = 0;
	  pDigest->isFullDigest = 1;
      }
  } /* endif */

  free (buffer);
}

void CloseSet (int set)
{
  int rc;

  rc = set_close (set);
  if (rc != 0) {
    printf ("Error closing set, rc = %d\n", rc);
  } /* endif */

  RemoveOpenSet (set);
}

