#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*

          Coda: an Experimental Distributed File System

       Copyright (c) 1996 Carnegie Mellon University
                      All Rights Reserved.

Permission  to use, copy, modify and distribute this software and
its documentation is hereby granted (including for commercial  or
for-profit use), provided that both the copyright notice and this
permission  notice  appear  in  all  copies  of   the   software,
derivative  works or modified versions, and any portions thereof,
and that both notices appear  in  supporting  documentation,  and
that  credit  is  given  to  Carnegie  Mellon  University  in all
publications reporting on direct or indirect use of this code  or
its derivatives.

CODA  IS  AN  EXPERIMENTAL  SOFTWARE PACKAGE AND IS KNOWN TO HAVE
BUGS, SOME OF WHICH MAY  HAVE  SERIOUS  CONSEQUENCES.    CARNEGIE
MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.
CARNEGIE MELLON DISCLAIMS ANY  LIABILITY  OF  ANY  KIND  FOR  ANY
DAMAGES  WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE
OF THIS SOFTWARE OR OF ANY DERIVATIVE WORK.

Carnegie Mellon encourages (but does not require) users  of  this
software to return any improvements or extensions that they make,
and to grant Carnegie Mellon the  rights  to  redistribute  these
changes  without  encumbrance.   

*/

/* Modified for use with Odyssey speech warden by J. Flinn 3/25/97 */

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#ifdef __NetBSD__
#include <sys/audioio.h>
#elif defined(__FreeBSD__)
#include <machine/soundcard.h>
#elif defined(__linux__)
#include <linux/soundcard.h>
#endif
#include "include/odytypes.h"
#include <X11/keysym.h>
#include <Xm/Xm.h>
#include "WWWSets.h"
#include "SpeechDialog.h"
#include "speech_awedio.h"


/* Since we are using the NetBSD "audio_info" structure, we need to define
   it for other archs. We only define the fields we care about */
#ifndef __NetBSD__

struct audio_prinfo {
    unsigned sample_rate;
    unsigned precision;
    unsigned encoding;
    unsigned channels;
};

struct audio_info {
    struct audio_prinfo play;
    struct audio_prinfo record;
    unsigned blocksize; /* read/write blocks size */
};
/*
 * Supported audio encodings
 */
/* Encoding ID's */
#define AUDIO_ENCODING_NONE             0 /* no encoding assigned */
#define AUDIO_ENCODING_ULAW             1 /* ITU G.711 mu-law */
#define AUDIO_ENCODING_ALAW             2 /* ITU G.711 A-law */
#define AUDIO_ENCODING_PCM16            3 /* signed linear PCM, obsolete */
#define AUDIO_ENCODING_LINEAR           AUDIO_ENCODING_PCM16 /* SunOS compat */
#define AUDIO_ENCODING_PCM8             4 /* unsigned linear PCM, obsolete */
#define AUDIO_ENCODING_LINEAR8          AUDIO_ENCODING_PCM8 /* SunOS compat */
#define AUDIO_ENCODING_ADPCM            5 /* adaptive differential PCM */
#define AUDIO_ENCODING_SLINEAR_LE       6
#define AUDIO_ENCODING_SLINEAR_BE       7
#define AUDIO_ENCODING_ULINEAR_LE       8
#define AUDIO_ENCODING_ULINEAR_BE       9
#define AUDIO_ENCODING_SLINEAR          10
#define AUDIO_ENCODING_ULINEAR          11
#define AUDIO_ENCODING_MPEG_L1_STREAM   12
#define AUDIO_ENCODING_MPEG_L1_PACKETS  13
#define AUDIO_ENCODING_MPEG_L1_SYSTEM   14
#define AUDIO_ENCODING_MPEG_L2_STREAM   15
#define AUDIO_ENCODING_MPEG_L2_PACKETS  16
#define AUDIO_ENCODING_MPEG_L2_SYSTEM   17

#endif

#define BLOCK_FACTOR 10

extern int ody_fd;

static int rec_on; /* Set to 1 while recording */

static void sighndlr()
{
  rec_on = 0;
}

static struct sigaction sig;

static int auopen(char *fname, int mode)
{
  int fd;
  
  fd = open(fname, mode);
  if (fd < 0) {
    printf("open %s failed, errno = %d\n", fname, errno);
  } /* endif */

  return (fd);
}

static void audio_getinfo(int fd, struct audio_info *ip)
{
  int ret = 0;
  
#ifdef __NetBSD__
  if ((ret = ioctl(fd, AUDIO_GETINFO, ip)) < 0) {
    printf("ioctl(%d, AUDIO_GETINFO, ... )failed, errno %d\n",
	   fd, errno);
    perror("ioctl");
  }
#else
  int ans;
  if ((ret = ioctl(fd, SNDCTL_DSP_GETFMTS, &ans)) < 0) {
      printf("ioctl(%d, SDNCTL_DSP_GETFMTS, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }
  switch (ans) {
  case AFMT_MU_LAW:
  case AFMT_A_LAW:
  case AFMT_IMA_ADPCM:
      ip->play.precision = 4;
      ip->record.precision = 4;
      break;
  case AFMT_U8:
  case AFMT_S8:
      ip->play.precision = 8;
      ip->record.precision = 8;
      break;
  case AFMT_S16_LE:
  case AFMT_S16_BE:
  case AFMT_U16_LE:
  case AFMT_U16_BE:
      ip->play.precision = 16;
      ip->record.precision = 16;
      break;
  case AFMT_MPEG:
  default:
      ip->play.precision = 0;
      ip->record.precision = 0;
      break;
  }
  
  switch (ans) {
  case AFMT_MU_LAW:
      ip->play.encoding = AUDIO_ENCODING_ULAW;
      ip->record.encoding = AUDIO_ENCODING_ULAW;
      break;
  case AFMT_A_LAW:
      ip->play.encoding = AUDIO_ENCODING_ALAW;
      ip->record.encoding = AUDIO_ENCODING_ALAW;
      break;
  case AFMT_IMA_ADPCM:
      ip->play.encoding = AUDIO_ENCODING_ADPCM;
      ip->record.encoding = AUDIO_ENCODING_ADPCM;
      break;
  case AFMT_U8:
      ip->play.encoding = AUDIO_ENCODING_PCM8;
      ip->record.encoding = AUDIO_ENCODING_PCM8;
      break;
  case AFMT_S8:
      ip->play.encoding = AUDIO_ENCODING_LINEAR8;
      ip->record.encoding = AUDIO_ENCODING_LINEAR8;
      break;
  case AFMT_S16_LE:
      ip->play.encoding = AUDIO_ENCODING_SLINEAR_LE;
      ip->record.encoding = AUDIO_ENCODING_SLINEAR_LE;
      break;
  case AFMT_S16_BE:
      ip->play.encoding = AUDIO_ENCODING_SLINEAR_BE;
      ip->record.encoding = AUDIO_ENCODING_SLINEAR_BE;
      break;
  case AFMT_U16_LE:
      ip->play.encoding = AUDIO_ENCODING_ULINEAR_LE;
      ip->record.encoding = AUDIO_ENCODING_ULINEAR_LE;
      break;
  case AFMT_U16_BE:
      ip->play.encoding = AUDIO_ENCODING_ULINEAR_BE;
      ip->record.encoding = AUDIO_ENCODING_ULINEAR_BE;
      break;
  case AFMT_MPEG:
  default:
      ip->play.encoding = AUDIO_ENCODING_NONE;
      ip->record.encoding = AUDIO_ENCODING_NONE;
      break;
  }
  if ((ret = ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &ans)) < 0) {
      printf("ioctl(%d, SDNCTL_DSP_GETBLKSIZE, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }
  ip->blocksize = ans;

  if ((ret = ioctl(fd, SOUND_PCM_READ_RATE, &ans)) < 0) {
      printf("ioctl(%d, SOUND_PCM_READ_RATE, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }
  ip->play.sample_rate = ip->record.sample_rate = ans;

  if ((ret = ioctl(fd, SOUND_PCM_READ_CHANNELS, &ans)) < 0) {
      printf("ioctl(%d, SOUND_PCM_READ_CHANNELS, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }
  ip->play.channels = ip->record.channels = ans;

  if ((ret = ioctl(fd, SOUND_PCM_READ_BITS, &ans)) < 0) {
      printf("ioctl(%d, SOUND_PCM_READ_BITS, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }
  ip->play.precision = ip->record.precision = ans;

#endif
}

static void audio_setinfo(int fd, struct audio_info *ip)
{
  int ret = 0;

#ifdef __NetBSD__
  if ((ret = ioctl(fd, AUDIO_SETINFO, ip)) < 0) {
    printf("ioctl(%d, AUDIO_SETINFO, ... )failed, errno %d\n",
	   fd, errno);
    perror("ioctl");
    exit(1);
  } else if ((ret = ioctl(fd, AUDIO_GETINFO, ip)) < 0) {
    printf("ioctl(%d, AUDIO_GETINFO, ... )failed, errno %d\n",
	   fd, errno);
    perror("ioctl");
  }
#else
  int ans;
  switch (ip->play.encoding | ip->record.encoding) {
  case AUDIO_ENCODING_ULAW:
      ans = AFMT_MU_LAW;
      break;
  case AUDIO_ENCODING_ALAW:
      ans = AFMT_A_LAW;
      break;
  case AUDIO_ENCODING_ADPCM:
      ans = AFMT_IMA_ADPCM;
      break;
  case AUDIO_ENCODING_PCM16:
      ans = AFMT_U8;
      break;
  case AUDIO_ENCODING_PCM8:
      ans = AFMT_U8;
      break;
/* case AUDIO_ENCODING_LINEAR8:
   ans = AFMT_S8;
   break;
   */
  case AUDIO_ENCODING_SLINEAR_LE:
      ans = AFMT_S16_LE;
      break;
  case AUDIO_ENCODING_SLINEAR_BE:
      ans = AFMT_S16_BE;
      break;
  case AUDIO_ENCODING_ULINEAR_LE:
      ans = AFMT_U16_LE;
      break;
  case AUDIO_ENCODING_ULINEAR_BE:
      ans = AFMT_U16_BE;
      break;
  case AUDIO_ENCODING_NONE:
  default:
      ans = 0;
      break;
  }

  if ((ret = ioctl(fd, SNDCTL_DSP_SETFMT, &ans)) < 0) {
      printf("ioctl(%d, SDNCTL_DSP_SETFMT, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }

  fprintf(stderr,"Warning: Don't know how to set block size under Linux!\n");

  ans = ip->play.sample_rate; /* | ip->record.sample_rate; by rvb. why? */
  if ((ret = ioctl(fd, SOUND_PCM_WRITE_RATE, &ans)) < 0) {
      printf("ioctl(%d, SOUND_PCM_WRITE_RATE, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }
  ans = ip->play.channels; /* - 1 | ip->record.channels - 1; by rvb. why? */
  if ((ret = ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &ans)) < 0) {
      printf("ioctl(%d, SDNCTL_DSP_STEREO, ...) failed, errno %d\n",
	     fd, errno);
      perror("ioctl");
      exit(1);
  }

  fprintf(stderr,"Warning: Cannot set precision under Linux\n");
#endif

}

/* Set parameters to the Janus defaults */
static void setparams_in(struct audio_info *ip)
{
  struct audio_prinfo *xp;
  
  xp = &ip->record;
  ip->record.sample_rate = 16000;
  ip->play.sample_rate = 16000; /* Is this necessary? */
  xp->encoding = AUDIO_ENCODING_SLINEAR_LE;
  xp->precision = 16;

   /* XXX because of limitations in the NetBSD 1.3 sound model,
          the record parameters are currently happily ignored.
          To "fix" this, we set the play parameters to be the same
          as the record parameters, after having been good up till 
          now. [-et Jun 4 1998] */

  /* XXX Because of limitations in the Linux interface, there is only one
     call to set each parameter, presumable for both play and record. Another
     reason to keep play and record parameters the same */

   ip->record = *xp;
   ip->play = *xp;
}

/* Set parameters to au standard */
static void setparams_out(struct audio_info *ip)
{
  struct audio_prinfo *xp;
  
  xp = &ip->play;
  ip->record.sample_rate = 8000; 
  ip->play.sample_rate = 8000; 
  xp->encoding = 1;
  xp->precision = 8;
}

static int audio_in (int fd, struct audio_info *ip, char* buffer, int len,
	      XtAppContext* pAppContext, SetsData* pSetsData)
{
  int blksize = ip->blocksize;
  int num = BLOCK_FACTOR;
  int i;
  int sz = num * blksize;
  XEvent event;
  int offset = 0;

  if (fcntl(0, F_SETFL, O_NONBLOCK) < 0) {	/* peek at stdin */
    printf("stdin O_NONBLOCK failed, errno = %d\n", errno);
    perror("fcntl");
    exit(1);
  }

  rec_on = 1;
  while (rec_on) {

    if (offset + sz > len) {
      printf ("Input buffer too small\n");
      return (0);
    } /* endif */

    i = read (fd, buffer+offset, sz);
    offset += i;
    if (i != sz) {
      if (errno != EINTR) {
	printf("short read %d vs %d\n",
	       i/blksize, sz/blksize);
      } /* endif */
    } /* endif */
    
    /* Dispatch X events that are pending - this allows user to interrupt
       the recording. */
    while (XtAppPending (*pAppContext)) {
      XtAppNextEvent (*pAppContext, &event);

      if (event.xany.type == KeyRelease) {
	if (event.xkey.keycode == 
	    XKeysymToKeycode (pSetsData->display, XK_F11)) {

	  Speech_Update_Utterance ("<RECOGNIZING>");

	  /* Stop recording audio input */
	  rec_on = 0;
	  
	} /* endif */
      } /* endif */
      
      XtDispatchEvent (&event);
    } /* endwhile */

  } /* endwhile */
  
  return (offset);
}

/* play back */
static void audio_out(int fd, struct audio_info *ip, char *fname)
{
  FILE *ofd;
  int blksize = ip->blocksize;
  int num = BLOCK_FACTOR;
  char *buf;
  int i;
  
  ofd = fopen(fname, "r");
  if (ofd == NULL) {
    printf("fopen %s failed, errno = %d\n",
	   fname, errno);
    perror("fopen");
    exit(1);
  }
  
  buf = (char *)malloc(num * blksize);
  
  printf ("Playing file\n");

  while (!feof(ofd)) {
    i = fread(buf, blksize, num, ofd);
    write(fd, buf, i * blksize);
  }
}

int record_audio (char* buffer, int len, XtAppContext* pAppContext, 
		  SetsData* pSetsData)
{
  int fd;
  int outputLen;
  struct audio_info info;
  
  fd = auopen("/dev/audio", O_RDONLY);
  if (fd < 0) {
    return (0);
  } /* endif */

  audio_getinfo(fd, &info);
  setparams_in(&info);
  audio_setinfo(fd, &info);

  sig.sa_handler = sighndlr;
  sigaction(SIGINT,  &sig, 0);
  sigaction(SIGQUIT, &sig, 0);
  outputLen = audio_in (fd, &info, buffer, len, pAppContext, pSetsData);
  audio_getinfo(fd, &info);

  close (fd); 

  return (outputLen);
}

void play_audio (char* fname)
{
  int fd;
  struct audio_info info;

  fd = auopen("/dev/audio", O_WRONLY);
  if (fd < 0) {
    return;
  } /* endif */

  audio_getinfo(fd, &info);
  setparams_out(&info);
  audio_setinfo(fd, &info);

  audio_out (fd, &info, fname);
  close (fd);
}


