
/* xa_qt_clnt_stub.c
   Access Odyssey movies via tsops
*/

#include "xanim.h"
#undef FREE /* odytypes.h has its own version */

#ifdef INSTRUMENTED
#include <ds_histo.h>
#include <sys/upcall.h>
#include <nettr/replay/ntrp_upcall.h>
#endif
   
#include <sys/time.h>
#include <strings.h>
#include <netinet/in.h>

#include "include/odytypes.h"
#include "include/odyssey.h"
#include "odyssey/wardens/qt/qt.h"

#ifdef BW_TOOL
#include <sys/socket.h>
#include "include/bw_tool.h"
#ifdef __FreeBSD__
#include <rpc/types.h>  /* for INADDR_LOOPBACK */
#endif
#endif

#ifdef PSCOPE
#include <linux/pscope.h>
static int pscope_qt_init = 0;
extern int pscope_profile;
#endif

int start_at_done = 0;

extern xaULONG xa_vidcodec_maxsize;
extern xaUBYTE *xa_vidcodec_buf;
extern XA_ANIM_HDR *cur_file, *first_file;
extern xaLONG xa_ody_indirect;
int cur_trackID;
#ifdef BW_TOOL
int mon_fd;
#endif
static int last_trackID;
extern XA_ANIM_HDR **tracks;
extern int num_tracks;
static int movie_fd = -1;

static int cur_bw = 0;
static int xa_bw_reqid = 0;
static int xa_en_reqid = 0;
static void adapt_new_bw(int reqid, unsigned int rsrc, int val);
static void adapt_new_en(int reqid, unsigned int rsrc, int val);
extern int adapt_power;
extern int start_at_fidelity;

#ifdef PSCOPE
int dropped_frames = 0;
#endif
#ifdef INSTRUMENTED
extern int instrumented;
extern struct hgram display_time_hg;
extern struct hgram lateness_hg;
extern struct hgram between_frames_hg;
extern struct hgram skip_hg;
struct hgram fetch_time_hg;
int times_switched_up;
int times_switched_down;
int nlower_quality;
int nhigher_quality;
extern int early_frames, late_frames;
int total_frames;
int dropped_frames = 0;
int last_frame;
struct timeval tv1, tv2, tvdiff;
FILE *timestamp_log;

#define NUMSNAPSHOTS 1024
#define SS_MAXTRACKS 10
static struct snapshot {
    xaLONG tsnap;
    int total_frames;
    int track_counts[SS_MAXTRACKS];
    int last_frame;
    int dropped_frames;
    int times_switched_up;
    int times_switched_down;
    int nhigher_quality;
    int nlower_quality;
    int late_frames;
    int early_frames;
}
snapshots[NUMSNAPSHOTS];

static int next_snap = 0;

/* function to grab a snapshot */
void take_snapshot(xaLONG tsnap)
{
    int i;
    ASSERT(next_snap < NUMSNAPSHOTS);
    ASSERT(SS_MAXTRACKS >= num_tracks);

    snapshots[next_snap].tsnap = tsnap;
    snapshots[next_snap].total_frames = total_frames;
    for (i = 0; i < num_tracks; i++)
	snapshots[next_snap].track_counts[i] = tracks[i]->total_frames;
    snapshots[next_snap].last_frame = last_frame;
    snapshots[next_snap].dropped_frames = dropped_frames;
    snapshots[next_snap].times_switched_up = times_switched_up;
    snapshots[next_snap].times_switched_down = times_switched_down;
    snapshots[next_snap].nhigher_quality = nhigher_quality;
    snapshots[next_snap].nlower_quality = nlower_quality;
    snapshots[next_snap].late_frames = late_frames;
    snapshots[next_snap].early_frames = early_frames;
    next_snap++;
}

/* function for dumping all the snapshots */
void dump_snaps(void) {
    int i;
    FILE *fp = fopen("xanim.snapshots","w");
    
    ASSERT(fp);

    fprintf(fp,"%5s %21s %4s %4s %3s %3s %3s %3s %3s %3s\n",
	    "time", "frame_counts", "lst", "drp", "up", "dn", "hi", "lo", "lt", "ea");
    for (i = 0; i < next_snap; i++) {
	fprintf(fp,"%5d %4d (%4d %4d %4d) %4d %4d %3d %3d %3d %3d %3d %3d\n",
		snapshots[i].tsnap, snapshots[i].total_frames,
		snapshots[i].track_counts[0],
		snapshots[i].track_counts[1],
		snapshots[i].track_counts[2],
		snapshots[i].last_frame,
		snapshots[i].dropped_frames,
		snapshots[i].times_switched_up,
		snapshots[i].times_switched_down,
		snapshots[i].nhigher_quality,
		snapshots[i].nlower_quality,
		snapshots[i].late_frames,
		snapshots[i].early_frames);
    }

    fclose(fp);
}


/* function for printing out all the statistics */
void do_dump(char *s)
{
    static int uniq = 0;
    FILE *fp;
    char filename[64];
    time_t t;
    struct tm *tp;
    int i;
    
    if (time(&t) == -1) {
	fprintf(stderr,"dump_stats: time(): %s",strerror(errno));
	return;
    }
    
    t = time(NULL);
    tp = localtime(&t);
    sprintf(filename,"xanim.stats.%d.%d.%d:%d",uniq++,
	    tp->tm_yday, tp->tm_hour, tp->tm_min);
    
    if (!(fp = fopen(filename,"w"))) {
	fprintf(stderr,"dump_stats: %s: %s",filename,strerror(errno));
	fflush(stderr);
	return;
    }
    
    fprintf(fp, "%s", s);

    if (tracks) {
	fprintf(fp,"track_names:");
	for (i = 0; i < num_tracks; i++)
	    fprintf(fp," %s", tracks[i]->codec_name);
	fprintf(fp,"\ntrack_counts:");
	for (i = 0; i < num_tracks; i++)
	    fprintf(fp," %d",tracks[i]->total_frames);
    }
    fprintf(fp,"\ntotal_frames: %d\n",total_frames);
    fprintf(fp,"\nlast_frame: %d\n",last_frame);
    fprintf(fp,"higher_switches: %d\n", times_switched_up);
    fprintf(fp,"lower_switches: %d\n", times_switched_up);
    fprintf(fp,"too_high: %d\n", nhigher_quality);
    fprintf(fp,"too_low: %d\n", nlower_quality);
    fprintf(fp,"too_late: %d\n",late_frames);
    fprintf(fp,"too_early: %d\n",early_frames);

    fprintf(fp,"Metric: time to fetch a frame from system (sec)\n");
    PrintHisto2("fetch_time",fp,&fetch_time_hg);
    fprintf(fp,"Metric: time to decode and display a frame (sec)\n");
    PrintHisto2("display_time",fp,&display_time_hg);
    fprintf(fp,"Metric: lateness of display time compared to frame time (sec)\n");
    PrintHisto2("lateness",fp,&lateness_hg);
    fprintf(fp,"Metric: interframe time (time between displays) (sec)\n");
    PrintHisto2("interframe_time",fp,&between_frames_hg);
    fprintf(fp,"Metric: # frames skipped between two successive ones\n");
    PrintHisto2("skip",fp,&skip_hg);

    fclose(fp);
}

static int ntrace_started = 0;
static int ntrace_ended = 0;
extern int ody_wait_upcall;
static int ntrace_transitioned = 0;
static struct timeval upcall_time;
static char tsev[1024];

extern void TheEnd(void);

/* dump statistics into a file every time we get an upcall from ntrace */
static struct timeval trace_start_time, trace_end_time;

static int
handle_upcall(upclass, upcall, argsz, argp, retszp, retp, flags)
    upclass_t upclass;
    upcall_t  upcall;
    size_t    argsz;
    void     *argp;
    size_t   *retszp;
    void     *retp;
    int       flags;
{
    struct ntrp_tev_arg *targp;

    ASSERT(ody_wait_upcall);
    switch (upclass) {
    case UC_NTRP:
	break;
    default:
	fprintf(stderr,"handle_upcall: unknown upclass %d\n", upclass);
	goto exit;
	break;
    }

    targp = (struct ntrp_tev_arg *)argp;

    switch (upcall) {
    case NTRP_NOTIFY_START:
	ntrace_started = 1;
	gettimeofday(&trace_start_time,0);
	break;
    case NTRP_NOTIFY_END:
	ntrace_ended = 1;
	gettimeofday(&trace_end_time,0);
	timersub(&trace_end_time,&trace_start_time,&trace_end_time);
	printf("Trace started %ld.%06ld, took %ld.%06ld\n",
	       trace_start_time.tv_sec,trace_start_time.tv_usec,
	       trace_end_time.tv_sec,trace_end_time.tv_usec);
	break;
    case NTRP_NOTIFY_TSEV:
	ntrace_transitioned = 1;
	break;
    default:
	fprintf(stderr,"handle_upcall: unknown UC_NTRP upcall %d\n", upcall);
	fflush(stderr);
	exit(1);
    }

    upcall_time = targp->nta_tv;

 exit:
    fflush(stderr);
    return 0;
}

#endif /* INSTRUMENTED */

/* outside limits for setting up bw bounds */
#define MAX_BW (10000000 * 8)
#define MIN_BW 0

/* code for adapting to changes in network bandwidth */

/* set up an upcall when bandwidth crosses given bounds */
static int set_up_request(int low_bw, int high_bw)
{
    struct ody_req_des or_des;
    
    or_des.ord_resource = ODY_GR_NETBW;
    or_des.ord_version.ov_gs = ODY_GENERIC_INTERFACE;
    or_des.ord_version.ov_type = QT_TTYPE;
    or_des.ord_version.ov_tsvnum = QT_VERSION;
    or_des.ord_low = low_bw;
    or_des.ord_high = high_bw;
    or_des.ord_fp = adapt_new_bw;

    if (xa_verbose)
	fprintf(stderr, "request range %d...%d\n", low_bw, high_bw);

    return ody_frequest(movie_fd, &or_des, &xa_bw_reqid);
}

void QT_SwitchTrack(new_trackID)
int new_trackID;
{
  XA_ANIM_HDR *new_file;
#ifdef INSTRUMENTED
  int new_size, old_size;
#endif /* INSTRUMENTED */
#ifdef BW_TOOL
  struct sockaddr_in mon_addr;
  int rc;
#endif

  if ((new_trackID < 0) || (new_trackID >= num_tracks) ||
      (tracks[new_trackID] == NULL))
      return;

#ifdef INSTRUMENTED
  if (instrumented) {
      new_size = tracks[new_trackID]->ave_fvid_size;
      old_size = tracks[cur_trackID]->ave_fvid_size;
      
      if (new_size > old_size)
	  times_switched_up++;
      else if (new_size < old_size)
	  times_switched_down++;

  }
#endif /* INSTRUMENTED */

#ifdef BW_TOOL
  /* Send the new string to the bwmon 
   * XXX no check if there IS a bwmon -- does it matter? XXX */
  mon_addr.sin_family = AF_INET;
  /* right way to do localhost? */
  mon_addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
  mon_addr.sin_port = htons(BW_QT_MON_PORT);
  rc = sendto(mon_fd, tracks[new_trackID]->codec_name,
	      strlen(tracks[new_trackID]->codec_name),
	      0, (struct sockaddr *)&mon_addr, sizeof(mon_addr));
  if (rc < strlen(tracks[new_trackID]->codec_name)) {
      perror("sendto");
  }
#endif

  new_file = tracks[new_trackID];
  cur_trackID = new_trackID;

  if (xa_verbose)
#ifdef PSCOPE 
    {
      struct timeval tv;
      
      gettimeofday (&tv, NULL);
      printf("%ld.%ld: QT_SwitchTrack(%d): %s\n", tv.tv_sec, tv.tv_usec,
	    cur_trackID, new_file->codec_name);
    }
#else 
    fprintf(stderr,"QT_SwitchTrack(%d): %s (%u/%u) needs %d bytes/sec\n",

	    cur_trackID, new_file->codec_name,new_file->ave_fvid_size,
	    new_file->max_fvid_size, new_file->bw_reqd);
#endif
}

/* function that is upcalled when bandwidth steps out of registered bounds */
/* it may be CALLED (rather than upcalled), for initializing the first
   request, in which case reqid will NOT equal xa_bw_reqid */

static void adapt_new_bw(int reqid, unsigned int rsrc, int val)
{

    ASSERT(reqid == xa_bw_reqid);
    ASSERT(rsrc == ODY_GR_NETBW);

    while(1) {
	int low_bw = MIN_BW, high_bw = MAX_BW;
	int new_track;

	if (xa_verbose)
	    fprintf(stderr,"Bandwidth is now %d bytes/sec\n",val);

	/* HACK: assume that the larger the codec, the better the quality */
	/* HACK 2: assume codecs are in increasing order of size/quality */

	/* find the largest codec for which we can sustain 100% of the
	   required data rate */

	for (new_track = num_tracks-1; new_track > 0; new_track--) {
	    int bw_reqd = tracks[new_track]->bw_reqd;
	    if (bw_reqd <= val)
		break;
	}

	/* if there's a smaller codec, set low-water mark to 95% of bw */
	if (new_track > 0) {
	    low_bw = tracks[new_track]->bw_reqd;
	    low_bw -= low_bw/20;
	}

	/* if there's a larger codec, set high-water mark to 100% of its bw */
	if (new_track < num_tracks-1) {
	    high_bw = tracks[new_track+1]->bw_reqd;
	}

	QT_SwitchTrack(new_track);

	if (set_up_request(low_bw, high_bw) < 0) {
	    if (errno != ODY_ENOTINWINDOW) {
		perror("set_up_request");
		exit(1);
	    }
	    val = xa_bw_reqid;
	    ASSERT(val >= MIN_BW);
	    ASSERT(val <= MAX_BW);
	    
	    if (xa_verbose)
		fprintf(stderr,"set_up_request: not in window (%d,%d)\n",
			low_bw, high_bw);
	}
	else {
#ifdef INSTRUMENTED
	   struct timeval tv;
	   if (instrumented) {
	       gettimeofday(&tv,NULL);
	       fprintf(timestamp_log,"%ld.%06ld: upcall: new bw %d,"
		       " window (%d,%d), track %d\n", tv.tv_sec, tv.tv_usec,
		       val, low_bw, high_bw, new_track);
	   }
#endif /* INSTRUMENTED */
	    break;
	}
    }

    if (xa_verbose)
	fprintf(stderr,"adapted to new bandwidth\n");
}

/* function that is upcalled when power buget is out of bounds */
/* it may be CALLED (rather than upcalled), for initializing the first
   request */
static void adapt_new_en(int reqid, unsigned int rsrc, int val)
{
    struct ody_req_des or_des;

    ASSERT(rsrc == ODY_GR_POWER);
    
    while(1) {
	if (val > num_tracks-1 || val < 0) {
	    ASSERT (0);
	}

	/* Switch to new fidelity level */
	if (!start_at_fidelity || start_at_done) QT_SwitchTrack(val);

	/* Register new fidelity level */
	or_des.ord_resource = ODY_GR_POWER;
	or_des.ord_version.ov_gs = ODY_GENERIC_INTERFACE;
	or_des.ord_version.ov_type = QT_TTYPE;
	or_des.ord_version.ov_tsvnum = QT_VERSION;
	or_des.ord_low = val;
	or_des.ord_high = val;
	or_des.ord_fp = adapt_new_en;

	if (ody_frequest(movie_fd, &or_des, &xa_en_reqid) < 0) {
	    if (errno != ODY_ENOTINWINDOW) {
		perror("ody_frequest");
		exit(1);
	    }
	    val = xa_en_reqid;
	    
	    if (xa_verbose)
		fprintf(stderr,"set_up_request: not in window (%d,%d)\n",
			val, val);
	}
	else {

	    if (start_at_fidelity && !start_at_done) {
		FILE* file = fopen("/tmp/qt_fidelity", "r");
	        start_at_done = 1;
		if (file) {
		    char line[16];
		    int start_fidelity;

		    fgets (line, sizeof(line), file);
		    start_fidelity = atoi (line);
		    printf ("QT starting at fidelity %d\n", start_fidelity);
		    QT_SwitchTrack(start_fidelity);
		    
		    or_des.ord_low = start_fidelity;
		    or_des.ord_high = start_fidelity;
		    if (ody_frequest(movie_fd, &or_des, &xa_en_reqid) < 0) {
			if (errno != ODY_ENOTINWINDOW) {
			    perror("ody_frequest");
			    exit(1);
			}
			val = xa_en_reqid;
			
			if (xa_verbose)
			    fprintf(stderr,
				    "set_up_request: not in window (%d,%d)\n",
				    val, val);
		    }
		    else {
			break;
		    }

		    fclose (file);
		    break;
		} else {
		  break;
		}
	    } else {
		break;
	    }
	}
	    
    }

    if (xa_verbose)
	fprintf(stderr,"adapted to new power budget\n");
}


#ifdef INSTRUMENTED

/* initialize ntr upcalls and other measurement-related stuff */

void Ody_Init(void)
{
    void *regrc;

    if (instrumented) {
	InitHisto(&fetch_time_hg, 0.0, 1.0, 10, LINEAR);
	InitHisto(&display_time_hg, 0.0, 0.2, 10, LINEAR);
	InitHisto(&lateness_hg, -5.0, 20.0, 2500, LINEAR);
	InitHisto(&skip_hg, 0.0, 100.0, 100, LINEAR);
	InitHisto(&between_frames_hg, 0.05, 0.15, 10, LINEAR);
	nlower_quality = 0;
	nhigher_quality = 0;
	times_switched_up = 0;
	times_switched_down = 0;
	total_frames = 0;

	timestamp_log = fopen("xanim.stats.timestamps","w");
    }

    if (ody_wait_upcall) {
	regrc = upcall_reg(UC_NTRP, NTRP_NOTIFY_START, handle_upcall);
	if (regrc == UC_ERR) {
	    perror("upcall_reg: NTRP_NOTIFY_START");
	    exit(errno);
	}
	
	regrc = upcall_reg(UC_NTRP, NTRP_NOTIFY_TSEV, handle_upcall);
	if (regrc == UC_ERR) {
	    perror("upcall_reg: NTRP_NOTIFY_TSEV");
	    exit(errno);
	}
	
	regrc = upcall_reg(UC_NTRP, NTRP_NOTIFY_END, handle_upcall);
	if (regrc == UC_ERR) {
	    perror("upcall_reg: NTRP_NOTIFY_END");
	    exit(errno);
	}
    }
}

#endif /* INSTRUMENTED */

/* just start up the whole thing by assuming current bandwidth is what
 we want for our current track */
void Init_Adaptation(void)
{
    int i;
    extern void Setup_Anim_Hdr();
    for (i = 0; i < num_tracks; i++) {
	if (tracks[i]) {
	    cur_file = tracks[i];
	    Setup_Anim_Hdr();
	}
    }
    cur_bw = tracks[cur_trackID]->bw_reqd;

    first_file = cur_file = tracks[cur_trackID];

    if (adapt_power) {
	adapt_new_en(xa_en_reqid,ODY_GR_POWER,cur_trackID);
    } else {
	adapt_new_bw(xa_bw_reqid,ODY_GR_NETBW,cur_bw);
    }
}

static struct qtw_frame_info *frame_info = NULL;

/* open a movie on several tracks */

XA_ANIM_HDR **QT_OpenMovie(stem, num_tracks, audio_attempt)
char *stem;
int *num_tracks;
xaULONG audio_attempt;
{
  XA_ACTION *ACT_Get_Action();
  XA_CHDR *ACT_Get_CHDR();
  XA_ANIM_HDR **tracks;
  void ACT_Add_CHDR_To_Action(XA_ACTION *act, XA_CHDR *chdr);

  struct qtw_track_hdr *qtw_thp;
  int i, len;

  char realmoviename[256];

  if ((movie_fd = open(stem,O_RDONLY,0)) < 0) {
      perror(stem);
      exit(1);
  }

  if (xa_ody_indirect == xaTRUE) {
      len = read(movie_fd, realmoviename, 256);
      if (len < 0) {
	  perror("read");
	  exit(1);
      }
      realmoviename[len] = '\0';
      close(movie_fd);
      if ((movie_fd = open(realmoviename,O_RDONLY,0)) < 0) {
	  perror(realmoviename);
	  exit(1);
      }      
  }

  /* allocate big arrays for all anim_hdrs, acts, chdrs, cmaps, and maps */
  
  qtw_thp = (struct qtw_track_hdr *)
      malloc(*num_tracks * sizeof(struct qtw_track_hdr));

  if (ody_qt_openmovie(movie_fd, num_tracks, qtw_thp) < 0) {
      perror("ody_qt_openmovie");
      exit(1);
  }

  if (*num_tracks <= 0)
    return NULL;

  /* allocate pointer array for track headers */
  tracks = (XA_ANIM_HDR **)calloc(*num_tracks,sizeof(XA_ANIM_HDR *));

  /* copy information from the array  */

  for (i = 0; i < *num_tracks; i++) {
    XA_ANIM_HDR *Get_Anim_Hdr();
    xaULONG QT_Setup_Codec();
    void XA_Free_Anim_Setup();
    XA_ANIM_SETUP *XA_Get_Anim_Setup(), *qt;

    /* copy anim_hdr info */
    tracks[i] = Get_Anim_Hdr(NULL,stem);
    tracks[i]->anim_type = qtw_thp[i].anim_type;
    strcpy(tracks[i]->codec_name,qtw_thp[i].codec_name);
    tracks[i]->qt_depth = qtw_thp[i].depth;
    tracks[i]->qt_id = qtw_thp[i].compression_type;

    tracks[i]->imagex = qtw_thp[i].imagex;
    tracks[i]->imagey = qtw_thp[i].imagey;
    tracks[i]->imaged = 8;
    tracks[i]->anim_flags = qtw_thp[i].anim_flags;
    tracks[i]->total_time = qtw_thp[i].total_time;
    tracks[i]->max_fvid_size = qtw_thp[i].max_fvid_size;
    tracks[i]->ave_fvid_size = qtw_thp[i].ave_fvid_size;
    tracks[i]->max_faud_size = 0;
    tracks[i]->last_frame = qtw_thp[i].last_frame;
#ifdef INSTRUMENTED
    tracks[i]->total_frames = 0;
#endif

    /* calculate bandwidth required to play this track without skipping */
     tracks[i]->bw_reqd = (int) ((double)tracks[i]->ave_fvid_size *
				 (tracks[i]->last_frame+1) * 1000 /
				 tracks[i]->total_time);

    qt = XA_Get_Anim_Setup();
    qt->imagex = tracks[i]->imagex;
    qt->imagey = tracks[i]->imagey;
    qt->depth = tracks[i]->qt_depth;
    if (QT_Setup_Codec(tracks[i],qt,tracks[i]->qt_id)) {
      tracks[i]->imagex = qt->imagex;
      tracks[i]->imagey = qt->imagey;
      tracks[i]->imagec = qt->imagec;

      /* copy act info */
      tracks[i]->acts = ACT_Get_Action(tracks[i],qtw_thp[i].act_type);
      tracks[i]->acts->cmap_rev = qtw_thp[i].cmap_rev;
      
      tracks[i]->acts->xapi_rev = qt->xapi_rev;
      tracks[i]->acts->decoder = qt->decoder;

      ACT_Add_CHDR_To_Action(tracks[i]->acts,qt->chdr);

      if (xa_vidcodec_maxsize < tracks[i]->max_fvid_size)
	xa_vidcodec_maxsize = tracks[i]->max_fvid_size;

      if (xa_verbose)
	  fprintf(stderr,"Codec %s: %d frames, %d max, %d ave, %d bytes/s\n",
		  tracks[i]->codec_name, tracks[i]->last_frame+1,
		  tracks[i]->max_fvid_size, tracks[i]->ave_fvid_size,
		  tracks[i]->bw_reqd);

    }

    else
      tracks[i] = NULL; /* this track is not usable */
    
    XA_Free_Anim_Setup(qt);

  }

  free(qtw_thp);

  if (adapt_power) {
      /* Makes more sense to start at HIGHEST track, since that is
	 what Viceroy does ??? */
      i = *num_tracks - 1;
  } else {
      /* find first usable (non-null) track */
      for (i = 0; ((tracks[i] == NULL) && (i < *num_tracks)); i++);
  }

  if (i == *num_tracks) {
    fprintf(stderr,"No usable tracks\n");
    TheEnd();
  }

  if (xa_verbose)
    fprintf(stderr,"Read %d tracks\n",*num_tracks);

  first_file = cur_file = tracks[i];
  last_trackID = cur_trackID = i;

  if ((frame_info=(struct qtw_frame_info *)
       malloc(xa_vidcodec_maxsize+sizeof(struct qtw_frame_info))) == NULL) {
    fprintf(stderr,"malloc error\n"); 
    TheEnd();
  }

  xa_vidcodec_buf = ((xaUBYTE *)frame_info) + sizeof(struct qtw_frame_info);

  return tracks;
}

/* tell server to close the movie, clean up */
void QT_CloseMovie(void)
{
  close(movie_fd);
}

/* get a frame from the server via SFTP 
   if *frame_no is -1, then frame is fetched according to zztime,
   else according to *frame_no */

static int Fetch(xaLONG zztime, int *frame_no, XA_FRAME *frame,
		  ACT_DLTA_HDR *dlta_hdr, xaUBYTE *vidcodec_buf)
{
  extern void Setup_Anim_Hdr();
  extern xaLONG file_is_started;
  struct qtw_frame_request frame_request;

#ifdef INSTRUMENTED
  extern struct timeval tv1, tv2, tvdiff;

  if (instrumented)
      gettimeofday(&tv1, NULL);

  if (ody_wait_upcall) {
      if (ntrace_ended) {
	  if (instrumented) {
	      sprintf(tsev,"Timestamp event received: %ld.%06ld\n", 
		      upcall_time.tv_sec, upcall_time.tv_usec);
	      do_dump(tsev);
	      dump_snaps();
	      fclose(timestamp_log);
	  }
	  TheEnd();
      }

      if (ntrace_transitioned) {
	  ntrace_transitioned = 0;
	  if (instrumented) {
	  sprintf(tsev,"Timestamp event received: %ld.%06ld\n", 
		  upcall_time.tv_sec, upcall_time.tv_usec);
	  do_dump(tsev);
	  }
      }
      /* wait for the ntrace to start, so we're all synchronized */
      if (!ntrace_started) {
	  if (xa_verbose)
	      fprintf(stderr,"waiting for ntrace replay\n");
	  while (!ntrace_started) {
	      select(0, (fd_set *)NULL, (fd_set *)NULL, (fd_set *)NULL,
		     (struct timeval *)NULL);
	  }
	  if (instrumented) {
	      sprintf(tsev,"Timestamp event received: %ld.%06ld\n", 
		      upcall_time.tv_sec, upcall_time.tv_usec);
	      do_dump(tsev);
	  }
      }
  }

#endif /* INSTRUMENTED */
  
  ASSERT(xa_vidcodec_buf == ((void *)frame_info) + sizeof(struct qtw_frame_info));
    
  frame_request.track_ID = cur_trackID;
  frame_request.zztime = zztime;
  if (ody_qt_getframe(movie_fd, &frame_request, frame_info,
		      xa_vidcodec_maxsize) < 0) { /* no such frame */
      ASSERT((errno == EINVAL) || (errno == ETIMEDOUT));
      return -1;
  }
  *frame_no = frame_info->frame_no;
  
  cur_file = tracks[frame_info->trackID];
  if (cur_trackID != frame_info->trackID) {
#if 0 /* This is too verbose, and not very useful */
      if (xa_verbose)
	  fprintf(stderr,"Fetch: on track %d but fetched from %d\n",
		  cur_trackID, frame_info->trackID);
#endif
#ifdef INSTRUMENTED
      if (instrumented) {
      /* if we got a track with smaller codecs than what we asked for,
	 it means we got a lower quality */
      if ((tracks[frame_info->trackID]->ave_fvid_size < 
	   tracks[cur_trackID]->ave_fvid_size))
	  nlower_quality++;
      else if ((tracks[frame_info->trackID]->ave_fvid_size > 
	   tracks[cur_trackID]->ave_fvid_size))
	  nhigher_quality++;
      }
#endif /* INSTRUMENTED */

  }
  if (last_trackID != frame_info->trackID) {
      file_is_started = xaFALSE; /* new track, may need to readjust window */
      last_trackID = frame_info->trackID;
#ifdef INSTRUMENTED
      if (instrumented) {
	  gettimeofday(&tv2,NULL);
	  fprintf(timestamp_log,"switchtrack: %d %ld.%06ld\n",
		  frame_info->trackID, tv2.tv_sec,tv2.tv_usec);
	  fflush(timestamp_log);
      }
#endif /* INSTRUMENTED */
  }

    frame->time_dur = (xaLONG) frame_info->time_dur;
    frame->zztime = (xaLONG) frame_info->zztime;
    frame->act = cur_file->acts;
    frame->act->data = NULL;
    frame->act->data = (xaUBYTE *)dlta_hdr;
    dlta_hdr->xapi_rev = (xaULONG) frame_info->xapi_rev;
    dlta_hdr->flags = (xaULONG) frame_info->flags;
    dlta_hdr->xpos = (xaULONG) frame_info->xpos;
    dlta_hdr->ypos = (xaULONG) frame_info->ypos;
    dlta_hdr->xsize = (xaULONG) frame_info->xsize;
    dlta_hdr->ysize = (xaULONG) frame_info->ysize;
    dlta_hdr->special = (xaULONG) frame_info->special;
    dlta_hdr->fsize = (xaULONG) frame_info->size;
    dlta_hdr->extra = 0; /* XXX -- this should actually be what
			    QT_Setup_Codec computes */
    dlta_hdr->fpos = -1;

#ifdef INSTRUMENTED
  if (instrumented) {
      tracks[frame_info->trackID]->total_frames++;
      total_frames++;
      timersub(&tv2, &tv1, &tvdiff);
      UpdateHisto(&fetch_time_hg, (double)tvdiff.tv_usec/1000000 +
		  tvdiff.tv_sec);
  }
#endif /* INSTRUMENTED */

  return 0;
}

/* find the number of (and fetch) the earliest frame whose zztime >= this */

static int frame_no = 0;
static XA_FRAME xa_frame;
static ACT_DLTA_HDR xa_dlta_hdr;
static XA_ACTION xa_act;

int QT_GetFrameNo(xaLONG zztime)
{
    int new_frame = -1;

#ifdef PSCOPE
    if (!pscope_qt_init) {
	pscope_qt_init = 1;
	if (pscope_profile) {
	    if (pscope_start (1, PSCOPE_INTR_LPT)) 
		perror ("pscope_start");
	}
    } /* endif */
#endif

    if (Fetch(zztime,&new_frame,&xa_frame,&xa_dlta_hdr,xa_vidcodec_buf) == 0) {
	xa_dlta_hdr.xapi_rev = xa_frame.act->xapi_rev;
	frame_no = new_frame;
    }
    else { /* we failed, if we timed out just return the last fetched frame */
	ASSERT((errno == EINVAL) || (errno == ETIMEDOUT));
	if (errno == EINVAL)
	    frame_no = -1;
	if (xa_verbose)
	    fprintf(stderr,"QT_GetFrameNo(%d): Fetch failed\n", zztime);
    }
    return frame_no;
}

/* fetch a frame by frame number */
XA_FRAME *QT_GetFrame(int cur_frame)
{

#ifdef PSCOPE
    if (!pscope_qt_init) {
	pscope_qt_init = 1;
	if (pscope_profile) {
	    if (pscope_start (1, PSCOPE_INTR_LPT)) 
		perror ("pscope_start");
	}
    } /* endif */
#endif
    if (cur_frame != frame_no) {
	frame_no = cur_frame;
	if (Fetch(0,&frame_no,&xa_frame,&xa_dlta_hdr,xa_vidcodec_buf) == 0)
	    xa_dlta_hdr.xapi_rev = xa_frame.act->xapi_rev;
	else {
	    if (xa_verbose)
		fprintf(stderr,"QT_GetFrame(%d): Fetch failed, returning frame %d\n",
			cur_frame, frame_no);
	}
    }
    /* XXX hack to prevent shared act from being clobbered */
    if (xa_frame.act) 
	xa_act = *(xa_frame.act);
    xa_frame.act = &xa_act;

    return &xa_frame;
}
