#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*

          Coda: an Experimental Distributed File System

       Copyright (c) 1996 Carnegie Mellon University
                      All Rights Reserved.

Permission  to use, copy, modify and distribute this software and
its documentation is hereby granted (including for commercial  or
for-profit use), provided that both the copyright notice and this
permission  notice  appear  in  all  copies  of   the   software,
derivative  works or modified versions, and any portions thereof,
and that both notices appear  in  supporting  documentation,  and
that  credit  is  given  to  Carnegie  Mellon  University  in all
publications reporting on direct or indirect use of this code  or
its derivatives.

CODA  IS  AN  EXPERIMENTAL  SOFTWARE PACKAGE AND IS KNOWN TO HAVE
BUGS, SOME OF WHICH MAY  HAVE  SERIOUS  CONSEQUENCES.    CARNEGIE
MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.
CARNEGIE MELLON DISCLAIMS ANY  LIABILITY  OF  ANY  KIND  FOR  ANY
DAMAGES  WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE
OF THIS SOFTWARE OR OF ANY DERIVATIVE WORK.

Carnegie Mellon encourages (but does not require) users  of  this
software to return any improvements or extensions that they make,
and to grant Carnegie Mellon the  rights  to  redistribute  these
changes  without  encumbrance.   

*/

/* Modified for use with Odyssey speech warden by J. Flinn 3/25/97 */

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <malloc.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/audioio.h>
#include "../speech_awedio.h"

#define BLOCK_FACTOR 10

int	debug = 0;
int	verbose = 0;
int	r_flag = 0;		/* open for record */
int	o_flag = 0;		/* send/rec packets */
char	*oname;
char	*freq = 0;		/* samples per second: < 48000 && > 4000 */
int	nfreq = 8000;
char	*chnl = 0;		/* channels: 1 or 2 */
int	nchnl = 1;
char	*prec = 0;		/* precision: 8 or 16 */
int	nprec = 8;
char	*enco = 0;		/* encoding: 0 thru 7 */
int	nenco = 1;
char	*blk = 0;		/* blksize: be careful */
int	nblk = 0;
char	*gain = 0;		/* audio gain: be careful */
int	ngain = 128;

void sighndlr();
struct sigaction sig =  { sighndlr, 0, 0 };

static int rec_on; /* Set to 1 while recording */

int record_audio (char* buffer, int len, check_fn fn, void* arg)
{
  int outputLen;
  int fd;
  struct audio_info info;
  
  fd = auopen("/dev/audio", 0);
  if (fd < 0) {
    return (0);
  } /* endif */

  /* It is not necessary to do a AUDIO_GETINFO before the SETINFO,
     iff you do an AUDIO_INITINFO() */
  audio_getinfo(fd, &info);
  AUDIO_INITINFO(&info);
  setparams_in(&info);
  audio_setinfo(fd, &info);

  sigaction(SIGINT,  &sig, 0);
  sigaction(SIGQUIT, &sig, 0);
  outputLen = audio_in(fd, &info, buffer, len, fn, arg);
  audio_getinfo(fd, &info);

  close (fd); 
  return (outputLen);
}

void play_audio (char* fname)
{
  int fd;
  struct audio_info info;

  fd = auopen("/dev/audio", 1);
  if (fd < 0) {
    return;
  } /* endif */

  /* It is not necessary to do a AUDIO_GETINFO before the SETINFO,
     iff you do an AUDIO_INITINFO() */
  audio_getinfo(fd, &info);
  AUDIO_INITINFO(&info);
  setparams_out(&info);
  audio_setinfo(fd, &info);

  audio_out (fd, &info, fname);
  close (fd);
}

/* Set parameters to the Janus defaults */
void setparams_in(struct audio_info *ip)
{
  struct audio_prinfo *xp;
  
  xp = &ip->record;
  ip->record.sample_rate = 16000;
  ip->play.sample_rate = 16000; /* Is this necessary? */
  xp->encoding = 3;
  xp->precision = 16;

   /* XXX because of limitations in the NetBSD 1.3 sound model,
          the record parameters are currently happily ignored.
          To "fix" this, we set the play parameters to be the same
          as the record parameters, after having been good up till 
          now. [-et Jun 4 1998] */
   ip->record = *xp;
   ip->play = *xp;
}

/* Set parameters to au standard */
void setparams_out(struct audio_info *ip)
{
  struct audio_prinfo *xp;
  
  xp = &ip->play;
  ip->record.sample_rate = 8000; 
  ip->play.sample_rate = 8000; 
  xp->encoding = 1;
  xp->precision = 8;
}

int auopen(char *fname, int mode)
{
  int fd;
  
  fd = open(fname, mode);
  if (fd < 0) {
    fprintf(stderr, "open %s failed, errno = %d\n", fname, errno);
  } /* endif */

  return (fd);
}

int audio_in(int fd, struct audio_info *ip, char* buffer, int len,
	     check_fn fn, void* arg)
{
  int blksize = ip->blocksize;
  int num = BLOCK_FACTOR;
  int i;
  int sz = num * blksize;
  int offset = 0;
  int rec_on = 1;

  if (fcntl(0, F_SETFL, O_NONBLOCK) < 0) {	/* peek at stdin */
    fprintf(stderr, "stdin O_NONBLOCK failed, errno = %d\n", errno);
    perror("fcntl");
    exit(1);
  }

  while (rec_on) {

    if (offset + sz > len) {
      fprintf (stderr, "Input buffer too small\n");
      return (0);
    } /* endif */

    i = read (fd, buffer+offset, sz);
    offset += i;
    if (i != sz) {
      if (errno != EINTR) {
	fprintf(stderr, "short read %d vs %d\n",
	       i/blksize, sz/blksize);
      } /* endif */
    } /* endif */

    /* Check if we're done */
    rec_on = (*fn) (arg);

  } /* endwhile */

  return (offset);
}

/* play back */
void audio_out(int fd, struct audio_info *ip, char *fname)
{
  FILE *ofd;
  int blksize = ip->blocksize;
  int num = BLOCK_FACTOR;
  char *buf;
  int i;
  
  ofd = fopen(fname, "r");
  if (ofd == NULL) {
    fprintf(stderr, "fopen %s failed, errno = %d\n",
	   fname, errno);
    perror("fopen");
    exit(1);
  }
  
  buf = (char *)malloc(num * blksize);
  
  fprintf (stderr, "Playing file\n");

  while (!feof(ofd)) {
    i = fread(buf, blksize, num, ofd);
    write(fd, buf, i * blksize);
  }
}

void sighndlr()
{
  rec_on = 0;
}

void audio_getinfo(int fd, struct audio_info *ip)
{
  int ret;
  
  if ((ret = ioctl(fd, AUDIO_GETINFO, ip)) < 0) {
    fprintf(stderr, "ioctl(%d, AUDIO_GETINFO, ... )failed, errno %d\n",
	   fd, errno);
    perror("ioctl");
  }
}

void audio_setinfo(int fd, struct audio_info *ip)
{
  int ret;
  
  if ((ret = ioctl(fd, AUDIO_SETINFO, ip)) < 0) {
    fprintf(stderr, "ioctl(%d, AUDIO_SETINFO, ... )failed, errno %d\n",
	   fd, errno);
    perror("ioctl");
    exit(1);
  } else if ((ret = ioctl(fd, AUDIO_GETINFO, ip)) < 0) {
    fprintf(stderr, "ioctl(%d, AUDIO_GETINFO, ... )failed, errno %d\n",
	   fd, errno);
    perror("ioctl");
  }
}

