#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <sys/uio.h>
#include "include/odyssey.h"
#ifdef MULTRES
#include <odyssey_multres.h>
#endif
#include "include/odytypes.h"
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "odyssey/wardens/speech/speechwarden.h"
#include "odyssey/wardens/speech/libodyspeech_tsops.h"
#include "speech_dialog.h"
#include "speech_client_main.h"
#include "speech_awedio.h"

/* Global data that will be accessed by the function that handles 
   Odyssey upcalls. */
StrategyModel strategyModel;
int localMinBandwidth, localMaxBandwidth;
int hybridMinBandwidth, hybridMaxBandwidth;
int remoteMinBandwidth, remoteMaxBandwidth;
int requestId = -1;
int fd;

enum strategyType currentStrategy = SPEECHW_STRATEGY_REMOTE;
int switchToLocal = 0;
int switchToRemote = 0;

extern int stop_recording;

#ifdef MULTRES

float localHybridEval (int num, ody_req_res_val_t* pValues)
{
  float localPerByteCost, hybridPerByteCost;
  float localCPU = 0.0, bandwidth = 0.0;
  int i;

  ASSERT (num == 2); /* Expect only one resource - CPU and bandwidth. */
  for (i = 0; i < num; i++) {
    if (pValues[i].orrv_resource == ODY_TEST_1) {
      localCPU = pValues[i].orrv_value / 10.0;
    } else if (pValues[i].orrv_resource == ODY_GR_NETBW) {
      bandwidth = pValues[i].orrv_value;
    } else {
      ASSERT (0); /* Unidentified resource was specified */
    } /* endif */
  } /* endfor */

#ifdef SUPER_DEBUG
  printf ("Local-Hybrid eval: CPU = %f, bandwidth = %f\n", localCPU, 
	  bandwidth);
#endif

  /* Handle bondary cases to avoid divide by zero */
  if (localCPU == 0) {
    return (-strategyModel.hybridRecPerByteCost * DEFAULT_REMOTE_CPU_RATIO
	    - 0.2 / bandwidth);
  } /* endif */
  if (bandwidth == 0) {
    return (- 999999999.0);
  } /* endif */

  /* Calculate cost of local strategy in seconds */
  /* local RPC time can be ignored since it is common to both cases */
  localPerByteCost = strategyModel.localRecPerByteCost / localCPU;

  /* Calculate cost of hybrid strategy in seconds */
  /* local RPC time can be ignored since it is common to both cases */
  hybridPerByteCost = strategyModel.hybridVQPerByteCost / localCPU
    + strategyModel.hybridRecPerByteCost * DEFAULT_REMOTE_CPU_RATIO
    + 0.2 / bandwidth;

#ifdef SUPER_DEBUG
  printf ("Local-Hybrid eval: result is: %f\n", 
	  localPerByteCost - hybridPerByteCost);
#endif
  
  /* Subtract the two values - positive value indicates that local is
     more expensive. */
  return (localPerByteCost - hybridPerByteCost);
}

float hybridRemoteEval (int num, ody_req_res_val_t* pValues)
{
  float hybridPerByteCost, remotePerByteCost;
  float localCPU = 0.0, bandwidth = 0.0;
  int i;

  ASSERT (num == 2); /* Expect only one resource - CPU and bandwidth. */
  for (i = 0; i < num; i++) {
    if (pValues[i].orrv_resource == ODY_TEST_1) {
      localCPU = pValues[i].orrv_value / 10.0;
    } else if (pValues[i].orrv_resource == ODY_GR_NETBW) {
      bandwidth = pValues[i].orrv_value;
    } else {
      ASSERT (0); /* Unidentified resource was specified */
    } /* endif */
  } /* endfor */

#ifdef SUPER_DEBUG
  printf ("Hybrid-Remote eval: CPU = %f, bandwidth = %f\n", localCPU, 
	  bandwidth);
#endif

  /* Handle bondary cases to avoid divide by zero */
  if (localCPU == 0 || bandwidth == 0) {
    return (-999999999.0);
  } /* endif */

  /* Calculate cost of hybrid strategy in seconds */
  /* local RPC time can be ignored since it is common to both cases */
  hybridPerByteCost = strategyModel.hybridVQPerByteCost / localCPU
    + 1.0 / LRPC_BANDWIDTH 
    + strategyModel.hybridRecPerByteCost * DEFAULT_REMOTE_CPU_RATIO
    + 0.2 / bandwidth;

  /* Calcualte cost of remote strategy in seconds */
  remotePerByteCost =  1.0 / bandwidth
    + strategyModel.remoteRecPerByteCost * DEFAULT_REMOTE_CPU_RATIO;

#ifdef SUPER_DEBUG
  printf ("Hybrid-Remote eval: result is: %f\n", 
	  hybridPerByteCost - remotePerByteCost);
#endif
  
  /* Subtract the two values - positive value indicates that local is
     more expensive. */
  return (hybridPerByteCost - remotePerByteCost);
}

void setResourceWindow (ody_req_res_val_t* presvals)
{
  ody_req_mres_t mres;
  ody_req_res_t resources[2];
  int done = 0
  enum strategyType previousStrategy = currentStrategy;

  do {
    
    if (localHybridEval (2, presvals) < 0) { 
      currentStrategy = SPEECHW_STRATEGY_LOCAL;
      printf ("Strategy is local\n");
    } else {
      if (hybridRemoteEval (2, presvals) < 0) {
	currentStrategy = SPEECHW_STRATEGY_HYBRID;
	printf ("Strategy is hybrid\n");
      } else {
	currentStrategy = SPEECHW_STRATEGY_REMOTE;
	printf ("Strategy is remote\n");
      } /* endif */
    } /* endif */
    speechSetStrategy (fd, currentStrategy);

    mres.orm_version.ov_gs = ODY_GENERIC_INTERFACE;
    mres.orm_version.ov_type = SPEECH_TTYPE;
    mres.orm_version.ov_tsvnum = SPEECHWARDEN_VERSION;
    mres.orm_resnum = 2;
    mres.orm_resources = resources;
    resources[0].orr_version = ODY_MULTRES_VERSION;
    resources[0].orr_resource = ODY_GR_NETBW;
    resources[1].orr_version = ODY_MULTRES_VERSION;
    resources[1].orr_resource = ODY_TEST_1;
    if (currentStrategy == SPEECHW_STRATEGY_LOCAL) {
      mres.orm_low_eval_fp = 0;
      mres.orm_high = 0.0; 
      mres.orm_high_eval_fp = localHybridEval;
    } else if (currentStrategy == SPEECHW_STRATEGY_HYBRID) {
      mres.orm_low_eval_fp = localHybridEval;
      mres.orm_low = 0.0; 
      mres.orm_high_eval_fp = hybridRemoteEval;
      mres.orm_high = 0.0;
    } else {
      mres.orm_low_eval_fp = hybridRemoteEval;
      mres.orm_low = 0.0; 
      mres.orm_high_eval_fp = 0;
    } /* endif */

    mres.orm_req_fp = resetWindow;

    FREE (presvals);

    if (ody_fmrequest(fd, &mres, &requestId, &presvals) == 0) {
      done = 1; /* Window set */
    } else {
      if (errno != ODY_ENOTINWINDOW) {
	perror("ody_frequest failed");
      } /* endif */
      printf ("Not in the window!\n");
    } /* endif */

  } while (!done);

  /* Set a flag so that local/remote switch will be announced.  Look
     for cases where a double-switch actually has no effect. */
  if (previousStrategy == SPEECHW_STRATEGY_LOCAL && 
      currentStrategy != SPEECHW_STRATEGY_LOCAL) {
    if (switchToLocal == 0) {
      switchToRemote = 1;
    } else {
      switchToLocal = 0;
    } /* endif */
  } else if (previousStrategy != SPEECHW_STRATEGY_LOCAL &&
	     currentStrategy == SPEECHW_STRATEGY_LOCAL) {
    if (switchToRemote == 0) {
      switchToLocal = 1;
    } else { 
      switchToRemote = 0;
    } /* endif */
  } /* endif */
}

void resetWindow (int reqid, int resnum, ody_req_res_val_t* pValues)
{
  printf ("Multi-resource notification received\n");

  if (reqid != requestId) {
    printf ("Error request id %d does not match stored value %d\n",
	    reqid, requestId);
  } /* endif */

  setResourceWindow (pValues);
}

int initStrategy (int fd, float localCPU, float remoteCPU, char* model)
{
  ody_req_res_val_t* presvals;
  int rc;

  /* Initialize the model and get back the performance parms */
  rc = speechInitModel (fd, model, &strategyModel);
  if (rc == 0) {
    printf ("Strategy Models:\n");
    if (strategyModel.localEnabled) {
      printf ("Local fidelty: %.1f\n", strategyModel.localFidelity);
      printf ("Local recognition per-byte cost: %f\n", 
	      strategyModel.localRecPerByteCost);
    } else {
      printf ("Local strategy not enabled\n");
    } /* endif */    
    if (strategyModel.hybridEnabled) {
      printf ("Hybrid fidelty: %.1f\n", strategyModel.hybridFidelity);
      printf ("Remote vector quantization per-byte cost: %f\n", 
	      strategyModel.hybridVQPerByteCost);
      printf ("Remote recognition per-byte cost: %f\n", 
	      strategyModel.hybridRecPerByteCost);
    } else {
      printf ("Hybrid strategy not enabled\n");
    } /* endif */
    if (strategyModel.remoteEnabled) {
      printf ("Remote fidelty: %.1f\n", strategyModel.remoteFidelity);
      printf ("Remote recognition per-byte cost: %f\n", 
	      strategyModel.remoteRecPerByteCost);
    } else {
      printf ("Remote strategy not enabled\n");
    } /* endif */
  } else {
    printf ("Unable to query strategies, rc = %d\n", rc);
  } /* endif */

  /* Make default bandwidth, cpu asumptions */
  NALLOC (presvals, ody_req_res_val_t, 2);
  presvals[0].orrv_version = ODY_MULTRES_VERSION;
  presvals[0].orrv_resource = ODY_GR_NETBW;
  presvals[0].orrv_value = DEFAULT_BANDWIDTH;
  presvals[1].orrv_version = ODY_MULTRES_VERSION;
  presvals[1].orrv_resource = ODY_TEST_1;
  presvals[1].orrv_value = localCPU;

  /* Set the multi-resource window */
  setResourceWindow (presvals);

  return (0);
}

#else

/* setBandwidthWindow is called to set the strategy for the speech 
   application.  It is called once when the program is started to 
   set the initial bandwidth, and then additionally every time the
   bandwidth goes outside of the window specified for the strategy.
   Whenever a new strategy is chosen, the function sets a new bandwidth
   window and changes the strategy through a speech warden TSOP. */
void setBandwidthWindow(int reqid, unsigned int rsrc, int val)
{
  struct ody_req_des or_des;
  int done = 0;
  enum strategyType previousStrategy = currentStrategy;

  if (reqid != requestId) {
    printf ("setBandwidthWindow received invalid requestId: %d\n", reqid);
    return;
  } /* endif */

  if (rsrc != ODY_GR_NETBW) {
    printf ("setBandiwidthWindow called with invalid resource: %d\n", rsrc);
    return;
  } /* endif */

  printf ("Setting new bandwidth window\n");

  do {
    
    or_des.ord_resource = ODY_GR_NETBW;
    or_des.ord_version.ov_gs = ODY_GENERIC_INTERFACE;
    or_des.ord_version.ov_type = SPEECH_TTYPE;
    or_des.ord_version.ov_tsvnum = SPEECHWARDEN_VERSION;
    or_des.ord_fp = setBandwidthWindow;

    if (strategyModel.localEnabled && val >= localMinBandwidth && 
	val <= localMaxBandwidth) {

      or_des.ord_low = localMinBandwidth;
      or_des.ord_high = localMaxBandwidth;
      currentStrategy = SPEECHW_STRATEGY_LOCAL;
      printf ("Strategy is local\n");

    } else if (strategyModel.hybridEnabled && val >= hybridMinBandwidth &&
	       val <= hybridMaxBandwidth) {

      or_des.ord_low = hybridMinBandwidth;
      or_des.ord_high = hybridMaxBandwidth;
      currentStrategy = SPEECHW_STRATEGY_HYBRID;
      printf ("Strategy is hybrid\n");

    } else if (strategyModel.remoteEnabled && val >= remoteMinBandwidth &&
	       val <= remoteMaxBandwidth) {

      or_des.ord_low = remoteMinBandwidth;
      or_des.ord_high = remoteMaxBandwidth;
      currentStrategy = SPEECHW_STRATEGY_REMOTE;
      printf ("Strategy is remote\n");

    } else {
      printf ("Bandwidth value %d does not fall within any strategy window\n",
	      val);
      return;
    } /* endif */

    speechSetStrategy (fd, currentStrategy);

    if (ody_frequest(fd, &or_des, &requestId) == 0) {
      done = 1;
    } else {
      if (errno != ODY_ENOTINWINDOW) {
	perror("ody_frequest failed");
      } else {
	val = requestId; /* New bandwidth value is returned in request Id */
      } /* endif */
    } /* endif */

  } while (!done);

  /* Set a flag so that local/remote switch will be announced.  Look
     for cases where a double-switch actually has no effect. */
  if (previousStrategy == SPEECHW_STRATEGY_LOCAL && 
      currentStrategy != SPEECHW_STRATEGY_LOCAL) {
    if (switchToLocal == 0) {
      switchToRemote = 1;
    } else {
      switchToLocal = 0;
    } /* endif */
  } else if (previousStrategy != SPEECHW_STRATEGY_LOCAL &&
	     currentStrategy == SPEECHW_STRATEGY_LOCAL) {
    if (switchToRemote == 0) {
      switchToLocal = 1;
    } else { 
      switchToRemote = 0;
    } /* endif */
  } /* endif */
}

int initStrategy (int fd, float localCPU, float remoteCPU, char* model)
{
  float localPerByteCost = 0.0, hybridPerByteCost= 0.0, remotePerByteCost= 0.0;
  float lhBandwidthBreakEven = 0.0;
  float lrBandwidthBreakEven = 0.0;
  float hrBandwidthBreakEven = 0.0;
  int rc;

  /* Query the available strategy models */
  rc = speechInitModel (fd, model, &strategyModel);
  if (rc == 0) {
    printf ("Strategy Models:\n");
    if (strategyModel.localEnabled) {
      printf ("Local fidelty: %.1f\n", strategyModel.localFidelity);
      printf ("Local recognition per-byte cost: %f\n", 
	      strategyModel.localRecPerByteCost);
    } else {
      printf ("Local strategy not enabled\n");
    } /* endif */    
    if (strategyModel.hybridEnabled) {
      printf ("Hybrid fidelty: %.1f\n", strategyModel.hybridFidelity);
      printf ("Remote vector quantization per-byte cost: %f\n", 
	      strategyModel.hybridVQPerByteCost);
      printf ("Remote recognition per-byte cost: %f\n", 
	      strategyModel.hybridRecPerByteCost);
    } else {
      printf ("Hybrid strategy not enabled\n");
    } /* endif */
    if (strategyModel.remoteEnabled) {
      printf ("Remote fidelty: %.1f\n", strategyModel.remoteFidelity);
      printf ("Remote recognition per-byte cost: %f\n", 
	      strategyModel.remoteRecPerByteCost);
    } else {
      printf ("Remote strategy not enabled\n");
    } /* endif */
  } else {
    printf ("Unable to query strategies, rc = %d\n", rc);
  } /* endif */

  /* Calculate fixed costs for each strategy */
  if (strategyModel.localEnabled) {
    localPerByteCost = strategyModel.localRecPerByteCost * localCPU
                     + 1.0 / LRPC_BANDWIDTH;
  } /* endif */

  if (strategyModel.hybridEnabled) {
    hybridPerByteCost = strategyModel.hybridVQPerByteCost * localCPU
                      + strategyModel.hybridRecPerByteCost * remoteCPU
                      + 1.0 / LRPC_BANDWIDTH;
  } /* endif */
    
  if (strategyModel.remoteEnabled) {
    remotePerByteCost = strategyModel.remoteRecPerByteCost * remoteCPU;
  } /* endif */

  /* Calculate breakeven points for each strategy in terms of bandwidth */
  if (strategyModel.localEnabled && strategyModel.hybridEnabled) {
    lhBandwidthBreakEven = 0.2 / (localPerByteCost - hybridPerByteCost);
    printf ("Local-Hybrid breakeven: %f\n", lhBandwidthBreakEven);
    if (lhBandwidthBreakEven < 0.0) {
      strategyModel.hybridEnabled = 0; 
      printf ("Hybrid strategy is always inferior to local\n");
    } /* endif */
  } /* endif */
  
  if (strategyModel.localEnabled && strategyModel.remoteEnabled) {
    lrBandwidthBreakEven = 1.0 / (localPerByteCost - remotePerByteCost);
    printf ("Local-Remote breakeven: %f\n", lrBandwidthBreakEven);
    if (lrBandwidthBreakEven < 0.0) {
      strategyModel.remoteEnabled = 0; 
      printf ("Remote strategy is always inferior to local\n");
    } /* endif */
  } /* endif */

  if (strategyModel.hybridEnabled && strategyModel.remoteEnabled) {
    hrBandwidthBreakEven = 0.8 / (hybridPerByteCost - remotePerByteCost);
    printf ("Hybrid-Remote breakeven: %f\n", hrBandwidthBreakEven);
    if (lrBandwidthBreakEven < 0.0) {
      strategyModel.remoteEnabled = 0; 
      printf ("Remote strategy is always inferior to local\n");
    } /* endif */
  } /* endif */

  /* Calcualte windows for each strategy in terms of bandwidth */
  if (strategyModel.localEnabled) {
    localMinBandwidth = 0;
    localMaxBandwidth = MAX_BANDWIDTH;
  } /* endif */

  if (strategyModel.hybridEnabled) {
    if (strategyModel.localEnabled) {
      localMaxBandwidth = lhBandwidthBreakEven;
      hybridMinBandwidth = lhBandwidthBreakEven+1;
    } else {
      hybridMinBandwidth = 0;
    } /* endif */
    hybridMaxBandwidth = MAX_BANDWIDTH;
  } /* endif */

  if (strategyModel.remoteEnabled) {
    if (strategyModel.hybridEnabled) {
      hybridMaxBandwidth = hrBandwidthBreakEven;
      remoteMinBandwidth = hrBandwidthBreakEven+1;
    } else if (strategyModel.localEnabled) {
      localMaxBandwidth = lrBandwidthBreakEven;
      remoteMinBandwidth = lrBandwidthBreakEven+1;
    } else {
      remoteMinBandwidth = 0;
    } /* endif */
    remoteMaxBandwidth = MAX_BANDWIDTH;
  } /* endif */

  printf ("Local bandwidth window is (%d,%d)\n", localMinBandwidth, 
	  localMaxBandwidth);
  printf ("Hybrid bandwidth window is (%d,%d)\n", hybridMinBandwidth, 
	  hybridMaxBandwidth);
  printf ("Remote bandwidth window is (%d,%d)\n", remoteMinBandwidth, 
	  remoteMaxBandwidth);

  if (strategyModel.hybridEnabled || strategyModel.remoteEnabled) {
    setBandwidthWindow (requestId, ODY_GR_NETBW, MAX_BANDWIDTH);
  } /* endif */

  return (0);
}

#endif

int checkForRelease (void* arg)
{
  XEvent event;
  XtAppContext* pAppContext = (XtAppContext *) arg;
  
  /* Dispatch X events that are pending - this allows user to interrupt
     the recording. */
  stop_recording = 0;
  while (XtAppPending (*pAppContext) && !stop_recording) {
    XtAppNextEvent (*pAppContext, &event);
    XtDispatchEvent (&event);
  } /* endwhile */

  return (!stop_recording);
}
 
int main (int argc, char* argv[]) 
{
  XtAppContext app_context;
  XEvent event;
  Widget topLevel, mainWindow, menu;
  ClientData* pClientData;
  float localCPU = DEFAULT_LOCAL_CPU_RATIO;
  float remoteCPU = DEFAULT_REMOTE_CPU_RATIO;
  char* model = DEFAULT_MODEL;
  int i = 0;
  char** p = 0;

  /* Parse arguments */
  while (i < argc) {
    if (!strcmp (argv[i], "-lcpu")) {
      localCPU = strtod (argv[i+1], p);
      i += 2;
    } else if (!strcmp (argv[i], "-rcpu")) {
      remoteCPU = strtod  (argv[i+1], p);
      i += 2;
    } else if (!strcmp (argv[i], "-m")) {
      model = argv[i+1];
      i+= 2;
    } else {
      i++;
    } /* endif */
  } /* endwhile */

  /* Open a file in the speech warden's name space*/
  fd = open ("/ody/speech/xclient", O_RDWR);
  if (fd < 0) {
    printf ("Unable to open /ody/speech/... file\n");
    printf ("Perhaps the Viceroy is not started\n");
    return (-1);
  } /* endif */

  initStrategy (fd, localCPU, remoteCPU, model);
    
  XtSetLanguageProc (NULL, (XtLanguageProc)NULL, NULL);

  topLevel = XtVaAppInitialize (&app_context, "xspeech", NULL, 0, 
				&argc, argv, NULL, NULL);

  mainWindow = XtVaCreateManagedWidget ("mainWindow", xmMainWindowWidgetClass,
					topLevel, NULL);

  /* Create the speech dialog */
  pClientData = Create_Speech_Frame (mainWindow, &app_context, fd, model);
  menu = Create_Speech_Menu (pClientData);

  XmMainWindowSetAreas (mainWindow, menu, NULL, NULL, NULL, 
			pClientData->frame);

  /* Process X events until user exits */
  XtRealizeWidget (topLevel);
  while (1) {
    XtAppNextEvent (app_context, &event);
    XtDispatchEvent (&event);

    if (switchToLocal) {
      play_audio ("/usr/odyssey/speech/disconn.au");
      switchToLocal = 0;
    } /* endif */
    if (switchToRemote) {
      play_audio ("/usr/odyssey/speech/reconn.au");
      switchToRemote = 0;
    } /* endif */

  } /* endwhile */

  return (0);
}
