#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <sys/uio.h>
#include "include/odyssey.h"
#include "include/odytypes.h"

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/SeparatoG.h>
#include <Xm/MessageB.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "odyssey/wardens/speech/libodyspeech_tsops.h"
#ifdef MULTRES
#include <odyssey_multres.h>
#endif
#include "speech_dialog.h"
#if 0
#include "speech_effort_dialog.h"
#endif
#include "speech_awedio.h"
#include "speech_client_main.h"

extern int switchToLocal;
extern int switchToRemote;
extern int requestId;

int stop_recording = 0;

/* readBytes reads the specified number of bytes from a descriptor.
   It keeps reading from until the number of bytes specified has been
   read or an error occurs.  It returns the number of bytes read on
   success, or the value returned by read on an error */
int readBytes (int fd, char* buffer, int bytesToRead)
{
  int bytesRead = 0;
  int len;

  do {
    len = read (fd, &buffer[bytesRead], bytesToRead - bytesRead);
    if (len > 0) {
      bytesRead += len;

#ifdef ODYDEBUG
      printf ("Read %d bytes on descriptor %d\n", len, fd);
#endif

    } else if (len == 0) {
      bytesRead = 0;
    } else {
      perror ("Error reading from socket");
      bytesRead = -1;
    } /* endif */
  } while (bytesRead > 0 && bytesRead < bytesToRead);

  return (bytesRead);
}

/* recognizeFile() recognizes the contents of a file denoted by 
   the specified file descriptor. */
int recognizeFile (int utt_fd, int ody_fd, char* model, 
		   char* translation, int len)
{
  struct stat fs;
  char* pBuffer = 0;
  int bytesRead;
  int totalBytesRead = 0;
  int rc = 0; /* Assume success */

  if (fstat (utt_fd, &fs) == 0) {

    /* Allocate buffer to contain file data */
    pBuffer = malloc (fs.st_size);
    assert (pBuffer);

    do {
      
      /* Read data into the buffer */
      bytesRead = read (utt_fd, &pBuffer[totalBytesRead], fs.st_size);
      if (bytesRead > 0) {
	totalBytesRead += bytesRead;
      } else {
	perror ("Error reading data file\n");
	rc = -1;
      } /* endif */
      
    } while (totalBytesRead < fs.st_size && rc == 0);

    /* Recognize the contents of the buffer */
    rc = speechRecognize (ody_fd, model, pBuffer, totalBytesRead, 
			  translation, len);
    
    free (pBuffer);

  } else {
    perror ("Error getting file statistics");
  } /* endif */

  return (rc);
}

/* This function is called when the user selects the Recognize push
   button on the dialog.  The first call starts recording and changes the push 
   button text to read Stop Recording. The second call stops recording and
   changes the push button text to read Recognizing. */
void Recognize_Callback (Widget w, XtPointer client_data, XtPointer call_data)
{
  ClientData* pClientData;
  char translation[1024];
  char audiobuf[1024*1024];
  XmString xmstr;
  Arg args[1];
  int len;
  int fd;
  int rc;

  pClientData = (ClientData *) client_data;  

  if (pClientData->input == USE_AUDIO) {

    if (pClientData->recording) {
      
      pClientData->recording = 0;
      stop_recording = 1;

      /* change the push button title */
      xmstr = XmStringCreateLtoR("Start Recognition", 
				 XmSTRING_DEFAULT_CHARSET);
      XtSetArg(args[0], XmNlabelString, xmstr);
      XtSetValues(pClientData->button, args, 1);
      XmStringFree(xmstr);      
      
    } else {
      
      pClientData->recording = 1;
      
      /* Blank out dialog box */
      XtSetArg(args[0], XmNvalue, "");
      XtSetValues (pClientData->text, args, 1);
      XSync (XtDisplay(pClientData->text), False);

      /* change the push button title */
      xmstr = XmStringCreateLtoR("Stop Recording", XmSTRING_DEFAULT_CHARSET);
      XtSetArg(args[0], XmNlabelString, xmstr);
      XtSetValues(pClientData->button, args, 1);
      XmStringFree(xmstr);      
      
      len = record_audio (audiobuf, sizeof(audiobuf), 
			  checkForRelease, pClientData->pAppContext);
      if (len) {
	rc = speechRecognize (pClientData->fd, pClientData->model, audiobuf, 
			      len, translation, sizeof(translation));
	if (rc == 0) {
	  XtSetArg(args[0], XmNvalue, translation);
	  XtSetValues(pClientData->text, args, 1);
	} else {
	  printf ("Error recognizing utterance, rc = %d\n", rc);
	} /* endif */
      } /* endif */

    } /* endif */

    if (switchToLocal) {
      play_audio ("/usr/odyssey/speech/disconn.au");
      switchToLocal = 0;
    } /* endif */
    if (switchToRemote) {
      play_audio ("/usr/odyssey/speech/reconn.au");
      switchToRemote = 0;
    } /* endif */

  } else {
    
    /* Blank out dialog box */
    XtSetArg(args[0], XmNvalue, "");
    XtSetValues (pClientData->text, args, 1);
    XSync (XtDisplay(pClientData->text), False);

    /* Copy data from the file to the Odyssey descriptor */
    fd = open (pClientData->inputFilename, O_RDONLY);
    if (fd >= 0) {
      rc = recognizeFile (fd, pClientData->fd, pClientData->model, 
			  translation, sizeof(translation));
      if (rc == 0) {
	XtSetArg(args[0], XmNvalue, translation);
	XtSetValues(pClientData->text, args, 1);
      } else {
	printf ("Error recognizing utterance, rc = %d\n", rc);
      } /* endif */
      close (fd);
    } else {
      perror ("Error opening input file");
    } /* endif */

  } /* endif */
}

/* Config_Local_Callback is called when the user selects the Local
   menu-item from the Configuration menu.  It tells the warden to
   use the local strategy */
void Config_Local_Callback (Widget w, XtPointer client_data, 
			    XtPointer call_data)
{
  XmToggleButtonCallbackStruct* pCallbackData;
  ClientData* pClientData;
  int rc;

  pClientData = (ClientData *) client_data;  
  pCallbackData = (XmToggleButtonCallbackStruct *) call_data;

  if (pCallbackData->set) {

    /* Cancel the resource request if there is one in progress */
    if (requestId >= 0) {
#ifdef MULTRES
      ody_mcancel (requestId);
#else 
      ody_cancel (requestId);
#endif
      requestId = -1;
    } /* endif */

    rc = speechSetStrategy (pClientData->fd, SPEECHW_STRATEGY_LOCAL); 
    if (rc) {
      printf ("Unable to set configuration to local, rc = %d\n", rc);
    } /* endif */

  } /* endif */
}

/* Config_Hybrid_Callback is called when the user selects the Hybrid
   menu-item from the Configuration menu.  It tells the warden to use
   the hybrid strategy. */
void Config_Hybrid_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  XmToggleButtonCallbackStruct* pCallbackData;
  ClientData* pClientData;
  int rc;

  pClientData = (ClientData *) client_data;  
  pCallbackData = (XmToggleButtonCallbackStruct *) call_data;

  if (pCallbackData->set) {

    /* Cancel the resource request if there is one in progress */
    if (requestId >= 0) {
#ifdef MULTRES
      ody_mcancel (requestId);
#else
      ody_cancel (requestId);
#endif
      requestId = -1;
    } /* endif */

    rc = speechSetStrategy (pClientData->fd, SPEECHW_STRATEGY_HYBRID); 
    if (rc) {
      printf ("Unable to set configuration to hybrid, rc = %d\n", rc);
    } /* endif */

  } /* endif */
}

/* Config_Remote_Callback is called when the user selects the Remote
   menu-item from the Configuration menu.  It tells the warden to use
   the remote strategy */
void Config_Remote_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  XmToggleButtonCallbackStruct* pCallbackData;
  ClientData* pClientData;
  int rc;

  pClientData = (ClientData *) client_data;  
  pCallbackData = (XmToggleButtonCallbackStruct *) call_data;

  if (pCallbackData->set) {

    /* Cancel the resource request if there is one in progress */
    if (requestId >= 0) {
#ifdef MULTRES
      ody_mcancel (requestId);
#else
      ody_cancel (requestId);
#endif
      requestId = -1;
    } /* endif */

    rc = speechSetStrategy (pClientData->fd, SPEECHW_STRATEGY_REMOTE); 
    if (rc) {
      printf ("Unable to set configuration to remote, rc = %d\n", rc);
    } /* endif */

  } /* endif */
}

/* Config_Odyssey_Callback is called when the user selects the Odyssey
   menu-item from the Configuration menu.  It switches the warden to use
   bandwidth estimation to determine the best connection strategy. */
void Config_Odyssey_Callback (Widget w, XtPointer client_data, 
			      XtPointer call_data)
{
  XmToggleButtonCallbackStruct* pCallbackData;
#ifdef MULTRES
  ody_req_res_val_t* presvals;
#endif

  pCallbackData = (XmToggleButtonCallbackStruct *) call_data;

  if (pCallbackData->set) {
    
    /* If there is no request in progress, then register a request.  Assume
       that the connection is good. */
    if (requestId < 0) {
#ifdef MULTRES
      NALLOC (presvals, ody_req_res_val_t, 2);
      presvals[0].orrv_version = ODY_MULTRES_VERSION;
      presvals[0].orrv_resource = ODY_GR_NETBW;
      presvals[0].orrv_value = DEFAULT_BANDWIDTH;
      presvals[1].orrv_version = ODY_MULTRES_VERSION;
      presvals[1].orrv_resource = ODY_TEST_1;
      presvals[1].orrv_value = DEFAULT_LOCAL_CPU_RATIO;
      
      /* Set the multi-resource window */
      setResourceWindow (presvals);
#else      
      setBandwidthWindow (requestId, ODY_GR_NETBW, MAX_BANDWIDTH); 
#endif
    } /* endif */

  } /* endif */
}

/* Input_Audio_Callback is called when the user selects the audio menu-item
   from the input menu.  It switches the input to be from audio */
void Input_Audio_Callback (Widget w, XtPointer client_data, 
			   XtPointer call_data)
{
  ClientData* pClientData;
  XmString xmstr;
  Arg args[1];

  pClientData = (ClientData *) client_data;  

  pClientData->input = USE_AUDIO;
  if (pClientData->inputFilename) {
    free (pClientData->inputFilename);
    pClientData->inputFilename = 0;
  } /* endif */

  /* Change input back to "Audio" */
  xmstr = XmStringCreateLtoR("Audio", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (args[0], XmNlabelString, xmstr);
  XtSetValues (pClientData->inputLabel, args, 1);

  XmStringFree (xmstr);
}

/* Input_File_Callback is called when the user selects the file menu-item
   from the input menu.  It displays a dialog and allows the user to select
   from which file input will be taken. */
void Input_File_Callback (Widget w, XtPointer client_data, 
			   XtPointer call_data)
{
  ClientData* pClientData;
  Widget help, ok, cancel;
  XmString xmstr;
  Arg args[2];

  pClientData = (ClientData *) client_data;  

  if (!pClientData->currentPopup) {

    /* Create a file selection dialog */
    xmstr = XmStringCreateLtoR("*.adc", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNpattern, xmstr);
    XtSetArg(args[1], XmNtitle, "Audio input file");
    pClientData->currentPopup = 
      XmCreateFileSelectionDialog (pClientData->frame, "Audio Input File",
				   args, 2);
    /* Remove help push button */
    help = XmFileSelectionBoxGetChild (pClientData->currentPopup, 
				       XmDIALOG_HELP_BUTTON);
    XtUnmanageChild (help);
    
    /* Add callbacks */
    ok = XmFileSelectionBoxGetChild (pClientData->currentPopup, 
				     XmDIALOG_OK_BUTTON);
    cancel = XmFileSelectionBoxGetChild (pClientData->currentPopup, 
					 XmDIALOG_CANCEL_BUTTON);
    XtAddCallback(ok, XmNactivateCallback, File_OK_Callback, 
		  pClientData);
    XtAddCallback(cancel, XmNactivateCallback, File_Cancel_Callback, 
		  pClientData);
  
    XmStringFree (xmstr);

  } /* endif */

  XtManageChild (pClientData->currentPopup);
}

void File_OK_Callback (Widget w, XtPointer client_data, 
		       XtPointer call_data)
{
  ClientData* pClientData;
  Arg args[1];
  XmString xmstr;

  pClientData = (ClientData *) client_data;  

  /* Query the file selected */
  XtSetArg (args[0], XmNdirSpec, &xmstr);
  XtGetValues (pClientData->currentPopup, args, 1);
  
  pClientData->input = USE_FILE;
  if (pClientData->inputFilename) {
    free (pClientData->inputFilename);
    pClientData->inputFilename = 0;
  } /* endif */
  XmStringGetLtoR (xmstr, XmFONTLIST_DEFAULT_TAG, &pClientData->inputFilename);
  
  /* Update the input label */
  XtSetArg (args[0], XmNlabelString, xmstr);
  XtSetValues (pClientData->inputLabel, args, 1);

  XmStringFree (xmstr);

  XtUnmanageChild (pClientData->currentPopup);
}

/* File_Cancel_Callback is called when the cancel button from the file
   selection dialog.  It unmanages the dialog. */
void File_Cancel_Callback (Widget w, XtPointer client_data, 
			   XtPointer call_data)
{
  ClientData* pClientData;

  pClientData = (ClientData *) client_data;  

  XtUnmanageChild (pClientData->currentPopup);
}

/* Options_Effort_Callback is called when the search effort button from the 
   options dialog.  The function pops up a dialog which allows the user
   to select the search effort. */
void Options_Effort_Callback (Widget w, XtPointer client_data, 
			      XtPointer call_data)
{
#if 0
  ClientData* pClientData;

  pClientData = (ClientData *) client_data;  

  Create_Effort_Dialog (pClientData);
#endif
}

/* Stats_Callback() is called when the user selects the Display stats push
   button on the dialog.  For now, it just causes the speech warden to
   dump statistics to a file.  */
void Stats_Callback (Widget w, XtPointer client_data, XtPointer call_data)
{
  ClientData* pClientData;
  Widget messageBox, help, cancel;
  char buffer[2048];
  char* pch;
  int fd;
  Arg args[2];
  XmString xmstr;

  pClientData = (ClientData *) client_data;  

  /* Clear out the file */
  fd = creat (STATS_FILE, 0777);
  if (fd >= 0) {
    close (fd);
  } else {
    perror ("Unable to open create/truncate file");
  } /* endif */
  
  /* Dump statistics to the file */
  dumpStats (pClientData->fd, STATS_FILE);

  /* Read statistics into buffer */
  fd = open (STATS_FILE, O_RDONLY);
  if (fd >= 0) {

    memset (buffer, 0, sizeof (buffer));
    if (readBytes (fd, buffer, sizeof(buffer)-1) >= 0) {

      /* Eliminate EOF character and first few uninformative output lines */
      buffer[strlen(buffer)-1] = '\0';
      pch = strstr (buffer, "Total bytes read");
      if (!pch) {
	pch = buffer; /* D'oh ? */
      } /* endif */

      /* Display statistics in a message dialog */
      xmstr = XmStringCreateLtoR(pch, XmSTRING_DEFAULT_CHARSET);

      XtSetArg(args[0], XmNmessageString, (XtArgVal) xmstr);
      XtSetArg(args[1], XmNtitle, "Statistics for Last Recognition");
      
      messageBox = XmCreateMessageDialog (pClientData->frame, 
					  "message box",
					  args, 2);
      XmStringFree (xmstr);

      /* Remove help ande cancel push buttons */
      help = XmMessageBoxGetChild (messageBox, XmDIALOG_HELP_BUTTON);
      cancel = XmMessageBoxGetChild (messageBox, XmDIALOG_CANCEL_BUTTON);
      XtUnmanageChild (help);
      XtUnmanageChild (cancel);

      XtManageChild (messageBox);

    } else {
      printf ("Statistics file contains no data");
    } /* endif */

    close (fd);

  } else {
    perror ("Unable to open stats file to read");
  } /* endif */
}

/* This function is called when the user selects the Cancel push
   button on the dialog.  It halts recognition by closing the open
   Odyssey file handle, closes the dialog, and exits. */
void Cancel_Callback (Widget w, XtPointer client_data, XtPointer call_data)
{
  ClientData* pClientData;

  /* Close the speech model */
  pClientData = (ClientData *) client_data;  
  speechCloseModel (pClientData->fd, pClientData->model);

  /* Then, close the open Odyssey file descriptor */
  close (pClientData->fd);

  exit (0); /* We're done */
}

/* This function creates a dialog which alows the user to interface with
   the speech warden. */
ClientData* Create_Speech_Frame (Widget parent, XtAppContext* pAppContext, 
				 int fd, char* model)
{
  Widget dialog_frame, form, dialog_sep, buttons_form;
  Widget inputText, inputLabel, effortText, effortLabel;
  Widget label, text, recognizeButton, statsButton, cancelButton;
  ClientData* pClientData; 
  Arg args[12];
 
  /* Create a new client data structure for this dialog */
  pClientData = (ClientData *) malloc (sizeof(ClientData));

  /* Create a frame for the speech dialog */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  dialog_frame = XtCreateManagedWidget ("frame", xmFrameWidgetClass,
					parent, args, 5);
      
  /* Create the main form for the dialog */
  form = XtCreateManagedWidget ("form", xmFormWidgetClass, dialog_frame,
				NULL, 0);

  /* Create labels for the input source */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  inputText = XtCreateManagedWidget ("Input:", xmLabelWidgetClass,
				     form, args, 4);

  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 2);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[4], XmNleftWidget, (XtArgVal) inputText);
  inputLabel = XtCreateManagedWidget ("Audio", xmLabelWidgetClass,
				      form, args, 5);   

  /* Create labels for the search effort */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNtopWidget, (XtArgVal) inputText);
  effortText = XtCreateManagedWidget ("Search effort:", xmLabelWidgetClass,
				      form, args, 5);

  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 2);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[4], XmNleftWidget, (XtArgVal) effortText);
  XtSetArg(args[5], XmNtopWidget, (XtArgVal) inputLabel);
  effortLabel = XtCreateManagedWidget ("1.00", xmLabelWidgetClass,
				       form, args, 6);   
  
  /* Create a label for the entry field */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNtopWidget, (XtArgVal) effortText);
  label = XtCreateManagedWidget ("Recognized Text: ", xmLabelWidgetClass,
                                 form, args, 5); 

  /* Create a read-only text field in which recognitions will be displayed */
  XtSetArg(args[0], XmNcolumns, (XtArgVal) 80);
  XtSetArg(args[1], XmNrows, (XtArgVal) 10);
  XtSetArg(args[2], XmNscrollVertical, (XtArgVal) True);
  XtSetArg(args[3], XmNtopOffset, (XtArgVal) 5);
  XtSetArg(args[4], XmNleftOffset, (XtArgVal) 10);
  XtSetArg(args[5], XmNrightOffset, (XtArgVal) 10);
  XtSetArg(args[6], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[7], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[8], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[9], XmNtopWidget, (XtArgVal) label);
  XtSetArg(args[10], XmNeditable, 0); /* Don't allow user to edit this field */
  XtSetArg(args[11], XmNeditMode, XmMULTI_LINE_EDIT);
  text = XmCreateText (form, "textfield", args, 12);
  XtManageChild (text);

  /* Create a separator between entry field and push buttons */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNtopWidget, (XtArgVal) text);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  dialog_sep = XmCreateSeparatorGadget (form, "separator", args, 5);
  XtManageChild (dialog_sep);

  /* Create form for push buttons */
  XtSetArg(args[0], XmNverticalSpacing, (XtArgVal) 8);
  XtSetArg(args[1], XmNfractionBase, (XtArgVal) 4);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNtopWidget, (XtArgVal) dialog_sep);
  XtSetArg(args[7], XmNtopOffset, (XtArgVal) 10);
  buttons_form = XtCreateManagedWidget ("form", xmFormWidgetClass, form,
					args, 8);

  /* Create recognize button */
  XtSetArg(args[0], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[1], XmNleftPosition, 0);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 8);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNrightPosition, 1);
  XtSetArg(args[5], XmNrightOffset, (XtArgVal) 4);
  XtSetArg(args[6], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNtopOffset, 4);
  XtSetArg(args[8], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[9], XmNbottomOffset, 4);
  recognizeButton = XtCreateManagedWidget ("Start recognition", 
					   xmPushButtonWidgetClass,
					   buttons_form, args, 10);
  XtAddCallback(recognizeButton, XmNactivateCallback, Recognize_Callback, 
		pClientData);
    
  /* Create and place the stats button */
  XtSetArg(args[0], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[1], XmNleftPosition, 1);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNrightPosition, 2);
  XtSetArg(args[5], XmNrightOffset, (XtArgVal) 4);
  XtSetArg(args[6], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNtopOffset, 4);
  XtSetArg(args[8], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[9], XmNbottomOffset, 4);
  statsButton = XtCreateManagedWidget ("Display stats", 
				       xmPushButtonWidgetClass,
				       buttons_form, args, 8);    
  XtAddCallback(statsButton, XmNactivateCallback, Stats_Callback, 
		pClientData);
    
  /* Create and place the cancel button */
  XtSetArg(args[0], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[1], XmNleftPosition, 2);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNrightPosition, 3);
  XtSetArg(args[5], XmNrightOffset, (XtArgVal) 8);
  XtSetArg(args[6], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNtopOffset, 4);
  XtSetArg(args[8], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[9], XmNbottomOffset, 4);
  cancelButton = XtCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
					buttons_form, args, 8);    
  XtAddCallback(cancelButton, XmNactivateCallback, Cancel_Callback, 
		pClientData);
    
  /* Populate Client Data structure */
  pClientData->pAppContext = pAppContext;
  pClientData->parent = parent;
  pClientData->frame = dialog_frame;
  pClientData->text = text;
  pClientData->button = recognizeButton;
  pClientData->fd = fd;
  pClientData->model = model;
  pClientData->recording = 0;
  pClientData->input = USE_AUDIO;
  pClientData->inputFilename = 0;
  pClientData->inputLabel = inputLabel;
  pClientData->effortLabel = effortLabel;
  pClientData->effortText = effortText;
  pClientData->currentPopup = 0;
  pClientData->searchEffort = 1.0;

  return (pClientData);
}

/* Create_Speech_Menu creates the menu-bar for the speech client program */
Widget Create_Speech_Menu (ClientData* pClientData) 
{
  Widget menuBar, configButton, closeButton, configMenu;
  Widget inputButton, inputMenu, audioButton, fileButton;
  Widget localButton, hybridButton, remoteButton, odysseyButton;
  Widget optionsButton, optionsMenu, effortButton;
  Arg args[1];

  /* Create menu bar as child of top-level window */
  menuBar = XmCreateMenuBar (pClientData->parent, "menuBar", NULL, 0);
  XtManageChild (menuBar);

  /* Create input menu */
  inputButton = XtVaCreateManagedWidget ("Input", 
					  xmCascadeButtonWidgetClass, 
					  menuBar, NULL);
  inputMenu = XmCreatePulldownMenu (menuBar, "inputMenu", args, 1);
  audioButton = XtVaCreateManagedWidget ("Audio", xmPushButtonWidgetClass, 
					 inputMenu, NULL);
  XtAddCallback (audioButton, XmNactivateCallback, Input_Audio_Callback, 
		 pClientData);
  fileButton = XtVaCreateManagedWidget ("File...", xmPushButtonWidgetClass, 
					inputMenu, NULL);
  XtAddCallback (fileButton, XmNactivateCallback, Input_File_Callback,
		 pClientData);

  /* Create configuration menu */
  configButton = XtVaCreateManagedWidget ("Configuration", 
					  xmCascadeButtonWidgetClass, 
					  menuBar, NULL);
  XtSetArg (args[0], XmNradioBehavior, True);
  configMenu = XmCreatePulldownMenu (menuBar, "configMenu", args, 1);
  localButton = XtVaCreateManagedWidget ("Local", xmToggleButtonWidgetClass, 
					 configMenu, NULL);
  XtAddCallback (localButton, XmNvalueChangedCallback, Config_Local_Callback, 
		 pClientData);
  hybridButton = XtVaCreateManagedWidget ("Hybrid", xmToggleButtonWidgetClass, 
					  configMenu, NULL);
  XtAddCallback (hybridButton, XmNvalueChangedCallback, Config_Hybrid_Callback,
		 pClientData);
  remoteButton = XtVaCreateManagedWidget ("Remote", xmToggleButtonWidgetClass, 
					  configMenu, NULL);
  XtAddCallback (remoteButton, XmNvalueChangedCallback, Config_Remote_Callback,
		 pClientData);
  odysseyButton = XtVaCreateManagedWidget ("Odyssey",
					   xmToggleButtonWidgetClass, 
					   configMenu, XmNset, True, NULL);
  XtAddCallback (odysseyButton, XmNvalueChangedCallback, 
		 Config_Odyssey_Callback, pClientData);

  /* Create options menu */
  optionsButton = XtVaCreateManagedWidget ("Options", 
					  xmCascadeButtonWidgetClass, 
					  menuBar, NULL);
  optionsMenu = XmCreatePulldownMenu (menuBar, "optionsMenu", args, 1);
  effortButton = XtVaCreateManagedWidget ("Set search effort...", 
					  xmPushButtonWidgetClass, 
					  optionsMenu, NULL);
  XtAddCallback (effortButton, XmNactivateCallback, Options_Effort_Callback, 
		 pClientData);

  closeButton = XtVaCreateManagedWidget ("Quit", xmCascadeButtonWidgetClass, 
					 menuBar, NULL);
  XtAddCallback (closeButton, XmNactivateCallback, Cancel_Callback, 
		 pClientData);

  /* Attach menus to buttons on menu-bar */
  XtVaSetValues (configButton, XmNsubMenuId, configMenu, NULL);
  XtVaSetValues (inputButton, XmNsubMenuId, inputMenu, NULL);
  XtVaSetValues (optionsButton, XmNsubMenuId, optionsMenu, NULL);

  return (menuBar);
}






