#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <Xm/Xm.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/SeparatoG.h>
#include <Xm/MessageB.h>
#include <libodyspeech_tsops.h>
#include "speech_dialog.h"
#include "speech_effort_dialog.h"

/* Effort_OK_Callback() is called when the user selects the OK push button 
   on the dialog.  The function validate that the entered value is correct,
   and if so destroys the dialog and changes the search effort value. */
void Effort_OK_Callback (Widget w, XtPointer client_data, 
			 XtPointer call_data)
{
  char* strEffort;
  float effort;
  Widget dialog;
  XmString title, message, xmstr;
  Arg args[2];
  ClientData* pClientData;
  int rc;
  
  pClientData = (ClientData *) client_data;

  /* Validate the entered value for search effort */
  strEffort = XmTextGetString (pClientData->effortText);
  effort = atof (strEffort);
  if (effort > 0.0 && effort <= 10.0) {

    /* Save new value */
    pClientData->searchEffort = effort;

    /* Change label to reflect the new value */
    xmstr = XmStringCreateLtoR(strEffort, XmSTRING_DEFAULT_CHARSET);
    XtSetArg (args[0], XmNlabelString, xmstr);
    XtSetValues (pClientData->effortLabel, args, 1);
    XmStringFree (xmstr);

    /* Send a request to the warden to change the value */
    rc = speechSetSearchEffort (pClientData->fd, pClientData->searchEffort);
    if (rc) {
      printf ("Unable to set search effort, rc = %d\n", rc);
    } /* endif */

    /* Close the dialog */
    Effort_Cancel_Callback (w, client_data, call_data);
    
  } else {

    /* Display an error that value entered was incorrect */
    title = XmStringCreateLtoR ("Incorrect value Entered", 
				XmSTRING_DEFAULT_CHARSET);
    message = XmStringCreateLtoR (
	       "You must enter a value between 0 and 10.", 
	       XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNdialogTitle, (XtArgVal) title);
    XtSetArg(args[1], XmNmessageString, (XtArgVal) message);
    dialog = XmCreateErrorDialog (w, "dialog", args, 2);
    XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));  
    XmStringFree (title);
    XmStringFree (message);
    XtManageChild (dialog);

  } /* endif */
}

/* This function is called when the user selects the Cancel push button on the
   dialog.  It closes the dialog. */
void Effort_Cancel_Callback (Widget w, XtPointer client_data, 
			     XtPointer call_data)
{
  ClientData* pClientData;

  pClientData = (ClientData *) client_data;
  XtDestroyWidget (pClientData->dialog);
}

/* Create_Effort_Dialog() is called when the user selects the effort 
   menu-item on the options menu.  It displays a dialog that allows the 
   user to specify the search effort. */
void Create_Effort_Dialog (ClientData* pClientData)
{
  Widget dialog_frame, form, label, dialog_sep;
  Widget buttons_form, okButton, cancelButton;
  XmString xmstr;
  Arg args[9];
  char strEffort[256];

  /* Create dialog box */
  xmstr = XmStringCreateLtoR ("Search Effort", XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0], XmNautoUnmanage, (XtArgVal) False);
  XtSetArg(args[1], XmNdialogTitle, (XtArgVal) xmstr);
  pClientData->dialog = XmCreateFormDialog (pClientData->frame, 
					    "search dialog", args, 2);
  XmStringFree (xmstr);

  /* Create a frame for the dialog window */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  dialog_frame = XtCreateManagedWidget ("frame", xmFrameWidgetClass,
					pClientData->dialog, args, 5);

  /* Create the main form for the dialog */
  form = XtCreateManagedWidget ("form", xmFormWidgetClass, dialog_frame,
				NULL, 0);
       
  /* Create a label for the entry field */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 14);
  XtSetArg(args[1], XmNleftOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  label = XtCreateManagedWidget ("Search effort: ", xmLabelWidgetClass,
                                 form, args, 4);  

  /* Create an entry field */
  XtSetArg(args[0], XmNwidth, (XtArgVal) 310);
  XtSetArg(args[1], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[2], XmNleftOffset, (XtArgVal) 5);
  XtSetArg(args[3], XmNrightOffset, (XtArgVal) 10);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[6], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNleftWidget, (XtArgVal) label);
  sprintf (strEffort, "%.2f", pClientData->searchEffort);
  XtSetArg(args[8], XmNvalue, (XtArgVal) strEffort);
  pClientData->effortText = XmCreateTextField (form, "text", args, 9);
  XtManageChild (pClientData->effortText);
  XtAddCallback (pClientData->effortText, XmNactivateCallback, 
		 Effort_OK_Callback, (XtPointer) pClientData);
  
  /* Create form for push buttons */
  XtSetArg(args[0], XmNverticalSpacing, (XtArgVal) 8);
  XtSetArg(args[1], XmNfractionBase, (XtArgVal) 4);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  buttons_form = XtCreateManagedWidget ("form", xmFormWidgetClass, form,
					args, 5);

  /* Create a separator between entry field and push buttons */
  XtSetArg(args[0], XmNtopOffset, (XtArgVal) 10);
  XtSetArg(args[1], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNbottomAttachment, XmATTACH_WIDGET);
  XtSetArg(args[3], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[5], XmNtopWidget, (XtArgVal) pClientData->effortText);
  XtSetArg(args[6], XmNbottomWidget, (XtArgVal) buttons_form);
  dialog_sep = XmCreateSeparatorGadget (form, "separator", args, 7);
  XtManageChild (dialog_sep);
      
  /* Create the OK push button */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_POSITION);
  XtSetArg(args[3], XmNrightAttachment, XmATTACH_POSITION);
  XtSetArg(args[4], XmNleftPosition, 0);
  XtSetArg(args[5], XmNrightPosition, 1);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 8);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 4);
  okButton = XtCreateManagedWidget ("OK", xmPushButtonWidgetClass,
				    buttons_form, args, 8);
  XtAddCallback (okButton, XmNactivateCallback, Effort_OK_Callback, 
		 (XtPointer) pClientData);

  /* Create the Cancel push button */
  XtSetArg(args[4], XmNleftPosition, 1);
  XtSetArg(args[5], XmNrightPosition, 2);
  XtSetArg(args[6], XmNleftOffset, (XtArgVal) 4);
  XtSetArg(args[7], XmNrightOffset, (XtArgVal) 8);
  cancelButton = XtCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
				        buttons_form, args, 8);
  XtAddCallback (cancelButton, XmNactivateCallback, Effort_Cancel_Callback, 
		 (XtPointer) pClientData);

  XtManageChild (pClientData->dialog);
}
