#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>

#include "base/ds/ds_thread.h"
#include "base/ds/ds_safeq.h"

#define NUM_READERS 4

char *poem[] = {
    "Mary had a little lamb",
    "Whose fleece was white as snow",
    "And everywhere that Mary went",
    "The lamb was sure to go",
    "",
    "Mary, Mary, quite contrary",
    "How does your garden grow?",
    "",
    "Mary, Mary!",
    "Why you buggin' me!"
};

int
Reader(void *arg)
{
    ds_safeq_t *q = (ds_safeq_t *)arg;
    char *string;

    while ((string = (char*)ds_safeq_deq(q))) {
	ds_thread_log(0, string);
	ds_thread_yield();
    }
    return 0;
}

int
Writer(void *arg)
{
    ds_safeq_t *q = (ds_safeq_t *)arg;
    int i;
    int j=0;

    for (i=0; i<(sizeof(poem)/sizeof(char*)); i++) {
	ds_safeq_enq(q, poem[i]);
	j++;
	if (!(j%NUM_READERS)) ds_thread_yield();
    }
    ds_safeq_destroy(q);

    return 0;
}
    

int
main(argc, argv)
    int argc;
    char *argv[];
{
    ds_safeq_t *q;
    int         i;


    ds_thread_init(DS_THREAD_STACKSZ, 0);
    q = ds_safeq_create();

    /* Spin off some readers */
    for (i=0; i<NUM_READERS; i++) {
	(void) ds_thread_create(Reader, q, DS_THREAD_NORMAL, "reader",
				0);
    }
    
    /* Call the writer directly */
    return (Writer(q));
}

