#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef __ODYSSEY_MULTRES_PRIVATE_H__
#define __ODYSSEY_MULTRES_PRIVATE_H__

/*
   odyssey_multres.private.h - multiple-resource private definitions

   Include file for the user-level impelementation of multiple-resource
   requests.  It provides definitions of internal data structures used
   to keep track of outstanding multiple-resource requests and map
   them to single-resource requests. 
*/

/* Defines whether the effect of an incremental increase in resource value
   has a positive or negative effect on the overall value of the 
   multiple-resource equation. */

#define INCR_EFFECT_NEGATIVE 0
#define INCR_EFFECT_POSITIVE 1

/* Theoretical maximum value for a resource.  This is used to detect
   nonsensical inputs and infinite loop conditions. */

#define MAX_RES_VALUE 999999999

/* This structure represents one resource in a multiple-resource 
   equation.  It includes the current value of the resource as well
   as the calculated resource window and the id of any outstanding
   request. */

typedef struct ody_req_res_prvt_t {
  unsigned int orrp_resource;      /* resource number */
  int          orrp_incr_effect;   /* incr. effect of adding resource */
  int          orrp_low_window_value; /* value for low window */
  int          orrp_high_window_value; /* value for high window */
  int          orrp_request_id;    /* id of current outstanding request */
} ody_req_res_prvt_t;  

/* This structure represents a multiple-resource requests.  It contains
   information about the multiple-resource request such as the window,
   evaluation function, and callback function.  It also contains information
   about each of the resources that comprise the request. */

typedef struct ody_mreq_res_prvt_t {
  int                  omrp_fd;         /* odyssey file descriptor */
  char*                omrp_path;       /* odyssey path (valid if fd = -1 */
  int                  omrp_request_id; /* id for this request */
  int                  omrp_resnum;     /* number of resources */
  float                omrp_low;        /* lower bound on function value */
  float                omrp_high;       /* upper bound on function value */
  ody_eval_fn_t        omrp_low_eval_fp; /* user-supplied evaluation func. */
  ody_eval_fn_t        omrp_high_eval_fp; /* user-supplied evaluation func. */
  ody_mreq_fn_t        omrp_req_fp;     /* call this when bound is violated */
  struct ody_vers      omrp_version;    /* Odyssey version structure */
  ody_req_res_val_t*   omrp_values;     /* current value of resources */
  ody_req_res_prvt_t*  omrp_resources;  /* resources in the request */
  struct ody_mreq_res_prvt_t* omrp_next; /* ptr to next request in list */
} ody_mreq_res_prvt_t;

/* Function prototypes */
void scaleWindow (int resnum, ody_req_res_prvt_t* pResData,
		  ody_req_res_val_t* pInValues, ody_req_res_val_t* pOutvalue, 
		  float scale);
int setResourceWindows (ody_mreq_res_prvt_t* pRequest, 
			float current_low_value, float current_high_value);
int ody_getResourceValue (int fd, char* path, int resource, 
			  struct ody_vers version);
static void outOfWindow(int reqid, unsigned int rsrc, int val);
int ody_multi_request (int fd, char* path, struct ody_req_mres* req, 
		       int* reqid, ody_req_res_val_t** val);
#endif
