#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1994 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Copyright (c) 1993 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software_Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */

/* note -- this was grabbed from
   /afs/cs.cmu.edu/misc/fac/src/fac/src/lib/libcs on May 11 1998.
   the routines in this file came from the three files nxtarg.c,
   fold.c, and skipto.c.
   -tilt */

#include <cs_support.h>

/*
 * HISTORY
 * Log: nxtarg.c,v
 * Revision 1.4  1994/11/07  22:31:33  chs
 * 	Added include of libcs.h.  De-linted.
 * 	[1994/11/06  02:53:14  chs]
 *
 * Revision 1.3  1994/06/28  19:23:57  chs
 * 	New copyright notice.
 * 	[1994/06/25  18:19:06  chs]
 * 
 * Revision 1.2  90/12/11  17:57:10  mja
 * 	Add copyright/disclaimer for distribution.
 * 
 * Revision 0.0  83/07/01            sas
 * 	Bug fix: added check for "back >= front" in loop to chop trailing
 * 	white space.
 * 	[83/07/01            sas]
 * 
 * Revision 0.0  79/11/20            sas
 * 	Rewritten for VAX.  By popular demand, a table of break characters
 * 	has been added (implemented as a string passed into nxtarg).
 * 
 * 	Originally	from klg (Ken Greer); IUS/SUS UNIX.
 * 	[79/11/20            sas]
 */

/*
 *  nxtarg -- strip off arguments from a string
 *
 *  Usage:  p = nxtarg (&q,brk);
 *	char *p,*q,*brk;
 *	extern char _argbreak;
 *
 *	q is pointer to next argument in string
 *	after call, p points to string containing argument,
 *	q points to remainder of string
 *
 *  Leading blanks and tabs are skipped; the argument ends at the
 *  first occurence of one of the characters in the string "brk".
 *  When such a character is found, it is put into the external
 *  variable "_argbreak", and replaced by a null character; if the
 *  arg string ends before that, then the null character is
 *  placed into _argbreak;
 *  If "brk" is 0, then " " is substituted.
 *
 */

char _argbreak;

char *nxtarg (q, brk)
char **q;
const char *brk;
{
	register char *front, *back;

	front = *q;			/* start of string */
	/* leading blanks and tabs */
	while (*front && (*front == ' ' || *front == '\t')) front++;
	/* find break character at end */
	if (brk == 0)  brk = " ";
	back = skipto (front,brk);
	_argbreak = *back;
	*q = (*back ? back+1 : back);	/* next arg start loc */
	/* elim trailing blanks and tabs */
	back -= 1;
	while ((back >= front) && (*back == ' ' || *back == '\t')) back--;
	back++;
	if (*back)  *back = '\0';
	return (front);
}

/*
 * HISTORY
 * Log: fold.c,v
 * Revision 1.4  1994/11/07  22:29:24  chs
 * 	Added include of libcs.h.  De-linted.
 * 	[1994/11/06  02:52:09  chs]
 *
 * Revision 1.3  1994/06/28  19:21:46  chs
 * 	New copyright notice.
 * 	[1994/06/23  19:14:20  chs]
 * 
 * Revision 1.2  90/12/11  17:53:16  mja
 * 	Add copyright/disclaimer for distribution.
 * 
 * Revision 0.0  86/09/23            gm0w
 * 	Unexported original fold routine.
 * 	[86/09/23            gm0w]
 * 
 * Revision 0.0  79/11/20            sas
 * 	Rewritten for VAX.  The foldup() and folddown() routines
 * 	are new.
 * 	[79/11/20            sas]
 * 
 * $EndLog$
 */

/*  fold  --  perform case folding
 *
 *  Usage:  p = foldup (out,in);
 *	    p = folddown (out,in);
 *	char *p,*in,*out;
 *
 *  Fold performs case-folding, moving string "in" to
 *  "out" and folding one case to another en route.
 *  Folding may be upper-to-lower case (folddown) or
 *  lower-to-upper case.
 *  Foldup folds to upper case; folddown folds to lower case.
 *  The same string may be specified as both "in" and "out".
 *  The address of "out" is returned for convenience.
 *
 */

typedef enum {FOLDUP, FOLDDOWN} FOLDMODE;

static
char *fold (o, i, whichway)
char *o;
const char *i;
FOLDMODE whichway;
{
	register char lower = 0;
	char upper = 0;
	int delta = 0;

	switch (whichway)
	{
	case FOLDUP:
		lower = 'a';		/* lower bound of range to change */
		upper = 'z';		/* upper bound of range */
		delta = 'A' - 'a';	/* amount of change */
		break;
	case FOLDDOWN:
		lower = 'A';
		upper = 'Z';
		delta = 'a' - 'A';
	}

	do {
		if (*i >= lower && *i <= upper)		*o++ = *i++ + delta;
		else					*o++ = *i++;
	} 
	while (*i);
	*o = '\0';
	return (o);
}

char *foldup (out,in)
char *out;
const char *in;
{
	return (fold(out,in,FOLDUP));
}

char *folddown (out,in)
char *out;
const char *in;
{
	return (fold(out,in,FOLDDOWN));
}

/*
 * HISTORY
 * Log: skipto.c,v
 * Revision 1.4  1994/11/07  22:32:34  chs
 * 	Added include of libcs.h.  De-linted.
 * 	[1994/11/06  02:53:51  chs]
 *
 * Revision 1.3  1994/06/30  01:28:55  chs
 * 	New copyright notice.
 * 	[1994/06/25  18:52:49  chs]
 * 
 * Revision 1.2  90/12/11  17:59:10  mja
 * 	Add copyright/disclaimer for distribution.
 * 
 * Revision 0.0  81/06/26            drs
 * 	Skipover, skipto rewritten to avoid inner loop at expense of space.
 * 	[81/06/26            drs]
 * 
 * Revision 0.0  79/11/20            sas
 * 	Skipover, skipto adapted for VAX from skip() and skipx() on the PDP-11
 * 	(from Ken Greer).  The names are more mnemonic.
 * 
 * 	Sindex adapted for VAX from indexs() on the PDP-11 (thanx to Ralph
 * 	Guggenheim).  The name has changed to be more like the index()
 * 	and rindex() functions from Bell Labs; the return value (pointer
 * 	rather than integer) has changed partly for the same reason,
 * 	and partly due to popular usage of this function.
 * 	[79/11/20            sas]
 * 
 */

/*
 *  skipover and skipto -- skip over characters in string
 *
 *  Usage:	p = skipto (string,charset);
 *		p = skipover (string,charset);
 *
 *  char *p,*charset,*string;
 *
 *  Skipto returns a pointer to the first character in string which
 *  is in the string charset; it "skips until" a character in charset.
 *  Skipover returns a pointer to the first character in string which
 *  is not in the string charset; it "skips over" characters in charset.
 */

static char tab[256] = { 0 };

char *skipto (string,charset)
const char *string, *charset;
{
	const char *setp,*strp;

	tab[0] = 1;		/* Stop on a null, too. */
	for (setp=charset;  *setp;  setp++) tab[(int)*setp]=1;
	for (strp=string;  tab[(int)*strp]==0;  strp++)  ;
	for (setp=charset;  *setp;  setp++) tab[(int)*setp]=0;
	return ((char *)strp);
}

char *skipover (string,charset)
const char *string, *charset;
{
	const char *setp,*strp;

	tab[0] = 0;		/* Do not skip over nulls. */
	for (setp=charset;  *setp;  setp++) tab[(int)*setp]=1;
	for (strp=string;  tab[(int)*strp];  strp++)  ;
	for (setp=charset;  *setp;  setp++) tab[(int)*setp]=0;
	return ((char *)strp);
}
