#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1994 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Copyright (c) 1993 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software_Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */

/* note -- this was grabbed from
   /afs/cs.cmu.edu/misc/fac/src/fac/src/lib/libcs on May 11 1998.
   the routines in this file came from the two files libcs.h and c.h,
   both of which were brutally shortened to only the bare essentials.
   -tilt */

/*
 * HISTORY
 * $Log: cs_support.h,v $
 * Revision 1.3  2000/01/14 00:08:25  bumba
 *
 *
 * GPL'ed the code base.
 *
 * Revision 1.2  1998/06/19 16:53:59  tilt
 * Added copyright notice. Massive commit.
 *
 * Revision 1.1  98/05/12  14:49:40  tilt
 * Various changes to
 * 
 *   * put the www code into the right places
 *   * get the www code to compile
 *   * deal with the viceroy_onode.h file, and other attempts at
 *     making borrowed kernel structures entirely local.
 * 
 * Revision 1.19  1996/08/15  19:23:01  moore
 * 	Added some minor linux support.
 * 	[1996/08/15  19:22:11  moore]
 *
 * Revision 1.18  1996/06/03  19:39:40  moore
 * 	Re-add sindex.  Subtle differences between sindex and strstr
 * 	in empty second parameter.
 * 	[1996/06/03  19:38:23  moore]
 * 
 * Revision 1.17  1994/12/07  22:28:31  chs
 * 	Commented out const in const char **.
 * 	Added prototypes for the rest of the exported functions.
 * 	[1994/12/07  22:16:29  chs]
 * 
 * Revision 1.16  1994/11/07  22:31:16  chs
 * 	Fixed prototypes for concat and pathof.
 * 	[1994/11/07  22:25:09  chs]
 * 
 * 	Use __P macro from BSD44.  Added more prototypes.
 * 	[1994/11/06  03:09:28  chs]
 * 
 * Revision 1.15  1994/11/03  21:49:37  chs
 * 	Fixed whitespace in history logs.
 * 	[1994/11/03  21:34:07  chs]
 * 
 * 	Removed sysent.h, strings.h
 * 	Don't define const as nothing.
 * 	Changed return values of gtime() and atot() from long to time_t.
 * 	Removed many decl's for things not in libcs.
 * 	[1994/11/03  04:27:48  chs]
 * 
 * Revision 1.14  1994/01/13  15:53:07  moore
 * 	Include <ctype.h>, <locale.h>, <ctype.h> and <stdlib.h> and removed
 * 	all local proc definitions for routines that were declared in those
 * 	include files.
 * 	[94/01/13  15:52:55  moore]
 * 
 * Revision 1.13  93/03/31  13:53:51  moore
 * 	Fixed type of second parameter for getopt.
 * 	I wish that folks would realize that this file is
 * 	a CMU-ism, and should probably go away ASAP.
 * 	[93/03/31  13:53:34  moore]
 * 
 * Revision 1.12  93/01/06  15:34:38  moore
 * 	Added execlp and execvp.
 * 	[93/01/06  15:34:18  moore]
 * 
 * Revision 1.11  92/06/15  14:39:57  chs
 * 	Removed the #ifdef's around the inclusions of other system headers,
 * 	since they should all already be protected against multiple
 * 	inclusions, and some of the checks were wrong anyway.
 * 	Changes for OSF/1:
 * 	  Define "const" as nothing for the duration of this file.
 * 	  If _POSIX_SOURCE is defined, include some standard headers
 * 	  rather than declaring a bunch of stuff (incorrectly) ourselves.
 * 	[92/06/15  14:38:55  chs]
 * 
 * Revision 1.10  91/07/01  17:44:51  neek
 * 	declare "extern char _argbreak" irregardless of whether
 * 	__STDC__ is defined or not.
 * 	[91/07/01            neek]
 * 
 * Revision 1.9  90/12/12  20:56:44  mja
 * 	Add copyright/disclaimer for distribution.
 * 
 * Revision 1.8  90/09/28  13:13:31  dlc
 * 	Added declaration for fdate().
 * 	[90/09/28            dlc]
 * 
 * Revision 1.7  89/04/03  11:10:45  vanryzin
 * 	Changed definition of qsort for c++ to indicate the procedure
 * 	passed to qsort has parameters.  Since we were unsure if ANSI C
 * 	could handle the syntax I placed the new definition within #if
 * 	defined(c_plusplus) conditionals.  This may not be necessary
 * 	and perhaps should be fixed at a later time.
 * 	[89/04/03            vanryzin]
 * 
 * Revision 1.6  89/02/05  15:55:57  gm0w
 * 	Added extern char *errmsg().
 * 	[89/02/04            gm0w]
 * 
 * Revision 1.5  89/01/20  15:34:40  gm0w
 * 	Moved all of the STDC changes to other existing include files
 * 	back into this one.  Added non-STDC extern declarations for
 * 	all functions without int return values to match those defined
 * 	by STDC.  Added include of sysent.h.  Removed obsolete cdate
 * 	extern declaration.
 * 	[88/12/17            gm0w]
 * 
 * Revision 1.4  88/12/22  16:58:56  mja
 * 	Correct __STDC__ parameter type for getenv().
 * 	[88/12/20            dld]
 * 
 * Revision 1.3  88/12/14  23:31:42  mja
 * 	Made file reentrant.  Added declarations for __STDC__.
 * 	[88/01/06            jjk]
 * 
 * Revision 0.0  88/04/30            gm0w
 * 	Added pathof() extern.
 * 	[88/04/30            gm0w]
 * 
 * Revision 0.0  85/12/01            gm0w
 * 	Added getname() extern.
 * 	[85/12/01            gm0w]
 * 
 * Revision 0.0  85/11/29            gm0w
 * 	Added lseek() extern.
 * 	[85/11/29            gm0w]
 * 
 * Revision 0.0  85/11/02            gm0w
 * 	Added salloc() extern.
 * 	[85/11/02            gm0w]
 * 
 * Revision 0.0  81/08/14            mja
 * 	Created.
 * 	[81/08/14            mja]
 * 
 * $EndLog$
 */

#ifndef	_LIBCS_H_
#define	_LIBCS_H_ 1

#include <sys/types.h>
#include <ctype.h>
#include <locale.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#if	__STDC__
#include <stdarg.h>		/* for va_list */
#endif

#ifndef	__P
#if	__STDC__
#define __P(x) x
#else
#define __P(x) ()
#endif
#endif

/*
 * Standard C macros
 */

#ifndef ABS
#define ABS(x) ((x)>=0?(x):-(x))
#endif

#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef	FALSE
#define FALSE	0
#endif	/* FALSE */
#ifndef	TRUE
#define TRUE	1
#endif	/* TRUE */

#define	CERROR		(-1)

#ifndef	bool
typedef enum	{ false = 0, true = 1 } bool;
#endif	/* bool */

#define	sizeofS(string)	(sizeof(string) - 1)
#define sizeofA(array)	(sizeof(array)/sizeof(array[0]))

#define caseE(enum_type)	case (int)(enum_type)


extern char *foldup __P((char *, const char *));
extern char *folddown __P((char *, const char *));
extern char *skipto __P((const char *, const char *));
extern char *skipover __P((const char *, const char *));
extern char *nxtarg __P((char **, const char *));

#endif	/* _LIBCS_H_ */
