#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * ping.c -- simple ping program for Software Systems project 1
 */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <base/simplewww/swww.h>
#include "include/odytypes.h"
#include "ping.h"

int main(int argc, char **argv) {
    int                   response_code;
    int                   pingtimes;
    int                   lowsize, highsize, incsize;
    int                   size;
    int                   loop;
    struct SWWW_fetch_in  in;
    struct SWWW_fetch_out out;
    struct SWWW_postinfo  pb;
    struct timeval time1, time2, result;

    if (argc != 6) {
	fprintf(stderr, USAGE, argv[0]);
	exit(-1);
    }
    
    if(SWWW_version(SWWW_VERSION)) {
	/* if library version doesn't match header file, we're hosed */
	exit(-1);
    }

    lowsize   = atoi(argv[2]);
    highsize  = atoi(argv[3]);
    incsize   = atoi(argv[4]);
    pingtimes = atoi(argv[5]);

    if (!lowsize || !highsize || !incsize || !pingtimes) {
	fprintf(stderr, USAGE, argv[0]);
	exit(-1);
    }

    /* Set up parameters */
    SWWW_SFI_INIT(&in);
    in.sfi_method        = POST;
    in.sfi_url           = argv[1];
    in.sfi_reqid         = 0;
    in.sfi_threading     = NO_THREADING;
    in.sfi_acceptheaders = NULL;
    in.sfi_acceptdata    = data_pump;
    in.sfi_postinfo      = &pb;

    pb.spi_postid = -1;
    pb.spi_buffer = NULL;

    for(size = lowsize; size <= highsize; size += incsize) {
	printf("Size: %d\n", size);
	pb.spi_length  = size;
	pb.spi_bufsize = pb.spi_length;
	if (pb.spi_buffer != NULL)
	    free(pb.spi_buffer);
	pb.spi_buffer  = (char *) malloc(pb.spi_bufsize);
	memset(pb.spi_buffer, 'Q', pb.spi_bufsize);
	pb.spi_fetchdata = NULL;
	
	for(loop = 0; loop < pingtimes; loop++) {
	    gettimeofday(&time1, NULL);
	    response_code = SWWW_fetch_url(&in, &out);
	    free(out.sfo_status);
	    SWWW_free_headerlist(out.sfo_headers);
	    gettimeofday(&time2, NULL);
	    
	    timersub(&time2, &time1, &result); /* sub first time from second */
	    printf("%d: %ld seconds, %ld microseconds\n",
		   loop + 1, result.tv_sec, result.tv_usec);
	} /* end tests per size loop */
	
    } /* end size loop */
    return(0);
}

int data_pump(int reqid, const void *data, int size) {
    /* function which accepts data and dumps it to a temp file (indicated
       by the fd which reqid corresponds to) */
    return(size);
}
