#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * SimpleWWW library
 *
 * This is the public interface for a simple www library. The
 * motivation for having such a library is that the www-lib supplied
 * by W3C is extremely large and complex, and -- more importantly --
 * is difficult to use with any degree of confidence (mostly due to
 * poor documentation). For applications with simple needs (such as
 * simply wanting to be able to fetch the headers and data associated
 * with a URL), using www-lib seems to be much more trouble than it is
 * worth. (Also, since the initial writing of this library, the writer
 * of www-lib has pretty much admitted that he's trying to support
 * graphical Web browser writers, which is not what we're interested in.)
 *
 * History
 *   28 Jun 1996 -- ? -- Original work, with working client-side interface.
 *    3 Jul 1996 -- 1 -- The original code organization has been redone,
 *                       so that there is now a single include file
 *                       (swww.h) which applications using swww need to
 *                       include. This history list will include information
 *                       about significant library-wide revisions and
 *                       interface changes.
 *
 *                       New functions with this revision include
 *                       SWWW_version(), for contraindicating version skew,
 *                       and SWWW_accept_requests(), which provides server
 *                       functionality.
 *
 *    6 Jul 1996 -- 2 -- acceptheaders_fn signature added, and
 *                       SWWW_fetch_url() functionality changed so that
 *                       acceptheaders_fn is called with the object headers
 *                       BEFORE acceptdata_fn is called with the object data.
 *
 *   15 Jul 1996 -- 3 -- headerent changed from having entire header
 *                       in "header" field to having it stored in "name"
 *                       and "value" fields.
 *
 *                       "headers" parameter of SWWW_fetch_url is now
 *                       a handle; specifically, this is now a list allocated
 *                       by SWWW_fetch_url (which should be dealloced by
 *                       calling function via SWWW_free_headerlist()).
 *
 *                       In a related change, acceptheaders_fn signature has
 *                       been changed to take a pointer to the header list.
 *
 *                       SWWW_free_headerlist() added for freeing header
 *                       lists returned by fetch_url_fns.
 *
 *   17 Jul 1996 -- 4 -- added LWP support, albeit in a hacky way.
 *                       specifically, a useLWP flag has been added to
 *                       SWWW_accept_requests, SWWW_fetch_url, and
 *                       fetch_url_fn. This mechanism may change, if I come
 *                       up with a more reasonable general mechanism
 *                       for specifying the threading mechanism.
 *
 *   19 Jul 1996 --  0. 0. 5 -- Changed "useLWP" to "threading", and
 *                              made it take an enum. This is still a
 *                              hack (since it requires explicit support
 *                              for each threading package), but it's
 *                              less objectionable. LWP is still the
 *                              only supported package.
 *
 *                              Also, the numbering scheme has become
 *                              a three-tuple, of the form xx.yy.zz.
 *                              This is represented as a six digit
 *                              integer of the form xxyyzz. The only
 *                              major effect of this is that these
 *                              comments have much more interesting
 *                              indenting.
 *
 *   29 Jul 1996 --          -- Added SWWW_sethvalue(). This doesn't
 *                              break the existing interface, so the
 *                              version number hasn't been bumped.
 *                              Also added SWWW_duplicate_headerlist().
 *
 *    1 Aug 1996 --  0. 0. 6 -- Changed acceptheaders_fn signature to
 *                              take the status code as an argument.
 *                              Also took the opportunity to reorganize
 *                              parameters to make slightly more sense.
 *
 *    9 Aug 1996 --  0. 0. 7 -- Changed parse_url() into SWWW_parse_url(),
 *                              which is a public function.
 *
 *   27 Sep 1996 --  0. 1. 0 -- Several name changes, in
 *                              the name of cleaning up. These include:
 *
 *                                headerent        -> SWWW_headers_t
 *                                http_method_t    -> SWWW_method_t
 *                                threading_type_t -> SWWW_threading_type_t
 *                                acceptheaders_fn -> SWWW_acceptheaders_fn
 *                                acceptdata_fn    -> SWWW_acceptdata_fn
 *                                fetch_url_fn     -> SWWW_fetch_url_fn
 *
 *                              The name changes were predominately to
 *                              ensure that the prefix "SWWW_" is in front
 *                              of all public names exported by the library.
 *
 *   27 Sep 1996 --  0. 1. 1 -- Support for HEAD and POST added, both
 *                              in SWWW_accept_requests() and
 *                              SWWW_fetch_url().
 *
 *                              The parameter list for SWWW_fetch_url()
 *                              was changed, to use in and out structures.
 *                              (This should have happened a while ago,
 *                              since it already has some large number
 *                              of parameters).
 *
 *                              As a corollary, the definition of
 *                              SWWW_fetch_url_fn has also been changed.
 *
 *                              The parameter list for SWWW_accept_requests()
 *                              is unchanged.
 *
 *                              Added SWWW_SFI_INIT() and SWWW_SPI_INIT()
 *                              macros to initialize structures to
 *                              sane values.
 *
 *   30 Sep 1996 --          -- Added SWWW_create_headerlist().
 *
 *                              SWWW_sethvalue() now returns 1 if
 *                              it adds a header to the list, and 0
 *                              if it modifies an existing header.
 *
 *                              Added SWWW_VERS_STRING definition
 *
 *   13 Feb 1997 --          -- Added SWWW_clearhvalue().
 *
 *   27 May 1997 --          -- Added SWWW_debugging().
 *
 *   27 Oct 1997 --          -- Added SWWW_{pack,unpack}_headers().
 *
 *   11 May 1998 --          -- Added SWWW_parse_date
 */

#ifndef	_SWWW_H_
#define _SWWW_H_

#include <base/ds/ds_safeq.h>

/*
 * Common SimpleWWW defines
 */

/* Version of library. Increment this every time you make an incompatible
   changes to the API (such as modifying the parameters to a known function,
   or changing a function's pre- or post-conditions) */

/* Current version is 0.1.1 */
#define SWWW_VERSION                     101
#define SWWW_VERS_STRING    "SimpleWWW/0.1.1"

/*
 * Common SimpleWWW data structures, enums, and function signatures
 */

/* list of headers */
typedef struct SWWW_headers {
    char                *name;
    char                *value;
    struct SWWW_headers *next;
} SWWW_headers_t;

/* Supported HTTP methods: */
typedef enum SWWW_method {
    METHOD_ERROR,
    GET,
    HEAD,
    POST
} SWWW_method_t;

/* What method of threading are we using? */
typedef enum SWWW_threading_type {
    NO_THREADING,
    LWP
} SWWW_threading_type_t;

/* Signature for acceptheaders function (passed as an argument to
 * SWWW_fetch_url()). SWWW_fetch_url() will pass this function the
 * list of headers before invoking the acceptdata function with object
 * data. This function should return 0 on success, non-zero on failure. */

typedef int (*SWWW_acceptheaders_fn) (int            reqid,
				      int            status_code,
				      char           *status, 
				      int            header_size,
				      SWWW_headers_t *headers);

/* Signature for acceptdata function (passed as an argument to SWWW_fetch_url).
 * This intentionally matches the signature for write(), so that the reqid
 * can simply be an open file descriptor (and write() can be used for pumping
 * data directly to a file). The return value for this function has
 * the same meaning as that of the write() function. */

typedef int (*SWWW_acceptdata_fn) (int        reqid,
				   const void *data,
				   int        size);

/* Signature for fetchdata function. This function is used by
 * the SWWW_fetch_url_fn to get POST information.
 *
 * This intentionally matches the signature for read(), so that the postid
 * can simply be an open file descriptor. */

typedef int (*SWWW_fetchdata_fn) (int  postid,
				  void *buf,
				  int  size);

/* IN/OUT parameters and signature for fetch_url function
 * (passed as an argument to SWWW_accept_requests()). */

struct SWWW_postinfo {
    int                spi_postid;    /* ID of post information. */
    int                spi_length;    /* Length of information to
					 be posted */
    char              *spi_buffer;    /* Initial buffer of post data,
					 if any. If none, should be NULL */
    int                spi_bufsize;   /* Size of initial buffer */
    SWWW_fetchdata_fn  spi_fetchdata; /* Function which can retrieve
					 additional sequential segments
					 of the post data (ala read()).
					 Can be NULL if
					 spi_bufsize >= spi_length */
};

/* Macro for initializing struct SWWW_postinfo. Using this macro will
   ensure that, if the structure changes, new members will get sane
   initializations */

#define SWWW_SPI_INIT(spip)                                \
do {                                                       \
    (spip)->spi_postid    = -1;                            \
    (spip)->spi_length    = -1;                            \
    (spip)->spi_buffer    = NULL;                          \
    (spip)->spi_bufsize   = 0;                             \
    (spip)->spi_fetchdata = NULL;                          \
} while (0)

struct SWWW_fetch_in {
    SWWW_method_t         sfi_method;	  /* HTTP method to use */
    char                 *sfi_url;        /* URL to fetch */
    int                   sfi_reqid;	  /* ID of request. This is passed
					     to the function pointed to by
					     acceptdata_fn, to allow that
					     function to discriminate between
					     the data. The ID is supplied by
					     the caller. */
    SWWW_headers_t       *sfi_headers;    /* Headers to be used when
					     fetching document, if
					     any. If there are none,
					     this should be NULL. */
    struct SWWW_postinfo *sfi_postinfo;   /* Information about the POST
					     data (if sfi_method is POST) */
    SWWW_threading_type_t sfi_threading;  /* type of threading to be
					     used (currently either
					     NO_THREADING, or LWP) */
    SWWW_acceptheaders_fn sfi_acceptheaders; /* Function which should be
						used to send headers to,
						once they have all arrived.
						If NULL, the function
						is not invoked. */
    SWWW_acceptdata_fn    sfi_acceptdata; /* Function which should be used to
					     pump data to, as it arrives.
					     Must be supplied. */
};

/* Macro for initializing struct SWWW_fetch_in. Using this macro will
   ensure that, if the structure changes, new members will get sane
   initializations */

#define SWWW_SFI_INIT(sfip)                              \
do {                                                     \
    (sfip)->sfi_method        = GET;                     \
    (sfip)->sfi_url           = NULL;                    \
    (sfip)->sfi_reqid         = -1;                      \
    (sfip)->sfi_headers       = NULL;                    \
    (sfip)->sfi_postinfo      = NULL;                    \
    (sfip)->sfi_threading     = NO_THREADING;            \
    (sfip)->sfi_acceptheaders = NULL;                    \
    (sfip)->sfi_acceptdata    = NULL;                    \
} while (0)

struct SWWW_fetch_out {
    char           *sfo_status;      /* HTTP status line.
					(e.g. "HTTP/1.0 200 Document follows").
					Allocated by SWWW_fetch_url_fn, but
					must be freed by calling function. */
    int             sfo_header_size; /* Number of headers returned in
					'headers' array. */
    SWWW_headers_t *sfo_headers;     /* List of the headers associated with
					the HTTP object specified by the url.
					Allocated by SWWW_fetch_url_fn, but
					must be freed by calling function. */
};

typedef int (*SWWW_fetch_url_fn) (struct SWWW_fetch_in  *inp,
				  struct SWWW_fetch_out *outp);

/*
 * The public interface for the SimpleWWW library provides the following
 * functions:
 *
 * UTILITY FUNCTIONS:
 *
 * SWWW_version -- accepts an expected version number from the application,
 *                 and compares it to the actual version number. If they
 *                 differ, a complaint is printed to stderr, and the
 *                 actual version number is returned. Otherwise, 0 is returned.
 */
int SWWW_version(int expectedversion);	/* expected version number */

/*
 * SWWW_debugging -- sets the SWWW debugging level.
 *                   Level 0 prints no messages,
 *                   level 1 prints all messages.
 */
void SWWW_debugging(int debuglevel);    /* new debug level */

/*
 * SWWW_create_headerlist -- returns an empty header list which can be
 *                           manipulated with the other header list
 *                           functions.
 */
SWWW_headers_t *SWWW_create_headerlist(void);

/*
 * SWWW_gethvalue -- accepts a list of headers (such as is generated by
 *                   SWWW_fetch_url), and the name of a header, and
 *                   returns the value of that header. If there is no
 *                   such header, NULL is returned. Calling code is
 *                   responsible for freeing the returned value.
 */

char *SWWW_gethvalue(SWWW_headers_t *headers,  /* list of headers */
		     const char     *name);    /* name of the header
                                                  for which we want the
						  value */

/*
 * SWWW_sethvalue -- accepts a list of headers (such as is generated by
 *                   SWWW_fetch_url), the name of a header, and
 *                   the new value for that header. The header is
 *                   added if it does not already exist.
 *
 *                   1 is returned if the header is added, and
 *                   0 is returned if an existing header is modified.
 *
 */

int SWWW_sethvalue(SWWW_headers_t *headers,   /* list of headers */
		   const char     *name,      /* name of the header
						 which we are changing */
		   const char     *value);    /* new value for the header */

/* SWWW_clearhvalue -- accepts a list of headers (such as is generated by
 *                     SWWW_fetch_url), and the name of a header.
 *                     The header is removed from the list of headers
 *                     if it exists.
 *
 *                     -1 is returned if the headers is removed, and
 *                      0 is returned if no headers match name.
 */

int SWWW_clearhvalue(SWWW_headers_t *headers, /* list of headers */
		     const char     *name);   /* name of the header which
						 we are clearing */

/*
 * SWWW_duplicate_headerlist -- given a pointer to the start of a list of
 *                              headers, it allocates space for a new copy
 *                              of the list, and returns a pointer to the
 *                              new list.
 */

SWWW_headers_t *SWWW_duplicate_headerlist(int            header_size,
					  SWWW_headers_t *headers);

/*
 * SWWW_free_headerlist -- given a pointer to the start of a list of headers,
 *                         it frees all memory used by that list (including
 *                         strings).
 */

void SWWW_free_headerlist(SWWW_headers_t *headers);

/*
 * SWWW_print_headerlist -- given a pointer to the start of a list of headers,
 *                          prints all name/value pairs).
 */

void SWWW_print_headerlist(SWWW_headers_t *headers);

/*
 * SWWW_pack_headers -- given a list of headers, return a packed buffer.
 *                      NOTE: the caller must free the returned buffer.
 */

void SWWW_pack_headers(SWWW_headers_t  *headers,
	               char           **packed_headers,
		       int             *size);

/*
 * SWWW_unpack_headers -- given packed headers, returns a list of headers
 *                        header_size is the number of headers packed;
 *                        if unknown, use -1. (there may be compatability
 *                        problems if -1 is used but SWWW_pack_headers was
 *                        not used to pack the headers, since some legacy
 *                        code this package is used doesn't indicate the
 *                        end clearly.)
 */

SWWW_headers_t *SWWW_unpack_headers(int header_size,
	                            char *packed_headers);

/*
 * SWWW_parse_url -- given a url, parses the URL, returning pointers to the
 *                   scheme, the host (and the port number, if given), and
 *                   the path. Returns 0 if successful, and -1 if not.
 *                   Caller is responsible for deallocating scheme, host, and
 *                   path.
 */

int SWWW_parse_url(const char *url,
		   char       **scheme,
		   char       **host,
		   int        *port,
		   char       **path);

/*
 * SWWW_parse_date -- given a date/time stamp string, attempts to convert
 *                    it into a tm structure. Currently, this looks
 *                    like the libcs parsedate() function, but doesn't
 *                    implement the functionality; this will need to
 *                    be fixed one way or the other. XXX
 */
void SWWW_parse_date(char      *datestr,
                     struct tm *tm,
	 	     int        currtm,
		     int        past,
		     int        error);


/*
 * CLIENT FUNCTIONS:
 *
 * SWWW_fetch_url -- given a URL and a method, initiates an HTTP request.
 *                   Returns the response code, once the server has been
 *                   contacted and the headers have been received.
 *                   See the definitions of SWWW_fetch_{in,out} for
 *                   descriptions of the parameter lists.
 *
 *                   Calling function is responsible for freeing
 *                   "status" and "headers" in the OUT parameters.
 *
 */

int SWWW_fetch_url(struct SWWW_fetch_in  *inp,
		   struct SWWW_fetch_out *outp);
/*
 * SERVER FUNCTIONS
 *
 * SWWW_server_init -- setup request queue and worker threads
 */
int SWWW_server_init(int num_fetchers,
		     ds_safeq_t **request_q);

/*
 * SWWW_accept_requests -- this function listens at the specified port,
 *                         and hands requests off to the specified
 *                         fetch_url_fn. If it returns, it will return
 *                         an error code indicating why it returned.
 *
 */
int SWWW_accept_requests(int port,        /* Port number to listen
					     for requests on */
			 SWWW_fetch_url_fn fetch_url,
                                          /* Function which should be used
					     to fetch URLs needed to
					     fulfill requests */
			 SWWW_threading_type_t threading,
                                          /* type of threading to be
					     used (currently either
					     NO_THREADING or LWP) */
			 ds_safeq_t *request_q);

#endif	_SWWW_H_
