#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * test-fetch.c -- test client for SimpleWWW library
 */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <base/simplewww/swww.h>
#include "test-fetch.h"

int main(int argc, char **argv) {
  int                   response_code;
  int                   temp_file;
  SWWW_headers_t        *bookmark;
  char                  *value;
  struct SWWW_fetch_in  in;
  struct SWWW_fetch_out out;
  struct SWWW_postinfo  pb;

  if (argc < 2 || argc > 4) {
      fprintf(stderr, USAGE, argv[0]);
      exit(-1);
  }

  if(SWWW_version(SWWW_VERSION)) {
    /* if library version doesn't match header file, we're hosed */
    exit(-1);
  }

  temp_file = open("foo", O_WRONLY | O_CREAT | O_TRUNC,
		   S_IRUSR | S_IWUSR);

  SWWW_SFI_INIT(&in);
  in.sfi_method        = GET;
  in.sfi_url           = argv[1];
  in.sfi_reqid         = temp_file;
  in.sfi_threading     = NO_THREADING;
  in.sfi_acceptheaders = NULL;
  in.sfi_acceptdata    = (SWWW_acceptdata_fn) data_pump;

  if (argc == 3) {
      if (strcmp(argv[2], "GET") == 0) {
	  in.sfi_method = GET;
      } else if (strcmp(argv[2], "HEAD") == 0) {
	  in.sfi_method = HEAD;
      } else {
	  fprintf(stderr, USAGE, argv[0]);
	  exit(-1);
      }
  } else if (argc == 4 && (strcmp(argv[2], "POST") == 0)) {
      in.sfi_method   = POST;
      in.sfi_postinfo = &pb;
      pb.spi_postid = open(argv[3], O_RDONLY);
      if (pb.spi_postid < 0) {
	  /* must be a number */
	  pb.spi_postid = -1;
	  pb.spi_length = atoi(argv[3]);
	  if (pb.spi_length == 0) {
	      /* not a number or a valid file */
	      fprintf(stderr, USAGE, argv[0]);
	      exit(-1);
	  }
	  /* it is a number, so fill the buffer */
	  pb.spi_bufsize = pb.spi_length;
	  pb.spi_buffer  = (char *) malloc(pb.spi_bufsize);
	  memset(pb.spi_buffer, 'Q', pb.spi_bufsize);
	  pb.spi_fetchdata = NULL;
      } else {
	  /* really is a file */
	  struct stat sb;

	  fstat(pb.spi_postid, &sb);

	  pb.spi_buffer    = NULL;
	  pb.spi_bufsize   = 0;
	  pb.spi_length    = sb.st_size;
	  pb.spi_fetchdata = (SWWW_fetchdata_fn) read;
      }
  }

  response_code = SWWW_fetch_url(&in, &out);

  printf("(%d) %s\n", response_code, out.sfo_status);

  value = SWWW_gethvalue(out.sfo_headers, "Content-type");
  if(value != NULL) {
    printf("content type is '%s'\n", value);
  }

  printf("%d headers follow:\n", out.sfo_header_size);
  bookmark = out.sfo_headers;
  if(out.sfo_header_size) {	/* make sure we have headers */
    while(bookmark->next != NULL) {
      printf("'%s: %s'\n", bookmark->name, bookmark->value);
      bookmark = bookmark->next;
    }
  }

  return(0);
}

void data_pump(int reqid, void *data, int size) {
  /* function which accepts data and dumps it to a temp file (indicated
     by the fd which reqid corresponds to) */
  write(reqid, data, size);
}
