#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <sys/uio.h>

#include "include/odyssey.h"
#include "include/odytypes.h"

typedef struct TransLine {
  int               num_keywords;
  char**            keywords;
  char*             command;
  struct TransLine* next;
} TransLine;

typedef struct TransTable {
  char*               name;
  struct TransLine*   translations;
  struct TransTable*  next;
} TransTable;

typedef struct TransHeader {
  char*               header;
  struct TransHeader* next;
} TransHeader;

typedef struct {
  TransHeader* headers;
  TransTable*  tables;
} TransInfo;

char headerFile[] = 
"/**********************************************/\n"
"/* This file is generated by the yyak utility */\n"
"/**********************************************/\n"
"\n"
"void parseUtt(char* utt);\n";

char includes[] =
"/**********************************************/\n"
"/* This file is generated by the yyak utility */\n"
"/**********************************************/\n"
"\n"
"#include <stdio.h>\n"
"#include <string.h>\n"
"#include <stdlib.h>\n"
"#include <sys/uio.h>\n"
"#include \"include/odyssey.h\"\n"
"#include \"include/odytypes.h\"\n";

char fileHeader[] = 
"\n"
"char*  args[256];\n"
"int    num_args;\n"
"\n"
"int is_function (char* keyword) {\n"
"  return (!strcmp (&keyword[strlen(keyword)-2], \"()\"));\n"
"}\n"
"\n"
"int is_all (char* keyword) {\n"
"  return (!strcmp (keyword, \"*\"));\n"
"}\n"
"\n"
"int is_word (char* keyword) {\n"
"  return (!is_all(keyword) && !is_function(keyword));\n"
"}\n";

char parseFnHeader[] = "\n"
"char* parse_%s (char* word)\n"
"{\n";

char parseFnBody[] = 
"  if (!strcmp (word, \"%s\")) \n"
"    return \"%s\";\n";

char parseFnTrailer[] = "\n"
"  return 0;\n"
"}\n";

char parseHeader[] = "\n"
"int parse (char** words, int num_words, char** keywords, int num_keywords)\n"
"{\n"
"  if (num_keywords == 0 && num_words == 0) \n"
"    return 1; /* Success */\n"
"\n"
"  if (num_words == 0 && is_all (keywords[0])) \n"
"    return (parse (words, num_words, keywords+1, num_keywords-1));\n"
"\n"
"  if (num_words == 0 || num_keywords == 0) \n"
"    return 0; /* Failure */\n"
"\n"
"  if (is_word (keywords[0])) {\n"
"    if (!strcmp (words[0], keywords[0])) {\n"
"      return (parse (words+1, num_words-1, keywords+1, num_keywords-1));\n"
"    } else {\n"
"      return 0; /* Failure */\n"
"    } /* endif */\n"
"  } \n"
"\n"
"  if (is_all (keywords[0])) { /* all */\n"
"    if (parse (words, num_words, keywords+1, num_keywords-1)) {\n"
"      return 1; /* Success */\n"
"    } else {\n"
"      return (parse (words+1, num_words-1, keywords, num_keywords));\n"
"    } /* endif */\n"
"  } /* endif */\n";

char parseBody[] = "\n"
"  if (!strcmp (keywords[0], \"%s()\")) {\n"
"    args[num_args] = parse_%s (words[0]);\n"
"    if (args[num_args]) {\n"
"      num_args++;\n"
"      return (parse (words+1, num_words-1, keywords+1, num_keywords-1));\n"
"    } else {\n"
"      return 0; /* Failure */\n"
"    } /* endif */\n"
"  } /* endif */\n";

char parseTrailer[] = "\n"
"  return 0; /* Failure */\n"
"}\n";

char parseMainHeader[] = "\n"
"int parse_main (char** words, int num_words)\n"
"{\n"
"  char* keywords[256];\n";

char parseMainTransHeader[] = "\n"
"  num_args = 0;\n";

char parseMainTransBody[] = 
"  keywords[%d] = \"%s\";\n";

char parseMainTransTrailer[] = 
"  if (parse (words, num_words, keywords, %d)) {\n"
"    %s;\n"
"    return 1;\n"
"  } /* endif */\n";

char parseMainTrailer[] = "\n"
"  return 0;\n"
"} \n";

char parseUtt[] = "\n"
"/* parseUtt() parses the specified utteance and executes any command which\n"
"corresponds to a translation of the utterance */\n"
"void parseUtt (char* utt)\n"
"{\n"
"  char** words = 0;\n"
"  int n = 0; \n"
"  char **ap, *p, *val;\n"
"  \n"
"  /* Parse out words from utterance */\n"
"  NALLOC (words, char *, strlen(utt));\n"
"  ap = words;\n"
"  for (p = utt; p != NULL; ) {\n"
"    while ((val = strsep(&p, \" \")) != NULL && *val == '\\0');\n"
"      if (val) {\n"
"        *ap++ = val;\n"
"        n++;\n"
"      } /* endif */\n"
"  } /* endfor */\n"
"  *ap = 0;\n"
"\n"
"  parse_main (words, n);\n"
"\n"
"  FREE (words); \n"
"}\n";

int getLine (char* line, int len, FILE* fp)
{
  char* p;
  int idx, i;

  do {
    p = fgets (line, len, fp);
    if (!p) {
      return (-1); /* eof */
    } /* endif */
    
    /* Remove trailing whitespaces */
    for (idx = 0; line[idx] && isspace(line[idx]); idx++);
    for (i = 0; i + idx <= strlen (line); i++)
      line[i] = line[i+idx];
    /* Remove trailing whitespaces */
    for (i = strlen(line)-1; i >= 0 && isspace(line[i]); i--);
    line[i+1] = '\0';
  } while (line[0] == '\0' || line[0] == '%');
  
  return (0);
}

char global_LGM[100];
char* globalp_LGM;

int parseTable (FILE* fp, TransTable* pTable)
{
  TransLine* newLine;
  TransLine* lastLine = 0;
  char line[256];
  char* sep;
  char** ap;
  char* p, *val;

  do {

    if (getLine (line, 255, fp) < 0) {
      printf ("Syntax error: unexpected EOF\n");
      return (-1);
    } /* endif */
    
    if (line[0] == '<') {
      
      if (line[strlen(line)-1] == '>' && line[1] == '/') {
	if (strlen(line) == strlen (pTable->name) + 3 &&
	    !strncmp (pTable->name, line+2, strlen(pTable->name))) {
	  return (0);
	} else {
	  printf ("Syntax error: %s\n", line);
	  return (-1);
	} /* endif */
      } else {
	printf ("Syntax error: %s\n", line);
	return (-1);
      } /* endif */
      
    } else {

      /* Parse out the line - this should be a command translation*/
      sep = strchr (line, ':');
      if (sep) {

	*sep = '\0';
	ALLOC (newLine, TransLine);
	/* Eliminate leading white-space */
	for (sep = sep+1; isspace (*sep); sep++);
	newLine->command = strdup (sep);

	NALLOC (newLine->keywords, char *, strlen(line));
	newLine->num_keywords = 0;
	ap = newLine->keywords;
	for (p = line; p != NULL; ) {
	  while ((val = strsep(&p, " \t")) != NULL && *val == '\0');
	  if (val) {
	    NALLOC (*ap, char, 256);       
	    strcpy (*ap++, val);
	    newLine->num_keywords++;
	  } /* endif */
	} /* endfor */

	if (lastLine) {
	  lastLine->next = newLine;
	} else {
	  pTable->translations = newLine;
	} /* endif */
	lastLine = newLine;

      } else {
	printf ("Syntax error: %s\n", line);
	return (-1);
      } /* endif */	
      
    } /* endif */
    
  } while (1);
  
  return (-1);
}

void printTransFile (TransInfo* info) 
{
  TransLine* line;
  TransTable* table = info->tables;
  TransHeader* header;
  int i;

  printf ("Header: \n");
  for (header = info->headers; header; header = header->next) 
    printf ("%s\n", header->header);
  printf ("\n");

  while (table) {
    printf ("Table: %s\n", table->name);
    for (line = table->translations; line; line = line->next) {
      printf ("Keywords: ");
      for (i = 0; i < line->num_keywords; i++) {
	printf ("%s", line->keywords[i]);
	if (i < line->num_keywords-1) printf (",");
      } /* endfor */
      printf (" triggers command %s\n", line->command);
    } /* endfor */
    table = table->next;
  } /* endwhile */
}

TransInfo* parseTransFile (char* filename)
{
  FILE* fp;
  char line[256];
  TransTable* newTable;
  TransHeader* newHeader = 0;
  TransHeader* lastHeader = 0;
  TransInfo* info = 0;
  int done;

  ALLOC (info, TransInfo);
  info->tables = 0;
  info->headers = 0;

  fp = fopen (filename, "r");
  if (!fp) {
    printf ("couldn't open input file: %s\n", filename);
    exit (1);
  } /* endif */

  do {

    if (getLine (line, 255, fp) < 0) {
      fclose (fp);
#if 0
      printTransFile (info);
#endif
      return (info);
    } /* endif */

    if (line[0] == '<') {

      if (!strcmp (line, "<header>")) {

	done = 0;
	do {

	  if (fgets (line, 255, fp) == 0) {
	    printf ("Syntax error: Unexpected EOF\n");
	    fclose (fp);
	    return (NULL);
	  } /* endif */
	  if (!strncmp (line, "</header>", 9)) {
	    done = 1;
	  } else {
	    ALLOC (newHeader, TransHeader);
	    newHeader->header = strdup (line);
	    newHeader->next = 0;
	    if (lastHeader) {
	      lastHeader->next = newHeader;
	    } else {
	      info->headers = newHeader;
	    } /* endif */
	    lastHeader = newHeader;
	  } /* endif */
	    
	} while (!done);

      } else if (line[strlen(line)-1] == '>' && line[1] != '/') {

	ALLOC (newTable, TransTable);
	NALLOC (newTable->name, char, strlen(line) - 1);
	strncpy (newTable->name, line+1, strlen(line) - 2);
	newTable->name[strlen(line)-1] = '\0';
	newTable->translations = 0;
	if (info->tables) {
	  newTable->next = info->tables;
	} /* endif */
	info->tables = newTable;

	if (parseTable (fp, newTable) < 0) return (0);

      } else {
	printf ("Syntax Error: %s\n", line);
	return (0);
      } /* endif */
    } /* endif */

  } while (1);

  return (0);
}

void showFormat ()
{
  printf ("format: yyak <translation table>\n");
  printf ("        example: yyak nettext.yyak would produce nettext.[ch] from nettext.yyak\n");
  exit (0);
}

int main (int argc, char* argv[]) 
{
  TransInfo* info;
  TransHeader* header;
  TransTable* table;
  TransLine* trans;
  char* inputFile;
  char* cFile;
  char* hFile;
  char* pch;
  char* pfile;
  FILE* fp;
  int i;
    
  if (argc != 2) {
    showFormat ();
  } /* endif */

  inputFile = argv[1];

  /* Write output files to the current directory */
  for (pch = inputFile, pfile = inputFile; *pch; pch++) {
    if (*pch == '/') {
      pfile = pch+1;
    } /* endif */
  } /* endfor */

  NALLOC (cFile, char, strlen(pfile)+3);
  strcpy (cFile, pfile);
  strcat (cFile, ".c");

  NALLOC (hFile, char, strlen(pfile)+3);
  strcpy (hFile, pfile);
  strcat (hFile, ".h");

  info = parseTransFile (inputFile);
  if (info) {
  
    fp = fopen (hFile, "w");
    if (!fp) {
      printf ("couldn't open %s\n", hFile);
      exit (1);
    } /* endif */

    fprintf (fp, headerFile);

    fclose (fp);

    fp = fopen (cFile, "w");
    if (!fp) {
      printf ("couldn't open %s\n", cFile);
      exit (1);
    } /* endif */
    
    fprintf (fp, includes);

    /* Write out user-specifie header */
    for (header = info->headers; header; header = header->next) {
      fprintf (fp, "%s\n", header->header);
    } /* endfor */

    fprintf (fp, fileHeader);
    
    /* Write out parse tables for functions */
    for (table = info->tables; table; table = table->next) {
      if (strcmp (table->name, "main")) {
	fprintf (fp, parseFnHeader, table->name);
	for (trans = table->translations; trans; trans = trans->next) {
	  if (trans->num_keywords != 1) {
	    printf ("function %s must take exactly one argument\n", 
		    table->name);
	    exit (1);
	  } /* endif */
	  fprintf (fp, parseFnBody, trans->keywords[0], trans->command);
	} /* endfor */
	fprintf (fp, parseFnTrailer);
      } /* endif */
    } /* endif */
    
    /* Write out parse function */
    fprintf (fp, parseHeader);
    for (table = info->tables; table; table = table->next) {
      if (strcmp (table->name, "main")) {
	fprintf (fp, parseBody, table->name, table->name);
      } /* endif */
    } /* endif */
    fprintf (fp, parseTrailer);

    /* write out parse_main function */
    for (table = info->tables; table; table = table->next) {
      if (!strcmp (table->name, "main")) {
	fprintf (fp, parseMainHeader);
	for (trans = table->translations; trans; trans = trans->next) {
	  fprintf (fp, parseMainTransHeader);
	  for (i = 0; i < trans->num_keywords; i++) {
	    fprintf (fp, parseMainTransBody, i, trans->keywords[i]);
	  } /* endif */
	  fprintf (fp, parseMainTransTrailer, trans->num_keywords, 
		   trans->command);
	} /* endfor */
	fprintf (fp, parseMainTrailer);
      } /* endif */
    } /* endif */
    
    fprintf (fp, parseUtt);
    fclose (fp);
    
  } /* endif */
  
  return (0);
}
	   
