#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _ODYSSEY_H_
#define _ODYSSEY_H_

/* FreeBSD hack */
#ifdef KERNEL
#define _KERNEL
#endif

/*
   odyssey.h - odyssey-aware application definitions

   Include file for odyssey-aware applications.  It contains only
   knowledge of generic (system-wide) functionality.  Any
   type-specific functionality is included in that type's header
   file.  Broken into the following sections:

   RESOURCES      - what generic resources are defined for the system
   REQUESTS       - how requests are made, cancelled
   TSOP           - how TSOP's are (generically) performed
                    (individual spec's for TSOP's are defined for
		     each Odyssey type.)
*/

#include <errno.h>

/* 
 * Need to pick up error codes from the api: applications should not
 * use those system calls directly.  It is only here for the error
 * codes
 */
#include <kernel/vfs/includes/odyapi.h>

/*
 * We need errno.h because we are going to be extending the set of
 * acceptable error numbers.  We should technically provide a new
 * perror/strerror set of routines to handle these.
 */

/************************************************* RESOURCES */

#define ODY_UNUSED               0
#define ODY_GENERIC_INTERFACE   68
#define ODY_NO_TYPE              0

struct ody_vers {
    unsigned int  ov_gs;       /* Version of generic interface known by app */
    unsigned int  ov_type;     /* Which type we mean to use */
    unsigned int  ov_tsvnum;   /* Version of type-specific interface known */
};


/* 
 * I'd like this to be an enum, but each type has to be able to extend
 * the set in possibly overlapping ways.
 */
    
#define ODY_GR_NETBW           101
#define ODY_GR_NETLAT          102
#define ODY_GR_DCACHE          103
#define ODY_GR_CPU             104
#define ODY_GR_POWER           105
#define ODY_TEST_1             201
#define ODY_TEST_2             202
#define ODY_MIN_TYPE_RSRC      400

/* All type-specific resources must be > ODY_MIN_TYPE_RSRC */

/************************************************* REQUESTS */

typedef void (*ody_req_fn_t) __P((int reqid, unsigned int rsrc, int val));

struct ody_req_des {
    unsigned int     ord_resource;    /* resource number for this request */
    struct ody_vers  ord_version;     /* which version of the interface */
    int              ord_low;         /* lower bound on resource value */
    int              ord_high;        /* upper bound */
    ody_req_fn_t     ord_fp;          /* function to call via upcall */
};


#ifndef _KERNEL
#ifndef VICEROY
#ifndef WARDEN

extern int ody_request  __P((char *path, struct ody_req_des *req, int *res));
extern int ody_frequest __P((int fd, struct ody_req_des *req, int *res));
extern int ody_cancel   __P((int reqid));

#endif /* WARDEN */
#endif /* VICEROY */
#endif /* _KERNEL */

/************************************************* TSOP'S */

#ifndef _KERNEL
#ifndef VICEROY
#ifndef WARDEN

extern int ody_tsop   __P((char *path, struct ody_vers *vers, unsigned int op,
			   size_t argsz, void *arg, size_t *retsz, void *ret));
extern int ody_ftsop  __P((int fd, struct ody_vers *vers, unsigned int op,
			   size_t argsz, void *arg, size_t *retsz, void *ret));
extern int ody_multif __P((struct ody_vers *vers, unsigned int op,
			   size_t argsz, void *arg, size_t *retsz, void *ret));

#endif /* WARDEN */
#endif /* VICEROY */
#endif /* _KERNEL */

#endif /*_ODYSSEY_H_*/
