#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _VICEROY_H_
#define _VICEROY_H_

typedef enum {
  odyssey = 1,
#ifdef INSTRUMENTED
  blind_optimism,
  laissez
#endif /* INSTRUMENTED */
} report_mode;

/* 
 * Resource requests by entities linked with the Viceroy.
 * 
 * Place a request for resource 'resource", in the context of fid
 * "op", placed by warden "wp", bounds [low .. high].  If the
 * resource exceeds these bounds, call the function "callback".
 * (The type signature for callback can be found in odyssey.h)
 *
 * The sencond field of argp, argval is passed to the callback
 * function if the request is triggered.  The callback function is
 * responsible for deallocating the space which it is passed.  The
 * first field of argp, free_fn, will be called if the request is
 * cancelled, with the argp->argval field as an argument.
 *
 * Implemented in rsrc.c
 * 
 * Can return:
 *
 * EINVAL: the version number, resource number, or ttype was bogus.
 * ODY_ENOTINWINDOW: resource already out-of-bounds: current value in
 *                   "value"
 *
 * OUT parameters:
 * cancelled: Meaningful only if 0 or ODY_ENOTINWINDOW returned.
 *            0 if no outstanding request cancelled by this one, 
 *            the cancelled reqid otherwise.
 * value: meaningful only if ODY_ENOTINWINDOW returned.
 *        In such cases, the current value of the resource is here.
 * reqid: meaningful only if 0 returned.
 *        The request id of the placed request.
 *            */

extern int vr_rsrc_request  __P((struct warden *wp, 
				 struct viceroy_onode *op,
				 unsigned int resource, 
				 unsigned int gvers, 
				 int low, 
				 int high, 
				 REQFNP callback, /* typedef in ds_rrlist.h */
				 ds_reqarg_t *argp,
				 int *cancelled, 
				 int *value,
				 int *reqid));

/*
 * Cancel an outstanding request.  Exactly like the user-level
 * appliation case.
 */
int vr_rsrc_cancel        __P((int reqid));



/*
 * Return the current value of a resource.  Assert if the resource or
 * version stamp is bogus.  
 */

extern int vr_rsrc_value    __P((struct viceroy_onode *op,
				 unsigned int resource,
				 unsigned int gvers));


#ifdef BW_TOOL
/*
 * Given a connid, return the current bandwidth estimation.
 */

extern int vr_rsrc_getbw    __P((cmgr_id_t cid));
#endif

#endif /* _VICEROY_H_ */
