package odyssey.api;
import  java.io.*;
import  java.util.*;
import  odyssey.util.*;

public class OdysseyAPI implements Runnable {
    /*
     * PATH
     *   * get this to compile
     *   * put a dummy C backend in to verify it works
     *   * upgrade kernel version and get odyssey built
     *   * install remaining odyssey crap
     *   * send buffers along
     *   * change current backend into "print this call" routines
     *   * real testing code
     *   * real C backend (except for details below)
     *   * use the pass-by-reference integer in request()
     *   * call a java upcall
     *   * track upcalls
     *   * send buffers back
     *   throw proper exceptions
     *   deal with err numbers (through exceptions, probably)
     *
     * some thoughts:
     *   there's no good way to check if the type signatures these
     *   emulate change. i'm doing this by encoding version numbers
     *   explicitly; if the underlying stuff changes, this stuff will
     *   stop working until someone updates the version numbers.
     *   the person doing the check needs to make sure things are
     *   still fixed.
     *
     */
    static Hashtable handlers;

    static int request(String         path,
		       OdyRequestDesc req,
		       IntegerRef     res)
    {
	int result;

	result = _request(path, req, res);

	if (result == 0) {
	    // if the result is 0, the request was successful, and
	    // res contains the request ID. if it was unsuccessful,
	    // res will contain the current value of the resource.
	    handlers.put(new Integer(res.value), req);
	}

	return result;
    }

    static public native int _request(String         path,
				      OdyRequestDesc req,
				      IntegerRef     res);

    static public native int cancel(int reqid);

    static public native int tsop (String  path,
				   OdyVers vers,
				   int     op,
				   byte    arg[],
				   byte    ret[]);

    public void run()
    {
	/*
	 * This is just a random thread, meant to stress test the
	 * application.
	 */

	while (true) {
	    synchronized(this) {
		try {
		    Thread.currentThread().sleep(5000);
		    System.out.println("tock");
		} catch (InterruptedException e) {
		    System.err.println(e); // XXX DEBUG
		}
	    }
	}
    }

    /*
     * upcall dispatching
     */
    static public void handleUpcall(int reqid, int rsrc, int val)
    {
	try {
	    OdyRequestDesc req =
		(OdyRequestDesc) handlers.remove(new Integer(reqid));
	    req.listener.handleUpcall(reqid, rsrc, val);
	} catch (Exception e) {
	    System.err.println("Got exception " + e);
	}
    }

    /*
     * testing code
     */
    public void test()
    {
	System.out.println("if you see this, the shared libraries loaded " +
			   "properly.");
    }

    static
    {
	/* "combined" combines multiple libraries, including RPC2, 
	   ody, and odysys. */
	System.loadLibrary("combined");
	handlers = new Hashtable();
    }

    public static void main(String[] args)
    {
	OdysseyAPI foo;

	foo = new OdysseyAPI();

	foo.test();
    }
}
