#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <jni.h>
#include "odyssey_api_OdysseyAPI.h"
#include <stdlib.h>
#include <stdio.h>

#include "include/odyssey.h"
#include "javaody_util.h"

/*
 * Stubs for Odyssey API calls (request, cancel, tsop).
 * This file should implement only the function protypes generated
 * by javah -jni on odyssey.api.OdysseyAPI
 *
 * NOTE: until there's a good way to extract fd's from Java,
 *       ftsop and frequest are unimplementable.
 */

/*
 * Class:     odyssey_api_OdysseyAPI
 * Method:    request
 * Signature: (Ljava/lang/String;Lodyssey/api/OdyRequestDesc;I)V
 */
JNIEXPORT jint JNICALL Java_odyssey_api_OdysseyAPI__1request
  (JNIEnv *env, jobject this,
   jstring str,
   jobject req,
   jobject resRef)
{
    struct ody_req_des  request;
    jclass              cls;
    jfieldID            fid;
    int                 res;
    int                 result;

    const char *path = (*env)->GetStringUTFChars(env, str, 0);

    get_ody_req_des(env, req, &request);

    cls = (*env)->GetObjectClass(env, resRef);
    fid = (*env)->GetFieldID(env, cls, "value", "I");
    if (fid == 0)
	goto exception;
    res = (*env)->GetIntField(env, resRef, fid);

    /* print_request(path, &request, &res); */
    result = ody_request((char *) path, &request, &res);

    (*env)->SetIntField(env, resRef, fid, res);

    (*env)->ReleaseStringUTFChars(env, str, path);

    return result;

 exception:
    printf("XXX bogus thing happened\n");
    return -1;
}

/*
 * Class:     odyssey_api_OdysseyAPI
 * Method:    cancel
 * Signature: (I)V
 */
JNIEXPORT jint JNICALL Java_odyssey_api_OdysseyAPI_cancel
  (JNIEnv *env, jobject this,
   jint    reqid)
{
    int result;

    /* print_cancel(reqid); */
    result = ody_cancel(reqid);

    return result;
}

/*
 * Class:     odyssey_api_OdysseyAPI
 * Method:    tsop
 * Signature: (Ljava/lang/String;Lodyssey/api/OdyVers;II[BI[B)V
 */
JNIEXPORT jint JNICALL Java_odyssey_api_OdysseyAPI_tsop
  (JNIEnv *env, jobject this,
   jstring    str,
   jobject    vers,
   jint       op,
   jbyteArray jarg,
   jbyteArray jret)
{
    struct ody_vers  version;
    const char      *path;
    int              result;
    int              argsz, retsz;
    int              maxretsz;
    void            *arg, *ret;

    path = (*env)->GetStringUTFChars(env, str, 0);

    get_ody_version(env, vers, &version);

    argsz = (*env)->GetArrayLength(env, jarg);
    retsz = (*env)->GetArrayLength(env, jret);

    maxretsz = retsz;

    if (argsz) {
	arg = (*env)->GetByteArrayElements(env, jarg, NULL);
    } else {
	arg = NULL;
    }

    if (retsz) {
	ret = (void *) malloc(retsz * sizeof(void));
    } else {
	ret = NULL;
    }

    result = ody_tsop((char *) path, &version, op, argsz, arg, &retsz, ret);

    (*env)->ReleaseStringUTFChars(env, str, path);

    if (retsz > 0) {
	(*env)->SetByteArrayRegion(env, jret, 0,
				   ((retsz < maxretsz) ? retsz : maxretsz),
				   ret);
    }

    free(ret);

    return result;
}
