#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <jni.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "include/odyssey.h"
#include "javaody_util.h"

/*
 * Utility functions for converting Java structures into C structures.
 */

void get_ody_version(JNIEnv          *env,
		     jobject          vers,
		     struct ody_vers *result)
{
    jclass   cls = (*env)->GetObjectClass(env, vers);
    jfieldID fid;
    jint     value;

    /* verify the object is of the right type */
    /* XXX how do i do this?? */

    /* get the values */
    fid = (*env)->GetFieldID(env, cls, "gs", "I");
    if (fid == 0)
	goto exception;
    value = (*env)->GetIntField(env, vers, fid);
    result->ov_gs = value;

    fid = (*env)->GetFieldID(env, cls, "type", "I");
    if (fid == 0)
	goto exception;
    value = (*env)->GetIntField(env, vers, fid);
    result->ov_type = value;

    fid = (*env)->GetFieldID(env, cls, "tsvnum", "I");
    if (fid == 0)
	goto exception;
    value = (*env)->GetIntField(env, vers, fid);
    result->ov_tsvnum = value;

    return;

 exception:
    // XXX handle error condition. are these acceptable defaults?
    printf("BOGUS THING HAPPENED.\n");
    result->ov_gs     = -1;
    result->ov_type   = -1;
    result->ov_tsvnum = -1;
    return;
}

void get_ody_req_des(JNIEnv *env,
		      jobject req,
		      struct ody_req_des *request)
{
    jclass   cls = (*env)->GetObjectClass(env, req);
    jfieldID fid;
    jint     value;
    jobject  vers;

    /* verify the object is of the right type */
    /* XXX how do i do this?? */

    /* get the values */
    fid = (*env)->GetFieldID(env, cls, "resource", "I");
    if (fid == 0)
	goto exception;
    value = (*env)->GetIntField(env, req, fid);
    request->ord_resource = value;

    fid = (*env)->GetFieldID(env, cls, "version", "Lodyssey/api/OdyVers;");
    if (fid == 0)
	goto exception;
    vers = (*env)->GetObjectField(env, req, fid);
    get_ody_version(env, vers, &(request->ord_version));

    fid = (*env)->GetFieldID(env, cls, "low", "I");
    if (fid == 0)
	goto exception;
    value = (*env)->GetIntField(env, req, fid);
    request->ord_low = value;

    fid = (*env)->GetFieldID(env, cls, "high", "I");
    if (fid == 0)
	goto exception;
    value = (*env)->GetIntField(env, req, fid);
    request->ord_high = value;

    request->ord_fp = upcall_handler;

    return;

 exception:
    // XXX handle error condition. are these acceptable defaults?
    printf("BOGUS THING HAPPENED.\n");
    request->ord_resource = -1;
    request->ord_version.ov_gs     = -1;
    request->ord_version.ov_type   = -1;
    request->ord_version.ov_tsvnum = -1;
    request->ord_low      = -1;
    request->ord_high     = -1;
    return;
}

void upcall_handler(int reqid, unsigned int rsrc, int val)
{
    JavaVM *vm;
    JNIEnv *env;
    jsize   total;
    jclass    class;
    jmethodID method;

    int result;

    result = JNI_GetCreatedJavaVMs(&vm, 1, &total);
    printf("get vms returned %d\n", result);

    result = (*vm)->AttachCurrentThread(vm, &env, NULL);
    printf("attach current thread returned %d\n", result);

    assert(result == 0);

    class = (*env)->FindClass(env, "odyssey/api/OdysseyAPI");
    printf("findclass result: %d\n", (int) class);

    method = (*env)->GetStaticMethodID(env, class, "handleUpcall", "(III)V");
    printf("getmethod result: %d\n", (int) method);

    (*env)->CallStaticVoidMethod(env, class, method, reqid, rsrc, val);

    result = (*vm)->DetachCurrentThread(vm);
    printf("detach current thread returned %d\n", result);

#ifdef 0
    JNIEnv *env = watcher;

    printf("got upcall!\n");

    (*env)->Throw(env, up_exception);
#endif 0
#ifdef 0    
    jclass    class;
    jmethodID method;

    class = (*env)->FindClass(env, "odyssey/api/OdysseyAPI");
    printf("findclass result: %d\n", (int) class);

    method = (*env)->GetStaticMethodID(env, class, "handleUpcall", "(III)V");
    printf("getmethod result: %d\n", (int) method);

    (*env)->CallStaticVoidMethod(env, class, method, reqid, rsrc, val);
#endif /* 0 */
}

/*
 * Utility functions for printing various things.
 */

void print_ody_version(struct ody_vers *version)
{
    printf("(ody vers: %d, %d, %d)",
	   version->ov_gs,
	   version->ov_type,
	   version->ov_tsvnum);
}

void print_ody_req_des(struct ody_req_des *request)
{
    printf("(ody req des: %d ", request->ord_resource);
    print_ody_version(&(request->ord_version));
    printf(" %d %d)", request->ord_low, request->ord_high);
}

void print_array(void *arg, int argsz)
{
    int  loop;
    char *buffer;

    buffer = (char *) arg;

    printf("\"");
    for (loop = 0; loop < argsz; loop++) {
	printf("%c", buffer[loop]);
    }
    printf("\"");
}

void print_request(const char         *path,
		   struct ody_req_des *req,
		   int                *res)
{
    printf("request: %s ", path);
    print_ody_req_des(req);
    printf(" %d\n", *res);
}

void print_cancel(int reqid)
{
    printf("cancel: %d\n", reqid);
}

void print_tsop(char            *path,
		struct ody_vers *vers,
		unsigned int     op,
		size_t           argsz,
		void            *arg,
		size_t          *retsz,
		void            *ret)
{
    printf("tsop: ");
    printf("%s ", path);
    print_ody_version(vers);
    printf(" %d (%d) ", op, argsz);
    print_array(arg, argsz);
    printf(" (%d) [not filled yet]\n", *retsz);
}
