package odyssey.awt;

import  java.awt.*;
import  java.awt.image.*;
import  java.util.*;
import  java.io.*;

/**
 * This class addresses the single most irritating thing about Images,
 * which is that sometimes you want to just wait and get the width
 * and height of the image back synchronously, instead of playing around
 * with all of this ImageObserver crud.
 */

public class BlockingImage implements ImageObserver, java.io.Serializable {
    public  transient Image     theImage;
    private           String    name;

    public BlockingImage(Image image, String name)
    {
	this.theImage = image;
	this.name     = name;
    }

    private void writeObject(java.io.ObjectOutputStream stream)
            throws IOException
    {
	stream.writeObject(name);
    }

    private void readObject(java.io.ObjectInputStream stream)
            throws IOException, ClassNotFoundException
    {
	name = (String) stream.readObject();

	// This is a terrible hack to get a toolkit. But, y'know what?
	// The AWT sucks, so I have to resort to this kind of stupid
	// stuff.
	Button foo = new Button();
	Toolkit tk = foo.getToolkit();

	theImage = tk.getImage(name);
    }

    public synchronized int getWidth()
    {
	int width = theImage.getWidth(this);
	if (width != -1) {
	    return width;
	}

	do {
	    try {
		wait();
	    } catch (InterruptedException e) {
	    }
	    width = theImage.getWidth(this);
	} while (width == -1);
	
	return width;
    }

    public synchronized int getHeight()
    {
	int height = theImage.getHeight(this);
	if (height != -1) {
	    return height;
	}

	do {
	    try {
		wait();
	    } catch (InterruptedException e) {
	    }
	    height = theImage.getHeight(this);
	} while (height == -1);

	return height;
    }

    public boolean imageUpdate(Image img,
			       int   infoflags,
			       int   x,
			       int   y,
			       int   width,
			       int   height)
    {
	boolean result = false;

        if ((infoflags & ImageObserver.WIDTH) != 0) {
	    synchronized (this) {
		notifyAll();
	    }
	    result = true;
	}

        if ((infoflags & ImageObserver.HEIGHT) != 0) {
	    synchronized (this) {
		notifyAll();
	    }
	    result = true;
	}

	if ((infoflags & ImageObserver.ALLBITS) != 0) {
	    result = true;
	}

	return result;
    }
}
