package odyssey.guestroom;

import java.rmi.*;
import java.rmi.registry.*;
import java.rmi.server.UnicastRemoteObject;
import java.net.*;
import java.io.*;

public class GuestServer
        extends UnicastRemoteObject
        implements GuestInterface,Runnable
{
    private        String       name;
    private        Object       bar;
    private        ServerSocket srvSock;
    private static ClassServer  classSrv;
    
    public GuestServer(String s) throws RemoteException, IOException
    {
	super();
	name = s;

	srvSock = new ServerSocket(0);
	(new Thread(this)).start();
    }

    public void run()
    {
	Socket incoming = null;
	NetworkObjectInputStream stream;

	// read off of the server socket
	while(true) {
	    try {
		incoming = srvSock.accept();
		stream = new NetworkObjectInputStream(incoming);
		int port = stream.readInt();
		stream.setURL(port);
		bar = stream.readObject();
	    } catch (Exception e) {
		System.err.println("in GuestServer.run()" + e);
	    } finally {
	        try {
                    OutputStream outStream = incoming.getOutputStream();
                    outStream.write('z'); // XXX probably not needed
	            outStream.close();
	        } catch (IOException e) {
	        }
	    }
	}
    }

    public synchronized static void registerObject(GuestInterface srv,
						   Object guest)
        throws RemoteException,
	       java.net.UnknownHostException,
	       IOException
    {
	System.out.println("registerObject: registering object");

	if (classSrv == null) {
	    classSrv = new ClassServer();
	}

	SocketDesc sockd = srv.getPort();
	Socket     sock  = new Socket(sockd.host,sockd.port);

	ObjectOutputStream stream =
	    new ObjectOutputStream(sock.getOutputStream());
	stream.writeInt(classSrv.getPort());
	stream.writeObject(guest);
	InputStream inputStream = sock.getInputStream();

	// wait for other side to close connection
	try {
	    int result = 0;
	    while(result != -1) {
		result = inputStream.read();
	    }
	} catch (IOException e) {
	}
	System.out.println("registerObject: done registering");
    }

    public SocketDesc getPort() throws java.rmi.RemoteException,
	                               java.net.UnknownHostException
    {
	SocketDesc result = new SocketDesc();
	result.host = InetAddress.getLocalHost();
	result.port = srvSock.getLocalPort();

	return result;
    }
    
    public Object getObject() throws java.rmi.RemoteException
    {
	return bar;
    }

    public static void verifyRegistry()
    {
	// make sure a registry exists
	try {
	    LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
	} catch (RemoteException e) {
	    // registry already exists; do nothing
	}
    }
    
    public static void main(String args[])
    {
	// Create and install a security manager
	System.setSecurityManager(new GuestSecurityManager());
	
	verifyRegistry();
	
	try {
	    GuestServer obj = new GuestServer("GuestServer");
	    Naming.rebind("//hades/GuestServer", obj);
	    System.out.println("GuestServer bound in registry");
	} catch (Exception e) {
	    System.out.println("GuestServer err: " + e.getMessage());
	    e.printStackTrace();
	}
    }
}
