package odyssey.map;

import  java.awt.*;
import  java.awt.event.*;
import  java.io.*;
import  java.util.*;

public class MapEdit extends Frame implements ActionListener, MouserListener {
    Map        map;
    MapMouser  mm;
    ScrollPane sp;
    MenuItem   save;

    MenuItem   undo;
    Vector     undolist = null;
    boolean    undid = true;

    public MapEdit(String name, boolean usefifo)
    {
	super(name);

	Image image;

	// set up the menu bar
	MenuBar  mb   = new MenuBar();
	Menu     menu = new Menu("File", true);
	MenuItem mi;

	mb.add(menu);

	mi = new MenuItem("Load...");
	mi.addActionListener(this);
	menu.add(mi);

	mi = new MenuItem("Save...");
	mi.addActionListener(this);
	menu.add(mi);
	save = mi;
	save.setEnabled(false);

	// XXX this is an evil short term hack
	mi = new MenuItem("Write Config File");
	mi.addActionListener(this);
	menu.add(mi);

	mi = new MenuItem("-");
	menu.add(mi);

	mi = new MenuItem("Quit", new MenuShortcut('q'));
	mi.addActionListener(this);
	menu.add(mi);

	menu = new Menu("Edit", true);
	mi   = new MenuItem("Undo", new MenuShortcut('z'));
	mi.addActionListener(this);

	undo = mi;
	undo.setEnabled(false);

	menu.add(mi);
	mb.add(menu);

	setMenuBar(mb);

	// get the map image
        Toolkit tk = getToolkit();
	image = tk.getImage(name);

	try {
	    map = new Map(image, name, "config-file", 5, 5);
	} catch (IOException e) {
	    System.err.println(e);
	    System.exit(-1);
	}

	map.setVisible(true);

	/* sp = new ScrollPane();
	sp.add(map);
	sp.setSize(540, 400);
	this.add("Center", sp); */
	this.add("Center", map);
	
	Panel east = new Panel();
	Label status = new Label("A REALLY LONG STRING", Label.LEFT);
	this.add("East", east);
	east.setLayout(new GridLayout(3, 1));

	mm = new MapMouser(status, this, map, usefifo);

	Button b1 = new Button("Paint");
	// Button b2 = new Button("Wizard of Oz");
	Button b2 = new Button("Repaint regions");
	east.add(b1);
	east.add(b2);
	b1.addActionListener(mm);
	b2.addActionListener(mm);
	east.add(status);

	map.addMouseListener(mm);
	map.addMouseMotionListener(mm);

	// set up color bar
	ColorBar cp = new ColorBar(mm);

	this.add("South", cp);

	// Dimension d = map.getSize();
	// d.height += 100;
	// d.width  += 200;

	setSize(780, 400);
	validate();

	this.setVisible(true);
    }

    public void actionPerformed(ActionEvent e)
    {
	String command = e.getActionCommand();

	if (command.equals("Load...")) {
	    if (save.isEnabled()) {
		System.err.println("should've saved first"); // XXX
	    }
	    try {
		FileInputStream istream = new FileInputStream("test.map");
		ObjectInputStream p = new ObjectInputStream(istream);
		Map oldMap = map;
		map = (Map) p.readObject();
		istream.close();
		save.setEnabled(false);

		// sp.add(map);
		remove(oldMap);
		add("Center", map);
		map.addMouseListener(mm);
		map.addMouseMotionListener(mm);
		mm.setNewMap(map);
		validate();
		repaint();
	    } catch (IOException err) {
		System.err.println("Error reading file: " + err); // XXX
	    } catch (ClassNotFoundException err) {
		System.err.println("Error reading file: " + err); // XXX
	    }
	    // System.out.println("You want I should load?");
	    return;
	}

	if (command.equals("Save...")) {
	    try {
		FileOutputStream ostream = new FileOutputStream("test.map");
		ObjectOutputStream p = new ObjectOutputStream(ostream);
		p.writeObject(map);
		p.flush();
		ostream.close();
		save.setEnabled(false);
	    } catch (IOException err) {
		System.err.println("Error writing file: " + err); // XXX
	    }
	    // System.out.println("You want I should save?");
	}

	if (command.equals("Write Config File")) {
	    try {
		map.dumpConfigFile();
	    } catch (IOException err) {
		System.err.println("Error saving config file: " + err); // XXX
	    }
	}

	if (command.equals("Undo")) {
	    // XXX it'd be nice to undo and have save grayed out
	    
	    // run through the list of undoable actions, undoing them
	    Vector newlist = new Vector();
	    for (int loop = undolist.size() - 1; loop >= 0; loop--) {
		UndoAction a = (UndoAction) undolist.elementAt(loop);
		newlist.addElement(a.getInverse());
		map.color(a.x, a.y, a.oldcolor);
	    }

	    undolist = newlist;

	    undid = true;
	}

	if (command.equals("Quit")) {
	    if (save.isEnabled()) {
		System.out.println("XXX there should be a quit confirm here");
		System.exit(-1);
	    } else {
		// nothing to save, so just quit.
		System.exit(0);
	    }
	}
    }

    public void coloringHappened(int x,
				 int y,
				 int newcolor,
				 int oldcolor)
    {
	save.setEnabled(true);
	if (undid) {
	    undo.setEnabled(true);
	    undolist = new Vector();
	    undid = false;
	}

	undolist.addElement(new UndoAction(x, y, newcolor, oldcolor));
    }


    public static void main(String argv[])
    {
	MapEdit me = new MapEdit(argv[0], false);
    }
}
