package odyssey.map;

import  java.awt.*;
import  java.awt.event.*;
import  java.io.*;

public class MapMouser
       implements MouseListener, MouseMotionListener, ActionListener {
    static final int PAINT = 93;
    static final int WOZ   = 105;

    int            mode = PAINT;
    Label          location;
    int            color;
    MouserListener listener;
    Map            theMap;

    PrintWriter locplay = null;

    public MapMouser(Label l, MouserListener ml, Map m)
    {
	this(l, ml, m, false);
    }

    public MapMouser(Label l, MouserListener ml, Map m, boolean usefifo)
    {
	location = l;
	color    = 255;
	listener = ml;
	theMap   = m;

	if (usefifo) {
	    try {
		FileOutputStream f =
                   new FileOutputStream("/usr/odyssey/locplay.fifo", true);
		locplay = new PrintWriter(f);
	    } catch (IOException e) {
		System.err.println("Can't open /usr/odyssey/locplay.fifo.");
		System.exit(-1);
	    }
	}
    }

    public void handleEvent(MouseEvent e, boolean handleEvent)
    {
	if (e.getComponent() instanceof Map) {
	    Map map = (Map) e.getComponent();
	    int x = e.getX();
	    int y = e.getY();

	    switch(mode) {
	    case PAINT:
		if (handleEvent) {
		    int result = map.color(x, y, color);
		    if (result != Map.NORESULT) {
			listener.coloringHappened(x, y, color, result);
		    }
		}
		break;

	    case WOZ:
		if (handleEvent) {
		    boolean changed = map.setPointer(x, y);
		    if (changed) {
			int latency = map.getColor(x,y);
			if (latency == map.TRANSPARENT) {
			    latency = 0;
			} else {
			    latency *= 1000;
			}
			if (locplay != null) {
			    locplay.println(latency);
			    locplay.flush();
			}
		    }
		}
		break;
	    }

	    location.setText(x + ", " + y +
			     " = " +
			     map.getColor(x, y));
	}
    }

    public void valueChanged(int i)
    {
	color = i;
    }

    public void setNewMap(Map m)
    {
	theMap = m;
    }

    public void actionPerformed(ActionEvent e)
    {
	String command = e.getActionCommand();

	if (command.equals("Paint")) {
	    mode = PAINT;
	} else if (command.equals("Wizard of Oz")) {
	    mode = WOZ;
	}

	if (command.equals("Repaint regions")) {
            // XXX currently just repaints regions
            theMap.repaintRegions();
        }
    }

    // Events which cause coloring to happen

    public void mouseClicked(MouseEvent e)
    {
	handleEvent(e, true);
    }

    public void mouseReleased(MouseEvent e)
    {
	handleEvent(e, true);
    }

    public void mouseDragged(MouseEvent e)
    {
	handleEvent(e, true);
    }

    public void mousePressed(MouseEvent e)
    {
	handleEvent(e, true);
    }

    // Events which don't cause coloring to happen

    public void mouseEntered(MouseEvent e)
    {
	handleEvent(e, false);
    }

    public void mouseExited(MouseEvent e)
    {
	handleEvent(e, false);
    }

    public void mouseMoved(MouseEvent e)
    {
	handleEvent(e, false);
    }
}
