#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <sys/param.h>
#ifdef __FreeBSD__
#include <sys/sysent.h>
#endif
#include <sys/systm.h>
#include <sys/conf.h>
#include <sys/mount.h>
#include <sys/exec.h>
#include <sys/lkm.h>
#include <sys/file.h>
#include <sys/errno.h>

#ifdef FREEBSD_BZERO_HACK
void
ody_bzero(char *p, int n)
{
    while ((n--) > 0)
	*(p++) = '\0';
}
#endif

#ifdef UPCALL
static struct lkm_table upcall_lkmt;
#endif
#ifdef SETS
static struct lkm_table sets_lkmt;
#endif
#ifdef ODYSSEY
static struct lkm_table odydev_lkmt;
static struct lkm_table ody_lkmt;
#endif

#ifdef __NetBSD__
MOD_MISC("odyssey");
#endif
#ifdef __FreeBSD__
MOD_MISC(odyssey);
#endif

odyssey_mod(lkmtp, cmd, ver)
     struct lkm_table *lkmtp;
     int cmd;
     int ver;
{
    extern int ucfs_mod(struct lkm_table *, int, int);
    extern int odydev_mod(struct lkm_table *, int, int);
    extern int ody_mod(struct lkm_table *, int, int);
    int error;

    /* XXX *BSD only allows us one lkm_table per modules; but we need
       multiple of them, since we have two VFS's and a device to load */

    /* We can't make them separate modules yet, since they reference
       each other, and unfortunately symbols of a loaded module are
       not make available to the next loaded module */

    /* So we pretend to be a single lkm of type misc, and multiplex all
       the others off that */

    /* NOTE: Since we unload the "submodules" one by one, it is possible
       for the unload to fail, but for some submodules to get unloaded
       (and hence disappear from the device/VFS/whatever space).

       I suppose this is a feature.
       */

#ifdef UPCALL
    error = ucfs_mod(&upcall_lkmt, cmd, ver);
    if (error)
	return error;
#endif
#ifdef SETS
    error = sets_mod(&sets_lkmt, cmd, ver);
    if (error)
	return error;
#endif
#ifdef ODYSSEY
    error = odydev_mod(&odydev_lkmt, cmd, ver);
    if (error)
	return error;
    error = ody_mod(&ody_lkmt, cmd, ver);
    if (error)
	return (error);
#endif
#ifdef __NetBSD__
    DISPATCH(lkmtp, cmd, ver, lkm_nofunc, lkm_nofunc, lkm_nofunc)
#endif
#ifdef __FreeBSD__
    DISPATCH(lkmtp, cmd, ver, lkm_nullcmd, lkm_nullcmd, lkm_nullcmd)
#endif
}

/* Create the default entry point */

#ifdef __NetBSD__
odyssey_lkmentry(lkmtp, cmd, ver)
#endif
#ifdef __FreeBSD__
odyssey(lkmtp, cmd, ver)
#endif
     struct lkm_table *lkmtp;
     int cmd;
     int ver;
{
     odyssey_mod(lkmtp, cmd, ver);
}
