#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* This library wrapper uses a (hopefully) temporary method for synchronizing 
   traces with the DMM.  It uses alternating hot and cold periods to produce
   a well-defined signal in the power output which can be detected by the
   post-processing code. */
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/errno.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/proc.h>
#include <sys/mount.h>
#include <sys/syscallargs.h>
#include "bprof.h"

int alarm_triggered;

static void ody_catch_signal()
{
  alarm_triggered = 1;
}

static void idle (int usec)
{
  struct itimerval interval;

  /* Set a timer, then go to sleep until woken up */
  timerclear(&interval.it_interval);
  interval.it_value.tv_sec = 0;
  interval.it_value.tv_usec = usec;
  alarm_triggered = 0;
  setitimer(ITIMER_REAL, &interval, NULL);
  pause();
}

static void peg (int usec)
{
  int a = 1002313, b = 232123, c;
  struct itimerval interval;

  /* Set a timer, then go to sleep until woken up */
  timerclear(&interval.it_interval);
  interval.it_value.tv_sec = 0;
  interval.it_value.tv_usec = usec;
  alarm_triggered = 0;
  setitimer(ITIMER_REAL, &interval, NULL); 
  
  while (!alarm_triggered) {
    c += b + a;
  }
}

/* short pattern that should be recognizable by inspection */
void pattern ()
{
  int i;

  signal (SIGALRM, ody_catch_signal);
  for (i = 0; i < 3; i++) {
    peg (100000);
    idle (100000);
  }
  signal (SIGALRM, SIG_DFL);
}

int bprofile_begin (void* profbuf, size_t size, int trigger_freq)
{
  pattern ();
  return (bprofile_start (profbuf, size, trigger_freq));
}

int bprofile_end (int* num_readings)
{
  int retval;

  retval = bprofile_stop (num_readings);
  pattern ();
  return (retval);
}

