#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* Map the Odyssey "system calls" to ioctls on the Odyssey control device */

#include <sys/ioctl.h>
#include <fcntl.h>

#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/odyapi.h"
#include "kernel/vfs/includes/ody_sys.h"
#include "include/odyssey.h"

int ody_ctl_fd = -1;

void
ody_ioctl_init(void)
{
    if (ody_ctl_fd < 0)
	ody_ctl_fd = open(ODY_CTL_FILE, O_RDONLY, 0);
}

int
ody_tsop(path, vers, op, argsz, arg, retsz, ret)
     char *path;
     struct ody_vers *vers;
     unsigned int op;
     size_t argsz;
     void *arg;
     size_t *retsz;
     void *ret;
{
    struct ody_tsop_args iarg;
    int result;

    iarg.path = path;
    iarg.vers = vers;
    iarg.op = op;
    iarg.argsz = argsz;
    iarg.arg = arg;
    iarg.retsz = retsz;
    iarg.ret = ret;

    ody_ioctl_init();
    result = ioctl(ody_ctl_fd, ODYI_TSOP, &iarg);
    if (result < -4095) {
	    errno = -result;
	    return -1;
    }
    return result;
}

int
ody_ftsop(fd, vers, op, argsz, arg, retsz, ret)
     int fd;
     struct ody_vers *vers;
     unsigned int op;
     size_t argsz;
     void *arg;
     size_t *retsz;
     void *ret;
{
    struct ody_ftsop_args iarg;
    int result;

    iarg.fd = fd;
    iarg.vers = vers;
    iarg.op = op;
    iarg.argsz = argsz;
    iarg.arg = arg;
    iarg.retsz = retsz;
    iarg.ret = ret;

    ody_ioctl_init();
    result = ioctl(ody_ctl_fd, ODYI_FTSOP, &iarg);
    if (result < -4095) {
	    errno = -result;
	    return -1;
    }
    return result;
}

int
__ody_req(path, ora)
     char *path;
     struct ody_reqargs *ora;
{
    struct ody_req_args iarg;
    int result;

    iarg.path = path;
    iarg.ora = ora;

    ody_ioctl_init();
    result = ioctl(ody_ctl_fd, ODYI_REQ, &iarg);
    if (result < -4095) {
	    errno = -result;
	    return -1;
    }
    return result;
}

int
__ody_freq(fd, ora)
     int fd;
     struct ody_reqargs *ora;
{
    struct ody_freq_args iarg;
    int result;

    iarg.fd = fd;
    iarg.ora = ora;

    ody_ioctl_init();
    result = ioctl(ody_ctl_fd, ODYI_FREQ, &iarg);
    if (result < -4095) {
	    errno = -result;
	    return -1;
    }
    return result;
}

int
__ody_cancel(path, reqid)
     char *path;
     int reqid;
{
    struct ody_cancel_args iarg;
    int result;

    iarg.path = path;
    iarg.reqid = reqid;

    ody_ioctl_init();
    result = ioctl(ody_ctl_fd, ODYI_CANCEL, &iarg);
    if (result < -4095) {
	    errno = -result;
	    return -1;
    }
    return result;
}
