#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* Map the upcall "system calls" to ioctls on the upcall device */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <sys/time.h>
#include <sys/proc.h>
#include <sys/ioctl.h>
#include <sys/ioccom.h>
#include <fcntl.h>

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/bsd44/sets_ioctl.h"

static int sets_fd = -1;

static inline void
sets_init(void)
{
    if (sets_fd < 0) {
	sets_fd = open(SETS_CTLOBJ, O_RDONLY, 0);
    }
}

int
set_open(const char *spec, int flags)
{
    struct setsi_open_args arg;
    int retval;
    arg.spec = (char *)spec;
    arg.flags = flags;

    sets_init();
    retval = ioctl(sets_fd, SETSI_OPEN, &arg);
    return (retval) ? retval : arg.handle;
}

int
set_close(int handle)
{
    struct setsi_close_args arg;
    arg.handle = handle;

    sets_init();
    return ioctl(sets_fd, SETSI_CLOSE, &arg);
}

int
set_daemon()
{
    sets_init();
    return ioctl(sets_fd, SETSI_DAEMON, NULL);
}

int
set_daemon_kill()
{
    sets_init();
    return ioctl(sets_fd, SETSI_DAEMON_KILL, NULL);
}

int
set_iterate(int handle, int flags, char *name,
	    u_int buffer_size, u_int *error)
{
    struct setsi_iterate_args arg;
    int retval;
    arg.handle = handle;
    arg.flags = flags;
    arg.name = name;
    arg.buffer_size = buffer_size;
    arg.error = error;

    sets_init();
    retval = ioctl(sets_fd, SETSI_ITERATE, &arg);
    return (retval) ? retval : arg.fd;
}

int
set_digest(int handle, struct DigestT *buffer, int buffer_size)
{
    struct setsi_digest_args arg;
    int retval;
    arg.handle = handle;
    arg.buffer = (char *)buffer;
    arg.buffer_size = buffer_size;

    sets_init();
    retval = ioctl(sets_fd, SETSI_DIGEST, &arg);
    return (retval) ? retval : arg.retsz;
}

int set_rewind_iterator(int handle)
{
    struct setsi_rew_iter_args arg;
    arg.handle = handle;

    sets_init();
    return ioctl(sets_fd, SETSI_REW_ITER, &arg);
}

int set_rewind_digest(int handle)
{
    struct setsi_rew_dig_args arg;
    arg.handle = handle;

    sets_init();
    return ioctl(sets_fd, SETSI_REW_DIG, &arg);
}

int set_member(int handle, char *name)
{
    struct setsi_member_args arg;
    arg.handle = handle;
    arg.name = name;

    sets_init();
    return ioctl(sets_fd, SETSI_MEMBER, &arg);
}

int set_size(int handle)
{
    struct setsi_size_args arg;
    int retval;
    arg.handle = handle;

    sets_init();
    retval = ioctl(sets_fd, SETSI_SIZE, &arg);
    return (retval) ? retval : arg.size;
}

int set_intersect(int handle1, int handle2, int flags)
{
    struct setsi_inter_args arg;
    int retval;
    arg.handle1 = handle1;
    arg.handle2 = handle2;
    arg.flags = flags;

    sets_init();
    retval = ioctl(sets_fd, SETSI_INTER, &arg);
    return (retval) ? retval : arg.handle;
}

int set_union(int handle1, int handle2, int flags)
{
    struct setsi_union_args arg;
    int retval;
    arg.handle1 = handle1;
    arg.handle2 = handle2;
    arg.flags = flags;

    sets_init();
    retval = ioctl(sets_fd, SETSI_UNION, &arg);
    return (retval) ? retval : arg.handle;
}

int set_restrict(int handle1, char *fname, int flags)
{
    struct setsi_restrict_args arg;
    int retval;
    arg.handle1 = handle1;
    arg.fname = fname;
    arg.flags = flags;

    sets_init();
    retval = ioctl(sets_fd, SETSI_RESTRICT, &arg);
    return (retval) ? retval : arg.handle;
}
