#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>
#include <errno.h>
#ifdef UC_SIGNAL
#include <signal.h>
#else
#include <pthread.h>
#endif

/* How many calls to upcall_block need to be unblocked? */
static int upcall_blockdepth = 0;  

#ifdef UC_SIGNAL
extern unsigned sig_upcall;
#else
extern pthread_mutex_t mutex;
#endif

int
upcall_block()
{
	sigset_t  blockme;

	/* 
	 * If we are at blockdepth >= 0, we must already be blocked
	 */
	if (upcall_blockdepth++) {
	    return 0;
	}

#ifdef UC_SIGNAL
	/* upcall_blockdepth is now 1 */
	if (sigemptyset(&blockme)
	    || sigaddset(&blockme,sig_upcall)
	    || sigprocmask(SIG_BLOCK, &blockme, NULL))
		return -1; /* Errno set by whoever failed */
	else
		return 0; /* A-OK */
#else
	return (pthread_mutex_lock (&mutex));
#endif

}

int
upcall_unblock()
{
	sigset_t  unblockme;

	/* 
	 * If we are at blockdepth > 1, then we can't actually unblock
	 * the signal just yet.
	 */
	upcall_blockdepth--;
	if (upcall_blockdepth) {
	    return 0;
	}

#ifdef UC_SIGNAL
	if (sigemptyset(&unblockme)
	    || sigaddset(&unblockme,sig_upcall)
	    || sigprocmask(SIG_UNBLOCK, &unblockme, NULL))
		return -1; /* Errno set by whoever failed */
	else
		return 0; /* A-OK */
#else
	return (pthread_mutex_unlock (&mutex));
#endif
}
