#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* Map the upcall "system calls" to ioctls on the upcall device */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#ifdef UC_SIGNAL
#include <signal.h>
#endif

#include "kernel/upcall/includes/upcall.h"
#include "kernel/upcall/linux21/uc_ioctl.h"

extern int errno;		/* is this bad?? */

#define UC_CTLOBJ "/ucfs"

static int uc_fd = -1;

#ifdef UC_SIGNAL
#ifdef SIGUPCALL
#define DEFAULT_SIG_UPCALL SIGUPCALL
#else
#define DEFAULT_SIG_UPCALL SIGUNUSED
#endif

unsigned sig_upcall = DEFAULT_SIG_UPCALL;
#endif /* UC_SIGNAL */

static void
upcall_init(void)
{
#ifdef UC_SIGNAL
    struct uci_set_sig_args arg;
#endif

    if (uc_fd < 0) {
	uc_fd = open(UC_CTLOBJ, O_RDONLY, 0);
#ifdef UC_SIGNAL
	if (uc_fd >= 0) {
	    arg.sig = sig_upcall; 
	    if (ioctl(uc_fd, UCI_SET_SIG, &arg) < 0) 
                uc_fd = -1;
	}
#endif
    }
}

int
upcall_sync(upclass, upcall, pid, argsz, argp, retszp, retp)
     upclass_t upclass;
     upcall_t upcall;
     pid_t pid;
     size_t argsz;
     void *argp;
     size_t *retszp;
     void *retp;
{
    struct uci_sync_args arg;

    arg.upclass = upclass;
    arg.upcall = upcall;
    arg.pid = pid;
    arg.argsz = argsz;
    arg.argp = argp;
    arg.retszp = retszp;
    arg.retp = retp;

    upcall_init();
    return ioctl(uc_fd, UCI_SYNC, &arg);
}

int
upcall_async(upclass, upcall, pid, argsz, argp)
     upclass_t upclass;
     upcall_t upcall;
     pid_t pid;
     size_t argsz;
     void *argp;
{
    struct uci_async_args arg;

    arg.upclass = upclass;
    arg.upcall = upcall;
    arg.pid = pid;
    arg.argsz = argsz;
    arg.argp = argp;

    upcall_init();
    return ioctl(uc_fd, UCI_ASYNC, &arg);
}

int
__upcall_reg(upclass, upcall, flags)
     upclass_t upclass;
     upcall_t upcall;
     int flags;
{
    struct uci_reg_args arg;

    arg.upclass = upclass;
    arg.upcall = upcall;
    arg.flags = flags;

    upcall_init();
    return ioctl(uc_fd, UCI_REG, &arg);
}

int
__upcall_next(upclassp, upcallp, argszp, argp, retszp, serialp)
     upclass_t *upclassp;
     upcall_t *upcallp;
     size_t *argszp;
     void *argp;
     size_t *retszp;
     serial_t *serialp;
{
    struct uci_next_args arg;

    arg.upclassp = upclassp;
    arg.upcallp = upcallp;
    arg.argszp = argszp;
    arg.argp = argp;
    arg.retszp = retszp;
    arg.serialp = serialp;

    upcall_init();
    return ioctl(uc_fd, UCI_NEXT, &arg);
}

int
__upcall_args(serial, bufsz, buf)
     serial_t serial;
     size_t bufsz;
     void *buf;
{
    struct uci_args_args arg;

    arg.serial = serial;
    arg.bufsz = bufsz;
    arg.buf = buf;

    upcall_init();
    return ioctl(uc_fd, UCI_ARGS, &arg);
}

int
__upcall_ret(serial, retsz, retp, errno)
     serial_t serial;
     size_t retsz;
     void *retp;
     int errno;
{
    struct uci_ret_args arg;

    arg.serial = serial;
    arg.retsz = retsz;
    arg.retp = retp;
    arg.errno = errno;

    upcall_init();
    return ioctl(uc_fd, UCI_RET, &arg);
}

#ifdef UC_SIGNAL
int 
upcall_set_sig(unsigned sig)
{
  /* Check if already inited */
  if (uc_fd >= 0) return -1;

  /* Check if signal is valid */
  if (sig >= NSIG) return -2;

  sig_upcall = sig;
  return 0;
}
#endif
