#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _SETS_NETBSD_H_
#define _SETS_NETBSD_H_

/*------------------------------------------------------+
 | all BSD44 system includes that are necessary        |
 +------------------------------------------------------*/

#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/queue.h>
#include <sys/malloc.h>
#include <sys/proc.h>
#include <sys/fcntl.h>
#include <sys/vnode.h>
#include <sys/filedesc.h>
#include <sys/file.h>
#include <sys/dirent.h>
#include <sys/uio.h>
#include <sys/namei.h>
#ifdef __NetBSD__
#ifdef _KERNEL
#include <sys/systm.h>
#endif
#include <sys/lock.h>
#else
#include <sys/mount.h>
#include <vm/vm.h>
#include <vm/lock.h>
#endif

/*------------------------------------------------------+
 | NetBSD module glue magic                             |
 +------------------------------------------------------*/

#define SETS_CTLOBJ "/sets"
#ifdef __NetBSD__
#define MOUNT_SETS "sets"
#endif

#define VT_SETS 202		/* this is arbitrary! */
#define M_SETS M_TEMP

/*------------------------------------------------------+
 | Memory management                                    |
 +------------------------------------------------------*/

#define SETS_ALLOC(ptr, cast, size) \
do {                                                                      \
/*    printf("sets malloc %d  %s:%d\n", size, __FILE__, __LINE__); */     \
    ptr = (cast)malloc((unsigned long) size, M_SETS, M_WAITOK);           \
    if (ptr == 0) {                                                       \
        panic("kernel malloc returns 0 at %s:%d\n", __FILE__, __LINE__);  \
    }                                                                     \
} while (0)
#define SETS_FREE(ptr, size)         \
do {                                    \
/*    printf("sets free %s:%d\n", __FILE__, __LINE__); */ \
    free((ptr),M_SETS);                                   \
} while (0)

#define MYALLOC SETS_ALLOC
#define MYFREE  SETS_FREE
#define STRALLOC(ptr) SETS_ALLOC(ptr, char *, MAXPATHLEN)
#define STRFREE(ptr)  SETS_FREE(ptr, MAXPATHLEN)

#define COPYINSTRING(a,b,c,err) {size_t t; err = -copyinstr(a,b,c,&t);}
#define COPYTOUSERSPACE copyout
#define PUT_USER_LONG(f,t) { long tmp=f; copyout((void *)&tmp,(void *)t,4);}
#define PUT_USER_INT(f,t) { long tmp=f; copyout((void *)&tmp,(void *)t,2);}
#define PUT_USER_BYTE(f,t) { long tmp=f; copyout((void *)&tmp,(void *)t,1);}

#define memset(s,foo,n) bzero(s,n)
#define memcpy(t,s,n) bcopy(s,t,n)

/*------------------------------------------------------+
 | NetBSD-specific type bindings                        |
 +------------------------------------------------------*/

typedef struct proc   *PROCESS_t;
typedef struct vnode  *FSOBJ_t;
typedef struct file   *FILEOBJ_t;
typedef int            FILEID;
typedef struct vnode  *CURSOBJ_t;
typedef struct lock    SETS_MUTEX;
typedef char           CONDITION;

/*------------------------------------------------------+
 | Process manipulation macros                          |
 +------------------------------------------------------*/

#define PID(p) ((p)->p_pid)
#define CURPID (curproc->p_pid)

/*------------------------------------------------------+
 | Directory manipulation macros                        |
 +------------------------------------------------------*/

#define Process_cdir (curproc->p_fd->fd_cdir)
#define Process_root (curproc->p_fd->fd_rdir ? \
		      curproc->p_fd->fd_rdir : rootvnode)
#define IS_DIR_OBJ(vp)  (vp->v_type == VDIR)
#define COMPONENT_MARKER '/'
#define COMPONENT_STRING "/"

#define SETS_MAXPATHLEN MAXPATHLEN

/*------------------------------------------------------+
 | FS object manipulation macros                        |
 +------------------------------------------------------*/

#ifdef _KERNEL
#define FILE_TYPE(vp)         vp->v_tag
#define FSOBJ_REF(vp)         VREF(vp)
#define FSOBJ_RELE(vp)        vrele(vp)
#define FSOBJ_UNLOCK(vp)      VOP_UNLOCK(vp)
#define OPEN_FSOBJ(vp,m,p)    VOP_OPEN(vp, m, p->p_cred->pc_ucred, p)
static inline int OPEN_CURSOBJ(struct vnode *vp, int mode, struct proc *p,
			struct vnode **cp)
{
    OPEN_FSOBJ(vp, mode, p);
    *(cp) = vp;
    return (0);
}
#define CLOSE_CURSOBJ(vp,p)   vn_close(vp, FREAD, p->p_ucred, p)
#endif

#define SETS_READ  FREAD
#define SETS_WRITE FWRITE

#define CURS_TO_FS(c)         c

#define FID_INVALID(f) (f<0)

/*------------------------------------------------------+
 | File manipulation macros                             |
 +------------------------------------------------------*/

#define RELEASE_FP(elem, p) \
{ \
    if (elem.fp->f_count == 1) { \
        /* on last deref, waste the fd but save the vnode.  Note: if \
	 * the last deref happens here, the fd has already disappeared \
	 * due to close(), so no need to fdrelease(). */ \
        elem.fp->f_ops = NULL; /* don't vn_close */ \
    } \
    closef(elem.fp, p); \
    elem.fp = NULL; \
    elem.fd = -1; \
    vrele(elem.fsp); \
}

/*------------------------------------------------------+
 | Synchronization primitives                           |
 +------------------------------------------------------*/

#define MUTEX_INIT(m)     /* simple_lock_init(&m) */
#define CONDITION_INIT(c)

#define CONDITION_SIGNAL(cond) \
do  { \
       DEBUG(THREAD,printf("Signaling condition %p %s:%d\n", \
			   (cond),__FILE__,__LINE__););      \
       wakeup((cond)); \
    } while (0);


#define MUTEX_UNLOCK(m)\
do  { \
	DEBUG(THREAD,printf("mutex_unlock RL(%p)%s:%d...", (m), \
			    __FILE__, __LINE__););              \
	/* simple_unlock((m)); */ \
        DEBUG(THREAD,printf("done\n");); \
    } while (0);

#define MUTEX_LOCK(m)\
do  { \
	DEBUG(THREAD,printf("mutex_lock OL(%p)%s:%d...", (m), \
			    __FILE__, __LINE__););            \
	/* simple_lock((m)); */ \
        DEBUG(THREAD,printf("done\n");); \
    } while (0);

#define CONDITION_WAIT(cond, lock) \
do  { \
       DEBUG(THREAD,printf("Waiting on condition %p %s:%d\n", (cond), \
			   __FILE__,__LINE__););                      \
       if (lock) MUTEX_UNLOCK((lock)); \
       tsleep((cond), PWAIT, "cond_wait", 0); \
       if (lock) MUTEX_LOCK((lock)); \
    } while(0);

/*------------------------------------------------------+
 | Miscellaneous                                        |
 +------------------------------------------------------*/

#ifdef __NetBSD__
#define TAILQ_NEXT(elm, field) (elm)->field.tqe_next
#endif

#define LOG_ENTRY(THREAD, TAG)
#define LOG_EXIT(THREAD, TAG)

#ifdef _READ
#undef PRE_READ			/* buffer cache hacks not implemented */
#endif

#ifdef __FreeBSD__
#ifdef FREEBSD_BZERO_HACK
#define bzero(p,n) ody_bzero(p,n)
#endif
#endif

#define exit(foo)  panic("DEATH!")

#define S_PRINTF printf

#endif /* _SETS_NETBSD_H_ */
