#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

/*
 * Stolen yet again from Odyssey/Upcall
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/conf.h>
#include <sys/namei.h>
#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/vnode.h>
#ifdef __FreeBSD__ /* XXX gruesome hack */
#include "specdev.h"
#else
#include <miscfs/specfs/specdev.h>
#endif

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/bsd44/sets_vfsops.h"
#include "kernel/sets/bsd44/sets_vnops.h"
#include "kernel/sets/common/sets_domain.h"

int sets_vfsop_print_entry = 0;
#ifdef __GNUC__
#define ENTRY    \
    if(sets_vfsop_print_entry) printf("Entered %s\n",__FUNCTION__)
#else
#define ENTRY
#endif 

extern int (**sets_vnodeop_p)(void *);

#ifdef	__NetBSD__
struct vfsops sets_vfsops = {
    MOUNT_SETS,
    sets_mount,
    sets_start,
    sets_unmount,
    sets_root,
    (int (*) (struct mount *, int, uid_t, caddr_t,
	      struct proc *)) eopnotsupp,
    sets_statfs,
    sets_sync, 
    (int (*) (struct mount *, ino_t, struct vnode **)) eopnotsupp,
    (int (*) (struct mount *, struct fid *, struct mbuf *, struct vnode **,
	      int *, struct ucred **)) eopnotsupp,
    (int (*) (struct vnode *, struct fid *)) eopnotsupp,
    sets_init,
#ifdef	NetBSD1_3
    (int (*)(void)) eopnotsupp,
#endif
    0
};

#elif	defined(__FreeBSD__)
struct vfsops sets_vfsops = {
    sets_mount,
    sets_start,
    sets_unmount,
    sets_root,
    (int (*) (struct mount *, int, uid_t, caddr_t,
	      struct proc *)) eopnotsupp,
    sets_statfs,
    sets_sync,
    (int (*) (struct mount *, ino_t, struct vnode **)) eopnotsupp,
    (int (*) (struct mount *, struct fid *, struct mbuf *, struct vnode **,
	      int *, struct ucred **)) eopnotsupp,
    (int (*) (struct vnode *, struct fid *)) eopnotsupp,
    sets_init,
};

#endif

struct vnode *sets_root_vp = NULL;
static int mounted = 0;

/*
 * Upcall mount vfsop
 * Set up mount info record and attach it to vfs struct.
 */
/*ARGSUSED*/
int sets_mount(vfsp, path, data, ndp, p)
    struct mount *vfsp;    /* Allocated and initialized by mount(2) */
#ifdef NetBSD1_3
    const char *path;      /* path covered: ignored by the fs-layer */
    void *data;            /* Need to define a data type for this in netbsd? */
#else
    char *path;            /* path covered: ignored by the fs-layer */
    caddr_t data;          /* Need to define a data type for this in netbsd? */
#endif
    struct nameidata *ndp;
    struct proc *p;        /* The ever-famous proc pointer */
{
    int error;

    if (p->p_cred->pc_ucred->cr_uid != 0) { /* only root can mount */
	return(EPERM);
    }

    if (mounted) {		/* can only mount once */
	return(EBUSY);
    }

    /*
     * Make a root vnode
     */

    error = getnewvnode(VT_SETS, vfsp, sets_vnodeop_p, &sets_root_vp);
    if (error)
	panic("ody: getnewvnode returned error %d\n", error);

    sets_root_vp->v_type = VCHR; /* So we can do ioctls on it */
    sets_root_vp->v_flag = VROOT;

    /* This is a really bad hack - copied from Odyssey vfs implementation */
    sets_root_vp->v_specinfo = (struct specinfo *)
      malloc((u_long) sizeof(struct specinfo), M_VNODE, M_NOWAIT);
    sets_root_vp->v_hashchain = &sets_root_vp; 
    sets_root_vp->v_specnext = sets_root_vp;
    sets_root_vp->v_specflags = 0;
    sets_root_vp->v_specinfo->si_rdev = makedev(0, 0);
#ifndef __FreeBSD__
    sets_root_vp->v_speclockf = NULL;
#endif

    mounted = 1;

    return(0);
}

int
sets_start(vfsp, flags, p)
    struct mount *vfsp;
    int flags;
    struct proc *p;
{
    ENTRY;

    setInit();
    return (0);
}

int
sets_statfs(vfsp, sbp, p)
    register struct mount *vfsp;
    struct statfs *sbp;
    struct proc *p;
{
    ENTRY;


    bzero(sbp, sizeof(struct statfs));
    /* XXX - what to do about f_flags, others? --bnoble */
    /* Below This is what AFS does
    	#define NB_SFS_SIZ 0x895440
     */
    /* Note: Normal fs's have a bsize of 0x400 == 1024 */
#ifdef	__NetBSD__
    sbp->f_type = 0;
#elif	defined(__FreeBSD__)
    sbp->f_type = MOUNT_SETS;
#endif
    sbp->f_bsize = 8192;	/* XXX -JJK */
    sbp->f_iosize = 8192;
#define NB_SFS_SIZ 0x8AB75D
    sbp->f_blocks = NB_SFS_SIZ;
    sbp->f_bfree = NB_SFS_SIZ;
    sbp->f_bavail = NB_SFS_SIZ;
    sbp->f_files = NB_SFS_SIZ;
    sbp->f_ffree = NB_SFS_SIZ;
    bcopy((caddr_t)&(vfsp->mnt_stat.f_fsid), (caddr_t)&(sbp->f_fsid),
	  sizeof (fsid_t));
#ifdef	__NetBSD__
    strncpy(sbp->f_fstypename, MOUNT_SETS, MFSNAMELEN-1);
#endif
    strcpy(sbp->f_mntonname, "/sets"); /* XXX cheesy */
    strcpy(sbp->f_mntfromname, "sets");
    
    return(0);
}

int
sets_unmount(vfsp, mntflags, p)
    struct mount *vfsp;
    int mntflags;
    struct proc *p;
{
    ENTRY;

    /* If there are outstanding registrations or sets open,
     * don't allow module to unload. */
    if (sets_exist_registrations() || lastOpenSet != 0)
	return (EBUSY);

	/* Notes: assumed that no domains are registered, and that no sets are
	 *        opened.  This implies that no messages are queued for daemons. */
	sets_hashexit();

    if (sets_root_vp->v_usecount > 1) {
	/* this little bit of trickery allows the sets daemons to be killed,
	 * and release their references to the root_vp.  This way, when a
	 * SECOND unmount is done, the vflush just below actually does the
	 * vgone.  This way, the vnode is dealt with NOW -- not at some later
	 * time when sets might have been unloaded, or re-mounted. */
        kill_daemons();
	return (EAGAIN);
    }

    vrele(sets_root_vp);
    vflush(vfsp, NULLVP, FORCECLOSE);

    /* Note: the sets daemons (if they were just killed above) have
     *       outstanding refs to the sets_root_vp (since they haven't
     *       processed their WM_EXIT msgs), which leads to vgone
     *       happening in the future (when their processes close
     *       references to /sets). */

    mounted = 0;

    /* Note: leave sets_root_vp alone, because the vgone in the future will
     *       barf if v_hashchain ( = &sets_root_vp ) is 0.  Of course
     *       we still have a problem if sets is remounted before the vgone
     *       occurs. */

    return (0);
}

/*
 * Flush any pending I/O - which, of cource, is a no-op for sets.
 */
int
sets_sync(vfsp, waitfor, cred, p)
    struct mount *vfsp;
    int    waitfor;
    struct ucred *cred;
    struct proc *p;
{
    return (0);
}


/*
 * find root of odyssey file sys
 */
int
sets_root(vfsp, vpp)
    struct mount *vfsp;
    struct vnode **vpp;
{
    *vpp = sets_root_vp;
    vref(*vpp);
    VOP_LOCK(*vpp);
    return (0);
}

/* called by LKM loader (or vfs_init code)
 * since we're a module, and don't get unload notification, rely on
 * sets_start (mount-time) and sets_unmount (unmount-time) routines
 * for startup/cleanup. */

#ifdef __NetBSD__
void
sets_init()
{
    ENTRY;
}
#elif defined(__FreeBSD__)
int
sets_init()
{
    ENTRY;
    return 0;
}
#endif
