#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1990 Carnegie-Mellon University
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

#include <sys/param.h>
#include <sys/malloc.h>
#include <sys/errno.h>
#include <sys/acct.h>
#include <sys/uio.h>
#include <sys/namei.h>
#include <sys/ioctl.h>
#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/vnode.h>
#include <sys/signal.h>
#ifdef NetBSD1_3
#include <sys/poll.h>
#include <sys/user.h>
#include <miscfs/genfs/genfs.h>
#endif
#include <vm/vm.h>

#ifdef	__FreeBSD__
#include <vm/vm_object.h>
#include <vm/vm_extern.h>
#endif

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_hash.h"
#include "kernel/sets/bsd44/sets_vnops.h"
#include "kernel/sets/bsd44/sets_ioctl.h"

/* Definition of the vfs operation vector */
int (**sets_vnodeop_p)(void *);
extern struct vnode *sets_root_vp;

/* Definition of the vnode operation vector */

struct vnodeopv_entry_desc sets_vnodeop_entries[] = {
    { &vop_default_desc, sets_eopnotsupp},
    { &vop_lookup_desc, sets_eopnotsupp },     /* lookup */
    { &vop_create_desc, sets_eopnotsupp },	/* create */
    { &vop_mknod_desc, sets_eopnotsupp },	/* mknod */
    { &vop_open_desc, sets_open },		/* open */
    { &vop_close_desc, sets_close },		/* close */
    { &vop_access_desc, sets_access },		/* access */
    { &vop_getattr_desc, sets_eopnotsupp },	/* getattr */
    { &vop_setattr_desc, sets_eopnotsupp },	/* setattr */
    { &vop_read_desc, sets_eopnotsupp },	/* read */
    { &vop_write_desc, sets_eopnotsupp },	/* write */
    { &vop_ioctl_desc, sets_ioctl },		/* ioctl */
#ifdef NetBSD1_3
    { &vop_poll_desc, sets_eopnotsupp },	/* poll */
#else
    { &vop_select_desc, sets_eopnotsupp },	/* select */
#endif
    { &vop_mmap_desc, sets_eopnotsupp },	/* mmap */
    { &vop_fsync_desc, sets_eopnotsupp },	/* fsync */
#ifdef NetBSD1_3
    { &vop_seek_desc, sets_eopnotsupp },	/* seek */
#else
    { &vop_seek_desc, sets_eopnotsupp },	/* seek */
#endif
    { &vop_remove_desc, sets_eopnotsupp },		/* remove */
    { &vop_link_desc, sets_eopnotsupp },		/* link */
    { &vop_rename_desc, sets_eopnotsupp},		/* rename */
    { &vop_mkdir_desc, sets_eopnotsupp },		/* mkdir */
    { &vop_rmdir_desc, sets_eopnotsupp },		/* rmdir */
    { &vop_symlink_desc, sets_eopnotsupp },		/* symlink */
    { &vop_readdir_desc, sets_eopnotsupp },		/* readdir */
    { &vop_readlink_desc, sets_eopnotsupp },	/* readlink */
    { &vop_abortop_desc, sets_eopnotsupp },	/* abortop */
    { &vop_inactive_desc, sets_eopnotsupp },	/* inactive */
    { &vop_reclaim_desc, sets_reclaim },	/* reclaim */
    { &vop_lock_desc, sets_lock },	/* lock */
    { &vop_unlock_desc, sets_unlock },	/* unlock */
    { &vop_bmap_desc, sets_eopnotsupp },	/* bmap */
    { &vop_strategy_desc, sets_eopnotsupp },	/* strategy */
    { &vop_print_desc, sets_eopnotsupp },	/* print */
    { &vop_islocked_desc, sets_eopnotsupp },	/* islocked */
    { &vop_pathconf_desc, sets_eopnotsupp },	/* pathconf */
    { &vop_advlock_desc, sets_eopnotsupp },	/* advlock */
    { &vop_blkatoff_desc, sets_eopnotsupp },	/* blkatoff */
    { &vop_valloc_desc, sets_eopnotsupp },	/* valloc */
    { &vop_vfree_desc, sets_eopnotsupp },	/* vfree */
    { &vop_truncate_desc, sets_eopnotsupp },	/* truncate */
    { &vop_update_desc, sets_eopnotsupp },	/* update */
    { &vop_bwrite_desc, sets_eopnotsupp },	/* bwrite */
#ifdef __NetBSD__
    { &vop_lease_desc, sets_eopnotsupp },	/* lease */
#endif
#ifdef	__FreeBSD__
    { &vop_getpages_desc, sets_eopnotsupp }, /* pager intf.*/
    { &vop_putpages_desc, sets_eopnotsupp }, /* pager intf.*/
#endif
    { (struct vnodeop_desc*)NULL, (int(*)(void *))NULL }
};

#ifdef __NetBSD__
struct vnodeopv_desc sets_vnodeop_opv_desc = 
        { &sets_vnodeop_p, sets_vnodeop_entries };
#endif

#ifdef __FreeBSD__
static struct vnodeopv_desc sets_vnodeop_opv_desc = 
		{ &sets_vnodeop_p, sets_vnodeop_entries };

#include <sys/kernel.h>
VNODEOP_SET(sets_vnodeop_opv_desc);

#endif

/* Definitions of NetBSD vnodeop interfaces */

int
sets_eopnotsupp(v)
     void *v;
{
    return (EOPNOTSUPP);
}

int
sets_open(v)
     void *v;
{
	/* nothing to do -- first ioctl will do setInit() */
        return 0;
}

int
sets_close(void *v)
{
	struct vop_close_args *ap = v;
	struct procSetInfo *settbl;

	/* process closed the sets ctlobj - cleanup open sets */
	if ((settbl = sets_hashlookup(PID(ap->a_p))) != 0) {
		sets_proc_exit(settbl);
		sets_hashdelete(settbl);
		/* All sets for this process are now deallocated, deallocate
		 * per-task stuff */
		MYFREE(settbl, sizeof(struct procSetInfo));
	}
	return 0;
}

int
sets_ioctl(void *v)
{
	struct vop_ioctl_args *ap = v;
	u_long com = ap->a_command;
	caddr_t data = ap->a_data;
	struct proc *p = ap->a_p;
	int len = IOCPARM_LEN(com);

	switch (com) {
	case SETSI_OPEN:
		if (len != sizeof(struct setsi_open_args)) return EINVAL;
		return setsi_open(p, (struct setsi_open_args *) data);
	case SETSI_CLOSE:
		if (len != sizeof(struct setsi_close_args)) return EINVAL;
		return setsi_close(p, (struct setsi_close_args *) data);
	case SETSI_DAEMON:
		return setsi_daemon(p);
	case SETSI_DAEMON_KILL:
		return setsi_daemon_kill(p);
	case SETSI_ITERATE:
		if (len != sizeof(struct setsi_iterate_args)) return EINVAL;
		return setsi_iterate(p, (struct setsi_iterate_args *) data);
	case SETSI_DIGEST:
		if (len != sizeof(struct setsi_digest_args)) return EINVAL;
		return setsi_digest(p, (struct setsi_digest_args *) data);
	case SETSI_REW_ITER:
		if (len != sizeof(struct setsi_rew_iter_args)) return EINVAL;
		return setsi_rew_iter(p, (struct setsi_rew_iter_args *) data);
	case SETSI_REW_DIG:
		if (len != sizeof(struct setsi_rew_dig_args)) return EINVAL;
		return setsi_rew_dig(p, (struct setsi_rew_dig_args *) data);
	case SETSI_MEMBER:
		if (len != sizeof(struct setsi_member_args)) return EINVAL;
		return setsi_member(p, (struct setsi_member_args *) data);
	case SETSI_SIZE:
		if (len != sizeof(struct setsi_size_args)) return EINVAL;
		return setsi_size(p, (struct setsi_size_args *) data);
	case SETSI_INTER:
		if (len != sizeof(struct setsi_inter_args)) return EINVAL;
		return setsi_inter(p, (struct setsi_inter_args *) data);
	case SETSI_UNION:
		if (len != sizeof(struct setsi_union_args)) return EINVAL;
		return setsi_union(p, (struct setsi_union_args *) data);
	case SETSI_RESTRICT:
		if (len != sizeof(struct setsi_restrict_args)) return EINVAL;
		return setsi_restrict(p, (struct setsi_restrict_args *) data);
	default:
		return EINVAL;
	}
}

int
sets_access(void *v)
{
	return (0);
}

int 
sets_reclaim (void *v)
{
    /* No file-system specific data */
    return (0);
}

int
sets_lock(void *v)
{
	return (0);
}

int
sets_unlock(void *v)
{
	return (0);
}

