#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_domain.h"

/*
 *  notes
 *   - handle comes with 1 REF for us (RELE it on the way out)
 *   - any query and expansion queued gets 1 REF on the vp/handle
 *   - let purgeMessage FSOBJ_RELE(vp) and CLOSE_VP(handle)
 */
int expand_cursor(workerMsgType *msg, int *free_msg, int myindex)
{
	u_int   bufsize;
	char   *buf = NULL;
	int    error, i, size;
	struct linktype *link;
	workerMsgType *newmsg;
	struct sets_domain *dom;

	DEBUG(QUERY, S_PRINTF("(%d): cursor expanding in %s (%s)\n", myindex, 
			    msg->msg.query.pathname, msg->msg.query.suffix););

	LOG_ENTRY(myindex, EXPANDCURSOR);
	
	/* find the cursor's domain */
	dom = sets_find_domain(CURS_TO_FS(msg->msg.query.handle));
	if (!dom)
		return SETS_BADDOMAIN;

	/* use the cursexpand function to get results */
	error = (dom->cursexpand_fn)(msg->msg.query.handle,
				     &bufsize, &buf,
				     msg->set->owner);

	if (error) {
		if (buf)
			MYFREE(buf, bufsize);
		
		if (error == 0xe0f) /* Simply an EOF. */
			/* clean out this onode??
			 * and what about cursor onodes that aren't
			 * saturated before set_close? */
			error = 0;
		else {
			DEBUG( QUERY, S_PRINTF("ODY_CURSEXPAND got error %d\n",
					     error););
			error = SETS_CRS_BADCEXPAND;
		}
		goto done;
	} else {
		/* For each link returned, queue a msg to further expand it. */
		size = 0;	/* Use size to hold the # of links returned. */
		i = 0;		/* i will be the current offset. */
		link = (struct linktype *)buf;
		
		while (link->next != 0 && i < bufsize) {
			char *name = &link->link[0];
			char *savename;
			STRALLOC(savename); 
			
			if (name[0] == '/') {
				/* We have an absolute link, don't use
				 * the prefix of the old path */
				savename[0] = '\0';
			} else {
				/* Need a new copy of the already
				 * expanded path... */
				strcpy(savename, msg->msg.query.pathname);
			}
			
			/* create a new msg to expand the newly returned */
			DEBUG(QUERY, 
			      S_PRINTF("ODY_CURSEXPAND got a name -%s-\n",
				     name); );
			
			/* NOTE!!! if name can have wildcards, should
			 * do an expand REGARDLESS of suffix, including 
			 * name in the query */
			if (*msg->msg.query.suffix) {
				size++;
				MAKE_NEW_MSG_FROM_OLD(newmsg, msg);
				newmsg->operation = WM_EXPAND;
				
				newmsg->msg.expand.pathname = savename;
				newmsg->msg.expand.vp = name[0] == '/' ? NULL :
					msg->msg.query.vp;
				if (newmsg->msg.expand.vp)
					FSOBJ_REF(newmsg->msg.expand.vp);
				
				/* Append the unexpanded suffix to our
				 * query result. */
				STRALLOC(newmsg->msg.expand.query);
				strcpy(newmsg->msg.expand.query, name);
				strcat(newmsg->msg.expand.query, "/");
				strcat(newmsg->msg.expand.query,
				       msg->msg.query.suffix);
				
				DEBUG(QUERY,S_PRINTF("enqueueing\n"););
				enqueueMessage(newmsg, myindex);
			} else {
				/* No suffix, we have a fully expanded name. */
				strcat(savename, name);
				insertNewMember(msg->set, savename, myindex);
			}
			
			i += link->next;
			link = (struct linktype *)((long)buf + i);
		}
		
		DEBUG(QUERY,
		      S_PRINTF("ODY_CURSEXPAND returned %d names to expand\n",
			     size); );
		
		/* One for each newmsg, one for query msg */
		CRITICAL(msg->set->lock, msg->set->expands+= size + 1);
		
		/* There is more in the cursor, requeue an expansion msg. */
		DEBUG(QUERY,S_PRINTF("requeueing\n"););
		enqueueMessage(msg, myindex);
		
		MYFREE(buf, bufsize);
		*free_msg = 0;
		return 0;
	}

 done:
	DEBUG(QUERY, S_PRINTF("ODY_CURSEXPAND finished\n"););
	*free_msg = 1;

	return error;
}
