#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_hash.h"

/* Close all yielded file descriptors, release vnodes and memory. */
int do_set_close(struct procSetInfo *settbl, int handle)
{
	struct setType *myset;
	int error = 0;
    
	LOG_ENTRY(0, SETCLOSE);
	
	DEBUG(MYCLOSE, S_PRINTF("Someone called setClose (%d)\n", handle););
    
	/* Make sure we actually have an open set, and that this process's set
	 * structure has been initialized. */
    
	if (handle < 0 || handle >= NOPENSETS) {
		error = -EBADF;
		goto out;
	}
    
	if ((myset = settbl->mySets[handle]) == NULL) {
		error = -EBADF;
		goto out;
	}

	/* Mark this set as released (data no good). */
	/* Do this here because purgeMessage calls releaseSet too */
	CRITICAL(myset->lock, {
		myset->unique = 0; /* indicates to daemon that messages
				    * ref'ing this set are old */

		if (--myset->refct == 0)
			releaseSet(myset, 0); /* Reclaim set if no refs */
	});
    
	settbl->mySets[handle] = NULL;
	settbl->nOpen--;
 out:
	LOG_EXIT(0, SETCLOSE);
	return error;
}

/* Internal kernel routine to close all sets. Assumes it is called by exit()
 * and all open files will be or have been closed. */
void sets_proc_exit(struct procSetInfo *settbl)
{
	int j;
	struct setType *myset;

	LOG_ENTRY(0, CLOSEMYSETS);

	for (j = 0; j < NOPENSETS; j++) {
		if (settbl->mySets[j]) {
			myset = settbl->mySets[j];

			/* Decrement the count on the set, and
			 * possibly deallocate it.  Mark this set as
			 * released (data no good).  Do this here
			 * because purgeMessage calls releaseSet too
			 * */
			CRITICAL(myset->lock, {
				myset->unique = 0;

				if (--myset->refct == 0) {
					/* Reclaim set */
					releaseSet(myset, 0);
				}
			});
		}
	}
	
	LOG_EXIT(0, CLOSEMYSETS);
}
