#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_hash.h"

int do_set_member(struct procSetInfo *settbl, int handle, char *name)
{
	struct setType *myset;
	PROCESS_t p = settbl->p;
	int i, result = 0;
	
	LOG_ENTRY(0, SETMEMBER);
	
	DEBUG(UTIL, S_PRINTF("Entering setMember(%d, %s)\n", handle, name););
	
	if (handle < 0 || handle >= NOPENSETS) {
		result = -EBADF;
		goto out;
	}
	
	if ((settbl = sets_hashlookup(PID(p))) == NULL) {
		result = -EINVAL;
		goto out;
	}
	
	if ((myset = settbl->mySets[handle]) == NULL) {
		result = -EBADF;
		goto out;
	}
	
	/* Don't need a critical section, since this can be 
	 * an approximation anyway */
	for (i = 0; i < myset->resSize; i++) {
		switch (myset->resolved[i].state) {
		case ELEM_TAKEN:
		case ELEM_SEEN:
		case ELEM_OPENED:
			if (myset->resolved[i].name)
				if (!strcmp(myset->resolved[i].name, name)) {
					result = 1;
					goto out;
				}
			break;
			
		default:
			break;
		}
	}
	
	result = 0;
	
out:
	LOG_EXIT(0, SETMEMBER);
	return result;
}

