#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_hash.h"

int do_set_open(struct procSetInfo *settbl,
		const char *setname, int flags, SETID *handle)
{
	struct setType *set;
	PROCESS_t p = settbl->p;
	char *name;
	int result;

	LOG_ENTRY(0, SETOPEN);
    
	DEBUG(OPEN, S_PRINTF("Entering setOpen(%x, %d)\n", 
			   (int)setname, flags););
    
	STRALLOC(name);
	COPYINSTRING(setname, name, SETS_MAXPATHLEN-1, result);
	if (result < 0)
		goto out;

	DEBUG(OPEN, S_PRINTF("Setname: %s, curproc: %d\n", name, CURPID););

	/* createSet will return a negative value on error,
	 * or the open file handle */
	result = createSet(name, settbl);
	if (result < 0) {
		goto out;
	}

	set = settbl->mySets[result];
	set->owner = p;

	if (flags & SETS_SHARESET) 
		set->flags |= SHARE_ON_FORK;
	else if (flags & SETS_DUP_ON_FORK)
		set->flags |= DUP_ON_FORK;

 	if (flags & SETS_PREFETCH)
	        set->flags |= OPEN_PREFETCHES;
 	if (flags & SETS_INTERPRET_PAGE)
	        set->flags |= INTERPRET_PAGES;

	/* The default is to wait for a call to digest or iterate to start
	 * work.  However, applications can hint that an iterate or digest
	 * will happen, in which case queue a message. */
	set->flags |= DELAYOPENS; /* By default */

	if ((flags & SETS_ANTICIPATE_ITERATE) ||
	    (flags & SETS_ANTICIPATE_DIGEST)) {
		workerMsgType *msg;
	
		if (flags & SETS_ANTICIPATE_ITERATE) {
			set->flags &= ~DELAYOPENS;
		}

		MAKE_NEW_MSG(msg, WM_EXPAND, set);

		STRALLOC(msg->msg.expand.pathname);
		*msg->msg.expand.pathname = 0; /* zero length string */
		msg->msg.expand.vp = (FSOBJ_t)NULL; /* start in cdir or rdir */
	
		STRALLOC(msg->msg.expand.query);
		strcpy(msg->msg.expand.query, name);
	
		/* Initialize the iterator */
		CRITICAL(set->lock, {
			(settbl->mySets[result])->iterator++;
			/* Indicate an expansion needs to be done. */
			set->expands++;
		});

		enqueueMessage(msg, -1); /* And yield (not on linux...) */
	}
	DEBUG(OPEN, setPrint(settbl->mySets[result]););

	*handle = result;
	result = 0;

 out:
	LOG_EXIT(0, SETOPEN);
	return result;
}
