#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _SETS_H_
#define _SETS_H_

/*
 * struct DigestT -- the buffer returned by set_digest is formatted
 *                   as an array of variable-sized DigestT's
 */
struct DigestT {
    struct DigestT *next;       /* pointer to start of next record */
    char            fetched;    /* flag indicated fetchedness */
    char            name;       /* placeholder for member name string */
};

/*
 * struct linktype -- the buffer returned by the cursexpand routine is
 *                    formatted as an array of variable-sized linktype
 */
struct linktype {
    long next;                  /* Offset into buffer of next element */
    char link[sizeof(char *)];  /* Place holder for data */
};

typedef int SETID;


/*
 * Dynamic Sets API prototypes
 */

/* flags for set_open */
#define SETS_NODUP_ON_FORK      0x01
#define SETS_DUP_ON_FORK        0x02
#define SETS_SHARESET           0x04
#define SETS_ANTICIPATE_DIGEST  0x08
#define SETS_ANTICIPATE_ITERATE 0x10
#define SETS_PREFETCH           0x20
#define SETS_INTERPRET_PAGE     0x40 /* XXX web specific? move elsewhere? */

/* flags for set_iterate */
#define SETS_BASIC              0x00
#define SETS_GETNAME            0x01
#define SETS_GETERR             0x02


/* Error codes */

#define SETS_BADDOMAIN       0x0001

#define SETS_EXP_BADSPEC     0x0101
#define SETS_EXP_NOTDIR      0x0102
#define SETS_EXP_NOMATCH     0x0103
#define SETS_EXP_BADLOOKUP   0x0105
#define SETS_EXP_BADCURSOR   0x0106
#define SETS_EXP_BADEXPAND   0x0107
#define SETS_EXP_BADDIR      0x0108
#define SETS_EXP_NOFP        0x0109

#define SETS_CRS_BADCEXPAND  0x0201


#ifndef WIN32
SETID set_open          __P((const char *spec,
                             int flags));

int set_close           __P((SETID handle));

int set_iterate         __P((SETID handle,
                             int flags,
                             char *buf,
                             unsigned int bufsize,
                             unsigned int *error));

int set_digest          __P((SETID handle,
                             struct DigestT *buf,
                             int bufsize));

int set_rewind_iterator __P((SETID handle));

int set_rewind_digest   __P((SETID handle));

int set_daemon          __P((void));

int set_daemon_kill     __P((void));

SETID set_union         __P((SETID handle1,
                             SETID handle2,
                             int flags));

SETID set_intersect     __P((SETID handle1,
                             SETID handle2,
                             int flags));       

int set_size            __P((SETID handle));

int set_member          __P((SETID handle,
                             char *name));

SETID set_restrict      __P((SETID, char *, int));

#else

#define DLLEXPORT __declspec(dllexport)

DLLEXPORT SETID  WINAPI set_open    (PCSTR spec, UINT flags);
DLLEXPORT VOID   WINAPI set_close   (SETID handle);
DLLEXPORT INT    WINAPI set_digest  (SETID handle, struct DigestT *buffer,
                                     UINT bufsize);
DLLEXPORT HANDLE WINAPI set_iterate (SETID handle, UINT flags, PSTR buffer,
                                     UINT bufsize, PUINT error);
DLLEXPORT INT    WINAPI set_rewind_digest   (SETID handle);
DLLEXPORT INT    WINAPI set_rewind_iterator (SETID handle);
DLLEXPORT INT    WINAPI set_size    (SETID handle);
DLLEXPORT SETID  WINAPI set_intersect(SETID handle1, SETID handle2, UINT flags);
DLLEXPORT SETID  WINAPI set_union   (SETID handle1, SETID handle2, UINT flags);

#endif /* WIN32 */

#endif /* _SETS_H_ */
