#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_domain.h"

static struct sets_domain *sets_domainlist = NULL;

int sets_register_domain(int id,
			 sets_expand_fn exp,
			 sets_cursexpand_fn cursexp,
			 sets_prefetch_fn prefetch)
{
    struct sets_domain *curr_dom = sets_domainlist;

    while (curr_dom) {
	    if (curr_dom->file_type == id)
		    return EINVAL;
	    curr_dom = curr_dom->next;
    }
    
    SETS_ALLOC(curr_dom, struct sets_domain *, sizeof(struct sets_domain));
    curr_dom->file_type = id;
    curr_dom->expand_fn = exp;
    curr_dom->cursexpand_fn = cursexp;
    curr_dom->prefetch_fn = prefetch;
    curr_dom->next = sets_domainlist;
    sets_domainlist = curr_dom;

    S_PRINTF("Sets: registering domain %d\n", id);

    return 0;
}

void sets_unregister_domain(int id)
{
    struct sets_domain *curr_dom = sets_domainlist;
    struct sets_domain *prev_dom = NULL;

    while (curr_dom) {
	    if ((int)curr_dom->file_type == id) {
		    if (prev_dom)
			    prev_dom->next = curr_dom->next;
		    else
			    sets_domainlist = curr_dom->next;
		    SETS_FREE(curr_dom, sizeof(struct sets_domain));
		    S_PRINTF("Sets: unregistering domain %d\n", id);
		    break;
	    }
	    prev_dom = curr_dom;
	    curr_dom = curr_dom->next;
    }
}

inline int sets_exist_registrations(void)
{
    return (sets_domainlist != NULL);
}

struct sets_domain *sets_find_domain(FSOBJ_t vp)
{
    struct sets_domain *curr_dom = sets_domainlist;

    while (curr_dom) {
	    if (curr_dom->file_type == FILE_TYPE(vp))
		    return curr_dom;
	    curr_dom = curr_dom->next;
    }

    return NULL;
}
