#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* pid-based hash lookup routines for SETS subsystem */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_hash.h"

#define SETSHASH(pid) (&sets_hash_tbl[(pid) & SETSHASHMASK])
#define SETSHASHSIZE 256
#define SETSHASHMASK (SETSHASHSIZE-1)

static struct procSetInfo **sets_hash_tbl = 0;

void
sets_hashinit(void)
{
    int i;
    MYALLOC(sets_hash_tbl, struct procSetInfo **,
	    SETSHASHSIZE * sizeof(struct procSetInfo *));
    for (i=0; i< SETSHASHSIZE; i++)
	sets_hash_tbl[i] = NULL;
	     
}

void
sets_hashexit(void)
{
    /* assuming that all sets are closed, process entries removed. */
    if (sets_hash_tbl) {
	MYFREE(sets_hash_tbl, SETSHASHSIZE*sizeof(*sets_hash_tbl));
	sets_hash_tbl = 0;
    }
}

struct procSetInfo *
sets_hashlookup(pid)
     pid_t pid;
{
    struct procSetInfo *sp;
    
    if (!sets_hash_tbl)
	return NULL;
    
    for (sp = *(SETSHASH(pid)); sp != 0; sp = sp->next)
	if (PID(sp->p) == pid)
	    return (sp);
    return (NULL);
}

void
sets_hashinsert(sp)
     struct procSetInfo *sp;
{
    struct procSetInfo **bucket;
    pid_t pid = PID(sp->p);

    if (sets_hashlookup(pid))
	panic("sets_hashinsert: attempting to reinsert pid %d", pid);

    bucket = SETSHASH(pid);
    if ((sp->next = *bucket) != NULL)
	(*bucket)->prev = &sp->next;
    *bucket = sp;
    sp->prev = bucket;
}

void
sets_hashdelete(sp)	
     struct procSetInfo *sp;
{
    if (sp->next != NULL)
	sp->next->prev = sp->prev;
    *sp->prev = sp->next;
}

