#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _SETS_MESSAGE_H_
#define _SETS_MESSAGE_H_

/* Message structures etc. */
typedef enum {
	WM_OPEN = 1, 
	WM_EXPAND, 
	WM_QUERY, 
	WM_UNION, 
	WM_INTERSECT,
	WM_READ,
	WM_EXIT
} opType;

typedef struct workerMsg {
	struct workerMsg *next;
	struct workerMsg **prev;
	opType operation;
	int flags;
	u_long unique;		/* Process id of initiator of this msg */
	struct setType *set;
	union {
		struct {
			char *pathname;	/* Already expanded */
			char *query; /* Left to expand */
			FSOBJ_t vp; /* vnode for pathname */
		} expand;
		struct {
			unsigned int index; /* index to elem in resolved[] */
		} open;
		struct {
			unsigned int index; /* index to elem in resolved[] */
		} read;
		struct {
			char *pathname;	/* Already expanded (without cursor) */
			FSOBJ_t vp; /* vp for dir of query 
				    * result lookup */
			CURSOBJ_t handle; /* cursor handle */
			char *suffix; /* Remainder of path to expand */
		} query;
		struct {
			char *bm1; /* bitmaps to keep track of */
			char *bm2; /*  elems of sets to be merged */
			int bm1Size, bm2Size; /* sizes of the bitmaps */
		} merge;
	} msg;
} workerMsgType;

/* simple linked list for wildcard matches */
struct strlist {
	char *str;
	struct strlist *next;
};

/* Macros for managing messages. */

/* create and initialize a new message. */
#define MAKE_NEW_MSG(MSG, OP, SET) \
     MYALLOC((MSG), workerMsgType *, sizeof(workerMsgType)); \
     (MSG)->operation = (OP); \
     (MSG)->set = (SET); \
     (MSG)->unique = (SET)->unique; \
     CRITICAL((SET)->lock, (SET)->refct++; ); /* Hld set while msg alive */
     
/* create and initialize a new message inside a critical section --
 * assume set->lock is locked */
#define MAKE_NEW_MSG_NO_LOCK(MSG, OP, SET) \
     MYALLOC((MSG), workerMsgType *, sizeof(workerMsgType)); \
     (MSG)->operation = (OP); \
     (MSG)->set = (SET); \
     (MSG)->unique = (SET)->unique; \
     (SET)->refct++;		/* Hold set while msg alive */
     
/* create a new message based on an old message. */
#define MAKE_NEW_MSG_FROM_OLD(newmsg, oldmsg) \
     MYALLOC(newmsg, workerMsgType *, sizeof(workerMsgType)); \
     newmsg->set = oldmsg->set; \
     CRITICAL(newmsg->set->lock, newmsg->set->refct++); \
     newmsg->unique = oldmsg->unique; 

/* Prototypes for message handling functions
 *  only for sets' use in the kernel */
extern void           printMessageQueue __P((void));
extern void           initMessages      __P((void));
extern void           enqueueMessage    __P((workerMsgType *, int));
extern workerMsgType *dequeueMessage    __P((int));
extern void           purgeMessage      __P((workerMsgType *, int));

#endif /* _SETS_MESSAGE_H_ */
