#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Directory operations for Coda filesystem
 * Original version: (C) 1996 P. Braam and M. Callahan
 * Rewritten for Linux 2.1. (C) 1997 Carnegie Mellon University
 * 
 * Carnegie Mellon encourages users to contribute improvements to
 * the Coda project. Contact Peter Braam (coda@cs.cmu.edu).
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/string.h>

#include <kernel/sets/common/sets_internal.h>
#include <kernel/sets/common/sets_hash.h>
#include <kernel/sets/linux21/sets_ioctl.h>

static int sets_permission(struct inode *inode, int mask);
static int sets_open(struct inode *i, struct file *f);
static int sets_release(struct inode *i, struct file *f);
static int sets_ioctl(struct inode * inode, struct file * filp,
		      unsigned int cmd, unsigned long user_data);

struct file_operations sets_dir_operations = {
        NULL,                   /* lseek */
        NULL,                   /* read */
        NULL,                   /* write */
        NULL,			/* readdir */
        NULL,                   /* poll */
        sets_ioctl,		/* ioctl */
        NULL,                   /* mmap */
        sets_open,		/* open */
	NULL,			/* flush */
        sets_release,		/* release */
	NULL,			/* fsync */
        NULL,                   /* fasync */
	NULL,			/* check_media_change */
	NULL,			/* revalidate */
	NULL			/* lock */
};

struct inode_operations sets_dir_inode_operations =
{
	&sets_dir_operations,
	NULL,			/* create */
	NULL,			/* lookup */
	NULL,			/* link */
	NULL,			/* unlink */
	NULL,			/* symlink */
	NULL,			/* mkdir */
	NULL,   	        /* rmdir */
	NULL,		        /* mknod */
	NULL,			/* rename */
	NULL,	                /* readlink */
	NULL,	                /* follow_link */
	NULL,	                /* readpage */
	NULL,		        /* writepage */
	NULL,		        /* bmap */
	NULL,	                /* truncate */
	sets_permission,	/* permission */
	NULL,                   /* smap */
	NULL,                   /* update page */
        NULL			/* revalidate */
};

static int sets_permission(struct inode *inode, int mask)
{
        ENTRY;
	return 0;
}

static int sets_open(struct inode *i, struct file *f)
{
	ENTRY;
        return 0;
}

static int sets_release(struct inode *i, struct file *f)
{
        struct procSetInfo *settbl;

        ENTRY;

	/* Gotcha: if the process opens /sets a second time, or 
	 * (shudder) an open set contained /sets, we'll get a call
	 * here.  need to distinguish from the "real" one. */
	if (f->private_data == SETS_IOCTL_MAGIC) {
		if ((settbl = sets_hashlookup(PID(current))) != NULL)
		        sets_proc_exit(settbl);
	} else
		printk("sets: avoided proc_exit for non-ioctl filp\n");

	EXIT;
        return 0;
}

static int sets_ioctl(struct inode *inode, struct file *filp,
		      unsigned int cmd, unsigned long user_data)
{
	int len = _IOC_SIZE(cmd);

	ENTRY;

	/* label the filp on which ioctls are done by marking the
	 * private_data.  the aim here is to avoid calling
	 * sets_proc_exit at the wrong time (see sets_release) */
	if (!filp->private_data)
		filp->private_data = SETS_IOCTL_MAGIC;

	switch (cmd) {
	case SETSI_OPEN:
		if (len != sizeof(struct setsi_open_args)) return -EINVAL;
		return setsi_open((struct setsi_open_args *) user_data);
	case SETSI_CLOSE:
		if (len != sizeof(struct setsi_close_args)) return -EINVAL;
		return setsi_close((struct setsi_close_args *) user_data);
	case SETSI_DAEMON:
		return setsi_daemon();
	case SETSI_DAEMON_KILL:
		return setsi_daemon_kill();
	case SETSI_ITERATE:
		if (len != sizeof(struct setsi_iterate_args)) return -EINVAL;
		return setsi_iterate((struct setsi_iterate_args *) user_data);
	case SETSI_DIGEST:
		if (len != sizeof(struct setsi_digest_args)) return -EINVAL;
		return setsi_digest((struct setsi_digest_args *) user_data);
	case SETSI_REW_ITER:
		if (len != sizeof(struct setsi_rew_iter_args)) return -EINVAL;
		return setsi_rew_iter((struct setsi_rew_iter_args *) user_data);
	case SETSI_REW_DIG:
		if (len != sizeof(struct setsi_rew_dig_args)) return -EINVAL;
		return setsi_rew_dig((struct setsi_rew_dig_args *) user_data);
	case SETSI_MEMBER:
		if (len != sizeof(struct setsi_member_args)) return -EINVAL;
		return setsi_member((struct setsi_member_args *) user_data);
	case SETSI_SIZE:
		if (len != sizeof(struct setsi_size_args)) return -EINVAL;
		return setsi_size((struct setsi_size_args *) user_data);
	case SETSI_INTER:
		if (len != sizeof(struct setsi_inter_args)) return -EINVAL;
		return setsi_inter((struct setsi_inter_args *) user_data);
	case SETSI_UNION:
		if (len != sizeof(struct setsi_union_args)) return -EINVAL;
		return setsi_union((struct setsi_union_args *) user_data);
	case SETSI_RESTRICT:
		if (len != sizeof(struct setsi_restrict_args)) return -EINVAL;
		return setsi_restrict((struct setsi_restrict_args *) user_data);
	default:
		return -EINVAL;
	}
}

