#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_hash.h"
#include "kernel/sets/linux21/sets_ioctl.h"

int setsi_open(struct setsi_open_args *ap)
{
	struct procSetInfo *settbl;

	if (ap->spec == 0)
		return -EINVAL;

	if ((settbl = sets_hashlookup(PID(current))) == NULL) {
		MYALLOC(settbl, struct procSetInfo *,
			sizeof(struct procSetInfo));
		memset(settbl, 0, sizeof(struct procSetInfo));
		settbl->p = current;
		sets_hashinsert(settbl);
	}
	
	if (settbl->nOpen > NOPENSETS)
		return EMFILE;

	return do_set_open(settbl, ap->spec, ap->flags, &ap->handle);
}

int setsi_close(struct setsi_close_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_close(settbl, ap->handle);
}

int setsi_daemon(void)
{
	return do_set_daemon();
}

int setsi_daemon_kill(void)
{
        kill_daemons();
	return 0;
}

int setsi_iterate(struct setsi_iterate_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_iterate(settbl, ap->handle, ap->flags, ap->name,
			      ap->buffer_size, ap->error, &ap->fd);
}

int setsi_digest(struct setsi_digest_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_digest(settbl, ap->handle, ap->buffer, ap->buffer_size,
			     &ap->retsz);
}

int setsi_rew_iter(struct setsi_rew_iter_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_rewind_iter(settbl, ap->handle);
}

int setsi_rew_dig(struct setsi_rew_dig_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_rewind_dig(settbl, ap->handle);
}

int setsi_member(struct setsi_member_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_member(settbl, ap->handle, ap->name);
}

int setsi_size(struct setsi_size_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_size(settbl, ap->handle, &ap->size);
}

int setsi_inter(struct setsi_inter_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_intersect(settbl, ap->handle1, ap->handle2,
				ap->flags, &ap->handle);
}

int setsi_union(struct setsi_union_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_union(settbl, ap->handle1, ap->handle2,
				  ap->flags, &ap->handle);
}

int setsi_restrict(struct setsi_restrict_args *ap)
{
	struct procSetInfo *settbl;

	if ((settbl = sets_hashlookup(PID(current))) == NULL)
		return EINVAL;

	return do_set_restrict(settbl, ap->handle1, ap->fname, ap->flags,
			       &ap->handle);
}
