#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef __SETS_LINUX_H__
#define __SETS_LINUX_H__

/*------------------------------------------------------+
 | all Linux system includes that are necessary         |
 +------------------------------------------------------*/

#ifdef __KERNEL__

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include <linux/file.h>
#include <asm/uaccess.h>

/*------------------------------------------------------+
 | Linux module glue magic                              |
 +------------------------------------------------------*/

#define SETS_SUPER_MAGIC 0x61728394

extern struct wait_queue wait;

/*------------------------------------------------------+
 | Memory management                                    |
 +------------------------------------------------------*/

#define SETS_ALLOC(ptr, cast, size) \
do {                                                                      \
    ptr = (cast)kmalloc((size_t) size, GFP_KERNEL);           \
/*    printk("sets malloc (%p) %d  %s:%d\n", ptr, size, __FILE__, __LINE__);*/\
    if (ptr == 0) {                                                       \
        panic("kernel malloc returns 0 at %s:%d\n", __FILE__, __LINE__);  \
    }                                                                     \
} while (0)
#define SETS_FREE(ptr, size)         \
do {                                    \
/*    printk("sets free (%p) %s:%d\n", ptr, __FILE__, __LINE__);    */ \
    kfree_s((ptr),(size_t)size);                                   \
} while (0)

#define MYALLOC SETS_ALLOC
#define MYFREE  SETS_FREE
#define STRALLOC(ptr) SETS_ALLOC(ptr, char *, SETS_MAXPATHLEN)
#define STRFREE(ptr)  SETS_FREE(ptr, SETS_MAXPATHLEN)

#define COPYINSTRING(src, dst, len, result) \
do { \
	 result = strncpy_from_user(dst, src, len); \
} while(0);

#define COPYTOUSERSPACE(src, dst, len) copy_to_user(dst, src, len)
#define PUT_USER_BYTE(x, dst) put_user(x, (char *)(dst))
#define PUT_USER_INT(x, dst) put_user(x, (int *)(dst))
#define PUT_USER_LONG(x, dst) put_user(x, (long *)(dst))

/*------------------------------------------------------+
 | Linux-specific type bindings                         |
 +------------------------------------------------------*/

typedef struct task_struct *PROCESS_t;
typedef struct dentry      *FSOBJ_t;
typedef struct file        *FILEOBJ_t;
typedef int                 FILEID;
typedef struct file        *CURSOBJ_t;
typedef struct semaphore    SETS_MUTEX;
typedef struct wait_queue  *CONDITION;

/*------------------------------------------------------+
 | Process manipulation macros                          |
 +------------------------------------------------------*/

#define PID(p) ((p)->pid)
#define CURPID (current->pid)

/*------------------------------------------------------+
 | Directory manipulation macros                        |
 +------------------------------------------------------*/

#define Process_cdir    (current->fs->pwd)
#define Process_root    (current->fs->root)
#define IS_DIR_OBJ(de)  (S_ISDIR(de->d_inode->i_mode))
#define COMPONENT_MARKER '/'
#define COMPONENT_STRING "/"

#define SETS_MAXPATHLEN 1024

/*------------------------------------------------------+
 | FS object manipulation macros                        |
 +------------------------------------------------------*/

#define FILE_TYPE(de)      (de->d_inode->i_sb->s_magic)
#define FSOBJ_REF(de)      dget(de)
#define FSOBJ_RELE(de)     dput(de)
#define FSOBJ_UNLOCK(a)	   /* nothing to do - namei doesn't lock the leaf */
int OPEN_FSOBJ(struct dentry *, int, /* sets_linux.c */
	       struct task_struct *, struct file **);
#define OPEN_CURSOBJ  OPEN_FSOBJ
#define CLOSE_CURSOBJ(c,p) fput(c)

#define SETS_READ  O_RDONLY
#define SETS_WRITE O_WRONLY

#define CURS_TO_FS(c)      ((c)->f_dentry)

#define FID_INVALID(f) (f<0)

/*------------------------------------------------------+
 | File manipulation macros                             |
 +------------------------------------------------------*/

#define RELEASE_FP(a,b)	   /* XXX what goes here? XXX */

/*------------------------------------------------------+
 | Synchronization primitives                           |
 +------------------------------------------------------*/

#define MUTEX_INIT(mutex)   sema_init(mutex, 1)
#define MUTEX_LOCK(mutex)   down(mutex)
#define MUTEX_UNLOCK(mutex) up(mutex)

#define CONDITION_INIT(cond) \
do { \
    printk("initting condition %p\n", cond); \
    init_waitqueue(cond); \
} while(0);
#define CONDITION_SIGNAL(cond) \
do { \
    printk("signalling on %p\n", cond); \
    wake_up(cond); \
} while(0);
#define CONDITION_WAIT(cond,mutex) \
do { \
    printk("waiting on %p\n", cond); \
    if (mutex) MUTEX_UNLOCK(mutex); \
    sleep_on(cond); /* uninterruptible -- avoid dealing with ^-C */ \
    if (mutex) MUTEX_LOCK(mutex); \
} while(0);

/*------------------------------------------------------+
 | Miscellaneous                                        |
 +------------------------------------------------------*/

#ifndef ENTRY
#define ENTRY   \
    if(setsDebugLevel) printk("Process %d entered %s\n",current->pid,__FUNCTION__)
#endif

#ifndef EXIT
#define EXIT    \
    if(setsDebugLevel) printk("Process %d leaving %s\n",current->pid,__FUNCTION__)
#endif

#define LOG_ENTRY(a,b)
#define LOG_EXIT(a,b)

#define printf printk
#define S_PRINTF printk
#define __P(v) v
#define exit(foo) panic("DEATH!")

#endif

#endif
