#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/locks.h>
#include <linux/unistd.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/lp.h>
#include <linux/malloc.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/skbuff.h>
#include <linux/vmalloc.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/poll.h>
#include <asm/uaccess.h>

#include <kernel/sets/common/sets_internal.h>
#include <kernel/sets/common/sets_domain.h>

struct wait_queue *work_q;

extern struct inode_operations sets_dir_inode_operations;

/* VFS super_block ops */
static struct super_block *sets_read_super(struct super_block *, void *, int);
static void sets_read_inode(struct inode *);
static void sets_put_inode(struct inode *);
static void sets_put_super(struct super_block *);
static int sets_statfs(struct super_block *sb, struct statfs *buf, 
		      int bufsiz);

/* exported operations */
struct super_operations sets_super_operations =
{
	sets_read_inode,	/* read_inode */
	NULL,                   /* write_inode */
	sets_put_inode,	        /* put_inode */
	NULL,			/* delete_inode */
	NULL,			/* notify_change */
	sets_put_super,	        /* put_super */
	NULL,			/* write_super */
	sets_statfs,   		/* statfs */
	NULL,			/* remount_fs */
	NULL,			/* clear_inode */
	NULL			/* umount_begin */
};

static struct super_block * sets_read_super(struct super_block *sb, 
					    void *data, int silent)
{
        struct inode *root = 0; 

	ENTRY;
        MOD_INC_USE_COUNT; 

        lock_super(sb);
        sb->u.generic_sbp = NULL;
        sb->s_blocksize = 1024;	/* XXXXX  what do we put here?? */
        sb->s_blocksize_bits = 10;
        sb->s_magic = SETS_SUPER_MAGIC;
        sb->s_op = &sets_super_operations;

	root = iget(sb, 0);
	root->i_op = &sets_dir_inode_operations;
	root->i_mode = S_IFDIR;
	sb->s_root = d_alloc_root(root, NULL);
	unlock_super(sb);

	init_waitqueue(&work_q);
	setInit();

	EXIT;  
        return sb;
}

static void sets_put_super(struct super_block *sb)
{
        ENTRY;

	setExit();

        MOD_DEC_USE_COUNT;
	EXIT;
}

static void sets_read_inode(struct inode *inode)
{
	ENTRY;
	EXIT;
}

static void sets_put_inode(struct inode *in) 
{
	ENTRY;

	if ( in->i_count == 1 ) 
		in->i_nlink = 0;

	EXIT;
}

/*  we need _something_ for this routine. Let's mimic AFS */
static int sets_statfs(struct super_block *sb, struct statfs *buf, 
		       int bufsiz)
{
	struct statfs tmp;

	tmp.f_type = SETS_SUPER_MAGIC;
	tmp.f_bsize = 1024;
	tmp.f_blocks = 9000000;
	tmp.f_bfree = 9000000;
	tmp.f_bavail = 9000000;
	tmp.f_files = 9000000;
	tmp.f_ffree = 9000000;
	tmp.f_namelen = 0;
	copy_to_user(buf, &tmp, bufsiz);
	return 0; 
}

struct file_system_type sets_fs_type = {
   "sets", 0, sets_read_super, NULL
};

#ifdef MODULE

MODULE_AUTHOR("Kip Walker <kwalker@cs.cmu.edu>\n");

EXPORT_SYMBOL(sets_register_domain);
EXPORT_SYMBOL(sets_unregister_domain);

int init_module(void)
{
	int status;
	printk(KERN_INFO "Dynamic Sets module 0.1\n");
	setsDebugLevel = 0xFFFF;

	status = register_filesystem(&sets_fs_type);
	if (status) {
		printk("sets: failed in register_filesystem!\n");
	}
	
/*	sets_sysctl_init();*/
	
	return status;
}

/* XXX this seems not to get called; what's going on? XXX */
void cleanup_module(void)
{
        int err;

        ENTRY;

        if ( (err = unregister_filesystem(&sets_fs_type)) != 0 ) {
                printk("sets: failed to unregister filesystem\n");
        }

/*        sets_sysctl_clean();*/

	printk(KERN_INFO "Dynamic Sets module 0.1 exitting.\n");
}

#endif
