#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/win32/sets_win32_private.h"

DLLEXPORT SETID WINAPI set_open(const char *spec, UINT flags)
{
	int   retval;
	SETID handle;

	if (spec == NULL)
		return -1;

	if (settbl.nOpen > NOPENSETS)
		return -1;

	retval = do_set_open(&settbl, spec, flags, &handle);
	return (!retval) ? handle : (SETID)-1;
}

DLLEXPORT VOID WINAPI set_close(SETID handle)
{
	do_set_close(&settbl, handle);
}

DLLEXPORT INT WINAPI set_digest(SETID handle, struct DigestT *buf,
								UINT bufsize)
{
	int retval, retsz;

	retval = do_set_digest(&settbl, handle, (char *)buf, bufsize, &retsz);
	if (!retval)
		return retsz;
	else
		return retval;
}

DLLEXPORT HANDLE WINAPI set_iterate(SETID handle, UINT flags, PSTR buffer,
									UINT bufsize, PUINT error)
{
	HANDLE fp;
	int retval;

	retval = do_set_iterate(&settbl, handle, flags, buffer, bufsize,
							error, &fp);
	return (!retval) ? fp : INVALID_HANDLE_VALUE;
}

DLLEXPORT INT WINAPI set_rewind_digest(SETID handle)
{
	return do_set_rewind_dig(&settbl, handle);
}

DLLEXPORT INT WINAPI set_rewind_iterator(SETID handle)
{
	return do_set_rewind_iter(&settbl, handle);
}

DLLEXPORT INT WINAPI set_size(SETID handle)
{
	int retval;
	INT size;

	retval = do_set_size(&settbl, handle, &size);
	return (!retval) ? size : -1;
}

DLLEXPORT SETID WINAPI set_intersect(SETID handle1, SETID handle2, UINT flags)
{
	int retval;
	SETID set;

	retval = do_set_intersect(&settbl, handle1, handle2, flags, &set);
	return (!retval) ? set : -1;	
}

DLLEXPORT SETID WINAPI set_union(SETID handle1, SETID handle2, UINT flags)
{
	int retval;
	SETID set;

	retval = do_set_union(&settbl, handle1, handle2, flags, &set);
	return (!retval) ? set : -1;	
}
