#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include "kernel/sets/common/sets_internal.h"
#include "sets_win32_private.h"

HANDLE threads[NUM_DAEMONS];
struct procSetInfo settbl;

#pragma data_seg("Shared")
FILE *f;
#pragma data_seg()

BOOL WINAPI DllMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
{
	char fname[200];

    switch (fdwReason)
    {
        case DLL_PROCESS_ATTACH:
            // The DLL is being loaded for the first time by a given process.
            // Perform per-process initialization here.  If the initialization
            // is successful, return TRUE; if unsuccessful, return FALSE.

			memset((void *)&settbl, 0, sizeof(struct procSetInfo));
			settbl.p = GetCurrentProcess();
			wsprintf(fname, "D:\\setslog%d.txt", GetCurrentProcessId());
			freopen(fname, "w", stdout);
			setInit();

			if (startDaemons(NUM_DAEMONS))
				exit(-1);

            break;

        case DLL_PROCESS_DETACH:
            // The DLL is being unloaded by a given process.  Do any
            // per-process clean up here, such as undoing what was done in
            // DLL_PROCESS_ATTACH.  The return value is ignored.
//			fclose(f);
            break;

        case DLL_THREAD_ATTACH:
            // A thread is being created in a process that has already loaded
            // this DLL.  Perform any per-thread initialization here.  The
            // return value is ignored.

            break;

        case DLL_THREAD_DETACH:
            // A thread is exiting cleanly in a process that has already
            // loaded this DLL.  Perform any per-thread clean up here.  The
            // return value is ignored.

            break;
    }
    return TRUE;
}

int startDaemons(UINT num_daemons)
{
	UINT i;

	for (i=0; i<num_daemons; i++) {
		DWORD tid;

		if ((threads[i] = CreateThread(NULL, 0, Daemon_Main, 
									   (LPVOID)i, 0, &tid))==NULL)
			return -1;
		S_PRINTF("new tid %x (%d)\n", threads[i], threads[i]);
	}

	return 0;
}

// XXX where can I have this be invoked??  PROCESS_DETACH is too late!
void stopDaemons(void)
{
	int i;

	kill_daemons();

	WaitForMultipleObjects(numWorkers, threads, TRUE, INFINITE);

	for (i=0; i<numWorkers; i++)
		CloseHandle(threads[i]);

	numWorkers = 0;
}

DWORD WINAPI Daemon_Main(LPVOID arg)
{
	return do_set_daemon();
}

void init_bufferManagement(void)
{
}

void purgePrefetchBuffers(void)
{
}

void close_elem(elemType *elem, PROCESS_t owner)
{
	if (elem->fsp != (FSOBJ_t)-1)
		CloseHandle(elem->fsp);
}

int open_object(struct setType *set, elemType *elem, int myindex)
{
	HANDLE fp;

	// XXX deal with prefetching case -- registered domains, etc.

	fp = CreateFile(elem->name, GENERIC_READ, FILE_SHARE_READ,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (fp == INVALID_HANDLE_VALUE)
		return ENFILE;

	elem->fsp = fp;

	return 0;
}

FILEID setup_fd(struct setType *set, int setnum, int elem)
{
	if (!DuplicateHandle(GetCurrentProcess(), set->resolved[elem].fsp,
					GetCurrentProcess(), &(set->resolved[elem].fd),
					0, FALSE, DUPLICATE_SAME_ACCESS))
		exit(-1);

	return set->resolved[elem].fd;
}

int expand_part(FSOBJ_t vp, char *name, FSOBJ_t *vpp,
		workerMsgType *msg)
{
	return 0;
}

int expand_wildcard(char *prefix, char *pattern,
					workerMsgType *msg, int myindex)
{
	char *suffix;
	HANDLE curs;
	WIN32_FIND_DATA data;
	int prefix_len = pattern-prefix;

	if (*pattern == '\\')
		pattern++;

	for (suffix = pattern; *suffix && *suffix != '\\'; suffix++)
		;
	if (*suffix)
		*suffix++ = '\0';

	curs = FindFirstFile((LPCTSTR)prefix, &data);
	if (curs == INVALID_HANDLE_VALUE)
		return 0;
	
	do {
		char *word = data.cFileName;

		if (*word == '.' && ((word[1] == '.' && word[2] == '\0') ||
							 word[1] == '\0'))
			continue;

		DEBUG(PATHNAME, S_PRINTF("(%d): glob matched %s\n",
				       myindex, word););
		if (*suffix != '\0') {
			workerMsgType *newmsg;
				
			MAKE_NEW_MSG_FROM_OLD(newmsg, msg);
			newmsg->operation = WM_EXPAND;
			newmsg->msg.expand.pathname = NULL;
			newmsg->msg.expand.vp = NULL;
			STRALLOC(newmsg->msg.expand.query);
			strncpy(newmsg->msg.expand.query, prefix, prefix_len);
			strcpy(newmsg->msg.expand.query+prefix_len, "\\");
			strcat(newmsg->msg.expand.query, word);
			strcat(newmsg->msg.expand.query, "\\");
			strcat(newmsg->msg.expand.query, suffix);
			msg->set->expands++;
			enqueueMessage(newmsg, myindex);
		} else {
			/* don't expand - just insert it! */
			char *newmember;
				
			STRALLOC(newmember);
			strncpy(newmember, prefix, prefix_len);
			strcpy(newmember+prefix_len, "\\");
			strcat(newmember, word);
			insertNewMember(msg->set, newmember, myindex);
		}
	} while (FindNextFile(curs, &data));

	return 0;
}
