#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _SETS_WIN32_H_
#define _SETS_WIN32_H_

/*------------------------------------------------------+
 | all WinNT includes that are necessary                |
 +------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <string.h>

/*------------------------------------------------------+
 | Memory management                                    |
 +------------------------------------------------------*/

#define SETS_ALLOC(ptr, cast, size) \
do {                                                                      \
/*    S_PRINTF("sets malloc %d  %s:%d\n", size, __FILE__, __LINE__); */     \
    ptr = (cast)malloc((unsigned long) size);                             \
    if (ptr == 0) {                                                       \
		exit(-1);                                                         \
    }                                                                     \
} while (0)
#define SETS_FREE(ptr, size)         \
do {                                    \
/*    S_PRINTF("sets free %s:%d\n", __FILE__, __LINE__); */ \
    free((ptr));                                          \
} while (0)

#define MYALLOC       SETS_ALLOC
#define MYFREE        SETS_FREE
#define STRALLOC(ptr) SETS_ALLOC(ptr, char *, MAX_PATH)
#define STRFREE(ptr)  SETS_FREE(ptr, MAX_PATH)

#define COPYINSTRING(s,d,l,err) { strncpy(d,s,l); err=0;}
#define COPYTOUSERSPACE(s,d,l)  memcpy(d,s,l)
#define PUT_USER_LONG(f,t)      (*(long *)(t)) = f
#define PUT_USER_INT(f,t)       (*(int *)(t)) = f
#define PUT_USER_BYTE(f,t)      (*(char *)(t)) = f

/*------------------------------------------------------+
 | Win32 type bindings                                  |
 +------------------------------------------------------*/

typedef HANDLE           PROCESS_t;
typedef HANDLE           FSOBJ_t;
typedef HANDLE           FILEOBJ_t;
typedef HANDLE           FILEID;
typedef HFILE            CURSOBJ_t;
typedef CRITICAL_SECTION SETS_MUTEX;
typedef	HANDLE           CONDITION;

/*------------------------------------------------------+
 | Process manipulation macros                          |
 +------------------------------------------------------*/

#define CURPID GetCurrentProcessId()

/*------------------------------------------------------+
 | Directory manipulation macros                        |
 +------------------------------------------------------*/

#define Process_cdir
#define Process_root
#define IS_DIR_OBJ(vp)  1
#define COMPONENT_MARKER '\\'
#define COMPONENT_STRING "\\"

#define SETS_MAXPATHLEN MAX_PATH

/*------------------------------------------------------+
 | FS object manipulation macros                        |
 +------------------------------------------------------*/

#define FILE_TYPE(vp)         
#define FSOBJ_REF(vp)         
#define FSOBJ_RELE(vp)        
#define FSOBJ_UNLOCK(vp)      
#define OPEN_FSOBJ(vp,m,p)    1
#define OPEN_CURSOBJ(vp,mode,p,cvpp)
#define CLOSE_CURSOBJ(vp,p)   

#define SETS_READ  FREAD
#define SETS_WRITE FWRITE

#define CURS_TO_FS(c)         c

/*------------------------------------------------------+
 | File manipulation macros                             |
 +------------------------------------------------------*/

#define FID_INVALID(f)  (f == INVALID_HANDLE_VALUE)
#define RELEASE_FP(elem, p) \
{ \
}

/*------------------------------------------------------+
 | Synchronization primitives                           |
 +------------------------------------------------------*/

#define MUTEX_INIT(m)        InitializeCriticalSection(m)
#define MUTEX_DESTROY(m)     ReleaseCriticalSection(m)
#define MUTEX_LOCK(m)        EnterCriticalSection(m)
#define MUTEX_UNLOCK(m)      LeaveCriticalSection(m)

#define CONDITION_INIT(c)    *(c) = CreateEvent(NULL, FALSE, FALSE, NULL)
#define CONTITION_DESTROY(c) CloseHandle(*(c))

#define CONDITION_WAIT(cond, lock)                                 \
do  {                                                              \
	DEBUG(THREAD,S_PRINTF("Waiting on condition %p %s:%d\n", *(cond),\
		  __FILE__,__LINE__););                                    \
	if (lock) MUTEX_UNLOCK(lock);                                  \
	WaitForSingleObject(*(cond), INFINITE);                        \
	if (lock) MUTEX_LOCK(lock);                                    \
} while(0);

#define CONDITION_SIGNAL(cond)                            \
do  {                                                     \
	DEBUG(THREAD,S_PRINTF("Signaling condition %p %s:%d\n", \
		  *(cond),__FILE__,__LINE__););                   \
	SetEvent(*(cond));                                    \
} while (0);

/*------------------------------------------------------+
 | Miscellaneous                                        |
 +------------------------------------------------------*/

#define __P(p) p

#define LOG_ENTRY(THREAD, TAG)
#define LOG_EXIT(THREAD, TAG)

#ifdef _READ
#undef PRE_READ			/* buffer cache hacks not implemented */
#endif

#define S_PRINTF printf

#define NUM_DAEMONS   4

/* errors */
#define EINVAL        1
#define EMFILE        2
#define ENFILE        3
#define EBADF         4
#define ENOENT        5

#endif /* _SETS_WIN32_H_ */
