#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* pid-based hash lookup routines for upcall subsystem */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/malloc.h>
#include <sys/proc.h>

#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_pending.h"
#include "kernel/upcall/includes/uc_hash.h"

#define UCHASH(pid) (&uc_hash_tbl[(pid) & uc_hash])

static LIST_HEAD(uc_hash_head_t, uc_info_t) *uc_hash_tbl;
static u_long uc_hash;

void
uc_hashinit(void)
{
    uc_hash_tbl = hashinit(maxproc / 4, M_TEMP, &uc_hash);
}

struct uc_info_t *
uc_hashlookup(pid)
     pid_t pid;
{
	struct uc_info_t *ucip;

	if (!uc_hash_tbl)
		return NULL;

	for (ucip = UCHASH(pid)->lh_first; ucip != 0;
	     ucip = ucip->uc_hash.le_next)
		if (ucip->uc_pid == pid)
			return (ucip);
	return (NULL);
}

void
uc_hashinsert(ucip)
     struct uc_info_t *ucip;
{
	pid_t pid = ucip->uc_pid;

	if (uc_hashlookup(pid))
		panic("uc_hashinsert: attempting to reinsert pid %d", pid);

	LIST_INSERT_HEAD(UCHASH(pid), ucip, uc_hash);
}

void
uc_hashdelete(ucip)
     struct uc_info_t *ucip;
{
	LIST_REMOVE(ucip, uc_hash);
}

