#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/malloc.h>
#include <sys/queue.h>

#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_upclass.h"
#include "uc_pending.h"

/* Used to post (asynchronous only) messages from within the kernel */

int
upcall_async(upclass, upcall, pid, argsz, argp, argmtype)
	upclass_t upclass;
	upcall_t  upcall;
	pid_t     pid;
	size_t    argsz;
	void      *argp;
	int       argmtype;
{
	struct uc_ucld_t   *ucldp;
	int                plsz;
	struct uc_plist_t  *plp=0;
	int                result = 0;

	if (upclass <= 0 || upclass > upcl_tabsz) {
		return EINVAL;
	}

	ucldp = upcl_tab[upclass];
	if (!ucldp->ucld_inited) {
		return EINVAL;
	}

	/* 
	 * Returns a negative number on error, the negation of which
	 * is the errno code. 
	 */
	plsz = (ucldp->ucld_rcpt)(ucldp, ucldp->ucld_table, pid, upcall, &plp);
	if (plsz < 0) 
		return -plsz;

	if (plsz == 0)
		return ESRCH;

	result = uc_post_async(plsz, plp, upclass, upcall, argsz,
			       argp, argmtype);
	return result;
}
	
