#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

/*
 * Stolen yet again from Odyssey
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/conf.h>
#include <sys/namei.h>
#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/vnode.h>
#ifdef __FreeBSD__ /* XXX gruesome hack */
#include "specdev.h"
#else
#include <miscfs/specfs/specdev.h>
#endif

#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_vfsops.h"
#include "uc_vnops.h"

#include "upcall_mount.h"

int ucfs_vfsop_print_entry = 0;
#ifdef __GNUC__
#define ENTRY    \
    if(ucfs_vfsop_print_entry) printf("Entered %s\n",__FUNCTION__)
#else
#define ENTRY
#endif 

extern int (**ucfs_vnodeop_p)(void *);

#ifdef	__NetBSD__
struct vfsops ucfs_vfsops = {
    MOUNT_UPCALL,
    ucfs_mount,
    ucfs_start,
    ucfs_unmount, /* just return EBUSY for now? */
    ucfs_root,
    (int (*) (struct mount *, int, uid_t, caddr_t,
	      struct proc *)) eopnotsupp,
    ucfs_statfs,
    ucfs_sync, 
    (int (*) (struct mount *, ino_t, struct vnode **)) eopnotsupp,
    (int (*) (struct mount *, struct fid *, struct mbuf *, struct vnode **,
	      int *, struct ucred **)) eopnotsupp,
    (int (*) (struct vnode *, struct fid *)) eopnotsupp,
    ucfs_init,
#ifdef	NetBSD1_3
    (int (*)(void)) eopnotsupp,
#endif
    0
};

#elif	defined(__FreeBSD__)
struct vfsops ucfs_vfsops = {
    ucfs_mount,
    ucfs_start,
    ucfs_unmount, /* just return EBUSY for now? */
    ucfs_root,
    (int (*) (struct mount *, int, uid_t, caddr_t,
	      struct proc *)) eopnotsupp,
    ucfs_statfs,
    ucfs_sync,
    (int (*) (struct mount *, ino_t, struct vnode **)) eopnotsupp,
    (int (*) (struct mount *, struct fid *, struct mbuf *, struct vnode **,
	      int *, struct ucred **)) eopnotsupp,
    (int (*) (struct vnode *, struct fid *)) eopnotsupp,
    ucfs_init,
};

#endif

struct vnode *ucfs_root_vp = NULL;

/*
 * Upcall mount vfsop
 * Set up mount info record and attach it to vfs struct.
 */
/*ARGSUSED*/
int ucfs_mount(vfsp, path, data, ndp, p)
    struct mount *vfsp;    /* Allocated and initialized by mount(2) */
#ifdef NetBSD1_3
    const char *path;      /* path covered: ignored by the fs-layer */
    void *data;            /* Need to define a data type for this in netbsd? */
#else
    char *path;            /* path covered: ignored by the fs-layer */
    caddr_t data;          /* Need to define a data type for this in netbsd? */
#endif
    struct nameidata *ndp;
    struct proc *p;        /* The ever-famous proc pointer */
{
    int error;

    if (p->p_cred->pc_ucred->cr_uid != 0) { /* only root can mount */
	return(EPERM);
    }

    if (ucfs_root_vp) { /* can only mount once */
	return(EBUSY);
    }

    /*
     * Make a root vnode
     */

    error = getnewvnode(VT_UPCALL, vfsp, ucfs_vnodeop_p, &ucfs_root_vp);
    if (error)
	panic("ody: getnewvnode returned error %d\n", error);

    ucfs_root_vp->v_type = VCHR; /* So we can do ioctls on it */
    ucfs_root_vp->v_flag = VROOT;

    /* This is a really bad hack - copied from Odyssey vfs implementation */
    ucfs_root_vp->v_specinfo = (struct specinfo *)
      malloc((u_long) sizeof(struct specinfo), M_VNODE, M_NOWAIT);
    ucfs_root_vp->v_hashchain = &ucfs_root_vp; 
    ucfs_root_vp->v_specnext = ucfs_root_vp;
    ucfs_root_vp->v_specflags = 0;
    ucfs_root_vp->v_specinfo->si_rdev = makedev(0, 0);
#ifndef __FreeBSD__
    ucfs_root_vp->v_speclockf = NULL;
#endif

    return(0);
}

int
ucfs_start(vfsp, flags, p)
    struct mount *vfsp;
    int flags;
    struct proc *p;
{
    ENTRY;
    uc_init();
    return (0);
}

ucfs_statfs(vfsp, sbp, p)
    register struct mount *vfsp;
    struct statfs *sbp;
    struct proc *p;
{
    ENTRY;


    bzero(sbp, sizeof(struct statfs));
    /* XXX - what to do about f_flags, others? --bnoble */
    /* Below This is what AFS does
    	#define NB_SFS_SIZ 0x895440
     */
    /* Note: Normal fs's have a bsize of 0x400 == 1024 */
#ifdef	__NetBSD__
    sbp->f_type = 0;
#elif	defined(__FreeBSD__)
    sbp->f_type = MOUNT_UPCALL;
#endif
    sbp->f_bsize = 8192;	/* XXX -JJK */
    sbp->f_iosize = 8192;
#define NB_SFS_SIZ 0x8AB75D
    sbp->f_blocks = NB_SFS_SIZ;
    sbp->f_bfree = NB_SFS_SIZ;
    sbp->f_bavail = NB_SFS_SIZ;
    sbp->f_files = NB_SFS_SIZ;
    sbp->f_ffree = NB_SFS_SIZ;
    bcopy((caddr_t)&(vfsp->mnt_stat.f_fsid), (caddr_t)&(sbp->f_fsid),
	  sizeof (fsid_t));
#ifdef	__NetBSD__
    strncpy(sbp->f_fstypename, MOUNT_UPCALL, MFSNAMELEN-1);
#endif
    strcpy(sbp->f_mntonname, "/ucfs"); /* XXX cheesy */
    strcpy(sbp->f_mntfromname, "upcall");
    
    return(0);
}

int
ucfs_unmount(vfsp, mntflags, p)
    struct mount *vfsp;
    int mntflags;
    struct proc *p;
{
    struct vnode* vp;

    ENTRY;

    /* If there are outstanding registrations, don't allow module to unload. */
    if (uc_exist_registrations()) 
	return (EBUSY);

    vrele(ucfs_root_vp);
    vflush(vfsp, NULLVP, FORCECLOSE);

    ucfs_root_vp = NULL;
    return (0);
}

/*
 * Flush any pending I/O - which, of cource, is a no-op for ucfs.
 */
int
ucfs_sync(vfsp, waitfor, cred, p)
    struct mount *vfsp;
    int    waitfor;
    struct ucred *cred;
    struct proc *p;
{
    return (0);
}


/*
 * find root of odyssey file sys
 */
int
ucfs_root(vfsp, vpp)
    struct mount *vfsp;
    struct vnode **vpp;
{
    *vpp = ucfs_root_vp;
    vref(*vpp);
    VOP_LOCK(*vpp);
    return (0);
}

#ifdef __NetBSD__
void
ucfs_init()
{
    ENTRY;
}
#elif defined(__FreeBSD__)
int
ucfs_init()
{
    ENTRY;
    return 0;
}
#endif
