#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1990 Carnegie-Mellon University
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

#include <sys/param.h>
#include <sys/malloc.h>
#include <sys/errno.h>
#include <sys/acct.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/namei.h>
#include <sys/ioctl.h>
#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/vnode.h>
#include <sys/signal.h>
#ifdef NetBSD1_3
#include <sys/poll.h>
#include <sys/user.h>
#include <miscfs/genfs/genfs.h>
#endif
#include <vm/vm.h>

#ifdef	__FreeBSD__
#include <vm/vm_object.h>
#include <vm/vm_extern.h>
#endif

#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_upclass.h"
#include "uc_pending.h"
#include "uc_ioctl.h"
#include "kernel/upcall/includes/uc_hash.h"
#include "uc_vnops.h"

/* Definition of the vfs operation vector */
int (**ucfs_vnodeop_p)(void *);
extern struct vnode *ucfs_root_vp;

/* Definition of the vnode operation vector */

struct vnodeopv_entry_desc ucfs_vnodeop_entries[] = {
    { &vop_default_desc, ucfs_eopnotsupp},
    { &vop_lookup_desc, ucfs_eopnotsupp },     /* lookup */
    { &vop_create_desc, ucfs_eopnotsupp },	/* create */
    { &vop_mknod_desc, ucfs_eopnotsupp },	/* mknod */
    { &vop_open_desc, ucfs_open },		/* open */
    { &vop_close_desc, ucfs_close },		/* close */
    { &vop_access_desc, ucfs_access },		/* access */
    { &vop_getattr_desc, ucfs_eopnotsupp },	/* getattr */
    { &vop_setattr_desc, ucfs_eopnotsupp },	/* setattr */
    { &vop_read_desc, ucfs_eopnotsupp },	/* read */
    { &vop_write_desc, ucfs_eopnotsupp },	/* write */
    { &vop_ioctl_desc, ucfs_ioctl },		/* ioctl */
#ifdef NetBSD1_3
    { &vop_poll_desc, ucfs_eopnotsupp },	/* poll */
#else
    { &vop_select_desc, ucfs_eopnotsupp },	/* select */
#endif
    { &vop_mmap_desc, ucfs_eopnotsupp },	/* mmap */
    { &vop_fsync_desc, ucfs_eopnotsupp },	/* fsync */
#ifdef NetBSD1_3
    { &vop_seek_desc, ucfs_eopnotsupp },	/* seek */
#else
    { &vop_seek_desc, ucfs_eopnotsupp },	/* seek */
#endif
    { &vop_remove_desc, ucfs_eopnotsupp },		/* remove */
    { &vop_link_desc, ucfs_eopnotsupp },		/* link */
    { &vop_rename_desc, ucfs_eopnotsupp},		/* rename */
    { &vop_mkdir_desc, ucfs_eopnotsupp },		/* mkdir */
    { &vop_rmdir_desc, ucfs_eopnotsupp },		/* rmdir */
    { &vop_symlink_desc, ucfs_eopnotsupp },		/* symlink */
    { &vop_readdir_desc, ucfs_eopnotsupp },		/* readdir */
    { &vop_readlink_desc, ucfs_eopnotsupp },	/* readlink */
    { &vop_abortop_desc, ucfs_eopnotsupp },	/* abortop */
    { &vop_inactive_desc, ucfs_eopnotsupp },	/* inactive */
    { &vop_reclaim_desc, ucfs_reclaim },	/* reclaim */
    { &vop_lock_desc, ucfs_lock },	/* lock */
    { &vop_unlock_desc, ucfs_unlock },	/* unlock */
    { &vop_bmap_desc, ucfs_eopnotsupp },	/* bmap */
    { &vop_strategy_desc, ucfs_eopnotsupp },	/* strategy */
    { &vop_print_desc, ucfs_eopnotsupp },	/* print */
    { &vop_islocked_desc, ucfs_eopnotsupp },	/* islocked */
    { &vop_pathconf_desc, ucfs_eopnotsupp },	/* pathconf */
    { &vop_advlock_desc, ucfs_eopnotsupp },	/* advlock */
    { &vop_blkatoff_desc, ucfs_eopnotsupp },	/* blkatoff */
    { &vop_valloc_desc, ucfs_eopnotsupp },	/* valloc */
    { &vop_vfree_desc, ucfs_eopnotsupp },	/* vfree */
    { &vop_truncate_desc, ucfs_eopnotsupp },	/* truncate */
    { &vop_update_desc, ucfs_eopnotsupp },	/* update */
    { &vop_bwrite_desc, ucfs_eopnotsupp },	/* bwrite */
#ifdef __NetBSD__
    { &vop_lease_desc, ucfs_eopnotsupp },	/* lease */
#endif
#ifdef	__FreeBSD__
    { &vop_getpages_desc, ucfs_eopnotsupp }, /* pager intf.*/
    { &vop_putpages_desc, ucfs_eopnotsupp }, /* pager intf.*/
#endif
    { (struct vnodeop_desc*)NULL, (int(*)(void *))NULL }
};

#ifdef __NetBSD__
struct vnodeopv_desc ucfs_vnodeop_opv_desc = 
        { &ucfs_vnodeop_p, ucfs_vnodeop_entries };
#endif

#ifdef __FreeBSD__
static struct vnodeopv_desc ucfs_vnodeop_opv_desc = 
		{ &ucfs_vnodeop_p, ucfs_vnodeop_entries };

#include <sys/kernel.h>
VNODEOP_SET(ucfs_vnodeop_opv_desc);

#endif

/* Definitions of NetBSD vnodeop interfaces */

int
ucfs_eopnotsupp(v)
     void *v;
{
    return (EOPNOTSUPP);
}

int
ucfs_open(v)
     void *v;
{
	struct vop_open_args *ap = v;
	struct proc *p = ap->a_p;
	struct uc_info_t *ucip;

	if (uc_hashlookup(p->p_pid))
		return (EBUSY); /* trying to open it twice! */
	
    /* Create a uc_info_t for this process */
	ucip = malloc(sizeof(struct uc_info_t), M_TEMP, M_NOWAIT);
#ifdef DIAGNOSTIC
	if (!ucip) panic("uc_open: malloc failed\n");
#endif
	
	LIST_INIT(&ucip->uci_upcalls);
	TAILQ_INIT(&ucip->uci_pending);
	ucip->uc_pid = p->p_pid;
	ucip->uci_sig = NSIG + 0xff; /* Forces user to set this prior to use */
	uc_hashinsert(ucip);
	return (0);
}

int
ucfs_close(void *v)
{
	struct vop_close_args *ap = v;
	struct proc *p = ap->a_p;
	if (uc_hashlookup(p->p_pid))
		uc_exit(p);
	return 0;
}

int
ucfs_ioctl(void *v)
{
	struct vop_ioctl_args *ap = v;
	u_long com = ap->a_command;
	caddr_t data = ap->a_data;
	struct proc *p = ap->a_p;
	int len = IOCPARM_LEN(com);

	/* If we don't know about this process, then it probably got the
	   fd through fork() inheritance */
	/* XXX we need to provide fork() inheritance of upcalls some day */

	if (!uc_hashlookup(p->p_pid))
		return EINVAL;

	switch (com) {
	case UCI_SYNC:
		if (len != sizeof(struct uci_sync_args)) return EINVAL;
		return uci_sync(p, (struct uci_sync_args *) data, NULL);
	case UCI_ASYNC:
		if (len != sizeof(struct uci_async_args)) return EINVAL;
		return uci_async(p, (struct uci_async_args *) data, NULL);
	case UCI_REG:
		if (len != sizeof(struct uci_reg_args)) return EINVAL;
		return uci_reg(p, (struct uci_reg_args *) data, NULL);
	case UCI_NEXT:
		if (len != sizeof(struct uci_next_args)) return EINVAL;
		return uci_next(p, (struct uci_next_args *) data, NULL);
	case UCI_ARGS:
		if (len != sizeof(struct uci_args_args)) return EINVAL;
		return uci_args(p, (struct uci_args_args *) data, NULL);
	case UCI_RET:
		if (len != sizeof(struct uci_ret_args)) return EINVAL;
		return uci_ret(p, (struct uci_ret_args *) data, NULL);
	case UCI_SET_SIG:
		if (len != sizeof(struct uci_set_sig_args)) return EINVAL;
		return uci_set_sig(p, (struct uci_set_sig_args *) data, NULL);
	default:
		return EINVAL;
	}
}

int
ucfs_access(void *v)
{
	return (0);
}

int 
ucfs_reclaim (void *v)
{
    /* No file-system specific data */
    return (0);
}

int
ucfs_lock(void *v)
{
	return (0);
}

int
ucfs_unlock(void *v)
{
	return (0);
}

