#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * sys/upcall.h -- definitions for the upcall facility
 */

#ifndef _SYS_UPCALL_H_
#define _SYS_UPCALL_H_

/* Fix discrepency between FreeBSD and NetBSD KERNEL defines */
#if defined(__FreeBSD__) && defined(KERNEL)
#define _KERNEL
#endif

/* Fix discrepency between FreeBSD and Linux KERNEL defines */
#if defined(__linux__) && defined(__KERNEL__)
#define _KERNEL
#endif

/* this is a hack; but I think using VT_NON is Bad. */
#define VT_UPCALL  242

/* 
 * Some types used by the upcall system.  Included everywhere for
 * now, since we don't want user-level code to have to define something
 * to use them.
 */
typedef int32_t         upclass_t;        /* subsystem offering upcalls */
typedef int32_t         upcall_t;         /* call w/in a class */
typedef u_int32_t       serial_t;         /* serial # used by upcall system */


/*
 * Warning: the kernel depends on these being sequentially numbered,
 * starting at zero.  This is probably a bug.
 */

#define UC_UNUSED       0
#define UC_TEST         1  /* Test upclass */
#define UC_ODYSSEY      2  /* Odyssey resoruce registration upclass */
#define UC_NTRP         3  /* Network trace replay upclass */
#define UC_RESLAN       4  /* Reservable local area network */

#define UPCALL_ASYNC 0x1

#define UPCALL_REGISTER   1  /* Else deregister */

#ifndef _KERNEL

/* Declarations of general interest */

/* typedef for upcall handlers */
typedef int (*uc_hdlr_t) __P((upclass_t, upcall_t, size_t, void *, 
			      size_t *, void *, int));

#define UC_ERR      ((void *)-1)

extern void *
upcall_reg     __P((upclass_t, upcall_t, uc_hdlr_t));

extern int
upcall_sync    __P((upclass_t, upcall_t, pid_t, size_t, void *, 
		    size_t *, void *));

extern int
upcall_async   __P((upclass_t, upcall_t, pid_t, size_t, void *));

extern int
upcall_block   __P((void));

extern int
upcall_unblock __P((void));

#ifdef _UPCALL_REG_IMPL

/* 
 * Declarations of interest only to the implementation of upcall_reg.
 *
 * User-level code, other than the libc routines that provide the
 * implementation for upcall_reg, should *NOT* make use of these
 * interfaces.  They should rather be thought of as kernel-internal
 * functions that are available to the "system routine" upcall_reg.
 */

/*
 * upcall_reg(3) keeps track of (upclass,upcall)->hanlder mappings.
 * But, we promise to inform subsystems about any processes that
 * register or deregister.  This to allow subsystems to keep track of
 * the processes that they know about as well as to cons up process
 * lists for broadcast upcalls cheaply (i.e.: without checking every
 * current process.)  The system call __upcall_reg does this.  It
 * takes three arguments: upclass, upcall, and whether this is a
 * registration or a deregistration.  Returns one of the following:
 * ENOMEM (couldn't allocate a registration entry -- probably means
 * the kernel is due to panic soon), ENXIO -- for some reason, the
 * subsystem can't initialize itself, EINVAL -- the class or call
 * number was bad.  
 */

extern int
__upcall_reg   __P((upclass_t, upcall_t, int));

/*
 * __upcall_next 
 *
 * This is the first call made by libc after being notified of a
 * pending upcall.  Arguments supplied by libc are all pointers that
 * are OUT values.  The only IN/OUT value is the size of the argument
 * buffer.  Returns the upclass, upcall nubmers, size of the argument
 * and return buffers supplied by the kernel, and copies in the
 * argument buffer if the libc-supplied one is big enough.  If it
 * isn't, libc will have to use __upcall_args.  Can return the usual
 * EFAULT if any of the pointers are bogus, or EALREADY if there is an
 * oustanding, delivered upcall that hasn't been __upcall_ret'd yet.
 */

extern int
__upcall_next  __P((upclass_t *, upcall_t *, size_t *, void *,
		    size_t *, serial_t *));

/*
 * Used by libc if the arguments are too big to slurp in during
 * __upcall_next.  Errors are EFAULT for a bad pointer, or ENOENT if
 * the upcall in question (named by serial) isn't currently in
 * progress.
 */

extern int
__upcall_args  __P((serial_t, size_t, void *));

/*
 * Used to send return values.  Same errors as __upcall_args, plus the
 * possibility of E2BIG if the return buffer is too big for the one
 * in-kernel due to recieve the results.
 */

extern int
__upcall_ret   __P((serial_t, size_t, void *, int));

/* Used to set the signal used by upcalls */
extern int
upcall_set_sig __P((unsigned));

#endif /* _UPCALL_REG_IMPL */

#endif /* !_KERNEL */

#endif /* _SYS_UPCALL_H_ */
