#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Directory operations for Coda filesystem
 * Original version: (C) 1996 P. Braam and M. Callahan
 * Rewritten for Linux 2.1. (C) 1997 Carnegie Mellon University
 * 
 * Carnegie Mellon encourages users to contribute improvements to
 * the Coda project. Contact Peter Braam (coda@cs.cmu.edu).
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>

#include "ody_queue.h"
#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_upclass.h"
#include "uc_ioctl.h"
#include "uc_linux.h"

/* dir inode-ops */
static int upcall_permission(struct inode *inode, int mask);
static int upcall_open(struct inode *i, struct file *f);
static int upcall_release(struct inode *i, struct file *f);
static int upcall_ioctl(struct inode * inode, struct file * filp, 
			unsigned int cmd, unsigned long user_data);

struct file_operations upcall_dir_operations = {
        NULL,                   /* lseek */
        NULL,                   /* read -- bad  */
        NULL,                   /* write */
        NULL,                   /* readdir */
        NULL,                   /* poll (was select) */
        upcall_ioctl,           /* ioctl */
        NULL,                   /* mmap */
        upcall_open,            /* open */
	NULL,			/* flush */
        upcall_release,         /* release */
	NULL,                   /* fsync */
        NULL,                   /* fasync */
	NULL,			/* check_media_change */
	NULL,			/* revalidate */
	NULL			/* lock */
};

struct inode_operations upcall_dir_inode_operations =
{
	&upcall_dir_operations,
	NULL,	                /* create */
	NULL,    	        /* lookup */
	NULL,	                /* link */
	NULL,                   /* unlink */
	NULL,	                /* symlink */
	NULL,	                /* mkdir */
	NULL,   	        /* rmdir */
	NULL,		        /* mknod */
	NULL,	                /* rename */
	NULL,	                /* readlink */
	NULL,	                /* follow_link */
	NULL,	                /* readpage */
	NULL,		        /* writepage */
	NULL,		        /* bmap */
	NULL,	                /* truncate */
	upcall_permission,      /* permission */
	NULL,                   /* smap */
	NULL,                   /* update page */
        NULL                    /* revalidate */
};

int upcall_permission(struct inode *inode, int mask)
{
	ENTRY;
	EXIT;
	return 0;
}

int upcall_open(struct inode *i, struct file *f)
{
	int retval;

        ENTRY;
	retval = uc_open();
	EXIT;
	
        return retval;
}

int upcall_release(struct inode *i, struct file *f)
{
	int retval;

	ENTRY;
	retval = uc_close();
	EXIT;

	return retval;
}

static int upcall_ioctl(struct inode * inode, struct file * filp, 
			unsigned int cmd, unsigned long user_data)
{
	int retval;
	int len = _IOC_SIZE(cmd);

	ENTRY;
	retval = uc_ioctl (cmd, (void *) user_data, len);
	EXIT;
	
	return retval;
}
