#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Super block/filesystem wide operations
 *
 * Copryright (C) 1996 Peter J. Braam <braam@maths.ox.ac.uk> and 
 * Michael Callahan <callahan@maths.ox.ac.uk> 
 * 
 * Rewritten for Linux 2.1.  Peter Braam <braam@cs.cmu.edu>
 * Copyright (C) Carnegie Mellon University
 */

/*
 * stolen for Odyssey
 */

#include <linux/config.h> /* for CONFIG_PROC_FS */
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/locks.h>
#include <linux/lp.h>
#include <linux/malloc.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/skbuff.h>
#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/poll.h>
#include <asm/uaccess.h>

#include "ody_queue.h"
#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_upclass.h"
#include "uc_linux.h"

#define UPCALL_SUPER_MAGIC 0xd456d171

/* VFS super_block ops */
static struct super_block *upcall_read_super(struct super_block *, 
					     void *, int);
static void upcall_read_inode(struct inode *);
static void ody_put_inode(struct inode *);
static void ody_put_super(struct super_block *);
static int upcall_statfs(struct super_block *sb, struct statfs *buf, 
		      int bufsiz);

extern void uc_init_upclass (int uc_id, struct uc_ucld_t* ucldp);

extern struct inode_operations upcall_dir_inode_operations;

/* exported operations */
struct super_operations upcall_super_operations =
{
        upcall_read_inode,      /* read_inode */
	NULL,                   /* write_inode */
	ody_put_inode,	        /* put_inode */
	NULL,                   /* delete_inode */
	NULL,           	/* notify_change */
	ody_put_super,	        /* put_super */
	NULL,			/* write_super */
	upcall_statfs,          /* statfs */
	NULL			/* remount_fs */
};

static struct super_block * upcall_read_super(struct super_block *sb, 
					      void *data, int silent)
{
        struct inode* i_root;

	ENTRY;
        MOD_INC_USE_COUNT; 

        lock_super(sb);
        sb->u.generic_sbp = NULL; 
        sb->s_blocksize = 1024;	/* XXXXX  what do we put here?? */
        sb->s_blocksize_bits = 10;
        sb->s_magic = UPCALL_SUPER_MAGIC;
        sb->s_op = &upcall_super_operations;

	i_root = iget (sb, 0);
	i_root->i_op = &upcall_dir_inode_operations;
	sb->s_root = d_alloc_root(i_root, NULL);
	unlock_super(sb);

	EXIT;  
        return sb;
}

static void ody_put_super(struct super_block *sb)
{
        ENTRY;

	printk("Upcall: Bye bye.\n");

        MOD_DEC_USE_COUNT;
	EXIT;
}

/* all filling in of inodes postponed until lookup */
static void upcall_read_inode(struct inode *inode)
{
	ENTRY;
	EXIT;
}

static void ody_put_inode(struct inode *in) 
{
	ENTRY;

        ODEBUG(D_SYSUPCALL,"ino: %ld, count %d\n", in->i_ino, in->i_count);

	if ( in->i_count == 1 ) 
		in->i_nlink = 0;
		
	EXIT;
}

/*  we need _something_ for this routine. Let's mimic AFS */
static int upcall_statfs(struct super_block *sb, struct statfs *buf, 
			 int bufsiz)
{
	struct statfs tmp;

	tmp.f_type = UPCALL_SUPER_MAGIC;
	tmp.f_bsize = 1024;
	tmp.f_blocks = 9000000;
	tmp.f_bfree = 9000000;
	tmp.f_bavail = 9000000;
	tmp.f_files = 9000000;
	tmp.f_ffree = 9000000;
	tmp.f_namelen = 0;
	copy_to_user(buf, &tmp, bufsiz);
	return 0; 
}

struct file_system_type upcall_fs_type = {
	"ucfs", 0, upcall_read_super, NULL
};

int init_upcall_fs(void)
{
	return register_filesystem(&upcall_fs_type);
}

#ifdef MODULE

MODULE_AUTHOR("Jason Flinn <jflinn@cs.cmu.edu>\n");

EXPORT_SYMBOL(uc_init_upclass);
EXPORT_SYMBOL(uc_init_default);
EXPORT_SYMBOL(uc_notify_default);
EXPORT_SYMBOL(uc_getpep_default);
EXPORT_SYMBOL(uc_rcpt_default);
EXPORT_SYMBOL(uc_print_default);

int init_module(void)
{
  int status;
  printk(KERN_INFO "Odyssey upcall module 2.0\n");

  status = init_upcall_fs();
  if (status) {
          printk("upcall: failed in init_upcall_fs\n");
  }

  uc_sysctl_init ();
  uc_init ();

  return status;
}


void cleanup_module(void)
{
	int err;

        ENTRY;

        if ( (err = unregister_filesystem(&upcall_fs_type)) != 0 ) {
                printk("upcall: failed to unregister filesystem\n");
        }

	uc_sysctl_clean();

	printk(KERN_INFO "Odyssey upcall module 2.0 exitting.\n");
}

void
change_use_count (int isreg)
{
        if (isreg) {
                MOD_INC_USE_COUNT;
        } else {
                MOD_DEC_USE_COUNT;
        } 
}

#endif
