#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <linux/param.h>
#include <linux/time.h>
#include <linux/malloc.h>
#include <linux/sem.h>

#include "ody_queue.h"
#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_upclass.h"
#include "uc_ioctl.h"
#include "uc_pending.h"
#include "kernel/upcall/includes/uc_hash.h"
#include "uc_linux.h"

int
uc_open()
{
	struct uc_info_t *ucip;

        ODEBUG(D_SYSUPCALL, "current->pid is %u\n", 
	       current->pid);

	if (uc_hashlookup(current->pid))
		return (EBUSY); /* trying to open it twice! */
	
        ODEBUG(D_SYSUPCALL, "hash lookup succeeded"); 

    /* Create a uc_info_t for this process */
	ucip = kmalloc(sizeof(struct uc_info_t), GFP_KERNEL);
	ODEBUG (D_SYSUPCALL, "ucip: %lx\n", ((unsigned long) ucip));
	
	sema_init(&ucip->uci_sem, 1);
	ODEBUG (D_SYSUPCALL, "Semaphore inited\n");
	ODY_LIST_INIT(&ucip->uci_upcalls);
	TAILQ_INIT(&ucip->uci_pending);
	ODEBUG (D_SYSUPCALL, "Lists inited\n");
	ucip->uc_pid = current->pid;
	uc_hashinsert(ucip);
	ODEBUG (D_SYSUPCALL, "About to return\n");
	return (0);
}

int
uc_close()
{
	if (uc_hashlookup(current->pid))
		uc_exit();
	return 0;
}

int uc_ioctl(unsigned long cmd, void *data, int len)
{       
	ODEBUG (D_SYSUPCALL, "ioctl: cmd = %ld, len = %d\n", cmd, len);
	switch (cmd) {
	case UCI_SYNC:
		if (len != sizeof(struct uci_sync_args)) return -EINVAL;
		return uci_sync ((struct uci_sync_args *) data);
	case UCI_ASYNC:
		if (len != sizeof(struct uci_async_args)) return -EINVAL;
		return uci_async((struct uci_async_args *) data);
	case UCI_REG:
		if (len != sizeof(struct uci_reg_args)) return -EINVAL;
		return uci_reg((struct uci_reg_args *) data);
	case UCI_NEXT:
		if (len != sizeof(struct uci_next_args)) return -EINVAL;
		return uci_next((struct uci_next_args *) data);
	case UCI_ARGS:
		if (len != sizeof(struct uci_args_args)) return -EINVAL;
		return uci_args((struct uci_args_args *) data);
	case UCI_RET:
		if (len != sizeof(struct uci_ret_args)) return -EINVAL;
		return uci_ret((struct uci_ret_args *) data);
#ifdef UC_SIGNAL
	case UCI_SET_SIG:
		ODEBUG (D_SYSUPCALL, "Set signal upcall\n");
		if (len != sizeof(struct uci_set_sig_args)) return -EINVAL;
		ODEBUG (D_SYSUPCALL, "About to call uci_set_sig\n");
		return uci_set_sig((struct uci_set_sig_args *) data);
#endif
	default:
		return -EINVAL;
	}
}
