#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


/* ioctl command values for upcall subsystem */

#ifndef _UC_IOCTL_H
#define _UC_IOCTL_H

/* Args struct for each of these ioctls */

struct uci_sync_args {
	upclass_t upclass;
	upcall_t upcall;
	pid_t pid;
	size_t argsz;
	void * argp;
	size_t * retszp;
	void * retp;
};

struct uci_async_args {
	upclass_t upclass;
	upcall_t upcall;
	pid_t pid;
	size_t argsz;
	void * argp;
};

struct uci_reg_args {
	upclass_t upclass;
	upcall_t upcall;
	int flags;
};

struct uci_next_args {
	upclass_t * upclassp;
	upcall_t * upcallp;
	size_t * argszp;
	void * argp;
	size_t * retszp;
	serial_t * serialp;
};

struct uci_args_args  {
	serial_t serial;
	size_t bufsz;
	void * buf;
};

struct uci_ret_args  {
	serial_t serial;
	size_t retsz;
	void * retp;
	int errno;
};

#ifdef UC_SIGNAL
struct uci_set_sig_args {
	unsigned sig;
};
#endif

#define UCI_SYNC       _IOW('u', 0, struct uci_sync_args)
#define UCI_ASYNC      _IOW('u', 1, struct uci_async_args)
#define UCI_REG        _IOW('u', 2, struct uci_reg_args)
#define UCI_NEXT       _IOW('u', 3, struct uci_next_args)
#define UCI_ARGS       _IOW('u', 4, struct uci_args_args)
#define UCI_RET        _IOW('u', 5, struct uci_ret_args)
#ifdef UC_SIGNAL
#define UCI_SET_SIG    _IOW('u', 6, struct uci_set_sig_args)
#endif

#ifdef __KERNEL__
int	uci_sync  __P((struct uci_sync_args *));
int	uci_async __P((struct uci_async_args *));
int	uci_reg   __P((struct uci_reg_args *));
int	uci_next  __P((struct uci_next_args *));
int	uci_args  __P((struct uci_args_args *));
int	uci_ret   __P((struct uci_ret_args *));
#ifdef UC_SIGNAL
int     uci_set_sig __P((struct uci_set_sig_args *));
#endif

int uc_open __P ((void));
int uc_close __P ((void));
int uc_ioctl __P ((unsigned long cmd, void *data, int len));

#endif

#endif /* _UC_IOCTL_H */
