#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * uc_upclass.c
 */

#include <linux/param.h>
#include <linux/time.h>
#include <linux/malloc.h>

#include "ody_queue.h"
#include "kernel/upcall/includes/upcall.h"
#include "upcall.private.h"
#include "uc_upclass.h"
#include "uc_pending.h"
#include "kernel/upcall/includes/uc_hash.h"

/* 
 * Header files that declare other upclass' tables.
 */

#ifdef ODYSSEY
#define _KERNEL
#include "kernel/vfs/includes/ody_upcall.h"
#endif

#ifdef NTRP
#include "ntrp.h"
#endif

#if NNTRP > 0
#include <nettr/replay/ntrp_upcall.h>
#endif

#ifdef RESLAN_UPCALL
#include <netinet/reslan_upcall.h>
#endif

/* 
 * Definitions of upclasses
 */

struct uc_ucld_t unused_ucld = { 
	UC_UNUSED, 
	0, 
	"Unused", 
	NULL, NULL, NULL, NULL, NULL, NULL,
	NULL 
};

struct uc_ucld_t test_ucld = {
	UC_TEST,
	0,
	"Test",
	uc_init_default,
	uc_notify_default,
	NULL,
	uc_getpep_default,
	uc_rcpt_default,
	uc_print_default,
	NULL
};

struct uc_ucld_t *upcl_tab[] = 
{
	&unused_ucld,
	&test_ucld,
	NULL,           /* Reserved for Odyssey upclass */
#if NNTRP > 0
	&ntrp_ucld,
#else
	NULL,
#endif
#ifdef RESLAN_UPCALL
	&rl_ucld,
#else
	NULL,
#endif
};

size_t upcl_tabsz = sizeof(upcl_tab) / sizeof(struct uc_ucld_t *);

void
uc_print()
{
	int i;
	struct uc_ucld_t      *ucldp;
	

	printk("Printing process registrations\n");
	
#if 0
	/* Note to self: for_each_task introduces wierd linkage dependencies.
	   This can be implemented by traversing the hash-table instead */
	for_each_task (t)
	{
		if (uc_hashlookup(t->pid)->uci_upcalls.lh_first) 
			printk("\tProcess %d\n", t->pid);
		for (pep = uc_hashlookup(t->pid)->uci_upcalls.lh_first; 
		     pep; 
		     pep = pep->ucpe_list.le_next)
		{
			printk("\t\tupclass %d, upcall %d\n",
			       pep->ucpe_upclass, pep->ucpe_upcall);
		}
	}
#endif

	printk("---------------------------------------\n");
	
	for (i=1; i<upcl_tabsz; i++) {
		ucldp = upcl_tab[i];
		if (ucldp->ucld_inited) {
			(ucldp->ucld_print)(ucldp, ucldp->ucld_table);
		}
	}
}

