#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef __UPCALL_PRIVATE_H__
#define __UPCALL_PRIVATE_H__

#ifdef __KERNEL__

/* Kernel-specific declarations */

/*
 * A list of upcalls that a process has registered for.  This is to be
 * a header for all class-specific, per-process upcall registrations
 * that we keep track of.  They are used to handle fork()
 * registrations and exec() deregistrations.  See
 * <upcall/uc_upclass.h> for a discussion of how upclasses use
 * uc_procent_t's, and <sys/proc.h> for what role they play in struct
 * proc's
 * 
 * XXX move to uc_depend/uc_upclass?
 */
struct uc_procent_t {
	unsigned int              ucpe_magic;  /* Tag to see if already in p */
       	ODY_LIST_ENTRY(uc_procent_t)  ucpe_list;   /* Chain for each p */
	upclass_t                 ucpe_upclass;/* Which upclass */
	upcall_t                  ucpe_upcall; /* Which upcall */
	void                     *ucpe_data;   /* upclass-specific data */
};

struct uc_ucld_t;
extern void uc_init_upclass __P ((int, struct uc_ucld_t*));

extern void uc_proc0init __P((void));

extern void uc_init __P((void));   /* Initialize the upcall structures */

/* 
 * On sigreturn, we may still have pending upclls.  If so, deal with
 * them.  (Post SIGUPCALL again).
 */
extern void uc_sigreturn __P((void));

/*
 * After a subclass does whatever it would like to do with a notify
 * request, it is obligated to return a pointer to a uc_procent_t,
 * which represents the registered or deregistered upcall.
 * uc_procnotify ensures that newly-registered-for upcalls are added
 * to the process' list of outstanding registrations,
 * re-registered-for upcalls are ingored (since they are already in
 * the process' list) and unregistered-for upcalls are removed.
 * Having this work depends on folks not mucking with ucpe_magic.  
 */
extern void uc_procnotify __P((struct uc_procent_t *ucpep,
			       int isreg));

/*
 * At fork time, we need to inherit all registrations from the parent
 * process to the child.  We also need to initialize the child's
 * pending lists.  Posted, but unresponded-to upcalls are *not*
 * inherited by children.
 */
extern void uc_fork __P((struct task_struct *t1, struct task_struct *t2));

/* 
 * At exec time, we need to remove all registrations from the process
 * to be overwritten.  All pending (sync) upcalls need responses.
 * XXX flushing pending isn't known yet.
 */
extern void uc_exec __P((void));

/* 
 * On exit, we need to remove all registrations from the dying
 * process.  All pending (sync) upcalls also need responses 
 * XXX flushing pending isn't known yet.
 */
extern void uc_exit __P((void));

/*
 * Some routines for internal versions of system calls 
 */

/*
 * uc_doreg: counterpart to sys___upcall_reg.  Also called by uc_fork
 * and uc_exec
 */
extern int uc_doreg __P((upclass_t upclass, upcall_t upcall, int isreg));

/*
 * uc_print.  Debugging dump
 */
extern void uc_print __P((void));

/*
 * uc_post_sync: post a synchronous upcall to process p.  Waits for a
 * result via tsleep().  Returns zero if upcall successfully posted
 * and answered, or something useful from errno.h if it fails.  Passes
 * arguments via argsz and argp.  If argsz is nonzero, argp is
 * expected to point to a rage of valid memory at least argsz long.
 * If argmtype is nonzero, argp will be freed after it is no longer
 * needed.  Return values are returned through retp; it must be at
 * least as big as *retszp (if *retszp is nonzerp); *retszp, on return
 * will be the size of the valid range pointed to by retp.  Caller is
 * responsible for allocating argp, retp, and is responsible for
 * deallocating retp always, and argp whenever argmtype is zero.  
 */
extern int uc_post_sync __P((struct task_struct* t, upclass_t upclass, 
			     upcall_t upcall, size_t argsz, void *argp, 
			     int argmtype, size_t *retszp, void *retp));

/* A list of processes */
ODY_LIST_HEAD(uc_plist_t, uc_plent_t);

/* An entry in said list */
struct uc_plent_t {
	ODY_LIST_ENTRY(uc_plent_t)  ucpl_list;
	struct task_struct         *ucpl_t;
};

/*
 * uc_post_async: post asynchronous upcalls to a set of processes plp,
 * that is plsz entries long.  No waiting.  Returns zero if the upcall
 * is successfully posted (which is damn near always).  If argsz is
 * nonzero, argp is expected to point to a rage of valid memory at
 * least argsz long.  If argmtype is nonzero, argp will be freed after
 * it is no longer needed.  uc_post_async will free the list of
 * processes, and if asked to will free the argp sometime later (after
 * it has been copied to all processes of interest.
 */
extern int uc_post_async __P((int ucplsz, struct uc_plist_t *ucplp, 
			      upclass_t upclass, upcall_t upcall, 
			      size_t argsz, void *argp, int argmtype));

#ifdef MODULE
extern void change_use_count __P((int isreg));
#endif

#endif /* __KERNEL__ */

#endif /* __UPCALL_PRIVATE_H__ */
