#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#define UIODEBUG
/* 
 * These routines are the device entry points for Odyssey.  They have
 * been stolen shamelessly from the Coda MiniCache
 */

#include <sys/param.h>
#include <sys/malloc.h>
#include <sys/proc.h>
#include <sys/mount.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#ifdef NetBSD1_3
#include <sys/poll.h>
#endif
#include <sys/select.h>

#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#include "kernel/vfs/includes/odync.h"
#include "kernel/vfs/includes/odyio.h"

#include "kernel/vfs/includes/odydev.h"

int ody_psdev_print_entry = 0;

#ifdef __GNUC__
#define ENTRY    \
    if(ody_psdev_print_entry) myprintf(("Entered %s\n",__FUNCTION__))
#else
#define ENTRY
#endif 

void odydevattach __P((int n));
int odydev_open   __P((dev_t dev, int flag, int mode, struct proc *p));
int odydev_close  __P((dev_t dev, int flag, int mode, struct proc *p));
int odydev_read   __P((dev_t dev, struct uio *uiop, int flag));
int odydev_write  __P((dev_t dev, struct uio *uiop, int flag));
int odydev_ioctl  __P((dev_t dev, int cmd, caddr_t addr,
		       int flag, struct proc *p));
#ifdef NetBSD1_3
int odydev_poll   __P((dev_t dev, int events, struct proc *p));
#else
int odydev_select __P((dev_t dev, int flag, struct proc *p));
#endif

#if defined(UIODEBUG) && defined(DEBUG)
extern void trap_dump __P((void));
#endif

struct omsg {
    struct queue om_chain;
    caddr_t	 om_data;
    u_short	 om_flags;
    u_short	 om_opcode; 	/* copied from data to save ptr lookup */
    size_t       om_inSize;	/* Size needed for request */
    size_t	 om_outSize;    /* Size needoed for response */
    int		 om_unique;
    char         om_sleep;	/* XXX - what is this for, Linux? */
};

#define	OM_READ	    1
#define	OM_WRITE    2
#define	OM_INTR	    4

/* odydevattach: do nothing */
void
odydevattach(int n)
{
}

/* Tells the lkm bits whether it is safe to take this device away */
/* If any of the minors is open, then it is not safe */

int
is_odydev_open(void)
{
    int i;

    for (i = 0; i < NODYDEV; i++)
	if (OC_OPEN(&ody_mnttbl[i].mi_ocomm))
	    return 1;
    return 0;
}

int 
odydev_open(dev, flag, mode, p)    
    dev_t        dev;      
    int          flag;     
    int          mode;     
    struct proc *p;
{
    register struct ocomm *ocp;
    
    ENTRY;

    if (minor(dev) >= NODYDEV || minor(dev) < 0)
	return(ENXIO);
    
    if (!odync_initialized)
	odync_init();
    
    ocp = &ody_mnttbl[minor(dev)].mi_ocomm;
    if (OC_OPEN(ocp))
	return(EBUSY);

    bzero(&ocp->oc_selproc, sizeof(struct selinfo));

    INIT_QUEUE(ocp->oc_requests);
    INIT_QUEUE(ocp->oc_replys);
    MARK_OC_OPEN(ocp);
    
    ody_mnttbl[minor(dev)].mi_vfsp = NULL;
    ody_mnttbl[minor(dev)].mi_rootvp = NULL;

    return(0);
}


int 
odydev_close (dev, flag, mode, p)    
    dev_t        dev;      
    int          flag;     
    int          mode;     
    struct proc  *p;
{
    register struct ocomm *ocp;
    register struct omsg  *omp;
    struct   ody_mntinfo  *mi;
    int                   err;
	
    ENTRY;

    if (minor(dev) >= NODYDEV || minor(dev) < 0)
	return(ENXIO);
    
    mi = &ody_mnttbl[minor(dev)];
    ocp = &(mi->mi_ocomm);
    
    if (!OC_OPEN(ocp))
	panic("odydev_close: not open");
    
    /* 
     * prevent future operations on this vfs from succeeding by auto-
     * unmounting any vfs mounted via this device. This frees user or
     * sysadm from having to remember where all mount points are located.
     * Put this before WAKEUPs to avoid queuing new messages between
     * the WAKEUP and the unmount (which can happen if we're unlucky)
     */
    if (mi->mi_rootvp) {
	VTOO(mi->mi_rootvp)->o_flags |= ON_UNMOUNTING;
	ody_unmounting(mi->mi_vfsp);
	err = dounmount(mi->mi_vfsp, flag, p);
	if (err)
	    myprintf(("Error %d unmounting vfs in odydev_close (%d)\n", 
		      err, minor(dev)));
    }
    
    /* Wakeup clients so they can return. */
    for (omp = (struct omsg *)GETNEXT(ocp->oc_requests);
	 !EOQ(omp, ocp->oc_requests);
	 omp = (struct omsg *)GETNEXT(omp->om_chain))
    {	    
	/* Free signal request messages and don't wakeup cause
	   no one is waiting. */
	if (omp->om_opcode == ODY_SIGNAL) {
	    ODY_FREE((caddr_t)omp->om_data, (u_int)OC_IN_NO_DATA);
	    ODY_FREE((caddr_t)omp, (u_int)sizeof(struct omsg));
	    continue;
	}
	
	wakeup(&omp->om_sleep);
    }
    
    for (omp = (struct omsg *)GETNEXT(ocp->oc_replys);
	 !EOQ(omp, ocp->oc_replys);
	 omp = (struct omsg *)GETNEXT(omp->om_chain))
    {
	wakeup(&omp->om_sleep);
    }
    
    MARK_OC_CLOSED(ocp);
    return(0);
}


int 
odydev_read(dev, uiop, flag)   
    dev_t        dev;  
    struct uio  *uiop; 
    int          flag;
{
    register struct ocomm *ocp;
    register struct omsg  *omp;
    int error = 0;
#ifdef UIODEBUG
    struct uio *uiosavep;
    int i;
#endif
    
    ENTRY;

    if (minor(dev) >= NODYDEV || minor(dev) < 0)
	return(ENXIO);
    
    ocp = &ody_mnttbl[minor(dev)].mi_ocomm;
    /* Get message at head of request queue. */
    if (EMPTY(ocp->oc_requests))
	    return(0);	/* Nothing to read */
    
    omp = (struct omsg *)GETNEXT(ocp->oc_requests);

    /* How much are we prepared to read? */
    if (uiop->uio_resid < omp->om_inSize) {
	    struct ody_toosmall_in fakemsg;

	    /* Make sure we have enough for the TOOSMALL message */
	    if (uiop->uio_resid < sizeof(struct ody_toosmall_in)) {
		    /* Too small for the TOOSMALL message */
		    return(E2BIG); /* A stretch, but ... */
	    }

	    /* 
	     * We are not prepared to read it all in: create an
	     * ODY_TOOSMALL message.  We only need opcode and
	     * the ody_toosmall arguments; others are ignored.
	     */

	    fakemsg.ih.opcode = ODY_TOOSMALL;
	    fakemsg.bufsiz = omp->om_inSize;
	    
	    error = uiomove((caddr_t)&fakemsg, sizeof (struct ody_toosmall_in),
			    uiop);
	    if (error) {
		    myprintf(("odydev_read: error (%d) on uiomove TOOSMALL\n",
			      error));
		    error = EINVAL;
	    }
	    return (error);
    }

#ifdef UIODEBUG
    /* Grab a copy of the uiop, in case. */
    ODY_ALLOC(uiosavep, struct uio *, sizeof(struct uio));
    /* Copy fields */
    *uiosavep = *uiop;
    /* Grab our own copy of the iovec array */
    ODY_ALLOC(uiosavep->uio_iov, struct iovec *,
	      sizeof(struct iovec)*uiosavep->uio_iovcnt);
    for (i=0; i<uiosavep->uio_iovcnt; i++) {
	    uiosavep->uio_iov[i] = uiop->uio_iov[i];
    }
#endif
    
    /* Move the input args into userspace */
    uiop->uio_rw = UIO_READ;
    error = uiomove(omp->om_data, omp->om_inSize, uiop);
    if (error) {
	    myprintf(("odydev_read: error (%d) on uiomove\n", error));
#ifdef UIODEBUG
	    printf("From %p for %d\n", omp->om_data, omp->om_inSize);
	    printf("to uio: cnt %d off %ld resid %d %s %s pid %d\n",
		   uiosavep->uio_iovcnt,
		   (long)uiosavep->uio_offset,
		   uiosavep->uio_resid,
		   (uiosavep->uio_segflg == UIO_USERSPACE) ? "USER" : "SYS", 
		   (uiosavep->uio_rw == UIO_READ) ? "READ" : "WRITE",
		   (uiosavep->uio_procp->p_pid) 
                           ? uiosavep->uio_procp->p_pid : 0);
	    for (i=0; i<uiosavep->uio_iovcnt; i++) {
		    printf("\tiov[%d]: %p (%d)\n", i,
			   uiosavep->uio_iov[i].iov_base,
			   uiosavep->uio_iov[i].iov_len);
	    }
#ifdef DEBUG
	    trap_dump();
#endif
	    panic("uioproblem encountered");
#endif
	    error = EINVAL;
    }
#ifdef UIODEBUG
    ODY_FREE(uiosavep->uio_iov, sizeof(struct iovec)*uiosavep->uio_iovcnt);
    ODY_FREE(uiosavep, sizeof(struct uio));
#endif

#ifdef DIAGNOSTIC    
    if (omp->om_chain.forw == 0 || omp->om_chain.back == 0)
	    panic("odydev_read: bad chain");
#endif

    REMQUE(omp->om_chain);
    
    /* 
     * If request was a signal, free up the message and don't enqueue
     * it in the reply queue. 
     */
    if (omp->om_opcode == ODY_SIGNAL) {
	    if (odydebug) {
		    myprintf(("odydev_read: signal msg (%d, %d)\n", 
			      omp->om_opcode, omp->om_unique));
	    }
	    ODY_FREE((caddr_t)omp->om_data, (u_int)OC_IN_NO_DATA);
	    ODY_FREE((caddr_t)omp, (u_int)sizeof(struct omsg));
	    return(error);
    }
    
    omp->om_flags |= OM_READ;
    INSQUE(omp->om_chain, ocp->oc_replys);
    
    return(error);
}

int
odydev_write(dev, uiop, flag)   
    dev_t        dev;  
    struct uio  *uiop; 
    int          flag;
{
    register struct ocomm *ocp;
    register struct omsg  *omp;
    struct ody_out_hdr *out;
    u_long seq;
    u_long opcode;
    int buf[2];
    int error = 0;

    ENTRY;

    if (minor(dev) >= NODYDEV || minor(dev) < 0)
	return(ENXIO);
    
    ocp = &ody_mnttbl[minor(dev)].mi_ocomm;

    /* Peek at the opcode, unique without transfering the data. */
    uiop->uio_rw = UIO_WRITE;
    error = uiomove((caddr_t)buf, sizeof(int) * 2, uiop);
    if (error) {
	myprintf(("odydev_write: error (%d) on uiomove\n", error));
	return(EINVAL);
    }
    
    opcode = buf[0];
    seq = buf[1];
	
    if (odydebug)
	myprintf(("odydev_write got a call for %ld.%ld\n", opcode, seq));
    
    if (DOWNCALL(opcode)) {
	union ody_outputArgs pbuf;
	
	/* get the rest of the data. */
	uiop->uio_rw = UIO_WRITE;
	error = uiomove((caddr_t)&pbuf.ody_purgeuser.oh.result, 
			sizeof(pbuf) - (sizeof(int) * 2),
			uiop);
	if (error) {
	    myprintf(("odydev_write: error (%d) on uiomove (Op %ld seq %ld)\n", 
		      error, opcode, seq));
	    return(EINVAL);
	}
	
	return ody_downcall(opcode, &pbuf);
    }
    
    /* Look for the message on the (waiting for) reply queue. */
    for (omp = (struct omsg *)GETNEXT(ocp->oc_replys);
	 !EOQ(omp, ocp->oc_replys);
	 omp = (struct omsg *)GETNEXT(omp->om_chain))
    {
	if (omp->om_unique == seq) break;
    }
    
    if (EOQ(omp, ocp->oc_replys)) {
	if (odydebug) {
	    myprintf(("odydev_write: msg (%ld, %ld) not found\n",
		      opcode, seq));
	}
	
	return(ESRCH);
    }
    
    /* Remove the message from the reply queue */
    REMQUE(omp->om_chain);
    
    /* move data into response buffer. */
    out = (struct ody_out_hdr *)omp->om_data;
    /* Don't need to copy opcode and uniquifier. */
    
    /* get the rest of the data. */
    if (omp->om_outSize < uiop->uio_resid) {
	myprintf(("odydev_write: more data than asked for (%d < %d)\n",
		  omp->om_outSize, uiop->uio_resid));
	wakeup(&omp->om_sleep);         /* Notify caller of the error. */
	return(EINVAL);
    } 
    
    buf[0] = uiop->uio_resid; 	/* Save this value. */
    uiop->uio_rw = UIO_WRITE;
    error = uiomove((caddr_t) &out->result,
		    omp->om_outSize - (sizeof(int) * 2),
		    uiop);
    if (error) {
	myprintf(("odydev_write: error (%d) on uiomove (op %ld seq %ld)\n", 
		  error, opcode, seq));
	return(EINVAL);
    }
    
    /* I don't think these are used, but just in case. */
    /* XXX - aren't these two already correct? -bnoble */
    out->opcode = opcode;
    out->unique = seq;
    omp->om_outSize = buf[0];    /* Amount of data transferred? */
    omp->om_flags |= OM_WRITE;
    wakeup(&omp->om_sleep);
    
    return(0);
}

int
odydev_ioctl(dev, cmd, addr, flag, p) 
    dev_t         dev;       
    int           cmd;       
    caddr_t       addr;      
    int           flag;      
    struct proc  *p;
{
    ENTRY;

    switch(cmd) {
    case ODYRESIZE: {
	struct ody_resize *data = (struct ody_resize *)addr;
	return(odync_resize(data->hashsize, data->heapsize, IS_DOWNCALL));
	break;
    }
    case ODYSTATS:
	if (odync_use) {
	    odync_gather_stats();
	    return(0);
	} else {
	    return(ENODEV);
	}
	break;
    case ODYPRINT:
	if (odync_use) {
	    print_odync();
	    return(0);
	} else {
	    return(ENODEV);
	}
	break;
    default :
	return(EINVAL);
	break;
    }
}

#ifdef NetBSD1_3
int
odydev_poll(dev, events, p)
    dev_t         dev;
    int           events;
    struct proc  *p;
{
    register struct ocomm *ocp;
    int event_msk = 0;
    
    ENTRY;
    
    if (minor(dev) >= NODYDEV || minor(dev) < 0)
	return(ENXIO);
    
    ocp = &ody_mnttbl[minor(dev)].mi_ocomm;
    
    event_msk = events & (POLLIN|POLLRDNORM);
    if (!event_msk)
	return(0);
    
    if (!EMPTY(ocp->oc_requests))
	return(events & (POLLIN|POLLRDNORM));

    selrecord(p, &ocp->oc_selproc);
    
    return(0);
}
#else
int
odydev_select(dev, flag, p)         
    dev_t         dev;    
    int           flag;   
    struct proc  *p;
{
    register struct ocomm *ocp;
    
    ENTRY;
    
    if (minor(dev) >= NODYDEV || minor(dev) < 0)
	return(ENXIO);
    
    ocp = &ody_mnttbl[minor(dev)].mi_ocomm;
    
    if (flag != FREAD)
	return(0);
    
    if (!EMPTY(ocp->oc_requests))
	return(1);
    
    selrecord(p, &ocp->oc_selproc);
    
    return(0);
}
#endif

/*
 * Statistics
 */
struct ody_clstat ody_clstat;

/* If you want this to be interruptible, set this to > PZERO */
int odycall_sleep = PZERO - 1;

int
odycall(mntinfo, inSize, outSize, buffer) 
    struct ody_mntinfo   *mntinfo; 
    int                  inSize; 
    int                  *outSize; 
    caddr_t              buffer;
{
        struct ocomm *ocp;
        struct omsg  *omp;
        int error;
    
	if (mntinfo == NULL) {
	    /* Unlikely, but could be a race condition with a dying warden */
	    return ENODEV;
	}

	ocp = &(mntinfo->mi_ocomm);
	
	ody_clstat.ncalls++;
	ody_clstat.reqs[((struct ody_in_hdr *)buffer)->opcode]++;

	if (!OC_OPEN(ocp))
	    return(ENODEV);
	
	ODY_ALLOC(omp,struct omsg *,sizeof(struct omsg));
	/* Format the request message. */
	omp->om_data = buffer;
	omp->om_flags = 0;
	omp->om_inSize = inSize;
	omp->om_outSize 
	    = *outSize ? *outSize : inSize; /* |buffer| >= inSize */
	omp->om_opcode = ((struct ody_in_hdr *)buffer)->opcode;
	omp->om_unique = ++ocp->oc_seq;
	if (odydebug)
	    myprintf(("Doing a call for %d.%d\n", 
		      omp->om_opcode, omp->om_unique));
	
	/* Fill in the common input args. */
	((struct ody_in_hdr *)buffer)->unique = omp->om_unique;

	/* Append msg to request queue and poke Venus. */
	INSQUE(omp->om_chain, ocp->oc_requests);
	selwakeup(&ocp->oc_selproc);

	/* We can be interrupted while we wait for Venus to process
	 * our request.  If the interrupt occurs before Venus has read
	 * the request, we dequeue and return. If it occurs after the
	 * read but before the reply, we dequeue, send a signal
	 * message, and return. If it occurs after the reply we ignore
	 * it. In no case do we want to restart the syscall.  If it
	 * was interrupted by a venus shutdown (vcclose), return
	 * ENODEV.  
	 */

	/* Ignore return, We have to check anyway */
	tsleep(&omp->om_sleep, odycall_sleep, "odycall", 0);

	if (OC_OPEN(ocp)) {	/* Viceroy is still alive */
 	/* Op went through, interrupt or not... */
	    if (omp->om_flags & OM_WRITE) {
		error = 0;
		*outSize = omp->om_outSize;
	    }

	    else if (!(omp->om_flags & OM_READ)) { 
		/* Interrupted before venus read it. */
		if (odydebug) {
		    myprintf(("interrupted before read: op = %d.%d, flags = %x\n",
			   omp->om_opcode, omp->om_unique, omp->om_flags));
		}
		REMQUE(omp->om_chain);
		error = EINTR;
	    }
	    
	    else { 	
		/* (!(omp->om_flags & OM_WRITE)) means interrupted after
                   upcall started */
		/* Interrupted after start of upcall, send venus a signal */
		struct ody_in_hdr *dog;
		struct omsg       *somp;
		
		if (odydebug)
		    myprintf(("Sending Venus a signal: op = %d.%d, flags = %x\n",
			   omp->om_opcode, omp->om_unique, omp->om_flags));
		
		REMQUE(omp->om_chain);
		error = EINTR;
		
		ODY_ALLOC(somp, struct omsg *, sizeof (struct omsg));

		ODY_ALLOC((somp->om_data), char *, sizeof(struct ody_in_hdr));
		dog = (struct ody_in_hdr *)somp->om_data;
		
		somp->om_flags = 0;
		dog->opcode = somp->om_opcode = ODY_SIGNAL;
		dog->unique = somp->om_unique = omp->om_unique;
		somp->om_inSize = sizeof (struct ody_in_hdr);
/*??? rvb*/	somp->om_outSize = sizeof (struct ody_in_hdr);
		
		if (odydebug)
		    myprintf(("odycall: enqueing signal msg (%d, %d)\n",
			      somp->om_opcode, somp->om_unique));
		
		/* insert at head of queue! */
		INSQUE(somp->om_chain, ocp->oc_requests);
		selwakeup(&ocp->oc_selproc);
	    }
	}

	else {	/* If venus died (!VC_OPEN(vcp)) */
	    if (odydebug)
		myprintf(("odydev_close woke op %d.%d flags %d\n",
			  omp->om_opcode, omp->om_unique, omp->om_flags));

	    error = ENODEV;
	}

	ODY_FREE(omp, sizeof(struct omsg));

	if (!error)
	    error = ((struct ody_out_hdr *)buffer)->result;
	return(error);
}
