#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Shamelessly stolen from the Coda MiniCache
 */

#include <sys/param.h>
#include <sys/malloc.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/mount.h>

#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#include "kernel/vfs/includes/ody_subr.h"
#include "kernel/vfs/includes/odync.h"
#include "kernel/vfs/includes/odydev.h"

#if	NODYDEV

int ody_active = 0;
int ody_reuse = 0;
int ody_new = 0;

struct onode *ody_freelist = NULL;
struct onode *ody_cache[ODY_CACHESIZE];

#define odyhash(fid)   (((fid)->ofid_ttype + (fid)->ofid_tome  \
			 + (fid)->ofid_vnum)                   \
			&(ODY_CACHESIZE-1))
    
#define	ONODE_NEXT(op)	((op)->o_next)

#define ODD(vnum)        ((vnum) & 0x1)

/*
 * Allocate an onode.
 */
struct onode *
ody_alloc()
{
    struct onode *op;

    if (ody_freelist) {
	op = ody_freelist;
	ody_freelist = ONODE_NEXT(op);
	ody_reuse++;
    }
    else {
	ODY_ALLOC(op, struct onode *, sizeof(struct onode));
	ody_new++;
    }
    bzero(op, sizeof(struct onode));

    return(op);
}

/*
 * Deallocate an onode.
 */
void
ody_free(op)
     register struct onode *op;
{
    ONODE_NEXT(op) = ody_freelist;
    ody_freelist = op;
}

/*
 * Put an onode in the hash table
 */
void
ody_save(op)
     struct onode *op;
{
    ONODE_NEXT(op) = ody_cache[odyhash(&op->o_fid)];
    ody_cache[odyhash(&op->o_fid)] = op;
}

/*
 * Remove an onode from the hash table
 */
void
ody_unsave(op)
     struct onode *op;
{
    struct onode *ptr;
    struct onode *ptrprev = NULL;
    
    ptr = ody_cache[odyhash(&op->o_fid)]; 
    while (ptr != NULL) { 
	if (ptr == op) { 
	    if (ptrprev == NULL) {
		ody_cache[odyhash(&op->o_fid)] = ONODE_NEXT(ptr);
	    } else {
		ONODE_NEXT(ptrprev) = ONODE_NEXT(ptr);
	    }
	    ONODE_NEXT(op) = NULL;
	    return; 
	}	
	ptrprev = ptr;
	ptr = ONODE_NEXT(ptr);
    }	
}

/*
 * Lookup a onode by fid. If the onode is dying, it is bogus so skip it.
 * NOTE: this allows multiple onodes with same fid -- dcs 1/25/95
 * 
 * Addendum: if we get a mount structure, they must match.  Null
 * matches all.  -bnoble 
 */
struct onode *
ody_find(fid, mp) 
    struct ofid *fid;
    struct mount *mp;
{
    struct onode *op;

    op = ody_cache[odyhash(fid)];
    while (op) {
	if (op->o_fid.ofid_ttype == fid->ofid_ttype && 
	    op->o_fid.ofid_tome == fid->ofid_tome &&
	    op->o_fid.ofid_vnum == fid->ofid_vnum &&
	    op->o_fid.ofid_uniq == fid->ofid_uniq &&
	    !IS_UNMOUNTING(op) &&
	    (!mp || OTOV(op)->v_mount == mp))
	{
	    ody_active++;
	    return(op); 
	}		    
	op = ONODE_NEXT(op);
    }
    return(NULL);
}

int
ody_kill(whoIam, dcstat)
    struct mount *whoIam;
    enum dc_status dcstat;
{
    int hash, count = 0;
    struct onode *op;

    /* 
     * Algorithm is as follows: 
     *     Flush whatever vnodes we can from the name cache.
     * 
     *     Step through whatever is left and mark them dying.
     *        This prevents any operation at all.
     */

    odync_flush(dcstat);
    
    for (hash = 0; hash < ODY_CACHESIZE; hash++) {
	for (op = ody_cache[hash]; op != NULL; op = ONODE_NEXT(op)) {
	    if ((OTOV(op)->v_mount) == whoIam) {
		count++;
		ODYDEBUG(ODY_FLUSH, 
			 myprintf(("Live Onode fid %lx.%lx.%lx.%lx flags %d count %d\n",
				   op->o_fid.ofid_ttype,
				   op->o_fid.ofid_tome,
				   op->o_fid.ofid_vnum,
				   op->o_fid.ofid_uniq,
				   op->o_flags,
				   OTOV(op)->v_usecount)); );
	    }
	}
    }
    return count;
}

/*
 * There are two reasons why an onode may be in use, it may be in the
 * name cache or it may be executing.  
 */
void
ody_flush(dcstat)
     enum dc_status dcstat;
{
#if 0
    int hash;
    struct onode *op;
#endif    

    ody_clstat.ncalls++;
    ody_clstat.reqs[ODY_FLUSH]++;
    
    odync_flush(dcstat);	/* flush files from the name cache */

    /* ody_vmflush doesn't do anything, so axe all this code */
#if 0
    for (hash = 0; hash < ODY_CACHESIZE; hash++) {
	for (op = ody_cache[hash]; op != NULL; op = ONODE_NEXT(op)) {  
	    if (!ODD(op->o_fid.vnum)) /* only files can be executed */
		ody_vmflush(op);
	}
    }
#endif
}

/*
 * As a debugging measure, print out any cnodes that lived through a
 * name cache flush.  
 */
void
ody_testflush()
{
    int hash;
    struct onode *op;
    
    for (hash = 0; hash < ODY_CACHESIZE; hash++) {
	for (op = ody_cache[hash];
	     op != NULL;
	     op = ONODE_NEXT(op)) 
	{  
	    myprintf(("Live onode fid %lx.%lx.%lx.%lx\n",
		      op->o_fid.ofid_ttype, op->o_fid.ofid_tome,
		      op->o_fid.ofid_vnum, op->o_fid.ofid_uniq));
	}
    }
}

/*
 *     First, step through all cnodes and mark them unmounting.
 *         NetBSD kernels may try to fsync them now that venus
 *         is dead, which would be a bad thing.
 *
 */
void
ody_unmounting(whoIam)
	struct mount *whoIam;
{	
	int hash;
	struct onode *op;

	for (hash = 0; hash < ODY_CACHESIZE; hash++) {
		for (op = ody_cache[hash]; op != NULL; op = ONODE_NEXT(op)) {
			if (OTOV(op)->v_mount == whoIam) {
				if (op->o_flags & (ON_LOCKED|ON_WANTED)) {
					printf("ody_unmounting: Unlocking %p\n", op);
					op->o_flags &= ~(ON_LOCKED|ON_WANTED);
					wakeup((caddr_t) op);
				}
				op->o_flags |= ON_UNMOUNTING;
			}
		}
	}
}

#ifdef	DEBUG
ody_checkunmounting(mp)
	struct mount *mp;
{	
	register struct vnode *vp, *nvp;
	struct onode *op;
	int count = 0, bad = 0;
loop:
	for (vp = mp->mnt_vnodelist.lh_first; vp; vp = nvp) {
		if (vp->v_mount != mp)
			goto loop;
		nvp = vp->v_mntvnodes.le_next;
		op = VTOO(vp);
		count++;
		if (!(op->o_flags & ON_UNMOUNTING)) {
			bad++;
			printf("vp %p, op %p missed\n", vp, op);
			op->o_flags |= ON_UNMOUNTING;
		}
	}
}

int
ody_cacheprint(whoIam)
	struct mount *whoIam;
{	
	int hash;
	struct onode *op;
	int count = 0;

	printf("ody_cacheprint: ody_ctlvp %p, op %p", ody_ctlvp, VTOO(ody_ctlvp));
	odync_name(ody_ctlvp);
	printf("\n");

	for (hash = 0; hash < ODY_CACHESIZE; hash++) {
		for (op = ody_cache[hash]; op != NULL; op = ONODE_NEXT(op)) {
			if (OTOV(op)->v_mount == whoIam) {
				printf("ody_cacheprint: vp %p, op %p", OTOV(op), op);
				odync_name(op);
				printf("\n");
				count++;
			}
		}
	}
	printf("ody_cacheprint: count %d\n", count);
}
#endif

/*
 * There are 6 cases where invalidations occur. The semantics of each
 * is listed here.
 *
 * ODY_FLUSH     -- flush all entries from the name cache and the onode cache.
 * ODY_PURGEUSER -- flush all entries from the name cache for a specific user
 *                  This call is a result of token expiration.
 *
 * The next two are the result of callbacks on a file or directory.
 * ODY_ZAPDIR    -- flush the attributes for the dir from its onode.
 *                  Zap all children of this directory from the namecache.
 * ODY_ZAPFILE   -- flush the attributes for a file.
 *
 * The fifth is a result of Venus detecting an inconsistent file.
 * ODY_PURGEFID  -- flush the attribute for the file
 *                  If it is a dir (odd vnode), purge its 
 *                  children from the namecache
 *                  remove the file from the namecache.
 *
 * The sixth allows Venus to replace local fids with global ones
 * during reintegration.
 *
 * ODY_REPLACE -- replace one ViceFid with another throughout the name cache */
 
int ody_downcall(opcode, out)
    int                  opcode; 
    union ody_outputArgs *out;
{
    /* Handle invalidate requests. */
    switch (opcode) {
	
        case ODY_FLUSH: {
	    ody_flush(IS_DOWNCALL);
	    
	    ODYDEBUG(ODY_FLUSH, ody_testflush();); /* print remaining onodes */
	    return(0);
	}
	    
	case ODY_PURGEUSER: {
	    ody_clstat.ncalls++;
	    ody_clstat.reqs[ODY_PURGEUSER]++;
	    
	    odync_purge_user(out->ody_purgeuser.cred.ocr_uid, IS_DOWNCALL);
	    return(0);
	}
	    
	case ODY_ZAPFILE : {
	    struct onode *op;
	    int error = 0;
	    
	    ody_clstat.ncalls++;
	    ody_clstat.reqs[ODY_ZAPFILE]++;
	    
	    op = ody_find(&out->ody_zapfile.fid, NULL);
	    if (op != NULL) {
		vref(OTOV(op));
		
		op->o_flags &= ~ON_VATTR;

		/* ody_vmflush doesn't do anything, so axe all this code */
#if 0
		if (OTOV(op)->v_flag & VTEXT)
		    error = ody_vmflush(op);
#endif
		
		ODYDEBUG(ODY_ZAPFILE, myprintf(("zapfile: fid = (%lx.%lx.%lx.%lx), refcnt = %d, error = %d\n",
						op->o_fid.ofid_ttype,
						op->o_fid.ofid_tome,
						op->o_fid.ofid_vnum,
						op->o_fid.ofid_uniq,
						OTOV(op)->v_usecount,error)););
		    
		if (OTOV(op)->v_usecount == 1) {
		    op->o_flags |= ON_PURGING;
		}
		vrele(OTOV(op));
	    }
	    
	    return(error);
	}
	    
	case ODY_ZAPDIR : {
	    struct onode *op;
	    
	    ody_clstat.ncalls++;
	    ody_clstat.reqs[ODY_ZAPDIR]++;
	    
	    op = ody_find(&out->ody_zapdir.fid, NULL);
	    if (op != NULL) {
		vref(OTOV(op));
		
		op->o_flags &= ~ON_VATTR;
		odync_zapParentfid(&out->ody_zapdir.fid, IS_DOWNCALL);
		
		ODYDEBUG(ODY_ZAPDIR, myprintf(("zapdir: fid = (%lx.%lx.%lx.%lx), refcnt = %d\n",
					       op->o_fid.ofid_ttype,
					       op->o_fid.ofid_tome,
					       op->o_fid.ofid_vnum,
					       op->o_fid.ofid_uniq,
					       OTOV(op)->v_usecount - 1)););
		    
		if (OTOV(op)->v_usecount == 1) {
		    op->o_flags |= ON_PURGING;
		}
		vrele(OTOV(op));
	    }
	    
	    return(0);
	}
	    
	case ODY_ZAPVNODE : {
	    ody_clstat.ncalls++;
	    ody_clstat.reqs[ODY_ZAPVNODE]++;
	    
	    myprintf(("ODY_ZAPVNODE: Called, but uniplemented\n"));
	    /* Not that below we must really translate the returned
	     * coda_cred to a netbsd cred.  This is a bit muddled at
	     * present and the odync_zapnode is further unimplemented,
	     * so punt!  I suppose we could use just the uid.  */
	    /* odync_zapvnode(&out->ody_zapvnode.VFid,
	       &out->ody_zapvnode.cred, IS_DOWNCALL); */
	    return(0);
	}	
	    
	case ODY_PURGEFID : {
	    struct onode *op;
	    int error = 0;
	    
	    ody_clstat.ncalls++;
	    ody_clstat.reqs[ODY_PURGEFID]++;
	    
	    op = ody_find(&out->ody_purgefid.fid, NULL);
	    if (op != NULL) {
		vref(OTOV(op));
		
		if (ODD(out->ody_purgefid.fid.ofid_vnum)) 
		{ /* Vnode is a directory */
		    odync_zapParentfid(&out->ody_purgefid.fid, IS_DOWNCALL);
		}
		
		op->o_flags &= ~ON_VATTR;
		odync_zapfid(&out->ody_purgefid.fid, IS_DOWNCALL);

		/* ody_vmflush doesn't do anything, so axe all this code */
#if 0
		if (!(ODD(out->ody_purgefid.fid.vnum)) 
		    && (OTOV(op)->v_flag & VTEXT)) 
		{
		    error = ody_vmflush(op);
		}
#endif
		ODYDEBUG(ODY_PURGEFID,
			 myprintf(("purgefid: fid = (%lx.%lx.%lx.%lx), refcnt = %d, error = %d\n",
				   op->o_fid.ofid_ttype,
				   op->o_fid.ofid_tome,
				   op->o_fid.ofid_vnum,
				   op->o_fid.ofid_uniq,
				   OTOV(op)->v_usecount - 1,
				   error)););
		
		if (OTOV(op)->v_usecount == 1) {
		    op->o_flags |= ON_PURGING;
		}
		vrele(OTOV(op));
	    }
	    return(error);
	}
	    
	case ODY_REPLACE : {
	    struct onode *op = NULL;
	    
	    ody_clstat.ncalls++;
	    ody_clstat.reqs[ODY_REPLACE]++;
	    
	    op = ody_find(&out->ody_replace.OldFid, NULL);
	    if (op != NULL) { 
		/* remove the onode from the hash table,
		   replace the fid, and reinsert */
		vref(OTOV(op));
		ody_unsave(op);
		op->o_fid = out->ody_replace.NewFid;
		ody_save(op);
		
		ODYDEBUG(ODY_REPLACE,
			 myprintf(("replace: oldfid = (%lx.%lx.%lx.%lx), newfid = (%lx.%lx.%lx.%lx), op = %p\n",
				   out->ody_replace.OldFid.ofid_ttype,
				   out->ody_replace.OldFid.ofid_tome,
				   out->ody_replace.OldFid.ofid_vnum,
				   out->ody_replace.OldFid.ofid_uniq,
				   op->o_fid.ofid_ttype,
				   op->o_fid.ofid_tome,
				   op->o_fid.ofid_vnum,
				   op->o_fid.ofid_uniq,
				   op));)
		vrele(OTOV(op));
	    }
	    return (0);
	}
        default: {
	    myprintf(("ody_downcall: unknown opcode %d\n", opcode));
	    return (EINVAL);
	}
    }
}

/* 
 * All calls to ody_vmflush should get commented out 
 */
#if 0
int
ody_vmflush(op)
     struct onode *op;
{
#if	0
  /* old code */
    /* Unset <device, inode> so that page_read doesn't try to use
       (possibly) invalid cache file. */
    op->o_device = 0;
    op->o_inode = 0;

    return(inode_uncache_try(VTOI(OTOV(op))) ? 0 : ETXTBSY);
#else /* __NetBSD__ || __FreeBSD__ */
    return 0;
#endif /* __NetBSD__ || __FreeBSD__ */
}
#endif

/*
 * Utilities used by both client and server
 * Standard levels:
 * 0) no debugging
 * 1) hard failures
 * 2) soft failures
 * 3) current test software
 * 4) main procedure entry points
 * 5) main procedure exit points
 * 6) utility procedure entry points
 * 7) utility procedure exit points
 * 8) obscure procedure entry points
 * 9) obscure procedure exit points
 * 10) random stuff
 * 11) all <= 1
 * 12) all <= 2
 * 13) all <= 3
 * ...
 */


#endif	/* NODYDEV */
