#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/proc.h>
#include <sys/malloc.h>
#include <sys/mbuf.h>
#include <sys/select.h>
#include <sys/vnode.h>
#include <sys/mount.h>
#include <sys/namei.h>

#include "kernel/upcall/includes/upcall.h"
#include "kernel/upcall/bsd44/upcall.private.h"
#include "kernel/upcall/bsd44/uc_upclass.h"

#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#include "kernel/vfs/includes/ody_upcall.h"
#include "kernel/vfs/includes/ody_viceroy.h"

int
ody_notify(ucldp, tabp, p, upcall, reg, peh)
	struct uc_ucld_t     *ucldp;
	void                 *tabp;
	struct proc          *p;
	upcall_t             upcall;
	int                  reg;
	struct uc_procent_t  **peh;
{
	struct nameidata nd;
	int              result;
	char             name[MAXPATHLEN];

	if (upcall != ODY_NOTIFICATION) {
		return EOPNOTSUPP;
	}

	result = uc_notify_default(ucldp, tabp, p, upcall, reg, peh);

	if (result) return result;

	if (!reg) {
		strcpy(name, "/ody/");
		strcat(name, ODY_CONTROL);
		NDINIT(&nd, LOOKUP, FOLLOW | LOCKLEAF, UIO_SYSSPACE,
		       name, p);
		if ((result = namei(&nd))) {
#ifdef DIAGNOSTIC
			printf("ody_notify: namei of %s failed (%d)\n",
			       name, result);
#endif
			return 0; /* No such path, probably */
		}
		/* Needs to be on control object */
		if (!IS_ODY_CTL_VP(nd.ni_vp)) {
		    ODYDEBUG(ODY_IOCTL, 
			     myprintf(("ody_cancpid error: vp != ctlvp")););
		    vput(nd.ni_vp);
		    return (EOPNOTSUPP);
		}
		result = viceroy_cancpid(vtomi(nd.ni_vp), p->p_ucred, p);
		vput(nd.ni_vp);
#ifdef DIAGNOSTIC
		if (result) {
			printf("ody_notify: viceroy_cancpid returns %d\n",
			       result);
		}
#endif
	}
	return 0;
}
		

struct uc_ucld_t ody_ucld = {
	UC_ODYSSEY,
	0,
	"Odyssey",
	uc_init_default,
	uc_exist_default,
	ody_notify,
	NULL,
	uc_getpep_default,
	uc_rcpt_default,
	uc_print_default,
	NULL
};
       
