#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/conf.h>
#include <sys/namei.h>
#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/vnode.h>
#ifdef __FreeBSD__ /* XXX gruesome hack */
#include "specdev.h"
#else
#include <miscfs/specfs/specdev.h>
#endif
#ifdef SETS
#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_domain.h"
#endif

#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#include "kernel/vfs/includes/ody_vfsops.h"
#include "kernel/vfs/includes/ody_vnodeops.h"
#include "kernel/vfs/includes/ody_viceroy.h"
#include "kernel/vfs/includes/ody_subr.h"
#include "kernel/vfs/includes/ody_opstats.h"
#include "kernel/vfs/includes/odydev.h"

int odydebug = 0; /* ODY_LOOKUP|ODY_OPEN|ODY_RDWR|ODY_CLOSE; */

int ody_vfsop_print_entry = 0;
#ifdef __GNUC__
#define ENTRY    \
    if(ody_vfsop_print_entry) myprintf(("Entered %s\n",__FUNCTION__))
#else
#define ENTRY
#endif 


struct vnode *ody_ctlvp = 0;
    
struct ody_mntinfo ody_mnttbl[NODYDEV]; /* indexed by minor device number */

/* structure to keep statistics of internally generated/satisfied calls */

struct ody_op_stats ody_vfsopstats[ODY_VFSOPS_SIZE];

#define MARK_ENTRY(op) (ody_vfsopstats[op].entries++)
#define MARK_INT_SAT(op) (ody_vfsopstats[op].sat_intrn++)
#define MARK_INT_FAIL(op) (ody_vfsopstats[op].unsat_intrn++)
#define MRAK_INT_GEN(op) (ody_vfsopstats[op].gen_intrn++)

extern int odync_initialized;     /* Set if cache has been initialized */
extern int odydev_open __P((dev_t, int, int, struct proc*));
#ifdef __NetBSD__
extern struct cdevsw cdevsw[];    /* For sanity check in ody_mount */
#endif

#ifdef	__NetBSD__
struct vfsops ody_vfsops = {
    MOUNT_ODY,
    ody_mount,
    ody_start,
    ody_unmount,
    ody_root,
    ody_quotactl,
    ody_statfs,
    ody_sync,
    ody_vget,
    (int (*) (struct mount *, struct fid *, struct mbuf *, struct vnode **,
	      int *, struct ucred **)) eopnotsupp,
    (int (*) (struct vnode *, struct fid *)) eopnotsupp,
    ody_init,
#ifdef	NetBSD1_3
    (int (*)(void)) eopnotsupp,
#endif
    0
};

#elif	defined(__FreeBSD__)
struct vfsops ody_vfsops = {
    ody_mount,
    ody_start,
    ody_unmount,
    ody_root,
    ody_quotactl,
    ody_statfs,
    ody_sync,
    ody_vget,
    (int (*) (struct mount *, struct fid *, struct mbuf *, struct vnode **,
	      int *, struct ucred **))
    eopnotsupp,
    (int (*) (struct vnode *, struct fid *)) eopnotsupp,
    ody_init,
};

#endif

int ody_vfsopstats_init(void)
{
	register int i;
	
	for (i=0;i<ODY_VFSOPS_SIZE;i++) {
		ody_vfsopstats[i].opcode = i;
		ody_vfsopstats[i].entries = 0;
		ody_vfsopstats[i].sat_intrn = 0;
		ody_vfsopstats[i].unsat_intrn = 0;
		ody_vfsopstats[i].gen_intrn = 0;
	}
	
	return 0;
}
	

/*
 * odyssey mount vfsop
 * Set up mount info record and attach it to vfs struct.
 */
/*ARGSUSED*/
int ody_mount(vfsp, path, data, ndp, p)
    struct mount *vfsp;    /* Allocated and initialized by mount(2) */
#ifdef NetBSD1_3
    const char *path;      /* path covered: ignored by the fs-layer */
    void *data;            /* Need to define a data type for this in netbsd? */
#else
    char *path;            /* path covered: ignored by the fs-layer */
    caddr_t data;          /* Need to define a data type for this in netbsd? */
#endif
    struct nameidata *ndp; /* Clobber this to lookup the device name */
    struct proc *p;        /* The ever-famous proc pointer */
{
    struct vnode *dvp;
    struct onode *op;
    dev_t dev;
    struct ody_mntinfo *mi;
    struct vnode *rootvp;
    struct ofid rootfid;
    struct ofid ctlfid;
    int error;

    ENTRY;

    ody_vfsopstats_init();
    ody_vnodeopstats_init();
    
    MARK_ENTRY(ODY_MOUNT_STATS);
    if (ODY_MOUNTED(vfsp)) {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	return(EBUSY);
    }
    
    /* Validate mount device.  Similar to getmdev(). */

    NDINIT(ndp, LOOKUP, FOLLOW, UIO_USERSPACE, data, p);
    error = namei(ndp);
    dvp = ndp->ni_vp;

    if (error) {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	return (error);
    }
    if (dvp->v_type != VCHR) {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	vrele(dvp);
	return(ENXIO);
    }
    dev = dvp->v_specinfo->si_rdev;
    vrele(dvp);
    if (major(dev) >= nchrdev || major(dev) < 0) {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	return(ENXIO);
    }
    
    /*
     * See if the device table matches our expectations.
     */
#ifdef	__NetBSD__
    if (cdevsw[major(dev)].d_open != odydev_open)
#elif	defined(__FreeBSD__)
    if (cdevsw[major(dev)]->d_open != odydev_open)
#endif
    {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	return(ENXIO);
    }
    
    if (minor(dev) >= NODYDEV || minor(dev) < 0) {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	return(ENXIO);
    }
    
    /*
     * Initialize the mount record and link it to the vfs struct
     */
    mi = &ody_mnttbl[minor(dev)];
    
    if (!OC_OPEN(&mi->mi_ocomm)) {
	MARK_INT_FAIL(ODY_MOUNT_STATS);
	return(ENODEV);
    }
    
    /* No initialization (here) of mi_vcomm! */
    vfsp->mnt_data = (qaddr_t)mi;
#ifdef	__NetBSD__
    vfsp->mnt_stat.f_fsid.val[0] = 0;
    vfsp->mnt_stat.f_fsid.val[1] = makefstype(MOUNT_ODY);
#elif	defined(__FreeBSD__)
    /* Seems a bit overkill, since usualy /coda is the only mount point
     * for ody.
     */
    getnewfsid (vfsp, MOUNT_ODY);
#endif
    mi->mi_vfsp = vfsp;
    
    /*
     * Make a root vnode to placate the Vnode interface, but don't
     * actually make the ODY_ROOT call to venus until the first call
     * to ody_root in case a server is down while venus is starting.
     */
    rootfid.ofid_ttype = 0;
    rootfid.ofid_tome = 0;
    rootfid.ofid_vnum = 0;
    rootfid.ofid_uniq = 0;
    op = makeodynode(&rootfid, vfsp, VDIR);
    rootvp = OTOV(op);
    rootvp->v_flag |= VROOT;
    
    if (ody_ctlvp == 0) {
	ctlfid.ofid_ttype = ODY_CTL_TYPE;
	ctlfid.ofid_tome = ODY_CTL_TOME;
	ctlfid.ofid_vnum = ODY_CTL_VNO;
	ctlfid.ofid_uniq = ODY_CTL_UNI;
/*  op = makeodynode(&ctlfid, vfsp, VCHR);
    The above code seems to cause a loop in the onode links.
    I don't totally understand when it happens, it is caught
    when closing down the system.
    */
	op = makeodynode(&ctlfid, vfsp, VCHR);
	ody_ctlvp = OTOV(op);
	ody_ctlvp->v_specinfo = (struct specinfo *)
	    malloc((u_long) sizeof(struct specinfo), M_VNODE, M_NOWAIT);

	/* XXX this is a gross hack; I have no idea why it works
	   (or doesn't) */
	ody_ctlvp->v_hashchain = &ody_ctlvp;
	ody_ctlvp->v_specnext = ody_ctlvp;
	ody_ctlvp->v_specflags = 0;	
	ody_ctlvp->v_specinfo->si_rdev = makedev(ODY_CTL_MAJOR, ODY_CTL_MINOR);
#ifndef __FreeBSD__
	ody_ctlvp->v_speclockf = NULL;
#endif
    }

    /* Add vfs and rootvp to chain of vfs hanging off mntinfo */
    mi->mi_vfsp = vfsp;
    mi->mi_rootvp = rootvp;
    
    /* set filesystem block size */
    vfsp->mnt_stat.f_bsize = 8192;	    /* XXX -JJK */
#ifdef	 __FreeBSD__
    /* Set f_iosize.  XXX -- inamura@isl.ntt.co.jp. 
       For vnode_pager_haspage() references. The value should be obtained 
       from underlying UFS. */
    /* Checked UFS. iosize is set as 8192 */
    vfsp->mnt_stat.f_iosize = 8192;
#endif
    
#ifdef SETS
    sets_register_domain(VT_ODYSSEY, ody_expand, ody_cursexpand, ody_prefetch);
#endif
    
    /* error is currently guaranteed to be zero, but in case some
       code changes... */
    ODYDEBUG(1, myprintf(("ody_mount returned %d\n",error)););
    if (error)
	MARK_INT_FAIL(ODY_MOUNT_STATS);
    else
	MARK_INT_SAT(ODY_MOUNT_STATS);
    
    return(error);
}

int
ody_start(vfsp, flags, p)
    struct mount *vfsp;
    int flags;
    struct proc *p;
{
    ENTRY;
    return (0);
}

int
ody_unmount(vfsp, mntflags, p)
    struct mount *vfsp;
    int mntflags;
    struct proc *p;
{
    struct ody_mntinfo *mi = vftomi(vfsp);
    int active, error = 0;
    
    ENTRY;
    MARK_ENTRY(ODY_UMOUNT_STATS);
    if (!ODY_MOUNTED(vfsp)) {
	MARK_INT_FAIL(ODY_UMOUNT_STATS);
	return(EINVAL);
    }

    if (ody_ctlvp) {
	vrele(ody_ctlvp);
    }

    if (mi->mi_vfsp == vfsp) {	/* We found the victim */
	if (!IS_UNMOUNTING(VTOO(mi->mi_rootvp)))
	    return (EBUSY); 	/* Venus is still running */
	
	vrele(mi->mi_rootvp);

	active = ody_kill(vfsp, NOT_DOWNCALL);

#ifdef NetBSD1_3
	if ((error = vfs_busy(mi->mi_vfsp)) == 0) {
	    error = vflush(mi->mi_vfsp, NULLVP, FORCECLOSE);
	    printf("ody_unmount: active = %d, vflush active %d\n", active,
		   error);
	    error = 0;
	} else {
	    printf("ody_unmount: busy\n");
	}
#else

	active = ody_kill(vfsp, NOT_DOWNCALL);

	error = vflush(mi->mi_vfsp, NULLVP, FORCECLOSE);
	printf("ody_unmount: active = %d, vflush active %d\n", active, error);
	error = 0;
#endif

	ody_ctlvp = 0;

	/* I'm going to take this out to allow lookups to go through. I'm
	 * not sure it's important anyway. -- DCS 2/2/94
	 */
	/* vfsp->VFS_DATA = NULL; */
	
	/* No more vfsp's to hold onto */
	mi->mi_vfsp = NULL;
	mi->mi_rootvp = NULL;
	
	if (error)
	    MARK_INT_FAIL(ODY_UMOUNT_STATS);
	else
	    MARK_INT_SAT(ODY_UMOUNT_STATS);

#ifdef SETS
	sets_unregister_domain(VT_ODYSSEY);
#endif

	return(error);
    }
    return(EINVAL);
}

/*
 * find root of odyssey file sys
 */
int
ody_root(vfsp, vpp)
    struct mount *vfsp;
    struct vnode **vpp;
{
    struct ody_mntinfo *mi = vftomi(vfsp);
    struct vnode **result;
    int error;
    struct proc *p = curproc;    /* XXX - bnoble */
    struct ofid fid;

    ENTRY;
    MARK_ENTRY(ODY_ROOT_STATS);
    result = NULL;
    
    if (vfsp == mi->mi_vfsp) {
	if ((VTOO(mi->mi_rootvp)->o_fid.ofid_ttype != 0) ||
	    (VTOO(mi->mi_rootvp)->o_fid.ofid_tome != 0)  ||
	    (VTOO(mi->mi_rootvp)->o_fid.ofid_vnum != 0)  ||
	    (VTOO(mi->mi_rootvp)->o_fid.ofid_uniq != 0))
	{
	    /* Found valid root. */
	    *vpp = mi->mi_rootvp;
	    vref(*vpp);
	    VOP_LOCK(*vpp);
	    MARK_INT_SAT(ODY_ROOT_STATS);
	    return(0);
	}
    }

    error = viceroy_root(vftomi(vfsp), p->p_cred->pc_ucred, p, &fid);

    if (!error) {
	/*
	 * Save the new rootfid in the cnode, and rehash the cnode into the
	 * cnode hash with the new fid key.
	 */
	ody_unsave(VTOO(mi->mi_rootvp));
	VTOO(mi->mi_rootvp)->o_fid = fid;
	ody_save(VTOO(mi->mi_rootvp));

	*vpp = mi->mi_rootvp;
	vref(*vpp);
	VOP_LOCK(*vpp);
	MARK_INT_SAT(ODY_ROOT_STATS);
	goto exit;
    } else if (error == ENODEV) {
	/* Gross hack here! */
	/*
	 * If Venus fails to respond to the ODY_ROOT call, odycall returns
	 * ENODEV. Return the uninitialized root vnode to allow vfs
	 * operations such as unmount to continue. Without this hack,
	 * there is no way to do an unmount if Venus dies before a 
	 * successful ODY_ROOT call is done. All vnode operations 
	 * will fail.
	 */
	*vpp = mi->mi_rootvp;
	vref(*vpp);
	VOP_LOCK(*vpp);
	MARK_INT_FAIL(ODY_ROOT_STATS);
	error = 0;
	goto exit;
    } else {
	ODYDEBUG( ODY_ROOT, myprintf(("error %d in ODY_ROOT\n", error)); );
	MARK_INT_FAIL(ODY_ROOT_STATS);
		
	goto exit;
    }
 exit:
    return(error);
}

int
ody_quotactl(vfsp, cmd, uid, arg, p)
    struct mount *vfsp;
    int cmd;
    uid_t uid;
    caddr_t arg;
    struct proc *p;
{
    ENTRY;
    return (EOPNOTSUPP);
}
    

/*
 * Get file system statistics.
 */
int
ody_statfs(vfsp, sbp, p)
    register struct mount *vfsp;
    struct statfs *sbp;
    struct proc *p;
{
    ENTRY;
    MARK_ENTRY(ODY_STATFS_STATS);
    if (!ODY_MOUNTED(vfsp)) {
	return(EINVAL);
    }

    bzero(sbp, sizeof(struct statfs));
    /* XXX - what to do about f_flags, others? --bnoble */
    /* Below This is what AFS does
    	#define NB_SFS_SIZ 0x895440
     */
    /* Note: Normal fs's have a bsize of 0x400 == 1024 */
#ifdef	__NetBSD__
    sbp->f_type = 0;
#elif	defined(__FreeBSD__)
    sbp->f_type = MOUNT_ODY;
#endif
    sbp->f_bsize = 8192;	/* XXX -JJK */
    sbp->f_iosize = 8192;
#define NB_SFS_SIZ 0x8AB75D
    sbp->f_blocks = NB_SFS_SIZ;
    sbp->f_bfree = NB_SFS_SIZ;
    sbp->f_bavail = NB_SFS_SIZ;
    sbp->f_files = NB_SFS_SIZ;
    sbp->f_ffree = NB_SFS_SIZ;
    bcopy((caddr_t)&(vfsp->mnt_stat.f_fsid), (caddr_t)&(sbp->f_fsid),
	  sizeof (fsid_t));
#ifdef	__NetBSD__
    strncpy(sbp->f_fstypename, MOUNT_ODY, MFSNAMELEN-1);
#endif
    strcpy(sbp->f_mntonname, "/ody"); /* XXX cheesy */
    strcpy(sbp->f_mntfromname, "ODY");
    
    MARK_INT_SAT(ODY_STATFS_STATS);
    return(0);
}

/*
 * Flush any pending I/O.
 */
int
ody_sync(vfsp, waitfor, cred, p)
    struct mount *vfsp;
    int    waitfor;
    struct ucred *cred;
    struct proc *p;
{
    ENTRY;
    MARK_ENTRY(ODY_SYNC_STATS);
    MARK_INT_SAT(ODY_SYNC_STATS);
    return(0);
}


int
ody_vget(vfsp, ino, vpp)
    struct mount *vfsp;
    ino_t ino;
    struct vnode **vpp;
{
    ENTRY;
    return (EOPNOTSUPP);
}

/* 
 * fhtovp is now what vget used to be in 4.3-derived systems.  For
 * some silly reason, vget is now keyed by a 32 bit ino_t, rather than
 * a type-specific fid.  
 */
int
ody_fhtovp(vfsp, fhp, nam, vpp, exflagsp, creadanonp)
    register struct mount *vfsp;    
    struct fid *fhp;
    struct mbuf *nam;
    struct vnode **vpp;
    int *exflagsp;
    struct ucred **creadanonp;
{
    struct ofid_overlay *ovlp = (struct ofid_overlay *)fhp;
    struct onode *op = 0;
    int error;
    struct proc *p = curproc; /* XXX -mach */
    struct ofid fid;
    int vtype;

    ENTRY;
    
    MARK_ENTRY(ODY_VGET_STATS);
    /* Check for vget of control object. */
    if (IS_ODY_CTL_FID(&ovlp->ov_fid)) {
	*vpp = ody_ctlvp;
	vref(ody_ctlvp);
	MARK_INT_SAT(ODY_VGET_STATS);
	return(0);
    }

    error = viceroy_fhtovp(vftomi(vfsp), &ovlp->ov_fid,
			   p->p_cred->pc_ucred, p, &fid, &vtype);
    
    if (error) {
	ODYDEBUG(ODY_VGET, myprintf(("vget error %d\n",error)););
	    *vpp = (struct vnode *)0;
    }
    else {
	ODYDEBUG(ODY_VGET, 
		 myprintf(("vget: (0x%lx.0x%lx.0x%lx.0x%lx) type %d result %d\n",
			   fid.ofid_ttype, fid.ofid_tome,
			   fid.ofid_vnum, fid.ofid_uniq,
			   vtype, error)); );
	    
	op = makeodynode(&fid, vfsp, vtype);
	*vpp = OTOV(op);
    }
    return(error);
}

int
ody_vptofh(vnp, fidp)
    struct vnode *vnp;
    struct fid   *fidp;
{
    ENTRY;
    return (EOPNOTSUPP);
}
 
#ifdef __NetBSD__
void
ody_init()
{
    ENTRY;
}
#elif defined(__FreeBSD__)
int
ody_init()
{
    ENTRY;
    return 0;
}
#endif

#include <ufs/ufs/quota.h>
#include <ufs/ufs/ufsmount.h>
/* get the mount structure corresponding to a given device.  Assume 
 * device corresponds to a UFS. Return NULL if no device is found.
 */ 
struct mount *ody_devtomp(dev)
    dev_t dev;
{
    struct mount *mp, *nmp;
    
    for (mp = mountlist.cqh_first; mp != (void*)&mountlist; mp = nmp) {
	nmp = mp->mnt_list.cqe_next;
	if (
#ifdef	__NetBSD__
	    (!strcmp(mp->mnt_op->vfs_name, MOUNT_UFS)) &&
#endif
	    ((VFSTOUFS(mp))->um_dev == (dev_t) dev)) {
	    /* mount corresponds to UFS and the device matches one we want */
	    return(mp); 
	}
    }
    /* mount structure wasn't found */ 
    return(NULL); 
}
