#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/time.h>
#include <sys/systm.h>
#include <sys/param.h>
#include <sys/malloc.h>
#include <sys/proc.h>
#include <sys/select.h>
#include <sys/ioctl.h>
/* for CNV_OFLAGS below */
#include <sys/fcntl.h>
/* for CNV_x_VATTR below */
#include <sys/stat.h>

#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#include "kernel/vfs/includes/ody_viceroy.h"
#include "kernel/vfs/includes/odyapi.h"

#define DECL_NO_IN(name) 				\
    struct ody_in_hdr *inp;				\
    struct name ## _out *outp;				\
    int name ## _size = sizeof (struct ody_in_hdr);	\
    int Isize = sizeof (struct ody_in_hdr);		\
    int Osize = sizeof (struct name ## _out);		\
    int error

#define DECL(name)					\
    struct name ## _in *inp;				\
    struct name ## _out *outp;				\
    int name ## _size = sizeof (struct name ## _in);	\
    int Isize = sizeof (struct name ## _in);		\
    int Osize = sizeof (struct name ## _out);		\
    int error

#define DECL_NO_OUT(name)				\
    struct name ## _in *inp;				\
    struct ody_out_hdr *outp;				\
    int name ## _size = sizeof (struct name ## _in);	\
    int Isize = sizeof (struct name ## _in);		\
    int Osize = sizeof (struct ody_out_hdr);		\
    int error

#define ALLOC_NO_IN(name)				\
    if (Osize > name ## _size)				\
    	name ## _size = Osize;				\
    ODY_ALLOC(inp, struct ody_in_hdr *, name ## _size);\
    outp = (struct name ## _out *) inp

#define ALLOC(name)					\
    if (Osize > name ## _size)				\
    	name ## _size = Osize;				\
    ODY_ALLOC(inp, struct name ## _in *, name ## _size);\
    outp = (struct name ## _out *) inp

#define ALLOC_NO_OUT(name)				\
    if (Osize > name ## _size)				\
    	name ## _size = Osize;				\
    ODY_ALLOC(inp, struct name ## _in *, name ## _size);\
    outp = (struct ody_out_hdr *) inp

#define STRCPY(struc, name, len) \
    bcopy(name, (char *)inp + (int)inp->struc, len); \
    ((char*)inp + (int)inp->struc)[len++] = 0; \
    Isize += len

#define INIT_IN(in, op, ident, p) \
	  (in)->opcode = (op); \
	  (in)->pid = p ? p->p_pid : -1; \
          (in)->pgid = p ? p->p_pgid : -1; \
          (in)->sid = (p && p->p_session && p->p_session->s_leader) ? (p->p_session->s_leader->p_pid) : -1; \
          if (ident != NOCRED) {                              \
	      (in)->cred.ocr_uid = ident->cr_uid;              \
	      (in)->cred.ocr_gid = ident->cr_gid;          \
          } else {                                            \
	      bzero(&((in)->cred),sizeof(struct ody_cred));  \
	      (in)->cred.ocr_uid = -1;                         \
	      (in)->cred.ocr_gid = -1;                     \
          }                                                   \

#define	CNV_OFLAG(to, from) 				\
    do { 						\
	  to = 0;					\
	  if (from & FREAD)   to |= O_O_READ; 		\
	  if (from & FWRITE)  to |= O_O_WRITE; 		\
          if (from & O_NONBLOCK) to |= O_O_NONBLOCK;    \
	  if (from & O_TRUNC) to |= O_O_TRUNC; 		\
	  if (from & O_EXCL)  to |= O_O_EXCL; 		\
    } while (0)

#define	CNV_V2VV_RWFLAG(to, from) 				   \
    do { 						   \
          if (from == UIO_READ) to = O_RW_READ;            \
          else                  to = O_RW_WRITE;           \
    } while (0)

#define	CNV_VV2V_RWFLAG(to, from) 				   \
    do { 						   \
          if (from == O_RW_READ) to = UIO_READ;            \
          else                   to = UIO_WRITE;           \
    } while (0)

#define	CNV_MODE(to, from) 				\
    do { 						\
          to = 0;                                       \
	  if (from == VREAD)       to = O_A_READ; 	\
	  else if (from == VWRITE) to = O_A_WRITE; 	\
	  else if (from == VEXEC)  to = O_A_EXEC; 	\
    } while (0)

#define CNV_VV2V_ATTR(top, fromp) \
	do { \
		(top)->va_type = (fromp)->va_type; \
                if ((fromp)->va_mode == O_S_READ)    \
                   (top)->va_mode = S_IRUSR | S_IRGRP | S_IROTH; \
                else \
                   (top)->va_mode = S_IRUSR | S_IRGRP | S_IROTH | \
			            S_IWUSR | S_IWGRP | S_IWOTH; \
		(top)->va_nlink = (fromp)->va_nlink; \
		(top)->va_uid = (fromp)->va_uid; \
		(top)->va_gid = (fromp)->va_gid; \
		(top)->va_fsid = VNOVAL; \
		(top)->va_fileid = (fromp)->va_fileid; \
		(top)->va_size = (fromp)->va_size; \
		(top)->va_blocksize = (fromp)->va_blocksize; \
		(top)->va_atime.tv_sec = (fromp)->va_atime; \
		(top)->va_atime.tv_nsec = 0; \
		(top)->va_mtime.tv_sec = (fromp)->va_mtime; \
		(top)->va_mtime.tv_nsec = 0; \
		(top)->va_ctime.tv_sec = (fromp)->va_ctime; \
		(top)->va_ctime.tv_nsec = 0; \
		(top)->va_gen = (fromp)->va_gen; \
		(top)->va_flags = (fromp)->va_flags; \
		(top)->va_rdev = (fromp)->va_rdev; \
		(top)->va_bytes = (fromp)->va_bytes; \
		(top)->va_filerev = (fromp)->va_filerev; \
		(top)->va_vaflags = VNOVAL; \
		(top)->va_spare = VNOVAL; \
	} while (0)

#define CNV_V2VV_ATTR(top, fromp) \
	do { \
		(top)->va_type = (fromp)->va_type; \
                if ((fromp)->va_mode & (S_IWUSR | S_IWGRP | S_IWOTH)) \
                    (top)->va_mode = O_S_WRITE; \
                else \
                    (top)->va_mode = O_S_READ; \
		(top)->va_nlink = (fromp)->va_nlink; \
		(top)->va_uid = (fromp)->va_uid; \
		(top)->va_gid = (fromp)->va_gid; \
		(top)->va_fileid = (fromp)->va_fileid; \
		(top)->va_size = (fromp)->va_size; \
		(top)->va_blocksize = (fromp)->va_blocksize; \
		(top)->va_atime = (fromp)->va_atime.tv_sec; \
		(top)->va_mtime = (fromp)->va_mtime.tv_sec; \
		(top)->va_ctime = (fromp)->va_ctime.tv_sec; \
		(top)->va_gen = (fromp)->va_gen; \
		(top)->va_flags = (fromp)->va_flags; \
		(top)->va_rdev = (fromp)->va_rdev; \
		(top)->va_bytes = (fromp)->va_bytes; \
		(top)->va_filerev = (fromp)->va_filerev; \
	} while (0)


int
viceroy_root(void *mdp,
	struct ucred *cred, struct proc *p,
/*out*/	struct ofid *VFid)
{
    DECL_NO_IN(ody_root);		/* sets Isize & Osize */
    ALLOC_NO_IN(ody_root);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(inp, ODY_ROOT, cred, p);  

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error)
	*VFid = outp->fid;

    ODY_FREE(inp, ody_root_size);
    return error;
}

int
viceroy_open(void *mdp, struct ofid *fid, int flag,
	struct ucred *cred, struct proc *p,
/*out*/	dev_t *dev, ino_t *inode, int *flags)
{
    int cflag;
    DECL(ody_open);			/* sets Isize & Osize */
    ALLOC(ody_open);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_OPEN, cred, p);
    inp->fid = *fid;
    CNV_OFLAG(cflag, flag);
    inp->flags = cflag;

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*dev =  outp->dev;
	*inode = outp->inode;
	*flags = outp->flags;
    }

    ODY_FREE(inp, ody_open_size);
    return error;
}

int
viceroy_close(void *mdp, struct ofid *fid, int flag,
	struct ucred *cred, struct proc *p)
{
    int cflag;
    DECL_NO_OUT(ody_close);		/* sets Isize & Osize */
    ALLOC_NO_OUT(ody_close);		/* sets inp & outp */

    INIT_IN(&inp->ih, ODY_CLOSE, cred, p);
    inp->fid = *fid;
    CNV_OFLAG(cflag, flag);
    inp->flags = cflag;

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_close_size);
    return error;
}

/*
 * these two calls will not exist!!!  the container file is read/written
 * directly.
 */
void
viceroy_read()
{
}

void
viceroy_write()
{
}

int
viceroy_rdwr(void *mdp, struct ofid *fid, int offset,
	int ioflag,
        struct ucred *cred, struct proc *p,
/*i/o*/ int *count, int *rwflag, caddr_t data)
{
    DECL(ody_rdwr);		/* sets Isize & Osize */
    ody_rdwr_size = OC_MAXMSGSIZE;
    ALLOC(ody_rdwr);		/* sets inp & outp */

    INIT_IN(&inp->ih, ODY_RDWR, cred, p);
    inp->fid = *fid;
    inp->rwflag = *rwflag;	/* XXX should convert this */
    inp->count = *count;
    inp->data = Isize;
    inp->offset = offset;
    CNV_V2VV_RWFLAG(inp->rwflag, *rwflag);
    inp->ioflag = (ioflag & O_NONBLOCK) ? O_IO_NDELAY : 0;
    if (*rwflag == UIO_WRITE)
	bcopy(data, (char *)inp + inp->data, *count);
    else
	Osize += *count;

    Isize += *count;

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	CNV_VV2V_RWFLAG(*rwflag, outp->rwflag);
	*count = outp->count;
	if (*rwflag == UIO_READ)
	    bcopy((char *)outp + outp->data, data, outp->count);
    }

    ODY_FREE(inp, ody_rdwr_size);
    return error;
}

/*
 * this is a bit sad too.  the ioctl's are for the control file, not for
 * normal files.
 */
int
viceroy_ioctl(void *mdp, struct ofid *fid,
	int com, int flag, caddr_t data,
	struct ucred *cred, struct proc *p)
{
    DECL(ody_ioctl);			/* sets Isize & Osize */
    struct OdyPioctlData *iap = (struct OdyPioctlData *)data;
    int tmp;

    ody_ioctl_size = OC_MAXMSGSIZE;
    ALLOC(ody_ioctl);			/* sets inp & outp */

    INIT_IN(&inp->ih, ODY_IOCTL, cred, p);
    inp->fid = *fid;

    /* command was mutated by increasing its size field to reflect the  
     * path and follow args. we need to subtract that out before sending
     * the command to viceroy.
     */
    inp->cmd = (com & ~(IOCPARM_MASK << 16));
    tmp = ((com >> 16) & IOCPARM_MASK) - sizeof (char *) - sizeof (int);
    inp->cmd |= (tmp & IOCPARM_MASK) <<	16;

    inp->rwflag = flag;
    inp->len = iap->vi.in_size;
    inp->data = (char *)(sizeof (struct ody_ioctl_in));

    error = copyin(iap->vi.in, (char*)inp + (int)inp->data, 
		   iap->vi.in_size);
    if (error) {
	ODY_FREE(inp, ody_ioctl_size);
	return(error);
    }

    Osize = OC_MAXMSGSIZE;
    error = odycall(mdp, Isize + iap->vi.in_size, &Osize, (char *)inp);

	/* copy out the out buffer. */
    if (!error) {
	if (outp->len > iap->vi.out_size) {
	    error = EINVAL;
	} else {
	    error = copyout((char *)outp + (int)outp->data, 
			    iap->vi.out, iap->vi.out_size);
	}
    }

    ODY_FREE(inp, ody_ioctl_size);
    return error;
}

int
viceroy_getattr(void *mdp, struct ofid *fid,
	struct ucred *cred, struct proc *p,
/*out*/	struct vattr *vap)
{
    DECL(ody_getattr);			/* sets Isize & Osize */
    ALLOC(ody_getattr);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_GETATTR, cred, p);
    inp->fid = *fid;

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	CNV_VV2V_ATTR(vap, &outp->attr);
    }

    ODY_FREE(inp, ody_getattr_size);
    return error;
}

int
viceroy_setattr(void *mdp, struct ofid *fid, struct vattr *vap,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_setattr);		/* sets Isize & Osize */
    ALLOC_NO_OUT(ody_setattr);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_SETATTR, cred, p);
    inp->fid = *fid;
    CNV_V2VV_ATTR(&inp->attr, vap);

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_setattr_size);
    return error;
}

int
viceroy_access(void *mdp, struct ofid *fid, int mode,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_access);		/* sets Isize & Osize */
    ALLOC_NO_OUT(ody_access);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_ACCESS, cred, p);
    inp->fid = *fid;
    CNV_MODE(inp->flags, mode);

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_access_size);
    return error;
}

int
viceroy_readlink(void *mdp, struct ofid *fid,
	struct ucred *cred, struct proc *p,
/*out*/	char **str, int *len)
{
    DECL(ody_readlink);			/* sets Isize & Osize */
    ody_readlink_size += ODY_MAXPATHLEN;
    ALLOC(ody_readlink);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_READLINK, cred, p);
    inp->fid = *fid;

    Osize += ODY_MAXPATHLEN;
    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	    ODY_ALLOC(*str, char *, outp->count);
	    *len = outp->count;
	    bcopy((char *)outp + (int)outp->data, *str, *len);
    }

    ODY_FREE(inp, ody_readlink_size);
    return error;
}

int
viceroy_fsync(void *mdp, struct ofid *fid,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_fsync);		/* sets Isize & Osize */
    ALLOC_NO_OUT(ody_fsync);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_FSYNC, cred, p);
    inp->fid = *fid;

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_fsync_size);
    return error;
}

int
viceroy_lookup(void *mdp, struct ofid *fid,
    	const char *nm, int len,
	struct ucred *cred, struct proc *p,
/*out*/	struct ofid *VFid, int *vtype)
{
    DECL(ody_lookup);			/* sets Isize & Osize */
    ody_lookup_size += len + 1;
    ALLOC(ody_lookup);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_LOOKUP, cred, p);
    inp->fid = *fid;

    inp->name = Isize;
    STRCPY(name, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*VFid = outp->fid;
	*vtype = outp->vtype;
    }

    ODY_FREE(inp, ody_lookup_size);
    return error;
}

int
viceroy_create(void *mdp, struct ofid *fid,
    	const char *nm, int len, int exclusive, int mode, struct vattr *va,
	struct ucred *cred, struct proc *p,
/*out*/	struct ofid *VFid, struct vattr *attr)
{
    DECL(ody_create);			/* sets Isize & Osize */
    ody_create_size += len + 1;
    ALLOC(ody_create);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_CREATE, cred, p);
    inp->fid = *fid;
    inp->excl = exclusive ? O_O_EXCL : 0;
#ifdef	NetBSD1_3
    inp->mode = mode<<6;
#else
    inp->mode = mode;
#endif
    CNV_V2VV_ATTR(&inp->attr, va);

    inp->name = Isize;
    STRCPY(name, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*VFid = outp->fid;
	CNV_VV2V_ATTR(attr, &outp->attr);
    }

    ODY_FREE(inp, ody_create_size);
    return error;
}

int
viceroy_remove(void *mdp, struct ofid *fid,
        const char *nm, int len,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_remove);		/* sets Isize & Osize */
    ody_remove_size += len + 1;
    ALLOC_NO_OUT(ody_remove);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_REMOVE, cred, p);
    inp->fid = *fid;

    inp->name = Isize;
    STRCPY(name, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_remove_size);
    return error;
}

int
viceroy_link(void *mdp, struct ofid *fid, struct ofid *tfid,
        const char *nm, int len,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_link);		/* sets Isize & Osize */
    ody_link_size += len + 1;
    ALLOC_NO_OUT(ody_link);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_LINK, cred, p);
    inp->sourceFid = *fid;
    inp->destFid = *tfid;

    inp->tname = Isize;
    STRCPY(tname, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_link_size);
    return error;
}

int
viceroy_rename(void *mdp, struct ofid *fid, struct ofid *tfid,
        const char *nm, int len, const char *tnm, int tlen,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_rename);		/* sets Isize & Osize */
    ody_rename_size += len + 1 + tlen + 1;
    ALLOC_NO_OUT(ody_rename);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_RENAME, cred, p);
    inp->sourceFid = *fid;
    inp->destFid = *tfid;

    inp->srcname = Isize;
    STRCPY(srcname, nm, len);		/* increments Isize */

    inp->destname = Isize;
    STRCPY(destname, tnm, tlen);	/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_rename_size);
    return error;
}

int
viceroy_mkdir(void *mdp, struct ofid *fid,
    	const char *nm, int len, struct vattr *va,
	struct ucred *cred, struct proc *p,
/*out*/	struct ofid *VFid, struct vattr *ova)
{
    DECL(ody_mkdir);			/* sets Isize & Osize */
    ody_mkdir_size += len + 1;
    ALLOC(ody_mkdir);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_MKDIR, cred, p);
    inp->fid = *fid;
    CNV_V2VV_ATTR(&inp->attr, va);

    inp->name = Isize;
    STRCPY(name, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*VFid = outp->fid;
	CNV_VV2V_ATTR(ova, &outp->attr);
    }

    ODY_FREE(inp, ody_mkdir_size);
    return error;
}

int
viceroy_rmdir(void *mdp, struct ofid *fid,
    	const char *nm, int len,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_rmdir);		/* sets Isize & Osize */
    ody_rmdir_size += len + 1;
    ALLOC_NO_OUT(ody_rmdir);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_RMDIR, cred, p);
    inp->fid = *fid;

    inp->name = Isize;
    STRCPY(name, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_rmdir_size);
    return error;
}

int
viceroy_symlink(void *mdp, struct ofid *fid,
        const char *lnm, int llen, const char *nm, int len, struct vattr *va,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_symlink);		/* sets Isize & Osize */
    ody_symlink_size += llen + 1 + len + 1;
    ALLOC_NO_OUT(ody_symlink);		/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_SYMLINK, cred, p);
    inp->fid = *fid;
    CNV_V2VV_ATTR(&inp->attr, va);

    inp->srcname = Isize;
    STRCPY(srcname, lnm, llen);		/* increments Isize */

    inp->tname = Isize;
    STRCPY(tname, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_symlink_size);
    return error;
}

int
viceroy_readdir(void *mdp, struct ofid *fid,
    	int count, int offset,
	struct ucred *cred, struct proc *p,
/*out*/	char *buffer, int *len)
{
    DECL(ody_readdir);			/* sets Isize & Osize */
    ody_readdir_size = OC_MAXMSGSIZE;
    ALLOC(ody_readdir);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_READDIR, cred, p);
    inp->fid = *fid;
    inp->count = count;
    inp->offset = offset;

    Osize = OC_MAXMSGSIZE;
    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	bcopy((char *)outp + (int)outp->data, buffer, outp->size);
	*len = outp->size;
    }

    ODY_FREE(inp, ody_readdir_size);
    return error;
}

#ifdef SETS

#include "kernel/sets/common/sets_internal.h"

int
viceroy_expand(void *mdp, struct ofid *fid,
    	const char *nm, int len,
	struct ucred *cred, struct proc *p,
/*out*/	struct ofid *VFid, int *vtype, int *cursor)
{
    DECL(ody_expand);			/* sets Isize & Osize */
    ody_expand_size += len + 1;
    ALLOC(ody_expand);			/* sets inp & outp */

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_EXPAND, cred, p);
    inp->fid = *fid;

    inp->name = Isize;
    STRCPY(name, nm, len);		/* increments Isize */

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*VFid = outp->fid;
	*vtype = outp->vtype;
	*cursor = outp->cursor;
    }

    ODY_FREE(inp, ody_expand_size);
    return error;
}

int
viceroy_cursexpand(void *mdp, struct ofid *fid,
	struct ucred *cred, struct proc *p,
/*out*/ char **buffer, int *retsz)
{
    DECL(ody_cursexpand);
    ody_cursexpand_size = OC_MAXDATASIZE;
    ALLOC(ody_cursexpand);

    /* send the open to viceroy. */
    INIT_IN(&inp->ih, ODY_CURSEXPAND, cred, p);
    inp->fid = *fid;
    inp->bufsz = OC_MAXDATASIZE;

    Osize += OC_MAXDATASIZE;

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	SETS_ALLOC((*buffer), char *, outp->retsz);
	bcopy((char *)outp + outp->data, *buffer, outp->retsz);
	*retsz = outp->retsz;
    } else {
	if (error != 0xe0f)
	    printf("odycall(ODY_CURSEXPAND) error %d\n", error);
	*retsz = 0;
	*buffer = NULL;
    }

    ODY_FREE(inp, ody_cursexpand_size);
    return error;
}

int
viceroy_prefetch(void *mdp, struct ofid *fid, int flag,
	struct ucred *cred, struct proc *p,
/*out*/ dev_t *dev, ino_t *ino, int *flags)
{
    DECL(ody_prefetch);
    ALLOC(ody_prefetch);

    INIT_IN(&inp->ih, ODY_PREFETCH, cred, p);
    inp->fid = *fid;
    CNV_OFLAG(inp->flags, flag);

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*dev = outp->dev;
	*ino = outp->inode;
	*flags = outp->flags;
    }

    ODY_FREE(inp, ody_prefetch_size);
    return (error);
}
#endif

int
viceroy_fhtovp(void *mdp, struct ofid *fid,
	struct ucred *cred, struct proc *p,
/*out*/	struct ofid *VFid, int *vtype)
{
    DECL(ody_vget);			/* sets Isize & Osize */
    ALLOC(ody_vget);			/* sets inp & outp */

    /* Send the open to Viceroy. */
    INIT_IN(&inp->ih, ODY_VGET, cred, p);
    inp->fid = *fid;

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	*VFid = outp->fid;
	*vtype = outp->vtype;
    }

    ODY_FREE(inp, ody_vget_size);
    return error;
}

int
viceroy_request(void *mdp, struct ofid *fid, u_int rsrc, u_int gs,
	u_int type, u_int ts, int low, int high,
	struct ucred *cred, struct proc *p,
/*out*/ int *cancelled, int *value, int *result)
{
    DECL(ody_request);
    ALLOC(ody_request);

    /* Send the open to Viceroy. */
    INIT_IN(&inp->ih, ODY_REQUEST, cred, p);
    inp->fid = *fid;
    inp->rsrc = rsrc;
    inp->gs = gs;
    inp->type = type;
    inp->ts = ts;
    inp->low = low;
    inp->high = high;

    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error || error == ODY_ENOTINWINDOW) {
	*cancelled = outp->cancelled;
	*value = outp->value;
	*result = outp->result;

	ODYDEBUG(ODY_IOCTL, myprintf(("request returns %d  (%d %d %d)\n",
				      error, *cancelled,
				      *value, *result)); );
    }

    ODY_FREE(inp, ody_request_size);
    return (error);
}

int
viceroy_cancel(void *mdp, int reqid,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_cancel);
    ALLOC_NO_OUT(ody_cancel);

    /* Send the open to Viceroy. */
    INIT_IN(&inp->ih, ODY_CANCEL, cred, p);
    inp->reqid = reqid;

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_cancel_size);
    return (error);
}

int
viceroy_cancpid(void *mdp,
	struct ucred *cred, struct proc *p)
{
    DECL_NO_OUT(ody_cancpid);
    ALLOC_NO_OUT(ody_cancpid);

    INIT_IN(&inp->ih, ODY_CANCPID, cred, p);

    error = odycall(mdp, Isize, &Osize, (char *)inp);

    ODY_FREE(inp, ody_cancpid_size);
    return (error);
}

int
viceroy_tsop(void *mdp, struct ofid *fid, u_int gs,
	u_int type, u_int ts, u_int op,
	size_t argsz, caddr_t src_data,
	struct ucred *cred, struct proc *p,
/*out*/ size_t *retsz, caddr_t dest_data)
{
    DECL(ody_tsop);
    ody_tsop_size += (argsz > *retsz) ? argsz : *retsz;
    ALLOC(ody_tsop);

    INIT_IN(&inp->ih, ODY_TSOP, cred, p);
    inp->fid = *fid;
    inp->gs = gs;
    inp->type = type;
    inp->ts = ts;
    inp->op = op;
    inp->argsz = argsz;
    inp->retsz = *retsz;
    inp->data = Isize;
    if (argsz) {
	if ((error = copyin(src_data, (char *)inp + (int)(inp->data), argsz)))
	    return (error);
    }
    Isize += argsz;
    Osize += *retsz;

    ODYDEBUG(ODY_IOCTL, 
	     printf("ody_tsop: (%lx.%lx.%lx.%lx) op %d, arg %d, ret %d\n",
		    fid->ofid_ttype, fid->ofid_tome,
		    fid->ofid_vnum, fid->ofid_uniq,
		    op, argsz, *retsz););
    
    error = odycall(mdp, Isize, &Osize, (char *)inp);
    if (!error) {
	if (outp->retsz > *retsz)
	    panic("ody_tsop: retbuf overgrowth");
	*retsz = outp->retsz;
	if (copyout((char *)outp + (int)outp->data, dest_data, *retsz))
	    panic("ody_tsop: copyout return buffer failed");
    }

    ODY_FREE(inp, ody_tsop_size);

    ODYDEBUG(ODY_IOCTL, printf("ody_tsop returns %d (retsz %d)\n",
			       error, *retsz););

    return (error);
}
