#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1990 Carnegie-Mellon University
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

#include <sys/param.h>
#include <sys/malloc.h>
#include <sys/errno.h>
#include <sys/acct.h>
#include <sys/filedesc.h>
#include <sys/uio.h>
#include <sys/namei.h>
#include <sys/ioctl.h>
#include <sys/ioccom.h>
#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/select.h>
#ifdef SETS
#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_domain.h"
#endif
#ifdef NetBSD1_3
#include <sys/poll.h>
#include <sys/user.h>
#include <miscfs/genfs/genfs.h>
#endif
#ifdef __FreeBSD__ /* XXX gruesome hack */
#include <sys/vnode.h>
#include "specdev.h"
#else
#include <miscfs/specfs/specdev.h>
#endif

#include <vm/vm.h>
#ifdef	__FreeBSD__
#include <vm/vm_object.h>
#include <vm/vm_extern.h>
#endif


#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#include "kernel/vfs/includes/ody_vnodeops.h"
#include "kernel/vfs/includes/ody_viceroy.h"
#include "kernel/vfs/includes/ody_opstats.h"
#include "kernel/vfs/includes/ody_subr.h"
#include "kernel/vfs/includes/odyapi.h"
#include "kernel/vfs/includes/odync.h"
#include "kernel/vfs/includes/ody_vfsops.h"
#include "kernel/vfs/includes/ody_sys.h"

/* 
 * These flags select various performance enhancements.
 */
int ody_intercept_rdwr = 1;    /* Set to handle read/write in the kernel */
int ody_attr_cache  = 1;       /* Set to cache attributes in the kernel */
int ody_symlink_cache = 1;     /* Set to cache symbolic link information */
int ody_access_cache = 1;      /* Set to handle some access checks directly */

/* structure to keep track of vfs calls */

struct ody_op_stats ody_vnodeopstats[ODY_VNODEOPS_SIZE];

#define MARK_ENTRY(op) (ody_vnodeopstats[op].entries++)
#define MARK_INT_SAT(op) (ody_vnodeopstats[op].sat_intrn++)
#define MARK_INT_FAIL(op) (ody_vnodeopstats[op].unsat_intrn++)
#define MARK_INT_GEN(op) (ody_vnodeopstats[op].gen_intrn++)

/* What we are delaying for in printf */
int ody_printf_delay = 0;  /* in microseconds */
int ody_vnop_print_entry = 0;
static int ody_lockdebug = 0;

/* Definition of the vfs operation vector */

/*
 * Some NetBSD details:
 * 
 *   ody_start is called at the end of the mount syscall.
 *   ody_init is called at boot time.
 */

#ifdef __GNUC__
#define ENTRY    \
   if(ody_vnop_print_entry) myprintf(("Entered %s\n",__FUNCTION__))
#else
#define ENTRY
#endif 

/* Definition of the vnode operation vector */

struct vnodeopv_entry_desc ody_vnodeop_entries[] = {
    { &vop_default_desc, ody_vop_error },
    { &vop_lookup_desc, ody_lookup },           /* lookup */
    { &vop_create_desc, ody_create },		/* create */
    { &vop_mknod_desc, ody_vop_error },	/* mknod */
    { &vop_open_desc, ody_open },		/* open */
    { &vop_close_desc, ody_close },		/* close */
    { &vop_access_desc, ody_access },		/* access */
    { &vop_getattr_desc, ody_getattr },		/* getattr */
    { &vop_setattr_desc, ody_setattr },		/* setattr */
    { &vop_read_desc, ody_read },		/* read */
    { &vop_write_desc, ody_write },		/* write */
    { &vop_ioctl_desc, ody_ioctl },		/* ioctl */
#ifdef NetBSD1_3
    { &vop_poll_desc, ody_poll },	/* poll */
#else
    { &vop_select_desc, ody_select },	/* select */
#endif
    { &vop_mmap_desc, ody_vop_error },	/* mmap */
    { &vop_fsync_desc, ody_fsync },		/* fsync */
#ifdef NetBSD1_3
    { &vop_seek_desc, genfs_seek },	/* seek */
#else
    { &vop_seek_desc, ody_vop_error },	/* seek */
#endif
    { &vop_remove_desc, ody_remove },		/* remove */
    { &vop_link_desc, ody_link },		/* link */
    { &vop_rename_desc, ody_rename },		/* rename */
    { &vop_mkdir_desc, ody_mkdir },		/* mkdir */
    { &vop_rmdir_desc, ody_rmdir },		/* rmdir */
    { &vop_symlink_desc, ody_symlink },		/* symlink */
    { &vop_readdir_desc, ody_readdir },		/* readdir */
    { &vop_readlink_desc, ody_readlink },	/* readlink */
    { &vop_abortop_desc, ody_abortop },	/* abortop */
    { &vop_inactive_desc, ody_inactive },	/* inactive */
    { &vop_reclaim_desc, ody_reclaim },	/* reclaim */
    { &vop_lock_desc, ody_lock },	/* lock */
    { &vop_unlock_desc, ody_unlock },	/* unlock */
    { &vop_bmap_desc, ody_bmap },		/* bmap */
    { &vop_strategy_desc, ody_strategy },	/* strategy */
    { &vop_print_desc, ody_vop_error },	/* print */
    { &vop_islocked_desc, ody_islocked },	/* islocked */
    { &vop_pathconf_desc, ody_vop_error },	/* pathconf */
    { &vop_advlock_desc, ody_vop_nop },	/* advlock */
    { &vop_blkatoff_desc, ody_vop_error },	/* blkatoff */
    { &vop_valloc_desc, ody_vop_error },	/* valloc */
    { &vop_vfree_desc, ody_vop_error },	/* vfree */
    { &vop_truncate_desc, ody_vop_error },	/* truncate */
    { &vop_update_desc, ody_vop_error },	/* update */
    { &vop_bwrite_desc, ody_vop_error },	/* bwrite */
#ifdef __NetBSD__
    { &vop_lease_desc, ody_vop_nop },          /* lease */
#endif
#ifdef	__FreeBSD__
    { &vop_getpages_desc, ody_fbsd_vnotsup }, /* pager intf.*/
    { &vop_putpages_desc, ody_fbsd_vnotsup }, /* pager intf.*/
#endif
    { (struct vnodeop_desc*)NULL, (int(*)(void *))NULL }
};

#ifdef __NetBSD__
struct vnodeopv_desc ody_vnodeop_opv_desc = 
        { &ody_vnodeop_p, ody_vnodeop_entries };
#endif

#ifdef __FreeBSD__
static struct vnodeopv_desc ody_vnodeop_opv_desc = 
		{ &ody_vnodeop_p, ody_vnodeop_entries };

#include <sys/kernel.h>
VNODEOP_SET(ody_vnodeop_opv_desc);

int
ody_fbsd_vnotsup(ap)
    void *ap;
{
    return(EOPNOTSUPP);
}

#endif

/* Definitions of NetBSD vnodeop interfaces */

/* A generic panic: we were called with something we didn't define yet */
int
ody_vop_error(void *anon) {
    struct vnodeop_desc **desc = (struct vnodeop_desc **)anon;

    myprintf(("Vnode operation %s called, but not defined\n",
	      (*desc)->vdesc_name));
    panic("ody_vop_error");
    return 0;
}

/* A generic do-nothing.  For lease_check, advlock */
int
ody_vop_nop(void *anon) {
    struct vnodeop_desc **desc = (struct vnodeop_desc **)anon;

    if (odydebug) {
	myprintf(("Vnode operation %s called, but unsupported\n",
		  (*desc)->vdesc_name));
    }
    return (0);
}

int
ody_vnodeopstats_init()
{
	register int i;
	
	for(i=0;i<ODY_VNODEOPS_SIZE;i++) {
		ody_vnodeopstats[i].opcode = i;
		ody_vnodeopstats[i].entries = 0;
		ody_vnodeopstats[i].sat_intrn = 0;
		ody_vnodeopstats[i].unsat_intrn = 0;
		ody_vnodeopstats[i].gen_intrn = 0;
	}
	
	return 0;
}
		
/*ARGSUSED*/
/* 
 * ody_open calls Venus to return the device, inode pair of the cache
 * file holding the data. Using iget, ody_open finds the vnode of the
 * cache file, and then opens it.
 */

int
ody_open(v)
    void *v;
{
/* true args */
    struct vop_open_args *ap = v;
    register struct vnode **vpp = &(ap->a_vp);
    struct onode *op = VTOO(*vpp);
    int flag = ap->a_mode;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;
/* locals */
    int error;
    struct vnode *vp;
    int uncacheable;
    dev_t dev;
    ino_t inode;
    int   flags;
    
    MARK_ENTRY(ODY_OPEN_STATS);
    
    /* Check for open of control file. */
    if (IS_ODY_CTL_VP(*vpp)) {
	/* XXX */
	/* if (WRITEABLE(flag)) */ 
	if (flag & (FWRITE | O_TRUNC | O_CREAT | O_EXCL)) {
	    MARK_INT_FAIL(ODY_OPEN_STATS);
	    return(EACCES);
	}
	MARK_INT_SAT(ODY_OPEN_STATS);
	return(0);
    }

    error = viceroy_open(vtomi((*vpp)), &op->o_fid, flag, cred, p,
			 &dev, &inode, &flags);

    if (error)
	return (error);
    
    uncacheable = flags & ON_UNCACHEABLE;
    if (uncacheable) {
	ODYDEBUG(ODY_OPEN,myprintf(("open: uncacheable file, result %d\n",
				    error)););
    } else {
	ODYDEBUG(ODY_OPEN,
		 myprintf(("open: dev %d inode %d result %d\n",
			   dev, inode, error)); );
    }
    
    /* Check state of uncacheable bit */
    if (op->o_ocount > 0) {
	if (uncacheable != (op->o_flags & ON_UNCACHEABLE))
	{
	    panic("Opened file (%lx.%lx.%lx.%lx): UNCACHABLE clash",
		  op->o_fid.ofid_ttype, op->o_fid.ofid_tome,
		  op->o_fid.ofid_vnum, op->o_fid.ofid_uniq);
	}
    } else {
	if (uncacheable) {
	    op->o_flags |= ON_UNCACHEABLE;
	} else {
	    op->o_flags &= ~ON_UNCACHEABLE;
	}
    }
	    
    /* If we aren't uncacheable, do the grab_vnode, otherwise don't */
    if (!uncacheable) { 
        if (dev == 0 && inode == 0)
	    panic("bogus container file info!");
	/* Translate the <device, inode> pair for the cache file into
	   an inode pointer. */
	error = ody_grab_vnode(dev, inode, &vp);
	if (error)
	    return (error);

	/* 
	 * We get the vnode back locked in both Mach and NetBSD.
	 * Needs unlocked 
	 */
	VOP_UNLOCK(vp);
	/* Keep a reference until the close comes in. */
	vref(*vpp);                

	/* Save the vnode pointer for the cache file. */
	if (op->o_ovp == NULL) {
	    op->o_ovp = vp;
	}
	else {
	    if (op->o_ovp != vp)
		panic("ody_open:  op->o_ovp != ITOV(ip)");
	}
	op->o_ocount++;
	
	/* Flush the attribute cached if writing the file. */
	if (flag & FWRITE) {
	    op->o_owrite++;
	    op->o_flags &= ~ON_VATTR;
	}

	/* Save the <device, inode> pair for the cache file to speed
	   up subsequent page_read's. */
	op->o_device = dev;
	op->o_inode = inode;
	
	/* Open the cache file. */
	error = VOP_OPEN(op->o_ovp, flag, cred, p); 
#ifdef	__FreeBSD__
	if (error) {
	    printf("ody_open: VOP_OPEN on container failed %d\n", error);
	    return (error);
	}
	if (vp->v_type == VREG) {
	    error = vfs_object_create(vp, p, cred, 1);
	    if (error != 0) {
		printf("ody_open: vfs_object_create() returns %d\n", error);
		vput(vp);
	    }
	}
#endif
    } else {
        /* Keep a reference on the opened file until the close comes in. */
        vref(*vpp);                
        op->o_ocount++;
        
        /* Flush the attribute cached if writing the file. */
        if (flag & FWRITE) {
            op->o_owrite++;
            op->o_flags &= ~ON_VATTR;
        }
    }

    return(error);
}

/*ARGSUSED*/
/*
 * Close the cache file used for I/O and notify Venus.
 */
int
ody_close(v)
    void *v;
{
/* true args */
    struct vop_close_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    int flag = ap->a_fflag;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;
/* locals */
    int error;
    
    MARK_ENTRY(ODY_CLOSE_STATS);

    /* Check for close of control file. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_SAT(ODY_CLOSE_STATS);
	return(0);
    }
    
    if (IS_UNMOUNTING(op)) {
	if (op->o_ovp) {
	    printf("ody_close: destroying container ref %d, ufs vp %p of vp %p/op %p\n",
		    vp->v_usecount, op->o_ovp, vp, op);
	    vgone(op->o_ovp);
	} else {
	    printf("ody_close: NO container vp %p/op %p\n", vp, op);
	}
	return ENODEV;
    } else if (!IS_UNCACHEABLE(op)){
	VOP_CLOSE(op->o_ovp, flag, cred, p); /* Do errors matter here? */
	vrele(op->o_ovp);
    }

    if (--op->o_ocount == 0)
	op->o_ovp = NULL;
    
    if (flag & FWRITE)                    /* file was opened for write */
	--op->o_owrite;
    
    error = viceroy_close(vtomi(vp), &op->o_fid, flag, cred, p);
    vrele(OTOV(op));

    ODYDEBUG(ODY_CLOSE, myprintf(("close: result %d\n",error)); );
    return(error);
}

int
ody_read(v)
    void *v;
{
    struct vop_read_args *ap = v;

    ENTRY;
    return(ody_rdwr(ap->a_vp, ap->a_uio, UIO_READ,
		    ap->a_ioflag, ap->a_cred, ap->a_uio->uio_procp));
}

int
ody_write(v)
    void *v;
{
    struct vop_write_args *ap = v;

    ENTRY;
    return(ody_rdwr(ap->a_vp, ap->a_uio, UIO_WRITE,
		    ap->a_ioflag, ap->a_cred, ap->a_uio->uio_procp));
}

int
ody_rdwr(vp, uiop, rw, ioflag, cred, p)
    struct vnode *vp;
    struct uio *uiop;
    enum uio_rw rw;
    int ioflag;
    struct ucred *cred;
    struct proc *p;
{ 
    struct onode *op = VTOO(vp);
    int error = 0;
    
    MARK_ENTRY(ODY_RDWR_STATS);
    
    ODYDEBUG(ODY_RDWR, myprintf(("ody_rdwr(%d, %p, %d, %qd, %d)\n", rw, 
				 uiop->uio_iov->iov_base, uiop->uio_resid, 
				 uiop->uio_offset, uiop->uio_segflg)); );
    
    /* Check for rdwr of control object. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_RDWR_STATS);
	return(EINVAL);
    }
    
    if (ody_intercept_rdwr && !IS_UNCACHEABLE(op)) {
	/* Redirect the request to UFS. */
	
	/* 
	 * If file is not already open this must be a page
	 * {read,write} request.  Iget the cache file's inode
	 * pointer if we still have its <device, inode> pair.
	 * Otherwise, we must do an internal open to derive the
	 * pair. 
	 */
	struct vnode *cfvp = op->o_ovp;
	int igot_internally = 0;
	int opened_internally = 0;
	if (cfvp == NULL) {
	    /* 
	     * If we're dumping core, do the internal open. Otherwise
	     * venus won't have the correct size of the core when
	     * it's completely written.
	     */
	    if (op->o_inode != 0 && !(p && (p->p_acflag & ACORE))) { 
		igot_internally = 1;
		error = ody_grab_vnode(op->o_device, op->o_inode, &cfvp);
		if (error) {
		    MARK_INT_FAIL(ODY_RDWR_STATS);
		    return(error);
		}
		/* 
		 * We get the vnode back locked in both Mach and
		 * NetBSD.  Needs unlocked 
		 */
		VOP_UNLOCK(cfvp);
	    }
	    else {
		opened_internally = 1;
		MARK_INT_GEN(ODY_OPEN_STATS);
		error = VOP_OPEN(vp, (rw == UIO_READ ? FREAD : FWRITE), 
				 cred, p);
#ifdef	__FreeBSD__
		if (error) {
		    printf("ody_rdwr: VOP_OPEN on container failed %d\n",
			   error);
		    return (error);
		}
		if (vp->v_type == VREG) {
		    error = vfs_object_create(vp, p, cred, 1);
		    if (error != 0) {
			printf("ody_rdwr: vfs_object_create() returns %d\n",
			       error);
			vput(vp);
		    }
		}
#endif
		if (error) {
		    MARK_INT_FAIL(ODY_RDWR_STATS);
		    return(error);
		}
		cfvp = op->o_ovp;
	    }
	}
	
	/* Have UFS handle the call. */
	ODYDEBUG(ODY_RDWR, 
		 myprintf(("indirect rdwr: fid = (%lx.%lx.%lx.%lx), refcnt = %d\n",
			   op->o_fid.ofid_ttype, op->o_fid.ofid_tome,
			   op->o_fid.ofid_vnum, op->o_fid.ofid_uniq,
			   OTOV(op)->v_usecount)););
	if (rw == UIO_READ) {
	    error = VOP_READ(cfvp, uiop, ioflag, cred);
	} else {
	    error = VOP_WRITE(cfvp, uiop, ioflag, cred);
#ifdef	__FreeBSD__
	    /* ufs_write updates the vnode_pager_setsize for the
               vnode/object */
	    {
		struct vattr attr;
	    
		if (VOP_GETATTR(cfvp, &attr, cred, p) == 0) {
		    vnode_pager_setsize(vp, attr.va_size);
#ifdef	__DEBUG_FreeBSD__
		    printf("write: vnode_pager_setsize(%p, %d)\n",
			   vp, attr.va_size);
#endif
		}
	    }
#endif
	}
	
	if (error)
	    MARK_INT_FAIL(ODY_RDWR_STATS);
	else
	    MARK_INT_SAT(ODY_RDWR_STATS);
	
	/* Do an internal close if necessary. */
	if (opened_internally) {
	    MARK_INT_GEN(ODY_CLOSE_STATS);
	    (void)VOP_CLOSE(vp, (rw == UIO_READ ? FREAD : FWRITE), cred, p);
	}
    }
    else {
	/* Read/Write the blocks from/to Venus. */

#if 0
	/* Ignore these: we see them both all the time */
#ifdef DIAGNOSTIC	
	if (ioflag) {
	    if (ioflag & IO_UNIT) 
		printf("ody_rdwr: WARNING: IO_UNIT\n");
	    if (ioflag & IO_NDELAY)
		printf("ody_rdwr: WARNING: IO_NDELAY\n");
	}
#endif
#endif

	while (uiop->uio_resid > 0) {
	    struct iovec *iovp = uiop->uio_iov;
	    unsigned count = iovp->iov_len;
	    int rwflag = rw;
	    unsigned in_count;

	    if (count == 0) {
		uiop->uio_iov++;
		uiop->uio_iovcnt--;
		continue;
	    }
	    if (count > OC_MAXDATASIZE)
		count = OC_MAXDATASIZE;

	    in_count = count;
	    error = viceroy_rdwr(vtomi(vp), &op->o_fid, uiop->uio_offset,
				 ioflag, cred, p,
				 &count, &rwflag, iovp->iov_base);

	    ODYDEBUG(ODY_RDWR, 
		     myprintf(("ody_rdwr(%d, %d, %qd, %d)",
			       rw, in_count, uiop->uio_offset, 
			       uiop->uio_segflg)););
	    ODYDEBUG(ODY_RDWR, 
		     myprintf((" returns (%d, %d)\n", error, 
			       count)); );
	    
	    if (error) break;

	    iovp->iov_base += count;
	    iovp->iov_len -= count;
	    uiop->uio_resid -= count;
	    uiop->uio_offset += count;
	    
	    /* Exit the loop if Venus R/W fewer bytes than we specified. */
	    /* Maybe we should continue if ANY bytes were R/W? -JJK */
	    if (count != in_count)
		break;
	}
    }
    
    /* Invalidate cached attributes if writing. */
    if (rw == UIO_WRITE)
	op->o_flags &= ~ON_VATTR;
    ODYDEBUG(ODY_RDWR,
	     myprintf(("ody_rdwr returns %d\n", error)););
    return(error);
}

/*ARGSUSED*/
int
ody_ioctl(v)
    void *v;
{
/* true args */
    struct vop_ioctl_args *ap = v;
    struct vnode *vp = ap->a_vp;
    int com = ap->a_command;
    caddr_t data = ap->a_data;
    int flag = ap->a_fflag;
    struct ucred *cred = ap->a_cred;
    struct proc  *p = ap->a_p;
/* locals */
    int error;
    struct vnode *tvp;
    struct nameidata *ndp=NULL;
    struct OdyPioctlData *iap = (struct OdyPioctlData *)data;
    int len = IOCPARM_LEN(com);

    /* Don't check for operation on a dying object, for ctlvp it
       shouldn't matter */
	
    /* Must be control object to succeed. */
    if (!IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_IOCTL_STATS);
	ODYDEBUG(ODY_IOCTL, myprintf(("ody_ioctl error: vp != ctlvp")););
	    return (EOPNOTSUPP);
    }

    /* Check if the ioctl corresponds to a first-class odyssey op
       (req, freq, cancel, tsop, ftsop) */
    switch (com) {
    case ODYI_REQ:
	if (len != sizeof(struct ody_req_args)) return EINVAL;
	return ody_req(p, (struct ody_req_args *) data, NULL);
    case ODYI_FREQ:
	if (len != sizeof(struct ody_freq_args)) return EINVAL;
	return ody_freq(p, (struct ody_freq_args *) data, NULL);
    case ODYI_CANCEL:
	if (len != sizeof(struct ody_cancel_args)) return EINVAL;
	return ody_cancel(p, (struct ody_cancel_args *) data, NULL);
    case ODYI_TSOP:
	if (len != sizeof(struct ody_tsop_args)) return EINVAL;
	return ody_tsop(p, (struct ody_tsop_args *) data, NULL);
    case ODYI_FTSOP:
	if (len != sizeof(struct ody_ftsop_args)) return EINVAL;
	return ody_ftsop(p, (struct ody_ftsop_args *) data, NULL);
    }

    /* If we dropped through to here, it's a "regular" ioctl */

    MARK_ENTRY(ODY_IOCTL_STATS);
    
    ODYDEBUG(ODY_IOCTL, myprintf(("in ody_ioctl on %s\n", iap->path)););
	
    /* Look up the pathname. */
    
    /* Should we use the name cache here? It would get it from
       lookupname sooner or later anyway, right? */
    
    NDINIT(ndp, LOOKUP, iap->follow ? FOLLOW : NOFOLLOW, UIO_USERSPACE,
	   (caddr_t)iap->path, p);
    error = namei(ndp);
    tvp = ndp->ni_vp;

    if (error) {
	MARK_INT_FAIL(ODY_IOCTL_STATS);
	ODYDEBUG(ODY_IOCTL, myprintf(("ody_ioctl error: lookup returns %d\n",
				      error)););
	return(error);
    }
    
    /* 
     * Make sure this is an ody style onode, but it may be a
     * different vfsp 
     */
    /* XXX: this totally violates the comment about vtagtype in vnode.h */
    if (tvp->v_tag != VT_ODYSSEY) {
	vrele(tvp);
	MARK_INT_FAIL(ODY_IOCTL_STATS);
	ODYDEBUG(ODY_IOCTL, 
		 myprintf(("ody_ioctl error: %s not a coda object\n", 
			   iap->path)););
	return(EINVAL);
    }
    
    if (iap->vi.in_size > OC_MAXMSGSIZE) {
	vrele(tvp);
	return(EINVAL);
    }
    error = viceroy_ioctl(vtomi(tvp), &((VTOO(tvp))->o_fid), com, flag, data,
			  cred, p);
    
    if (error)
	MARK_INT_FAIL(ODY_IOCTL_STATS);
    else
	ODYDEBUG(ODY_IOCTL, myprintf(("Ioctl returns %d \n", error)); );
    
    vrele(tvp);
    return(error);
}

#ifdef NetBSD1_3
/*ARGSUSED*/
int
ody_poll(v)
    void *v;
{
/* true args */
#if 0
    struct vop_poll_args *ap = v;
    struct vnode *vp = ap->a_vp;
    int events = ap->a_events;
    struct proc *p = ap->a_p;
#endif

	MARK_ENTRY(ODY_POLL_STATS);

	MARK_INT_FAIL(ODY_POLL_STATS);
	return (EOPNOTSUPP);
}
#else
/*ARGSUSED*/
int
ody_select(v)
    void *v;
{
/* true args */
    struct vop_select_args *ap = v;
    struct vnode *vp = ap->a_vp;
    int which = ap->a_which;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;

	MARK_ENTRY(ODY_SELECT_STATS);

	MARK_INT_FAIL(ODY_SELECT_STATS);
	return (EOPNOTSUPP);
}
#endif

/*
 * To reduce the cost of a user-level venus, we cache attributes in
 * the kernel.  Each cnode has storage allocated for an attribute. If
 * c_vattr is valid, return a reference to it. Otherwise, get the
 * attributes from venus and store them in the cnode.  There is some
 * question if this method is a security leak. But I think that in
 * order to make this call, the user must have done a lookup and
 * opened the file, and therefore should already have access.  
 */
int
ody_getattr(v)
    void *v;
{
/* true args */
    struct vop_getattr_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    struct vattr *vap = ap->a_vap;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;
/* locals */
    int error;
    
    MARK_ENTRY(ODY_GETATTR_STATS);

#ifdef	__FreeBSD__
    if (IS_UNMOUNTING(op))
	return ENODEV;
#endif
    /* Check for getattr of control object. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_GETATTR_STATS);
	return(ENOENT);
    }
    
    /* Check to see if the attributes have already been cached */
    if (VALID_VATTR(op)) { 
	ODYDEBUG(ODY_GETATTR, 
		 myprintf(("attr cache hit: (%lx.%lx.%lx.%lx)\n",
			   op->o_fid.ofid_ttype,
			   op->o_fid.ofid_tome,
			   op->o_fid.ofid_vnum,
			   op->o_fid.ofid_uniq)););
	ODYDEBUG(ODY_GETATTR, 
		 {
		     if (!(odydebug & ~ODY_GETATTR))
			 
			 ody_print_vattr(&op->o_vattr); 
		 });
	
	*vap = op->o_vattr;
	MARK_INT_SAT(ODY_GETATTR_STATS);
	return(0);
    }

    error = viceroy_getattr(vtomi(vp), &op->o_fid, cred, p, vap);

    if (!error) {
	ODYDEBUG(ODY_GETATTR, 
		 myprintf(("getattr miss (%lx.%lx.%lx.%lx): result %d\n",
			   op->o_fid.ofid_ttype,
			   op->o_fid.ofid_tome,
			   op->o_fid.ofid_vnum,
			   op->o_fid.ofid_uniq,
			   error)); );
	    
	ODYDEBUG(ODY_GETATTR, 
		 { 
		     if (!(odydebug & ~ODY_GETATTR))
			 ody_print_vattr(vap);
		 });

#ifdef	__FreeBSD__
	{
	    int size = vap->va_size;
	    struct vnode *convp = op->o_ovp;
	    if (convp != (struct vnode *)0) {
		vnode_pager_setsize(convp, size);
#ifdef	__DEBUG_FreeBSD__
		printf("getattr: vnode_pager_setsize(%p, %d)\n", convp, size);
#endif
	    }
	}
#endif

	/* If not open for write, store attributes in cnode */   
	if ((op->o_owrite == 0) && (ody_attr_cache)) {  
	    op->o_vattr = *vap;
	    op->o_flags |= ON_VATTR; 
	}
	
    }
    return(error);
}

int
ody_setattr(v)
    void *v;
{
/* true args */
    struct vop_setattr_args *ap = v;
    register struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    register struct vattr *vap = ap->a_vap;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;
/* locals */
    int error;
    
    MARK_ENTRY(ODY_SETATTR_STATS);
    
    /* Check for setattr of control object. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_SETATTR_STATS);
	return(ENOENT);
    }

    if (odydebug & ODYDBGMSK(ODY_SETATTR)) {
	ody_print_vattr(vap);
    }
    error = viceroy_setattr(vtomi(vp), &op->o_fid, vap, cred, p);
    
    if (!error)
	op->o_flags &= ~ON_VATTR;
    
#ifdef	__FreeBSD__
    {
	int size = vap->va_size;
    	struct vnode *convp = op->o_ovp;
	if (size != VNOVAL && convp != (struct vnode *)0) {
	    vnode_pager_setsize(convp, size);
#ifdef	__DEBUG_FreeBSD__
	    printf("setattr: vnode_pager_setsize(%p, %d)\n", convp, size);
#endif
	}
    }
#endif
    ODYDEBUG(ODY_SETATTR, myprintf(("setattr %d\n", error)); );
    return(error);
}

int
ody_access(v)
    void *v;
{
/* true args */
    struct vop_access_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    int mode = ap->a_mode;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;
/* locals */
    int error;
    
    MARK_ENTRY(ODY_ACCESS_STATS);
    
    /* Check for access of control object.  Only read access is
       allowed on it. */
    if (IS_ODY_CTL_VP(vp)) {
	/* bogus hack - all will be marked as successes */
	MARK_INT_SAT(ODY_ACCESS_STATS);
	return(((mode & VREAD) && !(mode & (VWRITE | VEXEC))) 
	       ? 0 : EACCES);
    }
    
    /*
     * if the file is a directory, and we are checking exec (eg lookup) 
     * access, and the file is in the namecache, then the user must have 
     * lookup access to it.
     */
    if (ody_access_cache) {
	if ((vp->v_type == VDIR) && (mode & VEXEC)) {
	    if (odync_lookup(op, ".", 1, cred)) {
		MARK_INT_SAT(ODY_ACCESS_STATS);
		return(0);                     /* it was in the cache */
	    }
	}
    }

    error = viceroy_access(vtomi(vp), &op->o_fid, mode, cred, p);

    return(error);
}

/*
 * Odyssey abort op, called after namei() when a CREATE/DELETE isn't actually
 * done. If a buffer has been saved in anticipation of a ody_create or
 * a ody_remove, delete it.
 */
/* ARGSUSED */
int
ody_abortop(v)
    void *v;
{
/* true args */
	struct vop_abortop_args /* {
		struct vnode *a_dvp;
		struct componentname *a_cnp;
	} */ *ap = v;

	if ((ap->a_cnp->cn_flags & (HASBUF | SAVESTART)) == HASBUF)
		free(ap->a_cnp->cn_pnbuf, M_NAMEI);
	return (0);
}

int
ody_readlink(v)
    void *v;
{
/* true args */
    struct vop_readlink_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    struct uio *uiop = ap->a_uio;
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_uio->uio_procp;
/* locals */
    int error;
    char *str;
    int len;
    
    MARK_ENTRY(ODY_READLINK_STATS);
    
    /* Check for readlink of control object. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_READLINK_STATS);
	return(ENOENT);
    }
    
    if ((ody_symlink_cache) && (VALID_SYMLINK(op))) { /* symlink was cached */
	uiop->uio_rw = UIO_READ;
	error = uiomove(op->o_symlink, op->o_symlen, uiop);

	if (error)
	    MARK_INT_FAIL(ODY_READLINK_STATS);
	else
	    MARK_INT_SAT(ODY_READLINK_STATS);
	return(error);
    }

    error = viceroy_readlink(vtomi(vp), &op->o_fid, cred, p, &str, &len);
    
    if (!error) {
	uiop->uio_rw = UIO_READ;
	error = uiomove(str, len, uiop);

	if (ody_symlink_cache) {
	    op->o_symlink = str;
	    op->o_symlen = len;
	    op->o_flags |= ON_SYMLINK;
	} else
	    ODY_FREE(str, len);
    }

    ODYDEBUG(ODY_READLINK, myprintf(("in readlink result %d\n",error)););
    return(error);
}

/*ARGSUSED*/
int
ody_fsync(v)
    void *v;
{
/* true args */
    struct vop_fsync_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    struct ucred *cred = ap->a_cred;
    struct proc *p = ap->a_p;
/* locals */
    struct vnode *convp = op->o_ovp;
    int error;
    
    MARK_ENTRY(ODY_FSYNC_STATS);

    /* Check for fsync on an unmounting object */
    /* The NetBSD kernel, in it's infinite wisdom, can try to fsync
     * after an unmount has been initiated.  This is a Bad Thing,
     * which we have to avoid.  Not a legitimate failure for stats.
     */
    if (IS_UNMOUNTING(op)) {
	return(ENODEV);
    }
    
    /* Check for fsync of control object. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_SAT(ODY_FSYNC_STATS);
	return(0);
    }

    if (convp)
	VOP_FSYNC(convp, cred, MNT_WAIT, p);

#ifdef	__FreeBSD__
    /*
     * We see fsyncs with usecount == 1 then usecount == 0.
     * For now we ignore them.
     */
    /*
    if (!vp->v_usecount) {
    	printf("ody_fsync on vnode %p with %d usecount.  o_flags = %x (%x)\n",
		vp, vp->v_usecount, op->o_flags, op->o_flags&ON_PURGING);
    }
    */
#endif

    /*
     * We can expect fsync on any vnode at all if venus is pruging it.
     * Venus can't very well answer the fsync request, now can it?
     * Hopefully, it won't have to, because hopefully, venus preserves
     * the (possibly untrue) invariant that it never purges an open
     * vnode.  Hopefully.
     */
    if (op->o_flags & ON_PURGING) {
	return (0);
    }

#ifdef	__FreeBSD__
    /* needs research */
    return 0;
#endif
    error = viceroy_fsync(vtomi(vp), &op->o_fid, cred, p);

    ODYDEBUG(ODY_FSYNC, myprintf(("in fsync result %d\n",error)); );
    
    return(error);
}

/*ARGSUSED*/
int
ody_inactive(v)
    void *v;
{
    /* XXX - at the moment, inactive doesn't look at cred, and doesn't
       have a proc pointer.  Oops. */
/* true args */
    struct vop_inactive_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    struct ucred *cred __attribute__((unused)) = NULL;
    struct proc *p __attribute__((unused)) = curproc;
    
    /* We don't need to send inactive to venus - DCS */
    MARK_ENTRY(ODY_INACTIVE_STATS);
    
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_SAT(ODY_INACTIVE_STATS);
	ody_unsave(op);
	return 0;
    }
    
    ODYDEBUG(ODY_INACTIVE, 
	     myprintf(("in inactive, %lx.%lx.%lx.%lx. vfsp %p\n",
		       op->o_fid.ofid_ttype,
		       op->o_fid.ofid_tome,
		       op->o_fid.ofid_vnum,
		       op->o_fid.ofid_uniq,
		       vp->v_mount)););
	
#ifdef	__DEBUG_FreeBSD__
    if (vp->v_flag & VXLOCK)
	printf ("Inactive: Vnode is Locked\n");
#endif

    /* If an array has been allocated to hold the symlink, deallocate it */
    if ((ody_symlink_cache) && (VALID_SYMLINK(op))) {
	if (op->o_symlink == NULL)
	    panic("ody_inactive: null symlink pointer in cnode");
	
	ODY_FREE(op->o_symlink, op->o_symlen);
	op->o_flags &= ~ON_SYMLINK;
	op->o_symlen = 0;
    }
    
    /* Remove it from the table so it can't be found. */
    ody_unsave(op);
    if (vtomi(vp) == NULL) {
	myprintf(("Help! vfsp->vfs_data was NULL, but vnode %p wasn't dying\n",
		  vp));
	panic("badness in ody_inactive\n");
    }

    if (IS_UNMOUNTING(op)) {
	if (op->o_ovp != NULL)
	    printf("ody_inactive: op->o_ovp != NULL use %d: vp %p, op %p\n",
	    	   vp->v_usecount, vp, op);
    } else {    
#ifdef DIAGNOSTIC
	if (OTOV(op)->v_usecount) {
	    panic("ody_inactive: nonzero reference count");
	}
	if (op->o_ovp != NULL) {
	    panic("ody_inactive:  op->o_ovp != NULL");
	}
#endif
	vgone(vp);
    }

    MARK_INT_SAT(ODY_INACTIVE_STATS);
    return(0);
}

/*
 * Remote file system operations having to do with directory manipulation.
 */

/* 
 * It appears that in NetBSD, lookup is supposed to return the vnode locked
 */
int
ody_lookup(v)
    void *v;
{
/* true args */
    struct vop_lookup_args *ap = v;
    struct vnode *dvp = ap->a_dvp;
    struct onode *dop = VTOO(dvp);
    struct vnode **vpp = ap->a_vpp;
    /* 
     * It looks as though ap->a_cnp->ni_cnd->cn_nameptr holds the rest
     * of the string to xlate, and that we must try to get at least
     * ap->a_cnp->ni_cnd->cn_namelen of those characters to macth.  I
     * could be wrong. 
     */
    struct componentname  *cnp = ap->a_cnp;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* locals */
    struct onode *op;
    const char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    struct ofid fid;
    int vtype;
    int error = 0;

    MARK_ENTRY(ODY_LOOKUP_STATS);

    ODYDEBUG(ODY_LOOKUP, myprintf(("lookup: %s in %lx.%lx.%lx.%lx\n",
				   nm, dop->o_fid.ofid_ttype,
				   dop->o_fid.ofid_tome,
				   dop->o_fid.ofid_vnum,
				   dop->o_fid.ofid_uniq)););
    
    /* Check for lookup of control object. */
    if (IS_ODY_CTL_NAME(dvp, nm, len)) {
	*vpp = ody_ctlvp;
	vref(*vpp);
	MARK_INT_SAT(ODY_LOOKUP_STATS);
	goto exit;		/* may need to release dad; error already 0 */
    }

    if (len+1 > ODY_MAXNAMLEN) {
	MARK_INT_FAIL(ODY_LOOKUP_STATS);
	ODYDEBUG(ODY_LOOKUP, myprintf(("name too long: lookup, %lx.%lx.%lx.%lx(%s)\n",
				       dop->o_fid.ofid_ttype,
				       dop->o_fid.ofid_tome,
				       dop->o_fid.ofid_vnum,
				       dop->o_fid.ofid_uniq, nm)););
	*vpp = (struct vnode *)0;
	error = EINVAL;
	goto exit;
    }

    /* First try to look the file up in the ody name cache */
    /* lock the parent vnode? */
    
    op = odync_lookup(dop, nm, len, cred);
    if (op) {
	*vpp = OTOV(op);
	vref(*vpp);
	ODYDEBUG(ODY_LOOKUP, 
		 myprintf(("lookup result %d vpp %p\n",error,*vpp)););
    } else {
	
	/* The name wasn't cached, so we need to contact Venus */
	error = viceroy_lookup(vtomi(dvp), &dop->o_fid, nm, len, cred, p,
			       &fid, &vtype);
	
	if (error) {
	    MARK_INT_FAIL(ODY_LOOKUP_STATS);
	    ODYDEBUG(ODY_LOOKUP, 
		     myprintf(("lookup error on %lx.%lx.%lx.%lx (%s) %d\n",
			       dop->o_fid.ofid_ttype,
			       dop->o_fid.ofid_tome,
			       dop->o_fid.ofid_vnum,
			       dop->o_fid.ofid_uniq, nm, error)););
	    *vpp = (struct vnode *)0;
	} else {
	    MARK_INT_SAT(ODY_LOOKUP_STATS);
	    ODYDEBUG(ODY_LOOKUP, 
		     myprintf(("lookup: (%lx.%lx.%lx.%lx) type %o result %d\n",
			       fid.ofid_ttype, fid.ofid_tome,
			       fid.ofid_vnum, fid.ofid_uniq,
			       vtype, error)); );
		
	    op = makeodynode(&fid, dvp->v_mount, vtype);
	    *vpp = OTOV(op);

	    /* enter the new vnode in the Name Cache only if the top bit isn't set */
	    /* And don't enter a new vnode for an invalid one! */
	    if (!(vtype & ODY_NOCACHE))
		odync_enter(VTOO(dvp), nm, len, cred, VTOO(*vpp));
	}
    }

 exit:
    /* 
     * If we are creating, and this was the last name to be looked up,
     * and the error was ENOENT, then there really shouldn't be an
     * error and we can make the leaf NULL and return success.  Since
     * this is supposed to work under Mach as well as NetBSD, we're
     * leaving this fn wrapped.  We also must tell lookup/namei that
     * we need to save the last component of the name.  (Create will
     * have to free the name buffer later...lucky us...)
     */
    if (((cnp->cn_nameiop == CREATE) || (cnp->cn_nameiop == RENAME))
	&& (cnp->cn_flags & ISLASTCN)
	&& (error == ENOENT))
    {
	error = EJUSTRETURN;
	cnp->cn_flags |= SAVENAME;
	*ap->a_vpp = NULL;
    }

    /* 
     * If we are removing, and we are at the last element, and we
     * found it, then we need to keep the name around so that the
     * removal will go ahead as planned.  Unfortunately, this will
     * probably also lock the to-be-removed vnode, which may or may
     * not be a good idea.  I'll have to look at the bits of
     * ody_remove to make sure.  We'll only save the name if we did in
     * fact find the name, otherwise ody_remove won't have a chance
     * to free the pathname.  
     */
    if ((cnp->cn_nameiop == DELETE)
	&& (cnp->cn_flags & ISLASTCN)
	&& !error)
    {
	cnp->cn_flags |= SAVENAME;
    }

    /* 
     * If the lookup went well, we need to (potentially?) unlock the
     * parent, and lock the child.  We are only responsible for
     * checking to see if the parent is supposed to be unlocked before
     * we return.  We must always lock the child (provided there is
     * one, and (the parent isn't locked or it isn't the same as the
     * parent.)  Simple, huh?  We can never leave the parent locked unless
     * we are ISLASTCN
     */
    if (!error || (error == EJUSTRETURN)) {
	if (!(cnp->cn_flags & LOCKPARENT) || !(cnp->cn_flags & ISLASTCN)) {
	    if ((error = VOP_UNLOCK(ap->a_dvp))) {
		return error; 
	    }	    
	    /* 
	     * The parent is unlocked.  As long as there is a child,
	     * lock it without bothering to check anything else. 
	     */
	    if (*ap->a_vpp) {
		if ((error = VOP_LOCK(*ap->a_vpp))) {
		    printf("ody_lookup: ");
		    panic("unlocked parent but couldn't lock child");
		}
	    }
	} else {
	    /* The parent is locked, and may be the same as the child */
	    if (*ap->a_vpp && (*ap->a_vpp != ap->a_dvp)) {
		/* Different, go ahead and lock it. */
		if ((error = VOP_LOCK(*ap->a_vpp))) {
		    printf("ody_lookup: ");
		    panic("unlocked parent but couldn't lock child");
		}
	    }
	}
    } else {
	/* If the lookup failed, we need to ensure that the leaf is NULL */
	/* Don't change any locking? */
	*ap->a_vpp = NULL;
    }
    return(error);
}

/*ARGSUSED*/
int
ody_create(v)
    void *v;
{
/* true args */
    struct vop_create_args *ap = v;
    struct vnode *dvp = ap->a_dvp;
    struct onode *dop = VTOO(dvp);	
    struct vattr *va = ap->a_vap;
    int exclusive = 1;
    int mode = ap->a_vap->va_mode;
    struct vnode **vpp = ap->a_vpp;
    struct componentname  *cnp = ap->a_cnp;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* locals */
    int error;
    struct onode *op;
    const char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    struct ofid fid;
    struct vattr attr;
    
    MARK_ENTRY(ODY_CREATE_STATS);
    
    /* Check for create of control object. */
    if (IS_ODY_CTL_NAME(dvp, nm, len)) {
	*vpp = (struct vnode *)0;
	MARK_INT_FAIL(ODY_CREATE_STATS);
	error = EACCES;
	goto exit;
    }

    error = viceroy_create(vtomi(dvp), &dop->o_fid, nm, len, 1, mode, va,
			   cred, p, &fid, &attr);
    
    if (!error) {
	
	/* If this is an exclusive create, panic if the file already exists. */
	/* Venus should have detected the file and reported EEXIST. */

	if ((exclusive == 1) &&
	    (ody_find(&fid, dvp->v_mount) != NULL))
	    panic("cnode existed for newly created file!");
	
	op = makeodynode(&fid, dvp->v_mount, attr.va_type);
	*vpp = OTOV(op);
	
	/* Update va to reflect the new attributes. */
	(*va) = attr;
	
	/* Update the attribute cache and mark it as valid */
	if (ody_attr_cache) {
	    VTOO(*vpp)->o_vattr = *va;
	    VTOO(*vpp)->o_flags |= ON_VATTR;       
	}
	
	/*
	 * Invalidate the parent's attr cache, the modification time
	 * has changed 
	 */
	VTOO(dvp)->o_flags &= ~ON_VATTR;
	
	/* enter the new vnode in the Name Cache */
	odync_enter(VTOO(dvp), nm, len, cred, VTOO(*vpp));
	
	ODYDEBUG(ODY_CREATE, 
		 myprintf(("create: (%lx.%lx.%lx.%lx), result %d\n",
			   fid.ofid_ttype, fid.ofid_tome,
			   fid.ofid_vnum, fid.ofid_uniq,
			   error)););
    } else {
	*vpp = (struct vnode *)0;
	ODYDEBUG(ODY_CREATE, myprintf(("create error %d\n",error)););
    }

exit:
    /* Locking strategy. */
    /*
     * In NetBSD, all creates must explicitly vput their dvp's.  We'll
     * go ahead and use the LOCKLEAF flag of the cnp argument.
     * However, I'm pretty sure that create must return the leaf
     * locked; so there is a DIAGNOSTIC check to ensure that this is
     * true.
     */
    vput(ap->a_dvp);
    if (!error) {
	if (cnp->cn_flags & LOCKLEAF) {
	    if ((error = VOP_LOCK(*ap->a_vpp))) {
		printf("ody_create: ");
		panic("unlocked parent but couldn't lock child");
	    }
	}
#ifdef DIAGNOSTIC
	else {
	    /* XXX - this was a printf. */
	    panic("ody_create: LOCKLEAF not set!\n");
	}
#endif /* DIAGNOSTIC */
    }
    /* Have to free the previously saved name */
    /* 
     * This condition is stolen from ufs_makeinode.  I have no idea
     * why it's here, but what the hey...
     */
    if ((cnp->cn_flags & SAVESTART) == 0) {
	FREE(cnp->cn_pnbuf, M_NAMEI);
    }
    return(error);
}

int
ody_remove(v)
    void *v;
{
/* true args */
    struct vop_remove_args *ap = v;
    struct vnode *dvp = ap->a_dvp;
    struct onode *op = VTOO(dvp);
    struct componentname  *cnp = ap->a_cnp;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* locals */
    int error;
    const char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    struct onode *tp;
    
    MARK_ENTRY(ODY_REMOVE_STATS);
    
    ODYDEBUG(ODY_REMOVE, myprintf(("remove: %s in %lx.%lx.%lx.%lx\n",
				   nm,
				   op->o_fid.ofid_ttype,
				   op->o_fid.ofid_tome,
				   op->o_fid.ofid_vnum,
				   op->o_fid.ofid_uniq)););
    
    /* Remove the file's entry from the ODY Name Cache */
    /* We're being conservative here, it might be that this person
     * doesn't really have sufficient access to delete the file
     * but we feel zapping the entry won't really hurt anyone -- dcs
     */
    /* I'm gonna go out on a limb here. If a file and a hardlink to it
     * exist, and one is removed, the link count on the other will be
     * off by 1. We could either invalidate the attrs if cached, or
     * fix them. I'll try to fix them. DCS 11/8/94
     */
    tp = odync_lookup(VTOO(dvp), nm, len, cred);
    if (tp) {
	if (VALID_VATTR(tp)) {	/* If attrs are cached */
	    if (tp->o_vattr.va_nlink > 1) {	/* If it's a hard link */
		tp->o_vattr.va_nlink--;
	    }
	}
	
	odync_zapfile(VTOO(dvp), nm, len);
	/* No need to flush it if it doesn't exist! */
    }
    /* Invalidate the parent's attr cache, the modification time has changed */
    VTOO(dvp)->o_flags &= ~ON_VATTR;
    
    /* Check for remove of control object. */
    if (IS_ODY_CTL_NAME(dvp, nm, len)) {
	MARK_INT_FAIL(ODY_REMOVE_STATS);
	error = ENOENT;
	goto exit;
    }

    error = viceroy_remove(vtomi(dvp), &op->o_fid, nm, len, cred, p);

exit:
    ODYDEBUG(ODY_REMOVE, myprintf(("in remove result %d\n",error)); );
     /* 
     * Regardless of what happens, we have to unconditionally drop
     * locks/refs on parent and child.  (I hope).  This is based on
     * what ufs_remove seems to be doing.
     */
    if (ap->a_dvp == ap->a_vp) {
	vrele(ap->a_vp);
    } else {
	vput(ap->a_vp);
    }
    vput(ap->a_dvp);

    if ((cnp->cn_flags & SAVESTART) == 0) {
	FREE(cnp->cn_pnbuf, M_NAMEI);
    }
    return(error);
}

int
ody_link(v)
    void *v;
{
/* true args */
/* true args */
    struct vop_link_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
#ifdef	__NetBSD__
    struct vnode *tdvp = ap->a_dvp;
#elif	defined(__FreeBSD__)
    struct vnode *tdvp = ap->a_tdvp;
#endif
    struct onode *tdop = VTOO(tdvp);
    struct componentname *cnp = ap->a_cnp;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* locals */
    int error;
    const char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    
    MARK_ENTRY(ODY_LINK_STATS);
    
    if (odydebug & ODYDBGMSK(ODY_LINK)) {
	myprintf(("link:   vp fid: (%lx.%lx.%lx.%lx)\n",
		  op->o_fid.ofid_ttype,
		  op->o_fid.ofid_tome,
		  op->o_fid.ofid_vnum,
		  op->o_fid.ofid_uniq));
	myprintf(("link: tdvp fid: (%lx.%lx.%lx.%lx)\n",
		  tdop->o_fid.ofid_ttype,
		  tdop->o_fid.ofid_tome,
		  tdop->o_fid.ofid_vnum,
		  tdop->o_fid.ofid_uniq));
    }
    
    /*
     * According to the ufs_link operation here's the locking situation:
     *     We enter with the thing called "vp" (the directory) locked.
     *     We must unconditionally drop locks on "vp"
     *
     *     We enter with the thing called "dvp" (the linked-to) unlocked,
     *       but ref'd (?)
     *     We seem to need to lock it before calling ody_link, and
     *       unconditionally unlock it after.
     */
    
    if ((ap->a_vp != tdvp) && (error = VOP_LOCK(ap->a_vp))) {
	goto exit;
    }

    
    /* Check for link to/from control object. */
    if (IS_ODY_CTL_NAME(tdvp, nm, len) || IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_LINK_STATS);
	error = EACCES;
	goto exit;
    }

    error = viceroy_link(vtomi(vp), &op->o_fid, &tdop->o_fid, nm, len,
			 cred, p);
    
    /* Invalidate the parent's attr cache, the modification time has changed */
    VTOO(tdvp)->o_flags &= ~ON_VATTR;
    VTOO(vp)->o_flags &= ~ON_VATTR;
    
exit:
    ODYDEBUG(ODY_LINK,	myprintf(("in link result %d\n",error)); );

    if (ap->a_vp != tdvp) {
	VOP_UNLOCK(ap->a_vp);
    }
    vput(tdvp);

    /* Drop the name buffer if we don't need to SAVESTART */
    if ((cnp->cn_flags & SAVESTART) == 0) {
	FREE(cnp->cn_pnbuf, M_NAMEI);
    }
    return(error);
}

int
ody_rename(v)
    void *v;
{
/* true args */
/* true args */
    struct vop_rename_args *ap = v;
    struct vnode *odvp = ap->a_fdvp;
    struct onode *odop = VTOO(odvp);
    struct componentname  *fcnp = ap->a_fcnp;
    struct vnode *ndvp = ap->a_tdvp;
    struct onode *ndop = VTOO(ndvp);
    struct componentname  *tcnp = ap->a_tcnp;
    struct ucred *cred = fcnp->cn_cred;
    struct proc *p = fcnp->cn_proc;
/* true args */
    int error;
    const char *fnm = fcnp->cn_nameptr;
    int flen = fcnp->cn_namelen;
    const char *tnm = tcnp->cn_nameptr;
    int tlen = tcnp->cn_namelen;
    
    MARK_ENTRY(ODY_RENAME_STATS);
    
    /* Hmmm.  The vnodes are already looked up.  Perhaps they are locked?
       This could be Bad. XXX */
#ifdef DIAGNOSTIC
    if ((fcnp->cn_cred != tcnp->cn_cred)
	|| (fcnp->cn_proc != tcnp->cn_proc))
    {
	panic("ody_rename: component names don't agree");
    }
#endif DIAGNOSTIC

    /* Check for rename involving control object. */ 
    if (IS_ODY_CTL_NAME(odvp, fnm, flen) || IS_ODY_CTL_NAME(ndvp, tnm, tlen)) {
	MARK_INT_FAIL(ODY_RENAME_STATS);
	error = EACCES;
	goto exit;
    }
    
    /* Problem with moving directories -- need to flush entry for .. */
    if (odvp != ndvp) {
	struct onode *oop = odync_lookup(VTOO(odvp), fnm, flen, cred);
	if (oop) {
	    struct vnode *ovp = OTOV(oop);
	    if ((ovp) &&
		(ovp->v_type == VDIR)) /* If it's a directory */
		odync_zapfile(VTOO(ovp),"..", 2);
	}
    }
    
    /* Remove the entries for both source and target files */
    odync_zapfile(VTOO(odvp), fnm, flen);
    odync_zapfile(VTOO(ndvp), tnm, tlen);
    
    /* Invalidate the parent's attr cache, the modification time has changed */
    VTOO(odvp)->o_flags &= ~ON_VATTR;
    VTOO(ndvp)->o_flags &= ~ON_VATTR;
    
    if (flen+1 > ODY_MAXNAMLEN) {
	MARK_INT_FAIL(ODY_RENAME_STATS);
	error = EINVAL;
	goto exit;
    }

    if (tlen+1 > ODY_MAXNAMLEN) {
	MARK_INT_FAIL(ODY_RENAME_STATS);
	error = EINVAL;
	goto exit;
    }

    error = viceroy_rename(vtomi(odvp), &odop->o_fid, &ndop->o_fid, fnm, flen,
			   tnm, tlen, cred, p);
    
 exit:
    ODYDEBUG(ODY_RENAME, myprintf(("in rename result %d\n",error)););
    /* XXX - do we need to call cache purge on the moved vnode? */
    cache_purge(ap->a_fvp);

    /* It seems to be incumbent on us to drop locks on all four vnodes */
    /* From-vnodes are not locked, only ref'd.  To-vnodes are locked. */
    
    vrele(ap->a_fvp);
    vrele(ap->a_fdvp);

    if (ap->a_tvp) {
	if (ap->a_tvp == ap->a_tdvp) {
	    vrele(ap->a_tvp);
	} else {
	    vput(ap->a_tvp);
	}
    }

    vput(ap->a_tdvp);
    
    return(error);
}

int
ody_mkdir(v)
    void *v;
{
/* true args */
    struct vop_mkdir_args *ap = v;
    struct vnode *dvp = ap->a_dvp;
    struct onode *dop = VTOO(dvp);	
    struct componentname  *cnp = ap->a_cnp;
    register struct vattr *va = ap->a_vap;
    struct vnode **vpp = ap->a_vpp;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* locals */
    int error;
    const char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    struct onode *op;
    struct ofid fid;
    struct vattr ova;
    
    MARK_ENTRY(ODY_MKDIR_STATS);
    
    /* Check for mkdir of target object. */
    if (IS_ODY_CTL_NAME(dvp, nm, len)) {
	*vpp = (struct vnode *)0;
	MARK_INT_FAIL(ODY_MKDIR_STATS);
	error = EACCES;
	goto exit;
    }
    
    if (len+1 > ODY_MAXNAMLEN) {
	*vpp = (struct vnode *)0;
	MARK_INT_FAIL(ODY_MKDIR_STATS);
	error = EACCES;
        goto exit;
    }

    error = viceroy_mkdir(vtomi(dvp), &dop->o_fid, nm, len, va, cred, p,
			  &fid, &ova);
    
    if (!error) {
	if (ody_find(&fid, dvp->v_mount) != NULL)
	    panic("onode existed for newly created directory!");
	
	
	op = makeodynode(&fid, dvp->v_mount, va->va_type);
	*vpp = OTOV(op);
	
	/* enter the new vnode in the Name Cache */
	odync_enter(VTOO(dvp), nm, len, cred, VTOO(*vpp));
	
	/* as a side effect, enter "." and ".." for the directory */
	odync_enter(VTOO(*vpp), ".", 1, cred, VTOO(*vpp));
	odync_enter(VTOO(*vpp), "..", 2, cred, VTOO(dvp));
	
	if (ody_attr_cache) {
	    /* update the attr cache */
	    VTOO(*vpp)->o_vattr = ova;
	    VTOO(*vpp)->o_flags |= ON_VATTR;   /* Valid attributes in cnode */
	}
	
	/* 
	 * Invalidate the parent's attr cache, the modification time
	 * has changed 
	 */
	VTOO(dvp)->o_flags &= ~ON_VATTR;
	
	ODYDEBUG(ODY_MKDIR, 
		 myprintf(("mkdir: (%lx.%lx.%lx.%lx) result %d\n",
			   fid.ofid_ttype, fid.ofid_tome,
			   fid.ofid_vnum, fid.ofid_uniq,
			   error)); );
    }
    else {
	*vpp = (struct vnode *)0;
	ODYDEBUG(ODY_MKDIR, myprintf(("mkdir error %d\n",error)););
    }

exit:
    /*
     * Currently, all mkdirs explicitly vput their dvp's.
     * It also appears that we *must* lock the vpp, since
     * lockleaf isn't set, but someone down the road is going
     * to try to unlock the new directory.
     */
    vput(ap->a_dvp);
    if (!error) {
	if ((error = VOP_LOCK(*ap->a_vpp))) {
	    panic("ody_mkdir: couldn't lock child");
	}
    }
    /* Have to free the previously saved name */
    /* 
     * ufs_mkdir doesn't check for SAVESTART before freeing the
     * pathname buffer, but ufs_create does.  For the moment, I'll
     * follow their lead, but this seems like it is probably
     * incorrect.  
     */
    FREE(cnp->cn_pnbuf, M_NAMEI);    
    return(error);
}

int
ody_rmdir(v)
    void *v;
{
/* true args */
    struct vop_rmdir_args *ap = v;
    struct vnode *dvp = ap->a_dvp;
    struct onode *dop = VTOO(dvp);
    struct componentname  *cnp = ap->a_cnp;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* true args */
    int error;
    const char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    struct onode *op;
    
    MARK_ENTRY(ODY_RMDIR_STATS);
    
    /* Check for rmdir of control object. */
    if (IS_ODY_CTL_NAME(dvp, nm, len)) {
	MARK_INT_FAIL(ODY_RMDIR_STATS);
	error = ENOENT;
	goto exit;
    }
    
    /* We're being conservative here, it might be that this person
     * doesn't really have sufficient access to delete the file
     * but we feel zapping the entry won't really hurt anyone -- dcs
     */
    /*
     * As a side effect of the rmdir, remove any entries for children of
     * the directory, especially "." and "..".
     */
    op = odync_lookup(dop, nm, len, cred);
    if (op) odync_zapParentfid(&(op->o_fid), NOT_DOWNCALL);
    
    /* Remove the file's entry from the ODY Name Cache */
    odync_zapfile(dop, nm, len);
    
    /* Invalidate the parent's attr cache, the modification time has changed */
    dop->o_flags &= ~ON_VATTR;

    error = viceroy_rmdir(vtomi(dvp), &dop->o_fid, nm, len, cred, p);
    
exit:
    ODYDEBUG(ODY_RMDIR, myprintf(("in rmdir result %d\n",error)); );

    /*
     * regardless of what happens, we need to drop locks/refs on the 
     * parent and child.  I think. 
     */
    if (ap->a_dvp == ap->a_vp) {
	vrele(ap->a_vp);
    } else {
	vput(ap->a_vp);
    }
    vput(ap->a_dvp);

    if ((cnp->cn_flags & SAVESTART) == 0) {
	FREE(cnp->cn_pnbuf, M_NAMEI);
    }
    return(error);
}

int
ody_symlink(v)
    void *v;
{
/* true args */
    struct vop_symlink_args *ap = v;
    struct vnode *tdvp = ap->a_dvp;
    struct onode *tdop = VTOO(tdvp);	
    struct componentname *cnp = ap->a_cnp;
    struct vattr *tva = ap->a_vap;
    char *path = ap->a_target;
    struct ucred *cred = cnp->cn_cred;
    struct proc *p = cnp->cn_proc;
/* locals */
    int error;
    /* 
     * XXX I'm assuming the following things about ody_symlink's
     * arguments: 
     *       t(foo) is the new name/parent/etc being created.
     *       lname is the contents of the new symlink. 
     */
#ifdef	NetBSD1_3
    const 
#endif
          char *nm = cnp->cn_nameptr;
    int len = cnp->cn_namelen;
    int plen = strlen(path);
    
    /* XXX What about the vpp argument?  Do we need it? */
    /* 
     * Here's the strategy for the moment: perform the symlink, then
     * do a lookup to grab the resulting vnode.  I know this requires
     * two communications with Venus for a new sybolic link, but
     * that's the way the ball bounces.  I don't yet want to change
     * the way the Mach symlink works.  When Mach support is
     * deprecated, we should change symlink so that the common case
     * returns the resultant vnode in a vpp argument.  If we're lucky,
     * the name cache will save us.
     */

    MARK_ENTRY(ODY_SYMLINK_STATS);
    
    /* Check for symlink of control object. */
    if (IS_ODY_CTL_NAME(tdvp, nm, len)) {
	MARK_INT_FAIL(ODY_SYMLINK_STATS);
	error = EACCES;
	goto exit;
    }
    
    if (plen+1 > ODY_MAXPATHLEN) {
	MARK_INT_FAIL(ODY_SYMLINK_STATS);
	error = EINVAL;
        goto exit;
    }

    if (len+1 > ODY_MAXNAMLEN) {
	MARK_INT_FAIL(ODY_SYMLINK_STATS);
	error = EINVAL;
	goto exit;
    }

    error = viceroy_symlink(vtomi(tdvp), &tdop->o_fid, path, plen, nm, len,
			    tva, cred, p);

    if (!error)
/*
	error = ody_lookup(ap->a_dvp, tname, ap->a_vpp,
	                    cnp->cn_cred, cnp->cn_proc);
 */
    {
	struct nameidata nd;
	NDINIT(&nd, LOOKUP, FOLLOW|LOCKLEAF, UIO_SYSSPACE, cnp->cn_nameptr,
	       cnp->cn_proc);
	nd.ni_cnd.cn_cred = cred;
	nd.ni_loopcnt = 0;
	nd.ni_startdir = tdvp;
	nd.ni_cnd.cn_pnbuf = (char *)nm;
	nd.ni_cnd.cn_nameptr = nd.ni_cnd.cn_pnbuf;
	nd.ni_pathlen = len;
	vput(tdvp);
	error = lookup(&nd);
	*ap->a_vpp = nd.ni_vp;
    }
    
    /* Invalidate the parent's attr cache, the modification time has changed */
    /* xxx if !error? XXX */
    tdop->o_flags &= ~ON_VATTR;

exit:
    /* 
     * Okay, now we have to drop locks on dvp.  vpp is unlocked, but
     * ref'd.  It doesn't matter what happens in either symlink or
     * lookup.  Furthermore, there isn't any way for (dvp == *vpp), so
     * we don't bother checking.  
     */
    /* vput(ap->a_dvp); released earlier */
    if (*ap->a_vpp) {
	VOP_UNLOCK(*ap->a_vpp);	/* not doing ody_lookup above -- need UNLOCK */
	vrele(*ap->a_vpp);
    }

    /* 
     * Free the name buffer 
     */
    if ((cnp->cn_flags & SAVESTART) == 0) {
	FREE(cnp->cn_pnbuf, M_NAMEI);
    }

    ODYDEBUG(ODY_SYMLINK, myprintf(("in symlink result %d\n",error)); );
    return(error);
}

/*
 * Read directory entries.
 */
int
ody_readdir(v)
    void *v;
{
/* true args */
    struct vop_readdir_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);
    register struct uio *uiop = ap->a_uio;
    struct ucred *cred = ap->a_cred;
    int *eofflag = ap->a_eofflag;
#ifdef	NetBSD1_3
    off_t *cookies = ap->a_cookies;
    int ncookies = ap->a_ncookies;
#elif	NetBSD1_2
    u_long *cookies = ap->a_cookies;
    int ncookies = ap->a_ncookies;
#elif	defined(__FreeBSD__)
    u_int **cookies = ap->a_cookies;
    int *ncookies = ap->a_ncookies;
#endif
    struct proc *p = ap->a_uio->uio_procp;
/* upcall decl */
/* locals */
    int error = 0;
    
    MARK_ENTRY(ODY_READDIR_STATS);
    
    ODYDEBUG(ODY_READDIR, 
	     myprintf(("ody_readdir(%p, %d, %qd, %d)\n", 
		       uiop->uio_iov->iov_base, uiop->uio_resid, 
		       uiop->uio_offset, uiop->uio_segflg)); );
    
    /* Check for readdir of control object. */
    if (IS_ODY_CTL_VP(vp)) {
	MARK_INT_FAIL(ODY_READDIR_STATS);
	return(ENOENT);
    }
    
    if (ody_intercept_rdwr && !IS_UNCACHEABLE(op)) {
	/* Redirect the request to UFS. */
	
	/* If directory is not already open do an "internal open" on it. */
	int opened_internally = 0;
	if (op->o_ovp == NULL) {
	    opened_internally = 1;
	    MARK_INT_GEN(ODY_OPEN_STATS);
	    error = VOP_OPEN(vp, FREAD, cred, p);
#ifdef	__FreeBSD__
	    if (error) {
		printf("ody_readdir: VOP_OPEN on container failed %d\n",
		       error);
		return (error);
	    }
	    if (vp->v_type == VREG) {
		error = vfs_object_create(vp, p, cred, 1);
		if (error != 0) {
		    printf("ody_readdir: vfs_object_create() returns %d\n",
			   error);
		    vput(vp);
		}
	    }
#endif
	    if (error) return(error);
	}
	
	/* Have UFS handle the call. */
	ODYDEBUG(ODY_READDIR, 
		 myprintf(("indirect readdir: fid = (%lx.%lx.%lx.%lx), refcnt = %d\n",
			   op->o_fid.ofid_ttype,
			   op->o_fid.ofid_tome,
			   op->o_fid.ofid_vnum,
			   op->o_fid.ofid_uniq,
			   vp->v_usecount)););
#ifdef	__NetBSD__
	error = VOP_READDIR(op->o_ovp, uiop, cred, eofflag, cookies,
			    ncookies);
#elif	defined(__FreeBSD__)
	error = VOP_READDIR(op->o_ovp, uiop, cred, eofflag, ncookies,
			       cookies);
#endif
	
	if (error)
	    MARK_INT_FAIL(ODY_READDIR_STATS);
	else
	    MARK_INT_SAT(ODY_READDIR_STATS);
	
	/* Do an "internal close" if necessary. */ 
	if (opened_internally) {
	    MARK_INT_GEN(ODY_CLOSE_STATS);
	    (void)VOP_CLOSE(vp, FREAD, cred, p);
	}
    }
    else {
	/* Read the block from Venus. */
	struct iovec *iovp = uiop->uio_iov;
	unsigned count = iovp->iov_len;
	int size;
	
	/* 
	 * Make the count a multiple of DIRBLKSIZ (borrowed from
	 * ufs_readdir). 
	 */
#define DIRBLKSIZ DEV_BSIZE
	if ((uiop->uio_iovcnt != 1) || (count < DIRBLKSIZ) ||
	    (uiop->uio_offset & (DIRBLKSIZ - 1)))
	    return (EINVAL);
	count &= ~(DIRBLKSIZ - 1);
	uiop->uio_resid -= iovp->iov_len - count;
	iovp->iov_len = count;
	if (count > OC_MAXDATASIZE)
	    return(EINVAL);

	error = viceroy_readdir(vtomi(vp), &op->o_fid, count,
				uiop->uio_offset, cred, p,
				iovp->iov_base, &size);
	
	ODYDEBUG(ODY_READDIR,
		 myprintf(("ody_readdir (%p, %d, %qd, %d) returns (%d, %d)\n",
			   iovp->iov_base, count,
			   uiop->uio_offset, uiop->uio_segflg, error,
			   size)); );
	    
	if (!error) {
	    iovp->iov_base += size;
	    iovp->iov_len -= size;
	    uiop->uio_resid -= size;
	    uiop->uio_offset += size;
	}
    }
    
    return(error);
}

/*
 * Convert from file system blocks to device blocks
 */
int
ody_bmap(v)
    void *v;
{
    /* XXX on the global proc */
/* true args */
    struct vop_bmap_args *ap = v;
    struct vnode *vp __attribute__((unused)) = ap->a_vp;	/* file's vnode */
    daddr_t bn __attribute__((unused)) = ap->a_bn;	/* fs block number */
    struct vnode **vpp = ap->a_vpp;			/* RETURN vp of device */
    daddr_t *bnp __attribute__((unused)) = ap->a_bnp;	/* RETURN device block number */
    struct proc *p __attribute__((unused)) = curproc;
/* upcall decl */
/* locals */

#ifdef	__FreeBSD__
        /* Just like nfs_bmap(). Do not touch *vpp, this cause pfault. */
	return(EOPNOTSUPP);
#else
	*vpp = (struct vnode *)0;
	myprintf(("ody_bmap called!\n"));
	return(EINVAL);
#endif
}

/*
 * I don't think the following two things are used anywhere, so I've
 * commented them out 
 * 
 * struct buf *async_bufhead; 
 * int async_daemon_count;
 */

int
ody_strategy(v)
    void *v;
{
/* true args */
    struct vop_strategy_args *ap = v;
    register struct buf *bp __attribute__((unused)) = ap->a_bp;
    struct proc *p __attribute__((unused)) = curproc;
/* upcall decl */
/* locals */

#ifdef	__FreeBSD__
#ifdef	__MAYBE_FreeBSD__
	struct vnode *vp;
	struct onode *op;
	/*
	 * This needs testing.  execve calls bread and will go thru
	 * this code.  So currently we comment out the "optimization"
	 * in exec.
	 */

	myprintf(("ody_strategy called!  "));
	vp = bp->b_vp;
	op = VTOO(vp);
	if (op->o_ovp) {
		printf("redirect thru container\n");
		bp->b_vp = op->o_ovp;
		return VOP_STRATEGY(bp);
	} else {
		printf("no container\n");
		return(EOPNOTSUPP);
	}
#else	/* ! __MAYBE_FreeBSD__ */
	myprintf(("ody_strategy called!  "));
	return(EOPNOTSUPP);
#endif	/* __MAYBE_FreeBSD__ */
#else	/* ! __FreeBSD__ */
	myprintf(("ody_strategy called!  "));
	return(EINVAL);
#endif	/* __FreeBSD__ */
}

int
ody_reclaim(v)
    void *v;
{
    struct vop_reclaim_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op = VTOO(vp);

    ENTRY;

    if (IS_UNMOUNTING(op)) {
#ifdef DEBUG
	if (op->o_ovp)
	    printf("ody_reclaim: o_ovp not void: vp %p, op %p\n", vp, op);
#endif
    } else {
#ifdef DIAGNOSTIC
	if (vp->v_usecount != 0) 
	    vprint("ody_reclaim: pushing active", vp);
	if (op->o_ovp)
	    panic("ody_reclaim: c_ovp not void");
#endif DIAGNOSTIC
    }
    cache_purge(vp);
    ody_free(op);
    VTOO(vp) = NULL;
    return (0);
}

int
ody_lock(v)
    void *v;
{
    struct vop_lock_args *ap = v;
    struct vnode *vp = ap->a_vp;
    struct onode *op;
    
    ENTRY;
    op = VTOO(vp);

    if (ody_lockdebug) {
	myprintf(("Attempting lock on %lx.%lx.%lx.%lx\n",
		  op->o_fid.ofid_ttype, op->o_fid.ofid_tome,
		  op->o_fid.ofid_vnum, op->o_fid.ofid_uniq));
    }
 start:
    while (vp->v_flag & VXLOCK) {
	vp->v_flag |= VXWANT;
#ifdef	__NetBSD__
	(void) sleep((caddr_t)vp, PINOD);
#elif	defined(__FreeBSD__)
	(void) tsleep((caddr_t)vp, PINOD, "ody_lock1", 0);
#endif
    }

    if (vp->v_tag == VT_NON)
	return (ENOENT);

    if (op->o_flags & ON_LOCKED) {
	op->o_flags |= ON_WANTED;
#ifdef DIAGNOSTIC
	myprintf(("ody_lock: lock contention\n"));
	odync_name(op);
	myprintf(("\n"));
#endif
#ifdef	__NetBSD__
	(void) sleep((caddr_t)op, PINOD);
#elif	defined(__FreeBSD__)
	(void) tsleep((caddr_t)op, PINOD, "ody_lock2", 0);
#endif
#ifdef DIAGNOSTIC
	myprintf(("ody_lock: contention resolved\n"));
#endif
	goto start;
    }
    op->o_flags |= ON_LOCKED;
    return (0);
}

int
ody_unlock(v)
    void *v;
{
    struct vop_unlock_args *ap = v;
    struct onode *op = VTOO(ap->a_vp);

    ENTRY;
    if (ody_lockdebug) {
	myprintf(("Attempting unlock on %lx.%lx.%lx.%lx\n",
		  op->o_fid.ofid_ttype, op->o_fid.ofid_tome,
		  op->o_fid.ofid_vnum, op->o_fid.ofid_uniq));
    }
#ifdef DIAGNOSTIC
    if ((op->o_flags & ON_LOCKED) == 0) 
	panic("ody_unlock: not locked");
#endif
    op->o_flags &= ~ON_LOCKED;
    if (op->o_flags & ON_WANTED) {
	op->o_flags &= ~ON_WANTED;
	wakeup((caddr_t)op);
    }
    return (0);
}

int
ody_islocked(v)
    void *v;
{
    struct vop_islocked_args *ap = v;

    ENTRY;
    if (VTOO(ap->a_vp)->o_flags & ON_LOCKED)
	return (1);
    return (0);
}

#ifdef SETS

/* Dynamic Sets vnode operations */

/*
  Would be nice to abstract the common parts of ody_lookup and ody_expand.
  Just need to make different odycalls, and set CURSOR flags in expand.
  */
int ody_expand(struct vnode *dvp,
	       struct vnode **vpp,
	       char *nm,
	       struct proc *p)
{
	struct onode *dop = VTOO(dvp);
	/* It looks as though ap->a_cnp->ni_cnd->cn_nameptr holds the
	 * rest of the string to xlate, and that we must try to get at
	 * least ap->a_cnp->ni_cnd->cn_namelen of those characters to
	 * macth.  I could be wrong.  */

/* locals */
	struct ucred *cred = p->p_cred->pc_ucred;
	struct onode *op;
	char *c = nm;
	int len = 0;
	struct ofid fid;
	int vtype;
	int error = 0;
	int cursor;
	
	while (*c && *c != '/') {
		len++;
		c++;
	}
	
	MARK_ENTRY(ODY_EXPAND_STATS);

	/* NOTE: using ODY_LOOKUP because ODYDEBUG breaks with
	 * operations > 31!! */
	ODYDEBUG(ODY_LOOKUP, myprintf(("expand: %s in %lx.%lx.%lx.%lx\n",
				       nm, dop->o_fid.ofid_ttype,
				       dop->o_fid.ofid_tome,
				       dop->o_fid.ofid_vnum,
				       dop->o_fid.ofid_uniq)););
	
	/* Check for expand of control object. */
	if (IS_ODY_CTL_NAME(dvp, nm, len)) {
		VOP_UNLOCK(dvp);
		MARK_INT_SAT(ODY_LOOKUP_STATS);
		error = EINVAL;
		goto exit;
	}
    
	if (len+1 > ODY_MAXNAMLEN) {
	    MARK_INT_FAIL(ODY_EXPAND_STATS);
	    ODYDEBUG(ODY_LOOKUP, myprintf(("name too long: lookup, %lx.%lx.%lx.%lx(%s)\n",
					   dop->o_fid.ofid_ttype,
					   dop->o_fid.ofid_tome,
					   dop->o_fid.ofid_vnum,
					   dop->o_fid.ofid_uniq, nm)););
	    error = EINVAL;
	    goto exit;
	}

	/* Cursors are never cached in minicache. */
	/* The name wasn't cached, so we need to contact Venus */
	error = viceroy_expand(vtomi(dvp), &dop->o_fid, nm, len, cred, p,
			       &fid, &vtype, &cursor);
	
	if (error) {
		MARK_INT_FAIL(ODY_EXPAND_STATS);
		ODYDEBUG(ODY_LOOKUP, 
			 myprintf(("expand error on %lx.%lx.%lx.%lx (%s) %d\n",
				   dop->o_fid.ofid_ttype,
				   dop->o_fid.ofid_tome,
				   dop->o_fid.ofid_vnum,
				   dop->o_fid.ofid_uniq, nm, error)););
	} else {
		MARK_INT_SAT(ODY_EXPAND_STATS);
		ODYDEBUG(ODY_LOOKUP, 
			 myprintf(("expand: (%lx.%lx.%lx.%lx) "
				   "type %o curs=%d result %d\n",
				   fid.ofid_ttype, fid.ofid_tome,
				   fid.ofid_vnum, fid.ofid_uniq,
				   vtype, cursor, error)); );
		
		op = makeodynode(&fid, dvp->v_mount, vtype);
		*vpp = OTOV(op);
		if (cursor) {
			op->o_flags |= ON_CURSOR;
		}
	}
 exit:
	/* 
	 * If the expand went well, we need to (potentially?) unlock the
	 * parent, and lock the child.  We are only responsible for
	 * checking to see if the parent is supposed to be unlocked before
	 * we return.  We must always lock the child (provided there is
	 * one, and (the parent isn't locked or it isn't the same as the
	 * parent.)  Simple, huh?  We can never leave the parent locked unless
	 * we are ISLASTCN
	 */
	if (!error || (error == EJUSTRETURN)) {
		if ((error = VOP_UNLOCK(dvp))) {
			printf("vop_unlock in ody_expand broke (%d)\n", error);
			return error; 
		}

		/* 
		 * The parent is unlocked.  As long as there is a
		 * child, lock it without bothering to check
		 * anything else. 
		 */
		if (*vpp) {
			if ((error = VOP_LOCK(*vpp))) {
				printf("ody_expand: ");
				panic("unlckd parent but couldn't lock child");
			}
		}
	} else {
		/* XXX do this here? */
		if ((error = VOP_UNLOCK(dvp))) {
			printf("vop_unlock2 in ody_expand broke (%d)\n",
			       error);
			return error; 
		}

		/* If the lookup failed, we need to ensure that the
		 * leaf is NULL */
		*vpp = NULL;
	}
	return(error);
}

int ody_cursexpand(struct vnode *vp,
		   size_t *bufsize,
		   char **buf,
		   struct proc *p)
{
/* locals */
	struct ucred *cred = p->p_cred->pc_ucred;
        struct onode *op = VTOO(vp);
        int error;

	MARK_ENTRY(ODY_CURSEXP_STATS);
	
	/* Check for rdwr of control object. */
	if (IS_ODY_CTL_VP(vp)) {
		MARK_INT_FAIL(ODY_RDWR_STATS);
		return(EINVAL);
	}

	error = viceroy_cursexpand(vtomi(vp), &op->o_fid, cred, p,
				   buf, bufsize);

	return error;
}

int ody_prefetch(struct vnode *vp, int mode, struct proc *p,
		 struct file **f)
{
	struct ucred *cred = p->p_cred->pc_ucred;
	struct onode *op = VTOO(vp);
	dev_t dev;
	ino_t ino;
	int error;
	int flags;
	
#if 0
        printf("prefetching\n");
#endif

	error = viceroy_prefetch(vtomi(vp), &op->o_fid, mode, cred, p,
				 &dev, &ino, &flags);

	if (error)
		return error;

#if 0
	if ((flags & ON_UNCACHEABLE) != (op->o_flags & ON_UNCACHEABLE))
		printf("uncacheable switch! %d -> %d\n", op->o_flags, flags);
#endif

	/* XXX unlike ody_open, I'm not checking that uncacheable has
	 *     changed if this guy was already open.  Think about what
	 *     this means, do it the same both places? */
	if (flags & ON_UNCACHEABLE)
		op->o_flags |= ON_UNCACHEABLE;
	else {
		struct vnode *cvp;
		
		op->o_flags &= ~ON_UNCACHEABLE;
		
		printf("grabbing %d %d\n", dev, ino);
		error = ody_grab_vnode(dev, ino, &cvp);
		if (error)
			return error;
		
		if (op->o_ovp == NULL) {
			op->o_ovp = cvp;
		} else {
			if (op->o_ovp != cvp)
				panic("ody_open:  op->o_ovp != ITOV(ip)");
		}
		error = VOP_OPEN(cvp, mode, cred, p);
		printf(" vnode %p gets %p (err %d)\n", vp, cvp, error);
		if (error)
			return error;
		
		VOP_UNLOCK(cvp); /* right? */
		op->o_device = dev;
		op->o_inode = ino;
	}
	op->o_ocount++;
	vref(vp);

	return 0;
}
#endif /* SETS */

/* How one looks up a vnode given a device/inode pair: */
int
ody_grab_vnode(dev_t dev, ino_t ino, struct vnode **vpp)
{
    /* This is like VFS_VGET() or igetinode()! */
    int           error;
    struct mount *mp;

    if (!(mp = ody_devtomp(dev))) {
	myprintf(("ody_grab_vnode: ody_devtomp(%d) returns NULL\n", dev));
	return(ENXIO);
    }

    /* XXX - ensure that nonzero-return means failure */
    error = VFS_VGET(mp,ino,vpp);
    if (error) {
	myprintf(("ody_grab_vnode: iget/vget(%d, %d) returns %p, err %d\n", 
		  dev, ino, *vpp, error));
	return(ENOENT);
    }
    return(0);
}

void
ody_print_vattr( attr )
	struct vattr *attr;
{
    char *typestr;

    switch (attr->va_type) {
    case VNON:
	typestr = "VNON";
	break;
    case VREG:
	typestr = "VREG";
	break;
    case VDIR:
	typestr = "VDIR";
	break;
    case VBLK:
	typestr = "VBLK";
	break;
    case VCHR:
	typestr = "VCHR";
	break;
    case VLNK:
	typestr = "VLNK";
	break;
    case VSOCK:
	typestr = "VSCK";
	break;
    case VFIFO:
	typestr = "VFFO";
	break;
    case VBAD:
	typestr = "VBAD";
	break;
    default:
	typestr = "????";
	break;
    }


    myprintf(("attr: type %s mode %d uid %d gid %d fsid %d rdev %d\n",
	      typestr, (int)attr->va_mode, (int)attr->va_uid,
	      (int)attr->va_gid, (int)attr->va_fsid, (int)attr->va_rdev));
    
    myprintf(("      fileid %d nlink %d size %d blocksize %d bytes %d\n",
	      (int)attr->va_fileid, (int)attr->va_nlink, 
	      (int)attr->va_size,
	      (int)attr->va_blocksize,(int)attr->va_bytes));
    myprintf(("      gen %ld flags %ld vaflags %d\n",
	      attr->va_gen, attr->va_flags, attr->va_vaflags));
    myprintf(("      atime sec %d nsec %d\n",
	      (int)attr->va_atime.tv_sec, (int)attr->va_atime.tv_nsec));
    myprintf(("      mtime sec %d nsec %d\n",
	      (int)attr->va_mtime.tv_sec, (int)attr->va_mtime.tv_nsec));
    myprintf(("      ctime sec %d nsec %d\n",
	      (int)attr->va_ctime.tv_sec, (int)attr->va_ctime.tv_nsec));
}

/* How to print a ucred */
void ody_print_cred(cred)
     struct ucred *cred;
{

	int i;

	myprintf(("ref %d\tuid %d\n",cred->cr_ref,cred->cr_uid));

	for (i=0; i < cred->cr_ngroups; i++)
		myprintf(("\tgroup %d: (%d)\n",i,cred->cr_groups[i]));
	myprintf(("\n"));

}

/*
 * Return a vnode for the given fid.
 * If no cnode exists for this fid create one and put it
 * in a table hashed by fid.Volume and fid.Vnode.  If the cnode for
 * this fid is already in the table return it (ref count is
 * incremented by ody_find.  The cnode will be flushed from the
 * table when ody_inactive calls ody_unsave.
 */
struct onode *
makeodynode(fid, vfsp, type)
    struct ofid  *fid; 
    struct mount *vfsp; 
    short        type;
{
    struct onode *op;
    int          err;
    
    if ((op = ody_find(fid, vfsp)) == NULL) {
	struct vnode *vp;
	
	op = ody_alloc();
	op->o_fid = *fid;
	
	err = getnewvnode(VT_ODYSSEY, vfsp, ody_vnodeop_p, &vp);
	if (err) panic("ody: getnewvnode returned error %d\n", err);
	vp->v_data = op;
	vp->v_type = type;
	op->o_vnode = vp;
	ody_save(op);

    } else {
	vref(OTOV(op));
    }
    
    return op;
}
