#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * odydevmod.c
 *
 * 05 Jun 93	Terry Lambert		Original
 *
 * Copyright (c) 1993 Terrence R. Lambert.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Terrence R. Lambert.
 * 4. The name Terrence R. Lambert may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TERRENCE R. LAMBERT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE TERRENCE R. LAMBERT BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$NetBSD: odydevmod.c,v 1.5 1997/10/13 11:21:12 lukem Exp $
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/ioctl.h>
#include <sys/conf.h>
#include <sys/mount.h>
#include <sys/exec.h>
#include <sys/lkm.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/kernel.h>

#include "kernel/vfs/includes/odydev.h"

int ody_dev_print_entry = 0;
#ifdef __GNUC__
#define ENTRY    \
    if(ody_dev_print_entry) printf("Entered %s\n",__FUNCTION__)
#else
#define ENTRY
#endif 

#ifdef __NetBSD__
#ifdef NetBSD1_3
/* open, close, read, write, ioctl, poll */
#define cdev_odydev_init(c,n) { \
        dev_init(c,n,open), dev_init(c,n,close), dev_init(c,n,read), \
        dev_init(c,n,write), dev_init(c,n,ioctl),  \
        (dev_type_stop((*))) nullop, 0, dev_init(c,n,poll), \
        (dev_type_mmap((*))) enodev}
#else
/* open, close, read, write, ioctl, select */
#define cdev_odydev_init(c,n) { \
        dev_init(c,n,open), dev_init(c,n,close), dev_init(c,n,read), \
        dev_init(c,n,write), dev_init(c,n,ioctl),  \
        (dev_type_stop((*))) nullop, 0, dev_init(c,n,select), \
        (dev_type_mmap((*))) enodev}
#endif
cdev_decl(odydev_);
static struct cdevsw odydevsw = cdev_odydev_init(NODYDEV,odydev_);
#endif /* __NetBSD__ */

#ifdef __FreeBSD__
d_open_t    odydev_open;
d_close_t   odydev_close;
d_read_t    odydev_read;
d_write_t   odydev_write;
d_ioctl_t   odydev_ioctl;
d_select_t  odydev_select;
static struct cdevsw odydevsw =
{ 
  odydev_open,      odydev_close,   odydev_read,       odydev_write,
  odydev_ioctl,     nostop,         nullreset,         nodevtotty,
  odydev_select,    nommap,         NULL,              "Odyssey", NULL, -1
};
#endif /* __FreeBSD__ */

#ifdef __NetBSD__
MOD_DEV("odydev", LM_DT_CHAR, ODYDEVNUM, &odydevsw);
#endif
#ifdef __FreeBSD__
MOD_DEV(odydev, LM_DT_CHAR, ODYDEVNUM, &odydevsw);
#endif

extern int is_odydev_open(void);

odydev_mod(lkmtp, cmd, ver)
	struct lkm_table *lkmtp;	
	int cmd;
	int ver;
{

	ENTRY;

	if ((cmd  == LKM_E_UNLOAD) && is_odydev_open()) {
	    printf("odydev_mod: cannot unload odydev\n");
	    return (EBUSY);
	}

#ifdef __FreeBSD__
	/* Someone was smoking bad crack. All the other MOD_<foo> functions
	   declare _module, but MOD_DEV will declare odydev_module */
#define _module odydev_module
	DISPATCH(lkmtp, cmd, ver, lkm_nullcmd, lkm_nullcmd, lkm_nullcmd)
#endif
#ifdef __NetBSD__
	DISPATCH(lkmtp, cmd, ver, lkm_nofunc, lkm_nofunc, lkm_nofunc)
#endif
}
