#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * odymod.c
 *
 * 05 Jun 93	Terry Lambert		Original
 *
 * Copyright (c) 1993 Terrence R. Lambert.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Terrence R. Lambert.
 * 4. The name Terrence R. Lambert may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TERRENCE R. LAMBERT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE TERRENCE R. LAMBERT BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$NetBSD: odymod.c,v 1.5 1997/10/13 11:21:12 lukem Exp $
 */
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/ioctl.h>
#include <sys/conf.h>
#include <sys/mount.h>
#include <sys/exec.h>
#include <sys/select.h>
#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/onode.h"
#ifdef __FreeBSD__
#include <sys/sysent.h>
#endif
#include <sys/lkm.h>
#include <sys/file.h>
#include <sys/errno.h>
#ifdef __NetBSD__
#include <sys/vnode.h>
#include <sys/malloc.h>
#endif

int ody_mod_print_entry = 0;
#ifdef __GNUC__
#define ENTRY    \
    if(ody_mod_print_entry) printf("Entered %s\n",__FUNCTION__)
#else
#define ENTRY
#endif 

/*
 * This is the vfsops table from /sys/miscfs/ody/ody_vfsops.c
 */
extern struct vfsops ody_vfsops;
extern struct vnodeopv_desc ody_vnodeop_opv_desc;
#ifdef __NetBSD__
MOD_VFS("ody", -1, &ody_vfsops)

ody_mod(lkmtp, cmd, ver)
	struct lkm_table *lkmtp;	
	int cmd;
	int ver;
{
	ENTRY;

	switch (cmd) {

	case LKM_E_LOAD:
	/*
	 * This is normally done automatically at boot time if the
	 * opv_desc is listed in vfs_opv_descs[] in vfs_conf.c.  For
	 * loaded modules, we have to do it manually.
	 */

	    vfs_opv_init_explicit(&ody_vnodeop_opv_desc);
	    vfs_opv_init_default(&ody_vnodeop_opv_desc);
	    break;

	case LKM_E_UNLOAD:
	    /* This gets allocated in vfs_opv_init_explicit.
	       there's gotta be a cleaner way to do this */
	    FREE(*(ody_vnodeop_opv_desc.opv_desc_vector_p), M_VNODE);
	    *(ody_vnodeop_opv_desc.opv_desc_vector_p) = NULL;
	    break;
	}

	DISPATCH(lkmtp, cmd, ver, lkm_nofunc, lkm_nofunc, lkm_nofunc)
}
#endif

#ifdef __FreeBSD__
/* VFS_SET(ody_vfsops, ody, MOUNT_ODY, VFCF_NETWORK); */
static struct vfsconf _fs_vfsconf = {
    &ody_vfsops,
    "ody",
    MOUNT_ODY,
    0,
    VFCF_NETWORK
};
extern struct linker_set MODVNOPS;
MOD_VFS("ody",MOUNT_ODY,&MODVNOPS,&_fs_vfsconf);
ody_mod(struct lkm_table *lkmtp, int cmd, int ver) {
    DISPATCH(lkmtp, cmd, ver, lkm_nullcmd, lkm_nullcmd, lkm_nullcmd);
}
#endif
