#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _ODY_HEADER_
#define _ODY_HEADER_

/*
 * This file is based on <cfs/coda.h>
 * re-synced by kwalker May '98
 *
 * It contains definitions that are shared by the kernel and the viceroy
 */

/* Catch new _KERNEL defn for NetBSD */
#ifdef __BSD44__
#include <sys/types.h>
#include <sys/time.h>
#endif 

#if defined(__linux__) && !defined(__KERNEL__)
#include <sys/types.h>
#endif

#if defined(__linux__) || defined(__CYGWIN32__)
#define odev_t ody_quad_t
#if !defined(_ODYQUAD_T_) && (!defined(__GLIBC__) || __GLIBC__ < 2)
#define _ODYQUAD_T_ 1
typedef unsigned long long ody_quad_t;
#else
#define ody_quad_t u_quad_t
#endif
#else
#define odev_t dev_t
#define ody_quad_t u_quad_t
#endif

#ifdef __CYGWIN32__
typedef unsigned char u_int8_t;
struct timespec {
        time_t  tv_sec;         /* seconds */
        long    tv_nsec;        /* nanoseconds */
};
#endif


/*
 * Define M_ODYSSEY for use in mallocing.
 * Currently, we define it to be the same
 * as M_TEMP, since we don't want to change the kernel sources
 */
#define M_ODYSSEY M_TEMP

/*
 * ODY constants
 */
#define ODY_MAXNAMLEN   255
#define ODY_MAXPATHLEN  1024
#define ODY_MAXSYMLINK  10

#define ODY_RPOGRAM	((u_long)0x20202020)
#define ODY_VERSION	((u_long)1)	

/* these flags are used in "open" calls for the flag (fcntl.h)
 * Other flags from fcntl.h might be needed eventually */
#define	O_O_READ	  0x001
#define	O_O_WRITE         0x002
#define O_O_NONBLOCK      0x004
#define O_O_TRUNC         0x008
#define O_O_EXCL	  0x010

/* these flags are used in "rdwr" calls for the ioflag (vnode.h) */
#define O_IO_NDELAY       0x10	/* FNDELAY flag set in file table */

/* these flags are used in "rdwr" calls for the rwflag (uio.h) */
#define O_RW_READ         0x0
#define O_RW_WRITE        0x1

/* these flags are used in "access" calls for the mode (vnode.h) */
#define O_A_READ          0x4	/* Test for read permission.  */
#define O_A_WRITE         0x2	/* Test for write permission.  */
#define O_A_EXEC          0x1	/* Test for execute permission.  */

/* these flags are used in "get/setattr" calls for the va_mode (stat.h)
 * For now, just supply shorthand since we don't care a whole lot */
#define O_S_READ          0000444 /* read access for everyone */
#define O_S_WRITE         0000666 /* rdwr access for everyone */


#ifndef _ODY_DIRENT_T_
#define _ODY_DIRENT_T_ 1
struct ody_dirent {
        unsigned long	d_fileno;		/* file number of entry */
        unsigned short	d_reclen;		/* length of this record */
        char 		d_type;			/* file type, see below */
        char		d_namlen;		/* length of string in d_name */
        char		d_name[ODY_MAXNAMLEN + 1];/* name must be no longer than this */
};
#undef DIRSIZ
#define DIRSIZ(dp)      ((sizeof (struct ody_dirent) - (ODY_MAXNAMLEN+1)) + \
                         (((dp)->d_namlen+1 + 3) &~ 3))

/*
 * File types
 */
#define	ODT_UNKNOWN	 0
#define	ODT_FIFO	 1
#define	ODT_CHR		 2
#define	ODT_DIR		 4
#define	ODT_BLK		 6
#define	ODT_REG		 8
#define	ODT_LNK		10
#define	ODT_SOCK	12
#define	ODT_WHT		14

/*
 * Convert between stat structure types and directory types.
 */
#define	IFTOODT(mode)	(((mode) & 0170000) >> 12)
#define	ODTTOIF(dirtype)	((dirtype) << 12)

#endif

#ifndef	_ODYFID_T_
#define _ODYFID_T_	1
typedef u_long ttype_t;
typedef u_long tome_t;
typedef u_long vnum_t;
typedef u_long uniq_t;
#endif 

#ifndef	_OFID_T_
#define _OFID_T_	1
struct ofid {
    ttype_t  ofid_ttype;
    tome_t   ofid_tome;
    vnum_t   ofid_vnum;
    uniq_t   ofid_uniq;
};
#endif	/* OFID_T */

static inline ino_t ody_f2i(struct ofid *fid)
{
      if ( fid ) {
              return ((fid->ofid_uniq&0xfff) +
		      ((fid->ofid_vnum&0xfff) << 12) +
		      ((fid->ofid_ttype&0x1f) << 24) +
		      ((fid->ofid_tome&0x7) << 29));
      } else { 
              return 0;
      }
}

#ifndef _OUID_T_
#define _OUID_T_
typedef u_long ouid_t;
typedef u_long ogid_t;
#endif /*_OUID_T_ */

#ifndef _ODYCRED_T_
#define _ODYCRED_T_
struct ody_cred {
    ouid_t ocr_uid, ocr_euid, ocr_suid, ocr_fsuid; /* Real,efftve,set,fs uid*/
    ogid_t ocr_gid, ocr_egid, ocr_sgid, ocr_fsgid; /* same for groups */
};
#endif 

#ifndef _VICEROY_VATTR_T_
#define _VICEROY_VATTR_T_
/*
 * Vnode types.  VNON means no type.
 */
enum ody_vtype	{ O_VNON, O_VREG, O_VDIR, O_VBLK, O_VCHR, O_VLNK, O_VSOCK, O_VFIFO, O_VBAD };

struct ody_vattr {
	enum ody_vtype	va_type;	/* vnode type (for create) */
	u_short		va_mode;	/* files access mode and type */
	short		va_nlink;	/* number of references to file */
	ouid_t		va_uid;		/* owner user id */
	ogid_t		va_gid;		/* owner group id */
	long		va_fsid;	/* file id */
	long		va_fileid;	/* file id */
	ody_quad_t	va_size;	/* file size in bytes */
	long		va_blocksize;	/* blocksize preferred for i/o */
	time_t	        va_atime;	/* time of last access */
	time_t	        va_mtime;	/* time of last modification */
	time_t	        va_ctime;	/* time file changed */
	u_long		va_gen;		/* generation number of file */
	u_long		va_flags;	/* flags defined for file */
	odev_t	        va_rdev;	/* device special file represents */
	ody_quad_t	va_bytes;	/* bytes of disk space held by file */
	ody_quad_t	va_filerev;	/* file modification number */
};

#endif 

/*
 * Kernel <--> Viceroy communications.
 */
	
#define ODY_ROOT	((u_long) 2)
#define ODY_SYNC	((u_long) 3)
#define ODY_OPEN	((u_long) 4)
#define ODY_CLOSE	((u_long) 5)
#define ODY_IOCTL	((u_long) 6)
#define ODY_GETATTR	((u_long) 7)
#define ODY_SETATTR	((u_long) 8)
#define ODY_ACCESS	((u_long) 9)
#define ODY_LOOKUP	((u_long) 10)
#define ODY_CREATE	((u_long) 11)
#define ODY_REMOVE	((u_long) 12)
#define ODY_LINK	((u_long) 13)
#define ODY_RENAME	((u_long) 14)
#define ODY_MKDIR	((u_long) 15)
#define ODY_RMDIR	((u_long) 16)
#define ODY_READDIR	((u_long) 17)
#define ODY_SYMLINK	((u_long) 18)
#define ODY_READLINK	((u_long) 19)
#define ODY_FSYNC	((u_long) 20)
#define ODY_INACTIVE	((u_long) 21)
#define	ODY_VGET	((u_long) 22)
#define	ODY_SIGNAL	((u_long) 23)
#define ODY_REPLACE	((u_long) 24)
#define ODY_FLUSH       ((u_long) 25)
#define ODY_PURGEUSER   ((u_long) 26)
#define ODY_ZAPFILE     ((u_long) 27)
#define ODY_ZAPDIR      ((u_long) 28)
#define ODY_ZAPVNODE    ((u_long) 29)
#define ODY_PURGEFID    ((u_long) 30)

/* Dynamic sets */
#define ODY_EXPAND	((u_long) 32) /* Don't use DEBUG >= 32 */
#define ODY_CURSEXPAND  ((u_long) 33)
#define ODY_PREFETCH    ((u_long) 34)

/* Odyssey */
#define	ODY_RDWR	((u_long) 31)
#define ODY_REQUEST     ((u_long) 35)
#define ODY_CANCEL      ((u_long) 36)
#define ODY_TSOP        ((u_long) 37)
#define ODY_CANCPID     ((u_long) 38)
#define ODY_TOOSMALL    ((u_long) 39)

/* Power - specific */
#define ODY_BMOD_START  ((u_long) 41)
#define ODY_BMOD_STOP   ((u_long) 42)
#define ODY_TIME_TO_RECHARGE ((u_long) 43)
#define ODY_MULTIF ((u_long) 44)
#define ODY_NCALLS 44

#ifdef __linux__
#define ODY_DOWNCALL(opcode) (opcode >= ODY_REPLACE && opcode <= ODY_PURGEFID)
#else
#define DOWNCALL(opcode) (opcode >= ODY_REPLACE && opcode <= ODY_PURGEFID)
#endif

#define OC_MAXDATASIZE	    4096
#define	OC_MAXMSGSIZE	    sizeof(union ody_inputArgs)+ \
                            sizeof(union ody_outputArgs) +\
                            OC_MAXDATASIZE  


/*
 *    Viceroy <-> Ody RPC arguments
 */
struct ody_in_hdr {
	unsigned long opcode;
	unsigned long unique;	/* Keep multiple outstanding msgs distinct */
	u_short       pid;	/* Common to all */
	u_short       pgid;	/* Common to all */
        u_short       sid;	/* Common to all */
	struct ody_cred cred;	/* Common to all */
};

/* Really important that opcode and unique are 1st two fields! */
struct ody_out_hdr {
        unsigned long opcode;
        unsigned long unique;	
        unsigned long result;
};


/* Note: trivial structs are still defined, since linux macros depend on
 *       their existence */

/* ody_root: NO_IN */
struct ody_root_in {
	struct ody_in_hdr ih;
};

struct ody_root_out {
        struct ody_out_hdr oh;
	struct ofid       fid;
};

/* ody_sync: */
/* Nothing needed for ody_sync */

/* ody_open: */
struct ody_open_in {
        struct ody_in_hdr ih;
	struct ofid   fid;
	int	      flags;
};

struct ody_open_out {
        struct ody_out_hdr oh;
	odev_t	dev;
	ino_t	inode;
	int     flags;
};

/* ody_close: NO_OUT */
struct ody_close_in {
        struct ody_in_hdr ih;
	struct ofid    fid;
	int	       flags;
};

struct ody_close_out {
	struct ody_out_hdr oh;
};

/* ody_ioctl: */
struct ody_ioctl_in {
        struct ody_in_hdr ih;
	struct ofid  fid;
	int          cmd;
	int          len;
	int          rwflag;
	char         *data;         /* Place holder for data. */
};

struct ody_ioctl_out {
        struct ody_out_hdr oh;
	int       len;
	caddr_t   data;	             /* Place holder for data. */
};

/* ody_getattr: */
struct ody_getattr_in {
        struct ody_in_hdr ih;
	struct ofid fid;
};

struct ody_getattr_out {
        struct ody_out_hdr oh;
	struct ody_vattr attr;
};

/* ody_setattr: NO_OUT */
struct ody_setattr_in {
        struct ody_in_hdr ih;
	struct ofid      fid;
	struct ody_vattr attr;
};

struct ody_setattr_out {
        struct ody_out_hdr oh;
};

/* ody_access: NO_OUT */
struct ody_access_in {
        struct ody_in_hdr ih;
	struct ofid fid;
	int	    flags;
};

struct ody_access_out {
        struct ody_out_hdr oh;
};

/* ody_lookup: */
struct  ody_lookup_in {
        struct ody_in_hdr ih;
	struct ofid fid;
	int        name;	/* Place holder for data. */
};

struct ody_lookup_out {
        struct ody_out_hdr oh;
	struct ofid fid;
	int         vtype;
};

/* ody_create: */
struct ody_create_in {
        struct ody_in_hdr ih;
	struct ofid      fid;
	struct ody_vattr attr;
	int              excl;
	int              mode;
	int              name;	/* Place holder for data. */
};

struct ody_create_out {
        struct ody_out_hdr oh;
	struct ofid      fid;
	struct ody_vattr attr;
};

/* ody_remove: NO_OUT */
struct ody_remove_in {
        struct ody_in_hdr ih;
	struct ofid fid;
	int         name;	/* Place holder for data. */
};

struct ody_remove_out {
        struct ody_out_hdr oh;
};

/* ody_link: NO_OUT */
struct ody_link_in {
        struct ody_in_hdr ih;
	struct ofid sourceFid;    /* onode to link *to* */
	struct ofid destFid;      /* Directory in which to place link */
	int         tname;	  /* Place holder for data. */
};

struct ody_link_out {
        struct ody_out_hdr oh;
};

/* ody_rename: NO_OUT */
struct ody_rename_in {
        struct ody_in_hdr ih;
	struct ofid sourceFid;   /* XXX */
	int         srcname;
	struct ofid destFid;     /* XXX */
	int         destname;
};

struct ody_rename_out {
        struct ody_out_hdr oh;
};

/* ody__mkdir: */
struct ody_mkdir_in {
        struct ody_in_hdr ih;
	struct ofid      fid;
	struct ody_vattr attr;
	int	         name;		/* Place holder for data. */
};

struct ody_mkdir_out {
        struct ody_out_hdr oh;
	struct ofid      fid;
	struct ody_vattr attr;
};

/* ody_rmdir: NO_OUT */
struct ody_rmdir_in {
        struct ody_in_hdr ih;
	struct ofid fid;
	int         name;  /* Place holder for data. */
};

struct ody_rmdir_out {
        struct ody_out_hdr oh;
};

/* ody_readdir: */
struct ody_readdir_in {
        struct ody_in_hdr ih;
	struct ofid fid;
	int         count;
	int         offset;
};

struct ody_readdir_out {
        struct ody_out_hdr oh;
	int     size;
	caddr_t	data;   /* Place holder for data. */
};

/* ody_symlink: NO_OUT */
struct ody_symlink_in {
        struct ody_in_hdr ih;
	struct ofid      fid;          /* Directory to put symlink in */
	int	         srcname;
	struct ody_vattr attr;
	int	         tname;
};

struct ody_symlink_out {
        struct ody_out_hdr oh;
};

/* ody_readlink: */
struct ody_readlink_in {
        struct ody_in_hdr ih;
	struct ofid fid;
};

struct ody_readlink_out {
        struct ody_out_hdr oh;
	int     count;
	caddr_t	data;		/* Place holder for data. */
};

/* ody_fsync: NO_OUT */
struct ody_fsync_in {
        struct ody_in_hdr ih;
	struct ofid fid;
};

struct ody_fsync_out {
        struct ody_out_hdr oh;
};

/* ody__inactive: NO_OUT */
struct ody_inactive_in {
        struct ody_in_hdr ih;
	struct ofid fid;
};

struct ody_inactive_out {
        struct ody_out_hdr oh;
};

/* ody_vget: */
struct ody_vget_in {
        struct ody_in_hdr ih;
	struct ofid fid;
};

struct ody_vget_out {
        struct ody_out_hdr oh;
	struct ofid fid;
	int         vtype;
};


/* ODY_SIGNAL is out-of-band, doesn't need data. */
/* ODY_INVALIDATE is a viceroy->kernel call */
/* ODY_FLUSH is a viceroy->kernel call */

/* ody_purgeuser: */
/* ODY_PURGEUSER is a viceroy->kernel call */
struct ody_purgeuser_out {
        struct ody_out_hdr oh;
	struct ody_cred cred;
};

/* ody_zapfile: */
/* ODY_ZAPFILE is a viceroy->kernel call */
struct ody_zapfile_out { 
        struct ody_out_hdr oh;
	struct ofid fid;
};

/* ody_zapdir: */
/* ODY_ZAPDIR is a viceroy->kernel call */	
struct ody_zapdir_out {
        struct ody_out_hdr oh;
	struct ofid fid;
};

/* ody_zapvnode: */
/* ODY_ZAPVNODE is a viceroy->kernel call */	
struct ody_zapvnode_out {
        struct ody_out_hdr oh;
	struct ody_cred cred;
	struct ofid     fid;
};

/* ody_purgefid: */
/* ODY_PURGEFID is a viceroy->kernel call */	
struct ody_purgefid_out {
        struct ody_out_hdr oh;
	struct ofid fid;
};

/* ody_replace: */
/* ODY_REPLACE is a viceroy->kernel call */	
struct ody_replace_out { 
        struct ody_out_hdr oh;
	struct ofid NewFid;
	struct ofid OldFid;
};

/* ody_bmod_start: */
struct  ody_bmod_start_in {
        struct ody_in_hdr ih;
	int               joules;
	float             charge;
};

struct  ody_bmod_start_out {
	struct ody_out_hdr oh;
};

/* ody_bmod_stop: */
struct  ody_bmod_stop_in {
        struct ody_in_hdr ih;
};

struct  ody_bmod_stop_out {
	struct ody_out_hdr oh;
};

/* ody_time_to_recharge: */
struct  ody_time_to_recharge_in {
        struct ody_in_hdr ih;
	int               seconds;
};

struct  ody_time_to_recharge_out {
	struct ody_out_hdr oh;
};

/* ody_multif: */
struct ody_multif_in {
        struct ody_in_hdr ih;
	unsigned int gs;
	unsigned int type;
	unsigned int ts;
	unsigned int op;
	size_t       argsz;
	size_t       retsz;
	int          data;	/* Place holder for data */
};

struct ody_multif_out {
        struct ody_out_hdr oh;
	size_t       retsz;
	int          data;	/* Place holder for data */
};

/* ody_expand: */
struct  ody_expand_in {
        struct ody_in_hdr ih;
	struct ofid       fid;
	int               name;	/* Place holder for data. */
};

struct ody_expand_out {
        struct ody_out_hdr oh;
	struct ofid        fid;
	int                vtype;
	int                cursor;
};

/* ody_cursexpand: */
struct  ody_cursexpand_in {
        struct ody_in_hdr ih;
	struct ofid       fid;
	int               bufsz;
};

struct ody_cursexpand_out {
        struct ody_out_hdr oh;
        int                retsz;
        int                data;	/* placeholder for data */
};

/* ody_prefetch: */
struct  ody_prefetch_in {
        struct ody_in_hdr ih;
	struct ofid       fid;
	int               flags;
};

struct ody_prefetch_out {
        struct ody_out_hdr oh;
	odev_t	           dev;
	ino_t	           inode;
	int                flags;
};


/* ody_rdwr: */
struct ody_rdwr_in {
        struct ody_in_hdr ih;
	struct ofid fid;
	int         rwflag;
	int         count;
	int         offset;
	int         ioflag;
	int         data;	/* Place holder for data. */	
};

struct ody_rdwr_out {
        struct ody_out_hdr oh;
	int     rwflag;
	int     count;
	int	data;		/* Place holder for data. */
};

/* ody_request: */
struct ody_request_in {
        struct ody_in_hdr ih;
	struct ofid   fid;
	unsigned int  rsrc;
	unsigned int  gs;
	unsigned int  type;
	unsigned int  ts; 
	int           low;
	int           high;
};

struct ody_request_out {
        struct ody_out_hdr oh;
	int  cancelled;
	int  value;
	int  result;
};

/* ody_cancel: NO_OUT */
struct ody_cancel_in {
        struct ody_in_hdr ih;
	int         reqid;
};

struct ody_cancel_out {
        struct ody_out_hdr oh;
};

/* ody_cancpid: NO_OUT */
struct ody_cancpid_in {
	struct ody_in_hdr ih;
};

struct ody_cancpid_out {
	struct ody_out_hdr oh;
};

/* ody_tsop: */
struct ody_tsop_in {
        struct ody_in_hdr ih;
	struct ofid  fid;
	unsigned int gs;
	unsigned int type;
	unsigned int ts;
	unsigned int op;
	size_t       argsz;
	size_t       retsz;
	int          data;	/* Place holder for data */
};

struct ody_tsop_out {
        struct ody_out_hdr oh;
	size_t       retsz;
	int          data;	/* Place holder for data */
};

/* ody_toosmall: NO_OUT */
/*
 * ody_toosmall is really a fake message: it is consed up when (and
 * only when) the viceroy tries to read using a buffer smaller than
 * that needed by the topmost pending call.  Since calls need to be
 * read atomically, we return the size of the buffer, and expect the
 * viceroy to increase it's read request size.
 *
 * This depends on the invariant that only one process can attach per
 * pseudodevice
 */

struct ody_toosmall_in {
        struct ody_in_hdr ih;
	unsigned int  bufsiz;
};

/* 
 * Occasionally, don't cache the fid returned by ODY_LOOKUP. For
 * instance, if the fid is inconsistent. This case is handled by
 * setting the top bit of the return result parameter.  XXX
 */
#define ODY_NOCACHE          0x80000000

union ody_inputArgs {
        struct ody_in_hdr ih;	/* NB: every struct below starts with ih */
        struct ody_open_in       ody_open;
        struct ody_close_in      ody_close;
        struct ody_ioctl_in      ody_ioctl;
        struct ody_getattr_in    ody_getattr;
        struct ody_setattr_in    ody_setattr;
        struct ody_access_in     ody_access;
        struct ody_lookup_in     ody_lookup;
        struct ody_create_in     ody_create;
        struct ody_remove_in     ody_remove;
        struct ody_link_in       ody_link;
        struct ody_rename_in     ody_rename;
        struct ody_mkdir_in      ody_mkdir;
        struct ody_rmdir_in      ody_rmdir;
        struct ody_readdir_in    ody_readdir;
        struct ody_symlink_in    ody_symlink;
        struct ody_readlink_in   ody_readlink;
        struct ody_fsync_in      ody_fsync;
        struct ody_inactive_in   ody_inactive;
        struct ody_vget_in       ody_vget;
        struct ody_expand_in     ody_expand;
        struct ody_cursexpand_in ody_cursexpand;
        struct ody_prefetch_in   ody_prefetch;
        struct ody_rdwr_in       ody_rdwr;
        struct ody_request_in    ody_request;
        struct ody_cancel_in     ody_cancel;
        struct ody_tsop_in       ody_tsop;
        struct ody_toosmall_in   ody_toosmall;
        struct ody_bmod_start_in ody_bmod_start;
        struct ody_bmod_stop_in  ody_bmod_stop;
        struct ody_time_to_recharge_in   ody_time_to_recharge;
        struct ody_multif_in       ody_multif;
};

union ody_outputArgs {
        struct ody_out_hdr oh;
        struct ody_root_out       ody_root;
        struct ody_open_out       ody_open;
        struct ody_ioctl_out      ody_ioctl;
        struct ody_getattr_out    ody_getattr;
        struct ody_lookup_out     ody_lookup;
        struct ody_create_out     ody_create;
        struct ody_mkdir_out      ody_mkdir;
        struct ody_readdir_out    ody_readdir;
        struct ody_readlink_out   ody_readlink;
        struct ody_vget_out       ody_vget;
        struct ody_purgeuser_out  ody_purgeuser;
        struct ody_zapfile_out    ody_zapfile;
        struct ody_zapdir_out     ody_zapdir;
        struct ody_zapvnode_out   ody_zapvnode;
        struct ody_purgefid_out   ody_purgefid;
        struct ody_replace_out    ody_replace;
        struct ody_expand_out     ody_expand;
        struct ody_cursexpand_out ody_cursexpand;
        struct ody_rdwr_out       ody_rdwr;
        struct ody_prefetch_out   ody_prefetch;
        struct ody_request_out    ody_request;
	struct ody_cancel_out     ody_cancel;
	struct ody_cancpid_out    ody_cancpid;
        struct ody_tsop_out       ody_tsop;
        struct ody_bmod_start_out ody_bmod_start;
        struct ody_bmod_stop_out  ody_bmod_stop;
        struct ody_time_to_recharge_out ody_time_to_recharge;
        struct ody_multif_out       ody_multif;
};    

union ody_downcalls {
        /* ODY_INVALIDATE is a venus->kernel call */
        /* ODY_FLUSH is a venus->kernel call */
        struct ody_purgeuser_out purgeuser;
        struct ody_zapfile_out   zapfile;
        struct ody_zapdir_out    zapdir;
        struct ody_zapvnode_out  zapvnode;
        struct ody_purgefid_out  purgefid;
        struct ody_replace_out   replace;
};


/*
 * Used for identifying usage of "Control" and pioctls
 */

#define PIOCPARM_MASK 0x0000ffff
struct OdyViceIoctl {
        caddr_t in, out;        /* Data to be transferred in, or out */
        short in_size;          /* Size of input buffer <= 2K */
        short out_size;         /* Maximum size of output buffer, <= 2K */
};

struct OdyPioctlData {
        const char *path;
        int follow;
        struct OdyViceIoctl vi;
};

#define	ODY_CONTROL		".CONTROL"
#define ODY_CONTROLLEN           8
#define	ODY_CTL_TYPE            -1
#define	ODY_CTL_TOME		-1
#define	ODY_CTL_VNO		-1
#define	ODY_CTL_UNI		-1
#define	ODY_CTL_INO		-1

#define ODY_CTL_FILE            "/ody/.CONTROL"
#define ODY_CTL_MAJOR            0
#define ODY_CTL_MINOR            0



#define	IS_ODY_CTL_FID(fidp)	((fidp)->ofid_ttype = ODY_CTL_TYPE &&\
                                 (fidp)->ofid_tome == ODY_CTL_TOME &&\
				 (fidp)->ofid_vnum == ODY_CTL_VNO &&\
				 (fidp)->ofid_uniq == ODY_CTL_UNI)

#ifdef __FreeBSD__
#ifdef FREEBSD_BZERO_HACK
#define bzero(p,n) ody_bzero(p,n)
#endif
#endif

#endif /* !_ODY_HEADER_ */
