#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * operation stats: what the minicache can intercept that
 * *isn't* seen by venus.  These stats are kept to augment
 * the stats maintained by the Volume-Session mechanism.
 */

#ifndef _ODY_OPSTATS_H_
#define _ODY_OPSTATS_H_

/* vfsops:
 *          mount: not currently bounced to Venus
 *          umount: nope
 *          root: only first call, rest is cached.
 *          statfs: none (bogus)
 *          sync: none (bogus)
 *          vget: all
 */

#define ODY_MOUNT_STATS  0
#define ODY_UMOUNT_STATS 1
#define ODY_ROOT_STATS   2
#define ODY_STATFS_STATS 3
#define ODY_SYNC_STATS   4
#define ODY_VGET_STATS   5
#define ODY_VFSOPS_SIZE  6

/* vnodeops:
 *            open: all to venus
 *            close: all to venus
 *            rdrw: bogus.  Maybe redirected to UFS.
 *                          May call open/close for internal opens/closes
 *                          (Does exec not call open?)
 *            ioctl: causes a lookupname
 *                   passes through
 *            select: can't get there from here.
 *            getattr: can be satsified by cache
 *            setattr: all go through
 *            access: can be satisfied by cache
 *            readlink: can be satisfied by cache
 *            fsync: passes through
 *            inactive: passes through
 *            lookup: can be satisfied by cache
 *            create: passes through
 *            remove: passes through
 *            link: passes through
 *            rename: passes through
 *            mkdir: passes through
 *            rmdir: passes through
 *            symlink: passes through
 *            readdir: may be redirected to UFS
 *                     may cause an "internal" open/close
 */

#define ODY_OPEN_STATS     0
#define ODY_CLOSE_STATS    1
#define ODY_RDWR_STATS     2
#define ODY_IOCTL_STATS    3
#ifdef NetBSD1_3
#define ODY_POLL_STATS     4
#else
#define ODY_SELECT_STATS   4
#endif
#define ODY_GETATTR_STATS  5
#define ODY_SETATTR_STATS  6
#define ODY_ACCESS_STATS   7
#define ODY_READLINK_STATS 8
#define ODY_FSYNC_STATS    9
#define ODY_INACTIVE_STATS 10
#define ODY_LOOKUP_STATS   11
#define ODY_CREATE_STATS   12
#define ODY_REMOVE_STATS   13
#define ODY_LINK_STATS     14
#define ODY_RENAME_STATS   15
#define ODY_MKDIR_STATS    16
#define ODY_RMDIR_STATS    17
#define ODY_SYMLINK_STATS  18
#define ODY_READDIR_STATS  19
#define ODY_EXPAND_STATS   20
#define ODY_CURSEXP_STATS  21
#define ODY_VNODEOPS_SIZE  22


/*
 * I propose the following structres:
 */


struct ody_op_stats {
    int opcode;       /* vfs opcode */
    long entries;     /* number of times call attempted */
    long sat_intrn;   /* number of times call satisfied by cache */
    long unsat_intrn; /* number of times call failed in cache, but
                         was not bounced to venus proper. */
    long gen_intrn;   /* number of times call generated internally */
                      /* (do we need that?) */
};


/*
 * With each call to the minicache, we'll bump the counters whenver
 * a call is satisfied internally (through the cache or through a
 * redirect), and whenever an operation is caused internally.
 * Then, we can add the total operations caught by the minicache
 * to the world-wide totals, and leave a caveat for the specific
 * graphs later.
 */

#endif _ODY_OPSTATS_H_
