#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Define the kernel-exported Odyssey API.  To be used only by the
 * user-level support library and *not* directly by applications.
 * Applications should use odyssey.h
 */

#ifndef _ODYAPI_H_
#define _ODYAPI_H_

/*
 * Registration system calls
 */

struct ody_reqargs {
    unsigned int ora_resource;
    unsigned int ora_gen_intf;
    unsigned int ora_obj_type;
    unsigned int ora_type_intf;
    int          ora_low;
    int          ora_high;
    int          ora_cancelled;   /* OUT */
    int          ora_value;       /* OUT */
    int          ora_result;      /* OUT */
};

#ifndef __FreeBSD__
int
__ody_req  __P((char *path, struct ody_reqargs *ora));

int
__ody_freq  __P((int fd, struct ody_reqargs *ora));

int
__ody_cancel __P((char *path, int reqid));
#endif

/* 
 * Odyssey error codes
 *
 * The Odyssey API extends the set of error codes defined in <errno.h>
 * with return values specific to the two Odyssey calls.  See the
 * manual pages for details.  We define ERRSLOP to be a range big
 * enough to handle most of our extensions, and as a side effect
 * something to give us breathing room from the errno-defined set.
 * I'm not convinced we need any of these, but I'll leave them in for
 * now.
 */

/* Note: In Linux, ELAST is 13013 */
#define ODY_ERRSLOP    50                 
#define ODY_FIRSTERR   (ELAST+ODY_ERRSLOP)

#define ODY_EREFUSE              ODY_FIRSTERR     
#define ODY_EBADVERSION          (ODY_EREFUSE+1)
#define ODY_EBADRSRC             (ODY_EBADVERSION+1)
#define ODY_ENOTINWINDOW         (ODY_EBADRSRC+1)
#define ODY_ENOCODEX             (ODY_ENOTINWINDOW+1)

/*
 * A range that the wardens can use to return errors. 
 */
#define ODY_TS_MINERRNO  (ODY_ENOCODEX+ODY_ERRSLOP) 
#define ODY_TS_MAXERRNO  (ODY_TSR_MINERRNO+ODY_ERRSLOP)


#endif _ODYAPI_H_

