#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Mach Operating System
 * Copyright (c) 1990 Carnegie-Mellon University
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * Stolen shamelessly from the Coda MiniCache
 */

#ifndef _ODYNC_HEADER_
#define _ODYNC_HEADER_

extern int odync_initialized;    /* Set if cache has been initialized */

/*
 * ody constants
 */
#define ODYNC_NAMELEN	15		/* longest name stored in cache */
#define ODYNC_CACHESIZE 256		/* Default cache size */
#define ODYNC_HASHSIZE	64		/* Must be multiple of 2 */

/*
 * Hash function for the primary hash.
 */

/* 
 * First try -- (first + last letters + length + (int)cp) mod size
 * 2nd try -- same, except dir fid.vnode instead of cp
 */

#define ODYNC_HASH(name, namelen, op) \
    ((name[0] + name[namelen-1] + namelen + (int)(op)) & (odync_hashsize-1))

#define ODY_NAMEMATCH(op, name, namelen, dop) \
	((namelen == op->namelen) && (dop == op->dop) && \
	((namelen > ODYNC_NAMELEN) ? \
	         ODY_LONGNAMEMATCH(op, name, namelen, dop) : \
	         ODY_SHORTNAMEMATCH(op, name, namelen, dop)))

#define ODY_LONGNAMEMATCH(op, name, namelen, dop) \
	(bcmp(op->n.longname,name,namelen) == 0)

#define ODY_SHORTNAMEMATCH(op, name, namelen, dop) \
        (bcmp(op->n.name,name,namelen) == 0)

/*
 * Functions to modify the hash and lru chains.
 * insque and remque assume that the pointers are the first thing
 * in the list node, thus the trickery for lru.
 */

#define ODYNC_HSHINS(elem, pred)	insque(elem,pred)
#define ODYNC_HSHREM(elem)		remque(elem)
#define ODYNC_HSHNUL(elem)		(elem)->hash_next = \
					(elem)->hash_prev = (elem)

#define ODYNC_LRUINS(elem, pred)	insque(LRU_PART(elem), LRU_PART(pred))
#define ODYNC_LRUREM(elem)		remque(LRU_PART(elem));
#define ODYNC_LRUGET(lruhead)		LRU_TOP((lruhead).lru_prev)

#define ODYNC_VALID(cncp)	(cncp->dop != (struct onode *)0)
 
#define LRU_PART(cncp)			(struct odycache *) \
				((char *)cncp + (2*sizeof(struct odycache *)))
#define LRU_TOP(cncp)				(struct odycache *) \
			((char *)cncp - (2*sizeof(struct odycache *)))
#define DATA_PART(cncp)				(struct odycache *) \
			((char *)cncp + (4*sizeof(struct odycache *)))
#define DATA_SIZE	(sizeof(struct odycache)-(4*sizeof(struct odycache *)))

#define LONG_NAME(len)  (len > ODYNC_NAMELEN)

#define ONCP_NAME(oncp) (LONG_NAME(oncp->namelen) ? oncp->n.longname : oncp->n.name)

#define CRED_EQ(cred1, cred2) \
        ((cred1->cr_uid == cred2->cr_uid) && (cred1->cr_gid == cred2->cr_gid))

/*
 * Structure for an element in the odyssey Name Cache.
 * NOTE: I use the position of arguments and their size in the
 * implementation of the functions ODYNC_LRUINS, ODYNC_LRUREM, and
 * DATA_PART.
 */

struct odycache {	
	struct odycache	*hash_next,*hash_prev;	/* Hash list */
	struct odycache	*lru_next, *lru_prev;	/* LRU list */
	struct onode	*op;			/* vnode of the file */
	struct onode	*dop;			/* parent's cnode */
	struct ucred	*cred;			/* user credentials */
	union {
		char	name[ODYNC_NAMELEN];	/* segment name */
		char   *longname;               /* long segment name */
	} n;
	int		namelen;		/* length of name */
};

struct	odylru {		/* Start of LRU chain */
	char *dummy1, *dummy2;			/* place holders */
	struct odycache *lru_next, *lru_prev;   /* position of pointers is important */
};


struct odyhash {		/* Start of Hash chain */
	struct odycache *hash_next, *hash_prev; /* NOTE: chain pointers must be first */
        int length;                             /* used for tuning purposes */
};


/* 
 * Symbols to aid in debugging the namecache code. Assumes the existence
 * of the variable odync_debug, which is defined in ody_namecache.c
 */
#define ODYNC_DEBUG(N, STMT)     { if (odync_debug & (1 <<N)) { STMT } }

/* Prototypes of functions exported within ody */
extern void odync_init __P((void));
extern void odync_enter __P((struct onode *, const char *, int,
			     struct ucred *, struct onode *));
extern struct onode *odync_lookup __P((struct onode *, const char *, int,
				       struct ucred *));

extern void odync_zapParentfid __P((struct ofid *, enum dc_status));
extern void odync_zapvnode __P((struct ofid *, struct ucred *,
				enum dc_status));
extern void odync_zapfid __P((struct ofid *, enum dc_status));
extern void odync_zapfile __P((struct onode *, const char *, int));
extern void odync_purge_user __P((ouid_t, enum dc_status));
extern void odync_flush __P((enum dc_status));

extern void print_odync __P((void));
extern void odync_gather_stats __P((void));
extern int  odync_resize __P((int, int, enum dc_status));
extern void odync_name(struct onode *op);

/*
 * Structure to contain statistics on the cache usage
 */

struct odync_statistics {
	unsigned	hits;
	unsigned	misses;
	unsigned	enters;
	unsigned	dbl_enters;
	unsigned	long_name_enters;
	unsigned	long_name_lookups;
	unsigned	long_remove;
	unsigned	lru_rm;
	unsigned	zapPfids;
	unsigned	zapFids;
	unsigned	zapFile;
	unsigned	zapUsers;
	unsigned	Flushes;
	unsigned        Sum_bucket_len;
	unsigned        Sum2_bucket_len;
	unsigned        Max_bucket_len;
	unsigned        Num_zero_len;
	unsigned        Search_len;
};

#define ODYNC_FIND		((u_long) 1)
#define ODYNC_REMOVE		((u_long) 2)
#define ODYNC_INIT		((u_long) 3)
#define ODYNC_ENTER		((u_long) 4)
#define ODYNC_LOOKUP		((u_long) 5)
#define ODYNC_ZAPPFID		((u_long) 6)
#define ODYNC_ZAPFID		((u_long) 7)
#define ODYNC_ZAPVNODE		((u_long) 8)
#define ODYNC_ZAPFILE		((u_long) 9)
#define ODYNC_PURGEUSER		((u_long) 10)
#define ODYNC_FLUSH		((u_long) 11)
#define ODYNC_PRINTODYNC	((u_long) 12)
#define ODYNC_PRINTSTATS	((u_long) 13)
#define ODYNC_REPLACE		((u_long) 14)

#endif /* _ODYNC_HEADER_ */
