#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * onode.h
 *
 * Describes an odyssey vnode
 */

#ifndef _ONODE_H_
#define _ONODE_H_

#include <sys/vnode.h>

/* XXX this is the best thing I can do without hacking the vtagtype into
   the system sources; there is some sort of "guarantee" that the kernel
   never looks at the vtag, so we might be ok */

/* just hope it never conflicts with something else */

#define VT_ODYSSEY 238

#ifdef	__FreeBSD__

/* yuck yuck yuck */
#define vref(x) cvref(x)
extern void cvref(struct vnode *vp);
/* yuck yuck yuck */

/* for the prototype of DELAY() */
#include <machine/clock.h>
#endif

/* Macros to manipulate the queue */
#ifndef INIT_QUEUE
struct queue {
    struct queue *forw, *back;
};

#define INIT_QUEUE(head)                     \
do {                                         \
    (head).forw = (struct queue *)&(head);   \
    (head).back = (struct queue *)&(head);   \
} while (0)

#define GETNEXT(head) (head).forw

#define EMPTY(head) ((head).forw == &(head))

#define EOQ(el, head) ((struct queue *)(el) == (struct queue *)&(head))
		   
#define INSQUE(el, head)                             \
do {                                                 \
	(el).forw = ((head).back)->forw;             \
	(el).back = (head).back;                     \
	((head).back)->forw = (struct queue *)&(el); \
	(head).back = (struct queue *)&(el);         \
} while (0)

#define REMQUE(el)                         \
do {                                       \
	((el).forw)->back = (el).back;     \
	(el).back->forw = (el).forw;       \
}  while (0)

#endif INIT_QUEUE


#define ODY_CACHESIZE 512

#define ODY_ALLOC(ptr, cast, size)                                        \
do {                                                                      \
    ptr = (cast)malloc((unsigned long) size, M_ODYSSEY, M_WAITOK);        \
    if (ptr == 0) {                                                       \
	panic("kernel malloc returns 0 at %s:%d\n", __FILE__, __LINE__);  \
    }                                                                     \
} while (0)

#define ODY_FREE(ptr, size)  free((ptr),M_ODYSSEY)

/*
 * global cache state control
 */
extern int odync_use;

/*
 * Used to select debugging statements throughout the ody code.
 */
extern int odydebug;
extern int odync_debug;
extern int ody_printf_delay;
extern int ody_vnop_print_entry;
extern int ody_psdev_print_entry;

#define ODYDBGMSK(N)            (1 << N)
#define ODYDEBUG(N, STMT)       { if (odydebug & ODYDBGMSK(N)) { STMT } }
#define myprintf(args)          \
do {                            \
    if (ody_printf_delay)       \
	DELAY(ody_printf_delay);\
    printf args ;               \
} while (0)

#ifdef __STDC__
#define COMPLAIN_BITTERLY(OP, FID)                             \
do {                                                           \
    myprintf(("Acck! %s on dead vnode %lx.%lx.%lx.%lx attempted ", \
              #OP, (FID).ofid_ttype, (FID).ofid_tome,          \
	      (FID).ofid_vnum, (FID).ofid_uniq));              \
    myprintf(("by Process %d (parent %d)\n",                   \
	      curproc->p_pid,                                  \
	      curproc->p_pptr->p_pid));                        \
} while (0)
#else
#define COMPLAIN_BITTERLY(OP, FID)                             \
do {                                                           \
    myprintf(("Acck! op on dead vnode %lx.%lx.%lx.%lx attempted ", \
              (FID).ofid_ttype, (FID).ofid_tome,               \
	      (FID).ofid_vnum, (FID).ofid_uniq));              \
    myprintf(("by Process %d (parent %d)\n",                   \
	      curproc->p_pid,                                  \
	      curproc->p_pptr->p_pid));                        \
} while (0)
#endif


struct onode_base {
};

struct onode {
    struct vnode     *o_vnode;	/* vnode for this onode */
    u_int32_t         o_flags;	/* Flags with meaning both to kernel/viceory */
    struct ofid       o_fid;	/* ofid of this onode */
    struct vnode     *o_ovp;	/* vnode of (open) cache file */
    u_int16_t         o_ocount;	/* number of openers */
    u_int16_t         o_owrite;	/* number open for write */
    struct vattr      o_vattr;	/* cached attributes */
    char             *o_symlink; /* Pointer to symlink name */
    size_t            o_symlen;	/* Length of same */
    dev_t             o_device;	/* Device cached file resides on */
    ino_t             o_inode;	/* Inode of cached file */
    struct onode     *o_next;	/* links if on NetBSD machine */
};
#define VTOO(vp)        ((struct onode *)(vp)->v_data)
#define OTOV(op)        ((struct vnode *)(op)->o_vnode)

/* Flags that we know about: some kernel, some not. Don't let them overlap */

#define ON_VATTR         0x1    /* valid vattr */
#define ON_SYMLINK       0x2    /* valid symlink */
#define ON_LOCKED        0x8    /* lock held */
#define ON_UNMOUNTING    0x10   /* being unmounted, but not dead yet */
#define ON_PURGING       0x20   /* Purging a fid (used on fsync) */

/* shared with viceroy (viceroy_onode.h) */
#define ON_WANTED        0x40   /* lock wanted */
#define ON_UNCACHEABLE   0x80   /* Cannot use ufs file as cache */
#define ON_CURSOR        0x100  /* Dynamic Sets style cursor */

/* Convenient macros for testing flags bigs */

#define VALID_VATTR(op)         ((op)->o_flags & ON_VATTR)
#define VALID_SYMLINK(op)       ((op)->o_flags & ON_SYMLINK)
#define IS_UNMOUNTING(op)       ((op)->o_flags & ON_UNMOUNTING)
#define IS_UNCACHEABLE(op)      ((op)->o_flags & ON_UNCACHEABLE)
#define IS_CURSOR(op)           ((op)->o_flags & ON_CURSOR)

struct ocomm {
	u_long		oc_seq;
	struct selinfo	oc_selproc;
	struct queue	oc_requests;
	struct queue	oc_replys;
};

#define	OC_OPEN(ocp)	     ((ocp)->oc_requests.forw != NULL)
#define MARK_OC_CLOSED(ocp)  (ocp)->oc_requests.forw = NULL
#define MARK_OC_OPEN(ocp)    /* MT odydev_open sets this */

struct ody_clstat {
	int	ncalls;			/* client requests */
	int	nbadcalls;		/* upcall failures */
	int	reqs[ODY_NCALLS];	/* count of each request */
};
extern struct ody_clstat ody_clstat;

/*
 * ODY structure to hold mount/file system information
 */
struct ody_mntinfo {
        struct vnode 	   *mi_rootvp;
	struct mount       *mi_vfsp;
	struct ocomm        mi_ocomm;
};
extern struct ody_mntinfo ody_mnttbl[]; /* indexed by minor device number */

/*
 * vfs pointer to mount info
 */
#define vftomi(vfsp)    ((struct ody_mntinfo *)((vfsp)->mnt_data))
#define	ODY_MOUNTED(vfsp)   (vftomi((vfsp)) != (struct ody_mntinfo *)0)

/*
 * vnode pointer to mount info
 */
#define vtomi(vp)       ((struct ody_mntinfo *)((vp)->v_mount->mnt_data))

/*
 * Used for identifying usage of "Control" object
 */
extern struct vnode *ody_ctlvp;
#define	IS_ODY_CTL_VP(vp)		((vp) == ody_ctlvp)
#define	IS_ODY_CTL_NAME(vp, name, l)((l == ODY_CONTROLLEN) \
 				 && ((vp) == vtomi((vp))->mi_rootvp) \
				 && strncmp(name, ODY_CONTROL, l) == 0)

/* 
 * An enum to tell us whether something that will remove a reference
 * to a cnode was a downcall or not
 */
enum dc_status {
    IS_DOWNCALL = 6,
    NOT_DOWNCALL = 7
};

/* ody_psdev.h */
int odycall __P((struct ody_mntinfo   *mntinfo,
		 int                  inSize,
		 int                  *outSize,
		 caddr_t              buffer));

/* ody_subr.h */
int  ody_downcall   __P((int                  opcode,
			 union ody_outputArgs *out));
void ody_unmounting __P((struct mount *whoIam));
int  ody_vmflush    __P((struct onode *op));

/* ody_vnodeops.h */
struct onode *makeodynode __P((struct ofid *fid,
			       struct mount *vfsp,
			       short type));
int ody_vnodeopstats_init __P((void));

/* ody_vfsops.h */
struct mount *ody_devtomp __P((dev_t));

#ifndef	NetBSD1_3
#define __RCSID(x) static char *rcsid = x
#endif

#endif /* _ONODE_H_ */
