#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Cache operations for Coda.
 * For Linux 2.1: (C) 1997 Carnegie Mellon University
 *
 * Carnegie Mellon encourages users of this code to contribute improvements
 * to the Coda project. Contact Peter Braam <coda@cs.cmu.edu>.
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/string.h>
#include <linux/list.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>

static void ody_ccinsert(struct ody_cache *el, struct super_block *sb);
static void ody_cninsert(struct ody_cache *el, struct ody_inode_info *oii);
static void ody_ccremove(struct ody_cache *el);
static void ody_cnremove(struct ody_cache *el);
static void ody_cache_create(struct inode *inode, int mask);
static struct ody_cache * ody_cache_find(struct inode *inode);


/* Keep various stats */
struct odync_statistics odync_stat;

/* insert a acl-cache entry in sb list */
static void ody_ccinsert(struct ody_cache *el, struct super_block *sb)
{
	struct ody_sb_info *sbi = ody_sbp(sb);
	ENTRY;
	/* third test verifies cc was initialized before adding it 
	   to the sblist. Probably superfluous */
	if ( !sbi || !el || ! list_empty(&el->oc_cclist) ) {
		printk("ody_ccinsert: NULL sbi or el->oc_cclist not empty!\n");
		return ;
	}

	list_add(&el->oc_cclist, &sbi->sbi_cchead);
}

/* insert a acl-cache entry in the inode list */
static void ody_cninsert(struct ody_cache *el, struct ody_inode_info *oii)
{
	ENTRY;
	if ( !oii || !el || ! list_empty(&el->oc_cnlist) ) {
		printk("ody_cninsert: NULL oii or el->oc_cnlist not empty!\n");
		return ;
	}
	list_add(&el->oc_cnlist, &oii->o_cnhead);
}

/* remove a cache entry from the superblock list */
static void ody_ccremove(struct ody_cache *el)
{
	ENTRY;
        if ( ! list_empty(&el->oc_cclist) )
	        list_del(&el->oc_cclist);
	else
		printk("ody_cnremove: loose cc entry!");
}

/* remove a cache entry from the inode's list */
static void ody_cnremove(struct ody_cache *el)
{
	ENTRY;
	if ( ! list_empty(&el->oc_cnlist) )
		list_del(&el->oc_cnlist);
	else
		printk("ody_cnremove: loose cn entry!");
}

/* create a new cache entry and enlist it */
static void ody_cache_create(struct inode *inode, int mask)
{
	struct ody_inode_info *oii = ITOO(inode);
	struct super_block *sb = inode->i_sb;
	struct ody_cache *oc = NULL;
	ENTRY;

	ODY_ALLOC(oc, struct ody_cache *, sizeof(*oc));
	INIT_LIST_HEAD(&oc->oc_cclist);
	INIT_LIST_HEAD(&oc->oc_cnlist);

	if ( !oc ) {
		printk("Out of memory in ody_cache_enter!\n");
		return;
	}
	ody_load_creds(&oc->oc_cred);
	oc->oc_mask = mask;
	ody_cninsert(oc, oii);
	ody_ccinsert(oc, sb);
	ODEBUG(D_CACHE, "create with mask %d\n", mask);
}

/* see if there is a match for the current 
   credentials already */
static struct ody_cache * ody_cache_find(struct inode *inode)
{
	struct ody_inode_info *oii = ITOO(inode);
	struct list_head *lh, *le;
	struct ody_cache *oc = NULL;
	
	le = lh = &oii->o_cnhead;
	while( (le = le->next ) != lh )  {
		/* compare name and creds */
		oc = list_entry(le, struct ody_cache, oc_cnlist);
		if ( !ody_cred_ok(&oc->oc_cred) )
			continue;
		ODEBUG(D_CACHE, "HIT for ino %ld\n", inode->i_ino );
		return oc; /* cache hit */
	}
		return NULL;
}

/* create or extend an acl cache hit */
void ody_cache_enter(struct inode *inode, int mask)
{
	struct ody_cache *oc;

	oc = ody_cache_find(inode);

	if ( oc ) {
		oc->oc_mask |= mask;
		ODEBUG(D_CACHE, "updating mask to %d\n", oc->oc_mask);
	} else {
		ody_cache_create(inode, mask);
	}
}

/* remove all cached acl matches from an inode */
void ody_cache_clear_inode(struct inode *inode)
{
	struct list_head *lh, *le;
	struct ody_inode_info *oii;
	struct ody_cache *oc;
	ENTRY;

	if ( !inode ) {
		ODEBUG(D_CACHE, "ody_cache_clear_inode: NULL inode\n");
		return;
	}
	oii = ITOO(inode);
	
	lh = le = &oii->o_cnhead;
	while ( (le = le->next ) != lh ) {
		oc = list_entry(le, struct ody_cache, oc_cnlist);
		ody_cnremove(oc);
		ody_ccremove(oc);
		ODY_FREE(oc, sizeof(*oc));
	}
}

/* remove all acl caches */
void ody_cache_clear_all(struct super_block *sb)
{
	struct list_head *lh, *le;
	struct ody_cache *oc;
	struct ody_sb_info *sbi = ody_sbp(sb);

	ENTRY;

	if ( !sbi ) {
		printk("ody_cache_clear_all: NULL sbi\n");
		return;
	}
	
	if ( list_empty(&sbi->sbi_cchead) )
		return;

	lh = le = &sbi->sbi_cchead;
	while ( (le = le->next ) != lh ) {
		oc = list_entry(le, struct ody_cache, oc_cclist);
		ody_cnremove(oc);
		ody_ccremove(oc);
		ODY_FREE(oc, sizeof(*oc));
	}

	EXIT;
}

/* remove all acl caches for a principal */
void ody_cache_clear_cred(struct super_block *sb, struct ody_cred *cred)
{
	struct list_head *lh, *le;
	struct ody_cache *oc;
	struct ody_sb_info *sbi = ody_sbp(sb);

	if ( !sbi ) {
		printk("ody_cache_clear_all: NULL sbi\n");
		return;
	}
	
	if (list_empty(&sbi->sbi_cchead))
		return;

	lh = le = &sbi->sbi_cchead;
	while ( (le = le->next ) != lh ) {
		oc = list_entry(le, struct ody_cache, oc_cclist);
		if ( ody_cred_eq(&oc->oc_cred, cred)) {
			ody_cnremove(oc);
			ody_ccremove(oc);
			ODY_FREE(oc, sizeof(*oc));
		}
	}
}


/* check if the mask has been matched against the acl
   already */
int ody_cache_check(struct inode *inode, int mask)
{
	struct ody_inode_info *oii = ITOO(inode);
	struct list_head *lh, *le;
	struct ody_cache *oc = NULL;
	
	le = lh = &oii->o_cnhead;
	while( (le = le->next ) != lh )  {
		/* compare name and creds */
		oc = list_entry(le, struct ody_cache, oc_cnlist);
		if ( (oc->oc_mask & mask) != mask ) {
			ODEBUG(D_CACHE, "mask missed: %d\n", mask);
			continue; 
		}
		if ( !ody_cred_ok(&oc->oc_cred) ) {
			ODEBUG(D_CACHE, "missed because of cred (%ld != %d)\n",
			       oc->oc_cred.ocr_fsuid, current->fsuid);
			continue;
		}
		ODEBUG(D_CACHE, "HIT for ino %lx\n", inode->i_ino );
		return 1; /* cache hit */
	}
		ODEBUG(D_CACHE, "MISS for ino %lx\n", inode->i_ino );
		return 0;
}


/* Purging dentries and children */
/* The following routines drop dentries which are not
   in use and flag dentries which are in use to be 
   zapped later.

   The flags are detected by:
   - ody_dentry_revalidate (for lookups) if the flag is ON_PURGE
   - ody_dentry_delete: to remove dentry from the cache when d_count
     falls to zero
   - an inode method ody_revalidate (for attributes) if the 
     flag is ON_VATTR
*/

/* 
   Some of this is pretty scary: what can disappear underneath us?
   - shrink_dcache_parent calls on purge_one_dentry which is safe:
     it only purges children.
   - dput is evil since it  may recurse up the dentry tree
 */

void ody_purge_dentries(struct inode *inode)
{
	struct list_head *tmp, *head = &inode->i_dentry;

	if (!inode)
		return ;

	/* better safe than sorry: dput could kill us */
	iget(inode->i_sb, inode->i_ino);
	/* catch the dentries later if some are still busy */
	ody_flag_inode(inode, ON_PURGE);

restart:
	tmp = head;
	while ((tmp = tmp->next) != head) {
		struct dentry *dentry = list_entry(tmp, struct dentry, d_alias);
		if (!dentry->d_count) {
			ODEBUG(D_DOWNCALL, 
			       "ody_free_dentries: freeing %s/%s, i_count=%d\n",
			       dentry->d_parent->d_name.name, dentry->d_name.name, 
			       inode->i_count);
			dget(dentry);
			d_drop(dentry);
			dput(dentry);
			goto restart;
		}
			
	}
	iput(inode);
}

/* this won't do any harm: just flag all children */
static void ody_flag_children(struct dentry *parent, int flag)
{
	struct list_head *child;
	struct dentry *de;

	child = parent->d_subdirs.next;
	while ( child != &parent->d_subdirs ) {
		de = list_entry(child, struct dentry, d_child);
		child = child->next;
		/* don't know what to do with negative dentries */
		if ( ! de->d_inode ) 
			continue;
		ODEBUG(D_DOWNCALL, "%d for %*s/%*s\n", flag, 
		       de->d_name.len, de->d_name.name, 
		       de->d_parent->d_name.len, de->d_parent->d_name.name);
		ody_flag_inode(de->d_inode, flag);
	}
	return; 
}

void ody_purge_children(struct inode *inode)
{
	struct list_head *alias;
	struct dentry *alias_de;

	if ( !inode ) 
		return; 

	if (list_empty(&inode->i_dentry))
	        return; 

	/* I believe that shrink_dcache_parent will not
	   remove dentries from the alias list. If it 
	   does we are toast. 
	*/
	alias = inode->i_dentry.next; 
	while ( alias != &inode->i_dentry ) {
		alias_de = list_entry(alias, struct dentry, d_alias);
		ody_flag_children(alias_de, ON_PURGE);
		shrink_dcache_parent(alias_de);
		alias = alias->next;
	}
}

void ody_flag_inode(struct inode *inode, int flag)
{
	struct ody_inode_info *oii;

	if ( !inode ) {
		ODEBUG(D_CACHE, " no inode!\n");
		return;
	}

	oii = ITOO(inode);
	oii->o_flags |= flag;
}		

int
odync_nc_info(char *buffer, char **start, off_t offset, int length, int dummy)
{
        int len=0;
        off_t begin;
	
	/* 	odync_gather_stats(); */

	/* this works as long as we are below 1024 characters! */    
	len += sprintf(buffer,"Ody minicache statistics\n\n");
	len += sprintf(buffer+len, "odync_hits : %d\n", odync_stat.hits);
	len += sprintf(buffer+len, "odync_misses : %d\n", odync_stat.misses);
	len += sprintf(buffer+len, "odync_enters : %d\n", odync_stat.enters);
	len += sprintf(buffer+len, "odync_dbl_enters : %d\n", odync_stat.dbl_enters);
	len += sprintf(buffer+len, "odync_long_name_enters : %d\n", odync_stat.long_name_enters);
	len += sprintf(buffer+len, "odync_long_name_lookups : %d\n", odync_stat.long_name_lookups);
	len += sprintf(buffer+len, "odync_long_remove : %d\n", odync_stat.long_remove);
	len += sprintf(buffer+len, "odync_lru_rm : %d\n", odync_stat.lru_rm);
	len += sprintf(buffer+len, "odync_zapPfids : %d\n", odync_stat.zapPfids);
	len += sprintf(buffer+len, "odync_zapFids : %d\n", odync_stat.zapFids);
	len += sprintf(buffer+len, "odync_zapFile : %d\n", odync_stat.zapFile);
	len += sprintf(buffer+len, "odync_zapUsers : %d\n", odync_stat.zapUsers);
	len += sprintf(buffer+len, "odync_Flushes : %d\n", odync_stat.Flushes);
	len += sprintf(buffer+len, "odync_SumLen : %d\n", odync_stat.Sum_bucket_len);
	len += sprintf(buffer+len, "odync_Sum2Len : %d\n", odync_stat.Sum2_bucket_len);
	len += sprintf(buffer+len,  "odync_# 0 len : %d\n", odync_stat.Num_zero_len);
	len += sprintf(buffer+len,  "odync_MaxLen : %d\n", odync_stat.Max_bucket_len);
	len += sprintf(buffer+len,  "odync_SearchLen : %d\n", odync_stat.Search_len);
       	begin =  offset;
       	*start = buffer + begin;
       	len -= begin;
	
        if(len>length)
                len = length;
	if (len< 0)
		len = 0;
        return len;
} 
