#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * File operations for Coda.
 * Original version: (C) 1996 Peter Braam 
 * Rewritten for Linux 2.1: (C) 1997 Carnegie Mellon University
 *
 * Carnegie Mellon encourages users of this code to contribute improvements
 * to the Coda project. Contact Peter Braam <coda@cs.cmu.edu>.
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <linux/string.h>
#include <asm/uaccess.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>

/* file operations */
static int ody_readpage(struct file *file, struct page * page);
static ssize_t ody_file_read(struct file *f, char *buf, size_t count, loff_t *off);
static ssize_t ody_file_write(struct file *f, const char *buf, size_t count, loff_t *off);
static int ody_file_mmap(struct file * file, struct vm_area_struct * vma);

/* also exported from this file (used for dirs) */
int ody_fsync(struct file *, struct dentry *dentry);

struct inode_operations ody_file_inode_operations = {
	&ody_file_operations,	/* default file operations */
	NULL,			/* create */
	NULL,		        /* lookup */
	NULL,			/* link */
	NULL,		        /* unlink */
	NULL,			/* symlink */
	NULL,			/* mkdir */
	NULL,			/* rmdir */
	NULL,			/* mknod */
	NULL,		        /* rename */
	NULL,			/* readlink */
	NULL,			/* follow_link */
	ody_readpage,    	/* readpage */
	NULL,			/* writepage */
	NULL,			/* bmap */
	NULL,			/* truncate */
        ody_permission,        /* permission */
	NULL,                   /* smap */
	NULL,                   /* update page */
        ody_revalidate_inode   /* revalidate */
};

struct file_operations ody_file_operations = {
	NULL,		        /* lseek - default should work for ody */
	ody_file_read,		/* read */
	ody_file_write,		/* write */
	NULL,          		/* readdir */
	NULL,			/* select - default */
	NULL,		        /* ioctl */
	ody_file_mmap,		/* mmap */
	ody_open,		/* open */
	NULL,                   /* flush */
	ody_release,		/* release */
	ody_fsync,		/* fsync */
	NULL,                   /* fasync */
	NULL,                   /* check_media_change */
	NULL,                   /* revalidate */
	NULL                    /* lock */
};

/*  File file operations */
static int ody_readpage(struct file * ody_file, struct page * page)
{
	struct dentry *de = ody_file->f_dentry;
	struct inode *ody_inode = de->d_inode;
	struct dentry cont_dentry;
	struct file cont_file;
        struct ody_inode_info *oii;

        ENTRY;
        
        oii = ITOO(ody_inode);

        if ( ! oii->o_ovp ) {
		if (IS_UNCACHEABLE(oii))
			printk("ody_readpage: called on UNCACHEABLE onp!\n");
		printk("ody_readpage: no open inode for ino %ld, %s\n", 
		       ody_inode->i_ino, de->d_name.name);
                return -ENXIO;
        }
       
        ody_prepare_openfile(ody_inode, ody_file, oii->o_ovp,
			     &cont_file, &cont_dentry);

        ODEBUG(D_INODE, "ody ino: %ld, cached ino %ld, page offset: %lx\n", 
	       ody_inode->i_ino, oii->o_ovp->i_ino, page->offset);

        generic_readpage(&cont_file, page);
        EXIT;
        return 0;
}

static int ody_file_mmap(struct file * file, struct vm_area_struct * vma)
{
        struct ody_inode_info *oii;
	int res;

        ENTRY;
	oii = ITOO(file->f_dentry->d_inode);
	oii->o_mmcount++;
  
	res =generic_file_mmap(file, vma);
	EXIT;
	return res;
}

static ssize_t ody_file_read(struct file *ody_file, char *buff, 
			     size_t count, loff_t *ppos)
{
        struct ody_inode_info *onp;
	struct inode *ody_inode = ody_file->f_dentry->d_inode;
        struct inode *cont_inode = NULL;
        struct file  cont_file;
	struct dentry cont_dentry;
        int result = 0;
        ENTRY;

        onp = ITOO(ody_inode);
        CHECK_ONODE(onp);
	
	if (!IS_UNCACHEABLE(onp)) {
		cont_inode = onp->o_ovp;
		if ( cont_inode == NULL ) {
			printk("ody_file_read: cached inode is 0!\n");
			return -1;
		}
		
		ody_prepare_openfile(ody_inode, ody_file, cont_inode, 
				     &cont_file, &cont_dentry);
		
		if (!cont_file.f_op || ! cont_file.f_op->read) { 
			printk( "ody_file_read: container file has no read in file operations.\n");
			return -1;
		}
		
		result = cont_file.f_op->read(&cont_file, buff, count, 
					      &(cont_file.f_pos));
		
		ODEBUG(D_FILE, "ops at %x result %d, count %d, position: %d\n",
		       (int)cont_file.f_op, result, count,
		       (int)cont_file.f_pos);
		
		ody_restore_odyfile(ody_inode, ody_file, cont_inode,
				    &cont_file);
	} else {
		size_t bytes_read = 0;
		size_t left = count;
		size_t len;
		int ioflag = (ody_file->f_flags & O_NONBLOCK) ? O_IO_NDELAY:0;
		while (left > 0) {
			len = (left > OC_MAXDATASIZE) ? OC_MAXDATASIZE : left;
			ODEBUG(D_FILE, "(%s): %d bytes at offset %d\n",
			       ody_f2s(&onp->o_fid), len,
			       (int)ody_file->f_pos);
			result = viceroy_rdwr(ody_inode->i_sb, &(onp->o_fid),
					      &(ody_file->f_pos),
					      ioflag, O_RW_READ, &len,
					      buff);

			if (result < 0)
				return (bytes_read ? bytes_read : result);

			if (len == 0)
				left = 0;
			else {
				buff += len;
				bytes_read += len;
				left -= len;
			}
		}
		result = bytes_read;
	}
        return result;
}


static ssize_t ody_file_write(struct file *ody_file, const char *buff, 
			    size_t count, loff_t *ppos)
{
        struct ody_inode_info *onp;
	struct inode *ody_inode = ody_file->f_dentry->d_inode;
        struct inode *cont_inode = NULL;
        struct file  cont_file;
	struct dentry cont_dentry;
        int result = 0;

        ENTRY;

        onp = ITOO(ody_inode);
        CHECK_ONODE(onp);

	if (!IS_UNCACHEABLE(onp)) {
		cont_inode = onp->o_ovp;
		if ( cont_inode == NULL ) {
			printk("ody_file_write: cached inode is 0!\n");
			return -1; 
		}

		ody_prepare_openfile(ody_inode, ody_file, cont_inode, 
				     &cont_file, &cont_dentry);
		
		if (!cont_file.f_op || !cont_file.f_op->write) {
			printk("ody_file_write: container file has no file ops.\n");
			return -1;
		}
		
		down(&cont_inode->i_sem);
		result = cont_file.f_op->write(&cont_file , buff, count, 
					       &(cont_file.f_pos));
		up(&cont_inode->i_sem);
		ody_restore_odyfile(ody_inode, ody_file, cont_inode,
				    &cont_file);
		if (result)
			onp->o_flags |= ON_VATTR;
	} else {
		size_t bytes_wrote = 0;
		size_t left = count;
		size_t len;
		int ioflag = (ody_file->f_flags & O_NONBLOCK) ? O_IO_NDELAY:0;
		ODEBUG(D_FILE, "start\n");
		while (left > 0) {
			len = (left > OC_MAXDATASIZE) ? OC_MAXDATASIZE : left;
			ODEBUG(D_FILE, "(%s): %d bytes\n",
			       ody_f2s(&onp->o_fid), len);
			result = viceroy_rdwr(ody_inode->i_sb, &(onp->o_fid),
					      &(ody_file->f_pos),
					      ioflag, O_RW_WRITE, &len,
					      (char *)buff);
			if (result < 0)
				return -1;

			buff += len;
			bytes_wrote += len;
			left -= len;
		}
		result = bytes_wrote;
	}
        return result;
}

int ody_fsync(struct file *ody_file, struct dentry *ody_dentry)
{
        struct ody_inode_info *onp;
	struct inode *ody_inode = ody_dentry->d_inode;
        struct inode *cont_inode = NULL;
        struct file  cont_file;
	struct dentry cont_dentry;
        int result = 0;
        ENTRY;

	if (!(S_ISREG(ody_inode->i_mode) || S_ISDIR(ody_inode->i_mode) ||
	      S_ISLNK(ody_inode->i_mode)))
		return -EINVAL;

        onp = ITOO(ody_inode);
        CHECK_ONODE(onp);

        cont_inode = onp->o_ovp;
        if ( cont_inode == NULL ) {
		if (IS_UNCACHEABLE(onp))
			panic("ody_fsync: called on UNCACHEABLE onp\n");
		else
			printk("ody_fsync: cached inode is 0!\n");
                return -1; 
        }

        ody_prepare_openfile(ody_inode, ody_file, cont_inode, 
			     &cont_file, &cont_dentry);

	down(&cont_inode->i_sem);

        result = file_fsync(&cont_file ,&cont_dentry);
	if ( result == 0 ) {
		result = viceroy_fsync(ody_inode->i_sb, &(onp->o_fid));
	}

	up(&cont_inode->i_sem);

        ody_restore_odyfile(ody_inode, ody_file, cont_inode, &cont_file);
        return result;
}
/* 
 * support routines
 */

/* instantiate the container file and dentry object to do io */                
void ody_prepare_openfile(struct inode *i, struct file *ody_file, 
			  struct inode *cont_inode, struct file *cont_file,
			  struct dentry *cont_dentry)
{
        cont_file->f_pos = ody_file->f_pos;
        cont_file->f_mode = ody_file->f_mode;
        cont_file->f_flags = ody_file->f_flags;
        cont_file->f_count  = ody_file->f_count;
        cont_file->f_owner  = ody_file->f_owner;
	cont_file->f_op = cont_inode->i_op->default_file_ops;
	cont_file->f_dentry = cont_dentry;
        cont_file->f_dentry->d_inode = cont_inode;
        return ;
}

/* update the Ody file & inode after I/O */
void ody_restore_odyfile(struct inode *ody_inode, struct file *ody_file, 
			 struct inode *open_inode, struct file *open_file)
{
        ody_file->f_pos = open_file->f_pos;
	/* XXX what about setting the mtime here too? */
	/* ody_inode->i_mtime = open_inode->i_mtime; */
	ody_inode->i_size = open_inode->i_size;
        return;
}

/* grab the ext2 inode of the container file */
int ody_inode_grab(dev_t dev, ino_t ino, struct inode **ind)
{
        struct super_block *sbptr;

        sbptr = get_super(dev);

        if ( !sbptr ) {
                printk("ody_inode_grab: ody_find_super returns NULL.\n");
                return -ENXIO;
        }
                
        *ind = NULL;
        *ind = iget(sbptr, ino);

        if ( *ind == NULL ) {
                printk("ody_inode_grab: iget(dev: %d, ino: %ld) 
                       returns NULL.\n", dev, ino);
                return -ENOENT;
        }
	ODEBUG(D_FILE, "ino: %ld, ops at %x\n", ino, (int)(*ind)->i_op);
        return 0;
}

