#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Super block/filesystem wide operations
 *
 * Copryright (C) 1996 Peter J. Braam <braam@maths.ox.ac.uk> and 
 * Michael Callahan <callahan@maths.ox.ac.uk> 
 * 
 * Rewritten for Linux 2.1.  Peter Braam <braam@cs.cmu.edu>
 * Copyright (C) Carnegie Mellon University
 */

/*
 * stolen for Odyssey
 */

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/unistd.h>

#include <asm/system.h>
#include <asm/uaccess.h>

#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/string.h>
#include <asm/uaccess.h>
#include <linux/vmalloc.h>
#include <asm/segment.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>
#include <linux/ody_sets.h>

#include "kernel/sets/common/sets_internal.h"
#include "kernel/sets/common/sets_domain.h"

/* VFS super_block ops */
static struct super_block *ody_read_super(struct super_block *, void *, int);
static void ody_read_inode(struct inode *);
static int  ody_notify_change(struct dentry *dentry, struct iattr *attr);
static void ody_put_inode(struct inode *);
static void ody_delete_inode(struct inode *);
static void ody_put_super(struct super_block *);
static int ody_statfs(struct super_block *sb, struct statfs *buf, 
		      int bufsiz);

/* helper functions */
static struct ocomm *ody_psinode2ocomm(struct inode *inode);
static int ody_get_psdev(void *, struct inode **);
static struct ody_sb_info *ody_psinode2sbi(struct inode *inode);

/* exported operations */
struct super_operations ody_super_operations =
{
	ody_read_inode,         /* read_inode */
	NULL,                   /* write_inode */
	ody_put_inode,	        /* put_inode */
	ody_delete_inode,       /* delete_inode */
	ody_notify_change,	/* notify_change */
	ody_put_super,	        /* put_super */
	NULL,			/* write_super */
	ody_statfs,   		/* statfs */
	NULL,			/* remount_fs */
	NULL,			/* clear_inode */
	NULL			/* umount_begin */
};

/*
 * globals
 */
struct ody_sb_info ody_super_info[MAX_ODYDEVS];


static struct super_block * ody_read_super(struct super_block *sb, 
					   void *data, int silent)
{
        struct inode *psdev = 0, *root = 0; 
	struct ody_sb_info *sbi = NULL;
	struct ocomm *oc = NULL;
        struct ofid fid;
	kdev_t dev = sb->s_dev;
        int error;

	ENTRY;
        MOD_INC_USE_COUNT; 
        if (ody_get_psdev(data, &psdev))
                goto error;

        oc = ody_psinode2ocomm(psdev);
        if ( !oc )
	        goto error;
	oc->oc_sb = sb;
	oc->oc_inuse = 1;

	sbi = ody_psinode2sbi(psdev);
	if ( !sbi )
	        goto error;
        sbi->sbi_psdev = psdev;
	sbi->sbi_ocomm = oc;
	INIT_LIST_HEAD(&(sbi->sbi_cchead));

        lock_super(sb);
        sb->u.generic_sbp = sbi;
        sb->s_blocksize = 1024;	/* XXXXX  what do we put here?? */
        sb->s_blocksize_bits = 10;
        sb->s_magic = ODY_SUPER_MAGIC;
        sb->s_dev = dev;
        sb->s_op = &ody_super_operations;

	/* get root fid from Venus: this needs the root inode */
	error = viceroy_rootfid(sb, &fid);
	if ( error ) {
	        printk("ody_read_super: ody_get_rootfid failed with %d\n",
		       error);
		sb->s_dev = 0;
	        unlock_super(sb);
		goto error;
	}	  
	printk("ody_read_super: rootfid is %s\n", ody_f2s(&fid));
	
	/* make root inode */
        error = ody_onode_make(&root, &fid, sb);
        if ( error || !root ) {
	    printk("Failure of ody_onode_make for root: error %d\n", error);
	    sb->s_dev = 0;
	    unlock_super(sb);
	    goto error;
	} 

	printk("ody_read_super: rootinode is %ld dev %d\n", 
	       root->i_ino, root->i_dev);
	sbi->sbi_root = root;
	sb->s_root = d_alloc_root(root, NULL);
	unlock_super(sb);

	sets_register_domain(ODY_SUPER_MAGIC, ody_expand, ody_cursexpand,
			     ody_prefetch);

	EXIT;  
        return sb;

error:
EXIT;  
	MOD_DEC_USE_COUNT;
	if (sbi) {
		sbi->sbi_ocomm = NULL;
		sbi->sbi_root = NULL;
	}
	if ( oc ) {
		oc->oc_sb = NULL;
		oc->oc_inuse = 0;
	}
        if (root) {
                iput(root);
        }
        sb->s_dev = 0;
        return NULL;
}

static void ody_put_super(struct super_block *sb)
{
        struct ody_sb_info *sb_info;

        ENTRY;

	sets_unregister_domain(ODY_SUPER_MAGIC);

	sb->s_dev = 0;
	ody_cache_clear_all(sb);
	sb_info = ody_sbp(sb);
	sb_info->sbi_ocomm->oc_inuse = 0;
	sb_info->sbi_ocomm->oc_sb = NULL;
	printk("Ody: Bye bye.\n");
	memset(sb_info, 0, sizeof(* sb_info));

        MOD_DEC_USE_COUNT;
	EXIT;
}

/* all filling in of inodes postponed until lookup */
static void ody_read_inode(struct inode *inode)
{
	struct ody_inode_info *oii;
	ENTRY;
	oii = ITOO(inode);
	oii->o_magic = 0;
	return;
}

static void ody_put_inode(struct inode *in) 
{
	ENTRY;

        ODEBUG(D_INODE,"ino: %ld, count %d\n", in->i_ino, in->i_count);

	if ( in->i_count == 1 ) 
		in->i_nlink = 0;
		
}

static void ody_delete_inode(struct inode *inode)
{
        struct ody_inode_info *oii;
        struct inode *open_inode;

        ENTRY;
        ODEBUG(D_SUPER, " inode->ino: %ld, count: %d\n", 
	       inode->i_ino, inode->i_count);        

        oii = ITOO(inode);
	if ( inode->i_ino == ODY_CTL_INO || oii->o_magic != ODY_ONODE_MAGIC ) {
	        clear_inode(inode);
		return;
	}

        open_inode = oii->o_ovp;
        if ( open_inode ) {
                ODEBUG(D_SUPER, "DELINO cached file: ino %ld count %d.\n",  
		       open_inode->i_ino,  open_inode->i_count);
                oii->o_ovp = NULL;
                iput(open_inode);
        }
	
	ody_cache_clear_inode(inode);
	ODEBUG(D_DOWNCALL, "clearing inode: %ld, %x\n", inode->i_ino,
	       oii->o_flags);
	inode->u.generic_ip = NULL;
        clear_inode(inode);
	EXIT;
}

static int  ody_notify_change(struct dentry *de, struct iattr *iattr)
{
	struct inode *inode = de->d_inode;
        struct ody_inode_info *oii;
        struct ody_vattr vattr;
        int error;
	
	ENTRY;
        memset(&vattr, 0, sizeof(vattr)); 
        oii = ITOO(inode);
        CHECK_ONODE(oii);

        ody_iattr_to_vattr(iattr, &vattr);
        vattr.va_type = O_VNON; /* cannot set type */
	ODEBUG(D_SUPER, "vattr.va_mode %o\n", vattr.va_mode);

        error = viceroy_setattr(inode->i_sb, &oii->o_fid, &vattr);

        if ( !error ) {
	        ody_vattr_to_iattr(inode, &vattr); 
		ody_cache_clear_inode(inode);
        }
	ODEBUG(D_SUPER, "inode.i_mode %o, error %d\n", 
	       inode->i_mode, error);

	EXIT;
        return error;
}

/*  we need _something_ for this routine. Let's mimic AFS */
static int ody_statfs(struct super_block *sb, struct statfs *buf, 
		       int bufsiz)
{
	struct statfs tmp;

	tmp.f_type = ODY_SUPER_MAGIC;
	tmp.f_bsize = 1024;
	tmp.f_blocks = 9000000;
	tmp.f_bfree = 9000000;
	tmp.f_bavail = 9000000 ;
	tmp.f_files = 9000000;
	tmp.f_ffree = 9000000;
	tmp.f_namelen = 0;
	copy_to_user(buf, &tmp, bufsiz);
	return 0; 
}


/* init_ody: used by filesystems.c to register ody */

struct file_system_type ody_fs_type = {
   "ody", 0, ody_read_super, NULL
};

int init_ody_fs(void)
{
#ifndef MODULE
	init_ody_psdev();
#endif
	return register_filesystem(&ody_fs_type);
}

/* MODULE stuff is in psdev.c */

/*  helpers */
static struct ocomm *ody_psinode2ocomm(struct inode *inode) 
{
        
	unsigned int minor = MINOR(inode->i_rdev);
	ODEBUG(D_PSDEV,"minor %d\n", minor);
	if ( minor < MAX_ODYDEVS ) 
	      return &(psdev_ocomm[minor]);
	else
	      return NULL;
}

static struct ody_sb_info *ody_psinode2sbi(struct inode *inode) 
{
	unsigned int minor = MINOR(inode->i_rdev);

	ODEBUG(D_PSDEV,"minor %d\n", minor);
	if ( (minor >= 0) && (minor < MAX_ODYDEVS)) 
	        return &(ody_super_info[minor]);
	else
	        return NULL;
}

/* name lookup for psdev passed in by mount */
static int ody_get_psdev(void *data, struct inode **res_dev)
{
        char **psdev_path;
        struct inode *psdev = 0;
	struct dentry *ent=NULL;

 
	if ( ! data ) { 
		printk("ody_get_psdev: no data!\n");
		return 1;
	} 

	psdev_path = data;
        ent = namei((char *) *psdev_path);
        if (IS_ERR(ent)) {
		printk("namei error %ld for %d\n", PTR_ERR(ent), 
		       (int) psdev_path);
		return 1;
        }
	psdev = ent->d_inode;

        if (!S_ISCHR(psdev->i_mode)) {
		printk("not a character device\n");
		return 1;
        }
	ODEBUG(D_PSDEV,"major %d, minor %d, count %d\n", 
	       MAJOR(psdev->i_rdev), 
	       MINOR(psdev->i_rdev), psdev->i_count);
        
        if (MAJOR(psdev->i_rdev) != ODY_PSDEV_MAJOR) {
		printk("device %d not a Ody PSDEV device\n", 
		       MAJOR(psdev->i_rdev));
		return 1;
        }

        if (MINOR(psdev->i_rdev) >= MAX_ODYDEVS) { 
		printk("minor %d not an allocated Ody PSDEV\n", 
		       psdev->i_rdev);
		return 1;
        }

        if (psdev->i_count < 1) {
		printk("ody device minor %d not open (i_count = %d)\n", 
		       MINOR(psdev->i_rdev), psdev->i_count);
		return 1;
        }
        
        *res_dev = psdev;
	EXIT;  
        return 0;
}
