#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* 
 * Coda File System, Linux Kernel module
 * 
 * Original version, adapted from cfs_mach.c, (C) Carnegie Mellon University
 * Linux modifications (C) 1996, Peter J. Braam
 * Rewritten for Linux 2.1 (C) 1997 Carnegie Mellon University
 *
 * Carnegie Mellon University encourages users of this software to
 * contribute improvements to the Coda project.
 */

/*
 * stolen for Odyssey
 */

#ifndef _LINUX_ODY_FS
#define _LINUX_ODY_FS

#include <linux/kernel.h>
#include <linux/param.h>
#include <linux/sched.h> 
#include <linux/mm.h>
#include <linux/vmalloc.h>
#include <linux/malloc.h>
#include <linux/wait.h>		
#include <linux/types.h>
#include <linux/fs.h>

/* operations */
extern struct inode_operations ody_dir_inode_operations;
extern struct inode_operations ody_file_inode_operations;
extern struct inode_operations ody_ioctl_inode_operations;
extern struct inode_operations upcall_ioctl_inode_operations;
extern struct inode_operations ody_symlink_inode_operations;

extern struct file_operations ody_dir_operations;
extern struct file_operations ody_file_operations;
extern struct file_operations ody_ioctl_operations;
extern struct file_operations upcall_ioctl_operations;

/* operations shared over more than one file */
int ody_open(struct inode *i, struct file *f);
int ody_release(struct inode *i, struct file *f);
int ody_permission(struct inode *inode, int mask);
int ody_revalidate_inode(struct dentry *);

/* global variables */
extern int ody_debug;
extern int ody_print_entry;
extern int ody_access_cache;

/* this file:  heloers */
static __inline__ struct ofid *ody_i2f(struct inode *);
char *ody_f2s(struct ofid *f);
int ody_isroot(struct inode *i);
int ody_iscontrol(const char *name, size_t length);
int ody_is_uccontrol(const char *name, size_t length);


void ody_load_creds(struct ody_cred *cred);
int ody_mycred(struct ody_cred *);
void ody_vattr_to_iattr(struct inode *, struct ody_vattr *);
void ody_iattr_to_vattr(struct iattr *, struct ody_vattr *);
unsigned short ody_flags_to_oflags(unsigned short);
unsigned short ody_ioflags_to_oioflags(unsigned short);
void print_vattr( struct ody_vattr *attr );
int ody_cred_ok(struct ody_cred *cred);
int ody_cred_eq(struct ody_cred *cred1, struct ody_cred *cred2);

/* defined in  file.c */
void ody_prepare_openfile(struct inode *ody_inode, struct file *ody_file, 
			   struct inode *open_inode,  struct file *open_file,
			   struct dentry *open_dentry);
void ody_restore_odyfile(struct inode *ody_inode, struct file *ody_file, 
			   struct inode *open_inode, struct file *open_file);
int ody_inode_grab(dev_t dev, ino_t ino, struct inode **ind);

/* defined in ody_sys.c */
int ody_ioctl(unsigned long cmd, void *data, int len, struct inode *inode);


#define NB_SFS_SIZ 0x895440

/* debugging masks */
#define D_SUPER     1   /* print results returned by Venus */ 
#define D_INODE     2   /* print entry and exit into procedure */
#define D_FILE      4   /* print malloc, de-alloc information */
#define D_CACHE     8   /* cache debugging */
#define D_MALLOC    16
#define D_ONODE     32
#define D_UPCALL    64  /* up and downcall debugging */
#define D_PSDEV    128  
#define D_PIOCTL   256
#define D_SPECIAL  512
#define D_TIMING  1024
#define D_DOWNCALL 2048
#define D_SYSUPCALL 4096
 
#define ODEBUG(mask, format, a...)                                \
  do {                                                            \
  if (ody_debug & mask) {                                        \
    printk("(%s,l. %d): ",  __FUNCTION__, __LINE__);              \
    printk(format, ## a); }                                       \
} while (0) ;                            

#define ENTRY    \
    if(ody_print_entry) printk("Process %d entered %s\n",current->pid,__FUNCTION__)

#define EXIT    \
    if(ody_print_entry) printk("Process %d leaving %s\n",current->pid,__FUNCTION__)

#define CHECK_ONODE(c) do {  } while (0);

#define ODY_ALLOC(ptr, cast, size)                                       \
do {                                                                      \
    if (size < 3000) {                                                    \
        ptr = (cast)kmalloc((unsigned long) size, GFP_KERNEL);            \
                ODEBUG(D_MALLOC, "kmalloced: %x at %x.\n", (int) size, (int) ptr);\
     }  else {                                                             \
        ptr = (cast)vmalloc((unsigned long) size);                        \
	ODEBUG(D_MALLOC, "vmalloced: %x at %x.\n", (int) size, (int) ptr);}\
    if (ptr == 0) {                                                       \
        printk("kernel malloc returns 0 at %s:%d\n", __FILE__, __LINE__);  \
    }                                                                     \
    memset( ptr, 0, size );                                                   \
} while (0)


#define ODY_FREE(ptr,size) do {if (size < 3000) { kfree_s((ptr), (size)); ODEBUG(D_MALLOC, "kfreed: %x at %x.\n", (int) size, (int) ptr); } else { vfree((ptr)); ODEBUG(D_MALLOC, "vfreed: %x at %x.\n", (int) size, (int) ptr);} } while (0)

/* inode to onode */

static __inline__ struct ofid *ody_i2f(struct inode *inode)
{
	struct ody_inode_info *op = (struct ody_inode_info *)(inode->u.generic_ip);
	return &(op->o_fid);
}

#define ITOO(inode) ((struct ody_inode_info *)&((inode)->u.generic_ip))

#endif

