#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * stolen for Odyssey
 */

#ifndef __ODY_PSDEV_H
#define __ODY_PSDEV_H

#define ODY_PSDEV_MAJOR 101
#define MAX_ODYDEVS  5	   /* how many do we allow */

extern struct ocomm psdev_ocomm[];

/* queue stuff; the rest is static to psdev.c */
struct queue {
    struct queue *forw, *back;
};
void ody_q_insert(struct queue *el, struct queue *q);
void ody_q_remove(struct queue *q);


#define ODY_SUPER_MAGIC	0x29295858

struct ody_sb_info
{
	struct inode *      sbi_psdev;     /* /dev/ody? Viceroy/kern device */
	struct inode *      sbi_ctlcp;     /* control magic file */
	int                 sbi_refct;
	struct ocomm *      sbi_ocomm;
	struct inode *      sbi_root;
	struct list_head    sbi_cchead;
};

/* communication pending/processing queues queues */
struct ocomm {
	u_long		    oc_seq;
	struct wait_queue  *oc_waitq; /* Venus wait queue */
	struct queue	    oc_pending;
	struct queue	    oc_processing;
	struct super_block *oc_sb;
	int                 oc_inuse;
};

static inline int ocomm_open(struct ocomm *ocp)
{
        return ((ocp)->oc_pending.forw != NULL);
}

static inline void mark_ocomm_closed(struct ocomm *ocp)
{
        (ocp)->oc_pending.forw = NULL;
}

static inline struct ody_sb_info *ody_sbp(struct super_block *sb)
{
    return ((struct ody_sb_info *)((sb)->u.generic_sbp));
}



extern void ody_psdev_detach(int unit);
extern int  init_ody_psdev(void);


/* upcalls */
int viceroy_rootfid(struct super_block *sb, struct ofid *fidp);
int viceroy_getattr(struct super_block *sb, struct ofid *fid, 
		    struct ody_vattr *attr);
int viceroy_setattr(struct super_block *, struct ofid *, 
		    struct ody_vattr *);
int viceroy_lookup(struct super_block *sb, struct ofid *fid, 
		   const char *name, int length, int *type, 
		   struct ofid *resfid);
int viceroy_release(struct super_block *sb, struct ofid *fid, int flags);
int viceroy_open(struct super_block *sb, struct ofid *fid,
		 int flag, ino_t *ino, dev_t *dev, int *flags);
int viceroy_rdwr(struct super_block *sb, struct ofid *fid, loff_t *offset,
		 int ioflag, int rwflag, int *count, char *buff);
int viceroy_mkdir(struct super_block *sb, struct ofid *dirfid, 
		  const char *name, int length, 
		  struct ofid *newfid, struct ody_vattr *attrs);
int viceroy_create(struct super_block *sb, struct ofid *dirfid, 
		   const char *name, int length, int excl, int mode, 
		   struct ofid *newfid, struct ody_vattr *attrs) ;
int viceroy_rmdir(struct super_block *sb, struct ofid *dirfid, 
		  const char *name, int length);
int viceroy_readdir(struct super_block *sb, struct ofid *dirfid,
		    int count, loff_t offset, caddr_t buff, int *size);
int viceroy_rmdir(struct super_block *sb, struct ofid *dirfid, 
		  const char *name, int length);
int viceroy_remove(struct super_block *sb, struct ofid *dirfid, 
		   const char *name, int length);
int viceroy_readlink(struct super_block *sb, struct ofid *fid, 
		     char *buffer, int *length);
int viceroy_rename(struct super_block *, struct ofid *new_fid, 
		   struct ofid *old_fid, size_t old_length, 
		   size_t new_length, const char *old_name, 
		   const char *new_name);
int viceroy_link(struct super_block *sb, struct ofid *fid, 
		 struct ofid *dirfid, const char *name, int len );
int viceroy_symlink(struct super_block *sb, struct ofid *fid,
		    const char *name, int len, const char *symname,
		    int symlen);
int viceroy_access(struct super_block *sb, struct ofid *fid, int mask);
int viceroy_pioctl(struct super_block *sb, struct ofid *fid,
		   unsigned int cmd, struct PioctlData *data);
int viceroy_request(struct super_block *sb, struct ofid *fid,
		    u_int rsrc, u_int gs, u_int type, u_int ts,
		    int low, int high,
		    int *cancelled, int *value, int *result);
int viceroy_cancel(struct super_block *sb, int reqid);
int viceroy_cancpid(struct super_block *sb);
int viceroy_tsop(struct super_block *sb, struct ofid *fid,
		 u_int gs, u_int type, u_int ts, u_int op,
		 size_t argsz, caddr_t src_data,
		 size_t *retsz, caddr_t dest_data);
int viceroy_bmod_start(struct super_block* sp, int joules, float charge);
int viceroy_bmod_stop(struct super_block* sp);
int viceroy_time_to_recharge(struct super_block* sp, int seconds);
int viceroy_multif(struct super_block *sb,
		   u_int gs, u_int type, u_int ts, u_int op,
		   size_t argsz, caddr_t src_data,
		   size_t *retsz, caddr_t dest_data);
int viceroy_expand(struct super_block *sb, struct ofid *fid, 
		   const char *name, int length, int *type, 
		   struct ofid *resfid);
int viceroy_cursexpand(struct super_block *sb, struct ofid *fid, 
		       char **buf, size_t *bufsize);
int viceroy_prefetch(struct super_block *sb, struct ofid *fid,
		     int flag, ino_t *ino, dev_t *dev, int *flags);
int ody_downcall(int opcode, union ody_outputArgs *out,
		 struct super_block *sb);
int ody_upcall(struct ody_sb_info *mntinfo, unsigned long inSize, 
	       unsigned long *outSize, union ody_inputArgs *buffer);
int viceroy_fsync(struct super_block *sb, struct ofid *fid);


/* messages between ody filesystem in kernel and Viceroy */
extern int ody_hard;
extern unsigned long ody_timeout;
struct omsg {
	struct queue        om_chain;
	caddr_t	            om_data;
	u_short	            om_flags;
	unsigned long       om_inSize;  /* Size is at most 5000 bytes */
	unsigned long       om_outSize;
	u_short	            om_opcode;  /* copied from data to save lookup */
	int		    om_unique;
	struct wait_queue  *om_sleep;   /* process' wait queue */
	unsigned long       om_posttime;
};


/*
 * Statistics
 */
struct ody_upcallstats {
	int	ncalls;			/* client requests */
	int	nbadcalls;		/* upcall failures */
	int	reqs[ODY_NCALLS];	/* count of each request */
} ;

extern struct ody_upcallstats ody_callstats;

static inline void clstats(int opcode)
{
    ody_callstats.ncalls++;
    if ( (0 <= opcode) && (opcode <= ODY_NCALLS) )
	ody_callstats.reqs[opcode]++;
    else
	printk("clstats called with bad opcode %d\n", opcode); 
}

static inline void badclstats(void)
{
    ody_callstats.nbadcalls++;
}

#endif
