#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Inode operations for Coda filesystem
 * Original version: (C) 1996 P. Braam and M. Callahan
 * Rewritten for Linux 2.1. (C) 1997 Carnegie Mellon University
 * 
 * Carnegie Mellon encourages users to contribute improvements to
 * the Coda project. Contact Peter Braam (coda@cs.cmu.edu).
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/string.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>

/* initialize the debugging variables */
int ody_debug = 0;
int ody_print_entry = 0; 
int ody_access_cache = 1;

/* caller must allocate 36 byte string ! */
char * ody_f2s(struct ofid *f)
{
	static char s[60];
	if ( f ) {
		sprintf(s, "(%lx.%lx.%lx.%lx)", 
			f->ofid_ttype, f->ofid_tome,
			f->ofid_vnum, f->ofid_uniq);
	}
	return s;
}

/* recognize special .CONTROL name */
int ody_iscontrol(const char *name, size_t length)
{
	if ((ODY_CONTROLLEN == length) && 
	    (strncmp(name, ODY_CONTROL, ODY_CONTROLLEN) == 0))
		return 1;
	return 0;
}

/* recognize /ody inode */
int ody_isroot(struct inode *i)
{
    if ( i->i_sb->s_root->d_inode == i ) {
	    return 1;
    } else {
	    return 0;
    }
}

/* put the current process credentials in the cred */
void ody_load_creds(struct ody_cred *cred)
{
        cred->ocr_uid = (vuid_t) current->uid;
        cred->ocr_euid = (vuid_t) current->euid;
        cred->ocr_suid = (vuid_t) current->suid;
        cred->ocr_fsuid = (vuid_t) current->fsuid;

        cred->ocr_gid = (vgid_t) current->gid;
        cred->ocr_egid = (vgid_t) current->egid;
        cred->ocr_sgid = (vgid_t) current->sgid;
        cred->ocr_fsgid = (vgid_t) current->fsgid;
}

int ody_cred_ok(struct ody_cred *cred)
{
	return(current->fsuid == cred->ocr_fsuid);
}

int ody_cred_eq(struct ody_cred *cred1, struct ody_cred *cred2)
{
	return (cred1->ocr_fsuid == cred2->ocr_fsuid);
}

unsigned short ody_flags_to_oflags(unsigned short flags)
{
	unsigned short ody_flags = 0;

	if ( (flags & O_ACCMODE) == O_RDONLY ) {
 		ODEBUG(D_FILE, "--> O_O_READ added\n");
		ody_flags |= O_O_READ;
	}

	if ( (flags & O_ACCMODE) == O_RDWR ) {
 		ODEBUG(D_FILE, "--> O_O_READ | O_O_WRITE added\n");
 		ody_flags |= O_O_READ | O_O_WRITE;
	}

 	if ( (flags & O_ACCMODE) == O_WRONLY ) { 
 		ODEBUG(D_FILE, "--> O_O_WRITE added\n");
		ody_flags |= O_O_WRITE;
	}

	if ( flags & O_NONBLOCK ) {
 		ODEBUG(D_FILE, "--> O_O_NONBLOCK added\n");
		ody_flags |= O_O_NONBLOCK;
	}

	if ( flags & O_TRUNC ) {
 		ODEBUG(D_FILE, "--> O_O_TRUNC added\n");
		ody_flags |= O_O_TRUNC;
	}

	if ( flags & O_EXCL ) {
 		ODEBUG(D_FILE, "--> O_O_EXCL added\n");
		ody_flags |= O_O_EXCL;
	}

	return ody_flags;
}


/* utility functions below */
void ody_vattr_to_iattr(struct inode *inode, struct ody_vattr *attr)
{
        int inode_type;
        /* inode's i_dev, i_flags, i_ino are set by iget 
           XXX: is this all we need ??
           */
        switch (attr->va_type) {
        case O_VNON:
                inode_type  = 0;
                break;
        case O_VREG:
                inode_type = S_IFREG;
                break;
        case O_VDIR:
                inode_type = S_IFDIR;
                break;
        case O_VLNK:
                inode_type = S_IFLNK;
                break;
        default:
                inode_type = 0;
        }
	inode->i_mode |= inode_type;

	if (attr->va_mode != (u_short) -1)
	        inode->i_mode = attr->va_mode | inode_type;
        if (attr->va_uid != -1) 
	        inode->i_uid = (uid_t) attr->va_uid;
        if (attr->va_gid != -1)
	        inode->i_gid = (gid_t) attr->va_gid;
	if (attr->va_nlink != -1)
	        inode->i_nlink = attr->va_nlink;
	if (attr->va_size != -1)
	        inode->i_size = attr->va_size;
	/*  XXX This needs further study */
	/*
        inode->i_blksize = attr->va_blocksize;
	inode->i_blocks = attr->va_size/attr->va_blocksize 
	  + (attr->va_size % attr->va_blocksize ? 1 : 0); 
	  */
#ifdef HAVE_STAT_TIMESPEC
	if (attr->va_atime.tv_sec != -1) 
	        inode->i_atime = attr->va_atime.tv_sec;
	if (attr->va_mtime.tv_sec != -1)
	        inode->i_mtime = attr->va_mtime.tv_sec;
        if (attr->va_ctime.tv_sec != -1)
	        inode->i_ctime = attr->va_ctime.tv_sec;
#else
	if (attr->va_atime != -1) 
	        inode->i_atime = attr->va_atime;
	if (attr->va_mtime != -1)
	        inode->i_mtime = attr->va_mtime;
        if (attr->va_ctime != -1)
	        inode->i_ctime = attr->va_ctime;
#endif
}
/* 
 * BSD sets attributes that need not be modified to -1. 
 * Linux uses the valid field to indicate what should be
 * looked at.  The BSD type field needs to be deduced from linux 
 * mode.
 * So we have to do some translations here.
 */

void ody_iattr_to_vattr(struct iattr *iattr, struct ody_vattr *vattr)
{
        unsigned int valid;

        /* clean out */        
        vattr->va_mode = (umode_t) -1;
        vattr->va_uid = (vuid_t) -1; 
        vattr->va_gid = (vgid_t) -1;
        vattr->va_size = (off_t) -1;
#ifdef HAVE_STAT_TIMESPEC
	vattr->va_atime.tv_sec = (time_t) -1;
        vattr->va_mtime.tv_sec  = (time_t) -1;
	vattr->va_ctime.tv_sec  = (time_t) -1;
	vattr->va_atime.tv_nsec =  (time_t) -1;
        vattr->va_mtime.tv_nsec = (time_t) -1;
	vattr->va_ctime.tv_nsec = (time_t) -1;
#else
	vattr->va_atime = (time_t) -1;
        vattr->va_mtime = (time_t) -1;
	vattr->va_ctime = (time_t) -1;
#endif
        vattr->va_type = O_VNON;
	vattr->va_fileid = -1;
	vattr->va_gen = -1;
	vattr->va_bytes = -1;
	vattr->va_nlink = -1;
	vattr->va_blocksize = -1;
	vattr->va_rdev = -1;
        vattr->va_flags = 0;

        /* determine the type */
#if 0
        mode = iattr->ia_mode;
                if ( S_ISDIR(mode) ) {
                vattr->va_type = O_VDIR; 
        } else if ( S_ISREG(mode) ) {
                vattr->va_type = O_VREG;
        } else if ( S_ISLNK(mode) ) {
                vattr->va_type = O_VLNK;
        } else {
                /* don't do others */
                vattr->va_type = O_VNON;
        }
#endif 

        /* set those vattrs that need change */
        valid = iattr->ia_valid;
        if ( valid & ATTR_MODE ) {
                vattr->va_mode = iattr->ia_mode;
	}
        if ( valid & ATTR_UID ) {
                vattr->va_uid = (vuid_t) iattr->ia_uid;
	}
        if ( valid & ATTR_GID ) {
                vattr->va_gid = (vgid_t) iattr->ia_gid;
	}
        if ( valid & ATTR_SIZE ) {
                vattr->va_size = iattr->ia_size;
	}
#ifdef HAVE_STAT_TIMESPEC
        if ( valid & ATTR_ATIME ) {
                vattr->va_atime.tv_sec = iattr->ia_atime;
                vattr->va_atime.tv_nsec = 0;
	}
        if ( valid & ATTR_MTIME ) {
                vattr->va_mtime.tv_sec = iattr->ia_mtime;
                vattr->va_mtime.tv_nsec = 0;
	}
        if ( valid & ATTR_CTIME ) {
                vattr->va_ctime.tv_sec = iattr->ia_ctime;
                vattr->va_ctime.tv_nsec = 0;
	}
#else
        if ( valid & ATTR_ATIME ) {
                vattr->va_atime = iattr->ia_atime;
	}
        if ( valid & ATTR_MTIME ) {
                vattr->va_mtime = iattr->ia_mtime;
	}
        if ( valid & ATTR_CTIME ) {
                vattr->va_ctime = iattr->ia_ctime;
	}
#endif
}

void print_vattr(struct ody_vattr *attr)
{
    char *typestr;

    switch (attr->va_type) {
    case O_VNON:
	typestr = "O_VNON";
	break;
    case O_VREG:
	typestr = "O_VREG";
	break;
    case O_VDIR:
	typestr = "O_VDIR";
	break;
    case O_VBLK:
	typestr = "O_VBLK";
	break;
    case O_VCHR:
	typestr = "O_VCHR";
	break;
    case O_VLNK:
	typestr = "O_VLNK";
	break;
    case O_VSOCK:
	typestr = "O_VSCK";
	break;
    case O_VFIFO:
	typestr = "O_VFFO";
	break;
    case O_VBAD:
	typestr = "O_VBAD";
	break;
    default:
	typestr = "????";
	break;
    }


    printk("attr: type %s (%o)  mode %o uid %d gid %d rdev %d\n",
	   typestr, (int)attr->va_type, (int)attr->va_mode, 
	   (int)attr->va_uid, (int)attr->va_gid, (int)attr->va_rdev);
    
    printk("      fileid %d nlink %d size %d blocksize %d bytes %d\n",
	      (int)attr->va_fileid, (int)attr->va_nlink, 
	      (int)attr->va_size,
	      (int)attr->va_blocksize,(int)attr->va_bytes);
    printk("      gen %ld flags %ld\n",
	      attr->va_gen, attr->va_flags);
#ifdef HAVE_STAT_TIMESPEC
    printk("      atime sec %d nsec %d\n",
	      (int)attr->va_atime.tv_sec, (int)attr->va_atime.tv_nsec);
    printk("      mtime sec %d nsec %d\n",
	      (int)attr->va_mtime.tv_sec, (int)attr->va_mtime.tv_nsec);
    printk("      ctime sec %d nsec %d\n",
	      (int)attr->va_ctime.tv_sec, (int)attr->va_ctime.tv_nsec);
#else
    printk("      atime sec %d\n",
	      (int)attr->va_atime);
    printk("      mtime sec %d\n",
	      (int)attr->va_mtime);
    printk("      ctime sec %d\n",
	      (int)attr->va_ctime);
#endif
}
