#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * ody_sys.c
 *
 * Implement the Odyssey API extensions.
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <linux/string.h>
#include <asm/uaccess.h>
#include <linux/sched.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>

#define __P(p) p
#include "kernel/vfs/includes/ody_sys.h"
#include "kernel/vfs/includes/ody_pow.h"
#include "kernel/vfs/includes/ody_multif.h"
#include "kernel/vfs/includes/odyapi.h"
#include "include/odyssey.h"

int ioctl_ody_req(struct ody_req_args *uap, struct inode *inode)
{
	struct ody_reqargs ora;
        struct dentry *target_de;
	struct inode *target_inode;
        struct ody_inode_info *onp;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_req: %s\n", uap->path);

	if (copy_from_user(&ora, uap->ora, sizeof(struct ody_reqargs)))
		return -EFAULT;

	target_de = namei(uap->path);
	if ( IS_ERR(target_de) ) {
                ODEBUG(D_PIOCTL, "ody_tsop: lookup fails.\n");
		return PTR_ERR(target_de);
        } else {
	        target_inode = target_de->d_inode;
	}

	/* return if it is not a Ody inode */
	if ( target_inode->i_sb != inode->i_sb ) {
  	        if ( target_de )
		        dput(target_de);
	        return -EOPNOTSUPP;
	}

	onp = ITOO(target_inode);
	CHECK_ONODE(onp);

	error = viceroy_request(target_inode->i_sb, &(onp->o_fid),
				ora.ora_resource, 
				ora.ora_gen_intf,
				ora.ora_obj_type, 
				ora.ora_type_intf, 
				ora.ora_low, 
				ora.ora_high, 
				&ora.ora_cancelled,
				&ora.ora_value,
				&ora.ora_result);

	if (target_de)
		dput(target_de);

	if (error && (error != -ODY_ENOTINWINDOW))
		return error;

	if (copy_to_user(uap->ora, &ora, sizeof(struct ody_reqargs)))
		panic("ioctl_ody_req: copy_from ok, copy_to failed.\n");

	EXIT;

	return error;
}

int ioctl_ody_freq(struct ody_freq_args *uap, struct inode *inode)
{
	struct file *file;
	struct ody_reqargs ora;
	struct inode *target_inode;
        struct ody_inode_info *onp;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_freq: %d\n", uap->fd);

	if (!current->files)
		return -EBADF;

	if (copy_from_user(&ora, uap->ora, sizeof(struct ody_reqargs)))
		return -EFAULT;

	file = fget(uap->fd);
	if (!file)
		return -EBADF;

	target_inode = file->f_dentry->d_inode;
	onp = ITOO(target_inode);
        CHECK_ONODE(onp);

	/* return if it is not a Ody inode */
	if ( target_inode->i_sb != inode->i_sb ) {
		error = -EOPNOTSUPP;
		goto exit;
	}

	error = viceroy_request(target_inode->i_sb, &(onp->o_fid),
				ora.ora_resource, 
				ora.ora_gen_intf,
				ora.ora_obj_type, 
				ora.ora_type_intf, 
				ora.ora_low, 
				ora.ora_high, 
				&ora.ora_cancelled,
				&ora.ora_value,
				&ora.ora_result);

	if (error && (error != -ODY_ENOTINWINDOW))
		goto exit;

	if (copy_to_user(uap->ora, &ora, sizeof(struct ody_reqargs)))
		panic("ioctl_ody_req: copy_from ok, copy_to failed.\n");

 exit:
	fput(file);
	EXIT;
	return error;
}

int ioctl_ody_cancel(struct ody_cancel_args *uap, struct inode *inode)
{
        struct dentry *target_de;
	struct inode *target_inode;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_cancel: %s, reqid %d\n", uap->path, uap->reqid);

	target_de = namei(uap->path);
	if ( IS_ERR(target_de) ) {
                ODEBUG(D_PIOCTL, "ody_tsop: lookup fails.\n");
		return PTR_ERR(target_de);
        } else {
	        target_inode = target_de->d_inode;
	}

	/* return if it is not the CTL inode */
	if ( target_inode != inode ) {
  	        if ( target_de )
		        dput(target_de);
	        return -EOPNOTSUPP;
	}

	error = viceroy_cancel(target_inode->i_sb, uap->reqid);

	if (target_de)
		dput(target_de);

	EXIT;
	return error;
}

int ioctl_ody_tsop(struct ody_tsop_args *uap, struct inode *inode)
{
        struct dentry *target_de;
	struct inode *target_inode;
        struct ody_inode_info *onp;
	struct ody_vers vers;
	size_t retsz;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_tsop: %s, op %d\n", uap->path, uap->op);

	if (copy_from_user(&vers, uap->vers, sizeof(struct ody_vers)))
		return -EFAULT;

	if (get_user(retsz, uap->retsz))
		return -EFAULT;

	if (put_user(retsz, uap->retsz))
		return -EFAULT;

	if (retsz) {
		if (put_user(0, (char *)uap->ret))
			return -EFAULT;
		if (put_user(0, (char *)uap->ret+(retsz-1)))
			return -EFAULT;
	}

	target_de = namei(uap->path);
	if ( IS_ERR(target_de) ) {
                ODEBUG(D_PIOCTL, "ody_tsop: lookup fails.\n");
		return PTR_ERR(target_de);
        } else {
	        target_inode = target_de->d_inode;
	}

	/* return if it is not a Ody inode */
	if ( target_inode->i_sb != inode->i_sb ) {
  	        if ( target_de )
		        dput(target_de);
	        return  -EOPNOTSUPP;
	}

	/* now proceed to make the upcall */
        onp = ITOO(target_inode);
        CHECK_ONODE(onp);

	/* copy_from_user of arg done by upcall */
	error = viceroy_tsop(target_inode->i_sb, &(onp->o_fid),
			     vers.ov_gs, vers.ov_type, vers.ov_tsvnum,
			     uap->op, uap->argsz, uap->arg,
			     &retsz, uap->ret);

	if (target_de)
		dput(target_de);

	if (error)
		return error;

	put_user(retsz, uap->retsz);
	/* copy_to_user of ret done by upcall */

	EXIT;

	return 0;
}

int ioctl_ody_ftsop(struct ody_ftsop_args *uap, struct inode *inode)
{
	struct inode *target_inode;
        struct ody_inode_info *onp;
	struct ody_vers vers;
	size_t retsz;
	int error;
	struct file *file;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_ftsop: %d, od %d\n", uap->fd, uap->op);

	if (!current->files)
		return -EBADF;

	file = fget(uap->fd);

	if (!file) {
		ODEBUG(D_PIOCTL, "couldn't get file.\n");
		return -EBADF;
	}

	if (copy_from_user(&vers, uap->vers, sizeof(struct ody_vers))) {
		error = -EFAULT;
		goto exit;
	}

	if (get_user(retsz, uap->retsz)) {
		error = -EFAULT;
		goto exit;
	}

	if (put_user(retsz, uap->retsz)) {
		error = -EFAULT;
		goto exit;
	}

	if (retsz) {
		if (put_user(0, (char *)uap->ret)) {
			error = -EFAULT;
			goto exit;
		}
		if (put_user(0, (char *)uap->ret+(retsz-1))) {
			error = -EFAULT;
			goto exit;
		}
	}

	target_inode = file->f_dentry->d_inode;
	onp = ITOO(target_inode);
        CHECK_ONODE(onp);

	/* return if it is not a Ody inode */
	if ( target_inode->i_sb != inode->i_sb ) {
		error = -EOPNOTSUPP;
		goto exit;
	}

	/* copy_from_user of arg done by upcall */
	error = viceroy_tsop(target_inode->i_sb,
			     &(ITOO(target_inode)->o_fid),
			     vers.ov_gs, vers.ov_type, vers.ov_tsvnum,
			     uap->op, uap->argsz, uap->arg,
			     &retsz, uap->ret);

	if (error)
		goto exit;

	put_user(retsz, uap->retsz);
	/* copy_to_user of ret done by upcall */

 exit:
	fput(file);
	EXIT;
	return error;
}

int ioctl_ody_bmod_start(struct ody_bmod_start_args *uap, struct inode *inode)
{
        struct ody_inode_info *onp;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_bmod_start: joules %d, charge %f\n", 
	       uap->joules, uap->charge);

	/* now proceed to make the upcall */
        onp = ITOO(inode);
        CHECK_ONODE(onp);

	/* copy_from_user of arg done by upcall */
	error = viceroy_bmod_start (inode->i_sb, uap->joules, uap->charge);


	EXIT;

	return error;
}

int ioctl_ody_bmod_stop(struct inode *inode)
{
        struct ody_inode_info *onp;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_bmod_stop\n");

	/* now proceed to make the upcall */
        onp = ITOO(inode);
        CHECK_ONODE(onp);

	/* copy_from_user of arg done by upcall */
	error = viceroy_bmod_stop (inode->i_sb);

	EXIT;

	return error;
}

int ioctl_ody_time_to_recharge(struct ody_time_to_recharge_args* uap,
			       struct inode* inode)
{
        struct ody_inode_info *onp;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_time_to_recharge: seconds %d\n", uap->seconds);

	/* now proceed to make the upcall */
        onp = ITOO(inode);
        CHECK_ONODE(onp);

	/* copy_from_user of arg done by upcall */
	error = viceroy_time_to_recharge (inode->i_sb, uap->seconds);

	EXIT;

	return error;
}

int ioctl_ody_multif(struct ody_multif_args *uap, struct inode *inode)
{
	struct ody_inode_info *onp;
	struct ody_vers vers;
	size_t retsz;
	int error;

	ENTRY;

	ODEBUG(D_PIOCTL, "ody_multif:  op %d\n", uap->op);

	if (copy_from_user(&vers, uap->vers, sizeof(struct ody_vers)))
		return -EFAULT;

	if (get_user(retsz, uap->retsz))
		return -EFAULT;

	if (put_user(retsz, uap->retsz))
		return -EFAULT;

	if (retsz) {
		if (put_user(0, (char *)uap->ret))
			return -EFAULT;
		if (put_user(0, (char *)uap->ret+(retsz-1)))
			return -EFAULT;
	}

	/* now proceed to make the upcall */
        onp = ITOO(inode);
        CHECK_ONODE(onp);

	/* copy_from_user of arg done by upcall */
	error = viceroy_multif(inode->i_sb,
			       vers.ov_gs, vers.ov_type, vers.ov_tsvnum,
			       uap->op, uap->argsz, uap->arg,
			       &retsz, uap->ret);

	if (error)
		return error;

	put_user(retsz, uap->retsz);
	/* copy_to_user of ret done by upcall */

	EXIT;

	return 0;
}

int ody_ioctl(unsigned long cmd, void *data, int len, struct inode *inode)
{       
	switch (cmd) {
	case ODYI_REQ:
		if (len != sizeof(struct ody_req_args)) return -EINVAL;
		return ioctl_ody_req(data, inode);
	case ODYI_FREQ:
		if (len != sizeof(struct ody_freq_args)) return -EINVAL;
		return ioctl_ody_freq(data, inode);
	case ODYI_CANCEL:
		if (len != sizeof(struct ody_cancel_args)) return -EINVAL;
		return ioctl_ody_freq(data, inode);
	case ODYI_TSOP:
		if (len != sizeof(struct ody_tsop_args)) return -EINVAL;
		return ioctl_ody_tsop(data, inode);
	case ODYI_FTSOP:
		if (len != sizeof(struct ody_ftsop_args)) return -EINVAL;
		return ioctl_ody_ftsop(data, inode);
	case ODYI_BMOD_START:
		if (len != sizeof(struct ody_bmod_start_args)) return -EINVAL;
		return ioctl_ody_bmod_start (data, inode);
	case ODYI_BMOD_STOP:
		if (len != 0) return -EINVAL;
		return ioctl_ody_bmod_stop (inode);
	case ODYI_TIME_TO_RECHARGE:
		if (len != sizeof (struct ody_time_to_recharge_args))
			return -EINVAL;
		return ioctl_ody_time_to_recharge (data, inode);
	case ODYI_MULTIF:
		if (len != sizeof(struct ody_multif_args)) return -EINVAL;
		return ioctl_ody_multif(data, inode);
	default:
		return -EINVAL;
	}
}
