#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <linux/param.h>
#include <linux/time.h>
#include <linux/malloc.h>

#include "kernel/upcall/linux21/ody_queue.h"
#include "kernel/upcall/includes/upcall.h"
#include "kernel/upcall/linux21/upcall.private.h"
#include "kernel/upcall/linux21/uc_upclass.h"

#include "kernel/vfs/includes/ody_upcall.h"
#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>

int
ody_notify(ucldp, tabp, t, upcall, reg, peh)
	struct uc_ucld_t     *ucldp;
	void                 *tabp;
	struct task_struct   *t;
	upcall_t             upcall;
	int                  reg;
	struct uc_procent_t  **peh;
{
        struct dentry *target_de;
	struct inode *target_inode;
	int              result;
	char             name[ODY_MAXPATHLEN];

	if (upcall != ODY_NOTIFICATION) {
		return EOPNOTSUPP;
	}

	ODEBUG(D_SYSUPCALL, "About to call uc_notify_default\n");
	result = uc_notify_default(ucldp, tabp, t, upcall, reg, peh);
	ODEBUG(D_SYSUPCALL, "uc_notify_default result=%d\n",result);

	if (result) return result;

	if (!reg) {
		strcpy(name, "/ody/");
		strcat(name, ODY_CONTROL);

		ODEBUG(D_SYSUPCALL, "About to do namei on %s\n", name);
		target_de = lookup_dentry(name, NULL, 1);
		ODEBUG(D_SYSUPCALL, "namei result, dentry = %ld\n", PTR_ERR(target_de));
		if (PTR_ERR(target_de) == -ENOENT) {
			ODEBUG(D_SYSUPCALL, 
			       "ody_notify: namei of %s failed\n", name);
			return 0; /* No such path, probably */
		} else {
		ODEBUG(D_SYSUPCALL, "About to get target_inode");
			target_inode = target_de->d_inode;
		}

		ODEBUG(D_SYSUPCALL, "About to check if control object\n");
		/* Needs to be on control object */
		if (target_inode->i_ino != ODY_CTL_INO) {
		    ODEBUG(D_SYSUPCALL, "ody_cancpid error: ino!=ODY_CTL_INO");
		    dput (target_de);
		    return (EOPNOTSUPP);
		}
		ODEBUG(D_SYSUPCALL, "About to call viceroy_cancpid\n");
		result = viceroy_cancpid(target_inode->i_sb);
		ODEBUG(D_SYSUPCALL, "viceroy_cancpid result=%d\n", result);
		dput(target_de);
		ODEBUG(D_SYSUPCALL, "dput succeeded\n");
		if (result) {
			ODEBUG(D_SYSUPCALL, 
			       "ody_notify: viceroy_cancpid returns %d\n",
			       result);
		}
	}
	return 0;
}
		

struct uc_ucld_t ody_ucld = {
	UC_ODYSSEY,
	0,
	"Odyssey",
	uc_init_default,
	ody_notify,
	NULL,
	uc_getpep_default,
	uc_rcpt_default,
	uc_print_default,
	NULL
};
       
