#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* cnode related routines for the coda kernel code
   (C) 1996 Peter Braam
   */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/time.h>
#include <linux/fs.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>

extern int ody_debug;
extern int ody_print_entry;

/* onode.c */

static void ody_fill_inode(struct inode *inode, struct ody_vattr *attr)
{
        ODEBUG(D_SUPER, "ino: %lx\n", inode->i_ino);

        if (ody_debug & D_SUPER ) 
		print_vattr(attr);

        ody_vattr_to_iattr(inode, attr);

        if (S_ISREG(inode->i_mode))
                inode->i_op = &ody_file_inode_operations;
        else if (S_ISDIR(inode->i_mode))
                inode->i_op = &ody_dir_inode_operations;
        else if (S_ISLNK(inode->i_mode))
                inode->i_op = &ody_symlink_inode_operations;
        else {
                printk ("ody_read_inode: what's this? i_mode = %o\n", 
			inode->i_mode);
                inode->i_op = NULL;
        }
}

/* this is effectively ody_iget:
   - get attributes (might be cached)
   - get the inode for the fid using vfs iget
   - link the two up if this is needed
   - fill in the attributes
*/
int ody_onode_make(struct inode **inode, struct ofid *fid,
		   struct super_block *sb)
{
        struct ody_inode_info *onp;
        struct ody_vattr attr;
        int error;
	ino_t ino;
        
        ENTRY;

        /* 
	 * We get inode numbers from Viceroy -- see viceroy source
	 */

	error = viceroy_getattr(sb, fid, &attr);
	if ( error ) {
	    ODEBUG(D_ONODE,
		   "ody_onode_make: ody_getvattr returned %d for %s.\n",
		   error, ody_f2s(fid));
	    *inode = NULL;
	    return error;
	} 

	ino = attr.va_fileid;
        *inode = iget(sb, ino);
        if ( !*inode ) {
                printk("ody_onode_make: iget failed\n");
                return -ENOMEM;
        }

	onp = ITOO(*inode);
	if  ( onp->o_magic == 0 ) {
		memset(onp, 0, (int) sizeof(struct ody_inode_info));
		onp->o_fid = *fid;
		onp->o_magic = ODY_ONODE_MAGIC;
		onp->o_flags = 0;
		onp->o_vnode = *inode;
		INIT_LIST_HEAD(&(onp->o_cnhead));
	} else {
 	        onp->o_flags = 0;
 		ODEBUG(D_ONODE, "ody_onode make on initialized"
 		       "inode %ld, %s!\n",
		       (*inode)->i_ino, ody_f2s(&onp->o_fid));
	}

        ody_fill_inode(*inode, &attr);
	ODEBUG(D_DOWNCALL, "Done linking: ino %ld,  count %d with %s\n",
	       (*inode)->i_ino, (int) (*inode)->i_count,
	       ody_f2s(&onp->o_fid));

        EXIT;
        return 0;
}


inline int ody_fideq(struct ofid *fid1, struct ofid *fid2)
{
	int eq;
	eq =   ( (fid1->ofid_ttype == fid2->ofid_ttype) &&
		 (fid1->ofid_tome == fid2->ofid_tome) &&
		 (fid1->ofid_vnum == fid2->ofid_vnum) &&
		 (fid1->ofid_uniq == fid2->ofid_uniq) );
	return eq;
}


/* convert a fid to an inode. Mostly we can compute
   the inode number from the FID, but not for volume
   mount points: those are in a list */
struct inode *ody_fid_to_inode(struct ofid *fid, struct super_block *sb) 
{
	ino_t nr;
	struct inode *inode;
	struct ody_inode_info *onp;
	ENTRY;

	ODEBUG(D_INODE, "%s\n", ody_f2s(fid));

	if ( !sb ) {
		printk("ody_fid_to_inode: no sb!\n");
		return NULL;
	}

	if ( !fid ) {
		printk("ody_fid_to_inode: no fid!\n");
		return NULL;
	}

	nr = ody_f2i(fid);
	inode = iget(sb, nr);
	if ( !inode ) {
		printk("ody_fid_to_inode: null from iget, sb %p, nr %ld.\n",
		       sb, nr);
		return NULL;
	}

	/* check if this inode is linked to a onode */
	onp = ITOO(inode);
	if ( onp->o_magic != ODY_ONODE_MAGIC ) {
		ODEBUG(D_INODE, "uninitialized inode. Return.\n");
		iput(inode);
		return NULL;
	}

	ODEBUG(D_INODE, "found %ld\n", inode->i_ino);
	iput(inode);	
	return inode;
}

/* the CONTROL inode is made without asking attributes from Viceroy */
int ody_onode_makectl(struct inode **inode, struct super_block *sb)
{
    int error = 0;

    *inode = iget(sb, ODY_CTL_INO);
    if ( *inode ) {
	(*inode)->i_op = &ody_ioctl_inode_operations;
	(*inode)->i_mode = 00444;
	error = 0;
    } else { 
	error = -ENOMEM;
    }
    
    return error;
}


