#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Pioctl operations for Coda.
 * Original version: (C) 1996 Peter Braam 
 * Rewritten for Linux 2.1: (C) 1997 Carnegie Mellon University
 *
 * Carnegie Mellon encourages users of this code to contribute improvements
 * to the Coda project. Contact Peter Braam <coda@cs.cmu.edu>.
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <linux/string.h>
#include <asm/uaccess.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_cache.h>
#include <linux/ody_psdev.h>

#include "kernel/vfs/includes/ody_sys.h"
#include "kernel/vfs/includes/ody_pow.h"
#include "kernel/vfs/includes/ody_multif.h"

/* pioctl ops */
static int ody_ioctl_permission(struct inode *inode, int mask);
static int ody_ioctl_open(struct inode *i, struct file *f);
static int ody_ioctl_release(struct inode *i, struct file *f);
static int ody_pioctl(struct inode * inode, struct file * filp, 
		      unsigned int cmd, unsigned long arg);

/* exported from this file */
struct inode_operations ody_ioctl_inode_operations =
{
	&ody_ioctl_operations,
	NULL,	                /* create */
	NULL,	                /* lookup */
	NULL,	                /* link */
	NULL,	                /* unlink */
	NULL,	                /* symlink */
	NULL,	                /* mkdir */
	NULL,	                /* rmdir */
	NULL,		        /* mknod */
	NULL,		        /* rename */
	NULL,	                /* readlink */
	NULL,	                /* follow_link */
	NULL,	                /* readpage */
	NULL,		        /* writepage */
	NULL,		        /* bmap */
	NULL,	                /* truncate */
	ody_ioctl_permission,   /* permission */
	NULL,                   /* smap */
	NULL,                   /* update page */
        NULL                    /* revalidate */
};

struct file_operations ody_ioctl_operations = {
	NULL,		        /* lseek - default should work for ody */
	NULL,                   /* read */
	NULL,                   /* write */
	NULL,          		/* readdir */
	NULL,			/* select - default */
	ody_pioctl,	        /* ioctl */
	NULL,                   /* mmap */
	ody_ioctl_open,         /* open */
	NULL,			/* flush */
	ody_ioctl_release,      /* release */
	NULL,		        /* fsync */
        NULL,                   /* fasync */
	NULL,			/* check_media_change */
	NULL,			/* revalidate */
	NULL			/* lock */
};

/* the ody pioctl inode ops */
static int ody_ioctl_permission(struct inode *inode, int mask)
{
        ENTRY;
	EXIT;

        return 0;
}

/* The pioctl file ops*/
int ody_ioctl_open(struct inode *i, struct file *f)
{
        ENTRY;
	EXIT;

        return 0;
}

int ody_ioctl_release(struct inode *i, struct file *f) 
{
        ENTRY;
	EXIT;

        return 0;
}


static int ody_pioctl(struct inode * inode, struct file * filp, 
		      unsigned int cmd, unsigned long user_data)
{
        struct dentry *target_de;
        int error;
	struct PioctlData data;
	void *ody_data;
        struct inode *target_inode = NULL;
        struct ody_inode_info *onp;
	int len = _IOC_SIZE(cmd);

        ENTRY;

	if (cmd != ODYI_REQ && cmd != ODYI_FREQ &&
	    cmd != ODYI_CANCEL && cmd != ODYI_TSOP &&
	    cmd != ODYI_FTSOP && cmd != ODYI_BMOD_START &&
	    cmd != ODYI_BMOD_STOP && cmd != ODYI_TIME_TO_RECHARGE &&
	    cmd != ODYI_MULTIF)
		goto non_ody_ioctl;

        /* get the Pioctl data arguments from user space */	
        ODY_ALLOC(ody_data, void *, len);
        if (copy_from_user(ody_data, (int *)user_data, len)) {
		error = -EINVAL;
	} else
		error = ody_ioctl(cmd, ody_data, len, inode);
	ODY_FREE(ody_data, len);
	return error;

 non_ody_ioctl:
	/* If we dropped through to here, it's a "regular" ioctl */

        /* get the Pioctl data arguments from user space */
        if (copy_from_user(&data, (int *)user_data, sizeof(data))) {
	    return -EINVAL;
	}

        /* 
         * Look up the pathname. Note that the pathname is in 
         * user memory, and namei takes care of this
         */
	ODEBUG(D_PIOCTL, "namei, data.follow = %d\n", 
	       data.follow);
        if ( data.follow ) {
                target_de = namei(data.path);
	} else {
	        target_de = lnamei(data.path);
	}
		
	if ( IS_ERR(target_de) ) {
                ODEBUG(D_PIOCTL, "error: lookup fails.\n");
		return PTR_ERR(target_de);
        } else {
	        target_inode = target_de->d_inode;
	}
	
	ODEBUG(D_PIOCTL, "target ino: 0x%ld, dev: 0x%d\n",
	       target_inode->i_ino, target_inode->i_dev);

	/* return if it is not a Ody inode */
	if ( target_inode->i_sb != inode->i_sb ) {
  	        if ( target_de )
		        dput(target_de);
	        return  -EINVAL;
	}

	/* now proceed to make the upcall */
        onp = ITOO(target_inode);

	error = viceroy_pioctl(inode->i_sb, &(onp->o_fid), cmd, &data);

        ODEBUG(D_PIOCTL, "ioctl on inode %ld\n", target_inode->i_ino);
	ODEBUG(D_DOWNCALL, "dput on ino: %ld, icount %d, dcount %d\n",
	       target_inode->i_ino, target_inode->i_count, target_de->d_count);

        if ( target_de ) 
	        dput(target_de);
        return error;
}

