#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 *      	An implementation of a loadable kernel mode driver providing
 *		multiple kernel/user space bidirectional communications links.
 *
 * 		Author: 	Alan Cox <alan@cymru.net>
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 * 
 *              Adapted to become the Linux 2.0 Coda pseudo device
 *              Peter  Braam  <braam@maths.ox.ac.uk> 
 *              Michael Callahan <mjc@emmy.smith.edu>           
 *
 *              Further adapted to become the Linux 2.1 Odyssey pseudo device
 *              Kip Walker <kwalker@cs.cmu.edu>
 *
 *              Changes for Linux 2.1
 *              Copyright (c) 1997 Carnegie-Mellon University
 */

#include <linux/config.h> /* for CONFIG_PROC_FS */
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/lp.h>
#include <linux/malloc.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/skbuff.h>
#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/poll.h>
#include <asm/uaccess.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>
#include <linux/ody_sysctl.h>

#include <kernel/upcall/linux21/ody_queue.h>
#include <kernel/upcall/includes/upcall.h>
#include <kernel/upcall/linux21/upcall.private.h>

/*
 * Where is the prototype?
 */

int proc_register_dynamic(struct proc_dir_entry * dir,
			  struct proc_dir_entry * dp);

/* 
 * Ody stuff
 */
extern struct file_system_type ody_fs_type;
extern struct file_system_type upcall_fs_type;
extern int init_ody_fs(void);
extern int init_upcall_fs(void);
extern int odync_get_info(char *buffer, char **start, off_t offset, int length, int dummy);
extern int odync_nc_info(char *buffer, char **start, off_t offset, int length, int dummy);
extern struct uc_ucld_t ody_ucld;

/* statistics */
struct ody_upcallstats ody_callstats;
int           ody_hard = 0;  /* allows signals during upcalls */
unsigned long ody_timeout = 30; /* .. secs, then signals will dequeue */
extern struct ody_sb_info ody_super_info[MAX_ODYDEVS];
struct ocomm psdev_ocomm[MAX_ODYDEVS];

/* queue stuff for the messages */
static inline void init_queue(struct queue *head)
{
	head->forw = head;
	head->back = head;
}

static inline struct omsg *q_getnext(struct queue *elt)
{
	return (struct omsg *)(elt->forw);
}

static inline int q_end(struct omsg *msg, struct queue *queue)
{
	return (struct queue *)msg == queue;
}

static inline int q_empty(struct queue *queue)
{
	return queue->forw == queue;
}

/* insert before head, ie. at the tail */
void ody_q_insert(struct queue *el, struct queue *head)
{
	el->forw = head->back->forw;
	el->back = head->back;
	head->back->forw = el;
	head->back = el;
}

void ody_q_remove(struct queue *el)
{
	el->forw->back = el->back;
	el->back->forw = el->forw;
}

static struct ocomm *ody_psdev2ocomm(struct file *file)
{
       	unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
	struct ocomm *ocp = NULL;

	if ( (minor >= 0)  && (minor < MAX_ODYDEVS) )
	        ocp = &psdev_ocomm[minor];
	return ocp;
}
	
/*
 * Device operations
 */


static unsigned int ody_psdev_poll(struct file *file, poll_table * wait)
{
        struct ocomm *ocp = ody_psdev2ocomm(file);
	unsigned int mask = POLLOUT | POLLWRNORM;

	if ( !ocp ) 
	        return -ENXIO;

	poll_wait(file, &(ocp->oc_waitq), wait);
	if (!q_empty(&(ocp->oc_pending)))
                mask |= POLLIN | POLLRDNORM;

	return mask;
}


/*
 *	Receive a message written by Viceroy to the psdev
 */
 
static ssize_t ody_psdev_write(struct file *file, const char *buf, 
			       size_t count, loff_t *off)
{
        struct ocomm *ocp = ody_psdev2ocomm(file);
        struct omsg *omp;
	int error = 0;
	int size;
        u_long uniq;
        u_long opcode;
        u_long opcodebuf[2];
        
        if (!ocp)
                return -ENXIO;

        /* Peek at the opcode, uniquefier */
	if (copy_from_user(opcodebuf, buf, 2 * sizeof(u_long)))
	        return -EFAULT;
	opcode = opcodebuf[0];
        uniq = opcodebuf[1];

	ODEBUG(D_PSDEV, "(process,opc,uniq)=(%d,%ld,%ld)\n", 
	       current->pid, opcode, uniq);

        if (ODY_DOWNCALL(opcode)) {
		struct super_block *sb = NULL;
                union ody_outputArgs *dcbuf;
		size = sizeof(*dcbuf);

		sb = ocp->oc_sb;
		if ( !sb ) {
			printk("ody_psdev_write: downcall, no SB!\n");
			return count;
		}
		ODEBUG(D_PSDEV, "handling downcall\n");

		if  ( count < sizeof(struct ody_out_hdr) ) {
		        printk("ody_downcall opc %ld uniq %ld, not enough!\n",
			       opcode, uniq);
			return count;
		}
		ODY_ALLOC(dcbuf, union ody_outputArgs *, size);
		if ( count > size ) {
		        printk("Ody: downcall opc %ld, uniq %ld, too much!",
			       opcode, uniq);
		        count = size;
		}
		if (copy_from_user(dcbuf, buf, count))
		        return -EFAULT;

		/* what downcall errors does Viceroy handle ? */
		error = ody_downcall(opcode, dcbuf, sb);

		if ( error) {
		        printk("psdev_write: ody_downcall error: %d\n", 
			       error);
			return 0;
		}
		ODY_FREE(dcbuf, size);
		return count;
        }

        
        /* Look for the message on the processing queue. */
        for (omp = q_getnext(&(ocp->oc_processing));
	     !q_end(omp, &(ocp->oc_processing));
             omp = q_getnext(&(omp->om_chain))) {
	        if (omp->om_unique == uniq) {
			break;
			ODEBUG(D_PSDEV,"Eureka: uniq %ld on queue!\n", uniq);
		}
	}
        if (q_end(omp, &(ocp->oc_processing))) {
	        printk("psdev_write: msg (%ld, %ld) not found\n", 
		       opcode, uniq);
		return(-ESRCH);
        }

        /* Remove the message from the processing queue */
        ody_q_remove(&(omp->om_chain));

        /* move data into response buffer. */
        if (omp->om_outSize < count) {
                printk("psdev_write: too much cnt: %ld, cnt: %d, opc: %ld, uniq: %ld.\n",
		       omp->om_outSize, count, opcode, uniq);
		count = omp->om_outSize; /* don't have more space! */
	}
        if (copy_from_user(omp->om_data, buf, count))
	        return -EFAULT;

	/* adjust outsize. is this usefull ?? */
        omp->om_outSize = count;	
        omp->om_flags |= VM_WRITE;

	ODEBUG(D_PSDEV, 
	       "Found! Count %d for (opc,uniq)=(%ld,%ld), omsg at %x\n", 
	        count, opcode, uniq, (int)&omp);

        wake_up(&omp->om_sleep);
        return(count);  
}

/*
 *	Read a message from the kernel to Viceroy
 */

static ssize_t ody_psdev_read(struct file * file, char * buf, 
			       size_t count, loff_t *off)
{
        struct ocomm *ocp = ody_psdev2ocomm(file);
        struct omsg *omp;
	int result = count ;

        if (!ocp)
              return -ENXIO;
        
        /* Get message at head of request queue. */
        if (q_empty(&(ocp->oc_pending))) {
	      ODEBUG(D_PSDEV, "read found empty queue!!\n");
              return 0;
        }
    
        omp = q_getnext(&(ocp->oc_pending));

	ODEBUG(D_PSDEV, "giving msg: %ld/%ld count: %d (%d.%d)\n",
	       omp->om_inSize, omp->om_outSize,
	       count,
	       omp->om_opcode, omp->om_unique);

        /* Move the input args into userspace */
        if (omp->om_inSize <= count)
              result = omp->om_inSize;

        if (count < omp->om_inSize) {
		struct ody_toosmall_in fakemsg;

		/* Make sure we have enough for the TOOSMALL message */
		if (count < sizeof(struct ody_toosmall_in)) {
			/* Too small for the TOOSMALL message */
			return -E2BIG;/* A stretch, but ... */
		}

		/* 
		 * We are not prepared to read it all in: create an
		 * ODY_TOOSMALL message.  We only need opcode and
		 * the ody_toosmall arguments; others are ignored.
		 */
		
		fakemsg.ih.opcode = ODY_TOOSMALL;
		fakemsg.bufsiz = omp->om_inSize;

		if (copy_to_user(buf, &fakemsg,
				 sizeof(struct ody_toosmall_in))) {
			printk ("psdev_read: couldn't send TOOSMALL\n");
			return -EFAULT;
		}

		return sizeof(struct ody_toosmall_in);
        }

        if ( copy_to_user(buf, omp->om_data, result))
	        return -EFAULT;
        
        if (omp->om_chain.forw == 0 || omp->om_chain.back == 0)
                printk("ody_psdev_read: bad chain");

        ody_q_remove(&(omp->om_chain));

        /* If request was a signal, don't enqueue */
        if (omp->om_opcode == ODY_SIGNAL) {
		ODEBUG(D_PSDEV, "vcread: signal msg (%d, %d)\n", 
		       omp->om_opcode, omp->om_unique);
		ODY_FREE(omp->om_data, sizeof(struct ody_in_hdr));
		ODY_FREE(omp, sizeof(struct omsg));
		return count;
        }
    
        omp->om_flags |= VM_READ;
        ody_q_insert(&(omp->om_chain), &(ocp->oc_processing));

        return result;
}


static int ody_psdev_open(struct inode * inode, struct file * file)
{
        register struct ocomm *ocp = NULL;
        ENTRY;
        
	ocp =ody_psdev2ocomm(file);

        if (!ocp)
		return -ENODEV;

	if (ocp->oc_inuse)
		return -EBUSY;

	memset(ocp, 0, sizeof(struct ocomm));
	ocp->oc_inuse = 1;
	MOD_INC_USE_COUNT;

        init_queue(&(ocp->oc_pending));
        init_queue(&(ocp->oc_processing));

	memset(&ody_callstats, 0, sizeof(struct ody_upcallstats));
	EXIT;
        return 0;
}


static int
ody_psdev_release(struct inode * inode, struct file * file)
{
        struct ocomm *ocp;
        struct omsg *omp;
	unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
	ENTRY;

        ocp = ody_psdev2ocomm(file);
        
        if ( !ocp || !ocomm_open(ocp) ) {
              printk("psdev_release: not open");
              return 0;
        }
    
	
	/* flush the name cache so that we can unmount */
	ODEBUG(D_PSDEV, "Flushing the cache.\n");
	/* odync_flush(); */
	/* odync_use = 0; */
	ODEBUG(D_PSDEV, "Done.\n");
	
	/* if operations are in progress perhaps the kernel
	   can profit from setting the ON_DYING flag on the root 
	   cnode of Ody filesystems */
        if (ody_super_info[minor].sbi_root) {
                struct ody_inode_info *cnp = 
			ITOO(ody_super_info[minor].sbi_root);
                cnp->o_flags |= ON_DYING;
        } else 
		ocp->oc_inuse = 0;	
	
    
        /* Wakeup clients so they can return. */
        for (omp = q_getnext(&(ocp->oc_pending));
             !q_end(omp, &(ocp->oc_pending));
             omp = q_getnext(&(omp->om_chain))) {	    
              /* Free signal request messages and don't wakeup cause
                 no one is waiting. */
              if (omp->om_opcode == ODY_SIGNAL) {
                    ODY_FREE(omp->om_data, sizeof(struct ody_in_hdr));
                    ODY_FREE(omp, (u_int)sizeof(struct omsg));
                    continue;
              }
              wake_up(&omp->om_sleep);
        }
        
        for (omp = q_getnext(&(ocp->oc_processing));
             !q_end(omp, &(ocp->oc_processing));
             omp = q_getnext(&(omp->om_chain))) {
	        wake_up(&omp->om_sleep);
        }
        
        mark_ocomm_closed(ocp);
	MOD_DEC_USE_COUNT;
	EXIT;
	return 0;
}


static struct file_operations ody_psdev_fops = {
      NULL,			/* llseek */
      ody_psdev_read,		/* read */
      ody_psdev_write,		/* write */
      NULL,			/* ody_psdev_readdir */
      ody_psdev_poll,		/* poll */
      NULL,			/* ioctl */
      NULL,			/* ody_psdev_mmap */
      ody_psdev_open,		/* open */
      NULL,			/* flush */
      ody_psdev_release,	/* release */
      NULL,			/* fsync */
      NULL,			/* fasync */
      NULL,			/* check_media_change */
      NULL,			/* revalidate */
      NULL			/* lock */
};


#if defined(CONFIG_PROC_FS) && defined(ODY_PROC_FS)

struct proc_dir_entry proc_ody = {
        0, 4, "ody",
        S_IFDIR | S_IRUGO | S_IXUGO | S_IWUSR, 2, 0, 0,
        0, &proc_net_inode_operations,

};

struct proc_dir_entry proc_ody_ncstats =  {
                0 , 12, "ody-ncstats",
                S_IFREG | S_IRUGO, 1, 0, 0,
                0, &proc_net_inode_operations,
                odync_nc_info
        };

#endif


int init_ody_psdev(void)
{
	if(register_chrdev(ODY_PSDEV_MAJOR,"ody_psdev", &ody_psdev_fops)) {
              printk(KERN_ERR "ody_psdev: unable to get major %d\n", 
		     ODY_PSDEV_MAJOR);
              return -EIO;
	}
	memset(psdev_ocomm, 0, sizeof(psdev_ocomm));
	memset(ody_super_info, 0, sizeof(ody_super_info));
	memset(&ody_callstats, 0, sizeof(ody_callstats));

#if defined(CONFIG_PROC_FS) && defined(ODY_PROC_FS)
	proc_register(&proc_root,&proc_ody);
	proc_register(&proc_ody, &proc_ody_ncstats);
#endif

#ifdef CONFIG_SYSCTL
        ody_sysctl_init();
#endif

	return 0;
}


#ifdef MODULE

EXPORT_NO_SYMBOLS;

MODULE_AUTHOR("Peter J. Braam <braam@cs.cmu.edu>\n"
	      "Kip Walker <kwalker@cs.cmu.edu>");

int init_module(void)
{
	int status;
	printk(KERN_INFO "Ody Kernel/User communications module 2.0\n");

	status = init_ody_psdev();
	if ( status ) {
		printk("Problem (%d) in init_ody_psdev\n", status);
		return status;
	}

	status = init_ody_fs();
	if (status) {
		printk("ody: failed in init_ody_fs!\n");
	}
	
	uc_init_upclass (UC_ODYSSEY, &ody_ucld);
	
	return status;
}


void cleanup_module(void)
{
        int err;

        ENTRY;

        if ( (err = unregister_filesystem(&ody_fs_type)) != 0 ) {
                printk("ody: failed to unregister filesystem\n");
        }
        unregister_chrdev(ODY_PSDEV_MAJOR,"ody_psdev");

#ifdef CONFIG_PROC_FS
        ody_sysctl_clean();
#ifdef ODY_PROC_FS
        proc_unregister(&proc_ody, proc_ody_ncstats.low_ino);
	proc_unregister(&proc_root, proc_ody.low_ino);
#endif
#endif 
	printk(KERN_INFO "Ody communications module 2.0 exitting.\n");
}

#endif

