#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Symlink inode operations for Coda filesystem
 * Original version: (C) 1996 P. Braam and M. Callahan
 * Rewritten for Linux 2.1. (C) 1997 Carnegie Mellon University
 * 
 * Carnegie Mellon encourages users to contribute improvements to
 * the Coda project. Contact Peter Braam (coda@cs.cmu.edu).
 */

/*
 * stolen for Odyssey
 */

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/string.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>

static int ody_readlink(struct dentry *de, char *buffer, int length);
static struct dentry *ody_follow_link(struct dentry *, struct dentry *,
				      unsigned int);

struct inode_operations ody_symlink_inode_operations = {
	NULL,			/* no file-operations */
	NULL,			/* create */
	NULL,			/* lookup */
	NULL,			/* link */
	NULL,			/* unlink */
	NULL,			/* symlink */
	NULL,			/* mkdir */
	NULL,			/* rmdir */
	NULL,			/* mknod */
	NULL,			/* rename */
	ody_readlink,		/* readlink */
	ody_follow_link,     	/* follow_link */
	NULL,			/* readpage */
	NULL,			/* writepage */
	NULL,			/* bmap */
	NULL,			/* truncate */
	NULL,            	/* permission */
	NULL,                   /* smap */
	NULL,                   /* update page */
        NULL                    /* revalidate */
};

static int ody_readlink(struct dentry *de, char *buffer, int length)
{
	struct inode *inode = de->d_inode;
        int len;
	int error;
        char *buf;
	struct ody_inode_info *op;
        ENTRY;

        op = ITOO(inode);
        CHECK_ONODE(op);

        /* the maximum length we receive is len */
        if ( length > ODY_MAXPATHLEN ) 
	        len = ODY_MAXPATHLEN;
	else
	        len = length;
	ODY_ALLOC(buf, char *, len);
	if ( !buf ) 
	        return -ENOMEM;
	
	error = viceroy_readlink(inode->i_sb, &(op->o_fid), buf, &len);

        ODEBUG(D_INODE, "result %s\n", buf);
	if (! error) {
		copy_to_user(buffer, buf, len);
		put_user('\0', buffer + len);
		error = len;
	}
	if ( buf )
		ODY_FREE(buf, len);
	return error;
}

static struct dentry *ody_follow_link(struct dentry *de, 
				      struct dentry *base,
				      unsigned int follow)
{
	struct inode *inode = de->d_inode;
	int error;
	struct ody_inode_info *onp;
	unsigned int len;
	char mem[ODY_MAXPATHLEN];
	char *path;
	ENTRY;

	ODEBUG(D_INODE, "(%x/%ld)\n", inode->i_dev, inode->i_ino);
	
        onp = ITOO(inode);
        CHECK_ONODE(onp);

	len = ODY_MAXPATHLEN;
	error = viceroy_readlink(inode->i_sb, &(onp->o_fid), mem, &len);

	if (error) {
		dput(base);
		return ERR_PTR(error);
	}
	len = strlen(mem);
	path = kmalloc(len + 1, GFP_KERNEL);
	if (!path) {
		dput(base);
		return ERR_PTR(-ENOMEM);
	}
	memcpy(path, mem, len);
	path[len] = 0;

	base = lookup_dentry(path, base, follow);
	kfree(path);
	return base;
}
