#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Sysctl operations for Ody filesystem
 * Original version: (C) 1996 P. Braam and M. Callahan
 * Rewritten for Linux 2.1. (C) 1997 Carnegie Mellon University
 * 
 * Carnegie Mellon encourages users to contribute improvements to
 * the Ody project. Contact Peter Braam (ody@cs.cmu.edu).
 */
/* sysctl entries for Ody! */

#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/sysctl.h>
#include <linux/swapctl.h>
#include <linux/proc_fs.h>
#include <linux/malloc.h>
#include <linux/stat.h>
#include <linux/ctype.h>
#include <asm/bitops.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/utsname.h>

#include <linux/ody_fs_i.h>
#include "kernel/vfs/includes/ody.h"
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>
extern int ody_debug;
/* extern int cfsnc_use; */
extern int ody_print_entry;
/* extern int cfsnc_flushme; */
extern int cfsnc_procsize;
/* extern void cfsnc_flush(void); */
void ody_sysctl_init(void);
void ody_sysctl_clean(void);

int ody_dointvec(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp);

struct ctl_table_header *fs_table_header, *ody_table_header;
#define FS_ODY         1       /* Ody file system */

#define ODY_DEBUG  	 1	 /* control debugging */
#define ODY_ENTRY	 2       /* control enter/leave pattern */
#define ODY_TIMEOUT    3       /* timeout on upcalls to become intrble */
#define ODY_MC         4       /* use/do not use the access cache */
#define ODY_HARD       5       /* mount type "hard" or "soft" */

static ctl_table ody_table[] = {
	{ODY_DEBUG, "debug", &ody_debug, sizeof(int), 0644, NULL, &ody_dointvec},
	{ODY_ENTRY, "printentry", &ody_print_entry, sizeof(int), 0644, NULL, &ody_dointvec},
 	{ODY_MC, "accesscache", &ody_access_cache, sizeof(int), 0644, NULL, &ody_dointvec}, 
 	{ODY_TIMEOUT, "timeout", &ody_timeout, sizeof(int), 0644, NULL, &ody_dointvec},
 	{ODY_HARD, "hard", &ody_hard, sizeof(int), 0644, NULL, &ody_dointvec},
	{ 0 }
};


static ctl_table fs_table[] = {
       {FS_ODY, "ody",    NULL, 0, 0555, ody_table},
       {0}
};



void ody_sysctl_init()
{
	fs_table_header = register_sysctl_table(fs_table, 0);
/*	ody_table_header = register_sysctl_table(ody_table, 0);*/
}

void ody_sysctl_clean() {
	/*unregister_sysctl_table(ody_table_header);*/
	unregister_sysctl_table(fs_table_header);
}

int ody_dointvec(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp)
{
        int *i, vleft, first=1, len, left, neg, val;
        #define TMPBUFLEN 20
        char buf[TMPBUFLEN], *p;
        
        if (!table->data || !table->maxlen || !*lenp ||
            (filp->f_pos && !write)) {
                *lenp = 0;
                return 0;
        }
        
        i = (int *) table->data;
        vleft = table->maxlen / sizeof(int);
        left = *lenp;
        
        for (; left && vleft--; i++, first=0) {
                if (write) {
		        while (left) {
			        char c;
				if(get_user(c,(char *) buffer))
					return -EFAULT;
				if (!isspace(c))
					break;
				left--;
				((char *) buffer)++;
			}
                        if (!left)
                                break;
                        neg = 0;
                        len = left;
                        if (len > TMPBUFLEN-1)
                                len = TMPBUFLEN-1;
                        if (copy_from_user(buf, buffer, len))
			        return -EFAULT;
                        buf[len] = 0;
                        p = buf;
                        if (*p == '-' && left > 1) {
                                neg = 1;
                                left--, p++;
                        }
                        if (*p < '0' || *p > '9')
                                break;
                        val = simple_strtoul(p, &p, 0);
                        len = p-buf;
                        if ((len < left) && *p && !isspace(*p))
                                break;
                        if (neg)
                                val = -val;
                        buffer += len;
                        left -= len;
                        *i = val;
                } else {
                        p = buf;
                        if (!first)
                                *p++ = '\t';
                        sprintf(p, "%d", *i);
                        len = strlen(buf);
                        if (len > left)
                                len = left;
                        if (copy_to_user(buffer, buf, len))
			    return -EFAULT;
                        left -= len;
                        buffer += len;
                }
        }

        if (!write && !first && left) {
                if(put_user('\n', (char *) buffer))
		        return -EFAULT;
                left--, buffer++;
        }
        if (write) {
                p = (char *) buffer;
                while (left) {
			char c;
			if(get_user(c, p++))
				return -EFAULT;
			if (!isspace(c))
				break;
			left--;
		}
        }
        if (write && first)
                return -EINVAL;
        *lenp -= left;
        filp->f_pos += *lenp;
        return 0;
}
