#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Mostly platform independent upcall operations to Venus:
 *  -- upcalls
 *  -- upcall routines
 *
 * Linux 2.0 version
 * Copyright (C) 1996 Peter J. Braam <braam@maths.ox.ac.uk>, 
 * Michael Callahan <callahan@maths.ox.ac.uk> 
 * 
 * Redone for Linux 2.1
 * Copyright (C) 1997 Carnegie Mellon University
 *
 * Adapted for Odyssey
 * Kip Walker <kwalker@cs.cmu.edu>
 *
 * Carnegie Mellon University encourages users of this code to contribute
 * improvements to the Coda project. Contact Peter Braam <coda@cs.cmu.edu>.
 */

#include <asm/system.h>
#include <asm/segment.h>
#include <asm/signal.h>
#include <linux/signal.h>

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/string.h>
#include <asm/uaccess.h>
#include <linux/vmalloc.h>
#include <asm/segment.h>

#include <linux/ody_fs_i.h>
#include <linux/ody_linux.h>
#include <linux/ody_psdev.h>
#include <linux/ody_cache.h>

#include "kernel/sets/common/sets_internal.h"
#include "kernel/vfs/includes/odyapi.h"

#define UPARG(op)\
do {\
  	ODY_ALLOC(inp, union ody_inputArgs *, insize);\
        if ( !inp ) { return -ENOMEM; }\
	outp = (union ody_outputArgs *) (inp);\
        inp->ih.opcode = (op);\
	inp->ih.pid = current->pid;\
	inp->ih.pgid = current->pgrp;\
	ody_load_creds(&(inp->ih.cred));\
        outsize = insize;\
} while (0)

static inline int max(int a, int b) 
{
	if ( a > b )
		return a; 
	else
		return b;
}

#define INSIZE(tag) sizeof(struct ody_ ## tag ## _in)
#define OUTSIZE(tag) sizeof(struct ody_ ## tag ## _out)
#define SIZE(tag)  max(INSIZE(tag), OUTSIZE(tag))


/* the upcalls */
int viceroy_rootfid(struct super_block *sb, struct ofid *fidp)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	ENTRY;

        insize = SIZE(root);
        UPARG(ODY_ROOT);
	error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);
	
	if (error) {
	        printk("ody_get_rootfid: error %d\n", error);
	} else {
	        *fidp = (struct ofid) outp->ody_root.fid;
		ODEBUG(D_SUPER, "ttype: %lx, tome: %lx, vol: %lx, uniq: %lx.\n",
		       fidp->ofid_ttype, fidp->ofid_tome,
		       fidp->ofid_vnum, fidp->ofid_uniq);
	}

	if (inp)
		ODY_FREE(inp, insize);
        EXIT;
	return error;
}

int viceroy_getattr(struct super_block *sb, struct ofid *fid, 
		    struct ody_vattr *attr) 
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	ENTRY;

        insize = SIZE(getattr); 
	UPARG(ODY_GETATTR);
        inp->ody_getattr.fid = *fid;
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);
	
	if ( !error )
	        *attr = outp->ody_getattr.attr;

        if (inp)
		ODY_FREE(inp, insize);
        EXIT;
        return error;
}

int viceroy_setattr(struct super_block *sb, struct ofid *fid, 
		     struct ody_vattr *vattr)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize= SIZE(setattr);
	UPARG(ODY_SETATTR);

        inp->ody_setattr.fid = *fid;
	inp->ody_setattr.attr = *vattr;

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

        ODEBUG(D_SUPER, "result %d\n", error); 
        if ( inp ) ODY_FREE(inp, insize);
        return error;
}

int viceroy_lookup(struct super_block *sb, struct ofid *fid, 
		   const char *name, int length, int * type, 
		   struct ofid *resfid)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset;

	offset = INSIZE(lookup);
        insize =  max(offset + length +1, OUTSIZE(lookup));
	UPARG(ODY_LOOKUP);

        inp->ody_lookup.fid = *fid;
	inp->ody_lookup.name = offset;
        /* send Viceroy a null terminated string */
        memcpy((char *)(inp) + offset, name, length);
        *((char *)inp + offset + length) = '\0';

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( !error ) {
	        *resfid = outp->ody_lookup.fid;
		*type = outp->ody_lookup.vtype;
	}
	if (inp) ODY_FREE(inp, insize);
		
	return error;
}


int viceroy_release(struct super_block *sb, struct ofid *fid, int flags)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize = SIZE(close);
	UPARG(ODY_CLOSE);

        inp->ody_close.fid = *fid;
        inp->ody_close.flags = flags;

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

        if (inp) 
		ODY_FREE(inp, insize);
        return error;
}

int viceroy_open(struct super_block *sb, struct ofid *fid,
		 int flag, ino_t *ino, dev_t *dev, int *flags)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
       
	insize = SIZE(open);
	UPARG(ODY_OPEN);

        inp->ody_open.fid = *fid;
        inp->ody_open.flags = flag;

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( !error ) {
	        *ino = outp->ody_open.inode;
	        *dev = outp->ody_open.dev;
		*flags = outp->ody_open.flags;
	} else {
	        *ino = 0;
		*dev = 0;
		*flags = 0;
	}

        if (inp) 
                ODY_FREE(inp, insize);

	return error;
}	

int viceroy_rdwr(struct super_block *sb, struct ofid *fid, loff_t *offset,
		 int ioflag, int rwflag, int *count, char *buff)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
       
	insize = OC_MAXMSGSIZE;
	UPARG(ODY_RDWR);

	ODEBUG(D_FILE, "ioflag %x\n", ioflag);
        inp->ody_rdwr.fid = *fid;
        inp->ody_rdwr.rwflag = rwflag;
        inp->ody_rdwr.ioflag = ioflag;
	inp->ody_rdwr.count = *count;
	inp->ody_rdwr.offset = (int)*offset;
	inp->ody_rdwr.data = sizeof(struct ody_rdwr_in);
	if (rwflag == O_RW_WRITE) {
		memcpy((char*)inp+(int)inp->ody_rdwr.data, buff, *count);
	}

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( !error ) {
	        *count = outp->ody_rdwr.count;
		*offset += *count;
		if (rwflag == O_RW_READ) {
			memcpy(buff, (char*)outp+(int)outp->ody_rdwr.data,
			       *count);
		}
	} else
		ODEBUG(D_FILE, "upcall failed - %d\n",
		       error);
	
        if (inp) 
                ODY_FREE(inp, insize);
	
	return error;
}

int viceroy_mkdir(struct super_block *sb, struct ofid *dirfid, 
		  const char *name, int length, 
		  struct ofid *newfid, struct ody_vattr *attrs)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset;

	offset = INSIZE(mkdir);
	insize = max(offset + length + 1, OUTSIZE(mkdir));
	UPARG(ODY_MKDIR);

        inp->ody_mkdir.fid = *dirfid;
        inp->ody_mkdir.attr = *attrs;
	inp->ody_mkdir.name = offset;
        /* Viceroy must get null terminated string */
        memcpy((char *)(inp) + offset, name, length);
        *((char *)inp + offset + length) = '\0';
        
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	*attrs = outp->ody_mkdir.attr;
	*newfid = outp->ody_mkdir.fid;

	if (inp) 
	        ODY_FREE(inp, insize);
	return error;        
}


int viceroy_rename(struct super_block *sb, struct ofid *old_fid, 
		   struct ofid *new_fid, size_t old_length, 
		   size_t new_length, const char *old_name, 
		   const char *new_name)
{
	union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset, s;
	
	offset = INSIZE(rename);
	insize = max(offset + new_length + old_length + 8,
		     OUTSIZE(rename)); 
 	UPARG(ODY_RENAME);

        inp->ody_rename.sourceFid = *old_fid;
        inp->ody_rename.destFid =  *new_fid;
        inp->ody_rename.srcname = offset;

        /* Viceroy must receive an null terminated string */
        s = ( old_length & ~0x3) +4; /* round up to word boundary */
        memcpy((char *)(inp) + offset, old_name, old_length);
        *((char *)inp + offset + old_length) = '\0';

        /* another null terminated string for Viceroy */
        offset += s;
        inp->ody_rename.destname = offset;
        s = ( new_length & ~0x3) +4; /* round up to word boundary */
        memcpy((char *)(inp) + offset, new_name, new_length);
        *((char *)inp + offset + new_length) = '\0';

        ODEBUG(D_INODE, "destname in packet: %s\n", 
              (char *)inp + (int) inp->ody_rename.destname);
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if (inp) ODY_FREE(inp, insize);
	return error;
}

int viceroy_create(struct super_block *sb, struct ofid *dirfid, 
		   const char *name, int length, int excl, int mode, 
		   struct ofid *newfid, struct ody_vattr *attrs) 
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset;

        offset = INSIZE(create);
	insize = max(offset + length + 1, OUTSIZE(create));
	UPARG(ODY_CREATE);

        inp->ody_create.fid = *dirfid;
        inp->ody_create.attr.va_mode = mode;
	inp->ody_create.excl = excl;
        inp->ody_create.mode = mode;
        inp->ody_create.name = offset;

        /* Viceroy must get null terminated string */
        memcpy((char *)(inp) + offset, name, length);
        *((char *)inp + offset + length) = '\0';
                
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	*attrs = outp->ody_create.attr;
	*newfid = outp->ody_create.fid;

	if (inp) 
	        ODY_FREE(inp, insize);
	return error;        
}

int viceroy_readdir(struct super_block *sb, struct ofid *dirfid,
		    int count, loff_t offset, caddr_t buff, int *size)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;

	insize = OC_MAXMSGSIZE;
	UPARG(ODY_READDIR);

	inp->ody_readdir.fid = *dirfid;
	inp->ody_readdir.count = count;
	inp->ody_readdir.offset = offset;

	ODEBUG(D_FILE, "dirofid %s\n", ody_f2s(dirfid));

	error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);
	if (!error) {
		memcpy(buff, (char *)outp+(int)outp->ody_readdir.data,
		       outp->ody_readdir.size);
		*size = outp->ody_readdir.size;
	}

	if (inp)
		ODY_FREE(inp, insize);

	return error;
}

int viceroy_rmdir(struct super_block *sb, struct ofid *dirfid, 
		  const char *name, int length)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset;

        offset = INSIZE(rmdir);
	insize = max(offset + length + 1, OUTSIZE(rmdir));
	UPARG(ODY_RMDIR);

        inp->ody_rmdir.fid = *dirfid;
        inp->ody_rmdir.name = offset;
        memcpy((char *)(inp) + offset, name, length);
	*((char *)inp + offset + length) = '\0';
        
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);
	if ( inp ) 
	        ODY_FREE(inp, insize);
	return error;
}

int viceroy_remove(struct super_block *sb, struct ofid *dirfid, 
		   const char *name, int length)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
	unsigned long insize, outsize;
        int error=0, offset;

        offset = INSIZE(remove);
	insize = max(offset + length + 1, OUTSIZE(remove));
	UPARG(ODY_REMOVE);

        inp->ody_remove.fid = *dirfid;
        inp->ody_remove.name = offset;
        memcpy((char *)(inp) + offset, name, length);
	*((char *)inp + offset + length) = '\0';
        
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);
	if ( inp ) 
	        ODY_FREE(inp, insize);
	return error;
}

int viceroy_readlink(struct super_block *sb, struct ofid *fid, 
		     char *buffer, int *length)
{ 
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, retlen;
        char *result;
        
	insize = max(INSIZE(readlink), OUTSIZE(readlink)+ *length + 1);
	UPARG(ODY_READLINK);

        inp->ody_readlink.fid = *fid;
    
        error =  ody_upcall(ody_sbp(sb), insize, &outsize, inp);
	
	if (! error) {
                retlen = outp->ody_readlink.count;
		if ( retlen > *length )
		        retlen = *length;
		*length = retlen;
		result =  (char *)outp + (int)outp->ody_readlink.data;
		memcpy(buffer, result, retlen);
		*(buffer + retlen) = '\0';
	}
        
        if (inp) ODY_FREE(inp, insize);
        ODEBUG(D_INODE, "result %d\n",error);
        EXIT;
        return error;
}



int viceroy_link(struct super_block *sb, struct ofid *fid, 
		 struct ofid *dirfid, const char *name, int len )
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset;

	offset = INSIZE(link);
	insize = max(offset  + len + 1, OUTSIZE(link));
        UPARG(ODY_LINK);

        inp->ody_link.sourceFid = *fid;
        inp->ody_link.destFid = *dirfid;
        inp->ody_link.tname = offset;

        /* make sure strings are null terminated */
        memcpy((char *)(inp) + offset, name, len);
        *((char *)inp + offset + len) = '\0';
        
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

        if (inp) 
	        ODY_FREE(inp, insize);
        ODEBUG(D_INODE, "result %d\n",error);
        EXIT;
        return error;
}

int viceroy_symlink(struct super_block *sb, struct ofid *fid,
		    const char *name, int len,
		    const char *symname, int symlen)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset, s;

        offset = INSIZE(symlink);
	insize = max(offset + len + symlen + 8, OUTSIZE(symlink));
	UPARG(ODY_SYMLINK);
        
        /*        inp->ody_symlink.attr = *tva; XXXXXX */ 
        inp->ody_symlink.fid = *fid;

	/* Round up to word boundary and null terminate */
        inp->ody_symlink.srcname = offset;
        s = ( symlen  & ~0x3 ) + 4; 
        memcpy((char *)(inp) + offset, symname, symlen);
        *((char *)inp + offset + symlen) = '\0';
        
	/* Round up to word boundary and null terminate */
        offset += s;
        inp->ody_symlink.tname = offset;
        s = (len & ~0x3) + 4;
        memcpy((char *)(inp) + offset, name, len);
        *((char *)inp + offset + len) = '\0';

	error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

        if (inp) 
	        ODY_FREE(inp, insize);
        ODEBUG(D_INODE, "result %d\n",error);
        EXIT;
        return error;
}

int viceroy_fsync(struct super_block *sb, struct ofid *fid)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(fsync);
	UPARG(ODY_FSYNC);

        inp->ody_fsync.fid = *fid;
        error = ody_upcall(ody_sbp(sb), sizeof(union ody_inputArgs), 
                            &outsize, inp);

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;
}

int viceroy_access(struct super_block *sb, struct ofid *fid, int mask)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;

	insize = SIZE(access);
	UPARG(ODY_ACCESS);

        inp->ody_access.fid = *fid;
        inp->ody_access.flags = mask;

	error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if (inp) ODY_FREE(inp, insize);
        EXIT;
	return error;
}


int viceroy_pioctl(struct super_block *sb, struct ofid *fid,
		   unsigned int cmd, struct PioctlData *data)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, iocsize;

	insize = OC_MAXMSGSIZE;
	UPARG(ODY_IOCTL);

        /* build packet for Viceroy */
        if (data->vi.in_size > OC_MAXDATASIZE) {
	        error = EINVAL;
		goto exit;
        }

        inp->ody_ioctl.fid = *fid;
    
        /* the cmd field was mutated by increasing its size field to
         * reflect the path and follow args. We need to subtract that
         * out before sending the command to Viceroy.  */
        inp->ody_ioctl.cmd = (cmd & ~(PIOCPARM_MASK << 16));	
        iocsize = ((cmd >> 16) & PIOCPARM_MASK) - sizeof(char *) - sizeof(int);
        inp->ody_ioctl.cmd |= (iocsize & PIOCPARM_MASK) <<	16;	
    
        /* in->ody_ioctl.rwflag = flag; */
        inp->ody_ioctl.len = data->vi.in_size;
        inp->ody_ioctl.data = (char *)(INSIZE(ioctl));
     
        /* get the data out of user space */
#ifdef L20
        memcpy_fromfs((char*)inp + (int)inp->ody_ioctl.data,
                      data->vi.in, data->vi.in_size);
#else
        if ( copy_from_user((char*)inp + (int)inp->ody_ioctl.data,
			    data->vi.in, data->vi.in_size) ) {
	        error = EINVAL;
	        goto exit;
	}
#endif
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);
        
        if (error) {
	        printk("ody_pioctl: Viceroy returns: %d for %s\n", 
		       error, ody_f2s(fid));
		goto exit; 
	}
        
	/* Copy out the OUT buffer. */
        if (outp->ody_ioctl.len > data->vi.out_size) {
                ODEBUG(D_FILE, "return len %d <= request len %d\n",
                      outp->ody_ioctl.len, 
                      data->vi.out_size);
                error = EINVAL;
        } else {
		error = verify_area(VERIFY_WRITE, data->vi.out, 
                                    data->vi.out_size);
		if ( error ) goto exit;
#ifdef L20
                memcpy_tofs(data->vi.out, 
                            (char *)outp + (int)outp->ody_ioctl.data, 
                            data->vi.out_size);                
#else
		if (copy_to_user(data->vi.out, 
				 (char *)outp + (int)outp->ody_ioctl.data, 
				 data->vi.out_size)) {
		        error = EINVAL;
			goto exit;
		}
#endif
        }

 exit:
        if (inp) 
	        ODY_FREE(inp, insize);
	return error;
}

/* Odyssey API upcalls */

int
viceroy_request(struct super_block *sb, struct ofid *fid,
		u_int rsrc, u_int gs, u_int type, u_int ts,
		int low, int high,
		int *cancelled, int *value, int *result)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(request);
	UPARG(ODY_REQUEST);

        inp->ody_request.fid = *fid;
	inp->ody_request.rsrc = rsrc;
	inp->ody_request.gs = gs;
	inp->ody_request.type = type;
	inp->ody_request.ts = ts;
	inp->ody_request.low = low;
	inp->ody_request.high = high;
	
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if (!error || error == -ODY_ENOTINWINDOW) {
		*cancelled = outp->ody_request.cancelled;
		*value = outp->ody_request.value;
		*result = outp->ody_request.result;
	}

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;
}

int
viceroy_cancel(struct super_block *sb, int reqid)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(cancel);
	UPARG(ODY_CANCEL);

        inp->ody_cancel.reqid = reqid;
	
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;
}

/* XXX will the viceroy actually get the pid? XXX */
int
viceroy_cancpid(struct super_block *sb)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(cancpid);
	UPARG(ODY_CANCPID);

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;	
}

int
viceroy_tsop(struct super_block *sb, struct ofid *fid,
	     u_int gs, u_int type, u_int ts, u_int op,
	     size_t argsz, caddr_t src_data,
	     size_t *retsz, caddr_t dest_data)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize, allocsize;
	int error;
	
	insize = max(INSIZE(tsop)+argsz,OUTSIZE(tsop)+*retsz);
	allocsize = insize;
	UPARG(ODY_TSOP);
	insize = INSIZE(tsop)+argsz;
	outsize = OUTSIZE(tsop)+*retsz;

	ODEBUG(D_PIOCTL, "%d.%d.%d.%d\n", gs, type, ts, op);
	
	inp->ody_tsop.fid = *fid;
	inp->ody_tsop.gs = gs;
	inp->ody_tsop.type = type;
	inp->ody_tsop.ts = ts;
	inp->ody_tsop.op = op;
	inp->ody_tsop.argsz = argsz;
	inp->ody_tsop.retsz = *retsz;
	inp->ody_tsop.data = SIZE(tsop);
	if (argsz) {
		copy_from_user((char *)inp+(int)(inp->ody_tsop.data),
			       src_data, argsz);
	}

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if (!error) {
		if (outp->ody_tsop.retsz > *retsz) {
			panic("viceroy_tsop: retbuf overgrowth!\n");
		}
		*retsz = outp->ody_tsop.retsz;
		copy_to_user(dest_data, (char *)outp+(int)outp->ody_tsop.data,
		       *retsz);
	}

	if (inp)
		ODY_FREE(inp, allocsize);
	return error;
}

int
viceroy_bmod_start(struct super_block *sb, int joules, float charge)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(bmod_start);
	UPARG(ODY_BMOD_START);

        inp->ody_bmod_start.joules = joules;
        inp->ody_bmod_start.charge = charge;
	
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;
}

int
viceroy_bmod_stop(struct super_block *sb)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(bmod_stop);
	UPARG(ODY_BMOD_STOP);

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;
}

int
viceroy_time_to_recharge(struct super_block *sb, int seconds)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
	
	insize=SIZE(time_to_recharge);
	UPARG(ODY_TIME_TO_RECHARGE);

        inp->ody_time_to_recharge.seconds = seconds;
	
        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( inp ) 
		ODY_FREE(inp, insize);
	return error;
}

int
viceroy_multif(struct super_block *sb,
	       u_int gs, u_int type, u_int ts, u_int op,
	       size_t argsz, caddr_t src_data,
	       size_t *retsz, caddr_t dest_data)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize, allocsize;
	int error;
	
	insize = max(INSIZE(multif)+argsz,OUTSIZE(multif)+*retsz);
	allocsize = insize;
	UPARG(ODY_MULTIF);
	insize = INSIZE(multif)+argsz;
	outsize = OUTSIZE(multif)+*retsz;

	ODEBUG(D_PIOCTL, "%d.%d.%d.%d\n", gs, type, ts, op);
	
	inp->ody_multif.gs = gs;
	inp->ody_multif.type = type;
	inp->ody_multif.ts = ts;
	inp->ody_multif.op = op;
	inp->ody_multif.argsz = argsz;
	inp->ody_multif.retsz = *retsz;
	inp->ody_multif.data = SIZE(multif);
	if (argsz) {
		copy_from_user((char *)inp+(int)(inp->ody_multif.data),
			       src_data, argsz);
	}

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if (!error) {
		if (outp->ody_multif.retsz > *retsz) {
			panic("viceroy_multif: retbuf overgrowth!\n");
		}
		*retsz = outp->ody_multif.retsz;
		copy_to_user(dest_data, (char *)outp+(int)outp->ody_multif.data,
		       *retsz);
	}

	if (inp)
		ODY_FREE(inp, allocsize);
	return error;
}

int viceroy_expand(struct super_block *sb, struct ofid *fid, 
		   const char *name, int length, int * type, 
		   struct ofid *resfid)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error, offset;

	offset = INSIZE(expand);
        insize =  max(offset + length +1, OUTSIZE(expand));
	UPARG(ODY_EXPAND);

        inp->ody_expand.fid = *fid;
	inp->ody_expand.name = offset;
        /* send Viceroy a null terminated string */
        memcpy((char *)(inp) + offset, name, length);
        *((char *)inp + offset + length) = '\0';

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( !error ) {
	        *resfid = outp->ody_expand.fid;
		*type = outp->ody_expand.vtype;
	}
	if (inp) ODY_FREE(inp, insize);
		
	return error;
}


int viceroy_cursexpand(struct super_block *sb, struct ofid *fid, 
		       char **buf, size_t *bufsize)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;

	insize = outsize = OC_MAXMSGSIZE;
	UPARG(ODY_CURSEXPAND);

        inp->ody_cursexpand.fid = *fid;
	inp->ody_cursexpand.bufsz = OC_MAXDATASIZE;

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( !error ) {
		*bufsize = outp->ody_cursexpand.retsz;
		SETS_ALLOC((*buf), char *, *bufsize);
		memcpy(*buf, (char *)outp+outp->ody_cursexpand.data, *bufsize);
	} else {
		if (error != 0xe0f)
			printf("cursexpand error %d\n", error);
		*bufsize = 0;
		*buf = NULL;
	}

	if (inp) ODY_FREE(inp, insize);
		
	return error;	
}

int viceroy_prefetch(struct super_block *sb, struct ofid *fid,
		     int flag, ino_t *ino, dev_t *dev, int *flags)
{
        union ody_inputArgs *inp;
        union ody_outputArgs *outp;
        unsigned long insize, outsize;
	int error;
       
	insize = SIZE(prefetch);
	UPARG(ODY_PREFETCH);

        inp->ody_open.fid = *fid;
        inp->ody_open.flags = flag;

        error = ody_upcall(ody_sbp(sb), insize, &outsize, inp);

	if ( !error ) {
	        *ino = outp->ody_open.inode;
	        *dev = outp->ody_open.dev;
		*flags = outp->ody_open.flags;
	} else {
	        *ino = 0;
		*dev = 0;
		*flags = 0;
	}

        if (inp) 
                ODY_FREE(inp, insize);

	return error;
}	


/*
 * ody_upcall and ody_downcall routines.
 * 
 */

static inline unsigned long ody_waitfor_upcall(struct omsg *omp)
{
	struct wait_queue	wait = { current, NULL };
	unsigned long posttime;

	omp->om_posttime = jiffies;
	posttime = jiffies;

	add_wait_queue(&omp->om_sleep, &wait);
	for (;;) {
		if ( ody_hard == 0 ) 
			current->state = TASK_INTERRUPTIBLE;
		else
			current->state = TASK_UNINTERRUPTIBLE;

		/* got a reply */
		if ( omp->om_flags & VM_WRITE )
			break;

 		if ( !ody_hard && signal_pending(current) ) {
 			/* if this process really wants to die, let it go */
 			if ( sigismember(&(current->signal), SIGKILL) ||
 			     sigismember(&(current->signal), SIGINT) )
 				break;
			/* signal is present: after timeout always return 
 			   really smart idea, probably useless ... */
 			if ( jiffies > omp->om_posttime + ody_timeout * HZ )
			    break; 
 		}
 		schedule();
	}
	remove_wait_queue(&omp->om_sleep, &wait);
	current->state = TASK_RUNNING;

	ODEBUG(D_SPECIAL, "posttime: %ld, returned: %ld\n",
	       posttime, jiffies-posttime);

	return (jiffies - posttime);
}


/*
 * ody_upcall will return an error in the case of
 * failed communication with Viceroy _or_ will peek at Viceroy
 * reply and return Viceroy's error.
 *
 * As Viceroy has 2 types of errors, normal errors (positive) and internal
 * errors (negative), normal errors are negated, while internal errors
 * are all mapped to -EINTR, while showing a nice warning message. (jh)
 *
 */
int ody_upcall(struct ody_sb_info *sbi, unsigned long inSize,
	       unsigned long *outSize, 
	       union ody_inputArgs *buffer) 
{
	unsigned long runtime;
	struct ocomm *ocommp;
	union ody_outputArgs *out;
	struct omsg *omp;
	int error = 0;
	ENTRY;

	if (sbi->sbi_ocomm == NULL) {
                return -ENODEV;
	}
	ocommp = sbi->sbi_ocomm;

	if (!ocomm_open(ocommp))
                return -ENODEV;

	/* Format the request message. */
	ODY_ALLOC(omp,struct omsg *,sizeof(struct omsg));
	omp->om_data = (void *)buffer;
	omp->om_flags = 0;
	omp->om_inSize = inSize;
	omp->om_outSize = *outSize ? *outSize : inSize;
	omp->om_opcode = ((union ody_inputArgs *)buffer)->ih.opcode;
	omp->om_unique = ++ocommp->oc_seq;
        omp->om_sleep = NULL;
	
	/* Fill in the common input args. */
	((union ody_inputArgs *)buffer)->ih.unique = omp->om_unique;

	/* Append msg to pending queue and poke Viceroy. */
	ody_q_insert(&(omp->om_chain), &(ocommp->oc_pending));
	ODEBUG(D_UPCALL, 
	       "Proc %d wake Viceroy for(opc,uniq) =(%d,%d) msg at %x.zzz.\n",
	       current->pid, omp->om_opcode, omp->om_unique, (int)omp);

	wake_up_interruptible(&ocommp->oc_waitq);
	/* We can be interrupted while we wait for Viceroy to process
	 * our request.  If the interrupt occurs before Viceroy has read
	 * the request, we dequeue and return. If it occurs after the
	 * read but before the reply, we dequeue, send a signal
	 * message, and return. If it occurs after the reply we ignore
	 * it. In no case do we want to restart the syscall.  If it
	 * was interrupted by a viceroy shutdown (psdev_close), return
	 * ENODEV.  */

	/* Go to sleep.  Wake up on signals only after the timeout. */
	runtime = ody_waitfor_upcall(omp);

	ODEBUG(D_TIMING, "opc: %d time: %ld uniq: %d size: %ld\n",
	       omp->om_opcode, jiffies - omp->om_posttime, 
	       omp->om_unique, omp->om_outSize);
	ODEBUG(D_UPCALL, 
	       "..process %d woken up by Viceroy for omp at 0x%x,data at %x\n",
	       current->pid, (int)omp, (int)omp->om_data);
	if (ocomm_open(ocommp)) {      /* i.e. Viceroy is still alive */
		/* Op went through, interrupt or not... */
		if (omp->om_flags & VM_WRITE) {
			out = (union ody_outputArgs *)omp->om_data;
			/* here we map positive Viceroy errs to kernel errs */
			if ( out->oh.result < 0 ) {
				printk("VICEROY RETURNS NEG. ERROR %ld, "
				       "for oc %ld\n",
				       out->oh.result, out->oh.opcode);
				out->oh.result = EINTR;
			}
			error = -out->oh.result;
			ODEBUG(D_UPCALL, 
			       "upcall: (u,o,r) (%ld, %ld, %ld) out at %p\n", 
			       out->oh.unique, out->oh.opcode, out->oh.result,
			       out);
			*outSize = omp->om_outSize;
			goto exit;
		}
		if ( !(omp->om_flags & VM_READ) && signal_pending(current)) { 
			/* Interrupted before viceroy read it. */
			ODEBUG(D_UPCALL, 
			       "Interrupted before read:(op,un) (%d.%d), "
			       "flags = %x\n",
			       omp->om_opcode, omp->om_unique, omp->om_flags);
			ody_q_remove(&(omp->om_chain));
			/* perhaps the best way to convince the app to
			   give up? */
			error = -EINTR;
			goto exit;
		} 
		if ( (omp->om_flags & VM_READ) && signal_pending(current) ) {
			/* interptd after Viceroy did its read, send signal */
			union ody_inputArgs *dog;
			struct omsg *somp;
			
			ODEBUG(D_UPCALL, 
			       "Sending Viceroy a signal: op = %d.%d, "
			       "flags = %x\n",
			       omp->om_opcode, omp->om_unique, omp->om_flags);
			
			ody_q_remove(&(omp->om_chain));
			error = -EINTR;
			ODY_ALLOC(somp, struct omsg *, sizeof (struct omsg));
			ODY_ALLOC((somp->om_data), char *,
				  sizeof(struct ody_in_hdr));
			
			dog = (union ody_inputArgs *)somp->om_data;
			dog->ih.opcode = ODY_SIGNAL;
			dog->ih.unique = omp->om_unique;
			
			somp->om_flags = 0;
			somp->om_opcode = dog->ih.opcode;
			somp->om_unique = dog->ih.unique;
			somp->om_inSize = sizeof(struct ody_in_hdr);
			somp->om_outSize = sizeof(struct ody_in_hdr);
			ODEBUG(D_UPCALL, 
			       "ody_upcall: enqueing signal msg (%d, %d)\n",
			       somp->om_opcode, somp->om_unique);
			
			/* insert at head of queue! */
			ody_q_insert(&(somp->om_chain),
				     ocommp->oc_pending.forw);
			wake_up_interruptible(&ocommp->oc_waitq);
		} else {
			printk("Ody: Strange interruption..\n");
			error = -EINTR;
		}
	} else {	/* If viceroy died i.e. !OC_OPEN(ocommp) */
	        printk("ody_upcall: Viceroy dead on (op,un)(%d.%d) flags %d\n",
		       omp->om_opcode, omp->om_unique, omp->om_flags);
		error = -ENODEV;
	}
	
 exit:
	ODY_FREE(omp, sizeof(struct omsg));
	if (error) 
	        badclstats();
	return error;
}




/* 
 * There are 7 cases where cache invalidations occur.  The semantics
 *  of each is listed here:
 *
 * ODY_FLUSH     -- flush all entries from the name cache and the cnode cache.
 * ODY_PURGEUSER -- flush all entries from the name cache for a specific user
 *                  This call is a result of token expiration.
 *
 * The next arise as the result of callbacks on a file or directory.
 * ODY_ZAPFILE   -- flush the cached attributes for a file.

 * ODY_ZAPDIR    -- flush the attributes for the dir and
 *                  force a new lookup for all the children
                    of this dir.

 *
 * The next is a result of Viceroy detecting an inconsistent file.
 * ODY_PURGEFID  -- flush the attribute for the file
 *                  purge it and its children from the dcache
 *
 * The last  allows Viceroy to replace local fids with global ones
 * during reintegration.
 *
 * ODY_REPLACE -- replace one ofid with another throughout the name cache */

int ody_downcall(int opcode, union ody_outputArgs * out,
		 struct super_block *sb)
{

	/* Handle invalidate requests. */
	if (!sb) {
		printk("ody_downcall: opcode %d, no sb!\n", opcode);
		return 0;
	}
	
	switch (opcode) {

	case ODY_FLUSH : {
		clstats(ODY_FLUSH);
		ODEBUG(D_DOWNCALL, "ODY_FLUSH\n");
		ody_cache_clear_all(sb);
		shrink_dcache_sb(sb);
		return(0);
	}
	
	case ODY_PURGEUSER : {
		struct ody_cred *cred = &out->ody_purgeuser.cred;
		ODEBUG(D_DOWNCALL, "ODY_PURGEUSER\n");
		if ( !cred ) {
			printk("PURGEUSER: null cred!\n");
			return 0;
		}
		clstats(ODY_PURGEUSER);
		ody_cache_clear_cred(sb, cred);
		return(0);
	}

	case ODY_ZAPDIR : {
		struct inode *inode;
		struct ofid *fid = &out->ody_zapdir.fid;
		ODEBUG(D_DOWNCALL, "zapdir: fid = %s\n", ody_f2s(fid));
		clstats(ODY_ZAPDIR);
		
		inode = ody_fid_to_inode(fid, sb);
		if (inode) {
			ODEBUG(D_DOWNCALL, "zapdir: inode = %ld children flagged\n",
			       inode->i_ino);
			ody_purge_children(inode);
			ODEBUG(D_DOWNCALL, "zapdir: inode = %ld cache cleared\n",
			       inode->i_ino);
			ody_flag_inode(inode, ON_VATTR);
		} else
			ODEBUG(D_DOWNCALL, "zapdir: no inode\n");

		return(0);
	}
	
	case ODY_ZAPFILE : {
		struct inode *inode;
		struct ofid *fid = &out->ody_zapfile.fid;
		clstats(ODY_ZAPFILE);
		ODEBUG(D_DOWNCALL, "zapfile: fid = %s\n", ody_f2s(fid));

		inode = ody_fid_to_inode(fid, sb);
		if (inode) {
			ODEBUG(D_DOWNCALL, "zapfile: inode = %ld\n",
			       inode->i_ino);
			ody_flag_inode(inode, ON_VATTR);
		} else
			ODEBUG(D_DOWNCALL, "zapfile: no inode\n");
		return 0;
	}
	
	case ODY_PURGEFID : {
		struct inode *inode;
		struct ofid *fid = &out->ody_purgefid.fid;
		clstats(ODY_PURGEFID);
		ODEBUG(D_DOWNCALL, "purgefid: fid = %s\n", ody_f2s(fid));
		
		inode = ody_fid_to_inode(fid, sb);
		if (inode) {
			ODEBUG(D_DOWNCALL, "purgefid: inode = %ld\n",
			       inode->i_ino);
			ody_purge_children(inode);
			ody_purge_dentries(inode);
		} else
			ODEBUG(D_DOWNCALL, "purgefid: no inode\n");
		return 0;
	}
	
	case ODY_REPLACE : {
		struct inode *inode;
		struct ofid *fid = &out->ody_replace.OldFid;
		clstats(ODY_REPLACE);
		ODEBUG(D_DOWNCALL, "ODY_REPLACE\n");
		inode = ody_fid_to_inode(fid, sb);
		if ( inode ) {
			ODEBUG(D_DOWNCALL, "replace: inode = %ld\n",
			       inode->i_ino);
			ody_purge_dentries(inode);
		} else
			ODEBUG(D_DOWNCALL, "replace: no inode\n");
		return (0);
	}			   
	}
	return 0;
}
