#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* Library that multiplexes multifidelity calls onto a single system call */

#include "include/odytypes.h"
#include "include/odyssey.h"
#include "odyssey/multif/multif.h"
#include "odyssey/multif/libmultif.h"

static struct ody_vers vers = {ODY_GENERIC_INTERFACE, ODY_NO_TYPE,
			       MULTIF_VERSION};

int
register_fidelity(int num_params, fid_param_t *params, int num_fids,
		  int *optype_idp)
{
    int retval, optype_id;
    size_t argsz, retsz = sizeof(optype_id);
    void *argp, *ptr;

    argsz = sizeof(num_params) + num_params * sizeof(*params) 
	+ sizeof(num_fids);
    BALLOC(argp, argsz);
    ptr = argp;

    *(int *)ptr = num_params;
    ptr += sizeof(int);
   
    memcpy(ptr, params, num_params * sizeof(*params));
    ptr += num_params * sizeof(*params);

    * (int *) ptr = num_fids;
    ptr += sizeof(int);

    ASSERT(ptr == argp + argsz);

    retval = ody_multif(&vers, MULTIF_REGISTER_FIDELITY, argsz, argp, &retsz,
			&optype_id);
    
    FREE(argp);
    if (retval == 0) *optype_idp = optype_id;
    return retval;
}

int
hint_utility(int optype_id, func_t utility_func)
{
    /* NOTIMPLEMENTED */
    ASSERT(0);
    return 0;
}

int 
hint_constraint(int optype_id, quality_t which_quality, double constraint,
		quality_callback_t callback_fn)
{
    size_t retsz = 0, argsz = sizeof(optype_id) + sizeof(which_quality) +
	sizeof(constraint);
    void *argp, *ptr;
    int retval;

    if (callback_fn) {
	fprintf(stderr,"Callbacks not yet supported\n");
	return EOPNOTSUPP;
    }

    BALLOC(argp, argsz);
    ptr = argp;

    * (int *) ptr = optype_id;
    ptr += sizeof(int);
    * (quality_t *) ptr = which_quality;
    ptr += sizeof(quality_t);
    * (double *) ptr = constraint;
    ptr += sizeof(double);

    ASSERT(ptr == argp + argsz);

    retval = ody_multif(&vers, MULTIF_HINT_CONSTRAINT, argsz, argp, &retsz,
			NULL);
    FREE(argp);
    return retval;
}

int
hint_resource(int optype_id, rsrc_t which_resource, func_t rsrc_func)
{
    void *argp, *ptr;
    int retval;
    size_t argsz, retsz = 0;

    argsz = 4 * sizeof(int) + sizeof(rsrc_t) + 
	rsrc_func.num_cases * sizeof(casefunc_t);

    BALLOC(argp, argsz);
    ptr = argp;

    * (int *) ptr = optype_id;
    ptr += sizeof(int);
    * (rsrc_t *) ptr = which_resource;
    ptr += sizeof(rsrc_t);
    * (int *) ptr = rsrc_func.num_discrete;
    ptr += sizeof(int);
    * (int *) ptr = rsrc_func.num_continuous;
    ptr += sizeof(int);
    * (int *) ptr = rsrc_func.num_cases;
    ptr += sizeof(int);

    memcpy(ptr, rsrc_func.cases, sizeof(casefunc_t) * rsrc_func.num_cases);
    ptr += sizeof(casefunc_t) * rsrc_func.num_cases;

    ASSERT(ptr == argp + argsz);


    retval = ody_multif(&vers, MULTIF_HINT_RESOURCE, argsz, argp, &retsz,
			NULL);
    FREE(argp);
    return retval;
}

int 
begin_fidelity_op(int optype_id, int *opidp, int num_params,
		  fid_param_val_t *fidparams)
{
    int retval;
    size_t argsz, retsz;
    void *argp, *retp, *ptr;

    argsz = sizeof(optype_id) + sizeof(num_params) +
	num_params * sizeof(fid_param_val_t);

    BALLOC(argp, argsz);
    ptr = argp;
    * (int *) ptr = optype_id;
    ptr += sizeof(int);
    * (int *) ptr = num_params;
    ptr += sizeof(int);
    memcpy(ptr, fidparams, num_params * sizeof(fid_param_val_t));
    ptr += num_params * sizeof(fid_param_val_t);
    ASSERT(ptr == argp + argsz);

    retsz = sizeof(int) + num_params * sizeof(fid_param_val_t);
    BALLOC(retp, retsz);

    retval = ody_multif(&vers, MULTIF_BEGIN_FIDELITY_OP, argsz, argp, &retsz,
			retp);
    if (retval == 0) {
	ptr = retp;
	ASSERT(retsz == sizeof(int) + num_params * sizeof(fid_param_val_t));
	*opidp = * (int *) ptr;
	ptr += sizeof(int);
	memcpy(fidparams, ptr, num_params * sizeof(fid_param_val_t));
    }
    
    FREE(argp);
    FREE(retp);
    return retval;
}

int
end_fidelity_op(int optype_id, int opid,
		int num_params, fid_param_val_t *fidparams,
		int num_fids, fid_param_val_t *fids)
{
    int retval;
    size_t argsz, retsz = 0;
    void *argp, *ptr;

    argsz = sizeof(optype_id) + sizeof(opid) +
	num_params * sizeof(fid_param_val_t) + 
	num_fids * sizeof(fid_param_val_t);

    BALLOC(argp, argsz);
    ptr = argp;

    * (int *) ptr = optype_id;
    ptr += sizeof(int);
    * (int *) ptr = opid;
    ptr += sizeof(int);
    memcpy(ptr, fidparams, num_params * sizeof(fid_param_val_t));
    ptr += num_params * sizeof(fid_param_val_t);
    memcpy(ptr, fids, num_fids * sizeof(fid_param_val_t));
    ptr += num_fids * sizeof(fid_param_val_t);

    ASSERT(ptr == argp + argsz);

    retval = ody_multif(&vers, MULTIF_END_FIDELITY_OP, argsz, argp, &retsz,
			NULL);
    FREE(argp);
    return retval;
}
