#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
  multif.h
 
  Implementation of the multifidelity API
*/

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdio.h>
#include "multif.h"

extern int find_best_fidelity(fidval *fp, int numfids, rsrc_constraint *rsrcp,
		       int numconstraints, double size, double *bestf);


static fidval *fidmap = NULL;
static int numfids = 0;

static rsrc_constraint *rsrcp = NULL;
static int numconstraints = 0;
static op_callback_t rsrc_callback;
static int cur_opid = -1, next_opid = 0;

int register_fidelity_map(IN int num, IN fidval *fvp)
{
    /* Probably should do some validation of the input arguments here */
    numfids = num;
    fidmap = fvp;
    fprintf(stderr,"register_fidelity_map: registered %d fidelity modes\n",
	    num);
    return 0;
}

int register_rsrc_constraints(IN int num, IN rsrc_constraint *rcp,
			      op_callback_t callback_fn)
{
    numconstraints = num;
    rsrcp = rcp;
    rsrc_callback = callback_fn;
    return 0;
}

static struct timeval op_begin_time;
static struct timeval op_begin_utime;

/* Signal beginning of an operation, and query the fidelity at which we
   should do it */
int begin_fidelity_op(OUT int *opid, IN double size, OUT
		      int *which_fidelity, OUT double *what_utility)
{
    int retval;
    struct rusage ru;

    /* currently we only allow one op in progress at a time */
    ASSERT(cur_opid == -1);
    cur_opid = next_opid;

    *opid = next_opid;
    next_opid++;

    retval = find_best_fidelity(fidmap, numfids, rsrcp, numconstraints, 
				size, what_utility);

    if (retval < 0) {
	fprintf(stderr, "Could not find a satisfactory fidval\n");
	exit(1);
    }

    fprintf(stderr, "Choosing mode %s at fidelity %f\n", 
	   fidmap[retval].fv_name, *what_utility);

    *which_fidelity = fidmap[retval].fv_id;

    gettimeofday(&op_begin_time, NULL);
    getrusage(RUSAGE_SELF, &ru);
    op_begin_utime = ru.ru_utime;
    return retval;
}
    

/* Signal end of an operation, and the fidelity at which we did it */
int end_fidelity_op(IN int opid, IN double size,
		    IN int which_fidelity, IN double what_utility)
{
    struct timeval op_end_time;
    struct rusage ru;
    struct timeval op_time, op_utime;

    getrusage(RUSAGE_SELF, &ru);
    timersub(&ru.ru_utime, &op_begin_utime, &op_utime);
    gettimeofday(&op_end_time, NULL);
    timersub(&op_end_time, &op_begin_time, &op_time);

    fprintf(stderr,"end_fidelity_op(%d, %f, %d, %f): time %ld.%06ld "
	    "utime %ld.%06ld\n", opid, size, which_fidelity, what_utility,
	    op_time.tv_sec, op_time.tv_usec, op_utime.tv_sec,
	    op_utime.tv_usec);

    ASSERT(cur_opid >= 0);
    cur_opid = -1;
    return 0;
}

