#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include "include/odytypes.h"
#include "kernel/vfs/includes/ody.h"
#include "kernel/vfs/includes/odyapi.h"

#include "base/ds/ds_thread.h"

#include "multif.h"
#include "multif_vic.h"
#include "multif_internal.h"
#include "libmultif.h"

/* This function does some hairy unmarshalling of the args to
   register_fidelity */

static
int do_register_fidelity(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    int num_params, num_fids;
    int *optype_idp;
    fid_param_t *params;
    void *ptr = argp;

    if (argsz < sizeof(num_params)) return EINVAL;
    num_params = * (int *) ptr;
    ptr += sizeof(num_params);
    if (num_params < 0) return EINVAL;
    if (argsz != sizeof(num_params) + num_params * sizeof(*params) +
	sizeof(num_fids))
	return EINVAL;
    params = ptr;
    ptr += num_params * sizeof(*params);
    num_fids = *(int *) ptr;

    if (*retsz < sizeof(int)) return EINVAL;
    ALLOC(optype_idp, int);
    *retsz = sizeof(int);
    *retp = optype_idp;

    return multif_register_fidelity(num_params, params, num_fids, 
				      optype_idp, cred, pid);
}

static
int do_hint_utility(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    return EOPNOTSUPP;
}

static
int do_hint_constraint(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    int optype_id;
    quality_t which_quality;
    double constraint;
    void *ptr = argp;

    if (argsz != sizeof(optype_id) + sizeof(which_quality) +
	sizeof(constraint))
	return EINVAL;

    optype_id = * (int *) ptr;
    ptr += sizeof(int);
    which_quality = * (quality_t *) ptr;
    ptr += sizeof(quality_t);
    constraint = * (double *) ptr;
    ptr += sizeof(double);

    *retsz = 0;

    return multif_hint_constraint(optype_id, which_quality, constraint,
				  cred, pid);
}

static
int do_hint_resource(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    int optype_id;
    rsrc_t which_resource;
    func_t rsrc_func;
    void *ptr = argp;

    if (argsz < 5 * sizeof(int) ) return EINVAL;

    optype_id = * (int *) ptr;
    ptr += sizeof(int);
    which_resource = * (rsrc_t *) ptr;
    ptr += sizeof(rsrc_t);
    rsrc_func.num_discrete = * (int *) ptr;
    ptr += sizeof(int);
    rsrc_func.num_continuous = * (int *) ptr;
    ptr += sizeof(int);
    rsrc_func.num_cases = * (int *) ptr;
    ptr += sizeof(int);

    if (rsrc_func.num_cases < 0) return EINVAL;
    if (argsz != 5 * sizeof(int) + rsrc_func.num_cases * sizeof(casefunc_t))
	return EINVAL;

    rsrc_func.cases = ptr;

    return multif_hint_resource(optype_id, which_resource, rsrc_func,
				  cred, pid);

}

static
int do_begin_fidelity_op(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    int optype_id;
    int *opidp;
    int num_params;
    fid_param_val_t *fidparams;
    void *ptr = argp;

    if (argsz < sizeof(optype_id) + sizeof(num_params)) return EINVAL;
    optype_id = * (int *) ptr;
    ptr += sizeof(int);
    num_params = * (int *) ptr;
    ptr += sizeof(int);
    if (num_params < 0) return EINVAL;
    if (argsz != sizeof(optype_id) + sizeof(num_params) +
	num_params * sizeof(fid_param_val_t))
	return EINVAL;
    fidparams = ptr;

    if (*retsz < sizeof(int) + num_params * sizeof(fid_param_val_t))
	return EINVAL;

    *retsz = sizeof(int) + num_params * sizeof(fid_param_val_t);
    BALLOC(*retp, *retsz);
    ptr = *retp;
    opidp = (int *) ptr;
    ptr += sizeof(int);
    memcpy(ptr, fidparams, num_params * sizeof(fid_param_val_t));
    fidparams = ptr;

    return multif_begin_fidelity_op(optype_id, opidp, num_params, fidparams,
				    cred, pid);

}

static
int do_end_fidelity_op(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    void *ptr;
    int optype_id, opid;
    int num_params, num_fids;
    fid_param_val_t *fidparams, *fids;

    ptr = argp;
    if (argsz < sizeof(optype_id) + sizeof(opid) + sizeof(num_params))
	return EINVAL;
    optype_id = * (int *) ptr;
    ptr += sizeof(int);
    opid  = * (int *) ptr;
    ptr += sizeof(int);
    num_params = * (int *) ptr;
    ptr += sizeof(int);
    if (num_params < 0) return EINVAL;
    if (argsz < sizeof(optype_id) + sizeof(opid) + sizeof(num_params)
	+ num_params * sizeof(fid_param_val_t) + sizeof(num_fids))
	return EINVAL;
    fidparams = ptr;
    ptr += num_params * sizeof(fid_param_val_t);
    num_fids = * (int *) ptr;
    ptr += sizeof(int);
    if (num_fids < 0) return EINVAL;
    if (argsz != sizeof(optype_id) + sizeof(opid) + sizeof(num_params)
	+ num_params * sizeof(fid_param_val_t) + sizeof(num_fids)
	+ num_fids * sizeof(fid_param_val_t))
	return EINVAL;
    fids = ptr;

    return multif_end_fidelity_op(optype_id, opid, num_params, fidparams,
				  num_fids, fids, cred, pid);
}

static
int do_test(argsz, argp, retsz, retp, cred, pid)
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    char *outstr = "Hi there!";
    char *instr = argp;

    if ((instr[argsz-1] != '\0') || (strlen(instr) != argsz-1)) {
	ds_thread_log(0, "MULTIF_TEST: string length is not %d", argsz-1);
	return EINVAL;
    }
    ds_thread_log(0, "MULTIF_TEST: got \"%s\" (%d)", argp, argsz);
    ds_thread_log(0, "MULTIF_TEST: returning \"%s\" (%d)", outstr,
		  strlen(outstr));
    *retsz = strlen(outstr);
    *retp = strdup(outstr); /* because multif_dispose will free it */
    return 0;
}

int multif_call(ts, opnum, argsz, argp, retsz, retp, cred, pid)
     unsigned int ts;
     unsigned int opnum;
     size_t argsz;
     void *argp;
     size_t *retsz;
     void **retp;
     struct ody_cred *cred;
     u_short pid;
{
    if (ts != MULTIF_VERSION) {
	ds_thread_log(0, "multif_call: version mismatch: %d instead of %d",
		      ts, MULTIF_VERSION);
	return ODY_EBADVERSION;
    }

    switch (opnum) {
    case MULTIF_REGISTER_FIDELITY:
	return do_register_fidelity(argsz, argp, retsz, retp, cred, pid);
    case MULTIF_HINT_UTILITY:
	return do_hint_utility(argsz, argp, retsz, retp, cred, pid);
    case MULTIF_HINT_CONSTRAINT:
	return do_hint_constraint(argsz, argp, retsz, retp, cred, pid);
    case MULTIF_HINT_RESOURCE:
	return do_hint_resource(argsz, argp, retsz, retp, cred, pid);
    case MULTIF_BEGIN_FIDELITY_OP:
	return do_begin_fidelity_op(argsz, argp, retsz, retp, cred, pid);
    case MULTIF_END_FIDELITY_OP:
	return do_end_fidelity_op(argsz, argp, retsz, retp, cred, pid);
    case MULTIF_TEST:
	return do_test(argsz, argp, retsz, retp, cred, pid);
    default:
	ds_thread_log(0,"multif_call: invalid opnum %d\n");
	return EINVAL;
    }
}
    
int multif_dispose(opnum, argsz, argp, retsz, retp, cred)
     unsigned int opnum;
     size_t argsz;
     void *argp;
     size_t retsz;
     void *retp;
     struct ody_cred *cred;
{
    ASSERT(retsz > 0);
    ASSERT(retp);
    FREE(retp);
    return 0;
}
