#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */


#define CMGR_HASHSIZE  67

#define CT_MAGIC  0x094b0c98
#define CIT_MAGIC 0xa296a70d

#define CMGR_VALID_ELT(eltp) (eltp && ((eltp)->ct_magic == CT_MAGIC))

#define CMGR_NAME_LEN   256

/* hash table entry for a connection */
typedef struct cmgr_elt {
    magic_t            ct_magic;
    cmgr_id_t          ct_connid;     /* the unique id */
    ds_hwlock_t        ct_lock;	      /* serialize access to connection */
    cmgr_connection_t  ct_export;     /* connection data that's exported */
    cmgr_initparams_t  ct_initparams; /* initparams used to create this conn */
    struct timeval     ct_last_valid; /* time last validated (or created) */
    int                ct_destroyed;  /* is it marked for destruction? */
    int                ct_refcount;   /* "puts" pending */
    int                ct_validating; /* someone trying to validate? */
    cmgr_status_t      ct_status;     /* is the connection alive or dead? */
    char               ct_name[CMGR_NAME_LEN]; /* Name of connection */
    struct timeval     ct_timeout;  /* timeout period set by user */
} cmgr_elt_t;

struct cmgr_iter_t {
    magic_t   cit_magic;
    ds_hash_iter_t *cit_hash_iter;
};

/* RPC2-related functions that only the connection manager calls */

/* create a new binding */
int cmgr_rpc2_create  __P((cmgr_rpc2_handle_t *crhp, 
			   cmgr_rpc2_initparams_t *cripp,
			   cmgr_status_t *status));

#if 0
/* get connection information */
int cmgr_rpc2_getinfo __P((cmgr_rpc2_handle_t *crhp, 
			   cmgr_rpc2_conninfo_t *crcp));
#endif

/* get connectin logs */
int cmgr_rpc2_getlogs __P((cmgr_rpc2_handle_t *crhp, 
			   cmgr_log_t *rttlog,
			   cmgr_log_t *tplog));

/* destroy a binding */
int cmgr_rpc2_destroy __P((cmgr_rpc2_handle_t *crhp));

/* Extract a name, and copy it into a string */
void cmgr_rpc2_name   __P((cmgr_rpc2_initparams_t *cripp, 
			   char *dst, int len));

/* Change the timeout associated with a binding */
int cmgr_rpc2_settimeout __P((cmgr_rpc2_handle_t *crhp,
			       struct timeval timo));
