#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * conn_mgr_rpc2.c
 * provides connection manager functionality such as bw/latency estimation
 * for RPC2 connections
 */

#include <stdlib.h>
#include <stdio.h>
#ifndef __linux__
#include <machine/limits.h>
#endif
#include <errno.h>
#include <string.h>

#include "base/ds/ds_thread.h"
#include "base/ds/ds_rrlist.h"

#include <lwp/lwp.h>
#include <rpc2/rpc2.h>

#include "include/cm_rpc2.h"
#include "include/cm.h"

/* create a new RPC2 binding */

int cmgr_rpc2_create(crhp, cripp, status)
    cmgr_rpc2_handle_t *crhp;
    cmgr_rpc2_initparams_t *cripp;
    cmgr_status_t *status;
{
    int rc;
    RPC2_Handle crh;

    *crhp = -1;

    rc = RPC2_NewBinding(&(cripp->hid), &(cripp->pid), &(cripp->sid),
			 &(cripp->bp), &crh);

    if (rc == RPC2_BADSERVER) {
	errno = CMGR_BADSERVER;
	return -1;
    }
    if (rc != RPC2_SUCCESS) {
        ds_thread_log (0, "Failed to create new RPC2 binding, rc = %d\n", rc);
	fflush (stdout);
	*status = CMGR_DEAD;
    }
    else {
	*status = CMGR_ALIVE;
	*crhp = (cmgr_rpc2_handle_t) crh;
    }

    return CMGR_SUCCESS;
}

int cmgr_rpc2_destroy(cmgr_rpc2_handle_t *crhp)
{
    int rc;

    rc = RPC2_Unbind(*crhp);
    if (rc == RPC2_SUCCESS) {
      return CMGR_SUCCESS;
      *crhp = -1;
    }
    else
      return CMGR_ERROR;
}

#if 0
/* get connection information */

/* NB: for now, the direction (in/out) is ignored; until I figure out how
to find this information from RPC2 log entries. The bandwidth returned
will be averaged over ALL the data on that connection. */

/* The averaging is done by considering log entries, least recent first,
   and adding them in using newbw = a*bw + (1-a)*obs = bw + (1-a)*(obs-bw)
   where obs is the bandwidth observed for that log entry.
   We use an a of 7/8
*/

#define BWSHIFT 3

#define WINDOW_TIME 30 /* seconds */

/* the latency field is also left untouched, for now */

int cmgr_rpc2_estimlevel = 1000;

int
cmgr_rpc2_getinfo(crhp,crcp)
    cmgr_rpc2_handle_t *crhp;
    cmgr_rpc2_conninfo_t *crcp;
{
    RPC2_Handle connid;
    RPC2_NetLog Log;
    RPC2_NetLog rttLog;
    RPC2_NetLogEntry Entries[RPC2_MAXLOGLENGTH];
    RPC2_NetLogEntry rttEntries[RPC2_MAXLOGLENGTH];
    int i;
    int rtt = 0;
    long ave_size = 0; /* ave transfer size in SELog */
    
    connid = *crhp;

    if (connid <= 0) {
	ds_thread_log(0, "Invalid RPC2_Handle %d", connid);
	ASSERT(0);
    }

    Log.NumEntries = RPC2_MAXLOGLENGTH;
    Log.Quantum = WINDOW_TIME*1000;
    Log.Entries = Entries;
    rttLog.NumEntries = RPC2_MAXLOGLENGTH;
    rttLog.Quantum = WINDOW_TIME*1000;
    rttLog.Entries = rttEntries;

    /* find average bandwidth of traffic seen over last window */
    /* a window is all log entries that are within WINDOW_TIME of
       the last log entry, plus one */

    /* for now, assume we won't have any RPC2_STATIC_NLE fields,
       i.e., that no one will call RPC2_PutNetInfo() on these
       connections */

    if (RPC2_GetNetInfo(connid, &rttLog, &Log) != RPC2_SUCCESS)
	return CMGR_ERROR;

    ds_thread_log(100, "cmgr_rpc2_getinfo: connection %d, %d entries in log",
		  connid, Log.ValidEntries);
 
    /* If we have no valid entries yet, just return (don't bother with
     * rtt either */
    if (!Log.ValidEntries) {
	    return CMGR_ERROR;
    }

    if (rttLog.ValidEntries) {	/* for sanity */
	struct timeval rtt_winstop = rttLog.Entries[0].TimeStamp;
	int prev_rtt = 0;

	for (i=0; i<rttLog.ValidEntries; i++) {
	    if (rttLog.Entries[i].TimeStamp.tv_sec <
		(rtt_winstop.tv_sec - WINDOW_TIME))
	    {
		break;
	    }
	}
	if (i == rttLog.ValidEntries) i--;
	for (; i>=0; i--) {
	    ds_thread_log(cmgr_rpc2_estimlevel+1,
			  "cmgr_rpc2_getinfo: Entry %d (%ld.%06ld): %d %d",
			  i, rttLog.Entries[i].TimeStamp.tv_sec, 
			  rttLog.Entries[i].TimeStamp.tv_usec,
                          rttLog.Entries[i].Value.Measured.ElapsedTime,
                          rttLog.Entries[i].Value.Measured.Bytes);
	    rtt = rttLog.Entries[i].Value.Measured.ElapsedTime;
	    if (prev_rtt == 0)
		prev_rtt = rtt;
	    else
		prev_rtt = rtt + ((prev_rtt - rtt) >> BWSHIFT);
	}
	rtt = prev_rtt;
	ds_thread_log(cmgr_rpc2_estimlevel, "Estimated rtt: %d (%d ents)", rtt,
                      rttLog.ValidEntries);
    }

    /* We now know there is at least one valid entry; it must be measured */
    ASSERT(Log.Entries[0].Tag == RPC2_MEASURED_NLE);

    crcp->crc_winstop = Log.Entries[0].TimeStamp;

    for (i=0; i < Log.ValidEntries; i++) {
	if (Log.Entries[i].TimeStamp.tv_sec < 
	       (crcp->crc_winstop.tv_sec - WINDOW_TIME) )
	{
	    break;
	}
    }
    if (i == Log.ValidEntries) i--;

    crcp->crc_winstart = Log.Entries[i].TimeStamp;
    crcp->crc_traffic = 0;

    /* Now do the aging/averaging stuff over the entries in our window,
       least recent first */

    for (; i >= 0; i--) {
	long bytes; /* bytes transferred */
	long bw;
	long transfer_time;
	long wiretime; /* diff between transfer time and latency */
	
	if (Log.Entries[i].Value.Measured.Conn != connid)
	  continue;

	bytes = Log.Entries[i].Value.Measured.Bytes;
	ASSERT(bytes < LONG_MAX/1000);

	if (ave_size == 0)
	    ave_size = bytes;
	else
	    ave_size = bytes + ((ave_size-bytes)>>BWSHIFT);

	transfer_time = Log.Entries[i].Value.Measured.ElapsedTime;
	wiretime = transfer_time - (rtt>>1);

	if (wiretime > 0) {
	    bw = bytes*1000/wiretime;

	    ds_thread_log(cmgr_rpc2_estimlevel,
			  "cmgr_rpc2_getinfo: Entry %d (%ld.%06ld): %dB/%d ms (%d B/s)",
			  i, Log.Entries[i].TimeStamp.tv_sec, 
			  Log.Entries[i].TimeStamp.tv_usec, 
			  bytes, transfer_time, bw);

	    if (crcp->crc_traffic == 0) {
		crcp->crc_bandwidth = bw;
	    } else {
		crcp->crc_bandwidth = 
		    bw + ((crcp->crc_bandwidth - bw) >> BWSHIFT);
	    }
	} /* if wiretime appears to be non-positive, bad entry */
	else
	    ds_thread_log(10,"cmgr_rpc2_getinfo: bad Entry %d (%ld.%06ld): %d bytes (ave %d), time %d ms, rtt %d ms",
			  i, Log.Entries[i].TimeStamp.tv_sec,
			  Log.Entries[i].TimeStamp.tv_usec,
			  bytes, ave_size, transfer_time, rtt);

	crcp->crc_traffic += Log.Entries[i].Value.Measured.Bytes;
    }

    /* succesfully filled in the bandwidth, traffic, winstart, and
       winstop fields */

    return CMGR_SUCCESS;
}
#endif

int
cmgr_rpc2_getlogs(crhp, rttlog, tplog)
    cmgr_rpc2_handle_t *crhp;
    cmgr_log_t         *rttlog;
    cmgr_log_t         *tplog;
{
    int rc;
    cmgr_rpc2_handle_t crh = *crhp;

    rc = RPC2_GetNetInfo(crh, rttlog, tplog);

    switch (rc) {
    case RPC2_SUCCESS:	{
	int new, old;
	
	/* delete all entries that don't correspond to this connection */
	
	for (old = 0, new = 0; old < rttlog->ValidEntries; old++) {
	    ASSERT((rttlog->Entries[old].Tag == RPC2_MEASURED_NLE) ||
		   (rttlog->Entries[old].Tag == RPC2_STATIC_NLE));
	    if (rttlog->Entries[old].Value.Measured.Conn == crh)
		rttlog->Entries[new++] = rttlog->Entries[old];
	    else
		ds_thread_log(20,"cmgr_getlogs: rttlogs [%d], conn %d has %d",
			      old, (int) crh,
			      (int) rttlog->Entries[old].Value.Measured.Conn);
	}
	rttlog->ValidEntries = new;
	
	for (old = 0, new = 0; old < tplog->ValidEntries; old++) {
	    ASSERT((tplog->Entries[old].Tag == RPC2_MEASURED_NLE) ||
		   (tplog->Entries[old].Tag == RPC2_STATIC_NLE));
	    if (tplog->Entries[old].Value.Measured.Conn == crh)
		tplog->Entries[new++] = tplog->Entries[old];
	    else
		ds_thread_log(20,"cmgr_getlogs: rttlogs [%d], conn %d has %d",
			      old, (int)crh,
			      (int) tplog->Entries[old].Value.Measured.Conn);
	}
	tplog->ValidEntries = new;

	return 0;
    }
    break;
    case RPC2_NOCONNECTION:
	return CMGR_CONNDOWN;
	break;
    default:
	ASSERT(0);
	break;
    }

    /* Should never reach here */
    return CMGR_ERROR;
}

		       

#define REMAIN(l,s)     ((len)-1-strlen(dst))

void
cmgr_rpc2_name(cripp, dst, len)
    cmgr_rpc2_initparams_t *cripp;
    char                   *dst;
    int                    len;
{
    unsigned long  addr;
    long           subsys;
    unsigned short port;
    char           tmp[1024];

    strncpy(dst, "", REMAIN(dst, len));
    strncat(dst, "RPC2 ", len-1-strlen(dst));
    if (REMAIN(dst, len) <= 0) return;

    switch(cripp->hid.Tag) {
    case RPC2_HOSTBYNAME:
	strncat(dst, cripp->hid.Value.Name, REMAIN(dst, len));
	break;
    case RPC2_HOSTBYINETADDR:
	addr = ntohl(cripp->hid.Value.InetAddress.s_addr);
	sprintf(tmp, "%lx.%lx.%lx.%lx", 
		(addr & 0xff000000) >> 24,
		(addr & 0xff0000) >> 16,
		(addr & 0xff00) >> 8,
		(addr & 0xff));
	strncat(dst, tmp, REMAIN(dst, len));
	break;
    case RPC2_DUMMYHOST:
	strncat(dst, "*", REMAIN(dst, len));
	break;
    default:
	ASSERT(0);
	break;
    }
    if (REMAIN(dst, len) <= 0) return;
    
    strncat(dst, ".", REMAIN(dst, len));
    if (REMAIN(dst, len) <= 0) return;

    switch(cripp->pid.Tag) {
    case RPC2_PORTBYNAME:
	strncat(dst, cripp->pid.Value.Name, REMAIN(dst, len));
	break;
    case RPC2_PORTBYINETNUMBER:
	port = ntohs(cripp->pid.Value.InetPortNumber);
	sprintf(tmp, "%d", port);
	strncat(dst, tmp, REMAIN(dst, len));
	break;
    case RPC2_DUMMYPORT:
	strncat(dst, "*", REMAIN(dst, len));
	break;
    default:
	ASSERT(0);
	break;
    }
    if (REMAIN(dst, len) <= 0) return;

    strncat(dst, ".", REMAIN(dst, len));
    if (REMAIN(dst, len) <= 0) return;

    switch (cripp->sid.Tag) {
    case RPC2_SUBSYSBYNAME:
	strncat(dst, cripp->sid.Value.Name, REMAIN(dst, len));
	break;
    case RPC2_SUBSYSBYID:
	subsys = ntohl(cripp->sid.Value.SubsysId);
	sprintf(tmp, "%ld", subsys);
	strncat(dst, tmp, REMAIN(dst, len));
	break;
    }

    return;
}

