#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * util.c -- general utilites for Onode Manager
 *
 * Written by Eric Tilton <tilt+@cs.cmu.edu>
 *
 * These functions provide general utilities (such as verifying
 * version numbers) for the onode manager package.
 *
 * History
 *   31 Jul 1996 --  0. 0. 1 -- Colorful first attempt
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include <kernel/vfs/includes/ody.h>

#include "include/viceroy_onode.h"
#include "include/onode-manager.h"

#include "om_storage.h"
/*
 * PUBLIC FUNCTIONS
 */

/*
 * onode_init -- initializes the Onode Manager. Must be called at least
 *               once before the package is used. Invocations after
 *               the package is initialized have no effect.
 *
 */

int onode_init(void) {
    /* do various initialization. */
    
    ds_lock_init(&om_lock);
    
    ds_thread_log(1000,"Onode manager initialized");
    return(0);
}

/*
 * onode_finalize -- cleans up the Onode Manager.
 */

int onode_finalize(void) {
    /* This currently does nothing, althought this may change at some later
       point. */
    ds_thread_log(1000,"Onode manager finalized");
    return(0);
}

/*
 * onode_version -- takes an expected version number and compares it against
 *                  the current version number. If they differ, it complains
 *                  to stderr and returns the correct version number.
 *                  If they match, 0 is returned.
 */

int onode_version(int expectedversion) {
    int rc;
    
    rc = (expectedversion == ONODE_MGR_VERSION ? 0 : ONODE_MGR_VERSION);
    if(rc) {
	fprintf(stderr, "onode_version: Onode Manager library version (%d) "
		"differs from expected version (%d)\n",
		ONODE_MGR_VERSION, expectedversion);
    }
    return(rc);
}

