#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/queue.h>
#include <sys/param.h>

#include "kernel/upcall/includes/upcall.h"
#include <kernel/vfs/includes/ody.h>
#include <kernel/vfs/includes/ody_upcall.h>
#include "include/viceroy_onode.h"

#include "include/odytypes.h"
#include "include/odyssey.h"
#include <lwp/lwp.h>
#include <lwp/lock.h>
#include <rpc2/rpc2.h>
#include "base/ds/ds_log.h"
#include "base/ds/ds_thread.h"
#include "base/ds/ds_rrlist.h"
#include "base/ds/ds_hash.h"
#include "base/ds/ds_hwlock.h"
#include "include/cm_rpc2.h"
#include "include/cm.h"
#include <rsrc.private.h>

#include "include/onode-manager.h"
#include "include/warden.h"
#include "include/viceroy.h"

#include "odyssey/viceroy/rsrc_en.h"

static ds_lock_t   rsrc_lock;
struct timeval     rsrc_interval = {0, 500000};
static ds_rrlist_t *test1;
static ds_rrlist_t *test2;
int                rsrc_loglevel = 0;
static int         rsrc_dotest = 0;
double             rsrc_rho = 0.20;
static double      rsrc_fu_upper = 0.0;

extern report_mode mode;	/* odyssey, bo, or lf */

/* Bandwidth logging info */
int                       rsrc_logbandwidth = 0;  /* ==1 => log, don't print */
int                       rsrc_log_minutes = 20;

/* Used only if above is nonzero */
static int                rsrc_log_lost = 0; /* How many log records lost */
static struct rsrc_logent *bwlog=NULL;       /* The Log */
static struct rsrc_logent *cur_bwlog=NULL;   /* Pointer to current entry */
static struct rsrc_logent *last_bwlog=NULL;  /* Pointer off end of log */

/* Raw connection-level logs */
static int raw_log_lost = 0;
struct raw_logent {
    cmgr_logent_t cmgr_logent;
    int rtt_estimate;
};
static struct raw_logent *rawlog=NULL;
static struct raw_logent *cur_rawlog=NULL;
static struct raw_logent *last_rawlog=NULL;

#ifdef INSTRUMENTED
extern int blind_optimism_bw;	/* starting value */
#endif /* INSTRUMENTED */

#ifdef RESLAN
extern unsigned int reslan_bandwidth;
#endif /* RESLAN */

#ifdef BW_TOOL
int current_bw = 0;		/* current total bandwidth estimation
				 * maintained by rsrc_netmonitor */
#endif

/* 
 * timerisneg: timer has negative interval
 */
#define timerisneg(tp) \
   (((tp)->tv_sec < 0) || ((tp)->tv_sec == 0 && (tp)->tv_usec < 0))

/*
 * Must call with resource lock NOT HELD.  Can callback into the
 * wardens, which may in turn call rsrc routines, which may need to
 * obtain locks.
 * 
 * N.B.: If this becomes an onerous task, change the ds_lock_t to a
 * ds_hw_lock_t 
 */
void
do_notify(val, to_notify, rsrc)
    long val;
    ds_list_t *to_notify;
    unsigned int rsrc;
{
    ds_list_iter_t *li;
    ds_request_t *reqp;
    struct ody_ucargs_t ntfy;
    int rc;

    ds_thread_log(20, "do_notify");
    li = ds_list_iter_create(to_notify);
    while ((reqp = (ds_request_t *)ds_list_iter_next(li))
	   != NULL)
    {
	/* Negative pids are actually warden type numbers */
	if (reqp->pid < 0) {
	    /* Warden request */
	    /* Warden is responsible for freeing reqp->argp->argval */
	    (reqp->fn)(reqp->reqid, rsrc, val, reqp->argp->argval);
	    ds_thread_log(15, "Notified warden %d for rsrc %d (%d)",
			  -reqp->pid, rsrc, val);
	} else {
	    /* Application request */
	    ntfy.oua_reqid = reqp->reqid;
	    ntfy.oua_rsrc = rsrc;
	    ntfy.oua_rval = val;
	    rc = upcall_async(UC_ODYSSEY, ODY_NOTIFICATION, reqp->pid,
			      sizeof(struct ody_ucargs_t), &ntfy);
	    if (rc) {
		ds_thread_log(0, "upcall_async pid %d failed: %s",
			      reqp->pid, strerror(errno));
	    } else {
		ds_thread_log(15, "Notified pid %d for rsrc %d (%d)",
			      reqp->pid, ntfy.oua_rsrc, ntfy.oua_rval);
	    }
	}
	ds_list_remove(to_notify, reqp);
	if (reqp->argp) {
	    /* The notified warden was responsible for freeing contents */
	    FREE(reqp->argp);
	}
	FREE(reqp);
    }
    ds_list_destroy(to_notify);
}

void
print_overlaps(lp)
    cmgr_log_t *lp;
{
    int i;
    struct timeval elapsed;
    struct timeval started;
    struct timeval overlap;
    struct timeval slop1 = { 0, 1000 }; /* 1 ms */
    struct timeval slop2 = { 0, 2000 }; /* 2 ms */
    
    if (lp->ValidEntries > 1) {
	for (i=0; i < lp->ValidEntries-1; i++) {
	    elapsed.tv_sec = lp->Entries[i].Value.Measured.ElapsedTime / 1000;
	    elapsed.tv_usec = 
		lp->Entries[i].Value.Measured.ElapsedTime % 1000;
	    timersub(&lp->Entries[i].TimeStamp, &elapsed, &started);
	    timersub(&lp->Entries[i+1].TimeStamp, &started, &overlap);
	    if (overlap.tv_sec > 0
		|| ((overlap.tv_sec == 0) && (overlap.tv_usec > 0)))
	    {
		fprintf(stderr,"Overlap ");
		if timercmp(&overlap, &slop1, >) {
		    fprintf(stderr, "by 1 ");
		}
		if timercmp(&overlap, &slop2, >) {
		    fprintf(stderr, "and 2");
		}
		fprintf(stderr, "\n");
	    }
	}
    }
}

static int
rsrc_buildlogs(clpp)
    struct rsrc_connlist **clpp;
{
    int                    count=0;
    struct rsrc_connlist   *clp;
    cmgr_id_t              cid;
    struct rsrc_connent    *cep;
    struct cmgr_iter_t     *iter;
    ds_list_t              *to_notify; 

    ALLOC(clp, struct rsrc_connlist);
    LIST_INIT(clp);

    /*  For each connection... */
    iter = cmgri_make();
    while (!cmgri_next(iter, &cid)) {
	ALLOC(cep, struct rsrc_connent);
	
	/* Set up ids, etc. */
	cep->ce_cid = cid;
	cep->ce_status = (cmgr_isvalid(cep->ce_cid) ? CMGR_ALIVE : CMGR_DEAD);
	cep->ce_connp = cmgr_get(cid, CMGR_NOLOCK);
	if (!cep->ce_connp  /* Connection was toasted */
	    || (cep->ce_connp->cc_flags & CMGR_CONNISLOCAL))
	{
	    if (cep->ce_connp) {
		cmgr_put(cid, CMGR_NOLOCK);
	    }
	    FREE(cep);
	    continue; /* move to the next one. */
	}
	cep->ce_connid = CMGR_RPC2_HANDLE(cep->ce_connp); /* XXX */

	/* Set up logs */
	cep->ce_rttlog.Entries = cep->ce_rlent;
	cep->ce_rttlog.NumEntries = CMGR_MAXLOGLENGTH;
	cep->ce_rttlog.Quantum = WINDOW_TIME*1000;
	cep->ce_tplog.Entries = cep->ce_tlent;
	cep->ce_tplog.NumEntries = CMGR_MAXLOGLENGTH;
	cep->ce_tplog.Quantum = WINDOW_TIME*1000;
	cep->ce_tlcur = 0;

	/* 
	 * Get logs: may fail for some transports if down, 
	 * but otherwise are obligated to succeed (afaik)
	 */
	if (cmgr_getlogs(cep->ce_cid, &cep->ce_rttlog, &cep->ce_tplog)) {
	    ASSERT(errno == CMGR_CONNDOWN);

	    /* 
	     * If transport is down, bandwidth is zero. Notify 
	     * any interested clients.
	     */
	    ds_lock_write(&rsrc_lock);
	    ds_rrlist_set_value(cep->ce_connp->cc_bwlist, 0, &to_notify);
	    ds_lock_release(&rsrc_lock);
	    if (to_notify) {
	        do_notify(0, to_notify, ODY_GR_NETBW);
		ds_thread_log(100, "did notify\n");
	    }	

    	    cmgr_put(cid, CMGR_NOLOCK);
	    FREE(cep);
	    continue;
	}

	/* 
	 * This log is meaningful iff:
	 * (a) it has some valid bandwidth entries
	 * and (b) it's youngest entry is MEASURED 
	 */

	if (cep->ce_tplog.ValidEntries
	    && (cep->ce_tplog.Entries[0].Tag == RPC2_MEASURED_NLE))
	{
	    /* Insert it into the clp */
	    LIST_INSERT_HEAD(clp, cep, ce_list);
	    ds_thread_log(110,"Inserting logs for connection %d",
			  cep->ce_cid);
	    count++;
	} else {
	    /* Toast it, as it isn't useful. */
	    cmgr_put(cep->ce_cid, CMGR_NOLOCK);
	    FREE(cep);
	}
    }
    cmgri_destroy(iter);

    *clpp = clp;
    return count;
}

#ifdef INSTRUMENTED
static int
rsrc_optimistic_buildlogs(clpp)
    struct rsrc_connlist **clpp;
{
    int                    count=0;
    struct rsrc_connlist   *clp;
    cmgr_id_t              cid;
    struct rsrc_connent    *cep;
    struct cmgr_iter_t     *iter;

    /* ds_list_t              *to_notify; */
            
    ALLOC(clp, struct rsrc_connlist);
    LIST_INIT(clp);

    /*  For each connection... */
    iter = cmgri_make();
    while (!cmgri_next(iter, &cid)) {
	ALLOC(cep, struct rsrc_connent);
	
	/* Set up ids, etc. */
	cep->ce_cid = cid;
	cep->ce_status = (cmgr_isvalid(cep->ce_cid) ? CMGR_ALIVE : CMGR_DEAD);
	cep->ce_connp = cmgr_get(cid, CMGR_NOLOCK);
	if (!cep->ce_connp) {
	    /* Someone toasted this connection: why was it in the table? */
	    FREE(cep);
	    continue; /* move to the next one. */
	}
	cep->ce_connid = CMGR_RPC2_HANDLE(cep->ce_connp); /* XXX */

	/* Insert it into the clp */
	LIST_INSERT_HEAD(clp, cep, ce_list);
	ds_thread_log(25,"Inserting logs for connection %d",
		      cep->ce_cid);
	count++;
    }
    cmgri_destroy(iter);

    *clpp = clp;
    return count;
}
#endif /* INSTRUMENTED */

/* 
 * Smoothing algorithm: used by both rtt and bandwidth estimations
 *
 * The averaging is done by considering log entries, eldest first, and
 * adding them in using 
 * 
 * newval = a*bw + (1-a)*obsval 
 *       = bw + * (1-a)*(obsval-newval) 
 * 
 * where obsval is the observed value for that
 * log entry.  We require a to be (n-1)/n where n is some power of
 * two: the log_2(n) is LOGSHIFT in rsrc.private.h 
 */

static void
rsrc_rtt(cep)
    struct rsrc_connent *cep;
{
    cmgr_log_t       *rtlp;
    struct timeval   rtt_winstop;
    int              this_rtt;
    int              test_rtt;
    int              i;

    cep->ce_rtt = 0;

    rtlp = &cep->ce_rttlog;

    /* Time of the latest entry */
    rtt_winstop = rtlp->Entries[0].TimeStamp;

    /* Look back at most WINDOW_TIME seconds */
    for (i=0; i< rtlp->ValidEntries; i++) {
	if (rtlp->Entries[i].TimeStamp.tv_sec < 
	    (rtt_winstop.tv_sec - WINDOW_TIME))
	{
	    break;
	}
    }
    /* Adjust if we fell off the end */
    if (i == rtlp->ValidEntries) i--;

    /* Do smoothing, from oldest to youngest. */
    for (; i>=0; i--) {
	ds_thread_log(110,"Conn %d RTT LogEnt %ld.%06ld Elapsed %d",
		      cep->ce_cid, 
		      rtlp->Entries[i].TimeStamp.tv_sec,
		      rtlp->Entries[i].TimeStamp.tv_usec,
		      rtlp->Entries[i].Value.Measured.ElapsedTime);
		      
	if (rtlp->Entries[i].Value.Measured.Conn != cep->ce_connid) {
	    continue;
	}
	this_rtt = rtlp->Entries[i].Value.Measured.ElapsedTime;
	if (cep->ce_rtt == 0) {
	    cep->ce_rtt = this_rtt;
	} else {
	    /* Cap upswings by RTT/2
	       due to anomolous bursts in RTT estimation. */
	    test_rtt = this_rtt + ((cep->ce_rtt - this_rtt) >> RTT_SHIFT);
	    if (test_rtt > (cep->ce_rtt + cep->ce_rtt/2)) {
	        cep->ce_rtt += cep->ce_rtt/2;
	    } else {
	      cep->ce_rtt = test_rtt;
	    }
	}
    }

    ASSERT(cep->ce_rtt >= 0);
}

static int
lookup_rtt(cid, clp)
    RPC2_Handle          cid; /* XXX */ 
    struct rsrc_connlist *clp;
{
    struct rsrc_connent *cep;
    
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	if (cep->ce_connid == cid) {
	    return cep->ce_rtt;
	}
    }
    return -1;
}

static int
rsrc_bandwidth(lp, clp)
    cmgr_log_t           *lp;
    struct rsrc_connlist *clp;
{
    int bandwidth = 0;
    int thisbw;
    int bytes;
    int transfer_time;
    int rtt;
    int wire_time;
    int i;
    /* Debugging to print out new log entries */
    static struct timeval last_ent_time = {0,0};

    /* 
     * Here, we are guaranteed that no records are STATIC (by
     * rsrc_buildmaster) and that there is at least one record present
     * (by rsrc_rtt).  This makes our lives significantly simpler than
     * they were in rsrc_rtt.  
     * 
     * To make our lives simpler in the bytecounting phase, we'll use
     * the whole log rather than check WINDOW
     */
    i = lp->ValidEntries-1;
    for (; i >= 0; i--) {
	ds_thread_log(110,"Master TP LogEnt %ld.%06ld Bytes %d Elapsed %d",
		      lp->Entries[i].TimeStamp.tv_sec,
		      lp->Entries[i].TimeStamp.tv_usec,
		      lp->Entries[i].Value.Measured.Bytes,
		      lp->Entries[i].Value.Measured.ElapsedTime);
	bytes = lp->Entries[i].Value.Measured.Bytes;
	ASSERT(bytes < INT_MAX/1000);
	rtt = lookup_rtt(lp->Entries[i].Value.Measured.Conn, clp);
	if (rtt < 0) {
	    rtt = 0;
	}
	transfer_time = lp->Entries[i].Value.Measured.ElapsedTime;
	wire_time = transfer_time - (rtt/2);
	if (wire_time > 0) {
	    thisbw = bytes*1000/wire_time;
	    if (bandwidth == 0) {
		bandwidth = thisbw;
	    } else {
		bandwidth = thisbw + ((bandwidth - thisbw) >> BW_SHIFT);
	    }
	}
	if (rsrc_logbandwidth) {
	    if (timercmp(&lp->Entries[i].TimeStamp, &last_ent_time, >)) {
		last_ent_time = lp->Entries[i].TimeStamp;
		if (cur_rawlog != last_rawlog) {
		    cur_rawlog->cmgr_logent = lp->Entries[i];
		    cur_rawlog->rtt_estimate = rtt;
		    cur_rawlog++;
		}
		else
		    raw_log_lost++;
	    }
	}
    }
    return bandwidth;
}

#ifdef INSTRUMENTED
static int
rsrc_laissez_bandwidth(lp, clp)
    cmgr_log_t           *lp;
    struct rsrc_connlist *clp;
{
    int bandwidth = 0;
    int thisbw;
    int bytes;
    int transfer_time;
    int rtt;
    int wire_time;
    int i;

    /* 
     * Here, we are guaranteed that no records are STATIC (by
     * rsrc_buildtemp) and that there is at least one record present
     * (by rsrc_rtt).  This makes our lives significantly simpler than
     * they were in rsrc_rtt.  
     * 
     * To make our lives simpler in the bytecounting phase, we'll use
     * the whole log rather than check WINDOW
     */
    i = lp->ValidEntries-1;
    for (; i >= 0; i--) {
	ds_thread_log(110,"Master TP LogEnt %ld.%06ld Bytes %d Elapsed %d",
		      lp->Entries[i].TimeStamp.tv_sec,
		      lp->Entries[i].TimeStamp.tv_usec,
		      lp->Entries[i].Value.Measured.Bytes,
		      lp->Entries[i].Value.Measured.ElapsedTime);
		      
	bytes = lp->Entries[i].Value.Measured.Bytes;
	ASSERT(bytes < INT_MAX/1000);
	rtt = lookup_rtt(lp->Entries[i].Value.Measured.Conn, clp);
	if (rtt < 0) {
	    rtt = 0;
	}
	transfer_time = lp->Entries[i].Value.Measured.ElapsedTime;
	wire_time = transfer_time - (rtt/2);
	if (wire_time > 0) {
	    thisbw = bytes*1000/wire_time;
	    if (bandwidth == 0) {
		bandwidth = thisbw;
	    } else {
		bandwidth = thisbw + ((bandwidth - thisbw) >> BW_SHIFT);
	    }
	}
    }
    return bandwidth;
}

#endif /* INSTRUMENTED */

/* 
 * get_youngest:
 *
 * When called with NULL, clears the static "lastlep", which is used
 * to suppress the returning of duplicate entries.
 */
static cmgr_logent_t *
get_youngest(clp)
    struct rsrc_connlist *clp;
{
    struct timeval       eldest;
    struct rsrc_connent  *eep=NULL;
    cmgr_logent_t        *lep;
    struct rsrc_connent  *cep;
    static cmgr_logent_t lastlep;

    if (!clp) {
	bzero(&lastlep, sizeof(cmgr_logent_t));
	return NULL;
    }

    /* While we haven't found a non-duplicate or run out of entries: */
    while (1) {
	timerclear(&eldest);
	for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	    /* 
	     * Does this list have anything left?
	     * It does not if either
	     * (a) it's cur pointer is == ValidEntries
	     * (b) it's cur pointer points to a STATIC_NLE */
	    if (cep->ce_tlcur >= cep->ce_tplog.ValidEntries
		|| (cep->ce_tplog.Entries[cep->ce_tlcur].Tag 
		    == RPC2_STATIC_NLE))
	    {
		/* There's nothing to see here, move along. --pgh police */
		continue;
	    }
	    /* Is it older than eldest? */
	    lep = &cep->ce_tplog.Entries[cep->ce_tlcur];
	    if (timercmp(&lep->TimeStamp, &eldest, >)) {
		eldest = lep->TimeStamp;
		eep = cep;
	    }
	}

	/* If eep is NULL, there is no next logent to return */
	if (!eep) {
	    return NULL;
	}
	
	/* eep now points to the connent with the eldest record.  Consume it */
	lep = &eep->ce_tplog.Entries[eep->ce_tlcur];
	eep->ce_tlcur++;

	/* Check to see if we just returned it */
	/*  Check for identity */
	if ((lep->TimeStamp.tv_sec == lastlep.TimeStamp.tv_sec)
	    && (lep->TimeStamp.tv_usec == lastlep.TimeStamp.tv_usec)
	    && (lep->Value.Measured.Conn == lastlep.Value.Measured.Conn)
	    && (lep->Value.Measured.Bytes == lastlep.Value.Measured.Bytes)
	    && (lep->Value.Measured.ElapsedTime 
		== lastlep.Value.Measured.ElapsedTime))
	{
	    /* 
	     * We don't care about this one, try again. 
	     * Make sure we don't see this list again, in case we
	     * fall through the for loop above 
	     */
	    eep = NULL;
	    continue;
	} else {
	    /* This is the one we want, so save it */
	    lastlep = *lep;
	    return lep;
	}
    }
}

#define SLOPUSECS 2000        /* 2ms slop for possible false overlap */

static void
rsrc_buildmaster(masterlog, clp)
    cmgr_log_t           *masterlog;
    struct rsrc_connlist *clp;
{
    cmgr_logent_t       *lep;
    cmgr_logent_t       *nlep;
    struct timeval      elapsed;
    struct timeval      started;
    struct timeval      nlepstart;
    struct timeval      slop = {0, SLOPUSECS};

    /* Forget the last youngest entry we looked at */
    (void)get_youngest(NULL);  /* this cancells the remembered log entry */
    
    /* We know we have at least one valid log with one valid entry */
    lep = get_youngest(clp);
    while (lep && (masterlog->ValidEntries < masterlog->NumEntries)) {
	/* Copy lep so that we can change it with impunity */
	ASSERT(lep->Tag == RPC2_MEASURED_NLE);
	nlep = get_youngest(clp);
	/* 
	 * nlep could overlap with lep.  This case occurs when there
	 * are two or more SFTP windows in flight at once.  
	 */
	if (nlep) {
	    ASSERT(nlep->Tag == RPC2_MEASURED_NLE);
	    /* Check for overlap */
	    elapsed.tv_sec = lep->Value.Measured.ElapsedTime / 1000;
	    elapsed.tv_usec = (lep->Value.Measured.ElapsedTime % 1000) * 1000;
	    timersub(&lep->TimeStamp, &elapsed, &started);
	    /* Could false-overlap by 2*grainsize of ElapsedTime */
	    timeradd(&started, &slop, &started);
	    if (timercmp(&started, &nlep->TimeStamp, <)) {
		/* 
		 * Coalesce these two records into one
		 * The byte count is the sum, and we add the portion
		 * of nlep's ElapsedTime that *didn't* overlap with
		 * lep's to lep's ElapsedTime.  We keep lep's Conn
		 * field, since it is 1/2 of lep's round trip that we
		 * eventually subtract (his ACK was in the critical
		 * timing path.)
		 */
		lep->Value.Measured.Bytes += nlep->Value.Measured.Bytes;
		/* Subtract the slop back from started, to get accurate time */
		timersub(&started, &slop, &started);
		/* How long did nlep take? */
		elapsed.tv_sec = nlep->Value.Measured.ElapsedTime / 1000;
		elapsed.tv_usec 
		    = (nlep->Value.Measured.ElapsedTime % 1000) * 1000;
		timersub(&nlep->TimeStamp, &elapsed, &nlepstart);
		/* How much earlier than lep was that? */
		timersub(&started, &nlepstart, &elapsed);
		/* Add that much time to lep */
		lep->Value.Measured.ElapsedTime
		    += (elapsed.tv_sec * 1000) + (elapsed.tv_usec / 1000);
		/* And start again */
		continue;
	    }
	}
	/* Non-overlapping.  Fill the next record */
	masterlog->Entries[masterlog->ValidEntries++] = *lep;
	/* Start again with nlep */
	lep = nlep;
    }
}

#ifdef INSTRUMENTED

static void
rsrc_buildtemp(templog, cep)
    cmgr_log_t           *templog;
    struct rsrc_connent  *cep;
{
    cmgr_logent_t       *lep;
    int i;

    templog->ValidEntries = 0;
    for (i=0; i<cep->ce_tplog.ValidEntries; i++) {
	lep = &cep->ce_tplog.Entries[i];
	if (lep->Tag != RPC2_STATIC_NLE)
	    templog->Entries[templog->ValidEntries++] = *lep;
    }
    ASSERT(templog->ValidEntries > 0);
}

#endif /* INSTRUMENTED */

static double
calc_bytes(clp)
    struct rsrc_connlist *clp;
{
    struct rsrc_connent *cep;
    double total = 0.0;
    
    /*
     * When we get here, each of the connlists should have a nonzero
     * tlcur.  We can just go backwards and count up the bytes in each
     * tlent and divide by the ttotal for the fraction.
     */
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	cep->ce_bytes = 0.0;
	if (cep->ce_tlcur) {
	    /* 
	     * We used some observations from this log; it can compete
	     * for the bandwidth up for grabs.
	     */
	    cep->ce_tlcur--;
	    for (; cep->ce_tlcur >= 0; cep->ce_tlcur--) {
		/* One more sanity check. */
		ASSERT(cep->ce_tlent[cep->ce_tlcur].Tag == RPC2_MEASURED_NLE);
		if (cep->ce_tlent[cep->ce_tlcur].Value.Measured.Conn
		    == cep->ce_connid)
		{
		    cep->ce_bytes
			+= cep->ce_tlent[cep->ce_tlcur].Value.Measured.Bytes;
		}
	    }
	    total += cep->ce_bytes;
	}
    }
    return total;
}

static int badcount=0;
static int numcount=0;

/*
 * XXX - this won't work if more than one interface can be up at once.  
 */
static void
rsrc_netmonitor(called_at)
     struct timeval *called_at;
{
    struct rsrc_connlist   *clp;
    int                    count;
    struct rsrc_connent    *cep;
    cmgr_log_t             masterlog;
    cmgr_logent_t          masterent[CMGR_MAXLOGLENGTH];
    int                    bandwidth;
    struct timeval         earliest;
    struct timeval         duration;
    cmgr_logent_t          *lastmlep;
    double                 total_bytes;
    double                 pred_bytes;
    double                 fracused;
    int                    residual_bandwidth=0;
    int                    this_bandwidth;
    ds_list_t              *to_notify;
    char                   *message;
#define ONE_ENTRY 16
    char                   oneshot[ONE_ENTRY];
    struct rsrc_log_conn   *rlcp_arr=NULL;
    int                    num_rlcp=0;

    /* Grab the logs for all connections */
    count = rsrc_buildlogs(&clp);

    if (!count) {
      FREE (clp); /* Must deallocate clp allocated in buildlogs(); */
      return;
    }

    /* 
     * 512 + ONE_ENTRY *count for total buffer
     * (need to have each conn, bw pair 
     */
    BALLOC(message, 512 + (ONE_ENTRY*count));
    
    /* Estimate each log's rtt independently. */
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	rsrc_rtt(cep);
    }

    /* Grab a master log */
    masterlog.ValidEntries = 0;
    masterlog.NumEntries = CMGR_MAXLOGLENGTH;
    masterlog.Entries = masterent;
    rsrc_buildmaster(&masterlog, clp);
    ASSERT(masterlog.ValidEntries);

    numcount++;

    /* What is the bandwidth reflected by the master log */
    bandwidth = rsrc_bandwidth(&masterlog, clp);
    if (bandwidth <= 0) {
	goto exit;
    }

    if (bandwidth >= 200000 && numcount > 60) {
        badcount++;
    }      

#ifdef RESLAN
    if (bandwidth > reslan_bandwidth) 
        bandwidth = reslan_bandwidth;
#endif

    if (rsrc_logbandwidth) {
	if (cur_bwlog != last_bwlog) {
	    cur_bwlog->rle_tv = *called_at;
	    cur_bwlog->rle_bandwidth = bandwidth;
	    cur_bwlog->rle_conns = count;
	    NALLOC(cur_bwlog->rle_rlcp, struct rsrc_log_conn, count);
	    rlcp_arr = cur_bwlog->rle_rlcp;
	    num_rlcp = 0;
	} else {
	    rsrc_log_lost++;
	}
    } else {
	sprintf(message, "Total %d ", bandwidth);
#ifdef BW_TOOL
	current_bw = bandwidth;
#endif /* BW_TOOL */
    }

    /* How much time does the master log represent? */
    lastmlep = &masterlog.Entries[masterlog.ValidEntries-1];
    earliest.tv_sec = lastmlep->TimeStamp.tv_sec 
	+ (lastmlep->Value.Measured.ElapsedTime / 1000);
    earliest.tv_usec = lastmlep->TimeStamp.tv_usec
	+ (lastmlep->Value.Measured.ElapsedTime % 1000) * 1000;
    while (earliest.tv_usec >= 1000000) {
	earliest.tv_sec++;
	earliest.tv_usec -= 1000000;
    }
    timersub(&masterlog.Entries[0].TimeStamp, &earliest, &duration);
    if (timerisneg(&duration)) {
	ds_thread_log(110, "Negative duration.");
	goto exit;
    }
    
    /* And how many bytes is that at this bandwidth? */
    pred_bytes = (double)bandwidth * (double)duration.tv_sec;
    pred_bytes += ((double)bandwidth * (double)duration.tv_usec) / 1000000.0;

    /* How many bytes (total) are consumed by the logs? */
    total_bytes = calc_bytes(clp);

    /* Multiply the larger of those two by 1/(1-\rho) */
    /* This reserves \rho percent of the bandwidth for later division */
    total_bytes = (total_bytes > pred_bytes) ? total_bytes : pred_bytes;
    total_bytes *= (1.0)/(1.0 - rsrc_rho);
    
    /* Calculate the fraction used for each of the connections */
    fracused = 0.0;
    for (cep = clp->lh_first; cep; cep = cep->ce_list.le_next) {
	cep->ce_fraction = cep->ce_bytes / total_bytes;
	fracused += cep->ce_fraction;
    }

    /* Check that fracused is not more than (1-\rho) + 0.01 */
    ASSERT(fracused < rsrc_fu_upper);

    /* 
     * That leaves bw*(1-fracused) as residual bandwidth, which is divided
     * amongst the count streams 
     */
    residual_bandwidth 
	= (int)(((double)bandwidth * (1.0 - fracused)) / (double)count);

    /* Set each of the new bandwidth values. */
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	this_bandwidth = 
	    (int)((double)bandwidth * cep->ce_fraction) + residual_bandwidth;
	if (rsrc_logbandwidth) {
	    if (cur_bwlog != last_bwlog) {
		ASSERT(num_rlcp < cur_bwlog->rle_conns);
		rlcp_arr[num_rlcp].rlc_connid = cep->ce_cid;
		rlcp_arr[num_rlcp].rlc_bandwidth = this_bandwidth;
		num_rlcp++;
	    }
	} else {
	    sprintf(oneshot, "%ld %d ", cep->ce_cid, this_bandwidth);
	    strcat(message, oneshot);
	    ds_thread_log(110,"Conn %d: bw %d rtt %d",
			  cep->ce_cid, this_bandwidth, cep->ce_rtt);
	}
	ds_lock_write(&rsrc_lock);
	ds_rrlist_set_value(cep->ce_connp->cc_bwlist, this_bandwidth,
			    &to_notify);
	ds_lock_release(&rsrc_lock);
	if (to_notify) {
	    do_notify(this_bandwidth, to_notify, ODY_GR_NETBW);
	}
    }

    if (rsrc_logbandwidth) {
	cur_bwlog++;
    }
    else {
	ds_thread_log(110, "%s", message);
    }

 exit:
    FREE(message);
    /* Destroy each of the cep's, and then the clp */
    while ((cep = clp->lh_first)) {
	LIST_REMOVE(cep, ce_list);
	if (cep->ce_connp) {
	    cmgr_put(cep->ce_cid, CMGR_NOLOCK);
	}
	FREE(cep);
    }
    FREE(clp);

}

#ifdef INSTRUMENTED
/*
 * This is the netmonitor to use with the -blind-optimism switch.
 * It ignores the logs, and instead uses the value of blind_optimism_bw
 * as the bandwidth for all connections. (This value is set whenever
 * a trace replay upcall occurs, and represents the nominal bandwidth
 * of the trace.)
 */
static void
rsrc_optimistic_netmonitor()
{
    struct rsrc_connlist   *clp;
    int                    count;
    struct rsrc_connent    *cep;
    int                    this_bandwidth;
    ds_list_t              *to_notify;

    /* Grab the logs for all connections */
    count = rsrc_optimistic_buildlogs(&clp);

    if (!count) return;
    
    /* Set each of the new bandwidth values. */
    this_bandwidth = blind_optimism_bw;
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	ds_thread_log(5,"Bandwidth on conn %d is %d",
		      cep->ce_cid, this_bandwidth);
	ds_lock_write(&rsrc_lock);
	ds_rrlist_set_value(cep->ce_connp->cc_bwlist, this_bandwidth,
			    &to_notify);
	ds_lock_release(&rsrc_lock);
	if (to_notify) {
	    do_notify(this_bandwidth, to_notify, ODY_GR_NETBW);
	}
    }

    /* Destroy each of the cep's, and then the clp */
    while ((cep = clp->lh_first)) {
	LIST_REMOVE(cep, ce_list);
	if (cep->ce_connp) {
	    cmgr_put(cep->ce_cid, CMGR_NOLOCK);
	}
	FREE(cep);
    }
    FREE(clp);
}

/*
 * rsrc_laissez_netmonitor -- Do the estimation for "laissez-faire II",
 *   where individual connection logs are used to estimate rtt and bandwidth
 */
static void
rsrc_laissez_netmonitor()
{
    struct rsrc_connlist   *clp;
    int                    count;
    cmgr_log_t             templog;
    cmgr_logent_t          tempent[CMGR_MAXLOGLENGTH];
    struct rsrc_connent    *cep;
    int                    bandwidth;
    ds_list_t              *to_notify;
    char                   *message;
#define ONE_ENTRY 16
    char                   oneshot[ONE_ENTRY];

    /* Grab the logs for all connections */
    count = rsrc_buildlogs(&clp);

    if (!count) return;
    
    /* 
     * 512 + ONE_ENTRY *count for total buffer
     * (need to have each conn, bw pair 
     */
    BALLOC(message, 512 + (ONE_ENTRY*count));
    *(char *)message = '\0';

    /* Estimate each log's rtt independently. */
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	rsrc_rtt(cep);
    }

    templog.NumEntries = CMGR_MAXLOGLENGTH;
    templog.Entries = tempent;
    for (cep = clp->lh_first; cep != NULL; cep = cep->ce_list.le_next) {
	/* Make a templog that conforms to rsrc_laissez_bandwidth's
	 * assumptions (all entries measured, >0 entry) */
	rsrc_buildtemp(&templog, cep);
	ASSERT(templog.ValidEntries);
	
	/* What is the bandwidth reflected by the conn log */
	bandwidth = rsrc_laissez_bandwidth(&templog, clp);
	if (bandwidth <= 0) {
	    continue;
	}
	
	/* Set each of the new bandwidth values. */
	sprintf(oneshot, "%ld %d ", cep->ce_cid, bandwidth);
	strcat(message, oneshot);
	ds_thread_log(10,"Conn %d: bw %d rtt %d",
		      cep->ce_cid, bandwidth, cep->ce_rtt);
	ds_lock_write(&rsrc_lock);
	ds_rrlist_set_value(cep->ce_connp->cc_bwlist, bandwidth,
			    &to_notify);
	ds_lock_release(&rsrc_lock);
	if (to_notify) {
	    do_notify(bandwidth, to_notify, ODY_GR_NETBW);
	}
    }
    ds_thread_log(5, "%s", message);
    
    FREE(message);
    /* Destroy each of the cep's, and then the clp */
    while ((cep = clp->lh_first)) {
	LIST_REMOVE(cep, ce_list);
	if (cep->ce_connp) {
	    cmgr_put(cep->ce_cid, CMGR_NOLOCK);
	}
	FREE(cep);
    }
    FREE(clp);
}

#endif /* INSTRUMENTED */


static int
rsrc_monitor(void *p)
{
    struct timeval   start;
    struct timeval   end;
    struct timeval   interval;
    int              readfds = 0;
    int              writefds = 0;
    int              exceptfds = 0;
    ds_list_t        *to_notify;
    

    while (1) {
	ds_thread_log(110,"rsrc_monitor: awake");
	timerclear(&interval);
	gettimeofday(&start, NULL);

	/* Check test resource */
	if (rsrc_dotest) {
	    ds_thread_log(0, "rsrc_dotest %d", rsrc_dotest);
	    ds_lock_write(&rsrc_lock);
	    ds_rrlist_set_value(test1, rsrc_dotest, &to_notify);
	    ds_lock_release(&rsrc_lock);
	    if (to_notify) {
		do_notify(rsrc_dotest, to_notify, ODY_TEST_1);
	    }
	    rsrc_dotest = 0;
	}

	/* check the network status */
	switch (mode) {
	case odyssey:
	    rsrc_netmonitor(&start);
	    break;
#ifdef INSTRUMENTED
	case blind_optimism:
	    rsrc_optimistic_netmonitor(&start);
	    break;
	case laissez:
	    rsrc_laissez_netmonitor(&start);
	    break;
#endif /* INSTRUMENTED */
	}	

	/* check the power resource */
	rsrc_en_monitor ();

	/* Done */
	gettimeofday(&end, NULL);
	timersub(&start, &end, &interval);
	while(timerisneg(&interval)) {
	    timeradd(&interval, &rsrc_interval, &interval);
	}
	ds_thread_log(110,"rsrc_monitor: sleeping for %ld.%06ld",
		      interval.tv_sec, 
		      interval.tv_usec);
	IOMGR_Select(0, &readfds, &writefds, &exceptfds, &interval);
    }
    return 0;
}

void
rsrc_clearlog()
{
    while (cur_bwlog > bwlog) {
	cur_bwlog--;
	FREE(cur_bwlog->rle_rlcp);
    }
    ASSERT(cur_bwlog == bwlog);
    cur_rawlog = rawlog;
}

void
rsrc_dumplog()
{
    static int uniq = 0;
    struct rsrc_logent *print_bwlog;
    struct raw_logent *print_rawlog;
    FILE *fp;
    char filename[64];
    time_t t;
    struct tm *tp;
    int i;

    t = time(NULL);
    tp = localtime(&t);

    sprintf(filename,"/tmp/raw.logs.%d.%d.%d:%d",uniq,
	    tp->tm_yday, tp->tm_hour, tp->tm_min);

    if (!(fp = fopen(filename,"w"))) {
	fprintf(stderr,"dump_stats: %s: %s",filename,strerror(errno));
	fflush(stderr);
	return;
    }

    fprintf(fp, "%d log entries dropped due to lack of space\n",
	    raw_log_lost);
    print_rawlog = rawlog;
    while (print_rawlog != cur_rawlog) {
	fprintf(fp, "TS %ld.%06ld Conn %02d Bytes %ld Elapsed %ld rtt %d\n",
		print_rawlog->cmgr_logent.TimeStamp.tv_sec,
		print_rawlog->cmgr_logent.TimeStamp.tv_usec,
		(int) print_rawlog->cmgr_logent.Value.Measured.Conn,
		print_rawlog->cmgr_logent.Value.Measured.Bytes,
		print_rawlog->cmgr_logent.Value.Measured.ElapsedTime,
		print_rawlog->rtt_estimate);
	print_rawlog++;
    }
    fclose(fp);

    sprintf(filename,"/tmp/rsrc.logs.%d.%d.%d:%d",uniq++,
	    tp->tm_yday, tp->tm_hour, tp->tm_min);

    if (!(fp = fopen(filename,"w"))) {
	fprintf(stderr,"dump_stats: %s: %s",filename,strerror(errno));
	fflush(stderr);
	return;
    }

    print_bwlog = bwlog;
    while (print_bwlog != cur_bwlog) {
	fprintf(fp, "%ld.%06ld:\t%6d", print_bwlog->rle_tv.tv_sec,
		print_bwlog->rle_tv.tv_usec, print_bwlog->rle_bandwidth);
	for (i = 0; i<print_bwlog->rle_conns; i++) {
	    fprintf(fp, "\t%2ld %6d",
		    print_bwlog->rle_rlcp[i].rlc_connid,
		    print_bwlog->rle_rlcp[i].rlc_bandwidth);
	}
	fprintf(fp, "\n");
	print_bwlog++;
    }
    fclose(fp);
}

void rsrc_init() 
{ 
    double   divisor;
    int      num_logents;

    /* Set up bandwidth reserve parameters */
    ASSERT((0.0 <= rsrc_rho) && (rsrc_rho <= 100.0));
    rsrc_fu_upper = (1.0 - rsrc_rho) + 0.01;

    ds_lock_init(&rsrc_lock);
    ASSERT((test1 = ds_rrlist_create(100)));
    ASSERT((test2 = ds_rrlist_create(100)));
    ASSERT(ds_thread_create(rsrc_monitor, NULL, DS_THREAD_CRITICAL, 
			    "rsrc_mon", rsrc_loglevel));

    /* 
     * if we are logging bandwidth notifications, allow up to
     * rsrc_log_minutes minutes worth of logging before saying "no
     * mas" 
     */
    if (rsrc_logbandwidth) {
	/* 1/divisor entries per second */
	divisor = (double)rsrc_interval.tv_sec
	    + ((double)rsrc_interval.tv_usec / 1000000.0);
	num_logents = (int)((double)rsrc_log_minutes * 60.0 / divisor);
	NALLOC(bwlog, struct rsrc_logent, num_logents);
	last_bwlog = &bwlog[num_logents];
	cur_bwlog = &bwlog[0];
	/* Use the same num_logents for the number of raw log entries, even
	   though it doesn't correspond to the amount of time before we
	   start dropping them */
	NALLOC(rawlog, struct raw_logent, num_logents);
	last_rawlog = &rawlog[num_logents];
	cur_rawlog = &rawlog[0];
    } else {
	bwlog = cur_bwlog = last_bwlog = NULL;
    }

    /* Initialize power resource */
    rsrc_en_init ();
}

int
rsrc_place_netbw(wp, op, low, high, pid, fn, argp, cancelled, value, reqid)
    struct warden *wp;
    struct viceroy_onode *op;
    int           low;
    int           high;
    pid_t         pid;
    REQFNP        fn;
    void          *argp;
    int           *cancelled;
    int           *value;
    int           *reqid;
{
    ds_request_t       *reqp=NULL;
    ds_request_t       *old_req=NULL;
    ds_rrlist_return_t rc;
    cmgr_id_t            cid;
    cmgr_connection_t    *connp;
        
    if (!wp->w_getcid)
	return EOPNOTSUPP;

    cid = (wp->w_getcid)(wp, op);
    connp = cmgr_get(cid, CMGR_LOCK);
    if (!connp)
	return EOPNOTSUPP;
    
    ALLOC(reqp, ds_request_t);
    DS_REQUEST_ALLOCATE(reqp, low, high, pid, fn, argp);

    ds_lock_write(&rsrc_lock);
    rc = ds_rrlist_request(connp->cc_bwlist, reqp, value, &old_req);
    ds_lock_release(&rsrc_lock);

    /* All of this should probably go in request, not here */

    /* Was it a duplicate? */
    if (rc & DS_RRLIST_DUPLICATE) {
	ds_thread_log(15, "\tDuplicated request %d", old_req->reqid);
	*cancelled = old_req->reqid;
	if (old_req->argp) {
	    (old_req->argp->free_fn)(old_req->argp->argval);
	    FREE(old_req->argp);
	}
	FREE(old_req);
    }

    /* Was the request placed?  If not, value is filled */
    if (!(rc & DS_RRLIST_OUTOFWINDOW)) {
	*reqid = reqp->reqid;
	ds_thread_log(15, "\tGranted reqid: %d, NETBW [%d..%d] on conn %d", 
		      reqp->reqid, reqp->low, reqp->high, cid);
    } else {
	ds_thread_log(15, "\tRequest out of window");
	FREE(reqp);
    }

    /* Convert return codes to errors */
    rc &= ~DS_RRLIST_DUPLICATE;  /* Duplicate isn't returned */

    if (rc == DS_RRLIST_OUTOFWINDOW) {
	rc = ODY_ENOTINWINDOW;
    } else {
	ASSERT(!rc);
    }

    cmgr_put(cid, CMGR_LOCK);
    return rc;
}

void
rsrc_testfire()
{
    rsrc_dotest = 10;
}

void
rsrc_testreset()
{
    rsrc_dotest = 100;
}

int
rsrc_place_test(low, high, pid, cancelled, value, reqid, rsrc)
    int           low;
    int           high;
    pid_t         pid;
    int           *cancelled;
    int           *value;
    int           *reqid;
    int           rsrc;
{
    ds_request_t         *reqp=NULL;
    ds_request_t         *old_req=NULL;
    ds_rrlist_return_t   rc=0;
    
    ALLOC(reqp, ds_request_t);
    DS_REQUEST_ALLOCATE(reqp, low, high, pid, NULL, NULL);
    
    ds_lock_write(&rsrc_lock);
    switch (rsrc) {
    case ODY_TEST_1:
	rc = ds_rrlist_request(test1, reqp, value, &old_req);
	break;
    case ODY_TEST_2:
	rc = ds_rrlist_request(test2, reqp, value, &old_req);
	break;
    default:
	ASSERT(0);
	break;
    }	
    ds_lock_release(&rsrc_lock);
	
    /* Was it a duplicate? */
    if (rc & DS_RRLIST_DUPLICATE) {
	ds_thread_log(15, "\tDuplicated request %d", old_req->reqid);
	*cancelled = old_req->reqid;
	if (old_req->argp) {
	    (old_req->argp->free_fn)(old_req->argp->argval);
	    FREE(old_req->argp);
	}
	FREE(old_req);
    }

    /* Was the request placed?  If not, value is filled */
    if (!(rc & DS_RRLIST_OUTOFWINDOW)) {
	*reqid = reqp->reqid;
	ds_thread_log(15, "\tGranted reqid: %d, %s [%d..%d]", 
		      reqp->reqid, 
		      (rsrc == ODY_TEST_1) ? "TEST1": "TEST2",
		      reqp->low, reqp->high);
    } else {
	ds_thread_log(15, "\tRequest out of window");
	FREE(reqp);
    }
    
    /* Convert return codes to errors */
    rc &= ~DS_RRLIST_DUPLICATE;  /* Duplicate isn't returned */

    if (rc == DS_RRLIST_OUTOFWINDOW) {
	rc = ODY_ENOTINWINDOW;
    } else {
	ASSERT(!rc);
    }
    return rc;
}

int
rsrc_request(fidp, rsrc, gs, type, ts, low, high, pid, cancelled, value, reqid)
    struct ofid *fidp;
    unsigned int rsrc;
    unsigned int gs;
    unsigned int type;
    unsigned int ts;
    int low;
    int high;
    pid_t pid;
    int *cancelled;
    int *value;
    int *reqid;
{
    struct warden *wp;
    struct viceroy_onode *op;
    int           result=0;

    ds_thread_log(100, "Request attempted for pid %d on rsrc %d [%d .. %d]",
		  pid, rsrc, low, high);
    
    switch(rsrc) {
    case ODY_GR_NETBW:
    case ODY_GR_POWER:
    case ODY_TEST_1:
    case ODY_TEST_2:
	break;
    default:
	return ODY_EBADRSRC;
	break;
    }
    
    if (gs != ODY_GENERIC_INTERFACE) {
	return ODY_EBADVERSION;
    }
    
    if (low > high) {
	return EINVAL;
    }

    wp = warden_get(fidp->ofid_ttype);
    if (!wp) {
	ds_thread_log(0,"rsrc_request: cannot find warden for 0x%x!", 
		      fidp->ofid_ttype);
	return ENXIO;  /* Warden died/not there. */
    }

    op = onode_get(fidp);
    if (!op) {
	ds_thread_log(0,"rsrc_request: cannot find onode for (%x.%x.%x.%x)!", 
		      fidp->ofid_ttype, fidp->ofid_tome,
		      fidp->ofid_vnum, fidp->ofid_uniq);
	return ENOENT;
    }

    switch (rsrc) {
    case ODY_GR_NETBW:
	result = rsrc_place_netbw(wp, op, low, high, pid, NULL, NULL,
				  cancelled, value, reqid);
	break;
    case ODY_GR_POWER:
	result = rsrc_place_power(wp, op, low, high, pid, NULL, NULL,
				  cancelled, value, reqid);
	break;
    case ODY_TEST_1:
    case ODY_TEST_2:
	result = rsrc_place_test(low, high, pid, cancelled, value, 
				 reqid, rsrc);
	break;
    default:
	ASSERT(0);
	break;
    }
    onode_put(op);
    return result;
}


int
rsrc_cancel(reqid)
    int reqid;
{
    ds_request_t *req;
    
    ds_lock_write(&rsrc_lock);

    if (ds_rrlist_cancel(reqid, &req) != DS_RRLIST_SUCCESS && 
	rsrc_en_cancel(reqid, &req) != DS_RRLIST_SUCCESS) {

	ds_thread_log(15,"Cancelled request missing (%d)",reqid);

    } else {

	ds_thread_log(15,"Request %d cancelled");
	if (req->argp) {
	    (req->argp->free_fn)(req->argp->argval);
	    FREE(req->argp);
	}
	FREE(req);

    }
    ds_lock_release(&rsrc_lock);

    return 0;
}

int
rsrc_cancpid(pid)
    pid_t pid;
{
    int err;
    ds_thread_log(15,"Odyssey process %d exited", pid);

    ds_lock_write(&rsrc_lock);
    ds_rrlist_purge(pid);
    err = rsrc_en_purge(pid);  /* Also need to purge from
				  energy resource struct. */
    ds_lock_release(&rsrc_lock);

    return 0;
}

/* Internal interface */
int
vr_rsrc_request(wp, op, rsrc, gv, low, high, fn, argp, canc, val, reqid)
    struct warden *wp;
    struct viceroy_onode *op;
    unsigned int rsrc;
    unsigned int gv;
    int low;
    int high;
    REQFNP fn;
    ds_reqarg_t *argp;
    int *canc;
    int *val;
    int *reqid;
{
    int   result=0;
    struct warden *owp;
    pid_t pid; /* -ttype */

    *canc = 0;
    *val = 0;
    *reqid = 0;

    ds_thread_log(100, 
		  "VR_Request attempted for warden %d on rsrc %d [%d..%d]",
		  wp->w_typenum, rsrc, low, high);
    
    if (rsrc != ODY_GR_NETBW) {
	return EINVAL;
    }
    if (gv != ODY_GENERIC_INTERFACE) {
	return EINVAL;
    }
    if (low > high) {
	return EINVAL;
    }

    if (wp->w_typenum == op->o_fid.ofid_ttype) {
	owp = wp;
    } else {
	owp = warden_get(op->o_fid.ofid_ttype);
	if (!owp) {
	    ds_thread_log(0,"vr_rsrc_request: cannot find warden for 0x%x!", 
			  op->o_fid.ofid_ttype);
	    return ENXIO;  /* Warden died/not there. */
	}
    }
	    

    pid = -wp->w_typenum;

    switch (rsrc) {
    case ODY_GR_NETBW:
	result = rsrc_place_netbw(owp, op, low, high, pid, fn, argp,
				  canc, val, reqid);
	break;
    default:
	ASSERT(0);
	break;
    }
    return result;
}

int
vr_rsrc_cancel(reqid)
    int reqid;
{
    return (rsrc_cancel(reqid));
}

int
vr_rsrc_value(op, rsrc, gs)
    struct viceroy_onode *op;
    unsigned int rsrc;
    unsigned int gs;
{
    struct warden *wp;
    int           result=0;

    ds_thread_log(10, "vr_rsrc_value");
    ASSERT(rsrc == ODY_GR_NETBW);
    ASSERT(gs == ODY_GENERIC_INTERFACE);

    wp = warden_get(op->o_fid.ofid_ttype);
    ASSERT(wp);
    
    switch (rsrc) {
    case ODY_GR_NETBW: 
        {
	    cmgr_id_t            cid;
	    cmgr_connection_t    *connp;
	    cid = (wp->w_getcid)(wp, op);
	    connp = cmgr_get(cid, CMGR_LOCK);
	    if (!connp) {
		result = 0; /* XXX */
	    } else {
	        ds_lock_write(&rsrc_lock);
		result = ds_rrlist_value(connp->cc_bwlist);
		ds_lock_release(&rsrc_lock);
		cmgr_put(cid, CMGR_LOCK);
	    }
	}
	break;
    default:
	ASSERT(0);
	break;
    }

    return result;
}

#ifdef BW_TOOL
int
vr_rsrc_getbw(cmgr_id_t cid)
{
    cmgr_connection_t    *connp;
    int result = 0;

    if (cid == -1)
	return current_bw;

    ds_thread_log(10, "vr_rsrc_getbw");
    /* Lookup the connection by cid 
       (get conn first to avoid deadlock with rsrc_monitor) */
    connp = cmgr_get(cid, CMGR_LOCK);
    if (!connp) {
	result = 0; /* XXX */
    } else {
        ds_lock_write(&rsrc_lock);
	result = ds_rrlist_value(connp->cc_bwlist);
	ds_lock_release(&rsrc_lock);
	cmgr_put(cid, CMGR_LOCK);
    }
    /* give back the bw estimation */
    return result;
}
#endif
