#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Viceroy interface for power management 
 *
 * Jason Flinn
 */

#ifndef __RSRC_EN_H__
#define __RSRC_EN_H__

#define APM_SLEEP  0
#define APM_WAKEUP 1

#ifdef __BSD44__
#define WL_NAME "wl0" /* Is there a better way to do this??? */
#define WD_DEV "/dev/red0a"
#ifdef PSCOPE
#define WD_CMD DIOCAPM
#else
#define WD_CMD 0
#define SIOCAPM 0
#endif
#else
#define WL_NAME "eth0" 
#define WD_DEV "/dev/hda"
#define WD_CMD HDIO_DRIVE_CMD

#define SIOCAPM 	SIOCDEVPRIVATE + 8	/* Set APM state */

#define APM_STANDBY 0 /* Enter standby mode */
#define APM_RESUME  1 /* Resume idle mode */

#endif

int wl_apm_sleep (int action);
int wd_apm_standby (void);
int wd_apm_sleep (void);

/* Make these macros eventually */
void APM_NW_INIT (void);
void APM_NW_UP (void);
void APM_NW_DOWN (void);
void APM_NW_CONN_UP (cmgr_id_t cid);
void APM_NW_CONN_DOWN (cmgr_id_t cid);

#define RSRC_EN_PORT 5436  /* Port for communication with Energy Monitor */
#define DEFAULT_VOLTAGE 16.51 /* Measured voltage for power supply */
#define EN_FUDGE 0.05 /* Fudge factor for hysteresis */
#define EN_FUDGEC 0.01 /* Constant fudge factor - 1% of total battery. */

struct em_data_t {
    float  em_scale; /* Scale for current samples */
    float  em_intvl; /* Seconds between samples */
};

/* These data structures define the resource request list for power
   resource requests.  */
struct power_req_t {
    pid_t                pr_pid;
    int                  pr_min_fidelity;
    int                  pr_max_fidelity;
    int                  pr_cur_fidelity;
    int                  pr_cur_adaptation;
    ds_request_t*        pr_prequest;
    struct power_req_t*  pr_pnext;
    struct power_req_t*  pr_pprev;
};

struct power_req_list_t {
    ds_hwlock_t         pl_lock;
    struct power_req_t* pl_head;
    struct power_req_t* pl_tail;
};

#define RSRC_EN_NONE -1

#define NO_MORE_ADAPTATION 1

/* Prototypes for energy resource module */
void rsrc_en_init ();
int  rsrc_en_set_strategy (char* arg);
void rsrc_en_monitor ();
int  rsrc_en_cancel (int reqid, ds_request_t** req);
int  rsrc_en_purge (pid_t pid);
int  rsrc_place_power (struct warden*        wp, 
		       struct viceroy_onode* op, 
		       int                   low, 
		       int                   high, 
		       pid_t                 pid, 
		       REQFNP                fn, 
		       void*                 argp, 
		       int*                  cancelled, 
		       int*                  value, 
		       int*                  reqid);

#endif






