#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef _VFSCALLS_H_
#define _VFSCALLS_H_

/*
 * Root: 
 * 
 * No in arguments, returns the ofid of the root onode in fidp.
 * Can never return an error.
 */

extern int vfs_root  __P((struct ofid *fidp));

/*
 * Lookup: 
 * 
 * Given a directory ofid, and a name to resolve within that
 * ofid, on success ruturns the ofid of the fully-resolved name, and
 * the vtype of that ofid.  Can return any of a range of errors.
 */

extern int vfs_lookup __P((struct ofid *dirp,
			   char *name,
			   struct ofid *fidp,
			   enum ody_vtype *vtypep,
			   struct ody_cred *cred));

/*
 * Access:
 *
 * Given an ofid, a mode of access, and a user credential, return
 * sucess if that user can access that ofid in that mode, and EACCESS
 * otherwise.
 */

extern int vfs_access __P((struct ofid *fidp, int mode,
			   struct ody_cred *cred));

/*
 * Open
 *
 * Given an ofid, a set of open-flags, return on success the
 * <device,inode> pair of the on-disk cache file.  Can fail in a
 * number of ways.  Sets uncacheable if the onode is not UFS cacheable
 */

extern int vfs_open __P((struct ofid *fidp, int flags, dev_t *devp,
			 ino_t *inop, struct ody_cred *cred, 
			 int *uncacheable));

/*
 * Close
 *
 * Given a previously-opened ofid, a set of open-flags, close the
 * file.  Can return any number of errors.
 */

extern int vfs_close __P((struct ofid *fidp, int flags,
			  struct ody_cred *cred));

/*
 * GetAttr 
 * 
 * Given an ofid, fill a struct vattr for that ofid.  Should return an
 * error only in unusual circumstances, since Access should have
 * already been called.  
 */
extern int vfs_getattr __P((struct ofid *fidp, struct ody_vattr *attrp, 
			    struct ody_cred *cred));

/* 
 * RdWr
 *
 * Perform a read or write */
extern int vfs_rdwr __P((struct ofid   *fidp,
			 int           rwflag,
			 int           *count,
			 int           offset,
			 int           ioflag,
			 void          **buf,
			 struct ody_cred  *cred));

/*
 * RdWr Dispose
 *
 * Dispose of a buffer (optionally) returned by rdwr
 */
extern int vfs_rdwrdisp __P((struct ofid   *fidp,
			     int           count,
			     int           offset,
			     void          *buf,
			     struct ody_cred  *cred));

/* 
 * ReadDir
 *
 * Read the contents of a directory */
extern int vfs_readdir __P((struct ofid   *fidp,
			    int           *count,
			    int           offset,
			    struct ody_dirent **buf,
			    struct ody_cred  *cred));

/*
 * ReaddDir Dispose
 *
 * Dispose of a buffer (optionally) returned by readdir
 */
extern int vfs_readdirdisp __P((struct ofid   *fidp,
				int           count,
				int           offset,
				void          *buf,
				struct ody_cred  *cred));

/*
 * tsop
 * 
 * Perform a type-specific operation: basically a pass-through.
 */
extern int vfs_tsop __P((struct ofid *fidp, unsigned int cs,
			 unsigned int type, unsigned int ts,
			 unsigned int opnum, size_t argsz,
			 void *argp, size_t *retsz, void **retpp,
			 struct ody_cred *cred));

/*
 * dispose
 *
 * Dispose of a tsop-generated buffer
 */
extern int vfs_dispose __P((struct ofid *fidp,
			    unsigned int opnum,
			    size_t argsz,
			    void *argp,
			    size_t retsz,
			    void *retp,
			    struct ody_cred *cred));

#ifdef SETS
/*
 * Expand: 
 * 
 * Given a directory ofid, and a name to resolve within that
 * ofid, on success ruturns the ofid of the fully-resolved name, and
 * the vtype of that ofid.  Can return any of a range of errors.
 * Marked as a CURSOR if it represents a query
 */

extern int vfs_expand __P((struct ofid *dirp,
			   char *name,
			   struct ofid *fidp,
			   enum ody_vtype *vtypep,
			   int *curs,
			   struct ody_cred *cred));

/*
 * CursExpand: 
 * 
 * Given a cursor ofid, and a max size, expand the cursor as much as
 * is available or will fit in buf and return the size in retsz.
 */

extern int vfs_cursexpand __P((struct ofid *fidp,
			       size_t size,
			       size_t *retsz,
			       char *buf,
			       struct ody_cred *cred));

/*
 * prefetch: 
 * 
 * Given a cursor ofid, and a mode, prefetch the object
 */

extern int vfs_prefetch   __P((struct ofid *fidp,
			       int flags,
			       dev_t *devp,
			       ino_t *inop,
			       struct ody_cred *cred,
			       int *uncacheable));
#endif

#endif /* _VFSCALLS_H_ */
