#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * bitwarden_tsop.c
 *
 *  Kip Walker, Jan 1997
 *
 * support for bitwarden tsops
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <errno.h>

#include "base/ds/ds_thread.h"
#include "base/ds/ds_hash.h"
#include "base/ds/ds_hwlock.h"
#include "base/ds/ds_rrlist.h"

#include <kernel/vfs/includes/ody.h>
#include "include/viceroy_onode.h"
#include "include/odytypes.h"
#include "include/cm_rpc2.h"
#include "include/cm.h"
#include "include/warden.h"

#include <lwp/lwp.h>
#include <rpc2/rpc2.h>

#include "bitwarden.h"
#include "bitwarden_onode.h"

/*
 * adjust the "quality" parameters
 */
int bitw_set_params(struct warden *wp,
		    struct viceroy_onode *op,
		    size_t        argsz,
		    void          *argp,
		    size_t        *retsz,
		    void          **retp)
{
    struct bitw_onode_data *wodp = (struct bitw_onode_data *) op->o_data;
    bitw_quality_t         *qual = (bitw_quality_t *) argp;
    bitw_quality_t         *oldq;
    char                   *buf;
    
    if (argsz < sizeof(bitw_quality_t)) {
	*retp = NULL;
	*retsz = 0;
	return EINVAL;
    }
    
    ALLOC(oldq, bitw_quality_t);
    assert(oldq);
    
    /* need to wait for a prefetch to finish */
    while (wodp->buf.prefetching)
	ds_thread_wait((char *)&wodp->buf_lock);
    
    ds_hwlock_write(&wodp->buf_lock);
    
    *oldq = wodp->params;
    
    /* need to start over since chunk_size changing */
    if (qual->chunk_size != oldq->chunk_size) {
	wodp->rpc_msec = 0;
	wodp->chunk_count = 0;
    }

    /* if the pf depth is increased, get a bigger buffer and copy data */
    if (qual->prefetch_depth > oldq->prefetch_depth) {
	if (qual->prefetch_depth > BITW_MAX_DEPTH) {
	    *retp = NULL;
	    *retsz = 0;
	    ds_hwlock_release_write(&wodp->buf_lock);
	    return EINVAL;
	}
	
	/* grow the prefetch buffer */
	NALLOC(buf, char, qual->prefetch_depth);
	ASSERT(buf);
	if (wodp->buf.consumer_left > wodp->buf.prefetched) {
	    bcopy(wodp->buf.consumer, buf, wodp->buf.prefetched);
	} else {
	    bcopy(wodp->buf.consumer, buf, wodp->buf.consumer_left);
	    bcopy(wodp->buf.buf, buf+wodp->buf.consumer_left,
		  wodp->buf.prefetched - wodp->buf.consumer_left);
	}
	free(wodp->buf.buf);
	wodp->buf.buf = buf;
	wodp->buf.consumer = buf;
	wodp->buf.producer = buf+wodp->buf.prefetched;
	wodp->buf.consumer_left = qual->prefetch_depth;
	wodp->buf.producer_left = qual->prefetch_depth - wodp->buf.prefetched;
	/* might want to initiate a prefetch here? */
    }
    wodp->params = *qual;

    ds_thread_log(1, "new pfd %d, new cs %d", qual->prefetch_depth,
		  qual->chunk_size);

    ds_hwlock_release_write(&wodp->buf_lock);
    
    *retsz = sizeof(bitw_quality_t);
    *retp = (void *)oldq;
    return 0;
}

