#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * libwebw_tsops.c -- wrappers for applications which want to
 *                    use bitwarden tsops
 */

#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <errno.h>

#if 0
#include <sys/queue.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#endif

#include <sys/param.h>
#include <sys/time.h>

#include "include/odyssey.h"
#include "include/odytypes.h"

#include "base/ds/ds_thread.h"
#include "base/ds/ds_hash.h"
#include "base/ds/ds_hwlock.h"

#include "bitwarden.h"

/*
 * PUBLIC FUNCTIONS
 */

/*
 * bits_set_params -- given the fd of an open bitwarden-managed object,
 *                    and a structure with quality control values,
 *                    call the tsop to update the warden's values
 */
int bits_set_params(int            fd, /* open file descriptor
					  of object */
		    bitw_quality_t *qual,
		    bitw_quality_t **oldq)
{
    struct ody_vers vers;
    size_t          retsz=sizeof(bitw_quality_t);
    int             result;

    ALLOC(*oldq, bitw_quality_t);
    ASSERT(*oldq);

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = BIT_TTYPE;
    vers.ov_tsvnum = BITWARDEN_VERSION;

    result = ody_ftsop(fd, &vers, BITW_SET_PARAMS, sizeof(bitw_quality_t),
		       (void *)qual, &retsz, *oldq);
    if (result) {
	FREE(*oldq);
	*oldq = NULL;
	return(result);
    }

    return(0);
}
