#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef __MAPW_PRIVATE_H__
#define __MAPW_PRIVATE_H__

#include "base/ds/ds_hwlock.h"

#define MAPW_SERVER_PORT        5125

/* TSOPs */
#define MAPW_SET_FILTERS        1
#define MAPW_SET_CROP           2

/* Structure that contains general data that will be stored in 
   the warden structure */
typedef struct {
  char*       mw_server;     /* Hostname of the map server */
  ds_hwlock_t mw_lock;       /* Warden-level lock */
  cmgr_id_t   mw_connid;     /* Connection to server */
} MapClientData;

/* Structure that contains onode specific data for map onodes. */
struct map_onode_t {
  int         mo_len;        /* Length of (undistilled) map data */
  int         mo_datalen;    /* Length of distilled data */
  char*       mo_data;       /* The map data */
};

/* Function Prototypes */
int mapw_parse_arguments(int cur, int argc, char *argv[]);
int mapw_initialize(struct warden *wp);
int mapw_get_root(struct warden *wp, tome_t tome, 
		  struct viceroy_onode **ppRoot);
int mapw_lookup(struct warden* wp, struct viceroy_onode* odp, 
		struct ody_cred* cred, char* name, char** remain, 
		struct viceroy_onode** opp);
int mapw_access(struct warden* wp, struct viceroy_onode* op, 
		struct ody_cred* cred, int mode);
int mapw_get_attr(struct warden* wp, struct viceroy_onode* op, 
		  struct ody_vattr *vap, struct ody_cred* cred);
int mapw_open (struct warden *wp, struct viceroy_onode *op, 
	       struct ody_cred *cred, int mode);
int mapw_close(struct warden *wp, struct viceroy_onode  *op, 
	       struct ody_cred  *cred, int mode);
int mapw_read_write (struct warden* wp, struct viceroy_onode* op, 
		     struct ody_cred* cred, int rwflag, int* count, 
		     int offset, int ioflag, void **data);
int mapw_read_write_dispose (struct warden* wp, struct viceroy_onode* op, 
			     struct ody_cred* cred, int count, int offset,
			     void* data);
int mapw_readdir (struct warden *wp, struct viceroy_onode *op,
		  struct ody_cred *cred, int *count, int offset,
		  void *data);
int mapw_TSOP(struct warden* wp, int tsvnum, struct viceroy_onode* op,
		struct ody_cred* cred, int opnum, size_t argsz,
		void* argp, size_t* retsz, void** retp);
int mapw_TSOP_dispose(struct warden* wp, struct viceroy_onode* op, 
		      struct ody_cred* cred, int opnum, size_t argsz, 
		      void* argp, size_t retsz, void* retp);
cmgr_id_t mapw_get_CID(struct warden *wp, struct viceroy_onode  *op);
void mapw_ofree(void* odata);

#endif

