#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/* Library that implements the Odyssey QuickTime API using TSOPs */

#ifdef WARDEN
#undef WARDEN
#endif

#include "include/odytypes.h"
#include "include/odyssey.h"
#include "qt.h"
#include "qt_warden.h"

int ody_qt_openmovie(fd, num_tracks, tracks)
int fd;
int *num_tracks;
struct qtw_track_hdr *tracks;
{
  struct ody_vers vers;
  int retsz, retval;

  vers.ov_gs = ODY_GENERIC_INTERFACE;
  vers.ov_type = QT_TTYPE;
  vers.ov_tsvnum = QT_VERSION;

  retsz = *num_tracks * sizeof(struct qtw_track_hdr);

  retval = ody_ftsop(fd, &vers, QT_OPENMOVIE, 0, NULL, &retsz, tracks);

  ASSERT((retval <0) || (retsz % sizeof(struct qtw_track_hdr) == 0));
  *num_tracks = retsz/sizeof(struct qtw_track_hdr);

  return retval;
}



int ody_qt_getframe(fd, frame_spec, frame, vidcodec_maxsize)
int fd;
struct qtw_frame_request *frame_spec;
struct qtw_frame_info *frame;
int vidcodec_maxsize;
{
  struct ody_vers vers;
  int retsz;
  int rc;
  
  vers.ov_gs = ODY_GENERIC_INTERFACE;
  vers.ov_type = QT_TTYPE;
  vers.ov_tsvnum = QT_VERSION;

  retsz = sizeof(struct qtw_frame_info) + vidcodec_maxsize;
    
  rc = ody_ftsop(fd, &vers, QT_GETFRAME, sizeof(*frame_spec),
		   frame_spec, &retsz, frame);
  ASSERT((rc < 0) || (retsz == frame->size + sizeof(struct qtw_frame_info)));
  return rc;
}


int ody_qt_path_openmovie(path, num_tracks, tracks)
char *path;
int *num_tracks;
struct qtw_track_hdr *tracks;
{
  struct ody_vers vers;
  int retsz, retval;

  vers.ov_gs = ODY_GENERIC_INTERFACE;
  vers.ov_type = QT_TTYPE;
  vers.ov_tsvnum = QT_VERSION;

  retsz = *num_tracks * sizeof(struct qtw_track_hdr);

  retval = ody_tsop(path, &vers, QT_OPENMOVIE, 0, NULL, &retsz, tracks);

  ASSERT((retval <0) || (retsz % sizeof(struct qtw_track_hdr) == 0));
  *num_tracks = retsz/sizeof(struct qtw_track_hdr);

  return retval;
}



int ody_qt_path_getframe(path, frame_spec, frame, vidcodec_maxsize)
char *path;
struct qtw_frame_request *frame_spec;
struct qtw_frame_info *frame;
int vidcodec_maxsize;
{
  struct ody_vers vers;
  int retsz;
  int rc;
  
  vers.ov_gs = ODY_GENERIC_INTERFACE;
  vers.ov_type = QT_TTYPE;
  vers.ov_tsvnum = QT_VERSION;

  retsz = sizeof(struct qtw_frame_info) + vidcodec_maxsize;
    
  rc = ody_tsop(path, &vers, QT_GETFRAME, sizeof(*frame_spec),
		frame_spec, &retsz, frame);
  ASSERT((rc < 0) || (retsz == frame->size + sizeof(struct qtw_frame_info)));
  return rc;
}
