#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * qtw_cache.c
 *
 * routines to handle viceroy-supported disk cache for QuickTime warden
 */

/* Bad header list */
#include <kernel/vfs/includes/ody.h>

#include <unistd.h>

#include <sys/queue.h>

#include "include/odytypes.h"
#include "include/viceroy_onode.h"
#include "include/odyssey.h"
#include "include/cache_mgr.h"
#include "qtw_cache.h"

/* Get a new (unused) cache entry */
struct cache_entry *qtw_cache_create(struct cache_entry *cache_files,
				     int ncache_files,
				     struct viceroy_onode *op)
{
    /* for now, just a simple scan-through of the linked list
       to find an unused entry */
    /* unused entries have no ce_data */
    while (cache_files) {
	if (cache_files->ce_data == NULL) {
	    struct qtw_cache_data *ce_data;
	    ALLOC(ce_data, struct qtw_cache_data);
	    ce_data->qtwc_op = op;
	    cache_files->ce_data = ce_data;
	    break;
	}
	else
	    cache_files = cache_files->ce_link.tqe_next;
    }
    return cache_files;
}

/* release a cache entry */
void qtw_cache_release(struct cache_entry *cep)
{
    truncate(cep->ce_fname, 0);
    FREE(cep->ce_data);
    cep->ce_data = NULL;
}

/* Open a cache entry for read/write, return a file descriptor */
int qtw_cache_open(struct cache_entry *cep, int flags)
{
    return cache_open(cep, flags);
}

/* Close a cache entry that was opened for read/write */
int qtw_cache_close(struct cache_entry *cep, int fd, int modified)
{
    return cache_close(cep, fd, modified);
}
