#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * qtw_prefetch.c
 *
 * routines to handle prefetching/buffering for QuickTime warden
 */

/* Bad header list */
#include <kernel/vfs/includes/ody.h>

#include <sys/queue.h>

#include <string.h>
#include <unistd.h>

#include "include/odytypes.h"

#include "base/ds/ds_thread.h"
#include "base/ds/ds_hash.h"
#include "base/ds/ds_log.h"
#include "base/ds/ds_safeq.h"
#include "base/ds/ds_hwlock.h"
#include "base/ds/ds_rrlist.h"

#include <lwp/lwp.h>
#include <lwp/lock.h>

#include <rpc2/rpc2.h>

#include "include/cm_rpc2.h"
#include "include/cm.h"

#include "include/viceroy_onode.h"
#include "include/odyssey.h"
#include "include/warden.h"
#include "include/viceroy.h"
#include "include/onode-manager.h"
#include "include/cache_mgr.h"

#include "qtw_cache.h"
#include "qt_server.h"
#include "ody_qt.h"
#include "qt.h"
#include "qt_warden.private.h"
#include "qtw_prefetch.h"

#include "odyssey/viceroy/rsrc_en.h"

int FRAMES_FETCHED = 0;

extern int to_multiple;
extern int refetch_low_quality;

/* given a the time_elapsed since frame frame_no was fetched, and that
   we've already fetched up to time last_fetched, figure out where to start
   the next prefetch. */
static
int
qtw_skipforward(qtwp, last_req_zztime, time_elapsed, last_fetched_zztime)
    struct qtw_prefetch *qtwp;
    int last_req_zztime;
    int time_elapsed;
    int last_fetched_zztime;
{
    int track_ID = qtwp->track_ID;
    struct viceroy_onode *op = qtwp->op;
    struct onode_data *o_data = op->o_data;
    int zztime;
    
    if (last_fetched_zztime == -1)  /* very first prefetch, start from 0 */
	return 0;

    ASSERT(o_data->all_hdrs);
    ASSERT((track_ID >= 0) && (track_ID < o_data->num_tracks));
    zztime = last_req_zztime + time_elapsed;

    if (zztime <= last_fetched_zztime) /* don't refetch stuff */
	zztime = last_fetched_zztime + 1;

    ASSERT(zztime >= 0);

    if (zztime > o_data->all_hdrs[track_ID].total_time) {
	/* we ran off the end, return error */
	ds_thread_log(10,"qtw_skipforward: zztime %d > last frame %d",
		      zztime, o_data->all_hdrs[track_ID].total_time);
	return -1;
    }

    return zztime;
}

/* fetch a buncha frames from the server */
/* frame_no is the frame# to start from, maxbytes is byte limit on frame
   data */
/* *nframes contains max frames to fetch on entry, #frames fetched on exit */
/* frame_info is array to put frame meta-data in (at least as large
   as *nframes) */
/* buf is buffer to put frame data in (at least as large as maxbytes) */

static
int
qtw_fetch_from_server(struct qtw_prefetch *qtwp,
		      void *buf,
		      QT_FrameInfo *frame_info,
		      int zztime,
		      int *nframes,
		      int maxbytes)
{
#if 0
    struct timeval tv1, tv2, tv3;
#endif

    struct timeval rpc_tv1, rpc_tv2, rpc_tv3;

    struct warden *wp;
    struct qtw_data *qtwdp;
    struct qtt_data *qttdp;
    struct viceroy_onode *op;
    struct onode_data *o_data;
    struct qtw_track_hdr *all_hdrs;
    RPC2_Integer vnum;
    tome_t tome;
    cmgr_connection_t *connp;
    RPC2_Integer track_ID;
    int result;
    int i;

    ASSERT(buf);
    ASSERT(op = qtwp->op);
    ASSERT(o_data = op->o_data);
    track_ID = qtwp->track_ID;
    ASSERT((track_ID >= 0) && (track_ID < o_data->num_tracks));

    ASSERT(wp = qtwp->wp);
    ASSERT(qtwdp = wp->w_data);
    tome = op->o_fid.ofid_tome;
    ASSERT((tome >= 0) && (tome < QT_MAXTOMES));
    ASSERT(qttdp = qtwdp->qtw_tome[tome]);
    vnum = op->o_fid.ofid_vnum;

    ASSERT(all_hdrs = o_data->all_hdrs);

    /* Fetch the data */
    if (cmgr_validate(o_data->od_cid)) {
	connp = cmgr_get(o_data->od_cid, CMGR_LOCK);
	if (!connp)
	    result = ENOTCONN;
	else {
	    SE_Descriptor frame_data_sftp;
	    RPC2_BoundedBS frame_infoBS;
	    
	    ASSERT(connp);
	    
	    bzero(&frame_data_sftp,sizeof(frame_data_sftp));
	    frame_data_sftp.Tag = SMARTFTP;
	    frame_data_sftp.Value.SmartFTPD.Tag = FILEINVM;
	    frame_data_sftp.Value.SmartFTPD.TransmissionDirection =
		SERVERTOCLIENT;
	    frame_data_sftp.Value.SmartFTPD.ByteQuota =
		(RPC2_Integer) maxbytes;
	    frame_data_sftp.Value.SmartFTPD.SeekOffset = 0;
	    frame_data_sftp.Value.SmartFTPD.FileInfo.ByAddr.vmfile.MaxSeqLen =
		(RPC2_Integer) maxbytes;
	    frame_data_sftp.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqBody = 
		(RPC2_ByteSeq) buf;
	    
	    frame_infoBS.SeqLen = 0;
	    frame_infoBS.MaxSeqLen = *nframes * sizeof(QT_FrameInfo);
	    frame_infoBS.SeqBody = (RPC2_ByteSeq) frame_info;
	    
	    ds_thread_log(100,"entering QT_GetFrame_RPC(%ld,%ld,%d,%d)",
			  vnum, track_ID, zztime, *nframes);
#if 0
	    gettimeofday (&tv1, 0);
	    timersub (&tv1, &tv2, &tv3);
	    ds_thread_log (0, "time since last network usage: %d.%06d",
			   tv3.tv_sec, tv3.tv_usec);
#endif
	    gettimeofday (&rpc_tv1, 0);

	    result = QT_GetFrame_RPC(connp->cc_handle.rpc2, vnum, track_ID,
				     zztime, maxbytes, &frame_infoBS,
				     &frame_data_sftp);

	    gettimeofday (&rpc_tv2, 0);
	    timersub (&rpc_tv2, &rpc_tv1, &rpc_tv3);
	    if (rpc_tv3.tv_sec >= 1) {
	      ds_thread_log (0, "fetched %d bytes in %d.%06d seconds",
			     maxbytes, rpc_tv3.tv_sec, rpc_tv3.tv_usec);
	    }

#if 0
	    gettimeofday (&tv2, 0);
#endif
	    ds_thread_log(100,"QT_GetFrame_RPC(%ld,%ld,%d,%d) returned %d",
			  vnum, track_ID, zztime, nframes, result);
	    cmgr_put(o_data->od_cid, CMGR_LOCK);
	    ds_thread_yield();	/* give someone else a chance (rsrc_mon)
				   This is because we held the conn_lock for
				   a long time. XXX */
	    
	    ASSERT(frame_infoBS.SeqLen <= *nframes * sizeof(QT_FrameInfo));
	    *nframes = frame_infoBS.SeqLen/sizeof(QT_FrameInfo);
	    ASSERT(frame_infoBS.SeqLen == *nframes * sizeof(QT_FrameInfo));
	}
    }
    else
	result = ETIMEDOUT; /* we assume the RPC would've
			       failed, since the conn is
			       marked down */
    
    if (result != RPC2_SUCCESS) {
	if (result < RPC2_FLIMIT) {
	    cmgr_markdown(o_data->od_cid);
	    ds_thread_log(0,
			  "QT_GetFrame_RPC(%ld,%ld,%d,%d): Connection %d down, rc = %d",
			  vnum, track_ID, zztime, nframes, o_data->od_cid, result);
	    result = ETIMEDOUT;
	}
	goto bad;
    }
    
    for (i = 0; i < *nframes; i++) {
	frame_info[i].track_ID = ntohl(frame_info[i].track_ID);
	frame_info[i].frame_no = ntohl(frame_info[i].frame_no);
	frame_info[i].time_dur = ntohl(frame_info[i].time_dur);
	frame_info[i].zztime = ntohl(frame_info[i].zztime);
	frame_info[i].xapi_rev = ntohl(frame_info[i].xapi_rev);
	frame_info[i].flags = ntohl(frame_info[i].flags);
	frame_info[i].xpos = ntohl(frame_info[i].xpos);
	frame_info[i].ypos = ntohl(frame_info[i].ypos);
	frame_info[i].xsize = ntohl(frame_info[i].xsize);
	frame_info[i].ysize = ntohl(frame_info[i].ysize);
	frame_info[i].special = ntohl(frame_info[i].special);
	frame_info[i].bufpos = ntohl(frame_info[i].bufpos);
	frame_info[i].size = ntohl(frame_info[i].size);
    }

    return 0;

 bad:
    ds_thread_log (0, "Oops: RPC rc is %d\n", result);

    ASSERT(result);
    return result;
}

/* Do a "ping" of the server for no reason other than to check if the
   network has improved. The server SFTP's back numbytes bytes to us */
static
int
active_ping(cmgr_id_t cid, int numbytes)
{
    cmgr_connection_t *connp;
    int result = 0;

    if (cmgr_validate(cid)) {
	connp = cmgr_get(cid, CMGR_LOCK);
	if (!connp)
	    result = ENOTCONN;
	else {
	    SE_Descriptor ping_data_sftp;
	    void *buf;
	    int result;
	    
	    BALLOC(buf, numbytes);
	    
	    bzero(&ping_data_sftp, sizeof(ping_data_sftp));
	    ping_data_sftp.Tag = SMARTFTP;
	    ping_data_sftp.Value.SmartFTPD.Tag = FILEINVM;
	    ping_data_sftp.Value.SmartFTPD.TransmissionDirection = 
		SERVERTOCLIENT;
	    ping_data_sftp.Value.SmartFTPD.ByteQuota = numbytes;
	    ping_data_sftp.Value.SmartFTPD.SeekOffset = 0;
	    ping_data_sftp.Value.SmartFTPD.FileInfo.ByAddr.vmfile.MaxSeqLen =
		(RPC2_Integer) numbytes;
	    ping_data_sftp.Value.SmartFTPD.FileInfo.ByAddr.vmfile.SeqBody = 
		(RPC2_ByteSeq) buf;
	    
	    result = QT_Ping_RPC(connp->cc_handle.rpc2,
				 (RPC2_Integer) numbytes, &ping_data_sftp);

	    FREE(buf);
	    cmgr_put(cid, CMGR_LOCK);
	}
    }
    else
	result = ETIMEDOUT;

    if (result != RPC2_SUCCESS) {
	if (result < RPC2_FLIMIT) {
	    cmgr_markdown(cid);
	    result = ETIMEDOUT;
	}
    }

    return result;
}
	
void
qtw_prefetch(qtwp)
    struct qtw_prefetch *qtwp;
{
    struct viceroy_onode *op = qtwp->op;
    struct onode_data *o_data = op->o_data;
    struct qtw_track_hdr *all_hdrs = o_data->all_hdrs;

    ds_hwlock_read(&qtwp->qtwp_lock);

    while(qtwp->prefetch_enable) {
	struct timeval tv_stop,time_since_last;
	int msec_since_last, zztime, free_space;
	int nframes, track_ID;
	QT_FrameInfo *frame_info = NULL;
	struct qtwp_fbuf *qtwfbp = NULL;
	int rc;

	track_ID = qtwp->track_ID;
	ASSERT((track_ID >= 0) && (track_ID < o_data->num_tracks));
	
	/* If we are in local mode, there's nothing to do */
	if (qtwp->is_cached) {
	    int result, chunksize = qtwp->maxchunksize;
	    cmgr_id_t cid = o_data->od_cid;
	    ds_hwlock_release_read(&qtwp->qtwp_lock);
	    ds_thread_log(10,"qtw_prefetch: is_cached, doing active_ping");
	    result = active_ping(cid, chunksize);
	    if (result != 0)
		ds_thread_log(10,"qtw_prefetch: active ping failed");
	    ds_thread_wait((char *) qtwp);
	    ds_hwlock_read(&qtwp->qtwp_lock);
	    continue;
	}

	/* how many frames can we allocate space for? */
	    
	gettimeofday(&tv_stop, NULL);
	timersub(&tv_stop, &qtwp->last_req_time, &time_since_last);
	msec_since_last = time_since_last.tv_sec*1000 +
	    time_since_last.tv_usec/1000;
	
	free_space = qtwp->max_bufsize - qtwp->cur_bufsize;
#ifdef QT_RPC_DEBUG
	ds_thread_log (0, "Buffer size: %d, free space: %d", qtwp->cur_bufsize,
		       free_space);
#endif

	/* We should always have enough for at least one frame */
	if (free_space < all_hdrs[track_ID].max_fvid_size) {
	    ds_hwlock_release_read(&qtwp->qtwp_lock);
	    ds_thread_log(10,"qtw_prefetch: Waiting for free space");
	    ds_thread_wait((char *)qtwp);
	    ds_thread_log(10,"qtw_prefetch: Got free space");
	    ds_hwlock_read(&qtwp->qtwp_lock);
	    continue;
	}

	/* We don't actually have streaming, so we fake it by
	   "quadruple buffering" -- never fetch more than 1/4 of the maximum
	   at any given time */

	if (free_space > qtwp->max_bufsize/4)
	    free_space = qtwp->max_bufsize/4;


	/* XXX Time-based limit. Never fetch more than a maxplaytime's worth of
	   frames */
	{
	    int nframes = (long) (all_hdrs[track_ID].last_frame+1) *
		qtwp->maxplaytime / all_hdrs[track_ID].total_time;
	    int seconds_worth = nframes * all_hdrs[track_ID].ave_fvid_size;
	    if (seconds_worth < all_hdrs[track_ID].max_fvid_size)
		seconds_worth = all_hdrs[track_ID].max_fvid_size;
	    if (free_space > seconds_worth)
		free_space = seconds_worth;
	}

	/* find the next zztime to fetch -- add the estimated time to
	   fetch to msec_since_last, so that we're not out of date
	   by the time we're done fetching */

	if (free_space > qtwp->maxchunksize) {
	    free_space = qtwp->maxchunksize;
	}

	if (free_space < all_hdrs[track_ID].max_fvid_size) {
	    free_space = all_hdrs[track_ID].max_fvid_size;
	    nframes = 1;
	}
	else {
	    nframes = MAX_FRAMES;
	    msec_since_last += ((long)free_space*1000)/(qtwp->bw);
	}
	    
	/* if we bumped the quality up, then disregard last_fetched */
	/* since we want to refetch everything at the higher quality */
	
	if (refetch_low_quality &&
	    (track_ID > qtwp->last_fetched_track_ID))
	    zztime = qtw_skipforward(qtwp, qtwp->last_req_zztime,
				     msec_since_last,
				     0);
	else
	    zztime = qtw_skipforward(qtwp, qtwp->last_req_zztime,
				     msec_since_last,
				     qtwp->last_fetched_zztime);

	if (qtwp->last_fetched_frame_no >= all_hdrs[track_ID].last_frame 
            || zztime < 0) {
#if 0
	if (zztime < 0) { /* we ran off the end, wait for app */
#endif
	    struct qtwp_frame *qtwfp;

	    /* Insert a EOF token to mark end of movie*/
	    ALLOC(qtwfp, struct qtwp_frame);
	    qtwfp->hdr.frame_no = -2; /* EOF token */
	    qtwfp->hdr.zztime = -1;
	    qtwfp->bufpos = -1;
	    qtwfp->buffer = NULL;
	    ds_safeq_enq(qtwp->full_buffers, qtwfp);

	    /* Wait until a significant event happens: either the app reaches
	       the end of the movie OR the quality improves and we would like
	       to refetch frames at a higher quality */
	    do {
	      ds_hwlock_release_read(&qtwp->qtwp_lock);
	      ds_thread_wait((char *)qtwp);
	      ds_hwlock_read(&qtwp->qtwp_lock);
	    } while (qtwp->prefetch_enable && 
		     !(refetch_low_quality && 
		       (track_ID > qtwp->last_fetched_track_ID)));

	    if (!qtwp->prefetch_enable) {
	      break;
	    }	

	    continue;
	}
	
	ASSERT((zztime > qtwp->last_fetched_zztime) ||
	       (qtwp->track_ID > qtwp->last_fetched_track_ID));

	/* allocate buffer space for the fetch */
	
	QTWFB_ALLOC(qtwfbp, nframes, free_space);
	NALLOC(frame_info, QT_FrameInfo, nframes);
	
	/* do the RPC to get the frame from the server */
	/* qtwp goes in unlocked, comes out unlocked */
	
	ds_hwlock_release_read(&qtwp->qtwp_lock);
	ds_thread_log(10,"bandwidth is %ld, fetching %d",
		      qtwp->bw,free_space);
	rc = qtw_fetch_from_server(qtwp,QTWFB_BUF(qtwfbp,0),frame_info,zztime,
				   &nframes, free_space);
	ds_hwlock_read(&qtwp->qtwp_lock);

	/* We may have fetched fewer frames, record that fact in the fbuf */
	qtwfbp->num_frames = nframes;
	
	/* did someone close the onode while we were doing RPCs? */
	if (!qtwp->prefetch_enable) {
	    QTWFB_FREE_ALL(qtwfbp);
	    FREE(frame_info);
	    break;
	}	
	ds_hwlock_write(&qtwp->qtwp_lock);
	
	if (rc == 0) {
	    int i;
	    /* if we refetched stuff, then we want to delete everything
	       >= zztime from the queue */
	    
	    ASSERT(nframes <= MAX_FRAMES);
	    ASSERT(nframes > 0);
	    
	    if (refetch_low_quality &&
		(track_ID > qtwp->last_fetched_track_ID)) {
		struct ds_safeq_t *newq = ds_safeq_create();
		struct ds_safeq_t *oldq = qtwp->full_buffers;
		int zztime = frame_info[0].zztime;
		
		ds_thread_log(100, "Refetch from track %d (old %d)",
			      track_ID, qtwp->last_fetched_track_ID);
		
		while(ds_safeq_count(oldq)) {
		    struct qtwp_frame *qtwfp = (struct qtwp_frame *)
			ds_safeq_deq(oldq);
		    if (qtwfp->hdr.zztime < zztime)
			ds_safeq_enq(newq, qtwfp);
		    else {
			ds_thread_log(100, "Discarded obsoleted frame %d, "
				      "track %d from prefetch queue",
				      qtwfp->hdr.frame_no,
				      qtwfp->hdr.trackID);
			QTWF_FREE(qtwp, qtwfp);
		    }
		}
		
		while (ds_safeq_count(newq))
		    ds_safeq_enq(oldq, ds_safeq_deq(newq));
		
		ds_safeq_destroy(newq);
	    }
	    
	    ASSERT((zztime >= 0) && (zztime <= all_hdrs[track_ID].total_time));
	    
	    {
		int last = frame_info[nframes-1].bufpos;
		int first = frame_info[0].bufpos;
		int size = frame_info[nframes-1].size;
		ASSERT(first == 0);
		ASSERT(last + size <= free_space);
	    }
	    
	    qtwp->last_fetched_track_ID = track_ID;
	    qtwp->last_fetched_zztime = frame_info[nframes-1].zztime;
	    qtwp->last_fetched_frame_no = frame_info[nframes-1].frame_no;
	    qtwp->cur_bufsize += free_space;

	    /* insert entries for each frame, into the queue */
	    for (i = 0; i < nframes; i++) {
		struct qtwp_frame *qtwfp = NULL;
	        FRAMES_FETCHED++;
		ALLOC(qtwfp, struct qtwp_frame);
		
		qtwfp->hdr.trackID = frame_info[i].track_ID;
		qtwfp->hdr.frame_no = frame_info[i].frame_no;
		qtwfp->hdr.time_dur = frame_info[i].time_dur;
		qtwfp->hdr.zztime = frame_info[i].zztime;
		qtwfp->hdr.xapi_rev = frame_info[i].xapi_rev;
		qtwfp->hdr.flags = frame_info[i].flags;
		qtwfp->hdr.xpos = frame_info[i].xpos;
		qtwfp->hdr.ypos = frame_info[i].ypos;
		qtwfp->hdr.xsize = frame_info[i].xsize;
		qtwfp->hdr.ysize = frame_info[i].ysize;
		qtwfp->hdr.special = frame_info[i].special;
		qtwfp->hdr.size = frame_info[i].size;
		qtwfp->bufpos = frame_info[i].bufpos;

		qtwfp->retbuf = NULL;
		qtwfp->buffer = qtwfbp;
		/* Add any extra space to the last frame */

		ASSERT(free_space >= 0);

		if (i < nframes - 1)
		    qtwfp->framesize = qtwfp->hdr.size;
		else
		    qtwfp->framesize = free_space;
		free_space -= qtwfp->framesize;

		ds_safeq_enq(qtwp->full_buffers, qtwfp);
	    }
	    
	    ASSERT(qtwp->cur_bufsize <= qtwp->max_bufsize);
	    FREE(frame_info);
	}
	else {
	    struct qtwp_frame *qtwfp = NULL;
	    /* put a "prefetch failed" entry into the queue */
	    QTWFB_FREE_ALL(qtwfbp);
	    FREE(frame_info);
	    ds_thread_log(10,"prefetch (zztime %d, maxbytes %d, track %d)"
			  " failed", zztime, free_space, track_ID);
	    ALLOC(qtwfp, struct qtwp_frame);
	    qtwfp->hdr.frame_no = -1;
	    qtwfp->hdr.zztime = -1;
	    qtwfp->bufpos = -1;
	    qtwfp->buffer = NULL;
	    ds_safeq_enq(qtwp->full_buffers, qtwfp);
	}
	ds_hwlock_release_write(&qtwp->qtwp_lock);
    }

    ASSERT(!qtwp->prefetch_enable);
    ds_hwlock_write(&qtwp->qtwp_lock);
    FREE(qtwp);
}

/* Request argument */
struct cbarg_t {
    struct warden *wp;
    struct viceroy_onode  *op;
    cmgr_id_t     cid;
};

/* Free a request argument.  We have to do a put on the onode */
static
void
qtwp_reqarg_free(argp)
    void *argp;
{
    struct cbarg_t *targp = (struct cbarg_t *)argp;
    
    onode_put(targp->op);
    FREE(targp);
}

/* Take a callback */
static
void
qtwp_callback(reqid, rsrc, val, argp)
    int reqid;
    unsigned int rsrc;
    int val;
    void *argp;
{
    struct cbarg_t *targp = (struct cbarg_t *)argp;
    int result;
    int value = val;
    int low;
    int high;
    int cancelled;
    int newreq;
    ds_reqarg_t *reqargp;
    struct onode_data *o_data;
    struct qtw_prefetch *qtwp;

    ds_thread_log(15,"qtwp_callback: cid %d, reqid %d, value %d",
		  targp->cid, reqid, value);

    /* Need a new reqargp */
    ALLOC(reqargp, ds_reqarg_t);
    reqargp->free_fn = qtwp_reqarg_free;
    /* 
     * Rather than freeing the argp (which we are responsible for
     * doing) we reuse it in the next request 
     */
    reqargp->argval = argp;

    do {
	low = (value/10)*9;
	high = (value/10)*11;

	result = vr_rsrc_request(targp->wp, targp->op, ODY_GR_NETBW, 
				 ODY_GENERIC_INTERFACE, low, high,
				 qtwp_callback, reqargp, &cancelled,
				 &value, &newreq);
	if (cancelled) {
	    ds_thread_log(100,"qtwp_callback: old request %d CANCELLED",
			  cancelled);
	}
	if (result == ODY_ENOTINWINDOW) {
	    ds_thread_log(100,"qtwp_callback: request OUTOFWINDOW (%d)",
			  value);
	    ASSERT(!newreq);
	} else {
	    ds_thread_log(100,"qtwp_callback: request %d placed",
			  newreq);

	    /* Remember which request id is associated with this onode */
	    o_data = (struct onode_data *)targp->op->o_data;
	    ASSERT(qtwp = o_data->prefetch);
	    ds_hwlock_write(&qtwp->qtwp_lock);
	    ASSERT(rsrc == ODY_GR_NETBW);
	    ASSERT(qtwp->bw_reqid == reqid);
	    qtwp->bw = val;
	    qtwp->maxchunksize = (val*(qtwp->maxchunktime))/1000;
	    ds_thread_log(100,"qtwp_callback: %d bytes/sec",val);
	    qtwp->bw_reqid = newreq;
	    ds_hwlock_release_write(&qtwp->qtwp_lock);
	}
    } while (!newreq);
}


void
qtwp_start(wp, op, bufsize, maxtime, mintime, maxplaytime)
    struct warden *wp;
    struct viceroy_onode *op;
    int bufsize;
    int maxtime;
    int mintime;
    int maxplaytime;
{
    struct qtw_data *qtwdp;
    ds_safeq_t *qtwprp;
    struct qtw_prefetch *qtwp;
    struct onode_data *o_data;
    struct cbarg_t *argp;
    int bwval;
    struct timeval timo;
    int timo_ms;

    ASSERT(wp);
    qtwdp = wp->w_data;
    ASSERT(qtwdp);
    qtwprp = qtwdp->qtw_prefetch_req;
    ASSERT(qtwprp);

    ASSERT(op);
    o_data = op->o_data;
    ASSERT(o_data);
    ASSERT(!o_data->prefetch);

    ALLOC(qtwp, struct qtw_prefetch);
    qtwp->max_bufsize = bufsize;
    qtwp->cur_bufsize = 0;
    qtwp->maxchunktime = maxtime;
    qtwp->maxplaytime = maxplaytime;

    qtwp->last_fetched_zztime = -1;
    qtwp->last_fetched_frame_no = -1;
    gettimeofday(&qtwp->last_req_time, NULL);
    qtwp->last_req_zztime = 0;

    /* optimistically start with best track; it's upto app to switch */
    qtwp->last_fetched_track_ID = qtwp->track_ID = o_data->num_tracks - 1;
    qtwp->is_cached = 0;

    qtwp->full_buffers = ds_safeq_create();
    /* safe-destroyed, unordered, with-duplicates list */
    qtwp->retd_frames = ds_list_create(NULL, TRUE, TRUE);
    
    qtwp->wp = wp;
    qtwp->op = op;
    qtwp->prefetch_enable = 1;

    ds_hwlock_init(&qtwp->qtwp_lock);

    o_data->prefetch = qtwp;

    /* Set the timeout to be the maximum time we expect a frame-fetch
       RPC to take multiplied by the specified multiple.  (default is 2). */
    if (to_multiple > 0) {
      timo_ms = qtwp->maxchunktime * to_multiple;
      timo.tv_sec = timo_ms / 1000;
      timo.tv_usec = (timo_ms % 1000) * 1000;
      if (cmgr_settimeout(o_data->od_cid, timo))
	ds_thread_log(0,"qtwp_start: cmgr_settimeout(%d,"
		      "%ld.%06ld): %s", o_data->od_cid, timo.tv_sec,
		      timo.tv_usec, strerror(errno));
    }

    /* find the current chunksizes, and call the callback function with
     it. this first, "fake" callback will inform the callback function of
     current bandwidth, so it'll set the chunksizes appropriately, and
     register for future callbacks.
     */

    ALLOC(argp, struct cbarg_t);
    argp->wp = wp;
    argp->op = onode_get(&op->o_fid);
    argp->cid = o_data->od_cid;
    
    bwval = vr_rsrc_value(op, ODY_GR_NETBW, ODY_GENERIC_INTERFACE);
    qtwp->bw_reqid = -1; /* fake reqid for fake callback */
    qtwp_callback(qtwp->bw_reqid, ODY_GR_NETBW, bwval, argp);

    /* add qtwp to the prefetch request queue for a prefetch thread to pick
       up */
    ds_safeq_enq(qtwprp, qtwp);
}

void qtwp_stop(qtwp)
    struct qtw_prefetch *qtwp;
{
    /* clean up everything, except leave the prefetch structure around,
       since the prefetch thread might still be reading it */

    ds_hwlock_write(&qtwp->qtwp_lock);

    qtwp->prefetch_enable = 0;
    qtwp->wp = NULL;
    qtwp->op = NULL;

    /* since we have a lock on qtwp, the prefetch thread won't be playing
       around with the queue. the next time the prefetch thread gets
       a lock, it'll notice that the prefetch_enable flag has been
       cleared, and close up shop */

    while (ds_safeq_count(qtwp->full_buffers)) {
	struct qtwp_frame *qtwfp = ds_safeq_deq(qtwp->full_buffers);
	QTWF_FREE(qtwp, qtwfp);
    }

    ds_safeq_destroy(qtwp->full_buffers);
    ds_thread_log(100,"qtwp_stop: stopped prefetch");
    ds_thread_log(0, "qtw_debug: frames fetched %d\n", FRAMES_FETCHED);
    
    ds_hwlock_release_write(&qtwp->qtwp_lock);
    /* prefetch thread might already be waiting for us to close the movie,
       if it prefetched beyond the end */
    ds_thread_wakeup((char *)qtwp);
}

