#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifdef WARDEN
#undef WARDEN
#endif

#include <stdio.h>
#include <string.h>

#include "include/odyssey.h"
#include "include/odytypes.h"
#include "base/ds/ds_thread.h"
#include "base/ds/ds_hash.h"
#include "base/ds/ds_hwlock.h"
#include "base/ds/ds_rrlist.h"

#include <include/viceroy_onode.h>
#include <kernel/vfs/includes/ody.h>
#include "include/cm_rpc2.h"
#include "include/cm.h"
#include "include/warden.h"
#include "include/onode-manager.h"

#include "speechwarden.h"
#include "libodyspeech_tsops.h"
#include "speechw_private.h"

/* speechSetServer() is a wrapper around the corresponding speech warden's
   TSOP.  The TSOP does not dynamically allocate memory, so some steps are
   omitted here.  If the TSOP is successfully executed, the function returns
   zero.  Otherwise, the function returns a return code from the execution
   of the TSOP or from the TSOP itself. */
int speechSetStrategy(int fd, /* open file descriptor of object */
		      enum strategyType strategy)
{
    struct ody_vers vers;
    int             rc;
    size_t          retsz = sizeof (rc);
    int             result = 0;

    /* Set up the Odyssey version structure */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = SPEECH_TTYPE;
    vers.ov_tsvnum = SPEECHWARDEN_VERSION;

    /* Make the TSOP - it does not allocate memory dynamically */
    result = ody_ftsop (fd, &vers, SPEECHW_SET_STRATEGY, sizeof(strategy),
			&strategy, &retsz, &rc);
    if (result == 0) {
      result = rc;
    } /* endif */

    return(result);
}

/* speechSetFidelity() is a wrapper around the corresponding speech warden's
   TSOP.  The TSOP does not dynamically allocate memory, so some steps are
   omitted here.  If the TSOP is successfully executed, the function returns
   zero.  Otherwise, the function returns a return code from the execution
   of the TSOP or from the TSOP itself. */
int speechSetFidelity(int fd, /* open file descriptor of object */
		      enum fidelityType fidelity)
{
    struct ody_vers vers;
    int             rc;
    size_t          retsz = sizeof (rc);
    int             result = 0;

    /* Set up the Odyssey version structure */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = SPEECH_TTYPE;
    vers.ov_tsvnum = SPEECHWARDEN_VERSION;

    /* Make the TSOP - it does not allocate memory dynamically */
    result = ody_ftsop (fd, &vers, SPEECHW_SET_FIDELITY, sizeof(fidelity),
			&fidelity, &retsz, &rc);
    if (result == 0) {
      result = rc;
    } /* endif */

    return(result);
}

#if 0
/* speechSetSearchEffort() is a wrapper around the corresponding
   speech warden's TSOP.  The TSOP does not dynamically allocate
   memory, so some steps are omitted here.  If the TSOP is
   successfully executed, the function returns zero.  Otherwise, the
   function returns a return code from the execution of the TSOP or
   from the TSOP itself. */
int speechSetSearchEffort(int fd, float effort)
{
    struct ody_vers vers;
    int             rc;
    size_t          retsz = sizeof (rc);
    int             result = 0;

    /* Set up the Odyssey version structure */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = SPEECH_TTYPE;
    vers.ov_tsvnum = SPEECHWARDEN_VERSION;

    /* Make the TSOP - it does not allocate memory dynamically */
    result = ody_ftsop (fd, &vers, SPEECHW_SET_SEARCH_EFFORT, sizeof(effort),
			&effort, &retsz, &rc);
    if (result == 0) {
      result = rc;
    } /* endif */

    return (result);
}
#endif

/* speechInitModel() is a wrapper around the corresponding
   speech warden's TSOP.  The TSOP does not dynamically allocate
   memory, so some steps are omitted here.  If the TSOP is
   successfully executed, the function returns zero and places the
   returned models in the specified structure.  Otherwise, the
   function returns a return code from the TSOP execution. */
int speechInitModel (int fd, char* model, StrategyModel* pModel)
{
    struct ody_vers vers;
    size_t          retsz = sizeof (StrategyModel);
    int             result = 0;

    /* Set up the Odyssey version structure */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = SPEECH_TTYPE;
    vers.ov_tsvnum = SPEECHWARDEN_VERSION;

    /* Make the TSOP - it does not allocate memory dynamically */
    result = ody_ftsop (fd, &vers, SPEECHW_INIT_MODELS, strlen(model)+1, model,
			&retsz, pModel);
    return (result);
}

/* speechRecognize() is a wrapper around the corresponding speech
   warden's TSOP.  The TSOP does not dynamically allocate memory, so
   some steps are omitted here.  If the TSOP is successfully executed,
   the function returns zero and places the translation in the
   specified buffer.  Otherwise, the function returns a return code
   from the TSOP execution. */
int speechRecognize (int fd, char* model, char* utterance, int uttlen,
		     char* outputBuffer, int outbuflen)
{
    struct ody_vers vers;
    int             result = 0;
    char*           inputBuffer = 0;
    int             inputLen;

    /* Set up the Odyssey version structure */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = SPEECH_TTYPE;
    vers.ov_tsvnum = SPEECHWARDEN_VERSION;

    /* Concatenate input data */
    inputLen = strlen (model) + 1 + uttlen;
    NALLOC (inputBuffer, char, inputLen);
    strcpy (inputBuffer, model);
    memcpy (inputBuffer + strlen(model) + 1, utterance, uttlen);
    
    /* Make the TSOP - it does not allocate memory dynamically */
    result = ody_ftsop (fd, &vers, SPEECHW_RECOGNIZE, inputLen, inputBuffer,
			&outbuflen, outputBuffer);

    FREE (inputBuffer);

    return (result);
}

/* speechCloseModel() is a wrapper around the corresponding speech
   warden's TSOP.  The TSOP does not dynamically allocate memory, so
   some steps are omitted here.  If the TSOP is successfully executed,
   the function returns zero.  Otherwise, the function returns a return code
   from the TSOP execution. */
int speechCloseModel (int fd, char* model)
{
    struct ody_vers vers;
    size_t          retsz = sizeof (int);
    int             rc;
    int             result = 0;

    /* Set up the Odyssey version structure */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = SPEECH_TTYPE;
    vers.ov_tsvnum = SPEECHWARDEN_VERSION;

    /* Make the TSOP - it does not allocate memory dynamically */
    result = ody_ftsop (fd, &vers, SPEECHW_CLOSE_MODELS, strlen(model)+1,
			model, &retsz, &rc);
    if (result == 0 && rc) result = rc;

    return (result);
}

/* dumpStats() is a wrapper around the corresponding speech warden's
   TSOP.  The TSOP does not dynamically allocate memory, so some steps are
   omitted here.  If the TSOP is successfully executed, the function returns
   zero.  Otherwise, the function returns a return code from the execution
   of the TSOP or from the TSOP itself. */
int dumpStats (int fd, char* filename)
{
  struct ody_vers vers;
  int             rc;
  size_t          retsz = sizeof (rc);
  int             result = 0;
  
  /* Set up the Odyssey version structure */
  vers.ov_gs = ODY_GENERIC_INTERFACE;
  vers.ov_type = SPEECH_TTYPE;
  vers.ov_tsvnum = SPEECHWARDEN_VERSION;
  
  /* Make the TSOP - it does not allocate memory dynamically */
  result = ody_ftsop (fd, &vers, SPEECHW_DUMP_STATS, strlen(filename)+1,
		      filename, &retsz, &rc);
  if (result == 0) {
    result = rc;
  } /* endif */
  
  return(result);
}

