#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#ifndef __SPEECHW_PRIVATE_H__
#define __SPEECHW_PRIVATE_H__

#include "base/ds/ds_hwlock.h"

/* TSOPs */
#define SPEECHW_SET_STRATEGY        1
#define SPEECHW_SET_FIDELITY        2
#define SPEECHW_INIT_MODELS         3
#define SPEECHW_RECOGNIZE           4
#define SPEECHW_CLOSE_MODELS        5
#if 0
#define SPEECHW_SET_SEARCH_EFFORT   6
#endif
#define SPEECHW_DUMP_STATS          7

/* Structure that contains location of Janus speech servers */
typedef struct {
  int localPort;
  char* remoteHostname;
  int remotePort;
} ServerLocations;

enum busyType { SPEECHW_NOT_BUSY, SPEECHW_BUSY };
enum stateType { SPEECHW_WRITING, SPEECHW_READING };

#ifdef INSTRUMENTED
/* The following structure maintains interesting data about a single 
   translation.  It is used to store info about the current/last 
   translation. */
typedef struct {
  long              totalBytes;
  enum strategyType strategy;
  long              MELfilesize;
  struct timeval    tvConnectToJanus;
  struct timeval    tvBeginTranslation;
  struct timeval    tvBeginMELConversion;
  struct timeval    tvEndMELConversion;
  struct timeval    tvBeginJanusRecognition;
  struct timeval    tvEndJanusRecognition;
  struct timeval    tvEndTranslation;
} SpeechMeasurements;
#endif

/* Structure that contains information about loaded speech models */
typedef struct {
  char* model; /* Name of an initialized model */
  int   initLocal; /* True if the model is initialized on the local server */
  int   initRemote; /* True if the model is initialized on the remote server */
} SpeechModel;
		  
/* Structure that contains connection information for the warden */
typedef struct {
  cmgr_id_t localConnection;
  cmgr_id_t remoteConnection;
} SpeechConnections;

/* Structure that contains general data that will be stored in 
   the warden structure */
typedef struct {
  ServerLocations* pLocations;
  SpeechConnections connections;
  int uniquifier;
  ds_hwlock_t lock;
#ifdef INSTRUMENTED
  SpeechMeasurements lastMeasurements; /* Data about last successful transl. */
#endif
} SpeechClientData;

/* Structure that contains input/output data buffered for recognizer.
   Not sure what this looks like yet. */
typedef struct {
  enum strategyType strategy;
  char*             fidelity;
#if 0
  float searchEffort; /* Regulates "depth" of the search tree: average = 1.0 */
#endif
#ifdef INSTRUMENTED
  SpeechMeasurements currentMeasurements; /* Data about current translation */
#endif
  ds_list_t* pModels; /* A list of initialized models */
} SpeechConnectionData;

/* Function Prototypes */
int speechwParseArguments(int cur, int argc, char *argv[]);
int speechwInitialize(struct warden *wp);
int speechwGetRoot(struct warden *wp, tome_t tome, struct viceroy_onode **ppRoot);
int speechwLookup(struct warden* wp, struct viceroy_onode* odp, struct ody_cred* cred,
		  char* name, char** remain, struct viceroy_onode** opp);
int speechwAccess(struct warden* wp, struct viceroy_onode* op, struct ody_cred* cred,
		  int mode);
int speechwGetAttr(struct warden* wp, struct viceroy_onode* op, struct ody_vattr *vap, struct ody_cred* cred);
int speechwOpen (struct warden *wp, struct viceroy_onode *op, struct ody_cred *cred,
		 int mode);
int speechwClose(struct warden *wp, struct viceroy_onode  *op, struct ody_cred  *cred,
		 int mode);
int speechwReadWrite (struct warden* wp, struct viceroy_onode* op, struct ody_cred* cred,
		      int rwflag, int* count, int offset, int ioflag,
		      void **data);
int speechwReadWriteDispose (struct warden* wp, struct viceroy_onode* op, 
			     struct ody_cred* cred, int count, int offset,
			     void* data);
int speechwTSOP(struct warden* wp, int tsvnum, struct viceroy_onode* op,
		struct ody_cred* cred, int opnum, size_t argsz,
		void* argp, size_t* retsz, void** retp);
int speechwTSOPDispose(struct warden* wp, struct viceroy_onode* op, struct ody_cred* cred,
		       int opnum, size_t argsz, void* argp, size_t retsz,
		       void* retp);
void speechwFreeONodeData (void* ptr);
long speechwSetStrategy (struct warden* wp, struct viceroy_onode* op, size_t argsz, 
			 void* argp, size_t* retsz, void** retp);
long speechwSetFidelity (struct warden* wp, struct viceroy_onode* op, 
			 size_t argsz, void* argp, size_t* retsz, void** retp);
SpeechModel* speechwFindModel (SpeechConnectionData* pConn, char* model);
long speechwInitModels (struct warden* wp, struct viceroy_onode* op, 
			size_t argsz, void* argp, size_t* retsz, 
			void** retp);
long speechwRecognize (struct warden* wp, struct viceroy_onode* op, 
		       size_t argsz, void* argp, size_t* retsz, 
		       void** retp);
void speechwUnloadModel (struct warden* wp, SpeechModel* pModel);
long speechwCloseModels (struct warden* wp, struct viceroy_onode* op,
			 size_t argsz, void* argp, size_t* retsz, 
			 void** retp);
#if 0
long speechwSetSearchEffort (struct warden* wp, struct viceroy_onode* op, 
			     size_t argsz, void* argp, size_t* retsz, 
			     void** retp);
#endif
long speechwDumpStats (struct warden* wp, size_t argsz, 
		       void* argp, size_t* retsz, void** retp);
cmgr_id_t speechwGetCID(struct warden *wp, struct viceroy_onode  *op);
void speechwPrintStatistics (struct warden* wp, FILE* file);

#endif

