#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * whiteboard_comm.c -- functions which talk to the peer
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/queue.h> 
#include <sys/stat.h>

#include <include/odytypes.h>
#include <base/ds/ds_safeq.h>
#include <base/ds/ds_thread.h>
#include <base/ds/ds_log.h>
#include <base/ds/ds_rrlist.h>
#include <base/ds/ds_hash.h>
#include <base/ds/ds_hwlock.h>

#include <include/odyssey.h>
#include <kernel/vfs/includes/ody.h>
#include <include/viceroy_onode.h>

#include <include/cm_rpc2.h>
#include <include/cm.h>
#include <include/onode-manager.h>
#include <include/warden.h>

#include "ody_wboard.h"
#include "whiteboard.h"
#include "whiteboard_comm.h"
#include "whiteboard_private.h"

int wboard_send_points_wrap(cmgr_id_t cid,
			    int       x1,
			    int       y1,
			    int       x2,
			    int       y2)
{
    long                        rc;
    cmgr_connection_t           *connection;

    /*
     * Setup IN parameters for the RPC, and then invoke it
     */
    
    ds_thread_log(100, "wboard_send_points_rpc2: sending points (%d %d %d %d)",
		  x1, y1, x2, y2);
    
    if (!(cmgr_validate(cid) > 0)) {
	rc = RPC2_NOCONNECTION;
    } else {
	connection = cmgr_get(cid, CMGR_LOCK);
	ASSERT(connection);
	/* perform the rpc */
	rc = wboard_send_points_rpc2 ( CMGR_RPC2_HANDLE(connection),
				       (RPC2_Integer) x1,
				       (RPC2_Integer) y1,
				       (RPC2_Integer) x2,
				       (RPC2_Integer) y2);
	cmgr_put(cid, CMGR_LOCK);
    }
    
    if(rc != RPC2_SUCCESS) {
	/* mark the connection as down if we encounter any problems.
	   XXX this may be overambitious */
	cmgr_markdown(cid);
	return -1;
    }

    return 0;
}
