#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * whiteboard_rpc2.c -- RPC2 support code
 *
 * written by Eric Tilton <tilt+@cs.cmu.edu>
 *
 * This file provides support routines for RPC2.
 *
 */

/* system/C headers */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

/* my headers */
#include <base/ds/ds_thread.h>
#include <base/ds/ds_log.h>
#include <base/ds/ds_rrlist.h>

/* RPC2 headers */
#include <lwp/lwp.h>
#include <lwp/timer.h>
#include <rpc2/rpc2.h>
#include <rpc2/se.h>
#include <rpc2/sftp.h>


#include <include/cm_rpc2.h>
#include <include/cm.h>

#include "vcroy.h"
#include "ody_wboard.h"

/*
 * The following code is RPC2 specific.
 * Web-specific code follows later.
 */

cmgr_id_t wboard_connect_to_machine(char *machine_name) {
    cmgr_initparams_t cip;
    cmgr_id_t cid;
    
    bzero(&cip, sizeof(cmgr_initparams_t));

    cip.ci_type    = CMGR_RPC2;

    cip.ci_restart.tv_sec = 20;	/* 20 seconds between restarts */
    cip.ci_restart.tv_usec = 0;

    cip.ci_params.ci_rpc2.hid.Tag = RPC2_HOSTBYNAME;
    if(strlen(machine_name) >= 64)
	ds_thread_log(100,
		      "wboard: Machine name %s too long!",
		      machine_name);
    strcpy(cip.ci_params.ci_rpc2.hid.Value.Name, machine_name);
    
    cip.ci_params.ci_rpc2.pid.Tag = RPC2_PORTBYINETNUMBER;
    cip.ci_params.ci_rpc2.pid.Value.InetPortNumber = htons(VCROY_PORTAL_DFLT);
    
    cip.ci_params.ci_rpc2.sid.Tag = RPC2_SUBSYSBYID;
    cip.ci_params.ci_rpc2.sid.Value.SubsysId = ODYWBOARDSUBSYSID;
    
    /* Bind Parameters */
    cip.ci_params.ci_rpc2.bp.SecurityLevel = RPC2_OPENKIMONO;
    cip.ci_params.ci_rpc2.bp.EncryptionType = 0;
    cip.ci_params.ci_rpc2.bp.SideEffectType = SMARTFTP;	/* allow use of SFTP */
    cip.ci_params.ci_rpc2.bp.ClientIdent = NULL;
    cip.ci_params.ci_rpc2.bp.SharedSecret = NULL;
    
    /* Create new binding */
    cid = cmgr_create(&cip);
    return cid;
}
