#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/queue.h>

#include <base/ds/ds_thread.h>
#include <base/ds/ds_log.h>
#include <base/ds/ds_safeq.h>
#include <base/ds/ds_hash.h>
#include <base/ds/ds_histo.h>
#include <base/ds/ds_rrlist.h>

#include <lwp/lwp.h>
#include <lwp/timer.h>
#include <rpc2/rpc2.h>
#include <rpc2/se.h>
#include <rpc2/sftp.h>

#include <include/odyssey.h>
#include <include/odytypes.h>
#include <include/viceroy_onode.h>

#include <include/cm_rpc2.h>
#include <include/cm.h>
#include <include/onode-manager.h>
#include <include/warden.h>

#include "ody_wboard.h"
#include "whiteboard_server.h"

ds_safeq_t *srvq = NULL;

int wboard_server_thread(void *pp)
{
    RPC2_Handle cid;
    RPC2_RequestFilter reqfilter;
    RPC2_PacketBuffer *reqbuffer;
    RPC2_SubsysIdent sid;
    char msg[100];
    int rc;
#if 0
    PROCESS pid;
    int loop, loop2;
    char *alias_fname = NULL;
    char *exec_fname;
#endif

    srvq = (ds_safeq_t *) pp;

    /* cont_sw_threshold.tv_sec = 0;
       cont_sw_threshold.tv_usec = 500000; */

    ds_thread_log(10, "wboard_server_thread: server started");

    /* export ody_wboard */
    sid.Tag = RPC2_SUBSYSBYID;
    sid.Value.SubsysId = ODYWBOARDSUBSYSID;
    rc = RPC2_Export(&sid) != RPC2_SUCCESS;
    if (rc != RPC2_SUCCESS) {
	sprintf(msg, "%s\nCan't export the ody_wboard subsystem",
		RPC2_ErrorMsg(rc));
	wboard_error_report(msg);
    }

    reqfilter.FromWhom = ANY;
    reqfilter.OldOrNew = OLDORNEW;
    reqfilter.ConnOrSubsys.SubsysId = ODYWBOARDSUBSYSID;

    /* Enter main event loop */
    while(1) {			/* wait for client calls to service */
	rc = RPC2_GetRequest(&reqfilter, &cid, &reqbuffer,
			     NULL, NULL, 0, NULL);

	/* extra NULL required for SE descriptor */
	if (rc != RPC2_SUCCESS)
	    fprintf(stderr, RPC2_ErrorMsg(rc));
	rc = odyssey_whiteboard_ExecuteRequest(cid, reqbuffer, NULL);
	if(rc != RPC2_SUCCESS)
	    fprintf(stderr, RPC2_ErrorMsg(rc));
    }
    
    return 0;
}

void wboard_error_report(char *message)
{
    /* Report errors to standard error. 'message' is error message, and
       does not need to be null-terminated */
    
    fprintf(stderr, message);
    fprintf(stderr, "\n");
    exit(1);
}


/* RPC2 procedures */

long wboard_send_points_rpc2(RPC2_Handle  _cid,
			     RPC2_Integer x1,
			     RPC2_Integer y1,
			     RPC2_Integer x2,
			     RPC2_Integer y2)
{
    struct wboard_points *points;

    ds_thread_log(1000, "wboard_send_points_rpc2: (%d, %d)-(%d, %d)",
		  x1, y1, x2, y2);

    ALLOC(points, struct wboard_points);
    points->x1 = x1;
    points->y1 = y1;
    points->x2 = x2;
    points->y2 = y2;

    ds_thread_log(1000, "enq happending (%d) (%d)", (int) srvq, (int) points);
    ds_safeq_enq(srvq, points);

    return RPC2_SUCCESS;
}
