#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * webwarden_tsops.c -- Web Warden TSOP handlers
 *
 * written by Eric Tilton <tilt+@cs.cmu.edu>
 *
 * This file provides handlers for the various Web Warden TSOPs.
 *
 * The currently supported TSOPs are:
 *
 *   WEBW_STAT
 *   WEBW_SETMINQUALITY
 *   WEBW_GETMINQUALITY
 *   WEBW_SETHEADERS
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <sys/queue.h>

#include <include/odytypes.h>
#include <base/ds/ds_thread.h>
#include <base/ds/ds_hash.h>
#include <base/ds/ds_hwlock.h>
#include <base/ds/ds_rrlist.h>
#include <base/ds/ds_safeq.h>

#include <include/viceroy_onode.h>
#include <include/cm_rpc2.h>
#include <include/cm.h>
#include <include/warden.h>
#include <include/downcall.h>

#include <lwp/lwp.h>
#include <rpc2/rpc2.h>

#include <base/simplewww/swww.h>

#include "whiteboard.h"
#include "whiteboard_private.h"
#include "whiteboard_tsops.h"
#include "whiteboard_comm.h"
#include "whiteboard_server.h"

int wboard_get_points(struct warden *wp,
		      struct viceroy_onode  *op,
		      size_t        argsz,
		      void          *argp,
		      size_t        *retsz,
		      void          **retp)
{
    struct wboard_data   *my_data;
    ds_safeq_t           *srvq;
    struct wboard_points *points;

    my_data = (struct wboard_data *)(wp->w_data);
    srvq = my_data->wbd_workq;
    
    ds_thread_log(1000, "wboard: get_points happening (%d) (%d)",
		  argsz, (int) srvq);

    if (ds_safeq_count(srvq) == 0) {
	ds_thread_log(1001, "wboard: nothing to get");
	*retsz = 0;
	*retp  = NULL;
	return(-1);
    }

    points = ds_safeq_deq(srvq);

    ds_thread_log(1000, "wboard: get_points happened (%d) (%d,%d)-(%d,%d)",
		  points, points->x1, points->y1, points->x2, points->y2);

    *retsz = sizeof(struct wboard_data);
    *retp = points;

    return(0);
}

int wboard_send_points(struct warden *wp,
		      struct viceroy_onode  *op,
		      size_t        argsz,
		      void          *argp,
		      size_t        *retsz,
		      void          **retp)
{
    unsigned char *temp;
    unsigned int   line[4];
    unsigned int   value, sum;
    int            index, loop;
    cmgr_id_t      cid;
    struct wboard_data  *my_data;

    temp = (char *) argp;
    for (index = 0; index < 4; index++) {
	/* get each point */
	sum = 0;
	for (loop = 0; loop < 4; loop++) {
	    value = temp[(index * 4) + loop];
	    sum += value << (8 * loop);
	}
	line[index] = sum;
    }

    ds_thread_log(1000, "wboard: send_points happened (%d %d %d %d)",
		  line[0], line[1], line[2], line[3]);

    my_data = (struct wboard_data *) wp->w_data;
    cid = my_data->wbd_cid;
    wboard_send_points_wrap(cid, line[0], line[1], line[2], line[3]);

    *retsz = 0;
    *retp = NULL;
    return(0);
}
