#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * libwebw_tsops.c -- wrappers for applications which want to
 *                    use webwarden tsops
 */

#ifdef WARDEN
#undef WARDEN
#endif

#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>

#include <include/odytypes.h>
#include <base/ds/ds_thread.h>
#include <base/ds/ds_hash.h>
#include <base/ds/ds_hwlock.h>
#include <include/odyssey.h>

#include <include/viceroy_onode.h>

#include <base/simplewww/swww.h>
#include "webwarden.h"

/*
 * PUBLIC FUNCTIONS
 */

/*
 * Web_openByURL -- opens the requested HTTP object as a file, and returns
 *                  the file descriptor.
 */
int Web_openByURL(char           *url,        /* the URL of the HTTP object */
		  char           **ody_path)  /* the path to the opened object
					         in odyssey */
{
    char *munged_url, *bookmark;
    int  ody_path_size;
    int  fd;

    /* convert the url's forward slashes into back slashes */
    
    ody_path_size  = strlen("/ody/www/");
    ody_path_size += strlen(url);

    munged_url = strdup(url);
    bookmark = munged_url;
    while(*bookmark != '\0') {
	if (*bookmark == '/')
	    *bookmark = '\\';
	bookmark++;
    }
    
    NALLOC(*ody_path, char, ody_path_size + 1);
    strcpy(*ody_path, "/ody/www/");
    strcat(*ody_path, munged_url);
    FREE(munged_url);

    fd = open(*ody_path, O_RDONLY, 0666);

    return(fd);
}

/*
 * Web_fstat -- given an open file descriptor to an odyssey object
 *              in a webwarden-managed tome, returns metadata (such
 *              as headers) related to that object.
 */

int Web_fstat(int            fd,              /* open file descriptor
						 of object */
	      int            *result_code,    /* numeric result of
						 HTTP request */
	      char           **result_string, /* verbose result */
	      int            *header_size,    /* number of headers */
	      SWWW_headers_t **headers)       /* headers associated w/ HTTP
						 object */
{
    struct ody_vers vers;
    size_t          retsz;
    char            *retbuffer;
    char            *cbookmark;
    int             *ibookmark;
    int             result;

    /* set up the return buffer */
    retsz     = 2048;
    NALLOC(retbuffer, char, retsz);

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    result = ody_ftsop(fd, &vers, WEBW_STAT, 0, NULL, &retsz, retbuffer);
    if (result) {
	FREE(retbuffer);
	return(result);
    }

    /* unpack the data */
    ibookmark = (int *) retbuffer;
    *result_code = *ibookmark; ibookmark++;
    *header_size = *ibookmark; ibookmark++;

    cbookmark = (char *) ibookmark;
    *result_string = strdup(cbookmark);
    cbookmark += strlen(*result_string) + 1;

    /* at this point, cbookmark points to the headers, and
       header_bytes contains the number of headers here. unroll this
       puppy. */
    *headers = SWWW_unpack_headers(*header_size, cbookmark);

    FREE(retbuffer);
    return(0);
}

/*
 * Web_stat -- given an open file descriptor to an odyssey object
 *             in a webwarden-managed tome, returns metadata (such
 *             as headers) related to that object.
 */

int Web_stat(char           *path,           /* open file descriptor
						of object */
	     int            *result_code,    /* numeric result of
						HTTP request */
	     char           **result_string, /* verbose result */
	     int            *header_size,    /* number of headers */
	     SWWW_headers_t **headers)       /* headers associated w/ HTTP
						object */
{
    struct ody_vers vers;
    size_t          retsz;
    char            *retbuffer;
    char            *cbookmark;
    int             *ibookmark;
    int             result;

    /* set up the return buffer */
    retsz     = 2048;
    NALLOC(retbuffer, char, retsz);

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    result = ody_tsop(path, &vers, WEBW_STAT, 0, NULL, &retsz, retbuffer);
    if (result) {
	FREE(retbuffer);
	return(result);
    }

    /* unpack the data */
    ibookmark = (int *) retbuffer;
    *result_code = *ibookmark; ibookmark++;
    *header_size = *ibookmark; ibookmark++;

    cbookmark = (char *) ibookmark;
    *result_string = strdup(cbookmark);
    cbookmark += strlen(*result_string) + 1;

    /* at this point, cbookmark points to the headers, and
       header_bytes contains the number of headers here. unroll this
       puppy. */
    *headers = SWWW_unpack_headers(*header_size, cbookmark);

    FREE(retbuffer);
    return(0);
}

/*
 * Web_setMinQuality -- allows the application to specify a minimum quality
 *                   level it is willing to accept, for a given MIME type.
 */
int Web_setMinQuality(char *mime_type,         /* MIME type */
		      webw_quality_t quality)  /* minimum quality vector */
{
    struct ody_vers vers;
    size_t          size;
    char            *buffer;
    size_t          retsz;
    char            *cbookmark;
    webw_quality_t  *qbookmark;
    int             result;

    /* set up the buffer */
    size =  sizeof(webw_quality_t) + strlen(mime_type) + 1;
    NALLOC(buffer, char, size);

    /* fill in the buffer */
    qbookmark  = (webw_quality_t *) buffer;
    *qbookmark = quality;
    qbookmark++;

    cbookmark = (char *) qbookmark;
    strcpy(cbookmark,mime_type);

    retsz = 0;			/* we don't expect a response */

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    result = ody_tsop("/ody/www", &vers,
		      WEBW_SETMINQUALITY,
		      size, buffer,
		      &retsz, NULL);

    FREE(buffer);
    return(result);
}

/*
 * Web_getMinQuality -- allows the application to get the current
 *                      minimum quality.
 *
 *                      XXX this only makes sense as long as the
 *                      XXX warden has a single min. quality.
 */
int Web_getMinQuality(char           *mime_type, /* MIME type */
		      webw_quality_t *quality)   /* quality */
{
    struct ody_vers vers;
    size_t          size;
    char            *buffer;
    size_t          retsz;
    int             result;

    /* set up the buffer */
    size =  strlen(mime_type) + 1;
    NALLOC(buffer, char, size);

    /* fill in the buffer */
    strcpy(buffer, mime_type);

    retsz = sizeof(webw_quality_t);

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    result = ody_tsop("/ody/www", &vers,
		      WEBW_GETMINQUALITY,
		      size, buffer,
		      &retsz, quality);

    FREE(buffer);
    return(result);
}

/*
 * Web_setMaxTime -- allows the application to specify the maximum
 *                   time it is willing to wait for any document.
 *                   used as a heuristic by the warden to determine
 *                   how much degradation (and humiliation?) to apply
 *                   to HTTP objects.
 */
int Web_setMaxTime(struct timeval time)        /* maximum time */
{
    struct ody_vers vers;
    size_t          size;
    struct timeval  *buffer;
    size_t          retsz;
    int             result;

    /* set up the buffer */
    size   = sizeof(struct timeval);
    NALLOC(buffer, struct timeval, size);

    /* fill in the buffer */
    *buffer = time;

    retsz = 0;			/* we don't expect a response */

    /* make the tsop */
    vers.ov_gs     = ODY_GENERIC_INTERFACE;
    vers.ov_type   = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    /* XXX can I perform a tsop without having a reference object? */
    result = ody_tsop("/ody/www", &vers,
		      WEBW_SETMAXTIME,
		      size, buffer,
		      &retsz, NULL);

    FREE(buffer);
    return(result);
}

/*
 * Web_invalidatePage -- allows an application to mark a cached object
 *                       as invalid, forcing the Web warden to perform
 *                       a conditional GET on the page when it is next
 *                       opened in order to get a more recent copy (if
 *                       one exists).
 */
int Web_invalidatePage(char *url)
{
    struct ody_vers  vers;
    int              retsz = 0;
    int              result;

    char *munged_url, *bookmark, *ody_path;
    int  ody_path_size;

    /* convert the url's forward slashes into back slashes */
    
    ody_path_size  = strlen("/ody/www/");
    ody_path_size += strlen(url);

    munged_url = strdup(url);
    bookmark = munged_url;
    while(*bookmark != '\0') {
	if (*bookmark == '/')
	    *bookmark = '\\';
	bookmark++;
    }
    
    NALLOC(ody_path, char, ody_path_size + 1);
    strcpy(ody_path, "/ody/www/");
    strcat(ody_path, munged_url);
    FREE(munged_url);

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    result = ody_tsop(ody_path, &vers, WEBW_INVALIDATEPAGE,
		      0, NULL, &retsz, NULL);

    FREE(ody_path);

    return result;
}

/*
 * Web_flushCache -- flush the disk cache of web objects
 *                   XXX make interface handle flush of named objects?
 */
int Web_flushCache(void)
{
    struct ody_vers  vers;
    int              retsz = 0;
    int              result;

    /* make the tsop */
    vers.ov_gs = ODY_GENERIC_INTERFACE;
    vers.ov_type = WEB_TTYPE;
    vers.ov_tsvnum = WEBWARDEN_VERSION;

    result = ody_tsop("/ody/www/", &vers, WEBW_FLUSHCACHE,
		      0, NULL, &retsz, NULL);

    return result;
}
