#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * webwarden.h
 *
 * written by Eric Tilton <tilt+@cs.cmu.edu>
 *
 * Common include file for web warden
 *
 * Dependencies:
 *   before including this file, you should also include
 *      sys/time.h
 *      swww.h
 *
 * History
 *   May & Jun 1996 -- ??    -- colorful first attempts.
 *   19 Jul 1996 --  0. 0. 1 -- first serious attempts to design TSOP
 *                              handlers in warden code, including
 *                              marshalling, etc. Also, first serious
 *                              attempts at lookup and fid2o functions.
 *                              common include file created.
 *
 *   11 Sep 1996 --          -- added two new TSOPs (Web_setMinQuality(),
 *                              Web_setMaxTime()) to allow applications
 *                              to describe bounds of tolerance.
 *
 *   27 Sep 1996 --  0. 0. 2 -- Minor parameter changes to reflect
 *                              modified SWWW names.
 *
 *   25 Feb 1997 --          -- added Web_getMinQuality().
 *
 *    6 Jun 1997 --          -- excised "quality" from Web_openByURL,
 *                              which wasn't using it.
 *
 *   27 Oct 1997 --  0. 0. 4 -- Added Web_setHeaders(), and added headers
 *                              and post_data flags to Web_openByURL().
 *
 *    5 Nov 1997 --  0. 0. 5 -- Removed changes from 0.0.4. Added
 *                              Web_invalidatePage().
 */

#ifndef _WEBWARDEN_H_
#define _WEBWARDEN_H_

/* common defines */

#define WEBWARDEN_VERSION 5
#define WEB_TTYPE         0x3

/* common data structures and enums */

#define WEBW_OPEN_BY_URL    1	/* TSOP numbers */
#define WEBW_STAT           2
#define WEBW_SETMINQUALITY  3
#define WEBW_SETMAXTIME     4
#define WEBW_GETMINQUALITY  5
/* #define WEBW_SETHEADERS     6 */ /* retired TSOP */
#define WEBW_INVALIDATEPAGE 7
#define WEBW_FLUSHCACHE     8

typedef int webw_quality_t;

#ifndef WEBWARDEN

extern struct warden web_warden;

/* Definitions of application interface to webwarden TSOPs */

/*
 * Web_openByURL -- opens the requested HTTP object as a file, and returns
 *                  the file descriptor.
 */
int Web_openByURL(char           *url,        /* the URL of the HTTP object */
		  char           **ody_path); /* the path to the opened object
					         in odyssey */

/*
 * Web_fstat -- returns web-specific stats about the open HTTP object. These
 *              are stats which cannot be retrieved via the normal stat()
 *              command.
 */
int Web_fstat(int            fd,              /* open file descriptor
						 of object */
	      int            *result_code,    /* numeric result of
						 HTTP request */
	      char           **result_string, /* verbose result */
	      int            *header_size,    /* number of headers */
	      SWWW_headers_t **headers);       /* headers associated w/ HTTP
						 object */

/*
 * Web_stat -- returns web-specific stats about the open HTTP object. These
 *             are stats which cannot be retrieved via the normal stat()
 *             command.
 */
int Web_stat(char           *path,           /* open file descriptor
						of object */
	     int            *result_code,    /* numeric result of
						HTTP request */
	     char           **result_string, /* verbose result */
	     int            *header_size,    /* number of headers */
	     SWWW_headers_t **headers);       /* headers associated w/ HTTP
						 object */

/*
 * The following TSOPs (Web_setMinQuality and Web_setMaxTime) are first
 * passes at allowing applications to describe their bounds of tolerance.
 * They are likely to change soon.
 */

/*
 * Web_setMinQuality -- allows the application to specify a minimum quality
 *                   level it is willing to accept, for a given MIME type.
 */
int Web_setMinQuality(char *mime_type,         /* MIME type */
		      webw_quality_t quality); /* minimum quality vector */

/*
 * Web_getMinQuality -- returns the current minimum quality.
 *
 *                      XXX this only makes sense as long as the
 *                      XXX warden has a single min. quality.
 */
int Web_getMinQuality(char           *mime_type, /* MIME type */
		      webw_quality_t *quality);  /* quality */

/*
 * Web_setMaxTime -- allows the application to specify the maximum
 *                   time it is willing to wait for any document.
 *                   used as a heuristic by the warden to determine
 *                   how much degradation (and humiliation?) to apply
 *                   to HTTP objects.
 */
int Web_setMaxTime(struct timeval time);       /* maximum time */

/*
 * Web_invalidatePage -- allows an application to mark a cached object
 *                       as invalid, forcing the Web warden to perform
 *                       a conditional GET on the page when it is next
 *                       opened in order to get a more recent copy (if
 *                       one exists).
 */
int Web_invalidatePage(char *url);

int Web_flushCache(void);

#endif /* WEBWARDEN */

#endif /* _WEBWARDEN_H_ */
