#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * HISTORY
 *  Dale Moore Aug 23, 1996 Carnegie Mellon University
 *  Added Solaris support (Sunos 5 sparc).
 * $Log: get_load.c,v $
 * Revision 1.8  2000/01/14 00:13:42  bumba
 *
 *
 * GPL'ed the code base.
 *
 * Revision 1.7  1999/05/17 21:07:40  kwalker
 * add include, fix sprintf tokens
 *
 * Revision 1.6  1999/04/30 21:16:28  bumba
 *
 *
 * Made the display line on the monitors more readable. All bandwidths are
 * now reported in KB/sec.
 *
 * Revision 1.5  98/06/19  16:57:52  tilt
 * Added copyright notice. Massive commit.
 * 
 * Revision 1.4  98/05/18  15:59:09  tilt
 * Removed nettr includes completely, since they don't appear to be
 * used.
 * 
 * Revision 1.3  1998/05/18 15:57:30  tilt
 * Various modifications to get things to compile under FreeBSD.
 *
 * Revision 1.2  1998/05/08 14:46:46  kwalker
 * fix include
 *
 * Revision 1.1  98/05/08  14:12:37  kwalker
 * formerly bwmon
 * 
 * Revision 1.1  98/05/08  00:17:48  kwalker
 * initial commit; port transitional code
 * 
 * Revision 1.3  1998/02/23 15:22:39  kwalker
 * support scale argument, total bandwidth reading
 *
 * Revision 1.2  98/01/05  14:57:01  kwalker
 * comments for documentation
 * 
 * Revision 1.1  98/01/05  14:47:18  kwalker
 * Bandwidth monitoring tool
 * 
 * Revision 1.1  97/11/24  14:44:19  bnoble
 * Added monitoring programs bwmon, latmon, and lossmon
 * 
 * $EndLog$
 */
/* $XConsortium: get_load.c /main/37 1996/03/09 09:38:04 kaleb $ */
/*

Copyright (c) 1989  X Consortium

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall
not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization
from the X Consortium.

*/

/*
 * get_load - get system load
 *
 * Authors:  Many and varied...
 *
 * Call InitLoadPoint() to initialize.
 * GetLoadPoint() is a callback for the StripChart widget.
 */

#include <X11/Xos.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "include/bw_tool.h"

extern int vice_fd, mode_fd;
extern int vice_cid;
extern Widget label_wid;

/* XXX */
/* 
 * This is totally evil, but I have no intention of 
 * hacking xload proper for a damn demo.
 */
/* this constant is for trace replay -- doesn't work for ether. */
/*static const double   max = 1.0/480.0E-8;*/
static const double   max = 1*1024*1024; /* 1 MBps */
double max_bw = 0.0;
static double cur_bw = 0.0;
static  char mode_string[256] = "";

void InitLoadPoint()
{
}

void GetLoadPoint( w, closure, call_data )
     Widget   w;          /* unused */
     caddr_t  closure;    /* unused */
     caddr_t  call_data;  /* pointer to (double) return value */
{
    ssize_t rc;
    struct sockaddr_in vice_addr;
    char buf[256];
    double this;
    Arg args[1];

    /* send a request packet to the viceroy */
    bzero(buf, 256);
    sprintf(buf, "%d", vice_cid);
    vice_addr.sin_family = AF_INET; /* better way to do localhost? */
    vice_addr.sin_addr.s_addr = inet_addr("127.0.0.1");	
    vice_addr.sin_port = htons(BW_VICE_PORT);
    rc = sendto(vice_fd, buf, strlen(buf), 0,
		(struct sockaddr *)&vice_addr, sizeof(vice_addr));
    if (rc < strlen(buf)) {
	perror("sendto");
    }
    
    /* receive the reply */
    rc = recv(vice_fd, buf, 256, 0);
    if (rc < strlen(buf)) {
	perror("recv");
    }
    buf[rc] = '\0';
    sscanf(buf, "%lf", &cur_bw);
    if (max_bw)
      this = cur_bw/max_bw;
    else
      this = cur_bw/max;
    *(double *)call_data = this > 1.0 ? 1.0 : this;

    /* Update the label */
    sprintf(buf, "%.0f KB/s %s", cur_bw/1000.0, mode_string);
    XtSetArg(args[0], XtNlabel, buf);
    XtSetValues(label_wid, args, ONE);
}

void GetModeString( w, closure, call_data )
    Widget   w;          /* unused */
    caddr_t  closure;    /* unused */
    caddr_t  call_data;  /* pointer to (double) return value */
{
    int rc;
    char label[256];
    Arg args[1];

    /* read the string from the application */
    rc = recv(mode_fd, mode_string, 256, 0);
    mode_string[rc] = '\0';
    if (rc < strlen(mode_string)) {
	perror("recv");
    }

    /* Update the label */
    sprintf(label, "%.0f KB/s %s", cur_bw/1000.0, mode_string);
    XtSetArg(args[0], XtNlabel, label);
    XtSetValues(label_wid, args, ONE);
}

