#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

/*
 * Copyright (c) 1980 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1980 Regents of the University of California.\n\
 All rights reserved.\n";
#endif not lint

#ifndef lint
static char sccsid[] = "@(#)grep.c	5.2 (Berkeley) 3/4/86";
#endif not lint

/*
 **********************************************************************
 * HISTORY
 * $Log: s_grep.c,v $
 * Revision 1.3  2000/01/14 00:13:45  bumba
 *
 *
 * GPL'ed the code base.
 *
 * Revision 1.2  1998/07/18 12:19:34  kwalker
 * fix sets includes
 *
 * Revision 1.1  98/07/09  15:57:53  kwalker
 * add sets utils
 * 
 * Revision 1.1  94/12/02  16:22:54  dcs
 * Initial revision
 * 
 * Revision 1.1  94/02/21  19:58:21  dcs
 * Initial revision
 * 
 * Revision 1.2  89/02/06  15:52:04  parker
 * 	Added bugfix for compile() from December 88 ACIS release.
 * 	[89/02/06            parker]
 * 
 *
 **********************************************************************
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/file.h>
#include <errno.h>

#include "kernel/sets/common/sets.h"

/*
 * grep -- print lines matching (or not matching) a pattern
 */

/* I need to modify it for my tests. The general idea is that I need
 * to extend grep to apply to a set of files. -- DCS
 */

#define assert(cond) 							\
    if (!(cond)) {    							\
	int *j = 0;   							\
	printf("Assert at line \"%s\", line %d\n", __FILE__, __LINE__); \
	*j = 1;  /* Cause a real SIGTRAP to happen, allow zombie */ 	\
    }
     
#include <sys/ioctl.h>
#ifdef  __STDC__
#define NSC_SHOW_COUNTER_INFO   _IO('c', 0)
#define NSC_GET_COUNTER         _IOR('c', 1, long)
#else   __STDC__
#define NSC_SHOW_COUNTER_INFO   _IO(c, 0)
#define NSC_GET_COUNTER         _IOR(c, 1, long)
#endif  __STDC__

int clockFD = 0;

#define TIMERDEFS	struct timeval after, before; int a, b;

#define TIME_BEFORE  	{ 		                                        \
			   if (clockFD > 0)                                     \
			       ioctl(clockFD, NSC_GET_COUNTER, &b);		\
			   else							\
			       gettimeofday(&before, 0);			\
			}

#define TIME_AFTER(total) {							\
			    if (clockFD > 0) {					\
			        ioctl(clockFD, NSC_GET_COUNTER, &a);		\
			        total += (a - b) / 25;			\
			    } else {						\
			        gettimeofday(&after, 0);			\
			        total += (after.tv_sec - before.tv_sec) * 1000000 + after.tv_usec - before.tv_usec; \
			    }							\
			  }
										


int totalbytes;


#define BLKSIZE 8192
#define	CCHR	2
#define	CDOT	4
#define	CCL	6
#define	NCCL	8
#define	CDOL	10
#define	GCEOF	11

#define	CBRC	14
#define	CLET	15
#define	STAR	01

#define	ESIZE	256

char	expbuf[ESIZE];
long	lnum;
char	linebuf[BUFSIZ+1];
int	bflag;
int	nflag;
int	cflag;
int	vflag;
int	iflag;
int	lflag;
int	wflag;
int	sflag;
int	nsucc;
int	circf;
int	blkno;
long	tln;
int	retcode = 0;
float total = 0.0;

main(argc, argv)
char **argv;
{
    struct timeval before, after;

    while (--argc > 0 && (++argv)[0][0]=='-') {
	char *cp = argv[0] + 1;
	while (*cp) switch (*cp++) {
	    
	  case 'v':
	    vflag++;
	    continue;
	    
	  case 'b':
	    bflag++;
	    continue;
	    
	  case 'i':
	  case 'y':	/* -y for compatibility with btl grep */
	    iflag++;
	    continue;
	    
	  case 'l':
	    lflag++;
	  case 'c':
	    cflag++;
	    continue;
	    
	  case 'w':
	    wflag++;
	    continue;
	    
	  case 's':
	    sflag++;
	    continue;
	    
	  case 'n':
	    nflag++;
	    continue;
	    
	  case 'e':
	    --argc;
	    ++argv;
	    goto out;
	    
	  case 'f': {	/* Use the NSC Counter. */
	      clockFD = open("/dev/cntr0", O_RDONLY);
	      if (clockFD == -1) {
		  perror("open /dev/cntr0");
		  exit(-1);
	      }
	      continue;
	  }

	  default:
	    fprintf(stderr, "grep: unknown flag\n");
	    continue;
	}
    }
  out:
    
    if (argc<=0)
	exit(2);

    gettimeofday(&before, 0);
    compile(*argv);
    if (argc<=0) {
	if (lflag)
	    exit(1);
	execute(0);
    }
    else while (--argc > 0) {
	argv++;
	processArg(*argv);
    }
    gettimeofday(&after, 0);
    if (before.tv_usec > after.tv_usec) {				
	after.tv_sec--;						
	after.tv_usec = 1000000 + after.tv_usec - before.tv_usec;	
    } else {								
	after.tv_usec = after.tv_usec - before.tv_usec;		
    }									
    after.tv_sec = after.tv_sec - before.tv_sec;

    exit(retcode != 0 ? retcode : nsucc == 0);
}

compile(astr)
char *astr;
{
	register c;
	register char *ep, *sp;
	char *lastep=NULL;
	int cclcnt;

	ep = expbuf;
	sp = astr;
	if (*sp == '^') {
		circf++;
		sp++;
	}
	if (wflag)
		*ep++ = CBRC;
	for (;;) {
		if (ep >= &expbuf[ESIZE])
			goto cerror;
		if ((c = *sp++) != '*')
			lastep = ep;
		switch (c) {

		case '\0':
			if (wflag)
				*ep++ = CLET;
			*ep++ = GCEOF;
			return;

		case '.':
			*ep++ = CDOT;
			continue;

		case '*':
			if (lastep==NULL)
				goto defchar;
			*lastep |= STAR;
			continue;

		case '$':
			if (*sp != '\0')
				goto defchar;
			*ep++ = CDOL;
			continue;

		case '[':
			*ep++ = CCL;
			*ep++ = 0;
			cclcnt = 1;
			if ((c = *sp++) == '^') {
				c = *sp++;
				ep[-2] = NCCL;
			}
			do {
				*ep++ = c;
				cclcnt++;
				if (c=='\0' || ep >= &expbuf[ESIZE])
					goto cerror;
			} while ((c = *sp++) != ']');
			lastep[1] = cclcnt;
			continue;

		case '\\':
			if ((c = *sp++) == '\0')
				goto cerror;
			if (c == '<') {
				*ep++ = CBRC;
				continue;
			}
			if (c == '>') {
				*ep++ = CLET;
				continue;
			}
		defchar:
		default:
			*ep++ = CCHR;
			*ep++ = c;
		}
	}
    cerror:
	fprintf(stderr, "grep: RE error\n");
	exit(2);
}

same(a, b)
	register int a, b;
{

	return (a == b || iflag && (a ^ b) == ' ' && letter(a) == letter(b));
}

letter(c)
	register int c;
{

	if (c >= 'a' && c <= 'z')
		return (c);
	if (c >= 'A' && c <= 'Z')
		return (c + 'a' - 'A');
	return (0);
}

/* Use nsccounters instead of gettimeofday if it is present. */

processArg(char *file)
{
    int set;
    int f, save;
    int repeat=1;
    TIMERDEFS;
    
    if (file) {
	TIME_BEFORE;

	set = set_open(file, SETS_ANTICIPATE_ITERATE);
	if (set < 0) {
	    perror(file);
	    retcode = 2;
	    return;
	}
	TIME_AFTER(total);
	
loop:
	f = 1; save = 0;
	while (f > 0) {
	    char line[1024];
	    errno = 0;
	    TIME_BEFORE;
	    f = set_iterate(set, 1, line, 1024, 0);
	    TIME_AFTER(total);

	    if (f > 0) 
		execute(f, line); 
	    else
		if (errno)	/* Otherwise, just end of set. */
		    perror("setIterate");
	}
#ifdef REWIND_IT
	if (repeat) {
		printf("----REWINDING----\n");
		set_rewind_iterator(set);	
		repeat = 0;
		goto loop;
	}
#endif
	if (set_close(set) != 0)
	    perror("setClose");
    } else 
	execute(0);
}    
     
execute(int f, char *file)
{
	register char *p1, *p2;
	register c;
	char *ebp, *cbp;
	static char *buf;
	static int blksize;
	struct stat stb;

	if (buf == NULL) {
		if (fstat(f, &stb) > 0 && stb.st_blksize > 0)
			blksize = stb.st_blksize;
		else
			blksize = BLKSIZE;
		buf = (char *)malloc(blksize);
		if (buf == NULL) {
			fprintf(stderr, "grep: no memory left\n");
			retcode = 2;
			return;
		}
	}
	ebp = buf;
	cbp = buf;
	lnum = 0;
	tln = 0;
	blkno = -1;
	for (;;) {
		lnum++;
		if((lnum&0377) == 0)
			fflush(stdout);
		p1 = linebuf;
		p2 = cbp;
		for (;;) {
			if (p2 >= ebp) {
				if ((c = read(f, buf, blksize)) <= 0) {
					close(f);
					if (cflag) {
						if (lflag) {
							if (tln && file) 
							printf("%s\n", file);
						} else {
							if (file)
								printf("%s:", file);
							printf("%ld\n", tln);
						}
					}
					return;
				}

				totalbytes += c;
				
				blkno++;
				p2 = buf;
				ebp = buf+c;
			}
			if ((c = *p2++) == '\n')
				break;
			if(c)
			if (p1 < &linebuf[BUFSIZ-1])
				*p1++ = c;
		}
		*p1++ = 0;
		cbp = p2;
		p1 = linebuf;
		p2 = expbuf;
		if (circf) {
			if (advance(p1, p2))
				goto found;
			goto nfound;
		}
		/* fast check for first character */
		if (*p2==CCHR) {
			c = p2[1];
			do {
				if (*p1!=c && (!iflag || (c ^ *p1) != ' '
					|| letter(c) != letter(*p1)))
					continue;
				if (advance(p1, p2))
					goto found;
			} while (*p1++);
			goto nfound;
		}
		/* regular algorithm */
		do {
			if (advance(p1, p2))
				goto found;
		} while (*p1++);
	nfound:
		if (vflag)
			succeed(file);
		continue;
	found:
		if (vflag==0)
			succeed(file);
	}
}

advance(alp, aep)
	char *alp, *aep;
{
	register char *lp, *ep, *curlp;
	char *nextep;

	lp = alp;
	ep = aep;
	for (;;) switch (*ep++) {

	case CCHR:
		if (!same(*ep, *lp))
			return (0);
		ep++, lp++;
		continue;

	case CDOT:
		if (*lp++)
			continue;
		return(0);

	case CDOL:
		if (*lp==0)
			continue;
		return(0);

	case GCEOF:
		return(1);

	case CCL:
		if (cclass(ep, *lp++, 1)) {
			ep += *ep;
			continue;
		}
		return(0);

	case NCCL:
		if (cclass(ep, *lp++, 0)) {
			ep += *ep;
			continue;
		}
		return(0);

	case CDOT|STAR:
		curlp = lp;
		while (*lp++);
		goto star;

	case CCHR|STAR:
		curlp = lp;
		while (same(*lp, *ep))
			lp++;
		lp++;
		ep++;
		goto star;

	case CCL|STAR:
	case NCCL|STAR:
		curlp = lp;
		while (cclass(ep, *lp++, ep[-1]==(CCL|STAR)));
		ep += *ep;
		goto star;

	star:
		do {
			lp--;
			if (advance(lp, ep))
				return(1);
		} while (lp > curlp);
		return(0);

	case CBRC:
		if (lp == expbuf)
			continue;
#define	uletter(c)	(letter(c) || c == '_')
		if ( ( uletter(*lp) || digit ( * lp ) )  && !uletter(lp[-1]) && !digit(lp[-1]))
			continue;
		return (0);

	case CLET:
		if (!uletter(*lp) && !digit(*lp))
			continue;
		return (0);

	default:
		fprintf(stderr, "grep: RE botch\n");
		exit(2);
	}
}

cclass(aset, ac, af)
	char *aset;
{
	register char *set, c;
	register n;

	set = aset;
	if ((c = ac) == 0)
		return(0);
	n = *set++;
	while (--n)
		if (n > 2 && set[1] == '-') {
			if (c >= (set[0] & 0177) && c <= (set[2] & 0177))
				return (af);
			set += 3;
			n -= 2;
		} else
			if ((*set++ & 0177) == c)
				return(af);
	return(!af);
}

succeed(f)
{
	nsucc = 1;
	if (sflag)
		return;
	if (cflag) {
		tln++;
		return;
	}
	if (f)
		printf("%s:", f);
	if (bflag)
		printf("%d:", blkno);
	if (nflag)
		printf("%ld:", lnum);
	printf("%s\n", linebuf);
}

digit(c)
	char c;
{
	return (c>='0' && c<='9');
}
