#ifndef _ODYBLURB_
#define _ODYBLURB_
/*
 *          Odyssey: an Experimental Platform for Mobile Computing
 *  
 *            Copyright (c) 1997-2000 Carnegie Mellon University
 *                   Additional copyrights listed below
 * 
 *  This  code  is  distributed "AS IS" without warranty of any kind under
 *  the terms of the GNU General Public Licence Version 2, as shown in the
 *  file  LICENSE.  The  technical and financial  contributors to Odyssey
 *  are listed in the file CREDITS.
 * 
 *                         Additional copyrights
 *                            none currently
 */
#endif /* _ODYBLURB_ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "kernel/sets/common/sets.h"

void usage(void)
{
  fprintf(stderr, "usage: s_test [-d || -i || -r] <setspec>\n");
  exit(-1);
}

int main(int argc, char **argv)
{
  char *spec=NULL;
  int sd;
  int flags = 0, again = 0;
  enum { NONE, DIG, ITER } op = NONE;

  if (argc == 2) {
    spec = argv[1];
    op = NONE;
  } else if (argc == 3 && *(argv[1]) == '-') {
    spec = argv[2];
    if ((argv[1])[1] == 'd') {
      op = DIG;
      flags = SETS_ANTICIPATE_DIGEST;
    } else if ((argv[1])[1] == 'i') {
      op = ITER;
      flags = SETS_ANTICIPATE_ITERATE | SETS_PREFETCH;
    } else if ((argv[1])[1] == 'r') {
      op = ITER;
      flags = SETS_ANTICIPATE_ITERATE;
      again = 1;
    }
  } else
    usage();

  sd = set_open(spec, flags);
  switch (op) {
  case DIG: {
    int nbytes;
    struct DigestT *buf, *elem;
    
    buf = (struct DigestT *)malloc(4096);
    while ((nbytes = set_digest(sd, buf, 4096)) > 0) {
      if (strcmp(&buf->name, "...") == 0) {
	/* Pause, because we exhausted the set so far */
	sleep(1);
      } else {
	for (elem = buf; elem; elem = elem->next) {
	  if (strcmp(&(elem->name), "...")) {
	    printf("%s\n", &elem->name);
	  }
	}
      }
    }
    break;
  }
  case ITER: {
    int fd;
    char buf[4096];
    int len;
    
  loop:
    while ((fd = set_iterate(sd, SETS_GETNAME, buf, 4095, 0)) >= 0) {
      printf("%s:\n", buf);
      while ((len = read(fd, buf, 4095)) > 0) {
	buf[len] = '\0';
	printf("%s\n", buf);
      }
      close(fd);
    }
    if (again) {
      again = 0;
      set_rewind_iterator(sd);
      goto loop;
    }
    break;
  }
  default:
  }
  
  printf("%d members\n", set_size(sd));
  set_close(sd);
  return 0;
}
